/* DO NOT EDIT! GENERATED AUTOMATICALLY! */

/* Virtual function table layout of superclass.  */
#include "ostream.vt.h"

/* Virtual function table layout of fd_ostream class.  */
      int          (*get_descriptor) (THIS_ARG);
   const char * (*get_filename) (THIS_ARG);
   bool         (*is_buffered) (THIS_ARG);
