#include "tree_sitter/parser.h"

#if defined(__GNUC__) || defined(__clang__)
#pragma GCC diagnostic ignored "-Wmissing-field-initializers"
#endif

#define LANGUAGE_VERSION 14
#define STATE_COUNT 3649
#define LARGE_STATE_COUNT 1014
#define SYMBOL_COUNT 346
#define ALIAS_COUNT 4
#define TOKEN_COUNT 155
#define EXTERNAL_TOKEN_COUNT 10
#define FIELD_COUNT 31
#define MAX_ALIAS_SEQUENCE_LENGTH 10
#define PRODUCTION_ID_COUNT 280

enum ts_symbol_identifiers {
  sym_identifier = 1,
  anon_sym_SEMI = 2,
  anon_sym_macro_rules_BANG = 3,
  anon_sym_LPAREN = 4,
  anon_sym_RPAREN = 5,
  anon_sym_LBRACK = 6,
  anon_sym_RBRACK = 7,
  anon_sym_LBRACE = 8,
  anon_sym_RBRACE = 9,
  anon_sym_EQ_GT = 10,
  anon_sym_COLON = 11,
  anon_sym_DOLLAR = 12,
  aux_sym_token_repetition_pattern_token1 = 13,
  anon_sym_PLUS = 14,
  anon_sym_STAR = 15,
  anon_sym_QMARK = 16,
  anon_sym_block = 17,
  anon_sym_expr = 18,
  anon_sym_ident = 19,
  anon_sym_item = 20,
  anon_sym_lifetime = 21,
  anon_sym_literal = 22,
  anon_sym_meta = 23,
  anon_sym_pat = 24,
  anon_sym_path = 25,
  anon_sym_stmt = 26,
  anon_sym_tt = 27,
  anon_sym_ty = 28,
  anon_sym_vis = 29,
  anon_sym_u8 = 30,
  anon_sym_i8 = 31,
  anon_sym_u16 = 32,
  anon_sym_i16 = 33,
  anon_sym_u32 = 34,
  anon_sym_i32 = 35,
  anon_sym_u64 = 36,
  anon_sym_i64 = 37,
  anon_sym_u128 = 38,
  anon_sym_i128 = 39,
  anon_sym_isize = 40,
  anon_sym_usize = 41,
  anon_sym_f32 = 42,
  anon_sym_f64 = 43,
  anon_sym_bool = 44,
  anon_sym_str = 45,
  anon_sym_char = 46,
  anon_sym_DASH = 47,
  anon_sym_SLASH = 48,
  anon_sym_PERCENT = 49,
  anon_sym_CARET = 50,
  anon_sym_BANG = 51,
  anon_sym_AMP = 52,
  anon_sym_PIPE = 53,
  anon_sym_AMP_AMP = 54,
  anon_sym_PIPE_PIPE = 55,
  anon_sym_LT_LT = 56,
  anon_sym_GT_GT = 57,
  anon_sym_PLUS_EQ = 58,
  anon_sym_DASH_EQ = 59,
  anon_sym_STAR_EQ = 60,
  anon_sym_SLASH_EQ = 61,
  anon_sym_PERCENT_EQ = 62,
  anon_sym_CARET_EQ = 63,
  anon_sym_AMP_EQ = 64,
  anon_sym_PIPE_EQ = 65,
  anon_sym_LT_LT_EQ = 66,
  anon_sym_GT_GT_EQ = 67,
  anon_sym_EQ = 68,
  anon_sym_EQ_EQ = 69,
  anon_sym_BANG_EQ = 70,
  anon_sym_GT = 71,
  anon_sym_LT = 72,
  anon_sym_GT_EQ = 73,
  anon_sym_LT_EQ = 74,
  anon_sym_AT = 75,
  anon_sym__ = 76,
  anon_sym_DOT = 77,
  anon_sym_DOT_DOT = 78,
  anon_sym_DOT_DOT_DOT = 79,
  anon_sym_DOT_DOT_EQ = 80,
  anon_sym_COMMA = 81,
  anon_sym_COLON_COLON = 82,
  anon_sym_DASH_GT = 83,
  anon_sym_POUND = 84,
  anon_sym_SQUOTE = 85,
  anon_sym_as = 86,
  anon_sym_async = 87,
  anon_sym_await = 88,
  anon_sym_break = 89,
  anon_sym_const = 90,
  anon_sym_continue = 91,
  anon_sym_default = 92,
  anon_sym_enum = 93,
  anon_sym_fn = 94,
  anon_sym_for = 95,
  anon_sym_gen = 96,
  anon_sym_if = 97,
  anon_sym_impl = 98,
  anon_sym_let = 99,
  anon_sym_loop = 100,
  anon_sym_match = 101,
  anon_sym_mod = 102,
  anon_sym_pub = 103,
  anon_sym_return = 104,
  anon_sym_static = 105,
  anon_sym_struct = 106,
  anon_sym_trait = 107,
  anon_sym_type = 108,
  anon_sym_union = 109,
  anon_sym_unsafe = 110,
  anon_sym_use = 111,
  anon_sym_where = 112,
  anon_sym_while = 113,
  anon_sym_extern = 114,
  anon_sym_ref = 115,
  anon_sym_else = 116,
  anon_sym_in = 117,
  anon_sym_LT2 = 118,
  anon_sym_dyn = 119,
  sym_mutable_specifier = 120,
  anon_sym_raw = 121,
  anon_sym_yield = 122,
  anon_sym_move = 123,
  anon_sym_try = 124,
  sym_integer_literal = 125,
  aux_sym_string_literal_token1 = 126,
  anon_sym_DQUOTE = 127,
  sym_char_literal = 128,
  sym_escape_sequence = 129,
  anon_sym_true = 130,
  anon_sym_false = 131,
  anon_sym_SLASH_SLASH = 132,
  aux_sym_line_comment_token1 = 133,
  aux_sym_line_comment_token2 = 134,
  aux_sym_line_comment_token3 = 135,
  anon_sym_BANG2 = 136,
  anon_sym_SLASH2 = 137,
  anon_sym_SLASH_STAR = 138,
  anon_sym_STAR_SLASH = 139,
  sym_shebang = 140,
  sym_self = 141,
  sym_super = 142,
  sym_crate = 143,
  sym_metavariable = 144,
  sym_string_content = 145,
  sym__raw_string_literal_start = 146,
  sym_raw_string_literal_content = 147,
  sym__raw_string_literal_end = 148,
  sym_float_literal = 149,
  sym__outer_block_doc_comment_marker = 150,
  sym__inner_block_doc_comment_marker = 151,
  sym__block_comment_content = 152,
  sym__line_doc_content = 153,
  sym__error_sentinel = 154,
  sym_source_file = 155,
  sym__statement = 156,
  sym_empty_statement = 157,
  sym_expression_statement = 158,
  sym_macro_definition = 159,
  sym_macro_rule = 160,
  sym__token_pattern = 161,
  sym_token_tree_pattern = 162,
  sym_token_binding_pattern = 163,
  sym_token_repetition_pattern = 164,
  sym_fragment_specifier = 165,
  sym_token_tree = 166,
  sym_token_repetition = 167,
  sym_attribute_item = 168,
  sym_inner_attribute_item = 169,
  sym_attribute = 170,
  sym_mod_item = 171,
  sym_foreign_mod_item = 172,
  sym_declaration_list = 173,
  sym_struct_item = 174,
  sym_union_item = 175,
  sym_enum_item = 176,
  sym_enum_variant_list = 177,
  sym_enum_variant = 178,
  sym_field_declaration_list = 179,
  sym_field_declaration = 180,
  sym_ordered_field_declaration_list = 181,
  sym_extern_crate_declaration = 182,
  sym_const_item = 183,
  sym_static_item = 184,
  sym_type_item = 185,
  sym_function_item = 186,
  sym_function_signature_item = 187,
  sym_function_modifiers = 188,
  sym_where_clause = 189,
  sym_where_predicate = 190,
  sym_impl_item = 191,
  sym_trait_item = 192,
  sym_associated_type = 193,
  sym_trait_bounds = 194,
  sym_higher_ranked_trait_bound = 195,
  sym_removed_trait_bound = 196,
  sym_type_parameters = 197,
  sym_const_parameter = 198,
  sym_constrained_type_parameter = 199,
  sym_optional_type_parameter = 200,
  sym_let_declaration = 201,
  sym_use_declaration = 202,
  sym__use_clause = 203,
  sym_scoped_use_list = 204,
  sym_use_list = 205,
  sym_use_as_clause = 206,
  sym_use_wildcard = 207,
  sym_parameters = 208,
  sym_self_parameter = 209,
  sym_variadic_parameter = 210,
  sym_parameter = 211,
  sym_extern_modifier = 212,
  sym_visibility_modifier = 213,
  sym__type = 214,
  sym_bracketed_type = 215,
  sym_qualified_type = 216,
  sym_lifetime = 217,
  sym_array_type = 218,
  sym_for_lifetimes = 219,
  sym_function_type = 220,
  sym_tuple_type = 221,
  sym_unit_type = 222,
  sym_generic_function = 223,
  sym_generic_type = 224,
  sym_generic_type_with_turbofish = 225,
  sym_bounded_type = 226,
  sym_type_arguments = 227,
  sym_type_binding = 228,
  sym_reference_type = 229,
  sym_pointer_type = 230,
  sym_never_type = 231,
  sym_abstract_type = 232,
  sym_dynamic_type = 233,
  sym__expression_except_range = 234,
  sym__expression = 235,
  sym_macro_invocation = 236,
  sym_delim_token_tree = 237,
  sym__delim_tokens = 238,
  sym__non_delim_token = 239,
  sym_scoped_identifier = 240,
  sym_scoped_type_identifier_in_expression_position = 241,
  sym_scoped_type_identifier = 242,
  sym_range_expression = 243,
  sym_unary_expression = 244,
  sym_try_expression = 245,
  sym_reference_expression = 246,
  sym_binary_expression = 247,
  sym_assignment_expression = 248,
  sym_compound_assignment_expr = 249,
  sym_type_cast_expression = 250,
  sym_return_expression = 251,
  sym_yield_expression = 252,
  sym_call_expression = 253,
  sym_arguments = 254,
  sym_array_expression = 255,
  sym_parenthesized_expression = 256,
  sym_tuple_expression = 257,
  sym_unit_expression = 258,
  sym_struct_expression = 259,
  sym_field_initializer_list = 260,
  sym_shorthand_field_initializer = 261,
  sym_field_initializer = 262,
  sym_base_field_initializer = 263,
  sym_if_expression = 264,
  sym_let_condition = 265,
  sym__let_chain = 266,
  sym__condition = 267,
  sym_else_clause = 268,
  sym_match_expression = 269,
  sym_match_block = 270,
  sym_match_arm = 271,
  sym_last_match_arm = 272,
  sym_match_pattern = 273,
  sym_while_expression = 274,
  sym_loop_expression = 275,
  sym_for_expression = 276,
  sym_const_block = 277,
  sym_closure_expression = 278,
  sym_closure_parameters = 279,
  sym_label = 280,
  sym_break_expression = 281,
  sym_continue_expression = 282,
  sym_index_expression = 283,
  sym_await_expression = 284,
  sym_field_expression = 285,
  sym_unsafe_block = 286,
  sym_async_block = 287,
  sym_gen_block = 288,
  sym_try_block = 289,
  sym_block = 290,
  sym__pattern = 291,
  sym_tuple_pattern = 292,
  sym_slice_pattern = 293,
  sym_tuple_struct_pattern = 294,
  sym_struct_pattern = 295,
  sym_field_pattern = 296,
  sym_remaining_field_pattern = 297,
  sym_mut_pattern = 298,
  sym_range_pattern = 299,
  sym_ref_pattern = 300,
  sym_captured_pattern = 301,
  sym_reference_pattern = 302,
  sym_or_pattern = 303,
  sym__literal = 304,
  sym__literal_pattern = 305,
  sym_negative_literal = 306,
  sym_string_literal = 307,
  sym_raw_string_literal = 308,
  sym_boolean_literal = 309,
  sym_line_comment = 310,
  sym__line_doc_comment_marker = 311,
  sym__inner_line_doc_comment_marker = 312,
  sym__outer_line_doc_comment_marker = 313,
  sym_block_comment = 314,
  sym__block_doc_comment_marker = 315,
  aux_sym_source_file_repeat1 = 316,
  aux_sym_macro_definition_repeat1 = 317,
  aux_sym_token_tree_pattern_repeat1 = 318,
  aux_sym_token_tree_repeat1 = 319,
  aux_sym__non_special_token_repeat1 = 320,
  aux_sym_declaration_list_repeat1 = 321,
  aux_sym_enum_variant_list_repeat1 = 322,
  aux_sym_enum_variant_list_repeat2 = 323,
  aux_sym_field_declaration_list_repeat1 = 324,
  aux_sym_ordered_field_declaration_list_repeat1 = 325,
  aux_sym_function_modifiers_repeat1 = 326,
  aux_sym_where_clause_repeat1 = 327,
  aux_sym_trait_bounds_repeat1 = 328,
  aux_sym_type_parameters_repeat1 = 329,
  aux_sym_use_list_repeat1 = 330,
  aux_sym_parameters_repeat1 = 331,
  aux_sym_for_lifetimes_repeat1 = 332,
  aux_sym_tuple_type_repeat1 = 333,
  aux_sym_type_arguments_repeat1 = 334,
  aux_sym_delim_token_tree_repeat1 = 335,
  aux_sym_arguments_repeat1 = 336,
  aux_sym_tuple_expression_repeat1 = 337,
  aux_sym_field_initializer_list_repeat1 = 338,
  aux_sym_match_block_repeat1 = 339,
  aux_sym_match_arm_repeat1 = 340,
  aux_sym_closure_parameters_repeat1 = 341,
  aux_sym_tuple_pattern_repeat1 = 342,
  aux_sym_slice_pattern_repeat1 = 343,
  aux_sym_struct_pattern_repeat1 = 344,
  aux_sym_string_literal_repeat1 = 345,
  alias_sym_field_identifier = 346,
  alias_sym_let_chain = 347,
  alias_sym_shorthand_field_identifier = 348,
  alias_sym_type_identifier = 349,
};

static const char * const ts_symbol_names[] = {
  [ts_builtin_sym_end] = "end",
  [sym_identifier] = "identifier",
  [anon_sym_SEMI] = ";",
  [anon_sym_macro_rules_BANG] = "macro_rules!",
  [anon_sym_LPAREN] = "(",
  [anon_sym_RPAREN] = ")",
  [anon_sym_LBRACK] = "[",
  [anon_sym_RBRACK] = "]",
  [anon_sym_LBRACE] = "{",
  [anon_sym_RBRACE] = "}",
  [anon_sym_EQ_GT] = "=>",
  [anon_sym_COLON] = ":",
  [anon_sym_DOLLAR] = "$",
  [aux_sym_token_repetition_pattern_token1] = "token_repetition_pattern_token1",
  [anon_sym_PLUS] = "+",
  [anon_sym_STAR] = "*",
  [anon_sym_QMARK] = "\?",
  [anon_sym_block] = "block",
  [anon_sym_expr] = "expr",
  [anon_sym_ident] = "ident",
  [anon_sym_item] = "item",
  [anon_sym_lifetime] = "lifetime",
  [anon_sym_literal] = "literal",
  [anon_sym_meta] = "meta",
  [anon_sym_pat] = "pat",
  [anon_sym_path] = "path",
  [anon_sym_stmt] = "stmt",
  [anon_sym_tt] = "tt",
  [anon_sym_ty] = "ty",
  [anon_sym_vis] = "vis",
  [anon_sym_u8] = "primitive_type",
  [anon_sym_i8] = "primitive_type",
  [anon_sym_u16] = "primitive_type",
  [anon_sym_i16] = "primitive_type",
  [anon_sym_u32] = "primitive_type",
  [anon_sym_i32] = "primitive_type",
  [anon_sym_u64] = "primitive_type",
  [anon_sym_i64] = "primitive_type",
  [anon_sym_u128] = "primitive_type",
  [anon_sym_i128] = "primitive_type",
  [anon_sym_isize] = "primitive_type",
  [anon_sym_usize] = "primitive_type",
  [anon_sym_f32] = "primitive_type",
  [anon_sym_f64] = "primitive_type",
  [anon_sym_bool] = "primitive_type",
  [anon_sym_str] = "primitive_type",
  [anon_sym_char] = "primitive_type",
  [anon_sym_DASH] = "-",
  [anon_sym_SLASH] = "/",
  [anon_sym_PERCENT] = "%",
  [anon_sym_CARET] = "^",
  [anon_sym_BANG] = "!",
  [anon_sym_AMP] = "&",
  [anon_sym_PIPE] = "|",
  [anon_sym_AMP_AMP] = "&&",
  [anon_sym_PIPE_PIPE] = "||",
  [anon_sym_LT_LT] = "<<",
  [anon_sym_GT_GT] = ">>",
  [anon_sym_PLUS_EQ] = "+=",
  [anon_sym_DASH_EQ] = "-=",
  [anon_sym_STAR_EQ] = "*=",
  [anon_sym_SLASH_EQ] = "/=",
  [anon_sym_PERCENT_EQ] = "%=",
  [anon_sym_CARET_EQ] = "^=",
  [anon_sym_AMP_EQ] = "&=",
  [anon_sym_PIPE_EQ] = "|=",
  [anon_sym_LT_LT_EQ] = "<<=",
  [anon_sym_GT_GT_EQ] = ">>=",
  [anon_sym_EQ] = "=",
  [anon_sym_EQ_EQ] = "==",
  [anon_sym_BANG_EQ] = "!=",
  [anon_sym_GT] = ">",
  [anon_sym_LT] = "<",
  [anon_sym_GT_EQ] = ">=",
  [anon_sym_LT_EQ] = "<=",
  [anon_sym_AT] = "@",
  [anon_sym__] = "_",
  [anon_sym_DOT] = ".",
  [anon_sym_DOT_DOT] = "..",
  [anon_sym_DOT_DOT_DOT] = "...",
  [anon_sym_DOT_DOT_EQ] = "..=",
  [anon_sym_COMMA] = ",",
  [anon_sym_COLON_COLON] = "::",
  [anon_sym_DASH_GT] = "->",
  [anon_sym_POUND] = "#",
  [anon_sym_SQUOTE] = "'",
  [anon_sym_as] = "as",
  [anon_sym_async] = "async",
  [anon_sym_await] = "await",
  [anon_sym_break] = "break",
  [anon_sym_const] = "const",
  [anon_sym_continue] = "continue",
  [anon_sym_default] = "default",
  [anon_sym_enum] = "enum",
  [anon_sym_fn] = "fn",
  [anon_sym_for] = "for",
  [anon_sym_gen] = "gen",
  [anon_sym_if] = "if",
  [anon_sym_impl] = "impl",
  [anon_sym_let] = "let",
  [anon_sym_loop] = "loop",
  [anon_sym_match] = "match",
  [anon_sym_mod] = "mod",
  [anon_sym_pub] = "pub",
  [anon_sym_return] = "return",
  [anon_sym_static] = "static",
  [anon_sym_struct] = "struct",
  [anon_sym_trait] = "trait",
  [anon_sym_type] = "type",
  [anon_sym_union] = "union",
  [anon_sym_unsafe] = "unsafe",
  [anon_sym_use] = "use",
  [anon_sym_where] = "where",
  [anon_sym_while] = "while",
  [anon_sym_extern] = "extern",
  [anon_sym_ref] = "ref",
  [anon_sym_else] = "else",
  [anon_sym_in] = "in",
  [anon_sym_LT2] = "<",
  [anon_sym_dyn] = "dyn",
  [sym_mutable_specifier] = "mutable_specifier",
  [anon_sym_raw] = "raw",
  [anon_sym_yield] = "yield",
  [anon_sym_move] = "move",
  [anon_sym_try] = "try",
  [sym_integer_literal] = "integer_literal",
  [aux_sym_string_literal_token1] = "\"",
  [anon_sym_DQUOTE] = "\"",
  [sym_char_literal] = "char_literal",
  [sym_escape_sequence] = "escape_sequence",
  [anon_sym_true] = "true",
  [anon_sym_false] = "false",
  [anon_sym_SLASH_SLASH] = "//",
  [aux_sym_line_comment_token1] = "line_comment_token1",
  [aux_sym_line_comment_token2] = "line_comment_token2",
  [aux_sym_line_comment_token3] = "line_comment_token3",
  [anon_sym_BANG2] = "!",
  [anon_sym_SLASH2] = "/",
  [anon_sym_SLASH_STAR] = "/*",
  [anon_sym_STAR_SLASH] = "*/",
  [sym_shebang] = "shebang",
  [sym_self] = "self",
  [sym_super] = "super",
  [sym_crate] = "crate",
  [sym_metavariable] = "metavariable",
  [sym_string_content] = "string_content",
  [sym__raw_string_literal_start] = "_raw_string_literal_start",
  [sym_raw_string_literal_content] = "string_content",
  [sym__raw_string_literal_end] = "_raw_string_literal_end",
  [sym_float_literal] = "float_literal",
  [sym__outer_block_doc_comment_marker] = "outer_doc_comment_marker",
  [sym__inner_block_doc_comment_marker] = "inner_doc_comment_marker",
  [sym__block_comment_content] = "_block_comment_content",
  [sym__line_doc_content] = "doc_comment",
  [sym__error_sentinel] = "_error_sentinel",
  [sym_source_file] = "source_file",
  [sym__statement] = "_statement",
  [sym_empty_statement] = "empty_statement",
  [sym_expression_statement] = "expression_statement",
  [sym_macro_definition] = "macro_definition",
  [sym_macro_rule] = "macro_rule",
  [sym__token_pattern] = "_token_pattern",
  [sym_token_tree_pattern] = "token_tree_pattern",
  [sym_token_binding_pattern] = "token_binding_pattern",
  [sym_token_repetition_pattern] = "token_repetition_pattern",
  [sym_fragment_specifier] = "fragment_specifier",
  [sym_token_tree] = "token_tree",
  [sym_token_repetition] = "token_repetition",
  [sym_attribute_item] = "attribute_item",
  [sym_inner_attribute_item] = "inner_attribute_item",
  [sym_attribute] = "attribute",
  [sym_mod_item] = "mod_item",
  [sym_foreign_mod_item] = "foreign_mod_item",
  [sym_declaration_list] = "declaration_list",
  [sym_struct_item] = "struct_item",
  [sym_union_item] = "union_item",
  [sym_enum_item] = "enum_item",
  [sym_enum_variant_list] = "enum_variant_list",
  [sym_enum_variant] = "enum_variant",
  [sym_field_declaration_list] = "field_declaration_list",
  [sym_field_declaration] = "field_declaration",
  [sym_ordered_field_declaration_list] = "ordered_field_declaration_list",
  [sym_extern_crate_declaration] = "extern_crate_declaration",
  [sym_const_item] = "const_item",
  [sym_static_item] = "static_item",
  [sym_type_item] = "type_item",
  [sym_function_item] = "function_item",
  [sym_function_signature_item] = "function_signature_item",
  [sym_function_modifiers] = "function_modifiers",
  [sym_where_clause] = "where_clause",
  [sym_where_predicate] = "where_predicate",
  [sym_impl_item] = "impl_item",
  [sym_trait_item] = "trait_item",
  [sym_associated_type] = "associated_type",
  [sym_trait_bounds] = "trait_bounds",
  [sym_higher_ranked_trait_bound] = "higher_ranked_trait_bound",
  [sym_removed_trait_bound] = "removed_trait_bound",
  [sym_type_parameters] = "type_parameters",
  [sym_const_parameter] = "const_parameter",
  [sym_constrained_type_parameter] = "constrained_type_parameter",
  [sym_optional_type_parameter] = "optional_type_parameter",
  [sym_let_declaration] = "let_declaration",
  [sym_use_declaration] = "use_declaration",
  [sym__use_clause] = "_use_clause",
  [sym_scoped_use_list] = "scoped_use_list",
  [sym_use_list] = "use_list",
  [sym_use_as_clause] = "use_as_clause",
  [sym_use_wildcard] = "use_wildcard",
  [sym_parameters] = "parameters",
  [sym_self_parameter] = "self_parameter",
  [sym_variadic_parameter] = "variadic_parameter",
  [sym_parameter] = "parameter",
  [sym_extern_modifier] = "extern_modifier",
  [sym_visibility_modifier] = "visibility_modifier",
  [sym__type] = "_type",
  [sym_bracketed_type] = "bracketed_type",
  [sym_qualified_type] = "qualified_type",
  [sym_lifetime] = "lifetime",
  [sym_array_type] = "array_type",
  [sym_for_lifetimes] = "for_lifetimes",
  [sym_function_type] = "function_type",
  [sym_tuple_type] = "tuple_type",
  [sym_unit_type] = "unit_type",
  [sym_generic_function] = "generic_function",
  [sym_generic_type] = "generic_type",
  [sym_generic_type_with_turbofish] = "generic_type_with_turbofish",
  [sym_bounded_type] = "bounded_type",
  [sym_type_arguments] = "type_arguments",
  [sym_type_binding] = "type_binding",
  [sym_reference_type] = "reference_type",
  [sym_pointer_type] = "pointer_type",
  [sym_never_type] = "never_type",
  [sym_abstract_type] = "abstract_type",
  [sym_dynamic_type] = "dynamic_type",
  [sym__expression_except_range] = "_expression_except_range",
  [sym__expression] = "_expression",
  [sym_macro_invocation] = "macro_invocation",
  [sym_delim_token_tree] = "token_tree",
  [sym__delim_tokens] = "_delim_tokens",
  [sym__non_delim_token] = "_non_delim_token",
  [sym_scoped_identifier] = "scoped_identifier",
  [sym_scoped_type_identifier_in_expression_position] = "scoped_type_identifier",
  [sym_scoped_type_identifier] = "scoped_type_identifier",
  [sym_range_expression] = "range_expression",
  [sym_unary_expression] = "unary_expression",
  [sym_try_expression] = "try_expression",
  [sym_reference_expression] = "reference_expression",
  [sym_binary_expression] = "binary_expression",
  [sym_assignment_expression] = "assignment_expression",
  [sym_compound_assignment_expr] = "compound_assignment_expr",
  [sym_type_cast_expression] = "type_cast_expression",
  [sym_return_expression] = "return_expression",
  [sym_yield_expression] = "yield_expression",
  [sym_call_expression] = "call_expression",
  [sym_arguments] = "arguments",
  [sym_array_expression] = "array_expression",
  [sym_parenthesized_expression] = "parenthesized_expression",
  [sym_tuple_expression] = "tuple_expression",
  [sym_unit_expression] = "unit_expression",
  [sym_struct_expression] = "struct_expression",
  [sym_field_initializer_list] = "field_initializer_list",
  [sym_shorthand_field_initializer] = "shorthand_field_initializer",
  [sym_field_initializer] = "field_initializer",
  [sym_base_field_initializer] = "base_field_initializer",
  [sym_if_expression] = "if_expression",
  [sym_let_condition] = "let_condition",
  [sym__let_chain] = "_let_chain",
  [sym__condition] = "_condition",
  [sym_else_clause] = "else_clause",
  [sym_match_expression] = "match_expression",
  [sym_match_block] = "match_block",
  [sym_match_arm] = "match_arm",
  [sym_last_match_arm] = "match_arm",
  [sym_match_pattern] = "match_pattern",
  [sym_while_expression] = "while_expression",
  [sym_loop_expression] = "loop_expression",
  [sym_for_expression] = "for_expression",
  [sym_const_block] = "const_block",
  [sym_closure_expression] = "closure_expression",
  [sym_closure_parameters] = "closure_parameters",
  [sym_label] = "label",
  [sym_break_expression] = "break_expression",
  [sym_continue_expression] = "continue_expression",
  [sym_index_expression] = "index_expression",
  [sym_await_expression] = "await_expression",
  [sym_field_expression] = "field_expression",
  [sym_unsafe_block] = "unsafe_block",
  [sym_async_block] = "async_block",
  [sym_gen_block] = "gen_block",
  [sym_try_block] = "try_block",
  [sym_block] = "block",
  [sym__pattern] = "_pattern",
  [sym_tuple_pattern] = "tuple_pattern",
  [sym_slice_pattern] = "slice_pattern",
  [sym_tuple_struct_pattern] = "tuple_struct_pattern",
  [sym_struct_pattern] = "struct_pattern",
  [sym_field_pattern] = "field_pattern",
  [sym_remaining_field_pattern] = "remaining_field_pattern",
  [sym_mut_pattern] = "mut_pattern",
  [sym_range_pattern] = "range_pattern",
  [sym_ref_pattern] = "ref_pattern",
  [sym_captured_pattern] = "captured_pattern",
  [sym_reference_pattern] = "reference_pattern",
  [sym_or_pattern] = "or_pattern",
  [sym__literal] = "_literal",
  [sym__literal_pattern] = "_literal_pattern",
  [sym_negative_literal] = "negative_literal",
  [sym_string_literal] = "string_literal",
  [sym_raw_string_literal] = "raw_string_literal",
  [sym_boolean_literal] = "boolean_literal",
  [sym_line_comment] = "line_comment",
  [sym__line_doc_comment_marker] = "_line_doc_comment_marker",
  [sym__inner_line_doc_comment_marker] = "inner_doc_comment_marker",
  [sym__outer_line_doc_comment_marker] = "outer_doc_comment_marker",
  [sym_block_comment] = "block_comment",
  [sym__block_doc_comment_marker] = "_block_doc_comment_marker",
  [aux_sym_source_file_repeat1] = "source_file_repeat1",
  [aux_sym_macro_definition_repeat1] = "macro_definition_repeat1",
  [aux_sym_token_tree_pattern_repeat1] = "token_tree_pattern_repeat1",
  [aux_sym_token_tree_repeat1] = "token_tree_repeat1",
  [aux_sym__non_special_token_repeat1] = "_non_special_token_repeat1",
  [aux_sym_declaration_list_repeat1] = "declaration_list_repeat1",
  [aux_sym_enum_variant_list_repeat1] = "enum_variant_list_repeat1",
  [aux_sym_enum_variant_list_repeat2] = "enum_variant_list_repeat2",
  [aux_sym_field_declaration_list_repeat1] = "field_declaration_list_repeat1",
  [aux_sym_ordered_field_declaration_list_repeat1] = "ordered_field_declaration_list_repeat1",
  [aux_sym_function_modifiers_repeat1] = "function_modifiers_repeat1",
  [aux_sym_where_clause_repeat1] = "where_clause_repeat1",
  [aux_sym_trait_bounds_repeat1] = "trait_bounds_repeat1",
  [aux_sym_type_parameters_repeat1] = "type_parameters_repeat1",
  [aux_sym_use_list_repeat1] = "use_list_repeat1",
  [aux_sym_parameters_repeat1] = "parameters_repeat1",
  [aux_sym_for_lifetimes_repeat1] = "for_lifetimes_repeat1",
  [aux_sym_tuple_type_repeat1] = "tuple_type_repeat1",
  [aux_sym_type_arguments_repeat1] = "type_arguments_repeat1",
  [aux_sym_delim_token_tree_repeat1] = "delim_token_tree_repeat1",
  [aux_sym_arguments_repeat1] = "arguments_repeat1",
  [aux_sym_tuple_expression_repeat1] = "tuple_expression_repeat1",
  [aux_sym_field_initializer_list_repeat1] = "field_initializer_list_repeat1",
  [aux_sym_match_block_repeat1] = "match_block_repeat1",
  [aux_sym_match_arm_repeat1] = "match_arm_repeat1",
  [aux_sym_closure_parameters_repeat1] = "closure_parameters_repeat1",
  [aux_sym_tuple_pattern_repeat1] = "tuple_pattern_repeat1",
  [aux_sym_slice_pattern_repeat1] = "slice_pattern_repeat1",
  [aux_sym_struct_pattern_repeat1] = "struct_pattern_repeat1",
  [aux_sym_string_literal_repeat1] = "string_literal_repeat1",
  [alias_sym_field_identifier] = "field_identifier",
  [alias_sym_let_chain] = "let_chain",
  [alias_sym_shorthand_field_identifier] = "shorthand_field_identifier",
  [alias_sym_type_identifier] = "type_identifier",
};

static const TSSymbol ts_symbol_map[] = {
  [ts_builtin_sym_end] = ts_builtin_sym_end,
  [sym_identifier] = sym_identifier,
  [anon_sym_SEMI] = anon_sym_SEMI,
  [anon_sym_macro_rules_BANG] = anon_sym_macro_rules_BANG,
  [anon_sym_LPAREN] = anon_sym_LPAREN,
  [anon_sym_RPAREN] = anon_sym_RPAREN,
  [anon_sym_LBRACK] = anon_sym_LBRACK,
  [anon_sym_RBRACK] = anon_sym_RBRACK,
  [anon_sym_LBRACE] = anon_sym_LBRACE,
  [anon_sym_RBRACE] = anon_sym_RBRACE,
  [anon_sym_EQ_GT] = anon_sym_EQ_GT,
  [anon_sym_COLON] = anon_sym_COLON,
  [anon_sym_DOLLAR] = anon_sym_DOLLAR,
  [aux_sym_token_repetition_pattern_token1] = aux_sym_token_repetition_pattern_token1,
  [anon_sym_PLUS] = anon_sym_PLUS,
  [anon_sym_STAR] = anon_sym_STAR,
  [anon_sym_QMARK] = anon_sym_QMARK,
  [anon_sym_block] = anon_sym_block,
  [anon_sym_expr] = anon_sym_expr,
  [anon_sym_ident] = anon_sym_ident,
  [anon_sym_item] = anon_sym_item,
  [anon_sym_lifetime] = anon_sym_lifetime,
  [anon_sym_literal] = anon_sym_literal,
  [anon_sym_meta] = anon_sym_meta,
  [anon_sym_pat] = anon_sym_pat,
  [anon_sym_path] = anon_sym_path,
  [anon_sym_stmt] = anon_sym_stmt,
  [anon_sym_tt] = anon_sym_tt,
  [anon_sym_ty] = anon_sym_ty,
  [anon_sym_vis] = anon_sym_vis,
  [anon_sym_u8] = anon_sym_u8,
  [anon_sym_i8] = anon_sym_u8,
  [anon_sym_u16] = anon_sym_u8,
  [anon_sym_i16] = anon_sym_u8,
  [anon_sym_u32] = anon_sym_u8,
  [anon_sym_i32] = anon_sym_u8,
  [anon_sym_u64] = anon_sym_u8,
  [anon_sym_i64] = anon_sym_u8,
  [anon_sym_u128] = anon_sym_u8,
  [anon_sym_i128] = anon_sym_u8,
  [anon_sym_isize] = anon_sym_u8,
  [anon_sym_usize] = anon_sym_u8,
  [anon_sym_f32] = anon_sym_u8,
  [anon_sym_f64] = anon_sym_u8,
  [anon_sym_bool] = anon_sym_u8,
  [anon_sym_str] = anon_sym_u8,
  [anon_sym_char] = anon_sym_u8,
  [anon_sym_DASH] = anon_sym_DASH,
  [anon_sym_SLASH] = anon_sym_SLASH,
  [anon_sym_PERCENT] = anon_sym_PERCENT,
  [anon_sym_CARET] = anon_sym_CARET,
  [anon_sym_BANG] = anon_sym_BANG,
  [anon_sym_AMP] = anon_sym_AMP,
  [anon_sym_PIPE] = anon_sym_PIPE,
  [anon_sym_AMP_AMP] = anon_sym_AMP_AMP,
  [anon_sym_PIPE_PIPE] = anon_sym_PIPE_PIPE,
  [anon_sym_LT_LT] = anon_sym_LT_LT,
  [anon_sym_GT_GT] = anon_sym_GT_GT,
  [anon_sym_PLUS_EQ] = anon_sym_PLUS_EQ,
  [anon_sym_DASH_EQ] = anon_sym_DASH_EQ,
  [anon_sym_STAR_EQ] = anon_sym_STAR_EQ,
  [anon_sym_SLASH_EQ] = anon_sym_SLASH_EQ,
  [anon_sym_PERCENT_EQ] = anon_sym_PERCENT_EQ,
  [anon_sym_CARET_EQ] = anon_sym_CARET_EQ,
  [anon_sym_AMP_EQ] = anon_sym_AMP_EQ,
  [anon_sym_PIPE_EQ] = anon_sym_PIPE_EQ,
  [anon_sym_LT_LT_EQ] = anon_sym_LT_LT_EQ,
  [anon_sym_GT_GT_EQ] = anon_sym_GT_GT_EQ,
  [anon_sym_EQ] = anon_sym_EQ,
  [anon_sym_EQ_EQ] = anon_sym_EQ_EQ,
  [anon_sym_BANG_EQ] = anon_sym_BANG_EQ,
  [anon_sym_GT] = anon_sym_GT,
  [anon_sym_LT] = anon_sym_LT,
  [anon_sym_GT_EQ] = anon_sym_GT_EQ,
  [anon_sym_LT_EQ] = anon_sym_LT_EQ,
  [anon_sym_AT] = anon_sym_AT,
  [anon_sym__] = anon_sym__,
  [anon_sym_DOT] = anon_sym_DOT,
  [anon_sym_DOT_DOT] = anon_sym_DOT_DOT,
  [anon_sym_DOT_DOT_DOT] = anon_sym_DOT_DOT_DOT,
  [anon_sym_DOT_DOT_EQ] = anon_sym_DOT_DOT_EQ,
  [anon_sym_COMMA] = anon_sym_COMMA,
  [anon_sym_COLON_COLON] = anon_sym_COLON_COLON,
  [anon_sym_DASH_GT] = anon_sym_DASH_GT,
  [anon_sym_POUND] = anon_sym_POUND,
  [anon_sym_SQUOTE] = anon_sym_SQUOTE,
  [anon_sym_as] = anon_sym_as,
  [anon_sym_async] = anon_sym_async,
  [anon_sym_await] = anon_sym_await,
  [anon_sym_break] = anon_sym_break,
  [anon_sym_const] = anon_sym_const,
  [anon_sym_continue] = anon_sym_continue,
  [anon_sym_default] = anon_sym_default,
  [anon_sym_enum] = anon_sym_enum,
  [anon_sym_fn] = anon_sym_fn,
  [anon_sym_for] = anon_sym_for,
  [anon_sym_gen] = anon_sym_gen,
  [anon_sym_if] = anon_sym_if,
  [anon_sym_impl] = anon_sym_impl,
  [anon_sym_let] = anon_sym_let,
  [anon_sym_loop] = anon_sym_loop,
  [anon_sym_match] = anon_sym_match,
  [anon_sym_mod] = anon_sym_mod,
  [anon_sym_pub] = anon_sym_pub,
  [anon_sym_return] = anon_sym_return,
  [anon_sym_static] = anon_sym_static,
  [anon_sym_struct] = anon_sym_struct,
  [anon_sym_trait] = anon_sym_trait,
  [anon_sym_type] = anon_sym_type,
  [anon_sym_union] = anon_sym_union,
  [anon_sym_unsafe] = anon_sym_unsafe,
  [anon_sym_use] = anon_sym_use,
  [anon_sym_where] = anon_sym_where,
  [anon_sym_while] = anon_sym_while,
  [anon_sym_extern] = anon_sym_extern,
  [anon_sym_ref] = anon_sym_ref,
  [anon_sym_else] = anon_sym_else,
  [anon_sym_in] = anon_sym_in,
  [anon_sym_LT2] = anon_sym_LT,
  [anon_sym_dyn] = anon_sym_dyn,
  [sym_mutable_specifier] = sym_mutable_specifier,
  [anon_sym_raw] = anon_sym_raw,
  [anon_sym_yield] = anon_sym_yield,
  [anon_sym_move] = anon_sym_move,
  [anon_sym_try] = anon_sym_try,
  [sym_integer_literal] = sym_integer_literal,
  [aux_sym_string_literal_token1] = anon_sym_DQUOTE,
  [anon_sym_DQUOTE] = anon_sym_DQUOTE,
  [sym_char_literal] = sym_char_literal,
  [sym_escape_sequence] = sym_escape_sequence,
  [anon_sym_true] = anon_sym_true,
  [anon_sym_false] = anon_sym_false,
  [anon_sym_SLASH_SLASH] = anon_sym_SLASH_SLASH,
  [aux_sym_line_comment_token1] = aux_sym_line_comment_token1,
  [aux_sym_line_comment_token2] = aux_sym_line_comment_token2,
  [aux_sym_line_comment_token3] = aux_sym_line_comment_token3,
  [anon_sym_BANG2] = anon_sym_BANG,
  [anon_sym_SLASH2] = anon_sym_SLASH,
  [anon_sym_SLASH_STAR] = anon_sym_SLASH_STAR,
  [anon_sym_STAR_SLASH] = anon_sym_STAR_SLASH,
  [sym_shebang] = sym_shebang,
  [sym_self] = sym_self,
  [sym_super] = sym_super,
  [sym_crate] = sym_crate,
  [sym_metavariable] = sym_metavariable,
  [sym_string_content] = sym_string_content,
  [sym__raw_string_literal_start] = sym__raw_string_literal_start,
  [sym_raw_string_literal_content] = sym_string_content,
  [sym__raw_string_literal_end] = sym__raw_string_literal_end,
  [sym_float_literal] = sym_float_literal,
  [sym__outer_block_doc_comment_marker] = sym__outer_block_doc_comment_marker,
  [sym__inner_block_doc_comment_marker] = sym__inner_block_doc_comment_marker,
  [sym__block_comment_content] = sym__block_comment_content,
  [sym__line_doc_content] = sym__line_doc_content,
  [sym__error_sentinel] = sym__error_sentinel,
  [sym_source_file] = sym_source_file,
  [sym__statement] = sym__statement,
  [sym_empty_statement] = sym_empty_statement,
  [sym_expression_statement] = sym_expression_statement,
  [sym_macro_definition] = sym_macro_definition,
  [sym_macro_rule] = sym_macro_rule,
  [sym__token_pattern] = sym__token_pattern,
  [sym_token_tree_pattern] = sym_token_tree_pattern,
  [sym_token_binding_pattern] = sym_token_binding_pattern,
  [sym_token_repetition_pattern] = sym_token_repetition_pattern,
  [sym_fragment_specifier] = sym_fragment_specifier,
  [sym_token_tree] = sym_token_tree,
  [sym_token_repetition] = sym_token_repetition,
  [sym_attribute_item] = sym_attribute_item,
  [sym_inner_attribute_item] = sym_inner_attribute_item,
  [sym_attribute] = sym_attribute,
  [sym_mod_item] = sym_mod_item,
  [sym_foreign_mod_item] = sym_foreign_mod_item,
  [sym_declaration_list] = sym_declaration_list,
  [sym_struct_item] = sym_struct_item,
  [sym_union_item] = sym_union_item,
  [sym_enum_item] = sym_enum_item,
  [sym_enum_variant_list] = sym_enum_variant_list,
  [sym_enum_variant] = sym_enum_variant,
  [sym_field_declaration_list] = sym_field_declaration_list,
  [sym_field_declaration] = sym_field_declaration,
  [sym_ordered_field_declaration_list] = sym_ordered_field_declaration_list,
  [sym_extern_crate_declaration] = sym_extern_crate_declaration,
  [sym_const_item] = sym_const_item,
  [sym_static_item] = sym_static_item,
  [sym_type_item] = sym_type_item,
  [sym_function_item] = sym_function_item,
  [sym_function_signature_item] = sym_function_signature_item,
  [sym_function_modifiers] = sym_function_modifiers,
  [sym_where_clause] = sym_where_clause,
  [sym_where_predicate] = sym_where_predicate,
  [sym_impl_item] = sym_impl_item,
  [sym_trait_item] = sym_trait_item,
  [sym_associated_type] = sym_associated_type,
  [sym_trait_bounds] = sym_trait_bounds,
  [sym_higher_ranked_trait_bound] = sym_higher_ranked_trait_bound,
  [sym_removed_trait_bound] = sym_removed_trait_bound,
  [sym_type_parameters] = sym_type_parameters,
  [sym_const_parameter] = sym_const_parameter,
  [sym_constrained_type_parameter] = sym_constrained_type_parameter,
  [sym_optional_type_parameter] = sym_optional_type_parameter,
  [sym_let_declaration] = sym_let_declaration,
  [sym_use_declaration] = sym_use_declaration,
  [sym__use_clause] = sym__use_clause,
  [sym_scoped_use_list] = sym_scoped_use_list,
  [sym_use_list] = sym_use_list,
  [sym_use_as_clause] = sym_use_as_clause,
  [sym_use_wildcard] = sym_use_wildcard,
  [sym_parameters] = sym_parameters,
  [sym_self_parameter] = sym_self_parameter,
  [sym_variadic_parameter] = sym_variadic_parameter,
  [sym_parameter] = sym_parameter,
  [sym_extern_modifier] = sym_extern_modifier,
  [sym_visibility_modifier] = sym_visibility_modifier,
  [sym__type] = sym__type,
  [sym_bracketed_type] = sym_bracketed_type,
  [sym_qualified_type] = sym_qualified_type,
  [sym_lifetime] = sym_lifetime,
  [sym_array_type] = sym_array_type,
  [sym_for_lifetimes] = sym_for_lifetimes,
  [sym_function_type] = sym_function_type,
  [sym_tuple_type] = sym_tuple_type,
  [sym_unit_type] = sym_unit_type,
  [sym_generic_function] = sym_generic_function,
  [sym_generic_type] = sym_generic_type,
  [sym_generic_type_with_turbofish] = sym_generic_type_with_turbofish,
  [sym_bounded_type] = sym_bounded_type,
  [sym_type_arguments] = sym_type_arguments,
  [sym_type_binding] = sym_type_binding,
  [sym_reference_type] = sym_reference_type,
  [sym_pointer_type] = sym_pointer_type,
  [sym_never_type] = sym_never_type,
  [sym_abstract_type] = sym_abstract_type,
  [sym_dynamic_type] = sym_dynamic_type,
  [sym__expression_except_range] = sym__expression_except_range,
  [sym__expression] = sym__expression,
  [sym_macro_invocation] = sym_macro_invocation,
  [sym_delim_token_tree] = sym_token_tree,
  [sym__delim_tokens] = sym__delim_tokens,
  [sym__non_delim_token] = sym__non_delim_token,
  [sym_scoped_identifier] = sym_scoped_identifier,
  [sym_scoped_type_identifier_in_expression_position] = sym_scoped_type_identifier,
  [sym_scoped_type_identifier] = sym_scoped_type_identifier,
  [sym_range_expression] = sym_range_expression,
  [sym_unary_expression] = sym_unary_expression,
  [sym_try_expression] = sym_try_expression,
  [sym_reference_expression] = sym_reference_expression,
  [sym_binary_expression] = sym_binary_expression,
  [sym_assignment_expression] = sym_assignment_expression,
  [sym_compound_assignment_expr] = sym_compound_assignment_expr,
  [sym_type_cast_expression] = sym_type_cast_expression,
  [sym_return_expression] = sym_return_expression,
  [sym_yield_expression] = sym_yield_expression,
  [sym_call_expression] = sym_call_expression,
  [sym_arguments] = sym_arguments,
  [sym_array_expression] = sym_array_expression,
  [sym_parenthesized_expression] = sym_parenthesized_expression,
  [sym_tuple_expression] = sym_tuple_expression,
  [sym_unit_expression] = sym_unit_expression,
  [sym_struct_expression] = sym_struct_expression,
  [sym_field_initializer_list] = sym_field_initializer_list,
  [sym_shorthand_field_initializer] = sym_shorthand_field_initializer,
  [sym_field_initializer] = sym_field_initializer,
  [sym_base_field_initializer] = sym_base_field_initializer,
  [sym_if_expression] = sym_if_expression,
  [sym_let_condition] = sym_let_condition,
  [sym__let_chain] = sym__let_chain,
  [sym__condition] = sym__condition,
  [sym_else_clause] = sym_else_clause,
  [sym_match_expression] = sym_match_expression,
  [sym_match_block] = sym_match_block,
  [sym_match_arm] = sym_match_arm,
  [sym_last_match_arm] = sym_match_arm,
  [sym_match_pattern] = sym_match_pattern,
  [sym_while_expression] = sym_while_expression,
  [sym_loop_expression] = sym_loop_expression,
  [sym_for_expression] = sym_for_expression,
  [sym_const_block] = sym_const_block,
  [sym_closure_expression] = sym_closure_expression,
  [sym_closure_parameters] = sym_closure_parameters,
  [sym_label] = sym_label,
  [sym_break_expression] = sym_break_expression,
  [sym_continue_expression] = sym_continue_expression,
  [sym_index_expression] = sym_index_expression,
  [sym_await_expression] = sym_await_expression,
  [sym_field_expression] = sym_field_expression,
  [sym_unsafe_block] = sym_unsafe_block,
  [sym_async_block] = sym_async_block,
  [sym_gen_block] = sym_gen_block,
  [sym_try_block] = sym_try_block,
  [sym_block] = sym_block,
  [sym__pattern] = sym__pattern,
  [sym_tuple_pattern] = sym_tuple_pattern,
  [sym_slice_pattern] = sym_slice_pattern,
  [sym_tuple_struct_pattern] = sym_tuple_struct_pattern,
  [sym_struct_pattern] = sym_struct_pattern,
  [sym_field_pattern] = sym_field_pattern,
  [sym_remaining_field_pattern] = sym_remaining_field_pattern,
  [sym_mut_pattern] = sym_mut_pattern,
  [sym_range_pattern] = sym_range_pattern,
  [sym_ref_pattern] = sym_ref_pattern,
  [sym_captured_pattern] = sym_captured_pattern,
  [sym_reference_pattern] = sym_reference_pattern,
  [sym_or_pattern] = sym_or_pattern,
  [sym__literal] = sym__literal,
  [sym__literal_pattern] = sym__literal_pattern,
  [sym_negative_literal] = sym_negative_literal,
  [sym_string_literal] = sym_string_literal,
  [sym_raw_string_literal] = sym_raw_string_literal,
  [sym_boolean_literal] = sym_boolean_literal,
  [sym_line_comment] = sym_line_comment,
  [sym__line_doc_comment_marker] = sym__line_doc_comment_marker,
  [sym__inner_line_doc_comment_marker] = sym__inner_block_doc_comment_marker,
  [sym__outer_line_doc_comment_marker] = sym__outer_block_doc_comment_marker,
  [sym_block_comment] = sym_block_comment,
  [sym__block_doc_comment_marker] = sym__block_doc_comment_marker,
  [aux_sym_source_file_repeat1] = aux_sym_source_file_repeat1,
  [aux_sym_macro_definition_repeat1] = aux_sym_macro_definition_repeat1,
  [aux_sym_token_tree_pattern_repeat1] = aux_sym_token_tree_pattern_repeat1,
  [aux_sym_token_tree_repeat1] = aux_sym_token_tree_repeat1,
  [aux_sym__non_special_token_repeat1] = aux_sym__non_special_token_repeat1,
  [aux_sym_declaration_list_repeat1] = aux_sym_declaration_list_repeat1,
  [aux_sym_enum_variant_list_repeat1] = aux_sym_enum_variant_list_repeat1,
  [aux_sym_enum_variant_list_repeat2] = aux_sym_enum_variant_list_repeat2,
  [aux_sym_field_declaration_list_repeat1] = aux_sym_field_declaration_list_repeat1,
  [aux_sym_ordered_field_declaration_list_repeat1] = aux_sym_ordered_field_declaration_list_repeat1,
  [aux_sym_function_modifiers_repeat1] = aux_sym_function_modifiers_repeat1,
  [aux_sym_where_clause_repeat1] = aux_sym_where_clause_repeat1,
  [aux_sym_trait_bounds_repeat1] = aux_sym_trait_bounds_repeat1,
  [aux_sym_type_parameters_repeat1] = aux_sym_type_parameters_repeat1,
  [aux_sym_use_list_repeat1] = aux_sym_use_list_repeat1,
  [aux_sym_parameters_repeat1] = aux_sym_parameters_repeat1,
  [aux_sym_for_lifetimes_repeat1] = aux_sym_for_lifetimes_repeat1,
  [aux_sym_tuple_type_repeat1] = aux_sym_tuple_type_repeat1,
  [aux_sym_type_arguments_repeat1] = aux_sym_type_arguments_repeat1,
  [aux_sym_delim_token_tree_repeat1] = aux_sym_delim_token_tree_repeat1,
  [aux_sym_arguments_repeat1] = aux_sym_arguments_repeat1,
  [aux_sym_tuple_expression_repeat1] = aux_sym_tuple_expression_repeat1,
  [aux_sym_field_initializer_list_repeat1] = aux_sym_field_initializer_list_repeat1,
  [aux_sym_match_block_repeat1] = aux_sym_match_block_repeat1,
  [aux_sym_match_arm_repeat1] = aux_sym_match_arm_repeat1,
  [aux_sym_closure_parameters_repeat1] = aux_sym_closure_parameters_repeat1,
  [aux_sym_tuple_pattern_repeat1] = aux_sym_tuple_pattern_repeat1,
  [aux_sym_slice_pattern_repeat1] = aux_sym_slice_pattern_repeat1,
  [aux_sym_struct_pattern_repeat1] = aux_sym_struct_pattern_repeat1,
  [aux_sym_string_literal_repeat1] = aux_sym_string_literal_repeat1,
  [alias_sym_field_identifier] = alias_sym_field_identifier,
  [alias_sym_let_chain] = alias_sym_let_chain,
  [alias_sym_shorthand_field_identifier] = alias_sym_shorthand_field_identifier,
  [alias_sym_type_identifier] = alias_sym_type_identifier,
};

static const TSSymbolMetadata ts_symbol_metadata[] = {
  [ts_builtin_sym_end] = {
    .visible = false,
    .named = true,
  },
  [sym_identifier] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_SEMI] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_macro_rules_BANG] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LPAREN] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_RPAREN] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LBRACK] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_RBRACK] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LBRACE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_RBRACE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQ_GT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_COLON] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DOLLAR] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_token_repetition_pattern_token1] = {
    .visible = false,
    .named = false,
  },
  [anon_sym_PLUS] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STAR] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_QMARK] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_block] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_expr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_ident] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_item] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_lifetime] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_literal] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_meta] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_pat] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_path] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_stmt] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_tt] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_ty] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_vis] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_u8] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_i8] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_u16] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_i16] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_u32] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_i32] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_u64] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_i64] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_u128] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_i128] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_isize] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_usize] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_f32] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_f64] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_bool] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_str] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_char] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_DASH] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SLASH] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PERCENT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_CARET] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_BANG] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AMP] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PIPE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AMP_AMP] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PIPE_PIPE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT_LT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GT_GT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PLUS_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DASH_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STAR_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SLASH_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PERCENT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_CARET_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AMP_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PIPE_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT_LT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GT_GT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQ_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_BANG_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym__] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DOT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DOT_DOT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DOT_DOT_DOT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DOT_DOT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_COMMA] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_COLON_COLON] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DASH_GT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_POUND] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SQUOTE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_as] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_async] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_await] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_break] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_const] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_continue] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_default] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_enum] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_fn] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_for] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_gen] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_if] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_impl] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_let] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_loop] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_match] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_mod] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_pub] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_return] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_static] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_struct] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_trait] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_type] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_union] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_unsafe] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_use] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_where] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_while] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_extern] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_ref] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_else] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_in] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_dyn] = {
    .visible = true,
    .named = false,
  },
  [sym_mutable_specifier] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_raw] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_yield] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_move] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_try] = {
    .visible = true,
    .named = false,
  },
  [sym_integer_literal] = {
    .visible = true,
    .named = true,
  },
  [aux_sym_string_literal_token1] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DQUOTE] = {
    .visible = true,
    .named = false,
  },
  [sym_char_literal] = {
    .visible = true,
    .named = true,
  },
  [sym_escape_sequence] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_true] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_false] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SLASH_SLASH] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_line_comment_token1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_line_comment_token2] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_line_comment_token3] = {
    .visible = false,
    .named = false,
  },
  [anon_sym_BANG2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SLASH2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SLASH_STAR] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STAR_SLASH] = {
    .visible = true,
    .named = false,
  },
  [sym_shebang] = {
    .visible = true,
    .named = true,
  },
  [sym_self] = {
    .visible = true,
    .named = true,
  },
  [sym_super] = {
    .visible = true,
    .named = true,
  },
  [sym_crate] = {
    .visible = true,
    .named = true,
  },
  [sym_metavariable] = {
    .visible = true,
    .named = true,
  },
  [sym_string_content] = {
    .visible = true,
    .named = true,
  },
  [sym__raw_string_literal_start] = {
    .visible = false,
    .named = true,
  },
  [sym_raw_string_literal_content] = {
    .visible = true,
    .named = true,
  },
  [sym__raw_string_literal_end] = {
    .visible = false,
    .named = true,
  },
  [sym_float_literal] = {
    .visible = true,
    .named = true,
  },
  [sym__outer_block_doc_comment_marker] = {
    .visible = true,
    .named = true,
  },
  [sym__inner_block_doc_comment_marker] = {
    .visible = true,
    .named = true,
  },
  [sym__block_comment_content] = {
    .visible = false,
    .named = true,
  },
  [sym__line_doc_content] = {
    .visible = true,
    .named = true,
  },
  [sym__error_sentinel] = {
    .visible = false,
    .named = true,
  },
  [sym_source_file] = {
    .visible = true,
    .named = true,
  },
  [sym__statement] = {
    .visible = false,
    .named = true,
  },
  [sym_empty_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_expression_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_macro_definition] = {
    .visible = true,
    .named = true,
  },
  [sym_macro_rule] = {
    .visible = true,
    .named = true,
  },
  [sym__token_pattern] = {
    .visible = false,
    .named = true,
  },
  [sym_token_tree_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_token_binding_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_token_repetition_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_fragment_specifier] = {
    .visible = true,
    .named = true,
  },
  [sym_token_tree] = {
    .visible = true,
    .named = true,
  },
  [sym_token_repetition] = {
    .visible = true,
    .named = true,
  },
  [sym_attribute_item] = {
    .visible = true,
    .named = true,
  },
  [sym_inner_attribute_item] = {
    .visible = true,
    .named = true,
  },
  [sym_attribute] = {
    .visible = true,
    .named = true,
  },
  [sym_mod_item] = {
    .visible = true,
    .named = true,
  },
  [sym_foreign_mod_item] = {
    .visible = true,
    .named = true,
  },
  [sym_declaration_list] = {
    .visible = true,
    .named = true,
  },
  [sym_struct_item] = {
    .visible = true,
    .named = true,
  },
  [sym_union_item] = {
    .visible = true,
    .named = true,
  },
  [sym_enum_item] = {
    .visible = true,
    .named = true,
  },
  [sym_enum_variant_list] = {
    .visible = true,
    .named = true,
  },
  [sym_enum_variant] = {
    .visible = true,
    .named = true,
  },
  [sym_field_declaration_list] = {
    .visible = true,
    .named = true,
  },
  [sym_field_declaration] = {
    .visible = true,
    .named = true,
  },
  [sym_ordered_field_declaration_list] = {
    .visible = true,
    .named = true,
  },
  [sym_extern_crate_declaration] = {
    .visible = true,
    .named = true,
  },
  [sym_const_item] = {
    .visible = true,
    .named = true,
  },
  [sym_static_item] = {
    .visible = true,
    .named = true,
  },
  [sym_type_item] = {
    .visible = true,
    .named = true,
  },
  [sym_function_item] = {
    .visible = true,
    .named = true,
  },
  [sym_function_signature_item] = {
    .visible = true,
    .named = true,
  },
  [sym_function_modifiers] = {
    .visible = true,
    .named = true,
  },
  [sym_where_clause] = {
    .visible = true,
    .named = true,
  },
  [sym_where_predicate] = {
    .visible = true,
    .named = true,
  },
  [sym_impl_item] = {
    .visible = true,
    .named = true,
  },
  [sym_trait_item] = {
    .visible = true,
    .named = true,
  },
  [sym_associated_type] = {
    .visible = true,
    .named = true,
  },
  [sym_trait_bounds] = {
    .visible = true,
    .named = true,
  },
  [sym_higher_ranked_trait_bound] = {
    .visible = true,
    .named = true,
  },
  [sym_removed_trait_bound] = {
    .visible = true,
    .named = true,
  },
  [sym_type_parameters] = {
    .visible = true,
    .named = true,
  },
  [sym_const_parameter] = {
    .visible = true,
    .named = true,
  },
  [sym_constrained_type_parameter] = {
    .visible = true,
    .named = true,
  },
  [sym_optional_type_parameter] = {
    .visible = true,
    .named = true,
  },
  [sym_let_declaration] = {
    .visible = true,
    .named = true,
  },
  [sym_use_declaration] = {
    .visible = true,
    .named = true,
  },
  [sym__use_clause] = {
    .visible = false,
    .named = true,
  },
  [sym_scoped_use_list] = {
    .visible = true,
    .named = true,
  },
  [sym_use_list] = {
    .visible = true,
    .named = true,
  },
  [sym_use_as_clause] = {
    .visible = true,
    .named = true,
  },
  [sym_use_wildcard] = {
    .visible = true,
    .named = true,
  },
  [sym_parameters] = {
    .visible = true,
    .named = true,
  },
  [sym_self_parameter] = {
    .visible = true,
    .named = true,
  },
  [sym_variadic_parameter] = {
    .visible = true,
    .named = true,
  },
  [sym_parameter] = {
    .visible = true,
    .named = true,
  },
  [sym_extern_modifier] = {
    .visible = true,
    .named = true,
  },
  [sym_visibility_modifier] = {
    .visible = true,
    .named = true,
  },
  [sym__type] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym_bracketed_type] = {
    .visible = true,
    .named = true,
  },
  [sym_qualified_type] = {
    .visible = true,
    .named = true,
  },
  [sym_lifetime] = {
    .visible = true,
    .named = true,
  },
  [sym_array_type] = {
    .visible = true,
    .named = true,
  },
  [sym_for_lifetimes] = {
    .visible = true,
    .named = true,
  },
  [sym_function_type] = {
    .visible = true,
    .named = true,
  },
  [sym_tuple_type] = {
    .visible = true,
    .named = true,
  },
  [sym_unit_type] = {
    .visible = true,
    .named = true,
  },
  [sym_generic_function] = {
    .visible = true,
    .named = true,
  },
  [sym_generic_type] = {
    .visible = true,
    .named = true,
  },
  [sym_generic_type_with_turbofish] = {
    .visible = true,
    .named = true,
  },
  [sym_bounded_type] = {
    .visible = true,
    .named = true,
  },
  [sym_type_arguments] = {
    .visible = true,
    .named = true,
  },
  [sym_type_binding] = {
    .visible = true,
    .named = true,
  },
  [sym_reference_type] = {
    .visible = true,
    .named = true,
  },
  [sym_pointer_type] = {
    .visible = true,
    .named = true,
  },
  [sym_never_type] = {
    .visible = true,
    .named = true,
  },
  [sym_abstract_type] = {
    .visible = true,
    .named = true,
  },
  [sym_dynamic_type] = {
    .visible = true,
    .named = true,
  },
  [sym__expression_except_range] = {
    .visible = false,
    .named = true,
  },
  [sym__expression] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym_macro_invocation] = {
    .visible = true,
    .named = true,
  },
  [sym_delim_token_tree] = {
    .visible = true,
    .named = true,
  },
  [sym__delim_tokens] = {
    .visible = false,
    .named = true,
  },
  [sym__non_delim_token] = {
    .visible = false,
    .named = true,
  },
  [sym_scoped_identifier] = {
    .visible = true,
    .named = true,
  },
  [sym_scoped_type_identifier_in_expression_position] = {
    .visible = true,
    .named = true,
  },
  [sym_scoped_type_identifier] = {
    .visible = true,
    .named = true,
  },
  [sym_range_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_unary_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_try_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_reference_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_binary_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_assignment_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_compound_assignment_expr] = {
    .visible = true,
    .named = true,
  },
  [sym_type_cast_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_return_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_yield_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_call_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_arguments] = {
    .visible = true,
    .named = true,
  },
  [sym_array_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_parenthesized_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_tuple_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_unit_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_struct_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_field_initializer_list] = {
    .visible = true,
    .named = true,
  },
  [sym_shorthand_field_initializer] = {
    .visible = true,
    .named = true,
  },
  [sym_field_initializer] = {
    .visible = true,
    .named = true,
  },
  [sym_base_field_initializer] = {
    .visible = true,
    .named = true,
  },
  [sym_if_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_let_condition] = {
    .visible = true,
    .named = true,
  },
  [sym__let_chain] = {
    .visible = false,
    .named = true,
  },
  [sym__condition] = {
    .visible = false,
    .named = true,
  },
  [sym_else_clause] = {
    .visible = true,
    .named = true,
  },
  [sym_match_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_match_block] = {
    .visible = true,
    .named = true,
  },
  [sym_match_arm] = {
    .visible = true,
    .named = true,
  },
  [sym_last_match_arm] = {
    .visible = true,
    .named = true,
  },
  [sym_match_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_while_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_loop_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_for_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_const_block] = {
    .visible = true,
    .named = true,
  },
  [sym_closure_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_closure_parameters] = {
    .visible = true,
    .named = true,
  },
  [sym_label] = {
    .visible = true,
    .named = true,
  },
  [sym_break_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_continue_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_index_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_await_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_field_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_unsafe_block] = {
    .visible = true,
    .named = true,
  },
  [sym_async_block] = {
    .visible = true,
    .named = true,
  },
  [sym_gen_block] = {
    .visible = true,
    .named = true,
  },
  [sym_try_block] = {
    .visible = true,
    .named = true,
  },
  [sym_block] = {
    .visible = true,
    .named = true,
  },
  [sym__pattern] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym_tuple_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_slice_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_tuple_struct_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_struct_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_field_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_remaining_field_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_mut_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_range_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_ref_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_captured_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_reference_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_or_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym__literal] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym__literal_pattern] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym_negative_literal] = {
    .visible = true,
    .named = true,
  },
  [sym_string_literal] = {
    .visible = true,
    .named = true,
  },
  [sym_raw_string_literal] = {
    .visible = true,
    .named = true,
  },
  [sym_boolean_literal] = {
    .visible = true,
    .named = true,
  },
  [sym_line_comment] = {
    .visible = true,
    .named = true,
  },
  [sym__line_doc_comment_marker] = {
    .visible = false,
    .named = true,
  },
  [sym__inner_line_doc_comment_marker] = {
    .visible = true,
    .named = true,
  },
  [sym__outer_line_doc_comment_marker] = {
    .visible = true,
    .named = true,
  },
  [sym_block_comment] = {
    .visible = true,
    .named = true,
  },
  [sym__block_doc_comment_marker] = {
    .visible = false,
    .named = true,
  },
  [aux_sym_source_file_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_macro_definition_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_token_tree_pattern_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_token_tree_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__non_special_token_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_declaration_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_enum_variant_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_enum_variant_list_repeat2] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_field_declaration_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_ordered_field_declaration_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_function_modifiers_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_where_clause_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_trait_bounds_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_type_parameters_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_use_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_parameters_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_for_lifetimes_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_tuple_type_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_type_arguments_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_delim_token_tree_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_arguments_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_tuple_expression_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_field_initializer_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_match_block_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_match_arm_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_closure_parameters_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_tuple_pattern_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_slice_pattern_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_struct_pattern_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_string_literal_repeat1] = {
    .visible = false,
    .named = false,
  },
  [alias_sym_field_identifier] = {
    .visible = true,
    .named = true,
  },
  [alias_sym_let_chain] = {
    .visible = true,
    .named = true,
  },
  [alias_sym_shorthand_field_identifier] = {
    .visible = true,
    .named = true,
  },
  [alias_sym_type_identifier] = {
    .visible = true,
    .named = true,
  },
};

enum ts_field_identifiers {
  field_alias = 1,
  field_alternative = 2,
  field_argument = 3,
  field_arguments = 4,
  field_body = 5,
  field_bounds = 6,
  field_condition = 7,
  field_consequence = 8,
  field_default_type = 9,
  field_doc = 10,
  field_element = 11,
  field_field = 12,
  field_function = 13,
  field_inner = 14,
  field_left = 15,
  field_length = 16,
  field_list = 17,
  field_macro = 18,
  field_name = 19,
  field_operator = 20,
  field_outer = 21,
  field_parameters = 22,
  field_path = 23,
  field_pattern = 24,
  field_return_type = 25,
  field_right = 26,
  field_trait = 27,
  field_type = 28,
  field_type_arguments = 29,
  field_type_parameters = 30,
  field_value = 31,
};

static const char * const ts_field_names[] = {
  [0] = NULL,
  [field_alias] = "alias",
  [field_alternative] = "alternative",
  [field_argument] = "argument",
  [field_arguments] = "arguments",
  [field_body] = "body",
  [field_bounds] = "bounds",
  [field_condition] = "condition",
  [field_consequence] = "consequence",
  [field_default_type] = "default_type",
  [field_doc] = "doc",
  [field_element] = "element",
  [field_field] = "field",
  [field_function] = "function",
  [field_inner] = "inner",
  [field_left] = "left",
  [field_length] = "length",
  [field_list] = "list",
  [field_macro] = "macro",
  [field_name] = "name",
  [field_operator] = "operator",
  [field_outer] = "outer",
  [field_parameters] = "parameters",
  [field_path] = "path",
  [field_pattern] = "pattern",
  [field_return_type] = "return_type",
  [field_right] = "right",
  [field_trait] = "trait",
  [field_type] = "type",
  [field_type_arguments] = "type_arguments",
  [field_type_parameters] = "type_parameters",
  [field_value] = "value",
};

static const TSFieldMapSlice ts_field_map_slices[PRODUCTION_ID_COUNT] = {
  [2] = {.index = 0, .length = 1},
  [3] = {.index = 1, .length = 1},
  [4] = {.index = 2, .length = 1},
  [6] = {.index = 3, .length = 1},
  [7] = {.index = 3, .length = 1},
  [8] = {.index = 4, .length = 1},
  [10] = {.index = 5, .length = 2},
  [11] = {.index = 5, .length = 2},
  [12] = {.index = 7, .length = 2},
  [13] = {.index = 9, .length = 2},
  [14] = {.index = 11, .length = 3},
  [15] = {.index = 14, .length = 2},
  [16] = {.index = 16, .length = 2},
  [17] = {.index = 16, .length = 2},
  [18] = {.index = 18, .length = 1},
  [19] = {.index = 19, .length = 2},
  [20] = {.index = 19, .length = 2},
  [21] = {.index = 19, .length = 2},
  [22] = {.index = 21, .length = 1},
  [23] = {.index = 22, .length = 1},
  [24] = {.index = 23, .length = 1},
  [25] = {.index = 23, .length = 1},
  [26] = {.index = 24, .length = 2},
  [27] = {.index = 24, .length = 2},
  [28] = {.index = 26, .length = 1},
  [29] = {.index = 27, .length = 2},
  [30] = {.index = 29, .length = 2},
  [31] = {.index = 31, .length = 2},
  [32] = {.index = 33, .length = 1},
  [33] = {.index = 34, .length = 2},
  [34] = {.index = 27, .length = 2},
  [35] = {.index = 36, .length = 2},
  [36] = {.index = 38, .length = 1},
  [37] = {.index = 39, .length = 1},
  [38] = {.index = 40, .length = 2},
  [39] = {.index = 26, .length = 1},
  [40] = {.index = 16, .length = 2},
  [41] = {.index = 16, .length = 2},
  [42] = {.index = 42, .length = 2},
  [43] = {.index = 44, .length = 2},
  [44] = {.index = 46, .length = 1},
  [45] = {.index = 16, .length = 2},
  [46] = {.index = 16, .length = 2},
  [47] = {.index = 47, .length = 3},
  [48] = {.index = 50, .length = 2},
  [49] = {.index = 52, .length = 2},
  [50] = {.index = 52, .length = 2},
  [51] = {.index = 54, .length = 2},
  [52] = {.index = 44, .length = 2},
  [53] = {.index = 11, .length = 3},
  [54] = {.index = 3, .length = 1},
  [55] = {.index = 56, .length = 1},
  [57] = {.index = 57, .length = 1},
  [58] = {.index = 57, .length = 1},
  [59] = {.index = 58, .length = 1},
  [61] = {.index = 59, .length = 2},
  [62] = {.index = 57, .length = 1},
  [63] = {.index = 61, .length = 1},
  [64] = {.index = 62, .length = 1},
  [65] = {.index = 63, .length = 1},
  [66] = {.index = 64, .length = 2},
  [67] = {.index = 66, .length = 2},
  [68] = {.index = 66, .length = 2},
  [69] = {.index = 68, .length = 1},
  [70] = {.index = 68, .length = 1},
  [71] = {.index = 58, .length = 1},
  [72] = {.index = 69, .length = 2},
  [73] = {.index = 71, .length = 3},
  [74] = {.index = 74, .length = 2},
  [75] = {.index = 76, .length = 3},
  [76] = {.index = 79, .length = 3},
  [78] = {.index = 82, .length = 2},
  [79] = {.index = 82, .length = 2},
  [80] = {.index = 84, .length = 2},
  [81] = {.index = 86, .length = 3},
  [82] = {.index = 89, .length = 2},
  [83] = {.index = 91, .length = 1},
  [84] = {.index = 92, .length = 2},
  [85] = {.index = 94, .length = 2},
  [86] = {.index = 96, .length = 3},
  [87] = {.index = 99, .length = 2},
  [88] = {.index = 101, .length = 2},
  [89] = {.index = 103, .length = 2},
  [90] = {.index = 105, .length = 2},
  [91] = {.index = 107, .length = 2},
  [92] = {.index = 105, .length = 2},
  [93] = {.index = 107, .length = 2},
  [94] = {.index = 109, .length = 1},
  [95] = {.index = 103, .length = 2},
  [96] = {.index = 109, .length = 1},
  [97] = {.index = 110, .length = 1},
  [98] = {.index = 111, .length = 3},
  [99] = {.index = 114, .length = 1},
  [100] = {.index = 115, .length = 1},
  [101] = {.index = 116, .length = 2},
  [102] = {.index = 3, .length = 1},
  [103] = {.index = 118, .length = 1},
  [104] = {.index = 119, .length = 2},
  [105] = {.index = 121, .length = 1},
  [106] = {.index = 121, .length = 1},
  [107] = {.index = 122, .length = 3},
  [108] = {.index = 125, .length = 1},
  [109] = {.index = 122, .length = 3},
  [110] = {.index = 18, .length = 1},
  [111] = {.index = 126, .length = 2},
  [112] = {.index = 128, .length = 3},
  [113] = {.index = 131, .length = 3},
  [114] = {.index = 134, .length = 4},
  [115] = {.index = 138, .length = 3},
  [116] = {.index = 141, .length = 3},
  [117] = {.index = 144, .length = 2},
  [118] = {.index = 146, .length = 2},
  [119] = {.index = 146, .length = 2},
  [121] = {.index = 148, .length = 2},
  [122] = {.index = 150, .length = 3},
  [123] = {.index = 153, .length = 3},
  [124] = {.index = 148, .length = 2},
  [125] = {.index = 150, .length = 3},
  [126] = {.index = 156, .length = 2},
  [128] = {.index = 158, .length = 3},
  [129] = {.index = 161, .length = 3},
  [130] = {.index = 164, .length = 4},
  [131] = {.index = 126, .length = 2},
  [132] = {.index = 168, .length = 3},
  [133] = {.index = 171, .length = 2},
  [134] = {.index = 173, .length = 3},
  [135] = {.index = 176, .length = 2},
  [136] = {.index = 178, .length = 2},
  [137] = {.index = 180, .length = 3},
  [138] = {.index = 183, .length = 3},
  [139] = {.index = 186, .length = 2},
  [140] = {.index = 186, .length = 2},
  [141] = {.index = 188, .length = 2},
  [142] = {.index = 190, .length = 3},
  [143] = {.index = 193, .length = 2},
  [144] = {.index = 195, .length = 2},
  [145] = {.index = 197, .length = 1},
  [146] = {.index = 198, .length = 2},
  [147] = {.index = 200, .length = 1},
  [148] = {.index = 201, .length = 2},
  [149] = {.index = 109, .length = 1},
  [150] = {.index = 203, .length = 2},
  [151] = {.index = 205, .length = 2},
  [152] = {.index = 207, .length = 1},
  [153] = {.index = 208, .length = 2},
  [154] = {.index = 210, .length = 3},
  [155] = {.index = 210, .length = 3},
  [156] = {.index = 213, .length = 2},
  [157] = {.index = 215, .length = 4},
  [158] = {.index = 219, .length = 3},
  [159] = {.index = 222, .length = 4},
  [160] = {.index = 226, .length = 2},
  [161] = {.index = 228, .length = 3},
  [162] = {.index = 226, .length = 2},
  [163] = {.index = 228, .length = 3},
  [164] = {.index = 231, .length = 3},
  [165] = {.index = 234, .length = 3},
  [166] = {.index = 237, .length = 4},
  [167] = {.index = 234, .length = 3},
  [168] = {.index = 237, .length = 4},
  [169] = {.index = 231, .length = 3},
  [170] = {.index = 241, .length = 2},
  [171] = {.index = 243, .length = 2},
  [172] = {.index = 245, .length = 2},
  [173] = {.index = 247, .length = 2},
  [174] = {.index = 249, .length = 1},
  [175] = {.index = 250, .length = 2},
  [176] = {.index = 252, .length = 3},
  [177] = {.index = 255, .length = 2},
  [178] = {.index = 257, .length = 2},
  [179] = {.index = 201, .length = 2},
  [180] = {.index = 259, .length = 4},
  [181] = {.index = 263, .length = 3},
  [182] = {.index = 266, .length = 3},
  [183] = {.index = 269, .length = 3},
  [184] = {.index = 272, .length = 3},
  [185] = {.index = 275, .length = 4},
  [186] = {.index = 279, .length = 2},
  [187] = {.index = 281, .length = 2},
  [188] = {.index = 281, .length = 2},
  [189] = {.index = 283, .length = 3},
  [190] = {.index = 286, .length = 4},
  [191] = {.index = 290, .length = 3},
  [192] = {.index = 250, .length = 2},
  [193] = {.index = 293, .length = 2},
  [194] = {.index = 295, .length = 3},
  [195] = {.index = 298, .length = 3},
  [196] = {.index = 301, .length = 2},
  [197] = {.index = 303, .length = 3},
  [198] = {.index = 201, .length = 2},
  [199] = {.index = 306, .length = 3},
  [200] = {.index = 309, .length = 2},
  [201] = {.index = 311, .length = 2},
  [202] = {.index = 313, .length = 3},
  [203] = {.index = 316, .length = 3},
  [204] = {.index = 319, .length = 2},
  [205] = {.index = 321, .length = 4},
  [206] = {.index = 325, .length = 5},
  [207] = {.index = 330, .length = 4},
  [208] = {.index = 334, .length = 3},
  [209] = {.index = 334, .length = 3},
  [210] = {.index = 337, .length = 3},
  [211] = {.index = 340, .length = 4},
  [212] = {.index = 337, .length = 3},
  [213] = {.index = 340, .length = 4},
  [214] = {.index = 344, .length = 4},
  [215] = {.index = 344, .length = 4},
  [216] = {.index = 348, .length = 3},
  [217] = {.index = 351, .length = 3},
  [218] = {.index = 354, .length = 3},
  [219] = {.index = 357, .length = 2},
  [220] = {.index = 359, .length = 2},
  [221] = {.index = 126, .length = 2},
  [222] = {.index = 361, .length = 2},
  [223] = {.index = 363, .length = 3},
  [224] = {.index = 361, .length = 2},
  [225] = {.index = 363, .length = 3},
  [226] = {.index = 366, .length = 3},
  [227] = {.index = 369, .length = 4},
  [228] = {.index = 366, .length = 3},
  [229] = {.index = 369, .length = 4},
  [230] = {.index = 373, .length = 4},
  [231] = {.index = 377, .length = 4},
  [232] = {.index = 381, .length = 3},
  [233] = {.index = 384, .length = 4},
  [234] = {.index = 388, .length = 3},
  [235] = {.index = 391, .length = 3},
  [236] = {.index = 394, .length = 3},
  [237] = {.index = 397, .length = 4},
  [238] = {.index = 401, .length = 2},
  [239] = {.index = 403, .length = 3},
  [240] = {.index = 406, .length = 4},
  [241] = {.index = 410, .length = 3},
  [242] = {.index = 413, .length = 3},
  [243] = {.index = 416, .length = 2},
  [244] = {.index = 418, .length = 3},
  [245] = {.index = 421, .length = 5},
  [246] = {.index = 426, .length = 4},
  [247] = {.index = 426, .length = 4},
  [248] = {.index = 430, .length = 3},
  [249] = {.index = 433, .length = 3},
  [250] = {.index = 436, .length = 3},
  [251] = {.index = 439, .length = 3},
  [252] = {.index = 442, .length = 2},
  [253] = {.index = 444, .length = 3},
  [254] = {.index = 444, .length = 3},
  [255] = {.index = 447, .length = 3},
  [256] = {.index = 450, .length = 4},
  [257] = {.index = 447, .length = 3},
  [258] = {.index = 450, .length = 4},
  [259] = {.index = 454, .length = 4},
  [260] = {.index = 454, .length = 4},
  [261] = {.index = 458, .length = 4},
  [262] = {.index = 462, .length = 5},
  [263] = {.index = 467, .length = 4},
  [264] = {.index = 471, .length = 2},
  [265] = {.index = 473, .length = 4},
  [266] = {.index = 477, .length = 4},
  [267] = {.index = 481, .length = 3},
  [268] = {.index = 484, .length = 4},
  [269] = {.index = 488, .length = 4},
  [270] = {.index = 492, .length = 3},
  [271] = {.index = 495, .length = 4},
  [272] = {.index = 495, .length = 4},
  [273] = {.index = 499, .length = 5},
  [274] = {.index = 504, .length = 4},
  [275] = {.index = 508, .length = 5},
  [276] = {.index = 513, .length = 4},
  [277] = {.index = 517, .length = 4},
  [278] = {.index = 521, .length = 3},
  [279] = {.index = 524, .length = 5},
};

static const TSFieldMapEntry ts_field_map_entries[] = {
  [0] =
    {field_inner, 0},
  [1] =
    {field_outer, 0},
  [2] =
    {field_value, 1},
  [3] =
    {field_name, 1},
  [4] =
    {field_body, 1},
  [5] =
    {field_body, 1},
    {field_name, 0},
  [7] =
    {field_arguments, 1},
    {field_function, 0},
  [9] =
    {field_body, 1},
    {field_parameters, 0},
  [11] =
    {field_doc, 2},
    {field_inner, 1, .inherited = true},
    {field_outer, 1, .inherited = true},
  [14] =
    {field_inner, 1, .inherited = true},
    {field_outer, 1, .inherited = true},
  [16] =
    {field_name, 2},
    {field_path, 0},
  [18] =
    {field_value, 2},
  [19] =
    {field_type, 0},
    {field_type_arguments, 1},
  [21] =
    {field_type, 1},
  [22] =
    {field_parameters, 1},
  [23] =
    {field_trait, 1},
  [24] =
    {field_parameters, 1},
    {field_trait, 0},
  [26] =
    {field_macro, 0},
  [27] =
    {field_body, 2},
    {field_name, 1},
  [29] =
    {field_condition, 1},
    {field_consequence, 2},
  [31] =
    {field_body, 2},
    {field_type, 1},
  [33] =
    {field_pattern, 1},
  [34] =
    {field_body, 2},
    {field_value, 1},
  [36] =
    {field_body, 2},
    {field_parameters, 1},
  [38] =
    {field_list, 1},
  [39] =
    {field_argument, 1},
  [40] =
    {field_body, 2},
    {field_condition, 1},
  [42] =
    {field_function, 0},
    {field_type_arguments, 2},
  [44] =
    {field_type, 0},
    {field_type_arguments, 2},
  [46] =
    {field_body, 2},
  [47] =
    {field_left, 0},
    {field_operator, 1},
    {field_right, 2},
  [50] =
    {field_left, 0},
    {field_right, 2},
  [52] =
    {field_field, 2},
    {field_value, 0},
  [54] =
    {field_type, 2},
    {field_value, 0},
  [56] =
    {field_value, 3},
  [57] =
    {field_type, 0},
  [58] =
    {field_name, 0},
  [59] =
    {field_pattern, 0},
    {field_type, 2},
  [61] =
    {field_element, 1},
  [62] =
    {field_type, 2},
  [63] =
    {field_parameters, 2},
  [64] =
    {field_alias, 2},
    {field_type, 0},
  [66] =
    {field_parameters, 2},
    {field_trait, 1},
  [68] =
    {field_arguments, 1},
  [69] =
    {field_body, 3},
    {field_name, 1},
  [71] =
    {field_body, 3},
    {field_name, 1},
    {field_type_parameters, 2},
  [74] =
    {field_name, 1},
    {field_parameters, 2},
  [76] =
    {field_body, 3},
    {field_name, 1},
    {field_parameters, 2},
  [79] =
    {field_alternative, 3},
    {field_condition, 1},
    {field_consequence, 2},
  [82] =
    {field_bounds, 1},
    {field_left, 0},
  [84] =
    {field_type, 2},
    {field_type_parameters, 1},
  [86] =
    {field_body, 3},
    {field_type, 2},
    {field_type_parameters, 1},
  [89] =
    {field_body, 3},
    {field_type, 1},
  [91] =
    {field_pattern, 2},
  [92] =
    {field_body, 3},
    {field_parameters, 2},
  [94] =
    {field_name, 1},
    {field_type_parameters, 2},
  [96] =
    {field_body, 3},
    {field_bounds, 2},
    {field_name, 1},
  [99] =
    {field_bounds, 2},
    {field_name, 1},
  [101] =
    {field_body, 3},
    {field_type, 2},
  [103] =
    {field_body, 3},
    {field_name, 2},
  [105] =
    {field_list, 2},
    {field_path, 0},
  [107] =
    {field_alias, 2},
    {field_path, 0},
  [109] =
    {field_name, 2},
  [110] =
    {field_argument, 2},
  [111] =
    {field_body, 3},
    {field_parameters, 0},
    {field_return_type, 2},
  [114] =
    {field_body, 3},
  [115] =
    {field_length, 3},
  [116] =
    {field_pattern, 1},
    {field_type, 3},
  [118] =
    {field_type, 3},
  [119] =
    {field_parameters, 1},
    {field_return_type, 3},
  [121] =
    {field_trait, 3},
  [122] =
    {field_parameters, 1},
    {field_return_type, 3},
    {field_trait, 0},
  [125] =
    {field_parameters, 3},
  [126] =
    {field_name, 1},
    {field_type, 3},
  [128] =
    {field_body, 4},
    {field_name, 1},
    {field_type_parameters, 2},
  [131] =
    {field_name, 1},
    {field_parameters, 3},
    {field_type_parameters, 2},
  [134] =
    {field_body, 4},
    {field_name, 1},
    {field_parameters, 3},
    {field_type_parameters, 2},
  [138] =
    {field_body, 4},
    {field_name, 1},
    {field_parameters, 2},
  [141] =
    {field_body, 4},
    {field_pattern, 1},
    {field_value, 3},
  [144] =
    {field_pattern, 1},
    {field_value, 3},
  [146] =
    {field_default_type, 2},
    {field_name, 0},
  [148] =
    {field_trait, 1},
    {field_type, 3},
  [150] =
    {field_body, 4},
    {field_trait, 1},
    {field_type, 3},
  [153] =
    {field_body, 4},
    {field_type, 2},
    {field_type_parameters, 1},
  [156] =
    {field_alternative, 3},
    {field_pattern, 1},
  [158] =
    {field_body, 4},
    {field_parameters, 1},
    {field_return_type, 3},
  [161] =
    {field_body, 4},
    {field_bounds, 2},
    {field_name, 1},
  [164] =
    {field_body, 4},
    {field_bounds, 3},
    {field_name, 1},
    {field_type_parameters, 2},
  [168] =
    {field_bounds, 3},
    {field_name, 1},
    {field_type_parameters, 2},
  [171] =
    {field_type, 3},
    {field_type_parameters, 2},
  [173] =
    {field_body, 4},
    {field_type, 3},
    {field_type_parameters, 2},
  [176] =
    {field_body, 4},
    {field_type, 2},
  [178] =
    {field_body, 4},
    {field_name, 2},
  [180] =
    {field_body, 4},
    {field_bounds, 3},
    {field_name, 2},
  [183] =
    {field_body, 4},
    {field_name, 2},
    {field_type_parameters, 3},
  [186] =
    {field_field, 0},
    {field_value, 2},
  [188] =
    {field_name, 2},
    {field_parameters, 3},
  [190] =
    {field_body, 4},
    {field_name, 2},
    {field_parameters, 3},
  [193] =
    {field_name, 2},
    {field_type_parameters, 3},
  [195] =
    {field_body, 4},
    {field_name, 3},
  [197] =
    {field_name, 3},
  [198] =
    {field_body, 4},
    {field_condition, 3},
  [200] =
    {field_length, 4},
  [201] =
    {field_name, 0},
    {field_type, 2},
  [203] =
    {field_name, 0},
    {field_pattern, 2},
  [205] =
    {field_element, 1},
    {field_length, 3},
  [207] =
    {field_pattern, 0},
  [208] =
    {field_parameters, 2},
    {field_return_type, 4},
  [210] =
    {field_parameters, 2},
    {field_return_type, 4},
    {field_trait, 1},
  [213] =
    {field_name, 0},
    {field_value, 2},
  [215] =
    {field_body, 5},
    {field_name, 1},
    {field_parameters, 3},
    {field_type_parameters, 2},
  [219] =
    {field_name, 1},
    {field_parameters, 2},
    {field_return_type, 4},
  [222] =
    {field_body, 5},
    {field_name, 1},
    {field_parameters, 2},
    {field_return_type, 4},
  [226] =
    {field_trait, 2},
    {field_type, 4},
  [228] =
    {field_body, 5},
    {field_trait, 2},
    {field_type, 4},
  [231] =
    {field_body, 5},
    {field_trait, 1},
    {field_type, 3},
  [234] =
    {field_trait, 2},
    {field_type, 4},
    {field_type_parameters, 1},
  [237] =
    {field_body, 5},
    {field_trait, 2},
    {field_type, 4},
    {field_type_parameters, 1},
  [241] =
    {field_pattern, 2},
    {field_type, 4},
  [243] =
    {field_pattern, 2},
    {field_value, 4},
  [245] =
    {field_alternative, 4},
    {field_pattern, 2},
  [247] =
    {field_pattern, 0},
    {field_value, 2},
  [249] =
    {field_condition, 2},
  [250] =
    {field_name, 2},
    {field_type, 4},
  [252] =
    {field_body, 5},
    {field_parameters, 2},
    {field_return_type, 4},
  [255] =
    {field_type, 1},
    {field_type, 2, .inherited = true},
  [257] =
    {field_type, 0, .inherited = true},
    {field_type, 1, .inherited = true},
  [259] =
    {field_body, 5},
    {field_bounds, 3},
    {field_name, 1},
    {field_type_parameters, 2},
  [263] =
    {field_name, 1},
    {field_type, 4},
    {field_type_parameters, 2},
  [266] =
    {field_body, 5},
    {field_type, 3},
    {field_type_parameters, 2},
  [269] =
    {field_body, 5},
    {field_bounds, 3},
    {field_name, 2},
  [272] =
    {field_body, 5},
    {field_name, 2},
    {field_type_parameters, 3},
  [275] =
    {field_body, 5},
    {field_bounds, 4},
    {field_name, 2},
    {field_type_parameters, 3},
  [279] =
    {field_alias, 4},
    {field_name, 2},
  [281] =
    {field_field, 1},
    {field_value, 3},
  [283] =
    {field_name, 2},
    {field_parameters, 4},
    {field_type_parameters, 3},
  [286] =
    {field_body, 5},
    {field_name, 2},
    {field_parameters, 4},
    {field_type_parameters, 3},
  [290] =
    {field_body, 5},
    {field_name, 2},
    {field_parameters, 3},
  [293] =
    {field_body, 5},
    {field_name, 3},
  [295] =
    {field_body, 5},
    {field_bounds, 4},
    {field_name, 3},
  [298] =
    {field_body, 5},
    {field_name, 3},
    {field_type_parameters, 4},
  [301] =
    {field_name, 3},
    {field_parameters, 4},
  [303] =
    {field_body, 5},
    {field_name, 3},
    {field_parameters, 4},
  [306] =
    {field_name, 0},
    {field_type, 3},
    {field_type_arguments, 1},
  [309] =
    {field_name, 1},
    {field_pattern, 3},
  [311] =
    {field_parameters, 3},
    {field_return_type, 5},
  [313] =
    {field_name, 1},
    {field_type, 3},
    {field_value, 5},
  [316] =
    {field_body, 1},
    {field_name, 0},
    {field_value, 3},
  [319] =
    {field_name, 1},
    {field_value, 3},
  [321] =
    {field_name, 1},
    {field_parameters, 3},
    {field_return_type, 5},
    {field_type_parameters, 2},
  [325] =
    {field_body, 6},
    {field_name, 1},
    {field_parameters, 3},
    {field_return_type, 5},
    {field_type_parameters, 2},
  [330] =
    {field_body, 6},
    {field_name, 1},
    {field_parameters, 2},
    {field_return_type, 4},
  [334] =
    {field_body, 6},
    {field_trait, 2},
    {field_type, 4},
  [337] =
    {field_trait, 3},
    {field_type, 5},
    {field_type_parameters, 1},
  [340] =
    {field_body, 6},
    {field_trait, 3},
    {field_type, 5},
    {field_type_parameters, 1},
  [344] =
    {field_body, 6},
    {field_trait, 2},
    {field_type, 4},
    {field_type_parameters, 1},
  [348] =
    {field_pattern, 1},
    {field_type, 3},
    {field_value, 5},
  [351] =
    {field_alternative, 5},
    {field_pattern, 1},
    {field_type, 3},
  [354] =
    {field_alternative, 5},
    {field_pattern, 1},
    {field_value, 3},
  [357] =
    {field_name, 3},
    {field_type, 5},
  [359] =
    {field_type, 2},
    {field_type, 3, .inherited = true},
  [361] =
    {field_trait, 3},
    {field_type, 5},
  [363] =
    {field_body, 6},
    {field_trait, 3},
    {field_type, 5},
  [366] =
    {field_trait, 3},
    {field_type, 5},
    {field_type_parameters, 2},
  [369] =
    {field_body, 6},
    {field_trait, 3},
    {field_type, 5},
    {field_type_parameters, 2},
  [373] =
    {field_body, 6},
    {field_bounds, 4},
    {field_name, 2},
    {field_type_parameters, 3},
  [377] =
    {field_body, 6},
    {field_name, 2},
    {field_parameters, 4},
    {field_type_parameters, 3},
  [381] =
    {field_name, 2},
    {field_parameters, 3},
    {field_return_type, 5},
  [384] =
    {field_body, 6},
    {field_name, 2},
    {field_parameters, 3},
    {field_return_type, 5},
  [388] =
    {field_name, 2},
    {field_type, 5},
    {field_type_parameters, 3},
  [391] =
    {field_body, 6},
    {field_bounds, 4},
    {field_name, 3},
  [394] =
    {field_body, 6},
    {field_name, 3},
    {field_type_parameters, 4},
  [397] =
    {field_body, 6},
    {field_bounds, 5},
    {field_name, 3},
    {field_type_parameters, 4},
  [401] =
    {field_alias, 5},
    {field_name, 3},
  [403] =
    {field_name, 3},
    {field_parameters, 5},
    {field_type_parameters, 4},
  [406] =
    {field_body, 6},
    {field_name, 3},
    {field_parameters, 5},
    {field_type_parameters, 4},
  [410] =
    {field_body, 6},
    {field_name, 3},
    {field_parameters, 4},
  [413] =
    {field_body, 6},
    {field_pattern, 3},
    {field_value, 5},
  [416] =
    {field_name, 2},
    {field_pattern, 4},
  [418] =
    {field_body, 2},
    {field_name, 1},
    {field_value, 4},
  [421] =
    {field_body, 7},
    {field_name, 1},
    {field_parameters, 3},
    {field_return_type, 5},
    {field_type_parameters, 2},
  [426] =
    {field_body, 7},
    {field_trait, 3},
    {field_type, 5},
    {field_type_parameters, 1},
  [430] =
    {field_pattern, 2},
    {field_type, 4},
    {field_value, 6},
  [433] =
    {field_alternative, 6},
    {field_pattern, 2},
    {field_type, 4},
  [436] =
    {field_alternative, 6},
    {field_pattern, 2},
    {field_value, 4},
  [439] =
    {field_name, 2},
    {field_type, 4},
    {field_value, 6},
  [442] =
    {field_type, 3},
    {field_type, 4, .inherited = true},
  [444] =
    {field_body, 7},
    {field_trait, 3},
    {field_type, 5},
  [447] =
    {field_trait, 4},
    {field_type, 6},
    {field_type_parameters, 2},
  [450] =
    {field_body, 7},
    {field_trait, 4},
    {field_type, 6},
    {field_type_parameters, 2},
  [454] =
    {field_body, 7},
    {field_trait, 3},
    {field_type, 5},
    {field_type_parameters, 2},
  [458] =
    {field_name, 2},
    {field_parameters, 4},
    {field_return_type, 6},
    {field_type_parameters, 3},
  [462] =
    {field_body, 7},
    {field_name, 2},
    {field_parameters, 4},
    {field_return_type, 6},
    {field_type_parameters, 3},
  [467] =
    {field_body, 7},
    {field_name, 2},
    {field_parameters, 3},
    {field_return_type, 5},
  [471] =
    {field_name, 4},
    {field_type, 6},
  [473] =
    {field_body, 7},
    {field_bounds, 5},
    {field_name, 3},
    {field_type_parameters, 4},
  [477] =
    {field_body, 7},
    {field_name, 3},
    {field_parameters, 5},
    {field_type_parameters, 4},
  [481] =
    {field_name, 3},
    {field_parameters, 4},
    {field_return_type, 6},
  [484] =
    {field_body, 7},
    {field_name, 3},
    {field_parameters, 4},
    {field_return_type, 6},
  [488] =
    {field_alternative, 7},
    {field_pattern, 1},
    {field_type, 3},
    {field_value, 5},
  [492] =
    {field_name, 3},
    {field_type, 5},
    {field_value, 7},
  [495] =
    {field_body, 8},
    {field_trait, 4},
    {field_type, 6},
    {field_type_parameters, 2},
  [499] =
    {field_body, 8},
    {field_name, 2},
    {field_parameters, 4},
    {field_return_type, 6},
    {field_type_parameters, 3},
  [504] =
    {field_name, 3},
    {field_parameters, 5},
    {field_return_type, 7},
    {field_type_parameters, 4},
  [508] =
    {field_body, 8},
    {field_name, 3},
    {field_parameters, 5},
    {field_return_type, 7},
    {field_type_parameters, 4},
  [513] =
    {field_body, 8},
    {field_name, 3},
    {field_parameters, 4},
    {field_return_type, 6},
  [517] =
    {field_alternative, 8},
    {field_pattern, 2},
    {field_type, 4},
    {field_value, 6},
  [521] =
    {field_name, 4},
    {field_type, 6},
    {field_value, 8},
  [524] =
    {field_body, 9},
    {field_name, 3},
    {field_parameters, 5},
    {field_return_type, 7},
    {field_type_parameters, 4},
};

static const TSSymbol ts_alias_sequences[PRODUCTION_ID_COUNT][MAX_ALIAS_SEQUENCE_LENGTH] = {
  [0] = {0},
  [1] = {
    [0] = sym_identifier,
  },
  [5] = {
    [0] = alias_sym_type_identifier,
  },
  [7] = {
    [1] = alias_sym_type_identifier,
  },
  [9] = {
    [0] = alias_sym_let_chain,
  },
  [10] = {
    [0] = alias_sym_type_identifier,
  },
  [16] = {
    [0] = sym_identifier,
  },
  [17] = {
    [0] = sym_identifier,
    [2] = alias_sym_type_identifier,
  },
  [19] = {
    [0] = sym_identifier,
  },
  [20] = {
    [0] = alias_sym_type_identifier,
  },
  [24] = {
    [1] = alias_sym_type_identifier,
  },
  [26] = {
    [0] = alias_sym_type_identifier,
  },
  [28] = {
    [0] = sym_identifier,
  },
  [29] = {
    [1] = alias_sym_type_identifier,
  },
  [41] = {
    [2] = alias_sym_type_identifier,
  },
  [43] = {
    [0] = alias_sym_type_identifier,
  },
  [45] = {
    [0] = sym_generic_type,
  },
  [46] = {
    [0] = sym_generic_type,
    [2] = alias_sym_type_identifier,
  },
  [50] = {
    [2] = alias_sym_field_identifier,
  },
  [53] = {
    [2] = sym__line_doc_content,
  },
  [54] = {
    [1] = sym_identifier,
  },
  [56] = {
    [0] = sym_identifier,
    [2] = sym_identifier,
  },
  [58] = {
    [0] = alias_sym_type_identifier,
  },
  [59] = {
    [0] = alias_sym_shorthand_field_identifier,
  },
  [60] = {
    [2] = sym_identifier,
  },
  [62] = {
    [0] = sym_generic_type,
  },
  [67] = {
    [1] = alias_sym_type_identifier,
  },
  [69] = {
    [0] = sym_identifier,
  },
  [72] = {
    [1] = alias_sym_type_identifier,
  },
  [73] = {
    [1] = alias_sym_type_identifier,
  },
  [77] = {
    [1] = alias_sym_type_identifier,
  },
  [78] = {
    [0] = alias_sym_type_identifier,
  },
  [85] = {
    [1] = alias_sym_type_identifier,
  },
  [86] = {
    [1] = alias_sym_type_identifier,
  },
  [87] = {
    [1] = alias_sym_type_identifier,
  },
  [89] = {
    [2] = alias_sym_type_identifier,
  },
  [90] = {
    [0] = sym_identifier,
  },
  [91] = {
    [0] = sym_identifier,
  },
  [96] = {
    [2] = alias_sym_type_identifier,
  },
  [102] = {
    [1] = alias_sym_shorthand_field_identifier,
  },
  [105] = {
    [3] = alias_sym_type_identifier,
  },
  [107] = {
    [0] = alias_sym_type_identifier,
  },
  [110] = {
    [0] = sym_identifier,
  },
  [112] = {
    [1] = alias_sym_type_identifier,
  },
  [118] = {
    [0] = alias_sym_type_identifier,
  },
  [120] = {
    [2] = alias_sym_type_identifier,
  },
  [121] = {
    [1] = alias_sym_type_identifier,
  },
  [122] = {
    [1] = alias_sym_type_identifier,
  },
  [127] = {
    [3] = sym_identifier,
  },
  [129] = {
    [1] = alias_sym_type_identifier,
  },
  [130] = {
    [1] = alias_sym_type_identifier,
  },
  [131] = {
    [1] = alias_sym_type_identifier,
  },
  [132] = {
    [1] = alias_sym_type_identifier,
  },
  [136] = {
    [2] = alias_sym_type_identifier,
  },
  [137] = {
    [2] = alias_sym_type_identifier,
  },
  [138] = {
    [2] = alias_sym_type_identifier,
  },
  [140] = {
    [0] = alias_sym_field_identifier,
  },
  [143] = {
    [2] = alias_sym_type_identifier,
  },
  [144] = {
    [3] = alias_sym_type_identifier,
  },
  [148] = {
    [0] = alias_sym_type_identifier,
  },
  [149] = {
    [2] = alias_sym_shorthand_field_identifier,
  },
  [150] = {
    [0] = alias_sym_field_identifier,
  },
  [154] = {
    [1] = alias_sym_type_identifier,
  },
  [160] = {
    [2] = alias_sym_type_identifier,
  },
  [161] = {
    [2] = alias_sym_type_identifier,
  },
  [164] = {
    [1] = alias_sym_type_identifier,
  },
  [165] = {
    [2] = alias_sym_type_identifier,
  },
  [166] = {
    [2] = alias_sym_type_identifier,
  },
  [179] = {
    [0] = alias_sym_field_identifier,
  },
  [180] = {
    [1] = alias_sym_type_identifier,
  },
  [181] = {
    [1] = alias_sym_type_identifier,
  },
  [183] = {
    [2] = alias_sym_type_identifier,
  },
  [184] = {
    [2] = alias_sym_type_identifier,
  },
  [185] = {
    [2] = alias_sym_type_identifier,
  },
  [188] = {
    [1] = alias_sym_field_identifier,
  },
  [192] = {
    [2] = alias_sym_type_identifier,
  },
  [193] = {
    [3] = alias_sym_type_identifier,
  },
  [194] = {
    [3] = alias_sym_type_identifier,
  },
  [195] = {
    [3] = alias_sym_type_identifier,
  },
  [199] = {
    [0] = alias_sym_type_identifier,
  },
  [200] = {
    [1] = alias_sym_field_identifier,
  },
  [208] = {
    [2] = alias_sym_type_identifier,
  },
  [210] = {
    [3] = alias_sym_type_identifier,
  },
  [211] = {
    [3] = alias_sym_type_identifier,
  },
  [214] = {
    [2] = alias_sym_type_identifier,
  },
  [221] = {
    [1] = alias_sym_field_identifier,
  },
  [222] = {
    [3] = alias_sym_type_identifier,
  },
  [223] = {
    [3] = alias_sym_type_identifier,
  },
  [226] = {
    [3] = alias_sym_type_identifier,
  },
  [227] = {
    [3] = alias_sym_type_identifier,
  },
  [230] = {
    [2] = alias_sym_type_identifier,
  },
  [234] = {
    [2] = alias_sym_type_identifier,
  },
  [235] = {
    [3] = alias_sym_type_identifier,
  },
  [236] = {
    [3] = alias_sym_type_identifier,
  },
  [237] = {
    [3] = alias_sym_type_identifier,
  },
  [243] = {
    [2] = alias_sym_field_identifier,
  },
  [246] = {
    [3] = alias_sym_type_identifier,
  },
  [253] = {
    [3] = alias_sym_type_identifier,
  },
  [255] = {
    [4] = alias_sym_type_identifier,
  },
  [256] = {
    [4] = alias_sym_type_identifier,
  },
  [259] = {
    [3] = alias_sym_type_identifier,
  },
  [265] = {
    [3] = alias_sym_type_identifier,
  },
  [271] = {
    [4] = alias_sym_type_identifier,
  },
};

static const uint16_t ts_non_terminal_alias_map[] = {
  sym_generic_type_with_turbofish, 2,
    sym_generic_type_with_turbofish,
    sym_generic_type,
  sym__let_chain, 2,
    sym__let_chain,
    alias_sym_let_chain,
  0,
};

static const TSStateId ts_primary_state_ids[STATE_COUNT] = {
  [0] = 0,
  [1] = 1,
  [2] = 2,
  [3] = 3,
  [4] = 4,
  [5] = 5,
  [6] = 6,
  [7] = 7,
  [8] = 8,
  [9] = 3,
  [10] = 2,
  [11] = 5,
  [12] = 7,
  [13] = 8,
  [14] = 3,
  [15] = 2,
  [16] = 5,
  [17] = 8,
  [18] = 3,
  [19] = 2,
  [20] = 5,
  [21] = 8,
  [22] = 3,
  [23] = 2,
  [24] = 5,
  [25] = 8,
  [26] = 3,
  [27] = 2,
  [28] = 5,
  [29] = 8,
  [30] = 3,
  [31] = 2,
  [32] = 5,
  [33] = 8,
  [34] = 34,
  [35] = 35,
  [36] = 36,
  [37] = 37,
  [38] = 38,
  [39] = 39,
  [40] = 38,
  [41] = 41,
  [42] = 37,
  [43] = 35,
  [44] = 39,
  [45] = 35,
  [46] = 36,
  [47] = 41,
  [48] = 38,
  [49] = 37,
  [50] = 38,
  [51] = 37,
  [52] = 35,
  [53] = 36,
  [54] = 37,
  [55] = 37,
  [56] = 38,
  [57] = 39,
  [58] = 38,
  [59] = 37,
  [60] = 41,
  [61] = 38,
  [62] = 37,
  [63] = 39,
  [64] = 38,
  [65] = 41,
  [66] = 36,
  [67] = 67,
  [68] = 68,
  [69] = 69,
  [70] = 70,
  [71] = 69,
  [72] = 72,
  [73] = 73,
  [74] = 74,
  [75] = 70,
  [76] = 68,
  [77] = 72,
  [78] = 73,
  [79] = 74,
  [80] = 80,
  [81] = 81,
  [82] = 82,
  [83] = 83,
  [84] = 84,
  [85] = 85,
  [86] = 86,
  [87] = 87,
  [88] = 88,
  [89] = 89,
  [90] = 90,
  [91] = 91,
  [92] = 92,
  [93] = 93,
  [94] = 94,
  [95] = 88,
  [96] = 89,
  [97] = 87,
  [98] = 98,
  [99] = 99,
  [100] = 100,
  [101] = 84,
  [102] = 100,
  [103] = 85,
  [104] = 86,
  [105] = 87,
  [106] = 84,
  [107] = 99,
  [108] = 100,
  [109] = 84,
  [110] = 85,
  [111] = 86,
  [112] = 87,
  [113] = 99,
  [114] = 99,
  [115] = 100,
  [116] = 84,
  [117] = 85,
  [118] = 86,
  [119] = 87,
  [120] = 85,
  [121] = 91,
  [122] = 99,
  [123] = 100,
  [124] = 99,
  [125] = 85,
  [126] = 86,
  [127] = 87,
  [128] = 100,
  [129] = 84,
  [130] = 92,
  [131] = 93,
  [132] = 86,
  [133] = 94,
  [134] = 134,
  [135] = 135,
  [136] = 136,
  [137] = 137,
  [138] = 134,
  [139] = 135,
  [140] = 140,
  [141] = 141,
  [142] = 140,
  [143] = 143,
  [144] = 141,
  [145] = 145,
  [146] = 146,
  [147] = 147,
  [148] = 148,
  [149] = 145,
  [150] = 150,
  [151] = 151,
  [152] = 152,
  [153] = 153,
  [154] = 146,
  [155] = 155,
  [156] = 156,
  [157] = 157,
  [158] = 153,
  [159] = 159,
  [160] = 160,
  [161] = 161,
  [162] = 162,
  [163] = 163,
  [164] = 164,
  [165] = 165,
  [166] = 166,
  [167] = 162,
  [168] = 147,
  [169] = 148,
  [170] = 170,
  [171] = 171,
  [172] = 172,
  [173] = 173,
  [174] = 174,
  [175] = 175,
  [176] = 176,
  [177] = 177,
  [178] = 164,
  [179] = 137,
  [180] = 180,
  [181] = 181,
  [182] = 182,
  [183] = 183,
  [184] = 184,
  [185] = 185,
  [186] = 165,
  [187] = 187,
  [188] = 181,
  [189] = 166,
  [190] = 187,
  [191] = 181,
  [192] = 166,
  [193] = 187,
  [194] = 181,
  [195] = 166,
  [196] = 187,
  [197] = 197,
  [198] = 198,
  [199] = 177,
  [200] = 180,
  [201] = 201,
  [202] = 182,
  [203] = 184,
  [204] = 185,
  [205] = 205,
  [206] = 206,
  [207] = 207,
  [208] = 208,
  [209] = 209,
  [210] = 210,
  [211] = 208,
  [212] = 201,
  [213] = 171,
  [214] = 214,
  [215] = 215,
  [216] = 215,
  [217] = 217,
  [218] = 218,
  [219] = 219,
  [220] = 220,
  [221] = 221,
  [222] = 222,
  [223] = 219,
  [224] = 224,
  [225] = 220,
  [226] = 226,
  [227] = 227,
  [228] = 222,
  [229] = 229,
  [230] = 227,
  [231] = 227,
  [232] = 220,
  [233] = 219,
  [234] = 219,
  [235] = 227,
  [236] = 226,
  [237] = 218,
  [238] = 215,
  [239] = 220,
  [240] = 215,
  [241] = 221,
  [242] = 227,
  [243] = 219,
  [244] = 217,
  [245] = 224,
  [246] = 220,
  [247] = 247,
  [248] = 248,
  [249] = 249,
  [250] = 250,
  [251] = 251,
  [252] = 252,
  [253] = 253,
  [254] = 254,
  [255] = 255,
  [256] = 256,
  [257] = 257,
  [258] = 258,
  [259] = 259,
  [260] = 260,
  [261] = 261,
  [262] = 262,
  [263] = 256,
  [264] = 264,
  [265] = 265,
  [266] = 266,
  [267] = 256,
  [268] = 268,
  [269] = 256,
  [270] = 270,
  [271] = 271,
  [272] = 256,
  [273] = 248,
  [274] = 249,
  [275] = 250,
  [276] = 276,
  [277] = 251,
  [278] = 252,
  [279] = 279,
  [280] = 280,
  [281] = 281,
  [282] = 282,
  [283] = 283,
  [284] = 284,
  [285] = 285,
  [286] = 286,
  [287] = 287,
  [288] = 253,
  [289] = 254,
  [290] = 255,
  [291] = 276,
  [292] = 276,
  [293] = 256,
  [294] = 247,
  [295] = 295,
  [296] = 296,
  [297] = 247,
  [298] = 298,
  [299] = 268,
  [300] = 270,
  [301] = 271,
  [302] = 248,
  [303] = 249,
  [304] = 250,
  [305] = 251,
  [306] = 252,
  [307] = 253,
  [308] = 276,
  [309] = 254,
  [310] = 255,
  [311] = 311,
  [312] = 256,
  [313] = 257,
  [314] = 314,
  [315] = 315,
  [316] = 296,
  [317] = 257,
  [318] = 318,
  [319] = 247,
  [320] = 320,
  [321] = 268,
  [322] = 270,
  [323] = 271,
  [324] = 248,
  [325] = 249,
  [326] = 250,
  [327] = 251,
  [328] = 252,
  [329] = 253,
  [330] = 254,
  [331] = 255,
  [332] = 332,
  [333] = 256,
  [334] = 334,
  [335] = 257,
  [336] = 336,
  [337] = 337,
  [338] = 315,
  [339] = 339,
  [340] = 259,
  [341] = 295,
  [342] = 342,
  [343] = 318,
  [344] = 339,
  [345] = 345,
  [346] = 346,
  [347] = 279,
  [348] = 283,
  [349] = 298,
  [350] = 320,
  [351] = 336,
  [352] = 342,
  [353] = 353,
  [354] = 262,
  [355] = 265,
  [356] = 266,
  [357] = 282,
  [358] = 286,
  [359] = 315,
  [360] = 318,
  [361] = 353,
  [362] = 283,
  [363] = 353,
  [364] = 315,
  [365] = 268,
  [366] = 318,
  [367] = 270,
  [368] = 353,
  [369] = 271,
  [370] = 370,
  [371] = 371,
  [372] = 372,
  [373] = 373,
  [374] = 374,
  [375] = 375,
  [376] = 376,
  [377] = 377,
  [378] = 378,
  [379] = 379,
  [380] = 380,
  [381] = 381,
  [382] = 382,
  [383] = 383,
  [384] = 210,
  [385] = 385,
  [386] = 386,
  [387] = 387,
  [388] = 209,
  [389] = 376,
  [390] = 380,
  [391] = 391,
  [392] = 392,
  [393] = 393,
  [394] = 376,
  [395] = 395,
  [396] = 396,
  [397] = 397,
  [398] = 398,
  [399] = 399,
  [400] = 400,
  [401] = 380,
  [402] = 402,
  [403] = 403,
  [404] = 404,
  [405] = 405,
  [406] = 406,
  [407] = 407,
  [408] = 406,
  [409] = 399,
  [410] = 410,
  [411] = 410,
  [412] = 410,
  [413] = 406,
  [414] = 407,
  [415] = 407,
  [416] = 416,
  [417] = 417,
  [418] = 417,
  [419] = 417,
  [420] = 420,
  [421] = 420,
  [422] = 420,
  [423] = 423,
  [424] = 424,
  [425] = 425,
  [426] = 426,
  [427] = 426,
  [428] = 428,
  [429] = 429,
  [430] = 428,
  [431] = 425,
  [432] = 428,
  [433] = 429,
  [434] = 429,
  [435] = 435,
  [436] = 435,
  [437] = 437,
  [438] = 438,
  [439] = 439,
  [440] = 440,
  [441] = 441,
  [442] = 441,
  [443] = 441,
  [444] = 440,
  [445] = 439,
  [446] = 439,
  [447] = 440,
  [448] = 448,
  [449] = 449,
  [450] = 448,
  [451] = 448,
  [452] = 332,
  [453] = 373,
  [454] = 370,
  [455] = 371,
  [456] = 374,
  [457] = 372,
  [458] = 381,
  [459] = 404,
  [460] = 398,
  [461] = 395,
  [462] = 462,
  [463] = 396,
  [464] = 379,
  [465] = 397,
  [466] = 383,
  [467] = 467,
  [468] = 386,
  [469] = 382,
  [470] = 378,
  [471] = 471,
  [472] = 467,
  [473] = 400,
  [474] = 467,
  [475] = 467,
  [476] = 403,
  [477] = 392,
  [478] = 405,
  [479] = 385,
  [480] = 393,
  [481] = 375,
  [482] = 391,
  [483] = 483,
  [484] = 483,
  [485] = 483,
  [486] = 483,
  [487] = 487,
  [488] = 488,
  [489] = 489,
  [490] = 490,
  [491] = 491,
  [492] = 492,
  [493] = 493,
  [494] = 494,
  [495] = 495,
  [496] = 496,
  [497] = 497,
  [498] = 498,
  [499] = 499,
  [500] = 500,
  [501] = 501,
  [502] = 502,
  [503] = 503,
  [504] = 504,
  [505] = 505,
  [506] = 506,
  [507] = 507,
  [508] = 508,
  [509] = 509,
  [510] = 510,
  [511] = 511,
  [512] = 512,
  [513] = 513,
  [514] = 514,
  [515] = 515,
  [516] = 516,
  [517] = 517,
  [518] = 518,
  [519] = 519,
  [520] = 520,
  [521] = 521,
  [522] = 522,
  [523] = 523,
  [524] = 524,
  [525] = 525,
  [526] = 526,
  [527] = 527,
  [528] = 528,
  [529] = 529,
  [530] = 530,
  [531] = 531,
  [532] = 532,
  [533] = 533,
  [534] = 534,
  [535] = 535,
  [536] = 536,
  [537] = 537,
  [538] = 538,
  [539] = 539,
  [540] = 540,
  [541] = 541,
  [542] = 542,
  [543] = 543,
  [544] = 544,
  [545] = 545,
  [546] = 546,
  [547] = 547,
  [548] = 548,
  [549] = 549,
  [550] = 550,
  [551] = 551,
  [552] = 552,
  [553] = 553,
  [554] = 554,
  [555] = 555,
  [556] = 556,
  [557] = 557,
  [558] = 558,
  [559] = 559,
  [560] = 560,
  [561] = 561,
  [562] = 562,
  [563] = 563,
  [564] = 564,
  [565] = 565,
  [566] = 566,
  [567] = 567,
  [568] = 568,
  [569] = 569,
  [570] = 570,
  [571] = 571,
  [572] = 572,
  [573] = 573,
  [574] = 574,
  [575] = 575,
  [576] = 576,
  [577] = 577,
  [578] = 578,
  [579] = 579,
  [580] = 580,
  [581] = 581,
  [582] = 582,
  [583] = 583,
  [584] = 584,
  [585] = 585,
  [586] = 586,
  [587] = 587,
  [588] = 588,
  [589] = 589,
  [590] = 590,
  [591] = 591,
  [592] = 592,
  [593] = 593,
  [594] = 594,
  [595] = 595,
  [596] = 596,
  [597] = 597,
  [598] = 598,
  [599] = 599,
  [600] = 600,
  [601] = 601,
  [602] = 602,
  [603] = 603,
  [604] = 604,
  [605] = 605,
  [606] = 606,
  [607] = 607,
  [608] = 608,
  [609] = 609,
  [610] = 610,
  [611] = 611,
  [612] = 612,
  [613] = 613,
  [614] = 614,
  [615] = 615,
  [616] = 616,
  [617] = 617,
  [618] = 618,
  [619] = 619,
  [620] = 620,
  [621] = 621,
  [622] = 622,
  [623] = 623,
  [624] = 624,
  [625] = 625,
  [626] = 626,
  [627] = 627,
  [628] = 628,
  [629] = 629,
  [630] = 374,
  [631] = 631,
  [632] = 632,
  [633] = 633,
  [634] = 634,
  [635] = 635,
  [636] = 636,
  [637] = 637,
  [638] = 638,
  [639] = 639,
  [640] = 640,
  [641] = 641,
  [642] = 642,
  [643] = 643,
  [644] = 644,
  [645] = 645,
  [646] = 370,
  [647] = 647,
  [648] = 648,
  [649] = 373,
  [650] = 372,
  [651] = 371,
  [652] = 652,
  [653] = 653,
  [654] = 654,
  [655] = 655,
  [656] = 656,
  [657] = 657,
  [658] = 658,
  [659] = 659,
  [660] = 660,
  [661] = 661,
  [662] = 662,
  [663] = 663,
  [664] = 664,
  [665] = 665,
  [666] = 666,
  [667] = 667,
  [668] = 668,
  [669] = 669,
  [670] = 670,
  [671] = 671,
  [672] = 672,
  [673] = 673,
  [674] = 662,
  [675] = 675,
  [676] = 676,
  [677] = 677,
  [678] = 678,
  [679] = 679,
  [680] = 680,
  [681] = 681,
  [682] = 682,
  [683] = 683,
  [684] = 684,
  [685] = 685,
  [686] = 686,
  [687] = 687,
  [688] = 688,
  [689] = 689,
  [690] = 690,
  [691] = 691,
  [692] = 692,
  [693] = 693,
  [694] = 694,
  [695] = 695,
  [696] = 696,
  [697] = 697,
  [698] = 698,
  [699] = 699,
  [700] = 667,
  [701] = 701,
  [702] = 702,
  [703] = 703,
  [704] = 704,
  [705] = 705,
  [706] = 706,
  [707] = 707,
  [708] = 708,
  [709] = 709,
  [710] = 710,
  [711] = 711,
  [712] = 712,
  [713] = 713,
  [714] = 714,
  [715] = 715,
  [716] = 716,
  [717] = 717,
  [718] = 718,
  [719] = 719,
  [720] = 720,
  [721] = 721,
  [722] = 722,
  [723] = 723,
  [724] = 724,
  [725] = 725,
  [726] = 726,
  [727] = 727,
  [728] = 728,
  [729] = 729,
  [730] = 730,
  [731] = 731,
  [732] = 732,
  [733] = 733,
  [734] = 734,
  [735] = 735,
  [736] = 736,
  [737] = 737,
  [738] = 738,
  [739] = 739,
  [740] = 740,
  [741] = 741,
  [742] = 742,
  [743] = 743,
  [744] = 744,
  [745] = 745,
  [746] = 746,
  [747] = 747,
  [748] = 748,
  [749] = 749,
  [750] = 750,
  [751] = 751,
  [752] = 752,
  [753] = 753,
  [754] = 754,
  [755] = 755,
  [756] = 756,
  [757] = 757,
  [758] = 758,
  [759] = 759,
  [760] = 760,
  [761] = 761,
  [762] = 762,
  [763] = 762,
  [764] = 764,
  [765] = 765,
  [766] = 764,
  [767] = 767,
  [768] = 765,
  [769] = 769,
  [770] = 770,
  [771] = 771,
  [772] = 772,
  [773] = 773,
  [774] = 774,
  [775] = 775,
  [776] = 776,
  [777] = 777,
  [778] = 778,
  [779] = 779,
  [780] = 780,
  [781] = 781,
  [782] = 782,
  [783] = 783,
  [784] = 784,
  [785] = 784,
  [786] = 780,
  [787] = 781,
  [788] = 788,
  [789] = 789,
  [790] = 790,
  [791] = 791,
  [792] = 792,
  [793] = 793,
  [794] = 794,
  [795] = 795,
  [796] = 794,
  [797] = 797,
  [798] = 791,
  [799] = 788,
  [800] = 800,
  [801] = 800,
  [802] = 797,
  [803] = 795,
  [804] = 804,
  [805] = 805,
  [806] = 806,
  [807] = 807,
  [808] = 808,
  [809] = 809,
  [810] = 810,
  [811] = 806,
  [812] = 812,
  [813] = 813,
  [814] = 814,
  [815] = 815,
  [816] = 816,
  [817] = 804,
  [818] = 816,
  [819] = 819,
  [820] = 814,
  [821] = 821,
  [822] = 813,
  [823] = 810,
  [824] = 824,
  [825] = 815,
  [826] = 807,
  [827] = 827,
  [828] = 805,
  [829] = 809,
  [830] = 808,
  [831] = 827,
  [832] = 804,
  [833] = 827,
  [834] = 804,
  [835] = 827,
  [836] = 836,
  [837] = 837,
  [838] = 837,
  [839] = 839,
  [840] = 837,
  [841] = 837,
  [842] = 842,
  [843] = 843,
  [844] = 844,
  [845] = 845,
  [846] = 846,
  [847] = 847,
  [848] = 848,
  [849] = 846,
  [850] = 845,
  [851] = 845,
  [852] = 847,
  [853] = 853,
  [854] = 854,
  [855] = 855,
  [856] = 848,
  [857] = 857,
  [858] = 858,
  [859] = 848,
  [860] = 858,
  [861] = 842,
  [862] = 847,
  [863] = 846,
  [864] = 848,
  [865] = 865,
  [866] = 866,
  [867] = 867,
  [868] = 868,
  [869] = 869,
  [870] = 870,
  [871] = 871,
  [872] = 869,
  [873] = 873,
  [874] = 874,
  [875] = 867,
  [876] = 876,
  [877] = 877,
  [878] = 878,
  [879] = 879,
  [880] = 880,
  [881] = 881,
  [882] = 882,
  [883] = 883,
  [884] = 884,
  [885] = 885,
  [886] = 886,
  [887] = 887,
  [888] = 873,
  [889] = 889,
  [890] = 874,
  [891] = 867,
  [892] = 892,
  [893] = 883,
  [894] = 894,
  [895] = 876,
  [896] = 877,
  [897] = 897,
  [898] = 878,
  [899] = 587,
  [900] = 900,
  [901] = 884,
  [902] = 902,
  [903] = 903,
  [904] = 904,
  [905] = 885,
  [906] = 906,
  [907] = 907,
  [908] = 883,
  [909] = 909,
  [910] = 884,
  [911] = 911,
  [912] = 885,
  [913] = 913,
  [914] = 873,
  [915] = 915,
  [916] = 916,
  [917] = 917,
  [918] = 918,
  [919] = 887,
  [920] = 920,
  [921] = 921,
  [922] = 922,
  [923] = 923,
  [924] = 871,
  [925] = 925,
  [926] = 926,
  [927] = 869,
  [928] = 928,
  [929] = 929,
  [930] = 930,
  [931] = 871,
  [932] = 877,
  [933] = 879,
  [934] = 934,
  [935] = 935,
  [936] = 936,
  [937] = 937,
  [938] = 917,
  [939] = 939,
  [940] = 940,
  [941] = 941,
  [942] = 942,
  [943] = 886,
  [944] = 871,
  [945] = 945,
  [946] = 892,
  [947] = 903,
  [948] = 929,
  [949] = 869,
  [950] = 950,
  [951] = 951,
  [952] = 952,
  [953] = 889,
  [954] = 945,
  [955] = 939,
  [956] = 926,
  [957] = 934,
  [958] = 935,
  [959] = 942,
  [960] = 960,
  [961] = 961,
  [962] = 962,
  [963] = 906,
  [964] = 897,
  [965] = 911,
  [966] = 918,
  [967] = 870,
  [968] = 881,
  [969] = 882,
  [970] = 950,
  [971] = 900,
  [972] = 951,
  [973] = 921,
  [974] = 922,
  [975] = 923,
  [976] = 925,
  [977] = 930,
  [978] = 866,
  [979] = 979,
  [980] = 980,
  [981] = 981,
  [982] = 982,
  [983] = 979,
  [984] = 984,
  [985] = 920,
  [986] = 984,
  [987] = 952,
  [988] = 915,
  [989] = 989,
  [990] = 981,
  [991] = 894,
  [992] = 917,
  [993] = 886,
  [994] = 982,
  [995] = 995,
  [996] = 960,
  [997] = 874,
  [998] = 962,
  [999] = 999,
  [1000] = 894,
  [1001] = 873,
  [1002] = 887,
  [1003] = 867,
  [1004] = 928,
  [1005] = 961,
  [1006] = 1006,
  [1007] = 876,
  [1008] = 878,
  [1009] = 941,
  [1010] = 1010,
  [1011] = 1011,
  [1012] = 1012,
  [1013] = 1013,
  [1014] = 1014,
  [1015] = 587,
  [1016] = 1016,
  [1017] = 1017,
  [1018] = 1018,
  [1019] = 1019,
  [1020] = 1020,
  [1021] = 210,
  [1022] = 1022,
  [1023] = 209,
  [1024] = 402,
  [1025] = 370,
  [1026] = 387,
  [1027] = 373,
  [1028] = 1028,
  [1029] = 374,
  [1030] = 1030,
  [1031] = 372,
  [1032] = 371,
  [1033] = 609,
  [1034] = 1034,
  [1035] = 705,
  [1036] = 1036,
  [1037] = 539,
  [1038] = 1038,
  [1039] = 501,
  [1040] = 1040,
  [1041] = 1041,
  [1042] = 1042,
  [1043] = 1043,
  [1044] = 612,
  [1045] = 1045,
  [1046] = 1046,
  [1047] = 1047,
  [1048] = 1047,
  [1049] = 1049,
  [1050] = 1050,
  [1051] = 1051,
  [1052] = 1052,
  [1053] = 1051,
  [1054] = 1054,
  [1055] = 1051,
  [1056] = 1056,
  [1057] = 1057,
  [1058] = 1047,
  [1059] = 1059,
  [1060] = 1060,
  [1061] = 1010,
  [1062] = 1019,
  [1063] = 1063,
  [1064] = 1064,
  [1065] = 1065,
  [1066] = 1066,
  [1067] = 1067,
  [1068] = 1068,
  [1069] = 1069,
  [1070] = 1070,
  [1071] = 1071,
  [1072] = 1072,
  [1073] = 1073,
  [1074] = 1074,
  [1075] = 1075,
  [1076] = 1076,
  [1077] = 1077,
  [1078] = 1078,
  [1079] = 435,
  [1080] = 736,
  [1081] = 1081,
  [1082] = 1082,
  [1083] = 1083,
  [1084] = 671,
  [1085] = 680,
  [1086] = 1018,
  [1087] = 1087,
  [1088] = 1017,
  [1089] = 1089,
  [1090] = 1090,
  [1091] = 1016,
  [1092] = 1092,
  [1093] = 1093,
  [1094] = 1094,
  [1095] = 1095,
  [1096] = 1096,
  [1097] = 670,
  [1098] = 332,
  [1099] = 697,
  [1100] = 698,
  [1101] = 727,
  [1102] = 743,
  [1103] = 744,
  [1104] = 749,
  [1105] = 758,
  [1106] = 595,
  [1107] = 602,
  [1108] = 608,
  [1109] = 688,
  [1110] = 693,
  [1111] = 701,
  [1112] = 703,
  [1113] = 537,
  [1114] = 558,
  [1115] = 1115,
  [1116] = 1116,
  [1117] = 1117,
  [1118] = 1118,
  [1119] = 1119,
  [1120] = 590,
  [1121] = 593,
  [1122] = 594,
  [1123] = 596,
  [1124] = 597,
  [1125] = 600,
  [1126] = 603,
  [1127] = 605,
  [1128] = 606,
  [1129] = 607,
  [1130] = 610,
  [1131] = 611,
  [1132] = 613,
  [1133] = 615,
  [1134] = 625,
  [1135] = 626,
  [1136] = 627,
  [1137] = 628,
  [1138] = 761,
  [1139] = 487,
  [1140] = 631,
  [1141] = 632,
  [1142] = 634,
  [1143] = 635,
  [1144] = 636,
  [1145] = 637,
  [1146] = 638,
  [1147] = 640,
  [1148] = 641,
  [1149] = 642,
  [1150] = 643,
  [1151] = 644,
  [1152] = 645,
  [1153] = 647,
  [1154] = 653,
  [1155] = 658,
  [1156] = 1156,
  [1157] = 1157,
  [1158] = 1158,
  [1159] = 1159,
  [1160] = 668,
  [1161] = 669,
  [1162] = 672,
  [1163] = 675,
  [1164] = 676,
  [1165] = 677,
  [1166] = 678,
  [1167] = 679,
  [1168] = 684,
  [1169] = 685,
  [1170] = 686,
  [1171] = 689,
  [1172] = 690,
  [1173] = 691,
  [1174] = 692,
  [1175] = 695,
  [1176] = 696,
  [1177] = 702,
  [1178] = 709,
  [1179] = 710,
  [1180] = 711,
  [1181] = 712,
  [1182] = 713,
  [1183] = 714,
  [1184] = 715,
  [1185] = 716,
  [1186] = 717,
  [1187] = 718,
  [1188] = 719,
  [1189] = 720,
  [1190] = 721,
  [1191] = 722,
  [1192] = 723,
  [1193] = 724,
  [1194] = 725,
  [1195] = 726,
  [1196] = 728,
  [1197] = 729,
  [1198] = 730,
  [1199] = 731,
  [1200] = 732,
  [1201] = 733,
  [1202] = 734,
  [1203] = 735,
  [1204] = 737,
  [1205] = 738,
  [1206] = 739,
  [1207] = 741,
  [1208] = 748,
  [1209] = 750,
  [1210] = 1210,
  [1211] = 1211,
  [1212] = 760,
  [1213] = 629,
  [1214] = 601,
  [1215] = 707,
  [1216] = 708,
  [1217] = 740,
  [1218] = 742,
  [1219] = 498,
  [1220] = 581,
  [1221] = 582,
  [1222] = 583,
  [1223] = 584,
  [1224] = 585,
  [1225] = 586,
  [1226] = 588,
  [1227] = 589,
  [1228] = 591,
  [1229] = 592,
  [1230] = 1230,
  [1231] = 599,
  [1232] = 604,
  [1233] = 614,
  [1234] = 616,
  [1235] = 617,
  [1236] = 618,
  [1237] = 619,
  [1238] = 620,
  [1239] = 621,
  [1240] = 622,
  [1241] = 623,
  [1242] = 624,
  [1243] = 633,
  [1244] = 648,
  [1245] = 652,
  [1246] = 654,
  [1247] = 655,
  [1248] = 656,
  [1249] = 657,
  [1250] = 660,
  [1251] = 661,
  [1252] = 663,
  [1253] = 664,
  [1254] = 666,
  [1255] = 673,
  [1256] = 681,
  [1257] = 682,
  [1258] = 683,
  [1259] = 687,
  [1260] = 704,
  [1261] = 706,
  [1262] = 1262,
  [1263] = 1263,
  [1264] = 745,
  [1265] = 746,
  [1266] = 747,
  [1267] = 751,
  [1268] = 752,
  [1269] = 753,
  [1270] = 754,
  [1271] = 755,
  [1272] = 756,
  [1273] = 757,
  [1274] = 759,
  [1275] = 488,
  [1276] = 489,
  [1277] = 490,
  [1278] = 491,
  [1279] = 492,
  [1280] = 493,
  [1281] = 494,
  [1282] = 495,
  [1283] = 496,
  [1284] = 497,
  [1285] = 499,
  [1286] = 500,
  [1287] = 502,
  [1288] = 503,
  [1289] = 504,
  [1290] = 505,
  [1291] = 506,
  [1292] = 507,
  [1293] = 508,
  [1294] = 509,
  [1295] = 510,
  [1296] = 511,
  [1297] = 512,
  [1298] = 513,
  [1299] = 514,
  [1300] = 515,
  [1301] = 516,
  [1302] = 517,
  [1303] = 518,
  [1304] = 519,
  [1305] = 520,
  [1306] = 521,
  [1307] = 522,
  [1308] = 523,
  [1309] = 524,
  [1310] = 525,
  [1311] = 526,
  [1312] = 1312,
  [1313] = 1313,
  [1314] = 527,
  [1315] = 528,
  [1316] = 529,
  [1317] = 530,
  [1318] = 531,
  [1319] = 532,
  [1320] = 533,
  [1321] = 534,
  [1322] = 535,
  [1323] = 536,
  [1324] = 538,
  [1325] = 540,
  [1326] = 541,
  [1327] = 542,
  [1328] = 543,
  [1329] = 544,
  [1330] = 545,
  [1331] = 546,
  [1332] = 547,
  [1333] = 548,
  [1334] = 549,
  [1335] = 550,
  [1336] = 551,
  [1337] = 552,
  [1338] = 553,
  [1339] = 554,
  [1340] = 555,
  [1341] = 556,
  [1342] = 557,
  [1343] = 559,
  [1344] = 560,
  [1345] = 561,
  [1346] = 562,
  [1347] = 563,
  [1348] = 564,
  [1349] = 565,
  [1350] = 566,
  [1351] = 567,
  [1352] = 568,
  [1353] = 569,
  [1354] = 570,
  [1355] = 571,
  [1356] = 572,
  [1357] = 573,
  [1358] = 574,
  [1359] = 575,
  [1360] = 576,
  [1361] = 577,
  [1362] = 578,
  [1363] = 579,
  [1364] = 580,
  [1365] = 1365,
  [1366] = 1366,
  [1367] = 1367,
  [1368] = 587,
  [1369] = 1369,
  [1370] = 1019,
  [1371] = 1371,
  [1372] = 598,
  [1373] = 1373,
  [1374] = 1374,
  [1375] = 405,
  [1376] = 373,
  [1377] = 1377,
  [1378] = 1378,
  [1379] = 1379,
  [1380] = 1380,
  [1381] = 1381,
  [1382] = 1382,
  [1383] = 1383,
  [1384] = 1384,
  [1385] = 209,
  [1386] = 1386,
  [1387] = 375,
  [1388] = 1388,
  [1389] = 1389,
  [1390] = 371,
  [1391] = 391,
  [1392] = 1392,
  [1393] = 1393,
  [1394] = 1394,
  [1395] = 1395,
  [1396] = 1396,
  [1397] = 1397,
  [1398] = 1398,
  [1399] = 1399,
  [1400] = 1400,
  [1401] = 1401,
  [1402] = 1402,
  [1403] = 1403,
  [1404] = 398,
  [1405] = 1011,
  [1406] = 1406,
  [1407] = 1407,
  [1408] = 374,
  [1409] = 1409,
  [1410] = 1410,
  [1411] = 1411,
  [1412] = 1412,
  [1413] = 180,
  [1414] = 1414,
  [1415] = 1415,
  [1416] = 1416,
  [1417] = 210,
  [1418] = 1418,
  [1419] = 381,
  [1420] = 397,
  [1421] = 395,
  [1422] = 396,
  [1423] = 403,
  [1424] = 392,
  [1425] = 404,
  [1426] = 379,
  [1427] = 383,
  [1428] = 1428,
  [1429] = 370,
  [1430] = 386,
  [1431] = 1431,
  [1432] = 1432,
  [1433] = 1433,
  [1434] = 1434,
  [1435] = 1435,
  [1436] = 1436,
  [1437] = 1437,
  [1438] = 1438,
  [1439] = 1439,
  [1440] = 1440,
  [1441] = 1441,
  [1442] = 1442,
  [1443] = 382,
  [1444] = 1444,
  [1445] = 1445,
  [1446] = 378,
  [1447] = 1447,
  [1448] = 1448,
  [1449] = 393,
  [1450] = 1450,
  [1451] = 185,
  [1452] = 1452,
  [1453] = 1453,
  [1454] = 372,
  [1455] = 400,
  [1456] = 1456,
  [1457] = 1457,
  [1458] = 1458,
  [1459] = 1459,
  [1460] = 1460,
  [1461] = 1461,
  [1462] = 587,
  [1463] = 1463,
  [1464] = 668,
  [1465] = 1019,
  [1466] = 1466,
  [1467] = 385,
  [1468] = 1468,
  [1469] = 1469,
  [1470] = 1470,
  [1471] = 1471,
  [1472] = 182,
  [1473] = 1473,
  [1474] = 1474,
  [1475] = 387,
  [1476] = 1476,
  [1477] = 1477,
  [1478] = 1478,
  [1479] = 1479,
  [1480] = 1480,
  [1481] = 1481,
  [1482] = 1482,
  [1483] = 1483,
  [1484] = 1484,
  [1485] = 1485,
  [1486] = 1486,
  [1487] = 1487,
  [1488] = 1488,
  [1489] = 177,
  [1490] = 1490,
  [1491] = 184,
  [1492] = 1492,
  [1493] = 1493,
  [1494] = 402,
  [1495] = 1495,
  [1496] = 1496,
  [1497] = 1019,
  [1498] = 1495,
  [1499] = 1019,
  [1500] = 1495,
  [1501] = 1501,
  [1502] = 1502,
  [1503] = 1503,
  [1504] = 1504,
  [1505] = 1505,
  [1506] = 1506,
  [1507] = 1507,
  [1508] = 1508,
  [1509] = 1509,
  [1510] = 1510,
  [1511] = 1511,
  [1512] = 1020,
  [1513] = 1513,
  [1514] = 1514,
  [1515] = 1057,
  [1516] = 1516,
  [1517] = 1517,
  [1518] = 1518,
  [1519] = 1519,
  [1520] = 1504,
  [1521] = 1521,
  [1522] = 1522,
  [1523] = 1509,
  [1524] = 1524,
  [1525] = 1525,
  [1526] = 1526,
  [1527] = 1527,
  [1528] = 1030,
  [1529] = 1529,
  [1530] = 1028,
  [1531] = 1531,
  [1532] = 1532,
  [1533] = 1533,
  [1534] = 1534,
  [1535] = 1535,
  [1536] = 1022,
  [1537] = 1537,
  [1538] = 1057,
  [1539] = 1057,
  [1540] = 1040,
  [1541] = 1073,
  [1542] = 1071,
  [1543] = 1046,
  [1544] = 1038,
  [1545] = 1041,
  [1546] = 1042,
  [1547] = 1064,
  [1548] = 1034,
  [1549] = 1071,
  [1550] = 1073,
  [1551] = 1078,
  [1552] = 1487,
  [1553] = 1064,
  [1554] = 1078,
  [1555] = 1453,
  [1556] = 1036,
  [1557] = 1043,
  [1558] = 1487,
  [1559] = 1501,
  [1560] = 1560,
  [1561] = 1561,
  [1562] = 1560,
  [1563] = 1456,
  [1564] = 1564,
  [1565] = 1565,
  [1566] = 1566,
  [1567] = 332,
  [1568] = 1373,
  [1569] = 1060,
  [1570] = 1365,
  [1571] = 1049,
  [1572] = 1564,
  [1573] = 1453,
  [1574] = 1566,
  [1575] = 1487,
  [1576] = 1564,
  [1577] = 1566,
  [1578] = 1210,
  [1579] = 1566,
  [1580] = 1496,
  [1581] = 1564,
  [1582] = 1365,
  [1583] = 1262,
  [1584] = 1565,
  [1585] = 1050,
  [1586] = 1373,
  [1587] = 1312,
  [1588] = 1054,
  [1589] = 1561,
  [1590] = 1156,
  [1591] = 1565,
  [1592] = 1456,
  [1593] = 1524,
  [1594] = 371,
  [1595] = 1511,
  [1596] = 1525,
  [1597] = 1371,
  [1598] = 1075,
  [1599] = 1599,
  [1600] = 1510,
  [1601] = 1115,
  [1602] = 1116,
  [1603] = 1117,
  [1604] = 1502,
  [1605] = 1505,
  [1606] = 1516,
  [1607] = 1503,
  [1608] = 1514,
  [1609] = 1519,
  [1610] = 1118,
  [1611] = 1119,
  [1612] = 1076,
  [1613] = 1599,
  [1614] = 1614,
  [1615] = 1087,
  [1616] = 1157,
  [1617] = 1617,
  [1618] = 1089,
  [1619] = 1158,
  [1620] = 1090,
  [1621] = 1517,
  [1622] = 1159,
  [1623] = 1623,
  [1624] = 1617,
  [1625] = 1092,
  [1626] = 1369,
  [1627] = 1506,
  [1628] = 1093,
  [1629] = 1211,
  [1630] = 1614,
  [1631] = 1507,
  [1632] = 1508,
  [1633] = 1081,
  [1634] = 1263,
  [1635] = 1094,
  [1636] = 1313,
  [1637] = 1496,
  [1638] = 1638,
  [1639] = 1513,
  [1640] = 1521,
  [1641] = 1526,
  [1642] = 1095,
  [1643] = 1643,
  [1644] = 1082,
  [1645] = 1522,
  [1646] = 1096,
  [1647] = 1501,
  [1648] = 374,
  [1649] = 1083,
  [1650] = 370,
  [1651] = 373,
  [1652] = 372,
  [1653] = 1518,
  [1654] = 1654,
  [1655] = 1399,
  [1656] = 1656,
  [1657] = 1402,
  [1658] = 1658,
  [1659] = 1470,
  [1660] = 1471,
  [1661] = 1444,
  [1662] = 1662,
  [1663] = 1476,
  [1664] = 1474,
  [1665] = 1665,
  [1666] = 1482,
  [1667] = 1432,
  [1668] = 1478,
  [1669] = 1479,
  [1670] = 1480,
  [1671] = 1459,
  [1672] = 1484,
  [1673] = 1485,
  [1674] = 1486,
  [1675] = 1386,
  [1676] = 1394,
  [1677] = 1418,
  [1678] = 1678,
  [1679] = 1679,
  [1680] = 1516,
  [1681] = 1503,
  [1682] = 1414,
  [1683] = 1384,
  [1684] = 1416,
  [1685] = 1514,
  [1686] = 1519,
  [1687] = 1687,
  [1688] = 1502,
  [1689] = 1473,
  [1690] = 1690,
  [1691] = 1450,
  [1692] = 1435,
  [1693] = 1436,
  [1694] = 1016,
  [1695] = 1695,
  [1696] = 1477,
  [1697] = 1439,
  [1698] = 1481,
  [1699] = 1483,
  [1700] = 1440,
  [1701] = 1488,
  [1702] = 1702,
  [1703] = 1517,
  [1704] = 1526,
  [1705] = 1705,
  [1706] = 1490,
  [1707] = 1492,
  [1708] = 1513,
  [1709] = 1521,
  [1710] = 1522,
  [1711] = 1524,
  [1712] = 1525,
  [1713] = 1505,
  [1714] = 1506,
  [1715] = 1507,
  [1716] = 1508,
  [1717] = 1510,
  [1718] = 1511,
  [1719] = 1493,
  [1720] = 1378,
  [1721] = 1379,
  [1722] = 1389,
  [1723] = 1382,
  [1724] = 1383,
  [1725] = 1431,
  [1726] = 1518,
  [1727] = 1406,
  [1728] = 1409,
  [1729] = 1410,
  [1730] = 1412,
  [1731] = 1415,
  [1732] = 1732,
  [1733] = 1397,
  [1734] = 1457,
  [1735] = 1458,
  [1736] = 1460,
  [1737] = 1461,
  [1738] = 1466,
  [1739] = 1392,
  [1740] = 1740,
  [1741] = 1400,
  [1742] = 1742,
  [1743] = 1403,
  [1744] = 1411,
  [1745] = 435,
  [1746] = 381,
  [1747] = 395,
  [1748] = 396,
  [1749] = 379,
  [1750] = 383,
  [1751] = 1502,
  [1752] = 1437,
  [1753] = 382,
  [1754] = 1517,
  [1755] = 1755,
  [1756] = 378,
  [1757] = 393,
  [1758] = 1526,
  [1759] = 385,
  [1760] = 1513,
  [1761] = 1521,
  [1762] = 1522,
  [1763] = 1524,
  [1764] = 1525,
  [1765] = 1505,
  [1766] = 1506,
  [1767] = 1507,
  [1768] = 1508,
  [1769] = 1510,
  [1770] = 1398,
  [1771] = 1380,
  [1772] = 1381,
  [1773] = 375,
  [1774] = 391,
  [1775] = 1518,
  [1776] = 398,
  [1777] = 1407,
  [1778] = 1463,
  [1779] = 403,
  [1780] = 404,
  [1781] = 386,
  [1782] = 400,
  [1783] = 405,
  [1784] = 1784,
  [1785] = 392,
  [1786] = 177,
  [1787] = 180,
  [1788] = 397,
  [1789] = 182,
  [1790] = 1502,
  [1791] = 184,
  [1792] = 402,
  [1793] = 1526,
  [1794] = 185,
  [1795] = 387,
  [1796] = 1518,
  [1797] = 209,
  [1798] = 210,
  [1799] = 1665,
  [1800] = 1800,
  [1801] = 1801,
  [1802] = 1802,
  [1803] = 1438,
  [1804] = 1702,
  [1805] = 1441,
  [1806] = 1801,
  [1807] = 1654,
  [1808] = 1808,
  [1809] = 1433,
  [1810] = 1658,
  [1811] = 1811,
  [1812] = 1784,
  [1813] = 1813,
  [1814] = 1814,
  [1815] = 1815,
  [1816] = 1816,
  [1817] = 1690,
  [1818] = 1818,
  [1819] = 1656,
  [1820] = 1678,
  [1821] = 1452,
  [1822] = 1808,
  [1823] = 1801,
  [1824] = 1516,
  [1825] = 1503,
  [1826] = 1702,
  [1827] = 1514,
  [1828] = 1519,
  [1829] = 1654,
  [1830] = 1434,
  [1831] = 1814,
  [1832] = 1818,
  [1833] = 1814,
  [1834] = 1818,
  [1835] = 1374,
  [1836] = 1377,
  [1837] = 1801,
  [1838] = 1702,
  [1839] = 1654,
  [1840] = 1814,
  [1841] = 1818,
  [1842] = 1801,
  [1843] = 1654,
  [1844] = 1815,
  [1845] = 1814,
  [1846] = 1818,
  [1847] = 1801,
  [1848] = 1654,
  [1849] = 1814,
  [1850] = 1818,
  [1851] = 1801,
  [1852] = 1654,
  [1853] = 1814,
  [1854] = 1818,
  [1855] = 1855,
  [1856] = 1388,
  [1857] = 1442,
  [1858] = 1445,
  [1859] = 1447,
  [1860] = 1393,
  [1861] = 1395,
  [1862] = 1862,
  [1863] = 1511,
  [1864] = 1864,
  [1865] = 1865,
  [1866] = 1862,
  [1867] = 1742,
  [1868] = 1687,
  [1869] = 1869,
  [1870] = 1870,
  [1871] = 1871,
  [1872] = 1872,
  [1873] = 1873,
  [1874] = 1874,
  [1875] = 1872,
  [1876] = 1876,
  [1877] = 1877,
  [1878] = 1878,
  [1879] = 1873,
  [1880] = 1880,
  [1881] = 1881,
  [1882] = 1882,
  [1883] = 1883,
  [1884] = 1871,
  [1885] = 1881,
  [1886] = 1886,
  [1887] = 1883,
  [1888] = 1877,
  [1889] = 1882,
  [1890] = 1876,
  [1891] = 1891,
  [1892] = 1874,
  [1893] = 1878,
  [1894] = 1894,
  [1895] = 1877,
  [1896] = 1896,
  [1897] = 1897,
  [1898] = 1891,
  [1899] = 1894,
  [1900] = 1865,
  [1901] = 1901,
  [1902] = 1902,
  [1903] = 1903,
  [1904] = 1901,
  [1905] = 1905,
  [1906] = 1905,
  [1907] = 1903,
  [1908] = 1902,
  [1909] = 1909,
  [1910] = 1910,
  [1911] = 1911,
  [1912] = 1912,
  [1913] = 1913,
  [1914] = 1913,
  [1915] = 1913,
  [1916] = 1912,
  [1917] = 1913,
  [1918] = 1912,
  [1919] = 1913,
  [1920] = 1920,
  [1921] = 1920,
  [1922] = 1020,
  [1923] = 182,
  [1924] = 185,
  [1925] = 1046,
  [1926] = 1030,
  [1927] = 1022,
  [1928] = 1038,
  [1929] = 1028,
  [1930] = 1041,
  [1931] = 1931,
  [1932] = 1042,
  [1933] = 1040,
  [1934] = 1046,
  [1935] = 1043,
  [1936] = 1034,
  [1937] = 1041,
  [1938] = 1036,
  [1939] = 1042,
  [1940] = 1038,
  [1941] = 1060,
  [1942] = 1054,
  [1943] = 1050,
  [1944] = 1049,
  [1945] = 1156,
  [1946] = 1210,
  [1947] = 1312,
  [1948] = 1115,
  [1949] = 1262,
  [1950] = 1950,
  [1951] = 1951,
  [1952] = 1087,
  [1953] = 1089,
  [1954] = 1950,
  [1955] = 1090,
  [1956] = 1956,
  [1957] = 1081,
  [1958] = 1116,
  [1959] = 1117,
  [1960] = 1118,
  [1961] = 1119,
  [1962] = 1083,
  [1963] = 1963,
  [1964] = 402,
  [1965] = 1092,
  [1966] = 1093,
  [1967] = 1967,
  [1968] = 1967,
  [1969] = 387,
  [1970] = 1082,
  [1971] = 209,
  [1972] = 1537,
  [1973] = 1211,
  [1974] = 1967,
  [1975] = 1096,
  [1976] = 210,
  [1977] = 1313,
  [1978] = 1094,
  [1979] = 1157,
  [1980] = 1158,
  [1981] = 1159,
  [1982] = 1095,
  [1983] = 1956,
  [1984] = 1263,
  [1985] = 1436,
  [1986] = 1414,
  [1987] = 1535,
  [1988] = 1988,
  [1989] = 1378,
  [1990] = 1956,
  [1991] = 1416,
  [1992] = 1381,
  [1993] = 1458,
  [1994] = 1439,
  [1995] = 1440,
  [1996] = 1435,
  [1997] = 1380,
  [1998] = 1461,
  [1999] = 1383,
  [2000] = 1529,
  [2001] = 1406,
  [2002] = 1407,
  [2003] = 1409,
  [2004] = 1437,
  [2005] = 1410,
  [2006] = 1460,
  [2007] = 1379,
  [2008] = 1463,
  [2009] = 1412,
  [2010] = 1415,
  [2011] = 1457,
  [2012] = 1466,
  [2013] = 2013,
  [2014] = 1382,
  [2015] = 1534,
  [2016] = 1392,
  [2017] = 2017,
  [2018] = 2018,
  [2019] = 184,
  [2020] = 180,
  [2021] = 2021,
  [2022] = 1041,
  [2023] = 1042,
  [2024] = 1038,
  [2025] = 2025,
  [2026] = 1046,
  [2027] = 1042,
  [2028] = 1038,
  [2029] = 1046,
  [2030] = 1046,
  [2031] = 1041,
  [2032] = 1931,
  [2033] = 1038,
  [2034] = 2034,
  [2035] = 1041,
  [2036] = 1042,
  [2037] = 2037,
  [2038] = 2038,
  [2039] = 2039,
  [2040] = 2039,
  [2041] = 2041,
  [2042] = 2042,
  [2043] = 2043,
  [2044] = 2044,
  [2045] = 2045,
  [2046] = 2043,
  [2047] = 2044,
  [2048] = 2038,
  [2049] = 2049,
  [2050] = 2045,
  [2051] = 2051,
  [2052] = 2049,
  [2053] = 2041,
  [2054] = 2054,
  [2055] = 2055,
  [2056] = 2056,
  [2057] = 2057,
  [2058] = 2058,
  [2059] = 2059,
  [2060] = 2060,
  [2061] = 2061,
  [2062] = 2062,
  [2063] = 2063,
  [2064] = 372,
  [2065] = 374,
  [2066] = 1046,
  [2067] = 1038,
  [2068] = 1041,
  [2069] = 1042,
  [2070] = 2061,
  [2071] = 2071,
  [2072] = 2072,
  [2073] = 2073,
  [2074] = 2074,
  [2075] = 2075,
  [2076] = 2076,
  [2077] = 2071,
  [2078] = 373,
  [2079] = 2079,
  [2080] = 2080,
  [2081] = 2081,
  [2082] = 2082,
  [2083] = 2083,
  [2084] = 1951,
  [2085] = 2082,
  [2086] = 2054,
  [2087] = 371,
  [2088] = 2057,
  [2089] = 2072,
  [2090] = 2055,
  [2091] = 2062,
  [2092] = 2092,
  [2093] = 370,
  [2094] = 2092,
  [2095] = 2095,
  [2096] = 2096,
  [2097] = 2097,
  [2098] = 2098,
  [2099] = 2099,
  [2100] = 2100,
  [2101] = 2101,
  [2102] = 2102,
  [2103] = 2103,
  [2104] = 2104,
  [2105] = 2105,
  [2106] = 2106,
  [2107] = 2107,
  [2108] = 2108,
  [2109] = 2109,
  [2110] = 2097,
  [2111] = 397,
  [2112] = 2097,
  [2113] = 2113,
  [2114] = 2114,
  [2115] = 2115,
  [2116] = 2116,
  [2117] = 2117,
  [2118] = 2118,
  [2119] = 2119,
  [2120] = 2120,
  [2121] = 2121,
  [2122] = 2122,
  [2123] = 2117,
  [2124] = 2124,
  [2125] = 1963,
  [2126] = 2126,
  [2127] = 2127,
  [2128] = 2128,
  [2129] = 2129,
  [2130] = 2130,
  [2131] = 2131,
  [2132] = 1951,
  [2133] = 2133,
  [2134] = 2134,
  [2135] = 2135,
  [2136] = 2136,
  [2137] = 2137,
  [2138] = 2109,
  [2139] = 1951,
  [2140] = 2140,
  [2141] = 2130,
  [2142] = 2142,
  [2143] = 2143,
  [2144] = 2144,
  [2145] = 2145,
  [2146] = 2133,
  [2147] = 2147,
  [2148] = 2148,
  [2149] = 2149,
  [2150] = 2150,
  [2151] = 1963,
  [2152] = 2152,
  [2153] = 2153,
  [2154] = 2154,
  [2155] = 2155,
  [2156] = 2156,
  [2157] = 2157,
  [2158] = 1049,
  [2159] = 1050,
  [2160] = 2160,
  [2161] = 1054,
  [2162] = 1060,
  [2163] = 2153,
  [2164] = 2164,
  [2165] = 2160,
  [2166] = 2166,
  [2167] = 2167,
  [2168] = 2168,
  [2169] = 2169,
  [2170] = 2170,
  [2171] = 2171,
  [2172] = 2172,
  [2173] = 2164,
  [2174] = 2174,
  [2175] = 2168,
  [2176] = 2176,
  [2177] = 2153,
  [2178] = 2178,
  [2179] = 2150,
  [2180] = 2148,
  [2181] = 2170,
  [2182] = 2172,
  [2183] = 2183,
  [2184] = 2164,
  [2185] = 2183,
  [2186] = 2186,
  [2187] = 2174,
  [2188] = 1951,
  [2189] = 2178,
  [2190] = 2176,
  [2191] = 1963,
  [2192] = 2152,
  [2193] = 2155,
  [2194] = 2167,
  [2195] = 2157,
  [2196] = 2196,
  [2197] = 2197,
  [2198] = 2198,
  [2199] = 2199,
  [2200] = 2200,
  [2201] = 2201,
  [2202] = 2202,
  [2203] = 2203,
  [2204] = 2204,
  [2205] = 2205,
  [2206] = 2206,
  [2207] = 2207,
  [2208] = 2208,
  [2209] = 2209,
  [2210] = 2206,
  [2211] = 2207,
  [2212] = 2208,
  [2213] = 2213,
  [2214] = 2214,
  [2215] = 2215,
  [2216] = 2216,
  [2217] = 2217,
  [2218] = 2218,
  [2219] = 2219,
  [2220] = 2220,
  [2221] = 2221,
  [2222] = 2196,
  [2223] = 2216,
  [2224] = 2224,
  [2225] = 2202,
  [2226] = 2226,
  [2227] = 2227,
  [2228] = 2228,
  [2229] = 2229,
  [2230] = 2230,
  [2231] = 2201,
  [2232] = 2217,
  [2233] = 2233,
  [2234] = 2215,
  [2235] = 2235,
  [2236] = 2236,
  [2237] = 2226,
  [2238] = 2238,
  [2239] = 2239,
  [2240] = 2229,
  [2241] = 2203,
  [2242] = 1963,
  [2243] = 2230,
  [2244] = 2213,
  [2245] = 2245,
  [2246] = 2200,
  [2247] = 2209,
  [2248] = 2221,
  [2249] = 2204,
  [2250] = 2218,
  [2251] = 2197,
  [2252] = 2252,
  [2253] = 2253,
  [2254] = 2254,
  [2255] = 2255,
  [2256] = 2256,
  [2257] = 2257,
  [2258] = 2258,
  [2259] = 2259,
  [2260] = 2259,
  [2261] = 2261,
  [2262] = 2252,
  [2263] = 2263,
  [2264] = 2264,
  [2265] = 2265,
  [2266] = 2266,
  [2267] = 2267,
  [2268] = 2268,
  [2269] = 2269,
  [2270] = 2270,
  [2271] = 2271,
  [2272] = 2272,
  [2273] = 2272,
  [2274] = 2274,
  [2275] = 2275,
  [2276] = 2276,
  [2277] = 2277,
  [2278] = 2278,
  [2279] = 2279,
  [2280] = 2280,
  [2281] = 2254,
  [2282] = 2264,
  [2283] = 2267,
  [2284] = 2255,
  [2285] = 2280,
  [2286] = 2286,
  [2287] = 2017,
  [2288] = 2288,
  [2289] = 2289,
  [2290] = 2269,
  [2291] = 2291,
  [2292] = 2274,
  [2293] = 2275,
  [2294] = 2253,
  [2295] = 2276,
  [2296] = 2277,
  [2297] = 2257,
  [2298] = 2278,
  [2299] = 2299,
  [2300] = 2286,
  [2301] = 2271,
  [2302] = 2291,
  [2303] = 2303,
  [2304] = 2288,
  [2305] = 2305,
  [2306] = 2306,
  [2307] = 2286,
  [2308] = 2256,
  [2309] = 2263,
  [2310] = 2310,
  [2311] = 2311,
  [2312] = 2312,
  [2313] = 2313,
  [2314] = 2311,
  [2315] = 2315,
  [2316] = 2316,
  [2317] = 2317,
  [2318] = 2318,
  [2319] = 2319,
  [2320] = 2320,
  [2321] = 2034,
  [2322] = 2313,
  [2323] = 2318,
  [2324] = 2324,
  [2325] = 2325,
  [2326] = 2324,
  [2327] = 2325,
  [2328] = 2328,
  [2329] = 2329,
  [2330] = 2025,
  [2331] = 2331,
  [2332] = 2332,
  [2333] = 2333,
  [2334] = 2334,
  [2335] = 2335,
  [2336] = 2333,
  [2337] = 2337,
  [2338] = 2338,
  [2339] = 2339,
  [2340] = 2339,
  [2341] = 2341,
  [2342] = 2335,
  [2343] = 2343,
  [2344] = 2344,
  [2345] = 2345,
  [2346] = 2346,
  [2347] = 2347,
  [2348] = 2328,
  [2349] = 2335,
  [2350] = 2331,
  [2351] = 2332,
  [2352] = 2352,
  [2353] = 2353,
  [2354] = 2312,
  [2355] = 2355,
  [2356] = 2356,
  [2357] = 2357,
  [2358] = 2358,
  [2359] = 2355,
  [2360] = 2341,
  [2361] = 2357,
  [2362] = 2333,
  [2363] = 2346,
  [2364] = 2037,
  [2365] = 2365,
  [2366] = 2366,
  [2367] = 2329,
  [2368] = 2319,
  [2369] = 2339,
  [2370] = 2341,
  [2371] = 2320,
  [2372] = 2329,
  [2373] = 2373,
  [2374] = 2374,
  [2375] = 2343,
  [2376] = 2352,
  [2377] = 2377,
  [2378] = 2335,
  [2379] = 2344,
  [2380] = 2345,
  [2381] = 2377,
  [2382] = 2382,
  [2383] = 2383,
  [2384] = 2384,
  [2385] = 2365,
  [2386] = 2374,
  [2387] = 2387,
  [2388] = 2353,
  [2389] = 180,
  [2390] = 2316,
  [2391] = 182,
  [2392] = 184,
  [2393] = 185,
  [2394] = 2317,
  [2395] = 2395,
  [2396] = 2396,
  [2397] = 2397,
  [2398] = 2397,
  [2399] = 2356,
  [2400] = 2400,
  [2401] = 2373,
  [2402] = 2315,
  [2403] = 2396,
  [2404] = 2366,
  [2405] = 2400,
  [2406] = 2384,
  [2407] = 2310,
  [2408] = 2383,
  [2409] = 2409,
  [2410] = 2410,
  [2411] = 2411,
  [2412] = 2411,
  [2413] = 2413,
  [2414] = 2414,
  [2415] = 2415,
  [2416] = 2416,
  [2417] = 2417,
  [2418] = 2418,
  [2419] = 2419,
  [2420] = 2420,
  [2421] = 2421,
  [2422] = 2422,
  [2423] = 2423,
  [2424] = 2413,
  [2425] = 2425,
  [2426] = 2426,
  [2427] = 2427,
  [2428] = 2425,
  [2429] = 2429,
  [2430] = 2430,
  [2431] = 2431,
  [2432] = 1016,
  [2433] = 2433,
  [2434] = 2434,
  [2435] = 2435,
  [2436] = 2436,
  [2437] = 2437,
  [2438] = 2423,
  [2439] = 2420,
  [2440] = 2440,
  [2441] = 2441,
  [2442] = 2442,
  [2443] = 2443,
  [2444] = 2420,
  [2445] = 2445,
  [2446] = 2410,
  [2447] = 2447,
  [2448] = 2416,
  [2449] = 2442,
  [2450] = 2437,
  [2451] = 2451,
  [2452] = 1068,
  [2453] = 2453,
  [2454] = 2454,
  [2455] = 2455,
  [2456] = 2456,
  [2457] = 2441,
  [2458] = 2418,
  [2459] = 2456,
  [2460] = 1366,
  [2461] = 2421,
  [2462] = 1077,
  [2463] = 1230,
  [2464] = 2464,
  [2465] = 2451,
  [2466] = 2466,
  [2467] = 1367,
  [2468] = 2468,
  [2469] = 2469,
  [2470] = 2409,
  [2471] = 2471,
  [2472] = 2419,
  [2473] = 2473,
  [2474] = 2474,
  [2475] = 2420,
  [2476] = 2433,
  [2477] = 2477,
  [2478] = 2427,
  [2479] = 2479,
  [2480] = 2480,
  [2481] = 2481,
  [2482] = 1069,
  [2483] = 2483,
  [2484] = 2454,
  [2485] = 2421,
  [2486] = 2422,
  [2487] = 2455,
  [2488] = 2488,
  [2489] = 2410,
  [2490] = 2490,
  [2491] = 2430,
  [2492] = 2492,
  [2493] = 2436,
  [2494] = 2443,
  [2495] = 2433,
  [2496] = 1065,
  [2497] = 1066,
  [2498] = 1067,
  [2499] = 2499,
  [2500] = 2430,
  [2501] = 2501,
  [2502] = 2502,
  [2503] = 2419,
  [2504] = 2504,
  [2505] = 2505,
  [2506] = 2506,
  [2507] = 2434,
  [2508] = 2508,
  [2509] = 2509,
  [2510] = 2510,
  [2511] = 2415,
  [2512] = 2409,
  [2513] = 1063,
  [2514] = 2514,
  [2515] = 2515,
  [2516] = 2516,
  [2517] = 2517,
  [2518] = 2518,
  [2519] = 2519,
  [2520] = 2515,
  [2521] = 2521,
  [2522] = 2522,
  [2523] = 2523,
  [2524] = 2524,
  [2525] = 2525,
  [2526] = 2526,
  [2527] = 2527,
  [2528] = 2515,
  [2529] = 2516,
  [2530] = 2530,
  [2531] = 2531,
  [2532] = 2532,
  [2533] = 197,
  [2534] = 2534,
  [2535] = 2535,
  [2536] = 2536,
  [2537] = 2537,
  [2538] = 2516,
  [2539] = 2536,
  [2540] = 2540,
  [2541] = 2517,
  [2542] = 2542,
  [2543] = 2543,
  [2544] = 2534,
  [2545] = 2545,
  [2546] = 2524,
  [2547] = 2547,
  [2548] = 2531,
  [2549] = 2549,
  [2550] = 2550,
  [2551] = 2551,
  [2552] = 2552,
  [2553] = 2553,
  [2554] = 2554,
  [2555] = 2555,
  [2556] = 2542,
  [2557] = 2545,
  [2558] = 2558,
  [2559] = 2525,
  [2560] = 2560,
  [2561] = 2561,
  [2562] = 2562,
  [2563] = 2563,
  [2564] = 2564,
  [2565] = 2565,
  [2566] = 2566,
  [2567] = 2567,
  [2568] = 2568,
  [2569] = 2569,
  [2570] = 2570,
  [2571] = 2571,
  [2572] = 2572,
  [2573] = 2551,
  [2574] = 2574,
  [2575] = 2575,
  [2576] = 2522,
  [2577] = 2552,
  [2578] = 2578,
  [2579] = 2579,
  [2580] = 2580,
  [2581] = 2558,
  [2582] = 2582,
  [2583] = 2521,
  [2584] = 2584,
  [2585] = 2585,
  [2586] = 2586,
  [2587] = 2582,
  [2588] = 2582,
  [2589] = 2589,
  [2590] = 2590,
  [2591] = 2569,
  [2592] = 2592,
  [2593] = 2593,
  [2594] = 2080,
  [2595] = 2595,
  [2596] = 2534,
  [2597] = 2545,
  [2598] = 2524,
  [2599] = 2531,
  [2600] = 2540,
  [2601] = 2542,
  [2602] = 2558,
  [2603] = 2525,
  [2604] = 2560,
  [2605] = 2562,
  [2606] = 2564,
  [2607] = 2565,
  [2608] = 2569,
  [2609] = 2570,
  [2610] = 2592,
  [2611] = 2611,
  [2612] = 2612,
  [2613] = 2552,
  [2614] = 2083,
  [2615] = 2562,
  [2616] = 2574,
  [2617] = 2582,
  [2618] = 2618,
  [2619] = 2524,
  [2620] = 2531,
  [2621] = 2621,
  [2622] = 2558,
  [2623] = 2564,
  [2624] = 2565,
  [2625] = 2569,
  [2626] = 2570,
  [2627] = 2534,
  [2628] = 2056,
  [2629] = 2545,
  [2630] = 2524,
  [2631] = 2564,
  [2632] = 2079,
  [2633] = 2569,
  [2634] = 2570,
  [2635] = 2524,
  [2636] = 2636,
  [2637] = 2549,
  [2638] = 2638,
  [2639] = 2569,
  [2640] = 2570,
  [2641] = 2641,
  [2642] = 2642,
  [2643] = 2569,
  [2644] = 2570,
  [2645] = 2569,
  [2646] = 2554,
  [2647] = 2569,
  [2648] = 2570,
  [2649] = 2569,
  [2650] = 2570,
  [2651] = 2569,
  [2652] = 2570,
  [2653] = 2653,
  [2654] = 2531,
  [2655] = 2567,
  [2656] = 2549,
  [2657] = 2537,
  [2658] = 2658,
  [2659] = 2553,
  [2660] = 2658,
  [2661] = 2661,
  [2662] = 2662,
  [2663] = 2663,
  [2664] = 2664,
  [2665] = 2537,
  [2666] = 2666,
  [2667] = 2667,
  [2668] = 2553,
  [2669] = 2669,
  [2670] = 2670,
  [2671] = 2671,
  [2672] = 2519,
  [2673] = 2673,
  [2674] = 2585,
  [2675] = 175,
  [2676] = 2542,
  [2677] = 2558,
  [2678] = 2678,
  [2679] = 2525,
  [2680] = 2678,
  [2681] = 2560,
  [2682] = 2579,
  [2683] = 2095,
  [2684] = 2663,
  [2685] = 2685,
  [2686] = 2642,
  [2687] = 2687,
  [2688] = 2688,
  [2689] = 2689,
  [2690] = 2690,
  [2691] = 2691,
  [2692] = 2692,
  [2693] = 2592,
  [2694] = 2575,
  [2695] = 2562,
  [2696] = 2590,
  [2697] = 2527,
  [2698] = 2698,
  [2699] = 2076,
  [2700] = 2671,
  [2701] = 2586,
  [2702] = 2564,
  [2703] = 2565,
  [2704] = 2535,
  [2705] = 2555,
  [2706] = 2706,
  [2707] = 2526,
  [2708] = 2566,
  [2709] = 2709,
  [2710] = 2710,
  [2711] = 2666,
  [2712] = 2060,
  [2713] = 2564,
  [2714] = 2567,
  [2715] = 2710,
  [2716] = 2552,
  [2717] = 2537,
  [2718] = 2074,
  [2719] = 2719,
  [2720] = 2663,
  [2721] = 2721,
  [2722] = 2722,
  [2723] = 2669,
  [2724] = 2724,
  [2725] = 2725,
  [2726] = 2726,
  [2727] = 2523,
  [2728] = 2571,
  [2729] = 2537,
  [2730] = 2565,
  [2731] = 2663,
  [2732] = 2560,
  [2733] = 2563,
  [2734] = 2734,
  [2735] = 2595,
  [2736] = 2621,
  [2737] = 2724,
  [2738] = 2670,
  [2739] = 2570,
  [2740] = 2569,
  [2741] = 2574,
  [2742] = 2568,
  [2743] = 2547,
  [2744] = 2570,
  [2745] = 2692,
  [2746] = 2584,
  [2747] = 2571,
  [2748] = 2561,
  [2749] = 2543,
  [2750] = 2550,
  [2751] = 2566,
  [2752] = 2687,
  [2753] = 2662,
  [2754] = 2584,
  [2755] = 2688,
  [2756] = 2570,
  [2757] = 2136,
  [2758] = 2758,
  [2759] = 2759,
  [2760] = 2760,
  [2761] = 2761,
  [2762] = 2762,
  [2763] = 2763,
  [2764] = 2764,
  [2765] = 2765,
  [2766] = 2766,
  [2767] = 2767,
  [2768] = 2768,
  [2769] = 2769,
  [2770] = 2107,
  [2771] = 2771,
  [2772] = 2113,
  [2773] = 2773,
  [2774] = 2774,
  [2775] = 2775,
  [2776] = 2776,
  [2777] = 2777,
  [2778] = 2778,
  [2779] = 2779,
  [2780] = 2780,
  [2781] = 2781,
  [2782] = 2782,
  [2783] = 2783,
  [2784] = 2784,
  [2785] = 2785,
  [2786] = 2140,
  [2787] = 2787,
  [2788] = 2788,
  [2789] = 2789,
  [2790] = 2790,
  [2791] = 2145,
  [2792] = 2099,
  [2793] = 2793,
  [2794] = 2102,
  [2795] = 2795,
  [2796] = 2796,
  [2797] = 2771,
  [2798] = 2798,
  [2799] = 2799,
  [2800] = 2800,
  [2801] = 2801,
  [2802] = 2802,
  [2803] = 2803,
  [2804] = 2804,
  [2805] = 2805,
  [2806] = 2780,
  [2807] = 2807,
  [2808] = 2808,
  [2809] = 2809,
  [2810] = 2810,
  [2811] = 2793,
  [2812] = 2812,
  [2813] = 2813,
  [2814] = 177,
  [2815] = 2815,
  [2816] = 2816,
  [2817] = 2817,
  [2818] = 2818,
  [2819] = 2819,
  [2820] = 2820,
  [2821] = 2821,
  [2822] = 2822,
  [2823] = 2823,
  [2824] = 2824,
  [2825] = 2825,
  [2826] = 2795,
  [2827] = 2827,
  [2828] = 2828,
  [2829] = 2829,
  [2830] = 2830,
  [2831] = 2831,
  [2832] = 2832,
  [2833] = 2833,
  [2834] = 2808,
  [2835] = 2835,
  [2836] = 2836,
  [2837] = 2796,
  [2838] = 2838,
  [2839] = 2839,
  [2840] = 2840,
  [2841] = 2841,
  [2842] = 2842,
  [2843] = 2843,
  [2844] = 2844,
  [2845] = 2845,
  [2846] = 2799,
  [2847] = 2847,
  [2848] = 2848,
  [2849] = 2849,
  [2850] = 2798,
  [2851] = 2851,
  [2852] = 2144,
  [2853] = 2851,
  [2854] = 2854,
  [2855] = 2100,
  [2856] = 2856,
  [2857] = 2776,
  [2858] = 2858,
  [2859] = 2859,
  [2860] = 2860,
  [2861] = 2847,
  [2862] = 2862,
  [2863] = 2863,
  [2864] = 2121,
  [2865] = 2842,
  [2866] = 2866,
  [2867] = 2867,
  [2868] = 2773,
  [2869] = 2869,
  [2870] = 2860,
  [2871] = 2871,
  [2872] = 2143,
  [2873] = 2128,
  [2874] = 2762,
  [2875] = 2766,
  [2876] = 2120,
  [2877] = 2106,
  [2878] = 2858,
  [2879] = 2867,
  [2880] = 2880,
  [2881] = 2881,
  [2882] = 2129,
  [2883] = 2883,
  [2884] = 2137,
  [2885] = 2115,
  [2886] = 2142,
  [2887] = 2887,
  [2888] = 2810,
  [2889] = 2822,
  [2890] = 2833,
  [2891] = 2891,
  [2892] = 2892,
  [2893] = 2893,
  [2894] = 2894,
  [2895] = 2895,
  [2896] = 2896,
  [2897] = 2869,
  [2898] = 2898,
  [2899] = 2899,
  [2900] = 2784,
  [2901] = 2777,
  [2902] = 2902,
  [2903] = 2789,
  [2904] = 2904,
  [2905] = 2758,
  [2906] = 2779,
  [2907] = 2907,
  [2908] = 2844,
  [2909] = 2909,
  [2910] = 2910,
  [2911] = 2911,
  [2912] = 2904,
  [2913] = 2913,
  [2914] = 2914,
  [2915] = 2131,
  [2916] = 2916,
  [2917] = 2813,
  [2918] = 397,
  [2919] = 2098,
  [2920] = 2779,
  [2921] = 2899,
  [2922] = 2782,
  [2923] = 2923,
  [2924] = 2854,
  [2925] = 2823,
  [2926] = 2824,
  [2927] = 2835,
  [2928] = 2836,
  [2929] = 2929,
  [2930] = 2930,
  [2931] = 2863,
  [2932] = 2869,
  [2933] = 2933,
  [2934] = 2801,
  [2935] = 2134,
  [2936] = 2119,
  [2937] = 2937,
  [2938] = 2789,
  [2939] = 2914,
  [2940] = 2892,
  [2941] = 2941,
  [2942] = 2122,
  [2943] = 2775,
  [2944] = 2944,
  [2945] = 2945,
  [2946] = 2914,
  [2947] = 2843,
  [2948] = 2893,
  [2949] = 2802,
  [2950] = 2950,
  [2951] = 2880,
  [2952] = 2789,
  [2953] = 2953,
  [2954] = 2954,
  [2955] = 2894,
  [2956] = 2774,
  [2957] = 2895,
  [2958] = 2958,
  [2959] = 2789,
  [2960] = 2960,
  [2961] = 2821,
  [2962] = 2962,
  [2963] = 2896,
  [2964] = 2964,
  [2965] = 2760,
  [2966] = 2911,
  [2967] = 2967,
  [2968] = 2968,
  [2969] = 2787,
  [2970] = 2845,
  [2971] = 2803,
  [2972] = 2126,
  [2973] = 2823,
  [2974] = 2774,
  [2975] = 2104,
  [2976] = 2976,
  [2977] = 2958,
  [2978] = 2114,
  [2979] = 2979,
  [2980] = 2980,
  [2981] = 2976,
  [2982] = 2790,
  [2983] = 2824,
  [2984] = 2108,
  [2985] = 2147,
  [2986] = 2135,
  [2987] = 2825,
  [2988] = 2105,
  [2989] = 2989,
  [2990] = 2990,
  [2991] = 2991,
  [2992] = 2992,
  [2993] = 2827,
  [2994] = 2994,
  [2995] = 2828,
  [2996] = 2996,
  [2997] = 2997,
  [2998] = 2830,
  [2999] = 2782,
  [3000] = 3000,
  [3001] = 3001,
  [3002] = 3002,
  [3003] = 3003,
  [3004] = 3004,
  [3005] = 2101,
  [3006] = 2124,
  [3007] = 3007,
  [3008] = 2103,
  [3009] = 2835,
  [3010] = 2960,
  [3011] = 2116,
  [3012] = 2836,
  [3013] = 3013,
  [3014] = 3014,
  [3015] = 3015,
  [3016] = 3016,
  [3017] = 3017,
  [3018] = 3018,
  [3019] = 2866,
  [3020] = 3020,
  [3021] = 2096,
  [3022] = 2848,
  [3023] = 3023,
  [3024] = 2881,
  [3025] = 2871,
  [3026] = 2904,
  [3027] = 2849,
  [3028] = 3028,
  [3029] = 3029,
  [3030] = 2838,
  [3031] = 2839,
  [3032] = 2914,
  [3033] = 2863,
  [3034] = 3034,
  [3035] = 3035,
  [3036] = 2805,
  [3037] = 3037,
  [3038] = 3038,
  [3039] = 3039,
  [3040] = 2953,
  [3041] = 2991,
  [3042] = 2937,
  [3043] = 3043,
  [3044] = 3044,
  [3045] = 2761,
  [3046] = 2967,
  [3047] = 3047,
  [3048] = 2841,
  [3049] = 2891,
  [3050] = 2764,
  [3051] = 3051,
  [3052] = 2759,
  [3053] = 2831,
  [3054] = 2898,
  [3055] = 3016,
  [3056] = 3056,
  [3057] = 3057,
  [3058] = 3058,
  [3059] = 3059,
  [3060] = 3060,
  [3061] = 3061,
  [3062] = 3062,
  [3063] = 3063,
  [3064] = 3064,
  [3065] = 3065,
  [3066] = 3066,
  [3067] = 3067,
  [3068] = 3068,
  [3069] = 3069,
  [3070] = 3070,
  [3071] = 3071,
  [3072] = 3072,
  [3073] = 3073,
  [3074] = 3074,
  [3075] = 3075,
  [3076] = 3076,
  [3077] = 3077,
  [3078] = 3078,
  [3079] = 3079,
  [3080] = 3080,
  [3081] = 3079,
  [3082] = 3082,
  [3083] = 3082,
  [3084] = 3066,
  [3085] = 3085,
  [3086] = 3067,
  [3087] = 3073,
  [3088] = 3088,
  [3089] = 3089,
  [3090] = 3066,
  [3091] = 3091,
  [3092] = 3092,
  [3093] = 3093,
  [3094] = 3094,
  [3095] = 3095,
  [3096] = 3096,
  [3097] = 3097,
  [3098] = 3098,
  [3099] = 3099,
  [3100] = 3100,
  [3101] = 3064,
  [3102] = 3102,
  [3103] = 3103,
  [3104] = 3088,
  [3105] = 3105,
  [3106] = 3106,
  [3107] = 3107,
  [3108] = 3108,
  [3109] = 3066,
  [3110] = 3110,
  [3111] = 3066,
  [3112] = 3069,
  [3113] = 3113,
  [3114] = 3114,
  [3115] = 3115,
  [3116] = 3116,
  [3117] = 1437,
  [3118] = 3118,
  [3119] = 3119,
  [3120] = 3120,
  [3121] = 3121,
  [3122] = 3088,
  [3123] = 3123,
  [3124] = 3124,
  [3125] = 3066,
  [3126] = 3126,
  [3127] = 3070,
  [3128] = 3093,
  [3129] = 1463,
  [3130] = 3070,
  [3131] = 3108,
  [3132] = 3075,
  [3133] = 3076,
  [3134] = 3134,
  [3135] = 3135,
  [3136] = 3082,
  [3137] = 3137,
  [3138] = 3067,
  [3139] = 3066,
  [3140] = 3140,
  [3141] = 3092,
  [3142] = 3093,
  [3143] = 3080,
  [3144] = 3144,
  [3145] = 3145,
  [3146] = 3146,
  [3147] = 3062,
  [3148] = 3068,
  [3149] = 3149,
  [3150] = 3150,
  [3151] = 3070,
  [3152] = 3152,
  [3153] = 3070,
  [3154] = 3154,
  [3155] = 3076,
  [3156] = 3082,
  [3157] = 3152,
  [3158] = 3158,
  [3159] = 3066,
  [3160] = 3160,
  [3161] = 3092,
  [3162] = 3093,
  [3163] = 3119,
  [3164] = 3063,
  [3165] = 3165,
  [3166] = 3166,
  [3167] = 3167,
  [3168] = 3102,
  [3169] = 3169,
  [3170] = 3076,
  [3171] = 3171,
  [3172] = 3066,
  [3173] = 3092,
  [3174] = 3093,
  [3175] = 3106,
  [3176] = 3176,
  [3177] = 3177,
  [3178] = 3178,
  [3179] = 3089,
  [3180] = 3091,
  [3181] = 3074,
  [3182] = 3076,
  [3183] = 3066,
  [3184] = 3075,
  [3185] = 3092,
  [3186] = 3093,
  [3187] = 3187,
  [3188] = 3065,
  [3189] = 3189,
  [3190] = 3076,
  [3191] = 3066,
  [3192] = 3076,
  [3193] = 3092,
  [3194] = 3093,
  [3195] = 3095,
  [3196] = 3076,
  [3197] = 3118,
  [3198] = 3092,
  [3199] = 3093,
  [3200] = 3160,
  [3201] = 3076,
  [3202] = 3099,
  [3203] = 3092,
  [3204] = 3093,
  [3205] = 3205,
  [3206] = 3076,
  [3207] = 3092,
  [3208] = 3093,
  [3209] = 3076,
  [3210] = 3080,
  [3211] = 3092,
  [3212] = 3093,
  [3213] = 3134,
  [3214] = 3214,
  [3215] = 3092,
  [3216] = 3060,
  [3217] = 3176,
  [3218] = 3218,
  [3219] = 3219,
  [3220] = 3220,
  [3221] = 3221,
  [3222] = 3160,
  [3223] = 3097,
  [3224] = 3121,
  [3225] = 3205,
  [3226] = 2883,
  [3227] = 3227,
  [3228] = 2887,
  [3229] = 3229,
  [3230] = 3230,
  [3231] = 3231,
  [3232] = 3178,
  [3233] = 3233,
  [3234] = 3124,
  [3235] = 3235,
  [3236] = 3082,
  [3237] = 3074,
  [3238] = 3059,
  [3239] = 3239,
  [3240] = 3240,
  [3241] = 3241,
  [3242] = 3242,
  [3243] = 3145,
  [3244] = 3100,
  [3245] = 3245,
  [3246] = 3246,
  [3247] = 3219,
  [3248] = 3248,
  [3249] = 3085,
  [3250] = 3241,
  [3251] = 3067,
  [3252] = 3252,
  [3253] = 3073,
  [3254] = 3254,
  [3255] = 3255,
  [3256] = 3256,
  [3257] = 3231,
  [3258] = 3258,
  [3259] = 3252,
  [3260] = 1380,
  [3261] = 3220,
  [3262] = 1381,
  [3263] = 3263,
  [3264] = 3144,
  [3265] = 3146,
  [3266] = 3150,
  [3267] = 3154,
  [3268] = 3227,
  [3269] = 3269,
  [3270] = 3270,
  [3271] = 3271,
  [3272] = 3076,
  [3273] = 3273,
  [3274] = 3069,
  [3275] = 3273,
  [3276] = 3169,
  [3277] = 3277,
  [3278] = 3240,
  [3279] = 3075,
  [3280] = 3066,
  [3281] = 3229,
  [3282] = 3282,
  [3283] = 3092,
  [3284] = 3284,
  [3285] = 3093,
  [3286] = 3286,
  [3287] = 3221,
  [3288] = 3121,
  [3289] = 3284,
  [3290] = 3290,
  [3291] = 3124,
  [3292] = 3077,
  [3293] = 3293,
  [3294] = 3290,
  [3295] = 3064,
  [3296] = 3296,
  [3297] = 3102,
  [3298] = 3068,
  [3299] = 3085,
  [3300] = 3245,
  [3301] = 3221,
  [3302] = 3098,
  [3303] = 3241,
  [3304] = 3221,
  [3305] = 3305,
  [3306] = 3286,
  [3307] = 3123,
  [3308] = 3120,
  [3309] = 3255,
  [3310] = 3310,
  [3311] = 3311,
  [3312] = 3305,
  [3313] = 3239,
  [3314] = 3277,
  [3315] = 3079,
  [3316] = 3107,
  [3317] = 3105,
  [3318] = 3218,
  [3319] = 3126,
  [3320] = 3320,
  [3321] = 3263,
  [3322] = 3061,
  [3323] = 3235,
  [3324] = 3320,
  [3325] = 1407,
  [3326] = 3114,
  [3327] = 3071,
  [3328] = 3110,
  [3329] = 3286,
  [3330] = 3120,
  [3331] = 3114,
  [3332] = 3120,
  [3333] = 3114,
  [3334] = 3334,
  [3335] = 3113,
  [3336] = 3189,
  [3337] = 3221,
  [3338] = 3057,
  [3339] = 3242,
  [3340] = 3160,
  [3341] = 3341,
  [3342] = 3342,
  [3343] = 3343,
  [3344] = 3344,
  [3345] = 3345,
  [3346] = 3346,
  [3347] = 3347,
  [3348] = 3348,
  [3349] = 3349,
  [3350] = 3350,
  [3351] = 3351,
  [3352] = 3352,
  [3353] = 3353,
  [3354] = 3354,
  [3355] = 3355,
  [3356] = 3356,
  [3357] = 3357,
  [3358] = 3358,
  [3359] = 3359,
  [3360] = 3360,
  [3361] = 3361,
  [3362] = 3362,
  [3363] = 3344,
  [3364] = 3364,
  [3365] = 3365,
  [3366] = 3366,
  [3367] = 3341,
  [3368] = 3368,
  [3369] = 3369,
  [3370] = 3360,
  [3371] = 3371,
  [3372] = 3372,
  [3373] = 3373,
  [3374] = 3374,
  [3375] = 3375,
  [3376] = 3376,
  [3377] = 3377,
  [3378] = 3349,
  [3379] = 3379,
  [3380] = 3380,
  [3381] = 3381,
  [3382] = 3362,
  [3383] = 3364,
  [3384] = 3377,
  [3385] = 3385,
  [3386] = 3341,
  [3387] = 3387,
  [3388] = 174,
  [3389] = 3389,
  [3390] = 3390,
  [3391] = 3391,
  [3392] = 3356,
  [3393] = 3393,
  [3394] = 3348,
  [3395] = 3395,
  [3396] = 3396,
  [3397] = 3397,
  [3398] = 3398,
  [3399] = 3399,
  [3400] = 3400,
  [3401] = 3365,
  [3402] = 3372,
  [3403] = 3403,
  [3404] = 3404,
  [3405] = 3359,
  [3406] = 3342,
  [3407] = 3407,
  [3408] = 3408,
  [3409] = 3409,
  [3410] = 3410,
  [3411] = 3346,
  [3412] = 3412,
  [3413] = 3371,
  [3414] = 3414,
  [3415] = 3415,
  [3416] = 3374,
  [3417] = 3417,
  [3418] = 3418,
  [3419] = 3419,
  [3420] = 3350,
  [3421] = 3421,
  [3422] = 3422,
  [3423] = 3379,
  [3424] = 3424,
  [3425] = 3360,
  [3426] = 3426,
  [3427] = 3427,
  [3428] = 3428,
  [3429] = 3381,
  [3430] = 3430,
  [3431] = 3431,
  [3432] = 3432,
  [3433] = 3433,
  [3434] = 3434,
  [3435] = 3435,
  [3436] = 3436,
  [3437] = 3437,
  [3438] = 3438,
  [3439] = 3439,
  [3440] = 3440,
  [3441] = 3422,
  [3442] = 3442,
  [3443] = 3397,
  [3444] = 3444,
  [3445] = 3445,
  [3446] = 3410,
  [3447] = 3437,
  [3448] = 3393,
  [3449] = 3393,
  [3450] = 3434,
  [3451] = 3451,
  [3452] = 3440,
  [3453] = 3360,
  [3454] = 3454,
  [3455] = 3439,
  [3456] = 3428,
  [3457] = 3457,
  [3458] = 3458,
  [3459] = 3459,
  [3460] = 3391,
  [3461] = 3461,
  [3462] = 3399,
  [3463] = 3463,
  [3464] = 3464,
  [3465] = 3389,
  [3466] = 3342,
  [3467] = 3467,
  [3468] = 3468,
  [3469] = 3428,
  [3470] = 3470,
  [3471] = 3426,
  [3472] = 3472,
  [3473] = 3430,
  [3474] = 3428,
  [3475] = 3475,
  [3476] = 3476,
  [3477] = 3414,
  [3478] = 3478,
  [3479] = 3479,
  [3480] = 3480,
  [3481] = 3343,
  [3482] = 3482,
  [3483] = 3385,
  [3484] = 172,
  [3485] = 3464,
  [3486] = 3486,
  [3487] = 3487,
  [3488] = 3368,
  [3489] = 3489,
  [3490] = 3490,
  [3491] = 3491,
  [3492] = 3442,
  [3493] = 3493,
  [3494] = 3351,
  [3495] = 3353,
  [3496] = 3496,
  [3497] = 3497,
  [3498] = 3498,
  [3499] = 3361,
  [3500] = 3500,
  [3501] = 3501,
  [3502] = 3502,
  [3503] = 3503,
  [3504] = 3380,
  [3505] = 3369,
  [3506] = 3342,
  [3507] = 3500,
  [3508] = 3379,
  [3509] = 3509,
  [3510] = 3347,
  [3511] = 3397,
  [3512] = 3493,
  [3513] = 3399,
  [3514] = 3400,
  [3515] = 3403,
  [3516] = 3372,
  [3517] = 3353,
  [3518] = 3518,
  [3519] = 3397,
  [3520] = 3400,
  [3521] = 3521,
  [3522] = 3522,
  [3523] = 3523,
  [3524] = 3461,
  [3525] = 3490,
  [3526] = 3428,
  [3527] = 3358,
  [3528] = 3417,
  [3529] = 3353,
  [3530] = 3374,
  [3531] = 3451,
  [3532] = 3361,
  [3533] = 3533,
  [3534] = 3374,
  [3535] = 3369,
  [3536] = 3536,
  [3537] = 3351,
  [3538] = 3399,
  [3539] = 3372,
  [3540] = 3540,
  [3541] = 3360,
  [3542] = 3408,
  [3543] = 3490,
  [3544] = 3361,
  [3545] = 3545,
  [3546] = 3369,
  [3547] = 3547,
  [3548] = 3372,
  [3549] = 3490,
  [3550] = 3361,
  [3551] = 3369,
  [3552] = 3372,
  [3553] = 3361,
  [3554] = 3369,
  [3555] = 3372,
  [3556] = 3361,
  [3557] = 3361,
  [3558] = 3361,
  [3559] = 3361,
  [3560] = 3361,
  [3561] = 3409,
  [3562] = 3562,
  [3563] = 3490,
  [3564] = 3564,
  [3565] = 3565,
  [3566] = 3463,
  [3567] = 3522,
  [3568] = 3568,
  [3569] = 3355,
  [3570] = 3373,
  [3571] = 3509,
  [3572] = 3354,
  [3573] = 3345,
  [3574] = 3574,
  [3575] = 3398,
  [3576] = 3498,
  [3577] = 3565,
  [3578] = 3418,
  [3579] = 3533,
  [3580] = 3376,
  [3581] = 3581,
  [3582] = 3489,
  [3583] = 3497,
  [3584] = 3501,
  [3585] = 3424,
  [3586] = 3418,
  [3587] = 3587,
  [3588] = 3432,
  [3589] = 3476,
  [3590] = 3431,
  [3591] = 3502,
  [3592] = 3445,
  [3593] = 3428,
  [3594] = 3439,
  [3595] = 3357,
  [3596] = 3596,
  [3597] = 3597,
  [3598] = 3598,
  [3599] = 3361,
  [3600] = 3600,
  [3601] = 3568,
  [3602] = 3602,
  [3603] = 3472,
  [3604] = 3472,
  [3605] = 3602,
  [3606] = 3598,
  [3607] = 3607,
  [3608] = 3565,
  [3609] = 3463,
  [3610] = 3568,
  [3611] = 3355,
  [3612] = 3612,
  [3613] = 3613,
  [3614] = 3565,
  [3615] = 3355,
  [3616] = 3616,
  [3617] = 3472,
  [3618] = 3355,
  [3619] = 3355,
  [3620] = 3355,
  [3621] = 3412,
  [3622] = 3607,
  [3623] = 3438,
  [3624] = 3486,
  [3625] = 3366,
  [3626] = 3489,
  [3627] = 3409,
  [3628] = 3375,
  [3629] = 3564,
  [3630] = 3490,
  [3631] = 3415,
  [3632] = 3419,
  [3633] = 3454,
  [3634] = 3418,
  [3635] = 3348,
  [3636] = 3547,
  [3637] = 3467,
  [3638] = 3638,
  [3639] = 3436,
  [3640] = 3521,
  [3641] = 3369,
  [3642] = 3642,
  [3643] = 3643,
  [3644] = 3644,
  [3645] = 3645,
  [3646] = 3646,
  [3647] = 3647,
  [3648] = 3648,
};

static TSCharacterRange sym_identifier_character_set_1[] = {
  {'A', 'Z'}, {'_', '_'}, {'a', 'z'}, {0xaa, 0xaa}, {0xb5, 0xb5}, {0xba, 0xba}, {0xc0, 0xd6}, {0xd8, 0xf6},
  {0xf8, 0x2c1}, {0x2c6, 0x2d1}, {0x2e0, 0x2e4}, {0x2ec, 0x2ec}, {0x2ee, 0x2ee}, {0x370, 0x374}, {0x376, 0x377}, {0x37b, 0x37d},
  {0x37f, 0x37f}, {0x386, 0x386}, {0x388, 0x38a}, {0x38c, 0x38c}, {0x38e, 0x3a1}, {0x3a3, 0x3f5}, {0x3f7, 0x481}, {0x48a, 0x52f},
  {0x531, 0x556}, {0x559, 0x559}, {0x560, 0x588}, {0x5d0, 0x5ea}, {0x5ef, 0x5f2}, {0x620, 0x64a}, {0x66e, 0x66f}, {0x671, 0x6d3},
  {0x6d5, 0x6d5}, {0x6e5, 0x6e6}, {0x6ee, 0x6ef}, {0x6fa, 0x6fc}, {0x6ff, 0x6ff}, {0x710, 0x710}, {0x712, 0x72f}, {0x74d, 0x7a5},
  {0x7b1, 0x7b1}, {0x7ca, 0x7ea}, {0x7f4, 0x7f5}, {0x7fa, 0x7fa}, {0x800, 0x815}, {0x81a, 0x81a}, {0x824, 0x824}, {0x828, 0x828},
  {0x840, 0x858}, {0x860, 0x86a}, {0x870, 0x887}, {0x889, 0x88e}, {0x8a0, 0x8c9}, {0x904, 0x939}, {0x93d, 0x93d}, {0x950, 0x950},
  {0x958, 0x961}, {0x971, 0x980}, {0x985, 0x98c}, {0x98f, 0x990}, {0x993, 0x9a8}, {0x9aa, 0x9b0}, {0x9b2, 0x9b2}, {0x9b6, 0x9b9},
  {0x9bd, 0x9bd}, {0x9ce, 0x9ce}, {0x9dc, 0x9dd}, {0x9df, 0x9e1}, {0x9f0, 0x9f1}, {0x9fc, 0x9fc}, {0xa05, 0xa0a}, {0xa0f, 0xa10},
  {0xa13, 0xa28}, {0xa2a, 0xa30}, {0xa32, 0xa33}, {0xa35, 0xa36}, {0xa38, 0xa39}, {0xa59, 0xa5c}, {0xa5e, 0xa5e}, {0xa72, 0xa74},
  {0xa85, 0xa8d}, {0xa8f, 0xa91}, {0xa93, 0xaa8}, {0xaaa, 0xab0}, {0xab2, 0xab3}, {0xab5, 0xab9}, {0xabd, 0xabd}, {0xad0, 0xad0},
  {0xae0, 0xae1}, {0xaf9, 0xaf9}, {0xb05, 0xb0c}, {0xb0f, 0xb10}, {0xb13, 0xb28}, {0xb2a, 0xb30}, {0xb32, 0xb33}, {0xb35, 0xb39},
  {0xb3d, 0xb3d}, {0xb5c, 0xb5d}, {0xb5f, 0xb61}, {0xb71, 0xb71}, {0xb83, 0xb83}, {0xb85, 0xb8a}, {0xb8e, 0xb90}, {0xb92, 0xb95},
  {0xb99, 0xb9a}, {0xb9c, 0xb9c}, {0xb9e, 0xb9f}, {0xba3, 0xba4}, {0xba8, 0xbaa}, {0xbae, 0xbb9}, {0xbd0, 0xbd0}, {0xc05, 0xc0c},
  {0xc0e, 0xc10}, {0xc12, 0xc28}, {0xc2a, 0xc39}, {0xc3d, 0xc3d}, {0xc58, 0xc5a}, {0xc5d, 0xc5d}, {0xc60, 0xc61}, {0xc80, 0xc80},
  {0xc85, 0xc8c}, {0xc8e, 0xc90}, {0xc92, 0xca8}, {0xcaa, 0xcb3}, {0xcb5, 0xcb9}, {0xcbd, 0xcbd}, {0xcdd, 0xcde}, {0xce0, 0xce1},
  {0xcf1, 0xcf2}, {0xd04, 0xd0c}, {0xd0e, 0xd10}, {0xd12, 0xd3a}, {0xd3d, 0xd3d}, {0xd4e, 0xd4e}, {0xd54, 0xd56}, {0xd5f, 0xd61},
  {0xd7a, 0xd7f}, {0xd85, 0xd96}, {0xd9a, 0xdb1}, {0xdb3, 0xdbb}, {0xdbd, 0xdbd}, {0xdc0, 0xdc6}, {0xe01, 0xe30}, {0xe32, 0xe32},
  {0xe40, 0xe46}, {0xe81, 0xe82}, {0xe84, 0xe84}, {0xe86, 0xe8a}, {0xe8c, 0xea3}, {0xea5, 0xea5}, {0xea7, 0xeb0}, {0xeb2, 0xeb2},
  {0xebd, 0xebd}, {0xec0, 0xec4}, {0xec6, 0xec6}, {0xedc, 0xedf}, {0xf00, 0xf00}, {0xf40, 0xf47}, {0xf49, 0xf6c}, {0xf88, 0xf8c},
  {0x1000, 0x102a}, {0x103f, 0x103f}, {0x1050, 0x1055}, {0x105a, 0x105d}, {0x1061, 0x1061}, {0x1065, 0x1066}, {0x106e, 0x1070}, {0x1075, 0x1081},
  {0x108e, 0x108e}, {0x10a0, 0x10c5}, {0x10c7, 0x10c7}, {0x10cd, 0x10cd}, {0x10d0, 0x10fa}, {0x10fc, 0x1248}, {0x124a, 0x124d}, {0x1250, 0x1256},
  {0x1258, 0x1258}, {0x125a, 0x125d}, {0x1260, 0x1288}, {0x128a, 0x128d}, {0x1290, 0x12b0}, {0x12b2, 0x12b5}, {0x12b8, 0x12be}, {0x12c0, 0x12c0},
  {0x12c2, 0x12c5}, {0x12c8, 0x12d6}, {0x12d8, 0x1310}, {0x1312, 0x1315}, {0x1318, 0x135a}, {0x1380, 0x138f}, {0x13a0, 0x13f5}, {0x13f8, 0x13fd},
  {0x1401, 0x166c}, {0x166f, 0x167f}, {0x1681, 0x169a}, {0x16a0, 0x16ea}, {0x16ee, 0x16f8}, {0x1700, 0x1711}, {0x171f, 0x1731}, {0x1740, 0x1751},
  {0x1760, 0x176c}, {0x176e, 0x1770}, {0x1780, 0x17b3}, {0x17d7, 0x17d7}, {0x17dc, 0x17dc}, {0x1820, 0x1878}, {0x1880, 0x18a8}, {0x18aa, 0x18aa},
  {0x18b0, 0x18f5}, {0x1900, 0x191e}, {0x1950, 0x196d}, {0x1970, 0x1974}, {0x1980, 0x19ab}, {0x19b0, 0x19c9}, {0x1a00, 0x1a16}, {0x1a20, 0x1a54},
  {0x1aa7, 0x1aa7}, {0x1b05, 0x1b33}, {0x1b45, 0x1b4c}, {0x1b83, 0x1ba0}, {0x1bae, 0x1baf}, {0x1bba, 0x1be5}, {0x1c00, 0x1c23}, {0x1c4d, 0x1c4f},
  {0x1c5a, 0x1c7d}, {0x1c80, 0x1c88}, {0x1c90, 0x1cba}, {0x1cbd, 0x1cbf}, {0x1ce9, 0x1cec}, {0x1cee, 0x1cf3}, {0x1cf5, 0x1cf6}, {0x1cfa, 0x1cfa},
  {0x1d00, 0x1dbf}, {0x1e00, 0x1f15}, {0x1f18, 0x1f1d}, {0x1f20, 0x1f45}, {0x1f48, 0x1f4d}, {0x1f50, 0x1f57}, {0x1f59, 0x1f59}, {0x1f5b, 0x1f5b},
  {0x1f5d, 0x1f5d}, {0x1f5f, 0x1f7d}, {0x1f80, 0x1fb4}, {0x1fb6, 0x1fbc}, {0x1fbe, 0x1fbe}, {0x1fc2, 0x1fc4}, {0x1fc6, 0x1fcc}, {0x1fd0, 0x1fd3},
  {0x1fd6, 0x1fdb}, {0x1fe0, 0x1fec}, {0x1ff2, 0x1ff4}, {0x1ff6, 0x1ffc}, {0x2071, 0x2071}, {0x207f, 0x207f}, {0x2090, 0x209c}, {0x2102, 0x2102},
  {0x2107, 0x2107}, {0x210a, 0x2113}, {0x2115, 0x2115}, {0x2118, 0x211d}, {0x2124, 0x2124}, {0x2126, 0x2126}, {0x2128, 0x2128}, {0x212a, 0x2139},
  {0x213c, 0x213f}, {0x2145, 0x2149}, {0x214e, 0x214e}, {0x2160, 0x2188}, {0x2c00, 0x2ce4}, {0x2ceb, 0x2cee}, {0x2cf2, 0x2cf3}, {0x2d00, 0x2d25},
  {0x2d27, 0x2d27}, {0x2d2d, 0x2d2d}, {0x2d30, 0x2d67}, {0x2d6f, 0x2d6f}, {0x2d80, 0x2d96}, {0x2da0, 0x2da6}, {0x2da8, 0x2dae}, {0x2db0, 0x2db6},
  {0x2db8, 0x2dbe}, {0x2dc0, 0x2dc6}, {0x2dc8, 0x2dce}, {0x2dd0, 0x2dd6}, {0x2dd8, 0x2dde}, {0x3005, 0x3007}, {0x3021, 0x3029}, {0x3031, 0x3035},
  {0x3038, 0x303c}, {0x3041, 0x3096}, {0x309d, 0x309f}, {0x30a1, 0x30fa}, {0x30fc, 0x30ff}, {0x3105, 0x312f}, {0x3131, 0x318e}, {0x31a0, 0x31bf},
  {0x31f0, 0x31ff}, {0x3400, 0x4dbf}, {0x4e00, 0xa48c}, {0xa4d0, 0xa4fd}, {0xa500, 0xa60c}, {0xa610, 0xa61f}, {0xa62a, 0xa62b}, {0xa640, 0xa66e},
  {0xa67f, 0xa69d}, {0xa6a0, 0xa6ef}, {0xa717, 0xa71f}, {0xa722, 0xa788}, {0xa78b, 0xa7ca}, {0xa7d0, 0xa7d1}, {0xa7d3, 0xa7d3}, {0xa7d5, 0xa7d9},
  {0xa7f2, 0xa801}, {0xa803, 0xa805}, {0xa807, 0xa80a}, {0xa80c, 0xa822}, {0xa840, 0xa873}, {0xa882, 0xa8b3}, {0xa8f2, 0xa8f7}, {0xa8fb, 0xa8fb},
  {0xa8fd, 0xa8fe}, {0xa90a, 0xa925}, {0xa930, 0xa946}, {0xa960, 0xa97c}, {0xa984, 0xa9b2}, {0xa9cf, 0xa9cf}, {0xa9e0, 0xa9e4}, {0xa9e6, 0xa9ef},
  {0xa9fa, 0xa9fe}, {0xaa00, 0xaa28}, {0xaa40, 0xaa42}, {0xaa44, 0xaa4b}, {0xaa60, 0xaa76}, {0xaa7a, 0xaa7a}, {0xaa7e, 0xaaaf}, {0xaab1, 0xaab1},
  {0xaab5, 0xaab6}, {0xaab9, 0xaabd}, {0xaac0, 0xaac0}, {0xaac2, 0xaac2}, {0xaadb, 0xaadd}, {0xaae0, 0xaaea}, {0xaaf2, 0xaaf4}, {0xab01, 0xab06},
  {0xab09, 0xab0e}, {0xab11, 0xab16}, {0xab20, 0xab26}, {0xab28, 0xab2e}, {0xab30, 0xab5a}, {0xab5c, 0xab69}, {0xab70, 0xabe2}, {0xac00, 0xd7a3},
  {0xd7b0, 0xd7c6}, {0xd7cb, 0xd7fb}, {0xf900, 0xfa6d}, {0xfa70, 0xfad9}, {0xfb00, 0xfb06}, {0xfb13, 0xfb17}, {0xfb1d, 0xfb1d}, {0xfb1f, 0xfb28},
  {0xfb2a, 0xfb36}, {0xfb38, 0xfb3c}, {0xfb3e, 0xfb3e}, {0xfb40, 0xfb41}, {0xfb43, 0xfb44}, {0xfb46, 0xfbb1}, {0xfbd3, 0xfc5d}, {0xfc64, 0xfd3d},
  {0xfd50, 0xfd8f}, {0xfd92, 0xfdc7}, {0xfdf0, 0xfdf9}, {0xfe71, 0xfe71}, {0xfe73, 0xfe73}, {0xfe77, 0xfe77}, {0xfe79, 0xfe79}, {0xfe7b, 0xfe7b},
  {0xfe7d, 0xfe7d}, {0xfe7f, 0xfefc}, {0xff21, 0xff3a}, {0xff41, 0xff5a}, {0xff66, 0xff9d}, {0xffa0, 0xffbe}, {0xffc2, 0xffc7}, {0xffca, 0xffcf},
  {0xffd2, 0xffd7}, {0xffda, 0xffdc}, {0x10000, 0x1000b}, {0x1000d, 0x10026}, {0x10028, 0x1003a}, {0x1003c, 0x1003d}, {0x1003f, 0x1004d}, {0x10050, 0x1005d},
  {0x10080, 0x100fa}, {0x10140, 0x10174}, {0x10280, 0x1029c}, {0x102a0, 0x102d0}, {0x10300, 0x1031f}, {0x1032d, 0x1034a}, {0x10350, 0x10375}, {0x10380, 0x1039d},
  {0x103a0, 0x103c3}, {0x103c8, 0x103cf}, {0x103d1, 0x103d5}, {0x10400, 0x1049d}, {0x104b0, 0x104d3}, {0x104d8, 0x104fb}, {0x10500, 0x10527}, {0x10530, 0x10563},
  {0x10570, 0x1057a}, {0x1057c, 0x1058a}, {0x1058c, 0x10592}, {0x10594, 0x10595}, {0x10597, 0x105a1}, {0x105a3, 0x105b1}, {0x105b3, 0x105b9}, {0x105bb, 0x105bc},
  {0x10600, 0x10736}, {0x10740, 0x10755}, {0x10760, 0x10767}, {0x10780, 0x10785}, {0x10787, 0x107b0}, {0x107b2, 0x107ba}, {0x10800, 0x10805}, {0x10808, 0x10808},
  {0x1080a, 0x10835}, {0x10837, 0x10838}, {0x1083c, 0x1083c}, {0x1083f, 0x10855}, {0x10860, 0x10876}, {0x10880, 0x1089e}, {0x108e0, 0x108f2}, {0x108f4, 0x108f5},
  {0x10900, 0x10915}, {0x10920, 0x10939}, {0x10980, 0x109b7}, {0x109be, 0x109bf}, {0x10a00, 0x10a00}, {0x10a10, 0x10a13}, {0x10a15, 0x10a17}, {0x10a19, 0x10a35},
  {0x10a60, 0x10a7c}, {0x10a80, 0x10a9c}, {0x10ac0, 0x10ac7}, {0x10ac9, 0x10ae4}, {0x10b00, 0x10b35}, {0x10b40, 0x10b55}, {0x10b60, 0x10b72}, {0x10b80, 0x10b91},
  {0x10c00, 0x10c48}, {0x10c80, 0x10cb2}, {0x10cc0, 0x10cf2}, {0x10d00, 0x10d23}, {0x10e80, 0x10ea9}, {0x10eb0, 0x10eb1}, {0x10f00, 0x10f1c}, {0x10f27, 0x10f27},
  {0x10f30, 0x10f45}, {0x10f70, 0x10f81}, {0x10fb0, 0x10fc4}, {0x10fe0, 0x10ff6}, {0x11003, 0x11037}, {0x11071, 0x11072}, {0x11075, 0x11075}, {0x11083, 0x110af},
  {0x110d0, 0x110e8}, {0x11103, 0x11126}, {0x11144, 0x11144}, {0x11147, 0x11147}, {0x11150, 0x11172}, {0x11176, 0x11176}, {0x11183, 0x111b2}, {0x111c1, 0x111c4},
  {0x111da, 0x111da}, {0x111dc, 0x111dc}, {0x11200, 0x11211}, {0x11213, 0x1122b}, {0x1123f, 0x11240}, {0x11280, 0x11286}, {0x11288, 0x11288}, {0x1128a, 0x1128d},
  {0x1128f, 0x1129d}, {0x1129f, 0x112a8}, {0x112b0, 0x112de}, {0x11305, 0x1130c}, {0x1130f, 0x11310}, {0x11313, 0x11328}, {0x1132a, 0x11330}, {0x11332, 0x11333},
  {0x11335, 0x11339}, {0x1133d, 0x1133d}, {0x11350, 0x11350}, {0x1135d, 0x11361}, {0x11400, 0x11434}, {0x11447, 0x1144a}, {0x1145f, 0x11461}, {0x11480, 0x114af},
  {0x114c4, 0x114c5}, {0x114c7, 0x114c7}, {0x11580, 0x115ae}, {0x115d8, 0x115db}, {0x11600, 0x1162f}, {0x11644, 0x11644}, {0x11680, 0x116aa}, {0x116b8, 0x116b8},
  {0x11700, 0x1171a}, {0x11740, 0x11746}, {0x11800, 0x1182b}, {0x118a0, 0x118df}, {0x118ff, 0x11906}, {0x11909, 0x11909}, {0x1190c, 0x11913}, {0x11915, 0x11916},
  {0x11918, 0x1192f}, {0x1193f, 0x1193f}, {0x11941, 0x11941}, {0x119a0, 0x119a7}, {0x119aa, 0x119d0}, {0x119e1, 0x119e1}, {0x119e3, 0x119e3}, {0x11a00, 0x11a00},
  {0x11a0b, 0x11a32}, {0x11a3a, 0x11a3a}, {0x11a50, 0x11a50}, {0x11a5c, 0x11a89}, {0x11a9d, 0x11a9d}, {0x11ab0, 0x11af8}, {0x11c00, 0x11c08}, {0x11c0a, 0x11c2e},
  {0x11c40, 0x11c40}, {0x11c72, 0x11c8f}, {0x11d00, 0x11d06}, {0x11d08, 0x11d09}, {0x11d0b, 0x11d30}, {0x11d46, 0x11d46}, {0x11d60, 0x11d65}, {0x11d67, 0x11d68},
  {0x11d6a, 0x11d89}, {0x11d98, 0x11d98}, {0x11ee0, 0x11ef2}, {0x11f02, 0x11f02}, {0x11f04, 0x11f10}, {0x11f12, 0x11f33}, {0x11fb0, 0x11fb0}, {0x12000, 0x12399},
  {0x12400, 0x1246e}, {0x12480, 0x12543}, {0x12f90, 0x12ff0}, {0x13000, 0x1342f}, {0x13441, 0x13446}, {0x14400, 0x14646}, {0x16800, 0x16a38}, {0x16a40, 0x16a5e},
  {0x16a70, 0x16abe}, {0x16ad0, 0x16aed}, {0x16b00, 0x16b2f}, {0x16b40, 0x16b43}, {0x16b63, 0x16b77}, {0x16b7d, 0x16b8f}, {0x16e40, 0x16e7f}, {0x16f00, 0x16f4a},
  {0x16f50, 0x16f50}, {0x16f93, 0x16f9f}, {0x16fe0, 0x16fe1}, {0x16fe3, 0x16fe3}, {0x17000, 0x187f7}, {0x18800, 0x18cd5}, {0x18d00, 0x18d08}, {0x1aff0, 0x1aff3},
  {0x1aff5, 0x1affb}, {0x1affd, 0x1affe}, {0x1b000, 0x1b122}, {0x1b132, 0x1b132}, {0x1b150, 0x1b152}, {0x1b155, 0x1b155}, {0x1b164, 0x1b167}, {0x1b170, 0x1b2fb},
  {0x1bc00, 0x1bc6a}, {0x1bc70, 0x1bc7c}, {0x1bc80, 0x1bc88}, {0x1bc90, 0x1bc99}, {0x1d400, 0x1d454}, {0x1d456, 0x1d49c}, {0x1d49e, 0x1d49f}, {0x1d4a2, 0x1d4a2},
  {0x1d4a5, 0x1d4a6}, {0x1d4a9, 0x1d4ac}, {0x1d4ae, 0x1d4b9}, {0x1d4bb, 0x1d4bb}, {0x1d4bd, 0x1d4c3}, {0x1d4c5, 0x1d505}, {0x1d507, 0x1d50a}, {0x1d50d, 0x1d514},
  {0x1d516, 0x1d51c}, {0x1d51e, 0x1d539}, {0x1d53b, 0x1d53e}, {0x1d540, 0x1d544}, {0x1d546, 0x1d546}, {0x1d54a, 0x1d550}, {0x1d552, 0x1d6a5}, {0x1d6a8, 0x1d6c0},
  {0x1d6c2, 0x1d6da}, {0x1d6dc, 0x1d6fa}, {0x1d6fc, 0x1d714}, {0x1d716, 0x1d734}, {0x1d736, 0x1d74e}, {0x1d750, 0x1d76e}, {0x1d770, 0x1d788}, {0x1d78a, 0x1d7a8},
  {0x1d7aa, 0x1d7c2}, {0x1d7c4, 0x1d7cb}, {0x1df00, 0x1df1e}, {0x1df25, 0x1df2a}, {0x1e030, 0x1e06d}, {0x1e100, 0x1e12c}, {0x1e137, 0x1e13d}, {0x1e14e, 0x1e14e},
  {0x1e290, 0x1e2ad}, {0x1e2c0, 0x1e2eb}, {0x1e4d0, 0x1e4eb}, {0x1e7e0, 0x1e7e6}, {0x1e7e8, 0x1e7eb}, {0x1e7ed, 0x1e7ee}, {0x1e7f0, 0x1e7fe}, {0x1e800, 0x1e8c4},
  {0x1e900, 0x1e943}, {0x1e94b, 0x1e94b}, {0x1ee00, 0x1ee03}, {0x1ee05, 0x1ee1f}, {0x1ee21, 0x1ee22}, {0x1ee24, 0x1ee24}, {0x1ee27, 0x1ee27}, {0x1ee29, 0x1ee32},
  {0x1ee34, 0x1ee37}, {0x1ee39, 0x1ee39}, {0x1ee3b, 0x1ee3b}, {0x1ee42, 0x1ee42}, {0x1ee47, 0x1ee47}, {0x1ee49, 0x1ee49}, {0x1ee4b, 0x1ee4b}, {0x1ee4d, 0x1ee4f},
  {0x1ee51, 0x1ee52}, {0x1ee54, 0x1ee54}, {0x1ee57, 0x1ee57}, {0x1ee59, 0x1ee59}, {0x1ee5b, 0x1ee5b}, {0x1ee5d, 0x1ee5d}, {0x1ee5f, 0x1ee5f}, {0x1ee61, 0x1ee62},
  {0x1ee64, 0x1ee64}, {0x1ee67, 0x1ee6a}, {0x1ee6c, 0x1ee72}, {0x1ee74, 0x1ee77}, {0x1ee79, 0x1ee7c}, {0x1ee7e, 0x1ee7e}, {0x1ee80, 0x1ee89}, {0x1ee8b, 0x1ee9b},
  {0x1eea1, 0x1eea3}, {0x1eea5, 0x1eea9}, {0x1eeab, 0x1eebb}, {0x20000, 0x2a6df}, {0x2a700, 0x2b739}, {0x2b740, 0x2b81d}, {0x2b820, 0x2cea1}, {0x2ceb0, 0x2ebe0},
  {0x2ebf0, 0x2ee5d}, {0x2f800, 0x2fa1d}, {0x30000, 0x3134a}, {0x31350, 0x323af},
};

static TSCharacterRange sym_identifier_character_set_3[] = {
  {'0', '9'}, {'A', 'Z'}, {'_', '_'}, {'a', 'z'}, {0xaa, 0xaa}, {0xb5, 0xb5}, {0xb7, 0xb7}, {0xba, 0xba},
  {0xc0, 0xd6}, {0xd8, 0xf6}, {0xf8, 0x2c1}, {0x2c6, 0x2d1}, {0x2e0, 0x2e4}, {0x2ec, 0x2ec}, {0x2ee, 0x2ee}, {0x300, 0x374},
  {0x376, 0x377}, {0x37b, 0x37d}, {0x37f, 0x37f}, {0x386, 0x38a}, {0x38c, 0x38c}, {0x38e, 0x3a1}, {0x3a3, 0x3f5}, {0x3f7, 0x481},
  {0x483, 0x487}, {0x48a, 0x52f}, {0x531, 0x556}, {0x559, 0x559}, {0x560, 0x588}, {0x591, 0x5bd}, {0x5bf, 0x5bf}, {0x5c1, 0x5c2},
  {0x5c4, 0x5c5}, {0x5c7, 0x5c7}, {0x5d0, 0x5ea}, {0x5ef, 0x5f2}, {0x610, 0x61a}, {0x620, 0x669}, {0x66e, 0x6d3}, {0x6d5, 0x6dc},
  {0x6df, 0x6e8}, {0x6ea, 0x6fc}, {0x6ff, 0x6ff}, {0x710, 0x74a}, {0x74d, 0x7b1}, {0x7c0, 0x7f5}, {0x7fa, 0x7fa}, {0x7fd, 0x7fd},
  {0x800, 0x82d}, {0x840, 0x85b}, {0x860, 0x86a}, {0x870, 0x887}, {0x889, 0x88e}, {0x898, 0x8e1}, {0x8e3, 0x963}, {0x966, 0x96f},
  {0x971, 0x983}, {0x985, 0x98c}, {0x98f, 0x990}, {0x993, 0x9a8}, {0x9aa, 0x9b0}, {0x9b2, 0x9b2}, {0x9b6, 0x9b9}, {0x9bc, 0x9c4},
  {0x9c7, 0x9c8}, {0x9cb, 0x9ce}, {0x9d7, 0x9d7}, {0x9dc, 0x9dd}, {0x9df, 0x9e3}, {0x9e6, 0x9f1}, {0x9fc, 0x9fc}, {0x9fe, 0x9fe},
  {0xa01, 0xa03}, {0xa05, 0xa0a}, {0xa0f, 0xa10}, {0xa13, 0xa28}, {0xa2a, 0xa30}, {0xa32, 0xa33}, {0xa35, 0xa36}, {0xa38, 0xa39},
  {0xa3c, 0xa3c}, {0xa3e, 0xa42}, {0xa47, 0xa48}, {0xa4b, 0xa4d}, {0xa51, 0xa51}, {0xa59, 0xa5c}, {0xa5e, 0xa5e}, {0xa66, 0xa75},
  {0xa81, 0xa83}, {0xa85, 0xa8d}, {0xa8f, 0xa91}, {0xa93, 0xaa8}, {0xaaa, 0xab0}, {0xab2, 0xab3}, {0xab5, 0xab9}, {0xabc, 0xac5},
  {0xac7, 0xac9}, {0xacb, 0xacd}, {0xad0, 0xad0}, {0xae0, 0xae3}, {0xae6, 0xaef}, {0xaf9, 0xaff}, {0xb01, 0xb03}, {0xb05, 0xb0c},
  {0xb0f, 0xb10}, {0xb13, 0xb28}, {0xb2a, 0xb30}, {0xb32, 0xb33}, {0xb35, 0xb39}, {0xb3c, 0xb44}, {0xb47, 0xb48}, {0xb4b, 0xb4d},
  {0xb55, 0xb57}, {0xb5c, 0xb5d}, {0xb5f, 0xb63}, {0xb66, 0xb6f}, {0xb71, 0xb71}, {0xb82, 0xb83}, {0xb85, 0xb8a}, {0xb8e, 0xb90},
  {0xb92, 0xb95}, {0xb99, 0xb9a}, {0xb9c, 0xb9c}, {0xb9e, 0xb9f}, {0xba3, 0xba4}, {0xba8, 0xbaa}, {0xbae, 0xbb9}, {0xbbe, 0xbc2},
  {0xbc6, 0xbc8}, {0xbca, 0xbcd}, {0xbd0, 0xbd0}, {0xbd7, 0xbd7}, {0xbe6, 0xbef}, {0xc00, 0xc0c}, {0xc0e, 0xc10}, {0xc12, 0xc28},
  {0xc2a, 0xc39}, {0xc3c, 0xc44}, {0xc46, 0xc48}, {0xc4a, 0xc4d}, {0xc55, 0xc56}, {0xc58, 0xc5a}, {0xc5d, 0xc5d}, {0xc60, 0xc63},
  {0xc66, 0xc6f}, {0xc80, 0xc83}, {0xc85, 0xc8c}, {0xc8e, 0xc90}, {0xc92, 0xca8}, {0xcaa, 0xcb3}, {0xcb5, 0xcb9}, {0xcbc, 0xcc4},
  {0xcc6, 0xcc8}, {0xcca, 0xccd}, {0xcd5, 0xcd6}, {0xcdd, 0xcde}, {0xce0, 0xce3}, {0xce6, 0xcef}, {0xcf1, 0xcf3}, {0xd00, 0xd0c},
  {0xd0e, 0xd10}, {0xd12, 0xd44}, {0xd46, 0xd48}, {0xd4a, 0xd4e}, {0xd54, 0xd57}, {0xd5f, 0xd63}, {0xd66, 0xd6f}, {0xd7a, 0xd7f},
  {0xd81, 0xd83}, {0xd85, 0xd96}, {0xd9a, 0xdb1}, {0xdb3, 0xdbb}, {0xdbd, 0xdbd}, {0xdc0, 0xdc6}, {0xdca, 0xdca}, {0xdcf, 0xdd4},
  {0xdd6, 0xdd6}, {0xdd8, 0xddf}, {0xde6, 0xdef}, {0xdf2, 0xdf3}, {0xe01, 0xe3a}, {0xe40, 0xe4e}, {0xe50, 0xe59}, {0xe81, 0xe82},
  {0xe84, 0xe84}, {0xe86, 0xe8a}, {0xe8c, 0xea3}, {0xea5, 0xea5}, {0xea7, 0xebd}, {0xec0, 0xec4}, {0xec6, 0xec6}, {0xec8, 0xece},
  {0xed0, 0xed9}, {0xedc, 0xedf}, {0xf00, 0xf00}, {0xf18, 0xf19}, {0xf20, 0xf29}, {0xf35, 0xf35}, {0xf37, 0xf37}, {0xf39, 0xf39},
  {0xf3e, 0xf47}, {0xf49, 0xf6c}, {0xf71, 0xf84}, {0xf86, 0xf97}, {0xf99, 0xfbc}, {0xfc6, 0xfc6}, {0x1000, 0x1049}, {0x1050, 0x109d},
  {0x10a0, 0x10c5}, {0x10c7, 0x10c7}, {0x10cd, 0x10cd}, {0x10d0, 0x10fa}, {0x10fc, 0x1248}, {0x124a, 0x124d}, {0x1250, 0x1256}, {0x1258, 0x1258},
  {0x125a, 0x125d}, {0x1260, 0x1288}, {0x128a, 0x128d}, {0x1290, 0x12b0}, {0x12b2, 0x12b5}, {0x12b8, 0x12be}, {0x12c0, 0x12c0}, {0x12c2, 0x12c5},
  {0x12c8, 0x12d6}, {0x12d8, 0x1310}, {0x1312, 0x1315}, {0x1318, 0x135a}, {0x135d, 0x135f}, {0x1369, 0x1371}, {0x1380, 0x138f}, {0x13a0, 0x13f5},
  {0x13f8, 0x13fd}, {0x1401, 0x166c}, {0x166f, 0x167f}, {0x1681, 0x169a}, {0x16a0, 0x16ea}, {0x16ee, 0x16f8}, {0x1700, 0x1715}, {0x171f, 0x1734},
  {0x1740, 0x1753}, {0x1760, 0x176c}, {0x176e, 0x1770}, {0x1772, 0x1773}, {0x1780, 0x17d3}, {0x17d7, 0x17d7}, {0x17dc, 0x17dd}, {0x17e0, 0x17e9},
  {0x180b, 0x180d}, {0x180f, 0x1819}, {0x1820, 0x1878}, {0x1880, 0x18aa}, {0x18b0, 0x18f5}, {0x1900, 0x191e}, {0x1920, 0x192b}, {0x1930, 0x193b},
  {0x1946, 0x196d}, {0x1970, 0x1974}, {0x1980, 0x19ab}, {0x19b0, 0x19c9}, {0x19d0, 0x19da}, {0x1a00, 0x1a1b}, {0x1a20, 0x1a5e}, {0x1a60, 0x1a7c},
  {0x1a7f, 0x1a89}, {0x1a90, 0x1a99}, {0x1aa7, 0x1aa7}, {0x1ab0, 0x1abd}, {0x1abf, 0x1ace}, {0x1b00, 0x1b4c}, {0x1b50, 0x1b59}, {0x1b6b, 0x1b73},
  {0x1b80, 0x1bf3}, {0x1c00, 0x1c37}, {0x1c40, 0x1c49}, {0x1c4d, 0x1c7d}, {0x1c80, 0x1c88}, {0x1c90, 0x1cba}, {0x1cbd, 0x1cbf}, {0x1cd0, 0x1cd2},
  {0x1cd4, 0x1cfa}, {0x1d00, 0x1f15}, {0x1f18, 0x1f1d}, {0x1f20, 0x1f45}, {0x1f48, 0x1f4d}, {0x1f50, 0x1f57}, {0x1f59, 0x1f59}, {0x1f5b, 0x1f5b},
  {0x1f5d, 0x1f5d}, {0x1f5f, 0x1f7d}, {0x1f80, 0x1fb4}, {0x1fb6, 0x1fbc}, {0x1fbe, 0x1fbe}, {0x1fc2, 0x1fc4}, {0x1fc6, 0x1fcc}, {0x1fd0, 0x1fd3},
  {0x1fd6, 0x1fdb}, {0x1fe0, 0x1fec}, {0x1ff2, 0x1ff4}, {0x1ff6, 0x1ffc}, {0x200c, 0x200d}, {0x203f, 0x2040}, {0x2054, 0x2054}, {0x2071, 0x2071},
  {0x207f, 0x207f}, {0x2090, 0x209c}, {0x20d0, 0x20dc}, {0x20e1, 0x20e1}, {0x20e5, 0x20f0}, {0x2102, 0x2102}, {0x2107, 0x2107}, {0x210a, 0x2113},
  {0x2115, 0x2115}, {0x2118, 0x211d}, {0x2124, 0x2124}, {0x2126, 0x2126}, {0x2128, 0x2128}, {0x212a, 0x2139}, {0x213c, 0x213f}, {0x2145, 0x2149},
  {0x214e, 0x214e}, {0x2160, 0x2188}, {0x2c00, 0x2ce4}, {0x2ceb, 0x2cf3}, {0x2d00, 0x2d25}, {0x2d27, 0x2d27}, {0x2d2d, 0x2d2d}, {0x2d30, 0x2d67},
  {0x2d6f, 0x2d6f}, {0x2d7f, 0x2d96}, {0x2da0, 0x2da6}, {0x2da8, 0x2dae}, {0x2db0, 0x2db6}, {0x2db8, 0x2dbe}, {0x2dc0, 0x2dc6}, {0x2dc8, 0x2dce},
  {0x2dd0, 0x2dd6}, {0x2dd8, 0x2dde}, {0x2de0, 0x2dff}, {0x3005, 0x3007}, {0x3021, 0x302f}, {0x3031, 0x3035}, {0x3038, 0x303c}, {0x3041, 0x3096},
  {0x3099, 0x309a}, {0x309d, 0x309f}, {0x30a1, 0x30ff}, {0x3105, 0x312f}, {0x3131, 0x318e}, {0x31a0, 0x31bf}, {0x31f0, 0x31ff}, {0x3400, 0x4dbf},
  {0x4e00, 0xa48c}, {0xa4d0, 0xa4fd}, {0xa500, 0xa60c}, {0xa610, 0xa62b}, {0xa640, 0xa66f}, {0xa674, 0xa67d}, {0xa67f, 0xa6f1}, {0xa717, 0xa71f},
  {0xa722, 0xa788}, {0xa78b, 0xa7ca}, {0xa7d0, 0xa7d1}, {0xa7d3, 0xa7d3}, {0xa7d5, 0xa7d9}, {0xa7f2, 0xa827}, {0xa82c, 0xa82c}, {0xa840, 0xa873},
  {0xa880, 0xa8c5}, {0xa8d0, 0xa8d9}, {0xa8e0, 0xa8f7}, {0xa8fb, 0xa8fb}, {0xa8fd, 0xa92d}, {0xa930, 0xa953}, {0xa960, 0xa97c}, {0xa980, 0xa9c0},
  {0xa9cf, 0xa9d9}, {0xa9e0, 0xa9fe}, {0xaa00, 0xaa36}, {0xaa40, 0xaa4d}, {0xaa50, 0xaa59}, {0xaa60, 0xaa76}, {0xaa7a, 0xaac2}, {0xaadb, 0xaadd},
  {0xaae0, 0xaaef}, {0xaaf2, 0xaaf6}, {0xab01, 0xab06}, {0xab09, 0xab0e}, {0xab11, 0xab16}, {0xab20, 0xab26}, {0xab28, 0xab2e}, {0xab30, 0xab5a},
  {0xab5c, 0xab69}, {0xab70, 0xabea}, {0xabec, 0xabed}, {0xabf0, 0xabf9}, {0xac00, 0xd7a3}, {0xd7b0, 0xd7c6}, {0xd7cb, 0xd7fb}, {0xf900, 0xfa6d},
  {0xfa70, 0xfad9}, {0xfb00, 0xfb06}, {0xfb13, 0xfb17}, {0xfb1d, 0xfb28}, {0xfb2a, 0xfb36}, {0xfb38, 0xfb3c}, {0xfb3e, 0xfb3e}, {0xfb40, 0xfb41},
  {0xfb43, 0xfb44}, {0xfb46, 0xfbb1}, {0xfbd3, 0xfc5d}, {0xfc64, 0xfd3d}, {0xfd50, 0xfd8f}, {0xfd92, 0xfdc7}, {0xfdf0, 0xfdf9}, {0xfe00, 0xfe0f},
  {0xfe20, 0xfe2f}, {0xfe33, 0xfe34}, {0xfe4d, 0xfe4f}, {0xfe71, 0xfe71}, {0xfe73, 0xfe73}, {0xfe77, 0xfe77}, {0xfe79, 0xfe79}, {0xfe7b, 0xfe7b},
  {0xfe7d, 0xfe7d}, {0xfe7f, 0xfefc}, {0xff10, 0xff19}, {0xff21, 0xff3a}, {0xff3f, 0xff3f}, {0xff41, 0xff5a}, {0xff65, 0xffbe}, {0xffc2, 0xffc7},
  {0xffca, 0xffcf}, {0xffd2, 0xffd7}, {0xffda, 0xffdc}, {0x10000, 0x1000b}, {0x1000d, 0x10026}, {0x10028, 0x1003a}, {0x1003c, 0x1003d}, {0x1003f, 0x1004d},
  {0x10050, 0x1005d}, {0x10080, 0x100fa}, {0x10140, 0x10174}, {0x101fd, 0x101fd}, {0x10280, 0x1029c}, {0x102a0, 0x102d0}, {0x102e0, 0x102e0}, {0x10300, 0x1031f},
  {0x1032d, 0x1034a}, {0x10350, 0x1037a}, {0x10380, 0x1039d}, {0x103a0, 0x103c3}, {0x103c8, 0x103cf}, {0x103d1, 0x103d5}, {0x10400, 0x1049d}, {0x104a0, 0x104a9},
  {0x104b0, 0x104d3}, {0x104d8, 0x104fb}, {0x10500, 0x10527}, {0x10530, 0x10563}, {0x10570, 0x1057a}, {0x1057c, 0x1058a}, {0x1058c, 0x10592}, {0x10594, 0x10595},
  {0x10597, 0x105a1}, {0x105a3, 0x105b1}, {0x105b3, 0x105b9}, {0x105bb, 0x105bc}, {0x10600, 0x10736}, {0x10740, 0x10755}, {0x10760, 0x10767}, {0x10780, 0x10785},
  {0x10787, 0x107b0}, {0x107b2, 0x107ba}, {0x10800, 0x10805}, {0x10808, 0x10808}, {0x1080a, 0x10835}, {0x10837, 0x10838}, {0x1083c, 0x1083c}, {0x1083f, 0x10855},
  {0x10860, 0x10876}, {0x10880, 0x1089e}, {0x108e0, 0x108f2}, {0x108f4, 0x108f5}, {0x10900, 0x10915}, {0x10920, 0x10939}, {0x10980, 0x109b7}, {0x109be, 0x109bf},
  {0x10a00, 0x10a03}, {0x10a05, 0x10a06}, {0x10a0c, 0x10a13}, {0x10a15, 0x10a17}, {0x10a19, 0x10a35}, {0x10a38, 0x10a3a}, {0x10a3f, 0x10a3f}, {0x10a60, 0x10a7c},
  {0x10a80, 0x10a9c}, {0x10ac0, 0x10ac7}, {0x10ac9, 0x10ae6}, {0x10b00, 0x10b35}, {0x10b40, 0x10b55}, {0x10b60, 0x10b72}, {0x10b80, 0x10b91}, {0x10c00, 0x10c48},
  {0x10c80, 0x10cb2}, {0x10cc0, 0x10cf2}, {0x10d00, 0x10d27}, {0x10d30, 0x10d39}, {0x10e80, 0x10ea9}, {0x10eab, 0x10eac}, {0x10eb0, 0x10eb1}, {0x10efd, 0x10f1c},
  {0x10f27, 0x10f27}, {0x10f30, 0x10f50}, {0x10f70, 0x10f85}, {0x10fb0, 0x10fc4}, {0x10fe0, 0x10ff6}, {0x11000, 0x11046}, {0x11066, 0x11075}, {0x1107f, 0x110ba},
  {0x110c2, 0x110c2}, {0x110d0, 0x110e8}, {0x110f0, 0x110f9}, {0x11100, 0x11134}, {0x11136, 0x1113f}, {0x11144, 0x11147}, {0x11150, 0x11173}, {0x11176, 0x11176},
  {0x11180, 0x111c4}, {0x111c9, 0x111cc}, {0x111ce, 0x111da}, {0x111dc, 0x111dc}, {0x11200, 0x11211}, {0x11213, 0x11237}, {0x1123e, 0x11241}, {0x11280, 0x11286},
  {0x11288, 0x11288}, {0x1128a, 0x1128d}, {0x1128f, 0x1129d}, {0x1129f, 0x112a8}, {0x112b0, 0x112ea}, {0x112f0, 0x112f9}, {0x11300, 0x11303}, {0x11305, 0x1130c},
  {0x1130f, 0x11310}, {0x11313, 0x11328}, {0x1132a, 0x11330}, {0x11332, 0x11333}, {0x11335, 0x11339}, {0x1133b, 0x11344}, {0x11347, 0x11348}, {0x1134b, 0x1134d},
  {0x11350, 0x11350}, {0x11357, 0x11357}, {0x1135d, 0x11363}, {0x11366, 0x1136c}, {0x11370, 0x11374}, {0x11400, 0x1144a}, {0x11450, 0x11459}, {0x1145e, 0x11461},
  {0x11480, 0x114c5}, {0x114c7, 0x114c7}, {0x114d0, 0x114d9}, {0x11580, 0x115b5}, {0x115b8, 0x115c0}, {0x115d8, 0x115dd}, {0x11600, 0x11640}, {0x11644, 0x11644},
  {0x11650, 0x11659}, {0x11680, 0x116b8}, {0x116c0, 0x116c9}, {0x11700, 0x1171a}, {0x1171d, 0x1172b}, {0x11730, 0x11739}, {0x11740, 0x11746}, {0x11800, 0x1183a},
  {0x118a0, 0x118e9}, {0x118ff, 0x11906}, {0x11909, 0x11909}, {0x1190c, 0x11913}, {0x11915, 0x11916}, {0x11918, 0x11935}, {0x11937, 0x11938}, {0x1193b, 0x11943},
  {0x11950, 0x11959}, {0x119a0, 0x119a7}, {0x119aa, 0x119d7}, {0x119da, 0x119e1}, {0x119e3, 0x119e4}, {0x11a00, 0x11a3e}, {0x11a47, 0x11a47}, {0x11a50, 0x11a99},
  {0x11a9d, 0x11a9d}, {0x11ab0, 0x11af8}, {0x11c00, 0x11c08}, {0x11c0a, 0x11c36}, {0x11c38, 0x11c40}, {0x11c50, 0x11c59}, {0x11c72, 0x11c8f}, {0x11c92, 0x11ca7},
  {0x11ca9, 0x11cb6}, {0x11d00, 0x11d06}, {0x11d08, 0x11d09}, {0x11d0b, 0x11d36}, {0x11d3a, 0x11d3a}, {0x11d3c, 0x11d3d}, {0x11d3f, 0x11d47}, {0x11d50, 0x11d59},
  {0x11d60, 0x11d65}, {0x11d67, 0x11d68}, {0x11d6a, 0x11d8e}, {0x11d90, 0x11d91}, {0x11d93, 0x11d98}, {0x11da0, 0x11da9}, {0x11ee0, 0x11ef6}, {0x11f00, 0x11f10},
  {0x11f12, 0x11f3a}, {0x11f3e, 0x11f42}, {0x11f50, 0x11f59}, {0x11fb0, 0x11fb0}, {0x12000, 0x12399}, {0x12400, 0x1246e}, {0x12480, 0x12543}, {0x12f90, 0x12ff0},
  {0x13000, 0x1342f}, {0x13440, 0x13455}, {0x14400, 0x14646}, {0x16800, 0x16a38}, {0x16a40, 0x16a5e}, {0x16a60, 0x16a69}, {0x16a70, 0x16abe}, {0x16ac0, 0x16ac9},
  {0x16ad0, 0x16aed}, {0x16af0, 0x16af4}, {0x16b00, 0x16b36}, {0x16b40, 0x16b43}, {0x16b50, 0x16b59}, {0x16b63, 0x16b77}, {0x16b7d, 0x16b8f}, {0x16e40, 0x16e7f},
  {0x16f00, 0x16f4a}, {0x16f4f, 0x16f87}, {0x16f8f, 0x16f9f}, {0x16fe0, 0x16fe1}, {0x16fe3, 0x16fe4}, {0x16ff0, 0x16ff1}, {0x17000, 0x187f7}, {0x18800, 0x18cd5},
  {0x18d00, 0x18d08}, {0x1aff0, 0x1aff3}, {0x1aff5, 0x1affb}, {0x1affd, 0x1affe}, {0x1b000, 0x1b122}, {0x1b132, 0x1b132}, {0x1b150, 0x1b152}, {0x1b155, 0x1b155},
  {0x1b164, 0x1b167}, {0x1b170, 0x1b2fb}, {0x1bc00, 0x1bc6a}, {0x1bc70, 0x1bc7c}, {0x1bc80, 0x1bc88}, {0x1bc90, 0x1bc99}, {0x1bc9d, 0x1bc9e}, {0x1cf00, 0x1cf2d},
  {0x1cf30, 0x1cf46}, {0x1d165, 0x1d169}, {0x1d16d, 0x1d172}, {0x1d17b, 0x1d182}, {0x1d185, 0x1d18b}, {0x1d1aa, 0x1d1ad}, {0x1d242, 0x1d244}, {0x1d400, 0x1d454},
  {0x1d456, 0x1d49c}, {0x1d49e, 0x1d49f}, {0x1d4a2, 0x1d4a2}, {0x1d4a5, 0x1d4a6}, {0x1d4a9, 0x1d4ac}, {0x1d4ae, 0x1d4b9}, {0x1d4bb, 0x1d4bb}, {0x1d4bd, 0x1d4c3},
  {0x1d4c5, 0x1d505}, {0x1d507, 0x1d50a}, {0x1d50d, 0x1d514}, {0x1d516, 0x1d51c}, {0x1d51e, 0x1d539}, {0x1d53b, 0x1d53e}, {0x1d540, 0x1d544}, {0x1d546, 0x1d546},
  {0x1d54a, 0x1d550}, {0x1d552, 0x1d6a5}, {0x1d6a8, 0x1d6c0}, {0x1d6c2, 0x1d6da}, {0x1d6dc, 0x1d6fa}, {0x1d6fc, 0x1d714}, {0x1d716, 0x1d734}, {0x1d736, 0x1d74e},
  {0x1d750, 0x1d76e}, {0x1d770, 0x1d788}, {0x1d78a, 0x1d7a8}, {0x1d7aa, 0x1d7c2}, {0x1d7c4, 0x1d7cb}, {0x1d7ce, 0x1d7ff}, {0x1da00, 0x1da36}, {0x1da3b, 0x1da6c},
  {0x1da75, 0x1da75}, {0x1da84, 0x1da84}, {0x1da9b, 0x1da9f}, {0x1daa1, 0x1daaf}, {0x1df00, 0x1df1e}, {0x1df25, 0x1df2a}, {0x1e000, 0x1e006}, {0x1e008, 0x1e018},
  {0x1e01b, 0x1e021}, {0x1e023, 0x1e024}, {0x1e026, 0x1e02a}, {0x1e030, 0x1e06d}, {0x1e08f, 0x1e08f}, {0x1e100, 0x1e12c}, {0x1e130, 0x1e13d}, {0x1e140, 0x1e149},
  {0x1e14e, 0x1e14e}, {0x1e290, 0x1e2ae}, {0x1e2c0, 0x1e2f9}, {0x1e4d0, 0x1e4f9}, {0x1e7e0, 0x1e7e6}, {0x1e7e8, 0x1e7eb}, {0x1e7ed, 0x1e7ee}, {0x1e7f0, 0x1e7fe},
  {0x1e800, 0x1e8c4}, {0x1e8d0, 0x1e8d6}, {0x1e900, 0x1e94b}, {0x1e950, 0x1e959}, {0x1ee00, 0x1ee03}, {0x1ee05, 0x1ee1f}, {0x1ee21, 0x1ee22}, {0x1ee24, 0x1ee24},
  {0x1ee27, 0x1ee27}, {0x1ee29, 0x1ee32}, {0x1ee34, 0x1ee37}, {0x1ee39, 0x1ee39}, {0x1ee3b, 0x1ee3b}, {0x1ee42, 0x1ee42}, {0x1ee47, 0x1ee47}, {0x1ee49, 0x1ee49},
  {0x1ee4b, 0x1ee4b}, {0x1ee4d, 0x1ee4f}, {0x1ee51, 0x1ee52}, {0x1ee54, 0x1ee54}, {0x1ee57, 0x1ee57}, {0x1ee59, 0x1ee59}, {0x1ee5b, 0x1ee5b}, {0x1ee5d, 0x1ee5d},
  {0x1ee5f, 0x1ee5f}, {0x1ee61, 0x1ee62}, {0x1ee64, 0x1ee64}, {0x1ee67, 0x1ee6a}, {0x1ee6c, 0x1ee72}, {0x1ee74, 0x1ee77}, {0x1ee79, 0x1ee7c}, {0x1ee7e, 0x1ee7e},
  {0x1ee80, 0x1ee89}, {0x1ee8b, 0x1ee9b}, {0x1eea1, 0x1eea3}, {0x1eea5, 0x1eea9}, {0x1eeab, 0x1eebb}, {0x1fbf0, 0x1fbf9}, {0x20000, 0x2a6df}, {0x2a700, 0x2b739},
  {0x2b740, 0x2b81d}, {0x2b820, 0x2cea1}, {0x2ceb0, 0x2ebe0}, {0x2ebf0, 0x2ee5d}, {0x2f800, 0x2fa1d}, {0x30000, 0x3134a}, {0x31350, 0x323af}, {0xe0100, 0xe01ef},
};

static bool ts_lex(TSLexer *lexer, TSStateId state) {
  START_LEXER();
  eof = lexer->eof(lexer);
  switch (state) {
    case 0:
      if (eof) ADVANCE(72);
      ADVANCE_MAP(
        '!', 171,
        '"', 157,
        '#', 143,
        '$', 85,
        '%', 99,
        '&', 104,
        '\'', 145,
        '(', 75,
        ')', 76,
        '*', 92,
        '+', 90,
        ',', 139,
        '-', 96,
        '.', 133,
        '/', 172,
        '0', 151,
        ':', 83,
        ';', 73,
        '<', 146,
        '=', 123,
        '>', 127,
        '?', 93,
        '@', 132,
        '[', 77,
        '\\', 46,
        ']', 78,
        '^', 100,
        'b', 178,
        'c', 179,
        'm', 182,
        'r', 180,
        '{', 79,
        '|', 106,
        '}', 80,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(68);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(154);
      if (set_contains(sym_identifier_character_set_1, 668, lookahead)) ADVANCE(191);
      END_STATE();
    case 1:
      if (lookahead == '\n') ADVANCE(1);
      if (lookahead == '[') ADVANCE(193);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(192);
      if (lookahead != 0) ADVANCE(193);
      END_STATE();
    case 2:
      ADVANCE_MAP(
        '!', 102,
        '"', 156,
        '#', 142,
        '$', 85,
        '%', 99,
        '&', 104,
        '\'', 145,
        '(', 75,
        ')', 76,
        '*', 92,
        '+', 90,
        ',', 139,
        '-', 96,
        '.', 133,
        '/', 98,
        '0', 151,
        ':', 83,
        ';', 73,
        '<', 129,
        '=', 123,
        '>', 127,
        '?', 93,
        '@', 132,
        '[', 77,
        ']', 78,
        '^', 100,
        'b', 178,
        'c', 179,
        'r', 180,
        '{', 79,
        '|', 106,
        '}', 80,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(2);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(154);
      if (set_contains(sym_identifier_character_set_1, 668, lookahead)) ADVANCE(191);
      END_STATE();
    case 3:
      ADVANCE_MAP(
        '!', 102,
        '"', 156,
        '#', 142,
        '$', 65,
        '%', 99,
        '&', 104,
        '\'', 145,
        '(', 75,
        ')', 76,
        '*', 92,
        '+', 90,
        ',', 139,
        '-', 95,
        '.', 133,
        '/', 98,
        '0', 151,
        ':', 83,
        ';', 73,
        '<', 129,
        '=', 122,
        '>', 127,
        '?', 93,
        '[', 77,
        ']', 78,
        '^', 100,
        'b', 178,
        'c', 179,
        'r', 180,
        '{', 79,
        '|', 106,
        '}', 80,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(3);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(154);
      if (set_contains(sym_identifier_character_set_1, 668, lookahead)) ADVANCE(191);
      END_STATE();
    case 4:
      ADVANCE_MAP(
        '!', 102,
        '"', 156,
        '#', 142,
        '$', 84,
        '%', 99,
        '&', 104,
        '\'', 145,
        '(', 75,
        ')', 76,
        '*', 92,
        '+', 90,
        ',', 139,
        '-', 96,
        '.', 133,
        '/', 98,
        '0', 151,
        ':', 83,
        ';', 73,
        '<', 129,
        '=', 123,
        '>', 127,
        '?', 93,
        '@', 132,
        '[', 77,
        ']', 78,
        '^', 100,
        'b', 178,
        'c', 179,
        'r', 180,
        '{', 79,
        '|', 106,
        '}', 80,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(4);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(154);
      if (set_contains(sym_identifier_character_set_1, 668, lookahead)) ADVANCE(191);
      END_STATE();
    case 5:
      ADVANCE_MAP(
        '!', 102,
        '"', 156,
        '$', 65,
        '%', 99,
        '&', 104,
        '\'', 145,
        '(', 75,
        '*', 92,
        '+', 90,
        '-', 95,
        '.', 133,
        '/', 98,
        '0', 151,
        ':', 83,
        '<', 129,
        '=', 123,
        '>', 127,
        '?', 93,
        '[', 77,
        '^', 100,
        'b', 178,
        'c', 179,
        'r', 180,
        '{', 79,
        '|', 106,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(5);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(154);
      if (set_contains(sym_identifier_character_set_1, 668, lookahead)) ADVANCE(191);
      END_STATE();
    case 6:
      ADVANCE_MAP(
        '!', 102,
        '%', 99,
        '&', 104,
        '\'', 144,
        '(', 75,
        ')', 76,
        '*', 92,
        '+', 90,
        ',', 139,
        '-', 95,
        '.', 133,
        '/', 98,
        ':', 83,
        ';', 73,
        '<', 146,
        '=', 122,
        '>', 127,
        '?', 93,
        '[', 77,
        ']', 78,
        '^', 100,
        'r', 180,
        '{', 79,
        '|', 106,
        '}', 80,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(6);
      if (set_contains(sym_identifier_character_set_1, 668, lookahead)) ADVANCE(191);
      END_STATE();
    case 7:
      ADVANCE_MAP(
        '!', 102,
        '%', 99,
        '&', 104,
        '\'', 144,
        '(', 75,
        ')', 76,
        '*', 92,
        '+', 90,
        ',', 139,
        '-', 95,
        '.', 133,
        '/', 98,
        ':', 40,
        ';', 73,
        '<', 129,
        '=', 122,
        '>', 127,
        '?', 93,
        '[', 77,
        ']', 78,
        '^', 100,
        'r', 180,
        '{', 79,
        '|', 106,
        '}', 80,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(7);
      if (set_contains(sym_identifier_character_set_1, 668, lookahead)) ADVANCE(191);
      END_STATE();
    case 8:
      ADVANCE_MAP(
        '!', 102,
        '%', 99,
        '&', 104,
        '\'', 144,
        '(', 75,
        '*', 92,
        '+', 90,
        '-', 95,
        '.', 133,
        '/', 98,
        ':', 40,
        '<', 129,
        '=', 123,
        '>', 127,
        '?', 93,
        '[', 77,
        '^', 100,
        'r', 180,
        '{', 79,
        '|', 106,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(8);
      if (set_contains(sym_identifier_character_set_1, 668, lookahead)) ADVANCE(191);
      END_STATE();
    case 9:
      ADVANCE_MAP(
        '!', 102,
        '%', 99,
        '&', 104,
        '(', 75,
        '*', 92,
        '+', 90,
        '-', 95,
        '.', 133,
        '/', 98,
        ':', 40,
        '<', 146,
        '=', 123,
        '>', 127,
        '?', 93,
        '[', 77,
        '^', 100,
        'r', 180,
        '|', 106,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(9);
      if (set_contains(sym_identifier_character_set_1, 668, lookahead)) ADVANCE(191);
      END_STATE();
    case 10:
      ADVANCE_MAP(
        '!', 101,
        '"', 157,
        '#', 142,
        '$', 65,
        '&', 24,
        '\'', 144,
        '(', 75,
        ')', 76,
        '*', 33,
        '+', 89,
        ',', 139,
        '.', 30,
        '/', 28,
        ':', 83,
        ';', 73,
        '<', 128,
        '=', 43,
        '>', 126,
        '\\', 46,
        ']', 78,
        'm', 182,
        'r', 180,
        '{', 79,
        '|', 105,
        '}', 80,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(15);
      if (set_contains(sym_identifier_character_set_1, 668, lookahead)) ADVANCE(191);
      END_STATE();
    case 11:
      ADVANCE_MAP(
        '!', 101,
        '"', 156,
        '#', 142,
        '$', 65,
        '&', 103,
        '\'', 145,
        '(', 75,
        ')', 76,
        '*', 91,
        ',', 139,
        '-', 94,
        '.', 31,
        '/', 28,
        '0', 151,
        ':', 83,
        ';', 73,
        '<', 128,
        '=', 121,
        '>', 126,
        '?', 93,
        '[', 77,
        ']', 78,
        'b', 178,
        'c', 179,
        'r', 180,
        '{', 79,
        '|', 105,
        '}', 80,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(11);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(154);
      if (set_contains(sym_identifier_character_set_1, 668, lookahead)) ADVANCE(191);
      END_STATE();
    case 12:
      ADVANCE_MAP(
        '!', 101,
        '"', 156,
        '#', 142,
        '$', 65,
        '&', 103,
        '\'', 145,
        '(', 75,
        ')', 76,
        '*', 91,
        ',', 139,
        '-', 94,
        '.', 32,
        '/', 28,
        '0', 151,
        ':', 40,
        '<', 128,
        '=', 43,
        '?', 93,
        '[', 77,
        'b', 178,
        'c', 179,
        'r', 180,
        '|', 105,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(12);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(154);
      if (set_contains(sym_identifier_character_set_1, 668, lookahead)) ADVANCE(191);
      END_STATE();
    case 13:
      ADVANCE_MAP(
        '!', 101,
        '"', 156,
        '$', 65,
        '&', 103,
        '\'', 145,
        '(', 75,
        '*', 91,
        '+', 89,
        ',', 139,
        '-', 97,
        '.', 31,
        '/', 28,
        '0', 151,
        ':', 40,
        ';', 73,
        '<', 128,
        '[', 77,
        ']', 78,
        'b', 178,
        'c', 179,
        'r', 180,
        '{', 79,
        '|', 105,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(13);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(154);
      if (set_contains(sym_identifier_character_set_1, 668, lookahead)) ADVANCE(191);
      END_STATE();
    case 14:
      ADVANCE_MAP(
        '!', 101,
        '#', 142,
        '$', 65,
        '&', 103,
        '\'', 144,
        '(', 75,
        ')', 76,
        '*', 91,
        '+', 89,
        ',', 139,
        '-', 42,
        '.', 30,
        '/', 28,
        '0', 151,
        ':', 83,
        ';', 73,
        '<', 128,
        '=', 121,
        '>', 126,
        '?', 93,
        '[', 77,
        ']', 78,
        'r', 180,
        '{', 79,
        '|', 105,
        '}', 80,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(14);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(154);
      if (set_contains(sym_identifier_character_set_1, 668, lookahead)) ADVANCE(191);
      END_STATE();
    case 15:
      ADVANCE_MAP(
        '!', 101,
        '#', 142,
        '$', 65,
        '&', 24,
        '\'', 144,
        '(', 75,
        ')', 76,
        '*', 33,
        '+', 89,
        ',', 139,
        '.', 30,
        '/', 28,
        ':', 83,
        ';', 73,
        '<', 128,
        '=', 43,
        '>', 126,
        ']', 78,
        'm', 182,
        'r', 180,
        '{', 79,
        '|', 105,
        '}', 80,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(15);
      if (set_contains(sym_identifier_character_set_1, 668, lookahead)) ADVANCE(191);
      END_STATE();
    case 16:
      ADVANCE_MAP(
        '!', 101,
        '\'', 144,
        '(', 75,
        ')', 76,
        '*', 91,
        '+', 89,
        ',', 139,
        '.', 30,
        '/', 28,
        ':', 83,
        ';', 73,
        '<', 146,
        '=', 121,
        '>', 126,
        '@', 132,
        ']', 78,
        'r', 180,
        '{', 79,
        '|', 105,
        '}', 80,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(16);
      if (set_contains(sym_identifier_character_set_1, 668, lookahead)) ADVANCE(191);
      END_STATE();
    case 17:
      ADVANCE_MAP(
        '!', 101,
        '(', 75,
        '.', 30,
        '/', 28,
        ':', 40,
        '<', 146,
        '=', 43,
        '@', 132,
        'r', 180,
        '{', 79,
        '|', 105,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(17);
      if (set_contains(sym_identifier_character_set_1, 668, lookahead)) ADVANCE(191);
      END_STATE();
    case 18:
      ADVANCE_MAP(
        '!', 41,
        '%', 99,
        '&', 104,
        '\'', 144,
        '(', 75,
        ')', 76,
        '*', 92,
        '+', 90,
        ',', 139,
        '-', 96,
        '.', 133,
        '/', 98,
        ';', 73,
        '<', 129,
        '=', 122,
        '>', 127,
        '?', 93,
        '[', 77,
        ']', 78,
        '^', 100,
        'r', 180,
        '{', 79,
        '|', 106,
        '}', 80,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(18);
      if (set_contains(sym_identifier_character_set_1, 668, lookahead)) ADVANCE(191);
      END_STATE();
    case 19:
      ADVANCE_MAP(
        '!', 41,
        '%', 99,
        '&', 104,
        '\'', 144,
        '(', 75,
        ')', 76,
        '*', 92,
        '+', 90,
        ',', 139,
        '-', 95,
        '.', 133,
        '/', 98,
        ':', 82,
        ';', 73,
        '<', 129,
        '=', 122,
        '>', 127,
        '?', 93,
        '[', 77,
        ']', 78,
        '^', 100,
        'r', 180,
        '{', 79,
        '|', 106,
        '}', 80,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(19);
      if (set_contains(sym_identifier_character_set_1, 668, lookahead)) ADVANCE(191);
      END_STATE();
    case 20:
      ADVANCE_MAP(
        '!', 41,
        '%', 99,
        '&', 104,
        '(', 75,
        '*', 92,
        '+', 90,
        '-', 96,
        '.', 133,
        '/', 98,
        '<', 129,
        '=', 123,
        '>', 127,
        '?', 93,
        '[', 77,
        '^', 100,
        'r', 180,
        '|', 106,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(20);
      if (set_contains(sym_identifier_character_set_1, 668, lookahead)) ADVANCE(191);
      END_STATE();
    case 21:
      if (lookahead == '"') ADVANCE(156);
      if (lookahead == '/') ADVANCE(28);
      if (lookahead == ';') ADVANCE(73);
      if (lookahead == 'r') ADVANCE(180);
      if (lookahead == '{') ADVANCE(79);
      if (lookahead == 'b' ||
          lookahead == 'c') ADVANCE(179);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(21);
      if (set_contains(sym_identifier_character_set_1, 668, lookahead)) ADVANCE(191);
      END_STATE();
    case 22:
      ADVANCE_MAP(
        '#', 142,
        '$', 65,
        ',', 139,
        '/', 28,
        ':', 40,
        ';', 73,
        '<', 128,
        '=', 121,
        'm', 182,
        'r', 180,
        '}', 80,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(22);
      if (set_contains(sym_identifier_character_set_1, 668, lookahead)) ADVANCE(191);
      END_STATE();
    case 23:
      ADVANCE_MAP(
        '#', 142,
        ',', 139,
        '.', 31,
        '/', 28,
        '0', 151,
        ':', 82,
        ';', 73,
        '<', 128,
        '=', 121,
        'r', 180,
        '{', 79,
        '}', 80,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(23);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(154);
      if (set_contains(sym_identifier_character_set_1, 668, lookahead)) ADVANCE(191);
      END_STATE();
    case 24:
      if (lookahead == '&') ADVANCE(107);
      END_STATE();
    case 25:
      if (lookahead == '\'') ADVANCE(158);
      END_STATE();
    case 26:
      if (lookahead == '\'') ADVANCE(158);
      if (lookahead == '\\') ADVANCE(47);
      if (lookahead != 0) ADVANCE(25);
      END_STATE();
    case 27:
      ADVANCE_MAP(
        '\'', 144,
        '(', 75,
        ')', 76,
        '+', 89,
        ',', 139,
        '-', 42,
        '.', 30,
        '/', 28,
        ':', 82,
        ';', 73,
        '<', 146,
        '=', 121,
        '>', 126,
        ']', 78,
        'r', 180,
        '{', 79,
        '|', 105,
        '}', 80,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(27);
      if (set_contains(sym_identifier_character_set_1, 668, lookahead)) ADVANCE(191);
      END_STATE();
    case 28:
      if (lookahead == '*') ADVANCE(174);
      if (lookahead == '/') ADVANCE(160);
      END_STATE();
    case 29:
      if (lookahead == '*') ADVANCE(91);
      if (lookahead == '+') ADVANCE(89);
      if (lookahead == '/') ADVANCE(86);
      if (lookahead == '?') ADVANCE(93);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(87);
      if (lookahead != 0) ADVANCE(88);
      END_STATE();
    case 30:
      if (lookahead == '.') ADVANCE(136);
      END_STATE();
    case 31:
      if (lookahead == '.') ADVANCE(134);
      END_STATE();
    case 32:
      if (lookahead == '.') ADVANCE(135);
      END_STATE();
    case 33:
      if (lookahead == '/') ADVANCE(176);
      END_STATE();
    case 34:
      if (lookahead == '1') ADVANCE(36);
      if (lookahead == '3') ADVANCE(35);
      if (lookahead == '6') ADVANCE(38);
      if (lookahead == '8') ADVANCE(147);
      if (lookahead == 's') ADVANCE(45);
      END_STATE();
    case 35:
      if (lookahead == '2') ADVANCE(147);
      END_STATE();
    case 36:
      if (lookahead == '2') ADVANCE(39);
      if (lookahead == '6') ADVANCE(147);
      END_STATE();
    case 37:
      if (lookahead == '3') ADVANCE(35);
      if (lookahead == '6') ADVANCE(38);
      END_STATE();
    case 38:
      if (lookahead == '4') ADVANCE(147);
      END_STATE();
    case 39:
      if (lookahead == '8') ADVANCE(147);
      END_STATE();
    case 40:
      if (lookahead == ':') ADVANCE(140);
      END_STATE();
    case 41:
      if (lookahead == '=') ADVANCE(125);
      END_STATE();
    case 42:
      if (lookahead == '>') ADVANCE(141);
      END_STATE();
    case 43:
      if (lookahead == '>') ADVANCE(81);
      END_STATE();
    case 44:
      if (lookahead == 'e') ADVANCE(147);
      END_STATE();
    case 45:
      if (lookahead == 'i') ADVANCE(48);
      END_STATE();
    case 46:
      if (lookahead == 'u') ADVANCE(49);
      if (lookahead == 'x') ADVANCE(61);
      if (lookahead != 0) ADVANCE(159);
      END_STATE();
    case 47:
      if (lookahead == 'u') ADVANCE(50);
      if (lookahead == 'x') ADVANCE(62);
      if (lookahead != 0) ADVANCE(25);
      END_STATE();
    case 48:
      if (lookahead == 'z') ADVANCE(44);
      END_STATE();
    case 49:
      if (lookahead == '{') ADVANCE(59);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(57);
      END_STATE();
    case 50:
      if (lookahead == '{') ADVANCE(60);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(63);
      END_STATE();
    case 51:
      if (lookahead == '}') ADVANCE(25);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(51);
      END_STATE();
    case 52:
      if (lookahead == '}') ADVANCE(159);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(52);
      END_STATE();
    case 53:
      if (lookahead == '0' ||
          lookahead == '1' ||
          lookahead == '_') ADVANCE(152);
      END_STATE();
    case 54:
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(1);
      if (lookahead != 0 &&
          lookahead != '[') ADVANCE(67);
      END_STATE();
    case 55:
      if (('0' <= lookahead && lookahead <= '7') ||
          lookahead == '_') ADVANCE(153);
      END_STATE();
    case 56:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(25);
      END_STATE();
    case 57:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(61);
      END_STATE();
    case 58:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(159);
      END_STATE();
    case 59:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(52);
      END_STATE();
    case 60:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(51);
      END_STATE();
    case 61:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(58);
      END_STATE();
    case 62:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(56);
      END_STATE();
    case 63:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(62);
      END_STATE();
    case 64:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(155);
      END_STATE();
    case 65:
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(194);
      END_STATE();
    case 66:
      if (set_contains(sym_identifier_character_set_1, 668, lookahead)) ADVANCE(191);
      END_STATE();
    case 67:
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(193);
      END_STATE();
    case 68:
      if (eof) ADVANCE(72);
      ADVANCE_MAP(
        '!', 102,
        '"', 156,
        '#', 143,
        '$', 85,
        '%', 99,
        '&', 104,
        '\'', 145,
        '(', 75,
        ')', 76,
        '*', 92,
        '+', 90,
        ',', 139,
        '-', 96,
        '.', 133,
        '/', 98,
        '0', 151,
        ':', 83,
        ';', 73,
        '<', 146,
        '=', 123,
        '>', 127,
        '?', 93,
        '@', 132,
        '[', 77,
        ']', 78,
        '^', 100,
        'b', 178,
        'c', 179,
        'm', 182,
        'r', 180,
        '{', 79,
        '|', 106,
        '}', 80,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(68);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(154);
      if (set_contains(sym_identifier_character_set_1, 668, lookahead)) ADVANCE(191);
      END_STATE();
    case 69:
      if (eof) ADVANCE(72);
      ADVANCE_MAP(
        '!', 102,
        '"', 156,
        '#', 142,
        '$', 65,
        '%', 99,
        '&', 104,
        '\'', 145,
        '(', 75,
        '*', 92,
        '+', 90,
        '-', 95,
        '.', 133,
        '/', 98,
        '0', 151,
        ':', 40,
        ';', 73,
        '<', 129,
        '=', 122,
        '>', 127,
        '?', 93,
        '[', 77,
        '^', 100,
        'b', 178,
        'c', 179,
        'm', 182,
        'r', 180,
        '{', 79,
        '|', 106,
        '}', 80,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(69);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(154);
      if (set_contains(sym_identifier_character_set_1, 668, lookahead)) ADVANCE(191);
      END_STATE();
    case 70:
      if (eof) ADVANCE(72);
      ADVANCE_MAP(
        '!', 101,
        '"', 156,
        '#', 143,
        '$', 65,
        '&', 103,
        '\'', 145,
        '(', 75,
        ')', 76,
        '*', 91,
        '+', 89,
        ',', 139,
        '-', 94,
        '.', 31,
        '/', 28,
        '0', 151,
        ':', 83,
        ';', 73,
        '<', 128,
        '=', 121,
        '>', 126,
        '?', 93,
        '[', 77,
        ']', 78,
        'b', 178,
        'c', 179,
        'm', 182,
        'r', 180,
        '{', 79,
        '|', 105,
        '}', 80,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(70);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(154);
      if (set_contains(sym_identifier_character_set_1, 668, lookahead)) ADVANCE(191);
      END_STATE();
    case 71:
      if (eof) ADVANCE(72);
      ADVANCE_MAP(
        '!', 101,
        '"', 156,
        '#', 142,
        '$', 65,
        '&', 103,
        '\'', 145,
        '(', 75,
        '*', 91,
        '-', 94,
        '.', 31,
        '/', 28,
        '0', 151,
        ':', 40,
        ';', 73,
        '<', 128,
        '=', 43,
        '[', 77,
        'b', 178,
        'c', 179,
        'm', 182,
        'r', 180,
        '{', 79,
        '|', 105,
        '}', 80,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(71);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(154);
      if (set_contains(sym_identifier_character_set_1, 668, lookahead)) ADVANCE(191);
      END_STATE();
    case 72:
      ACCEPT_TOKEN(ts_builtin_sym_end);
      END_STATE();
    case 73:
      ACCEPT_TOKEN(anon_sym_SEMI);
      END_STATE();
    case 74:
      ACCEPT_TOKEN(anon_sym_macro_rules_BANG);
      END_STATE();
    case 75:
      ACCEPT_TOKEN(anon_sym_LPAREN);
      END_STATE();
    case 76:
      ACCEPT_TOKEN(anon_sym_RPAREN);
      END_STATE();
    case 77:
      ACCEPT_TOKEN(anon_sym_LBRACK);
      END_STATE();
    case 78:
      ACCEPT_TOKEN(anon_sym_RBRACK);
      END_STATE();
    case 79:
      ACCEPT_TOKEN(anon_sym_LBRACE);
      END_STATE();
    case 80:
      ACCEPT_TOKEN(anon_sym_RBRACE);
      END_STATE();
    case 81:
      ACCEPT_TOKEN(anon_sym_EQ_GT);
      END_STATE();
    case 82:
      ACCEPT_TOKEN(anon_sym_COLON);
      END_STATE();
    case 83:
      ACCEPT_TOKEN(anon_sym_COLON);
      if (lookahead == ':') ADVANCE(140);
      END_STATE();
    case 84:
      ACCEPT_TOKEN(anon_sym_DOLLAR);
      END_STATE();
    case 85:
      ACCEPT_TOKEN(anon_sym_DOLLAR);
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(194);
      END_STATE();
    case 86:
      ACCEPT_TOKEN(aux_sym_token_repetition_pattern_token1);
      if (lookahead == '*') ADVANCE(174);
      if (lookahead == '/') ADVANCE(161);
      if (lookahead != 0 &&
          lookahead != '*' &&
          lookahead != '+' &&
          lookahead != '?') ADVANCE(88);
      END_STATE();
    case 87:
      ACCEPT_TOKEN(aux_sym_token_repetition_pattern_token1);
      if (lookahead == '/') ADVANCE(86);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(87);
      if (lookahead != 0 &&
          lookahead != '*' &&
          lookahead != '+' &&
          lookahead != '?') ADVANCE(88);
      END_STATE();
    case 88:
      ACCEPT_TOKEN(aux_sym_token_repetition_pattern_token1);
      if (lookahead != 0 &&
          lookahead != '*' &&
          lookahead != '+' &&
          lookahead != '?') ADVANCE(88);
      END_STATE();
    case 89:
      ACCEPT_TOKEN(anon_sym_PLUS);
      END_STATE();
    case 90:
      ACCEPT_TOKEN(anon_sym_PLUS);
      if (lookahead == '=') ADVANCE(111);
      END_STATE();
    case 91:
      ACCEPT_TOKEN(anon_sym_STAR);
      END_STATE();
    case 92:
      ACCEPT_TOKEN(anon_sym_STAR);
      if (lookahead == '=') ADVANCE(113);
      END_STATE();
    case 93:
      ACCEPT_TOKEN(anon_sym_QMARK);
      END_STATE();
    case 94:
      ACCEPT_TOKEN(anon_sym_DASH);
      END_STATE();
    case 95:
      ACCEPT_TOKEN(anon_sym_DASH);
      if (lookahead == '=') ADVANCE(112);
      END_STATE();
    case 96:
      ACCEPT_TOKEN(anon_sym_DASH);
      if (lookahead == '=') ADVANCE(112);
      if (lookahead == '>') ADVANCE(141);
      END_STATE();
    case 97:
      ACCEPT_TOKEN(anon_sym_DASH);
      if (lookahead == '>') ADVANCE(141);
      END_STATE();
    case 98:
      ACCEPT_TOKEN(anon_sym_SLASH);
      if (lookahead == '*') ADVANCE(174);
      if (lookahead == '/') ADVANCE(160);
      if (lookahead == '=') ADVANCE(114);
      END_STATE();
    case 99:
      ACCEPT_TOKEN(anon_sym_PERCENT);
      if (lookahead == '=') ADVANCE(115);
      END_STATE();
    case 100:
      ACCEPT_TOKEN(anon_sym_CARET);
      if (lookahead == '=') ADVANCE(116);
      END_STATE();
    case 101:
      ACCEPT_TOKEN(anon_sym_BANG);
      END_STATE();
    case 102:
      ACCEPT_TOKEN(anon_sym_BANG);
      if (lookahead == '=') ADVANCE(125);
      END_STATE();
    case 103:
      ACCEPT_TOKEN(anon_sym_AMP);
      END_STATE();
    case 104:
      ACCEPT_TOKEN(anon_sym_AMP);
      if (lookahead == '&') ADVANCE(107);
      if (lookahead == '=') ADVANCE(117);
      END_STATE();
    case 105:
      ACCEPT_TOKEN(anon_sym_PIPE);
      END_STATE();
    case 106:
      ACCEPT_TOKEN(anon_sym_PIPE);
      if (lookahead == '=') ADVANCE(118);
      if (lookahead == '|') ADVANCE(108);
      END_STATE();
    case 107:
      ACCEPT_TOKEN(anon_sym_AMP_AMP);
      END_STATE();
    case 108:
      ACCEPT_TOKEN(anon_sym_PIPE_PIPE);
      END_STATE();
    case 109:
      ACCEPT_TOKEN(anon_sym_LT_LT);
      if (lookahead == '=') ADVANCE(119);
      END_STATE();
    case 110:
      ACCEPT_TOKEN(anon_sym_GT_GT);
      if (lookahead == '=') ADVANCE(120);
      END_STATE();
    case 111:
      ACCEPT_TOKEN(anon_sym_PLUS_EQ);
      END_STATE();
    case 112:
      ACCEPT_TOKEN(anon_sym_DASH_EQ);
      END_STATE();
    case 113:
      ACCEPT_TOKEN(anon_sym_STAR_EQ);
      END_STATE();
    case 114:
      ACCEPT_TOKEN(anon_sym_SLASH_EQ);
      END_STATE();
    case 115:
      ACCEPT_TOKEN(anon_sym_PERCENT_EQ);
      END_STATE();
    case 116:
      ACCEPT_TOKEN(anon_sym_CARET_EQ);
      END_STATE();
    case 117:
      ACCEPT_TOKEN(anon_sym_AMP_EQ);
      END_STATE();
    case 118:
      ACCEPT_TOKEN(anon_sym_PIPE_EQ);
      END_STATE();
    case 119:
      ACCEPT_TOKEN(anon_sym_LT_LT_EQ);
      END_STATE();
    case 120:
      ACCEPT_TOKEN(anon_sym_GT_GT_EQ);
      END_STATE();
    case 121:
      ACCEPT_TOKEN(anon_sym_EQ);
      END_STATE();
    case 122:
      ACCEPT_TOKEN(anon_sym_EQ);
      if (lookahead == '=') ADVANCE(124);
      END_STATE();
    case 123:
      ACCEPT_TOKEN(anon_sym_EQ);
      if (lookahead == '=') ADVANCE(124);
      if (lookahead == '>') ADVANCE(81);
      END_STATE();
    case 124:
      ACCEPT_TOKEN(anon_sym_EQ_EQ);
      END_STATE();
    case 125:
      ACCEPT_TOKEN(anon_sym_BANG_EQ);
      END_STATE();
    case 126:
      ACCEPT_TOKEN(anon_sym_GT);
      END_STATE();
    case 127:
      ACCEPT_TOKEN(anon_sym_GT);
      if (lookahead == '=') ADVANCE(130);
      if (lookahead == '>') ADVANCE(110);
      END_STATE();
    case 128:
      ACCEPT_TOKEN(anon_sym_LT);
      END_STATE();
    case 129:
      ACCEPT_TOKEN(anon_sym_LT);
      if (lookahead == '<') ADVANCE(109);
      if (lookahead == '=') ADVANCE(131);
      END_STATE();
    case 130:
      ACCEPT_TOKEN(anon_sym_GT_EQ);
      END_STATE();
    case 131:
      ACCEPT_TOKEN(anon_sym_LT_EQ);
      END_STATE();
    case 132:
      ACCEPT_TOKEN(anon_sym_AT);
      END_STATE();
    case 133:
      ACCEPT_TOKEN(anon_sym_DOT);
      if (lookahead == '.') ADVANCE(136);
      END_STATE();
    case 134:
      ACCEPT_TOKEN(anon_sym_DOT_DOT);
      END_STATE();
    case 135:
      ACCEPT_TOKEN(anon_sym_DOT_DOT);
      if (lookahead == '.') ADVANCE(137);
      END_STATE();
    case 136:
      ACCEPT_TOKEN(anon_sym_DOT_DOT);
      if (lookahead == '.') ADVANCE(137);
      if (lookahead == '=') ADVANCE(138);
      END_STATE();
    case 137:
      ACCEPT_TOKEN(anon_sym_DOT_DOT_DOT);
      END_STATE();
    case 138:
      ACCEPT_TOKEN(anon_sym_DOT_DOT_EQ);
      END_STATE();
    case 139:
      ACCEPT_TOKEN(anon_sym_COMMA);
      END_STATE();
    case 140:
      ACCEPT_TOKEN(anon_sym_COLON_COLON);
      END_STATE();
    case 141:
      ACCEPT_TOKEN(anon_sym_DASH_GT);
      END_STATE();
    case 142:
      ACCEPT_TOKEN(anon_sym_POUND);
      END_STATE();
    case 143:
      ACCEPT_TOKEN(anon_sym_POUND);
      if (lookahead == '!') ADVANCE(54);
      END_STATE();
    case 144:
      ACCEPT_TOKEN(anon_sym_SQUOTE);
      END_STATE();
    case 145:
      ACCEPT_TOKEN(anon_sym_SQUOTE);
      if (lookahead == '\'') ADVANCE(158);
      if (lookahead == '\\') ADVANCE(47);
      if (lookahead != 0) ADVANCE(25);
      END_STATE();
    case 146:
      ACCEPT_TOKEN(anon_sym_LT2);
      END_STATE();
    case 147:
      ACCEPT_TOKEN(sym_integer_literal);
      END_STATE();
    case 148:
      ACCEPT_TOKEN(sym_integer_literal);
      if (lookahead == '2') ADVANCE(155);
      if (lookahead == 'f') ADVANCE(149);
      if (lookahead == 'i') ADVANCE(34);
      if (lookahead == 'u') ADVANCE(34);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'e')) ADVANCE(155);
      END_STATE();
    case 149:
      ACCEPT_TOKEN(sym_integer_literal);
      if (lookahead == '3') ADVANCE(148);
      if (lookahead == '6') ADVANCE(150);
      if (lookahead == 'f') ADVANCE(149);
      if (lookahead == 'i') ADVANCE(34);
      if (lookahead == 'u') ADVANCE(34);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'e')) ADVANCE(155);
      END_STATE();
    case 150:
      ACCEPT_TOKEN(sym_integer_literal);
      if (lookahead == '4') ADVANCE(155);
      if (lookahead == 'f') ADVANCE(149);
      if (lookahead == 'i') ADVANCE(34);
      if (lookahead == 'u') ADVANCE(34);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'e')) ADVANCE(155);
      END_STATE();
    case 151:
      ACCEPT_TOKEN(sym_integer_literal);
      if (lookahead == 'b') ADVANCE(53);
      if (lookahead == 'f') ADVANCE(37);
      if (lookahead == 'i') ADVANCE(34);
      if (lookahead == 'o') ADVANCE(55);
      if (lookahead == 'u') ADVANCE(34);
      if (lookahead == 'x') ADVANCE(64);
      if (('0' <= lookahead && lookahead <= '9') ||
          lookahead == '_') ADVANCE(154);
      END_STATE();
    case 152:
      ACCEPT_TOKEN(sym_integer_literal);
      if (lookahead == 'f') ADVANCE(37);
      if (lookahead == 'i') ADVANCE(34);
      if (lookahead == 'u') ADVANCE(34);
      if (lookahead == '0' ||
          lookahead == '1' ||
          lookahead == '_') ADVANCE(152);
      END_STATE();
    case 153:
      ACCEPT_TOKEN(sym_integer_literal);
      if (lookahead == 'f') ADVANCE(37);
      if (lookahead == 'i') ADVANCE(34);
      if (lookahead == 'u') ADVANCE(34);
      if (('0' <= lookahead && lookahead <= '7') ||
          lookahead == '_') ADVANCE(153);
      END_STATE();
    case 154:
      ACCEPT_TOKEN(sym_integer_literal);
      if (lookahead == 'f') ADVANCE(37);
      if (lookahead == 'i') ADVANCE(34);
      if (lookahead == 'u') ADVANCE(34);
      if (('0' <= lookahead && lookahead <= '9') ||
          lookahead == '_') ADVANCE(154);
      END_STATE();
    case 155:
      ACCEPT_TOKEN(sym_integer_literal);
      if (lookahead == 'f') ADVANCE(149);
      if (lookahead == 'i') ADVANCE(34);
      if (lookahead == 'u') ADVANCE(34);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'e')) ADVANCE(155);
      END_STATE();
    case 156:
      ACCEPT_TOKEN(aux_sym_string_literal_token1);
      END_STATE();
    case 157:
      ACCEPT_TOKEN(anon_sym_DQUOTE);
      END_STATE();
    case 158:
      ACCEPT_TOKEN(sym_char_literal);
      END_STATE();
    case 159:
      ACCEPT_TOKEN(sym_escape_sequence);
      END_STATE();
    case 160:
      ACCEPT_TOKEN(anon_sym_SLASH_SLASH);
      END_STATE();
    case 161:
      ACCEPT_TOKEN(anon_sym_SLASH_SLASH);
      if (lookahead != 0 &&
          lookahead != '*' &&
          lookahead != '+' &&
          lookahead != '?') ADVANCE(88);
      END_STATE();
    case 162:
      ACCEPT_TOKEN(anon_sym_SLASH_SLASH);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(166);
      END_STATE();
    case 163:
      ACCEPT_TOKEN(aux_sym_line_comment_token1);
      END_STATE();
    case 164:
      ACCEPT_TOKEN(aux_sym_line_comment_token2);
      if (lookahead == '*') ADVANCE(175);
      if (lookahead == '/') ADVANCE(162);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(166);
      END_STATE();
    case 165:
      ACCEPT_TOKEN(aux_sym_line_comment_token2);
      if (lookahead == '/') ADVANCE(164);
      if (lookahead == '\t' ||
          (0x0b <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(165);
      if (lookahead != 0 &&
          (lookahead < '\t' || '\r' < lookahead)) ADVANCE(166);
      END_STATE();
    case 166:
      ACCEPT_TOKEN(aux_sym_line_comment_token2);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(166);
      END_STATE();
    case 167:
      ACCEPT_TOKEN(aux_sym_line_comment_token3);
      if (lookahead == '!') ADVANCE(171);
      if (lookahead == '/') ADVANCE(173);
      if (lookahead == '\t' ||
          (0x0b <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(169);
      if (lookahead != 0 &&
          (lookahead < '\t' || '\r' < lookahead)) ADVANCE(170);
      END_STATE();
    case 168:
      ACCEPT_TOKEN(aux_sym_line_comment_token3);
      if (lookahead == '*') ADVANCE(170);
      if (lookahead == '/') ADVANCE(170);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(170);
      END_STATE();
    case 169:
      ACCEPT_TOKEN(aux_sym_line_comment_token3);
      if (lookahead == '/') ADVANCE(168);
      if (lookahead == '\t' ||
          (0x0b <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(169);
      if (lookahead != 0 &&
          (lookahead < '\t' || '\r' < lookahead)) ADVANCE(170);
      END_STATE();
    case 170:
      ACCEPT_TOKEN(aux_sym_line_comment_token3);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(170);
      END_STATE();
    case 171:
      ACCEPT_TOKEN(anon_sym_BANG2);
      END_STATE();
    case 172:
      ACCEPT_TOKEN(anon_sym_SLASH2);
      END_STATE();
    case 173:
      ACCEPT_TOKEN(anon_sym_SLASH2);
      if (lookahead == '/') ADVANCE(163);
      END_STATE();
    case 174:
      ACCEPT_TOKEN(anon_sym_SLASH_STAR);
      END_STATE();
    case 175:
      ACCEPT_TOKEN(anon_sym_SLASH_STAR);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(166);
      END_STATE();
    case 176:
      ACCEPT_TOKEN(anon_sym_STAR_SLASH);
      END_STATE();
    case 177:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '!') ADVANCE(74);
      if (set_contains(sym_identifier_character_set_3, 776, lookahead)) ADVANCE(191);
      END_STATE();
    case 178:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '"') ADVANCE(156);
      if (lookahead == '\'') ADVANCE(26);
      if (set_contains(sym_identifier_character_set_3, 776, lookahead)) ADVANCE(191);
      END_STATE();
    case 179:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '"') ADVANCE(156);
      if (set_contains(sym_identifier_character_set_3, 776, lookahead)) ADVANCE(191);
      END_STATE();
    case 180:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '#') ADVANCE(66);
      if (set_contains(sym_identifier_character_set_3, 776, lookahead)) ADVANCE(191);
      END_STATE();
    case 181:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(188);
      if (set_contains(sym_identifier_character_set_3, 776, lookahead)) ADVANCE(191);
      END_STATE();
    case 182:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'a') ADVANCE(183);
      if (set_contains(sym_identifier_character_set_3, 776, lookahead)) ADVANCE(191);
      END_STATE();
    case 183:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'c') ADVANCE(187);
      if (set_contains(sym_identifier_character_set_3, 776, lookahead)) ADVANCE(191);
      END_STATE();
    case 184:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'e') ADVANCE(189);
      if (set_contains(sym_identifier_character_set_3, 776, lookahead)) ADVANCE(191);
      END_STATE();
    case 185:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'l') ADVANCE(184);
      if (set_contains(sym_identifier_character_set_3, 776, lookahead)) ADVANCE(191);
      END_STATE();
    case 186:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'o') ADVANCE(181);
      if (set_contains(sym_identifier_character_set_3, 776, lookahead)) ADVANCE(191);
      END_STATE();
    case 187:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'r') ADVANCE(186);
      if (set_contains(sym_identifier_character_set_3, 776, lookahead)) ADVANCE(191);
      END_STATE();
    case 188:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'r') ADVANCE(190);
      if (set_contains(sym_identifier_character_set_3, 776, lookahead)) ADVANCE(191);
      END_STATE();
    case 189:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 's') ADVANCE(177);
      if (set_contains(sym_identifier_character_set_3, 776, lookahead)) ADVANCE(191);
      END_STATE();
    case 190:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'u') ADVANCE(185);
      if (set_contains(sym_identifier_character_set_3, 776, lookahead)) ADVANCE(191);
      END_STATE();
    case 191:
      ACCEPT_TOKEN(sym_identifier);
      if (set_contains(sym_identifier_character_set_3, 776, lookahead)) ADVANCE(191);
      END_STATE();
    case 192:
      ACCEPT_TOKEN(sym_shebang);
      if (lookahead == '\n') ADVANCE(1);
      if (lookahead == '[') ADVANCE(193);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(192);
      if (lookahead != 0) ADVANCE(193);
      END_STATE();
    case 193:
      ACCEPT_TOKEN(sym_shebang);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(193);
      END_STATE();
    case 194:
      ACCEPT_TOKEN(sym_metavariable);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(194);
      END_STATE();
    default:
      return false;
  }
}

static bool ts_lex_keywords(TSLexer *lexer, TSStateId state) {
  START_LEXER();
  eof = lexer->eof(lexer);
  switch (state) {
    case 0:
      ADVANCE_MAP(
        '_', 1,
        'a', 2,
        'b', 3,
        'c', 4,
        'd', 5,
        'e', 6,
        'f', 7,
        'g', 8,
        'i', 9,
        'l', 10,
        'm', 11,
        'p', 12,
        'r', 13,
        's', 14,
        't', 15,
        'u', 16,
        'v', 17,
        'w', 18,
        'y', 19,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(0);
      END_STATE();
    case 1:
      ACCEPT_TOKEN(anon_sym__);
      END_STATE();
    case 2:
      if (lookahead == 's') ADVANCE(20);
      if (lookahead == 'w') ADVANCE(21);
      END_STATE();
    case 3:
      if (lookahead == 'l') ADVANCE(22);
      if (lookahead == 'o') ADVANCE(23);
      if (lookahead == 'r') ADVANCE(24);
      END_STATE();
    case 4:
      if (lookahead == 'h') ADVANCE(25);
      if (lookahead == 'o') ADVANCE(26);
      if (lookahead == 'r') ADVANCE(27);
      END_STATE();
    case 5:
      if (lookahead == 'e') ADVANCE(28);
      if (lookahead == 'y') ADVANCE(29);
      END_STATE();
    case 6:
      if (lookahead == 'l') ADVANCE(30);
      if (lookahead == 'n') ADVANCE(31);
      if (lookahead == 'x') ADVANCE(32);
      END_STATE();
    case 7:
      if (lookahead == '3') ADVANCE(33);
      if (lookahead == '6') ADVANCE(34);
      if (lookahead == 'a') ADVANCE(35);
      if (lookahead == 'n') ADVANCE(36);
      if (lookahead == 'o') ADVANCE(37);
      END_STATE();
    case 8:
      if (lookahead == 'e') ADVANCE(38);
      END_STATE();
    case 9:
      ADVANCE_MAP(
        '1', 39,
        '3', 40,
        '6', 41,
        '8', 42,
        'd', 43,
        'f', 44,
        'm', 45,
        'n', 46,
        's', 47,
        't', 48,
      );
      END_STATE();
    case 10:
      if (lookahead == 'e') ADVANCE(49);
      if (lookahead == 'i') ADVANCE(50);
      if (lookahead == 'o') ADVANCE(51);
      END_STATE();
    case 11:
      if (lookahead == 'a') ADVANCE(52);
      if (lookahead == 'e') ADVANCE(53);
      if (lookahead == 'o') ADVANCE(54);
      if (lookahead == 'u') ADVANCE(55);
      END_STATE();
    case 12:
      if (lookahead == 'a') ADVANCE(56);
      if (lookahead == 'u') ADVANCE(57);
      END_STATE();
    case 13:
      if (lookahead == 'a') ADVANCE(58);
      if (lookahead == 'e') ADVANCE(59);
      END_STATE();
    case 14:
      if (lookahead == 'e') ADVANCE(60);
      if (lookahead == 't') ADVANCE(61);
      if (lookahead == 'u') ADVANCE(62);
      END_STATE();
    case 15:
      if (lookahead == 'r') ADVANCE(63);
      if (lookahead == 't') ADVANCE(64);
      if (lookahead == 'y') ADVANCE(65);
      END_STATE();
    case 16:
      if (lookahead == '1') ADVANCE(66);
      if (lookahead == '3') ADVANCE(67);
      if (lookahead == '6') ADVANCE(68);
      if (lookahead == '8') ADVANCE(69);
      if (lookahead == 'n') ADVANCE(70);
      if (lookahead == 's') ADVANCE(71);
      END_STATE();
    case 17:
      if (lookahead == 'i') ADVANCE(72);
      END_STATE();
    case 18:
      if (lookahead == 'h') ADVANCE(73);
      END_STATE();
    case 19:
      if (lookahead == 'i') ADVANCE(74);
      END_STATE();
    case 20:
      ACCEPT_TOKEN(anon_sym_as);
      if (lookahead == 'y') ADVANCE(75);
      END_STATE();
    case 21:
      if (lookahead == 'a') ADVANCE(76);
      END_STATE();
    case 22:
      if (lookahead == 'o') ADVANCE(77);
      END_STATE();
    case 23:
      if (lookahead == 'o') ADVANCE(78);
      END_STATE();
    case 24:
      if (lookahead == 'e') ADVANCE(79);
      END_STATE();
    case 25:
      if (lookahead == 'a') ADVANCE(80);
      END_STATE();
    case 26:
      if (lookahead == 'n') ADVANCE(81);
      END_STATE();
    case 27:
      if (lookahead == 'a') ADVANCE(82);
      END_STATE();
    case 28:
      if (lookahead == 'f') ADVANCE(83);
      END_STATE();
    case 29:
      if (lookahead == 'n') ADVANCE(84);
      END_STATE();
    case 30:
      if (lookahead == 's') ADVANCE(85);
      END_STATE();
    case 31:
      if (lookahead == 'u') ADVANCE(86);
      END_STATE();
    case 32:
      if (lookahead == 'p') ADVANCE(87);
      if (lookahead == 't') ADVANCE(88);
      END_STATE();
    case 33:
      if (lookahead == '2') ADVANCE(89);
      END_STATE();
    case 34:
      if (lookahead == '4') ADVANCE(90);
      END_STATE();
    case 35:
      if (lookahead == 'l') ADVANCE(91);
      END_STATE();
    case 36:
      ACCEPT_TOKEN(anon_sym_fn);
      END_STATE();
    case 37:
      if (lookahead == 'r') ADVANCE(92);
      END_STATE();
    case 38:
      if (lookahead == 'n') ADVANCE(93);
      END_STATE();
    case 39:
      if (lookahead == '2') ADVANCE(94);
      if (lookahead == '6') ADVANCE(95);
      END_STATE();
    case 40:
      if (lookahead == '2') ADVANCE(96);
      END_STATE();
    case 41:
      if (lookahead == '4') ADVANCE(97);
      END_STATE();
    case 42:
      ACCEPT_TOKEN(anon_sym_i8);
      END_STATE();
    case 43:
      if (lookahead == 'e') ADVANCE(98);
      END_STATE();
    case 44:
      ACCEPT_TOKEN(anon_sym_if);
      END_STATE();
    case 45:
      if (lookahead == 'p') ADVANCE(99);
      END_STATE();
    case 46:
      ACCEPT_TOKEN(anon_sym_in);
      END_STATE();
    case 47:
      if (lookahead == 'i') ADVANCE(100);
      END_STATE();
    case 48:
      if (lookahead == 'e') ADVANCE(101);
      END_STATE();
    case 49:
      if (lookahead == 't') ADVANCE(102);
      END_STATE();
    case 50:
      if (lookahead == 'f') ADVANCE(103);
      if (lookahead == 't') ADVANCE(104);
      END_STATE();
    case 51:
      if (lookahead == 'o') ADVANCE(105);
      END_STATE();
    case 52:
      if (lookahead == 't') ADVANCE(106);
      END_STATE();
    case 53:
      if (lookahead == 't') ADVANCE(107);
      END_STATE();
    case 54:
      if (lookahead == 'd') ADVANCE(108);
      if (lookahead == 'v') ADVANCE(109);
      END_STATE();
    case 55:
      if (lookahead == 't') ADVANCE(110);
      END_STATE();
    case 56:
      if (lookahead == 't') ADVANCE(111);
      END_STATE();
    case 57:
      if (lookahead == 'b') ADVANCE(112);
      END_STATE();
    case 58:
      if (lookahead == 'w') ADVANCE(113);
      END_STATE();
    case 59:
      if (lookahead == 'f') ADVANCE(114);
      if (lookahead == 't') ADVANCE(115);
      END_STATE();
    case 60:
      if (lookahead == 'l') ADVANCE(116);
      END_STATE();
    case 61:
      if (lookahead == 'a') ADVANCE(117);
      if (lookahead == 'm') ADVANCE(118);
      if (lookahead == 'r') ADVANCE(119);
      END_STATE();
    case 62:
      if (lookahead == 'p') ADVANCE(120);
      END_STATE();
    case 63:
      if (lookahead == 'a') ADVANCE(121);
      if (lookahead == 'u') ADVANCE(122);
      if (lookahead == 'y') ADVANCE(123);
      END_STATE();
    case 64:
      ACCEPT_TOKEN(anon_sym_tt);
      END_STATE();
    case 65:
      ACCEPT_TOKEN(anon_sym_ty);
      if (lookahead == 'p') ADVANCE(124);
      END_STATE();
    case 66:
      if (lookahead == '2') ADVANCE(125);
      if (lookahead == '6') ADVANCE(126);
      END_STATE();
    case 67:
      if (lookahead == '2') ADVANCE(127);
      END_STATE();
    case 68:
      if (lookahead == '4') ADVANCE(128);
      END_STATE();
    case 69:
      ACCEPT_TOKEN(anon_sym_u8);
      END_STATE();
    case 70:
      if (lookahead == 'i') ADVANCE(129);
      if (lookahead == 's') ADVANCE(130);
      END_STATE();
    case 71:
      if (lookahead == 'e') ADVANCE(131);
      if (lookahead == 'i') ADVANCE(132);
      END_STATE();
    case 72:
      if (lookahead == 's') ADVANCE(133);
      END_STATE();
    case 73:
      if (lookahead == 'e') ADVANCE(134);
      if (lookahead == 'i') ADVANCE(135);
      END_STATE();
    case 74:
      if (lookahead == 'e') ADVANCE(136);
      END_STATE();
    case 75:
      if (lookahead == 'n') ADVANCE(137);
      END_STATE();
    case 76:
      if (lookahead == 'i') ADVANCE(138);
      END_STATE();
    case 77:
      if (lookahead == 'c') ADVANCE(139);
      END_STATE();
    case 78:
      if (lookahead == 'l') ADVANCE(140);
      END_STATE();
    case 79:
      if (lookahead == 'a') ADVANCE(141);
      END_STATE();
    case 80:
      if (lookahead == 'r') ADVANCE(142);
      END_STATE();
    case 81:
      if (lookahead == 's') ADVANCE(143);
      if (lookahead == 't') ADVANCE(144);
      END_STATE();
    case 82:
      if (lookahead == 't') ADVANCE(145);
      END_STATE();
    case 83:
      if (lookahead == 'a') ADVANCE(146);
      END_STATE();
    case 84:
      ACCEPT_TOKEN(anon_sym_dyn);
      END_STATE();
    case 85:
      if (lookahead == 'e') ADVANCE(147);
      END_STATE();
    case 86:
      if (lookahead == 'm') ADVANCE(148);
      END_STATE();
    case 87:
      if (lookahead == 'r') ADVANCE(149);
      END_STATE();
    case 88:
      if (lookahead == 'e') ADVANCE(150);
      END_STATE();
    case 89:
      ACCEPT_TOKEN(anon_sym_f32);
      END_STATE();
    case 90:
      ACCEPT_TOKEN(anon_sym_f64);
      END_STATE();
    case 91:
      if (lookahead == 's') ADVANCE(151);
      END_STATE();
    case 92:
      ACCEPT_TOKEN(anon_sym_for);
      END_STATE();
    case 93:
      ACCEPT_TOKEN(anon_sym_gen);
      END_STATE();
    case 94:
      if (lookahead == '8') ADVANCE(152);
      END_STATE();
    case 95:
      ACCEPT_TOKEN(anon_sym_i16);
      END_STATE();
    case 96:
      ACCEPT_TOKEN(anon_sym_i32);
      END_STATE();
    case 97:
      ACCEPT_TOKEN(anon_sym_i64);
      END_STATE();
    case 98:
      if (lookahead == 'n') ADVANCE(153);
      END_STATE();
    case 99:
      if (lookahead == 'l') ADVANCE(154);
      END_STATE();
    case 100:
      if (lookahead == 'z') ADVANCE(155);
      END_STATE();
    case 101:
      if (lookahead == 'm') ADVANCE(156);
      END_STATE();
    case 102:
      ACCEPT_TOKEN(anon_sym_let);
      END_STATE();
    case 103:
      if (lookahead == 'e') ADVANCE(157);
      END_STATE();
    case 104:
      if (lookahead == 'e') ADVANCE(158);
      END_STATE();
    case 105:
      if (lookahead == 'p') ADVANCE(159);
      END_STATE();
    case 106:
      if (lookahead == 'c') ADVANCE(160);
      END_STATE();
    case 107:
      if (lookahead == 'a') ADVANCE(161);
      END_STATE();
    case 108:
      ACCEPT_TOKEN(anon_sym_mod);
      END_STATE();
    case 109:
      if (lookahead == 'e') ADVANCE(162);
      END_STATE();
    case 110:
      ACCEPT_TOKEN(sym_mutable_specifier);
      END_STATE();
    case 111:
      ACCEPT_TOKEN(anon_sym_pat);
      if (lookahead == 'h') ADVANCE(163);
      END_STATE();
    case 112:
      ACCEPT_TOKEN(anon_sym_pub);
      END_STATE();
    case 113:
      ACCEPT_TOKEN(anon_sym_raw);
      END_STATE();
    case 114:
      ACCEPT_TOKEN(anon_sym_ref);
      END_STATE();
    case 115:
      if (lookahead == 'u') ADVANCE(164);
      END_STATE();
    case 116:
      if (lookahead == 'f') ADVANCE(165);
      END_STATE();
    case 117:
      if (lookahead == 't') ADVANCE(166);
      END_STATE();
    case 118:
      if (lookahead == 't') ADVANCE(167);
      END_STATE();
    case 119:
      ACCEPT_TOKEN(anon_sym_str);
      if (lookahead == 'u') ADVANCE(168);
      END_STATE();
    case 120:
      if (lookahead == 'e') ADVANCE(169);
      END_STATE();
    case 121:
      if (lookahead == 'i') ADVANCE(170);
      END_STATE();
    case 122:
      if (lookahead == 'e') ADVANCE(171);
      END_STATE();
    case 123:
      ACCEPT_TOKEN(anon_sym_try);
      END_STATE();
    case 124:
      if (lookahead == 'e') ADVANCE(172);
      END_STATE();
    case 125:
      if (lookahead == '8') ADVANCE(173);
      END_STATE();
    case 126:
      ACCEPT_TOKEN(anon_sym_u16);
      END_STATE();
    case 127:
      ACCEPT_TOKEN(anon_sym_u32);
      END_STATE();
    case 128:
      ACCEPT_TOKEN(anon_sym_u64);
      END_STATE();
    case 129:
      if (lookahead == 'o') ADVANCE(174);
      END_STATE();
    case 130:
      if (lookahead == 'a') ADVANCE(175);
      END_STATE();
    case 131:
      ACCEPT_TOKEN(anon_sym_use);
      END_STATE();
    case 132:
      if (lookahead == 'z') ADVANCE(176);
      END_STATE();
    case 133:
      ACCEPT_TOKEN(anon_sym_vis);
      END_STATE();
    case 134:
      if (lookahead == 'r') ADVANCE(177);
      END_STATE();
    case 135:
      if (lookahead == 'l') ADVANCE(178);
      END_STATE();
    case 136:
      if (lookahead == 'l') ADVANCE(179);
      END_STATE();
    case 137:
      if (lookahead == 'c') ADVANCE(180);
      END_STATE();
    case 138:
      if (lookahead == 't') ADVANCE(181);
      END_STATE();
    case 139:
      if (lookahead == 'k') ADVANCE(182);
      END_STATE();
    case 140:
      ACCEPT_TOKEN(anon_sym_bool);
      END_STATE();
    case 141:
      if (lookahead == 'k') ADVANCE(183);
      END_STATE();
    case 142:
      ACCEPT_TOKEN(anon_sym_char);
      END_STATE();
    case 143:
      if (lookahead == 't') ADVANCE(184);
      END_STATE();
    case 144:
      if (lookahead == 'i') ADVANCE(185);
      END_STATE();
    case 145:
      if (lookahead == 'e') ADVANCE(186);
      END_STATE();
    case 146:
      if (lookahead == 'u') ADVANCE(187);
      END_STATE();
    case 147:
      ACCEPT_TOKEN(anon_sym_else);
      END_STATE();
    case 148:
      ACCEPT_TOKEN(anon_sym_enum);
      END_STATE();
    case 149:
      ACCEPT_TOKEN(anon_sym_expr);
      END_STATE();
    case 150:
      if (lookahead == 'r') ADVANCE(188);
      END_STATE();
    case 151:
      if (lookahead == 'e') ADVANCE(189);
      END_STATE();
    case 152:
      ACCEPT_TOKEN(anon_sym_i128);
      END_STATE();
    case 153:
      if (lookahead == 't') ADVANCE(190);
      END_STATE();
    case 154:
      ACCEPT_TOKEN(anon_sym_impl);
      END_STATE();
    case 155:
      if (lookahead == 'e') ADVANCE(191);
      END_STATE();
    case 156:
      ACCEPT_TOKEN(anon_sym_item);
      END_STATE();
    case 157:
      if (lookahead == 't') ADVANCE(192);
      END_STATE();
    case 158:
      if (lookahead == 'r') ADVANCE(193);
      END_STATE();
    case 159:
      ACCEPT_TOKEN(anon_sym_loop);
      END_STATE();
    case 160:
      if (lookahead == 'h') ADVANCE(194);
      END_STATE();
    case 161:
      ACCEPT_TOKEN(anon_sym_meta);
      END_STATE();
    case 162:
      ACCEPT_TOKEN(anon_sym_move);
      END_STATE();
    case 163:
      ACCEPT_TOKEN(anon_sym_path);
      END_STATE();
    case 164:
      if (lookahead == 'r') ADVANCE(195);
      END_STATE();
    case 165:
      ACCEPT_TOKEN(sym_self);
      END_STATE();
    case 166:
      if (lookahead == 'i') ADVANCE(196);
      END_STATE();
    case 167:
      ACCEPT_TOKEN(anon_sym_stmt);
      END_STATE();
    case 168:
      if (lookahead == 'c') ADVANCE(197);
      END_STATE();
    case 169:
      if (lookahead == 'r') ADVANCE(198);
      END_STATE();
    case 170:
      if (lookahead == 't') ADVANCE(199);
      END_STATE();
    case 171:
      ACCEPT_TOKEN(anon_sym_true);
      END_STATE();
    case 172:
      ACCEPT_TOKEN(anon_sym_type);
      END_STATE();
    case 173:
      ACCEPT_TOKEN(anon_sym_u128);
      END_STATE();
    case 174:
      if (lookahead == 'n') ADVANCE(200);
      END_STATE();
    case 175:
      if (lookahead == 'f') ADVANCE(201);
      END_STATE();
    case 176:
      if (lookahead == 'e') ADVANCE(202);
      END_STATE();
    case 177:
      if (lookahead == 'e') ADVANCE(203);
      END_STATE();
    case 178:
      if (lookahead == 'e') ADVANCE(204);
      END_STATE();
    case 179:
      if (lookahead == 'd') ADVANCE(205);
      END_STATE();
    case 180:
      ACCEPT_TOKEN(anon_sym_async);
      END_STATE();
    case 181:
      ACCEPT_TOKEN(anon_sym_await);
      END_STATE();
    case 182:
      ACCEPT_TOKEN(anon_sym_block);
      END_STATE();
    case 183:
      ACCEPT_TOKEN(anon_sym_break);
      END_STATE();
    case 184:
      ACCEPT_TOKEN(anon_sym_const);
      END_STATE();
    case 185:
      if (lookahead == 'n') ADVANCE(206);
      END_STATE();
    case 186:
      ACCEPT_TOKEN(sym_crate);
      END_STATE();
    case 187:
      if (lookahead == 'l') ADVANCE(207);
      END_STATE();
    case 188:
      if (lookahead == 'n') ADVANCE(208);
      END_STATE();
    case 189:
      ACCEPT_TOKEN(anon_sym_false);
      END_STATE();
    case 190:
      ACCEPT_TOKEN(anon_sym_ident);
      END_STATE();
    case 191:
      ACCEPT_TOKEN(anon_sym_isize);
      END_STATE();
    case 192:
      if (lookahead == 'i') ADVANCE(209);
      END_STATE();
    case 193:
      if (lookahead == 'a') ADVANCE(210);
      END_STATE();
    case 194:
      ACCEPT_TOKEN(anon_sym_match);
      END_STATE();
    case 195:
      if (lookahead == 'n') ADVANCE(211);
      END_STATE();
    case 196:
      if (lookahead == 'c') ADVANCE(212);
      END_STATE();
    case 197:
      if (lookahead == 't') ADVANCE(213);
      END_STATE();
    case 198:
      ACCEPT_TOKEN(sym_super);
      END_STATE();
    case 199:
      ACCEPT_TOKEN(anon_sym_trait);
      END_STATE();
    case 200:
      ACCEPT_TOKEN(anon_sym_union);
      END_STATE();
    case 201:
      if (lookahead == 'e') ADVANCE(214);
      END_STATE();
    case 202:
      ACCEPT_TOKEN(anon_sym_usize);
      END_STATE();
    case 203:
      ACCEPT_TOKEN(anon_sym_where);
      END_STATE();
    case 204:
      ACCEPT_TOKEN(anon_sym_while);
      END_STATE();
    case 205:
      ACCEPT_TOKEN(anon_sym_yield);
      END_STATE();
    case 206:
      if (lookahead == 'u') ADVANCE(215);
      END_STATE();
    case 207:
      if (lookahead == 't') ADVANCE(216);
      END_STATE();
    case 208:
      ACCEPT_TOKEN(anon_sym_extern);
      END_STATE();
    case 209:
      if (lookahead == 'm') ADVANCE(217);
      END_STATE();
    case 210:
      if (lookahead == 'l') ADVANCE(218);
      END_STATE();
    case 211:
      ACCEPT_TOKEN(anon_sym_return);
      END_STATE();
    case 212:
      ACCEPT_TOKEN(anon_sym_static);
      END_STATE();
    case 213:
      ACCEPT_TOKEN(anon_sym_struct);
      END_STATE();
    case 214:
      ACCEPT_TOKEN(anon_sym_unsafe);
      END_STATE();
    case 215:
      if (lookahead == 'e') ADVANCE(219);
      END_STATE();
    case 216:
      ACCEPT_TOKEN(anon_sym_default);
      END_STATE();
    case 217:
      if (lookahead == 'e') ADVANCE(220);
      END_STATE();
    case 218:
      ACCEPT_TOKEN(anon_sym_literal);
      END_STATE();
    case 219:
      ACCEPT_TOKEN(anon_sym_continue);
      END_STATE();
    case 220:
      ACCEPT_TOKEN(anon_sym_lifetime);
      END_STATE();
    default:
      return false;
  }
}

static const TSLexMode ts_lex_modes[STATE_COUNT] = {
  [0] = {.lex_state = 0, .external_lex_state = 1},
  [1] = {.lex_state = 70, .external_lex_state = 2},
  [2] = {.lex_state = 71, .external_lex_state = 2},
  [3] = {.lex_state = 71, .external_lex_state = 2},
  [4] = {.lex_state = 71, .external_lex_state = 2},
  [5] = {.lex_state = 71, .external_lex_state = 2},
  [6] = {.lex_state = 71, .external_lex_state = 2},
  [7] = {.lex_state = 71, .external_lex_state = 2},
  [8] = {.lex_state = 71, .external_lex_state = 2},
  [9] = {.lex_state = 71, .external_lex_state = 2},
  [10] = {.lex_state = 71, .external_lex_state = 2},
  [11] = {.lex_state = 71, .external_lex_state = 2},
  [12] = {.lex_state = 71, .external_lex_state = 2},
  [13] = {.lex_state = 71, .external_lex_state = 2},
  [14] = {.lex_state = 71, .external_lex_state = 2},
  [15] = {.lex_state = 71, .external_lex_state = 2},
  [16] = {.lex_state = 71, .external_lex_state = 2},
  [17] = {.lex_state = 71, .external_lex_state = 2},
  [18] = {.lex_state = 71, .external_lex_state = 2},
  [19] = {.lex_state = 71, .external_lex_state = 2},
  [20] = {.lex_state = 71, .external_lex_state = 2},
  [21] = {.lex_state = 71, .external_lex_state = 2},
  [22] = {.lex_state = 71, .external_lex_state = 2},
  [23] = {.lex_state = 71, .external_lex_state = 2},
  [24] = {.lex_state = 71, .external_lex_state = 2},
  [25] = {.lex_state = 71, .external_lex_state = 2},
  [26] = {.lex_state = 71, .external_lex_state = 2},
  [27] = {.lex_state = 71, .external_lex_state = 2},
  [28] = {.lex_state = 71, .external_lex_state = 2},
  [29] = {.lex_state = 71, .external_lex_state = 2},
  [30] = {.lex_state = 71, .external_lex_state = 2},
  [31] = {.lex_state = 71, .external_lex_state = 2},
  [32] = {.lex_state = 71, .external_lex_state = 2},
  [33] = {.lex_state = 71, .external_lex_state = 2},
  [34] = {.lex_state = 71, .external_lex_state = 2},
  [35] = {.lex_state = 3, .external_lex_state = 2},
  [36] = {.lex_state = 3, .external_lex_state = 2},
  [37] = {.lex_state = 3, .external_lex_state = 2},
  [38] = {.lex_state = 3, .external_lex_state = 2},
  [39] = {.lex_state = 3, .external_lex_state = 2},
  [40] = {.lex_state = 3, .external_lex_state = 2},
  [41] = {.lex_state = 3, .external_lex_state = 2},
  [42] = {.lex_state = 3, .external_lex_state = 2},
  [43] = {.lex_state = 5, .external_lex_state = 2},
  [44] = {.lex_state = 5, .external_lex_state = 2},
  [45] = {.lex_state = 3, .external_lex_state = 2},
  [46] = {.lex_state = 5, .external_lex_state = 2},
  [47] = {.lex_state = 5, .external_lex_state = 2},
  [48] = {.lex_state = 5, .external_lex_state = 2},
  [49] = {.lex_state = 5, .external_lex_state = 2},
  [50] = {.lex_state = 5, .external_lex_state = 2},
  [51] = {.lex_state = 5, .external_lex_state = 2},
  [52] = {.lex_state = 3, .external_lex_state = 2},
  [53] = {.lex_state = 3, .external_lex_state = 2},
  [54] = {.lex_state = 3, .external_lex_state = 2},
  [55] = {.lex_state = 3, .external_lex_state = 2},
  [56] = {.lex_state = 3, .external_lex_state = 2},
  [57] = {.lex_state = 3, .external_lex_state = 2},
  [58] = {.lex_state = 3, .external_lex_state = 2},
  [59] = {.lex_state = 3, .external_lex_state = 2},
  [60] = {.lex_state = 3, .external_lex_state = 2},
  [61] = {.lex_state = 3, .external_lex_state = 2},
  [62] = {.lex_state = 3, .external_lex_state = 2},
  [63] = {.lex_state = 3, .external_lex_state = 2},
  [64] = {.lex_state = 3, .external_lex_state = 2},
  [65] = {.lex_state = 3, .external_lex_state = 2},
  [66] = {.lex_state = 3, .external_lex_state = 2},
  [67] = {.lex_state = 2, .external_lex_state = 2},
  [68] = {.lex_state = 2, .external_lex_state = 2},
  [69] = {.lex_state = 2, .external_lex_state = 2},
  [70] = {.lex_state = 2, .external_lex_state = 2},
  [71] = {.lex_state = 2, .external_lex_state = 2},
  [72] = {.lex_state = 2, .external_lex_state = 2},
  [73] = {.lex_state = 2, .external_lex_state = 2},
  [74] = {.lex_state = 2, .external_lex_state = 2},
  [75] = {.lex_state = 2, .external_lex_state = 2},
  [76] = {.lex_state = 2, .external_lex_state = 2},
  [77] = {.lex_state = 2, .external_lex_state = 2},
  [78] = {.lex_state = 2, .external_lex_state = 2},
  [79] = {.lex_state = 2, .external_lex_state = 2},
  [80] = {.lex_state = 2, .external_lex_state = 2},
  [81] = {.lex_state = 2, .external_lex_state = 2},
  [82] = {.lex_state = 2, .external_lex_state = 2},
  [83] = {.lex_state = 4, .external_lex_state = 2},
  [84] = {.lex_state = 4, .external_lex_state = 2},
  [85] = {.lex_state = 4, .external_lex_state = 2},
  [86] = {.lex_state = 4, .external_lex_state = 2},
  [87] = {.lex_state = 4, .external_lex_state = 2},
  [88] = {.lex_state = 2, .external_lex_state = 2},
  [89] = {.lex_state = 2, .external_lex_state = 2},
  [90] = {.lex_state = 2, .external_lex_state = 2},
  [91] = {.lex_state = 2, .external_lex_state = 2},
  [92] = {.lex_state = 2, .external_lex_state = 2},
  [93] = {.lex_state = 2, .external_lex_state = 2},
  [94] = {.lex_state = 2, .external_lex_state = 2},
  [95] = {.lex_state = 2, .external_lex_state = 2},
  [96] = {.lex_state = 2, .external_lex_state = 2},
  [97] = {.lex_state = 4, .external_lex_state = 2},
  [98] = {.lex_state = 2, .external_lex_state = 2},
  [99] = {.lex_state = 4, .external_lex_state = 2},
  [100] = {.lex_state = 4, .external_lex_state = 2},
  [101] = {.lex_state = 4, .external_lex_state = 2},
  [102] = {.lex_state = 4, .external_lex_state = 2},
  [103] = {.lex_state = 4, .external_lex_state = 2},
  [104] = {.lex_state = 4, .external_lex_state = 2},
  [105] = {.lex_state = 4, .external_lex_state = 2},
  [106] = {.lex_state = 4, .external_lex_state = 2},
  [107] = {.lex_state = 4, .external_lex_state = 2},
  [108] = {.lex_state = 4, .external_lex_state = 2},
  [109] = {.lex_state = 4, .external_lex_state = 2},
  [110] = {.lex_state = 4, .external_lex_state = 2},
  [111] = {.lex_state = 4, .external_lex_state = 2},
  [112] = {.lex_state = 4, .external_lex_state = 2},
  [113] = {.lex_state = 4, .external_lex_state = 2},
  [114] = {.lex_state = 4, .external_lex_state = 2},
  [115] = {.lex_state = 4, .external_lex_state = 2},
  [116] = {.lex_state = 4, .external_lex_state = 2},
  [117] = {.lex_state = 4, .external_lex_state = 2},
  [118] = {.lex_state = 4, .external_lex_state = 2},
  [119] = {.lex_state = 4, .external_lex_state = 2},
  [120] = {.lex_state = 4, .external_lex_state = 2},
  [121] = {.lex_state = 2, .external_lex_state = 2},
  [122] = {.lex_state = 4, .external_lex_state = 2},
  [123] = {.lex_state = 4, .external_lex_state = 2},
  [124] = {.lex_state = 4, .external_lex_state = 2},
  [125] = {.lex_state = 4, .external_lex_state = 2},
  [126] = {.lex_state = 4, .external_lex_state = 2},
  [127] = {.lex_state = 4, .external_lex_state = 2},
  [128] = {.lex_state = 4, .external_lex_state = 2},
  [129] = {.lex_state = 4, .external_lex_state = 2},
  [130] = {.lex_state = 2, .external_lex_state = 2},
  [131] = {.lex_state = 2, .external_lex_state = 2},
  [132] = {.lex_state = 4, .external_lex_state = 2},
  [133] = {.lex_state = 2, .external_lex_state = 2},
  [134] = {.lex_state = 11, .external_lex_state = 2},
  [135] = {.lex_state = 11, .external_lex_state = 2},
  [136] = {.lex_state = 2, .external_lex_state = 2},
  [137] = {.lex_state = 2, .external_lex_state = 2},
  [138] = {.lex_state = 11, .external_lex_state = 2},
  [139] = {.lex_state = 11, .external_lex_state = 2},
  [140] = {.lex_state = 11, .external_lex_state = 2},
  [141] = {.lex_state = 11, .external_lex_state = 2},
  [142] = {.lex_state = 11, .external_lex_state = 2},
  [143] = {.lex_state = 2, .external_lex_state = 2},
  [144] = {.lex_state = 11, .external_lex_state = 2},
  [145] = {.lex_state = 11, .external_lex_state = 2},
  [146] = {.lex_state = 11, .external_lex_state = 2},
  [147] = {.lex_state = 11, .external_lex_state = 2},
  [148] = {.lex_state = 11, .external_lex_state = 2},
  [149] = {.lex_state = 11, .external_lex_state = 2},
  [150] = {.lex_state = 2, .external_lex_state = 2},
  [151] = {.lex_state = 2, .external_lex_state = 2},
  [152] = {.lex_state = 2, .external_lex_state = 2},
  [153] = {.lex_state = 11, .external_lex_state = 2},
  [154] = {.lex_state = 11, .external_lex_state = 2},
  [155] = {.lex_state = 2, .external_lex_state = 2},
  [156] = {.lex_state = 2, .external_lex_state = 2},
  [157] = {.lex_state = 4, .external_lex_state = 2},
  [158] = {.lex_state = 11, .external_lex_state = 2},
  [159] = {.lex_state = 2, .external_lex_state = 2},
  [160] = {.lex_state = 2, .external_lex_state = 2},
  [161] = {.lex_state = 2, .external_lex_state = 2},
  [162] = {.lex_state = 11, .external_lex_state = 2},
  [163] = {.lex_state = 2, .external_lex_state = 2},
  [164] = {.lex_state = 11, .external_lex_state = 2},
  [165] = {.lex_state = 11, .external_lex_state = 2},
  [166] = {.lex_state = 11, .external_lex_state = 2},
  [167] = {.lex_state = 11, .external_lex_state = 2},
  [168] = {.lex_state = 11, .external_lex_state = 2},
  [169] = {.lex_state = 11, .external_lex_state = 2},
  [170] = {.lex_state = 2, .external_lex_state = 2},
  [171] = {.lex_state = 2, .external_lex_state = 2},
  [172] = {.lex_state = 2, .external_lex_state = 2},
  [173] = {.lex_state = 2, .external_lex_state = 2},
  [174] = {.lex_state = 2, .external_lex_state = 2},
  [175] = {.lex_state = 2, .external_lex_state = 2},
  [176] = {.lex_state = 2, .external_lex_state = 2},
  [177] = {.lex_state = 2, .external_lex_state = 2},
  [178] = {.lex_state = 11, .external_lex_state = 2},
  [179] = {.lex_state = 4, .external_lex_state = 2},
  [180] = {.lex_state = 2, .external_lex_state = 2},
  [181] = {.lex_state = 11, .external_lex_state = 2},
  [182] = {.lex_state = 2, .external_lex_state = 2},
  [183] = {.lex_state = 11, .external_lex_state = 2},
  [184] = {.lex_state = 2, .external_lex_state = 2},
  [185] = {.lex_state = 2, .external_lex_state = 2},
  [186] = {.lex_state = 11, .external_lex_state = 2},
  [187] = {.lex_state = 11, .external_lex_state = 2},
  [188] = {.lex_state = 11, .external_lex_state = 2},
  [189] = {.lex_state = 11, .external_lex_state = 2},
  [190] = {.lex_state = 11, .external_lex_state = 2},
  [191] = {.lex_state = 11, .external_lex_state = 2},
  [192] = {.lex_state = 11, .external_lex_state = 2},
  [193] = {.lex_state = 11, .external_lex_state = 2},
  [194] = {.lex_state = 11, .external_lex_state = 2},
  [195] = {.lex_state = 11, .external_lex_state = 2},
  [196] = {.lex_state = 11, .external_lex_state = 2},
  [197] = {.lex_state = 2, .external_lex_state = 2},
  [198] = {.lex_state = 4, .external_lex_state = 2},
  [199] = {.lex_state = 4, .external_lex_state = 2},
  [200] = {.lex_state = 4, .external_lex_state = 2},
  [201] = {.lex_state = 11, .external_lex_state = 2},
  [202] = {.lex_state = 4, .external_lex_state = 2},
  [203] = {.lex_state = 4, .external_lex_state = 2},
  [204] = {.lex_state = 4, .external_lex_state = 2},
  [205] = {.lex_state = 11, .external_lex_state = 2},
  [206] = {.lex_state = 4, .external_lex_state = 2},
  [207] = {.lex_state = 11, .external_lex_state = 2},
  [208] = {.lex_state = 11, .external_lex_state = 2},
  [209] = {.lex_state = 4, .external_lex_state = 2},
  [210] = {.lex_state = 4, .external_lex_state = 2},
  [211] = {.lex_state = 11, .external_lex_state = 2},
  [212] = {.lex_state = 11, .external_lex_state = 2},
  [213] = {.lex_state = 4, .external_lex_state = 2},
  [214] = {.lex_state = 4, .external_lex_state = 2},
  [215] = {.lex_state = 11, .external_lex_state = 2},
  [216] = {.lex_state = 11, .external_lex_state = 2},
  [217] = {.lex_state = 11, .external_lex_state = 2},
  [218] = {.lex_state = 11, .external_lex_state = 2},
  [219] = {.lex_state = 13, .external_lex_state = 2},
  [220] = {.lex_state = 13, .external_lex_state = 2},
  [221] = {.lex_state = 11, .external_lex_state = 2},
  [222] = {.lex_state = 11, .external_lex_state = 2},
  [223] = {.lex_state = 13, .external_lex_state = 2},
  [224] = {.lex_state = 11, .external_lex_state = 2},
  [225] = {.lex_state = 13, .external_lex_state = 2},
  [226] = {.lex_state = 11, .external_lex_state = 2},
  [227] = {.lex_state = 13, .external_lex_state = 2},
  [228] = {.lex_state = 11, .external_lex_state = 2},
  [229] = {.lex_state = 11, .external_lex_state = 2},
  [230] = {.lex_state = 13, .external_lex_state = 2},
  [231] = {.lex_state = 13, .external_lex_state = 2},
  [232] = {.lex_state = 13, .external_lex_state = 2},
  [233] = {.lex_state = 13, .external_lex_state = 2},
  [234] = {.lex_state = 13, .external_lex_state = 2},
  [235] = {.lex_state = 13, .external_lex_state = 2},
  [236] = {.lex_state = 11, .external_lex_state = 2},
  [237] = {.lex_state = 11, .external_lex_state = 2},
  [238] = {.lex_state = 11, .external_lex_state = 2},
  [239] = {.lex_state = 13, .external_lex_state = 2},
  [240] = {.lex_state = 11, .external_lex_state = 2},
  [241] = {.lex_state = 11, .external_lex_state = 2},
  [242] = {.lex_state = 13, .external_lex_state = 2},
  [243] = {.lex_state = 13, .external_lex_state = 2},
  [244] = {.lex_state = 11, .external_lex_state = 2},
  [245] = {.lex_state = 11, .external_lex_state = 2},
  [246] = {.lex_state = 13, .external_lex_state = 2},
  [247] = {.lex_state = 11, .external_lex_state = 2},
  [248] = {.lex_state = 11, .external_lex_state = 2},
  [249] = {.lex_state = 11, .external_lex_state = 2},
  [250] = {.lex_state = 11, .external_lex_state = 2},
  [251] = {.lex_state = 11, .external_lex_state = 2},
  [252] = {.lex_state = 11, .external_lex_state = 2},
  [253] = {.lex_state = 11, .external_lex_state = 2},
  [254] = {.lex_state = 11, .external_lex_state = 2},
  [255] = {.lex_state = 11, .external_lex_state = 2},
  [256] = {.lex_state = 11, .external_lex_state = 2},
  [257] = {.lex_state = 11, .external_lex_state = 2},
  [258] = {.lex_state = 11, .external_lex_state = 2},
  [259] = {.lex_state = 11, .external_lex_state = 2},
  [260] = {.lex_state = 11, .external_lex_state = 2},
  [261] = {.lex_state = 11, .external_lex_state = 2},
  [262] = {.lex_state = 11, .external_lex_state = 2},
  [263] = {.lex_state = 11, .external_lex_state = 2},
  [264] = {.lex_state = 11, .external_lex_state = 2},
  [265] = {.lex_state = 11, .external_lex_state = 2},
  [266] = {.lex_state = 11, .external_lex_state = 2},
  [267] = {.lex_state = 11, .external_lex_state = 2},
  [268] = {.lex_state = 11, .external_lex_state = 2},
  [269] = {.lex_state = 11, .external_lex_state = 2},
  [270] = {.lex_state = 11, .external_lex_state = 2},
  [271] = {.lex_state = 11, .external_lex_state = 2},
  [272] = {.lex_state = 11, .external_lex_state = 2},
  [273] = {.lex_state = 11, .external_lex_state = 2},
  [274] = {.lex_state = 11, .external_lex_state = 2},
  [275] = {.lex_state = 11, .external_lex_state = 2},
  [276] = {.lex_state = 11, .external_lex_state = 2},
  [277] = {.lex_state = 11, .external_lex_state = 2},
  [278] = {.lex_state = 11, .external_lex_state = 2},
  [279] = {.lex_state = 11, .external_lex_state = 2},
  [280] = {.lex_state = 11, .external_lex_state = 2},
  [281] = {.lex_state = 11, .external_lex_state = 2},
  [282] = {.lex_state = 11, .external_lex_state = 2},
  [283] = {.lex_state = 11, .external_lex_state = 2},
  [284] = {.lex_state = 11, .external_lex_state = 2},
  [285] = {.lex_state = 11, .external_lex_state = 2},
  [286] = {.lex_state = 11, .external_lex_state = 2},
  [287] = {.lex_state = 11, .external_lex_state = 2},
  [288] = {.lex_state = 11, .external_lex_state = 2},
  [289] = {.lex_state = 11, .external_lex_state = 2},
  [290] = {.lex_state = 11, .external_lex_state = 2},
  [291] = {.lex_state = 11, .external_lex_state = 2},
  [292] = {.lex_state = 11, .external_lex_state = 2},
  [293] = {.lex_state = 11, .external_lex_state = 2},
  [294] = {.lex_state = 11, .external_lex_state = 2},
  [295] = {.lex_state = 11, .external_lex_state = 2},
  [296] = {.lex_state = 11, .external_lex_state = 2},
  [297] = {.lex_state = 11, .external_lex_state = 2},
  [298] = {.lex_state = 11, .external_lex_state = 2},
  [299] = {.lex_state = 11, .external_lex_state = 2},
  [300] = {.lex_state = 11, .external_lex_state = 2},
  [301] = {.lex_state = 11, .external_lex_state = 2},
  [302] = {.lex_state = 11, .external_lex_state = 2},
  [303] = {.lex_state = 11, .external_lex_state = 2},
  [304] = {.lex_state = 11, .external_lex_state = 2},
  [305] = {.lex_state = 11, .external_lex_state = 2},
  [306] = {.lex_state = 11, .external_lex_state = 2},
  [307] = {.lex_state = 11, .external_lex_state = 2},
  [308] = {.lex_state = 11, .external_lex_state = 2},
  [309] = {.lex_state = 11, .external_lex_state = 2},
  [310] = {.lex_state = 11, .external_lex_state = 2},
  [311] = {.lex_state = 11, .external_lex_state = 2},
  [312] = {.lex_state = 11, .external_lex_state = 2},
  [313] = {.lex_state = 11, .external_lex_state = 2},
  [314] = {.lex_state = 11, .external_lex_state = 2},
  [315] = {.lex_state = 11, .external_lex_state = 2},
  [316] = {.lex_state = 11, .external_lex_state = 2},
  [317] = {.lex_state = 11, .external_lex_state = 2},
  [318] = {.lex_state = 11, .external_lex_state = 2},
  [319] = {.lex_state = 11, .external_lex_state = 2},
  [320] = {.lex_state = 11, .external_lex_state = 2},
  [321] = {.lex_state = 11, .external_lex_state = 2},
  [322] = {.lex_state = 11, .external_lex_state = 2},
  [323] = {.lex_state = 11, .external_lex_state = 2},
  [324] = {.lex_state = 11, .external_lex_state = 2},
  [325] = {.lex_state = 11, .external_lex_state = 2},
  [326] = {.lex_state = 11, .external_lex_state = 2},
  [327] = {.lex_state = 11, .external_lex_state = 2},
  [328] = {.lex_state = 11, .external_lex_state = 2},
  [329] = {.lex_state = 11, .external_lex_state = 2},
  [330] = {.lex_state = 11, .external_lex_state = 2},
  [331] = {.lex_state = 11, .external_lex_state = 2},
  [332] = {.lex_state = 69, .external_lex_state = 2},
  [333] = {.lex_state = 11, .external_lex_state = 2},
  [334] = {.lex_state = 11, .external_lex_state = 2},
  [335] = {.lex_state = 11, .external_lex_state = 2},
  [336] = {.lex_state = 11, .external_lex_state = 2},
  [337] = {.lex_state = 11, .external_lex_state = 2},
  [338] = {.lex_state = 11, .external_lex_state = 2},
  [339] = {.lex_state = 11, .external_lex_state = 2},
  [340] = {.lex_state = 11, .external_lex_state = 2},
  [341] = {.lex_state = 11, .external_lex_state = 2},
  [342] = {.lex_state = 11, .external_lex_state = 2},
  [343] = {.lex_state = 11, .external_lex_state = 2},
  [344] = {.lex_state = 11, .external_lex_state = 2},
  [345] = {.lex_state = 11, .external_lex_state = 2},
  [346] = {.lex_state = 11, .external_lex_state = 2},
  [347] = {.lex_state = 11, .external_lex_state = 2},
  [348] = {.lex_state = 11, .external_lex_state = 2},
  [349] = {.lex_state = 11, .external_lex_state = 2},
  [350] = {.lex_state = 11, .external_lex_state = 2},
  [351] = {.lex_state = 11, .external_lex_state = 2},
  [352] = {.lex_state = 11, .external_lex_state = 2},
  [353] = {.lex_state = 11, .external_lex_state = 2},
  [354] = {.lex_state = 11, .external_lex_state = 2},
  [355] = {.lex_state = 11, .external_lex_state = 2},
  [356] = {.lex_state = 11, .external_lex_state = 2},
  [357] = {.lex_state = 11, .external_lex_state = 2},
  [358] = {.lex_state = 11, .external_lex_state = 2},
  [359] = {.lex_state = 11, .external_lex_state = 2},
  [360] = {.lex_state = 11, .external_lex_state = 2},
  [361] = {.lex_state = 11, .external_lex_state = 2},
  [362] = {.lex_state = 11, .external_lex_state = 2},
  [363] = {.lex_state = 11, .external_lex_state = 2},
  [364] = {.lex_state = 11, .external_lex_state = 2},
  [365] = {.lex_state = 11, .external_lex_state = 2},
  [366] = {.lex_state = 11, .external_lex_state = 2},
  [367] = {.lex_state = 11, .external_lex_state = 2},
  [368] = {.lex_state = 11, .external_lex_state = 2},
  [369] = {.lex_state = 11, .external_lex_state = 2},
  [370] = {.lex_state = 69, .external_lex_state = 2},
  [371] = {.lex_state = 69, .external_lex_state = 2},
  [372] = {.lex_state = 69, .external_lex_state = 2},
  [373] = {.lex_state = 69, .external_lex_state = 2},
  [374] = {.lex_state = 69, .external_lex_state = 2},
  [375] = {.lex_state = 69, .external_lex_state = 2},
  [376] = {.lex_state = 12, .external_lex_state = 2},
  [377] = {.lex_state = 69, .external_lex_state = 2},
  [378] = {.lex_state = 69, .external_lex_state = 2},
  [379] = {.lex_state = 69, .external_lex_state = 2},
  [380] = {.lex_state = 12, .external_lex_state = 2},
  [381] = {.lex_state = 69, .external_lex_state = 2},
  [382] = {.lex_state = 69, .external_lex_state = 2},
  [383] = {.lex_state = 69, .external_lex_state = 2},
  [384] = {.lex_state = 69, .external_lex_state = 2},
  [385] = {.lex_state = 69, .external_lex_state = 2},
  [386] = {.lex_state = 69, .external_lex_state = 2},
  [387] = {.lex_state = 69, .external_lex_state = 2},
  [388] = {.lex_state = 69, .external_lex_state = 2},
  [389] = {.lex_state = 12, .external_lex_state = 2},
  [390] = {.lex_state = 12, .external_lex_state = 2},
  [391] = {.lex_state = 69, .external_lex_state = 2},
  [392] = {.lex_state = 69, .external_lex_state = 2},
  [393] = {.lex_state = 69, .external_lex_state = 2},
  [394] = {.lex_state = 12, .external_lex_state = 2},
  [395] = {.lex_state = 69, .external_lex_state = 2},
  [396] = {.lex_state = 69, .external_lex_state = 2},
  [397] = {.lex_state = 69, .external_lex_state = 2},
  [398] = {.lex_state = 69, .external_lex_state = 2},
  [399] = {.lex_state = 69, .external_lex_state = 2},
  [400] = {.lex_state = 69, .external_lex_state = 2},
  [401] = {.lex_state = 12, .external_lex_state = 2},
  [402] = {.lex_state = 69, .external_lex_state = 2},
  [403] = {.lex_state = 69, .external_lex_state = 2},
  [404] = {.lex_state = 69, .external_lex_state = 2},
  [405] = {.lex_state = 69, .external_lex_state = 2},
  [406] = {.lex_state = 12, .external_lex_state = 2},
  [407] = {.lex_state = 12, .external_lex_state = 2},
  [408] = {.lex_state = 12, .external_lex_state = 2},
  [409] = {.lex_state = 69, .external_lex_state = 2},
  [410] = {.lex_state = 12, .external_lex_state = 2},
  [411] = {.lex_state = 12, .external_lex_state = 2},
  [412] = {.lex_state = 12, .external_lex_state = 2},
  [413] = {.lex_state = 12, .external_lex_state = 2},
  [414] = {.lex_state = 12, .external_lex_state = 2},
  [415] = {.lex_state = 12, .external_lex_state = 2},
  [416] = {.lex_state = 12, .external_lex_state = 2},
  [417] = {.lex_state = 11, .external_lex_state = 2},
  [418] = {.lex_state = 11, .external_lex_state = 2},
  [419] = {.lex_state = 11, .external_lex_state = 2},
  [420] = {.lex_state = 12, .external_lex_state = 2},
  [421] = {.lex_state = 12, .external_lex_state = 2},
  [422] = {.lex_state = 12, .external_lex_state = 2},
  [423] = {.lex_state = 12, .external_lex_state = 2},
  [424] = {.lex_state = 11, .external_lex_state = 2},
  [425] = {.lex_state = 11, .external_lex_state = 2},
  [426] = {.lex_state = 11, .external_lex_state = 2},
  [427] = {.lex_state = 11, .external_lex_state = 2},
  [428] = {.lex_state = 11, .external_lex_state = 2},
  [429] = {.lex_state = 11, .external_lex_state = 2},
  [430] = {.lex_state = 11, .external_lex_state = 2},
  [431] = {.lex_state = 11, .external_lex_state = 2},
  [432] = {.lex_state = 11, .external_lex_state = 2},
  [433] = {.lex_state = 11, .external_lex_state = 2},
  [434] = {.lex_state = 11, .external_lex_state = 2},
  [435] = {.lex_state = 3, .external_lex_state = 2},
  [436] = {.lex_state = 5, .external_lex_state = 2},
  [437] = {.lex_state = 13, .external_lex_state = 2},
  [438] = {.lex_state = 13, .external_lex_state = 2},
  [439] = {.lex_state = 11, .external_lex_state = 2},
  [440] = {.lex_state = 11, .external_lex_state = 2},
  [441] = {.lex_state = 11, .external_lex_state = 2},
  [442] = {.lex_state = 11, .external_lex_state = 2},
  [443] = {.lex_state = 11, .external_lex_state = 2},
  [444] = {.lex_state = 11, .external_lex_state = 2},
  [445] = {.lex_state = 11, .external_lex_state = 2},
  [446] = {.lex_state = 11, .external_lex_state = 2},
  [447] = {.lex_state = 11, .external_lex_state = 2},
  [448] = {.lex_state = 11, .external_lex_state = 2},
  [449] = {.lex_state = 11, .external_lex_state = 2},
  [450] = {.lex_state = 11, .external_lex_state = 2},
  [451] = {.lex_state = 11, .external_lex_state = 2},
  [452] = {.lex_state = 3, .external_lex_state = 2},
  [453] = {.lex_state = 3, .external_lex_state = 2},
  [454] = {.lex_state = 3, .external_lex_state = 2},
  [455] = {.lex_state = 3, .external_lex_state = 2},
  [456] = {.lex_state = 3, .external_lex_state = 2},
  [457] = {.lex_state = 3, .external_lex_state = 2},
  [458] = {.lex_state = 3, .external_lex_state = 2},
  [459] = {.lex_state = 3, .external_lex_state = 2},
  [460] = {.lex_state = 3, .external_lex_state = 2},
  [461] = {.lex_state = 3, .external_lex_state = 2},
  [462] = {.lex_state = 3, .external_lex_state = 2},
  [463] = {.lex_state = 3, .external_lex_state = 2},
  [464] = {.lex_state = 3, .external_lex_state = 2},
  [465] = {.lex_state = 3, .external_lex_state = 2},
  [466] = {.lex_state = 3, .external_lex_state = 2},
  [467] = {.lex_state = 11, .external_lex_state = 2},
  [468] = {.lex_state = 3, .external_lex_state = 2},
  [469] = {.lex_state = 3, .external_lex_state = 2},
  [470] = {.lex_state = 3, .external_lex_state = 2},
  [471] = {.lex_state = 3, .external_lex_state = 2},
  [472] = {.lex_state = 11, .external_lex_state = 2},
  [473] = {.lex_state = 3, .external_lex_state = 2},
  [474] = {.lex_state = 11, .external_lex_state = 2},
  [475] = {.lex_state = 11, .external_lex_state = 2},
  [476] = {.lex_state = 3, .external_lex_state = 2},
  [477] = {.lex_state = 3, .external_lex_state = 2},
  [478] = {.lex_state = 3, .external_lex_state = 2},
  [479] = {.lex_state = 3, .external_lex_state = 2},
  [480] = {.lex_state = 3, .external_lex_state = 2},
  [481] = {.lex_state = 3, .external_lex_state = 2},
  [482] = {.lex_state = 3, .external_lex_state = 2},
  [483] = {.lex_state = 11, .external_lex_state = 2},
  [484] = {.lex_state = 11, .external_lex_state = 2},
  [485] = {.lex_state = 11, .external_lex_state = 2},
  [486] = {.lex_state = 11, .external_lex_state = 2},
  [487] = {.lex_state = 71, .external_lex_state = 2},
  [488] = {.lex_state = 71, .external_lex_state = 2},
  [489] = {.lex_state = 71, .external_lex_state = 2},
  [490] = {.lex_state = 71, .external_lex_state = 2},
  [491] = {.lex_state = 71, .external_lex_state = 2},
  [492] = {.lex_state = 71, .external_lex_state = 2},
  [493] = {.lex_state = 71, .external_lex_state = 2},
  [494] = {.lex_state = 71, .external_lex_state = 2},
  [495] = {.lex_state = 71, .external_lex_state = 2},
  [496] = {.lex_state = 71, .external_lex_state = 2},
  [497] = {.lex_state = 71, .external_lex_state = 2},
  [498] = {.lex_state = 71, .external_lex_state = 2},
  [499] = {.lex_state = 71, .external_lex_state = 2},
  [500] = {.lex_state = 71, .external_lex_state = 2},
  [501] = {.lex_state = 71, .external_lex_state = 2},
  [502] = {.lex_state = 71, .external_lex_state = 2},
  [503] = {.lex_state = 71, .external_lex_state = 2},
  [504] = {.lex_state = 71, .external_lex_state = 2},
  [505] = {.lex_state = 71, .external_lex_state = 2},
  [506] = {.lex_state = 71, .external_lex_state = 2},
  [507] = {.lex_state = 71, .external_lex_state = 2},
  [508] = {.lex_state = 71, .external_lex_state = 2},
  [509] = {.lex_state = 71, .external_lex_state = 2},
  [510] = {.lex_state = 71, .external_lex_state = 2},
  [511] = {.lex_state = 71, .external_lex_state = 2},
  [512] = {.lex_state = 71, .external_lex_state = 2},
  [513] = {.lex_state = 71, .external_lex_state = 2},
  [514] = {.lex_state = 71, .external_lex_state = 2},
  [515] = {.lex_state = 71, .external_lex_state = 2},
  [516] = {.lex_state = 71, .external_lex_state = 2},
  [517] = {.lex_state = 71, .external_lex_state = 2},
  [518] = {.lex_state = 71, .external_lex_state = 2},
  [519] = {.lex_state = 71, .external_lex_state = 2},
  [520] = {.lex_state = 71, .external_lex_state = 2},
  [521] = {.lex_state = 71, .external_lex_state = 2},
  [522] = {.lex_state = 71, .external_lex_state = 2},
  [523] = {.lex_state = 71, .external_lex_state = 2},
  [524] = {.lex_state = 71, .external_lex_state = 2},
  [525] = {.lex_state = 71, .external_lex_state = 2},
  [526] = {.lex_state = 71, .external_lex_state = 2},
  [527] = {.lex_state = 71, .external_lex_state = 2},
  [528] = {.lex_state = 71, .external_lex_state = 2},
  [529] = {.lex_state = 71, .external_lex_state = 2},
  [530] = {.lex_state = 71, .external_lex_state = 2},
  [531] = {.lex_state = 71, .external_lex_state = 2},
  [532] = {.lex_state = 71, .external_lex_state = 2},
  [533] = {.lex_state = 71, .external_lex_state = 2},
  [534] = {.lex_state = 71, .external_lex_state = 2},
  [535] = {.lex_state = 71, .external_lex_state = 2},
  [536] = {.lex_state = 71, .external_lex_state = 2},
  [537] = {.lex_state = 71, .external_lex_state = 2},
  [538] = {.lex_state = 71, .external_lex_state = 2},
  [539] = {.lex_state = 71, .external_lex_state = 2},
  [540] = {.lex_state = 71, .external_lex_state = 2},
  [541] = {.lex_state = 71, .external_lex_state = 2},
  [542] = {.lex_state = 71, .external_lex_state = 2},
  [543] = {.lex_state = 71, .external_lex_state = 2},
  [544] = {.lex_state = 71, .external_lex_state = 2},
  [545] = {.lex_state = 71, .external_lex_state = 2},
  [546] = {.lex_state = 71, .external_lex_state = 2},
  [547] = {.lex_state = 71, .external_lex_state = 2},
  [548] = {.lex_state = 71, .external_lex_state = 2},
  [549] = {.lex_state = 71, .external_lex_state = 2},
  [550] = {.lex_state = 71, .external_lex_state = 2},
  [551] = {.lex_state = 71, .external_lex_state = 2},
  [552] = {.lex_state = 71, .external_lex_state = 2},
  [553] = {.lex_state = 71, .external_lex_state = 2},
  [554] = {.lex_state = 71, .external_lex_state = 2},
  [555] = {.lex_state = 71, .external_lex_state = 2},
  [556] = {.lex_state = 71, .external_lex_state = 2},
  [557] = {.lex_state = 71, .external_lex_state = 2},
  [558] = {.lex_state = 71, .external_lex_state = 2},
  [559] = {.lex_state = 71, .external_lex_state = 2},
  [560] = {.lex_state = 71, .external_lex_state = 2},
  [561] = {.lex_state = 71, .external_lex_state = 2},
  [562] = {.lex_state = 71, .external_lex_state = 2},
  [563] = {.lex_state = 71, .external_lex_state = 2},
  [564] = {.lex_state = 71, .external_lex_state = 2},
  [565] = {.lex_state = 71, .external_lex_state = 2},
  [566] = {.lex_state = 71, .external_lex_state = 2},
  [567] = {.lex_state = 71, .external_lex_state = 2},
  [568] = {.lex_state = 71, .external_lex_state = 2},
  [569] = {.lex_state = 71, .external_lex_state = 2},
  [570] = {.lex_state = 71, .external_lex_state = 2},
  [571] = {.lex_state = 71, .external_lex_state = 2},
  [572] = {.lex_state = 71, .external_lex_state = 2},
  [573] = {.lex_state = 71, .external_lex_state = 2},
  [574] = {.lex_state = 71, .external_lex_state = 2},
  [575] = {.lex_state = 71, .external_lex_state = 2},
  [576] = {.lex_state = 71, .external_lex_state = 2},
  [577] = {.lex_state = 71, .external_lex_state = 2},
  [578] = {.lex_state = 71, .external_lex_state = 2},
  [579] = {.lex_state = 71, .external_lex_state = 2},
  [580] = {.lex_state = 71, .external_lex_state = 2},
  [581] = {.lex_state = 71, .external_lex_state = 2},
  [582] = {.lex_state = 71, .external_lex_state = 2},
  [583] = {.lex_state = 71, .external_lex_state = 2},
  [584] = {.lex_state = 71, .external_lex_state = 2},
  [585] = {.lex_state = 71, .external_lex_state = 2},
  [586] = {.lex_state = 71, .external_lex_state = 2},
  [587] = {.lex_state = 71, .external_lex_state = 2},
  [588] = {.lex_state = 71, .external_lex_state = 2},
  [589] = {.lex_state = 71, .external_lex_state = 2},
  [590] = {.lex_state = 71, .external_lex_state = 2},
  [591] = {.lex_state = 71, .external_lex_state = 2},
  [592] = {.lex_state = 71, .external_lex_state = 2},
  [593] = {.lex_state = 71, .external_lex_state = 2},
  [594] = {.lex_state = 71, .external_lex_state = 2},
  [595] = {.lex_state = 71, .external_lex_state = 2},
  [596] = {.lex_state = 71, .external_lex_state = 2},
  [597] = {.lex_state = 71, .external_lex_state = 2},
  [598] = {.lex_state = 71, .external_lex_state = 2},
  [599] = {.lex_state = 71, .external_lex_state = 2},
  [600] = {.lex_state = 71, .external_lex_state = 2},
  [601] = {.lex_state = 71, .external_lex_state = 2},
  [602] = {.lex_state = 71, .external_lex_state = 2},
  [603] = {.lex_state = 71, .external_lex_state = 2},
  [604] = {.lex_state = 71, .external_lex_state = 2},
  [605] = {.lex_state = 71, .external_lex_state = 2},
  [606] = {.lex_state = 71, .external_lex_state = 2},
  [607] = {.lex_state = 71, .external_lex_state = 2},
  [608] = {.lex_state = 71, .external_lex_state = 2},
  [609] = {.lex_state = 71, .external_lex_state = 2},
  [610] = {.lex_state = 71, .external_lex_state = 2},
  [611] = {.lex_state = 71, .external_lex_state = 2},
  [612] = {.lex_state = 71, .external_lex_state = 2},
  [613] = {.lex_state = 71, .external_lex_state = 2},
  [614] = {.lex_state = 71, .external_lex_state = 2},
  [615] = {.lex_state = 71, .external_lex_state = 2},
  [616] = {.lex_state = 71, .external_lex_state = 2},
  [617] = {.lex_state = 71, .external_lex_state = 2},
  [618] = {.lex_state = 71, .external_lex_state = 2},
  [619] = {.lex_state = 71, .external_lex_state = 2},
  [620] = {.lex_state = 71, .external_lex_state = 2},
  [621] = {.lex_state = 71, .external_lex_state = 2},
  [622] = {.lex_state = 71, .external_lex_state = 2},
  [623] = {.lex_state = 71, .external_lex_state = 2},
  [624] = {.lex_state = 71, .external_lex_state = 2},
  [625] = {.lex_state = 71, .external_lex_state = 2},
  [626] = {.lex_state = 71, .external_lex_state = 2},
  [627] = {.lex_state = 71, .external_lex_state = 2},
  [628] = {.lex_state = 71, .external_lex_state = 2},
  [629] = {.lex_state = 71, .external_lex_state = 2},
  [630] = {.lex_state = 71, .external_lex_state = 2},
  [631] = {.lex_state = 71, .external_lex_state = 2},
  [632] = {.lex_state = 71, .external_lex_state = 2},
  [633] = {.lex_state = 71, .external_lex_state = 2},
  [634] = {.lex_state = 71, .external_lex_state = 2},
  [635] = {.lex_state = 71, .external_lex_state = 2},
  [636] = {.lex_state = 71, .external_lex_state = 2},
  [637] = {.lex_state = 71, .external_lex_state = 2},
  [638] = {.lex_state = 71, .external_lex_state = 2},
  [639] = {.lex_state = 10},
  [640] = {.lex_state = 71, .external_lex_state = 2},
  [641] = {.lex_state = 71, .external_lex_state = 2},
  [642] = {.lex_state = 71, .external_lex_state = 2},
  [643] = {.lex_state = 71, .external_lex_state = 2},
  [644] = {.lex_state = 71, .external_lex_state = 2},
  [645] = {.lex_state = 71, .external_lex_state = 2},
  [646] = {.lex_state = 71, .external_lex_state = 2},
  [647] = {.lex_state = 71, .external_lex_state = 2},
  [648] = {.lex_state = 71, .external_lex_state = 2},
  [649] = {.lex_state = 71, .external_lex_state = 2},
  [650] = {.lex_state = 71, .external_lex_state = 2},
  [651] = {.lex_state = 71, .external_lex_state = 2},
  [652] = {.lex_state = 71, .external_lex_state = 2},
  [653] = {.lex_state = 71, .external_lex_state = 2},
  [654] = {.lex_state = 71, .external_lex_state = 2},
  [655] = {.lex_state = 71, .external_lex_state = 2},
  [656] = {.lex_state = 71, .external_lex_state = 2},
  [657] = {.lex_state = 71, .external_lex_state = 2},
  [658] = {.lex_state = 71, .external_lex_state = 2},
  [659] = {.lex_state = 71, .external_lex_state = 2},
  [660] = {.lex_state = 71, .external_lex_state = 2},
  [661] = {.lex_state = 71, .external_lex_state = 2},
  [662] = {.lex_state = 10},
  [663] = {.lex_state = 71, .external_lex_state = 2},
  [664] = {.lex_state = 71, .external_lex_state = 2},
  [665] = {.lex_state = 71, .external_lex_state = 2},
  [666] = {.lex_state = 71, .external_lex_state = 2},
  [667] = {.lex_state = 10},
  [668] = {.lex_state = 71, .external_lex_state = 2},
  [669] = {.lex_state = 71, .external_lex_state = 2},
  [670] = {.lex_state = 71, .external_lex_state = 2},
  [671] = {.lex_state = 71, .external_lex_state = 2},
  [672] = {.lex_state = 71, .external_lex_state = 2},
  [673] = {.lex_state = 71, .external_lex_state = 2},
  [674] = {.lex_state = 10},
  [675] = {.lex_state = 71, .external_lex_state = 2},
  [676] = {.lex_state = 71, .external_lex_state = 2},
  [677] = {.lex_state = 71, .external_lex_state = 2},
  [678] = {.lex_state = 71, .external_lex_state = 2},
  [679] = {.lex_state = 71, .external_lex_state = 2},
  [680] = {.lex_state = 71, .external_lex_state = 2},
  [681] = {.lex_state = 71, .external_lex_state = 2},
  [682] = {.lex_state = 71, .external_lex_state = 2},
  [683] = {.lex_state = 71, .external_lex_state = 2},
  [684] = {.lex_state = 71, .external_lex_state = 2},
  [685] = {.lex_state = 71, .external_lex_state = 2},
  [686] = {.lex_state = 71, .external_lex_state = 2},
  [687] = {.lex_state = 71, .external_lex_state = 2},
  [688] = {.lex_state = 71, .external_lex_state = 2},
  [689] = {.lex_state = 71, .external_lex_state = 2},
  [690] = {.lex_state = 71, .external_lex_state = 2},
  [691] = {.lex_state = 71, .external_lex_state = 2},
  [692] = {.lex_state = 71, .external_lex_state = 2},
  [693] = {.lex_state = 71, .external_lex_state = 2},
  [694] = {.lex_state = 71, .external_lex_state = 2},
  [695] = {.lex_state = 71, .external_lex_state = 2},
  [696] = {.lex_state = 71, .external_lex_state = 2},
  [697] = {.lex_state = 71, .external_lex_state = 2},
  [698] = {.lex_state = 71, .external_lex_state = 2},
  [699] = {.lex_state = 11, .external_lex_state = 2},
  [700] = {.lex_state = 10},
  [701] = {.lex_state = 71, .external_lex_state = 2},
  [702] = {.lex_state = 71, .external_lex_state = 2},
  [703] = {.lex_state = 71, .external_lex_state = 2},
  [704] = {.lex_state = 71, .external_lex_state = 2},
  [705] = {.lex_state = 71, .external_lex_state = 2},
  [706] = {.lex_state = 71, .external_lex_state = 2},
  [707] = {.lex_state = 71, .external_lex_state = 2},
  [708] = {.lex_state = 71, .external_lex_state = 2},
  [709] = {.lex_state = 71, .external_lex_state = 2},
  [710] = {.lex_state = 71, .external_lex_state = 2},
  [711] = {.lex_state = 71, .external_lex_state = 2},
  [712] = {.lex_state = 71, .external_lex_state = 2},
  [713] = {.lex_state = 71, .external_lex_state = 2},
  [714] = {.lex_state = 71, .external_lex_state = 2},
  [715] = {.lex_state = 71, .external_lex_state = 2},
  [716] = {.lex_state = 71, .external_lex_state = 2},
  [717] = {.lex_state = 71, .external_lex_state = 2},
  [718] = {.lex_state = 71, .external_lex_state = 2},
  [719] = {.lex_state = 71, .external_lex_state = 2},
  [720] = {.lex_state = 71, .external_lex_state = 2},
  [721] = {.lex_state = 71, .external_lex_state = 2},
  [722] = {.lex_state = 71, .external_lex_state = 2},
  [723] = {.lex_state = 71, .external_lex_state = 2},
  [724] = {.lex_state = 71, .external_lex_state = 2},
  [725] = {.lex_state = 71, .external_lex_state = 2},
  [726] = {.lex_state = 71, .external_lex_state = 2},
  [727] = {.lex_state = 71, .external_lex_state = 2},
  [728] = {.lex_state = 71, .external_lex_state = 2},
  [729] = {.lex_state = 71, .external_lex_state = 2},
  [730] = {.lex_state = 71, .external_lex_state = 2},
  [731] = {.lex_state = 71, .external_lex_state = 2},
  [732] = {.lex_state = 71, .external_lex_state = 2},
  [733] = {.lex_state = 71, .external_lex_state = 2},
  [734] = {.lex_state = 71, .external_lex_state = 2},
  [735] = {.lex_state = 71, .external_lex_state = 2},
  [736] = {.lex_state = 71, .external_lex_state = 2},
  [737] = {.lex_state = 71, .external_lex_state = 2},
  [738] = {.lex_state = 71, .external_lex_state = 2},
  [739] = {.lex_state = 71, .external_lex_state = 2},
  [740] = {.lex_state = 71, .external_lex_state = 2},
  [741] = {.lex_state = 71, .external_lex_state = 2},
  [742] = {.lex_state = 71, .external_lex_state = 2},
  [743] = {.lex_state = 71, .external_lex_state = 2},
  [744] = {.lex_state = 71, .external_lex_state = 2},
  [745] = {.lex_state = 71, .external_lex_state = 2},
  [746] = {.lex_state = 71, .external_lex_state = 2},
  [747] = {.lex_state = 71, .external_lex_state = 2},
  [748] = {.lex_state = 71, .external_lex_state = 2},
  [749] = {.lex_state = 71, .external_lex_state = 2},
  [750] = {.lex_state = 71, .external_lex_state = 2},
  [751] = {.lex_state = 71, .external_lex_state = 2},
  [752] = {.lex_state = 71, .external_lex_state = 2},
  [753] = {.lex_state = 71, .external_lex_state = 2},
  [754] = {.lex_state = 71, .external_lex_state = 2},
  [755] = {.lex_state = 71, .external_lex_state = 2},
  [756] = {.lex_state = 71, .external_lex_state = 2},
  [757] = {.lex_state = 71, .external_lex_state = 2},
  [758] = {.lex_state = 71, .external_lex_state = 2},
  [759] = {.lex_state = 71, .external_lex_state = 2},
  [760] = {.lex_state = 71, .external_lex_state = 2},
  [761] = {.lex_state = 71, .external_lex_state = 2},
  [762] = {.lex_state = 11, .external_lex_state = 2},
  [763] = {.lex_state = 11, .external_lex_state = 2},
  [764] = {.lex_state = 11, .external_lex_state = 2},
  [765] = {.lex_state = 11, .external_lex_state = 2},
  [766] = {.lex_state = 11, .external_lex_state = 2},
  [767] = {.lex_state = 11, .external_lex_state = 2},
  [768] = {.lex_state = 11, .external_lex_state = 2},
  [769] = {.lex_state = 11, .external_lex_state = 2},
  [770] = {.lex_state = 14},
  [771] = {.lex_state = 14},
  [772] = {.lex_state = 11, .external_lex_state = 2},
  [773] = {.lex_state = 14},
  [774] = {.lex_state = 14},
  [775] = {.lex_state = 14},
  [776] = {.lex_state = 14},
  [777] = {.lex_state = 14},
  [778] = {.lex_state = 14},
  [779] = {.lex_state = 14},
  [780] = {.lex_state = 11, .external_lex_state = 2},
  [781] = {.lex_state = 11, .external_lex_state = 2},
  [782] = {.lex_state = 14},
  [783] = {.lex_state = 14},
  [784] = {.lex_state = 11, .external_lex_state = 2},
  [785] = {.lex_state = 11, .external_lex_state = 2},
  [786] = {.lex_state = 11, .external_lex_state = 2},
  [787] = {.lex_state = 11, .external_lex_state = 2},
  [788] = {.lex_state = 11, .external_lex_state = 2},
  [789] = {.lex_state = 11, .external_lex_state = 2},
  [790] = {.lex_state = 11, .external_lex_state = 2},
  [791] = {.lex_state = 11, .external_lex_state = 2},
  [792] = {.lex_state = 11, .external_lex_state = 2},
  [793] = {.lex_state = 12, .external_lex_state = 2},
  [794] = {.lex_state = 11, .external_lex_state = 2},
  [795] = {.lex_state = 11, .external_lex_state = 2},
  [796] = {.lex_state = 11, .external_lex_state = 2},
  [797] = {.lex_state = 11, .external_lex_state = 2},
  [798] = {.lex_state = 11, .external_lex_state = 2},
  [799] = {.lex_state = 11, .external_lex_state = 2},
  [800] = {.lex_state = 11, .external_lex_state = 2},
  [801] = {.lex_state = 11, .external_lex_state = 2},
  [802] = {.lex_state = 11, .external_lex_state = 2},
  [803] = {.lex_state = 11, .external_lex_state = 2},
  [804] = {.lex_state = 11, .external_lex_state = 2},
  [805] = {.lex_state = 11, .external_lex_state = 2},
  [806] = {.lex_state = 11, .external_lex_state = 2},
  [807] = {.lex_state = 11, .external_lex_state = 2},
  [808] = {.lex_state = 11, .external_lex_state = 2},
  [809] = {.lex_state = 11, .external_lex_state = 2},
  [810] = {.lex_state = 11, .external_lex_state = 2},
  [811] = {.lex_state = 11, .external_lex_state = 2},
  [812] = {.lex_state = 11, .external_lex_state = 2},
  [813] = {.lex_state = 11, .external_lex_state = 2},
  [814] = {.lex_state = 11, .external_lex_state = 2},
  [815] = {.lex_state = 11, .external_lex_state = 2},
  [816] = {.lex_state = 11, .external_lex_state = 2},
  [817] = {.lex_state = 11, .external_lex_state = 2},
  [818] = {.lex_state = 11, .external_lex_state = 2},
  [819] = {.lex_state = 11, .external_lex_state = 2},
  [820] = {.lex_state = 11, .external_lex_state = 2},
  [821] = {.lex_state = 11, .external_lex_state = 2},
  [822] = {.lex_state = 11, .external_lex_state = 2},
  [823] = {.lex_state = 11, .external_lex_state = 2},
  [824] = {.lex_state = 11, .external_lex_state = 2},
  [825] = {.lex_state = 11, .external_lex_state = 2},
  [826] = {.lex_state = 11, .external_lex_state = 2},
  [827] = {.lex_state = 11, .external_lex_state = 2},
  [828] = {.lex_state = 11, .external_lex_state = 2},
  [829] = {.lex_state = 11, .external_lex_state = 2},
  [830] = {.lex_state = 11, .external_lex_state = 2},
  [831] = {.lex_state = 11, .external_lex_state = 2},
  [832] = {.lex_state = 11, .external_lex_state = 2},
  [833] = {.lex_state = 11, .external_lex_state = 2},
  [834] = {.lex_state = 11, .external_lex_state = 2},
  [835] = {.lex_state = 11, .external_lex_state = 2},
  [836] = {.lex_state = 11, .external_lex_state = 2},
  [837] = {.lex_state = 14},
  [838] = {.lex_state = 14},
  [839] = {.lex_state = 14},
  [840] = {.lex_state = 14},
  [841] = {.lex_state = 14},
  [842] = {.lex_state = 14},
  [843] = {.lex_state = 14},
  [844] = {.lex_state = 14},
  [845] = {.lex_state = 14},
  [846] = {.lex_state = 14},
  [847] = {.lex_state = 14},
  [848] = {.lex_state = 14},
  [849] = {.lex_state = 14},
  [850] = {.lex_state = 14},
  [851] = {.lex_state = 14},
  [852] = {.lex_state = 14},
  [853] = {.lex_state = 14},
  [854] = {.lex_state = 14},
  [855] = {.lex_state = 14},
  [856] = {.lex_state = 14},
  [857] = {.lex_state = 14},
  [858] = {.lex_state = 14},
  [859] = {.lex_state = 14},
  [860] = {.lex_state = 14},
  [861] = {.lex_state = 14},
  [862] = {.lex_state = 14},
  [863] = {.lex_state = 14},
  [864] = {.lex_state = 14},
  [865] = {.lex_state = 14},
  [866] = {.lex_state = 14},
  [867] = {.lex_state = 14},
  [868] = {.lex_state = 14},
  [869] = {.lex_state = 14},
  [870] = {.lex_state = 14},
  [871] = {.lex_state = 14},
  [872] = {.lex_state = 14},
  [873] = {.lex_state = 14},
  [874] = {.lex_state = 14},
  [875] = {.lex_state = 14},
  [876] = {.lex_state = 14},
  [877] = {.lex_state = 14},
  [878] = {.lex_state = 14},
  [879] = {.lex_state = 14},
  [880] = {.lex_state = 14},
  [881] = {.lex_state = 14},
  [882] = {.lex_state = 14},
  [883] = {.lex_state = 14},
  [884] = {.lex_state = 14},
  [885] = {.lex_state = 14},
  [886] = {.lex_state = 14},
  [887] = {.lex_state = 14},
  [888] = {.lex_state = 14},
  [889] = {.lex_state = 14},
  [890] = {.lex_state = 14},
  [891] = {.lex_state = 14},
  [892] = {.lex_state = 14},
  [893] = {.lex_state = 14},
  [894] = {.lex_state = 14},
  [895] = {.lex_state = 14},
  [896] = {.lex_state = 14},
  [897] = {.lex_state = 14},
  [898] = {.lex_state = 14},
  [899] = {.lex_state = 11, .external_lex_state = 2},
  [900] = {.lex_state = 14},
  [901] = {.lex_state = 14},
  [902] = {.lex_state = 14},
  [903] = {.lex_state = 14},
  [904] = {.lex_state = 14},
  [905] = {.lex_state = 14},
  [906] = {.lex_state = 14},
  [907] = {.lex_state = 14},
  [908] = {.lex_state = 14},
  [909] = {.lex_state = 14},
  [910] = {.lex_state = 14},
  [911] = {.lex_state = 14},
  [912] = {.lex_state = 14},
  [913] = {.lex_state = 14},
  [914] = {.lex_state = 14},
  [915] = {.lex_state = 14},
  [916] = {.lex_state = 14},
  [917] = {.lex_state = 14},
  [918] = {.lex_state = 14},
  [919] = {.lex_state = 14},
  [920] = {.lex_state = 14},
  [921] = {.lex_state = 14},
  [922] = {.lex_state = 14},
  [923] = {.lex_state = 14},
  [924] = {.lex_state = 14},
  [925] = {.lex_state = 14},
  [926] = {.lex_state = 14},
  [927] = {.lex_state = 14},
  [928] = {.lex_state = 14},
  [929] = {.lex_state = 14},
  [930] = {.lex_state = 14},
  [931] = {.lex_state = 14},
  [932] = {.lex_state = 14},
  [933] = {.lex_state = 14},
  [934] = {.lex_state = 14},
  [935] = {.lex_state = 14},
  [936] = {.lex_state = 14},
  [937] = {.lex_state = 14},
  [938] = {.lex_state = 14},
  [939] = {.lex_state = 14},
  [940] = {.lex_state = 14},
  [941] = {.lex_state = 14},
  [942] = {.lex_state = 14},
  [943] = {.lex_state = 14},
  [944] = {.lex_state = 14},
  [945] = {.lex_state = 14},
  [946] = {.lex_state = 14},
  [947] = {.lex_state = 14},
  [948] = {.lex_state = 14},
  [949] = {.lex_state = 14},
  [950] = {.lex_state = 14},
  [951] = {.lex_state = 14},
  [952] = {.lex_state = 14},
  [953] = {.lex_state = 14},
  [954] = {.lex_state = 14},
  [955] = {.lex_state = 14},
  [956] = {.lex_state = 14},
  [957] = {.lex_state = 14},
  [958] = {.lex_state = 14},
  [959] = {.lex_state = 14},
  [960] = {.lex_state = 14},
  [961] = {.lex_state = 14},
  [962] = {.lex_state = 14},
  [963] = {.lex_state = 14},
  [964] = {.lex_state = 14},
  [965] = {.lex_state = 14},
  [966] = {.lex_state = 14},
  [967] = {.lex_state = 14},
  [968] = {.lex_state = 14},
  [969] = {.lex_state = 14},
  [970] = {.lex_state = 14},
  [971] = {.lex_state = 14},
  [972] = {.lex_state = 14},
  [973] = {.lex_state = 14},
  [974] = {.lex_state = 14},
  [975] = {.lex_state = 14},
  [976] = {.lex_state = 14},
  [977] = {.lex_state = 14},
  [978] = {.lex_state = 14},
  [979] = {.lex_state = 14},
  [980] = {.lex_state = 14},
  [981] = {.lex_state = 14},
  [982] = {.lex_state = 14},
  [983] = {.lex_state = 14},
  [984] = {.lex_state = 14},
  [985] = {.lex_state = 14},
  [986] = {.lex_state = 14},
  [987] = {.lex_state = 14},
  [988] = {.lex_state = 14},
  [989] = {.lex_state = 14},
  [990] = {.lex_state = 14},
  [991] = {.lex_state = 14},
  [992] = {.lex_state = 14},
  [993] = {.lex_state = 14},
  [994] = {.lex_state = 14},
  [995] = {.lex_state = 14},
  [996] = {.lex_state = 14},
  [997] = {.lex_state = 14},
  [998] = {.lex_state = 14},
  [999] = {.lex_state = 14},
  [1000] = {.lex_state = 14},
  [1001] = {.lex_state = 14},
  [1002] = {.lex_state = 14},
  [1003] = {.lex_state = 14},
  [1004] = {.lex_state = 14},
  [1005] = {.lex_state = 14},
  [1006] = {.lex_state = 14},
  [1007] = {.lex_state = 14},
  [1008] = {.lex_state = 14},
  [1009] = {.lex_state = 14},
  [1010] = {.lex_state = 11, .external_lex_state = 2},
  [1011] = {.lex_state = 11, .external_lex_state = 2},
  [1012] = {.lex_state = 13, .external_lex_state = 2},
  [1013] = {.lex_state = 13, .external_lex_state = 2},
  [1014] = {.lex_state = 11, .external_lex_state = 2},
  [1015] = {.lex_state = 12, .external_lex_state = 2},
  [1016] = {.lex_state = 14},
  [1017] = {.lex_state = 11, .external_lex_state = 2},
  [1018] = {.lex_state = 11, .external_lex_state = 2},
  [1019] = {.lex_state = 7},
  [1020] = {.lex_state = 6},
  [1021] = {.lex_state = 10},
  [1022] = {.lex_state = 6},
  [1023] = {.lex_state = 10},
  [1024] = {.lex_state = 10},
  [1025] = {.lex_state = 10},
  [1026] = {.lex_state = 10},
  [1027] = {.lex_state = 10},
  [1028] = {.lex_state = 6},
  [1029] = {.lex_state = 10},
  [1030] = {.lex_state = 6},
  [1031] = {.lex_state = 10},
  [1032] = {.lex_state = 10},
  [1033] = {.lex_state = 22},
  [1034] = {.lex_state = 6},
  [1035] = {.lex_state = 22},
  [1036] = {.lex_state = 6},
  [1037] = {.lex_state = 22},
  [1038] = {.lex_state = 6},
  [1039] = {.lex_state = 22},
  [1040] = {.lex_state = 6},
  [1041] = {.lex_state = 6},
  [1042] = {.lex_state = 6},
  [1043] = {.lex_state = 6},
  [1044] = {.lex_state = 22},
  [1045] = {.lex_state = 11, .external_lex_state = 2},
  [1046] = {.lex_state = 6},
  [1047] = {.lex_state = 14},
  [1048] = {.lex_state = 14},
  [1049] = {.lex_state = 7},
  [1050] = {.lex_state = 7},
  [1051] = {.lex_state = 14},
  [1052] = {.lex_state = 14},
  [1053] = {.lex_state = 14},
  [1054] = {.lex_state = 7},
  [1055] = {.lex_state = 14},
  [1056] = {.lex_state = 14},
  [1057] = {.lex_state = 7},
  [1058] = {.lex_state = 14},
  [1059] = {.lex_state = 7},
  [1060] = {.lex_state = 7},
  [1061] = {.lex_state = 14},
  [1062] = {.lex_state = 7},
  [1063] = {.lex_state = 14},
  [1064] = {.lex_state = 7},
  [1065] = {.lex_state = 14},
  [1066] = {.lex_state = 14},
  [1067] = {.lex_state = 14},
  [1068] = {.lex_state = 14},
  [1069] = {.lex_state = 14},
  [1070] = {.lex_state = 11, .external_lex_state = 2},
  [1071] = {.lex_state = 7},
  [1072] = {.lex_state = 10},
  [1073] = {.lex_state = 7},
  [1074] = {.lex_state = 11, .external_lex_state = 2},
  [1075] = {.lex_state = 7},
  [1076] = {.lex_state = 7},
  [1077] = {.lex_state = 14},
  [1078] = {.lex_state = 7},
  [1079] = {.lex_state = 19},
  [1080] = {.lex_state = 10},
  [1081] = {.lex_state = 7},
  [1082] = {.lex_state = 7},
  [1083] = {.lex_state = 7},
  [1084] = {.lex_state = 10},
  [1085] = {.lex_state = 10},
  [1086] = {.lex_state = 12, .external_lex_state = 2},
  [1087] = {.lex_state = 7},
  [1088] = {.lex_state = 12, .external_lex_state = 2},
  [1089] = {.lex_state = 7},
  [1090] = {.lex_state = 7},
  [1091] = {.lex_state = 19},
  [1092] = {.lex_state = 18},
  [1093] = {.lex_state = 7},
  [1094] = {.lex_state = 7},
  [1095] = {.lex_state = 18},
  [1096] = {.lex_state = 18},
  [1097] = {.lex_state = 10},
  [1098] = {.lex_state = 7},
  [1099] = {.lex_state = 10},
  [1100] = {.lex_state = 10},
  [1101] = {.lex_state = 10},
  [1102] = {.lex_state = 10},
  [1103] = {.lex_state = 10},
  [1104] = {.lex_state = 10},
  [1105] = {.lex_state = 10},
  [1106] = {.lex_state = 10},
  [1107] = {.lex_state = 10},
  [1108] = {.lex_state = 10},
  [1109] = {.lex_state = 10},
  [1110] = {.lex_state = 10},
  [1111] = {.lex_state = 10},
  [1112] = {.lex_state = 10},
  [1113] = {.lex_state = 10},
  [1114] = {.lex_state = 10},
  [1115] = {.lex_state = 6},
  [1116] = {.lex_state = 18},
  [1117] = {.lex_state = 18},
  [1118] = {.lex_state = 18},
  [1119] = {.lex_state = 18},
  [1120] = {.lex_state = 10},
  [1121] = {.lex_state = 10},
  [1122] = {.lex_state = 10},
  [1123] = {.lex_state = 10},
  [1124] = {.lex_state = 10},
  [1125] = {.lex_state = 10},
  [1126] = {.lex_state = 10},
  [1127] = {.lex_state = 10},
  [1128] = {.lex_state = 10},
  [1129] = {.lex_state = 10},
  [1130] = {.lex_state = 10},
  [1131] = {.lex_state = 10},
  [1132] = {.lex_state = 10},
  [1133] = {.lex_state = 10},
  [1134] = {.lex_state = 10},
  [1135] = {.lex_state = 10},
  [1136] = {.lex_state = 10},
  [1137] = {.lex_state = 10},
  [1138] = {.lex_state = 10},
  [1139] = {.lex_state = 10},
  [1140] = {.lex_state = 10},
  [1141] = {.lex_state = 10},
  [1142] = {.lex_state = 10},
  [1143] = {.lex_state = 10},
  [1144] = {.lex_state = 10},
  [1145] = {.lex_state = 10},
  [1146] = {.lex_state = 10},
  [1147] = {.lex_state = 10},
  [1148] = {.lex_state = 10},
  [1149] = {.lex_state = 10},
  [1150] = {.lex_state = 10},
  [1151] = {.lex_state = 10},
  [1152] = {.lex_state = 10},
  [1153] = {.lex_state = 10},
  [1154] = {.lex_state = 10},
  [1155] = {.lex_state = 10},
  [1156] = {.lex_state = 7},
  [1157] = {.lex_state = 18},
  [1158] = {.lex_state = 7},
  [1159] = {.lex_state = 18},
  [1160] = {.lex_state = 10},
  [1161] = {.lex_state = 10},
  [1162] = {.lex_state = 10},
  [1163] = {.lex_state = 10},
  [1164] = {.lex_state = 10},
  [1165] = {.lex_state = 10},
  [1166] = {.lex_state = 10},
  [1167] = {.lex_state = 10},
  [1168] = {.lex_state = 10},
  [1169] = {.lex_state = 10},
  [1170] = {.lex_state = 10},
  [1171] = {.lex_state = 10},
  [1172] = {.lex_state = 10},
  [1173] = {.lex_state = 10},
  [1174] = {.lex_state = 10},
  [1175] = {.lex_state = 10},
  [1176] = {.lex_state = 10},
  [1177] = {.lex_state = 10},
  [1178] = {.lex_state = 10},
  [1179] = {.lex_state = 10},
  [1180] = {.lex_state = 10},
  [1181] = {.lex_state = 10},
  [1182] = {.lex_state = 10},
  [1183] = {.lex_state = 10},
  [1184] = {.lex_state = 10},
  [1185] = {.lex_state = 10},
  [1186] = {.lex_state = 10},
  [1187] = {.lex_state = 10},
  [1188] = {.lex_state = 10},
  [1189] = {.lex_state = 10},
  [1190] = {.lex_state = 10},
  [1191] = {.lex_state = 10},
  [1192] = {.lex_state = 10},
  [1193] = {.lex_state = 10},
  [1194] = {.lex_state = 10},
  [1195] = {.lex_state = 10},
  [1196] = {.lex_state = 10},
  [1197] = {.lex_state = 10},
  [1198] = {.lex_state = 10},
  [1199] = {.lex_state = 10},
  [1200] = {.lex_state = 10},
  [1201] = {.lex_state = 10},
  [1202] = {.lex_state = 10},
  [1203] = {.lex_state = 10},
  [1204] = {.lex_state = 10},
  [1205] = {.lex_state = 10},
  [1206] = {.lex_state = 10},
  [1207] = {.lex_state = 10},
  [1208] = {.lex_state = 10},
  [1209] = {.lex_state = 10},
  [1210] = {.lex_state = 7},
  [1211] = {.lex_state = 18},
  [1212] = {.lex_state = 10},
  [1213] = {.lex_state = 10},
  [1214] = {.lex_state = 10},
  [1215] = {.lex_state = 10},
  [1216] = {.lex_state = 10},
  [1217] = {.lex_state = 10},
  [1218] = {.lex_state = 10},
  [1219] = {.lex_state = 10},
  [1220] = {.lex_state = 10},
  [1221] = {.lex_state = 10},
  [1222] = {.lex_state = 10},
  [1223] = {.lex_state = 10},
  [1224] = {.lex_state = 10},
  [1225] = {.lex_state = 10},
  [1226] = {.lex_state = 10},
  [1227] = {.lex_state = 10},
  [1228] = {.lex_state = 10},
  [1229] = {.lex_state = 10},
  [1230] = {.lex_state = 14},
  [1231] = {.lex_state = 10},
  [1232] = {.lex_state = 10},
  [1233] = {.lex_state = 10},
  [1234] = {.lex_state = 10},
  [1235] = {.lex_state = 10},
  [1236] = {.lex_state = 10},
  [1237] = {.lex_state = 10},
  [1238] = {.lex_state = 10},
  [1239] = {.lex_state = 10},
  [1240] = {.lex_state = 10},
  [1241] = {.lex_state = 10},
  [1242] = {.lex_state = 10},
  [1243] = {.lex_state = 10},
  [1244] = {.lex_state = 10},
  [1245] = {.lex_state = 10},
  [1246] = {.lex_state = 10},
  [1247] = {.lex_state = 10},
  [1248] = {.lex_state = 10},
  [1249] = {.lex_state = 10},
  [1250] = {.lex_state = 10},
  [1251] = {.lex_state = 10},
  [1252] = {.lex_state = 10},
  [1253] = {.lex_state = 10},
  [1254] = {.lex_state = 10},
  [1255] = {.lex_state = 10},
  [1256] = {.lex_state = 10},
  [1257] = {.lex_state = 10},
  [1258] = {.lex_state = 10},
  [1259] = {.lex_state = 10},
  [1260] = {.lex_state = 10},
  [1261] = {.lex_state = 10},
  [1262] = {.lex_state = 7},
  [1263] = {.lex_state = 18},
  [1264] = {.lex_state = 10},
  [1265] = {.lex_state = 10},
  [1266] = {.lex_state = 10},
  [1267] = {.lex_state = 10},
  [1268] = {.lex_state = 10},
  [1269] = {.lex_state = 10},
  [1270] = {.lex_state = 10},
  [1271] = {.lex_state = 10},
  [1272] = {.lex_state = 10},
  [1273] = {.lex_state = 10},
  [1274] = {.lex_state = 10},
  [1275] = {.lex_state = 10},
  [1276] = {.lex_state = 10},
  [1277] = {.lex_state = 10},
  [1278] = {.lex_state = 10},
  [1279] = {.lex_state = 10},
  [1280] = {.lex_state = 10},
  [1281] = {.lex_state = 10},
  [1282] = {.lex_state = 10},
  [1283] = {.lex_state = 10},
  [1284] = {.lex_state = 10},
  [1285] = {.lex_state = 10},
  [1286] = {.lex_state = 10},
  [1287] = {.lex_state = 10},
  [1288] = {.lex_state = 10},
  [1289] = {.lex_state = 10},
  [1290] = {.lex_state = 10},
  [1291] = {.lex_state = 10},
  [1292] = {.lex_state = 10},
  [1293] = {.lex_state = 10},
  [1294] = {.lex_state = 10},
  [1295] = {.lex_state = 10},
  [1296] = {.lex_state = 10},
  [1297] = {.lex_state = 10},
  [1298] = {.lex_state = 10},
  [1299] = {.lex_state = 10},
  [1300] = {.lex_state = 10},
  [1301] = {.lex_state = 10},
  [1302] = {.lex_state = 10},
  [1303] = {.lex_state = 10},
  [1304] = {.lex_state = 10},
  [1305] = {.lex_state = 10},
  [1306] = {.lex_state = 10},
  [1307] = {.lex_state = 10},
  [1308] = {.lex_state = 10},
  [1309] = {.lex_state = 10},
  [1310] = {.lex_state = 10},
  [1311] = {.lex_state = 10},
  [1312] = {.lex_state = 7},
  [1313] = {.lex_state = 18},
  [1314] = {.lex_state = 10},
  [1315] = {.lex_state = 10},
  [1316] = {.lex_state = 10},
  [1317] = {.lex_state = 10},
  [1318] = {.lex_state = 10},
  [1319] = {.lex_state = 10},
  [1320] = {.lex_state = 10},
  [1321] = {.lex_state = 10},
  [1322] = {.lex_state = 10},
  [1323] = {.lex_state = 10},
  [1324] = {.lex_state = 10},
  [1325] = {.lex_state = 10},
  [1326] = {.lex_state = 10},
  [1327] = {.lex_state = 10},
  [1328] = {.lex_state = 10},
  [1329] = {.lex_state = 10},
  [1330] = {.lex_state = 10},
  [1331] = {.lex_state = 10},
  [1332] = {.lex_state = 10},
  [1333] = {.lex_state = 10},
  [1334] = {.lex_state = 10},
  [1335] = {.lex_state = 10},
  [1336] = {.lex_state = 10},
  [1337] = {.lex_state = 10},
  [1338] = {.lex_state = 10},
  [1339] = {.lex_state = 10},
  [1340] = {.lex_state = 10},
  [1341] = {.lex_state = 10},
  [1342] = {.lex_state = 10},
  [1343] = {.lex_state = 10},
  [1344] = {.lex_state = 10},
  [1345] = {.lex_state = 10},
  [1346] = {.lex_state = 10},
  [1347] = {.lex_state = 10},
  [1348] = {.lex_state = 10},
  [1349] = {.lex_state = 10},
  [1350] = {.lex_state = 10},
  [1351] = {.lex_state = 10},
  [1352] = {.lex_state = 10},
  [1353] = {.lex_state = 10},
  [1354] = {.lex_state = 10},
  [1355] = {.lex_state = 10},
  [1356] = {.lex_state = 10},
  [1357] = {.lex_state = 10},
  [1358] = {.lex_state = 10},
  [1359] = {.lex_state = 10},
  [1360] = {.lex_state = 10},
  [1361] = {.lex_state = 10},
  [1362] = {.lex_state = 10},
  [1363] = {.lex_state = 10},
  [1364] = {.lex_state = 10},
  [1365] = {.lex_state = 7},
  [1366] = {.lex_state = 14},
  [1367] = {.lex_state = 14},
  [1368] = {.lex_state = 10},
  [1369] = {.lex_state = 7},
  [1370] = {.lex_state = 7},
  [1371] = {.lex_state = 7},
  [1372] = {.lex_state = 10},
  [1373] = {.lex_state = 7},
  [1374] = {.lex_state = 7},
  [1375] = {.lex_state = 7},
  [1376] = {.lex_state = 7},
  [1377] = {.lex_state = 7},
  [1378] = {.lex_state = 7},
  [1379] = {.lex_state = 7},
  [1380] = {.lex_state = 7},
  [1381] = {.lex_state = 7},
  [1382] = {.lex_state = 7},
  [1383] = {.lex_state = 7},
  [1384] = {.lex_state = 7},
  [1385] = {.lex_state = 7},
  [1386] = {.lex_state = 7},
  [1387] = {.lex_state = 7},
  [1388] = {.lex_state = 7},
  [1389] = {.lex_state = 7},
  [1390] = {.lex_state = 7},
  [1391] = {.lex_state = 7},
  [1392] = {.lex_state = 7},
  [1393] = {.lex_state = 7},
  [1394] = {.lex_state = 7},
  [1395] = {.lex_state = 7},
  [1396] = {.lex_state = 11, .external_lex_state = 2},
  [1397] = {.lex_state = 7},
  [1398] = {.lex_state = 7},
  [1399] = {.lex_state = 7},
  [1400] = {.lex_state = 7},
  [1401] = {.lex_state = 11, .external_lex_state = 2},
  [1402] = {.lex_state = 7},
  [1403] = {.lex_state = 7},
  [1404] = {.lex_state = 7},
  [1405] = {.lex_state = 14},
  [1406] = {.lex_state = 7},
  [1407] = {.lex_state = 7},
  [1408] = {.lex_state = 7},
  [1409] = {.lex_state = 7},
  [1410] = {.lex_state = 7},
  [1411] = {.lex_state = 7},
  [1412] = {.lex_state = 7},
  [1413] = {.lex_state = 7},
  [1414] = {.lex_state = 7},
  [1415] = {.lex_state = 7},
  [1416] = {.lex_state = 7},
  [1417] = {.lex_state = 7},
  [1418] = {.lex_state = 7},
  [1419] = {.lex_state = 7},
  [1420] = {.lex_state = 7},
  [1421] = {.lex_state = 7},
  [1422] = {.lex_state = 7},
  [1423] = {.lex_state = 7},
  [1424] = {.lex_state = 7},
  [1425] = {.lex_state = 7},
  [1426] = {.lex_state = 7},
  [1427] = {.lex_state = 7},
  [1428] = {.lex_state = 14},
  [1429] = {.lex_state = 7},
  [1430] = {.lex_state = 7},
  [1431] = {.lex_state = 7},
  [1432] = {.lex_state = 7},
  [1433] = {.lex_state = 7},
  [1434] = {.lex_state = 7},
  [1435] = {.lex_state = 7},
  [1436] = {.lex_state = 7},
  [1437] = {.lex_state = 7},
  [1438] = {.lex_state = 7},
  [1439] = {.lex_state = 7},
  [1440] = {.lex_state = 7},
  [1441] = {.lex_state = 7},
  [1442] = {.lex_state = 7},
  [1443] = {.lex_state = 7},
  [1444] = {.lex_state = 7},
  [1445] = {.lex_state = 7},
  [1446] = {.lex_state = 7},
  [1447] = {.lex_state = 7},
  [1448] = {.lex_state = 14},
  [1449] = {.lex_state = 7},
  [1450] = {.lex_state = 7},
  [1451] = {.lex_state = 7},
  [1452] = {.lex_state = 7},
  [1453] = {.lex_state = 7},
  [1454] = {.lex_state = 7},
  [1455] = {.lex_state = 7},
  [1456] = {.lex_state = 7},
  [1457] = {.lex_state = 7},
  [1458] = {.lex_state = 7},
  [1459] = {.lex_state = 7},
  [1460] = {.lex_state = 7},
  [1461] = {.lex_state = 7},
  [1462] = {.lex_state = 14},
  [1463] = {.lex_state = 7},
  [1464] = {.lex_state = 11, .external_lex_state = 2},
  [1465] = {.lex_state = 8},
  [1466] = {.lex_state = 7},
  [1467] = {.lex_state = 7},
  [1468] = {.lex_state = 11, .external_lex_state = 2},
  [1469] = {.lex_state = 11, .external_lex_state = 2},
  [1470] = {.lex_state = 7},
  [1471] = {.lex_state = 7},
  [1472] = {.lex_state = 7},
  [1473] = {.lex_state = 7},
  [1474] = {.lex_state = 7},
  [1475] = {.lex_state = 7},
  [1476] = {.lex_state = 7},
  [1477] = {.lex_state = 7},
  [1478] = {.lex_state = 7},
  [1479] = {.lex_state = 7},
  [1480] = {.lex_state = 7},
  [1481] = {.lex_state = 7},
  [1482] = {.lex_state = 7},
  [1483] = {.lex_state = 7},
  [1484] = {.lex_state = 7},
  [1485] = {.lex_state = 7},
  [1486] = {.lex_state = 7},
  [1487] = {.lex_state = 7},
  [1488] = {.lex_state = 7},
  [1489] = {.lex_state = 7},
  [1490] = {.lex_state = 7},
  [1491] = {.lex_state = 7},
  [1492] = {.lex_state = 7},
  [1493] = {.lex_state = 7},
  [1494] = {.lex_state = 7},
  [1495] = {.lex_state = 14},
  [1496] = {.lex_state = 7},
  [1497] = {.lex_state = 7},
  [1498] = {.lex_state = 14},
  [1499] = {.lex_state = 7},
  [1500] = {.lex_state = 14},
  [1501] = {.lex_state = 7},
  [1502] = {.lex_state = 7},
  [1503] = {.lex_state = 7},
  [1504] = {.lex_state = 11, .external_lex_state = 2},
  [1505] = {.lex_state = 7},
  [1506] = {.lex_state = 7},
  [1507] = {.lex_state = 7},
  [1508] = {.lex_state = 7},
  [1509] = {.lex_state = 11, .external_lex_state = 2},
  [1510] = {.lex_state = 7},
  [1511] = {.lex_state = 7},
  [1512] = {.lex_state = 9},
  [1513] = {.lex_state = 7},
  [1514] = {.lex_state = 7},
  [1515] = {.lex_state = 7},
  [1516] = {.lex_state = 7},
  [1517] = {.lex_state = 7},
  [1518] = {.lex_state = 7},
  [1519] = {.lex_state = 7},
  [1520] = {.lex_state = 11, .external_lex_state = 2},
  [1521] = {.lex_state = 7},
  [1522] = {.lex_state = 7},
  [1523] = {.lex_state = 11, .external_lex_state = 2},
  [1524] = {.lex_state = 7},
  [1525] = {.lex_state = 7},
  [1526] = {.lex_state = 7},
  [1527] = {.lex_state = 7},
  [1528] = {.lex_state = 9},
  [1529] = {.lex_state = 14},
  [1530] = {.lex_state = 9},
  [1531] = {.lex_state = 14},
  [1532] = {.lex_state = 14},
  [1533] = {.lex_state = 14},
  [1534] = {.lex_state = 14},
  [1535] = {.lex_state = 14},
  [1536] = {.lex_state = 9},
  [1537] = {.lex_state = 14},
  [1538] = {.lex_state = 7},
  [1539] = {.lex_state = 8},
  [1540] = {.lex_state = 9},
  [1541] = {.lex_state = 7},
  [1542] = {.lex_state = 8},
  [1543] = {.lex_state = 9},
  [1544] = {.lex_state = 9},
  [1545] = {.lex_state = 9},
  [1546] = {.lex_state = 9},
  [1547] = {.lex_state = 7},
  [1548] = {.lex_state = 9},
  [1549] = {.lex_state = 7},
  [1550] = {.lex_state = 8},
  [1551] = {.lex_state = 8},
  [1552] = {.lex_state = 7},
  [1553] = {.lex_state = 8},
  [1554] = {.lex_state = 7},
  [1555] = {.lex_state = 7},
  [1556] = {.lex_state = 9},
  [1557] = {.lex_state = 9},
  [1558] = {.lex_state = 7},
  [1559] = {.lex_state = 7},
  [1560] = {.lex_state = 7},
  [1561] = {.lex_state = 7},
  [1562] = {.lex_state = 7},
  [1563] = {.lex_state = 8},
  [1564] = {.lex_state = 7},
  [1565] = {.lex_state = 14},
  [1566] = {.lex_state = 7},
  [1567] = {.lex_state = 8},
  [1568] = {.lex_state = 7},
  [1569] = {.lex_state = 8},
  [1570] = {.lex_state = 8},
  [1571] = {.lex_state = 8},
  [1572] = {.lex_state = 7},
  [1573] = {.lex_state = 8},
  [1574] = {.lex_state = 7},
  [1575] = {.lex_state = 8},
  [1576] = {.lex_state = 7},
  [1577] = {.lex_state = 7},
  [1578] = {.lex_state = 8},
  [1579] = {.lex_state = 7},
  [1580] = {.lex_state = 7},
  [1581] = {.lex_state = 7},
  [1582] = {.lex_state = 7},
  [1583] = {.lex_state = 8},
  [1584] = {.lex_state = 14},
  [1585] = {.lex_state = 8},
  [1586] = {.lex_state = 8},
  [1587] = {.lex_state = 8},
  [1588] = {.lex_state = 8},
  [1589] = {.lex_state = 7},
  [1590] = {.lex_state = 8},
  [1591] = {.lex_state = 14},
  [1592] = {.lex_state = 7},
  [1593] = {.lex_state = 7},
  [1594] = {.lex_state = 8},
  [1595] = {.lex_state = 7},
  [1596] = {.lex_state = 7},
  [1597] = {.lex_state = 8},
  [1598] = {.lex_state = 8},
  [1599] = {.lex_state = 7},
  [1600] = {.lex_state = 7},
  [1601] = {.lex_state = 9},
  [1602] = {.lex_state = 20},
  [1603] = {.lex_state = 20},
  [1604] = {.lex_state = 7},
  [1605] = {.lex_state = 7},
  [1606] = {.lex_state = 7},
  [1607] = {.lex_state = 7},
  [1608] = {.lex_state = 7},
  [1609] = {.lex_state = 7},
  [1610] = {.lex_state = 20},
  [1611] = {.lex_state = 20},
  [1612] = {.lex_state = 8},
  [1613] = {.lex_state = 7},
  [1614] = {.lex_state = 7},
  [1615] = {.lex_state = 8},
  [1616] = {.lex_state = 20},
  [1617] = {.lex_state = 7},
  [1618] = {.lex_state = 8},
  [1619] = {.lex_state = 8},
  [1620] = {.lex_state = 8},
  [1621] = {.lex_state = 7},
  [1622] = {.lex_state = 20},
  [1623] = {.lex_state = 7},
  [1624] = {.lex_state = 7},
  [1625] = {.lex_state = 20},
  [1626] = {.lex_state = 8},
  [1627] = {.lex_state = 7},
  [1628] = {.lex_state = 8},
  [1629] = {.lex_state = 20},
  [1630] = {.lex_state = 7},
  [1631] = {.lex_state = 7},
  [1632] = {.lex_state = 7},
  [1633] = {.lex_state = 8},
  [1634] = {.lex_state = 20},
  [1635] = {.lex_state = 8},
  [1636] = {.lex_state = 20},
  [1637] = {.lex_state = 8},
  [1638] = {.lex_state = 7},
  [1639] = {.lex_state = 7},
  [1640] = {.lex_state = 7},
  [1641] = {.lex_state = 7},
  [1642] = {.lex_state = 20},
  [1643] = {.lex_state = 14},
  [1644] = {.lex_state = 8},
  [1645] = {.lex_state = 7},
  [1646] = {.lex_state = 20},
  [1647] = {.lex_state = 8},
  [1648] = {.lex_state = 8},
  [1649] = {.lex_state = 8},
  [1650] = {.lex_state = 8},
  [1651] = {.lex_state = 8},
  [1652] = {.lex_state = 8},
  [1653] = {.lex_state = 7},
  [1654] = {.lex_state = 7},
  [1655] = {.lex_state = 8},
  [1656] = {.lex_state = 7},
  [1657] = {.lex_state = 8},
  [1658] = {.lex_state = 7},
  [1659] = {.lex_state = 8},
  [1660] = {.lex_state = 8},
  [1661] = {.lex_state = 8},
  [1662] = {.lex_state = 7},
  [1663] = {.lex_state = 8},
  [1664] = {.lex_state = 8},
  [1665] = {.lex_state = 7},
  [1666] = {.lex_state = 8},
  [1667] = {.lex_state = 8},
  [1668] = {.lex_state = 8},
  [1669] = {.lex_state = 8},
  [1670] = {.lex_state = 8},
  [1671] = {.lex_state = 8},
  [1672] = {.lex_state = 8},
  [1673] = {.lex_state = 8},
  [1674] = {.lex_state = 8},
  [1675] = {.lex_state = 8},
  [1676] = {.lex_state = 8},
  [1677] = {.lex_state = 8},
  [1678] = {.lex_state = 7},
  [1679] = {.lex_state = 7},
  [1680] = {.lex_state = 8},
  [1681] = {.lex_state = 8},
  [1682] = {.lex_state = 8},
  [1683] = {.lex_state = 8},
  [1684] = {.lex_state = 8},
  [1685] = {.lex_state = 8},
  [1686] = {.lex_state = 8},
  [1687] = {.lex_state = 7},
  [1688] = {.lex_state = 7},
  [1689] = {.lex_state = 8},
  [1690] = {.lex_state = 7},
  [1691] = {.lex_state = 8},
  [1692] = {.lex_state = 8},
  [1693] = {.lex_state = 8},
  [1694] = {.lex_state = 8},
  [1695] = {.lex_state = 7},
  [1696] = {.lex_state = 8},
  [1697] = {.lex_state = 8},
  [1698] = {.lex_state = 8},
  [1699] = {.lex_state = 8},
  [1700] = {.lex_state = 8},
  [1701] = {.lex_state = 8},
  [1702] = {.lex_state = 7},
  [1703] = {.lex_state = 7},
  [1704] = {.lex_state = 7},
  [1705] = {.lex_state = 7},
  [1706] = {.lex_state = 8},
  [1707] = {.lex_state = 8},
  [1708] = {.lex_state = 7},
  [1709] = {.lex_state = 7},
  [1710] = {.lex_state = 7},
  [1711] = {.lex_state = 7},
  [1712] = {.lex_state = 7},
  [1713] = {.lex_state = 7},
  [1714] = {.lex_state = 7},
  [1715] = {.lex_state = 7},
  [1716] = {.lex_state = 7},
  [1717] = {.lex_state = 7},
  [1718] = {.lex_state = 7},
  [1719] = {.lex_state = 8},
  [1720] = {.lex_state = 8},
  [1721] = {.lex_state = 8},
  [1722] = {.lex_state = 8},
  [1723] = {.lex_state = 8},
  [1724] = {.lex_state = 8},
  [1725] = {.lex_state = 8},
  [1726] = {.lex_state = 7},
  [1727] = {.lex_state = 8},
  [1728] = {.lex_state = 8},
  [1729] = {.lex_state = 8},
  [1730] = {.lex_state = 8},
  [1731] = {.lex_state = 8},
  [1732] = {.lex_state = 14},
  [1733] = {.lex_state = 8},
  [1734] = {.lex_state = 8},
  [1735] = {.lex_state = 8},
  [1736] = {.lex_state = 8},
  [1737] = {.lex_state = 8},
  [1738] = {.lex_state = 8},
  [1739] = {.lex_state = 8},
  [1740] = {.lex_state = 7},
  [1741] = {.lex_state = 8},
  [1742] = {.lex_state = 7},
  [1743] = {.lex_state = 8},
  [1744] = {.lex_state = 8},
  [1745] = {.lex_state = 8},
  [1746] = {.lex_state = 8},
  [1747] = {.lex_state = 8},
  [1748] = {.lex_state = 8},
  [1749] = {.lex_state = 8},
  [1750] = {.lex_state = 8},
  [1751] = {.lex_state = 7},
  [1752] = {.lex_state = 8},
  [1753] = {.lex_state = 8},
  [1754] = {.lex_state = 8},
  [1755] = {.lex_state = 7},
  [1756] = {.lex_state = 8},
  [1757] = {.lex_state = 8},
  [1758] = {.lex_state = 7},
  [1759] = {.lex_state = 8},
  [1760] = {.lex_state = 8},
  [1761] = {.lex_state = 8},
  [1762] = {.lex_state = 8},
  [1763] = {.lex_state = 8},
  [1764] = {.lex_state = 8},
  [1765] = {.lex_state = 8},
  [1766] = {.lex_state = 8},
  [1767] = {.lex_state = 8},
  [1768] = {.lex_state = 8},
  [1769] = {.lex_state = 8},
  [1770] = {.lex_state = 8},
  [1771] = {.lex_state = 8},
  [1772] = {.lex_state = 8},
  [1773] = {.lex_state = 8},
  [1774] = {.lex_state = 8},
  [1775] = {.lex_state = 7},
  [1776] = {.lex_state = 8},
  [1777] = {.lex_state = 8},
  [1778] = {.lex_state = 8},
  [1779] = {.lex_state = 8},
  [1780] = {.lex_state = 8},
  [1781] = {.lex_state = 8},
  [1782] = {.lex_state = 8},
  [1783] = {.lex_state = 8},
  [1784] = {.lex_state = 7},
  [1785] = {.lex_state = 8},
  [1786] = {.lex_state = 8},
  [1787] = {.lex_state = 8},
  [1788] = {.lex_state = 8},
  [1789] = {.lex_state = 8},
  [1790] = {.lex_state = 8},
  [1791] = {.lex_state = 8},
  [1792] = {.lex_state = 8},
  [1793] = {.lex_state = 8},
  [1794] = {.lex_state = 8},
  [1795] = {.lex_state = 8},
  [1796] = {.lex_state = 8},
  [1797] = {.lex_state = 8},
  [1798] = {.lex_state = 8},
  [1799] = {.lex_state = 7},
  [1800] = {.lex_state = 7},
  [1801] = {.lex_state = 7},
  [1802] = {.lex_state = 7},
  [1803] = {.lex_state = 8},
  [1804] = {.lex_state = 7},
  [1805] = {.lex_state = 8},
  [1806] = {.lex_state = 7},
  [1807] = {.lex_state = 7},
  [1808] = {.lex_state = 7},
  [1809] = {.lex_state = 8},
  [1810] = {.lex_state = 7},
  [1811] = {.lex_state = 14},
  [1812] = {.lex_state = 7},
  [1813] = {.lex_state = 7},
  [1814] = {.lex_state = 7},
  [1815] = {.lex_state = 7},
  [1816] = {.lex_state = 7},
  [1817] = {.lex_state = 7},
  [1818] = {.lex_state = 7},
  [1819] = {.lex_state = 7},
  [1820] = {.lex_state = 7},
  [1821] = {.lex_state = 8},
  [1822] = {.lex_state = 7},
  [1823] = {.lex_state = 7},
  [1824] = {.lex_state = 7},
  [1825] = {.lex_state = 7},
  [1826] = {.lex_state = 7},
  [1827] = {.lex_state = 7},
  [1828] = {.lex_state = 7},
  [1829] = {.lex_state = 7},
  [1830] = {.lex_state = 8},
  [1831] = {.lex_state = 7},
  [1832] = {.lex_state = 7},
  [1833] = {.lex_state = 7},
  [1834] = {.lex_state = 7},
  [1835] = {.lex_state = 8},
  [1836] = {.lex_state = 8},
  [1837] = {.lex_state = 7},
  [1838] = {.lex_state = 7},
  [1839] = {.lex_state = 7},
  [1840] = {.lex_state = 7},
  [1841] = {.lex_state = 7},
  [1842] = {.lex_state = 7},
  [1843] = {.lex_state = 7},
  [1844] = {.lex_state = 7},
  [1845] = {.lex_state = 7},
  [1846] = {.lex_state = 7},
  [1847] = {.lex_state = 7},
  [1848] = {.lex_state = 7},
  [1849] = {.lex_state = 7},
  [1850] = {.lex_state = 7},
  [1851] = {.lex_state = 7},
  [1852] = {.lex_state = 7},
  [1853] = {.lex_state = 7},
  [1854] = {.lex_state = 7},
  [1855] = {.lex_state = 7},
  [1856] = {.lex_state = 8},
  [1857] = {.lex_state = 8},
  [1858] = {.lex_state = 8},
  [1859] = {.lex_state = 8},
  [1860] = {.lex_state = 8},
  [1861] = {.lex_state = 8},
  [1862] = {.lex_state = 7},
  [1863] = {.lex_state = 8},
  [1864] = {.lex_state = 7},
  [1865] = {.lex_state = 7},
  [1866] = {.lex_state = 8},
  [1867] = {.lex_state = 8},
  [1868] = {.lex_state = 8},
  [1869] = {.lex_state = 7},
  [1870] = {.lex_state = 14},
  [1871] = {.lex_state = 7},
  [1872] = {.lex_state = 7},
  [1873] = {.lex_state = 7},
  [1874] = {.lex_state = 7},
  [1875] = {.lex_state = 7},
  [1876] = {.lex_state = 7},
  [1877] = {.lex_state = 7},
  [1878] = {.lex_state = 14},
  [1879] = {.lex_state = 7},
  [1880] = {.lex_state = 7},
  [1881] = {.lex_state = 7},
  [1882] = {.lex_state = 7},
  [1883] = {.lex_state = 7},
  [1884] = {.lex_state = 7},
  [1885] = {.lex_state = 7},
  [1886] = {.lex_state = 7},
  [1887] = {.lex_state = 7},
  [1888] = {.lex_state = 7},
  [1889] = {.lex_state = 7},
  [1890] = {.lex_state = 7},
  [1891] = {.lex_state = 14},
  [1892] = {.lex_state = 7},
  [1893] = {.lex_state = 14},
  [1894] = {.lex_state = 7},
  [1895] = {.lex_state = 7},
  [1896] = {.lex_state = 7},
  [1897] = {.lex_state = 7},
  [1898] = {.lex_state = 14},
  [1899] = {.lex_state = 7},
  [1900] = {.lex_state = 7},
  [1901] = {.lex_state = 14},
  [1902] = {.lex_state = 14},
  [1903] = {.lex_state = 14},
  [1904] = {.lex_state = 14},
  [1905] = {.lex_state = 14},
  [1906] = {.lex_state = 14},
  [1907] = {.lex_state = 14},
  [1908] = {.lex_state = 14},
  [1909] = {.lex_state = 14},
  [1910] = {.lex_state = 14},
  [1911] = {.lex_state = 14},
  [1912] = {.lex_state = 14},
  [1913] = {.lex_state = 14},
  [1914] = {.lex_state = 14},
  [1915] = {.lex_state = 14},
  [1916] = {.lex_state = 14},
  [1917] = {.lex_state = 14},
  [1918] = {.lex_state = 14},
  [1919] = {.lex_state = 14},
  [1920] = {.lex_state = 14},
  [1921] = {.lex_state = 14},
  [1922] = {.lex_state = 16},
  [1923] = {.lex_state = 27},
  [1924] = {.lex_state = 27},
  [1925] = {.lex_state = 16},
  [1926] = {.lex_state = 16},
  [1927] = {.lex_state = 16},
  [1928] = {.lex_state = 16},
  [1929] = {.lex_state = 16},
  [1930] = {.lex_state = 16},
  [1931] = {.lex_state = 16},
  [1932] = {.lex_state = 16},
  [1933] = {.lex_state = 27},
  [1934] = {.lex_state = 16},
  [1935] = {.lex_state = 27},
  [1936] = {.lex_state = 27},
  [1937] = {.lex_state = 16},
  [1938] = {.lex_state = 27},
  [1939] = {.lex_state = 16},
  [1940] = {.lex_state = 16},
  [1941] = {.lex_state = 14},
  [1942] = {.lex_state = 14},
  [1943] = {.lex_state = 14},
  [1944] = {.lex_state = 14},
  [1945] = {.lex_state = 14},
  [1946] = {.lex_state = 14},
  [1947] = {.lex_state = 14},
  [1948] = {.lex_state = 27},
  [1949] = {.lex_state = 14},
  [1950] = {.lex_state = 14},
  [1951] = {.lex_state = 16},
  [1952] = {.lex_state = 14},
  [1953] = {.lex_state = 14},
  [1954] = {.lex_state = 14},
  [1955] = {.lex_state = 14},
  [1956] = {.lex_state = 16},
  [1957] = {.lex_state = 14},
  [1958] = {.lex_state = 27},
  [1959] = {.lex_state = 27},
  [1960] = {.lex_state = 27},
  [1961] = {.lex_state = 27},
  [1962] = {.lex_state = 14},
  [1963] = {.lex_state = 14},
  [1964] = {.lex_state = 27},
  [1965] = {.lex_state = 27},
  [1966] = {.lex_state = 14},
  [1967] = {.lex_state = 16},
  [1968] = {.lex_state = 16},
  [1969] = {.lex_state = 27},
  [1970] = {.lex_state = 14},
  [1971] = {.lex_state = 27},
  [1972] = {.lex_state = 14},
  [1973] = {.lex_state = 27},
  [1974] = {.lex_state = 16},
  [1975] = {.lex_state = 27},
  [1976] = {.lex_state = 27},
  [1977] = {.lex_state = 27},
  [1978] = {.lex_state = 14},
  [1979] = {.lex_state = 27},
  [1980] = {.lex_state = 14},
  [1981] = {.lex_state = 27},
  [1982] = {.lex_state = 27},
  [1983] = {.lex_state = 16},
  [1984] = {.lex_state = 27},
  [1985] = {.lex_state = 27},
  [1986] = {.lex_state = 27},
  [1987] = {.lex_state = 14},
  [1988] = {.lex_state = 14},
  [1989] = {.lex_state = 27},
  [1990] = {.lex_state = 16},
  [1991] = {.lex_state = 27},
  [1992] = {.lex_state = 27},
  [1993] = {.lex_state = 27},
  [1994] = {.lex_state = 27},
  [1995] = {.lex_state = 27},
  [1996] = {.lex_state = 27},
  [1997] = {.lex_state = 27},
  [1998] = {.lex_state = 27},
  [1999] = {.lex_state = 27},
  [2000] = {.lex_state = 14},
  [2001] = {.lex_state = 27},
  [2002] = {.lex_state = 27},
  [2003] = {.lex_state = 27},
  [2004] = {.lex_state = 27},
  [2005] = {.lex_state = 27},
  [2006] = {.lex_state = 27},
  [2007] = {.lex_state = 27},
  [2008] = {.lex_state = 27},
  [2009] = {.lex_state = 27},
  [2010] = {.lex_state = 27},
  [2011] = {.lex_state = 27},
  [2012] = {.lex_state = 27},
  [2013] = {.lex_state = 14},
  [2014] = {.lex_state = 27},
  [2015] = {.lex_state = 14},
  [2016] = {.lex_state = 27},
  [2017] = {.lex_state = 14},
  [2018] = {.lex_state = 16},
  [2019] = {.lex_state = 27},
  [2020] = {.lex_state = 27},
  [2021] = {.lex_state = 14},
  [2022] = {.lex_state = 16},
  [2023] = {.lex_state = 16},
  [2024] = {.lex_state = 16},
  [2025] = {.lex_state = 27},
  [2026] = {.lex_state = 16},
  [2027] = {.lex_state = 16},
  [2028] = {.lex_state = 16},
  [2029] = {.lex_state = 16},
  [2030] = {.lex_state = 16},
  [2031] = {.lex_state = 16},
  [2032] = {.lex_state = 17},
  [2033] = {.lex_state = 16},
  [2034] = {.lex_state = 27},
  [2035] = {.lex_state = 16},
  [2036] = {.lex_state = 16},
  [2037] = {.lex_state = 27},
  [2038] = {.lex_state = 14},
  [2039] = {.lex_state = 14},
  [2040] = {.lex_state = 14},
  [2041] = {.lex_state = 14},
  [2042] = {.lex_state = 14},
  [2043] = {.lex_state = 14},
  [2044] = {.lex_state = 14},
  [2045] = {.lex_state = 14},
  [2046] = {.lex_state = 14},
  [2047] = {.lex_state = 14},
  [2048] = {.lex_state = 14},
  [2049] = {.lex_state = 14},
  [2050] = {.lex_state = 14},
  [2051] = {.lex_state = 14},
  [2052] = {.lex_state = 14},
  [2053] = {.lex_state = 14},
  [2054] = {.lex_state = 16},
  [2055] = {.lex_state = 16},
  [2056] = {.lex_state = 14},
  [2057] = {.lex_state = 21},
  [2058] = {.lex_state = 14},
  [2059] = {.lex_state = 14},
  [2060] = {.lex_state = 14},
  [2061] = {.lex_state = 14},
  [2062] = {.lex_state = 16},
  [2063] = {.lex_state = 16},
  [2064] = {.lex_state = 27},
  [2065] = {.lex_state = 27},
  [2066] = {.lex_state = 17},
  [2067] = {.lex_state = 17},
  [2068] = {.lex_state = 17},
  [2069] = {.lex_state = 17},
  [2070] = {.lex_state = 14},
  [2071] = {.lex_state = 14},
  [2072] = {.lex_state = 21},
  [2073] = {.lex_state = 14},
  [2074] = {.lex_state = 14},
  [2075] = {.lex_state = 14},
  [2076] = {.lex_state = 14},
  [2077] = {.lex_state = 14},
  [2078] = {.lex_state = 27},
  [2079] = {.lex_state = 14},
  [2080] = {.lex_state = 14},
  [2081] = {.lex_state = 14},
  [2082] = {.lex_state = 23},
  [2083] = {.lex_state = 14},
  [2084] = {.lex_state = 16},
  [2085] = {.lex_state = 23},
  [2086] = {.lex_state = 16},
  [2087] = {.lex_state = 27},
  [2088] = {.lex_state = 21},
  [2089] = {.lex_state = 21},
  [2090] = {.lex_state = 16},
  [2091] = {.lex_state = 16},
  [2092] = {.lex_state = 16},
  [2093] = {.lex_state = 27},
  [2094] = {.lex_state = 16},
  [2095] = {.lex_state = 14},
  [2096] = {.lex_state = 27},
  [2097] = {.lex_state = 16},
  [2098] = {.lex_state = 27},
  [2099] = {.lex_state = 27},
  [2100] = {.lex_state = 27},
  [2101] = {.lex_state = 27},
  [2102] = {.lex_state = 27},
  [2103] = {.lex_state = 27},
  [2104] = {.lex_state = 27},
  [2105] = {.lex_state = 27},
  [2106] = {.lex_state = 27},
  [2107] = {.lex_state = 27},
  [2108] = {.lex_state = 27},
  [2109] = {.lex_state = 23},
  [2110] = {.lex_state = 16},
  [2111] = {.lex_state = 27},
  [2112] = {.lex_state = 16},
  [2113] = {.lex_state = 27},
  [2114] = {.lex_state = 27},
  [2115] = {.lex_state = 27},
  [2116] = {.lex_state = 27},
  [2117] = {.lex_state = 23},
  [2118] = {.lex_state = 16},
  [2119] = {.lex_state = 27},
  [2120] = {.lex_state = 27},
  [2121] = {.lex_state = 27},
  [2122] = {.lex_state = 27},
  [2123] = {.lex_state = 23},
  [2124] = {.lex_state = 27},
  [2125] = {.lex_state = 10},
  [2126] = {.lex_state = 27},
  [2127] = {.lex_state = 14},
  [2128] = {.lex_state = 27},
  [2129] = {.lex_state = 27},
  [2130] = {.lex_state = 14},
  [2131] = {.lex_state = 27},
  [2132] = {.lex_state = 16},
  [2133] = {.lex_state = 14},
  [2134] = {.lex_state = 27},
  [2135] = {.lex_state = 27},
  [2136] = {.lex_state = 27},
  [2137] = {.lex_state = 27},
  [2138] = {.lex_state = 23},
  [2139] = {.lex_state = 17},
  [2140] = {.lex_state = 27},
  [2141] = {.lex_state = 14},
  [2142] = {.lex_state = 27},
  [2143] = {.lex_state = 27},
  [2144] = {.lex_state = 27},
  [2145] = {.lex_state = 27},
  [2146] = {.lex_state = 14},
  [2147] = {.lex_state = 27},
  [2148] = {.lex_state = 14},
  [2149] = {.lex_state = 14},
  [2150] = {.lex_state = 14},
  [2151] = {.lex_state = 17},
  [2152] = {.lex_state = 16},
  [2153] = {.lex_state = 10},
  [2154] = {.lex_state = 14},
  [2155] = {.lex_state = 16},
  [2156] = {.lex_state = 23},
  [2157] = {.lex_state = 14},
  [2158] = {.lex_state = 17},
  [2159] = {.lex_state = 17},
  [2160] = {.lex_state = 14},
  [2161] = {.lex_state = 17},
  [2162] = {.lex_state = 17},
  [2163] = {.lex_state = 10},
  [2164] = {.lex_state = 6},
  [2165] = {.lex_state = 14},
  [2166] = {.lex_state = 14},
  [2167] = {.lex_state = 16},
  [2168] = {.lex_state = 16},
  [2169] = {.lex_state = 14},
  [2170] = {.lex_state = 14},
  [2171] = {.lex_state = 14},
  [2172] = {.lex_state = 14},
  [2173] = {.lex_state = 6},
  [2174] = {.lex_state = 14},
  [2175] = {.lex_state = 16},
  [2176] = {.lex_state = 14},
  [2177] = {.lex_state = 10},
  [2178] = {.lex_state = 14},
  [2179] = {.lex_state = 14},
  [2180] = {.lex_state = 14},
  [2181] = {.lex_state = 14},
  [2182] = {.lex_state = 14},
  [2183] = {.lex_state = 14},
  [2184] = {.lex_state = 6},
  [2185] = {.lex_state = 14},
  [2186] = {.lex_state = 70},
  [2187] = {.lex_state = 14},
  [2188] = {.lex_state = 16},
  [2189] = {.lex_state = 14},
  [2190] = {.lex_state = 14},
  [2191] = {.lex_state = 10},
  [2192] = {.lex_state = 16},
  [2193] = {.lex_state = 16},
  [2194] = {.lex_state = 16},
  [2195] = {.lex_state = 14},
  [2196] = {.lex_state = 14},
  [2197] = {.lex_state = 14},
  [2198] = {.lex_state = 14},
  [2199] = {.lex_state = 14},
  [2200] = {.lex_state = 14},
  [2201] = {.lex_state = 14},
  [2202] = {.lex_state = 14},
  [2203] = {.lex_state = 23},
  [2204] = {.lex_state = 23},
  [2205] = {.lex_state = 14},
  [2206] = {.lex_state = 23},
  [2207] = {.lex_state = 14},
  [2208] = {.lex_state = 23},
  [2209] = {.lex_state = 23},
  [2210] = {.lex_state = 23},
  [2211] = {.lex_state = 14},
  [2212] = {.lex_state = 23},
  [2213] = {.lex_state = 14},
  [2214] = {.lex_state = 14},
  [2215] = {.lex_state = 14},
  [2216] = {.lex_state = 14},
  [2217] = {.lex_state = 23},
  [2218] = {.lex_state = 14},
  [2219] = {.lex_state = 14},
  [2220] = {.lex_state = 14},
  [2221] = {.lex_state = 14},
  [2222] = {.lex_state = 14},
  [2223] = {.lex_state = 14},
  [2224] = {.lex_state = 14},
  [2225] = {.lex_state = 14},
  [2226] = {.lex_state = 14},
  [2227] = {.lex_state = 14},
  [2228] = {.lex_state = 14},
  [2229] = {.lex_state = 14},
  [2230] = {.lex_state = 14},
  [2231] = {.lex_state = 14},
  [2232] = {.lex_state = 23},
  [2233] = {.lex_state = 21},
  [2234] = {.lex_state = 14},
  [2235] = {.lex_state = 14},
  [2236] = {.lex_state = 14},
  [2237] = {.lex_state = 14},
  [2238] = {.lex_state = 14},
  [2239] = {.lex_state = 14},
  [2240] = {.lex_state = 14},
  [2241] = {.lex_state = 23},
  [2242] = {.lex_state = 10},
  [2243] = {.lex_state = 14},
  [2244] = {.lex_state = 14},
  [2245] = {.lex_state = 14},
  [2246] = {.lex_state = 14},
  [2247] = {.lex_state = 23},
  [2248] = {.lex_state = 14},
  [2249] = {.lex_state = 23},
  [2250] = {.lex_state = 14},
  [2251] = {.lex_state = 14},
  [2252] = {.lex_state = 23},
  [2253] = {.lex_state = 70},
  [2254] = {.lex_state = 70},
  [2255] = {.lex_state = 70},
  [2256] = {.lex_state = 14},
  [2257] = {.lex_state = 70},
  [2258] = {.lex_state = 14},
  [2259] = {.lex_state = 14},
  [2260] = {.lex_state = 14},
  [2261] = {.lex_state = 70},
  [2262] = {.lex_state = 23},
  [2263] = {.lex_state = 23},
  [2264] = {.lex_state = 70},
  [2265] = {.lex_state = 14},
  [2266] = {.lex_state = 16},
  [2267] = {.lex_state = 70},
  [2268] = {.lex_state = 70},
  [2269] = {.lex_state = 23},
  [2270] = {.lex_state = 70},
  [2271] = {.lex_state = 70},
  [2272] = {.lex_state = 14},
  [2273] = {.lex_state = 14},
  [2274] = {.lex_state = 70},
  [2275] = {.lex_state = 70},
  [2276] = {.lex_state = 70},
  [2277] = {.lex_state = 70},
  [2278] = {.lex_state = 70},
  [2279] = {.lex_state = 16},
  [2280] = {.lex_state = 3},
  [2281] = {.lex_state = 70},
  [2282] = {.lex_state = 70},
  [2283] = {.lex_state = 70},
  [2284] = {.lex_state = 70},
  [2285] = {.lex_state = 3},
  [2286] = {.lex_state = 10},
  [2287] = {.lex_state = 17},
  [2288] = {.lex_state = 14},
  [2289] = {.lex_state = 3},
  [2290] = {.lex_state = 23},
  [2291] = {.lex_state = 23},
  [2292] = {.lex_state = 70},
  [2293] = {.lex_state = 70},
  [2294] = {.lex_state = 70},
  [2295] = {.lex_state = 70},
  [2296] = {.lex_state = 70},
  [2297] = {.lex_state = 70},
  [2298] = {.lex_state = 70},
  [2299] = {.lex_state = 70},
  [2300] = {.lex_state = 10},
  [2301] = {.lex_state = 70},
  [2302] = {.lex_state = 23},
  [2303] = {.lex_state = 167},
  [2304] = {.lex_state = 14},
  [2305] = {.lex_state = 70},
  [2306] = {.lex_state = 70},
  [2307] = {.lex_state = 10},
  [2308] = {.lex_state = 14},
  [2309] = {.lex_state = 23},
  [2310] = {.lex_state = 14},
  [2311] = {.lex_state = 19},
  [2312] = {.lex_state = 14},
  [2313] = {.lex_state = 14},
  [2314] = {.lex_state = 19},
  [2315] = {.lex_state = 14},
  [2316] = {.lex_state = 14},
  [2317] = {.lex_state = 14},
  [2318] = {.lex_state = 14},
  [2319] = {.lex_state = 14},
  [2320] = {.lex_state = 14},
  [2321] = {.lex_state = 17},
  [2322] = {.lex_state = 14},
  [2323] = {.lex_state = 14},
  [2324] = {.lex_state = 14},
  [2325] = {.lex_state = 14},
  [2326] = {.lex_state = 14},
  [2327] = {.lex_state = 14},
  [2328] = {.lex_state = 14},
  [2329] = {.lex_state = 13},
  [2330] = {.lex_state = 17},
  [2331] = {.lex_state = 14},
  [2332] = {.lex_state = 14},
  [2333] = {.lex_state = 13},
  [2334] = {.lex_state = 14},
  [2335] = {.lex_state = 14},
  [2336] = {.lex_state = 27},
  [2337] = {.lex_state = 14},
  [2338] = {.lex_state = 27},
  [2339] = {.lex_state = 27},
  [2340] = {.lex_state = 27},
  [2341] = {.lex_state = 27},
  [2342] = {.lex_state = 14},
  [2343] = {.lex_state = 27},
  [2344] = {.lex_state = 27},
  [2345] = {.lex_state = 14},
  [2346] = {.lex_state = 14},
  [2347] = {.lex_state = 70},
  [2348] = {.lex_state = 14},
  [2349] = {.lex_state = 14},
  [2350] = {.lex_state = 14},
  [2351] = {.lex_state = 14},
  [2352] = {.lex_state = 19},
  [2353] = {.lex_state = 27},
  [2354] = {.lex_state = 14},
  [2355] = {.lex_state = 14},
  [2356] = {.lex_state = 14},
  [2357] = {.lex_state = 14},
  [2358] = {.lex_state = 70},
  [2359] = {.lex_state = 14},
  [2360] = {.lex_state = 27},
  [2361] = {.lex_state = 14},
  [2362] = {.lex_state = 27},
  [2363] = {.lex_state = 14},
  [2364] = {.lex_state = 17},
  [2365] = {.lex_state = 14},
  [2366] = {.lex_state = 14},
  [2367] = {.lex_state = 27},
  [2368] = {.lex_state = 14},
  [2369] = {.lex_state = 27},
  [2370] = {.lex_state = 27},
  [2371] = {.lex_state = 14},
  [2372] = {.lex_state = 27},
  [2373] = {.lex_state = 14},
  [2374] = {.lex_state = 19},
  [2375] = {.lex_state = 27},
  [2376] = {.lex_state = 19},
  [2377] = {.lex_state = 3},
  [2378] = {.lex_state = 14},
  [2379] = {.lex_state = 27},
  [2380] = {.lex_state = 14},
  [2381] = {.lex_state = 3},
  [2382] = {.lex_state = 27},
  [2383] = {.lex_state = 14},
  [2384] = {.lex_state = 3},
  [2385] = {.lex_state = 14},
  [2386] = {.lex_state = 19},
  [2387] = {.lex_state = 14},
  [2388] = {.lex_state = 27},
  [2389] = {.lex_state = 17},
  [2390] = {.lex_state = 14},
  [2391] = {.lex_state = 17},
  [2392] = {.lex_state = 17},
  [2393] = {.lex_state = 17},
  [2394] = {.lex_state = 14},
  [2395] = {.lex_state = 23},
  [2396] = {.lex_state = 14},
  [2397] = {.lex_state = 14},
  [2398] = {.lex_state = 14},
  [2399] = {.lex_state = 14},
  [2400] = {.lex_state = 14},
  [2401] = {.lex_state = 14},
  [2402] = {.lex_state = 14},
  [2403] = {.lex_state = 14},
  [2404] = {.lex_state = 14},
  [2405] = {.lex_state = 14},
  [2406] = {.lex_state = 3},
  [2407] = {.lex_state = 14},
  [2408] = {.lex_state = 14},
  [2409] = {.lex_state = 6},
  [2410] = {.lex_state = 27},
  [2411] = {.lex_state = 27},
  [2412] = {.lex_state = 27},
  [2413] = {.lex_state = 10},
  [2414] = {.lex_state = 14},
  [2415] = {.lex_state = 10},
  [2416] = {.lex_state = 10},
  [2417] = {.lex_state = 27},
  [2418] = {.lex_state = 10},
  [2419] = {.lex_state = 27},
  [2420] = {.lex_state = 27},
  [2421] = {.lex_state = 14},
  [2422] = {.lex_state = 27},
  [2423] = {.lex_state = 10},
  [2424] = {.lex_state = 10},
  [2425] = {.lex_state = 27},
  [2426] = {.lex_state = 27},
  [2427] = {.lex_state = 10},
  [2428] = {.lex_state = 27},
  [2429] = {.lex_state = 14},
  [2430] = {.lex_state = 27},
  [2431] = {.lex_state = 27},
  [2432] = {.lex_state = 27},
  [2433] = {.lex_state = 27},
  [2434] = {.lex_state = 10},
  [2435] = {.lex_state = 14},
  [2436] = {.lex_state = 27},
  [2437] = {.lex_state = 10},
  [2438] = {.lex_state = 10},
  [2439] = {.lex_state = 27},
  [2440] = {.lex_state = 14},
  [2441] = {.lex_state = 10},
  [2442] = {.lex_state = 10},
  [2443] = {.lex_state = 10},
  [2444] = {.lex_state = 27},
  [2445] = {.lex_state = 14},
  [2446] = {.lex_state = 27},
  [2447] = {.lex_state = 14},
  [2448] = {.lex_state = 10},
  [2449] = {.lex_state = 10},
  [2450] = {.lex_state = 10},
  [2451] = {.lex_state = 10},
  [2452] = {.lex_state = 27},
  [2453] = {.lex_state = 14},
  [2454] = {.lex_state = 10},
  [2455] = {.lex_state = 10},
  [2456] = {.lex_state = 10},
  [2457] = {.lex_state = 10},
  [2458] = {.lex_state = 10},
  [2459] = {.lex_state = 10},
  [2460] = {.lex_state = 27},
  [2461] = {.lex_state = 14},
  [2462] = {.lex_state = 27},
  [2463] = {.lex_state = 27},
  [2464] = {.lex_state = 14},
  [2465] = {.lex_state = 10},
  [2466] = {.lex_state = 14},
  [2467] = {.lex_state = 27},
  [2468] = {.lex_state = 14},
  [2469] = {.lex_state = 14},
  [2470] = {.lex_state = 6},
  [2471] = {.lex_state = 14},
  [2472] = {.lex_state = 27},
  [2473] = {.lex_state = 14},
  [2474] = {.lex_state = 14},
  [2475] = {.lex_state = 27},
  [2476] = {.lex_state = 27},
  [2477] = {.lex_state = 14},
  [2478] = {.lex_state = 10},
  [2479] = {.lex_state = 14},
  [2480] = {.lex_state = 14},
  [2481] = {.lex_state = 10, .external_lex_state = 3},
  [2482] = {.lex_state = 27},
  [2483] = {.lex_state = 14},
  [2484] = {.lex_state = 10},
  [2485] = {.lex_state = 14},
  [2486] = {.lex_state = 27},
  [2487] = {.lex_state = 10},
  [2488] = {.lex_state = 10},
  [2489] = {.lex_state = 70},
  [2490] = {.lex_state = 6},
  [2491] = {.lex_state = 27},
  [2492] = {.lex_state = 10},
  [2493] = {.lex_state = 27},
  [2494] = {.lex_state = 10},
  [2495] = {.lex_state = 27},
  [2496] = {.lex_state = 27},
  [2497] = {.lex_state = 27},
  [2498] = {.lex_state = 27},
  [2499] = {.lex_state = 27},
  [2500] = {.lex_state = 70},
  [2501] = {.lex_state = 27},
  [2502] = {.lex_state = 27},
  [2503] = {.lex_state = 27},
  [2504] = {.lex_state = 10},
  [2505] = {.lex_state = 14},
  [2506] = {.lex_state = 14},
  [2507] = {.lex_state = 10},
  [2508] = {.lex_state = 14},
  [2509] = {.lex_state = 3},
  [2510] = {.lex_state = 3},
  [2511] = {.lex_state = 10},
  [2512] = {.lex_state = 6},
  [2513] = {.lex_state = 27},
  [2514] = {.lex_state = 70},
  [2515] = {.lex_state = 6},
  [2516] = {.lex_state = 6},
  [2517] = {.lex_state = 14},
  [2518] = {.lex_state = 29},
  [2519] = {.lex_state = 10},
  [2520] = {.lex_state = 6},
  [2521] = {.lex_state = 14},
  [2522] = {.lex_state = 14},
  [2523] = {.lex_state = 14},
  [2524] = {.lex_state = 10, .external_lex_state = 4},
  [2525] = {.lex_state = 10},
  [2526] = {.lex_state = 10},
  [2527] = {.lex_state = 10},
  [2528] = {.lex_state = 6},
  [2529] = {.lex_state = 6},
  [2530] = {.lex_state = 27},
  [2531] = {.lex_state = 10},
  [2532] = {.lex_state = 70},
  [2533] = {.lex_state = 70},
  [2534] = {.lex_state = 10},
  [2535] = {.lex_state = 10},
  [2536] = {.lex_state = 70},
  [2537] = {.lex_state = 3},
  [2538] = {.lex_state = 6},
  [2539] = {.lex_state = 70},
  [2540] = {.lex_state = 70},
  [2541] = {.lex_state = 14},
  [2542] = {.lex_state = 10},
  [2543] = {.lex_state = 6},
  [2544] = {.lex_state = 10},
  [2545] = {.lex_state = 10},
  [2546] = {.lex_state = 10, .external_lex_state = 4},
  [2547] = {.lex_state = 14},
  [2548] = {.lex_state = 10},
  [2549] = {.lex_state = 10},
  [2550] = {.lex_state = 14},
  [2551] = {.lex_state = 14},
  [2552] = {.lex_state = 10},
  [2553] = {.lex_state = 16},
  [2554] = {.lex_state = 14},
  [2555] = {.lex_state = 10},
  [2556] = {.lex_state = 10},
  [2557] = {.lex_state = 10},
  [2558] = {.lex_state = 70},
  [2559] = {.lex_state = 10},
  [2560] = {.lex_state = 10},
  [2561] = {.lex_state = 6},
  [2562] = {.lex_state = 10},
  [2563] = {.lex_state = 14},
  [2564] = {.lex_state = 10, .external_lex_state = 4},
  [2565] = {.lex_state = 70},
  [2566] = {.lex_state = 6},
  [2567] = {.lex_state = 6},
  [2568] = {.lex_state = 6},
  [2569] = {.lex_state = 6},
  [2570] = {.lex_state = 6},
  [2571] = {.lex_state = 70},
  [2572] = {.lex_state = 27},
  [2573] = {.lex_state = 14},
  [2574] = {.lex_state = 6},
  [2575] = {.lex_state = 70},
  [2576] = {.lex_state = 14},
  [2577] = {.lex_state = 10},
  [2578] = {.lex_state = 27},
  [2579] = {.lex_state = 14},
  [2580] = {.lex_state = 10},
  [2581] = {.lex_state = 70},
  [2582] = {.lex_state = 10},
  [2583] = {.lex_state = 14},
  [2584] = {.lex_state = 70},
  [2585] = {.lex_state = 70},
  [2586] = {.lex_state = 14},
  [2587] = {.lex_state = 10},
  [2588] = {.lex_state = 10},
  [2589] = {.lex_state = 70},
  [2590] = {.lex_state = 10},
  [2591] = {.lex_state = 6},
  [2592] = {.lex_state = 70},
  [2593] = {.lex_state = 27},
  [2594] = {.lex_state = 17},
  [2595] = {.lex_state = 14},
  [2596] = {.lex_state = 10},
  [2597] = {.lex_state = 10},
  [2598] = {.lex_state = 10, .external_lex_state = 4},
  [2599] = {.lex_state = 10},
  [2600] = {.lex_state = 70},
  [2601] = {.lex_state = 10},
  [2602] = {.lex_state = 70},
  [2603] = {.lex_state = 10},
  [2604] = {.lex_state = 10},
  [2605] = {.lex_state = 10},
  [2606] = {.lex_state = 10, .external_lex_state = 4},
  [2607] = {.lex_state = 70},
  [2608] = {.lex_state = 6},
  [2609] = {.lex_state = 6},
  [2610] = {.lex_state = 70},
  [2611] = {.lex_state = 70},
  [2612] = {.lex_state = 27},
  [2613] = {.lex_state = 10},
  [2614] = {.lex_state = 17},
  [2615] = {.lex_state = 10},
  [2616] = {.lex_state = 6},
  [2617] = {.lex_state = 10},
  [2618] = {.lex_state = 70},
  [2619] = {.lex_state = 10, .external_lex_state = 4},
  [2620] = {.lex_state = 10},
  [2621] = {.lex_state = 14},
  [2622] = {.lex_state = 70},
  [2623] = {.lex_state = 10, .external_lex_state = 4},
  [2624] = {.lex_state = 70},
  [2625] = {.lex_state = 6},
  [2626] = {.lex_state = 6},
  [2627] = {.lex_state = 10},
  [2628] = {.lex_state = 17},
  [2629] = {.lex_state = 10},
  [2630] = {.lex_state = 10, .external_lex_state = 4},
  [2631] = {.lex_state = 10, .external_lex_state = 4},
  [2632] = {.lex_state = 17},
  [2633] = {.lex_state = 6},
  [2634] = {.lex_state = 6},
  [2635] = {.lex_state = 10, .external_lex_state = 4},
  [2636] = {.lex_state = 70},
  [2637] = {.lex_state = 10},
  [2638] = {.lex_state = 70},
  [2639] = {.lex_state = 6},
  [2640] = {.lex_state = 6},
  [2641] = {.lex_state = 70},
  [2642] = {.lex_state = 14},
  [2643] = {.lex_state = 6},
  [2644] = {.lex_state = 6},
  [2645] = {.lex_state = 6},
  [2646] = {.lex_state = 14},
  [2647] = {.lex_state = 6},
  [2648] = {.lex_state = 6},
  [2649] = {.lex_state = 6},
  [2650] = {.lex_state = 6},
  [2651] = {.lex_state = 6},
  [2652] = {.lex_state = 6},
  [2653] = {.lex_state = 27},
  [2654] = {.lex_state = 10},
  [2655] = {.lex_state = 6},
  [2656] = {.lex_state = 10},
  [2657] = {.lex_state = 3},
  [2658] = {.lex_state = 70},
  [2659] = {.lex_state = 16},
  [2660] = {.lex_state = 70},
  [2661] = {.lex_state = 70},
  [2662] = {.lex_state = 14},
  [2663] = {.lex_state = 14},
  [2664] = {.lex_state = 10},
  [2665] = {.lex_state = 3},
  [2666] = {.lex_state = 14},
  [2667] = {.lex_state = 27},
  [2668] = {.lex_state = 16},
  [2669] = {.lex_state = 70},
  [2670] = {.lex_state = 70},
  [2671] = {.lex_state = 70},
  [2672] = {.lex_state = 10},
  [2673] = {.lex_state = 29},
  [2674] = {.lex_state = 70},
  [2675] = {.lex_state = 70},
  [2676] = {.lex_state = 10},
  [2677] = {.lex_state = 70},
  [2678] = {.lex_state = 14},
  [2679] = {.lex_state = 10},
  [2680] = {.lex_state = 14},
  [2681] = {.lex_state = 10},
  [2682] = {.lex_state = 14},
  [2683] = {.lex_state = 17},
  [2684] = {.lex_state = 14},
  [2685] = {.lex_state = 29},
  [2686] = {.lex_state = 14},
  [2687] = {.lex_state = 6},
  [2688] = {.lex_state = 10},
  [2689] = {.lex_state = 27},
  [2690] = {.lex_state = 27},
  [2691] = {.lex_state = 29},
  [2692] = {.lex_state = 14},
  [2693] = {.lex_state = 70},
  [2694] = {.lex_state = 70},
  [2695] = {.lex_state = 10},
  [2696] = {.lex_state = 10},
  [2697] = {.lex_state = 10},
  [2698] = {.lex_state = 27},
  [2699] = {.lex_state = 17},
  [2700] = {.lex_state = 70},
  [2701] = {.lex_state = 14},
  [2702] = {.lex_state = 10, .external_lex_state = 4},
  [2703] = {.lex_state = 70},
  [2704] = {.lex_state = 10},
  [2705] = {.lex_state = 10},
  [2706] = {.lex_state = 27},
  [2707] = {.lex_state = 10},
  [2708] = {.lex_state = 6},
  [2709] = {.lex_state = 70},
  [2710] = {.lex_state = 10},
  [2711] = {.lex_state = 14},
  [2712] = {.lex_state = 17},
  [2713] = {.lex_state = 10, .external_lex_state = 4},
  [2714] = {.lex_state = 6},
  [2715] = {.lex_state = 10},
  [2716] = {.lex_state = 10},
  [2717] = {.lex_state = 3},
  [2718] = {.lex_state = 17},
  [2719] = {.lex_state = 14},
  [2720] = {.lex_state = 14},
  [2721] = {.lex_state = 27},
  [2722] = {.lex_state = 27},
  [2723] = {.lex_state = 70},
  [2724] = {.lex_state = 70},
  [2725] = {.lex_state = 10},
  [2726] = {.lex_state = 10},
  [2727] = {.lex_state = 14},
  [2728] = {.lex_state = 70},
  [2729] = {.lex_state = 3},
  [2730] = {.lex_state = 70},
  [2731] = {.lex_state = 14},
  [2732] = {.lex_state = 10},
  [2733] = {.lex_state = 14},
  [2734] = {.lex_state = 10, .external_lex_state = 4},
  [2735] = {.lex_state = 14},
  [2736] = {.lex_state = 14},
  [2737] = {.lex_state = 70},
  [2738] = {.lex_state = 70},
  [2739] = {.lex_state = 6},
  [2740] = {.lex_state = 6},
  [2741] = {.lex_state = 6},
  [2742] = {.lex_state = 6},
  [2743] = {.lex_state = 14},
  [2744] = {.lex_state = 6},
  [2745] = {.lex_state = 14},
  [2746] = {.lex_state = 70},
  [2747] = {.lex_state = 70},
  [2748] = {.lex_state = 6},
  [2749] = {.lex_state = 6},
  [2750] = {.lex_state = 14},
  [2751] = {.lex_state = 6},
  [2752] = {.lex_state = 6},
  [2753] = {.lex_state = 14},
  [2754] = {.lex_state = 70},
  [2755] = {.lex_state = 10},
  [2756] = {.lex_state = 6},
  [2757] = {.lex_state = 17},
  [2758] = {.lex_state = 70},
  [2759] = {.lex_state = 6},
  [2760] = {.lex_state = 70},
  [2761] = {.lex_state = 70},
  [2762] = {.lex_state = 70},
  [2763] = {.lex_state = 70},
  [2764] = {.lex_state = 6},
  [2765] = {.lex_state = 70},
  [2766] = {.lex_state = 70},
  [2767] = {.lex_state = 70},
  [2768] = {.lex_state = 70},
  [2769] = {.lex_state = 70},
  [2770] = {.lex_state = 17},
  [2771] = {.lex_state = 70},
  [2772] = {.lex_state = 17},
  [2773] = {.lex_state = 70},
  [2774] = {.lex_state = 70},
  [2775] = {.lex_state = 70},
  [2776] = {.lex_state = 70},
  [2777] = {.lex_state = 70},
  [2778] = {.lex_state = 70},
  [2779] = {.lex_state = 70},
  [2780] = {.lex_state = 70},
  [2781] = {.lex_state = 70},
  [2782] = {.lex_state = 70},
  [2783] = {.lex_state = 70},
  [2784] = {.lex_state = 70},
  [2785] = {.lex_state = 70},
  [2786] = {.lex_state = 17},
  [2787] = {.lex_state = 70},
  [2788] = {.lex_state = 70},
  [2789] = {.lex_state = 14},
  [2790] = {.lex_state = 6},
  [2791] = {.lex_state = 17},
  [2792] = {.lex_state = 17},
  [2793] = {.lex_state = 70},
  [2794] = {.lex_state = 17},
  [2795] = {.lex_state = 70},
  [2796] = {.lex_state = 70},
  [2797] = {.lex_state = 70},
  [2798] = {.lex_state = 70},
  [2799] = {.lex_state = 70},
  [2800] = {.lex_state = 14},
  [2801] = {.lex_state = 70},
  [2802] = {.lex_state = 70},
  [2803] = {.lex_state = 14},
  [2804] = {.lex_state = 70},
  [2805] = {.lex_state = 14},
  [2806] = {.lex_state = 70},
  [2807] = {.lex_state = 70},
  [2808] = {.lex_state = 70},
  [2809] = {.lex_state = 70},
  [2810] = {.lex_state = 14},
  [2811] = {.lex_state = 70},
  [2812] = {.lex_state = 70},
  [2813] = {.lex_state = 70},
  [2814] = {.lex_state = 27},
  [2815] = {.lex_state = 70},
  [2816] = {.lex_state = 70},
  [2817] = {.lex_state = 70},
  [2818] = {.lex_state = 70},
  [2819] = {.lex_state = 70},
  [2820] = {.lex_state = 70},
  [2821] = {.lex_state = 70},
  [2822] = {.lex_state = 70},
  [2823] = {.lex_state = 70},
  [2824] = {.lex_state = 70},
  [2825] = {.lex_state = 70},
  [2826] = {.lex_state = 70},
  [2827] = {.lex_state = 70},
  [2828] = {.lex_state = 70},
  [2829] = {.lex_state = 70},
  [2830] = {.lex_state = 70},
  [2831] = {.lex_state = 14},
  [2832] = {.lex_state = 70},
  [2833] = {.lex_state = 70},
  [2834] = {.lex_state = 70},
  [2835] = {.lex_state = 70},
  [2836] = {.lex_state = 70},
  [2837] = {.lex_state = 70},
  [2838] = {.lex_state = 70},
  [2839] = {.lex_state = 70},
  [2840] = {.lex_state = 70},
  [2841] = {.lex_state = 6},
  [2842] = {.lex_state = 70},
  [2843] = {.lex_state = 70},
  [2844] = {.lex_state = 70},
  [2845] = {.lex_state = 70},
  [2846] = {.lex_state = 70},
  [2847] = {.lex_state = 70},
  [2848] = {.lex_state = 70},
  [2849] = {.lex_state = 70},
  [2850] = {.lex_state = 70},
  [2851] = {.lex_state = 70},
  [2852] = {.lex_state = 17},
  [2853] = {.lex_state = 70},
  [2854] = {.lex_state = 14},
  [2855] = {.lex_state = 17},
  [2856] = {.lex_state = 19},
  [2857] = {.lex_state = 70},
  [2858] = {.lex_state = 70},
  [2859] = {.lex_state = 19},
  [2860] = {.lex_state = 70},
  [2861] = {.lex_state = 70},
  [2862] = {.lex_state = 70},
  [2863] = {.lex_state = 70},
  [2864] = {.lex_state = 17},
  [2865] = {.lex_state = 70},
  [2866] = {.lex_state = 70},
  [2867] = {.lex_state = 70},
  [2868] = {.lex_state = 70},
  [2869] = {.lex_state = 70},
  [2870] = {.lex_state = 70},
  [2871] = {.lex_state = 70},
  [2872] = {.lex_state = 17},
  [2873] = {.lex_state = 17},
  [2874] = {.lex_state = 70},
  [2875] = {.lex_state = 70},
  [2876] = {.lex_state = 17},
  [2877] = {.lex_state = 17},
  [2878] = {.lex_state = 70},
  [2879] = {.lex_state = 70},
  [2880] = {.lex_state = 70},
  [2881] = {.lex_state = 70},
  [2882] = {.lex_state = 17},
  [2883] = {.lex_state = 10},
  [2884] = {.lex_state = 17},
  [2885] = {.lex_state = 17},
  [2886] = {.lex_state = 17},
  [2887] = {.lex_state = 10},
  [2888] = {.lex_state = 14},
  [2889] = {.lex_state = 70},
  [2890] = {.lex_state = 70},
  [2891] = {.lex_state = 14},
  [2892] = {.lex_state = 70},
  [2893] = {.lex_state = 70},
  [2894] = {.lex_state = 70},
  [2895] = {.lex_state = 70},
  [2896] = {.lex_state = 70},
  [2897] = {.lex_state = 70},
  [2898] = {.lex_state = 70},
  [2899] = {.lex_state = 70},
  [2900] = {.lex_state = 70},
  [2901] = {.lex_state = 70},
  [2902] = {.lex_state = 19},
  [2903] = {.lex_state = 14},
  [2904] = {.lex_state = 70},
  [2905] = {.lex_state = 70},
  [2906] = {.lex_state = 70},
  [2907] = {.lex_state = 17},
  [2908] = {.lex_state = 70},
  [2909] = {.lex_state = 70},
  [2910] = {.lex_state = 70},
  [2911] = {.lex_state = 14},
  [2912] = {.lex_state = 70},
  [2913] = {.lex_state = 70},
  [2914] = {.lex_state = 6},
  [2915] = {.lex_state = 17},
  [2916] = {.lex_state = 70},
  [2917] = {.lex_state = 70},
  [2918] = {.lex_state = 17},
  [2919] = {.lex_state = 17},
  [2920] = {.lex_state = 70},
  [2921] = {.lex_state = 70},
  [2922] = {.lex_state = 70},
  [2923] = {.lex_state = 70},
  [2924] = {.lex_state = 14},
  [2925] = {.lex_state = 70},
  [2926] = {.lex_state = 70},
  [2927] = {.lex_state = 70},
  [2928] = {.lex_state = 70},
  [2929] = {.lex_state = 70},
  [2930] = {.lex_state = 70},
  [2931] = {.lex_state = 70},
  [2932] = {.lex_state = 70},
  [2933] = {.lex_state = 70},
  [2934] = {.lex_state = 70},
  [2935] = {.lex_state = 17},
  [2936] = {.lex_state = 17},
  [2937] = {.lex_state = 6},
  [2938] = {.lex_state = 14},
  [2939] = {.lex_state = 6},
  [2940] = {.lex_state = 70},
  [2941] = {.lex_state = 70},
  [2942] = {.lex_state = 17},
  [2943] = {.lex_state = 70},
  [2944] = {.lex_state = 14},
  [2945] = {.lex_state = 70},
  [2946] = {.lex_state = 6},
  [2947] = {.lex_state = 70},
  [2948] = {.lex_state = 70},
  [2949] = {.lex_state = 70},
  [2950] = {.lex_state = 14},
  [2951] = {.lex_state = 70},
  [2952] = {.lex_state = 14},
  [2953] = {.lex_state = 70},
  [2954] = {.lex_state = 70},
  [2955] = {.lex_state = 70},
  [2956] = {.lex_state = 70},
  [2957] = {.lex_state = 70},
  [2958] = {.lex_state = 70},
  [2959] = {.lex_state = 14},
  [2960] = {.lex_state = 14},
  [2961] = {.lex_state = 70},
  [2962] = {.lex_state = 70},
  [2963] = {.lex_state = 70},
  [2964] = {.lex_state = 70},
  [2965] = {.lex_state = 70},
  [2966] = {.lex_state = 14},
  [2967] = {.lex_state = 70},
  [2968] = {.lex_state = 10, .external_lex_state = 4},
  [2969] = {.lex_state = 70},
  [2970] = {.lex_state = 70},
  [2971] = {.lex_state = 14},
  [2972] = {.lex_state = 17},
  [2973] = {.lex_state = 70},
  [2974] = {.lex_state = 70},
  [2975] = {.lex_state = 17},
  [2976] = {.lex_state = 70},
  [2977] = {.lex_state = 70},
  [2978] = {.lex_state = 17},
  [2979] = {.lex_state = 70},
  [2980] = {.lex_state = 27},
  [2981] = {.lex_state = 70},
  [2982] = {.lex_state = 6},
  [2983] = {.lex_state = 70},
  [2984] = {.lex_state = 17},
  [2985] = {.lex_state = 17},
  [2986] = {.lex_state = 17},
  [2987] = {.lex_state = 70},
  [2988] = {.lex_state = 17},
  [2989] = {.lex_state = 70},
  [2990] = {.lex_state = 70},
  [2991] = {.lex_state = 70},
  [2992] = {.lex_state = 19},
  [2993] = {.lex_state = 70},
  [2994] = {.lex_state = 14},
  [2995] = {.lex_state = 70},
  [2996] = {.lex_state = 70},
  [2997] = {.lex_state = 70},
  [2998] = {.lex_state = 70},
  [2999] = {.lex_state = 70},
  [3000] = {.lex_state = 70},
  [3001] = {.lex_state = 70},
  [3002] = {.lex_state = 70},
  [3003] = {.lex_state = 70},
  [3004] = {.lex_state = 70},
  [3005] = {.lex_state = 17},
  [3006] = {.lex_state = 17},
  [3007] = {.lex_state = 70},
  [3008] = {.lex_state = 17},
  [3009] = {.lex_state = 70},
  [3010] = {.lex_state = 14},
  [3011] = {.lex_state = 17},
  [3012] = {.lex_state = 70},
  [3013] = {.lex_state = 70},
  [3014] = {.lex_state = 70},
  [3015] = {.lex_state = 70},
  [3016] = {.lex_state = 70},
  [3017] = {.lex_state = 10},
  [3018] = {.lex_state = 70},
  [3019] = {.lex_state = 70},
  [3020] = {.lex_state = 70},
  [3021] = {.lex_state = 17},
  [3022] = {.lex_state = 70},
  [3023] = {.lex_state = 70},
  [3024] = {.lex_state = 70},
  [3025] = {.lex_state = 70},
  [3026] = {.lex_state = 70},
  [3027] = {.lex_state = 70},
  [3028] = {.lex_state = 70},
  [3029] = {.lex_state = 70},
  [3030] = {.lex_state = 70},
  [3031] = {.lex_state = 70},
  [3032] = {.lex_state = 6},
  [3033] = {.lex_state = 70},
  [3034] = {.lex_state = 70},
  [3035] = {.lex_state = 70},
  [3036] = {.lex_state = 14},
  [3037] = {.lex_state = 70},
  [3038] = {.lex_state = 70},
  [3039] = {.lex_state = 70},
  [3040] = {.lex_state = 70},
  [3041] = {.lex_state = 70},
  [3042] = {.lex_state = 6},
  [3043] = {.lex_state = 19},
  [3044] = {.lex_state = 70},
  [3045] = {.lex_state = 70},
  [3046] = {.lex_state = 70},
  [3047] = {.lex_state = 70},
  [3048] = {.lex_state = 6},
  [3049] = {.lex_state = 14},
  [3050] = {.lex_state = 6},
  [3051] = {.lex_state = 70},
  [3052] = {.lex_state = 6},
  [3053] = {.lex_state = 14},
  [3054] = {.lex_state = 70},
  [3055] = {.lex_state = 70},
  [3056] = {.lex_state = 10},
  [3057] = {.lex_state = 14},
  [3058] = {.lex_state = 70},
  [3059] = {.lex_state = 70},
  [3060] = {.lex_state = 70},
  [3061] = {.lex_state = 70},
  [3062] = {.lex_state = 70},
  [3063] = {.lex_state = 70},
  [3064] = {.lex_state = 70},
  [3065] = {.lex_state = 70},
  [3066] = {.lex_state = 14},
  [3067] = {.lex_state = 14},
  [3068] = {.lex_state = 14},
  [3069] = {.lex_state = 70},
  [3070] = {.lex_state = 70},
  [3071] = {.lex_state = 70},
  [3072] = {.lex_state = 70},
  [3073] = {.lex_state = 14},
  [3074] = {.lex_state = 14},
  [3075] = {.lex_state = 14},
  [3076] = {.lex_state = 14},
  [3077] = {.lex_state = 70},
  [3078] = {.lex_state = 70},
  [3079] = {.lex_state = 14},
  [3080] = {.lex_state = 70},
  [3081] = {.lex_state = 14},
  [3082] = {.lex_state = 70},
  [3083] = {.lex_state = 70},
  [3084] = {.lex_state = 14},
  [3085] = {.lex_state = 14},
  [3086] = {.lex_state = 14},
  [3087] = {.lex_state = 14},
  [3088] = {.lex_state = 14},
  [3089] = {.lex_state = 70},
  [3090] = {.lex_state = 14},
  [3091] = {.lex_state = 70},
  [3092] = {.lex_state = 14},
  [3093] = {.lex_state = 14},
  [3094] = {.lex_state = 70},
  [3095] = {.lex_state = 70},
  [3096] = {.lex_state = 70},
  [3097] = {.lex_state = 70},
  [3098] = {.lex_state = 14},
  [3099] = {.lex_state = 70},
  [3100] = {.lex_state = 70},
  [3101] = {.lex_state = 70},
  [3102] = {.lex_state = 70},
  [3103] = {.lex_state = 10},
  [3104] = {.lex_state = 14},
  [3105] = {.lex_state = 70},
  [3106] = {.lex_state = 70},
  [3107] = {.lex_state = 14},
  [3108] = {.lex_state = 70},
  [3109] = {.lex_state = 14},
  [3110] = {.lex_state = 14},
  [3111] = {.lex_state = 14},
  [3112] = {.lex_state = 70},
  [3113] = {.lex_state = 14},
  [3114] = {.lex_state = 14},
  [3115] = {.lex_state = 70},
  [3116] = {.lex_state = 70},
  [3117] = {.lex_state = 27},
  [3118] = {.lex_state = 70},
  [3119] = {.lex_state = 70},
  [3120] = {.lex_state = 14},
  [3121] = {.lex_state = 70},
  [3122] = {.lex_state = 14},
  [3123] = {.lex_state = 14},
  [3124] = {.lex_state = 70},
  [3125] = {.lex_state = 14},
  [3126] = {.lex_state = 14},
  [3127] = {.lex_state = 70},
  [3128] = {.lex_state = 14},
  [3129] = {.lex_state = 27},
  [3130] = {.lex_state = 70},
  [3131] = {.lex_state = 70},
  [3132] = {.lex_state = 14},
  [3133] = {.lex_state = 14},
  [3134] = {.lex_state = 70},
  [3135] = {.lex_state = 70},
  [3136] = {.lex_state = 70},
  [3137] = {.lex_state = 70},
  [3138] = {.lex_state = 14},
  [3139] = {.lex_state = 14},
  [3140] = {.lex_state = 70},
  [3141] = {.lex_state = 14},
  [3142] = {.lex_state = 14},
  [3143] = {.lex_state = 70},
  [3144] = {.lex_state = 70},
  [3145] = {.lex_state = 70},
  [3146] = {.lex_state = 70},
  [3147] = {.lex_state = 70},
  [3148] = {.lex_state = 14},
  [3149] = {.lex_state = 14},
  [3150] = {.lex_state = 70},
  [3151] = {.lex_state = 70},
  [3152] = {.lex_state = 70},
  [3153] = {.lex_state = 70},
  [3154] = {.lex_state = 70},
  [3155] = {.lex_state = 14},
  [3156] = {.lex_state = 70},
  [3157] = {.lex_state = 70},
  [3158] = {.lex_state = 70},
  [3159] = {.lex_state = 14},
  [3160] = {.lex_state = 14},
  [3161] = {.lex_state = 14},
  [3162] = {.lex_state = 14},
  [3163] = {.lex_state = 70},
  [3164] = {.lex_state = 70},
  [3165] = {.lex_state = 70},
  [3166] = {.lex_state = 70},
  [3167] = {.lex_state = 70},
  [3168] = {.lex_state = 70},
  [3169] = {.lex_state = 70},
  [3170] = {.lex_state = 14},
  [3171] = {.lex_state = 70},
  [3172] = {.lex_state = 14},
  [3173] = {.lex_state = 14},
  [3174] = {.lex_state = 14},
  [3175] = {.lex_state = 70},
  [3176] = {.lex_state = 70},
  [3177] = {.lex_state = 70},
  [3178] = {.lex_state = 14},
  [3179] = {.lex_state = 70},
  [3180] = {.lex_state = 70},
  [3181] = {.lex_state = 14},
  [3182] = {.lex_state = 14},
  [3183] = {.lex_state = 14},
  [3184] = {.lex_state = 14},
  [3185] = {.lex_state = 14},
  [3186] = {.lex_state = 14},
  [3187] = {.lex_state = 10},
  [3188] = {.lex_state = 70},
  [3189] = {.lex_state = 70, .external_lex_state = 5},
  [3190] = {.lex_state = 14},
  [3191] = {.lex_state = 14},
  [3192] = {.lex_state = 14},
  [3193] = {.lex_state = 14},
  [3194] = {.lex_state = 14},
  [3195] = {.lex_state = 70},
  [3196] = {.lex_state = 14},
  [3197] = {.lex_state = 70},
  [3198] = {.lex_state = 14},
  [3199] = {.lex_state = 14},
  [3200] = {.lex_state = 14},
  [3201] = {.lex_state = 14},
  [3202] = {.lex_state = 70},
  [3203] = {.lex_state = 14},
  [3204] = {.lex_state = 14},
  [3205] = {.lex_state = 70},
  [3206] = {.lex_state = 14},
  [3207] = {.lex_state = 14},
  [3208] = {.lex_state = 14},
  [3209] = {.lex_state = 14},
  [3210] = {.lex_state = 70},
  [3211] = {.lex_state = 14},
  [3212] = {.lex_state = 14},
  [3213] = {.lex_state = 70},
  [3214] = {.lex_state = 70},
  [3215] = {.lex_state = 14},
  [3216] = {.lex_state = 70},
  [3217] = {.lex_state = 70},
  [3218] = {.lex_state = 14},
  [3219] = {.lex_state = 70},
  [3220] = {.lex_state = 70},
  [3221] = {.lex_state = 70},
  [3222] = {.lex_state = 14},
  [3223] = {.lex_state = 70},
  [3224] = {.lex_state = 70},
  [3225] = {.lex_state = 70},
  [3226] = {.lex_state = 5},
  [3227] = {.lex_state = 70},
  [3228] = {.lex_state = 5},
  [3229] = {.lex_state = 70},
  [3230] = {.lex_state = 70},
  [3231] = {.lex_state = 70},
  [3232] = {.lex_state = 14},
  [3233] = {.lex_state = 70},
  [3234] = {.lex_state = 70},
  [3235] = {.lex_state = 70},
  [3236] = {.lex_state = 70},
  [3237] = {.lex_state = 14},
  [3238] = {.lex_state = 70},
  [3239] = {.lex_state = 70},
  [3240] = {.lex_state = 70},
  [3241] = {.lex_state = 14},
  [3242] = {.lex_state = 70},
  [3243] = {.lex_state = 70},
  [3244] = {.lex_state = 70},
  [3245] = {.lex_state = 70},
  [3246] = {.lex_state = 70},
  [3247] = {.lex_state = 70},
  [3248] = {.lex_state = 70},
  [3249] = {.lex_state = 14},
  [3250] = {.lex_state = 14},
  [3251] = {.lex_state = 14},
  [3252] = {.lex_state = 6},
  [3253] = {.lex_state = 14},
  [3254] = {.lex_state = 10, .external_lex_state = 6},
  [3255] = {.lex_state = 70},
  [3256] = {.lex_state = 27},
  [3257] = {.lex_state = 70},
  [3258] = {.lex_state = 70},
  [3259] = {.lex_state = 6},
  [3260] = {.lex_state = 27},
  [3261] = {.lex_state = 70},
  [3262] = {.lex_state = 27},
  [3263] = {.lex_state = 14},
  [3264] = {.lex_state = 70},
  [3265] = {.lex_state = 70},
  [3266] = {.lex_state = 70},
  [3267] = {.lex_state = 70},
  [3268] = {.lex_state = 70},
  [3269] = {.lex_state = 70},
  [3270] = {.lex_state = 70},
  [3271] = {.lex_state = 70},
  [3272] = {.lex_state = 14},
  [3273] = {.lex_state = 70},
  [3274] = {.lex_state = 70},
  [3275] = {.lex_state = 70},
  [3276] = {.lex_state = 70},
  [3277] = {.lex_state = 70},
  [3278] = {.lex_state = 70},
  [3279] = {.lex_state = 14},
  [3280] = {.lex_state = 14},
  [3281] = {.lex_state = 70},
  [3282] = {.lex_state = 70},
  [3283] = {.lex_state = 14},
  [3284] = {.lex_state = 70},
  [3285] = {.lex_state = 14},
  [3286] = {.lex_state = 70},
  [3287] = {.lex_state = 70},
  [3288] = {.lex_state = 70},
  [3289] = {.lex_state = 70},
  [3290] = {.lex_state = 70},
  [3291] = {.lex_state = 70},
  [3292] = {.lex_state = 70},
  [3293] = {.lex_state = 19},
  [3294] = {.lex_state = 70},
  [3295] = {.lex_state = 70},
  [3296] = {.lex_state = 10, .external_lex_state = 6},
  [3297] = {.lex_state = 70},
  [3298] = {.lex_state = 14},
  [3299] = {.lex_state = 14},
  [3300] = {.lex_state = 70},
  [3301] = {.lex_state = 70},
  [3302] = {.lex_state = 14},
  [3303] = {.lex_state = 14},
  [3304] = {.lex_state = 70},
  [3305] = {.lex_state = 14},
  [3306] = {.lex_state = 70},
  [3307] = {.lex_state = 14},
  [3308] = {.lex_state = 14},
  [3309] = {.lex_state = 70},
  [3310] = {.lex_state = 70},
  [3311] = {.lex_state = 70},
  [3312] = {.lex_state = 14},
  [3313] = {.lex_state = 70},
  [3314] = {.lex_state = 70},
  [3315] = {.lex_state = 14},
  [3316] = {.lex_state = 14},
  [3317] = {.lex_state = 70},
  [3318] = {.lex_state = 14},
  [3319] = {.lex_state = 14},
  [3320] = {.lex_state = 70},
  [3321] = {.lex_state = 14},
  [3322] = {.lex_state = 70},
  [3323] = {.lex_state = 70},
  [3324] = {.lex_state = 70},
  [3325] = {.lex_state = 27},
  [3326] = {.lex_state = 14},
  [3327] = {.lex_state = 70},
  [3328] = {.lex_state = 14},
  [3329] = {.lex_state = 70},
  [3330] = {.lex_state = 14},
  [3331] = {.lex_state = 14},
  [3332] = {.lex_state = 14},
  [3333] = {.lex_state = 14},
  [3334] = {.lex_state = 10, .external_lex_state = 6},
  [3335] = {.lex_state = 14},
  [3336] = {.lex_state = 70, .external_lex_state = 5},
  [3337] = {.lex_state = 70},
  [3338] = {.lex_state = 14},
  [3339] = {.lex_state = 70},
  [3340] = {.lex_state = 14},
  [3341] = {.lex_state = 14},
  [3342] = {.lex_state = 70},
  [3343] = {.lex_state = 14},
  [3344] = {.lex_state = 14},
  [3345] = {.lex_state = 14},
  [3346] = {.lex_state = 14},
  [3347] = {.lex_state = 70},
  [3348] = {.lex_state = 70},
  [3349] = {.lex_state = 14},
  [3350] = {.lex_state = 14},
  [3351] = {.lex_state = 70},
  [3352] = {.lex_state = 71},
  [3353] = {.lex_state = 70},
  [3354] = {.lex_state = 70},
  [3355] = {.lex_state = 19},
  [3356] = {.lex_state = 70},
  [3357] = {.lex_state = 19},
  [3358] = {.lex_state = 70},
  [3359] = {.lex_state = 70},
  [3360] = {.lex_state = 70},
  [3361] = {.lex_state = 70},
  [3362] = {.lex_state = 14},
  [3363] = {.lex_state = 14},
  [3364] = {.lex_state = 14},
  [3365] = {.lex_state = 70},
  [3366] = {.lex_state = 14},
  [3367] = {.lex_state = 14},
  [3368] = {.lex_state = 70},
  [3369] = {.lex_state = 70},
  [3370] = {.lex_state = 70},
  [3371] = {.lex_state = 70},
  [3372] = {.lex_state = 70},
  [3373] = {.lex_state = 19},
  [3374] = {.lex_state = 70},
  [3375] = {.lex_state = 14},
  [3376] = {.lex_state = 70},
  [3377] = {.lex_state = 14},
  [3378] = {.lex_state = 14},
  [3379] = {.lex_state = 70},
  [3380] = {.lex_state = 14},
  [3381] = {.lex_state = 14},
  [3382] = {.lex_state = 14},
  [3383] = {.lex_state = 14},
  [3384] = {.lex_state = 14},
  [3385] = {.lex_state = 70},
  [3386] = {.lex_state = 14},
  [3387] = {.lex_state = 14},
  [3388] = {.lex_state = 71},
  [3389] = {.lex_state = 70},
  [3390] = {.lex_state = 19},
  [3391] = {.lex_state = 70},
  [3392] = {.lex_state = 70},
  [3393] = {.lex_state = 70},
  [3394] = {.lex_state = 70},
  [3395] = {.lex_state = 70, .external_lex_state = 7},
  [3396] = {.lex_state = 19},
  [3397] = {.lex_state = 70},
  [3398] = {.lex_state = 19},
  [3399] = {.lex_state = 70},
  [3400] = {.lex_state = 14},
  [3401] = {.lex_state = 70},
  [3402] = {.lex_state = 70},
  [3403] = {.lex_state = 70},
  [3404] = {.lex_state = 70},
  [3405] = {.lex_state = 70},
  [3406] = {.lex_state = 70},
  [3407] = {.lex_state = 14},
  [3408] = {.lex_state = 70},
  [3409] = {.lex_state = 70},
  [3410] = {.lex_state = 70},
  [3411] = {.lex_state = 14},
  [3412] = {.lex_state = 70},
  [3413] = {.lex_state = 70},
  [3414] = {.lex_state = 70},
  [3415] = {.lex_state = 14},
  [3416] = {.lex_state = 70},
  [3417] = {.lex_state = 70},
  [3418] = {.lex_state = 14},
  [3419] = {.lex_state = 14},
  [3420] = {.lex_state = 14},
  [3421] = {.lex_state = 14},
  [3422] = {.lex_state = 70},
  [3423] = {.lex_state = 70},
  [3424] = {.lex_state = 19},
  [3425] = {.lex_state = 70},
  [3426] = {.lex_state = 70},
  [3427] = {.lex_state = 70},
  [3428] = {.lex_state = 70, .external_lex_state = 8},
  [3429] = {.lex_state = 14},
  [3430] = {.lex_state = 14},
  [3431] = {.lex_state = 70},
  [3432] = {.lex_state = 70},
  [3433] = {.lex_state = 14},
  [3434] = {.lex_state = 14},
  [3435] = {.lex_state = 14},
  [3436] = {.lex_state = 70},
  [3437] = {.lex_state = 70},
  [3438] = {.lex_state = 70},
  [3439] = {.lex_state = 70},
  [3440] = {.lex_state = 70},
  [3441] = {.lex_state = 70},
  [3442] = {.lex_state = 70},
  [3443] = {.lex_state = 70},
  [3444] = {.lex_state = 71},
  [3445] = {.lex_state = 70},
  [3446] = {.lex_state = 70},
  [3447] = {.lex_state = 70},
  [3448] = {.lex_state = 70},
  [3449] = {.lex_state = 70},
  [3450] = {.lex_state = 14},
  [3451] = {.lex_state = 70},
  [3452] = {.lex_state = 70},
  [3453] = {.lex_state = 70},
  [3454] = {.lex_state = 14},
  [3455] = {.lex_state = 70},
  [3456] = {.lex_state = 70, .external_lex_state = 8},
  [3457] = {.lex_state = 70},
  [3458] = {.lex_state = 165},
  [3459] = {.lex_state = 14},
  [3460] = {.lex_state = 70},
  [3461] = {.lex_state = 70},
  [3462] = {.lex_state = 70},
  [3463] = {.lex_state = 70},
  [3464] = {.lex_state = 70},
  [3465] = {.lex_state = 70},
  [3466] = {.lex_state = 70},
  [3467] = {.lex_state = 70},
  [3468] = {.lex_state = 70},
  [3469] = {.lex_state = 70, .external_lex_state = 8},
  [3470] = {.lex_state = 70},
  [3471] = {.lex_state = 70},
  [3472] = {.lex_state = 14},
  [3473] = {.lex_state = 14},
  [3474] = {.lex_state = 70, .external_lex_state = 8},
  [3475] = {.lex_state = 70},
  [3476] = {.lex_state = 19},
  [3477] = {.lex_state = 70},
  [3478] = {.lex_state = 70, .external_lex_state = 7},
  [3479] = {.lex_state = 10},
  [3480] = {.lex_state = 19},
  [3481] = {.lex_state = 14},
  [3482] = {.lex_state = 19},
  [3483] = {.lex_state = 70},
  [3484] = {.lex_state = 71},
  [3485] = {.lex_state = 70},
  [3486] = {.lex_state = 70},
  [3487] = {.lex_state = 71},
  [3488] = {.lex_state = 70},
  [3489] = {.lex_state = 14},
  [3490] = {.lex_state = 70, .external_lex_state = 9},
  [3491] = {.lex_state = 70},
  [3492] = {.lex_state = 70},
  [3493] = {.lex_state = 70},
  [3494] = {.lex_state = 70},
  [3495] = {.lex_state = 70},
  [3496] = {.lex_state = 14},
  [3497] = {.lex_state = 70},
  [3498] = {.lex_state = 70},
  [3499] = {.lex_state = 70},
  [3500] = {.lex_state = 70},
  [3501] = {.lex_state = 70},
  [3502] = {.lex_state = 70},
  [3503] = {.lex_state = 70},
  [3504] = {.lex_state = 14},
  [3505] = {.lex_state = 70},
  [3506] = {.lex_state = 70},
  [3507] = {.lex_state = 70},
  [3508] = {.lex_state = 70},
  [3509] = {.lex_state = 70},
  [3510] = {.lex_state = 70},
  [3511] = {.lex_state = 70},
  [3512] = {.lex_state = 70},
  [3513] = {.lex_state = 70},
  [3514] = {.lex_state = 14},
  [3515] = {.lex_state = 70},
  [3516] = {.lex_state = 70},
  [3517] = {.lex_state = 70},
  [3518] = {.lex_state = 70, .external_lex_state = 7},
  [3519] = {.lex_state = 70},
  [3520] = {.lex_state = 14},
  [3521] = {.lex_state = 70},
  [3522] = {.lex_state = 70},
  [3523] = {.lex_state = 14},
  [3524] = {.lex_state = 70},
  [3525] = {.lex_state = 70, .external_lex_state = 9},
  [3526] = {.lex_state = 70, .external_lex_state = 8},
  [3527] = {.lex_state = 70},
  [3528] = {.lex_state = 70},
  [3529] = {.lex_state = 70},
  [3530] = {.lex_state = 70},
  [3531] = {.lex_state = 70},
  [3532] = {.lex_state = 70},
  [3533] = {.lex_state = 70},
  [3534] = {.lex_state = 70},
  [3535] = {.lex_state = 70},
  [3536] = {.lex_state = 71},
  [3537] = {.lex_state = 70},
  [3538] = {.lex_state = 70},
  [3539] = {.lex_state = 70},
  [3540] = {.lex_state = 10},
  [3541] = {.lex_state = 70},
  [3542] = {.lex_state = 70},
  [3543] = {.lex_state = 70, .external_lex_state = 9},
  [3544] = {.lex_state = 70},
  [3545] = {.lex_state = 70},
  [3546] = {.lex_state = 70},
  [3547] = {.lex_state = 14},
  [3548] = {.lex_state = 70},
  [3549] = {.lex_state = 70, .external_lex_state = 9},
  [3550] = {.lex_state = 70},
  [3551] = {.lex_state = 70},
  [3552] = {.lex_state = 70},
  [3553] = {.lex_state = 70},
  [3554] = {.lex_state = 70},
  [3555] = {.lex_state = 70},
  [3556] = {.lex_state = 70},
  [3557] = {.lex_state = 70},
  [3558] = {.lex_state = 70},
  [3559] = {.lex_state = 70},
  [3560] = {.lex_state = 70},
  [3561] = {.lex_state = 70},
  [3562] = {.lex_state = 70, .external_lex_state = 7},
  [3563] = {.lex_state = 70, .external_lex_state = 9},
  [3564] = {.lex_state = 14},
  [3565] = {.lex_state = 19},
  [3566] = {.lex_state = 70},
  [3567] = {.lex_state = 70},
  [3568] = {.lex_state = 70},
  [3569] = {.lex_state = 19},
  [3570] = {.lex_state = 19},
  [3571] = {.lex_state = 70},
  [3572] = {.lex_state = 70},
  [3573] = {.lex_state = 14},
  [3574] = {.lex_state = 70},
  [3575] = {.lex_state = 19},
  [3576] = {.lex_state = 70},
  [3577] = {.lex_state = 19},
  [3578] = {.lex_state = 14},
  [3579] = {.lex_state = 70},
  [3580] = {.lex_state = 70},
  [3581] = {.lex_state = 70},
  [3582] = {.lex_state = 14},
  [3583] = {.lex_state = 70},
  [3584] = {.lex_state = 70},
  [3585] = {.lex_state = 19},
  [3586] = {.lex_state = 14},
  [3587] = {.lex_state = 70},
  [3588] = {.lex_state = 70},
  [3589] = {.lex_state = 19},
  [3590] = {.lex_state = 70},
  [3591] = {.lex_state = 70},
  [3592] = {.lex_state = 70},
  [3593] = {.lex_state = 70, .external_lex_state = 8},
  [3594] = {.lex_state = 70},
  [3595] = {.lex_state = 19},
  [3596] = {.lex_state = 71},
  [3597] = {.lex_state = 14},
  [3598] = {.lex_state = 19},
  [3599] = {.lex_state = 70},
  [3600] = {.lex_state = 19},
  [3601] = {.lex_state = 70},
  [3602] = {.lex_state = 70},
  [3603] = {.lex_state = 14},
  [3604] = {.lex_state = 14},
  [3605] = {.lex_state = 70},
  [3606] = {.lex_state = 19},
  [3607] = {.lex_state = 14},
  [3608] = {.lex_state = 19},
  [3609] = {.lex_state = 70},
  [3610] = {.lex_state = 70},
  [3611] = {.lex_state = 19},
  [3612] = {.lex_state = 14},
  [3613] = {.lex_state = 70},
  [3614] = {.lex_state = 19},
  [3615] = {.lex_state = 19},
  [3616] = {.lex_state = 71},
  [3617] = {.lex_state = 14},
  [3618] = {.lex_state = 19},
  [3619] = {.lex_state = 19},
  [3620] = {.lex_state = 19},
  [3621] = {.lex_state = 70},
  [3622] = {.lex_state = 14},
  [3623] = {.lex_state = 70},
  [3624] = {.lex_state = 70},
  [3625] = {.lex_state = 14},
  [3626] = {.lex_state = 14},
  [3627] = {.lex_state = 70},
  [3628] = {.lex_state = 14},
  [3629] = {.lex_state = 14},
  [3630] = {.lex_state = 70, .external_lex_state = 9},
  [3631] = {.lex_state = 14},
  [3632] = {.lex_state = 14},
  [3633] = {.lex_state = 14},
  [3634] = {.lex_state = 14},
  [3635] = {.lex_state = 70},
  [3636] = {.lex_state = 14},
  [3637] = {.lex_state = 70},
  [3638] = {.lex_state = 70, .external_lex_state = 7},
  [3639] = {.lex_state = 70},
  [3640] = {.lex_state = 70},
  [3641] = {.lex_state = 70},
  [3642] = {(TSStateId)(-1)},
  [3643] = {(TSStateId)(-1)},
  [3644] = {(TSStateId)(-1)},
  [3645] = {(TSStateId)(-1)},
  [3646] = {(TSStateId)(-1)},
  [3647] = {(TSStateId)(-1)},
  [3648] = {(TSStateId)(-1)},
};

static const uint16_t ts_parse_table[LARGE_STATE_COUNT][SYMBOL_COUNT] = {
  [0] = {
    [sym_line_comment] = STATE(0),
    [sym_block_comment] = STATE(0),
    [ts_builtin_sym_end] = ACTIONS(1),
    [sym_identifier] = ACTIONS(1),
    [anon_sym_SEMI] = ACTIONS(1),
    [anon_sym_macro_rules_BANG] = ACTIONS(1),
    [anon_sym_LPAREN] = ACTIONS(1),
    [anon_sym_RPAREN] = ACTIONS(1),
    [anon_sym_LBRACK] = ACTIONS(1),
    [anon_sym_RBRACK] = ACTIONS(1),
    [anon_sym_LBRACE] = ACTIONS(1),
    [anon_sym_RBRACE] = ACTIONS(1),
    [anon_sym_EQ_GT] = ACTIONS(1),
    [anon_sym_COLON] = ACTIONS(1),
    [anon_sym_DOLLAR] = ACTIONS(1),
    [anon_sym_PLUS] = ACTIONS(1),
    [anon_sym_STAR] = ACTIONS(1),
    [anon_sym_QMARK] = ACTIONS(1),
    [anon_sym_block] = ACTIONS(1),
    [anon_sym_expr] = ACTIONS(1),
    [anon_sym_ident] = ACTIONS(1),
    [anon_sym_item] = ACTIONS(1),
    [anon_sym_lifetime] = ACTIONS(1),
    [anon_sym_literal] = ACTIONS(1),
    [anon_sym_meta] = ACTIONS(1),
    [anon_sym_pat] = ACTIONS(1),
    [anon_sym_path] = ACTIONS(1),
    [anon_sym_stmt] = ACTIONS(1),
    [anon_sym_tt] = ACTIONS(1),
    [anon_sym_ty] = ACTIONS(1),
    [anon_sym_vis] = ACTIONS(1),
    [anon_sym_u8] = ACTIONS(1),
    [anon_sym_i8] = ACTIONS(1),
    [anon_sym_u16] = ACTIONS(1),
    [anon_sym_i16] = ACTIONS(1),
    [anon_sym_u32] = ACTIONS(1),
    [anon_sym_i32] = ACTIONS(1),
    [anon_sym_u64] = ACTIONS(1),
    [anon_sym_i64] = ACTIONS(1),
    [anon_sym_u128] = ACTIONS(1),
    [anon_sym_i128] = ACTIONS(1),
    [anon_sym_isize] = ACTIONS(1),
    [anon_sym_usize] = ACTIONS(1),
    [anon_sym_f32] = ACTIONS(1),
    [anon_sym_f64] = ACTIONS(1),
    [anon_sym_bool] = ACTIONS(1),
    [anon_sym_str] = ACTIONS(1),
    [anon_sym_char] = ACTIONS(1),
    [anon_sym_DASH] = ACTIONS(1),
    [anon_sym_SLASH] = ACTIONS(1),
    [anon_sym_PERCENT] = ACTIONS(1),
    [anon_sym_CARET] = ACTIONS(1),
    [anon_sym_BANG] = ACTIONS(1),
    [anon_sym_AMP] = ACTIONS(1),
    [anon_sym_PIPE] = ACTIONS(1),
    [anon_sym_AMP_AMP] = ACTIONS(1),
    [anon_sym_PIPE_PIPE] = ACTIONS(1),
    [anon_sym_LT_LT] = ACTIONS(1),
    [anon_sym_GT_GT] = ACTIONS(1),
    [anon_sym_PLUS_EQ] = ACTIONS(1),
    [anon_sym_DASH_EQ] = ACTIONS(1),
    [anon_sym_STAR_EQ] = ACTIONS(1),
    [anon_sym_SLASH_EQ] = ACTIONS(1),
    [anon_sym_PERCENT_EQ] = ACTIONS(1),
    [anon_sym_CARET_EQ] = ACTIONS(1),
    [anon_sym_AMP_EQ] = ACTIONS(1),
    [anon_sym_PIPE_EQ] = ACTIONS(1),
    [anon_sym_LT_LT_EQ] = ACTIONS(1),
    [anon_sym_GT_GT_EQ] = ACTIONS(1),
    [anon_sym_EQ] = ACTIONS(1),
    [anon_sym_EQ_EQ] = ACTIONS(1),
    [anon_sym_BANG_EQ] = ACTIONS(1),
    [anon_sym_GT] = ACTIONS(1),
    [anon_sym_LT] = ACTIONS(1),
    [anon_sym_GT_EQ] = ACTIONS(1),
    [anon_sym_LT_EQ] = ACTIONS(1),
    [anon_sym_AT] = ACTIONS(1),
    [anon_sym__] = ACTIONS(1),
    [anon_sym_DOT] = ACTIONS(1),
    [anon_sym_DOT_DOT] = ACTIONS(1),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1),
    [anon_sym_COMMA] = ACTIONS(1),
    [anon_sym_COLON_COLON] = ACTIONS(1),
    [anon_sym_DASH_GT] = ACTIONS(1),
    [anon_sym_POUND] = ACTIONS(1),
    [anon_sym_SQUOTE] = ACTIONS(1),
    [anon_sym_as] = ACTIONS(1),
    [anon_sym_async] = ACTIONS(1),
    [anon_sym_await] = ACTIONS(1),
    [anon_sym_break] = ACTIONS(1),
    [anon_sym_const] = ACTIONS(1),
    [anon_sym_continue] = ACTIONS(1),
    [anon_sym_default] = ACTIONS(1),
    [anon_sym_enum] = ACTIONS(1),
    [anon_sym_fn] = ACTIONS(1),
    [anon_sym_for] = ACTIONS(1),
    [anon_sym_gen] = ACTIONS(1),
    [anon_sym_if] = ACTIONS(1),
    [anon_sym_impl] = ACTIONS(1),
    [anon_sym_let] = ACTIONS(1),
    [anon_sym_loop] = ACTIONS(1),
    [anon_sym_match] = ACTIONS(1),
    [anon_sym_mod] = ACTIONS(1),
    [anon_sym_pub] = ACTIONS(1),
    [anon_sym_return] = ACTIONS(1),
    [anon_sym_static] = ACTIONS(1),
    [anon_sym_struct] = ACTIONS(1),
    [anon_sym_trait] = ACTIONS(1),
    [anon_sym_type] = ACTIONS(1),
    [anon_sym_union] = ACTIONS(1),
    [anon_sym_unsafe] = ACTIONS(1),
    [anon_sym_use] = ACTIONS(1),
    [anon_sym_where] = ACTIONS(1),
    [anon_sym_while] = ACTIONS(1),
    [anon_sym_extern] = ACTIONS(1),
    [anon_sym_ref] = ACTIONS(1),
    [anon_sym_else] = ACTIONS(1),
    [anon_sym_in] = ACTIONS(1),
    [anon_sym_LT2] = ACTIONS(1),
    [anon_sym_dyn] = ACTIONS(1),
    [sym_mutable_specifier] = ACTIONS(1),
    [anon_sym_raw] = ACTIONS(1),
    [anon_sym_yield] = ACTIONS(1),
    [anon_sym_move] = ACTIONS(1),
    [anon_sym_try] = ACTIONS(1),
    [sym_integer_literal] = ACTIONS(1),
    [aux_sym_string_literal_token1] = ACTIONS(1),
    [anon_sym_DQUOTE] = ACTIONS(1),
    [sym_char_literal] = ACTIONS(1),
    [sym_escape_sequence] = ACTIONS(1),
    [anon_sym_true] = ACTIONS(1),
    [anon_sym_false] = ACTIONS(1),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_BANG2] = ACTIONS(1),
    [anon_sym_SLASH2] = ACTIONS(1),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_shebang] = ACTIONS(1),
    [sym_self] = ACTIONS(1),
    [sym_super] = ACTIONS(1),
    [sym_crate] = ACTIONS(1),
    [sym_metavariable] = ACTIONS(1),
    [sym_string_content] = ACTIONS(1),
    [sym__raw_string_literal_start] = ACTIONS(1),
    [sym_raw_string_literal_content] = ACTIONS(1),
    [sym__raw_string_literal_end] = ACTIONS(1),
    [sym_float_literal] = ACTIONS(1),
    [sym__outer_block_doc_comment_marker] = ACTIONS(1),
    [sym__inner_block_doc_comment_marker] = ACTIONS(1),
    [sym__block_comment_content] = ACTIONS(1),
    [sym__line_doc_content] = ACTIONS(1),
    [sym__error_sentinel] = ACTIONS(1),
  },
  [1] = {
    [sym_source_file] = STATE(3457),
    [sym__statement] = STATE(659),
    [sym_empty_statement] = STATE(665),
    [sym_expression_statement] = STATE(665),
    [sym_macro_definition] = STATE(665),
    [sym_attribute_item] = STATE(665),
    [sym_inner_attribute_item] = STATE(665),
    [sym_mod_item] = STATE(665),
    [sym_foreign_mod_item] = STATE(665),
    [sym_struct_item] = STATE(665),
    [sym_union_item] = STATE(665),
    [sym_enum_item] = STATE(665),
    [sym_extern_crate_declaration] = STATE(665),
    [sym_const_item] = STATE(665),
    [sym_static_item] = STATE(665),
    [sym_type_item] = STATE(665),
    [sym_function_item] = STATE(665),
    [sym_function_signature_item] = STATE(665),
    [sym_function_modifiers] = STATE(3607),
    [sym_impl_item] = STATE(665),
    [sym_trait_item] = STATE(665),
    [sym_associated_type] = STATE(665),
    [sym_let_declaration] = STATE(665),
    [sym_use_declaration] = STATE(665),
    [sym_extern_modifier] = STATE(2160),
    [sym_visibility_modifier] = STATE(1950),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1886),
    [sym_macro_invocation] = STATE(399),
    [sym_scoped_identifier] = STATE(1552),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(377),
    [sym_match_expression] = STATE(377),
    [sym_while_expression] = STATE(377),
    [sym_loop_expression] = STATE(377),
    [sym_for_expression] = STATE(377),
    [sym_const_block] = STATE(377),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3577),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(377),
    [sym_async_block] = STATE(377),
    [sym_gen_block] = STATE(377),
    [sym_try_block] = STATE(377),
    [sym_block] = STATE(377),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(1),
    [sym_block_comment] = STATE(1),
    [aux_sym_source_file_repeat1] = STATE(34),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [ts_builtin_sym_end] = ACTIONS(7),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro_rules_BANG] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(19),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(35),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(43),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(47),
    [anon_sym_enum] = ACTIONS(49),
    [anon_sym_fn] = ACTIONS(51),
    [anon_sym_for] = ACTIONS(53),
    [anon_sym_gen] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_impl] = ACTIONS(59),
    [anon_sym_let] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(63),
    [anon_sym_match] = ACTIONS(65),
    [anon_sym_mod] = ACTIONS(67),
    [anon_sym_pub] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(73),
    [anon_sym_struct] = ACTIONS(75),
    [anon_sym_trait] = ACTIONS(77),
    [anon_sym_type] = ACTIONS(79),
    [anon_sym_union] = ACTIONS(81),
    [anon_sym_unsafe] = ACTIONS(83),
    [anon_sym_use] = ACTIONS(85),
    [anon_sym_while] = ACTIONS(87),
    [anon_sym_extern] = ACTIONS(89),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(95),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_shebang] = ACTIONS(107),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(113),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [2] = {
    [sym__statement] = STATE(659),
    [sym_empty_statement] = STATE(665),
    [sym_expression_statement] = STATE(665),
    [sym_macro_definition] = STATE(665),
    [sym_attribute_item] = STATE(665),
    [sym_inner_attribute_item] = STATE(665),
    [sym_mod_item] = STATE(665),
    [sym_foreign_mod_item] = STATE(665),
    [sym_struct_item] = STATE(665),
    [sym_union_item] = STATE(665),
    [sym_enum_item] = STATE(665),
    [sym_extern_crate_declaration] = STATE(665),
    [sym_const_item] = STATE(665),
    [sym_static_item] = STATE(665),
    [sym_type_item] = STATE(665),
    [sym_function_item] = STATE(665),
    [sym_function_signature_item] = STATE(665),
    [sym_function_modifiers] = STATE(3607),
    [sym_impl_item] = STATE(665),
    [sym_trait_item] = STATE(665),
    [sym_associated_type] = STATE(665),
    [sym_let_declaration] = STATE(665),
    [sym_use_declaration] = STATE(665),
    [sym_extern_modifier] = STATE(2160),
    [sym_visibility_modifier] = STATE(1950),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1806),
    [sym_macro_invocation] = STATE(399),
    [sym_scoped_identifier] = STATE(1552),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(377),
    [sym_match_expression] = STATE(377),
    [sym_while_expression] = STATE(377),
    [sym_loop_expression] = STATE(377),
    [sym_for_expression] = STATE(377),
    [sym_const_block] = STATE(377),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3577),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(377),
    [sym_async_block] = STATE(377),
    [sym_gen_block] = STATE(377),
    [sym_try_block] = STATE(377),
    [sym_block] = STATE(377),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(2),
    [sym_block_comment] = STATE(2),
    [aux_sym_source_file_repeat1] = STATE(5),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro_rules_BANG] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(19),
    [anon_sym_RBRACE] = ACTIONS(119),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(121),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(43),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(47),
    [anon_sym_enum] = ACTIONS(49),
    [anon_sym_fn] = ACTIONS(51),
    [anon_sym_for] = ACTIONS(53),
    [anon_sym_gen] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_impl] = ACTIONS(59),
    [anon_sym_let] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(63),
    [anon_sym_match] = ACTIONS(65),
    [anon_sym_mod] = ACTIONS(67),
    [anon_sym_pub] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(73),
    [anon_sym_struct] = ACTIONS(75),
    [anon_sym_trait] = ACTIONS(77),
    [anon_sym_type] = ACTIONS(79),
    [anon_sym_union] = ACTIONS(81),
    [anon_sym_unsafe] = ACTIONS(83),
    [anon_sym_use] = ACTIONS(85),
    [anon_sym_while] = ACTIONS(87),
    [anon_sym_extern] = ACTIONS(89),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(95),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(113),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [3] = {
    [sym__statement] = STATE(659),
    [sym_empty_statement] = STATE(665),
    [sym_expression_statement] = STATE(665),
    [sym_macro_definition] = STATE(665),
    [sym_attribute_item] = STATE(665),
    [sym_inner_attribute_item] = STATE(665),
    [sym_mod_item] = STATE(665),
    [sym_foreign_mod_item] = STATE(665),
    [sym_struct_item] = STATE(665),
    [sym_union_item] = STATE(665),
    [sym_enum_item] = STATE(665),
    [sym_extern_crate_declaration] = STATE(665),
    [sym_const_item] = STATE(665),
    [sym_static_item] = STATE(665),
    [sym_type_item] = STATE(665),
    [sym_function_item] = STATE(665),
    [sym_function_signature_item] = STATE(665),
    [sym_function_modifiers] = STATE(3607),
    [sym_impl_item] = STATE(665),
    [sym_trait_item] = STATE(665),
    [sym_associated_type] = STATE(665),
    [sym_let_declaration] = STATE(665),
    [sym_use_declaration] = STATE(665),
    [sym_extern_modifier] = STATE(2160),
    [sym_visibility_modifier] = STATE(1950),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1854),
    [sym_macro_invocation] = STATE(399),
    [sym_scoped_identifier] = STATE(1552),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(377),
    [sym_match_expression] = STATE(377),
    [sym_while_expression] = STATE(377),
    [sym_loop_expression] = STATE(377),
    [sym_for_expression] = STATE(377),
    [sym_const_block] = STATE(377),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3577),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(377),
    [sym_async_block] = STATE(377),
    [sym_gen_block] = STATE(377),
    [sym_try_block] = STATE(377),
    [sym_block] = STATE(377),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(3),
    [sym_block_comment] = STATE(3),
    [aux_sym_source_file_repeat1] = STATE(12),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro_rules_BANG] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(19),
    [anon_sym_RBRACE] = ACTIONS(123),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(121),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(43),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(47),
    [anon_sym_enum] = ACTIONS(49),
    [anon_sym_fn] = ACTIONS(51),
    [anon_sym_for] = ACTIONS(53),
    [anon_sym_gen] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_impl] = ACTIONS(59),
    [anon_sym_let] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(63),
    [anon_sym_match] = ACTIONS(65),
    [anon_sym_mod] = ACTIONS(67),
    [anon_sym_pub] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(73),
    [anon_sym_struct] = ACTIONS(75),
    [anon_sym_trait] = ACTIONS(77),
    [anon_sym_type] = ACTIONS(79),
    [anon_sym_union] = ACTIONS(81),
    [anon_sym_unsafe] = ACTIONS(83),
    [anon_sym_use] = ACTIONS(85),
    [anon_sym_while] = ACTIONS(87),
    [anon_sym_extern] = ACTIONS(89),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(95),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(113),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [4] = {
    [sym__statement] = STATE(659),
    [sym_empty_statement] = STATE(665),
    [sym_expression_statement] = STATE(665),
    [sym_macro_definition] = STATE(665),
    [sym_attribute_item] = STATE(665),
    [sym_inner_attribute_item] = STATE(665),
    [sym_mod_item] = STATE(665),
    [sym_foreign_mod_item] = STATE(665),
    [sym_struct_item] = STATE(665),
    [sym_union_item] = STATE(665),
    [sym_enum_item] = STATE(665),
    [sym_extern_crate_declaration] = STATE(665),
    [sym_const_item] = STATE(665),
    [sym_static_item] = STATE(665),
    [sym_type_item] = STATE(665),
    [sym_function_item] = STATE(665),
    [sym_function_signature_item] = STATE(665),
    [sym_function_modifiers] = STATE(3607),
    [sym_impl_item] = STATE(665),
    [sym_trait_item] = STATE(665),
    [sym_associated_type] = STATE(665),
    [sym_let_declaration] = STATE(665),
    [sym_use_declaration] = STATE(665),
    [sym_extern_modifier] = STATE(2160),
    [sym_visibility_modifier] = STATE(1950),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1886),
    [sym_macro_invocation] = STATE(399),
    [sym_scoped_identifier] = STATE(1552),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(377),
    [sym_match_expression] = STATE(377),
    [sym_while_expression] = STATE(377),
    [sym_loop_expression] = STATE(377),
    [sym_for_expression] = STATE(377),
    [sym_const_block] = STATE(377),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3577),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(377),
    [sym_async_block] = STATE(377),
    [sym_gen_block] = STATE(377),
    [sym_try_block] = STATE(377),
    [sym_block] = STATE(377),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(4),
    [sym_block_comment] = STATE(4),
    [aux_sym_source_file_repeat1] = STATE(6),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [ts_builtin_sym_end] = ACTIONS(125),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro_rules_BANG] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(19),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(121),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(43),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(47),
    [anon_sym_enum] = ACTIONS(49),
    [anon_sym_fn] = ACTIONS(51),
    [anon_sym_for] = ACTIONS(53),
    [anon_sym_gen] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_impl] = ACTIONS(59),
    [anon_sym_let] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(63),
    [anon_sym_match] = ACTIONS(65),
    [anon_sym_mod] = ACTIONS(67),
    [anon_sym_pub] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(73),
    [anon_sym_struct] = ACTIONS(75),
    [anon_sym_trait] = ACTIONS(77),
    [anon_sym_type] = ACTIONS(79),
    [anon_sym_union] = ACTIONS(81),
    [anon_sym_unsafe] = ACTIONS(83),
    [anon_sym_use] = ACTIONS(85),
    [anon_sym_while] = ACTIONS(87),
    [anon_sym_extern] = ACTIONS(89),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(95),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(113),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [5] = {
    [sym__statement] = STATE(659),
    [sym_empty_statement] = STATE(665),
    [sym_expression_statement] = STATE(665),
    [sym_macro_definition] = STATE(665),
    [sym_attribute_item] = STATE(665),
    [sym_inner_attribute_item] = STATE(665),
    [sym_mod_item] = STATE(665),
    [sym_foreign_mod_item] = STATE(665),
    [sym_struct_item] = STATE(665),
    [sym_union_item] = STATE(665),
    [sym_enum_item] = STATE(665),
    [sym_extern_crate_declaration] = STATE(665),
    [sym_const_item] = STATE(665),
    [sym_static_item] = STATE(665),
    [sym_type_item] = STATE(665),
    [sym_function_item] = STATE(665),
    [sym_function_signature_item] = STATE(665),
    [sym_function_modifiers] = STATE(3607),
    [sym_impl_item] = STATE(665),
    [sym_trait_item] = STATE(665),
    [sym_associated_type] = STATE(665),
    [sym_let_declaration] = STATE(665),
    [sym_use_declaration] = STATE(665),
    [sym_extern_modifier] = STATE(2160),
    [sym_visibility_modifier] = STATE(1950),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1654),
    [sym_macro_invocation] = STATE(399),
    [sym_scoped_identifier] = STATE(1552),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(377),
    [sym_match_expression] = STATE(377),
    [sym_while_expression] = STATE(377),
    [sym_loop_expression] = STATE(377),
    [sym_for_expression] = STATE(377),
    [sym_const_block] = STATE(377),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3577),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(377),
    [sym_async_block] = STATE(377),
    [sym_gen_block] = STATE(377),
    [sym_try_block] = STATE(377),
    [sym_block] = STATE(377),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(5),
    [sym_block_comment] = STATE(5),
    [aux_sym_source_file_repeat1] = STATE(12),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro_rules_BANG] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(19),
    [anon_sym_RBRACE] = ACTIONS(127),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(121),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(43),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(47),
    [anon_sym_enum] = ACTIONS(49),
    [anon_sym_fn] = ACTIONS(51),
    [anon_sym_for] = ACTIONS(53),
    [anon_sym_gen] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_impl] = ACTIONS(59),
    [anon_sym_let] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(63),
    [anon_sym_match] = ACTIONS(65),
    [anon_sym_mod] = ACTIONS(67),
    [anon_sym_pub] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(73),
    [anon_sym_struct] = ACTIONS(75),
    [anon_sym_trait] = ACTIONS(77),
    [anon_sym_type] = ACTIONS(79),
    [anon_sym_union] = ACTIONS(81),
    [anon_sym_unsafe] = ACTIONS(83),
    [anon_sym_use] = ACTIONS(85),
    [anon_sym_while] = ACTIONS(87),
    [anon_sym_extern] = ACTIONS(89),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(95),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(113),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [6] = {
    [sym__statement] = STATE(659),
    [sym_empty_statement] = STATE(665),
    [sym_expression_statement] = STATE(665),
    [sym_macro_definition] = STATE(665),
    [sym_attribute_item] = STATE(665),
    [sym_inner_attribute_item] = STATE(665),
    [sym_mod_item] = STATE(665),
    [sym_foreign_mod_item] = STATE(665),
    [sym_struct_item] = STATE(665),
    [sym_union_item] = STATE(665),
    [sym_enum_item] = STATE(665),
    [sym_extern_crate_declaration] = STATE(665),
    [sym_const_item] = STATE(665),
    [sym_static_item] = STATE(665),
    [sym_type_item] = STATE(665),
    [sym_function_item] = STATE(665),
    [sym_function_signature_item] = STATE(665),
    [sym_function_modifiers] = STATE(3607),
    [sym_impl_item] = STATE(665),
    [sym_trait_item] = STATE(665),
    [sym_associated_type] = STATE(665),
    [sym_let_declaration] = STATE(665),
    [sym_use_declaration] = STATE(665),
    [sym_extern_modifier] = STATE(2160),
    [sym_visibility_modifier] = STATE(1950),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1886),
    [sym_macro_invocation] = STATE(399),
    [sym_scoped_identifier] = STATE(1552),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(377),
    [sym_match_expression] = STATE(377),
    [sym_while_expression] = STATE(377),
    [sym_loop_expression] = STATE(377),
    [sym_for_expression] = STATE(377),
    [sym_const_block] = STATE(377),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3577),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(377),
    [sym_async_block] = STATE(377),
    [sym_gen_block] = STATE(377),
    [sym_try_block] = STATE(377),
    [sym_block] = STATE(377),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(6),
    [sym_block_comment] = STATE(6),
    [aux_sym_source_file_repeat1] = STATE(7),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [ts_builtin_sym_end] = ACTIONS(129),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro_rules_BANG] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(19),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(121),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(43),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(47),
    [anon_sym_enum] = ACTIONS(49),
    [anon_sym_fn] = ACTIONS(51),
    [anon_sym_for] = ACTIONS(53),
    [anon_sym_gen] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_impl] = ACTIONS(59),
    [anon_sym_let] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(63),
    [anon_sym_match] = ACTIONS(65),
    [anon_sym_mod] = ACTIONS(67),
    [anon_sym_pub] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(73),
    [anon_sym_struct] = ACTIONS(75),
    [anon_sym_trait] = ACTIONS(77),
    [anon_sym_type] = ACTIONS(79),
    [anon_sym_union] = ACTIONS(81),
    [anon_sym_unsafe] = ACTIONS(83),
    [anon_sym_use] = ACTIONS(85),
    [anon_sym_while] = ACTIONS(87),
    [anon_sym_extern] = ACTIONS(89),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(95),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(113),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [7] = {
    [sym__statement] = STATE(659),
    [sym_empty_statement] = STATE(665),
    [sym_expression_statement] = STATE(665),
    [sym_macro_definition] = STATE(665),
    [sym_attribute_item] = STATE(665),
    [sym_inner_attribute_item] = STATE(665),
    [sym_mod_item] = STATE(665),
    [sym_foreign_mod_item] = STATE(665),
    [sym_struct_item] = STATE(665),
    [sym_union_item] = STATE(665),
    [sym_enum_item] = STATE(665),
    [sym_extern_crate_declaration] = STATE(665),
    [sym_const_item] = STATE(665),
    [sym_static_item] = STATE(665),
    [sym_type_item] = STATE(665),
    [sym_function_item] = STATE(665),
    [sym_function_signature_item] = STATE(665),
    [sym_function_modifiers] = STATE(3607),
    [sym_impl_item] = STATE(665),
    [sym_trait_item] = STATE(665),
    [sym_associated_type] = STATE(665),
    [sym_let_declaration] = STATE(665),
    [sym_use_declaration] = STATE(665),
    [sym_extern_modifier] = STATE(2160),
    [sym_visibility_modifier] = STATE(1950),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1886),
    [sym_macro_invocation] = STATE(399),
    [sym_scoped_identifier] = STATE(1552),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(377),
    [sym_match_expression] = STATE(377),
    [sym_while_expression] = STATE(377),
    [sym_loop_expression] = STATE(377),
    [sym_for_expression] = STATE(377),
    [sym_const_block] = STATE(377),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3577),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(377),
    [sym_async_block] = STATE(377),
    [sym_gen_block] = STATE(377),
    [sym_try_block] = STATE(377),
    [sym_block] = STATE(377),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(7),
    [sym_block_comment] = STATE(7),
    [aux_sym_source_file_repeat1] = STATE(7),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [ts_builtin_sym_end] = ACTIONS(131),
    [sym_identifier] = ACTIONS(133),
    [anon_sym_SEMI] = ACTIONS(136),
    [anon_sym_macro_rules_BANG] = ACTIONS(139),
    [anon_sym_LPAREN] = ACTIONS(142),
    [anon_sym_LBRACK] = ACTIONS(145),
    [anon_sym_LBRACE] = ACTIONS(148),
    [anon_sym_STAR] = ACTIONS(151),
    [anon_sym_u8] = ACTIONS(154),
    [anon_sym_i8] = ACTIONS(154),
    [anon_sym_u16] = ACTIONS(154),
    [anon_sym_i16] = ACTIONS(154),
    [anon_sym_u32] = ACTIONS(154),
    [anon_sym_i32] = ACTIONS(154),
    [anon_sym_u64] = ACTIONS(154),
    [anon_sym_i64] = ACTIONS(154),
    [anon_sym_u128] = ACTIONS(154),
    [anon_sym_i128] = ACTIONS(154),
    [anon_sym_isize] = ACTIONS(154),
    [anon_sym_usize] = ACTIONS(154),
    [anon_sym_f32] = ACTIONS(154),
    [anon_sym_f64] = ACTIONS(154),
    [anon_sym_bool] = ACTIONS(154),
    [anon_sym_str] = ACTIONS(154),
    [anon_sym_char] = ACTIONS(154),
    [anon_sym_DASH] = ACTIONS(151),
    [anon_sym_BANG] = ACTIONS(151),
    [anon_sym_AMP] = ACTIONS(157),
    [anon_sym_PIPE] = ACTIONS(160),
    [anon_sym_LT] = ACTIONS(163),
    [anon_sym_DOT_DOT] = ACTIONS(166),
    [anon_sym_COLON_COLON] = ACTIONS(169),
    [anon_sym_POUND] = ACTIONS(172),
    [anon_sym_SQUOTE] = ACTIONS(175),
    [anon_sym_async] = ACTIONS(178),
    [anon_sym_break] = ACTIONS(181),
    [anon_sym_const] = ACTIONS(184),
    [anon_sym_continue] = ACTIONS(187),
    [anon_sym_default] = ACTIONS(190),
    [anon_sym_enum] = ACTIONS(193),
    [anon_sym_fn] = ACTIONS(196),
    [anon_sym_for] = ACTIONS(199),
    [anon_sym_gen] = ACTIONS(202),
    [anon_sym_if] = ACTIONS(205),
    [anon_sym_impl] = ACTIONS(208),
    [anon_sym_let] = ACTIONS(211),
    [anon_sym_loop] = ACTIONS(214),
    [anon_sym_match] = ACTIONS(217),
    [anon_sym_mod] = ACTIONS(220),
    [anon_sym_pub] = ACTIONS(223),
    [anon_sym_return] = ACTIONS(226),
    [anon_sym_static] = ACTIONS(229),
    [anon_sym_struct] = ACTIONS(232),
    [anon_sym_trait] = ACTIONS(235),
    [anon_sym_type] = ACTIONS(238),
    [anon_sym_union] = ACTIONS(241),
    [anon_sym_unsafe] = ACTIONS(244),
    [anon_sym_use] = ACTIONS(247),
    [anon_sym_while] = ACTIONS(250),
    [anon_sym_extern] = ACTIONS(253),
    [anon_sym_yield] = ACTIONS(256),
    [anon_sym_move] = ACTIONS(259),
    [anon_sym_try] = ACTIONS(262),
    [sym_integer_literal] = ACTIONS(265),
    [aux_sym_string_literal_token1] = ACTIONS(268),
    [sym_char_literal] = ACTIONS(265),
    [anon_sym_true] = ACTIONS(271),
    [anon_sym_false] = ACTIONS(271),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(274),
    [sym_super] = ACTIONS(277),
    [sym_crate] = ACTIONS(280),
    [sym_metavariable] = ACTIONS(283),
    [sym__raw_string_literal_start] = ACTIONS(286),
    [sym_float_literal] = ACTIONS(265),
  },
  [8] = {
    [sym__statement] = STATE(659),
    [sym_empty_statement] = STATE(665),
    [sym_expression_statement] = STATE(665),
    [sym_macro_definition] = STATE(665),
    [sym_attribute_item] = STATE(665),
    [sym_inner_attribute_item] = STATE(665),
    [sym_mod_item] = STATE(665),
    [sym_foreign_mod_item] = STATE(665),
    [sym_struct_item] = STATE(665),
    [sym_union_item] = STATE(665),
    [sym_enum_item] = STATE(665),
    [sym_extern_crate_declaration] = STATE(665),
    [sym_const_item] = STATE(665),
    [sym_static_item] = STATE(665),
    [sym_type_item] = STATE(665),
    [sym_function_item] = STATE(665),
    [sym_function_signature_item] = STATE(665),
    [sym_function_modifiers] = STATE(3607),
    [sym_impl_item] = STATE(665),
    [sym_trait_item] = STATE(665),
    [sym_associated_type] = STATE(665),
    [sym_let_declaration] = STATE(665),
    [sym_use_declaration] = STATE(665),
    [sym_extern_modifier] = STATE(2160),
    [sym_visibility_modifier] = STATE(1950),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1831),
    [sym_macro_invocation] = STATE(399),
    [sym_scoped_identifier] = STATE(1552),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(377),
    [sym_match_expression] = STATE(377),
    [sym_while_expression] = STATE(377),
    [sym_loop_expression] = STATE(377),
    [sym_for_expression] = STATE(377),
    [sym_const_block] = STATE(377),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3577),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(377),
    [sym_async_block] = STATE(377),
    [sym_gen_block] = STATE(377),
    [sym_try_block] = STATE(377),
    [sym_block] = STATE(377),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(8),
    [sym_block_comment] = STATE(8),
    [aux_sym_source_file_repeat1] = STATE(9),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro_rules_BANG] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(19),
    [anon_sym_RBRACE] = ACTIONS(289),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(121),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(43),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(47),
    [anon_sym_enum] = ACTIONS(49),
    [anon_sym_fn] = ACTIONS(51),
    [anon_sym_for] = ACTIONS(53),
    [anon_sym_gen] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_impl] = ACTIONS(59),
    [anon_sym_let] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(63),
    [anon_sym_match] = ACTIONS(65),
    [anon_sym_mod] = ACTIONS(67),
    [anon_sym_pub] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(73),
    [anon_sym_struct] = ACTIONS(75),
    [anon_sym_trait] = ACTIONS(77),
    [anon_sym_type] = ACTIONS(79),
    [anon_sym_union] = ACTIONS(81),
    [anon_sym_unsafe] = ACTIONS(83),
    [anon_sym_use] = ACTIONS(85),
    [anon_sym_while] = ACTIONS(87),
    [anon_sym_extern] = ACTIONS(89),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(95),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(113),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [9] = {
    [sym__statement] = STATE(659),
    [sym_empty_statement] = STATE(665),
    [sym_expression_statement] = STATE(665),
    [sym_macro_definition] = STATE(665),
    [sym_attribute_item] = STATE(665),
    [sym_inner_attribute_item] = STATE(665),
    [sym_mod_item] = STATE(665),
    [sym_foreign_mod_item] = STATE(665),
    [sym_struct_item] = STATE(665),
    [sym_union_item] = STATE(665),
    [sym_enum_item] = STATE(665),
    [sym_extern_crate_declaration] = STATE(665),
    [sym_const_item] = STATE(665),
    [sym_static_item] = STATE(665),
    [sym_type_item] = STATE(665),
    [sym_function_item] = STATE(665),
    [sym_function_signature_item] = STATE(665),
    [sym_function_modifiers] = STATE(3607),
    [sym_impl_item] = STATE(665),
    [sym_trait_item] = STATE(665),
    [sym_associated_type] = STATE(665),
    [sym_let_declaration] = STATE(665),
    [sym_use_declaration] = STATE(665),
    [sym_extern_modifier] = STATE(2160),
    [sym_visibility_modifier] = STATE(1950),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1832),
    [sym_macro_invocation] = STATE(399),
    [sym_scoped_identifier] = STATE(1552),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(377),
    [sym_match_expression] = STATE(377),
    [sym_while_expression] = STATE(377),
    [sym_loop_expression] = STATE(377),
    [sym_for_expression] = STATE(377),
    [sym_const_block] = STATE(377),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3577),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(377),
    [sym_async_block] = STATE(377),
    [sym_gen_block] = STATE(377),
    [sym_try_block] = STATE(377),
    [sym_block] = STATE(377),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(9),
    [sym_block_comment] = STATE(9),
    [aux_sym_source_file_repeat1] = STATE(12),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro_rules_BANG] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(19),
    [anon_sym_RBRACE] = ACTIONS(291),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(121),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(43),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(47),
    [anon_sym_enum] = ACTIONS(49),
    [anon_sym_fn] = ACTIONS(51),
    [anon_sym_for] = ACTIONS(53),
    [anon_sym_gen] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_impl] = ACTIONS(59),
    [anon_sym_let] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(63),
    [anon_sym_match] = ACTIONS(65),
    [anon_sym_mod] = ACTIONS(67),
    [anon_sym_pub] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(73),
    [anon_sym_struct] = ACTIONS(75),
    [anon_sym_trait] = ACTIONS(77),
    [anon_sym_type] = ACTIONS(79),
    [anon_sym_union] = ACTIONS(81),
    [anon_sym_unsafe] = ACTIONS(83),
    [anon_sym_use] = ACTIONS(85),
    [anon_sym_while] = ACTIONS(87),
    [anon_sym_extern] = ACTIONS(89),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(95),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(113),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [10] = {
    [sym__statement] = STATE(659),
    [sym_empty_statement] = STATE(665),
    [sym_expression_statement] = STATE(665),
    [sym_macro_definition] = STATE(665),
    [sym_attribute_item] = STATE(665),
    [sym_inner_attribute_item] = STATE(665),
    [sym_mod_item] = STATE(665),
    [sym_foreign_mod_item] = STATE(665),
    [sym_struct_item] = STATE(665),
    [sym_union_item] = STATE(665),
    [sym_enum_item] = STATE(665),
    [sym_extern_crate_declaration] = STATE(665),
    [sym_const_item] = STATE(665),
    [sym_static_item] = STATE(665),
    [sym_type_item] = STATE(665),
    [sym_function_item] = STATE(665),
    [sym_function_signature_item] = STATE(665),
    [sym_function_modifiers] = STATE(3607),
    [sym_impl_item] = STATE(665),
    [sym_trait_item] = STATE(665),
    [sym_associated_type] = STATE(665),
    [sym_let_declaration] = STATE(665),
    [sym_use_declaration] = STATE(665),
    [sym_extern_modifier] = STATE(2160),
    [sym_visibility_modifier] = STATE(1950),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1801),
    [sym_macro_invocation] = STATE(399),
    [sym_scoped_identifier] = STATE(1552),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(377),
    [sym_match_expression] = STATE(377),
    [sym_while_expression] = STATE(377),
    [sym_loop_expression] = STATE(377),
    [sym_for_expression] = STATE(377),
    [sym_const_block] = STATE(377),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3577),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(377),
    [sym_async_block] = STATE(377),
    [sym_gen_block] = STATE(377),
    [sym_try_block] = STATE(377),
    [sym_block] = STATE(377),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(10),
    [sym_block_comment] = STATE(10),
    [aux_sym_source_file_repeat1] = STATE(11),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro_rules_BANG] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(19),
    [anon_sym_RBRACE] = ACTIONS(293),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(121),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(43),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(47),
    [anon_sym_enum] = ACTIONS(49),
    [anon_sym_fn] = ACTIONS(51),
    [anon_sym_for] = ACTIONS(53),
    [anon_sym_gen] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_impl] = ACTIONS(59),
    [anon_sym_let] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(63),
    [anon_sym_match] = ACTIONS(65),
    [anon_sym_mod] = ACTIONS(67),
    [anon_sym_pub] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(73),
    [anon_sym_struct] = ACTIONS(75),
    [anon_sym_trait] = ACTIONS(77),
    [anon_sym_type] = ACTIONS(79),
    [anon_sym_union] = ACTIONS(81),
    [anon_sym_unsafe] = ACTIONS(83),
    [anon_sym_use] = ACTIONS(85),
    [anon_sym_while] = ACTIONS(87),
    [anon_sym_extern] = ACTIONS(89),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(95),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(113),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [11] = {
    [sym__statement] = STATE(659),
    [sym_empty_statement] = STATE(665),
    [sym_expression_statement] = STATE(665),
    [sym_macro_definition] = STATE(665),
    [sym_attribute_item] = STATE(665),
    [sym_inner_attribute_item] = STATE(665),
    [sym_mod_item] = STATE(665),
    [sym_foreign_mod_item] = STATE(665),
    [sym_struct_item] = STATE(665),
    [sym_union_item] = STATE(665),
    [sym_enum_item] = STATE(665),
    [sym_extern_crate_declaration] = STATE(665),
    [sym_const_item] = STATE(665),
    [sym_static_item] = STATE(665),
    [sym_type_item] = STATE(665),
    [sym_function_item] = STATE(665),
    [sym_function_signature_item] = STATE(665),
    [sym_function_modifiers] = STATE(3607),
    [sym_impl_item] = STATE(665),
    [sym_trait_item] = STATE(665),
    [sym_associated_type] = STATE(665),
    [sym_let_declaration] = STATE(665),
    [sym_use_declaration] = STATE(665),
    [sym_extern_modifier] = STATE(2160),
    [sym_visibility_modifier] = STATE(1950),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1807),
    [sym_macro_invocation] = STATE(399),
    [sym_scoped_identifier] = STATE(1552),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(377),
    [sym_match_expression] = STATE(377),
    [sym_while_expression] = STATE(377),
    [sym_loop_expression] = STATE(377),
    [sym_for_expression] = STATE(377),
    [sym_const_block] = STATE(377),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3577),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(377),
    [sym_async_block] = STATE(377),
    [sym_gen_block] = STATE(377),
    [sym_try_block] = STATE(377),
    [sym_block] = STATE(377),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(11),
    [sym_block_comment] = STATE(11),
    [aux_sym_source_file_repeat1] = STATE(12),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro_rules_BANG] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(19),
    [anon_sym_RBRACE] = ACTIONS(295),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(121),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(43),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(47),
    [anon_sym_enum] = ACTIONS(49),
    [anon_sym_fn] = ACTIONS(51),
    [anon_sym_for] = ACTIONS(53),
    [anon_sym_gen] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_impl] = ACTIONS(59),
    [anon_sym_let] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(63),
    [anon_sym_match] = ACTIONS(65),
    [anon_sym_mod] = ACTIONS(67),
    [anon_sym_pub] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(73),
    [anon_sym_struct] = ACTIONS(75),
    [anon_sym_trait] = ACTIONS(77),
    [anon_sym_type] = ACTIONS(79),
    [anon_sym_union] = ACTIONS(81),
    [anon_sym_unsafe] = ACTIONS(83),
    [anon_sym_use] = ACTIONS(85),
    [anon_sym_while] = ACTIONS(87),
    [anon_sym_extern] = ACTIONS(89),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(95),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(113),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [12] = {
    [sym__statement] = STATE(659),
    [sym_empty_statement] = STATE(665),
    [sym_expression_statement] = STATE(665),
    [sym_macro_definition] = STATE(665),
    [sym_attribute_item] = STATE(665),
    [sym_inner_attribute_item] = STATE(665),
    [sym_mod_item] = STATE(665),
    [sym_foreign_mod_item] = STATE(665),
    [sym_struct_item] = STATE(665),
    [sym_union_item] = STATE(665),
    [sym_enum_item] = STATE(665),
    [sym_extern_crate_declaration] = STATE(665),
    [sym_const_item] = STATE(665),
    [sym_static_item] = STATE(665),
    [sym_type_item] = STATE(665),
    [sym_function_item] = STATE(665),
    [sym_function_signature_item] = STATE(665),
    [sym_function_modifiers] = STATE(3607),
    [sym_impl_item] = STATE(665),
    [sym_trait_item] = STATE(665),
    [sym_associated_type] = STATE(665),
    [sym_let_declaration] = STATE(665),
    [sym_use_declaration] = STATE(665),
    [sym_extern_modifier] = STATE(2160),
    [sym_visibility_modifier] = STATE(1950),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1886),
    [sym_macro_invocation] = STATE(409),
    [sym_scoped_identifier] = STATE(1552),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(377),
    [sym_match_expression] = STATE(377),
    [sym_while_expression] = STATE(377),
    [sym_loop_expression] = STATE(377),
    [sym_for_expression] = STATE(377),
    [sym_const_block] = STATE(377),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3577),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(377),
    [sym_async_block] = STATE(377),
    [sym_gen_block] = STATE(377),
    [sym_try_block] = STATE(377),
    [sym_block] = STATE(377),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(12),
    [sym_block_comment] = STATE(12),
    [aux_sym_source_file_repeat1] = STATE(12),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(133),
    [anon_sym_SEMI] = ACTIONS(136),
    [anon_sym_macro_rules_BANG] = ACTIONS(139),
    [anon_sym_LPAREN] = ACTIONS(142),
    [anon_sym_LBRACK] = ACTIONS(145),
    [anon_sym_LBRACE] = ACTIONS(148),
    [anon_sym_RBRACE] = ACTIONS(131),
    [anon_sym_STAR] = ACTIONS(151),
    [anon_sym_u8] = ACTIONS(154),
    [anon_sym_i8] = ACTIONS(154),
    [anon_sym_u16] = ACTIONS(154),
    [anon_sym_i16] = ACTIONS(154),
    [anon_sym_u32] = ACTIONS(154),
    [anon_sym_i32] = ACTIONS(154),
    [anon_sym_u64] = ACTIONS(154),
    [anon_sym_i64] = ACTIONS(154),
    [anon_sym_u128] = ACTIONS(154),
    [anon_sym_i128] = ACTIONS(154),
    [anon_sym_isize] = ACTIONS(154),
    [anon_sym_usize] = ACTIONS(154),
    [anon_sym_f32] = ACTIONS(154),
    [anon_sym_f64] = ACTIONS(154),
    [anon_sym_bool] = ACTIONS(154),
    [anon_sym_str] = ACTIONS(154),
    [anon_sym_char] = ACTIONS(154),
    [anon_sym_DASH] = ACTIONS(151),
    [anon_sym_BANG] = ACTIONS(151),
    [anon_sym_AMP] = ACTIONS(157),
    [anon_sym_PIPE] = ACTIONS(160),
    [anon_sym_LT] = ACTIONS(163),
    [anon_sym_DOT_DOT] = ACTIONS(166),
    [anon_sym_COLON_COLON] = ACTIONS(169),
    [anon_sym_POUND] = ACTIONS(172),
    [anon_sym_SQUOTE] = ACTIONS(175),
    [anon_sym_async] = ACTIONS(178),
    [anon_sym_break] = ACTIONS(181),
    [anon_sym_const] = ACTIONS(184),
    [anon_sym_continue] = ACTIONS(187),
    [anon_sym_default] = ACTIONS(190),
    [anon_sym_enum] = ACTIONS(193),
    [anon_sym_fn] = ACTIONS(196),
    [anon_sym_for] = ACTIONS(199),
    [anon_sym_gen] = ACTIONS(202),
    [anon_sym_if] = ACTIONS(205),
    [anon_sym_impl] = ACTIONS(208),
    [anon_sym_let] = ACTIONS(211),
    [anon_sym_loop] = ACTIONS(214),
    [anon_sym_match] = ACTIONS(217),
    [anon_sym_mod] = ACTIONS(220),
    [anon_sym_pub] = ACTIONS(223),
    [anon_sym_return] = ACTIONS(226),
    [anon_sym_static] = ACTIONS(229),
    [anon_sym_struct] = ACTIONS(232),
    [anon_sym_trait] = ACTIONS(235),
    [anon_sym_type] = ACTIONS(238),
    [anon_sym_union] = ACTIONS(241),
    [anon_sym_unsafe] = ACTIONS(244),
    [anon_sym_use] = ACTIONS(247),
    [anon_sym_while] = ACTIONS(250),
    [anon_sym_extern] = ACTIONS(253),
    [anon_sym_yield] = ACTIONS(256),
    [anon_sym_move] = ACTIONS(259),
    [anon_sym_try] = ACTIONS(262),
    [sym_integer_literal] = ACTIONS(265),
    [aux_sym_string_literal_token1] = ACTIONS(268),
    [sym_char_literal] = ACTIONS(265),
    [anon_sym_true] = ACTIONS(271),
    [anon_sym_false] = ACTIONS(271),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(274),
    [sym_super] = ACTIONS(277),
    [sym_crate] = ACTIONS(280),
    [sym_metavariable] = ACTIONS(283),
    [sym__raw_string_literal_start] = ACTIONS(286),
    [sym_float_literal] = ACTIONS(265),
  },
  [13] = {
    [sym__statement] = STATE(659),
    [sym_empty_statement] = STATE(665),
    [sym_expression_statement] = STATE(665),
    [sym_macro_definition] = STATE(665),
    [sym_attribute_item] = STATE(665),
    [sym_inner_attribute_item] = STATE(665),
    [sym_mod_item] = STATE(665),
    [sym_foreign_mod_item] = STATE(665),
    [sym_struct_item] = STATE(665),
    [sym_union_item] = STATE(665),
    [sym_enum_item] = STATE(665),
    [sym_extern_crate_declaration] = STATE(665),
    [sym_const_item] = STATE(665),
    [sym_static_item] = STATE(665),
    [sym_type_item] = STATE(665),
    [sym_function_item] = STATE(665),
    [sym_function_signature_item] = STATE(665),
    [sym_function_modifiers] = STATE(3607),
    [sym_impl_item] = STATE(665),
    [sym_trait_item] = STATE(665),
    [sym_associated_type] = STATE(665),
    [sym_let_declaration] = STATE(665),
    [sym_use_declaration] = STATE(665),
    [sym_extern_modifier] = STATE(2160),
    [sym_visibility_modifier] = STATE(1950),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1814),
    [sym_macro_invocation] = STATE(399),
    [sym_scoped_identifier] = STATE(1552),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(377),
    [sym_match_expression] = STATE(377),
    [sym_while_expression] = STATE(377),
    [sym_loop_expression] = STATE(377),
    [sym_for_expression] = STATE(377),
    [sym_const_block] = STATE(377),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3577),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(377),
    [sym_async_block] = STATE(377),
    [sym_gen_block] = STATE(377),
    [sym_try_block] = STATE(377),
    [sym_block] = STATE(377),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(13),
    [sym_block_comment] = STATE(13),
    [aux_sym_source_file_repeat1] = STATE(14),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro_rules_BANG] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(19),
    [anon_sym_RBRACE] = ACTIONS(297),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(121),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(43),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(47),
    [anon_sym_enum] = ACTIONS(49),
    [anon_sym_fn] = ACTIONS(51),
    [anon_sym_for] = ACTIONS(53),
    [anon_sym_gen] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_impl] = ACTIONS(59),
    [anon_sym_let] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(63),
    [anon_sym_match] = ACTIONS(65),
    [anon_sym_mod] = ACTIONS(67),
    [anon_sym_pub] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(73),
    [anon_sym_struct] = ACTIONS(75),
    [anon_sym_trait] = ACTIONS(77),
    [anon_sym_type] = ACTIONS(79),
    [anon_sym_union] = ACTIONS(81),
    [anon_sym_unsafe] = ACTIONS(83),
    [anon_sym_use] = ACTIONS(85),
    [anon_sym_while] = ACTIONS(87),
    [anon_sym_extern] = ACTIONS(89),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(95),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(113),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [14] = {
    [sym__statement] = STATE(659),
    [sym_empty_statement] = STATE(665),
    [sym_expression_statement] = STATE(665),
    [sym_macro_definition] = STATE(665),
    [sym_attribute_item] = STATE(665),
    [sym_inner_attribute_item] = STATE(665),
    [sym_mod_item] = STATE(665),
    [sym_foreign_mod_item] = STATE(665),
    [sym_struct_item] = STATE(665),
    [sym_union_item] = STATE(665),
    [sym_enum_item] = STATE(665),
    [sym_extern_crate_declaration] = STATE(665),
    [sym_const_item] = STATE(665),
    [sym_static_item] = STATE(665),
    [sym_type_item] = STATE(665),
    [sym_function_item] = STATE(665),
    [sym_function_signature_item] = STATE(665),
    [sym_function_modifiers] = STATE(3607),
    [sym_impl_item] = STATE(665),
    [sym_trait_item] = STATE(665),
    [sym_associated_type] = STATE(665),
    [sym_let_declaration] = STATE(665),
    [sym_use_declaration] = STATE(665),
    [sym_extern_modifier] = STATE(2160),
    [sym_visibility_modifier] = STATE(1950),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1818),
    [sym_macro_invocation] = STATE(399),
    [sym_scoped_identifier] = STATE(1552),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(377),
    [sym_match_expression] = STATE(377),
    [sym_while_expression] = STATE(377),
    [sym_loop_expression] = STATE(377),
    [sym_for_expression] = STATE(377),
    [sym_const_block] = STATE(377),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3577),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(377),
    [sym_async_block] = STATE(377),
    [sym_gen_block] = STATE(377),
    [sym_try_block] = STATE(377),
    [sym_block] = STATE(377),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(14),
    [sym_block_comment] = STATE(14),
    [aux_sym_source_file_repeat1] = STATE(12),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro_rules_BANG] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(19),
    [anon_sym_RBRACE] = ACTIONS(299),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(121),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(43),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(47),
    [anon_sym_enum] = ACTIONS(49),
    [anon_sym_fn] = ACTIONS(51),
    [anon_sym_for] = ACTIONS(53),
    [anon_sym_gen] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_impl] = ACTIONS(59),
    [anon_sym_let] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(63),
    [anon_sym_match] = ACTIONS(65),
    [anon_sym_mod] = ACTIONS(67),
    [anon_sym_pub] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(73),
    [anon_sym_struct] = ACTIONS(75),
    [anon_sym_trait] = ACTIONS(77),
    [anon_sym_type] = ACTIONS(79),
    [anon_sym_union] = ACTIONS(81),
    [anon_sym_unsafe] = ACTIONS(83),
    [anon_sym_use] = ACTIONS(85),
    [anon_sym_while] = ACTIONS(87),
    [anon_sym_extern] = ACTIONS(89),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(95),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(113),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [15] = {
    [sym__statement] = STATE(659),
    [sym_empty_statement] = STATE(665),
    [sym_expression_statement] = STATE(665),
    [sym_macro_definition] = STATE(665),
    [sym_attribute_item] = STATE(665),
    [sym_inner_attribute_item] = STATE(665),
    [sym_mod_item] = STATE(665),
    [sym_foreign_mod_item] = STATE(665),
    [sym_struct_item] = STATE(665),
    [sym_union_item] = STATE(665),
    [sym_enum_item] = STATE(665),
    [sym_extern_crate_declaration] = STATE(665),
    [sym_const_item] = STATE(665),
    [sym_static_item] = STATE(665),
    [sym_type_item] = STATE(665),
    [sym_function_item] = STATE(665),
    [sym_function_signature_item] = STATE(665),
    [sym_function_modifiers] = STATE(3607),
    [sym_impl_item] = STATE(665),
    [sym_trait_item] = STATE(665),
    [sym_associated_type] = STATE(665),
    [sym_let_declaration] = STATE(665),
    [sym_use_declaration] = STATE(665),
    [sym_extern_modifier] = STATE(2160),
    [sym_visibility_modifier] = STATE(1950),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1823),
    [sym_macro_invocation] = STATE(399),
    [sym_scoped_identifier] = STATE(1552),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(377),
    [sym_match_expression] = STATE(377),
    [sym_while_expression] = STATE(377),
    [sym_loop_expression] = STATE(377),
    [sym_for_expression] = STATE(377),
    [sym_const_block] = STATE(377),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3577),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(377),
    [sym_async_block] = STATE(377),
    [sym_gen_block] = STATE(377),
    [sym_try_block] = STATE(377),
    [sym_block] = STATE(377),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(15),
    [sym_block_comment] = STATE(15),
    [aux_sym_source_file_repeat1] = STATE(16),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro_rules_BANG] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(19),
    [anon_sym_RBRACE] = ACTIONS(301),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(121),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(43),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(47),
    [anon_sym_enum] = ACTIONS(49),
    [anon_sym_fn] = ACTIONS(51),
    [anon_sym_for] = ACTIONS(53),
    [anon_sym_gen] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_impl] = ACTIONS(59),
    [anon_sym_let] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(63),
    [anon_sym_match] = ACTIONS(65),
    [anon_sym_mod] = ACTIONS(67),
    [anon_sym_pub] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(73),
    [anon_sym_struct] = ACTIONS(75),
    [anon_sym_trait] = ACTIONS(77),
    [anon_sym_type] = ACTIONS(79),
    [anon_sym_union] = ACTIONS(81),
    [anon_sym_unsafe] = ACTIONS(83),
    [anon_sym_use] = ACTIONS(85),
    [anon_sym_while] = ACTIONS(87),
    [anon_sym_extern] = ACTIONS(89),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(95),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(113),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [16] = {
    [sym__statement] = STATE(659),
    [sym_empty_statement] = STATE(665),
    [sym_expression_statement] = STATE(665),
    [sym_macro_definition] = STATE(665),
    [sym_attribute_item] = STATE(665),
    [sym_inner_attribute_item] = STATE(665),
    [sym_mod_item] = STATE(665),
    [sym_foreign_mod_item] = STATE(665),
    [sym_struct_item] = STATE(665),
    [sym_union_item] = STATE(665),
    [sym_enum_item] = STATE(665),
    [sym_extern_crate_declaration] = STATE(665),
    [sym_const_item] = STATE(665),
    [sym_static_item] = STATE(665),
    [sym_type_item] = STATE(665),
    [sym_function_item] = STATE(665),
    [sym_function_signature_item] = STATE(665),
    [sym_function_modifiers] = STATE(3607),
    [sym_impl_item] = STATE(665),
    [sym_trait_item] = STATE(665),
    [sym_associated_type] = STATE(665),
    [sym_let_declaration] = STATE(665),
    [sym_use_declaration] = STATE(665),
    [sym_extern_modifier] = STATE(2160),
    [sym_visibility_modifier] = STATE(1950),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1829),
    [sym_macro_invocation] = STATE(399),
    [sym_scoped_identifier] = STATE(1552),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(377),
    [sym_match_expression] = STATE(377),
    [sym_while_expression] = STATE(377),
    [sym_loop_expression] = STATE(377),
    [sym_for_expression] = STATE(377),
    [sym_const_block] = STATE(377),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3577),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(377),
    [sym_async_block] = STATE(377),
    [sym_gen_block] = STATE(377),
    [sym_try_block] = STATE(377),
    [sym_block] = STATE(377),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(16),
    [sym_block_comment] = STATE(16),
    [aux_sym_source_file_repeat1] = STATE(12),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro_rules_BANG] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(19),
    [anon_sym_RBRACE] = ACTIONS(303),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(121),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(43),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(47),
    [anon_sym_enum] = ACTIONS(49),
    [anon_sym_fn] = ACTIONS(51),
    [anon_sym_for] = ACTIONS(53),
    [anon_sym_gen] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_impl] = ACTIONS(59),
    [anon_sym_let] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(63),
    [anon_sym_match] = ACTIONS(65),
    [anon_sym_mod] = ACTIONS(67),
    [anon_sym_pub] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(73),
    [anon_sym_struct] = ACTIONS(75),
    [anon_sym_trait] = ACTIONS(77),
    [anon_sym_type] = ACTIONS(79),
    [anon_sym_union] = ACTIONS(81),
    [anon_sym_unsafe] = ACTIONS(83),
    [anon_sym_use] = ACTIONS(85),
    [anon_sym_while] = ACTIONS(87),
    [anon_sym_extern] = ACTIONS(89),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(95),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(113),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [17] = {
    [sym__statement] = STATE(659),
    [sym_empty_statement] = STATE(665),
    [sym_expression_statement] = STATE(665),
    [sym_macro_definition] = STATE(665),
    [sym_attribute_item] = STATE(665),
    [sym_inner_attribute_item] = STATE(665),
    [sym_mod_item] = STATE(665),
    [sym_foreign_mod_item] = STATE(665),
    [sym_struct_item] = STATE(665),
    [sym_union_item] = STATE(665),
    [sym_enum_item] = STATE(665),
    [sym_extern_crate_declaration] = STATE(665),
    [sym_const_item] = STATE(665),
    [sym_static_item] = STATE(665),
    [sym_type_item] = STATE(665),
    [sym_function_item] = STATE(665),
    [sym_function_signature_item] = STATE(665),
    [sym_function_modifiers] = STATE(3607),
    [sym_impl_item] = STATE(665),
    [sym_trait_item] = STATE(665),
    [sym_associated_type] = STATE(665),
    [sym_let_declaration] = STATE(665),
    [sym_use_declaration] = STATE(665),
    [sym_extern_modifier] = STATE(2160),
    [sym_visibility_modifier] = STATE(1950),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1833),
    [sym_macro_invocation] = STATE(399),
    [sym_scoped_identifier] = STATE(1552),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(377),
    [sym_match_expression] = STATE(377),
    [sym_while_expression] = STATE(377),
    [sym_loop_expression] = STATE(377),
    [sym_for_expression] = STATE(377),
    [sym_const_block] = STATE(377),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3577),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(377),
    [sym_async_block] = STATE(377),
    [sym_gen_block] = STATE(377),
    [sym_try_block] = STATE(377),
    [sym_block] = STATE(377),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(17),
    [sym_block_comment] = STATE(17),
    [aux_sym_source_file_repeat1] = STATE(18),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro_rules_BANG] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(19),
    [anon_sym_RBRACE] = ACTIONS(305),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(121),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(43),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(47),
    [anon_sym_enum] = ACTIONS(49),
    [anon_sym_fn] = ACTIONS(51),
    [anon_sym_for] = ACTIONS(53),
    [anon_sym_gen] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_impl] = ACTIONS(59),
    [anon_sym_let] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(63),
    [anon_sym_match] = ACTIONS(65),
    [anon_sym_mod] = ACTIONS(67),
    [anon_sym_pub] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(73),
    [anon_sym_struct] = ACTIONS(75),
    [anon_sym_trait] = ACTIONS(77),
    [anon_sym_type] = ACTIONS(79),
    [anon_sym_union] = ACTIONS(81),
    [anon_sym_unsafe] = ACTIONS(83),
    [anon_sym_use] = ACTIONS(85),
    [anon_sym_while] = ACTIONS(87),
    [anon_sym_extern] = ACTIONS(89),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(95),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(113),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [18] = {
    [sym__statement] = STATE(659),
    [sym_empty_statement] = STATE(665),
    [sym_expression_statement] = STATE(665),
    [sym_macro_definition] = STATE(665),
    [sym_attribute_item] = STATE(665),
    [sym_inner_attribute_item] = STATE(665),
    [sym_mod_item] = STATE(665),
    [sym_foreign_mod_item] = STATE(665),
    [sym_struct_item] = STATE(665),
    [sym_union_item] = STATE(665),
    [sym_enum_item] = STATE(665),
    [sym_extern_crate_declaration] = STATE(665),
    [sym_const_item] = STATE(665),
    [sym_static_item] = STATE(665),
    [sym_type_item] = STATE(665),
    [sym_function_item] = STATE(665),
    [sym_function_signature_item] = STATE(665),
    [sym_function_modifiers] = STATE(3607),
    [sym_impl_item] = STATE(665),
    [sym_trait_item] = STATE(665),
    [sym_associated_type] = STATE(665),
    [sym_let_declaration] = STATE(665),
    [sym_use_declaration] = STATE(665),
    [sym_extern_modifier] = STATE(2160),
    [sym_visibility_modifier] = STATE(1950),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1834),
    [sym_macro_invocation] = STATE(399),
    [sym_scoped_identifier] = STATE(1552),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(377),
    [sym_match_expression] = STATE(377),
    [sym_while_expression] = STATE(377),
    [sym_loop_expression] = STATE(377),
    [sym_for_expression] = STATE(377),
    [sym_const_block] = STATE(377),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3577),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(377),
    [sym_async_block] = STATE(377),
    [sym_gen_block] = STATE(377),
    [sym_try_block] = STATE(377),
    [sym_block] = STATE(377),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(18),
    [sym_block_comment] = STATE(18),
    [aux_sym_source_file_repeat1] = STATE(12),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro_rules_BANG] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(19),
    [anon_sym_RBRACE] = ACTIONS(307),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(121),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(43),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(47),
    [anon_sym_enum] = ACTIONS(49),
    [anon_sym_fn] = ACTIONS(51),
    [anon_sym_for] = ACTIONS(53),
    [anon_sym_gen] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_impl] = ACTIONS(59),
    [anon_sym_let] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(63),
    [anon_sym_match] = ACTIONS(65),
    [anon_sym_mod] = ACTIONS(67),
    [anon_sym_pub] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(73),
    [anon_sym_struct] = ACTIONS(75),
    [anon_sym_trait] = ACTIONS(77),
    [anon_sym_type] = ACTIONS(79),
    [anon_sym_union] = ACTIONS(81),
    [anon_sym_unsafe] = ACTIONS(83),
    [anon_sym_use] = ACTIONS(85),
    [anon_sym_while] = ACTIONS(87),
    [anon_sym_extern] = ACTIONS(89),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(95),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(113),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [19] = {
    [sym__statement] = STATE(659),
    [sym_empty_statement] = STATE(665),
    [sym_expression_statement] = STATE(665),
    [sym_macro_definition] = STATE(665),
    [sym_attribute_item] = STATE(665),
    [sym_inner_attribute_item] = STATE(665),
    [sym_mod_item] = STATE(665),
    [sym_foreign_mod_item] = STATE(665),
    [sym_struct_item] = STATE(665),
    [sym_union_item] = STATE(665),
    [sym_enum_item] = STATE(665),
    [sym_extern_crate_declaration] = STATE(665),
    [sym_const_item] = STATE(665),
    [sym_static_item] = STATE(665),
    [sym_type_item] = STATE(665),
    [sym_function_item] = STATE(665),
    [sym_function_signature_item] = STATE(665),
    [sym_function_modifiers] = STATE(3607),
    [sym_impl_item] = STATE(665),
    [sym_trait_item] = STATE(665),
    [sym_associated_type] = STATE(665),
    [sym_let_declaration] = STATE(665),
    [sym_use_declaration] = STATE(665),
    [sym_extern_modifier] = STATE(2160),
    [sym_visibility_modifier] = STATE(1950),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1837),
    [sym_macro_invocation] = STATE(399),
    [sym_scoped_identifier] = STATE(1552),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(377),
    [sym_match_expression] = STATE(377),
    [sym_while_expression] = STATE(377),
    [sym_loop_expression] = STATE(377),
    [sym_for_expression] = STATE(377),
    [sym_const_block] = STATE(377),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3577),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(377),
    [sym_async_block] = STATE(377),
    [sym_gen_block] = STATE(377),
    [sym_try_block] = STATE(377),
    [sym_block] = STATE(377),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(19),
    [sym_block_comment] = STATE(19),
    [aux_sym_source_file_repeat1] = STATE(20),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro_rules_BANG] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(19),
    [anon_sym_RBRACE] = ACTIONS(309),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(121),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(43),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(47),
    [anon_sym_enum] = ACTIONS(49),
    [anon_sym_fn] = ACTIONS(51),
    [anon_sym_for] = ACTIONS(53),
    [anon_sym_gen] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_impl] = ACTIONS(59),
    [anon_sym_let] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(63),
    [anon_sym_match] = ACTIONS(65),
    [anon_sym_mod] = ACTIONS(67),
    [anon_sym_pub] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(73),
    [anon_sym_struct] = ACTIONS(75),
    [anon_sym_trait] = ACTIONS(77),
    [anon_sym_type] = ACTIONS(79),
    [anon_sym_union] = ACTIONS(81),
    [anon_sym_unsafe] = ACTIONS(83),
    [anon_sym_use] = ACTIONS(85),
    [anon_sym_while] = ACTIONS(87),
    [anon_sym_extern] = ACTIONS(89),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(95),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(113),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [20] = {
    [sym__statement] = STATE(659),
    [sym_empty_statement] = STATE(665),
    [sym_expression_statement] = STATE(665),
    [sym_macro_definition] = STATE(665),
    [sym_attribute_item] = STATE(665),
    [sym_inner_attribute_item] = STATE(665),
    [sym_mod_item] = STATE(665),
    [sym_foreign_mod_item] = STATE(665),
    [sym_struct_item] = STATE(665),
    [sym_union_item] = STATE(665),
    [sym_enum_item] = STATE(665),
    [sym_extern_crate_declaration] = STATE(665),
    [sym_const_item] = STATE(665),
    [sym_static_item] = STATE(665),
    [sym_type_item] = STATE(665),
    [sym_function_item] = STATE(665),
    [sym_function_signature_item] = STATE(665),
    [sym_function_modifiers] = STATE(3607),
    [sym_impl_item] = STATE(665),
    [sym_trait_item] = STATE(665),
    [sym_associated_type] = STATE(665),
    [sym_let_declaration] = STATE(665),
    [sym_use_declaration] = STATE(665),
    [sym_extern_modifier] = STATE(2160),
    [sym_visibility_modifier] = STATE(1950),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1839),
    [sym_macro_invocation] = STATE(399),
    [sym_scoped_identifier] = STATE(1552),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(377),
    [sym_match_expression] = STATE(377),
    [sym_while_expression] = STATE(377),
    [sym_loop_expression] = STATE(377),
    [sym_for_expression] = STATE(377),
    [sym_const_block] = STATE(377),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3577),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(377),
    [sym_async_block] = STATE(377),
    [sym_gen_block] = STATE(377),
    [sym_try_block] = STATE(377),
    [sym_block] = STATE(377),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(20),
    [sym_block_comment] = STATE(20),
    [aux_sym_source_file_repeat1] = STATE(12),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro_rules_BANG] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(19),
    [anon_sym_RBRACE] = ACTIONS(311),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(121),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(43),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(47),
    [anon_sym_enum] = ACTIONS(49),
    [anon_sym_fn] = ACTIONS(51),
    [anon_sym_for] = ACTIONS(53),
    [anon_sym_gen] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_impl] = ACTIONS(59),
    [anon_sym_let] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(63),
    [anon_sym_match] = ACTIONS(65),
    [anon_sym_mod] = ACTIONS(67),
    [anon_sym_pub] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(73),
    [anon_sym_struct] = ACTIONS(75),
    [anon_sym_trait] = ACTIONS(77),
    [anon_sym_type] = ACTIONS(79),
    [anon_sym_union] = ACTIONS(81),
    [anon_sym_unsafe] = ACTIONS(83),
    [anon_sym_use] = ACTIONS(85),
    [anon_sym_while] = ACTIONS(87),
    [anon_sym_extern] = ACTIONS(89),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(95),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(113),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [21] = {
    [sym__statement] = STATE(659),
    [sym_empty_statement] = STATE(665),
    [sym_expression_statement] = STATE(665),
    [sym_macro_definition] = STATE(665),
    [sym_attribute_item] = STATE(665),
    [sym_inner_attribute_item] = STATE(665),
    [sym_mod_item] = STATE(665),
    [sym_foreign_mod_item] = STATE(665),
    [sym_struct_item] = STATE(665),
    [sym_union_item] = STATE(665),
    [sym_enum_item] = STATE(665),
    [sym_extern_crate_declaration] = STATE(665),
    [sym_const_item] = STATE(665),
    [sym_static_item] = STATE(665),
    [sym_type_item] = STATE(665),
    [sym_function_item] = STATE(665),
    [sym_function_signature_item] = STATE(665),
    [sym_function_modifiers] = STATE(3607),
    [sym_impl_item] = STATE(665),
    [sym_trait_item] = STATE(665),
    [sym_associated_type] = STATE(665),
    [sym_let_declaration] = STATE(665),
    [sym_use_declaration] = STATE(665),
    [sym_extern_modifier] = STATE(2160),
    [sym_visibility_modifier] = STATE(1950),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1840),
    [sym_macro_invocation] = STATE(399),
    [sym_scoped_identifier] = STATE(1552),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(377),
    [sym_match_expression] = STATE(377),
    [sym_while_expression] = STATE(377),
    [sym_loop_expression] = STATE(377),
    [sym_for_expression] = STATE(377),
    [sym_const_block] = STATE(377),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3577),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(377),
    [sym_async_block] = STATE(377),
    [sym_gen_block] = STATE(377),
    [sym_try_block] = STATE(377),
    [sym_block] = STATE(377),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(21),
    [sym_block_comment] = STATE(21),
    [aux_sym_source_file_repeat1] = STATE(22),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro_rules_BANG] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(19),
    [anon_sym_RBRACE] = ACTIONS(313),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(121),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(43),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(47),
    [anon_sym_enum] = ACTIONS(49),
    [anon_sym_fn] = ACTIONS(51),
    [anon_sym_for] = ACTIONS(53),
    [anon_sym_gen] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_impl] = ACTIONS(59),
    [anon_sym_let] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(63),
    [anon_sym_match] = ACTIONS(65),
    [anon_sym_mod] = ACTIONS(67),
    [anon_sym_pub] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(73),
    [anon_sym_struct] = ACTIONS(75),
    [anon_sym_trait] = ACTIONS(77),
    [anon_sym_type] = ACTIONS(79),
    [anon_sym_union] = ACTIONS(81),
    [anon_sym_unsafe] = ACTIONS(83),
    [anon_sym_use] = ACTIONS(85),
    [anon_sym_while] = ACTIONS(87),
    [anon_sym_extern] = ACTIONS(89),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(95),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(113),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [22] = {
    [sym__statement] = STATE(659),
    [sym_empty_statement] = STATE(665),
    [sym_expression_statement] = STATE(665),
    [sym_macro_definition] = STATE(665),
    [sym_attribute_item] = STATE(665),
    [sym_inner_attribute_item] = STATE(665),
    [sym_mod_item] = STATE(665),
    [sym_foreign_mod_item] = STATE(665),
    [sym_struct_item] = STATE(665),
    [sym_union_item] = STATE(665),
    [sym_enum_item] = STATE(665),
    [sym_extern_crate_declaration] = STATE(665),
    [sym_const_item] = STATE(665),
    [sym_static_item] = STATE(665),
    [sym_type_item] = STATE(665),
    [sym_function_item] = STATE(665),
    [sym_function_signature_item] = STATE(665),
    [sym_function_modifiers] = STATE(3607),
    [sym_impl_item] = STATE(665),
    [sym_trait_item] = STATE(665),
    [sym_associated_type] = STATE(665),
    [sym_let_declaration] = STATE(665),
    [sym_use_declaration] = STATE(665),
    [sym_extern_modifier] = STATE(2160),
    [sym_visibility_modifier] = STATE(1950),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1841),
    [sym_macro_invocation] = STATE(399),
    [sym_scoped_identifier] = STATE(1552),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(377),
    [sym_match_expression] = STATE(377),
    [sym_while_expression] = STATE(377),
    [sym_loop_expression] = STATE(377),
    [sym_for_expression] = STATE(377),
    [sym_const_block] = STATE(377),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3577),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(377),
    [sym_async_block] = STATE(377),
    [sym_gen_block] = STATE(377),
    [sym_try_block] = STATE(377),
    [sym_block] = STATE(377),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(22),
    [sym_block_comment] = STATE(22),
    [aux_sym_source_file_repeat1] = STATE(12),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro_rules_BANG] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(19),
    [anon_sym_RBRACE] = ACTIONS(315),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(121),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(43),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(47),
    [anon_sym_enum] = ACTIONS(49),
    [anon_sym_fn] = ACTIONS(51),
    [anon_sym_for] = ACTIONS(53),
    [anon_sym_gen] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_impl] = ACTIONS(59),
    [anon_sym_let] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(63),
    [anon_sym_match] = ACTIONS(65),
    [anon_sym_mod] = ACTIONS(67),
    [anon_sym_pub] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(73),
    [anon_sym_struct] = ACTIONS(75),
    [anon_sym_trait] = ACTIONS(77),
    [anon_sym_type] = ACTIONS(79),
    [anon_sym_union] = ACTIONS(81),
    [anon_sym_unsafe] = ACTIONS(83),
    [anon_sym_use] = ACTIONS(85),
    [anon_sym_while] = ACTIONS(87),
    [anon_sym_extern] = ACTIONS(89),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(95),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(113),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [23] = {
    [sym__statement] = STATE(659),
    [sym_empty_statement] = STATE(665),
    [sym_expression_statement] = STATE(665),
    [sym_macro_definition] = STATE(665),
    [sym_attribute_item] = STATE(665),
    [sym_inner_attribute_item] = STATE(665),
    [sym_mod_item] = STATE(665),
    [sym_foreign_mod_item] = STATE(665),
    [sym_struct_item] = STATE(665),
    [sym_union_item] = STATE(665),
    [sym_enum_item] = STATE(665),
    [sym_extern_crate_declaration] = STATE(665),
    [sym_const_item] = STATE(665),
    [sym_static_item] = STATE(665),
    [sym_type_item] = STATE(665),
    [sym_function_item] = STATE(665),
    [sym_function_signature_item] = STATE(665),
    [sym_function_modifiers] = STATE(3607),
    [sym_impl_item] = STATE(665),
    [sym_trait_item] = STATE(665),
    [sym_associated_type] = STATE(665),
    [sym_let_declaration] = STATE(665),
    [sym_use_declaration] = STATE(665),
    [sym_extern_modifier] = STATE(2160),
    [sym_visibility_modifier] = STATE(1950),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1842),
    [sym_macro_invocation] = STATE(399),
    [sym_scoped_identifier] = STATE(1552),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(377),
    [sym_match_expression] = STATE(377),
    [sym_while_expression] = STATE(377),
    [sym_loop_expression] = STATE(377),
    [sym_for_expression] = STATE(377),
    [sym_const_block] = STATE(377),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3577),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(377),
    [sym_async_block] = STATE(377),
    [sym_gen_block] = STATE(377),
    [sym_try_block] = STATE(377),
    [sym_block] = STATE(377),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(23),
    [sym_block_comment] = STATE(23),
    [aux_sym_source_file_repeat1] = STATE(24),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro_rules_BANG] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(19),
    [anon_sym_RBRACE] = ACTIONS(317),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(121),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(43),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(47),
    [anon_sym_enum] = ACTIONS(49),
    [anon_sym_fn] = ACTIONS(51),
    [anon_sym_for] = ACTIONS(53),
    [anon_sym_gen] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_impl] = ACTIONS(59),
    [anon_sym_let] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(63),
    [anon_sym_match] = ACTIONS(65),
    [anon_sym_mod] = ACTIONS(67),
    [anon_sym_pub] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(73),
    [anon_sym_struct] = ACTIONS(75),
    [anon_sym_trait] = ACTIONS(77),
    [anon_sym_type] = ACTIONS(79),
    [anon_sym_union] = ACTIONS(81),
    [anon_sym_unsafe] = ACTIONS(83),
    [anon_sym_use] = ACTIONS(85),
    [anon_sym_while] = ACTIONS(87),
    [anon_sym_extern] = ACTIONS(89),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(95),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(113),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [24] = {
    [sym__statement] = STATE(659),
    [sym_empty_statement] = STATE(665),
    [sym_expression_statement] = STATE(665),
    [sym_macro_definition] = STATE(665),
    [sym_attribute_item] = STATE(665),
    [sym_inner_attribute_item] = STATE(665),
    [sym_mod_item] = STATE(665),
    [sym_foreign_mod_item] = STATE(665),
    [sym_struct_item] = STATE(665),
    [sym_union_item] = STATE(665),
    [sym_enum_item] = STATE(665),
    [sym_extern_crate_declaration] = STATE(665),
    [sym_const_item] = STATE(665),
    [sym_static_item] = STATE(665),
    [sym_type_item] = STATE(665),
    [sym_function_item] = STATE(665),
    [sym_function_signature_item] = STATE(665),
    [sym_function_modifiers] = STATE(3607),
    [sym_impl_item] = STATE(665),
    [sym_trait_item] = STATE(665),
    [sym_associated_type] = STATE(665),
    [sym_let_declaration] = STATE(665),
    [sym_use_declaration] = STATE(665),
    [sym_extern_modifier] = STATE(2160),
    [sym_visibility_modifier] = STATE(1950),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1843),
    [sym_macro_invocation] = STATE(399),
    [sym_scoped_identifier] = STATE(1552),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(377),
    [sym_match_expression] = STATE(377),
    [sym_while_expression] = STATE(377),
    [sym_loop_expression] = STATE(377),
    [sym_for_expression] = STATE(377),
    [sym_const_block] = STATE(377),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3577),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(377),
    [sym_async_block] = STATE(377),
    [sym_gen_block] = STATE(377),
    [sym_try_block] = STATE(377),
    [sym_block] = STATE(377),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(24),
    [sym_block_comment] = STATE(24),
    [aux_sym_source_file_repeat1] = STATE(12),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro_rules_BANG] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(19),
    [anon_sym_RBRACE] = ACTIONS(319),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(121),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(43),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(47),
    [anon_sym_enum] = ACTIONS(49),
    [anon_sym_fn] = ACTIONS(51),
    [anon_sym_for] = ACTIONS(53),
    [anon_sym_gen] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_impl] = ACTIONS(59),
    [anon_sym_let] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(63),
    [anon_sym_match] = ACTIONS(65),
    [anon_sym_mod] = ACTIONS(67),
    [anon_sym_pub] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(73),
    [anon_sym_struct] = ACTIONS(75),
    [anon_sym_trait] = ACTIONS(77),
    [anon_sym_type] = ACTIONS(79),
    [anon_sym_union] = ACTIONS(81),
    [anon_sym_unsafe] = ACTIONS(83),
    [anon_sym_use] = ACTIONS(85),
    [anon_sym_while] = ACTIONS(87),
    [anon_sym_extern] = ACTIONS(89),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(95),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(113),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [25] = {
    [sym__statement] = STATE(659),
    [sym_empty_statement] = STATE(665),
    [sym_expression_statement] = STATE(665),
    [sym_macro_definition] = STATE(665),
    [sym_attribute_item] = STATE(665),
    [sym_inner_attribute_item] = STATE(665),
    [sym_mod_item] = STATE(665),
    [sym_foreign_mod_item] = STATE(665),
    [sym_struct_item] = STATE(665),
    [sym_union_item] = STATE(665),
    [sym_enum_item] = STATE(665),
    [sym_extern_crate_declaration] = STATE(665),
    [sym_const_item] = STATE(665),
    [sym_static_item] = STATE(665),
    [sym_type_item] = STATE(665),
    [sym_function_item] = STATE(665),
    [sym_function_signature_item] = STATE(665),
    [sym_function_modifiers] = STATE(3607),
    [sym_impl_item] = STATE(665),
    [sym_trait_item] = STATE(665),
    [sym_associated_type] = STATE(665),
    [sym_let_declaration] = STATE(665),
    [sym_use_declaration] = STATE(665),
    [sym_extern_modifier] = STATE(2160),
    [sym_visibility_modifier] = STATE(1950),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1845),
    [sym_macro_invocation] = STATE(399),
    [sym_scoped_identifier] = STATE(1552),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(377),
    [sym_match_expression] = STATE(377),
    [sym_while_expression] = STATE(377),
    [sym_loop_expression] = STATE(377),
    [sym_for_expression] = STATE(377),
    [sym_const_block] = STATE(377),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3577),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(377),
    [sym_async_block] = STATE(377),
    [sym_gen_block] = STATE(377),
    [sym_try_block] = STATE(377),
    [sym_block] = STATE(377),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(25),
    [sym_block_comment] = STATE(25),
    [aux_sym_source_file_repeat1] = STATE(26),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro_rules_BANG] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(19),
    [anon_sym_RBRACE] = ACTIONS(321),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(121),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(43),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(47),
    [anon_sym_enum] = ACTIONS(49),
    [anon_sym_fn] = ACTIONS(51),
    [anon_sym_for] = ACTIONS(53),
    [anon_sym_gen] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_impl] = ACTIONS(59),
    [anon_sym_let] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(63),
    [anon_sym_match] = ACTIONS(65),
    [anon_sym_mod] = ACTIONS(67),
    [anon_sym_pub] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(73),
    [anon_sym_struct] = ACTIONS(75),
    [anon_sym_trait] = ACTIONS(77),
    [anon_sym_type] = ACTIONS(79),
    [anon_sym_union] = ACTIONS(81),
    [anon_sym_unsafe] = ACTIONS(83),
    [anon_sym_use] = ACTIONS(85),
    [anon_sym_while] = ACTIONS(87),
    [anon_sym_extern] = ACTIONS(89),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(95),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(113),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [26] = {
    [sym__statement] = STATE(659),
    [sym_empty_statement] = STATE(665),
    [sym_expression_statement] = STATE(665),
    [sym_macro_definition] = STATE(665),
    [sym_attribute_item] = STATE(665),
    [sym_inner_attribute_item] = STATE(665),
    [sym_mod_item] = STATE(665),
    [sym_foreign_mod_item] = STATE(665),
    [sym_struct_item] = STATE(665),
    [sym_union_item] = STATE(665),
    [sym_enum_item] = STATE(665),
    [sym_extern_crate_declaration] = STATE(665),
    [sym_const_item] = STATE(665),
    [sym_static_item] = STATE(665),
    [sym_type_item] = STATE(665),
    [sym_function_item] = STATE(665),
    [sym_function_signature_item] = STATE(665),
    [sym_function_modifiers] = STATE(3607),
    [sym_impl_item] = STATE(665),
    [sym_trait_item] = STATE(665),
    [sym_associated_type] = STATE(665),
    [sym_let_declaration] = STATE(665),
    [sym_use_declaration] = STATE(665),
    [sym_extern_modifier] = STATE(2160),
    [sym_visibility_modifier] = STATE(1950),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1846),
    [sym_macro_invocation] = STATE(399),
    [sym_scoped_identifier] = STATE(1552),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(377),
    [sym_match_expression] = STATE(377),
    [sym_while_expression] = STATE(377),
    [sym_loop_expression] = STATE(377),
    [sym_for_expression] = STATE(377),
    [sym_const_block] = STATE(377),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3577),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(377),
    [sym_async_block] = STATE(377),
    [sym_gen_block] = STATE(377),
    [sym_try_block] = STATE(377),
    [sym_block] = STATE(377),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(26),
    [sym_block_comment] = STATE(26),
    [aux_sym_source_file_repeat1] = STATE(12),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro_rules_BANG] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(19),
    [anon_sym_RBRACE] = ACTIONS(323),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(121),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(43),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(47),
    [anon_sym_enum] = ACTIONS(49),
    [anon_sym_fn] = ACTIONS(51),
    [anon_sym_for] = ACTIONS(53),
    [anon_sym_gen] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_impl] = ACTIONS(59),
    [anon_sym_let] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(63),
    [anon_sym_match] = ACTIONS(65),
    [anon_sym_mod] = ACTIONS(67),
    [anon_sym_pub] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(73),
    [anon_sym_struct] = ACTIONS(75),
    [anon_sym_trait] = ACTIONS(77),
    [anon_sym_type] = ACTIONS(79),
    [anon_sym_union] = ACTIONS(81),
    [anon_sym_unsafe] = ACTIONS(83),
    [anon_sym_use] = ACTIONS(85),
    [anon_sym_while] = ACTIONS(87),
    [anon_sym_extern] = ACTIONS(89),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(95),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(113),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [27] = {
    [sym__statement] = STATE(659),
    [sym_empty_statement] = STATE(665),
    [sym_expression_statement] = STATE(665),
    [sym_macro_definition] = STATE(665),
    [sym_attribute_item] = STATE(665),
    [sym_inner_attribute_item] = STATE(665),
    [sym_mod_item] = STATE(665),
    [sym_foreign_mod_item] = STATE(665),
    [sym_struct_item] = STATE(665),
    [sym_union_item] = STATE(665),
    [sym_enum_item] = STATE(665),
    [sym_extern_crate_declaration] = STATE(665),
    [sym_const_item] = STATE(665),
    [sym_static_item] = STATE(665),
    [sym_type_item] = STATE(665),
    [sym_function_item] = STATE(665),
    [sym_function_signature_item] = STATE(665),
    [sym_function_modifiers] = STATE(3607),
    [sym_impl_item] = STATE(665),
    [sym_trait_item] = STATE(665),
    [sym_associated_type] = STATE(665),
    [sym_let_declaration] = STATE(665),
    [sym_use_declaration] = STATE(665),
    [sym_extern_modifier] = STATE(2160),
    [sym_visibility_modifier] = STATE(1950),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1847),
    [sym_macro_invocation] = STATE(399),
    [sym_scoped_identifier] = STATE(1552),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(377),
    [sym_match_expression] = STATE(377),
    [sym_while_expression] = STATE(377),
    [sym_loop_expression] = STATE(377),
    [sym_for_expression] = STATE(377),
    [sym_const_block] = STATE(377),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3577),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(377),
    [sym_async_block] = STATE(377),
    [sym_gen_block] = STATE(377),
    [sym_try_block] = STATE(377),
    [sym_block] = STATE(377),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(27),
    [sym_block_comment] = STATE(27),
    [aux_sym_source_file_repeat1] = STATE(28),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro_rules_BANG] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(19),
    [anon_sym_RBRACE] = ACTIONS(325),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(121),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(43),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(47),
    [anon_sym_enum] = ACTIONS(49),
    [anon_sym_fn] = ACTIONS(51),
    [anon_sym_for] = ACTIONS(53),
    [anon_sym_gen] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_impl] = ACTIONS(59),
    [anon_sym_let] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(63),
    [anon_sym_match] = ACTIONS(65),
    [anon_sym_mod] = ACTIONS(67),
    [anon_sym_pub] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(73),
    [anon_sym_struct] = ACTIONS(75),
    [anon_sym_trait] = ACTIONS(77),
    [anon_sym_type] = ACTIONS(79),
    [anon_sym_union] = ACTIONS(81),
    [anon_sym_unsafe] = ACTIONS(83),
    [anon_sym_use] = ACTIONS(85),
    [anon_sym_while] = ACTIONS(87),
    [anon_sym_extern] = ACTIONS(89),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(95),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(113),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [28] = {
    [sym__statement] = STATE(659),
    [sym_empty_statement] = STATE(665),
    [sym_expression_statement] = STATE(665),
    [sym_macro_definition] = STATE(665),
    [sym_attribute_item] = STATE(665),
    [sym_inner_attribute_item] = STATE(665),
    [sym_mod_item] = STATE(665),
    [sym_foreign_mod_item] = STATE(665),
    [sym_struct_item] = STATE(665),
    [sym_union_item] = STATE(665),
    [sym_enum_item] = STATE(665),
    [sym_extern_crate_declaration] = STATE(665),
    [sym_const_item] = STATE(665),
    [sym_static_item] = STATE(665),
    [sym_type_item] = STATE(665),
    [sym_function_item] = STATE(665),
    [sym_function_signature_item] = STATE(665),
    [sym_function_modifiers] = STATE(3607),
    [sym_impl_item] = STATE(665),
    [sym_trait_item] = STATE(665),
    [sym_associated_type] = STATE(665),
    [sym_let_declaration] = STATE(665),
    [sym_use_declaration] = STATE(665),
    [sym_extern_modifier] = STATE(2160),
    [sym_visibility_modifier] = STATE(1950),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1848),
    [sym_macro_invocation] = STATE(399),
    [sym_scoped_identifier] = STATE(1552),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(377),
    [sym_match_expression] = STATE(377),
    [sym_while_expression] = STATE(377),
    [sym_loop_expression] = STATE(377),
    [sym_for_expression] = STATE(377),
    [sym_const_block] = STATE(377),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3577),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(377),
    [sym_async_block] = STATE(377),
    [sym_gen_block] = STATE(377),
    [sym_try_block] = STATE(377),
    [sym_block] = STATE(377),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(28),
    [sym_block_comment] = STATE(28),
    [aux_sym_source_file_repeat1] = STATE(12),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro_rules_BANG] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(19),
    [anon_sym_RBRACE] = ACTIONS(327),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(121),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(43),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(47),
    [anon_sym_enum] = ACTIONS(49),
    [anon_sym_fn] = ACTIONS(51),
    [anon_sym_for] = ACTIONS(53),
    [anon_sym_gen] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_impl] = ACTIONS(59),
    [anon_sym_let] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(63),
    [anon_sym_match] = ACTIONS(65),
    [anon_sym_mod] = ACTIONS(67),
    [anon_sym_pub] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(73),
    [anon_sym_struct] = ACTIONS(75),
    [anon_sym_trait] = ACTIONS(77),
    [anon_sym_type] = ACTIONS(79),
    [anon_sym_union] = ACTIONS(81),
    [anon_sym_unsafe] = ACTIONS(83),
    [anon_sym_use] = ACTIONS(85),
    [anon_sym_while] = ACTIONS(87),
    [anon_sym_extern] = ACTIONS(89),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(95),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(113),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [29] = {
    [sym__statement] = STATE(659),
    [sym_empty_statement] = STATE(665),
    [sym_expression_statement] = STATE(665),
    [sym_macro_definition] = STATE(665),
    [sym_attribute_item] = STATE(665),
    [sym_inner_attribute_item] = STATE(665),
    [sym_mod_item] = STATE(665),
    [sym_foreign_mod_item] = STATE(665),
    [sym_struct_item] = STATE(665),
    [sym_union_item] = STATE(665),
    [sym_enum_item] = STATE(665),
    [sym_extern_crate_declaration] = STATE(665),
    [sym_const_item] = STATE(665),
    [sym_static_item] = STATE(665),
    [sym_type_item] = STATE(665),
    [sym_function_item] = STATE(665),
    [sym_function_signature_item] = STATE(665),
    [sym_function_modifiers] = STATE(3607),
    [sym_impl_item] = STATE(665),
    [sym_trait_item] = STATE(665),
    [sym_associated_type] = STATE(665),
    [sym_let_declaration] = STATE(665),
    [sym_use_declaration] = STATE(665),
    [sym_extern_modifier] = STATE(2160),
    [sym_visibility_modifier] = STATE(1950),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1849),
    [sym_macro_invocation] = STATE(399),
    [sym_scoped_identifier] = STATE(1552),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(377),
    [sym_match_expression] = STATE(377),
    [sym_while_expression] = STATE(377),
    [sym_loop_expression] = STATE(377),
    [sym_for_expression] = STATE(377),
    [sym_const_block] = STATE(377),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3577),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(377),
    [sym_async_block] = STATE(377),
    [sym_gen_block] = STATE(377),
    [sym_try_block] = STATE(377),
    [sym_block] = STATE(377),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(29),
    [sym_block_comment] = STATE(29),
    [aux_sym_source_file_repeat1] = STATE(30),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro_rules_BANG] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(19),
    [anon_sym_RBRACE] = ACTIONS(329),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(121),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(43),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(47),
    [anon_sym_enum] = ACTIONS(49),
    [anon_sym_fn] = ACTIONS(51),
    [anon_sym_for] = ACTIONS(53),
    [anon_sym_gen] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_impl] = ACTIONS(59),
    [anon_sym_let] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(63),
    [anon_sym_match] = ACTIONS(65),
    [anon_sym_mod] = ACTIONS(67),
    [anon_sym_pub] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(73),
    [anon_sym_struct] = ACTIONS(75),
    [anon_sym_trait] = ACTIONS(77),
    [anon_sym_type] = ACTIONS(79),
    [anon_sym_union] = ACTIONS(81),
    [anon_sym_unsafe] = ACTIONS(83),
    [anon_sym_use] = ACTIONS(85),
    [anon_sym_while] = ACTIONS(87),
    [anon_sym_extern] = ACTIONS(89),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(95),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(113),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [30] = {
    [sym__statement] = STATE(659),
    [sym_empty_statement] = STATE(665),
    [sym_expression_statement] = STATE(665),
    [sym_macro_definition] = STATE(665),
    [sym_attribute_item] = STATE(665),
    [sym_inner_attribute_item] = STATE(665),
    [sym_mod_item] = STATE(665),
    [sym_foreign_mod_item] = STATE(665),
    [sym_struct_item] = STATE(665),
    [sym_union_item] = STATE(665),
    [sym_enum_item] = STATE(665),
    [sym_extern_crate_declaration] = STATE(665),
    [sym_const_item] = STATE(665),
    [sym_static_item] = STATE(665),
    [sym_type_item] = STATE(665),
    [sym_function_item] = STATE(665),
    [sym_function_signature_item] = STATE(665),
    [sym_function_modifiers] = STATE(3607),
    [sym_impl_item] = STATE(665),
    [sym_trait_item] = STATE(665),
    [sym_associated_type] = STATE(665),
    [sym_let_declaration] = STATE(665),
    [sym_use_declaration] = STATE(665),
    [sym_extern_modifier] = STATE(2160),
    [sym_visibility_modifier] = STATE(1950),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1850),
    [sym_macro_invocation] = STATE(399),
    [sym_scoped_identifier] = STATE(1552),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(377),
    [sym_match_expression] = STATE(377),
    [sym_while_expression] = STATE(377),
    [sym_loop_expression] = STATE(377),
    [sym_for_expression] = STATE(377),
    [sym_const_block] = STATE(377),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3577),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(377),
    [sym_async_block] = STATE(377),
    [sym_gen_block] = STATE(377),
    [sym_try_block] = STATE(377),
    [sym_block] = STATE(377),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(30),
    [sym_block_comment] = STATE(30),
    [aux_sym_source_file_repeat1] = STATE(12),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro_rules_BANG] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(19),
    [anon_sym_RBRACE] = ACTIONS(331),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(121),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(43),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(47),
    [anon_sym_enum] = ACTIONS(49),
    [anon_sym_fn] = ACTIONS(51),
    [anon_sym_for] = ACTIONS(53),
    [anon_sym_gen] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_impl] = ACTIONS(59),
    [anon_sym_let] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(63),
    [anon_sym_match] = ACTIONS(65),
    [anon_sym_mod] = ACTIONS(67),
    [anon_sym_pub] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(73),
    [anon_sym_struct] = ACTIONS(75),
    [anon_sym_trait] = ACTIONS(77),
    [anon_sym_type] = ACTIONS(79),
    [anon_sym_union] = ACTIONS(81),
    [anon_sym_unsafe] = ACTIONS(83),
    [anon_sym_use] = ACTIONS(85),
    [anon_sym_while] = ACTIONS(87),
    [anon_sym_extern] = ACTIONS(89),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(95),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(113),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [31] = {
    [sym__statement] = STATE(659),
    [sym_empty_statement] = STATE(665),
    [sym_expression_statement] = STATE(665),
    [sym_macro_definition] = STATE(665),
    [sym_attribute_item] = STATE(665),
    [sym_inner_attribute_item] = STATE(665),
    [sym_mod_item] = STATE(665),
    [sym_foreign_mod_item] = STATE(665),
    [sym_struct_item] = STATE(665),
    [sym_union_item] = STATE(665),
    [sym_enum_item] = STATE(665),
    [sym_extern_crate_declaration] = STATE(665),
    [sym_const_item] = STATE(665),
    [sym_static_item] = STATE(665),
    [sym_type_item] = STATE(665),
    [sym_function_item] = STATE(665),
    [sym_function_signature_item] = STATE(665),
    [sym_function_modifiers] = STATE(3607),
    [sym_impl_item] = STATE(665),
    [sym_trait_item] = STATE(665),
    [sym_associated_type] = STATE(665),
    [sym_let_declaration] = STATE(665),
    [sym_use_declaration] = STATE(665),
    [sym_extern_modifier] = STATE(2160),
    [sym_visibility_modifier] = STATE(1950),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1851),
    [sym_macro_invocation] = STATE(399),
    [sym_scoped_identifier] = STATE(1552),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(377),
    [sym_match_expression] = STATE(377),
    [sym_while_expression] = STATE(377),
    [sym_loop_expression] = STATE(377),
    [sym_for_expression] = STATE(377),
    [sym_const_block] = STATE(377),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3577),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(377),
    [sym_async_block] = STATE(377),
    [sym_gen_block] = STATE(377),
    [sym_try_block] = STATE(377),
    [sym_block] = STATE(377),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(31),
    [sym_block_comment] = STATE(31),
    [aux_sym_source_file_repeat1] = STATE(32),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro_rules_BANG] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(19),
    [anon_sym_RBRACE] = ACTIONS(333),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(121),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(43),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(47),
    [anon_sym_enum] = ACTIONS(49),
    [anon_sym_fn] = ACTIONS(51),
    [anon_sym_for] = ACTIONS(53),
    [anon_sym_gen] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_impl] = ACTIONS(59),
    [anon_sym_let] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(63),
    [anon_sym_match] = ACTIONS(65),
    [anon_sym_mod] = ACTIONS(67),
    [anon_sym_pub] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(73),
    [anon_sym_struct] = ACTIONS(75),
    [anon_sym_trait] = ACTIONS(77),
    [anon_sym_type] = ACTIONS(79),
    [anon_sym_union] = ACTIONS(81),
    [anon_sym_unsafe] = ACTIONS(83),
    [anon_sym_use] = ACTIONS(85),
    [anon_sym_while] = ACTIONS(87),
    [anon_sym_extern] = ACTIONS(89),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(95),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(113),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [32] = {
    [sym__statement] = STATE(659),
    [sym_empty_statement] = STATE(665),
    [sym_expression_statement] = STATE(665),
    [sym_macro_definition] = STATE(665),
    [sym_attribute_item] = STATE(665),
    [sym_inner_attribute_item] = STATE(665),
    [sym_mod_item] = STATE(665),
    [sym_foreign_mod_item] = STATE(665),
    [sym_struct_item] = STATE(665),
    [sym_union_item] = STATE(665),
    [sym_enum_item] = STATE(665),
    [sym_extern_crate_declaration] = STATE(665),
    [sym_const_item] = STATE(665),
    [sym_static_item] = STATE(665),
    [sym_type_item] = STATE(665),
    [sym_function_item] = STATE(665),
    [sym_function_signature_item] = STATE(665),
    [sym_function_modifiers] = STATE(3607),
    [sym_impl_item] = STATE(665),
    [sym_trait_item] = STATE(665),
    [sym_associated_type] = STATE(665),
    [sym_let_declaration] = STATE(665),
    [sym_use_declaration] = STATE(665),
    [sym_extern_modifier] = STATE(2160),
    [sym_visibility_modifier] = STATE(1950),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1852),
    [sym_macro_invocation] = STATE(399),
    [sym_scoped_identifier] = STATE(1552),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(377),
    [sym_match_expression] = STATE(377),
    [sym_while_expression] = STATE(377),
    [sym_loop_expression] = STATE(377),
    [sym_for_expression] = STATE(377),
    [sym_const_block] = STATE(377),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3577),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(377),
    [sym_async_block] = STATE(377),
    [sym_gen_block] = STATE(377),
    [sym_try_block] = STATE(377),
    [sym_block] = STATE(377),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(32),
    [sym_block_comment] = STATE(32),
    [aux_sym_source_file_repeat1] = STATE(12),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro_rules_BANG] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(19),
    [anon_sym_RBRACE] = ACTIONS(335),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(121),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(43),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(47),
    [anon_sym_enum] = ACTIONS(49),
    [anon_sym_fn] = ACTIONS(51),
    [anon_sym_for] = ACTIONS(53),
    [anon_sym_gen] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_impl] = ACTIONS(59),
    [anon_sym_let] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(63),
    [anon_sym_match] = ACTIONS(65),
    [anon_sym_mod] = ACTIONS(67),
    [anon_sym_pub] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(73),
    [anon_sym_struct] = ACTIONS(75),
    [anon_sym_trait] = ACTIONS(77),
    [anon_sym_type] = ACTIONS(79),
    [anon_sym_union] = ACTIONS(81),
    [anon_sym_unsafe] = ACTIONS(83),
    [anon_sym_use] = ACTIONS(85),
    [anon_sym_while] = ACTIONS(87),
    [anon_sym_extern] = ACTIONS(89),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(95),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(113),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [33] = {
    [sym__statement] = STATE(659),
    [sym_empty_statement] = STATE(665),
    [sym_expression_statement] = STATE(665),
    [sym_macro_definition] = STATE(665),
    [sym_attribute_item] = STATE(665),
    [sym_inner_attribute_item] = STATE(665),
    [sym_mod_item] = STATE(665),
    [sym_foreign_mod_item] = STATE(665),
    [sym_struct_item] = STATE(665),
    [sym_union_item] = STATE(665),
    [sym_enum_item] = STATE(665),
    [sym_extern_crate_declaration] = STATE(665),
    [sym_const_item] = STATE(665),
    [sym_static_item] = STATE(665),
    [sym_type_item] = STATE(665),
    [sym_function_item] = STATE(665),
    [sym_function_signature_item] = STATE(665),
    [sym_function_modifiers] = STATE(3607),
    [sym_impl_item] = STATE(665),
    [sym_trait_item] = STATE(665),
    [sym_associated_type] = STATE(665),
    [sym_let_declaration] = STATE(665),
    [sym_use_declaration] = STATE(665),
    [sym_extern_modifier] = STATE(2160),
    [sym_visibility_modifier] = STATE(1950),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1853),
    [sym_macro_invocation] = STATE(399),
    [sym_scoped_identifier] = STATE(1552),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(377),
    [sym_match_expression] = STATE(377),
    [sym_while_expression] = STATE(377),
    [sym_loop_expression] = STATE(377),
    [sym_for_expression] = STATE(377),
    [sym_const_block] = STATE(377),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3577),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(377),
    [sym_async_block] = STATE(377),
    [sym_gen_block] = STATE(377),
    [sym_try_block] = STATE(377),
    [sym_block] = STATE(377),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(33),
    [sym_block_comment] = STATE(33),
    [aux_sym_source_file_repeat1] = STATE(3),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro_rules_BANG] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(19),
    [anon_sym_RBRACE] = ACTIONS(337),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(121),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(43),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(47),
    [anon_sym_enum] = ACTIONS(49),
    [anon_sym_fn] = ACTIONS(51),
    [anon_sym_for] = ACTIONS(53),
    [anon_sym_gen] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_impl] = ACTIONS(59),
    [anon_sym_let] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(63),
    [anon_sym_match] = ACTIONS(65),
    [anon_sym_mod] = ACTIONS(67),
    [anon_sym_pub] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(73),
    [anon_sym_struct] = ACTIONS(75),
    [anon_sym_trait] = ACTIONS(77),
    [anon_sym_type] = ACTIONS(79),
    [anon_sym_union] = ACTIONS(81),
    [anon_sym_unsafe] = ACTIONS(83),
    [anon_sym_use] = ACTIONS(85),
    [anon_sym_while] = ACTIONS(87),
    [anon_sym_extern] = ACTIONS(89),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(95),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(113),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [34] = {
    [sym__statement] = STATE(659),
    [sym_empty_statement] = STATE(665),
    [sym_expression_statement] = STATE(665),
    [sym_macro_definition] = STATE(665),
    [sym_attribute_item] = STATE(665),
    [sym_inner_attribute_item] = STATE(665),
    [sym_mod_item] = STATE(665),
    [sym_foreign_mod_item] = STATE(665),
    [sym_struct_item] = STATE(665),
    [sym_union_item] = STATE(665),
    [sym_enum_item] = STATE(665),
    [sym_extern_crate_declaration] = STATE(665),
    [sym_const_item] = STATE(665),
    [sym_static_item] = STATE(665),
    [sym_type_item] = STATE(665),
    [sym_function_item] = STATE(665),
    [sym_function_signature_item] = STATE(665),
    [sym_function_modifiers] = STATE(3607),
    [sym_impl_item] = STATE(665),
    [sym_trait_item] = STATE(665),
    [sym_associated_type] = STATE(665),
    [sym_let_declaration] = STATE(665),
    [sym_use_declaration] = STATE(665),
    [sym_extern_modifier] = STATE(2160),
    [sym_visibility_modifier] = STATE(1950),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1886),
    [sym_macro_invocation] = STATE(399),
    [sym_scoped_identifier] = STATE(1552),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(377),
    [sym_match_expression] = STATE(377),
    [sym_while_expression] = STATE(377),
    [sym_loop_expression] = STATE(377),
    [sym_for_expression] = STATE(377),
    [sym_const_block] = STATE(377),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3577),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(377),
    [sym_async_block] = STATE(377),
    [sym_gen_block] = STATE(377),
    [sym_try_block] = STATE(377),
    [sym_block] = STATE(377),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(34),
    [sym_block_comment] = STATE(34),
    [aux_sym_source_file_repeat1] = STATE(7),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [ts_builtin_sym_end] = ACTIONS(125),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro_rules_BANG] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(19),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(121),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(43),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(47),
    [anon_sym_enum] = ACTIONS(49),
    [anon_sym_fn] = ACTIONS(51),
    [anon_sym_for] = ACTIONS(53),
    [anon_sym_gen] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_impl] = ACTIONS(59),
    [anon_sym_let] = ACTIONS(61),
    [anon_sym_loop] = ACTIONS(63),
    [anon_sym_match] = ACTIONS(65),
    [anon_sym_mod] = ACTIONS(67),
    [anon_sym_pub] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(73),
    [anon_sym_struct] = ACTIONS(75),
    [anon_sym_trait] = ACTIONS(77),
    [anon_sym_type] = ACTIONS(79),
    [anon_sym_union] = ACTIONS(81),
    [anon_sym_unsafe] = ACTIONS(83),
    [anon_sym_use] = ACTIONS(85),
    [anon_sym_while] = ACTIONS(87),
    [anon_sym_extern] = ACTIONS(89),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(95),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(113),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [35] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1517),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(35),
    [sym_block_comment] = STATE(35),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_SEMI] = ACTIONS(341),
    [anon_sym_LPAREN] = ACTIONS(341),
    [anon_sym_RPAREN] = ACTIONS(341),
    [anon_sym_LBRACK] = ACTIONS(341),
    [anon_sym_RBRACK] = ACTIONS(341),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_RBRACE] = ACTIONS(341),
    [anon_sym_COLON] = ACTIONS(345),
    [anon_sym_PLUS] = ACTIONS(347),
    [anon_sym_STAR] = ACTIONS(347),
    [anon_sym_QMARK] = ACTIONS(341),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(347),
    [anon_sym_SLASH] = ACTIONS(347),
    [anon_sym_PERCENT] = ACTIONS(347),
    [anon_sym_CARET] = ACTIONS(347),
    [anon_sym_BANG] = ACTIONS(349),
    [anon_sym_AMP] = ACTIONS(347),
    [anon_sym_PIPE] = ACTIONS(347),
    [anon_sym_AMP_AMP] = ACTIONS(341),
    [anon_sym_PIPE_PIPE] = ACTIONS(341),
    [anon_sym_LT_LT] = ACTIONS(347),
    [anon_sym_GT_GT] = ACTIONS(347),
    [anon_sym_PLUS_EQ] = ACTIONS(341),
    [anon_sym_DASH_EQ] = ACTIONS(341),
    [anon_sym_STAR_EQ] = ACTIONS(341),
    [anon_sym_SLASH_EQ] = ACTIONS(341),
    [anon_sym_PERCENT_EQ] = ACTIONS(341),
    [anon_sym_CARET_EQ] = ACTIONS(341),
    [anon_sym_AMP_EQ] = ACTIONS(341),
    [anon_sym_PIPE_EQ] = ACTIONS(341),
    [anon_sym_LT_LT_EQ] = ACTIONS(341),
    [anon_sym_GT_GT_EQ] = ACTIONS(341),
    [anon_sym_EQ] = ACTIONS(347),
    [anon_sym_EQ_EQ] = ACTIONS(341),
    [anon_sym_BANG_EQ] = ACTIONS(341),
    [anon_sym_GT] = ACTIONS(347),
    [anon_sym_LT] = ACTIONS(347),
    [anon_sym_GT_EQ] = ACTIONS(341),
    [anon_sym_LT_EQ] = ACTIONS(341),
    [anon_sym_DOT] = ACTIONS(347),
    [anon_sym_DOT_DOT] = ACTIONS(347),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(341),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(341),
    [anon_sym_COMMA] = ACTIONS(341),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_as] = ACTIONS(347),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_else] = ACTIONS(347),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [36] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1519),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(36),
    [sym_block_comment] = STATE(36),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_SEMI] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(375),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_RBRACK] = ACTIONS(375),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_RBRACE] = ACTIONS(375),
    [anon_sym_PLUS] = ACTIONS(377),
    [anon_sym_STAR] = ACTIONS(349),
    [anon_sym_QMARK] = ACTIONS(375),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(349),
    [anon_sym_SLASH] = ACTIONS(377),
    [anon_sym_PERCENT] = ACTIONS(377),
    [anon_sym_CARET] = ACTIONS(377),
    [anon_sym_BANG] = ACTIONS(349),
    [anon_sym_AMP] = ACTIONS(379),
    [anon_sym_PIPE] = ACTIONS(381),
    [anon_sym_AMP_AMP] = ACTIONS(375),
    [anon_sym_PIPE_PIPE] = ACTIONS(375),
    [anon_sym_LT_LT] = ACTIONS(377),
    [anon_sym_GT_GT] = ACTIONS(377),
    [anon_sym_PLUS_EQ] = ACTIONS(375),
    [anon_sym_DASH_EQ] = ACTIONS(375),
    [anon_sym_STAR_EQ] = ACTIONS(375),
    [anon_sym_SLASH_EQ] = ACTIONS(375),
    [anon_sym_PERCENT_EQ] = ACTIONS(375),
    [anon_sym_CARET_EQ] = ACTIONS(375),
    [anon_sym_AMP_EQ] = ACTIONS(375),
    [anon_sym_PIPE_EQ] = ACTIONS(375),
    [anon_sym_LT_LT_EQ] = ACTIONS(375),
    [anon_sym_GT_GT_EQ] = ACTIONS(375),
    [anon_sym_EQ] = ACTIONS(377),
    [anon_sym_EQ_EQ] = ACTIONS(375),
    [anon_sym_BANG_EQ] = ACTIONS(375),
    [anon_sym_GT] = ACTIONS(377),
    [anon_sym_LT] = ACTIONS(383),
    [anon_sym_GT_EQ] = ACTIONS(375),
    [anon_sym_LT_EQ] = ACTIONS(375),
    [anon_sym_DOT] = ACTIONS(377),
    [anon_sym_DOT_DOT] = ACTIONS(385),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(375),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(375),
    [anon_sym_COMMA] = ACTIONS(375),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_as] = ACTIONS(377),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_else] = ACTIONS(377),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [37] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1511),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(37),
    [sym_block_comment] = STATE(37),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_SEMI] = ACTIONS(387),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(387),
    [anon_sym_LBRACK] = ACTIONS(387),
    [anon_sym_RBRACK] = ACTIONS(387),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_RBRACE] = ACTIONS(387),
    [anon_sym_PLUS] = ACTIONS(389),
    [anon_sym_STAR] = ACTIONS(389),
    [anon_sym_QMARK] = ACTIONS(387),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(389),
    [anon_sym_SLASH] = ACTIONS(389),
    [anon_sym_PERCENT] = ACTIONS(389),
    [anon_sym_CARET] = ACTIONS(389),
    [anon_sym_BANG] = ACTIONS(349),
    [anon_sym_AMP] = ACTIONS(389),
    [anon_sym_PIPE] = ACTIONS(389),
    [anon_sym_AMP_AMP] = ACTIONS(387),
    [anon_sym_PIPE_PIPE] = ACTIONS(387),
    [anon_sym_LT_LT] = ACTIONS(389),
    [anon_sym_GT_GT] = ACTIONS(389),
    [anon_sym_PLUS_EQ] = ACTIONS(387),
    [anon_sym_DASH_EQ] = ACTIONS(387),
    [anon_sym_STAR_EQ] = ACTIONS(387),
    [anon_sym_SLASH_EQ] = ACTIONS(387),
    [anon_sym_PERCENT_EQ] = ACTIONS(387),
    [anon_sym_CARET_EQ] = ACTIONS(387),
    [anon_sym_AMP_EQ] = ACTIONS(387),
    [anon_sym_PIPE_EQ] = ACTIONS(387),
    [anon_sym_LT_LT_EQ] = ACTIONS(387),
    [anon_sym_GT_GT_EQ] = ACTIONS(387),
    [anon_sym_EQ] = ACTIONS(389),
    [anon_sym_EQ_EQ] = ACTIONS(387),
    [anon_sym_BANG_EQ] = ACTIONS(387),
    [anon_sym_GT] = ACTIONS(389),
    [anon_sym_LT] = ACTIONS(389),
    [anon_sym_GT_EQ] = ACTIONS(387),
    [anon_sym_LT_EQ] = ACTIONS(387),
    [anon_sym_DOT] = ACTIONS(389),
    [anon_sym_DOT_DOT] = ACTIONS(389),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(387),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(387),
    [anon_sym_COMMA] = ACTIONS(387),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_as] = ACTIONS(389),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_else] = ACTIONS(389),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [38] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1516),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(38),
    [sym_block_comment] = STATE(38),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_SEMI] = ACTIONS(391),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(391),
    [anon_sym_LBRACK] = ACTIONS(391),
    [anon_sym_RBRACK] = ACTIONS(391),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_RBRACE] = ACTIONS(391),
    [anon_sym_PLUS] = ACTIONS(393),
    [anon_sym_STAR] = ACTIONS(393),
    [anon_sym_QMARK] = ACTIONS(391),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(393),
    [anon_sym_SLASH] = ACTIONS(393),
    [anon_sym_PERCENT] = ACTIONS(393),
    [anon_sym_CARET] = ACTIONS(393),
    [anon_sym_BANG] = ACTIONS(349),
    [anon_sym_AMP] = ACTIONS(393),
    [anon_sym_PIPE] = ACTIONS(393),
    [anon_sym_AMP_AMP] = ACTIONS(391),
    [anon_sym_PIPE_PIPE] = ACTIONS(391),
    [anon_sym_LT_LT] = ACTIONS(393),
    [anon_sym_GT_GT] = ACTIONS(393),
    [anon_sym_PLUS_EQ] = ACTIONS(391),
    [anon_sym_DASH_EQ] = ACTIONS(391),
    [anon_sym_STAR_EQ] = ACTIONS(391),
    [anon_sym_SLASH_EQ] = ACTIONS(391),
    [anon_sym_PERCENT_EQ] = ACTIONS(391),
    [anon_sym_CARET_EQ] = ACTIONS(391),
    [anon_sym_AMP_EQ] = ACTIONS(391),
    [anon_sym_PIPE_EQ] = ACTIONS(391),
    [anon_sym_LT_LT_EQ] = ACTIONS(391),
    [anon_sym_GT_GT_EQ] = ACTIONS(391),
    [anon_sym_EQ] = ACTIONS(393),
    [anon_sym_EQ_EQ] = ACTIONS(391),
    [anon_sym_BANG_EQ] = ACTIONS(391),
    [anon_sym_GT] = ACTIONS(393),
    [anon_sym_LT] = ACTIONS(393),
    [anon_sym_GT_EQ] = ACTIONS(391),
    [anon_sym_LT_EQ] = ACTIONS(391),
    [anon_sym_DOT] = ACTIONS(393),
    [anon_sym_DOT_DOT] = ACTIONS(393),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(391),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(391),
    [anon_sym_COMMA] = ACTIONS(391),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_as] = ACTIONS(393),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_else] = ACTIONS(393),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [39] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1503),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(35),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(39),
    [sym_block_comment] = STATE(39),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_SEMI] = ACTIONS(395),
    [anon_sym_LPAREN] = ACTIONS(395),
    [anon_sym_RPAREN] = ACTIONS(395),
    [anon_sym_LBRACK] = ACTIONS(395),
    [anon_sym_RBRACK] = ACTIONS(395),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_RBRACE] = ACTIONS(395),
    [anon_sym_PLUS] = ACTIONS(397),
    [anon_sym_STAR] = ACTIONS(397),
    [anon_sym_QMARK] = ACTIONS(395),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(397),
    [anon_sym_SLASH] = ACTIONS(397),
    [anon_sym_PERCENT] = ACTIONS(397),
    [anon_sym_CARET] = ACTIONS(397),
    [anon_sym_BANG] = ACTIONS(349),
    [anon_sym_AMP] = ACTIONS(397),
    [anon_sym_PIPE] = ACTIONS(397),
    [anon_sym_AMP_AMP] = ACTIONS(395),
    [anon_sym_PIPE_PIPE] = ACTIONS(395),
    [anon_sym_LT_LT] = ACTIONS(397),
    [anon_sym_GT_GT] = ACTIONS(397),
    [anon_sym_PLUS_EQ] = ACTIONS(395),
    [anon_sym_DASH_EQ] = ACTIONS(395),
    [anon_sym_STAR_EQ] = ACTIONS(395),
    [anon_sym_SLASH_EQ] = ACTIONS(395),
    [anon_sym_PERCENT_EQ] = ACTIONS(395),
    [anon_sym_CARET_EQ] = ACTIONS(395),
    [anon_sym_AMP_EQ] = ACTIONS(395),
    [anon_sym_PIPE_EQ] = ACTIONS(395),
    [anon_sym_LT_LT_EQ] = ACTIONS(395),
    [anon_sym_GT_GT_EQ] = ACTIONS(395),
    [anon_sym_EQ] = ACTIONS(397),
    [anon_sym_EQ_EQ] = ACTIONS(395),
    [anon_sym_BANG_EQ] = ACTIONS(395),
    [anon_sym_GT] = ACTIONS(397),
    [anon_sym_LT] = ACTIONS(397),
    [anon_sym_GT_EQ] = ACTIONS(395),
    [anon_sym_LT_EQ] = ACTIONS(395),
    [anon_sym_DOT] = ACTIONS(397),
    [anon_sym_DOT_DOT] = ACTIONS(397),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(395),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(395),
    [anon_sym_COMMA] = ACTIONS(395),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(399),
    [anon_sym_as] = ACTIONS(397),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_else] = ACTIONS(397),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [40] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1516),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(40),
    [sym_block_comment] = STATE(40),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_SEMI] = ACTIONS(391),
    [anon_sym_LPAREN] = ACTIONS(391),
    [anon_sym_RPAREN] = ACTIONS(391),
    [anon_sym_LBRACK] = ACTIONS(391),
    [anon_sym_RBRACK] = ACTIONS(391),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_RBRACE] = ACTIONS(391),
    [anon_sym_PLUS] = ACTIONS(393),
    [anon_sym_STAR] = ACTIONS(393),
    [anon_sym_QMARK] = ACTIONS(391),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(393),
    [anon_sym_SLASH] = ACTIONS(393),
    [anon_sym_PERCENT] = ACTIONS(393),
    [anon_sym_CARET] = ACTIONS(393),
    [anon_sym_BANG] = ACTIONS(349),
    [anon_sym_AMP] = ACTIONS(393),
    [anon_sym_PIPE] = ACTIONS(393),
    [anon_sym_AMP_AMP] = ACTIONS(391),
    [anon_sym_PIPE_PIPE] = ACTIONS(391),
    [anon_sym_LT_LT] = ACTIONS(393),
    [anon_sym_GT_GT] = ACTIONS(393),
    [anon_sym_PLUS_EQ] = ACTIONS(391),
    [anon_sym_DASH_EQ] = ACTIONS(391),
    [anon_sym_STAR_EQ] = ACTIONS(391),
    [anon_sym_SLASH_EQ] = ACTIONS(391),
    [anon_sym_PERCENT_EQ] = ACTIONS(391),
    [anon_sym_CARET_EQ] = ACTIONS(391),
    [anon_sym_AMP_EQ] = ACTIONS(391),
    [anon_sym_PIPE_EQ] = ACTIONS(391),
    [anon_sym_LT_LT_EQ] = ACTIONS(391),
    [anon_sym_GT_GT_EQ] = ACTIONS(391),
    [anon_sym_EQ] = ACTIONS(393),
    [anon_sym_EQ_EQ] = ACTIONS(391),
    [anon_sym_BANG_EQ] = ACTIONS(391),
    [anon_sym_GT] = ACTIONS(393),
    [anon_sym_LT] = ACTIONS(393),
    [anon_sym_GT_EQ] = ACTIONS(391),
    [anon_sym_LT_EQ] = ACTIONS(391),
    [anon_sym_DOT] = ACTIONS(393),
    [anon_sym_DOT_DOT] = ACTIONS(393),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(391),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(391),
    [anon_sym_COMMA] = ACTIONS(391),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_as] = ACTIONS(393),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_else] = ACTIONS(393),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [41] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1514),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(41),
    [sym_block_comment] = STATE(41),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_SEMI] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(401),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_RBRACK] = ACTIONS(401),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_RBRACE] = ACTIONS(401),
    [anon_sym_PLUS] = ACTIONS(403),
    [anon_sym_STAR] = ACTIONS(349),
    [anon_sym_QMARK] = ACTIONS(401),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(349),
    [anon_sym_SLASH] = ACTIONS(403),
    [anon_sym_PERCENT] = ACTIONS(403),
    [anon_sym_CARET] = ACTIONS(403),
    [anon_sym_BANG] = ACTIONS(349),
    [anon_sym_AMP] = ACTIONS(379),
    [anon_sym_PIPE] = ACTIONS(381),
    [anon_sym_AMP_AMP] = ACTIONS(401),
    [anon_sym_PIPE_PIPE] = ACTIONS(401),
    [anon_sym_LT_LT] = ACTIONS(403),
    [anon_sym_GT_GT] = ACTIONS(403),
    [anon_sym_PLUS_EQ] = ACTIONS(401),
    [anon_sym_DASH_EQ] = ACTIONS(401),
    [anon_sym_STAR_EQ] = ACTIONS(401),
    [anon_sym_SLASH_EQ] = ACTIONS(401),
    [anon_sym_PERCENT_EQ] = ACTIONS(401),
    [anon_sym_CARET_EQ] = ACTIONS(401),
    [anon_sym_AMP_EQ] = ACTIONS(401),
    [anon_sym_PIPE_EQ] = ACTIONS(401),
    [anon_sym_LT_LT_EQ] = ACTIONS(401),
    [anon_sym_GT_GT_EQ] = ACTIONS(401),
    [anon_sym_EQ] = ACTIONS(403),
    [anon_sym_EQ_EQ] = ACTIONS(401),
    [anon_sym_BANG_EQ] = ACTIONS(401),
    [anon_sym_GT] = ACTIONS(403),
    [anon_sym_LT] = ACTIONS(383),
    [anon_sym_GT_EQ] = ACTIONS(401),
    [anon_sym_LT_EQ] = ACTIONS(401),
    [anon_sym_DOT] = ACTIONS(403),
    [anon_sym_DOT_DOT] = ACTIONS(385),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(401),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(401),
    [anon_sym_COMMA] = ACTIONS(401),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_as] = ACTIONS(403),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_else] = ACTIONS(403),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [42] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1511),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(42),
    [sym_block_comment] = STATE(42),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_SEMI] = ACTIONS(387),
    [anon_sym_LPAREN] = ACTIONS(387),
    [anon_sym_RPAREN] = ACTIONS(387),
    [anon_sym_LBRACK] = ACTIONS(387),
    [anon_sym_RBRACK] = ACTIONS(387),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_RBRACE] = ACTIONS(387),
    [anon_sym_PLUS] = ACTIONS(389),
    [anon_sym_STAR] = ACTIONS(389),
    [anon_sym_QMARK] = ACTIONS(387),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(389),
    [anon_sym_SLASH] = ACTIONS(389),
    [anon_sym_PERCENT] = ACTIONS(389),
    [anon_sym_CARET] = ACTIONS(389),
    [anon_sym_BANG] = ACTIONS(349),
    [anon_sym_AMP] = ACTIONS(389),
    [anon_sym_PIPE] = ACTIONS(389),
    [anon_sym_AMP_AMP] = ACTIONS(387),
    [anon_sym_PIPE_PIPE] = ACTIONS(387),
    [anon_sym_LT_LT] = ACTIONS(389),
    [anon_sym_GT_GT] = ACTIONS(389),
    [anon_sym_PLUS_EQ] = ACTIONS(387),
    [anon_sym_DASH_EQ] = ACTIONS(387),
    [anon_sym_STAR_EQ] = ACTIONS(387),
    [anon_sym_SLASH_EQ] = ACTIONS(387),
    [anon_sym_PERCENT_EQ] = ACTIONS(387),
    [anon_sym_CARET_EQ] = ACTIONS(387),
    [anon_sym_AMP_EQ] = ACTIONS(387),
    [anon_sym_PIPE_EQ] = ACTIONS(387),
    [anon_sym_LT_LT_EQ] = ACTIONS(387),
    [anon_sym_GT_GT_EQ] = ACTIONS(387),
    [anon_sym_EQ] = ACTIONS(389),
    [anon_sym_EQ_EQ] = ACTIONS(387),
    [anon_sym_BANG_EQ] = ACTIONS(387),
    [anon_sym_GT] = ACTIONS(389),
    [anon_sym_LT] = ACTIONS(389),
    [anon_sym_GT_EQ] = ACTIONS(387),
    [anon_sym_LT_EQ] = ACTIONS(387),
    [anon_sym_DOT] = ACTIONS(389),
    [anon_sym_DOT_DOT] = ACTIONS(389),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(387),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(387),
    [anon_sym_COMMA] = ACTIONS(387),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_as] = ACTIONS(389),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_else] = ACTIONS(389),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [43] = {
    [sym_bracketed_type] = STATE(3529),
    [sym_generic_function] = STATE(1664),
    [sym_generic_type_with_turbofish] = STATE(2956),
    [sym__expression_except_range] = STATE(1626),
    [sym__expression] = STATE(1754),
    [sym_macro_invocation] = STATE(1671),
    [sym_scoped_identifier] = STATE(1575),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3163),
    [sym_range_expression] = STATE(1666),
    [sym_unary_expression] = STATE(1664),
    [sym_try_expression] = STATE(1664),
    [sym_reference_expression] = STATE(1664),
    [sym_binary_expression] = STATE(1664),
    [sym_assignment_expression] = STATE(1664),
    [sym_compound_assignment_expr] = STATE(1664),
    [sym_type_cast_expression] = STATE(1664),
    [sym_return_expression] = STATE(1664),
    [sym_yield_expression] = STATE(1664),
    [sym_call_expression] = STATE(1664),
    [sym_array_expression] = STATE(1664),
    [sym_parenthesized_expression] = STATE(1664),
    [sym_tuple_expression] = STATE(1664),
    [sym_unit_expression] = STATE(1664),
    [sym_struct_expression] = STATE(1664),
    [sym_if_expression] = STATE(1664),
    [sym_match_expression] = STATE(1664),
    [sym_while_expression] = STATE(1664),
    [sym_loop_expression] = STATE(1664),
    [sym_for_expression] = STATE(1664),
    [sym_const_block] = STATE(1664),
    [sym_closure_expression] = STATE(1664),
    [sym_closure_parameters] = STATE(232),
    [sym_label] = STATE(3614),
    [sym_break_expression] = STATE(1664),
    [sym_continue_expression] = STATE(1664),
    [sym_index_expression] = STATE(1664),
    [sym_await_expression] = STATE(1664),
    [sym_field_expression] = STATE(1597),
    [sym_unsafe_block] = STATE(1664),
    [sym_async_block] = STATE(1664),
    [sym_gen_block] = STATE(1664),
    [sym_try_block] = STATE(1664),
    [sym_block] = STATE(1664),
    [sym__literal] = STATE(1664),
    [sym_string_literal] = STATE(1786),
    [sym_raw_string_literal] = STATE(1786),
    [sym_boolean_literal] = STATE(1786),
    [sym_line_comment] = STATE(43),
    [sym_block_comment] = STATE(43),
    [sym_identifier] = ACTIONS(405),
    [anon_sym_LPAREN] = ACTIONS(341),
    [anon_sym_LBRACK] = ACTIONS(341),
    [anon_sym_LBRACE] = ACTIONS(407),
    [anon_sym_EQ_GT] = ACTIONS(341),
    [anon_sym_COLON] = ACTIONS(409),
    [anon_sym_PLUS] = ACTIONS(347),
    [anon_sym_STAR] = ACTIONS(347),
    [anon_sym_QMARK] = ACTIONS(341),
    [anon_sym_u8] = ACTIONS(411),
    [anon_sym_i8] = ACTIONS(411),
    [anon_sym_u16] = ACTIONS(411),
    [anon_sym_i16] = ACTIONS(411),
    [anon_sym_u32] = ACTIONS(411),
    [anon_sym_i32] = ACTIONS(411),
    [anon_sym_u64] = ACTIONS(411),
    [anon_sym_i64] = ACTIONS(411),
    [anon_sym_u128] = ACTIONS(411),
    [anon_sym_i128] = ACTIONS(411),
    [anon_sym_isize] = ACTIONS(411),
    [anon_sym_usize] = ACTIONS(411),
    [anon_sym_f32] = ACTIONS(411),
    [anon_sym_f64] = ACTIONS(411),
    [anon_sym_bool] = ACTIONS(411),
    [anon_sym_str] = ACTIONS(411),
    [anon_sym_char] = ACTIONS(411),
    [anon_sym_DASH] = ACTIONS(347),
    [anon_sym_SLASH] = ACTIONS(347),
    [anon_sym_PERCENT] = ACTIONS(347),
    [anon_sym_CARET] = ACTIONS(347),
    [anon_sym_BANG] = ACTIONS(413),
    [anon_sym_AMP] = ACTIONS(347),
    [anon_sym_PIPE] = ACTIONS(347),
    [anon_sym_AMP_AMP] = ACTIONS(341),
    [anon_sym_PIPE_PIPE] = ACTIONS(341),
    [anon_sym_LT_LT] = ACTIONS(347),
    [anon_sym_GT_GT] = ACTIONS(347),
    [anon_sym_PLUS_EQ] = ACTIONS(341),
    [anon_sym_DASH_EQ] = ACTIONS(341),
    [anon_sym_STAR_EQ] = ACTIONS(341),
    [anon_sym_SLASH_EQ] = ACTIONS(341),
    [anon_sym_PERCENT_EQ] = ACTIONS(341),
    [anon_sym_CARET_EQ] = ACTIONS(341),
    [anon_sym_AMP_EQ] = ACTIONS(341),
    [anon_sym_PIPE_EQ] = ACTIONS(341),
    [anon_sym_LT_LT_EQ] = ACTIONS(341),
    [anon_sym_GT_GT_EQ] = ACTIONS(341),
    [anon_sym_EQ] = ACTIONS(347),
    [anon_sym_EQ_EQ] = ACTIONS(341),
    [anon_sym_BANG_EQ] = ACTIONS(341),
    [anon_sym_GT] = ACTIONS(347),
    [anon_sym_LT] = ACTIONS(347),
    [anon_sym_GT_EQ] = ACTIONS(341),
    [anon_sym_LT_EQ] = ACTIONS(341),
    [anon_sym_DOT] = ACTIONS(347),
    [anon_sym_DOT_DOT] = ACTIONS(347),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(341),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(341),
    [anon_sym_COLON_COLON] = ACTIONS(415),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_as] = ACTIONS(347),
    [anon_sym_async] = ACTIONS(417),
    [anon_sym_break] = ACTIONS(419),
    [anon_sym_const] = ACTIONS(421),
    [anon_sym_continue] = ACTIONS(423),
    [anon_sym_default] = ACTIONS(425),
    [anon_sym_for] = ACTIONS(427),
    [anon_sym_gen] = ACTIONS(429),
    [anon_sym_if] = ACTIONS(431),
    [anon_sym_loop] = ACTIONS(433),
    [anon_sym_match] = ACTIONS(435),
    [anon_sym_return] = ACTIONS(437),
    [anon_sym_static] = ACTIONS(439),
    [anon_sym_union] = ACTIONS(425),
    [anon_sym_unsafe] = ACTIONS(441),
    [anon_sym_while] = ACTIONS(443),
    [anon_sym_yield] = ACTIONS(445),
    [anon_sym_move] = ACTIONS(447),
    [anon_sym_try] = ACTIONS(449),
    [sym_integer_literal] = ACTIONS(451),
    [aux_sym_string_literal_token1] = ACTIONS(453),
    [sym_char_literal] = ACTIONS(451),
    [anon_sym_true] = ACTIONS(455),
    [anon_sym_false] = ACTIONS(455),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(457),
    [sym_super] = ACTIONS(459),
    [sym_crate] = ACTIONS(459),
    [sym_metavariable] = ACTIONS(461),
    [sym__raw_string_literal_start] = ACTIONS(463),
    [sym_float_literal] = ACTIONS(451),
  },
  [44] = {
    [sym_bracketed_type] = STATE(3529),
    [sym_generic_function] = STATE(1664),
    [sym_generic_type_with_turbofish] = STATE(2956),
    [sym__expression_except_range] = STATE(1626),
    [sym__expression] = STATE(1681),
    [sym_macro_invocation] = STATE(1671),
    [sym_scoped_identifier] = STATE(1575),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3163),
    [sym_range_expression] = STATE(1666),
    [sym_unary_expression] = STATE(1664),
    [sym_try_expression] = STATE(1664),
    [sym_reference_expression] = STATE(1664),
    [sym_binary_expression] = STATE(1664),
    [sym_assignment_expression] = STATE(1664),
    [sym_compound_assignment_expr] = STATE(1664),
    [sym_type_cast_expression] = STATE(1664),
    [sym_return_expression] = STATE(1664),
    [sym_yield_expression] = STATE(1664),
    [sym_call_expression] = STATE(1664),
    [sym_array_expression] = STATE(1664),
    [sym_parenthesized_expression] = STATE(1664),
    [sym_tuple_expression] = STATE(1664),
    [sym_unit_expression] = STATE(1664),
    [sym_struct_expression] = STATE(1664),
    [sym_if_expression] = STATE(1664),
    [sym_match_expression] = STATE(1664),
    [sym_while_expression] = STATE(1664),
    [sym_loop_expression] = STATE(1664),
    [sym_for_expression] = STATE(1664),
    [sym_const_block] = STATE(1664),
    [sym_closure_expression] = STATE(1664),
    [sym_closure_parameters] = STATE(232),
    [sym_label] = STATE(43),
    [sym_break_expression] = STATE(1664),
    [sym_continue_expression] = STATE(1664),
    [sym_index_expression] = STATE(1664),
    [sym_await_expression] = STATE(1664),
    [sym_field_expression] = STATE(1597),
    [sym_unsafe_block] = STATE(1664),
    [sym_async_block] = STATE(1664),
    [sym_gen_block] = STATE(1664),
    [sym_try_block] = STATE(1664),
    [sym_block] = STATE(1664),
    [sym__literal] = STATE(1664),
    [sym_string_literal] = STATE(1786),
    [sym_raw_string_literal] = STATE(1786),
    [sym_boolean_literal] = STATE(1786),
    [sym_line_comment] = STATE(44),
    [sym_block_comment] = STATE(44),
    [sym_identifier] = ACTIONS(405),
    [anon_sym_LPAREN] = ACTIONS(395),
    [anon_sym_LBRACK] = ACTIONS(395),
    [anon_sym_LBRACE] = ACTIONS(407),
    [anon_sym_EQ_GT] = ACTIONS(395),
    [anon_sym_PLUS] = ACTIONS(397),
    [anon_sym_STAR] = ACTIONS(397),
    [anon_sym_QMARK] = ACTIONS(395),
    [anon_sym_u8] = ACTIONS(411),
    [anon_sym_i8] = ACTIONS(411),
    [anon_sym_u16] = ACTIONS(411),
    [anon_sym_i16] = ACTIONS(411),
    [anon_sym_u32] = ACTIONS(411),
    [anon_sym_i32] = ACTIONS(411),
    [anon_sym_u64] = ACTIONS(411),
    [anon_sym_i64] = ACTIONS(411),
    [anon_sym_u128] = ACTIONS(411),
    [anon_sym_i128] = ACTIONS(411),
    [anon_sym_isize] = ACTIONS(411),
    [anon_sym_usize] = ACTIONS(411),
    [anon_sym_f32] = ACTIONS(411),
    [anon_sym_f64] = ACTIONS(411),
    [anon_sym_bool] = ACTIONS(411),
    [anon_sym_str] = ACTIONS(411),
    [anon_sym_char] = ACTIONS(411),
    [anon_sym_DASH] = ACTIONS(397),
    [anon_sym_SLASH] = ACTIONS(397),
    [anon_sym_PERCENT] = ACTIONS(397),
    [anon_sym_CARET] = ACTIONS(397),
    [anon_sym_BANG] = ACTIONS(413),
    [anon_sym_AMP] = ACTIONS(397),
    [anon_sym_PIPE] = ACTIONS(397),
    [anon_sym_AMP_AMP] = ACTIONS(395),
    [anon_sym_PIPE_PIPE] = ACTIONS(395),
    [anon_sym_LT_LT] = ACTIONS(397),
    [anon_sym_GT_GT] = ACTIONS(397),
    [anon_sym_PLUS_EQ] = ACTIONS(395),
    [anon_sym_DASH_EQ] = ACTIONS(395),
    [anon_sym_STAR_EQ] = ACTIONS(395),
    [anon_sym_SLASH_EQ] = ACTIONS(395),
    [anon_sym_PERCENT_EQ] = ACTIONS(395),
    [anon_sym_CARET_EQ] = ACTIONS(395),
    [anon_sym_AMP_EQ] = ACTIONS(395),
    [anon_sym_PIPE_EQ] = ACTIONS(395),
    [anon_sym_LT_LT_EQ] = ACTIONS(395),
    [anon_sym_GT_GT_EQ] = ACTIONS(395),
    [anon_sym_EQ] = ACTIONS(397),
    [anon_sym_EQ_EQ] = ACTIONS(395),
    [anon_sym_BANG_EQ] = ACTIONS(395),
    [anon_sym_GT] = ACTIONS(397),
    [anon_sym_LT] = ACTIONS(397),
    [anon_sym_GT_EQ] = ACTIONS(395),
    [anon_sym_LT_EQ] = ACTIONS(395),
    [anon_sym_DOT] = ACTIONS(397),
    [anon_sym_DOT_DOT] = ACTIONS(397),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(395),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(395),
    [anon_sym_COLON_COLON] = ACTIONS(415),
    [anon_sym_SQUOTE] = ACTIONS(465),
    [anon_sym_as] = ACTIONS(397),
    [anon_sym_async] = ACTIONS(417),
    [anon_sym_break] = ACTIONS(419),
    [anon_sym_const] = ACTIONS(421),
    [anon_sym_continue] = ACTIONS(423),
    [anon_sym_default] = ACTIONS(425),
    [anon_sym_for] = ACTIONS(427),
    [anon_sym_gen] = ACTIONS(429),
    [anon_sym_if] = ACTIONS(431),
    [anon_sym_loop] = ACTIONS(433),
    [anon_sym_match] = ACTIONS(435),
    [anon_sym_return] = ACTIONS(437),
    [anon_sym_static] = ACTIONS(439),
    [anon_sym_union] = ACTIONS(425),
    [anon_sym_unsafe] = ACTIONS(441),
    [anon_sym_while] = ACTIONS(443),
    [anon_sym_yield] = ACTIONS(445),
    [anon_sym_move] = ACTIONS(447),
    [anon_sym_try] = ACTIONS(449),
    [sym_integer_literal] = ACTIONS(451),
    [aux_sym_string_literal_token1] = ACTIONS(453),
    [sym_char_literal] = ACTIONS(451),
    [anon_sym_true] = ACTIONS(455),
    [anon_sym_false] = ACTIONS(455),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(457),
    [sym_super] = ACTIONS(459),
    [sym_crate] = ACTIONS(459),
    [sym_metavariable] = ACTIONS(461),
    [sym__raw_string_literal_start] = ACTIONS(463),
    [sym_float_literal] = ACTIONS(451),
  },
  [45] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1703),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(246),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(45),
    [sym_block_comment] = STATE(45),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(341),
    [anon_sym_LBRACK] = ACTIONS(341),
    [anon_sym_LBRACE] = ACTIONS(341),
    [anon_sym_COLON] = ACTIONS(345),
    [anon_sym_PLUS] = ACTIONS(347),
    [anon_sym_STAR] = ACTIONS(347),
    [anon_sym_QMARK] = ACTIONS(341),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(347),
    [anon_sym_SLASH] = ACTIONS(347),
    [anon_sym_PERCENT] = ACTIONS(347),
    [anon_sym_CARET] = ACTIONS(347),
    [anon_sym_BANG] = ACTIONS(471),
    [anon_sym_AMP] = ACTIONS(347),
    [anon_sym_PIPE] = ACTIONS(347),
    [anon_sym_AMP_AMP] = ACTIONS(341),
    [anon_sym_PIPE_PIPE] = ACTIONS(341),
    [anon_sym_LT_LT] = ACTIONS(347),
    [anon_sym_GT_GT] = ACTIONS(347),
    [anon_sym_PLUS_EQ] = ACTIONS(341),
    [anon_sym_DASH_EQ] = ACTIONS(341),
    [anon_sym_STAR_EQ] = ACTIONS(341),
    [anon_sym_SLASH_EQ] = ACTIONS(341),
    [anon_sym_PERCENT_EQ] = ACTIONS(341),
    [anon_sym_CARET_EQ] = ACTIONS(341),
    [anon_sym_AMP_EQ] = ACTIONS(341),
    [anon_sym_PIPE_EQ] = ACTIONS(341),
    [anon_sym_LT_LT_EQ] = ACTIONS(341),
    [anon_sym_GT_GT_EQ] = ACTIONS(341),
    [anon_sym_EQ] = ACTIONS(347),
    [anon_sym_EQ_EQ] = ACTIONS(341),
    [anon_sym_BANG_EQ] = ACTIONS(341),
    [anon_sym_GT] = ACTIONS(347),
    [anon_sym_LT] = ACTIONS(347),
    [anon_sym_GT_EQ] = ACTIONS(341),
    [anon_sym_LT_EQ] = ACTIONS(341),
    [anon_sym_DOT] = ACTIONS(347),
    [anon_sym_DOT_DOT] = ACTIONS(347),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(341),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(341),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_as] = ACTIONS(347),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(475),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(479),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(481),
    [anon_sym_static] = ACTIONS(483),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(485),
    [anon_sym_move] = ACTIONS(487),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [46] = {
    [sym_bracketed_type] = STATE(3529),
    [sym_generic_function] = STATE(1664),
    [sym_generic_type_with_turbofish] = STATE(2956),
    [sym__expression_except_range] = STATE(1626),
    [sym__expression] = STATE(1686),
    [sym_macro_invocation] = STATE(1671),
    [sym_scoped_identifier] = STATE(1575),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3163),
    [sym_range_expression] = STATE(1666),
    [sym_unary_expression] = STATE(1664),
    [sym_try_expression] = STATE(1664),
    [sym_reference_expression] = STATE(1664),
    [sym_binary_expression] = STATE(1664),
    [sym_assignment_expression] = STATE(1664),
    [sym_compound_assignment_expr] = STATE(1664),
    [sym_type_cast_expression] = STATE(1664),
    [sym_return_expression] = STATE(1664),
    [sym_yield_expression] = STATE(1664),
    [sym_call_expression] = STATE(1664),
    [sym_array_expression] = STATE(1664),
    [sym_parenthesized_expression] = STATE(1664),
    [sym_tuple_expression] = STATE(1664),
    [sym_unit_expression] = STATE(1664),
    [sym_struct_expression] = STATE(1664),
    [sym_if_expression] = STATE(1664),
    [sym_match_expression] = STATE(1664),
    [sym_while_expression] = STATE(1664),
    [sym_loop_expression] = STATE(1664),
    [sym_for_expression] = STATE(1664),
    [sym_const_block] = STATE(1664),
    [sym_closure_expression] = STATE(1664),
    [sym_closure_parameters] = STATE(232),
    [sym_label] = STATE(3614),
    [sym_break_expression] = STATE(1664),
    [sym_continue_expression] = STATE(1664),
    [sym_index_expression] = STATE(1664),
    [sym_await_expression] = STATE(1664),
    [sym_field_expression] = STATE(1597),
    [sym_unsafe_block] = STATE(1664),
    [sym_async_block] = STATE(1664),
    [sym_gen_block] = STATE(1664),
    [sym_try_block] = STATE(1664),
    [sym_block] = STATE(1664),
    [sym__literal] = STATE(1664),
    [sym_string_literal] = STATE(1786),
    [sym_raw_string_literal] = STATE(1786),
    [sym_boolean_literal] = STATE(1786),
    [sym_line_comment] = STATE(46),
    [sym_block_comment] = STATE(46),
    [sym_identifier] = ACTIONS(405),
    [anon_sym_LPAREN] = ACTIONS(495),
    [anon_sym_LBRACK] = ACTIONS(497),
    [anon_sym_LBRACE] = ACTIONS(407),
    [anon_sym_EQ_GT] = ACTIONS(375),
    [anon_sym_PLUS] = ACTIONS(377),
    [anon_sym_STAR] = ACTIONS(413),
    [anon_sym_QMARK] = ACTIONS(375),
    [anon_sym_u8] = ACTIONS(411),
    [anon_sym_i8] = ACTIONS(411),
    [anon_sym_u16] = ACTIONS(411),
    [anon_sym_i16] = ACTIONS(411),
    [anon_sym_u32] = ACTIONS(411),
    [anon_sym_i32] = ACTIONS(411),
    [anon_sym_u64] = ACTIONS(411),
    [anon_sym_i64] = ACTIONS(411),
    [anon_sym_u128] = ACTIONS(411),
    [anon_sym_i128] = ACTIONS(411),
    [anon_sym_isize] = ACTIONS(411),
    [anon_sym_usize] = ACTIONS(411),
    [anon_sym_f32] = ACTIONS(411),
    [anon_sym_f64] = ACTIONS(411),
    [anon_sym_bool] = ACTIONS(411),
    [anon_sym_str] = ACTIONS(411),
    [anon_sym_char] = ACTIONS(411),
    [anon_sym_DASH] = ACTIONS(413),
    [anon_sym_SLASH] = ACTIONS(377),
    [anon_sym_PERCENT] = ACTIONS(377),
    [anon_sym_CARET] = ACTIONS(377),
    [anon_sym_BANG] = ACTIONS(413),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_PIPE] = ACTIONS(381),
    [anon_sym_AMP_AMP] = ACTIONS(375),
    [anon_sym_PIPE_PIPE] = ACTIONS(375),
    [anon_sym_LT_LT] = ACTIONS(377),
    [anon_sym_GT_GT] = ACTIONS(377),
    [anon_sym_PLUS_EQ] = ACTIONS(375),
    [anon_sym_DASH_EQ] = ACTIONS(375),
    [anon_sym_STAR_EQ] = ACTIONS(375),
    [anon_sym_SLASH_EQ] = ACTIONS(375),
    [anon_sym_PERCENT_EQ] = ACTIONS(375),
    [anon_sym_CARET_EQ] = ACTIONS(375),
    [anon_sym_AMP_EQ] = ACTIONS(375),
    [anon_sym_PIPE_EQ] = ACTIONS(375),
    [anon_sym_LT_LT_EQ] = ACTIONS(375),
    [anon_sym_GT_GT_EQ] = ACTIONS(375),
    [anon_sym_EQ] = ACTIONS(377),
    [anon_sym_EQ_EQ] = ACTIONS(375),
    [anon_sym_BANG_EQ] = ACTIONS(375),
    [anon_sym_GT] = ACTIONS(377),
    [anon_sym_LT] = ACTIONS(383),
    [anon_sym_GT_EQ] = ACTIONS(375),
    [anon_sym_LT_EQ] = ACTIONS(375),
    [anon_sym_DOT] = ACTIONS(377),
    [anon_sym_DOT_DOT] = ACTIONS(501),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(375),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(375),
    [anon_sym_COLON_COLON] = ACTIONS(415),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_as] = ACTIONS(377),
    [anon_sym_async] = ACTIONS(417),
    [anon_sym_break] = ACTIONS(419),
    [anon_sym_const] = ACTIONS(421),
    [anon_sym_continue] = ACTIONS(423),
    [anon_sym_default] = ACTIONS(425),
    [anon_sym_for] = ACTIONS(427),
    [anon_sym_gen] = ACTIONS(429),
    [anon_sym_if] = ACTIONS(431),
    [anon_sym_loop] = ACTIONS(433),
    [anon_sym_match] = ACTIONS(435),
    [anon_sym_return] = ACTIONS(437),
    [anon_sym_static] = ACTIONS(439),
    [anon_sym_union] = ACTIONS(425),
    [anon_sym_unsafe] = ACTIONS(441),
    [anon_sym_while] = ACTIONS(443),
    [anon_sym_yield] = ACTIONS(445),
    [anon_sym_move] = ACTIONS(447),
    [anon_sym_try] = ACTIONS(449),
    [sym_integer_literal] = ACTIONS(451),
    [aux_sym_string_literal_token1] = ACTIONS(453),
    [sym_char_literal] = ACTIONS(451),
    [anon_sym_true] = ACTIONS(455),
    [anon_sym_false] = ACTIONS(455),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(457),
    [sym_super] = ACTIONS(459),
    [sym_crate] = ACTIONS(459),
    [sym_metavariable] = ACTIONS(461),
    [sym__raw_string_literal_start] = ACTIONS(463),
    [sym_float_literal] = ACTIONS(451),
  },
  [47] = {
    [sym_bracketed_type] = STATE(3529),
    [sym_generic_function] = STATE(1664),
    [sym_generic_type_with_turbofish] = STATE(2956),
    [sym__expression_except_range] = STATE(1626),
    [sym__expression] = STATE(1685),
    [sym_macro_invocation] = STATE(1671),
    [sym_scoped_identifier] = STATE(1575),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3163),
    [sym_range_expression] = STATE(1666),
    [sym_unary_expression] = STATE(1664),
    [sym_try_expression] = STATE(1664),
    [sym_reference_expression] = STATE(1664),
    [sym_binary_expression] = STATE(1664),
    [sym_assignment_expression] = STATE(1664),
    [sym_compound_assignment_expr] = STATE(1664),
    [sym_type_cast_expression] = STATE(1664),
    [sym_return_expression] = STATE(1664),
    [sym_yield_expression] = STATE(1664),
    [sym_call_expression] = STATE(1664),
    [sym_array_expression] = STATE(1664),
    [sym_parenthesized_expression] = STATE(1664),
    [sym_tuple_expression] = STATE(1664),
    [sym_unit_expression] = STATE(1664),
    [sym_struct_expression] = STATE(1664),
    [sym_if_expression] = STATE(1664),
    [sym_match_expression] = STATE(1664),
    [sym_while_expression] = STATE(1664),
    [sym_loop_expression] = STATE(1664),
    [sym_for_expression] = STATE(1664),
    [sym_const_block] = STATE(1664),
    [sym_closure_expression] = STATE(1664),
    [sym_closure_parameters] = STATE(232),
    [sym_label] = STATE(3614),
    [sym_break_expression] = STATE(1664),
    [sym_continue_expression] = STATE(1664),
    [sym_index_expression] = STATE(1664),
    [sym_await_expression] = STATE(1664),
    [sym_field_expression] = STATE(1597),
    [sym_unsafe_block] = STATE(1664),
    [sym_async_block] = STATE(1664),
    [sym_gen_block] = STATE(1664),
    [sym_try_block] = STATE(1664),
    [sym_block] = STATE(1664),
    [sym__literal] = STATE(1664),
    [sym_string_literal] = STATE(1786),
    [sym_raw_string_literal] = STATE(1786),
    [sym_boolean_literal] = STATE(1786),
    [sym_line_comment] = STATE(47),
    [sym_block_comment] = STATE(47),
    [sym_identifier] = ACTIONS(405),
    [anon_sym_LPAREN] = ACTIONS(495),
    [anon_sym_LBRACK] = ACTIONS(497),
    [anon_sym_LBRACE] = ACTIONS(407),
    [anon_sym_EQ_GT] = ACTIONS(401),
    [anon_sym_PLUS] = ACTIONS(403),
    [anon_sym_STAR] = ACTIONS(413),
    [anon_sym_QMARK] = ACTIONS(401),
    [anon_sym_u8] = ACTIONS(411),
    [anon_sym_i8] = ACTIONS(411),
    [anon_sym_u16] = ACTIONS(411),
    [anon_sym_i16] = ACTIONS(411),
    [anon_sym_u32] = ACTIONS(411),
    [anon_sym_i32] = ACTIONS(411),
    [anon_sym_u64] = ACTIONS(411),
    [anon_sym_i64] = ACTIONS(411),
    [anon_sym_u128] = ACTIONS(411),
    [anon_sym_i128] = ACTIONS(411),
    [anon_sym_isize] = ACTIONS(411),
    [anon_sym_usize] = ACTIONS(411),
    [anon_sym_f32] = ACTIONS(411),
    [anon_sym_f64] = ACTIONS(411),
    [anon_sym_bool] = ACTIONS(411),
    [anon_sym_str] = ACTIONS(411),
    [anon_sym_char] = ACTIONS(411),
    [anon_sym_DASH] = ACTIONS(413),
    [anon_sym_SLASH] = ACTIONS(403),
    [anon_sym_PERCENT] = ACTIONS(403),
    [anon_sym_CARET] = ACTIONS(403),
    [anon_sym_BANG] = ACTIONS(413),
    [anon_sym_AMP] = ACTIONS(499),
    [anon_sym_PIPE] = ACTIONS(381),
    [anon_sym_AMP_AMP] = ACTIONS(401),
    [anon_sym_PIPE_PIPE] = ACTIONS(401),
    [anon_sym_LT_LT] = ACTIONS(403),
    [anon_sym_GT_GT] = ACTIONS(403),
    [anon_sym_PLUS_EQ] = ACTIONS(401),
    [anon_sym_DASH_EQ] = ACTIONS(401),
    [anon_sym_STAR_EQ] = ACTIONS(401),
    [anon_sym_SLASH_EQ] = ACTIONS(401),
    [anon_sym_PERCENT_EQ] = ACTIONS(401),
    [anon_sym_CARET_EQ] = ACTIONS(401),
    [anon_sym_AMP_EQ] = ACTIONS(401),
    [anon_sym_PIPE_EQ] = ACTIONS(401),
    [anon_sym_LT_LT_EQ] = ACTIONS(401),
    [anon_sym_GT_GT_EQ] = ACTIONS(401),
    [anon_sym_EQ] = ACTIONS(403),
    [anon_sym_EQ_EQ] = ACTIONS(401),
    [anon_sym_BANG_EQ] = ACTIONS(401),
    [anon_sym_GT] = ACTIONS(403),
    [anon_sym_LT] = ACTIONS(383),
    [anon_sym_GT_EQ] = ACTIONS(401),
    [anon_sym_LT_EQ] = ACTIONS(401),
    [anon_sym_DOT] = ACTIONS(403),
    [anon_sym_DOT_DOT] = ACTIONS(501),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(401),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(401),
    [anon_sym_COLON_COLON] = ACTIONS(415),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_as] = ACTIONS(403),
    [anon_sym_async] = ACTIONS(417),
    [anon_sym_break] = ACTIONS(419),
    [anon_sym_const] = ACTIONS(421),
    [anon_sym_continue] = ACTIONS(423),
    [anon_sym_default] = ACTIONS(425),
    [anon_sym_for] = ACTIONS(427),
    [anon_sym_gen] = ACTIONS(429),
    [anon_sym_if] = ACTIONS(431),
    [anon_sym_loop] = ACTIONS(433),
    [anon_sym_match] = ACTIONS(435),
    [anon_sym_return] = ACTIONS(437),
    [anon_sym_static] = ACTIONS(439),
    [anon_sym_union] = ACTIONS(425),
    [anon_sym_unsafe] = ACTIONS(441),
    [anon_sym_while] = ACTIONS(443),
    [anon_sym_yield] = ACTIONS(445),
    [anon_sym_move] = ACTIONS(447),
    [anon_sym_try] = ACTIONS(449),
    [sym_integer_literal] = ACTIONS(451),
    [aux_sym_string_literal_token1] = ACTIONS(453),
    [sym_char_literal] = ACTIONS(451),
    [anon_sym_true] = ACTIONS(455),
    [anon_sym_false] = ACTIONS(455),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(457),
    [sym_super] = ACTIONS(459),
    [sym_crate] = ACTIONS(459),
    [sym_metavariable] = ACTIONS(461),
    [sym__raw_string_literal_start] = ACTIONS(463),
    [sym_float_literal] = ACTIONS(451),
  },
  [48] = {
    [sym_bracketed_type] = STATE(3529),
    [sym_generic_function] = STATE(1664),
    [sym_generic_type_with_turbofish] = STATE(2956),
    [sym__expression_except_range] = STATE(1626),
    [sym__expression] = STATE(1680),
    [sym_macro_invocation] = STATE(1671),
    [sym_scoped_identifier] = STATE(1575),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3163),
    [sym_range_expression] = STATE(1666),
    [sym_unary_expression] = STATE(1664),
    [sym_try_expression] = STATE(1664),
    [sym_reference_expression] = STATE(1664),
    [sym_binary_expression] = STATE(1664),
    [sym_assignment_expression] = STATE(1664),
    [sym_compound_assignment_expr] = STATE(1664),
    [sym_type_cast_expression] = STATE(1664),
    [sym_return_expression] = STATE(1664),
    [sym_yield_expression] = STATE(1664),
    [sym_call_expression] = STATE(1664),
    [sym_array_expression] = STATE(1664),
    [sym_parenthesized_expression] = STATE(1664),
    [sym_tuple_expression] = STATE(1664),
    [sym_unit_expression] = STATE(1664),
    [sym_struct_expression] = STATE(1664),
    [sym_if_expression] = STATE(1664),
    [sym_match_expression] = STATE(1664),
    [sym_while_expression] = STATE(1664),
    [sym_loop_expression] = STATE(1664),
    [sym_for_expression] = STATE(1664),
    [sym_const_block] = STATE(1664),
    [sym_closure_expression] = STATE(1664),
    [sym_closure_parameters] = STATE(232),
    [sym_label] = STATE(3614),
    [sym_break_expression] = STATE(1664),
    [sym_continue_expression] = STATE(1664),
    [sym_index_expression] = STATE(1664),
    [sym_await_expression] = STATE(1664),
    [sym_field_expression] = STATE(1597),
    [sym_unsafe_block] = STATE(1664),
    [sym_async_block] = STATE(1664),
    [sym_gen_block] = STATE(1664),
    [sym_try_block] = STATE(1664),
    [sym_block] = STATE(1664),
    [sym__literal] = STATE(1664),
    [sym_string_literal] = STATE(1786),
    [sym_raw_string_literal] = STATE(1786),
    [sym_boolean_literal] = STATE(1786),
    [sym_line_comment] = STATE(48),
    [sym_block_comment] = STATE(48),
    [sym_identifier] = ACTIONS(405),
    [anon_sym_LPAREN] = ACTIONS(495),
    [anon_sym_LBRACK] = ACTIONS(391),
    [anon_sym_LBRACE] = ACTIONS(407),
    [anon_sym_EQ_GT] = ACTIONS(391),
    [anon_sym_PLUS] = ACTIONS(393),
    [anon_sym_STAR] = ACTIONS(393),
    [anon_sym_QMARK] = ACTIONS(391),
    [anon_sym_u8] = ACTIONS(411),
    [anon_sym_i8] = ACTIONS(411),
    [anon_sym_u16] = ACTIONS(411),
    [anon_sym_i16] = ACTIONS(411),
    [anon_sym_u32] = ACTIONS(411),
    [anon_sym_i32] = ACTIONS(411),
    [anon_sym_u64] = ACTIONS(411),
    [anon_sym_i64] = ACTIONS(411),
    [anon_sym_u128] = ACTIONS(411),
    [anon_sym_i128] = ACTIONS(411),
    [anon_sym_isize] = ACTIONS(411),
    [anon_sym_usize] = ACTIONS(411),
    [anon_sym_f32] = ACTIONS(411),
    [anon_sym_f64] = ACTIONS(411),
    [anon_sym_bool] = ACTIONS(411),
    [anon_sym_str] = ACTIONS(411),
    [anon_sym_char] = ACTIONS(411),
    [anon_sym_DASH] = ACTIONS(393),
    [anon_sym_SLASH] = ACTIONS(393),
    [anon_sym_PERCENT] = ACTIONS(393),
    [anon_sym_CARET] = ACTIONS(393),
    [anon_sym_BANG] = ACTIONS(413),
    [anon_sym_AMP] = ACTIONS(393),
    [anon_sym_PIPE] = ACTIONS(393),
    [anon_sym_AMP_AMP] = ACTIONS(391),
    [anon_sym_PIPE_PIPE] = ACTIONS(391),
    [anon_sym_LT_LT] = ACTIONS(393),
    [anon_sym_GT_GT] = ACTIONS(393),
    [anon_sym_PLUS_EQ] = ACTIONS(391),
    [anon_sym_DASH_EQ] = ACTIONS(391),
    [anon_sym_STAR_EQ] = ACTIONS(391),
    [anon_sym_SLASH_EQ] = ACTIONS(391),
    [anon_sym_PERCENT_EQ] = ACTIONS(391),
    [anon_sym_CARET_EQ] = ACTIONS(391),
    [anon_sym_AMP_EQ] = ACTIONS(391),
    [anon_sym_PIPE_EQ] = ACTIONS(391),
    [anon_sym_LT_LT_EQ] = ACTIONS(391),
    [anon_sym_GT_GT_EQ] = ACTIONS(391),
    [anon_sym_EQ] = ACTIONS(393),
    [anon_sym_EQ_EQ] = ACTIONS(391),
    [anon_sym_BANG_EQ] = ACTIONS(391),
    [anon_sym_GT] = ACTIONS(393),
    [anon_sym_LT] = ACTIONS(393),
    [anon_sym_GT_EQ] = ACTIONS(391),
    [anon_sym_LT_EQ] = ACTIONS(391),
    [anon_sym_DOT] = ACTIONS(393),
    [anon_sym_DOT_DOT] = ACTIONS(393),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(391),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(391),
    [anon_sym_COLON_COLON] = ACTIONS(415),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_as] = ACTIONS(393),
    [anon_sym_async] = ACTIONS(417),
    [anon_sym_break] = ACTIONS(419),
    [anon_sym_const] = ACTIONS(421),
    [anon_sym_continue] = ACTIONS(423),
    [anon_sym_default] = ACTIONS(425),
    [anon_sym_for] = ACTIONS(427),
    [anon_sym_gen] = ACTIONS(429),
    [anon_sym_if] = ACTIONS(431),
    [anon_sym_loop] = ACTIONS(433),
    [anon_sym_match] = ACTIONS(435),
    [anon_sym_return] = ACTIONS(437),
    [anon_sym_static] = ACTIONS(439),
    [anon_sym_union] = ACTIONS(425),
    [anon_sym_unsafe] = ACTIONS(441),
    [anon_sym_while] = ACTIONS(443),
    [anon_sym_yield] = ACTIONS(445),
    [anon_sym_move] = ACTIONS(447),
    [anon_sym_try] = ACTIONS(449),
    [sym_integer_literal] = ACTIONS(451),
    [aux_sym_string_literal_token1] = ACTIONS(453),
    [sym_char_literal] = ACTIONS(451),
    [anon_sym_true] = ACTIONS(455),
    [anon_sym_false] = ACTIONS(455),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(457),
    [sym_super] = ACTIONS(459),
    [sym_crate] = ACTIONS(459),
    [sym_metavariable] = ACTIONS(461),
    [sym__raw_string_literal_start] = ACTIONS(463),
    [sym_float_literal] = ACTIONS(451),
  },
  [49] = {
    [sym_bracketed_type] = STATE(3529),
    [sym_generic_function] = STATE(1664),
    [sym_generic_type_with_turbofish] = STATE(2956),
    [sym__expression_except_range] = STATE(1626),
    [sym__expression] = STATE(1863),
    [sym_macro_invocation] = STATE(1671),
    [sym_scoped_identifier] = STATE(1575),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3163),
    [sym_range_expression] = STATE(1666),
    [sym_unary_expression] = STATE(1664),
    [sym_try_expression] = STATE(1664),
    [sym_reference_expression] = STATE(1664),
    [sym_binary_expression] = STATE(1664),
    [sym_assignment_expression] = STATE(1664),
    [sym_compound_assignment_expr] = STATE(1664),
    [sym_type_cast_expression] = STATE(1664),
    [sym_return_expression] = STATE(1664),
    [sym_yield_expression] = STATE(1664),
    [sym_call_expression] = STATE(1664),
    [sym_array_expression] = STATE(1664),
    [sym_parenthesized_expression] = STATE(1664),
    [sym_tuple_expression] = STATE(1664),
    [sym_unit_expression] = STATE(1664),
    [sym_struct_expression] = STATE(1664),
    [sym_if_expression] = STATE(1664),
    [sym_match_expression] = STATE(1664),
    [sym_while_expression] = STATE(1664),
    [sym_loop_expression] = STATE(1664),
    [sym_for_expression] = STATE(1664),
    [sym_const_block] = STATE(1664),
    [sym_closure_expression] = STATE(1664),
    [sym_closure_parameters] = STATE(232),
    [sym_label] = STATE(3614),
    [sym_break_expression] = STATE(1664),
    [sym_continue_expression] = STATE(1664),
    [sym_index_expression] = STATE(1664),
    [sym_await_expression] = STATE(1664),
    [sym_field_expression] = STATE(1597),
    [sym_unsafe_block] = STATE(1664),
    [sym_async_block] = STATE(1664),
    [sym_gen_block] = STATE(1664),
    [sym_try_block] = STATE(1664),
    [sym_block] = STATE(1664),
    [sym__literal] = STATE(1664),
    [sym_string_literal] = STATE(1786),
    [sym_raw_string_literal] = STATE(1786),
    [sym_boolean_literal] = STATE(1786),
    [sym_line_comment] = STATE(49),
    [sym_block_comment] = STATE(49),
    [sym_identifier] = ACTIONS(405),
    [anon_sym_LPAREN] = ACTIONS(495),
    [anon_sym_LBRACK] = ACTIONS(387),
    [anon_sym_LBRACE] = ACTIONS(407),
    [anon_sym_EQ_GT] = ACTIONS(387),
    [anon_sym_PLUS] = ACTIONS(389),
    [anon_sym_STAR] = ACTIONS(389),
    [anon_sym_QMARK] = ACTIONS(387),
    [anon_sym_u8] = ACTIONS(411),
    [anon_sym_i8] = ACTIONS(411),
    [anon_sym_u16] = ACTIONS(411),
    [anon_sym_i16] = ACTIONS(411),
    [anon_sym_u32] = ACTIONS(411),
    [anon_sym_i32] = ACTIONS(411),
    [anon_sym_u64] = ACTIONS(411),
    [anon_sym_i64] = ACTIONS(411),
    [anon_sym_u128] = ACTIONS(411),
    [anon_sym_i128] = ACTIONS(411),
    [anon_sym_isize] = ACTIONS(411),
    [anon_sym_usize] = ACTIONS(411),
    [anon_sym_f32] = ACTIONS(411),
    [anon_sym_f64] = ACTIONS(411),
    [anon_sym_bool] = ACTIONS(411),
    [anon_sym_str] = ACTIONS(411),
    [anon_sym_char] = ACTIONS(411),
    [anon_sym_DASH] = ACTIONS(389),
    [anon_sym_SLASH] = ACTIONS(389),
    [anon_sym_PERCENT] = ACTIONS(389),
    [anon_sym_CARET] = ACTIONS(389),
    [anon_sym_BANG] = ACTIONS(413),
    [anon_sym_AMP] = ACTIONS(389),
    [anon_sym_PIPE] = ACTIONS(389),
    [anon_sym_AMP_AMP] = ACTIONS(387),
    [anon_sym_PIPE_PIPE] = ACTIONS(387),
    [anon_sym_LT_LT] = ACTIONS(389),
    [anon_sym_GT_GT] = ACTIONS(389),
    [anon_sym_PLUS_EQ] = ACTIONS(387),
    [anon_sym_DASH_EQ] = ACTIONS(387),
    [anon_sym_STAR_EQ] = ACTIONS(387),
    [anon_sym_SLASH_EQ] = ACTIONS(387),
    [anon_sym_PERCENT_EQ] = ACTIONS(387),
    [anon_sym_CARET_EQ] = ACTIONS(387),
    [anon_sym_AMP_EQ] = ACTIONS(387),
    [anon_sym_PIPE_EQ] = ACTIONS(387),
    [anon_sym_LT_LT_EQ] = ACTIONS(387),
    [anon_sym_GT_GT_EQ] = ACTIONS(387),
    [anon_sym_EQ] = ACTIONS(389),
    [anon_sym_EQ_EQ] = ACTIONS(387),
    [anon_sym_BANG_EQ] = ACTIONS(387),
    [anon_sym_GT] = ACTIONS(389),
    [anon_sym_LT] = ACTIONS(389),
    [anon_sym_GT_EQ] = ACTIONS(387),
    [anon_sym_LT_EQ] = ACTIONS(387),
    [anon_sym_DOT] = ACTIONS(389),
    [anon_sym_DOT_DOT] = ACTIONS(389),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(387),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(387),
    [anon_sym_COLON_COLON] = ACTIONS(415),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_as] = ACTIONS(389),
    [anon_sym_async] = ACTIONS(417),
    [anon_sym_break] = ACTIONS(419),
    [anon_sym_const] = ACTIONS(421),
    [anon_sym_continue] = ACTIONS(423),
    [anon_sym_default] = ACTIONS(425),
    [anon_sym_for] = ACTIONS(427),
    [anon_sym_gen] = ACTIONS(429),
    [anon_sym_if] = ACTIONS(431),
    [anon_sym_loop] = ACTIONS(433),
    [anon_sym_match] = ACTIONS(435),
    [anon_sym_return] = ACTIONS(437),
    [anon_sym_static] = ACTIONS(439),
    [anon_sym_union] = ACTIONS(425),
    [anon_sym_unsafe] = ACTIONS(441),
    [anon_sym_while] = ACTIONS(443),
    [anon_sym_yield] = ACTIONS(445),
    [anon_sym_move] = ACTIONS(447),
    [anon_sym_try] = ACTIONS(449),
    [sym_integer_literal] = ACTIONS(451),
    [aux_sym_string_literal_token1] = ACTIONS(453),
    [sym_char_literal] = ACTIONS(451),
    [anon_sym_true] = ACTIONS(455),
    [anon_sym_false] = ACTIONS(455),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(457),
    [sym_super] = ACTIONS(459),
    [sym_crate] = ACTIONS(459),
    [sym_metavariable] = ACTIONS(461),
    [sym__raw_string_literal_start] = ACTIONS(463),
    [sym_float_literal] = ACTIONS(451),
  },
  [50] = {
    [sym_bracketed_type] = STATE(3529),
    [sym_generic_function] = STATE(1664),
    [sym_generic_type_with_turbofish] = STATE(2956),
    [sym__expression_except_range] = STATE(1626),
    [sym__expression] = STATE(1680),
    [sym_macro_invocation] = STATE(1671),
    [sym_scoped_identifier] = STATE(1575),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3163),
    [sym_range_expression] = STATE(1666),
    [sym_unary_expression] = STATE(1664),
    [sym_try_expression] = STATE(1664),
    [sym_reference_expression] = STATE(1664),
    [sym_binary_expression] = STATE(1664),
    [sym_assignment_expression] = STATE(1664),
    [sym_compound_assignment_expr] = STATE(1664),
    [sym_type_cast_expression] = STATE(1664),
    [sym_return_expression] = STATE(1664),
    [sym_yield_expression] = STATE(1664),
    [sym_call_expression] = STATE(1664),
    [sym_array_expression] = STATE(1664),
    [sym_parenthesized_expression] = STATE(1664),
    [sym_tuple_expression] = STATE(1664),
    [sym_unit_expression] = STATE(1664),
    [sym_struct_expression] = STATE(1664),
    [sym_if_expression] = STATE(1664),
    [sym_match_expression] = STATE(1664),
    [sym_while_expression] = STATE(1664),
    [sym_loop_expression] = STATE(1664),
    [sym_for_expression] = STATE(1664),
    [sym_const_block] = STATE(1664),
    [sym_closure_expression] = STATE(1664),
    [sym_closure_parameters] = STATE(232),
    [sym_label] = STATE(3614),
    [sym_break_expression] = STATE(1664),
    [sym_continue_expression] = STATE(1664),
    [sym_index_expression] = STATE(1664),
    [sym_await_expression] = STATE(1664),
    [sym_field_expression] = STATE(1597),
    [sym_unsafe_block] = STATE(1664),
    [sym_async_block] = STATE(1664),
    [sym_gen_block] = STATE(1664),
    [sym_try_block] = STATE(1664),
    [sym_block] = STATE(1664),
    [sym__literal] = STATE(1664),
    [sym_string_literal] = STATE(1786),
    [sym_raw_string_literal] = STATE(1786),
    [sym_boolean_literal] = STATE(1786),
    [sym_line_comment] = STATE(50),
    [sym_block_comment] = STATE(50),
    [sym_identifier] = ACTIONS(405),
    [anon_sym_LPAREN] = ACTIONS(391),
    [anon_sym_LBRACK] = ACTIONS(391),
    [anon_sym_LBRACE] = ACTIONS(407),
    [anon_sym_EQ_GT] = ACTIONS(391),
    [anon_sym_PLUS] = ACTIONS(393),
    [anon_sym_STAR] = ACTIONS(393),
    [anon_sym_QMARK] = ACTIONS(391),
    [anon_sym_u8] = ACTIONS(411),
    [anon_sym_i8] = ACTIONS(411),
    [anon_sym_u16] = ACTIONS(411),
    [anon_sym_i16] = ACTIONS(411),
    [anon_sym_u32] = ACTIONS(411),
    [anon_sym_i32] = ACTIONS(411),
    [anon_sym_u64] = ACTIONS(411),
    [anon_sym_i64] = ACTIONS(411),
    [anon_sym_u128] = ACTIONS(411),
    [anon_sym_i128] = ACTIONS(411),
    [anon_sym_isize] = ACTIONS(411),
    [anon_sym_usize] = ACTIONS(411),
    [anon_sym_f32] = ACTIONS(411),
    [anon_sym_f64] = ACTIONS(411),
    [anon_sym_bool] = ACTIONS(411),
    [anon_sym_str] = ACTIONS(411),
    [anon_sym_char] = ACTIONS(411),
    [anon_sym_DASH] = ACTIONS(393),
    [anon_sym_SLASH] = ACTIONS(393),
    [anon_sym_PERCENT] = ACTIONS(393),
    [anon_sym_CARET] = ACTIONS(393),
    [anon_sym_BANG] = ACTIONS(413),
    [anon_sym_AMP] = ACTIONS(393),
    [anon_sym_PIPE] = ACTIONS(393),
    [anon_sym_AMP_AMP] = ACTIONS(391),
    [anon_sym_PIPE_PIPE] = ACTIONS(391),
    [anon_sym_LT_LT] = ACTIONS(393),
    [anon_sym_GT_GT] = ACTIONS(393),
    [anon_sym_PLUS_EQ] = ACTIONS(391),
    [anon_sym_DASH_EQ] = ACTIONS(391),
    [anon_sym_STAR_EQ] = ACTIONS(391),
    [anon_sym_SLASH_EQ] = ACTIONS(391),
    [anon_sym_PERCENT_EQ] = ACTIONS(391),
    [anon_sym_CARET_EQ] = ACTIONS(391),
    [anon_sym_AMP_EQ] = ACTIONS(391),
    [anon_sym_PIPE_EQ] = ACTIONS(391),
    [anon_sym_LT_LT_EQ] = ACTIONS(391),
    [anon_sym_GT_GT_EQ] = ACTIONS(391),
    [anon_sym_EQ] = ACTIONS(393),
    [anon_sym_EQ_EQ] = ACTIONS(391),
    [anon_sym_BANG_EQ] = ACTIONS(391),
    [anon_sym_GT] = ACTIONS(393),
    [anon_sym_LT] = ACTIONS(393),
    [anon_sym_GT_EQ] = ACTIONS(391),
    [anon_sym_LT_EQ] = ACTIONS(391),
    [anon_sym_DOT] = ACTIONS(393),
    [anon_sym_DOT_DOT] = ACTIONS(393),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(391),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(391),
    [anon_sym_COLON_COLON] = ACTIONS(415),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_as] = ACTIONS(393),
    [anon_sym_async] = ACTIONS(417),
    [anon_sym_break] = ACTIONS(419),
    [anon_sym_const] = ACTIONS(421),
    [anon_sym_continue] = ACTIONS(423),
    [anon_sym_default] = ACTIONS(425),
    [anon_sym_for] = ACTIONS(427),
    [anon_sym_gen] = ACTIONS(429),
    [anon_sym_if] = ACTIONS(431),
    [anon_sym_loop] = ACTIONS(433),
    [anon_sym_match] = ACTIONS(435),
    [anon_sym_return] = ACTIONS(437),
    [anon_sym_static] = ACTIONS(439),
    [anon_sym_union] = ACTIONS(425),
    [anon_sym_unsafe] = ACTIONS(441),
    [anon_sym_while] = ACTIONS(443),
    [anon_sym_yield] = ACTIONS(445),
    [anon_sym_move] = ACTIONS(447),
    [anon_sym_try] = ACTIONS(449),
    [sym_integer_literal] = ACTIONS(451),
    [aux_sym_string_literal_token1] = ACTIONS(453),
    [sym_char_literal] = ACTIONS(451),
    [anon_sym_true] = ACTIONS(455),
    [anon_sym_false] = ACTIONS(455),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(457),
    [sym_super] = ACTIONS(459),
    [sym_crate] = ACTIONS(459),
    [sym_metavariable] = ACTIONS(461),
    [sym__raw_string_literal_start] = ACTIONS(463),
    [sym_float_literal] = ACTIONS(451),
  },
  [51] = {
    [sym_bracketed_type] = STATE(3529),
    [sym_generic_function] = STATE(1664),
    [sym_generic_type_with_turbofish] = STATE(2956),
    [sym__expression_except_range] = STATE(1626),
    [sym__expression] = STATE(1863),
    [sym_macro_invocation] = STATE(1671),
    [sym_scoped_identifier] = STATE(1575),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3163),
    [sym_range_expression] = STATE(1666),
    [sym_unary_expression] = STATE(1664),
    [sym_try_expression] = STATE(1664),
    [sym_reference_expression] = STATE(1664),
    [sym_binary_expression] = STATE(1664),
    [sym_assignment_expression] = STATE(1664),
    [sym_compound_assignment_expr] = STATE(1664),
    [sym_type_cast_expression] = STATE(1664),
    [sym_return_expression] = STATE(1664),
    [sym_yield_expression] = STATE(1664),
    [sym_call_expression] = STATE(1664),
    [sym_array_expression] = STATE(1664),
    [sym_parenthesized_expression] = STATE(1664),
    [sym_tuple_expression] = STATE(1664),
    [sym_unit_expression] = STATE(1664),
    [sym_struct_expression] = STATE(1664),
    [sym_if_expression] = STATE(1664),
    [sym_match_expression] = STATE(1664),
    [sym_while_expression] = STATE(1664),
    [sym_loop_expression] = STATE(1664),
    [sym_for_expression] = STATE(1664),
    [sym_const_block] = STATE(1664),
    [sym_closure_expression] = STATE(1664),
    [sym_closure_parameters] = STATE(232),
    [sym_label] = STATE(3614),
    [sym_break_expression] = STATE(1664),
    [sym_continue_expression] = STATE(1664),
    [sym_index_expression] = STATE(1664),
    [sym_await_expression] = STATE(1664),
    [sym_field_expression] = STATE(1597),
    [sym_unsafe_block] = STATE(1664),
    [sym_async_block] = STATE(1664),
    [sym_gen_block] = STATE(1664),
    [sym_try_block] = STATE(1664),
    [sym_block] = STATE(1664),
    [sym__literal] = STATE(1664),
    [sym_string_literal] = STATE(1786),
    [sym_raw_string_literal] = STATE(1786),
    [sym_boolean_literal] = STATE(1786),
    [sym_line_comment] = STATE(51),
    [sym_block_comment] = STATE(51),
    [sym_identifier] = ACTIONS(405),
    [anon_sym_LPAREN] = ACTIONS(387),
    [anon_sym_LBRACK] = ACTIONS(387),
    [anon_sym_LBRACE] = ACTIONS(407),
    [anon_sym_EQ_GT] = ACTIONS(387),
    [anon_sym_PLUS] = ACTIONS(389),
    [anon_sym_STAR] = ACTIONS(389),
    [anon_sym_QMARK] = ACTIONS(387),
    [anon_sym_u8] = ACTIONS(411),
    [anon_sym_i8] = ACTIONS(411),
    [anon_sym_u16] = ACTIONS(411),
    [anon_sym_i16] = ACTIONS(411),
    [anon_sym_u32] = ACTIONS(411),
    [anon_sym_i32] = ACTIONS(411),
    [anon_sym_u64] = ACTIONS(411),
    [anon_sym_i64] = ACTIONS(411),
    [anon_sym_u128] = ACTIONS(411),
    [anon_sym_i128] = ACTIONS(411),
    [anon_sym_isize] = ACTIONS(411),
    [anon_sym_usize] = ACTIONS(411),
    [anon_sym_f32] = ACTIONS(411),
    [anon_sym_f64] = ACTIONS(411),
    [anon_sym_bool] = ACTIONS(411),
    [anon_sym_str] = ACTIONS(411),
    [anon_sym_char] = ACTIONS(411),
    [anon_sym_DASH] = ACTIONS(389),
    [anon_sym_SLASH] = ACTIONS(389),
    [anon_sym_PERCENT] = ACTIONS(389),
    [anon_sym_CARET] = ACTIONS(389),
    [anon_sym_BANG] = ACTIONS(413),
    [anon_sym_AMP] = ACTIONS(389),
    [anon_sym_PIPE] = ACTIONS(389),
    [anon_sym_AMP_AMP] = ACTIONS(387),
    [anon_sym_PIPE_PIPE] = ACTIONS(387),
    [anon_sym_LT_LT] = ACTIONS(389),
    [anon_sym_GT_GT] = ACTIONS(389),
    [anon_sym_PLUS_EQ] = ACTIONS(387),
    [anon_sym_DASH_EQ] = ACTIONS(387),
    [anon_sym_STAR_EQ] = ACTIONS(387),
    [anon_sym_SLASH_EQ] = ACTIONS(387),
    [anon_sym_PERCENT_EQ] = ACTIONS(387),
    [anon_sym_CARET_EQ] = ACTIONS(387),
    [anon_sym_AMP_EQ] = ACTIONS(387),
    [anon_sym_PIPE_EQ] = ACTIONS(387),
    [anon_sym_LT_LT_EQ] = ACTIONS(387),
    [anon_sym_GT_GT_EQ] = ACTIONS(387),
    [anon_sym_EQ] = ACTIONS(389),
    [anon_sym_EQ_EQ] = ACTIONS(387),
    [anon_sym_BANG_EQ] = ACTIONS(387),
    [anon_sym_GT] = ACTIONS(389),
    [anon_sym_LT] = ACTIONS(389),
    [anon_sym_GT_EQ] = ACTIONS(387),
    [anon_sym_LT_EQ] = ACTIONS(387),
    [anon_sym_DOT] = ACTIONS(389),
    [anon_sym_DOT_DOT] = ACTIONS(389),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(387),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(387),
    [anon_sym_COLON_COLON] = ACTIONS(415),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_as] = ACTIONS(389),
    [anon_sym_async] = ACTIONS(417),
    [anon_sym_break] = ACTIONS(419),
    [anon_sym_const] = ACTIONS(421),
    [anon_sym_continue] = ACTIONS(423),
    [anon_sym_default] = ACTIONS(425),
    [anon_sym_for] = ACTIONS(427),
    [anon_sym_gen] = ACTIONS(429),
    [anon_sym_if] = ACTIONS(431),
    [anon_sym_loop] = ACTIONS(433),
    [anon_sym_match] = ACTIONS(435),
    [anon_sym_return] = ACTIONS(437),
    [anon_sym_static] = ACTIONS(439),
    [anon_sym_union] = ACTIONS(425),
    [anon_sym_unsafe] = ACTIONS(441),
    [anon_sym_while] = ACTIONS(443),
    [anon_sym_yield] = ACTIONS(445),
    [anon_sym_move] = ACTIONS(447),
    [anon_sym_try] = ACTIONS(449),
    [sym_integer_literal] = ACTIONS(451),
    [aux_sym_string_literal_token1] = ACTIONS(453),
    [sym_char_literal] = ACTIONS(451),
    [anon_sym_true] = ACTIONS(455),
    [anon_sym_false] = ACTIONS(455),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(457),
    [sym_super] = ACTIONS(459),
    [sym_crate] = ACTIONS(459),
    [sym_metavariable] = ACTIONS(461),
    [sym__raw_string_literal_start] = ACTIONS(463),
    [sym_float_literal] = ACTIONS(451),
  },
  [52] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1621),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(52),
    [sym_block_comment] = STATE(52),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(341),
    [anon_sym_LBRACK] = ACTIONS(341),
    [anon_sym_LBRACE] = ACTIONS(341),
    [anon_sym_COLON] = ACTIONS(345),
    [anon_sym_PLUS] = ACTIONS(347),
    [anon_sym_STAR] = ACTIONS(347),
    [anon_sym_QMARK] = ACTIONS(341),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(347),
    [anon_sym_SLASH] = ACTIONS(347),
    [anon_sym_PERCENT] = ACTIONS(347),
    [anon_sym_CARET] = ACTIONS(347),
    [anon_sym_BANG] = ACTIONS(503),
    [anon_sym_AMP] = ACTIONS(347),
    [anon_sym_PIPE] = ACTIONS(347),
    [anon_sym_AMP_AMP] = ACTIONS(341),
    [anon_sym_PIPE_PIPE] = ACTIONS(341),
    [anon_sym_LT_LT] = ACTIONS(347),
    [anon_sym_GT_GT] = ACTIONS(347),
    [anon_sym_PLUS_EQ] = ACTIONS(341),
    [anon_sym_DASH_EQ] = ACTIONS(341),
    [anon_sym_STAR_EQ] = ACTIONS(341),
    [anon_sym_SLASH_EQ] = ACTIONS(341),
    [anon_sym_PERCENT_EQ] = ACTIONS(341),
    [anon_sym_CARET_EQ] = ACTIONS(341),
    [anon_sym_AMP_EQ] = ACTIONS(341),
    [anon_sym_PIPE_EQ] = ACTIONS(341),
    [anon_sym_LT_LT_EQ] = ACTIONS(341),
    [anon_sym_GT_GT_EQ] = ACTIONS(341),
    [anon_sym_EQ] = ACTIONS(347),
    [anon_sym_EQ_EQ] = ACTIONS(341),
    [anon_sym_BANG_EQ] = ACTIONS(341),
    [anon_sym_GT] = ACTIONS(347),
    [anon_sym_LT] = ACTIONS(347),
    [anon_sym_GT_EQ] = ACTIONS(341),
    [anon_sym_LT_EQ] = ACTIONS(341),
    [anon_sym_DOT] = ACTIONS(347),
    [anon_sym_DOT_DOT] = ACTIONS(347),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(341),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(341),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(347),
    [anon_sym_as] = ACTIONS(347),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [53] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1609),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(53),
    [sym_block_comment] = STATE(53),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_PLUS] = ACTIONS(377),
    [anon_sym_STAR] = ACTIONS(503),
    [anon_sym_QMARK] = ACTIONS(375),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(503),
    [anon_sym_SLASH] = ACTIONS(377),
    [anon_sym_PERCENT] = ACTIONS(377),
    [anon_sym_CARET] = ACTIONS(377),
    [anon_sym_BANG] = ACTIONS(503),
    [anon_sym_AMP] = ACTIONS(519),
    [anon_sym_PIPE] = ACTIONS(381),
    [anon_sym_AMP_AMP] = ACTIONS(375),
    [anon_sym_PIPE_PIPE] = ACTIONS(375),
    [anon_sym_LT_LT] = ACTIONS(377),
    [anon_sym_GT_GT] = ACTIONS(377),
    [anon_sym_PLUS_EQ] = ACTIONS(375),
    [anon_sym_DASH_EQ] = ACTIONS(375),
    [anon_sym_STAR_EQ] = ACTIONS(375),
    [anon_sym_SLASH_EQ] = ACTIONS(375),
    [anon_sym_PERCENT_EQ] = ACTIONS(375),
    [anon_sym_CARET_EQ] = ACTIONS(375),
    [anon_sym_AMP_EQ] = ACTIONS(375),
    [anon_sym_PIPE_EQ] = ACTIONS(375),
    [anon_sym_LT_LT_EQ] = ACTIONS(375),
    [anon_sym_GT_GT_EQ] = ACTIONS(375),
    [anon_sym_EQ] = ACTIONS(377),
    [anon_sym_EQ_EQ] = ACTIONS(375),
    [anon_sym_BANG_EQ] = ACTIONS(375),
    [anon_sym_GT] = ACTIONS(377),
    [anon_sym_LT] = ACTIONS(383),
    [anon_sym_GT_EQ] = ACTIONS(375),
    [anon_sym_LT_EQ] = ACTIONS(375),
    [anon_sym_DOT] = ACTIONS(377),
    [anon_sym_DOT_DOT] = ACTIONS(521),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(375),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(375),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_as] = ACTIONS(377),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [54] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1718),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(246),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(54),
    [sym_block_comment] = STATE(54),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(387),
    [anon_sym_LBRACK] = ACTIONS(387),
    [anon_sym_LBRACE] = ACTIONS(387),
    [anon_sym_PLUS] = ACTIONS(389),
    [anon_sym_STAR] = ACTIONS(389),
    [anon_sym_QMARK] = ACTIONS(387),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(389),
    [anon_sym_SLASH] = ACTIONS(389),
    [anon_sym_PERCENT] = ACTIONS(389),
    [anon_sym_CARET] = ACTIONS(389),
    [anon_sym_BANG] = ACTIONS(471),
    [anon_sym_AMP] = ACTIONS(389),
    [anon_sym_PIPE] = ACTIONS(389),
    [anon_sym_AMP_AMP] = ACTIONS(387),
    [anon_sym_PIPE_PIPE] = ACTIONS(387),
    [anon_sym_LT_LT] = ACTIONS(389),
    [anon_sym_GT_GT] = ACTIONS(389),
    [anon_sym_PLUS_EQ] = ACTIONS(387),
    [anon_sym_DASH_EQ] = ACTIONS(387),
    [anon_sym_STAR_EQ] = ACTIONS(387),
    [anon_sym_SLASH_EQ] = ACTIONS(387),
    [anon_sym_PERCENT_EQ] = ACTIONS(387),
    [anon_sym_CARET_EQ] = ACTIONS(387),
    [anon_sym_AMP_EQ] = ACTIONS(387),
    [anon_sym_PIPE_EQ] = ACTIONS(387),
    [anon_sym_LT_LT_EQ] = ACTIONS(387),
    [anon_sym_GT_GT_EQ] = ACTIONS(387),
    [anon_sym_EQ] = ACTIONS(389),
    [anon_sym_EQ_EQ] = ACTIONS(387),
    [anon_sym_BANG_EQ] = ACTIONS(387),
    [anon_sym_GT] = ACTIONS(389),
    [anon_sym_LT] = ACTIONS(389),
    [anon_sym_GT_EQ] = ACTIONS(387),
    [anon_sym_LT_EQ] = ACTIONS(387),
    [anon_sym_DOT] = ACTIONS(389),
    [anon_sym_DOT_DOT] = ACTIONS(389),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(387),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(387),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_as] = ACTIONS(389),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(475),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(479),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(481),
    [anon_sym_static] = ACTIONS(483),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(485),
    [anon_sym_move] = ACTIONS(487),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [55] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1595),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(55),
    [sym_block_comment] = STATE(55),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(387),
    [anon_sym_LBRACE] = ACTIONS(387),
    [anon_sym_PLUS] = ACTIONS(389),
    [anon_sym_STAR] = ACTIONS(389),
    [anon_sym_QMARK] = ACTIONS(387),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(389),
    [anon_sym_SLASH] = ACTIONS(389),
    [anon_sym_PERCENT] = ACTIONS(389),
    [anon_sym_CARET] = ACTIONS(389),
    [anon_sym_BANG] = ACTIONS(503),
    [anon_sym_AMP] = ACTIONS(389),
    [anon_sym_PIPE] = ACTIONS(389),
    [anon_sym_AMP_AMP] = ACTIONS(387),
    [anon_sym_PIPE_PIPE] = ACTIONS(387),
    [anon_sym_LT_LT] = ACTIONS(389),
    [anon_sym_GT_GT] = ACTIONS(389),
    [anon_sym_PLUS_EQ] = ACTIONS(387),
    [anon_sym_DASH_EQ] = ACTIONS(387),
    [anon_sym_STAR_EQ] = ACTIONS(387),
    [anon_sym_SLASH_EQ] = ACTIONS(387),
    [anon_sym_PERCENT_EQ] = ACTIONS(387),
    [anon_sym_CARET_EQ] = ACTIONS(387),
    [anon_sym_AMP_EQ] = ACTIONS(387),
    [anon_sym_PIPE_EQ] = ACTIONS(387),
    [anon_sym_LT_LT_EQ] = ACTIONS(387),
    [anon_sym_GT_GT_EQ] = ACTIONS(387),
    [anon_sym_EQ] = ACTIONS(389),
    [anon_sym_EQ_EQ] = ACTIONS(387),
    [anon_sym_BANG_EQ] = ACTIONS(387),
    [anon_sym_GT] = ACTIONS(389),
    [anon_sym_LT] = ACTIONS(389),
    [anon_sym_GT_EQ] = ACTIONS(387),
    [anon_sym_LT_EQ] = ACTIONS(387),
    [anon_sym_DOT] = ACTIONS(389),
    [anon_sym_DOT_DOT] = ACTIONS(389),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(387),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(387),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(389),
    [anon_sym_as] = ACTIONS(389),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [56] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1824),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(246),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(56),
    [sym_block_comment] = STATE(56),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(391),
    [anon_sym_LBRACE] = ACTIONS(391),
    [anon_sym_PLUS] = ACTIONS(393),
    [anon_sym_STAR] = ACTIONS(393),
    [anon_sym_QMARK] = ACTIONS(391),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(393),
    [anon_sym_SLASH] = ACTIONS(393),
    [anon_sym_PERCENT] = ACTIONS(393),
    [anon_sym_CARET] = ACTIONS(393),
    [anon_sym_BANG] = ACTIONS(471),
    [anon_sym_AMP] = ACTIONS(393),
    [anon_sym_PIPE] = ACTIONS(393),
    [anon_sym_AMP_AMP] = ACTIONS(391),
    [anon_sym_PIPE_PIPE] = ACTIONS(391),
    [anon_sym_LT_LT] = ACTIONS(393),
    [anon_sym_GT_GT] = ACTIONS(393),
    [anon_sym_PLUS_EQ] = ACTIONS(391),
    [anon_sym_DASH_EQ] = ACTIONS(391),
    [anon_sym_STAR_EQ] = ACTIONS(391),
    [anon_sym_SLASH_EQ] = ACTIONS(391),
    [anon_sym_PERCENT_EQ] = ACTIONS(391),
    [anon_sym_CARET_EQ] = ACTIONS(391),
    [anon_sym_AMP_EQ] = ACTIONS(391),
    [anon_sym_PIPE_EQ] = ACTIONS(391),
    [anon_sym_LT_LT_EQ] = ACTIONS(391),
    [anon_sym_GT_GT_EQ] = ACTIONS(391),
    [anon_sym_EQ] = ACTIONS(393),
    [anon_sym_EQ_EQ] = ACTIONS(391),
    [anon_sym_BANG_EQ] = ACTIONS(391),
    [anon_sym_GT] = ACTIONS(393),
    [anon_sym_LT] = ACTIONS(393),
    [anon_sym_GT_EQ] = ACTIONS(391),
    [anon_sym_LT_EQ] = ACTIONS(391),
    [anon_sym_DOT] = ACTIONS(393),
    [anon_sym_DOT_DOT] = ACTIONS(393),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(391),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(391),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_as] = ACTIONS(393),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(475),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(479),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(481),
    [anon_sym_static] = ACTIONS(483),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(485),
    [anon_sym_move] = ACTIONS(487),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [57] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1825),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(246),
    [sym_label] = STATE(45),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(57),
    [sym_block_comment] = STATE(57),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(395),
    [anon_sym_LBRACK] = ACTIONS(395),
    [anon_sym_LBRACE] = ACTIONS(395),
    [anon_sym_PLUS] = ACTIONS(397),
    [anon_sym_STAR] = ACTIONS(397),
    [anon_sym_QMARK] = ACTIONS(395),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(397),
    [anon_sym_SLASH] = ACTIONS(397),
    [anon_sym_PERCENT] = ACTIONS(397),
    [anon_sym_CARET] = ACTIONS(397),
    [anon_sym_BANG] = ACTIONS(471),
    [anon_sym_AMP] = ACTIONS(397),
    [anon_sym_PIPE] = ACTIONS(397),
    [anon_sym_AMP_AMP] = ACTIONS(395),
    [anon_sym_PIPE_PIPE] = ACTIONS(395),
    [anon_sym_LT_LT] = ACTIONS(397),
    [anon_sym_GT_GT] = ACTIONS(397),
    [anon_sym_PLUS_EQ] = ACTIONS(395),
    [anon_sym_DASH_EQ] = ACTIONS(395),
    [anon_sym_STAR_EQ] = ACTIONS(395),
    [anon_sym_SLASH_EQ] = ACTIONS(395),
    [anon_sym_PERCENT_EQ] = ACTIONS(395),
    [anon_sym_CARET_EQ] = ACTIONS(395),
    [anon_sym_AMP_EQ] = ACTIONS(395),
    [anon_sym_PIPE_EQ] = ACTIONS(395),
    [anon_sym_LT_LT_EQ] = ACTIONS(395),
    [anon_sym_GT_GT_EQ] = ACTIONS(395),
    [anon_sym_EQ] = ACTIONS(397),
    [anon_sym_EQ_EQ] = ACTIONS(395),
    [anon_sym_BANG_EQ] = ACTIONS(395),
    [anon_sym_GT] = ACTIONS(397),
    [anon_sym_LT] = ACTIONS(397),
    [anon_sym_GT_EQ] = ACTIONS(395),
    [anon_sym_LT_EQ] = ACTIONS(395),
    [anon_sym_DOT] = ACTIONS(397),
    [anon_sym_DOT_DOT] = ACTIONS(397),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(395),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(395),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(399),
    [anon_sym_as] = ACTIONS(397),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(475),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(479),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(481),
    [anon_sym_static] = ACTIONS(483),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(485),
    [anon_sym_move] = ACTIONS(487),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [58] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1824),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(246),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(58),
    [sym_block_comment] = STATE(58),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(391),
    [anon_sym_LBRACK] = ACTIONS(391),
    [anon_sym_LBRACE] = ACTIONS(391),
    [anon_sym_PLUS] = ACTIONS(393),
    [anon_sym_STAR] = ACTIONS(393),
    [anon_sym_QMARK] = ACTIONS(391),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(393),
    [anon_sym_SLASH] = ACTIONS(393),
    [anon_sym_PERCENT] = ACTIONS(393),
    [anon_sym_CARET] = ACTIONS(393),
    [anon_sym_BANG] = ACTIONS(471),
    [anon_sym_AMP] = ACTIONS(393),
    [anon_sym_PIPE] = ACTIONS(393),
    [anon_sym_AMP_AMP] = ACTIONS(391),
    [anon_sym_PIPE_PIPE] = ACTIONS(391),
    [anon_sym_LT_LT] = ACTIONS(393),
    [anon_sym_GT_GT] = ACTIONS(393),
    [anon_sym_PLUS_EQ] = ACTIONS(391),
    [anon_sym_DASH_EQ] = ACTIONS(391),
    [anon_sym_STAR_EQ] = ACTIONS(391),
    [anon_sym_SLASH_EQ] = ACTIONS(391),
    [anon_sym_PERCENT_EQ] = ACTIONS(391),
    [anon_sym_CARET_EQ] = ACTIONS(391),
    [anon_sym_AMP_EQ] = ACTIONS(391),
    [anon_sym_PIPE_EQ] = ACTIONS(391),
    [anon_sym_LT_LT_EQ] = ACTIONS(391),
    [anon_sym_GT_GT_EQ] = ACTIONS(391),
    [anon_sym_EQ] = ACTIONS(393),
    [anon_sym_EQ_EQ] = ACTIONS(391),
    [anon_sym_BANG_EQ] = ACTIONS(391),
    [anon_sym_GT] = ACTIONS(393),
    [anon_sym_LT] = ACTIONS(393),
    [anon_sym_GT_EQ] = ACTIONS(391),
    [anon_sym_LT_EQ] = ACTIONS(391),
    [anon_sym_DOT] = ACTIONS(393),
    [anon_sym_DOT_DOT] = ACTIONS(393),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(391),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(391),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_as] = ACTIONS(393),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(475),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(479),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(481),
    [anon_sym_static] = ACTIONS(483),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(485),
    [anon_sym_move] = ACTIONS(487),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [59] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1595),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(59),
    [sym_block_comment] = STATE(59),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(387),
    [anon_sym_LBRACK] = ACTIONS(387),
    [anon_sym_LBRACE] = ACTIONS(387),
    [anon_sym_PLUS] = ACTIONS(389),
    [anon_sym_STAR] = ACTIONS(389),
    [anon_sym_QMARK] = ACTIONS(387),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(389),
    [anon_sym_SLASH] = ACTIONS(389),
    [anon_sym_PERCENT] = ACTIONS(389),
    [anon_sym_CARET] = ACTIONS(389),
    [anon_sym_BANG] = ACTIONS(503),
    [anon_sym_AMP] = ACTIONS(389),
    [anon_sym_PIPE] = ACTIONS(389),
    [anon_sym_AMP_AMP] = ACTIONS(387),
    [anon_sym_PIPE_PIPE] = ACTIONS(387),
    [anon_sym_LT_LT] = ACTIONS(389),
    [anon_sym_GT_GT] = ACTIONS(389),
    [anon_sym_PLUS_EQ] = ACTIONS(387),
    [anon_sym_DASH_EQ] = ACTIONS(387),
    [anon_sym_STAR_EQ] = ACTIONS(387),
    [anon_sym_SLASH_EQ] = ACTIONS(387),
    [anon_sym_PERCENT_EQ] = ACTIONS(387),
    [anon_sym_CARET_EQ] = ACTIONS(387),
    [anon_sym_AMP_EQ] = ACTIONS(387),
    [anon_sym_PIPE_EQ] = ACTIONS(387),
    [anon_sym_LT_LT_EQ] = ACTIONS(387),
    [anon_sym_GT_GT_EQ] = ACTIONS(387),
    [anon_sym_EQ] = ACTIONS(389),
    [anon_sym_EQ_EQ] = ACTIONS(387),
    [anon_sym_BANG_EQ] = ACTIONS(387),
    [anon_sym_GT] = ACTIONS(389),
    [anon_sym_LT] = ACTIONS(389),
    [anon_sym_GT_EQ] = ACTIONS(387),
    [anon_sym_LT_EQ] = ACTIONS(387),
    [anon_sym_DOT] = ACTIONS(389),
    [anon_sym_DOT_DOT] = ACTIONS(389),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(387),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(387),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(389),
    [anon_sym_as] = ACTIONS(389),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [60] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1608),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(60),
    [sym_block_comment] = STATE(60),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_PLUS] = ACTIONS(403),
    [anon_sym_STAR] = ACTIONS(503),
    [anon_sym_QMARK] = ACTIONS(401),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(503),
    [anon_sym_SLASH] = ACTIONS(403),
    [anon_sym_PERCENT] = ACTIONS(403),
    [anon_sym_CARET] = ACTIONS(403),
    [anon_sym_BANG] = ACTIONS(503),
    [anon_sym_AMP] = ACTIONS(519),
    [anon_sym_PIPE] = ACTIONS(381),
    [anon_sym_AMP_AMP] = ACTIONS(401),
    [anon_sym_PIPE_PIPE] = ACTIONS(401),
    [anon_sym_LT_LT] = ACTIONS(403),
    [anon_sym_GT_GT] = ACTIONS(403),
    [anon_sym_PLUS_EQ] = ACTIONS(401),
    [anon_sym_DASH_EQ] = ACTIONS(401),
    [anon_sym_STAR_EQ] = ACTIONS(401),
    [anon_sym_SLASH_EQ] = ACTIONS(401),
    [anon_sym_PERCENT_EQ] = ACTIONS(401),
    [anon_sym_CARET_EQ] = ACTIONS(401),
    [anon_sym_AMP_EQ] = ACTIONS(401),
    [anon_sym_PIPE_EQ] = ACTIONS(401),
    [anon_sym_LT_LT_EQ] = ACTIONS(401),
    [anon_sym_GT_GT_EQ] = ACTIONS(401),
    [anon_sym_EQ] = ACTIONS(403),
    [anon_sym_EQ_EQ] = ACTIONS(401),
    [anon_sym_BANG_EQ] = ACTIONS(401),
    [anon_sym_GT] = ACTIONS(403),
    [anon_sym_LT] = ACTIONS(383),
    [anon_sym_GT_EQ] = ACTIONS(401),
    [anon_sym_LT_EQ] = ACTIONS(401),
    [anon_sym_DOT] = ACTIONS(403),
    [anon_sym_DOT_DOT] = ACTIONS(521),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(401),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(401),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_as] = ACTIONS(403),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [61] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1606),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(61),
    [sym_block_comment] = STATE(61),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(391),
    [anon_sym_LBRACE] = ACTIONS(391),
    [anon_sym_PLUS] = ACTIONS(393),
    [anon_sym_STAR] = ACTIONS(393),
    [anon_sym_QMARK] = ACTIONS(391),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(393),
    [anon_sym_SLASH] = ACTIONS(393),
    [anon_sym_PERCENT] = ACTIONS(393),
    [anon_sym_CARET] = ACTIONS(393),
    [anon_sym_BANG] = ACTIONS(503),
    [anon_sym_AMP] = ACTIONS(393),
    [anon_sym_PIPE] = ACTIONS(393),
    [anon_sym_AMP_AMP] = ACTIONS(391),
    [anon_sym_PIPE_PIPE] = ACTIONS(391),
    [anon_sym_LT_LT] = ACTIONS(393),
    [anon_sym_GT_GT] = ACTIONS(393),
    [anon_sym_PLUS_EQ] = ACTIONS(391),
    [anon_sym_DASH_EQ] = ACTIONS(391),
    [anon_sym_STAR_EQ] = ACTIONS(391),
    [anon_sym_SLASH_EQ] = ACTIONS(391),
    [anon_sym_PERCENT_EQ] = ACTIONS(391),
    [anon_sym_CARET_EQ] = ACTIONS(391),
    [anon_sym_AMP_EQ] = ACTIONS(391),
    [anon_sym_PIPE_EQ] = ACTIONS(391),
    [anon_sym_LT_LT_EQ] = ACTIONS(391),
    [anon_sym_GT_GT_EQ] = ACTIONS(391),
    [anon_sym_EQ] = ACTIONS(393),
    [anon_sym_EQ_EQ] = ACTIONS(391),
    [anon_sym_BANG_EQ] = ACTIONS(391),
    [anon_sym_GT] = ACTIONS(393),
    [anon_sym_LT] = ACTIONS(393),
    [anon_sym_GT_EQ] = ACTIONS(391),
    [anon_sym_LT_EQ] = ACTIONS(391),
    [anon_sym_DOT] = ACTIONS(393),
    [anon_sym_DOT_DOT] = ACTIONS(393),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(391),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(391),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(393),
    [anon_sym_as] = ACTIONS(393),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [62] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1718),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(246),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(62),
    [sym_block_comment] = STATE(62),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(387),
    [anon_sym_LBRACE] = ACTIONS(387),
    [anon_sym_PLUS] = ACTIONS(389),
    [anon_sym_STAR] = ACTIONS(389),
    [anon_sym_QMARK] = ACTIONS(387),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(389),
    [anon_sym_SLASH] = ACTIONS(389),
    [anon_sym_PERCENT] = ACTIONS(389),
    [anon_sym_CARET] = ACTIONS(389),
    [anon_sym_BANG] = ACTIONS(471),
    [anon_sym_AMP] = ACTIONS(389),
    [anon_sym_PIPE] = ACTIONS(389),
    [anon_sym_AMP_AMP] = ACTIONS(387),
    [anon_sym_PIPE_PIPE] = ACTIONS(387),
    [anon_sym_LT_LT] = ACTIONS(389),
    [anon_sym_GT_GT] = ACTIONS(389),
    [anon_sym_PLUS_EQ] = ACTIONS(387),
    [anon_sym_DASH_EQ] = ACTIONS(387),
    [anon_sym_STAR_EQ] = ACTIONS(387),
    [anon_sym_SLASH_EQ] = ACTIONS(387),
    [anon_sym_PERCENT_EQ] = ACTIONS(387),
    [anon_sym_CARET_EQ] = ACTIONS(387),
    [anon_sym_AMP_EQ] = ACTIONS(387),
    [anon_sym_PIPE_EQ] = ACTIONS(387),
    [anon_sym_LT_LT_EQ] = ACTIONS(387),
    [anon_sym_GT_GT_EQ] = ACTIONS(387),
    [anon_sym_EQ] = ACTIONS(389),
    [anon_sym_EQ_EQ] = ACTIONS(387),
    [anon_sym_BANG_EQ] = ACTIONS(387),
    [anon_sym_GT] = ACTIONS(389),
    [anon_sym_LT] = ACTIONS(389),
    [anon_sym_GT_EQ] = ACTIONS(387),
    [anon_sym_LT_EQ] = ACTIONS(387),
    [anon_sym_DOT] = ACTIONS(389),
    [anon_sym_DOT_DOT] = ACTIONS(389),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(387),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(387),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_as] = ACTIONS(389),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(475),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(479),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(481),
    [anon_sym_static] = ACTIONS(483),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(485),
    [anon_sym_move] = ACTIONS(487),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [63] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1607),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(52),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(63),
    [sym_block_comment] = STATE(63),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(395),
    [anon_sym_LBRACK] = ACTIONS(395),
    [anon_sym_LBRACE] = ACTIONS(395),
    [anon_sym_PLUS] = ACTIONS(397),
    [anon_sym_STAR] = ACTIONS(397),
    [anon_sym_QMARK] = ACTIONS(395),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(397),
    [anon_sym_SLASH] = ACTIONS(397),
    [anon_sym_PERCENT] = ACTIONS(397),
    [anon_sym_CARET] = ACTIONS(397),
    [anon_sym_BANG] = ACTIONS(503),
    [anon_sym_AMP] = ACTIONS(397),
    [anon_sym_PIPE] = ACTIONS(397),
    [anon_sym_AMP_AMP] = ACTIONS(395),
    [anon_sym_PIPE_PIPE] = ACTIONS(395),
    [anon_sym_LT_LT] = ACTIONS(397),
    [anon_sym_GT_GT] = ACTIONS(397),
    [anon_sym_PLUS_EQ] = ACTIONS(395),
    [anon_sym_DASH_EQ] = ACTIONS(395),
    [anon_sym_STAR_EQ] = ACTIONS(395),
    [anon_sym_SLASH_EQ] = ACTIONS(395),
    [anon_sym_PERCENT_EQ] = ACTIONS(395),
    [anon_sym_CARET_EQ] = ACTIONS(395),
    [anon_sym_AMP_EQ] = ACTIONS(395),
    [anon_sym_PIPE_EQ] = ACTIONS(395),
    [anon_sym_LT_LT_EQ] = ACTIONS(395),
    [anon_sym_GT_GT_EQ] = ACTIONS(395),
    [anon_sym_EQ] = ACTIONS(397),
    [anon_sym_EQ_EQ] = ACTIONS(395),
    [anon_sym_BANG_EQ] = ACTIONS(395),
    [anon_sym_GT] = ACTIONS(397),
    [anon_sym_LT] = ACTIONS(397),
    [anon_sym_GT_EQ] = ACTIONS(395),
    [anon_sym_LT_EQ] = ACTIONS(395),
    [anon_sym_DOT] = ACTIONS(397),
    [anon_sym_DOT_DOT] = ACTIONS(397),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(395),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(395),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(397),
    [anon_sym_as] = ACTIONS(397),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [64] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1606),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(64),
    [sym_block_comment] = STATE(64),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(391),
    [anon_sym_LBRACK] = ACTIONS(391),
    [anon_sym_LBRACE] = ACTIONS(391),
    [anon_sym_PLUS] = ACTIONS(393),
    [anon_sym_STAR] = ACTIONS(393),
    [anon_sym_QMARK] = ACTIONS(391),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(393),
    [anon_sym_SLASH] = ACTIONS(393),
    [anon_sym_PERCENT] = ACTIONS(393),
    [anon_sym_CARET] = ACTIONS(393),
    [anon_sym_BANG] = ACTIONS(503),
    [anon_sym_AMP] = ACTIONS(393),
    [anon_sym_PIPE] = ACTIONS(393),
    [anon_sym_AMP_AMP] = ACTIONS(391),
    [anon_sym_PIPE_PIPE] = ACTIONS(391),
    [anon_sym_LT_LT] = ACTIONS(393),
    [anon_sym_GT_GT] = ACTIONS(393),
    [anon_sym_PLUS_EQ] = ACTIONS(391),
    [anon_sym_DASH_EQ] = ACTIONS(391),
    [anon_sym_STAR_EQ] = ACTIONS(391),
    [anon_sym_SLASH_EQ] = ACTIONS(391),
    [anon_sym_PERCENT_EQ] = ACTIONS(391),
    [anon_sym_CARET_EQ] = ACTIONS(391),
    [anon_sym_AMP_EQ] = ACTIONS(391),
    [anon_sym_PIPE_EQ] = ACTIONS(391),
    [anon_sym_LT_LT_EQ] = ACTIONS(391),
    [anon_sym_GT_GT_EQ] = ACTIONS(391),
    [anon_sym_EQ] = ACTIONS(393),
    [anon_sym_EQ_EQ] = ACTIONS(391),
    [anon_sym_BANG_EQ] = ACTIONS(391),
    [anon_sym_GT] = ACTIONS(393),
    [anon_sym_LT] = ACTIONS(393),
    [anon_sym_GT_EQ] = ACTIONS(391),
    [anon_sym_LT_EQ] = ACTIONS(391),
    [anon_sym_DOT] = ACTIONS(393),
    [anon_sym_DOT_DOT] = ACTIONS(393),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(391),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(391),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(393),
    [anon_sym_as] = ACTIONS(393),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [65] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1827),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(246),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(65),
    [sym_block_comment] = STATE(65),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_PLUS] = ACTIONS(403),
    [anon_sym_STAR] = ACTIONS(471),
    [anon_sym_QMARK] = ACTIONS(401),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(471),
    [anon_sym_SLASH] = ACTIONS(403),
    [anon_sym_PERCENT] = ACTIONS(403),
    [anon_sym_CARET] = ACTIONS(403),
    [anon_sym_BANG] = ACTIONS(471),
    [anon_sym_AMP] = ACTIONS(523),
    [anon_sym_PIPE] = ACTIONS(381),
    [anon_sym_AMP_AMP] = ACTIONS(401),
    [anon_sym_PIPE_PIPE] = ACTIONS(401),
    [anon_sym_LT_LT] = ACTIONS(403),
    [anon_sym_GT_GT] = ACTIONS(403),
    [anon_sym_PLUS_EQ] = ACTIONS(401),
    [anon_sym_DASH_EQ] = ACTIONS(401),
    [anon_sym_STAR_EQ] = ACTIONS(401),
    [anon_sym_SLASH_EQ] = ACTIONS(401),
    [anon_sym_PERCENT_EQ] = ACTIONS(401),
    [anon_sym_CARET_EQ] = ACTIONS(401),
    [anon_sym_AMP_EQ] = ACTIONS(401),
    [anon_sym_PIPE_EQ] = ACTIONS(401),
    [anon_sym_LT_LT_EQ] = ACTIONS(401),
    [anon_sym_GT_GT_EQ] = ACTIONS(401),
    [anon_sym_EQ] = ACTIONS(403),
    [anon_sym_EQ_EQ] = ACTIONS(401),
    [anon_sym_BANG_EQ] = ACTIONS(401),
    [anon_sym_GT] = ACTIONS(403),
    [anon_sym_LT] = ACTIONS(383),
    [anon_sym_GT_EQ] = ACTIONS(401),
    [anon_sym_LT_EQ] = ACTIONS(401),
    [anon_sym_DOT] = ACTIONS(403),
    [anon_sym_DOT_DOT] = ACTIONS(525),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(401),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(401),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_as] = ACTIONS(403),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(475),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(479),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(481),
    [anon_sym_static] = ACTIONS(483),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(485),
    [anon_sym_move] = ACTIONS(487),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [66] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1828),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(246),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(66),
    [sym_block_comment] = STATE(66),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_PLUS] = ACTIONS(377),
    [anon_sym_STAR] = ACTIONS(471),
    [anon_sym_QMARK] = ACTIONS(375),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(471),
    [anon_sym_SLASH] = ACTIONS(377),
    [anon_sym_PERCENT] = ACTIONS(377),
    [anon_sym_CARET] = ACTIONS(377),
    [anon_sym_BANG] = ACTIONS(471),
    [anon_sym_AMP] = ACTIONS(523),
    [anon_sym_PIPE] = ACTIONS(381),
    [anon_sym_AMP_AMP] = ACTIONS(375),
    [anon_sym_PIPE_PIPE] = ACTIONS(375),
    [anon_sym_LT_LT] = ACTIONS(377),
    [anon_sym_GT_GT] = ACTIONS(377),
    [anon_sym_PLUS_EQ] = ACTIONS(375),
    [anon_sym_DASH_EQ] = ACTIONS(375),
    [anon_sym_STAR_EQ] = ACTIONS(375),
    [anon_sym_SLASH_EQ] = ACTIONS(375),
    [anon_sym_PERCENT_EQ] = ACTIONS(375),
    [anon_sym_CARET_EQ] = ACTIONS(375),
    [anon_sym_AMP_EQ] = ACTIONS(375),
    [anon_sym_PIPE_EQ] = ACTIONS(375),
    [anon_sym_LT_LT_EQ] = ACTIONS(375),
    [anon_sym_GT_GT_EQ] = ACTIONS(375),
    [anon_sym_EQ] = ACTIONS(377),
    [anon_sym_EQ_EQ] = ACTIONS(375),
    [anon_sym_BANG_EQ] = ACTIONS(375),
    [anon_sym_GT] = ACTIONS(377),
    [anon_sym_LT] = ACTIONS(383),
    [anon_sym_GT_EQ] = ACTIONS(375),
    [anon_sym_LT_EQ] = ACTIONS(375),
    [anon_sym_DOT] = ACTIONS(377),
    [anon_sym_DOT_DOT] = ACTIONS(525),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(375),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(375),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_as] = ACTIONS(377),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(475),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(479),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(481),
    [anon_sym_static] = ACTIONS(483),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(485),
    [anon_sym_move] = ACTIONS(487),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [67] = {
    [sym__token_pattern] = STATE(176),
    [sym_token_tree_pattern] = STATE(170),
    [sym_token_binding_pattern] = STATE(170),
    [sym_token_repetition_pattern] = STATE(170),
    [sym__literal] = STATE(170),
    [sym_string_literal] = STATE(177),
    [sym_raw_string_literal] = STATE(177),
    [sym_boolean_literal] = STATE(177),
    [sym_line_comment] = STATE(67),
    [sym_block_comment] = STATE(67),
    [aux_sym_token_tree_pattern_repeat1] = STATE(67),
    [aux_sym__non_special_token_repeat1] = STATE(136),
    [sym_identifier] = ACTIONS(527),
    [anon_sym_SEMI] = ACTIONS(530),
    [anon_sym_LPAREN] = ACTIONS(533),
    [anon_sym_RPAREN] = ACTIONS(536),
    [anon_sym_LBRACK] = ACTIONS(538),
    [anon_sym_RBRACK] = ACTIONS(536),
    [anon_sym_LBRACE] = ACTIONS(541),
    [anon_sym_RBRACE] = ACTIONS(536),
    [anon_sym_EQ_GT] = ACTIONS(530),
    [anon_sym_COLON] = ACTIONS(544),
    [anon_sym_DOLLAR] = ACTIONS(547),
    [anon_sym_PLUS] = ACTIONS(544),
    [anon_sym_STAR] = ACTIONS(544),
    [anon_sym_QMARK] = ACTIONS(530),
    [anon_sym_u8] = ACTIONS(527),
    [anon_sym_i8] = ACTIONS(527),
    [anon_sym_u16] = ACTIONS(527),
    [anon_sym_i16] = ACTIONS(527),
    [anon_sym_u32] = ACTIONS(527),
    [anon_sym_i32] = ACTIONS(527),
    [anon_sym_u64] = ACTIONS(527),
    [anon_sym_i64] = ACTIONS(527),
    [anon_sym_u128] = ACTIONS(527),
    [anon_sym_i128] = ACTIONS(527),
    [anon_sym_isize] = ACTIONS(527),
    [anon_sym_usize] = ACTIONS(527),
    [anon_sym_f32] = ACTIONS(527),
    [anon_sym_f64] = ACTIONS(527),
    [anon_sym_bool] = ACTIONS(527),
    [anon_sym_str] = ACTIONS(527),
    [anon_sym_char] = ACTIONS(527),
    [anon_sym_DASH] = ACTIONS(544),
    [anon_sym_SLASH] = ACTIONS(544),
    [anon_sym_PERCENT] = ACTIONS(544),
    [anon_sym_CARET] = ACTIONS(544),
    [anon_sym_BANG] = ACTIONS(544),
    [anon_sym_AMP] = ACTIONS(544),
    [anon_sym_PIPE] = ACTIONS(544),
    [anon_sym_AMP_AMP] = ACTIONS(530),
    [anon_sym_PIPE_PIPE] = ACTIONS(530),
    [anon_sym_LT_LT] = ACTIONS(544),
    [anon_sym_GT_GT] = ACTIONS(544),
    [anon_sym_PLUS_EQ] = ACTIONS(530),
    [anon_sym_DASH_EQ] = ACTIONS(530),
    [anon_sym_STAR_EQ] = ACTIONS(530),
    [anon_sym_SLASH_EQ] = ACTIONS(530),
    [anon_sym_PERCENT_EQ] = ACTIONS(530),
    [anon_sym_CARET_EQ] = ACTIONS(530),
    [anon_sym_AMP_EQ] = ACTIONS(530),
    [anon_sym_PIPE_EQ] = ACTIONS(530),
    [anon_sym_LT_LT_EQ] = ACTIONS(530),
    [anon_sym_GT_GT_EQ] = ACTIONS(530),
    [anon_sym_EQ] = ACTIONS(544),
    [anon_sym_EQ_EQ] = ACTIONS(530),
    [anon_sym_BANG_EQ] = ACTIONS(530),
    [anon_sym_GT] = ACTIONS(544),
    [anon_sym_LT] = ACTIONS(544),
    [anon_sym_GT_EQ] = ACTIONS(530),
    [anon_sym_LT_EQ] = ACTIONS(530),
    [anon_sym_AT] = ACTIONS(530),
    [anon_sym__] = ACTIONS(544),
    [anon_sym_DOT] = ACTIONS(544),
    [anon_sym_DOT_DOT] = ACTIONS(544),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(530),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(530),
    [anon_sym_COMMA] = ACTIONS(530),
    [anon_sym_COLON_COLON] = ACTIONS(530),
    [anon_sym_DASH_GT] = ACTIONS(530),
    [anon_sym_POUND] = ACTIONS(530),
    [anon_sym_SQUOTE] = ACTIONS(527),
    [anon_sym_as] = ACTIONS(527),
    [anon_sym_async] = ACTIONS(527),
    [anon_sym_await] = ACTIONS(527),
    [anon_sym_break] = ACTIONS(527),
    [anon_sym_const] = ACTIONS(527),
    [anon_sym_continue] = ACTIONS(527),
    [anon_sym_default] = ACTIONS(527),
    [anon_sym_enum] = ACTIONS(527),
    [anon_sym_fn] = ACTIONS(527),
    [anon_sym_for] = ACTIONS(527),
    [anon_sym_gen] = ACTIONS(527),
    [anon_sym_if] = ACTIONS(527),
    [anon_sym_impl] = ACTIONS(527),
    [anon_sym_let] = ACTIONS(527),
    [anon_sym_loop] = ACTIONS(527),
    [anon_sym_match] = ACTIONS(527),
    [anon_sym_mod] = ACTIONS(527),
    [anon_sym_pub] = ACTIONS(527),
    [anon_sym_return] = ACTIONS(527),
    [anon_sym_static] = ACTIONS(527),
    [anon_sym_struct] = ACTIONS(527),
    [anon_sym_trait] = ACTIONS(527),
    [anon_sym_type] = ACTIONS(527),
    [anon_sym_union] = ACTIONS(527),
    [anon_sym_unsafe] = ACTIONS(527),
    [anon_sym_use] = ACTIONS(527),
    [anon_sym_where] = ACTIONS(527),
    [anon_sym_while] = ACTIONS(527),
    [sym_mutable_specifier] = ACTIONS(527),
    [sym_integer_literal] = ACTIONS(550),
    [aux_sym_string_literal_token1] = ACTIONS(553),
    [sym_char_literal] = ACTIONS(550),
    [anon_sym_true] = ACTIONS(556),
    [anon_sym_false] = ACTIONS(556),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(527),
    [sym_super] = ACTIONS(527),
    [sym_crate] = ACTIONS(527),
    [sym_metavariable] = ACTIONS(559),
    [sym__raw_string_literal_start] = ACTIONS(562),
    [sym_float_literal] = ACTIONS(550),
  },
  [68] = {
    [sym__token_pattern] = STATE(176),
    [sym_token_tree_pattern] = STATE(170),
    [sym_token_binding_pattern] = STATE(170),
    [sym_token_repetition_pattern] = STATE(170),
    [sym__literal] = STATE(170),
    [sym_string_literal] = STATE(177),
    [sym_raw_string_literal] = STATE(177),
    [sym_boolean_literal] = STATE(177),
    [sym_line_comment] = STATE(68),
    [sym_block_comment] = STATE(68),
    [aux_sym_token_tree_pattern_repeat1] = STATE(72),
    [aux_sym__non_special_token_repeat1] = STATE(136),
    [sym_identifier] = ACTIONS(565),
    [anon_sym_SEMI] = ACTIONS(567),
    [anon_sym_LPAREN] = ACTIONS(569),
    [anon_sym_RPAREN] = ACTIONS(571),
    [anon_sym_LBRACK] = ACTIONS(573),
    [anon_sym_LBRACE] = ACTIONS(575),
    [anon_sym_EQ_GT] = ACTIONS(567),
    [anon_sym_COLON] = ACTIONS(577),
    [anon_sym_DOLLAR] = ACTIONS(579),
    [anon_sym_PLUS] = ACTIONS(577),
    [anon_sym_STAR] = ACTIONS(577),
    [anon_sym_QMARK] = ACTIONS(567),
    [anon_sym_u8] = ACTIONS(565),
    [anon_sym_i8] = ACTIONS(565),
    [anon_sym_u16] = ACTIONS(565),
    [anon_sym_i16] = ACTIONS(565),
    [anon_sym_u32] = ACTIONS(565),
    [anon_sym_i32] = ACTIONS(565),
    [anon_sym_u64] = ACTIONS(565),
    [anon_sym_i64] = ACTIONS(565),
    [anon_sym_u128] = ACTIONS(565),
    [anon_sym_i128] = ACTIONS(565),
    [anon_sym_isize] = ACTIONS(565),
    [anon_sym_usize] = ACTIONS(565),
    [anon_sym_f32] = ACTIONS(565),
    [anon_sym_f64] = ACTIONS(565),
    [anon_sym_bool] = ACTIONS(565),
    [anon_sym_str] = ACTIONS(565),
    [anon_sym_char] = ACTIONS(565),
    [anon_sym_DASH] = ACTIONS(577),
    [anon_sym_SLASH] = ACTIONS(577),
    [anon_sym_PERCENT] = ACTIONS(577),
    [anon_sym_CARET] = ACTIONS(577),
    [anon_sym_BANG] = ACTIONS(577),
    [anon_sym_AMP] = ACTIONS(577),
    [anon_sym_PIPE] = ACTIONS(577),
    [anon_sym_AMP_AMP] = ACTIONS(567),
    [anon_sym_PIPE_PIPE] = ACTIONS(567),
    [anon_sym_LT_LT] = ACTIONS(577),
    [anon_sym_GT_GT] = ACTIONS(577),
    [anon_sym_PLUS_EQ] = ACTIONS(567),
    [anon_sym_DASH_EQ] = ACTIONS(567),
    [anon_sym_STAR_EQ] = ACTIONS(567),
    [anon_sym_SLASH_EQ] = ACTIONS(567),
    [anon_sym_PERCENT_EQ] = ACTIONS(567),
    [anon_sym_CARET_EQ] = ACTIONS(567),
    [anon_sym_AMP_EQ] = ACTIONS(567),
    [anon_sym_PIPE_EQ] = ACTIONS(567),
    [anon_sym_LT_LT_EQ] = ACTIONS(567),
    [anon_sym_GT_GT_EQ] = ACTIONS(567),
    [anon_sym_EQ] = ACTIONS(577),
    [anon_sym_EQ_EQ] = ACTIONS(567),
    [anon_sym_BANG_EQ] = ACTIONS(567),
    [anon_sym_GT] = ACTIONS(577),
    [anon_sym_LT] = ACTIONS(577),
    [anon_sym_GT_EQ] = ACTIONS(567),
    [anon_sym_LT_EQ] = ACTIONS(567),
    [anon_sym_AT] = ACTIONS(567),
    [anon_sym__] = ACTIONS(577),
    [anon_sym_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(567),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(567),
    [anon_sym_COMMA] = ACTIONS(567),
    [anon_sym_COLON_COLON] = ACTIONS(567),
    [anon_sym_DASH_GT] = ACTIONS(567),
    [anon_sym_POUND] = ACTIONS(567),
    [anon_sym_SQUOTE] = ACTIONS(565),
    [anon_sym_as] = ACTIONS(565),
    [anon_sym_async] = ACTIONS(565),
    [anon_sym_await] = ACTIONS(565),
    [anon_sym_break] = ACTIONS(565),
    [anon_sym_const] = ACTIONS(565),
    [anon_sym_continue] = ACTIONS(565),
    [anon_sym_default] = ACTIONS(565),
    [anon_sym_enum] = ACTIONS(565),
    [anon_sym_fn] = ACTIONS(565),
    [anon_sym_for] = ACTIONS(565),
    [anon_sym_gen] = ACTIONS(565),
    [anon_sym_if] = ACTIONS(565),
    [anon_sym_impl] = ACTIONS(565),
    [anon_sym_let] = ACTIONS(565),
    [anon_sym_loop] = ACTIONS(565),
    [anon_sym_match] = ACTIONS(565),
    [anon_sym_mod] = ACTIONS(565),
    [anon_sym_pub] = ACTIONS(565),
    [anon_sym_return] = ACTIONS(565),
    [anon_sym_static] = ACTIONS(565),
    [anon_sym_struct] = ACTIONS(565),
    [anon_sym_trait] = ACTIONS(565),
    [anon_sym_type] = ACTIONS(565),
    [anon_sym_union] = ACTIONS(565),
    [anon_sym_unsafe] = ACTIONS(565),
    [anon_sym_use] = ACTIONS(565),
    [anon_sym_where] = ACTIONS(565),
    [anon_sym_while] = ACTIONS(565),
    [sym_mutable_specifier] = ACTIONS(565),
    [sym_integer_literal] = ACTIONS(581),
    [aux_sym_string_literal_token1] = ACTIONS(583),
    [sym_char_literal] = ACTIONS(581),
    [anon_sym_true] = ACTIONS(585),
    [anon_sym_false] = ACTIONS(585),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(565),
    [sym_super] = ACTIONS(565),
    [sym_crate] = ACTIONS(565),
    [sym_metavariable] = ACTIONS(587),
    [sym__raw_string_literal_start] = ACTIONS(589),
    [sym_float_literal] = ACTIONS(581),
  },
  [69] = {
    [sym__token_pattern] = STATE(176),
    [sym_token_tree_pattern] = STATE(170),
    [sym_token_binding_pattern] = STATE(170),
    [sym_token_repetition_pattern] = STATE(170),
    [sym__literal] = STATE(170),
    [sym_string_literal] = STATE(177),
    [sym_raw_string_literal] = STATE(177),
    [sym_boolean_literal] = STATE(177),
    [sym_line_comment] = STATE(69),
    [sym_block_comment] = STATE(69),
    [aux_sym_token_tree_pattern_repeat1] = STATE(73),
    [aux_sym__non_special_token_repeat1] = STATE(136),
    [sym_identifier] = ACTIONS(565),
    [anon_sym_SEMI] = ACTIONS(567),
    [anon_sym_LPAREN] = ACTIONS(569),
    [anon_sym_LBRACK] = ACTIONS(573),
    [anon_sym_RBRACK] = ACTIONS(571),
    [anon_sym_LBRACE] = ACTIONS(575),
    [anon_sym_EQ_GT] = ACTIONS(567),
    [anon_sym_COLON] = ACTIONS(577),
    [anon_sym_DOLLAR] = ACTIONS(579),
    [anon_sym_PLUS] = ACTIONS(577),
    [anon_sym_STAR] = ACTIONS(577),
    [anon_sym_QMARK] = ACTIONS(567),
    [anon_sym_u8] = ACTIONS(565),
    [anon_sym_i8] = ACTIONS(565),
    [anon_sym_u16] = ACTIONS(565),
    [anon_sym_i16] = ACTIONS(565),
    [anon_sym_u32] = ACTIONS(565),
    [anon_sym_i32] = ACTIONS(565),
    [anon_sym_u64] = ACTIONS(565),
    [anon_sym_i64] = ACTIONS(565),
    [anon_sym_u128] = ACTIONS(565),
    [anon_sym_i128] = ACTIONS(565),
    [anon_sym_isize] = ACTIONS(565),
    [anon_sym_usize] = ACTIONS(565),
    [anon_sym_f32] = ACTIONS(565),
    [anon_sym_f64] = ACTIONS(565),
    [anon_sym_bool] = ACTIONS(565),
    [anon_sym_str] = ACTIONS(565),
    [anon_sym_char] = ACTIONS(565),
    [anon_sym_DASH] = ACTIONS(577),
    [anon_sym_SLASH] = ACTIONS(577),
    [anon_sym_PERCENT] = ACTIONS(577),
    [anon_sym_CARET] = ACTIONS(577),
    [anon_sym_BANG] = ACTIONS(577),
    [anon_sym_AMP] = ACTIONS(577),
    [anon_sym_PIPE] = ACTIONS(577),
    [anon_sym_AMP_AMP] = ACTIONS(567),
    [anon_sym_PIPE_PIPE] = ACTIONS(567),
    [anon_sym_LT_LT] = ACTIONS(577),
    [anon_sym_GT_GT] = ACTIONS(577),
    [anon_sym_PLUS_EQ] = ACTIONS(567),
    [anon_sym_DASH_EQ] = ACTIONS(567),
    [anon_sym_STAR_EQ] = ACTIONS(567),
    [anon_sym_SLASH_EQ] = ACTIONS(567),
    [anon_sym_PERCENT_EQ] = ACTIONS(567),
    [anon_sym_CARET_EQ] = ACTIONS(567),
    [anon_sym_AMP_EQ] = ACTIONS(567),
    [anon_sym_PIPE_EQ] = ACTIONS(567),
    [anon_sym_LT_LT_EQ] = ACTIONS(567),
    [anon_sym_GT_GT_EQ] = ACTIONS(567),
    [anon_sym_EQ] = ACTIONS(577),
    [anon_sym_EQ_EQ] = ACTIONS(567),
    [anon_sym_BANG_EQ] = ACTIONS(567),
    [anon_sym_GT] = ACTIONS(577),
    [anon_sym_LT] = ACTIONS(577),
    [anon_sym_GT_EQ] = ACTIONS(567),
    [anon_sym_LT_EQ] = ACTIONS(567),
    [anon_sym_AT] = ACTIONS(567),
    [anon_sym__] = ACTIONS(577),
    [anon_sym_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(567),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(567),
    [anon_sym_COMMA] = ACTIONS(567),
    [anon_sym_COLON_COLON] = ACTIONS(567),
    [anon_sym_DASH_GT] = ACTIONS(567),
    [anon_sym_POUND] = ACTIONS(567),
    [anon_sym_SQUOTE] = ACTIONS(565),
    [anon_sym_as] = ACTIONS(565),
    [anon_sym_async] = ACTIONS(565),
    [anon_sym_await] = ACTIONS(565),
    [anon_sym_break] = ACTIONS(565),
    [anon_sym_const] = ACTIONS(565),
    [anon_sym_continue] = ACTIONS(565),
    [anon_sym_default] = ACTIONS(565),
    [anon_sym_enum] = ACTIONS(565),
    [anon_sym_fn] = ACTIONS(565),
    [anon_sym_for] = ACTIONS(565),
    [anon_sym_gen] = ACTIONS(565),
    [anon_sym_if] = ACTIONS(565),
    [anon_sym_impl] = ACTIONS(565),
    [anon_sym_let] = ACTIONS(565),
    [anon_sym_loop] = ACTIONS(565),
    [anon_sym_match] = ACTIONS(565),
    [anon_sym_mod] = ACTIONS(565),
    [anon_sym_pub] = ACTIONS(565),
    [anon_sym_return] = ACTIONS(565),
    [anon_sym_static] = ACTIONS(565),
    [anon_sym_struct] = ACTIONS(565),
    [anon_sym_trait] = ACTIONS(565),
    [anon_sym_type] = ACTIONS(565),
    [anon_sym_union] = ACTIONS(565),
    [anon_sym_unsafe] = ACTIONS(565),
    [anon_sym_use] = ACTIONS(565),
    [anon_sym_where] = ACTIONS(565),
    [anon_sym_while] = ACTIONS(565),
    [sym_mutable_specifier] = ACTIONS(565),
    [sym_integer_literal] = ACTIONS(581),
    [aux_sym_string_literal_token1] = ACTIONS(583),
    [sym_char_literal] = ACTIONS(581),
    [anon_sym_true] = ACTIONS(585),
    [anon_sym_false] = ACTIONS(585),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(565),
    [sym_super] = ACTIONS(565),
    [sym_crate] = ACTIONS(565),
    [sym_metavariable] = ACTIONS(587),
    [sym__raw_string_literal_start] = ACTIONS(589),
    [sym_float_literal] = ACTIONS(581),
  },
  [70] = {
    [sym__token_pattern] = STATE(176),
    [sym_token_tree_pattern] = STATE(170),
    [sym_token_binding_pattern] = STATE(170),
    [sym_token_repetition_pattern] = STATE(170),
    [sym__literal] = STATE(170),
    [sym_string_literal] = STATE(177),
    [sym_raw_string_literal] = STATE(177),
    [sym_boolean_literal] = STATE(177),
    [sym_line_comment] = STATE(70),
    [sym_block_comment] = STATE(70),
    [aux_sym_token_tree_pattern_repeat1] = STATE(74),
    [aux_sym__non_special_token_repeat1] = STATE(136),
    [sym_identifier] = ACTIONS(565),
    [anon_sym_SEMI] = ACTIONS(567),
    [anon_sym_LPAREN] = ACTIONS(569),
    [anon_sym_LBRACK] = ACTIONS(573),
    [anon_sym_LBRACE] = ACTIONS(575),
    [anon_sym_RBRACE] = ACTIONS(571),
    [anon_sym_EQ_GT] = ACTIONS(567),
    [anon_sym_COLON] = ACTIONS(577),
    [anon_sym_DOLLAR] = ACTIONS(579),
    [anon_sym_PLUS] = ACTIONS(577),
    [anon_sym_STAR] = ACTIONS(577),
    [anon_sym_QMARK] = ACTIONS(567),
    [anon_sym_u8] = ACTIONS(565),
    [anon_sym_i8] = ACTIONS(565),
    [anon_sym_u16] = ACTIONS(565),
    [anon_sym_i16] = ACTIONS(565),
    [anon_sym_u32] = ACTIONS(565),
    [anon_sym_i32] = ACTIONS(565),
    [anon_sym_u64] = ACTIONS(565),
    [anon_sym_i64] = ACTIONS(565),
    [anon_sym_u128] = ACTIONS(565),
    [anon_sym_i128] = ACTIONS(565),
    [anon_sym_isize] = ACTIONS(565),
    [anon_sym_usize] = ACTIONS(565),
    [anon_sym_f32] = ACTIONS(565),
    [anon_sym_f64] = ACTIONS(565),
    [anon_sym_bool] = ACTIONS(565),
    [anon_sym_str] = ACTIONS(565),
    [anon_sym_char] = ACTIONS(565),
    [anon_sym_DASH] = ACTIONS(577),
    [anon_sym_SLASH] = ACTIONS(577),
    [anon_sym_PERCENT] = ACTIONS(577),
    [anon_sym_CARET] = ACTIONS(577),
    [anon_sym_BANG] = ACTIONS(577),
    [anon_sym_AMP] = ACTIONS(577),
    [anon_sym_PIPE] = ACTIONS(577),
    [anon_sym_AMP_AMP] = ACTIONS(567),
    [anon_sym_PIPE_PIPE] = ACTIONS(567),
    [anon_sym_LT_LT] = ACTIONS(577),
    [anon_sym_GT_GT] = ACTIONS(577),
    [anon_sym_PLUS_EQ] = ACTIONS(567),
    [anon_sym_DASH_EQ] = ACTIONS(567),
    [anon_sym_STAR_EQ] = ACTIONS(567),
    [anon_sym_SLASH_EQ] = ACTIONS(567),
    [anon_sym_PERCENT_EQ] = ACTIONS(567),
    [anon_sym_CARET_EQ] = ACTIONS(567),
    [anon_sym_AMP_EQ] = ACTIONS(567),
    [anon_sym_PIPE_EQ] = ACTIONS(567),
    [anon_sym_LT_LT_EQ] = ACTIONS(567),
    [anon_sym_GT_GT_EQ] = ACTIONS(567),
    [anon_sym_EQ] = ACTIONS(577),
    [anon_sym_EQ_EQ] = ACTIONS(567),
    [anon_sym_BANG_EQ] = ACTIONS(567),
    [anon_sym_GT] = ACTIONS(577),
    [anon_sym_LT] = ACTIONS(577),
    [anon_sym_GT_EQ] = ACTIONS(567),
    [anon_sym_LT_EQ] = ACTIONS(567),
    [anon_sym_AT] = ACTIONS(567),
    [anon_sym__] = ACTIONS(577),
    [anon_sym_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(567),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(567),
    [anon_sym_COMMA] = ACTIONS(567),
    [anon_sym_COLON_COLON] = ACTIONS(567),
    [anon_sym_DASH_GT] = ACTIONS(567),
    [anon_sym_POUND] = ACTIONS(567),
    [anon_sym_SQUOTE] = ACTIONS(565),
    [anon_sym_as] = ACTIONS(565),
    [anon_sym_async] = ACTIONS(565),
    [anon_sym_await] = ACTIONS(565),
    [anon_sym_break] = ACTIONS(565),
    [anon_sym_const] = ACTIONS(565),
    [anon_sym_continue] = ACTIONS(565),
    [anon_sym_default] = ACTIONS(565),
    [anon_sym_enum] = ACTIONS(565),
    [anon_sym_fn] = ACTIONS(565),
    [anon_sym_for] = ACTIONS(565),
    [anon_sym_gen] = ACTIONS(565),
    [anon_sym_if] = ACTIONS(565),
    [anon_sym_impl] = ACTIONS(565),
    [anon_sym_let] = ACTIONS(565),
    [anon_sym_loop] = ACTIONS(565),
    [anon_sym_match] = ACTIONS(565),
    [anon_sym_mod] = ACTIONS(565),
    [anon_sym_pub] = ACTIONS(565),
    [anon_sym_return] = ACTIONS(565),
    [anon_sym_static] = ACTIONS(565),
    [anon_sym_struct] = ACTIONS(565),
    [anon_sym_trait] = ACTIONS(565),
    [anon_sym_type] = ACTIONS(565),
    [anon_sym_union] = ACTIONS(565),
    [anon_sym_unsafe] = ACTIONS(565),
    [anon_sym_use] = ACTIONS(565),
    [anon_sym_where] = ACTIONS(565),
    [anon_sym_while] = ACTIONS(565),
    [sym_mutable_specifier] = ACTIONS(565),
    [sym_integer_literal] = ACTIONS(581),
    [aux_sym_string_literal_token1] = ACTIONS(583),
    [sym_char_literal] = ACTIONS(581),
    [anon_sym_true] = ACTIONS(585),
    [anon_sym_false] = ACTIONS(585),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(565),
    [sym_super] = ACTIONS(565),
    [sym_crate] = ACTIONS(565),
    [sym_metavariable] = ACTIONS(587),
    [sym__raw_string_literal_start] = ACTIONS(589),
    [sym_float_literal] = ACTIONS(581),
  },
  [71] = {
    [sym__token_pattern] = STATE(176),
    [sym_token_tree_pattern] = STATE(170),
    [sym_token_binding_pattern] = STATE(170),
    [sym_token_repetition_pattern] = STATE(170),
    [sym__literal] = STATE(170),
    [sym_string_literal] = STATE(177),
    [sym_raw_string_literal] = STATE(177),
    [sym_boolean_literal] = STATE(177),
    [sym_line_comment] = STATE(71),
    [sym_block_comment] = STATE(71),
    [aux_sym_token_tree_pattern_repeat1] = STATE(78),
    [aux_sym__non_special_token_repeat1] = STATE(136),
    [sym_identifier] = ACTIONS(565),
    [anon_sym_SEMI] = ACTIONS(567),
    [anon_sym_LPAREN] = ACTIONS(569),
    [anon_sym_LBRACK] = ACTIONS(573),
    [anon_sym_RBRACK] = ACTIONS(591),
    [anon_sym_LBRACE] = ACTIONS(575),
    [anon_sym_EQ_GT] = ACTIONS(567),
    [anon_sym_COLON] = ACTIONS(577),
    [anon_sym_DOLLAR] = ACTIONS(579),
    [anon_sym_PLUS] = ACTIONS(577),
    [anon_sym_STAR] = ACTIONS(577),
    [anon_sym_QMARK] = ACTIONS(567),
    [anon_sym_u8] = ACTIONS(565),
    [anon_sym_i8] = ACTIONS(565),
    [anon_sym_u16] = ACTIONS(565),
    [anon_sym_i16] = ACTIONS(565),
    [anon_sym_u32] = ACTIONS(565),
    [anon_sym_i32] = ACTIONS(565),
    [anon_sym_u64] = ACTIONS(565),
    [anon_sym_i64] = ACTIONS(565),
    [anon_sym_u128] = ACTIONS(565),
    [anon_sym_i128] = ACTIONS(565),
    [anon_sym_isize] = ACTIONS(565),
    [anon_sym_usize] = ACTIONS(565),
    [anon_sym_f32] = ACTIONS(565),
    [anon_sym_f64] = ACTIONS(565),
    [anon_sym_bool] = ACTIONS(565),
    [anon_sym_str] = ACTIONS(565),
    [anon_sym_char] = ACTIONS(565),
    [anon_sym_DASH] = ACTIONS(577),
    [anon_sym_SLASH] = ACTIONS(577),
    [anon_sym_PERCENT] = ACTIONS(577),
    [anon_sym_CARET] = ACTIONS(577),
    [anon_sym_BANG] = ACTIONS(577),
    [anon_sym_AMP] = ACTIONS(577),
    [anon_sym_PIPE] = ACTIONS(577),
    [anon_sym_AMP_AMP] = ACTIONS(567),
    [anon_sym_PIPE_PIPE] = ACTIONS(567),
    [anon_sym_LT_LT] = ACTIONS(577),
    [anon_sym_GT_GT] = ACTIONS(577),
    [anon_sym_PLUS_EQ] = ACTIONS(567),
    [anon_sym_DASH_EQ] = ACTIONS(567),
    [anon_sym_STAR_EQ] = ACTIONS(567),
    [anon_sym_SLASH_EQ] = ACTIONS(567),
    [anon_sym_PERCENT_EQ] = ACTIONS(567),
    [anon_sym_CARET_EQ] = ACTIONS(567),
    [anon_sym_AMP_EQ] = ACTIONS(567),
    [anon_sym_PIPE_EQ] = ACTIONS(567),
    [anon_sym_LT_LT_EQ] = ACTIONS(567),
    [anon_sym_GT_GT_EQ] = ACTIONS(567),
    [anon_sym_EQ] = ACTIONS(577),
    [anon_sym_EQ_EQ] = ACTIONS(567),
    [anon_sym_BANG_EQ] = ACTIONS(567),
    [anon_sym_GT] = ACTIONS(577),
    [anon_sym_LT] = ACTIONS(577),
    [anon_sym_GT_EQ] = ACTIONS(567),
    [anon_sym_LT_EQ] = ACTIONS(567),
    [anon_sym_AT] = ACTIONS(567),
    [anon_sym__] = ACTIONS(577),
    [anon_sym_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(567),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(567),
    [anon_sym_COMMA] = ACTIONS(567),
    [anon_sym_COLON_COLON] = ACTIONS(567),
    [anon_sym_DASH_GT] = ACTIONS(567),
    [anon_sym_POUND] = ACTIONS(567),
    [anon_sym_SQUOTE] = ACTIONS(565),
    [anon_sym_as] = ACTIONS(565),
    [anon_sym_async] = ACTIONS(565),
    [anon_sym_await] = ACTIONS(565),
    [anon_sym_break] = ACTIONS(565),
    [anon_sym_const] = ACTIONS(565),
    [anon_sym_continue] = ACTIONS(565),
    [anon_sym_default] = ACTIONS(565),
    [anon_sym_enum] = ACTIONS(565),
    [anon_sym_fn] = ACTIONS(565),
    [anon_sym_for] = ACTIONS(565),
    [anon_sym_gen] = ACTIONS(565),
    [anon_sym_if] = ACTIONS(565),
    [anon_sym_impl] = ACTIONS(565),
    [anon_sym_let] = ACTIONS(565),
    [anon_sym_loop] = ACTIONS(565),
    [anon_sym_match] = ACTIONS(565),
    [anon_sym_mod] = ACTIONS(565),
    [anon_sym_pub] = ACTIONS(565),
    [anon_sym_return] = ACTIONS(565),
    [anon_sym_static] = ACTIONS(565),
    [anon_sym_struct] = ACTIONS(565),
    [anon_sym_trait] = ACTIONS(565),
    [anon_sym_type] = ACTIONS(565),
    [anon_sym_union] = ACTIONS(565),
    [anon_sym_unsafe] = ACTIONS(565),
    [anon_sym_use] = ACTIONS(565),
    [anon_sym_where] = ACTIONS(565),
    [anon_sym_while] = ACTIONS(565),
    [sym_mutable_specifier] = ACTIONS(565),
    [sym_integer_literal] = ACTIONS(581),
    [aux_sym_string_literal_token1] = ACTIONS(583),
    [sym_char_literal] = ACTIONS(581),
    [anon_sym_true] = ACTIONS(585),
    [anon_sym_false] = ACTIONS(585),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(565),
    [sym_super] = ACTIONS(565),
    [sym_crate] = ACTIONS(565),
    [sym_metavariable] = ACTIONS(587),
    [sym__raw_string_literal_start] = ACTIONS(589),
    [sym_float_literal] = ACTIONS(581),
  },
  [72] = {
    [sym__token_pattern] = STATE(176),
    [sym_token_tree_pattern] = STATE(170),
    [sym_token_binding_pattern] = STATE(170),
    [sym_token_repetition_pattern] = STATE(170),
    [sym__literal] = STATE(170),
    [sym_string_literal] = STATE(177),
    [sym_raw_string_literal] = STATE(177),
    [sym_boolean_literal] = STATE(177),
    [sym_line_comment] = STATE(72),
    [sym_block_comment] = STATE(72),
    [aux_sym_token_tree_pattern_repeat1] = STATE(67),
    [aux_sym__non_special_token_repeat1] = STATE(136),
    [sym_identifier] = ACTIONS(565),
    [anon_sym_SEMI] = ACTIONS(567),
    [anon_sym_LPAREN] = ACTIONS(569),
    [anon_sym_RPAREN] = ACTIONS(593),
    [anon_sym_LBRACK] = ACTIONS(573),
    [anon_sym_LBRACE] = ACTIONS(575),
    [anon_sym_EQ_GT] = ACTIONS(567),
    [anon_sym_COLON] = ACTIONS(577),
    [anon_sym_DOLLAR] = ACTIONS(579),
    [anon_sym_PLUS] = ACTIONS(577),
    [anon_sym_STAR] = ACTIONS(577),
    [anon_sym_QMARK] = ACTIONS(567),
    [anon_sym_u8] = ACTIONS(565),
    [anon_sym_i8] = ACTIONS(565),
    [anon_sym_u16] = ACTIONS(565),
    [anon_sym_i16] = ACTIONS(565),
    [anon_sym_u32] = ACTIONS(565),
    [anon_sym_i32] = ACTIONS(565),
    [anon_sym_u64] = ACTIONS(565),
    [anon_sym_i64] = ACTIONS(565),
    [anon_sym_u128] = ACTIONS(565),
    [anon_sym_i128] = ACTIONS(565),
    [anon_sym_isize] = ACTIONS(565),
    [anon_sym_usize] = ACTIONS(565),
    [anon_sym_f32] = ACTIONS(565),
    [anon_sym_f64] = ACTIONS(565),
    [anon_sym_bool] = ACTIONS(565),
    [anon_sym_str] = ACTIONS(565),
    [anon_sym_char] = ACTIONS(565),
    [anon_sym_DASH] = ACTIONS(577),
    [anon_sym_SLASH] = ACTIONS(577),
    [anon_sym_PERCENT] = ACTIONS(577),
    [anon_sym_CARET] = ACTIONS(577),
    [anon_sym_BANG] = ACTIONS(577),
    [anon_sym_AMP] = ACTIONS(577),
    [anon_sym_PIPE] = ACTIONS(577),
    [anon_sym_AMP_AMP] = ACTIONS(567),
    [anon_sym_PIPE_PIPE] = ACTIONS(567),
    [anon_sym_LT_LT] = ACTIONS(577),
    [anon_sym_GT_GT] = ACTIONS(577),
    [anon_sym_PLUS_EQ] = ACTIONS(567),
    [anon_sym_DASH_EQ] = ACTIONS(567),
    [anon_sym_STAR_EQ] = ACTIONS(567),
    [anon_sym_SLASH_EQ] = ACTIONS(567),
    [anon_sym_PERCENT_EQ] = ACTIONS(567),
    [anon_sym_CARET_EQ] = ACTIONS(567),
    [anon_sym_AMP_EQ] = ACTIONS(567),
    [anon_sym_PIPE_EQ] = ACTIONS(567),
    [anon_sym_LT_LT_EQ] = ACTIONS(567),
    [anon_sym_GT_GT_EQ] = ACTIONS(567),
    [anon_sym_EQ] = ACTIONS(577),
    [anon_sym_EQ_EQ] = ACTIONS(567),
    [anon_sym_BANG_EQ] = ACTIONS(567),
    [anon_sym_GT] = ACTIONS(577),
    [anon_sym_LT] = ACTIONS(577),
    [anon_sym_GT_EQ] = ACTIONS(567),
    [anon_sym_LT_EQ] = ACTIONS(567),
    [anon_sym_AT] = ACTIONS(567),
    [anon_sym__] = ACTIONS(577),
    [anon_sym_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(567),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(567),
    [anon_sym_COMMA] = ACTIONS(567),
    [anon_sym_COLON_COLON] = ACTIONS(567),
    [anon_sym_DASH_GT] = ACTIONS(567),
    [anon_sym_POUND] = ACTIONS(567),
    [anon_sym_SQUOTE] = ACTIONS(565),
    [anon_sym_as] = ACTIONS(565),
    [anon_sym_async] = ACTIONS(565),
    [anon_sym_await] = ACTIONS(565),
    [anon_sym_break] = ACTIONS(565),
    [anon_sym_const] = ACTIONS(565),
    [anon_sym_continue] = ACTIONS(565),
    [anon_sym_default] = ACTIONS(565),
    [anon_sym_enum] = ACTIONS(565),
    [anon_sym_fn] = ACTIONS(565),
    [anon_sym_for] = ACTIONS(565),
    [anon_sym_gen] = ACTIONS(565),
    [anon_sym_if] = ACTIONS(565),
    [anon_sym_impl] = ACTIONS(565),
    [anon_sym_let] = ACTIONS(565),
    [anon_sym_loop] = ACTIONS(565),
    [anon_sym_match] = ACTIONS(565),
    [anon_sym_mod] = ACTIONS(565),
    [anon_sym_pub] = ACTIONS(565),
    [anon_sym_return] = ACTIONS(565),
    [anon_sym_static] = ACTIONS(565),
    [anon_sym_struct] = ACTIONS(565),
    [anon_sym_trait] = ACTIONS(565),
    [anon_sym_type] = ACTIONS(565),
    [anon_sym_union] = ACTIONS(565),
    [anon_sym_unsafe] = ACTIONS(565),
    [anon_sym_use] = ACTIONS(565),
    [anon_sym_where] = ACTIONS(565),
    [anon_sym_while] = ACTIONS(565),
    [sym_mutable_specifier] = ACTIONS(565),
    [sym_integer_literal] = ACTIONS(581),
    [aux_sym_string_literal_token1] = ACTIONS(583),
    [sym_char_literal] = ACTIONS(581),
    [anon_sym_true] = ACTIONS(585),
    [anon_sym_false] = ACTIONS(585),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(565),
    [sym_super] = ACTIONS(565),
    [sym_crate] = ACTIONS(565),
    [sym_metavariable] = ACTIONS(587),
    [sym__raw_string_literal_start] = ACTIONS(589),
    [sym_float_literal] = ACTIONS(581),
  },
  [73] = {
    [sym__token_pattern] = STATE(176),
    [sym_token_tree_pattern] = STATE(170),
    [sym_token_binding_pattern] = STATE(170),
    [sym_token_repetition_pattern] = STATE(170),
    [sym__literal] = STATE(170),
    [sym_string_literal] = STATE(177),
    [sym_raw_string_literal] = STATE(177),
    [sym_boolean_literal] = STATE(177),
    [sym_line_comment] = STATE(73),
    [sym_block_comment] = STATE(73),
    [aux_sym_token_tree_pattern_repeat1] = STATE(67),
    [aux_sym__non_special_token_repeat1] = STATE(136),
    [sym_identifier] = ACTIONS(565),
    [anon_sym_SEMI] = ACTIONS(567),
    [anon_sym_LPAREN] = ACTIONS(569),
    [anon_sym_LBRACK] = ACTIONS(573),
    [anon_sym_RBRACK] = ACTIONS(593),
    [anon_sym_LBRACE] = ACTIONS(575),
    [anon_sym_EQ_GT] = ACTIONS(567),
    [anon_sym_COLON] = ACTIONS(577),
    [anon_sym_DOLLAR] = ACTIONS(579),
    [anon_sym_PLUS] = ACTIONS(577),
    [anon_sym_STAR] = ACTIONS(577),
    [anon_sym_QMARK] = ACTIONS(567),
    [anon_sym_u8] = ACTIONS(565),
    [anon_sym_i8] = ACTIONS(565),
    [anon_sym_u16] = ACTIONS(565),
    [anon_sym_i16] = ACTIONS(565),
    [anon_sym_u32] = ACTIONS(565),
    [anon_sym_i32] = ACTIONS(565),
    [anon_sym_u64] = ACTIONS(565),
    [anon_sym_i64] = ACTIONS(565),
    [anon_sym_u128] = ACTIONS(565),
    [anon_sym_i128] = ACTIONS(565),
    [anon_sym_isize] = ACTIONS(565),
    [anon_sym_usize] = ACTIONS(565),
    [anon_sym_f32] = ACTIONS(565),
    [anon_sym_f64] = ACTIONS(565),
    [anon_sym_bool] = ACTIONS(565),
    [anon_sym_str] = ACTIONS(565),
    [anon_sym_char] = ACTIONS(565),
    [anon_sym_DASH] = ACTIONS(577),
    [anon_sym_SLASH] = ACTIONS(577),
    [anon_sym_PERCENT] = ACTIONS(577),
    [anon_sym_CARET] = ACTIONS(577),
    [anon_sym_BANG] = ACTIONS(577),
    [anon_sym_AMP] = ACTIONS(577),
    [anon_sym_PIPE] = ACTIONS(577),
    [anon_sym_AMP_AMP] = ACTIONS(567),
    [anon_sym_PIPE_PIPE] = ACTIONS(567),
    [anon_sym_LT_LT] = ACTIONS(577),
    [anon_sym_GT_GT] = ACTIONS(577),
    [anon_sym_PLUS_EQ] = ACTIONS(567),
    [anon_sym_DASH_EQ] = ACTIONS(567),
    [anon_sym_STAR_EQ] = ACTIONS(567),
    [anon_sym_SLASH_EQ] = ACTIONS(567),
    [anon_sym_PERCENT_EQ] = ACTIONS(567),
    [anon_sym_CARET_EQ] = ACTIONS(567),
    [anon_sym_AMP_EQ] = ACTIONS(567),
    [anon_sym_PIPE_EQ] = ACTIONS(567),
    [anon_sym_LT_LT_EQ] = ACTIONS(567),
    [anon_sym_GT_GT_EQ] = ACTIONS(567),
    [anon_sym_EQ] = ACTIONS(577),
    [anon_sym_EQ_EQ] = ACTIONS(567),
    [anon_sym_BANG_EQ] = ACTIONS(567),
    [anon_sym_GT] = ACTIONS(577),
    [anon_sym_LT] = ACTIONS(577),
    [anon_sym_GT_EQ] = ACTIONS(567),
    [anon_sym_LT_EQ] = ACTIONS(567),
    [anon_sym_AT] = ACTIONS(567),
    [anon_sym__] = ACTIONS(577),
    [anon_sym_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(567),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(567),
    [anon_sym_COMMA] = ACTIONS(567),
    [anon_sym_COLON_COLON] = ACTIONS(567),
    [anon_sym_DASH_GT] = ACTIONS(567),
    [anon_sym_POUND] = ACTIONS(567),
    [anon_sym_SQUOTE] = ACTIONS(565),
    [anon_sym_as] = ACTIONS(565),
    [anon_sym_async] = ACTIONS(565),
    [anon_sym_await] = ACTIONS(565),
    [anon_sym_break] = ACTIONS(565),
    [anon_sym_const] = ACTIONS(565),
    [anon_sym_continue] = ACTIONS(565),
    [anon_sym_default] = ACTIONS(565),
    [anon_sym_enum] = ACTIONS(565),
    [anon_sym_fn] = ACTIONS(565),
    [anon_sym_for] = ACTIONS(565),
    [anon_sym_gen] = ACTIONS(565),
    [anon_sym_if] = ACTIONS(565),
    [anon_sym_impl] = ACTIONS(565),
    [anon_sym_let] = ACTIONS(565),
    [anon_sym_loop] = ACTIONS(565),
    [anon_sym_match] = ACTIONS(565),
    [anon_sym_mod] = ACTIONS(565),
    [anon_sym_pub] = ACTIONS(565),
    [anon_sym_return] = ACTIONS(565),
    [anon_sym_static] = ACTIONS(565),
    [anon_sym_struct] = ACTIONS(565),
    [anon_sym_trait] = ACTIONS(565),
    [anon_sym_type] = ACTIONS(565),
    [anon_sym_union] = ACTIONS(565),
    [anon_sym_unsafe] = ACTIONS(565),
    [anon_sym_use] = ACTIONS(565),
    [anon_sym_where] = ACTIONS(565),
    [anon_sym_while] = ACTIONS(565),
    [sym_mutable_specifier] = ACTIONS(565),
    [sym_integer_literal] = ACTIONS(581),
    [aux_sym_string_literal_token1] = ACTIONS(583),
    [sym_char_literal] = ACTIONS(581),
    [anon_sym_true] = ACTIONS(585),
    [anon_sym_false] = ACTIONS(585),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(565),
    [sym_super] = ACTIONS(565),
    [sym_crate] = ACTIONS(565),
    [sym_metavariable] = ACTIONS(587),
    [sym__raw_string_literal_start] = ACTIONS(589),
    [sym_float_literal] = ACTIONS(581),
  },
  [74] = {
    [sym__token_pattern] = STATE(176),
    [sym_token_tree_pattern] = STATE(170),
    [sym_token_binding_pattern] = STATE(170),
    [sym_token_repetition_pattern] = STATE(170),
    [sym__literal] = STATE(170),
    [sym_string_literal] = STATE(177),
    [sym_raw_string_literal] = STATE(177),
    [sym_boolean_literal] = STATE(177),
    [sym_line_comment] = STATE(74),
    [sym_block_comment] = STATE(74),
    [aux_sym_token_tree_pattern_repeat1] = STATE(67),
    [aux_sym__non_special_token_repeat1] = STATE(136),
    [sym_identifier] = ACTIONS(565),
    [anon_sym_SEMI] = ACTIONS(567),
    [anon_sym_LPAREN] = ACTIONS(569),
    [anon_sym_LBRACK] = ACTIONS(573),
    [anon_sym_LBRACE] = ACTIONS(575),
    [anon_sym_RBRACE] = ACTIONS(593),
    [anon_sym_EQ_GT] = ACTIONS(567),
    [anon_sym_COLON] = ACTIONS(577),
    [anon_sym_DOLLAR] = ACTIONS(579),
    [anon_sym_PLUS] = ACTIONS(577),
    [anon_sym_STAR] = ACTIONS(577),
    [anon_sym_QMARK] = ACTIONS(567),
    [anon_sym_u8] = ACTIONS(565),
    [anon_sym_i8] = ACTIONS(565),
    [anon_sym_u16] = ACTIONS(565),
    [anon_sym_i16] = ACTIONS(565),
    [anon_sym_u32] = ACTIONS(565),
    [anon_sym_i32] = ACTIONS(565),
    [anon_sym_u64] = ACTIONS(565),
    [anon_sym_i64] = ACTIONS(565),
    [anon_sym_u128] = ACTIONS(565),
    [anon_sym_i128] = ACTIONS(565),
    [anon_sym_isize] = ACTIONS(565),
    [anon_sym_usize] = ACTIONS(565),
    [anon_sym_f32] = ACTIONS(565),
    [anon_sym_f64] = ACTIONS(565),
    [anon_sym_bool] = ACTIONS(565),
    [anon_sym_str] = ACTIONS(565),
    [anon_sym_char] = ACTIONS(565),
    [anon_sym_DASH] = ACTIONS(577),
    [anon_sym_SLASH] = ACTIONS(577),
    [anon_sym_PERCENT] = ACTIONS(577),
    [anon_sym_CARET] = ACTIONS(577),
    [anon_sym_BANG] = ACTIONS(577),
    [anon_sym_AMP] = ACTIONS(577),
    [anon_sym_PIPE] = ACTIONS(577),
    [anon_sym_AMP_AMP] = ACTIONS(567),
    [anon_sym_PIPE_PIPE] = ACTIONS(567),
    [anon_sym_LT_LT] = ACTIONS(577),
    [anon_sym_GT_GT] = ACTIONS(577),
    [anon_sym_PLUS_EQ] = ACTIONS(567),
    [anon_sym_DASH_EQ] = ACTIONS(567),
    [anon_sym_STAR_EQ] = ACTIONS(567),
    [anon_sym_SLASH_EQ] = ACTIONS(567),
    [anon_sym_PERCENT_EQ] = ACTIONS(567),
    [anon_sym_CARET_EQ] = ACTIONS(567),
    [anon_sym_AMP_EQ] = ACTIONS(567),
    [anon_sym_PIPE_EQ] = ACTIONS(567),
    [anon_sym_LT_LT_EQ] = ACTIONS(567),
    [anon_sym_GT_GT_EQ] = ACTIONS(567),
    [anon_sym_EQ] = ACTIONS(577),
    [anon_sym_EQ_EQ] = ACTIONS(567),
    [anon_sym_BANG_EQ] = ACTIONS(567),
    [anon_sym_GT] = ACTIONS(577),
    [anon_sym_LT] = ACTIONS(577),
    [anon_sym_GT_EQ] = ACTIONS(567),
    [anon_sym_LT_EQ] = ACTIONS(567),
    [anon_sym_AT] = ACTIONS(567),
    [anon_sym__] = ACTIONS(577),
    [anon_sym_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(567),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(567),
    [anon_sym_COMMA] = ACTIONS(567),
    [anon_sym_COLON_COLON] = ACTIONS(567),
    [anon_sym_DASH_GT] = ACTIONS(567),
    [anon_sym_POUND] = ACTIONS(567),
    [anon_sym_SQUOTE] = ACTIONS(565),
    [anon_sym_as] = ACTIONS(565),
    [anon_sym_async] = ACTIONS(565),
    [anon_sym_await] = ACTIONS(565),
    [anon_sym_break] = ACTIONS(565),
    [anon_sym_const] = ACTIONS(565),
    [anon_sym_continue] = ACTIONS(565),
    [anon_sym_default] = ACTIONS(565),
    [anon_sym_enum] = ACTIONS(565),
    [anon_sym_fn] = ACTIONS(565),
    [anon_sym_for] = ACTIONS(565),
    [anon_sym_gen] = ACTIONS(565),
    [anon_sym_if] = ACTIONS(565),
    [anon_sym_impl] = ACTIONS(565),
    [anon_sym_let] = ACTIONS(565),
    [anon_sym_loop] = ACTIONS(565),
    [anon_sym_match] = ACTIONS(565),
    [anon_sym_mod] = ACTIONS(565),
    [anon_sym_pub] = ACTIONS(565),
    [anon_sym_return] = ACTIONS(565),
    [anon_sym_static] = ACTIONS(565),
    [anon_sym_struct] = ACTIONS(565),
    [anon_sym_trait] = ACTIONS(565),
    [anon_sym_type] = ACTIONS(565),
    [anon_sym_union] = ACTIONS(565),
    [anon_sym_unsafe] = ACTIONS(565),
    [anon_sym_use] = ACTIONS(565),
    [anon_sym_where] = ACTIONS(565),
    [anon_sym_while] = ACTIONS(565),
    [sym_mutable_specifier] = ACTIONS(565),
    [sym_integer_literal] = ACTIONS(581),
    [aux_sym_string_literal_token1] = ACTIONS(583),
    [sym_char_literal] = ACTIONS(581),
    [anon_sym_true] = ACTIONS(585),
    [anon_sym_false] = ACTIONS(585),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(565),
    [sym_super] = ACTIONS(565),
    [sym_crate] = ACTIONS(565),
    [sym_metavariable] = ACTIONS(587),
    [sym__raw_string_literal_start] = ACTIONS(589),
    [sym_float_literal] = ACTIONS(581),
  },
  [75] = {
    [sym__token_pattern] = STATE(176),
    [sym_token_tree_pattern] = STATE(170),
    [sym_token_binding_pattern] = STATE(170),
    [sym_token_repetition_pattern] = STATE(170),
    [sym__literal] = STATE(170),
    [sym_string_literal] = STATE(177),
    [sym_raw_string_literal] = STATE(177),
    [sym_boolean_literal] = STATE(177),
    [sym_line_comment] = STATE(75),
    [sym_block_comment] = STATE(75),
    [aux_sym_token_tree_pattern_repeat1] = STATE(79),
    [aux_sym__non_special_token_repeat1] = STATE(136),
    [sym_identifier] = ACTIONS(565),
    [anon_sym_SEMI] = ACTIONS(567),
    [anon_sym_LPAREN] = ACTIONS(569),
    [anon_sym_LBRACK] = ACTIONS(573),
    [anon_sym_LBRACE] = ACTIONS(575),
    [anon_sym_RBRACE] = ACTIONS(591),
    [anon_sym_EQ_GT] = ACTIONS(567),
    [anon_sym_COLON] = ACTIONS(577),
    [anon_sym_DOLLAR] = ACTIONS(579),
    [anon_sym_PLUS] = ACTIONS(577),
    [anon_sym_STAR] = ACTIONS(577),
    [anon_sym_QMARK] = ACTIONS(567),
    [anon_sym_u8] = ACTIONS(565),
    [anon_sym_i8] = ACTIONS(565),
    [anon_sym_u16] = ACTIONS(565),
    [anon_sym_i16] = ACTIONS(565),
    [anon_sym_u32] = ACTIONS(565),
    [anon_sym_i32] = ACTIONS(565),
    [anon_sym_u64] = ACTIONS(565),
    [anon_sym_i64] = ACTIONS(565),
    [anon_sym_u128] = ACTIONS(565),
    [anon_sym_i128] = ACTIONS(565),
    [anon_sym_isize] = ACTIONS(565),
    [anon_sym_usize] = ACTIONS(565),
    [anon_sym_f32] = ACTIONS(565),
    [anon_sym_f64] = ACTIONS(565),
    [anon_sym_bool] = ACTIONS(565),
    [anon_sym_str] = ACTIONS(565),
    [anon_sym_char] = ACTIONS(565),
    [anon_sym_DASH] = ACTIONS(577),
    [anon_sym_SLASH] = ACTIONS(577),
    [anon_sym_PERCENT] = ACTIONS(577),
    [anon_sym_CARET] = ACTIONS(577),
    [anon_sym_BANG] = ACTIONS(577),
    [anon_sym_AMP] = ACTIONS(577),
    [anon_sym_PIPE] = ACTIONS(577),
    [anon_sym_AMP_AMP] = ACTIONS(567),
    [anon_sym_PIPE_PIPE] = ACTIONS(567),
    [anon_sym_LT_LT] = ACTIONS(577),
    [anon_sym_GT_GT] = ACTIONS(577),
    [anon_sym_PLUS_EQ] = ACTIONS(567),
    [anon_sym_DASH_EQ] = ACTIONS(567),
    [anon_sym_STAR_EQ] = ACTIONS(567),
    [anon_sym_SLASH_EQ] = ACTIONS(567),
    [anon_sym_PERCENT_EQ] = ACTIONS(567),
    [anon_sym_CARET_EQ] = ACTIONS(567),
    [anon_sym_AMP_EQ] = ACTIONS(567),
    [anon_sym_PIPE_EQ] = ACTIONS(567),
    [anon_sym_LT_LT_EQ] = ACTIONS(567),
    [anon_sym_GT_GT_EQ] = ACTIONS(567),
    [anon_sym_EQ] = ACTIONS(577),
    [anon_sym_EQ_EQ] = ACTIONS(567),
    [anon_sym_BANG_EQ] = ACTIONS(567),
    [anon_sym_GT] = ACTIONS(577),
    [anon_sym_LT] = ACTIONS(577),
    [anon_sym_GT_EQ] = ACTIONS(567),
    [anon_sym_LT_EQ] = ACTIONS(567),
    [anon_sym_AT] = ACTIONS(567),
    [anon_sym__] = ACTIONS(577),
    [anon_sym_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(567),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(567),
    [anon_sym_COMMA] = ACTIONS(567),
    [anon_sym_COLON_COLON] = ACTIONS(567),
    [anon_sym_DASH_GT] = ACTIONS(567),
    [anon_sym_POUND] = ACTIONS(567),
    [anon_sym_SQUOTE] = ACTIONS(565),
    [anon_sym_as] = ACTIONS(565),
    [anon_sym_async] = ACTIONS(565),
    [anon_sym_await] = ACTIONS(565),
    [anon_sym_break] = ACTIONS(565),
    [anon_sym_const] = ACTIONS(565),
    [anon_sym_continue] = ACTIONS(565),
    [anon_sym_default] = ACTIONS(565),
    [anon_sym_enum] = ACTIONS(565),
    [anon_sym_fn] = ACTIONS(565),
    [anon_sym_for] = ACTIONS(565),
    [anon_sym_gen] = ACTIONS(565),
    [anon_sym_if] = ACTIONS(565),
    [anon_sym_impl] = ACTIONS(565),
    [anon_sym_let] = ACTIONS(565),
    [anon_sym_loop] = ACTIONS(565),
    [anon_sym_match] = ACTIONS(565),
    [anon_sym_mod] = ACTIONS(565),
    [anon_sym_pub] = ACTIONS(565),
    [anon_sym_return] = ACTIONS(565),
    [anon_sym_static] = ACTIONS(565),
    [anon_sym_struct] = ACTIONS(565),
    [anon_sym_trait] = ACTIONS(565),
    [anon_sym_type] = ACTIONS(565),
    [anon_sym_union] = ACTIONS(565),
    [anon_sym_unsafe] = ACTIONS(565),
    [anon_sym_use] = ACTIONS(565),
    [anon_sym_where] = ACTIONS(565),
    [anon_sym_while] = ACTIONS(565),
    [sym_mutable_specifier] = ACTIONS(565),
    [sym_integer_literal] = ACTIONS(581),
    [aux_sym_string_literal_token1] = ACTIONS(583),
    [sym_char_literal] = ACTIONS(581),
    [anon_sym_true] = ACTIONS(585),
    [anon_sym_false] = ACTIONS(585),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(565),
    [sym_super] = ACTIONS(565),
    [sym_crate] = ACTIONS(565),
    [sym_metavariable] = ACTIONS(587),
    [sym__raw_string_literal_start] = ACTIONS(589),
    [sym_float_literal] = ACTIONS(581),
  },
  [76] = {
    [sym__token_pattern] = STATE(176),
    [sym_token_tree_pattern] = STATE(170),
    [sym_token_binding_pattern] = STATE(170),
    [sym_token_repetition_pattern] = STATE(170),
    [sym__literal] = STATE(170),
    [sym_string_literal] = STATE(177),
    [sym_raw_string_literal] = STATE(177),
    [sym_boolean_literal] = STATE(177),
    [sym_line_comment] = STATE(76),
    [sym_block_comment] = STATE(76),
    [aux_sym_token_tree_pattern_repeat1] = STATE(77),
    [aux_sym__non_special_token_repeat1] = STATE(136),
    [sym_identifier] = ACTIONS(565),
    [anon_sym_SEMI] = ACTIONS(567),
    [anon_sym_LPAREN] = ACTIONS(569),
    [anon_sym_RPAREN] = ACTIONS(591),
    [anon_sym_LBRACK] = ACTIONS(573),
    [anon_sym_LBRACE] = ACTIONS(575),
    [anon_sym_EQ_GT] = ACTIONS(567),
    [anon_sym_COLON] = ACTIONS(577),
    [anon_sym_DOLLAR] = ACTIONS(579),
    [anon_sym_PLUS] = ACTIONS(577),
    [anon_sym_STAR] = ACTIONS(577),
    [anon_sym_QMARK] = ACTIONS(567),
    [anon_sym_u8] = ACTIONS(565),
    [anon_sym_i8] = ACTIONS(565),
    [anon_sym_u16] = ACTIONS(565),
    [anon_sym_i16] = ACTIONS(565),
    [anon_sym_u32] = ACTIONS(565),
    [anon_sym_i32] = ACTIONS(565),
    [anon_sym_u64] = ACTIONS(565),
    [anon_sym_i64] = ACTIONS(565),
    [anon_sym_u128] = ACTIONS(565),
    [anon_sym_i128] = ACTIONS(565),
    [anon_sym_isize] = ACTIONS(565),
    [anon_sym_usize] = ACTIONS(565),
    [anon_sym_f32] = ACTIONS(565),
    [anon_sym_f64] = ACTIONS(565),
    [anon_sym_bool] = ACTIONS(565),
    [anon_sym_str] = ACTIONS(565),
    [anon_sym_char] = ACTIONS(565),
    [anon_sym_DASH] = ACTIONS(577),
    [anon_sym_SLASH] = ACTIONS(577),
    [anon_sym_PERCENT] = ACTIONS(577),
    [anon_sym_CARET] = ACTIONS(577),
    [anon_sym_BANG] = ACTIONS(577),
    [anon_sym_AMP] = ACTIONS(577),
    [anon_sym_PIPE] = ACTIONS(577),
    [anon_sym_AMP_AMP] = ACTIONS(567),
    [anon_sym_PIPE_PIPE] = ACTIONS(567),
    [anon_sym_LT_LT] = ACTIONS(577),
    [anon_sym_GT_GT] = ACTIONS(577),
    [anon_sym_PLUS_EQ] = ACTIONS(567),
    [anon_sym_DASH_EQ] = ACTIONS(567),
    [anon_sym_STAR_EQ] = ACTIONS(567),
    [anon_sym_SLASH_EQ] = ACTIONS(567),
    [anon_sym_PERCENT_EQ] = ACTIONS(567),
    [anon_sym_CARET_EQ] = ACTIONS(567),
    [anon_sym_AMP_EQ] = ACTIONS(567),
    [anon_sym_PIPE_EQ] = ACTIONS(567),
    [anon_sym_LT_LT_EQ] = ACTIONS(567),
    [anon_sym_GT_GT_EQ] = ACTIONS(567),
    [anon_sym_EQ] = ACTIONS(577),
    [anon_sym_EQ_EQ] = ACTIONS(567),
    [anon_sym_BANG_EQ] = ACTIONS(567),
    [anon_sym_GT] = ACTIONS(577),
    [anon_sym_LT] = ACTIONS(577),
    [anon_sym_GT_EQ] = ACTIONS(567),
    [anon_sym_LT_EQ] = ACTIONS(567),
    [anon_sym_AT] = ACTIONS(567),
    [anon_sym__] = ACTIONS(577),
    [anon_sym_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(567),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(567),
    [anon_sym_COMMA] = ACTIONS(567),
    [anon_sym_COLON_COLON] = ACTIONS(567),
    [anon_sym_DASH_GT] = ACTIONS(567),
    [anon_sym_POUND] = ACTIONS(567),
    [anon_sym_SQUOTE] = ACTIONS(565),
    [anon_sym_as] = ACTIONS(565),
    [anon_sym_async] = ACTIONS(565),
    [anon_sym_await] = ACTIONS(565),
    [anon_sym_break] = ACTIONS(565),
    [anon_sym_const] = ACTIONS(565),
    [anon_sym_continue] = ACTIONS(565),
    [anon_sym_default] = ACTIONS(565),
    [anon_sym_enum] = ACTIONS(565),
    [anon_sym_fn] = ACTIONS(565),
    [anon_sym_for] = ACTIONS(565),
    [anon_sym_gen] = ACTIONS(565),
    [anon_sym_if] = ACTIONS(565),
    [anon_sym_impl] = ACTIONS(565),
    [anon_sym_let] = ACTIONS(565),
    [anon_sym_loop] = ACTIONS(565),
    [anon_sym_match] = ACTIONS(565),
    [anon_sym_mod] = ACTIONS(565),
    [anon_sym_pub] = ACTIONS(565),
    [anon_sym_return] = ACTIONS(565),
    [anon_sym_static] = ACTIONS(565),
    [anon_sym_struct] = ACTIONS(565),
    [anon_sym_trait] = ACTIONS(565),
    [anon_sym_type] = ACTIONS(565),
    [anon_sym_union] = ACTIONS(565),
    [anon_sym_unsafe] = ACTIONS(565),
    [anon_sym_use] = ACTIONS(565),
    [anon_sym_where] = ACTIONS(565),
    [anon_sym_while] = ACTIONS(565),
    [sym_mutable_specifier] = ACTIONS(565),
    [sym_integer_literal] = ACTIONS(581),
    [aux_sym_string_literal_token1] = ACTIONS(583),
    [sym_char_literal] = ACTIONS(581),
    [anon_sym_true] = ACTIONS(585),
    [anon_sym_false] = ACTIONS(585),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(565),
    [sym_super] = ACTIONS(565),
    [sym_crate] = ACTIONS(565),
    [sym_metavariable] = ACTIONS(587),
    [sym__raw_string_literal_start] = ACTIONS(589),
    [sym_float_literal] = ACTIONS(581),
  },
  [77] = {
    [sym__token_pattern] = STATE(176),
    [sym_token_tree_pattern] = STATE(170),
    [sym_token_binding_pattern] = STATE(170),
    [sym_token_repetition_pattern] = STATE(170),
    [sym__literal] = STATE(170),
    [sym_string_literal] = STATE(177),
    [sym_raw_string_literal] = STATE(177),
    [sym_boolean_literal] = STATE(177),
    [sym_line_comment] = STATE(77),
    [sym_block_comment] = STATE(77),
    [aux_sym_token_tree_pattern_repeat1] = STATE(67),
    [aux_sym__non_special_token_repeat1] = STATE(136),
    [sym_identifier] = ACTIONS(565),
    [anon_sym_SEMI] = ACTIONS(567),
    [anon_sym_LPAREN] = ACTIONS(569),
    [anon_sym_RPAREN] = ACTIONS(595),
    [anon_sym_LBRACK] = ACTIONS(573),
    [anon_sym_LBRACE] = ACTIONS(575),
    [anon_sym_EQ_GT] = ACTIONS(567),
    [anon_sym_COLON] = ACTIONS(577),
    [anon_sym_DOLLAR] = ACTIONS(579),
    [anon_sym_PLUS] = ACTIONS(577),
    [anon_sym_STAR] = ACTIONS(577),
    [anon_sym_QMARK] = ACTIONS(567),
    [anon_sym_u8] = ACTIONS(565),
    [anon_sym_i8] = ACTIONS(565),
    [anon_sym_u16] = ACTIONS(565),
    [anon_sym_i16] = ACTIONS(565),
    [anon_sym_u32] = ACTIONS(565),
    [anon_sym_i32] = ACTIONS(565),
    [anon_sym_u64] = ACTIONS(565),
    [anon_sym_i64] = ACTIONS(565),
    [anon_sym_u128] = ACTIONS(565),
    [anon_sym_i128] = ACTIONS(565),
    [anon_sym_isize] = ACTIONS(565),
    [anon_sym_usize] = ACTIONS(565),
    [anon_sym_f32] = ACTIONS(565),
    [anon_sym_f64] = ACTIONS(565),
    [anon_sym_bool] = ACTIONS(565),
    [anon_sym_str] = ACTIONS(565),
    [anon_sym_char] = ACTIONS(565),
    [anon_sym_DASH] = ACTIONS(577),
    [anon_sym_SLASH] = ACTIONS(577),
    [anon_sym_PERCENT] = ACTIONS(577),
    [anon_sym_CARET] = ACTIONS(577),
    [anon_sym_BANG] = ACTIONS(577),
    [anon_sym_AMP] = ACTIONS(577),
    [anon_sym_PIPE] = ACTIONS(577),
    [anon_sym_AMP_AMP] = ACTIONS(567),
    [anon_sym_PIPE_PIPE] = ACTIONS(567),
    [anon_sym_LT_LT] = ACTIONS(577),
    [anon_sym_GT_GT] = ACTIONS(577),
    [anon_sym_PLUS_EQ] = ACTIONS(567),
    [anon_sym_DASH_EQ] = ACTIONS(567),
    [anon_sym_STAR_EQ] = ACTIONS(567),
    [anon_sym_SLASH_EQ] = ACTIONS(567),
    [anon_sym_PERCENT_EQ] = ACTIONS(567),
    [anon_sym_CARET_EQ] = ACTIONS(567),
    [anon_sym_AMP_EQ] = ACTIONS(567),
    [anon_sym_PIPE_EQ] = ACTIONS(567),
    [anon_sym_LT_LT_EQ] = ACTIONS(567),
    [anon_sym_GT_GT_EQ] = ACTIONS(567),
    [anon_sym_EQ] = ACTIONS(577),
    [anon_sym_EQ_EQ] = ACTIONS(567),
    [anon_sym_BANG_EQ] = ACTIONS(567),
    [anon_sym_GT] = ACTIONS(577),
    [anon_sym_LT] = ACTIONS(577),
    [anon_sym_GT_EQ] = ACTIONS(567),
    [anon_sym_LT_EQ] = ACTIONS(567),
    [anon_sym_AT] = ACTIONS(567),
    [anon_sym__] = ACTIONS(577),
    [anon_sym_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(567),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(567),
    [anon_sym_COMMA] = ACTIONS(567),
    [anon_sym_COLON_COLON] = ACTIONS(567),
    [anon_sym_DASH_GT] = ACTIONS(567),
    [anon_sym_POUND] = ACTIONS(567),
    [anon_sym_SQUOTE] = ACTIONS(565),
    [anon_sym_as] = ACTIONS(565),
    [anon_sym_async] = ACTIONS(565),
    [anon_sym_await] = ACTIONS(565),
    [anon_sym_break] = ACTIONS(565),
    [anon_sym_const] = ACTIONS(565),
    [anon_sym_continue] = ACTIONS(565),
    [anon_sym_default] = ACTIONS(565),
    [anon_sym_enum] = ACTIONS(565),
    [anon_sym_fn] = ACTIONS(565),
    [anon_sym_for] = ACTIONS(565),
    [anon_sym_gen] = ACTIONS(565),
    [anon_sym_if] = ACTIONS(565),
    [anon_sym_impl] = ACTIONS(565),
    [anon_sym_let] = ACTIONS(565),
    [anon_sym_loop] = ACTIONS(565),
    [anon_sym_match] = ACTIONS(565),
    [anon_sym_mod] = ACTIONS(565),
    [anon_sym_pub] = ACTIONS(565),
    [anon_sym_return] = ACTIONS(565),
    [anon_sym_static] = ACTIONS(565),
    [anon_sym_struct] = ACTIONS(565),
    [anon_sym_trait] = ACTIONS(565),
    [anon_sym_type] = ACTIONS(565),
    [anon_sym_union] = ACTIONS(565),
    [anon_sym_unsafe] = ACTIONS(565),
    [anon_sym_use] = ACTIONS(565),
    [anon_sym_where] = ACTIONS(565),
    [anon_sym_while] = ACTIONS(565),
    [sym_mutable_specifier] = ACTIONS(565),
    [sym_integer_literal] = ACTIONS(581),
    [aux_sym_string_literal_token1] = ACTIONS(583),
    [sym_char_literal] = ACTIONS(581),
    [anon_sym_true] = ACTIONS(585),
    [anon_sym_false] = ACTIONS(585),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(565),
    [sym_super] = ACTIONS(565),
    [sym_crate] = ACTIONS(565),
    [sym_metavariable] = ACTIONS(587),
    [sym__raw_string_literal_start] = ACTIONS(589),
    [sym_float_literal] = ACTIONS(581),
  },
  [78] = {
    [sym__token_pattern] = STATE(176),
    [sym_token_tree_pattern] = STATE(170),
    [sym_token_binding_pattern] = STATE(170),
    [sym_token_repetition_pattern] = STATE(170),
    [sym__literal] = STATE(170),
    [sym_string_literal] = STATE(177),
    [sym_raw_string_literal] = STATE(177),
    [sym_boolean_literal] = STATE(177),
    [sym_line_comment] = STATE(78),
    [sym_block_comment] = STATE(78),
    [aux_sym_token_tree_pattern_repeat1] = STATE(67),
    [aux_sym__non_special_token_repeat1] = STATE(136),
    [sym_identifier] = ACTIONS(565),
    [anon_sym_SEMI] = ACTIONS(567),
    [anon_sym_LPAREN] = ACTIONS(569),
    [anon_sym_LBRACK] = ACTIONS(573),
    [anon_sym_RBRACK] = ACTIONS(595),
    [anon_sym_LBRACE] = ACTIONS(575),
    [anon_sym_EQ_GT] = ACTIONS(567),
    [anon_sym_COLON] = ACTIONS(577),
    [anon_sym_DOLLAR] = ACTIONS(579),
    [anon_sym_PLUS] = ACTIONS(577),
    [anon_sym_STAR] = ACTIONS(577),
    [anon_sym_QMARK] = ACTIONS(567),
    [anon_sym_u8] = ACTIONS(565),
    [anon_sym_i8] = ACTIONS(565),
    [anon_sym_u16] = ACTIONS(565),
    [anon_sym_i16] = ACTIONS(565),
    [anon_sym_u32] = ACTIONS(565),
    [anon_sym_i32] = ACTIONS(565),
    [anon_sym_u64] = ACTIONS(565),
    [anon_sym_i64] = ACTIONS(565),
    [anon_sym_u128] = ACTIONS(565),
    [anon_sym_i128] = ACTIONS(565),
    [anon_sym_isize] = ACTIONS(565),
    [anon_sym_usize] = ACTIONS(565),
    [anon_sym_f32] = ACTIONS(565),
    [anon_sym_f64] = ACTIONS(565),
    [anon_sym_bool] = ACTIONS(565),
    [anon_sym_str] = ACTIONS(565),
    [anon_sym_char] = ACTIONS(565),
    [anon_sym_DASH] = ACTIONS(577),
    [anon_sym_SLASH] = ACTIONS(577),
    [anon_sym_PERCENT] = ACTIONS(577),
    [anon_sym_CARET] = ACTIONS(577),
    [anon_sym_BANG] = ACTIONS(577),
    [anon_sym_AMP] = ACTIONS(577),
    [anon_sym_PIPE] = ACTIONS(577),
    [anon_sym_AMP_AMP] = ACTIONS(567),
    [anon_sym_PIPE_PIPE] = ACTIONS(567),
    [anon_sym_LT_LT] = ACTIONS(577),
    [anon_sym_GT_GT] = ACTIONS(577),
    [anon_sym_PLUS_EQ] = ACTIONS(567),
    [anon_sym_DASH_EQ] = ACTIONS(567),
    [anon_sym_STAR_EQ] = ACTIONS(567),
    [anon_sym_SLASH_EQ] = ACTIONS(567),
    [anon_sym_PERCENT_EQ] = ACTIONS(567),
    [anon_sym_CARET_EQ] = ACTIONS(567),
    [anon_sym_AMP_EQ] = ACTIONS(567),
    [anon_sym_PIPE_EQ] = ACTIONS(567),
    [anon_sym_LT_LT_EQ] = ACTIONS(567),
    [anon_sym_GT_GT_EQ] = ACTIONS(567),
    [anon_sym_EQ] = ACTIONS(577),
    [anon_sym_EQ_EQ] = ACTIONS(567),
    [anon_sym_BANG_EQ] = ACTIONS(567),
    [anon_sym_GT] = ACTIONS(577),
    [anon_sym_LT] = ACTIONS(577),
    [anon_sym_GT_EQ] = ACTIONS(567),
    [anon_sym_LT_EQ] = ACTIONS(567),
    [anon_sym_AT] = ACTIONS(567),
    [anon_sym__] = ACTIONS(577),
    [anon_sym_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(567),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(567),
    [anon_sym_COMMA] = ACTIONS(567),
    [anon_sym_COLON_COLON] = ACTIONS(567),
    [anon_sym_DASH_GT] = ACTIONS(567),
    [anon_sym_POUND] = ACTIONS(567),
    [anon_sym_SQUOTE] = ACTIONS(565),
    [anon_sym_as] = ACTIONS(565),
    [anon_sym_async] = ACTIONS(565),
    [anon_sym_await] = ACTIONS(565),
    [anon_sym_break] = ACTIONS(565),
    [anon_sym_const] = ACTIONS(565),
    [anon_sym_continue] = ACTIONS(565),
    [anon_sym_default] = ACTIONS(565),
    [anon_sym_enum] = ACTIONS(565),
    [anon_sym_fn] = ACTIONS(565),
    [anon_sym_for] = ACTIONS(565),
    [anon_sym_gen] = ACTIONS(565),
    [anon_sym_if] = ACTIONS(565),
    [anon_sym_impl] = ACTIONS(565),
    [anon_sym_let] = ACTIONS(565),
    [anon_sym_loop] = ACTIONS(565),
    [anon_sym_match] = ACTIONS(565),
    [anon_sym_mod] = ACTIONS(565),
    [anon_sym_pub] = ACTIONS(565),
    [anon_sym_return] = ACTIONS(565),
    [anon_sym_static] = ACTIONS(565),
    [anon_sym_struct] = ACTIONS(565),
    [anon_sym_trait] = ACTIONS(565),
    [anon_sym_type] = ACTIONS(565),
    [anon_sym_union] = ACTIONS(565),
    [anon_sym_unsafe] = ACTIONS(565),
    [anon_sym_use] = ACTIONS(565),
    [anon_sym_where] = ACTIONS(565),
    [anon_sym_while] = ACTIONS(565),
    [sym_mutable_specifier] = ACTIONS(565),
    [sym_integer_literal] = ACTIONS(581),
    [aux_sym_string_literal_token1] = ACTIONS(583),
    [sym_char_literal] = ACTIONS(581),
    [anon_sym_true] = ACTIONS(585),
    [anon_sym_false] = ACTIONS(585),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(565),
    [sym_super] = ACTIONS(565),
    [sym_crate] = ACTIONS(565),
    [sym_metavariable] = ACTIONS(587),
    [sym__raw_string_literal_start] = ACTIONS(589),
    [sym_float_literal] = ACTIONS(581),
  },
  [79] = {
    [sym__token_pattern] = STATE(176),
    [sym_token_tree_pattern] = STATE(170),
    [sym_token_binding_pattern] = STATE(170),
    [sym_token_repetition_pattern] = STATE(170),
    [sym__literal] = STATE(170),
    [sym_string_literal] = STATE(177),
    [sym_raw_string_literal] = STATE(177),
    [sym_boolean_literal] = STATE(177),
    [sym_line_comment] = STATE(79),
    [sym_block_comment] = STATE(79),
    [aux_sym_token_tree_pattern_repeat1] = STATE(67),
    [aux_sym__non_special_token_repeat1] = STATE(136),
    [sym_identifier] = ACTIONS(565),
    [anon_sym_SEMI] = ACTIONS(567),
    [anon_sym_LPAREN] = ACTIONS(569),
    [anon_sym_LBRACK] = ACTIONS(573),
    [anon_sym_LBRACE] = ACTIONS(575),
    [anon_sym_RBRACE] = ACTIONS(595),
    [anon_sym_EQ_GT] = ACTIONS(567),
    [anon_sym_COLON] = ACTIONS(577),
    [anon_sym_DOLLAR] = ACTIONS(579),
    [anon_sym_PLUS] = ACTIONS(577),
    [anon_sym_STAR] = ACTIONS(577),
    [anon_sym_QMARK] = ACTIONS(567),
    [anon_sym_u8] = ACTIONS(565),
    [anon_sym_i8] = ACTIONS(565),
    [anon_sym_u16] = ACTIONS(565),
    [anon_sym_i16] = ACTIONS(565),
    [anon_sym_u32] = ACTIONS(565),
    [anon_sym_i32] = ACTIONS(565),
    [anon_sym_u64] = ACTIONS(565),
    [anon_sym_i64] = ACTIONS(565),
    [anon_sym_u128] = ACTIONS(565),
    [anon_sym_i128] = ACTIONS(565),
    [anon_sym_isize] = ACTIONS(565),
    [anon_sym_usize] = ACTIONS(565),
    [anon_sym_f32] = ACTIONS(565),
    [anon_sym_f64] = ACTIONS(565),
    [anon_sym_bool] = ACTIONS(565),
    [anon_sym_str] = ACTIONS(565),
    [anon_sym_char] = ACTIONS(565),
    [anon_sym_DASH] = ACTIONS(577),
    [anon_sym_SLASH] = ACTIONS(577),
    [anon_sym_PERCENT] = ACTIONS(577),
    [anon_sym_CARET] = ACTIONS(577),
    [anon_sym_BANG] = ACTIONS(577),
    [anon_sym_AMP] = ACTIONS(577),
    [anon_sym_PIPE] = ACTIONS(577),
    [anon_sym_AMP_AMP] = ACTIONS(567),
    [anon_sym_PIPE_PIPE] = ACTIONS(567),
    [anon_sym_LT_LT] = ACTIONS(577),
    [anon_sym_GT_GT] = ACTIONS(577),
    [anon_sym_PLUS_EQ] = ACTIONS(567),
    [anon_sym_DASH_EQ] = ACTIONS(567),
    [anon_sym_STAR_EQ] = ACTIONS(567),
    [anon_sym_SLASH_EQ] = ACTIONS(567),
    [anon_sym_PERCENT_EQ] = ACTIONS(567),
    [anon_sym_CARET_EQ] = ACTIONS(567),
    [anon_sym_AMP_EQ] = ACTIONS(567),
    [anon_sym_PIPE_EQ] = ACTIONS(567),
    [anon_sym_LT_LT_EQ] = ACTIONS(567),
    [anon_sym_GT_GT_EQ] = ACTIONS(567),
    [anon_sym_EQ] = ACTIONS(577),
    [anon_sym_EQ_EQ] = ACTIONS(567),
    [anon_sym_BANG_EQ] = ACTIONS(567),
    [anon_sym_GT] = ACTIONS(577),
    [anon_sym_LT] = ACTIONS(577),
    [anon_sym_GT_EQ] = ACTIONS(567),
    [anon_sym_LT_EQ] = ACTIONS(567),
    [anon_sym_AT] = ACTIONS(567),
    [anon_sym__] = ACTIONS(577),
    [anon_sym_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(567),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(567),
    [anon_sym_COMMA] = ACTIONS(567),
    [anon_sym_COLON_COLON] = ACTIONS(567),
    [anon_sym_DASH_GT] = ACTIONS(567),
    [anon_sym_POUND] = ACTIONS(567),
    [anon_sym_SQUOTE] = ACTIONS(565),
    [anon_sym_as] = ACTIONS(565),
    [anon_sym_async] = ACTIONS(565),
    [anon_sym_await] = ACTIONS(565),
    [anon_sym_break] = ACTIONS(565),
    [anon_sym_const] = ACTIONS(565),
    [anon_sym_continue] = ACTIONS(565),
    [anon_sym_default] = ACTIONS(565),
    [anon_sym_enum] = ACTIONS(565),
    [anon_sym_fn] = ACTIONS(565),
    [anon_sym_for] = ACTIONS(565),
    [anon_sym_gen] = ACTIONS(565),
    [anon_sym_if] = ACTIONS(565),
    [anon_sym_impl] = ACTIONS(565),
    [anon_sym_let] = ACTIONS(565),
    [anon_sym_loop] = ACTIONS(565),
    [anon_sym_match] = ACTIONS(565),
    [anon_sym_mod] = ACTIONS(565),
    [anon_sym_pub] = ACTIONS(565),
    [anon_sym_return] = ACTIONS(565),
    [anon_sym_static] = ACTIONS(565),
    [anon_sym_struct] = ACTIONS(565),
    [anon_sym_trait] = ACTIONS(565),
    [anon_sym_type] = ACTIONS(565),
    [anon_sym_union] = ACTIONS(565),
    [anon_sym_unsafe] = ACTIONS(565),
    [anon_sym_use] = ACTIONS(565),
    [anon_sym_where] = ACTIONS(565),
    [anon_sym_while] = ACTIONS(565),
    [sym_mutable_specifier] = ACTIONS(565),
    [sym_integer_literal] = ACTIONS(581),
    [aux_sym_string_literal_token1] = ACTIONS(583),
    [sym_char_literal] = ACTIONS(581),
    [anon_sym_true] = ACTIONS(585),
    [anon_sym_false] = ACTIONS(585),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(565),
    [sym_super] = ACTIONS(565),
    [sym_crate] = ACTIONS(565),
    [sym_metavariable] = ACTIONS(587),
    [sym__raw_string_literal_start] = ACTIONS(589),
    [sym_float_literal] = ACTIONS(581),
  },
  [80] = {
    [sym__token_pattern] = STATE(176),
    [sym_token_tree_pattern] = STATE(170),
    [sym_token_binding_pattern] = STATE(170),
    [sym_token_repetition_pattern] = STATE(170),
    [sym__literal] = STATE(170),
    [sym_string_literal] = STATE(177),
    [sym_raw_string_literal] = STATE(177),
    [sym_boolean_literal] = STATE(177),
    [sym_line_comment] = STATE(80),
    [sym_block_comment] = STATE(80),
    [aux_sym_token_tree_pattern_repeat1] = STATE(67),
    [aux_sym__non_special_token_repeat1] = STATE(136),
    [sym_identifier] = ACTIONS(565),
    [anon_sym_SEMI] = ACTIONS(567),
    [anon_sym_LPAREN] = ACTIONS(569),
    [anon_sym_RPAREN] = ACTIONS(597),
    [anon_sym_LBRACK] = ACTIONS(573),
    [anon_sym_LBRACE] = ACTIONS(575),
    [anon_sym_EQ_GT] = ACTIONS(567),
    [anon_sym_COLON] = ACTIONS(577),
    [anon_sym_DOLLAR] = ACTIONS(579),
    [anon_sym_PLUS] = ACTIONS(577),
    [anon_sym_STAR] = ACTIONS(577),
    [anon_sym_QMARK] = ACTIONS(567),
    [anon_sym_u8] = ACTIONS(565),
    [anon_sym_i8] = ACTIONS(565),
    [anon_sym_u16] = ACTIONS(565),
    [anon_sym_i16] = ACTIONS(565),
    [anon_sym_u32] = ACTIONS(565),
    [anon_sym_i32] = ACTIONS(565),
    [anon_sym_u64] = ACTIONS(565),
    [anon_sym_i64] = ACTIONS(565),
    [anon_sym_u128] = ACTIONS(565),
    [anon_sym_i128] = ACTIONS(565),
    [anon_sym_isize] = ACTIONS(565),
    [anon_sym_usize] = ACTIONS(565),
    [anon_sym_f32] = ACTIONS(565),
    [anon_sym_f64] = ACTIONS(565),
    [anon_sym_bool] = ACTIONS(565),
    [anon_sym_str] = ACTIONS(565),
    [anon_sym_char] = ACTIONS(565),
    [anon_sym_DASH] = ACTIONS(577),
    [anon_sym_SLASH] = ACTIONS(577),
    [anon_sym_PERCENT] = ACTIONS(577),
    [anon_sym_CARET] = ACTIONS(577),
    [anon_sym_BANG] = ACTIONS(577),
    [anon_sym_AMP] = ACTIONS(577),
    [anon_sym_PIPE] = ACTIONS(577),
    [anon_sym_AMP_AMP] = ACTIONS(567),
    [anon_sym_PIPE_PIPE] = ACTIONS(567),
    [anon_sym_LT_LT] = ACTIONS(577),
    [anon_sym_GT_GT] = ACTIONS(577),
    [anon_sym_PLUS_EQ] = ACTIONS(567),
    [anon_sym_DASH_EQ] = ACTIONS(567),
    [anon_sym_STAR_EQ] = ACTIONS(567),
    [anon_sym_SLASH_EQ] = ACTIONS(567),
    [anon_sym_PERCENT_EQ] = ACTIONS(567),
    [anon_sym_CARET_EQ] = ACTIONS(567),
    [anon_sym_AMP_EQ] = ACTIONS(567),
    [anon_sym_PIPE_EQ] = ACTIONS(567),
    [anon_sym_LT_LT_EQ] = ACTIONS(567),
    [anon_sym_GT_GT_EQ] = ACTIONS(567),
    [anon_sym_EQ] = ACTIONS(577),
    [anon_sym_EQ_EQ] = ACTIONS(567),
    [anon_sym_BANG_EQ] = ACTIONS(567),
    [anon_sym_GT] = ACTIONS(577),
    [anon_sym_LT] = ACTIONS(577),
    [anon_sym_GT_EQ] = ACTIONS(567),
    [anon_sym_LT_EQ] = ACTIONS(567),
    [anon_sym_AT] = ACTIONS(567),
    [anon_sym__] = ACTIONS(577),
    [anon_sym_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(567),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(567),
    [anon_sym_COMMA] = ACTIONS(567),
    [anon_sym_COLON_COLON] = ACTIONS(567),
    [anon_sym_DASH_GT] = ACTIONS(567),
    [anon_sym_POUND] = ACTIONS(567),
    [anon_sym_SQUOTE] = ACTIONS(565),
    [anon_sym_as] = ACTIONS(565),
    [anon_sym_async] = ACTIONS(565),
    [anon_sym_await] = ACTIONS(565),
    [anon_sym_break] = ACTIONS(565),
    [anon_sym_const] = ACTIONS(565),
    [anon_sym_continue] = ACTIONS(565),
    [anon_sym_default] = ACTIONS(565),
    [anon_sym_enum] = ACTIONS(565),
    [anon_sym_fn] = ACTIONS(565),
    [anon_sym_for] = ACTIONS(565),
    [anon_sym_gen] = ACTIONS(565),
    [anon_sym_if] = ACTIONS(565),
    [anon_sym_impl] = ACTIONS(565),
    [anon_sym_let] = ACTIONS(565),
    [anon_sym_loop] = ACTIONS(565),
    [anon_sym_match] = ACTIONS(565),
    [anon_sym_mod] = ACTIONS(565),
    [anon_sym_pub] = ACTIONS(565),
    [anon_sym_return] = ACTIONS(565),
    [anon_sym_static] = ACTIONS(565),
    [anon_sym_struct] = ACTIONS(565),
    [anon_sym_trait] = ACTIONS(565),
    [anon_sym_type] = ACTIONS(565),
    [anon_sym_union] = ACTIONS(565),
    [anon_sym_unsafe] = ACTIONS(565),
    [anon_sym_use] = ACTIONS(565),
    [anon_sym_where] = ACTIONS(565),
    [anon_sym_while] = ACTIONS(565),
    [sym_mutable_specifier] = ACTIONS(565),
    [sym_integer_literal] = ACTIONS(581),
    [aux_sym_string_literal_token1] = ACTIONS(583),
    [sym_char_literal] = ACTIONS(581),
    [anon_sym_true] = ACTIONS(585),
    [anon_sym_false] = ACTIONS(585),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(565),
    [sym_super] = ACTIONS(565),
    [sym_crate] = ACTIONS(565),
    [sym_metavariable] = ACTIONS(587),
    [sym__raw_string_literal_start] = ACTIONS(589),
    [sym_float_literal] = ACTIONS(581),
  },
  [81] = {
    [sym__token_pattern] = STATE(176),
    [sym_token_tree_pattern] = STATE(170),
    [sym_token_binding_pattern] = STATE(170),
    [sym_token_repetition_pattern] = STATE(170),
    [sym__literal] = STATE(170),
    [sym_string_literal] = STATE(177),
    [sym_raw_string_literal] = STATE(177),
    [sym_boolean_literal] = STATE(177),
    [sym_line_comment] = STATE(81),
    [sym_block_comment] = STATE(81),
    [aux_sym_token_tree_pattern_repeat1] = STATE(80),
    [aux_sym__non_special_token_repeat1] = STATE(136),
    [sym_identifier] = ACTIONS(565),
    [anon_sym_SEMI] = ACTIONS(567),
    [anon_sym_LPAREN] = ACTIONS(569),
    [anon_sym_RPAREN] = ACTIONS(599),
    [anon_sym_LBRACK] = ACTIONS(573),
    [anon_sym_LBRACE] = ACTIONS(575),
    [anon_sym_EQ_GT] = ACTIONS(567),
    [anon_sym_COLON] = ACTIONS(577),
    [anon_sym_DOLLAR] = ACTIONS(579),
    [anon_sym_PLUS] = ACTIONS(577),
    [anon_sym_STAR] = ACTIONS(577),
    [anon_sym_QMARK] = ACTIONS(567),
    [anon_sym_u8] = ACTIONS(565),
    [anon_sym_i8] = ACTIONS(565),
    [anon_sym_u16] = ACTIONS(565),
    [anon_sym_i16] = ACTIONS(565),
    [anon_sym_u32] = ACTIONS(565),
    [anon_sym_i32] = ACTIONS(565),
    [anon_sym_u64] = ACTIONS(565),
    [anon_sym_i64] = ACTIONS(565),
    [anon_sym_u128] = ACTIONS(565),
    [anon_sym_i128] = ACTIONS(565),
    [anon_sym_isize] = ACTIONS(565),
    [anon_sym_usize] = ACTIONS(565),
    [anon_sym_f32] = ACTIONS(565),
    [anon_sym_f64] = ACTIONS(565),
    [anon_sym_bool] = ACTIONS(565),
    [anon_sym_str] = ACTIONS(565),
    [anon_sym_char] = ACTIONS(565),
    [anon_sym_DASH] = ACTIONS(577),
    [anon_sym_SLASH] = ACTIONS(577),
    [anon_sym_PERCENT] = ACTIONS(577),
    [anon_sym_CARET] = ACTIONS(577),
    [anon_sym_BANG] = ACTIONS(577),
    [anon_sym_AMP] = ACTIONS(577),
    [anon_sym_PIPE] = ACTIONS(577),
    [anon_sym_AMP_AMP] = ACTIONS(567),
    [anon_sym_PIPE_PIPE] = ACTIONS(567),
    [anon_sym_LT_LT] = ACTIONS(577),
    [anon_sym_GT_GT] = ACTIONS(577),
    [anon_sym_PLUS_EQ] = ACTIONS(567),
    [anon_sym_DASH_EQ] = ACTIONS(567),
    [anon_sym_STAR_EQ] = ACTIONS(567),
    [anon_sym_SLASH_EQ] = ACTIONS(567),
    [anon_sym_PERCENT_EQ] = ACTIONS(567),
    [anon_sym_CARET_EQ] = ACTIONS(567),
    [anon_sym_AMP_EQ] = ACTIONS(567),
    [anon_sym_PIPE_EQ] = ACTIONS(567),
    [anon_sym_LT_LT_EQ] = ACTIONS(567),
    [anon_sym_GT_GT_EQ] = ACTIONS(567),
    [anon_sym_EQ] = ACTIONS(577),
    [anon_sym_EQ_EQ] = ACTIONS(567),
    [anon_sym_BANG_EQ] = ACTIONS(567),
    [anon_sym_GT] = ACTIONS(577),
    [anon_sym_LT] = ACTIONS(577),
    [anon_sym_GT_EQ] = ACTIONS(567),
    [anon_sym_LT_EQ] = ACTIONS(567),
    [anon_sym_AT] = ACTIONS(567),
    [anon_sym__] = ACTIONS(577),
    [anon_sym_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(567),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(567),
    [anon_sym_COMMA] = ACTIONS(567),
    [anon_sym_COLON_COLON] = ACTIONS(567),
    [anon_sym_DASH_GT] = ACTIONS(567),
    [anon_sym_POUND] = ACTIONS(567),
    [anon_sym_SQUOTE] = ACTIONS(565),
    [anon_sym_as] = ACTIONS(565),
    [anon_sym_async] = ACTIONS(565),
    [anon_sym_await] = ACTIONS(565),
    [anon_sym_break] = ACTIONS(565),
    [anon_sym_const] = ACTIONS(565),
    [anon_sym_continue] = ACTIONS(565),
    [anon_sym_default] = ACTIONS(565),
    [anon_sym_enum] = ACTIONS(565),
    [anon_sym_fn] = ACTIONS(565),
    [anon_sym_for] = ACTIONS(565),
    [anon_sym_gen] = ACTIONS(565),
    [anon_sym_if] = ACTIONS(565),
    [anon_sym_impl] = ACTIONS(565),
    [anon_sym_let] = ACTIONS(565),
    [anon_sym_loop] = ACTIONS(565),
    [anon_sym_match] = ACTIONS(565),
    [anon_sym_mod] = ACTIONS(565),
    [anon_sym_pub] = ACTIONS(565),
    [anon_sym_return] = ACTIONS(565),
    [anon_sym_static] = ACTIONS(565),
    [anon_sym_struct] = ACTIONS(565),
    [anon_sym_trait] = ACTIONS(565),
    [anon_sym_type] = ACTIONS(565),
    [anon_sym_union] = ACTIONS(565),
    [anon_sym_unsafe] = ACTIONS(565),
    [anon_sym_use] = ACTIONS(565),
    [anon_sym_where] = ACTIONS(565),
    [anon_sym_while] = ACTIONS(565),
    [sym_mutable_specifier] = ACTIONS(565),
    [sym_integer_literal] = ACTIONS(581),
    [aux_sym_string_literal_token1] = ACTIONS(583),
    [sym_char_literal] = ACTIONS(581),
    [anon_sym_true] = ACTIONS(585),
    [anon_sym_false] = ACTIONS(585),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(565),
    [sym_super] = ACTIONS(565),
    [sym_crate] = ACTIONS(565),
    [sym_metavariable] = ACTIONS(587),
    [sym__raw_string_literal_start] = ACTIONS(589),
    [sym_float_literal] = ACTIONS(581),
  },
  [82] = {
    [sym_token_tree] = STATE(152),
    [sym_token_repetition] = STATE(152),
    [sym__literal] = STATE(152),
    [sym_string_literal] = STATE(177),
    [sym_raw_string_literal] = STATE(177),
    [sym_boolean_literal] = STATE(177),
    [sym_line_comment] = STATE(82),
    [sym_block_comment] = STATE(82),
    [aux_sym_token_tree_repeat1] = STATE(82),
    [aux_sym__non_special_token_repeat1] = STATE(143),
    [sym_identifier] = ACTIONS(601),
    [anon_sym_SEMI] = ACTIONS(604),
    [anon_sym_LPAREN] = ACTIONS(607),
    [anon_sym_RPAREN] = ACTIONS(610),
    [anon_sym_LBRACK] = ACTIONS(612),
    [anon_sym_RBRACK] = ACTIONS(610),
    [anon_sym_LBRACE] = ACTIONS(615),
    [anon_sym_RBRACE] = ACTIONS(610),
    [anon_sym_EQ_GT] = ACTIONS(604),
    [anon_sym_COLON] = ACTIONS(618),
    [anon_sym_DOLLAR] = ACTIONS(621),
    [anon_sym_PLUS] = ACTIONS(618),
    [anon_sym_STAR] = ACTIONS(618),
    [anon_sym_QMARK] = ACTIONS(604),
    [anon_sym_u8] = ACTIONS(601),
    [anon_sym_i8] = ACTIONS(601),
    [anon_sym_u16] = ACTIONS(601),
    [anon_sym_i16] = ACTIONS(601),
    [anon_sym_u32] = ACTIONS(601),
    [anon_sym_i32] = ACTIONS(601),
    [anon_sym_u64] = ACTIONS(601),
    [anon_sym_i64] = ACTIONS(601),
    [anon_sym_u128] = ACTIONS(601),
    [anon_sym_i128] = ACTIONS(601),
    [anon_sym_isize] = ACTIONS(601),
    [anon_sym_usize] = ACTIONS(601),
    [anon_sym_f32] = ACTIONS(601),
    [anon_sym_f64] = ACTIONS(601),
    [anon_sym_bool] = ACTIONS(601),
    [anon_sym_str] = ACTIONS(601),
    [anon_sym_char] = ACTIONS(601),
    [anon_sym_DASH] = ACTIONS(618),
    [anon_sym_SLASH] = ACTIONS(618),
    [anon_sym_PERCENT] = ACTIONS(618),
    [anon_sym_CARET] = ACTIONS(618),
    [anon_sym_BANG] = ACTIONS(618),
    [anon_sym_AMP] = ACTIONS(618),
    [anon_sym_PIPE] = ACTIONS(618),
    [anon_sym_AMP_AMP] = ACTIONS(604),
    [anon_sym_PIPE_PIPE] = ACTIONS(604),
    [anon_sym_LT_LT] = ACTIONS(618),
    [anon_sym_GT_GT] = ACTIONS(618),
    [anon_sym_PLUS_EQ] = ACTIONS(604),
    [anon_sym_DASH_EQ] = ACTIONS(604),
    [anon_sym_STAR_EQ] = ACTIONS(604),
    [anon_sym_SLASH_EQ] = ACTIONS(604),
    [anon_sym_PERCENT_EQ] = ACTIONS(604),
    [anon_sym_CARET_EQ] = ACTIONS(604),
    [anon_sym_AMP_EQ] = ACTIONS(604),
    [anon_sym_PIPE_EQ] = ACTIONS(604),
    [anon_sym_LT_LT_EQ] = ACTIONS(604),
    [anon_sym_GT_GT_EQ] = ACTIONS(604),
    [anon_sym_EQ] = ACTIONS(618),
    [anon_sym_EQ_EQ] = ACTIONS(604),
    [anon_sym_BANG_EQ] = ACTIONS(604),
    [anon_sym_GT] = ACTIONS(618),
    [anon_sym_LT] = ACTIONS(618),
    [anon_sym_GT_EQ] = ACTIONS(604),
    [anon_sym_LT_EQ] = ACTIONS(604),
    [anon_sym_AT] = ACTIONS(604),
    [anon_sym__] = ACTIONS(618),
    [anon_sym_DOT] = ACTIONS(618),
    [anon_sym_DOT_DOT] = ACTIONS(618),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(604),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(604),
    [anon_sym_COMMA] = ACTIONS(604),
    [anon_sym_COLON_COLON] = ACTIONS(604),
    [anon_sym_DASH_GT] = ACTIONS(604),
    [anon_sym_POUND] = ACTIONS(604),
    [anon_sym_SQUOTE] = ACTIONS(601),
    [anon_sym_as] = ACTIONS(601),
    [anon_sym_async] = ACTIONS(601),
    [anon_sym_await] = ACTIONS(601),
    [anon_sym_break] = ACTIONS(601),
    [anon_sym_const] = ACTIONS(601),
    [anon_sym_continue] = ACTIONS(601),
    [anon_sym_default] = ACTIONS(601),
    [anon_sym_enum] = ACTIONS(601),
    [anon_sym_fn] = ACTIONS(601),
    [anon_sym_for] = ACTIONS(601),
    [anon_sym_gen] = ACTIONS(601),
    [anon_sym_if] = ACTIONS(601),
    [anon_sym_impl] = ACTIONS(601),
    [anon_sym_let] = ACTIONS(601),
    [anon_sym_loop] = ACTIONS(601),
    [anon_sym_match] = ACTIONS(601),
    [anon_sym_mod] = ACTIONS(601),
    [anon_sym_pub] = ACTIONS(601),
    [anon_sym_return] = ACTIONS(601),
    [anon_sym_static] = ACTIONS(601),
    [anon_sym_struct] = ACTIONS(601),
    [anon_sym_trait] = ACTIONS(601),
    [anon_sym_type] = ACTIONS(601),
    [anon_sym_union] = ACTIONS(601),
    [anon_sym_unsafe] = ACTIONS(601),
    [anon_sym_use] = ACTIONS(601),
    [anon_sym_where] = ACTIONS(601),
    [anon_sym_while] = ACTIONS(601),
    [sym_mutable_specifier] = ACTIONS(601),
    [sym_integer_literal] = ACTIONS(624),
    [aux_sym_string_literal_token1] = ACTIONS(627),
    [sym_char_literal] = ACTIONS(624),
    [anon_sym_true] = ACTIONS(630),
    [anon_sym_false] = ACTIONS(630),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(601),
    [sym_super] = ACTIONS(601),
    [sym_crate] = ACTIONS(601),
    [sym_metavariable] = ACTIONS(633),
    [sym__raw_string_literal_start] = ACTIONS(636),
    [sym_float_literal] = ACTIONS(624),
  },
  [83] = {
    [sym_delim_token_tree] = STATE(198),
    [sym__delim_tokens] = STATE(206),
    [sym__non_delim_token] = STATE(198),
    [sym__literal] = STATE(214),
    [sym_string_literal] = STATE(199),
    [sym_raw_string_literal] = STATE(199),
    [sym_boolean_literal] = STATE(199),
    [sym_line_comment] = STATE(83),
    [sym_block_comment] = STATE(83),
    [aux_sym__non_special_token_repeat1] = STATE(157),
    [aux_sym_delim_token_tree_repeat1] = STATE(83),
    [sym_identifier] = ACTIONS(639),
    [anon_sym_SEMI] = ACTIONS(642),
    [anon_sym_LPAREN] = ACTIONS(645),
    [anon_sym_RPAREN] = ACTIONS(648),
    [anon_sym_LBRACK] = ACTIONS(650),
    [anon_sym_RBRACK] = ACTIONS(648),
    [anon_sym_LBRACE] = ACTIONS(653),
    [anon_sym_RBRACE] = ACTIONS(648),
    [anon_sym_EQ_GT] = ACTIONS(642),
    [anon_sym_COLON] = ACTIONS(656),
    [anon_sym_DOLLAR] = ACTIONS(659),
    [anon_sym_PLUS] = ACTIONS(656),
    [anon_sym_STAR] = ACTIONS(656),
    [anon_sym_QMARK] = ACTIONS(642),
    [anon_sym_u8] = ACTIONS(639),
    [anon_sym_i8] = ACTIONS(639),
    [anon_sym_u16] = ACTIONS(639),
    [anon_sym_i16] = ACTIONS(639),
    [anon_sym_u32] = ACTIONS(639),
    [anon_sym_i32] = ACTIONS(639),
    [anon_sym_u64] = ACTIONS(639),
    [anon_sym_i64] = ACTIONS(639),
    [anon_sym_u128] = ACTIONS(639),
    [anon_sym_i128] = ACTIONS(639),
    [anon_sym_isize] = ACTIONS(639),
    [anon_sym_usize] = ACTIONS(639),
    [anon_sym_f32] = ACTIONS(639),
    [anon_sym_f64] = ACTIONS(639),
    [anon_sym_bool] = ACTIONS(639),
    [anon_sym_str] = ACTIONS(639),
    [anon_sym_char] = ACTIONS(639),
    [anon_sym_DASH] = ACTIONS(656),
    [anon_sym_SLASH] = ACTIONS(656),
    [anon_sym_PERCENT] = ACTIONS(656),
    [anon_sym_CARET] = ACTIONS(656),
    [anon_sym_BANG] = ACTIONS(656),
    [anon_sym_AMP] = ACTIONS(656),
    [anon_sym_PIPE] = ACTIONS(656),
    [anon_sym_AMP_AMP] = ACTIONS(642),
    [anon_sym_PIPE_PIPE] = ACTIONS(642),
    [anon_sym_LT_LT] = ACTIONS(656),
    [anon_sym_GT_GT] = ACTIONS(656),
    [anon_sym_PLUS_EQ] = ACTIONS(642),
    [anon_sym_DASH_EQ] = ACTIONS(642),
    [anon_sym_STAR_EQ] = ACTIONS(642),
    [anon_sym_SLASH_EQ] = ACTIONS(642),
    [anon_sym_PERCENT_EQ] = ACTIONS(642),
    [anon_sym_CARET_EQ] = ACTIONS(642),
    [anon_sym_AMP_EQ] = ACTIONS(642),
    [anon_sym_PIPE_EQ] = ACTIONS(642),
    [anon_sym_LT_LT_EQ] = ACTIONS(642),
    [anon_sym_GT_GT_EQ] = ACTIONS(642),
    [anon_sym_EQ] = ACTIONS(656),
    [anon_sym_EQ_EQ] = ACTIONS(642),
    [anon_sym_BANG_EQ] = ACTIONS(642),
    [anon_sym_GT] = ACTIONS(656),
    [anon_sym_LT] = ACTIONS(656),
    [anon_sym_GT_EQ] = ACTIONS(642),
    [anon_sym_LT_EQ] = ACTIONS(642),
    [anon_sym_AT] = ACTIONS(642),
    [anon_sym__] = ACTIONS(656),
    [anon_sym_DOT] = ACTIONS(656),
    [anon_sym_DOT_DOT] = ACTIONS(656),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(642),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(642),
    [anon_sym_COMMA] = ACTIONS(642),
    [anon_sym_COLON_COLON] = ACTIONS(642),
    [anon_sym_DASH_GT] = ACTIONS(642),
    [anon_sym_POUND] = ACTIONS(642),
    [anon_sym_SQUOTE] = ACTIONS(639),
    [anon_sym_as] = ACTIONS(639),
    [anon_sym_async] = ACTIONS(639),
    [anon_sym_await] = ACTIONS(639),
    [anon_sym_break] = ACTIONS(639),
    [anon_sym_const] = ACTIONS(639),
    [anon_sym_continue] = ACTIONS(639),
    [anon_sym_default] = ACTIONS(639),
    [anon_sym_enum] = ACTIONS(639),
    [anon_sym_fn] = ACTIONS(639),
    [anon_sym_for] = ACTIONS(639),
    [anon_sym_gen] = ACTIONS(639),
    [anon_sym_if] = ACTIONS(639),
    [anon_sym_impl] = ACTIONS(639),
    [anon_sym_let] = ACTIONS(639),
    [anon_sym_loop] = ACTIONS(639),
    [anon_sym_match] = ACTIONS(639),
    [anon_sym_mod] = ACTIONS(639),
    [anon_sym_pub] = ACTIONS(639),
    [anon_sym_return] = ACTIONS(639),
    [anon_sym_static] = ACTIONS(639),
    [anon_sym_struct] = ACTIONS(639),
    [anon_sym_trait] = ACTIONS(639),
    [anon_sym_type] = ACTIONS(639),
    [anon_sym_union] = ACTIONS(639),
    [anon_sym_unsafe] = ACTIONS(639),
    [anon_sym_use] = ACTIONS(639),
    [anon_sym_where] = ACTIONS(639),
    [anon_sym_while] = ACTIONS(639),
    [sym_mutable_specifier] = ACTIONS(639),
    [sym_integer_literal] = ACTIONS(662),
    [aux_sym_string_literal_token1] = ACTIONS(665),
    [sym_char_literal] = ACTIONS(662),
    [anon_sym_true] = ACTIONS(668),
    [anon_sym_false] = ACTIONS(668),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(639),
    [sym_super] = ACTIONS(639),
    [sym_crate] = ACTIONS(639),
    [sym__raw_string_literal_start] = ACTIONS(671),
    [sym_float_literal] = ACTIONS(662),
  },
  [84] = {
    [sym_delim_token_tree] = STATE(198),
    [sym__delim_tokens] = STATE(206),
    [sym__non_delim_token] = STATE(198),
    [sym__literal] = STATE(214),
    [sym_string_literal] = STATE(199),
    [sym_raw_string_literal] = STATE(199),
    [sym_boolean_literal] = STATE(199),
    [sym_line_comment] = STATE(84),
    [sym_block_comment] = STATE(84),
    [aux_sym__non_special_token_repeat1] = STATE(157),
    [aux_sym_delim_token_tree_repeat1] = STATE(127),
    [sym_identifier] = ACTIONS(674),
    [anon_sym_SEMI] = ACTIONS(676),
    [anon_sym_LPAREN] = ACTIONS(678),
    [anon_sym_LBRACK] = ACTIONS(680),
    [anon_sym_LBRACE] = ACTIONS(682),
    [anon_sym_RBRACE] = ACTIONS(684),
    [anon_sym_EQ_GT] = ACTIONS(676),
    [anon_sym_COLON] = ACTIONS(686),
    [anon_sym_DOLLAR] = ACTIONS(688),
    [anon_sym_PLUS] = ACTIONS(686),
    [anon_sym_STAR] = ACTIONS(686),
    [anon_sym_QMARK] = ACTIONS(676),
    [anon_sym_u8] = ACTIONS(674),
    [anon_sym_i8] = ACTIONS(674),
    [anon_sym_u16] = ACTIONS(674),
    [anon_sym_i16] = ACTIONS(674),
    [anon_sym_u32] = ACTIONS(674),
    [anon_sym_i32] = ACTIONS(674),
    [anon_sym_u64] = ACTIONS(674),
    [anon_sym_i64] = ACTIONS(674),
    [anon_sym_u128] = ACTIONS(674),
    [anon_sym_i128] = ACTIONS(674),
    [anon_sym_isize] = ACTIONS(674),
    [anon_sym_usize] = ACTIONS(674),
    [anon_sym_f32] = ACTIONS(674),
    [anon_sym_f64] = ACTIONS(674),
    [anon_sym_bool] = ACTIONS(674),
    [anon_sym_str] = ACTIONS(674),
    [anon_sym_char] = ACTIONS(674),
    [anon_sym_DASH] = ACTIONS(686),
    [anon_sym_SLASH] = ACTIONS(686),
    [anon_sym_PERCENT] = ACTIONS(686),
    [anon_sym_CARET] = ACTIONS(686),
    [anon_sym_BANG] = ACTIONS(686),
    [anon_sym_AMP] = ACTIONS(686),
    [anon_sym_PIPE] = ACTIONS(686),
    [anon_sym_AMP_AMP] = ACTIONS(676),
    [anon_sym_PIPE_PIPE] = ACTIONS(676),
    [anon_sym_LT_LT] = ACTIONS(686),
    [anon_sym_GT_GT] = ACTIONS(686),
    [anon_sym_PLUS_EQ] = ACTIONS(676),
    [anon_sym_DASH_EQ] = ACTIONS(676),
    [anon_sym_STAR_EQ] = ACTIONS(676),
    [anon_sym_SLASH_EQ] = ACTIONS(676),
    [anon_sym_PERCENT_EQ] = ACTIONS(676),
    [anon_sym_CARET_EQ] = ACTIONS(676),
    [anon_sym_AMP_EQ] = ACTIONS(676),
    [anon_sym_PIPE_EQ] = ACTIONS(676),
    [anon_sym_LT_LT_EQ] = ACTIONS(676),
    [anon_sym_GT_GT_EQ] = ACTIONS(676),
    [anon_sym_EQ] = ACTIONS(686),
    [anon_sym_EQ_EQ] = ACTIONS(676),
    [anon_sym_BANG_EQ] = ACTIONS(676),
    [anon_sym_GT] = ACTIONS(686),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT_EQ] = ACTIONS(676),
    [anon_sym_LT_EQ] = ACTIONS(676),
    [anon_sym_AT] = ACTIONS(676),
    [anon_sym__] = ACTIONS(686),
    [anon_sym_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(676),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(676),
    [anon_sym_COMMA] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(676),
    [anon_sym_DASH_GT] = ACTIONS(676),
    [anon_sym_POUND] = ACTIONS(676),
    [anon_sym_SQUOTE] = ACTIONS(674),
    [anon_sym_as] = ACTIONS(674),
    [anon_sym_async] = ACTIONS(674),
    [anon_sym_await] = ACTIONS(674),
    [anon_sym_break] = ACTIONS(674),
    [anon_sym_const] = ACTIONS(674),
    [anon_sym_continue] = ACTIONS(674),
    [anon_sym_default] = ACTIONS(674),
    [anon_sym_enum] = ACTIONS(674),
    [anon_sym_fn] = ACTIONS(674),
    [anon_sym_for] = ACTIONS(674),
    [anon_sym_gen] = ACTIONS(674),
    [anon_sym_if] = ACTIONS(674),
    [anon_sym_impl] = ACTIONS(674),
    [anon_sym_let] = ACTIONS(674),
    [anon_sym_loop] = ACTIONS(674),
    [anon_sym_match] = ACTIONS(674),
    [anon_sym_mod] = ACTIONS(674),
    [anon_sym_pub] = ACTIONS(674),
    [anon_sym_return] = ACTIONS(674),
    [anon_sym_static] = ACTIONS(674),
    [anon_sym_struct] = ACTIONS(674),
    [anon_sym_trait] = ACTIONS(674),
    [anon_sym_type] = ACTIONS(674),
    [anon_sym_union] = ACTIONS(674),
    [anon_sym_unsafe] = ACTIONS(674),
    [anon_sym_use] = ACTIONS(674),
    [anon_sym_where] = ACTIONS(674),
    [anon_sym_while] = ACTIONS(674),
    [sym_mutable_specifier] = ACTIONS(674),
    [sym_integer_literal] = ACTIONS(690),
    [aux_sym_string_literal_token1] = ACTIONS(692),
    [sym_char_literal] = ACTIONS(690),
    [anon_sym_true] = ACTIONS(694),
    [anon_sym_false] = ACTIONS(694),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(674),
    [sym_super] = ACTIONS(674),
    [sym_crate] = ACTIONS(674),
    [sym__raw_string_literal_start] = ACTIONS(696),
    [sym_float_literal] = ACTIONS(690),
  },
  [85] = {
    [sym_delim_token_tree] = STATE(198),
    [sym__delim_tokens] = STATE(206),
    [sym__non_delim_token] = STATE(198),
    [sym__literal] = STATE(214),
    [sym_string_literal] = STATE(199),
    [sym_raw_string_literal] = STATE(199),
    [sym_boolean_literal] = STATE(199),
    [sym_line_comment] = STATE(85),
    [sym_block_comment] = STATE(85),
    [aux_sym__non_special_token_repeat1] = STATE(157),
    [aux_sym_delim_token_tree_repeat1] = STATE(83),
    [sym_identifier] = ACTIONS(674),
    [anon_sym_SEMI] = ACTIONS(676),
    [anon_sym_LPAREN] = ACTIONS(678),
    [anon_sym_RPAREN] = ACTIONS(698),
    [anon_sym_LBRACK] = ACTIONS(680),
    [anon_sym_LBRACE] = ACTIONS(682),
    [anon_sym_EQ_GT] = ACTIONS(676),
    [anon_sym_COLON] = ACTIONS(686),
    [anon_sym_DOLLAR] = ACTIONS(688),
    [anon_sym_PLUS] = ACTIONS(686),
    [anon_sym_STAR] = ACTIONS(686),
    [anon_sym_QMARK] = ACTIONS(676),
    [anon_sym_u8] = ACTIONS(674),
    [anon_sym_i8] = ACTIONS(674),
    [anon_sym_u16] = ACTIONS(674),
    [anon_sym_i16] = ACTIONS(674),
    [anon_sym_u32] = ACTIONS(674),
    [anon_sym_i32] = ACTIONS(674),
    [anon_sym_u64] = ACTIONS(674),
    [anon_sym_i64] = ACTIONS(674),
    [anon_sym_u128] = ACTIONS(674),
    [anon_sym_i128] = ACTIONS(674),
    [anon_sym_isize] = ACTIONS(674),
    [anon_sym_usize] = ACTIONS(674),
    [anon_sym_f32] = ACTIONS(674),
    [anon_sym_f64] = ACTIONS(674),
    [anon_sym_bool] = ACTIONS(674),
    [anon_sym_str] = ACTIONS(674),
    [anon_sym_char] = ACTIONS(674),
    [anon_sym_DASH] = ACTIONS(686),
    [anon_sym_SLASH] = ACTIONS(686),
    [anon_sym_PERCENT] = ACTIONS(686),
    [anon_sym_CARET] = ACTIONS(686),
    [anon_sym_BANG] = ACTIONS(686),
    [anon_sym_AMP] = ACTIONS(686),
    [anon_sym_PIPE] = ACTIONS(686),
    [anon_sym_AMP_AMP] = ACTIONS(676),
    [anon_sym_PIPE_PIPE] = ACTIONS(676),
    [anon_sym_LT_LT] = ACTIONS(686),
    [anon_sym_GT_GT] = ACTIONS(686),
    [anon_sym_PLUS_EQ] = ACTIONS(676),
    [anon_sym_DASH_EQ] = ACTIONS(676),
    [anon_sym_STAR_EQ] = ACTIONS(676),
    [anon_sym_SLASH_EQ] = ACTIONS(676),
    [anon_sym_PERCENT_EQ] = ACTIONS(676),
    [anon_sym_CARET_EQ] = ACTIONS(676),
    [anon_sym_AMP_EQ] = ACTIONS(676),
    [anon_sym_PIPE_EQ] = ACTIONS(676),
    [anon_sym_LT_LT_EQ] = ACTIONS(676),
    [anon_sym_GT_GT_EQ] = ACTIONS(676),
    [anon_sym_EQ] = ACTIONS(686),
    [anon_sym_EQ_EQ] = ACTIONS(676),
    [anon_sym_BANG_EQ] = ACTIONS(676),
    [anon_sym_GT] = ACTIONS(686),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT_EQ] = ACTIONS(676),
    [anon_sym_LT_EQ] = ACTIONS(676),
    [anon_sym_AT] = ACTIONS(676),
    [anon_sym__] = ACTIONS(686),
    [anon_sym_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(676),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(676),
    [anon_sym_COMMA] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(676),
    [anon_sym_DASH_GT] = ACTIONS(676),
    [anon_sym_POUND] = ACTIONS(676),
    [anon_sym_SQUOTE] = ACTIONS(674),
    [anon_sym_as] = ACTIONS(674),
    [anon_sym_async] = ACTIONS(674),
    [anon_sym_await] = ACTIONS(674),
    [anon_sym_break] = ACTIONS(674),
    [anon_sym_const] = ACTIONS(674),
    [anon_sym_continue] = ACTIONS(674),
    [anon_sym_default] = ACTIONS(674),
    [anon_sym_enum] = ACTIONS(674),
    [anon_sym_fn] = ACTIONS(674),
    [anon_sym_for] = ACTIONS(674),
    [anon_sym_gen] = ACTIONS(674),
    [anon_sym_if] = ACTIONS(674),
    [anon_sym_impl] = ACTIONS(674),
    [anon_sym_let] = ACTIONS(674),
    [anon_sym_loop] = ACTIONS(674),
    [anon_sym_match] = ACTIONS(674),
    [anon_sym_mod] = ACTIONS(674),
    [anon_sym_pub] = ACTIONS(674),
    [anon_sym_return] = ACTIONS(674),
    [anon_sym_static] = ACTIONS(674),
    [anon_sym_struct] = ACTIONS(674),
    [anon_sym_trait] = ACTIONS(674),
    [anon_sym_type] = ACTIONS(674),
    [anon_sym_union] = ACTIONS(674),
    [anon_sym_unsafe] = ACTIONS(674),
    [anon_sym_use] = ACTIONS(674),
    [anon_sym_where] = ACTIONS(674),
    [anon_sym_while] = ACTIONS(674),
    [sym_mutable_specifier] = ACTIONS(674),
    [sym_integer_literal] = ACTIONS(690),
    [aux_sym_string_literal_token1] = ACTIONS(692),
    [sym_char_literal] = ACTIONS(690),
    [anon_sym_true] = ACTIONS(694),
    [anon_sym_false] = ACTIONS(694),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(674),
    [sym_super] = ACTIONS(674),
    [sym_crate] = ACTIONS(674),
    [sym__raw_string_literal_start] = ACTIONS(696),
    [sym_float_literal] = ACTIONS(690),
  },
  [86] = {
    [sym_delim_token_tree] = STATE(198),
    [sym__delim_tokens] = STATE(206),
    [sym__non_delim_token] = STATE(198),
    [sym__literal] = STATE(214),
    [sym_string_literal] = STATE(199),
    [sym_raw_string_literal] = STATE(199),
    [sym_boolean_literal] = STATE(199),
    [sym_line_comment] = STATE(86),
    [sym_block_comment] = STATE(86),
    [aux_sym__non_special_token_repeat1] = STATE(157),
    [aux_sym_delim_token_tree_repeat1] = STATE(83),
    [sym_identifier] = ACTIONS(674),
    [anon_sym_SEMI] = ACTIONS(676),
    [anon_sym_LPAREN] = ACTIONS(678),
    [anon_sym_LBRACK] = ACTIONS(680),
    [anon_sym_RBRACK] = ACTIONS(698),
    [anon_sym_LBRACE] = ACTIONS(682),
    [anon_sym_EQ_GT] = ACTIONS(676),
    [anon_sym_COLON] = ACTIONS(686),
    [anon_sym_DOLLAR] = ACTIONS(688),
    [anon_sym_PLUS] = ACTIONS(686),
    [anon_sym_STAR] = ACTIONS(686),
    [anon_sym_QMARK] = ACTIONS(676),
    [anon_sym_u8] = ACTIONS(674),
    [anon_sym_i8] = ACTIONS(674),
    [anon_sym_u16] = ACTIONS(674),
    [anon_sym_i16] = ACTIONS(674),
    [anon_sym_u32] = ACTIONS(674),
    [anon_sym_i32] = ACTIONS(674),
    [anon_sym_u64] = ACTIONS(674),
    [anon_sym_i64] = ACTIONS(674),
    [anon_sym_u128] = ACTIONS(674),
    [anon_sym_i128] = ACTIONS(674),
    [anon_sym_isize] = ACTIONS(674),
    [anon_sym_usize] = ACTIONS(674),
    [anon_sym_f32] = ACTIONS(674),
    [anon_sym_f64] = ACTIONS(674),
    [anon_sym_bool] = ACTIONS(674),
    [anon_sym_str] = ACTIONS(674),
    [anon_sym_char] = ACTIONS(674),
    [anon_sym_DASH] = ACTIONS(686),
    [anon_sym_SLASH] = ACTIONS(686),
    [anon_sym_PERCENT] = ACTIONS(686),
    [anon_sym_CARET] = ACTIONS(686),
    [anon_sym_BANG] = ACTIONS(686),
    [anon_sym_AMP] = ACTIONS(686),
    [anon_sym_PIPE] = ACTIONS(686),
    [anon_sym_AMP_AMP] = ACTIONS(676),
    [anon_sym_PIPE_PIPE] = ACTIONS(676),
    [anon_sym_LT_LT] = ACTIONS(686),
    [anon_sym_GT_GT] = ACTIONS(686),
    [anon_sym_PLUS_EQ] = ACTIONS(676),
    [anon_sym_DASH_EQ] = ACTIONS(676),
    [anon_sym_STAR_EQ] = ACTIONS(676),
    [anon_sym_SLASH_EQ] = ACTIONS(676),
    [anon_sym_PERCENT_EQ] = ACTIONS(676),
    [anon_sym_CARET_EQ] = ACTIONS(676),
    [anon_sym_AMP_EQ] = ACTIONS(676),
    [anon_sym_PIPE_EQ] = ACTIONS(676),
    [anon_sym_LT_LT_EQ] = ACTIONS(676),
    [anon_sym_GT_GT_EQ] = ACTIONS(676),
    [anon_sym_EQ] = ACTIONS(686),
    [anon_sym_EQ_EQ] = ACTIONS(676),
    [anon_sym_BANG_EQ] = ACTIONS(676),
    [anon_sym_GT] = ACTIONS(686),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT_EQ] = ACTIONS(676),
    [anon_sym_LT_EQ] = ACTIONS(676),
    [anon_sym_AT] = ACTIONS(676),
    [anon_sym__] = ACTIONS(686),
    [anon_sym_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(676),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(676),
    [anon_sym_COMMA] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(676),
    [anon_sym_DASH_GT] = ACTIONS(676),
    [anon_sym_POUND] = ACTIONS(676),
    [anon_sym_SQUOTE] = ACTIONS(674),
    [anon_sym_as] = ACTIONS(674),
    [anon_sym_async] = ACTIONS(674),
    [anon_sym_await] = ACTIONS(674),
    [anon_sym_break] = ACTIONS(674),
    [anon_sym_const] = ACTIONS(674),
    [anon_sym_continue] = ACTIONS(674),
    [anon_sym_default] = ACTIONS(674),
    [anon_sym_enum] = ACTIONS(674),
    [anon_sym_fn] = ACTIONS(674),
    [anon_sym_for] = ACTIONS(674),
    [anon_sym_gen] = ACTIONS(674),
    [anon_sym_if] = ACTIONS(674),
    [anon_sym_impl] = ACTIONS(674),
    [anon_sym_let] = ACTIONS(674),
    [anon_sym_loop] = ACTIONS(674),
    [anon_sym_match] = ACTIONS(674),
    [anon_sym_mod] = ACTIONS(674),
    [anon_sym_pub] = ACTIONS(674),
    [anon_sym_return] = ACTIONS(674),
    [anon_sym_static] = ACTIONS(674),
    [anon_sym_struct] = ACTIONS(674),
    [anon_sym_trait] = ACTIONS(674),
    [anon_sym_type] = ACTIONS(674),
    [anon_sym_union] = ACTIONS(674),
    [anon_sym_unsafe] = ACTIONS(674),
    [anon_sym_use] = ACTIONS(674),
    [anon_sym_where] = ACTIONS(674),
    [anon_sym_while] = ACTIONS(674),
    [sym_mutable_specifier] = ACTIONS(674),
    [sym_integer_literal] = ACTIONS(690),
    [aux_sym_string_literal_token1] = ACTIONS(692),
    [sym_char_literal] = ACTIONS(690),
    [anon_sym_true] = ACTIONS(694),
    [anon_sym_false] = ACTIONS(694),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(674),
    [sym_super] = ACTIONS(674),
    [sym_crate] = ACTIONS(674),
    [sym__raw_string_literal_start] = ACTIONS(696),
    [sym_float_literal] = ACTIONS(690),
  },
  [87] = {
    [sym_delim_token_tree] = STATE(198),
    [sym__delim_tokens] = STATE(206),
    [sym__non_delim_token] = STATE(198),
    [sym__literal] = STATE(214),
    [sym_string_literal] = STATE(199),
    [sym_raw_string_literal] = STATE(199),
    [sym_boolean_literal] = STATE(199),
    [sym_line_comment] = STATE(87),
    [sym_block_comment] = STATE(87),
    [aux_sym__non_special_token_repeat1] = STATE(157),
    [aux_sym_delim_token_tree_repeat1] = STATE(83),
    [sym_identifier] = ACTIONS(674),
    [anon_sym_SEMI] = ACTIONS(676),
    [anon_sym_LPAREN] = ACTIONS(678),
    [anon_sym_LBRACK] = ACTIONS(680),
    [anon_sym_LBRACE] = ACTIONS(682),
    [anon_sym_RBRACE] = ACTIONS(698),
    [anon_sym_EQ_GT] = ACTIONS(676),
    [anon_sym_COLON] = ACTIONS(686),
    [anon_sym_DOLLAR] = ACTIONS(688),
    [anon_sym_PLUS] = ACTIONS(686),
    [anon_sym_STAR] = ACTIONS(686),
    [anon_sym_QMARK] = ACTIONS(676),
    [anon_sym_u8] = ACTIONS(674),
    [anon_sym_i8] = ACTIONS(674),
    [anon_sym_u16] = ACTIONS(674),
    [anon_sym_i16] = ACTIONS(674),
    [anon_sym_u32] = ACTIONS(674),
    [anon_sym_i32] = ACTIONS(674),
    [anon_sym_u64] = ACTIONS(674),
    [anon_sym_i64] = ACTIONS(674),
    [anon_sym_u128] = ACTIONS(674),
    [anon_sym_i128] = ACTIONS(674),
    [anon_sym_isize] = ACTIONS(674),
    [anon_sym_usize] = ACTIONS(674),
    [anon_sym_f32] = ACTIONS(674),
    [anon_sym_f64] = ACTIONS(674),
    [anon_sym_bool] = ACTIONS(674),
    [anon_sym_str] = ACTIONS(674),
    [anon_sym_char] = ACTIONS(674),
    [anon_sym_DASH] = ACTIONS(686),
    [anon_sym_SLASH] = ACTIONS(686),
    [anon_sym_PERCENT] = ACTIONS(686),
    [anon_sym_CARET] = ACTIONS(686),
    [anon_sym_BANG] = ACTIONS(686),
    [anon_sym_AMP] = ACTIONS(686),
    [anon_sym_PIPE] = ACTIONS(686),
    [anon_sym_AMP_AMP] = ACTIONS(676),
    [anon_sym_PIPE_PIPE] = ACTIONS(676),
    [anon_sym_LT_LT] = ACTIONS(686),
    [anon_sym_GT_GT] = ACTIONS(686),
    [anon_sym_PLUS_EQ] = ACTIONS(676),
    [anon_sym_DASH_EQ] = ACTIONS(676),
    [anon_sym_STAR_EQ] = ACTIONS(676),
    [anon_sym_SLASH_EQ] = ACTIONS(676),
    [anon_sym_PERCENT_EQ] = ACTIONS(676),
    [anon_sym_CARET_EQ] = ACTIONS(676),
    [anon_sym_AMP_EQ] = ACTIONS(676),
    [anon_sym_PIPE_EQ] = ACTIONS(676),
    [anon_sym_LT_LT_EQ] = ACTIONS(676),
    [anon_sym_GT_GT_EQ] = ACTIONS(676),
    [anon_sym_EQ] = ACTIONS(686),
    [anon_sym_EQ_EQ] = ACTIONS(676),
    [anon_sym_BANG_EQ] = ACTIONS(676),
    [anon_sym_GT] = ACTIONS(686),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT_EQ] = ACTIONS(676),
    [anon_sym_LT_EQ] = ACTIONS(676),
    [anon_sym_AT] = ACTIONS(676),
    [anon_sym__] = ACTIONS(686),
    [anon_sym_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(676),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(676),
    [anon_sym_COMMA] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(676),
    [anon_sym_DASH_GT] = ACTIONS(676),
    [anon_sym_POUND] = ACTIONS(676),
    [anon_sym_SQUOTE] = ACTIONS(674),
    [anon_sym_as] = ACTIONS(674),
    [anon_sym_async] = ACTIONS(674),
    [anon_sym_await] = ACTIONS(674),
    [anon_sym_break] = ACTIONS(674),
    [anon_sym_const] = ACTIONS(674),
    [anon_sym_continue] = ACTIONS(674),
    [anon_sym_default] = ACTIONS(674),
    [anon_sym_enum] = ACTIONS(674),
    [anon_sym_fn] = ACTIONS(674),
    [anon_sym_for] = ACTIONS(674),
    [anon_sym_gen] = ACTIONS(674),
    [anon_sym_if] = ACTIONS(674),
    [anon_sym_impl] = ACTIONS(674),
    [anon_sym_let] = ACTIONS(674),
    [anon_sym_loop] = ACTIONS(674),
    [anon_sym_match] = ACTIONS(674),
    [anon_sym_mod] = ACTIONS(674),
    [anon_sym_pub] = ACTIONS(674),
    [anon_sym_return] = ACTIONS(674),
    [anon_sym_static] = ACTIONS(674),
    [anon_sym_struct] = ACTIONS(674),
    [anon_sym_trait] = ACTIONS(674),
    [anon_sym_type] = ACTIONS(674),
    [anon_sym_union] = ACTIONS(674),
    [anon_sym_unsafe] = ACTIONS(674),
    [anon_sym_use] = ACTIONS(674),
    [anon_sym_where] = ACTIONS(674),
    [anon_sym_while] = ACTIONS(674),
    [sym_mutable_specifier] = ACTIONS(674),
    [sym_integer_literal] = ACTIONS(690),
    [aux_sym_string_literal_token1] = ACTIONS(692),
    [sym_char_literal] = ACTIONS(690),
    [anon_sym_true] = ACTIONS(694),
    [anon_sym_false] = ACTIONS(694),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(674),
    [sym_super] = ACTIONS(674),
    [sym_crate] = ACTIONS(674),
    [sym__raw_string_literal_start] = ACTIONS(696),
    [sym_float_literal] = ACTIONS(690),
  },
  [88] = {
    [sym_token_tree] = STATE(152),
    [sym_token_repetition] = STATE(152),
    [sym__literal] = STATE(152),
    [sym_string_literal] = STATE(177),
    [sym_raw_string_literal] = STATE(177),
    [sym_boolean_literal] = STATE(177),
    [sym_line_comment] = STATE(88),
    [sym_block_comment] = STATE(88),
    [aux_sym_token_tree_repeat1] = STATE(82),
    [aux_sym__non_special_token_repeat1] = STATE(143),
    [sym_identifier] = ACTIONS(700),
    [anon_sym_SEMI] = ACTIONS(567),
    [anon_sym_LPAREN] = ACTIONS(702),
    [anon_sym_LBRACK] = ACTIONS(704),
    [anon_sym_RBRACK] = ACTIONS(706),
    [anon_sym_LBRACE] = ACTIONS(708),
    [anon_sym_EQ_GT] = ACTIONS(567),
    [anon_sym_COLON] = ACTIONS(577),
    [anon_sym_DOLLAR] = ACTIONS(710),
    [anon_sym_PLUS] = ACTIONS(577),
    [anon_sym_STAR] = ACTIONS(577),
    [anon_sym_QMARK] = ACTIONS(567),
    [anon_sym_u8] = ACTIONS(700),
    [anon_sym_i8] = ACTIONS(700),
    [anon_sym_u16] = ACTIONS(700),
    [anon_sym_i16] = ACTIONS(700),
    [anon_sym_u32] = ACTIONS(700),
    [anon_sym_i32] = ACTIONS(700),
    [anon_sym_u64] = ACTIONS(700),
    [anon_sym_i64] = ACTIONS(700),
    [anon_sym_u128] = ACTIONS(700),
    [anon_sym_i128] = ACTIONS(700),
    [anon_sym_isize] = ACTIONS(700),
    [anon_sym_usize] = ACTIONS(700),
    [anon_sym_f32] = ACTIONS(700),
    [anon_sym_f64] = ACTIONS(700),
    [anon_sym_bool] = ACTIONS(700),
    [anon_sym_str] = ACTIONS(700),
    [anon_sym_char] = ACTIONS(700),
    [anon_sym_DASH] = ACTIONS(577),
    [anon_sym_SLASH] = ACTIONS(577),
    [anon_sym_PERCENT] = ACTIONS(577),
    [anon_sym_CARET] = ACTIONS(577),
    [anon_sym_BANG] = ACTIONS(577),
    [anon_sym_AMP] = ACTIONS(577),
    [anon_sym_PIPE] = ACTIONS(577),
    [anon_sym_AMP_AMP] = ACTIONS(567),
    [anon_sym_PIPE_PIPE] = ACTIONS(567),
    [anon_sym_LT_LT] = ACTIONS(577),
    [anon_sym_GT_GT] = ACTIONS(577),
    [anon_sym_PLUS_EQ] = ACTIONS(567),
    [anon_sym_DASH_EQ] = ACTIONS(567),
    [anon_sym_STAR_EQ] = ACTIONS(567),
    [anon_sym_SLASH_EQ] = ACTIONS(567),
    [anon_sym_PERCENT_EQ] = ACTIONS(567),
    [anon_sym_CARET_EQ] = ACTIONS(567),
    [anon_sym_AMP_EQ] = ACTIONS(567),
    [anon_sym_PIPE_EQ] = ACTIONS(567),
    [anon_sym_LT_LT_EQ] = ACTIONS(567),
    [anon_sym_GT_GT_EQ] = ACTIONS(567),
    [anon_sym_EQ] = ACTIONS(577),
    [anon_sym_EQ_EQ] = ACTIONS(567),
    [anon_sym_BANG_EQ] = ACTIONS(567),
    [anon_sym_GT] = ACTIONS(577),
    [anon_sym_LT] = ACTIONS(577),
    [anon_sym_GT_EQ] = ACTIONS(567),
    [anon_sym_LT_EQ] = ACTIONS(567),
    [anon_sym_AT] = ACTIONS(567),
    [anon_sym__] = ACTIONS(577),
    [anon_sym_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(567),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(567),
    [anon_sym_COMMA] = ACTIONS(567),
    [anon_sym_COLON_COLON] = ACTIONS(567),
    [anon_sym_DASH_GT] = ACTIONS(567),
    [anon_sym_POUND] = ACTIONS(567),
    [anon_sym_SQUOTE] = ACTIONS(700),
    [anon_sym_as] = ACTIONS(700),
    [anon_sym_async] = ACTIONS(700),
    [anon_sym_await] = ACTIONS(700),
    [anon_sym_break] = ACTIONS(700),
    [anon_sym_const] = ACTIONS(700),
    [anon_sym_continue] = ACTIONS(700),
    [anon_sym_default] = ACTIONS(700),
    [anon_sym_enum] = ACTIONS(700),
    [anon_sym_fn] = ACTIONS(700),
    [anon_sym_for] = ACTIONS(700),
    [anon_sym_gen] = ACTIONS(700),
    [anon_sym_if] = ACTIONS(700),
    [anon_sym_impl] = ACTIONS(700),
    [anon_sym_let] = ACTIONS(700),
    [anon_sym_loop] = ACTIONS(700),
    [anon_sym_match] = ACTIONS(700),
    [anon_sym_mod] = ACTIONS(700),
    [anon_sym_pub] = ACTIONS(700),
    [anon_sym_return] = ACTIONS(700),
    [anon_sym_static] = ACTIONS(700),
    [anon_sym_struct] = ACTIONS(700),
    [anon_sym_trait] = ACTIONS(700),
    [anon_sym_type] = ACTIONS(700),
    [anon_sym_union] = ACTIONS(700),
    [anon_sym_unsafe] = ACTIONS(700),
    [anon_sym_use] = ACTIONS(700),
    [anon_sym_where] = ACTIONS(700),
    [anon_sym_while] = ACTIONS(700),
    [sym_mutable_specifier] = ACTIONS(700),
    [sym_integer_literal] = ACTIONS(581),
    [aux_sym_string_literal_token1] = ACTIONS(583),
    [sym_char_literal] = ACTIONS(581),
    [anon_sym_true] = ACTIONS(585),
    [anon_sym_false] = ACTIONS(585),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(700),
    [sym_super] = ACTIONS(700),
    [sym_crate] = ACTIONS(700),
    [sym_metavariable] = ACTIONS(712),
    [sym__raw_string_literal_start] = ACTIONS(589),
    [sym_float_literal] = ACTIONS(581),
  },
  [89] = {
    [sym_token_tree] = STATE(152),
    [sym_token_repetition] = STATE(152),
    [sym__literal] = STATE(152),
    [sym_string_literal] = STATE(177),
    [sym_raw_string_literal] = STATE(177),
    [sym_boolean_literal] = STATE(177),
    [sym_line_comment] = STATE(89),
    [sym_block_comment] = STATE(89),
    [aux_sym_token_tree_repeat1] = STATE(82),
    [aux_sym__non_special_token_repeat1] = STATE(143),
    [sym_identifier] = ACTIONS(700),
    [anon_sym_SEMI] = ACTIONS(567),
    [anon_sym_LPAREN] = ACTIONS(702),
    [anon_sym_LBRACK] = ACTIONS(704),
    [anon_sym_LBRACE] = ACTIONS(708),
    [anon_sym_RBRACE] = ACTIONS(706),
    [anon_sym_EQ_GT] = ACTIONS(567),
    [anon_sym_COLON] = ACTIONS(577),
    [anon_sym_DOLLAR] = ACTIONS(710),
    [anon_sym_PLUS] = ACTIONS(577),
    [anon_sym_STAR] = ACTIONS(577),
    [anon_sym_QMARK] = ACTIONS(567),
    [anon_sym_u8] = ACTIONS(700),
    [anon_sym_i8] = ACTIONS(700),
    [anon_sym_u16] = ACTIONS(700),
    [anon_sym_i16] = ACTIONS(700),
    [anon_sym_u32] = ACTIONS(700),
    [anon_sym_i32] = ACTIONS(700),
    [anon_sym_u64] = ACTIONS(700),
    [anon_sym_i64] = ACTIONS(700),
    [anon_sym_u128] = ACTIONS(700),
    [anon_sym_i128] = ACTIONS(700),
    [anon_sym_isize] = ACTIONS(700),
    [anon_sym_usize] = ACTIONS(700),
    [anon_sym_f32] = ACTIONS(700),
    [anon_sym_f64] = ACTIONS(700),
    [anon_sym_bool] = ACTIONS(700),
    [anon_sym_str] = ACTIONS(700),
    [anon_sym_char] = ACTIONS(700),
    [anon_sym_DASH] = ACTIONS(577),
    [anon_sym_SLASH] = ACTIONS(577),
    [anon_sym_PERCENT] = ACTIONS(577),
    [anon_sym_CARET] = ACTIONS(577),
    [anon_sym_BANG] = ACTIONS(577),
    [anon_sym_AMP] = ACTIONS(577),
    [anon_sym_PIPE] = ACTIONS(577),
    [anon_sym_AMP_AMP] = ACTIONS(567),
    [anon_sym_PIPE_PIPE] = ACTIONS(567),
    [anon_sym_LT_LT] = ACTIONS(577),
    [anon_sym_GT_GT] = ACTIONS(577),
    [anon_sym_PLUS_EQ] = ACTIONS(567),
    [anon_sym_DASH_EQ] = ACTIONS(567),
    [anon_sym_STAR_EQ] = ACTIONS(567),
    [anon_sym_SLASH_EQ] = ACTIONS(567),
    [anon_sym_PERCENT_EQ] = ACTIONS(567),
    [anon_sym_CARET_EQ] = ACTIONS(567),
    [anon_sym_AMP_EQ] = ACTIONS(567),
    [anon_sym_PIPE_EQ] = ACTIONS(567),
    [anon_sym_LT_LT_EQ] = ACTIONS(567),
    [anon_sym_GT_GT_EQ] = ACTIONS(567),
    [anon_sym_EQ] = ACTIONS(577),
    [anon_sym_EQ_EQ] = ACTIONS(567),
    [anon_sym_BANG_EQ] = ACTIONS(567),
    [anon_sym_GT] = ACTIONS(577),
    [anon_sym_LT] = ACTIONS(577),
    [anon_sym_GT_EQ] = ACTIONS(567),
    [anon_sym_LT_EQ] = ACTIONS(567),
    [anon_sym_AT] = ACTIONS(567),
    [anon_sym__] = ACTIONS(577),
    [anon_sym_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(567),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(567),
    [anon_sym_COMMA] = ACTIONS(567),
    [anon_sym_COLON_COLON] = ACTIONS(567),
    [anon_sym_DASH_GT] = ACTIONS(567),
    [anon_sym_POUND] = ACTIONS(567),
    [anon_sym_SQUOTE] = ACTIONS(700),
    [anon_sym_as] = ACTIONS(700),
    [anon_sym_async] = ACTIONS(700),
    [anon_sym_await] = ACTIONS(700),
    [anon_sym_break] = ACTIONS(700),
    [anon_sym_const] = ACTIONS(700),
    [anon_sym_continue] = ACTIONS(700),
    [anon_sym_default] = ACTIONS(700),
    [anon_sym_enum] = ACTIONS(700),
    [anon_sym_fn] = ACTIONS(700),
    [anon_sym_for] = ACTIONS(700),
    [anon_sym_gen] = ACTIONS(700),
    [anon_sym_if] = ACTIONS(700),
    [anon_sym_impl] = ACTIONS(700),
    [anon_sym_let] = ACTIONS(700),
    [anon_sym_loop] = ACTIONS(700),
    [anon_sym_match] = ACTIONS(700),
    [anon_sym_mod] = ACTIONS(700),
    [anon_sym_pub] = ACTIONS(700),
    [anon_sym_return] = ACTIONS(700),
    [anon_sym_static] = ACTIONS(700),
    [anon_sym_struct] = ACTIONS(700),
    [anon_sym_trait] = ACTIONS(700),
    [anon_sym_type] = ACTIONS(700),
    [anon_sym_union] = ACTIONS(700),
    [anon_sym_unsafe] = ACTIONS(700),
    [anon_sym_use] = ACTIONS(700),
    [anon_sym_where] = ACTIONS(700),
    [anon_sym_while] = ACTIONS(700),
    [sym_mutable_specifier] = ACTIONS(700),
    [sym_integer_literal] = ACTIONS(581),
    [aux_sym_string_literal_token1] = ACTIONS(583),
    [sym_char_literal] = ACTIONS(581),
    [anon_sym_true] = ACTIONS(585),
    [anon_sym_false] = ACTIONS(585),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(700),
    [sym_super] = ACTIONS(700),
    [sym_crate] = ACTIONS(700),
    [sym_metavariable] = ACTIONS(712),
    [sym__raw_string_literal_start] = ACTIONS(589),
    [sym_float_literal] = ACTIONS(581),
  },
  [90] = {
    [sym_token_tree] = STATE(152),
    [sym_token_repetition] = STATE(152),
    [sym__literal] = STATE(152),
    [sym_string_literal] = STATE(177),
    [sym_raw_string_literal] = STATE(177),
    [sym_boolean_literal] = STATE(177),
    [sym_line_comment] = STATE(90),
    [sym_block_comment] = STATE(90),
    [aux_sym_token_tree_repeat1] = STATE(98),
    [aux_sym__non_special_token_repeat1] = STATE(143),
    [sym_identifier] = ACTIONS(700),
    [anon_sym_SEMI] = ACTIONS(567),
    [anon_sym_LPAREN] = ACTIONS(702),
    [anon_sym_RPAREN] = ACTIONS(714),
    [anon_sym_LBRACK] = ACTIONS(704),
    [anon_sym_LBRACE] = ACTIONS(708),
    [anon_sym_EQ_GT] = ACTIONS(567),
    [anon_sym_COLON] = ACTIONS(577),
    [anon_sym_DOLLAR] = ACTIONS(710),
    [anon_sym_PLUS] = ACTIONS(577),
    [anon_sym_STAR] = ACTIONS(577),
    [anon_sym_QMARK] = ACTIONS(567),
    [anon_sym_u8] = ACTIONS(700),
    [anon_sym_i8] = ACTIONS(700),
    [anon_sym_u16] = ACTIONS(700),
    [anon_sym_i16] = ACTIONS(700),
    [anon_sym_u32] = ACTIONS(700),
    [anon_sym_i32] = ACTIONS(700),
    [anon_sym_u64] = ACTIONS(700),
    [anon_sym_i64] = ACTIONS(700),
    [anon_sym_u128] = ACTIONS(700),
    [anon_sym_i128] = ACTIONS(700),
    [anon_sym_isize] = ACTIONS(700),
    [anon_sym_usize] = ACTIONS(700),
    [anon_sym_f32] = ACTIONS(700),
    [anon_sym_f64] = ACTIONS(700),
    [anon_sym_bool] = ACTIONS(700),
    [anon_sym_str] = ACTIONS(700),
    [anon_sym_char] = ACTIONS(700),
    [anon_sym_DASH] = ACTIONS(577),
    [anon_sym_SLASH] = ACTIONS(577),
    [anon_sym_PERCENT] = ACTIONS(577),
    [anon_sym_CARET] = ACTIONS(577),
    [anon_sym_BANG] = ACTIONS(577),
    [anon_sym_AMP] = ACTIONS(577),
    [anon_sym_PIPE] = ACTIONS(577),
    [anon_sym_AMP_AMP] = ACTIONS(567),
    [anon_sym_PIPE_PIPE] = ACTIONS(567),
    [anon_sym_LT_LT] = ACTIONS(577),
    [anon_sym_GT_GT] = ACTIONS(577),
    [anon_sym_PLUS_EQ] = ACTIONS(567),
    [anon_sym_DASH_EQ] = ACTIONS(567),
    [anon_sym_STAR_EQ] = ACTIONS(567),
    [anon_sym_SLASH_EQ] = ACTIONS(567),
    [anon_sym_PERCENT_EQ] = ACTIONS(567),
    [anon_sym_CARET_EQ] = ACTIONS(567),
    [anon_sym_AMP_EQ] = ACTIONS(567),
    [anon_sym_PIPE_EQ] = ACTIONS(567),
    [anon_sym_LT_LT_EQ] = ACTIONS(567),
    [anon_sym_GT_GT_EQ] = ACTIONS(567),
    [anon_sym_EQ] = ACTIONS(577),
    [anon_sym_EQ_EQ] = ACTIONS(567),
    [anon_sym_BANG_EQ] = ACTIONS(567),
    [anon_sym_GT] = ACTIONS(577),
    [anon_sym_LT] = ACTIONS(577),
    [anon_sym_GT_EQ] = ACTIONS(567),
    [anon_sym_LT_EQ] = ACTIONS(567),
    [anon_sym_AT] = ACTIONS(567),
    [anon_sym__] = ACTIONS(577),
    [anon_sym_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(567),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(567),
    [anon_sym_COMMA] = ACTIONS(567),
    [anon_sym_COLON_COLON] = ACTIONS(567),
    [anon_sym_DASH_GT] = ACTIONS(567),
    [anon_sym_POUND] = ACTIONS(567),
    [anon_sym_SQUOTE] = ACTIONS(700),
    [anon_sym_as] = ACTIONS(700),
    [anon_sym_async] = ACTIONS(700),
    [anon_sym_await] = ACTIONS(700),
    [anon_sym_break] = ACTIONS(700),
    [anon_sym_const] = ACTIONS(700),
    [anon_sym_continue] = ACTIONS(700),
    [anon_sym_default] = ACTIONS(700),
    [anon_sym_enum] = ACTIONS(700),
    [anon_sym_fn] = ACTIONS(700),
    [anon_sym_for] = ACTIONS(700),
    [anon_sym_gen] = ACTIONS(700),
    [anon_sym_if] = ACTIONS(700),
    [anon_sym_impl] = ACTIONS(700),
    [anon_sym_let] = ACTIONS(700),
    [anon_sym_loop] = ACTIONS(700),
    [anon_sym_match] = ACTIONS(700),
    [anon_sym_mod] = ACTIONS(700),
    [anon_sym_pub] = ACTIONS(700),
    [anon_sym_return] = ACTIONS(700),
    [anon_sym_static] = ACTIONS(700),
    [anon_sym_struct] = ACTIONS(700),
    [anon_sym_trait] = ACTIONS(700),
    [anon_sym_type] = ACTIONS(700),
    [anon_sym_union] = ACTIONS(700),
    [anon_sym_unsafe] = ACTIONS(700),
    [anon_sym_use] = ACTIONS(700),
    [anon_sym_where] = ACTIONS(700),
    [anon_sym_while] = ACTIONS(700),
    [sym_mutable_specifier] = ACTIONS(700),
    [sym_integer_literal] = ACTIONS(581),
    [aux_sym_string_literal_token1] = ACTIONS(583),
    [sym_char_literal] = ACTIONS(581),
    [anon_sym_true] = ACTIONS(585),
    [anon_sym_false] = ACTIONS(585),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(700),
    [sym_super] = ACTIONS(700),
    [sym_crate] = ACTIONS(700),
    [sym_metavariable] = ACTIONS(712),
    [sym__raw_string_literal_start] = ACTIONS(589),
    [sym_float_literal] = ACTIONS(581),
  },
  [91] = {
    [sym_token_tree] = STATE(152),
    [sym_token_repetition] = STATE(152),
    [sym__literal] = STATE(152),
    [sym_string_literal] = STATE(177),
    [sym_raw_string_literal] = STATE(177),
    [sym_boolean_literal] = STATE(177),
    [sym_line_comment] = STATE(91),
    [sym_block_comment] = STATE(91),
    [aux_sym_token_tree_repeat1] = STATE(94),
    [aux_sym__non_special_token_repeat1] = STATE(143),
    [sym_identifier] = ACTIONS(700),
    [anon_sym_SEMI] = ACTIONS(567),
    [anon_sym_LPAREN] = ACTIONS(702),
    [anon_sym_RPAREN] = ACTIONS(716),
    [anon_sym_LBRACK] = ACTIONS(704),
    [anon_sym_LBRACE] = ACTIONS(708),
    [anon_sym_EQ_GT] = ACTIONS(567),
    [anon_sym_COLON] = ACTIONS(577),
    [anon_sym_DOLLAR] = ACTIONS(710),
    [anon_sym_PLUS] = ACTIONS(577),
    [anon_sym_STAR] = ACTIONS(577),
    [anon_sym_QMARK] = ACTIONS(567),
    [anon_sym_u8] = ACTIONS(700),
    [anon_sym_i8] = ACTIONS(700),
    [anon_sym_u16] = ACTIONS(700),
    [anon_sym_i16] = ACTIONS(700),
    [anon_sym_u32] = ACTIONS(700),
    [anon_sym_i32] = ACTIONS(700),
    [anon_sym_u64] = ACTIONS(700),
    [anon_sym_i64] = ACTIONS(700),
    [anon_sym_u128] = ACTIONS(700),
    [anon_sym_i128] = ACTIONS(700),
    [anon_sym_isize] = ACTIONS(700),
    [anon_sym_usize] = ACTIONS(700),
    [anon_sym_f32] = ACTIONS(700),
    [anon_sym_f64] = ACTIONS(700),
    [anon_sym_bool] = ACTIONS(700),
    [anon_sym_str] = ACTIONS(700),
    [anon_sym_char] = ACTIONS(700),
    [anon_sym_DASH] = ACTIONS(577),
    [anon_sym_SLASH] = ACTIONS(577),
    [anon_sym_PERCENT] = ACTIONS(577),
    [anon_sym_CARET] = ACTIONS(577),
    [anon_sym_BANG] = ACTIONS(577),
    [anon_sym_AMP] = ACTIONS(577),
    [anon_sym_PIPE] = ACTIONS(577),
    [anon_sym_AMP_AMP] = ACTIONS(567),
    [anon_sym_PIPE_PIPE] = ACTIONS(567),
    [anon_sym_LT_LT] = ACTIONS(577),
    [anon_sym_GT_GT] = ACTIONS(577),
    [anon_sym_PLUS_EQ] = ACTIONS(567),
    [anon_sym_DASH_EQ] = ACTIONS(567),
    [anon_sym_STAR_EQ] = ACTIONS(567),
    [anon_sym_SLASH_EQ] = ACTIONS(567),
    [anon_sym_PERCENT_EQ] = ACTIONS(567),
    [anon_sym_CARET_EQ] = ACTIONS(567),
    [anon_sym_AMP_EQ] = ACTIONS(567),
    [anon_sym_PIPE_EQ] = ACTIONS(567),
    [anon_sym_LT_LT_EQ] = ACTIONS(567),
    [anon_sym_GT_GT_EQ] = ACTIONS(567),
    [anon_sym_EQ] = ACTIONS(577),
    [anon_sym_EQ_EQ] = ACTIONS(567),
    [anon_sym_BANG_EQ] = ACTIONS(567),
    [anon_sym_GT] = ACTIONS(577),
    [anon_sym_LT] = ACTIONS(577),
    [anon_sym_GT_EQ] = ACTIONS(567),
    [anon_sym_LT_EQ] = ACTIONS(567),
    [anon_sym_AT] = ACTIONS(567),
    [anon_sym__] = ACTIONS(577),
    [anon_sym_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(567),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(567),
    [anon_sym_COMMA] = ACTIONS(567),
    [anon_sym_COLON_COLON] = ACTIONS(567),
    [anon_sym_DASH_GT] = ACTIONS(567),
    [anon_sym_POUND] = ACTIONS(567),
    [anon_sym_SQUOTE] = ACTIONS(700),
    [anon_sym_as] = ACTIONS(700),
    [anon_sym_async] = ACTIONS(700),
    [anon_sym_await] = ACTIONS(700),
    [anon_sym_break] = ACTIONS(700),
    [anon_sym_const] = ACTIONS(700),
    [anon_sym_continue] = ACTIONS(700),
    [anon_sym_default] = ACTIONS(700),
    [anon_sym_enum] = ACTIONS(700),
    [anon_sym_fn] = ACTIONS(700),
    [anon_sym_for] = ACTIONS(700),
    [anon_sym_gen] = ACTIONS(700),
    [anon_sym_if] = ACTIONS(700),
    [anon_sym_impl] = ACTIONS(700),
    [anon_sym_let] = ACTIONS(700),
    [anon_sym_loop] = ACTIONS(700),
    [anon_sym_match] = ACTIONS(700),
    [anon_sym_mod] = ACTIONS(700),
    [anon_sym_pub] = ACTIONS(700),
    [anon_sym_return] = ACTIONS(700),
    [anon_sym_static] = ACTIONS(700),
    [anon_sym_struct] = ACTIONS(700),
    [anon_sym_trait] = ACTIONS(700),
    [anon_sym_type] = ACTIONS(700),
    [anon_sym_union] = ACTIONS(700),
    [anon_sym_unsafe] = ACTIONS(700),
    [anon_sym_use] = ACTIONS(700),
    [anon_sym_where] = ACTIONS(700),
    [anon_sym_while] = ACTIONS(700),
    [sym_mutable_specifier] = ACTIONS(700),
    [sym_integer_literal] = ACTIONS(581),
    [aux_sym_string_literal_token1] = ACTIONS(583),
    [sym_char_literal] = ACTIONS(581),
    [anon_sym_true] = ACTIONS(585),
    [anon_sym_false] = ACTIONS(585),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(700),
    [sym_super] = ACTIONS(700),
    [sym_crate] = ACTIONS(700),
    [sym_metavariable] = ACTIONS(712),
    [sym__raw_string_literal_start] = ACTIONS(589),
    [sym_float_literal] = ACTIONS(581),
  },
  [92] = {
    [sym_token_tree] = STATE(152),
    [sym_token_repetition] = STATE(152),
    [sym__literal] = STATE(152),
    [sym_string_literal] = STATE(177),
    [sym_raw_string_literal] = STATE(177),
    [sym_boolean_literal] = STATE(177),
    [sym_line_comment] = STATE(92),
    [sym_block_comment] = STATE(92),
    [aux_sym_token_tree_repeat1] = STATE(95),
    [aux_sym__non_special_token_repeat1] = STATE(143),
    [sym_identifier] = ACTIONS(700),
    [anon_sym_SEMI] = ACTIONS(567),
    [anon_sym_LPAREN] = ACTIONS(702),
    [anon_sym_LBRACK] = ACTIONS(704),
    [anon_sym_RBRACK] = ACTIONS(716),
    [anon_sym_LBRACE] = ACTIONS(708),
    [anon_sym_EQ_GT] = ACTIONS(567),
    [anon_sym_COLON] = ACTIONS(577),
    [anon_sym_DOLLAR] = ACTIONS(710),
    [anon_sym_PLUS] = ACTIONS(577),
    [anon_sym_STAR] = ACTIONS(577),
    [anon_sym_QMARK] = ACTIONS(567),
    [anon_sym_u8] = ACTIONS(700),
    [anon_sym_i8] = ACTIONS(700),
    [anon_sym_u16] = ACTIONS(700),
    [anon_sym_i16] = ACTIONS(700),
    [anon_sym_u32] = ACTIONS(700),
    [anon_sym_i32] = ACTIONS(700),
    [anon_sym_u64] = ACTIONS(700),
    [anon_sym_i64] = ACTIONS(700),
    [anon_sym_u128] = ACTIONS(700),
    [anon_sym_i128] = ACTIONS(700),
    [anon_sym_isize] = ACTIONS(700),
    [anon_sym_usize] = ACTIONS(700),
    [anon_sym_f32] = ACTIONS(700),
    [anon_sym_f64] = ACTIONS(700),
    [anon_sym_bool] = ACTIONS(700),
    [anon_sym_str] = ACTIONS(700),
    [anon_sym_char] = ACTIONS(700),
    [anon_sym_DASH] = ACTIONS(577),
    [anon_sym_SLASH] = ACTIONS(577),
    [anon_sym_PERCENT] = ACTIONS(577),
    [anon_sym_CARET] = ACTIONS(577),
    [anon_sym_BANG] = ACTIONS(577),
    [anon_sym_AMP] = ACTIONS(577),
    [anon_sym_PIPE] = ACTIONS(577),
    [anon_sym_AMP_AMP] = ACTIONS(567),
    [anon_sym_PIPE_PIPE] = ACTIONS(567),
    [anon_sym_LT_LT] = ACTIONS(577),
    [anon_sym_GT_GT] = ACTIONS(577),
    [anon_sym_PLUS_EQ] = ACTIONS(567),
    [anon_sym_DASH_EQ] = ACTIONS(567),
    [anon_sym_STAR_EQ] = ACTIONS(567),
    [anon_sym_SLASH_EQ] = ACTIONS(567),
    [anon_sym_PERCENT_EQ] = ACTIONS(567),
    [anon_sym_CARET_EQ] = ACTIONS(567),
    [anon_sym_AMP_EQ] = ACTIONS(567),
    [anon_sym_PIPE_EQ] = ACTIONS(567),
    [anon_sym_LT_LT_EQ] = ACTIONS(567),
    [anon_sym_GT_GT_EQ] = ACTIONS(567),
    [anon_sym_EQ] = ACTIONS(577),
    [anon_sym_EQ_EQ] = ACTIONS(567),
    [anon_sym_BANG_EQ] = ACTIONS(567),
    [anon_sym_GT] = ACTIONS(577),
    [anon_sym_LT] = ACTIONS(577),
    [anon_sym_GT_EQ] = ACTIONS(567),
    [anon_sym_LT_EQ] = ACTIONS(567),
    [anon_sym_AT] = ACTIONS(567),
    [anon_sym__] = ACTIONS(577),
    [anon_sym_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(567),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(567),
    [anon_sym_COMMA] = ACTIONS(567),
    [anon_sym_COLON_COLON] = ACTIONS(567),
    [anon_sym_DASH_GT] = ACTIONS(567),
    [anon_sym_POUND] = ACTIONS(567),
    [anon_sym_SQUOTE] = ACTIONS(700),
    [anon_sym_as] = ACTIONS(700),
    [anon_sym_async] = ACTIONS(700),
    [anon_sym_await] = ACTIONS(700),
    [anon_sym_break] = ACTIONS(700),
    [anon_sym_const] = ACTIONS(700),
    [anon_sym_continue] = ACTIONS(700),
    [anon_sym_default] = ACTIONS(700),
    [anon_sym_enum] = ACTIONS(700),
    [anon_sym_fn] = ACTIONS(700),
    [anon_sym_for] = ACTIONS(700),
    [anon_sym_gen] = ACTIONS(700),
    [anon_sym_if] = ACTIONS(700),
    [anon_sym_impl] = ACTIONS(700),
    [anon_sym_let] = ACTIONS(700),
    [anon_sym_loop] = ACTIONS(700),
    [anon_sym_match] = ACTIONS(700),
    [anon_sym_mod] = ACTIONS(700),
    [anon_sym_pub] = ACTIONS(700),
    [anon_sym_return] = ACTIONS(700),
    [anon_sym_static] = ACTIONS(700),
    [anon_sym_struct] = ACTIONS(700),
    [anon_sym_trait] = ACTIONS(700),
    [anon_sym_type] = ACTIONS(700),
    [anon_sym_union] = ACTIONS(700),
    [anon_sym_unsafe] = ACTIONS(700),
    [anon_sym_use] = ACTIONS(700),
    [anon_sym_where] = ACTIONS(700),
    [anon_sym_while] = ACTIONS(700),
    [sym_mutable_specifier] = ACTIONS(700),
    [sym_integer_literal] = ACTIONS(581),
    [aux_sym_string_literal_token1] = ACTIONS(583),
    [sym_char_literal] = ACTIONS(581),
    [anon_sym_true] = ACTIONS(585),
    [anon_sym_false] = ACTIONS(585),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(700),
    [sym_super] = ACTIONS(700),
    [sym_crate] = ACTIONS(700),
    [sym_metavariable] = ACTIONS(712),
    [sym__raw_string_literal_start] = ACTIONS(589),
    [sym_float_literal] = ACTIONS(581),
  },
  [93] = {
    [sym_token_tree] = STATE(152),
    [sym_token_repetition] = STATE(152),
    [sym__literal] = STATE(152),
    [sym_string_literal] = STATE(177),
    [sym_raw_string_literal] = STATE(177),
    [sym_boolean_literal] = STATE(177),
    [sym_line_comment] = STATE(93),
    [sym_block_comment] = STATE(93),
    [aux_sym_token_tree_repeat1] = STATE(96),
    [aux_sym__non_special_token_repeat1] = STATE(143),
    [sym_identifier] = ACTIONS(700),
    [anon_sym_SEMI] = ACTIONS(567),
    [anon_sym_LPAREN] = ACTIONS(702),
    [anon_sym_LBRACK] = ACTIONS(704),
    [anon_sym_LBRACE] = ACTIONS(708),
    [anon_sym_RBRACE] = ACTIONS(716),
    [anon_sym_EQ_GT] = ACTIONS(567),
    [anon_sym_COLON] = ACTIONS(577),
    [anon_sym_DOLLAR] = ACTIONS(710),
    [anon_sym_PLUS] = ACTIONS(577),
    [anon_sym_STAR] = ACTIONS(577),
    [anon_sym_QMARK] = ACTIONS(567),
    [anon_sym_u8] = ACTIONS(700),
    [anon_sym_i8] = ACTIONS(700),
    [anon_sym_u16] = ACTIONS(700),
    [anon_sym_i16] = ACTIONS(700),
    [anon_sym_u32] = ACTIONS(700),
    [anon_sym_i32] = ACTIONS(700),
    [anon_sym_u64] = ACTIONS(700),
    [anon_sym_i64] = ACTIONS(700),
    [anon_sym_u128] = ACTIONS(700),
    [anon_sym_i128] = ACTIONS(700),
    [anon_sym_isize] = ACTIONS(700),
    [anon_sym_usize] = ACTIONS(700),
    [anon_sym_f32] = ACTIONS(700),
    [anon_sym_f64] = ACTIONS(700),
    [anon_sym_bool] = ACTIONS(700),
    [anon_sym_str] = ACTIONS(700),
    [anon_sym_char] = ACTIONS(700),
    [anon_sym_DASH] = ACTIONS(577),
    [anon_sym_SLASH] = ACTIONS(577),
    [anon_sym_PERCENT] = ACTIONS(577),
    [anon_sym_CARET] = ACTIONS(577),
    [anon_sym_BANG] = ACTIONS(577),
    [anon_sym_AMP] = ACTIONS(577),
    [anon_sym_PIPE] = ACTIONS(577),
    [anon_sym_AMP_AMP] = ACTIONS(567),
    [anon_sym_PIPE_PIPE] = ACTIONS(567),
    [anon_sym_LT_LT] = ACTIONS(577),
    [anon_sym_GT_GT] = ACTIONS(577),
    [anon_sym_PLUS_EQ] = ACTIONS(567),
    [anon_sym_DASH_EQ] = ACTIONS(567),
    [anon_sym_STAR_EQ] = ACTIONS(567),
    [anon_sym_SLASH_EQ] = ACTIONS(567),
    [anon_sym_PERCENT_EQ] = ACTIONS(567),
    [anon_sym_CARET_EQ] = ACTIONS(567),
    [anon_sym_AMP_EQ] = ACTIONS(567),
    [anon_sym_PIPE_EQ] = ACTIONS(567),
    [anon_sym_LT_LT_EQ] = ACTIONS(567),
    [anon_sym_GT_GT_EQ] = ACTIONS(567),
    [anon_sym_EQ] = ACTIONS(577),
    [anon_sym_EQ_EQ] = ACTIONS(567),
    [anon_sym_BANG_EQ] = ACTIONS(567),
    [anon_sym_GT] = ACTIONS(577),
    [anon_sym_LT] = ACTIONS(577),
    [anon_sym_GT_EQ] = ACTIONS(567),
    [anon_sym_LT_EQ] = ACTIONS(567),
    [anon_sym_AT] = ACTIONS(567),
    [anon_sym__] = ACTIONS(577),
    [anon_sym_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(567),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(567),
    [anon_sym_COMMA] = ACTIONS(567),
    [anon_sym_COLON_COLON] = ACTIONS(567),
    [anon_sym_DASH_GT] = ACTIONS(567),
    [anon_sym_POUND] = ACTIONS(567),
    [anon_sym_SQUOTE] = ACTIONS(700),
    [anon_sym_as] = ACTIONS(700),
    [anon_sym_async] = ACTIONS(700),
    [anon_sym_await] = ACTIONS(700),
    [anon_sym_break] = ACTIONS(700),
    [anon_sym_const] = ACTIONS(700),
    [anon_sym_continue] = ACTIONS(700),
    [anon_sym_default] = ACTIONS(700),
    [anon_sym_enum] = ACTIONS(700),
    [anon_sym_fn] = ACTIONS(700),
    [anon_sym_for] = ACTIONS(700),
    [anon_sym_gen] = ACTIONS(700),
    [anon_sym_if] = ACTIONS(700),
    [anon_sym_impl] = ACTIONS(700),
    [anon_sym_let] = ACTIONS(700),
    [anon_sym_loop] = ACTIONS(700),
    [anon_sym_match] = ACTIONS(700),
    [anon_sym_mod] = ACTIONS(700),
    [anon_sym_pub] = ACTIONS(700),
    [anon_sym_return] = ACTIONS(700),
    [anon_sym_static] = ACTIONS(700),
    [anon_sym_struct] = ACTIONS(700),
    [anon_sym_trait] = ACTIONS(700),
    [anon_sym_type] = ACTIONS(700),
    [anon_sym_union] = ACTIONS(700),
    [anon_sym_unsafe] = ACTIONS(700),
    [anon_sym_use] = ACTIONS(700),
    [anon_sym_where] = ACTIONS(700),
    [anon_sym_while] = ACTIONS(700),
    [sym_mutable_specifier] = ACTIONS(700),
    [sym_integer_literal] = ACTIONS(581),
    [aux_sym_string_literal_token1] = ACTIONS(583),
    [sym_char_literal] = ACTIONS(581),
    [anon_sym_true] = ACTIONS(585),
    [anon_sym_false] = ACTIONS(585),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(700),
    [sym_super] = ACTIONS(700),
    [sym_crate] = ACTIONS(700),
    [sym_metavariable] = ACTIONS(712),
    [sym__raw_string_literal_start] = ACTIONS(589),
    [sym_float_literal] = ACTIONS(581),
  },
  [94] = {
    [sym_token_tree] = STATE(152),
    [sym_token_repetition] = STATE(152),
    [sym__literal] = STATE(152),
    [sym_string_literal] = STATE(177),
    [sym_raw_string_literal] = STATE(177),
    [sym_boolean_literal] = STATE(177),
    [sym_line_comment] = STATE(94),
    [sym_block_comment] = STATE(94),
    [aux_sym_token_tree_repeat1] = STATE(82),
    [aux_sym__non_special_token_repeat1] = STATE(143),
    [sym_identifier] = ACTIONS(700),
    [anon_sym_SEMI] = ACTIONS(567),
    [anon_sym_LPAREN] = ACTIONS(702),
    [anon_sym_RPAREN] = ACTIONS(718),
    [anon_sym_LBRACK] = ACTIONS(704),
    [anon_sym_LBRACE] = ACTIONS(708),
    [anon_sym_EQ_GT] = ACTIONS(567),
    [anon_sym_COLON] = ACTIONS(577),
    [anon_sym_DOLLAR] = ACTIONS(710),
    [anon_sym_PLUS] = ACTIONS(577),
    [anon_sym_STAR] = ACTIONS(577),
    [anon_sym_QMARK] = ACTIONS(567),
    [anon_sym_u8] = ACTIONS(700),
    [anon_sym_i8] = ACTIONS(700),
    [anon_sym_u16] = ACTIONS(700),
    [anon_sym_i16] = ACTIONS(700),
    [anon_sym_u32] = ACTIONS(700),
    [anon_sym_i32] = ACTIONS(700),
    [anon_sym_u64] = ACTIONS(700),
    [anon_sym_i64] = ACTIONS(700),
    [anon_sym_u128] = ACTIONS(700),
    [anon_sym_i128] = ACTIONS(700),
    [anon_sym_isize] = ACTIONS(700),
    [anon_sym_usize] = ACTIONS(700),
    [anon_sym_f32] = ACTIONS(700),
    [anon_sym_f64] = ACTIONS(700),
    [anon_sym_bool] = ACTIONS(700),
    [anon_sym_str] = ACTIONS(700),
    [anon_sym_char] = ACTIONS(700),
    [anon_sym_DASH] = ACTIONS(577),
    [anon_sym_SLASH] = ACTIONS(577),
    [anon_sym_PERCENT] = ACTIONS(577),
    [anon_sym_CARET] = ACTIONS(577),
    [anon_sym_BANG] = ACTIONS(577),
    [anon_sym_AMP] = ACTIONS(577),
    [anon_sym_PIPE] = ACTIONS(577),
    [anon_sym_AMP_AMP] = ACTIONS(567),
    [anon_sym_PIPE_PIPE] = ACTIONS(567),
    [anon_sym_LT_LT] = ACTIONS(577),
    [anon_sym_GT_GT] = ACTIONS(577),
    [anon_sym_PLUS_EQ] = ACTIONS(567),
    [anon_sym_DASH_EQ] = ACTIONS(567),
    [anon_sym_STAR_EQ] = ACTIONS(567),
    [anon_sym_SLASH_EQ] = ACTIONS(567),
    [anon_sym_PERCENT_EQ] = ACTIONS(567),
    [anon_sym_CARET_EQ] = ACTIONS(567),
    [anon_sym_AMP_EQ] = ACTIONS(567),
    [anon_sym_PIPE_EQ] = ACTIONS(567),
    [anon_sym_LT_LT_EQ] = ACTIONS(567),
    [anon_sym_GT_GT_EQ] = ACTIONS(567),
    [anon_sym_EQ] = ACTIONS(577),
    [anon_sym_EQ_EQ] = ACTIONS(567),
    [anon_sym_BANG_EQ] = ACTIONS(567),
    [anon_sym_GT] = ACTIONS(577),
    [anon_sym_LT] = ACTIONS(577),
    [anon_sym_GT_EQ] = ACTIONS(567),
    [anon_sym_LT_EQ] = ACTIONS(567),
    [anon_sym_AT] = ACTIONS(567),
    [anon_sym__] = ACTIONS(577),
    [anon_sym_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(567),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(567),
    [anon_sym_COMMA] = ACTIONS(567),
    [anon_sym_COLON_COLON] = ACTIONS(567),
    [anon_sym_DASH_GT] = ACTIONS(567),
    [anon_sym_POUND] = ACTIONS(567),
    [anon_sym_SQUOTE] = ACTIONS(700),
    [anon_sym_as] = ACTIONS(700),
    [anon_sym_async] = ACTIONS(700),
    [anon_sym_await] = ACTIONS(700),
    [anon_sym_break] = ACTIONS(700),
    [anon_sym_const] = ACTIONS(700),
    [anon_sym_continue] = ACTIONS(700),
    [anon_sym_default] = ACTIONS(700),
    [anon_sym_enum] = ACTIONS(700),
    [anon_sym_fn] = ACTIONS(700),
    [anon_sym_for] = ACTIONS(700),
    [anon_sym_gen] = ACTIONS(700),
    [anon_sym_if] = ACTIONS(700),
    [anon_sym_impl] = ACTIONS(700),
    [anon_sym_let] = ACTIONS(700),
    [anon_sym_loop] = ACTIONS(700),
    [anon_sym_match] = ACTIONS(700),
    [anon_sym_mod] = ACTIONS(700),
    [anon_sym_pub] = ACTIONS(700),
    [anon_sym_return] = ACTIONS(700),
    [anon_sym_static] = ACTIONS(700),
    [anon_sym_struct] = ACTIONS(700),
    [anon_sym_trait] = ACTIONS(700),
    [anon_sym_type] = ACTIONS(700),
    [anon_sym_union] = ACTIONS(700),
    [anon_sym_unsafe] = ACTIONS(700),
    [anon_sym_use] = ACTIONS(700),
    [anon_sym_where] = ACTIONS(700),
    [anon_sym_while] = ACTIONS(700),
    [sym_mutable_specifier] = ACTIONS(700),
    [sym_integer_literal] = ACTIONS(581),
    [aux_sym_string_literal_token1] = ACTIONS(583),
    [sym_char_literal] = ACTIONS(581),
    [anon_sym_true] = ACTIONS(585),
    [anon_sym_false] = ACTIONS(585),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(700),
    [sym_super] = ACTIONS(700),
    [sym_crate] = ACTIONS(700),
    [sym_metavariable] = ACTIONS(712),
    [sym__raw_string_literal_start] = ACTIONS(589),
    [sym_float_literal] = ACTIONS(581),
  },
  [95] = {
    [sym_token_tree] = STATE(152),
    [sym_token_repetition] = STATE(152),
    [sym__literal] = STATE(152),
    [sym_string_literal] = STATE(177),
    [sym_raw_string_literal] = STATE(177),
    [sym_boolean_literal] = STATE(177),
    [sym_line_comment] = STATE(95),
    [sym_block_comment] = STATE(95),
    [aux_sym_token_tree_repeat1] = STATE(82),
    [aux_sym__non_special_token_repeat1] = STATE(143),
    [sym_identifier] = ACTIONS(700),
    [anon_sym_SEMI] = ACTIONS(567),
    [anon_sym_LPAREN] = ACTIONS(702),
    [anon_sym_LBRACK] = ACTIONS(704),
    [anon_sym_RBRACK] = ACTIONS(718),
    [anon_sym_LBRACE] = ACTIONS(708),
    [anon_sym_EQ_GT] = ACTIONS(567),
    [anon_sym_COLON] = ACTIONS(577),
    [anon_sym_DOLLAR] = ACTIONS(710),
    [anon_sym_PLUS] = ACTIONS(577),
    [anon_sym_STAR] = ACTIONS(577),
    [anon_sym_QMARK] = ACTIONS(567),
    [anon_sym_u8] = ACTIONS(700),
    [anon_sym_i8] = ACTIONS(700),
    [anon_sym_u16] = ACTIONS(700),
    [anon_sym_i16] = ACTIONS(700),
    [anon_sym_u32] = ACTIONS(700),
    [anon_sym_i32] = ACTIONS(700),
    [anon_sym_u64] = ACTIONS(700),
    [anon_sym_i64] = ACTIONS(700),
    [anon_sym_u128] = ACTIONS(700),
    [anon_sym_i128] = ACTIONS(700),
    [anon_sym_isize] = ACTIONS(700),
    [anon_sym_usize] = ACTIONS(700),
    [anon_sym_f32] = ACTIONS(700),
    [anon_sym_f64] = ACTIONS(700),
    [anon_sym_bool] = ACTIONS(700),
    [anon_sym_str] = ACTIONS(700),
    [anon_sym_char] = ACTIONS(700),
    [anon_sym_DASH] = ACTIONS(577),
    [anon_sym_SLASH] = ACTIONS(577),
    [anon_sym_PERCENT] = ACTIONS(577),
    [anon_sym_CARET] = ACTIONS(577),
    [anon_sym_BANG] = ACTIONS(577),
    [anon_sym_AMP] = ACTIONS(577),
    [anon_sym_PIPE] = ACTIONS(577),
    [anon_sym_AMP_AMP] = ACTIONS(567),
    [anon_sym_PIPE_PIPE] = ACTIONS(567),
    [anon_sym_LT_LT] = ACTIONS(577),
    [anon_sym_GT_GT] = ACTIONS(577),
    [anon_sym_PLUS_EQ] = ACTIONS(567),
    [anon_sym_DASH_EQ] = ACTIONS(567),
    [anon_sym_STAR_EQ] = ACTIONS(567),
    [anon_sym_SLASH_EQ] = ACTIONS(567),
    [anon_sym_PERCENT_EQ] = ACTIONS(567),
    [anon_sym_CARET_EQ] = ACTIONS(567),
    [anon_sym_AMP_EQ] = ACTIONS(567),
    [anon_sym_PIPE_EQ] = ACTIONS(567),
    [anon_sym_LT_LT_EQ] = ACTIONS(567),
    [anon_sym_GT_GT_EQ] = ACTIONS(567),
    [anon_sym_EQ] = ACTIONS(577),
    [anon_sym_EQ_EQ] = ACTIONS(567),
    [anon_sym_BANG_EQ] = ACTIONS(567),
    [anon_sym_GT] = ACTIONS(577),
    [anon_sym_LT] = ACTIONS(577),
    [anon_sym_GT_EQ] = ACTIONS(567),
    [anon_sym_LT_EQ] = ACTIONS(567),
    [anon_sym_AT] = ACTIONS(567),
    [anon_sym__] = ACTIONS(577),
    [anon_sym_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(567),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(567),
    [anon_sym_COMMA] = ACTIONS(567),
    [anon_sym_COLON_COLON] = ACTIONS(567),
    [anon_sym_DASH_GT] = ACTIONS(567),
    [anon_sym_POUND] = ACTIONS(567),
    [anon_sym_SQUOTE] = ACTIONS(700),
    [anon_sym_as] = ACTIONS(700),
    [anon_sym_async] = ACTIONS(700),
    [anon_sym_await] = ACTIONS(700),
    [anon_sym_break] = ACTIONS(700),
    [anon_sym_const] = ACTIONS(700),
    [anon_sym_continue] = ACTIONS(700),
    [anon_sym_default] = ACTIONS(700),
    [anon_sym_enum] = ACTIONS(700),
    [anon_sym_fn] = ACTIONS(700),
    [anon_sym_for] = ACTIONS(700),
    [anon_sym_gen] = ACTIONS(700),
    [anon_sym_if] = ACTIONS(700),
    [anon_sym_impl] = ACTIONS(700),
    [anon_sym_let] = ACTIONS(700),
    [anon_sym_loop] = ACTIONS(700),
    [anon_sym_match] = ACTIONS(700),
    [anon_sym_mod] = ACTIONS(700),
    [anon_sym_pub] = ACTIONS(700),
    [anon_sym_return] = ACTIONS(700),
    [anon_sym_static] = ACTIONS(700),
    [anon_sym_struct] = ACTIONS(700),
    [anon_sym_trait] = ACTIONS(700),
    [anon_sym_type] = ACTIONS(700),
    [anon_sym_union] = ACTIONS(700),
    [anon_sym_unsafe] = ACTIONS(700),
    [anon_sym_use] = ACTIONS(700),
    [anon_sym_where] = ACTIONS(700),
    [anon_sym_while] = ACTIONS(700),
    [sym_mutable_specifier] = ACTIONS(700),
    [sym_integer_literal] = ACTIONS(581),
    [aux_sym_string_literal_token1] = ACTIONS(583),
    [sym_char_literal] = ACTIONS(581),
    [anon_sym_true] = ACTIONS(585),
    [anon_sym_false] = ACTIONS(585),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(700),
    [sym_super] = ACTIONS(700),
    [sym_crate] = ACTIONS(700),
    [sym_metavariable] = ACTIONS(712),
    [sym__raw_string_literal_start] = ACTIONS(589),
    [sym_float_literal] = ACTIONS(581),
  },
  [96] = {
    [sym_token_tree] = STATE(152),
    [sym_token_repetition] = STATE(152),
    [sym__literal] = STATE(152),
    [sym_string_literal] = STATE(177),
    [sym_raw_string_literal] = STATE(177),
    [sym_boolean_literal] = STATE(177),
    [sym_line_comment] = STATE(96),
    [sym_block_comment] = STATE(96),
    [aux_sym_token_tree_repeat1] = STATE(82),
    [aux_sym__non_special_token_repeat1] = STATE(143),
    [sym_identifier] = ACTIONS(700),
    [anon_sym_SEMI] = ACTIONS(567),
    [anon_sym_LPAREN] = ACTIONS(702),
    [anon_sym_LBRACK] = ACTIONS(704),
    [anon_sym_LBRACE] = ACTIONS(708),
    [anon_sym_RBRACE] = ACTIONS(718),
    [anon_sym_EQ_GT] = ACTIONS(567),
    [anon_sym_COLON] = ACTIONS(577),
    [anon_sym_DOLLAR] = ACTIONS(710),
    [anon_sym_PLUS] = ACTIONS(577),
    [anon_sym_STAR] = ACTIONS(577),
    [anon_sym_QMARK] = ACTIONS(567),
    [anon_sym_u8] = ACTIONS(700),
    [anon_sym_i8] = ACTIONS(700),
    [anon_sym_u16] = ACTIONS(700),
    [anon_sym_i16] = ACTIONS(700),
    [anon_sym_u32] = ACTIONS(700),
    [anon_sym_i32] = ACTIONS(700),
    [anon_sym_u64] = ACTIONS(700),
    [anon_sym_i64] = ACTIONS(700),
    [anon_sym_u128] = ACTIONS(700),
    [anon_sym_i128] = ACTIONS(700),
    [anon_sym_isize] = ACTIONS(700),
    [anon_sym_usize] = ACTIONS(700),
    [anon_sym_f32] = ACTIONS(700),
    [anon_sym_f64] = ACTIONS(700),
    [anon_sym_bool] = ACTIONS(700),
    [anon_sym_str] = ACTIONS(700),
    [anon_sym_char] = ACTIONS(700),
    [anon_sym_DASH] = ACTIONS(577),
    [anon_sym_SLASH] = ACTIONS(577),
    [anon_sym_PERCENT] = ACTIONS(577),
    [anon_sym_CARET] = ACTIONS(577),
    [anon_sym_BANG] = ACTIONS(577),
    [anon_sym_AMP] = ACTIONS(577),
    [anon_sym_PIPE] = ACTIONS(577),
    [anon_sym_AMP_AMP] = ACTIONS(567),
    [anon_sym_PIPE_PIPE] = ACTIONS(567),
    [anon_sym_LT_LT] = ACTIONS(577),
    [anon_sym_GT_GT] = ACTIONS(577),
    [anon_sym_PLUS_EQ] = ACTIONS(567),
    [anon_sym_DASH_EQ] = ACTIONS(567),
    [anon_sym_STAR_EQ] = ACTIONS(567),
    [anon_sym_SLASH_EQ] = ACTIONS(567),
    [anon_sym_PERCENT_EQ] = ACTIONS(567),
    [anon_sym_CARET_EQ] = ACTIONS(567),
    [anon_sym_AMP_EQ] = ACTIONS(567),
    [anon_sym_PIPE_EQ] = ACTIONS(567),
    [anon_sym_LT_LT_EQ] = ACTIONS(567),
    [anon_sym_GT_GT_EQ] = ACTIONS(567),
    [anon_sym_EQ] = ACTIONS(577),
    [anon_sym_EQ_EQ] = ACTIONS(567),
    [anon_sym_BANG_EQ] = ACTIONS(567),
    [anon_sym_GT] = ACTIONS(577),
    [anon_sym_LT] = ACTIONS(577),
    [anon_sym_GT_EQ] = ACTIONS(567),
    [anon_sym_LT_EQ] = ACTIONS(567),
    [anon_sym_AT] = ACTIONS(567),
    [anon_sym__] = ACTIONS(577),
    [anon_sym_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(567),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(567),
    [anon_sym_COMMA] = ACTIONS(567),
    [anon_sym_COLON_COLON] = ACTIONS(567),
    [anon_sym_DASH_GT] = ACTIONS(567),
    [anon_sym_POUND] = ACTIONS(567),
    [anon_sym_SQUOTE] = ACTIONS(700),
    [anon_sym_as] = ACTIONS(700),
    [anon_sym_async] = ACTIONS(700),
    [anon_sym_await] = ACTIONS(700),
    [anon_sym_break] = ACTIONS(700),
    [anon_sym_const] = ACTIONS(700),
    [anon_sym_continue] = ACTIONS(700),
    [anon_sym_default] = ACTIONS(700),
    [anon_sym_enum] = ACTIONS(700),
    [anon_sym_fn] = ACTIONS(700),
    [anon_sym_for] = ACTIONS(700),
    [anon_sym_gen] = ACTIONS(700),
    [anon_sym_if] = ACTIONS(700),
    [anon_sym_impl] = ACTIONS(700),
    [anon_sym_let] = ACTIONS(700),
    [anon_sym_loop] = ACTIONS(700),
    [anon_sym_match] = ACTIONS(700),
    [anon_sym_mod] = ACTIONS(700),
    [anon_sym_pub] = ACTIONS(700),
    [anon_sym_return] = ACTIONS(700),
    [anon_sym_static] = ACTIONS(700),
    [anon_sym_struct] = ACTIONS(700),
    [anon_sym_trait] = ACTIONS(700),
    [anon_sym_type] = ACTIONS(700),
    [anon_sym_union] = ACTIONS(700),
    [anon_sym_unsafe] = ACTIONS(700),
    [anon_sym_use] = ACTIONS(700),
    [anon_sym_where] = ACTIONS(700),
    [anon_sym_while] = ACTIONS(700),
    [sym_mutable_specifier] = ACTIONS(700),
    [sym_integer_literal] = ACTIONS(581),
    [aux_sym_string_literal_token1] = ACTIONS(583),
    [sym_char_literal] = ACTIONS(581),
    [anon_sym_true] = ACTIONS(585),
    [anon_sym_false] = ACTIONS(585),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(700),
    [sym_super] = ACTIONS(700),
    [sym_crate] = ACTIONS(700),
    [sym_metavariable] = ACTIONS(712),
    [sym__raw_string_literal_start] = ACTIONS(589),
    [sym_float_literal] = ACTIONS(581),
  },
  [97] = {
    [sym_delim_token_tree] = STATE(198),
    [sym__delim_tokens] = STATE(206),
    [sym__non_delim_token] = STATE(198),
    [sym__literal] = STATE(214),
    [sym_string_literal] = STATE(199),
    [sym_raw_string_literal] = STATE(199),
    [sym_boolean_literal] = STATE(199),
    [sym_line_comment] = STATE(97),
    [sym_block_comment] = STATE(97),
    [aux_sym__non_special_token_repeat1] = STATE(157),
    [aux_sym_delim_token_tree_repeat1] = STATE(83),
    [sym_identifier] = ACTIONS(674),
    [anon_sym_SEMI] = ACTIONS(676),
    [anon_sym_LPAREN] = ACTIONS(678),
    [anon_sym_LBRACK] = ACTIONS(680),
    [anon_sym_LBRACE] = ACTIONS(682),
    [anon_sym_RBRACE] = ACTIONS(720),
    [anon_sym_EQ_GT] = ACTIONS(676),
    [anon_sym_COLON] = ACTIONS(686),
    [anon_sym_DOLLAR] = ACTIONS(688),
    [anon_sym_PLUS] = ACTIONS(686),
    [anon_sym_STAR] = ACTIONS(686),
    [anon_sym_QMARK] = ACTIONS(676),
    [anon_sym_u8] = ACTIONS(674),
    [anon_sym_i8] = ACTIONS(674),
    [anon_sym_u16] = ACTIONS(674),
    [anon_sym_i16] = ACTIONS(674),
    [anon_sym_u32] = ACTIONS(674),
    [anon_sym_i32] = ACTIONS(674),
    [anon_sym_u64] = ACTIONS(674),
    [anon_sym_i64] = ACTIONS(674),
    [anon_sym_u128] = ACTIONS(674),
    [anon_sym_i128] = ACTIONS(674),
    [anon_sym_isize] = ACTIONS(674),
    [anon_sym_usize] = ACTIONS(674),
    [anon_sym_f32] = ACTIONS(674),
    [anon_sym_f64] = ACTIONS(674),
    [anon_sym_bool] = ACTIONS(674),
    [anon_sym_str] = ACTIONS(674),
    [anon_sym_char] = ACTIONS(674),
    [anon_sym_DASH] = ACTIONS(686),
    [anon_sym_SLASH] = ACTIONS(686),
    [anon_sym_PERCENT] = ACTIONS(686),
    [anon_sym_CARET] = ACTIONS(686),
    [anon_sym_BANG] = ACTIONS(686),
    [anon_sym_AMP] = ACTIONS(686),
    [anon_sym_PIPE] = ACTIONS(686),
    [anon_sym_AMP_AMP] = ACTIONS(676),
    [anon_sym_PIPE_PIPE] = ACTIONS(676),
    [anon_sym_LT_LT] = ACTIONS(686),
    [anon_sym_GT_GT] = ACTIONS(686),
    [anon_sym_PLUS_EQ] = ACTIONS(676),
    [anon_sym_DASH_EQ] = ACTIONS(676),
    [anon_sym_STAR_EQ] = ACTIONS(676),
    [anon_sym_SLASH_EQ] = ACTIONS(676),
    [anon_sym_PERCENT_EQ] = ACTIONS(676),
    [anon_sym_CARET_EQ] = ACTIONS(676),
    [anon_sym_AMP_EQ] = ACTIONS(676),
    [anon_sym_PIPE_EQ] = ACTIONS(676),
    [anon_sym_LT_LT_EQ] = ACTIONS(676),
    [anon_sym_GT_GT_EQ] = ACTIONS(676),
    [anon_sym_EQ] = ACTIONS(686),
    [anon_sym_EQ_EQ] = ACTIONS(676),
    [anon_sym_BANG_EQ] = ACTIONS(676),
    [anon_sym_GT] = ACTIONS(686),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT_EQ] = ACTIONS(676),
    [anon_sym_LT_EQ] = ACTIONS(676),
    [anon_sym_AT] = ACTIONS(676),
    [anon_sym__] = ACTIONS(686),
    [anon_sym_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(676),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(676),
    [anon_sym_COMMA] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(676),
    [anon_sym_DASH_GT] = ACTIONS(676),
    [anon_sym_POUND] = ACTIONS(676),
    [anon_sym_SQUOTE] = ACTIONS(674),
    [anon_sym_as] = ACTIONS(674),
    [anon_sym_async] = ACTIONS(674),
    [anon_sym_await] = ACTIONS(674),
    [anon_sym_break] = ACTIONS(674),
    [anon_sym_const] = ACTIONS(674),
    [anon_sym_continue] = ACTIONS(674),
    [anon_sym_default] = ACTIONS(674),
    [anon_sym_enum] = ACTIONS(674),
    [anon_sym_fn] = ACTIONS(674),
    [anon_sym_for] = ACTIONS(674),
    [anon_sym_gen] = ACTIONS(674),
    [anon_sym_if] = ACTIONS(674),
    [anon_sym_impl] = ACTIONS(674),
    [anon_sym_let] = ACTIONS(674),
    [anon_sym_loop] = ACTIONS(674),
    [anon_sym_match] = ACTIONS(674),
    [anon_sym_mod] = ACTIONS(674),
    [anon_sym_pub] = ACTIONS(674),
    [anon_sym_return] = ACTIONS(674),
    [anon_sym_static] = ACTIONS(674),
    [anon_sym_struct] = ACTIONS(674),
    [anon_sym_trait] = ACTIONS(674),
    [anon_sym_type] = ACTIONS(674),
    [anon_sym_union] = ACTIONS(674),
    [anon_sym_unsafe] = ACTIONS(674),
    [anon_sym_use] = ACTIONS(674),
    [anon_sym_where] = ACTIONS(674),
    [anon_sym_while] = ACTIONS(674),
    [sym_mutable_specifier] = ACTIONS(674),
    [sym_integer_literal] = ACTIONS(690),
    [aux_sym_string_literal_token1] = ACTIONS(692),
    [sym_char_literal] = ACTIONS(690),
    [anon_sym_true] = ACTIONS(694),
    [anon_sym_false] = ACTIONS(694),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(674),
    [sym_super] = ACTIONS(674),
    [sym_crate] = ACTIONS(674),
    [sym__raw_string_literal_start] = ACTIONS(696),
    [sym_float_literal] = ACTIONS(690),
  },
  [98] = {
    [sym_token_tree] = STATE(152),
    [sym_token_repetition] = STATE(152),
    [sym__literal] = STATE(152),
    [sym_string_literal] = STATE(177),
    [sym_raw_string_literal] = STATE(177),
    [sym_boolean_literal] = STATE(177),
    [sym_line_comment] = STATE(98),
    [sym_block_comment] = STATE(98),
    [aux_sym_token_tree_repeat1] = STATE(82),
    [aux_sym__non_special_token_repeat1] = STATE(143),
    [sym_identifier] = ACTIONS(700),
    [anon_sym_SEMI] = ACTIONS(567),
    [anon_sym_LPAREN] = ACTIONS(702),
    [anon_sym_RPAREN] = ACTIONS(722),
    [anon_sym_LBRACK] = ACTIONS(704),
    [anon_sym_LBRACE] = ACTIONS(708),
    [anon_sym_EQ_GT] = ACTIONS(567),
    [anon_sym_COLON] = ACTIONS(577),
    [anon_sym_DOLLAR] = ACTIONS(710),
    [anon_sym_PLUS] = ACTIONS(577),
    [anon_sym_STAR] = ACTIONS(577),
    [anon_sym_QMARK] = ACTIONS(567),
    [anon_sym_u8] = ACTIONS(700),
    [anon_sym_i8] = ACTIONS(700),
    [anon_sym_u16] = ACTIONS(700),
    [anon_sym_i16] = ACTIONS(700),
    [anon_sym_u32] = ACTIONS(700),
    [anon_sym_i32] = ACTIONS(700),
    [anon_sym_u64] = ACTIONS(700),
    [anon_sym_i64] = ACTIONS(700),
    [anon_sym_u128] = ACTIONS(700),
    [anon_sym_i128] = ACTIONS(700),
    [anon_sym_isize] = ACTIONS(700),
    [anon_sym_usize] = ACTIONS(700),
    [anon_sym_f32] = ACTIONS(700),
    [anon_sym_f64] = ACTIONS(700),
    [anon_sym_bool] = ACTIONS(700),
    [anon_sym_str] = ACTIONS(700),
    [anon_sym_char] = ACTIONS(700),
    [anon_sym_DASH] = ACTIONS(577),
    [anon_sym_SLASH] = ACTIONS(577),
    [anon_sym_PERCENT] = ACTIONS(577),
    [anon_sym_CARET] = ACTIONS(577),
    [anon_sym_BANG] = ACTIONS(577),
    [anon_sym_AMP] = ACTIONS(577),
    [anon_sym_PIPE] = ACTIONS(577),
    [anon_sym_AMP_AMP] = ACTIONS(567),
    [anon_sym_PIPE_PIPE] = ACTIONS(567),
    [anon_sym_LT_LT] = ACTIONS(577),
    [anon_sym_GT_GT] = ACTIONS(577),
    [anon_sym_PLUS_EQ] = ACTIONS(567),
    [anon_sym_DASH_EQ] = ACTIONS(567),
    [anon_sym_STAR_EQ] = ACTIONS(567),
    [anon_sym_SLASH_EQ] = ACTIONS(567),
    [anon_sym_PERCENT_EQ] = ACTIONS(567),
    [anon_sym_CARET_EQ] = ACTIONS(567),
    [anon_sym_AMP_EQ] = ACTIONS(567),
    [anon_sym_PIPE_EQ] = ACTIONS(567),
    [anon_sym_LT_LT_EQ] = ACTIONS(567),
    [anon_sym_GT_GT_EQ] = ACTIONS(567),
    [anon_sym_EQ] = ACTIONS(577),
    [anon_sym_EQ_EQ] = ACTIONS(567),
    [anon_sym_BANG_EQ] = ACTIONS(567),
    [anon_sym_GT] = ACTIONS(577),
    [anon_sym_LT] = ACTIONS(577),
    [anon_sym_GT_EQ] = ACTIONS(567),
    [anon_sym_LT_EQ] = ACTIONS(567),
    [anon_sym_AT] = ACTIONS(567),
    [anon_sym__] = ACTIONS(577),
    [anon_sym_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(567),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(567),
    [anon_sym_COMMA] = ACTIONS(567),
    [anon_sym_COLON_COLON] = ACTIONS(567),
    [anon_sym_DASH_GT] = ACTIONS(567),
    [anon_sym_POUND] = ACTIONS(567),
    [anon_sym_SQUOTE] = ACTIONS(700),
    [anon_sym_as] = ACTIONS(700),
    [anon_sym_async] = ACTIONS(700),
    [anon_sym_await] = ACTIONS(700),
    [anon_sym_break] = ACTIONS(700),
    [anon_sym_const] = ACTIONS(700),
    [anon_sym_continue] = ACTIONS(700),
    [anon_sym_default] = ACTIONS(700),
    [anon_sym_enum] = ACTIONS(700),
    [anon_sym_fn] = ACTIONS(700),
    [anon_sym_for] = ACTIONS(700),
    [anon_sym_gen] = ACTIONS(700),
    [anon_sym_if] = ACTIONS(700),
    [anon_sym_impl] = ACTIONS(700),
    [anon_sym_let] = ACTIONS(700),
    [anon_sym_loop] = ACTIONS(700),
    [anon_sym_match] = ACTIONS(700),
    [anon_sym_mod] = ACTIONS(700),
    [anon_sym_pub] = ACTIONS(700),
    [anon_sym_return] = ACTIONS(700),
    [anon_sym_static] = ACTIONS(700),
    [anon_sym_struct] = ACTIONS(700),
    [anon_sym_trait] = ACTIONS(700),
    [anon_sym_type] = ACTIONS(700),
    [anon_sym_union] = ACTIONS(700),
    [anon_sym_unsafe] = ACTIONS(700),
    [anon_sym_use] = ACTIONS(700),
    [anon_sym_where] = ACTIONS(700),
    [anon_sym_while] = ACTIONS(700),
    [sym_mutable_specifier] = ACTIONS(700),
    [sym_integer_literal] = ACTIONS(581),
    [aux_sym_string_literal_token1] = ACTIONS(583),
    [sym_char_literal] = ACTIONS(581),
    [anon_sym_true] = ACTIONS(585),
    [anon_sym_false] = ACTIONS(585),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(700),
    [sym_super] = ACTIONS(700),
    [sym_crate] = ACTIONS(700),
    [sym_metavariable] = ACTIONS(712),
    [sym__raw_string_literal_start] = ACTIONS(589),
    [sym_float_literal] = ACTIONS(581),
  },
  [99] = {
    [sym_delim_token_tree] = STATE(198),
    [sym__delim_tokens] = STATE(206),
    [sym__non_delim_token] = STATE(198),
    [sym__literal] = STATE(214),
    [sym_string_literal] = STATE(199),
    [sym_raw_string_literal] = STATE(199),
    [sym_boolean_literal] = STATE(199),
    [sym_line_comment] = STATE(99),
    [sym_block_comment] = STATE(99),
    [aux_sym__non_special_token_repeat1] = STATE(157),
    [aux_sym_delim_token_tree_repeat1] = STATE(103),
    [sym_identifier] = ACTIONS(674),
    [anon_sym_SEMI] = ACTIONS(676),
    [anon_sym_LPAREN] = ACTIONS(678),
    [anon_sym_RPAREN] = ACTIONS(724),
    [anon_sym_LBRACK] = ACTIONS(680),
    [anon_sym_LBRACE] = ACTIONS(682),
    [anon_sym_EQ_GT] = ACTIONS(676),
    [anon_sym_COLON] = ACTIONS(686),
    [anon_sym_DOLLAR] = ACTIONS(688),
    [anon_sym_PLUS] = ACTIONS(686),
    [anon_sym_STAR] = ACTIONS(686),
    [anon_sym_QMARK] = ACTIONS(676),
    [anon_sym_u8] = ACTIONS(674),
    [anon_sym_i8] = ACTIONS(674),
    [anon_sym_u16] = ACTIONS(674),
    [anon_sym_i16] = ACTIONS(674),
    [anon_sym_u32] = ACTIONS(674),
    [anon_sym_i32] = ACTIONS(674),
    [anon_sym_u64] = ACTIONS(674),
    [anon_sym_i64] = ACTIONS(674),
    [anon_sym_u128] = ACTIONS(674),
    [anon_sym_i128] = ACTIONS(674),
    [anon_sym_isize] = ACTIONS(674),
    [anon_sym_usize] = ACTIONS(674),
    [anon_sym_f32] = ACTIONS(674),
    [anon_sym_f64] = ACTIONS(674),
    [anon_sym_bool] = ACTIONS(674),
    [anon_sym_str] = ACTIONS(674),
    [anon_sym_char] = ACTIONS(674),
    [anon_sym_DASH] = ACTIONS(686),
    [anon_sym_SLASH] = ACTIONS(686),
    [anon_sym_PERCENT] = ACTIONS(686),
    [anon_sym_CARET] = ACTIONS(686),
    [anon_sym_BANG] = ACTIONS(686),
    [anon_sym_AMP] = ACTIONS(686),
    [anon_sym_PIPE] = ACTIONS(686),
    [anon_sym_AMP_AMP] = ACTIONS(676),
    [anon_sym_PIPE_PIPE] = ACTIONS(676),
    [anon_sym_LT_LT] = ACTIONS(686),
    [anon_sym_GT_GT] = ACTIONS(686),
    [anon_sym_PLUS_EQ] = ACTIONS(676),
    [anon_sym_DASH_EQ] = ACTIONS(676),
    [anon_sym_STAR_EQ] = ACTIONS(676),
    [anon_sym_SLASH_EQ] = ACTIONS(676),
    [anon_sym_PERCENT_EQ] = ACTIONS(676),
    [anon_sym_CARET_EQ] = ACTIONS(676),
    [anon_sym_AMP_EQ] = ACTIONS(676),
    [anon_sym_PIPE_EQ] = ACTIONS(676),
    [anon_sym_LT_LT_EQ] = ACTIONS(676),
    [anon_sym_GT_GT_EQ] = ACTIONS(676),
    [anon_sym_EQ] = ACTIONS(686),
    [anon_sym_EQ_EQ] = ACTIONS(676),
    [anon_sym_BANG_EQ] = ACTIONS(676),
    [anon_sym_GT] = ACTIONS(686),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT_EQ] = ACTIONS(676),
    [anon_sym_LT_EQ] = ACTIONS(676),
    [anon_sym_AT] = ACTIONS(676),
    [anon_sym__] = ACTIONS(686),
    [anon_sym_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(676),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(676),
    [anon_sym_COMMA] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(676),
    [anon_sym_DASH_GT] = ACTIONS(676),
    [anon_sym_POUND] = ACTIONS(676),
    [anon_sym_SQUOTE] = ACTIONS(674),
    [anon_sym_as] = ACTIONS(674),
    [anon_sym_async] = ACTIONS(674),
    [anon_sym_await] = ACTIONS(674),
    [anon_sym_break] = ACTIONS(674),
    [anon_sym_const] = ACTIONS(674),
    [anon_sym_continue] = ACTIONS(674),
    [anon_sym_default] = ACTIONS(674),
    [anon_sym_enum] = ACTIONS(674),
    [anon_sym_fn] = ACTIONS(674),
    [anon_sym_for] = ACTIONS(674),
    [anon_sym_gen] = ACTIONS(674),
    [anon_sym_if] = ACTIONS(674),
    [anon_sym_impl] = ACTIONS(674),
    [anon_sym_let] = ACTIONS(674),
    [anon_sym_loop] = ACTIONS(674),
    [anon_sym_match] = ACTIONS(674),
    [anon_sym_mod] = ACTIONS(674),
    [anon_sym_pub] = ACTIONS(674),
    [anon_sym_return] = ACTIONS(674),
    [anon_sym_static] = ACTIONS(674),
    [anon_sym_struct] = ACTIONS(674),
    [anon_sym_trait] = ACTIONS(674),
    [anon_sym_type] = ACTIONS(674),
    [anon_sym_union] = ACTIONS(674),
    [anon_sym_unsafe] = ACTIONS(674),
    [anon_sym_use] = ACTIONS(674),
    [anon_sym_where] = ACTIONS(674),
    [anon_sym_while] = ACTIONS(674),
    [sym_mutable_specifier] = ACTIONS(674),
    [sym_integer_literal] = ACTIONS(690),
    [aux_sym_string_literal_token1] = ACTIONS(692),
    [sym_char_literal] = ACTIONS(690),
    [anon_sym_true] = ACTIONS(694),
    [anon_sym_false] = ACTIONS(694),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(674),
    [sym_super] = ACTIONS(674),
    [sym_crate] = ACTIONS(674),
    [sym__raw_string_literal_start] = ACTIONS(696),
    [sym_float_literal] = ACTIONS(690),
  },
  [100] = {
    [sym_delim_token_tree] = STATE(198),
    [sym__delim_tokens] = STATE(206),
    [sym__non_delim_token] = STATE(198),
    [sym__literal] = STATE(214),
    [sym_string_literal] = STATE(199),
    [sym_raw_string_literal] = STATE(199),
    [sym_boolean_literal] = STATE(199),
    [sym_line_comment] = STATE(100),
    [sym_block_comment] = STATE(100),
    [aux_sym__non_special_token_repeat1] = STATE(157),
    [aux_sym_delim_token_tree_repeat1] = STATE(104),
    [sym_identifier] = ACTIONS(674),
    [anon_sym_SEMI] = ACTIONS(676),
    [anon_sym_LPAREN] = ACTIONS(678),
    [anon_sym_LBRACK] = ACTIONS(680),
    [anon_sym_RBRACK] = ACTIONS(724),
    [anon_sym_LBRACE] = ACTIONS(682),
    [anon_sym_EQ_GT] = ACTIONS(676),
    [anon_sym_COLON] = ACTIONS(686),
    [anon_sym_DOLLAR] = ACTIONS(688),
    [anon_sym_PLUS] = ACTIONS(686),
    [anon_sym_STAR] = ACTIONS(686),
    [anon_sym_QMARK] = ACTIONS(676),
    [anon_sym_u8] = ACTIONS(674),
    [anon_sym_i8] = ACTIONS(674),
    [anon_sym_u16] = ACTIONS(674),
    [anon_sym_i16] = ACTIONS(674),
    [anon_sym_u32] = ACTIONS(674),
    [anon_sym_i32] = ACTIONS(674),
    [anon_sym_u64] = ACTIONS(674),
    [anon_sym_i64] = ACTIONS(674),
    [anon_sym_u128] = ACTIONS(674),
    [anon_sym_i128] = ACTIONS(674),
    [anon_sym_isize] = ACTIONS(674),
    [anon_sym_usize] = ACTIONS(674),
    [anon_sym_f32] = ACTIONS(674),
    [anon_sym_f64] = ACTIONS(674),
    [anon_sym_bool] = ACTIONS(674),
    [anon_sym_str] = ACTIONS(674),
    [anon_sym_char] = ACTIONS(674),
    [anon_sym_DASH] = ACTIONS(686),
    [anon_sym_SLASH] = ACTIONS(686),
    [anon_sym_PERCENT] = ACTIONS(686),
    [anon_sym_CARET] = ACTIONS(686),
    [anon_sym_BANG] = ACTIONS(686),
    [anon_sym_AMP] = ACTIONS(686),
    [anon_sym_PIPE] = ACTIONS(686),
    [anon_sym_AMP_AMP] = ACTIONS(676),
    [anon_sym_PIPE_PIPE] = ACTIONS(676),
    [anon_sym_LT_LT] = ACTIONS(686),
    [anon_sym_GT_GT] = ACTIONS(686),
    [anon_sym_PLUS_EQ] = ACTIONS(676),
    [anon_sym_DASH_EQ] = ACTIONS(676),
    [anon_sym_STAR_EQ] = ACTIONS(676),
    [anon_sym_SLASH_EQ] = ACTIONS(676),
    [anon_sym_PERCENT_EQ] = ACTIONS(676),
    [anon_sym_CARET_EQ] = ACTIONS(676),
    [anon_sym_AMP_EQ] = ACTIONS(676),
    [anon_sym_PIPE_EQ] = ACTIONS(676),
    [anon_sym_LT_LT_EQ] = ACTIONS(676),
    [anon_sym_GT_GT_EQ] = ACTIONS(676),
    [anon_sym_EQ] = ACTIONS(686),
    [anon_sym_EQ_EQ] = ACTIONS(676),
    [anon_sym_BANG_EQ] = ACTIONS(676),
    [anon_sym_GT] = ACTIONS(686),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT_EQ] = ACTIONS(676),
    [anon_sym_LT_EQ] = ACTIONS(676),
    [anon_sym_AT] = ACTIONS(676),
    [anon_sym__] = ACTIONS(686),
    [anon_sym_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(676),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(676),
    [anon_sym_COMMA] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(676),
    [anon_sym_DASH_GT] = ACTIONS(676),
    [anon_sym_POUND] = ACTIONS(676),
    [anon_sym_SQUOTE] = ACTIONS(674),
    [anon_sym_as] = ACTIONS(674),
    [anon_sym_async] = ACTIONS(674),
    [anon_sym_await] = ACTIONS(674),
    [anon_sym_break] = ACTIONS(674),
    [anon_sym_const] = ACTIONS(674),
    [anon_sym_continue] = ACTIONS(674),
    [anon_sym_default] = ACTIONS(674),
    [anon_sym_enum] = ACTIONS(674),
    [anon_sym_fn] = ACTIONS(674),
    [anon_sym_for] = ACTIONS(674),
    [anon_sym_gen] = ACTIONS(674),
    [anon_sym_if] = ACTIONS(674),
    [anon_sym_impl] = ACTIONS(674),
    [anon_sym_let] = ACTIONS(674),
    [anon_sym_loop] = ACTIONS(674),
    [anon_sym_match] = ACTIONS(674),
    [anon_sym_mod] = ACTIONS(674),
    [anon_sym_pub] = ACTIONS(674),
    [anon_sym_return] = ACTIONS(674),
    [anon_sym_static] = ACTIONS(674),
    [anon_sym_struct] = ACTIONS(674),
    [anon_sym_trait] = ACTIONS(674),
    [anon_sym_type] = ACTIONS(674),
    [anon_sym_union] = ACTIONS(674),
    [anon_sym_unsafe] = ACTIONS(674),
    [anon_sym_use] = ACTIONS(674),
    [anon_sym_where] = ACTIONS(674),
    [anon_sym_while] = ACTIONS(674),
    [sym_mutable_specifier] = ACTIONS(674),
    [sym_integer_literal] = ACTIONS(690),
    [aux_sym_string_literal_token1] = ACTIONS(692),
    [sym_char_literal] = ACTIONS(690),
    [anon_sym_true] = ACTIONS(694),
    [anon_sym_false] = ACTIONS(694),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(674),
    [sym_super] = ACTIONS(674),
    [sym_crate] = ACTIONS(674),
    [sym__raw_string_literal_start] = ACTIONS(696),
    [sym_float_literal] = ACTIONS(690),
  },
  [101] = {
    [sym_delim_token_tree] = STATE(198),
    [sym__delim_tokens] = STATE(206),
    [sym__non_delim_token] = STATE(198),
    [sym__literal] = STATE(214),
    [sym_string_literal] = STATE(199),
    [sym_raw_string_literal] = STATE(199),
    [sym_boolean_literal] = STATE(199),
    [sym_line_comment] = STATE(101),
    [sym_block_comment] = STATE(101),
    [aux_sym__non_special_token_repeat1] = STATE(157),
    [aux_sym_delim_token_tree_repeat1] = STATE(105),
    [sym_identifier] = ACTIONS(674),
    [anon_sym_SEMI] = ACTIONS(676),
    [anon_sym_LPAREN] = ACTIONS(678),
    [anon_sym_LBRACK] = ACTIONS(680),
    [anon_sym_LBRACE] = ACTIONS(682),
    [anon_sym_RBRACE] = ACTIONS(724),
    [anon_sym_EQ_GT] = ACTIONS(676),
    [anon_sym_COLON] = ACTIONS(686),
    [anon_sym_DOLLAR] = ACTIONS(688),
    [anon_sym_PLUS] = ACTIONS(686),
    [anon_sym_STAR] = ACTIONS(686),
    [anon_sym_QMARK] = ACTIONS(676),
    [anon_sym_u8] = ACTIONS(674),
    [anon_sym_i8] = ACTIONS(674),
    [anon_sym_u16] = ACTIONS(674),
    [anon_sym_i16] = ACTIONS(674),
    [anon_sym_u32] = ACTIONS(674),
    [anon_sym_i32] = ACTIONS(674),
    [anon_sym_u64] = ACTIONS(674),
    [anon_sym_i64] = ACTIONS(674),
    [anon_sym_u128] = ACTIONS(674),
    [anon_sym_i128] = ACTIONS(674),
    [anon_sym_isize] = ACTIONS(674),
    [anon_sym_usize] = ACTIONS(674),
    [anon_sym_f32] = ACTIONS(674),
    [anon_sym_f64] = ACTIONS(674),
    [anon_sym_bool] = ACTIONS(674),
    [anon_sym_str] = ACTIONS(674),
    [anon_sym_char] = ACTIONS(674),
    [anon_sym_DASH] = ACTIONS(686),
    [anon_sym_SLASH] = ACTIONS(686),
    [anon_sym_PERCENT] = ACTIONS(686),
    [anon_sym_CARET] = ACTIONS(686),
    [anon_sym_BANG] = ACTIONS(686),
    [anon_sym_AMP] = ACTIONS(686),
    [anon_sym_PIPE] = ACTIONS(686),
    [anon_sym_AMP_AMP] = ACTIONS(676),
    [anon_sym_PIPE_PIPE] = ACTIONS(676),
    [anon_sym_LT_LT] = ACTIONS(686),
    [anon_sym_GT_GT] = ACTIONS(686),
    [anon_sym_PLUS_EQ] = ACTIONS(676),
    [anon_sym_DASH_EQ] = ACTIONS(676),
    [anon_sym_STAR_EQ] = ACTIONS(676),
    [anon_sym_SLASH_EQ] = ACTIONS(676),
    [anon_sym_PERCENT_EQ] = ACTIONS(676),
    [anon_sym_CARET_EQ] = ACTIONS(676),
    [anon_sym_AMP_EQ] = ACTIONS(676),
    [anon_sym_PIPE_EQ] = ACTIONS(676),
    [anon_sym_LT_LT_EQ] = ACTIONS(676),
    [anon_sym_GT_GT_EQ] = ACTIONS(676),
    [anon_sym_EQ] = ACTIONS(686),
    [anon_sym_EQ_EQ] = ACTIONS(676),
    [anon_sym_BANG_EQ] = ACTIONS(676),
    [anon_sym_GT] = ACTIONS(686),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT_EQ] = ACTIONS(676),
    [anon_sym_LT_EQ] = ACTIONS(676),
    [anon_sym_AT] = ACTIONS(676),
    [anon_sym__] = ACTIONS(686),
    [anon_sym_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(676),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(676),
    [anon_sym_COMMA] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(676),
    [anon_sym_DASH_GT] = ACTIONS(676),
    [anon_sym_POUND] = ACTIONS(676),
    [anon_sym_SQUOTE] = ACTIONS(674),
    [anon_sym_as] = ACTIONS(674),
    [anon_sym_async] = ACTIONS(674),
    [anon_sym_await] = ACTIONS(674),
    [anon_sym_break] = ACTIONS(674),
    [anon_sym_const] = ACTIONS(674),
    [anon_sym_continue] = ACTIONS(674),
    [anon_sym_default] = ACTIONS(674),
    [anon_sym_enum] = ACTIONS(674),
    [anon_sym_fn] = ACTIONS(674),
    [anon_sym_for] = ACTIONS(674),
    [anon_sym_gen] = ACTIONS(674),
    [anon_sym_if] = ACTIONS(674),
    [anon_sym_impl] = ACTIONS(674),
    [anon_sym_let] = ACTIONS(674),
    [anon_sym_loop] = ACTIONS(674),
    [anon_sym_match] = ACTIONS(674),
    [anon_sym_mod] = ACTIONS(674),
    [anon_sym_pub] = ACTIONS(674),
    [anon_sym_return] = ACTIONS(674),
    [anon_sym_static] = ACTIONS(674),
    [anon_sym_struct] = ACTIONS(674),
    [anon_sym_trait] = ACTIONS(674),
    [anon_sym_type] = ACTIONS(674),
    [anon_sym_union] = ACTIONS(674),
    [anon_sym_unsafe] = ACTIONS(674),
    [anon_sym_use] = ACTIONS(674),
    [anon_sym_where] = ACTIONS(674),
    [anon_sym_while] = ACTIONS(674),
    [sym_mutable_specifier] = ACTIONS(674),
    [sym_integer_literal] = ACTIONS(690),
    [aux_sym_string_literal_token1] = ACTIONS(692),
    [sym_char_literal] = ACTIONS(690),
    [anon_sym_true] = ACTIONS(694),
    [anon_sym_false] = ACTIONS(694),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(674),
    [sym_super] = ACTIONS(674),
    [sym_crate] = ACTIONS(674),
    [sym__raw_string_literal_start] = ACTIONS(696),
    [sym_float_literal] = ACTIONS(690),
  },
  [102] = {
    [sym_delim_token_tree] = STATE(198),
    [sym__delim_tokens] = STATE(206),
    [sym__non_delim_token] = STATE(198),
    [sym__literal] = STATE(214),
    [sym_string_literal] = STATE(199),
    [sym_raw_string_literal] = STATE(199),
    [sym_boolean_literal] = STATE(199),
    [sym_line_comment] = STATE(102),
    [sym_block_comment] = STATE(102),
    [aux_sym__non_special_token_repeat1] = STATE(157),
    [aux_sym_delim_token_tree_repeat1] = STATE(132),
    [sym_identifier] = ACTIONS(674),
    [anon_sym_SEMI] = ACTIONS(676),
    [anon_sym_LPAREN] = ACTIONS(678),
    [anon_sym_LBRACK] = ACTIONS(680),
    [anon_sym_RBRACK] = ACTIONS(726),
    [anon_sym_LBRACE] = ACTIONS(682),
    [anon_sym_EQ_GT] = ACTIONS(676),
    [anon_sym_COLON] = ACTIONS(686),
    [anon_sym_DOLLAR] = ACTIONS(688),
    [anon_sym_PLUS] = ACTIONS(686),
    [anon_sym_STAR] = ACTIONS(686),
    [anon_sym_QMARK] = ACTIONS(676),
    [anon_sym_u8] = ACTIONS(674),
    [anon_sym_i8] = ACTIONS(674),
    [anon_sym_u16] = ACTIONS(674),
    [anon_sym_i16] = ACTIONS(674),
    [anon_sym_u32] = ACTIONS(674),
    [anon_sym_i32] = ACTIONS(674),
    [anon_sym_u64] = ACTIONS(674),
    [anon_sym_i64] = ACTIONS(674),
    [anon_sym_u128] = ACTIONS(674),
    [anon_sym_i128] = ACTIONS(674),
    [anon_sym_isize] = ACTIONS(674),
    [anon_sym_usize] = ACTIONS(674),
    [anon_sym_f32] = ACTIONS(674),
    [anon_sym_f64] = ACTIONS(674),
    [anon_sym_bool] = ACTIONS(674),
    [anon_sym_str] = ACTIONS(674),
    [anon_sym_char] = ACTIONS(674),
    [anon_sym_DASH] = ACTIONS(686),
    [anon_sym_SLASH] = ACTIONS(686),
    [anon_sym_PERCENT] = ACTIONS(686),
    [anon_sym_CARET] = ACTIONS(686),
    [anon_sym_BANG] = ACTIONS(686),
    [anon_sym_AMP] = ACTIONS(686),
    [anon_sym_PIPE] = ACTIONS(686),
    [anon_sym_AMP_AMP] = ACTIONS(676),
    [anon_sym_PIPE_PIPE] = ACTIONS(676),
    [anon_sym_LT_LT] = ACTIONS(686),
    [anon_sym_GT_GT] = ACTIONS(686),
    [anon_sym_PLUS_EQ] = ACTIONS(676),
    [anon_sym_DASH_EQ] = ACTIONS(676),
    [anon_sym_STAR_EQ] = ACTIONS(676),
    [anon_sym_SLASH_EQ] = ACTIONS(676),
    [anon_sym_PERCENT_EQ] = ACTIONS(676),
    [anon_sym_CARET_EQ] = ACTIONS(676),
    [anon_sym_AMP_EQ] = ACTIONS(676),
    [anon_sym_PIPE_EQ] = ACTIONS(676),
    [anon_sym_LT_LT_EQ] = ACTIONS(676),
    [anon_sym_GT_GT_EQ] = ACTIONS(676),
    [anon_sym_EQ] = ACTIONS(686),
    [anon_sym_EQ_EQ] = ACTIONS(676),
    [anon_sym_BANG_EQ] = ACTIONS(676),
    [anon_sym_GT] = ACTIONS(686),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT_EQ] = ACTIONS(676),
    [anon_sym_LT_EQ] = ACTIONS(676),
    [anon_sym_AT] = ACTIONS(676),
    [anon_sym__] = ACTIONS(686),
    [anon_sym_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(676),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(676),
    [anon_sym_COMMA] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(676),
    [anon_sym_DASH_GT] = ACTIONS(676),
    [anon_sym_POUND] = ACTIONS(676),
    [anon_sym_SQUOTE] = ACTIONS(674),
    [anon_sym_as] = ACTIONS(674),
    [anon_sym_async] = ACTIONS(674),
    [anon_sym_await] = ACTIONS(674),
    [anon_sym_break] = ACTIONS(674),
    [anon_sym_const] = ACTIONS(674),
    [anon_sym_continue] = ACTIONS(674),
    [anon_sym_default] = ACTIONS(674),
    [anon_sym_enum] = ACTIONS(674),
    [anon_sym_fn] = ACTIONS(674),
    [anon_sym_for] = ACTIONS(674),
    [anon_sym_gen] = ACTIONS(674),
    [anon_sym_if] = ACTIONS(674),
    [anon_sym_impl] = ACTIONS(674),
    [anon_sym_let] = ACTIONS(674),
    [anon_sym_loop] = ACTIONS(674),
    [anon_sym_match] = ACTIONS(674),
    [anon_sym_mod] = ACTIONS(674),
    [anon_sym_pub] = ACTIONS(674),
    [anon_sym_return] = ACTIONS(674),
    [anon_sym_static] = ACTIONS(674),
    [anon_sym_struct] = ACTIONS(674),
    [anon_sym_trait] = ACTIONS(674),
    [anon_sym_type] = ACTIONS(674),
    [anon_sym_union] = ACTIONS(674),
    [anon_sym_unsafe] = ACTIONS(674),
    [anon_sym_use] = ACTIONS(674),
    [anon_sym_where] = ACTIONS(674),
    [anon_sym_while] = ACTIONS(674),
    [sym_mutable_specifier] = ACTIONS(674),
    [sym_integer_literal] = ACTIONS(690),
    [aux_sym_string_literal_token1] = ACTIONS(692),
    [sym_char_literal] = ACTIONS(690),
    [anon_sym_true] = ACTIONS(694),
    [anon_sym_false] = ACTIONS(694),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(674),
    [sym_super] = ACTIONS(674),
    [sym_crate] = ACTIONS(674),
    [sym__raw_string_literal_start] = ACTIONS(696),
    [sym_float_literal] = ACTIONS(690),
  },
  [103] = {
    [sym_delim_token_tree] = STATE(198),
    [sym__delim_tokens] = STATE(206),
    [sym__non_delim_token] = STATE(198),
    [sym__literal] = STATE(214),
    [sym_string_literal] = STATE(199),
    [sym_raw_string_literal] = STATE(199),
    [sym_boolean_literal] = STATE(199),
    [sym_line_comment] = STATE(103),
    [sym_block_comment] = STATE(103),
    [aux_sym__non_special_token_repeat1] = STATE(157),
    [aux_sym_delim_token_tree_repeat1] = STATE(83),
    [sym_identifier] = ACTIONS(674),
    [anon_sym_SEMI] = ACTIONS(676),
    [anon_sym_LPAREN] = ACTIONS(678),
    [anon_sym_RPAREN] = ACTIONS(728),
    [anon_sym_LBRACK] = ACTIONS(680),
    [anon_sym_LBRACE] = ACTIONS(682),
    [anon_sym_EQ_GT] = ACTIONS(676),
    [anon_sym_COLON] = ACTIONS(686),
    [anon_sym_DOLLAR] = ACTIONS(688),
    [anon_sym_PLUS] = ACTIONS(686),
    [anon_sym_STAR] = ACTIONS(686),
    [anon_sym_QMARK] = ACTIONS(676),
    [anon_sym_u8] = ACTIONS(674),
    [anon_sym_i8] = ACTIONS(674),
    [anon_sym_u16] = ACTIONS(674),
    [anon_sym_i16] = ACTIONS(674),
    [anon_sym_u32] = ACTIONS(674),
    [anon_sym_i32] = ACTIONS(674),
    [anon_sym_u64] = ACTIONS(674),
    [anon_sym_i64] = ACTIONS(674),
    [anon_sym_u128] = ACTIONS(674),
    [anon_sym_i128] = ACTIONS(674),
    [anon_sym_isize] = ACTIONS(674),
    [anon_sym_usize] = ACTIONS(674),
    [anon_sym_f32] = ACTIONS(674),
    [anon_sym_f64] = ACTIONS(674),
    [anon_sym_bool] = ACTIONS(674),
    [anon_sym_str] = ACTIONS(674),
    [anon_sym_char] = ACTIONS(674),
    [anon_sym_DASH] = ACTIONS(686),
    [anon_sym_SLASH] = ACTIONS(686),
    [anon_sym_PERCENT] = ACTIONS(686),
    [anon_sym_CARET] = ACTIONS(686),
    [anon_sym_BANG] = ACTIONS(686),
    [anon_sym_AMP] = ACTIONS(686),
    [anon_sym_PIPE] = ACTIONS(686),
    [anon_sym_AMP_AMP] = ACTIONS(676),
    [anon_sym_PIPE_PIPE] = ACTIONS(676),
    [anon_sym_LT_LT] = ACTIONS(686),
    [anon_sym_GT_GT] = ACTIONS(686),
    [anon_sym_PLUS_EQ] = ACTIONS(676),
    [anon_sym_DASH_EQ] = ACTIONS(676),
    [anon_sym_STAR_EQ] = ACTIONS(676),
    [anon_sym_SLASH_EQ] = ACTIONS(676),
    [anon_sym_PERCENT_EQ] = ACTIONS(676),
    [anon_sym_CARET_EQ] = ACTIONS(676),
    [anon_sym_AMP_EQ] = ACTIONS(676),
    [anon_sym_PIPE_EQ] = ACTIONS(676),
    [anon_sym_LT_LT_EQ] = ACTIONS(676),
    [anon_sym_GT_GT_EQ] = ACTIONS(676),
    [anon_sym_EQ] = ACTIONS(686),
    [anon_sym_EQ_EQ] = ACTIONS(676),
    [anon_sym_BANG_EQ] = ACTIONS(676),
    [anon_sym_GT] = ACTIONS(686),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT_EQ] = ACTIONS(676),
    [anon_sym_LT_EQ] = ACTIONS(676),
    [anon_sym_AT] = ACTIONS(676),
    [anon_sym__] = ACTIONS(686),
    [anon_sym_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(676),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(676),
    [anon_sym_COMMA] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(676),
    [anon_sym_DASH_GT] = ACTIONS(676),
    [anon_sym_POUND] = ACTIONS(676),
    [anon_sym_SQUOTE] = ACTIONS(674),
    [anon_sym_as] = ACTIONS(674),
    [anon_sym_async] = ACTIONS(674),
    [anon_sym_await] = ACTIONS(674),
    [anon_sym_break] = ACTIONS(674),
    [anon_sym_const] = ACTIONS(674),
    [anon_sym_continue] = ACTIONS(674),
    [anon_sym_default] = ACTIONS(674),
    [anon_sym_enum] = ACTIONS(674),
    [anon_sym_fn] = ACTIONS(674),
    [anon_sym_for] = ACTIONS(674),
    [anon_sym_gen] = ACTIONS(674),
    [anon_sym_if] = ACTIONS(674),
    [anon_sym_impl] = ACTIONS(674),
    [anon_sym_let] = ACTIONS(674),
    [anon_sym_loop] = ACTIONS(674),
    [anon_sym_match] = ACTIONS(674),
    [anon_sym_mod] = ACTIONS(674),
    [anon_sym_pub] = ACTIONS(674),
    [anon_sym_return] = ACTIONS(674),
    [anon_sym_static] = ACTIONS(674),
    [anon_sym_struct] = ACTIONS(674),
    [anon_sym_trait] = ACTIONS(674),
    [anon_sym_type] = ACTIONS(674),
    [anon_sym_union] = ACTIONS(674),
    [anon_sym_unsafe] = ACTIONS(674),
    [anon_sym_use] = ACTIONS(674),
    [anon_sym_where] = ACTIONS(674),
    [anon_sym_while] = ACTIONS(674),
    [sym_mutable_specifier] = ACTIONS(674),
    [sym_integer_literal] = ACTIONS(690),
    [aux_sym_string_literal_token1] = ACTIONS(692),
    [sym_char_literal] = ACTIONS(690),
    [anon_sym_true] = ACTIONS(694),
    [anon_sym_false] = ACTIONS(694),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(674),
    [sym_super] = ACTIONS(674),
    [sym_crate] = ACTIONS(674),
    [sym__raw_string_literal_start] = ACTIONS(696),
    [sym_float_literal] = ACTIONS(690),
  },
  [104] = {
    [sym_delim_token_tree] = STATE(198),
    [sym__delim_tokens] = STATE(206),
    [sym__non_delim_token] = STATE(198),
    [sym__literal] = STATE(214),
    [sym_string_literal] = STATE(199),
    [sym_raw_string_literal] = STATE(199),
    [sym_boolean_literal] = STATE(199),
    [sym_line_comment] = STATE(104),
    [sym_block_comment] = STATE(104),
    [aux_sym__non_special_token_repeat1] = STATE(157),
    [aux_sym_delim_token_tree_repeat1] = STATE(83),
    [sym_identifier] = ACTIONS(674),
    [anon_sym_SEMI] = ACTIONS(676),
    [anon_sym_LPAREN] = ACTIONS(678),
    [anon_sym_LBRACK] = ACTIONS(680),
    [anon_sym_RBRACK] = ACTIONS(728),
    [anon_sym_LBRACE] = ACTIONS(682),
    [anon_sym_EQ_GT] = ACTIONS(676),
    [anon_sym_COLON] = ACTIONS(686),
    [anon_sym_DOLLAR] = ACTIONS(688),
    [anon_sym_PLUS] = ACTIONS(686),
    [anon_sym_STAR] = ACTIONS(686),
    [anon_sym_QMARK] = ACTIONS(676),
    [anon_sym_u8] = ACTIONS(674),
    [anon_sym_i8] = ACTIONS(674),
    [anon_sym_u16] = ACTIONS(674),
    [anon_sym_i16] = ACTIONS(674),
    [anon_sym_u32] = ACTIONS(674),
    [anon_sym_i32] = ACTIONS(674),
    [anon_sym_u64] = ACTIONS(674),
    [anon_sym_i64] = ACTIONS(674),
    [anon_sym_u128] = ACTIONS(674),
    [anon_sym_i128] = ACTIONS(674),
    [anon_sym_isize] = ACTIONS(674),
    [anon_sym_usize] = ACTIONS(674),
    [anon_sym_f32] = ACTIONS(674),
    [anon_sym_f64] = ACTIONS(674),
    [anon_sym_bool] = ACTIONS(674),
    [anon_sym_str] = ACTIONS(674),
    [anon_sym_char] = ACTIONS(674),
    [anon_sym_DASH] = ACTIONS(686),
    [anon_sym_SLASH] = ACTIONS(686),
    [anon_sym_PERCENT] = ACTIONS(686),
    [anon_sym_CARET] = ACTIONS(686),
    [anon_sym_BANG] = ACTIONS(686),
    [anon_sym_AMP] = ACTIONS(686),
    [anon_sym_PIPE] = ACTIONS(686),
    [anon_sym_AMP_AMP] = ACTIONS(676),
    [anon_sym_PIPE_PIPE] = ACTIONS(676),
    [anon_sym_LT_LT] = ACTIONS(686),
    [anon_sym_GT_GT] = ACTIONS(686),
    [anon_sym_PLUS_EQ] = ACTIONS(676),
    [anon_sym_DASH_EQ] = ACTIONS(676),
    [anon_sym_STAR_EQ] = ACTIONS(676),
    [anon_sym_SLASH_EQ] = ACTIONS(676),
    [anon_sym_PERCENT_EQ] = ACTIONS(676),
    [anon_sym_CARET_EQ] = ACTIONS(676),
    [anon_sym_AMP_EQ] = ACTIONS(676),
    [anon_sym_PIPE_EQ] = ACTIONS(676),
    [anon_sym_LT_LT_EQ] = ACTIONS(676),
    [anon_sym_GT_GT_EQ] = ACTIONS(676),
    [anon_sym_EQ] = ACTIONS(686),
    [anon_sym_EQ_EQ] = ACTIONS(676),
    [anon_sym_BANG_EQ] = ACTIONS(676),
    [anon_sym_GT] = ACTIONS(686),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT_EQ] = ACTIONS(676),
    [anon_sym_LT_EQ] = ACTIONS(676),
    [anon_sym_AT] = ACTIONS(676),
    [anon_sym__] = ACTIONS(686),
    [anon_sym_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(676),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(676),
    [anon_sym_COMMA] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(676),
    [anon_sym_DASH_GT] = ACTIONS(676),
    [anon_sym_POUND] = ACTIONS(676),
    [anon_sym_SQUOTE] = ACTIONS(674),
    [anon_sym_as] = ACTIONS(674),
    [anon_sym_async] = ACTIONS(674),
    [anon_sym_await] = ACTIONS(674),
    [anon_sym_break] = ACTIONS(674),
    [anon_sym_const] = ACTIONS(674),
    [anon_sym_continue] = ACTIONS(674),
    [anon_sym_default] = ACTIONS(674),
    [anon_sym_enum] = ACTIONS(674),
    [anon_sym_fn] = ACTIONS(674),
    [anon_sym_for] = ACTIONS(674),
    [anon_sym_gen] = ACTIONS(674),
    [anon_sym_if] = ACTIONS(674),
    [anon_sym_impl] = ACTIONS(674),
    [anon_sym_let] = ACTIONS(674),
    [anon_sym_loop] = ACTIONS(674),
    [anon_sym_match] = ACTIONS(674),
    [anon_sym_mod] = ACTIONS(674),
    [anon_sym_pub] = ACTIONS(674),
    [anon_sym_return] = ACTIONS(674),
    [anon_sym_static] = ACTIONS(674),
    [anon_sym_struct] = ACTIONS(674),
    [anon_sym_trait] = ACTIONS(674),
    [anon_sym_type] = ACTIONS(674),
    [anon_sym_union] = ACTIONS(674),
    [anon_sym_unsafe] = ACTIONS(674),
    [anon_sym_use] = ACTIONS(674),
    [anon_sym_where] = ACTIONS(674),
    [anon_sym_while] = ACTIONS(674),
    [sym_mutable_specifier] = ACTIONS(674),
    [sym_integer_literal] = ACTIONS(690),
    [aux_sym_string_literal_token1] = ACTIONS(692),
    [sym_char_literal] = ACTIONS(690),
    [anon_sym_true] = ACTIONS(694),
    [anon_sym_false] = ACTIONS(694),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(674),
    [sym_super] = ACTIONS(674),
    [sym_crate] = ACTIONS(674),
    [sym__raw_string_literal_start] = ACTIONS(696),
    [sym_float_literal] = ACTIONS(690),
  },
  [105] = {
    [sym_delim_token_tree] = STATE(198),
    [sym__delim_tokens] = STATE(206),
    [sym__non_delim_token] = STATE(198),
    [sym__literal] = STATE(214),
    [sym_string_literal] = STATE(199),
    [sym_raw_string_literal] = STATE(199),
    [sym_boolean_literal] = STATE(199),
    [sym_line_comment] = STATE(105),
    [sym_block_comment] = STATE(105),
    [aux_sym__non_special_token_repeat1] = STATE(157),
    [aux_sym_delim_token_tree_repeat1] = STATE(83),
    [sym_identifier] = ACTIONS(674),
    [anon_sym_SEMI] = ACTIONS(676),
    [anon_sym_LPAREN] = ACTIONS(678),
    [anon_sym_LBRACK] = ACTIONS(680),
    [anon_sym_LBRACE] = ACTIONS(682),
    [anon_sym_RBRACE] = ACTIONS(728),
    [anon_sym_EQ_GT] = ACTIONS(676),
    [anon_sym_COLON] = ACTIONS(686),
    [anon_sym_DOLLAR] = ACTIONS(688),
    [anon_sym_PLUS] = ACTIONS(686),
    [anon_sym_STAR] = ACTIONS(686),
    [anon_sym_QMARK] = ACTIONS(676),
    [anon_sym_u8] = ACTIONS(674),
    [anon_sym_i8] = ACTIONS(674),
    [anon_sym_u16] = ACTIONS(674),
    [anon_sym_i16] = ACTIONS(674),
    [anon_sym_u32] = ACTIONS(674),
    [anon_sym_i32] = ACTIONS(674),
    [anon_sym_u64] = ACTIONS(674),
    [anon_sym_i64] = ACTIONS(674),
    [anon_sym_u128] = ACTIONS(674),
    [anon_sym_i128] = ACTIONS(674),
    [anon_sym_isize] = ACTIONS(674),
    [anon_sym_usize] = ACTIONS(674),
    [anon_sym_f32] = ACTIONS(674),
    [anon_sym_f64] = ACTIONS(674),
    [anon_sym_bool] = ACTIONS(674),
    [anon_sym_str] = ACTIONS(674),
    [anon_sym_char] = ACTIONS(674),
    [anon_sym_DASH] = ACTIONS(686),
    [anon_sym_SLASH] = ACTIONS(686),
    [anon_sym_PERCENT] = ACTIONS(686),
    [anon_sym_CARET] = ACTIONS(686),
    [anon_sym_BANG] = ACTIONS(686),
    [anon_sym_AMP] = ACTIONS(686),
    [anon_sym_PIPE] = ACTIONS(686),
    [anon_sym_AMP_AMP] = ACTIONS(676),
    [anon_sym_PIPE_PIPE] = ACTIONS(676),
    [anon_sym_LT_LT] = ACTIONS(686),
    [anon_sym_GT_GT] = ACTIONS(686),
    [anon_sym_PLUS_EQ] = ACTIONS(676),
    [anon_sym_DASH_EQ] = ACTIONS(676),
    [anon_sym_STAR_EQ] = ACTIONS(676),
    [anon_sym_SLASH_EQ] = ACTIONS(676),
    [anon_sym_PERCENT_EQ] = ACTIONS(676),
    [anon_sym_CARET_EQ] = ACTIONS(676),
    [anon_sym_AMP_EQ] = ACTIONS(676),
    [anon_sym_PIPE_EQ] = ACTIONS(676),
    [anon_sym_LT_LT_EQ] = ACTIONS(676),
    [anon_sym_GT_GT_EQ] = ACTIONS(676),
    [anon_sym_EQ] = ACTIONS(686),
    [anon_sym_EQ_EQ] = ACTIONS(676),
    [anon_sym_BANG_EQ] = ACTIONS(676),
    [anon_sym_GT] = ACTIONS(686),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT_EQ] = ACTIONS(676),
    [anon_sym_LT_EQ] = ACTIONS(676),
    [anon_sym_AT] = ACTIONS(676),
    [anon_sym__] = ACTIONS(686),
    [anon_sym_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(676),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(676),
    [anon_sym_COMMA] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(676),
    [anon_sym_DASH_GT] = ACTIONS(676),
    [anon_sym_POUND] = ACTIONS(676),
    [anon_sym_SQUOTE] = ACTIONS(674),
    [anon_sym_as] = ACTIONS(674),
    [anon_sym_async] = ACTIONS(674),
    [anon_sym_await] = ACTIONS(674),
    [anon_sym_break] = ACTIONS(674),
    [anon_sym_const] = ACTIONS(674),
    [anon_sym_continue] = ACTIONS(674),
    [anon_sym_default] = ACTIONS(674),
    [anon_sym_enum] = ACTIONS(674),
    [anon_sym_fn] = ACTIONS(674),
    [anon_sym_for] = ACTIONS(674),
    [anon_sym_gen] = ACTIONS(674),
    [anon_sym_if] = ACTIONS(674),
    [anon_sym_impl] = ACTIONS(674),
    [anon_sym_let] = ACTIONS(674),
    [anon_sym_loop] = ACTIONS(674),
    [anon_sym_match] = ACTIONS(674),
    [anon_sym_mod] = ACTIONS(674),
    [anon_sym_pub] = ACTIONS(674),
    [anon_sym_return] = ACTIONS(674),
    [anon_sym_static] = ACTIONS(674),
    [anon_sym_struct] = ACTIONS(674),
    [anon_sym_trait] = ACTIONS(674),
    [anon_sym_type] = ACTIONS(674),
    [anon_sym_union] = ACTIONS(674),
    [anon_sym_unsafe] = ACTIONS(674),
    [anon_sym_use] = ACTIONS(674),
    [anon_sym_where] = ACTIONS(674),
    [anon_sym_while] = ACTIONS(674),
    [sym_mutable_specifier] = ACTIONS(674),
    [sym_integer_literal] = ACTIONS(690),
    [aux_sym_string_literal_token1] = ACTIONS(692),
    [sym_char_literal] = ACTIONS(690),
    [anon_sym_true] = ACTIONS(694),
    [anon_sym_false] = ACTIONS(694),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(674),
    [sym_super] = ACTIONS(674),
    [sym_crate] = ACTIONS(674),
    [sym__raw_string_literal_start] = ACTIONS(696),
    [sym_float_literal] = ACTIONS(690),
  },
  [106] = {
    [sym_delim_token_tree] = STATE(198),
    [sym__delim_tokens] = STATE(206),
    [sym__non_delim_token] = STATE(198),
    [sym__literal] = STATE(214),
    [sym_string_literal] = STATE(199),
    [sym_raw_string_literal] = STATE(199),
    [sym_boolean_literal] = STATE(199),
    [sym_line_comment] = STATE(106),
    [sym_block_comment] = STATE(106),
    [aux_sym__non_special_token_repeat1] = STATE(157),
    [aux_sym_delim_token_tree_repeat1] = STATE(97),
    [sym_identifier] = ACTIONS(674),
    [anon_sym_SEMI] = ACTIONS(676),
    [anon_sym_LPAREN] = ACTIONS(678),
    [anon_sym_LBRACK] = ACTIONS(680),
    [anon_sym_LBRACE] = ACTIONS(682),
    [anon_sym_RBRACE] = ACTIONS(726),
    [anon_sym_EQ_GT] = ACTIONS(676),
    [anon_sym_COLON] = ACTIONS(686),
    [anon_sym_DOLLAR] = ACTIONS(688),
    [anon_sym_PLUS] = ACTIONS(686),
    [anon_sym_STAR] = ACTIONS(686),
    [anon_sym_QMARK] = ACTIONS(676),
    [anon_sym_u8] = ACTIONS(674),
    [anon_sym_i8] = ACTIONS(674),
    [anon_sym_u16] = ACTIONS(674),
    [anon_sym_i16] = ACTIONS(674),
    [anon_sym_u32] = ACTIONS(674),
    [anon_sym_i32] = ACTIONS(674),
    [anon_sym_u64] = ACTIONS(674),
    [anon_sym_i64] = ACTIONS(674),
    [anon_sym_u128] = ACTIONS(674),
    [anon_sym_i128] = ACTIONS(674),
    [anon_sym_isize] = ACTIONS(674),
    [anon_sym_usize] = ACTIONS(674),
    [anon_sym_f32] = ACTIONS(674),
    [anon_sym_f64] = ACTIONS(674),
    [anon_sym_bool] = ACTIONS(674),
    [anon_sym_str] = ACTIONS(674),
    [anon_sym_char] = ACTIONS(674),
    [anon_sym_DASH] = ACTIONS(686),
    [anon_sym_SLASH] = ACTIONS(686),
    [anon_sym_PERCENT] = ACTIONS(686),
    [anon_sym_CARET] = ACTIONS(686),
    [anon_sym_BANG] = ACTIONS(686),
    [anon_sym_AMP] = ACTIONS(686),
    [anon_sym_PIPE] = ACTIONS(686),
    [anon_sym_AMP_AMP] = ACTIONS(676),
    [anon_sym_PIPE_PIPE] = ACTIONS(676),
    [anon_sym_LT_LT] = ACTIONS(686),
    [anon_sym_GT_GT] = ACTIONS(686),
    [anon_sym_PLUS_EQ] = ACTIONS(676),
    [anon_sym_DASH_EQ] = ACTIONS(676),
    [anon_sym_STAR_EQ] = ACTIONS(676),
    [anon_sym_SLASH_EQ] = ACTIONS(676),
    [anon_sym_PERCENT_EQ] = ACTIONS(676),
    [anon_sym_CARET_EQ] = ACTIONS(676),
    [anon_sym_AMP_EQ] = ACTIONS(676),
    [anon_sym_PIPE_EQ] = ACTIONS(676),
    [anon_sym_LT_LT_EQ] = ACTIONS(676),
    [anon_sym_GT_GT_EQ] = ACTIONS(676),
    [anon_sym_EQ] = ACTIONS(686),
    [anon_sym_EQ_EQ] = ACTIONS(676),
    [anon_sym_BANG_EQ] = ACTIONS(676),
    [anon_sym_GT] = ACTIONS(686),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT_EQ] = ACTIONS(676),
    [anon_sym_LT_EQ] = ACTIONS(676),
    [anon_sym_AT] = ACTIONS(676),
    [anon_sym__] = ACTIONS(686),
    [anon_sym_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(676),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(676),
    [anon_sym_COMMA] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(676),
    [anon_sym_DASH_GT] = ACTIONS(676),
    [anon_sym_POUND] = ACTIONS(676),
    [anon_sym_SQUOTE] = ACTIONS(674),
    [anon_sym_as] = ACTIONS(674),
    [anon_sym_async] = ACTIONS(674),
    [anon_sym_await] = ACTIONS(674),
    [anon_sym_break] = ACTIONS(674),
    [anon_sym_const] = ACTIONS(674),
    [anon_sym_continue] = ACTIONS(674),
    [anon_sym_default] = ACTIONS(674),
    [anon_sym_enum] = ACTIONS(674),
    [anon_sym_fn] = ACTIONS(674),
    [anon_sym_for] = ACTIONS(674),
    [anon_sym_gen] = ACTIONS(674),
    [anon_sym_if] = ACTIONS(674),
    [anon_sym_impl] = ACTIONS(674),
    [anon_sym_let] = ACTIONS(674),
    [anon_sym_loop] = ACTIONS(674),
    [anon_sym_match] = ACTIONS(674),
    [anon_sym_mod] = ACTIONS(674),
    [anon_sym_pub] = ACTIONS(674),
    [anon_sym_return] = ACTIONS(674),
    [anon_sym_static] = ACTIONS(674),
    [anon_sym_struct] = ACTIONS(674),
    [anon_sym_trait] = ACTIONS(674),
    [anon_sym_type] = ACTIONS(674),
    [anon_sym_union] = ACTIONS(674),
    [anon_sym_unsafe] = ACTIONS(674),
    [anon_sym_use] = ACTIONS(674),
    [anon_sym_where] = ACTIONS(674),
    [anon_sym_while] = ACTIONS(674),
    [sym_mutable_specifier] = ACTIONS(674),
    [sym_integer_literal] = ACTIONS(690),
    [aux_sym_string_literal_token1] = ACTIONS(692),
    [sym_char_literal] = ACTIONS(690),
    [anon_sym_true] = ACTIONS(694),
    [anon_sym_false] = ACTIONS(694),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(674),
    [sym_super] = ACTIONS(674),
    [sym_crate] = ACTIONS(674),
    [sym__raw_string_literal_start] = ACTIONS(696),
    [sym_float_literal] = ACTIONS(690),
  },
  [107] = {
    [sym_delim_token_tree] = STATE(198),
    [sym__delim_tokens] = STATE(206),
    [sym__non_delim_token] = STATE(198),
    [sym__literal] = STATE(214),
    [sym_string_literal] = STATE(199),
    [sym_raw_string_literal] = STATE(199),
    [sym_boolean_literal] = STATE(199),
    [sym_line_comment] = STATE(107),
    [sym_block_comment] = STATE(107),
    [aux_sym__non_special_token_repeat1] = STATE(157),
    [aux_sym_delim_token_tree_repeat1] = STATE(110),
    [sym_identifier] = ACTIONS(674),
    [anon_sym_SEMI] = ACTIONS(676),
    [anon_sym_LPAREN] = ACTIONS(678),
    [anon_sym_RPAREN] = ACTIONS(730),
    [anon_sym_LBRACK] = ACTIONS(680),
    [anon_sym_LBRACE] = ACTIONS(682),
    [anon_sym_EQ_GT] = ACTIONS(676),
    [anon_sym_COLON] = ACTIONS(686),
    [anon_sym_DOLLAR] = ACTIONS(688),
    [anon_sym_PLUS] = ACTIONS(686),
    [anon_sym_STAR] = ACTIONS(686),
    [anon_sym_QMARK] = ACTIONS(676),
    [anon_sym_u8] = ACTIONS(674),
    [anon_sym_i8] = ACTIONS(674),
    [anon_sym_u16] = ACTIONS(674),
    [anon_sym_i16] = ACTIONS(674),
    [anon_sym_u32] = ACTIONS(674),
    [anon_sym_i32] = ACTIONS(674),
    [anon_sym_u64] = ACTIONS(674),
    [anon_sym_i64] = ACTIONS(674),
    [anon_sym_u128] = ACTIONS(674),
    [anon_sym_i128] = ACTIONS(674),
    [anon_sym_isize] = ACTIONS(674),
    [anon_sym_usize] = ACTIONS(674),
    [anon_sym_f32] = ACTIONS(674),
    [anon_sym_f64] = ACTIONS(674),
    [anon_sym_bool] = ACTIONS(674),
    [anon_sym_str] = ACTIONS(674),
    [anon_sym_char] = ACTIONS(674),
    [anon_sym_DASH] = ACTIONS(686),
    [anon_sym_SLASH] = ACTIONS(686),
    [anon_sym_PERCENT] = ACTIONS(686),
    [anon_sym_CARET] = ACTIONS(686),
    [anon_sym_BANG] = ACTIONS(686),
    [anon_sym_AMP] = ACTIONS(686),
    [anon_sym_PIPE] = ACTIONS(686),
    [anon_sym_AMP_AMP] = ACTIONS(676),
    [anon_sym_PIPE_PIPE] = ACTIONS(676),
    [anon_sym_LT_LT] = ACTIONS(686),
    [anon_sym_GT_GT] = ACTIONS(686),
    [anon_sym_PLUS_EQ] = ACTIONS(676),
    [anon_sym_DASH_EQ] = ACTIONS(676),
    [anon_sym_STAR_EQ] = ACTIONS(676),
    [anon_sym_SLASH_EQ] = ACTIONS(676),
    [anon_sym_PERCENT_EQ] = ACTIONS(676),
    [anon_sym_CARET_EQ] = ACTIONS(676),
    [anon_sym_AMP_EQ] = ACTIONS(676),
    [anon_sym_PIPE_EQ] = ACTIONS(676),
    [anon_sym_LT_LT_EQ] = ACTIONS(676),
    [anon_sym_GT_GT_EQ] = ACTIONS(676),
    [anon_sym_EQ] = ACTIONS(686),
    [anon_sym_EQ_EQ] = ACTIONS(676),
    [anon_sym_BANG_EQ] = ACTIONS(676),
    [anon_sym_GT] = ACTIONS(686),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT_EQ] = ACTIONS(676),
    [anon_sym_LT_EQ] = ACTIONS(676),
    [anon_sym_AT] = ACTIONS(676),
    [anon_sym__] = ACTIONS(686),
    [anon_sym_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(676),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(676),
    [anon_sym_COMMA] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(676),
    [anon_sym_DASH_GT] = ACTIONS(676),
    [anon_sym_POUND] = ACTIONS(676),
    [anon_sym_SQUOTE] = ACTIONS(674),
    [anon_sym_as] = ACTIONS(674),
    [anon_sym_async] = ACTIONS(674),
    [anon_sym_await] = ACTIONS(674),
    [anon_sym_break] = ACTIONS(674),
    [anon_sym_const] = ACTIONS(674),
    [anon_sym_continue] = ACTIONS(674),
    [anon_sym_default] = ACTIONS(674),
    [anon_sym_enum] = ACTIONS(674),
    [anon_sym_fn] = ACTIONS(674),
    [anon_sym_for] = ACTIONS(674),
    [anon_sym_gen] = ACTIONS(674),
    [anon_sym_if] = ACTIONS(674),
    [anon_sym_impl] = ACTIONS(674),
    [anon_sym_let] = ACTIONS(674),
    [anon_sym_loop] = ACTIONS(674),
    [anon_sym_match] = ACTIONS(674),
    [anon_sym_mod] = ACTIONS(674),
    [anon_sym_pub] = ACTIONS(674),
    [anon_sym_return] = ACTIONS(674),
    [anon_sym_static] = ACTIONS(674),
    [anon_sym_struct] = ACTIONS(674),
    [anon_sym_trait] = ACTIONS(674),
    [anon_sym_type] = ACTIONS(674),
    [anon_sym_union] = ACTIONS(674),
    [anon_sym_unsafe] = ACTIONS(674),
    [anon_sym_use] = ACTIONS(674),
    [anon_sym_where] = ACTIONS(674),
    [anon_sym_while] = ACTIONS(674),
    [sym_mutable_specifier] = ACTIONS(674),
    [sym_integer_literal] = ACTIONS(690),
    [aux_sym_string_literal_token1] = ACTIONS(692),
    [sym_char_literal] = ACTIONS(690),
    [anon_sym_true] = ACTIONS(694),
    [anon_sym_false] = ACTIONS(694),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(674),
    [sym_super] = ACTIONS(674),
    [sym_crate] = ACTIONS(674),
    [sym__raw_string_literal_start] = ACTIONS(696),
    [sym_float_literal] = ACTIONS(690),
  },
  [108] = {
    [sym_delim_token_tree] = STATE(198),
    [sym__delim_tokens] = STATE(206),
    [sym__non_delim_token] = STATE(198),
    [sym__literal] = STATE(214),
    [sym_string_literal] = STATE(199),
    [sym_raw_string_literal] = STATE(199),
    [sym_boolean_literal] = STATE(199),
    [sym_line_comment] = STATE(108),
    [sym_block_comment] = STATE(108),
    [aux_sym__non_special_token_repeat1] = STATE(157),
    [aux_sym_delim_token_tree_repeat1] = STATE(111),
    [sym_identifier] = ACTIONS(674),
    [anon_sym_SEMI] = ACTIONS(676),
    [anon_sym_LPAREN] = ACTIONS(678),
    [anon_sym_LBRACK] = ACTIONS(680),
    [anon_sym_RBRACK] = ACTIONS(730),
    [anon_sym_LBRACE] = ACTIONS(682),
    [anon_sym_EQ_GT] = ACTIONS(676),
    [anon_sym_COLON] = ACTIONS(686),
    [anon_sym_DOLLAR] = ACTIONS(688),
    [anon_sym_PLUS] = ACTIONS(686),
    [anon_sym_STAR] = ACTIONS(686),
    [anon_sym_QMARK] = ACTIONS(676),
    [anon_sym_u8] = ACTIONS(674),
    [anon_sym_i8] = ACTIONS(674),
    [anon_sym_u16] = ACTIONS(674),
    [anon_sym_i16] = ACTIONS(674),
    [anon_sym_u32] = ACTIONS(674),
    [anon_sym_i32] = ACTIONS(674),
    [anon_sym_u64] = ACTIONS(674),
    [anon_sym_i64] = ACTIONS(674),
    [anon_sym_u128] = ACTIONS(674),
    [anon_sym_i128] = ACTIONS(674),
    [anon_sym_isize] = ACTIONS(674),
    [anon_sym_usize] = ACTIONS(674),
    [anon_sym_f32] = ACTIONS(674),
    [anon_sym_f64] = ACTIONS(674),
    [anon_sym_bool] = ACTIONS(674),
    [anon_sym_str] = ACTIONS(674),
    [anon_sym_char] = ACTIONS(674),
    [anon_sym_DASH] = ACTIONS(686),
    [anon_sym_SLASH] = ACTIONS(686),
    [anon_sym_PERCENT] = ACTIONS(686),
    [anon_sym_CARET] = ACTIONS(686),
    [anon_sym_BANG] = ACTIONS(686),
    [anon_sym_AMP] = ACTIONS(686),
    [anon_sym_PIPE] = ACTIONS(686),
    [anon_sym_AMP_AMP] = ACTIONS(676),
    [anon_sym_PIPE_PIPE] = ACTIONS(676),
    [anon_sym_LT_LT] = ACTIONS(686),
    [anon_sym_GT_GT] = ACTIONS(686),
    [anon_sym_PLUS_EQ] = ACTIONS(676),
    [anon_sym_DASH_EQ] = ACTIONS(676),
    [anon_sym_STAR_EQ] = ACTIONS(676),
    [anon_sym_SLASH_EQ] = ACTIONS(676),
    [anon_sym_PERCENT_EQ] = ACTIONS(676),
    [anon_sym_CARET_EQ] = ACTIONS(676),
    [anon_sym_AMP_EQ] = ACTIONS(676),
    [anon_sym_PIPE_EQ] = ACTIONS(676),
    [anon_sym_LT_LT_EQ] = ACTIONS(676),
    [anon_sym_GT_GT_EQ] = ACTIONS(676),
    [anon_sym_EQ] = ACTIONS(686),
    [anon_sym_EQ_EQ] = ACTIONS(676),
    [anon_sym_BANG_EQ] = ACTIONS(676),
    [anon_sym_GT] = ACTIONS(686),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT_EQ] = ACTIONS(676),
    [anon_sym_LT_EQ] = ACTIONS(676),
    [anon_sym_AT] = ACTIONS(676),
    [anon_sym__] = ACTIONS(686),
    [anon_sym_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(676),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(676),
    [anon_sym_COMMA] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(676),
    [anon_sym_DASH_GT] = ACTIONS(676),
    [anon_sym_POUND] = ACTIONS(676),
    [anon_sym_SQUOTE] = ACTIONS(674),
    [anon_sym_as] = ACTIONS(674),
    [anon_sym_async] = ACTIONS(674),
    [anon_sym_await] = ACTIONS(674),
    [anon_sym_break] = ACTIONS(674),
    [anon_sym_const] = ACTIONS(674),
    [anon_sym_continue] = ACTIONS(674),
    [anon_sym_default] = ACTIONS(674),
    [anon_sym_enum] = ACTIONS(674),
    [anon_sym_fn] = ACTIONS(674),
    [anon_sym_for] = ACTIONS(674),
    [anon_sym_gen] = ACTIONS(674),
    [anon_sym_if] = ACTIONS(674),
    [anon_sym_impl] = ACTIONS(674),
    [anon_sym_let] = ACTIONS(674),
    [anon_sym_loop] = ACTIONS(674),
    [anon_sym_match] = ACTIONS(674),
    [anon_sym_mod] = ACTIONS(674),
    [anon_sym_pub] = ACTIONS(674),
    [anon_sym_return] = ACTIONS(674),
    [anon_sym_static] = ACTIONS(674),
    [anon_sym_struct] = ACTIONS(674),
    [anon_sym_trait] = ACTIONS(674),
    [anon_sym_type] = ACTIONS(674),
    [anon_sym_union] = ACTIONS(674),
    [anon_sym_unsafe] = ACTIONS(674),
    [anon_sym_use] = ACTIONS(674),
    [anon_sym_where] = ACTIONS(674),
    [anon_sym_while] = ACTIONS(674),
    [sym_mutable_specifier] = ACTIONS(674),
    [sym_integer_literal] = ACTIONS(690),
    [aux_sym_string_literal_token1] = ACTIONS(692),
    [sym_char_literal] = ACTIONS(690),
    [anon_sym_true] = ACTIONS(694),
    [anon_sym_false] = ACTIONS(694),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(674),
    [sym_super] = ACTIONS(674),
    [sym_crate] = ACTIONS(674),
    [sym__raw_string_literal_start] = ACTIONS(696),
    [sym_float_literal] = ACTIONS(690),
  },
  [109] = {
    [sym_delim_token_tree] = STATE(198),
    [sym__delim_tokens] = STATE(206),
    [sym__non_delim_token] = STATE(198),
    [sym__literal] = STATE(214),
    [sym_string_literal] = STATE(199),
    [sym_raw_string_literal] = STATE(199),
    [sym_boolean_literal] = STATE(199),
    [sym_line_comment] = STATE(109),
    [sym_block_comment] = STATE(109),
    [aux_sym__non_special_token_repeat1] = STATE(157),
    [aux_sym_delim_token_tree_repeat1] = STATE(112),
    [sym_identifier] = ACTIONS(674),
    [anon_sym_SEMI] = ACTIONS(676),
    [anon_sym_LPAREN] = ACTIONS(678),
    [anon_sym_LBRACK] = ACTIONS(680),
    [anon_sym_LBRACE] = ACTIONS(682),
    [anon_sym_RBRACE] = ACTIONS(730),
    [anon_sym_EQ_GT] = ACTIONS(676),
    [anon_sym_COLON] = ACTIONS(686),
    [anon_sym_DOLLAR] = ACTIONS(688),
    [anon_sym_PLUS] = ACTIONS(686),
    [anon_sym_STAR] = ACTIONS(686),
    [anon_sym_QMARK] = ACTIONS(676),
    [anon_sym_u8] = ACTIONS(674),
    [anon_sym_i8] = ACTIONS(674),
    [anon_sym_u16] = ACTIONS(674),
    [anon_sym_i16] = ACTIONS(674),
    [anon_sym_u32] = ACTIONS(674),
    [anon_sym_i32] = ACTIONS(674),
    [anon_sym_u64] = ACTIONS(674),
    [anon_sym_i64] = ACTIONS(674),
    [anon_sym_u128] = ACTIONS(674),
    [anon_sym_i128] = ACTIONS(674),
    [anon_sym_isize] = ACTIONS(674),
    [anon_sym_usize] = ACTIONS(674),
    [anon_sym_f32] = ACTIONS(674),
    [anon_sym_f64] = ACTIONS(674),
    [anon_sym_bool] = ACTIONS(674),
    [anon_sym_str] = ACTIONS(674),
    [anon_sym_char] = ACTIONS(674),
    [anon_sym_DASH] = ACTIONS(686),
    [anon_sym_SLASH] = ACTIONS(686),
    [anon_sym_PERCENT] = ACTIONS(686),
    [anon_sym_CARET] = ACTIONS(686),
    [anon_sym_BANG] = ACTIONS(686),
    [anon_sym_AMP] = ACTIONS(686),
    [anon_sym_PIPE] = ACTIONS(686),
    [anon_sym_AMP_AMP] = ACTIONS(676),
    [anon_sym_PIPE_PIPE] = ACTIONS(676),
    [anon_sym_LT_LT] = ACTIONS(686),
    [anon_sym_GT_GT] = ACTIONS(686),
    [anon_sym_PLUS_EQ] = ACTIONS(676),
    [anon_sym_DASH_EQ] = ACTIONS(676),
    [anon_sym_STAR_EQ] = ACTIONS(676),
    [anon_sym_SLASH_EQ] = ACTIONS(676),
    [anon_sym_PERCENT_EQ] = ACTIONS(676),
    [anon_sym_CARET_EQ] = ACTIONS(676),
    [anon_sym_AMP_EQ] = ACTIONS(676),
    [anon_sym_PIPE_EQ] = ACTIONS(676),
    [anon_sym_LT_LT_EQ] = ACTIONS(676),
    [anon_sym_GT_GT_EQ] = ACTIONS(676),
    [anon_sym_EQ] = ACTIONS(686),
    [anon_sym_EQ_EQ] = ACTIONS(676),
    [anon_sym_BANG_EQ] = ACTIONS(676),
    [anon_sym_GT] = ACTIONS(686),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT_EQ] = ACTIONS(676),
    [anon_sym_LT_EQ] = ACTIONS(676),
    [anon_sym_AT] = ACTIONS(676),
    [anon_sym__] = ACTIONS(686),
    [anon_sym_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(676),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(676),
    [anon_sym_COMMA] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(676),
    [anon_sym_DASH_GT] = ACTIONS(676),
    [anon_sym_POUND] = ACTIONS(676),
    [anon_sym_SQUOTE] = ACTIONS(674),
    [anon_sym_as] = ACTIONS(674),
    [anon_sym_async] = ACTIONS(674),
    [anon_sym_await] = ACTIONS(674),
    [anon_sym_break] = ACTIONS(674),
    [anon_sym_const] = ACTIONS(674),
    [anon_sym_continue] = ACTIONS(674),
    [anon_sym_default] = ACTIONS(674),
    [anon_sym_enum] = ACTIONS(674),
    [anon_sym_fn] = ACTIONS(674),
    [anon_sym_for] = ACTIONS(674),
    [anon_sym_gen] = ACTIONS(674),
    [anon_sym_if] = ACTIONS(674),
    [anon_sym_impl] = ACTIONS(674),
    [anon_sym_let] = ACTIONS(674),
    [anon_sym_loop] = ACTIONS(674),
    [anon_sym_match] = ACTIONS(674),
    [anon_sym_mod] = ACTIONS(674),
    [anon_sym_pub] = ACTIONS(674),
    [anon_sym_return] = ACTIONS(674),
    [anon_sym_static] = ACTIONS(674),
    [anon_sym_struct] = ACTIONS(674),
    [anon_sym_trait] = ACTIONS(674),
    [anon_sym_type] = ACTIONS(674),
    [anon_sym_union] = ACTIONS(674),
    [anon_sym_unsafe] = ACTIONS(674),
    [anon_sym_use] = ACTIONS(674),
    [anon_sym_where] = ACTIONS(674),
    [anon_sym_while] = ACTIONS(674),
    [sym_mutable_specifier] = ACTIONS(674),
    [sym_integer_literal] = ACTIONS(690),
    [aux_sym_string_literal_token1] = ACTIONS(692),
    [sym_char_literal] = ACTIONS(690),
    [anon_sym_true] = ACTIONS(694),
    [anon_sym_false] = ACTIONS(694),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(674),
    [sym_super] = ACTIONS(674),
    [sym_crate] = ACTIONS(674),
    [sym__raw_string_literal_start] = ACTIONS(696),
    [sym_float_literal] = ACTIONS(690),
  },
  [110] = {
    [sym_delim_token_tree] = STATE(198),
    [sym__delim_tokens] = STATE(206),
    [sym__non_delim_token] = STATE(198),
    [sym__literal] = STATE(214),
    [sym_string_literal] = STATE(199),
    [sym_raw_string_literal] = STATE(199),
    [sym_boolean_literal] = STATE(199),
    [sym_line_comment] = STATE(110),
    [sym_block_comment] = STATE(110),
    [aux_sym__non_special_token_repeat1] = STATE(157),
    [aux_sym_delim_token_tree_repeat1] = STATE(83),
    [sym_identifier] = ACTIONS(674),
    [anon_sym_SEMI] = ACTIONS(676),
    [anon_sym_LPAREN] = ACTIONS(678),
    [anon_sym_RPAREN] = ACTIONS(732),
    [anon_sym_LBRACK] = ACTIONS(680),
    [anon_sym_LBRACE] = ACTIONS(682),
    [anon_sym_EQ_GT] = ACTIONS(676),
    [anon_sym_COLON] = ACTIONS(686),
    [anon_sym_DOLLAR] = ACTIONS(688),
    [anon_sym_PLUS] = ACTIONS(686),
    [anon_sym_STAR] = ACTIONS(686),
    [anon_sym_QMARK] = ACTIONS(676),
    [anon_sym_u8] = ACTIONS(674),
    [anon_sym_i8] = ACTIONS(674),
    [anon_sym_u16] = ACTIONS(674),
    [anon_sym_i16] = ACTIONS(674),
    [anon_sym_u32] = ACTIONS(674),
    [anon_sym_i32] = ACTIONS(674),
    [anon_sym_u64] = ACTIONS(674),
    [anon_sym_i64] = ACTIONS(674),
    [anon_sym_u128] = ACTIONS(674),
    [anon_sym_i128] = ACTIONS(674),
    [anon_sym_isize] = ACTIONS(674),
    [anon_sym_usize] = ACTIONS(674),
    [anon_sym_f32] = ACTIONS(674),
    [anon_sym_f64] = ACTIONS(674),
    [anon_sym_bool] = ACTIONS(674),
    [anon_sym_str] = ACTIONS(674),
    [anon_sym_char] = ACTIONS(674),
    [anon_sym_DASH] = ACTIONS(686),
    [anon_sym_SLASH] = ACTIONS(686),
    [anon_sym_PERCENT] = ACTIONS(686),
    [anon_sym_CARET] = ACTIONS(686),
    [anon_sym_BANG] = ACTIONS(686),
    [anon_sym_AMP] = ACTIONS(686),
    [anon_sym_PIPE] = ACTIONS(686),
    [anon_sym_AMP_AMP] = ACTIONS(676),
    [anon_sym_PIPE_PIPE] = ACTIONS(676),
    [anon_sym_LT_LT] = ACTIONS(686),
    [anon_sym_GT_GT] = ACTIONS(686),
    [anon_sym_PLUS_EQ] = ACTIONS(676),
    [anon_sym_DASH_EQ] = ACTIONS(676),
    [anon_sym_STAR_EQ] = ACTIONS(676),
    [anon_sym_SLASH_EQ] = ACTIONS(676),
    [anon_sym_PERCENT_EQ] = ACTIONS(676),
    [anon_sym_CARET_EQ] = ACTIONS(676),
    [anon_sym_AMP_EQ] = ACTIONS(676),
    [anon_sym_PIPE_EQ] = ACTIONS(676),
    [anon_sym_LT_LT_EQ] = ACTIONS(676),
    [anon_sym_GT_GT_EQ] = ACTIONS(676),
    [anon_sym_EQ] = ACTIONS(686),
    [anon_sym_EQ_EQ] = ACTIONS(676),
    [anon_sym_BANG_EQ] = ACTIONS(676),
    [anon_sym_GT] = ACTIONS(686),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT_EQ] = ACTIONS(676),
    [anon_sym_LT_EQ] = ACTIONS(676),
    [anon_sym_AT] = ACTIONS(676),
    [anon_sym__] = ACTIONS(686),
    [anon_sym_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(676),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(676),
    [anon_sym_COMMA] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(676),
    [anon_sym_DASH_GT] = ACTIONS(676),
    [anon_sym_POUND] = ACTIONS(676),
    [anon_sym_SQUOTE] = ACTIONS(674),
    [anon_sym_as] = ACTIONS(674),
    [anon_sym_async] = ACTIONS(674),
    [anon_sym_await] = ACTIONS(674),
    [anon_sym_break] = ACTIONS(674),
    [anon_sym_const] = ACTIONS(674),
    [anon_sym_continue] = ACTIONS(674),
    [anon_sym_default] = ACTIONS(674),
    [anon_sym_enum] = ACTIONS(674),
    [anon_sym_fn] = ACTIONS(674),
    [anon_sym_for] = ACTIONS(674),
    [anon_sym_gen] = ACTIONS(674),
    [anon_sym_if] = ACTIONS(674),
    [anon_sym_impl] = ACTIONS(674),
    [anon_sym_let] = ACTIONS(674),
    [anon_sym_loop] = ACTIONS(674),
    [anon_sym_match] = ACTIONS(674),
    [anon_sym_mod] = ACTIONS(674),
    [anon_sym_pub] = ACTIONS(674),
    [anon_sym_return] = ACTIONS(674),
    [anon_sym_static] = ACTIONS(674),
    [anon_sym_struct] = ACTIONS(674),
    [anon_sym_trait] = ACTIONS(674),
    [anon_sym_type] = ACTIONS(674),
    [anon_sym_union] = ACTIONS(674),
    [anon_sym_unsafe] = ACTIONS(674),
    [anon_sym_use] = ACTIONS(674),
    [anon_sym_where] = ACTIONS(674),
    [anon_sym_while] = ACTIONS(674),
    [sym_mutable_specifier] = ACTIONS(674),
    [sym_integer_literal] = ACTIONS(690),
    [aux_sym_string_literal_token1] = ACTIONS(692),
    [sym_char_literal] = ACTIONS(690),
    [anon_sym_true] = ACTIONS(694),
    [anon_sym_false] = ACTIONS(694),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(674),
    [sym_super] = ACTIONS(674),
    [sym_crate] = ACTIONS(674),
    [sym__raw_string_literal_start] = ACTIONS(696),
    [sym_float_literal] = ACTIONS(690),
  },
  [111] = {
    [sym_delim_token_tree] = STATE(198),
    [sym__delim_tokens] = STATE(206),
    [sym__non_delim_token] = STATE(198),
    [sym__literal] = STATE(214),
    [sym_string_literal] = STATE(199),
    [sym_raw_string_literal] = STATE(199),
    [sym_boolean_literal] = STATE(199),
    [sym_line_comment] = STATE(111),
    [sym_block_comment] = STATE(111),
    [aux_sym__non_special_token_repeat1] = STATE(157),
    [aux_sym_delim_token_tree_repeat1] = STATE(83),
    [sym_identifier] = ACTIONS(674),
    [anon_sym_SEMI] = ACTIONS(676),
    [anon_sym_LPAREN] = ACTIONS(678),
    [anon_sym_LBRACK] = ACTIONS(680),
    [anon_sym_RBRACK] = ACTIONS(732),
    [anon_sym_LBRACE] = ACTIONS(682),
    [anon_sym_EQ_GT] = ACTIONS(676),
    [anon_sym_COLON] = ACTIONS(686),
    [anon_sym_DOLLAR] = ACTIONS(688),
    [anon_sym_PLUS] = ACTIONS(686),
    [anon_sym_STAR] = ACTIONS(686),
    [anon_sym_QMARK] = ACTIONS(676),
    [anon_sym_u8] = ACTIONS(674),
    [anon_sym_i8] = ACTIONS(674),
    [anon_sym_u16] = ACTIONS(674),
    [anon_sym_i16] = ACTIONS(674),
    [anon_sym_u32] = ACTIONS(674),
    [anon_sym_i32] = ACTIONS(674),
    [anon_sym_u64] = ACTIONS(674),
    [anon_sym_i64] = ACTIONS(674),
    [anon_sym_u128] = ACTIONS(674),
    [anon_sym_i128] = ACTIONS(674),
    [anon_sym_isize] = ACTIONS(674),
    [anon_sym_usize] = ACTIONS(674),
    [anon_sym_f32] = ACTIONS(674),
    [anon_sym_f64] = ACTIONS(674),
    [anon_sym_bool] = ACTIONS(674),
    [anon_sym_str] = ACTIONS(674),
    [anon_sym_char] = ACTIONS(674),
    [anon_sym_DASH] = ACTIONS(686),
    [anon_sym_SLASH] = ACTIONS(686),
    [anon_sym_PERCENT] = ACTIONS(686),
    [anon_sym_CARET] = ACTIONS(686),
    [anon_sym_BANG] = ACTIONS(686),
    [anon_sym_AMP] = ACTIONS(686),
    [anon_sym_PIPE] = ACTIONS(686),
    [anon_sym_AMP_AMP] = ACTIONS(676),
    [anon_sym_PIPE_PIPE] = ACTIONS(676),
    [anon_sym_LT_LT] = ACTIONS(686),
    [anon_sym_GT_GT] = ACTIONS(686),
    [anon_sym_PLUS_EQ] = ACTIONS(676),
    [anon_sym_DASH_EQ] = ACTIONS(676),
    [anon_sym_STAR_EQ] = ACTIONS(676),
    [anon_sym_SLASH_EQ] = ACTIONS(676),
    [anon_sym_PERCENT_EQ] = ACTIONS(676),
    [anon_sym_CARET_EQ] = ACTIONS(676),
    [anon_sym_AMP_EQ] = ACTIONS(676),
    [anon_sym_PIPE_EQ] = ACTIONS(676),
    [anon_sym_LT_LT_EQ] = ACTIONS(676),
    [anon_sym_GT_GT_EQ] = ACTIONS(676),
    [anon_sym_EQ] = ACTIONS(686),
    [anon_sym_EQ_EQ] = ACTIONS(676),
    [anon_sym_BANG_EQ] = ACTIONS(676),
    [anon_sym_GT] = ACTIONS(686),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT_EQ] = ACTIONS(676),
    [anon_sym_LT_EQ] = ACTIONS(676),
    [anon_sym_AT] = ACTIONS(676),
    [anon_sym__] = ACTIONS(686),
    [anon_sym_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(676),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(676),
    [anon_sym_COMMA] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(676),
    [anon_sym_DASH_GT] = ACTIONS(676),
    [anon_sym_POUND] = ACTIONS(676),
    [anon_sym_SQUOTE] = ACTIONS(674),
    [anon_sym_as] = ACTIONS(674),
    [anon_sym_async] = ACTIONS(674),
    [anon_sym_await] = ACTIONS(674),
    [anon_sym_break] = ACTIONS(674),
    [anon_sym_const] = ACTIONS(674),
    [anon_sym_continue] = ACTIONS(674),
    [anon_sym_default] = ACTIONS(674),
    [anon_sym_enum] = ACTIONS(674),
    [anon_sym_fn] = ACTIONS(674),
    [anon_sym_for] = ACTIONS(674),
    [anon_sym_gen] = ACTIONS(674),
    [anon_sym_if] = ACTIONS(674),
    [anon_sym_impl] = ACTIONS(674),
    [anon_sym_let] = ACTIONS(674),
    [anon_sym_loop] = ACTIONS(674),
    [anon_sym_match] = ACTIONS(674),
    [anon_sym_mod] = ACTIONS(674),
    [anon_sym_pub] = ACTIONS(674),
    [anon_sym_return] = ACTIONS(674),
    [anon_sym_static] = ACTIONS(674),
    [anon_sym_struct] = ACTIONS(674),
    [anon_sym_trait] = ACTIONS(674),
    [anon_sym_type] = ACTIONS(674),
    [anon_sym_union] = ACTIONS(674),
    [anon_sym_unsafe] = ACTIONS(674),
    [anon_sym_use] = ACTIONS(674),
    [anon_sym_where] = ACTIONS(674),
    [anon_sym_while] = ACTIONS(674),
    [sym_mutable_specifier] = ACTIONS(674),
    [sym_integer_literal] = ACTIONS(690),
    [aux_sym_string_literal_token1] = ACTIONS(692),
    [sym_char_literal] = ACTIONS(690),
    [anon_sym_true] = ACTIONS(694),
    [anon_sym_false] = ACTIONS(694),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(674),
    [sym_super] = ACTIONS(674),
    [sym_crate] = ACTIONS(674),
    [sym__raw_string_literal_start] = ACTIONS(696),
    [sym_float_literal] = ACTIONS(690),
  },
  [112] = {
    [sym_delim_token_tree] = STATE(198),
    [sym__delim_tokens] = STATE(206),
    [sym__non_delim_token] = STATE(198),
    [sym__literal] = STATE(214),
    [sym_string_literal] = STATE(199),
    [sym_raw_string_literal] = STATE(199),
    [sym_boolean_literal] = STATE(199),
    [sym_line_comment] = STATE(112),
    [sym_block_comment] = STATE(112),
    [aux_sym__non_special_token_repeat1] = STATE(157),
    [aux_sym_delim_token_tree_repeat1] = STATE(83),
    [sym_identifier] = ACTIONS(674),
    [anon_sym_SEMI] = ACTIONS(676),
    [anon_sym_LPAREN] = ACTIONS(678),
    [anon_sym_LBRACK] = ACTIONS(680),
    [anon_sym_LBRACE] = ACTIONS(682),
    [anon_sym_RBRACE] = ACTIONS(732),
    [anon_sym_EQ_GT] = ACTIONS(676),
    [anon_sym_COLON] = ACTIONS(686),
    [anon_sym_DOLLAR] = ACTIONS(688),
    [anon_sym_PLUS] = ACTIONS(686),
    [anon_sym_STAR] = ACTIONS(686),
    [anon_sym_QMARK] = ACTIONS(676),
    [anon_sym_u8] = ACTIONS(674),
    [anon_sym_i8] = ACTIONS(674),
    [anon_sym_u16] = ACTIONS(674),
    [anon_sym_i16] = ACTIONS(674),
    [anon_sym_u32] = ACTIONS(674),
    [anon_sym_i32] = ACTIONS(674),
    [anon_sym_u64] = ACTIONS(674),
    [anon_sym_i64] = ACTIONS(674),
    [anon_sym_u128] = ACTIONS(674),
    [anon_sym_i128] = ACTIONS(674),
    [anon_sym_isize] = ACTIONS(674),
    [anon_sym_usize] = ACTIONS(674),
    [anon_sym_f32] = ACTIONS(674),
    [anon_sym_f64] = ACTIONS(674),
    [anon_sym_bool] = ACTIONS(674),
    [anon_sym_str] = ACTIONS(674),
    [anon_sym_char] = ACTIONS(674),
    [anon_sym_DASH] = ACTIONS(686),
    [anon_sym_SLASH] = ACTIONS(686),
    [anon_sym_PERCENT] = ACTIONS(686),
    [anon_sym_CARET] = ACTIONS(686),
    [anon_sym_BANG] = ACTIONS(686),
    [anon_sym_AMP] = ACTIONS(686),
    [anon_sym_PIPE] = ACTIONS(686),
    [anon_sym_AMP_AMP] = ACTIONS(676),
    [anon_sym_PIPE_PIPE] = ACTIONS(676),
    [anon_sym_LT_LT] = ACTIONS(686),
    [anon_sym_GT_GT] = ACTIONS(686),
    [anon_sym_PLUS_EQ] = ACTIONS(676),
    [anon_sym_DASH_EQ] = ACTIONS(676),
    [anon_sym_STAR_EQ] = ACTIONS(676),
    [anon_sym_SLASH_EQ] = ACTIONS(676),
    [anon_sym_PERCENT_EQ] = ACTIONS(676),
    [anon_sym_CARET_EQ] = ACTIONS(676),
    [anon_sym_AMP_EQ] = ACTIONS(676),
    [anon_sym_PIPE_EQ] = ACTIONS(676),
    [anon_sym_LT_LT_EQ] = ACTIONS(676),
    [anon_sym_GT_GT_EQ] = ACTIONS(676),
    [anon_sym_EQ] = ACTIONS(686),
    [anon_sym_EQ_EQ] = ACTIONS(676),
    [anon_sym_BANG_EQ] = ACTIONS(676),
    [anon_sym_GT] = ACTIONS(686),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT_EQ] = ACTIONS(676),
    [anon_sym_LT_EQ] = ACTIONS(676),
    [anon_sym_AT] = ACTIONS(676),
    [anon_sym__] = ACTIONS(686),
    [anon_sym_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(676),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(676),
    [anon_sym_COMMA] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(676),
    [anon_sym_DASH_GT] = ACTIONS(676),
    [anon_sym_POUND] = ACTIONS(676),
    [anon_sym_SQUOTE] = ACTIONS(674),
    [anon_sym_as] = ACTIONS(674),
    [anon_sym_async] = ACTIONS(674),
    [anon_sym_await] = ACTIONS(674),
    [anon_sym_break] = ACTIONS(674),
    [anon_sym_const] = ACTIONS(674),
    [anon_sym_continue] = ACTIONS(674),
    [anon_sym_default] = ACTIONS(674),
    [anon_sym_enum] = ACTIONS(674),
    [anon_sym_fn] = ACTIONS(674),
    [anon_sym_for] = ACTIONS(674),
    [anon_sym_gen] = ACTIONS(674),
    [anon_sym_if] = ACTIONS(674),
    [anon_sym_impl] = ACTIONS(674),
    [anon_sym_let] = ACTIONS(674),
    [anon_sym_loop] = ACTIONS(674),
    [anon_sym_match] = ACTIONS(674),
    [anon_sym_mod] = ACTIONS(674),
    [anon_sym_pub] = ACTIONS(674),
    [anon_sym_return] = ACTIONS(674),
    [anon_sym_static] = ACTIONS(674),
    [anon_sym_struct] = ACTIONS(674),
    [anon_sym_trait] = ACTIONS(674),
    [anon_sym_type] = ACTIONS(674),
    [anon_sym_union] = ACTIONS(674),
    [anon_sym_unsafe] = ACTIONS(674),
    [anon_sym_use] = ACTIONS(674),
    [anon_sym_where] = ACTIONS(674),
    [anon_sym_while] = ACTIONS(674),
    [sym_mutable_specifier] = ACTIONS(674),
    [sym_integer_literal] = ACTIONS(690),
    [aux_sym_string_literal_token1] = ACTIONS(692),
    [sym_char_literal] = ACTIONS(690),
    [anon_sym_true] = ACTIONS(694),
    [anon_sym_false] = ACTIONS(694),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(674),
    [sym_super] = ACTIONS(674),
    [sym_crate] = ACTIONS(674),
    [sym__raw_string_literal_start] = ACTIONS(696),
    [sym_float_literal] = ACTIONS(690),
  },
  [113] = {
    [sym_delim_token_tree] = STATE(198),
    [sym__delim_tokens] = STATE(206),
    [sym__non_delim_token] = STATE(198),
    [sym__literal] = STATE(214),
    [sym_string_literal] = STATE(199),
    [sym_raw_string_literal] = STATE(199),
    [sym_boolean_literal] = STATE(199),
    [sym_line_comment] = STATE(113),
    [sym_block_comment] = STATE(113),
    [aux_sym__non_special_token_repeat1] = STATE(157),
    [aux_sym_delim_token_tree_repeat1] = STATE(120),
    [sym_identifier] = ACTIONS(674),
    [anon_sym_SEMI] = ACTIONS(676),
    [anon_sym_LPAREN] = ACTIONS(678),
    [anon_sym_RPAREN] = ACTIONS(726),
    [anon_sym_LBRACK] = ACTIONS(680),
    [anon_sym_LBRACE] = ACTIONS(682),
    [anon_sym_EQ_GT] = ACTIONS(676),
    [anon_sym_COLON] = ACTIONS(686),
    [anon_sym_DOLLAR] = ACTIONS(688),
    [anon_sym_PLUS] = ACTIONS(686),
    [anon_sym_STAR] = ACTIONS(686),
    [anon_sym_QMARK] = ACTIONS(676),
    [anon_sym_u8] = ACTIONS(674),
    [anon_sym_i8] = ACTIONS(674),
    [anon_sym_u16] = ACTIONS(674),
    [anon_sym_i16] = ACTIONS(674),
    [anon_sym_u32] = ACTIONS(674),
    [anon_sym_i32] = ACTIONS(674),
    [anon_sym_u64] = ACTIONS(674),
    [anon_sym_i64] = ACTIONS(674),
    [anon_sym_u128] = ACTIONS(674),
    [anon_sym_i128] = ACTIONS(674),
    [anon_sym_isize] = ACTIONS(674),
    [anon_sym_usize] = ACTIONS(674),
    [anon_sym_f32] = ACTIONS(674),
    [anon_sym_f64] = ACTIONS(674),
    [anon_sym_bool] = ACTIONS(674),
    [anon_sym_str] = ACTIONS(674),
    [anon_sym_char] = ACTIONS(674),
    [anon_sym_DASH] = ACTIONS(686),
    [anon_sym_SLASH] = ACTIONS(686),
    [anon_sym_PERCENT] = ACTIONS(686),
    [anon_sym_CARET] = ACTIONS(686),
    [anon_sym_BANG] = ACTIONS(686),
    [anon_sym_AMP] = ACTIONS(686),
    [anon_sym_PIPE] = ACTIONS(686),
    [anon_sym_AMP_AMP] = ACTIONS(676),
    [anon_sym_PIPE_PIPE] = ACTIONS(676),
    [anon_sym_LT_LT] = ACTIONS(686),
    [anon_sym_GT_GT] = ACTIONS(686),
    [anon_sym_PLUS_EQ] = ACTIONS(676),
    [anon_sym_DASH_EQ] = ACTIONS(676),
    [anon_sym_STAR_EQ] = ACTIONS(676),
    [anon_sym_SLASH_EQ] = ACTIONS(676),
    [anon_sym_PERCENT_EQ] = ACTIONS(676),
    [anon_sym_CARET_EQ] = ACTIONS(676),
    [anon_sym_AMP_EQ] = ACTIONS(676),
    [anon_sym_PIPE_EQ] = ACTIONS(676),
    [anon_sym_LT_LT_EQ] = ACTIONS(676),
    [anon_sym_GT_GT_EQ] = ACTIONS(676),
    [anon_sym_EQ] = ACTIONS(686),
    [anon_sym_EQ_EQ] = ACTIONS(676),
    [anon_sym_BANG_EQ] = ACTIONS(676),
    [anon_sym_GT] = ACTIONS(686),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT_EQ] = ACTIONS(676),
    [anon_sym_LT_EQ] = ACTIONS(676),
    [anon_sym_AT] = ACTIONS(676),
    [anon_sym__] = ACTIONS(686),
    [anon_sym_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(676),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(676),
    [anon_sym_COMMA] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(676),
    [anon_sym_DASH_GT] = ACTIONS(676),
    [anon_sym_POUND] = ACTIONS(676),
    [anon_sym_SQUOTE] = ACTIONS(674),
    [anon_sym_as] = ACTIONS(674),
    [anon_sym_async] = ACTIONS(674),
    [anon_sym_await] = ACTIONS(674),
    [anon_sym_break] = ACTIONS(674),
    [anon_sym_const] = ACTIONS(674),
    [anon_sym_continue] = ACTIONS(674),
    [anon_sym_default] = ACTIONS(674),
    [anon_sym_enum] = ACTIONS(674),
    [anon_sym_fn] = ACTIONS(674),
    [anon_sym_for] = ACTIONS(674),
    [anon_sym_gen] = ACTIONS(674),
    [anon_sym_if] = ACTIONS(674),
    [anon_sym_impl] = ACTIONS(674),
    [anon_sym_let] = ACTIONS(674),
    [anon_sym_loop] = ACTIONS(674),
    [anon_sym_match] = ACTIONS(674),
    [anon_sym_mod] = ACTIONS(674),
    [anon_sym_pub] = ACTIONS(674),
    [anon_sym_return] = ACTIONS(674),
    [anon_sym_static] = ACTIONS(674),
    [anon_sym_struct] = ACTIONS(674),
    [anon_sym_trait] = ACTIONS(674),
    [anon_sym_type] = ACTIONS(674),
    [anon_sym_union] = ACTIONS(674),
    [anon_sym_unsafe] = ACTIONS(674),
    [anon_sym_use] = ACTIONS(674),
    [anon_sym_where] = ACTIONS(674),
    [anon_sym_while] = ACTIONS(674),
    [sym_mutable_specifier] = ACTIONS(674),
    [sym_integer_literal] = ACTIONS(690),
    [aux_sym_string_literal_token1] = ACTIONS(692),
    [sym_char_literal] = ACTIONS(690),
    [anon_sym_true] = ACTIONS(694),
    [anon_sym_false] = ACTIONS(694),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(674),
    [sym_super] = ACTIONS(674),
    [sym_crate] = ACTIONS(674),
    [sym__raw_string_literal_start] = ACTIONS(696),
    [sym_float_literal] = ACTIONS(690),
  },
  [114] = {
    [sym_delim_token_tree] = STATE(198),
    [sym__delim_tokens] = STATE(206),
    [sym__non_delim_token] = STATE(198),
    [sym__literal] = STATE(214),
    [sym_string_literal] = STATE(199),
    [sym_raw_string_literal] = STATE(199),
    [sym_boolean_literal] = STATE(199),
    [sym_line_comment] = STATE(114),
    [sym_block_comment] = STATE(114),
    [aux_sym__non_special_token_repeat1] = STATE(157),
    [aux_sym_delim_token_tree_repeat1] = STATE(117),
    [sym_identifier] = ACTIONS(674),
    [anon_sym_SEMI] = ACTIONS(676),
    [anon_sym_LPAREN] = ACTIONS(678),
    [anon_sym_RPAREN] = ACTIONS(734),
    [anon_sym_LBRACK] = ACTIONS(680),
    [anon_sym_LBRACE] = ACTIONS(682),
    [anon_sym_EQ_GT] = ACTIONS(676),
    [anon_sym_COLON] = ACTIONS(686),
    [anon_sym_DOLLAR] = ACTIONS(688),
    [anon_sym_PLUS] = ACTIONS(686),
    [anon_sym_STAR] = ACTIONS(686),
    [anon_sym_QMARK] = ACTIONS(676),
    [anon_sym_u8] = ACTIONS(674),
    [anon_sym_i8] = ACTIONS(674),
    [anon_sym_u16] = ACTIONS(674),
    [anon_sym_i16] = ACTIONS(674),
    [anon_sym_u32] = ACTIONS(674),
    [anon_sym_i32] = ACTIONS(674),
    [anon_sym_u64] = ACTIONS(674),
    [anon_sym_i64] = ACTIONS(674),
    [anon_sym_u128] = ACTIONS(674),
    [anon_sym_i128] = ACTIONS(674),
    [anon_sym_isize] = ACTIONS(674),
    [anon_sym_usize] = ACTIONS(674),
    [anon_sym_f32] = ACTIONS(674),
    [anon_sym_f64] = ACTIONS(674),
    [anon_sym_bool] = ACTIONS(674),
    [anon_sym_str] = ACTIONS(674),
    [anon_sym_char] = ACTIONS(674),
    [anon_sym_DASH] = ACTIONS(686),
    [anon_sym_SLASH] = ACTIONS(686),
    [anon_sym_PERCENT] = ACTIONS(686),
    [anon_sym_CARET] = ACTIONS(686),
    [anon_sym_BANG] = ACTIONS(686),
    [anon_sym_AMP] = ACTIONS(686),
    [anon_sym_PIPE] = ACTIONS(686),
    [anon_sym_AMP_AMP] = ACTIONS(676),
    [anon_sym_PIPE_PIPE] = ACTIONS(676),
    [anon_sym_LT_LT] = ACTIONS(686),
    [anon_sym_GT_GT] = ACTIONS(686),
    [anon_sym_PLUS_EQ] = ACTIONS(676),
    [anon_sym_DASH_EQ] = ACTIONS(676),
    [anon_sym_STAR_EQ] = ACTIONS(676),
    [anon_sym_SLASH_EQ] = ACTIONS(676),
    [anon_sym_PERCENT_EQ] = ACTIONS(676),
    [anon_sym_CARET_EQ] = ACTIONS(676),
    [anon_sym_AMP_EQ] = ACTIONS(676),
    [anon_sym_PIPE_EQ] = ACTIONS(676),
    [anon_sym_LT_LT_EQ] = ACTIONS(676),
    [anon_sym_GT_GT_EQ] = ACTIONS(676),
    [anon_sym_EQ] = ACTIONS(686),
    [anon_sym_EQ_EQ] = ACTIONS(676),
    [anon_sym_BANG_EQ] = ACTIONS(676),
    [anon_sym_GT] = ACTIONS(686),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT_EQ] = ACTIONS(676),
    [anon_sym_LT_EQ] = ACTIONS(676),
    [anon_sym_AT] = ACTIONS(676),
    [anon_sym__] = ACTIONS(686),
    [anon_sym_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(676),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(676),
    [anon_sym_COMMA] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(676),
    [anon_sym_DASH_GT] = ACTIONS(676),
    [anon_sym_POUND] = ACTIONS(676),
    [anon_sym_SQUOTE] = ACTIONS(674),
    [anon_sym_as] = ACTIONS(674),
    [anon_sym_async] = ACTIONS(674),
    [anon_sym_await] = ACTIONS(674),
    [anon_sym_break] = ACTIONS(674),
    [anon_sym_const] = ACTIONS(674),
    [anon_sym_continue] = ACTIONS(674),
    [anon_sym_default] = ACTIONS(674),
    [anon_sym_enum] = ACTIONS(674),
    [anon_sym_fn] = ACTIONS(674),
    [anon_sym_for] = ACTIONS(674),
    [anon_sym_gen] = ACTIONS(674),
    [anon_sym_if] = ACTIONS(674),
    [anon_sym_impl] = ACTIONS(674),
    [anon_sym_let] = ACTIONS(674),
    [anon_sym_loop] = ACTIONS(674),
    [anon_sym_match] = ACTIONS(674),
    [anon_sym_mod] = ACTIONS(674),
    [anon_sym_pub] = ACTIONS(674),
    [anon_sym_return] = ACTIONS(674),
    [anon_sym_static] = ACTIONS(674),
    [anon_sym_struct] = ACTIONS(674),
    [anon_sym_trait] = ACTIONS(674),
    [anon_sym_type] = ACTIONS(674),
    [anon_sym_union] = ACTIONS(674),
    [anon_sym_unsafe] = ACTIONS(674),
    [anon_sym_use] = ACTIONS(674),
    [anon_sym_where] = ACTIONS(674),
    [anon_sym_while] = ACTIONS(674),
    [sym_mutable_specifier] = ACTIONS(674),
    [sym_integer_literal] = ACTIONS(690),
    [aux_sym_string_literal_token1] = ACTIONS(692),
    [sym_char_literal] = ACTIONS(690),
    [anon_sym_true] = ACTIONS(694),
    [anon_sym_false] = ACTIONS(694),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(674),
    [sym_super] = ACTIONS(674),
    [sym_crate] = ACTIONS(674),
    [sym__raw_string_literal_start] = ACTIONS(696),
    [sym_float_literal] = ACTIONS(690),
  },
  [115] = {
    [sym_delim_token_tree] = STATE(198),
    [sym__delim_tokens] = STATE(206),
    [sym__non_delim_token] = STATE(198),
    [sym__literal] = STATE(214),
    [sym_string_literal] = STATE(199),
    [sym_raw_string_literal] = STATE(199),
    [sym_boolean_literal] = STATE(199),
    [sym_line_comment] = STATE(115),
    [sym_block_comment] = STATE(115),
    [aux_sym__non_special_token_repeat1] = STATE(157),
    [aux_sym_delim_token_tree_repeat1] = STATE(118),
    [sym_identifier] = ACTIONS(674),
    [anon_sym_SEMI] = ACTIONS(676),
    [anon_sym_LPAREN] = ACTIONS(678),
    [anon_sym_LBRACK] = ACTIONS(680),
    [anon_sym_RBRACK] = ACTIONS(734),
    [anon_sym_LBRACE] = ACTIONS(682),
    [anon_sym_EQ_GT] = ACTIONS(676),
    [anon_sym_COLON] = ACTIONS(686),
    [anon_sym_DOLLAR] = ACTIONS(688),
    [anon_sym_PLUS] = ACTIONS(686),
    [anon_sym_STAR] = ACTIONS(686),
    [anon_sym_QMARK] = ACTIONS(676),
    [anon_sym_u8] = ACTIONS(674),
    [anon_sym_i8] = ACTIONS(674),
    [anon_sym_u16] = ACTIONS(674),
    [anon_sym_i16] = ACTIONS(674),
    [anon_sym_u32] = ACTIONS(674),
    [anon_sym_i32] = ACTIONS(674),
    [anon_sym_u64] = ACTIONS(674),
    [anon_sym_i64] = ACTIONS(674),
    [anon_sym_u128] = ACTIONS(674),
    [anon_sym_i128] = ACTIONS(674),
    [anon_sym_isize] = ACTIONS(674),
    [anon_sym_usize] = ACTIONS(674),
    [anon_sym_f32] = ACTIONS(674),
    [anon_sym_f64] = ACTIONS(674),
    [anon_sym_bool] = ACTIONS(674),
    [anon_sym_str] = ACTIONS(674),
    [anon_sym_char] = ACTIONS(674),
    [anon_sym_DASH] = ACTIONS(686),
    [anon_sym_SLASH] = ACTIONS(686),
    [anon_sym_PERCENT] = ACTIONS(686),
    [anon_sym_CARET] = ACTIONS(686),
    [anon_sym_BANG] = ACTIONS(686),
    [anon_sym_AMP] = ACTIONS(686),
    [anon_sym_PIPE] = ACTIONS(686),
    [anon_sym_AMP_AMP] = ACTIONS(676),
    [anon_sym_PIPE_PIPE] = ACTIONS(676),
    [anon_sym_LT_LT] = ACTIONS(686),
    [anon_sym_GT_GT] = ACTIONS(686),
    [anon_sym_PLUS_EQ] = ACTIONS(676),
    [anon_sym_DASH_EQ] = ACTIONS(676),
    [anon_sym_STAR_EQ] = ACTIONS(676),
    [anon_sym_SLASH_EQ] = ACTIONS(676),
    [anon_sym_PERCENT_EQ] = ACTIONS(676),
    [anon_sym_CARET_EQ] = ACTIONS(676),
    [anon_sym_AMP_EQ] = ACTIONS(676),
    [anon_sym_PIPE_EQ] = ACTIONS(676),
    [anon_sym_LT_LT_EQ] = ACTIONS(676),
    [anon_sym_GT_GT_EQ] = ACTIONS(676),
    [anon_sym_EQ] = ACTIONS(686),
    [anon_sym_EQ_EQ] = ACTIONS(676),
    [anon_sym_BANG_EQ] = ACTIONS(676),
    [anon_sym_GT] = ACTIONS(686),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT_EQ] = ACTIONS(676),
    [anon_sym_LT_EQ] = ACTIONS(676),
    [anon_sym_AT] = ACTIONS(676),
    [anon_sym__] = ACTIONS(686),
    [anon_sym_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(676),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(676),
    [anon_sym_COMMA] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(676),
    [anon_sym_DASH_GT] = ACTIONS(676),
    [anon_sym_POUND] = ACTIONS(676),
    [anon_sym_SQUOTE] = ACTIONS(674),
    [anon_sym_as] = ACTIONS(674),
    [anon_sym_async] = ACTIONS(674),
    [anon_sym_await] = ACTIONS(674),
    [anon_sym_break] = ACTIONS(674),
    [anon_sym_const] = ACTIONS(674),
    [anon_sym_continue] = ACTIONS(674),
    [anon_sym_default] = ACTIONS(674),
    [anon_sym_enum] = ACTIONS(674),
    [anon_sym_fn] = ACTIONS(674),
    [anon_sym_for] = ACTIONS(674),
    [anon_sym_gen] = ACTIONS(674),
    [anon_sym_if] = ACTIONS(674),
    [anon_sym_impl] = ACTIONS(674),
    [anon_sym_let] = ACTIONS(674),
    [anon_sym_loop] = ACTIONS(674),
    [anon_sym_match] = ACTIONS(674),
    [anon_sym_mod] = ACTIONS(674),
    [anon_sym_pub] = ACTIONS(674),
    [anon_sym_return] = ACTIONS(674),
    [anon_sym_static] = ACTIONS(674),
    [anon_sym_struct] = ACTIONS(674),
    [anon_sym_trait] = ACTIONS(674),
    [anon_sym_type] = ACTIONS(674),
    [anon_sym_union] = ACTIONS(674),
    [anon_sym_unsafe] = ACTIONS(674),
    [anon_sym_use] = ACTIONS(674),
    [anon_sym_where] = ACTIONS(674),
    [anon_sym_while] = ACTIONS(674),
    [sym_mutable_specifier] = ACTIONS(674),
    [sym_integer_literal] = ACTIONS(690),
    [aux_sym_string_literal_token1] = ACTIONS(692),
    [sym_char_literal] = ACTIONS(690),
    [anon_sym_true] = ACTIONS(694),
    [anon_sym_false] = ACTIONS(694),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(674),
    [sym_super] = ACTIONS(674),
    [sym_crate] = ACTIONS(674),
    [sym__raw_string_literal_start] = ACTIONS(696),
    [sym_float_literal] = ACTIONS(690),
  },
  [116] = {
    [sym_delim_token_tree] = STATE(198),
    [sym__delim_tokens] = STATE(206),
    [sym__non_delim_token] = STATE(198),
    [sym__literal] = STATE(214),
    [sym_string_literal] = STATE(199),
    [sym_raw_string_literal] = STATE(199),
    [sym_boolean_literal] = STATE(199),
    [sym_line_comment] = STATE(116),
    [sym_block_comment] = STATE(116),
    [aux_sym__non_special_token_repeat1] = STATE(157),
    [aux_sym_delim_token_tree_repeat1] = STATE(119),
    [sym_identifier] = ACTIONS(674),
    [anon_sym_SEMI] = ACTIONS(676),
    [anon_sym_LPAREN] = ACTIONS(678),
    [anon_sym_LBRACK] = ACTIONS(680),
    [anon_sym_LBRACE] = ACTIONS(682),
    [anon_sym_RBRACE] = ACTIONS(734),
    [anon_sym_EQ_GT] = ACTIONS(676),
    [anon_sym_COLON] = ACTIONS(686),
    [anon_sym_DOLLAR] = ACTIONS(688),
    [anon_sym_PLUS] = ACTIONS(686),
    [anon_sym_STAR] = ACTIONS(686),
    [anon_sym_QMARK] = ACTIONS(676),
    [anon_sym_u8] = ACTIONS(674),
    [anon_sym_i8] = ACTIONS(674),
    [anon_sym_u16] = ACTIONS(674),
    [anon_sym_i16] = ACTIONS(674),
    [anon_sym_u32] = ACTIONS(674),
    [anon_sym_i32] = ACTIONS(674),
    [anon_sym_u64] = ACTIONS(674),
    [anon_sym_i64] = ACTIONS(674),
    [anon_sym_u128] = ACTIONS(674),
    [anon_sym_i128] = ACTIONS(674),
    [anon_sym_isize] = ACTIONS(674),
    [anon_sym_usize] = ACTIONS(674),
    [anon_sym_f32] = ACTIONS(674),
    [anon_sym_f64] = ACTIONS(674),
    [anon_sym_bool] = ACTIONS(674),
    [anon_sym_str] = ACTIONS(674),
    [anon_sym_char] = ACTIONS(674),
    [anon_sym_DASH] = ACTIONS(686),
    [anon_sym_SLASH] = ACTIONS(686),
    [anon_sym_PERCENT] = ACTIONS(686),
    [anon_sym_CARET] = ACTIONS(686),
    [anon_sym_BANG] = ACTIONS(686),
    [anon_sym_AMP] = ACTIONS(686),
    [anon_sym_PIPE] = ACTIONS(686),
    [anon_sym_AMP_AMP] = ACTIONS(676),
    [anon_sym_PIPE_PIPE] = ACTIONS(676),
    [anon_sym_LT_LT] = ACTIONS(686),
    [anon_sym_GT_GT] = ACTIONS(686),
    [anon_sym_PLUS_EQ] = ACTIONS(676),
    [anon_sym_DASH_EQ] = ACTIONS(676),
    [anon_sym_STAR_EQ] = ACTIONS(676),
    [anon_sym_SLASH_EQ] = ACTIONS(676),
    [anon_sym_PERCENT_EQ] = ACTIONS(676),
    [anon_sym_CARET_EQ] = ACTIONS(676),
    [anon_sym_AMP_EQ] = ACTIONS(676),
    [anon_sym_PIPE_EQ] = ACTIONS(676),
    [anon_sym_LT_LT_EQ] = ACTIONS(676),
    [anon_sym_GT_GT_EQ] = ACTIONS(676),
    [anon_sym_EQ] = ACTIONS(686),
    [anon_sym_EQ_EQ] = ACTIONS(676),
    [anon_sym_BANG_EQ] = ACTIONS(676),
    [anon_sym_GT] = ACTIONS(686),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT_EQ] = ACTIONS(676),
    [anon_sym_LT_EQ] = ACTIONS(676),
    [anon_sym_AT] = ACTIONS(676),
    [anon_sym__] = ACTIONS(686),
    [anon_sym_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(676),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(676),
    [anon_sym_COMMA] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(676),
    [anon_sym_DASH_GT] = ACTIONS(676),
    [anon_sym_POUND] = ACTIONS(676),
    [anon_sym_SQUOTE] = ACTIONS(674),
    [anon_sym_as] = ACTIONS(674),
    [anon_sym_async] = ACTIONS(674),
    [anon_sym_await] = ACTIONS(674),
    [anon_sym_break] = ACTIONS(674),
    [anon_sym_const] = ACTIONS(674),
    [anon_sym_continue] = ACTIONS(674),
    [anon_sym_default] = ACTIONS(674),
    [anon_sym_enum] = ACTIONS(674),
    [anon_sym_fn] = ACTIONS(674),
    [anon_sym_for] = ACTIONS(674),
    [anon_sym_gen] = ACTIONS(674),
    [anon_sym_if] = ACTIONS(674),
    [anon_sym_impl] = ACTIONS(674),
    [anon_sym_let] = ACTIONS(674),
    [anon_sym_loop] = ACTIONS(674),
    [anon_sym_match] = ACTIONS(674),
    [anon_sym_mod] = ACTIONS(674),
    [anon_sym_pub] = ACTIONS(674),
    [anon_sym_return] = ACTIONS(674),
    [anon_sym_static] = ACTIONS(674),
    [anon_sym_struct] = ACTIONS(674),
    [anon_sym_trait] = ACTIONS(674),
    [anon_sym_type] = ACTIONS(674),
    [anon_sym_union] = ACTIONS(674),
    [anon_sym_unsafe] = ACTIONS(674),
    [anon_sym_use] = ACTIONS(674),
    [anon_sym_where] = ACTIONS(674),
    [anon_sym_while] = ACTIONS(674),
    [sym_mutable_specifier] = ACTIONS(674),
    [sym_integer_literal] = ACTIONS(690),
    [aux_sym_string_literal_token1] = ACTIONS(692),
    [sym_char_literal] = ACTIONS(690),
    [anon_sym_true] = ACTIONS(694),
    [anon_sym_false] = ACTIONS(694),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(674),
    [sym_super] = ACTIONS(674),
    [sym_crate] = ACTIONS(674),
    [sym__raw_string_literal_start] = ACTIONS(696),
    [sym_float_literal] = ACTIONS(690),
  },
  [117] = {
    [sym_delim_token_tree] = STATE(198),
    [sym__delim_tokens] = STATE(206),
    [sym__non_delim_token] = STATE(198),
    [sym__literal] = STATE(214),
    [sym_string_literal] = STATE(199),
    [sym_raw_string_literal] = STATE(199),
    [sym_boolean_literal] = STATE(199),
    [sym_line_comment] = STATE(117),
    [sym_block_comment] = STATE(117),
    [aux_sym__non_special_token_repeat1] = STATE(157),
    [aux_sym_delim_token_tree_repeat1] = STATE(83),
    [sym_identifier] = ACTIONS(674),
    [anon_sym_SEMI] = ACTIONS(676),
    [anon_sym_LPAREN] = ACTIONS(678),
    [anon_sym_RPAREN] = ACTIONS(736),
    [anon_sym_LBRACK] = ACTIONS(680),
    [anon_sym_LBRACE] = ACTIONS(682),
    [anon_sym_EQ_GT] = ACTIONS(676),
    [anon_sym_COLON] = ACTIONS(686),
    [anon_sym_DOLLAR] = ACTIONS(688),
    [anon_sym_PLUS] = ACTIONS(686),
    [anon_sym_STAR] = ACTIONS(686),
    [anon_sym_QMARK] = ACTIONS(676),
    [anon_sym_u8] = ACTIONS(674),
    [anon_sym_i8] = ACTIONS(674),
    [anon_sym_u16] = ACTIONS(674),
    [anon_sym_i16] = ACTIONS(674),
    [anon_sym_u32] = ACTIONS(674),
    [anon_sym_i32] = ACTIONS(674),
    [anon_sym_u64] = ACTIONS(674),
    [anon_sym_i64] = ACTIONS(674),
    [anon_sym_u128] = ACTIONS(674),
    [anon_sym_i128] = ACTIONS(674),
    [anon_sym_isize] = ACTIONS(674),
    [anon_sym_usize] = ACTIONS(674),
    [anon_sym_f32] = ACTIONS(674),
    [anon_sym_f64] = ACTIONS(674),
    [anon_sym_bool] = ACTIONS(674),
    [anon_sym_str] = ACTIONS(674),
    [anon_sym_char] = ACTIONS(674),
    [anon_sym_DASH] = ACTIONS(686),
    [anon_sym_SLASH] = ACTIONS(686),
    [anon_sym_PERCENT] = ACTIONS(686),
    [anon_sym_CARET] = ACTIONS(686),
    [anon_sym_BANG] = ACTIONS(686),
    [anon_sym_AMP] = ACTIONS(686),
    [anon_sym_PIPE] = ACTIONS(686),
    [anon_sym_AMP_AMP] = ACTIONS(676),
    [anon_sym_PIPE_PIPE] = ACTIONS(676),
    [anon_sym_LT_LT] = ACTIONS(686),
    [anon_sym_GT_GT] = ACTIONS(686),
    [anon_sym_PLUS_EQ] = ACTIONS(676),
    [anon_sym_DASH_EQ] = ACTIONS(676),
    [anon_sym_STAR_EQ] = ACTIONS(676),
    [anon_sym_SLASH_EQ] = ACTIONS(676),
    [anon_sym_PERCENT_EQ] = ACTIONS(676),
    [anon_sym_CARET_EQ] = ACTIONS(676),
    [anon_sym_AMP_EQ] = ACTIONS(676),
    [anon_sym_PIPE_EQ] = ACTIONS(676),
    [anon_sym_LT_LT_EQ] = ACTIONS(676),
    [anon_sym_GT_GT_EQ] = ACTIONS(676),
    [anon_sym_EQ] = ACTIONS(686),
    [anon_sym_EQ_EQ] = ACTIONS(676),
    [anon_sym_BANG_EQ] = ACTIONS(676),
    [anon_sym_GT] = ACTIONS(686),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT_EQ] = ACTIONS(676),
    [anon_sym_LT_EQ] = ACTIONS(676),
    [anon_sym_AT] = ACTIONS(676),
    [anon_sym__] = ACTIONS(686),
    [anon_sym_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(676),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(676),
    [anon_sym_COMMA] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(676),
    [anon_sym_DASH_GT] = ACTIONS(676),
    [anon_sym_POUND] = ACTIONS(676),
    [anon_sym_SQUOTE] = ACTIONS(674),
    [anon_sym_as] = ACTIONS(674),
    [anon_sym_async] = ACTIONS(674),
    [anon_sym_await] = ACTIONS(674),
    [anon_sym_break] = ACTIONS(674),
    [anon_sym_const] = ACTIONS(674),
    [anon_sym_continue] = ACTIONS(674),
    [anon_sym_default] = ACTIONS(674),
    [anon_sym_enum] = ACTIONS(674),
    [anon_sym_fn] = ACTIONS(674),
    [anon_sym_for] = ACTIONS(674),
    [anon_sym_gen] = ACTIONS(674),
    [anon_sym_if] = ACTIONS(674),
    [anon_sym_impl] = ACTIONS(674),
    [anon_sym_let] = ACTIONS(674),
    [anon_sym_loop] = ACTIONS(674),
    [anon_sym_match] = ACTIONS(674),
    [anon_sym_mod] = ACTIONS(674),
    [anon_sym_pub] = ACTIONS(674),
    [anon_sym_return] = ACTIONS(674),
    [anon_sym_static] = ACTIONS(674),
    [anon_sym_struct] = ACTIONS(674),
    [anon_sym_trait] = ACTIONS(674),
    [anon_sym_type] = ACTIONS(674),
    [anon_sym_union] = ACTIONS(674),
    [anon_sym_unsafe] = ACTIONS(674),
    [anon_sym_use] = ACTIONS(674),
    [anon_sym_where] = ACTIONS(674),
    [anon_sym_while] = ACTIONS(674),
    [sym_mutable_specifier] = ACTIONS(674),
    [sym_integer_literal] = ACTIONS(690),
    [aux_sym_string_literal_token1] = ACTIONS(692),
    [sym_char_literal] = ACTIONS(690),
    [anon_sym_true] = ACTIONS(694),
    [anon_sym_false] = ACTIONS(694),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(674),
    [sym_super] = ACTIONS(674),
    [sym_crate] = ACTIONS(674),
    [sym__raw_string_literal_start] = ACTIONS(696),
    [sym_float_literal] = ACTIONS(690),
  },
  [118] = {
    [sym_delim_token_tree] = STATE(198),
    [sym__delim_tokens] = STATE(206),
    [sym__non_delim_token] = STATE(198),
    [sym__literal] = STATE(214),
    [sym_string_literal] = STATE(199),
    [sym_raw_string_literal] = STATE(199),
    [sym_boolean_literal] = STATE(199),
    [sym_line_comment] = STATE(118),
    [sym_block_comment] = STATE(118),
    [aux_sym__non_special_token_repeat1] = STATE(157),
    [aux_sym_delim_token_tree_repeat1] = STATE(83),
    [sym_identifier] = ACTIONS(674),
    [anon_sym_SEMI] = ACTIONS(676),
    [anon_sym_LPAREN] = ACTIONS(678),
    [anon_sym_LBRACK] = ACTIONS(680),
    [anon_sym_RBRACK] = ACTIONS(736),
    [anon_sym_LBRACE] = ACTIONS(682),
    [anon_sym_EQ_GT] = ACTIONS(676),
    [anon_sym_COLON] = ACTIONS(686),
    [anon_sym_DOLLAR] = ACTIONS(688),
    [anon_sym_PLUS] = ACTIONS(686),
    [anon_sym_STAR] = ACTIONS(686),
    [anon_sym_QMARK] = ACTIONS(676),
    [anon_sym_u8] = ACTIONS(674),
    [anon_sym_i8] = ACTIONS(674),
    [anon_sym_u16] = ACTIONS(674),
    [anon_sym_i16] = ACTIONS(674),
    [anon_sym_u32] = ACTIONS(674),
    [anon_sym_i32] = ACTIONS(674),
    [anon_sym_u64] = ACTIONS(674),
    [anon_sym_i64] = ACTIONS(674),
    [anon_sym_u128] = ACTIONS(674),
    [anon_sym_i128] = ACTIONS(674),
    [anon_sym_isize] = ACTIONS(674),
    [anon_sym_usize] = ACTIONS(674),
    [anon_sym_f32] = ACTIONS(674),
    [anon_sym_f64] = ACTIONS(674),
    [anon_sym_bool] = ACTIONS(674),
    [anon_sym_str] = ACTIONS(674),
    [anon_sym_char] = ACTIONS(674),
    [anon_sym_DASH] = ACTIONS(686),
    [anon_sym_SLASH] = ACTIONS(686),
    [anon_sym_PERCENT] = ACTIONS(686),
    [anon_sym_CARET] = ACTIONS(686),
    [anon_sym_BANG] = ACTIONS(686),
    [anon_sym_AMP] = ACTIONS(686),
    [anon_sym_PIPE] = ACTIONS(686),
    [anon_sym_AMP_AMP] = ACTIONS(676),
    [anon_sym_PIPE_PIPE] = ACTIONS(676),
    [anon_sym_LT_LT] = ACTIONS(686),
    [anon_sym_GT_GT] = ACTIONS(686),
    [anon_sym_PLUS_EQ] = ACTIONS(676),
    [anon_sym_DASH_EQ] = ACTIONS(676),
    [anon_sym_STAR_EQ] = ACTIONS(676),
    [anon_sym_SLASH_EQ] = ACTIONS(676),
    [anon_sym_PERCENT_EQ] = ACTIONS(676),
    [anon_sym_CARET_EQ] = ACTIONS(676),
    [anon_sym_AMP_EQ] = ACTIONS(676),
    [anon_sym_PIPE_EQ] = ACTIONS(676),
    [anon_sym_LT_LT_EQ] = ACTIONS(676),
    [anon_sym_GT_GT_EQ] = ACTIONS(676),
    [anon_sym_EQ] = ACTIONS(686),
    [anon_sym_EQ_EQ] = ACTIONS(676),
    [anon_sym_BANG_EQ] = ACTIONS(676),
    [anon_sym_GT] = ACTIONS(686),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT_EQ] = ACTIONS(676),
    [anon_sym_LT_EQ] = ACTIONS(676),
    [anon_sym_AT] = ACTIONS(676),
    [anon_sym__] = ACTIONS(686),
    [anon_sym_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(676),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(676),
    [anon_sym_COMMA] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(676),
    [anon_sym_DASH_GT] = ACTIONS(676),
    [anon_sym_POUND] = ACTIONS(676),
    [anon_sym_SQUOTE] = ACTIONS(674),
    [anon_sym_as] = ACTIONS(674),
    [anon_sym_async] = ACTIONS(674),
    [anon_sym_await] = ACTIONS(674),
    [anon_sym_break] = ACTIONS(674),
    [anon_sym_const] = ACTIONS(674),
    [anon_sym_continue] = ACTIONS(674),
    [anon_sym_default] = ACTIONS(674),
    [anon_sym_enum] = ACTIONS(674),
    [anon_sym_fn] = ACTIONS(674),
    [anon_sym_for] = ACTIONS(674),
    [anon_sym_gen] = ACTIONS(674),
    [anon_sym_if] = ACTIONS(674),
    [anon_sym_impl] = ACTIONS(674),
    [anon_sym_let] = ACTIONS(674),
    [anon_sym_loop] = ACTIONS(674),
    [anon_sym_match] = ACTIONS(674),
    [anon_sym_mod] = ACTIONS(674),
    [anon_sym_pub] = ACTIONS(674),
    [anon_sym_return] = ACTIONS(674),
    [anon_sym_static] = ACTIONS(674),
    [anon_sym_struct] = ACTIONS(674),
    [anon_sym_trait] = ACTIONS(674),
    [anon_sym_type] = ACTIONS(674),
    [anon_sym_union] = ACTIONS(674),
    [anon_sym_unsafe] = ACTIONS(674),
    [anon_sym_use] = ACTIONS(674),
    [anon_sym_where] = ACTIONS(674),
    [anon_sym_while] = ACTIONS(674),
    [sym_mutable_specifier] = ACTIONS(674),
    [sym_integer_literal] = ACTIONS(690),
    [aux_sym_string_literal_token1] = ACTIONS(692),
    [sym_char_literal] = ACTIONS(690),
    [anon_sym_true] = ACTIONS(694),
    [anon_sym_false] = ACTIONS(694),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(674),
    [sym_super] = ACTIONS(674),
    [sym_crate] = ACTIONS(674),
    [sym__raw_string_literal_start] = ACTIONS(696),
    [sym_float_literal] = ACTIONS(690),
  },
  [119] = {
    [sym_delim_token_tree] = STATE(198),
    [sym__delim_tokens] = STATE(206),
    [sym__non_delim_token] = STATE(198),
    [sym__literal] = STATE(214),
    [sym_string_literal] = STATE(199),
    [sym_raw_string_literal] = STATE(199),
    [sym_boolean_literal] = STATE(199),
    [sym_line_comment] = STATE(119),
    [sym_block_comment] = STATE(119),
    [aux_sym__non_special_token_repeat1] = STATE(157),
    [aux_sym_delim_token_tree_repeat1] = STATE(83),
    [sym_identifier] = ACTIONS(674),
    [anon_sym_SEMI] = ACTIONS(676),
    [anon_sym_LPAREN] = ACTIONS(678),
    [anon_sym_LBRACK] = ACTIONS(680),
    [anon_sym_LBRACE] = ACTIONS(682),
    [anon_sym_RBRACE] = ACTIONS(736),
    [anon_sym_EQ_GT] = ACTIONS(676),
    [anon_sym_COLON] = ACTIONS(686),
    [anon_sym_DOLLAR] = ACTIONS(688),
    [anon_sym_PLUS] = ACTIONS(686),
    [anon_sym_STAR] = ACTIONS(686),
    [anon_sym_QMARK] = ACTIONS(676),
    [anon_sym_u8] = ACTIONS(674),
    [anon_sym_i8] = ACTIONS(674),
    [anon_sym_u16] = ACTIONS(674),
    [anon_sym_i16] = ACTIONS(674),
    [anon_sym_u32] = ACTIONS(674),
    [anon_sym_i32] = ACTIONS(674),
    [anon_sym_u64] = ACTIONS(674),
    [anon_sym_i64] = ACTIONS(674),
    [anon_sym_u128] = ACTIONS(674),
    [anon_sym_i128] = ACTIONS(674),
    [anon_sym_isize] = ACTIONS(674),
    [anon_sym_usize] = ACTIONS(674),
    [anon_sym_f32] = ACTIONS(674),
    [anon_sym_f64] = ACTIONS(674),
    [anon_sym_bool] = ACTIONS(674),
    [anon_sym_str] = ACTIONS(674),
    [anon_sym_char] = ACTIONS(674),
    [anon_sym_DASH] = ACTIONS(686),
    [anon_sym_SLASH] = ACTIONS(686),
    [anon_sym_PERCENT] = ACTIONS(686),
    [anon_sym_CARET] = ACTIONS(686),
    [anon_sym_BANG] = ACTIONS(686),
    [anon_sym_AMP] = ACTIONS(686),
    [anon_sym_PIPE] = ACTIONS(686),
    [anon_sym_AMP_AMP] = ACTIONS(676),
    [anon_sym_PIPE_PIPE] = ACTIONS(676),
    [anon_sym_LT_LT] = ACTIONS(686),
    [anon_sym_GT_GT] = ACTIONS(686),
    [anon_sym_PLUS_EQ] = ACTIONS(676),
    [anon_sym_DASH_EQ] = ACTIONS(676),
    [anon_sym_STAR_EQ] = ACTIONS(676),
    [anon_sym_SLASH_EQ] = ACTIONS(676),
    [anon_sym_PERCENT_EQ] = ACTIONS(676),
    [anon_sym_CARET_EQ] = ACTIONS(676),
    [anon_sym_AMP_EQ] = ACTIONS(676),
    [anon_sym_PIPE_EQ] = ACTIONS(676),
    [anon_sym_LT_LT_EQ] = ACTIONS(676),
    [anon_sym_GT_GT_EQ] = ACTIONS(676),
    [anon_sym_EQ] = ACTIONS(686),
    [anon_sym_EQ_EQ] = ACTIONS(676),
    [anon_sym_BANG_EQ] = ACTIONS(676),
    [anon_sym_GT] = ACTIONS(686),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT_EQ] = ACTIONS(676),
    [anon_sym_LT_EQ] = ACTIONS(676),
    [anon_sym_AT] = ACTIONS(676),
    [anon_sym__] = ACTIONS(686),
    [anon_sym_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(676),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(676),
    [anon_sym_COMMA] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(676),
    [anon_sym_DASH_GT] = ACTIONS(676),
    [anon_sym_POUND] = ACTIONS(676),
    [anon_sym_SQUOTE] = ACTIONS(674),
    [anon_sym_as] = ACTIONS(674),
    [anon_sym_async] = ACTIONS(674),
    [anon_sym_await] = ACTIONS(674),
    [anon_sym_break] = ACTIONS(674),
    [anon_sym_const] = ACTIONS(674),
    [anon_sym_continue] = ACTIONS(674),
    [anon_sym_default] = ACTIONS(674),
    [anon_sym_enum] = ACTIONS(674),
    [anon_sym_fn] = ACTIONS(674),
    [anon_sym_for] = ACTIONS(674),
    [anon_sym_gen] = ACTIONS(674),
    [anon_sym_if] = ACTIONS(674),
    [anon_sym_impl] = ACTIONS(674),
    [anon_sym_let] = ACTIONS(674),
    [anon_sym_loop] = ACTIONS(674),
    [anon_sym_match] = ACTIONS(674),
    [anon_sym_mod] = ACTIONS(674),
    [anon_sym_pub] = ACTIONS(674),
    [anon_sym_return] = ACTIONS(674),
    [anon_sym_static] = ACTIONS(674),
    [anon_sym_struct] = ACTIONS(674),
    [anon_sym_trait] = ACTIONS(674),
    [anon_sym_type] = ACTIONS(674),
    [anon_sym_union] = ACTIONS(674),
    [anon_sym_unsafe] = ACTIONS(674),
    [anon_sym_use] = ACTIONS(674),
    [anon_sym_where] = ACTIONS(674),
    [anon_sym_while] = ACTIONS(674),
    [sym_mutable_specifier] = ACTIONS(674),
    [sym_integer_literal] = ACTIONS(690),
    [aux_sym_string_literal_token1] = ACTIONS(692),
    [sym_char_literal] = ACTIONS(690),
    [anon_sym_true] = ACTIONS(694),
    [anon_sym_false] = ACTIONS(694),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(674),
    [sym_super] = ACTIONS(674),
    [sym_crate] = ACTIONS(674),
    [sym__raw_string_literal_start] = ACTIONS(696),
    [sym_float_literal] = ACTIONS(690),
  },
  [120] = {
    [sym_delim_token_tree] = STATE(198),
    [sym__delim_tokens] = STATE(206),
    [sym__non_delim_token] = STATE(198),
    [sym__literal] = STATE(214),
    [sym_string_literal] = STATE(199),
    [sym_raw_string_literal] = STATE(199),
    [sym_boolean_literal] = STATE(199),
    [sym_line_comment] = STATE(120),
    [sym_block_comment] = STATE(120),
    [aux_sym__non_special_token_repeat1] = STATE(157),
    [aux_sym_delim_token_tree_repeat1] = STATE(83),
    [sym_identifier] = ACTIONS(674),
    [anon_sym_SEMI] = ACTIONS(676),
    [anon_sym_LPAREN] = ACTIONS(678),
    [anon_sym_RPAREN] = ACTIONS(720),
    [anon_sym_LBRACK] = ACTIONS(680),
    [anon_sym_LBRACE] = ACTIONS(682),
    [anon_sym_EQ_GT] = ACTIONS(676),
    [anon_sym_COLON] = ACTIONS(686),
    [anon_sym_DOLLAR] = ACTIONS(688),
    [anon_sym_PLUS] = ACTIONS(686),
    [anon_sym_STAR] = ACTIONS(686),
    [anon_sym_QMARK] = ACTIONS(676),
    [anon_sym_u8] = ACTIONS(674),
    [anon_sym_i8] = ACTIONS(674),
    [anon_sym_u16] = ACTIONS(674),
    [anon_sym_i16] = ACTIONS(674),
    [anon_sym_u32] = ACTIONS(674),
    [anon_sym_i32] = ACTIONS(674),
    [anon_sym_u64] = ACTIONS(674),
    [anon_sym_i64] = ACTIONS(674),
    [anon_sym_u128] = ACTIONS(674),
    [anon_sym_i128] = ACTIONS(674),
    [anon_sym_isize] = ACTIONS(674),
    [anon_sym_usize] = ACTIONS(674),
    [anon_sym_f32] = ACTIONS(674),
    [anon_sym_f64] = ACTIONS(674),
    [anon_sym_bool] = ACTIONS(674),
    [anon_sym_str] = ACTIONS(674),
    [anon_sym_char] = ACTIONS(674),
    [anon_sym_DASH] = ACTIONS(686),
    [anon_sym_SLASH] = ACTIONS(686),
    [anon_sym_PERCENT] = ACTIONS(686),
    [anon_sym_CARET] = ACTIONS(686),
    [anon_sym_BANG] = ACTIONS(686),
    [anon_sym_AMP] = ACTIONS(686),
    [anon_sym_PIPE] = ACTIONS(686),
    [anon_sym_AMP_AMP] = ACTIONS(676),
    [anon_sym_PIPE_PIPE] = ACTIONS(676),
    [anon_sym_LT_LT] = ACTIONS(686),
    [anon_sym_GT_GT] = ACTIONS(686),
    [anon_sym_PLUS_EQ] = ACTIONS(676),
    [anon_sym_DASH_EQ] = ACTIONS(676),
    [anon_sym_STAR_EQ] = ACTIONS(676),
    [anon_sym_SLASH_EQ] = ACTIONS(676),
    [anon_sym_PERCENT_EQ] = ACTIONS(676),
    [anon_sym_CARET_EQ] = ACTIONS(676),
    [anon_sym_AMP_EQ] = ACTIONS(676),
    [anon_sym_PIPE_EQ] = ACTIONS(676),
    [anon_sym_LT_LT_EQ] = ACTIONS(676),
    [anon_sym_GT_GT_EQ] = ACTIONS(676),
    [anon_sym_EQ] = ACTIONS(686),
    [anon_sym_EQ_EQ] = ACTIONS(676),
    [anon_sym_BANG_EQ] = ACTIONS(676),
    [anon_sym_GT] = ACTIONS(686),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT_EQ] = ACTIONS(676),
    [anon_sym_LT_EQ] = ACTIONS(676),
    [anon_sym_AT] = ACTIONS(676),
    [anon_sym__] = ACTIONS(686),
    [anon_sym_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(676),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(676),
    [anon_sym_COMMA] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(676),
    [anon_sym_DASH_GT] = ACTIONS(676),
    [anon_sym_POUND] = ACTIONS(676),
    [anon_sym_SQUOTE] = ACTIONS(674),
    [anon_sym_as] = ACTIONS(674),
    [anon_sym_async] = ACTIONS(674),
    [anon_sym_await] = ACTIONS(674),
    [anon_sym_break] = ACTIONS(674),
    [anon_sym_const] = ACTIONS(674),
    [anon_sym_continue] = ACTIONS(674),
    [anon_sym_default] = ACTIONS(674),
    [anon_sym_enum] = ACTIONS(674),
    [anon_sym_fn] = ACTIONS(674),
    [anon_sym_for] = ACTIONS(674),
    [anon_sym_gen] = ACTIONS(674),
    [anon_sym_if] = ACTIONS(674),
    [anon_sym_impl] = ACTIONS(674),
    [anon_sym_let] = ACTIONS(674),
    [anon_sym_loop] = ACTIONS(674),
    [anon_sym_match] = ACTIONS(674),
    [anon_sym_mod] = ACTIONS(674),
    [anon_sym_pub] = ACTIONS(674),
    [anon_sym_return] = ACTIONS(674),
    [anon_sym_static] = ACTIONS(674),
    [anon_sym_struct] = ACTIONS(674),
    [anon_sym_trait] = ACTIONS(674),
    [anon_sym_type] = ACTIONS(674),
    [anon_sym_union] = ACTIONS(674),
    [anon_sym_unsafe] = ACTIONS(674),
    [anon_sym_use] = ACTIONS(674),
    [anon_sym_where] = ACTIONS(674),
    [anon_sym_while] = ACTIONS(674),
    [sym_mutable_specifier] = ACTIONS(674),
    [sym_integer_literal] = ACTIONS(690),
    [aux_sym_string_literal_token1] = ACTIONS(692),
    [sym_char_literal] = ACTIONS(690),
    [anon_sym_true] = ACTIONS(694),
    [anon_sym_false] = ACTIONS(694),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(674),
    [sym_super] = ACTIONS(674),
    [sym_crate] = ACTIONS(674),
    [sym__raw_string_literal_start] = ACTIONS(696),
    [sym_float_literal] = ACTIONS(690),
  },
  [121] = {
    [sym_token_tree] = STATE(152),
    [sym_token_repetition] = STATE(152),
    [sym__literal] = STATE(152),
    [sym_string_literal] = STATE(177),
    [sym_raw_string_literal] = STATE(177),
    [sym_boolean_literal] = STATE(177),
    [sym_line_comment] = STATE(121),
    [sym_block_comment] = STATE(121),
    [aux_sym_token_tree_repeat1] = STATE(133),
    [aux_sym__non_special_token_repeat1] = STATE(143),
    [sym_identifier] = ACTIONS(700),
    [anon_sym_SEMI] = ACTIONS(567),
    [anon_sym_LPAREN] = ACTIONS(702),
    [anon_sym_RPAREN] = ACTIONS(738),
    [anon_sym_LBRACK] = ACTIONS(704),
    [anon_sym_LBRACE] = ACTIONS(708),
    [anon_sym_EQ_GT] = ACTIONS(567),
    [anon_sym_COLON] = ACTIONS(577),
    [anon_sym_DOLLAR] = ACTIONS(710),
    [anon_sym_PLUS] = ACTIONS(577),
    [anon_sym_STAR] = ACTIONS(577),
    [anon_sym_QMARK] = ACTIONS(567),
    [anon_sym_u8] = ACTIONS(700),
    [anon_sym_i8] = ACTIONS(700),
    [anon_sym_u16] = ACTIONS(700),
    [anon_sym_i16] = ACTIONS(700),
    [anon_sym_u32] = ACTIONS(700),
    [anon_sym_i32] = ACTIONS(700),
    [anon_sym_u64] = ACTIONS(700),
    [anon_sym_i64] = ACTIONS(700),
    [anon_sym_u128] = ACTIONS(700),
    [anon_sym_i128] = ACTIONS(700),
    [anon_sym_isize] = ACTIONS(700),
    [anon_sym_usize] = ACTIONS(700),
    [anon_sym_f32] = ACTIONS(700),
    [anon_sym_f64] = ACTIONS(700),
    [anon_sym_bool] = ACTIONS(700),
    [anon_sym_str] = ACTIONS(700),
    [anon_sym_char] = ACTIONS(700),
    [anon_sym_DASH] = ACTIONS(577),
    [anon_sym_SLASH] = ACTIONS(577),
    [anon_sym_PERCENT] = ACTIONS(577),
    [anon_sym_CARET] = ACTIONS(577),
    [anon_sym_BANG] = ACTIONS(577),
    [anon_sym_AMP] = ACTIONS(577),
    [anon_sym_PIPE] = ACTIONS(577),
    [anon_sym_AMP_AMP] = ACTIONS(567),
    [anon_sym_PIPE_PIPE] = ACTIONS(567),
    [anon_sym_LT_LT] = ACTIONS(577),
    [anon_sym_GT_GT] = ACTIONS(577),
    [anon_sym_PLUS_EQ] = ACTIONS(567),
    [anon_sym_DASH_EQ] = ACTIONS(567),
    [anon_sym_STAR_EQ] = ACTIONS(567),
    [anon_sym_SLASH_EQ] = ACTIONS(567),
    [anon_sym_PERCENT_EQ] = ACTIONS(567),
    [anon_sym_CARET_EQ] = ACTIONS(567),
    [anon_sym_AMP_EQ] = ACTIONS(567),
    [anon_sym_PIPE_EQ] = ACTIONS(567),
    [anon_sym_LT_LT_EQ] = ACTIONS(567),
    [anon_sym_GT_GT_EQ] = ACTIONS(567),
    [anon_sym_EQ] = ACTIONS(577),
    [anon_sym_EQ_EQ] = ACTIONS(567),
    [anon_sym_BANG_EQ] = ACTIONS(567),
    [anon_sym_GT] = ACTIONS(577),
    [anon_sym_LT] = ACTIONS(577),
    [anon_sym_GT_EQ] = ACTIONS(567),
    [anon_sym_LT_EQ] = ACTIONS(567),
    [anon_sym_AT] = ACTIONS(567),
    [anon_sym__] = ACTIONS(577),
    [anon_sym_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(567),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(567),
    [anon_sym_COMMA] = ACTIONS(567),
    [anon_sym_COLON_COLON] = ACTIONS(567),
    [anon_sym_DASH_GT] = ACTIONS(567),
    [anon_sym_POUND] = ACTIONS(567),
    [anon_sym_SQUOTE] = ACTIONS(700),
    [anon_sym_as] = ACTIONS(700),
    [anon_sym_async] = ACTIONS(700),
    [anon_sym_await] = ACTIONS(700),
    [anon_sym_break] = ACTIONS(700),
    [anon_sym_const] = ACTIONS(700),
    [anon_sym_continue] = ACTIONS(700),
    [anon_sym_default] = ACTIONS(700),
    [anon_sym_enum] = ACTIONS(700),
    [anon_sym_fn] = ACTIONS(700),
    [anon_sym_for] = ACTIONS(700),
    [anon_sym_gen] = ACTIONS(700),
    [anon_sym_if] = ACTIONS(700),
    [anon_sym_impl] = ACTIONS(700),
    [anon_sym_let] = ACTIONS(700),
    [anon_sym_loop] = ACTIONS(700),
    [anon_sym_match] = ACTIONS(700),
    [anon_sym_mod] = ACTIONS(700),
    [anon_sym_pub] = ACTIONS(700),
    [anon_sym_return] = ACTIONS(700),
    [anon_sym_static] = ACTIONS(700),
    [anon_sym_struct] = ACTIONS(700),
    [anon_sym_trait] = ACTIONS(700),
    [anon_sym_type] = ACTIONS(700),
    [anon_sym_union] = ACTIONS(700),
    [anon_sym_unsafe] = ACTIONS(700),
    [anon_sym_use] = ACTIONS(700),
    [anon_sym_where] = ACTIONS(700),
    [anon_sym_while] = ACTIONS(700),
    [sym_mutable_specifier] = ACTIONS(700),
    [sym_integer_literal] = ACTIONS(581),
    [aux_sym_string_literal_token1] = ACTIONS(583),
    [sym_char_literal] = ACTIONS(581),
    [anon_sym_true] = ACTIONS(585),
    [anon_sym_false] = ACTIONS(585),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(700),
    [sym_super] = ACTIONS(700),
    [sym_crate] = ACTIONS(700),
    [sym_metavariable] = ACTIONS(712),
    [sym__raw_string_literal_start] = ACTIONS(589),
    [sym_float_literal] = ACTIONS(581),
  },
  [122] = {
    [sym_delim_token_tree] = STATE(198),
    [sym__delim_tokens] = STATE(206),
    [sym__non_delim_token] = STATE(198),
    [sym__literal] = STATE(214),
    [sym_string_literal] = STATE(199),
    [sym_raw_string_literal] = STATE(199),
    [sym_boolean_literal] = STATE(199),
    [sym_line_comment] = STATE(122),
    [sym_block_comment] = STATE(122),
    [aux_sym__non_special_token_repeat1] = STATE(157),
    [aux_sym_delim_token_tree_repeat1] = STATE(125),
    [sym_identifier] = ACTIONS(674),
    [anon_sym_SEMI] = ACTIONS(676),
    [anon_sym_LPAREN] = ACTIONS(678),
    [anon_sym_RPAREN] = ACTIONS(684),
    [anon_sym_LBRACK] = ACTIONS(680),
    [anon_sym_LBRACE] = ACTIONS(682),
    [anon_sym_EQ_GT] = ACTIONS(676),
    [anon_sym_COLON] = ACTIONS(686),
    [anon_sym_DOLLAR] = ACTIONS(688),
    [anon_sym_PLUS] = ACTIONS(686),
    [anon_sym_STAR] = ACTIONS(686),
    [anon_sym_QMARK] = ACTIONS(676),
    [anon_sym_u8] = ACTIONS(674),
    [anon_sym_i8] = ACTIONS(674),
    [anon_sym_u16] = ACTIONS(674),
    [anon_sym_i16] = ACTIONS(674),
    [anon_sym_u32] = ACTIONS(674),
    [anon_sym_i32] = ACTIONS(674),
    [anon_sym_u64] = ACTIONS(674),
    [anon_sym_i64] = ACTIONS(674),
    [anon_sym_u128] = ACTIONS(674),
    [anon_sym_i128] = ACTIONS(674),
    [anon_sym_isize] = ACTIONS(674),
    [anon_sym_usize] = ACTIONS(674),
    [anon_sym_f32] = ACTIONS(674),
    [anon_sym_f64] = ACTIONS(674),
    [anon_sym_bool] = ACTIONS(674),
    [anon_sym_str] = ACTIONS(674),
    [anon_sym_char] = ACTIONS(674),
    [anon_sym_DASH] = ACTIONS(686),
    [anon_sym_SLASH] = ACTIONS(686),
    [anon_sym_PERCENT] = ACTIONS(686),
    [anon_sym_CARET] = ACTIONS(686),
    [anon_sym_BANG] = ACTIONS(686),
    [anon_sym_AMP] = ACTIONS(686),
    [anon_sym_PIPE] = ACTIONS(686),
    [anon_sym_AMP_AMP] = ACTIONS(676),
    [anon_sym_PIPE_PIPE] = ACTIONS(676),
    [anon_sym_LT_LT] = ACTIONS(686),
    [anon_sym_GT_GT] = ACTIONS(686),
    [anon_sym_PLUS_EQ] = ACTIONS(676),
    [anon_sym_DASH_EQ] = ACTIONS(676),
    [anon_sym_STAR_EQ] = ACTIONS(676),
    [anon_sym_SLASH_EQ] = ACTIONS(676),
    [anon_sym_PERCENT_EQ] = ACTIONS(676),
    [anon_sym_CARET_EQ] = ACTIONS(676),
    [anon_sym_AMP_EQ] = ACTIONS(676),
    [anon_sym_PIPE_EQ] = ACTIONS(676),
    [anon_sym_LT_LT_EQ] = ACTIONS(676),
    [anon_sym_GT_GT_EQ] = ACTIONS(676),
    [anon_sym_EQ] = ACTIONS(686),
    [anon_sym_EQ_EQ] = ACTIONS(676),
    [anon_sym_BANG_EQ] = ACTIONS(676),
    [anon_sym_GT] = ACTIONS(686),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT_EQ] = ACTIONS(676),
    [anon_sym_LT_EQ] = ACTIONS(676),
    [anon_sym_AT] = ACTIONS(676),
    [anon_sym__] = ACTIONS(686),
    [anon_sym_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(676),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(676),
    [anon_sym_COMMA] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(676),
    [anon_sym_DASH_GT] = ACTIONS(676),
    [anon_sym_POUND] = ACTIONS(676),
    [anon_sym_SQUOTE] = ACTIONS(674),
    [anon_sym_as] = ACTIONS(674),
    [anon_sym_async] = ACTIONS(674),
    [anon_sym_await] = ACTIONS(674),
    [anon_sym_break] = ACTIONS(674),
    [anon_sym_const] = ACTIONS(674),
    [anon_sym_continue] = ACTIONS(674),
    [anon_sym_default] = ACTIONS(674),
    [anon_sym_enum] = ACTIONS(674),
    [anon_sym_fn] = ACTIONS(674),
    [anon_sym_for] = ACTIONS(674),
    [anon_sym_gen] = ACTIONS(674),
    [anon_sym_if] = ACTIONS(674),
    [anon_sym_impl] = ACTIONS(674),
    [anon_sym_let] = ACTIONS(674),
    [anon_sym_loop] = ACTIONS(674),
    [anon_sym_match] = ACTIONS(674),
    [anon_sym_mod] = ACTIONS(674),
    [anon_sym_pub] = ACTIONS(674),
    [anon_sym_return] = ACTIONS(674),
    [anon_sym_static] = ACTIONS(674),
    [anon_sym_struct] = ACTIONS(674),
    [anon_sym_trait] = ACTIONS(674),
    [anon_sym_type] = ACTIONS(674),
    [anon_sym_union] = ACTIONS(674),
    [anon_sym_unsafe] = ACTIONS(674),
    [anon_sym_use] = ACTIONS(674),
    [anon_sym_where] = ACTIONS(674),
    [anon_sym_while] = ACTIONS(674),
    [sym_mutable_specifier] = ACTIONS(674),
    [sym_integer_literal] = ACTIONS(690),
    [aux_sym_string_literal_token1] = ACTIONS(692),
    [sym_char_literal] = ACTIONS(690),
    [anon_sym_true] = ACTIONS(694),
    [anon_sym_false] = ACTIONS(694),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(674),
    [sym_super] = ACTIONS(674),
    [sym_crate] = ACTIONS(674),
    [sym__raw_string_literal_start] = ACTIONS(696),
    [sym_float_literal] = ACTIONS(690),
  },
  [123] = {
    [sym_delim_token_tree] = STATE(198),
    [sym__delim_tokens] = STATE(206),
    [sym__non_delim_token] = STATE(198),
    [sym__literal] = STATE(214),
    [sym_string_literal] = STATE(199),
    [sym_raw_string_literal] = STATE(199),
    [sym_boolean_literal] = STATE(199),
    [sym_line_comment] = STATE(123),
    [sym_block_comment] = STATE(123),
    [aux_sym__non_special_token_repeat1] = STATE(157),
    [aux_sym_delim_token_tree_repeat1] = STATE(126),
    [sym_identifier] = ACTIONS(674),
    [anon_sym_SEMI] = ACTIONS(676),
    [anon_sym_LPAREN] = ACTIONS(678),
    [anon_sym_LBRACK] = ACTIONS(680),
    [anon_sym_RBRACK] = ACTIONS(684),
    [anon_sym_LBRACE] = ACTIONS(682),
    [anon_sym_EQ_GT] = ACTIONS(676),
    [anon_sym_COLON] = ACTIONS(686),
    [anon_sym_DOLLAR] = ACTIONS(688),
    [anon_sym_PLUS] = ACTIONS(686),
    [anon_sym_STAR] = ACTIONS(686),
    [anon_sym_QMARK] = ACTIONS(676),
    [anon_sym_u8] = ACTIONS(674),
    [anon_sym_i8] = ACTIONS(674),
    [anon_sym_u16] = ACTIONS(674),
    [anon_sym_i16] = ACTIONS(674),
    [anon_sym_u32] = ACTIONS(674),
    [anon_sym_i32] = ACTIONS(674),
    [anon_sym_u64] = ACTIONS(674),
    [anon_sym_i64] = ACTIONS(674),
    [anon_sym_u128] = ACTIONS(674),
    [anon_sym_i128] = ACTIONS(674),
    [anon_sym_isize] = ACTIONS(674),
    [anon_sym_usize] = ACTIONS(674),
    [anon_sym_f32] = ACTIONS(674),
    [anon_sym_f64] = ACTIONS(674),
    [anon_sym_bool] = ACTIONS(674),
    [anon_sym_str] = ACTIONS(674),
    [anon_sym_char] = ACTIONS(674),
    [anon_sym_DASH] = ACTIONS(686),
    [anon_sym_SLASH] = ACTIONS(686),
    [anon_sym_PERCENT] = ACTIONS(686),
    [anon_sym_CARET] = ACTIONS(686),
    [anon_sym_BANG] = ACTIONS(686),
    [anon_sym_AMP] = ACTIONS(686),
    [anon_sym_PIPE] = ACTIONS(686),
    [anon_sym_AMP_AMP] = ACTIONS(676),
    [anon_sym_PIPE_PIPE] = ACTIONS(676),
    [anon_sym_LT_LT] = ACTIONS(686),
    [anon_sym_GT_GT] = ACTIONS(686),
    [anon_sym_PLUS_EQ] = ACTIONS(676),
    [anon_sym_DASH_EQ] = ACTIONS(676),
    [anon_sym_STAR_EQ] = ACTIONS(676),
    [anon_sym_SLASH_EQ] = ACTIONS(676),
    [anon_sym_PERCENT_EQ] = ACTIONS(676),
    [anon_sym_CARET_EQ] = ACTIONS(676),
    [anon_sym_AMP_EQ] = ACTIONS(676),
    [anon_sym_PIPE_EQ] = ACTIONS(676),
    [anon_sym_LT_LT_EQ] = ACTIONS(676),
    [anon_sym_GT_GT_EQ] = ACTIONS(676),
    [anon_sym_EQ] = ACTIONS(686),
    [anon_sym_EQ_EQ] = ACTIONS(676),
    [anon_sym_BANG_EQ] = ACTIONS(676),
    [anon_sym_GT] = ACTIONS(686),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT_EQ] = ACTIONS(676),
    [anon_sym_LT_EQ] = ACTIONS(676),
    [anon_sym_AT] = ACTIONS(676),
    [anon_sym__] = ACTIONS(686),
    [anon_sym_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(676),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(676),
    [anon_sym_COMMA] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(676),
    [anon_sym_DASH_GT] = ACTIONS(676),
    [anon_sym_POUND] = ACTIONS(676),
    [anon_sym_SQUOTE] = ACTIONS(674),
    [anon_sym_as] = ACTIONS(674),
    [anon_sym_async] = ACTIONS(674),
    [anon_sym_await] = ACTIONS(674),
    [anon_sym_break] = ACTIONS(674),
    [anon_sym_const] = ACTIONS(674),
    [anon_sym_continue] = ACTIONS(674),
    [anon_sym_default] = ACTIONS(674),
    [anon_sym_enum] = ACTIONS(674),
    [anon_sym_fn] = ACTIONS(674),
    [anon_sym_for] = ACTIONS(674),
    [anon_sym_gen] = ACTIONS(674),
    [anon_sym_if] = ACTIONS(674),
    [anon_sym_impl] = ACTIONS(674),
    [anon_sym_let] = ACTIONS(674),
    [anon_sym_loop] = ACTIONS(674),
    [anon_sym_match] = ACTIONS(674),
    [anon_sym_mod] = ACTIONS(674),
    [anon_sym_pub] = ACTIONS(674),
    [anon_sym_return] = ACTIONS(674),
    [anon_sym_static] = ACTIONS(674),
    [anon_sym_struct] = ACTIONS(674),
    [anon_sym_trait] = ACTIONS(674),
    [anon_sym_type] = ACTIONS(674),
    [anon_sym_union] = ACTIONS(674),
    [anon_sym_unsafe] = ACTIONS(674),
    [anon_sym_use] = ACTIONS(674),
    [anon_sym_where] = ACTIONS(674),
    [anon_sym_while] = ACTIONS(674),
    [sym_mutable_specifier] = ACTIONS(674),
    [sym_integer_literal] = ACTIONS(690),
    [aux_sym_string_literal_token1] = ACTIONS(692),
    [sym_char_literal] = ACTIONS(690),
    [anon_sym_true] = ACTIONS(694),
    [anon_sym_false] = ACTIONS(694),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(674),
    [sym_super] = ACTIONS(674),
    [sym_crate] = ACTIONS(674),
    [sym__raw_string_literal_start] = ACTIONS(696),
    [sym_float_literal] = ACTIONS(690),
  },
  [124] = {
    [sym_delim_token_tree] = STATE(198),
    [sym__delim_tokens] = STATE(206),
    [sym__non_delim_token] = STATE(198),
    [sym__literal] = STATE(214),
    [sym_string_literal] = STATE(199),
    [sym_raw_string_literal] = STATE(199),
    [sym_boolean_literal] = STATE(199),
    [sym_line_comment] = STATE(124),
    [sym_block_comment] = STATE(124),
    [aux_sym__non_special_token_repeat1] = STATE(157),
    [aux_sym_delim_token_tree_repeat1] = STATE(85),
    [sym_identifier] = ACTIONS(674),
    [anon_sym_SEMI] = ACTIONS(676),
    [anon_sym_LPAREN] = ACTIONS(678),
    [anon_sym_RPAREN] = ACTIONS(740),
    [anon_sym_LBRACK] = ACTIONS(680),
    [anon_sym_LBRACE] = ACTIONS(682),
    [anon_sym_EQ_GT] = ACTIONS(676),
    [anon_sym_COLON] = ACTIONS(686),
    [anon_sym_DOLLAR] = ACTIONS(688),
    [anon_sym_PLUS] = ACTIONS(686),
    [anon_sym_STAR] = ACTIONS(686),
    [anon_sym_QMARK] = ACTIONS(676),
    [anon_sym_u8] = ACTIONS(674),
    [anon_sym_i8] = ACTIONS(674),
    [anon_sym_u16] = ACTIONS(674),
    [anon_sym_i16] = ACTIONS(674),
    [anon_sym_u32] = ACTIONS(674),
    [anon_sym_i32] = ACTIONS(674),
    [anon_sym_u64] = ACTIONS(674),
    [anon_sym_i64] = ACTIONS(674),
    [anon_sym_u128] = ACTIONS(674),
    [anon_sym_i128] = ACTIONS(674),
    [anon_sym_isize] = ACTIONS(674),
    [anon_sym_usize] = ACTIONS(674),
    [anon_sym_f32] = ACTIONS(674),
    [anon_sym_f64] = ACTIONS(674),
    [anon_sym_bool] = ACTIONS(674),
    [anon_sym_str] = ACTIONS(674),
    [anon_sym_char] = ACTIONS(674),
    [anon_sym_DASH] = ACTIONS(686),
    [anon_sym_SLASH] = ACTIONS(686),
    [anon_sym_PERCENT] = ACTIONS(686),
    [anon_sym_CARET] = ACTIONS(686),
    [anon_sym_BANG] = ACTIONS(686),
    [anon_sym_AMP] = ACTIONS(686),
    [anon_sym_PIPE] = ACTIONS(686),
    [anon_sym_AMP_AMP] = ACTIONS(676),
    [anon_sym_PIPE_PIPE] = ACTIONS(676),
    [anon_sym_LT_LT] = ACTIONS(686),
    [anon_sym_GT_GT] = ACTIONS(686),
    [anon_sym_PLUS_EQ] = ACTIONS(676),
    [anon_sym_DASH_EQ] = ACTIONS(676),
    [anon_sym_STAR_EQ] = ACTIONS(676),
    [anon_sym_SLASH_EQ] = ACTIONS(676),
    [anon_sym_PERCENT_EQ] = ACTIONS(676),
    [anon_sym_CARET_EQ] = ACTIONS(676),
    [anon_sym_AMP_EQ] = ACTIONS(676),
    [anon_sym_PIPE_EQ] = ACTIONS(676),
    [anon_sym_LT_LT_EQ] = ACTIONS(676),
    [anon_sym_GT_GT_EQ] = ACTIONS(676),
    [anon_sym_EQ] = ACTIONS(686),
    [anon_sym_EQ_EQ] = ACTIONS(676),
    [anon_sym_BANG_EQ] = ACTIONS(676),
    [anon_sym_GT] = ACTIONS(686),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT_EQ] = ACTIONS(676),
    [anon_sym_LT_EQ] = ACTIONS(676),
    [anon_sym_AT] = ACTIONS(676),
    [anon_sym__] = ACTIONS(686),
    [anon_sym_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(676),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(676),
    [anon_sym_COMMA] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(676),
    [anon_sym_DASH_GT] = ACTIONS(676),
    [anon_sym_POUND] = ACTIONS(676),
    [anon_sym_SQUOTE] = ACTIONS(674),
    [anon_sym_as] = ACTIONS(674),
    [anon_sym_async] = ACTIONS(674),
    [anon_sym_await] = ACTIONS(674),
    [anon_sym_break] = ACTIONS(674),
    [anon_sym_const] = ACTIONS(674),
    [anon_sym_continue] = ACTIONS(674),
    [anon_sym_default] = ACTIONS(674),
    [anon_sym_enum] = ACTIONS(674),
    [anon_sym_fn] = ACTIONS(674),
    [anon_sym_for] = ACTIONS(674),
    [anon_sym_gen] = ACTIONS(674),
    [anon_sym_if] = ACTIONS(674),
    [anon_sym_impl] = ACTIONS(674),
    [anon_sym_let] = ACTIONS(674),
    [anon_sym_loop] = ACTIONS(674),
    [anon_sym_match] = ACTIONS(674),
    [anon_sym_mod] = ACTIONS(674),
    [anon_sym_pub] = ACTIONS(674),
    [anon_sym_return] = ACTIONS(674),
    [anon_sym_static] = ACTIONS(674),
    [anon_sym_struct] = ACTIONS(674),
    [anon_sym_trait] = ACTIONS(674),
    [anon_sym_type] = ACTIONS(674),
    [anon_sym_union] = ACTIONS(674),
    [anon_sym_unsafe] = ACTIONS(674),
    [anon_sym_use] = ACTIONS(674),
    [anon_sym_where] = ACTIONS(674),
    [anon_sym_while] = ACTIONS(674),
    [sym_mutable_specifier] = ACTIONS(674),
    [sym_integer_literal] = ACTIONS(690),
    [aux_sym_string_literal_token1] = ACTIONS(692),
    [sym_char_literal] = ACTIONS(690),
    [anon_sym_true] = ACTIONS(694),
    [anon_sym_false] = ACTIONS(694),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(674),
    [sym_super] = ACTIONS(674),
    [sym_crate] = ACTIONS(674),
    [sym__raw_string_literal_start] = ACTIONS(696),
    [sym_float_literal] = ACTIONS(690),
  },
  [125] = {
    [sym_delim_token_tree] = STATE(198),
    [sym__delim_tokens] = STATE(206),
    [sym__non_delim_token] = STATE(198),
    [sym__literal] = STATE(214),
    [sym_string_literal] = STATE(199),
    [sym_raw_string_literal] = STATE(199),
    [sym_boolean_literal] = STATE(199),
    [sym_line_comment] = STATE(125),
    [sym_block_comment] = STATE(125),
    [aux_sym__non_special_token_repeat1] = STATE(157),
    [aux_sym_delim_token_tree_repeat1] = STATE(83),
    [sym_identifier] = ACTIONS(674),
    [anon_sym_SEMI] = ACTIONS(676),
    [anon_sym_LPAREN] = ACTIONS(678),
    [anon_sym_RPAREN] = ACTIONS(742),
    [anon_sym_LBRACK] = ACTIONS(680),
    [anon_sym_LBRACE] = ACTIONS(682),
    [anon_sym_EQ_GT] = ACTIONS(676),
    [anon_sym_COLON] = ACTIONS(686),
    [anon_sym_DOLLAR] = ACTIONS(688),
    [anon_sym_PLUS] = ACTIONS(686),
    [anon_sym_STAR] = ACTIONS(686),
    [anon_sym_QMARK] = ACTIONS(676),
    [anon_sym_u8] = ACTIONS(674),
    [anon_sym_i8] = ACTIONS(674),
    [anon_sym_u16] = ACTIONS(674),
    [anon_sym_i16] = ACTIONS(674),
    [anon_sym_u32] = ACTIONS(674),
    [anon_sym_i32] = ACTIONS(674),
    [anon_sym_u64] = ACTIONS(674),
    [anon_sym_i64] = ACTIONS(674),
    [anon_sym_u128] = ACTIONS(674),
    [anon_sym_i128] = ACTIONS(674),
    [anon_sym_isize] = ACTIONS(674),
    [anon_sym_usize] = ACTIONS(674),
    [anon_sym_f32] = ACTIONS(674),
    [anon_sym_f64] = ACTIONS(674),
    [anon_sym_bool] = ACTIONS(674),
    [anon_sym_str] = ACTIONS(674),
    [anon_sym_char] = ACTIONS(674),
    [anon_sym_DASH] = ACTIONS(686),
    [anon_sym_SLASH] = ACTIONS(686),
    [anon_sym_PERCENT] = ACTIONS(686),
    [anon_sym_CARET] = ACTIONS(686),
    [anon_sym_BANG] = ACTIONS(686),
    [anon_sym_AMP] = ACTIONS(686),
    [anon_sym_PIPE] = ACTIONS(686),
    [anon_sym_AMP_AMP] = ACTIONS(676),
    [anon_sym_PIPE_PIPE] = ACTIONS(676),
    [anon_sym_LT_LT] = ACTIONS(686),
    [anon_sym_GT_GT] = ACTIONS(686),
    [anon_sym_PLUS_EQ] = ACTIONS(676),
    [anon_sym_DASH_EQ] = ACTIONS(676),
    [anon_sym_STAR_EQ] = ACTIONS(676),
    [anon_sym_SLASH_EQ] = ACTIONS(676),
    [anon_sym_PERCENT_EQ] = ACTIONS(676),
    [anon_sym_CARET_EQ] = ACTIONS(676),
    [anon_sym_AMP_EQ] = ACTIONS(676),
    [anon_sym_PIPE_EQ] = ACTIONS(676),
    [anon_sym_LT_LT_EQ] = ACTIONS(676),
    [anon_sym_GT_GT_EQ] = ACTIONS(676),
    [anon_sym_EQ] = ACTIONS(686),
    [anon_sym_EQ_EQ] = ACTIONS(676),
    [anon_sym_BANG_EQ] = ACTIONS(676),
    [anon_sym_GT] = ACTIONS(686),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT_EQ] = ACTIONS(676),
    [anon_sym_LT_EQ] = ACTIONS(676),
    [anon_sym_AT] = ACTIONS(676),
    [anon_sym__] = ACTIONS(686),
    [anon_sym_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(676),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(676),
    [anon_sym_COMMA] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(676),
    [anon_sym_DASH_GT] = ACTIONS(676),
    [anon_sym_POUND] = ACTIONS(676),
    [anon_sym_SQUOTE] = ACTIONS(674),
    [anon_sym_as] = ACTIONS(674),
    [anon_sym_async] = ACTIONS(674),
    [anon_sym_await] = ACTIONS(674),
    [anon_sym_break] = ACTIONS(674),
    [anon_sym_const] = ACTIONS(674),
    [anon_sym_continue] = ACTIONS(674),
    [anon_sym_default] = ACTIONS(674),
    [anon_sym_enum] = ACTIONS(674),
    [anon_sym_fn] = ACTIONS(674),
    [anon_sym_for] = ACTIONS(674),
    [anon_sym_gen] = ACTIONS(674),
    [anon_sym_if] = ACTIONS(674),
    [anon_sym_impl] = ACTIONS(674),
    [anon_sym_let] = ACTIONS(674),
    [anon_sym_loop] = ACTIONS(674),
    [anon_sym_match] = ACTIONS(674),
    [anon_sym_mod] = ACTIONS(674),
    [anon_sym_pub] = ACTIONS(674),
    [anon_sym_return] = ACTIONS(674),
    [anon_sym_static] = ACTIONS(674),
    [anon_sym_struct] = ACTIONS(674),
    [anon_sym_trait] = ACTIONS(674),
    [anon_sym_type] = ACTIONS(674),
    [anon_sym_union] = ACTIONS(674),
    [anon_sym_unsafe] = ACTIONS(674),
    [anon_sym_use] = ACTIONS(674),
    [anon_sym_where] = ACTIONS(674),
    [anon_sym_while] = ACTIONS(674),
    [sym_mutable_specifier] = ACTIONS(674),
    [sym_integer_literal] = ACTIONS(690),
    [aux_sym_string_literal_token1] = ACTIONS(692),
    [sym_char_literal] = ACTIONS(690),
    [anon_sym_true] = ACTIONS(694),
    [anon_sym_false] = ACTIONS(694),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(674),
    [sym_super] = ACTIONS(674),
    [sym_crate] = ACTIONS(674),
    [sym__raw_string_literal_start] = ACTIONS(696),
    [sym_float_literal] = ACTIONS(690),
  },
  [126] = {
    [sym_delim_token_tree] = STATE(198),
    [sym__delim_tokens] = STATE(206),
    [sym__non_delim_token] = STATE(198),
    [sym__literal] = STATE(214),
    [sym_string_literal] = STATE(199),
    [sym_raw_string_literal] = STATE(199),
    [sym_boolean_literal] = STATE(199),
    [sym_line_comment] = STATE(126),
    [sym_block_comment] = STATE(126),
    [aux_sym__non_special_token_repeat1] = STATE(157),
    [aux_sym_delim_token_tree_repeat1] = STATE(83),
    [sym_identifier] = ACTIONS(674),
    [anon_sym_SEMI] = ACTIONS(676),
    [anon_sym_LPAREN] = ACTIONS(678),
    [anon_sym_LBRACK] = ACTIONS(680),
    [anon_sym_RBRACK] = ACTIONS(742),
    [anon_sym_LBRACE] = ACTIONS(682),
    [anon_sym_EQ_GT] = ACTIONS(676),
    [anon_sym_COLON] = ACTIONS(686),
    [anon_sym_DOLLAR] = ACTIONS(688),
    [anon_sym_PLUS] = ACTIONS(686),
    [anon_sym_STAR] = ACTIONS(686),
    [anon_sym_QMARK] = ACTIONS(676),
    [anon_sym_u8] = ACTIONS(674),
    [anon_sym_i8] = ACTIONS(674),
    [anon_sym_u16] = ACTIONS(674),
    [anon_sym_i16] = ACTIONS(674),
    [anon_sym_u32] = ACTIONS(674),
    [anon_sym_i32] = ACTIONS(674),
    [anon_sym_u64] = ACTIONS(674),
    [anon_sym_i64] = ACTIONS(674),
    [anon_sym_u128] = ACTIONS(674),
    [anon_sym_i128] = ACTIONS(674),
    [anon_sym_isize] = ACTIONS(674),
    [anon_sym_usize] = ACTIONS(674),
    [anon_sym_f32] = ACTIONS(674),
    [anon_sym_f64] = ACTIONS(674),
    [anon_sym_bool] = ACTIONS(674),
    [anon_sym_str] = ACTIONS(674),
    [anon_sym_char] = ACTIONS(674),
    [anon_sym_DASH] = ACTIONS(686),
    [anon_sym_SLASH] = ACTIONS(686),
    [anon_sym_PERCENT] = ACTIONS(686),
    [anon_sym_CARET] = ACTIONS(686),
    [anon_sym_BANG] = ACTIONS(686),
    [anon_sym_AMP] = ACTIONS(686),
    [anon_sym_PIPE] = ACTIONS(686),
    [anon_sym_AMP_AMP] = ACTIONS(676),
    [anon_sym_PIPE_PIPE] = ACTIONS(676),
    [anon_sym_LT_LT] = ACTIONS(686),
    [anon_sym_GT_GT] = ACTIONS(686),
    [anon_sym_PLUS_EQ] = ACTIONS(676),
    [anon_sym_DASH_EQ] = ACTIONS(676),
    [anon_sym_STAR_EQ] = ACTIONS(676),
    [anon_sym_SLASH_EQ] = ACTIONS(676),
    [anon_sym_PERCENT_EQ] = ACTIONS(676),
    [anon_sym_CARET_EQ] = ACTIONS(676),
    [anon_sym_AMP_EQ] = ACTIONS(676),
    [anon_sym_PIPE_EQ] = ACTIONS(676),
    [anon_sym_LT_LT_EQ] = ACTIONS(676),
    [anon_sym_GT_GT_EQ] = ACTIONS(676),
    [anon_sym_EQ] = ACTIONS(686),
    [anon_sym_EQ_EQ] = ACTIONS(676),
    [anon_sym_BANG_EQ] = ACTIONS(676),
    [anon_sym_GT] = ACTIONS(686),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT_EQ] = ACTIONS(676),
    [anon_sym_LT_EQ] = ACTIONS(676),
    [anon_sym_AT] = ACTIONS(676),
    [anon_sym__] = ACTIONS(686),
    [anon_sym_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(676),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(676),
    [anon_sym_COMMA] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(676),
    [anon_sym_DASH_GT] = ACTIONS(676),
    [anon_sym_POUND] = ACTIONS(676),
    [anon_sym_SQUOTE] = ACTIONS(674),
    [anon_sym_as] = ACTIONS(674),
    [anon_sym_async] = ACTIONS(674),
    [anon_sym_await] = ACTIONS(674),
    [anon_sym_break] = ACTIONS(674),
    [anon_sym_const] = ACTIONS(674),
    [anon_sym_continue] = ACTIONS(674),
    [anon_sym_default] = ACTIONS(674),
    [anon_sym_enum] = ACTIONS(674),
    [anon_sym_fn] = ACTIONS(674),
    [anon_sym_for] = ACTIONS(674),
    [anon_sym_gen] = ACTIONS(674),
    [anon_sym_if] = ACTIONS(674),
    [anon_sym_impl] = ACTIONS(674),
    [anon_sym_let] = ACTIONS(674),
    [anon_sym_loop] = ACTIONS(674),
    [anon_sym_match] = ACTIONS(674),
    [anon_sym_mod] = ACTIONS(674),
    [anon_sym_pub] = ACTIONS(674),
    [anon_sym_return] = ACTIONS(674),
    [anon_sym_static] = ACTIONS(674),
    [anon_sym_struct] = ACTIONS(674),
    [anon_sym_trait] = ACTIONS(674),
    [anon_sym_type] = ACTIONS(674),
    [anon_sym_union] = ACTIONS(674),
    [anon_sym_unsafe] = ACTIONS(674),
    [anon_sym_use] = ACTIONS(674),
    [anon_sym_where] = ACTIONS(674),
    [anon_sym_while] = ACTIONS(674),
    [sym_mutable_specifier] = ACTIONS(674),
    [sym_integer_literal] = ACTIONS(690),
    [aux_sym_string_literal_token1] = ACTIONS(692),
    [sym_char_literal] = ACTIONS(690),
    [anon_sym_true] = ACTIONS(694),
    [anon_sym_false] = ACTIONS(694),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(674),
    [sym_super] = ACTIONS(674),
    [sym_crate] = ACTIONS(674),
    [sym__raw_string_literal_start] = ACTIONS(696),
    [sym_float_literal] = ACTIONS(690),
  },
  [127] = {
    [sym_delim_token_tree] = STATE(198),
    [sym__delim_tokens] = STATE(206),
    [sym__non_delim_token] = STATE(198),
    [sym__literal] = STATE(214),
    [sym_string_literal] = STATE(199),
    [sym_raw_string_literal] = STATE(199),
    [sym_boolean_literal] = STATE(199),
    [sym_line_comment] = STATE(127),
    [sym_block_comment] = STATE(127),
    [aux_sym__non_special_token_repeat1] = STATE(157),
    [aux_sym_delim_token_tree_repeat1] = STATE(83),
    [sym_identifier] = ACTIONS(674),
    [anon_sym_SEMI] = ACTIONS(676),
    [anon_sym_LPAREN] = ACTIONS(678),
    [anon_sym_LBRACK] = ACTIONS(680),
    [anon_sym_LBRACE] = ACTIONS(682),
    [anon_sym_RBRACE] = ACTIONS(742),
    [anon_sym_EQ_GT] = ACTIONS(676),
    [anon_sym_COLON] = ACTIONS(686),
    [anon_sym_DOLLAR] = ACTIONS(688),
    [anon_sym_PLUS] = ACTIONS(686),
    [anon_sym_STAR] = ACTIONS(686),
    [anon_sym_QMARK] = ACTIONS(676),
    [anon_sym_u8] = ACTIONS(674),
    [anon_sym_i8] = ACTIONS(674),
    [anon_sym_u16] = ACTIONS(674),
    [anon_sym_i16] = ACTIONS(674),
    [anon_sym_u32] = ACTIONS(674),
    [anon_sym_i32] = ACTIONS(674),
    [anon_sym_u64] = ACTIONS(674),
    [anon_sym_i64] = ACTIONS(674),
    [anon_sym_u128] = ACTIONS(674),
    [anon_sym_i128] = ACTIONS(674),
    [anon_sym_isize] = ACTIONS(674),
    [anon_sym_usize] = ACTIONS(674),
    [anon_sym_f32] = ACTIONS(674),
    [anon_sym_f64] = ACTIONS(674),
    [anon_sym_bool] = ACTIONS(674),
    [anon_sym_str] = ACTIONS(674),
    [anon_sym_char] = ACTIONS(674),
    [anon_sym_DASH] = ACTIONS(686),
    [anon_sym_SLASH] = ACTIONS(686),
    [anon_sym_PERCENT] = ACTIONS(686),
    [anon_sym_CARET] = ACTIONS(686),
    [anon_sym_BANG] = ACTIONS(686),
    [anon_sym_AMP] = ACTIONS(686),
    [anon_sym_PIPE] = ACTIONS(686),
    [anon_sym_AMP_AMP] = ACTIONS(676),
    [anon_sym_PIPE_PIPE] = ACTIONS(676),
    [anon_sym_LT_LT] = ACTIONS(686),
    [anon_sym_GT_GT] = ACTIONS(686),
    [anon_sym_PLUS_EQ] = ACTIONS(676),
    [anon_sym_DASH_EQ] = ACTIONS(676),
    [anon_sym_STAR_EQ] = ACTIONS(676),
    [anon_sym_SLASH_EQ] = ACTIONS(676),
    [anon_sym_PERCENT_EQ] = ACTIONS(676),
    [anon_sym_CARET_EQ] = ACTIONS(676),
    [anon_sym_AMP_EQ] = ACTIONS(676),
    [anon_sym_PIPE_EQ] = ACTIONS(676),
    [anon_sym_LT_LT_EQ] = ACTIONS(676),
    [anon_sym_GT_GT_EQ] = ACTIONS(676),
    [anon_sym_EQ] = ACTIONS(686),
    [anon_sym_EQ_EQ] = ACTIONS(676),
    [anon_sym_BANG_EQ] = ACTIONS(676),
    [anon_sym_GT] = ACTIONS(686),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT_EQ] = ACTIONS(676),
    [anon_sym_LT_EQ] = ACTIONS(676),
    [anon_sym_AT] = ACTIONS(676),
    [anon_sym__] = ACTIONS(686),
    [anon_sym_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(676),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(676),
    [anon_sym_COMMA] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(676),
    [anon_sym_DASH_GT] = ACTIONS(676),
    [anon_sym_POUND] = ACTIONS(676),
    [anon_sym_SQUOTE] = ACTIONS(674),
    [anon_sym_as] = ACTIONS(674),
    [anon_sym_async] = ACTIONS(674),
    [anon_sym_await] = ACTIONS(674),
    [anon_sym_break] = ACTIONS(674),
    [anon_sym_const] = ACTIONS(674),
    [anon_sym_continue] = ACTIONS(674),
    [anon_sym_default] = ACTIONS(674),
    [anon_sym_enum] = ACTIONS(674),
    [anon_sym_fn] = ACTIONS(674),
    [anon_sym_for] = ACTIONS(674),
    [anon_sym_gen] = ACTIONS(674),
    [anon_sym_if] = ACTIONS(674),
    [anon_sym_impl] = ACTIONS(674),
    [anon_sym_let] = ACTIONS(674),
    [anon_sym_loop] = ACTIONS(674),
    [anon_sym_match] = ACTIONS(674),
    [anon_sym_mod] = ACTIONS(674),
    [anon_sym_pub] = ACTIONS(674),
    [anon_sym_return] = ACTIONS(674),
    [anon_sym_static] = ACTIONS(674),
    [anon_sym_struct] = ACTIONS(674),
    [anon_sym_trait] = ACTIONS(674),
    [anon_sym_type] = ACTIONS(674),
    [anon_sym_union] = ACTIONS(674),
    [anon_sym_unsafe] = ACTIONS(674),
    [anon_sym_use] = ACTIONS(674),
    [anon_sym_where] = ACTIONS(674),
    [anon_sym_while] = ACTIONS(674),
    [sym_mutable_specifier] = ACTIONS(674),
    [sym_integer_literal] = ACTIONS(690),
    [aux_sym_string_literal_token1] = ACTIONS(692),
    [sym_char_literal] = ACTIONS(690),
    [anon_sym_true] = ACTIONS(694),
    [anon_sym_false] = ACTIONS(694),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(674),
    [sym_super] = ACTIONS(674),
    [sym_crate] = ACTIONS(674),
    [sym__raw_string_literal_start] = ACTIONS(696),
    [sym_float_literal] = ACTIONS(690),
  },
  [128] = {
    [sym_delim_token_tree] = STATE(198),
    [sym__delim_tokens] = STATE(206),
    [sym__non_delim_token] = STATE(198),
    [sym__literal] = STATE(214),
    [sym_string_literal] = STATE(199),
    [sym_raw_string_literal] = STATE(199),
    [sym_boolean_literal] = STATE(199),
    [sym_line_comment] = STATE(128),
    [sym_block_comment] = STATE(128),
    [aux_sym__non_special_token_repeat1] = STATE(157),
    [aux_sym_delim_token_tree_repeat1] = STATE(86),
    [sym_identifier] = ACTIONS(674),
    [anon_sym_SEMI] = ACTIONS(676),
    [anon_sym_LPAREN] = ACTIONS(678),
    [anon_sym_LBRACK] = ACTIONS(680),
    [anon_sym_RBRACK] = ACTIONS(740),
    [anon_sym_LBRACE] = ACTIONS(682),
    [anon_sym_EQ_GT] = ACTIONS(676),
    [anon_sym_COLON] = ACTIONS(686),
    [anon_sym_DOLLAR] = ACTIONS(688),
    [anon_sym_PLUS] = ACTIONS(686),
    [anon_sym_STAR] = ACTIONS(686),
    [anon_sym_QMARK] = ACTIONS(676),
    [anon_sym_u8] = ACTIONS(674),
    [anon_sym_i8] = ACTIONS(674),
    [anon_sym_u16] = ACTIONS(674),
    [anon_sym_i16] = ACTIONS(674),
    [anon_sym_u32] = ACTIONS(674),
    [anon_sym_i32] = ACTIONS(674),
    [anon_sym_u64] = ACTIONS(674),
    [anon_sym_i64] = ACTIONS(674),
    [anon_sym_u128] = ACTIONS(674),
    [anon_sym_i128] = ACTIONS(674),
    [anon_sym_isize] = ACTIONS(674),
    [anon_sym_usize] = ACTIONS(674),
    [anon_sym_f32] = ACTIONS(674),
    [anon_sym_f64] = ACTIONS(674),
    [anon_sym_bool] = ACTIONS(674),
    [anon_sym_str] = ACTIONS(674),
    [anon_sym_char] = ACTIONS(674),
    [anon_sym_DASH] = ACTIONS(686),
    [anon_sym_SLASH] = ACTIONS(686),
    [anon_sym_PERCENT] = ACTIONS(686),
    [anon_sym_CARET] = ACTIONS(686),
    [anon_sym_BANG] = ACTIONS(686),
    [anon_sym_AMP] = ACTIONS(686),
    [anon_sym_PIPE] = ACTIONS(686),
    [anon_sym_AMP_AMP] = ACTIONS(676),
    [anon_sym_PIPE_PIPE] = ACTIONS(676),
    [anon_sym_LT_LT] = ACTIONS(686),
    [anon_sym_GT_GT] = ACTIONS(686),
    [anon_sym_PLUS_EQ] = ACTIONS(676),
    [anon_sym_DASH_EQ] = ACTIONS(676),
    [anon_sym_STAR_EQ] = ACTIONS(676),
    [anon_sym_SLASH_EQ] = ACTIONS(676),
    [anon_sym_PERCENT_EQ] = ACTIONS(676),
    [anon_sym_CARET_EQ] = ACTIONS(676),
    [anon_sym_AMP_EQ] = ACTIONS(676),
    [anon_sym_PIPE_EQ] = ACTIONS(676),
    [anon_sym_LT_LT_EQ] = ACTIONS(676),
    [anon_sym_GT_GT_EQ] = ACTIONS(676),
    [anon_sym_EQ] = ACTIONS(686),
    [anon_sym_EQ_EQ] = ACTIONS(676),
    [anon_sym_BANG_EQ] = ACTIONS(676),
    [anon_sym_GT] = ACTIONS(686),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT_EQ] = ACTIONS(676),
    [anon_sym_LT_EQ] = ACTIONS(676),
    [anon_sym_AT] = ACTIONS(676),
    [anon_sym__] = ACTIONS(686),
    [anon_sym_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(676),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(676),
    [anon_sym_COMMA] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(676),
    [anon_sym_DASH_GT] = ACTIONS(676),
    [anon_sym_POUND] = ACTIONS(676),
    [anon_sym_SQUOTE] = ACTIONS(674),
    [anon_sym_as] = ACTIONS(674),
    [anon_sym_async] = ACTIONS(674),
    [anon_sym_await] = ACTIONS(674),
    [anon_sym_break] = ACTIONS(674),
    [anon_sym_const] = ACTIONS(674),
    [anon_sym_continue] = ACTIONS(674),
    [anon_sym_default] = ACTIONS(674),
    [anon_sym_enum] = ACTIONS(674),
    [anon_sym_fn] = ACTIONS(674),
    [anon_sym_for] = ACTIONS(674),
    [anon_sym_gen] = ACTIONS(674),
    [anon_sym_if] = ACTIONS(674),
    [anon_sym_impl] = ACTIONS(674),
    [anon_sym_let] = ACTIONS(674),
    [anon_sym_loop] = ACTIONS(674),
    [anon_sym_match] = ACTIONS(674),
    [anon_sym_mod] = ACTIONS(674),
    [anon_sym_pub] = ACTIONS(674),
    [anon_sym_return] = ACTIONS(674),
    [anon_sym_static] = ACTIONS(674),
    [anon_sym_struct] = ACTIONS(674),
    [anon_sym_trait] = ACTIONS(674),
    [anon_sym_type] = ACTIONS(674),
    [anon_sym_union] = ACTIONS(674),
    [anon_sym_unsafe] = ACTIONS(674),
    [anon_sym_use] = ACTIONS(674),
    [anon_sym_where] = ACTIONS(674),
    [anon_sym_while] = ACTIONS(674),
    [sym_mutable_specifier] = ACTIONS(674),
    [sym_integer_literal] = ACTIONS(690),
    [aux_sym_string_literal_token1] = ACTIONS(692),
    [sym_char_literal] = ACTIONS(690),
    [anon_sym_true] = ACTIONS(694),
    [anon_sym_false] = ACTIONS(694),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(674),
    [sym_super] = ACTIONS(674),
    [sym_crate] = ACTIONS(674),
    [sym__raw_string_literal_start] = ACTIONS(696),
    [sym_float_literal] = ACTIONS(690),
  },
  [129] = {
    [sym_delim_token_tree] = STATE(198),
    [sym__delim_tokens] = STATE(206),
    [sym__non_delim_token] = STATE(198),
    [sym__literal] = STATE(214),
    [sym_string_literal] = STATE(199),
    [sym_raw_string_literal] = STATE(199),
    [sym_boolean_literal] = STATE(199),
    [sym_line_comment] = STATE(129),
    [sym_block_comment] = STATE(129),
    [aux_sym__non_special_token_repeat1] = STATE(157),
    [aux_sym_delim_token_tree_repeat1] = STATE(87),
    [sym_identifier] = ACTIONS(674),
    [anon_sym_SEMI] = ACTIONS(676),
    [anon_sym_LPAREN] = ACTIONS(678),
    [anon_sym_LBRACK] = ACTIONS(680),
    [anon_sym_LBRACE] = ACTIONS(682),
    [anon_sym_RBRACE] = ACTIONS(740),
    [anon_sym_EQ_GT] = ACTIONS(676),
    [anon_sym_COLON] = ACTIONS(686),
    [anon_sym_DOLLAR] = ACTIONS(688),
    [anon_sym_PLUS] = ACTIONS(686),
    [anon_sym_STAR] = ACTIONS(686),
    [anon_sym_QMARK] = ACTIONS(676),
    [anon_sym_u8] = ACTIONS(674),
    [anon_sym_i8] = ACTIONS(674),
    [anon_sym_u16] = ACTIONS(674),
    [anon_sym_i16] = ACTIONS(674),
    [anon_sym_u32] = ACTIONS(674),
    [anon_sym_i32] = ACTIONS(674),
    [anon_sym_u64] = ACTIONS(674),
    [anon_sym_i64] = ACTIONS(674),
    [anon_sym_u128] = ACTIONS(674),
    [anon_sym_i128] = ACTIONS(674),
    [anon_sym_isize] = ACTIONS(674),
    [anon_sym_usize] = ACTIONS(674),
    [anon_sym_f32] = ACTIONS(674),
    [anon_sym_f64] = ACTIONS(674),
    [anon_sym_bool] = ACTIONS(674),
    [anon_sym_str] = ACTIONS(674),
    [anon_sym_char] = ACTIONS(674),
    [anon_sym_DASH] = ACTIONS(686),
    [anon_sym_SLASH] = ACTIONS(686),
    [anon_sym_PERCENT] = ACTIONS(686),
    [anon_sym_CARET] = ACTIONS(686),
    [anon_sym_BANG] = ACTIONS(686),
    [anon_sym_AMP] = ACTIONS(686),
    [anon_sym_PIPE] = ACTIONS(686),
    [anon_sym_AMP_AMP] = ACTIONS(676),
    [anon_sym_PIPE_PIPE] = ACTIONS(676),
    [anon_sym_LT_LT] = ACTIONS(686),
    [anon_sym_GT_GT] = ACTIONS(686),
    [anon_sym_PLUS_EQ] = ACTIONS(676),
    [anon_sym_DASH_EQ] = ACTIONS(676),
    [anon_sym_STAR_EQ] = ACTIONS(676),
    [anon_sym_SLASH_EQ] = ACTIONS(676),
    [anon_sym_PERCENT_EQ] = ACTIONS(676),
    [anon_sym_CARET_EQ] = ACTIONS(676),
    [anon_sym_AMP_EQ] = ACTIONS(676),
    [anon_sym_PIPE_EQ] = ACTIONS(676),
    [anon_sym_LT_LT_EQ] = ACTIONS(676),
    [anon_sym_GT_GT_EQ] = ACTIONS(676),
    [anon_sym_EQ] = ACTIONS(686),
    [anon_sym_EQ_EQ] = ACTIONS(676),
    [anon_sym_BANG_EQ] = ACTIONS(676),
    [anon_sym_GT] = ACTIONS(686),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT_EQ] = ACTIONS(676),
    [anon_sym_LT_EQ] = ACTIONS(676),
    [anon_sym_AT] = ACTIONS(676),
    [anon_sym__] = ACTIONS(686),
    [anon_sym_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(676),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(676),
    [anon_sym_COMMA] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(676),
    [anon_sym_DASH_GT] = ACTIONS(676),
    [anon_sym_POUND] = ACTIONS(676),
    [anon_sym_SQUOTE] = ACTIONS(674),
    [anon_sym_as] = ACTIONS(674),
    [anon_sym_async] = ACTIONS(674),
    [anon_sym_await] = ACTIONS(674),
    [anon_sym_break] = ACTIONS(674),
    [anon_sym_const] = ACTIONS(674),
    [anon_sym_continue] = ACTIONS(674),
    [anon_sym_default] = ACTIONS(674),
    [anon_sym_enum] = ACTIONS(674),
    [anon_sym_fn] = ACTIONS(674),
    [anon_sym_for] = ACTIONS(674),
    [anon_sym_gen] = ACTIONS(674),
    [anon_sym_if] = ACTIONS(674),
    [anon_sym_impl] = ACTIONS(674),
    [anon_sym_let] = ACTIONS(674),
    [anon_sym_loop] = ACTIONS(674),
    [anon_sym_match] = ACTIONS(674),
    [anon_sym_mod] = ACTIONS(674),
    [anon_sym_pub] = ACTIONS(674),
    [anon_sym_return] = ACTIONS(674),
    [anon_sym_static] = ACTIONS(674),
    [anon_sym_struct] = ACTIONS(674),
    [anon_sym_trait] = ACTIONS(674),
    [anon_sym_type] = ACTIONS(674),
    [anon_sym_union] = ACTIONS(674),
    [anon_sym_unsafe] = ACTIONS(674),
    [anon_sym_use] = ACTIONS(674),
    [anon_sym_where] = ACTIONS(674),
    [anon_sym_while] = ACTIONS(674),
    [sym_mutable_specifier] = ACTIONS(674),
    [sym_integer_literal] = ACTIONS(690),
    [aux_sym_string_literal_token1] = ACTIONS(692),
    [sym_char_literal] = ACTIONS(690),
    [anon_sym_true] = ACTIONS(694),
    [anon_sym_false] = ACTIONS(694),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(674),
    [sym_super] = ACTIONS(674),
    [sym_crate] = ACTIONS(674),
    [sym__raw_string_literal_start] = ACTIONS(696),
    [sym_float_literal] = ACTIONS(690),
  },
  [130] = {
    [sym_token_tree] = STATE(152),
    [sym_token_repetition] = STATE(152),
    [sym__literal] = STATE(152),
    [sym_string_literal] = STATE(177),
    [sym_raw_string_literal] = STATE(177),
    [sym_boolean_literal] = STATE(177),
    [sym_line_comment] = STATE(130),
    [sym_block_comment] = STATE(130),
    [aux_sym_token_tree_repeat1] = STATE(88),
    [aux_sym__non_special_token_repeat1] = STATE(143),
    [sym_identifier] = ACTIONS(700),
    [anon_sym_SEMI] = ACTIONS(567),
    [anon_sym_LPAREN] = ACTIONS(702),
    [anon_sym_LBRACK] = ACTIONS(704),
    [anon_sym_RBRACK] = ACTIONS(738),
    [anon_sym_LBRACE] = ACTIONS(708),
    [anon_sym_EQ_GT] = ACTIONS(567),
    [anon_sym_COLON] = ACTIONS(577),
    [anon_sym_DOLLAR] = ACTIONS(710),
    [anon_sym_PLUS] = ACTIONS(577),
    [anon_sym_STAR] = ACTIONS(577),
    [anon_sym_QMARK] = ACTIONS(567),
    [anon_sym_u8] = ACTIONS(700),
    [anon_sym_i8] = ACTIONS(700),
    [anon_sym_u16] = ACTIONS(700),
    [anon_sym_i16] = ACTIONS(700),
    [anon_sym_u32] = ACTIONS(700),
    [anon_sym_i32] = ACTIONS(700),
    [anon_sym_u64] = ACTIONS(700),
    [anon_sym_i64] = ACTIONS(700),
    [anon_sym_u128] = ACTIONS(700),
    [anon_sym_i128] = ACTIONS(700),
    [anon_sym_isize] = ACTIONS(700),
    [anon_sym_usize] = ACTIONS(700),
    [anon_sym_f32] = ACTIONS(700),
    [anon_sym_f64] = ACTIONS(700),
    [anon_sym_bool] = ACTIONS(700),
    [anon_sym_str] = ACTIONS(700),
    [anon_sym_char] = ACTIONS(700),
    [anon_sym_DASH] = ACTIONS(577),
    [anon_sym_SLASH] = ACTIONS(577),
    [anon_sym_PERCENT] = ACTIONS(577),
    [anon_sym_CARET] = ACTIONS(577),
    [anon_sym_BANG] = ACTIONS(577),
    [anon_sym_AMP] = ACTIONS(577),
    [anon_sym_PIPE] = ACTIONS(577),
    [anon_sym_AMP_AMP] = ACTIONS(567),
    [anon_sym_PIPE_PIPE] = ACTIONS(567),
    [anon_sym_LT_LT] = ACTIONS(577),
    [anon_sym_GT_GT] = ACTIONS(577),
    [anon_sym_PLUS_EQ] = ACTIONS(567),
    [anon_sym_DASH_EQ] = ACTIONS(567),
    [anon_sym_STAR_EQ] = ACTIONS(567),
    [anon_sym_SLASH_EQ] = ACTIONS(567),
    [anon_sym_PERCENT_EQ] = ACTIONS(567),
    [anon_sym_CARET_EQ] = ACTIONS(567),
    [anon_sym_AMP_EQ] = ACTIONS(567),
    [anon_sym_PIPE_EQ] = ACTIONS(567),
    [anon_sym_LT_LT_EQ] = ACTIONS(567),
    [anon_sym_GT_GT_EQ] = ACTIONS(567),
    [anon_sym_EQ] = ACTIONS(577),
    [anon_sym_EQ_EQ] = ACTIONS(567),
    [anon_sym_BANG_EQ] = ACTIONS(567),
    [anon_sym_GT] = ACTIONS(577),
    [anon_sym_LT] = ACTIONS(577),
    [anon_sym_GT_EQ] = ACTIONS(567),
    [anon_sym_LT_EQ] = ACTIONS(567),
    [anon_sym_AT] = ACTIONS(567),
    [anon_sym__] = ACTIONS(577),
    [anon_sym_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(567),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(567),
    [anon_sym_COMMA] = ACTIONS(567),
    [anon_sym_COLON_COLON] = ACTIONS(567),
    [anon_sym_DASH_GT] = ACTIONS(567),
    [anon_sym_POUND] = ACTIONS(567),
    [anon_sym_SQUOTE] = ACTIONS(700),
    [anon_sym_as] = ACTIONS(700),
    [anon_sym_async] = ACTIONS(700),
    [anon_sym_await] = ACTIONS(700),
    [anon_sym_break] = ACTIONS(700),
    [anon_sym_const] = ACTIONS(700),
    [anon_sym_continue] = ACTIONS(700),
    [anon_sym_default] = ACTIONS(700),
    [anon_sym_enum] = ACTIONS(700),
    [anon_sym_fn] = ACTIONS(700),
    [anon_sym_for] = ACTIONS(700),
    [anon_sym_gen] = ACTIONS(700),
    [anon_sym_if] = ACTIONS(700),
    [anon_sym_impl] = ACTIONS(700),
    [anon_sym_let] = ACTIONS(700),
    [anon_sym_loop] = ACTIONS(700),
    [anon_sym_match] = ACTIONS(700),
    [anon_sym_mod] = ACTIONS(700),
    [anon_sym_pub] = ACTIONS(700),
    [anon_sym_return] = ACTIONS(700),
    [anon_sym_static] = ACTIONS(700),
    [anon_sym_struct] = ACTIONS(700),
    [anon_sym_trait] = ACTIONS(700),
    [anon_sym_type] = ACTIONS(700),
    [anon_sym_union] = ACTIONS(700),
    [anon_sym_unsafe] = ACTIONS(700),
    [anon_sym_use] = ACTIONS(700),
    [anon_sym_where] = ACTIONS(700),
    [anon_sym_while] = ACTIONS(700),
    [sym_mutable_specifier] = ACTIONS(700),
    [sym_integer_literal] = ACTIONS(581),
    [aux_sym_string_literal_token1] = ACTIONS(583),
    [sym_char_literal] = ACTIONS(581),
    [anon_sym_true] = ACTIONS(585),
    [anon_sym_false] = ACTIONS(585),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(700),
    [sym_super] = ACTIONS(700),
    [sym_crate] = ACTIONS(700),
    [sym_metavariable] = ACTIONS(712),
    [sym__raw_string_literal_start] = ACTIONS(589),
    [sym_float_literal] = ACTIONS(581),
  },
  [131] = {
    [sym_token_tree] = STATE(152),
    [sym_token_repetition] = STATE(152),
    [sym__literal] = STATE(152),
    [sym_string_literal] = STATE(177),
    [sym_raw_string_literal] = STATE(177),
    [sym_boolean_literal] = STATE(177),
    [sym_line_comment] = STATE(131),
    [sym_block_comment] = STATE(131),
    [aux_sym_token_tree_repeat1] = STATE(89),
    [aux_sym__non_special_token_repeat1] = STATE(143),
    [sym_identifier] = ACTIONS(700),
    [anon_sym_SEMI] = ACTIONS(567),
    [anon_sym_LPAREN] = ACTIONS(702),
    [anon_sym_LBRACK] = ACTIONS(704),
    [anon_sym_LBRACE] = ACTIONS(708),
    [anon_sym_RBRACE] = ACTIONS(738),
    [anon_sym_EQ_GT] = ACTIONS(567),
    [anon_sym_COLON] = ACTIONS(577),
    [anon_sym_DOLLAR] = ACTIONS(710),
    [anon_sym_PLUS] = ACTIONS(577),
    [anon_sym_STAR] = ACTIONS(577),
    [anon_sym_QMARK] = ACTIONS(567),
    [anon_sym_u8] = ACTIONS(700),
    [anon_sym_i8] = ACTIONS(700),
    [anon_sym_u16] = ACTIONS(700),
    [anon_sym_i16] = ACTIONS(700),
    [anon_sym_u32] = ACTIONS(700),
    [anon_sym_i32] = ACTIONS(700),
    [anon_sym_u64] = ACTIONS(700),
    [anon_sym_i64] = ACTIONS(700),
    [anon_sym_u128] = ACTIONS(700),
    [anon_sym_i128] = ACTIONS(700),
    [anon_sym_isize] = ACTIONS(700),
    [anon_sym_usize] = ACTIONS(700),
    [anon_sym_f32] = ACTIONS(700),
    [anon_sym_f64] = ACTIONS(700),
    [anon_sym_bool] = ACTIONS(700),
    [anon_sym_str] = ACTIONS(700),
    [anon_sym_char] = ACTIONS(700),
    [anon_sym_DASH] = ACTIONS(577),
    [anon_sym_SLASH] = ACTIONS(577),
    [anon_sym_PERCENT] = ACTIONS(577),
    [anon_sym_CARET] = ACTIONS(577),
    [anon_sym_BANG] = ACTIONS(577),
    [anon_sym_AMP] = ACTIONS(577),
    [anon_sym_PIPE] = ACTIONS(577),
    [anon_sym_AMP_AMP] = ACTIONS(567),
    [anon_sym_PIPE_PIPE] = ACTIONS(567),
    [anon_sym_LT_LT] = ACTIONS(577),
    [anon_sym_GT_GT] = ACTIONS(577),
    [anon_sym_PLUS_EQ] = ACTIONS(567),
    [anon_sym_DASH_EQ] = ACTIONS(567),
    [anon_sym_STAR_EQ] = ACTIONS(567),
    [anon_sym_SLASH_EQ] = ACTIONS(567),
    [anon_sym_PERCENT_EQ] = ACTIONS(567),
    [anon_sym_CARET_EQ] = ACTIONS(567),
    [anon_sym_AMP_EQ] = ACTIONS(567),
    [anon_sym_PIPE_EQ] = ACTIONS(567),
    [anon_sym_LT_LT_EQ] = ACTIONS(567),
    [anon_sym_GT_GT_EQ] = ACTIONS(567),
    [anon_sym_EQ] = ACTIONS(577),
    [anon_sym_EQ_EQ] = ACTIONS(567),
    [anon_sym_BANG_EQ] = ACTIONS(567),
    [anon_sym_GT] = ACTIONS(577),
    [anon_sym_LT] = ACTIONS(577),
    [anon_sym_GT_EQ] = ACTIONS(567),
    [anon_sym_LT_EQ] = ACTIONS(567),
    [anon_sym_AT] = ACTIONS(567),
    [anon_sym__] = ACTIONS(577),
    [anon_sym_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(567),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(567),
    [anon_sym_COMMA] = ACTIONS(567),
    [anon_sym_COLON_COLON] = ACTIONS(567),
    [anon_sym_DASH_GT] = ACTIONS(567),
    [anon_sym_POUND] = ACTIONS(567),
    [anon_sym_SQUOTE] = ACTIONS(700),
    [anon_sym_as] = ACTIONS(700),
    [anon_sym_async] = ACTIONS(700),
    [anon_sym_await] = ACTIONS(700),
    [anon_sym_break] = ACTIONS(700),
    [anon_sym_const] = ACTIONS(700),
    [anon_sym_continue] = ACTIONS(700),
    [anon_sym_default] = ACTIONS(700),
    [anon_sym_enum] = ACTIONS(700),
    [anon_sym_fn] = ACTIONS(700),
    [anon_sym_for] = ACTIONS(700),
    [anon_sym_gen] = ACTIONS(700),
    [anon_sym_if] = ACTIONS(700),
    [anon_sym_impl] = ACTIONS(700),
    [anon_sym_let] = ACTIONS(700),
    [anon_sym_loop] = ACTIONS(700),
    [anon_sym_match] = ACTIONS(700),
    [anon_sym_mod] = ACTIONS(700),
    [anon_sym_pub] = ACTIONS(700),
    [anon_sym_return] = ACTIONS(700),
    [anon_sym_static] = ACTIONS(700),
    [anon_sym_struct] = ACTIONS(700),
    [anon_sym_trait] = ACTIONS(700),
    [anon_sym_type] = ACTIONS(700),
    [anon_sym_union] = ACTIONS(700),
    [anon_sym_unsafe] = ACTIONS(700),
    [anon_sym_use] = ACTIONS(700),
    [anon_sym_where] = ACTIONS(700),
    [anon_sym_while] = ACTIONS(700),
    [sym_mutable_specifier] = ACTIONS(700),
    [sym_integer_literal] = ACTIONS(581),
    [aux_sym_string_literal_token1] = ACTIONS(583),
    [sym_char_literal] = ACTIONS(581),
    [anon_sym_true] = ACTIONS(585),
    [anon_sym_false] = ACTIONS(585),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(700),
    [sym_super] = ACTIONS(700),
    [sym_crate] = ACTIONS(700),
    [sym_metavariable] = ACTIONS(712),
    [sym__raw_string_literal_start] = ACTIONS(589),
    [sym_float_literal] = ACTIONS(581),
  },
  [132] = {
    [sym_delim_token_tree] = STATE(198),
    [sym__delim_tokens] = STATE(206),
    [sym__non_delim_token] = STATE(198),
    [sym__literal] = STATE(214),
    [sym_string_literal] = STATE(199),
    [sym_raw_string_literal] = STATE(199),
    [sym_boolean_literal] = STATE(199),
    [sym_line_comment] = STATE(132),
    [sym_block_comment] = STATE(132),
    [aux_sym__non_special_token_repeat1] = STATE(157),
    [aux_sym_delim_token_tree_repeat1] = STATE(83),
    [sym_identifier] = ACTIONS(674),
    [anon_sym_SEMI] = ACTIONS(676),
    [anon_sym_LPAREN] = ACTIONS(678),
    [anon_sym_LBRACK] = ACTIONS(680),
    [anon_sym_RBRACK] = ACTIONS(720),
    [anon_sym_LBRACE] = ACTIONS(682),
    [anon_sym_EQ_GT] = ACTIONS(676),
    [anon_sym_COLON] = ACTIONS(686),
    [anon_sym_DOLLAR] = ACTIONS(688),
    [anon_sym_PLUS] = ACTIONS(686),
    [anon_sym_STAR] = ACTIONS(686),
    [anon_sym_QMARK] = ACTIONS(676),
    [anon_sym_u8] = ACTIONS(674),
    [anon_sym_i8] = ACTIONS(674),
    [anon_sym_u16] = ACTIONS(674),
    [anon_sym_i16] = ACTIONS(674),
    [anon_sym_u32] = ACTIONS(674),
    [anon_sym_i32] = ACTIONS(674),
    [anon_sym_u64] = ACTIONS(674),
    [anon_sym_i64] = ACTIONS(674),
    [anon_sym_u128] = ACTIONS(674),
    [anon_sym_i128] = ACTIONS(674),
    [anon_sym_isize] = ACTIONS(674),
    [anon_sym_usize] = ACTIONS(674),
    [anon_sym_f32] = ACTIONS(674),
    [anon_sym_f64] = ACTIONS(674),
    [anon_sym_bool] = ACTIONS(674),
    [anon_sym_str] = ACTIONS(674),
    [anon_sym_char] = ACTIONS(674),
    [anon_sym_DASH] = ACTIONS(686),
    [anon_sym_SLASH] = ACTIONS(686),
    [anon_sym_PERCENT] = ACTIONS(686),
    [anon_sym_CARET] = ACTIONS(686),
    [anon_sym_BANG] = ACTIONS(686),
    [anon_sym_AMP] = ACTIONS(686),
    [anon_sym_PIPE] = ACTIONS(686),
    [anon_sym_AMP_AMP] = ACTIONS(676),
    [anon_sym_PIPE_PIPE] = ACTIONS(676),
    [anon_sym_LT_LT] = ACTIONS(686),
    [anon_sym_GT_GT] = ACTIONS(686),
    [anon_sym_PLUS_EQ] = ACTIONS(676),
    [anon_sym_DASH_EQ] = ACTIONS(676),
    [anon_sym_STAR_EQ] = ACTIONS(676),
    [anon_sym_SLASH_EQ] = ACTIONS(676),
    [anon_sym_PERCENT_EQ] = ACTIONS(676),
    [anon_sym_CARET_EQ] = ACTIONS(676),
    [anon_sym_AMP_EQ] = ACTIONS(676),
    [anon_sym_PIPE_EQ] = ACTIONS(676),
    [anon_sym_LT_LT_EQ] = ACTIONS(676),
    [anon_sym_GT_GT_EQ] = ACTIONS(676),
    [anon_sym_EQ] = ACTIONS(686),
    [anon_sym_EQ_EQ] = ACTIONS(676),
    [anon_sym_BANG_EQ] = ACTIONS(676),
    [anon_sym_GT] = ACTIONS(686),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT_EQ] = ACTIONS(676),
    [anon_sym_LT_EQ] = ACTIONS(676),
    [anon_sym_AT] = ACTIONS(676),
    [anon_sym__] = ACTIONS(686),
    [anon_sym_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(676),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(676),
    [anon_sym_COMMA] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(676),
    [anon_sym_DASH_GT] = ACTIONS(676),
    [anon_sym_POUND] = ACTIONS(676),
    [anon_sym_SQUOTE] = ACTIONS(674),
    [anon_sym_as] = ACTIONS(674),
    [anon_sym_async] = ACTIONS(674),
    [anon_sym_await] = ACTIONS(674),
    [anon_sym_break] = ACTIONS(674),
    [anon_sym_const] = ACTIONS(674),
    [anon_sym_continue] = ACTIONS(674),
    [anon_sym_default] = ACTIONS(674),
    [anon_sym_enum] = ACTIONS(674),
    [anon_sym_fn] = ACTIONS(674),
    [anon_sym_for] = ACTIONS(674),
    [anon_sym_gen] = ACTIONS(674),
    [anon_sym_if] = ACTIONS(674),
    [anon_sym_impl] = ACTIONS(674),
    [anon_sym_let] = ACTIONS(674),
    [anon_sym_loop] = ACTIONS(674),
    [anon_sym_match] = ACTIONS(674),
    [anon_sym_mod] = ACTIONS(674),
    [anon_sym_pub] = ACTIONS(674),
    [anon_sym_return] = ACTIONS(674),
    [anon_sym_static] = ACTIONS(674),
    [anon_sym_struct] = ACTIONS(674),
    [anon_sym_trait] = ACTIONS(674),
    [anon_sym_type] = ACTIONS(674),
    [anon_sym_union] = ACTIONS(674),
    [anon_sym_unsafe] = ACTIONS(674),
    [anon_sym_use] = ACTIONS(674),
    [anon_sym_where] = ACTIONS(674),
    [anon_sym_while] = ACTIONS(674),
    [sym_mutable_specifier] = ACTIONS(674),
    [sym_integer_literal] = ACTIONS(690),
    [aux_sym_string_literal_token1] = ACTIONS(692),
    [sym_char_literal] = ACTIONS(690),
    [anon_sym_true] = ACTIONS(694),
    [anon_sym_false] = ACTIONS(694),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(674),
    [sym_super] = ACTIONS(674),
    [sym_crate] = ACTIONS(674),
    [sym__raw_string_literal_start] = ACTIONS(696),
    [sym_float_literal] = ACTIONS(690),
  },
  [133] = {
    [sym_token_tree] = STATE(152),
    [sym_token_repetition] = STATE(152),
    [sym__literal] = STATE(152),
    [sym_string_literal] = STATE(177),
    [sym_raw_string_literal] = STATE(177),
    [sym_boolean_literal] = STATE(177),
    [sym_line_comment] = STATE(133),
    [sym_block_comment] = STATE(133),
    [aux_sym_token_tree_repeat1] = STATE(82),
    [aux_sym__non_special_token_repeat1] = STATE(143),
    [sym_identifier] = ACTIONS(700),
    [anon_sym_SEMI] = ACTIONS(567),
    [anon_sym_LPAREN] = ACTIONS(702),
    [anon_sym_RPAREN] = ACTIONS(706),
    [anon_sym_LBRACK] = ACTIONS(704),
    [anon_sym_LBRACE] = ACTIONS(708),
    [anon_sym_EQ_GT] = ACTIONS(567),
    [anon_sym_COLON] = ACTIONS(577),
    [anon_sym_DOLLAR] = ACTIONS(710),
    [anon_sym_PLUS] = ACTIONS(577),
    [anon_sym_STAR] = ACTIONS(577),
    [anon_sym_QMARK] = ACTIONS(567),
    [anon_sym_u8] = ACTIONS(700),
    [anon_sym_i8] = ACTIONS(700),
    [anon_sym_u16] = ACTIONS(700),
    [anon_sym_i16] = ACTIONS(700),
    [anon_sym_u32] = ACTIONS(700),
    [anon_sym_i32] = ACTIONS(700),
    [anon_sym_u64] = ACTIONS(700),
    [anon_sym_i64] = ACTIONS(700),
    [anon_sym_u128] = ACTIONS(700),
    [anon_sym_i128] = ACTIONS(700),
    [anon_sym_isize] = ACTIONS(700),
    [anon_sym_usize] = ACTIONS(700),
    [anon_sym_f32] = ACTIONS(700),
    [anon_sym_f64] = ACTIONS(700),
    [anon_sym_bool] = ACTIONS(700),
    [anon_sym_str] = ACTIONS(700),
    [anon_sym_char] = ACTIONS(700),
    [anon_sym_DASH] = ACTIONS(577),
    [anon_sym_SLASH] = ACTIONS(577),
    [anon_sym_PERCENT] = ACTIONS(577),
    [anon_sym_CARET] = ACTIONS(577),
    [anon_sym_BANG] = ACTIONS(577),
    [anon_sym_AMP] = ACTIONS(577),
    [anon_sym_PIPE] = ACTIONS(577),
    [anon_sym_AMP_AMP] = ACTIONS(567),
    [anon_sym_PIPE_PIPE] = ACTIONS(567),
    [anon_sym_LT_LT] = ACTIONS(577),
    [anon_sym_GT_GT] = ACTIONS(577),
    [anon_sym_PLUS_EQ] = ACTIONS(567),
    [anon_sym_DASH_EQ] = ACTIONS(567),
    [anon_sym_STAR_EQ] = ACTIONS(567),
    [anon_sym_SLASH_EQ] = ACTIONS(567),
    [anon_sym_PERCENT_EQ] = ACTIONS(567),
    [anon_sym_CARET_EQ] = ACTIONS(567),
    [anon_sym_AMP_EQ] = ACTIONS(567),
    [anon_sym_PIPE_EQ] = ACTIONS(567),
    [anon_sym_LT_LT_EQ] = ACTIONS(567),
    [anon_sym_GT_GT_EQ] = ACTIONS(567),
    [anon_sym_EQ] = ACTIONS(577),
    [anon_sym_EQ_EQ] = ACTIONS(567),
    [anon_sym_BANG_EQ] = ACTIONS(567),
    [anon_sym_GT] = ACTIONS(577),
    [anon_sym_LT] = ACTIONS(577),
    [anon_sym_GT_EQ] = ACTIONS(567),
    [anon_sym_LT_EQ] = ACTIONS(567),
    [anon_sym_AT] = ACTIONS(567),
    [anon_sym__] = ACTIONS(577),
    [anon_sym_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(567),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(567),
    [anon_sym_COMMA] = ACTIONS(567),
    [anon_sym_COLON_COLON] = ACTIONS(567),
    [anon_sym_DASH_GT] = ACTIONS(567),
    [anon_sym_POUND] = ACTIONS(567),
    [anon_sym_SQUOTE] = ACTIONS(700),
    [anon_sym_as] = ACTIONS(700),
    [anon_sym_async] = ACTIONS(700),
    [anon_sym_await] = ACTIONS(700),
    [anon_sym_break] = ACTIONS(700),
    [anon_sym_const] = ACTIONS(700),
    [anon_sym_continue] = ACTIONS(700),
    [anon_sym_default] = ACTIONS(700),
    [anon_sym_enum] = ACTIONS(700),
    [anon_sym_fn] = ACTIONS(700),
    [anon_sym_for] = ACTIONS(700),
    [anon_sym_gen] = ACTIONS(700),
    [anon_sym_if] = ACTIONS(700),
    [anon_sym_impl] = ACTIONS(700),
    [anon_sym_let] = ACTIONS(700),
    [anon_sym_loop] = ACTIONS(700),
    [anon_sym_match] = ACTIONS(700),
    [anon_sym_mod] = ACTIONS(700),
    [anon_sym_pub] = ACTIONS(700),
    [anon_sym_return] = ACTIONS(700),
    [anon_sym_static] = ACTIONS(700),
    [anon_sym_struct] = ACTIONS(700),
    [anon_sym_trait] = ACTIONS(700),
    [anon_sym_type] = ACTIONS(700),
    [anon_sym_union] = ACTIONS(700),
    [anon_sym_unsafe] = ACTIONS(700),
    [anon_sym_use] = ACTIONS(700),
    [anon_sym_where] = ACTIONS(700),
    [anon_sym_while] = ACTIONS(700),
    [sym_mutable_specifier] = ACTIONS(700),
    [sym_integer_literal] = ACTIONS(581),
    [aux_sym_string_literal_token1] = ACTIONS(583),
    [sym_char_literal] = ACTIONS(581),
    [anon_sym_true] = ACTIONS(585),
    [anon_sym_false] = ACTIONS(585),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(700),
    [sym_super] = ACTIONS(700),
    [sym_crate] = ACTIONS(700),
    [sym_metavariable] = ACTIONS(712),
    [sym__raw_string_literal_start] = ACTIONS(589),
    [sym_float_literal] = ACTIONS(581),
  },
  [134] = {
    [sym_attribute_item] = STATE(1011),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1561),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(134),
    [sym_block_comment] = STATE(134),
    [aux_sym_enum_variant_list_repeat1] = STATE(139),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_RBRACK] = ACTIONS(744),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COMMA] = ACTIONS(746),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(748),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [135] = {
    [sym_attribute_item] = STATE(1011),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1560),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(135),
    [sym_block_comment] = STATE(135),
    [aux_sym_enum_variant_list_repeat1] = STATE(144),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_RBRACK] = ACTIONS(750),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COMMA] = ACTIONS(752),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(748),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [136] = {
    [sym_line_comment] = STATE(136),
    [sym_block_comment] = STATE(136),
    [aux_sym__non_special_token_repeat1] = STATE(137),
    [sym_identifier] = ACTIONS(754),
    [anon_sym_SEMI] = ACTIONS(567),
    [anon_sym_LPAREN] = ACTIONS(756),
    [anon_sym_RPAREN] = ACTIONS(756),
    [anon_sym_LBRACK] = ACTIONS(756),
    [anon_sym_RBRACK] = ACTIONS(756),
    [anon_sym_LBRACE] = ACTIONS(756),
    [anon_sym_RBRACE] = ACTIONS(756),
    [anon_sym_EQ_GT] = ACTIONS(567),
    [anon_sym_COLON] = ACTIONS(577),
    [anon_sym_DOLLAR] = ACTIONS(754),
    [anon_sym_PLUS] = ACTIONS(577),
    [anon_sym_STAR] = ACTIONS(577),
    [anon_sym_QMARK] = ACTIONS(567),
    [anon_sym_u8] = ACTIONS(754),
    [anon_sym_i8] = ACTIONS(754),
    [anon_sym_u16] = ACTIONS(754),
    [anon_sym_i16] = ACTIONS(754),
    [anon_sym_u32] = ACTIONS(754),
    [anon_sym_i32] = ACTIONS(754),
    [anon_sym_u64] = ACTIONS(754),
    [anon_sym_i64] = ACTIONS(754),
    [anon_sym_u128] = ACTIONS(754),
    [anon_sym_i128] = ACTIONS(754),
    [anon_sym_isize] = ACTIONS(754),
    [anon_sym_usize] = ACTIONS(754),
    [anon_sym_f32] = ACTIONS(754),
    [anon_sym_f64] = ACTIONS(754),
    [anon_sym_bool] = ACTIONS(754),
    [anon_sym_str] = ACTIONS(754),
    [anon_sym_char] = ACTIONS(754),
    [anon_sym_DASH] = ACTIONS(577),
    [anon_sym_SLASH] = ACTIONS(577),
    [anon_sym_PERCENT] = ACTIONS(577),
    [anon_sym_CARET] = ACTIONS(577),
    [anon_sym_BANG] = ACTIONS(577),
    [anon_sym_AMP] = ACTIONS(577),
    [anon_sym_PIPE] = ACTIONS(577),
    [anon_sym_AMP_AMP] = ACTIONS(567),
    [anon_sym_PIPE_PIPE] = ACTIONS(567),
    [anon_sym_LT_LT] = ACTIONS(577),
    [anon_sym_GT_GT] = ACTIONS(577),
    [anon_sym_PLUS_EQ] = ACTIONS(567),
    [anon_sym_DASH_EQ] = ACTIONS(567),
    [anon_sym_STAR_EQ] = ACTIONS(567),
    [anon_sym_SLASH_EQ] = ACTIONS(567),
    [anon_sym_PERCENT_EQ] = ACTIONS(567),
    [anon_sym_CARET_EQ] = ACTIONS(567),
    [anon_sym_AMP_EQ] = ACTIONS(567),
    [anon_sym_PIPE_EQ] = ACTIONS(567),
    [anon_sym_LT_LT_EQ] = ACTIONS(567),
    [anon_sym_GT_GT_EQ] = ACTIONS(567),
    [anon_sym_EQ] = ACTIONS(577),
    [anon_sym_EQ_EQ] = ACTIONS(567),
    [anon_sym_BANG_EQ] = ACTIONS(567),
    [anon_sym_GT] = ACTIONS(577),
    [anon_sym_LT] = ACTIONS(577),
    [anon_sym_GT_EQ] = ACTIONS(567),
    [anon_sym_LT_EQ] = ACTIONS(567),
    [anon_sym_AT] = ACTIONS(567),
    [anon_sym__] = ACTIONS(577),
    [anon_sym_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(567),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(567),
    [anon_sym_COMMA] = ACTIONS(567),
    [anon_sym_COLON_COLON] = ACTIONS(567),
    [anon_sym_DASH_GT] = ACTIONS(567),
    [anon_sym_POUND] = ACTIONS(567),
    [anon_sym_SQUOTE] = ACTIONS(754),
    [anon_sym_as] = ACTIONS(754),
    [anon_sym_async] = ACTIONS(754),
    [anon_sym_await] = ACTIONS(754),
    [anon_sym_break] = ACTIONS(754),
    [anon_sym_const] = ACTIONS(754),
    [anon_sym_continue] = ACTIONS(754),
    [anon_sym_default] = ACTIONS(754),
    [anon_sym_enum] = ACTIONS(754),
    [anon_sym_fn] = ACTIONS(754),
    [anon_sym_for] = ACTIONS(754),
    [anon_sym_gen] = ACTIONS(754),
    [anon_sym_if] = ACTIONS(754),
    [anon_sym_impl] = ACTIONS(754),
    [anon_sym_let] = ACTIONS(754),
    [anon_sym_loop] = ACTIONS(754),
    [anon_sym_match] = ACTIONS(754),
    [anon_sym_mod] = ACTIONS(754),
    [anon_sym_pub] = ACTIONS(754),
    [anon_sym_return] = ACTIONS(754),
    [anon_sym_static] = ACTIONS(754),
    [anon_sym_struct] = ACTIONS(754),
    [anon_sym_trait] = ACTIONS(754),
    [anon_sym_type] = ACTIONS(754),
    [anon_sym_union] = ACTIONS(754),
    [anon_sym_unsafe] = ACTIONS(754),
    [anon_sym_use] = ACTIONS(754),
    [anon_sym_where] = ACTIONS(754),
    [anon_sym_while] = ACTIONS(754),
    [sym_mutable_specifier] = ACTIONS(754),
    [sym_integer_literal] = ACTIONS(756),
    [aux_sym_string_literal_token1] = ACTIONS(756),
    [sym_char_literal] = ACTIONS(756),
    [anon_sym_true] = ACTIONS(754),
    [anon_sym_false] = ACTIONS(754),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(754),
    [sym_super] = ACTIONS(754),
    [sym_crate] = ACTIONS(754),
    [sym_metavariable] = ACTIONS(756),
    [sym__raw_string_literal_start] = ACTIONS(756),
    [sym_float_literal] = ACTIONS(756),
  },
  [137] = {
    [sym_line_comment] = STATE(137),
    [sym_block_comment] = STATE(137),
    [aux_sym__non_special_token_repeat1] = STATE(137),
    [sym_identifier] = ACTIONS(758),
    [anon_sym_SEMI] = ACTIONS(760),
    [anon_sym_LPAREN] = ACTIONS(763),
    [anon_sym_RPAREN] = ACTIONS(763),
    [anon_sym_LBRACK] = ACTIONS(763),
    [anon_sym_RBRACK] = ACTIONS(763),
    [anon_sym_LBRACE] = ACTIONS(763),
    [anon_sym_RBRACE] = ACTIONS(763),
    [anon_sym_EQ_GT] = ACTIONS(760),
    [anon_sym_COLON] = ACTIONS(765),
    [anon_sym_DOLLAR] = ACTIONS(758),
    [anon_sym_PLUS] = ACTIONS(765),
    [anon_sym_STAR] = ACTIONS(765),
    [anon_sym_QMARK] = ACTIONS(760),
    [anon_sym_u8] = ACTIONS(758),
    [anon_sym_i8] = ACTIONS(758),
    [anon_sym_u16] = ACTIONS(758),
    [anon_sym_i16] = ACTIONS(758),
    [anon_sym_u32] = ACTIONS(758),
    [anon_sym_i32] = ACTIONS(758),
    [anon_sym_u64] = ACTIONS(758),
    [anon_sym_i64] = ACTIONS(758),
    [anon_sym_u128] = ACTIONS(758),
    [anon_sym_i128] = ACTIONS(758),
    [anon_sym_isize] = ACTIONS(758),
    [anon_sym_usize] = ACTIONS(758),
    [anon_sym_f32] = ACTIONS(758),
    [anon_sym_f64] = ACTIONS(758),
    [anon_sym_bool] = ACTIONS(758),
    [anon_sym_str] = ACTIONS(758),
    [anon_sym_char] = ACTIONS(758),
    [anon_sym_DASH] = ACTIONS(765),
    [anon_sym_SLASH] = ACTIONS(765),
    [anon_sym_PERCENT] = ACTIONS(765),
    [anon_sym_CARET] = ACTIONS(765),
    [anon_sym_BANG] = ACTIONS(765),
    [anon_sym_AMP] = ACTIONS(765),
    [anon_sym_PIPE] = ACTIONS(765),
    [anon_sym_AMP_AMP] = ACTIONS(760),
    [anon_sym_PIPE_PIPE] = ACTIONS(760),
    [anon_sym_LT_LT] = ACTIONS(765),
    [anon_sym_GT_GT] = ACTIONS(765),
    [anon_sym_PLUS_EQ] = ACTIONS(760),
    [anon_sym_DASH_EQ] = ACTIONS(760),
    [anon_sym_STAR_EQ] = ACTIONS(760),
    [anon_sym_SLASH_EQ] = ACTIONS(760),
    [anon_sym_PERCENT_EQ] = ACTIONS(760),
    [anon_sym_CARET_EQ] = ACTIONS(760),
    [anon_sym_AMP_EQ] = ACTIONS(760),
    [anon_sym_PIPE_EQ] = ACTIONS(760),
    [anon_sym_LT_LT_EQ] = ACTIONS(760),
    [anon_sym_GT_GT_EQ] = ACTIONS(760),
    [anon_sym_EQ] = ACTIONS(765),
    [anon_sym_EQ_EQ] = ACTIONS(760),
    [anon_sym_BANG_EQ] = ACTIONS(760),
    [anon_sym_GT] = ACTIONS(765),
    [anon_sym_LT] = ACTIONS(765),
    [anon_sym_GT_EQ] = ACTIONS(760),
    [anon_sym_LT_EQ] = ACTIONS(760),
    [anon_sym_AT] = ACTIONS(760),
    [anon_sym__] = ACTIONS(765),
    [anon_sym_DOT] = ACTIONS(765),
    [anon_sym_DOT_DOT] = ACTIONS(765),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(760),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(760),
    [anon_sym_COMMA] = ACTIONS(760),
    [anon_sym_COLON_COLON] = ACTIONS(760),
    [anon_sym_DASH_GT] = ACTIONS(760),
    [anon_sym_POUND] = ACTIONS(760),
    [anon_sym_SQUOTE] = ACTIONS(758),
    [anon_sym_as] = ACTIONS(758),
    [anon_sym_async] = ACTIONS(758),
    [anon_sym_await] = ACTIONS(758),
    [anon_sym_break] = ACTIONS(758),
    [anon_sym_const] = ACTIONS(758),
    [anon_sym_continue] = ACTIONS(758),
    [anon_sym_default] = ACTIONS(758),
    [anon_sym_enum] = ACTIONS(758),
    [anon_sym_fn] = ACTIONS(758),
    [anon_sym_for] = ACTIONS(758),
    [anon_sym_gen] = ACTIONS(758),
    [anon_sym_if] = ACTIONS(758),
    [anon_sym_impl] = ACTIONS(758),
    [anon_sym_let] = ACTIONS(758),
    [anon_sym_loop] = ACTIONS(758),
    [anon_sym_match] = ACTIONS(758),
    [anon_sym_mod] = ACTIONS(758),
    [anon_sym_pub] = ACTIONS(758),
    [anon_sym_return] = ACTIONS(758),
    [anon_sym_static] = ACTIONS(758),
    [anon_sym_struct] = ACTIONS(758),
    [anon_sym_trait] = ACTIONS(758),
    [anon_sym_type] = ACTIONS(758),
    [anon_sym_union] = ACTIONS(758),
    [anon_sym_unsafe] = ACTIONS(758),
    [anon_sym_use] = ACTIONS(758),
    [anon_sym_where] = ACTIONS(758),
    [anon_sym_while] = ACTIONS(758),
    [sym_mutable_specifier] = ACTIONS(758),
    [sym_integer_literal] = ACTIONS(763),
    [aux_sym_string_literal_token1] = ACTIONS(763),
    [sym_char_literal] = ACTIONS(763),
    [anon_sym_true] = ACTIONS(758),
    [anon_sym_false] = ACTIONS(758),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(758),
    [sym_super] = ACTIONS(758),
    [sym_crate] = ACTIONS(758),
    [sym_metavariable] = ACTIONS(763),
    [sym__raw_string_literal_start] = ACTIONS(763),
    [sym_float_literal] = ACTIONS(763),
  },
  [138] = {
    [sym_attribute_item] = STATE(1011),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1589),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(138),
    [sym_block_comment] = STATE(138),
    [aux_sym_enum_variant_list_repeat1] = STATE(135),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_RBRACK] = ACTIONS(768),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COMMA] = ACTIONS(770),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(748),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [139] = {
    [sym_attribute_item] = STATE(1011),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1562),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(139),
    [sym_block_comment] = STATE(139),
    [aux_sym_enum_variant_list_repeat1] = STATE(141),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_RBRACK] = ACTIONS(772),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COMMA] = ACTIONS(774),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(748),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [140] = {
    [sym_attribute_item] = STATE(1011),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1613),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(140),
    [sym_block_comment] = STATE(140),
    [aux_sym_enum_variant_list_repeat1] = STATE(212),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(776),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COMMA] = ACTIONS(778),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(748),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [141] = {
    [sym_attribute_item] = STATE(1011),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1630),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(141),
    [sym_block_comment] = STATE(141),
    [aux_sym_enum_variant_list_repeat1] = STATE(1010),
    [sym_identifier] = ACTIONS(780),
    [anon_sym_LPAREN] = ACTIONS(783),
    [anon_sym_LBRACK] = ACTIONS(786),
    [anon_sym_RBRACK] = ACTIONS(789),
    [anon_sym_LBRACE] = ACTIONS(791),
    [anon_sym_STAR] = ACTIONS(794),
    [anon_sym_u8] = ACTIONS(797),
    [anon_sym_i8] = ACTIONS(797),
    [anon_sym_u16] = ACTIONS(797),
    [anon_sym_i16] = ACTIONS(797),
    [anon_sym_u32] = ACTIONS(797),
    [anon_sym_i32] = ACTIONS(797),
    [anon_sym_u64] = ACTIONS(797),
    [anon_sym_i64] = ACTIONS(797),
    [anon_sym_u128] = ACTIONS(797),
    [anon_sym_i128] = ACTIONS(797),
    [anon_sym_isize] = ACTIONS(797),
    [anon_sym_usize] = ACTIONS(797),
    [anon_sym_f32] = ACTIONS(797),
    [anon_sym_f64] = ACTIONS(797),
    [anon_sym_bool] = ACTIONS(797),
    [anon_sym_str] = ACTIONS(797),
    [anon_sym_char] = ACTIONS(797),
    [anon_sym_DASH] = ACTIONS(794),
    [anon_sym_BANG] = ACTIONS(794),
    [anon_sym_AMP] = ACTIONS(800),
    [anon_sym_PIPE] = ACTIONS(803),
    [anon_sym_LT] = ACTIONS(806),
    [anon_sym_DOT_DOT] = ACTIONS(809),
    [anon_sym_COMMA] = ACTIONS(789),
    [anon_sym_COLON_COLON] = ACTIONS(812),
    [anon_sym_POUND] = ACTIONS(815),
    [anon_sym_SQUOTE] = ACTIONS(818),
    [anon_sym_async] = ACTIONS(821),
    [anon_sym_break] = ACTIONS(824),
    [anon_sym_const] = ACTIONS(827),
    [anon_sym_continue] = ACTIONS(830),
    [anon_sym_default] = ACTIONS(833),
    [anon_sym_for] = ACTIONS(836),
    [anon_sym_gen] = ACTIONS(839),
    [anon_sym_if] = ACTIONS(842),
    [anon_sym_loop] = ACTIONS(845),
    [anon_sym_match] = ACTIONS(848),
    [anon_sym_return] = ACTIONS(851),
    [anon_sym_static] = ACTIONS(854),
    [anon_sym_union] = ACTIONS(833),
    [anon_sym_unsafe] = ACTIONS(857),
    [anon_sym_while] = ACTIONS(860),
    [anon_sym_yield] = ACTIONS(863),
    [anon_sym_move] = ACTIONS(866),
    [anon_sym_try] = ACTIONS(869),
    [sym_integer_literal] = ACTIONS(872),
    [aux_sym_string_literal_token1] = ACTIONS(875),
    [sym_char_literal] = ACTIONS(872),
    [anon_sym_true] = ACTIONS(878),
    [anon_sym_false] = ACTIONS(878),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(881),
    [sym_super] = ACTIONS(884),
    [sym_crate] = ACTIONS(884),
    [sym_metavariable] = ACTIONS(887),
    [sym__raw_string_literal_start] = ACTIONS(890),
    [sym_float_literal] = ACTIONS(872),
  },
  [142] = {
    [sym_attribute_item] = STATE(1011),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1599),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(142),
    [sym_block_comment] = STATE(142),
    [aux_sym_enum_variant_list_repeat1] = STATE(201),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(893),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COMMA] = ACTIONS(895),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(748),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [143] = {
    [sym_line_comment] = STATE(143),
    [sym_block_comment] = STATE(143),
    [aux_sym__non_special_token_repeat1] = STATE(137),
    [sym_identifier] = ACTIONS(897),
    [anon_sym_SEMI] = ACTIONS(567),
    [anon_sym_LPAREN] = ACTIONS(899),
    [anon_sym_RPAREN] = ACTIONS(899),
    [anon_sym_LBRACK] = ACTIONS(899),
    [anon_sym_RBRACK] = ACTIONS(899),
    [anon_sym_LBRACE] = ACTIONS(899),
    [anon_sym_RBRACE] = ACTIONS(899),
    [anon_sym_EQ_GT] = ACTIONS(567),
    [anon_sym_COLON] = ACTIONS(577),
    [anon_sym_DOLLAR] = ACTIONS(897),
    [anon_sym_PLUS] = ACTIONS(577),
    [anon_sym_STAR] = ACTIONS(577),
    [anon_sym_QMARK] = ACTIONS(567),
    [anon_sym_u8] = ACTIONS(897),
    [anon_sym_i8] = ACTIONS(897),
    [anon_sym_u16] = ACTIONS(897),
    [anon_sym_i16] = ACTIONS(897),
    [anon_sym_u32] = ACTIONS(897),
    [anon_sym_i32] = ACTIONS(897),
    [anon_sym_u64] = ACTIONS(897),
    [anon_sym_i64] = ACTIONS(897),
    [anon_sym_u128] = ACTIONS(897),
    [anon_sym_i128] = ACTIONS(897),
    [anon_sym_isize] = ACTIONS(897),
    [anon_sym_usize] = ACTIONS(897),
    [anon_sym_f32] = ACTIONS(897),
    [anon_sym_f64] = ACTIONS(897),
    [anon_sym_bool] = ACTIONS(897),
    [anon_sym_str] = ACTIONS(897),
    [anon_sym_char] = ACTIONS(897),
    [anon_sym_DASH] = ACTIONS(577),
    [anon_sym_SLASH] = ACTIONS(577),
    [anon_sym_PERCENT] = ACTIONS(577),
    [anon_sym_CARET] = ACTIONS(577),
    [anon_sym_BANG] = ACTIONS(577),
    [anon_sym_AMP] = ACTIONS(577),
    [anon_sym_PIPE] = ACTIONS(577),
    [anon_sym_AMP_AMP] = ACTIONS(567),
    [anon_sym_PIPE_PIPE] = ACTIONS(567),
    [anon_sym_LT_LT] = ACTIONS(577),
    [anon_sym_GT_GT] = ACTIONS(577),
    [anon_sym_PLUS_EQ] = ACTIONS(567),
    [anon_sym_DASH_EQ] = ACTIONS(567),
    [anon_sym_STAR_EQ] = ACTIONS(567),
    [anon_sym_SLASH_EQ] = ACTIONS(567),
    [anon_sym_PERCENT_EQ] = ACTIONS(567),
    [anon_sym_CARET_EQ] = ACTIONS(567),
    [anon_sym_AMP_EQ] = ACTIONS(567),
    [anon_sym_PIPE_EQ] = ACTIONS(567),
    [anon_sym_LT_LT_EQ] = ACTIONS(567),
    [anon_sym_GT_GT_EQ] = ACTIONS(567),
    [anon_sym_EQ] = ACTIONS(577),
    [anon_sym_EQ_EQ] = ACTIONS(567),
    [anon_sym_BANG_EQ] = ACTIONS(567),
    [anon_sym_GT] = ACTIONS(577),
    [anon_sym_LT] = ACTIONS(577),
    [anon_sym_GT_EQ] = ACTIONS(567),
    [anon_sym_LT_EQ] = ACTIONS(567),
    [anon_sym_AT] = ACTIONS(567),
    [anon_sym__] = ACTIONS(577),
    [anon_sym_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT] = ACTIONS(577),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(567),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(567),
    [anon_sym_COMMA] = ACTIONS(567),
    [anon_sym_COLON_COLON] = ACTIONS(567),
    [anon_sym_DASH_GT] = ACTIONS(567),
    [anon_sym_POUND] = ACTIONS(567),
    [anon_sym_SQUOTE] = ACTIONS(897),
    [anon_sym_as] = ACTIONS(897),
    [anon_sym_async] = ACTIONS(897),
    [anon_sym_await] = ACTIONS(897),
    [anon_sym_break] = ACTIONS(897),
    [anon_sym_const] = ACTIONS(897),
    [anon_sym_continue] = ACTIONS(897),
    [anon_sym_default] = ACTIONS(897),
    [anon_sym_enum] = ACTIONS(897),
    [anon_sym_fn] = ACTIONS(897),
    [anon_sym_for] = ACTIONS(897),
    [anon_sym_gen] = ACTIONS(897),
    [anon_sym_if] = ACTIONS(897),
    [anon_sym_impl] = ACTIONS(897),
    [anon_sym_let] = ACTIONS(897),
    [anon_sym_loop] = ACTIONS(897),
    [anon_sym_match] = ACTIONS(897),
    [anon_sym_mod] = ACTIONS(897),
    [anon_sym_pub] = ACTIONS(897),
    [anon_sym_return] = ACTIONS(897),
    [anon_sym_static] = ACTIONS(897),
    [anon_sym_struct] = ACTIONS(897),
    [anon_sym_trait] = ACTIONS(897),
    [anon_sym_type] = ACTIONS(897),
    [anon_sym_union] = ACTIONS(897),
    [anon_sym_unsafe] = ACTIONS(897),
    [anon_sym_use] = ACTIONS(897),
    [anon_sym_where] = ACTIONS(897),
    [anon_sym_while] = ACTIONS(897),
    [sym_mutable_specifier] = ACTIONS(897),
    [sym_integer_literal] = ACTIONS(899),
    [aux_sym_string_literal_token1] = ACTIONS(899),
    [sym_char_literal] = ACTIONS(899),
    [anon_sym_true] = ACTIONS(897),
    [anon_sym_false] = ACTIONS(897),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(897),
    [sym_super] = ACTIONS(897),
    [sym_crate] = ACTIONS(897),
    [sym_metavariable] = ACTIONS(899),
    [sym__raw_string_literal_start] = ACTIONS(899),
    [sym_float_literal] = ACTIONS(899),
  },
  [144] = {
    [sym_attribute_item] = STATE(1011),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1614),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(144),
    [sym_block_comment] = STATE(144),
    [aux_sym_enum_variant_list_repeat1] = STATE(1010),
    [sym_identifier] = ACTIONS(780),
    [anon_sym_LPAREN] = ACTIONS(783),
    [anon_sym_LBRACK] = ACTIONS(786),
    [anon_sym_RBRACK] = ACTIONS(789),
    [anon_sym_LBRACE] = ACTIONS(791),
    [anon_sym_STAR] = ACTIONS(794),
    [anon_sym_u8] = ACTIONS(797),
    [anon_sym_i8] = ACTIONS(797),
    [anon_sym_u16] = ACTIONS(797),
    [anon_sym_i16] = ACTIONS(797),
    [anon_sym_u32] = ACTIONS(797),
    [anon_sym_i32] = ACTIONS(797),
    [anon_sym_u64] = ACTIONS(797),
    [anon_sym_i64] = ACTIONS(797),
    [anon_sym_u128] = ACTIONS(797),
    [anon_sym_i128] = ACTIONS(797),
    [anon_sym_isize] = ACTIONS(797),
    [anon_sym_usize] = ACTIONS(797),
    [anon_sym_f32] = ACTIONS(797),
    [anon_sym_f64] = ACTIONS(797),
    [anon_sym_bool] = ACTIONS(797),
    [anon_sym_str] = ACTIONS(797),
    [anon_sym_char] = ACTIONS(797),
    [anon_sym_DASH] = ACTIONS(794),
    [anon_sym_BANG] = ACTIONS(794),
    [anon_sym_AMP] = ACTIONS(800),
    [anon_sym_PIPE] = ACTIONS(803),
    [anon_sym_LT] = ACTIONS(806),
    [anon_sym_DOT_DOT] = ACTIONS(809),
    [anon_sym_COMMA] = ACTIONS(789),
    [anon_sym_COLON_COLON] = ACTIONS(812),
    [anon_sym_POUND] = ACTIONS(815),
    [anon_sym_SQUOTE] = ACTIONS(818),
    [anon_sym_async] = ACTIONS(821),
    [anon_sym_break] = ACTIONS(824),
    [anon_sym_const] = ACTIONS(827),
    [anon_sym_continue] = ACTIONS(830),
    [anon_sym_default] = ACTIONS(833),
    [anon_sym_for] = ACTIONS(836),
    [anon_sym_gen] = ACTIONS(839),
    [anon_sym_if] = ACTIONS(842),
    [anon_sym_loop] = ACTIONS(845),
    [anon_sym_match] = ACTIONS(848),
    [anon_sym_return] = ACTIONS(851),
    [anon_sym_static] = ACTIONS(854),
    [anon_sym_union] = ACTIONS(833),
    [anon_sym_unsafe] = ACTIONS(857),
    [anon_sym_while] = ACTIONS(860),
    [anon_sym_yield] = ACTIONS(863),
    [anon_sym_move] = ACTIONS(866),
    [anon_sym_try] = ACTIONS(869),
    [sym_integer_literal] = ACTIONS(872),
    [aux_sym_string_literal_token1] = ACTIONS(875),
    [sym_char_literal] = ACTIONS(872),
    [anon_sym_true] = ACTIONS(878),
    [anon_sym_false] = ACTIONS(878),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(881),
    [sym_super] = ACTIONS(884),
    [sym_crate] = ACTIONS(884),
    [sym_metavariable] = ACTIONS(887),
    [sym__raw_string_literal_start] = ACTIONS(890),
    [sym_float_literal] = ACTIONS(872),
  },
  [145] = {
    [sym_attribute_item] = STATE(1011),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1665),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(145),
    [sym_block_comment] = STATE(145),
    [aux_sym_enum_variant_list_repeat1] = STATE(211),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(901),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(748),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [146] = {
    [sym_attribute_item] = STATE(1011),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1623),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(146),
    [sym_block_comment] = STATE(146),
    [aux_sym_enum_variant_list_repeat1] = STATE(207),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_RBRACK] = ACTIONS(903),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(748),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [147] = {
    [sym_attribute_item] = STATE(1011),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1623),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(147),
    [sym_block_comment] = STATE(147),
    [aux_sym_enum_variant_list_repeat1] = STATE(207),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_RBRACK] = ACTIONS(905),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(748),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [148] = {
    [sym_attribute_item] = STATE(1011),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1623),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(148),
    [sym_block_comment] = STATE(148),
    [aux_sym_enum_variant_list_repeat1] = STATE(207),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(907),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(748),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [149] = {
    [sym_attribute_item] = STATE(1011),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1799),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(149),
    [sym_block_comment] = STATE(149),
    [aux_sym_enum_variant_list_repeat1] = STATE(208),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(909),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(748),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [150] = {
    [sym_line_comment] = STATE(150),
    [sym_block_comment] = STATE(150),
    [sym_identifier] = ACTIONS(911),
    [anon_sym_SEMI] = ACTIONS(913),
    [anon_sym_LPAREN] = ACTIONS(913),
    [anon_sym_RPAREN] = ACTIONS(913),
    [anon_sym_LBRACK] = ACTIONS(913),
    [anon_sym_RBRACK] = ACTIONS(913),
    [anon_sym_LBRACE] = ACTIONS(913),
    [anon_sym_RBRACE] = ACTIONS(913),
    [anon_sym_EQ_GT] = ACTIONS(913),
    [anon_sym_COLON] = ACTIONS(911),
    [anon_sym_DOLLAR] = ACTIONS(911),
    [anon_sym_PLUS] = ACTIONS(911),
    [anon_sym_STAR] = ACTIONS(911),
    [anon_sym_QMARK] = ACTIONS(913),
    [anon_sym_u8] = ACTIONS(911),
    [anon_sym_i8] = ACTIONS(911),
    [anon_sym_u16] = ACTIONS(911),
    [anon_sym_i16] = ACTIONS(911),
    [anon_sym_u32] = ACTIONS(911),
    [anon_sym_i32] = ACTIONS(911),
    [anon_sym_u64] = ACTIONS(911),
    [anon_sym_i64] = ACTIONS(911),
    [anon_sym_u128] = ACTIONS(911),
    [anon_sym_i128] = ACTIONS(911),
    [anon_sym_isize] = ACTIONS(911),
    [anon_sym_usize] = ACTIONS(911),
    [anon_sym_f32] = ACTIONS(911),
    [anon_sym_f64] = ACTIONS(911),
    [anon_sym_bool] = ACTIONS(911),
    [anon_sym_str] = ACTIONS(911),
    [anon_sym_char] = ACTIONS(911),
    [anon_sym_DASH] = ACTIONS(911),
    [anon_sym_SLASH] = ACTIONS(911),
    [anon_sym_PERCENT] = ACTIONS(911),
    [anon_sym_CARET] = ACTIONS(911),
    [anon_sym_BANG] = ACTIONS(911),
    [anon_sym_AMP] = ACTIONS(911),
    [anon_sym_PIPE] = ACTIONS(911),
    [anon_sym_AMP_AMP] = ACTIONS(913),
    [anon_sym_PIPE_PIPE] = ACTIONS(913),
    [anon_sym_LT_LT] = ACTIONS(911),
    [anon_sym_GT_GT] = ACTIONS(911),
    [anon_sym_PLUS_EQ] = ACTIONS(913),
    [anon_sym_DASH_EQ] = ACTIONS(913),
    [anon_sym_STAR_EQ] = ACTIONS(913),
    [anon_sym_SLASH_EQ] = ACTIONS(913),
    [anon_sym_PERCENT_EQ] = ACTIONS(913),
    [anon_sym_CARET_EQ] = ACTIONS(913),
    [anon_sym_AMP_EQ] = ACTIONS(913),
    [anon_sym_PIPE_EQ] = ACTIONS(913),
    [anon_sym_LT_LT_EQ] = ACTIONS(913),
    [anon_sym_GT_GT_EQ] = ACTIONS(913),
    [anon_sym_EQ] = ACTIONS(911),
    [anon_sym_EQ_EQ] = ACTIONS(913),
    [anon_sym_BANG_EQ] = ACTIONS(913),
    [anon_sym_GT] = ACTIONS(911),
    [anon_sym_LT] = ACTIONS(911),
    [anon_sym_GT_EQ] = ACTIONS(913),
    [anon_sym_LT_EQ] = ACTIONS(913),
    [anon_sym_AT] = ACTIONS(913),
    [anon_sym__] = ACTIONS(911),
    [anon_sym_DOT] = ACTIONS(911),
    [anon_sym_DOT_DOT] = ACTIONS(911),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(913),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(913),
    [anon_sym_COMMA] = ACTIONS(913),
    [anon_sym_COLON_COLON] = ACTIONS(913),
    [anon_sym_DASH_GT] = ACTIONS(913),
    [anon_sym_POUND] = ACTIONS(913),
    [anon_sym_SQUOTE] = ACTIONS(911),
    [anon_sym_as] = ACTIONS(911),
    [anon_sym_async] = ACTIONS(911),
    [anon_sym_await] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(911),
    [anon_sym_const] = ACTIONS(911),
    [anon_sym_continue] = ACTIONS(911),
    [anon_sym_default] = ACTIONS(911),
    [anon_sym_enum] = ACTIONS(911),
    [anon_sym_fn] = ACTIONS(911),
    [anon_sym_for] = ACTIONS(911),
    [anon_sym_gen] = ACTIONS(911),
    [anon_sym_if] = ACTIONS(911),
    [anon_sym_impl] = ACTIONS(911),
    [anon_sym_let] = ACTIONS(911),
    [anon_sym_loop] = ACTIONS(911),
    [anon_sym_match] = ACTIONS(911),
    [anon_sym_mod] = ACTIONS(911),
    [anon_sym_pub] = ACTIONS(911),
    [anon_sym_return] = ACTIONS(911),
    [anon_sym_static] = ACTIONS(911),
    [anon_sym_struct] = ACTIONS(911),
    [anon_sym_trait] = ACTIONS(911),
    [anon_sym_type] = ACTIONS(911),
    [anon_sym_union] = ACTIONS(911),
    [anon_sym_unsafe] = ACTIONS(911),
    [anon_sym_use] = ACTIONS(911),
    [anon_sym_where] = ACTIONS(911),
    [anon_sym_while] = ACTIONS(911),
    [sym_mutable_specifier] = ACTIONS(911),
    [sym_integer_literal] = ACTIONS(913),
    [aux_sym_string_literal_token1] = ACTIONS(913),
    [sym_char_literal] = ACTIONS(913),
    [anon_sym_true] = ACTIONS(911),
    [anon_sym_false] = ACTIONS(911),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(911),
    [sym_super] = ACTIONS(911),
    [sym_crate] = ACTIONS(911),
    [sym_metavariable] = ACTIONS(913),
    [sym__raw_string_literal_start] = ACTIONS(913),
    [sym_float_literal] = ACTIONS(913),
  },
  [151] = {
    [sym_line_comment] = STATE(151),
    [sym_block_comment] = STATE(151),
    [sym_identifier] = ACTIONS(915),
    [anon_sym_SEMI] = ACTIONS(917),
    [anon_sym_LPAREN] = ACTIONS(917),
    [anon_sym_RPAREN] = ACTIONS(917),
    [anon_sym_LBRACK] = ACTIONS(917),
    [anon_sym_RBRACK] = ACTIONS(917),
    [anon_sym_LBRACE] = ACTIONS(917),
    [anon_sym_RBRACE] = ACTIONS(917),
    [anon_sym_EQ_GT] = ACTIONS(917),
    [anon_sym_COLON] = ACTIONS(915),
    [anon_sym_DOLLAR] = ACTIONS(915),
    [anon_sym_PLUS] = ACTIONS(915),
    [anon_sym_STAR] = ACTIONS(915),
    [anon_sym_QMARK] = ACTIONS(917),
    [anon_sym_u8] = ACTIONS(915),
    [anon_sym_i8] = ACTIONS(915),
    [anon_sym_u16] = ACTIONS(915),
    [anon_sym_i16] = ACTIONS(915),
    [anon_sym_u32] = ACTIONS(915),
    [anon_sym_i32] = ACTIONS(915),
    [anon_sym_u64] = ACTIONS(915),
    [anon_sym_i64] = ACTIONS(915),
    [anon_sym_u128] = ACTIONS(915),
    [anon_sym_i128] = ACTIONS(915),
    [anon_sym_isize] = ACTIONS(915),
    [anon_sym_usize] = ACTIONS(915),
    [anon_sym_f32] = ACTIONS(915),
    [anon_sym_f64] = ACTIONS(915),
    [anon_sym_bool] = ACTIONS(915),
    [anon_sym_str] = ACTIONS(915),
    [anon_sym_char] = ACTIONS(915),
    [anon_sym_DASH] = ACTIONS(915),
    [anon_sym_SLASH] = ACTIONS(915),
    [anon_sym_PERCENT] = ACTIONS(915),
    [anon_sym_CARET] = ACTIONS(915),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_AMP] = ACTIONS(915),
    [anon_sym_PIPE] = ACTIONS(915),
    [anon_sym_AMP_AMP] = ACTIONS(917),
    [anon_sym_PIPE_PIPE] = ACTIONS(917),
    [anon_sym_LT_LT] = ACTIONS(915),
    [anon_sym_GT_GT] = ACTIONS(915),
    [anon_sym_PLUS_EQ] = ACTIONS(917),
    [anon_sym_DASH_EQ] = ACTIONS(917),
    [anon_sym_STAR_EQ] = ACTIONS(917),
    [anon_sym_SLASH_EQ] = ACTIONS(917),
    [anon_sym_PERCENT_EQ] = ACTIONS(917),
    [anon_sym_CARET_EQ] = ACTIONS(917),
    [anon_sym_AMP_EQ] = ACTIONS(917),
    [anon_sym_PIPE_EQ] = ACTIONS(917),
    [anon_sym_LT_LT_EQ] = ACTIONS(917),
    [anon_sym_GT_GT_EQ] = ACTIONS(917),
    [anon_sym_EQ] = ACTIONS(915),
    [anon_sym_EQ_EQ] = ACTIONS(917),
    [anon_sym_BANG_EQ] = ACTIONS(917),
    [anon_sym_GT] = ACTIONS(915),
    [anon_sym_LT] = ACTIONS(915),
    [anon_sym_GT_EQ] = ACTIONS(917),
    [anon_sym_LT_EQ] = ACTIONS(917),
    [anon_sym_AT] = ACTIONS(917),
    [anon_sym__] = ACTIONS(915),
    [anon_sym_DOT] = ACTIONS(915),
    [anon_sym_DOT_DOT] = ACTIONS(915),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(917),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(917),
    [anon_sym_COMMA] = ACTIONS(917),
    [anon_sym_COLON_COLON] = ACTIONS(917),
    [anon_sym_DASH_GT] = ACTIONS(917),
    [anon_sym_POUND] = ACTIONS(917),
    [anon_sym_SQUOTE] = ACTIONS(915),
    [anon_sym_as] = ACTIONS(915),
    [anon_sym_async] = ACTIONS(915),
    [anon_sym_await] = ACTIONS(915),
    [anon_sym_break] = ACTIONS(915),
    [anon_sym_const] = ACTIONS(915),
    [anon_sym_continue] = ACTIONS(915),
    [anon_sym_default] = ACTIONS(915),
    [anon_sym_enum] = ACTIONS(915),
    [anon_sym_fn] = ACTIONS(915),
    [anon_sym_for] = ACTIONS(915),
    [anon_sym_gen] = ACTIONS(915),
    [anon_sym_if] = ACTIONS(915),
    [anon_sym_impl] = ACTIONS(915),
    [anon_sym_let] = ACTIONS(915),
    [anon_sym_loop] = ACTIONS(915),
    [anon_sym_match] = ACTIONS(915),
    [anon_sym_mod] = ACTIONS(915),
    [anon_sym_pub] = ACTIONS(915),
    [anon_sym_return] = ACTIONS(915),
    [anon_sym_static] = ACTIONS(915),
    [anon_sym_struct] = ACTIONS(915),
    [anon_sym_trait] = ACTIONS(915),
    [anon_sym_type] = ACTIONS(915),
    [anon_sym_union] = ACTIONS(915),
    [anon_sym_unsafe] = ACTIONS(915),
    [anon_sym_use] = ACTIONS(915),
    [anon_sym_where] = ACTIONS(915),
    [anon_sym_while] = ACTIONS(915),
    [sym_mutable_specifier] = ACTIONS(915),
    [sym_integer_literal] = ACTIONS(917),
    [aux_sym_string_literal_token1] = ACTIONS(917),
    [sym_char_literal] = ACTIONS(917),
    [anon_sym_true] = ACTIONS(915),
    [anon_sym_false] = ACTIONS(915),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(915),
    [sym_super] = ACTIONS(915),
    [sym_crate] = ACTIONS(915),
    [sym_metavariable] = ACTIONS(917),
    [sym__raw_string_literal_start] = ACTIONS(917),
    [sym_float_literal] = ACTIONS(917),
  },
  [152] = {
    [sym_line_comment] = STATE(152),
    [sym_block_comment] = STATE(152),
    [sym_identifier] = ACTIONS(897),
    [anon_sym_SEMI] = ACTIONS(899),
    [anon_sym_LPAREN] = ACTIONS(899),
    [anon_sym_RPAREN] = ACTIONS(899),
    [anon_sym_LBRACK] = ACTIONS(899),
    [anon_sym_RBRACK] = ACTIONS(899),
    [anon_sym_LBRACE] = ACTIONS(899),
    [anon_sym_RBRACE] = ACTIONS(899),
    [anon_sym_EQ_GT] = ACTIONS(899),
    [anon_sym_COLON] = ACTIONS(897),
    [anon_sym_DOLLAR] = ACTIONS(897),
    [anon_sym_PLUS] = ACTIONS(897),
    [anon_sym_STAR] = ACTIONS(897),
    [anon_sym_QMARK] = ACTIONS(899),
    [anon_sym_u8] = ACTIONS(897),
    [anon_sym_i8] = ACTIONS(897),
    [anon_sym_u16] = ACTIONS(897),
    [anon_sym_i16] = ACTIONS(897),
    [anon_sym_u32] = ACTIONS(897),
    [anon_sym_i32] = ACTIONS(897),
    [anon_sym_u64] = ACTIONS(897),
    [anon_sym_i64] = ACTIONS(897),
    [anon_sym_u128] = ACTIONS(897),
    [anon_sym_i128] = ACTIONS(897),
    [anon_sym_isize] = ACTIONS(897),
    [anon_sym_usize] = ACTIONS(897),
    [anon_sym_f32] = ACTIONS(897),
    [anon_sym_f64] = ACTIONS(897),
    [anon_sym_bool] = ACTIONS(897),
    [anon_sym_str] = ACTIONS(897),
    [anon_sym_char] = ACTIONS(897),
    [anon_sym_DASH] = ACTIONS(897),
    [anon_sym_SLASH] = ACTIONS(897),
    [anon_sym_PERCENT] = ACTIONS(897),
    [anon_sym_CARET] = ACTIONS(897),
    [anon_sym_BANG] = ACTIONS(897),
    [anon_sym_AMP] = ACTIONS(897),
    [anon_sym_PIPE] = ACTIONS(897),
    [anon_sym_AMP_AMP] = ACTIONS(899),
    [anon_sym_PIPE_PIPE] = ACTIONS(899),
    [anon_sym_LT_LT] = ACTIONS(897),
    [anon_sym_GT_GT] = ACTIONS(897),
    [anon_sym_PLUS_EQ] = ACTIONS(899),
    [anon_sym_DASH_EQ] = ACTIONS(899),
    [anon_sym_STAR_EQ] = ACTIONS(899),
    [anon_sym_SLASH_EQ] = ACTIONS(899),
    [anon_sym_PERCENT_EQ] = ACTIONS(899),
    [anon_sym_CARET_EQ] = ACTIONS(899),
    [anon_sym_AMP_EQ] = ACTIONS(899),
    [anon_sym_PIPE_EQ] = ACTIONS(899),
    [anon_sym_LT_LT_EQ] = ACTIONS(899),
    [anon_sym_GT_GT_EQ] = ACTIONS(899),
    [anon_sym_EQ] = ACTIONS(897),
    [anon_sym_EQ_EQ] = ACTIONS(899),
    [anon_sym_BANG_EQ] = ACTIONS(899),
    [anon_sym_GT] = ACTIONS(897),
    [anon_sym_LT] = ACTIONS(897),
    [anon_sym_GT_EQ] = ACTIONS(899),
    [anon_sym_LT_EQ] = ACTIONS(899),
    [anon_sym_AT] = ACTIONS(899),
    [anon_sym__] = ACTIONS(897),
    [anon_sym_DOT] = ACTIONS(897),
    [anon_sym_DOT_DOT] = ACTIONS(897),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(899),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(899),
    [anon_sym_COMMA] = ACTIONS(899),
    [anon_sym_COLON_COLON] = ACTIONS(899),
    [anon_sym_DASH_GT] = ACTIONS(899),
    [anon_sym_POUND] = ACTIONS(899),
    [anon_sym_SQUOTE] = ACTIONS(897),
    [anon_sym_as] = ACTIONS(897),
    [anon_sym_async] = ACTIONS(897),
    [anon_sym_await] = ACTIONS(897),
    [anon_sym_break] = ACTIONS(897),
    [anon_sym_const] = ACTIONS(897),
    [anon_sym_continue] = ACTIONS(897),
    [anon_sym_default] = ACTIONS(897),
    [anon_sym_enum] = ACTIONS(897),
    [anon_sym_fn] = ACTIONS(897),
    [anon_sym_for] = ACTIONS(897),
    [anon_sym_gen] = ACTIONS(897),
    [anon_sym_if] = ACTIONS(897),
    [anon_sym_impl] = ACTIONS(897),
    [anon_sym_let] = ACTIONS(897),
    [anon_sym_loop] = ACTIONS(897),
    [anon_sym_match] = ACTIONS(897),
    [anon_sym_mod] = ACTIONS(897),
    [anon_sym_pub] = ACTIONS(897),
    [anon_sym_return] = ACTIONS(897),
    [anon_sym_static] = ACTIONS(897),
    [anon_sym_struct] = ACTIONS(897),
    [anon_sym_trait] = ACTIONS(897),
    [anon_sym_type] = ACTIONS(897),
    [anon_sym_union] = ACTIONS(897),
    [anon_sym_unsafe] = ACTIONS(897),
    [anon_sym_use] = ACTIONS(897),
    [anon_sym_where] = ACTIONS(897),
    [anon_sym_while] = ACTIONS(897),
    [sym_mutable_specifier] = ACTIONS(897),
    [sym_integer_literal] = ACTIONS(899),
    [aux_sym_string_literal_token1] = ACTIONS(899),
    [sym_char_literal] = ACTIONS(899),
    [anon_sym_true] = ACTIONS(897),
    [anon_sym_false] = ACTIONS(897),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(897),
    [sym_super] = ACTIONS(897),
    [sym_crate] = ACTIONS(897),
    [sym_metavariable] = ACTIONS(899),
    [sym__raw_string_literal_start] = ACTIONS(899),
    [sym_float_literal] = ACTIONS(899),
  },
  [153] = {
    [sym_attribute_item] = STATE(1011),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1623),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(153),
    [sym_block_comment] = STATE(153),
    [aux_sym_enum_variant_list_repeat1] = STATE(207),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_RBRACK] = ACTIONS(919),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(748),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [154] = {
    [sym_attribute_item] = STATE(1011),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1623),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(154),
    [sym_block_comment] = STATE(154),
    [aux_sym_enum_variant_list_repeat1] = STATE(207),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_RBRACK] = ACTIONS(921),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(748),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [155] = {
    [sym_line_comment] = STATE(155),
    [sym_block_comment] = STATE(155),
    [sym_identifier] = ACTIONS(923),
    [anon_sym_SEMI] = ACTIONS(925),
    [anon_sym_LPAREN] = ACTIONS(925),
    [anon_sym_RPAREN] = ACTIONS(925),
    [anon_sym_LBRACK] = ACTIONS(925),
    [anon_sym_RBRACK] = ACTIONS(925),
    [anon_sym_LBRACE] = ACTIONS(925),
    [anon_sym_RBRACE] = ACTIONS(925),
    [anon_sym_EQ_GT] = ACTIONS(925),
    [anon_sym_COLON] = ACTIONS(923),
    [anon_sym_DOLLAR] = ACTIONS(923),
    [anon_sym_PLUS] = ACTIONS(923),
    [anon_sym_STAR] = ACTIONS(923),
    [anon_sym_QMARK] = ACTIONS(925),
    [anon_sym_u8] = ACTIONS(923),
    [anon_sym_i8] = ACTIONS(923),
    [anon_sym_u16] = ACTIONS(923),
    [anon_sym_i16] = ACTIONS(923),
    [anon_sym_u32] = ACTIONS(923),
    [anon_sym_i32] = ACTIONS(923),
    [anon_sym_u64] = ACTIONS(923),
    [anon_sym_i64] = ACTIONS(923),
    [anon_sym_u128] = ACTIONS(923),
    [anon_sym_i128] = ACTIONS(923),
    [anon_sym_isize] = ACTIONS(923),
    [anon_sym_usize] = ACTIONS(923),
    [anon_sym_f32] = ACTIONS(923),
    [anon_sym_f64] = ACTIONS(923),
    [anon_sym_bool] = ACTIONS(923),
    [anon_sym_str] = ACTIONS(923),
    [anon_sym_char] = ACTIONS(923),
    [anon_sym_DASH] = ACTIONS(923),
    [anon_sym_SLASH] = ACTIONS(923),
    [anon_sym_PERCENT] = ACTIONS(923),
    [anon_sym_CARET] = ACTIONS(923),
    [anon_sym_BANG] = ACTIONS(923),
    [anon_sym_AMP] = ACTIONS(923),
    [anon_sym_PIPE] = ACTIONS(923),
    [anon_sym_AMP_AMP] = ACTIONS(925),
    [anon_sym_PIPE_PIPE] = ACTIONS(925),
    [anon_sym_LT_LT] = ACTIONS(923),
    [anon_sym_GT_GT] = ACTIONS(923),
    [anon_sym_PLUS_EQ] = ACTIONS(925),
    [anon_sym_DASH_EQ] = ACTIONS(925),
    [anon_sym_STAR_EQ] = ACTIONS(925),
    [anon_sym_SLASH_EQ] = ACTIONS(925),
    [anon_sym_PERCENT_EQ] = ACTIONS(925),
    [anon_sym_CARET_EQ] = ACTIONS(925),
    [anon_sym_AMP_EQ] = ACTIONS(925),
    [anon_sym_PIPE_EQ] = ACTIONS(925),
    [anon_sym_LT_LT_EQ] = ACTIONS(925),
    [anon_sym_GT_GT_EQ] = ACTIONS(925),
    [anon_sym_EQ] = ACTIONS(923),
    [anon_sym_EQ_EQ] = ACTIONS(925),
    [anon_sym_BANG_EQ] = ACTIONS(925),
    [anon_sym_GT] = ACTIONS(923),
    [anon_sym_LT] = ACTIONS(923),
    [anon_sym_GT_EQ] = ACTIONS(925),
    [anon_sym_LT_EQ] = ACTIONS(925),
    [anon_sym_AT] = ACTIONS(925),
    [anon_sym__] = ACTIONS(923),
    [anon_sym_DOT] = ACTIONS(923),
    [anon_sym_DOT_DOT] = ACTIONS(923),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(925),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(925),
    [anon_sym_COMMA] = ACTIONS(925),
    [anon_sym_COLON_COLON] = ACTIONS(925),
    [anon_sym_DASH_GT] = ACTIONS(925),
    [anon_sym_POUND] = ACTIONS(925),
    [anon_sym_SQUOTE] = ACTIONS(923),
    [anon_sym_as] = ACTIONS(923),
    [anon_sym_async] = ACTIONS(923),
    [anon_sym_await] = ACTIONS(923),
    [anon_sym_break] = ACTIONS(923),
    [anon_sym_const] = ACTIONS(923),
    [anon_sym_continue] = ACTIONS(923),
    [anon_sym_default] = ACTIONS(923),
    [anon_sym_enum] = ACTIONS(923),
    [anon_sym_fn] = ACTIONS(923),
    [anon_sym_for] = ACTIONS(923),
    [anon_sym_gen] = ACTIONS(923),
    [anon_sym_if] = ACTIONS(923),
    [anon_sym_impl] = ACTIONS(923),
    [anon_sym_let] = ACTIONS(923),
    [anon_sym_loop] = ACTIONS(923),
    [anon_sym_match] = ACTIONS(923),
    [anon_sym_mod] = ACTIONS(923),
    [anon_sym_pub] = ACTIONS(923),
    [anon_sym_return] = ACTIONS(923),
    [anon_sym_static] = ACTIONS(923),
    [anon_sym_struct] = ACTIONS(923),
    [anon_sym_trait] = ACTIONS(923),
    [anon_sym_type] = ACTIONS(923),
    [anon_sym_union] = ACTIONS(923),
    [anon_sym_unsafe] = ACTIONS(923),
    [anon_sym_use] = ACTIONS(923),
    [anon_sym_where] = ACTIONS(923),
    [anon_sym_while] = ACTIONS(923),
    [sym_mutable_specifier] = ACTIONS(923),
    [sym_integer_literal] = ACTIONS(925),
    [aux_sym_string_literal_token1] = ACTIONS(925),
    [sym_char_literal] = ACTIONS(925),
    [anon_sym_true] = ACTIONS(923),
    [anon_sym_false] = ACTIONS(923),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(923),
    [sym_super] = ACTIONS(923),
    [sym_crate] = ACTIONS(923),
    [sym_metavariable] = ACTIONS(925),
    [sym__raw_string_literal_start] = ACTIONS(925),
    [sym_float_literal] = ACTIONS(925),
  },
  [156] = {
    [sym_line_comment] = STATE(156),
    [sym_block_comment] = STATE(156),
    [sym_identifier] = ACTIONS(927),
    [anon_sym_SEMI] = ACTIONS(929),
    [anon_sym_LPAREN] = ACTIONS(929),
    [anon_sym_RPAREN] = ACTIONS(929),
    [anon_sym_LBRACK] = ACTIONS(929),
    [anon_sym_RBRACK] = ACTIONS(929),
    [anon_sym_LBRACE] = ACTIONS(929),
    [anon_sym_RBRACE] = ACTIONS(929),
    [anon_sym_EQ_GT] = ACTIONS(929),
    [anon_sym_COLON] = ACTIONS(927),
    [anon_sym_DOLLAR] = ACTIONS(927),
    [anon_sym_PLUS] = ACTIONS(927),
    [anon_sym_STAR] = ACTIONS(927),
    [anon_sym_QMARK] = ACTIONS(929),
    [anon_sym_u8] = ACTIONS(927),
    [anon_sym_i8] = ACTIONS(927),
    [anon_sym_u16] = ACTIONS(927),
    [anon_sym_i16] = ACTIONS(927),
    [anon_sym_u32] = ACTIONS(927),
    [anon_sym_i32] = ACTIONS(927),
    [anon_sym_u64] = ACTIONS(927),
    [anon_sym_i64] = ACTIONS(927),
    [anon_sym_u128] = ACTIONS(927),
    [anon_sym_i128] = ACTIONS(927),
    [anon_sym_isize] = ACTIONS(927),
    [anon_sym_usize] = ACTIONS(927),
    [anon_sym_f32] = ACTIONS(927),
    [anon_sym_f64] = ACTIONS(927),
    [anon_sym_bool] = ACTIONS(927),
    [anon_sym_str] = ACTIONS(927),
    [anon_sym_char] = ACTIONS(927),
    [anon_sym_DASH] = ACTIONS(927),
    [anon_sym_SLASH] = ACTIONS(927),
    [anon_sym_PERCENT] = ACTIONS(927),
    [anon_sym_CARET] = ACTIONS(927),
    [anon_sym_BANG] = ACTIONS(927),
    [anon_sym_AMP] = ACTIONS(927),
    [anon_sym_PIPE] = ACTIONS(927),
    [anon_sym_AMP_AMP] = ACTIONS(929),
    [anon_sym_PIPE_PIPE] = ACTIONS(929),
    [anon_sym_LT_LT] = ACTIONS(927),
    [anon_sym_GT_GT] = ACTIONS(927),
    [anon_sym_PLUS_EQ] = ACTIONS(929),
    [anon_sym_DASH_EQ] = ACTIONS(929),
    [anon_sym_STAR_EQ] = ACTIONS(929),
    [anon_sym_SLASH_EQ] = ACTIONS(929),
    [anon_sym_PERCENT_EQ] = ACTIONS(929),
    [anon_sym_CARET_EQ] = ACTIONS(929),
    [anon_sym_AMP_EQ] = ACTIONS(929),
    [anon_sym_PIPE_EQ] = ACTIONS(929),
    [anon_sym_LT_LT_EQ] = ACTIONS(929),
    [anon_sym_GT_GT_EQ] = ACTIONS(929),
    [anon_sym_EQ] = ACTIONS(927),
    [anon_sym_EQ_EQ] = ACTIONS(929),
    [anon_sym_BANG_EQ] = ACTIONS(929),
    [anon_sym_GT] = ACTIONS(927),
    [anon_sym_LT] = ACTIONS(927),
    [anon_sym_GT_EQ] = ACTIONS(929),
    [anon_sym_LT_EQ] = ACTIONS(929),
    [anon_sym_AT] = ACTIONS(929),
    [anon_sym__] = ACTIONS(927),
    [anon_sym_DOT] = ACTIONS(927),
    [anon_sym_DOT_DOT] = ACTIONS(927),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(929),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(929),
    [anon_sym_COMMA] = ACTIONS(929),
    [anon_sym_COLON_COLON] = ACTIONS(929),
    [anon_sym_DASH_GT] = ACTIONS(929),
    [anon_sym_POUND] = ACTIONS(929),
    [anon_sym_SQUOTE] = ACTIONS(927),
    [anon_sym_as] = ACTIONS(927),
    [anon_sym_async] = ACTIONS(927),
    [anon_sym_await] = ACTIONS(927),
    [anon_sym_break] = ACTIONS(927),
    [anon_sym_const] = ACTIONS(927),
    [anon_sym_continue] = ACTIONS(927),
    [anon_sym_default] = ACTIONS(927),
    [anon_sym_enum] = ACTIONS(927),
    [anon_sym_fn] = ACTIONS(927),
    [anon_sym_for] = ACTIONS(927),
    [anon_sym_gen] = ACTIONS(927),
    [anon_sym_if] = ACTIONS(927),
    [anon_sym_impl] = ACTIONS(927),
    [anon_sym_let] = ACTIONS(927),
    [anon_sym_loop] = ACTIONS(927),
    [anon_sym_match] = ACTIONS(927),
    [anon_sym_mod] = ACTIONS(927),
    [anon_sym_pub] = ACTIONS(927),
    [anon_sym_return] = ACTIONS(927),
    [anon_sym_static] = ACTIONS(927),
    [anon_sym_struct] = ACTIONS(927),
    [anon_sym_trait] = ACTIONS(927),
    [anon_sym_type] = ACTIONS(927),
    [anon_sym_union] = ACTIONS(927),
    [anon_sym_unsafe] = ACTIONS(927),
    [anon_sym_use] = ACTIONS(927),
    [anon_sym_where] = ACTIONS(927),
    [anon_sym_while] = ACTIONS(927),
    [sym_mutable_specifier] = ACTIONS(927),
    [sym_integer_literal] = ACTIONS(929),
    [aux_sym_string_literal_token1] = ACTIONS(929),
    [sym_char_literal] = ACTIONS(929),
    [anon_sym_true] = ACTIONS(927),
    [anon_sym_false] = ACTIONS(927),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(927),
    [sym_super] = ACTIONS(927),
    [sym_crate] = ACTIONS(927),
    [sym_metavariable] = ACTIONS(929),
    [sym__raw_string_literal_start] = ACTIONS(929),
    [sym_float_literal] = ACTIONS(929),
  },
  [157] = {
    [sym_line_comment] = STATE(157),
    [sym_block_comment] = STATE(157),
    [aux_sym__non_special_token_repeat1] = STATE(179),
    [sym_identifier] = ACTIONS(931),
    [anon_sym_SEMI] = ACTIONS(676),
    [anon_sym_LPAREN] = ACTIONS(933),
    [anon_sym_RPAREN] = ACTIONS(933),
    [anon_sym_LBRACK] = ACTIONS(933),
    [anon_sym_RBRACK] = ACTIONS(933),
    [anon_sym_LBRACE] = ACTIONS(933),
    [anon_sym_RBRACE] = ACTIONS(933),
    [anon_sym_EQ_GT] = ACTIONS(676),
    [anon_sym_COLON] = ACTIONS(686),
    [anon_sym_DOLLAR] = ACTIONS(933),
    [anon_sym_PLUS] = ACTIONS(686),
    [anon_sym_STAR] = ACTIONS(686),
    [anon_sym_QMARK] = ACTIONS(676),
    [anon_sym_u8] = ACTIONS(931),
    [anon_sym_i8] = ACTIONS(931),
    [anon_sym_u16] = ACTIONS(931),
    [anon_sym_i16] = ACTIONS(931),
    [anon_sym_u32] = ACTIONS(931),
    [anon_sym_i32] = ACTIONS(931),
    [anon_sym_u64] = ACTIONS(931),
    [anon_sym_i64] = ACTIONS(931),
    [anon_sym_u128] = ACTIONS(931),
    [anon_sym_i128] = ACTIONS(931),
    [anon_sym_isize] = ACTIONS(931),
    [anon_sym_usize] = ACTIONS(931),
    [anon_sym_f32] = ACTIONS(931),
    [anon_sym_f64] = ACTIONS(931),
    [anon_sym_bool] = ACTIONS(931),
    [anon_sym_str] = ACTIONS(931),
    [anon_sym_char] = ACTIONS(931),
    [anon_sym_DASH] = ACTIONS(686),
    [anon_sym_SLASH] = ACTIONS(686),
    [anon_sym_PERCENT] = ACTIONS(686),
    [anon_sym_CARET] = ACTIONS(686),
    [anon_sym_BANG] = ACTIONS(686),
    [anon_sym_AMP] = ACTIONS(686),
    [anon_sym_PIPE] = ACTIONS(686),
    [anon_sym_AMP_AMP] = ACTIONS(676),
    [anon_sym_PIPE_PIPE] = ACTIONS(676),
    [anon_sym_LT_LT] = ACTIONS(686),
    [anon_sym_GT_GT] = ACTIONS(686),
    [anon_sym_PLUS_EQ] = ACTIONS(676),
    [anon_sym_DASH_EQ] = ACTIONS(676),
    [anon_sym_STAR_EQ] = ACTIONS(676),
    [anon_sym_SLASH_EQ] = ACTIONS(676),
    [anon_sym_PERCENT_EQ] = ACTIONS(676),
    [anon_sym_CARET_EQ] = ACTIONS(676),
    [anon_sym_AMP_EQ] = ACTIONS(676),
    [anon_sym_PIPE_EQ] = ACTIONS(676),
    [anon_sym_LT_LT_EQ] = ACTIONS(676),
    [anon_sym_GT_GT_EQ] = ACTIONS(676),
    [anon_sym_EQ] = ACTIONS(686),
    [anon_sym_EQ_EQ] = ACTIONS(676),
    [anon_sym_BANG_EQ] = ACTIONS(676),
    [anon_sym_GT] = ACTIONS(686),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT_EQ] = ACTIONS(676),
    [anon_sym_LT_EQ] = ACTIONS(676),
    [anon_sym_AT] = ACTIONS(676),
    [anon_sym__] = ACTIONS(686),
    [anon_sym_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT] = ACTIONS(686),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(676),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(676),
    [anon_sym_COMMA] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(676),
    [anon_sym_DASH_GT] = ACTIONS(676),
    [anon_sym_POUND] = ACTIONS(676),
    [anon_sym_SQUOTE] = ACTIONS(931),
    [anon_sym_as] = ACTIONS(931),
    [anon_sym_async] = ACTIONS(931),
    [anon_sym_await] = ACTIONS(931),
    [anon_sym_break] = ACTIONS(931),
    [anon_sym_const] = ACTIONS(931),
    [anon_sym_continue] = ACTIONS(931),
    [anon_sym_default] = ACTIONS(931),
    [anon_sym_enum] = ACTIONS(931),
    [anon_sym_fn] = ACTIONS(931),
    [anon_sym_for] = ACTIONS(931),
    [anon_sym_gen] = ACTIONS(931),
    [anon_sym_if] = ACTIONS(931),
    [anon_sym_impl] = ACTIONS(931),
    [anon_sym_let] = ACTIONS(931),
    [anon_sym_loop] = ACTIONS(931),
    [anon_sym_match] = ACTIONS(931),
    [anon_sym_mod] = ACTIONS(931),
    [anon_sym_pub] = ACTIONS(931),
    [anon_sym_return] = ACTIONS(931),
    [anon_sym_static] = ACTIONS(931),
    [anon_sym_struct] = ACTIONS(931),
    [anon_sym_trait] = ACTIONS(931),
    [anon_sym_type] = ACTIONS(931),
    [anon_sym_union] = ACTIONS(931),
    [anon_sym_unsafe] = ACTIONS(931),
    [anon_sym_use] = ACTIONS(931),
    [anon_sym_where] = ACTIONS(931),
    [anon_sym_while] = ACTIONS(931),
    [sym_mutable_specifier] = ACTIONS(931),
    [sym_integer_literal] = ACTIONS(933),
    [aux_sym_string_literal_token1] = ACTIONS(933),
    [sym_char_literal] = ACTIONS(933),
    [anon_sym_true] = ACTIONS(931),
    [anon_sym_false] = ACTIONS(931),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(931),
    [sym_super] = ACTIONS(931),
    [sym_crate] = ACTIONS(931),
    [sym__raw_string_literal_start] = ACTIONS(933),
    [sym_float_literal] = ACTIONS(933),
  },
  [158] = {
    [sym_attribute_item] = STATE(1011),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1623),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(158),
    [sym_block_comment] = STATE(158),
    [aux_sym_enum_variant_list_repeat1] = STATE(207),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_RBRACK] = ACTIONS(935),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(748),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [159] = {
    [sym_line_comment] = STATE(159),
    [sym_block_comment] = STATE(159),
    [sym_identifier] = ACTIONS(937),
    [anon_sym_SEMI] = ACTIONS(939),
    [anon_sym_LPAREN] = ACTIONS(939),
    [anon_sym_RPAREN] = ACTIONS(939),
    [anon_sym_LBRACK] = ACTIONS(939),
    [anon_sym_RBRACK] = ACTIONS(939),
    [anon_sym_LBRACE] = ACTIONS(939),
    [anon_sym_RBRACE] = ACTIONS(939),
    [anon_sym_EQ_GT] = ACTIONS(939),
    [anon_sym_COLON] = ACTIONS(937),
    [anon_sym_DOLLAR] = ACTIONS(937),
    [anon_sym_PLUS] = ACTIONS(937),
    [anon_sym_STAR] = ACTIONS(937),
    [anon_sym_QMARK] = ACTIONS(939),
    [anon_sym_u8] = ACTIONS(937),
    [anon_sym_i8] = ACTIONS(937),
    [anon_sym_u16] = ACTIONS(937),
    [anon_sym_i16] = ACTIONS(937),
    [anon_sym_u32] = ACTIONS(937),
    [anon_sym_i32] = ACTIONS(937),
    [anon_sym_u64] = ACTIONS(937),
    [anon_sym_i64] = ACTIONS(937),
    [anon_sym_u128] = ACTIONS(937),
    [anon_sym_i128] = ACTIONS(937),
    [anon_sym_isize] = ACTIONS(937),
    [anon_sym_usize] = ACTIONS(937),
    [anon_sym_f32] = ACTIONS(937),
    [anon_sym_f64] = ACTIONS(937),
    [anon_sym_bool] = ACTIONS(937),
    [anon_sym_str] = ACTIONS(937),
    [anon_sym_char] = ACTIONS(937),
    [anon_sym_DASH] = ACTIONS(937),
    [anon_sym_SLASH] = ACTIONS(937),
    [anon_sym_PERCENT] = ACTIONS(937),
    [anon_sym_CARET] = ACTIONS(937),
    [anon_sym_BANG] = ACTIONS(937),
    [anon_sym_AMP] = ACTIONS(937),
    [anon_sym_PIPE] = ACTIONS(937),
    [anon_sym_AMP_AMP] = ACTIONS(939),
    [anon_sym_PIPE_PIPE] = ACTIONS(939),
    [anon_sym_LT_LT] = ACTIONS(937),
    [anon_sym_GT_GT] = ACTIONS(937),
    [anon_sym_PLUS_EQ] = ACTIONS(939),
    [anon_sym_DASH_EQ] = ACTIONS(939),
    [anon_sym_STAR_EQ] = ACTIONS(939),
    [anon_sym_SLASH_EQ] = ACTIONS(939),
    [anon_sym_PERCENT_EQ] = ACTIONS(939),
    [anon_sym_CARET_EQ] = ACTIONS(939),
    [anon_sym_AMP_EQ] = ACTIONS(939),
    [anon_sym_PIPE_EQ] = ACTIONS(939),
    [anon_sym_LT_LT_EQ] = ACTIONS(939),
    [anon_sym_GT_GT_EQ] = ACTIONS(939),
    [anon_sym_EQ] = ACTIONS(937),
    [anon_sym_EQ_EQ] = ACTIONS(939),
    [anon_sym_BANG_EQ] = ACTIONS(939),
    [anon_sym_GT] = ACTIONS(937),
    [anon_sym_LT] = ACTIONS(937),
    [anon_sym_GT_EQ] = ACTIONS(939),
    [anon_sym_LT_EQ] = ACTIONS(939),
    [anon_sym_AT] = ACTIONS(939),
    [anon_sym__] = ACTIONS(937),
    [anon_sym_DOT] = ACTIONS(937),
    [anon_sym_DOT_DOT] = ACTIONS(937),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(939),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(939),
    [anon_sym_COMMA] = ACTIONS(939),
    [anon_sym_COLON_COLON] = ACTIONS(939),
    [anon_sym_DASH_GT] = ACTIONS(939),
    [anon_sym_POUND] = ACTIONS(939),
    [anon_sym_SQUOTE] = ACTIONS(937),
    [anon_sym_as] = ACTIONS(937),
    [anon_sym_async] = ACTIONS(937),
    [anon_sym_await] = ACTIONS(937),
    [anon_sym_break] = ACTIONS(937),
    [anon_sym_const] = ACTIONS(937),
    [anon_sym_continue] = ACTIONS(937),
    [anon_sym_default] = ACTIONS(937),
    [anon_sym_enum] = ACTIONS(937),
    [anon_sym_fn] = ACTIONS(937),
    [anon_sym_for] = ACTIONS(937),
    [anon_sym_gen] = ACTIONS(937),
    [anon_sym_if] = ACTIONS(937),
    [anon_sym_impl] = ACTIONS(937),
    [anon_sym_let] = ACTIONS(937),
    [anon_sym_loop] = ACTIONS(937),
    [anon_sym_match] = ACTIONS(937),
    [anon_sym_mod] = ACTIONS(937),
    [anon_sym_pub] = ACTIONS(937),
    [anon_sym_return] = ACTIONS(937),
    [anon_sym_static] = ACTIONS(937),
    [anon_sym_struct] = ACTIONS(937),
    [anon_sym_trait] = ACTIONS(937),
    [anon_sym_type] = ACTIONS(937),
    [anon_sym_union] = ACTIONS(937),
    [anon_sym_unsafe] = ACTIONS(937),
    [anon_sym_use] = ACTIONS(937),
    [anon_sym_where] = ACTIONS(937),
    [anon_sym_while] = ACTIONS(937),
    [sym_mutable_specifier] = ACTIONS(937),
    [sym_integer_literal] = ACTIONS(939),
    [aux_sym_string_literal_token1] = ACTIONS(939),
    [sym_char_literal] = ACTIONS(939),
    [anon_sym_true] = ACTIONS(937),
    [anon_sym_false] = ACTIONS(937),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(937),
    [sym_super] = ACTIONS(937),
    [sym_crate] = ACTIONS(937),
    [sym_metavariable] = ACTIONS(939),
    [sym__raw_string_literal_start] = ACTIONS(939),
    [sym_float_literal] = ACTIONS(939),
  },
  [160] = {
    [sym_line_comment] = STATE(160),
    [sym_block_comment] = STATE(160),
    [sym_identifier] = ACTIONS(941),
    [anon_sym_SEMI] = ACTIONS(943),
    [anon_sym_LPAREN] = ACTIONS(943),
    [anon_sym_RPAREN] = ACTIONS(943),
    [anon_sym_LBRACK] = ACTIONS(943),
    [anon_sym_RBRACK] = ACTIONS(943),
    [anon_sym_LBRACE] = ACTIONS(943),
    [anon_sym_RBRACE] = ACTIONS(943),
    [anon_sym_EQ_GT] = ACTIONS(943),
    [anon_sym_COLON] = ACTIONS(941),
    [anon_sym_DOLLAR] = ACTIONS(941),
    [anon_sym_PLUS] = ACTIONS(941),
    [anon_sym_STAR] = ACTIONS(941),
    [anon_sym_QMARK] = ACTIONS(943),
    [anon_sym_u8] = ACTIONS(941),
    [anon_sym_i8] = ACTIONS(941),
    [anon_sym_u16] = ACTIONS(941),
    [anon_sym_i16] = ACTIONS(941),
    [anon_sym_u32] = ACTIONS(941),
    [anon_sym_i32] = ACTIONS(941),
    [anon_sym_u64] = ACTIONS(941),
    [anon_sym_i64] = ACTIONS(941),
    [anon_sym_u128] = ACTIONS(941),
    [anon_sym_i128] = ACTIONS(941),
    [anon_sym_isize] = ACTIONS(941),
    [anon_sym_usize] = ACTIONS(941),
    [anon_sym_f32] = ACTIONS(941),
    [anon_sym_f64] = ACTIONS(941),
    [anon_sym_bool] = ACTIONS(941),
    [anon_sym_str] = ACTIONS(941),
    [anon_sym_char] = ACTIONS(941),
    [anon_sym_DASH] = ACTIONS(941),
    [anon_sym_SLASH] = ACTIONS(941),
    [anon_sym_PERCENT] = ACTIONS(941),
    [anon_sym_CARET] = ACTIONS(941),
    [anon_sym_BANG] = ACTIONS(941),
    [anon_sym_AMP] = ACTIONS(941),
    [anon_sym_PIPE] = ACTIONS(941),
    [anon_sym_AMP_AMP] = ACTIONS(943),
    [anon_sym_PIPE_PIPE] = ACTIONS(943),
    [anon_sym_LT_LT] = ACTIONS(941),
    [anon_sym_GT_GT] = ACTIONS(941),
    [anon_sym_PLUS_EQ] = ACTIONS(943),
    [anon_sym_DASH_EQ] = ACTIONS(943),
    [anon_sym_STAR_EQ] = ACTIONS(943),
    [anon_sym_SLASH_EQ] = ACTIONS(943),
    [anon_sym_PERCENT_EQ] = ACTIONS(943),
    [anon_sym_CARET_EQ] = ACTIONS(943),
    [anon_sym_AMP_EQ] = ACTIONS(943),
    [anon_sym_PIPE_EQ] = ACTIONS(943),
    [anon_sym_LT_LT_EQ] = ACTIONS(943),
    [anon_sym_GT_GT_EQ] = ACTIONS(943),
    [anon_sym_EQ] = ACTIONS(941),
    [anon_sym_EQ_EQ] = ACTIONS(943),
    [anon_sym_BANG_EQ] = ACTIONS(943),
    [anon_sym_GT] = ACTIONS(941),
    [anon_sym_LT] = ACTIONS(941),
    [anon_sym_GT_EQ] = ACTIONS(943),
    [anon_sym_LT_EQ] = ACTIONS(943),
    [anon_sym_AT] = ACTIONS(943),
    [anon_sym__] = ACTIONS(941),
    [anon_sym_DOT] = ACTIONS(941),
    [anon_sym_DOT_DOT] = ACTIONS(941),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(943),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(943),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_COLON_COLON] = ACTIONS(943),
    [anon_sym_DASH_GT] = ACTIONS(943),
    [anon_sym_POUND] = ACTIONS(943),
    [anon_sym_SQUOTE] = ACTIONS(941),
    [anon_sym_as] = ACTIONS(941),
    [anon_sym_async] = ACTIONS(941),
    [anon_sym_await] = ACTIONS(941),
    [anon_sym_break] = ACTIONS(941),
    [anon_sym_const] = ACTIONS(941),
    [anon_sym_continue] = ACTIONS(941),
    [anon_sym_default] = ACTIONS(941),
    [anon_sym_enum] = ACTIONS(941),
    [anon_sym_fn] = ACTIONS(941),
    [anon_sym_for] = ACTIONS(941),
    [anon_sym_gen] = ACTIONS(941),
    [anon_sym_if] = ACTIONS(941),
    [anon_sym_impl] = ACTIONS(941),
    [anon_sym_let] = ACTIONS(941),
    [anon_sym_loop] = ACTIONS(941),
    [anon_sym_match] = ACTIONS(941),
    [anon_sym_mod] = ACTIONS(941),
    [anon_sym_pub] = ACTIONS(941),
    [anon_sym_return] = ACTIONS(941),
    [anon_sym_static] = ACTIONS(941),
    [anon_sym_struct] = ACTIONS(941),
    [anon_sym_trait] = ACTIONS(941),
    [anon_sym_type] = ACTIONS(941),
    [anon_sym_union] = ACTIONS(941),
    [anon_sym_unsafe] = ACTIONS(941),
    [anon_sym_use] = ACTIONS(941),
    [anon_sym_where] = ACTIONS(941),
    [anon_sym_while] = ACTIONS(941),
    [sym_mutable_specifier] = ACTIONS(941),
    [sym_integer_literal] = ACTIONS(943),
    [aux_sym_string_literal_token1] = ACTIONS(943),
    [sym_char_literal] = ACTIONS(943),
    [anon_sym_true] = ACTIONS(941),
    [anon_sym_false] = ACTIONS(941),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(941),
    [sym_super] = ACTIONS(941),
    [sym_crate] = ACTIONS(941),
    [sym_metavariable] = ACTIONS(943),
    [sym__raw_string_literal_start] = ACTIONS(943),
    [sym_float_literal] = ACTIONS(943),
  },
  [161] = {
    [sym_line_comment] = STATE(161),
    [sym_block_comment] = STATE(161),
    [sym_identifier] = ACTIONS(945),
    [anon_sym_SEMI] = ACTIONS(947),
    [anon_sym_LPAREN] = ACTIONS(947),
    [anon_sym_RPAREN] = ACTIONS(947),
    [anon_sym_LBRACK] = ACTIONS(947),
    [anon_sym_RBRACK] = ACTIONS(947),
    [anon_sym_LBRACE] = ACTIONS(947),
    [anon_sym_RBRACE] = ACTIONS(947),
    [anon_sym_EQ_GT] = ACTIONS(947),
    [anon_sym_COLON] = ACTIONS(945),
    [anon_sym_DOLLAR] = ACTIONS(945),
    [anon_sym_PLUS] = ACTIONS(945),
    [anon_sym_STAR] = ACTIONS(945),
    [anon_sym_QMARK] = ACTIONS(947),
    [anon_sym_u8] = ACTIONS(945),
    [anon_sym_i8] = ACTIONS(945),
    [anon_sym_u16] = ACTIONS(945),
    [anon_sym_i16] = ACTIONS(945),
    [anon_sym_u32] = ACTIONS(945),
    [anon_sym_i32] = ACTIONS(945),
    [anon_sym_u64] = ACTIONS(945),
    [anon_sym_i64] = ACTIONS(945),
    [anon_sym_u128] = ACTIONS(945),
    [anon_sym_i128] = ACTIONS(945),
    [anon_sym_isize] = ACTIONS(945),
    [anon_sym_usize] = ACTIONS(945),
    [anon_sym_f32] = ACTIONS(945),
    [anon_sym_f64] = ACTIONS(945),
    [anon_sym_bool] = ACTIONS(945),
    [anon_sym_str] = ACTIONS(945),
    [anon_sym_char] = ACTIONS(945),
    [anon_sym_DASH] = ACTIONS(945),
    [anon_sym_SLASH] = ACTIONS(945),
    [anon_sym_PERCENT] = ACTIONS(945),
    [anon_sym_CARET] = ACTIONS(945),
    [anon_sym_BANG] = ACTIONS(945),
    [anon_sym_AMP] = ACTIONS(945),
    [anon_sym_PIPE] = ACTIONS(945),
    [anon_sym_AMP_AMP] = ACTIONS(947),
    [anon_sym_PIPE_PIPE] = ACTIONS(947),
    [anon_sym_LT_LT] = ACTIONS(945),
    [anon_sym_GT_GT] = ACTIONS(945),
    [anon_sym_PLUS_EQ] = ACTIONS(947),
    [anon_sym_DASH_EQ] = ACTIONS(947),
    [anon_sym_STAR_EQ] = ACTIONS(947),
    [anon_sym_SLASH_EQ] = ACTIONS(947),
    [anon_sym_PERCENT_EQ] = ACTIONS(947),
    [anon_sym_CARET_EQ] = ACTIONS(947),
    [anon_sym_AMP_EQ] = ACTIONS(947),
    [anon_sym_PIPE_EQ] = ACTIONS(947),
    [anon_sym_LT_LT_EQ] = ACTIONS(947),
    [anon_sym_GT_GT_EQ] = ACTIONS(947),
    [anon_sym_EQ] = ACTIONS(945),
    [anon_sym_EQ_EQ] = ACTIONS(947),
    [anon_sym_BANG_EQ] = ACTIONS(947),
    [anon_sym_GT] = ACTIONS(945),
    [anon_sym_LT] = ACTIONS(945),
    [anon_sym_GT_EQ] = ACTIONS(947),
    [anon_sym_LT_EQ] = ACTIONS(947),
    [anon_sym_AT] = ACTIONS(947),
    [anon_sym__] = ACTIONS(945),
    [anon_sym_DOT] = ACTIONS(945),
    [anon_sym_DOT_DOT] = ACTIONS(945),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(947),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(947),
    [anon_sym_COMMA] = ACTIONS(947),
    [anon_sym_COLON_COLON] = ACTIONS(947),
    [anon_sym_DASH_GT] = ACTIONS(947),
    [anon_sym_POUND] = ACTIONS(947),
    [anon_sym_SQUOTE] = ACTIONS(945),
    [anon_sym_as] = ACTIONS(945),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_await] = ACTIONS(945),
    [anon_sym_break] = ACTIONS(945),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_continue] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(945),
    [anon_sym_enum] = ACTIONS(945),
    [anon_sym_fn] = ACTIONS(945),
    [anon_sym_for] = ACTIONS(945),
    [anon_sym_gen] = ACTIONS(945),
    [anon_sym_if] = ACTIONS(945),
    [anon_sym_impl] = ACTIONS(945),
    [anon_sym_let] = ACTIONS(945),
    [anon_sym_loop] = ACTIONS(945),
    [anon_sym_match] = ACTIONS(945),
    [anon_sym_mod] = ACTIONS(945),
    [anon_sym_pub] = ACTIONS(945),
    [anon_sym_return] = ACTIONS(945),
    [anon_sym_static] = ACTIONS(945),
    [anon_sym_struct] = ACTIONS(945),
    [anon_sym_trait] = ACTIONS(945),
    [anon_sym_type] = ACTIONS(945),
    [anon_sym_union] = ACTIONS(945),
    [anon_sym_unsafe] = ACTIONS(945),
    [anon_sym_use] = ACTIONS(945),
    [anon_sym_where] = ACTIONS(945),
    [anon_sym_while] = ACTIONS(945),
    [sym_mutable_specifier] = ACTIONS(945),
    [sym_integer_literal] = ACTIONS(947),
    [aux_sym_string_literal_token1] = ACTIONS(947),
    [sym_char_literal] = ACTIONS(947),
    [anon_sym_true] = ACTIONS(945),
    [anon_sym_false] = ACTIONS(945),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(945),
    [sym_super] = ACTIONS(945),
    [sym_crate] = ACTIONS(945),
    [sym_metavariable] = ACTIONS(947),
    [sym__raw_string_literal_start] = ACTIONS(947),
    [sym_float_literal] = ACTIONS(947),
  },
  [162] = {
    [sym_attribute_item] = STATE(1011),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1623),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(162),
    [sym_block_comment] = STATE(162),
    [aux_sym_enum_variant_list_repeat1] = STATE(207),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(949),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(748),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [163] = {
    [sym_line_comment] = STATE(163),
    [sym_block_comment] = STATE(163),
    [sym_identifier] = ACTIONS(951),
    [anon_sym_SEMI] = ACTIONS(953),
    [anon_sym_LPAREN] = ACTIONS(953),
    [anon_sym_RPAREN] = ACTIONS(953),
    [anon_sym_LBRACK] = ACTIONS(953),
    [anon_sym_RBRACK] = ACTIONS(953),
    [anon_sym_LBRACE] = ACTIONS(953),
    [anon_sym_RBRACE] = ACTIONS(953),
    [anon_sym_EQ_GT] = ACTIONS(953),
    [anon_sym_COLON] = ACTIONS(951),
    [anon_sym_DOLLAR] = ACTIONS(951),
    [anon_sym_PLUS] = ACTIONS(951),
    [anon_sym_STAR] = ACTIONS(951),
    [anon_sym_QMARK] = ACTIONS(953),
    [anon_sym_u8] = ACTIONS(951),
    [anon_sym_i8] = ACTIONS(951),
    [anon_sym_u16] = ACTIONS(951),
    [anon_sym_i16] = ACTIONS(951),
    [anon_sym_u32] = ACTIONS(951),
    [anon_sym_i32] = ACTIONS(951),
    [anon_sym_u64] = ACTIONS(951),
    [anon_sym_i64] = ACTIONS(951),
    [anon_sym_u128] = ACTIONS(951),
    [anon_sym_i128] = ACTIONS(951),
    [anon_sym_isize] = ACTIONS(951),
    [anon_sym_usize] = ACTIONS(951),
    [anon_sym_f32] = ACTIONS(951),
    [anon_sym_f64] = ACTIONS(951),
    [anon_sym_bool] = ACTIONS(951),
    [anon_sym_str] = ACTIONS(951),
    [anon_sym_char] = ACTIONS(951),
    [anon_sym_DASH] = ACTIONS(951),
    [anon_sym_SLASH] = ACTIONS(951),
    [anon_sym_PERCENT] = ACTIONS(951),
    [anon_sym_CARET] = ACTIONS(951),
    [anon_sym_BANG] = ACTIONS(951),
    [anon_sym_AMP] = ACTIONS(951),
    [anon_sym_PIPE] = ACTIONS(951),
    [anon_sym_AMP_AMP] = ACTIONS(953),
    [anon_sym_PIPE_PIPE] = ACTIONS(953),
    [anon_sym_LT_LT] = ACTIONS(951),
    [anon_sym_GT_GT] = ACTIONS(951),
    [anon_sym_PLUS_EQ] = ACTIONS(953),
    [anon_sym_DASH_EQ] = ACTIONS(953),
    [anon_sym_STAR_EQ] = ACTIONS(953),
    [anon_sym_SLASH_EQ] = ACTIONS(953),
    [anon_sym_PERCENT_EQ] = ACTIONS(953),
    [anon_sym_CARET_EQ] = ACTIONS(953),
    [anon_sym_AMP_EQ] = ACTIONS(953),
    [anon_sym_PIPE_EQ] = ACTIONS(953),
    [anon_sym_LT_LT_EQ] = ACTIONS(953),
    [anon_sym_GT_GT_EQ] = ACTIONS(953),
    [anon_sym_EQ] = ACTIONS(951),
    [anon_sym_EQ_EQ] = ACTIONS(953),
    [anon_sym_BANG_EQ] = ACTIONS(953),
    [anon_sym_GT] = ACTIONS(951),
    [anon_sym_LT] = ACTIONS(951),
    [anon_sym_GT_EQ] = ACTIONS(953),
    [anon_sym_LT_EQ] = ACTIONS(953),
    [anon_sym_AT] = ACTIONS(953),
    [anon_sym__] = ACTIONS(951),
    [anon_sym_DOT] = ACTIONS(951),
    [anon_sym_DOT_DOT] = ACTIONS(951),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(953),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(953),
    [anon_sym_COMMA] = ACTIONS(953),
    [anon_sym_COLON_COLON] = ACTIONS(953),
    [anon_sym_DASH_GT] = ACTIONS(953),
    [anon_sym_POUND] = ACTIONS(953),
    [anon_sym_SQUOTE] = ACTIONS(951),
    [anon_sym_as] = ACTIONS(951),
    [anon_sym_async] = ACTIONS(951),
    [anon_sym_await] = ACTIONS(951),
    [anon_sym_break] = ACTIONS(951),
    [anon_sym_const] = ACTIONS(951),
    [anon_sym_continue] = ACTIONS(951),
    [anon_sym_default] = ACTIONS(951),
    [anon_sym_enum] = ACTIONS(951),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(951),
    [anon_sym_gen] = ACTIONS(951),
    [anon_sym_if] = ACTIONS(951),
    [anon_sym_impl] = ACTIONS(951),
    [anon_sym_let] = ACTIONS(951),
    [anon_sym_loop] = ACTIONS(951),
    [anon_sym_match] = ACTIONS(951),
    [anon_sym_mod] = ACTIONS(951),
    [anon_sym_pub] = ACTIONS(951),
    [anon_sym_return] = ACTIONS(951),
    [anon_sym_static] = ACTIONS(951),
    [anon_sym_struct] = ACTIONS(951),
    [anon_sym_trait] = ACTIONS(951),
    [anon_sym_type] = ACTIONS(951),
    [anon_sym_union] = ACTIONS(951),
    [anon_sym_unsafe] = ACTIONS(951),
    [anon_sym_use] = ACTIONS(951),
    [anon_sym_where] = ACTIONS(951),
    [anon_sym_while] = ACTIONS(951),
    [sym_mutable_specifier] = ACTIONS(951),
    [sym_integer_literal] = ACTIONS(953),
    [aux_sym_string_literal_token1] = ACTIONS(953),
    [sym_char_literal] = ACTIONS(953),
    [anon_sym_true] = ACTIONS(951),
    [anon_sym_false] = ACTIONS(951),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(951),
    [sym_super] = ACTIONS(951),
    [sym_crate] = ACTIONS(951),
    [sym_metavariable] = ACTIONS(953),
    [sym__raw_string_literal_start] = ACTIONS(953),
    [sym_float_literal] = ACTIONS(953),
  },
  [164] = {
    [sym_attribute_item] = STATE(1011),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1623),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(164),
    [sym_block_comment] = STATE(164),
    [aux_sym_enum_variant_list_repeat1] = STATE(207),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_RBRACK] = ACTIONS(955),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(748),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [165] = {
    [sym_attribute_item] = STATE(1011),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1623),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(165),
    [sym_block_comment] = STATE(165),
    [aux_sym_enum_variant_list_repeat1] = STATE(207),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(957),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(748),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [166] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1687),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_let_condition] = STATE(2883),
    [sym__let_chain] = STATE(2887),
    [sym__condition] = STATE(2615),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(166),
    [sym_block_comment] = STATE(166),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(959),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(963),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_let] = ACTIONS(965),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [167] = {
    [sym_attribute_item] = STATE(1011),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1623),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(167),
    [sym_block_comment] = STATE(167),
    [aux_sym_enum_variant_list_repeat1] = STATE(207),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(967),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(748),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [168] = {
    [sym_attribute_item] = STATE(1011),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1623),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(168),
    [sym_block_comment] = STATE(168),
    [aux_sym_enum_variant_list_repeat1] = STATE(207),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_RBRACK] = ACTIONS(969),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(748),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [169] = {
    [sym_attribute_item] = STATE(1011),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1623),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(169),
    [sym_block_comment] = STATE(169),
    [aux_sym_enum_variant_list_repeat1] = STATE(207),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(971),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(748),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [170] = {
    [sym_line_comment] = STATE(170),
    [sym_block_comment] = STATE(170),
    [sym_identifier] = ACTIONS(754),
    [anon_sym_SEMI] = ACTIONS(756),
    [anon_sym_LPAREN] = ACTIONS(756),
    [anon_sym_RPAREN] = ACTIONS(756),
    [anon_sym_LBRACK] = ACTIONS(756),
    [anon_sym_RBRACK] = ACTIONS(756),
    [anon_sym_LBRACE] = ACTIONS(756),
    [anon_sym_RBRACE] = ACTIONS(756),
    [anon_sym_EQ_GT] = ACTIONS(756),
    [anon_sym_COLON] = ACTIONS(754),
    [anon_sym_DOLLAR] = ACTIONS(754),
    [anon_sym_PLUS] = ACTIONS(754),
    [anon_sym_STAR] = ACTIONS(754),
    [anon_sym_QMARK] = ACTIONS(756),
    [anon_sym_u8] = ACTIONS(754),
    [anon_sym_i8] = ACTIONS(754),
    [anon_sym_u16] = ACTIONS(754),
    [anon_sym_i16] = ACTIONS(754),
    [anon_sym_u32] = ACTIONS(754),
    [anon_sym_i32] = ACTIONS(754),
    [anon_sym_u64] = ACTIONS(754),
    [anon_sym_i64] = ACTIONS(754),
    [anon_sym_u128] = ACTIONS(754),
    [anon_sym_i128] = ACTIONS(754),
    [anon_sym_isize] = ACTIONS(754),
    [anon_sym_usize] = ACTIONS(754),
    [anon_sym_f32] = ACTIONS(754),
    [anon_sym_f64] = ACTIONS(754),
    [anon_sym_bool] = ACTIONS(754),
    [anon_sym_str] = ACTIONS(754),
    [anon_sym_char] = ACTIONS(754),
    [anon_sym_DASH] = ACTIONS(754),
    [anon_sym_SLASH] = ACTIONS(754),
    [anon_sym_PERCENT] = ACTIONS(754),
    [anon_sym_CARET] = ACTIONS(754),
    [anon_sym_BANG] = ACTIONS(754),
    [anon_sym_AMP] = ACTIONS(754),
    [anon_sym_PIPE] = ACTIONS(754),
    [anon_sym_AMP_AMP] = ACTIONS(756),
    [anon_sym_PIPE_PIPE] = ACTIONS(756),
    [anon_sym_LT_LT] = ACTIONS(754),
    [anon_sym_GT_GT] = ACTIONS(754),
    [anon_sym_PLUS_EQ] = ACTIONS(756),
    [anon_sym_DASH_EQ] = ACTIONS(756),
    [anon_sym_STAR_EQ] = ACTIONS(756),
    [anon_sym_SLASH_EQ] = ACTIONS(756),
    [anon_sym_PERCENT_EQ] = ACTIONS(756),
    [anon_sym_CARET_EQ] = ACTIONS(756),
    [anon_sym_AMP_EQ] = ACTIONS(756),
    [anon_sym_PIPE_EQ] = ACTIONS(756),
    [anon_sym_LT_LT_EQ] = ACTIONS(756),
    [anon_sym_GT_GT_EQ] = ACTIONS(756),
    [anon_sym_EQ] = ACTIONS(754),
    [anon_sym_EQ_EQ] = ACTIONS(756),
    [anon_sym_BANG_EQ] = ACTIONS(756),
    [anon_sym_GT] = ACTIONS(754),
    [anon_sym_LT] = ACTIONS(754),
    [anon_sym_GT_EQ] = ACTIONS(756),
    [anon_sym_LT_EQ] = ACTIONS(756),
    [anon_sym_AT] = ACTIONS(756),
    [anon_sym__] = ACTIONS(754),
    [anon_sym_DOT] = ACTIONS(754),
    [anon_sym_DOT_DOT] = ACTIONS(754),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(756),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(756),
    [anon_sym_COMMA] = ACTIONS(756),
    [anon_sym_COLON_COLON] = ACTIONS(756),
    [anon_sym_DASH_GT] = ACTIONS(756),
    [anon_sym_POUND] = ACTIONS(756),
    [anon_sym_SQUOTE] = ACTIONS(754),
    [anon_sym_as] = ACTIONS(754),
    [anon_sym_async] = ACTIONS(754),
    [anon_sym_await] = ACTIONS(754),
    [anon_sym_break] = ACTIONS(754),
    [anon_sym_const] = ACTIONS(754),
    [anon_sym_continue] = ACTIONS(754),
    [anon_sym_default] = ACTIONS(754),
    [anon_sym_enum] = ACTIONS(754),
    [anon_sym_fn] = ACTIONS(754),
    [anon_sym_for] = ACTIONS(754),
    [anon_sym_gen] = ACTIONS(754),
    [anon_sym_if] = ACTIONS(754),
    [anon_sym_impl] = ACTIONS(754),
    [anon_sym_let] = ACTIONS(754),
    [anon_sym_loop] = ACTIONS(754),
    [anon_sym_match] = ACTIONS(754),
    [anon_sym_mod] = ACTIONS(754),
    [anon_sym_pub] = ACTIONS(754),
    [anon_sym_return] = ACTIONS(754),
    [anon_sym_static] = ACTIONS(754),
    [anon_sym_struct] = ACTIONS(754),
    [anon_sym_trait] = ACTIONS(754),
    [anon_sym_type] = ACTIONS(754),
    [anon_sym_union] = ACTIONS(754),
    [anon_sym_unsafe] = ACTIONS(754),
    [anon_sym_use] = ACTIONS(754),
    [anon_sym_where] = ACTIONS(754),
    [anon_sym_while] = ACTIONS(754),
    [sym_mutable_specifier] = ACTIONS(754),
    [sym_integer_literal] = ACTIONS(756),
    [aux_sym_string_literal_token1] = ACTIONS(756),
    [sym_char_literal] = ACTIONS(756),
    [anon_sym_true] = ACTIONS(754),
    [anon_sym_false] = ACTIONS(754),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(754),
    [sym_super] = ACTIONS(754),
    [sym_crate] = ACTIONS(754),
    [sym_metavariable] = ACTIONS(756),
    [sym__raw_string_literal_start] = ACTIONS(756),
    [sym_float_literal] = ACTIONS(756),
  },
  [171] = {
    [sym_line_comment] = STATE(171),
    [sym_block_comment] = STATE(171),
    [sym_identifier] = ACTIONS(973),
    [anon_sym_SEMI] = ACTIONS(975),
    [anon_sym_LPAREN] = ACTIONS(975),
    [anon_sym_RPAREN] = ACTIONS(975),
    [anon_sym_LBRACK] = ACTIONS(975),
    [anon_sym_RBRACK] = ACTIONS(975),
    [anon_sym_LBRACE] = ACTIONS(975),
    [anon_sym_RBRACE] = ACTIONS(975),
    [anon_sym_EQ_GT] = ACTIONS(975),
    [anon_sym_COLON] = ACTIONS(973),
    [anon_sym_DOLLAR] = ACTIONS(973),
    [anon_sym_PLUS] = ACTIONS(973),
    [anon_sym_STAR] = ACTIONS(973),
    [anon_sym_QMARK] = ACTIONS(975),
    [anon_sym_u8] = ACTIONS(973),
    [anon_sym_i8] = ACTIONS(973),
    [anon_sym_u16] = ACTIONS(973),
    [anon_sym_i16] = ACTIONS(973),
    [anon_sym_u32] = ACTIONS(973),
    [anon_sym_i32] = ACTIONS(973),
    [anon_sym_u64] = ACTIONS(973),
    [anon_sym_i64] = ACTIONS(973),
    [anon_sym_u128] = ACTIONS(973),
    [anon_sym_i128] = ACTIONS(973),
    [anon_sym_isize] = ACTIONS(973),
    [anon_sym_usize] = ACTIONS(973),
    [anon_sym_f32] = ACTIONS(973),
    [anon_sym_f64] = ACTIONS(973),
    [anon_sym_bool] = ACTIONS(973),
    [anon_sym_str] = ACTIONS(973),
    [anon_sym_char] = ACTIONS(973),
    [anon_sym_DASH] = ACTIONS(973),
    [anon_sym_SLASH] = ACTIONS(973),
    [anon_sym_PERCENT] = ACTIONS(973),
    [anon_sym_CARET] = ACTIONS(973),
    [anon_sym_BANG] = ACTIONS(973),
    [anon_sym_AMP] = ACTIONS(973),
    [anon_sym_PIPE] = ACTIONS(973),
    [anon_sym_AMP_AMP] = ACTIONS(975),
    [anon_sym_PIPE_PIPE] = ACTIONS(975),
    [anon_sym_LT_LT] = ACTIONS(973),
    [anon_sym_GT_GT] = ACTIONS(973),
    [anon_sym_PLUS_EQ] = ACTIONS(975),
    [anon_sym_DASH_EQ] = ACTIONS(975),
    [anon_sym_STAR_EQ] = ACTIONS(975),
    [anon_sym_SLASH_EQ] = ACTIONS(975),
    [anon_sym_PERCENT_EQ] = ACTIONS(975),
    [anon_sym_CARET_EQ] = ACTIONS(975),
    [anon_sym_AMP_EQ] = ACTIONS(975),
    [anon_sym_PIPE_EQ] = ACTIONS(975),
    [anon_sym_LT_LT_EQ] = ACTIONS(975),
    [anon_sym_GT_GT_EQ] = ACTIONS(975),
    [anon_sym_EQ] = ACTIONS(973),
    [anon_sym_EQ_EQ] = ACTIONS(975),
    [anon_sym_BANG_EQ] = ACTIONS(975),
    [anon_sym_GT] = ACTIONS(973),
    [anon_sym_LT] = ACTIONS(973),
    [anon_sym_GT_EQ] = ACTIONS(975),
    [anon_sym_LT_EQ] = ACTIONS(975),
    [anon_sym_AT] = ACTIONS(975),
    [anon_sym__] = ACTIONS(973),
    [anon_sym_DOT] = ACTIONS(973),
    [anon_sym_DOT_DOT] = ACTIONS(973),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(975),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(975),
    [anon_sym_COMMA] = ACTIONS(975),
    [anon_sym_COLON_COLON] = ACTIONS(975),
    [anon_sym_DASH_GT] = ACTIONS(975),
    [anon_sym_POUND] = ACTIONS(975),
    [anon_sym_SQUOTE] = ACTIONS(973),
    [anon_sym_as] = ACTIONS(973),
    [anon_sym_async] = ACTIONS(973),
    [anon_sym_await] = ACTIONS(973),
    [anon_sym_break] = ACTIONS(973),
    [anon_sym_const] = ACTIONS(973),
    [anon_sym_continue] = ACTIONS(973),
    [anon_sym_default] = ACTIONS(973),
    [anon_sym_enum] = ACTIONS(973),
    [anon_sym_fn] = ACTIONS(973),
    [anon_sym_for] = ACTIONS(973),
    [anon_sym_gen] = ACTIONS(973),
    [anon_sym_if] = ACTIONS(973),
    [anon_sym_impl] = ACTIONS(973),
    [anon_sym_let] = ACTIONS(973),
    [anon_sym_loop] = ACTIONS(973),
    [anon_sym_match] = ACTIONS(973),
    [anon_sym_mod] = ACTIONS(973),
    [anon_sym_pub] = ACTIONS(973),
    [anon_sym_return] = ACTIONS(973),
    [anon_sym_static] = ACTIONS(973),
    [anon_sym_struct] = ACTIONS(973),
    [anon_sym_trait] = ACTIONS(973),
    [anon_sym_type] = ACTIONS(973),
    [anon_sym_union] = ACTIONS(973),
    [anon_sym_unsafe] = ACTIONS(973),
    [anon_sym_use] = ACTIONS(973),
    [anon_sym_where] = ACTIONS(973),
    [anon_sym_while] = ACTIONS(973),
    [sym_mutable_specifier] = ACTIONS(973),
    [sym_integer_literal] = ACTIONS(975),
    [aux_sym_string_literal_token1] = ACTIONS(975),
    [sym_char_literal] = ACTIONS(975),
    [anon_sym_true] = ACTIONS(973),
    [anon_sym_false] = ACTIONS(973),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(973),
    [sym_super] = ACTIONS(973),
    [sym_crate] = ACTIONS(973),
    [sym_metavariable] = ACTIONS(975),
    [sym__raw_string_literal_start] = ACTIONS(975),
    [sym_float_literal] = ACTIONS(975),
  },
  [172] = {
    [sym_line_comment] = STATE(172),
    [sym_block_comment] = STATE(172),
    [sym_identifier] = ACTIONS(977),
    [anon_sym_SEMI] = ACTIONS(979),
    [anon_sym_LPAREN] = ACTIONS(979),
    [anon_sym_RPAREN] = ACTIONS(979),
    [anon_sym_LBRACK] = ACTIONS(979),
    [anon_sym_RBRACK] = ACTIONS(979),
    [anon_sym_LBRACE] = ACTIONS(979),
    [anon_sym_RBRACE] = ACTIONS(979),
    [anon_sym_EQ_GT] = ACTIONS(979),
    [anon_sym_COLON] = ACTIONS(977),
    [anon_sym_DOLLAR] = ACTIONS(977),
    [anon_sym_PLUS] = ACTIONS(977),
    [anon_sym_STAR] = ACTIONS(977),
    [anon_sym_QMARK] = ACTIONS(979),
    [anon_sym_u8] = ACTIONS(977),
    [anon_sym_i8] = ACTIONS(977),
    [anon_sym_u16] = ACTIONS(977),
    [anon_sym_i16] = ACTIONS(977),
    [anon_sym_u32] = ACTIONS(977),
    [anon_sym_i32] = ACTIONS(977),
    [anon_sym_u64] = ACTIONS(977),
    [anon_sym_i64] = ACTIONS(977),
    [anon_sym_u128] = ACTIONS(977),
    [anon_sym_i128] = ACTIONS(977),
    [anon_sym_isize] = ACTIONS(977),
    [anon_sym_usize] = ACTIONS(977),
    [anon_sym_f32] = ACTIONS(977),
    [anon_sym_f64] = ACTIONS(977),
    [anon_sym_bool] = ACTIONS(977),
    [anon_sym_str] = ACTIONS(977),
    [anon_sym_char] = ACTIONS(977),
    [anon_sym_DASH] = ACTIONS(977),
    [anon_sym_SLASH] = ACTIONS(977),
    [anon_sym_PERCENT] = ACTIONS(977),
    [anon_sym_CARET] = ACTIONS(977),
    [anon_sym_BANG] = ACTIONS(977),
    [anon_sym_AMP] = ACTIONS(977),
    [anon_sym_PIPE] = ACTIONS(977),
    [anon_sym_AMP_AMP] = ACTIONS(979),
    [anon_sym_PIPE_PIPE] = ACTIONS(979),
    [anon_sym_LT_LT] = ACTIONS(977),
    [anon_sym_GT_GT] = ACTIONS(977),
    [anon_sym_PLUS_EQ] = ACTIONS(979),
    [anon_sym_DASH_EQ] = ACTIONS(979),
    [anon_sym_STAR_EQ] = ACTIONS(979),
    [anon_sym_SLASH_EQ] = ACTIONS(979),
    [anon_sym_PERCENT_EQ] = ACTIONS(979),
    [anon_sym_CARET_EQ] = ACTIONS(979),
    [anon_sym_AMP_EQ] = ACTIONS(979),
    [anon_sym_PIPE_EQ] = ACTIONS(979),
    [anon_sym_LT_LT_EQ] = ACTIONS(979),
    [anon_sym_GT_GT_EQ] = ACTIONS(979),
    [anon_sym_EQ] = ACTIONS(977),
    [anon_sym_EQ_EQ] = ACTIONS(979),
    [anon_sym_BANG_EQ] = ACTIONS(979),
    [anon_sym_GT] = ACTIONS(977),
    [anon_sym_LT] = ACTIONS(977),
    [anon_sym_GT_EQ] = ACTIONS(979),
    [anon_sym_LT_EQ] = ACTIONS(979),
    [anon_sym_AT] = ACTIONS(979),
    [anon_sym__] = ACTIONS(977),
    [anon_sym_DOT] = ACTIONS(977),
    [anon_sym_DOT_DOT] = ACTIONS(977),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(979),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(979),
    [anon_sym_COMMA] = ACTIONS(979),
    [anon_sym_COLON_COLON] = ACTIONS(979),
    [anon_sym_DASH_GT] = ACTIONS(979),
    [anon_sym_POUND] = ACTIONS(979),
    [anon_sym_SQUOTE] = ACTIONS(977),
    [anon_sym_as] = ACTIONS(977),
    [anon_sym_async] = ACTIONS(977),
    [anon_sym_await] = ACTIONS(977),
    [anon_sym_break] = ACTIONS(977),
    [anon_sym_const] = ACTIONS(977),
    [anon_sym_continue] = ACTIONS(977),
    [anon_sym_default] = ACTIONS(977),
    [anon_sym_enum] = ACTIONS(977),
    [anon_sym_fn] = ACTIONS(977),
    [anon_sym_for] = ACTIONS(977),
    [anon_sym_gen] = ACTIONS(977),
    [anon_sym_if] = ACTIONS(977),
    [anon_sym_impl] = ACTIONS(977),
    [anon_sym_let] = ACTIONS(977),
    [anon_sym_loop] = ACTIONS(977),
    [anon_sym_match] = ACTIONS(977),
    [anon_sym_mod] = ACTIONS(977),
    [anon_sym_pub] = ACTIONS(977),
    [anon_sym_return] = ACTIONS(977),
    [anon_sym_static] = ACTIONS(977),
    [anon_sym_struct] = ACTIONS(977),
    [anon_sym_trait] = ACTIONS(977),
    [anon_sym_type] = ACTIONS(977),
    [anon_sym_union] = ACTIONS(977),
    [anon_sym_unsafe] = ACTIONS(977),
    [anon_sym_use] = ACTIONS(977),
    [anon_sym_where] = ACTIONS(977),
    [anon_sym_while] = ACTIONS(977),
    [sym_mutable_specifier] = ACTIONS(977),
    [sym_integer_literal] = ACTIONS(979),
    [aux_sym_string_literal_token1] = ACTIONS(979),
    [sym_char_literal] = ACTIONS(979),
    [anon_sym_true] = ACTIONS(977),
    [anon_sym_false] = ACTIONS(977),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(977),
    [sym_super] = ACTIONS(977),
    [sym_crate] = ACTIONS(977),
    [sym_metavariable] = ACTIONS(979),
    [sym__raw_string_literal_start] = ACTIONS(979),
    [sym_float_literal] = ACTIONS(979),
  },
  [173] = {
    [sym_line_comment] = STATE(173),
    [sym_block_comment] = STATE(173),
    [sym_identifier] = ACTIONS(754),
    [anon_sym_SEMI] = ACTIONS(756),
    [anon_sym_LPAREN] = ACTIONS(756),
    [anon_sym_RPAREN] = ACTIONS(756),
    [anon_sym_LBRACK] = ACTIONS(756),
    [anon_sym_RBRACK] = ACTIONS(756),
    [anon_sym_LBRACE] = ACTIONS(756),
    [anon_sym_RBRACE] = ACTIONS(756),
    [anon_sym_EQ_GT] = ACTIONS(756),
    [anon_sym_COLON] = ACTIONS(981),
    [anon_sym_DOLLAR] = ACTIONS(754),
    [anon_sym_PLUS] = ACTIONS(754),
    [anon_sym_STAR] = ACTIONS(754),
    [anon_sym_QMARK] = ACTIONS(756),
    [anon_sym_u8] = ACTIONS(754),
    [anon_sym_i8] = ACTIONS(754),
    [anon_sym_u16] = ACTIONS(754),
    [anon_sym_i16] = ACTIONS(754),
    [anon_sym_u32] = ACTIONS(754),
    [anon_sym_i32] = ACTIONS(754),
    [anon_sym_u64] = ACTIONS(754),
    [anon_sym_i64] = ACTIONS(754),
    [anon_sym_u128] = ACTIONS(754),
    [anon_sym_i128] = ACTIONS(754),
    [anon_sym_isize] = ACTIONS(754),
    [anon_sym_usize] = ACTIONS(754),
    [anon_sym_f32] = ACTIONS(754),
    [anon_sym_f64] = ACTIONS(754),
    [anon_sym_bool] = ACTIONS(754),
    [anon_sym_str] = ACTIONS(754),
    [anon_sym_char] = ACTIONS(754),
    [anon_sym_DASH] = ACTIONS(754),
    [anon_sym_SLASH] = ACTIONS(754),
    [anon_sym_PERCENT] = ACTIONS(754),
    [anon_sym_CARET] = ACTIONS(754),
    [anon_sym_BANG] = ACTIONS(754),
    [anon_sym_AMP] = ACTIONS(754),
    [anon_sym_PIPE] = ACTIONS(754),
    [anon_sym_AMP_AMP] = ACTIONS(756),
    [anon_sym_PIPE_PIPE] = ACTIONS(756),
    [anon_sym_LT_LT] = ACTIONS(754),
    [anon_sym_GT_GT] = ACTIONS(754),
    [anon_sym_PLUS_EQ] = ACTIONS(756),
    [anon_sym_DASH_EQ] = ACTIONS(756),
    [anon_sym_STAR_EQ] = ACTIONS(756),
    [anon_sym_SLASH_EQ] = ACTIONS(756),
    [anon_sym_PERCENT_EQ] = ACTIONS(756),
    [anon_sym_CARET_EQ] = ACTIONS(756),
    [anon_sym_AMP_EQ] = ACTIONS(756),
    [anon_sym_PIPE_EQ] = ACTIONS(756),
    [anon_sym_LT_LT_EQ] = ACTIONS(756),
    [anon_sym_GT_GT_EQ] = ACTIONS(756),
    [anon_sym_EQ] = ACTIONS(754),
    [anon_sym_EQ_EQ] = ACTIONS(756),
    [anon_sym_BANG_EQ] = ACTIONS(756),
    [anon_sym_GT] = ACTIONS(754),
    [anon_sym_LT] = ACTIONS(754),
    [anon_sym_GT_EQ] = ACTIONS(756),
    [anon_sym_LT_EQ] = ACTIONS(756),
    [anon_sym_AT] = ACTIONS(756),
    [anon_sym__] = ACTIONS(754),
    [anon_sym_DOT] = ACTIONS(754),
    [anon_sym_DOT_DOT] = ACTIONS(754),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(756),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(756),
    [anon_sym_COMMA] = ACTIONS(756),
    [anon_sym_COLON_COLON] = ACTIONS(756),
    [anon_sym_DASH_GT] = ACTIONS(756),
    [anon_sym_POUND] = ACTIONS(756),
    [anon_sym_SQUOTE] = ACTIONS(754),
    [anon_sym_as] = ACTIONS(754),
    [anon_sym_async] = ACTIONS(754),
    [anon_sym_await] = ACTIONS(754),
    [anon_sym_break] = ACTIONS(754),
    [anon_sym_const] = ACTIONS(754),
    [anon_sym_continue] = ACTIONS(754),
    [anon_sym_default] = ACTIONS(754),
    [anon_sym_enum] = ACTIONS(754),
    [anon_sym_fn] = ACTIONS(754),
    [anon_sym_for] = ACTIONS(754),
    [anon_sym_gen] = ACTIONS(754),
    [anon_sym_if] = ACTIONS(754),
    [anon_sym_impl] = ACTIONS(754),
    [anon_sym_let] = ACTIONS(754),
    [anon_sym_loop] = ACTIONS(754),
    [anon_sym_match] = ACTIONS(754),
    [anon_sym_mod] = ACTIONS(754),
    [anon_sym_pub] = ACTIONS(754),
    [anon_sym_return] = ACTIONS(754),
    [anon_sym_static] = ACTIONS(754),
    [anon_sym_struct] = ACTIONS(754),
    [anon_sym_trait] = ACTIONS(754),
    [anon_sym_type] = ACTIONS(754),
    [anon_sym_union] = ACTIONS(754),
    [anon_sym_unsafe] = ACTIONS(754),
    [anon_sym_use] = ACTIONS(754),
    [anon_sym_where] = ACTIONS(754),
    [anon_sym_while] = ACTIONS(754),
    [sym_mutable_specifier] = ACTIONS(754),
    [sym_integer_literal] = ACTIONS(756),
    [aux_sym_string_literal_token1] = ACTIONS(756),
    [sym_char_literal] = ACTIONS(756),
    [anon_sym_true] = ACTIONS(754),
    [anon_sym_false] = ACTIONS(754),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(754),
    [sym_super] = ACTIONS(754),
    [sym_crate] = ACTIONS(754),
    [sym_metavariable] = ACTIONS(756),
    [sym__raw_string_literal_start] = ACTIONS(756),
    [sym_float_literal] = ACTIONS(756),
  },
  [174] = {
    [sym_line_comment] = STATE(174),
    [sym_block_comment] = STATE(174),
    [sym_identifier] = ACTIONS(983),
    [anon_sym_SEMI] = ACTIONS(985),
    [anon_sym_LPAREN] = ACTIONS(985),
    [anon_sym_RPAREN] = ACTIONS(985),
    [anon_sym_LBRACK] = ACTIONS(985),
    [anon_sym_RBRACK] = ACTIONS(985),
    [anon_sym_LBRACE] = ACTIONS(985),
    [anon_sym_RBRACE] = ACTIONS(985),
    [anon_sym_EQ_GT] = ACTIONS(985),
    [anon_sym_COLON] = ACTIONS(983),
    [anon_sym_DOLLAR] = ACTIONS(983),
    [anon_sym_PLUS] = ACTIONS(983),
    [anon_sym_STAR] = ACTIONS(983),
    [anon_sym_QMARK] = ACTIONS(985),
    [anon_sym_u8] = ACTIONS(983),
    [anon_sym_i8] = ACTIONS(983),
    [anon_sym_u16] = ACTIONS(983),
    [anon_sym_i16] = ACTIONS(983),
    [anon_sym_u32] = ACTIONS(983),
    [anon_sym_i32] = ACTIONS(983),
    [anon_sym_u64] = ACTIONS(983),
    [anon_sym_i64] = ACTIONS(983),
    [anon_sym_u128] = ACTIONS(983),
    [anon_sym_i128] = ACTIONS(983),
    [anon_sym_isize] = ACTIONS(983),
    [anon_sym_usize] = ACTIONS(983),
    [anon_sym_f32] = ACTIONS(983),
    [anon_sym_f64] = ACTIONS(983),
    [anon_sym_bool] = ACTIONS(983),
    [anon_sym_str] = ACTIONS(983),
    [anon_sym_char] = ACTIONS(983),
    [anon_sym_DASH] = ACTIONS(983),
    [anon_sym_SLASH] = ACTIONS(983),
    [anon_sym_PERCENT] = ACTIONS(983),
    [anon_sym_CARET] = ACTIONS(983),
    [anon_sym_BANG] = ACTIONS(983),
    [anon_sym_AMP] = ACTIONS(983),
    [anon_sym_PIPE] = ACTIONS(983),
    [anon_sym_AMP_AMP] = ACTIONS(985),
    [anon_sym_PIPE_PIPE] = ACTIONS(985),
    [anon_sym_LT_LT] = ACTIONS(983),
    [anon_sym_GT_GT] = ACTIONS(983),
    [anon_sym_PLUS_EQ] = ACTIONS(985),
    [anon_sym_DASH_EQ] = ACTIONS(985),
    [anon_sym_STAR_EQ] = ACTIONS(985),
    [anon_sym_SLASH_EQ] = ACTIONS(985),
    [anon_sym_PERCENT_EQ] = ACTIONS(985),
    [anon_sym_CARET_EQ] = ACTIONS(985),
    [anon_sym_AMP_EQ] = ACTIONS(985),
    [anon_sym_PIPE_EQ] = ACTIONS(985),
    [anon_sym_LT_LT_EQ] = ACTIONS(985),
    [anon_sym_GT_GT_EQ] = ACTIONS(985),
    [anon_sym_EQ] = ACTIONS(983),
    [anon_sym_EQ_EQ] = ACTIONS(985),
    [anon_sym_BANG_EQ] = ACTIONS(985),
    [anon_sym_GT] = ACTIONS(983),
    [anon_sym_LT] = ACTIONS(983),
    [anon_sym_GT_EQ] = ACTIONS(985),
    [anon_sym_LT_EQ] = ACTIONS(985),
    [anon_sym_AT] = ACTIONS(985),
    [anon_sym__] = ACTIONS(983),
    [anon_sym_DOT] = ACTIONS(983),
    [anon_sym_DOT_DOT] = ACTIONS(983),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(985),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(985),
    [anon_sym_COMMA] = ACTIONS(985),
    [anon_sym_COLON_COLON] = ACTIONS(985),
    [anon_sym_DASH_GT] = ACTIONS(985),
    [anon_sym_POUND] = ACTIONS(985),
    [anon_sym_SQUOTE] = ACTIONS(983),
    [anon_sym_as] = ACTIONS(983),
    [anon_sym_async] = ACTIONS(983),
    [anon_sym_await] = ACTIONS(983),
    [anon_sym_break] = ACTIONS(983),
    [anon_sym_const] = ACTIONS(983),
    [anon_sym_continue] = ACTIONS(983),
    [anon_sym_default] = ACTIONS(983),
    [anon_sym_enum] = ACTIONS(983),
    [anon_sym_fn] = ACTIONS(983),
    [anon_sym_for] = ACTIONS(983),
    [anon_sym_gen] = ACTIONS(983),
    [anon_sym_if] = ACTIONS(983),
    [anon_sym_impl] = ACTIONS(983),
    [anon_sym_let] = ACTIONS(983),
    [anon_sym_loop] = ACTIONS(983),
    [anon_sym_match] = ACTIONS(983),
    [anon_sym_mod] = ACTIONS(983),
    [anon_sym_pub] = ACTIONS(983),
    [anon_sym_return] = ACTIONS(983),
    [anon_sym_static] = ACTIONS(983),
    [anon_sym_struct] = ACTIONS(983),
    [anon_sym_trait] = ACTIONS(983),
    [anon_sym_type] = ACTIONS(983),
    [anon_sym_union] = ACTIONS(983),
    [anon_sym_unsafe] = ACTIONS(983),
    [anon_sym_use] = ACTIONS(983),
    [anon_sym_where] = ACTIONS(983),
    [anon_sym_while] = ACTIONS(983),
    [sym_mutable_specifier] = ACTIONS(983),
    [sym_integer_literal] = ACTIONS(985),
    [aux_sym_string_literal_token1] = ACTIONS(985),
    [sym_char_literal] = ACTIONS(985),
    [anon_sym_true] = ACTIONS(983),
    [anon_sym_false] = ACTIONS(983),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(983),
    [sym_super] = ACTIONS(983),
    [sym_crate] = ACTIONS(983),
    [sym_metavariable] = ACTIONS(985),
    [sym__raw_string_literal_start] = ACTIONS(985),
    [sym_float_literal] = ACTIONS(985),
  },
  [175] = {
    [sym_line_comment] = STATE(175),
    [sym_block_comment] = STATE(175),
    [sym_identifier] = ACTIONS(987),
    [anon_sym_SEMI] = ACTIONS(989),
    [anon_sym_LPAREN] = ACTIONS(989),
    [anon_sym_RPAREN] = ACTIONS(989),
    [anon_sym_LBRACK] = ACTIONS(989),
    [anon_sym_RBRACK] = ACTIONS(989),
    [anon_sym_LBRACE] = ACTIONS(989),
    [anon_sym_RBRACE] = ACTIONS(989),
    [anon_sym_EQ_GT] = ACTIONS(989),
    [anon_sym_COLON] = ACTIONS(987),
    [anon_sym_DOLLAR] = ACTIONS(987),
    [anon_sym_PLUS] = ACTIONS(987),
    [anon_sym_STAR] = ACTIONS(987),
    [anon_sym_QMARK] = ACTIONS(989),
    [anon_sym_u8] = ACTIONS(987),
    [anon_sym_i8] = ACTIONS(987),
    [anon_sym_u16] = ACTIONS(987),
    [anon_sym_i16] = ACTIONS(987),
    [anon_sym_u32] = ACTIONS(987),
    [anon_sym_i32] = ACTIONS(987),
    [anon_sym_u64] = ACTIONS(987),
    [anon_sym_i64] = ACTIONS(987),
    [anon_sym_u128] = ACTIONS(987),
    [anon_sym_i128] = ACTIONS(987),
    [anon_sym_isize] = ACTIONS(987),
    [anon_sym_usize] = ACTIONS(987),
    [anon_sym_f32] = ACTIONS(987),
    [anon_sym_f64] = ACTIONS(987),
    [anon_sym_bool] = ACTIONS(987),
    [anon_sym_str] = ACTIONS(987),
    [anon_sym_char] = ACTIONS(987),
    [anon_sym_DASH] = ACTIONS(987),
    [anon_sym_SLASH] = ACTIONS(987),
    [anon_sym_PERCENT] = ACTIONS(987),
    [anon_sym_CARET] = ACTIONS(987),
    [anon_sym_BANG] = ACTIONS(987),
    [anon_sym_AMP] = ACTIONS(987),
    [anon_sym_PIPE] = ACTIONS(987),
    [anon_sym_AMP_AMP] = ACTIONS(989),
    [anon_sym_PIPE_PIPE] = ACTIONS(989),
    [anon_sym_LT_LT] = ACTIONS(987),
    [anon_sym_GT_GT] = ACTIONS(987),
    [anon_sym_PLUS_EQ] = ACTIONS(989),
    [anon_sym_DASH_EQ] = ACTIONS(989),
    [anon_sym_STAR_EQ] = ACTIONS(989),
    [anon_sym_SLASH_EQ] = ACTIONS(989),
    [anon_sym_PERCENT_EQ] = ACTIONS(989),
    [anon_sym_CARET_EQ] = ACTIONS(989),
    [anon_sym_AMP_EQ] = ACTIONS(989),
    [anon_sym_PIPE_EQ] = ACTIONS(989),
    [anon_sym_LT_LT_EQ] = ACTIONS(989),
    [anon_sym_GT_GT_EQ] = ACTIONS(989),
    [anon_sym_EQ] = ACTIONS(987),
    [anon_sym_EQ_EQ] = ACTIONS(989),
    [anon_sym_BANG_EQ] = ACTIONS(989),
    [anon_sym_GT] = ACTIONS(987),
    [anon_sym_LT] = ACTIONS(987),
    [anon_sym_GT_EQ] = ACTIONS(989),
    [anon_sym_LT_EQ] = ACTIONS(989),
    [anon_sym_AT] = ACTIONS(989),
    [anon_sym__] = ACTIONS(987),
    [anon_sym_DOT] = ACTIONS(987),
    [anon_sym_DOT_DOT] = ACTIONS(987),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(989),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(989),
    [anon_sym_COMMA] = ACTIONS(989),
    [anon_sym_COLON_COLON] = ACTIONS(989),
    [anon_sym_DASH_GT] = ACTIONS(989),
    [anon_sym_POUND] = ACTIONS(989),
    [anon_sym_SQUOTE] = ACTIONS(987),
    [anon_sym_as] = ACTIONS(987),
    [anon_sym_async] = ACTIONS(987),
    [anon_sym_await] = ACTIONS(987),
    [anon_sym_break] = ACTIONS(987),
    [anon_sym_const] = ACTIONS(987),
    [anon_sym_continue] = ACTIONS(987),
    [anon_sym_default] = ACTIONS(987),
    [anon_sym_enum] = ACTIONS(987),
    [anon_sym_fn] = ACTIONS(987),
    [anon_sym_for] = ACTIONS(987),
    [anon_sym_gen] = ACTIONS(987),
    [anon_sym_if] = ACTIONS(987),
    [anon_sym_impl] = ACTIONS(987),
    [anon_sym_let] = ACTIONS(987),
    [anon_sym_loop] = ACTIONS(987),
    [anon_sym_match] = ACTIONS(987),
    [anon_sym_mod] = ACTIONS(987),
    [anon_sym_pub] = ACTIONS(987),
    [anon_sym_return] = ACTIONS(987),
    [anon_sym_static] = ACTIONS(987),
    [anon_sym_struct] = ACTIONS(987),
    [anon_sym_trait] = ACTIONS(987),
    [anon_sym_type] = ACTIONS(987),
    [anon_sym_union] = ACTIONS(987),
    [anon_sym_unsafe] = ACTIONS(987),
    [anon_sym_use] = ACTIONS(987),
    [anon_sym_where] = ACTIONS(987),
    [anon_sym_while] = ACTIONS(987),
    [sym_mutable_specifier] = ACTIONS(987),
    [sym_integer_literal] = ACTIONS(989),
    [aux_sym_string_literal_token1] = ACTIONS(989),
    [sym_char_literal] = ACTIONS(989),
    [anon_sym_true] = ACTIONS(987),
    [anon_sym_false] = ACTIONS(987),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(987),
    [sym_super] = ACTIONS(987),
    [sym_crate] = ACTIONS(987),
    [sym_metavariable] = ACTIONS(989),
    [sym__raw_string_literal_start] = ACTIONS(989),
    [sym_float_literal] = ACTIONS(989),
  },
  [176] = {
    [sym_line_comment] = STATE(176),
    [sym_block_comment] = STATE(176),
    [sym_identifier] = ACTIONS(991),
    [anon_sym_SEMI] = ACTIONS(993),
    [anon_sym_LPAREN] = ACTIONS(993),
    [anon_sym_RPAREN] = ACTIONS(993),
    [anon_sym_LBRACK] = ACTIONS(993),
    [anon_sym_RBRACK] = ACTIONS(993),
    [anon_sym_LBRACE] = ACTIONS(993),
    [anon_sym_RBRACE] = ACTIONS(993),
    [anon_sym_EQ_GT] = ACTIONS(993),
    [anon_sym_COLON] = ACTIONS(991),
    [anon_sym_DOLLAR] = ACTIONS(991),
    [anon_sym_PLUS] = ACTIONS(991),
    [anon_sym_STAR] = ACTIONS(991),
    [anon_sym_QMARK] = ACTIONS(993),
    [anon_sym_u8] = ACTIONS(991),
    [anon_sym_i8] = ACTIONS(991),
    [anon_sym_u16] = ACTIONS(991),
    [anon_sym_i16] = ACTIONS(991),
    [anon_sym_u32] = ACTIONS(991),
    [anon_sym_i32] = ACTIONS(991),
    [anon_sym_u64] = ACTIONS(991),
    [anon_sym_i64] = ACTIONS(991),
    [anon_sym_u128] = ACTIONS(991),
    [anon_sym_i128] = ACTIONS(991),
    [anon_sym_isize] = ACTIONS(991),
    [anon_sym_usize] = ACTIONS(991),
    [anon_sym_f32] = ACTIONS(991),
    [anon_sym_f64] = ACTIONS(991),
    [anon_sym_bool] = ACTIONS(991),
    [anon_sym_str] = ACTIONS(991),
    [anon_sym_char] = ACTIONS(991),
    [anon_sym_DASH] = ACTIONS(991),
    [anon_sym_SLASH] = ACTIONS(991),
    [anon_sym_PERCENT] = ACTIONS(991),
    [anon_sym_CARET] = ACTIONS(991),
    [anon_sym_BANG] = ACTIONS(991),
    [anon_sym_AMP] = ACTIONS(991),
    [anon_sym_PIPE] = ACTIONS(991),
    [anon_sym_AMP_AMP] = ACTIONS(993),
    [anon_sym_PIPE_PIPE] = ACTIONS(993),
    [anon_sym_LT_LT] = ACTIONS(991),
    [anon_sym_GT_GT] = ACTIONS(991),
    [anon_sym_PLUS_EQ] = ACTIONS(993),
    [anon_sym_DASH_EQ] = ACTIONS(993),
    [anon_sym_STAR_EQ] = ACTIONS(993),
    [anon_sym_SLASH_EQ] = ACTIONS(993),
    [anon_sym_PERCENT_EQ] = ACTIONS(993),
    [anon_sym_CARET_EQ] = ACTIONS(993),
    [anon_sym_AMP_EQ] = ACTIONS(993),
    [anon_sym_PIPE_EQ] = ACTIONS(993),
    [anon_sym_LT_LT_EQ] = ACTIONS(993),
    [anon_sym_GT_GT_EQ] = ACTIONS(993),
    [anon_sym_EQ] = ACTIONS(991),
    [anon_sym_EQ_EQ] = ACTIONS(993),
    [anon_sym_BANG_EQ] = ACTIONS(993),
    [anon_sym_GT] = ACTIONS(991),
    [anon_sym_LT] = ACTIONS(991),
    [anon_sym_GT_EQ] = ACTIONS(993),
    [anon_sym_LT_EQ] = ACTIONS(993),
    [anon_sym_AT] = ACTIONS(993),
    [anon_sym__] = ACTIONS(991),
    [anon_sym_DOT] = ACTIONS(991),
    [anon_sym_DOT_DOT] = ACTIONS(991),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(993),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(993),
    [anon_sym_COMMA] = ACTIONS(993),
    [anon_sym_COLON_COLON] = ACTIONS(993),
    [anon_sym_DASH_GT] = ACTIONS(993),
    [anon_sym_POUND] = ACTIONS(993),
    [anon_sym_SQUOTE] = ACTIONS(991),
    [anon_sym_as] = ACTIONS(991),
    [anon_sym_async] = ACTIONS(991),
    [anon_sym_await] = ACTIONS(991),
    [anon_sym_break] = ACTIONS(991),
    [anon_sym_const] = ACTIONS(991),
    [anon_sym_continue] = ACTIONS(991),
    [anon_sym_default] = ACTIONS(991),
    [anon_sym_enum] = ACTIONS(991),
    [anon_sym_fn] = ACTIONS(991),
    [anon_sym_for] = ACTIONS(991),
    [anon_sym_gen] = ACTIONS(991),
    [anon_sym_if] = ACTIONS(991),
    [anon_sym_impl] = ACTIONS(991),
    [anon_sym_let] = ACTIONS(991),
    [anon_sym_loop] = ACTIONS(991),
    [anon_sym_match] = ACTIONS(991),
    [anon_sym_mod] = ACTIONS(991),
    [anon_sym_pub] = ACTIONS(991),
    [anon_sym_return] = ACTIONS(991),
    [anon_sym_static] = ACTIONS(991),
    [anon_sym_struct] = ACTIONS(991),
    [anon_sym_trait] = ACTIONS(991),
    [anon_sym_type] = ACTIONS(991),
    [anon_sym_union] = ACTIONS(991),
    [anon_sym_unsafe] = ACTIONS(991),
    [anon_sym_use] = ACTIONS(991),
    [anon_sym_where] = ACTIONS(991),
    [anon_sym_while] = ACTIONS(991),
    [sym_mutable_specifier] = ACTIONS(991),
    [sym_integer_literal] = ACTIONS(993),
    [aux_sym_string_literal_token1] = ACTIONS(993),
    [sym_char_literal] = ACTIONS(993),
    [anon_sym_true] = ACTIONS(991),
    [anon_sym_false] = ACTIONS(991),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(991),
    [sym_super] = ACTIONS(991),
    [sym_crate] = ACTIONS(991),
    [sym_metavariable] = ACTIONS(993),
    [sym__raw_string_literal_start] = ACTIONS(993),
    [sym_float_literal] = ACTIONS(993),
  },
  [177] = {
    [sym_line_comment] = STATE(177),
    [sym_block_comment] = STATE(177),
    [sym_identifier] = ACTIONS(995),
    [anon_sym_SEMI] = ACTIONS(997),
    [anon_sym_LPAREN] = ACTIONS(997),
    [anon_sym_RPAREN] = ACTIONS(997),
    [anon_sym_LBRACK] = ACTIONS(997),
    [anon_sym_RBRACK] = ACTIONS(997),
    [anon_sym_LBRACE] = ACTIONS(997),
    [anon_sym_RBRACE] = ACTIONS(997),
    [anon_sym_EQ_GT] = ACTIONS(997),
    [anon_sym_COLON] = ACTIONS(995),
    [anon_sym_DOLLAR] = ACTIONS(995),
    [anon_sym_PLUS] = ACTIONS(995),
    [anon_sym_STAR] = ACTIONS(995),
    [anon_sym_QMARK] = ACTIONS(997),
    [anon_sym_u8] = ACTIONS(995),
    [anon_sym_i8] = ACTIONS(995),
    [anon_sym_u16] = ACTIONS(995),
    [anon_sym_i16] = ACTIONS(995),
    [anon_sym_u32] = ACTIONS(995),
    [anon_sym_i32] = ACTIONS(995),
    [anon_sym_u64] = ACTIONS(995),
    [anon_sym_i64] = ACTIONS(995),
    [anon_sym_u128] = ACTIONS(995),
    [anon_sym_i128] = ACTIONS(995),
    [anon_sym_isize] = ACTIONS(995),
    [anon_sym_usize] = ACTIONS(995),
    [anon_sym_f32] = ACTIONS(995),
    [anon_sym_f64] = ACTIONS(995),
    [anon_sym_bool] = ACTIONS(995),
    [anon_sym_str] = ACTIONS(995),
    [anon_sym_char] = ACTIONS(995),
    [anon_sym_DASH] = ACTIONS(995),
    [anon_sym_SLASH] = ACTIONS(995),
    [anon_sym_PERCENT] = ACTIONS(995),
    [anon_sym_CARET] = ACTIONS(995),
    [anon_sym_BANG] = ACTIONS(995),
    [anon_sym_AMP] = ACTIONS(995),
    [anon_sym_PIPE] = ACTIONS(995),
    [anon_sym_AMP_AMP] = ACTIONS(997),
    [anon_sym_PIPE_PIPE] = ACTIONS(997),
    [anon_sym_LT_LT] = ACTIONS(995),
    [anon_sym_GT_GT] = ACTIONS(995),
    [anon_sym_PLUS_EQ] = ACTIONS(997),
    [anon_sym_DASH_EQ] = ACTIONS(997),
    [anon_sym_STAR_EQ] = ACTIONS(997),
    [anon_sym_SLASH_EQ] = ACTIONS(997),
    [anon_sym_PERCENT_EQ] = ACTIONS(997),
    [anon_sym_CARET_EQ] = ACTIONS(997),
    [anon_sym_AMP_EQ] = ACTIONS(997),
    [anon_sym_PIPE_EQ] = ACTIONS(997),
    [anon_sym_LT_LT_EQ] = ACTIONS(997),
    [anon_sym_GT_GT_EQ] = ACTIONS(997),
    [anon_sym_EQ] = ACTIONS(995),
    [anon_sym_EQ_EQ] = ACTIONS(997),
    [anon_sym_BANG_EQ] = ACTIONS(997),
    [anon_sym_GT] = ACTIONS(995),
    [anon_sym_LT] = ACTIONS(995),
    [anon_sym_GT_EQ] = ACTIONS(997),
    [anon_sym_LT_EQ] = ACTIONS(997),
    [anon_sym_AT] = ACTIONS(997),
    [anon_sym__] = ACTIONS(995),
    [anon_sym_DOT] = ACTIONS(995),
    [anon_sym_DOT_DOT] = ACTIONS(995),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(997),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(997),
    [anon_sym_COMMA] = ACTIONS(997),
    [anon_sym_COLON_COLON] = ACTIONS(997),
    [anon_sym_DASH_GT] = ACTIONS(997),
    [anon_sym_POUND] = ACTIONS(997),
    [anon_sym_SQUOTE] = ACTIONS(995),
    [anon_sym_as] = ACTIONS(995),
    [anon_sym_async] = ACTIONS(995),
    [anon_sym_await] = ACTIONS(995),
    [anon_sym_break] = ACTIONS(995),
    [anon_sym_const] = ACTIONS(995),
    [anon_sym_continue] = ACTIONS(995),
    [anon_sym_default] = ACTIONS(995),
    [anon_sym_enum] = ACTIONS(995),
    [anon_sym_fn] = ACTIONS(995),
    [anon_sym_for] = ACTIONS(995),
    [anon_sym_gen] = ACTIONS(995),
    [anon_sym_if] = ACTIONS(995),
    [anon_sym_impl] = ACTIONS(995),
    [anon_sym_let] = ACTIONS(995),
    [anon_sym_loop] = ACTIONS(995),
    [anon_sym_match] = ACTIONS(995),
    [anon_sym_mod] = ACTIONS(995),
    [anon_sym_pub] = ACTIONS(995),
    [anon_sym_return] = ACTIONS(995),
    [anon_sym_static] = ACTIONS(995),
    [anon_sym_struct] = ACTIONS(995),
    [anon_sym_trait] = ACTIONS(995),
    [anon_sym_type] = ACTIONS(995),
    [anon_sym_union] = ACTIONS(995),
    [anon_sym_unsafe] = ACTIONS(995),
    [anon_sym_use] = ACTIONS(995),
    [anon_sym_where] = ACTIONS(995),
    [anon_sym_while] = ACTIONS(995),
    [sym_mutable_specifier] = ACTIONS(995),
    [sym_integer_literal] = ACTIONS(997),
    [aux_sym_string_literal_token1] = ACTIONS(997),
    [sym_char_literal] = ACTIONS(997),
    [anon_sym_true] = ACTIONS(995),
    [anon_sym_false] = ACTIONS(995),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(995),
    [sym_super] = ACTIONS(995),
    [sym_crate] = ACTIONS(995),
    [sym_metavariable] = ACTIONS(997),
    [sym__raw_string_literal_start] = ACTIONS(997),
    [sym_float_literal] = ACTIONS(997),
  },
  [178] = {
    [sym_attribute_item] = STATE(1011),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1623),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(178),
    [sym_block_comment] = STATE(178),
    [aux_sym_enum_variant_list_repeat1] = STATE(207),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_RBRACK] = ACTIONS(999),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(748),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [179] = {
    [sym_line_comment] = STATE(179),
    [sym_block_comment] = STATE(179),
    [aux_sym__non_special_token_repeat1] = STATE(179),
    [sym_identifier] = ACTIONS(758),
    [anon_sym_SEMI] = ACTIONS(1001),
    [anon_sym_LPAREN] = ACTIONS(763),
    [anon_sym_RPAREN] = ACTIONS(763),
    [anon_sym_LBRACK] = ACTIONS(763),
    [anon_sym_RBRACK] = ACTIONS(763),
    [anon_sym_LBRACE] = ACTIONS(763),
    [anon_sym_RBRACE] = ACTIONS(763),
    [anon_sym_EQ_GT] = ACTIONS(1001),
    [anon_sym_COLON] = ACTIONS(1004),
    [anon_sym_DOLLAR] = ACTIONS(763),
    [anon_sym_PLUS] = ACTIONS(1004),
    [anon_sym_STAR] = ACTIONS(1004),
    [anon_sym_QMARK] = ACTIONS(1001),
    [anon_sym_u8] = ACTIONS(758),
    [anon_sym_i8] = ACTIONS(758),
    [anon_sym_u16] = ACTIONS(758),
    [anon_sym_i16] = ACTIONS(758),
    [anon_sym_u32] = ACTIONS(758),
    [anon_sym_i32] = ACTIONS(758),
    [anon_sym_u64] = ACTIONS(758),
    [anon_sym_i64] = ACTIONS(758),
    [anon_sym_u128] = ACTIONS(758),
    [anon_sym_i128] = ACTIONS(758),
    [anon_sym_isize] = ACTIONS(758),
    [anon_sym_usize] = ACTIONS(758),
    [anon_sym_f32] = ACTIONS(758),
    [anon_sym_f64] = ACTIONS(758),
    [anon_sym_bool] = ACTIONS(758),
    [anon_sym_str] = ACTIONS(758),
    [anon_sym_char] = ACTIONS(758),
    [anon_sym_DASH] = ACTIONS(1004),
    [anon_sym_SLASH] = ACTIONS(1004),
    [anon_sym_PERCENT] = ACTIONS(1004),
    [anon_sym_CARET] = ACTIONS(1004),
    [anon_sym_BANG] = ACTIONS(1004),
    [anon_sym_AMP] = ACTIONS(1004),
    [anon_sym_PIPE] = ACTIONS(1004),
    [anon_sym_AMP_AMP] = ACTIONS(1001),
    [anon_sym_PIPE_PIPE] = ACTIONS(1001),
    [anon_sym_LT_LT] = ACTIONS(1004),
    [anon_sym_GT_GT] = ACTIONS(1004),
    [anon_sym_PLUS_EQ] = ACTIONS(1001),
    [anon_sym_DASH_EQ] = ACTIONS(1001),
    [anon_sym_STAR_EQ] = ACTIONS(1001),
    [anon_sym_SLASH_EQ] = ACTIONS(1001),
    [anon_sym_PERCENT_EQ] = ACTIONS(1001),
    [anon_sym_CARET_EQ] = ACTIONS(1001),
    [anon_sym_AMP_EQ] = ACTIONS(1001),
    [anon_sym_PIPE_EQ] = ACTIONS(1001),
    [anon_sym_LT_LT_EQ] = ACTIONS(1001),
    [anon_sym_GT_GT_EQ] = ACTIONS(1001),
    [anon_sym_EQ] = ACTIONS(1004),
    [anon_sym_EQ_EQ] = ACTIONS(1001),
    [anon_sym_BANG_EQ] = ACTIONS(1001),
    [anon_sym_GT] = ACTIONS(1004),
    [anon_sym_LT] = ACTIONS(1004),
    [anon_sym_GT_EQ] = ACTIONS(1001),
    [anon_sym_LT_EQ] = ACTIONS(1001),
    [anon_sym_AT] = ACTIONS(1001),
    [anon_sym__] = ACTIONS(1004),
    [anon_sym_DOT] = ACTIONS(1004),
    [anon_sym_DOT_DOT] = ACTIONS(1004),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1001),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1001),
    [anon_sym_COMMA] = ACTIONS(1001),
    [anon_sym_COLON_COLON] = ACTIONS(1001),
    [anon_sym_DASH_GT] = ACTIONS(1001),
    [anon_sym_POUND] = ACTIONS(1001),
    [anon_sym_SQUOTE] = ACTIONS(758),
    [anon_sym_as] = ACTIONS(758),
    [anon_sym_async] = ACTIONS(758),
    [anon_sym_await] = ACTIONS(758),
    [anon_sym_break] = ACTIONS(758),
    [anon_sym_const] = ACTIONS(758),
    [anon_sym_continue] = ACTIONS(758),
    [anon_sym_default] = ACTIONS(758),
    [anon_sym_enum] = ACTIONS(758),
    [anon_sym_fn] = ACTIONS(758),
    [anon_sym_for] = ACTIONS(758),
    [anon_sym_gen] = ACTIONS(758),
    [anon_sym_if] = ACTIONS(758),
    [anon_sym_impl] = ACTIONS(758),
    [anon_sym_let] = ACTIONS(758),
    [anon_sym_loop] = ACTIONS(758),
    [anon_sym_match] = ACTIONS(758),
    [anon_sym_mod] = ACTIONS(758),
    [anon_sym_pub] = ACTIONS(758),
    [anon_sym_return] = ACTIONS(758),
    [anon_sym_static] = ACTIONS(758),
    [anon_sym_struct] = ACTIONS(758),
    [anon_sym_trait] = ACTIONS(758),
    [anon_sym_type] = ACTIONS(758),
    [anon_sym_union] = ACTIONS(758),
    [anon_sym_unsafe] = ACTIONS(758),
    [anon_sym_use] = ACTIONS(758),
    [anon_sym_where] = ACTIONS(758),
    [anon_sym_while] = ACTIONS(758),
    [sym_mutable_specifier] = ACTIONS(758),
    [sym_integer_literal] = ACTIONS(763),
    [aux_sym_string_literal_token1] = ACTIONS(763),
    [sym_char_literal] = ACTIONS(763),
    [anon_sym_true] = ACTIONS(758),
    [anon_sym_false] = ACTIONS(758),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(758),
    [sym_super] = ACTIONS(758),
    [sym_crate] = ACTIONS(758),
    [sym__raw_string_literal_start] = ACTIONS(763),
    [sym_float_literal] = ACTIONS(763),
  },
  [180] = {
    [sym_line_comment] = STATE(180),
    [sym_block_comment] = STATE(180),
    [sym_identifier] = ACTIONS(1007),
    [anon_sym_SEMI] = ACTIONS(1009),
    [anon_sym_LPAREN] = ACTIONS(1009),
    [anon_sym_RPAREN] = ACTIONS(1009),
    [anon_sym_LBRACK] = ACTIONS(1009),
    [anon_sym_RBRACK] = ACTIONS(1009),
    [anon_sym_LBRACE] = ACTIONS(1009),
    [anon_sym_RBRACE] = ACTIONS(1009),
    [anon_sym_EQ_GT] = ACTIONS(1009),
    [anon_sym_COLON] = ACTIONS(1007),
    [anon_sym_DOLLAR] = ACTIONS(1007),
    [anon_sym_PLUS] = ACTIONS(1007),
    [anon_sym_STAR] = ACTIONS(1007),
    [anon_sym_QMARK] = ACTIONS(1009),
    [anon_sym_u8] = ACTIONS(1007),
    [anon_sym_i8] = ACTIONS(1007),
    [anon_sym_u16] = ACTIONS(1007),
    [anon_sym_i16] = ACTIONS(1007),
    [anon_sym_u32] = ACTIONS(1007),
    [anon_sym_i32] = ACTIONS(1007),
    [anon_sym_u64] = ACTIONS(1007),
    [anon_sym_i64] = ACTIONS(1007),
    [anon_sym_u128] = ACTIONS(1007),
    [anon_sym_i128] = ACTIONS(1007),
    [anon_sym_isize] = ACTIONS(1007),
    [anon_sym_usize] = ACTIONS(1007),
    [anon_sym_f32] = ACTIONS(1007),
    [anon_sym_f64] = ACTIONS(1007),
    [anon_sym_bool] = ACTIONS(1007),
    [anon_sym_str] = ACTIONS(1007),
    [anon_sym_char] = ACTIONS(1007),
    [anon_sym_DASH] = ACTIONS(1007),
    [anon_sym_SLASH] = ACTIONS(1007),
    [anon_sym_PERCENT] = ACTIONS(1007),
    [anon_sym_CARET] = ACTIONS(1007),
    [anon_sym_BANG] = ACTIONS(1007),
    [anon_sym_AMP] = ACTIONS(1007),
    [anon_sym_PIPE] = ACTIONS(1007),
    [anon_sym_AMP_AMP] = ACTIONS(1009),
    [anon_sym_PIPE_PIPE] = ACTIONS(1009),
    [anon_sym_LT_LT] = ACTIONS(1007),
    [anon_sym_GT_GT] = ACTIONS(1007),
    [anon_sym_PLUS_EQ] = ACTIONS(1009),
    [anon_sym_DASH_EQ] = ACTIONS(1009),
    [anon_sym_STAR_EQ] = ACTIONS(1009),
    [anon_sym_SLASH_EQ] = ACTIONS(1009),
    [anon_sym_PERCENT_EQ] = ACTIONS(1009),
    [anon_sym_CARET_EQ] = ACTIONS(1009),
    [anon_sym_AMP_EQ] = ACTIONS(1009),
    [anon_sym_PIPE_EQ] = ACTIONS(1009),
    [anon_sym_LT_LT_EQ] = ACTIONS(1009),
    [anon_sym_GT_GT_EQ] = ACTIONS(1009),
    [anon_sym_EQ] = ACTIONS(1007),
    [anon_sym_EQ_EQ] = ACTIONS(1009),
    [anon_sym_BANG_EQ] = ACTIONS(1009),
    [anon_sym_GT] = ACTIONS(1007),
    [anon_sym_LT] = ACTIONS(1007),
    [anon_sym_GT_EQ] = ACTIONS(1009),
    [anon_sym_LT_EQ] = ACTIONS(1009),
    [anon_sym_AT] = ACTIONS(1009),
    [anon_sym__] = ACTIONS(1007),
    [anon_sym_DOT] = ACTIONS(1007),
    [anon_sym_DOT_DOT] = ACTIONS(1007),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1009),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1009),
    [anon_sym_COMMA] = ACTIONS(1009),
    [anon_sym_COLON_COLON] = ACTIONS(1009),
    [anon_sym_DASH_GT] = ACTIONS(1009),
    [anon_sym_POUND] = ACTIONS(1009),
    [anon_sym_SQUOTE] = ACTIONS(1007),
    [anon_sym_as] = ACTIONS(1007),
    [anon_sym_async] = ACTIONS(1007),
    [anon_sym_await] = ACTIONS(1007),
    [anon_sym_break] = ACTIONS(1007),
    [anon_sym_const] = ACTIONS(1007),
    [anon_sym_continue] = ACTIONS(1007),
    [anon_sym_default] = ACTIONS(1007),
    [anon_sym_enum] = ACTIONS(1007),
    [anon_sym_fn] = ACTIONS(1007),
    [anon_sym_for] = ACTIONS(1007),
    [anon_sym_gen] = ACTIONS(1007),
    [anon_sym_if] = ACTIONS(1007),
    [anon_sym_impl] = ACTIONS(1007),
    [anon_sym_let] = ACTIONS(1007),
    [anon_sym_loop] = ACTIONS(1007),
    [anon_sym_match] = ACTIONS(1007),
    [anon_sym_mod] = ACTIONS(1007),
    [anon_sym_pub] = ACTIONS(1007),
    [anon_sym_return] = ACTIONS(1007),
    [anon_sym_static] = ACTIONS(1007),
    [anon_sym_struct] = ACTIONS(1007),
    [anon_sym_trait] = ACTIONS(1007),
    [anon_sym_type] = ACTIONS(1007),
    [anon_sym_union] = ACTIONS(1007),
    [anon_sym_unsafe] = ACTIONS(1007),
    [anon_sym_use] = ACTIONS(1007),
    [anon_sym_where] = ACTIONS(1007),
    [anon_sym_while] = ACTIONS(1007),
    [sym_mutable_specifier] = ACTIONS(1007),
    [sym_integer_literal] = ACTIONS(1009),
    [aux_sym_string_literal_token1] = ACTIONS(1009),
    [sym_char_literal] = ACTIONS(1009),
    [anon_sym_true] = ACTIONS(1007),
    [anon_sym_false] = ACTIONS(1007),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1007),
    [sym_super] = ACTIONS(1007),
    [sym_crate] = ACTIONS(1007),
    [sym_metavariable] = ACTIONS(1009),
    [sym__raw_string_literal_start] = ACTIONS(1009),
    [sym_float_literal] = ACTIONS(1009),
  },
  [181] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1687),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_let_condition] = STATE(2883),
    [sym__let_chain] = STATE(2887),
    [sym__condition] = STATE(2732),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(181),
    [sym_block_comment] = STATE(181),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(959),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(963),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_let] = ACTIONS(965),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [182] = {
    [sym_line_comment] = STATE(182),
    [sym_block_comment] = STATE(182),
    [sym_identifier] = ACTIONS(1011),
    [anon_sym_SEMI] = ACTIONS(1013),
    [anon_sym_LPAREN] = ACTIONS(1013),
    [anon_sym_RPAREN] = ACTIONS(1013),
    [anon_sym_LBRACK] = ACTIONS(1013),
    [anon_sym_RBRACK] = ACTIONS(1013),
    [anon_sym_LBRACE] = ACTIONS(1013),
    [anon_sym_RBRACE] = ACTIONS(1013),
    [anon_sym_EQ_GT] = ACTIONS(1013),
    [anon_sym_COLON] = ACTIONS(1011),
    [anon_sym_DOLLAR] = ACTIONS(1011),
    [anon_sym_PLUS] = ACTIONS(1011),
    [anon_sym_STAR] = ACTIONS(1011),
    [anon_sym_QMARK] = ACTIONS(1013),
    [anon_sym_u8] = ACTIONS(1011),
    [anon_sym_i8] = ACTIONS(1011),
    [anon_sym_u16] = ACTIONS(1011),
    [anon_sym_i16] = ACTIONS(1011),
    [anon_sym_u32] = ACTIONS(1011),
    [anon_sym_i32] = ACTIONS(1011),
    [anon_sym_u64] = ACTIONS(1011),
    [anon_sym_i64] = ACTIONS(1011),
    [anon_sym_u128] = ACTIONS(1011),
    [anon_sym_i128] = ACTIONS(1011),
    [anon_sym_isize] = ACTIONS(1011),
    [anon_sym_usize] = ACTIONS(1011),
    [anon_sym_f32] = ACTIONS(1011),
    [anon_sym_f64] = ACTIONS(1011),
    [anon_sym_bool] = ACTIONS(1011),
    [anon_sym_str] = ACTIONS(1011),
    [anon_sym_char] = ACTIONS(1011),
    [anon_sym_DASH] = ACTIONS(1011),
    [anon_sym_SLASH] = ACTIONS(1011),
    [anon_sym_PERCENT] = ACTIONS(1011),
    [anon_sym_CARET] = ACTIONS(1011),
    [anon_sym_BANG] = ACTIONS(1011),
    [anon_sym_AMP] = ACTIONS(1011),
    [anon_sym_PIPE] = ACTIONS(1011),
    [anon_sym_AMP_AMP] = ACTIONS(1013),
    [anon_sym_PIPE_PIPE] = ACTIONS(1013),
    [anon_sym_LT_LT] = ACTIONS(1011),
    [anon_sym_GT_GT] = ACTIONS(1011),
    [anon_sym_PLUS_EQ] = ACTIONS(1013),
    [anon_sym_DASH_EQ] = ACTIONS(1013),
    [anon_sym_STAR_EQ] = ACTIONS(1013),
    [anon_sym_SLASH_EQ] = ACTIONS(1013),
    [anon_sym_PERCENT_EQ] = ACTIONS(1013),
    [anon_sym_CARET_EQ] = ACTIONS(1013),
    [anon_sym_AMP_EQ] = ACTIONS(1013),
    [anon_sym_PIPE_EQ] = ACTIONS(1013),
    [anon_sym_LT_LT_EQ] = ACTIONS(1013),
    [anon_sym_GT_GT_EQ] = ACTIONS(1013),
    [anon_sym_EQ] = ACTIONS(1011),
    [anon_sym_EQ_EQ] = ACTIONS(1013),
    [anon_sym_BANG_EQ] = ACTIONS(1013),
    [anon_sym_GT] = ACTIONS(1011),
    [anon_sym_LT] = ACTIONS(1011),
    [anon_sym_GT_EQ] = ACTIONS(1013),
    [anon_sym_LT_EQ] = ACTIONS(1013),
    [anon_sym_AT] = ACTIONS(1013),
    [anon_sym__] = ACTIONS(1011),
    [anon_sym_DOT] = ACTIONS(1011),
    [anon_sym_DOT_DOT] = ACTIONS(1011),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1013),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1013),
    [anon_sym_COMMA] = ACTIONS(1013),
    [anon_sym_COLON_COLON] = ACTIONS(1013),
    [anon_sym_DASH_GT] = ACTIONS(1013),
    [anon_sym_POUND] = ACTIONS(1013),
    [anon_sym_SQUOTE] = ACTIONS(1011),
    [anon_sym_as] = ACTIONS(1011),
    [anon_sym_async] = ACTIONS(1011),
    [anon_sym_await] = ACTIONS(1011),
    [anon_sym_break] = ACTIONS(1011),
    [anon_sym_const] = ACTIONS(1011),
    [anon_sym_continue] = ACTIONS(1011),
    [anon_sym_default] = ACTIONS(1011),
    [anon_sym_enum] = ACTIONS(1011),
    [anon_sym_fn] = ACTIONS(1011),
    [anon_sym_for] = ACTIONS(1011),
    [anon_sym_gen] = ACTIONS(1011),
    [anon_sym_if] = ACTIONS(1011),
    [anon_sym_impl] = ACTIONS(1011),
    [anon_sym_let] = ACTIONS(1011),
    [anon_sym_loop] = ACTIONS(1011),
    [anon_sym_match] = ACTIONS(1011),
    [anon_sym_mod] = ACTIONS(1011),
    [anon_sym_pub] = ACTIONS(1011),
    [anon_sym_return] = ACTIONS(1011),
    [anon_sym_static] = ACTIONS(1011),
    [anon_sym_struct] = ACTIONS(1011),
    [anon_sym_trait] = ACTIONS(1011),
    [anon_sym_type] = ACTIONS(1011),
    [anon_sym_union] = ACTIONS(1011),
    [anon_sym_unsafe] = ACTIONS(1011),
    [anon_sym_use] = ACTIONS(1011),
    [anon_sym_where] = ACTIONS(1011),
    [anon_sym_while] = ACTIONS(1011),
    [sym_mutable_specifier] = ACTIONS(1011),
    [sym_integer_literal] = ACTIONS(1013),
    [aux_sym_string_literal_token1] = ACTIONS(1013),
    [sym_char_literal] = ACTIONS(1013),
    [anon_sym_true] = ACTIONS(1011),
    [anon_sym_false] = ACTIONS(1011),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1011),
    [sym_super] = ACTIONS(1011),
    [sym_crate] = ACTIONS(1011),
    [sym_metavariable] = ACTIONS(1013),
    [sym__raw_string_literal_start] = ACTIONS(1013),
    [sym_float_literal] = ACTIONS(1013),
  },
  [183] = {
    [sym_bracketed_type] = STATE(3529),
    [sym_generic_function] = STATE(1664),
    [sym_generic_type_with_turbofish] = STATE(2956),
    [sym__expression_except_range] = STATE(1626),
    [sym__expression] = STATE(1868),
    [sym_macro_invocation] = STATE(1671),
    [sym_scoped_identifier] = STATE(1575),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3163),
    [sym_range_expression] = STATE(1666),
    [sym_unary_expression] = STATE(1664),
    [sym_try_expression] = STATE(1664),
    [sym_reference_expression] = STATE(1664),
    [sym_binary_expression] = STATE(1664),
    [sym_assignment_expression] = STATE(1664),
    [sym_compound_assignment_expr] = STATE(1664),
    [sym_type_cast_expression] = STATE(1664),
    [sym_return_expression] = STATE(1664),
    [sym_yield_expression] = STATE(1664),
    [sym_call_expression] = STATE(1664),
    [sym_array_expression] = STATE(1664),
    [sym_parenthesized_expression] = STATE(1664),
    [sym_tuple_expression] = STATE(1664),
    [sym_unit_expression] = STATE(1664),
    [sym_struct_expression] = STATE(1664),
    [sym_if_expression] = STATE(1664),
    [sym_let_condition] = STATE(3226),
    [sym__let_chain] = STATE(3228),
    [sym__condition] = STATE(3536),
    [sym_match_expression] = STATE(1664),
    [sym_while_expression] = STATE(1664),
    [sym_loop_expression] = STATE(1664),
    [sym_for_expression] = STATE(1664),
    [sym_const_block] = STATE(1664),
    [sym_closure_expression] = STATE(1664),
    [sym_closure_parameters] = STATE(232),
    [sym_label] = STATE(3614),
    [sym_break_expression] = STATE(1664),
    [sym_continue_expression] = STATE(1664),
    [sym_index_expression] = STATE(1664),
    [sym_await_expression] = STATE(1664),
    [sym_field_expression] = STATE(1597),
    [sym_unsafe_block] = STATE(1664),
    [sym_async_block] = STATE(1664),
    [sym_gen_block] = STATE(1664),
    [sym_try_block] = STATE(1664),
    [sym_block] = STATE(1664),
    [sym__literal] = STATE(1664),
    [sym_string_literal] = STATE(1786),
    [sym_raw_string_literal] = STATE(1786),
    [sym_boolean_literal] = STATE(1786),
    [sym_line_comment] = STATE(183),
    [sym_block_comment] = STATE(183),
    [sym_identifier] = ACTIONS(405),
    [anon_sym_LPAREN] = ACTIONS(495),
    [anon_sym_LBRACK] = ACTIONS(497),
    [anon_sym_LBRACE] = ACTIONS(407),
    [anon_sym_STAR] = ACTIONS(1015),
    [anon_sym_u8] = ACTIONS(411),
    [anon_sym_i8] = ACTIONS(411),
    [anon_sym_u16] = ACTIONS(411),
    [anon_sym_i16] = ACTIONS(411),
    [anon_sym_u32] = ACTIONS(411),
    [anon_sym_i32] = ACTIONS(411),
    [anon_sym_u64] = ACTIONS(411),
    [anon_sym_i64] = ACTIONS(411),
    [anon_sym_u128] = ACTIONS(411),
    [anon_sym_i128] = ACTIONS(411),
    [anon_sym_isize] = ACTIONS(411),
    [anon_sym_usize] = ACTIONS(411),
    [anon_sym_f32] = ACTIONS(411),
    [anon_sym_f64] = ACTIONS(411),
    [anon_sym_bool] = ACTIONS(411),
    [anon_sym_str] = ACTIONS(411),
    [anon_sym_char] = ACTIONS(411),
    [anon_sym_DASH] = ACTIONS(1015),
    [anon_sym_BANG] = ACTIONS(1015),
    [anon_sym_AMP] = ACTIONS(1017),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1019),
    [anon_sym_COLON_COLON] = ACTIONS(415),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(417),
    [anon_sym_break] = ACTIONS(419),
    [anon_sym_const] = ACTIONS(421),
    [anon_sym_continue] = ACTIONS(423),
    [anon_sym_default] = ACTIONS(425),
    [anon_sym_for] = ACTIONS(427),
    [anon_sym_gen] = ACTIONS(429),
    [anon_sym_if] = ACTIONS(431),
    [anon_sym_let] = ACTIONS(1021),
    [anon_sym_loop] = ACTIONS(433),
    [anon_sym_match] = ACTIONS(435),
    [anon_sym_return] = ACTIONS(437),
    [anon_sym_static] = ACTIONS(439),
    [anon_sym_union] = ACTIONS(425),
    [anon_sym_unsafe] = ACTIONS(441),
    [anon_sym_while] = ACTIONS(443),
    [anon_sym_yield] = ACTIONS(445),
    [anon_sym_move] = ACTIONS(447),
    [anon_sym_try] = ACTIONS(449),
    [sym_integer_literal] = ACTIONS(451),
    [aux_sym_string_literal_token1] = ACTIONS(453),
    [sym_char_literal] = ACTIONS(451),
    [anon_sym_true] = ACTIONS(455),
    [anon_sym_false] = ACTIONS(455),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(457),
    [sym_super] = ACTIONS(459),
    [sym_crate] = ACTIONS(459),
    [sym_metavariable] = ACTIONS(461),
    [sym__raw_string_literal_start] = ACTIONS(463),
    [sym_float_literal] = ACTIONS(451),
  },
  [184] = {
    [sym_line_comment] = STATE(184),
    [sym_block_comment] = STATE(184),
    [sym_identifier] = ACTIONS(1023),
    [anon_sym_SEMI] = ACTIONS(1025),
    [anon_sym_LPAREN] = ACTIONS(1025),
    [anon_sym_RPAREN] = ACTIONS(1025),
    [anon_sym_LBRACK] = ACTIONS(1025),
    [anon_sym_RBRACK] = ACTIONS(1025),
    [anon_sym_LBRACE] = ACTIONS(1025),
    [anon_sym_RBRACE] = ACTIONS(1025),
    [anon_sym_EQ_GT] = ACTIONS(1025),
    [anon_sym_COLON] = ACTIONS(1023),
    [anon_sym_DOLLAR] = ACTIONS(1023),
    [anon_sym_PLUS] = ACTIONS(1023),
    [anon_sym_STAR] = ACTIONS(1023),
    [anon_sym_QMARK] = ACTIONS(1025),
    [anon_sym_u8] = ACTIONS(1023),
    [anon_sym_i8] = ACTIONS(1023),
    [anon_sym_u16] = ACTIONS(1023),
    [anon_sym_i16] = ACTIONS(1023),
    [anon_sym_u32] = ACTIONS(1023),
    [anon_sym_i32] = ACTIONS(1023),
    [anon_sym_u64] = ACTIONS(1023),
    [anon_sym_i64] = ACTIONS(1023),
    [anon_sym_u128] = ACTIONS(1023),
    [anon_sym_i128] = ACTIONS(1023),
    [anon_sym_isize] = ACTIONS(1023),
    [anon_sym_usize] = ACTIONS(1023),
    [anon_sym_f32] = ACTIONS(1023),
    [anon_sym_f64] = ACTIONS(1023),
    [anon_sym_bool] = ACTIONS(1023),
    [anon_sym_str] = ACTIONS(1023),
    [anon_sym_char] = ACTIONS(1023),
    [anon_sym_DASH] = ACTIONS(1023),
    [anon_sym_SLASH] = ACTIONS(1023),
    [anon_sym_PERCENT] = ACTIONS(1023),
    [anon_sym_CARET] = ACTIONS(1023),
    [anon_sym_BANG] = ACTIONS(1023),
    [anon_sym_AMP] = ACTIONS(1023),
    [anon_sym_PIPE] = ACTIONS(1023),
    [anon_sym_AMP_AMP] = ACTIONS(1025),
    [anon_sym_PIPE_PIPE] = ACTIONS(1025),
    [anon_sym_LT_LT] = ACTIONS(1023),
    [anon_sym_GT_GT] = ACTIONS(1023),
    [anon_sym_PLUS_EQ] = ACTIONS(1025),
    [anon_sym_DASH_EQ] = ACTIONS(1025),
    [anon_sym_STAR_EQ] = ACTIONS(1025),
    [anon_sym_SLASH_EQ] = ACTIONS(1025),
    [anon_sym_PERCENT_EQ] = ACTIONS(1025),
    [anon_sym_CARET_EQ] = ACTIONS(1025),
    [anon_sym_AMP_EQ] = ACTIONS(1025),
    [anon_sym_PIPE_EQ] = ACTIONS(1025),
    [anon_sym_LT_LT_EQ] = ACTIONS(1025),
    [anon_sym_GT_GT_EQ] = ACTIONS(1025),
    [anon_sym_EQ] = ACTIONS(1023),
    [anon_sym_EQ_EQ] = ACTIONS(1025),
    [anon_sym_BANG_EQ] = ACTIONS(1025),
    [anon_sym_GT] = ACTIONS(1023),
    [anon_sym_LT] = ACTIONS(1023),
    [anon_sym_GT_EQ] = ACTIONS(1025),
    [anon_sym_LT_EQ] = ACTIONS(1025),
    [anon_sym_AT] = ACTIONS(1025),
    [anon_sym__] = ACTIONS(1023),
    [anon_sym_DOT] = ACTIONS(1023),
    [anon_sym_DOT_DOT] = ACTIONS(1023),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1025),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1025),
    [anon_sym_COMMA] = ACTIONS(1025),
    [anon_sym_COLON_COLON] = ACTIONS(1025),
    [anon_sym_DASH_GT] = ACTIONS(1025),
    [anon_sym_POUND] = ACTIONS(1025),
    [anon_sym_SQUOTE] = ACTIONS(1023),
    [anon_sym_as] = ACTIONS(1023),
    [anon_sym_async] = ACTIONS(1023),
    [anon_sym_await] = ACTIONS(1023),
    [anon_sym_break] = ACTIONS(1023),
    [anon_sym_const] = ACTIONS(1023),
    [anon_sym_continue] = ACTIONS(1023),
    [anon_sym_default] = ACTIONS(1023),
    [anon_sym_enum] = ACTIONS(1023),
    [anon_sym_fn] = ACTIONS(1023),
    [anon_sym_for] = ACTIONS(1023),
    [anon_sym_gen] = ACTIONS(1023),
    [anon_sym_if] = ACTIONS(1023),
    [anon_sym_impl] = ACTIONS(1023),
    [anon_sym_let] = ACTIONS(1023),
    [anon_sym_loop] = ACTIONS(1023),
    [anon_sym_match] = ACTIONS(1023),
    [anon_sym_mod] = ACTIONS(1023),
    [anon_sym_pub] = ACTIONS(1023),
    [anon_sym_return] = ACTIONS(1023),
    [anon_sym_static] = ACTIONS(1023),
    [anon_sym_struct] = ACTIONS(1023),
    [anon_sym_trait] = ACTIONS(1023),
    [anon_sym_type] = ACTIONS(1023),
    [anon_sym_union] = ACTIONS(1023),
    [anon_sym_unsafe] = ACTIONS(1023),
    [anon_sym_use] = ACTIONS(1023),
    [anon_sym_where] = ACTIONS(1023),
    [anon_sym_while] = ACTIONS(1023),
    [sym_mutable_specifier] = ACTIONS(1023),
    [sym_integer_literal] = ACTIONS(1025),
    [aux_sym_string_literal_token1] = ACTIONS(1025),
    [sym_char_literal] = ACTIONS(1025),
    [anon_sym_true] = ACTIONS(1023),
    [anon_sym_false] = ACTIONS(1023),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1023),
    [sym_super] = ACTIONS(1023),
    [sym_crate] = ACTIONS(1023),
    [sym_metavariable] = ACTIONS(1025),
    [sym__raw_string_literal_start] = ACTIONS(1025),
    [sym_float_literal] = ACTIONS(1025),
  },
  [185] = {
    [sym_line_comment] = STATE(185),
    [sym_block_comment] = STATE(185),
    [sym_identifier] = ACTIONS(1027),
    [anon_sym_SEMI] = ACTIONS(1029),
    [anon_sym_LPAREN] = ACTIONS(1029),
    [anon_sym_RPAREN] = ACTIONS(1029),
    [anon_sym_LBRACK] = ACTIONS(1029),
    [anon_sym_RBRACK] = ACTIONS(1029),
    [anon_sym_LBRACE] = ACTIONS(1029),
    [anon_sym_RBRACE] = ACTIONS(1029),
    [anon_sym_EQ_GT] = ACTIONS(1029),
    [anon_sym_COLON] = ACTIONS(1027),
    [anon_sym_DOLLAR] = ACTIONS(1027),
    [anon_sym_PLUS] = ACTIONS(1027),
    [anon_sym_STAR] = ACTIONS(1027),
    [anon_sym_QMARK] = ACTIONS(1029),
    [anon_sym_u8] = ACTIONS(1027),
    [anon_sym_i8] = ACTIONS(1027),
    [anon_sym_u16] = ACTIONS(1027),
    [anon_sym_i16] = ACTIONS(1027),
    [anon_sym_u32] = ACTIONS(1027),
    [anon_sym_i32] = ACTIONS(1027),
    [anon_sym_u64] = ACTIONS(1027),
    [anon_sym_i64] = ACTIONS(1027),
    [anon_sym_u128] = ACTIONS(1027),
    [anon_sym_i128] = ACTIONS(1027),
    [anon_sym_isize] = ACTIONS(1027),
    [anon_sym_usize] = ACTIONS(1027),
    [anon_sym_f32] = ACTIONS(1027),
    [anon_sym_f64] = ACTIONS(1027),
    [anon_sym_bool] = ACTIONS(1027),
    [anon_sym_str] = ACTIONS(1027),
    [anon_sym_char] = ACTIONS(1027),
    [anon_sym_DASH] = ACTIONS(1027),
    [anon_sym_SLASH] = ACTIONS(1027),
    [anon_sym_PERCENT] = ACTIONS(1027),
    [anon_sym_CARET] = ACTIONS(1027),
    [anon_sym_BANG] = ACTIONS(1027),
    [anon_sym_AMP] = ACTIONS(1027),
    [anon_sym_PIPE] = ACTIONS(1027),
    [anon_sym_AMP_AMP] = ACTIONS(1029),
    [anon_sym_PIPE_PIPE] = ACTIONS(1029),
    [anon_sym_LT_LT] = ACTIONS(1027),
    [anon_sym_GT_GT] = ACTIONS(1027),
    [anon_sym_PLUS_EQ] = ACTIONS(1029),
    [anon_sym_DASH_EQ] = ACTIONS(1029),
    [anon_sym_STAR_EQ] = ACTIONS(1029),
    [anon_sym_SLASH_EQ] = ACTIONS(1029),
    [anon_sym_PERCENT_EQ] = ACTIONS(1029),
    [anon_sym_CARET_EQ] = ACTIONS(1029),
    [anon_sym_AMP_EQ] = ACTIONS(1029),
    [anon_sym_PIPE_EQ] = ACTIONS(1029),
    [anon_sym_LT_LT_EQ] = ACTIONS(1029),
    [anon_sym_GT_GT_EQ] = ACTIONS(1029),
    [anon_sym_EQ] = ACTIONS(1027),
    [anon_sym_EQ_EQ] = ACTIONS(1029),
    [anon_sym_BANG_EQ] = ACTIONS(1029),
    [anon_sym_GT] = ACTIONS(1027),
    [anon_sym_LT] = ACTIONS(1027),
    [anon_sym_GT_EQ] = ACTIONS(1029),
    [anon_sym_LT_EQ] = ACTIONS(1029),
    [anon_sym_AT] = ACTIONS(1029),
    [anon_sym__] = ACTIONS(1027),
    [anon_sym_DOT] = ACTIONS(1027),
    [anon_sym_DOT_DOT] = ACTIONS(1027),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1029),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1029),
    [anon_sym_COMMA] = ACTIONS(1029),
    [anon_sym_COLON_COLON] = ACTIONS(1029),
    [anon_sym_DASH_GT] = ACTIONS(1029),
    [anon_sym_POUND] = ACTIONS(1029),
    [anon_sym_SQUOTE] = ACTIONS(1027),
    [anon_sym_as] = ACTIONS(1027),
    [anon_sym_async] = ACTIONS(1027),
    [anon_sym_await] = ACTIONS(1027),
    [anon_sym_break] = ACTIONS(1027),
    [anon_sym_const] = ACTIONS(1027),
    [anon_sym_continue] = ACTIONS(1027),
    [anon_sym_default] = ACTIONS(1027),
    [anon_sym_enum] = ACTIONS(1027),
    [anon_sym_fn] = ACTIONS(1027),
    [anon_sym_for] = ACTIONS(1027),
    [anon_sym_gen] = ACTIONS(1027),
    [anon_sym_if] = ACTIONS(1027),
    [anon_sym_impl] = ACTIONS(1027),
    [anon_sym_let] = ACTIONS(1027),
    [anon_sym_loop] = ACTIONS(1027),
    [anon_sym_match] = ACTIONS(1027),
    [anon_sym_mod] = ACTIONS(1027),
    [anon_sym_pub] = ACTIONS(1027),
    [anon_sym_return] = ACTIONS(1027),
    [anon_sym_static] = ACTIONS(1027),
    [anon_sym_struct] = ACTIONS(1027),
    [anon_sym_trait] = ACTIONS(1027),
    [anon_sym_type] = ACTIONS(1027),
    [anon_sym_union] = ACTIONS(1027),
    [anon_sym_unsafe] = ACTIONS(1027),
    [anon_sym_use] = ACTIONS(1027),
    [anon_sym_where] = ACTIONS(1027),
    [anon_sym_while] = ACTIONS(1027),
    [sym_mutable_specifier] = ACTIONS(1027),
    [sym_integer_literal] = ACTIONS(1029),
    [aux_sym_string_literal_token1] = ACTIONS(1029),
    [sym_char_literal] = ACTIONS(1029),
    [anon_sym_true] = ACTIONS(1027),
    [anon_sym_false] = ACTIONS(1027),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1027),
    [sym_super] = ACTIONS(1027),
    [sym_crate] = ACTIONS(1027),
    [sym_metavariable] = ACTIONS(1029),
    [sym__raw_string_literal_start] = ACTIONS(1029),
    [sym_float_literal] = ACTIONS(1029),
  },
  [186] = {
    [sym_attribute_item] = STATE(1011),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1623),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(186),
    [sym_block_comment] = STATE(186),
    [aux_sym_enum_variant_list_repeat1] = STATE(207),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(1031),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(748),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [187] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1687),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_let_condition] = STATE(2883),
    [sym__let_chain] = STATE(2887),
    [sym__condition] = STATE(2587),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(187),
    [sym_block_comment] = STATE(187),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(959),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(963),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_let] = ACTIONS(965),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [188] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1687),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_let_condition] = STATE(2883),
    [sym__let_chain] = STATE(2887),
    [sym__condition] = STATE(2681),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(188),
    [sym_block_comment] = STATE(188),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(959),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(963),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_let] = ACTIONS(965),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [189] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1687),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_let_condition] = STATE(2883),
    [sym__let_chain] = STATE(2887),
    [sym__condition] = STATE(2695),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(189),
    [sym_block_comment] = STATE(189),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(959),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(963),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_let] = ACTIONS(965),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [190] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1687),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_let_condition] = STATE(2883),
    [sym__let_chain] = STATE(2887),
    [sym__condition] = STATE(2582),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(190),
    [sym_block_comment] = STATE(190),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(959),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(963),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_let] = ACTIONS(965),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [191] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1687),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_let_condition] = STATE(2883),
    [sym__let_chain] = STATE(2887),
    [sym__condition] = STATE(2560),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(191),
    [sym_block_comment] = STATE(191),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(959),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(963),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_let] = ACTIONS(965),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [192] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1687),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_let_condition] = STATE(2883),
    [sym__let_chain] = STATE(2887),
    [sym__condition] = STATE(2562),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(192),
    [sym_block_comment] = STATE(192),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(959),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(963),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_let] = ACTIONS(965),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [193] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1687),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_let_condition] = STATE(2883),
    [sym__let_chain] = STATE(2887),
    [sym__condition] = STATE(2588),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(193),
    [sym_block_comment] = STATE(193),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(959),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(963),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_let] = ACTIONS(965),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [194] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1687),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_let_condition] = STATE(2883),
    [sym__let_chain] = STATE(2887),
    [sym__condition] = STATE(2604),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(194),
    [sym_block_comment] = STATE(194),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(959),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(963),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_let] = ACTIONS(965),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [195] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1687),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_let_condition] = STATE(2883),
    [sym__let_chain] = STATE(2887),
    [sym__condition] = STATE(2605),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(195),
    [sym_block_comment] = STATE(195),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(959),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(963),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_let] = ACTIONS(965),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [196] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1687),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_let_condition] = STATE(2883),
    [sym__let_chain] = STATE(2887),
    [sym__condition] = STATE(2617),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(196),
    [sym_block_comment] = STATE(196),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(959),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(963),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_let] = ACTIONS(965),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [197] = {
    [sym_line_comment] = STATE(197),
    [sym_block_comment] = STATE(197),
    [sym_identifier] = ACTIONS(1033),
    [anon_sym_SEMI] = ACTIONS(1035),
    [anon_sym_LPAREN] = ACTIONS(1035),
    [anon_sym_RPAREN] = ACTIONS(1035),
    [anon_sym_LBRACK] = ACTIONS(1035),
    [anon_sym_RBRACK] = ACTIONS(1035),
    [anon_sym_LBRACE] = ACTIONS(1035),
    [anon_sym_RBRACE] = ACTIONS(1035),
    [anon_sym_EQ_GT] = ACTIONS(1035),
    [anon_sym_COLON] = ACTIONS(1033),
    [anon_sym_DOLLAR] = ACTIONS(1033),
    [anon_sym_PLUS] = ACTIONS(1033),
    [anon_sym_STAR] = ACTIONS(1033),
    [anon_sym_QMARK] = ACTIONS(1035),
    [anon_sym_u8] = ACTIONS(1033),
    [anon_sym_i8] = ACTIONS(1033),
    [anon_sym_u16] = ACTIONS(1033),
    [anon_sym_i16] = ACTIONS(1033),
    [anon_sym_u32] = ACTIONS(1033),
    [anon_sym_i32] = ACTIONS(1033),
    [anon_sym_u64] = ACTIONS(1033),
    [anon_sym_i64] = ACTIONS(1033),
    [anon_sym_u128] = ACTIONS(1033),
    [anon_sym_i128] = ACTIONS(1033),
    [anon_sym_isize] = ACTIONS(1033),
    [anon_sym_usize] = ACTIONS(1033),
    [anon_sym_f32] = ACTIONS(1033),
    [anon_sym_f64] = ACTIONS(1033),
    [anon_sym_bool] = ACTIONS(1033),
    [anon_sym_str] = ACTIONS(1033),
    [anon_sym_char] = ACTIONS(1033),
    [anon_sym_DASH] = ACTIONS(1033),
    [anon_sym_SLASH] = ACTIONS(1033),
    [anon_sym_PERCENT] = ACTIONS(1033),
    [anon_sym_CARET] = ACTIONS(1033),
    [anon_sym_BANG] = ACTIONS(1033),
    [anon_sym_AMP] = ACTIONS(1033),
    [anon_sym_PIPE] = ACTIONS(1033),
    [anon_sym_AMP_AMP] = ACTIONS(1035),
    [anon_sym_PIPE_PIPE] = ACTIONS(1035),
    [anon_sym_LT_LT] = ACTIONS(1033),
    [anon_sym_GT_GT] = ACTIONS(1033),
    [anon_sym_PLUS_EQ] = ACTIONS(1035),
    [anon_sym_DASH_EQ] = ACTIONS(1035),
    [anon_sym_STAR_EQ] = ACTIONS(1035),
    [anon_sym_SLASH_EQ] = ACTIONS(1035),
    [anon_sym_PERCENT_EQ] = ACTIONS(1035),
    [anon_sym_CARET_EQ] = ACTIONS(1035),
    [anon_sym_AMP_EQ] = ACTIONS(1035),
    [anon_sym_PIPE_EQ] = ACTIONS(1035),
    [anon_sym_LT_LT_EQ] = ACTIONS(1035),
    [anon_sym_GT_GT_EQ] = ACTIONS(1035),
    [anon_sym_EQ] = ACTIONS(1033),
    [anon_sym_EQ_EQ] = ACTIONS(1035),
    [anon_sym_BANG_EQ] = ACTIONS(1035),
    [anon_sym_GT] = ACTIONS(1033),
    [anon_sym_LT] = ACTIONS(1033),
    [anon_sym_GT_EQ] = ACTIONS(1035),
    [anon_sym_LT_EQ] = ACTIONS(1035),
    [anon_sym_AT] = ACTIONS(1035),
    [anon_sym__] = ACTIONS(1033),
    [anon_sym_DOT] = ACTIONS(1033),
    [anon_sym_DOT_DOT] = ACTIONS(1033),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1035),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1035),
    [anon_sym_COMMA] = ACTIONS(1035),
    [anon_sym_COLON_COLON] = ACTIONS(1035),
    [anon_sym_DASH_GT] = ACTIONS(1035),
    [anon_sym_POUND] = ACTIONS(1035),
    [anon_sym_SQUOTE] = ACTIONS(1033),
    [anon_sym_as] = ACTIONS(1033),
    [anon_sym_async] = ACTIONS(1033),
    [anon_sym_await] = ACTIONS(1033),
    [anon_sym_break] = ACTIONS(1033),
    [anon_sym_const] = ACTIONS(1033),
    [anon_sym_continue] = ACTIONS(1033),
    [anon_sym_default] = ACTIONS(1033),
    [anon_sym_enum] = ACTIONS(1033),
    [anon_sym_fn] = ACTIONS(1033),
    [anon_sym_for] = ACTIONS(1033),
    [anon_sym_gen] = ACTIONS(1033),
    [anon_sym_if] = ACTIONS(1033),
    [anon_sym_impl] = ACTIONS(1033),
    [anon_sym_let] = ACTIONS(1033),
    [anon_sym_loop] = ACTIONS(1033),
    [anon_sym_match] = ACTIONS(1033),
    [anon_sym_mod] = ACTIONS(1033),
    [anon_sym_pub] = ACTIONS(1033),
    [anon_sym_return] = ACTIONS(1033),
    [anon_sym_static] = ACTIONS(1033),
    [anon_sym_struct] = ACTIONS(1033),
    [anon_sym_trait] = ACTIONS(1033),
    [anon_sym_type] = ACTIONS(1033),
    [anon_sym_union] = ACTIONS(1033),
    [anon_sym_unsafe] = ACTIONS(1033),
    [anon_sym_use] = ACTIONS(1033),
    [anon_sym_where] = ACTIONS(1033),
    [anon_sym_while] = ACTIONS(1033),
    [sym_mutable_specifier] = ACTIONS(1033),
    [sym_integer_literal] = ACTIONS(1035),
    [aux_sym_string_literal_token1] = ACTIONS(1035),
    [sym_char_literal] = ACTIONS(1035),
    [anon_sym_true] = ACTIONS(1033),
    [anon_sym_false] = ACTIONS(1033),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1033),
    [sym_super] = ACTIONS(1033),
    [sym_crate] = ACTIONS(1033),
    [sym_metavariable] = ACTIONS(1035),
    [sym__raw_string_literal_start] = ACTIONS(1035),
    [sym_float_literal] = ACTIONS(1035),
  },
  [198] = {
    [sym_line_comment] = STATE(198),
    [sym_block_comment] = STATE(198),
    [sym_identifier] = ACTIONS(1037),
    [anon_sym_SEMI] = ACTIONS(1039),
    [anon_sym_LPAREN] = ACTIONS(1039),
    [anon_sym_RPAREN] = ACTIONS(1039),
    [anon_sym_LBRACK] = ACTIONS(1039),
    [anon_sym_RBRACK] = ACTIONS(1039),
    [anon_sym_LBRACE] = ACTIONS(1039),
    [anon_sym_RBRACE] = ACTIONS(1039),
    [anon_sym_EQ_GT] = ACTIONS(1039),
    [anon_sym_COLON] = ACTIONS(1037),
    [anon_sym_DOLLAR] = ACTIONS(1039),
    [anon_sym_PLUS] = ACTIONS(1037),
    [anon_sym_STAR] = ACTIONS(1037),
    [anon_sym_QMARK] = ACTIONS(1039),
    [anon_sym_u8] = ACTIONS(1037),
    [anon_sym_i8] = ACTIONS(1037),
    [anon_sym_u16] = ACTIONS(1037),
    [anon_sym_i16] = ACTIONS(1037),
    [anon_sym_u32] = ACTIONS(1037),
    [anon_sym_i32] = ACTIONS(1037),
    [anon_sym_u64] = ACTIONS(1037),
    [anon_sym_i64] = ACTIONS(1037),
    [anon_sym_u128] = ACTIONS(1037),
    [anon_sym_i128] = ACTIONS(1037),
    [anon_sym_isize] = ACTIONS(1037),
    [anon_sym_usize] = ACTIONS(1037),
    [anon_sym_f32] = ACTIONS(1037),
    [anon_sym_f64] = ACTIONS(1037),
    [anon_sym_bool] = ACTIONS(1037),
    [anon_sym_str] = ACTIONS(1037),
    [anon_sym_char] = ACTIONS(1037),
    [anon_sym_DASH] = ACTIONS(1037),
    [anon_sym_SLASH] = ACTIONS(1037),
    [anon_sym_PERCENT] = ACTIONS(1037),
    [anon_sym_CARET] = ACTIONS(1037),
    [anon_sym_BANG] = ACTIONS(1037),
    [anon_sym_AMP] = ACTIONS(1037),
    [anon_sym_PIPE] = ACTIONS(1037),
    [anon_sym_AMP_AMP] = ACTIONS(1039),
    [anon_sym_PIPE_PIPE] = ACTIONS(1039),
    [anon_sym_LT_LT] = ACTIONS(1037),
    [anon_sym_GT_GT] = ACTIONS(1037),
    [anon_sym_PLUS_EQ] = ACTIONS(1039),
    [anon_sym_DASH_EQ] = ACTIONS(1039),
    [anon_sym_STAR_EQ] = ACTIONS(1039),
    [anon_sym_SLASH_EQ] = ACTIONS(1039),
    [anon_sym_PERCENT_EQ] = ACTIONS(1039),
    [anon_sym_CARET_EQ] = ACTIONS(1039),
    [anon_sym_AMP_EQ] = ACTIONS(1039),
    [anon_sym_PIPE_EQ] = ACTIONS(1039),
    [anon_sym_LT_LT_EQ] = ACTIONS(1039),
    [anon_sym_GT_GT_EQ] = ACTIONS(1039),
    [anon_sym_EQ] = ACTIONS(1037),
    [anon_sym_EQ_EQ] = ACTIONS(1039),
    [anon_sym_BANG_EQ] = ACTIONS(1039),
    [anon_sym_GT] = ACTIONS(1037),
    [anon_sym_LT] = ACTIONS(1037),
    [anon_sym_GT_EQ] = ACTIONS(1039),
    [anon_sym_LT_EQ] = ACTIONS(1039),
    [anon_sym_AT] = ACTIONS(1039),
    [anon_sym__] = ACTIONS(1037),
    [anon_sym_DOT] = ACTIONS(1037),
    [anon_sym_DOT_DOT] = ACTIONS(1037),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1039),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1039),
    [anon_sym_COMMA] = ACTIONS(1039),
    [anon_sym_COLON_COLON] = ACTIONS(1039),
    [anon_sym_DASH_GT] = ACTIONS(1039),
    [anon_sym_POUND] = ACTIONS(1039),
    [anon_sym_SQUOTE] = ACTIONS(1037),
    [anon_sym_as] = ACTIONS(1037),
    [anon_sym_async] = ACTIONS(1037),
    [anon_sym_await] = ACTIONS(1037),
    [anon_sym_break] = ACTIONS(1037),
    [anon_sym_const] = ACTIONS(1037),
    [anon_sym_continue] = ACTIONS(1037),
    [anon_sym_default] = ACTIONS(1037),
    [anon_sym_enum] = ACTIONS(1037),
    [anon_sym_fn] = ACTIONS(1037),
    [anon_sym_for] = ACTIONS(1037),
    [anon_sym_gen] = ACTIONS(1037),
    [anon_sym_if] = ACTIONS(1037),
    [anon_sym_impl] = ACTIONS(1037),
    [anon_sym_let] = ACTIONS(1037),
    [anon_sym_loop] = ACTIONS(1037),
    [anon_sym_match] = ACTIONS(1037),
    [anon_sym_mod] = ACTIONS(1037),
    [anon_sym_pub] = ACTIONS(1037),
    [anon_sym_return] = ACTIONS(1037),
    [anon_sym_static] = ACTIONS(1037),
    [anon_sym_struct] = ACTIONS(1037),
    [anon_sym_trait] = ACTIONS(1037),
    [anon_sym_type] = ACTIONS(1037),
    [anon_sym_union] = ACTIONS(1037),
    [anon_sym_unsafe] = ACTIONS(1037),
    [anon_sym_use] = ACTIONS(1037),
    [anon_sym_where] = ACTIONS(1037),
    [anon_sym_while] = ACTIONS(1037),
    [sym_mutable_specifier] = ACTIONS(1037),
    [sym_integer_literal] = ACTIONS(1039),
    [aux_sym_string_literal_token1] = ACTIONS(1039),
    [sym_char_literal] = ACTIONS(1039),
    [anon_sym_true] = ACTIONS(1037),
    [anon_sym_false] = ACTIONS(1037),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1037),
    [sym_super] = ACTIONS(1037),
    [sym_crate] = ACTIONS(1037),
    [sym__raw_string_literal_start] = ACTIONS(1039),
    [sym_float_literal] = ACTIONS(1039),
  },
  [199] = {
    [sym_line_comment] = STATE(199),
    [sym_block_comment] = STATE(199),
    [sym_identifier] = ACTIONS(995),
    [anon_sym_SEMI] = ACTIONS(997),
    [anon_sym_LPAREN] = ACTIONS(997),
    [anon_sym_RPAREN] = ACTIONS(997),
    [anon_sym_LBRACK] = ACTIONS(997),
    [anon_sym_RBRACK] = ACTIONS(997),
    [anon_sym_LBRACE] = ACTIONS(997),
    [anon_sym_RBRACE] = ACTIONS(997),
    [anon_sym_EQ_GT] = ACTIONS(997),
    [anon_sym_COLON] = ACTIONS(995),
    [anon_sym_DOLLAR] = ACTIONS(997),
    [anon_sym_PLUS] = ACTIONS(995),
    [anon_sym_STAR] = ACTIONS(995),
    [anon_sym_QMARK] = ACTIONS(997),
    [anon_sym_u8] = ACTIONS(995),
    [anon_sym_i8] = ACTIONS(995),
    [anon_sym_u16] = ACTIONS(995),
    [anon_sym_i16] = ACTIONS(995),
    [anon_sym_u32] = ACTIONS(995),
    [anon_sym_i32] = ACTIONS(995),
    [anon_sym_u64] = ACTIONS(995),
    [anon_sym_i64] = ACTIONS(995),
    [anon_sym_u128] = ACTIONS(995),
    [anon_sym_i128] = ACTIONS(995),
    [anon_sym_isize] = ACTIONS(995),
    [anon_sym_usize] = ACTIONS(995),
    [anon_sym_f32] = ACTIONS(995),
    [anon_sym_f64] = ACTIONS(995),
    [anon_sym_bool] = ACTIONS(995),
    [anon_sym_str] = ACTIONS(995),
    [anon_sym_char] = ACTIONS(995),
    [anon_sym_DASH] = ACTIONS(995),
    [anon_sym_SLASH] = ACTIONS(995),
    [anon_sym_PERCENT] = ACTIONS(995),
    [anon_sym_CARET] = ACTIONS(995),
    [anon_sym_BANG] = ACTIONS(995),
    [anon_sym_AMP] = ACTIONS(995),
    [anon_sym_PIPE] = ACTIONS(995),
    [anon_sym_AMP_AMP] = ACTIONS(997),
    [anon_sym_PIPE_PIPE] = ACTIONS(997),
    [anon_sym_LT_LT] = ACTIONS(995),
    [anon_sym_GT_GT] = ACTIONS(995),
    [anon_sym_PLUS_EQ] = ACTIONS(997),
    [anon_sym_DASH_EQ] = ACTIONS(997),
    [anon_sym_STAR_EQ] = ACTIONS(997),
    [anon_sym_SLASH_EQ] = ACTIONS(997),
    [anon_sym_PERCENT_EQ] = ACTIONS(997),
    [anon_sym_CARET_EQ] = ACTIONS(997),
    [anon_sym_AMP_EQ] = ACTIONS(997),
    [anon_sym_PIPE_EQ] = ACTIONS(997),
    [anon_sym_LT_LT_EQ] = ACTIONS(997),
    [anon_sym_GT_GT_EQ] = ACTIONS(997),
    [anon_sym_EQ] = ACTIONS(995),
    [anon_sym_EQ_EQ] = ACTIONS(997),
    [anon_sym_BANG_EQ] = ACTIONS(997),
    [anon_sym_GT] = ACTIONS(995),
    [anon_sym_LT] = ACTIONS(995),
    [anon_sym_GT_EQ] = ACTIONS(997),
    [anon_sym_LT_EQ] = ACTIONS(997),
    [anon_sym_AT] = ACTIONS(997),
    [anon_sym__] = ACTIONS(995),
    [anon_sym_DOT] = ACTIONS(995),
    [anon_sym_DOT_DOT] = ACTIONS(995),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(997),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(997),
    [anon_sym_COMMA] = ACTIONS(997),
    [anon_sym_COLON_COLON] = ACTIONS(997),
    [anon_sym_DASH_GT] = ACTIONS(997),
    [anon_sym_POUND] = ACTIONS(997),
    [anon_sym_SQUOTE] = ACTIONS(995),
    [anon_sym_as] = ACTIONS(995),
    [anon_sym_async] = ACTIONS(995),
    [anon_sym_await] = ACTIONS(995),
    [anon_sym_break] = ACTIONS(995),
    [anon_sym_const] = ACTIONS(995),
    [anon_sym_continue] = ACTIONS(995),
    [anon_sym_default] = ACTIONS(995),
    [anon_sym_enum] = ACTIONS(995),
    [anon_sym_fn] = ACTIONS(995),
    [anon_sym_for] = ACTIONS(995),
    [anon_sym_gen] = ACTIONS(995),
    [anon_sym_if] = ACTIONS(995),
    [anon_sym_impl] = ACTIONS(995),
    [anon_sym_let] = ACTIONS(995),
    [anon_sym_loop] = ACTIONS(995),
    [anon_sym_match] = ACTIONS(995),
    [anon_sym_mod] = ACTIONS(995),
    [anon_sym_pub] = ACTIONS(995),
    [anon_sym_return] = ACTIONS(995),
    [anon_sym_static] = ACTIONS(995),
    [anon_sym_struct] = ACTIONS(995),
    [anon_sym_trait] = ACTIONS(995),
    [anon_sym_type] = ACTIONS(995),
    [anon_sym_union] = ACTIONS(995),
    [anon_sym_unsafe] = ACTIONS(995),
    [anon_sym_use] = ACTIONS(995),
    [anon_sym_where] = ACTIONS(995),
    [anon_sym_while] = ACTIONS(995),
    [sym_mutable_specifier] = ACTIONS(995),
    [sym_integer_literal] = ACTIONS(997),
    [aux_sym_string_literal_token1] = ACTIONS(997),
    [sym_char_literal] = ACTIONS(997),
    [anon_sym_true] = ACTIONS(995),
    [anon_sym_false] = ACTIONS(995),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(995),
    [sym_super] = ACTIONS(995),
    [sym_crate] = ACTIONS(995),
    [sym__raw_string_literal_start] = ACTIONS(997),
    [sym_float_literal] = ACTIONS(997),
  },
  [200] = {
    [sym_line_comment] = STATE(200),
    [sym_block_comment] = STATE(200),
    [sym_identifier] = ACTIONS(1007),
    [anon_sym_SEMI] = ACTIONS(1009),
    [anon_sym_LPAREN] = ACTIONS(1009),
    [anon_sym_RPAREN] = ACTIONS(1009),
    [anon_sym_LBRACK] = ACTIONS(1009),
    [anon_sym_RBRACK] = ACTIONS(1009),
    [anon_sym_LBRACE] = ACTIONS(1009),
    [anon_sym_RBRACE] = ACTIONS(1009),
    [anon_sym_EQ_GT] = ACTIONS(1009),
    [anon_sym_COLON] = ACTIONS(1007),
    [anon_sym_DOLLAR] = ACTIONS(1009),
    [anon_sym_PLUS] = ACTIONS(1007),
    [anon_sym_STAR] = ACTIONS(1007),
    [anon_sym_QMARK] = ACTIONS(1009),
    [anon_sym_u8] = ACTIONS(1007),
    [anon_sym_i8] = ACTIONS(1007),
    [anon_sym_u16] = ACTIONS(1007),
    [anon_sym_i16] = ACTIONS(1007),
    [anon_sym_u32] = ACTIONS(1007),
    [anon_sym_i32] = ACTIONS(1007),
    [anon_sym_u64] = ACTIONS(1007),
    [anon_sym_i64] = ACTIONS(1007),
    [anon_sym_u128] = ACTIONS(1007),
    [anon_sym_i128] = ACTIONS(1007),
    [anon_sym_isize] = ACTIONS(1007),
    [anon_sym_usize] = ACTIONS(1007),
    [anon_sym_f32] = ACTIONS(1007),
    [anon_sym_f64] = ACTIONS(1007),
    [anon_sym_bool] = ACTIONS(1007),
    [anon_sym_str] = ACTIONS(1007),
    [anon_sym_char] = ACTIONS(1007),
    [anon_sym_DASH] = ACTIONS(1007),
    [anon_sym_SLASH] = ACTIONS(1007),
    [anon_sym_PERCENT] = ACTIONS(1007),
    [anon_sym_CARET] = ACTIONS(1007),
    [anon_sym_BANG] = ACTIONS(1007),
    [anon_sym_AMP] = ACTIONS(1007),
    [anon_sym_PIPE] = ACTIONS(1007),
    [anon_sym_AMP_AMP] = ACTIONS(1009),
    [anon_sym_PIPE_PIPE] = ACTIONS(1009),
    [anon_sym_LT_LT] = ACTIONS(1007),
    [anon_sym_GT_GT] = ACTIONS(1007),
    [anon_sym_PLUS_EQ] = ACTIONS(1009),
    [anon_sym_DASH_EQ] = ACTIONS(1009),
    [anon_sym_STAR_EQ] = ACTIONS(1009),
    [anon_sym_SLASH_EQ] = ACTIONS(1009),
    [anon_sym_PERCENT_EQ] = ACTIONS(1009),
    [anon_sym_CARET_EQ] = ACTIONS(1009),
    [anon_sym_AMP_EQ] = ACTIONS(1009),
    [anon_sym_PIPE_EQ] = ACTIONS(1009),
    [anon_sym_LT_LT_EQ] = ACTIONS(1009),
    [anon_sym_GT_GT_EQ] = ACTIONS(1009),
    [anon_sym_EQ] = ACTIONS(1007),
    [anon_sym_EQ_EQ] = ACTIONS(1009),
    [anon_sym_BANG_EQ] = ACTIONS(1009),
    [anon_sym_GT] = ACTIONS(1007),
    [anon_sym_LT] = ACTIONS(1007),
    [anon_sym_GT_EQ] = ACTIONS(1009),
    [anon_sym_LT_EQ] = ACTIONS(1009),
    [anon_sym_AT] = ACTIONS(1009),
    [anon_sym__] = ACTIONS(1007),
    [anon_sym_DOT] = ACTIONS(1007),
    [anon_sym_DOT_DOT] = ACTIONS(1007),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1009),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1009),
    [anon_sym_COMMA] = ACTIONS(1009),
    [anon_sym_COLON_COLON] = ACTIONS(1009),
    [anon_sym_DASH_GT] = ACTIONS(1009),
    [anon_sym_POUND] = ACTIONS(1009),
    [anon_sym_SQUOTE] = ACTIONS(1007),
    [anon_sym_as] = ACTIONS(1007),
    [anon_sym_async] = ACTIONS(1007),
    [anon_sym_await] = ACTIONS(1007),
    [anon_sym_break] = ACTIONS(1007),
    [anon_sym_const] = ACTIONS(1007),
    [anon_sym_continue] = ACTIONS(1007),
    [anon_sym_default] = ACTIONS(1007),
    [anon_sym_enum] = ACTIONS(1007),
    [anon_sym_fn] = ACTIONS(1007),
    [anon_sym_for] = ACTIONS(1007),
    [anon_sym_gen] = ACTIONS(1007),
    [anon_sym_if] = ACTIONS(1007),
    [anon_sym_impl] = ACTIONS(1007),
    [anon_sym_let] = ACTIONS(1007),
    [anon_sym_loop] = ACTIONS(1007),
    [anon_sym_match] = ACTIONS(1007),
    [anon_sym_mod] = ACTIONS(1007),
    [anon_sym_pub] = ACTIONS(1007),
    [anon_sym_return] = ACTIONS(1007),
    [anon_sym_static] = ACTIONS(1007),
    [anon_sym_struct] = ACTIONS(1007),
    [anon_sym_trait] = ACTIONS(1007),
    [anon_sym_type] = ACTIONS(1007),
    [anon_sym_union] = ACTIONS(1007),
    [anon_sym_unsafe] = ACTIONS(1007),
    [anon_sym_use] = ACTIONS(1007),
    [anon_sym_where] = ACTIONS(1007),
    [anon_sym_while] = ACTIONS(1007),
    [sym_mutable_specifier] = ACTIONS(1007),
    [sym_integer_literal] = ACTIONS(1009),
    [aux_sym_string_literal_token1] = ACTIONS(1009),
    [sym_char_literal] = ACTIONS(1009),
    [anon_sym_true] = ACTIONS(1007),
    [anon_sym_false] = ACTIONS(1007),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1007),
    [sym_super] = ACTIONS(1007),
    [sym_crate] = ACTIONS(1007),
    [sym__raw_string_literal_start] = ACTIONS(1009),
    [sym_float_literal] = ACTIONS(1009),
  },
  [201] = {
    [sym_attribute_item] = STATE(1011),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1624),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(201),
    [sym_block_comment] = STATE(201),
    [aux_sym_enum_variant_list_repeat1] = STATE(1010),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(748),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [202] = {
    [sym_line_comment] = STATE(202),
    [sym_block_comment] = STATE(202),
    [sym_identifier] = ACTIONS(1011),
    [anon_sym_SEMI] = ACTIONS(1013),
    [anon_sym_LPAREN] = ACTIONS(1013),
    [anon_sym_RPAREN] = ACTIONS(1013),
    [anon_sym_LBRACK] = ACTIONS(1013),
    [anon_sym_RBRACK] = ACTIONS(1013),
    [anon_sym_LBRACE] = ACTIONS(1013),
    [anon_sym_RBRACE] = ACTIONS(1013),
    [anon_sym_EQ_GT] = ACTIONS(1013),
    [anon_sym_COLON] = ACTIONS(1011),
    [anon_sym_DOLLAR] = ACTIONS(1013),
    [anon_sym_PLUS] = ACTIONS(1011),
    [anon_sym_STAR] = ACTIONS(1011),
    [anon_sym_QMARK] = ACTIONS(1013),
    [anon_sym_u8] = ACTIONS(1011),
    [anon_sym_i8] = ACTIONS(1011),
    [anon_sym_u16] = ACTIONS(1011),
    [anon_sym_i16] = ACTIONS(1011),
    [anon_sym_u32] = ACTIONS(1011),
    [anon_sym_i32] = ACTIONS(1011),
    [anon_sym_u64] = ACTIONS(1011),
    [anon_sym_i64] = ACTIONS(1011),
    [anon_sym_u128] = ACTIONS(1011),
    [anon_sym_i128] = ACTIONS(1011),
    [anon_sym_isize] = ACTIONS(1011),
    [anon_sym_usize] = ACTIONS(1011),
    [anon_sym_f32] = ACTIONS(1011),
    [anon_sym_f64] = ACTIONS(1011),
    [anon_sym_bool] = ACTIONS(1011),
    [anon_sym_str] = ACTIONS(1011),
    [anon_sym_char] = ACTIONS(1011),
    [anon_sym_DASH] = ACTIONS(1011),
    [anon_sym_SLASH] = ACTIONS(1011),
    [anon_sym_PERCENT] = ACTIONS(1011),
    [anon_sym_CARET] = ACTIONS(1011),
    [anon_sym_BANG] = ACTIONS(1011),
    [anon_sym_AMP] = ACTIONS(1011),
    [anon_sym_PIPE] = ACTIONS(1011),
    [anon_sym_AMP_AMP] = ACTIONS(1013),
    [anon_sym_PIPE_PIPE] = ACTIONS(1013),
    [anon_sym_LT_LT] = ACTIONS(1011),
    [anon_sym_GT_GT] = ACTIONS(1011),
    [anon_sym_PLUS_EQ] = ACTIONS(1013),
    [anon_sym_DASH_EQ] = ACTIONS(1013),
    [anon_sym_STAR_EQ] = ACTIONS(1013),
    [anon_sym_SLASH_EQ] = ACTIONS(1013),
    [anon_sym_PERCENT_EQ] = ACTIONS(1013),
    [anon_sym_CARET_EQ] = ACTIONS(1013),
    [anon_sym_AMP_EQ] = ACTIONS(1013),
    [anon_sym_PIPE_EQ] = ACTIONS(1013),
    [anon_sym_LT_LT_EQ] = ACTIONS(1013),
    [anon_sym_GT_GT_EQ] = ACTIONS(1013),
    [anon_sym_EQ] = ACTIONS(1011),
    [anon_sym_EQ_EQ] = ACTIONS(1013),
    [anon_sym_BANG_EQ] = ACTIONS(1013),
    [anon_sym_GT] = ACTIONS(1011),
    [anon_sym_LT] = ACTIONS(1011),
    [anon_sym_GT_EQ] = ACTIONS(1013),
    [anon_sym_LT_EQ] = ACTIONS(1013),
    [anon_sym_AT] = ACTIONS(1013),
    [anon_sym__] = ACTIONS(1011),
    [anon_sym_DOT] = ACTIONS(1011),
    [anon_sym_DOT_DOT] = ACTIONS(1011),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1013),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1013),
    [anon_sym_COMMA] = ACTIONS(1013),
    [anon_sym_COLON_COLON] = ACTIONS(1013),
    [anon_sym_DASH_GT] = ACTIONS(1013),
    [anon_sym_POUND] = ACTIONS(1013),
    [anon_sym_SQUOTE] = ACTIONS(1011),
    [anon_sym_as] = ACTIONS(1011),
    [anon_sym_async] = ACTIONS(1011),
    [anon_sym_await] = ACTIONS(1011),
    [anon_sym_break] = ACTIONS(1011),
    [anon_sym_const] = ACTIONS(1011),
    [anon_sym_continue] = ACTIONS(1011),
    [anon_sym_default] = ACTIONS(1011),
    [anon_sym_enum] = ACTIONS(1011),
    [anon_sym_fn] = ACTIONS(1011),
    [anon_sym_for] = ACTIONS(1011),
    [anon_sym_gen] = ACTIONS(1011),
    [anon_sym_if] = ACTIONS(1011),
    [anon_sym_impl] = ACTIONS(1011),
    [anon_sym_let] = ACTIONS(1011),
    [anon_sym_loop] = ACTIONS(1011),
    [anon_sym_match] = ACTIONS(1011),
    [anon_sym_mod] = ACTIONS(1011),
    [anon_sym_pub] = ACTIONS(1011),
    [anon_sym_return] = ACTIONS(1011),
    [anon_sym_static] = ACTIONS(1011),
    [anon_sym_struct] = ACTIONS(1011),
    [anon_sym_trait] = ACTIONS(1011),
    [anon_sym_type] = ACTIONS(1011),
    [anon_sym_union] = ACTIONS(1011),
    [anon_sym_unsafe] = ACTIONS(1011),
    [anon_sym_use] = ACTIONS(1011),
    [anon_sym_where] = ACTIONS(1011),
    [anon_sym_while] = ACTIONS(1011),
    [sym_mutable_specifier] = ACTIONS(1011),
    [sym_integer_literal] = ACTIONS(1013),
    [aux_sym_string_literal_token1] = ACTIONS(1013),
    [sym_char_literal] = ACTIONS(1013),
    [anon_sym_true] = ACTIONS(1011),
    [anon_sym_false] = ACTIONS(1011),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1011),
    [sym_super] = ACTIONS(1011),
    [sym_crate] = ACTIONS(1011),
    [sym__raw_string_literal_start] = ACTIONS(1013),
    [sym_float_literal] = ACTIONS(1013),
  },
  [203] = {
    [sym_line_comment] = STATE(203),
    [sym_block_comment] = STATE(203),
    [sym_identifier] = ACTIONS(1023),
    [anon_sym_SEMI] = ACTIONS(1025),
    [anon_sym_LPAREN] = ACTIONS(1025),
    [anon_sym_RPAREN] = ACTIONS(1025),
    [anon_sym_LBRACK] = ACTIONS(1025),
    [anon_sym_RBRACK] = ACTIONS(1025),
    [anon_sym_LBRACE] = ACTIONS(1025),
    [anon_sym_RBRACE] = ACTIONS(1025),
    [anon_sym_EQ_GT] = ACTIONS(1025),
    [anon_sym_COLON] = ACTIONS(1023),
    [anon_sym_DOLLAR] = ACTIONS(1025),
    [anon_sym_PLUS] = ACTIONS(1023),
    [anon_sym_STAR] = ACTIONS(1023),
    [anon_sym_QMARK] = ACTIONS(1025),
    [anon_sym_u8] = ACTIONS(1023),
    [anon_sym_i8] = ACTIONS(1023),
    [anon_sym_u16] = ACTIONS(1023),
    [anon_sym_i16] = ACTIONS(1023),
    [anon_sym_u32] = ACTIONS(1023),
    [anon_sym_i32] = ACTIONS(1023),
    [anon_sym_u64] = ACTIONS(1023),
    [anon_sym_i64] = ACTIONS(1023),
    [anon_sym_u128] = ACTIONS(1023),
    [anon_sym_i128] = ACTIONS(1023),
    [anon_sym_isize] = ACTIONS(1023),
    [anon_sym_usize] = ACTIONS(1023),
    [anon_sym_f32] = ACTIONS(1023),
    [anon_sym_f64] = ACTIONS(1023),
    [anon_sym_bool] = ACTIONS(1023),
    [anon_sym_str] = ACTIONS(1023),
    [anon_sym_char] = ACTIONS(1023),
    [anon_sym_DASH] = ACTIONS(1023),
    [anon_sym_SLASH] = ACTIONS(1023),
    [anon_sym_PERCENT] = ACTIONS(1023),
    [anon_sym_CARET] = ACTIONS(1023),
    [anon_sym_BANG] = ACTIONS(1023),
    [anon_sym_AMP] = ACTIONS(1023),
    [anon_sym_PIPE] = ACTIONS(1023),
    [anon_sym_AMP_AMP] = ACTIONS(1025),
    [anon_sym_PIPE_PIPE] = ACTIONS(1025),
    [anon_sym_LT_LT] = ACTIONS(1023),
    [anon_sym_GT_GT] = ACTIONS(1023),
    [anon_sym_PLUS_EQ] = ACTIONS(1025),
    [anon_sym_DASH_EQ] = ACTIONS(1025),
    [anon_sym_STAR_EQ] = ACTIONS(1025),
    [anon_sym_SLASH_EQ] = ACTIONS(1025),
    [anon_sym_PERCENT_EQ] = ACTIONS(1025),
    [anon_sym_CARET_EQ] = ACTIONS(1025),
    [anon_sym_AMP_EQ] = ACTIONS(1025),
    [anon_sym_PIPE_EQ] = ACTIONS(1025),
    [anon_sym_LT_LT_EQ] = ACTIONS(1025),
    [anon_sym_GT_GT_EQ] = ACTIONS(1025),
    [anon_sym_EQ] = ACTIONS(1023),
    [anon_sym_EQ_EQ] = ACTIONS(1025),
    [anon_sym_BANG_EQ] = ACTIONS(1025),
    [anon_sym_GT] = ACTIONS(1023),
    [anon_sym_LT] = ACTIONS(1023),
    [anon_sym_GT_EQ] = ACTIONS(1025),
    [anon_sym_LT_EQ] = ACTIONS(1025),
    [anon_sym_AT] = ACTIONS(1025),
    [anon_sym__] = ACTIONS(1023),
    [anon_sym_DOT] = ACTIONS(1023),
    [anon_sym_DOT_DOT] = ACTIONS(1023),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1025),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1025),
    [anon_sym_COMMA] = ACTIONS(1025),
    [anon_sym_COLON_COLON] = ACTIONS(1025),
    [anon_sym_DASH_GT] = ACTIONS(1025),
    [anon_sym_POUND] = ACTIONS(1025),
    [anon_sym_SQUOTE] = ACTIONS(1023),
    [anon_sym_as] = ACTIONS(1023),
    [anon_sym_async] = ACTIONS(1023),
    [anon_sym_await] = ACTIONS(1023),
    [anon_sym_break] = ACTIONS(1023),
    [anon_sym_const] = ACTIONS(1023),
    [anon_sym_continue] = ACTIONS(1023),
    [anon_sym_default] = ACTIONS(1023),
    [anon_sym_enum] = ACTIONS(1023),
    [anon_sym_fn] = ACTIONS(1023),
    [anon_sym_for] = ACTIONS(1023),
    [anon_sym_gen] = ACTIONS(1023),
    [anon_sym_if] = ACTIONS(1023),
    [anon_sym_impl] = ACTIONS(1023),
    [anon_sym_let] = ACTIONS(1023),
    [anon_sym_loop] = ACTIONS(1023),
    [anon_sym_match] = ACTIONS(1023),
    [anon_sym_mod] = ACTIONS(1023),
    [anon_sym_pub] = ACTIONS(1023),
    [anon_sym_return] = ACTIONS(1023),
    [anon_sym_static] = ACTIONS(1023),
    [anon_sym_struct] = ACTIONS(1023),
    [anon_sym_trait] = ACTIONS(1023),
    [anon_sym_type] = ACTIONS(1023),
    [anon_sym_union] = ACTIONS(1023),
    [anon_sym_unsafe] = ACTIONS(1023),
    [anon_sym_use] = ACTIONS(1023),
    [anon_sym_where] = ACTIONS(1023),
    [anon_sym_while] = ACTIONS(1023),
    [sym_mutable_specifier] = ACTIONS(1023),
    [sym_integer_literal] = ACTIONS(1025),
    [aux_sym_string_literal_token1] = ACTIONS(1025),
    [sym_char_literal] = ACTIONS(1025),
    [anon_sym_true] = ACTIONS(1023),
    [anon_sym_false] = ACTIONS(1023),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1023),
    [sym_super] = ACTIONS(1023),
    [sym_crate] = ACTIONS(1023),
    [sym__raw_string_literal_start] = ACTIONS(1025),
    [sym_float_literal] = ACTIONS(1025),
  },
  [204] = {
    [sym_line_comment] = STATE(204),
    [sym_block_comment] = STATE(204),
    [sym_identifier] = ACTIONS(1027),
    [anon_sym_SEMI] = ACTIONS(1029),
    [anon_sym_LPAREN] = ACTIONS(1029),
    [anon_sym_RPAREN] = ACTIONS(1029),
    [anon_sym_LBRACK] = ACTIONS(1029),
    [anon_sym_RBRACK] = ACTIONS(1029),
    [anon_sym_LBRACE] = ACTIONS(1029),
    [anon_sym_RBRACE] = ACTIONS(1029),
    [anon_sym_EQ_GT] = ACTIONS(1029),
    [anon_sym_COLON] = ACTIONS(1027),
    [anon_sym_DOLLAR] = ACTIONS(1029),
    [anon_sym_PLUS] = ACTIONS(1027),
    [anon_sym_STAR] = ACTIONS(1027),
    [anon_sym_QMARK] = ACTIONS(1029),
    [anon_sym_u8] = ACTIONS(1027),
    [anon_sym_i8] = ACTIONS(1027),
    [anon_sym_u16] = ACTIONS(1027),
    [anon_sym_i16] = ACTIONS(1027),
    [anon_sym_u32] = ACTIONS(1027),
    [anon_sym_i32] = ACTIONS(1027),
    [anon_sym_u64] = ACTIONS(1027),
    [anon_sym_i64] = ACTIONS(1027),
    [anon_sym_u128] = ACTIONS(1027),
    [anon_sym_i128] = ACTIONS(1027),
    [anon_sym_isize] = ACTIONS(1027),
    [anon_sym_usize] = ACTIONS(1027),
    [anon_sym_f32] = ACTIONS(1027),
    [anon_sym_f64] = ACTIONS(1027),
    [anon_sym_bool] = ACTIONS(1027),
    [anon_sym_str] = ACTIONS(1027),
    [anon_sym_char] = ACTIONS(1027),
    [anon_sym_DASH] = ACTIONS(1027),
    [anon_sym_SLASH] = ACTIONS(1027),
    [anon_sym_PERCENT] = ACTIONS(1027),
    [anon_sym_CARET] = ACTIONS(1027),
    [anon_sym_BANG] = ACTIONS(1027),
    [anon_sym_AMP] = ACTIONS(1027),
    [anon_sym_PIPE] = ACTIONS(1027),
    [anon_sym_AMP_AMP] = ACTIONS(1029),
    [anon_sym_PIPE_PIPE] = ACTIONS(1029),
    [anon_sym_LT_LT] = ACTIONS(1027),
    [anon_sym_GT_GT] = ACTIONS(1027),
    [anon_sym_PLUS_EQ] = ACTIONS(1029),
    [anon_sym_DASH_EQ] = ACTIONS(1029),
    [anon_sym_STAR_EQ] = ACTIONS(1029),
    [anon_sym_SLASH_EQ] = ACTIONS(1029),
    [anon_sym_PERCENT_EQ] = ACTIONS(1029),
    [anon_sym_CARET_EQ] = ACTIONS(1029),
    [anon_sym_AMP_EQ] = ACTIONS(1029),
    [anon_sym_PIPE_EQ] = ACTIONS(1029),
    [anon_sym_LT_LT_EQ] = ACTIONS(1029),
    [anon_sym_GT_GT_EQ] = ACTIONS(1029),
    [anon_sym_EQ] = ACTIONS(1027),
    [anon_sym_EQ_EQ] = ACTIONS(1029),
    [anon_sym_BANG_EQ] = ACTIONS(1029),
    [anon_sym_GT] = ACTIONS(1027),
    [anon_sym_LT] = ACTIONS(1027),
    [anon_sym_GT_EQ] = ACTIONS(1029),
    [anon_sym_LT_EQ] = ACTIONS(1029),
    [anon_sym_AT] = ACTIONS(1029),
    [anon_sym__] = ACTIONS(1027),
    [anon_sym_DOT] = ACTIONS(1027),
    [anon_sym_DOT_DOT] = ACTIONS(1027),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1029),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1029),
    [anon_sym_COMMA] = ACTIONS(1029),
    [anon_sym_COLON_COLON] = ACTIONS(1029),
    [anon_sym_DASH_GT] = ACTIONS(1029),
    [anon_sym_POUND] = ACTIONS(1029),
    [anon_sym_SQUOTE] = ACTIONS(1027),
    [anon_sym_as] = ACTIONS(1027),
    [anon_sym_async] = ACTIONS(1027),
    [anon_sym_await] = ACTIONS(1027),
    [anon_sym_break] = ACTIONS(1027),
    [anon_sym_const] = ACTIONS(1027),
    [anon_sym_continue] = ACTIONS(1027),
    [anon_sym_default] = ACTIONS(1027),
    [anon_sym_enum] = ACTIONS(1027),
    [anon_sym_fn] = ACTIONS(1027),
    [anon_sym_for] = ACTIONS(1027),
    [anon_sym_gen] = ACTIONS(1027),
    [anon_sym_if] = ACTIONS(1027),
    [anon_sym_impl] = ACTIONS(1027),
    [anon_sym_let] = ACTIONS(1027),
    [anon_sym_loop] = ACTIONS(1027),
    [anon_sym_match] = ACTIONS(1027),
    [anon_sym_mod] = ACTIONS(1027),
    [anon_sym_pub] = ACTIONS(1027),
    [anon_sym_return] = ACTIONS(1027),
    [anon_sym_static] = ACTIONS(1027),
    [anon_sym_struct] = ACTIONS(1027),
    [anon_sym_trait] = ACTIONS(1027),
    [anon_sym_type] = ACTIONS(1027),
    [anon_sym_union] = ACTIONS(1027),
    [anon_sym_unsafe] = ACTIONS(1027),
    [anon_sym_use] = ACTIONS(1027),
    [anon_sym_where] = ACTIONS(1027),
    [anon_sym_while] = ACTIONS(1027),
    [sym_mutable_specifier] = ACTIONS(1027),
    [sym_integer_literal] = ACTIONS(1029),
    [aux_sym_string_literal_token1] = ACTIONS(1029),
    [sym_char_literal] = ACTIONS(1029),
    [anon_sym_true] = ACTIONS(1027),
    [anon_sym_false] = ACTIONS(1027),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1027),
    [sym_super] = ACTIONS(1027),
    [sym_crate] = ACTIONS(1027),
    [sym__raw_string_literal_start] = ACTIONS(1029),
    [sym_float_literal] = ACTIONS(1029),
  },
  [205] = {
    [sym_attribute_item] = STATE(1011),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1623),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(205),
    [sym_block_comment] = STATE(205),
    [aux_sym_enum_variant_list_repeat1] = STATE(207),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(748),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [206] = {
    [sym_line_comment] = STATE(206),
    [sym_block_comment] = STATE(206),
    [sym_identifier] = ACTIONS(1041),
    [anon_sym_SEMI] = ACTIONS(1043),
    [anon_sym_LPAREN] = ACTIONS(1043),
    [anon_sym_RPAREN] = ACTIONS(1043),
    [anon_sym_LBRACK] = ACTIONS(1043),
    [anon_sym_RBRACK] = ACTIONS(1043),
    [anon_sym_LBRACE] = ACTIONS(1043),
    [anon_sym_RBRACE] = ACTIONS(1043),
    [anon_sym_EQ_GT] = ACTIONS(1043),
    [anon_sym_COLON] = ACTIONS(1041),
    [anon_sym_DOLLAR] = ACTIONS(1043),
    [anon_sym_PLUS] = ACTIONS(1041),
    [anon_sym_STAR] = ACTIONS(1041),
    [anon_sym_QMARK] = ACTIONS(1043),
    [anon_sym_u8] = ACTIONS(1041),
    [anon_sym_i8] = ACTIONS(1041),
    [anon_sym_u16] = ACTIONS(1041),
    [anon_sym_i16] = ACTIONS(1041),
    [anon_sym_u32] = ACTIONS(1041),
    [anon_sym_i32] = ACTIONS(1041),
    [anon_sym_u64] = ACTIONS(1041),
    [anon_sym_i64] = ACTIONS(1041),
    [anon_sym_u128] = ACTIONS(1041),
    [anon_sym_i128] = ACTIONS(1041),
    [anon_sym_isize] = ACTIONS(1041),
    [anon_sym_usize] = ACTIONS(1041),
    [anon_sym_f32] = ACTIONS(1041),
    [anon_sym_f64] = ACTIONS(1041),
    [anon_sym_bool] = ACTIONS(1041),
    [anon_sym_str] = ACTIONS(1041),
    [anon_sym_char] = ACTIONS(1041),
    [anon_sym_DASH] = ACTIONS(1041),
    [anon_sym_SLASH] = ACTIONS(1041),
    [anon_sym_PERCENT] = ACTIONS(1041),
    [anon_sym_CARET] = ACTIONS(1041),
    [anon_sym_BANG] = ACTIONS(1041),
    [anon_sym_AMP] = ACTIONS(1041),
    [anon_sym_PIPE] = ACTIONS(1041),
    [anon_sym_AMP_AMP] = ACTIONS(1043),
    [anon_sym_PIPE_PIPE] = ACTIONS(1043),
    [anon_sym_LT_LT] = ACTIONS(1041),
    [anon_sym_GT_GT] = ACTIONS(1041),
    [anon_sym_PLUS_EQ] = ACTIONS(1043),
    [anon_sym_DASH_EQ] = ACTIONS(1043),
    [anon_sym_STAR_EQ] = ACTIONS(1043),
    [anon_sym_SLASH_EQ] = ACTIONS(1043),
    [anon_sym_PERCENT_EQ] = ACTIONS(1043),
    [anon_sym_CARET_EQ] = ACTIONS(1043),
    [anon_sym_AMP_EQ] = ACTIONS(1043),
    [anon_sym_PIPE_EQ] = ACTIONS(1043),
    [anon_sym_LT_LT_EQ] = ACTIONS(1043),
    [anon_sym_GT_GT_EQ] = ACTIONS(1043),
    [anon_sym_EQ] = ACTIONS(1041),
    [anon_sym_EQ_EQ] = ACTIONS(1043),
    [anon_sym_BANG_EQ] = ACTIONS(1043),
    [anon_sym_GT] = ACTIONS(1041),
    [anon_sym_LT] = ACTIONS(1041),
    [anon_sym_GT_EQ] = ACTIONS(1043),
    [anon_sym_LT_EQ] = ACTIONS(1043),
    [anon_sym_AT] = ACTIONS(1043),
    [anon_sym__] = ACTIONS(1041),
    [anon_sym_DOT] = ACTIONS(1041),
    [anon_sym_DOT_DOT] = ACTIONS(1041),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1043),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1043),
    [anon_sym_COMMA] = ACTIONS(1043),
    [anon_sym_COLON_COLON] = ACTIONS(1043),
    [anon_sym_DASH_GT] = ACTIONS(1043),
    [anon_sym_POUND] = ACTIONS(1043),
    [anon_sym_SQUOTE] = ACTIONS(1041),
    [anon_sym_as] = ACTIONS(1041),
    [anon_sym_async] = ACTIONS(1041),
    [anon_sym_await] = ACTIONS(1041),
    [anon_sym_break] = ACTIONS(1041),
    [anon_sym_const] = ACTIONS(1041),
    [anon_sym_continue] = ACTIONS(1041),
    [anon_sym_default] = ACTIONS(1041),
    [anon_sym_enum] = ACTIONS(1041),
    [anon_sym_fn] = ACTIONS(1041),
    [anon_sym_for] = ACTIONS(1041),
    [anon_sym_gen] = ACTIONS(1041),
    [anon_sym_if] = ACTIONS(1041),
    [anon_sym_impl] = ACTIONS(1041),
    [anon_sym_let] = ACTIONS(1041),
    [anon_sym_loop] = ACTIONS(1041),
    [anon_sym_match] = ACTIONS(1041),
    [anon_sym_mod] = ACTIONS(1041),
    [anon_sym_pub] = ACTIONS(1041),
    [anon_sym_return] = ACTIONS(1041),
    [anon_sym_static] = ACTIONS(1041),
    [anon_sym_struct] = ACTIONS(1041),
    [anon_sym_trait] = ACTIONS(1041),
    [anon_sym_type] = ACTIONS(1041),
    [anon_sym_union] = ACTIONS(1041),
    [anon_sym_unsafe] = ACTIONS(1041),
    [anon_sym_use] = ACTIONS(1041),
    [anon_sym_where] = ACTIONS(1041),
    [anon_sym_while] = ACTIONS(1041),
    [sym_mutable_specifier] = ACTIONS(1041),
    [sym_integer_literal] = ACTIONS(1043),
    [aux_sym_string_literal_token1] = ACTIONS(1043),
    [sym_char_literal] = ACTIONS(1043),
    [anon_sym_true] = ACTIONS(1041),
    [anon_sym_false] = ACTIONS(1041),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1041),
    [sym_super] = ACTIONS(1041),
    [sym_crate] = ACTIONS(1041),
    [sym__raw_string_literal_start] = ACTIONS(1043),
    [sym_float_literal] = ACTIONS(1043),
  },
  [207] = {
    [sym_attribute_item] = STATE(1011),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1638),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(207),
    [sym_block_comment] = STATE(207),
    [aux_sym_enum_variant_list_repeat1] = STATE(1010),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(748),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [208] = {
    [sym_attribute_item] = STATE(1011),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1894),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(208),
    [sym_block_comment] = STATE(208),
    [aux_sym_enum_variant_list_repeat1] = STATE(1010),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(748),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [209] = {
    [sym_line_comment] = STATE(209),
    [sym_block_comment] = STATE(209),
    [sym_identifier] = ACTIONS(1045),
    [anon_sym_SEMI] = ACTIONS(1047),
    [anon_sym_LPAREN] = ACTIONS(1047),
    [anon_sym_RPAREN] = ACTIONS(1047),
    [anon_sym_LBRACK] = ACTIONS(1047),
    [anon_sym_RBRACK] = ACTIONS(1047),
    [anon_sym_LBRACE] = ACTIONS(1047),
    [anon_sym_RBRACE] = ACTIONS(1047),
    [anon_sym_EQ_GT] = ACTIONS(1047),
    [anon_sym_COLON] = ACTIONS(1045),
    [anon_sym_DOLLAR] = ACTIONS(1047),
    [anon_sym_PLUS] = ACTIONS(1045),
    [anon_sym_STAR] = ACTIONS(1045),
    [anon_sym_QMARK] = ACTIONS(1047),
    [anon_sym_u8] = ACTIONS(1045),
    [anon_sym_i8] = ACTIONS(1045),
    [anon_sym_u16] = ACTIONS(1045),
    [anon_sym_i16] = ACTIONS(1045),
    [anon_sym_u32] = ACTIONS(1045),
    [anon_sym_i32] = ACTIONS(1045),
    [anon_sym_u64] = ACTIONS(1045),
    [anon_sym_i64] = ACTIONS(1045),
    [anon_sym_u128] = ACTIONS(1045),
    [anon_sym_i128] = ACTIONS(1045),
    [anon_sym_isize] = ACTIONS(1045),
    [anon_sym_usize] = ACTIONS(1045),
    [anon_sym_f32] = ACTIONS(1045),
    [anon_sym_f64] = ACTIONS(1045),
    [anon_sym_bool] = ACTIONS(1045),
    [anon_sym_str] = ACTIONS(1045),
    [anon_sym_char] = ACTIONS(1045),
    [anon_sym_DASH] = ACTIONS(1045),
    [anon_sym_SLASH] = ACTIONS(1045),
    [anon_sym_PERCENT] = ACTIONS(1045),
    [anon_sym_CARET] = ACTIONS(1045),
    [anon_sym_BANG] = ACTIONS(1045),
    [anon_sym_AMP] = ACTIONS(1045),
    [anon_sym_PIPE] = ACTIONS(1045),
    [anon_sym_AMP_AMP] = ACTIONS(1047),
    [anon_sym_PIPE_PIPE] = ACTIONS(1047),
    [anon_sym_LT_LT] = ACTIONS(1045),
    [anon_sym_GT_GT] = ACTIONS(1045),
    [anon_sym_PLUS_EQ] = ACTIONS(1047),
    [anon_sym_DASH_EQ] = ACTIONS(1047),
    [anon_sym_STAR_EQ] = ACTIONS(1047),
    [anon_sym_SLASH_EQ] = ACTIONS(1047),
    [anon_sym_PERCENT_EQ] = ACTIONS(1047),
    [anon_sym_CARET_EQ] = ACTIONS(1047),
    [anon_sym_AMP_EQ] = ACTIONS(1047),
    [anon_sym_PIPE_EQ] = ACTIONS(1047),
    [anon_sym_LT_LT_EQ] = ACTIONS(1047),
    [anon_sym_GT_GT_EQ] = ACTIONS(1047),
    [anon_sym_EQ] = ACTIONS(1045),
    [anon_sym_EQ_EQ] = ACTIONS(1047),
    [anon_sym_BANG_EQ] = ACTIONS(1047),
    [anon_sym_GT] = ACTIONS(1045),
    [anon_sym_LT] = ACTIONS(1045),
    [anon_sym_GT_EQ] = ACTIONS(1047),
    [anon_sym_LT_EQ] = ACTIONS(1047),
    [anon_sym_AT] = ACTIONS(1047),
    [anon_sym__] = ACTIONS(1045),
    [anon_sym_DOT] = ACTIONS(1045),
    [anon_sym_DOT_DOT] = ACTIONS(1045),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1047),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1047),
    [anon_sym_COMMA] = ACTIONS(1047),
    [anon_sym_COLON_COLON] = ACTIONS(1047),
    [anon_sym_DASH_GT] = ACTIONS(1047),
    [anon_sym_POUND] = ACTIONS(1047),
    [anon_sym_SQUOTE] = ACTIONS(1045),
    [anon_sym_as] = ACTIONS(1045),
    [anon_sym_async] = ACTIONS(1045),
    [anon_sym_await] = ACTIONS(1045),
    [anon_sym_break] = ACTIONS(1045),
    [anon_sym_const] = ACTIONS(1045),
    [anon_sym_continue] = ACTIONS(1045),
    [anon_sym_default] = ACTIONS(1045),
    [anon_sym_enum] = ACTIONS(1045),
    [anon_sym_fn] = ACTIONS(1045),
    [anon_sym_for] = ACTIONS(1045),
    [anon_sym_gen] = ACTIONS(1045),
    [anon_sym_if] = ACTIONS(1045),
    [anon_sym_impl] = ACTIONS(1045),
    [anon_sym_let] = ACTIONS(1045),
    [anon_sym_loop] = ACTIONS(1045),
    [anon_sym_match] = ACTIONS(1045),
    [anon_sym_mod] = ACTIONS(1045),
    [anon_sym_pub] = ACTIONS(1045),
    [anon_sym_return] = ACTIONS(1045),
    [anon_sym_static] = ACTIONS(1045),
    [anon_sym_struct] = ACTIONS(1045),
    [anon_sym_trait] = ACTIONS(1045),
    [anon_sym_type] = ACTIONS(1045),
    [anon_sym_union] = ACTIONS(1045),
    [anon_sym_unsafe] = ACTIONS(1045),
    [anon_sym_use] = ACTIONS(1045),
    [anon_sym_where] = ACTIONS(1045),
    [anon_sym_while] = ACTIONS(1045),
    [sym_mutable_specifier] = ACTIONS(1045),
    [sym_integer_literal] = ACTIONS(1047),
    [aux_sym_string_literal_token1] = ACTIONS(1047),
    [sym_char_literal] = ACTIONS(1047),
    [anon_sym_true] = ACTIONS(1045),
    [anon_sym_false] = ACTIONS(1045),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1045),
    [sym_super] = ACTIONS(1045),
    [sym_crate] = ACTIONS(1045),
    [sym__raw_string_literal_start] = ACTIONS(1047),
    [sym_float_literal] = ACTIONS(1047),
  },
  [210] = {
    [sym_line_comment] = STATE(210),
    [sym_block_comment] = STATE(210),
    [sym_identifier] = ACTIONS(1049),
    [anon_sym_SEMI] = ACTIONS(1051),
    [anon_sym_LPAREN] = ACTIONS(1051),
    [anon_sym_RPAREN] = ACTIONS(1051),
    [anon_sym_LBRACK] = ACTIONS(1051),
    [anon_sym_RBRACK] = ACTIONS(1051),
    [anon_sym_LBRACE] = ACTIONS(1051),
    [anon_sym_RBRACE] = ACTIONS(1051),
    [anon_sym_EQ_GT] = ACTIONS(1051),
    [anon_sym_COLON] = ACTIONS(1049),
    [anon_sym_DOLLAR] = ACTIONS(1051),
    [anon_sym_PLUS] = ACTIONS(1049),
    [anon_sym_STAR] = ACTIONS(1049),
    [anon_sym_QMARK] = ACTIONS(1051),
    [anon_sym_u8] = ACTIONS(1049),
    [anon_sym_i8] = ACTIONS(1049),
    [anon_sym_u16] = ACTIONS(1049),
    [anon_sym_i16] = ACTIONS(1049),
    [anon_sym_u32] = ACTIONS(1049),
    [anon_sym_i32] = ACTIONS(1049),
    [anon_sym_u64] = ACTIONS(1049),
    [anon_sym_i64] = ACTIONS(1049),
    [anon_sym_u128] = ACTIONS(1049),
    [anon_sym_i128] = ACTIONS(1049),
    [anon_sym_isize] = ACTIONS(1049),
    [anon_sym_usize] = ACTIONS(1049),
    [anon_sym_f32] = ACTIONS(1049),
    [anon_sym_f64] = ACTIONS(1049),
    [anon_sym_bool] = ACTIONS(1049),
    [anon_sym_str] = ACTIONS(1049),
    [anon_sym_char] = ACTIONS(1049),
    [anon_sym_DASH] = ACTIONS(1049),
    [anon_sym_SLASH] = ACTIONS(1049),
    [anon_sym_PERCENT] = ACTIONS(1049),
    [anon_sym_CARET] = ACTIONS(1049),
    [anon_sym_BANG] = ACTIONS(1049),
    [anon_sym_AMP] = ACTIONS(1049),
    [anon_sym_PIPE] = ACTIONS(1049),
    [anon_sym_AMP_AMP] = ACTIONS(1051),
    [anon_sym_PIPE_PIPE] = ACTIONS(1051),
    [anon_sym_LT_LT] = ACTIONS(1049),
    [anon_sym_GT_GT] = ACTIONS(1049),
    [anon_sym_PLUS_EQ] = ACTIONS(1051),
    [anon_sym_DASH_EQ] = ACTIONS(1051),
    [anon_sym_STAR_EQ] = ACTIONS(1051),
    [anon_sym_SLASH_EQ] = ACTIONS(1051),
    [anon_sym_PERCENT_EQ] = ACTIONS(1051),
    [anon_sym_CARET_EQ] = ACTIONS(1051),
    [anon_sym_AMP_EQ] = ACTIONS(1051),
    [anon_sym_PIPE_EQ] = ACTIONS(1051),
    [anon_sym_LT_LT_EQ] = ACTIONS(1051),
    [anon_sym_GT_GT_EQ] = ACTIONS(1051),
    [anon_sym_EQ] = ACTIONS(1049),
    [anon_sym_EQ_EQ] = ACTIONS(1051),
    [anon_sym_BANG_EQ] = ACTIONS(1051),
    [anon_sym_GT] = ACTIONS(1049),
    [anon_sym_LT] = ACTIONS(1049),
    [anon_sym_GT_EQ] = ACTIONS(1051),
    [anon_sym_LT_EQ] = ACTIONS(1051),
    [anon_sym_AT] = ACTIONS(1051),
    [anon_sym__] = ACTIONS(1049),
    [anon_sym_DOT] = ACTIONS(1049),
    [anon_sym_DOT_DOT] = ACTIONS(1049),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1051),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1051),
    [anon_sym_COMMA] = ACTIONS(1051),
    [anon_sym_COLON_COLON] = ACTIONS(1051),
    [anon_sym_DASH_GT] = ACTIONS(1051),
    [anon_sym_POUND] = ACTIONS(1051),
    [anon_sym_SQUOTE] = ACTIONS(1049),
    [anon_sym_as] = ACTIONS(1049),
    [anon_sym_async] = ACTIONS(1049),
    [anon_sym_await] = ACTIONS(1049),
    [anon_sym_break] = ACTIONS(1049),
    [anon_sym_const] = ACTIONS(1049),
    [anon_sym_continue] = ACTIONS(1049),
    [anon_sym_default] = ACTIONS(1049),
    [anon_sym_enum] = ACTIONS(1049),
    [anon_sym_fn] = ACTIONS(1049),
    [anon_sym_for] = ACTIONS(1049),
    [anon_sym_gen] = ACTIONS(1049),
    [anon_sym_if] = ACTIONS(1049),
    [anon_sym_impl] = ACTIONS(1049),
    [anon_sym_let] = ACTIONS(1049),
    [anon_sym_loop] = ACTIONS(1049),
    [anon_sym_match] = ACTIONS(1049),
    [anon_sym_mod] = ACTIONS(1049),
    [anon_sym_pub] = ACTIONS(1049),
    [anon_sym_return] = ACTIONS(1049),
    [anon_sym_static] = ACTIONS(1049),
    [anon_sym_struct] = ACTIONS(1049),
    [anon_sym_trait] = ACTIONS(1049),
    [anon_sym_type] = ACTIONS(1049),
    [anon_sym_union] = ACTIONS(1049),
    [anon_sym_unsafe] = ACTIONS(1049),
    [anon_sym_use] = ACTIONS(1049),
    [anon_sym_where] = ACTIONS(1049),
    [anon_sym_while] = ACTIONS(1049),
    [sym_mutable_specifier] = ACTIONS(1049),
    [sym_integer_literal] = ACTIONS(1051),
    [aux_sym_string_literal_token1] = ACTIONS(1051),
    [sym_char_literal] = ACTIONS(1051),
    [anon_sym_true] = ACTIONS(1049),
    [anon_sym_false] = ACTIONS(1049),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1049),
    [sym_super] = ACTIONS(1049),
    [sym_crate] = ACTIONS(1049),
    [sym__raw_string_literal_start] = ACTIONS(1051),
    [sym_float_literal] = ACTIONS(1051),
  },
  [211] = {
    [sym_attribute_item] = STATE(1011),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1899),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(211),
    [sym_block_comment] = STATE(211),
    [aux_sym_enum_variant_list_repeat1] = STATE(1010),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(748),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [212] = {
    [sym_attribute_item] = STATE(1011),
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1617),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(212),
    [sym_block_comment] = STATE(212),
    [aux_sym_enum_variant_list_repeat1] = STATE(1010),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_POUND] = ACTIONS(748),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [213] = {
    [sym_line_comment] = STATE(213),
    [sym_block_comment] = STATE(213),
    [sym_identifier] = ACTIONS(973),
    [anon_sym_SEMI] = ACTIONS(975),
    [anon_sym_LPAREN] = ACTIONS(975),
    [anon_sym_RPAREN] = ACTIONS(975),
    [anon_sym_LBRACK] = ACTIONS(975),
    [anon_sym_RBRACK] = ACTIONS(975),
    [anon_sym_LBRACE] = ACTIONS(975),
    [anon_sym_RBRACE] = ACTIONS(975),
    [anon_sym_EQ_GT] = ACTIONS(975),
    [anon_sym_COLON] = ACTIONS(973),
    [anon_sym_DOLLAR] = ACTIONS(975),
    [anon_sym_PLUS] = ACTIONS(973),
    [anon_sym_STAR] = ACTIONS(973),
    [anon_sym_QMARK] = ACTIONS(975),
    [anon_sym_u8] = ACTIONS(973),
    [anon_sym_i8] = ACTIONS(973),
    [anon_sym_u16] = ACTIONS(973),
    [anon_sym_i16] = ACTIONS(973),
    [anon_sym_u32] = ACTIONS(973),
    [anon_sym_i32] = ACTIONS(973),
    [anon_sym_u64] = ACTIONS(973),
    [anon_sym_i64] = ACTIONS(973),
    [anon_sym_u128] = ACTIONS(973),
    [anon_sym_i128] = ACTIONS(973),
    [anon_sym_isize] = ACTIONS(973),
    [anon_sym_usize] = ACTIONS(973),
    [anon_sym_f32] = ACTIONS(973),
    [anon_sym_f64] = ACTIONS(973),
    [anon_sym_bool] = ACTIONS(973),
    [anon_sym_str] = ACTIONS(973),
    [anon_sym_char] = ACTIONS(973),
    [anon_sym_DASH] = ACTIONS(973),
    [anon_sym_SLASH] = ACTIONS(973),
    [anon_sym_PERCENT] = ACTIONS(973),
    [anon_sym_CARET] = ACTIONS(973),
    [anon_sym_BANG] = ACTIONS(973),
    [anon_sym_AMP] = ACTIONS(973),
    [anon_sym_PIPE] = ACTIONS(973),
    [anon_sym_AMP_AMP] = ACTIONS(975),
    [anon_sym_PIPE_PIPE] = ACTIONS(975),
    [anon_sym_LT_LT] = ACTIONS(973),
    [anon_sym_GT_GT] = ACTIONS(973),
    [anon_sym_PLUS_EQ] = ACTIONS(975),
    [anon_sym_DASH_EQ] = ACTIONS(975),
    [anon_sym_STAR_EQ] = ACTIONS(975),
    [anon_sym_SLASH_EQ] = ACTIONS(975),
    [anon_sym_PERCENT_EQ] = ACTIONS(975),
    [anon_sym_CARET_EQ] = ACTIONS(975),
    [anon_sym_AMP_EQ] = ACTIONS(975),
    [anon_sym_PIPE_EQ] = ACTIONS(975),
    [anon_sym_LT_LT_EQ] = ACTIONS(975),
    [anon_sym_GT_GT_EQ] = ACTIONS(975),
    [anon_sym_EQ] = ACTIONS(973),
    [anon_sym_EQ_EQ] = ACTIONS(975),
    [anon_sym_BANG_EQ] = ACTIONS(975),
    [anon_sym_GT] = ACTIONS(973),
    [anon_sym_LT] = ACTIONS(973),
    [anon_sym_GT_EQ] = ACTIONS(975),
    [anon_sym_LT_EQ] = ACTIONS(975),
    [anon_sym_AT] = ACTIONS(975),
    [anon_sym__] = ACTIONS(973),
    [anon_sym_DOT] = ACTIONS(973),
    [anon_sym_DOT_DOT] = ACTIONS(973),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(975),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(975),
    [anon_sym_COMMA] = ACTIONS(975),
    [anon_sym_COLON_COLON] = ACTIONS(975),
    [anon_sym_DASH_GT] = ACTIONS(975),
    [anon_sym_POUND] = ACTIONS(975),
    [anon_sym_SQUOTE] = ACTIONS(973),
    [anon_sym_as] = ACTIONS(973),
    [anon_sym_async] = ACTIONS(973),
    [anon_sym_await] = ACTIONS(973),
    [anon_sym_break] = ACTIONS(973),
    [anon_sym_const] = ACTIONS(973),
    [anon_sym_continue] = ACTIONS(973),
    [anon_sym_default] = ACTIONS(973),
    [anon_sym_enum] = ACTIONS(973),
    [anon_sym_fn] = ACTIONS(973),
    [anon_sym_for] = ACTIONS(973),
    [anon_sym_gen] = ACTIONS(973),
    [anon_sym_if] = ACTIONS(973),
    [anon_sym_impl] = ACTIONS(973),
    [anon_sym_let] = ACTIONS(973),
    [anon_sym_loop] = ACTIONS(973),
    [anon_sym_match] = ACTIONS(973),
    [anon_sym_mod] = ACTIONS(973),
    [anon_sym_pub] = ACTIONS(973),
    [anon_sym_return] = ACTIONS(973),
    [anon_sym_static] = ACTIONS(973),
    [anon_sym_struct] = ACTIONS(973),
    [anon_sym_trait] = ACTIONS(973),
    [anon_sym_type] = ACTIONS(973),
    [anon_sym_union] = ACTIONS(973),
    [anon_sym_unsafe] = ACTIONS(973),
    [anon_sym_use] = ACTIONS(973),
    [anon_sym_where] = ACTIONS(973),
    [anon_sym_while] = ACTIONS(973),
    [sym_mutable_specifier] = ACTIONS(973),
    [sym_integer_literal] = ACTIONS(975),
    [aux_sym_string_literal_token1] = ACTIONS(975),
    [sym_char_literal] = ACTIONS(975),
    [anon_sym_true] = ACTIONS(973),
    [anon_sym_false] = ACTIONS(973),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(973),
    [sym_super] = ACTIONS(973),
    [sym_crate] = ACTIONS(973),
    [sym__raw_string_literal_start] = ACTIONS(975),
    [sym_float_literal] = ACTIONS(975),
  },
  [214] = {
    [sym_line_comment] = STATE(214),
    [sym_block_comment] = STATE(214),
    [sym_identifier] = ACTIONS(931),
    [anon_sym_SEMI] = ACTIONS(933),
    [anon_sym_LPAREN] = ACTIONS(933),
    [anon_sym_RPAREN] = ACTIONS(933),
    [anon_sym_LBRACK] = ACTIONS(933),
    [anon_sym_RBRACK] = ACTIONS(933),
    [anon_sym_LBRACE] = ACTIONS(933),
    [anon_sym_RBRACE] = ACTIONS(933),
    [anon_sym_EQ_GT] = ACTIONS(933),
    [anon_sym_COLON] = ACTIONS(931),
    [anon_sym_DOLLAR] = ACTIONS(933),
    [anon_sym_PLUS] = ACTIONS(931),
    [anon_sym_STAR] = ACTIONS(931),
    [anon_sym_QMARK] = ACTIONS(933),
    [anon_sym_u8] = ACTIONS(931),
    [anon_sym_i8] = ACTIONS(931),
    [anon_sym_u16] = ACTIONS(931),
    [anon_sym_i16] = ACTIONS(931),
    [anon_sym_u32] = ACTIONS(931),
    [anon_sym_i32] = ACTIONS(931),
    [anon_sym_u64] = ACTIONS(931),
    [anon_sym_i64] = ACTIONS(931),
    [anon_sym_u128] = ACTIONS(931),
    [anon_sym_i128] = ACTIONS(931),
    [anon_sym_isize] = ACTIONS(931),
    [anon_sym_usize] = ACTIONS(931),
    [anon_sym_f32] = ACTIONS(931),
    [anon_sym_f64] = ACTIONS(931),
    [anon_sym_bool] = ACTIONS(931),
    [anon_sym_str] = ACTIONS(931),
    [anon_sym_char] = ACTIONS(931),
    [anon_sym_DASH] = ACTIONS(931),
    [anon_sym_SLASH] = ACTIONS(931),
    [anon_sym_PERCENT] = ACTIONS(931),
    [anon_sym_CARET] = ACTIONS(931),
    [anon_sym_BANG] = ACTIONS(931),
    [anon_sym_AMP] = ACTIONS(931),
    [anon_sym_PIPE] = ACTIONS(931),
    [anon_sym_AMP_AMP] = ACTIONS(933),
    [anon_sym_PIPE_PIPE] = ACTIONS(933),
    [anon_sym_LT_LT] = ACTIONS(931),
    [anon_sym_GT_GT] = ACTIONS(931),
    [anon_sym_PLUS_EQ] = ACTIONS(933),
    [anon_sym_DASH_EQ] = ACTIONS(933),
    [anon_sym_STAR_EQ] = ACTIONS(933),
    [anon_sym_SLASH_EQ] = ACTIONS(933),
    [anon_sym_PERCENT_EQ] = ACTIONS(933),
    [anon_sym_CARET_EQ] = ACTIONS(933),
    [anon_sym_AMP_EQ] = ACTIONS(933),
    [anon_sym_PIPE_EQ] = ACTIONS(933),
    [anon_sym_LT_LT_EQ] = ACTIONS(933),
    [anon_sym_GT_GT_EQ] = ACTIONS(933),
    [anon_sym_EQ] = ACTIONS(931),
    [anon_sym_EQ_EQ] = ACTIONS(933),
    [anon_sym_BANG_EQ] = ACTIONS(933),
    [anon_sym_GT] = ACTIONS(931),
    [anon_sym_LT] = ACTIONS(931),
    [anon_sym_GT_EQ] = ACTIONS(933),
    [anon_sym_LT_EQ] = ACTIONS(933),
    [anon_sym_AT] = ACTIONS(933),
    [anon_sym__] = ACTIONS(931),
    [anon_sym_DOT] = ACTIONS(931),
    [anon_sym_DOT_DOT] = ACTIONS(931),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(933),
    [anon_sym_COMMA] = ACTIONS(933),
    [anon_sym_COLON_COLON] = ACTIONS(933),
    [anon_sym_DASH_GT] = ACTIONS(933),
    [anon_sym_POUND] = ACTIONS(933),
    [anon_sym_SQUOTE] = ACTIONS(931),
    [anon_sym_as] = ACTIONS(931),
    [anon_sym_async] = ACTIONS(931),
    [anon_sym_await] = ACTIONS(931),
    [anon_sym_break] = ACTIONS(931),
    [anon_sym_const] = ACTIONS(931),
    [anon_sym_continue] = ACTIONS(931),
    [anon_sym_default] = ACTIONS(931),
    [anon_sym_enum] = ACTIONS(931),
    [anon_sym_fn] = ACTIONS(931),
    [anon_sym_for] = ACTIONS(931),
    [anon_sym_gen] = ACTIONS(931),
    [anon_sym_if] = ACTIONS(931),
    [anon_sym_impl] = ACTIONS(931),
    [anon_sym_let] = ACTIONS(931),
    [anon_sym_loop] = ACTIONS(931),
    [anon_sym_match] = ACTIONS(931),
    [anon_sym_mod] = ACTIONS(931),
    [anon_sym_pub] = ACTIONS(931),
    [anon_sym_return] = ACTIONS(931),
    [anon_sym_static] = ACTIONS(931),
    [anon_sym_struct] = ACTIONS(931),
    [anon_sym_trait] = ACTIONS(931),
    [anon_sym_type] = ACTIONS(931),
    [anon_sym_union] = ACTIONS(931),
    [anon_sym_unsafe] = ACTIONS(931),
    [anon_sym_use] = ACTIONS(931),
    [anon_sym_where] = ACTIONS(931),
    [anon_sym_while] = ACTIONS(931),
    [sym_mutable_specifier] = ACTIONS(931),
    [sym_integer_literal] = ACTIONS(933),
    [aux_sym_string_literal_token1] = ACTIONS(933),
    [sym_char_literal] = ACTIONS(933),
    [anon_sym_true] = ACTIONS(931),
    [anon_sym_false] = ACTIONS(931),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(931),
    [sym_super] = ACTIONS(931),
    [sym_crate] = ACTIONS(931),
    [sym__raw_string_literal_start] = ACTIONS(933),
    [sym_float_literal] = ACTIONS(933),
  },
  [215] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1418),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(215),
    [sym_block_comment] = STATE(215),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(959),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1053),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [sym_mutable_specifier] = ACTIONS(1055),
    [anon_sym_raw] = ACTIONS(1057),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [216] = {
    [sym_bracketed_type] = STATE(3529),
    [sym_generic_function] = STATE(1664),
    [sym_generic_type_with_turbofish] = STATE(2956),
    [sym__expression_except_range] = STATE(1626),
    [sym__expression] = STATE(1677),
    [sym_macro_invocation] = STATE(1671),
    [sym_scoped_identifier] = STATE(1575),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3163),
    [sym_range_expression] = STATE(1666),
    [sym_unary_expression] = STATE(1664),
    [sym_try_expression] = STATE(1664),
    [sym_reference_expression] = STATE(1664),
    [sym_binary_expression] = STATE(1664),
    [sym_assignment_expression] = STATE(1664),
    [sym_compound_assignment_expr] = STATE(1664),
    [sym_type_cast_expression] = STATE(1664),
    [sym_return_expression] = STATE(1664),
    [sym_yield_expression] = STATE(1664),
    [sym_call_expression] = STATE(1664),
    [sym_array_expression] = STATE(1664),
    [sym_parenthesized_expression] = STATE(1664),
    [sym_tuple_expression] = STATE(1664),
    [sym_unit_expression] = STATE(1664),
    [sym_struct_expression] = STATE(1664),
    [sym_if_expression] = STATE(1664),
    [sym_match_expression] = STATE(1664),
    [sym_while_expression] = STATE(1664),
    [sym_loop_expression] = STATE(1664),
    [sym_for_expression] = STATE(1664),
    [sym_const_block] = STATE(1664),
    [sym_closure_expression] = STATE(1664),
    [sym_closure_parameters] = STATE(232),
    [sym_label] = STATE(3614),
    [sym_break_expression] = STATE(1664),
    [sym_continue_expression] = STATE(1664),
    [sym_index_expression] = STATE(1664),
    [sym_await_expression] = STATE(1664),
    [sym_field_expression] = STATE(1597),
    [sym_unsafe_block] = STATE(1664),
    [sym_async_block] = STATE(1664),
    [sym_gen_block] = STATE(1664),
    [sym_try_block] = STATE(1664),
    [sym_block] = STATE(1664),
    [sym__literal] = STATE(1664),
    [sym_string_literal] = STATE(1786),
    [sym_raw_string_literal] = STATE(1786),
    [sym_boolean_literal] = STATE(1786),
    [sym_line_comment] = STATE(216),
    [sym_block_comment] = STATE(216),
    [sym_identifier] = ACTIONS(405),
    [anon_sym_LPAREN] = ACTIONS(495),
    [anon_sym_LBRACK] = ACTIONS(497),
    [anon_sym_LBRACE] = ACTIONS(407),
    [anon_sym_STAR] = ACTIONS(1015),
    [anon_sym_u8] = ACTIONS(411),
    [anon_sym_i8] = ACTIONS(411),
    [anon_sym_u16] = ACTIONS(411),
    [anon_sym_i16] = ACTIONS(411),
    [anon_sym_u32] = ACTIONS(411),
    [anon_sym_i32] = ACTIONS(411),
    [anon_sym_u64] = ACTIONS(411),
    [anon_sym_i64] = ACTIONS(411),
    [anon_sym_u128] = ACTIONS(411),
    [anon_sym_i128] = ACTIONS(411),
    [anon_sym_isize] = ACTIONS(411),
    [anon_sym_usize] = ACTIONS(411),
    [anon_sym_f32] = ACTIONS(411),
    [anon_sym_f64] = ACTIONS(411),
    [anon_sym_bool] = ACTIONS(411),
    [anon_sym_str] = ACTIONS(411),
    [anon_sym_char] = ACTIONS(411),
    [anon_sym_DASH] = ACTIONS(1015),
    [anon_sym_BANG] = ACTIONS(1015),
    [anon_sym_AMP] = ACTIONS(1017),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1059),
    [anon_sym_COLON_COLON] = ACTIONS(415),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(417),
    [anon_sym_break] = ACTIONS(419),
    [anon_sym_const] = ACTIONS(421),
    [anon_sym_continue] = ACTIONS(423),
    [anon_sym_default] = ACTIONS(425),
    [anon_sym_for] = ACTIONS(427),
    [anon_sym_gen] = ACTIONS(429),
    [anon_sym_if] = ACTIONS(431),
    [anon_sym_loop] = ACTIONS(433),
    [anon_sym_match] = ACTIONS(435),
    [anon_sym_return] = ACTIONS(437),
    [anon_sym_static] = ACTIONS(439),
    [anon_sym_union] = ACTIONS(425),
    [anon_sym_unsafe] = ACTIONS(441),
    [anon_sym_while] = ACTIONS(443),
    [sym_mutable_specifier] = ACTIONS(1061),
    [anon_sym_raw] = ACTIONS(1063),
    [anon_sym_yield] = ACTIONS(445),
    [anon_sym_move] = ACTIONS(447),
    [anon_sym_try] = ACTIONS(449),
    [sym_integer_literal] = ACTIONS(451),
    [aux_sym_string_literal_token1] = ACTIONS(453),
    [sym_char_literal] = ACTIONS(451),
    [anon_sym_true] = ACTIONS(455),
    [anon_sym_false] = ACTIONS(455),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(457),
    [sym_super] = ACTIONS(459),
    [sym_crate] = ACTIONS(459),
    [sym_metavariable] = ACTIONS(461),
    [sym__raw_string_literal_start] = ACTIONS(463),
    [sym_float_literal] = ACTIONS(451),
  },
  [217] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1819),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(217),
    [sym_block_comment] = STATE(217),
    [aux_sym_tuple_expression_repeat1] = STATE(229),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(1065),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [218] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1596),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_let_condition] = STATE(2580),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(218),
    [sym_block_comment] = STATE(218),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(959),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1053),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_let] = ACTIONS(965),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [219] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1704),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(246),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1441),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(219),
    [sym_block_comment] = STATE(219),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(1067),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(471),
    [anon_sym_BANG] = ACTIONS(1067),
    [anon_sym_AMP] = ACTIONS(1069),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1071),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_DASH_GT] = ACTIONS(1075),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(475),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(479),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(481),
    [anon_sym_static] = ACTIONS(483),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(485),
    [anon_sym_move] = ACTIONS(487),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [220] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1751),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1490),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(220),
    [sym_block_comment] = STATE(220),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(349),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1077),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_DASH_GT] = ACTIONS(1079),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [221] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1742),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_let_condition] = STATE(2580),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(221),
    [sym_block_comment] = STATE(221),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(959),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(963),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_let] = ACTIONS(965),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [222] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1658),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(222),
    [sym_block_comment] = STATE(222),
    [aux_sym_tuple_expression_repeat1] = STATE(245),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(1081),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [223] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1758),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1441),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(223),
    [sym_block_comment] = STATE(223),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(349),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1071),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_DASH_GT] = ACTIONS(1075),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [224] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1815),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(224),
    [sym_block_comment] = STATE(224),
    [aux_sym_tuple_expression_repeat1] = STATE(229),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(1083),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [225] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1604),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1490),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(225),
    [sym_block_comment] = STATE(225),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(503),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1077),
    [anon_sym_DOT_DOT] = ACTIONS(1053),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_DASH_GT] = ACTIONS(1079),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [226] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1815),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(226),
    [sym_block_comment] = STATE(226),
    [aux_sym_tuple_expression_repeat1] = STATE(217),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(1083),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [227] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1775),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1471),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(227),
    [sym_block_comment] = STATE(227),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(349),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1085),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_DASH_GT] = ACTIONS(1087),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [228] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1810),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(228),
    [sym_block_comment] = STATE(228),
    [aux_sym_tuple_expression_repeat1] = STATE(224),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(1089),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [229] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1864),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(229),
    [sym_block_comment] = STATE(229),
    [aux_sym_tuple_expression_repeat1] = STATE(229),
    [sym_identifier] = ACTIONS(1091),
    [anon_sym_LPAREN] = ACTIONS(1094),
    [anon_sym_RPAREN] = ACTIONS(1097),
    [anon_sym_LBRACK] = ACTIONS(1099),
    [anon_sym_LBRACE] = ACTIONS(1102),
    [anon_sym_STAR] = ACTIONS(1105),
    [anon_sym_u8] = ACTIONS(1108),
    [anon_sym_i8] = ACTIONS(1108),
    [anon_sym_u16] = ACTIONS(1108),
    [anon_sym_i16] = ACTIONS(1108),
    [anon_sym_u32] = ACTIONS(1108),
    [anon_sym_i32] = ACTIONS(1108),
    [anon_sym_u64] = ACTIONS(1108),
    [anon_sym_i64] = ACTIONS(1108),
    [anon_sym_u128] = ACTIONS(1108),
    [anon_sym_i128] = ACTIONS(1108),
    [anon_sym_isize] = ACTIONS(1108),
    [anon_sym_usize] = ACTIONS(1108),
    [anon_sym_f32] = ACTIONS(1108),
    [anon_sym_f64] = ACTIONS(1108),
    [anon_sym_bool] = ACTIONS(1108),
    [anon_sym_str] = ACTIONS(1108),
    [anon_sym_char] = ACTIONS(1108),
    [anon_sym_DASH] = ACTIONS(1105),
    [anon_sym_BANG] = ACTIONS(1105),
    [anon_sym_AMP] = ACTIONS(1111),
    [anon_sym_PIPE] = ACTIONS(1114),
    [anon_sym_LT] = ACTIONS(1117),
    [anon_sym_DOT_DOT] = ACTIONS(1120),
    [anon_sym_COLON_COLON] = ACTIONS(1123),
    [anon_sym_SQUOTE] = ACTIONS(1126),
    [anon_sym_async] = ACTIONS(1129),
    [anon_sym_break] = ACTIONS(1132),
    [anon_sym_const] = ACTIONS(1135),
    [anon_sym_continue] = ACTIONS(1138),
    [anon_sym_default] = ACTIONS(1141),
    [anon_sym_for] = ACTIONS(1144),
    [anon_sym_gen] = ACTIONS(1147),
    [anon_sym_if] = ACTIONS(1150),
    [anon_sym_loop] = ACTIONS(1153),
    [anon_sym_match] = ACTIONS(1156),
    [anon_sym_return] = ACTIONS(1159),
    [anon_sym_static] = ACTIONS(1162),
    [anon_sym_union] = ACTIONS(1141),
    [anon_sym_unsafe] = ACTIONS(1165),
    [anon_sym_while] = ACTIONS(1168),
    [anon_sym_yield] = ACTIONS(1171),
    [anon_sym_move] = ACTIONS(1174),
    [anon_sym_try] = ACTIONS(1177),
    [sym_integer_literal] = ACTIONS(1180),
    [aux_sym_string_literal_token1] = ACTIONS(1183),
    [sym_char_literal] = ACTIONS(1180),
    [anon_sym_true] = ACTIONS(1186),
    [anon_sym_false] = ACTIONS(1186),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1189),
    [sym_super] = ACTIONS(1192),
    [sym_crate] = ACTIONS(1192),
    [sym_metavariable] = ACTIONS(1195),
    [sym__raw_string_literal_start] = ACTIONS(1198),
    [sym_float_literal] = ACTIONS(1180),
  },
  [230] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1518),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1471),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(230),
    [sym_block_comment] = STATE(230),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(349),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1085),
    [anon_sym_DOT_DOT] = ACTIONS(1201),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_DASH_GT] = ACTIONS(1087),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [231] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1726),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(246),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1471),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(231),
    [sym_block_comment] = STATE(231),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(1067),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(471),
    [anon_sym_BANG] = ACTIONS(1067),
    [anon_sym_AMP] = ACTIONS(1069),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1085),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_DASH_GT] = ACTIONS(1087),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(475),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(479),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(481),
    [anon_sym_static] = ACTIONS(483),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(485),
    [anon_sym_move] = ACTIONS(487),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [232] = {
    [sym_bracketed_type] = STATE(3529),
    [sym_generic_function] = STATE(1664),
    [sym_generic_type_with_turbofish] = STATE(2956),
    [sym__expression_except_range] = STATE(1626),
    [sym__expression] = STATE(1790),
    [sym_macro_invocation] = STATE(1671),
    [sym_scoped_identifier] = STATE(1575),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3163),
    [sym_range_expression] = STATE(1666),
    [sym_unary_expression] = STATE(1664),
    [sym_try_expression] = STATE(1664),
    [sym_reference_expression] = STATE(1664),
    [sym_binary_expression] = STATE(1664),
    [sym_assignment_expression] = STATE(1664),
    [sym_compound_assignment_expr] = STATE(1664),
    [sym_type_cast_expression] = STATE(1664),
    [sym_return_expression] = STATE(1664),
    [sym_yield_expression] = STATE(1664),
    [sym_call_expression] = STATE(1664),
    [sym_array_expression] = STATE(1664),
    [sym_parenthesized_expression] = STATE(1664),
    [sym_tuple_expression] = STATE(1664),
    [sym_unit_expression] = STATE(1664),
    [sym_struct_expression] = STATE(1664),
    [sym_if_expression] = STATE(1664),
    [sym_match_expression] = STATE(1664),
    [sym_while_expression] = STATE(1664),
    [sym_loop_expression] = STATE(1664),
    [sym_for_expression] = STATE(1664),
    [sym_const_block] = STATE(1664),
    [sym_closure_expression] = STATE(1664),
    [sym_closure_parameters] = STATE(232),
    [sym_label] = STATE(3614),
    [sym_break_expression] = STATE(1664),
    [sym_continue_expression] = STATE(1664),
    [sym_index_expression] = STATE(1664),
    [sym_await_expression] = STATE(1664),
    [sym_field_expression] = STATE(1597),
    [sym_unsafe_block] = STATE(1664),
    [sym_async_block] = STATE(1664),
    [sym_gen_block] = STATE(1664),
    [sym_try_block] = STATE(1664),
    [sym_block] = STATE(1706),
    [sym__literal] = STATE(1664),
    [sym_string_literal] = STATE(1786),
    [sym_raw_string_literal] = STATE(1786),
    [sym_boolean_literal] = STATE(1786),
    [sym_line_comment] = STATE(232),
    [sym_block_comment] = STATE(232),
    [sym_identifier] = ACTIONS(405),
    [anon_sym_LPAREN] = ACTIONS(495),
    [anon_sym_LBRACK] = ACTIONS(497),
    [anon_sym_LBRACE] = ACTIONS(407),
    [anon_sym_STAR] = ACTIONS(1015),
    [anon_sym_u8] = ACTIONS(411),
    [anon_sym_i8] = ACTIONS(411),
    [anon_sym_u16] = ACTIONS(411),
    [anon_sym_i16] = ACTIONS(411),
    [anon_sym_u32] = ACTIONS(411),
    [anon_sym_i32] = ACTIONS(411),
    [anon_sym_u64] = ACTIONS(411),
    [anon_sym_i64] = ACTIONS(411),
    [anon_sym_u128] = ACTIONS(411),
    [anon_sym_i128] = ACTIONS(411),
    [anon_sym_isize] = ACTIONS(411),
    [anon_sym_usize] = ACTIONS(411),
    [anon_sym_f32] = ACTIONS(411),
    [anon_sym_f64] = ACTIONS(411),
    [anon_sym_bool] = ACTIONS(411),
    [anon_sym_str] = ACTIONS(411),
    [anon_sym_char] = ACTIONS(411),
    [anon_sym_DASH] = ACTIONS(413),
    [anon_sym_BANG] = ACTIONS(1015),
    [anon_sym_AMP] = ACTIONS(1017),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1203),
    [anon_sym_DOT_DOT] = ACTIONS(1059),
    [anon_sym_COLON_COLON] = ACTIONS(415),
    [anon_sym_DASH_GT] = ACTIONS(1205),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(417),
    [anon_sym_break] = ACTIONS(419),
    [anon_sym_const] = ACTIONS(421),
    [anon_sym_continue] = ACTIONS(423),
    [anon_sym_default] = ACTIONS(425),
    [anon_sym_for] = ACTIONS(427),
    [anon_sym_gen] = ACTIONS(429),
    [anon_sym_if] = ACTIONS(431),
    [anon_sym_loop] = ACTIONS(433),
    [anon_sym_match] = ACTIONS(435),
    [anon_sym_return] = ACTIONS(437),
    [anon_sym_static] = ACTIONS(439),
    [anon_sym_union] = ACTIONS(425),
    [anon_sym_unsafe] = ACTIONS(441),
    [anon_sym_while] = ACTIONS(443),
    [anon_sym_yield] = ACTIONS(445),
    [anon_sym_move] = ACTIONS(447),
    [anon_sym_try] = ACTIONS(449),
    [sym_integer_literal] = ACTIONS(451),
    [aux_sym_string_literal_token1] = ACTIONS(453),
    [sym_char_literal] = ACTIONS(451),
    [anon_sym_true] = ACTIONS(455),
    [anon_sym_false] = ACTIONS(455),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(457),
    [sym_super] = ACTIONS(459),
    [sym_crate] = ACTIONS(459),
    [sym_metavariable] = ACTIONS(461),
    [sym__raw_string_literal_start] = ACTIONS(463),
    [sym_float_literal] = ACTIONS(451),
  },
  [233] = {
    [sym_bracketed_type] = STATE(3529),
    [sym_generic_function] = STATE(1664),
    [sym_generic_type_with_turbofish] = STATE(2956),
    [sym__expression_except_range] = STATE(1626),
    [sym__expression] = STATE(1793),
    [sym_macro_invocation] = STATE(1671),
    [sym_scoped_identifier] = STATE(1575),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3163),
    [sym_range_expression] = STATE(1666),
    [sym_unary_expression] = STATE(1664),
    [sym_try_expression] = STATE(1664),
    [sym_reference_expression] = STATE(1664),
    [sym_binary_expression] = STATE(1664),
    [sym_assignment_expression] = STATE(1664),
    [sym_compound_assignment_expr] = STATE(1664),
    [sym_type_cast_expression] = STATE(1664),
    [sym_return_expression] = STATE(1664),
    [sym_yield_expression] = STATE(1664),
    [sym_call_expression] = STATE(1664),
    [sym_array_expression] = STATE(1664),
    [sym_parenthesized_expression] = STATE(1664),
    [sym_tuple_expression] = STATE(1664),
    [sym_unit_expression] = STATE(1664),
    [sym_struct_expression] = STATE(1664),
    [sym_if_expression] = STATE(1664),
    [sym_match_expression] = STATE(1664),
    [sym_while_expression] = STATE(1664),
    [sym_loop_expression] = STATE(1664),
    [sym_for_expression] = STATE(1664),
    [sym_const_block] = STATE(1664),
    [sym_closure_expression] = STATE(1664),
    [sym_closure_parameters] = STATE(232),
    [sym_label] = STATE(3614),
    [sym_break_expression] = STATE(1664),
    [sym_continue_expression] = STATE(1664),
    [sym_index_expression] = STATE(1664),
    [sym_await_expression] = STATE(1664),
    [sym_field_expression] = STATE(1597),
    [sym_unsafe_block] = STATE(1664),
    [sym_async_block] = STATE(1664),
    [sym_gen_block] = STATE(1664),
    [sym_try_block] = STATE(1664),
    [sym_block] = STATE(1805),
    [sym__literal] = STATE(1664),
    [sym_string_literal] = STATE(1786),
    [sym_raw_string_literal] = STATE(1786),
    [sym_boolean_literal] = STATE(1786),
    [sym_line_comment] = STATE(233),
    [sym_block_comment] = STATE(233),
    [sym_identifier] = ACTIONS(405),
    [anon_sym_LPAREN] = ACTIONS(495),
    [anon_sym_LBRACK] = ACTIONS(497),
    [anon_sym_LBRACE] = ACTIONS(407),
    [anon_sym_STAR] = ACTIONS(1015),
    [anon_sym_u8] = ACTIONS(411),
    [anon_sym_i8] = ACTIONS(411),
    [anon_sym_u16] = ACTIONS(411),
    [anon_sym_i16] = ACTIONS(411),
    [anon_sym_u32] = ACTIONS(411),
    [anon_sym_i32] = ACTIONS(411),
    [anon_sym_u64] = ACTIONS(411),
    [anon_sym_i64] = ACTIONS(411),
    [anon_sym_u128] = ACTIONS(411),
    [anon_sym_i128] = ACTIONS(411),
    [anon_sym_isize] = ACTIONS(411),
    [anon_sym_usize] = ACTIONS(411),
    [anon_sym_f32] = ACTIONS(411),
    [anon_sym_f64] = ACTIONS(411),
    [anon_sym_bool] = ACTIONS(411),
    [anon_sym_str] = ACTIONS(411),
    [anon_sym_char] = ACTIONS(411),
    [anon_sym_DASH] = ACTIONS(413),
    [anon_sym_BANG] = ACTIONS(1015),
    [anon_sym_AMP] = ACTIONS(1017),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1207),
    [anon_sym_DOT_DOT] = ACTIONS(1059),
    [anon_sym_COLON_COLON] = ACTIONS(415),
    [anon_sym_DASH_GT] = ACTIONS(1209),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(417),
    [anon_sym_break] = ACTIONS(419),
    [anon_sym_const] = ACTIONS(421),
    [anon_sym_continue] = ACTIONS(423),
    [anon_sym_default] = ACTIONS(425),
    [anon_sym_for] = ACTIONS(427),
    [anon_sym_gen] = ACTIONS(429),
    [anon_sym_if] = ACTIONS(431),
    [anon_sym_loop] = ACTIONS(433),
    [anon_sym_match] = ACTIONS(435),
    [anon_sym_return] = ACTIONS(437),
    [anon_sym_static] = ACTIONS(439),
    [anon_sym_union] = ACTIONS(425),
    [anon_sym_unsafe] = ACTIONS(441),
    [anon_sym_while] = ACTIONS(443),
    [anon_sym_yield] = ACTIONS(445),
    [anon_sym_move] = ACTIONS(447),
    [anon_sym_try] = ACTIONS(449),
    [sym_integer_literal] = ACTIONS(451),
    [aux_sym_string_literal_token1] = ACTIONS(453),
    [sym_char_literal] = ACTIONS(451),
    [anon_sym_true] = ACTIONS(455),
    [anon_sym_false] = ACTIONS(455),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(457),
    [sym_super] = ACTIONS(459),
    [sym_crate] = ACTIONS(459),
    [sym_metavariable] = ACTIONS(461),
    [sym__raw_string_literal_start] = ACTIONS(463),
    [sym_float_literal] = ACTIONS(451),
  },
  [234] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1641),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1441),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(234),
    [sym_block_comment] = STATE(234),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(503),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1071),
    [anon_sym_DOT_DOT] = ACTIONS(1053),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_DASH_GT] = ACTIONS(1075),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [235] = {
    [sym_bracketed_type] = STATE(3529),
    [sym_generic_function] = STATE(1664),
    [sym_generic_type_with_turbofish] = STATE(2956),
    [sym__expression_except_range] = STATE(1626),
    [sym__expression] = STATE(1796),
    [sym_macro_invocation] = STATE(1671),
    [sym_scoped_identifier] = STATE(1575),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3163),
    [sym_range_expression] = STATE(1666),
    [sym_unary_expression] = STATE(1664),
    [sym_try_expression] = STATE(1664),
    [sym_reference_expression] = STATE(1664),
    [sym_binary_expression] = STATE(1664),
    [sym_assignment_expression] = STATE(1664),
    [sym_compound_assignment_expr] = STATE(1664),
    [sym_type_cast_expression] = STATE(1664),
    [sym_return_expression] = STATE(1664),
    [sym_yield_expression] = STATE(1664),
    [sym_call_expression] = STATE(1664),
    [sym_array_expression] = STATE(1664),
    [sym_parenthesized_expression] = STATE(1664),
    [sym_tuple_expression] = STATE(1664),
    [sym_unit_expression] = STATE(1664),
    [sym_struct_expression] = STATE(1664),
    [sym_if_expression] = STATE(1664),
    [sym_match_expression] = STATE(1664),
    [sym_while_expression] = STATE(1664),
    [sym_loop_expression] = STATE(1664),
    [sym_for_expression] = STATE(1664),
    [sym_const_block] = STATE(1664),
    [sym_closure_expression] = STATE(1664),
    [sym_closure_parameters] = STATE(232),
    [sym_label] = STATE(3614),
    [sym_break_expression] = STATE(1664),
    [sym_continue_expression] = STATE(1664),
    [sym_index_expression] = STATE(1664),
    [sym_await_expression] = STATE(1664),
    [sym_field_expression] = STATE(1597),
    [sym_unsafe_block] = STATE(1664),
    [sym_async_block] = STATE(1664),
    [sym_gen_block] = STATE(1664),
    [sym_try_block] = STATE(1664),
    [sym_block] = STATE(1660),
    [sym__literal] = STATE(1664),
    [sym_string_literal] = STATE(1786),
    [sym_raw_string_literal] = STATE(1786),
    [sym_boolean_literal] = STATE(1786),
    [sym_line_comment] = STATE(235),
    [sym_block_comment] = STATE(235),
    [sym_identifier] = ACTIONS(405),
    [anon_sym_LPAREN] = ACTIONS(495),
    [anon_sym_LBRACK] = ACTIONS(497),
    [anon_sym_LBRACE] = ACTIONS(407),
    [anon_sym_STAR] = ACTIONS(1015),
    [anon_sym_u8] = ACTIONS(411),
    [anon_sym_i8] = ACTIONS(411),
    [anon_sym_u16] = ACTIONS(411),
    [anon_sym_i16] = ACTIONS(411),
    [anon_sym_u32] = ACTIONS(411),
    [anon_sym_i32] = ACTIONS(411),
    [anon_sym_u64] = ACTIONS(411),
    [anon_sym_i64] = ACTIONS(411),
    [anon_sym_u128] = ACTIONS(411),
    [anon_sym_i128] = ACTIONS(411),
    [anon_sym_isize] = ACTIONS(411),
    [anon_sym_usize] = ACTIONS(411),
    [anon_sym_f32] = ACTIONS(411),
    [anon_sym_f64] = ACTIONS(411),
    [anon_sym_bool] = ACTIONS(411),
    [anon_sym_str] = ACTIONS(411),
    [anon_sym_char] = ACTIONS(411),
    [anon_sym_DASH] = ACTIONS(413),
    [anon_sym_BANG] = ACTIONS(1015),
    [anon_sym_AMP] = ACTIONS(1017),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1211),
    [anon_sym_DOT_DOT] = ACTIONS(1059),
    [anon_sym_COLON_COLON] = ACTIONS(415),
    [anon_sym_DASH_GT] = ACTIONS(1213),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(417),
    [anon_sym_break] = ACTIONS(419),
    [anon_sym_const] = ACTIONS(421),
    [anon_sym_continue] = ACTIONS(423),
    [anon_sym_default] = ACTIONS(425),
    [anon_sym_for] = ACTIONS(427),
    [anon_sym_gen] = ACTIONS(429),
    [anon_sym_if] = ACTIONS(431),
    [anon_sym_loop] = ACTIONS(433),
    [anon_sym_match] = ACTIONS(435),
    [anon_sym_return] = ACTIONS(437),
    [anon_sym_static] = ACTIONS(439),
    [anon_sym_union] = ACTIONS(425),
    [anon_sym_unsafe] = ACTIONS(441),
    [anon_sym_while] = ACTIONS(443),
    [anon_sym_yield] = ACTIONS(445),
    [anon_sym_move] = ACTIONS(447),
    [anon_sym_try] = ACTIONS(449),
    [sym_integer_literal] = ACTIONS(451),
    [aux_sym_string_literal_token1] = ACTIONS(453),
    [sym_char_literal] = ACTIONS(451),
    [anon_sym_true] = ACTIONS(455),
    [anon_sym_false] = ACTIONS(455),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(457),
    [sym_super] = ACTIONS(459),
    [sym_crate] = ACTIONS(459),
    [sym_metavariable] = ACTIONS(461),
    [sym__raw_string_literal_start] = ACTIONS(463),
    [sym_float_literal] = ACTIONS(451),
  },
  [236] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1844),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(236),
    [sym_block_comment] = STATE(236),
    [aux_sym_tuple_expression_repeat1] = STATE(244),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(1215),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [237] = {
    [sym_bracketed_type] = STATE(3529),
    [sym_generic_function] = STATE(1664),
    [sym_generic_type_with_turbofish] = STATE(2956),
    [sym__expression_except_range] = STATE(1626),
    [sym__expression] = STATE(1764),
    [sym_macro_invocation] = STATE(1671),
    [sym_scoped_identifier] = STATE(1575),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3163),
    [sym_range_expression] = STATE(1666),
    [sym_unary_expression] = STATE(1664),
    [sym_try_expression] = STATE(1664),
    [sym_reference_expression] = STATE(1664),
    [sym_binary_expression] = STATE(1664),
    [sym_assignment_expression] = STATE(1664),
    [sym_compound_assignment_expr] = STATE(1664),
    [sym_type_cast_expression] = STATE(1664),
    [sym_return_expression] = STATE(1664),
    [sym_yield_expression] = STATE(1664),
    [sym_call_expression] = STATE(1664),
    [sym_array_expression] = STATE(1664),
    [sym_parenthesized_expression] = STATE(1664),
    [sym_tuple_expression] = STATE(1664),
    [sym_unit_expression] = STATE(1664),
    [sym_struct_expression] = STATE(1664),
    [sym_if_expression] = STATE(1664),
    [sym_let_condition] = STATE(2580),
    [sym_match_expression] = STATE(1664),
    [sym_while_expression] = STATE(1664),
    [sym_loop_expression] = STATE(1664),
    [sym_for_expression] = STATE(1664),
    [sym_const_block] = STATE(1664),
    [sym_closure_expression] = STATE(1664),
    [sym_closure_parameters] = STATE(232),
    [sym_label] = STATE(3614),
    [sym_break_expression] = STATE(1664),
    [sym_continue_expression] = STATE(1664),
    [sym_index_expression] = STATE(1664),
    [sym_await_expression] = STATE(1664),
    [sym_field_expression] = STATE(1597),
    [sym_unsafe_block] = STATE(1664),
    [sym_async_block] = STATE(1664),
    [sym_gen_block] = STATE(1664),
    [sym_try_block] = STATE(1664),
    [sym_block] = STATE(1664),
    [sym__literal] = STATE(1664),
    [sym_string_literal] = STATE(1786),
    [sym_raw_string_literal] = STATE(1786),
    [sym_boolean_literal] = STATE(1786),
    [sym_line_comment] = STATE(237),
    [sym_block_comment] = STATE(237),
    [sym_identifier] = ACTIONS(405),
    [anon_sym_LPAREN] = ACTIONS(495),
    [anon_sym_LBRACK] = ACTIONS(497),
    [anon_sym_LBRACE] = ACTIONS(407),
    [anon_sym_STAR] = ACTIONS(1015),
    [anon_sym_u8] = ACTIONS(411),
    [anon_sym_i8] = ACTIONS(411),
    [anon_sym_u16] = ACTIONS(411),
    [anon_sym_i16] = ACTIONS(411),
    [anon_sym_u32] = ACTIONS(411),
    [anon_sym_i32] = ACTIONS(411),
    [anon_sym_u64] = ACTIONS(411),
    [anon_sym_i64] = ACTIONS(411),
    [anon_sym_u128] = ACTIONS(411),
    [anon_sym_i128] = ACTIONS(411),
    [anon_sym_isize] = ACTIONS(411),
    [anon_sym_usize] = ACTIONS(411),
    [anon_sym_f32] = ACTIONS(411),
    [anon_sym_f64] = ACTIONS(411),
    [anon_sym_bool] = ACTIONS(411),
    [anon_sym_str] = ACTIONS(411),
    [anon_sym_char] = ACTIONS(411),
    [anon_sym_DASH] = ACTIONS(1015),
    [anon_sym_BANG] = ACTIONS(1015),
    [anon_sym_AMP] = ACTIONS(1017),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1059),
    [anon_sym_COLON_COLON] = ACTIONS(415),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(417),
    [anon_sym_break] = ACTIONS(419),
    [anon_sym_const] = ACTIONS(421),
    [anon_sym_continue] = ACTIONS(423),
    [anon_sym_default] = ACTIONS(425),
    [anon_sym_for] = ACTIONS(427),
    [anon_sym_gen] = ACTIONS(429),
    [anon_sym_if] = ACTIONS(431),
    [anon_sym_let] = ACTIONS(1021),
    [anon_sym_loop] = ACTIONS(433),
    [anon_sym_match] = ACTIONS(435),
    [anon_sym_return] = ACTIONS(437),
    [anon_sym_static] = ACTIONS(439),
    [anon_sym_union] = ACTIONS(425),
    [anon_sym_unsafe] = ACTIONS(441),
    [anon_sym_while] = ACTIONS(443),
    [anon_sym_yield] = ACTIONS(445),
    [anon_sym_move] = ACTIONS(447),
    [anon_sym_try] = ACTIONS(449),
    [sym_integer_literal] = ACTIONS(451),
    [aux_sym_string_literal_token1] = ACTIONS(453),
    [sym_char_literal] = ACTIONS(451),
    [anon_sym_true] = ACTIONS(455),
    [anon_sym_false] = ACTIONS(455),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(457),
    [sym_super] = ACTIONS(459),
    [sym_crate] = ACTIONS(459),
    [sym_metavariable] = ACTIONS(461),
    [sym__raw_string_literal_start] = ACTIONS(463),
    [sym_float_literal] = ACTIONS(451),
  },
  [238] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1418),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(238),
    [sym_block_comment] = STATE(238),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1201),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [sym_mutable_specifier] = ACTIONS(1217),
    [anon_sym_raw] = ACTIONS(1219),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [239] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1502),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1490),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(239),
    [sym_block_comment] = STATE(239),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(349),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1077),
    [anon_sym_DOT_DOT] = ACTIONS(1201),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_DASH_GT] = ACTIONS(1079),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [240] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1418),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(246),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(240),
    [sym_block_comment] = STATE(240),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(1067),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(1067),
    [anon_sym_BANG] = ACTIONS(1067),
    [anon_sym_AMP] = ACTIONS(1069),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(475),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(479),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(481),
    [anon_sym_static] = ACTIONS(483),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [sym_mutable_specifier] = ACTIONS(1221),
    [anon_sym_raw] = ACTIONS(1223),
    [anon_sym_yield] = ACTIONS(485),
    [anon_sym_move] = ACTIONS(487),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [241] = {
    [sym_bracketed_type] = STATE(3529),
    [sym_generic_function] = STATE(1664),
    [sym_generic_type_with_turbofish] = STATE(2956),
    [sym__expression_except_range] = STATE(1626),
    [sym__expression] = STATE(1867),
    [sym_macro_invocation] = STATE(1671),
    [sym_scoped_identifier] = STATE(1575),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3163),
    [sym_range_expression] = STATE(1666),
    [sym_unary_expression] = STATE(1664),
    [sym_try_expression] = STATE(1664),
    [sym_reference_expression] = STATE(1664),
    [sym_binary_expression] = STATE(1664),
    [sym_assignment_expression] = STATE(1664),
    [sym_compound_assignment_expr] = STATE(1664),
    [sym_type_cast_expression] = STATE(1664),
    [sym_return_expression] = STATE(1664),
    [sym_yield_expression] = STATE(1664),
    [sym_call_expression] = STATE(1664),
    [sym_array_expression] = STATE(1664),
    [sym_parenthesized_expression] = STATE(1664),
    [sym_tuple_expression] = STATE(1664),
    [sym_unit_expression] = STATE(1664),
    [sym_struct_expression] = STATE(1664),
    [sym_if_expression] = STATE(1664),
    [sym_let_condition] = STATE(2580),
    [sym_match_expression] = STATE(1664),
    [sym_while_expression] = STATE(1664),
    [sym_loop_expression] = STATE(1664),
    [sym_for_expression] = STATE(1664),
    [sym_const_block] = STATE(1664),
    [sym_closure_expression] = STATE(1664),
    [sym_closure_parameters] = STATE(232),
    [sym_label] = STATE(3614),
    [sym_break_expression] = STATE(1664),
    [sym_continue_expression] = STATE(1664),
    [sym_index_expression] = STATE(1664),
    [sym_await_expression] = STATE(1664),
    [sym_field_expression] = STATE(1597),
    [sym_unsafe_block] = STATE(1664),
    [sym_async_block] = STATE(1664),
    [sym_gen_block] = STATE(1664),
    [sym_try_block] = STATE(1664),
    [sym_block] = STATE(1664),
    [sym__literal] = STATE(1664),
    [sym_string_literal] = STATE(1786),
    [sym_raw_string_literal] = STATE(1786),
    [sym_boolean_literal] = STATE(1786),
    [sym_line_comment] = STATE(241),
    [sym_block_comment] = STATE(241),
    [sym_identifier] = ACTIONS(405),
    [anon_sym_LPAREN] = ACTIONS(495),
    [anon_sym_LBRACK] = ACTIONS(497),
    [anon_sym_LBRACE] = ACTIONS(407),
    [anon_sym_STAR] = ACTIONS(1015),
    [anon_sym_u8] = ACTIONS(411),
    [anon_sym_i8] = ACTIONS(411),
    [anon_sym_u16] = ACTIONS(411),
    [anon_sym_i16] = ACTIONS(411),
    [anon_sym_u32] = ACTIONS(411),
    [anon_sym_i32] = ACTIONS(411),
    [anon_sym_u64] = ACTIONS(411),
    [anon_sym_i64] = ACTIONS(411),
    [anon_sym_u128] = ACTIONS(411),
    [anon_sym_i128] = ACTIONS(411),
    [anon_sym_isize] = ACTIONS(411),
    [anon_sym_usize] = ACTIONS(411),
    [anon_sym_f32] = ACTIONS(411),
    [anon_sym_f64] = ACTIONS(411),
    [anon_sym_bool] = ACTIONS(411),
    [anon_sym_str] = ACTIONS(411),
    [anon_sym_char] = ACTIONS(411),
    [anon_sym_DASH] = ACTIONS(1015),
    [anon_sym_BANG] = ACTIONS(1015),
    [anon_sym_AMP] = ACTIONS(1017),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1019),
    [anon_sym_COLON_COLON] = ACTIONS(415),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(417),
    [anon_sym_break] = ACTIONS(419),
    [anon_sym_const] = ACTIONS(421),
    [anon_sym_continue] = ACTIONS(423),
    [anon_sym_default] = ACTIONS(425),
    [anon_sym_for] = ACTIONS(427),
    [anon_sym_gen] = ACTIONS(429),
    [anon_sym_if] = ACTIONS(431),
    [anon_sym_let] = ACTIONS(1021),
    [anon_sym_loop] = ACTIONS(433),
    [anon_sym_match] = ACTIONS(435),
    [anon_sym_return] = ACTIONS(437),
    [anon_sym_static] = ACTIONS(439),
    [anon_sym_union] = ACTIONS(425),
    [anon_sym_unsafe] = ACTIONS(441),
    [anon_sym_while] = ACTIONS(443),
    [anon_sym_yield] = ACTIONS(445),
    [anon_sym_move] = ACTIONS(447),
    [anon_sym_try] = ACTIONS(449),
    [sym_integer_literal] = ACTIONS(451),
    [aux_sym_string_literal_token1] = ACTIONS(453),
    [sym_char_literal] = ACTIONS(451),
    [anon_sym_true] = ACTIONS(455),
    [anon_sym_false] = ACTIONS(455),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(457),
    [sym_super] = ACTIONS(459),
    [sym_crate] = ACTIONS(459),
    [sym_metavariable] = ACTIONS(461),
    [sym__raw_string_literal_start] = ACTIONS(463),
    [sym_float_literal] = ACTIONS(451),
  },
  [242] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1653),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1471),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(242),
    [sym_block_comment] = STATE(242),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(503),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1085),
    [anon_sym_DOT_DOT] = ACTIONS(1053),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_DASH_GT] = ACTIONS(1087),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [243] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1526),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1441),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(243),
    [sym_block_comment] = STATE(243),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(349),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1071),
    [anon_sym_DOT_DOT] = ACTIONS(1201),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_DASH_GT] = ACTIONS(1075),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [244] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1656),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(244),
    [sym_block_comment] = STATE(244),
    [aux_sym_tuple_expression_repeat1] = STATE(229),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(1225),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [245] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1844),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(245),
    [sym_block_comment] = STATE(245),
    [aux_sym_tuple_expression_repeat1] = STATE(229),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(1215),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [246] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1688),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(246),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1490),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(246),
    [sym_block_comment] = STATE(246),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(1067),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(471),
    [anon_sym_BANG] = ACTIONS(1067),
    [anon_sym_AMP] = ACTIONS(1069),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1077),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_DASH_GT] = ACTIONS(1079),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(475),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(479),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(481),
    [anon_sym_static] = ACTIONS(483),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(485),
    [anon_sym_move] = ACTIONS(487),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [247] = {
    [sym_bracketed_type] = STATE(3529),
    [sym_generic_function] = STATE(1664),
    [sym_generic_type_with_turbofish] = STATE(2956),
    [sym__expression_except_range] = STATE(1626),
    [sym__expression] = STATE(1725),
    [sym_macro_invocation] = STATE(1671),
    [sym_scoped_identifier] = STATE(1575),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3163),
    [sym_range_expression] = STATE(1666),
    [sym_unary_expression] = STATE(1664),
    [sym_try_expression] = STATE(1664),
    [sym_reference_expression] = STATE(1664),
    [sym_binary_expression] = STATE(1664),
    [sym_assignment_expression] = STATE(1664),
    [sym_compound_assignment_expr] = STATE(1664),
    [sym_type_cast_expression] = STATE(1664),
    [sym_return_expression] = STATE(1664),
    [sym_yield_expression] = STATE(1664),
    [sym_call_expression] = STATE(1664),
    [sym_array_expression] = STATE(1664),
    [sym_parenthesized_expression] = STATE(1664),
    [sym_tuple_expression] = STATE(1664),
    [sym_unit_expression] = STATE(1664),
    [sym_struct_expression] = STATE(1664),
    [sym_if_expression] = STATE(1664),
    [sym_match_expression] = STATE(1664),
    [sym_while_expression] = STATE(1664),
    [sym_loop_expression] = STATE(1664),
    [sym_for_expression] = STATE(1664),
    [sym_const_block] = STATE(1664),
    [sym_closure_expression] = STATE(1664),
    [sym_closure_parameters] = STATE(232),
    [sym_label] = STATE(3614),
    [sym_break_expression] = STATE(1664),
    [sym_continue_expression] = STATE(1664),
    [sym_index_expression] = STATE(1664),
    [sym_await_expression] = STATE(1664),
    [sym_field_expression] = STATE(1597),
    [sym_unsafe_block] = STATE(1664),
    [sym_async_block] = STATE(1664),
    [sym_gen_block] = STATE(1664),
    [sym_try_block] = STATE(1664),
    [sym_block] = STATE(1664),
    [sym__literal] = STATE(1664),
    [sym_string_literal] = STATE(1786),
    [sym_raw_string_literal] = STATE(1786),
    [sym_boolean_literal] = STATE(1786),
    [sym_line_comment] = STATE(247),
    [sym_block_comment] = STATE(247),
    [sym_identifier] = ACTIONS(405),
    [anon_sym_LPAREN] = ACTIONS(495),
    [anon_sym_LBRACK] = ACTIONS(497),
    [anon_sym_LBRACE] = ACTIONS(407),
    [anon_sym_STAR] = ACTIONS(1015),
    [anon_sym_u8] = ACTIONS(411),
    [anon_sym_i8] = ACTIONS(411),
    [anon_sym_u16] = ACTIONS(411),
    [anon_sym_i16] = ACTIONS(411),
    [anon_sym_u32] = ACTIONS(411),
    [anon_sym_i32] = ACTIONS(411),
    [anon_sym_u64] = ACTIONS(411),
    [anon_sym_i64] = ACTIONS(411),
    [anon_sym_u128] = ACTIONS(411),
    [anon_sym_i128] = ACTIONS(411),
    [anon_sym_isize] = ACTIONS(411),
    [anon_sym_usize] = ACTIONS(411),
    [anon_sym_f32] = ACTIONS(411),
    [anon_sym_f64] = ACTIONS(411),
    [anon_sym_bool] = ACTIONS(411),
    [anon_sym_str] = ACTIONS(411),
    [anon_sym_char] = ACTIONS(411),
    [anon_sym_DASH] = ACTIONS(1015),
    [anon_sym_BANG] = ACTIONS(1015),
    [anon_sym_AMP] = ACTIONS(1017),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1059),
    [anon_sym_COLON_COLON] = ACTIONS(415),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(417),
    [anon_sym_break] = ACTIONS(419),
    [anon_sym_const] = ACTIONS(421),
    [anon_sym_continue] = ACTIONS(423),
    [anon_sym_default] = ACTIONS(425),
    [anon_sym_for] = ACTIONS(427),
    [anon_sym_gen] = ACTIONS(429),
    [anon_sym_if] = ACTIONS(431),
    [anon_sym_loop] = ACTIONS(433),
    [anon_sym_match] = ACTIONS(435),
    [anon_sym_return] = ACTIONS(437),
    [anon_sym_static] = ACTIONS(439),
    [anon_sym_union] = ACTIONS(425),
    [anon_sym_unsafe] = ACTIONS(441),
    [anon_sym_while] = ACTIONS(443),
    [anon_sym_yield] = ACTIONS(445),
    [anon_sym_move] = ACTIONS(447),
    [anon_sym_try] = ACTIONS(449),
    [sym_integer_literal] = ACTIONS(451),
    [aux_sym_string_literal_token1] = ACTIONS(453),
    [sym_char_literal] = ACTIONS(451),
    [anon_sym_true] = ACTIONS(455),
    [anon_sym_false] = ACTIONS(455),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(457),
    [sym_super] = ACTIONS(459),
    [sym_crate] = ACTIONS(459),
    [sym_metavariable] = ACTIONS(461),
    [sym__raw_string_literal_start] = ACTIONS(463),
    [sym_float_literal] = ACTIONS(451),
  },
  [248] = {
    [sym_bracketed_type] = STATE(3529),
    [sym_generic_function] = STATE(1664),
    [sym_generic_type_with_turbofish] = STATE(2956),
    [sym__expression_except_range] = STATE(1626),
    [sym__expression] = STATE(1762),
    [sym_macro_invocation] = STATE(1671),
    [sym_scoped_identifier] = STATE(1575),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3163),
    [sym_range_expression] = STATE(1666),
    [sym_unary_expression] = STATE(1664),
    [sym_try_expression] = STATE(1664),
    [sym_reference_expression] = STATE(1664),
    [sym_binary_expression] = STATE(1664),
    [sym_assignment_expression] = STATE(1664),
    [sym_compound_assignment_expr] = STATE(1664),
    [sym_type_cast_expression] = STATE(1664),
    [sym_return_expression] = STATE(1664),
    [sym_yield_expression] = STATE(1664),
    [sym_call_expression] = STATE(1664),
    [sym_array_expression] = STATE(1664),
    [sym_parenthesized_expression] = STATE(1664),
    [sym_tuple_expression] = STATE(1664),
    [sym_unit_expression] = STATE(1664),
    [sym_struct_expression] = STATE(1664),
    [sym_if_expression] = STATE(1664),
    [sym_match_expression] = STATE(1664),
    [sym_while_expression] = STATE(1664),
    [sym_loop_expression] = STATE(1664),
    [sym_for_expression] = STATE(1664),
    [sym_const_block] = STATE(1664),
    [sym_closure_expression] = STATE(1664),
    [sym_closure_parameters] = STATE(232),
    [sym_label] = STATE(3614),
    [sym_break_expression] = STATE(1664),
    [sym_continue_expression] = STATE(1664),
    [sym_index_expression] = STATE(1664),
    [sym_await_expression] = STATE(1664),
    [sym_field_expression] = STATE(1597),
    [sym_unsafe_block] = STATE(1664),
    [sym_async_block] = STATE(1664),
    [sym_gen_block] = STATE(1664),
    [sym_try_block] = STATE(1664),
    [sym_block] = STATE(1664),
    [sym__literal] = STATE(1664),
    [sym_string_literal] = STATE(1786),
    [sym_raw_string_literal] = STATE(1786),
    [sym_boolean_literal] = STATE(1786),
    [sym_line_comment] = STATE(248),
    [sym_block_comment] = STATE(248),
    [sym_identifier] = ACTIONS(405),
    [anon_sym_LPAREN] = ACTIONS(495),
    [anon_sym_LBRACK] = ACTIONS(497),
    [anon_sym_LBRACE] = ACTIONS(407),
    [anon_sym_STAR] = ACTIONS(1015),
    [anon_sym_u8] = ACTIONS(411),
    [anon_sym_i8] = ACTIONS(411),
    [anon_sym_u16] = ACTIONS(411),
    [anon_sym_i16] = ACTIONS(411),
    [anon_sym_u32] = ACTIONS(411),
    [anon_sym_i32] = ACTIONS(411),
    [anon_sym_u64] = ACTIONS(411),
    [anon_sym_i64] = ACTIONS(411),
    [anon_sym_u128] = ACTIONS(411),
    [anon_sym_i128] = ACTIONS(411),
    [anon_sym_isize] = ACTIONS(411),
    [anon_sym_usize] = ACTIONS(411),
    [anon_sym_f32] = ACTIONS(411),
    [anon_sym_f64] = ACTIONS(411),
    [anon_sym_bool] = ACTIONS(411),
    [anon_sym_str] = ACTIONS(411),
    [anon_sym_char] = ACTIONS(411),
    [anon_sym_DASH] = ACTIONS(1015),
    [anon_sym_BANG] = ACTIONS(1015),
    [anon_sym_AMP] = ACTIONS(1017),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1059),
    [anon_sym_COLON_COLON] = ACTIONS(415),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(417),
    [anon_sym_break] = ACTIONS(419),
    [anon_sym_const] = ACTIONS(421),
    [anon_sym_continue] = ACTIONS(423),
    [anon_sym_default] = ACTIONS(425),
    [anon_sym_for] = ACTIONS(427),
    [anon_sym_gen] = ACTIONS(429),
    [anon_sym_if] = ACTIONS(431),
    [anon_sym_loop] = ACTIONS(433),
    [anon_sym_match] = ACTIONS(435),
    [anon_sym_return] = ACTIONS(437),
    [anon_sym_static] = ACTIONS(439),
    [anon_sym_union] = ACTIONS(425),
    [anon_sym_unsafe] = ACTIONS(441),
    [anon_sym_while] = ACTIONS(443),
    [anon_sym_yield] = ACTIONS(445),
    [anon_sym_move] = ACTIONS(447),
    [anon_sym_try] = ACTIONS(449),
    [sym_integer_literal] = ACTIONS(451),
    [aux_sym_string_literal_token1] = ACTIONS(453),
    [sym_char_literal] = ACTIONS(451),
    [anon_sym_true] = ACTIONS(455),
    [anon_sym_false] = ACTIONS(455),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(457),
    [sym_super] = ACTIONS(459),
    [sym_crate] = ACTIONS(459),
    [sym_metavariable] = ACTIONS(461),
    [sym__raw_string_literal_start] = ACTIONS(463),
    [sym_float_literal] = ACTIONS(451),
  },
  [249] = {
    [sym_bracketed_type] = STATE(3529),
    [sym_generic_function] = STATE(1664),
    [sym_generic_type_with_turbofish] = STATE(2956),
    [sym__expression_except_range] = STATE(1626),
    [sym__expression] = STATE(1763),
    [sym_macro_invocation] = STATE(1671),
    [sym_scoped_identifier] = STATE(1575),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3163),
    [sym_range_expression] = STATE(1666),
    [sym_unary_expression] = STATE(1664),
    [sym_try_expression] = STATE(1664),
    [sym_reference_expression] = STATE(1664),
    [sym_binary_expression] = STATE(1664),
    [sym_assignment_expression] = STATE(1664),
    [sym_compound_assignment_expr] = STATE(1664),
    [sym_type_cast_expression] = STATE(1664),
    [sym_return_expression] = STATE(1664),
    [sym_yield_expression] = STATE(1664),
    [sym_call_expression] = STATE(1664),
    [sym_array_expression] = STATE(1664),
    [sym_parenthesized_expression] = STATE(1664),
    [sym_tuple_expression] = STATE(1664),
    [sym_unit_expression] = STATE(1664),
    [sym_struct_expression] = STATE(1664),
    [sym_if_expression] = STATE(1664),
    [sym_match_expression] = STATE(1664),
    [sym_while_expression] = STATE(1664),
    [sym_loop_expression] = STATE(1664),
    [sym_for_expression] = STATE(1664),
    [sym_const_block] = STATE(1664),
    [sym_closure_expression] = STATE(1664),
    [sym_closure_parameters] = STATE(232),
    [sym_label] = STATE(3614),
    [sym_break_expression] = STATE(1664),
    [sym_continue_expression] = STATE(1664),
    [sym_index_expression] = STATE(1664),
    [sym_await_expression] = STATE(1664),
    [sym_field_expression] = STATE(1597),
    [sym_unsafe_block] = STATE(1664),
    [sym_async_block] = STATE(1664),
    [sym_gen_block] = STATE(1664),
    [sym_try_block] = STATE(1664),
    [sym_block] = STATE(1664),
    [sym__literal] = STATE(1664),
    [sym_string_literal] = STATE(1786),
    [sym_raw_string_literal] = STATE(1786),
    [sym_boolean_literal] = STATE(1786),
    [sym_line_comment] = STATE(249),
    [sym_block_comment] = STATE(249),
    [sym_identifier] = ACTIONS(405),
    [anon_sym_LPAREN] = ACTIONS(495),
    [anon_sym_LBRACK] = ACTIONS(497),
    [anon_sym_LBRACE] = ACTIONS(407),
    [anon_sym_STAR] = ACTIONS(1015),
    [anon_sym_u8] = ACTIONS(411),
    [anon_sym_i8] = ACTIONS(411),
    [anon_sym_u16] = ACTIONS(411),
    [anon_sym_i16] = ACTIONS(411),
    [anon_sym_u32] = ACTIONS(411),
    [anon_sym_i32] = ACTIONS(411),
    [anon_sym_u64] = ACTIONS(411),
    [anon_sym_i64] = ACTIONS(411),
    [anon_sym_u128] = ACTIONS(411),
    [anon_sym_i128] = ACTIONS(411),
    [anon_sym_isize] = ACTIONS(411),
    [anon_sym_usize] = ACTIONS(411),
    [anon_sym_f32] = ACTIONS(411),
    [anon_sym_f64] = ACTIONS(411),
    [anon_sym_bool] = ACTIONS(411),
    [anon_sym_str] = ACTIONS(411),
    [anon_sym_char] = ACTIONS(411),
    [anon_sym_DASH] = ACTIONS(1015),
    [anon_sym_BANG] = ACTIONS(1015),
    [anon_sym_AMP] = ACTIONS(1017),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1059),
    [anon_sym_COLON_COLON] = ACTIONS(415),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(417),
    [anon_sym_break] = ACTIONS(419),
    [anon_sym_const] = ACTIONS(421),
    [anon_sym_continue] = ACTIONS(423),
    [anon_sym_default] = ACTIONS(425),
    [anon_sym_for] = ACTIONS(427),
    [anon_sym_gen] = ACTIONS(429),
    [anon_sym_if] = ACTIONS(431),
    [anon_sym_loop] = ACTIONS(433),
    [anon_sym_match] = ACTIONS(435),
    [anon_sym_return] = ACTIONS(437),
    [anon_sym_static] = ACTIONS(439),
    [anon_sym_union] = ACTIONS(425),
    [anon_sym_unsafe] = ACTIONS(441),
    [anon_sym_while] = ACTIONS(443),
    [anon_sym_yield] = ACTIONS(445),
    [anon_sym_move] = ACTIONS(447),
    [anon_sym_try] = ACTIONS(449),
    [sym_integer_literal] = ACTIONS(451),
    [aux_sym_string_literal_token1] = ACTIONS(453),
    [sym_char_literal] = ACTIONS(451),
    [anon_sym_true] = ACTIONS(455),
    [anon_sym_false] = ACTIONS(455),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(457),
    [sym_super] = ACTIONS(459),
    [sym_crate] = ACTIONS(459),
    [sym_metavariable] = ACTIONS(461),
    [sym__raw_string_literal_start] = ACTIONS(463),
    [sym_float_literal] = ACTIONS(451),
  },
  [250] = {
    [sym_bracketed_type] = STATE(3529),
    [sym_generic_function] = STATE(1664),
    [sym_generic_type_with_turbofish] = STATE(2956),
    [sym__expression_except_range] = STATE(1626),
    [sym__expression] = STATE(1764),
    [sym_macro_invocation] = STATE(1671),
    [sym_scoped_identifier] = STATE(1575),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3163),
    [sym_range_expression] = STATE(1666),
    [sym_unary_expression] = STATE(1664),
    [sym_try_expression] = STATE(1664),
    [sym_reference_expression] = STATE(1664),
    [sym_binary_expression] = STATE(1664),
    [sym_assignment_expression] = STATE(1664),
    [sym_compound_assignment_expr] = STATE(1664),
    [sym_type_cast_expression] = STATE(1664),
    [sym_return_expression] = STATE(1664),
    [sym_yield_expression] = STATE(1664),
    [sym_call_expression] = STATE(1664),
    [sym_array_expression] = STATE(1664),
    [sym_parenthesized_expression] = STATE(1664),
    [sym_tuple_expression] = STATE(1664),
    [sym_unit_expression] = STATE(1664),
    [sym_struct_expression] = STATE(1664),
    [sym_if_expression] = STATE(1664),
    [sym_match_expression] = STATE(1664),
    [sym_while_expression] = STATE(1664),
    [sym_loop_expression] = STATE(1664),
    [sym_for_expression] = STATE(1664),
    [sym_const_block] = STATE(1664),
    [sym_closure_expression] = STATE(1664),
    [sym_closure_parameters] = STATE(232),
    [sym_label] = STATE(3614),
    [sym_break_expression] = STATE(1664),
    [sym_continue_expression] = STATE(1664),
    [sym_index_expression] = STATE(1664),
    [sym_await_expression] = STATE(1664),
    [sym_field_expression] = STATE(1597),
    [sym_unsafe_block] = STATE(1664),
    [sym_async_block] = STATE(1664),
    [sym_gen_block] = STATE(1664),
    [sym_try_block] = STATE(1664),
    [sym_block] = STATE(1664),
    [sym__literal] = STATE(1664),
    [sym_string_literal] = STATE(1786),
    [sym_raw_string_literal] = STATE(1786),
    [sym_boolean_literal] = STATE(1786),
    [sym_line_comment] = STATE(250),
    [sym_block_comment] = STATE(250),
    [sym_identifier] = ACTIONS(405),
    [anon_sym_LPAREN] = ACTIONS(495),
    [anon_sym_LBRACK] = ACTIONS(497),
    [anon_sym_LBRACE] = ACTIONS(407),
    [anon_sym_STAR] = ACTIONS(1015),
    [anon_sym_u8] = ACTIONS(411),
    [anon_sym_i8] = ACTIONS(411),
    [anon_sym_u16] = ACTIONS(411),
    [anon_sym_i16] = ACTIONS(411),
    [anon_sym_u32] = ACTIONS(411),
    [anon_sym_i32] = ACTIONS(411),
    [anon_sym_u64] = ACTIONS(411),
    [anon_sym_i64] = ACTIONS(411),
    [anon_sym_u128] = ACTIONS(411),
    [anon_sym_i128] = ACTIONS(411),
    [anon_sym_isize] = ACTIONS(411),
    [anon_sym_usize] = ACTIONS(411),
    [anon_sym_f32] = ACTIONS(411),
    [anon_sym_f64] = ACTIONS(411),
    [anon_sym_bool] = ACTIONS(411),
    [anon_sym_str] = ACTIONS(411),
    [anon_sym_char] = ACTIONS(411),
    [anon_sym_DASH] = ACTIONS(1015),
    [anon_sym_BANG] = ACTIONS(1015),
    [anon_sym_AMP] = ACTIONS(1017),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1059),
    [anon_sym_COLON_COLON] = ACTIONS(415),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(417),
    [anon_sym_break] = ACTIONS(419),
    [anon_sym_const] = ACTIONS(421),
    [anon_sym_continue] = ACTIONS(423),
    [anon_sym_default] = ACTIONS(425),
    [anon_sym_for] = ACTIONS(427),
    [anon_sym_gen] = ACTIONS(429),
    [anon_sym_if] = ACTIONS(431),
    [anon_sym_loop] = ACTIONS(433),
    [anon_sym_match] = ACTIONS(435),
    [anon_sym_return] = ACTIONS(437),
    [anon_sym_static] = ACTIONS(439),
    [anon_sym_union] = ACTIONS(425),
    [anon_sym_unsafe] = ACTIONS(441),
    [anon_sym_while] = ACTIONS(443),
    [anon_sym_yield] = ACTIONS(445),
    [anon_sym_move] = ACTIONS(447),
    [anon_sym_try] = ACTIONS(449),
    [sym_integer_literal] = ACTIONS(451),
    [aux_sym_string_literal_token1] = ACTIONS(453),
    [sym_char_literal] = ACTIONS(451),
    [anon_sym_true] = ACTIONS(455),
    [anon_sym_false] = ACTIONS(455),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(457),
    [sym_super] = ACTIONS(459),
    [sym_crate] = ACTIONS(459),
    [sym_metavariable] = ACTIONS(461),
    [sym__raw_string_literal_start] = ACTIONS(463),
    [sym_float_literal] = ACTIONS(451),
  },
  [251] = {
    [sym_bracketed_type] = STATE(3529),
    [sym_generic_function] = STATE(1664),
    [sym_generic_type_with_turbofish] = STATE(2956),
    [sym__expression_except_range] = STATE(1626),
    [sym__expression] = STATE(1765),
    [sym_macro_invocation] = STATE(1671),
    [sym_scoped_identifier] = STATE(1575),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3163),
    [sym_range_expression] = STATE(1666),
    [sym_unary_expression] = STATE(1664),
    [sym_try_expression] = STATE(1664),
    [sym_reference_expression] = STATE(1664),
    [sym_binary_expression] = STATE(1664),
    [sym_assignment_expression] = STATE(1664),
    [sym_compound_assignment_expr] = STATE(1664),
    [sym_type_cast_expression] = STATE(1664),
    [sym_return_expression] = STATE(1664),
    [sym_yield_expression] = STATE(1664),
    [sym_call_expression] = STATE(1664),
    [sym_array_expression] = STATE(1664),
    [sym_parenthesized_expression] = STATE(1664),
    [sym_tuple_expression] = STATE(1664),
    [sym_unit_expression] = STATE(1664),
    [sym_struct_expression] = STATE(1664),
    [sym_if_expression] = STATE(1664),
    [sym_match_expression] = STATE(1664),
    [sym_while_expression] = STATE(1664),
    [sym_loop_expression] = STATE(1664),
    [sym_for_expression] = STATE(1664),
    [sym_const_block] = STATE(1664),
    [sym_closure_expression] = STATE(1664),
    [sym_closure_parameters] = STATE(232),
    [sym_label] = STATE(3614),
    [sym_break_expression] = STATE(1664),
    [sym_continue_expression] = STATE(1664),
    [sym_index_expression] = STATE(1664),
    [sym_await_expression] = STATE(1664),
    [sym_field_expression] = STATE(1597),
    [sym_unsafe_block] = STATE(1664),
    [sym_async_block] = STATE(1664),
    [sym_gen_block] = STATE(1664),
    [sym_try_block] = STATE(1664),
    [sym_block] = STATE(1664),
    [sym__literal] = STATE(1664),
    [sym_string_literal] = STATE(1786),
    [sym_raw_string_literal] = STATE(1786),
    [sym_boolean_literal] = STATE(1786),
    [sym_line_comment] = STATE(251),
    [sym_block_comment] = STATE(251),
    [sym_identifier] = ACTIONS(405),
    [anon_sym_LPAREN] = ACTIONS(495),
    [anon_sym_LBRACK] = ACTIONS(497),
    [anon_sym_LBRACE] = ACTIONS(407),
    [anon_sym_STAR] = ACTIONS(1015),
    [anon_sym_u8] = ACTIONS(411),
    [anon_sym_i8] = ACTIONS(411),
    [anon_sym_u16] = ACTIONS(411),
    [anon_sym_i16] = ACTIONS(411),
    [anon_sym_u32] = ACTIONS(411),
    [anon_sym_i32] = ACTIONS(411),
    [anon_sym_u64] = ACTIONS(411),
    [anon_sym_i64] = ACTIONS(411),
    [anon_sym_u128] = ACTIONS(411),
    [anon_sym_i128] = ACTIONS(411),
    [anon_sym_isize] = ACTIONS(411),
    [anon_sym_usize] = ACTIONS(411),
    [anon_sym_f32] = ACTIONS(411),
    [anon_sym_f64] = ACTIONS(411),
    [anon_sym_bool] = ACTIONS(411),
    [anon_sym_str] = ACTIONS(411),
    [anon_sym_char] = ACTIONS(411),
    [anon_sym_DASH] = ACTIONS(1015),
    [anon_sym_BANG] = ACTIONS(1015),
    [anon_sym_AMP] = ACTIONS(1017),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1059),
    [anon_sym_COLON_COLON] = ACTIONS(415),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(417),
    [anon_sym_break] = ACTIONS(419),
    [anon_sym_const] = ACTIONS(421),
    [anon_sym_continue] = ACTIONS(423),
    [anon_sym_default] = ACTIONS(425),
    [anon_sym_for] = ACTIONS(427),
    [anon_sym_gen] = ACTIONS(429),
    [anon_sym_if] = ACTIONS(431),
    [anon_sym_loop] = ACTIONS(433),
    [anon_sym_match] = ACTIONS(435),
    [anon_sym_return] = ACTIONS(437),
    [anon_sym_static] = ACTIONS(439),
    [anon_sym_union] = ACTIONS(425),
    [anon_sym_unsafe] = ACTIONS(441),
    [anon_sym_while] = ACTIONS(443),
    [anon_sym_yield] = ACTIONS(445),
    [anon_sym_move] = ACTIONS(447),
    [anon_sym_try] = ACTIONS(449),
    [sym_integer_literal] = ACTIONS(451),
    [aux_sym_string_literal_token1] = ACTIONS(453),
    [sym_char_literal] = ACTIONS(451),
    [anon_sym_true] = ACTIONS(455),
    [anon_sym_false] = ACTIONS(455),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(457),
    [sym_super] = ACTIONS(459),
    [sym_crate] = ACTIONS(459),
    [sym_metavariable] = ACTIONS(461),
    [sym__raw_string_literal_start] = ACTIONS(463),
    [sym_float_literal] = ACTIONS(451),
  },
  [252] = {
    [sym_bracketed_type] = STATE(3529),
    [sym_generic_function] = STATE(1664),
    [sym_generic_type_with_turbofish] = STATE(2956),
    [sym__expression_except_range] = STATE(1626),
    [sym__expression] = STATE(1766),
    [sym_macro_invocation] = STATE(1671),
    [sym_scoped_identifier] = STATE(1575),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3163),
    [sym_range_expression] = STATE(1666),
    [sym_unary_expression] = STATE(1664),
    [sym_try_expression] = STATE(1664),
    [sym_reference_expression] = STATE(1664),
    [sym_binary_expression] = STATE(1664),
    [sym_assignment_expression] = STATE(1664),
    [sym_compound_assignment_expr] = STATE(1664),
    [sym_type_cast_expression] = STATE(1664),
    [sym_return_expression] = STATE(1664),
    [sym_yield_expression] = STATE(1664),
    [sym_call_expression] = STATE(1664),
    [sym_array_expression] = STATE(1664),
    [sym_parenthesized_expression] = STATE(1664),
    [sym_tuple_expression] = STATE(1664),
    [sym_unit_expression] = STATE(1664),
    [sym_struct_expression] = STATE(1664),
    [sym_if_expression] = STATE(1664),
    [sym_match_expression] = STATE(1664),
    [sym_while_expression] = STATE(1664),
    [sym_loop_expression] = STATE(1664),
    [sym_for_expression] = STATE(1664),
    [sym_const_block] = STATE(1664),
    [sym_closure_expression] = STATE(1664),
    [sym_closure_parameters] = STATE(232),
    [sym_label] = STATE(3614),
    [sym_break_expression] = STATE(1664),
    [sym_continue_expression] = STATE(1664),
    [sym_index_expression] = STATE(1664),
    [sym_await_expression] = STATE(1664),
    [sym_field_expression] = STATE(1597),
    [sym_unsafe_block] = STATE(1664),
    [sym_async_block] = STATE(1664),
    [sym_gen_block] = STATE(1664),
    [sym_try_block] = STATE(1664),
    [sym_block] = STATE(1664),
    [sym__literal] = STATE(1664),
    [sym_string_literal] = STATE(1786),
    [sym_raw_string_literal] = STATE(1786),
    [sym_boolean_literal] = STATE(1786),
    [sym_line_comment] = STATE(252),
    [sym_block_comment] = STATE(252),
    [sym_identifier] = ACTIONS(405),
    [anon_sym_LPAREN] = ACTIONS(495),
    [anon_sym_LBRACK] = ACTIONS(497),
    [anon_sym_LBRACE] = ACTIONS(407),
    [anon_sym_STAR] = ACTIONS(1015),
    [anon_sym_u8] = ACTIONS(411),
    [anon_sym_i8] = ACTIONS(411),
    [anon_sym_u16] = ACTIONS(411),
    [anon_sym_i16] = ACTIONS(411),
    [anon_sym_u32] = ACTIONS(411),
    [anon_sym_i32] = ACTIONS(411),
    [anon_sym_u64] = ACTIONS(411),
    [anon_sym_i64] = ACTIONS(411),
    [anon_sym_u128] = ACTIONS(411),
    [anon_sym_i128] = ACTIONS(411),
    [anon_sym_isize] = ACTIONS(411),
    [anon_sym_usize] = ACTIONS(411),
    [anon_sym_f32] = ACTIONS(411),
    [anon_sym_f64] = ACTIONS(411),
    [anon_sym_bool] = ACTIONS(411),
    [anon_sym_str] = ACTIONS(411),
    [anon_sym_char] = ACTIONS(411),
    [anon_sym_DASH] = ACTIONS(1015),
    [anon_sym_BANG] = ACTIONS(1015),
    [anon_sym_AMP] = ACTIONS(1017),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1059),
    [anon_sym_COLON_COLON] = ACTIONS(415),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(417),
    [anon_sym_break] = ACTIONS(419),
    [anon_sym_const] = ACTIONS(421),
    [anon_sym_continue] = ACTIONS(423),
    [anon_sym_default] = ACTIONS(425),
    [anon_sym_for] = ACTIONS(427),
    [anon_sym_gen] = ACTIONS(429),
    [anon_sym_if] = ACTIONS(431),
    [anon_sym_loop] = ACTIONS(433),
    [anon_sym_match] = ACTIONS(435),
    [anon_sym_return] = ACTIONS(437),
    [anon_sym_static] = ACTIONS(439),
    [anon_sym_union] = ACTIONS(425),
    [anon_sym_unsafe] = ACTIONS(441),
    [anon_sym_while] = ACTIONS(443),
    [anon_sym_yield] = ACTIONS(445),
    [anon_sym_move] = ACTIONS(447),
    [anon_sym_try] = ACTIONS(449),
    [sym_integer_literal] = ACTIONS(451),
    [aux_sym_string_literal_token1] = ACTIONS(453),
    [sym_char_literal] = ACTIONS(451),
    [anon_sym_true] = ACTIONS(455),
    [anon_sym_false] = ACTIONS(455),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(457),
    [sym_super] = ACTIONS(459),
    [sym_crate] = ACTIONS(459),
    [sym_metavariable] = ACTIONS(461),
    [sym__raw_string_literal_start] = ACTIONS(463),
    [sym_float_literal] = ACTIONS(451),
  },
  [253] = {
    [sym_bracketed_type] = STATE(3529),
    [sym_generic_function] = STATE(1664),
    [sym_generic_type_with_turbofish] = STATE(2956),
    [sym__expression_except_range] = STATE(1626),
    [sym__expression] = STATE(1767),
    [sym_macro_invocation] = STATE(1671),
    [sym_scoped_identifier] = STATE(1575),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3163),
    [sym_range_expression] = STATE(1666),
    [sym_unary_expression] = STATE(1664),
    [sym_try_expression] = STATE(1664),
    [sym_reference_expression] = STATE(1664),
    [sym_binary_expression] = STATE(1664),
    [sym_assignment_expression] = STATE(1664),
    [sym_compound_assignment_expr] = STATE(1664),
    [sym_type_cast_expression] = STATE(1664),
    [sym_return_expression] = STATE(1664),
    [sym_yield_expression] = STATE(1664),
    [sym_call_expression] = STATE(1664),
    [sym_array_expression] = STATE(1664),
    [sym_parenthesized_expression] = STATE(1664),
    [sym_tuple_expression] = STATE(1664),
    [sym_unit_expression] = STATE(1664),
    [sym_struct_expression] = STATE(1664),
    [sym_if_expression] = STATE(1664),
    [sym_match_expression] = STATE(1664),
    [sym_while_expression] = STATE(1664),
    [sym_loop_expression] = STATE(1664),
    [sym_for_expression] = STATE(1664),
    [sym_const_block] = STATE(1664),
    [sym_closure_expression] = STATE(1664),
    [sym_closure_parameters] = STATE(232),
    [sym_label] = STATE(3614),
    [sym_break_expression] = STATE(1664),
    [sym_continue_expression] = STATE(1664),
    [sym_index_expression] = STATE(1664),
    [sym_await_expression] = STATE(1664),
    [sym_field_expression] = STATE(1597),
    [sym_unsafe_block] = STATE(1664),
    [sym_async_block] = STATE(1664),
    [sym_gen_block] = STATE(1664),
    [sym_try_block] = STATE(1664),
    [sym_block] = STATE(1664),
    [sym__literal] = STATE(1664),
    [sym_string_literal] = STATE(1786),
    [sym_raw_string_literal] = STATE(1786),
    [sym_boolean_literal] = STATE(1786),
    [sym_line_comment] = STATE(253),
    [sym_block_comment] = STATE(253),
    [sym_identifier] = ACTIONS(405),
    [anon_sym_LPAREN] = ACTIONS(495),
    [anon_sym_LBRACK] = ACTIONS(497),
    [anon_sym_LBRACE] = ACTIONS(407),
    [anon_sym_STAR] = ACTIONS(1015),
    [anon_sym_u8] = ACTIONS(411),
    [anon_sym_i8] = ACTIONS(411),
    [anon_sym_u16] = ACTIONS(411),
    [anon_sym_i16] = ACTIONS(411),
    [anon_sym_u32] = ACTIONS(411),
    [anon_sym_i32] = ACTIONS(411),
    [anon_sym_u64] = ACTIONS(411),
    [anon_sym_i64] = ACTIONS(411),
    [anon_sym_u128] = ACTIONS(411),
    [anon_sym_i128] = ACTIONS(411),
    [anon_sym_isize] = ACTIONS(411),
    [anon_sym_usize] = ACTIONS(411),
    [anon_sym_f32] = ACTIONS(411),
    [anon_sym_f64] = ACTIONS(411),
    [anon_sym_bool] = ACTIONS(411),
    [anon_sym_str] = ACTIONS(411),
    [anon_sym_char] = ACTIONS(411),
    [anon_sym_DASH] = ACTIONS(1015),
    [anon_sym_BANG] = ACTIONS(1015),
    [anon_sym_AMP] = ACTIONS(1017),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1059),
    [anon_sym_COLON_COLON] = ACTIONS(415),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(417),
    [anon_sym_break] = ACTIONS(419),
    [anon_sym_const] = ACTIONS(421),
    [anon_sym_continue] = ACTIONS(423),
    [anon_sym_default] = ACTIONS(425),
    [anon_sym_for] = ACTIONS(427),
    [anon_sym_gen] = ACTIONS(429),
    [anon_sym_if] = ACTIONS(431),
    [anon_sym_loop] = ACTIONS(433),
    [anon_sym_match] = ACTIONS(435),
    [anon_sym_return] = ACTIONS(437),
    [anon_sym_static] = ACTIONS(439),
    [anon_sym_union] = ACTIONS(425),
    [anon_sym_unsafe] = ACTIONS(441),
    [anon_sym_while] = ACTIONS(443),
    [anon_sym_yield] = ACTIONS(445),
    [anon_sym_move] = ACTIONS(447),
    [anon_sym_try] = ACTIONS(449),
    [sym_integer_literal] = ACTIONS(451),
    [aux_sym_string_literal_token1] = ACTIONS(453),
    [sym_char_literal] = ACTIONS(451),
    [anon_sym_true] = ACTIONS(455),
    [anon_sym_false] = ACTIONS(455),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(457),
    [sym_super] = ACTIONS(459),
    [sym_crate] = ACTIONS(459),
    [sym_metavariable] = ACTIONS(461),
    [sym__raw_string_literal_start] = ACTIONS(463),
    [sym_float_literal] = ACTIONS(451),
  },
  [254] = {
    [sym_bracketed_type] = STATE(3529),
    [sym_generic_function] = STATE(1664),
    [sym_generic_type_with_turbofish] = STATE(2956),
    [sym__expression_except_range] = STATE(1626),
    [sym__expression] = STATE(1768),
    [sym_macro_invocation] = STATE(1671),
    [sym_scoped_identifier] = STATE(1575),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3163),
    [sym_range_expression] = STATE(1666),
    [sym_unary_expression] = STATE(1664),
    [sym_try_expression] = STATE(1664),
    [sym_reference_expression] = STATE(1664),
    [sym_binary_expression] = STATE(1664),
    [sym_assignment_expression] = STATE(1664),
    [sym_compound_assignment_expr] = STATE(1664),
    [sym_type_cast_expression] = STATE(1664),
    [sym_return_expression] = STATE(1664),
    [sym_yield_expression] = STATE(1664),
    [sym_call_expression] = STATE(1664),
    [sym_array_expression] = STATE(1664),
    [sym_parenthesized_expression] = STATE(1664),
    [sym_tuple_expression] = STATE(1664),
    [sym_unit_expression] = STATE(1664),
    [sym_struct_expression] = STATE(1664),
    [sym_if_expression] = STATE(1664),
    [sym_match_expression] = STATE(1664),
    [sym_while_expression] = STATE(1664),
    [sym_loop_expression] = STATE(1664),
    [sym_for_expression] = STATE(1664),
    [sym_const_block] = STATE(1664),
    [sym_closure_expression] = STATE(1664),
    [sym_closure_parameters] = STATE(232),
    [sym_label] = STATE(3614),
    [sym_break_expression] = STATE(1664),
    [sym_continue_expression] = STATE(1664),
    [sym_index_expression] = STATE(1664),
    [sym_await_expression] = STATE(1664),
    [sym_field_expression] = STATE(1597),
    [sym_unsafe_block] = STATE(1664),
    [sym_async_block] = STATE(1664),
    [sym_gen_block] = STATE(1664),
    [sym_try_block] = STATE(1664),
    [sym_block] = STATE(1664),
    [sym__literal] = STATE(1664),
    [sym_string_literal] = STATE(1786),
    [sym_raw_string_literal] = STATE(1786),
    [sym_boolean_literal] = STATE(1786),
    [sym_line_comment] = STATE(254),
    [sym_block_comment] = STATE(254),
    [sym_identifier] = ACTIONS(405),
    [anon_sym_LPAREN] = ACTIONS(495),
    [anon_sym_LBRACK] = ACTIONS(497),
    [anon_sym_LBRACE] = ACTIONS(407),
    [anon_sym_STAR] = ACTIONS(1015),
    [anon_sym_u8] = ACTIONS(411),
    [anon_sym_i8] = ACTIONS(411),
    [anon_sym_u16] = ACTIONS(411),
    [anon_sym_i16] = ACTIONS(411),
    [anon_sym_u32] = ACTIONS(411),
    [anon_sym_i32] = ACTIONS(411),
    [anon_sym_u64] = ACTIONS(411),
    [anon_sym_i64] = ACTIONS(411),
    [anon_sym_u128] = ACTIONS(411),
    [anon_sym_i128] = ACTIONS(411),
    [anon_sym_isize] = ACTIONS(411),
    [anon_sym_usize] = ACTIONS(411),
    [anon_sym_f32] = ACTIONS(411),
    [anon_sym_f64] = ACTIONS(411),
    [anon_sym_bool] = ACTIONS(411),
    [anon_sym_str] = ACTIONS(411),
    [anon_sym_char] = ACTIONS(411),
    [anon_sym_DASH] = ACTIONS(1015),
    [anon_sym_BANG] = ACTIONS(1015),
    [anon_sym_AMP] = ACTIONS(1017),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1059),
    [anon_sym_COLON_COLON] = ACTIONS(415),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(417),
    [anon_sym_break] = ACTIONS(419),
    [anon_sym_const] = ACTIONS(421),
    [anon_sym_continue] = ACTIONS(423),
    [anon_sym_default] = ACTIONS(425),
    [anon_sym_for] = ACTIONS(427),
    [anon_sym_gen] = ACTIONS(429),
    [anon_sym_if] = ACTIONS(431),
    [anon_sym_loop] = ACTIONS(433),
    [anon_sym_match] = ACTIONS(435),
    [anon_sym_return] = ACTIONS(437),
    [anon_sym_static] = ACTIONS(439),
    [anon_sym_union] = ACTIONS(425),
    [anon_sym_unsafe] = ACTIONS(441),
    [anon_sym_while] = ACTIONS(443),
    [anon_sym_yield] = ACTIONS(445),
    [anon_sym_move] = ACTIONS(447),
    [anon_sym_try] = ACTIONS(449),
    [sym_integer_literal] = ACTIONS(451),
    [aux_sym_string_literal_token1] = ACTIONS(453),
    [sym_char_literal] = ACTIONS(451),
    [anon_sym_true] = ACTIONS(455),
    [anon_sym_false] = ACTIONS(455),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(457),
    [sym_super] = ACTIONS(459),
    [sym_crate] = ACTIONS(459),
    [sym_metavariable] = ACTIONS(461),
    [sym__raw_string_literal_start] = ACTIONS(463),
    [sym_float_literal] = ACTIONS(451),
  },
  [255] = {
    [sym_bracketed_type] = STATE(3529),
    [sym_generic_function] = STATE(1664),
    [sym_generic_type_with_turbofish] = STATE(2956),
    [sym__expression_except_range] = STATE(1626),
    [sym__expression] = STATE(1769),
    [sym_macro_invocation] = STATE(1671),
    [sym_scoped_identifier] = STATE(1575),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3163),
    [sym_range_expression] = STATE(1666),
    [sym_unary_expression] = STATE(1664),
    [sym_try_expression] = STATE(1664),
    [sym_reference_expression] = STATE(1664),
    [sym_binary_expression] = STATE(1664),
    [sym_assignment_expression] = STATE(1664),
    [sym_compound_assignment_expr] = STATE(1664),
    [sym_type_cast_expression] = STATE(1664),
    [sym_return_expression] = STATE(1664),
    [sym_yield_expression] = STATE(1664),
    [sym_call_expression] = STATE(1664),
    [sym_array_expression] = STATE(1664),
    [sym_parenthesized_expression] = STATE(1664),
    [sym_tuple_expression] = STATE(1664),
    [sym_unit_expression] = STATE(1664),
    [sym_struct_expression] = STATE(1664),
    [sym_if_expression] = STATE(1664),
    [sym_match_expression] = STATE(1664),
    [sym_while_expression] = STATE(1664),
    [sym_loop_expression] = STATE(1664),
    [sym_for_expression] = STATE(1664),
    [sym_const_block] = STATE(1664),
    [sym_closure_expression] = STATE(1664),
    [sym_closure_parameters] = STATE(232),
    [sym_label] = STATE(3614),
    [sym_break_expression] = STATE(1664),
    [sym_continue_expression] = STATE(1664),
    [sym_index_expression] = STATE(1664),
    [sym_await_expression] = STATE(1664),
    [sym_field_expression] = STATE(1597),
    [sym_unsafe_block] = STATE(1664),
    [sym_async_block] = STATE(1664),
    [sym_gen_block] = STATE(1664),
    [sym_try_block] = STATE(1664),
    [sym_block] = STATE(1664),
    [sym__literal] = STATE(1664),
    [sym_string_literal] = STATE(1786),
    [sym_raw_string_literal] = STATE(1786),
    [sym_boolean_literal] = STATE(1786),
    [sym_line_comment] = STATE(255),
    [sym_block_comment] = STATE(255),
    [sym_identifier] = ACTIONS(405),
    [anon_sym_LPAREN] = ACTIONS(495),
    [anon_sym_LBRACK] = ACTIONS(497),
    [anon_sym_LBRACE] = ACTIONS(407),
    [anon_sym_STAR] = ACTIONS(1015),
    [anon_sym_u8] = ACTIONS(411),
    [anon_sym_i8] = ACTIONS(411),
    [anon_sym_u16] = ACTIONS(411),
    [anon_sym_i16] = ACTIONS(411),
    [anon_sym_u32] = ACTIONS(411),
    [anon_sym_i32] = ACTIONS(411),
    [anon_sym_u64] = ACTIONS(411),
    [anon_sym_i64] = ACTIONS(411),
    [anon_sym_u128] = ACTIONS(411),
    [anon_sym_i128] = ACTIONS(411),
    [anon_sym_isize] = ACTIONS(411),
    [anon_sym_usize] = ACTIONS(411),
    [anon_sym_f32] = ACTIONS(411),
    [anon_sym_f64] = ACTIONS(411),
    [anon_sym_bool] = ACTIONS(411),
    [anon_sym_str] = ACTIONS(411),
    [anon_sym_char] = ACTIONS(411),
    [anon_sym_DASH] = ACTIONS(1015),
    [anon_sym_BANG] = ACTIONS(1015),
    [anon_sym_AMP] = ACTIONS(1017),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1059),
    [anon_sym_COLON_COLON] = ACTIONS(415),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(417),
    [anon_sym_break] = ACTIONS(419),
    [anon_sym_const] = ACTIONS(421),
    [anon_sym_continue] = ACTIONS(423),
    [anon_sym_default] = ACTIONS(425),
    [anon_sym_for] = ACTIONS(427),
    [anon_sym_gen] = ACTIONS(429),
    [anon_sym_if] = ACTIONS(431),
    [anon_sym_loop] = ACTIONS(433),
    [anon_sym_match] = ACTIONS(435),
    [anon_sym_return] = ACTIONS(437),
    [anon_sym_static] = ACTIONS(439),
    [anon_sym_union] = ACTIONS(425),
    [anon_sym_unsafe] = ACTIONS(441),
    [anon_sym_while] = ACTIONS(443),
    [anon_sym_yield] = ACTIONS(445),
    [anon_sym_move] = ACTIONS(447),
    [anon_sym_try] = ACTIONS(449),
    [sym_integer_literal] = ACTIONS(451),
    [aux_sym_string_literal_token1] = ACTIONS(453),
    [sym_char_literal] = ACTIONS(451),
    [anon_sym_true] = ACTIONS(455),
    [anon_sym_false] = ACTIONS(455),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(457),
    [sym_super] = ACTIONS(459),
    [sym_crate] = ACTIONS(459),
    [sym_metavariable] = ACTIONS(461),
    [sym__raw_string_literal_start] = ACTIONS(463),
    [sym_float_literal] = ACTIONS(451),
  },
  [256] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1718),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(246),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(256),
    [sym_block_comment] = STATE(256),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(1067),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(1067),
    [anon_sym_BANG] = ACTIONS(1067),
    [anon_sym_AMP] = ACTIONS(1069),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1227),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(475),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(479),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(481),
    [anon_sym_static] = ACTIONS(483),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(485),
    [anon_sym_move] = ACTIONS(487),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [257] = {
    [sym_bracketed_type] = STATE(3529),
    [sym_generic_function] = STATE(1664),
    [sym_generic_type_with_turbofish] = STATE(2956),
    [sym__expression_except_range] = STATE(1626),
    [sym__expression] = STATE(1657),
    [sym_macro_invocation] = STATE(1671),
    [sym_scoped_identifier] = STATE(1575),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3163),
    [sym_range_expression] = STATE(1666),
    [sym_unary_expression] = STATE(1664),
    [sym_try_expression] = STATE(1664),
    [sym_reference_expression] = STATE(1664),
    [sym_binary_expression] = STATE(1664),
    [sym_assignment_expression] = STATE(1664),
    [sym_compound_assignment_expr] = STATE(1664),
    [sym_type_cast_expression] = STATE(1664),
    [sym_return_expression] = STATE(1664),
    [sym_yield_expression] = STATE(1664),
    [sym_call_expression] = STATE(1664),
    [sym_array_expression] = STATE(1664),
    [sym_parenthesized_expression] = STATE(1664),
    [sym_tuple_expression] = STATE(1664),
    [sym_unit_expression] = STATE(1664),
    [sym_struct_expression] = STATE(1664),
    [sym_if_expression] = STATE(1664),
    [sym_match_expression] = STATE(1664),
    [sym_while_expression] = STATE(1664),
    [sym_loop_expression] = STATE(1664),
    [sym_for_expression] = STATE(1664),
    [sym_const_block] = STATE(1664),
    [sym_closure_expression] = STATE(1664),
    [sym_closure_parameters] = STATE(232),
    [sym_label] = STATE(3614),
    [sym_break_expression] = STATE(1664),
    [sym_continue_expression] = STATE(1664),
    [sym_index_expression] = STATE(1664),
    [sym_await_expression] = STATE(1664),
    [sym_field_expression] = STATE(1597),
    [sym_unsafe_block] = STATE(1664),
    [sym_async_block] = STATE(1664),
    [sym_gen_block] = STATE(1664),
    [sym_try_block] = STATE(1664),
    [sym_block] = STATE(1664),
    [sym__literal] = STATE(1664),
    [sym_string_literal] = STATE(1786),
    [sym_raw_string_literal] = STATE(1786),
    [sym_boolean_literal] = STATE(1786),
    [sym_line_comment] = STATE(257),
    [sym_block_comment] = STATE(257),
    [sym_identifier] = ACTIONS(405),
    [anon_sym_LPAREN] = ACTIONS(495),
    [anon_sym_LBRACK] = ACTIONS(497),
    [anon_sym_LBRACE] = ACTIONS(407),
    [anon_sym_STAR] = ACTIONS(1015),
    [anon_sym_u8] = ACTIONS(411),
    [anon_sym_i8] = ACTIONS(411),
    [anon_sym_u16] = ACTIONS(411),
    [anon_sym_i16] = ACTIONS(411),
    [anon_sym_u32] = ACTIONS(411),
    [anon_sym_i32] = ACTIONS(411),
    [anon_sym_u64] = ACTIONS(411),
    [anon_sym_i64] = ACTIONS(411),
    [anon_sym_u128] = ACTIONS(411),
    [anon_sym_i128] = ACTIONS(411),
    [anon_sym_isize] = ACTIONS(411),
    [anon_sym_usize] = ACTIONS(411),
    [anon_sym_f32] = ACTIONS(411),
    [anon_sym_f64] = ACTIONS(411),
    [anon_sym_bool] = ACTIONS(411),
    [anon_sym_str] = ACTIONS(411),
    [anon_sym_char] = ACTIONS(411),
    [anon_sym_DASH] = ACTIONS(1015),
    [anon_sym_BANG] = ACTIONS(1015),
    [anon_sym_AMP] = ACTIONS(1017),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1059),
    [anon_sym_COLON_COLON] = ACTIONS(415),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(417),
    [anon_sym_break] = ACTIONS(419),
    [anon_sym_const] = ACTIONS(421),
    [anon_sym_continue] = ACTIONS(423),
    [anon_sym_default] = ACTIONS(425),
    [anon_sym_for] = ACTIONS(427),
    [anon_sym_gen] = ACTIONS(429),
    [anon_sym_if] = ACTIONS(431),
    [anon_sym_loop] = ACTIONS(433),
    [anon_sym_match] = ACTIONS(435),
    [anon_sym_return] = ACTIONS(437),
    [anon_sym_static] = ACTIONS(439),
    [anon_sym_union] = ACTIONS(425),
    [anon_sym_unsafe] = ACTIONS(441),
    [anon_sym_while] = ACTIONS(443),
    [anon_sym_yield] = ACTIONS(445),
    [anon_sym_move] = ACTIONS(447),
    [anon_sym_try] = ACTIONS(449),
    [sym_integer_literal] = ACTIONS(451),
    [aux_sym_string_literal_token1] = ACTIONS(453),
    [sym_char_literal] = ACTIONS(451),
    [anon_sym_true] = ACTIONS(455),
    [anon_sym_false] = ACTIONS(455),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(457),
    [sym_super] = ACTIONS(459),
    [sym_crate] = ACTIONS(459),
    [sym_metavariable] = ACTIONS(461),
    [sym__raw_string_literal_start] = ACTIONS(463),
    [sym_float_literal] = ACTIONS(451),
  },
  [258] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1816),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(258),
    [sym_block_comment] = STATE(258),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [259] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1892),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(259),
    [sym_block_comment] = STATE(259),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [260] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1800),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(260),
    [sym_block_comment] = STATE(260),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [261] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1802),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(261),
    [sym_block_comment] = STATE(261),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [262] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1808),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(262),
    [sym_block_comment] = STATE(262),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [263] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1595),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(263),
    [sym_block_comment] = STATE(263),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(959),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1053),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [264] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1813),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(471),
    [sym_match_expression] = STATE(471),
    [sym_while_expression] = STATE(471),
    [sym_loop_expression] = STATE(471),
    [sym_for_expression] = STATE(471),
    [sym_const_block] = STATE(471),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3608),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(471),
    [sym_async_block] = STATE(471),
    [sym_gen_block] = STATE(471),
    [sym_try_block] = STATE(471),
    [sym_block] = STATE(471),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(264),
    [sym_block_comment] = STATE(264),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(1229),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(1231),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(1233),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(1235),
    [anon_sym_gen] = ACTIONS(1237),
    [anon_sym_if] = ACTIONS(1239),
    [anon_sym_loop] = ACTIONS(1241),
    [anon_sym_match] = ACTIONS(1243),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(1245),
    [anon_sym_while] = ACTIONS(1247),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(1249),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [265] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1885),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(265),
    [sym_block_comment] = STATE(265),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [266] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1889),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(266),
    [sym_block_comment] = STATE(266),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [267] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1718),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(246),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(267),
    [sym_block_comment] = STATE(267),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(1067),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(1067),
    [anon_sym_BANG] = ACTIONS(1067),
    [anon_sym_AMP] = ACTIONS(1069),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(475),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(479),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(481),
    [anon_sym_static] = ACTIONS(483),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(485),
    [anon_sym_move] = ACTIONS(487),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [268] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1513),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(268),
    [sym_block_comment] = STATE(268),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1201),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [269] = {
    [sym_bracketed_type] = STATE(3529),
    [sym_generic_function] = STATE(1664),
    [sym_generic_type_with_turbofish] = STATE(2956),
    [sym__expression_except_range] = STATE(1626),
    [sym__expression] = STATE(1863),
    [sym_macro_invocation] = STATE(1671),
    [sym_scoped_identifier] = STATE(1575),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3163),
    [sym_range_expression] = STATE(1666),
    [sym_unary_expression] = STATE(1664),
    [sym_try_expression] = STATE(1664),
    [sym_reference_expression] = STATE(1664),
    [sym_binary_expression] = STATE(1664),
    [sym_assignment_expression] = STATE(1664),
    [sym_compound_assignment_expr] = STATE(1664),
    [sym_type_cast_expression] = STATE(1664),
    [sym_return_expression] = STATE(1664),
    [sym_yield_expression] = STATE(1664),
    [sym_call_expression] = STATE(1664),
    [sym_array_expression] = STATE(1664),
    [sym_parenthesized_expression] = STATE(1664),
    [sym_tuple_expression] = STATE(1664),
    [sym_unit_expression] = STATE(1664),
    [sym_struct_expression] = STATE(1664),
    [sym_if_expression] = STATE(1664),
    [sym_match_expression] = STATE(1664),
    [sym_while_expression] = STATE(1664),
    [sym_loop_expression] = STATE(1664),
    [sym_for_expression] = STATE(1664),
    [sym_const_block] = STATE(1664),
    [sym_closure_expression] = STATE(1664),
    [sym_closure_parameters] = STATE(232),
    [sym_label] = STATE(3614),
    [sym_break_expression] = STATE(1664),
    [sym_continue_expression] = STATE(1664),
    [sym_index_expression] = STATE(1664),
    [sym_await_expression] = STATE(1664),
    [sym_field_expression] = STATE(1597),
    [sym_unsafe_block] = STATE(1664),
    [sym_async_block] = STATE(1664),
    [sym_gen_block] = STATE(1664),
    [sym_try_block] = STATE(1664),
    [sym_block] = STATE(1664),
    [sym__literal] = STATE(1664),
    [sym_string_literal] = STATE(1786),
    [sym_raw_string_literal] = STATE(1786),
    [sym_boolean_literal] = STATE(1786),
    [sym_line_comment] = STATE(269),
    [sym_block_comment] = STATE(269),
    [sym_identifier] = ACTIONS(405),
    [anon_sym_LPAREN] = ACTIONS(495),
    [anon_sym_LBRACK] = ACTIONS(497),
    [anon_sym_LBRACE] = ACTIONS(407),
    [anon_sym_STAR] = ACTIONS(1015),
    [anon_sym_u8] = ACTIONS(411),
    [anon_sym_i8] = ACTIONS(411),
    [anon_sym_u16] = ACTIONS(411),
    [anon_sym_i16] = ACTIONS(411),
    [anon_sym_u32] = ACTIONS(411),
    [anon_sym_i32] = ACTIONS(411),
    [anon_sym_u64] = ACTIONS(411),
    [anon_sym_i64] = ACTIONS(411),
    [anon_sym_u128] = ACTIONS(411),
    [anon_sym_i128] = ACTIONS(411),
    [anon_sym_isize] = ACTIONS(411),
    [anon_sym_usize] = ACTIONS(411),
    [anon_sym_f32] = ACTIONS(411),
    [anon_sym_f64] = ACTIONS(411),
    [anon_sym_bool] = ACTIONS(411),
    [anon_sym_str] = ACTIONS(411),
    [anon_sym_char] = ACTIONS(411),
    [anon_sym_DASH] = ACTIONS(1015),
    [anon_sym_BANG] = ACTIONS(1015),
    [anon_sym_AMP] = ACTIONS(1017),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1019),
    [anon_sym_COLON_COLON] = ACTIONS(415),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(417),
    [anon_sym_break] = ACTIONS(419),
    [anon_sym_const] = ACTIONS(421),
    [anon_sym_continue] = ACTIONS(423),
    [anon_sym_default] = ACTIONS(425),
    [anon_sym_for] = ACTIONS(427),
    [anon_sym_gen] = ACTIONS(429),
    [anon_sym_if] = ACTIONS(431),
    [anon_sym_loop] = ACTIONS(433),
    [anon_sym_match] = ACTIONS(435),
    [anon_sym_return] = ACTIONS(437),
    [anon_sym_static] = ACTIONS(439),
    [anon_sym_union] = ACTIONS(425),
    [anon_sym_unsafe] = ACTIONS(441),
    [anon_sym_while] = ACTIONS(443),
    [anon_sym_yield] = ACTIONS(445),
    [anon_sym_move] = ACTIONS(447),
    [anon_sym_try] = ACTIONS(449),
    [sym_integer_literal] = ACTIONS(451),
    [aux_sym_string_literal_token1] = ACTIONS(453),
    [sym_char_literal] = ACTIONS(451),
    [anon_sym_true] = ACTIONS(455),
    [anon_sym_false] = ACTIONS(455),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(457),
    [sym_super] = ACTIONS(459),
    [sym_crate] = ACTIONS(459),
    [sym_metavariable] = ACTIONS(461),
    [sym__raw_string_literal_start] = ACTIONS(463),
    [sym_float_literal] = ACTIONS(451),
  },
  [270] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1377),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(270),
    [sym_block_comment] = STATE(270),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1201),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [271] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1521),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(271),
    [sym_block_comment] = STATE(271),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1201),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [272] = {
    [sym_bracketed_type] = STATE(3529),
    [sym_generic_function] = STATE(1664),
    [sym_generic_type_with_turbofish] = STATE(2956),
    [sym__expression_except_range] = STATE(1626),
    [sym__expression] = STATE(1863),
    [sym_macro_invocation] = STATE(1671),
    [sym_scoped_identifier] = STATE(1575),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3163),
    [sym_range_expression] = STATE(1666),
    [sym_unary_expression] = STATE(1664),
    [sym_try_expression] = STATE(1664),
    [sym_reference_expression] = STATE(1664),
    [sym_binary_expression] = STATE(1664),
    [sym_assignment_expression] = STATE(1664),
    [sym_compound_assignment_expr] = STATE(1664),
    [sym_type_cast_expression] = STATE(1664),
    [sym_return_expression] = STATE(1664),
    [sym_yield_expression] = STATE(1664),
    [sym_call_expression] = STATE(1664),
    [sym_array_expression] = STATE(1664),
    [sym_parenthesized_expression] = STATE(1664),
    [sym_tuple_expression] = STATE(1664),
    [sym_unit_expression] = STATE(1664),
    [sym_struct_expression] = STATE(1664),
    [sym_if_expression] = STATE(1664),
    [sym_match_expression] = STATE(1664),
    [sym_while_expression] = STATE(1664),
    [sym_loop_expression] = STATE(1664),
    [sym_for_expression] = STATE(1664),
    [sym_const_block] = STATE(1664),
    [sym_closure_expression] = STATE(1664),
    [sym_closure_parameters] = STATE(232),
    [sym_label] = STATE(3614),
    [sym_break_expression] = STATE(1664),
    [sym_continue_expression] = STATE(1664),
    [sym_index_expression] = STATE(1664),
    [sym_await_expression] = STATE(1664),
    [sym_field_expression] = STATE(1597),
    [sym_unsafe_block] = STATE(1664),
    [sym_async_block] = STATE(1664),
    [sym_gen_block] = STATE(1664),
    [sym_try_block] = STATE(1664),
    [sym_block] = STATE(1664),
    [sym__literal] = STATE(1664),
    [sym_string_literal] = STATE(1786),
    [sym_raw_string_literal] = STATE(1786),
    [sym_boolean_literal] = STATE(1786),
    [sym_line_comment] = STATE(272),
    [sym_block_comment] = STATE(272),
    [sym_identifier] = ACTIONS(405),
    [anon_sym_LPAREN] = ACTIONS(495),
    [anon_sym_LBRACK] = ACTIONS(497),
    [anon_sym_LBRACE] = ACTIONS(407),
    [anon_sym_STAR] = ACTIONS(1015),
    [anon_sym_u8] = ACTIONS(411),
    [anon_sym_i8] = ACTIONS(411),
    [anon_sym_u16] = ACTIONS(411),
    [anon_sym_i16] = ACTIONS(411),
    [anon_sym_u32] = ACTIONS(411),
    [anon_sym_i32] = ACTIONS(411),
    [anon_sym_u64] = ACTIONS(411),
    [anon_sym_i64] = ACTIONS(411),
    [anon_sym_u128] = ACTIONS(411),
    [anon_sym_i128] = ACTIONS(411),
    [anon_sym_isize] = ACTIONS(411),
    [anon_sym_usize] = ACTIONS(411),
    [anon_sym_f32] = ACTIONS(411),
    [anon_sym_f64] = ACTIONS(411),
    [anon_sym_bool] = ACTIONS(411),
    [anon_sym_str] = ACTIONS(411),
    [anon_sym_char] = ACTIONS(411),
    [anon_sym_DASH] = ACTIONS(1015),
    [anon_sym_BANG] = ACTIONS(1015),
    [anon_sym_AMP] = ACTIONS(1017),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1059),
    [anon_sym_COLON_COLON] = ACTIONS(415),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(417),
    [anon_sym_break] = ACTIONS(419),
    [anon_sym_const] = ACTIONS(421),
    [anon_sym_continue] = ACTIONS(423),
    [anon_sym_default] = ACTIONS(425),
    [anon_sym_for] = ACTIONS(427),
    [anon_sym_gen] = ACTIONS(429),
    [anon_sym_if] = ACTIONS(431),
    [anon_sym_loop] = ACTIONS(433),
    [anon_sym_match] = ACTIONS(435),
    [anon_sym_return] = ACTIONS(437),
    [anon_sym_static] = ACTIONS(439),
    [anon_sym_union] = ACTIONS(425),
    [anon_sym_unsafe] = ACTIONS(441),
    [anon_sym_while] = ACTIONS(443),
    [anon_sym_yield] = ACTIONS(445),
    [anon_sym_move] = ACTIONS(447),
    [anon_sym_try] = ACTIONS(449),
    [sym_integer_literal] = ACTIONS(451),
    [aux_sym_string_literal_token1] = ACTIONS(453),
    [sym_char_literal] = ACTIONS(451),
    [anon_sym_true] = ACTIONS(455),
    [anon_sym_false] = ACTIONS(455),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(457),
    [sym_super] = ACTIONS(459),
    [sym_crate] = ACTIONS(459),
    [sym_metavariable] = ACTIONS(461),
    [sym__raw_string_literal_start] = ACTIONS(463),
    [sym_float_literal] = ACTIONS(451),
  },
  [273] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1522),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(273),
    [sym_block_comment] = STATE(273),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1201),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [274] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1524),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(274),
    [sym_block_comment] = STATE(274),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1201),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [275] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1525),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(275),
    [sym_block_comment] = STATE(275),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1201),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [276] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1394),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(276),
    [sym_block_comment] = STATE(276),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(959),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1053),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [277] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1505),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(277),
    [sym_block_comment] = STATE(277),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1201),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [278] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1506),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(278),
    [sym_block_comment] = STATE(278),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1201),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [279] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1890),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(279),
    [sym_block_comment] = STATE(279),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [280] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1855),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(280),
    [sym_block_comment] = STATE(280),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [281] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1880),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(462),
    [sym_match_expression] = STATE(462),
    [sym_while_expression] = STATE(462),
    [sym_loop_expression] = STATE(462),
    [sym_for_expression] = STATE(462),
    [sym_const_block] = STATE(462),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3608),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(462),
    [sym_async_block] = STATE(462),
    [sym_gen_block] = STATE(462),
    [sym_try_block] = STATE(462),
    [sym_block] = STATE(462),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(281),
    [sym_block_comment] = STATE(281),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(1229),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(1231),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(1233),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(1235),
    [anon_sym_gen] = ACTIONS(1237),
    [anon_sym_if] = ACTIONS(1239),
    [anon_sym_loop] = ACTIONS(1241),
    [anon_sym_match] = ACTIONS(1243),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(1245),
    [anon_sym_while] = ACTIONS(1247),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(1249),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [282] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1887),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(282),
    [sym_block_comment] = STATE(282),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [283] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1895),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(283),
    [sym_block_comment] = STATE(283),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [284] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1896),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(284),
    [sym_block_comment] = STATE(284),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [285] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1869),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(471),
    [sym_match_expression] = STATE(471),
    [sym_while_expression] = STATE(471),
    [sym_loop_expression] = STATE(471),
    [sym_for_expression] = STATE(471),
    [sym_const_block] = STATE(471),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3608),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(471),
    [sym_async_block] = STATE(471),
    [sym_gen_block] = STATE(471),
    [sym_try_block] = STATE(471),
    [sym_block] = STATE(471),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(285),
    [sym_block_comment] = STATE(285),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(1229),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(1231),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(1233),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(1235),
    [anon_sym_gen] = ACTIONS(1237),
    [anon_sym_if] = ACTIONS(1239),
    [anon_sym_loop] = ACTIONS(1241),
    [anon_sym_match] = ACTIONS(1243),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(1245),
    [anon_sym_while] = ACTIONS(1247),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(1249),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [286] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1871),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(286),
    [sym_block_comment] = STATE(286),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [287] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1897),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(287),
    [sym_block_comment] = STATE(287),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [288] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1507),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(288),
    [sym_block_comment] = STATE(288),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1201),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [289] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1508),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(289),
    [sym_block_comment] = STATE(289),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1201),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [290] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1510),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(290),
    [sym_block_comment] = STATE(290),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1201),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [291] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1394),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(291),
    [sym_block_comment] = STATE(291),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1201),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [292] = {
    [sym_bracketed_type] = STATE(3529),
    [sym_generic_function] = STATE(1664),
    [sym_generic_type_with_turbofish] = STATE(2956),
    [sym__expression_except_range] = STATE(1626),
    [sym__expression] = STATE(1676),
    [sym_macro_invocation] = STATE(1671),
    [sym_scoped_identifier] = STATE(1575),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3163),
    [sym_range_expression] = STATE(1666),
    [sym_unary_expression] = STATE(1664),
    [sym_try_expression] = STATE(1664),
    [sym_reference_expression] = STATE(1664),
    [sym_binary_expression] = STATE(1664),
    [sym_assignment_expression] = STATE(1664),
    [sym_compound_assignment_expr] = STATE(1664),
    [sym_type_cast_expression] = STATE(1664),
    [sym_return_expression] = STATE(1664),
    [sym_yield_expression] = STATE(1664),
    [sym_call_expression] = STATE(1664),
    [sym_array_expression] = STATE(1664),
    [sym_parenthesized_expression] = STATE(1664),
    [sym_tuple_expression] = STATE(1664),
    [sym_unit_expression] = STATE(1664),
    [sym_struct_expression] = STATE(1664),
    [sym_if_expression] = STATE(1664),
    [sym_match_expression] = STATE(1664),
    [sym_while_expression] = STATE(1664),
    [sym_loop_expression] = STATE(1664),
    [sym_for_expression] = STATE(1664),
    [sym_const_block] = STATE(1664),
    [sym_closure_expression] = STATE(1664),
    [sym_closure_parameters] = STATE(232),
    [sym_label] = STATE(3614),
    [sym_break_expression] = STATE(1664),
    [sym_continue_expression] = STATE(1664),
    [sym_index_expression] = STATE(1664),
    [sym_await_expression] = STATE(1664),
    [sym_field_expression] = STATE(1597),
    [sym_unsafe_block] = STATE(1664),
    [sym_async_block] = STATE(1664),
    [sym_gen_block] = STATE(1664),
    [sym_try_block] = STATE(1664),
    [sym_block] = STATE(1664),
    [sym__literal] = STATE(1664),
    [sym_string_literal] = STATE(1786),
    [sym_raw_string_literal] = STATE(1786),
    [sym_boolean_literal] = STATE(1786),
    [sym_line_comment] = STATE(292),
    [sym_block_comment] = STATE(292),
    [sym_identifier] = ACTIONS(405),
    [anon_sym_LPAREN] = ACTIONS(495),
    [anon_sym_LBRACK] = ACTIONS(497),
    [anon_sym_LBRACE] = ACTIONS(407),
    [anon_sym_STAR] = ACTIONS(1015),
    [anon_sym_u8] = ACTIONS(411),
    [anon_sym_i8] = ACTIONS(411),
    [anon_sym_u16] = ACTIONS(411),
    [anon_sym_i16] = ACTIONS(411),
    [anon_sym_u32] = ACTIONS(411),
    [anon_sym_i32] = ACTIONS(411),
    [anon_sym_u64] = ACTIONS(411),
    [anon_sym_i64] = ACTIONS(411),
    [anon_sym_u128] = ACTIONS(411),
    [anon_sym_i128] = ACTIONS(411),
    [anon_sym_isize] = ACTIONS(411),
    [anon_sym_usize] = ACTIONS(411),
    [anon_sym_f32] = ACTIONS(411),
    [anon_sym_f64] = ACTIONS(411),
    [anon_sym_bool] = ACTIONS(411),
    [anon_sym_str] = ACTIONS(411),
    [anon_sym_char] = ACTIONS(411),
    [anon_sym_DASH] = ACTIONS(1015),
    [anon_sym_BANG] = ACTIONS(1015),
    [anon_sym_AMP] = ACTIONS(1017),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1059),
    [anon_sym_COLON_COLON] = ACTIONS(415),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(417),
    [anon_sym_break] = ACTIONS(419),
    [anon_sym_const] = ACTIONS(421),
    [anon_sym_continue] = ACTIONS(423),
    [anon_sym_default] = ACTIONS(425),
    [anon_sym_for] = ACTIONS(427),
    [anon_sym_gen] = ACTIONS(429),
    [anon_sym_if] = ACTIONS(431),
    [anon_sym_loop] = ACTIONS(433),
    [anon_sym_match] = ACTIONS(435),
    [anon_sym_return] = ACTIONS(437),
    [anon_sym_static] = ACTIONS(439),
    [anon_sym_union] = ACTIONS(425),
    [anon_sym_unsafe] = ACTIONS(441),
    [anon_sym_while] = ACTIONS(443),
    [anon_sym_yield] = ACTIONS(445),
    [anon_sym_move] = ACTIONS(447),
    [anon_sym_try] = ACTIONS(449),
    [sym_integer_literal] = ACTIONS(451),
    [aux_sym_string_literal_token1] = ACTIONS(453),
    [sym_char_literal] = ACTIONS(451),
    [anon_sym_true] = ACTIONS(455),
    [anon_sym_false] = ACTIONS(455),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(457),
    [sym_super] = ACTIONS(459),
    [sym_crate] = ACTIONS(459),
    [sym_metavariable] = ACTIONS(461),
    [sym__raw_string_literal_start] = ACTIONS(463),
    [sym_float_literal] = ACTIONS(451),
  },
  [293] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1511),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(293),
    [sym_block_comment] = STATE(293),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [294] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1431),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(294),
    [sym_block_comment] = STATE(294),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1201),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [295] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1872),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(295),
    [sym_block_comment] = STATE(295),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [296] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1862),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(296),
    [sym_block_comment] = STATE(296),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(959),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(963),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [297] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1431),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(297),
    [sym_block_comment] = STATE(297),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(959),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1053),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [298] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1690),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(298),
    [sym_block_comment] = STATE(298),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [299] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1639),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(299),
    [sym_block_comment] = STATE(299),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(959),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1053),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [300] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1377),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(300),
    [sym_block_comment] = STATE(300),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(959),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1053),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [301] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1640),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(301),
    [sym_block_comment] = STATE(301),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(959),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1053),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [302] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1645),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(302),
    [sym_block_comment] = STATE(302),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(959),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1053),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [303] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1593),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(303),
    [sym_block_comment] = STATE(303),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(959),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1053),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [304] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1712),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(246),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(304),
    [sym_block_comment] = STATE(304),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(1067),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(1067),
    [anon_sym_BANG] = ACTIONS(1067),
    [anon_sym_AMP] = ACTIONS(1069),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(475),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(479),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(481),
    [anon_sym_static] = ACTIONS(483),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(485),
    [anon_sym_move] = ACTIONS(487),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [305] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1605),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(305),
    [sym_block_comment] = STATE(305),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(959),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1053),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [306] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1627),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(306),
    [sym_block_comment] = STATE(306),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(959),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1053),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [307] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1631),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(307),
    [sym_block_comment] = STATE(307),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(959),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1053),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [308] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1394),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(246),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(308),
    [sym_block_comment] = STATE(308),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(1067),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(1067),
    [anon_sym_BANG] = ACTIONS(1067),
    [anon_sym_AMP] = ACTIONS(1069),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(475),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(479),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(481),
    [anon_sym_static] = ACTIONS(483),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(485),
    [anon_sym_move] = ACTIONS(487),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [309] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1632),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(309),
    [sym_block_comment] = STATE(309),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(959),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1053),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [310] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1600),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(310),
    [sym_block_comment] = STATE(310),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(959),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1053),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [311] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1740),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(311),
    [sym_block_comment] = STATE(311),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [312] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1511),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(312),
    [sym_block_comment] = STATE(312),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1201),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [313] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1402),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(313),
    [sym_block_comment] = STATE(313),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(959),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1053),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [314] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1755),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(314),
    [sym_block_comment] = STATE(314),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [315] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1702),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(246),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(315),
    [sym_block_comment] = STATE(315),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(1067),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(1067),
    [anon_sym_BANG] = ACTIONS(1067),
    [anon_sym_AMP] = ACTIONS(1069),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1227),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(475),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(479),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(481),
    [anon_sym_static] = ACTIONS(483),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(485),
    [anon_sym_move] = ACTIONS(487),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [316] = {
    [sym_bracketed_type] = STATE(3529),
    [sym_generic_function] = STATE(1664),
    [sym_generic_type_with_turbofish] = STATE(2956),
    [sym__expression_except_range] = STATE(1626),
    [sym__expression] = STATE(1866),
    [sym_macro_invocation] = STATE(1671),
    [sym_scoped_identifier] = STATE(1575),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3163),
    [sym_range_expression] = STATE(1666),
    [sym_unary_expression] = STATE(1664),
    [sym_try_expression] = STATE(1664),
    [sym_reference_expression] = STATE(1664),
    [sym_binary_expression] = STATE(1664),
    [sym_assignment_expression] = STATE(1664),
    [sym_compound_assignment_expr] = STATE(1664),
    [sym_type_cast_expression] = STATE(1664),
    [sym_return_expression] = STATE(1664),
    [sym_yield_expression] = STATE(1664),
    [sym_call_expression] = STATE(1664),
    [sym_array_expression] = STATE(1664),
    [sym_parenthesized_expression] = STATE(1664),
    [sym_tuple_expression] = STATE(1664),
    [sym_unit_expression] = STATE(1664),
    [sym_struct_expression] = STATE(1664),
    [sym_if_expression] = STATE(1664),
    [sym_match_expression] = STATE(1664),
    [sym_while_expression] = STATE(1664),
    [sym_loop_expression] = STATE(1664),
    [sym_for_expression] = STATE(1664),
    [sym_const_block] = STATE(1664),
    [sym_closure_expression] = STATE(1664),
    [sym_closure_parameters] = STATE(232),
    [sym_label] = STATE(3614),
    [sym_break_expression] = STATE(1664),
    [sym_continue_expression] = STATE(1664),
    [sym_index_expression] = STATE(1664),
    [sym_await_expression] = STATE(1664),
    [sym_field_expression] = STATE(1597),
    [sym_unsafe_block] = STATE(1664),
    [sym_async_block] = STATE(1664),
    [sym_gen_block] = STATE(1664),
    [sym_try_block] = STATE(1664),
    [sym_block] = STATE(1664),
    [sym__literal] = STATE(1664),
    [sym_string_literal] = STATE(1786),
    [sym_raw_string_literal] = STATE(1786),
    [sym_boolean_literal] = STATE(1786),
    [sym_line_comment] = STATE(316),
    [sym_block_comment] = STATE(316),
    [sym_identifier] = ACTIONS(405),
    [anon_sym_LPAREN] = ACTIONS(495),
    [anon_sym_LBRACK] = ACTIONS(497),
    [anon_sym_LBRACE] = ACTIONS(407),
    [anon_sym_STAR] = ACTIONS(1015),
    [anon_sym_u8] = ACTIONS(411),
    [anon_sym_i8] = ACTIONS(411),
    [anon_sym_u16] = ACTIONS(411),
    [anon_sym_i16] = ACTIONS(411),
    [anon_sym_u32] = ACTIONS(411),
    [anon_sym_i32] = ACTIONS(411),
    [anon_sym_u64] = ACTIONS(411),
    [anon_sym_i64] = ACTIONS(411),
    [anon_sym_u128] = ACTIONS(411),
    [anon_sym_i128] = ACTIONS(411),
    [anon_sym_isize] = ACTIONS(411),
    [anon_sym_usize] = ACTIONS(411),
    [anon_sym_f32] = ACTIONS(411),
    [anon_sym_f64] = ACTIONS(411),
    [anon_sym_bool] = ACTIONS(411),
    [anon_sym_str] = ACTIONS(411),
    [anon_sym_char] = ACTIONS(411),
    [anon_sym_DASH] = ACTIONS(1015),
    [anon_sym_BANG] = ACTIONS(1015),
    [anon_sym_AMP] = ACTIONS(1017),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1019),
    [anon_sym_COLON_COLON] = ACTIONS(415),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(417),
    [anon_sym_break] = ACTIONS(419),
    [anon_sym_const] = ACTIONS(421),
    [anon_sym_continue] = ACTIONS(423),
    [anon_sym_default] = ACTIONS(425),
    [anon_sym_for] = ACTIONS(427),
    [anon_sym_gen] = ACTIONS(429),
    [anon_sym_if] = ACTIONS(431),
    [anon_sym_loop] = ACTIONS(433),
    [anon_sym_match] = ACTIONS(435),
    [anon_sym_return] = ACTIONS(437),
    [anon_sym_static] = ACTIONS(439),
    [anon_sym_union] = ACTIONS(425),
    [anon_sym_unsafe] = ACTIONS(441),
    [anon_sym_while] = ACTIONS(443),
    [anon_sym_yield] = ACTIONS(445),
    [anon_sym_move] = ACTIONS(447),
    [anon_sym_try] = ACTIONS(449),
    [sym_integer_literal] = ACTIONS(451),
    [aux_sym_string_literal_token1] = ACTIONS(453),
    [sym_char_literal] = ACTIONS(451),
    [anon_sym_true] = ACTIONS(455),
    [anon_sym_false] = ACTIONS(455),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(457),
    [sym_super] = ACTIONS(459),
    [sym_crate] = ACTIONS(459),
    [sym_metavariable] = ACTIONS(461),
    [sym__raw_string_literal_start] = ACTIONS(463),
    [sym_float_literal] = ACTIONS(451),
  },
  [317] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1402),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(317),
    [sym_block_comment] = STATE(317),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1201),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [318] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1581),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(318),
    [sym_block_comment] = STATE(318),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(959),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(963),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [319] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1431),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(246),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(319),
    [sym_block_comment] = STATE(319),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(1067),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(1067),
    [anon_sym_BANG] = ACTIONS(1067),
    [anon_sym_AMP] = ACTIONS(1069),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(475),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(479),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(481),
    [anon_sym_static] = ACTIONS(483),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(485),
    [anon_sym_move] = ACTIONS(487),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [320] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1873),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(320),
    [sym_block_comment] = STATE(320),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [321] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1708),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(246),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(321),
    [sym_block_comment] = STATE(321),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(1067),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(1067),
    [anon_sym_BANG] = ACTIONS(1067),
    [anon_sym_AMP] = ACTIONS(1069),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(475),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(479),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(481),
    [anon_sym_static] = ACTIONS(483),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(485),
    [anon_sym_move] = ACTIONS(487),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [322] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1377),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(246),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(322),
    [sym_block_comment] = STATE(322),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(1067),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(1067),
    [anon_sym_BANG] = ACTIONS(1067),
    [anon_sym_AMP] = ACTIONS(1069),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(475),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(479),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(481),
    [anon_sym_static] = ACTIONS(483),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(485),
    [anon_sym_move] = ACTIONS(487),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [323] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1709),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(246),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(323),
    [sym_block_comment] = STATE(323),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(1067),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(1067),
    [anon_sym_BANG] = ACTIONS(1067),
    [anon_sym_AMP] = ACTIONS(1069),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(475),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(479),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(481),
    [anon_sym_static] = ACTIONS(483),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(485),
    [anon_sym_move] = ACTIONS(487),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [324] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1710),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(246),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(324),
    [sym_block_comment] = STATE(324),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(1067),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(1067),
    [anon_sym_BANG] = ACTIONS(1067),
    [anon_sym_AMP] = ACTIONS(1069),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(475),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(479),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(481),
    [anon_sym_static] = ACTIONS(483),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(485),
    [anon_sym_move] = ACTIONS(487),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [325] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1711),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(246),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(325),
    [sym_block_comment] = STATE(325),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(1067),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(1067),
    [anon_sym_BANG] = ACTIONS(1067),
    [anon_sym_AMP] = ACTIONS(1069),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(475),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(479),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(481),
    [anon_sym_static] = ACTIONS(483),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(485),
    [anon_sym_move] = ACTIONS(487),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [326] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1596),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(326),
    [sym_block_comment] = STATE(326),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(959),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1053),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [327] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1713),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(246),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(327),
    [sym_block_comment] = STATE(327),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(1067),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(1067),
    [anon_sym_BANG] = ACTIONS(1067),
    [anon_sym_AMP] = ACTIONS(1069),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(475),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(479),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(481),
    [anon_sym_static] = ACTIONS(483),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(485),
    [anon_sym_move] = ACTIONS(487),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [328] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1714),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(246),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(328),
    [sym_block_comment] = STATE(328),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(1067),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(1067),
    [anon_sym_BANG] = ACTIONS(1067),
    [anon_sym_AMP] = ACTIONS(1069),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(475),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(479),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(481),
    [anon_sym_static] = ACTIONS(483),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(485),
    [anon_sym_move] = ACTIONS(487),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [329] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1715),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(246),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(329),
    [sym_block_comment] = STATE(329),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(1067),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(1067),
    [anon_sym_BANG] = ACTIONS(1067),
    [anon_sym_AMP] = ACTIONS(1069),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(475),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(479),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(481),
    [anon_sym_static] = ACTIONS(483),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(485),
    [anon_sym_move] = ACTIONS(487),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [330] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1716),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(246),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(330),
    [sym_block_comment] = STATE(330),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(1067),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(1067),
    [anon_sym_BANG] = ACTIONS(1067),
    [anon_sym_AMP] = ACTIONS(1069),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(475),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(479),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(481),
    [anon_sym_static] = ACTIONS(483),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(485),
    [anon_sym_move] = ACTIONS(487),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [331] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1717),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(246),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(331),
    [sym_block_comment] = STATE(331),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(1067),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(1067),
    [anon_sym_BANG] = ACTIONS(1067),
    [anon_sym_AMP] = ACTIONS(1069),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(475),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(479),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(481),
    [anon_sym_static] = ACTIONS(483),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(485),
    [anon_sym_move] = ACTIONS(487),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [332] = {
    [sym_else_clause] = STATE(375),
    [sym_line_comment] = STATE(332),
    [sym_block_comment] = STATE(332),
    [ts_builtin_sym_end] = ACTIONS(1251),
    [sym_identifier] = ACTIONS(1253),
    [anon_sym_SEMI] = ACTIONS(1251),
    [anon_sym_macro_rules_BANG] = ACTIONS(1251),
    [anon_sym_LPAREN] = ACTIONS(1251),
    [anon_sym_LBRACK] = ACTIONS(1251),
    [anon_sym_LBRACE] = ACTIONS(1251),
    [anon_sym_RBRACE] = ACTIONS(1251),
    [anon_sym_PLUS] = ACTIONS(1253),
    [anon_sym_STAR] = ACTIONS(1253),
    [anon_sym_QMARK] = ACTIONS(1251),
    [anon_sym_u8] = ACTIONS(1253),
    [anon_sym_i8] = ACTIONS(1253),
    [anon_sym_u16] = ACTIONS(1253),
    [anon_sym_i16] = ACTIONS(1253),
    [anon_sym_u32] = ACTIONS(1253),
    [anon_sym_i32] = ACTIONS(1253),
    [anon_sym_u64] = ACTIONS(1253),
    [anon_sym_i64] = ACTIONS(1253),
    [anon_sym_u128] = ACTIONS(1253),
    [anon_sym_i128] = ACTIONS(1253),
    [anon_sym_isize] = ACTIONS(1253),
    [anon_sym_usize] = ACTIONS(1253),
    [anon_sym_f32] = ACTIONS(1253),
    [anon_sym_f64] = ACTIONS(1253),
    [anon_sym_bool] = ACTIONS(1253),
    [anon_sym_str] = ACTIONS(1253),
    [anon_sym_char] = ACTIONS(1253),
    [anon_sym_DASH] = ACTIONS(1253),
    [anon_sym_SLASH] = ACTIONS(1253),
    [anon_sym_PERCENT] = ACTIONS(1253),
    [anon_sym_CARET] = ACTIONS(1253),
    [anon_sym_BANG] = ACTIONS(1253),
    [anon_sym_AMP] = ACTIONS(1253),
    [anon_sym_PIPE] = ACTIONS(1253),
    [anon_sym_AMP_AMP] = ACTIONS(1251),
    [anon_sym_PIPE_PIPE] = ACTIONS(1251),
    [anon_sym_LT_LT] = ACTIONS(1253),
    [anon_sym_GT_GT] = ACTIONS(1253),
    [anon_sym_PLUS_EQ] = ACTIONS(1251),
    [anon_sym_DASH_EQ] = ACTIONS(1251),
    [anon_sym_STAR_EQ] = ACTIONS(1251),
    [anon_sym_SLASH_EQ] = ACTIONS(1251),
    [anon_sym_PERCENT_EQ] = ACTIONS(1251),
    [anon_sym_CARET_EQ] = ACTIONS(1251),
    [anon_sym_AMP_EQ] = ACTIONS(1251),
    [anon_sym_PIPE_EQ] = ACTIONS(1251),
    [anon_sym_LT_LT_EQ] = ACTIONS(1251),
    [anon_sym_GT_GT_EQ] = ACTIONS(1251),
    [anon_sym_EQ] = ACTIONS(1253),
    [anon_sym_EQ_EQ] = ACTIONS(1251),
    [anon_sym_BANG_EQ] = ACTIONS(1251),
    [anon_sym_GT] = ACTIONS(1253),
    [anon_sym_LT] = ACTIONS(1253),
    [anon_sym_GT_EQ] = ACTIONS(1251),
    [anon_sym_LT_EQ] = ACTIONS(1251),
    [anon_sym_DOT] = ACTIONS(1253),
    [anon_sym_DOT_DOT] = ACTIONS(1253),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1251),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1251),
    [anon_sym_COLON_COLON] = ACTIONS(1251),
    [anon_sym_POUND] = ACTIONS(1251),
    [anon_sym_SQUOTE] = ACTIONS(1253),
    [anon_sym_as] = ACTIONS(1253),
    [anon_sym_async] = ACTIONS(1253),
    [anon_sym_break] = ACTIONS(1253),
    [anon_sym_const] = ACTIONS(1253),
    [anon_sym_continue] = ACTIONS(1253),
    [anon_sym_default] = ACTIONS(1253),
    [anon_sym_enum] = ACTIONS(1253),
    [anon_sym_fn] = ACTIONS(1253),
    [anon_sym_for] = ACTIONS(1253),
    [anon_sym_gen] = ACTIONS(1253),
    [anon_sym_if] = ACTIONS(1253),
    [anon_sym_impl] = ACTIONS(1253),
    [anon_sym_let] = ACTIONS(1253),
    [anon_sym_loop] = ACTIONS(1253),
    [anon_sym_match] = ACTIONS(1253),
    [anon_sym_mod] = ACTIONS(1253),
    [anon_sym_pub] = ACTIONS(1253),
    [anon_sym_return] = ACTIONS(1253),
    [anon_sym_static] = ACTIONS(1253),
    [anon_sym_struct] = ACTIONS(1253),
    [anon_sym_trait] = ACTIONS(1253),
    [anon_sym_type] = ACTIONS(1253),
    [anon_sym_union] = ACTIONS(1253),
    [anon_sym_unsafe] = ACTIONS(1253),
    [anon_sym_use] = ACTIONS(1253),
    [anon_sym_while] = ACTIONS(1253),
    [anon_sym_extern] = ACTIONS(1253),
    [anon_sym_else] = ACTIONS(1255),
    [anon_sym_yield] = ACTIONS(1253),
    [anon_sym_move] = ACTIONS(1253),
    [anon_sym_try] = ACTIONS(1253),
    [sym_integer_literal] = ACTIONS(1251),
    [aux_sym_string_literal_token1] = ACTIONS(1251),
    [sym_char_literal] = ACTIONS(1251),
    [anon_sym_true] = ACTIONS(1253),
    [anon_sym_false] = ACTIONS(1253),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1253),
    [sym_super] = ACTIONS(1253),
    [sym_crate] = ACTIONS(1253),
    [sym_metavariable] = ACTIONS(1251),
    [sym__raw_string_literal_start] = ACTIONS(1251),
    [sym_float_literal] = ACTIONS(1251),
  },
  [333] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1595),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(333),
    [sym_block_comment] = STATE(333),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(959),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(963),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [334] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1662),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(334),
    [sym_block_comment] = STATE(334),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [335] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1402),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(246),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(335),
    [sym_block_comment] = STATE(335),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(1067),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(1067),
    [anon_sym_BANG] = ACTIONS(1067),
    [anon_sym_AMP] = ACTIONS(1069),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(475),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(479),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(481),
    [anon_sym_static] = ACTIONS(483),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(485),
    [anon_sym_move] = ACTIONS(487),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [336] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1678),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(336),
    [sym_block_comment] = STATE(336),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [337] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1679),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(462),
    [sym_match_expression] = STATE(462),
    [sym_while_expression] = STATE(462),
    [sym_loop_expression] = STATE(462),
    [sym_for_expression] = STATE(462),
    [sym_const_block] = STATE(462),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3608),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(462),
    [sym_async_block] = STATE(462),
    [sym_gen_block] = STATE(462),
    [sym_try_block] = STATE(462),
    [sym_block] = STATE(462),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(337),
    [sym_block_comment] = STATE(337),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(1229),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(1231),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(1233),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(1235),
    [anon_sym_gen] = ACTIONS(1237),
    [anon_sym_if] = ACTIONS(1239),
    [anon_sym_loop] = ACTIONS(1241),
    [anon_sym_match] = ACTIONS(1243),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(1245),
    [anon_sym_while] = ACTIONS(1247),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(1249),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [338] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1804),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(246),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(338),
    [sym_block_comment] = STATE(338),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(1067),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(1067),
    [anon_sym_BANG] = ACTIONS(1067),
    [anon_sym_AMP] = ACTIONS(1069),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1227),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(475),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(479),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(481),
    [anon_sym_static] = ACTIONS(483),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(485),
    [anon_sym_move] = ACTIONS(487),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [339] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1784),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(339),
    [sym_block_comment] = STATE(339),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [340] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1874),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(340),
    [sym_block_comment] = STATE(340),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [341] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1875),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(341),
    [sym_block_comment] = STATE(341),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [342] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1865),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(342),
    [sym_block_comment] = STATE(342),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [343] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1564),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(343),
    [sym_block_comment] = STATE(343),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(959),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(963),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [344] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1812),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(344),
    [sym_block_comment] = STATE(344),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [345] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1695),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(345),
    [sym_block_comment] = STATE(345),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [346] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1705),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(346),
    [sym_block_comment] = STATE(346),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [347] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1876),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(347),
    [sym_block_comment] = STATE(347),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [348] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1877),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(348),
    [sym_block_comment] = STATE(348),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [349] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1817),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(349),
    [sym_block_comment] = STATE(349),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [350] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1879),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(350),
    [sym_block_comment] = STATE(350),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [351] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1820),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(351),
    [sym_block_comment] = STATE(351),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [352] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1900),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(352),
    [sym_block_comment] = STATE(352),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [353] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1566),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(353),
    [sym_block_comment] = STATE(353),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(959),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(963),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [354] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1822),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(354),
    [sym_block_comment] = STATE(354),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [355] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1881),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(355),
    [sym_block_comment] = STATE(355),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [356] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1882),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(356),
    [sym_block_comment] = STATE(356),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [357] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1883),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(357),
    [sym_block_comment] = STATE(357),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [358] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1884),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(358),
    [sym_block_comment] = STATE(358),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [359] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1826),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(246),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(359),
    [sym_block_comment] = STATE(359),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(1067),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(1067),
    [anon_sym_BANG] = ACTIONS(1067),
    [anon_sym_AMP] = ACTIONS(1069),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1227),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(475),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(479),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(481),
    [anon_sym_static] = ACTIONS(483),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(485),
    [anon_sym_move] = ACTIONS(487),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [360] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1572),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(360),
    [sym_block_comment] = STATE(360),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(959),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(963),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [361] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1579),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(361),
    [sym_block_comment] = STATE(361),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(959),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(963),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [362] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2774),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1888),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1487),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(239),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(362),
    [sym_block_comment] = STATE(362),
    [sym_identifier] = ACTIONS(339),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(23),
    [anon_sym_i8] = ACTIONS(23),
    [anon_sym_u16] = ACTIONS(23),
    [anon_sym_i16] = ACTIONS(23),
    [anon_sym_u32] = ACTIONS(23),
    [anon_sym_i32] = ACTIONS(23),
    [anon_sym_u64] = ACTIONS(23),
    [anon_sym_i64] = ACTIONS(23),
    [anon_sym_u128] = ACTIONS(23),
    [anon_sym_i128] = ACTIONS(23),
    [anon_sym_isize] = ACTIONS(23),
    [anon_sym_usize] = ACTIONS(23),
    [anon_sym_f32] = ACTIONS(23),
    [anon_sym_f64] = ACTIONS(23),
    [anon_sym_bool] = ACTIONS(23),
    [anon_sym_str] = ACTIONS(23),
    [anon_sym_char] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(33),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(355),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(359),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_static] = ACTIONS(367),
    [anon_sym_union] = ACTIONS(355),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(91),
    [anon_sym_move] = ACTIONS(93),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(109),
    [sym_super] = ACTIONS(111),
    [sym_crate] = ACTIONS(111),
    [sym_metavariable] = ACTIONS(115),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [363] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1574),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(363),
    [sym_block_comment] = STATE(363),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(959),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(963),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [364] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1838),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(246),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(364),
    [sym_block_comment] = STATE(364),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(1067),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(1067),
    [anon_sym_BANG] = ACTIONS(1067),
    [anon_sym_AMP] = ACTIONS(1069),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1227),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(475),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(45),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(479),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(481),
    [anon_sym_static] = ACTIONS(483),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(485),
    [anon_sym_move] = ACTIONS(487),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [365] = {
    [sym_bracketed_type] = STATE(3529),
    [sym_generic_function] = STATE(1664),
    [sym_generic_type_with_turbofish] = STATE(2956),
    [sym__expression_except_range] = STATE(1626),
    [sym__expression] = STATE(1760),
    [sym_macro_invocation] = STATE(1671),
    [sym_scoped_identifier] = STATE(1575),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3163),
    [sym_range_expression] = STATE(1666),
    [sym_unary_expression] = STATE(1664),
    [sym_try_expression] = STATE(1664),
    [sym_reference_expression] = STATE(1664),
    [sym_binary_expression] = STATE(1664),
    [sym_assignment_expression] = STATE(1664),
    [sym_compound_assignment_expr] = STATE(1664),
    [sym_type_cast_expression] = STATE(1664),
    [sym_return_expression] = STATE(1664),
    [sym_yield_expression] = STATE(1664),
    [sym_call_expression] = STATE(1664),
    [sym_array_expression] = STATE(1664),
    [sym_parenthesized_expression] = STATE(1664),
    [sym_tuple_expression] = STATE(1664),
    [sym_unit_expression] = STATE(1664),
    [sym_struct_expression] = STATE(1664),
    [sym_if_expression] = STATE(1664),
    [sym_match_expression] = STATE(1664),
    [sym_while_expression] = STATE(1664),
    [sym_loop_expression] = STATE(1664),
    [sym_for_expression] = STATE(1664),
    [sym_const_block] = STATE(1664),
    [sym_closure_expression] = STATE(1664),
    [sym_closure_parameters] = STATE(232),
    [sym_label] = STATE(3614),
    [sym_break_expression] = STATE(1664),
    [sym_continue_expression] = STATE(1664),
    [sym_index_expression] = STATE(1664),
    [sym_await_expression] = STATE(1664),
    [sym_field_expression] = STATE(1597),
    [sym_unsafe_block] = STATE(1664),
    [sym_async_block] = STATE(1664),
    [sym_gen_block] = STATE(1664),
    [sym_try_block] = STATE(1664),
    [sym_block] = STATE(1664),
    [sym__literal] = STATE(1664),
    [sym_string_literal] = STATE(1786),
    [sym_raw_string_literal] = STATE(1786),
    [sym_boolean_literal] = STATE(1786),
    [sym_line_comment] = STATE(365),
    [sym_block_comment] = STATE(365),
    [sym_identifier] = ACTIONS(405),
    [anon_sym_LPAREN] = ACTIONS(495),
    [anon_sym_LBRACK] = ACTIONS(497),
    [anon_sym_LBRACE] = ACTIONS(407),
    [anon_sym_STAR] = ACTIONS(1015),
    [anon_sym_u8] = ACTIONS(411),
    [anon_sym_i8] = ACTIONS(411),
    [anon_sym_u16] = ACTIONS(411),
    [anon_sym_i16] = ACTIONS(411),
    [anon_sym_u32] = ACTIONS(411),
    [anon_sym_i32] = ACTIONS(411),
    [anon_sym_u64] = ACTIONS(411),
    [anon_sym_i64] = ACTIONS(411),
    [anon_sym_u128] = ACTIONS(411),
    [anon_sym_i128] = ACTIONS(411),
    [anon_sym_isize] = ACTIONS(411),
    [anon_sym_usize] = ACTIONS(411),
    [anon_sym_f32] = ACTIONS(411),
    [anon_sym_f64] = ACTIONS(411),
    [anon_sym_bool] = ACTIONS(411),
    [anon_sym_str] = ACTIONS(411),
    [anon_sym_char] = ACTIONS(411),
    [anon_sym_DASH] = ACTIONS(1015),
    [anon_sym_BANG] = ACTIONS(1015),
    [anon_sym_AMP] = ACTIONS(1017),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1059),
    [anon_sym_COLON_COLON] = ACTIONS(415),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(417),
    [anon_sym_break] = ACTIONS(419),
    [anon_sym_const] = ACTIONS(421),
    [anon_sym_continue] = ACTIONS(423),
    [anon_sym_default] = ACTIONS(425),
    [anon_sym_for] = ACTIONS(427),
    [anon_sym_gen] = ACTIONS(429),
    [anon_sym_if] = ACTIONS(431),
    [anon_sym_loop] = ACTIONS(433),
    [anon_sym_match] = ACTIONS(435),
    [anon_sym_return] = ACTIONS(437),
    [anon_sym_static] = ACTIONS(439),
    [anon_sym_union] = ACTIONS(425),
    [anon_sym_unsafe] = ACTIONS(441),
    [anon_sym_while] = ACTIONS(443),
    [anon_sym_yield] = ACTIONS(445),
    [anon_sym_move] = ACTIONS(447),
    [anon_sym_try] = ACTIONS(449),
    [sym_integer_literal] = ACTIONS(451),
    [aux_sym_string_literal_token1] = ACTIONS(453),
    [sym_char_literal] = ACTIONS(451),
    [anon_sym_true] = ACTIONS(455),
    [anon_sym_false] = ACTIONS(455),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(457),
    [sym_super] = ACTIONS(459),
    [sym_crate] = ACTIONS(459),
    [sym_metavariable] = ACTIONS(461),
    [sym__raw_string_literal_start] = ACTIONS(463),
    [sym_float_literal] = ACTIONS(451),
  },
  [366] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1576),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(366),
    [sym_block_comment] = STATE(366),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(959),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(963),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [367] = {
    [sym_bracketed_type] = STATE(3529),
    [sym_generic_function] = STATE(1664),
    [sym_generic_type_with_turbofish] = STATE(2956),
    [sym__expression_except_range] = STATE(1626),
    [sym__expression] = STATE(1836),
    [sym_macro_invocation] = STATE(1671),
    [sym_scoped_identifier] = STATE(1575),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3163),
    [sym_range_expression] = STATE(1666),
    [sym_unary_expression] = STATE(1664),
    [sym_try_expression] = STATE(1664),
    [sym_reference_expression] = STATE(1664),
    [sym_binary_expression] = STATE(1664),
    [sym_assignment_expression] = STATE(1664),
    [sym_compound_assignment_expr] = STATE(1664),
    [sym_type_cast_expression] = STATE(1664),
    [sym_return_expression] = STATE(1664),
    [sym_yield_expression] = STATE(1664),
    [sym_call_expression] = STATE(1664),
    [sym_array_expression] = STATE(1664),
    [sym_parenthesized_expression] = STATE(1664),
    [sym_tuple_expression] = STATE(1664),
    [sym_unit_expression] = STATE(1664),
    [sym_struct_expression] = STATE(1664),
    [sym_if_expression] = STATE(1664),
    [sym_match_expression] = STATE(1664),
    [sym_while_expression] = STATE(1664),
    [sym_loop_expression] = STATE(1664),
    [sym_for_expression] = STATE(1664),
    [sym_const_block] = STATE(1664),
    [sym_closure_expression] = STATE(1664),
    [sym_closure_parameters] = STATE(232),
    [sym_label] = STATE(3614),
    [sym_break_expression] = STATE(1664),
    [sym_continue_expression] = STATE(1664),
    [sym_index_expression] = STATE(1664),
    [sym_await_expression] = STATE(1664),
    [sym_field_expression] = STATE(1597),
    [sym_unsafe_block] = STATE(1664),
    [sym_async_block] = STATE(1664),
    [sym_gen_block] = STATE(1664),
    [sym_try_block] = STATE(1664),
    [sym_block] = STATE(1664),
    [sym__literal] = STATE(1664),
    [sym_string_literal] = STATE(1786),
    [sym_raw_string_literal] = STATE(1786),
    [sym_boolean_literal] = STATE(1786),
    [sym_line_comment] = STATE(367),
    [sym_block_comment] = STATE(367),
    [sym_identifier] = ACTIONS(405),
    [anon_sym_LPAREN] = ACTIONS(495),
    [anon_sym_LBRACK] = ACTIONS(497),
    [anon_sym_LBRACE] = ACTIONS(407),
    [anon_sym_STAR] = ACTIONS(1015),
    [anon_sym_u8] = ACTIONS(411),
    [anon_sym_i8] = ACTIONS(411),
    [anon_sym_u16] = ACTIONS(411),
    [anon_sym_i16] = ACTIONS(411),
    [anon_sym_u32] = ACTIONS(411),
    [anon_sym_i32] = ACTIONS(411),
    [anon_sym_u64] = ACTIONS(411),
    [anon_sym_i64] = ACTIONS(411),
    [anon_sym_u128] = ACTIONS(411),
    [anon_sym_i128] = ACTIONS(411),
    [anon_sym_isize] = ACTIONS(411),
    [anon_sym_usize] = ACTIONS(411),
    [anon_sym_f32] = ACTIONS(411),
    [anon_sym_f64] = ACTIONS(411),
    [anon_sym_bool] = ACTIONS(411),
    [anon_sym_str] = ACTIONS(411),
    [anon_sym_char] = ACTIONS(411),
    [anon_sym_DASH] = ACTIONS(1015),
    [anon_sym_BANG] = ACTIONS(1015),
    [anon_sym_AMP] = ACTIONS(1017),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1059),
    [anon_sym_COLON_COLON] = ACTIONS(415),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(417),
    [anon_sym_break] = ACTIONS(419),
    [anon_sym_const] = ACTIONS(421),
    [anon_sym_continue] = ACTIONS(423),
    [anon_sym_default] = ACTIONS(425),
    [anon_sym_for] = ACTIONS(427),
    [anon_sym_gen] = ACTIONS(429),
    [anon_sym_if] = ACTIONS(431),
    [anon_sym_loop] = ACTIONS(433),
    [anon_sym_match] = ACTIONS(435),
    [anon_sym_return] = ACTIONS(437),
    [anon_sym_static] = ACTIONS(439),
    [anon_sym_union] = ACTIONS(425),
    [anon_sym_unsafe] = ACTIONS(441),
    [anon_sym_while] = ACTIONS(443),
    [anon_sym_yield] = ACTIONS(445),
    [anon_sym_move] = ACTIONS(447),
    [anon_sym_try] = ACTIONS(449),
    [sym_integer_literal] = ACTIONS(451),
    [aux_sym_string_literal_token1] = ACTIONS(453),
    [sym_char_literal] = ACTIONS(451),
    [anon_sym_true] = ACTIONS(455),
    [anon_sym_false] = ACTIONS(455),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(457),
    [sym_super] = ACTIONS(459),
    [sym_crate] = ACTIONS(459),
    [sym_metavariable] = ACTIONS(461),
    [sym__raw_string_literal_start] = ACTIONS(463),
    [sym_float_literal] = ACTIONS(451),
  },
  [368] = {
    [sym_bracketed_type] = STATE(3517),
    [sym_generic_function] = STATE(1474),
    [sym_generic_type_with_turbofish] = STATE(2974),
    [sym__expression_except_range] = STATE(1369),
    [sym__expression] = STATE(1577),
    [sym_macro_invocation] = STATE(1459),
    [sym_scoped_identifier] = STATE(1558),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3119),
    [sym_range_expression] = STATE(1482),
    [sym_unary_expression] = STATE(1474),
    [sym_try_expression] = STATE(1474),
    [sym_reference_expression] = STATE(1474),
    [sym_binary_expression] = STATE(1474),
    [sym_assignment_expression] = STATE(1474),
    [sym_compound_assignment_expr] = STATE(1474),
    [sym_type_cast_expression] = STATE(1474),
    [sym_return_expression] = STATE(1474),
    [sym_yield_expression] = STATE(1474),
    [sym_call_expression] = STATE(1474),
    [sym_array_expression] = STATE(1474),
    [sym_parenthesized_expression] = STATE(1474),
    [sym_tuple_expression] = STATE(1474),
    [sym_unit_expression] = STATE(1474),
    [sym_struct_expression] = STATE(1474),
    [sym_if_expression] = STATE(1474),
    [sym_match_expression] = STATE(1474),
    [sym_while_expression] = STATE(1474),
    [sym_loop_expression] = STATE(1474),
    [sym_for_expression] = STATE(1474),
    [sym_const_block] = STATE(1474),
    [sym_closure_expression] = STATE(1474),
    [sym_closure_parameters] = STATE(225),
    [sym_label] = STATE(3565),
    [sym_break_expression] = STATE(1474),
    [sym_continue_expression] = STATE(1474),
    [sym_index_expression] = STATE(1474),
    [sym_await_expression] = STATE(1474),
    [sym_field_expression] = STATE(1371),
    [sym_unsafe_block] = STATE(1474),
    [sym_async_block] = STATE(1474),
    [sym_gen_block] = STATE(1474),
    [sym_try_block] = STATE(1474),
    [sym_block] = STATE(1474),
    [sym__literal] = STATE(1474),
    [sym_string_literal] = STATE(1489),
    [sym_raw_string_literal] = STATE(1489),
    [sym_boolean_literal] = STATE(1489),
    [sym_line_comment] = STATE(368),
    [sym_block_comment] = STATE(368),
    [sym_identifier] = ACTIONS(467),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACK] = ACTIONS(17),
    [anon_sym_LBRACE] = ACTIONS(343),
    [anon_sym_STAR] = ACTIONS(959),
    [anon_sym_u8] = ACTIONS(469),
    [anon_sym_i8] = ACTIONS(469),
    [anon_sym_u16] = ACTIONS(469),
    [anon_sym_i16] = ACTIONS(469),
    [anon_sym_u32] = ACTIONS(469),
    [anon_sym_i32] = ACTIONS(469),
    [anon_sym_u64] = ACTIONS(469),
    [anon_sym_i64] = ACTIONS(469),
    [anon_sym_u128] = ACTIONS(469),
    [anon_sym_i128] = ACTIONS(469),
    [anon_sym_isize] = ACTIONS(469),
    [anon_sym_usize] = ACTIONS(469),
    [anon_sym_f32] = ACTIONS(469),
    [anon_sym_f64] = ACTIONS(469),
    [anon_sym_bool] = ACTIONS(469),
    [anon_sym_str] = ACTIONS(469),
    [anon_sym_char] = ACTIONS(469),
    [anon_sym_DASH] = ACTIONS(959),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_AMP] = ACTIONS(961),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(963),
    [anon_sym_COLON_COLON] = ACTIONS(473),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(351),
    [anon_sym_break] = ACTIONS(505),
    [anon_sym_const] = ACTIONS(353),
    [anon_sym_continue] = ACTIONS(507),
    [anon_sym_default] = ACTIONS(477),
    [anon_sym_for] = ACTIONS(357),
    [anon_sym_gen] = ACTIONS(509),
    [anon_sym_if] = ACTIONS(361),
    [anon_sym_loop] = ACTIONS(363),
    [anon_sym_match] = ACTIONS(365),
    [anon_sym_return] = ACTIONS(511),
    [anon_sym_static] = ACTIONS(513),
    [anon_sym_union] = ACTIONS(477),
    [anon_sym_unsafe] = ACTIONS(369),
    [anon_sym_while] = ACTIONS(371),
    [anon_sym_yield] = ACTIONS(515),
    [anon_sym_move] = ACTIONS(517),
    [anon_sym_try] = ACTIONS(373),
    [sym_integer_literal] = ACTIONS(97),
    [aux_sym_string_literal_token1] = ACTIONS(99),
    [sym_char_literal] = ACTIONS(97),
    [anon_sym_true] = ACTIONS(101),
    [anon_sym_false] = ACTIONS(101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(489),
    [sym_super] = ACTIONS(491),
    [sym_crate] = ACTIONS(491),
    [sym_metavariable] = ACTIONS(493),
    [sym__raw_string_literal_start] = ACTIONS(117),
    [sym_float_literal] = ACTIONS(97),
  },
  [369] = {
    [sym_bracketed_type] = STATE(3529),
    [sym_generic_function] = STATE(1664),
    [sym_generic_type_with_turbofish] = STATE(2956),
    [sym__expression_except_range] = STATE(1626),
    [sym__expression] = STATE(1761),
    [sym_macro_invocation] = STATE(1671),
    [sym_scoped_identifier] = STATE(1575),
    [sym_scoped_type_identifier_in_expression_position] = STATE(3163),
    [sym_range_expression] = STATE(1666),
    [sym_unary_expression] = STATE(1664),
    [sym_try_expression] = STATE(1664),
    [sym_reference_expression] = STATE(1664),
    [sym_binary_expression] = STATE(1664),
    [sym_assignment_expression] = STATE(1664),
    [sym_compound_assignment_expr] = STATE(1664),
    [sym_type_cast_expression] = STATE(1664),
    [sym_return_expression] = STATE(1664),
    [sym_yield_expression] = STATE(1664),
    [sym_call_expression] = STATE(1664),
    [sym_array_expression] = STATE(1664),
    [sym_parenthesized_expression] = STATE(1664),
    [sym_tuple_expression] = STATE(1664),
    [sym_unit_expression] = STATE(1664),
    [sym_struct_expression] = STATE(1664),
    [sym_if_expression] = STATE(1664),
    [sym_match_expression] = STATE(1664),
    [sym_while_expression] = STATE(1664),
    [sym_loop_expression] = STATE(1664),
    [sym_for_expression] = STATE(1664),
    [sym_const_block] = STATE(1664),
    [sym_closure_expression] = STATE(1664),
    [sym_closure_parameters] = STATE(232),
    [sym_label] = STATE(3614),
    [sym_break_expression] = STATE(1664),
    [sym_continue_expression] = STATE(1664),
    [sym_index_expression] = STATE(1664),
    [sym_await_expression] = STATE(1664),
    [sym_field_expression] = STATE(1597),
    [sym_unsafe_block] = STATE(1664),
    [sym_async_block] = STATE(1664),
    [sym_gen_block] = STATE(1664),
    [sym_try_block] = STATE(1664),
    [sym_block] = STATE(1664),
    [sym__literal] = STATE(1664),
    [sym_string_literal] = STATE(1786),
    [sym_raw_string_literal] = STATE(1786),
    [sym_boolean_literal] = STATE(1786),
    [sym_line_comment] = STATE(369),
    [sym_block_comment] = STATE(369),
    [sym_identifier] = ACTIONS(405),
    [anon_sym_LPAREN] = ACTIONS(495),
    [anon_sym_LBRACK] = ACTIONS(497),
    [anon_sym_LBRACE] = ACTIONS(407),
    [anon_sym_STAR] = ACTIONS(1015),
    [anon_sym_u8] = ACTIONS(411),
    [anon_sym_i8] = ACTIONS(411),
    [anon_sym_u16] = ACTIONS(411),
    [anon_sym_i16] = ACTIONS(411),
    [anon_sym_u32] = ACTIONS(411),
    [anon_sym_i32] = ACTIONS(411),
    [anon_sym_u64] = ACTIONS(411),
    [anon_sym_i64] = ACTIONS(411),
    [anon_sym_u128] = ACTIONS(411),
    [anon_sym_i128] = ACTIONS(411),
    [anon_sym_isize] = ACTIONS(411),
    [anon_sym_usize] = ACTIONS(411),
    [anon_sym_f32] = ACTIONS(411),
    [anon_sym_f64] = ACTIONS(411),
    [anon_sym_bool] = ACTIONS(411),
    [anon_sym_str] = ACTIONS(411),
    [anon_sym_char] = ACTIONS(411),
    [anon_sym_DASH] = ACTIONS(1015),
    [anon_sym_BANG] = ACTIONS(1015),
    [anon_sym_AMP] = ACTIONS(1017),
    [anon_sym_PIPE] = ACTIONS(27),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT] = ACTIONS(1059),
    [anon_sym_COLON_COLON] = ACTIONS(415),
    [anon_sym_SQUOTE] = ACTIONS(37),
    [anon_sym_async] = ACTIONS(417),
    [anon_sym_break] = ACTIONS(419),
    [anon_sym_const] = ACTIONS(421),
    [anon_sym_continue] = ACTIONS(423),
    [anon_sym_default] = ACTIONS(425),
    [anon_sym_for] = ACTIONS(427),
    [anon_sym_gen] = ACTIONS(429),
    [anon_sym_if] = ACTIONS(431),
    [anon_sym_loop] = ACTIONS(433),
    [anon_sym_match] = ACTIONS(435),
    [anon_sym_return] = ACTIONS(437),
    [anon_sym_static] = ACTIONS(439),
    [anon_sym_union] = ACTIONS(425),
    [anon_sym_unsafe] = ACTIONS(441),
    [anon_sym_while] = ACTIONS(443),
    [anon_sym_yield] = ACTIONS(445),
    [anon_sym_move] = ACTIONS(447),
    [anon_sym_try] = ACTIONS(449),
    [sym_integer_literal] = ACTIONS(451),
    [aux_sym_string_literal_token1] = ACTIONS(453),
    [sym_char_literal] = ACTIONS(451),
    [anon_sym_true] = ACTIONS(455),
    [anon_sym_false] = ACTIONS(455),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(457),
    [sym_super] = ACTIONS(459),
    [sym_crate] = ACTIONS(459),
    [sym_metavariable] = ACTIONS(461),
    [sym__raw_string_literal_start] = ACTIONS(463),
    [sym_float_literal] = ACTIONS(451),
  },
  [370] = {
    [sym_line_comment] = STATE(370),
    [sym_block_comment] = STATE(370),
    [ts_builtin_sym_end] = ACTIONS(1257),
    [sym_identifier] = ACTIONS(1259),
    [anon_sym_SEMI] = ACTIONS(1257),
    [anon_sym_macro_rules_BANG] = ACTIONS(1257),
    [anon_sym_LPAREN] = ACTIONS(1257),
    [anon_sym_LBRACK] = ACTIONS(1257),
    [anon_sym_LBRACE] = ACTIONS(1257),
    [anon_sym_RBRACE] = ACTIONS(1257),
    [anon_sym_PLUS] = ACTIONS(1259),
    [anon_sym_STAR] = ACTIONS(1259),
    [anon_sym_QMARK] = ACTIONS(1257),
    [anon_sym_u8] = ACTIONS(1259),
    [anon_sym_i8] = ACTIONS(1259),
    [anon_sym_u16] = ACTIONS(1259),
    [anon_sym_i16] = ACTIONS(1259),
    [anon_sym_u32] = ACTIONS(1259),
    [anon_sym_i32] = ACTIONS(1259),
    [anon_sym_u64] = ACTIONS(1259),
    [anon_sym_i64] = ACTIONS(1259),
    [anon_sym_u128] = ACTIONS(1259),
    [anon_sym_i128] = ACTIONS(1259),
    [anon_sym_isize] = ACTIONS(1259),
    [anon_sym_usize] = ACTIONS(1259),
    [anon_sym_f32] = ACTIONS(1259),
    [anon_sym_f64] = ACTIONS(1259),
    [anon_sym_bool] = ACTIONS(1259),
    [anon_sym_str] = ACTIONS(1259),
    [anon_sym_char] = ACTIONS(1259),
    [anon_sym_DASH] = ACTIONS(1259),
    [anon_sym_SLASH] = ACTIONS(1259),
    [anon_sym_PERCENT] = ACTIONS(1259),
    [anon_sym_CARET] = ACTIONS(1259),
    [anon_sym_BANG] = ACTIONS(1259),
    [anon_sym_AMP] = ACTIONS(1259),
    [anon_sym_PIPE] = ACTIONS(1259),
    [anon_sym_AMP_AMP] = ACTIONS(1257),
    [anon_sym_PIPE_PIPE] = ACTIONS(1257),
    [anon_sym_LT_LT] = ACTIONS(1259),
    [anon_sym_GT_GT] = ACTIONS(1259),
    [anon_sym_PLUS_EQ] = ACTIONS(1257),
    [anon_sym_DASH_EQ] = ACTIONS(1257),
    [anon_sym_STAR_EQ] = ACTIONS(1257),
    [anon_sym_SLASH_EQ] = ACTIONS(1257),
    [anon_sym_PERCENT_EQ] = ACTIONS(1257),
    [anon_sym_CARET_EQ] = ACTIONS(1257),
    [anon_sym_AMP_EQ] = ACTIONS(1257),
    [anon_sym_PIPE_EQ] = ACTIONS(1257),
    [anon_sym_LT_LT_EQ] = ACTIONS(1257),
    [anon_sym_GT_GT_EQ] = ACTIONS(1257),
    [anon_sym_EQ] = ACTIONS(1259),
    [anon_sym_EQ_EQ] = ACTIONS(1257),
    [anon_sym_BANG_EQ] = ACTIONS(1257),
    [anon_sym_GT] = ACTIONS(1259),
    [anon_sym_LT] = ACTIONS(1259),
    [anon_sym_GT_EQ] = ACTIONS(1257),
    [anon_sym_LT_EQ] = ACTIONS(1257),
    [anon_sym_DOT] = ACTIONS(1259),
    [anon_sym_DOT_DOT] = ACTIONS(1259),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1257),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1257),
    [anon_sym_COLON_COLON] = ACTIONS(1257),
    [anon_sym_POUND] = ACTIONS(1257),
    [anon_sym_SQUOTE] = ACTIONS(1259),
    [anon_sym_as] = ACTIONS(1259),
    [anon_sym_async] = ACTIONS(1259),
    [anon_sym_break] = ACTIONS(1259),
    [anon_sym_const] = ACTIONS(1259),
    [anon_sym_continue] = ACTIONS(1259),
    [anon_sym_default] = ACTIONS(1259),
    [anon_sym_enum] = ACTIONS(1259),
    [anon_sym_fn] = ACTIONS(1259),
    [anon_sym_for] = ACTIONS(1259),
    [anon_sym_gen] = ACTIONS(1259),
    [anon_sym_if] = ACTIONS(1259),
    [anon_sym_impl] = ACTIONS(1259),
    [anon_sym_let] = ACTIONS(1259),
    [anon_sym_loop] = ACTIONS(1259),
    [anon_sym_match] = ACTIONS(1259),
    [anon_sym_mod] = ACTIONS(1259),
    [anon_sym_pub] = ACTIONS(1259),
    [anon_sym_return] = ACTIONS(1259),
    [anon_sym_static] = ACTIONS(1259),
    [anon_sym_struct] = ACTIONS(1259),
    [anon_sym_trait] = ACTIONS(1259),
    [anon_sym_type] = ACTIONS(1259),
    [anon_sym_union] = ACTIONS(1259),
    [anon_sym_unsafe] = ACTIONS(1259),
    [anon_sym_use] = ACTIONS(1259),
    [anon_sym_while] = ACTIONS(1259),
    [anon_sym_extern] = ACTIONS(1259),
    [anon_sym_else] = ACTIONS(1259),
    [anon_sym_yield] = ACTIONS(1259),
    [anon_sym_move] = ACTIONS(1259),
    [anon_sym_try] = ACTIONS(1259),
    [sym_integer_literal] = ACTIONS(1257),
    [aux_sym_string_literal_token1] = ACTIONS(1257),
    [sym_char_literal] = ACTIONS(1257),
    [anon_sym_true] = ACTIONS(1259),
    [anon_sym_false] = ACTIONS(1259),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1259),
    [sym_super] = ACTIONS(1259),
    [sym_crate] = ACTIONS(1259),
    [sym_metavariable] = ACTIONS(1257),
    [sym__raw_string_literal_start] = ACTIONS(1257),
    [sym_float_literal] = ACTIONS(1257),
  },
  [371] = {
    [sym_line_comment] = STATE(371),
    [sym_block_comment] = STATE(371),
    [ts_builtin_sym_end] = ACTIONS(1261),
    [sym_identifier] = ACTIONS(1263),
    [anon_sym_SEMI] = ACTIONS(1261),
    [anon_sym_macro_rules_BANG] = ACTIONS(1261),
    [anon_sym_LPAREN] = ACTIONS(1261),
    [anon_sym_LBRACK] = ACTIONS(1261),
    [anon_sym_LBRACE] = ACTIONS(1261),
    [anon_sym_RBRACE] = ACTIONS(1261),
    [anon_sym_PLUS] = ACTIONS(1263),
    [anon_sym_STAR] = ACTIONS(1263),
    [anon_sym_QMARK] = ACTIONS(1261),
    [anon_sym_u8] = ACTIONS(1263),
    [anon_sym_i8] = ACTIONS(1263),
    [anon_sym_u16] = ACTIONS(1263),
    [anon_sym_i16] = ACTIONS(1263),
    [anon_sym_u32] = ACTIONS(1263),
    [anon_sym_i32] = ACTIONS(1263),
    [anon_sym_u64] = ACTIONS(1263),
    [anon_sym_i64] = ACTIONS(1263),
    [anon_sym_u128] = ACTIONS(1263),
    [anon_sym_i128] = ACTIONS(1263),
    [anon_sym_isize] = ACTIONS(1263),
    [anon_sym_usize] = ACTIONS(1263),
    [anon_sym_f32] = ACTIONS(1263),
    [anon_sym_f64] = ACTIONS(1263),
    [anon_sym_bool] = ACTIONS(1263),
    [anon_sym_str] = ACTIONS(1263),
    [anon_sym_char] = ACTIONS(1263),
    [anon_sym_DASH] = ACTIONS(1263),
    [anon_sym_SLASH] = ACTIONS(1263),
    [anon_sym_PERCENT] = ACTIONS(1263),
    [anon_sym_CARET] = ACTIONS(1263),
    [anon_sym_BANG] = ACTIONS(1263),
    [anon_sym_AMP] = ACTIONS(1263),
    [anon_sym_PIPE] = ACTIONS(1263),
    [anon_sym_AMP_AMP] = ACTIONS(1261),
    [anon_sym_PIPE_PIPE] = ACTIONS(1261),
    [anon_sym_LT_LT] = ACTIONS(1263),
    [anon_sym_GT_GT] = ACTIONS(1263),
    [anon_sym_PLUS_EQ] = ACTIONS(1261),
    [anon_sym_DASH_EQ] = ACTIONS(1261),
    [anon_sym_STAR_EQ] = ACTIONS(1261),
    [anon_sym_SLASH_EQ] = ACTIONS(1261),
    [anon_sym_PERCENT_EQ] = ACTIONS(1261),
    [anon_sym_CARET_EQ] = ACTIONS(1261),
    [anon_sym_AMP_EQ] = ACTIONS(1261),
    [anon_sym_PIPE_EQ] = ACTIONS(1261),
    [anon_sym_LT_LT_EQ] = ACTIONS(1261),
    [anon_sym_GT_GT_EQ] = ACTIONS(1261),
    [anon_sym_EQ] = ACTIONS(1263),
    [anon_sym_EQ_EQ] = ACTIONS(1261),
    [anon_sym_BANG_EQ] = ACTIONS(1261),
    [anon_sym_GT] = ACTIONS(1263),
    [anon_sym_LT] = ACTIONS(1263),
    [anon_sym_GT_EQ] = ACTIONS(1261),
    [anon_sym_LT_EQ] = ACTIONS(1261),
    [anon_sym_DOT] = ACTIONS(1263),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1261),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1261),
    [anon_sym_COLON_COLON] = ACTIONS(1261),
    [anon_sym_POUND] = ACTIONS(1261),
    [anon_sym_SQUOTE] = ACTIONS(1263),
    [anon_sym_as] = ACTIONS(1263),
    [anon_sym_async] = ACTIONS(1263),
    [anon_sym_break] = ACTIONS(1263),
    [anon_sym_const] = ACTIONS(1263),
    [anon_sym_continue] = ACTIONS(1263),
    [anon_sym_default] = ACTIONS(1263),
    [anon_sym_enum] = ACTIONS(1263),
    [anon_sym_fn] = ACTIONS(1263),
    [anon_sym_for] = ACTIONS(1263),
    [anon_sym_gen] = ACTIONS(1263),
    [anon_sym_if] = ACTIONS(1263),
    [anon_sym_impl] = ACTIONS(1263),
    [anon_sym_let] = ACTIONS(1263),
    [anon_sym_loop] = ACTIONS(1263),
    [anon_sym_match] = ACTIONS(1263),
    [anon_sym_mod] = ACTIONS(1263),
    [anon_sym_pub] = ACTIONS(1263),
    [anon_sym_return] = ACTIONS(1263),
    [anon_sym_static] = ACTIONS(1263),
    [anon_sym_struct] = ACTIONS(1263),
    [anon_sym_trait] = ACTIONS(1263),
    [anon_sym_type] = ACTIONS(1263),
    [anon_sym_union] = ACTIONS(1263),
    [anon_sym_unsafe] = ACTIONS(1263),
    [anon_sym_use] = ACTIONS(1263),
    [anon_sym_while] = ACTIONS(1263),
    [anon_sym_extern] = ACTIONS(1263),
    [anon_sym_else] = ACTIONS(1263),
    [anon_sym_yield] = ACTIONS(1263),
    [anon_sym_move] = ACTIONS(1263),
    [anon_sym_try] = ACTIONS(1263),
    [sym_integer_literal] = ACTIONS(1261),
    [aux_sym_string_literal_token1] = ACTIONS(1261),
    [sym_char_literal] = ACTIONS(1261),
    [anon_sym_true] = ACTIONS(1263),
    [anon_sym_false] = ACTIONS(1263),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1263),
    [sym_super] = ACTIONS(1263),
    [sym_crate] = ACTIONS(1263),
    [sym_metavariable] = ACTIONS(1261),
    [sym__raw_string_literal_start] = ACTIONS(1261),
    [sym_float_literal] = ACTIONS(1261),
  },
  [372] = {
    [sym_line_comment] = STATE(372),
    [sym_block_comment] = STATE(372),
    [ts_builtin_sym_end] = ACTIONS(1265),
    [sym_identifier] = ACTIONS(1267),
    [anon_sym_SEMI] = ACTIONS(1265),
    [anon_sym_macro_rules_BANG] = ACTIONS(1265),
    [anon_sym_LPAREN] = ACTIONS(1265),
    [anon_sym_LBRACK] = ACTIONS(1265),
    [anon_sym_LBRACE] = ACTIONS(1265),
    [anon_sym_RBRACE] = ACTIONS(1265),
    [anon_sym_PLUS] = ACTIONS(1267),
    [anon_sym_STAR] = ACTIONS(1267),
    [anon_sym_QMARK] = ACTIONS(1265),
    [anon_sym_u8] = ACTIONS(1267),
    [anon_sym_i8] = ACTIONS(1267),
    [anon_sym_u16] = ACTIONS(1267),
    [anon_sym_i16] = ACTIONS(1267),
    [anon_sym_u32] = ACTIONS(1267),
    [anon_sym_i32] = ACTIONS(1267),
    [anon_sym_u64] = ACTIONS(1267),
    [anon_sym_i64] = ACTIONS(1267),
    [anon_sym_u128] = ACTIONS(1267),
    [anon_sym_i128] = ACTIONS(1267),
    [anon_sym_isize] = ACTIONS(1267),
    [anon_sym_usize] = ACTIONS(1267),
    [anon_sym_f32] = ACTIONS(1267),
    [anon_sym_f64] = ACTIONS(1267),
    [anon_sym_bool] = ACTIONS(1267),
    [anon_sym_str] = ACTIONS(1267),
    [anon_sym_char] = ACTIONS(1267),
    [anon_sym_DASH] = ACTIONS(1267),
    [anon_sym_SLASH] = ACTIONS(1267),
    [anon_sym_PERCENT] = ACTIONS(1267),
    [anon_sym_CARET] = ACTIONS(1267),
    [anon_sym_BANG] = ACTIONS(1267),
    [anon_sym_AMP] = ACTIONS(1267),
    [anon_sym_PIPE] = ACTIONS(1267),
    [anon_sym_AMP_AMP] = ACTIONS(1265),
    [anon_sym_PIPE_PIPE] = ACTIONS(1265),
    [anon_sym_LT_LT] = ACTIONS(1267),
    [anon_sym_GT_GT] = ACTIONS(1267),
    [anon_sym_PLUS_EQ] = ACTIONS(1265),
    [anon_sym_DASH_EQ] = ACTIONS(1265),
    [anon_sym_STAR_EQ] = ACTIONS(1265),
    [anon_sym_SLASH_EQ] = ACTIONS(1265),
    [anon_sym_PERCENT_EQ] = ACTIONS(1265),
    [anon_sym_CARET_EQ] = ACTIONS(1265),
    [anon_sym_AMP_EQ] = ACTIONS(1265),
    [anon_sym_PIPE_EQ] = ACTIONS(1265),
    [anon_sym_LT_LT_EQ] = ACTIONS(1265),
    [anon_sym_GT_GT_EQ] = ACTIONS(1265),
    [anon_sym_EQ] = ACTIONS(1267),
    [anon_sym_EQ_EQ] = ACTIONS(1265),
    [anon_sym_BANG_EQ] = ACTIONS(1265),
    [anon_sym_GT] = ACTIONS(1267),
    [anon_sym_LT] = ACTIONS(1267),
    [anon_sym_GT_EQ] = ACTIONS(1265),
    [anon_sym_LT_EQ] = ACTIONS(1265),
    [anon_sym_DOT] = ACTIONS(1267),
    [anon_sym_DOT_DOT] = ACTIONS(1267),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1265),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1265),
    [anon_sym_COLON_COLON] = ACTIONS(1265),
    [anon_sym_POUND] = ACTIONS(1265),
    [anon_sym_SQUOTE] = ACTIONS(1267),
    [anon_sym_as] = ACTIONS(1267),
    [anon_sym_async] = ACTIONS(1267),
    [anon_sym_break] = ACTIONS(1267),
    [anon_sym_const] = ACTIONS(1267),
    [anon_sym_continue] = ACTIONS(1267),
    [anon_sym_default] = ACTIONS(1267),
    [anon_sym_enum] = ACTIONS(1267),
    [anon_sym_fn] = ACTIONS(1267),
    [anon_sym_for] = ACTIONS(1267),
    [anon_sym_gen] = ACTIONS(1267),
    [anon_sym_if] = ACTIONS(1267),
    [anon_sym_impl] = ACTIONS(1267),
    [anon_sym_let] = ACTIONS(1267),
    [anon_sym_loop] = ACTIONS(1267),
    [anon_sym_match] = ACTIONS(1267),
    [anon_sym_mod] = ACTIONS(1267),
    [anon_sym_pub] = ACTIONS(1267),
    [anon_sym_return] = ACTIONS(1267),
    [anon_sym_static] = ACTIONS(1267),
    [anon_sym_struct] = ACTIONS(1267),
    [anon_sym_trait] = ACTIONS(1267),
    [anon_sym_type] = ACTIONS(1267),
    [anon_sym_union] = ACTIONS(1267),
    [anon_sym_unsafe] = ACTIONS(1267),
    [anon_sym_use] = ACTIONS(1267),
    [anon_sym_while] = ACTIONS(1267),
    [anon_sym_extern] = ACTIONS(1267),
    [anon_sym_else] = ACTIONS(1267),
    [anon_sym_yield] = ACTIONS(1267),
    [anon_sym_move] = ACTIONS(1267),
    [anon_sym_try] = ACTIONS(1267),
    [sym_integer_literal] = ACTIONS(1265),
    [aux_sym_string_literal_token1] = ACTIONS(1265),
    [sym_char_literal] = ACTIONS(1265),
    [anon_sym_true] = ACTIONS(1267),
    [anon_sym_false] = ACTIONS(1267),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1267),
    [sym_super] = ACTIONS(1267),
    [sym_crate] = ACTIONS(1267),
    [sym_metavariable] = ACTIONS(1265),
    [sym__raw_string_literal_start] = ACTIONS(1265),
    [sym_float_literal] = ACTIONS(1265),
  },
  [373] = {
    [sym_line_comment] = STATE(373),
    [sym_block_comment] = STATE(373),
    [ts_builtin_sym_end] = ACTIONS(1269),
    [sym_identifier] = ACTIONS(1271),
    [anon_sym_SEMI] = ACTIONS(1269),
    [anon_sym_macro_rules_BANG] = ACTIONS(1269),
    [anon_sym_LPAREN] = ACTIONS(1269),
    [anon_sym_LBRACK] = ACTIONS(1269),
    [anon_sym_LBRACE] = ACTIONS(1269),
    [anon_sym_RBRACE] = ACTIONS(1269),
    [anon_sym_PLUS] = ACTIONS(1271),
    [anon_sym_STAR] = ACTIONS(1271),
    [anon_sym_QMARK] = ACTIONS(1269),
    [anon_sym_u8] = ACTIONS(1271),
    [anon_sym_i8] = ACTIONS(1271),
    [anon_sym_u16] = ACTIONS(1271),
    [anon_sym_i16] = ACTIONS(1271),
    [anon_sym_u32] = ACTIONS(1271),
    [anon_sym_i32] = ACTIONS(1271),
    [anon_sym_u64] = ACTIONS(1271),
    [anon_sym_i64] = ACTIONS(1271),
    [anon_sym_u128] = ACTIONS(1271),
    [anon_sym_i128] = ACTIONS(1271),
    [anon_sym_isize] = ACTIONS(1271),
    [anon_sym_usize] = ACTIONS(1271),
    [anon_sym_f32] = ACTIONS(1271),
    [anon_sym_f64] = ACTIONS(1271),
    [anon_sym_bool] = ACTIONS(1271),
    [anon_sym_str] = ACTIONS(1271),
    [anon_sym_char] = ACTIONS(1271),
    [anon_sym_DASH] = ACTIONS(1271),
    [anon_sym_SLASH] = ACTIONS(1271),
    [anon_sym_PERCENT] = ACTIONS(1271),
    [anon_sym_CARET] = ACTIONS(1271),
    [anon_sym_BANG] = ACTIONS(1271),
    [anon_sym_AMP] = ACTIONS(1271),
    [anon_sym_PIPE] = ACTIONS(1271),
    [anon_sym_AMP_AMP] = ACTIONS(1269),
    [anon_sym_PIPE_PIPE] = ACTIONS(1269),
    [anon_sym_LT_LT] = ACTIONS(1271),
    [anon_sym_GT_GT] = ACTIONS(1271),
    [anon_sym_PLUS_EQ] = ACTIONS(1269),
    [anon_sym_DASH_EQ] = ACTIONS(1269),
    [anon_sym_STAR_EQ] = ACTIONS(1269),
    [anon_sym_SLASH_EQ] = ACTIONS(1269),
    [anon_sym_PERCENT_EQ] = ACTIONS(1269),
    [anon_sym_CARET_EQ] = ACTIONS(1269),
    [anon_sym_AMP_EQ] = ACTIONS(1269),
    [anon_sym_PIPE_EQ] = ACTIONS(1269),
    [anon_sym_LT_LT_EQ] = ACTIONS(1269),
    [anon_sym_GT_GT_EQ] = ACTIONS(1269),
    [anon_sym_EQ] = ACTIONS(1271),
    [anon_sym_EQ_EQ] = ACTIONS(1269),
    [anon_sym_BANG_EQ] = ACTIONS(1269),
    [anon_sym_GT] = ACTIONS(1271),
    [anon_sym_LT] = ACTIONS(1271),
    [anon_sym_GT_EQ] = ACTIONS(1269),
    [anon_sym_LT_EQ] = ACTIONS(1269),
    [anon_sym_DOT] = ACTIONS(1271),
    [anon_sym_DOT_DOT] = ACTIONS(1271),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1269),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1269),
    [anon_sym_COLON_COLON] = ACTIONS(1269),
    [anon_sym_POUND] = ACTIONS(1269),
    [anon_sym_SQUOTE] = ACTIONS(1271),
    [anon_sym_as] = ACTIONS(1271),
    [anon_sym_async] = ACTIONS(1271),
    [anon_sym_break] = ACTIONS(1271),
    [anon_sym_const] = ACTIONS(1271),
    [anon_sym_continue] = ACTIONS(1271),
    [anon_sym_default] = ACTIONS(1271),
    [anon_sym_enum] = ACTIONS(1271),
    [anon_sym_fn] = ACTIONS(1271),
    [anon_sym_for] = ACTIONS(1271),
    [anon_sym_gen] = ACTIONS(1271),
    [anon_sym_if] = ACTIONS(1271),
    [anon_sym_impl] = ACTIONS(1271),
    [anon_sym_let] = ACTIONS(1271),
    [anon_sym_loop] = ACTIONS(1271),
    [anon_sym_match] = ACTIONS(1271),
    [anon_sym_mod] = ACTIONS(1271),
    [anon_sym_pub] = ACTIONS(1271),
    [anon_sym_return] = ACTIONS(1271),
    [anon_sym_static] = ACTIONS(1271),
    [anon_sym_struct] = ACTIONS(1271),
    [anon_sym_trait] = ACTIONS(1271),
    [anon_sym_type] = ACTIONS(1271),
    [anon_sym_union] = ACTIONS(1271),
    [anon_sym_unsafe] = ACTIONS(1271),
    [anon_sym_use] = ACTIONS(1271),
    [anon_sym_while] = ACTIONS(1271),
    [anon_sym_extern] = ACTIONS(1271),
    [anon_sym_else] = ACTIONS(1271),
    [anon_sym_yield] = ACTIONS(1271),
    [anon_sym_move] = ACTIONS(1271),
    [anon_sym_try] = ACTIONS(1271),
    [sym_integer_literal] = ACTIONS(1269),
    [aux_sym_string_literal_token1] = ACTIONS(1269),
    [sym_char_literal] = ACTIONS(1269),
    [anon_sym_true] = ACTIONS(1271),
    [anon_sym_false] = ACTIONS(1271),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1271),
    [sym_super] = ACTIONS(1271),
    [sym_crate] = ACTIONS(1271),
    [sym_metavariable] = ACTIONS(1269),
    [sym__raw_string_literal_start] = ACTIONS(1269),
    [sym_float_literal] = ACTIONS(1269),
  },
  [374] = {
    [sym_line_comment] = STATE(374),
    [sym_block_comment] = STATE(374),
    [ts_builtin_sym_end] = ACTIONS(1273),
    [sym_identifier] = ACTIONS(1275),
    [anon_sym_SEMI] = ACTIONS(1273),
    [anon_sym_macro_rules_BANG] = ACTIONS(1273),
    [anon_sym_LPAREN] = ACTIONS(1273),
    [anon_sym_LBRACK] = ACTIONS(1273),
    [anon_sym_LBRACE] = ACTIONS(1273),
    [anon_sym_RBRACE] = ACTIONS(1273),
    [anon_sym_PLUS] = ACTIONS(1275),
    [anon_sym_STAR] = ACTIONS(1275),
    [anon_sym_QMARK] = ACTIONS(1273),
    [anon_sym_u8] = ACTIONS(1275),
    [anon_sym_i8] = ACTIONS(1275),
    [anon_sym_u16] = ACTIONS(1275),
    [anon_sym_i16] = ACTIONS(1275),
    [anon_sym_u32] = ACTIONS(1275),
    [anon_sym_i32] = ACTIONS(1275),
    [anon_sym_u64] = ACTIONS(1275),
    [anon_sym_i64] = ACTIONS(1275),
    [anon_sym_u128] = ACTIONS(1275),
    [anon_sym_i128] = ACTIONS(1275),
    [anon_sym_isize] = ACTIONS(1275),
    [anon_sym_usize] = ACTIONS(1275),
    [anon_sym_f32] = ACTIONS(1275),
    [anon_sym_f64] = ACTIONS(1275),
    [anon_sym_bool] = ACTIONS(1275),
    [anon_sym_str] = ACTIONS(1275),
    [anon_sym_char] = ACTIONS(1275),
    [anon_sym_DASH] = ACTIONS(1275),
    [anon_sym_SLASH] = ACTIONS(1275),
    [anon_sym_PERCENT] = ACTIONS(1275),
    [anon_sym_CARET] = ACTIONS(1275),
    [anon_sym_BANG] = ACTIONS(1275),
    [anon_sym_AMP] = ACTIONS(1275),
    [anon_sym_PIPE] = ACTIONS(1275),
    [anon_sym_AMP_AMP] = ACTIONS(1273),
    [anon_sym_PIPE_PIPE] = ACTIONS(1273),
    [anon_sym_LT_LT] = ACTIONS(1275),
    [anon_sym_GT_GT] = ACTIONS(1275),
    [anon_sym_PLUS_EQ] = ACTIONS(1273),
    [anon_sym_DASH_EQ] = ACTIONS(1273),
    [anon_sym_STAR_EQ] = ACTIONS(1273),
    [anon_sym_SLASH_EQ] = ACTIONS(1273),
    [anon_sym_PERCENT_EQ] = ACTIONS(1273),
    [anon_sym_CARET_EQ] = ACTIONS(1273),
    [anon_sym_AMP_EQ] = ACTIONS(1273),
    [anon_sym_PIPE_EQ] = ACTIONS(1273),
    [anon_sym_LT_LT_EQ] = ACTIONS(1273),
    [anon_sym_GT_GT_EQ] = ACTIONS(1273),
    [anon_sym_EQ] = ACTIONS(1275),
    [anon_sym_EQ_EQ] = ACTIONS(1273),
    [anon_sym_BANG_EQ] = ACTIONS(1273),
    [anon_sym_GT] = ACTIONS(1275),
    [anon_sym_LT] = ACTIONS(1275),
    [anon_sym_GT_EQ] = ACTIONS(1273),
    [anon_sym_LT_EQ] = ACTIONS(1273),
    [anon_sym_DOT] = ACTIONS(1275),
    [anon_sym_DOT_DOT] = ACTIONS(1275),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1273),
    [anon_sym_COLON_COLON] = ACTIONS(1273),
    [anon_sym_POUND] = ACTIONS(1273),
    [anon_sym_SQUOTE] = ACTIONS(1275),
    [anon_sym_as] = ACTIONS(1275),
    [anon_sym_async] = ACTIONS(1275),
    [anon_sym_break] = ACTIONS(1275),
    [anon_sym_const] = ACTIONS(1275),
    [anon_sym_continue] = ACTIONS(1275),
    [anon_sym_default] = ACTIONS(1275),
    [anon_sym_enum] = ACTIONS(1275),
    [anon_sym_fn] = ACTIONS(1275),
    [anon_sym_for] = ACTIONS(1275),
    [anon_sym_gen] = ACTIONS(1275),
    [anon_sym_if] = ACTIONS(1275),
    [anon_sym_impl] = ACTIONS(1275),
    [anon_sym_let] = ACTIONS(1275),
    [anon_sym_loop] = ACTIONS(1275),
    [anon_sym_match] = ACTIONS(1275),
    [anon_sym_mod] = ACTIONS(1275),
    [anon_sym_pub] = ACTIONS(1275),
    [anon_sym_return] = ACTIONS(1275),
    [anon_sym_static] = ACTIONS(1275),
    [anon_sym_struct] = ACTIONS(1275),
    [anon_sym_trait] = ACTIONS(1275),
    [anon_sym_type] = ACTIONS(1275),
    [anon_sym_union] = ACTIONS(1275),
    [anon_sym_unsafe] = ACTIONS(1275),
    [anon_sym_use] = ACTIONS(1275),
    [anon_sym_while] = ACTIONS(1275),
    [anon_sym_extern] = ACTIONS(1275),
    [anon_sym_else] = ACTIONS(1275),
    [anon_sym_yield] = ACTIONS(1275),
    [anon_sym_move] = ACTIONS(1275),
    [anon_sym_try] = ACTIONS(1275),
    [sym_integer_literal] = ACTIONS(1273),
    [aux_sym_string_literal_token1] = ACTIONS(1273),
    [sym_char_literal] = ACTIONS(1273),
    [anon_sym_true] = ACTIONS(1275),
    [anon_sym_false] = ACTIONS(1275),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1275),
    [sym_super] = ACTIONS(1275),
    [sym_crate] = ACTIONS(1275),
    [sym_metavariable] = ACTIONS(1273),
    [sym__raw_string_literal_start] = ACTIONS(1273),
    [sym_float_literal] = ACTIONS(1273),
  },
  [375] = {
    [sym_line_comment] = STATE(375),
    [sym_block_comment] = STATE(375),
    [ts_builtin_sym_end] = ACTIONS(1277),
    [sym_identifier] = ACTIONS(1279),
    [anon_sym_SEMI] = ACTIONS(1277),
    [anon_sym_macro_rules_BANG] = ACTIONS(1277),
    [anon_sym_LPAREN] = ACTIONS(1277),
    [anon_sym_LBRACK] = ACTIONS(1277),
    [anon_sym_LBRACE] = ACTIONS(1277),
    [anon_sym_RBRACE] = ACTIONS(1277),
    [anon_sym_PLUS] = ACTIONS(1279),
    [anon_sym_STAR] = ACTIONS(1279),
    [anon_sym_QMARK] = ACTIONS(1277),
    [anon_sym_u8] = ACTIONS(1279),
    [anon_sym_i8] = ACTIONS(1279),
    [anon_sym_u16] = ACTIONS(1279),
    [anon_sym_i16] = ACTIONS(1279),
    [anon_sym_u32] = ACTIONS(1279),
    [anon_sym_i32] = ACTIONS(1279),
    [anon_sym_u64] = ACTIONS(1279),
    [anon_sym_i64] = ACTIONS(1279),
    [anon_sym_u128] = ACTIONS(1279),
    [anon_sym_i128] = ACTIONS(1279),
    [anon_sym_isize] = ACTIONS(1279),
    [anon_sym_usize] = ACTIONS(1279),
    [anon_sym_f32] = ACTIONS(1279),
    [anon_sym_f64] = ACTIONS(1279),
    [anon_sym_bool] = ACTIONS(1279),
    [anon_sym_str] = ACTIONS(1279),
    [anon_sym_char] = ACTIONS(1279),
    [anon_sym_DASH] = ACTIONS(1279),
    [anon_sym_SLASH] = ACTIONS(1279),
    [anon_sym_PERCENT] = ACTIONS(1279),
    [anon_sym_CARET] = ACTIONS(1279),
    [anon_sym_BANG] = ACTIONS(1279),
    [anon_sym_AMP] = ACTIONS(1279),
    [anon_sym_PIPE] = ACTIONS(1279),
    [anon_sym_AMP_AMP] = ACTIONS(1277),
    [anon_sym_PIPE_PIPE] = ACTIONS(1277),
    [anon_sym_LT_LT] = ACTIONS(1279),
    [anon_sym_GT_GT] = ACTIONS(1279),
    [anon_sym_PLUS_EQ] = ACTIONS(1277),
    [anon_sym_DASH_EQ] = ACTIONS(1277),
    [anon_sym_STAR_EQ] = ACTIONS(1277),
    [anon_sym_SLASH_EQ] = ACTIONS(1277),
    [anon_sym_PERCENT_EQ] = ACTIONS(1277),
    [anon_sym_CARET_EQ] = ACTIONS(1277),
    [anon_sym_AMP_EQ] = ACTIONS(1277),
    [anon_sym_PIPE_EQ] = ACTIONS(1277),
    [anon_sym_LT_LT_EQ] = ACTIONS(1277),
    [anon_sym_GT_GT_EQ] = ACTIONS(1277),
    [anon_sym_EQ] = ACTIONS(1279),
    [anon_sym_EQ_EQ] = ACTIONS(1277),
    [anon_sym_BANG_EQ] = ACTIONS(1277),
    [anon_sym_GT] = ACTIONS(1279),
    [anon_sym_LT] = ACTIONS(1279),
    [anon_sym_GT_EQ] = ACTIONS(1277),
    [anon_sym_LT_EQ] = ACTIONS(1277),
    [anon_sym_DOT] = ACTIONS(1279),
    [anon_sym_DOT_DOT] = ACTIONS(1279),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1277),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1277),
    [anon_sym_COLON_COLON] = ACTIONS(1277),
    [anon_sym_POUND] = ACTIONS(1277),
    [anon_sym_SQUOTE] = ACTIONS(1279),
    [anon_sym_as] = ACTIONS(1279),
    [anon_sym_async] = ACTIONS(1279),
    [anon_sym_break] = ACTIONS(1279),
    [anon_sym_const] = ACTIONS(1279),
    [anon_sym_continue] = ACTIONS(1279),
    [anon_sym_default] = ACTIONS(1279),
    [anon_sym_enum] = ACTIONS(1279),
    [anon_sym_fn] = ACTIONS(1279),
    [anon_sym_for] = ACTIONS(1279),
    [anon_sym_gen] = ACTIONS(1279),
    [anon_sym_if] = ACTIONS(1279),
    [anon_sym_impl] = ACTIONS(1279),
    [anon_sym_let] = ACTIONS(1279),
    [anon_sym_loop] = ACTIONS(1279),
    [anon_sym_match] = ACTIONS(1279),
    [anon_sym_mod] = ACTIONS(1279),
    [anon_sym_pub] = ACTIONS(1279),
    [anon_sym_return] = ACTIONS(1279),
    [anon_sym_static] = ACTIONS(1279),
    [anon_sym_struct] = ACTIONS(1279),
    [anon_sym_trait] = ACTIONS(1279),
    [anon_sym_type] = ACTIONS(1279),
    [anon_sym_union] = ACTIONS(1279),
    [anon_sym_unsafe] = ACTIONS(1279),
    [anon_sym_use] = ACTIONS(1279),
    [anon_sym_while] = ACTIONS(1279),
    [anon_sym_extern] = ACTIONS(1279),
    [anon_sym_yield] = ACTIONS(1279),
    [anon_sym_move] = ACTIONS(1279),
    [anon_sym_try] = ACTIONS(1279),
    [sym_integer_literal] = ACTIONS(1277),
    [aux_sym_string_literal_token1] = ACTIONS(1277),
    [sym_char_literal] = ACTIONS(1277),
    [anon_sym_true] = ACTIONS(1279),
    [anon_sym_false] = ACTIONS(1279),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1279),
    [sym_super] = ACTIONS(1279),
    [sym_crate] = ACTIONS(1279),
    [sym_metavariable] = ACTIONS(1277),
    [sym__raw_string_literal_start] = ACTIONS(1277),
    [sym_float_literal] = ACTIONS(1277),
  },
  [376] = {
    [sym_attribute_item] = STATE(420),
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_self_parameter] = STATE(2999),
    [sym_variadic_parameter] = STATE(2999),
    [sym_parameter] = STATE(2999),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2746),
    [sym_bracketed_type] = STATE(3550),
    [sym_lifetime] = STATE(2800),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3221),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(2410),
    [sym_scoped_identifier] = STATE(2242),
    [sym_scoped_type_identifier] = STATE(2118),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(3256),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(376),
    [sym_block_comment] = STATE(376),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1281),
    [anon_sym_LPAREN] = ACTIONS(1283),
    [anon_sym_RPAREN] = ACTIONS(1285),
    [anon_sym_LBRACK] = ACTIONS(1287),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1293),
    [anon_sym_i8] = ACTIONS(1293),
    [anon_sym_u16] = ACTIONS(1293),
    [anon_sym_i16] = ACTIONS(1293),
    [anon_sym_u32] = ACTIONS(1293),
    [anon_sym_i32] = ACTIONS(1293),
    [anon_sym_u64] = ACTIONS(1293),
    [anon_sym_i64] = ACTIONS(1293),
    [anon_sym_u128] = ACTIONS(1293),
    [anon_sym_i128] = ACTIONS(1293),
    [anon_sym_isize] = ACTIONS(1293),
    [anon_sym_usize] = ACTIONS(1293),
    [anon_sym_f32] = ACTIONS(1293),
    [anon_sym_f64] = ACTIONS(1293),
    [anon_sym_bool] = ACTIONS(1293),
    [anon_sym_str] = ACTIONS(1293),
    [anon_sym_char] = ACTIONS(1293),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1299),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1303),
    [anon_sym_DOT_DOT] = ACTIONS(1305),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1307),
    [anon_sym_COMMA] = ACTIONS(1309),
    [anon_sym_COLON_COLON] = ACTIONS(1311),
    [anon_sym_POUND] = ACTIONS(1313),
    [anon_sym_SQUOTE] = ACTIONS(1315),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1319),
    [anon_sym_default] = ACTIONS(1321),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1327),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1327),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_ref] = ACTIONS(1333),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_mutable_specifier] = ACTIONS(1337),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1345),
    [sym_super] = ACTIONS(1347),
    [sym_crate] = ACTIONS(1347),
    [sym_metavariable] = ACTIONS(1349),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [377] = {
    [sym_line_comment] = STATE(377),
    [sym_block_comment] = STATE(377),
    [ts_builtin_sym_end] = ACTIONS(1353),
    [sym_identifier] = ACTIONS(1355),
    [anon_sym_SEMI] = ACTIONS(1353),
    [anon_sym_macro_rules_BANG] = ACTIONS(1353),
    [anon_sym_LPAREN] = ACTIONS(1353),
    [anon_sym_LBRACK] = ACTIONS(1353),
    [anon_sym_LBRACE] = ACTIONS(1353),
    [anon_sym_RBRACE] = ACTIONS(1353),
    [anon_sym_PLUS] = ACTIONS(1357),
    [anon_sym_STAR] = ACTIONS(1355),
    [anon_sym_QMARK] = ACTIONS(1359),
    [anon_sym_u8] = ACTIONS(1355),
    [anon_sym_i8] = ACTIONS(1355),
    [anon_sym_u16] = ACTIONS(1355),
    [anon_sym_i16] = ACTIONS(1355),
    [anon_sym_u32] = ACTIONS(1355),
    [anon_sym_i32] = ACTIONS(1355),
    [anon_sym_u64] = ACTIONS(1355),
    [anon_sym_i64] = ACTIONS(1355),
    [anon_sym_u128] = ACTIONS(1355),
    [anon_sym_i128] = ACTIONS(1355),
    [anon_sym_isize] = ACTIONS(1355),
    [anon_sym_usize] = ACTIONS(1355),
    [anon_sym_f32] = ACTIONS(1355),
    [anon_sym_f64] = ACTIONS(1355),
    [anon_sym_bool] = ACTIONS(1355),
    [anon_sym_str] = ACTIONS(1355),
    [anon_sym_char] = ACTIONS(1355),
    [anon_sym_DASH] = ACTIONS(1355),
    [anon_sym_SLASH] = ACTIONS(1357),
    [anon_sym_PERCENT] = ACTIONS(1357),
    [anon_sym_CARET] = ACTIONS(1357),
    [anon_sym_BANG] = ACTIONS(1355),
    [anon_sym_AMP] = ACTIONS(1355),
    [anon_sym_PIPE] = ACTIONS(1355),
    [anon_sym_AMP_AMP] = ACTIONS(1359),
    [anon_sym_PIPE_PIPE] = ACTIONS(1359),
    [anon_sym_LT_LT] = ACTIONS(1357),
    [anon_sym_GT_GT] = ACTIONS(1357),
    [anon_sym_PLUS_EQ] = ACTIONS(1359),
    [anon_sym_DASH_EQ] = ACTIONS(1359),
    [anon_sym_STAR_EQ] = ACTIONS(1359),
    [anon_sym_SLASH_EQ] = ACTIONS(1359),
    [anon_sym_PERCENT_EQ] = ACTIONS(1359),
    [anon_sym_CARET_EQ] = ACTIONS(1359),
    [anon_sym_AMP_EQ] = ACTIONS(1359),
    [anon_sym_PIPE_EQ] = ACTIONS(1359),
    [anon_sym_LT_LT_EQ] = ACTIONS(1359),
    [anon_sym_GT_GT_EQ] = ACTIONS(1359),
    [anon_sym_EQ] = ACTIONS(1357),
    [anon_sym_EQ_EQ] = ACTIONS(1359),
    [anon_sym_BANG_EQ] = ACTIONS(1359),
    [anon_sym_GT] = ACTIONS(1357),
    [anon_sym_LT] = ACTIONS(1355),
    [anon_sym_GT_EQ] = ACTIONS(1359),
    [anon_sym_LT_EQ] = ACTIONS(1359),
    [anon_sym_DOT] = ACTIONS(1357),
    [anon_sym_DOT_DOT] = ACTIONS(1355),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1359),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1359),
    [anon_sym_COLON_COLON] = ACTIONS(1353),
    [anon_sym_POUND] = ACTIONS(1353),
    [anon_sym_SQUOTE] = ACTIONS(1355),
    [anon_sym_as] = ACTIONS(1357),
    [anon_sym_async] = ACTIONS(1355),
    [anon_sym_break] = ACTIONS(1355),
    [anon_sym_const] = ACTIONS(1355),
    [anon_sym_continue] = ACTIONS(1355),
    [anon_sym_default] = ACTIONS(1355),
    [anon_sym_enum] = ACTIONS(1355),
    [anon_sym_fn] = ACTIONS(1355),
    [anon_sym_for] = ACTIONS(1355),
    [anon_sym_gen] = ACTIONS(1355),
    [anon_sym_if] = ACTIONS(1355),
    [anon_sym_impl] = ACTIONS(1355),
    [anon_sym_let] = ACTIONS(1355),
    [anon_sym_loop] = ACTIONS(1355),
    [anon_sym_match] = ACTIONS(1355),
    [anon_sym_mod] = ACTIONS(1355),
    [anon_sym_pub] = ACTIONS(1355),
    [anon_sym_return] = ACTIONS(1355),
    [anon_sym_static] = ACTIONS(1355),
    [anon_sym_struct] = ACTIONS(1355),
    [anon_sym_trait] = ACTIONS(1355),
    [anon_sym_type] = ACTIONS(1355),
    [anon_sym_union] = ACTIONS(1355),
    [anon_sym_unsafe] = ACTIONS(1355),
    [anon_sym_use] = ACTIONS(1355),
    [anon_sym_while] = ACTIONS(1355),
    [anon_sym_extern] = ACTIONS(1355),
    [anon_sym_yield] = ACTIONS(1355),
    [anon_sym_move] = ACTIONS(1355),
    [anon_sym_try] = ACTIONS(1355),
    [sym_integer_literal] = ACTIONS(1353),
    [aux_sym_string_literal_token1] = ACTIONS(1353),
    [sym_char_literal] = ACTIONS(1353),
    [anon_sym_true] = ACTIONS(1355),
    [anon_sym_false] = ACTIONS(1355),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1355),
    [sym_super] = ACTIONS(1355),
    [sym_crate] = ACTIONS(1355),
    [sym_metavariable] = ACTIONS(1353),
    [sym__raw_string_literal_start] = ACTIONS(1353),
    [sym_float_literal] = ACTIONS(1353),
  },
  [378] = {
    [sym_line_comment] = STATE(378),
    [sym_block_comment] = STATE(378),
    [ts_builtin_sym_end] = ACTIONS(1361),
    [sym_identifier] = ACTIONS(1363),
    [anon_sym_SEMI] = ACTIONS(1361),
    [anon_sym_macro_rules_BANG] = ACTIONS(1361),
    [anon_sym_LPAREN] = ACTIONS(1361),
    [anon_sym_LBRACK] = ACTIONS(1361),
    [anon_sym_LBRACE] = ACTIONS(1361),
    [anon_sym_RBRACE] = ACTIONS(1361),
    [anon_sym_PLUS] = ACTIONS(1363),
    [anon_sym_STAR] = ACTIONS(1363),
    [anon_sym_QMARK] = ACTIONS(1361),
    [anon_sym_u8] = ACTIONS(1363),
    [anon_sym_i8] = ACTIONS(1363),
    [anon_sym_u16] = ACTIONS(1363),
    [anon_sym_i16] = ACTIONS(1363),
    [anon_sym_u32] = ACTIONS(1363),
    [anon_sym_i32] = ACTIONS(1363),
    [anon_sym_u64] = ACTIONS(1363),
    [anon_sym_i64] = ACTIONS(1363),
    [anon_sym_u128] = ACTIONS(1363),
    [anon_sym_i128] = ACTIONS(1363),
    [anon_sym_isize] = ACTIONS(1363),
    [anon_sym_usize] = ACTIONS(1363),
    [anon_sym_f32] = ACTIONS(1363),
    [anon_sym_f64] = ACTIONS(1363),
    [anon_sym_bool] = ACTIONS(1363),
    [anon_sym_str] = ACTIONS(1363),
    [anon_sym_char] = ACTIONS(1363),
    [anon_sym_DASH] = ACTIONS(1363),
    [anon_sym_SLASH] = ACTIONS(1363),
    [anon_sym_PERCENT] = ACTIONS(1363),
    [anon_sym_CARET] = ACTIONS(1363),
    [anon_sym_BANG] = ACTIONS(1363),
    [anon_sym_AMP] = ACTIONS(1363),
    [anon_sym_PIPE] = ACTIONS(1363),
    [anon_sym_AMP_AMP] = ACTIONS(1361),
    [anon_sym_PIPE_PIPE] = ACTIONS(1361),
    [anon_sym_LT_LT] = ACTIONS(1363),
    [anon_sym_GT_GT] = ACTIONS(1363),
    [anon_sym_PLUS_EQ] = ACTIONS(1361),
    [anon_sym_DASH_EQ] = ACTIONS(1361),
    [anon_sym_STAR_EQ] = ACTIONS(1361),
    [anon_sym_SLASH_EQ] = ACTIONS(1361),
    [anon_sym_PERCENT_EQ] = ACTIONS(1361),
    [anon_sym_CARET_EQ] = ACTIONS(1361),
    [anon_sym_AMP_EQ] = ACTIONS(1361),
    [anon_sym_PIPE_EQ] = ACTIONS(1361),
    [anon_sym_LT_LT_EQ] = ACTIONS(1361),
    [anon_sym_GT_GT_EQ] = ACTIONS(1361),
    [anon_sym_EQ] = ACTIONS(1363),
    [anon_sym_EQ_EQ] = ACTIONS(1361),
    [anon_sym_BANG_EQ] = ACTIONS(1361),
    [anon_sym_GT] = ACTIONS(1363),
    [anon_sym_LT] = ACTIONS(1363),
    [anon_sym_GT_EQ] = ACTIONS(1361),
    [anon_sym_LT_EQ] = ACTIONS(1361),
    [anon_sym_DOT] = ACTIONS(1363),
    [anon_sym_DOT_DOT] = ACTIONS(1363),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1361),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1361),
    [anon_sym_COLON_COLON] = ACTIONS(1361),
    [anon_sym_POUND] = ACTIONS(1361),
    [anon_sym_SQUOTE] = ACTIONS(1363),
    [anon_sym_as] = ACTIONS(1363),
    [anon_sym_async] = ACTIONS(1363),
    [anon_sym_break] = ACTIONS(1363),
    [anon_sym_const] = ACTIONS(1363),
    [anon_sym_continue] = ACTIONS(1363),
    [anon_sym_default] = ACTIONS(1363),
    [anon_sym_enum] = ACTIONS(1363),
    [anon_sym_fn] = ACTIONS(1363),
    [anon_sym_for] = ACTIONS(1363),
    [anon_sym_gen] = ACTIONS(1363),
    [anon_sym_if] = ACTIONS(1363),
    [anon_sym_impl] = ACTIONS(1363),
    [anon_sym_let] = ACTIONS(1363),
    [anon_sym_loop] = ACTIONS(1363),
    [anon_sym_match] = ACTIONS(1363),
    [anon_sym_mod] = ACTIONS(1363),
    [anon_sym_pub] = ACTIONS(1363),
    [anon_sym_return] = ACTIONS(1363),
    [anon_sym_static] = ACTIONS(1363),
    [anon_sym_struct] = ACTIONS(1363),
    [anon_sym_trait] = ACTIONS(1363),
    [anon_sym_type] = ACTIONS(1363),
    [anon_sym_union] = ACTIONS(1363),
    [anon_sym_unsafe] = ACTIONS(1363),
    [anon_sym_use] = ACTIONS(1363),
    [anon_sym_while] = ACTIONS(1363),
    [anon_sym_extern] = ACTIONS(1363),
    [anon_sym_yield] = ACTIONS(1363),
    [anon_sym_move] = ACTIONS(1363),
    [anon_sym_try] = ACTIONS(1363),
    [sym_integer_literal] = ACTIONS(1361),
    [aux_sym_string_literal_token1] = ACTIONS(1361),
    [sym_char_literal] = ACTIONS(1361),
    [anon_sym_true] = ACTIONS(1363),
    [anon_sym_false] = ACTIONS(1363),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1363),
    [sym_super] = ACTIONS(1363),
    [sym_crate] = ACTIONS(1363),
    [sym_metavariable] = ACTIONS(1361),
    [sym__raw_string_literal_start] = ACTIONS(1361),
    [sym_float_literal] = ACTIONS(1361),
  },
  [379] = {
    [sym_line_comment] = STATE(379),
    [sym_block_comment] = STATE(379),
    [ts_builtin_sym_end] = ACTIONS(1365),
    [sym_identifier] = ACTIONS(1367),
    [anon_sym_SEMI] = ACTIONS(1365),
    [anon_sym_macro_rules_BANG] = ACTIONS(1365),
    [anon_sym_LPAREN] = ACTIONS(1365),
    [anon_sym_LBRACK] = ACTIONS(1365),
    [anon_sym_LBRACE] = ACTIONS(1365),
    [anon_sym_RBRACE] = ACTIONS(1365),
    [anon_sym_PLUS] = ACTIONS(1367),
    [anon_sym_STAR] = ACTIONS(1367),
    [anon_sym_QMARK] = ACTIONS(1365),
    [anon_sym_u8] = ACTIONS(1367),
    [anon_sym_i8] = ACTIONS(1367),
    [anon_sym_u16] = ACTIONS(1367),
    [anon_sym_i16] = ACTIONS(1367),
    [anon_sym_u32] = ACTIONS(1367),
    [anon_sym_i32] = ACTIONS(1367),
    [anon_sym_u64] = ACTIONS(1367),
    [anon_sym_i64] = ACTIONS(1367),
    [anon_sym_u128] = ACTIONS(1367),
    [anon_sym_i128] = ACTIONS(1367),
    [anon_sym_isize] = ACTIONS(1367),
    [anon_sym_usize] = ACTIONS(1367),
    [anon_sym_f32] = ACTIONS(1367),
    [anon_sym_f64] = ACTIONS(1367),
    [anon_sym_bool] = ACTIONS(1367),
    [anon_sym_str] = ACTIONS(1367),
    [anon_sym_char] = ACTIONS(1367),
    [anon_sym_DASH] = ACTIONS(1367),
    [anon_sym_SLASH] = ACTIONS(1367),
    [anon_sym_PERCENT] = ACTIONS(1367),
    [anon_sym_CARET] = ACTIONS(1367),
    [anon_sym_BANG] = ACTIONS(1367),
    [anon_sym_AMP] = ACTIONS(1367),
    [anon_sym_PIPE] = ACTIONS(1367),
    [anon_sym_AMP_AMP] = ACTIONS(1365),
    [anon_sym_PIPE_PIPE] = ACTIONS(1365),
    [anon_sym_LT_LT] = ACTIONS(1367),
    [anon_sym_GT_GT] = ACTIONS(1367),
    [anon_sym_PLUS_EQ] = ACTIONS(1365),
    [anon_sym_DASH_EQ] = ACTIONS(1365),
    [anon_sym_STAR_EQ] = ACTIONS(1365),
    [anon_sym_SLASH_EQ] = ACTIONS(1365),
    [anon_sym_PERCENT_EQ] = ACTIONS(1365),
    [anon_sym_CARET_EQ] = ACTIONS(1365),
    [anon_sym_AMP_EQ] = ACTIONS(1365),
    [anon_sym_PIPE_EQ] = ACTIONS(1365),
    [anon_sym_LT_LT_EQ] = ACTIONS(1365),
    [anon_sym_GT_GT_EQ] = ACTIONS(1365),
    [anon_sym_EQ] = ACTIONS(1367),
    [anon_sym_EQ_EQ] = ACTIONS(1365),
    [anon_sym_BANG_EQ] = ACTIONS(1365),
    [anon_sym_GT] = ACTIONS(1367),
    [anon_sym_LT] = ACTIONS(1367),
    [anon_sym_GT_EQ] = ACTIONS(1365),
    [anon_sym_LT_EQ] = ACTIONS(1365),
    [anon_sym_DOT] = ACTIONS(1367),
    [anon_sym_DOT_DOT] = ACTIONS(1367),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1365),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1365),
    [anon_sym_COLON_COLON] = ACTIONS(1365),
    [anon_sym_POUND] = ACTIONS(1365),
    [anon_sym_SQUOTE] = ACTIONS(1367),
    [anon_sym_as] = ACTIONS(1367),
    [anon_sym_async] = ACTIONS(1367),
    [anon_sym_break] = ACTIONS(1367),
    [anon_sym_const] = ACTIONS(1367),
    [anon_sym_continue] = ACTIONS(1367),
    [anon_sym_default] = ACTIONS(1367),
    [anon_sym_enum] = ACTIONS(1367),
    [anon_sym_fn] = ACTIONS(1367),
    [anon_sym_for] = ACTIONS(1367),
    [anon_sym_gen] = ACTIONS(1367),
    [anon_sym_if] = ACTIONS(1367),
    [anon_sym_impl] = ACTIONS(1367),
    [anon_sym_let] = ACTIONS(1367),
    [anon_sym_loop] = ACTIONS(1367),
    [anon_sym_match] = ACTIONS(1367),
    [anon_sym_mod] = ACTIONS(1367),
    [anon_sym_pub] = ACTIONS(1367),
    [anon_sym_return] = ACTIONS(1367),
    [anon_sym_static] = ACTIONS(1367),
    [anon_sym_struct] = ACTIONS(1367),
    [anon_sym_trait] = ACTIONS(1367),
    [anon_sym_type] = ACTIONS(1367),
    [anon_sym_union] = ACTIONS(1367),
    [anon_sym_unsafe] = ACTIONS(1367),
    [anon_sym_use] = ACTIONS(1367),
    [anon_sym_while] = ACTIONS(1367),
    [anon_sym_extern] = ACTIONS(1367),
    [anon_sym_yield] = ACTIONS(1367),
    [anon_sym_move] = ACTIONS(1367),
    [anon_sym_try] = ACTIONS(1367),
    [sym_integer_literal] = ACTIONS(1365),
    [aux_sym_string_literal_token1] = ACTIONS(1365),
    [sym_char_literal] = ACTIONS(1365),
    [anon_sym_true] = ACTIONS(1367),
    [anon_sym_false] = ACTIONS(1367),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1367),
    [sym_super] = ACTIONS(1367),
    [sym_crate] = ACTIONS(1367),
    [sym_metavariable] = ACTIONS(1365),
    [sym__raw_string_literal_start] = ACTIONS(1365),
    [sym_float_literal] = ACTIONS(1365),
  },
  [380] = {
    [sym_attribute_item] = STATE(420),
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_self_parameter] = STATE(2999),
    [sym_variadic_parameter] = STATE(2999),
    [sym_parameter] = STATE(2999),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2746),
    [sym_bracketed_type] = STATE(3557),
    [sym_lifetime] = STATE(2800),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3301),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(2446),
    [sym_scoped_identifier] = STATE(2125),
    [sym_scoped_type_identifier] = STATE(2118),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2431),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(380),
    [sym_block_comment] = STATE(380),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1369),
    [anon_sym_LPAREN] = ACTIONS(1371),
    [anon_sym_RPAREN] = ACTIONS(1373),
    [anon_sym_LBRACK] = ACTIONS(1287),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1375),
    [anon_sym_i8] = ACTIONS(1375),
    [anon_sym_u16] = ACTIONS(1375),
    [anon_sym_i16] = ACTIONS(1375),
    [anon_sym_u32] = ACTIONS(1375),
    [anon_sym_i32] = ACTIONS(1375),
    [anon_sym_u64] = ACTIONS(1375),
    [anon_sym_i64] = ACTIONS(1375),
    [anon_sym_u128] = ACTIONS(1375),
    [anon_sym_i128] = ACTIONS(1375),
    [anon_sym_isize] = ACTIONS(1375),
    [anon_sym_usize] = ACTIONS(1375),
    [anon_sym_f32] = ACTIONS(1375),
    [anon_sym_f64] = ACTIONS(1375),
    [anon_sym_bool] = ACTIONS(1375),
    [anon_sym_str] = ACTIONS(1375),
    [anon_sym_char] = ACTIONS(1375),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1377),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1379),
    [anon_sym_DOT_DOT] = ACTIONS(1305),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1307),
    [anon_sym_COMMA] = ACTIONS(1381),
    [anon_sym_COLON_COLON] = ACTIONS(1383),
    [anon_sym_POUND] = ACTIONS(1313),
    [anon_sym_SQUOTE] = ACTIONS(1315),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1319),
    [anon_sym_default] = ACTIONS(1385),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1387),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1387),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_ref] = ACTIONS(1333),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_mutable_specifier] = ACTIONS(1337),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1389),
    [sym_super] = ACTIONS(1391),
    [sym_crate] = ACTIONS(1391),
    [sym_metavariable] = ACTIONS(1393),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [381] = {
    [sym_line_comment] = STATE(381),
    [sym_block_comment] = STATE(381),
    [ts_builtin_sym_end] = ACTIONS(1395),
    [sym_identifier] = ACTIONS(1397),
    [anon_sym_SEMI] = ACTIONS(1395),
    [anon_sym_macro_rules_BANG] = ACTIONS(1395),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_LBRACK] = ACTIONS(1395),
    [anon_sym_LBRACE] = ACTIONS(1395),
    [anon_sym_RBRACE] = ACTIONS(1395),
    [anon_sym_PLUS] = ACTIONS(1397),
    [anon_sym_STAR] = ACTIONS(1397),
    [anon_sym_QMARK] = ACTIONS(1395),
    [anon_sym_u8] = ACTIONS(1397),
    [anon_sym_i8] = ACTIONS(1397),
    [anon_sym_u16] = ACTIONS(1397),
    [anon_sym_i16] = ACTIONS(1397),
    [anon_sym_u32] = ACTIONS(1397),
    [anon_sym_i32] = ACTIONS(1397),
    [anon_sym_u64] = ACTIONS(1397),
    [anon_sym_i64] = ACTIONS(1397),
    [anon_sym_u128] = ACTIONS(1397),
    [anon_sym_i128] = ACTIONS(1397),
    [anon_sym_isize] = ACTIONS(1397),
    [anon_sym_usize] = ACTIONS(1397),
    [anon_sym_f32] = ACTIONS(1397),
    [anon_sym_f64] = ACTIONS(1397),
    [anon_sym_bool] = ACTIONS(1397),
    [anon_sym_str] = ACTIONS(1397),
    [anon_sym_char] = ACTIONS(1397),
    [anon_sym_DASH] = ACTIONS(1397),
    [anon_sym_SLASH] = ACTIONS(1397),
    [anon_sym_PERCENT] = ACTIONS(1397),
    [anon_sym_CARET] = ACTIONS(1397),
    [anon_sym_BANG] = ACTIONS(1397),
    [anon_sym_AMP] = ACTIONS(1397),
    [anon_sym_PIPE] = ACTIONS(1397),
    [anon_sym_AMP_AMP] = ACTIONS(1395),
    [anon_sym_PIPE_PIPE] = ACTIONS(1395),
    [anon_sym_LT_LT] = ACTIONS(1397),
    [anon_sym_GT_GT] = ACTIONS(1397),
    [anon_sym_PLUS_EQ] = ACTIONS(1395),
    [anon_sym_DASH_EQ] = ACTIONS(1395),
    [anon_sym_STAR_EQ] = ACTIONS(1395),
    [anon_sym_SLASH_EQ] = ACTIONS(1395),
    [anon_sym_PERCENT_EQ] = ACTIONS(1395),
    [anon_sym_CARET_EQ] = ACTIONS(1395),
    [anon_sym_AMP_EQ] = ACTIONS(1395),
    [anon_sym_PIPE_EQ] = ACTIONS(1395),
    [anon_sym_LT_LT_EQ] = ACTIONS(1395),
    [anon_sym_GT_GT_EQ] = ACTIONS(1395),
    [anon_sym_EQ] = ACTIONS(1397),
    [anon_sym_EQ_EQ] = ACTIONS(1395),
    [anon_sym_BANG_EQ] = ACTIONS(1395),
    [anon_sym_GT] = ACTIONS(1397),
    [anon_sym_LT] = ACTIONS(1397),
    [anon_sym_GT_EQ] = ACTIONS(1395),
    [anon_sym_LT_EQ] = ACTIONS(1395),
    [anon_sym_DOT] = ACTIONS(1397),
    [anon_sym_DOT_DOT] = ACTIONS(1397),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1395),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1395),
    [anon_sym_POUND] = ACTIONS(1395),
    [anon_sym_SQUOTE] = ACTIONS(1397),
    [anon_sym_as] = ACTIONS(1397),
    [anon_sym_async] = ACTIONS(1397),
    [anon_sym_break] = ACTIONS(1397),
    [anon_sym_const] = ACTIONS(1397),
    [anon_sym_continue] = ACTIONS(1397),
    [anon_sym_default] = ACTIONS(1397),
    [anon_sym_enum] = ACTIONS(1397),
    [anon_sym_fn] = ACTIONS(1397),
    [anon_sym_for] = ACTIONS(1397),
    [anon_sym_gen] = ACTIONS(1397),
    [anon_sym_if] = ACTIONS(1397),
    [anon_sym_impl] = ACTIONS(1397),
    [anon_sym_let] = ACTIONS(1397),
    [anon_sym_loop] = ACTIONS(1397),
    [anon_sym_match] = ACTIONS(1397),
    [anon_sym_mod] = ACTIONS(1397),
    [anon_sym_pub] = ACTIONS(1397),
    [anon_sym_return] = ACTIONS(1397),
    [anon_sym_static] = ACTIONS(1397),
    [anon_sym_struct] = ACTIONS(1397),
    [anon_sym_trait] = ACTIONS(1397),
    [anon_sym_type] = ACTIONS(1397),
    [anon_sym_union] = ACTIONS(1397),
    [anon_sym_unsafe] = ACTIONS(1397),
    [anon_sym_use] = ACTIONS(1397),
    [anon_sym_while] = ACTIONS(1397),
    [anon_sym_extern] = ACTIONS(1397),
    [anon_sym_yield] = ACTIONS(1397),
    [anon_sym_move] = ACTIONS(1397),
    [anon_sym_try] = ACTIONS(1397),
    [sym_integer_literal] = ACTIONS(1395),
    [aux_sym_string_literal_token1] = ACTIONS(1395),
    [sym_char_literal] = ACTIONS(1395),
    [anon_sym_true] = ACTIONS(1397),
    [anon_sym_false] = ACTIONS(1397),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1397),
    [sym_super] = ACTIONS(1397),
    [sym_crate] = ACTIONS(1397),
    [sym_metavariable] = ACTIONS(1395),
    [sym__raw_string_literal_start] = ACTIONS(1395),
    [sym_float_literal] = ACTIONS(1395),
  },
  [382] = {
    [sym_line_comment] = STATE(382),
    [sym_block_comment] = STATE(382),
    [ts_builtin_sym_end] = ACTIONS(1399),
    [sym_identifier] = ACTIONS(1401),
    [anon_sym_SEMI] = ACTIONS(1399),
    [anon_sym_macro_rules_BANG] = ACTIONS(1399),
    [anon_sym_LPAREN] = ACTIONS(1399),
    [anon_sym_LBRACK] = ACTIONS(1399),
    [anon_sym_LBRACE] = ACTIONS(1399),
    [anon_sym_RBRACE] = ACTIONS(1399),
    [anon_sym_PLUS] = ACTIONS(1401),
    [anon_sym_STAR] = ACTIONS(1401),
    [anon_sym_QMARK] = ACTIONS(1399),
    [anon_sym_u8] = ACTIONS(1401),
    [anon_sym_i8] = ACTIONS(1401),
    [anon_sym_u16] = ACTIONS(1401),
    [anon_sym_i16] = ACTIONS(1401),
    [anon_sym_u32] = ACTIONS(1401),
    [anon_sym_i32] = ACTIONS(1401),
    [anon_sym_u64] = ACTIONS(1401),
    [anon_sym_i64] = ACTIONS(1401),
    [anon_sym_u128] = ACTIONS(1401),
    [anon_sym_i128] = ACTIONS(1401),
    [anon_sym_isize] = ACTIONS(1401),
    [anon_sym_usize] = ACTIONS(1401),
    [anon_sym_f32] = ACTIONS(1401),
    [anon_sym_f64] = ACTIONS(1401),
    [anon_sym_bool] = ACTIONS(1401),
    [anon_sym_str] = ACTIONS(1401),
    [anon_sym_char] = ACTIONS(1401),
    [anon_sym_DASH] = ACTIONS(1401),
    [anon_sym_SLASH] = ACTIONS(1401),
    [anon_sym_PERCENT] = ACTIONS(1401),
    [anon_sym_CARET] = ACTIONS(1401),
    [anon_sym_BANG] = ACTIONS(1401),
    [anon_sym_AMP] = ACTIONS(1401),
    [anon_sym_PIPE] = ACTIONS(1401),
    [anon_sym_AMP_AMP] = ACTIONS(1399),
    [anon_sym_PIPE_PIPE] = ACTIONS(1399),
    [anon_sym_LT_LT] = ACTIONS(1401),
    [anon_sym_GT_GT] = ACTIONS(1401),
    [anon_sym_PLUS_EQ] = ACTIONS(1399),
    [anon_sym_DASH_EQ] = ACTIONS(1399),
    [anon_sym_STAR_EQ] = ACTIONS(1399),
    [anon_sym_SLASH_EQ] = ACTIONS(1399),
    [anon_sym_PERCENT_EQ] = ACTIONS(1399),
    [anon_sym_CARET_EQ] = ACTIONS(1399),
    [anon_sym_AMP_EQ] = ACTIONS(1399),
    [anon_sym_PIPE_EQ] = ACTIONS(1399),
    [anon_sym_LT_LT_EQ] = ACTIONS(1399),
    [anon_sym_GT_GT_EQ] = ACTIONS(1399),
    [anon_sym_EQ] = ACTIONS(1401),
    [anon_sym_EQ_EQ] = ACTIONS(1399),
    [anon_sym_BANG_EQ] = ACTIONS(1399),
    [anon_sym_GT] = ACTIONS(1401),
    [anon_sym_LT] = ACTIONS(1401),
    [anon_sym_GT_EQ] = ACTIONS(1399),
    [anon_sym_LT_EQ] = ACTIONS(1399),
    [anon_sym_DOT] = ACTIONS(1401),
    [anon_sym_DOT_DOT] = ACTIONS(1401),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1399),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1399),
    [anon_sym_COLON_COLON] = ACTIONS(1399),
    [anon_sym_POUND] = ACTIONS(1399),
    [anon_sym_SQUOTE] = ACTIONS(1401),
    [anon_sym_as] = ACTIONS(1401),
    [anon_sym_async] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1401),
    [anon_sym_const] = ACTIONS(1401),
    [anon_sym_continue] = ACTIONS(1401),
    [anon_sym_default] = ACTIONS(1401),
    [anon_sym_enum] = ACTIONS(1401),
    [anon_sym_fn] = ACTIONS(1401),
    [anon_sym_for] = ACTIONS(1401),
    [anon_sym_gen] = ACTIONS(1401),
    [anon_sym_if] = ACTIONS(1401),
    [anon_sym_impl] = ACTIONS(1401),
    [anon_sym_let] = ACTIONS(1401),
    [anon_sym_loop] = ACTIONS(1401),
    [anon_sym_match] = ACTIONS(1401),
    [anon_sym_mod] = ACTIONS(1401),
    [anon_sym_pub] = ACTIONS(1401),
    [anon_sym_return] = ACTIONS(1401),
    [anon_sym_static] = ACTIONS(1401),
    [anon_sym_struct] = ACTIONS(1401),
    [anon_sym_trait] = ACTIONS(1401),
    [anon_sym_type] = ACTIONS(1401),
    [anon_sym_union] = ACTIONS(1401),
    [anon_sym_unsafe] = ACTIONS(1401),
    [anon_sym_use] = ACTIONS(1401),
    [anon_sym_while] = ACTIONS(1401),
    [anon_sym_extern] = ACTIONS(1401),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_move] = ACTIONS(1401),
    [anon_sym_try] = ACTIONS(1401),
    [sym_integer_literal] = ACTIONS(1399),
    [aux_sym_string_literal_token1] = ACTIONS(1399),
    [sym_char_literal] = ACTIONS(1399),
    [anon_sym_true] = ACTIONS(1401),
    [anon_sym_false] = ACTIONS(1401),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1401),
    [sym_super] = ACTIONS(1401),
    [sym_crate] = ACTIONS(1401),
    [sym_metavariable] = ACTIONS(1399),
    [sym__raw_string_literal_start] = ACTIONS(1399),
    [sym_float_literal] = ACTIONS(1399),
  },
  [383] = {
    [sym_line_comment] = STATE(383),
    [sym_block_comment] = STATE(383),
    [ts_builtin_sym_end] = ACTIONS(1403),
    [sym_identifier] = ACTIONS(1405),
    [anon_sym_SEMI] = ACTIONS(1403),
    [anon_sym_macro_rules_BANG] = ACTIONS(1403),
    [anon_sym_LPAREN] = ACTIONS(1403),
    [anon_sym_LBRACK] = ACTIONS(1403),
    [anon_sym_LBRACE] = ACTIONS(1403),
    [anon_sym_RBRACE] = ACTIONS(1403),
    [anon_sym_PLUS] = ACTIONS(1405),
    [anon_sym_STAR] = ACTIONS(1405),
    [anon_sym_QMARK] = ACTIONS(1403),
    [anon_sym_u8] = ACTIONS(1405),
    [anon_sym_i8] = ACTIONS(1405),
    [anon_sym_u16] = ACTIONS(1405),
    [anon_sym_i16] = ACTIONS(1405),
    [anon_sym_u32] = ACTIONS(1405),
    [anon_sym_i32] = ACTIONS(1405),
    [anon_sym_u64] = ACTIONS(1405),
    [anon_sym_i64] = ACTIONS(1405),
    [anon_sym_u128] = ACTIONS(1405),
    [anon_sym_i128] = ACTIONS(1405),
    [anon_sym_isize] = ACTIONS(1405),
    [anon_sym_usize] = ACTIONS(1405),
    [anon_sym_f32] = ACTIONS(1405),
    [anon_sym_f64] = ACTIONS(1405),
    [anon_sym_bool] = ACTIONS(1405),
    [anon_sym_str] = ACTIONS(1405),
    [anon_sym_char] = ACTIONS(1405),
    [anon_sym_DASH] = ACTIONS(1405),
    [anon_sym_SLASH] = ACTIONS(1405),
    [anon_sym_PERCENT] = ACTIONS(1405),
    [anon_sym_CARET] = ACTIONS(1405),
    [anon_sym_BANG] = ACTIONS(1405),
    [anon_sym_AMP] = ACTIONS(1405),
    [anon_sym_PIPE] = ACTIONS(1405),
    [anon_sym_AMP_AMP] = ACTIONS(1403),
    [anon_sym_PIPE_PIPE] = ACTIONS(1403),
    [anon_sym_LT_LT] = ACTIONS(1405),
    [anon_sym_GT_GT] = ACTIONS(1405),
    [anon_sym_PLUS_EQ] = ACTIONS(1403),
    [anon_sym_DASH_EQ] = ACTIONS(1403),
    [anon_sym_STAR_EQ] = ACTIONS(1403),
    [anon_sym_SLASH_EQ] = ACTIONS(1403),
    [anon_sym_PERCENT_EQ] = ACTIONS(1403),
    [anon_sym_CARET_EQ] = ACTIONS(1403),
    [anon_sym_AMP_EQ] = ACTIONS(1403),
    [anon_sym_PIPE_EQ] = ACTIONS(1403),
    [anon_sym_LT_LT_EQ] = ACTIONS(1403),
    [anon_sym_GT_GT_EQ] = ACTIONS(1403),
    [anon_sym_EQ] = ACTIONS(1405),
    [anon_sym_EQ_EQ] = ACTIONS(1403),
    [anon_sym_BANG_EQ] = ACTIONS(1403),
    [anon_sym_GT] = ACTIONS(1405),
    [anon_sym_LT] = ACTIONS(1405),
    [anon_sym_GT_EQ] = ACTIONS(1403),
    [anon_sym_LT_EQ] = ACTIONS(1403),
    [anon_sym_DOT] = ACTIONS(1405),
    [anon_sym_DOT_DOT] = ACTIONS(1405),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1403),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1403),
    [anon_sym_COLON_COLON] = ACTIONS(1403),
    [anon_sym_POUND] = ACTIONS(1403),
    [anon_sym_SQUOTE] = ACTIONS(1405),
    [anon_sym_as] = ACTIONS(1405),
    [anon_sym_async] = ACTIONS(1405),
    [anon_sym_break] = ACTIONS(1405),
    [anon_sym_const] = ACTIONS(1405),
    [anon_sym_continue] = ACTIONS(1405),
    [anon_sym_default] = ACTIONS(1405),
    [anon_sym_enum] = ACTIONS(1405),
    [anon_sym_fn] = ACTIONS(1405),
    [anon_sym_for] = ACTIONS(1405),
    [anon_sym_gen] = ACTIONS(1405),
    [anon_sym_if] = ACTIONS(1405),
    [anon_sym_impl] = ACTIONS(1405),
    [anon_sym_let] = ACTIONS(1405),
    [anon_sym_loop] = ACTIONS(1405),
    [anon_sym_match] = ACTIONS(1405),
    [anon_sym_mod] = ACTIONS(1405),
    [anon_sym_pub] = ACTIONS(1405),
    [anon_sym_return] = ACTIONS(1405),
    [anon_sym_static] = ACTIONS(1405),
    [anon_sym_struct] = ACTIONS(1405),
    [anon_sym_trait] = ACTIONS(1405),
    [anon_sym_type] = ACTIONS(1405),
    [anon_sym_union] = ACTIONS(1405),
    [anon_sym_unsafe] = ACTIONS(1405),
    [anon_sym_use] = ACTIONS(1405),
    [anon_sym_while] = ACTIONS(1405),
    [anon_sym_extern] = ACTIONS(1405),
    [anon_sym_yield] = ACTIONS(1405),
    [anon_sym_move] = ACTIONS(1405),
    [anon_sym_try] = ACTIONS(1405),
    [sym_integer_literal] = ACTIONS(1403),
    [aux_sym_string_literal_token1] = ACTIONS(1403),
    [sym_char_literal] = ACTIONS(1403),
    [anon_sym_true] = ACTIONS(1405),
    [anon_sym_false] = ACTIONS(1405),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1405),
    [sym_super] = ACTIONS(1405),
    [sym_crate] = ACTIONS(1405),
    [sym_metavariable] = ACTIONS(1403),
    [sym__raw_string_literal_start] = ACTIONS(1403),
    [sym_float_literal] = ACTIONS(1403),
  },
  [384] = {
    [sym_line_comment] = STATE(384),
    [sym_block_comment] = STATE(384),
    [ts_builtin_sym_end] = ACTIONS(1051),
    [sym_identifier] = ACTIONS(1049),
    [anon_sym_SEMI] = ACTIONS(1051),
    [anon_sym_macro_rules_BANG] = ACTIONS(1051),
    [anon_sym_LPAREN] = ACTIONS(1051),
    [anon_sym_LBRACK] = ACTIONS(1051),
    [anon_sym_LBRACE] = ACTIONS(1051),
    [anon_sym_RBRACE] = ACTIONS(1051),
    [anon_sym_PLUS] = ACTIONS(1049),
    [anon_sym_STAR] = ACTIONS(1049),
    [anon_sym_QMARK] = ACTIONS(1051),
    [anon_sym_u8] = ACTIONS(1049),
    [anon_sym_i8] = ACTIONS(1049),
    [anon_sym_u16] = ACTIONS(1049),
    [anon_sym_i16] = ACTIONS(1049),
    [anon_sym_u32] = ACTIONS(1049),
    [anon_sym_i32] = ACTIONS(1049),
    [anon_sym_u64] = ACTIONS(1049),
    [anon_sym_i64] = ACTIONS(1049),
    [anon_sym_u128] = ACTIONS(1049),
    [anon_sym_i128] = ACTIONS(1049),
    [anon_sym_isize] = ACTIONS(1049),
    [anon_sym_usize] = ACTIONS(1049),
    [anon_sym_f32] = ACTIONS(1049),
    [anon_sym_f64] = ACTIONS(1049),
    [anon_sym_bool] = ACTIONS(1049),
    [anon_sym_str] = ACTIONS(1049),
    [anon_sym_char] = ACTIONS(1049),
    [anon_sym_DASH] = ACTIONS(1049),
    [anon_sym_SLASH] = ACTIONS(1049),
    [anon_sym_PERCENT] = ACTIONS(1049),
    [anon_sym_CARET] = ACTIONS(1049),
    [anon_sym_BANG] = ACTIONS(1049),
    [anon_sym_AMP] = ACTIONS(1049),
    [anon_sym_PIPE] = ACTIONS(1049),
    [anon_sym_AMP_AMP] = ACTIONS(1051),
    [anon_sym_PIPE_PIPE] = ACTIONS(1051),
    [anon_sym_LT_LT] = ACTIONS(1049),
    [anon_sym_GT_GT] = ACTIONS(1049),
    [anon_sym_PLUS_EQ] = ACTIONS(1051),
    [anon_sym_DASH_EQ] = ACTIONS(1051),
    [anon_sym_STAR_EQ] = ACTIONS(1051),
    [anon_sym_SLASH_EQ] = ACTIONS(1051),
    [anon_sym_PERCENT_EQ] = ACTIONS(1051),
    [anon_sym_CARET_EQ] = ACTIONS(1051),
    [anon_sym_AMP_EQ] = ACTIONS(1051),
    [anon_sym_PIPE_EQ] = ACTIONS(1051),
    [anon_sym_LT_LT_EQ] = ACTIONS(1051),
    [anon_sym_GT_GT_EQ] = ACTIONS(1051),
    [anon_sym_EQ] = ACTIONS(1049),
    [anon_sym_EQ_EQ] = ACTIONS(1051),
    [anon_sym_BANG_EQ] = ACTIONS(1051),
    [anon_sym_GT] = ACTIONS(1049),
    [anon_sym_LT] = ACTIONS(1049),
    [anon_sym_GT_EQ] = ACTIONS(1051),
    [anon_sym_LT_EQ] = ACTIONS(1051),
    [anon_sym_DOT] = ACTIONS(1049),
    [anon_sym_DOT_DOT] = ACTIONS(1049),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1051),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1051),
    [anon_sym_COLON_COLON] = ACTIONS(1051),
    [anon_sym_POUND] = ACTIONS(1051),
    [anon_sym_SQUOTE] = ACTIONS(1049),
    [anon_sym_as] = ACTIONS(1049),
    [anon_sym_async] = ACTIONS(1049),
    [anon_sym_break] = ACTIONS(1049),
    [anon_sym_const] = ACTIONS(1049),
    [anon_sym_continue] = ACTIONS(1049),
    [anon_sym_default] = ACTIONS(1049),
    [anon_sym_enum] = ACTIONS(1049),
    [anon_sym_fn] = ACTIONS(1049),
    [anon_sym_for] = ACTIONS(1049),
    [anon_sym_gen] = ACTIONS(1049),
    [anon_sym_if] = ACTIONS(1049),
    [anon_sym_impl] = ACTIONS(1049),
    [anon_sym_let] = ACTIONS(1049),
    [anon_sym_loop] = ACTIONS(1049),
    [anon_sym_match] = ACTIONS(1049),
    [anon_sym_mod] = ACTIONS(1049),
    [anon_sym_pub] = ACTIONS(1049),
    [anon_sym_return] = ACTIONS(1049),
    [anon_sym_static] = ACTIONS(1049),
    [anon_sym_struct] = ACTIONS(1049),
    [anon_sym_trait] = ACTIONS(1049),
    [anon_sym_type] = ACTIONS(1049),
    [anon_sym_union] = ACTIONS(1049),
    [anon_sym_unsafe] = ACTIONS(1049),
    [anon_sym_use] = ACTIONS(1049),
    [anon_sym_while] = ACTIONS(1049),
    [anon_sym_extern] = ACTIONS(1049),
    [anon_sym_yield] = ACTIONS(1049),
    [anon_sym_move] = ACTIONS(1049),
    [anon_sym_try] = ACTIONS(1049),
    [sym_integer_literal] = ACTIONS(1051),
    [aux_sym_string_literal_token1] = ACTIONS(1051),
    [sym_char_literal] = ACTIONS(1051),
    [anon_sym_true] = ACTIONS(1049),
    [anon_sym_false] = ACTIONS(1049),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1049),
    [sym_super] = ACTIONS(1049),
    [sym_crate] = ACTIONS(1049),
    [sym_metavariable] = ACTIONS(1051),
    [sym__raw_string_literal_start] = ACTIONS(1051),
    [sym_float_literal] = ACTIONS(1051),
  },
  [385] = {
    [sym_line_comment] = STATE(385),
    [sym_block_comment] = STATE(385),
    [ts_builtin_sym_end] = ACTIONS(1407),
    [sym_identifier] = ACTIONS(1409),
    [anon_sym_SEMI] = ACTIONS(1407),
    [anon_sym_macro_rules_BANG] = ACTIONS(1407),
    [anon_sym_LPAREN] = ACTIONS(1407),
    [anon_sym_LBRACK] = ACTIONS(1407),
    [anon_sym_LBRACE] = ACTIONS(1407),
    [anon_sym_RBRACE] = ACTIONS(1407),
    [anon_sym_PLUS] = ACTIONS(1409),
    [anon_sym_STAR] = ACTIONS(1409),
    [anon_sym_QMARK] = ACTIONS(1407),
    [anon_sym_u8] = ACTIONS(1409),
    [anon_sym_i8] = ACTIONS(1409),
    [anon_sym_u16] = ACTIONS(1409),
    [anon_sym_i16] = ACTIONS(1409),
    [anon_sym_u32] = ACTIONS(1409),
    [anon_sym_i32] = ACTIONS(1409),
    [anon_sym_u64] = ACTIONS(1409),
    [anon_sym_i64] = ACTIONS(1409),
    [anon_sym_u128] = ACTIONS(1409),
    [anon_sym_i128] = ACTIONS(1409),
    [anon_sym_isize] = ACTIONS(1409),
    [anon_sym_usize] = ACTIONS(1409),
    [anon_sym_f32] = ACTIONS(1409),
    [anon_sym_f64] = ACTIONS(1409),
    [anon_sym_bool] = ACTIONS(1409),
    [anon_sym_str] = ACTIONS(1409),
    [anon_sym_char] = ACTIONS(1409),
    [anon_sym_DASH] = ACTIONS(1409),
    [anon_sym_SLASH] = ACTIONS(1409),
    [anon_sym_PERCENT] = ACTIONS(1409),
    [anon_sym_CARET] = ACTIONS(1409),
    [anon_sym_BANG] = ACTIONS(1409),
    [anon_sym_AMP] = ACTIONS(1409),
    [anon_sym_PIPE] = ACTIONS(1409),
    [anon_sym_AMP_AMP] = ACTIONS(1407),
    [anon_sym_PIPE_PIPE] = ACTIONS(1407),
    [anon_sym_LT_LT] = ACTIONS(1409),
    [anon_sym_GT_GT] = ACTIONS(1409),
    [anon_sym_PLUS_EQ] = ACTIONS(1407),
    [anon_sym_DASH_EQ] = ACTIONS(1407),
    [anon_sym_STAR_EQ] = ACTIONS(1407),
    [anon_sym_SLASH_EQ] = ACTIONS(1407),
    [anon_sym_PERCENT_EQ] = ACTIONS(1407),
    [anon_sym_CARET_EQ] = ACTIONS(1407),
    [anon_sym_AMP_EQ] = ACTIONS(1407),
    [anon_sym_PIPE_EQ] = ACTIONS(1407),
    [anon_sym_LT_LT_EQ] = ACTIONS(1407),
    [anon_sym_GT_GT_EQ] = ACTIONS(1407),
    [anon_sym_EQ] = ACTIONS(1409),
    [anon_sym_EQ_EQ] = ACTIONS(1407),
    [anon_sym_BANG_EQ] = ACTIONS(1407),
    [anon_sym_GT] = ACTIONS(1409),
    [anon_sym_LT] = ACTIONS(1409),
    [anon_sym_GT_EQ] = ACTIONS(1407),
    [anon_sym_LT_EQ] = ACTIONS(1407),
    [anon_sym_DOT] = ACTIONS(1409),
    [anon_sym_DOT_DOT] = ACTIONS(1409),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1407),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1407),
    [anon_sym_COLON_COLON] = ACTIONS(1407),
    [anon_sym_POUND] = ACTIONS(1407),
    [anon_sym_SQUOTE] = ACTIONS(1409),
    [anon_sym_as] = ACTIONS(1409),
    [anon_sym_async] = ACTIONS(1409),
    [anon_sym_break] = ACTIONS(1409),
    [anon_sym_const] = ACTIONS(1409),
    [anon_sym_continue] = ACTIONS(1409),
    [anon_sym_default] = ACTIONS(1409),
    [anon_sym_enum] = ACTIONS(1409),
    [anon_sym_fn] = ACTIONS(1409),
    [anon_sym_for] = ACTIONS(1409),
    [anon_sym_gen] = ACTIONS(1409),
    [anon_sym_if] = ACTIONS(1409),
    [anon_sym_impl] = ACTIONS(1409),
    [anon_sym_let] = ACTIONS(1409),
    [anon_sym_loop] = ACTIONS(1409),
    [anon_sym_match] = ACTIONS(1409),
    [anon_sym_mod] = ACTIONS(1409),
    [anon_sym_pub] = ACTIONS(1409),
    [anon_sym_return] = ACTIONS(1409),
    [anon_sym_static] = ACTIONS(1409),
    [anon_sym_struct] = ACTIONS(1409),
    [anon_sym_trait] = ACTIONS(1409),
    [anon_sym_type] = ACTIONS(1409),
    [anon_sym_union] = ACTIONS(1409),
    [anon_sym_unsafe] = ACTIONS(1409),
    [anon_sym_use] = ACTIONS(1409),
    [anon_sym_while] = ACTIONS(1409),
    [anon_sym_extern] = ACTIONS(1409),
    [anon_sym_yield] = ACTIONS(1409),
    [anon_sym_move] = ACTIONS(1409),
    [anon_sym_try] = ACTIONS(1409),
    [sym_integer_literal] = ACTIONS(1407),
    [aux_sym_string_literal_token1] = ACTIONS(1407),
    [sym_char_literal] = ACTIONS(1407),
    [anon_sym_true] = ACTIONS(1409),
    [anon_sym_false] = ACTIONS(1409),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1409),
    [sym_super] = ACTIONS(1409),
    [sym_crate] = ACTIONS(1409),
    [sym_metavariable] = ACTIONS(1407),
    [sym__raw_string_literal_start] = ACTIONS(1407),
    [sym_float_literal] = ACTIONS(1407),
  },
  [386] = {
    [sym_line_comment] = STATE(386),
    [sym_block_comment] = STATE(386),
    [ts_builtin_sym_end] = ACTIONS(1411),
    [sym_identifier] = ACTIONS(1413),
    [anon_sym_SEMI] = ACTIONS(1411),
    [anon_sym_macro_rules_BANG] = ACTIONS(1411),
    [anon_sym_LPAREN] = ACTIONS(1411),
    [anon_sym_LBRACK] = ACTIONS(1411),
    [anon_sym_LBRACE] = ACTIONS(1411),
    [anon_sym_RBRACE] = ACTIONS(1411),
    [anon_sym_PLUS] = ACTIONS(1413),
    [anon_sym_STAR] = ACTIONS(1413),
    [anon_sym_QMARK] = ACTIONS(1411),
    [anon_sym_u8] = ACTIONS(1413),
    [anon_sym_i8] = ACTIONS(1413),
    [anon_sym_u16] = ACTIONS(1413),
    [anon_sym_i16] = ACTIONS(1413),
    [anon_sym_u32] = ACTIONS(1413),
    [anon_sym_i32] = ACTIONS(1413),
    [anon_sym_u64] = ACTIONS(1413),
    [anon_sym_i64] = ACTIONS(1413),
    [anon_sym_u128] = ACTIONS(1413),
    [anon_sym_i128] = ACTIONS(1413),
    [anon_sym_isize] = ACTIONS(1413),
    [anon_sym_usize] = ACTIONS(1413),
    [anon_sym_f32] = ACTIONS(1413),
    [anon_sym_f64] = ACTIONS(1413),
    [anon_sym_bool] = ACTIONS(1413),
    [anon_sym_str] = ACTIONS(1413),
    [anon_sym_char] = ACTIONS(1413),
    [anon_sym_DASH] = ACTIONS(1413),
    [anon_sym_SLASH] = ACTIONS(1413),
    [anon_sym_PERCENT] = ACTIONS(1413),
    [anon_sym_CARET] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_AMP] = ACTIONS(1413),
    [anon_sym_PIPE] = ACTIONS(1413),
    [anon_sym_AMP_AMP] = ACTIONS(1411),
    [anon_sym_PIPE_PIPE] = ACTIONS(1411),
    [anon_sym_LT_LT] = ACTIONS(1413),
    [anon_sym_GT_GT] = ACTIONS(1413),
    [anon_sym_PLUS_EQ] = ACTIONS(1411),
    [anon_sym_DASH_EQ] = ACTIONS(1411),
    [anon_sym_STAR_EQ] = ACTIONS(1411),
    [anon_sym_SLASH_EQ] = ACTIONS(1411),
    [anon_sym_PERCENT_EQ] = ACTIONS(1411),
    [anon_sym_CARET_EQ] = ACTIONS(1411),
    [anon_sym_AMP_EQ] = ACTIONS(1411),
    [anon_sym_PIPE_EQ] = ACTIONS(1411),
    [anon_sym_LT_LT_EQ] = ACTIONS(1411),
    [anon_sym_GT_GT_EQ] = ACTIONS(1411),
    [anon_sym_EQ] = ACTIONS(1413),
    [anon_sym_EQ_EQ] = ACTIONS(1411),
    [anon_sym_BANG_EQ] = ACTIONS(1411),
    [anon_sym_GT] = ACTIONS(1413),
    [anon_sym_LT] = ACTIONS(1413),
    [anon_sym_GT_EQ] = ACTIONS(1411),
    [anon_sym_LT_EQ] = ACTIONS(1411),
    [anon_sym_DOT] = ACTIONS(1413),
    [anon_sym_DOT_DOT] = ACTIONS(1413),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1411),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1411),
    [anon_sym_COLON_COLON] = ACTIONS(1411),
    [anon_sym_POUND] = ACTIONS(1411),
    [anon_sym_SQUOTE] = ACTIONS(1413),
    [anon_sym_as] = ACTIONS(1413),
    [anon_sym_async] = ACTIONS(1413),
    [anon_sym_break] = ACTIONS(1413),
    [anon_sym_const] = ACTIONS(1413),
    [anon_sym_continue] = ACTIONS(1413),
    [anon_sym_default] = ACTIONS(1413),
    [anon_sym_enum] = ACTIONS(1413),
    [anon_sym_fn] = ACTIONS(1413),
    [anon_sym_for] = ACTIONS(1413),
    [anon_sym_gen] = ACTIONS(1413),
    [anon_sym_if] = ACTIONS(1413),
    [anon_sym_impl] = ACTIONS(1413),
    [anon_sym_let] = ACTIONS(1413),
    [anon_sym_loop] = ACTIONS(1413),
    [anon_sym_match] = ACTIONS(1413),
    [anon_sym_mod] = ACTIONS(1413),
    [anon_sym_pub] = ACTIONS(1413),
    [anon_sym_return] = ACTIONS(1413),
    [anon_sym_static] = ACTIONS(1413),
    [anon_sym_struct] = ACTIONS(1413),
    [anon_sym_trait] = ACTIONS(1413),
    [anon_sym_type] = ACTIONS(1413),
    [anon_sym_union] = ACTIONS(1413),
    [anon_sym_unsafe] = ACTIONS(1413),
    [anon_sym_use] = ACTIONS(1413),
    [anon_sym_while] = ACTIONS(1413),
    [anon_sym_extern] = ACTIONS(1413),
    [anon_sym_yield] = ACTIONS(1413),
    [anon_sym_move] = ACTIONS(1413),
    [anon_sym_try] = ACTIONS(1413),
    [sym_integer_literal] = ACTIONS(1411),
    [aux_sym_string_literal_token1] = ACTIONS(1411),
    [sym_char_literal] = ACTIONS(1411),
    [anon_sym_true] = ACTIONS(1413),
    [anon_sym_false] = ACTIONS(1413),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1413),
    [sym_super] = ACTIONS(1413),
    [sym_crate] = ACTIONS(1413),
    [sym_metavariable] = ACTIONS(1411),
    [sym__raw_string_literal_start] = ACTIONS(1411),
    [sym_float_literal] = ACTIONS(1411),
  },
  [387] = {
    [sym_line_comment] = STATE(387),
    [sym_block_comment] = STATE(387),
    [ts_builtin_sym_end] = ACTIONS(1415),
    [sym_identifier] = ACTIONS(1417),
    [anon_sym_SEMI] = ACTIONS(1415),
    [anon_sym_macro_rules_BANG] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(1415),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_RBRACE] = ACTIONS(1415),
    [anon_sym_PLUS] = ACTIONS(1417),
    [anon_sym_STAR] = ACTIONS(1417),
    [anon_sym_QMARK] = ACTIONS(1415),
    [anon_sym_u8] = ACTIONS(1417),
    [anon_sym_i8] = ACTIONS(1417),
    [anon_sym_u16] = ACTIONS(1417),
    [anon_sym_i16] = ACTIONS(1417),
    [anon_sym_u32] = ACTIONS(1417),
    [anon_sym_i32] = ACTIONS(1417),
    [anon_sym_u64] = ACTIONS(1417),
    [anon_sym_i64] = ACTIONS(1417),
    [anon_sym_u128] = ACTIONS(1417),
    [anon_sym_i128] = ACTIONS(1417),
    [anon_sym_isize] = ACTIONS(1417),
    [anon_sym_usize] = ACTIONS(1417),
    [anon_sym_f32] = ACTIONS(1417),
    [anon_sym_f64] = ACTIONS(1417),
    [anon_sym_bool] = ACTIONS(1417),
    [anon_sym_str] = ACTIONS(1417),
    [anon_sym_char] = ACTIONS(1417),
    [anon_sym_DASH] = ACTIONS(1417),
    [anon_sym_SLASH] = ACTIONS(1417),
    [anon_sym_PERCENT] = ACTIONS(1417),
    [anon_sym_CARET] = ACTIONS(1417),
    [anon_sym_BANG] = ACTIONS(1417),
    [anon_sym_AMP] = ACTIONS(1417),
    [anon_sym_PIPE] = ACTIONS(1417),
    [anon_sym_AMP_AMP] = ACTIONS(1415),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_LT_LT] = ACTIONS(1417),
    [anon_sym_GT_GT] = ACTIONS(1417),
    [anon_sym_PLUS_EQ] = ACTIONS(1415),
    [anon_sym_DASH_EQ] = ACTIONS(1415),
    [anon_sym_STAR_EQ] = ACTIONS(1415),
    [anon_sym_SLASH_EQ] = ACTIONS(1415),
    [anon_sym_PERCENT_EQ] = ACTIONS(1415),
    [anon_sym_CARET_EQ] = ACTIONS(1415),
    [anon_sym_AMP_EQ] = ACTIONS(1415),
    [anon_sym_PIPE_EQ] = ACTIONS(1415),
    [anon_sym_LT_LT_EQ] = ACTIONS(1415),
    [anon_sym_GT_GT_EQ] = ACTIONS(1415),
    [anon_sym_EQ] = ACTIONS(1417),
    [anon_sym_EQ_EQ] = ACTIONS(1415),
    [anon_sym_BANG_EQ] = ACTIONS(1415),
    [anon_sym_GT] = ACTIONS(1417),
    [anon_sym_LT] = ACTIONS(1417),
    [anon_sym_GT_EQ] = ACTIONS(1415),
    [anon_sym_LT_EQ] = ACTIONS(1415),
    [anon_sym_DOT] = ACTIONS(1417),
    [anon_sym_DOT_DOT] = ACTIONS(1417),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1415),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1415),
    [anon_sym_COLON_COLON] = ACTIONS(1415),
    [anon_sym_POUND] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1417),
    [anon_sym_as] = ACTIONS(1417),
    [anon_sym_async] = ACTIONS(1417),
    [anon_sym_break] = ACTIONS(1417),
    [anon_sym_const] = ACTIONS(1417),
    [anon_sym_continue] = ACTIONS(1417),
    [anon_sym_default] = ACTIONS(1417),
    [anon_sym_enum] = ACTIONS(1417),
    [anon_sym_fn] = ACTIONS(1417),
    [anon_sym_for] = ACTIONS(1417),
    [anon_sym_gen] = ACTIONS(1417),
    [anon_sym_if] = ACTIONS(1417),
    [anon_sym_impl] = ACTIONS(1417),
    [anon_sym_let] = ACTIONS(1417),
    [anon_sym_loop] = ACTIONS(1417),
    [anon_sym_match] = ACTIONS(1417),
    [anon_sym_mod] = ACTIONS(1417),
    [anon_sym_pub] = ACTIONS(1417),
    [anon_sym_return] = ACTIONS(1417),
    [anon_sym_static] = ACTIONS(1417),
    [anon_sym_struct] = ACTIONS(1417),
    [anon_sym_trait] = ACTIONS(1417),
    [anon_sym_type] = ACTIONS(1417),
    [anon_sym_union] = ACTIONS(1417),
    [anon_sym_unsafe] = ACTIONS(1417),
    [anon_sym_use] = ACTIONS(1417),
    [anon_sym_while] = ACTIONS(1417),
    [anon_sym_extern] = ACTIONS(1417),
    [anon_sym_yield] = ACTIONS(1417),
    [anon_sym_move] = ACTIONS(1417),
    [anon_sym_try] = ACTIONS(1417),
    [sym_integer_literal] = ACTIONS(1415),
    [aux_sym_string_literal_token1] = ACTIONS(1415),
    [sym_char_literal] = ACTIONS(1415),
    [anon_sym_true] = ACTIONS(1417),
    [anon_sym_false] = ACTIONS(1417),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1417),
    [sym_super] = ACTIONS(1417),
    [sym_crate] = ACTIONS(1417),
    [sym_metavariable] = ACTIONS(1415),
    [sym__raw_string_literal_start] = ACTIONS(1415),
    [sym_float_literal] = ACTIONS(1415),
  },
  [388] = {
    [sym_line_comment] = STATE(388),
    [sym_block_comment] = STATE(388),
    [ts_builtin_sym_end] = ACTIONS(1047),
    [sym_identifier] = ACTIONS(1045),
    [anon_sym_SEMI] = ACTIONS(1047),
    [anon_sym_macro_rules_BANG] = ACTIONS(1047),
    [anon_sym_LPAREN] = ACTIONS(1047),
    [anon_sym_LBRACK] = ACTIONS(1047),
    [anon_sym_LBRACE] = ACTIONS(1047),
    [anon_sym_RBRACE] = ACTIONS(1047),
    [anon_sym_PLUS] = ACTIONS(1045),
    [anon_sym_STAR] = ACTIONS(1045),
    [anon_sym_QMARK] = ACTIONS(1047),
    [anon_sym_u8] = ACTIONS(1045),
    [anon_sym_i8] = ACTIONS(1045),
    [anon_sym_u16] = ACTIONS(1045),
    [anon_sym_i16] = ACTIONS(1045),
    [anon_sym_u32] = ACTIONS(1045),
    [anon_sym_i32] = ACTIONS(1045),
    [anon_sym_u64] = ACTIONS(1045),
    [anon_sym_i64] = ACTIONS(1045),
    [anon_sym_u128] = ACTIONS(1045),
    [anon_sym_i128] = ACTIONS(1045),
    [anon_sym_isize] = ACTIONS(1045),
    [anon_sym_usize] = ACTIONS(1045),
    [anon_sym_f32] = ACTIONS(1045),
    [anon_sym_f64] = ACTIONS(1045),
    [anon_sym_bool] = ACTIONS(1045),
    [anon_sym_str] = ACTIONS(1045),
    [anon_sym_char] = ACTIONS(1045),
    [anon_sym_DASH] = ACTIONS(1045),
    [anon_sym_SLASH] = ACTIONS(1045),
    [anon_sym_PERCENT] = ACTIONS(1045),
    [anon_sym_CARET] = ACTIONS(1045),
    [anon_sym_BANG] = ACTIONS(1045),
    [anon_sym_AMP] = ACTIONS(1045),
    [anon_sym_PIPE] = ACTIONS(1045),
    [anon_sym_AMP_AMP] = ACTIONS(1047),
    [anon_sym_PIPE_PIPE] = ACTIONS(1047),
    [anon_sym_LT_LT] = ACTIONS(1045),
    [anon_sym_GT_GT] = ACTIONS(1045),
    [anon_sym_PLUS_EQ] = ACTIONS(1047),
    [anon_sym_DASH_EQ] = ACTIONS(1047),
    [anon_sym_STAR_EQ] = ACTIONS(1047),
    [anon_sym_SLASH_EQ] = ACTIONS(1047),
    [anon_sym_PERCENT_EQ] = ACTIONS(1047),
    [anon_sym_CARET_EQ] = ACTIONS(1047),
    [anon_sym_AMP_EQ] = ACTIONS(1047),
    [anon_sym_PIPE_EQ] = ACTIONS(1047),
    [anon_sym_LT_LT_EQ] = ACTIONS(1047),
    [anon_sym_GT_GT_EQ] = ACTIONS(1047),
    [anon_sym_EQ] = ACTIONS(1045),
    [anon_sym_EQ_EQ] = ACTIONS(1047),
    [anon_sym_BANG_EQ] = ACTIONS(1047),
    [anon_sym_GT] = ACTIONS(1045),
    [anon_sym_LT] = ACTIONS(1045),
    [anon_sym_GT_EQ] = ACTIONS(1047),
    [anon_sym_LT_EQ] = ACTIONS(1047),
    [anon_sym_DOT] = ACTIONS(1045),
    [anon_sym_DOT_DOT] = ACTIONS(1045),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1047),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1047),
    [anon_sym_COLON_COLON] = ACTIONS(1047),
    [anon_sym_POUND] = ACTIONS(1047),
    [anon_sym_SQUOTE] = ACTIONS(1045),
    [anon_sym_as] = ACTIONS(1045),
    [anon_sym_async] = ACTIONS(1045),
    [anon_sym_break] = ACTIONS(1045),
    [anon_sym_const] = ACTIONS(1045),
    [anon_sym_continue] = ACTIONS(1045),
    [anon_sym_default] = ACTIONS(1045),
    [anon_sym_enum] = ACTIONS(1045),
    [anon_sym_fn] = ACTIONS(1045),
    [anon_sym_for] = ACTIONS(1045),
    [anon_sym_gen] = ACTIONS(1045),
    [anon_sym_if] = ACTIONS(1045),
    [anon_sym_impl] = ACTIONS(1045),
    [anon_sym_let] = ACTIONS(1045),
    [anon_sym_loop] = ACTIONS(1045),
    [anon_sym_match] = ACTIONS(1045),
    [anon_sym_mod] = ACTIONS(1045),
    [anon_sym_pub] = ACTIONS(1045),
    [anon_sym_return] = ACTIONS(1045),
    [anon_sym_static] = ACTIONS(1045),
    [anon_sym_struct] = ACTIONS(1045),
    [anon_sym_trait] = ACTIONS(1045),
    [anon_sym_type] = ACTIONS(1045),
    [anon_sym_union] = ACTIONS(1045),
    [anon_sym_unsafe] = ACTIONS(1045),
    [anon_sym_use] = ACTIONS(1045),
    [anon_sym_while] = ACTIONS(1045),
    [anon_sym_extern] = ACTIONS(1045),
    [anon_sym_yield] = ACTIONS(1045),
    [anon_sym_move] = ACTIONS(1045),
    [anon_sym_try] = ACTIONS(1045),
    [sym_integer_literal] = ACTIONS(1047),
    [aux_sym_string_literal_token1] = ACTIONS(1047),
    [sym_char_literal] = ACTIONS(1047),
    [anon_sym_true] = ACTIONS(1045),
    [anon_sym_false] = ACTIONS(1045),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1045),
    [sym_super] = ACTIONS(1045),
    [sym_crate] = ACTIONS(1045),
    [sym_metavariable] = ACTIONS(1047),
    [sym__raw_string_literal_start] = ACTIONS(1047),
    [sym_float_literal] = ACTIONS(1047),
  },
  [389] = {
    [sym_attribute_item] = STATE(422),
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_self_parameter] = STATE(2922),
    [sym_variadic_parameter] = STATE(2922),
    [sym_parameter] = STATE(2922),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2584),
    [sym_bracketed_type] = STATE(3550),
    [sym_lifetime] = STATE(2800),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3221),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(2410),
    [sym_scoped_identifier] = STATE(2242),
    [sym_scoped_type_identifier] = STATE(2118),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(3256),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(389),
    [sym_block_comment] = STATE(389),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1281),
    [anon_sym_LPAREN] = ACTIONS(1283),
    [anon_sym_RPAREN] = ACTIONS(1419),
    [anon_sym_LBRACK] = ACTIONS(1287),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1293),
    [anon_sym_i8] = ACTIONS(1293),
    [anon_sym_u16] = ACTIONS(1293),
    [anon_sym_i16] = ACTIONS(1293),
    [anon_sym_u32] = ACTIONS(1293),
    [anon_sym_i32] = ACTIONS(1293),
    [anon_sym_u64] = ACTIONS(1293),
    [anon_sym_i64] = ACTIONS(1293),
    [anon_sym_u128] = ACTIONS(1293),
    [anon_sym_i128] = ACTIONS(1293),
    [anon_sym_isize] = ACTIONS(1293),
    [anon_sym_usize] = ACTIONS(1293),
    [anon_sym_f32] = ACTIONS(1293),
    [anon_sym_f64] = ACTIONS(1293),
    [anon_sym_bool] = ACTIONS(1293),
    [anon_sym_str] = ACTIONS(1293),
    [anon_sym_char] = ACTIONS(1293),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1299),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1421),
    [anon_sym_DOT_DOT] = ACTIONS(1305),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1307),
    [anon_sym_COMMA] = ACTIONS(1423),
    [anon_sym_COLON_COLON] = ACTIONS(1311),
    [anon_sym_POUND] = ACTIONS(1313),
    [anon_sym_SQUOTE] = ACTIONS(1315),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1319),
    [anon_sym_default] = ACTIONS(1321),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1327),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1327),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_ref] = ACTIONS(1333),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_mutable_specifier] = ACTIONS(1337),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1345),
    [sym_super] = ACTIONS(1347),
    [sym_crate] = ACTIONS(1347),
    [sym_metavariable] = ACTIONS(1349),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [390] = {
    [sym_attribute_item] = STATE(420),
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_self_parameter] = STATE(2999),
    [sym_variadic_parameter] = STATE(2999),
    [sym_parameter] = STATE(2999),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2746),
    [sym_bracketed_type] = STATE(3557),
    [sym_lifetime] = STATE(2800),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3301),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(2446),
    [sym_scoped_identifier] = STATE(2125),
    [sym_scoped_type_identifier] = STATE(2118),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2431),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(390),
    [sym_block_comment] = STATE(390),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1369),
    [anon_sym_LPAREN] = ACTIONS(1371),
    [anon_sym_RPAREN] = ACTIONS(1425),
    [anon_sym_LBRACK] = ACTIONS(1287),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1375),
    [anon_sym_i8] = ACTIONS(1375),
    [anon_sym_u16] = ACTIONS(1375),
    [anon_sym_i16] = ACTIONS(1375),
    [anon_sym_u32] = ACTIONS(1375),
    [anon_sym_i32] = ACTIONS(1375),
    [anon_sym_u64] = ACTIONS(1375),
    [anon_sym_i64] = ACTIONS(1375),
    [anon_sym_u128] = ACTIONS(1375),
    [anon_sym_i128] = ACTIONS(1375),
    [anon_sym_isize] = ACTIONS(1375),
    [anon_sym_usize] = ACTIONS(1375),
    [anon_sym_f32] = ACTIONS(1375),
    [anon_sym_f64] = ACTIONS(1375),
    [anon_sym_bool] = ACTIONS(1375),
    [anon_sym_str] = ACTIONS(1375),
    [anon_sym_char] = ACTIONS(1375),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1377),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1379),
    [anon_sym_DOT_DOT] = ACTIONS(1305),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1307),
    [anon_sym_COMMA] = ACTIONS(1427),
    [anon_sym_COLON_COLON] = ACTIONS(1383),
    [anon_sym_POUND] = ACTIONS(1313),
    [anon_sym_SQUOTE] = ACTIONS(1315),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1319),
    [anon_sym_default] = ACTIONS(1385),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1387),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1387),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_ref] = ACTIONS(1333),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_mutable_specifier] = ACTIONS(1337),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1389),
    [sym_super] = ACTIONS(1391),
    [sym_crate] = ACTIONS(1391),
    [sym_metavariable] = ACTIONS(1393),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [391] = {
    [sym_line_comment] = STATE(391),
    [sym_block_comment] = STATE(391),
    [ts_builtin_sym_end] = ACTIONS(1429),
    [sym_identifier] = ACTIONS(1431),
    [anon_sym_SEMI] = ACTIONS(1429),
    [anon_sym_macro_rules_BANG] = ACTIONS(1429),
    [anon_sym_LPAREN] = ACTIONS(1429),
    [anon_sym_LBRACK] = ACTIONS(1429),
    [anon_sym_LBRACE] = ACTIONS(1429),
    [anon_sym_RBRACE] = ACTIONS(1429),
    [anon_sym_PLUS] = ACTIONS(1431),
    [anon_sym_STAR] = ACTIONS(1431),
    [anon_sym_QMARK] = ACTIONS(1429),
    [anon_sym_u8] = ACTIONS(1431),
    [anon_sym_i8] = ACTIONS(1431),
    [anon_sym_u16] = ACTIONS(1431),
    [anon_sym_i16] = ACTIONS(1431),
    [anon_sym_u32] = ACTIONS(1431),
    [anon_sym_i32] = ACTIONS(1431),
    [anon_sym_u64] = ACTIONS(1431),
    [anon_sym_i64] = ACTIONS(1431),
    [anon_sym_u128] = ACTIONS(1431),
    [anon_sym_i128] = ACTIONS(1431),
    [anon_sym_isize] = ACTIONS(1431),
    [anon_sym_usize] = ACTIONS(1431),
    [anon_sym_f32] = ACTIONS(1431),
    [anon_sym_f64] = ACTIONS(1431),
    [anon_sym_bool] = ACTIONS(1431),
    [anon_sym_str] = ACTIONS(1431),
    [anon_sym_char] = ACTIONS(1431),
    [anon_sym_DASH] = ACTIONS(1431),
    [anon_sym_SLASH] = ACTIONS(1431),
    [anon_sym_PERCENT] = ACTIONS(1431),
    [anon_sym_CARET] = ACTIONS(1431),
    [anon_sym_BANG] = ACTIONS(1431),
    [anon_sym_AMP] = ACTIONS(1431),
    [anon_sym_PIPE] = ACTIONS(1431),
    [anon_sym_AMP_AMP] = ACTIONS(1429),
    [anon_sym_PIPE_PIPE] = ACTIONS(1429),
    [anon_sym_LT_LT] = ACTIONS(1431),
    [anon_sym_GT_GT] = ACTIONS(1431),
    [anon_sym_PLUS_EQ] = ACTIONS(1429),
    [anon_sym_DASH_EQ] = ACTIONS(1429),
    [anon_sym_STAR_EQ] = ACTIONS(1429),
    [anon_sym_SLASH_EQ] = ACTIONS(1429),
    [anon_sym_PERCENT_EQ] = ACTIONS(1429),
    [anon_sym_CARET_EQ] = ACTIONS(1429),
    [anon_sym_AMP_EQ] = ACTIONS(1429),
    [anon_sym_PIPE_EQ] = ACTIONS(1429),
    [anon_sym_LT_LT_EQ] = ACTIONS(1429),
    [anon_sym_GT_GT_EQ] = ACTIONS(1429),
    [anon_sym_EQ] = ACTIONS(1431),
    [anon_sym_EQ_EQ] = ACTIONS(1429),
    [anon_sym_BANG_EQ] = ACTIONS(1429),
    [anon_sym_GT] = ACTIONS(1431),
    [anon_sym_LT] = ACTIONS(1431),
    [anon_sym_GT_EQ] = ACTIONS(1429),
    [anon_sym_LT_EQ] = ACTIONS(1429),
    [anon_sym_DOT] = ACTIONS(1431),
    [anon_sym_DOT_DOT] = ACTIONS(1431),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1429),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1429),
    [anon_sym_COLON_COLON] = ACTIONS(1429),
    [anon_sym_POUND] = ACTIONS(1429),
    [anon_sym_SQUOTE] = ACTIONS(1431),
    [anon_sym_as] = ACTIONS(1431),
    [anon_sym_async] = ACTIONS(1431),
    [anon_sym_break] = ACTIONS(1431),
    [anon_sym_const] = ACTIONS(1431),
    [anon_sym_continue] = ACTIONS(1431),
    [anon_sym_default] = ACTIONS(1431),
    [anon_sym_enum] = ACTIONS(1431),
    [anon_sym_fn] = ACTIONS(1431),
    [anon_sym_for] = ACTIONS(1431),
    [anon_sym_gen] = ACTIONS(1431),
    [anon_sym_if] = ACTIONS(1431),
    [anon_sym_impl] = ACTIONS(1431),
    [anon_sym_let] = ACTIONS(1431),
    [anon_sym_loop] = ACTIONS(1431),
    [anon_sym_match] = ACTIONS(1431),
    [anon_sym_mod] = ACTIONS(1431),
    [anon_sym_pub] = ACTIONS(1431),
    [anon_sym_return] = ACTIONS(1431),
    [anon_sym_static] = ACTIONS(1431),
    [anon_sym_struct] = ACTIONS(1431),
    [anon_sym_trait] = ACTIONS(1431),
    [anon_sym_type] = ACTIONS(1431),
    [anon_sym_union] = ACTIONS(1431),
    [anon_sym_unsafe] = ACTIONS(1431),
    [anon_sym_use] = ACTIONS(1431),
    [anon_sym_while] = ACTIONS(1431),
    [anon_sym_extern] = ACTIONS(1431),
    [anon_sym_yield] = ACTIONS(1431),
    [anon_sym_move] = ACTIONS(1431),
    [anon_sym_try] = ACTIONS(1431),
    [sym_integer_literal] = ACTIONS(1429),
    [aux_sym_string_literal_token1] = ACTIONS(1429),
    [sym_char_literal] = ACTIONS(1429),
    [anon_sym_true] = ACTIONS(1431),
    [anon_sym_false] = ACTIONS(1431),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1431),
    [sym_super] = ACTIONS(1431),
    [sym_crate] = ACTIONS(1431),
    [sym_metavariable] = ACTIONS(1429),
    [sym__raw_string_literal_start] = ACTIONS(1429),
    [sym_float_literal] = ACTIONS(1429),
  },
  [392] = {
    [sym_line_comment] = STATE(392),
    [sym_block_comment] = STATE(392),
    [ts_builtin_sym_end] = ACTIONS(1433),
    [sym_identifier] = ACTIONS(1435),
    [anon_sym_SEMI] = ACTIONS(1433),
    [anon_sym_macro_rules_BANG] = ACTIONS(1433),
    [anon_sym_LPAREN] = ACTIONS(1433),
    [anon_sym_LBRACK] = ACTIONS(1433),
    [anon_sym_LBRACE] = ACTIONS(1433),
    [anon_sym_RBRACE] = ACTIONS(1433),
    [anon_sym_PLUS] = ACTIONS(1435),
    [anon_sym_STAR] = ACTIONS(1435),
    [anon_sym_QMARK] = ACTIONS(1433),
    [anon_sym_u8] = ACTIONS(1435),
    [anon_sym_i8] = ACTIONS(1435),
    [anon_sym_u16] = ACTIONS(1435),
    [anon_sym_i16] = ACTIONS(1435),
    [anon_sym_u32] = ACTIONS(1435),
    [anon_sym_i32] = ACTIONS(1435),
    [anon_sym_u64] = ACTIONS(1435),
    [anon_sym_i64] = ACTIONS(1435),
    [anon_sym_u128] = ACTIONS(1435),
    [anon_sym_i128] = ACTIONS(1435),
    [anon_sym_isize] = ACTIONS(1435),
    [anon_sym_usize] = ACTIONS(1435),
    [anon_sym_f32] = ACTIONS(1435),
    [anon_sym_f64] = ACTIONS(1435),
    [anon_sym_bool] = ACTIONS(1435),
    [anon_sym_str] = ACTIONS(1435),
    [anon_sym_char] = ACTIONS(1435),
    [anon_sym_DASH] = ACTIONS(1435),
    [anon_sym_SLASH] = ACTIONS(1435),
    [anon_sym_PERCENT] = ACTIONS(1435),
    [anon_sym_CARET] = ACTIONS(1435),
    [anon_sym_BANG] = ACTIONS(1435),
    [anon_sym_AMP] = ACTIONS(1435),
    [anon_sym_PIPE] = ACTIONS(1435),
    [anon_sym_AMP_AMP] = ACTIONS(1433),
    [anon_sym_PIPE_PIPE] = ACTIONS(1433),
    [anon_sym_LT_LT] = ACTIONS(1435),
    [anon_sym_GT_GT] = ACTIONS(1435),
    [anon_sym_PLUS_EQ] = ACTIONS(1433),
    [anon_sym_DASH_EQ] = ACTIONS(1433),
    [anon_sym_STAR_EQ] = ACTIONS(1433),
    [anon_sym_SLASH_EQ] = ACTIONS(1433),
    [anon_sym_PERCENT_EQ] = ACTIONS(1433),
    [anon_sym_CARET_EQ] = ACTIONS(1433),
    [anon_sym_AMP_EQ] = ACTIONS(1433),
    [anon_sym_PIPE_EQ] = ACTIONS(1433),
    [anon_sym_LT_LT_EQ] = ACTIONS(1433),
    [anon_sym_GT_GT_EQ] = ACTIONS(1433),
    [anon_sym_EQ] = ACTIONS(1435),
    [anon_sym_EQ_EQ] = ACTIONS(1433),
    [anon_sym_BANG_EQ] = ACTIONS(1433),
    [anon_sym_GT] = ACTIONS(1435),
    [anon_sym_LT] = ACTIONS(1435),
    [anon_sym_GT_EQ] = ACTIONS(1433),
    [anon_sym_LT_EQ] = ACTIONS(1433),
    [anon_sym_DOT] = ACTIONS(1435),
    [anon_sym_DOT_DOT] = ACTIONS(1435),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1433),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1433),
    [anon_sym_COLON_COLON] = ACTIONS(1433),
    [anon_sym_POUND] = ACTIONS(1433),
    [anon_sym_SQUOTE] = ACTIONS(1435),
    [anon_sym_as] = ACTIONS(1435),
    [anon_sym_async] = ACTIONS(1435),
    [anon_sym_break] = ACTIONS(1435),
    [anon_sym_const] = ACTIONS(1435),
    [anon_sym_continue] = ACTIONS(1435),
    [anon_sym_default] = ACTIONS(1435),
    [anon_sym_enum] = ACTIONS(1435),
    [anon_sym_fn] = ACTIONS(1435),
    [anon_sym_for] = ACTIONS(1435),
    [anon_sym_gen] = ACTIONS(1435),
    [anon_sym_if] = ACTIONS(1435),
    [anon_sym_impl] = ACTIONS(1435),
    [anon_sym_let] = ACTIONS(1435),
    [anon_sym_loop] = ACTIONS(1435),
    [anon_sym_match] = ACTIONS(1435),
    [anon_sym_mod] = ACTIONS(1435),
    [anon_sym_pub] = ACTIONS(1435),
    [anon_sym_return] = ACTIONS(1435),
    [anon_sym_static] = ACTIONS(1435),
    [anon_sym_struct] = ACTIONS(1435),
    [anon_sym_trait] = ACTIONS(1435),
    [anon_sym_type] = ACTIONS(1435),
    [anon_sym_union] = ACTIONS(1435),
    [anon_sym_unsafe] = ACTIONS(1435),
    [anon_sym_use] = ACTIONS(1435),
    [anon_sym_while] = ACTIONS(1435),
    [anon_sym_extern] = ACTIONS(1435),
    [anon_sym_yield] = ACTIONS(1435),
    [anon_sym_move] = ACTIONS(1435),
    [anon_sym_try] = ACTIONS(1435),
    [sym_integer_literal] = ACTIONS(1433),
    [aux_sym_string_literal_token1] = ACTIONS(1433),
    [sym_char_literal] = ACTIONS(1433),
    [anon_sym_true] = ACTIONS(1435),
    [anon_sym_false] = ACTIONS(1435),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1435),
    [sym_super] = ACTIONS(1435),
    [sym_crate] = ACTIONS(1435),
    [sym_metavariable] = ACTIONS(1433),
    [sym__raw_string_literal_start] = ACTIONS(1433),
    [sym_float_literal] = ACTIONS(1433),
  },
  [393] = {
    [sym_line_comment] = STATE(393),
    [sym_block_comment] = STATE(393),
    [ts_builtin_sym_end] = ACTIONS(1437),
    [sym_identifier] = ACTIONS(1439),
    [anon_sym_SEMI] = ACTIONS(1437),
    [anon_sym_macro_rules_BANG] = ACTIONS(1437),
    [anon_sym_LPAREN] = ACTIONS(1437),
    [anon_sym_LBRACK] = ACTIONS(1437),
    [anon_sym_LBRACE] = ACTIONS(1437),
    [anon_sym_RBRACE] = ACTIONS(1437),
    [anon_sym_PLUS] = ACTIONS(1439),
    [anon_sym_STAR] = ACTIONS(1439),
    [anon_sym_QMARK] = ACTIONS(1437),
    [anon_sym_u8] = ACTIONS(1439),
    [anon_sym_i8] = ACTIONS(1439),
    [anon_sym_u16] = ACTIONS(1439),
    [anon_sym_i16] = ACTIONS(1439),
    [anon_sym_u32] = ACTIONS(1439),
    [anon_sym_i32] = ACTIONS(1439),
    [anon_sym_u64] = ACTIONS(1439),
    [anon_sym_i64] = ACTIONS(1439),
    [anon_sym_u128] = ACTIONS(1439),
    [anon_sym_i128] = ACTIONS(1439),
    [anon_sym_isize] = ACTIONS(1439),
    [anon_sym_usize] = ACTIONS(1439),
    [anon_sym_f32] = ACTIONS(1439),
    [anon_sym_f64] = ACTIONS(1439),
    [anon_sym_bool] = ACTIONS(1439),
    [anon_sym_str] = ACTIONS(1439),
    [anon_sym_char] = ACTIONS(1439),
    [anon_sym_DASH] = ACTIONS(1439),
    [anon_sym_SLASH] = ACTIONS(1439),
    [anon_sym_PERCENT] = ACTIONS(1439),
    [anon_sym_CARET] = ACTIONS(1439),
    [anon_sym_BANG] = ACTIONS(1439),
    [anon_sym_AMP] = ACTIONS(1439),
    [anon_sym_PIPE] = ACTIONS(1439),
    [anon_sym_AMP_AMP] = ACTIONS(1437),
    [anon_sym_PIPE_PIPE] = ACTIONS(1437),
    [anon_sym_LT_LT] = ACTIONS(1439),
    [anon_sym_GT_GT] = ACTIONS(1439),
    [anon_sym_PLUS_EQ] = ACTIONS(1437),
    [anon_sym_DASH_EQ] = ACTIONS(1437),
    [anon_sym_STAR_EQ] = ACTIONS(1437),
    [anon_sym_SLASH_EQ] = ACTIONS(1437),
    [anon_sym_PERCENT_EQ] = ACTIONS(1437),
    [anon_sym_CARET_EQ] = ACTIONS(1437),
    [anon_sym_AMP_EQ] = ACTIONS(1437),
    [anon_sym_PIPE_EQ] = ACTIONS(1437),
    [anon_sym_LT_LT_EQ] = ACTIONS(1437),
    [anon_sym_GT_GT_EQ] = ACTIONS(1437),
    [anon_sym_EQ] = ACTIONS(1439),
    [anon_sym_EQ_EQ] = ACTIONS(1437),
    [anon_sym_BANG_EQ] = ACTIONS(1437),
    [anon_sym_GT] = ACTIONS(1439),
    [anon_sym_LT] = ACTIONS(1439),
    [anon_sym_GT_EQ] = ACTIONS(1437),
    [anon_sym_LT_EQ] = ACTIONS(1437),
    [anon_sym_DOT] = ACTIONS(1439),
    [anon_sym_DOT_DOT] = ACTIONS(1439),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1437),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1437),
    [anon_sym_COLON_COLON] = ACTIONS(1437),
    [anon_sym_POUND] = ACTIONS(1437),
    [anon_sym_SQUOTE] = ACTIONS(1439),
    [anon_sym_as] = ACTIONS(1439),
    [anon_sym_async] = ACTIONS(1439),
    [anon_sym_break] = ACTIONS(1439),
    [anon_sym_const] = ACTIONS(1439),
    [anon_sym_continue] = ACTIONS(1439),
    [anon_sym_default] = ACTIONS(1439),
    [anon_sym_enum] = ACTIONS(1439),
    [anon_sym_fn] = ACTIONS(1439),
    [anon_sym_for] = ACTIONS(1439),
    [anon_sym_gen] = ACTIONS(1439),
    [anon_sym_if] = ACTIONS(1439),
    [anon_sym_impl] = ACTIONS(1439),
    [anon_sym_let] = ACTIONS(1439),
    [anon_sym_loop] = ACTIONS(1439),
    [anon_sym_match] = ACTIONS(1439),
    [anon_sym_mod] = ACTIONS(1439),
    [anon_sym_pub] = ACTIONS(1439),
    [anon_sym_return] = ACTIONS(1439),
    [anon_sym_static] = ACTIONS(1439),
    [anon_sym_struct] = ACTIONS(1439),
    [anon_sym_trait] = ACTIONS(1439),
    [anon_sym_type] = ACTIONS(1439),
    [anon_sym_union] = ACTIONS(1439),
    [anon_sym_unsafe] = ACTIONS(1439),
    [anon_sym_use] = ACTIONS(1439),
    [anon_sym_while] = ACTIONS(1439),
    [anon_sym_extern] = ACTIONS(1439),
    [anon_sym_yield] = ACTIONS(1439),
    [anon_sym_move] = ACTIONS(1439),
    [anon_sym_try] = ACTIONS(1439),
    [sym_integer_literal] = ACTIONS(1437),
    [aux_sym_string_literal_token1] = ACTIONS(1437),
    [sym_char_literal] = ACTIONS(1437),
    [anon_sym_true] = ACTIONS(1439),
    [anon_sym_false] = ACTIONS(1439),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1439),
    [sym_super] = ACTIONS(1439),
    [sym_crate] = ACTIONS(1439),
    [sym_metavariable] = ACTIONS(1437),
    [sym__raw_string_literal_start] = ACTIONS(1437),
    [sym_float_literal] = ACTIONS(1437),
  },
  [394] = {
    [sym_attribute_item] = STATE(421),
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_self_parameter] = STATE(2782),
    [sym_variadic_parameter] = STATE(2782),
    [sym_parameter] = STATE(2782),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2754),
    [sym_bracketed_type] = STATE(3550),
    [sym_lifetime] = STATE(2800),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3221),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(2410),
    [sym_scoped_identifier] = STATE(2242),
    [sym_scoped_type_identifier] = STATE(2118),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(3256),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(394),
    [sym_block_comment] = STATE(394),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1281),
    [anon_sym_LPAREN] = ACTIONS(1283),
    [anon_sym_RPAREN] = ACTIONS(1441),
    [anon_sym_LBRACK] = ACTIONS(1287),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1293),
    [anon_sym_i8] = ACTIONS(1293),
    [anon_sym_u16] = ACTIONS(1293),
    [anon_sym_i16] = ACTIONS(1293),
    [anon_sym_u32] = ACTIONS(1293),
    [anon_sym_i32] = ACTIONS(1293),
    [anon_sym_u64] = ACTIONS(1293),
    [anon_sym_i64] = ACTIONS(1293),
    [anon_sym_u128] = ACTIONS(1293),
    [anon_sym_i128] = ACTIONS(1293),
    [anon_sym_isize] = ACTIONS(1293),
    [anon_sym_usize] = ACTIONS(1293),
    [anon_sym_f32] = ACTIONS(1293),
    [anon_sym_f64] = ACTIONS(1293),
    [anon_sym_bool] = ACTIONS(1293),
    [anon_sym_str] = ACTIONS(1293),
    [anon_sym_char] = ACTIONS(1293),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1299),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1443),
    [anon_sym_DOT_DOT] = ACTIONS(1305),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1307),
    [anon_sym_COMMA] = ACTIONS(1445),
    [anon_sym_COLON_COLON] = ACTIONS(1311),
    [anon_sym_POUND] = ACTIONS(1313),
    [anon_sym_SQUOTE] = ACTIONS(1315),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1319),
    [anon_sym_default] = ACTIONS(1321),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1327),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1327),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_ref] = ACTIONS(1333),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_mutable_specifier] = ACTIONS(1337),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1345),
    [sym_super] = ACTIONS(1347),
    [sym_crate] = ACTIONS(1347),
    [sym_metavariable] = ACTIONS(1349),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [395] = {
    [sym_line_comment] = STATE(395),
    [sym_block_comment] = STATE(395),
    [ts_builtin_sym_end] = ACTIONS(1447),
    [sym_identifier] = ACTIONS(1449),
    [anon_sym_SEMI] = ACTIONS(1447),
    [anon_sym_macro_rules_BANG] = ACTIONS(1447),
    [anon_sym_LPAREN] = ACTIONS(1447),
    [anon_sym_LBRACK] = ACTIONS(1447),
    [anon_sym_LBRACE] = ACTIONS(1447),
    [anon_sym_RBRACE] = ACTIONS(1447),
    [anon_sym_PLUS] = ACTIONS(1449),
    [anon_sym_STAR] = ACTIONS(1449),
    [anon_sym_QMARK] = ACTIONS(1447),
    [anon_sym_u8] = ACTIONS(1449),
    [anon_sym_i8] = ACTIONS(1449),
    [anon_sym_u16] = ACTIONS(1449),
    [anon_sym_i16] = ACTIONS(1449),
    [anon_sym_u32] = ACTIONS(1449),
    [anon_sym_i32] = ACTIONS(1449),
    [anon_sym_u64] = ACTIONS(1449),
    [anon_sym_i64] = ACTIONS(1449),
    [anon_sym_u128] = ACTIONS(1449),
    [anon_sym_i128] = ACTIONS(1449),
    [anon_sym_isize] = ACTIONS(1449),
    [anon_sym_usize] = ACTIONS(1449),
    [anon_sym_f32] = ACTIONS(1449),
    [anon_sym_f64] = ACTIONS(1449),
    [anon_sym_bool] = ACTIONS(1449),
    [anon_sym_str] = ACTIONS(1449),
    [anon_sym_char] = ACTIONS(1449),
    [anon_sym_DASH] = ACTIONS(1449),
    [anon_sym_SLASH] = ACTIONS(1449),
    [anon_sym_PERCENT] = ACTIONS(1449),
    [anon_sym_CARET] = ACTIONS(1449),
    [anon_sym_BANG] = ACTIONS(1449),
    [anon_sym_AMP] = ACTIONS(1449),
    [anon_sym_PIPE] = ACTIONS(1449),
    [anon_sym_AMP_AMP] = ACTIONS(1447),
    [anon_sym_PIPE_PIPE] = ACTIONS(1447),
    [anon_sym_LT_LT] = ACTIONS(1449),
    [anon_sym_GT_GT] = ACTIONS(1449),
    [anon_sym_PLUS_EQ] = ACTIONS(1447),
    [anon_sym_DASH_EQ] = ACTIONS(1447),
    [anon_sym_STAR_EQ] = ACTIONS(1447),
    [anon_sym_SLASH_EQ] = ACTIONS(1447),
    [anon_sym_PERCENT_EQ] = ACTIONS(1447),
    [anon_sym_CARET_EQ] = ACTIONS(1447),
    [anon_sym_AMP_EQ] = ACTIONS(1447),
    [anon_sym_PIPE_EQ] = ACTIONS(1447),
    [anon_sym_LT_LT_EQ] = ACTIONS(1447),
    [anon_sym_GT_GT_EQ] = ACTIONS(1447),
    [anon_sym_EQ] = ACTIONS(1449),
    [anon_sym_EQ_EQ] = ACTIONS(1447),
    [anon_sym_BANG_EQ] = ACTIONS(1447),
    [anon_sym_GT] = ACTIONS(1449),
    [anon_sym_LT] = ACTIONS(1449),
    [anon_sym_GT_EQ] = ACTIONS(1447),
    [anon_sym_LT_EQ] = ACTIONS(1447),
    [anon_sym_DOT] = ACTIONS(1449),
    [anon_sym_DOT_DOT] = ACTIONS(1449),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1447),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1447),
    [anon_sym_COLON_COLON] = ACTIONS(1447),
    [anon_sym_POUND] = ACTIONS(1447),
    [anon_sym_SQUOTE] = ACTIONS(1449),
    [anon_sym_as] = ACTIONS(1449),
    [anon_sym_async] = ACTIONS(1449),
    [anon_sym_break] = ACTIONS(1449),
    [anon_sym_const] = ACTIONS(1449),
    [anon_sym_continue] = ACTIONS(1449),
    [anon_sym_default] = ACTIONS(1449),
    [anon_sym_enum] = ACTIONS(1449),
    [anon_sym_fn] = ACTIONS(1449),
    [anon_sym_for] = ACTIONS(1449),
    [anon_sym_gen] = ACTIONS(1449),
    [anon_sym_if] = ACTIONS(1449),
    [anon_sym_impl] = ACTIONS(1449),
    [anon_sym_let] = ACTIONS(1449),
    [anon_sym_loop] = ACTIONS(1449),
    [anon_sym_match] = ACTIONS(1449),
    [anon_sym_mod] = ACTIONS(1449),
    [anon_sym_pub] = ACTIONS(1449),
    [anon_sym_return] = ACTIONS(1449),
    [anon_sym_static] = ACTIONS(1449),
    [anon_sym_struct] = ACTIONS(1449),
    [anon_sym_trait] = ACTIONS(1449),
    [anon_sym_type] = ACTIONS(1449),
    [anon_sym_union] = ACTIONS(1449),
    [anon_sym_unsafe] = ACTIONS(1449),
    [anon_sym_use] = ACTIONS(1449),
    [anon_sym_while] = ACTIONS(1449),
    [anon_sym_extern] = ACTIONS(1449),
    [anon_sym_yield] = ACTIONS(1449),
    [anon_sym_move] = ACTIONS(1449),
    [anon_sym_try] = ACTIONS(1449),
    [sym_integer_literal] = ACTIONS(1447),
    [aux_sym_string_literal_token1] = ACTIONS(1447),
    [sym_char_literal] = ACTIONS(1447),
    [anon_sym_true] = ACTIONS(1449),
    [anon_sym_false] = ACTIONS(1449),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1449),
    [sym_super] = ACTIONS(1449),
    [sym_crate] = ACTIONS(1449),
    [sym_metavariable] = ACTIONS(1447),
    [sym__raw_string_literal_start] = ACTIONS(1447),
    [sym_float_literal] = ACTIONS(1447),
  },
  [396] = {
    [sym_line_comment] = STATE(396),
    [sym_block_comment] = STATE(396),
    [ts_builtin_sym_end] = ACTIONS(1451),
    [sym_identifier] = ACTIONS(1453),
    [anon_sym_SEMI] = ACTIONS(1451),
    [anon_sym_macro_rules_BANG] = ACTIONS(1451),
    [anon_sym_LPAREN] = ACTIONS(1451),
    [anon_sym_LBRACK] = ACTIONS(1451),
    [anon_sym_LBRACE] = ACTIONS(1451),
    [anon_sym_RBRACE] = ACTIONS(1451),
    [anon_sym_PLUS] = ACTIONS(1453),
    [anon_sym_STAR] = ACTIONS(1453),
    [anon_sym_QMARK] = ACTIONS(1451),
    [anon_sym_u8] = ACTIONS(1453),
    [anon_sym_i8] = ACTIONS(1453),
    [anon_sym_u16] = ACTIONS(1453),
    [anon_sym_i16] = ACTIONS(1453),
    [anon_sym_u32] = ACTIONS(1453),
    [anon_sym_i32] = ACTIONS(1453),
    [anon_sym_u64] = ACTIONS(1453),
    [anon_sym_i64] = ACTIONS(1453),
    [anon_sym_u128] = ACTIONS(1453),
    [anon_sym_i128] = ACTIONS(1453),
    [anon_sym_isize] = ACTIONS(1453),
    [anon_sym_usize] = ACTIONS(1453),
    [anon_sym_f32] = ACTIONS(1453),
    [anon_sym_f64] = ACTIONS(1453),
    [anon_sym_bool] = ACTIONS(1453),
    [anon_sym_str] = ACTIONS(1453),
    [anon_sym_char] = ACTIONS(1453),
    [anon_sym_DASH] = ACTIONS(1453),
    [anon_sym_SLASH] = ACTIONS(1453),
    [anon_sym_PERCENT] = ACTIONS(1453),
    [anon_sym_CARET] = ACTIONS(1453),
    [anon_sym_BANG] = ACTIONS(1453),
    [anon_sym_AMP] = ACTIONS(1453),
    [anon_sym_PIPE] = ACTIONS(1453),
    [anon_sym_AMP_AMP] = ACTIONS(1451),
    [anon_sym_PIPE_PIPE] = ACTIONS(1451),
    [anon_sym_LT_LT] = ACTIONS(1453),
    [anon_sym_GT_GT] = ACTIONS(1453),
    [anon_sym_PLUS_EQ] = ACTIONS(1451),
    [anon_sym_DASH_EQ] = ACTIONS(1451),
    [anon_sym_STAR_EQ] = ACTIONS(1451),
    [anon_sym_SLASH_EQ] = ACTIONS(1451),
    [anon_sym_PERCENT_EQ] = ACTIONS(1451),
    [anon_sym_CARET_EQ] = ACTIONS(1451),
    [anon_sym_AMP_EQ] = ACTIONS(1451),
    [anon_sym_PIPE_EQ] = ACTIONS(1451),
    [anon_sym_LT_LT_EQ] = ACTIONS(1451),
    [anon_sym_GT_GT_EQ] = ACTIONS(1451),
    [anon_sym_EQ] = ACTIONS(1453),
    [anon_sym_EQ_EQ] = ACTIONS(1451),
    [anon_sym_BANG_EQ] = ACTIONS(1451),
    [anon_sym_GT] = ACTIONS(1453),
    [anon_sym_LT] = ACTIONS(1453),
    [anon_sym_GT_EQ] = ACTIONS(1451),
    [anon_sym_LT_EQ] = ACTIONS(1451),
    [anon_sym_DOT] = ACTIONS(1453),
    [anon_sym_DOT_DOT] = ACTIONS(1453),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1451),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1451),
    [anon_sym_COLON_COLON] = ACTIONS(1451),
    [anon_sym_POUND] = ACTIONS(1451),
    [anon_sym_SQUOTE] = ACTIONS(1453),
    [anon_sym_as] = ACTIONS(1453),
    [anon_sym_async] = ACTIONS(1453),
    [anon_sym_break] = ACTIONS(1453),
    [anon_sym_const] = ACTIONS(1453),
    [anon_sym_continue] = ACTIONS(1453),
    [anon_sym_default] = ACTIONS(1453),
    [anon_sym_enum] = ACTIONS(1453),
    [anon_sym_fn] = ACTIONS(1453),
    [anon_sym_for] = ACTIONS(1453),
    [anon_sym_gen] = ACTIONS(1453),
    [anon_sym_if] = ACTIONS(1453),
    [anon_sym_impl] = ACTIONS(1453),
    [anon_sym_let] = ACTIONS(1453),
    [anon_sym_loop] = ACTIONS(1453),
    [anon_sym_match] = ACTIONS(1453),
    [anon_sym_mod] = ACTIONS(1453),
    [anon_sym_pub] = ACTIONS(1453),
    [anon_sym_return] = ACTIONS(1453),
    [anon_sym_static] = ACTIONS(1453),
    [anon_sym_struct] = ACTIONS(1453),
    [anon_sym_trait] = ACTIONS(1453),
    [anon_sym_type] = ACTIONS(1453),
    [anon_sym_union] = ACTIONS(1453),
    [anon_sym_unsafe] = ACTIONS(1453),
    [anon_sym_use] = ACTIONS(1453),
    [anon_sym_while] = ACTIONS(1453),
    [anon_sym_extern] = ACTIONS(1453),
    [anon_sym_yield] = ACTIONS(1453),
    [anon_sym_move] = ACTIONS(1453),
    [anon_sym_try] = ACTIONS(1453),
    [sym_integer_literal] = ACTIONS(1451),
    [aux_sym_string_literal_token1] = ACTIONS(1451),
    [sym_char_literal] = ACTIONS(1451),
    [anon_sym_true] = ACTIONS(1453),
    [anon_sym_false] = ACTIONS(1453),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1453),
    [sym_super] = ACTIONS(1453),
    [sym_crate] = ACTIONS(1453),
    [sym_metavariable] = ACTIONS(1451),
    [sym__raw_string_literal_start] = ACTIONS(1451),
    [sym_float_literal] = ACTIONS(1451),
  },
  [397] = {
    [sym_line_comment] = STATE(397),
    [sym_block_comment] = STATE(397),
    [ts_builtin_sym_end] = ACTIONS(1455),
    [sym_identifier] = ACTIONS(1457),
    [anon_sym_SEMI] = ACTIONS(1455),
    [anon_sym_macro_rules_BANG] = ACTIONS(1455),
    [anon_sym_LPAREN] = ACTIONS(1455),
    [anon_sym_LBRACK] = ACTIONS(1455),
    [anon_sym_LBRACE] = ACTIONS(1455),
    [anon_sym_RBRACE] = ACTIONS(1455),
    [anon_sym_PLUS] = ACTIONS(1457),
    [anon_sym_STAR] = ACTIONS(1457),
    [anon_sym_QMARK] = ACTIONS(1455),
    [anon_sym_u8] = ACTIONS(1457),
    [anon_sym_i8] = ACTIONS(1457),
    [anon_sym_u16] = ACTIONS(1457),
    [anon_sym_i16] = ACTIONS(1457),
    [anon_sym_u32] = ACTIONS(1457),
    [anon_sym_i32] = ACTIONS(1457),
    [anon_sym_u64] = ACTIONS(1457),
    [anon_sym_i64] = ACTIONS(1457),
    [anon_sym_u128] = ACTIONS(1457),
    [anon_sym_i128] = ACTIONS(1457),
    [anon_sym_isize] = ACTIONS(1457),
    [anon_sym_usize] = ACTIONS(1457),
    [anon_sym_f32] = ACTIONS(1457),
    [anon_sym_f64] = ACTIONS(1457),
    [anon_sym_bool] = ACTIONS(1457),
    [anon_sym_str] = ACTIONS(1457),
    [anon_sym_char] = ACTIONS(1457),
    [anon_sym_DASH] = ACTIONS(1457),
    [anon_sym_SLASH] = ACTIONS(1457),
    [anon_sym_PERCENT] = ACTIONS(1457),
    [anon_sym_CARET] = ACTIONS(1457),
    [anon_sym_BANG] = ACTIONS(1457),
    [anon_sym_AMP] = ACTIONS(1457),
    [anon_sym_PIPE] = ACTIONS(1457),
    [anon_sym_AMP_AMP] = ACTIONS(1455),
    [anon_sym_PIPE_PIPE] = ACTIONS(1455),
    [anon_sym_LT_LT] = ACTIONS(1457),
    [anon_sym_GT_GT] = ACTIONS(1457),
    [anon_sym_PLUS_EQ] = ACTIONS(1455),
    [anon_sym_DASH_EQ] = ACTIONS(1455),
    [anon_sym_STAR_EQ] = ACTIONS(1455),
    [anon_sym_SLASH_EQ] = ACTIONS(1455),
    [anon_sym_PERCENT_EQ] = ACTIONS(1455),
    [anon_sym_CARET_EQ] = ACTIONS(1455),
    [anon_sym_AMP_EQ] = ACTIONS(1455),
    [anon_sym_PIPE_EQ] = ACTIONS(1455),
    [anon_sym_LT_LT_EQ] = ACTIONS(1455),
    [anon_sym_GT_GT_EQ] = ACTIONS(1455),
    [anon_sym_EQ] = ACTIONS(1457),
    [anon_sym_EQ_EQ] = ACTIONS(1455),
    [anon_sym_BANG_EQ] = ACTIONS(1455),
    [anon_sym_GT] = ACTIONS(1457),
    [anon_sym_LT] = ACTIONS(1457),
    [anon_sym_GT_EQ] = ACTIONS(1455),
    [anon_sym_LT_EQ] = ACTIONS(1455),
    [anon_sym_DOT] = ACTIONS(1457),
    [anon_sym_DOT_DOT] = ACTIONS(1457),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1455),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1455),
    [anon_sym_COLON_COLON] = ACTIONS(1455),
    [anon_sym_POUND] = ACTIONS(1455),
    [anon_sym_SQUOTE] = ACTIONS(1457),
    [anon_sym_as] = ACTIONS(1457),
    [anon_sym_async] = ACTIONS(1457),
    [anon_sym_break] = ACTIONS(1457),
    [anon_sym_const] = ACTIONS(1457),
    [anon_sym_continue] = ACTIONS(1457),
    [anon_sym_default] = ACTIONS(1457),
    [anon_sym_enum] = ACTIONS(1457),
    [anon_sym_fn] = ACTIONS(1457),
    [anon_sym_for] = ACTIONS(1457),
    [anon_sym_gen] = ACTIONS(1457),
    [anon_sym_if] = ACTIONS(1457),
    [anon_sym_impl] = ACTIONS(1457),
    [anon_sym_let] = ACTIONS(1457),
    [anon_sym_loop] = ACTIONS(1457),
    [anon_sym_match] = ACTIONS(1457),
    [anon_sym_mod] = ACTIONS(1457),
    [anon_sym_pub] = ACTIONS(1457),
    [anon_sym_return] = ACTIONS(1457),
    [anon_sym_static] = ACTIONS(1457),
    [anon_sym_struct] = ACTIONS(1457),
    [anon_sym_trait] = ACTIONS(1457),
    [anon_sym_type] = ACTIONS(1457),
    [anon_sym_union] = ACTIONS(1457),
    [anon_sym_unsafe] = ACTIONS(1457),
    [anon_sym_use] = ACTIONS(1457),
    [anon_sym_while] = ACTIONS(1457),
    [anon_sym_extern] = ACTIONS(1457),
    [anon_sym_yield] = ACTIONS(1457),
    [anon_sym_move] = ACTIONS(1457),
    [anon_sym_try] = ACTIONS(1457),
    [sym_integer_literal] = ACTIONS(1455),
    [aux_sym_string_literal_token1] = ACTIONS(1455),
    [sym_char_literal] = ACTIONS(1455),
    [anon_sym_true] = ACTIONS(1457),
    [anon_sym_false] = ACTIONS(1457),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1457),
    [sym_super] = ACTIONS(1457),
    [sym_crate] = ACTIONS(1457),
    [sym_metavariable] = ACTIONS(1455),
    [sym__raw_string_literal_start] = ACTIONS(1455),
    [sym_float_literal] = ACTIONS(1455),
  },
  [398] = {
    [sym_line_comment] = STATE(398),
    [sym_block_comment] = STATE(398),
    [ts_builtin_sym_end] = ACTIONS(1459),
    [sym_identifier] = ACTIONS(1461),
    [anon_sym_SEMI] = ACTIONS(1459),
    [anon_sym_macro_rules_BANG] = ACTIONS(1459),
    [anon_sym_LPAREN] = ACTIONS(1459),
    [anon_sym_LBRACK] = ACTIONS(1459),
    [anon_sym_LBRACE] = ACTIONS(1459),
    [anon_sym_RBRACE] = ACTIONS(1459),
    [anon_sym_PLUS] = ACTIONS(1461),
    [anon_sym_STAR] = ACTIONS(1461),
    [anon_sym_QMARK] = ACTIONS(1459),
    [anon_sym_u8] = ACTIONS(1461),
    [anon_sym_i8] = ACTIONS(1461),
    [anon_sym_u16] = ACTIONS(1461),
    [anon_sym_i16] = ACTIONS(1461),
    [anon_sym_u32] = ACTIONS(1461),
    [anon_sym_i32] = ACTIONS(1461),
    [anon_sym_u64] = ACTIONS(1461),
    [anon_sym_i64] = ACTIONS(1461),
    [anon_sym_u128] = ACTIONS(1461),
    [anon_sym_i128] = ACTIONS(1461),
    [anon_sym_isize] = ACTIONS(1461),
    [anon_sym_usize] = ACTIONS(1461),
    [anon_sym_f32] = ACTIONS(1461),
    [anon_sym_f64] = ACTIONS(1461),
    [anon_sym_bool] = ACTIONS(1461),
    [anon_sym_str] = ACTIONS(1461),
    [anon_sym_char] = ACTIONS(1461),
    [anon_sym_DASH] = ACTIONS(1461),
    [anon_sym_SLASH] = ACTIONS(1461),
    [anon_sym_PERCENT] = ACTIONS(1461),
    [anon_sym_CARET] = ACTIONS(1461),
    [anon_sym_BANG] = ACTIONS(1461),
    [anon_sym_AMP] = ACTIONS(1461),
    [anon_sym_PIPE] = ACTIONS(1461),
    [anon_sym_AMP_AMP] = ACTIONS(1459),
    [anon_sym_PIPE_PIPE] = ACTIONS(1459),
    [anon_sym_LT_LT] = ACTIONS(1461),
    [anon_sym_GT_GT] = ACTIONS(1461),
    [anon_sym_PLUS_EQ] = ACTIONS(1459),
    [anon_sym_DASH_EQ] = ACTIONS(1459),
    [anon_sym_STAR_EQ] = ACTIONS(1459),
    [anon_sym_SLASH_EQ] = ACTIONS(1459),
    [anon_sym_PERCENT_EQ] = ACTIONS(1459),
    [anon_sym_CARET_EQ] = ACTIONS(1459),
    [anon_sym_AMP_EQ] = ACTIONS(1459),
    [anon_sym_PIPE_EQ] = ACTIONS(1459),
    [anon_sym_LT_LT_EQ] = ACTIONS(1459),
    [anon_sym_GT_GT_EQ] = ACTIONS(1459),
    [anon_sym_EQ] = ACTIONS(1461),
    [anon_sym_EQ_EQ] = ACTIONS(1459),
    [anon_sym_BANG_EQ] = ACTIONS(1459),
    [anon_sym_GT] = ACTIONS(1461),
    [anon_sym_LT] = ACTIONS(1461),
    [anon_sym_GT_EQ] = ACTIONS(1459),
    [anon_sym_LT_EQ] = ACTIONS(1459),
    [anon_sym_DOT] = ACTIONS(1461),
    [anon_sym_DOT_DOT] = ACTIONS(1461),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1459),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1459),
    [anon_sym_COLON_COLON] = ACTIONS(1459),
    [anon_sym_POUND] = ACTIONS(1459),
    [anon_sym_SQUOTE] = ACTIONS(1461),
    [anon_sym_as] = ACTIONS(1461),
    [anon_sym_async] = ACTIONS(1461),
    [anon_sym_break] = ACTIONS(1461),
    [anon_sym_const] = ACTIONS(1461),
    [anon_sym_continue] = ACTIONS(1461),
    [anon_sym_default] = ACTIONS(1461),
    [anon_sym_enum] = ACTIONS(1461),
    [anon_sym_fn] = ACTIONS(1461),
    [anon_sym_for] = ACTIONS(1461),
    [anon_sym_gen] = ACTIONS(1461),
    [anon_sym_if] = ACTIONS(1461),
    [anon_sym_impl] = ACTIONS(1461),
    [anon_sym_let] = ACTIONS(1461),
    [anon_sym_loop] = ACTIONS(1461),
    [anon_sym_match] = ACTIONS(1461),
    [anon_sym_mod] = ACTIONS(1461),
    [anon_sym_pub] = ACTIONS(1461),
    [anon_sym_return] = ACTIONS(1461),
    [anon_sym_static] = ACTIONS(1461),
    [anon_sym_struct] = ACTIONS(1461),
    [anon_sym_trait] = ACTIONS(1461),
    [anon_sym_type] = ACTIONS(1461),
    [anon_sym_union] = ACTIONS(1461),
    [anon_sym_unsafe] = ACTIONS(1461),
    [anon_sym_use] = ACTIONS(1461),
    [anon_sym_while] = ACTIONS(1461),
    [anon_sym_extern] = ACTIONS(1461),
    [anon_sym_yield] = ACTIONS(1461),
    [anon_sym_move] = ACTIONS(1461),
    [anon_sym_try] = ACTIONS(1461),
    [sym_integer_literal] = ACTIONS(1459),
    [aux_sym_string_literal_token1] = ACTIONS(1459),
    [sym_char_literal] = ACTIONS(1459),
    [anon_sym_true] = ACTIONS(1461),
    [anon_sym_false] = ACTIONS(1461),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1461),
    [sym_super] = ACTIONS(1461),
    [sym_crate] = ACTIONS(1461),
    [sym_metavariable] = ACTIONS(1459),
    [sym__raw_string_literal_start] = ACTIONS(1459),
    [sym_float_literal] = ACTIONS(1459),
  },
  [399] = {
    [sym_line_comment] = STATE(399),
    [sym_block_comment] = STATE(399),
    [ts_builtin_sym_end] = ACTIONS(1463),
    [sym_identifier] = ACTIONS(1465),
    [anon_sym_SEMI] = ACTIONS(1359),
    [anon_sym_macro_rules_BANG] = ACTIONS(1463),
    [anon_sym_LPAREN] = ACTIONS(1359),
    [anon_sym_LBRACK] = ACTIONS(1359),
    [anon_sym_LBRACE] = ACTIONS(1463),
    [anon_sym_RBRACE] = ACTIONS(1359),
    [anon_sym_PLUS] = ACTIONS(1357),
    [anon_sym_STAR] = ACTIONS(1357),
    [anon_sym_QMARK] = ACTIONS(1359),
    [anon_sym_u8] = ACTIONS(1465),
    [anon_sym_i8] = ACTIONS(1465),
    [anon_sym_u16] = ACTIONS(1465),
    [anon_sym_i16] = ACTIONS(1465),
    [anon_sym_u32] = ACTIONS(1465),
    [anon_sym_i32] = ACTIONS(1465),
    [anon_sym_u64] = ACTIONS(1465),
    [anon_sym_i64] = ACTIONS(1465),
    [anon_sym_u128] = ACTIONS(1465),
    [anon_sym_i128] = ACTIONS(1465),
    [anon_sym_isize] = ACTIONS(1465),
    [anon_sym_usize] = ACTIONS(1465),
    [anon_sym_f32] = ACTIONS(1465),
    [anon_sym_f64] = ACTIONS(1465),
    [anon_sym_bool] = ACTIONS(1465),
    [anon_sym_str] = ACTIONS(1465),
    [anon_sym_char] = ACTIONS(1465),
    [anon_sym_DASH] = ACTIONS(1357),
    [anon_sym_SLASH] = ACTIONS(1357),
    [anon_sym_PERCENT] = ACTIONS(1357),
    [anon_sym_CARET] = ACTIONS(1357),
    [anon_sym_BANG] = ACTIONS(1465),
    [anon_sym_AMP] = ACTIONS(1357),
    [anon_sym_PIPE] = ACTIONS(1357),
    [anon_sym_AMP_AMP] = ACTIONS(1359),
    [anon_sym_PIPE_PIPE] = ACTIONS(1359),
    [anon_sym_LT_LT] = ACTIONS(1357),
    [anon_sym_GT_GT] = ACTIONS(1357),
    [anon_sym_PLUS_EQ] = ACTIONS(1359),
    [anon_sym_DASH_EQ] = ACTIONS(1359),
    [anon_sym_STAR_EQ] = ACTIONS(1359),
    [anon_sym_SLASH_EQ] = ACTIONS(1359),
    [anon_sym_PERCENT_EQ] = ACTIONS(1359),
    [anon_sym_CARET_EQ] = ACTIONS(1359),
    [anon_sym_AMP_EQ] = ACTIONS(1359),
    [anon_sym_PIPE_EQ] = ACTIONS(1359),
    [anon_sym_LT_LT_EQ] = ACTIONS(1359),
    [anon_sym_GT_GT_EQ] = ACTIONS(1359),
    [anon_sym_EQ] = ACTIONS(1357),
    [anon_sym_EQ_EQ] = ACTIONS(1359),
    [anon_sym_BANG_EQ] = ACTIONS(1359),
    [anon_sym_GT] = ACTIONS(1357),
    [anon_sym_LT] = ACTIONS(1357),
    [anon_sym_GT_EQ] = ACTIONS(1359),
    [anon_sym_LT_EQ] = ACTIONS(1359),
    [anon_sym_DOT] = ACTIONS(1357),
    [anon_sym_DOT_DOT] = ACTIONS(1357),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1359),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1359),
    [anon_sym_COLON_COLON] = ACTIONS(1463),
    [anon_sym_POUND] = ACTIONS(1463),
    [anon_sym_SQUOTE] = ACTIONS(1465),
    [anon_sym_as] = ACTIONS(1357),
    [anon_sym_async] = ACTIONS(1465),
    [anon_sym_break] = ACTIONS(1465),
    [anon_sym_const] = ACTIONS(1465),
    [anon_sym_continue] = ACTIONS(1465),
    [anon_sym_default] = ACTIONS(1465),
    [anon_sym_enum] = ACTIONS(1465),
    [anon_sym_fn] = ACTIONS(1465),
    [anon_sym_for] = ACTIONS(1465),
    [anon_sym_gen] = ACTIONS(1465),
    [anon_sym_if] = ACTIONS(1465),
    [anon_sym_impl] = ACTIONS(1465),
    [anon_sym_let] = ACTIONS(1465),
    [anon_sym_loop] = ACTIONS(1465),
    [anon_sym_match] = ACTIONS(1465),
    [anon_sym_mod] = ACTIONS(1465),
    [anon_sym_pub] = ACTIONS(1465),
    [anon_sym_return] = ACTIONS(1465),
    [anon_sym_static] = ACTIONS(1465),
    [anon_sym_struct] = ACTIONS(1465),
    [anon_sym_trait] = ACTIONS(1465),
    [anon_sym_type] = ACTIONS(1465),
    [anon_sym_union] = ACTIONS(1465),
    [anon_sym_unsafe] = ACTIONS(1465),
    [anon_sym_use] = ACTIONS(1465),
    [anon_sym_while] = ACTIONS(1465),
    [anon_sym_extern] = ACTIONS(1465),
    [anon_sym_yield] = ACTIONS(1465),
    [anon_sym_move] = ACTIONS(1465),
    [anon_sym_try] = ACTIONS(1465),
    [sym_integer_literal] = ACTIONS(1463),
    [aux_sym_string_literal_token1] = ACTIONS(1463),
    [sym_char_literal] = ACTIONS(1463),
    [anon_sym_true] = ACTIONS(1465),
    [anon_sym_false] = ACTIONS(1465),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1465),
    [sym_super] = ACTIONS(1465),
    [sym_crate] = ACTIONS(1465),
    [sym_metavariable] = ACTIONS(1463),
    [sym__raw_string_literal_start] = ACTIONS(1463),
    [sym_float_literal] = ACTIONS(1463),
  },
  [400] = {
    [sym_line_comment] = STATE(400),
    [sym_block_comment] = STATE(400),
    [ts_builtin_sym_end] = ACTIONS(1467),
    [sym_identifier] = ACTIONS(1469),
    [anon_sym_SEMI] = ACTIONS(1467),
    [anon_sym_macro_rules_BANG] = ACTIONS(1467),
    [anon_sym_LPAREN] = ACTIONS(1467),
    [anon_sym_LBRACK] = ACTIONS(1467),
    [anon_sym_LBRACE] = ACTIONS(1467),
    [anon_sym_RBRACE] = ACTIONS(1467),
    [anon_sym_PLUS] = ACTIONS(1469),
    [anon_sym_STAR] = ACTIONS(1469),
    [anon_sym_QMARK] = ACTIONS(1467),
    [anon_sym_u8] = ACTIONS(1469),
    [anon_sym_i8] = ACTIONS(1469),
    [anon_sym_u16] = ACTIONS(1469),
    [anon_sym_i16] = ACTIONS(1469),
    [anon_sym_u32] = ACTIONS(1469),
    [anon_sym_i32] = ACTIONS(1469),
    [anon_sym_u64] = ACTIONS(1469),
    [anon_sym_i64] = ACTIONS(1469),
    [anon_sym_u128] = ACTIONS(1469),
    [anon_sym_i128] = ACTIONS(1469),
    [anon_sym_isize] = ACTIONS(1469),
    [anon_sym_usize] = ACTIONS(1469),
    [anon_sym_f32] = ACTIONS(1469),
    [anon_sym_f64] = ACTIONS(1469),
    [anon_sym_bool] = ACTIONS(1469),
    [anon_sym_str] = ACTIONS(1469),
    [anon_sym_char] = ACTIONS(1469),
    [anon_sym_DASH] = ACTIONS(1469),
    [anon_sym_SLASH] = ACTIONS(1469),
    [anon_sym_PERCENT] = ACTIONS(1469),
    [anon_sym_CARET] = ACTIONS(1469),
    [anon_sym_BANG] = ACTIONS(1469),
    [anon_sym_AMP] = ACTIONS(1469),
    [anon_sym_PIPE] = ACTIONS(1469),
    [anon_sym_AMP_AMP] = ACTIONS(1467),
    [anon_sym_PIPE_PIPE] = ACTIONS(1467),
    [anon_sym_LT_LT] = ACTIONS(1469),
    [anon_sym_GT_GT] = ACTIONS(1469),
    [anon_sym_PLUS_EQ] = ACTIONS(1467),
    [anon_sym_DASH_EQ] = ACTIONS(1467),
    [anon_sym_STAR_EQ] = ACTIONS(1467),
    [anon_sym_SLASH_EQ] = ACTIONS(1467),
    [anon_sym_PERCENT_EQ] = ACTIONS(1467),
    [anon_sym_CARET_EQ] = ACTIONS(1467),
    [anon_sym_AMP_EQ] = ACTIONS(1467),
    [anon_sym_PIPE_EQ] = ACTIONS(1467),
    [anon_sym_LT_LT_EQ] = ACTIONS(1467),
    [anon_sym_GT_GT_EQ] = ACTIONS(1467),
    [anon_sym_EQ] = ACTIONS(1469),
    [anon_sym_EQ_EQ] = ACTIONS(1467),
    [anon_sym_BANG_EQ] = ACTIONS(1467),
    [anon_sym_GT] = ACTIONS(1469),
    [anon_sym_LT] = ACTIONS(1469),
    [anon_sym_GT_EQ] = ACTIONS(1467),
    [anon_sym_LT_EQ] = ACTIONS(1467),
    [anon_sym_DOT] = ACTIONS(1469),
    [anon_sym_DOT_DOT] = ACTIONS(1469),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1467),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1467),
    [anon_sym_COLON_COLON] = ACTIONS(1467),
    [anon_sym_POUND] = ACTIONS(1467),
    [anon_sym_SQUOTE] = ACTIONS(1469),
    [anon_sym_as] = ACTIONS(1469),
    [anon_sym_async] = ACTIONS(1469),
    [anon_sym_break] = ACTIONS(1469),
    [anon_sym_const] = ACTIONS(1469),
    [anon_sym_continue] = ACTIONS(1469),
    [anon_sym_default] = ACTIONS(1469),
    [anon_sym_enum] = ACTIONS(1469),
    [anon_sym_fn] = ACTIONS(1469),
    [anon_sym_for] = ACTIONS(1469),
    [anon_sym_gen] = ACTIONS(1469),
    [anon_sym_if] = ACTIONS(1469),
    [anon_sym_impl] = ACTIONS(1469),
    [anon_sym_let] = ACTIONS(1469),
    [anon_sym_loop] = ACTIONS(1469),
    [anon_sym_match] = ACTIONS(1469),
    [anon_sym_mod] = ACTIONS(1469),
    [anon_sym_pub] = ACTIONS(1469),
    [anon_sym_return] = ACTIONS(1469),
    [anon_sym_static] = ACTIONS(1469),
    [anon_sym_struct] = ACTIONS(1469),
    [anon_sym_trait] = ACTIONS(1469),
    [anon_sym_type] = ACTIONS(1469),
    [anon_sym_union] = ACTIONS(1469),
    [anon_sym_unsafe] = ACTIONS(1469),
    [anon_sym_use] = ACTIONS(1469),
    [anon_sym_while] = ACTIONS(1469),
    [anon_sym_extern] = ACTIONS(1469),
    [anon_sym_yield] = ACTIONS(1469),
    [anon_sym_move] = ACTIONS(1469),
    [anon_sym_try] = ACTIONS(1469),
    [sym_integer_literal] = ACTIONS(1467),
    [aux_sym_string_literal_token1] = ACTIONS(1467),
    [sym_char_literal] = ACTIONS(1467),
    [anon_sym_true] = ACTIONS(1469),
    [anon_sym_false] = ACTIONS(1469),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1469),
    [sym_super] = ACTIONS(1469),
    [sym_crate] = ACTIONS(1469),
    [sym_metavariable] = ACTIONS(1467),
    [sym__raw_string_literal_start] = ACTIONS(1467),
    [sym_float_literal] = ACTIONS(1467),
  },
  [401] = {
    [sym_attribute_item] = STATE(420),
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_self_parameter] = STATE(2999),
    [sym_variadic_parameter] = STATE(2999),
    [sym_parameter] = STATE(2999),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2746),
    [sym_bracketed_type] = STATE(3557),
    [sym_lifetime] = STATE(2800),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3301),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(2446),
    [sym_scoped_identifier] = STATE(2125),
    [sym_scoped_type_identifier] = STATE(2118),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2431),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(401),
    [sym_block_comment] = STATE(401),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1369),
    [anon_sym_LPAREN] = ACTIONS(1371),
    [anon_sym_RPAREN] = ACTIONS(1471),
    [anon_sym_LBRACK] = ACTIONS(1287),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1375),
    [anon_sym_i8] = ACTIONS(1375),
    [anon_sym_u16] = ACTIONS(1375),
    [anon_sym_i16] = ACTIONS(1375),
    [anon_sym_u32] = ACTIONS(1375),
    [anon_sym_i32] = ACTIONS(1375),
    [anon_sym_u64] = ACTIONS(1375),
    [anon_sym_i64] = ACTIONS(1375),
    [anon_sym_u128] = ACTIONS(1375),
    [anon_sym_i128] = ACTIONS(1375),
    [anon_sym_isize] = ACTIONS(1375),
    [anon_sym_usize] = ACTIONS(1375),
    [anon_sym_f32] = ACTIONS(1375),
    [anon_sym_f64] = ACTIONS(1375),
    [anon_sym_bool] = ACTIONS(1375),
    [anon_sym_str] = ACTIONS(1375),
    [anon_sym_char] = ACTIONS(1375),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1377),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1379),
    [anon_sym_DOT_DOT] = ACTIONS(1305),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1307),
    [anon_sym_COMMA] = ACTIONS(1473),
    [anon_sym_COLON_COLON] = ACTIONS(1383),
    [anon_sym_POUND] = ACTIONS(1313),
    [anon_sym_SQUOTE] = ACTIONS(1315),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1319),
    [anon_sym_default] = ACTIONS(1385),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1387),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1387),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_ref] = ACTIONS(1333),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_mutable_specifier] = ACTIONS(1337),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1389),
    [sym_super] = ACTIONS(1391),
    [sym_crate] = ACTIONS(1391),
    [sym_metavariable] = ACTIONS(1393),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [402] = {
    [sym_line_comment] = STATE(402),
    [sym_block_comment] = STATE(402),
    [ts_builtin_sym_end] = ACTIONS(1475),
    [sym_identifier] = ACTIONS(1477),
    [anon_sym_SEMI] = ACTIONS(1475),
    [anon_sym_macro_rules_BANG] = ACTIONS(1475),
    [anon_sym_LPAREN] = ACTIONS(1475),
    [anon_sym_LBRACK] = ACTIONS(1475),
    [anon_sym_LBRACE] = ACTIONS(1475),
    [anon_sym_RBRACE] = ACTIONS(1475),
    [anon_sym_PLUS] = ACTIONS(1477),
    [anon_sym_STAR] = ACTIONS(1477),
    [anon_sym_QMARK] = ACTIONS(1475),
    [anon_sym_u8] = ACTIONS(1477),
    [anon_sym_i8] = ACTIONS(1477),
    [anon_sym_u16] = ACTIONS(1477),
    [anon_sym_i16] = ACTIONS(1477),
    [anon_sym_u32] = ACTIONS(1477),
    [anon_sym_i32] = ACTIONS(1477),
    [anon_sym_u64] = ACTIONS(1477),
    [anon_sym_i64] = ACTIONS(1477),
    [anon_sym_u128] = ACTIONS(1477),
    [anon_sym_i128] = ACTIONS(1477),
    [anon_sym_isize] = ACTIONS(1477),
    [anon_sym_usize] = ACTIONS(1477),
    [anon_sym_f32] = ACTIONS(1477),
    [anon_sym_f64] = ACTIONS(1477),
    [anon_sym_bool] = ACTIONS(1477),
    [anon_sym_str] = ACTIONS(1477),
    [anon_sym_char] = ACTIONS(1477),
    [anon_sym_DASH] = ACTIONS(1477),
    [anon_sym_SLASH] = ACTIONS(1477),
    [anon_sym_PERCENT] = ACTIONS(1477),
    [anon_sym_CARET] = ACTIONS(1477),
    [anon_sym_BANG] = ACTIONS(1477),
    [anon_sym_AMP] = ACTIONS(1477),
    [anon_sym_PIPE] = ACTIONS(1477),
    [anon_sym_AMP_AMP] = ACTIONS(1475),
    [anon_sym_PIPE_PIPE] = ACTIONS(1475),
    [anon_sym_LT_LT] = ACTIONS(1477),
    [anon_sym_GT_GT] = ACTIONS(1477),
    [anon_sym_PLUS_EQ] = ACTIONS(1475),
    [anon_sym_DASH_EQ] = ACTIONS(1475),
    [anon_sym_STAR_EQ] = ACTIONS(1475),
    [anon_sym_SLASH_EQ] = ACTIONS(1475),
    [anon_sym_PERCENT_EQ] = ACTIONS(1475),
    [anon_sym_CARET_EQ] = ACTIONS(1475),
    [anon_sym_AMP_EQ] = ACTIONS(1475),
    [anon_sym_PIPE_EQ] = ACTIONS(1475),
    [anon_sym_LT_LT_EQ] = ACTIONS(1475),
    [anon_sym_GT_GT_EQ] = ACTIONS(1475),
    [anon_sym_EQ] = ACTIONS(1477),
    [anon_sym_EQ_EQ] = ACTIONS(1475),
    [anon_sym_BANG_EQ] = ACTIONS(1475),
    [anon_sym_GT] = ACTIONS(1477),
    [anon_sym_LT] = ACTIONS(1477),
    [anon_sym_GT_EQ] = ACTIONS(1475),
    [anon_sym_LT_EQ] = ACTIONS(1475),
    [anon_sym_DOT] = ACTIONS(1477),
    [anon_sym_DOT_DOT] = ACTIONS(1477),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1475),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1475),
    [anon_sym_COLON_COLON] = ACTIONS(1475),
    [anon_sym_POUND] = ACTIONS(1475),
    [anon_sym_SQUOTE] = ACTIONS(1477),
    [anon_sym_as] = ACTIONS(1477),
    [anon_sym_async] = ACTIONS(1477),
    [anon_sym_break] = ACTIONS(1477),
    [anon_sym_const] = ACTIONS(1477),
    [anon_sym_continue] = ACTIONS(1477),
    [anon_sym_default] = ACTIONS(1477),
    [anon_sym_enum] = ACTIONS(1477),
    [anon_sym_fn] = ACTIONS(1477),
    [anon_sym_for] = ACTIONS(1477),
    [anon_sym_gen] = ACTIONS(1477),
    [anon_sym_if] = ACTIONS(1477),
    [anon_sym_impl] = ACTIONS(1477),
    [anon_sym_let] = ACTIONS(1477),
    [anon_sym_loop] = ACTIONS(1477),
    [anon_sym_match] = ACTIONS(1477),
    [anon_sym_mod] = ACTIONS(1477),
    [anon_sym_pub] = ACTIONS(1477),
    [anon_sym_return] = ACTIONS(1477),
    [anon_sym_static] = ACTIONS(1477),
    [anon_sym_struct] = ACTIONS(1477),
    [anon_sym_trait] = ACTIONS(1477),
    [anon_sym_type] = ACTIONS(1477),
    [anon_sym_union] = ACTIONS(1477),
    [anon_sym_unsafe] = ACTIONS(1477),
    [anon_sym_use] = ACTIONS(1477),
    [anon_sym_while] = ACTIONS(1477),
    [anon_sym_extern] = ACTIONS(1477),
    [anon_sym_yield] = ACTIONS(1477),
    [anon_sym_move] = ACTIONS(1477),
    [anon_sym_try] = ACTIONS(1477),
    [sym_integer_literal] = ACTIONS(1475),
    [aux_sym_string_literal_token1] = ACTIONS(1475),
    [sym_char_literal] = ACTIONS(1475),
    [anon_sym_true] = ACTIONS(1477),
    [anon_sym_false] = ACTIONS(1477),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1477),
    [sym_super] = ACTIONS(1477),
    [sym_crate] = ACTIONS(1477),
    [sym_metavariable] = ACTIONS(1475),
    [sym__raw_string_literal_start] = ACTIONS(1475),
    [sym_float_literal] = ACTIONS(1475),
  },
  [403] = {
    [sym_line_comment] = STATE(403),
    [sym_block_comment] = STATE(403),
    [ts_builtin_sym_end] = ACTIONS(1479),
    [sym_identifier] = ACTIONS(1481),
    [anon_sym_SEMI] = ACTIONS(1479),
    [anon_sym_macro_rules_BANG] = ACTIONS(1479),
    [anon_sym_LPAREN] = ACTIONS(1479),
    [anon_sym_LBRACK] = ACTIONS(1479),
    [anon_sym_LBRACE] = ACTIONS(1479),
    [anon_sym_RBRACE] = ACTIONS(1479),
    [anon_sym_PLUS] = ACTIONS(1481),
    [anon_sym_STAR] = ACTIONS(1481),
    [anon_sym_QMARK] = ACTIONS(1479),
    [anon_sym_u8] = ACTIONS(1481),
    [anon_sym_i8] = ACTIONS(1481),
    [anon_sym_u16] = ACTIONS(1481),
    [anon_sym_i16] = ACTIONS(1481),
    [anon_sym_u32] = ACTIONS(1481),
    [anon_sym_i32] = ACTIONS(1481),
    [anon_sym_u64] = ACTIONS(1481),
    [anon_sym_i64] = ACTIONS(1481),
    [anon_sym_u128] = ACTIONS(1481),
    [anon_sym_i128] = ACTIONS(1481),
    [anon_sym_isize] = ACTIONS(1481),
    [anon_sym_usize] = ACTIONS(1481),
    [anon_sym_f32] = ACTIONS(1481),
    [anon_sym_f64] = ACTIONS(1481),
    [anon_sym_bool] = ACTIONS(1481),
    [anon_sym_str] = ACTIONS(1481),
    [anon_sym_char] = ACTIONS(1481),
    [anon_sym_DASH] = ACTIONS(1481),
    [anon_sym_SLASH] = ACTIONS(1481),
    [anon_sym_PERCENT] = ACTIONS(1481),
    [anon_sym_CARET] = ACTIONS(1481),
    [anon_sym_BANG] = ACTIONS(1481),
    [anon_sym_AMP] = ACTIONS(1481),
    [anon_sym_PIPE] = ACTIONS(1481),
    [anon_sym_AMP_AMP] = ACTIONS(1479),
    [anon_sym_PIPE_PIPE] = ACTIONS(1479),
    [anon_sym_LT_LT] = ACTIONS(1481),
    [anon_sym_GT_GT] = ACTIONS(1481),
    [anon_sym_PLUS_EQ] = ACTIONS(1479),
    [anon_sym_DASH_EQ] = ACTIONS(1479),
    [anon_sym_STAR_EQ] = ACTIONS(1479),
    [anon_sym_SLASH_EQ] = ACTIONS(1479),
    [anon_sym_PERCENT_EQ] = ACTIONS(1479),
    [anon_sym_CARET_EQ] = ACTIONS(1479),
    [anon_sym_AMP_EQ] = ACTIONS(1479),
    [anon_sym_PIPE_EQ] = ACTIONS(1479),
    [anon_sym_LT_LT_EQ] = ACTIONS(1479),
    [anon_sym_GT_GT_EQ] = ACTIONS(1479),
    [anon_sym_EQ] = ACTIONS(1481),
    [anon_sym_EQ_EQ] = ACTIONS(1479),
    [anon_sym_BANG_EQ] = ACTIONS(1479),
    [anon_sym_GT] = ACTIONS(1481),
    [anon_sym_LT] = ACTIONS(1481),
    [anon_sym_GT_EQ] = ACTIONS(1479),
    [anon_sym_LT_EQ] = ACTIONS(1479),
    [anon_sym_DOT] = ACTIONS(1481),
    [anon_sym_DOT_DOT] = ACTIONS(1481),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1479),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1479),
    [anon_sym_COLON_COLON] = ACTIONS(1479),
    [anon_sym_POUND] = ACTIONS(1479),
    [anon_sym_SQUOTE] = ACTIONS(1481),
    [anon_sym_as] = ACTIONS(1481),
    [anon_sym_async] = ACTIONS(1481),
    [anon_sym_break] = ACTIONS(1481),
    [anon_sym_const] = ACTIONS(1481),
    [anon_sym_continue] = ACTIONS(1481),
    [anon_sym_default] = ACTIONS(1481),
    [anon_sym_enum] = ACTIONS(1481),
    [anon_sym_fn] = ACTIONS(1481),
    [anon_sym_for] = ACTIONS(1481),
    [anon_sym_gen] = ACTIONS(1481),
    [anon_sym_if] = ACTIONS(1481),
    [anon_sym_impl] = ACTIONS(1481),
    [anon_sym_let] = ACTIONS(1481),
    [anon_sym_loop] = ACTIONS(1481),
    [anon_sym_match] = ACTIONS(1481),
    [anon_sym_mod] = ACTIONS(1481),
    [anon_sym_pub] = ACTIONS(1481),
    [anon_sym_return] = ACTIONS(1481),
    [anon_sym_static] = ACTIONS(1481),
    [anon_sym_struct] = ACTIONS(1481),
    [anon_sym_trait] = ACTIONS(1481),
    [anon_sym_type] = ACTIONS(1481),
    [anon_sym_union] = ACTIONS(1481),
    [anon_sym_unsafe] = ACTIONS(1481),
    [anon_sym_use] = ACTIONS(1481),
    [anon_sym_while] = ACTIONS(1481),
    [anon_sym_extern] = ACTIONS(1481),
    [anon_sym_yield] = ACTIONS(1481),
    [anon_sym_move] = ACTIONS(1481),
    [anon_sym_try] = ACTIONS(1481),
    [sym_integer_literal] = ACTIONS(1479),
    [aux_sym_string_literal_token1] = ACTIONS(1479),
    [sym_char_literal] = ACTIONS(1479),
    [anon_sym_true] = ACTIONS(1481),
    [anon_sym_false] = ACTIONS(1481),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1481),
    [sym_super] = ACTIONS(1481),
    [sym_crate] = ACTIONS(1481),
    [sym_metavariable] = ACTIONS(1479),
    [sym__raw_string_literal_start] = ACTIONS(1479),
    [sym_float_literal] = ACTIONS(1479),
  },
  [404] = {
    [sym_line_comment] = STATE(404),
    [sym_block_comment] = STATE(404),
    [ts_builtin_sym_end] = ACTIONS(1483),
    [sym_identifier] = ACTIONS(1485),
    [anon_sym_SEMI] = ACTIONS(1483),
    [anon_sym_macro_rules_BANG] = ACTIONS(1483),
    [anon_sym_LPAREN] = ACTIONS(1483),
    [anon_sym_LBRACK] = ACTIONS(1483),
    [anon_sym_LBRACE] = ACTIONS(1483),
    [anon_sym_RBRACE] = ACTIONS(1483),
    [anon_sym_PLUS] = ACTIONS(1485),
    [anon_sym_STAR] = ACTIONS(1485),
    [anon_sym_QMARK] = ACTIONS(1483),
    [anon_sym_u8] = ACTIONS(1485),
    [anon_sym_i8] = ACTIONS(1485),
    [anon_sym_u16] = ACTIONS(1485),
    [anon_sym_i16] = ACTIONS(1485),
    [anon_sym_u32] = ACTIONS(1485),
    [anon_sym_i32] = ACTIONS(1485),
    [anon_sym_u64] = ACTIONS(1485),
    [anon_sym_i64] = ACTIONS(1485),
    [anon_sym_u128] = ACTIONS(1485),
    [anon_sym_i128] = ACTIONS(1485),
    [anon_sym_isize] = ACTIONS(1485),
    [anon_sym_usize] = ACTIONS(1485),
    [anon_sym_f32] = ACTIONS(1485),
    [anon_sym_f64] = ACTIONS(1485),
    [anon_sym_bool] = ACTIONS(1485),
    [anon_sym_str] = ACTIONS(1485),
    [anon_sym_char] = ACTIONS(1485),
    [anon_sym_DASH] = ACTIONS(1485),
    [anon_sym_SLASH] = ACTIONS(1485),
    [anon_sym_PERCENT] = ACTIONS(1485),
    [anon_sym_CARET] = ACTIONS(1485),
    [anon_sym_BANG] = ACTIONS(1485),
    [anon_sym_AMP] = ACTIONS(1485),
    [anon_sym_PIPE] = ACTIONS(1485),
    [anon_sym_AMP_AMP] = ACTIONS(1483),
    [anon_sym_PIPE_PIPE] = ACTIONS(1483),
    [anon_sym_LT_LT] = ACTIONS(1485),
    [anon_sym_GT_GT] = ACTIONS(1485),
    [anon_sym_PLUS_EQ] = ACTIONS(1483),
    [anon_sym_DASH_EQ] = ACTIONS(1483),
    [anon_sym_STAR_EQ] = ACTIONS(1483),
    [anon_sym_SLASH_EQ] = ACTIONS(1483),
    [anon_sym_PERCENT_EQ] = ACTIONS(1483),
    [anon_sym_CARET_EQ] = ACTIONS(1483),
    [anon_sym_AMP_EQ] = ACTIONS(1483),
    [anon_sym_PIPE_EQ] = ACTIONS(1483),
    [anon_sym_LT_LT_EQ] = ACTIONS(1483),
    [anon_sym_GT_GT_EQ] = ACTIONS(1483),
    [anon_sym_EQ] = ACTIONS(1485),
    [anon_sym_EQ_EQ] = ACTIONS(1483),
    [anon_sym_BANG_EQ] = ACTIONS(1483),
    [anon_sym_GT] = ACTIONS(1485),
    [anon_sym_LT] = ACTIONS(1485),
    [anon_sym_GT_EQ] = ACTIONS(1483),
    [anon_sym_LT_EQ] = ACTIONS(1483),
    [anon_sym_DOT] = ACTIONS(1485),
    [anon_sym_DOT_DOT] = ACTIONS(1485),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1483),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1483),
    [anon_sym_COLON_COLON] = ACTIONS(1483),
    [anon_sym_POUND] = ACTIONS(1483),
    [anon_sym_SQUOTE] = ACTIONS(1485),
    [anon_sym_as] = ACTIONS(1485),
    [anon_sym_async] = ACTIONS(1485),
    [anon_sym_break] = ACTIONS(1485),
    [anon_sym_const] = ACTIONS(1485),
    [anon_sym_continue] = ACTIONS(1485),
    [anon_sym_default] = ACTIONS(1485),
    [anon_sym_enum] = ACTIONS(1485),
    [anon_sym_fn] = ACTIONS(1485),
    [anon_sym_for] = ACTIONS(1485),
    [anon_sym_gen] = ACTIONS(1485),
    [anon_sym_if] = ACTIONS(1485),
    [anon_sym_impl] = ACTIONS(1485),
    [anon_sym_let] = ACTIONS(1485),
    [anon_sym_loop] = ACTIONS(1485),
    [anon_sym_match] = ACTIONS(1485),
    [anon_sym_mod] = ACTIONS(1485),
    [anon_sym_pub] = ACTIONS(1485),
    [anon_sym_return] = ACTIONS(1485),
    [anon_sym_static] = ACTIONS(1485),
    [anon_sym_struct] = ACTIONS(1485),
    [anon_sym_trait] = ACTIONS(1485),
    [anon_sym_type] = ACTIONS(1485),
    [anon_sym_union] = ACTIONS(1485),
    [anon_sym_unsafe] = ACTIONS(1485),
    [anon_sym_use] = ACTIONS(1485),
    [anon_sym_while] = ACTIONS(1485),
    [anon_sym_extern] = ACTIONS(1485),
    [anon_sym_yield] = ACTIONS(1485),
    [anon_sym_move] = ACTIONS(1485),
    [anon_sym_try] = ACTIONS(1485),
    [sym_integer_literal] = ACTIONS(1483),
    [aux_sym_string_literal_token1] = ACTIONS(1483),
    [sym_char_literal] = ACTIONS(1483),
    [anon_sym_true] = ACTIONS(1485),
    [anon_sym_false] = ACTIONS(1485),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1485),
    [sym_super] = ACTIONS(1485),
    [sym_crate] = ACTIONS(1485),
    [sym_metavariable] = ACTIONS(1483),
    [sym__raw_string_literal_start] = ACTIONS(1483),
    [sym_float_literal] = ACTIONS(1483),
  },
  [405] = {
    [sym_line_comment] = STATE(405),
    [sym_block_comment] = STATE(405),
    [ts_builtin_sym_end] = ACTIONS(1487),
    [sym_identifier] = ACTIONS(1489),
    [anon_sym_SEMI] = ACTIONS(1487),
    [anon_sym_macro_rules_BANG] = ACTIONS(1487),
    [anon_sym_LPAREN] = ACTIONS(1487),
    [anon_sym_LBRACK] = ACTIONS(1487),
    [anon_sym_LBRACE] = ACTIONS(1487),
    [anon_sym_RBRACE] = ACTIONS(1487),
    [anon_sym_PLUS] = ACTIONS(1489),
    [anon_sym_STAR] = ACTIONS(1489),
    [anon_sym_QMARK] = ACTIONS(1487),
    [anon_sym_u8] = ACTIONS(1489),
    [anon_sym_i8] = ACTIONS(1489),
    [anon_sym_u16] = ACTIONS(1489),
    [anon_sym_i16] = ACTIONS(1489),
    [anon_sym_u32] = ACTIONS(1489),
    [anon_sym_i32] = ACTIONS(1489),
    [anon_sym_u64] = ACTIONS(1489),
    [anon_sym_i64] = ACTIONS(1489),
    [anon_sym_u128] = ACTIONS(1489),
    [anon_sym_i128] = ACTIONS(1489),
    [anon_sym_isize] = ACTIONS(1489),
    [anon_sym_usize] = ACTIONS(1489),
    [anon_sym_f32] = ACTIONS(1489),
    [anon_sym_f64] = ACTIONS(1489),
    [anon_sym_bool] = ACTIONS(1489),
    [anon_sym_str] = ACTIONS(1489),
    [anon_sym_char] = ACTIONS(1489),
    [anon_sym_DASH] = ACTIONS(1489),
    [anon_sym_SLASH] = ACTIONS(1489),
    [anon_sym_PERCENT] = ACTIONS(1489),
    [anon_sym_CARET] = ACTIONS(1489),
    [anon_sym_BANG] = ACTIONS(1489),
    [anon_sym_AMP] = ACTIONS(1489),
    [anon_sym_PIPE] = ACTIONS(1489),
    [anon_sym_AMP_AMP] = ACTIONS(1487),
    [anon_sym_PIPE_PIPE] = ACTIONS(1487),
    [anon_sym_LT_LT] = ACTIONS(1489),
    [anon_sym_GT_GT] = ACTIONS(1489),
    [anon_sym_PLUS_EQ] = ACTIONS(1487),
    [anon_sym_DASH_EQ] = ACTIONS(1487),
    [anon_sym_STAR_EQ] = ACTIONS(1487),
    [anon_sym_SLASH_EQ] = ACTIONS(1487),
    [anon_sym_PERCENT_EQ] = ACTIONS(1487),
    [anon_sym_CARET_EQ] = ACTIONS(1487),
    [anon_sym_AMP_EQ] = ACTIONS(1487),
    [anon_sym_PIPE_EQ] = ACTIONS(1487),
    [anon_sym_LT_LT_EQ] = ACTIONS(1487),
    [anon_sym_GT_GT_EQ] = ACTIONS(1487),
    [anon_sym_EQ] = ACTIONS(1489),
    [anon_sym_EQ_EQ] = ACTIONS(1487),
    [anon_sym_BANG_EQ] = ACTIONS(1487),
    [anon_sym_GT] = ACTIONS(1489),
    [anon_sym_LT] = ACTIONS(1489),
    [anon_sym_GT_EQ] = ACTIONS(1487),
    [anon_sym_LT_EQ] = ACTIONS(1487),
    [anon_sym_DOT] = ACTIONS(1489),
    [anon_sym_DOT_DOT] = ACTIONS(1489),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1487),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1487),
    [anon_sym_COLON_COLON] = ACTIONS(1487),
    [anon_sym_POUND] = ACTIONS(1487),
    [anon_sym_SQUOTE] = ACTIONS(1489),
    [anon_sym_as] = ACTIONS(1489),
    [anon_sym_async] = ACTIONS(1489),
    [anon_sym_break] = ACTIONS(1489),
    [anon_sym_const] = ACTIONS(1489),
    [anon_sym_continue] = ACTIONS(1489),
    [anon_sym_default] = ACTIONS(1489),
    [anon_sym_enum] = ACTIONS(1489),
    [anon_sym_fn] = ACTIONS(1489),
    [anon_sym_for] = ACTIONS(1489),
    [anon_sym_gen] = ACTIONS(1489),
    [anon_sym_if] = ACTIONS(1489),
    [anon_sym_impl] = ACTIONS(1489),
    [anon_sym_let] = ACTIONS(1489),
    [anon_sym_loop] = ACTIONS(1489),
    [anon_sym_match] = ACTIONS(1489),
    [anon_sym_mod] = ACTIONS(1489),
    [anon_sym_pub] = ACTIONS(1489),
    [anon_sym_return] = ACTIONS(1489),
    [anon_sym_static] = ACTIONS(1489),
    [anon_sym_struct] = ACTIONS(1489),
    [anon_sym_trait] = ACTIONS(1489),
    [anon_sym_type] = ACTIONS(1489),
    [anon_sym_union] = ACTIONS(1489),
    [anon_sym_unsafe] = ACTIONS(1489),
    [anon_sym_use] = ACTIONS(1489),
    [anon_sym_while] = ACTIONS(1489),
    [anon_sym_extern] = ACTIONS(1489),
    [anon_sym_yield] = ACTIONS(1489),
    [anon_sym_move] = ACTIONS(1489),
    [anon_sym_try] = ACTIONS(1489),
    [sym_integer_literal] = ACTIONS(1487),
    [aux_sym_string_literal_token1] = ACTIONS(1487),
    [sym_char_literal] = ACTIONS(1487),
    [anon_sym_true] = ACTIONS(1489),
    [anon_sym_false] = ACTIONS(1489),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1489),
    [sym_super] = ACTIONS(1489),
    [sym_crate] = ACTIONS(1489),
    [sym_metavariable] = ACTIONS(1487),
    [sym__raw_string_literal_start] = ACTIONS(1487),
    [sym_float_literal] = ACTIONS(1487),
  },
  [406] = {
    [sym_attribute_item] = STATE(423),
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_self_parameter] = STATE(3137),
    [sym_variadic_parameter] = STATE(3137),
    [sym_parameter] = STATE(3137),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2807),
    [sym_bracketed_type] = STATE(3550),
    [sym_lifetime] = STATE(2800),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3221),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(2410),
    [sym_scoped_identifier] = STATE(2242),
    [sym_scoped_type_identifier] = STATE(2118),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(3256),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(406),
    [sym_block_comment] = STATE(406),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1281),
    [anon_sym_LPAREN] = ACTIONS(1283),
    [anon_sym_RPAREN] = ACTIONS(1491),
    [anon_sym_LBRACK] = ACTIONS(1287),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1293),
    [anon_sym_i8] = ACTIONS(1293),
    [anon_sym_u16] = ACTIONS(1293),
    [anon_sym_i16] = ACTIONS(1293),
    [anon_sym_u32] = ACTIONS(1293),
    [anon_sym_i32] = ACTIONS(1293),
    [anon_sym_u64] = ACTIONS(1293),
    [anon_sym_i64] = ACTIONS(1293),
    [anon_sym_u128] = ACTIONS(1293),
    [anon_sym_i128] = ACTIONS(1293),
    [anon_sym_isize] = ACTIONS(1293),
    [anon_sym_usize] = ACTIONS(1293),
    [anon_sym_f32] = ACTIONS(1293),
    [anon_sym_f64] = ACTIONS(1293),
    [anon_sym_bool] = ACTIONS(1293),
    [anon_sym_str] = ACTIONS(1293),
    [anon_sym_char] = ACTIONS(1293),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1299),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1493),
    [anon_sym_DOT_DOT] = ACTIONS(1305),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1307),
    [anon_sym_COLON_COLON] = ACTIONS(1311),
    [anon_sym_POUND] = ACTIONS(1313),
    [anon_sym_SQUOTE] = ACTIONS(1315),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1319),
    [anon_sym_default] = ACTIONS(1321),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1327),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1327),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_ref] = ACTIONS(1333),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_mutable_specifier] = ACTIONS(1337),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1345),
    [sym_super] = ACTIONS(1347),
    [sym_crate] = ACTIONS(1347),
    [sym_metavariable] = ACTIONS(1349),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [407] = {
    [sym_attribute_item] = STATE(423),
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_self_parameter] = STATE(3137),
    [sym_variadic_parameter] = STATE(3137),
    [sym_parameter] = STATE(3137),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2807),
    [sym_bracketed_type] = STATE(3550),
    [sym_lifetime] = STATE(2800),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3221),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(2410),
    [sym_scoped_identifier] = STATE(2242),
    [sym_scoped_type_identifier] = STATE(2118),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(3256),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(407),
    [sym_block_comment] = STATE(407),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1281),
    [anon_sym_LPAREN] = ACTIONS(1283),
    [anon_sym_RPAREN] = ACTIONS(1495),
    [anon_sym_LBRACK] = ACTIONS(1287),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1293),
    [anon_sym_i8] = ACTIONS(1293),
    [anon_sym_u16] = ACTIONS(1293),
    [anon_sym_i16] = ACTIONS(1293),
    [anon_sym_u32] = ACTIONS(1293),
    [anon_sym_i32] = ACTIONS(1293),
    [anon_sym_u64] = ACTIONS(1293),
    [anon_sym_i64] = ACTIONS(1293),
    [anon_sym_u128] = ACTIONS(1293),
    [anon_sym_i128] = ACTIONS(1293),
    [anon_sym_isize] = ACTIONS(1293),
    [anon_sym_usize] = ACTIONS(1293),
    [anon_sym_f32] = ACTIONS(1293),
    [anon_sym_f64] = ACTIONS(1293),
    [anon_sym_bool] = ACTIONS(1293),
    [anon_sym_str] = ACTIONS(1293),
    [anon_sym_char] = ACTIONS(1293),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1299),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1493),
    [anon_sym_DOT_DOT] = ACTIONS(1305),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1307),
    [anon_sym_COLON_COLON] = ACTIONS(1311),
    [anon_sym_POUND] = ACTIONS(1313),
    [anon_sym_SQUOTE] = ACTIONS(1315),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1319),
    [anon_sym_default] = ACTIONS(1321),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1327),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1327),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_ref] = ACTIONS(1333),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_mutable_specifier] = ACTIONS(1337),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1345),
    [sym_super] = ACTIONS(1347),
    [sym_crate] = ACTIONS(1347),
    [sym_metavariable] = ACTIONS(1349),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [408] = {
    [sym_attribute_item] = STATE(423),
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_self_parameter] = STATE(3137),
    [sym_variadic_parameter] = STATE(3137),
    [sym_parameter] = STATE(3137),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2807),
    [sym_bracketed_type] = STATE(3550),
    [sym_lifetime] = STATE(2800),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3221),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(2410),
    [sym_scoped_identifier] = STATE(2242),
    [sym_scoped_type_identifier] = STATE(2118),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(3256),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(408),
    [sym_block_comment] = STATE(408),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1281),
    [anon_sym_LPAREN] = ACTIONS(1283),
    [anon_sym_RPAREN] = ACTIONS(1497),
    [anon_sym_LBRACK] = ACTIONS(1287),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1293),
    [anon_sym_i8] = ACTIONS(1293),
    [anon_sym_u16] = ACTIONS(1293),
    [anon_sym_i16] = ACTIONS(1293),
    [anon_sym_u32] = ACTIONS(1293),
    [anon_sym_i32] = ACTIONS(1293),
    [anon_sym_u64] = ACTIONS(1293),
    [anon_sym_i64] = ACTIONS(1293),
    [anon_sym_u128] = ACTIONS(1293),
    [anon_sym_i128] = ACTIONS(1293),
    [anon_sym_isize] = ACTIONS(1293),
    [anon_sym_usize] = ACTIONS(1293),
    [anon_sym_f32] = ACTIONS(1293),
    [anon_sym_f64] = ACTIONS(1293),
    [anon_sym_bool] = ACTIONS(1293),
    [anon_sym_str] = ACTIONS(1293),
    [anon_sym_char] = ACTIONS(1293),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1299),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1493),
    [anon_sym_DOT_DOT] = ACTIONS(1305),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1307),
    [anon_sym_COLON_COLON] = ACTIONS(1311),
    [anon_sym_POUND] = ACTIONS(1313),
    [anon_sym_SQUOTE] = ACTIONS(1315),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1319),
    [anon_sym_default] = ACTIONS(1321),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1327),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1327),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_ref] = ACTIONS(1333),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_mutable_specifier] = ACTIONS(1337),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1345),
    [sym_super] = ACTIONS(1347),
    [sym_crate] = ACTIONS(1347),
    [sym_metavariable] = ACTIONS(1349),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [409] = {
    [sym_line_comment] = STATE(409),
    [sym_block_comment] = STATE(409),
    [sym_identifier] = ACTIONS(1465),
    [anon_sym_SEMI] = ACTIONS(1359),
    [anon_sym_macro_rules_BANG] = ACTIONS(1463),
    [anon_sym_LPAREN] = ACTIONS(1359),
    [anon_sym_LBRACK] = ACTIONS(1359),
    [anon_sym_LBRACE] = ACTIONS(1463),
    [anon_sym_RBRACE] = ACTIONS(1463),
    [anon_sym_PLUS] = ACTIONS(1357),
    [anon_sym_STAR] = ACTIONS(1357),
    [anon_sym_QMARK] = ACTIONS(1359),
    [anon_sym_u8] = ACTIONS(1465),
    [anon_sym_i8] = ACTIONS(1465),
    [anon_sym_u16] = ACTIONS(1465),
    [anon_sym_i16] = ACTIONS(1465),
    [anon_sym_u32] = ACTIONS(1465),
    [anon_sym_i32] = ACTIONS(1465),
    [anon_sym_u64] = ACTIONS(1465),
    [anon_sym_i64] = ACTIONS(1465),
    [anon_sym_u128] = ACTIONS(1465),
    [anon_sym_i128] = ACTIONS(1465),
    [anon_sym_isize] = ACTIONS(1465),
    [anon_sym_usize] = ACTIONS(1465),
    [anon_sym_f32] = ACTIONS(1465),
    [anon_sym_f64] = ACTIONS(1465),
    [anon_sym_bool] = ACTIONS(1465),
    [anon_sym_str] = ACTIONS(1465),
    [anon_sym_char] = ACTIONS(1465),
    [anon_sym_DASH] = ACTIONS(1357),
    [anon_sym_SLASH] = ACTIONS(1357),
    [anon_sym_PERCENT] = ACTIONS(1357),
    [anon_sym_CARET] = ACTIONS(1357),
    [anon_sym_BANG] = ACTIONS(1465),
    [anon_sym_AMP] = ACTIONS(1357),
    [anon_sym_PIPE] = ACTIONS(1357),
    [anon_sym_AMP_AMP] = ACTIONS(1359),
    [anon_sym_PIPE_PIPE] = ACTIONS(1359),
    [anon_sym_LT_LT] = ACTIONS(1357),
    [anon_sym_GT_GT] = ACTIONS(1357),
    [anon_sym_PLUS_EQ] = ACTIONS(1359),
    [anon_sym_DASH_EQ] = ACTIONS(1359),
    [anon_sym_STAR_EQ] = ACTIONS(1359),
    [anon_sym_SLASH_EQ] = ACTIONS(1359),
    [anon_sym_PERCENT_EQ] = ACTIONS(1359),
    [anon_sym_CARET_EQ] = ACTIONS(1359),
    [anon_sym_AMP_EQ] = ACTIONS(1359),
    [anon_sym_PIPE_EQ] = ACTIONS(1359),
    [anon_sym_LT_LT_EQ] = ACTIONS(1359),
    [anon_sym_GT_GT_EQ] = ACTIONS(1359),
    [anon_sym_EQ] = ACTIONS(1357),
    [anon_sym_EQ_EQ] = ACTIONS(1359),
    [anon_sym_BANG_EQ] = ACTIONS(1359),
    [anon_sym_GT] = ACTIONS(1357),
    [anon_sym_LT] = ACTIONS(1357),
    [anon_sym_GT_EQ] = ACTIONS(1359),
    [anon_sym_LT_EQ] = ACTIONS(1359),
    [anon_sym_DOT] = ACTIONS(1357),
    [anon_sym_DOT_DOT] = ACTIONS(1357),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1359),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1359),
    [anon_sym_COLON_COLON] = ACTIONS(1463),
    [anon_sym_POUND] = ACTIONS(1463),
    [anon_sym_SQUOTE] = ACTIONS(1465),
    [anon_sym_as] = ACTIONS(1357),
    [anon_sym_async] = ACTIONS(1465),
    [anon_sym_break] = ACTIONS(1465),
    [anon_sym_const] = ACTIONS(1465),
    [anon_sym_continue] = ACTIONS(1465),
    [anon_sym_default] = ACTIONS(1465),
    [anon_sym_enum] = ACTIONS(1465),
    [anon_sym_fn] = ACTIONS(1465),
    [anon_sym_for] = ACTIONS(1465),
    [anon_sym_gen] = ACTIONS(1465),
    [anon_sym_if] = ACTIONS(1465),
    [anon_sym_impl] = ACTIONS(1465),
    [anon_sym_let] = ACTIONS(1465),
    [anon_sym_loop] = ACTIONS(1465),
    [anon_sym_match] = ACTIONS(1465),
    [anon_sym_mod] = ACTIONS(1465),
    [anon_sym_pub] = ACTIONS(1465),
    [anon_sym_return] = ACTIONS(1465),
    [anon_sym_static] = ACTIONS(1465),
    [anon_sym_struct] = ACTIONS(1465),
    [anon_sym_trait] = ACTIONS(1465),
    [anon_sym_type] = ACTIONS(1465),
    [anon_sym_union] = ACTIONS(1465),
    [anon_sym_unsafe] = ACTIONS(1465),
    [anon_sym_use] = ACTIONS(1465),
    [anon_sym_while] = ACTIONS(1465),
    [anon_sym_extern] = ACTIONS(1465),
    [anon_sym_yield] = ACTIONS(1465),
    [anon_sym_move] = ACTIONS(1465),
    [anon_sym_try] = ACTIONS(1465),
    [sym_integer_literal] = ACTIONS(1463),
    [aux_sym_string_literal_token1] = ACTIONS(1463),
    [sym_char_literal] = ACTIONS(1463),
    [anon_sym_true] = ACTIONS(1465),
    [anon_sym_false] = ACTIONS(1465),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1465),
    [sym_super] = ACTIONS(1465),
    [sym_crate] = ACTIONS(1465),
    [sym_metavariable] = ACTIONS(1463),
    [sym__raw_string_literal_start] = ACTIONS(1463),
    [sym_float_literal] = ACTIONS(1463),
  },
  [410] = {
    [sym_attribute_item] = STATE(423),
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_self_parameter] = STATE(3137),
    [sym_variadic_parameter] = STATE(3137),
    [sym_parameter] = STATE(3137),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2807),
    [sym_bracketed_type] = STATE(3550),
    [sym_lifetime] = STATE(2800),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3221),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(2410),
    [sym_scoped_identifier] = STATE(2242),
    [sym_scoped_type_identifier] = STATE(2118),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(3256),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(410),
    [sym_block_comment] = STATE(410),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1281),
    [anon_sym_LPAREN] = ACTIONS(1283),
    [anon_sym_RPAREN] = ACTIONS(1499),
    [anon_sym_LBRACK] = ACTIONS(1287),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1293),
    [anon_sym_i8] = ACTIONS(1293),
    [anon_sym_u16] = ACTIONS(1293),
    [anon_sym_i16] = ACTIONS(1293),
    [anon_sym_u32] = ACTIONS(1293),
    [anon_sym_i32] = ACTIONS(1293),
    [anon_sym_u64] = ACTIONS(1293),
    [anon_sym_i64] = ACTIONS(1293),
    [anon_sym_u128] = ACTIONS(1293),
    [anon_sym_i128] = ACTIONS(1293),
    [anon_sym_isize] = ACTIONS(1293),
    [anon_sym_usize] = ACTIONS(1293),
    [anon_sym_f32] = ACTIONS(1293),
    [anon_sym_f64] = ACTIONS(1293),
    [anon_sym_bool] = ACTIONS(1293),
    [anon_sym_str] = ACTIONS(1293),
    [anon_sym_char] = ACTIONS(1293),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1299),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1493),
    [anon_sym_DOT_DOT] = ACTIONS(1305),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1307),
    [anon_sym_COLON_COLON] = ACTIONS(1311),
    [anon_sym_POUND] = ACTIONS(1313),
    [anon_sym_SQUOTE] = ACTIONS(1315),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1319),
    [anon_sym_default] = ACTIONS(1321),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1327),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1327),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_ref] = ACTIONS(1333),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_mutable_specifier] = ACTIONS(1337),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1345),
    [sym_super] = ACTIONS(1347),
    [sym_crate] = ACTIONS(1347),
    [sym_metavariable] = ACTIONS(1349),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [411] = {
    [sym_attribute_item] = STATE(423),
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_self_parameter] = STATE(3137),
    [sym_variadic_parameter] = STATE(3137),
    [sym_parameter] = STATE(3137),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2807),
    [sym_bracketed_type] = STATE(3550),
    [sym_lifetime] = STATE(2800),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3221),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(2410),
    [sym_scoped_identifier] = STATE(2242),
    [sym_scoped_type_identifier] = STATE(2118),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(3256),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(411),
    [sym_block_comment] = STATE(411),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1281),
    [anon_sym_LPAREN] = ACTIONS(1283),
    [anon_sym_RPAREN] = ACTIONS(1501),
    [anon_sym_LBRACK] = ACTIONS(1287),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1293),
    [anon_sym_i8] = ACTIONS(1293),
    [anon_sym_u16] = ACTIONS(1293),
    [anon_sym_i16] = ACTIONS(1293),
    [anon_sym_u32] = ACTIONS(1293),
    [anon_sym_i32] = ACTIONS(1293),
    [anon_sym_u64] = ACTIONS(1293),
    [anon_sym_i64] = ACTIONS(1293),
    [anon_sym_u128] = ACTIONS(1293),
    [anon_sym_i128] = ACTIONS(1293),
    [anon_sym_isize] = ACTIONS(1293),
    [anon_sym_usize] = ACTIONS(1293),
    [anon_sym_f32] = ACTIONS(1293),
    [anon_sym_f64] = ACTIONS(1293),
    [anon_sym_bool] = ACTIONS(1293),
    [anon_sym_str] = ACTIONS(1293),
    [anon_sym_char] = ACTIONS(1293),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1299),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1493),
    [anon_sym_DOT_DOT] = ACTIONS(1305),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1307),
    [anon_sym_COLON_COLON] = ACTIONS(1311),
    [anon_sym_POUND] = ACTIONS(1313),
    [anon_sym_SQUOTE] = ACTIONS(1315),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1319),
    [anon_sym_default] = ACTIONS(1321),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1327),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1327),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_ref] = ACTIONS(1333),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_mutable_specifier] = ACTIONS(1337),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1345),
    [sym_super] = ACTIONS(1347),
    [sym_crate] = ACTIONS(1347),
    [sym_metavariable] = ACTIONS(1349),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [412] = {
    [sym_attribute_item] = STATE(423),
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_self_parameter] = STATE(3137),
    [sym_variadic_parameter] = STATE(3137),
    [sym_parameter] = STATE(3137),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2807),
    [sym_bracketed_type] = STATE(3550),
    [sym_lifetime] = STATE(2800),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3221),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(2410),
    [sym_scoped_identifier] = STATE(2242),
    [sym_scoped_type_identifier] = STATE(2118),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(3256),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(412),
    [sym_block_comment] = STATE(412),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1281),
    [anon_sym_LPAREN] = ACTIONS(1283),
    [anon_sym_RPAREN] = ACTIONS(1503),
    [anon_sym_LBRACK] = ACTIONS(1287),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1293),
    [anon_sym_i8] = ACTIONS(1293),
    [anon_sym_u16] = ACTIONS(1293),
    [anon_sym_i16] = ACTIONS(1293),
    [anon_sym_u32] = ACTIONS(1293),
    [anon_sym_i32] = ACTIONS(1293),
    [anon_sym_u64] = ACTIONS(1293),
    [anon_sym_i64] = ACTIONS(1293),
    [anon_sym_u128] = ACTIONS(1293),
    [anon_sym_i128] = ACTIONS(1293),
    [anon_sym_isize] = ACTIONS(1293),
    [anon_sym_usize] = ACTIONS(1293),
    [anon_sym_f32] = ACTIONS(1293),
    [anon_sym_f64] = ACTIONS(1293),
    [anon_sym_bool] = ACTIONS(1293),
    [anon_sym_str] = ACTIONS(1293),
    [anon_sym_char] = ACTIONS(1293),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1299),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1493),
    [anon_sym_DOT_DOT] = ACTIONS(1305),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1307),
    [anon_sym_COLON_COLON] = ACTIONS(1311),
    [anon_sym_POUND] = ACTIONS(1313),
    [anon_sym_SQUOTE] = ACTIONS(1315),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1319),
    [anon_sym_default] = ACTIONS(1321),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1327),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1327),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_ref] = ACTIONS(1333),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_mutable_specifier] = ACTIONS(1337),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1345),
    [sym_super] = ACTIONS(1347),
    [sym_crate] = ACTIONS(1347),
    [sym_metavariable] = ACTIONS(1349),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [413] = {
    [sym_attribute_item] = STATE(423),
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_self_parameter] = STATE(3137),
    [sym_variadic_parameter] = STATE(3137),
    [sym_parameter] = STATE(3137),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2807),
    [sym_bracketed_type] = STATE(3550),
    [sym_lifetime] = STATE(2800),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3221),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(2410),
    [sym_scoped_identifier] = STATE(2242),
    [sym_scoped_type_identifier] = STATE(2118),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(3256),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(413),
    [sym_block_comment] = STATE(413),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1281),
    [anon_sym_LPAREN] = ACTIONS(1283),
    [anon_sym_RPAREN] = ACTIONS(1505),
    [anon_sym_LBRACK] = ACTIONS(1287),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1293),
    [anon_sym_i8] = ACTIONS(1293),
    [anon_sym_u16] = ACTIONS(1293),
    [anon_sym_i16] = ACTIONS(1293),
    [anon_sym_u32] = ACTIONS(1293),
    [anon_sym_i32] = ACTIONS(1293),
    [anon_sym_u64] = ACTIONS(1293),
    [anon_sym_i64] = ACTIONS(1293),
    [anon_sym_u128] = ACTIONS(1293),
    [anon_sym_i128] = ACTIONS(1293),
    [anon_sym_isize] = ACTIONS(1293),
    [anon_sym_usize] = ACTIONS(1293),
    [anon_sym_f32] = ACTIONS(1293),
    [anon_sym_f64] = ACTIONS(1293),
    [anon_sym_bool] = ACTIONS(1293),
    [anon_sym_str] = ACTIONS(1293),
    [anon_sym_char] = ACTIONS(1293),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1299),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1493),
    [anon_sym_DOT_DOT] = ACTIONS(1305),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1307),
    [anon_sym_COLON_COLON] = ACTIONS(1311),
    [anon_sym_POUND] = ACTIONS(1313),
    [anon_sym_SQUOTE] = ACTIONS(1315),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1319),
    [anon_sym_default] = ACTIONS(1321),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1327),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1327),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_ref] = ACTIONS(1333),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_mutable_specifier] = ACTIONS(1337),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1345),
    [sym_super] = ACTIONS(1347),
    [sym_crate] = ACTIONS(1347),
    [sym_metavariable] = ACTIONS(1349),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [414] = {
    [sym_attribute_item] = STATE(423),
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_self_parameter] = STATE(3137),
    [sym_variadic_parameter] = STATE(3137),
    [sym_parameter] = STATE(3137),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2807),
    [sym_bracketed_type] = STATE(3550),
    [sym_lifetime] = STATE(2800),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3221),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(2410),
    [sym_scoped_identifier] = STATE(2242),
    [sym_scoped_type_identifier] = STATE(2118),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(3256),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(414),
    [sym_block_comment] = STATE(414),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1281),
    [anon_sym_LPAREN] = ACTIONS(1283),
    [anon_sym_RPAREN] = ACTIONS(1507),
    [anon_sym_LBRACK] = ACTIONS(1287),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1293),
    [anon_sym_i8] = ACTIONS(1293),
    [anon_sym_u16] = ACTIONS(1293),
    [anon_sym_i16] = ACTIONS(1293),
    [anon_sym_u32] = ACTIONS(1293),
    [anon_sym_i32] = ACTIONS(1293),
    [anon_sym_u64] = ACTIONS(1293),
    [anon_sym_i64] = ACTIONS(1293),
    [anon_sym_u128] = ACTIONS(1293),
    [anon_sym_i128] = ACTIONS(1293),
    [anon_sym_isize] = ACTIONS(1293),
    [anon_sym_usize] = ACTIONS(1293),
    [anon_sym_f32] = ACTIONS(1293),
    [anon_sym_f64] = ACTIONS(1293),
    [anon_sym_bool] = ACTIONS(1293),
    [anon_sym_str] = ACTIONS(1293),
    [anon_sym_char] = ACTIONS(1293),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1299),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1493),
    [anon_sym_DOT_DOT] = ACTIONS(1305),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1307),
    [anon_sym_COLON_COLON] = ACTIONS(1311),
    [anon_sym_POUND] = ACTIONS(1313),
    [anon_sym_SQUOTE] = ACTIONS(1315),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1319),
    [anon_sym_default] = ACTIONS(1321),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1327),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1327),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_ref] = ACTIONS(1333),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_mutable_specifier] = ACTIONS(1337),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1345),
    [sym_super] = ACTIONS(1347),
    [sym_crate] = ACTIONS(1347),
    [sym_metavariable] = ACTIONS(1349),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [415] = {
    [sym_attribute_item] = STATE(423),
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_self_parameter] = STATE(3137),
    [sym_variadic_parameter] = STATE(3137),
    [sym_parameter] = STATE(3137),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2807),
    [sym_bracketed_type] = STATE(3550),
    [sym_lifetime] = STATE(2800),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3221),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(2410),
    [sym_scoped_identifier] = STATE(2242),
    [sym_scoped_type_identifier] = STATE(2118),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(3256),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(415),
    [sym_block_comment] = STATE(415),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1281),
    [anon_sym_LPAREN] = ACTIONS(1283),
    [anon_sym_RPAREN] = ACTIONS(1509),
    [anon_sym_LBRACK] = ACTIONS(1287),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1293),
    [anon_sym_i8] = ACTIONS(1293),
    [anon_sym_u16] = ACTIONS(1293),
    [anon_sym_i16] = ACTIONS(1293),
    [anon_sym_u32] = ACTIONS(1293),
    [anon_sym_i32] = ACTIONS(1293),
    [anon_sym_u64] = ACTIONS(1293),
    [anon_sym_i64] = ACTIONS(1293),
    [anon_sym_u128] = ACTIONS(1293),
    [anon_sym_i128] = ACTIONS(1293),
    [anon_sym_isize] = ACTIONS(1293),
    [anon_sym_usize] = ACTIONS(1293),
    [anon_sym_f32] = ACTIONS(1293),
    [anon_sym_f64] = ACTIONS(1293),
    [anon_sym_bool] = ACTIONS(1293),
    [anon_sym_str] = ACTIONS(1293),
    [anon_sym_char] = ACTIONS(1293),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1299),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1493),
    [anon_sym_DOT_DOT] = ACTIONS(1305),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1307),
    [anon_sym_COLON_COLON] = ACTIONS(1311),
    [anon_sym_POUND] = ACTIONS(1313),
    [anon_sym_SQUOTE] = ACTIONS(1315),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1319),
    [anon_sym_default] = ACTIONS(1321),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1327),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1327),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_ref] = ACTIONS(1333),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_mutable_specifier] = ACTIONS(1337),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1345),
    [sym_super] = ACTIONS(1347),
    [sym_crate] = ACTIONS(1347),
    [sym_metavariable] = ACTIONS(1349),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [416] = {
    [sym_attribute_item] = STATE(423),
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_self_parameter] = STATE(3137),
    [sym_variadic_parameter] = STATE(3137),
    [sym_parameter] = STATE(3137),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2807),
    [sym_bracketed_type] = STATE(3550),
    [sym_lifetime] = STATE(2800),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3221),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(2410),
    [sym_scoped_identifier] = STATE(2242),
    [sym_scoped_type_identifier] = STATE(2118),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(3256),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(416),
    [sym_block_comment] = STATE(416),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1281),
    [anon_sym_LPAREN] = ACTIONS(1283),
    [anon_sym_LBRACK] = ACTIONS(1287),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1293),
    [anon_sym_i8] = ACTIONS(1293),
    [anon_sym_u16] = ACTIONS(1293),
    [anon_sym_i16] = ACTIONS(1293),
    [anon_sym_u32] = ACTIONS(1293),
    [anon_sym_i32] = ACTIONS(1293),
    [anon_sym_u64] = ACTIONS(1293),
    [anon_sym_i64] = ACTIONS(1293),
    [anon_sym_u128] = ACTIONS(1293),
    [anon_sym_i128] = ACTIONS(1293),
    [anon_sym_isize] = ACTIONS(1293),
    [anon_sym_usize] = ACTIONS(1293),
    [anon_sym_f32] = ACTIONS(1293),
    [anon_sym_f64] = ACTIONS(1293),
    [anon_sym_bool] = ACTIONS(1293),
    [anon_sym_str] = ACTIONS(1293),
    [anon_sym_char] = ACTIONS(1293),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1299),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1493),
    [anon_sym_DOT_DOT] = ACTIONS(1305),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1307),
    [anon_sym_COLON_COLON] = ACTIONS(1311),
    [anon_sym_POUND] = ACTIONS(1313),
    [anon_sym_SQUOTE] = ACTIONS(1315),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1319),
    [anon_sym_default] = ACTIONS(1321),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1327),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1327),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_ref] = ACTIONS(1333),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_mutable_specifier] = ACTIONS(1337),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1345),
    [sym_super] = ACTIONS(1347),
    [sym_crate] = ACTIONS(1347),
    [sym_metavariable] = ACTIONS(1349),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [417] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2728),
    [sym_bracketed_type] = STATE(3557),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3301),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(2446),
    [sym_scoped_identifier] = STATE(2125),
    [sym_scoped_type_identifier] = STATE(2118),
    [sym_const_block] = STATE(2136),
    [sym_closure_expression] = STATE(2866),
    [sym_closure_parameters] = STATE(220),
    [sym__pattern] = STATE(2724),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(417),
    [sym_block_comment] = STATE(417),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1369),
    [anon_sym_LPAREN] = ACTIONS(1371),
    [anon_sym_RPAREN] = ACTIONS(1511),
    [anon_sym_LBRACK] = ACTIONS(1287),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1375),
    [anon_sym_i8] = ACTIONS(1375),
    [anon_sym_u16] = ACTIONS(1375),
    [anon_sym_i16] = ACTIONS(1375),
    [anon_sym_u32] = ACTIONS(1375),
    [anon_sym_i32] = ACTIONS(1375),
    [anon_sym_u64] = ACTIONS(1375),
    [anon_sym_i64] = ACTIONS(1375),
    [anon_sym_u128] = ACTIONS(1375),
    [anon_sym_i128] = ACTIONS(1375),
    [anon_sym_isize] = ACTIONS(1375),
    [anon_sym_usize] = ACTIONS(1375),
    [anon_sym_f32] = ACTIONS(1375),
    [anon_sym_f64] = ACTIONS(1375),
    [anon_sym_bool] = ACTIONS(1375),
    [anon_sym_str] = ACTIONS(1375),
    [anon_sym_char] = ACTIONS(1375),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1513),
    [anon_sym_PIPE] = ACTIONS(1515),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COMMA] = ACTIONS(1521),
    [anon_sym_COLON_COLON] = ACTIONS(1383),
    [anon_sym_SQUOTE] = ACTIONS(1315),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1319),
    [anon_sym_default] = ACTIONS(1385),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1387),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_static] = ACTIONS(1523),
    [anon_sym_union] = ACTIONS(1387),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_ref] = ACTIONS(1333),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_mutable_specifier] = ACTIONS(1525),
    [anon_sym_move] = ACTIONS(1527),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1391),
    [sym_super] = ACTIONS(1391),
    [sym_crate] = ACTIONS(1391),
    [sym_metavariable] = ACTIONS(1393),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [418] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2728),
    [sym_bracketed_type] = STATE(3557),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3301),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(2446),
    [sym_scoped_identifier] = STATE(2125),
    [sym_scoped_type_identifier] = STATE(2118),
    [sym_const_block] = STATE(2136),
    [sym_closure_expression] = STATE(2866),
    [sym_closure_parameters] = STATE(220),
    [sym__pattern] = STATE(2724),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(418),
    [sym_block_comment] = STATE(418),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1369),
    [anon_sym_LPAREN] = ACTIONS(1371),
    [anon_sym_RPAREN] = ACTIONS(1529),
    [anon_sym_LBRACK] = ACTIONS(1287),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1375),
    [anon_sym_i8] = ACTIONS(1375),
    [anon_sym_u16] = ACTIONS(1375),
    [anon_sym_i16] = ACTIONS(1375),
    [anon_sym_u32] = ACTIONS(1375),
    [anon_sym_i32] = ACTIONS(1375),
    [anon_sym_u64] = ACTIONS(1375),
    [anon_sym_i64] = ACTIONS(1375),
    [anon_sym_u128] = ACTIONS(1375),
    [anon_sym_i128] = ACTIONS(1375),
    [anon_sym_isize] = ACTIONS(1375),
    [anon_sym_usize] = ACTIONS(1375),
    [anon_sym_f32] = ACTIONS(1375),
    [anon_sym_f64] = ACTIONS(1375),
    [anon_sym_bool] = ACTIONS(1375),
    [anon_sym_str] = ACTIONS(1375),
    [anon_sym_char] = ACTIONS(1375),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1513),
    [anon_sym_PIPE] = ACTIONS(1515),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COMMA] = ACTIONS(1521),
    [anon_sym_COLON_COLON] = ACTIONS(1383),
    [anon_sym_SQUOTE] = ACTIONS(1315),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1319),
    [anon_sym_default] = ACTIONS(1385),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1387),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_static] = ACTIONS(1523),
    [anon_sym_union] = ACTIONS(1387),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_ref] = ACTIONS(1333),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_mutable_specifier] = ACTIONS(1525),
    [anon_sym_move] = ACTIONS(1527),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1391),
    [sym_super] = ACTIONS(1391),
    [sym_crate] = ACTIONS(1391),
    [sym_metavariable] = ACTIONS(1393),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [419] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2728),
    [sym_bracketed_type] = STATE(3557),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3301),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(2446),
    [sym_scoped_identifier] = STATE(2125),
    [sym_scoped_type_identifier] = STATE(2118),
    [sym_const_block] = STATE(2136),
    [sym_closure_expression] = STATE(2866),
    [sym_closure_parameters] = STATE(220),
    [sym__pattern] = STATE(2724),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(419),
    [sym_block_comment] = STATE(419),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1369),
    [anon_sym_LPAREN] = ACTIONS(1371),
    [anon_sym_RPAREN] = ACTIONS(1531),
    [anon_sym_LBRACK] = ACTIONS(1287),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1375),
    [anon_sym_i8] = ACTIONS(1375),
    [anon_sym_u16] = ACTIONS(1375),
    [anon_sym_i16] = ACTIONS(1375),
    [anon_sym_u32] = ACTIONS(1375),
    [anon_sym_i32] = ACTIONS(1375),
    [anon_sym_u64] = ACTIONS(1375),
    [anon_sym_i64] = ACTIONS(1375),
    [anon_sym_u128] = ACTIONS(1375),
    [anon_sym_i128] = ACTIONS(1375),
    [anon_sym_isize] = ACTIONS(1375),
    [anon_sym_usize] = ACTIONS(1375),
    [anon_sym_f32] = ACTIONS(1375),
    [anon_sym_f64] = ACTIONS(1375),
    [anon_sym_bool] = ACTIONS(1375),
    [anon_sym_str] = ACTIONS(1375),
    [anon_sym_char] = ACTIONS(1375),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1513),
    [anon_sym_PIPE] = ACTIONS(1515),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COMMA] = ACTIONS(1521),
    [anon_sym_COLON_COLON] = ACTIONS(1383),
    [anon_sym_SQUOTE] = ACTIONS(1315),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1319),
    [anon_sym_default] = ACTIONS(1385),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1387),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_static] = ACTIONS(1523),
    [anon_sym_union] = ACTIONS(1387),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_ref] = ACTIONS(1333),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_mutable_specifier] = ACTIONS(1525),
    [anon_sym_move] = ACTIONS(1527),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1391),
    [sym_super] = ACTIONS(1391),
    [sym_crate] = ACTIONS(1391),
    [sym_metavariable] = ACTIONS(1393),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [420] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_self_parameter] = STATE(3012),
    [sym_variadic_parameter] = STATE(3012),
    [sym_parameter] = STATE(3012),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2693),
    [sym_bracketed_type] = STATE(3550),
    [sym_lifetime] = STATE(2800),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3221),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(2410),
    [sym_scoped_identifier] = STATE(2242),
    [sym_scoped_type_identifier] = STATE(2118),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(3256),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(420),
    [sym_block_comment] = STATE(420),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1281),
    [anon_sym_LPAREN] = ACTIONS(1283),
    [anon_sym_LBRACK] = ACTIONS(1287),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1293),
    [anon_sym_i8] = ACTIONS(1293),
    [anon_sym_u16] = ACTIONS(1293),
    [anon_sym_i16] = ACTIONS(1293),
    [anon_sym_u32] = ACTIONS(1293),
    [anon_sym_i32] = ACTIONS(1293),
    [anon_sym_u64] = ACTIONS(1293),
    [anon_sym_i64] = ACTIONS(1293),
    [anon_sym_u128] = ACTIONS(1293),
    [anon_sym_i128] = ACTIONS(1293),
    [anon_sym_isize] = ACTIONS(1293),
    [anon_sym_usize] = ACTIONS(1293),
    [anon_sym_f32] = ACTIONS(1293),
    [anon_sym_f64] = ACTIONS(1293),
    [anon_sym_bool] = ACTIONS(1293),
    [anon_sym_str] = ACTIONS(1293),
    [anon_sym_char] = ACTIONS(1293),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1299),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1533),
    [anon_sym_DOT_DOT] = ACTIONS(1305),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1307),
    [anon_sym_COLON_COLON] = ACTIONS(1311),
    [anon_sym_SQUOTE] = ACTIONS(1315),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1319),
    [anon_sym_default] = ACTIONS(1321),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1327),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1327),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_ref] = ACTIONS(1333),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_mutable_specifier] = ACTIONS(1337),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1345),
    [sym_super] = ACTIONS(1347),
    [sym_crate] = ACTIONS(1347),
    [sym_metavariable] = ACTIONS(1349),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [421] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_self_parameter] = STATE(2836),
    [sym_variadic_parameter] = STATE(2836),
    [sym_parameter] = STATE(2836),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2610),
    [sym_bracketed_type] = STATE(3550),
    [sym_lifetime] = STATE(2800),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3221),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(2410),
    [sym_scoped_identifier] = STATE(2242),
    [sym_scoped_type_identifier] = STATE(2118),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(3256),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(421),
    [sym_block_comment] = STATE(421),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1281),
    [anon_sym_LPAREN] = ACTIONS(1283),
    [anon_sym_LBRACK] = ACTIONS(1287),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1293),
    [anon_sym_i8] = ACTIONS(1293),
    [anon_sym_u16] = ACTIONS(1293),
    [anon_sym_i16] = ACTIONS(1293),
    [anon_sym_u32] = ACTIONS(1293),
    [anon_sym_i32] = ACTIONS(1293),
    [anon_sym_u64] = ACTIONS(1293),
    [anon_sym_i64] = ACTIONS(1293),
    [anon_sym_u128] = ACTIONS(1293),
    [anon_sym_i128] = ACTIONS(1293),
    [anon_sym_isize] = ACTIONS(1293),
    [anon_sym_usize] = ACTIONS(1293),
    [anon_sym_f32] = ACTIONS(1293),
    [anon_sym_f64] = ACTIONS(1293),
    [anon_sym_bool] = ACTIONS(1293),
    [anon_sym_str] = ACTIONS(1293),
    [anon_sym_char] = ACTIONS(1293),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1299),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1535),
    [anon_sym_DOT_DOT] = ACTIONS(1305),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1307),
    [anon_sym_COLON_COLON] = ACTIONS(1311),
    [anon_sym_SQUOTE] = ACTIONS(1315),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1319),
    [anon_sym_default] = ACTIONS(1321),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1327),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1327),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_ref] = ACTIONS(1333),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_mutable_specifier] = ACTIONS(1337),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1345),
    [sym_super] = ACTIONS(1347),
    [sym_crate] = ACTIONS(1347),
    [sym_metavariable] = ACTIONS(1349),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [422] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_self_parameter] = STATE(2928),
    [sym_variadic_parameter] = STATE(2928),
    [sym_parameter] = STATE(2928),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2592),
    [sym_bracketed_type] = STATE(3550),
    [sym_lifetime] = STATE(2800),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3221),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(2410),
    [sym_scoped_identifier] = STATE(2242),
    [sym_scoped_type_identifier] = STATE(2118),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(3256),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(422),
    [sym_block_comment] = STATE(422),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1281),
    [anon_sym_LPAREN] = ACTIONS(1283),
    [anon_sym_LBRACK] = ACTIONS(1287),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1293),
    [anon_sym_i8] = ACTIONS(1293),
    [anon_sym_u16] = ACTIONS(1293),
    [anon_sym_i16] = ACTIONS(1293),
    [anon_sym_u32] = ACTIONS(1293),
    [anon_sym_i32] = ACTIONS(1293),
    [anon_sym_u64] = ACTIONS(1293),
    [anon_sym_i64] = ACTIONS(1293),
    [anon_sym_u128] = ACTIONS(1293),
    [anon_sym_i128] = ACTIONS(1293),
    [anon_sym_isize] = ACTIONS(1293),
    [anon_sym_usize] = ACTIONS(1293),
    [anon_sym_f32] = ACTIONS(1293),
    [anon_sym_f64] = ACTIONS(1293),
    [anon_sym_bool] = ACTIONS(1293),
    [anon_sym_str] = ACTIONS(1293),
    [anon_sym_char] = ACTIONS(1293),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1299),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1537),
    [anon_sym_DOT_DOT] = ACTIONS(1305),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1307),
    [anon_sym_COLON_COLON] = ACTIONS(1311),
    [anon_sym_SQUOTE] = ACTIONS(1315),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1319),
    [anon_sym_default] = ACTIONS(1321),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1327),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1327),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_ref] = ACTIONS(1333),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_mutable_specifier] = ACTIONS(1337),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1345),
    [sym_super] = ACTIONS(1347),
    [sym_crate] = ACTIONS(1347),
    [sym_metavariable] = ACTIONS(1349),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [423] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_self_parameter] = STATE(3140),
    [sym_variadic_parameter] = STATE(3140),
    [sym_parameter] = STATE(3140),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2945),
    [sym_bracketed_type] = STATE(3550),
    [sym_lifetime] = STATE(2800),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3221),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(2410),
    [sym_scoped_identifier] = STATE(2242),
    [sym_scoped_type_identifier] = STATE(2118),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(3256),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(423),
    [sym_block_comment] = STATE(423),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1281),
    [anon_sym_LPAREN] = ACTIONS(1283),
    [anon_sym_LBRACK] = ACTIONS(1287),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1293),
    [anon_sym_i8] = ACTIONS(1293),
    [anon_sym_u16] = ACTIONS(1293),
    [anon_sym_i16] = ACTIONS(1293),
    [anon_sym_u32] = ACTIONS(1293),
    [anon_sym_i32] = ACTIONS(1293),
    [anon_sym_u64] = ACTIONS(1293),
    [anon_sym_i64] = ACTIONS(1293),
    [anon_sym_u128] = ACTIONS(1293),
    [anon_sym_i128] = ACTIONS(1293),
    [anon_sym_isize] = ACTIONS(1293),
    [anon_sym_usize] = ACTIONS(1293),
    [anon_sym_f32] = ACTIONS(1293),
    [anon_sym_f64] = ACTIONS(1293),
    [anon_sym_bool] = ACTIONS(1293),
    [anon_sym_str] = ACTIONS(1293),
    [anon_sym_char] = ACTIONS(1293),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1299),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1539),
    [anon_sym_DOT_DOT] = ACTIONS(1305),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1307),
    [anon_sym_COLON_COLON] = ACTIONS(1311),
    [anon_sym_SQUOTE] = ACTIONS(1315),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1319),
    [anon_sym_default] = ACTIONS(1321),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1327),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1327),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_ref] = ACTIONS(1333),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_mutable_specifier] = ACTIONS(1337),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1345),
    [sym_super] = ACTIONS(1347),
    [sym_crate] = ACTIONS(1347),
    [sym_metavariable] = ACTIONS(1349),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [424] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2904),
    [sym_bracketed_type] = STATE(3558),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3304),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(2489),
    [sym_scoped_identifier] = STATE(2191),
    [sym_scoped_type_identifier] = STATE(2118),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2670),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(424),
    [sym_block_comment] = STATE(424),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1541),
    [anon_sym_LPAREN] = ACTIONS(1543),
    [anon_sym_LBRACK] = ACTIONS(1287),
    [anon_sym_RBRACK] = ACTIONS(1545),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1547),
    [anon_sym_i8] = ACTIONS(1547),
    [anon_sym_u16] = ACTIONS(1547),
    [anon_sym_i16] = ACTIONS(1547),
    [anon_sym_u32] = ACTIONS(1547),
    [anon_sym_i32] = ACTIONS(1547),
    [anon_sym_u64] = ACTIONS(1547),
    [anon_sym_i64] = ACTIONS(1547),
    [anon_sym_u128] = ACTIONS(1547),
    [anon_sym_i128] = ACTIONS(1547),
    [anon_sym_isize] = ACTIONS(1547),
    [anon_sym_usize] = ACTIONS(1547),
    [anon_sym_f32] = ACTIONS(1547),
    [anon_sym_f64] = ACTIONS(1547),
    [anon_sym_bool] = ACTIONS(1547),
    [anon_sym_str] = ACTIONS(1547),
    [anon_sym_char] = ACTIONS(1547),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1549),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COMMA] = ACTIONS(1551),
    [anon_sym_COLON_COLON] = ACTIONS(1553),
    [anon_sym_SQUOTE] = ACTIONS(1315),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1319),
    [anon_sym_default] = ACTIONS(1555),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1557),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1557),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_ref] = ACTIONS(1333),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1559),
    [sym_super] = ACTIONS(1559),
    [sym_crate] = ACTIONS(1559),
    [sym_metavariable] = ACTIONS(1561),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [425] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1992),
    [sym_bracketed_type] = STATE(3557),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3301),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(2446),
    [sym_scoped_identifier] = STATE(2125),
    [sym_scoped_type_identifier] = STATE(2118),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2143),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(425),
    [sym_block_comment] = STATE(425),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1369),
    [anon_sym_LPAREN] = ACTIONS(1371),
    [anon_sym_LBRACK] = ACTIONS(1287),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1375),
    [anon_sym_i8] = ACTIONS(1375),
    [anon_sym_u16] = ACTIONS(1375),
    [anon_sym_i16] = ACTIONS(1375),
    [anon_sym_u32] = ACTIONS(1375),
    [anon_sym_i32] = ACTIONS(1375),
    [anon_sym_u64] = ACTIONS(1375),
    [anon_sym_i64] = ACTIONS(1375),
    [anon_sym_u128] = ACTIONS(1375),
    [anon_sym_i128] = ACTIONS(1375),
    [anon_sym_isize] = ACTIONS(1375),
    [anon_sym_usize] = ACTIONS(1375),
    [anon_sym_f32] = ACTIONS(1375),
    [anon_sym_f64] = ACTIONS(1375),
    [anon_sym_bool] = ACTIONS(1375),
    [anon_sym_str] = ACTIONS(1375),
    [anon_sym_char] = ACTIONS(1375),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1513),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(1383),
    [anon_sym_SQUOTE] = ACTIONS(1315),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1319),
    [anon_sym_default] = ACTIONS(1385),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1387),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1387),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_ref] = ACTIONS(1333),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1563),
    [sym_super] = ACTIONS(1391),
    [sym_crate] = ACTIONS(1391),
    [sym_metavariable] = ACTIONS(1393),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [426] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2004),
    [sym_bracketed_type] = STATE(3550),
    [sym_lifetime] = STATE(839),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3221),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(2410),
    [sym_scoped_identifier] = STATE(2242),
    [sym_scoped_type_identifier] = STATE(2118),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2134),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(426),
    [sym_block_comment] = STATE(426),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1281),
    [anon_sym_LPAREN] = ACTIONS(1283),
    [anon_sym_LBRACK] = ACTIONS(1287),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1293),
    [anon_sym_i8] = ACTIONS(1293),
    [anon_sym_u16] = ACTIONS(1293),
    [anon_sym_i16] = ACTIONS(1293),
    [anon_sym_u32] = ACTIONS(1293),
    [anon_sym_i32] = ACTIONS(1293),
    [anon_sym_u64] = ACTIONS(1293),
    [anon_sym_i64] = ACTIONS(1293),
    [anon_sym_u128] = ACTIONS(1293),
    [anon_sym_i128] = ACTIONS(1293),
    [anon_sym_isize] = ACTIONS(1293),
    [anon_sym_usize] = ACTIONS(1293),
    [anon_sym_f32] = ACTIONS(1293),
    [anon_sym_f64] = ACTIONS(1293),
    [anon_sym_bool] = ACTIONS(1293),
    [anon_sym_str] = ACTIONS(1293),
    [anon_sym_char] = ACTIONS(1293),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1565),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(1311),
    [anon_sym_SQUOTE] = ACTIONS(1315),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1319),
    [anon_sym_default] = ACTIONS(1321),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1327),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1327),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_ref] = ACTIONS(1333),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_mutable_specifier] = ACTIONS(1567),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1569),
    [sym_super] = ACTIONS(1347),
    [sym_crate] = ACTIONS(1347),
    [sym_metavariable] = ACTIONS(1349),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [427] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2004),
    [sym_bracketed_type] = STATE(3557),
    [sym_lifetime] = STATE(839),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3301),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(2446),
    [sym_scoped_identifier] = STATE(2125),
    [sym_scoped_type_identifier] = STATE(2118),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2134),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(427),
    [sym_block_comment] = STATE(427),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1369),
    [anon_sym_LPAREN] = ACTIONS(1371),
    [anon_sym_LBRACK] = ACTIONS(1287),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1375),
    [anon_sym_i8] = ACTIONS(1375),
    [anon_sym_u16] = ACTIONS(1375),
    [anon_sym_i16] = ACTIONS(1375),
    [anon_sym_u32] = ACTIONS(1375),
    [anon_sym_i32] = ACTIONS(1375),
    [anon_sym_u64] = ACTIONS(1375),
    [anon_sym_i64] = ACTIONS(1375),
    [anon_sym_u128] = ACTIONS(1375),
    [anon_sym_i128] = ACTIONS(1375),
    [anon_sym_isize] = ACTIONS(1375),
    [anon_sym_usize] = ACTIONS(1375),
    [anon_sym_f32] = ACTIONS(1375),
    [anon_sym_f64] = ACTIONS(1375),
    [anon_sym_bool] = ACTIONS(1375),
    [anon_sym_str] = ACTIONS(1375),
    [anon_sym_char] = ACTIONS(1375),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1513),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(1383),
    [anon_sym_SQUOTE] = ACTIONS(1315),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1319),
    [anon_sym_default] = ACTIONS(1385),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1387),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1387),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_ref] = ACTIONS(1333),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_mutable_specifier] = ACTIONS(1571),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1573),
    [sym_super] = ACTIONS(1391),
    [sym_crate] = ACTIONS(1391),
    [sym_metavariable] = ACTIONS(1393),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [428] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2004),
    [sym_bracketed_type] = STATE(3550),
    [sym_lifetime] = STATE(840),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3221),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(2410),
    [sym_scoped_identifier] = STATE(2242),
    [sym_scoped_type_identifier] = STATE(2118),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2134),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(428),
    [sym_block_comment] = STATE(428),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1281),
    [anon_sym_LPAREN] = ACTIONS(1283),
    [anon_sym_LBRACK] = ACTIONS(1287),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1293),
    [anon_sym_i8] = ACTIONS(1293),
    [anon_sym_u16] = ACTIONS(1293),
    [anon_sym_i16] = ACTIONS(1293),
    [anon_sym_u32] = ACTIONS(1293),
    [anon_sym_i32] = ACTIONS(1293),
    [anon_sym_u64] = ACTIONS(1293),
    [anon_sym_i64] = ACTIONS(1293),
    [anon_sym_u128] = ACTIONS(1293),
    [anon_sym_i128] = ACTIONS(1293),
    [anon_sym_isize] = ACTIONS(1293),
    [anon_sym_usize] = ACTIONS(1293),
    [anon_sym_f32] = ACTIONS(1293),
    [anon_sym_f64] = ACTIONS(1293),
    [anon_sym_bool] = ACTIONS(1293),
    [anon_sym_str] = ACTIONS(1293),
    [anon_sym_char] = ACTIONS(1293),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1565),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(1311),
    [anon_sym_SQUOTE] = ACTIONS(1315),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1319),
    [anon_sym_default] = ACTIONS(1321),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1327),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1327),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_ref] = ACTIONS(1333),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_mutable_specifier] = ACTIONS(1575),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1347),
    [sym_super] = ACTIONS(1347),
    [sym_crate] = ACTIONS(1347),
    [sym_metavariable] = ACTIONS(1349),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [429] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1992),
    [sym_bracketed_type] = STATE(3550),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3221),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(2410),
    [sym_scoped_identifier] = STATE(2242),
    [sym_scoped_type_identifier] = STATE(2118),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2143),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(429),
    [sym_block_comment] = STATE(429),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1281),
    [anon_sym_LPAREN] = ACTIONS(1283),
    [anon_sym_LBRACK] = ACTIONS(1287),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1293),
    [anon_sym_i8] = ACTIONS(1293),
    [anon_sym_u16] = ACTIONS(1293),
    [anon_sym_i16] = ACTIONS(1293),
    [anon_sym_u32] = ACTIONS(1293),
    [anon_sym_i32] = ACTIONS(1293),
    [anon_sym_u64] = ACTIONS(1293),
    [anon_sym_i64] = ACTIONS(1293),
    [anon_sym_u128] = ACTIONS(1293),
    [anon_sym_i128] = ACTIONS(1293),
    [anon_sym_isize] = ACTIONS(1293),
    [anon_sym_usize] = ACTIONS(1293),
    [anon_sym_f32] = ACTIONS(1293),
    [anon_sym_f64] = ACTIONS(1293),
    [anon_sym_bool] = ACTIONS(1293),
    [anon_sym_str] = ACTIONS(1293),
    [anon_sym_char] = ACTIONS(1293),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1565),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(1311),
    [anon_sym_SQUOTE] = ACTIONS(1315),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1319),
    [anon_sym_default] = ACTIONS(1321),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1327),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1327),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_ref] = ACTIONS(1333),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1347),
    [sym_super] = ACTIONS(1347),
    [sym_crate] = ACTIONS(1347),
    [sym_metavariable] = ACTIONS(1349),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [430] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2004),
    [sym_bracketed_type] = STATE(3557),
    [sym_lifetime] = STATE(840),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3301),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(2446),
    [sym_scoped_identifier] = STATE(2125),
    [sym_scoped_type_identifier] = STATE(2118),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2134),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(430),
    [sym_block_comment] = STATE(430),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1369),
    [anon_sym_LPAREN] = ACTIONS(1371),
    [anon_sym_LBRACK] = ACTIONS(1287),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1375),
    [anon_sym_i8] = ACTIONS(1375),
    [anon_sym_u16] = ACTIONS(1375),
    [anon_sym_i16] = ACTIONS(1375),
    [anon_sym_u32] = ACTIONS(1375),
    [anon_sym_i32] = ACTIONS(1375),
    [anon_sym_u64] = ACTIONS(1375),
    [anon_sym_i64] = ACTIONS(1375),
    [anon_sym_u128] = ACTIONS(1375),
    [anon_sym_i128] = ACTIONS(1375),
    [anon_sym_isize] = ACTIONS(1375),
    [anon_sym_usize] = ACTIONS(1375),
    [anon_sym_f32] = ACTIONS(1375),
    [anon_sym_f64] = ACTIONS(1375),
    [anon_sym_bool] = ACTIONS(1375),
    [anon_sym_str] = ACTIONS(1375),
    [anon_sym_char] = ACTIONS(1375),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1513),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(1383),
    [anon_sym_SQUOTE] = ACTIONS(1315),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1319),
    [anon_sym_default] = ACTIONS(1385),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1387),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1387),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_ref] = ACTIONS(1333),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_mutable_specifier] = ACTIONS(1577),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1391),
    [sym_super] = ACTIONS(1391),
    [sym_crate] = ACTIONS(1391),
    [sym_metavariable] = ACTIONS(1393),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [431] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1992),
    [sym_bracketed_type] = STATE(3550),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3221),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(2410),
    [sym_scoped_identifier] = STATE(2242),
    [sym_scoped_type_identifier] = STATE(2118),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2143),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(431),
    [sym_block_comment] = STATE(431),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1281),
    [anon_sym_LPAREN] = ACTIONS(1283),
    [anon_sym_LBRACK] = ACTIONS(1287),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1293),
    [anon_sym_i8] = ACTIONS(1293),
    [anon_sym_u16] = ACTIONS(1293),
    [anon_sym_i16] = ACTIONS(1293),
    [anon_sym_u32] = ACTIONS(1293),
    [anon_sym_i32] = ACTIONS(1293),
    [anon_sym_u64] = ACTIONS(1293),
    [anon_sym_i64] = ACTIONS(1293),
    [anon_sym_u128] = ACTIONS(1293),
    [anon_sym_i128] = ACTIONS(1293),
    [anon_sym_isize] = ACTIONS(1293),
    [anon_sym_usize] = ACTIONS(1293),
    [anon_sym_f32] = ACTIONS(1293),
    [anon_sym_f64] = ACTIONS(1293),
    [anon_sym_bool] = ACTIONS(1293),
    [anon_sym_str] = ACTIONS(1293),
    [anon_sym_char] = ACTIONS(1293),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1565),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(1311),
    [anon_sym_SQUOTE] = ACTIONS(1315),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1319),
    [anon_sym_default] = ACTIONS(1321),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1327),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1327),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_ref] = ACTIONS(1333),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1579),
    [sym_super] = ACTIONS(1347),
    [sym_crate] = ACTIONS(1347),
    [sym_metavariable] = ACTIONS(1349),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [432] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2004),
    [sym_bracketed_type] = STATE(3558),
    [sym_lifetime] = STATE(840),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3304),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(2489),
    [sym_scoped_identifier] = STATE(2191),
    [sym_scoped_type_identifier] = STATE(2118),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2134),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(432),
    [sym_block_comment] = STATE(432),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1541),
    [anon_sym_LPAREN] = ACTIONS(1543),
    [anon_sym_LBRACK] = ACTIONS(1287),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1547),
    [anon_sym_i8] = ACTIONS(1547),
    [anon_sym_u16] = ACTIONS(1547),
    [anon_sym_i16] = ACTIONS(1547),
    [anon_sym_u32] = ACTIONS(1547),
    [anon_sym_i32] = ACTIONS(1547),
    [anon_sym_u64] = ACTIONS(1547),
    [anon_sym_i64] = ACTIONS(1547),
    [anon_sym_u128] = ACTIONS(1547),
    [anon_sym_i128] = ACTIONS(1547),
    [anon_sym_isize] = ACTIONS(1547),
    [anon_sym_usize] = ACTIONS(1547),
    [anon_sym_f32] = ACTIONS(1547),
    [anon_sym_f64] = ACTIONS(1547),
    [anon_sym_bool] = ACTIONS(1547),
    [anon_sym_str] = ACTIONS(1547),
    [anon_sym_char] = ACTIONS(1547),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1549),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(1553),
    [anon_sym_SQUOTE] = ACTIONS(1315),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1319),
    [anon_sym_default] = ACTIONS(1555),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1557),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1557),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_ref] = ACTIONS(1333),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_mutable_specifier] = ACTIONS(1581),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1559),
    [sym_super] = ACTIONS(1559),
    [sym_crate] = ACTIONS(1559),
    [sym_metavariable] = ACTIONS(1561),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [433] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1992),
    [sym_bracketed_type] = STATE(3558),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3304),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(2489),
    [sym_scoped_identifier] = STATE(2191),
    [sym_scoped_type_identifier] = STATE(2118),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2143),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(433),
    [sym_block_comment] = STATE(433),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1541),
    [anon_sym_LPAREN] = ACTIONS(1543),
    [anon_sym_LBRACK] = ACTIONS(1287),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1547),
    [anon_sym_i8] = ACTIONS(1547),
    [anon_sym_u16] = ACTIONS(1547),
    [anon_sym_i16] = ACTIONS(1547),
    [anon_sym_u32] = ACTIONS(1547),
    [anon_sym_i32] = ACTIONS(1547),
    [anon_sym_u64] = ACTIONS(1547),
    [anon_sym_i64] = ACTIONS(1547),
    [anon_sym_u128] = ACTIONS(1547),
    [anon_sym_i128] = ACTIONS(1547),
    [anon_sym_isize] = ACTIONS(1547),
    [anon_sym_usize] = ACTIONS(1547),
    [anon_sym_f32] = ACTIONS(1547),
    [anon_sym_f64] = ACTIONS(1547),
    [anon_sym_bool] = ACTIONS(1547),
    [anon_sym_str] = ACTIONS(1547),
    [anon_sym_char] = ACTIONS(1547),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1549),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(1553),
    [anon_sym_SQUOTE] = ACTIONS(1315),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1319),
    [anon_sym_default] = ACTIONS(1555),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1557),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1557),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_ref] = ACTIONS(1333),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1559),
    [sym_super] = ACTIONS(1559),
    [sym_crate] = ACTIONS(1559),
    [sym_metavariable] = ACTIONS(1561),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [434] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1992),
    [sym_bracketed_type] = STATE(3557),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3301),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(2446),
    [sym_scoped_identifier] = STATE(2125),
    [sym_scoped_type_identifier] = STATE(2118),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2143),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(434),
    [sym_block_comment] = STATE(434),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1369),
    [anon_sym_LPAREN] = ACTIONS(1371),
    [anon_sym_LBRACK] = ACTIONS(1287),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1375),
    [anon_sym_i8] = ACTIONS(1375),
    [anon_sym_u16] = ACTIONS(1375),
    [anon_sym_i16] = ACTIONS(1375),
    [anon_sym_u32] = ACTIONS(1375),
    [anon_sym_i32] = ACTIONS(1375),
    [anon_sym_u64] = ACTIONS(1375),
    [anon_sym_i64] = ACTIONS(1375),
    [anon_sym_u128] = ACTIONS(1375),
    [anon_sym_i128] = ACTIONS(1375),
    [anon_sym_isize] = ACTIONS(1375),
    [anon_sym_usize] = ACTIONS(1375),
    [anon_sym_f32] = ACTIONS(1375),
    [anon_sym_f64] = ACTIONS(1375),
    [anon_sym_bool] = ACTIONS(1375),
    [anon_sym_str] = ACTIONS(1375),
    [anon_sym_char] = ACTIONS(1375),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1513),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(1383),
    [anon_sym_SQUOTE] = ACTIONS(1315),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1319),
    [anon_sym_default] = ACTIONS(1385),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1387),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1387),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_ref] = ACTIONS(1333),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1391),
    [sym_super] = ACTIONS(1391),
    [sym_crate] = ACTIONS(1391),
    [sym_metavariable] = ACTIONS(1393),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [435] = {
    [sym_line_comment] = STATE(435),
    [sym_block_comment] = STATE(435),
    [sym_identifier] = ACTIONS(1583),
    [anon_sym_SEMI] = ACTIONS(1585),
    [anon_sym_LPAREN] = ACTIONS(1585),
    [anon_sym_RPAREN] = ACTIONS(1585),
    [anon_sym_LBRACK] = ACTIONS(1585),
    [anon_sym_RBRACK] = ACTIONS(1585),
    [anon_sym_LBRACE] = ACTIONS(1585),
    [anon_sym_RBRACE] = ACTIONS(1585),
    [anon_sym_COLON] = ACTIONS(1583),
    [anon_sym_PLUS] = ACTIONS(1583),
    [anon_sym_STAR] = ACTIONS(1583),
    [anon_sym_QMARK] = ACTIONS(1585),
    [anon_sym_u8] = ACTIONS(1583),
    [anon_sym_i8] = ACTIONS(1583),
    [anon_sym_u16] = ACTIONS(1583),
    [anon_sym_i16] = ACTIONS(1583),
    [anon_sym_u32] = ACTIONS(1583),
    [anon_sym_i32] = ACTIONS(1583),
    [anon_sym_u64] = ACTIONS(1583),
    [anon_sym_i64] = ACTIONS(1583),
    [anon_sym_u128] = ACTIONS(1583),
    [anon_sym_i128] = ACTIONS(1583),
    [anon_sym_isize] = ACTIONS(1583),
    [anon_sym_usize] = ACTIONS(1583),
    [anon_sym_f32] = ACTIONS(1583),
    [anon_sym_f64] = ACTIONS(1583),
    [anon_sym_bool] = ACTIONS(1583),
    [anon_sym_str] = ACTIONS(1583),
    [anon_sym_char] = ACTIONS(1583),
    [anon_sym_DASH] = ACTIONS(1583),
    [anon_sym_SLASH] = ACTIONS(1583),
    [anon_sym_PERCENT] = ACTIONS(1583),
    [anon_sym_CARET] = ACTIONS(1583),
    [anon_sym_BANG] = ACTIONS(1583),
    [anon_sym_AMP] = ACTIONS(1583),
    [anon_sym_PIPE] = ACTIONS(1583),
    [anon_sym_AMP_AMP] = ACTIONS(1585),
    [anon_sym_PIPE_PIPE] = ACTIONS(1585),
    [anon_sym_LT_LT] = ACTIONS(1583),
    [anon_sym_GT_GT] = ACTIONS(1583),
    [anon_sym_PLUS_EQ] = ACTIONS(1585),
    [anon_sym_DASH_EQ] = ACTIONS(1585),
    [anon_sym_STAR_EQ] = ACTIONS(1585),
    [anon_sym_SLASH_EQ] = ACTIONS(1585),
    [anon_sym_PERCENT_EQ] = ACTIONS(1585),
    [anon_sym_CARET_EQ] = ACTIONS(1585),
    [anon_sym_AMP_EQ] = ACTIONS(1585),
    [anon_sym_PIPE_EQ] = ACTIONS(1585),
    [anon_sym_LT_LT_EQ] = ACTIONS(1585),
    [anon_sym_GT_GT_EQ] = ACTIONS(1585),
    [anon_sym_EQ] = ACTIONS(1583),
    [anon_sym_EQ_EQ] = ACTIONS(1585),
    [anon_sym_BANG_EQ] = ACTIONS(1585),
    [anon_sym_GT] = ACTIONS(1583),
    [anon_sym_LT] = ACTIONS(1583),
    [anon_sym_GT_EQ] = ACTIONS(1585),
    [anon_sym_LT_EQ] = ACTIONS(1585),
    [anon_sym_DOT] = ACTIONS(1583),
    [anon_sym_DOT_DOT] = ACTIONS(1583),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1585),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1585),
    [anon_sym_COMMA] = ACTIONS(1585),
    [anon_sym_COLON_COLON] = ACTIONS(1585),
    [anon_sym_SQUOTE] = ACTIONS(1583),
    [anon_sym_as] = ACTIONS(1583),
    [anon_sym_async] = ACTIONS(1583),
    [anon_sym_break] = ACTIONS(1583),
    [anon_sym_const] = ACTIONS(1583),
    [anon_sym_continue] = ACTIONS(1583),
    [anon_sym_default] = ACTIONS(1583),
    [anon_sym_for] = ACTIONS(1583),
    [anon_sym_gen] = ACTIONS(1583),
    [anon_sym_if] = ACTIONS(1583),
    [anon_sym_loop] = ACTIONS(1583),
    [anon_sym_match] = ACTIONS(1583),
    [anon_sym_return] = ACTIONS(1583),
    [anon_sym_static] = ACTIONS(1583),
    [anon_sym_union] = ACTIONS(1583),
    [anon_sym_unsafe] = ACTIONS(1583),
    [anon_sym_while] = ACTIONS(1583),
    [anon_sym_else] = ACTIONS(1583),
    [anon_sym_yield] = ACTIONS(1583),
    [anon_sym_move] = ACTIONS(1583),
    [anon_sym_try] = ACTIONS(1583),
    [sym_integer_literal] = ACTIONS(1585),
    [aux_sym_string_literal_token1] = ACTIONS(1585),
    [sym_char_literal] = ACTIONS(1585),
    [anon_sym_true] = ACTIONS(1583),
    [anon_sym_false] = ACTIONS(1583),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1583),
    [sym_super] = ACTIONS(1583),
    [sym_crate] = ACTIONS(1583),
    [sym_metavariable] = ACTIONS(1585),
    [sym__raw_string_literal_start] = ACTIONS(1585),
    [sym_float_literal] = ACTIONS(1585),
  },
  [436] = {
    [sym_line_comment] = STATE(436),
    [sym_block_comment] = STATE(436),
    [sym_identifier] = ACTIONS(1583),
    [anon_sym_LPAREN] = ACTIONS(1585),
    [anon_sym_LBRACK] = ACTIONS(1585),
    [anon_sym_LBRACE] = ACTIONS(1585),
    [anon_sym_EQ_GT] = ACTIONS(1585),
    [anon_sym_COLON] = ACTIONS(1583),
    [anon_sym_PLUS] = ACTIONS(1583),
    [anon_sym_STAR] = ACTIONS(1583),
    [anon_sym_QMARK] = ACTIONS(1585),
    [anon_sym_u8] = ACTIONS(1583),
    [anon_sym_i8] = ACTIONS(1583),
    [anon_sym_u16] = ACTIONS(1583),
    [anon_sym_i16] = ACTIONS(1583),
    [anon_sym_u32] = ACTIONS(1583),
    [anon_sym_i32] = ACTIONS(1583),
    [anon_sym_u64] = ACTIONS(1583),
    [anon_sym_i64] = ACTIONS(1583),
    [anon_sym_u128] = ACTIONS(1583),
    [anon_sym_i128] = ACTIONS(1583),
    [anon_sym_isize] = ACTIONS(1583),
    [anon_sym_usize] = ACTIONS(1583),
    [anon_sym_f32] = ACTIONS(1583),
    [anon_sym_f64] = ACTIONS(1583),
    [anon_sym_bool] = ACTIONS(1583),
    [anon_sym_str] = ACTIONS(1583),
    [anon_sym_char] = ACTIONS(1583),
    [anon_sym_DASH] = ACTIONS(1583),
    [anon_sym_SLASH] = ACTIONS(1583),
    [anon_sym_PERCENT] = ACTIONS(1583),
    [anon_sym_CARET] = ACTIONS(1583),
    [anon_sym_BANG] = ACTIONS(1583),
    [anon_sym_AMP] = ACTIONS(1583),
    [anon_sym_PIPE] = ACTIONS(1583),
    [anon_sym_AMP_AMP] = ACTIONS(1585),
    [anon_sym_PIPE_PIPE] = ACTIONS(1585),
    [anon_sym_LT_LT] = ACTIONS(1583),
    [anon_sym_GT_GT] = ACTIONS(1583),
    [anon_sym_PLUS_EQ] = ACTIONS(1585),
    [anon_sym_DASH_EQ] = ACTIONS(1585),
    [anon_sym_STAR_EQ] = ACTIONS(1585),
    [anon_sym_SLASH_EQ] = ACTIONS(1585),
    [anon_sym_PERCENT_EQ] = ACTIONS(1585),
    [anon_sym_CARET_EQ] = ACTIONS(1585),
    [anon_sym_AMP_EQ] = ACTIONS(1585),
    [anon_sym_PIPE_EQ] = ACTIONS(1585),
    [anon_sym_LT_LT_EQ] = ACTIONS(1585),
    [anon_sym_GT_GT_EQ] = ACTIONS(1585),
    [anon_sym_EQ] = ACTIONS(1583),
    [anon_sym_EQ_EQ] = ACTIONS(1585),
    [anon_sym_BANG_EQ] = ACTIONS(1585),
    [anon_sym_GT] = ACTIONS(1583),
    [anon_sym_LT] = ACTIONS(1583),
    [anon_sym_GT_EQ] = ACTIONS(1585),
    [anon_sym_LT_EQ] = ACTIONS(1585),
    [anon_sym_DOT] = ACTIONS(1583),
    [anon_sym_DOT_DOT] = ACTIONS(1583),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1585),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1585),
    [anon_sym_COLON_COLON] = ACTIONS(1585),
    [anon_sym_SQUOTE] = ACTIONS(1583),
    [anon_sym_as] = ACTIONS(1583),
    [anon_sym_async] = ACTIONS(1583),
    [anon_sym_break] = ACTIONS(1583),
    [anon_sym_const] = ACTIONS(1583),
    [anon_sym_continue] = ACTIONS(1583),
    [anon_sym_default] = ACTIONS(1583),
    [anon_sym_for] = ACTIONS(1583),
    [anon_sym_gen] = ACTIONS(1583),
    [anon_sym_if] = ACTIONS(1583),
    [anon_sym_loop] = ACTIONS(1583),
    [anon_sym_match] = ACTIONS(1583),
    [anon_sym_return] = ACTIONS(1583),
    [anon_sym_static] = ACTIONS(1583),
    [anon_sym_union] = ACTIONS(1583),
    [anon_sym_unsafe] = ACTIONS(1583),
    [anon_sym_while] = ACTIONS(1583),
    [anon_sym_yield] = ACTIONS(1583),
    [anon_sym_move] = ACTIONS(1583),
    [anon_sym_try] = ACTIONS(1583),
    [sym_integer_literal] = ACTIONS(1585),
    [aux_sym_string_literal_token1] = ACTIONS(1585),
    [sym_char_literal] = ACTIONS(1585),
    [anon_sym_true] = ACTIONS(1583),
    [anon_sym_false] = ACTIONS(1583),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1583),
    [sym_super] = ACTIONS(1583),
    [sym_crate] = ACTIONS(1583),
    [sym_metavariable] = ACTIONS(1585),
    [sym__raw_string_literal_start] = ACTIONS(1585),
    [sym_float_literal] = ACTIONS(1585),
  },
  [437] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2142),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(437),
    [sym_block_comment] = STATE(437),
    [sym_identifier] = ACTIONS(1587),
    [anon_sym_LPAREN] = ACTIONS(1589),
    [anon_sym_LBRACK] = ACTIONS(1589),
    [anon_sym_LBRACE] = ACTIONS(1589),
    [anon_sym_STAR] = ACTIONS(1589),
    [anon_sym_u8] = ACTIONS(1587),
    [anon_sym_i8] = ACTIONS(1587),
    [anon_sym_u16] = ACTIONS(1587),
    [anon_sym_i16] = ACTIONS(1587),
    [anon_sym_u32] = ACTIONS(1587),
    [anon_sym_i32] = ACTIONS(1587),
    [anon_sym_u64] = ACTIONS(1587),
    [anon_sym_i64] = ACTIONS(1587),
    [anon_sym_u128] = ACTIONS(1587),
    [anon_sym_i128] = ACTIONS(1587),
    [anon_sym_isize] = ACTIONS(1587),
    [anon_sym_usize] = ACTIONS(1587),
    [anon_sym_f32] = ACTIONS(1587),
    [anon_sym_f64] = ACTIONS(1587),
    [anon_sym_bool] = ACTIONS(1587),
    [anon_sym_str] = ACTIONS(1587),
    [anon_sym_char] = ACTIONS(1587),
    [anon_sym_DASH] = ACTIONS(1587),
    [anon_sym_BANG] = ACTIONS(1589),
    [anon_sym_AMP] = ACTIONS(1589),
    [anon_sym_PIPE] = ACTIONS(1589),
    [anon_sym_LT] = ACTIONS(1589),
    [anon_sym__] = ACTIONS(1587),
    [anon_sym_DOT_DOT] = ACTIONS(1589),
    [anon_sym_COLON_COLON] = ACTIONS(1589),
    [anon_sym_DASH_GT] = ACTIONS(1589),
    [anon_sym_SQUOTE] = ACTIONS(1587),
    [anon_sym_async] = ACTIONS(1587),
    [anon_sym_break] = ACTIONS(1587),
    [anon_sym_const] = ACTIONS(1587),
    [anon_sym_continue] = ACTIONS(1587),
    [anon_sym_default] = ACTIONS(1587),
    [anon_sym_for] = ACTIONS(1587),
    [anon_sym_gen] = ACTIONS(1587),
    [anon_sym_if] = ACTIONS(1587),
    [anon_sym_loop] = ACTIONS(1587),
    [anon_sym_match] = ACTIONS(1587),
    [anon_sym_return] = ACTIONS(1587),
    [anon_sym_static] = ACTIONS(1587),
    [anon_sym_union] = ACTIONS(1587),
    [anon_sym_unsafe] = ACTIONS(1587),
    [anon_sym_while] = ACTIONS(1587),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [anon_sym_yield] = ACTIONS(1587),
    [anon_sym_move] = ACTIONS(1587),
    [anon_sym_try] = ACTIONS(1587),
    [sym_integer_literal] = ACTIONS(1589),
    [aux_sym_string_literal_token1] = ACTIONS(1589),
    [sym_char_literal] = ACTIONS(1589),
    [anon_sym_true] = ACTIONS(1587),
    [anon_sym_false] = ACTIONS(1587),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1587),
    [sym_super] = ACTIONS(1587),
    [sym_crate] = ACTIONS(1587),
    [sym_metavariable] = ACTIONS(1589),
    [sym__raw_string_literal_start] = ACTIONS(1589),
    [sym_float_literal] = ACTIONS(1589),
  },
  [438] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2119),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(438),
    [sym_block_comment] = STATE(438),
    [sym_identifier] = ACTIONS(1591),
    [anon_sym_LPAREN] = ACTIONS(1593),
    [anon_sym_LBRACK] = ACTIONS(1593),
    [anon_sym_LBRACE] = ACTIONS(1593),
    [anon_sym_STAR] = ACTIONS(1593),
    [anon_sym_u8] = ACTIONS(1591),
    [anon_sym_i8] = ACTIONS(1591),
    [anon_sym_u16] = ACTIONS(1591),
    [anon_sym_i16] = ACTIONS(1591),
    [anon_sym_u32] = ACTIONS(1591),
    [anon_sym_i32] = ACTIONS(1591),
    [anon_sym_u64] = ACTIONS(1591),
    [anon_sym_i64] = ACTIONS(1591),
    [anon_sym_u128] = ACTIONS(1591),
    [anon_sym_i128] = ACTIONS(1591),
    [anon_sym_isize] = ACTIONS(1591),
    [anon_sym_usize] = ACTIONS(1591),
    [anon_sym_f32] = ACTIONS(1591),
    [anon_sym_f64] = ACTIONS(1591),
    [anon_sym_bool] = ACTIONS(1591),
    [anon_sym_str] = ACTIONS(1591),
    [anon_sym_char] = ACTIONS(1591),
    [anon_sym_DASH] = ACTIONS(1591),
    [anon_sym_BANG] = ACTIONS(1593),
    [anon_sym_AMP] = ACTIONS(1593),
    [anon_sym_PIPE] = ACTIONS(1593),
    [anon_sym_LT] = ACTIONS(1593),
    [anon_sym__] = ACTIONS(1591),
    [anon_sym_DOT_DOT] = ACTIONS(1593),
    [anon_sym_COLON_COLON] = ACTIONS(1593),
    [anon_sym_DASH_GT] = ACTIONS(1593),
    [anon_sym_SQUOTE] = ACTIONS(1591),
    [anon_sym_async] = ACTIONS(1591),
    [anon_sym_break] = ACTIONS(1591),
    [anon_sym_const] = ACTIONS(1591),
    [anon_sym_continue] = ACTIONS(1591),
    [anon_sym_default] = ACTIONS(1591),
    [anon_sym_for] = ACTIONS(1591),
    [anon_sym_gen] = ACTIONS(1591),
    [anon_sym_if] = ACTIONS(1591),
    [anon_sym_loop] = ACTIONS(1591),
    [anon_sym_match] = ACTIONS(1591),
    [anon_sym_return] = ACTIONS(1591),
    [anon_sym_static] = ACTIONS(1591),
    [anon_sym_union] = ACTIONS(1591),
    [anon_sym_unsafe] = ACTIONS(1591),
    [anon_sym_while] = ACTIONS(1591),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [anon_sym_yield] = ACTIONS(1591),
    [anon_sym_move] = ACTIONS(1591),
    [anon_sym_try] = ACTIONS(1591),
    [sym_integer_literal] = ACTIONS(1593),
    [aux_sym_string_literal_token1] = ACTIONS(1593),
    [sym_char_literal] = ACTIONS(1593),
    [anon_sym_true] = ACTIONS(1591),
    [anon_sym_false] = ACTIONS(1591),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1591),
    [sym_super] = ACTIONS(1591),
    [sym_crate] = ACTIONS(1591),
    [sym_metavariable] = ACTIONS(1593),
    [sym__raw_string_literal_start] = ACTIONS(1593),
    [sym_float_literal] = ACTIONS(1593),
  },
  [439] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2501),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(2502),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_type_binding] = STATE(2721),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_label] = STATE(3611),
    [sym_block] = STATE(2721),
    [sym__literal] = STATE(2721),
    [sym_string_literal] = STATE(2814),
    [sym_raw_string_literal] = STATE(2814),
    [sym_boolean_literal] = STATE(2814),
    [sym_line_comment] = STATE(439),
    [sym_block_comment] = STATE(439),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1595),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_LBRACE] = ACTIONS(1601),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_GT] = ACTIONS(1607),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(1611),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_integer_literal] = ACTIONS(1617),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1617),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1617),
  },
  [440] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2501),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(2502),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_type_binding] = STATE(2721),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_label] = STATE(3611),
    [sym_block] = STATE(2721),
    [sym__literal] = STATE(2721),
    [sym_string_literal] = STATE(2814),
    [sym_raw_string_literal] = STATE(2814),
    [sym_boolean_literal] = STATE(2814),
    [sym_line_comment] = STATE(440),
    [sym_block_comment] = STATE(440),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1595),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_LBRACE] = ACTIONS(1601),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_GT] = ACTIONS(1623),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(1611),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_integer_literal] = ACTIONS(1617),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1617),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1617),
  },
  [441] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2501),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(2502),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_type_binding] = STATE(2721),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_label] = STATE(3611),
    [sym_block] = STATE(2721),
    [sym__literal] = STATE(2721),
    [sym_string_literal] = STATE(2814),
    [sym_raw_string_literal] = STATE(2814),
    [sym_boolean_literal] = STATE(2814),
    [sym_line_comment] = STATE(441),
    [sym_block_comment] = STATE(441),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1595),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_LBRACE] = ACTIONS(1601),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_GT] = ACTIONS(1625),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(1611),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_integer_literal] = ACTIONS(1617),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1617),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1617),
  },
  [442] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2501),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(2502),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_type_binding] = STATE(2721),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_label] = STATE(3611),
    [sym_block] = STATE(2721),
    [sym__literal] = STATE(2721),
    [sym_string_literal] = STATE(2814),
    [sym_raw_string_literal] = STATE(2814),
    [sym_boolean_literal] = STATE(2814),
    [sym_line_comment] = STATE(442),
    [sym_block_comment] = STATE(442),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1595),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_LBRACE] = ACTIONS(1601),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_GT] = ACTIONS(1627),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(1611),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_integer_literal] = ACTIONS(1617),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1617),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1617),
  },
  [443] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2501),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(2502),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_type_binding] = STATE(2721),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_label] = STATE(3611),
    [sym_block] = STATE(2721),
    [sym__literal] = STATE(2721),
    [sym_string_literal] = STATE(2814),
    [sym_raw_string_literal] = STATE(2814),
    [sym_boolean_literal] = STATE(2814),
    [sym_line_comment] = STATE(443),
    [sym_block_comment] = STATE(443),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1595),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_LBRACE] = ACTIONS(1601),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_GT] = ACTIONS(1629),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(1611),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_integer_literal] = ACTIONS(1617),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1617),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1617),
  },
  [444] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2501),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(2502),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_type_binding] = STATE(2721),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_label] = STATE(3611),
    [sym_block] = STATE(2721),
    [sym__literal] = STATE(2721),
    [sym_string_literal] = STATE(2814),
    [sym_raw_string_literal] = STATE(2814),
    [sym_boolean_literal] = STATE(2814),
    [sym_line_comment] = STATE(444),
    [sym_block_comment] = STATE(444),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1595),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_LBRACE] = ACTIONS(1601),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_GT] = ACTIONS(1631),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(1611),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_integer_literal] = ACTIONS(1617),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1617),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1617),
  },
  [445] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2501),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(2502),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_type_binding] = STATE(2721),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_label] = STATE(3611),
    [sym_block] = STATE(2721),
    [sym__literal] = STATE(2721),
    [sym_string_literal] = STATE(2814),
    [sym_raw_string_literal] = STATE(2814),
    [sym_boolean_literal] = STATE(2814),
    [sym_line_comment] = STATE(445),
    [sym_block_comment] = STATE(445),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1595),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_LBRACE] = ACTIONS(1601),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_GT] = ACTIONS(1633),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(1611),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_integer_literal] = ACTIONS(1617),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1617),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1617),
  },
  [446] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2501),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(2502),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_type_binding] = STATE(2721),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_label] = STATE(3611),
    [sym_block] = STATE(2721),
    [sym__literal] = STATE(2721),
    [sym_string_literal] = STATE(2814),
    [sym_raw_string_literal] = STATE(2814),
    [sym_boolean_literal] = STATE(2814),
    [sym_line_comment] = STATE(446),
    [sym_block_comment] = STATE(446),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1595),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_LBRACE] = ACTIONS(1601),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_GT] = ACTIONS(1635),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(1611),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_integer_literal] = ACTIONS(1617),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1617),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1617),
  },
  [447] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2501),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(2502),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_type_binding] = STATE(2721),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_label] = STATE(3611),
    [sym_block] = STATE(2721),
    [sym__literal] = STATE(2721),
    [sym_string_literal] = STATE(2814),
    [sym_raw_string_literal] = STATE(2814),
    [sym_boolean_literal] = STATE(2814),
    [sym_line_comment] = STATE(447),
    [sym_block_comment] = STATE(447),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1595),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_LBRACE] = ACTIONS(1601),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_GT] = ACTIONS(1637),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(1611),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_integer_literal] = ACTIONS(1617),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1617),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1617),
  },
  [448] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2340),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(2341),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_type_binding] = STATE(2472),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_label] = STATE(3611),
    [sym_block] = STATE(2472),
    [sym__literal] = STATE(2472),
    [sym_string_literal] = STATE(2814),
    [sym_raw_string_literal] = STATE(2814),
    [sym_boolean_literal] = STATE(2814),
    [sym_line_comment] = STATE(448),
    [sym_block_comment] = STATE(448),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1595),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_LBRACE] = ACTIONS(1601),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(1611),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_integer_literal] = ACTIONS(1617),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1617),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1617),
  },
  [449] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2501),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(2502),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_type_binding] = STATE(2721),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_label] = STATE(3611),
    [sym_block] = STATE(2721),
    [sym__literal] = STATE(2721),
    [sym_string_literal] = STATE(2814),
    [sym_raw_string_literal] = STATE(2814),
    [sym_boolean_literal] = STATE(2814),
    [sym_line_comment] = STATE(449),
    [sym_block_comment] = STATE(449),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1595),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_LBRACE] = ACTIONS(1601),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(1611),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_integer_literal] = ACTIONS(1617),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1617),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1617),
  },
  [450] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2369),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(2370),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_type_binding] = STATE(2419),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_label] = STATE(3611),
    [sym_block] = STATE(2419),
    [sym__literal] = STATE(2419),
    [sym_string_literal] = STATE(2814),
    [sym_raw_string_literal] = STATE(2814),
    [sym_boolean_literal] = STATE(2814),
    [sym_line_comment] = STATE(450),
    [sym_block_comment] = STATE(450),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1595),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_LBRACE] = ACTIONS(1601),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(1611),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_integer_literal] = ACTIONS(1617),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1617),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1617),
  },
  [451] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2339),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(2360),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_type_binding] = STATE(2503),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_label] = STATE(3611),
    [sym_block] = STATE(2503),
    [sym__literal] = STATE(2503),
    [sym_string_literal] = STATE(2814),
    [sym_raw_string_literal] = STATE(2814),
    [sym_boolean_literal] = STATE(2814),
    [sym_line_comment] = STATE(451),
    [sym_block_comment] = STATE(451),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(1595),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_LBRACE] = ACTIONS(1601),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(1611),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_integer_literal] = ACTIONS(1617),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1617),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1617),
  },
  [452] = {
    [sym_else_clause] = STATE(481),
    [sym_line_comment] = STATE(452),
    [sym_block_comment] = STATE(452),
    [sym_identifier] = ACTIONS(1253),
    [anon_sym_LPAREN] = ACTIONS(1251),
    [anon_sym_LBRACK] = ACTIONS(1251),
    [anon_sym_RBRACE] = ACTIONS(1251),
    [anon_sym_PLUS] = ACTIONS(1253),
    [anon_sym_STAR] = ACTIONS(1253),
    [anon_sym_QMARK] = ACTIONS(1251),
    [anon_sym_u8] = ACTIONS(1253),
    [anon_sym_i8] = ACTIONS(1253),
    [anon_sym_u16] = ACTIONS(1253),
    [anon_sym_i16] = ACTIONS(1253),
    [anon_sym_u32] = ACTIONS(1253),
    [anon_sym_i32] = ACTIONS(1253),
    [anon_sym_u64] = ACTIONS(1253),
    [anon_sym_i64] = ACTIONS(1253),
    [anon_sym_u128] = ACTIONS(1253),
    [anon_sym_i128] = ACTIONS(1253),
    [anon_sym_isize] = ACTIONS(1253),
    [anon_sym_usize] = ACTIONS(1253),
    [anon_sym_f32] = ACTIONS(1253),
    [anon_sym_f64] = ACTIONS(1253),
    [anon_sym_bool] = ACTIONS(1253),
    [anon_sym_str] = ACTIONS(1253),
    [anon_sym_char] = ACTIONS(1253),
    [anon_sym_DASH] = ACTIONS(1253),
    [anon_sym_SLASH] = ACTIONS(1253),
    [anon_sym_PERCENT] = ACTIONS(1253),
    [anon_sym_CARET] = ACTIONS(1253),
    [anon_sym_AMP] = ACTIONS(1253),
    [anon_sym_PIPE] = ACTIONS(1253),
    [anon_sym_AMP_AMP] = ACTIONS(1251),
    [anon_sym_PIPE_PIPE] = ACTIONS(1251),
    [anon_sym_LT_LT] = ACTIONS(1253),
    [anon_sym_GT_GT] = ACTIONS(1253),
    [anon_sym_PLUS_EQ] = ACTIONS(1251),
    [anon_sym_DASH_EQ] = ACTIONS(1251),
    [anon_sym_STAR_EQ] = ACTIONS(1251),
    [anon_sym_SLASH_EQ] = ACTIONS(1251),
    [anon_sym_PERCENT_EQ] = ACTIONS(1251),
    [anon_sym_CARET_EQ] = ACTIONS(1251),
    [anon_sym_AMP_EQ] = ACTIONS(1251),
    [anon_sym_PIPE_EQ] = ACTIONS(1251),
    [anon_sym_LT_LT_EQ] = ACTIONS(1251),
    [anon_sym_GT_GT_EQ] = ACTIONS(1251),
    [anon_sym_EQ] = ACTIONS(1253),
    [anon_sym_EQ_EQ] = ACTIONS(1251),
    [anon_sym_BANG_EQ] = ACTIONS(1251),
    [anon_sym_GT] = ACTIONS(1253),
    [anon_sym_LT] = ACTIONS(1253),
    [anon_sym_GT_EQ] = ACTIONS(1251),
    [anon_sym_LT_EQ] = ACTIONS(1251),
    [anon_sym__] = ACTIONS(1253),
    [anon_sym_DOT] = ACTIONS(1253),
    [anon_sym_DOT_DOT] = ACTIONS(1253),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1251),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1251),
    [anon_sym_COMMA] = ACTIONS(1251),
    [anon_sym_COLON_COLON] = ACTIONS(1251),
    [anon_sym_POUND] = ACTIONS(1251),
    [anon_sym_as] = ACTIONS(1253),
    [anon_sym_const] = ACTIONS(1253),
    [anon_sym_default] = ACTIONS(1253),
    [anon_sym_gen] = ACTIONS(1253),
    [anon_sym_union] = ACTIONS(1253),
    [anon_sym_ref] = ACTIONS(1253),
    [anon_sym_else] = ACTIONS(1639),
    [sym_mutable_specifier] = ACTIONS(1253),
    [sym_integer_literal] = ACTIONS(1251),
    [aux_sym_string_literal_token1] = ACTIONS(1251),
    [sym_char_literal] = ACTIONS(1251),
    [anon_sym_true] = ACTIONS(1253),
    [anon_sym_false] = ACTIONS(1253),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1253),
    [sym_super] = ACTIONS(1253),
    [sym_crate] = ACTIONS(1253),
    [sym_metavariable] = ACTIONS(1251),
    [sym__raw_string_literal_start] = ACTIONS(1251),
    [sym_float_literal] = ACTIONS(1251),
  },
  [453] = {
    [sym_line_comment] = STATE(453),
    [sym_block_comment] = STATE(453),
    [sym_identifier] = ACTIONS(1271),
    [anon_sym_LPAREN] = ACTIONS(1269),
    [anon_sym_LBRACK] = ACTIONS(1269),
    [anon_sym_RBRACE] = ACTIONS(1269),
    [anon_sym_PLUS] = ACTIONS(1271),
    [anon_sym_STAR] = ACTIONS(1271),
    [anon_sym_QMARK] = ACTIONS(1269),
    [anon_sym_u8] = ACTIONS(1271),
    [anon_sym_i8] = ACTIONS(1271),
    [anon_sym_u16] = ACTIONS(1271),
    [anon_sym_i16] = ACTIONS(1271),
    [anon_sym_u32] = ACTIONS(1271),
    [anon_sym_i32] = ACTIONS(1271),
    [anon_sym_u64] = ACTIONS(1271),
    [anon_sym_i64] = ACTIONS(1271),
    [anon_sym_u128] = ACTIONS(1271),
    [anon_sym_i128] = ACTIONS(1271),
    [anon_sym_isize] = ACTIONS(1271),
    [anon_sym_usize] = ACTIONS(1271),
    [anon_sym_f32] = ACTIONS(1271),
    [anon_sym_f64] = ACTIONS(1271),
    [anon_sym_bool] = ACTIONS(1271),
    [anon_sym_str] = ACTIONS(1271),
    [anon_sym_char] = ACTIONS(1271),
    [anon_sym_DASH] = ACTIONS(1271),
    [anon_sym_SLASH] = ACTIONS(1271),
    [anon_sym_PERCENT] = ACTIONS(1271),
    [anon_sym_CARET] = ACTIONS(1271),
    [anon_sym_AMP] = ACTIONS(1271),
    [anon_sym_PIPE] = ACTIONS(1271),
    [anon_sym_AMP_AMP] = ACTIONS(1269),
    [anon_sym_PIPE_PIPE] = ACTIONS(1269),
    [anon_sym_LT_LT] = ACTIONS(1271),
    [anon_sym_GT_GT] = ACTIONS(1271),
    [anon_sym_PLUS_EQ] = ACTIONS(1269),
    [anon_sym_DASH_EQ] = ACTIONS(1269),
    [anon_sym_STAR_EQ] = ACTIONS(1269),
    [anon_sym_SLASH_EQ] = ACTIONS(1269),
    [anon_sym_PERCENT_EQ] = ACTIONS(1269),
    [anon_sym_CARET_EQ] = ACTIONS(1269),
    [anon_sym_AMP_EQ] = ACTIONS(1269),
    [anon_sym_PIPE_EQ] = ACTIONS(1269),
    [anon_sym_LT_LT_EQ] = ACTIONS(1269),
    [anon_sym_GT_GT_EQ] = ACTIONS(1269),
    [anon_sym_EQ] = ACTIONS(1271),
    [anon_sym_EQ_EQ] = ACTIONS(1269),
    [anon_sym_BANG_EQ] = ACTIONS(1269),
    [anon_sym_GT] = ACTIONS(1271),
    [anon_sym_LT] = ACTIONS(1271),
    [anon_sym_GT_EQ] = ACTIONS(1269),
    [anon_sym_LT_EQ] = ACTIONS(1269),
    [anon_sym__] = ACTIONS(1271),
    [anon_sym_DOT] = ACTIONS(1271),
    [anon_sym_DOT_DOT] = ACTIONS(1271),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1269),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1269),
    [anon_sym_COMMA] = ACTIONS(1269),
    [anon_sym_COLON_COLON] = ACTIONS(1269),
    [anon_sym_POUND] = ACTIONS(1269),
    [anon_sym_as] = ACTIONS(1271),
    [anon_sym_const] = ACTIONS(1271),
    [anon_sym_default] = ACTIONS(1271),
    [anon_sym_gen] = ACTIONS(1271),
    [anon_sym_union] = ACTIONS(1271),
    [anon_sym_ref] = ACTIONS(1271),
    [anon_sym_else] = ACTIONS(1271),
    [sym_mutable_specifier] = ACTIONS(1271),
    [sym_integer_literal] = ACTIONS(1269),
    [aux_sym_string_literal_token1] = ACTIONS(1269),
    [sym_char_literal] = ACTIONS(1269),
    [anon_sym_true] = ACTIONS(1271),
    [anon_sym_false] = ACTIONS(1271),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1271),
    [sym_super] = ACTIONS(1271),
    [sym_crate] = ACTIONS(1271),
    [sym_metavariable] = ACTIONS(1269),
    [sym__raw_string_literal_start] = ACTIONS(1269),
    [sym_float_literal] = ACTIONS(1269),
  },
  [454] = {
    [sym_line_comment] = STATE(454),
    [sym_block_comment] = STATE(454),
    [sym_identifier] = ACTIONS(1259),
    [anon_sym_LPAREN] = ACTIONS(1257),
    [anon_sym_LBRACK] = ACTIONS(1257),
    [anon_sym_RBRACE] = ACTIONS(1257),
    [anon_sym_PLUS] = ACTIONS(1259),
    [anon_sym_STAR] = ACTIONS(1259),
    [anon_sym_QMARK] = ACTIONS(1257),
    [anon_sym_u8] = ACTIONS(1259),
    [anon_sym_i8] = ACTIONS(1259),
    [anon_sym_u16] = ACTIONS(1259),
    [anon_sym_i16] = ACTIONS(1259),
    [anon_sym_u32] = ACTIONS(1259),
    [anon_sym_i32] = ACTIONS(1259),
    [anon_sym_u64] = ACTIONS(1259),
    [anon_sym_i64] = ACTIONS(1259),
    [anon_sym_u128] = ACTIONS(1259),
    [anon_sym_i128] = ACTIONS(1259),
    [anon_sym_isize] = ACTIONS(1259),
    [anon_sym_usize] = ACTIONS(1259),
    [anon_sym_f32] = ACTIONS(1259),
    [anon_sym_f64] = ACTIONS(1259),
    [anon_sym_bool] = ACTIONS(1259),
    [anon_sym_str] = ACTIONS(1259),
    [anon_sym_char] = ACTIONS(1259),
    [anon_sym_DASH] = ACTIONS(1259),
    [anon_sym_SLASH] = ACTIONS(1259),
    [anon_sym_PERCENT] = ACTIONS(1259),
    [anon_sym_CARET] = ACTIONS(1259),
    [anon_sym_AMP] = ACTIONS(1259),
    [anon_sym_PIPE] = ACTIONS(1259),
    [anon_sym_AMP_AMP] = ACTIONS(1257),
    [anon_sym_PIPE_PIPE] = ACTIONS(1257),
    [anon_sym_LT_LT] = ACTIONS(1259),
    [anon_sym_GT_GT] = ACTIONS(1259),
    [anon_sym_PLUS_EQ] = ACTIONS(1257),
    [anon_sym_DASH_EQ] = ACTIONS(1257),
    [anon_sym_STAR_EQ] = ACTIONS(1257),
    [anon_sym_SLASH_EQ] = ACTIONS(1257),
    [anon_sym_PERCENT_EQ] = ACTIONS(1257),
    [anon_sym_CARET_EQ] = ACTIONS(1257),
    [anon_sym_AMP_EQ] = ACTIONS(1257),
    [anon_sym_PIPE_EQ] = ACTIONS(1257),
    [anon_sym_LT_LT_EQ] = ACTIONS(1257),
    [anon_sym_GT_GT_EQ] = ACTIONS(1257),
    [anon_sym_EQ] = ACTIONS(1259),
    [anon_sym_EQ_EQ] = ACTIONS(1257),
    [anon_sym_BANG_EQ] = ACTIONS(1257),
    [anon_sym_GT] = ACTIONS(1259),
    [anon_sym_LT] = ACTIONS(1259),
    [anon_sym_GT_EQ] = ACTIONS(1257),
    [anon_sym_LT_EQ] = ACTIONS(1257),
    [anon_sym__] = ACTIONS(1259),
    [anon_sym_DOT] = ACTIONS(1259),
    [anon_sym_DOT_DOT] = ACTIONS(1259),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1257),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1257),
    [anon_sym_COMMA] = ACTIONS(1257),
    [anon_sym_COLON_COLON] = ACTIONS(1257),
    [anon_sym_POUND] = ACTIONS(1257),
    [anon_sym_as] = ACTIONS(1259),
    [anon_sym_const] = ACTIONS(1259),
    [anon_sym_default] = ACTIONS(1259),
    [anon_sym_gen] = ACTIONS(1259),
    [anon_sym_union] = ACTIONS(1259),
    [anon_sym_ref] = ACTIONS(1259),
    [anon_sym_else] = ACTIONS(1259),
    [sym_mutable_specifier] = ACTIONS(1259),
    [sym_integer_literal] = ACTIONS(1257),
    [aux_sym_string_literal_token1] = ACTIONS(1257),
    [sym_char_literal] = ACTIONS(1257),
    [anon_sym_true] = ACTIONS(1259),
    [anon_sym_false] = ACTIONS(1259),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1259),
    [sym_super] = ACTIONS(1259),
    [sym_crate] = ACTIONS(1259),
    [sym_metavariable] = ACTIONS(1257),
    [sym__raw_string_literal_start] = ACTIONS(1257),
    [sym_float_literal] = ACTIONS(1257),
  },
  [455] = {
    [sym_line_comment] = STATE(455),
    [sym_block_comment] = STATE(455),
    [sym_identifier] = ACTIONS(1263),
    [anon_sym_LPAREN] = ACTIONS(1261),
    [anon_sym_LBRACK] = ACTIONS(1261),
    [anon_sym_RBRACE] = ACTIONS(1261),
    [anon_sym_PLUS] = ACTIONS(1263),
    [anon_sym_STAR] = ACTIONS(1263),
    [anon_sym_QMARK] = ACTIONS(1261),
    [anon_sym_u8] = ACTIONS(1263),
    [anon_sym_i8] = ACTIONS(1263),
    [anon_sym_u16] = ACTIONS(1263),
    [anon_sym_i16] = ACTIONS(1263),
    [anon_sym_u32] = ACTIONS(1263),
    [anon_sym_i32] = ACTIONS(1263),
    [anon_sym_u64] = ACTIONS(1263),
    [anon_sym_i64] = ACTIONS(1263),
    [anon_sym_u128] = ACTIONS(1263),
    [anon_sym_i128] = ACTIONS(1263),
    [anon_sym_isize] = ACTIONS(1263),
    [anon_sym_usize] = ACTIONS(1263),
    [anon_sym_f32] = ACTIONS(1263),
    [anon_sym_f64] = ACTIONS(1263),
    [anon_sym_bool] = ACTIONS(1263),
    [anon_sym_str] = ACTIONS(1263),
    [anon_sym_char] = ACTIONS(1263),
    [anon_sym_DASH] = ACTIONS(1263),
    [anon_sym_SLASH] = ACTIONS(1263),
    [anon_sym_PERCENT] = ACTIONS(1263),
    [anon_sym_CARET] = ACTIONS(1263),
    [anon_sym_AMP] = ACTIONS(1263),
    [anon_sym_PIPE] = ACTIONS(1263),
    [anon_sym_AMP_AMP] = ACTIONS(1261),
    [anon_sym_PIPE_PIPE] = ACTIONS(1261),
    [anon_sym_LT_LT] = ACTIONS(1263),
    [anon_sym_GT_GT] = ACTIONS(1263),
    [anon_sym_PLUS_EQ] = ACTIONS(1261),
    [anon_sym_DASH_EQ] = ACTIONS(1261),
    [anon_sym_STAR_EQ] = ACTIONS(1261),
    [anon_sym_SLASH_EQ] = ACTIONS(1261),
    [anon_sym_PERCENT_EQ] = ACTIONS(1261),
    [anon_sym_CARET_EQ] = ACTIONS(1261),
    [anon_sym_AMP_EQ] = ACTIONS(1261),
    [anon_sym_PIPE_EQ] = ACTIONS(1261),
    [anon_sym_LT_LT_EQ] = ACTIONS(1261),
    [anon_sym_GT_GT_EQ] = ACTIONS(1261),
    [anon_sym_EQ] = ACTIONS(1263),
    [anon_sym_EQ_EQ] = ACTIONS(1261),
    [anon_sym_BANG_EQ] = ACTIONS(1261),
    [anon_sym_GT] = ACTIONS(1263),
    [anon_sym_LT] = ACTIONS(1263),
    [anon_sym_GT_EQ] = ACTIONS(1261),
    [anon_sym_LT_EQ] = ACTIONS(1261),
    [anon_sym__] = ACTIONS(1263),
    [anon_sym_DOT] = ACTIONS(1263),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1261),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1261),
    [anon_sym_COMMA] = ACTIONS(1261),
    [anon_sym_COLON_COLON] = ACTIONS(1261),
    [anon_sym_POUND] = ACTIONS(1261),
    [anon_sym_as] = ACTIONS(1263),
    [anon_sym_const] = ACTIONS(1263),
    [anon_sym_default] = ACTIONS(1263),
    [anon_sym_gen] = ACTIONS(1263),
    [anon_sym_union] = ACTIONS(1263),
    [anon_sym_ref] = ACTIONS(1263),
    [anon_sym_else] = ACTIONS(1263),
    [sym_mutable_specifier] = ACTIONS(1263),
    [sym_integer_literal] = ACTIONS(1261),
    [aux_sym_string_literal_token1] = ACTIONS(1261),
    [sym_char_literal] = ACTIONS(1261),
    [anon_sym_true] = ACTIONS(1263),
    [anon_sym_false] = ACTIONS(1263),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1263),
    [sym_super] = ACTIONS(1263),
    [sym_crate] = ACTIONS(1263),
    [sym_metavariable] = ACTIONS(1261),
    [sym__raw_string_literal_start] = ACTIONS(1261),
    [sym_float_literal] = ACTIONS(1261),
  },
  [456] = {
    [sym_line_comment] = STATE(456),
    [sym_block_comment] = STATE(456),
    [sym_identifier] = ACTIONS(1275),
    [anon_sym_LPAREN] = ACTIONS(1273),
    [anon_sym_LBRACK] = ACTIONS(1273),
    [anon_sym_RBRACE] = ACTIONS(1273),
    [anon_sym_PLUS] = ACTIONS(1275),
    [anon_sym_STAR] = ACTIONS(1275),
    [anon_sym_QMARK] = ACTIONS(1273),
    [anon_sym_u8] = ACTIONS(1275),
    [anon_sym_i8] = ACTIONS(1275),
    [anon_sym_u16] = ACTIONS(1275),
    [anon_sym_i16] = ACTIONS(1275),
    [anon_sym_u32] = ACTIONS(1275),
    [anon_sym_i32] = ACTIONS(1275),
    [anon_sym_u64] = ACTIONS(1275),
    [anon_sym_i64] = ACTIONS(1275),
    [anon_sym_u128] = ACTIONS(1275),
    [anon_sym_i128] = ACTIONS(1275),
    [anon_sym_isize] = ACTIONS(1275),
    [anon_sym_usize] = ACTIONS(1275),
    [anon_sym_f32] = ACTIONS(1275),
    [anon_sym_f64] = ACTIONS(1275),
    [anon_sym_bool] = ACTIONS(1275),
    [anon_sym_str] = ACTIONS(1275),
    [anon_sym_char] = ACTIONS(1275),
    [anon_sym_DASH] = ACTIONS(1275),
    [anon_sym_SLASH] = ACTIONS(1275),
    [anon_sym_PERCENT] = ACTIONS(1275),
    [anon_sym_CARET] = ACTIONS(1275),
    [anon_sym_AMP] = ACTIONS(1275),
    [anon_sym_PIPE] = ACTIONS(1275),
    [anon_sym_AMP_AMP] = ACTIONS(1273),
    [anon_sym_PIPE_PIPE] = ACTIONS(1273),
    [anon_sym_LT_LT] = ACTIONS(1275),
    [anon_sym_GT_GT] = ACTIONS(1275),
    [anon_sym_PLUS_EQ] = ACTIONS(1273),
    [anon_sym_DASH_EQ] = ACTIONS(1273),
    [anon_sym_STAR_EQ] = ACTIONS(1273),
    [anon_sym_SLASH_EQ] = ACTIONS(1273),
    [anon_sym_PERCENT_EQ] = ACTIONS(1273),
    [anon_sym_CARET_EQ] = ACTIONS(1273),
    [anon_sym_AMP_EQ] = ACTIONS(1273),
    [anon_sym_PIPE_EQ] = ACTIONS(1273),
    [anon_sym_LT_LT_EQ] = ACTIONS(1273),
    [anon_sym_GT_GT_EQ] = ACTIONS(1273),
    [anon_sym_EQ] = ACTIONS(1275),
    [anon_sym_EQ_EQ] = ACTIONS(1273),
    [anon_sym_BANG_EQ] = ACTIONS(1273),
    [anon_sym_GT] = ACTIONS(1275),
    [anon_sym_LT] = ACTIONS(1275),
    [anon_sym_GT_EQ] = ACTIONS(1273),
    [anon_sym_LT_EQ] = ACTIONS(1273),
    [anon_sym__] = ACTIONS(1275),
    [anon_sym_DOT] = ACTIONS(1275),
    [anon_sym_DOT_DOT] = ACTIONS(1275),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1273),
    [anon_sym_COMMA] = ACTIONS(1273),
    [anon_sym_COLON_COLON] = ACTIONS(1273),
    [anon_sym_POUND] = ACTIONS(1273),
    [anon_sym_as] = ACTIONS(1275),
    [anon_sym_const] = ACTIONS(1275),
    [anon_sym_default] = ACTIONS(1275),
    [anon_sym_gen] = ACTIONS(1275),
    [anon_sym_union] = ACTIONS(1275),
    [anon_sym_ref] = ACTIONS(1275),
    [anon_sym_else] = ACTIONS(1275),
    [sym_mutable_specifier] = ACTIONS(1275),
    [sym_integer_literal] = ACTIONS(1273),
    [aux_sym_string_literal_token1] = ACTIONS(1273),
    [sym_char_literal] = ACTIONS(1273),
    [anon_sym_true] = ACTIONS(1275),
    [anon_sym_false] = ACTIONS(1275),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1275),
    [sym_super] = ACTIONS(1275),
    [sym_crate] = ACTIONS(1275),
    [sym_metavariable] = ACTIONS(1273),
    [sym__raw_string_literal_start] = ACTIONS(1273),
    [sym_float_literal] = ACTIONS(1273),
  },
  [457] = {
    [sym_line_comment] = STATE(457),
    [sym_block_comment] = STATE(457),
    [sym_identifier] = ACTIONS(1267),
    [anon_sym_LPAREN] = ACTIONS(1265),
    [anon_sym_LBRACK] = ACTIONS(1265),
    [anon_sym_RBRACE] = ACTIONS(1265),
    [anon_sym_PLUS] = ACTIONS(1267),
    [anon_sym_STAR] = ACTIONS(1267),
    [anon_sym_QMARK] = ACTIONS(1265),
    [anon_sym_u8] = ACTIONS(1267),
    [anon_sym_i8] = ACTIONS(1267),
    [anon_sym_u16] = ACTIONS(1267),
    [anon_sym_i16] = ACTIONS(1267),
    [anon_sym_u32] = ACTIONS(1267),
    [anon_sym_i32] = ACTIONS(1267),
    [anon_sym_u64] = ACTIONS(1267),
    [anon_sym_i64] = ACTIONS(1267),
    [anon_sym_u128] = ACTIONS(1267),
    [anon_sym_i128] = ACTIONS(1267),
    [anon_sym_isize] = ACTIONS(1267),
    [anon_sym_usize] = ACTIONS(1267),
    [anon_sym_f32] = ACTIONS(1267),
    [anon_sym_f64] = ACTIONS(1267),
    [anon_sym_bool] = ACTIONS(1267),
    [anon_sym_str] = ACTIONS(1267),
    [anon_sym_char] = ACTIONS(1267),
    [anon_sym_DASH] = ACTIONS(1267),
    [anon_sym_SLASH] = ACTIONS(1267),
    [anon_sym_PERCENT] = ACTIONS(1267),
    [anon_sym_CARET] = ACTIONS(1267),
    [anon_sym_AMP] = ACTIONS(1267),
    [anon_sym_PIPE] = ACTIONS(1267),
    [anon_sym_AMP_AMP] = ACTIONS(1265),
    [anon_sym_PIPE_PIPE] = ACTIONS(1265),
    [anon_sym_LT_LT] = ACTIONS(1267),
    [anon_sym_GT_GT] = ACTIONS(1267),
    [anon_sym_PLUS_EQ] = ACTIONS(1265),
    [anon_sym_DASH_EQ] = ACTIONS(1265),
    [anon_sym_STAR_EQ] = ACTIONS(1265),
    [anon_sym_SLASH_EQ] = ACTIONS(1265),
    [anon_sym_PERCENT_EQ] = ACTIONS(1265),
    [anon_sym_CARET_EQ] = ACTIONS(1265),
    [anon_sym_AMP_EQ] = ACTIONS(1265),
    [anon_sym_PIPE_EQ] = ACTIONS(1265),
    [anon_sym_LT_LT_EQ] = ACTIONS(1265),
    [anon_sym_GT_GT_EQ] = ACTIONS(1265),
    [anon_sym_EQ] = ACTIONS(1267),
    [anon_sym_EQ_EQ] = ACTIONS(1265),
    [anon_sym_BANG_EQ] = ACTIONS(1265),
    [anon_sym_GT] = ACTIONS(1267),
    [anon_sym_LT] = ACTIONS(1267),
    [anon_sym_GT_EQ] = ACTIONS(1265),
    [anon_sym_LT_EQ] = ACTIONS(1265),
    [anon_sym__] = ACTIONS(1267),
    [anon_sym_DOT] = ACTIONS(1267),
    [anon_sym_DOT_DOT] = ACTIONS(1267),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1265),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1265),
    [anon_sym_COMMA] = ACTIONS(1265),
    [anon_sym_COLON_COLON] = ACTIONS(1265),
    [anon_sym_POUND] = ACTIONS(1265),
    [anon_sym_as] = ACTIONS(1267),
    [anon_sym_const] = ACTIONS(1267),
    [anon_sym_default] = ACTIONS(1267),
    [anon_sym_gen] = ACTIONS(1267),
    [anon_sym_union] = ACTIONS(1267),
    [anon_sym_ref] = ACTIONS(1267),
    [anon_sym_else] = ACTIONS(1267),
    [sym_mutable_specifier] = ACTIONS(1267),
    [sym_integer_literal] = ACTIONS(1265),
    [aux_sym_string_literal_token1] = ACTIONS(1265),
    [sym_char_literal] = ACTIONS(1265),
    [anon_sym_true] = ACTIONS(1267),
    [anon_sym_false] = ACTIONS(1267),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1267),
    [sym_super] = ACTIONS(1267),
    [sym_crate] = ACTIONS(1267),
    [sym_metavariable] = ACTIONS(1265),
    [sym__raw_string_literal_start] = ACTIONS(1265),
    [sym_float_literal] = ACTIONS(1265),
  },
  [458] = {
    [sym_line_comment] = STATE(458),
    [sym_block_comment] = STATE(458),
    [sym_identifier] = ACTIONS(1397),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_LBRACK] = ACTIONS(1395),
    [anon_sym_RBRACE] = ACTIONS(1395),
    [anon_sym_PLUS] = ACTIONS(1397),
    [anon_sym_STAR] = ACTIONS(1397),
    [anon_sym_QMARK] = ACTIONS(1395),
    [anon_sym_u8] = ACTIONS(1397),
    [anon_sym_i8] = ACTIONS(1397),
    [anon_sym_u16] = ACTIONS(1397),
    [anon_sym_i16] = ACTIONS(1397),
    [anon_sym_u32] = ACTIONS(1397),
    [anon_sym_i32] = ACTIONS(1397),
    [anon_sym_u64] = ACTIONS(1397),
    [anon_sym_i64] = ACTIONS(1397),
    [anon_sym_u128] = ACTIONS(1397),
    [anon_sym_i128] = ACTIONS(1397),
    [anon_sym_isize] = ACTIONS(1397),
    [anon_sym_usize] = ACTIONS(1397),
    [anon_sym_f32] = ACTIONS(1397),
    [anon_sym_f64] = ACTIONS(1397),
    [anon_sym_bool] = ACTIONS(1397),
    [anon_sym_str] = ACTIONS(1397),
    [anon_sym_char] = ACTIONS(1397),
    [anon_sym_DASH] = ACTIONS(1397),
    [anon_sym_SLASH] = ACTIONS(1397),
    [anon_sym_PERCENT] = ACTIONS(1397),
    [anon_sym_CARET] = ACTIONS(1397),
    [anon_sym_AMP] = ACTIONS(1397),
    [anon_sym_PIPE] = ACTIONS(1397),
    [anon_sym_AMP_AMP] = ACTIONS(1395),
    [anon_sym_PIPE_PIPE] = ACTIONS(1395),
    [anon_sym_LT_LT] = ACTIONS(1397),
    [anon_sym_GT_GT] = ACTIONS(1397),
    [anon_sym_PLUS_EQ] = ACTIONS(1395),
    [anon_sym_DASH_EQ] = ACTIONS(1395),
    [anon_sym_STAR_EQ] = ACTIONS(1395),
    [anon_sym_SLASH_EQ] = ACTIONS(1395),
    [anon_sym_PERCENT_EQ] = ACTIONS(1395),
    [anon_sym_CARET_EQ] = ACTIONS(1395),
    [anon_sym_AMP_EQ] = ACTIONS(1395),
    [anon_sym_PIPE_EQ] = ACTIONS(1395),
    [anon_sym_LT_LT_EQ] = ACTIONS(1395),
    [anon_sym_GT_GT_EQ] = ACTIONS(1395),
    [anon_sym_EQ] = ACTIONS(1397),
    [anon_sym_EQ_EQ] = ACTIONS(1395),
    [anon_sym_BANG_EQ] = ACTIONS(1395),
    [anon_sym_GT] = ACTIONS(1397),
    [anon_sym_LT] = ACTIONS(1397),
    [anon_sym_GT_EQ] = ACTIONS(1395),
    [anon_sym_LT_EQ] = ACTIONS(1395),
    [anon_sym__] = ACTIONS(1397),
    [anon_sym_DOT] = ACTIONS(1397),
    [anon_sym_DOT_DOT] = ACTIONS(1397),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1395),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1395),
    [anon_sym_COMMA] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1395),
    [anon_sym_POUND] = ACTIONS(1395),
    [anon_sym_as] = ACTIONS(1397),
    [anon_sym_const] = ACTIONS(1397),
    [anon_sym_default] = ACTIONS(1397),
    [anon_sym_gen] = ACTIONS(1397),
    [anon_sym_union] = ACTIONS(1397),
    [anon_sym_ref] = ACTIONS(1397),
    [sym_mutable_specifier] = ACTIONS(1397),
    [sym_integer_literal] = ACTIONS(1395),
    [aux_sym_string_literal_token1] = ACTIONS(1395),
    [sym_char_literal] = ACTIONS(1395),
    [anon_sym_true] = ACTIONS(1397),
    [anon_sym_false] = ACTIONS(1397),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1397),
    [sym_super] = ACTIONS(1397),
    [sym_crate] = ACTIONS(1397),
    [sym_metavariable] = ACTIONS(1395),
    [sym__raw_string_literal_start] = ACTIONS(1395),
    [sym_float_literal] = ACTIONS(1395),
  },
  [459] = {
    [sym_line_comment] = STATE(459),
    [sym_block_comment] = STATE(459),
    [sym_identifier] = ACTIONS(1485),
    [anon_sym_LPAREN] = ACTIONS(1483),
    [anon_sym_LBRACK] = ACTIONS(1483),
    [anon_sym_RBRACE] = ACTIONS(1483),
    [anon_sym_PLUS] = ACTIONS(1485),
    [anon_sym_STAR] = ACTIONS(1485),
    [anon_sym_QMARK] = ACTIONS(1483),
    [anon_sym_u8] = ACTIONS(1485),
    [anon_sym_i8] = ACTIONS(1485),
    [anon_sym_u16] = ACTIONS(1485),
    [anon_sym_i16] = ACTIONS(1485),
    [anon_sym_u32] = ACTIONS(1485),
    [anon_sym_i32] = ACTIONS(1485),
    [anon_sym_u64] = ACTIONS(1485),
    [anon_sym_i64] = ACTIONS(1485),
    [anon_sym_u128] = ACTIONS(1485),
    [anon_sym_i128] = ACTIONS(1485),
    [anon_sym_isize] = ACTIONS(1485),
    [anon_sym_usize] = ACTIONS(1485),
    [anon_sym_f32] = ACTIONS(1485),
    [anon_sym_f64] = ACTIONS(1485),
    [anon_sym_bool] = ACTIONS(1485),
    [anon_sym_str] = ACTIONS(1485),
    [anon_sym_char] = ACTIONS(1485),
    [anon_sym_DASH] = ACTIONS(1485),
    [anon_sym_SLASH] = ACTIONS(1485),
    [anon_sym_PERCENT] = ACTIONS(1485),
    [anon_sym_CARET] = ACTIONS(1485),
    [anon_sym_AMP] = ACTIONS(1485),
    [anon_sym_PIPE] = ACTIONS(1485),
    [anon_sym_AMP_AMP] = ACTIONS(1483),
    [anon_sym_PIPE_PIPE] = ACTIONS(1483),
    [anon_sym_LT_LT] = ACTIONS(1485),
    [anon_sym_GT_GT] = ACTIONS(1485),
    [anon_sym_PLUS_EQ] = ACTIONS(1483),
    [anon_sym_DASH_EQ] = ACTIONS(1483),
    [anon_sym_STAR_EQ] = ACTIONS(1483),
    [anon_sym_SLASH_EQ] = ACTIONS(1483),
    [anon_sym_PERCENT_EQ] = ACTIONS(1483),
    [anon_sym_CARET_EQ] = ACTIONS(1483),
    [anon_sym_AMP_EQ] = ACTIONS(1483),
    [anon_sym_PIPE_EQ] = ACTIONS(1483),
    [anon_sym_LT_LT_EQ] = ACTIONS(1483),
    [anon_sym_GT_GT_EQ] = ACTIONS(1483),
    [anon_sym_EQ] = ACTIONS(1485),
    [anon_sym_EQ_EQ] = ACTIONS(1483),
    [anon_sym_BANG_EQ] = ACTIONS(1483),
    [anon_sym_GT] = ACTIONS(1485),
    [anon_sym_LT] = ACTIONS(1485),
    [anon_sym_GT_EQ] = ACTIONS(1483),
    [anon_sym_LT_EQ] = ACTIONS(1483),
    [anon_sym__] = ACTIONS(1485),
    [anon_sym_DOT] = ACTIONS(1485),
    [anon_sym_DOT_DOT] = ACTIONS(1485),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1483),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1483),
    [anon_sym_COMMA] = ACTIONS(1483),
    [anon_sym_COLON_COLON] = ACTIONS(1483),
    [anon_sym_POUND] = ACTIONS(1483),
    [anon_sym_as] = ACTIONS(1485),
    [anon_sym_const] = ACTIONS(1485),
    [anon_sym_default] = ACTIONS(1485),
    [anon_sym_gen] = ACTIONS(1485),
    [anon_sym_union] = ACTIONS(1485),
    [anon_sym_ref] = ACTIONS(1485),
    [sym_mutable_specifier] = ACTIONS(1485),
    [sym_integer_literal] = ACTIONS(1483),
    [aux_sym_string_literal_token1] = ACTIONS(1483),
    [sym_char_literal] = ACTIONS(1483),
    [anon_sym_true] = ACTIONS(1485),
    [anon_sym_false] = ACTIONS(1485),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1485),
    [sym_super] = ACTIONS(1485),
    [sym_crate] = ACTIONS(1485),
    [sym_metavariable] = ACTIONS(1483),
    [sym__raw_string_literal_start] = ACTIONS(1483),
    [sym_float_literal] = ACTIONS(1483),
  },
  [460] = {
    [sym_line_comment] = STATE(460),
    [sym_block_comment] = STATE(460),
    [sym_identifier] = ACTIONS(1461),
    [anon_sym_LPAREN] = ACTIONS(1459),
    [anon_sym_LBRACK] = ACTIONS(1459),
    [anon_sym_RBRACE] = ACTIONS(1459),
    [anon_sym_PLUS] = ACTIONS(1461),
    [anon_sym_STAR] = ACTIONS(1461),
    [anon_sym_QMARK] = ACTIONS(1459),
    [anon_sym_u8] = ACTIONS(1461),
    [anon_sym_i8] = ACTIONS(1461),
    [anon_sym_u16] = ACTIONS(1461),
    [anon_sym_i16] = ACTIONS(1461),
    [anon_sym_u32] = ACTIONS(1461),
    [anon_sym_i32] = ACTIONS(1461),
    [anon_sym_u64] = ACTIONS(1461),
    [anon_sym_i64] = ACTIONS(1461),
    [anon_sym_u128] = ACTIONS(1461),
    [anon_sym_i128] = ACTIONS(1461),
    [anon_sym_isize] = ACTIONS(1461),
    [anon_sym_usize] = ACTIONS(1461),
    [anon_sym_f32] = ACTIONS(1461),
    [anon_sym_f64] = ACTIONS(1461),
    [anon_sym_bool] = ACTIONS(1461),
    [anon_sym_str] = ACTIONS(1461),
    [anon_sym_char] = ACTIONS(1461),
    [anon_sym_DASH] = ACTIONS(1461),
    [anon_sym_SLASH] = ACTIONS(1461),
    [anon_sym_PERCENT] = ACTIONS(1461),
    [anon_sym_CARET] = ACTIONS(1461),
    [anon_sym_AMP] = ACTIONS(1461),
    [anon_sym_PIPE] = ACTIONS(1461),
    [anon_sym_AMP_AMP] = ACTIONS(1459),
    [anon_sym_PIPE_PIPE] = ACTIONS(1459),
    [anon_sym_LT_LT] = ACTIONS(1461),
    [anon_sym_GT_GT] = ACTIONS(1461),
    [anon_sym_PLUS_EQ] = ACTIONS(1459),
    [anon_sym_DASH_EQ] = ACTIONS(1459),
    [anon_sym_STAR_EQ] = ACTIONS(1459),
    [anon_sym_SLASH_EQ] = ACTIONS(1459),
    [anon_sym_PERCENT_EQ] = ACTIONS(1459),
    [anon_sym_CARET_EQ] = ACTIONS(1459),
    [anon_sym_AMP_EQ] = ACTIONS(1459),
    [anon_sym_PIPE_EQ] = ACTIONS(1459),
    [anon_sym_LT_LT_EQ] = ACTIONS(1459),
    [anon_sym_GT_GT_EQ] = ACTIONS(1459),
    [anon_sym_EQ] = ACTIONS(1461),
    [anon_sym_EQ_EQ] = ACTIONS(1459),
    [anon_sym_BANG_EQ] = ACTIONS(1459),
    [anon_sym_GT] = ACTIONS(1461),
    [anon_sym_LT] = ACTIONS(1461),
    [anon_sym_GT_EQ] = ACTIONS(1459),
    [anon_sym_LT_EQ] = ACTIONS(1459),
    [anon_sym__] = ACTIONS(1461),
    [anon_sym_DOT] = ACTIONS(1461),
    [anon_sym_DOT_DOT] = ACTIONS(1461),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1459),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1459),
    [anon_sym_COMMA] = ACTIONS(1459),
    [anon_sym_COLON_COLON] = ACTIONS(1459),
    [anon_sym_POUND] = ACTIONS(1459),
    [anon_sym_as] = ACTIONS(1461),
    [anon_sym_const] = ACTIONS(1461),
    [anon_sym_default] = ACTIONS(1461),
    [anon_sym_gen] = ACTIONS(1461),
    [anon_sym_union] = ACTIONS(1461),
    [anon_sym_ref] = ACTIONS(1461),
    [sym_mutable_specifier] = ACTIONS(1461),
    [sym_integer_literal] = ACTIONS(1459),
    [aux_sym_string_literal_token1] = ACTIONS(1459),
    [sym_char_literal] = ACTIONS(1459),
    [anon_sym_true] = ACTIONS(1461),
    [anon_sym_false] = ACTIONS(1461),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1461),
    [sym_super] = ACTIONS(1461),
    [sym_crate] = ACTIONS(1461),
    [sym_metavariable] = ACTIONS(1459),
    [sym__raw_string_literal_start] = ACTIONS(1459),
    [sym_float_literal] = ACTIONS(1459),
  },
  [461] = {
    [sym_line_comment] = STATE(461),
    [sym_block_comment] = STATE(461),
    [sym_identifier] = ACTIONS(1449),
    [anon_sym_LPAREN] = ACTIONS(1447),
    [anon_sym_LBRACK] = ACTIONS(1447),
    [anon_sym_RBRACE] = ACTIONS(1447),
    [anon_sym_PLUS] = ACTIONS(1449),
    [anon_sym_STAR] = ACTIONS(1449),
    [anon_sym_QMARK] = ACTIONS(1447),
    [anon_sym_u8] = ACTIONS(1449),
    [anon_sym_i8] = ACTIONS(1449),
    [anon_sym_u16] = ACTIONS(1449),
    [anon_sym_i16] = ACTIONS(1449),
    [anon_sym_u32] = ACTIONS(1449),
    [anon_sym_i32] = ACTIONS(1449),
    [anon_sym_u64] = ACTIONS(1449),
    [anon_sym_i64] = ACTIONS(1449),
    [anon_sym_u128] = ACTIONS(1449),
    [anon_sym_i128] = ACTIONS(1449),
    [anon_sym_isize] = ACTIONS(1449),
    [anon_sym_usize] = ACTIONS(1449),
    [anon_sym_f32] = ACTIONS(1449),
    [anon_sym_f64] = ACTIONS(1449),
    [anon_sym_bool] = ACTIONS(1449),
    [anon_sym_str] = ACTIONS(1449),
    [anon_sym_char] = ACTIONS(1449),
    [anon_sym_DASH] = ACTIONS(1449),
    [anon_sym_SLASH] = ACTIONS(1449),
    [anon_sym_PERCENT] = ACTIONS(1449),
    [anon_sym_CARET] = ACTIONS(1449),
    [anon_sym_AMP] = ACTIONS(1449),
    [anon_sym_PIPE] = ACTIONS(1449),
    [anon_sym_AMP_AMP] = ACTIONS(1447),
    [anon_sym_PIPE_PIPE] = ACTIONS(1447),
    [anon_sym_LT_LT] = ACTIONS(1449),
    [anon_sym_GT_GT] = ACTIONS(1449),
    [anon_sym_PLUS_EQ] = ACTIONS(1447),
    [anon_sym_DASH_EQ] = ACTIONS(1447),
    [anon_sym_STAR_EQ] = ACTIONS(1447),
    [anon_sym_SLASH_EQ] = ACTIONS(1447),
    [anon_sym_PERCENT_EQ] = ACTIONS(1447),
    [anon_sym_CARET_EQ] = ACTIONS(1447),
    [anon_sym_AMP_EQ] = ACTIONS(1447),
    [anon_sym_PIPE_EQ] = ACTIONS(1447),
    [anon_sym_LT_LT_EQ] = ACTIONS(1447),
    [anon_sym_GT_GT_EQ] = ACTIONS(1447),
    [anon_sym_EQ] = ACTIONS(1449),
    [anon_sym_EQ_EQ] = ACTIONS(1447),
    [anon_sym_BANG_EQ] = ACTIONS(1447),
    [anon_sym_GT] = ACTIONS(1449),
    [anon_sym_LT] = ACTIONS(1449),
    [anon_sym_GT_EQ] = ACTIONS(1447),
    [anon_sym_LT_EQ] = ACTIONS(1447),
    [anon_sym__] = ACTIONS(1449),
    [anon_sym_DOT] = ACTIONS(1449),
    [anon_sym_DOT_DOT] = ACTIONS(1449),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1447),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1447),
    [anon_sym_COMMA] = ACTIONS(1447),
    [anon_sym_COLON_COLON] = ACTIONS(1447),
    [anon_sym_POUND] = ACTIONS(1447),
    [anon_sym_as] = ACTIONS(1449),
    [anon_sym_const] = ACTIONS(1449),
    [anon_sym_default] = ACTIONS(1449),
    [anon_sym_gen] = ACTIONS(1449),
    [anon_sym_union] = ACTIONS(1449),
    [anon_sym_ref] = ACTIONS(1449),
    [sym_mutable_specifier] = ACTIONS(1449),
    [sym_integer_literal] = ACTIONS(1447),
    [aux_sym_string_literal_token1] = ACTIONS(1447),
    [sym_char_literal] = ACTIONS(1447),
    [anon_sym_true] = ACTIONS(1449),
    [anon_sym_false] = ACTIONS(1449),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1449),
    [sym_super] = ACTIONS(1449),
    [sym_crate] = ACTIONS(1449),
    [sym_metavariable] = ACTIONS(1447),
    [sym__raw_string_literal_start] = ACTIONS(1447),
    [sym_float_literal] = ACTIONS(1447),
  },
  [462] = {
    [sym_line_comment] = STATE(462),
    [sym_block_comment] = STATE(462),
    [sym_identifier] = ACTIONS(1641),
    [anon_sym_LPAREN] = ACTIONS(1643),
    [anon_sym_LBRACK] = ACTIONS(1643),
    [anon_sym_RBRACE] = ACTIONS(1359),
    [anon_sym_PLUS] = ACTIONS(1357),
    [anon_sym_STAR] = ACTIONS(1357),
    [anon_sym_QMARK] = ACTIONS(1359),
    [anon_sym_u8] = ACTIONS(1641),
    [anon_sym_i8] = ACTIONS(1641),
    [anon_sym_u16] = ACTIONS(1641),
    [anon_sym_i16] = ACTIONS(1641),
    [anon_sym_u32] = ACTIONS(1641),
    [anon_sym_i32] = ACTIONS(1641),
    [anon_sym_u64] = ACTIONS(1641),
    [anon_sym_i64] = ACTIONS(1641),
    [anon_sym_u128] = ACTIONS(1641),
    [anon_sym_i128] = ACTIONS(1641),
    [anon_sym_isize] = ACTIONS(1641),
    [anon_sym_usize] = ACTIONS(1641),
    [anon_sym_f32] = ACTIONS(1641),
    [anon_sym_f64] = ACTIONS(1641),
    [anon_sym_bool] = ACTIONS(1641),
    [anon_sym_str] = ACTIONS(1641),
    [anon_sym_char] = ACTIONS(1641),
    [anon_sym_DASH] = ACTIONS(1641),
    [anon_sym_SLASH] = ACTIONS(1357),
    [anon_sym_PERCENT] = ACTIONS(1357),
    [anon_sym_CARET] = ACTIONS(1357),
    [anon_sym_AMP] = ACTIONS(1641),
    [anon_sym_PIPE] = ACTIONS(1641),
    [anon_sym_AMP_AMP] = ACTIONS(1359),
    [anon_sym_PIPE_PIPE] = ACTIONS(1359),
    [anon_sym_LT_LT] = ACTIONS(1357),
    [anon_sym_GT_GT] = ACTIONS(1357),
    [anon_sym_PLUS_EQ] = ACTIONS(1359),
    [anon_sym_DASH_EQ] = ACTIONS(1359),
    [anon_sym_STAR_EQ] = ACTIONS(1359),
    [anon_sym_SLASH_EQ] = ACTIONS(1359),
    [anon_sym_PERCENT_EQ] = ACTIONS(1359),
    [anon_sym_CARET_EQ] = ACTIONS(1359),
    [anon_sym_AMP_EQ] = ACTIONS(1359),
    [anon_sym_PIPE_EQ] = ACTIONS(1359),
    [anon_sym_LT_LT_EQ] = ACTIONS(1359),
    [anon_sym_GT_GT_EQ] = ACTIONS(1359),
    [anon_sym_EQ] = ACTIONS(1357),
    [anon_sym_EQ_EQ] = ACTIONS(1359),
    [anon_sym_BANG_EQ] = ACTIONS(1359),
    [anon_sym_GT] = ACTIONS(1357),
    [anon_sym_LT] = ACTIONS(1641),
    [anon_sym_GT_EQ] = ACTIONS(1359),
    [anon_sym_LT_EQ] = ACTIONS(1359),
    [anon_sym__] = ACTIONS(1641),
    [anon_sym_DOT] = ACTIONS(1357),
    [anon_sym_DOT_DOT] = ACTIONS(1641),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1359),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1359),
    [anon_sym_COMMA] = ACTIONS(1359),
    [anon_sym_COLON_COLON] = ACTIONS(1643),
    [anon_sym_POUND] = ACTIONS(1643),
    [anon_sym_as] = ACTIONS(1357),
    [anon_sym_const] = ACTIONS(1641),
    [anon_sym_default] = ACTIONS(1641),
    [anon_sym_gen] = ACTIONS(1641),
    [anon_sym_union] = ACTIONS(1641),
    [anon_sym_ref] = ACTIONS(1641),
    [sym_mutable_specifier] = ACTIONS(1641),
    [sym_integer_literal] = ACTIONS(1643),
    [aux_sym_string_literal_token1] = ACTIONS(1643),
    [sym_char_literal] = ACTIONS(1643),
    [anon_sym_true] = ACTIONS(1641),
    [anon_sym_false] = ACTIONS(1641),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1641),
    [sym_super] = ACTIONS(1641),
    [sym_crate] = ACTIONS(1641),
    [sym_metavariable] = ACTIONS(1643),
    [sym__raw_string_literal_start] = ACTIONS(1643),
    [sym_float_literal] = ACTIONS(1643),
  },
  [463] = {
    [sym_line_comment] = STATE(463),
    [sym_block_comment] = STATE(463),
    [sym_identifier] = ACTIONS(1453),
    [anon_sym_LPAREN] = ACTIONS(1451),
    [anon_sym_LBRACK] = ACTIONS(1451),
    [anon_sym_RBRACE] = ACTIONS(1451),
    [anon_sym_PLUS] = ACTIONS(1453),
    [anon_sym_STAR] = ACTIONS(1453),
    [anon_sym_QMARK] = ACTIONS(1451),
    [anon_sym_u8] = ACTIONS(1453),
    [anon_sym_i8] = ACTIONS(1453),
    [anon_sym_u16] = ACTIONS(1453),
    [anon_sym_i16] = ACTIONS(1453),
    [anon_sym_u32] = ACTIONS(1453),
    [anon_sym_i32] = ACTIONS(1453),
    [anon_sym_u64] = ACTIONS(1453),
    [anon_sym_i64] = ACTIONS(1453),
    [anon_sym_u128] = ACTIONS(1453),
    [anon_sym_i128] = ACTIONS(1453),
    [anon_sym_isize] = ACTIONS(1453),
    [anon_sym_usize] = ACTIONS(1453),
    [anon_sym_f32] = ACTIONS(1453),
    [anon_sym_f64] = ACTIONS(1453),
    [anon_sym_bool] = ACTIONS(1453),
    [anon_sym_str] = ACTIONS(1453),
    [anon_sym_char] = ACTIONS(1453),
    [anon_sym_DASH] = ACTIONS(1453),
    [anon_sym_SLASH] = ACTIONS(1453),
    [anon_sym_PERCENT] = ACTIONS(1453),
    [anon_sym_CARET] = ACTIONS(1453),
    [anon_sym_AMP] = ACTIONS(1453),
    [anon_sym_PIPE] = ACTIONS(1453),
    [anon_sym_AMP_AMP] = ACTIONS(1451),
    [anon_sym_PIPE_PIPE] = ACTIONS(1451),
    [anon_sym_LT_LT] = ACTIONS(1453),
    [anon_sym_GT_GT] = ACTIONS(1453),
    [anon_sym_PLUS_EQ] = ACTIONS(1451),
    [anon_sym_DASH_EQ] = ACTIONS(1451),
    [anon_sym_STAR_EQ] = ACTIONS(1451),
    [anon_sym_SLASH_EQ] = ACTIONS(1451),
    [anon_sym_PERCENT_EQ] = ACTIONS(1451),
    [anon_sym_CARET_EQ] = ACTIONS(1451),
    [anon_sym_AMP_EQ] = ACTIONS(1451),
    [anon_sym_PIPE_EQ] = ACTIONS(1451),
    [anon_sym_LT_LT_EQ] = ACTIONS(1451),
    [anon_sym_GT_GT_EQ] = ACTIONS(1451),
    [anon_sym_EQ] = ACTIONS(1453),
    [anon_sym_EQ_EQ] = ACTIONS(1451),
    [anon_sym_BANG_EQ] = ACTIONS(1451),
    [anon_sym_GT] = ACTIONS(1453),
    [anon_sym_LT] = ACTIONS(1453),
    [anon_sym_GT_EQ] = ACTIONS(1451),
    [anon_sym_LT_EQ] = ACTIONS(1451),
    [anon_sym__] = ACTIONS(1453),
    [anon_sym_DOT] = ACTIONS(1453),
    [anon_sym_DOT_DOT] = ACTIONS(1453),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1451),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1451),
    [anon_sym_COMMA] = ACTIONS(1451),
    [anon_sym_COLON_COLON] = ACTIONS(1451),
    [anon_sym_POUND] = ACTIONS(1451),
    [anon_sym_as] = ACTIONS(1453),
    [anon_sym_const] = ACTIONS(1453),
    [anon_sym_default] = ACTIONS(1453),
    [anon_sym_gen] = ACTIONS(1453),
    [anon_sym_union] = ACTIONS(1453),
    [anon_sym_ref] = ACTIONS(1453),
    [sym_mutable_specifier] = ACTIONS(1453),
    [sym_integer_literal] = ACTIONS(1451),
    [aux_sym_string_literal_token1] = ACTIONS(1451),
    [sym_char_literal] = ACTIONS(1451),
    [anon_sym_true] = ACTIONS(1453),
    [anon_sym_false] = ACTIONS(1453),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1453),
    [sym_super] = ACTIONS(1453),
    [sym_crate] = ACTIONS(1453),
    [sym_metavariable] = ACTIONS(1451),
    [sym__raw_string_literal_start] = ACTIONS(1451),
    [sym_float_literal] = ACTIONS(1451),
  },
  [464] = {
    [sym_line_comment] = STATE(464),
    [sym_block_comment] = STATE(464),
    [sym_identifier] = ACTIONS(1367),
    [anon_sym_LPAREN] = ACTIONS(1365),
    [anon_sym_LBRACK] = ACTIONS(1365),
    [anon_sym_RBRACE] = ACTIONS(1365),
    [anon_sym_PLUS] = ACTIONS(1367),
    [anon_sym_STAR] = ACTIONS(1367),
    [anon_sym_QMARK] = ACTIONS(1365),
    [anon_sym_u8] = ACTIONS(1367),
    [anon_sym_i8] = ACTIONS(1367),
    [anon_sym_u16] = ACTIONS(1367),
    [anon_sym_i16] = ACTIONS(1367),
    [anon_sym_u32] = ACTIONS(1367),
    [anon_sym_i32] = ACTIONS(1367),
    [anon_sym_u64] = ACTIONS(1367),
    [anon_sym_i64] = ACTIONS(1367),
    [anon_sym_u128] = ACTIONS(1367),
    [anon_sym_i128] = ACTIONS(1367),
    [anon_sym_isize] = ACTIONS(1367),
    [anon_sym_usize] = ACTIONS(1367),
    [anon_sym_f32] = ACTIONS(1367),
    [anon_sym_f64] = ACTIONS(1367),
    [anon_sym_bool] = ACTIONS(1367),
    [anon_sym_str] = ACTIONS(1367),
    [anon_sym_char] = ACTIONS(1367),
    [anon_sym_DASH] = ACTIONS(1367),
    [anon_sym_SLASH] = ACTIONS(1367),
    [anon_sym_PERCENT] = ACTIONS(1367),
    [anon_sym_CARET] = ACTIONS(1367),
    [anon_sym_AMP] = ACTIONS(1367),
    [anon_sym_PIPE] = ACTIONS(1367),
    [anon_sym_AMP_AMP] = ACTIONS(1365),
    [anon_sym_PIPE_PIPE] = ACTIONS(1365),
    [anon_sym_LT_LT] = ACTIONS(1367),
    [anon_sym_GT_GT] = ACTIONS(1367),
    [anon_sym_PLUS_EQ] = ACTIONS(1365),
    [anon_sym_DASH_EQ] = ACTIONS(1365),
    [anon_sym_STAR_EQ] = ACTIONS(1365),
    [anon_sym_SLASH_EQ] = ACTIONS(1365),
    [anon_sym_PERCENT_EQ] = ACTIONS(1365),
    [anon_sym_CARET_EQ] = ACTIONS(1365),
    [anon_sym_AMP_EQ] = ACTIONS(1365),
    [anon_sym_PIPE_EQ] = ACTIONS(1365),
    [anon_sym_LT_LT_EQ] = ACTIONS(1365),
    [anon_sym_GT_GT_EQ] = ACTIONS(1365),
    [anon_sym_EQ] = ACTIONS(1367),
    [anon_sym_EQ_EQ] = ACTIONS(1365),
    [anon_sym_BANG_EQ] = ACTIONS(1365),
    [anon_sym_GT] = ACTIONS(1367),
    [anon_sym_LT] = ACTIONS(1367),
    [anon_sym_GT_EQ] = ACTIONS(1365),
    [anon_sym_LT_EQ] = ACTIONS(1365),
    [anon_sym__] = ACTIONS(1367),
    [anon_sym_DOT] = ACTIONS(1367),
    [anon_sym_DOT_DOT] = ACTIONS(1367),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1365),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1365),
    [anon_sym_COMMA] = ACTIONS(1365),
    [anon_sym_COLON_COLON] = ACTIONS(1365),
    [anon_sym_POUND] = ACTIONS(1365),
    [anon_sym_as] = ACTIONS(1367),
    [anon_sym_const] = ACTIONS(1367),
    [anon_sym_default] = ACTIONS(1367),
    [anon_sym_gen] = ACTIONS(1367),
    [anon_sym_union] = ACTIONS(1367),
    [anon_sym_ref] = ACTIONS(1367),
    [sym_mutable_specifier] = ACTIONS(1367),
    [sym_integer_literal] = ACTIONS(1365),
    [aux_sym_string_literal_token1] = ACTIONS(1365),
    [sym_char_literal] = ACTIONS(1365),
    [anon_sym_true] = ACTIONS(1367),
    [anon_sym_false] = ACTIONS(1367),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1367),
    [sym_super] = ACTIONS(1367),
    [sym_crate] = ACTIONS(1367),
    [sym_metavariable] = ACTIONS(1365),
    [sym__raw_string_literal_start] = ACTIONS(1365),
    [sym_float_literal] = ACTIONS(1365),
  },
  [465] = {
    [sym_line_comment] = STATE(465),
    [sym_block_comment] = STATE(465),
    [sym_identifier] = ACTIONS(1457),
    [anon_sym_LPAREN] = ACTIONS(1455),
    [anon_sym_LBRACK] = ACTIONS(1455),
    [anon_sym_RBRACE] = ACTIONS(1455),
    [anon_sym_PLUS] = ACTIONS(1457),
    [anon_sym_STAR] = ACTIONS(1457),
    [anon_sym_QMARK] = ACTIONS(1455),
    [anon_sym_u8] = ACTIONS(1457),
    [anon_sym_i8] = ACTIONS(1457),
    [anon_sym_u16] = ACTIONS(1457),
    [anon_sym_i16] = ACTIONS(1457),
    [anon_sym_u32] = ACTIONS(1457),
    [anon_sym_i32] = ACTIONS(1457),
    [anon_sym_u64] = ACTIONS(1457),
    [anon_sym_i64] = ACTIONS(1457),
    [anon_sym_u128] = ACTIONS(1457),
    [anon_sym_i128] = ACTIONS(1457),
    [anon_sym_isize] = ACTIONS(1457),
    [anon_sym_usize] = ACTIONS(1457),
    [anon_sym_f32] = ACTIONS(1457),
    [anon_sym_f64] = ACTIONS(1457),
    [anon_sym_bool] = ACTIONS(1457),
    [anon_sym_str] = ACTIONS(1457),
    [anon_sym_char] = ACTIONS(1457),
    [anon_sym_DASH] = ACTIONS(1457),
    [anon_sym_SLASH] = ACTIONS(1457),
    [anon_sym_PERCENT] = ACTIONS(1457),
    [anon_sym_CARET] = ACTIONS(1457),
    [anon_sym_AMP] = ACTIONS(1457),
    [anon_sym_PIPE] = ACTIONS(1457),
    [anon_sym_AMP_AMP] = ACTIONS(1455),
    [anon_sym_PIPE_PIPE] = ACTIONS(1455),
    [anon_sym_LT_LT] = ACTIONS(1457),
    [anon_sym_GT_GT] = ACTIONS(1457),
    [anon_sym_PLUS_EQ] = ACTIONS(1455),
    [anon_sym_DASH_EQ] = ACTIONS(1455),
    [anon_sym_STAR_EQ] = ACTIONS(1455),
    [anon_sym_SLASH_EQ] = ACTIONS(1455),
    [anon_sym_PERCENT_EQ] = ACTIONS(1455),
    [anon_sym_CARET_EQ] = ACTIONS(1455),
    [anon_sym_AMP_EQ] = ACTIONS(1455),
    [anon_sym_PIPE_EQ] = ACTIONS(1455),
    [anon_sym_LT_LT_EQ] = ACTIONS(1455),
    [anon_sym_GT_GT_EQ] = ACTIONS(1455),
    [anon_sym_EQ] = ACTIONS(1457),
    [anon_sym_EQ_EQ] = ACTIONS(1455),
    [anon_sym_BANG_EQ] = ACTIONS(1455),
    [anon_sym_GT] = ACTIONS(1457),
    [anon_sym_LT] = ACTIONS(1457),
    [anon_sym_GT_EQ] = ACTIONS(1455),
    [anon_sym_LT_EQ] = ACTIONS(1455),
    [anon_sym__] = ACTIONS(1457),
    [anon_sym_DOT] = ACTIONS(1457),
    [anon_sym_DOT_DOT] = ACTIONS(1457),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1455),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1455),
    [anon_sym_COMMA] = ACTIONS(1455),
    [anon_sym_COLON_COLON] = ACTIONS(1455),
    [anon_sym_POUND] = ACTIONS(1455),
    [anon_sym_as] = ACTIONS(1457),
    [anon_sym_const] = ACTIONS(1457),
    [anon_sym_default] = ACTIONS(1457),
    [anon_sym_gen] = ACTIONS(1457),
    [anon_sym_union] = ACTIONS(1457),
    [anon_sym_ref] = ACTIONS(1457),
    [sym_mutable_specifier] = ACTIONS(1457),
    [sym_integer_literal] = ACTIONS(1455),
    [aux_sym_string_literal_token1] = ACTIONS(1455),
    [sym_char_literal] = ACTIONS(1455),
    [anon_sym_true] = ACTIONS(1457),
    [anon_sym_false] = ACTIONS(1457),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1457),
    [sym_super] = ACTIONS(1457),
    [sym_crate] = ACTIONS(1457),
    [sym_metavariable] = ACTIONS(1455),
    [sym__raw_string_literal_start] = ACTIONS(1455),
    [sym_float_literal] = ACTIONS(1455),
  },
  [466] = {
    [sym_line_comment] = STATE(466),
    [sym_block_comment] = STATE(466),
    [sym_identifier] = ACTIONS(1405),
    [anon_sym_LPAREN] = ACTIONS(1403),
    [anon_sym_LBRACK] = ACTIONS(1403),
    [anon_sym_RBRACE] = ACTIONS(1403),
    [anon_sym_PLUS] = ACTIONS(1405),
    [anon_sym_STAR] = ACTIONS(1405),
    [anon_sym_QMARK] = ACTIONS(1403),
    [anon_sym_u8] = ACTIONS(1405),
    [anon_sym_i8] = ACTIONS(1405),
    [anon_sym_u16] = ACTIONS(1405),
    [anon_sym_i16] = ACTIONS(1405),
    [anon_sym_u32] = ACTIONS(1405),
    [anon_sym_i32] = ACTIONS(1405),
    [anon_sym_u64] = ACTIONS(1405),
    [anon_sym_i64] = ACTIONS(1405),
    [anon_sym_u128] = ACTIONS(1405),
    [anon_sym_i128] = ACTIONS(1405),
    [anon_sym_isize] = ACTIONS(1405),
    [anon_sym_usize] = ACTIONS(1405),
    [anon_sym_f32] = ACTIONS(1405),
    [anon_sym_f64] = ACTIONS(1405),
    [anon_sym_bool] = ACTIONS(1405),
    [anon_sym_str] = ACTIONS(1405),
    [anon_sym_char] = ACTIONS(1405),
    [anon_sym_DASH] = ACTIONS(1405),
    [anon_sym_SLASH] = ACTIONS(1405),
    [anon_sym_PERCENT] = ACTIONS(1405),
    [anon_sym_CARET] = ACTIONS(1405),
    [anon_sym_AMP] = ACTIONS(1405),
    [anon_sym_PIPE] = ACTIONS(1405),
    [anon_sym_AMP_AMP] = ACTIONS(1403),
    [anon_sym_PIPE_PIPE] = ACTIONS(1403),
    [anon_sym_LT_LT] = ACTIONS(1405),
    [anon_sym_GT_GT] = ACTIONS(1405),
    [anon_sym_PLUS_EQ] = ACTIONS(1403),
    [anon_sym_DASH_EQ] = ACTIONS(1403),
    [anon_sym_STAR_EQ] = ACTIONS(1403),
    [anon_sym_SLASH_EQ] = ACTIONS(1403),
    [anon_sym_PERCENT_EQ] = ACTIONS(1403),
    [anon_sym_CARET_EQ] = ACTIONS(1403),
    [anon_sym_AMP_EQ] = ACTIONS(1403),
    [anon_sym_PIPE_EQ] = ACTIONS(1403),
    [anon_sym_LT_LT_EQ] = ACTIONS(1403),
    [anon_sym_GT_GT_EQ] = ACTIONS(1403),
    [anon_sym_EQ] = ACTIONS(1405),
    [anon_sym_EQ_EQ] = ACTIONS(1403),
    [anon_sym_BANG_EQ] = ACTIONS(1403),
    [anon_sym_GT] = ACTIONS(1405),
    [anon_sym_LT] = ACTIONS(1405),
    [anon_sym_GT_EQ] = ACTIONS(1403),
    [anon_sym_LT_EQ] = ACTIONS(1403),
    [anon_sym__] = ACTIONS(1405),
    [anon_sym_DOT] = ACTIONS(1405),
    [anon_sym_DOT_DOT] = ACTIONS(1405),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1403),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1403),
    [anon_sym_COMMA] = ACTIONS(1403),
    [anon_sym_COLON_COLON] = ACTIONS(1403),
    [anon_sym_POUND] = ACTIONS(1403),
    [anon_sym_as] = ACTIONS(1405),
    [anon_sym_const] = ACTIONS(1405),
    [anon_sym_default] = ACTIONS(1405),
    [anon_sym_gen] = ACTIONS(1405),
    [anon_sym_union] = ACTIONS(1405),
    [anon_sym_ref] = ACTIONS(1405),
    [sym_mutable_specifier] = ACTIONS(1405),
    [sym_integer_literal] = ACTIONS(1403),
    [aux_sym_string_literal_token1] = ACTIONS(1403),
    [sym_char_literal] = ACTIONS(1403),
    [anon_sym_true] = ACTIONS(1405),
    [anon_sym_false] = ACTIONS(1405),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1405),
    [sym_super] = ACTIONS(1405),
    [sym_crate] = ACTIONS(1405),
    [sym_metavariable] = ACTIONS(1403),
    [sym__raw_string_literal_start] = ACTIONS(1403),
    [sym_float_literal] = ACTIONS(1403),
  },
  [467] = {
    [sym_attribute_item] = STATE(1468),
    [sym_inner_attribute_item] = STATE(1468),
    [sym_bracketed_type] = STATE(3553),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3287),
    [sym_macro_invocation] = STATE(2757),
    [sym_scoped_identifier] = STATE(2151),
    [sym_scoped_type_identifier] = STATE(2982),
    [sym_match_arm] = STATE(1469),
    [sym_last_match_arm] = STATE(3466),
    [sym_match_pattern] = STATE(3352),
    [sym_const_block] = STATE(2757),
    [sym__pattern] = STATE(2907),
    [sym_tuple_pattern] = STATE(2757),
    [sym_slice_pattern] = STATE(2757),
    [sym_tuple_struct_pattern] = STATE(2757),
    [sym_struct_pattern] = STATE(2757),
    [sym_remaining_field_pattern] = STATE(2757),
    [sym_mut_pattern] = STATE(2757),
    [sym_range_pattern] = STATE(2757),
    [sym_ref_pattern] = STATE(2757),
    [sym_captured_pattern] = STATE(2757),
    [sym_reference_pattern] = STATE(2757),
    [sym_or_pattern] = STATE(2757),
    [sym__literal_pattern] = STATE(2330),
    [sym_negative_literal] = STATE(2364),
    [sym_string_literal] = STATE(2364),
    [sym_raw_string_literal] = STATE(2364),
    [sym_boolean_literal] = STATE(2364),
    [sym_line_comment] = STATE(467),
    [sym_block_comment] = STATE(467),
    [aux_sym_match_block_repeat1] = STATE(486),
    [aux_sym_match_arm_repeat1] = STATE(767),
    [sym_identifier] = ACTIONS(1645),
    [anon_sym_LPAREN] = ACTIONS(1647),
    [anon_sym_LBRACK] = ACTIONS(1649),
    [anon_sym_RBRACE] = ACTIONS(1651),
    [anon_sym_u8] = ACTIONS(1653),
    [anon_sym_i8] = ACTIONS(1653),
    [anon_sym_u16] = ACTIONS(1653),
    [anon_sym_i16] = ACTIONS(1653),
    [anon_sym_u32] = ACTIONS(1653),
    [anon_sym_i32] = ACTIONS(1653),
    [anon_sym_u64] = ACTIONS(1653),
    [anon_sym_i64] = ACTIONS(1653),
    [anon_sym_u128] = ACTIONS(1653),
    [anon_sym_i128] = ACTIONS(1653),
    [anon_sym_isize] = ACTIONS(1653),
    [anon_sym_usize] = ACTIONS(1653),
    [anon_sym_f32] = ACTIONS(1653),
    [anon_sym_f64] = ACTIONS(1653),
    [anon_sym_bool] = ACTIONS(1653),
    [anon_sym_str] = ACTIONS(1653),
    [anon_sym_char] = ACTIONS(1653),
    [anon_sym_DASH] = ACTIONS(1655),
    [anon_sym_AMP] = ACTIONS(1657),
    [anon_sym_PIPE] = ACTIONS(1659),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1661),
    [anon_sym_DOT_DOT] = ACTIONS(1663),
    [anon_sym_COLON_COLON] = ACTIONS(1665),
    [anon_sym_POUND] = ACTIONS(1667),
    [anon_sym_const] = ACTIONS(1669),
    [anon_sym_default] = ACTIONS(1671),
    [anon_sym_gen] = ACTIONS(1671),
    [anon_sym_union] = ACTIONS(1671),
    [anon_sym_ref] = ACTIONS(1673),
    [sym_mutable_specifier] = ACTIONS(1675),
    [sym_integer_literal] = ACTIONS(1677),
    [aux_sym_string_literal_token1] = ACTIONS(1679),
    [sym_char_literal] = ACTIONS(1677),
    [anon_sym_true] = ACTIONS(1681),
    [anon_sym_false] = ACTIONS(1681),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1683),
    [sym_super] = ACTIONS(1683),
    [sym_crate] = ACTIONS(1683),
    [sym_metavariable] = ACTIONS(1685),
    [sym__raw_string_literal_start] = ACTIONS(1687),
    [sym_float_literal] = ACTIONS(1677),
  },
  [468] = {
    [sym_line_comment] = STATE(468),
    [sym_block_comment] = STATE(468),
    [sym_identifier] = ACTIONS(1413),
    [anon_sym_LPAREN] = ACTIONS(1411),
    [anon_sym_LBRACK] = ACTIONS(1411),
    [anon_sym_RBRACE] = ACTIONS(1411),
    [anon_sym_PLUS] = ACTIONS(1413),
    [anon_sym_STAR] = ACTIONS(1413),
    [anon_sym_QMARK] = ACTIONS(1411),
    [anon_sym_u8] = ACTIONS(1413),
    [anon_sym_i8] = ACTIONS(1413),
    [anon_sym_u16] = ACTIONS(1413),
    [anon_sym_i16] = ACTIONS(1413),
    [anon_sym_u32] = ACTIONS(1413),
    [anon_sym_i32] = ACTIONS(1413),
    [anon_sym_u64] = ACTIONS(1413),
    [anon_sym_i64] = ACTIONS(1413),
    [anon_sym_u128] = ACTIONS(1413),
    [anon_sym_i128] = ACTIONS(1413),
    [anon_sym_isize] = ACTIONS(1413),
    [anon_sym_usize] = ACTIONS(1413),
    [anon_sym_f32] = ACTIONS(1413),
    [anon_sym_f64] = ACTIONS(1413),
    [anon_sym_bool] = ACTIONS(1413),
    [anon_sym_str] = ACTIONS(1413),
    [anon_sym_char] = ACTIONS(1413),
    [anon_sym_DASH] = ACTIONS(1413),
    [anon_sym_SLASH] = ACTIONS(1413),
    [anon_sym_PERCENT] = ACTIONS(1413),
    [anon_sym_CARET] = ACTIONS(1413),
    [anon_sym_AMP] = ACTIONS(1413),
    [anon_sym_PIPE] = ACTIONS(1413),
    [anon_sym_AMP_AMP] = ACTIONS(1411),
    [anon_sym_PIPE_PIPE] = ACTIONS(1411),
    [anon_sym_LT_LT] = ACTIONS(1413),
    [anon_sym_GT_GT] = ACTIONS(1413),
    [anon_sym_PLUS_EQ] = ACTIONS(1411),
    [anon_sym_DASH_EQ] = ACTIONS(1411),
    [anon_sym_STAR_EQ] = ACTIONS(1411),
    [anon_sym_SLASH_EQ] = ACTIONS(1411),
    [anon_sym_PERCENT_EQ] = ACTIONS(1411),
    [anon_sym_CARET_EQ] = ACTIONS(1411),
    [anon_sym_AMP_EQ] = ACTIONS(1411),
    [anon_sym_PIPE_EQ] = ACTIONS(1411),
    [anon_sym_LT_LT_EQ] = ACTIONS(1411),
    [anon_sym_GT_GT_EQ] = ACTIONS(1411),
    [anon_sym_EQ] = ACTIONS(1413),
    [anon_sym_EQ_EQ] = ACTIONS(1411),
    [anon_sym_BANG_EQ] = ACTIONS(1411),
    [anon_sym_GT] = ACTIONS(1413),
    [anon_sym_LT] = ACTIONS(1413),
    [anon_sym_GT_EQ] = ACTIONS(1411),
    [anon_sym_LT_EQ] = ACTIONS(1411),
    [anon_sym__] = ACTIONS(1413),
    [anon_sym_DOT] = ACTIONS(1413),
    [anon_sym_DOT_DOT] = ACTIONS(1413),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1411),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1411),
    [anon_sym_COMMA] = ACTIONS(1411),
    [anon_sym_COLON_COLON] = ACTIONS(1411),
    [anon_sym_POUND] = ACTIONS(1411),
    [anon_sym_as] = ACTIONS(1413),
    [anon_sym_const] = ACTIONS(1413),
    [anon_sym_default] = ACTIONS(1413),
    [anon_sym_gen] = ACTIONS(1413),
    [anon_sym_union] = ACTIONS(1413),
    [anon_sym_ref] = ACTIONS(1413),
    [sym_mutable_specifier] = ACTIONS(1413),
    [sym_integer_literal] = ACTIONS(1411),
    [aux_sym_string_literal_token1] = ACTIONS(1411),
    [sym_char_literal] = ACTIONS(1411),
    [anon_sym_true] = ACTIONS(1413),
    [anon_sym_false] = ACTIONS(1413),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1413),
    [sym_super] = ACTIONS(1413),
    [sym_crate] = ACTIONS(1413),
    [sym_metavariable] = ACTIONS(1411),
    [sym__raw_string_literal_start] = ACTIONS(1411),
    [sym_float_literal] = ACTIONS(1411),
  },
  [469] = {
    [sym_line_comment] = STATE(469),
    [sym_block_comment] = STATE(469),
    [sym_identifier] = ACTIONS(1401),
    [anon_sym_LPAREN] = ACTIONS(1399),
    [anon_sym_LBRACK] = ACTIONS(1399),
    [anon_sym_RBRACE] = ACTIONS(1399),
    [anon_sym_PLUS] = ACTIONS(1401),
    [anon_sym_STAR] = ACTIONS(1401),
    [anon_sym_QMARK] = ACTIONS(1399),
    [anon_sym_u8] = ACTIONS(1401),
    [anon_sym_i8] = ACTIONS(1401),
    [anon_sym_u16] = ACTIONS(1401),
    [anon_sym_i16] = ACTIONS(1401),
    [anon_sym_u32] = ACTIONS(1401),
    [anon_sym_i32] = ACTIONS(1401),
    [anon_sym_u64] = ACTIONS(1401),
    [anon_sym_i64] = ACTIONS(1401),
    [anon_sym_u128] = ACTIONS(1401),
    [anon_sym_i128] = ACTIONS(1401),
    [anon_sym_isize] = ACTIONS(1401),
    [anon_sym_usize] = ACTIONS(1401),
    [anon_sym_f32] = ACTIONS(1401),
    [anon_sym_f64] = ACTIONS(1401),
    [anon_sym_bool] = ACTIONS(1401),
    [anon_sym_str] = ACTIONS(1401),
    [anon_sym_char] = ACTIONS(1401),
    [anon_sym_DASH] = ACTIONS(1401),
    [anon_sym_SLASH] = ACTIONS(1401),
    [anon_sym_PERCENT] = ACTIONS(1401),
    [anon_sym_CARET] = ACTIONS(1401),
    [anon_sym_AMP] = ACTIONS(1401),
    [anon_sym_PIPE] = ACTIONS(1401),
    [anon_sym_AMP_AMP] = ACTIONS(1399),
    [anon_sym_PIPE_PIPE] = ACTIONS(1399),
    [anon_sym_LT_LT] = ACTIONS(1401),
    [anon_sym_GT_GT] = ACTIONS(1401),
    [anon_sym_PLUS_EQ] = ACTIONS(1399),
    [anon_sym_DASH_EQ] = ACTIONS(1399),
    [anon_sym_STAR_EQ] = ACTIONS(1399),
    [anon_sym_SLASH_EQ] = ACTIONS(1399),
    [anon_sym_PERCENT_EQ] = ACTIONS(1399),
    [anon_sym_CARET_EQ] = ACTIONS(1399),
    [anon_sym_AMP_EQ] = ACTIONS(1399),
    [anon_sym_PIPE_EQ] = ACTIONS(1399),
    [anon_sym_LT_LT_EQ] = ACTIONS(1399),
    [anon_sym_GT_GT_EQ] = ACTIONS(1399),
    [anon_sym_EQ] = ACTIONS(1401),
    [anon_sym_EQ_EQ] = ACTIONS(1399),
    [anon_sym_BANG_EQ] = ACTIONS(1399),
    [anon_sym_GT] = ACTIONS(1401),
    [anon_sym_LT] = ACTIONS(1401),
    [anon_sym_GT_EQ] = ACTIONS(1399),
    [anon_sym_LT_EQ] = ACTIONS(1399),
    [anon_sym__] = ACTIONS(1401),
    [anon_sym_DOT] = ACTIONS(1401),
    [anon_sym_DOT_DOT] = ACTIONS(1401),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1399),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1399),
    [anon_sym_COMMA] = ACTIONS(1399),
    [anon_sym_COLON_COLON] = ACTIONS(1399),
    [anon_sym_POUND] = ACTIONS(1399),
    [anon_sym_as] = ACTIONS(1401),
    [anon_sym_const] = ACTIONS(1401),
    [anon_sym_default] = ACTIONS(1401),
    [anon_sym_gen] = ACTIONS(1401),
    [anon_sym_union] = ACTIONS(1401),
    [anon_sym_ref] = ACTIONS(1401),
    [sym_mutable_specifier] = ACTIONS(1401),
    [sym_integer_literal] = ACTIONS(1399),
    [aux_sym_string_literal_token1] = ACTIONS(1399),
    [sym_char_literal] = ACTIONS(1399),
    [anon_sym_true] = ACTIONS(1401),
    [anon_sym_false] = ACTIONS(1401),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1401),
    [sym_super] = ACTIONS(1401),
    [sym_crate] = ACTIONS(1401),
    [sym_metavariable] = ACTIONS(1399),
    [sym__raw_string_literal_start] = ACTIONS(1399),
    [sym_float_literal] = ACTIONS(1399),
  },
  [470] = {
    [sym_line_comment] = STATE(470),
    [sym_block_comment] = STATE(470),
    [sym_identifier] = ACTIONS(1363),
    [anon_sym_LPAREN] = ACTIONS(1361),
    [anon_sym_LBRACK] = ACTIONS(1361),
    [anon_sym_RBRACE] = ACTIONS(1361),
    [anon_sym_PLUS] = ACTIONS(1363),
    [anon_sym_STAR] = ACTIONS(1363),
    [anon_sym_QMARK] = ACTIONS(1361),
    [anon_sym_u8] = ACTIONS(1363),
    [anon_sym_i8] = ACTIONS(1363),
    [anon_sym_u16] = ACTIONS(1363),
    [anon_sym_i16] = ACTIONS(1363),
    [anon_sym_u32] = ACTIONS(1363),
    [anon_sym_i32] = ACTIONS(1363),
    [anon_sym_u64] = ACTIONS(1363),
    [anon_sym_i64] = ACTIONS(1363),
    [anon_sym_u128] = ACTIONS(1363),
    [anon_sym_i128] = ACTIONS(1363),
    [anon_sym_isize] = ACTIONS(1363),
    [anon_sym_usize] = ACTIONS(1363),
    [anon_sym_f32] = ACTIONS(1363),
    [anon_sym_f64] = ACTIONS(1363),
    [anon_sym_bool] = ACTIONS(1363),
    [anon_sym_str] = ACTIONS(1363),
    [anon_sym_char] = ACTIONS(1363),
    [anon_sym_DASH] = ACTIONS(1363),
    [anon_sym_SLASH] = ACTIONS(1363),
    [anon_sym_PERCENT] = ACTIONS(1363),
    [anon_sym_CARET] = ACTIONS(1363),
    [anon_sym_AMP] = ACTIONS(1363),
    [anon_sym_PIPE] = ACTIONS(1363),
    [anon_sym_AMP_AMP] = ACTIONS(1361),
    [anon_sym_PIPE_PIPE] = ACTIONS(1361),
    [anon_sym_LT_LT] = ACTIONS(1363),
    [anon_sym_GT_GT] = ACTIONS(1363),
    [anon_sym_PLUS_EQ] = ACTIONS(1361),
    [anon_sym_DASH_EQ] = ACTIONS(1361),
    [anon_sym_STAR_EQ] = ACTIONS(1361),
    [anon_sym_SLASH_EQ] = ACTIONS(1361),
    [anon_sym_PERCENT_EQ] = ACTIONS(1361),
    [anon_sym_CARET_EQ] = ACTIONS(1361),
    [anon_sym_AMP_EQ] = ACTIONS(1361),
    [anon_sym_PIPE_EQ] = ACTIONS(1361),
    [anon_sym_LT_LT_EQ] = ACTIONS(1361),
    [anon_sym_GT_GT_EQ] = ACTIONS(1361),
    [anon_sym_EQ] = ACTIONS(1363),
    [anon_sym_EQ_EQ] = ACTIONS(1361),
    [anon_sym_BANG_EQ] = ACTIONS(1361),
    [anon_sym_GT] = ACTIONS(1363),
    [anon_sym_LT] = ACTIONS(1363),
    [anon_sym_GT_EQ] = ACTIONS(1361),
    [anon_sym_LT_EQ] = ACTIONS(1361),
    [anon_sym__] = ACTIONS(1363),
    [anon_sym_DOT] = ACTIONS(1363),
    [anon_sym_DOT_DOT] = ACTIONS(1363),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1361),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1361),
    [anon_sym_COMMA] = ACTIONS(1361),
    [anon_sym_COLON_COLON] = ACTIONS(1361),
    [anon_sym_POUND] = ACTIONS(1361),
    [anon_sym_as] = ACTIONS(1363),
    [anon_sym_const] = ACTIONS(1363),
    [anon_sym_default] = ACTIONS(1363),
    [anon_sym_gen] = ACTIONS(1363),
    [anon_sym_union] = ACTIONS(1363),
    [anon_sym_ref] = ACTIONS(1363),
    [sym_mutable_specifier] = ACTIONS(1363),
    [sym_integer_literal] = ACTIONS(1361),
    [aux_sym_string_literal_token1] = ACTIONS(1361),
    [sym_char_literal] = ACTIONS(1361),
    [anon_sym_true] = ACTIONS(1363),
    [anon_sym_false] = ACTIONS(1363),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1363),
    [sym_super] = ACTIONS(1363),
    [sym_crate] = ACTIONS(1363),
    [sym_metavariable] = ACTIONS(1361),
    [sym__raw_string_literal_start] = ACTIONS(1361),
    [sym_float_literal] = ACTIONS(1361),
  },
  [471] = {
    [sym_line_comment] = STATE(471),
    [sym_block_comment] = STATE(471),
    [sym_identifier] = ACTIONS(1689),
    [anon_sym_LPAREN] = ACTIONS(1691),
    [anon_sym_LBRACK] = ACTIONS(1691),
    [anon_sym_RBRACE] = ACTIONS(1359),
    [anon_sym_PLUS] = ACTIONS(1357),
    [anon_sym_STAR] = ACTIONS(1357),
    [anon_sym_QMARK] = ACTIONS(1359),
    [anon_sym_u8] = ACTIONS(1689),
    [anon_sym_i8] = ACTIONS(1689),
    [anon_sym_u16] = ACTIONS(1689),
    [anon_sym_i16] = ACTIONS(1689),
    [anon_sym_u32] = ACTIONS(1689),
    [anon_sym_i32] = ACTIONS(1689),
    [anon_sym_u64] = ACTIONS(1689),
    [anon_sym_i64] = ACTIONS(1689),
    [anon_sym_u128] = ACTIONS(1689),
    [anon_sym_i128] = ACTIONS(1689),
    [anon_sym_isize] = ACTIONS(1689),
    [anon_sym_usize] = ACTIONS(1689),
    [anon_sym_f32] = ACTIONS(1689),
    [anon_sym_f64] = ACTIONS(1689),
    [anon_sym_bool] = ACTIONS(1689),
    [anon_sym_str] = ACTIONS(1689),
    [anon_sym_char] = ACTIONS(1689),
    [anon_sym_DASH] = ACTIONS(1689),
    [anon_sym_SLASH] = ACTIONS(1357),
    [anon_sym_PERCENT] = ACTIONS(1357),
    [anon_sym_CARET] = ACTIONS(1357),
    [anon_sym_AMP] = ACTIONS(1689),
    [anon_sym_PIPE] = ACTIONS(1689),
    [anon_sym_AMP_AMP] = ACTIONS(1359),
    [anon_sym_PIPE_PIPE] = ACTIONS(1359),
    [anon_sym_LT_LT] = ACTIONS(1357),
    [anon_sym_GT_GT] = ACTIONS(1357),
    [anon_sym_PLUS_EQ] = ACTIONS(1359),
    [anon_sym_DASH_EQ] = ACTIONS(1359),
    [anon_sym_STAR_EQ] = ACTIONS(1359),
    [anon_sym_SLASH_EQ] = ACTIONS(1359),
    [anon_sym_PERCENT_EQ] = ACTIONS(1359),
    [anon_sym_CARET_EQ] = ACTIONS(1359),
    [anon_sym_AMP_EQ] = ACTIONS(1359),
    [anon_sym_PIPE_EQ] = ACTIONS(1359),
    [anon_sym_LT_LT_EQ] = ACTIONS(1359),
    [anon_sym_GT_GT_EQ] = ACTIONS(1359),
    [anon_sym_EQ] = ACTIONS(1357),
    [anon_sym_EQ_EQ] = ACTIONS(1359),
    [anon_sym_BANG_EQ] = ACTIONS(1359),
    [anon_sym_GT] = ACTIONS(1357),
    [anon_sym_LT] = ACTIONS(1689),
    [anon_sym_GT_EQ] = ACTIONS(1359),
    [anon_sym_LT_EQ] = ACTIONS(1359),
    [anon_sym__] = ACTIONS(1689),
    [anon_sym_DOT] = ACTIONS(1357),
    [anon_sym_DOT_DOT] = ACTIONS(1689),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1359),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1359),
    [anon_sym_COMMA] = ACTIONS(1359),
    [anon_sym_COLON_COLON] = ACTIONS(1691),
    [anon_sym_POUND] = ACTIONS(1691),
    [anon_sym_as] = ACTIONS(1357),
    [anon_sym_const] = ACTIONS(1689),
    [anon_sym_default] = ACTIONS(1689),
    [anon_sym_gen] = ACTIONS(1689),
    [anon_sym_union] = ACTIONS(1689),
    [anon_sym_ref] = ACTIONS(1689),
    [sym_mutable_specifier] = ACTIONS(1689),
    [sym_integer_literal] = ACTIONS(1691),
    [aux_sym_string_literal_token1] = ACTIONS(1691),
    [sym_char_literal] = ACTIONS(1691),
    [anon_sym_true] = ACTIONS(1689),
    [anon_sym_false] = ACTIONS(1689),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1689),
    [sym_super] = ACTIONS(1689),
    [sym_crate] = ACTIONS(1689),
    [sym_metavariable] = ACTIONS(1691),
    [sym__raw_string_literal_start] = ACTIONS(1691),
    [sym_float_literal] = ACTIONS(1691),
  },
  [472] = {
    [sym_attribute_item] = STATE(1468),
    [sym_inner_attribute_item] = STATE(1468),
    [sym_bracketed_type] = STATE(3553),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3287),
    [sym_macro_invocation] = STATE(2757),
    [sym_scoped_identifier] = STATE(2151),
    [sym_scoped_type_identifier] = STATE(2982),
    [sym_match_arm] = STATE(1469),
    [sym_last_match_arm] = STATE(3406),
    [sym_match_pattern] = STATE(3352),
    [sym_const_block] = STATE(2757),
    [sym__pattern] = STATE(2907),
    [sym_tuple_pattern] = STATE(2757),
    [sym_slice_pattern] = STATE(2757),
    [sym_tuple_struct_pattern] = STATE(2757),
    [sym_struct_pattern] = STATE(2757),
    [sym_remaining_field_pattern] = STATE(2757),
    [sym_mut_pattern] = STATE(2757),
    [sym_range_pattern] = STATE(2757),
    [sym_ref_pattern] = STATE(2757),
    [sym_captured_pattern] = STATE(2757),
    [sym_reference_pattern] = STATE(2757),
    [sym_or_pattern] = STATE(2757),
    [sym__literal_pattern] = STATE(2330),
    [sym_negative_literal] = STATE(2364),
    [sym_string_literal] = STATE(2364),
    [sym_raw_string_literal] = STATE(2364),
    [sym_boolean_literal] = STATE(2364),
    [sym_line_comment] = STATE(472),
    [sym_block_comment] = STATE(472),
    [aux_sym_match_block_repeat1] = STATE(484),
    [aux_sym_match_arm_repeat1] = STATE(767),
    [sym_identifier] = ACTIONS(1645),
    [anon_sym_LPAREN] = ACTIONS(1647),
    [anon_sym_LBRACK] = ACTIONS(1649),
    [anon_sym_RBRACE] = ACTIONS(1693),
    [anon_sym_u8] = ACTIONS(1653),
    [anon_sym_i8] = ACTIONS(1653),
    [anon_sym_u16] = ACTIONS(1653),
    [anon_sym_i16] = ACTIONS(1653),
    [anon_sym_u32] = ACTIONS(1653),
    [anon_sym_i32] = ACTIONS(1653),
    [anon_sym_u64] = ACTIONS(1653),
    [anon_sym_i64] = ACTIONS(1653),
    [anon_sym_u128] = ACTIONS(1653),
    [anon_sym_i128] = ACTIONS(1653),
    [anon_sym_isize] = ACTIONS(1653),
    [anon_sym_usize] = ACTIONS(1653),
    [anon_sym_f32] = ACTIONS(1653),
    [anon_sym_f64] = ACTIONS(1653),
    [anon_sym_bool] = ACTIONS(1653),
    [anon_sym_str] = ACTIONS(1653),
    [anon_sym_char] = ACTIONS(1653),
    [anon_sym_DASH] = ACTIONS(1655),
    [anon_sym_AMP] = ACTIONS(1657),
    [anon_sym_PIPE] = ACTIONS(1659),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1661),
    [anon_sym_DOT_DOT] = ACTIONS(1663),
    [anon_sym_COLON_COLON] = ACTIONS(1665),
    [anon_sym_POUND] = ACTIONS(1667),
    [anon_sym_const] = ACTIONS(1669),
    [anon_sym_default] = ACTIONS(1671),
    [anon_sym_gen] = ACTIONS(1671),
    [anon_sym_union] = ACTIONS(1671),
    [anon_sym_ref] = ACTIONS(1673),
    [sym_mutable_specifier] = ACTIONS(1675),
    [sym_integer_literal] = ACTIONS(1677),
    [aux_sym_string_literal_token1] = ACTIONS(1679),
    [sym_char_literal] = ACTIONS(1677),
    [anon_sym_true] = ACTIONS(1681),
    [anon_sym_false] = ACTIONS(1681),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1683),
    [sym_super] = ACTIONS(1683),
    [sym_crate] = ACTIONS(1683),
    [sym_metavariable] = ACTIONS(1685),
    [sym__raw_string_literal_start] = ACTIONS(1687),
    [sym_float_literal] = ACTIONS(1677),
  },
  [473] = {
    [sym_line_comment] = STATE(473),
    [sym_block_comment] = STATE(473),
    [sym_identifier] = ACTIONS(1469),
    [anon_sym_LPAREN] = ACTIONS(1467),
    [anon_sym_LBRACK] = ACTIONS(1467),
    [anon_sym_RBRACE] = ACTIONS(1467),
    [anon_sym_PLUS] = ACTIONS(1469),
    [anon_sym_STAR] = ACTIONS(1469),
    [anon_sym_QMARK] = ACTIONS(1467),
    [anon_sym_u8] = ACTIONS(1469),
    [anon_sym_i8] = ACTIONS(1469),
    [anon_sym_u16] = ACTIONS(1469),
    [anon_sym_i16] = ACTIONS(1469),
    [anon_sym_u32] = ACTIONS(1469),
    [anon_sym_i32] = ACTIONS(1469),
    [anon_sym_u64] = ACTIONS(1469),
    [anon_sym_i64] = ACTIONS(1469),
    [anon_sym_u128] = ACTIONS(1469),
    [anon_sym_i128] = ACTIONS(1469),
    [anon_sym_isize] = ACTIONS(1469),
    [anon_sym_usize] = ACTIONS(1469),
    [anon_sym_f32] = ACTIONS(1469),
    [anon_sym_f64] = ACTIONS(1469),
    [anon_sym_bool] = ACTIONS(1469),
    [anon_sym_str] = ACTIONS(1469),
    [anon_sym_char] = ACTIONS(1469),
    [anon_sym_DASH] = ACTIONS(1469),
    [anon_sym_SLASH] = ACTIONS(1469),
    [anon_sym_PERCENT] = ACTIONS(1469),
    [anon_sym_CARET] = ACTIONS(1469),
    [anon_sym_AMP] = ACTIONS(1469),
    [anon_sym_PIPE] = ACTIONS(1469),
    [anon_sym_AMP_AMP] = ACTIONS(1467),
    [anon_sym_PIPE_PIPE] = ACTIONS(1467),
    [anon_sym_LT_LT] = ACTIONS(1469),
    [anon_sym_GT_GT] = ACTIONS(1469),
    [anon_sym_PLUS_EQ] = ACTIONS(1467),
    [anon_sym_DASH_EQ] = ACTIONS(1467),
    [anon_sym_STAR_EQ] = ACTIONS(1467),
    [anon_sym_SLASH_EQ] = ACTIONS(1467),
    [anon_sym_PERCENT_EQ] = ACTIONS(1467),
    [anon_sym_CARET_EQ] = ACTIONS(1467),
    [anon_sym_AMP_EQ] = ACTIONS(1467),
    [anon_sym_PIPE_EQ] = ACTIONS(1467),
    [anon_sym_LT_LT_EQ] = ACTIONS(1467),
    [anon_sym_GT_GT_EQ] = ACTIONS(1467),
    [anon_sym_EQ] = ACTIONS(1469),
    [anon_sym_EQ_EQ] = ACTIONS(1467),
    [anon_sym_BANG_EQ] = ACTIONS(1467),
    [anon_sym_GT] = ACTIONS(1469),
    [anon_sym_LT] = ACTIONS(1469),
    [anon_sym_GT_EQ] = ACTIONS(1467),
    [anon_sym_LT_EQ] = ACTIONS(1467),
    [anon_sym__] = ACTIONS(1469),
    [anon_sym_DOT] = ACTIONS(1469),
    [anon_sym_DOT_DOT] = ACTIONS(1469),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1467),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1467),
    [anon_sym_COMMA] = ACTIONS(1467),
    [anon_sym_COLON_COLON] = ACTIONS(1467),
    [anon_sym_POUND] = ACTIONS(1467),
    [anon_sym_as] = ACTIONS(1469),
    [anon_sym_const] = ACTIONS(1469),
    [anon_sym_default] = ACTIONS(1469),
    [anon_sym_gen] = ACTIONS(1469),
    [anon_sym_union] = ACTIONS(1469),
    [anon_sym_ref] = ACTIONS(1469),
    [sym_mutable_specifier] = ACTIONS(1469),
    [sym_integer_literal] = ACTIONS(1467),
    [aux_sym_string_literal_token1] = ACTIONS(1467),
    [sym_char_literal] = ACTIONS(1467),
    [anon_sym_true] = ACTIONS(1469),
    [anon_sym_false] = ACTIONS(1469),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1469),
    [sym_super] = ACTIONS(1469),
    [sym_crate] = ACTIONS(1469),
    [sym_metavariable] = ACTIONS(1467),
    [sym__raw_string_literal_start] = ACTIONS(1467),
    [sym_float_literal] = ACTIONS(1467),
  },
  [474] = {
    [sym_attribute_item] = STATE(1468),
    [sym_inner_attribute_item] = STATE(1468),
    [sym_bracketed_type] = STATE(3553),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3287),
    [sym_macro_invocation] = STATE(2757),
    [sym_scoped_identifier] = STATE(2151),
    [sym_scoped_type_identifier] = STATE(2982),
    [sym_match_arm] = STATE(1469),
    [sym_last_match_arm] = STATE(3342),
    [sym_match_pattern] = STATE(3352),
    [sym_const_block] = STATE(2757),
    [sym__pattern] = STATE(2907),
    [sym_tuple_pattern] = STATE(2757),
    [sym_slice_pattern] = STATE(2757),
    [sym_tuple_struct_pattern] = STATE(2757),
    [sym_struct_pattern] = STATE(2757),
    [sym_remaining_field_pattern] = STATE(2757),
    [sym_mut_pattern] = STATE(2757),
    [sym_range_pattern] = STATE(2757),
    [sym_ref_pattern] = STATE(2757),
    [sym_captured_pattern] = STATE(2757),
    [sym_reference_pattern] = STATE(2757),
    [sym_or_pattern] = STATE(2757),
    [sym__literal_pattern] = STATE(2330),
    [sym_negative_literal] = STATE(2364),
    [sym_string_literal] = STATE(2364),
    [sym_raw_string_literal] = STATE(2364),
    [sym_boolean_literal] = STATE(2364),
    [sym_line_comment] = STATE(474),
    [sym_block_comment] = STATE(474),
    [aux_sym_match_block_repeat1] = STATE(483),
    [aux_sym_match_arm_repeat1] = STATE(767),
    [sym_identifier] = ACTIONS(1645),
    [anon_sym_LPAREN] = ACTIONS(1647),
    [anon_sym_LBRACK] = ACTIONS(1649),
    [anon_sym_RBRACE] = ACTIONS(1695),
    [anon_sym_u8] = ACTIONS(1653),
    [anon_sym_i8] = ACTIONS(1653),
    [anon_sym_u16] = ACTIONS(1653),
    [anon_sym_i16] = ACTIONS(1653),
    [anon_sym_u32] = ACTIONS(1653),
    [anon_sym_i32] = ACTIONS(1653),
    [anon_sym_u64] = ACTIONS(1653),
    [anon_sym_i64] = ACTIONS(1653),
    [anon_sym_u128] = ACTIONS(1653),
    [anon_sym_i128] = ACTIONS(1653),
    [anon_sym_isize] = ACTIONS(1653),
    [anon_sym_usize] = ACTIONS(1653),
    [anon_sym_f32] = ACTIONS(1653),
    [anon_sym_f64] = ACTIONS(1653),
    [anon_sym_bool] = ACTIONS(1653),
    [anon_sym_str] = ACTIONS(1653),
    [anon_sym_char] = ACTIONS(1653),
    [anon_sym_DASH] = ACTIONS(1655),
    [anon_sym_AMP] = ACTIONS(1657),
    [anon_sym_PIPE] = ACTIONS(1659),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1661),
    [anon_sym_DOT_DOT] = ACTIONS(1663),
    [anon_sym_COLON_COLON] = ACTIONS(1665),
    [anon_sym_POUND] = ACTIONS(1667),
    [anon_sym_const] = ACTIONS(1669),
    [anon_sym_default] = ACTIONS(1671),
    [anon_sym_gen] = ACTIONS(1671),
    [anon_sym_union] = ACTIONS(1671),
    [anon_sym_ref] = ACTIONS(1673),
    [sym_mutable_specifier] = ACTIONS(1675),
    [sym_integer_literal] = ACTIONS(1677),
    [aux_sym_string_literal_token1] = ACTIONS(1679),
    [sym_char_literal] = ACTIONS(1677),
    [anon_sym_true] = ACTIONS(1681),
    [anon_sym_false] = ACTIONS(1681),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1683),
    [sym_super] = ACTIONS(1683),
    [sym_crate] = ACTIONS(1683),
    [sym_metavariable] = ACTIONS(1685),
    [sym__raw_string_literal_start] = ACTIONS(1687),
    [sym_float_literal] = ACTIONS(1677),
  },
  [475] = {
    [sym_attribute_item] = STATE(1468),
    [sym_inner_attribute_item] = STATE(1468),
    [sym_bracketed_type] = STATE(3553),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3287),
    [sym_macro_invocation] = STATE(2757),
    [sym_scoped_identifier] = STATE(2151),
    [sym_scoped_type_identifier] = STATE(2982),
    [sym_match_arm] = STATE(1469),
    [sym_last_match_arm] = STATE(3506),
    [sym_match_pattern] = STATE(3352),
    [sym_const_block] = STATE(2757),
    [sym__pattern] = STATE(2907),
    [sym_tuple_pattern] = STATE(2757),
    [sym_slice_pattern] = STATE(2757),
    [sym_tuple_struct_pattern] = STATE(2757),
    [sym_struct_pattern] = STATE(2757),
    [sym_remaining_field_pattern] = STATE(2757),
    [sym_mut_pattern] = STATE(2757),
    [sym_range_pattern] = STATE(2757),
    [sym_ref_pattern] = STATE(2757),
    [sym_captured_pattern] = STATE(2757),
    [sym_reference_pattern] = STATE(2757),
    [sym_or_pattern] = STATE(2757),
    [sym__literal_pattern] = STATE(2330),
    [sym_negative_literal] = STATE(2364),
    [sym_string_literal] = STATE(2364),
    [sym_raw_string_literal] = STATE(2364),
    [sym_boolean_literal] = STATE(2364),
    [sym_line_comment] = STATE(475),
    [sym_block_comment] = STATE(475),
    [aux_sym_match_block_repeat1] = STATE(485),
    [aux_sym_match_arm_repeat1] = STATE(767),
    [sym_identifier] = ACTIONS(1645),
    [anon_sym_LPAREN] = ACTIONS(1647),
    [anon_sym_LBRACK] = ACTIONS(1649),
    [anon_sym_RBRACE] = ACTIONS(1697),
    [anon_sym_u8] = ACTIONS(1653),
    [anon_sym_i8] = ACTIONS(1653),
    [anon_sym_u16] = ACTIONS(1653),
    [anon_sym_i16] = ACTIONS(1653),
    [anon_sym_u32] = ACTIONS(1653),
    [anon_sym_i32] = ACTIONS(1653),
    [anon_sym_u64] = ACTIONS(1653),
    [anon_sym_i64] = ACTIONS(1653),
    [anon_sym_u128] = ACTIONS(1653),
    [anon_sym_i128] = ACTIONS(1653),
    [anon_sym_isize] = ACTIONS(1653),
    [anon_sym_usize] = ACTIONS(1653),
    [anon_sym_f32] = ACTIONS(1653),
    [anon_sym_f64] = ACTIONS(1653),
    [anon_sym_bool] = ACTIONS(1653),
    [anon_sym_str] = ACTIONS(1653),
    [anon_sym_char] = ACTIONS(1653),
    [anon_sym_DASH] = ACTIONS(1655),
    [anon_sym_AMP] = ACTIONS(1657),
    [anon_sym_PIPE] = ACTIONS(1659),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1661),
    [anon_sym_DOT_DOT] = ACTIONS(1663),
    [anon_sym_COLON_COLON] = ACTIONS(1665),
    [anon_sym_POUND] = ACTIONS(1667),
    [anon_sym_const] = ACTIONS(1669),
    [anon_sym_default] = ACTIONS(1671),
    [anon_sym_gen] = ACTIONS(1671),
    [anon_sym_union] = ACTIONS(1671),
    [anon_sym_ref] = ACTIONS(1673),
    [sym_mutable_specifier] = ACTIONS(1675),
    [sym_integer_literal] = ACTIONS(1677),
    [aux_sym_string_literal_token1] = ACTIONS(1679),
    [sym_char_literal] = ACTIONS(1677),
    [anon_sym_true] = ACTIONS(1681),
    [anon_sym_false] = ACTIONS(1681),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1683),
    [sym_super] = ACTIONS(1683),
    [sym_crate] = ACTIONS(1683),
    [sym_metavariable] = ACTIONS(1685),
    [sym__raw_string_literal_start] = ACTIONS(1687),
    [sym_float_literal] = ACTIONS(1677),
  },
  [476] = {
    [sym_line_comment] = STATE(476),
    [sym_block_comment] = STATE(476),
    [sym_identifier] = ACTIONS(1481),
    [anon_sym_LPAREN] = ACTIONS(1479),
    [anon_sym_LBRACK] = ACTIONS(1479),
    [anon_sym_RBRACE] = ACTIONS(1479),
    [anon_sym_PLUS] = ACTIONS(1481),
    [anon_sym_STAR] = ACTIONS(1481),
    [anon_sym_QMARK] = ACTIONS(1479),
    [anon_sym_u8] = ACTIONS(1481),
    [anon_sym_i8] = ACTIONS(1481),
    [anon_sym_u16] = ACTIONS(1481),
    [anon_sym_i16] = ACTIONS(1481),
    [anon_sym_u32] = ACTIONS(1481),
    [anon_sym_i32] = ACTIONS(1481),
    [anon_sym_u64] = ACTIONS(1481),
    [anon_sym_i64] = ACTIONS(1481),
    [anon_sym_u128] = ACTIONS(1481),
    [anon_sym_i128] = ACTIONS(1481),
    [anon_sym_isize] = ACTIONS(1481),
    [anon_sym_usize] = ACTIONS(1481),
    [anon_sym_f32] = ACTIONS(1481),
    [anon_sym_f64] = ACTIONS(1481),
    [anon_sym_bool] = ACTIONS(1481),
    [anon_sym_str] = ACTIONS(1481),
    [anon_sym_char] = ACTIONS(1481),
    [anon_sym_DASH] = ACTIONS(1481),
    [anon_sym_SLASH] = ACTIONS(1481),
    [anon_sym_PERCENT] = ACTIONS(1481),
    [anon_sym_CARET] = ACTIONS(1481),
    [anon_sym_AMP] = ACTIONS(1481),
    [anon_sym_PIPE] = ACTIONS(1481),
    [anon_sym_AMP_AMP] = ACTIONS(1479),
    [anon_sym_PIPE_PIPE] = ACTIONS(1479),
    [anon_sym_LT_LT] = ACTIONS(1481),
    [anon_sym_GT_GT] = ACTIONS(1481),
    [anon_sym_PLUS_EQ] = ACTIONS(1479),
    [anon_sym_DASH_EQ] = ACTIONS(1479),
    [anon_sym_STAR_EQ] = ACTIONS(1479),
    [anon_sym_SLASH_EQ] = ACTIONS(1479),
    [anon_sym_PERCENT_EQ] = ACTIONS(1479),
    [anon_sym_CARET_EQ] = ACTIONS(1479),
    [anon_sym_AMP_EQ] = ACTIONS(1479),
    [anon_sym_PIPE_EQ] = ACTIONS(1479),
    [anon_sym_LT_LT_EQ] = ACTIONS(1479),
    [anon_sym_GT_GT_EQ] = ACTIONS(1479),
    [anon_sym_EQ] = ACTIONS(1481),
    [anon_sym_EQ_EQ] = ACTIONS(1479),
    [anon_sym_BANG_EQ] = ACTIONS(1479),
    [anon_sym_GT] = ACTIONS(1481),
    [anon_sym_LT] = ACTIONS(1481),
    [anon_sym_GT_EQ] = ACTIONS(1479),
    [anon_sym_LT_EQ] = ACTIONS(1479),
    [anon_sym__] = ACTIONS(1481),
    [anon_sym_DOT] = ACTIONS(1481),
    [anon_sym_DOT_DOT] = ACTIONS(1481),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1479),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1479),
    [anon_sym_COMMA] = ACTIONS(1479),
    [anon_sym_COLON_COLON] = ACTIONS(1479),
    [anon_sym_POUND] = ACTIONS(1479),
    [anon_sym_as] = ACTIONS(1481),
    [anon_sym_const] = ACTIONS(1481),
    [anon_sym_default] = ACTIONS(1481),
    [anon_sym_gen] = ACTIONS(1481),
    [anon_sym_union] = ACTIONS(1481),
    [anon_sym_ref] = ACTIONS(1481),
    [sym_mutable_specifier] = ACTIONS(1481),
    [sym_integer_literal] = ACTIONS(1479),
    [aux_sym_string_literal_token1] = ACTIONS(1479),
    [sym_char_literal] = ACTIONS(1479),
    [anon_sym_true] = ACTIONS(1481),
    [anon_sym_false] = ACTIONS(1481),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1481),
    [sym_super] = ACTIONS(1481),
    [sym_crate] = ACTIONS(1481),
    [sym_metavariable] = ACTIONS(1479),
    [sym__raw_string_literal_start] = ACTIONS(1479),
    [sym_float_literal] = ACTIONS(1479),
  },
  [477] = {
    [sym_line_comment] = STATE(477),
    [sym_block_comment] = STATE(477),
    [sym_identifier] = ACTIONS(1435),
    [anon_sym_LPAREN] = ACTIONS(1433),
    [anon_sym_LBRACK] = ACTIONS(1433),
    [anon_sym_RBRACE] = ACTIONS(1433),
    [anon_sym_PLUS] = ACTIONS(1435),
    [anon_sym_STAR] = ACTIONS(1435),
    [anon_sym_QMARK] = ACTIONS(1433),
    [anon_sym_u8] = ACTIONS(1435),
    [anon_sym_i8] = ACTIONS(1435),
    [anon_sym_u16] = ACTIONS(1435),
    [anon_sym_i16] = ACTIONS(1435),
    [anon_sym_u32] = ACTIONS(1435),
    [anon_sym_i32] = ACTIONS(1435),
    [anon_sym_u64] = ACTIONS(1435),
    [anon_sym_i64] = ACTIONS(1435),
    [anon_sym_u128] = ACTIONS(1435),
    [anon_sym_i128] = ACTIONS(1435),
    [anon_sym_isize] = ACTIONS(1435),
    [anon_sym_usize] = ACTIONS(1435),
    [anon_sym_f32] = ACTIONS(1435),
    [anon_sym_f64] = ACTIONS(1435),
    [anon_sym_bool] = ACTIONS(1435),
    [anon_sym_str] = ACTIONS(1435),
    [anon_sym_char] = ACTIONS(1435),
    [anon_sym_DASH] = ACTIONS(1435),
    [anon_sym_SLASH] = ACTIONS(1435),
    [anon_sym_PERCENT] = ACTIONS(1435),
    [anon_sym_CARET] = ACTIONS(1435),
    [anon_sym_AMP] = ACTIONS(1435),
    [anon_sym_PIPE] = ACTIONS(1435),
    [anon_sym_AMP_AMP] = ACTIONS(1433),
    [anon_sym_PIPE_PIPE] = ACTIONS(1433),
    [anon_sym_LT_LT] = ACTIONS(1435),
    [anon_sym_GT_GT] = ACTIONS(1435),
    [anon_sym_PLUS_EQ] = ACTIONS(1433),
    [anon_sym_DASH_EQ] = ACTIONS(1433),
    [anon_sym_STAR_EQ] = ACTIONS(1433),
    [anon_sym_SLASH_EQ] = ACTIONS(1433),
    [anon_sym_PERCENT_EQ] = ACTIONS(1433),
    [anon_sym_CARET_EQ] = ACTIONS(1433),
    [anon_sym_AMP_EQ] = ACTIONS(1433),
    [anon_sym_PIPE_EQ] = ACTIONS(1433),
    [anon_sym_LT_LT_EQ] = ACTIONS(1433),
    [anon_sym_GT_GT_EQ] = ACTIONS(1433),
    [anon_sym_EQ] = ACTIONS(1435),
    [anon_sym_EQ_EQ] = ACTIONS(1433),
    [anon_sym_BANG_EQ] = ACTIONS(1433),
    [anon_sym_GT] = ACTIONS(1435),
    [anon_sym_LT] = ACTIONS(1435),
    [anon_sym_GT_EQ] = ACTIONS(1433),
    [anon_sym_LT_EQ] = ACTIONS(1433),
    [anon_sym__] = ACTIONS(1435),
    [anon_sym_DOT] = ACTIONS(1435),
    [anon_sym_DOT_DOT] = ACTIONS(1435),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1433),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1433),
    [anon_sym_COMMA] = ACTIONS(1433),
    [anon_sym_COLON_COLON] = ACTIONS(1433),
    [anon_sym_POUND] = ACTIONS(1433),
    [anon_sym_as] = ACTIONS(1435),
    [anon_sym_const] = ACTIONS(1435),
    [anon_sym_default] = ACTIONS(1435),
    [anon_sym_gen] = ACTIONS(1435),
    [anon_sym_union] = ACTIONS(1435),
    [anon_sym_ref] = ACTIONS(1435),
    [sym_mutable_specifier] = ACTIONS(1435),
    [sym_integer_literal] = ACTIONS(1433),
    [aux_sym_string_literal_token1] = ACTIONS(1433),
    [sym_char_literal] = ACTIONS(1433),
    [anon_sym_true] = ACTIONS(1435),
    [anon_sym_false] = ACTIONS(1435),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1435),
    [sym_super] = ACTIONS(1435),
    [sym_crate] = ACTIONS(1435),
    [sym_metavariable] = ACTIONS(1433),
    [sym__raw_string_literal_start] = ACTIONS(1433),
    [sym_float_literal] = ACTIONS(1433),
  },
  [478] = {
    [sym_line_comment] = STATE(478),
    [sym_block_comment] = STATE(478),
    [sym_identifier] = ACTIONS(1489),
    [anon_sym_LPAREN] = ACTIONS(1487),
    [anon_sym_LBRACK] = ACTIONS(1487),
    [anon_sym_RBRACE] = ACTIONS(1487),
    [anon_sym_PLUS] = ACTIONS(1489),
    [anon_sym_STAR] = ACTIONS(1489),
    [anon_sym_QMARK] = ACTIONS(1487),
    [anon_sym_u8] = ACTIONS(1489),
    [anon_sym_i8] = ACTIONS(1489),
    [anon_sym_u16] = ACTIONS(1489),
    [anon_sym_i16] = ACTIONS(1489),
    [anon_sym_u32] = ACTIONS(1489),
    [anon_sym_i32] = ACTIONS(1489),
    [anon_sym_u64] = ACTIONS(1489),
    [anon_sym_i64] = ACTIONS(1489),
    [anon_sym_u128] = ACTIONS(1489),
    [anon_sym_i128] = ACTIONS(1489),
    [anon_sym_isize] = ACTIONS(1489),
    [anon_sym_usize] = ACTIONS(1489),
    [anon_sym_f32] = ACTIONS(1489),
    [anon_sym_f64] = ACTIONS(1489),
    [anon_sym_bool] = ACTIONS(1489),
    [anon_sym_str] = ACTIONS(1489),
    [anon_sym_char] = ACTIONS(1489),
    [anon_sym_DASH] = ACTIONS(1489),
    [anon_sym_SLASH] = ACTIONS(1489),
    [anon_sym_PERCENT] = ACTIONS(1489),
    [anon_sym_CARET] = ACTIONS(1489),
    [anon_sym_AMP] = ACTIONS(1489),
    [anon_sym_PIPE] = ACTIONS(1489),
    [anon_sym_AMP_AMP] = ACTIONS(1487),
    [anon_sym_PIPE_PIPE] = ACTIONS(1487),
    [anon_sym_LT_LT] = ACTIONS(1489),
    [anon_sym_GT_GT] = ACTIONS(1489),
    [anon_sym_PLUS_EQ] = ACTIONS(1487),
    [anon_sym_DASH_EQ] = ACTIONS(1487),
    [anon_sym_STAR_EQ] = ACTIONS(1487),
    [anon_sym_SLASH_EQ] = ACTIONS(1487),
    [anon_sym_PERCENT_EQ] = ACTIONS(1487),
    [anon_sym_CARET_EQ] = ACTIONS(1487),
    [anon_sym_AMP_EQ] = ACTIONS(1487),
    [anon_sym_PIPE_EQ] = ACTIONS(1487),
    [anon_sym_LT_LT_EQ] = ACTIONS(1487),
    [anon_sym_GT_GT_EQ] = ACTIONS(1487),
    [anon_sym_EQ] = ACTIONS(1489),
    [anon_sym_EQ_EQ] = ACTIONS(1487),
    [anon_sym_BANG_EQ] = ACTIONS(1487),
    [anon_sym_GT] = ACTIONS(1489),
    [anon_sym_LT] = ACTIONS(1489),
    [anon_sym_GT_EQ] = ACTIONS(1487),
    [anon_sym_LT_EQ] = ACTIONS(1487),
    [anon_sym__] = ACTIONS(1489),
    [anon_sym_DOT] = ACTIONS(1489),
    [anon_sym_DOT_DOT] = ACTIONS(1489),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1487),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1487),
    [anon_sym_COMMA] = ACTIONS(1487),
    [anon_sym_COLON_COLON] = ACTIONS(1487),
    [anon_sym_POUND] = ACTIONS(1487),
    [anon_sym_as] = ACTIONS(1489),
    [anon_sym_const] = ACTIONS(1489),
    [anon_sym_default] = ACTIONS(1489),
    [anon_sym_gen] = ACTIONS(1489),
    [anon_sym_union] = ACTIONS(1489),
    [anon_sym_ref] = ACTIONS(1489),
    [sym_mutable_specifier] = ACTIONS(1489),
    [sym_integer_literal] = ACTIONS(1487),
    [aux_sym_string_literal_token1] = ACTIONS(1487),
    [sym_char_literal] = ACTIONS(1487),
    [anon_sym_true] = ACTIONS(1489),
    [anon_sym_false] = ACTIONS(1489),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1489),
    [sym_super] = ACTIONS(1489),
    [sym_crate] = ACTIONS(1489),
    [sym_metavariable] = ACTIONS(1487),
    [sym__raw_string_literal_start] = ACTIONS(1487),
    [sym_float_literal] = ACTIONS(1487),
  },
  [479] = {
    [sym_line_comment] = STATE(479),
    [sym_block_comment] = STATE(479),
    [sym_identifier] = ACTIONS(1409),
    [anon_sym_LPAREN] = ACTIONS(1407),
    [anon_sym_LBRACK] = ACTIONS(1407),
    [anon_sym_RBRACE] = ACTIONS(1407),
    [anon_sym_PLUS] = ACTIONS(1409),
    [anon_sym_STAR] = ACTIONS(1409),
    [anon_sym_QMARK] = ACTIONS(1407),
    [anon_sym_u8] = ACTIONS(1409),
    [anon_sym_i8] = ACTIONS(1409),
    [anon_sym_u16] = ACTIONS(1409),
    [anon_sym_i16] = ACTIONS(1409),
    [anon_sym_u32] = ACTIONS(1409),
    [anon_sym_i32] = ACTIONS(1409),
    [anon_sym_u64] = ACTIONS(1409),
    [anon_sym_i64] = ACTIONS(1409),
    [anon_sym_u128] = ACTIONS(1409),
    [anon_sym_i128] = ACTIONS(1409),
    [anon_sym_isize] = ACTIONS(1409),
    [anon_sym_usize] = ACTIONS(1409),
    [anon_sym_f32] = ACTIONS(1409),
    [anon_sym_f64] = ACTIONS(1409),
    [anon_sym_bool] = ACTIONS(1409),
    [anon_sym_str] = ACTIONS(1409),
    [anon_sym_char] = ACTIONS(1409),
    [anon_sym_DASH] = ACTIONS(1409),
    [anon_sym_SLASH] = ACTIONS(1409),
    [anon_sym_PERCENT] = ACTIONS(1409),
    [anon_sym_CARET] = ACTIONS(1409),
    [anon_sym_AMP] = ACTIONS(1409),
    [anon_sym_PIPE] = ACTIONS(1409),
    [anon_sym_AMP_AMP] = ACTIONS(1407),
    [anon_sym_PIPE_PIPE] = ACTIONS(1407),
    [anon_sym_LT_LT] = ACTIONS(1409),
    [anon_sym_GT_GT] = ACTIONS(1409),
    [anon_sym_PLUS_EQ] = ACTIONS(1407),
    [anon_sym_DASH_EQ] = ACTIONS(1407),
    [anon_sym_STAR_EQ] = ACTIONS(1407),
    [anon_sym_SLASH_EQ] = ACTIONS(1407),
    [anon_sym_PERCENT_EQ] = ACTIONS(1407),
    [anon_sym_CARET_EQ] = ACTIONS(1407),
    [anon_sym_AMP_EQ] = ACTIONS(1407),
    [anon_sym_PIPE_EQ] = ACTIONS(1407),
    [anon_sym_LT_LT_EQ] = ACTIONS(1407),
    [anon_sym_GT_GT_EQ] = ACTIONS(1407),
    [anon_sym_EQ] = ACTIONS(1409),
    [anon_sym_EQ_EQ] = ACTIONS(1407),
    [anon_sym_BANG_EQ] = ACTIONS(1407),
    [anon_sym_GT] = ACTIONS(1409),
    [anon_sym_LT] = ACTIONS(1409),
    [anon_sym_GT_EQ] = ACTIONS(1407),
    [anon_sym_LT_EQ] = ACTIONS(1407),
    [anon_sym__] = ACTIONS(1409),
    [anon_sym_DOT] = ACTIONS(1409),
    [anon_sym_DOT_DOT] = ACTIONS(1409),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1407),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1407),
    [anon_sym_COMMA] = ACTIONS(1407),
    [anon_sym_COLON_COLON] = ACTIONS(1407),
    [anon_sym_POUND] = ACTIONS(1407),
    [anon_sym_as] = ACTIONS(1409),
    [anon_sym_const] = ACTIONS(1409),
    [anon_sym_default] = ACTIONS(1409),
    [anon_sym_gen] = ACTIONS(1409),
    [anon_sym_union] = ACTIONS(1409),
    [anon_sym_ref] = ACTIONS(1409),
    [sym_mutable_specifier] = ACTIONS(1409),
    [sym_integer_literal] = ACTIONS(1407),
    [aux_sym_string_literal_token1] = ACTIONS(1407),
    [sym_char_literal] = ACTIONS(1407),
    [anon_sym_true] = ACTIONS(1409),
    [anon_sym_false] = ACTIONS(1409),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1409),
    [sym_super] = ACTIONS(1409),
    [sym_crate] = ACTIONS(1409),
    [sym_metavariable] = ACTIONS(1407),
    [sym__raw_string_literal_start] = ACTIONS(1407),
    [sym_float_literal] = ACTIONS(1407),
  },
  [480] = {
    [sym_line_comment] = STATE(480),
    [sym_block_comment] = STATE(480),
    [sym_identifier] = ACTIONS(1439),
    [anon_sym_LPAREN] = ACTIONS(1437),
    [anon_sym_LBRACK] = ACTIONS(1437),
    [anon_sym_RBRACE] = ACTIONS(1437),
    [anon_sym_PLUS] = ACTIONS(1439),
    [anon_sym_STAR] = ACTIONS(1439),
    [anon_sym_QMARK] = ACTIONS(1437),
    [anon_sym_u8] = ACTIONS(1439),
    [anon_sym_i8] = ACTIONS(1439),
    [anon_sym_u16] = ACTIONS(1439),
    [anon_sym_i16] = ACTIONS(1439),
    [anon_sym_u32] = ACTIONS(1439),
    [anon_sym_i32] = ACTIONS(1439),
    [anon_sym_u64] = ACTIONS(1439),
    [anon_sym_i64] = ACTIONS(1439),
    [anon_sym_u128] = ACTIONS(1439),
    [anon_sym_i128] = ACTIONS(1439),
    [anon_sym_isize] = ACTIONS(1439),
    [anon_sym_usize] = ACTIONS(1439),
    [anon_sym_f32] = ACTIONS(1439),
    [anon_sym_f64] = ACTIONS(1439),
    [anon_sym_bool] = ACTIONS(1439),
    [anon_sym_str] = ACTIONS(1439),
    [anon_sym_char] = ACTIONS(1439),
    [anon_sym_DASH] = ACTIONS(1439),
    [anon_sym_SLASH] = ACTIONS(1439),
    [anon_sym_PERCENT] = ACTIONS(1439),
    [anon_sym_CARET] = ACTIONS(1439),
    [anon_sym_AMP] = ACTIONS(1439),
    [anon_sym_PIPE] = ACTIONS(1439),
    [anon_sym_AMP_AMP] = ACTIONS(1437),
    [anon_sym_PIPE_PIPE] = ACTIONS(1437),
    [anon_sym_LT_LT] = ACTIONS(1439),
    [anon_sym_GT_GT] = ACTIONS(1439),
    [anon_sym_PLUS_EQ] = ACTIONS(1437),
    [anon_sym_DASH_EQ] = ACTIONS(1437),
    [anon_sym_STAR_EQ] = ACTIONS(1437),
    [anon_sym_SLASH_EQ] = ACTIONS(1437),
    [anon_sym_PERCENT_EQ] = ACTIONS(1437),
    [anon_sym_CARET_EQ] = ACTIONS(1437),
    [anon_sym_AMP_EQ] = ACTIONS(1437),
    [anon_sym_PIPE_EQ] = ACTIONS(1437),
    [anon_sym_LT_LT_EQ] = ACTIONS(1437),
    [anon_sym_GT_GT_EQ] = ACTIONS(1437),
    [anon_sym_EQ] = ACTIONS(1439),
    [anon_sym_EQ_EQ] = ACTIONS(1437),
    [anon_sym_BANG_EQ] = ACTIONS(1437),
    [anon_sym_GT] = ACTIONS(1439),
    [anon_sym_LT] = ACTIONS(1439),
    [anon_sym_GT_EQ] = ACTIONS(1437),
    [anon_sym_LT_EQ] = ACTIONS(1437),
    [anon_sym__] = ACTIONS(1439),
    [anon_sym_DOT] = ACTIONS(1439),
    [anon_sym_DOT_DOT] = ACTIONS(1439),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1437),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1437),
    [anon_sym_COMMA] = ACTIONS(1437),
    [anon_sym_COLON_COLON] = ACTIONS(1437),
    [anon_sym_POUND] = ACTIONS(1437),
    [anon_sym_as] = ACTIONS(1439),
    [anon_sym_const] = ACTIONS(1439),
    [anon_sym_default] = ACTIONS(1439),
    [anon_sym_gen] = ACTIONS(1439),
    [anon_sym_union] = ACTIONS(1439),
    [anon_sym_ref] = ACTIONS(1439),
    [sym_mutable_specifier] = ACTIONS(1439),
    [sym_integer_literal] = ACTIONS(1437),
    [aux_sym_string_literal_token1] = ACTIONS(1437),
    [sym_char_literal] = ACTIONS(1437),
    [anon_sym_true] = ACTIONS(1439),
    [anon_sym_false] = ACTIONS(1439),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1439),
    [sym_super] = ACTIONS(1439),
    [sym_crate] = ACTIONS(1439),
    [sym_metavariable] = ACTIONS(1437),
    [sym__raw_string_literal_start] = ACTIONS(1437),
    [sym_float_literal] = ACTIONS(1437),
  },
  [481] = {
    [sym_line_comment] = STATE(481),
    [sym_block_comment] = STATE(481),
    [sym_identifier] = ACTIONS(1279),
    [anon_sym_LPAREN] = ACTIONS(1277),
    [anon_sym_LBRACK] = ACTIONS(1277),
    [anon_sym_RBRACE] = ACTIONS(1277),
    [anon_sym_PLUS] = ACTIONS(1279),
    [anon_sym_STAR] = ACTIONS(1279),
    [anon_sym_QMARK] = ACTIONS(1277),
    [anon_sym_u8] = ACTIONS(1279),
    [anon_sym_i8] = ACTIONS(1279),
    [anon_sym_u16] = ACTIONS(1279),
    [anon_sym_i16] = ACTIONS(1279),
    [anon_sym_u32] = ACTIONS(1279),
    [anon_sym_i32] = ACTIONS(1279),
    [anon_sym_u64] = ACTIONS(1279),
    [anon_sym_i64] = ACTIONS(1279),
    [anon_sym_u128] = ACTIONS(1279),
    [anon_sym_i128] = ACTIONS(1279),
    [anon_sym_isize] = ACTIONS(1279),
    [anon_sym_usize] = ACTIONS(1279),
    [anon_sym_f32] = ACTIONS(1279),
    [anon_sym_f64] = ACTIONS(1279),
    [anon_sym_bool] = ACTIONS(1279),
    [anon_sym_str] = ACTIONS(1279),
    [anon_sym_char] = ACTIONS(1279),
    [anon_sym_DASH] = ACTIONS(1279),
    [anon_sym_SLASH] = ACTIONS(1279),
    [anon_sym_PERCENT] = ACTIONS(1279),
    [anon_sym_CARET] = ACTIONS(1279),
    [anon_sym_AMP] = ACTIONS(1279),
    [anon_sym_PIPE] = ACTIONS(1279),
    [anon_sym_AMP_AMP] = ACTIONS(1277),
    [anon_sym_PIPE_PIPE] = ACTIONS(1277),
    [anon_sym_LT_LT] = ACTIONS(1279),
    [anon_sym_GT_GT] = ACTIONS(1279),
    [anon_sym_PLUS_EQ] = ACTIONS(1277),
    [anon_sym_DASH_EQ] = ACTIONS(1277),
    [anon_sym_STAR_EQ] = ACTIONS(1277),
    [anon_sym_SLASH_EQ] = ACTIONS(1277),
    [anon_sym_PERCENT_EQ] = ACTIONS(1277),
    [anon_sym_CARET_EQ] = ACTIONS(1277),
    [anon_sym_AMP_EQ] = ACTIONS(1277),
    [anon_sym_PIPE_EQ] = ACTIONS(1277),
    [anon_sym_LT_LT_EQ] = ACTIONS(1277),
    [anon_sym_GT_GT_EQ] = ACTIONS(1277),
    [anon_sym_EQ] = ACTIONS(1279),
    [anon_sym_EQ_EQ] = ACTIONS(1277),
    [anon_sym_BANG_EQ] = ACTIONS(1277),
    [anon_sym_GT] = ACTIONS(1279),
    [anon_sym_LT] = ACTIONS(1279),
    [anon_sym_GT_EQ] = ACTIONS(1277),
    [anon_sym_LT_EQ] = ACTIONS(1277),
    [anon_sym__] = ACTIONS(1279),
    [anon_sym_DOT] = ACTIONS(1279),
    [anon_sym_DOT_DOT] = ACTIONS(1279),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1277),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1277),
    [anon_sym_COMMA] = ACTIONS(1277),
    [anon_sym_COLON_COLON] = ACTIONS(1277),
    [anon_sym_POUND] = ACTIONS(1277),
    [anon_sym_as] = ACTIONS(1279),
    [anon_sym_const] = ACTIONS(1279),
    [anon_sym_default] = ACTIONS(1279),
    [anon_sym_gen] = ACTIONS(1279),
    [anon_sym_union] = ACTIONS(1279),
    [anon_sym_ref] = ACTIONS(1279),
    [sym_mutable_specifier] = ACTIONS(1279),
    [sym_integer_literal] = ACTIONS(1277),
    [aux_sym_string_literal_token1] = ACTIONS(1277),
    [sym_char_literal] = ACTIONS(1277),
    [anon_sym_true] = ACTIONS(1279),
    [anon_sym_false] = ACTIONS(1279),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1279),
    [sym_super] = ACTIONS(1279),
    [sym_crate] = ACTIONS(1279),
    [sym_metavariable] = ACTIONS(1277),
    [sym__raw_string_literal_start] = ACTIONS(1277),
    [sym_float_literal] = ACTIONS(1277),
  },
  [482] = {
    [sym_line_comment] = STATE(482),
    [sym_block_comment] = STATE(482),
    [sym_identifier] = ACTIONS(1431),
    [anon_sym_LPAREN] = ACTIONS(1429),
    [anon_sym_LBRACK] = ACTIONS(1429),
    [anon_sym_RBRACE] = ACTIONS(1429),
    [anon_sym_PLUS] = ACTIONS(1431),
    [anon_sym_STAR] = ACTIONS(1431),
    [anon_sym_QMARK] = ACTIONS(1429),
    [anon_sym_u8] = ACTIONS(1431),
    [anon_sym_i8] = ACTIONS(1431),
    [anon_sym_u16] = ACTIONS(1431),
    [anon_sym_i16] = ACTIONS(1431),
    [anon_sym_u32] = ACTIONS(1431),
    [anon_sym_i32] = ACTIONS(1431),
    [anon_sym_u64] = ACTIONS(1431),
    [anon_sym_i64] = ACTIONS(1431),
    [anon_sym_u128] = ACTIONS(1431),
    [anon_sym_i128] = ACTIONS(1431),
    [anon_sym_isize] = ACTIONS(1431),
    [anon_sym_usize] = ACTIONS(1431),
    [anon_sym_f32] = ACTIONS(1431),
    [anon_sym_f64] = ACTIONS(1431),
    [anon_sym_bool] = ACTIONS(1431),
    [anon_sym_str] = ACTIONS(1431),
    [anon_sym_char] = ACTIONS(1431),
    [anon_sym_DASH] = ACTIONS(1431),
    [anon_sym_SLASH] = ACTIONS(1431),
    [anon_sym_PERCENT] = ACTIONS(1431),
    [anon_sym_CARET] = ACTIONS(1431),
    [anon_sym_AMP] = ACTIONS(1431),
    [anon_sym_PIPE] = ACTIONS(1431),
    [anon_sym_AMP_AMP] = ACTIONS(1429),
    [anon_sym_PIPE_PIPE] = ACTIONS(1429),
    [anon_sym_LT_LT] = ACTIONS(1431),
    [anon_sym_GT_GT] = ACTIONS(1431),
    [anon_sym_PLUS_EQ] = ACTIONS(1429),
    [anon_sym_DASH_EQ] = ACTIONS(1429),
    [anon_sym_STAR_EQ] = ACTIONS(1429),
    [anon_sym_SLASH_EQ] = ACTIONS(1429),
    [anon_sym_PERCENT_EQ] = ACTIONS(1429),
    [anon_sym_CARET_EQ] = ACTIONS(1429),
    [anon_sym_AMP_EQ] = ACTIONS(1429),
    [anon_sym_PIPE_EQ] = ACTIONS(1429),
    [anon_sym_LT_LT_EQ] = ACTIONS(1429),
    [anon_sym_GT_GT_EQ] = ACTIONS(1429),
    [anon_sym_EQ] = ACTIONS(1431),
    [anon_sym_EQ_EQ] = ACTIONS(1429),
    [anon_sym_BANG_EQ] = ACTIONS(1429),
    [anon_sym_GT] = ACTIONS(1431),
    [anon_sym_LT] = ACTIONS(1431),
    [anon_sym_GT_EQ] = ACTIONS(1429),
    [anon_sym_LT_EQ] = ACTIONS(1429),
    [anon_sym__] = ACTIONS(1431),
    [anon_sym_DOT] = ACTIONS(1431),
    [anon_sym_DOT_DOT] = ACTIONS(1431),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1429),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1429),
    [anon_sym_COMMA] = ACTIONS(1429),
    [anon_sym_COLON_COLON] = ACTIONS(1429),
    [anon_sym_POUND] = ACTIONS(1429),
    [anon_sym_as] = ACTIONS(1431),
    [anon_sym_const] = ACTIONS(1431),
    [anon_sym_default] = ACTIONS(1431),
    [anon_sym_gen] = ACTIONS(1431),
    [anon_sym_union] = ACTIONS(1431),
    [anon_sym_ref] = ACTIONS(1431),
    [sym_mutable_specifier] = ACTIONS(1431),
    [sym_integer_literal] = ACTIONS(1429),
    [aux_sym_string_literal_token1] = ACTIONS(1429),
    [sym_char_literal] = ACTIONS(1429),
    [anon_sym_true] = ACTIONS(1431),
    [anon_sym_false] = ACTIONS(1431),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1431),
    [sym_super] = ACTIONS(1431),
    [sym_crate] = ACTIONS(1431),
    [sym_metavariable] = ACTIONS(1429),
    [sym__raw_string_literal_start] = ACTIONS(1429),
    [sym_float_literal] = ACTIONS(1429),
  },
  [483] = {
    [sym_attribute_item] = STATE(1468),
    [sym_inner_attribute_item] = STATE(1468),
    [sym_bracketed_type] = STATE(3553),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3287),
    [sym_macro_invocation] = STATE(2757),
    [sym_scoped_identifier] = STATE(2151),
    [sym_scoped_type_identifier] = STATE(2982),
    [sym_match_arm] = STATE(1469),
    [sym_last_match_arm] = STATE(3534),
    [sym_match_pattern] = STATE(3352),
    [sym_const_block] = STATE(2757),
    [sym__pattern] = STATE(2907),
    [sym_tuple_pattern] = STATE(2757),
    [sym_slice_pattern] = STATE(2757),
    [sym_tuple_struct_pattern] = STATE(2757),
    [sym_struct_pattern] = STATE(2757),
    [sym_remaining_field_pattern] = STATE(2757),
    [sym_mut_pattern] = STATE(2757),
    [sym_range_pattern] = STATE(2757),
    [sym_ref_pattern] = STATE(2757),
    [sym_captured_pattern] = STATE(2757),
    [sym_reference_pattern] = STATE(2757),
    [sym_or_pattern] = STATE(2757),
    [sym__literal_pattern] = STATE(2330),
    [sym_negative_literal] = STATE(2364),
    [sym_string_literal] = STATE(2364),
    [sym_raw_string_literal] = STATE(2364),
    [sym_boolean_literal] = STATE(2364),
    [sym_line_comment] = STATE(483),
    [sym_block_comment] = STATE(483),
    [aux_sym_match_block_repeat1] = STATE(699),
    [aux_sym_match_arm_repeat1] = STATE(767),
    [sym_identifier] = ACTIONS(1645),
    [anon_sym_LPAREN] = ACTIONS(1647),
    [anon_sym_LBRACK] = ACTIONS(1649),
    [anon_sym_u8] = ACTIONS(1653),
    [anon_sym_i8] = ACTIONS(1653),
    [anon_sym_u16] = ACTIONS(1653),
    [anon_sym_i16] = ACTIONS(1653),
    [anon_sym_u32] = ACTIONS(1653),
    [anon_sym_i32] = ACTIONS(1653),
    [anon_sym_u64] = ACTIONS(1653),
    [anon_sym_i64] = ACTIONS(1653),
    [anon_sym_u128] = ACTIONS(1653),
    [anon_sym_i128] = ACTIONS(1653),
    [anon_sym_isize] = ACTIONS(1653),
    [anon_sym_usize] = ACTIONS(1653),
    [anon_sym_f32] = ACTIONS(1653),
    [anon_sym_f64] = ACTIONS(1653),
    [anon_sym_bool] = ACTIONS(1653),
    [anon_sym_str] = ACTIONS(1653),
    [anon_sym_char] = ACTIONS(1653),
    [anon_sym_DASH] = ACTIONS(1655),
    [anon_sym_AMP] = ACTIONS(1657),
    [anon_sym_PIPE] = ACTIONS(1659),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1661),
    [anon_sym_DOT_DOT] = ACTIONS(1663),
    [anon_sym_COLON_COLON] = ACTIONS(1665),
    [anon_sym_POUND] = ACTIONS(1667),
    [anon_sym_const] = ACTIONS(1669),
    [anon_sym_default] = ACTIONS(1671),
    [anon_sym_gen] = ACTIONS(1671),
    [anon_sym_union] = ACTIONS(1671),
    [anon_sym_ref] = ACTIONS(1673),
    [sym_mutable_specifier] = ACTIONS(1675),
    [sym_integer_literal] = ACTIONS(1677),
    [aux_sym_string_literal_token1] = ACTIONS(1679),
    [sym_char_literal] = ACTIONS(1677),
    [anon_sym_true] = ACTIONS(1681),
    [anon_sym_false] = ACTIONS(1681),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1683),
    [sym_super] = ACTIONS(1683),
    [sym_crate] = ACTIONS(1683),
    [sym_metavariable] = ACTIONS(1685),
    [sym__raw_string_literal_start] = ACTIONS(1687),
    [sym_float_literal] = ACTIONS(1677),
  },
  [484] = {
    [sym_attribute_item] = STATE(1468),
    [sym_inner_attribute_item] = STATE(1468),
    [sym_bracketed_type] = STATE(3553),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3287),
    [sym_macro_invocation] = STATE(2757),
    [sym_scoped_identifier] = STATE(2151),
    [sym_scoped_type_identifier] = STATE(2982),
    [sym_match_arm] = STATE(1469),
    [sym_last_match_arm] = STATE(3416),
    [sym_match_pattern] = STATE(3352),
    [sym_const_block] = STATE(2757),
    [sym__pattern] = STATE(2907),
    [sym_tuple_pattern] = STATE(2757),
    [sym_slice_pattern] = STATE(2757),
    [sym_tuple_struct_pattern] = STATE(2757),
    [sym_struct_pattern] = STATE(2757),
    [sym_remaining_field_pattern] = STATE(2757),
    [sym_mut_pattern] = STATE(2757),
    [sym_range_pattern] = STATE(2757),
    [sym_ref_pattern] = STATE(2757),
    [sym_captured_pattern] = STATE(2757),
    [sym_reference_pattern] = STATE(2757),
    [sym_or_pattern] = STATE(2757),
    [sym__literal_pattern] = STATE(2330),
    [sym_negative_literal] = STATE(2364),
    [sym_string_literal] = STATE(2364),
    [sym_raw_string_literal] = STATE(2364),
    [sym_boolean_literal] = STATE(2364),
    [sym_line_comment] = STATE(484),
    [sym_block_comment] = STATE(484),
    [aux_sym_match_block_repeat1] = STATE(699),
    [aux_sym_match_arm_repeat1] = STATE(767),
    [sym_identifier] = ACTIONS(1645),
    [anon_sym_LPAREN] = ACTIONS(1647),
    [anon_sym_LBRACK] = ACTIONS(1649),
    [anon_sym_u8] = ACTIONS(1653),
    [anon_sym_i8] = ACTIONS(1653),
    [anon_sym_u16] = ACTIONS(1653),
    [anon_sym_i16] = ACTIONS(1653),
    [anon_sym_u32] = ACTIONS(1653),
    [anon_sym_i32] = ACTIONS(1653),
    [anon_sym_u64] = ACTIONS(1653),
    [anon_sym_i64] = ACTIONS(1653),
    [anon_sym_u128] = ACTIONS(1653),
    [anon_sym_i128] = ACTIONS(1653),
    [anon_sym_isize] = ACTIONS(1653),
    [anon_sym_usize] = ACTIONS(1653),
    [anon_sym_f32] = ACTIONS(1653),
    [anon_sym_f64] = ACTIONS(1653),
    [anon_sym_bool] = ACTIONS(1653),
    [anon_sym_str] = ACTIONS(1653),
    [anon_sym_char] = ACTIONS(1653),
    [anon_sym_DASH] = ACTIONS(1655),
    [anon_sym_AMP] = ACTIONS(1657),
    [anon_sym_PIPE] = ACTIONS(1659),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1661),
    [anon_sym_DOT_DOT] = ACTIONS(1663),
    [anon_sym_COLON_COLON] = ACTIONS(1665),
    [anon_sym_POUND] = ACTIONS(1667),
    [anon_sym_const] = ACTIONS(1669),
    [anon_sym_default] = ACTIONS(1671),
    [anon_sym_gen] = ACTIONS(1671),
    [anon_sym_union] = ACTIONS(1671),
    [anon_sym_ref] = ACTIONS(1673),
    [sym_mutable_specifier] = ACTIONS(1675),
    [sym_integer_literal] = ACTIONS(1677),
    [aux_sym_string_literal_token1] = ACTIONS(1679),
    [sym_char_literal] = ACTIONS(1677),
    [anon_sym_true] = ACTIONS(1681),
    [anon_sym_false] = ACTIONS(1681),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1683),
    [sym_super] = ACTIONS(1683),
    [sym_crate] = ACTIONS(1683),
    [sym_metavariable] = ACTIONS(1685),
    [sym__raw_string_literal_start] = ACTIONS(1687),
    [sym_float_literal] = ACTIONS(1677),
  },
  [485] = {
    [sym_attribute_item] = STATE(1468),
    [sym_inner_attribute_item] = STATE(1468),
    [sym_bracketed_type] = STATE(3553),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3287),
    [sym_macro_invocation] = STATE(2757),
    [sym_scoped_identifier] = STATE(2151),
    [sym_scoped_type_identifier] = STATE(2982),
    [sym_match_arm] = STATE(1469),
    [sym_last_match_arm] = STATE(3374),
    [sym_match_pattern] = STATE(3352),
    [sym_const_block] = STATE(2757),
    [sym__pattern] = STATE(2907),
    [sym_tuple_pattern] = STATE(2757),
    [sym_slice_pattern] = STATE(2757),
    [sym_tuple_struct_pattern] = STATE(2757),
    [sym_struct_pattern] = STATE(2757),
    [sym_remaining_field_pattern] = STATE(2757),
    [sym_mut_pattern] = STATE(2757),
    [sym_range_pattern] = STATE(2757),
    [sym_ref_pattern] = STATE(2757),
    [sym_captured_pattern] = STATE(2757),
    [sym_reference_pattern] = STATE(2757),
    [sym_or_pattern] = STATE(2757),
    [sym__literal_pattern] = STATE(2330),
    [sym_negative_literal] = STATE(2364),
    [sym_string_literal] = STATE(2364),
    [sym_raw_string_literal] = STATE(2364),
    [sym_boolean_literal] = STATE(2364),
    [sym_line_comment] = STATE(485),
    [sym_block_comment] = STATE(485),
    [aux_sym_match_block_repeat1] = STATE(699),
    [aux_sym_match_arm_repeat1] = STATE(767),
    [sym_identifier] = ACTIONS(1645),
    [anon_sym_LPAREN] = ACTIONS(1647),
    [anon_sym_LBRACK] = ACTIONS(1649),
    [anon_sym_u8] = ACTIONS(1653),
    [anon_sym_i8] = ACTIONS(1653),
    [anon_sym_u16] = ACTIONS(1653),
    [anon_sym_i16] = ACTIONS(1653),
    [anon_sym_u32] = ACTIONS(1653),
    [anon_sym_i32] = ACTIONS(1653),
    [anon_sym_u64] = ACTIONS(1653),
    [anon_sym_i64] = ACTIONS(1653),
    [anon_sym_u128] = ACTIONS(1653),
    [anon_sym_i128] = ACTIONS(1653),
    [anon_sym_isize] = ACTIONS(1653),
    [anon_sym_usize] = ACTIONS(1653),
    [anon_sym_f32] = ACTIONS(1653),
    [anon_sym_f64] = ACTIONS(1653),
    [anon_sym_bool] = ACTIONS(1653),
    [anon_sym_str] = ACTIONS(1653),
    [anon_sym_char] = ACTIONS(1653),
    [anon_sym_DASH] = ACTIONS(1655),
    [anon_sym_AMP] = ACTIONS(1657),
    [anon_sym_PIPE] = ACTIONS(1659),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1661),
    [anon_sym_DOT_DOT] = ACTIONS(1663),
    [anon_sym_COLON_COLON] = ACTIONS(1665),
    [anon_sym_POUND] = ACTIONS(1667),
    [anon_sym_const] = ACTIONS(1669),
    [anon_sym_default] = ACTIONS(1671),
    [anon_sym_gen] = ACTIONS(1671),
    [anon_sym_union] = ACTIONS(1671),
    [anon_sym_ref] = ACTIONS(1673),
    [sym_mutable_specifier] = ACTIONS(1675),
    [sym_integer_literal] = ACTIONS(1677),
    [aux_sym_string_literal_token1] = ACTIONS(1679),
    [sym_char_literal] = ACTIONS(1677),
    [anon_sym_true] = ACTIONS(1681),
    [anon_sym_false] = ACTIONS(1681),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1683),
    [sym_super] = ACTIONS(1683),
    [sym_crate] = ACTIONS(1683),
    [sym_metavariable] = ACTIONS(1685),
    [sym__raw_string_literal_start] = ACTIONS(1687),
    [sym_float_literal] = ACTIONS(1677),
  },
  [486] = {
    [sym_attribute_item] = STATE(1468),
    [sym_inner_attribute_item] = STATE(1468),
    [sym_bracketed_type] = STATE(3553),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3287),
    [sym_macro_invocation] = STATE(2757),
    [sym_scoped_identifier] = STATE(2151),
    [sym_scoped_type_identifier] = STATE(2982),
    [sym_match_arm] = STATE(1469),
    [sym_last_match_arm] = STATE(3530),
    [sym_match_pattern] = STATE(3352),
    [sym_const_block] = STATE(2757),
    [sym__pattern] = STATE(2907),
    [sym_tuple_pattern] = STATE(2757),
    [sym_slice_pattern] = STATE(2757),
    [sym_tuple_struct_pattern] = STATE(2757),
    [sym_struct_pattern] = STATE(2757),
    [sym_remaining_field_pattern] = STATE(2757),
    [sym_mut_pattern] = STATE(2757),
    [sym_range_pattern] = STATE(2757),
    [sym_ref_pattern] = STATE(2757),
    [sym_captured_pattern] = STATE(2757),
    [sym_reference_pattern] = STATE(2757),
    [sym_or_pattern] = STATE(2757),
    [sym__literal_pattern] = STATE(2330),
    [sym_negative_literal] = STATE(2364),
    [sym_string_literal] = STATE(2364),
    [sym_raw_string_literal] = STATE(2364),
    [sym_boolean_literal] = STATE(2364),
    [sym_line_comment] = STATE(486),
    [sym_block_comment] = STATE(486),
    [aux_sym_match_block_repeat1] = STATE(699),
    [aux_sym_match_arm_repeat1] = STATE(767),
    [sym_identifier] = ACTIONS(1645),
    [anon_sym_LPAREN] = ACTIONS(1647),
    [anon_sym_LBRACK] = ACTIONS(1649),
    [anon_sym_u8] = ACTIONS(1653),
    [anon_sym_i8] = ACTIONS(1653),
    [anon_sym_u16] = ACTIONS(1653),
    [anon_sym_i16] = ACTIONS(1653),
    [anon_sym_u32] = ACTIONS(1653),
    [anon_sym_i32] = ACTIONS(1653),
    [anon_sym_u64] = ACTIONS(1653),
    [anon_sym_i64] = ACTIONS(1653),
    [anon_sym_u128] = ACTIONS(1653),
    [anon_sym_i128] = ACTIONS(1653),
    [anon_sym_isize] = ACTIONS(1653),
    [anon_sym_usize] = ACTIONS(1653),
    [anon_sym_f32] = ACTIONS(1653),
    [anon_sym_f64] = ACTIONS(1653),
    [anon_sym_bool] = ACTIONS(1653),
    [anon_sym_str] = ACTIONS(1653),
    [anon_sym_char] = ACTIONS(1653),
    [anon_sym_DASH] = ACTIONS(1655),
    [anon_sym_AMP] = ACTIONS(1657),
    [anon_sym_PIPE] = ACTIONS(1659),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1661),
    [anon_sym_DOT_DOT] = ACTIONS(1663),
    [anon_sym_COLON_COLON] = ACTIONS(1665),
    [anon_sym_POUND] = ACTIONS(1667),
    [anon_sym_const] = ACTIONS(1669),
    [anon_sym_default] = ACTIONS(1671),
    [anon_sym_gen] = ACTIONS(1671),
    [anon_sym_union] = ACTIONS(1671),
    [anon_sym_ref] = ACTIONS(1673),
    [sym_mutable_specifier] = ACTIONS(1675),
    [sym_integer_literal] = ACTIONS(1677),
    [aux_sym_string_literal_token1] = ACTIONS(1679),
    [sym_char_literal] = ACTIONS(1677),
    [anon_sym_true] = ACTIONS(1681),
    [anon_sym_false] = ACTIONS(1681),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1683),
    [sym_super] = ACTIONS(1683),
    [sym_crate] = ACTIONS(1683),
    [sym_metavariable] = ACTIONS(1685),
    [sym__raw_string_literal_start] = ACTIONS(1687),
    [sym_float_literal] = ACTIONS(1677),
  },
  [487] = {
    [sym_line_comment] = STATE(487),
    [sym_block_comment] = STATE(487),
    [ts_builtin_sym_end] = ACTIONS(1699),
    [sym_identifier] = ACTIONS(1701),
    [anon_sym_SEMI] = ACTIONS(1699),
    [anon_sym_macro_rules_BANG] = ACTIONS(1699),
    [anon_sym_LPAREN] = ACTIONS(1699),
    [anon_sym_LBRACK] = ACTIONS(1699),
    [anon_sym_LBRACE] = ACTIONS(1699),
    [anon_sym_RBRACE] = ACTIONS(1699),
    [anon_sym_STAR] = ACTIONS(1699),
    [anon_sym_u8] = ACTIONS(1701),
    [anon_sym_i8] = ACTIONS(1701),
    [anon_sym_u16] = ACTIONS(1701),
    [anon_sym_i16] = ACTIONS(1701),
    [anon_sym_u32] = ACTIONS(1701),
    [anon_sym_i32] = ACTIONS(1701),
    [anon_sym_u64] = ACTIONS(1701),
    [anon_sym_i64] = ACTIONS(1701),
    [anon_sym_u128] = ACTIONS(1701),
    [anon_sym_i128] = ACTIONS(1701),
    [anon_sym_isize] = ACTIONS(1701),
    [anon_sym_usize] = ACTIONS(1701),
    [anon_sym_f32] = ACTIONS(1701),
    [anon_sym_f64] = ACTIONS(1701),
    [anon_sym_bool] = ACTIONS(1701),
    [anon_sym_str] = ACTIONS(1701),
    [anon_sym_char] = ACTIONS(1701),
    [anon_sym_DASH] = ACTIONS(1699),
    [anon_sym_BANG] = ACTIONS(1699),
    [anon_sym_AMP] = ACTIONS(1699),
    [anon_sym_PIPE] = ACTIONS(1699),
    [anon_sym_LT] = ACTIONS(1699),
    [anon_sym_DOT_DOT] = ACTIONS(1699),
    [anon_sym_COLON_COLON] = ACTIONS(1699),
    [anon_sym_POUND] = ACTIONS(1699),
    [anon_sym_SQUOTE] = ACTIONS(1701),
    [anon_sym_async] = ACTIONS(1701),
    [anon_sym_break] = ACTIONS(1701),
    [anon_sym_const] = ACTIONS(1701),
    [anon_sym_continue] = ACTIONS(1701),
    [anon_sym_default] = ACTIONS(1701),
    [anon_sym_enum] = ACTIONS(1701),
    [anon_sym_fn] = ACTIONS(1701),
    [anon_sym_for] = ACTIONS(1701),
    [anon_sym_gen] = ACTIONS(1701),
    [anon_sym_if] = ACTIONS(1701),
    [anon_sym_impl] = ACTIONS(1701),
    [anon_sym_let] = ACTIONS(1701),
    [anon_sym_loop] = ACTIONS(1701),
    [anon_sym_match] = ACTIONS(1701),
    [anon_sym_mod] = ACTIONS(1701),
    [anon_sym_pub] = ACTIONS(1701),
    [anon_sym_return] = ACTIONS(1701),
    [anon_sym_static] = ACTIONS(1701),
    [anon_sym_struct] = ACTIONS(1701),
    [anon_sym_trait] = ACTIONS(1701),
    [anon_sym_type] = ACTIONS(1701),
    [anon_sym_union] = ACTIONS(1701),
    [anon_sym_unsafe] = ACTIONS(1701),
    [anon_sym_use] = ACTIONS(1701),
    [anon_sym_while] = ACTIONS(1701),
    [anon_sym_extern] = ACTIONS(1701),
    [anon_sym_yield] = ACTIONS(1701),
    [anon_sym_move] = ACTIONS(1701),
    [anon_sym_try] = ACTIONS(1701),
    [sym_integer_literal] = ACTIONS(1699),
    [aux_sym_string_literal_token1] = ACTIONS(1699),
    [sym_char_literal] = ACTIONS(1699),
    [anon_sym_true] = ACTIONS(1701),
    [anon_sym_false] = ACTIONS(1701),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1701),
    [sym_super] = ACTIONS(1701),
    [sym_crate] = ACTIONS(1701),
    [sym_metavariable] = ACTIONS(1699),
    [sym__raw_string_literal_start] = ACTIONS(1699),
    [sym_float_literal] = ACTIONS(1699),
  },
  [488] = {
    [sym_line_comment] = STATE(488),
    [sym_block_comment] = STATE(488),
    [ts_builtin_sym_end] = ACTIONS(1703),
    [sym_identifier] = ACTIONS(1705),
    [anon_sym_SEMI] = ACTIONS(1703),
    [anon_sym_macro_rules_BANG] = ACTIONS(1703),
    [anon_sym_LPAREN] = ACTIONS(1703),
    [anon_sym_LBRACK] = ACTIONS(1703),
    [anon_sym_LBRACE] = ACTIONS(1703),
    [anon_sym_RBRACE] = ACTIONS(1703),
    [anon_sym_STAR] = ACTIONS(1703),
    [anon_sym_u8] = ACTIONS(1705),
    [anon_sym_i8] = ACTIONS(1705),
    [anon_sym_u16] = ACTIONS(1705),
    [anon_sym_i16] = ACTIONS(1705),
    [anon_sym_u32] = ACTIONS(1705),
    [anon_sym_i32] = ACTIONS(1705),
    [anon_sym_u64] = ACTIONS(1705),
    [anon_sym_i64] = ACTIONS(1705),
    [anon_sym_u128] = ACTIONS(1705),
    [anon_sym_i128] = ACTIONS(1705),
    [anon_sym_isize] = ACTIONS(1705),
    [anon_sym_usize] = ACTIONS(1705),
    [anon_sym_f32] = ACTIONS(1705),
    [anon_sym_f64] = ACTIONS(1705),
    [anon_sym_bool] = ACTIONS(1705),
    [anon_sym_str] = ACTIONS(1705),
    [anon_sym_char] = ACTIONS(1705),
    [anon_sym_DASH] = ACTIONS(1703),
    [anon_sym_BANG] = ACTIONS(1703),
    [anon_sym_AMP] = ACTIONS(1703),
    [anon_sym_PIPE] = ACTIONS(1703),
    [anon_sym_LT] = ACTIONS(1703),
    [anon_sym_DOT_DOT] = ACTIONS(1703),
    [anon_sym_COLON_COLON] = ACTIONS(1703),
    [anon_sym_POUND] = ACTIONS(1703),
    [anon_sym_SQUOTE] = ACTIONS(1705),
    [anon_sym_async] = ACTIONS(1705),
    [anon_sym_break] = ACTIONS(1705),
    [anon_sym_const] = ACTIONS(1705),
    [anon_sym_continue] = ACTIONS(1705),
    [anon_sym_default] = ACTIONS(1705),
    [anon_sym_enum] = ACTIONS(1705),
    [anon_sym_fn] = ACTIONS(1705),
    [anon_sym_for] = ACTIONS(1705),
    [anon_sym_gen] = ACTIONS(1705),
    [anon_sym_if] = ACTIONS(1705),
    [anon_sym_impl] = ACTIONS(1705),
    [anon_sym_let] = ACTIONS(1705),
    [anon_sym_loop] = ACTIONS(1705),
    [anon_sym_match] = ACTIONS(1705),
    [anon_sym_mod] = ACTIONS(1705),
    [anon_sym_pub] = ACTIONS(1705),
    [anon_sym_return] = ACTIONS(1705),
    [anon_sym_static] = ACTIONS(1705),
    [anon_sym_struct] = ACTIONS(1705),
    [anon_sym_trait] = ACTIONS(1705),
    [anon_sym_type] = ACTIONS(1705),
    [anon_sym_union] = ACTIONS(1705),
    [anon_sym_unsafe] = ACTIONS(1705),
    [anon_sym_use] = ACTIONS(1705),
    [anon_sym_while] = ACTIONS(1705),
    [anon_sym_extern] = ACTIONS(1705),
    [anon_sym_yield] = ACTIONS(1705),
    [anon_sym_move] = ACTIONS(1705),
    [anon_sym_try] = ACTIONS(1705),
    [sym_integer_literal] = ACTIONS(1703),
    [aux_sym_string_literal_token1] = ACTIONS(1703),
    [sym_char_literal] = ACTIONS(1703),
    [anon_sym_true] = ACTIONS(1705),
    [anon_sym_false] = ACTIONS(1705),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1705),
    [sym_super] = ACTIONS(1705),
    [sym_crate] = ACTIONS(1705),
    [sym_metavariable] = ACTIONS(1703),
    [sym__raw_string_literal_start] = ACTIONS(1703),
    [sym_float_literal] = ACTIONS(1703),
  },
  [489] = {
    [sym_line_comment] = STATE(489),
    [sym_block_comment] = STATE(489),
    [ts_builtin_sym_end] = ACTIONS(1707),
    [sym_identifier] = ACTIONS(1709),
    [anon_sym_SEMI] = ACTIONS(1707),
    [anon_sym_macro_rules_BANG] = ACTIONS(1707),
    [anon_sym_LPAREN] = ACTIONS(1707),
    [anon_sym_LBRACK] = ACTIONS(1707),
    [anon_sym_LBRACE] = ACTIONS(1707),
    [anon_sym_RBRACE] = ACTIONS(1707),
    [anon_sym_STAR] = ACTIONS(1707),
    [anon_sym_u8] = ACTIONS(1709),
    [anon_sym_i8] = ACTIONS(1709),
    [anon_sym_u16] = ACTIONS(1709),
    [anon_sym_i16] = ACTIONS(1709),
    [anon_sym_u32] = ACTIONS(1709),
    [anon_sym_i32] = ACTIONS(1709),
    [anon_sym_u64] = ACTIONS(1709),
    [anon_sym_i64] = ACTIONS(1709),
    [anon_sym_u128] = ACTIONS(1709),
    [anon_sym_i128] = ACTIONS(1709),
    [anon_sym_isize] = ACTIONS(1709),
    [anon_sym_usize] = ACTIONS(1709),
    [anon_sym_f32] = ACTIONS(1709),
    [anon_sym_f64] = ACTIONS(1709),
    [anon_sym_bool] = ACTIONS(1709),
    [anon_sym_str] = ACTIONS(1709),
    [anon_sym_char] = ACTIONS(1709),
    [anon_sym_DASH] = ACTIONS(1707),
    [anon_sym_BANG] = ACTIONS(1707),
    [anon_sym_AMP] = ACTIONS(1707),
    [anon_sym_PIPE] = ACTIONS(1707),
    [anon_sym_LT] = ACTIONS(1707),
    [anon_sym_DOT_DOT] = ACTIONS(1707),
    [anon_sym_COLON_COLON] = ACTIONS(1707),
    [anon_sym_POUND] = ACTIONS(1707),
    [anon_sym_SQUOTE] = ACTIONS(1709),
    [anon_sym_async] = ACTIONS(1709),
    [anon_sym_break] = ACTIONS(1709),
    [anon_sym_const] = ACTIONS(1709),
    [anon_sym_continue] = ACTIONS(1709),
    [anon_sym_default] = ACTIONS(1709),
    [anon_sym_enum] = ACTIONS(1709),
    [anon_sym_fn] = ACTIONS(1709),
    [anon_sym_for] = ACTIONS(1709),
    [anon_sym_gen] = ACTIONS(1709),
    [anon_sym_if] = ACTIONS(1709),
    [anon_sym_impl] = ACTIONS(1709),
    [anon_sym_let] = ACTIONS(1709),
    [anon_sym_loop] = ACTIONS(1709),
    [anon_sym_match] = ACTIONS(1709),
    [anon_sym_mod] = ACTIONS(1709),
    [anon_sym_pub] = ACTIONS(1709),
    [anon_sym_return] = ACTIONS(1709),
    [anon_sym_static] = ACTIONS(1709),
    [anon_sym_struct] = ACTIONS(1709),
    [anon_sym_trait] = ACTIONS(1709),
    [anon_sym_type] = ACTIONS(1709),
    [anon_sym_union] = ACTIONS(1709),
    [anon_sym_unsafe] = ACTIONS(1709),
    [anon_sym_use] = ACTIONS(1709),
    [anon_sym_while] = ACTIONS(1709),
    [anon_sym_extern] = ACTIONS(1709),
    [anon_sym_yield] = ACTIONS(1709),
    [anon_sym_move] = ACTIONS(1709),
    [anon_sym_try] = ACTIONS(1709),
    [sym_integer_literal] = ACTIONS(1707),
    [aux_sym_string_literal_token1] = ACTIONS(1707),
    [sym_char_literal] = ACTIONS(1707),
    [anon_sym_true] = ACTIONS(1709),
    [anon_sym_false] = ACTIONS(1709),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1709),
    [sym_super] = ACTIONS(1709),
    [sym_crate] = ACTIONS(1709),
    [sym_metavariable] = ACTIONS(1707),
    [sym__raw_string_literal_start] = ACTIONS(1707),
    [sym_float_literal] = ACTIONS(1707),
  },
  [490] = {
    [sym_line_comment] = STATE(490),
    [sym_block_comment] = STATE(490),
    [ts_builtin_sym_end] = ACTIONS(1711),
    [sym_identifier] = ACTIONS(1713),
    [anon_sym_SEMI] = ACTIONS(1711),
    [anon_sym_macro_rules_BANG] = ACTIONS(1711),
    [anon_sym_LPAREN] = ACTIONS(1711),
    [anon_sym_LBRACK] = ACTIONS(1711),
    [anon_sym_LBRACE] = ACTIONS(1711),
    [anon_sym_RBRACE] = ACTIONS(1711),
    [anon_sym_STAR] = ACTIONS(1711),
    [anon_sym_u8] = ACTIONS(1713),
    [anon_sym_i8] = ACTIONS(1713),
    [anon_sym_u16] = ACTIONS(1713),
    [anon_sym_i16] = ACTIONS(1713),
    [anon_sym_u32] = ACTIONS(1713),
    [anon_sym_i32] = ACTIONS(1713),
    [anon_sym_u64] = ACTIONS(1713),
    [anon_sym_i64] = ACTIONS(1713),
    [anon_sym_u128] = ACTIONS(1713),
    [anon_sym_i128] = ACTIONS(1713),
    [anon_sym_isize] = ACTIONS(1713),
    [anon_sym_usize] = ACTIONS(1713),
    [anon_sym_f32] = ACTIONS(1713),
    [anon_sym_f64] = ACTIONS(1713),
    [anon_sym_bool] = ACTIONS(1713),
    [anon_sym_str] = ACTIONS(1713),
    [anon_sym_char] = ACTIONS(1713),
    [anon_sym_DASH] = ACTIONS(1711),
    [anon_sym_BANG] = ACTIONS(1711),
    [anon_sym_AMP] = ACTIONS(1711),
    [anon_sym_PIPE] = ACTIONS(1711),
    [anon_sym_LT] = ACTIONS(1711),
    [anon_sym_DOT_DOT] = ACTIONS(1711),
    [anon_sym_COLON_COLON] = ACTIONS(1711),
    [anon_sym_POUND] = ACTIONS(1711),
    [anon_sym_SQUOTE] = ACTIONS(1713),
    [anon_sym_async] = ACTIONS(1713),
    [anon_sym_break] = ACTIONS(1713),
    [anon_sym_const] = ACTIONS(1713),
    [anon_sym_continue] = ACTIONS(1713),
    [anon_sym_default] = ACTIONS(1713),
    [anon_sym_enum] = ACTIONS(1713),
    [anon_sym_fn] = ACTIONS(1713),
    [anon_sym_for] = ACTIONS(1713),
    [anon_sym_gen] = ACTIONS(1713),
    [anon_sym_if] = ACTIONS(1713),
    [anon_sym_impl] = ACTIONS(1713),
    [anon_sym_let] = ACTIONS(1713),
    [anon_sym_loop] = ACTIONS(1713),
    [anon_sym_match] = ACTIONS(1713),
    [anon_sym_mod] = ACTIONS(1713),
    [anon_sym_pub] = ACTIONS(1713),
    [anon_sym_return] = ACTIONS(1713),
    [anon_sym_static] = ACTIONS(1713),
    [anon_sym_struct] = ACTIONS(1713),
    [anon_sym_trait] = ACTIONS(1713),
    [anon_sym_type] = ACTIONS(1713),
    [anon_sym_union] = ACTIONS(1713),
    [anon_sym_unsafe] = ACTIONS(1713),
    [anon_sym_use] = ACTIONS(1713),
    [anon_sym_while] = ACTIONS(1713),
    [anon_sym_extern] = ACTIONS(1713),
    [anon_sym_yield] = ACTIONS(1713),
    [anon_sym_move] = ACTIONS(1713),
    [anon_sym_try] = ACTIONS(1713),
    [sym_integer_literal] = ACTIONS(1711),
    [aux_sym_string_literal_token1] = ACTIONS(1711),
    [sym_char_literal] = ACTIONS(1711),
    [anon_sym_true] = ACTIONS(1713),
    [anon_sym_false] = ACTIONS(1713),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1713),
    [sym_super] = ACTIONS(1713),
    [sym_crate] = ACTIONS(1713),
    [sym_metavariable] = ACTIONS(1711),
    [sym__raw_string_literal_start] = ACTIONS(1711),
    [sym_float_literal] = ACTIONS(1711),
  },
  [491] = {
    [sym_line_comment] = STATE(491),
    [sym_block_comment] = STATE(491),
    [ts_builtin_sym_end] = ACTIONS(1715),
    [sym_identifier] = ACTIONS(1717),
    [anon_sym_SEMI] = ACTIONS(1715),
    [anon_sym_macro_rules_BANG] = ACTIONS(1715),
    [anon_sym_LPAREN] = ACTIONS(1715),
    [anon_sym_LBRACK] = ACTIONS(1715),
    [anon_sym_LBRACE] = ACTIONS(1715),
    [anon_sym_RBRACE] = ACTIONS(1715),
    [anon_sym_STAR] = ACTIONS(1715),
    [anon_sym_u8] = ACTIONS(1717),
    [anon_sym_i8] = ACTIONS(1717),
    [anon_sym_u16] = ACTIONS(1717),
    [anon_sym_i16] = ACTIONS(1717),
    [anon_sym_u32] = ACTIONS(1717),
    [anon_sym_i32] = ACTIONS(1717),
    [anon_sym_u64] = ACTIONS(1717),
    [anon_sym_i64] = ACTIONS(1717),
    [anon_sym_u128] = ACTIONS(1717),
    [anon_sym_i128] = ACTIONS(1717),
    [anon_sym_isize] = ACTIONS(1717),
    [anon_sym_usize] = ACTIONS(1717),
    [anon_sym_f32] = ACTIONS(1717),
    [anon_sym_f64] = ACTIONS(1717),
    [anon_sym_bool] = ACTIONS(1717),
    [anon_sym_str] = ACTIONS(1717),
    [anon_sym_char] = ACTIONS(1717),
    [anon_sym_DASH] = ACTIONS(1715),
    [anon_sym_BANG] = ACTIONS(1715),
    [anon_sym_AMP] = ACTIONS(1715),
    [anon_sym_PIPE] = ACTIONS(1715),
    [anon_sym_LT] = ACTIONS(1715),
    [anon_sym_DOT_DOT] = ACTIONS(1715),
    [anon_sym_COLON_COLON] = ACTIONS(1715),
    [anon_sym_POUND] = ACTIONS(1715),
    [anon_sym_SQUOTE] = ACTIONS(1717),
    [anon_sym_async] = ACTIONS(1717),
    [anon_sym_break] = ACTIONS(1717),
    [anon_sym_const] = ACTIONS(1717),
    [anon_sym_continue] = ACTIONS(1717),
    [anon_sym_default] = ACTIONS(1717),
    [anon_sym_enum] = ACTIONS(1717),
    [anon_sym_fn] = ACTIONS(1717),
    [anon_sym_for] = ACTIONS(1717),
    [anon_sym_gen] = ACTIONS(1717),
    [anon_sym_if] = ACTIONS(1717),
    [anon_sym_impl] = ACTIONS(1717),
    [anon_sym_let] = ACTIONS(1717),
    [anon_sym_loop] = ACTIONS(1717),
    [anon_sym_match] = ACTIONS(1717),
    [anon_sym_mod] = ACTIONS(1717),
    [anon_sym_pub] = ACTIONS(1717),
    [anon_sym_return] = ACTIONS(1717),
    [anon_sym_static] = ACTIONS(1717),
    [anon_sym_struct] = ACTIONS(1717),
    [anon_sym_trait] = ACTIONS(1717),
    [anon_sym_type] = ACTIONS(1717),
    [anon_sym_union] = ACTIONS(1717),
    [anon_sym_unsafe] = ACTIONS(1717),
    [anon_sym_use] = ACTIONS(1717),
    [anon_sym_while] = ACTIONS(1717),
    [anon_sym_extern] = ACTIONS(1717),
    [anon_sym_yield] = ACTIONS(1717),
    [anon_sym_move] = ACTIONS(1717),
    [anon_sym_try] = ACTIONS(1717),
    [sym_integer_literal] = ACTIONS(1715),
    [aux_sym_string_literal_token1] = ACTIONS(1715),
    [sym_char_literal] = ACTIONS(1715),
    [anon_sym_true] = ACTIONS(1717),
    [anon_sym_false] = ACTIONS(1717),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1717),
    [sym_super] = ACTIONS(1717),
    [sym_crate] = ACTIONS(1717),
    [sym_metavariable] = ACTIONS(1715),
    [sym__raw_string_literal_start] = ACTIONS(1715),
    [sym_float_literal] = ACTIONS(1715),
  },
  [492] = {
    [sym_line_comment] = STATE(492),
    [sym_block_comment] = STATE(492),
    [ts_builtin_sym_end] = ACTIONS(1719),
    [sym_identifier] = ACTIONS(1721),
    [anon_sym_SEMI] = ACTIONS(1719),
    [anon_sym_macro_rules_BANG] = ACTIONS(1719),
    [anon_sym_LPAREN] = ACTIONS(1719),
    [anon_sym_LBRACK] = ACTIONS(1719),
    [anon_sym_LBRACE] = ACTIONS(1719),
    [anon_sym_RBRACE] = ACTIONS(1719),
    [anon_sym_STAR] = ACTIONS(1719),
    [anon_sym_u8] = ACTIONS(1721),
    [anon_sym_i8] = ACTIONS(1721),
    [anon_sym_u16] = ACTIONS(1721),
    [anon_sym_i16] = ACTIONS(1721),
    [anon_sym_u32] = ACTIONS(1721),
    [anon_sym_i32] = ACTIONS(1721),
    [anon_sym_u64] = ACTIONS(1721),
    [anon_sym_i64] = ACTIONS(1721),
    [anon_sym_u128] = ACTIONS(1721),
    [anon_sym_i128] = ACTIONS(1721),
    [anon_sym_isize] = ACTIONS(1721),
    [anon_sym_usize] = ACTIONS(1721),
    [anon_sym_f32] = ACTIONS(1721),
    [anon_sym_f64] = ACTIONS(1721),
    [anon_sym_bool] = ACTIONS(1721),
    [anon_sym_str] = ACTIONS(1721),
    [anon_sym_char] = ACTIONS(1721),
    [anon_sym_DASH] = ACTIONS(1719),
    [anon_sym_BANG] = ACTIONS(1719),
    [anon_sym_AMP] = ACTIONS(1719),
    [anon_sym_PIPE] = ACTIONS(1719),
    [anon_sym_LT] = ACTIONS(1719),
    [anon_sym_DOT_DOT] = ACTIONS(1719),
    [anon_sym_COLON_COLON] = ACTIONS(1719),
    [anon_sym_POUND] = ACTIONS(1719),
    [anon_sym_SQUOTE] = ACTIONS(1721),
    [anon_sym_async] = ACTIONS(1721),
    [anon_sym_break] = ACTIONS(1721),
    [anon_sym_const] = ACTIONS(1721),
    [anon_sym_continue] = ACTIONS(1721),
    [anon_sym_default] = ACTIONS(1721),
    [anon_sym_enum] = ACTIONS(1721),
    [anon_sym_fn] = ACTIONS(1721),
    [anon_sym_for] = ACTIONS(1721),
    [anon_sym_gen] = ACTIONS(1721),
    [anon_sym_if] = ACTIONS(1721),
    [anon_sym_impl] = ACTIONS(1721),
    [anon_sym_let] = ACTIONS(1721),
    [anon_sym_loop] = ACTIONS(1721),
    [anon_sym_match] = ACTIONS(1721),
    [anon_sym_mod] = ACTIONS(1721),
    [anon_sym_pub] = ACTIONS(1721),
    [anon_sym_return] = ACTIONS(1721),
    [anon_sym_static] = ACTIONS(1721),
    [anon_sym_struct] = ACTIONS(1721),
    [anon_sym_trait] = ACTIONS(1721),
    [anon_sym_type] = ACTIONS(1721),
    [anon_sym_union] = ACTIONS(1721),
    [anon_sym_unsafe] = ACTIONS(1721),
    [anon_sym_use] = ACTIONS(1721),
    [anon_sym_while] = ACTIONS(1721),
    [anon_sym_extern] = ACTIONS(1721),
    [anon_sym_yield] = ACTIONS(1721),
    [anon_sym_move] = ACTIONS(1721),
    [anon_sym_try] = ACTIONS(1721),
    [sym_integer_literal] = ACTIONS(1719),
    [aux_sym_string_literal_token1] = ACTIONS(1719),
    [sym_char_literal] = ACTIONS(1719),
    [anon_sym_true] = ACTIONS(1721),
    [anon_sym_false] = ACTIONS(1721),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1721),
    [sym_super] = ACTIONS(1721),
    [sym_crate] = ACTIONS(1721),
    [sym_metavariable] = ACTIONS(1719),
    [sym__raw_string_literal_start] = ACTIONS(1719),
    [sym_float_literal] = ACTIONS(1719),
  },
  [493] = {
    [sym_line_comment] = STATE(493),
    [sym_block_comment] = STATE(493),
    [ts_builtin_sym_end] = ACTIONS(1723),
    [sym_identifier] = ACTIONS(1725),
    [anon_sym_SEMI] = ACTIONS(1723),
    [anon_sym_macro_rules_BANG] = ACTIONS(1723),
    [anon_sym_LPAREN] = ACTIONS(1723),
    [anon_sym_LBRACK] = ACTIONS(1723),
    [anon_sym_LBRACE] = ACTIONS(1723),
    [anon_sym_RBRACE] = ACTIONS(1723),
    [anon_sym_STAR] = ACTIONS(1723),
    [anon_sym_u8] = ACTIONS(1725),
    [anon_sym_i8] = ACTIONS(1725),
    [anon_sym_u16] = ACTIONS(1725),
    [anon_sym_i16] = ACTIONS(1725),
    [anon_sym_u32] = ACTIONS(1725),
    [anon_sym_i32] = ACTIONS(1725),
    [anon_sym_u64] = ACTIONS(1725),
    [anon_sym_i64] = ACTIONS(1725),
    [anon_sym_u128] = ACTIONS(1725),
    [anon_sym_i128] = ACTIONS(1725),
    [anon_sym_isize] = ACTIONS(1725),
    [anon_sym_usize] = ACTIONS(1725),
    [anon_sym_f32] = ACTIONS(1725),
    [anon_sym_f64] = ACTIONS(1725),
    [anon_sym_bool] = ACTIONS(1725),
    [anon_sym_str] = ACTIONS(1725),
    [anon_sym_char] = ACTIONS(1725),
    [anon_sym_DASH] = ACTIONS(1723),
    [anon_sym_BANG] = ACTIONS(1723),
    [anon_sym_AMP] = ACTIONS(1723),
    [anon_sym_PIPE] = ACTIONS(1723),
    [anon_sym_LT] = ACTIONS(1723),
    [anon_sym_DOT_DOT] = ACTIONS(1723),
    [anon_sym_COLON_COLON] = ACTIONS(1723),
    [anon_sym_POUND] = ACTIONS(1723),
    [anon_sym_SQUOTE] = ACTIONS(1725),
    [anon_sym_async] = ACTIONS(1725),
    [anon_sym_break] = ACTIONS(1725),
    [anon_sym_const] = ACTIONS(1725),
    [anon_sym_continue] = ACTIONS(1725),
    [anon_sym_default] = ACTIONS(1725),
    [anon_sym_enum] = ACTIONS(1725),
    [anon_sym_fn] = ACTIONS(1725),
    [anon_sym_for] = ACTIONS(1725),
    [anon_sym_gen] = ACTIONS(1725),
    [anon_sym_if] = ACTIONS(1725),
    [anon_sym_impl] = ACTIONS(1725),
    [anon_sym_let] = ACTIONS(1725),
    [anon_sym_loop] = ACTIONS(1725),
    [anon_sym_match] = ACTIONS(1725),
    [anon_sym_mod] = ACTIONS(1725),
    [anon_sym_pub] = ACTIONS(1725),
    [anon_sym_return] = ACTIONS(1725),
    [anon_sym_static] = ACTIONS(1725),
    [anon_sym_struct] = ACTIONS(1725),
    [anon_sym_trait] = ACTIONS(1725),
    [anon_sym_type] = ACTIONS(1725),
    [anon_sym_union] = ACTIONS(1725),
    [anon_sym_unsafe] = ACTIONS(1725),
    [anon_sym_use] = ACTIONS(1725),
    [anon_sym_while] = ACTIONS(1725),
    [anon_sym_extern] = ACTIONS(1725),
    [anon_sym_yield] = ACTIONS(1725),
    [anon_sym_move] = ACTIONS(1725),
    [anon_sym_try] = ACTIONS(1725),
    [sym_integer_literal] = ACTIONS(1723),
    [aux_sym_string_literal_token1] = ACTIONS(1723),
    [sym_char_literal] = ACTIONS(1723),
    [anon_sym_true] = ACTIONS(1725),
    [anon_sym_false] = ACTIONS(1725),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1725),
    [sym_super] = ACTIONS(1725),
    [sym_crate] = ACTIONS(1725),
    [sym_metavariable] = ACTIONS(1723),
    [sym__raw_string_literal_start] = ACTIONS(1723),
    [sym_float_literal] = ACTIONS(1723),
  },
  [494] = {
    [sym_line_comment] = STATE(494),
    [sym_block_comment] = STATE(494),
    [ts_builtin_sym_end] = ACTIONS(1727),
    [sym_identifier] = ACTIONS(1729),
    [anon_sym_SEMI] = ACTIONS(1727),
    [anon_sym_macro_rules_BANG] = ACTIONS(1727),
    [anon_sym_LPAREN] = ACTIONS(1727),
    [anon_sym_LBRACK] = ACTIONS(1727),
    [anon_sym_LBRACE] = ACTIONS(1727),
    [anon_sym_RBRACE] = ACTIONS(1727),
    [anon_sym_STAR] = ACTIONS(1727),
    [anon_sym_u8] = ACTIONS(1729),
    [anon_sym_i8] = ACTIONS(1729),
    [anon_sym_u16] = ACTIONS(1729),
    [anon_sym_i16] = ACTIONS(1729),
    [anon_sym_u32] = ACTIONS(1729),
    [anon_sym_i32] = ACTIONS(1729),
    [anon_sym_u64] = ACTIONS(1729),
    [anon_sym_i64] = ACTIONS(1729),
    [anon_sym_u128] = ACTIONS(1729),
    [anon_sym_i128] = ACTIONS(1729),
    [anon_sym_isize] = ACTIONS(1729),
    [anon_sym_usize] = ACTIONS(1729),
    [anon_sym_f32] = ACTIONS(1729),
    [anon_sym_f64] = ACTIONS(1729),
    [anon_sym_bool] = ACTIONS(1729),
    [anon_sym_str] = ACTIONS(1729),
    [anon_sym_char] = ACTIONS(1729),
    [anon_sym_DASH] = ACTIONS(1727),
    [anon_sym_BANG] = ACTIONS(1727),
    [anon_sym_AMP] = ACTIONS(1727),
    [anon_sym_PIPE] = ACTIONS(1727),
    [anon_sym_LT] = ACTIONS(1727),
    [anon_sym_DOT_DOT] = ACTIONS(1727),
    [anon_sym_COLON_COLON] = ACTIONS(1727),
    [anon_sym_POUND] = ACTIONS(1727),
    [anon_sym_SQUOTE] = ACTIONS(1729),
    [anon_sym_async] = ACTIONS(1729),
    [anon_sym_break] = ACTIONS(1729),
    [anon_sym_const] = ACTIONS(1729),
    [anon_sym_continue] = ACTIONS(1729),
    [anon_sym_default] = ACTIONS(1729),
    [anon_sym_enum] = ACTIONS(1729),
    [anon_sym_fn] = ACTIONS(1729),
    [anon_sym_for] = ACTIONS(1729),
    [anon_sym_gen] = ACTIONS(1729),
    [anon_sym_if] = ACTIONS(1729),
    [anon_sym_impl] = ACTIONS(1729),
    [anon_sym_let] = ACTIONS(1729),
    [anon_sym_loop] = ACTIONS(1729),
    [anon_sym_match] = ACTIONS(1729),
    [anon_sym_mod] = ACTIONS(1729),
    [anon_sym_pub] = ACTIONS(1729),
    [anon_sym_return] = ACTIONS(1729),
    [anon_sym_static] = ACTIONS(1729),
    [anon_sym_struct] = ACTIONS(1729),
    [anon_sym_trait] = ACTIONS(1729),
    [anon_sym_type] = ACTIONS(1729),
    [anon_sym_union] = ACTIONS(1729),
    [anon_sym_unsafe] = ACTIONS(1729),
    [anon_sym_use] = ACTIONS(1729),
    [anon_sym_while] = ACTIONS(1729),
    [anon_sym_extern] = ACTIONS(1729),
    [anon_sym_yield] = ACTIONS(1729),
    [anon_sym_move] = ACTIONS(1729),
    [anon_sym_try] = ACTIONS(1729),
    [sym_integer_literal] = ACTIONS(1727),
    [aux_sym_string_literal_token1] = ACTIONS(1727),
    [sym_char_literal] = ACTIONS(1727),
    [anon_sym_true] = ACTIONS(1729),
    [anon_sym_false] = ACTIONS(1729),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1729),
    [sym_super] = ACTIONS(1729),
    [sym_crate] = ACTIONS(1729),
    [sym_metavariable] = ACTIONS(1727),
    [sym__raw_string_literal_start] = ACTIONS(1727),
    [sym_float_literal] = ACTIONS(1727),
  },
  [495] = {
    [sym_line_comment] = STATE(495),
    [sym_block_comment] = STATE(495),
    [ts_builtin_sym_end] = ACTIONS(1731),
    [sym_identifier] = ACTIONS(1733),
    [anon_sym_SEMI] = ACTIONS(1731),
    [anon_sym_macro_rules_BANG] = ACTIONS(1731),
    [anon_sym_LPAREN] = ACTIONS(1731),
    [anon_sym_LBRACK] = ACTIONS(1731),
    [anon_sym_LBRACE] = ACTIONS(1731),
    [anon_sym_RBRACE] = ACTIONS(1731),
    [anon_sym_STAR] = ACTIONS(1731),
    [anon_sym_u8] = ACTIONS(1733),
    [anon_sym_i8] = ACTIONS(1733),
    [anon_sym_u16] = ACTIONS(1733),
    [anon_sym_i16] = ACTIONS(1733),
    [anon_sym_u32] = ACTIONS(1733),
    [anon_sym_i32] = ACTIONS(1733),
    [anon_sym_u64] = ACTIONS(1733),
    [anon_sym_i64] = ACTIONS(1733),
    [anon_sym_u128] = ACTIONS(1733),
    [anon_sym_i128] = ACTIONS(1733),
    [anon_sym_isize] = ACTIONS(1733),
    [anon_sym_usize] = ACTIONS(1733),
    [anon_sym_f32] = ACTIONS(1733),
    [anon_sym_f64] = ACTIONS(1733),
    [anon_sym_bool] = ACTIONS(1733),
    [anon_sym_str] = ACTIONS(1733),
    [anon_sym_char] = ACTIONS(1733),
    [anon_sym_DASH] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1731),
    [anon_sym_AMP] = ACTIONS(1731),
    [anon_sym_PIPE] = ACTIONS(1731),
    [anon_sym_LT] = ACTIONS(1731),
    [anon_sym_DOT_DOT] = ACTIONS(1731),
    [anon_sym_COLON_COLON] = ACTIONS(1731),
    [anon_sym_POUND] = ACTIONS(1731),
    [anon_sym_SQUOTE] = ACTIONS(1733),
    [anon_sym_async] = ACTIONS(1733),
    [anon_sym_break] = ACTIONS(1733),
    [anon_sym_const] = ACTIONS(1733),
    [anon_sym_continue] = ACTIONS(1733),
    [anon_sym_default] = ACTIONS(1733),
    [anon_sym_enum] = ACTIONS(1733),
    [anon_sym_fn] = ACTIONS(1733),
    [anon_sym_for] = ACTIONS(1733),
    [anon_sym_gen] = ACTIONS(1733),
    [anon_sym_if] = ACTIONS(1733),
    [anon_sym_impl] = ACTIONS(1733),
    [anon_sym_let] = ACTIONS(1733),
    [anon_sym_loop] = ACTIONS(1733),
    [anon_sym_match] = ACTIONS(1733),
    [anon_sym_mod] = ACTIONS(1733),
    [anon_sym_pub] = ACTIONS(1733),
    [anon_sym_return] = ACTIONS(1733),
    [anon_sym_static] = ACTIONS(1733),
    [anon_sym_struct] = ACTIONS(1733),
    [anon_sym_trait] = ACTIONS(1733),
    [anon_sym_type] = ACTIONS(1733),
    [anon_sym_union] = ACTIONS(1733),
    [anon_sym_unsafe] = ACTIONS(1733),
    [anon_sym_use] = ACTIONS(1733),
    [anon_sym_while] = ACTIONS(1733),
    [anon_sym_extern] = ACTIONS(1733),
    [anon_sym_yield] = ACTIONS(1733),
    [anon_sym_move] = ACTIONS(1733),
    [anon_sym_try] = ACTIONS(1733),
    [sym_integer_literal] = ACTIONS(1731),
    [aux_sym_string_literal_token1] = ACTIONS(1731),
    [sym_char_literal] = ACTIONS(1731),
    [anon_sym_true] = ACTIONS(1733),
    [anon_sym_false] = ACTIONS(1733),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1733),
    [sym_super] = ACTIONS(1733),
    [sym_crate] = ACTIONS(1733),
    [sym_metavariable] = ACTIONS(1731),
    [sym__raw_string_literal_start] = ACTIONS(1731),
    [sym_float_literal] = ACTIONS(1731),
  },
  [496] = {
    [sym_line_comment] = STATE(496),
    [sym_block_comment] = STATE(496),
    [ts_builtin_sym_end] = ACTIONS(1735),
    [sym_identifier] = ACTIONS(1737),
    [anon_sym_SEMI] = ACTIONS(1735),
    [anon_sym_macro_rules_BANG] = ACTIONS(1735),
    [anon_sym_LPAREN] = ACTIONS(1735),
    [anon_sym_LBRACK] = ACTIONS(1735),
    [anon_sym_LBRACE] = ACTIONS(1735),
    [anon_sym_RBRACE] = ACTIONS(1735),
    [anon_sym_STAR] = ACTIONS(1735),
    [anon_sym_u8] = ACTIONS(1737),
    [anon_sym_i8] = ACTIONS(1737),
    [anon_sym_u16] = ACTIONS(1737),
    [anon_sym_i16] = ACTIONS(1737),
    [anon_sym_u32] = ACTIONS(1737),
    [anon_sym_i32] = ACTIONS(1737),
    [anon_sym_u64] = ACTIONS(1737),
    [anon_sym_i64] = ACTIONS(1737),
    [anon_sym_u128] = ACTIONS(1737),
    [anon_sym_i128] = ACTIONS(1737),
    [anon_sym_isize] = ACTIONS(1737),
    [anon_sym_usize] = ACTIONS(1737),
    [anon_sym_f32] = ACTIONS(1737),
    [anon_sym_f64] = ACTIONS(1737),
    [anon_sym_bool] = ACTIONS(1737),
    [anon_sym_str] = ACTIONS(1737),
    [anon_sym_char] = ACTIONS(1737),
    [anon_sym_DASH] = ACTIONS(1735),
    [anon_sym_BANG] = ACTIONS(1735),
    [anon_sym_AMP] = ACTIONS(1735),
    [anon_sym_PIPE] = ACTIONS(1735),
    [anon_sym_LT] = ACTIONS(1735),
    [anon_sym_DOT_DOT] = ACTIONS(1735),
    [anon_sym_COLON_COLON] = ACTIONS(1735),
    [anon_sym_POUND] = ACTIONS(1735),
    [anon_sym_SQUOTE] = ACTIONS(1737),
    [anon_sym_async] = ACTIONS(1737),
    [anon_sym_break] = ACTIONS(1737),
    [anon_sym_const] = ACTIONS(1737),
    [anon_sym_continue] = ACTIONS(1737),
    [anon_sym_default] = ACTIONS(1737),
    [anon_sym_enum] = ACTIONS(1737),
    [anon_sym_fn] = ACTIONS(1737),
    [anon_sym_for] = ACTIONS(1737),
    [anon_sym_gen] = ACTIONS(1737),
    [anon_sym_if] = ACTIONS(1737),
    [anon_sym_impl] = ACTIONS(1737),
    [anon_sym_let] = ACTIONS(1737),
    [anon_sym_loop] = ACTIONS(1737),
    [anon_sym_match] = ACTIONS(1737),
    [anon_sym_mod] = ACTIONS(1737),
    [anon_sym_pub] = ACTIONS(1737),
    [anon_sym_return] = ACTIONS(1737),
    [anon_sym_static] = ACTIONS(1737),
    [anon_sym_struct] = ACTIONS(1737),
    [anon_sym_trait] = ACTIONS(1737),
    [anon_sym_type] = ACTIONS(1737),
    [anon_sym_union] = ACTIONS(1737),
    [anon_sym_unsafe] = ACTIONS(1737),
    [anon_sym_use] = ACTIONS(1737),
    [anon_sym_while] = ACTIONS(1737),
    [anon_sym_extern] = ACTIONS(1737),
    [anon_sym_yield] = ACTIONS(1737),
    [anon_sym_move] = ACTIONS(1737),
    [anon_sym_try] = ACTIONS(1737),
    [sym_integer_literal] = ACTIONS(1735),
    [aux_sym_string_literal_token1] = ACTIONS(1735),
    [sym_char_literal] = ACTIONS(1735),
    [anon_sym_true] = ACTIONS(1737),
    [anon_sym_false] = ACTIONS(1737),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1737),
    [sym_super] = ACTIONS(1737),
    [sym_crate] = ACTIONS(1737),
    [sym_metavariable] = ACTIONS(1735),
    [sym__raw_string_literal_start] = ACTIONS(1735),
    [sym_float_literal] = ACTIONS(1735),
  },
  [497] = {
    [sym_line_comment] = STATE(497),
    [sym_block_comment] = STATE(497),
    [ts_builtin_sym_end] = ACTIONS(1739),
    [sym_identifier] = ACTIONS(1741),
    [anon_sym_SEMI] = ACTIONS(1739),
    [anon_sym_macro_rules_BANG] = ACTIONS(1739),
    [anon_sym_LPAREN] = ACTIONS(1739),
    [anon_sym_LBRACK] = ACTIONS(1739),
    [anon_sym_LBRACE] = ACTIONS(1739),
    [anon_sym_RBRACE] = ACTIONS(1739),
    [anon_sym_STAR] = ACTIONS(1739),
    [anon_sym_u8] = ACTIONS(1741),
    [anon_sym_i8] = ACTIONS(1741),
    [anon_sym_u16] = ACTIONS(1741),
    [anon_sym_i16] = ACTIONS(1741),
    [anon_sym_u32] = ACTIONS(1741),
    [anon_sym_i32] = ACTIONS(1741),
    [anon_sym_u64] = ACTIONS(1741),
    [anon_sym_i64] = ACTIONS(1741),
    [anon_sym_u128] = ACTIONS(1741),
    [anon_sym_i128] = ACTIONS(1741),
    [anon_sym_isize] = ACTIONS(1741),
    [anon_sym_usize] = ACTIONS(1741),
    [anon_sym_f32] = ACTIONS(1741),
    [anon_sym_f64] = ACTIONS(1741),
    [anon_sym_bool] = ACTIONS(1741),
    [anon_sym_str] = ACTIONS(1741),
    [anon_sym_char] = ACTIONS(1741),
    [anon_sym_DASH] = ACTIONS(1739),
    [anon_sym_BANG] = ACTIONS(1739),
    [anon_sym_AMP] = ACTIONS(1739),
    [anon_sym_PIPE] = ACTIONS(1739),
    [anon_sym_LT] = ACTIONS(1739),
    [anon_sym_DOT_DOT] = ACTIONS(1739),
    [anon_sym_COLON_COLON] = ACTIONS(1739),
    [anon_sym_POUND] = ACTIONS(1739),
    [anon_sym_SQUOTE] = ACTIONS(1741),
    [anon_sym_async] = ACTIONS(1741),
    [anon_sym_break] = ACTIONS(1741),
    [anon_sym_const] = ACTIONS(1741),
    [anon_sym_continue] = ACTIONS(1741),
    [anon_sym_default] = ACTIONS(1741),
    [anon_sym_enum] = ACTIONS(1741),
    [anon_sym_fn] = ACTIONS(1741),
    [anon_sym_for] = ACTIONS(1741),
    [anon_sym_gen] = ACTIONS(1741),
    [anon_sym_if] = ACTIONS(1741),
    [anon_sym_impl] = ACTIONS(1741),
    [anon_sym_let] = ACTIONS(1741),
    [anon_sym_loop] = ACTIONS(1741),
    [anon_sym_match] = ACTIONS(1741),
    [anon_sym_mod] = ACTIONS(1741),
    [anon_sym_pub] = ACTIONS(1741),
    [anon_sym_return] = ACTIONS(1741),
    [anon_sym_static] = ACTIONS(1741),
    [anon_sym_struct] = ACTIONS(1741),
    [anon_sym_trait] = ACTIONS(1741),
    [anon_sym_type] = ACTIONS(1741),
    [anon_sym_union] = ACTIONS(1741),
    [anon_sym_unsafe] = ACTIONS(1741),
    [anon_sym_use] = ACTIONS(1741),
    [anon_sym_while] = ACTIONS(1741),
    [anon_sym_extern] = ACTIONS(1741),
    [anon_sym_yield] = ACTIONS(1741),
    [anon_sym_move] = ACTIONS(1741),
    [anon_sym_try] = ACTIONS(1741),
    [sym_integer_literal] = ACTIONS(1739),
    [aux_sym_string_literal_token1] = ACTIONS(1739),
    [sym_char_literal] = ACTIONS(1739),
    [anon_sym_true] = ACTIONS(1741),
    [anon_sym_false] = ACTIONS(1741),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1741),
    [sym_super] = ACTIONS(1741),
    [sym_crate] = ACTIONS(1741),
    [sym_metavariable] = ACTIONS(1739),
    [sym__raw_string_literal_start] = ACTIONS(1739),
    [sym_float_literal] = ACTIONS(1739),
  },
  [498] = {
    [sym_line_comment] = STATE(498),
    [sym_block_comment] = STATE(498),
    [ts_builtin_sym_end] = ACTIONS(1743),
    [sym_identifier] = ACTIONS(1745),
    [anon_sym_SEMI] = ACTIONS(1743),
    [anon_sym_macro_rules_BANG] = ACTIONS(1743),
    [anon_sym_LPAREN] = ACTIONS(1743),
    [anon_sym_LBRACK] = ACTIONS(1743),
    [anon_sym_LBRACE] = ACTIONS(1743),
    [anon_sym_RBRACE] = ACTIONS(1743),
    [anon_sym_STAR] = ACTIONS(1743),
    [anon_sym_u8] = ACTIONS(1745),
    [anon_sym_i8] = ACTIONS(1745),
    [anon_sym_u16] = ACTIONS(1745),
    [anon_sym_i16] = ACTIONS(1745),
    [anon_sym_u32] = ACTIONS(1745),
    [anon_sym_i32] = ACTIONS(1745),
    [anon_sym_u64] = ACTIONS(1745),
    [anon_sym_i64] = ACTIONS(1745),
    [anon_sym_u128] = ACTIONS(1745),
    [anon_sym_i128] = ACTIONS(1745),
    [anon_sym_isize] = ACTIONS(1745),
    [anon_sym_usize] = ACTIONS(1745),
    [anon_sym_f32] = ACTIONS(1745),
    [anon_sym_f64] = ACTIONS(1745),
    [anon_sym_bool] = ACTIONS(1745),
    [anon_sym_str] = ACTIONS(1745),
    [anon_sym_char] = ACTIONS(1745),
    [anon_sym_DASH] = ACTIONS(1743),
    [anon_sym_BANG] = ACTIONS(1743),
    [anon_sym_AMP] = ACTIONS(1743),
    [anon_sym_PIPE] = ACTIONS(1743),
    [anon_sym_LT] = ACTIONS(1743),
    [anon_sym_DOT_DOT] = ACTIONS(1743),
    [anon_sym_COLON_COLON] = ACTIONS(1743),
    [anon_sym_POUND] = ACTIONS(1743),
    [anon_sym_SQUOTE] = ACTIONS(1745),
    [anon_sym_async] = ACTIONS(1745),
    [anon_sym_break] = ACTIONS(1745),
    [anon_sym_const] = ACTIONS(1745),
    [anon_sym_continue] = ACTIONS(1745),
    [anon_sym_default] = ACTIONS(1745),
    [anon_sym_enum] = ACTIONS(1745),
    [anon_sym_fn] = ACTIONS(1745),
    [anon_sym_for] = ACTIONS(1745),
    [anon_sym_gen] = ACTIONS(1745),
    [anon_sym_if] = ACTIONS(1745),
    [anon_sym_impl] = ACTIONS(1745),
    [anon_sym_let] = ACTIONS(1745),
    [anon_sym_loop] = ACTIONS(1745),
    [anon_sym_match] = ACTIONS(1745),
    [anon_sym_mod] = ACTIONS(1745),
    [anon_sym_pub] = ACTIONS(1745),
    [anon_sym_return] = ACTIONS(1745),
    [anon_sym_static] = ACTIONS(1745),
    [anon_sym_struct] = ACTIONS(1745),
    [anon_sym_trait] = ACTIONS(1745),
    [anon_sym_type] = ACTIONS(1745),
    [anon_sym_union] = ACTIONS(1745),
    [anon_sym_unsafe] = ACTIONS(1745),
    [anon_sym_use] = ACTIONS(1745),
    [anon_sym_while] = ACTIONS(1745),
    [anon_sym_extern] = ACTIONS(1745),
    [anon_sym_yield] = ACTIONS(1745),
    [anon_sym_move] = ACTIONS(1745),
    [anon_sym_try] = ACTIONS(1745),
    [sym_integer_literal] = ACTIONS(1743),
    [aux_sym_string_literal_token1] = ACTIONS(1743),
    [sym_char_literal] = ACTIONS(1743),
    [anon_sym_true] = ACTIONS(1745),
    [anon_sym_false] = ACTIONS(1745),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1745),
    [sym_super] = ACTIONS(1745),
    [sym_crate] = ACTIONS(1745),
    [sym_metavariable] = ACTIONS(1743),
    [sym__raw_string_literal_start] = ACTIONS(1743),
    [sym_float_literal] = ACTIONS(1743),
  },
  [499] = {
    [sym_line_comment] = STATE(499),
    [sym_block_comment] = STATE(499),
    [ts_builtin_sym_end] = ACTIONS(1747),
    [sym_identifier] = ACTIONS(1749),
    [anon_sym_SEMI] = ACTIONS(1747),
    [anon_sym_macro_rules_BANG] = ACTIONS(1747),
    [anon_sym_LPAREN] = ACTIONS(1747),
    [anon_sym_LBRACK] = ACTIONS(1747),
    [anon_sym_LBRACE] = ACTIONS(1747),
    [anon_sym_RBRACE] = ACTIONS(1747),
    [anon_sym_STAR] = ACTIONS(1747),
    [anon_sym_u8] = ACTIONS(1749),
    [anon_sym_i8] = ACTIONS(1749),
    [anon_sym_u16] = ACTIONS(1749),
    [anon_sym_i16] = ACTIONS(1749),
    [anon_sym_u32] = ACTIONS(1749),
    [anon_sym_i32] = ACTIONS(1749),
    [anon_sym_u64] = ACTIONS(1749),
    [anon_sym_i64] = ACTIONS(1749),
    [anon_sym_u128] = ACTIONS(1749),
    [anon_sym_i128] = ACTIONS(1749),
    [anon_sym_isize] = ACTIONS(1749),
    [anon_sym_usize] = ACTIONS(1749),
    [anon_sym_f32] = ACTIONS(1749),
    [anon_sym_f64] = ACTIONS(1749),
    [anon_sym_bool] = ACTIONS(1749),
    [anon_sym_str] = ACTIONS(1749),
    [anon_sym_char] = ACTIONS(1749),
    [anon_sym_DASH] = ACTIONS(1747),
    [anon_sym_BANG] = ACTIONS(1747),
    [anon_sym_AMP] = ACTIONS(1747),
    [anon_sym_PIPE] = ACTIONS(1747),
    [anon_sym_LT] = ACTIONS(1747),
    [anon_sym_DOT_DOT] = ACTIONS(1747),
    [anon_sym_COLON_COLON] = ACTIONS(1747),
    [anon_sym_POUND] = ACTIONS(1747),
    [anon_sym_SQUOTE] = ACTIONS(1749),
    [anon_sym_async] = ACTIONS(1749),
    [anon_sym_break] = ACTIONS(1749),
    [anon_sym_const] = ACTIONS(1749),
    [anon_sym_continue] = ACTIONS(1749),
    [anon_sym_default] = ACTIONS(1749),
    [anon_sym_enum] = ACTIONS(1749),
    [anon_sym_fn] = ACTIONS(1749),
    [anon_sym_for] = ACTIONS(1749),
    [anon_sym_gen] = ACTIONS(1749),
    [anon_sym_if] = ACTIONS(1749),
    [anon_sym_impl] = ACTIONS(1749),
    [anon_sym_let] = ACTIONS(1749),
    [anon_sym_loop] = ACTIONS(1749),
    [anon_sym_match] = ACTIONS(1749),
    [anon_sym_mod] = ACTIONS(1749),
    [anon_sym_pub] = ACTIONS(1749),
    [anon_sym_return] = ACTIONS(1749),
    [anon_sym_static] = ACTIONS(1749),
    [anon_sym_struct] = ACTIONS(1749),
    [anon_sym_trait] = ACTIONS(1749),
    [anon_sym_type] = ACTIONS(1749),
    [anon_sym_union] = ACTIONS(1749),
    [anon_sym_unsafe] = ACTIONS(1749),
    [anon_sym_use] = ACTIONS(1749),
    [anon_sym_while] = ACTIONS(1749),
    [anon_sym_extern] = ACTIONS(1749),
    [anon_sym_yield] = ACTIONS(1749),
    [anon_sym_move] = ACTIONS(1749),
    [anon_sym_try] = ACTIONS(1749),
    [sym_integer_literal] = ACTIONS(1747),
    [aux_sym_string_literal_token1] = ACTIONS(1747),
    [sym_char_literal] = ACTIONS(1747),
    [anon_sym_true] = ACTIONS(1749),
    [anon_sym_false] = ACTIONS(1749),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1749),
    [sym_super] = ACTIONS(1749),
    [sym_crate] = ACTIONS(1749),
    [sym_metavariable] = ACTIONS(1747),
    [sym__raw_string_literal_start] = ACTIONS(1747),
    [sym_float_literal] = ACTIONS(1747),
  },
  [500] = {
    [sym_line_comment] = STATE(500),
    [sym_block_comment] = STATE(500),
    [ts_builtin_sym_end] = ACTIONS(1751),
    [sym_identifier] = ACTIONS(1753),
    [anon_sym_SEMI] = ACTIONS(1751),
    [anon_sym_macro_rules_BANG] = ACTIONS(1751),
    [anon_sym_LPAREN] = ACTIONS(1751),
    [anon_sym_LBRACK] = ACTIONS(1751),
    [anon_sym_LBRACE] = ACTIONS(1751),
    [anon_sym_RBRACE] = ACTIONS(1751),
    [anon_sym_STAR] = ACTIONS(1751),
    [anon_sym_u8] = ACTIONS(1753),
    [anon_sym_i8] = ACTIONS(1753),
    [anon_sym_u16] = ACTIONS(1753),
    [anon_sym_i16] = ACTIONS(1753),
    [anon_sym_u32] = ACTIONS(1753),
    [anon_sym_i32] = ACTIONS(1753),
    [anon_sym_u64] = ACTIONS(1753),
    [anon_sym_i64] = ACTIONS(1753),
    [anon_sym_u128] = ACTIONS(1753),
    [anon_sym_i128] = ACTIONS(1753),
    [anon_sym_isize] = ACTIONS(1753),
    [anon_sym_usize] = ACTIONS(1753),
    [anon_sym_f32] = ACTIONS(1753),
    [anon_sym_f64] = ACTIONS(1753),
    [anon_sym_bool] = ACTIONS(1753),
    [anon_sym_str] = ACTIONS(1753),
    [anon_sym_char] = ACTIONS(1753),
    [anon_sym_DASH] = ACTIONS(1751),
    [anon_sym_BANG] = ACTIONS(1751),
    [anon_sym_AMP] = ACTIONS(1751),
    [anon_sym_PIPE] = ACTIONS(1751),
    [anon_sym_LT] = ACTIONS(1751),
    [anon_sym_DOT_DOT] = ACTIONS(1751),
    [anon_sym_COLON_COLON] = ACTIONS(1751),
    [anon_sym_POUND] = ACTIONS(1751),
    [anon_sym_SQUOTE] = ACTIONS(1753),
    [anon_sym_async] = ACTIONS(1753),
    [anon_sym_break] = ACTIONS(1753),
    [anon_sym_const] = ACTIONS(1753),
    [anon_sym_continue] = ACTIONS(1753),
    [anon_sym_default] = ACTIONS(1753),
    [anon_sym_enum] = ACTIONS(1753),
    [anon_sym_fn] = ACTIONS(1753),
    [anon_sym_for] = ACTIONS(1753),
    [anon_sym_gen] = ACTIONS(1753),
    [anon_sym_if] = ACTIONS(1753),
    [anon_sym_impl] = ACTIONS(1753),
    [anon_sym_let] = ACTIONS(1753),
    [anon_sym_loop] = ACTIONS(1753),
    [anon_sym_match] = ACTIONS(1753),
    [anon_sym_mod] = ACTIONS(1753),
    [anon_sym_pub] = ACTIONS(1753),
    [anon_sym_return] = ACTIONS(1753),
    [anon_sym_static] = ACTIONS(1753),
    [anon_sym_struct] = ACTIONS(1753),
    [anon_sym_trait] = ACTIONS(1753),
    [anon_sym_type] = ACTIONS(1753),
    [anon_sym_union] = ACTIONS(1753),
    [anon_sym_unsafe] = ACTIONS(1753),
    [anon_sym_use] = ACTIONS(1753),
    [anon_sym_while] = ACTIONS(1753),
    [anon_sym_extern] = ACTIONS(1753),
    [anon_sym_yield] = ACTIONS(1753),
    [anon_sym_move] = ACTIONS(1753),
    [anon_sym_try] = ACTIONS(1753),
    [sym_integer_literal] = ACTIONS(1751),
    [aux_sym_string_literal_token1] = ACTIONS(1751),
    [sym_char_literal] = ACTIONS(1751),
    [anon_sym_true] = ACTIONS(1753),
    [anon_sym_false] = ACTIONS(1753),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1753),
    [sym_super] = ACTIONS(1753),
    [sym_crate] = ACTIONS(1753),
    [sym_metavariable] = ACTIONS(1751),
    [sym__raw_string_literal_start] = ACTIONS(1751),
    [sym_float_literal] = ACTIONS(1751),
  },
  [501] = {
    [sym_line_comment] = STATE(501),
    [sym_block_comment] = STATE(501),
    [ts_builtin_sym_end] = ACTIONS(1755),
    [sym_identifier] = ACTIONS(1757),
    [anon_sym_SEMI] = ACTIONS(1755),
    [anon_sym_macro_rules_BANG] = ACTIONS(1755),
    [anon_sym_LPAREN] = ACTIONS(1755),
    [anon_sym_LBRACK] = ACTIONS(1755),
    [anon_sym_LBRACE] = ACTIONS(1755),
    [anon_sym_RBRACE] = ACTIONS(1755),
    [anon_sym_STAR] = ACTIONS(1755),
    [anon_sym_u8] = ACTIONS(1757),
    [anon_sym_i8] = ACTIONS(1757),
    [anon_sym_u16] = ACTIONS(1757),
    [anon_sym_i16] = ACTIONS(1757),
    [anon_sym_u32] = ACTIONS(1757),
    [anon_sym_i32] = ACTIONS(1757),
    [anon_sym_u64] = ACTIONS(1757),
    [anon_sym_i64] = ACTIONS(1757),
    [anon_sym_u128] = ACTIONS(1757),
    [anon_sym_i128] = ACTIONS(1757),
    [anon_sym_isize] = ACTIONS(1757),
    [anon_sym_usize] = ACTIONS(1757),
    [anon_sym_f32] = ACTIONS(1757),
    [anon_sym_f64] = ACTIONS(1757),
    [anon_sym_bool] = ACTIONS(1757),
    [anon_sym_str] = ACTIONS(1757),
    [anon_sym_char] = ACTIONS(1757),
    [anon_sym_DASH] = ACTIONS(1755),
    [anon_sym_BANG] = ACTIONS(1755),
    [anon_sym_AMP] = ACTIONS(1755),
    [anon_sym_PIPE] = ACTIONS(1755),
    [anon_sym_LT] = ACTIONS(1755),
    [anon_sym_DOT_DOT] = ACTIONS(1755),
    [anon_sym_COLON_COLON] = ACTIONS(1755),
    [anon_sym_POUND] = ACTIONS(1755),
    [anon_sym_SQUOTE] = ACTIONS(1757),
    [anon_sym_async] = ACTIONS(1757),
    [anon_sym_break] = ACTIONS(1757),
    [anon_sym_const] = ACTIONS(1757),
    [anon_sym_continue] = ACTIONS(1757),
    [anon_sym_default] = ACTIONS(1757),
    [anon_sym_enum] = ACTIONS(1757),
    [anon_sym_fn] = ACTIONS(1757),
    [anon_sym_for] = ACTIONS(1757),
    [anon_sym_gen] = ACTIONS(1757),
    [anon_sym_if] = ACTIONS(1757),
    [anon_sym_impl] = ACTIONS(1757),
    [anon_sym_let] = ACTIONS(1757),
    [anon_sym_loop] = ACTIONS(1757),
    [anon_sym_match] = ACTIONS(1757),
    [anon_sym_mod] = ACTIONS(1757),
    [anon_sym_pub] = ACTIONS(1757),
    [anon_sym_return] = ACTIONS(1757),
    [anon_sym_static] = ACTIONS(1757),
    [anon_sym_struct] = ACTIONS(1757),
    [anon_sym_trait] = ACTIONS(1757),
    [anon_sym_type] = ACTIONS(1757),
    [anon_sym_union] = ACTIONS(1757),
    [anon_sym_unsafe] = ACTIONS(1757),
    [anon_sym_use] = ACTIONS(1757),
    [anon_sym_while] = ACTIONS(1757),
    [anon_sym_extern] = ACTIONS(1757),
    [anon_sym_yield] = ACTIONS(1757),
    [anon_sym_move] = ACTIONS(1757),
    [anon_sym_try] = ACTIONS(1757),
    [sym_integer_literal] = ACTIONS(1755),
    [aux_sym_string_literal_token1] = ACTIONS(1755),
    [sym_char_literal] = ACTIONS(1755),
    [anon_sym_true] = ACTIONS(1757),
    [anon_sym_false] = ACTIONS(1757),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1757),
    [sym_super] = ACTIONS(1757),
    [sym_crate] = ACTIONS(1757),
    [sym_metavariable] = ACTIONS(1755),
    [sym__raw_string_literal_start] = ACTIONS(1755),
    [sym_float_literal] = ACTIONS(1755),
  },
  [502] = {
    [sym_line_comment] = STATE(502),
    [sym_block_comment] = STATE(502),
    [ts_builtin_sym_end] = ACTIONS(1759),
    [sym_identifier] = ACTIONS(1761),
    [anon_sym_SEMI] = ACTIONS(1759),
    [anon_sym_macro_rules_BANG] = ACTIONS(1759),
    [anon_sym_LPAREN] = ACTIONS(1759),
    [anon_sym_LBRACK] = ACTIONS(1759),
    [anon_sym_LBRACE] = ACTIONS(1759),
    [anon_sym_RBRACE] = ACTIONS(1759),
    [anon_sym_STAR] = ACTIONS(1759),
    [anon_sym_u8] = ACTIONS(1761),
    [anon_sym_i8] = ACTIONS(1761),
    [anon_sym_u16] = ACTIONS(1761),
    [anon_sym_i16] = ACTIONS(1761),
    [anon_sym_u32] = ACTIONS(1761),
    [anon_sym_i32] = ACTIONS(1761),
    [anon_sym_u64] = ACTIONS(1761),
    [anon_sym_i64] = ACTIONS(1761),
    [anon_sym_u128] = ACTIONS(1761),
    [anon_sym_i128] = ACTIONS(1761),
    [anon_sym_isize] = ACTIONS(1761),
    [anon_sym_usize] = ACTIONS(1761),
    [anon_sym_f32] = ACTIONS(1761),
    [anon_sym_f64] = ACTIONS(1761),
    [anon_sym_bool] = ACTIONS(1761),
    [anon_sym_str] = ACTIONS(1761),
    [anon_sym_char] = ACTIONS(1761),
    [anon_sym_DASH] = ACTIONS(1759),
    [anon_sym_BANG] = ACTIONS(1759),
    [anon_sym_AMP] = ACTIONS(1759),
    [anon_sym_PIPE] = ACTIONS(1759),
    [anon_sym_LT] = ACTIONS(1759),
    [anon_sym_DOT_DOT] = ACTIONS(1759),
    [anon_sym_COLON_COLON] = ACTIONS(1759),
    [anon_sym_POUND] = ACTIONS(1759),
    [anon_sym_SQUOTE] = ACTIONS(1761),
    [anon_sym_async] = ACTIONS(1761),
    [anon_sym_break] = ACTIONS(1761),
    [anon_sym_const] = ACTIONS(1761),
    [anon_sym_continue] = ACTIONS(1761),
    [anon_sym_default] = ACTIONS(1761),
    [anon_sym_enum] = ACTIONS(1761),
    [anon_sym_fn] = ACTIONS(1761),
    [anon_sym_for] = ACTIONS(1761),
    [anon_sym_gen] = ACTIONS(1761),
    [anon_sym_if] = ACTIONS(1761),
    [anon_sym_impl] = ACTIONS(1761),
    [anon_sym_let] = ACTIONS(1761),
    [anon_sym_loop] = ACTIONS(1761),
    [anon_sym_match] = ACTIONS(1761),
    [anon_sym_mod] = ACTIONS(1761),
    [anon_sym_pub] = ACTIONS(1761),
    [anon_sym_return] = ACTIONS(1761),
    [anon_sym_static] = ACTIONS(1761),
    [anon_sym_struct] = ACTIONS(1761),
    [anon_sym_trait] = ACTIONS(1761),
    [anon_sym_type] = ACTIONS(1761),
    [anon_sym_union] = ACTIONS(1761),
    [anon_sym_unsafe] = ACTIONS(1761),
    [anon_sym_use] = ACTIONS(1761),
    [anon_sym_while] = ACTIONS(1761),
    [anon_sym_extern] = ACTIONS(1761),
    [anon_sym_yield] = ACTIONS(1761),
    [anon_sym_move] = ACTIONS(1761),
    [anon_sym_try] = ACTIONS(1761),
    [sym_integer_literal] = ACTIONS(1759),
    [aux_sym_string_literal_token1] = ACTIONS(1759),
    [sym_char_literal] = ACTIONS(1759),
    [anon_sym_true] = ACTIONS(1761),
    [anon_sym_false] = ACTIONS(1761),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1761),
    [sym_super] = ACTIONS(1761),
    [sym_crate] = ACTIONS(1761),
    [sym_metavariable] = ACTIONS(1759),
    [sym__raw_string_literal_start] = ACTIONS(1759),
    [sym_float_literal] = ACTIONS(1759),
  },
  [503] = {
    [sym_line_comment] = STATE(503),
    [sym_block_comment] = STATE(503),
    [ts_builtin_sym_end] = ACTIONS(1763),
    [sym_identifier] = ACTIONS(1765),
    [anon_sym_SEMI] = ACTIONS(1763),
    [anon_sym_macro_rules_BANG] = ACTIONS(1763),
    [anon_sym_LPAREN] = ACTIONS(1763),
    [anon_sym_LBRACK] = ACTIONS(1763),
    [anon_sym_LBRACE] = ACTIONS(1763),
    [anon_sym_RBRACE] = ACTIONS(1763),
    [anon_sym_STAR] = ACTIONS(1763),
    [anon_sym_u8] = ACTIONS(1765),
    [anon_sym_i8] = ACTIONS(1765),
    [anon_sym_u16] = ACTIONS(1765),
    [anon_sym_i16] = ACTIONS(1765),
    [anon_sym_u32] = ACTIONS(1765),
    [anon_sym_i32] = ACTIONS(1765),
    [anon_sym_u64] = ACTIONS(1765),
    [anon_sym_i64] = ACTIONS(1765),
    [anon_sym_u128] = ACTIONS(1765),
    [anon_sym_i128] = ACTIONS(1765),
    [anon_sym_isize] = ACTIONS(1765),
    [anon_sym_usize] = ACTIONS(1765),
    [anon_sym_f32] = ACTIONS(1765),
    [anon_sym_f64] = ACTIONS(1765),
    [anon_sym_bool] = ACTIONS(1765),
    [anon_sym_str] = ACTIONS(1765),
    [anon_sym_char] = ACTIONS(1765),
    [anon_sym_DASH] = ACTIONS(1763),
    [anon_sym_BANG] = ACTIONS(1763),
    [anon_sym_AMP] = ACTIONS(1763),
    [anon_sym_PIPE] = ACTIONS(1763),
    [anon_sym_LT] = ACTIONS(1763),
    [anon_sym_DOT_DOT] = ACTIONS(1763),
    [anon_sym_COLON_COLON] = ACTIONS(1763),
    [anon_sym_POUND] = ACTIONS(1763),
    [anon_sym_SQUOTE] = ACTIONS(1765),
    [anon_sym_async] = ACTIONS(1765),
    [anon_sym_break] = ACTIONS(1765),
    [anon_sym_const] = ACTIONS(1765),
    [anon_sym_continue] = ACTIONS(1765),
    [anon_sym_default] = ACTIONS(1765),
    [anon_sym_enum] = ACTIONS(1765),
    [anon_sym_fn] = ACTIONS(1765),
    [anon_sym_for] = ACTIONS(1765),
    [anon_sym_gen] = ACTIONS(1765),
    [anon_sym_if] = ACTIONS(1765),
    [anon_sym_impl] = ACTIONS(1765),
    [anon_sym_let] = ACTIONS(1765),
    [anon_sym_loop] = ACTIONS(1765),
    [anon_sym_match] = ACTIONS(1765),
    [anon_sym_mod] = ACTIONS(1765),
    [anon_sym_pub] = ACTIONS(1765),
    [anon_sym_return] = ACTIONS(1765),
    [anon_sym_static] = ACTIONS(1765),
    [anon_sym_struct] = ACTIONS(1765),
    [anon_sym_trait] = ACTIONS(1765),
    [anon_sym_type] = ACTIONS(1765),
    [anon_sym_union] = ACTIONS(1765),
    [anon_sym_unsafe] = ACTIONS(1765),
    [anon_sym_use] = ACTIONS(1765),
    [anon_sym_while] = ACTIONS(1765),
    [anon_sym_extern] = ACTIONS(1765),
    [anon_sym_yield] = ACTIONS(1765),
    [anon_sym_move] = ACTIONS(1765),
    [anon_sym_try] = ACTIONS(1765),
    [sym_integer_literal] = ACTIONS(1763),
    [aux_sym_string_literal_token1] = ACTIONS(1763),
    [sym_char_literal] = ACTIONS(1763),
    [anon_sym_true] = ACTIONS(1765),
    [anon_sym_false] = ACTIONS(1765),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1765),
    [sym_super] = ACTIONS(1765),
    [sym_crate] = ACTIONS(1765),
    [sym_metavariable] = ACTIONS(1763),
    [sym__raw_string_literal_start] = ACTIONS(1763),
    [sym_float_literal] = ACTIONS(1763),
  },
  [504] = {
    [sym_line_comment] = STATE(504),
    [sym_block_comment] = STATE(504),
    [ts_builtin_sym_end] = ACTIONS(1767),
    [sym_identifier] = ACTIONS(1769),
    [anon_sym_SEMI] = ACTIONS(1767),
    [anon_sym_macro_rules_BANG] = ACTIONS(1767),
    [anon_sym_LPAREN] = ACTIONS(1767),
    [anon_sym_LBRACK] = ACTIONS(1767),
    [anon_sym_LBRACE] = ACTIONS(1767),
    [anon_sym_RBRACE] = ACTIONS(1767),
    [anon_sym_STAR] = ACTIONS(1767),
    [anon_sym_u8] = ACTIONS(1769),
    [anon_sym_i8] = ACTIONS(1769),
    [anon_sym_u16] = ACTIONS(1769),
    [anon_sym_i16] = ACTIONS(1769),
    [anon_sym_u32] = ACTIONS(1769),
    [anon_sym_i32] = ACTIONS(1769),
    [anon_sym_u64] = ACTIONS(1769),
    [anon_sym_i64] = ACTIONS(1769),
    [anon_sym_u128] = ACTIONS(1769),
    [anon_sym_i128] = ACTIONS(1769),
    [anon_sym_isize] = ACTIONS(1769),
    [anon_sym_usize] = ACTIONS(1769),
    [anon_sym_f32] = ACTIONS(1769),
    [anon_sym_f64] = ACTIONS(1769),
    [anon_sym_bool] = ACTIONS(1769),
    [anon_sym_str] = ACTIONS(1769),
    [anon_sym_char] = ACTIONS(1769),
    [anon_sym_DASH] = ACTIONS(1767),
    [anon_sym_BANG] = ACTIONS(1767),
    [anon_sym_AMP] = ACTIONS(1767),
    [anon_sym_PIPE] = ACTIONS(1767),
    [anon_sym_LT] = ACTIONS(1767),
    [anon_sym_DOT_DOT] = ACTIONS(1767),
    [anon_sym_COLON_COLON] = ACTIONS(1767),
    [anon_sym_POUND] = ACTIONS(1767),
    [anon_sym_SQUOTE] = ACTIONS(1769),
    [anon_sym_async] = ACTIONS(1769),
    [anon_sym_break] = ACTIONS(1769),
    [anon_sym_const] = ACTIONS(1769),
    [anon_sym_continue] = ACTIONS(1769),
    [anon_sym_default] = ACTIONS(1769),
    [anon_sym_enum] = ACTIONS(1769),
    [anon_sym_fn] = ACTIONS(1769),
    [anon_sym_for] = ACTIONS(1769),
    [anon_sym_gen] = ACTIONS(1769),
    [anon_sym_if] = ACTIONS(1769),
    [anon_sym_impl] = ACTIONS(1769),
    [anon_sym_let] = ACTIONS(1769),
    [anon_sym_loop] = ACTIONS(1769),
    [anon_sym_match] = ACTIONS(1769),
    [anon_sym_mod] = ACTIONS(1769),
    [anon_sym_pub] = ACTIONS(1769),
    [anon_sym_return] = ACTIONS(1769),
    [anon_sym_static] = ACTIONS(1769),
    [anon_sym_struct] = ACTIONS(1769),
    [anon_sym_trait] = ACTIONS(1769),
    [anon_sym_type] = ACTIONS(1769),
    [anon_sym_union] = ACTIONS(1769),
    [anon_sym_unsafe] = ACTIONS(1769),
    [anon_sym_use] = ACTIONS(1769),
    [anon_sym_while] = ACTIONS(1769),
    [anon_sym_extern] = ACTIONS(1769),
    [anon_sym_yield] = ACTIONS(1769),
    [anon_sym_move] = ACTIONS(1769),
    [anon_sym_try] = ACTIONS(1769),
    [sym_integer_literal] = ACTIONS(1767),
    [aux_sym_string_literal_token1] = ACTIONS(1767),
    [sym_char_literal] = ACTIONS(1767),
    [anon_sym_true] = ACTIONS(1769),
    [anon_sym_false] = ACTIONS(1769),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1769),
    [sym_super] = ACTIONS(1769),
    [sym_crate] = ACTIONS(1769),
    [sym_metavariable] = ACTIONS(1767),
    [sym__raw_string_literal_start] = ACTIONS(1767),
    [sym_float_literal] = ACTIONS(1767),
  },
  [505] = {
    [sym_line_comment] = STATE(505),
    [sym_block_comment] = STATE(505),
    [ts_builtin_sym_end] = ACTIONS(1771),
    [sym_identifier] = ACTIONS(1773),
    [anon_sym_SEMI] = ACTIONS(1771),
    [anon_sym_macro_rules_BANG] = ACTIONS(1771),
    [anon_sym_LPAREN] = ACTIONS(1771),
    [anon_sym_LBRACK] = ACTIONS(1771),
    [anon_sym_LBRACE] = ACTIONS(1771),
    [anon_sym_RBRACE] = ACTIONS(1771),
    [anon_sym_STAR] = ACTIONS(1771),
    [anon_sym_u8] = ACTIONS(1773),
    [anon_sym_i8] = ACTIONS(1773),
    [anon_sym_u16] = ACTIONS(1773),
    [anon_sym_i16] = ACTIONS(1773),
    [anon_sym_u32] = ACTIONS(1773),
    [anon_sym_i32] = ACTIONS(1773),
    [anon_sym_u64] = ACTIONS(1773),
    [anon_sym_i64] = ACTIONS(1773),
    [anon_sym_u128] = ACTIONS(1773),
    [anon_sym_i128] = ACTIONS(1773),
    [anon_sym_isize] = ACTIONS(1773),
    [anon_sym_usize] = ACTIONS(1773),
    [anon_sym_f32] = ACTIONS(1773),
    [anon_sym_f64] = ACTIONS(1773),
    [anon_sym_bool] = ACTIONS(1773),
    [anon_sym_str] = ACTIONS(1773),
    [anon_sym_char] = ACTIONS(1773),
    [anon_sym_DASH] = ACTIONS(1771),
    [anon_sym_BANG] = ACTIONS(1771),
    [anon_sym_AMP] = ACTIONS(1771),
    [anon_sym_PIPE] = ACTIONS(1771),
    [anon_sym_LT] = ACTIONS(1771),
    [anon_sym_DOT_DOT] = ACTIONS(1771),
    [anon_sym_COLON_COLON] = ACTIONS(1771),
    [anon_sym_POUND] = ACTIONS(1771),
    [anon_sym_SQUOTE] = ACTIONS(1773),
    [anon_sym_async] = ACTIONS(1773),
    [anon_sym_break] = ACTIONS(1773),
    [anon_sym_const] = ACTIONS(1773),
    [anon_sym_continue] = ACTIONS(1773),
    [anon_sym_default] = ACTIONS(1773),
    [anon_sym_enum] = ACTIONS(1773),
    [anon_sym_fn] = ACTIONS(1773),
    [anon_sym_for] = ACTIONS(1773),
    [anon_sym_gen] = ACTIONS(1773),
    [anon_sym_if] = ACTIONS(1773),
    [anon_sym_impl] = ACTIONS(1773),
    [anon_sym_let] = ACTIONS(1773),
    [anon_sym_loop] = ACTIONS(1773),
    [anon_sym_match] = ACTIONS(1773),
    [anon_sym_mod] = ACTIONS(1773),
    [anon_sym_pub] = ACTIONS(1773),
    [anon_sym_return] = ACTIONS(1773),
    [anon_sym_static] = ACTIONS(1773),
    [anon_sym_struct] = ACTIONS(1773),
    [anon_sym_trait] = ACTIONS(1773),
    [anon_sym_type] = ACTIONS(1773),
    [anon_sym_union] = ACTIONS(1773),
    [anon_sym_unsafe] = ACTIONS(1773),
    [anon_sym_use] = ACTIONS(1773),
    [anon_sym_while] = ACTIONS(1773),
    [anon_sym_extern] = ACTIONS(1773),
    [anon_sym_yield] = ACTIONS(1773),
    [anon_sym_move] = ACTIONS(1773),
    [anon_sym_try] = ACTIONS(1773),
    [sym_integer_literal] = ACTIONS(1771),
    [aux_sym_string_literal_token1] = ACTIONS(1771),
    [sym_char_literal] = ACTIONS(1771),
    [anon_sym_true] = ACTIONS(1773),
    [anon_sym_false] = ACTIONS(1773),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1773),
    [sym_super] = ACTIONS(1773),
    [sym_crate] = ACTIONS(1773),
    [sym_metavariable] = ACTIONS(1771),
    [sym__raw_string_literal_start] = ACTIONS(1771),
    [sym_float_literal] = ACTIONS(1771),
  },
  [506] = {
    [sym_line_comment] = STATE(506),
    [sym_block_comment] = STATE(506),
    [ts_builtin_sym_end] = ACTIONS(1775),
    [sym_identifier] = ACTIONS(1777),
    [anon_sym_SEMI] = ACTIONS(1775),
    [anon_sym_macro_rules_BANG] = ACTIONS(1775),
    [anon_sym_LPAREN] = ACTIONS(1775),
    [anon_sym_LBRACK] = ACTIONS(1775),
    [anon_sym_LBRACE] = ACTIONS(1775),
    [anon_sym_RBRACE] = ACTIONS(1775),
    [anon_sym_STAR] = ACTIONS(1775),
    [anon_sym_u8] = ACTIONS(1777),
    [anon_sym_i8] = ACTIONS(1777),
    [anon_sym_u16] = ACTIONS(1777),
    [anon_sym_i16] = ACTIONS(1777),
    [anon_sym_u32] = ACTIONS(1777),
    [anon_sym_i32] = ACTIONS(1777),
    [anon_sym_u64] = ACTIONS(1777),
    [anon_sym_i64] = ACTIONS(1777),
    [anon_sym_u128] = ACTIONS(1777),
    [anon_sym_i128] = ACTIONS(1777),
    [anon_sym_isize] = ACTIONS(1777),
    [anon_sym_usize] = ACTIONS(1777),
    [anon_sym_f32] = ACTIONS(1777),
    [anon_sym_f64] = ACTIONS(1777),
    [anon_sym_bool] = ACTIONS(1777),
    [anon_sym_str] = ACTIONS(1777),
    [anon_sym_char] = ACTIONS(1777),
    [anon_sym_DASH] = ACTIONS(1775),
    [anon_sym_BANG] = ACTIONS(1775),
    [anon_sym_AMP] = ACTIONS(1775),
    [anon_sym_PIPE] = ACTIONS(1775),
    [anon_sym_LT] = ACTIONS(1775),
    [anon_sym_DOT_DOT] = ACTIONS(1775),
    [anon_sym_COLON_COLON] = ACTIONS(1775),
    [anon_sym_POUND] = ACTIONS(1775),
    [anon_sym_SQUOTE] = ACTIONS(1777),
    [anon_sym_async] = ACTIONS(1777),
    [anon_sym_break] = ACTIONS(1777),
    [anon_sym_const] = ACTIONS(1777),
    [anon_sym_continue] = ACTIONS(1777),
    [anon_sym_default] = ACTIONS(1777),
    [anon_sym_enum] = ACTIONS(1777),
    [anon_sym_fn] = ACTIONS(1777),
    [anon_sym_for] = ACTIONS(1777),
    [anon_sym_gen] = ACTIONS(1777),
    [anon_sym_if] = ACTIONS(1777),
    [anon_sym_impl] = ACTIONS(1777),
    [anon_sym_let] = ACTIONS(1777),
    [anon_sym_loop] = ACTIONS(1777),
    [anon_sym_match] = ACTIONS(1777),
    [anon_sym_mod] = ACTIONS(1777),
    [anon_sym_pub] = ACTIONS(1777),
    [anon_sym_return] = ACTIONS(1777),
    [anon_sym_static] = ACTIONS(1777),
    [anon_sym_struct] = ACTIONS(1777),
    [anon_sym_trait] = ACTIONS(1777),
    [anon_sym_type] = ACTIONS(1777),
    [anon_sym_union] = ACTIONS(1777),
    [anon_sym_unsafe] = ACTIONS(1777),
    [anon_sym_use] = ACTIONS(1777),
    [anon_sym_while] = ACTIONS(1777),
    [anon_sym_extern] = ACTIONS(1777),
    [anon_sym_yield] = ACTIONS(1777),
    [anon_sym_move] = ACTIONS(1777),
    [anon_sym_try] = ACTIONS(1777),
    [sym_integer_literal] = ACTIONS(1775),
    [aux_sym_string_literal_token1] = ACTIONS(1775),
    [sym_char_literal] = ACTIONS(1775),
    [anon_sym_true] = ACTIONS(1777),
    [anon_sym_false] = ACTIONS(1777),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1777),
    [sym_super] = ACTIONS(1777),
    [sym_crate] = ACTIONS(1777),
    [sym_metavariable] = ACTIONS(1775),
    [sym__raw_string_literal_start] = ACTIONS(1775),
    [sym_float_literal] = ACTIONS(1775),
  },
  [507] = {
    [sym_line_comment] = STATE(507),
    [sym_block_comment] = STATE(507),
    [ts_builtin_sym_end] = ACTIONS(1779),
    [sym_identifier] = ACTIONS(1781),
    [anon_sym_SEMI] = ACTIONS(1779),
    [anon_sym_macro_rules_BANG] = ACTIONS(1779),
    [anon_sym_LPAREN] = ACTIONS(1779),
    [anon_sym_LBRACK] = ACTIONS(1779),
    [anon_sym_LBRACE] = ACTIONS(1779),
    [anon_sym_RBRACE] = ACTIONS(1779),
    [anon_sym_STAR] = ACTIONS(1779),
    [anon_sym_u8] = ACTIONS(1781),
    [anon_sym_i8] = ACTIONS(1781),
    [anon_sym_u16] = ACTIONS(1781),
    [anon_sym_i16] = ACTIONS(1781),
    [anon_sym_u32] = ACTIONS(1781),
    [anon_sym_i32] = ACTIONS(1781),
    [anon_sym_u64] = ACTIONS(1781),
    [anon_sym_i64] = ACTIONS(1781),
    [anon_sym_u128] = ACTIONS(1781),
    [anon_sym_i128] = ACTIONS(1781),
    [anon_sym_isize] = ACTIONS(1781),
    [anon_sym_usize] = ACTIONS(1781),
    [anon_sym_f32] = ACTIONS(1781),
    [anon_sym_f64] = ACTIONS(1781),
    [anon_sym_bool] = ACTIONS(1781),
    [anon_sym_str] = ACTIONS(1781),
    [anon_sym_char] = ACTIONS(1781),
    [anon_sym_DASH] = ACTIONS(1779),
    [anon_sym_BANG] = ACTIONS(1779),
    [anon_sym_AMP] = ACTIONS(1779),
    [anon_sym_PIPE] = ACTIONS(1779),
    [anon_sym_LT] = ACTIONS(1779),
    [anon_sym_DOT_DOT] = ACTIONS(1779),
    [anon_sym_COLON_COLON] = ACTIONS(1779),
    [anon_sym_POUND] = ACTIONS(1779),
    [anon_sym_SQUOTE] = ACTIONS(1781),
    [anon_sym_async] = ACTIONS(1781),
    [anon_sym_break] = ACTIONS(1781),
    [anon_sym_const] = ACTIONS(1781),
    [anon_sym_continue] = ACTIONS(1781),
    [anon_sym_default] = ACTIONS(1781),
    [anon_sym_enum] = ACTIONS(1781),
    [anon_sym_fn] = ACTIONS(1781),
    [anon_sym_for] = ACTIONS(1781),
    [anon_sym_gen] = ACTIONS(1781),
    [anon_sym_if] = ACTIONS(1781),
    [anon_sym_impl] = ACTIONS(1781),
    [anon_sym_let] = ACTIONS(1781),
    [anon_sym_loop] = ACTIONS(1781),
    [anon_sym_match] = ACTIONS(1781),
    [anon_sym_mod] = ACTIONS(1781),
    [anon_sym_pub] = ACTIONS(1781),
    [anon_sym_return] = ACTIONS(1781),
    [anon_sym_static] = ACTIONS(1781),
    [anon_sym_struct] = ACTIONS(1781),
    [anon_sym_trait] = ACTIONS(1781),
    [anon_sym_type] = ACTIONS(1781),
    [anon_sym_union] = ACTIONS(1781),
    [anon_sym_unsafe] = ACTIONS(1781),
    [anon_sym_use] = ACTIONS(1781),
    [anon_sym_while] = ACTIONS(1781),
    [anon_sym_extern] = ACTIONS(1781),
    [anon_sym_yield] = ACTIONS(1781),
    [anon_sym_move] = ACTIONS(1781),
    [anon_sym_try] = ACTIONS(1781),
    [sym_integer_literal] = ACTIONS(1779),
    [aux_sym_string_literal_token1] = ACTIONS(1779),
    [sym_char_literal] = ACTIONS(1779),
    [anon_sym_true] = ACTIONS(1781),
    [anon_sym_false] = ACTIONS(1781),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1781),
    [sym_super] = ACTIONS(1781),
    [sym_crate] = ACTIONS(1781),
    [sym_metavariable] = ACTIONS(1779),
    [sym__raw_string_literal_start] = ACTIONS(1779),
    [sym_float_literal] = ACTIONS(1779),
  },
  [508] = {
    [sym_line_comment] = STATE(508),
    [sym_block_comment] = STATE(508),
    [ts_builtin_sym_end] = ACTIONS(1783),
    [sym_identifier] = ACTIONS(1785),
    [anon_sym_SEMI] = ACTIONS(1783),
    [anon_sym_macro_rules_BANG] = ACTIONS(1783),
    [anon_sym_LPAREN] = ACTIONS(1783),
    [anon_sym_LBRACK] = ACTIONS(1783),
    [anon_sym_LBRACE] = ACTIONS(1783),
    [anon_sym_RBRACE] = ACTIONS(1783),
    [anon_sym_STAR] = ACTIONS(1783),
    [anon_sym_u8] = ACTIONS(1785),
    [anon_sym_i8] = ACTIONS(1785),
    [anon_sym_u16] = ACTIONS(1785),
    [anon_sym_i16] = ACTIONS(1785),
    [anon_sym_u32] = ACTIONS(1785),
    [anon_sym_i32] = ACTIONS(1785),
    [anon_sym_u64] = ACTIONS(1785),
    [anon_sym_i64] = ACTIONS(1785),
    [anon_sym_u128] = ACTIONS(1785),
    [anon_sym_i128] = ACTIONS(1785),
    [anon_sym_isize] = ACTIONS(1785),
    [anon_sym_usize] = ACTIONS(1785),
    [anon_sym_f32] = ACTIONS(1785),
    [anon_sym_f64] = ACTIONS(1785),
    [anon_sym_bool] = ACTIONS(1785),
    [anon_sym_str] = ACTIONS(1785),
    [anon_sym_char] = ACTIONS(1785),
    [anon_sym_DASH] = ACTIONS(1783),
    [anon_sym_BANG] = ACTIONS(1783),
    [anon_sym_AMP] = ACTIONS(1783),
    [anon_sym_PIPE] = ACTIONS(1783),
    [anon_sym_LT] = ACTIONS(1783),
    [anon_sym_DOT_DOT] = ACTIONS(1783),
    [anon_sym_COLON_COLON] = ACTIONS(1783),
    [anon_sym_POUND] = ACTIONS(1783),
    [anon_sym_SQUOTE] = ACTIONS(1785),
    [anon_sym_async] = ACTIONS(1785),
    [anon_sym_break] = ACTIONS(1785),
    [anon_sym_const] = ACTIONS(1785),
    [anon_sym_continue] = ACTIONS(1785),
    [anon_sym_default] = ACTIONS(1785),
    [anon_sym_enum] = ACTIONS(1785),
    [anon_sym_fn] = ACTIONS(1785),
    [anon_sym_for] = ACTIONS(1785),
    [anon_sym_gen] = ACTIONS(1785),
    [anon_sym_if] = ACTIONS(1785),
    [anon_sym_impl] = ACTIONS(1785),
    [anon_sym_let] = ACTIONS(1785),
    [anon_sym_loop] = ACTIONS(1785),
    [anon_sym_match] = ACTIONS(1785),
    [anon_sym_mod] = ACTIONS(1785),
    [anon_sym_pub] = ACTIONS(1785),
    [anon_sym_return] = ACTIONS(1785),
    [anon_sym_static] = ACTIONS(1785),
    [anon_sym_struct] = ACTIONS(1785),
    [anon_sym_trait] = ACTIONS(1785),
    [anon_sym_type] = ACTIONS(1785),
    [anon_sym_union] = ACTIONS(1785),
    [anon_sym_unsafe] = ACTIONS(1785),
    [anon_sym_use] = ACTIONS(1785),
    [anon_sym_while] = ACTIONS(1785),
    [anon_sym_extern] = ACTIONS(1785),
    [anon_sym_yield] = ACTIONS(1785),
    [anon_sym_move] = ACTIONS(1785),
    [anon_sym_try] = ACTIONS(1785),
    [sym_integer_literal] = ACTIONS(1783),
    [aux_sym_string_literal_token1] = ACTIONS(1783),
    [sym_char_literal] = ACTIONS(1783),
    [anon_sym_true] = ACTIONS(1785),
    [anon_sym_false] = ACTIONS(1785),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1785),
    [sym_super] = ACTIONS(1785),
    [sym_crate] = ACTIONS(1785),
    [sym_metavariable] = ACTIONS(1783),
    [sym__raw_string_literal_start] = ACTIONS(1783),
    [sym_float_literal] = ACTIONS(1783),
  },
  [509] = {
    [sym_line_comment] = STATE(509),
    [sym_block_comment] = STATE(509),
    [ts_builtin_sym_end] = ACTIONS(1787),
    [sym_identifier] = ACTIONS(1789),
    [anon_sym_SEMI] = ACTIONS(1787),
    [anon_sym_macro_rules_BANG] = ACTIONS(1787),
    [anon_sym_LPAREN] = ACTIONS(1787),
    [anon_sym_LBRACK] = ACTIONS(1787),
    [anon_sym_LBRACE] = ACTIONS(1787),
    [anon_sym_RBRACE] = ACTIONS(1787),
    [anon_sym_STAR] = ACTIONS(1787),
    [anon_sym_u8] = ACTIONS(1789),
    [anon_sym_i8] = ACTIONS(1789),
    [anon_sym_u16] = ACTIONS(1789),
    [anon_sym_i16] = ACTIONS(1789),
    [anon_sym_u32] = ACTIONS(1789),
    [anon_sym_i32] = ACTIONS(1789),
    [anon_sym_u64] = ACTIONS(1789),
    [anon_sym_i64] = ACTIONS(1789),
    [anon_sym_u128] = ACTIONS(1789),
    [anon_sym_i128] = ACTIONS(1789),
    [anon_sym_isize] = ACTIONS(1789),
    [anon_sym_usize] = ACTIONS(1789),
    [anon_sym_f32] = ACTIONS(1789),
    [anon_sym_f64] = ACTIONS(1789),
    [anon_sym_bool] = ACTIONS(1789),
    [anon_sym_str] = ACTIONS(1789),
    [anon_sym_char] = ACTIONS(1789),
    [anon_sym_DASH] = ACTIONS(1787),
    [anon_sym_BANG] = ACTIONS(1787),
    [anon_sym_AMP] = ACTIONS(1787),
    [anon_sym_PIPE] = ACTIONS(1787),
    [anon_sym_LT] = ACTIONS(1787),
    [anon_sym_DOT_DOT] = ACTIONS(1787),
    [anon_sym_COLON_COLON] = ACTIONS(1787),
    [anon_sym_POUND] = ACTIONS(1787),
    [anon_sym_SQUOTE] = ACTIONS(1789),
    [anon_sym_async] = ACTIONS(1789),
    [anon_sym_break] = ACTIONS(1789),
    [anon_sym_const] = ACTIONS(1789),
    [anon_sym_continue] = ACTIONS(1789),
    [anon_sym_default] = ACTIONS(1789),
    [anon_sym_enum] = ACTIONS(1789),
    [anon_sym_fn] = ACTIONS(1789),
    [anon_sym_for] = ACTIONS(1789),
    [anon_sym_gen] = ACTIONS(1789),
    [anon_sym_if] = ACTIONS(1789),
    [anon_sym_impl] = ACTIONS(1789),
    [anon_sym_let] = ACTIONS(1789),
    [anon_sym_loop] = ACTIONS(1789),
    [anon_sym_match] = ACTIONS(1789),
    [anon_sym_mod] = ACTIONS(1789),
    [anon_sym_pub] = ACTIONS(1789),
    [anon_sym_return] = ACTIONS(1789),
    [anon_sym_static] = ACTIONS(1789),
    [anon_sym_struct] = ACTIONS(1789),
    [anon_sym_trait] = ACTIONS(1789),
    [anon_sym_type] = ACTIONS(1789),
    [anon_sym_union] = ACTIONS(1789),
    [anon_sym_unsafe] = ACTIONS(1789),
    [anon_sym_use] = ACTIONS(1789),
    [anon_sym_while] = ACTIONS(1789),
    [anon_sym_extern] = ACTIONS(1789),
    [anon_sym_yield] = ACTIONS(1789),
    [anon_sym_move] = ACTIONS(1789),
    [anon_sym_try] = ACTIONS(1789),
    [sym_integer_literal] = ACTIONS(1787),
    [aux_sym_string_literal_token1] = ACTIONS(1787),
    [sym_char_literal] = ACTIONS(1787),
    [anon_sym_true] = ACTIONS(1789),
    [anon_sym_false] = ACTIONS(1789),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1789),
    [sym_super] = ACTIONS(1789),
    [sym_crate] = ACTIONS(1789),
    [sym_metavariable] = ACTIONS(1787),
    [sym__raw_string_literal_start] = ACTIONS(1787),
    [sym_float_literal] = ACTIONS(1787),
  },
  [510] = {
    [sym_line_comment] = STATE(510),
    [sym_block_comment] = STATE(510),
    [ts_builtin_sym_end] = ACTIONS(1791),
    [sym_identifier] = ACTIONS(1793),
    [anon_sym_SEMI] = ACTIONS(1791),
    [anon_sym_macro_rules_BANG] = ACTIONS(1791),
    [anon_sym_LPAREN] = ACTIONS(1791),
    [anon_sym_LBRACK] = ACTIONS(1791),
    [anon_sym_LBRACE] = ACTIONS(1791),
    [anon_sym_RBRACE] = ACTIONS(1791),
    [anon_sym_STAR] = ACTIONS(1791),
    [anon_sym_u8] = ACTIONS(1793),
    [anon_sym_i8] = ACTIONS(1793),
    [anon_sym_u16] = ACTIONS(1793),
    [anon_sym_i16] = ACTIONS(1793),
    [anon_sym_u32] = ACTIONS(1793),
    [anon_sym_i32] = ACTIONS(1793),
    [anon_sym_u64] = ACTIONS(1793),
    [anon_sym_i64] = ACTIONS(1793),
    [anon_sym_u128] = ACTIONS(1793),
    [anon_sym_i128] = ACTIONS(1793),
    [anon_sym_isize] = ACTIONS(1793),
    [anon_sym_usize] = ACTIONS(1793),
    [anon_sym_f32] = ACTIONS(1793),
    [anon_sym_f64] = ACTIONS(1793),
    [anon_sym_bool] = ACTIONS(1793),
    [anon_sym_str] = ACTIONS(1793),
    [anon_sym_char] = ACTIONS(1793),
    [anon_sym_DASH] = ACTIONS(1791),
    [anon_sym_BANG] = ACTIONS(1791),
    [anon_sym_AMP] = ACTIONS(1791),
    [anon_sym_PIPE] = ACTIONS(1791),
    [anon_sym_LT] = ACTIONS(1791),
    [anon_sym_DOT_DOT] = ACTIONS(1791),
    [anon_sym_COLON_COLON] = ACTIONS(1791),
    [anon_sym_POUND] = ACTIONS(1791),
    [anon_sym_SQUOTE] = ACTIONS(1793),
    [anon_sym_async] = ACTIONS(1793),
    [anon_sym_break] = ACTIONS(1793),
    [anon_sym_const] = ACTIONS(1793),
    [anon_sym_continue] = ACTIONS(1793),
    [anon_sym_default] = ACTIONS(1793),
    [anon_sym_enum] = ACTIONS(1793),
    [anon_sym_fn] = ACTIONS(1793),
    [anon_sym_for] = ACTIONS(1793),
    [anon_sym_gen] = ACTIONS(1793),
    [anon_sym_if] = ACTIONS(1793),
    [anon_sym_impl] = ACTIONS(1793),
    [anon_sym_let] = ACTIONS(1793),
    [anon_sym_loop] = ACTIONS(1793),
    [anon_sym_match] = ACTIONS(1793),
    [anon_sym_mod] = ACTIONS(1793),
    [anon_sym_pub] = ACTIONS(1793),
    [anon_sym_return] = ACTIONS(1793),
    [anon_sym_static] = ACTIONS(1793),
    [anon_sym_struct] = ACTIONS(1793),
    [anon_sym_trait] = ACTIONS(1793),
    [anon_sym_type] = ACTIONS(1793),
    [anon_sym_union] = ACTIONS(1793),
    [anon_sym_unsafe] = ACTIONS(1793),
    [anon_sym_use] = ACTIONS(1793),
    [anon_sym_while] = ACTIONS(1793),
    [anon_sym_extern] = ACTIONS(1793),
    [anon_sym_yield] = ACTIONS(1793),
    [anon_sym_move] = ACTIONS(1793),
    [anon_sym_try] = ACTIONS(1793),
    [sym_integer_literal] = ACTIONS(1791),
    [aux_sym_string_literal_token1] = ACTIONS(1791),
    [sym_char_literal] = ACTIONS(1791),
    [anon_sym_true] = ACTIONS(1793),
    [anon_sym_false] = ACTIONS(1793),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1793),
    [sym_super] = ACTIONS(1793),
    [sym_crate] = ACTIONS(1793),
    [sym_metavariable] = ACTIONS(1791),
    [sym__raw_string_literal_start] = ACTIONS(1791),
    [sym_float_literal] = ACTIONS(1791),
  },
  [511] = {
    [sym_line_comment] = STATE(511),
    [sym_block_comment] = STATE(511),
    [ts_builtin_sym_end] = ACTIONS(1795),
    [sym_identifier] = ACTIONS(1797),
    [anon_sym_SEMI] = ACTIONS(1795),
    [anon_sym_macro_rules_BANG] = ACTIONS(1795),
    [anon_sym_LPAREN] = ACTIONS(1795),
    [anon_sym_LBRACK] = ACTIONS(1795),
    [anon_sym_LBRACE] = ACTIONS(1795),
    [anon_sym_RBRACE] = ACTIONS(1795),
    [anon_sym_STAR] = ACTIONS(1795),
    [anon_sym_u8] = ACTIONS(1797),
    [anon_sym_i8] = ACTIONS(1797),
    [anon_sym_u16] = ACTIONS(1797),
    [anon_sym_i16] = ACTIONS(1797),
    [anon_sym_u32] = ACTIONS(1797),
    [anon_sym_i32] = ACTIONS(1797),
    [anon_sym_u64] = ACTIONS(1797),
    [anon_sym_i64] = ACTIONS(1797),
    [anon_sym_u128] = ACTIONS(1797),
    [anon_sym_i128] = ACTIONS(1797),
    [anon_sym_isize] = ACTIONS(1797),
    [anon_sym_usize] = ACTIONS(1797),
    [anon_sym_f32] = ACTIONS(1797),
    [anon_sym_f64] = ACTIONS(1797),
    [anon_sym_bool] = ACTIONS(1797),
    [anon_sym_str] = ACTIONS(1797),
    [anon_sym_char] = ACTIONS(1797),
    [anon_sym_DASH] = ACTIONS(1795),
    [anon_sym_BANG] = ACTIONS(1795),
    [anon_sym_AMP] = ACTIONS(1795),
    [anon_sym_PIPE] = ACTIONS(1795),
    [anon_sym_LT] = ACTIONS(1795),
    [anon_sym_DOT_DOT] = ACTIONS(1795),
    [anon_sym_COLON_COLON] = ACTIONS(1795),
    [anon_sym_POUND] = ACTIONS(1795),
    [anon_sym_SQUOTE] = ACTIONS(1797),
    [anon_sym_async] = ACTIONS(1797),
    [anon_sym_break] = ACTIONS(1797),
    [anon_sym_const] = ACTIONS(1797),
    [anon_sym_continue] = ACTIONS(1797),
    [anon_sym_default] = ACTIONS(1797),
    [anon_sym_enum] = ACTIONS(1797),
    [anon_sym_fn] = ACTIONS(1797),
    [anon_sym_for] = ACTIONS(1797),
    [anon_sym_gen] = ACTIONS(1797),
    [anon_sym_if] = ACTIONS(1797),
    [anon_sym_impl] = ACTIONS(1797),
    [anon_sym_let] = ACTIONS(1797),
    [anon_sym_loop] = ACTIONS(1797),
    [anon_sym_match] = ACTIONS(1797),
    [anon_sym_mod] = ACTIONS(1797),
    [anon_sym_pub] = ACTIONS(1797),
    [anon_sym_return] = ACTIONS(1797),
    [anon_sym_static] = ACTIONS(1797),
    [anon_sym_struct] = ACTIONS(1797),
    [anon_sym_trait] = ACTIONS(1797),
    [anon_sym_type] = ACTIONS(1797),
    [anon_sym_union] = ACTIONS(1797),
    [anon_sym_unsafe] = ACTIONS(1797),
    [anon_sym_use] = ACTIONS(1797),
    [anon_sym_while] = ACTIONS(1797),
    [anon_sym_extern] = ACTIONS(1797),
    [anon_sym_yield] = ACTIONS(1797),
    [anon_sym_move] = ACTIONS(1797),
    [anon_sym_try] = ACTIONS(1797),
    [sym_integer_literal] = ACTIONS(1795),
    [aux_sym_string_literal_token1] = ACTIONS(1795),
    [sym_char_literal] = ACTIONS(1795),
    [anon_sym_true] = ACTIONS(1797),
    [anon_sym_false] = ACTIONS(1797),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1797),
    [sym_super] = ACTIONS(1797),
    [sym_crate] = ACTIONS(1797),
    [sym_metavariable] = ACTIONS(1795),
    [sym__raw_string_literal_start] = ACTIONS(1795),
    [sym_float_literal] = ACTIONS(1795),
  },
  [512] = {
    [sym_line_comment] = STATE(512),
    [sym_block_comment] = STATE(512),
    [ts_builtin_sym_end] = ACTIONS(1799),
    [sym_identifier] = ACTIONS(1801),
    [anon_sym_SEMI] = ACTIONS(1799),
    [anon_sym_macro_rules_BANG] = ACTIONS(1799),
    [anon_sym_LPAREN] = ACTIONS(1799),
    [anon_sym_LBRACK] = ACTIONS(1799),
    [anon_sym_LBRACE] = ACTIONS(1799),
    [anon_sym_RBRACE] = ACTIONS(1799),
    [anon_sym_STAR] = ACTIONS(1799),
    [anon_sym_u8] = ACTIONS(1801),
    [anon_sym_i8] = ACTIONS(1801),
    [anon_sym_u16] = ACTIONS(1801),
    [anon_sym_i16] = ACTIONS(1801),
    [anon_sym_u32] = ACTIONS(1801),
    [anon_sym_i32] = ACTIONS(1801),
    [anon_sym_u64] = ACTIONS(1801),
    [anon_sym_i64] = ACTIONS(1801),
    [anon_sym_u128] = ACTIONS(1801),
    [anon_sym_i128] = ACTIONS(1801),
    [anon_sym_isize] = ACTIONS(1801),
    [anon_sym_usize] = ACTIONS(1801),
    [anon_sym_f32] = ACTIONS(1801),
    [anon_sym_f64] = ACTIONS(1801),
    [anon_sym_bool] = ACTIONS(1801),
    [anon_sym_str] = ACTIONS(1801),
    [anon_sym_char] = ACTIONS(1801),
    [anon_sym_DASH] = ACTIONS(1799),
    [anon_sym_BANG] = ACTIONS(1799),
    [anon_sym_AMP] = ACTIONS(1799),
    [anon_sym_PIPE] = ACTIONS(1799),
    [anon_sym_LT] = ACTIONS(1799),
    [anon_sym_DOT_DOT] = ACTIONS(1799),
    [anon_sym_COLON_COLON] = ACTIONS(1799),
    [anon_sym_POUND] = ACTIONS(1799),
    [anon_sym_SQUOTE] = ACTIONS(1801),
    [anon_sym_async] = ACTIONS(1801),
    [anon_sym_break] = ACTIONS(1801),
    [anon_sym_const] = ACTIONS(1801),
    [anon_sym_continue] = ACTIONS(1801),
    [anon_sym_default] = ACTIONS(1801),
    [anon_sym_enum] = ACTIONS(1801),
    [anon_sym_fn] = ACTIONS(1801),
    [anon_sym_for] = ACTIONS(1801),
    [anon_sym_gen] = ACTIONS(1801),
    [anon_sym_if] = ACTIONS(1801),
    [anon_sym_impl] = ACTIONS(1801),
    [anon_sym_let] = ACTIONS(1801),
    [anon_sym_loop] = ACTIONS(1801),
    [anon_sym_match] = ACTIONS(1801),
    [anon_sym_mod] = ACTIONS(1801),
    [anon_sym_pub] = ACTIONS(1801),
    [anon_sym_return] = ACTIONS(1801),
    [anon_sym_static] = ACTIONS(1801),
    [anon_sym_struct] = ACTIONS(1801),
    [anon_sym_trait] = ACTIONS(1801),
    [anon_sym_type] = ACTIONS(1801),
    [anon_sym_union] = ACTIONS(1801),
    [anon_sym_unsafe] = ACTIONS(1801),
    [anon_sym_use] = ACTIONS(1801),
    [anon_sym_while] = ACTIONS(1801),
    [anon_sym_extern] = ACTIONS(1801),
    [anon_sym_yield] = ACTIONS(1801),
    [anon_sym_move] = ACTIONS(1801),
    [anon_sym_try] = ACTIONS(1801),
    [sym_integer_literal] = ACTIONS(1799),
    [aux_sym_string_literal_token1] = ACTIONS(1799),
    [sym_char_literal] = ACTIONS(1799),
    [anon_sym_true] = ACTIONS(1801),
    [anon_sym_false] = ACTIONS(1801),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1801),
    [sym_super] = ACTIONS(1801),
    [sym_crate] = ACTIONS(1801),
    [sym_metavariable] = ACTIONS(1799),
    [sym__raw_string_literal_start] = ACTIONS(1799),
    [sym_float_literal] = ACTIONS(1799),
  },
  [513] = {
    [sym_line_comment] = STATE(513),
    [sym_block_comment] = STATE(513),
    [ts_builtin_sym_end] = ACTIONS(1803),
    [sym_identifier] = ACTIONS(1805),
    [anon_sym_SEMI] = ACTIONS(1803),
    [anon_sym_macro_rules_BANG] = ACTIONS(1803),
    [anon_sym_LPAREN] = ACTIONS(1803),
    [anon_sym_LBRACK] = ACTIONS(1803),
    [anon_sym_LBRACE] = ACTIONS(1803),
    [anon_sym_RBRACE] = ACTIONS(1803),
    [anon_sym_STAR] = ACTIONS(1803),
    [anon_sym_u8] = ACTIONS(1805),
    [anon_sym_i8] = ACTIONS(1805),
    [anon_sym_u16] = ACTIONS(1805),
    [anon_sym_i16] = ACTIONS(1805),
    [anon_sym_u32] = ACTIONS(1805),
    [anon_sym_i32] = ACTIONS(1805),
    [anon_sym_u64] = ACTIONS(1805),
    [anon_sym_i64] = ACTIONS(1805),
    [anon_sym_u128] = ACTIONS(1805),
    [anon_sym_i128] = ACTIONS(1805),
    [anon_sym_isize] = ACTIONS(1805),
    [anon_sym_usize] = ACTIONS(1805),
    [anon_sym_f32] = ACTIONS(1805),
    [anon_sym_f64] = ACTIONS(1805),
    [anon_sym_bool] = ACTIONS(1805),
    [anon_sym_str] = ACTIONS(1805),
    [anon_sym_char] = ACTIONS(1805),
    [anon_sym_DASH] = ACTIONS(1803),
    [anon_sym_BANG] = ACTIONS(1803),
    [anon_sym_AMP] = ACTIONS(1803),
    [anon_sym_PIPE] = ACTIONS(1803),
    [anon_sym_LT] = ACTIONS(1803),
    [anon_sym_DOT_DOT] = ACTIONS(1803),
    [anon_sym_COLON_COLON] = ACTIONS(1803),
    [anon_sym_POUND] = ACTIONS(1803),
    [anon_sym_SQUOTE] = ACTIONS(1805),
    [anon_sym_async] = ACTIONS(1805),
    [anon_sym_break] = ACTIONS(1805),
    [anon_sym_const] = ACTIONS(1805),
    [anon_sym_continue] = ACTIONS(1805),
    [anon_sym_default] = ACTIONS(1805),
    [anon_sym_enum] = ACTIONS(1805),
    [anon_sym_fn] = ACTIONS(1805),
    [anon_sym_for] = ACTIONS(1805),
    [anon_sym_gen] = ACTIONS(1805),
    [anon_sym_if] = ACTIONS(1805),
    [anon_sym_impl] = ACTIONS(1805),
    [anon_sym_let] = ACTIONS(1805),
    [anon_sym_loop] = ACTIONS(1805),
    [anon_sym_match] = ACTIONS(1805),
    [anon_sym_mod] = ACTIONS(1805),
    [anon_sym_pub] = ACTIONS(1805),
    [anon_sym_return] = ACTIONS(1805),
    [anon_sym_static] = ACTIONS(1805),
    [anon_sym_struct] = ACTIONS(1805),
    [anon_sym_trait] = ACTIONS(1805),
    [anon_sym_type] = ACTIONS(1805),
    [anon_sym_union] = ACTIONS(1805),
    [anon_sym_unsafe] = ACTIONS(1805),
    [anon_sym_use] = ACTIONS(1805),
    [anon_sym_while] = ACTIONS(1805),
    [anon_sym_extern] = ACTIONS(1805),
    [anon_sym_yield] = ACTIONS(1805),
    [anon_sym_move] = ACTIONS(1805),
    [anon_sym_try] = ACTIONS(1805),
    [sym_integer_literal] = ACTIONS(1803),
    [aux_sym_string_literal_token1] = ACTIONS(1803),
    [sym_char_literal] = ACTIONS(1803),
    [anon_sym_true] = ACTIONS(1805),
    [anon_sym_false] = ACTIONS(1805),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1805),
    [sym_super] = ACTIONS(1805),
    [sym_crate] = ACTIONS(1805),
    [sym_metavariable] = ACTIONS(1803),
    [sym__raw_string_literal_start] = ACTIONS(1803),
    [sym_float_literal] = ACTIONS(1803),
  },
  [514] = {
    [sym_line_comment] = STATE(514),
    [sym_block_comment] = STATE(514),
    [ts_builtin_sym_end] = ACTIONS(1807),
    [sym_identifier] = ACTIONS(1809),
    [anon_sym_SEMI] = ACTIONS(1807),
    [anon_sym_macro_rules_BANG] = ACTIONS(1807),
    [anon_sym_LPAREN] = ACTIONS(1807),
    [anon_sym_LBRACK] = ACTIONS(1807),
    [anon_sym_LBRACE] = ACTIONS(1807),
    [anon_sym_RBRACE] = ACTIONS(1807),
    [anon_sym_STAR] = ACTIONS(1807),
    [anon_sym_u8] = ACTIONS(1809),
    [anon_sym_i8] = ACTIONS(1809),
    [anon_sym_u16] = ACTIONS(1809),
    [anon_sym_i16] = ACTIONS(1809),
    [anon_sym_u32] = ACTIONS(1809),
    [anon_sym_i32] = ACTIONS(1809),
    [anon_sym_u64] = ACTIONS(1809),
    [anon_sym_i64] = ACTIONS(1809),
    [anon_sym_u128] = ACTIONS(1809),
    [anon_sym_i128] = ACTIONS(1809),
    [anon_sym_isize] = ACTIONS(1809),
    [anon_sym_usize] = ACTIONS(1809),
    [anon_sym_f32] = ACTIONS(1809),
    [anon_sym_f64] = ACTIONS(1809),
    [anon_sym_bool] = ACTIONS(1809),
    [anon_sym_str] = ACTIONS(1809),
    [anon_sym_char] = ACTIONS(1809),
    [anon_sym_DASH] = ACTIONS(1807),
    [anon_sym_BANG] = ACTIONS(1807),
    [anon_sym_AMP] = ACTIONS(1807),
    [anon_sym_PIPE] = ACTIONS(1807),
    [anon_sym_LT] = ACTIONS(1807),
    [anon_sym_DOT_DOT] = ACTIONS(1807),
    [anon_sym_COLON_COLON] = ACTIONS(1807),
    [anon_sym_POUND] = ACTIONS(1807),
    [anon_sym_SQUOTE] = ACTIONS(1809),
    [anon_sym_async] = ACTIONS(1809),
    [anon_sym_break] = ACTIONS(1809),
    [anon_sym_const] = ACTIONS(1809),
    [anon_sym_continue] = ACTIONS(1809),
    [anon_sym_default] = ACTIONS(1809),
    [anon_sym_enum] = ACTIONS(1809),
    [anon_sym_fn] = ACTIONS(1809),
    [anon_sym_for] = ACTIONS(1809),
    [anon_sym_gen] = ACTIONS(1809),
    [anon_sym_if] = ACTIONS(1809),
    [anon_sym_impl] = ACTIONS(1809),
    [anon_sym_let] = ACTIONS(1809),
    [anon_sym_loop] = ACTIONS(1809),
    [anon_sym_match] = ACTIONS(1809),
    [anon_sym_mod] = ACTIONS(1809),
    [anon_sym_pub] = ACTIONS(1809),
    [anon_sym_return] = ACTIONS(1809),
    [anon_sym_static] = ACTIONS(1809),
    [anon_sym_struct] = ACTIONS(1809),
    [anon_sym_trait] = ACTIONS(1809),
    [anon_sym_type] = ACTIONS(1809),
    [anon_sym_union] = ACTIONS(1809),
    [anon_sym_unsafe] = ACTIONS(1809),
    [anon_sym_use] = ACTIONS(1809),
    [anon_sym_while] = ACTIONS(1809),
    [anon_sym_extern] = ACTIONS(1809),
    [anon_sym_yield] = ACTIONS(1809),
    [anon_sym_move] = ACTIONS(1809),
    [anon_sym_try] = ACTIONS(1809),
    [sym_integer_literal] = ACTIONS(1807),
    [aux_sym_string_literal_token1] = ACTIONS(1807),
    [sym_char_literal] = ACTIONS(1807),
    [anon_sym_true] = ACTIONS(1809),
    [anon_sym_false] = ACTIONS(1809),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1809),
    [sym_super] = ACTIONS(1809),
    [sym_crate] = ACTIONS(1809),
    [sym_metavariable] = ACTIONS(1807),
    [sym__raw_string_literal_start] = ACTIONS(1807),
    [sym_float_literal] = ACTIONS(1807),
  },
  [515] = {
    [sym_line_comment] = STATE(515),
    [sym_block_comment] = STATE(515),
    [ts_builtin_sym_end] = ACTIONS(1811),
    [sym_identifier] = ACTIONS(1813),
    [anon_sym_SEMI] = ACTIONS(1811),
    [anon_sym_macro_rules_BANG] = ACTIONS(1811),
    [anon_sym_LPAREN] = ACTIONS(1811),
    [anon_sym_LBRACK] = ACTIONS(1811),
    [anon_sym_LBRACE] = ACTIONS(1811),
    [anon_sym_RBRACE] = ACTIONS(1811),
    [anon_sym_STAR] = ACTIONS(1811),
    [anon_sym_u8] = ACTIONS(1813),
    [anon_sym_i8] = ACTIONS(1813),
    [anon_sym_u16] = ACTIONS(1813),
    [anon_sym_i16] = ACTIONS(1813),
    [anon_sym_u32] = ACTIONS(1813),
    [anon_sym_i32] = ACTIONS(1813),
    [anon_sym_u64] = ACTIONS(1813),
    [anon_sym_i64] = ACTIONS(1813),
    [anon_sym_u128] = ACTIONS(1813),
    [anon_sym_i128] = ACTIONS(1813),
    [anon_sym_isize] = ACTIONS(1813),
    [anon_sym_usize] = ACTIONS(1813),
    [anon_sym_f32] = ACTIONS(1813),
    [anon_sym_f64] = ACTIONS(1813),
    [anon_sym_bool] = ACTIONS(1813),
    [anon_sym_str] = ACTIONS(1813),
    [anon_sym_char] = ACTIONS(1813),
    [anon_sym_DASH] = ACTIONS(1811),
    [anon_sym_BANG] = ACTIONS(1811),
    [anon_sym_AMP] = ACTIONS(1811),
    [anon_sym_PIPE] = ACTIONS(1811),
    [anon_sym_LT] = ACTIONS(1811),
    [anon_sym_DOT_DOT] = ACTIONS(1811),
    [anon_sym_COLON_COLON] = ACTIONS(1811),
    [anon_sym_POUND] = ACTIONS(1811),
    [anon_sym_SQUOTE] = ACTIONS(1813),
    [anon_sym_async] = ACTIONS(1813),
    [anon_sym_break] = ACTIONS(1813),
    [anon_sym_const] = ACTIONS(1813),
    [anon_sym_continue] = ACTIONS(1813),
    [anon_sym_default] = ACTIONS(1813),
    [anon_sym_enum] = ACTIONS(1813),
    [anon_sym_fn] = ACTIONS(1813),
    [anon_sym_for] = ACTIONS(1813),
    [anon_sym_gen] = ACTIONS(1813),
    [anon_sym_if] = ACTIONS(1813),
    [anon_sym_impl] = ACTIONS(1813),
    [anon_sym_let] = ACTIONS(1813),
    [anon_sym_loop] = ACTIONS(1813),
    [anon_sym_match] = ACTIONS(1813),
    [anon_sym_mod] = ACTIONS(1813),
    [anon_sym_pub] = ACTIONS(1813),
    [anon_sym_return] = ACTIONS(1813),
    [anon_sym_static] = ACTIONS(1813),
    [anon_sym_struct] = ACTIONS(1813),
    [anon_sym_trait] = ACTIONS(1813),
    [anon_sym_type] = ACTIONS(1813),
    [anon_sym_union] = ACTIONS(1813),
    [anon_sym_unsafe] = ACTIONS(1813),
    [anon_sym_use] = ACTIONS(1813),
    [anon_sym_while] = ACTIONS(1813),
    [anon_sym_extern] = ACTIONS(1813),
    [anon_sym_yield] = ACTIONS(1813),
    [anon_sym_move] = ACTIONS(1813),
    [anon_sym_try] = ACTIONS(1813),
    [sym_integer_literal] = ACTIONS(1811),
    [aux_sym_string_literal_token1] = ACTIONS(1811),
    [sym_char_literal] = ACTIONS(1811),
    [anon_sym_true] = ACTIONS(1813),
    [anon_sym_false] = ACTIONS(1813),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1813),
    [sym_super] = ACTIONS(1813),
    [sym_crate] = ACTIONS(1813),
    [sym_metavariable] = ACTIONS(1811),
    [sym__raw_string_literal_start] = ACTIONS(1811),
    [sym_float_literal] = ACTIONS(1811),
  },
  [516] = {
    [sym_line_comment] = STATE(516),
    [sym_block_comment] = STATE(516),
    [ts_builtin_sym_end] = ACTIONS(1815),
    [sym_identifier] = ACTIONS(1817),
    [anon_sym_SEMI] = ACTIONS(1815),
    [anon_sym_macro_rules_BANG] = ACTIONS(1815),
    [anon_sym_LPAREN] = ACTIONS(1815),
    [anon_sym_LBRACK] = ACTIONS(1815),
    [anon_sym_LBRACE] = ACTIONS(1815),
    [anon_sym_RBRACE] = ACTIONS(1815),
    [anon_sym_STAR] = ACTIONS(1815),
    [anon_sym_u8] = ACTIONS(1817),
    [anon_sym_i8] = ACTIONS(1817),
    [anon_sym_u16] = ACTIONS(1817),
    [anon_sym_i16] = ACTIONS(1817),
    [anon_sym_u32] = ACTIONS(1817),
    [anon_sym_i32] = ACTIONS(1817),
    [anon_sym_u64] = ACTIONS(1817),
    [anon_sym_i64] = ACTIONS(1817),
    [anon_sym_u128] = ACTIONS(1817),
    [anon_sym_i128] = ACTIONS(1817),
    [anon_sym_isize] = ACTIONS(1817),
    [anon_sym_usize] = ACTIONS(1817),
    [anon_sym_f32] = ACTIONS(1817),
    [anon_sym_f64] = ACTIONS(1817),
    [anon_sym_bool] = ACTIONS(1817),
    [anon_sym_str] = ACTIONS(1817),
    [anon_sym_char] = ACTIONS(1817),
    [anon_sym_DASH] = ACTIONS(1815),
    [anon_sym_BANG] = ACTIONS(1815),
    [anon_sym_AMP] = ACTIONS(1815),
    [anon_sym_PIPE] = ACTIONS(1815),
    [anon_sym_LT] = ACTIONS(1815),
    [anon_sym_DOT_DOT] = ACTIONS(1815),
    [anon_sym_COLON_COLON] = ACTIONS(1815),
    [anon_sym_POUND] = ACTIONS(1815),
    [anon_sym_SQUOTE] = ACTIONS(1817),
    [anon_sym_async] = ACTIONS(1817),
    [anon_sym_break] = ACTIONS(1817),
    [anon_sym_const] = ACTIONS(1817),
    [anon_sym_continue] = ACTIONS(1817),
    [anon_sym_default] = ACTIONS(1817),
    [anon_sym_enum] = ACTIONS(1817),
    [anon_sym_fn] = ACTIONS(1817),
    [anon_sym_for] = ACTIONS(1817),
    [anon_sym_gen] = ACTIONS(1817),
    [anon_sym_if] = ACTIONS(1817),
    [anon_sym_impl] = ACTIONS(1817),
    [anon_sym_let] = ACTIONS(1817),
    [anon_sym_loop] = ACTIONS(1817),
    [anon_sym_match] = ACTIONS(1817),
    [anon_sym_mod] = ACTIONS(1817),
    [anon_sym_pub] = ACTIONS(1817),
    [anon_sym_return] = ACTIONS(1817),
    [anon_sym_static] = ACTIONS(1817),
    [anon_sym_struct] = ACTIONS(1817),
    [anon_sym_trait] = ACTIONS(1817),
    [anon_sym_type] = ACTIONS(1817),
    [anon_sym_union] = ACTIONS(1817),
    [anon_sym_unsafe] = ACTIONS(1817),
    [anon_sym_use] = ACTIONS(1817),
    [anon_sym_while] = ACTIONS(1817),
    [anon_sym_extern] = ACTIONS(1817),
    [anon_sym_yield] = ACTIONS(1817),
    [anon_sym_move] = ACTIONS(1817),
    [anon_sym_try] = ACTIONS(1817),
    [sym_integer_literal] = ACTIONS(1815),
    [aux_sym_string_literal_token1] = ACTIONS(1815),
    [sym_char_literal] = ACTIONS(1815),
    [anon_sym_true] = ACTIONS(1817),
    [anon_sym_false] = ACTIONS(1817),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1817),
    [sym_super] = ACTIONS(1817),
    [sym_crate] = ACTIONS(1817),
    [sym_metavariable] = ACTIONS(1815),
    [sym__raw_string_literal_start] = ACTIONS(1815),
    [sym_float_literal] = ACTIONS(1815),
  },
  [517] = {
    [sym_line_comment] = STATE(517),
    [sym_block_comment] = STATE(517),
    [ts_builtin_sym_end] = ACTIONS(1819),
    [sym_identifier] = ACTIONS(1821),
    [anon_sym_SEMI] = ACTIONS(1819),
    [anon_sym_macro_rules_BANG] = ACTIONS(1819),
    [anon_sym_LPAREN] = ACTIONS(1819),
    [anon_sym_LBRACK] = ACTIONS(1819),
    [anon_sym_LBRACE] = ACTIONS(1819),
    [anon_sym_RBRACE] = ACTIONS(1819),
    [anon_sym_STAR] = ACTIONS(1819),
    [anon_sym_u8] = ACTIONS(1821),
    [anon_sym_i8] = ACTIONS(1821),
    [anon_sym_u16] = ACTIONS(1821),
    [anon_sym_i16] = ACTIONS(1821),
    [anon_sym_u32] = ACTIONS(1821),
    [anon_sym_i32] = ACTIONS(1821),
    [anon_sym_u64] = ACTIONS(1821),
    [anon_sym_i64] = ACTIONS(1821),
    [anon_sym_u128] = ACTIONS(1821),
    [anon_sym_i128] = ACTIONS(1821),
    [anon_sym_isize] = ACTIONS(1821),
    [anon_sym_usize] = ACTIONS(1821),
    [anon_sym_f32] = ACTIONS(1821),
    [anon_sym_f64] = ACTIONS(1821),
    [anon_sym_bool] = ACTIONS(1821),
    [anon_sym_str] = ACTIONS(1821),
    [anon_sym_char] = ACTIONS(1821),
    [anon_sym_DASH] = ACTIONS(1819),
    [anon_sym_BANG] = ACTIONS(1819),
    [anon_sym_AMP] = ACTIONS(1819),
    [anon_sym_PIPE] = ACTIONS(1819),
    [anon_sym_LT] = ACTIONS(1819),
    [anon_sym_DOT_DOT] = ACTIONS(1819),
    [anon_sym_COLON_COLON] = ACTIONS(1819),
    [anon_sym_POUND] = ACTIONS(1819),
    [anon_sym_SQUOTE] = ACTIONS(1821),
    [anon_sym_async] = ACTIONS(1821),
    [anon_sym_break] = ACTIONS(1821),
    [anon_sym_const] = ACTIONS(1821),
    [anon_sym_continue] = ACTIONS(1821),
    [anon_sym_default] = ACTIONS(1821),
    [anon_sym_enum] = ACTIONS(1821),
    [anon_sym_fn] = ACTIONS(1821),
    [anon_sym_for] = ACTIONS(1821),
    [anon_sym_gen] = ACTIONS(1821),
    [anon_sym_if] = ACTIONS(1821),
    [anon_sym_impl] = ACTIONS(1821),
    [anon_sym_let] = ACTIONS(1821),
    [anon_sym_loop] = ACTIONS(1821),
    [anon_sym_match] = ACTIONS(1821),
    [anon_sym_mod] = ACTIONS(1821),
    [anon_sym_pub] = ACTIONS(1821),
    [anon_sym_return] = ACTIONS(1821),
    [anon_sym_static] = ACTIONS(1821),
    [anon_sym_struct] = ACTIONS(1821),
    [anon_sym_trait] = ACTIONS(1821),
    [anon_sym_type] = ACTIONS(1821),
    [anon_sym_union] = ACTIONS(1821),
    [anon_sym_unsafe] = ACTIONS(1821),
    [anon_sym_use] = ACTIONS(1821),
    [anon_sym_while] = ACTIONS(1821),
    [anon_sym_extern] = ACTIONS(1821),
    [anon_sym_yield] = ACTIONS(1821),
    [anon_sym_move] = ACTIONS(1821),
    [anon_sym_try] = ACTIONS(1821),
    [sym_integer_literal] = ACTIONS(1819),
    [aux_sym_string_literal_token1] = ACTIONS(1819),
    [sym_char_literal] = ACTIONS(1819),
    [anon_sym_true] = ACTIONS(1821),
    [anon_sym_false] = ACTIONS(1821),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1821),
    [sym_super] = ACTIONS(1821),
    [sym_crate] = ACTIONS(1821),
    [sym_metavariable] = ACTIONS(1819),
    [sym__raw_string_literal_start] = ACTIONS(1819),
    [sym_float_literal] = ACTIONS(1819),
  },
  [518] = {
    [sym_line_comment] = STATE(518),
    [sym_block_comment] = STATE(518),
    [ts_builtin_sym_end] = ACTIONS(1823),
    [sym_identifier] = ACTIONS(1825),
    [anon_sym_SEMI] = ACTIONS(1823),
    [anon_sym_macro_rules_BANG] = ACTIONS(1823),
    [anon_sym_LPAREN] = ACTIONS(1823),
    [anon_sym_LBRACK] = ACTIONS(1823),
    [anon_sym_LBRACE] = ACTIONS(1823),
    [anon_sym_RBRACE] = ACTIONS(1823),
    [anon_sym_STAR] = ACTIONS(1823),
    [anon_sym_u8] = ACTIONS(1825),
    [anon_sym_i8] = ACTIONS(1825),
    [anon_sym_u16] = ACTIONS(1825),
    [anon_sym_i16] = ACTIONS(1825),
    [anon_sym_u32] = ACTIONS(1825),
    [anon_sym_i32] = ACTIONS(1825),
    [anon_sym_u64] = ACTIONS(1825),
    [anon_sym_i64] = ACTIONS(1825),
    [anon_sym_u128] = ACTIONS(1825),
    [anon_sym_i128] = ACTIONS(1825),
    [anon_sym_isize] = ACTIONS(1825),
    [anon_sym_usize] = ACTIONS(1825),
    [anon_sym_f32] = ACTIONS(1825),
    [anon_sym_f64] = ACTIONS(1825),
    [anon_sym_bool] = ACTIONS(1825),
    [anon_sym_str] = ACTIONS(1825),
    [anon_sym_char] = ACTIONS(1825),
    [anon_sym_DASH] = ACTIONS(1823),
    [anon_sym_BANG] = ACTIONS(1823),
    [anon_sym_AMP] = ACTIONS(1823),
    [anon_sym_PIPE] = ACTIONS(1823),
    [anon_sym_LT] = ACTIONS(1823),
    [anon_sym_DOT_DOT] = ACTIONS(1823),
    [anon_sym_COLON_COLON] = ACTIONS(1823),
    [anon_sym_POUND] = ACTIONS(1823),
    [anon_sym_SQUOTE] = ACTIONS(1825),
    [anon_sym_async] = ACTIONS(1825),
    [anon_sym_break] = ACTIONS(1825),
    [anon_sym_const] = ACTIONS(1825),
    [anon_sym_continue] = ACTIONS(1825),
    [anon_sym_default] = ACTIONS(1825),
    [anon_sym_enum] = ACTIONS(1825),
    [anon_sym_fn] = ACTIONS(1825),
    [anon_sym_for] = ACTIONS(1825),
    [anon_sym_gen] = ACTIONS(1825),
    [anon_sym_if] = ACTIONS(1825),
    [anon_sym_impl] = ACTIONS(1825),
    [anon_sym_let] = ACTIONS(1825),
    [anon_sym_loop] = ACTIONS(1825),
    [anon_sym_match] = ACTIONS(1825),
    [anon_sym_mod] = ACTIONS(1825),
    [anon_sym_pub] = ACTIONS(1825),
    [anon_sym_return] = ACTIONS(1825),
    [anon_sym_static] = ACTIONS(1825),
    [anon_sym_struct] = ACTIONS(1825),
    [anon_sym_trait] = ACTIONS(1825),
    [anon_sym_type] = ACTIONS(1825),
    [anon_sym_union] = ACTIONS(1825),
    [anon_sym_unsafe] = ACTIONS(1825),
    [anon_sym_use] = ACTIONS(1825),
    [anon_sym_while] = ACTIONS(1825),
    [anon_sym_extern] = ACTIONS(1825),
    [anon_sym_yield] = ACTIONS(1825),
    [anon_sym_move] = ACTIONS(1825),
    [anon_sym_try] = ACTIONS(1825),
    [sym_integer_literal] = ACTIONS(1823),
    [aux_sym_string_literal_token1] = ACTIONS(1823),
    [sym_char_literal] = ACTIONS(1823),
    [anon_sym_true] = ACTIONS(1825),
    [anon_sym_false] = ACTIONS(1825),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1825),
    [sym_super] = ACTIONS(1825),
    [sym_crate] = ACTIONS(1825),
    [sym_metavariable] = ACTIONS(1823),
    [sym__raw_string_literal_start] = ACTIONS(1823),
    [sym_float_literal] = ACTIONS(1823),
  },
  [519] = {
    [sym_line_comment] = STATE(519),
    [sym_block_comment] = STATE(519),
    [ts_builtin_sym_end] = ACTIONS(1827),
    [sym_identifier] = ACTIONS(1829),
    [anon_sym_SEMI] = ACTIONS(1827),
    [anon_sym_macro_rules_BANG] = ACTIONS(1827),
    [anon_sym_LPAREN] = ACTIONS(1827),
    [anon_sym_LBRACK] = ACTIONS(1827),
    [anon_sym_LBRACE] = ACTIONS(1827),
    [anon_sym_RBRACE] = ACTIONS(1827),
    [anon_sym_STAR] = ACTIONS(1827),
    [anon_sym_u8] = ACTIONS(1829),
    [anon_sym_i8] = ACTIONS(1829),
    [anon_sym_u16] = ACTIONS(1829),
    [anon_sym_i16] = ACTIONS(1829),
    [anon_sym_u32] = ACTIONS(1829),
    [anon_sym_i32] = ACTIONS(1829),
    [anon_sym_u64] = ACTIONS(1829),
    [anon_sym_i64] = ACTIONS(1829),
    [anon_sym_u128] = ACTIONS(1829),
    [anon_sym_i128] = ACTIONS(1829),
    [anon_sym_isize] = ACTIONS(1829),
    [anon_sym_usize] = ACTIONS(1829),
    [anon_sym_f32] = ACTIONS(1829),
    [anon_sym_f64] = ACTIONS(1829),
    [anon_sym_bool] = ACTIONS(1829),
    [anon_sym_str] = ACTIONS(1829),
    [anon_sym_char] = ACTIONS(1829),
    [anon_sym_DASH] = ACTIONS(1827),
    [anon_sym_BANG] = ACTIONS(1827),
    [anon_sym_AMP] = ACTIONS(1827),
    [anon_sym_PIPE] = ACTIONS(1827),
    [anon_sym_LT] = ACTIONS(1827),
    [anon_sym_DOT_DOT] = ACTIONS(1827),
    [anon_sym_COLON_COLON] = ACTIONS(1827),
    [anon_sym_POUND] = ACTIONS(1827),
    [anon_sym_SQUOTE] = ACTIONS(1829),
    [anon_sym_async] = ACTIONS(1829),
    [anon_sym_break] = ACTIONS(1829),
    [anon_sym_const] = ACTIONS(1829),
    [anon_sym_continue] = ACTIONS(1829),
    [anon_sym_default] = ACTIONS(1829),
    [anon_sym_enum] = ACTIONS(1829),
    [anon_sym_fn] = ACTIONS(1829),
    [anon_sym_for] = ACTIONS(1829),
    [anon_sym_gen] = ACTIONS(1829),
    [anon_sym_if] = ACTIONS(1829),
    [anon_sym_impl] = ACTIONS(1829),
    [anon_sym_let] = ACTIONS(1829),
    [anon_sym_loop] = ACTIONS(1829),
    [anon_sym_match] = ACTIONS(1829),
    [anon_sym_mod] = ACTIONS(1829),
    [anon_sym_pub] = ACTIONS(1829),
    [anon_sym_return] = ACTIONS(1829),
    [anon_sym_static] = ACTIONS(1829),
    [anon_sym_struct] = ACTIONS(1829),
    [anon_sym_trait] = ACTIONS(1829),
    [anon_sym_type] = ACTIONS(1829),
    [anon_sym_union] = ACTIONS(1829),
    [anon_sym_unsafe] = ACTIONS(1829),
    [anon_sym_use] = ACTIONS(1829),
    [anon_sym_while] = ACTIONS(1829),
    [anon_sym_extern] = ACTIONS(1829),
    [anon_sym_yield] = ACTIONS(1829),
    [anon_sym_move] = ACTIONS(1829),
    [anon_sym_try] = ACTIONS(1829),
    [sym_integer_literal] = ACTIONS(1827),
    [aux_sym_string_literal_token1] = ACTIONS(1827),
    [sym_char_literal] = ACTIONS(1827),
    [anon_sym_true] = ACTIONS(1829),
    [anon_sym_false] = ACTIONS(1829),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1829),
    [sym_super] = ACTIONS(1829),
    [sym_crate] = ACTIONS(1829),
    [sym_metavariable] = ACTIONS(1827),
    [sym__raw_string_literal_start] = ACTIONS(1827),
    [sym_float_literal] = ACTIONS(1827),
  },
  [520] = {
    [sym_line_comment] = STATE(520),
    [sym_block_comment] = STATE(520),
    [ts_builtin_sym_end] = ACTIONS(1831),
    [sym_identifier] = ACTIONS(1833),
    [anon_sym_SEMI] = ACTIONS(1831),
    [anon_sym_macro_rules_BANG] = ACTIONS(1831),
    [anon_sym_LPAREN] = ACTIONS(1831),
    [anon_sym_LBRACK] = ACTIONS(1831),
    [anon_sym_LBRACE] = ACTIONS(1831),
    [anon_sym_RBRACE] = ACTIONS(1831),
    [anon_sym_STAR] = ACTIONS(1831),
    [anon_sym_u8] = ACTIONS(1833),
    [anon_sym_i8] = ACTIONS(1833),
    [anon_sym_u16] = ACTIONS(1833),
    [anon_sym_i16] = ACTIONS(1833),
    [anon_sym_u32] = ACTIONS(1833),
    [anon_sym_i32] = ACTIONS(1833),
    [anon_sym_u64] = ACTIONS(1833),
    [anon_sym_i64] = ACTIONS(1833),
    [anon_sym_u128] = ACTIONS(1833),
    [anon_sym_i128] = ACTIONS(1833),
    [anon_sym_isize] = ACTIONS(1833),
    [anon_sym_usize] = ACTIONS(1833),
    [anon_sym_f32] = ACTIONS(1833),
    [anon_sym_f64] = ACTIONS(1833),
    [anon_sym_bool] = ACTIONS(1833),
    [anon_sym_str] = ACTIONS(1833),
    [anon_sym_char] = ACTIONS(1833),
    [anon_sym_DASH] = ACTIONS(1831),
    [anon_sym_BANG] = ACTIONS(1831),
    [anon_sym_AMP] = ACTIONS(1831),
    [anon_sym_PIPE] = ACTIONS(1831),
    [anon_sym_LT] = ACTIONS(1831),
    [anon_sym_DOT_DOT] = ACTIONS(1831),
    [anon_sym_COLON_COLON] = ACTIONS(1831),
    [anon_sym_POUND] = ACTIONS(1831),
    [anon_sym_SQUOTE] = ACTIONS(1833),
    [anon_sym_async] = ACTIONS(1833),
    [anon_sym_break] = ACTIONS(1833),
    [anon_sym_const] = ACTIONS(1833),
    [anon_sym_continue] = ACTIONS(1833),
    [anon_sym_default] = ACTIONS(1833),
    [anon_sym_enum] = ACTIONS(1833),
    [anon_sym_fn] = ACTIONS(1833),
    [anon_sym_for] = ACTIONS(1833),
    [anon_sym_gen] = ACTIONS(1833),
    [anon_sym_if] = ACTIONS(1833),
    [anon_sym_impl] = ACTIONS(1833),
    [anon_sym_let] = ACTIONS(1833),
    [anon_sym_loop] = ACTIONS(1833),
    [anon_sym_match] = ACTIONS(1833),
    [anon_sym_mod] = ACTIONS(1833),
    [anon_sym_pub] = ACTIONS(1833),
    [anon_sym_return] = ACTIONS(1833),
    [anon_sym_static] = ACTIONS(1833),
    [anon_sym_struct] = ACTIONS(1833),
    [anon_sym_trait] = ACTIONS(1833),
    [anon_sym_type] = ACTIONS(1833),
    [anon_sym_union] = ACTIONS(1833),
    [anon_sym_unsafe] = ACTIONS(1833),
    [anon_sym_use] = ACTIONS(1833),
    [anon_sym_while] = ACTIONS(1833),
    [anon_sym_extern] = ACTIONS(1833),
    [anon_sym_yield] = ACTIONS(1833),
    [anon_sym_move] = ACTIONS(1833),
    [anon_sym_try] = ACTIONS(1833),
    [sym_integer_literal] = ACTIONS(1831),
    [aux_sym_string_literal_token1] = ACTIONS(1831),
    [sym_char_literal] = ACTIONS(1831),
    [anon_sym_true] = ACTIONS(1833),
    [anon_sym_false] = ACTIONS(1833),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1833),
    [sym_super] = ACTIONS(1833),
    [sym_crate] = ACTIONS(1833),
    [sym_metavariable] = ACTIONS(1831),
    [sym__raw_string_literal_start] = ACTIONS(1831),
    [sym_float_literal] = ACTIONS(1831),
  },
  [521] = {
    [sym_line_comment] = STATE(521),
    [sym_block_comment] = STATE(521),
    [ts_builtin_sym_end] = ACTIONS(1835),
    [sym_identifier] = ACTIONS(1837),
    [anon_sym_SEMI] = ACTIONS(1835),
    [anon_sym_macro_rules_BANG] = ACTIONS(1835),
    [anon_sym_LPAREN] = ACTIONS(1835),
    [anon_sym_LBRACK] = ACTIONS(1835),
    [anon_sym_LBRACE] = ACTIONS(1835),
    [anon_sym_RBRACE] = ACTIONS(1835),
    [anon_sym_STAR] = ACTIONS(1835),
    [anon_sym_u8] = ACTIONS(1837),
    [anon_sym_i8] = ACTIONS(1837),
    [anon_sym_u16] = ACTIONS(1837),
    [anon_sym_i16] = ACTIONS(1837),
    [anon_sym_u32] = ACTIONS(1837),
    [anon_sym_i32] = ACTIONS(1837),
    [anon_sym_u64] = ACTIONS(1837),
    [anon_sym_i64] = ACTIONS(1837),
    [anon_sym_u128] = ACTIONS(1837),
    [anon_sym_i128] = ACTIONS(1837),
    [anon_sym_isize] = ACTIONS(1837),
    [anon_sym_usize] = ACTIONS(1837),
    [anon_sym_f32] = ACTIONS(1837),
    [anon_sym_f64] = ACTIONS(1837),
    [anon_sym_bool] = ACTIONS(1837),
    [anon_sym_str] = ACTIONS(1837),
    [anon_sym_char] = ACTIONS(1837),
    [anon_sym_DASH] = ACTIONS(1835),
    [anon_sym_BANG] = ACTIONS(1835),
    [anon_sym_AMP] = ACTIONS(1835),
    [anon_sym_PIPE] = ACTIONS(1835),
    [anon_sym_LT] = ACTIONS(1835),
    [anon_sym_DOT_DOT] = ACTIONS(1835),
    [anon_sym_COLON_COLON] = ACTIONS(1835),
    [anon_sym_POUND] = ACTIONS(1835),
    [anon_sym_SQUOTE] = ACTIONS(1837),
    [anon_sym_async] = ACTIONS(1837),
    [anon_sym_break] = ACTIONS(1837),
    [anon_sym_const] = ACTIONS(1837),
    [anon_sym_continue] = ACTIONS(1837),
    [anon_sym_default] = ACTIONS(1837),
    [anon_sym_enum] = ACTIONS(1837),
    [anon_sym_fn] = ACTIONS(1837),
    [anon_sym_for] = ACTIONS(1837),
    [anon_sym_gen] = ACTIONS(1837),
    [anon_sym_if] = ACTIONS(1837),
    [anon_sym_impl] = ACTIONS(1837),
    [anon_sym_let] = ACTIONS(1837),
    [anon_sym_loop] = ACTIONS(1837),
    [anon_sym_match] = ACTIONS(1837),
    [anon_sym_mod] = ACTIONS(1837),
    [anon_sym_pub] = ACTIONS(1837),
    [anon_sym_return] = ACTIONS(1837),
    [anon_sym_static] = ACTIONS(1837),
    [anon_sym_struct] = ACTIONS(1837),
    [anon_sym_trait] = ACTIONS(1837),
    [anon_sym_type] = ACTIONS(1837),
    [anon_sym_union] = ACTIONS(1837),
    [anon_sym_unsafe] = ACTIONS(1837),
    [anon_sym_use] = ACTIONS(1837),
    [anon_sym_while] = ACTIONS(1837),
    [anon_sym_extern] = ACTIONS(1837),
    [anon_sym_yield] = ACTIONS(1837),
    [anon_sym_move] = ACTIONS(1837),
    [anon_sym_try] = ACTIONS(1837),
    [sym_integer_literal] = ACTIONS(1835),
    [aux_sym_string_literal_token1] = ACTIONS(1835),
    [sym_char_literal] = ACTIONS(1835),
    [anon_sym_true] = ACTIONS(1837),
    [anon_sym_false] = ACTIONS(1837),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1837),
    [sym_super] = ACTIONS(1837),
    [sym_crate] = ACTIONS(1837),
    [sym_metavariable] = ACTIONS(1835),
    [sym__raw_string_literal_start] = ACTIONS(1835),
    [sym_float_literal] = ACTIONS(1835),
  },
  [522] = {
    [sym_line_comment] = STATE(522),
    [sym_block_comment] = STATE(522),
    [ts_builtin_sym_end] = ACTIONS(1839),
    [sym_identifier] = ACTIONS(1841),
    [anon_sym_SEMI] = ACTIONS(1839),
    [anon_sym_macro_rules_BANG] = ACTIONS(1839),
    [anon_sym_LPAREN] = ACTIONS(1839),
    [anon_sym_LBRACK] = ACTIONS(1839),
    [anon_sym_LBRACE] = ACTIONS(1839),
    [anon_sym_RBRACE] = ACTIONS(1839),
    [anon_sym_STAR] = ACTIONS(1839),
    [anon_sym_u8] = ACTIONS(1841),
    [anon_sym_i8] = ACTIONS(1841),
    [anon_sym_u16] = ACTIONS(1841),
    [anon_sym_i16] = ACTIONS(1841),
    [anon_sym_u32] = ACTIONS(1841),
    [anon_sym_i32] = ACTIONS(1841),
    [anon_sym_u64] = ACTIONS(1841),
    [anon_sym_i64] = ACTIONS(1841),
    [anon_sym_u128] = ACTIONS(1841),
    [anon_sym_i128] = ACTIONS(1841),
    [anon_sym_isize] = ACTIONS(1841),
    [anon_sym_usize] = ACTIONS(1841),
    [anon_sym_f32] = ACTIONS(1841),
    [anon_sym_f64] = ACTIONS(1841),
    [anon_sym_bool] = ACTIONS(1841),
    [anon_sym_str] = ACTIONS(1841),
    [anon_sym_char] = ACTIONS(1841),
    [anon_sym_DASH] = ACTIONS(1839),
    [anon_sym_BANG] = ACTIONS(1839),
    [anon_sym_AMP] = ACTIONS(1839),
    [anon_sym_PIPE] = ACTIONS(1839),
    [anon_sym_LT] = ACTIONS(1839),
    [anon_sym_DOT_DOT] = ACTIONS(1839),
    [anon_sym_COLON_COLON] = ACTIONS(1839),
    [anon_sym_POUND] = ACTIONS(1839),
    [anon_sym_SQUOTE] = ACTIONS(1841),
    [anon_sym_async] = ACTIONS(1841),
    [anon_sym_break] = ACTIONS(1841),
    [anon_sym_const] = ACTIONS(1841),
    [anon_sym_continue] = ACTIONS(1841),
    [anon_sym_default] = ACTIONS(1841),
    [anon_sym_enum] = ACTIONS(1841),
    [anon_sym_fn] = ACTIONS(1841),
    [anon_sym_for] = ACTIONS(1841),
    [anon_sym_gen] = ACTIONS(1841),
    [anon_sym_if] = ACTIONS(1841),
    [anon_sym_impl] = ACTIONS(1841),
    [anon_sym_let] = ACTIONS(1841),
    [anon_sym_loop] = ACTIONS(1841),
    [anon_sym_match] = ACTIONS(1841),
    [anon_sym_mod] = ACTIONS(1841),
    [anon_sym_pub] = ACTIONS(1841),
    [anon_sym_return] = ACTIONS(1841),
    [anon_sym_static] = ACTIONS(1841),
    [anon_sym_struct] = ACTIONS(1841),
    [anon_sym_trait] = ACTIONS(1841),
    [anon_sym_type] = ACTIONS(1841),
    [anon_sym_union] = ACTIONS(1841),
    [anon_sym_unsafe] = ACTIONS(1841),
    [anon_sym_use] = ACTIONS(1841),
    [anon_sym_while] = ACTIONS(1841),
    [anon_sym_extern] = ACTIONS(1841),
    [anon_sym_yield] = ACTIONS(1841),
    [anon_sym_move] = ACTIONS(1841),
    [anon_sym_try] = ACTIONS(1841),
    [sym_integer_literal] = ACTIONS(1839),
    [aux_sym_string_literal_token1] = ACTIONS(1839),
    [sym_char_literal] = ACTIONS(1839),
    [anon_sym_true] = ACTIONS(1841),
    [anon_sym_false] = ACTIONS(1841),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1841),
    [sym_super] = ACTIONS(1841),
    [sym_crate] = ACTIONS(1841),
    [sym_metavariable] = ACTIONS(1839),
    [sym__raw_string_literal_start] = ACTIONS(1839),
    [sym_float_literal] = ACTIONS(1839),
  },
  [523] = {
    [sym_line_comment] = STATE(523),
    [sym_block_comment] = STATE(523),
    [ts_builtin_sym_end] = ACTIONS(1843),
    [sym_identifier] = ACTIONS(1845),
    [anon_sym_SEMI] = ACTIONS(1843),
    [anon_sym_macro_rules_BANG] = ACTIONS(1843),
    [anon_sym_LPAREN] = ACTIONS(1843),
    [anon_sym_LBRACK] = ACTIONS(1843),
    [anon_sym_LBRACE] = ACTIONS(1843),
    [anon_sym_RBRACE] = ACTIONS(1843),
    [anon_sym_STAR] = ACTIONS(1843),
    [anon_sym_u8] = ACTIONS(1845),
    [anon_sym_i8] = ACTIONS(1845),
    [anon_sym_u16] = ACTIONS(1845),
    [anon_sym_i16] = ACTIONS(1845),
    [anon_sym_u32] = ACTIONS(1845),
    [anon_sym_i32] = ACTIONS(1845),
    [anon_sym_u64] = ACTIONS(1845),
    [anon_sym_i64] = ACTIONS(1845),
    [anon_sym_u128] = ACTIONS(1845),
    [anon_sym_i128] = ACTIONS(1845),
    [anon_sym_isize] = ACTIONS(1845),
    [anon_sym_usize] = ACTIONS(1845),
    [anon_sym_f32] = ACTIONS(1845),
    [anon_sym_f64] = ACTIONS(1845),
    [anon_sym_bool] = ACTIONS(1845),
    [anon_sym_str] = ACTIONS(1845),
    [anon_sym_char] = ACTIONS(1845),
    [anon_sym_DASH] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1843),
    [anon_sym_AMP] = ACTIONS(1843),
    [anon_sym_PIPE] = ACTIONS(1843),
    [anon_sym_LT] = ACTIONS(1843),
    [anon_sym_DOT_DOT] = ACTIONS(1843),
    [anon_sym_COLON_COLON] = ACTIONS(1843),
    [anon_sym_POUND] = ACTIONS(1843),
    [anon_sym_SQUOTE] = ACTIONS(1845),
    [anon_sym_async] = ACTIONS(1845),
    [anon_sym_break] = ACTIONS(1845),
    [anon_sym_const] = ACTIONS(1845),
    [anon_sym_continue] = ACTIONS(1845),
    [anon_sym_default] = ACTIONS(1845),
    [anon_sym_enum] = ACTIONS(1845),
    [anon_sym_fn] = ACTIONS(1845),
    [anon_sym_for] = ACTIONS(1845),
    [anon_sym_gen] = ACTIONS(1845),
    [anon_sym_if] = ACTIONS(1845),
    [anon_sym_impl] = ACTIONS(1845),
    [anon_sym_let] = ACTIONS(1845),
    [anon_sym_loop] = ACTIONS(1845),
    [anon_sym_match] = ACTIONS(1845),
    [anon_sym_mod] = ACTIONS(1845),
    [anon_sym_pub] = ACTIONS(1845),
    [anon_sym_return] = ACTIONS(1845),
    [anon_sym_static] = ACTIONS(1845),
    [anon_sym_struct] = ACTIONS(1845),
    [anon_sym_trait] = ACTIONS(1845),
    [anon_sym_type] = ACTIONS(1845),
    [anon_sym_union] = ACTIONS(1845),
    [anon_sym_unsafe] = ACTIONS(1845),
    [anon_sym_use] = ACTIONS(1845),
    [anon_sym_while] = ACTIONS(1845),
    [anon_sym_extern] = ACTIONS(1845),
    [anon_sym_yield] = ACTIONS(1845),
    [anon_sym_move] = ACTIONS(1845),
    [anon_sym_try] = ACTIONS(1845),
    [sym_integer_literal] = ACTIONS(1843),
    [aux_sym_string_literal_token1] = ACTIONS(1843),
    [sym_char_literal] = ACTIONS(1843),
    [anon_sym_true] = ACTIONS(1845),
    [anon_sym_false] = ACTIONS(1845),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1845),
    [sym_super] = ACTIONS(1845),
    [sym_crate] = ACTIONS(1845),
    [sym_metavariable] = ACTIONS(1843),
    [sym__raw_string_literal_start] = ACTIONS(1843),
    [sym_float_literal] = ACTIONS(1843),
  },
  [524] = {
    [sym_line_comment] = STATE(524),
    [sym_block_comment] = STATE(524),
    [ts_builtin_sym_end] = ACTIONS(1847),
    [sym_identifier] = ACTIONS(1849),
    [anon_sym_SEMI] = ACTIONS(1847),
    [anon_sym_macro_rules_BANG] = ACTIONS(1847),
    [anon_sym_LPAREN] = ACTIONS(1847),
    [anon_sym_LBRACK] = ACTIONS(1847),
    [anon_sym_LBRACE] = ACTIONS(1847),
    [anon_sym_RBRACE] = ACTIONS(1847),
    [anon_sym_STAR] = ACTIONS(1847),
    [anon_sym_u8] = ACTIONS(1849),
    [anon_sym_i8] = ACTIONS(1849),
    [anon_sym_u16] = ACTIONS(1849),
    [anon_sym_i16] = ACTIONS(1849),
    [anon_sym_u32] = ACTIONS(1849),
    [anon_sym_i32] = ACTIONS(1849),
    [anon_sym_u64] = ACTIONS(1849),
    [anon_sym_i64] = ACTIONS(1849),
    [anon_sym_u128] = ACTIONS(1849),
    [anon_sym_i128] = ACTIONS(1849),
    [anon_sym_isize] = ACTIONS(1849),
    [anon_sym_usize] = ACTIONS(1849),
    [anon_sym_f32] = ACTIONS(1849),
    [anon_sym_f64] = ACTIONS(1849),
    [anon_sym_bool] = ACTIONS(1849),
    [anon_sym_str] = ACTIONS(1849),
    [anon_sym_char] = ACTIONS(1849),
    [anon_sym_DASH] = ACTIONS(1847),
    [anon_sym_BANG] = ACTIONS(1847),
    [anon_sym_AMP] = ACTIONS(1847),
    [anon_sym_PIPE] = ACTIONS(1847),
    [anon_sym_LT] = ACTIONS(1847),
    [anon_sym_DOT_DOT] = ACTIONS(1847),
    [anon_sym_COLON_COLON] = ACTIONS(1847),
    [anon_sym_POUND] = ACTIONS(1847),
    [anon_sym_SQUOTE] = ACTIONS(1849),
    [anon_sym_async] = ACTIONS(1849),
    [anon_sym_break] = ACTIONS(1849),
    [anon_sym_const] = ACTIONS(1849),
    [anon_sym_continue] = ACTIONS(1849),
    [anon_sym_default] = ACTIONS(1849),
    [anon_sym_enum] = ACTIONS(1849),
    [anon_sym_fn] = ACTIONS(1849),
    [anon_sym_for] = ACTIONS(1849),
    [anon_sym_gen] = ACTIONS(1849),
    [anon_sym_if] = ACTIONS(1849),
    [anon_sym_impl] = ACTIONS(1849),
    [anon_sym_let] = ACTIONS(1849),
    [anon_sym_loop] = ACTIONS(1849),
    [anon_sym_match] = ACTIONS(1849),
    [anon_sym_mod] = ACTIONS(1849),
    [anon_sym_pub] = ACTIONS(1849),
    [anon_sym_return] = ACTIONS(1849),
    [anon_sym_static] = ACTIONS(1849),
    [anon_sym_struct] = ACTIONS(1849),
    [anon_sym_trait] = ACTIONS(1849),
    [anon_sym_type] = ACTIONS(1849),
    [anon_sym_union] = ACTIONS(1849),
    [anon_sym_unsafe] = ACTIONS(1849),
    [anon_sym_use] = ACTIONS(1849),
    [anon_sym_while] = ACTIONS(1849),
    [anon_sym_extern] = ACTIONS(1849),
    [anon_sym_yield] = ACTIONS(1849),
    [anon_sym_move] = ACTIONS(1849),
    [anon_sym_try] = ACTIONS(1849),
    [sym_integer_literal] = ACTIONS(1847),
    [aux_sym_string_literal_token1] = ACTIONS(1847),
    [sym_char_literal] = ACTIONS(1847),
    [anon_sym_true] = ACTIONS(1849),
    [anon_sym_false] = ACTIONS(1849),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1849),
    [sym_super] = ACTIONS(1849),
    [sym_crate] = ACTIONS(1849),
    [sym_metavariable] = ACTIONS(1847),
    [sym__raw_string_literal_start] = ACTIONS(1847),
    [sym_float_literal] = ACTIONS(1847),
  },
  [525] = {
    [sym_line_comment] = STATE(525),
    [sym_block_comment] = STATE(525),
    [ts_builtin_sym_end] = ACTIONS(1851),
    [sym_identifier] = ACTIONS(1853),
    [anon_sym_SEMI] = ACTIONS(1851),
    [anon_sym_macro_rules_BANG] = ACTIONS(1851),
    [anon_sym_LPAREN] = ACTIONS(1851),
    [anon_sym_LBRACK] = ACTIONS(1851),
    [anon_sym_LBRACE] = ACTIONS(1851),
    [anon_sym_RBRACE] = ACTIONS(1851),
    [anon_sym_STAR] = ACTIONS(1851),
    [anon_sym_u8] = ACTIONS(1853),
    [anon_sym_i8] = ACTIONS(1853),
    [anon_sym_u16] = ACTIONS(1853),
    [anon_sym_i16] = ACTIONS(1853),
    [anon_sym_u32] = ACTIONS(1853),
    [anon_sym_i32] = ACTIONS(1853),
    [anon_sym_u64] = ACTIONS(1853),
    [anon_sym_i64] = ACTIONS(1853),
    [anon_sym_u128] = ACTIONS(1853),
    [anon_sym_i128] = ACTIONS(1853),
    [anon_sym_isize] = ACTIONS(1853),
    [anon_sym_usize] = ACTIONS(1853),
    [anon_sym_f32] = ACTIONS(1853),
    [anon_sym_f64] = ACTIONS(1853),
    [anon_sym_bool] = ACTIONS(1853),
    [anon_sym_str] = ACTIONS(1853),
    [anon_sym_char] = ACTIONS(1853),
    [anon_sym_DASH] = ACTIONS(1851),
    [anon_sym_BANG] = ACTIONS(1851),
    [anon_sym_AMP] = ACTIONS(1851),
    [anon_sym_PIPE] = ACTIONS(1851),
    [anon_sym_LT] = ACTIONS(1851),
    [anon_sym_DOT_DOT] = ACTIONS(1851),
    [anon_sym_COLON_COLON] = ACTIONS(1851),
    [anon_sym_POUND] = ACTIONS(1851),
    [anon_sym_SQUOTE] = ACTIONS(1853),
    [anon_sym_async] = ACTIONS(1853),
    [anon_sym_break] = ACTIONS(1853),
    [anon_sym_const] = ACTIONS(1853),
    [anon_sym_continue] = ACTIONS(1853),
    [anon_sym_default] = ACTIONS(1853),
    [anon_sym_enum] = ACTIONS(1853),
    [anon_sym_fn] = ACTIONS(1853),
    [anon_sym_for] = ACTIONS(1853),
    [anon_sym_gen] = ACTIONS(1853),
    [anon_sym_if] = ACTIONS(1853),
    [anon_sym_impl] = ACTIONS(1853),
    [anon_sym_let] = ACTIONS(1853),
    [anon_sym_loop] = ACTIONS(1853),
    [anon_sym_match] = ACTIONS(1853),
    [anon_sym_mod] = ACTIONS(1853),
    [anon_sym_pub] = ACTIONS(1853),
    [anon_sym_return] = ACTIONS(1853),
    [anon_sym_static] = ACTIONS(1853),
    [anon_sym_struct] = ACTIONS(1853),
    [anon_sym_trait] = ACTIONS(1853),
    [anon_sym_type] = ACTIONS(1853),
    [anon_sym_union] = ACTIONS(1853),
    [anon_sym_unsafe] = ACTIONS(1853),
    [anon_sym_use] = ACTIONS(1853),
    [anon_sym_while] = ACTIONS(1853),
    [anon_sym_extern] = ACTIONS(1853),
    [anon_sym_yield] = ACTIONS(1853),
    [anon_sym_move] = ACTIONS(1853),
    [anon_sym_try] = ACTIONS(1853),
    [sym_integer_literal] = ACTIONS(1851),
    [aux_sym_string_literal_token1] = ACTIONS(1851),
    [sym_char_literal] = ACTIONS(1851),
    [anon_sym_true] = ACTIONS(1853),
    [anon_sym_false] = ACTIONS(1853),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1853),
    [sym_super] = ACTIONS(1853),
    [sym_crate] = ACTIONS(1853),
    [sym_metavariable] = ACTIONS(1851),
    [sym__raw_string_literal_start] = ACTIONS(1851),
    [sym_float_literal] = ACTIONS(1851),
  },
  [526] = {
    [sym_line_comment] = STATE(526),
    [sym_block_comment] = STATE(526),
    [ts_builtin_sym_end] = ACTIONS(1855),
    [sym_identifier] = ACTIONS(1857),
    [anon_sym_SEMI] = ACTIONS(1855),
    [anon_sym_macro_rules_BANG] = ACTIONS(1855),
    [anon_sym_LPAREN] = ACTIONS(1855),
    [anon_sym_LBRACK] = ACTIONS(1855),
    [anon_sym_LBRACE] = ACTIONS(1855),
    [anon_sym_RBRACE] = ACTIONS(1855),
    [anon_sym_STAR] = ACTIONS(1855),
    [anon_sym_u8] = ACTIONS(1857),
    [anon_sym_i8] = ACTIONS(1857),
    [anon_sym_u16] = ACTIONS(1857),
    [anon_sym_i16] = ACTIONS(1857),
    [anon_sym_u32] = ACTIONS(1857),
    [anon_sym_i32] = ACTIONS(1857),
    [anon_sym_u64] = ACTIONS(1857),
    [anon_sym_i64] = ACTIONS(1857),
    [anon_sym_u128] = ACTIONS(1857),
    [anon_sym_i128] = ACTIONS(1857),
    [anon_sym_isize] = ACTIONS(1857),
    [anon_sym_usize] = ACTIONS(1857),
    [anon_sym_f32] = ACTIONS(1857),
    [anon_sym_f64] = ACTIONS(1857),
    [anon_sym_bool] = ACTIONS(1857),
    [anon_sym_str] = ACTIONS(1857),
    [anon_sym_char] = ACTIONS(1857),
    [anon_sym_DASH] = ACTIONS(1855),
    [anon_sym_BANG] = ACTIONS(1855),
    [anon_sym_AMP] = ACTIONS(1855),
    [anon_sym_PIPE] = ACTIONS(1855),
    [anon_sym_LT] = ACTIONS(1855),
    [anon_sym_DOT_DOT] = ACTIONS(1855),
    [anon_sym_COLON_COLON] = ACTIONS(1855),
    [anon_sym_POUND] = ACTIONS(1855),
    [anon_sym_SQUOTE] = ACTIONS(1857),
    [anon_sym_async] = ACTIONS(1857),
    [anon_sym_break] = ACTIONS(1857),
    [anon_sym_const] = ACTIONS(1857),
    [anon_sym_continue] = ACTIONS(1857),
    [anon_sym_default] = ACTIONS(1857),
    [anon_sym_enum] = ACTIONS(1857),
    [anon_sym_fn] = ACTIONS(1857),
    [anon_sym_for] = ACTIONS(1857),
    [anon_sym_gen] = ACTIONS(1857),
    [anon_sym_if] = ACTIONS(1857),
    [anon_sym_impl] = ACTIONS(1857),
    [anon_sym_let] = ACTIONS(1857),
    [anon_sym_loop] = ACTIONS(1857),
    [anon_sym_match] = ACTIONS(1857),
    [anon_sym_mod] = ACTIONS(1857),
    [anon_sym_pub] = ACTIONS(1857),
    [anon_sym_return] = ACTIONS(1857),
    [anon_sym_static] = ACTIONS(1857),
    [anon_sym_struct] = ACTIONS(1857),
    [anon_sym_trait] = ACTIONS(1857),
    [anon_sym_type] = ACTIONS(1857),
    [anon_sym_union] = ACTIONS(1857),
    [anon_sym_unsafe] = ACTIONS(1857),
    [anon_sym_use] = ACTIONS(1857),
    [anon_sym_while] = ACTIONS(1857),
    [anon_sym_extern] = ACTIONS(1857),
    [anon_sym_yield] = ACTIONS(1857),
    [anon_sym_move] = ACTIONS(1857),
    [anon_sym_try] = ACTIONS(1857),
    [sym_integer_literal] = ACTIONS(1855),
    [aux_sym_string_literal_token1] = ACTIONS(1855),
    [sym_char_literal] = ACTIONS(1855),
    [anon_sym_true] = ACTIONS(1857),
    [anon_sym_false] = ACTIONS(1857),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1857),
    [sym_super] = ACTIONS(1857),
    [sym_crate] = ACTIONS(1857),
    [sym_metavariable] = ACTIONS(1855),
    [sym__raw_string_literal_start] = ACTIONS(1855),
    [sym_float_literal] = ACTIONS(1855),
  },
  [527] = {
    [sym_line_comment] = STATE(527),
    [sym_block_comment] = STATE(527),
    [ts_builtin_sym_end] = ACTIONS(1859),
    [sym_identifier] = ACTIONS(1861),
    [anon_sym_SEMI] = ACTIONS(1859),
    [anon_sym_macro_rules_BANG] = ACTIONS(1859),
    [anon_sym_LPAREN] = ACTIONS(1859),
    [anon_sym_LBRACK] = ACTIONS(1859),
    [anon_sym_LBRACE] = ACTIONS(1859),
    [anon_sym_RBRACE] = ACTIONS(1859),
    [anon_sym_STAR] = ACTIONS(1859),
    [anon_sym_u8] = ACTIONS(1861),
    [anon_sym_i8] = ACTIONS(1861),
    [anon_sym_u16] = ACTIONS(1861),
    [anon_sym_i16] = ACTIONS(1861),
    [anon_sym_u32] = ACTIONS(1861),
    [anon_sym_i32] = ACTIONS(1861),
    [anon_sym_u64] = ACTIONS(1861),
    [anon_sym_i64] = ACTIONS(1861),
    [anon_sym_u128] = ACTIONS(1861),
    [anon_sym_i128] = ACTIONS(1861),
    [anon_sym_isize] = ACTIONS(1861),
    [anon_sym_usize] = ACTIONS(1861),
    [anon_sym_f32] = ACTIONS(1861),
    [anon_sym_f64] = ACTIONS(1861),
    [anon_sym_bool] = ACTIONS(1861),
    [anon_sym_str] = ACTIONS(1861),
    [anon_sym_char] = ACTIONS(1861),
    [anon_sym_DASH] = ACTIONS(1859),
    [anon_sym_BANG] = ACTIONS(1859),
    [anon_sym_AMP] = ACTIONS(1859),
    [anon_sym_PIPE] = ACTIONS(1859),
    [anon_sym_LT] = ACTIONS(1859),
    [anon_sym_DOT_DOT] = ACTIONS(1859),
    [anon_sym_COLON_COLON] = ACTIONS(1859),
    [anon_sym_POUND] = ACTIONS(1859),
    [anon_sym_SQUOTE] = ACTIONS(1861),
    [anon_sym_async] = ACTIONS(1861),
    [anon_sym_break] = ACTIONS(1861),
    [anon_sym_const] = ACTIONS(1861),
    [anon_sym_continue] = ACTIONS(1861),
    [anon_sym_default] = ACTIONS(1861),
    [anon_sym_enum] = ACTIONS(1861),
    [anon_sym_fn] = ACTIONS(1861),
    [anon_sym_for] = ACTIONS(1861),
    [anon_sym_gen] = ACTIONS(1861),
    [anon_sym_if] = ACTIONS(1861),
    [anon_sym_impl] = ACTIONS(1861),
    [anon_sym_let] = ACTIONS(1861),
    [anon_sym_loop] = ACTIONS(1861),
    [anon_sym_match] = ACTIONS(1861),
    [anon_sym_mod] = ACTIONS(1861),
    [anon_sym_pub] = ACTIONS(1861),
    [anon_sym_return] = ACTIONS(1861),
    [anon_sym_static] = ACTIONS(1861),
    [anon_sym_struct] = ACTIONS(1861),
    [anon_sym_trait] = ACTIONS(1861),
    [anon_sym_type] = ACTIONS(1861),
    [anon_sym_union] = ACTIONS(1861),
    [anon_sym_unsafe] = ACTIONS(1861),
    [anon_sym_use] = ACTIONS(1861),
    [anon_sym_while] = ACTIONS(1861),
    [anon_sym_extern] = ACTIONS(1861),
    [anon_sym_yield] = ACTIONS(1861),
    [anon_sym_move] = ACTIONS(1861),
    [anon_sym_try] = ACTIONS(1861),
    [sym_integer_literal] = ACTIONS(1859),
    [aux_sym_string_literal_token1] = ACTIONS(1859),
    [sym_char_literal] = ACTIONS(1859),
    [anon_sym_true] = ACTIONS(1861),
    [anon_sym_false] = ACTIONS(1861),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1861),
    [sym_super] = ACTIONS(1861),
    [sym_crate] = ACTIONS(1861),
    [sym_metavariable] = ACTIONS(1859),
    [sym__raw_string_literal_start] = ACTIONS(1859),
    [sym_float_literal] = ACTIONS(1859),
  },
  [528] = {
    [sym_line_comment] = STATE(528),
    [sym_block_comment] = STATE(528),
    [ts_builtin_sym_end] = ACTIONS(1863),
    [sym_identifier] = ACTIONS(1865),
    [anon_sym_SEMI] = ACTIONS(1863),
    [anon_sym_macro_rules_BANG] = ACTIONS(1863),
    [anon_sym_LPAREN] = ACTIONS(1863),
    [anon_sym_LBRACK] = ACTIONS(1863),
    [anon_sym_LBRACE] = ACTIONS(1863),
    [anon_sym_RBRACE] = ACTIONS(1863),
    [anon_sym_STAR] = ACTIONS(1863),
    [anon_sym_u8] = ACTIONS(1865),
    [anon_sym_i8] = ACTIONS(1865),
    [anon_sym_u16] = ACTIONS(1865),
    [anon_sym_i16] = ACTIONS(1865),
    [anon_sym_u32] = ACTIONS(1865),
    [anon_sym_i32] = ACTIONS(1865),
    [anon_sym_u64] = ACTIONS(1865),
    [anon_sym_i64] = ACTIONS(1865),
    [anon_sym_u128] = ACTIONS(1865),
    [anon_sym_i128] = ACTIONS(1865),
    [anon_sym_isize] = ACTIONS(1865),
    [anon_sym_usize] = ACTIONS(1865),
    [anon_sym_f32] = ACTIONS(1865),
    [anon_sym_f64] = ACTIONS(1865),
    [anon_sym_bool] = ACTIONS(1865),
    [anon_sym_str] = ACTIONS(1865),
    [anon_sym_char] = ACTIONS(1865),
    [anon_sym_DASH] = ACTIONS(1863),
    [anon_sym_BANG] = ACTIONS(1863),
    [anon_sym_AMP] = ACTIONS(1863),
    [anon_sym_PIPE] = ACTIONS(1863),
    [anon_sym_LT] = ACTIONS(1863),
    [anon_sym_DOT_DOT] = ACTIONS(1863),
    [anon_sym_COLON_COLON] = ACTIONS(1863),
    [anon_sym_POUND] = ACTIONS(1863),
    [anon_sym_SQUOTE] = ACTIONS(1865),
    [anon_sym_async] = ACTIONS(1865),
    [anon_sym_break] = ACTIONS(1865),
    [anon_sym_const] = ACTIONS(1865),
    [anon_sym_continue] = ACTIONS(1865),
    [anon_sym_default] = ACTIONS(1865),
    [anon_sym_enum] = ACTIONS(1865),
    [anon_sym_fn] = ACTIONS(1865),
    [anon_sym_for] = ACTIONS(1865),
    [anon_sym_gen] = ACTIONS(1865),
    [anon_sym_if] = ACTIONS(1865),
    [anon_sym_impl] = ACTIONS(1865),
    [anon_sym_let] = ACTIONS(1865),
    [anon_sym_loop] = ACTIONS(1865),
    [anon_sym_match] = ACTIONS(1865),
    [anon_sym_mod] = ACTIONS(1865),
    [anon_sym_pub] = ACTIONS(1865),
    [anon_sym_return] = ACTIONS(1865),
    [anon_sym_static] = ACTIONS(1865),
    [anon_sym_struct] = ACTIONS(1865),
    [anon_sym_trait] = ACTIONS(1865),
    [anon_sym_type] = ACTIONS(1865),
    [anon_sym_union] = ACTIONS(1865),
    [anon_sym_unsafe] = ACTIONS(1865),
    [anon_sym_use] = ACTIONS(1865),
    [anon_sym_while] = ACTIONS(1865),
    [anon_sym_extern] = ACTIONS(1865),
    [anon_sym_yield] = ACTIONS(1865),
    [anon_sym_move] = ACTIONS(1865),
    [anon_sym_try] = ACTIONS(1865),
    [sym_integer_literal] = ACTIONS(1863),
    [aux_sym_string_literal_token1] = ACTIONS(1863),
    [sym_char_literal] = ACTIONS(1863),
    [anon_sym_true] = ACTIONS(1865),
    [anon_sym_false] = ACTIONS(1865),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1865),
    [sym_super] = ACTIONS(1865),
    [sym_crate] = ACTIONS(1865),
    [sym_metavariable] = ACTIONS(1863),
    [sym__raw_string_literal_start] = ACTIONS(1863),
    [sym_float_literal] = ACTIONS(1863),
  },
  [529] = {
    [sym_line_comment] = STATE(529),
    [sym_block_comment] = STATE(529),
    [ts_builtin_sym_end] = ACTIONS(1867),
    [sym_identifier] = ACTIONS(1869),
    [anon_sym_SEMI] = ACTIONS(1867),
    [anon_sym_macro_rules_BANG] = ACTIONS(1867),
    [anon_sym_LPAREN] = ACTIONS(1867),
    [anon_sym_LBRACK] = ACTIONS(1867),
    [anon_sym_LBRACE] = ACTIONS(1867),
    [anon_sym_RBRACE] = ACTIONS(1867),
    [anon_sym_STAR] = ACTIONS(1867),
    [anon_sym_u8] = ACTIONS(1869),
    [anon_sym_i8] = ACTIONS(1869),
    [anon_sym_u16] = ACTIONS(1869),
    [anon_sym_i16] = ACTIONS(1869),
    [anon_sym_u32] = ACTIONS(1869),
    [anon_sym_i32] = ACTIONS(1869),
    [anon_sym_u64] = ACTIONS(1869),
    [anon_sym_i64] = ACTIONS(1869),
    [anon_sym_u128] = ACTIONS(1869),
    [anon_sym_i128] = ACTIONS(1869),
    [anon_sym_isize] = ACTIONS(1869),
    [anon_sym_usize] = ACTIONS(1869),
    [anon_sym_f32] = ACTIONS(1869),
    [anon_sym_f64] = ACTIONS(1869),
    [anon_sym_bool] = ACTIONS(1869),
    [anon_sym_str] = ACTIONS(1869),
    [anon_sym_char] = ACTIONS(1869),
    [anon_sym_DASH] = ACTIONS(1867),
    [anon_sym_BANG] = ACTIONS(1867),
    [anon_sym_AMP] = ACTIONS(1867),
    [anon_sym_PIPE] = ACTIONS(1867),
    [anon_sym_LT] = ACTIONS(1867),
    [anon_sym_DOT_DOT] = ACTIONS(1867),
    [anon_sym_COLON_COLON] = ACTIONS(1867),
    [anon_sym_POUND] = ACTIONS(1867),
    [anon_sym_SQUOTE] = ACTIONS(1869),
    [anon_sym_async] = ACTIONS(1869),
    [anon_sym_break] = ACTIONS(1869),
    [anon_sym_const] = ACTIONS(1869),
    [anon_sym_continue] = ACTIONS(1869),
    [anon_sym_default] = ACTIONS(1869),
    [anon_sym_enum] = ACTIONS(1869),
    [anon_sym_fn] = ACTIONS(1869),
    [anon_sym_for] = ACTIONS(1869),
    [anon_sym_gen] = ACTIONS(1869),
    [anon_sym_if] = ACTIONS(1869),
    [anon_sym_impl] = ACTIONS(1869),
    [anon_sym_let] = ACTIONS(1869),
    [anon_sym_loop] = ACTIONS(1869),
    [anon_sym_match] = ACTIONS(1869),
    [anon_sym_mod] = ACTIONS(1869),
    [anon_sym_pub] = ACTIONS(1869),
    [anon_sym_return] = ACTIONS(1869),
    [anon_sym_static] = ACTIONS(1869),
    [anon_sym_struct] = ACTIONS(1869),
    [anon_sym_trait] = ACTIONS(1869),
    [anon_sym_type] = ACTIONS(1869),
    [anon_sym_union] = ACTIONS(1869),
    [anon_sym_unsafe] = ACTIONS(1869),
    [anon_sym_use] = ACTIONS(1869),
    [anon_sym_while] = ACTIONS(1869),
    [anon_sym_extern] = ACTIONS(1869),
    [anon_sym_yield] = ACTIONS(1869),
    [anon_sym_move] = ACTIONS(1869),
    [anon_sym_try] = ACTIONS(1869),
    [sym_integer_literal] = ACTIONS(1867),
    [aux_sym_string_literal_token1] = ACTIONS(1867),
    [sym_char_literal] = ACTIONS(1867),
    [anon_sym_true] = ACTIONS(1869),
    [anon_sym_false] = ACTIONS(1869),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1869),
    [sym_super] = ACTIONS(1869),
    [sym_crate] = ACTIONS(1869),
    [sym_metavariable] = ACTIONS(1867),
    [sym__raw_string_literal_start] = ACTIONS(1867),
    [sym_float_literal] = ACTIONS(1867),
  },
  [530] = {
    [sym_line_comment] = STATE(530),
    [sym_block_comment] = STATE(530),
    [ts_builtin_sym_end] = ACTIONS(1871),
    [sym_identifier] = ACTIONS(1873),
    [anon_sym_SEMI] = ACTIONS(1871),
    [anon_sym_macro_rules_BANG] = ACTIONS(1871),
    [anon_sym_LPAREN] = ACTIONS(1871),
    [anon_sym_LBRACK] = ACTIONS(1871),
    [anon_sym_LBRACE] = ACTIONS(1871),
    [anon_sym_RBRACE] = ACTIONS(1871),
    [anon_sym_STAR] = ACTIONS(1871),
    [anon_sym_u8] = ACTIONS(1873),
    [anon_sym_i8] = ACTIONS(1873),
    [anon_sym_u16] = ACTIONS(1873),
    [anon_sym_i16] = ACTIONS(1873),
    [anon_sym_u32] = ACTIONS(1873),
    [anon_sym_i32] = ACTIONS(1873),
    [anon_sym_u64] = ACTIONS(1873),
    [anon_sym_i64] = ACTIONS(1873),
    [anon_sym_u128] = ACTIONS(1873),
    [anon_sym_i128] = ACTIONS(1873),
    [anon_sym_isize] = ACTIONS(1873),
    [anon_sym_usize] = ACTIONS(1873),
    [anon_sym_f32] = ACTIONS(1873),
    [anon_sym_f64] = ACTIONS(1873),
    [anon_sym_bool] = ACTIONS(1873),
    [anon_sym_str] = ACTIONS(1873),
    [anon_sym_char] = ACTIONS(1873),
    [anon_sym_DASH] = ACTIONS(1871),
    [anon_sym_BANG] = ACTIONS(1871),
    [anon_sym_AMP] = ACTIONS(1871),
    [anon_sym_PIPE] = ACTIONS(1871),
    [anon_sym_LT] = ACTIONS(1871),
    [anon_sym_DOT_DOT] = ACTIONS(1871),
    [anon_sym_COLON_COLON] = ACTIONS(1871),
    [anon_sym_POUND] = ACTIONS(1871),
    [anon_sym_SQUOTE] = ACTIONS(1873),
    [anon_sym_async] = ACTIONS(1873),
    [anon_sym_break] = ACTIONS(1873),
    [anon_sym_const] = ACTIONS(1873),
    [anon_sym_continue] = ACTIONS(1873),
    [anon_sym_default] = ACTIONS(1873),
    [anon_sym_enum] = ACTIONS(1873),
    [anon_sym_fn] = ACTIONS(1873),
    [anon_sym_for] = ACTIONS(1873),
    [anon_sym_gen] = ACTIONS(1873),
    [anon_sym_if] = ACTIONS(1873),
    [anon_sym_impl] = ACTIONS(1873),
    [anon_sym_let] = ACTIONS(1873),
    [anon_sym_loop] = ACTIONS(1873),
    [anon_sym_match] = ACTIONS(1873),
    [anon_sym_mod] = ACTIONS(1873),
    [anon_sym_pub] = ACTIONS(1873),
    [anon_sym_return] = ACTIONS(1873),
    [anon_sym_static] = ACTIONS(1873),
    [anon_sym_struct] = ACTIONS(1873),
    [anon_sym_trait] = ACTIONS(1873),
    [anon_sym_type] = ACTIONS(1873),
    [anon_sym_union] = ACTIONS(1873),
    [anon_sym_unsafe] = ACTIONS(1873),
    [anon_sym_use] = ACTIONS(1873),
    [anon_sym_while] = ACTIONS(1873),
    [anon_sym_extern] = ACTIONS(1873),
    [anon_sym_yield] = ACTIONS(1873),
    [anon_sym_move] = ACTIONS(1873),
    [anon_sym_try] = ACTIONS(1873),
    [sym_integer_literal] = ACTIONS(1871),
    [aux_sym_string_literal_token1] = ACTIONS(1871),
    [sym_char_literal] = ACTIONS(1871),
    [anon_sym_true] = ACTIONS(1873),
    [anon_sym_false] = ACTIONS(1873),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1873),
    [sym_super] = ACTIONS(1873),
    [sym_crate] = ACTIONS(1873),
    [sym_metavariable] = ACTIONS(1871),
    [sym__raw_string_literal_start] = ACTIONS(1871),
    [sym_float_literal] = ACTIONS(1871),
  },
  [531] = {
    [sym_line_comment] = STATE(531),
    [sym_block_comment] = STATE(531),
    [ts_builtin_sym_end] = ACTIONS(1875),
    [sym_identifier] = ACTIONS(1877),
    [anon_sym_SEMI] = ACTIONS(1875),
    [anon_sym_macro_rules_BANG] = ACTIONS(1875),
    [anon_sym_LPAREN] = ACTIONS(1875),
    [anon_sym_LBRACK] = ACTIONS(1875),
    [anon_sym_LBRACE] = ACTIONS(1875),
    [anon_sym_RBRACE] = ACTIONS(1875),
    [anon_sym_STAR] = ACTIONS(1875),
    [anon_sym_u8] = ACTIONS(1877),
    [anon_sym_i8] = ACTIONS(1877),
    [anon_sym_u16] = ACTIONS(1877),
    [anon_sym_i16] = ACTIONS(1877),
    [anon_sym_u32] = ACTIONS(1877),
    [anon_sym_i32] = ACTIONS(1877),
    [anon_sym_u64] = ACTIONS(1877),
    [anon_sym_i64] = ACTIONS(1877),
    [anon_sym_u128] = ACTIONS(1877),
    [anon_sym_i128] = ACTIONS(1877),
    [anon_sym_isize] = ACTIONS(1877),
    [anon_sym_usize] = ACTIONS(1877),
    [anon_sym_f32] = ACTIONS(1877),
    [anon_sym_f64] = ACTIONS(1877),
    [anon_sym_bool] = ACTIONS(1877),
    [anon_sym_str] = ACTIONS(1877),
    [anon_sym_char] = ACTIONS(1877),
    [anon_sym_DASH] = ACTIONS(1875),
    [anon_sym_BANG] = ACTIONS(1875),
    [anon_sym_AMP] = ACTIONS(1875),
    [anon_sym_PIPE] = ACTIONS(1875),
    [anon_sym_LT] = ACTIONS(1875),
    [anon_sym_DOT_DOT] = ACTIONS(1875),
    [anon_sym_COLON_COLON] = ACTIONS(1875),
    [anon_sym_POUND] = ACTIONS(1875),
    [anon_sym_SQUOTE] = ACTIONS(1877),
    [anon_sym_async] = ACTIONS(1877),
    [anon_sym_break] = ACTIONS(1877),
    [anon_sym_const] = ACTIONS(1877),
    [anon_sym_continue] = ACTIONS(1877),
    [anon_sym_default] = ACTIONS(1877),
    [anon_sym_enum] = ACTIONS(1877),
    [anon_sym_fn] = ACTIONS(1877),
    [anon_sym_for] = ACTIONS(1877),
    [anon_sym_gen] = ACTIONS(1877),
    [anon_sym_if] = ACTIONS(1877),
    [anon_sym_impl] = ACTIONS(1877),
    [anon_sym_let] = ACTIONS(1877),
    [anon_sym_loop] = ACTIONS(1877),
    [anon_sym_match] = ACTIONS(1877),
    [anon_sym_mod] = ACTIONS(1877),
    [anon_sym_pub] = ACTIONS(1877),
    [anon_sym_return] = ACTIONS(1877),
    [anon_sym_static] = ACTIONS(1877),
    [anon_sym_struct] = ACTIONS(1877),
    [anon_sym_trait] = ACTIONS(1877),
    [anon_sym_type] = ACTIONS(1877),
    [anon_sym_union] = ACTIONS(1877),
    [anon_sym_unsafe] = ACTIONS(1877),
    [anon_sym_use] = ACTIONS(1877),
    [anon_sym_while] = ACTIONS(1877),
    [anon_sym_extern] = ACTIONS(1877),
    [anon_sym_yield] = ACTIONS(1877),
    [anon_sym_move] = ACTIONS(1877),
    [anon_sym_try] = ACTIONS(1877),
    [sym_integer_literal] = ACTIONS(1875),
    [aux_sym_string_literal_token1] = ACTIONS(1875),
    [sym_char_literal] = ACTIONS(1875),
    [anon_sym_true] = ACTIONS(1877),
    [anon_sym_false] = ACTIONS(1877),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1877),
    [sym_super] = ACTIONS(1877),
    [sym_crate] = ACTIONS(1877),
    [sym_metavariable] = ACTIONS(1875),
    [sym__raw_string_literal_start] = ACTIONS(1875),
    [sym_float_literal] = ACTIONS(1875),
  },
  [532] = {
    [sym_line_comment] = STATE(532),
    [sym_block_comment] = STATE(532),
    [ts_builtin_sym_end] = ACTIONS(1879),
    [sym_identifier] = ACTIONS(1881),
    [anon_sym_SEMI] = ACTIONS(1879),
    [anon_sym_macro_rules_BANG] = ACTIONS(1879),
    [anon_sym_LPAREN] = ACTIONS(1879),
    [anon_sym_LBRACK] = ACTIONS(1879),
    [anon_sym_LBRACE] = ACTIONS(1879),
    [anon_sym_RBRACE] = ACTIONS(1879),
    [anon_sym_STAR] = ACTIONS(1879),
    [anon_sym_u8] = ACTIONS(1881),
    [anon_sym_i8] = ACTIONS(1881),
    [anon_sym_u16] = ACTIONS(1881),
    [anon_sym_i16] = ACTIONS(1881),
    [anon_sym_u32] = ACTIONS(1881),
    [anon_sym_i32] = ACTIONS(1881),
    [anon_sym_u64] = ACTIONS(1881),
    [anon_sym_i64] = ACTIONS(1881),
    [anon_sym_u128] = ACTIONS(1881),
    [anon_sym_i128] = ACTIONS(1881),
    [anon_sym_isize] = ACTIONS(1881),
    [anon_sym_usize] = ACTIONS(1881),
    [anon_sym_f32] = ACTIONS(1881),
    [anon_sym_f64] = ACTIONS(1881),
    [anon_sym_bool] = ACTIONS(1881),
    [anon_sym_str] = ACTIONS(1881),
    [anon_sym_char] = ACTIONS(1881),
    [anon_sym_DASH] = ACTIONS(1879),
    [anon_sym_BANG] = ACTIONS(1879),
    [anon_sym_AMP] = ACTIONS(1879),
    [anon_sym_PIPE] = ACTIONS(1879),
    [anon_sym_LT] = ACTIONS(1879),
    [anon_sym_DOT_DOT] = ACTIONS(1879),
    [anon_sym_COLON_COLON] = ACTIONS(1879),
    [anon_sym_POUND] = ACTIONS(1879),
    [anon_sym_SQUOTE] = ACTIONS(1881),
    [anon_sym_async] = ACTIONS(1881),
    [anon_sym_break] = ACTIONS(1881),
    [anon_sym_const] = ACTIONS(1881),
    [anon_sym_continue] = ACTIONS(1881),
    [anon_sym_default] = ACTIONS(1881),
    [anon_sym_enum] = ACTIONS(1881),
    [anon_sym_fn] = ACTIONS(1881),
    [anon_sym_for] = ACTIONS(1881),
    [anon_sym_gen] = ACTIONS(1881),
    [anon_sym_if] = ACTIONS(1881),
    [anon_sym_impl] = ACTIONS(1881),
    [anon_sym_let] = ACTIONS(1881),
    [anon_sym_loop] = ACTIONS(1881),
    [anon_sym_match] = ACTIONS(1881),
    [anon_sym_mod] = ACTIONS(1881),
    [anon_sym_pub] = ACTIONS(1881),
    [anon_sym_return] = ACTIONS(1881),
    [anon_sym_static] = ACTIONS(1881),
    [anon_sym_struct] = ACTIONS(1881),
    [anon_sym_trait] = ACTIONS(1881),
    [anon_sym_type] = ACTIONS(1881),
    [anon_sym_union] = ACTIONS(1881),
    [anon_sym_unsafe] = ACTIONS(1881),
    [anon_sym_use] = ACTIONS(1881),
    [anon_sym_while] = ACTIONS(1881),
    [anon_sym_extern] = ACTIONS(1881),
    [anon_sym_yield] = ACTIONS(1881),
    [anon_sym_move] = ACTIONS(1881),
    [anon_sym_try] = ACTIONS(1881),
    [sym_integer_literal] = ACTIONS(1879),
    [aux_sym_string_literal_token1] = ACTIONS(1879),
    [sym_char_literal] = ACTIONS(1879),
    [anon_sym_true] = ACTIONS(1881),
    [anon_sym_false] = ACTIONS(1881),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1881),
    [sym_super] = ACTIONS(1881),
    [sym_crate] = ACTIONS(1881),
    [sym_metavariable] = ACTIONS(1879),
    [sym__raw_string_literal_start] = ACTIONS(1879),
    [sym_float_literal] = ACTIONS(1879),
  },
  [533] = {
    [sym_line_comment] = STATE(533),
    [sym_block_comment] = STATE(533),
    [ts_builtin_sym_end] = ACTIONS(1883),
    [sym_identifier] = ACTIONS(1885),
    [anon_sym_SEMI] = ACTIONS(1883),
    [anon_sym_macro_rules_BANG] = ACTIONS(1883),
    [anon_sym_LPAREN] = ACTIONS(1883),
    [anon_sym_LBRACK] = ACTIONS(1883),
    [anon_sym_LBRACE] = ACTIONS(1883),
    [anon_sym_RBRACE] = ACTIONS(1883),
    [anon_sym_STAR] = ACTIONS(1883),
    [anon_sym_u8] = ACTIONS(1885),
    [anon_sym_i8] = ACTIONS(1885),
    [anon_sym_u16] = ACTIONS(1885),
    [anon_sym_i16] = ACTIONS(1885),
    [anon_sym_u32] = ACTIONS(1885),
    [anon_sym_i32] = ACTIONS(1885),
    [anon_sym_u64] = ACTIONS(1885),
    [anon_sym_i64] = ACTIONS(1885),
    [anon_sym_u128] = ACTIONS(1885),
    [anon_sym_i128] = ACTIONS(1885),
    [anon_sym_isize] = ACTIONS(1885),
    [anon_sym_usize] = ACTIONS(1885),
    [anon_sym_f32] = ACTIONS(1885),
    [anon_sym_f64] = ACTIONS(1885),
    [anon_sym_bool] = ACTIONS(1885),
    [anon_sym_str] = ACTIONS(1885),
    [anon_sym_char] = ACTIONS(1885),
    [anon_sym_DASH] = ACTIONS(1883),
    [anon_sym_BANG] = ACTIONS(1883),
    [anon_sym_AMP] = ACTIONS(1883),
    [anon_sym_PIPE] = ACTIONS(1883),
    [anon_sym_LT] = ACTIONS(1883),
    [anon_sym_DOT_DOT] = ACTIONS(1883),
    [anon_sym_COLON_COLON] = ACTIONS(1883),
    [anon_sym_POUND] = ACTIONS(1883),
    [anon_sym_SQUOTE] = ACTIONS(1885),
    [anon_sym_async] = ACTIONS(1885),
    [anon_sym_break] = ACTIONS(1885),
    [anon_sym_const] = ACTIONS(1885),
    [anon_sym_continue] = ACTIONS(1885),
    [anon_sym_default] = ACTIONS(1885),
    [anon_sym_enum] = ACTIONS(1885),
    [anon_sym_fn] = ACTIONS(1885),
    [anon_sym_for] = ACTIONS(1885),
    [anon_sym_gen] = ACTIONS(1885),
    [anon_sym_if] = ACTIONS(1885),
    [anon_sym_impl] = ACTIONS(1885),
    [anon_sym_let] = ACTIONS(1885),
    [anon_sym_loop] = ACTIONS(1885),
    [anon_sym_match] = ACTIONS(1885),
    [anon_sym_mod] = ACTIONS(1885),
    [anon_sym_pub] = ACTIONS(1885),
    [anon_sym_return] = ACTIONS(1885),
    [anon_sym_static] = ACTIONS(1885),
    [anon_sym_struct] = ACTIONS(1885),
    [anon_sym_trait] = ACTIONS(1885),
    [anon_sym_type] = ACTIONS(1885),
    [anon_sym_union] = ACTIONS(1885),
    [anon_sym_unsafe] = ACTIONS(1885),
    [anon_sym_use] = ACTIONS(1885),
    [anon_sym_while] = ACTIONS(1885),
    [anon_sym_extern] = ACTIONS(1885),
    [anon_sym_yield] = ACTIONS(1885),
    [anon_sym_move] = ACTIONS(1885),
    [anon_sym_try] = ACTIONS(1885),
    [sym_integer_literal] = ACTIONS(1883),
    [aux_sym_string_literal_token1] = ACTIONS(1883),
    [sym_char_literal] = ACTIONS(1883),
    [anon_sym_true] = ACTIONS(1885),
    [anon_sym_false] = ACTIONS(1885),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1885),
    [sym_super] = ACTIONS(1885),
    [sym_crate] = ACTIONS(1885),
    [sym_metavariable] = ACTIONS(1883),
    [sym__raw_string_literal_start] = ACTIONS(1883),
    [sym_float_literal] = ACTIONS(1883),
  },
  [534] = {
    [sym_line_comment] = STATE(534),
    [sym_block_comment] = STATE(534),
    [ts_builtin_sym_end] = ACTIONS(1887),
    [sym_identifier] = ACTIONS(1889),
    [anon_sym_SEMI] = ACTIONS(1887),
    [anon_sym_macro_rules_BANG] = ACTIONS(1887),
    [anon_sym_LPAREN] = ACTIONS(1887),
    [anon_sym_LBRACK] = ACTIONS(1887),
    [anon_sym_LBRACE] = ACTIONS(1887),
    [anon_sym_RBRACE] = ACTIONS(1887),
    [anon_sym_STAR] = ACTIONS(1887),
    [anon_sym_u8] = ACTIONS(1889),
    [anon_sym_i8] = ACTIONS(1889),
    [anon_sym_u16] = ACTIONS(1889),
    [anon_sym_i16] = ACTIONS(1889),
    [anon_sym_u32] = ACTIONS(1889),
    [anon_sym_i32] = ACTIONS(1889),
    [anon_sym_u64] = ACTIONS(1889),
    [anon_sym_i64] = ACTIONS(1889),
    [anon_sym_u128] = ACTIONS(1889),
    [anon_sym_i128] = ACTIONS(1889),
    [anon_sym_isize] = ACTIONS(1889),
    [anon_sym_usize] = ACTIONS(1889),
    [anon_sym_f32] = ACTIONS(1889),
    [anon_sym_f64] = ACTIONS(1889),
    [anon_sym_bool] = ACTIONS(1889),
    [anon_sym_str] = ACTIONS(1889),
    [anon_sym_char] = ACTIONS(1889),
    [anon_sym_DASH] = ACTIONS(1887),
    [anon_sym_BANG] = ACTIONS(1887),
    [anon_sym_AMP] = ACTIONS(1887),
    [anon_sym_PIPE] = ACTIONS(1887),
    [anon_sym_LT] = ACTIONS(1887),
    [anon_sym_DOT_DOT] = ACTIONS(1887),
    [anon_sym_COLON_COLON] = ACTIONS(1887),
    [anon_sym_POUND] = ACTIONS(1887),
    [anon_sym_SQUOTE] = ACTIONS(1889),
    [anon_sym_async] = ACTIONS(1889),
    [anon_sym_break] = ACTIONS(1889),
    [anon_sym_const] = ACTIONS(1889),
    [anon_sym_continue] = ACTIONS(1889),
    [anon_sym_default] = ACTIONS(1889),
    [anon_sym_enum] = ACTIONS(1889),
    [anon_sym_fn] = ACTIONS(1889),
    [anon_sym_for] = ACTIONS(1889),
    [anon_sym_gen] = ACTIONS(1889),
    [anon_sym_if] = ACTIONS(1889),
    [anon_sym_impl] = ACTIONS(1889),
    [anon_sym_let] = ACTIONS(1889),
    [anon_sym_loop] = ACTIONS(1889),
    [anon_sym_match] = ACTIONS(1889),
    [anon_sym_mod] = ACTIONS(1889),
    [anon_sym_pub] = ACTIONS(1889),
    [anon_sym_return] = ACTIONS(1889),
    [anon_sym_static] = ACTIONS(1889),
    [anon_sym_struct] = ACTIONS(1889),
    [anon_sym_trait] = ACTIONS(1889),
    [anon_sym_type] = ACTIONS(1889),
    [anon_sym_union] = ACTIONS(1889),
    [anon_sym_unsafe] = ACTIONS(1889),
    [anon_sym_use] = ACTIONS(1889),
    [anon_sym_while] = ACTIONS(1889),
    [anon_sym_extern] = ACTIONS(1889),
    [anon_sym_yield] = ACTIONS(1889),
    [anon_sym_move] = ACTIONS(1889),
    [anon_sym_try] = ACTIONS(1889),
    [sym_integer_literal] = ACTIONS(1887),
    [aux_sym_string_literal_token1] = ACTIONS(1887),
    [sym_char_literal] = ACTIONS(1887),
    [anon_sym_true] = ACTIONS(1889),
    [anon_sym_false] = ACTIONS(1889),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1889),
    [sym_super] = ACTIONS(1889),
    [sym_crate] = ACTIONS(1889),
    [sym_metavariable] = ACTIONS(1887),
    [sym__raw_string_literal_start] = ACTIONS(1887),
    [sym_float_literal] = ACTIONS(1887),
  },
  [535] = {
    [sym_line_comment] = STATE(535),
    [sym_block_comment] = STATE(535),
    [ts_builtin_sym_end] = ACTIONS(1891),
    [sym_identifier] = ACTIONS(1893),
    [anon_sym_SEMI] = ACTIONS(1891),
    [anon_sym_macro_rules_BANG] = ACTIONS(1891),
    [anon_sym_LPAREN] = ACTIONS(1891),
    [anon_sym_LBRACK] = ACTIONS(1891),
    [anon_sym_LBRACE] = ACTIONS(1891),
    [anon_sym_RBRACE] = ACTIONS(1891),
    [anon_sym_STAR] = ACTIONS(1891),
    [anon_sym_u8] = ACTIONS(1893),
    [anon_sym_i8] = ACTIONS(1893),
    [anon_sym_u16] = ACTIONS(1893),
    [anon_sym_i16] = ACTIONS(1893),
    [anon_sym_u32] = ACTIONS(1893),
    [anon_sym_i32] = ACTIONS(1893),
    [anon_sym_u64] = ACTIONS(1893),
    [anon_sym_i64] = ACTIONS(1893),
    [anon_sym_u128] = ACTIONS(1893),
    [anon_sym_i128] = ACTIONS(1893),
    [anon_sym_isize] = ACTIONS(1893),
    [anon_sym_usize] = ACTIONS(1893),
    [anon_sym_f32] = ACTIONS(1893),
    [anon_sym_f64] = ACTIONS(1893),
    [anon_sym_bool] = ACTIONS(1893),
    [anon_sym_str] = ACTIONS(1893),
    [anon_sym_char] = ACTIONS(1893),
    [anon_sym_DASH] = ACTIONS(1891),
    [anon_sym_BANG] = ACTIONS(1891),
    [anon_sym_AMP] = ACTIONS(1891),
    [anon_sym_PIPE] = ACTIONS(1891),
    [anon_sym_LT] = ACTIONS(1891),
    [anon_sym_DOT_DOT] = ACTIONS(1891),
    [anon_sym_COLON_COLON] = ACTIONS(1891),
    [anon_sym_POUND] = ACTIONS(1891),
    [anon_sym_SQUOTE] = ACTIONS(1893),
    [anon_sym_async] = ACTIONS(1893),
    [anon_sym_break] = ACTIONS(1893),
    [anon_sym_const] = ACTIONS(1893),
    [anon_sym_continue] = ACTIONS(1893),
    [anon_sym_default] = ACTIONS(1893),
    [anon_sym_enum] = ACTIONS(1893),
    [anon_sym_fn] = ACTIONS(1893),
    [anon_sym_for] = ACTIONS(1893),
    [anon_sym_gen] = ACTIONS(1893),
    [anon_sym_if] = ACTIONS(1893),
    [anon_sym_impl] = ACTIONS(1893),
    [anon_sym_let] = ACTIONS(1893),
    [anon_sym_loop] = ACTIONS(1893),
    [anon_sym_match] = ACTIONS(1893),
    [anon_sym_mod] = ACTIONS(1893),
    [anon_sym_pub] = ACTIONS(1893),
    [anon_sym_return] = ACTIONS(1893),
    [anon_sym_static] = ACTIONS(1893),
    [anon_sym_struct] = ACTIONS(1893),
    [anon_sym_trait] = ACTIONS(1893),
    [anon_sym_type] = ACTIONS(1893),
    [anon_sym_union] = ACTIONS(1893),
    [anon_sym_unsafe] = ACTIONS(1893),
    [anon_sym_use] = ACTIONS(1893),
    [anon_sym_while] = ACTIONS(1893),
    [anon_sym_extern] = ACTIONS(1893),
    [anon_sym_yield] = ACTIONS(1893),
    [anon_sym_move] = ACTIONS(1893),
    [anon_sym_try] = ACTIONS(1893),
    [sym_integer_literal] = ACTIONS(1891),
    [aux_sym_string_literal_token1] = ACTIONS(1891),
    [sym_char_literal] = ACTIONS(1891),
    [anon_sym_true] = ACTIONS(1893),
    [anon_sym_false] = ACTIONS(1893),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1893),
    [sym_super] = ACTIONS(1893),
    [sym_crate] = ACTIONS(1893),
    [sym_metavariable] = ACTIONS(1891),
    [sym__raw_string_literal_start] = ACTIONS(1891),
    [sym_float_literal] = ACTIONS(1891),
  },
  [536] = {
    [sym_line_comment] = STATE(536),
    [sym_block_comment] = STATE(536),
    [ts_builtin_sym_end] = ACTIONS(1895),
    [sym_identifier] = ACTIONS(1897),
    [anon_sym_SEMI] = ACTIONS(1895),
    [anon_sym_macro_rules_BANG] = ACTIONS(1895),
    [anon_sym_LPAREN] = ACTIONS(1895),
    [anon_sym_LBRACK] = ACTIONS(1895),
    [anon_sym_LBRACE] = ACTIONS(1895),
    [anon_sym_RBRACE] = ACTIONS(1895),
    [anon_sym_STAR] = ACTIONS(1895),
    [anon_sym_u8] = ACTIONS(1897),
    [anon_sym_i8] = ACTIONS(1897),
    [anon_sym_u16] = ACTIONS(1897),
    [anon_sym_i16] = ACTIONS(1897),
    [anon_sym_u32] = ACTIONS(1897),
    [anon_sym_i32] = ACTIONS(1897),
    [anon_sym_u64] = ACTIONS(1897),
    [anon_sym_i64] = ACTIONS(1897),
    [anon_sym_u128] = ACTIONS(1897),
    [anon_sym_i128] = ACTIONS(1897),
    [anon_sym_isize] = ACTIONS(1897),
    [anon_sym_usize] = ACTIONS(1897),
    [anon_sym_f32] = ACTIONS(1897),
    [anon_sym_f64] = ACTIONS(1897),
    [anon_sym_bool] = ACTIONS(1897),
    [anon_sym_str] = ACTIONS(1897),
    [anon_sym_char] = ACTIONS(1897),
    [anon_sym_DASH] = ACTIONS(1895),
    [anon_sym_BANG] = ACTIONS(1895),
    [anon_sym_AMP] = ACTIONS(1895),
    [anon_sym_PIPE] = ACTIONS(1895),
    [anon_sym_LT] = ACTIONS(1895),
    [anon_sym_DOT_DOT] = ACTIONS(1895),
    [anon_sym_COLON_COLON] = ACTIONS(1895),
    [anon_sym_POUND] = ACTIONS(1895),
    [anon_sym_SQUOTE] = ACTIONS(1897),
    [anon_sym_async] = ACTIONS(1897),
    [anon_sym_break] = ACTIONS(1897),
    [anon_sym_const] = ACTIONS(1897),
    [anon_sym_continue] = ACTIONS(1897),
    [anon_sym_default] = ACTIONS(1897),
    [anon_sym_enum] = ACTIONS(1897),
    [anon_sym_fn] = ACTIONS(1897),
    [anon_sym_for] = ACTIONS(1897),
    [anon_sym_gen] = ACTIONS(1897),
    [anon_sym_if] = ACTIONS(1897),
    [anon_sym_impl] = ACTIONS(1897),
    [anon_sym_let] = ACTIONS(1897),
    [anon_sym_loop] = ACTIONS(1897),
    [anon_sym_match] = ACTIONS(1897),
    [anon_sym_mod] = ACTIONS(1897),
    [anon_sym_pub] = ACTIONS(1897),
    [anon_sym_return] = ACTIONS(1897),
    [anon_sym_static] = ACTIONS(1897),
    [anon_sym_struct] = ACTIONS(1897),
    [anon_sym_trait] = ACTIONS(1897),
    [anon_sym_type] = ACTIONS(1897),
    [anon_sym_union] = ACTIONS(1897),
    [anon_sym_unsafe] = ACTIONS(1897),
    [anon_sym_use] = ACTIONS(1897),
    [anon_sym_while] = ACTIONS(1897),
    [anon_sym_extern] = ACTIONS(1897),
    [anon_sym_yield] = ACTIONS(1897),
    [anon_sym_move] = ACTIONS(1897),
    [anon_sym_try] = ACTIONS(1897),
    [sym_integer_literal] = ACTIONS(1895),
    [aux_sym_string_literal_token1] = ACTIONS(1895),
    [sym_char_literal] = ACTIONS(1895),
    [anon_sym_true] = ACTIONS(1897),
    [anon_sym_false] = ACTIONS(1897),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1897),
    [sym_super] = ACTIONS(1897),
    [sym_crate] = ACTIONS(1897),
    [sym_metavariable] = ACTIONS(1895),
    [sym__raw_string_literal_start] = ACTIONS(1895),
    [sym_float_literal] = ACTIONS(1895),
  },
  [537] = {
    [sym_line_comment] = STATE(537),
    [sym_block_comment] = STATE(537),
    [ts_builtin_sym_end] = ACTIONS(1899),
    [sym_identifier] = ACTIONS(1901),
    [anon_sym_SEMI] = ACTIONS(1899),
    [anon_sym_macro_rules_BANG] = ACTIONS(1899),
    [anon_sym_LPAREN] = ACTIONS(1899),
    [anon_sym_LBRACK] = ACTIONS(1899),
    [anon_sym_LBRACE] = ACTIONS(1899),
    [anon_sym_RBRACE] = ACTIONS(1899),
    [anon_sym_STAR] = ACTIONS(1899),
    [anon_sym_u8] = ACTIONS(1901),
    [anon_sym_i8] = ACTIONS(1901),
    [anon_sym_u16] = ACTIONS(1901),
    [anon_sym_i16] = ACTIONS(1901),
    [anon_sym_u32] = ACTIONS(1901),
    [anon_sym_i32] = ACTIONS(1901),
    [anon_sym_u64] = ACTIONS(1901),
    [anon_sym_i64] = ACTIONS(1901),
    [anon_sym_u128] = ACTIONS(1901),
    [anon_sym_i128] = ACTIONS(1901),
    [anon_sym_isize] = ACTIONS(1901),
    [anon_sym_usize] = ACTIONS(1901),
    [anon_sym_f32] = ACTIONS(1901),
    [anon_sym_f64] = ACTIONS(1901),
    [anon_sym_bool] = ACTIONS(1901),
    [anon_sym_str] = ACTIONS(1901),
    [anon_sym_char] = ACTIONS(1901),
    [anon_sym_DASH] = ACTIONS(1899),
    [anon_sym_BANG] = ACTIONS(1899),
    [anon_sym_AMP] = ACTIONS(1899),
    [anon_sym_PIPE] = ACTIONS(1899),
    [anon_sym_LT] = ACTIONS(1899),
    [anon_sym_DOT_DOT] = ACTIONS(1899),
    [anon_sym_COLON_COLON] = ACTIONS(1899),
    [anon_sym_POUND] = ACTIONS(1899),
    [anon_sym_SQUOTE] = ACTIONS(1901),
    [anon_sym_async] = ACTIONS(1901),
    [anon_sym_break] = ACTIONS(1901),
    [anon_sym_const] = ACTIONS(1901),
    [anon_sym_continue] = ACTIONS(1901),
    [anon_sym_default] = ACTIONS(1901),
    [anon_sym_enum] = ACTIONS(1901),
    [anon_sym_fn] = ACTIONS(1901),
    [anon_sym_for] = ACTIONS(1901),
    [anon_sym_gen] = ACTIONS(1901),
    [anon_sym_if] = ACTIONS(1901),
    [anon_sym_impl] = ACTIONS(1901),
    [anon_sym_let] = ACTIONS(1901),
    [anon_sym_loop] = ACTIONS(1901),
    [anon_sym_match] = ACTIONS(1901),
    [anon_sym_mod] = ACTIONS(1901),
    [anon_sym_pub] = ACTIONS(1901),
    [anon_sym_return] = ACTIONS(1901),
    [anon_sym_static] = ACTIONS(1901),
    [anon_sym_struct] = ACTIONS(1901),
    [anon_sym_trait] = ACTIONS(1901),
    [anon_sym_type] = ACTIONS(1901),
    [anon_sym_union] = ACTIONS(1901),
    [anon_sym_unsafe] = ACTIONS(1901),
    [anon_sym_use] = ACTIONS(1901),
    [anon_sym_while] = ACTIONS(1901),
    [anon_sym_extern] = ACTIONS(1901),
    [anon_sym_yield] = ACTIONS(1901),
    [anon_sym_move] = ACTIONS(1901),
    [anon_sym_try] = ACTIONS(1901),
    [sym_integer_literal] = ACTIONS(1899),
    [aux_sym_string_literal_token1] = ACTIONS(1899),
    [sym_char_literal] = ACTIONS(1899),
    [anon_sym_true] = ACTIONS(1901),
    [anon_sym_false] = ACTIONS(1901),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1901),
    [sym_super] = ACTIONS(1901),
    [sym_crate] = ACTIONS(1901),
    [sym_metavariable] = ACTIONS(1899),
    [sym__raw_string_literal_start] = ACTIONS(1899),
    [sym_float_literal] = ACTIONS(1899),
  },
  [538] = {
    [sym_line_comment] = STATE(538),
    [sym_block_comment] = STATE(538),
    [ts_builtin_sym_end] = ACTIONS(1903),
    [sym_identifier] = ACTIONS(1905),
    [anon_sym_SEMI] = ACTIONS(1903),
    [anon_sym_macro_rules_BANG] = ACTIONS(1903),
    [anon_sym_LPAREN] = ACTIONS(1903),
    [anon_sym_LBRACK] = ACTIONS(1903),
    [anon_sym_LBRACE] = ACTIONS(1903),
    [anon_sym_RBRACE] = ACTIONS(1903),
    [anon_sym_STAR] = ACTIONS(1903),
    [anon_sym_u8] = ACTIONS(1905),
    [anon_sym_i8] = ACTIONS(1905),
    [anon_sym_u16] = ACTIONS(1905),
    [anon_sym_i16] = ACTIONS(1905),
    [anon_sym_u32] = ACTIONS(1905),
    [anon_sym_i32] = ACTIONS(1905),
    [anon_sym_u64] = ACTIONS(1905),
    [anon_sym_i64] = ACTIONS(1905),
    [anon_sym_u128] = ACTIONS(1905),
    [anon_sym_i128] = ACTIONS(1905),
    [anon_sym_isize] = ACTIONS(1905),
    [anon_sym_usize] = ACTIONS(1905),
    [anon_sym_f32] = ACTIONS(1905),
    [anon_sym_f64] = ACTIONS(1905),
    [anon_sym_bool] = ACTIONS(1905),
    [anon_sym_str] = ACTIONS(1905),
    [anon_sym_char] = ACTIONS(1905),
    [anon_sym_DASH] = ACTIONS(1903),
    [anon_sym_BANG] = ACTIONS(1903),
    [anon_sym_AMP] = ACTIONS(1903),
    [anon_sym_PIPE] = ACTIONS(1903),
    [anon_sym_LT] = ACTIONS(1903),
    [anon_sym_DOT_DOT] = ACTIONS(1903),
    [anon_sym_COLON_COLON] = ACTIONS(1903),
    [anon_sym_POUND] = ACTIONS(1903),
    [anon_sym_SQUOTE] = ACTIONS(1905),
    [anon_sym_async] = ACTIONS(1905),
    [anon_sym_break] = ACTIONS(1905),
    [anon_sym_const] = ACTIONS(1905),
    [anon_sym_continue] = ACTIONS(1905),
    [anon_sym_default] = ACTIONS(1905),
    [anon_sym_enum] = ACTIONS(1905),
    [anon_sym_fn] = ACTIONS(1905),
    [anon_sym_for] = ACTIONS(1905),
    [anon_sym_gen] = ACTIONS(1905),
    [anon_sym_if] = ACTIONS(1905),
    [anon_sym_impl] = ACTIONS(1905),
    [anon_sym_let] = ACTIONS(1905),
    [anon_sym_loop] = ACTIONS(1905),
    [anon_sym_match] = ACTIONS(1905),
    [anon_sym_mod] = ACTIONS(1905),
    [anon_sym_pub] = ACTIONS(1905),
    [anon_sym_return] = ACTIONS(1905),
    [anon_sym_static] = ACTIONS(1905),
    [anon_sym_struct] = ACTIONS(1905),
    [anon_sym_trait] = ACTIONS(1905),
    [anon_sym_type] = ACTIONS(1905),
    [anon_sym_union] = ACTIONS(1905),
    [anon_sym_unsafe] = ACTIONS(1905),
    [anon_sym_use] = ACTIONS(1905),
    [anon_sym_while] = ACTIONS(1905),
    [anon_sym_extern] = ACTIONS(1905),
    [anon_sym_yield] = ACTIONS(1905),
    [anon_sym_move] = ACTIONS(1905),
    [anon_sym_try] = ACTIONS(1905),
    [sym_integer_literal] = ACTIONS(1903),
    [aux_sym_string_literal_token1] = ACTIONS(1903),
    [sym_char_literal] = ACTIONS(1903),
    [anon_sym_true] = ACTIONS(1905),
    [anon_sym_false] = ACTIONS(1905),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1905),
    [sym_super] = ACTIONS(1905),
    [sym_crate] = ACTIONS(1905),
    [sym_metavariable] = ACTIONS(1903),
    [sym__raw_string_literal_start] = ACTIONS(1903),
    [sym_float_literal] = ACTIONS(1903),
  },
  [539] = {
    [sym_line_comment] = STATE(539),
    [sym_block_comment] = STATE(539),
    [ts_builtin_sym_end] = ACTIONS(1907),
    [sym_identifier] = ACTIONS(1909),
    [anon_sym_SEMI] = ACTIONS(1907),
    [anon_sym_macro_rules_BANG] = ACTIONS(1907),
    [anon_sym_LPAREN] = ACTIONS(1907),
    [anon_sym_LBRACK] = ACTIONS(1907),
    [anon_sym_LBRACE] = ACTIONS(1907),
    [anon_sym_RBRACE] = ACTIONS(1907),
    [anon_sym_STAR] = ACTIONS(1907),
    [anon_sym_u8] = ACTIONS(1909),
    [anon_sym_i8] = ACTIONS(1909),
    [anon_sym_u16] = ACTIONS(1909),
    [anon_sym_i16] = ACTIONS(1909),
    [anon_sym_u32] = ACTIONS(1909),
    [anon_sym_i32] = ACTIONS(1909),
    [anon_sym_u64] = ACTIONS(1909),
    [anon_sym_i64] = ACTIONS(1909),
    [anon_sym_u128] = ACTIONS(1909),
    [anon_sym_i128] = ACTIONS(1909),
    [anon_sym_isize] = ACTIONS(1909),
    [anon_sym_usize] = ACTIONS(1909),
    [anon_sym_f32] = ACTIONS(1909),
    [anon_sym_f64] = ACTIONS(1909),
    [anon_sym_bool] = ACTIONS(1909),
    [anon_sym_str] = ACTIONS(1909),
    [anon_sym_char] = ACTIONS(1909),
    [anon_sym_DASH] = ACTIONS(1907),
    [anon_sym_BANG] = ACTIONS(1907),
    [anon_sym_AMP] = ACTIONS(1907),
    [anon_sym_PIPE] = ACTIONS(1907),
    [anon_sym_LT] = ACTIONS(1907),
    [anon_sym_DOT_DOT] = ACTIONS(1907),
    [anon_sym_COLON_COLON] = ACTIONS(1907),
    [anon_sym_POUND] = ACTIONS(1907),
    [anon_sym_SQUOTE] = ACTIONS(1909),
    [anon_sym_async] = ACTIONS(1909),
    [anon_sym_break] = ACTIONS(1909),
    [anon_sym_const] = ACTIONS(1909),
    [anon_sym_continue] = ACTIONS(1909),
    [anon_sym_default] = ACTIONS(1909),
    [anon_sym_enum] = ACTIONS(1909),
    [anon_sym_fn] = ACTIONS(1909),
    [anon_sym_for] = ACTIONS(1909),
    [anon_sym_gen] = ACTIONS(1909),
    [anon_sym_if] = ACTIONS(1909),
    [anon_sym_impl] = ACTIONS(1909),
    [anon_sym_let] = ACTIONS(1909),
    [anon_sym_loop] = ACTIONS(1909),
    [anon_sym_match] = ACTIONS(1909),
    [anon_sym_mod] = ACTIONS(1909),
    [anon_sym_pub] = ACTIONS(1909),
    [anon_sym_return] = ACTIONS(1909),
    [anon_sym_static] = ACTIONS(1909),
    [anon_sym_struct] = ACTIONS(1909),
    [anon_sym_trait] = ACTIONS(1909),
    [anon_sym_type] = ACTIONS(1909),
    [anon_sym_union] = ACTIONS(1909),
    [anon_sym_unsafe] = ACTIONS(1909),
    [anon_sym_use] = ACTIONS(1909),
    [anon_sym_while] = ACTIONS(1909),
    [anon_sym_extern] = ACTIONS(1909),
    [anon_sym_yield] = ACTIONS(1909),
    [anon_sym_move] = ACTIONS(1909),
    [anon_sym_try] = ACTIONS(1909),
    [sym_integer_literal] = ACTIONS(1907),
    [aux_sym_string_literal_token1] = ACTIONS(1907),
    [sym_char_literal] = ACTIONS(1907),
    [anon_sym_true] = ACTIONS(1909),
    [anon_sym_false] = ACTIONS(1909),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1909),
    [sym_super] = ACTIONS(1909),
    [sym_crate] = ACTIONS(1909),
    [sym_metavariable] = ACTIONS(1907),
    [sym__raw_string_literal_start] = ACTIONS(1907),
    [sym_float_literal] = ACTIONS(1907),
  },
  [540] = {
    [sym_line_comment] = STATE(540),
    [sym_block_comment] = STATE(540),
    [ts_builtin_sym_end] = ACTIONS(1911),
    [sym_identifier] = ACTIONS(1913),
    [anon_sym_SEMI] = ACTIONS(1911),
    [anon_sym_macro_rules_BANG] = ACTIONS(1911),
    [anon_sym_LPAREN] = ACTIONS(1911),
    [anon_sym_LBRACK] = ACTIONS(1911),
    [anon_sym_LBRACE] = ACTIONS(1911),
    [anon_sym_RBRACE] = ACTIONS(1911),
    [anon_sym_STAR] = ACTIONS(1911),
    [anon_sym_u8] = ACTIONS(1913),
    [anon_sym_i8] = ACTIONS(1913),
    [anon_sym_u16] = ACTIONS(1913),
    [anon_sym_i16] = ACTIONS(1913),
    [anon_sym_u32] = ACTIONS(1913),
    [anon_sym_i32] = ACTIONS(1913),
    [anon_sym_u64] = ACTIONS(1913),
    [anon_sym_i64] = ACTIONS(1913),
    [anon_sym_u128] = ACTIONS(1913),
    [anon_sym_i128] = ACTIONS(1913),
    [anon_sym_isize] = ACTIONS(1913),
    [anon_sym_usize] = ACTIONS(1913),
    [anon_sym_f32] = ACTIONS(1913),
    [anon_sym_f64] = ACTIONS(1913),
    [anon_sym_bool] = ACTIONS(1913),
    [anon_sym_str] = ACTIONS(1913),
    [anon_sym_char] = ACTIONS(1913),
    [anon_sym_DASH] = ACTIONS(1911),
    [anon_sym_BANG] = ACTIONS(1911),
    [anon_sym_AMP] = ACTIONS(1911),
    [anon_sym_PIPE] = ACTIONS(1911),
    [anon_sym_LT] = ACTIONS(1911),
    [anon_sym_DOT_DOT] = ACTIONS(1911),
    [anon_sym_COLON_COLON] = ACTIONS(1911),
    [anon_sym_POUND] = ACTIONS(1911),
    [anon_sym_SQUOTE] = ACTIONS(1913),
    [anon_sym_async] = ACTIONS(1913),
    [anon_sym_break] = ACTIONS(1913),
    [anon_sym_const] = ACTIONS(1913),
    [anon_sym_continue] = ACTIONS(1913),
    [anon_sym_default] = ACTIONS(1913),
    [anon_sym_enum] = ACTIONS(1913),
    [anon_sym_fn] = ACTIONS(1913),
    [anon_sym_for] = ACTIONS(1913),
    [anon_sym_gen] = ACTIONS(1913),
    [anon_sym_if] = ACTIONS(1913),
    [anon_sym_impl] = ACTIONS(1913),
    [anon_sym_let] = ACTIONS(1913),
    [anon_sym_loop] = ACTIONS(1913),
    [anon_sym_match] = ACTIONS(1913),
    [anon_sym_mod] = ACTIONS(1913),
    [anon_sym_pub] = ACTIONS(1913),
    [anon_sym_return] = ACTIONS(1913),
    [anon_sym_static] = ACTIONS(1913),
    [anon_sym_struct] = ACTIONS(1913),
    [anon_sym_trait] = ACTIONS(1913),
    [anon_sym_type] = ACTIONS(1913),
    [anon_sym_union] = ACTIONS(1913),
    [anon_sym_unsafe] = ACTIONS(1913),
    [anon_sym_use] = ACTIONS(1913),
    [anon_sym_while] = ACTIONS(1913),
    [anon_sym_extern] = ACTIONS(1913),
    [anon_sym_yield] = ACTIONS(1913),
    [anon_sym_move] = ACTIONS(1913),
    [anon_sym_try] = ACTIONS(1913),
    [sym_integer_literal] = ACTIONS(1911),
    [aux_sym_string_literal_token1] = ACTIONS(1911),
    [sym_char_literal] = ACTIONS(1911),
    [anon_sym_true] = ACTIONS(1913),
    [anon_sym_false] = ACTIONS(1913),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1913),
    [sym_super] = ACTIONS(1913),
    [sym_crate] = ACTIONS(1913),
    [sym_metavariable] = ACTIONS(1911),
    [sym__raw_string_literal_start] = ACTIONS(1911),
    [sym_float_literal] = ACTIONS(1911),
  },
  [541] = {
    [sym_line_comment] = STATE(541),
    [sym_block_comment] = STATE(541),
    [ts_builtin_sym_end] = ACTIONS(1915),
    [sym_identifier] = ACTIONS(1917),
    [anon_sym_SEMI] = ACTIONS(1915),
    [anon_sym_macro_rules_BANG] = ACTIONS(1915),
    [anon_sym_LPAREN] = ACTIONS(1915),
    [anon_sym_LBRACK] = ACTIONS(1915),
    [anon_sym_LBRACE] = ACTIONS(1915),
    [anon_sym_RBRACE] = ACTIONS(1915),
    [anon_sym_STAR] = ACTIONS(1915),
    [anon_sym_u8] = ACTIONS(1917),
    [anon_sym_i8] = ACTIONS(1917),
    [anon_sym_u16] = ACTIONS(1917),
    [anon_sym_i16] = ACTIONS(1917),
    [anon_sym_u32] = ACTIONS(1917),
    [anon_sym_i32] = ACTIONS(1917),
    [anon_sym_u64] = ACTIONS(1917),
    [anon_sym_i64] = ACTIONS(1917),
    [anon_sym_u128] = ACTIONS(1917),
    [anon_sym_i128] = ACTIONS(1917),
    [anon_sym_isize] = ACTIONS(1917),
    [anon_sym_usize] = ACTIONS(1917),
    [anon_sym_f32] = ACTIONS(1917),
    [anon_sym_f64] = ACTIONS(1917),
    [anon_sym_bool] = ACTIONS(1917),
    [anon_sym_str] = ACTIONS(1917),
    [anon_sym_char] = ACTIONS(1917),
    [anon_sym_DASH] = ACTIONS(1915),
    [anon_sym_BANG] = ACTIONS(1915),
    [anon_sym_AMP] = ACTIONS(1915),
    [anon_sym_PIPE] = ACTIONS(1915),
    [anon_sym_LT] = ACTIONS(1915),
    [anon_sym_DOT_DOT] = ACTIONS(1915),
    [anon_sym_COLON_COLON] = ACTIONS(1915),
    [anon_sym_POUND] = ACTIONS(1915),
    [anon_sym_SQUOTE] = ACTIONS(1917),
    [anon_sym_async] = ACTIONS(1917),
    [anon_sym_break] = ACTIONS(1917),
    [anon_sym_const] = ACTIONS(1917),
    [anon_sym_continue] = ACTIONS(1917),
    [anon_sym_default] = ACTIONS(1917),
    [anon_sym_enum] = ACTIONS(1917),
    [anon_sym_fn] = ACTIONS(1917),
    [anon_sym_for] = ACTIONS(1917),
    [anon_sym_gen] = ACTIONS(1917),
    [anon_sym_if] = ACTIONS(1917),
    [anon_sym_impl] = ACTIONS(1917),
    [anon_sym_let] = ACTIONS(1917),
    [anon_sym_loop] = ACTIONS(1917),
    [anon_sym_match] = ACTIONS(1917),
    [anon_sym_mod] = ACTIONS(1917),
    [anon_sym_pub] = ACTIONS(1917),
    [anon_sym_return] = ACTIONS(1917),
    [anon_sym_static] = ACTIONS(1917),
    [anon_sym_struct] = ACTIONS(1917),
    [anon_sym_trait] = ACTIONS(1917),
    [anon_sym_type] = ACTIONS(1917),
    [anon_sym_union] = ACTIONS(1917),
    [anon_sym_unsafe] = ACTIONS(1917),
    [anon_sym_use] = ACTIONS(1917),
    [anon_sym_while] = ACTIONS(1917),
    [anon_sym_extern] = ACTIONS(1917),
    [anon_sym_yield] = ACTIONS(1917),
    [anon_sym_move] = ACTIONS(1917),
    [anon_sym_try] = ACTIONS(1917),
    [sym_integer_literal] = ACTIONS(1915),
    [aux_sym_string_literal_token1] = ACTIONS(1915),
    [sym_char_literal] = ACTIONS(1915),
    [anon_sym_true] = ACTIONS(1917),
    [anon_sym_false] = ACTIONS(1917),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1917),
    [sym_super] = ACTIONS(1917),
    [sym_crate] = ACTIONS(1917),
    [sym_metavariable] = ACTIONS(1915),
    [sym__raw_string_literal_start] = ACTIONS(1915),
    [sym_float_literal] = ACTIONS(1915),
  },
  [542] = {
    [sym_line_comment] = STATE(542),
    [sym_block_comment] = STATE(542),
    [ts_builtin_sym_end] = ACTIONS(1919),
    [sym_identifier] = ACTIONS(1921),
    [anon_sym_SEMI] = ACTIONS(1919),
    [anon_sym_macro_rules_BANG] = ACTIONS(1919),
    [anon_sym_LPAREN] = ACTIONS(1919),
    [anon_sym_LBRACK] = ACTIONS(1919),
    [anon_sym_LBRACE] = ACTIONS(1919),
    [anon_sym_RBRACE] = ACTIONS(1919),
    [anon_sym_STAR] = ACTIONS(1919),
    [anon_sym_u8] = ACTIONS(1921),
    [anon_sym_i8] = ACTIONS(1921),
    [anon_sym_u16] = ACTIONS(1921),
    [anon_sym_i16] = ACTIONS(1921),
    [anon_sym_u32] = ACTIONS(1921),
    [anon_sym_i32] = ACTIONS(1921),
    [anon_sym_u64] = ACTIONS(1921),
    [anon_sym_i64] = ACTIONS(1921),
    [anon_sym_u128] = ACTIONS(1921),
    [anon_sym_i128] = ACTIONS(1921),
    [anon_sym_isize] = ACTIONS(1921),
    [anon_sym_usize] = ACTIONS(1921),
    [anon_sym_f32] = ACTIONS(1921),
    [anon_sym_f64] = ACTIONS(1921),
    [anon_sym_bool] = ACTIONS(1921),
    [anon_sym_str] = ACTIONS(1921),
    [anon_sym_char] = ACTIONS(1921),
    [anon_sym_DASH] = ACTIONS(1919),
    [anon_sym_BANG] = ACTIONS(1919),
    [anon_sym_AMP] = ACTIONS(1919),
    [anon_sym_PIPE] = ACTIONS(1919),
    [anon_sym_LT] = ACTIONS(1919),
    [anon_sym_DOT_DOT] = ACTIONS(1919),
    [anon_sym_COLON_COLON] = ACTIONS(1919),
    [anon_sym_POUND] = ACTIONS(1919),
    [anon_sym_SQUOTE] = ACTIONS(1921),
    [anon_sym_async] = ACTIONS(1921),
    [anon_sym_break] = ACTIONS(1921),
    [anon_sym_const] = ACTIONS(1921),
    [anon_sym_continue] = ACTIONS(1921),
    [anon_sym_default] = ACTIONS(1921),
    [anon_sym_enum] = ACTIONS(1921),
    [anon_sym_fn] = ACTIONS(1921),
    [anon_sym_for] = ACTIONS(1921),
    [anon_sym_gen] = ACTIONS(1921),
    [anon_sym_if] = ACTIONS(1921),
    [anon_sym_impl] = ACTIONS(1921),
    [anon_sym_let] = ACTIONS(1921),
    [anon_sym_loop] = ACTIONS(1921),
    [anon_sym_match] = ACTIONS(1921),
    [anon_sym_mod] = ACTIONS(1921),
    [anon_sym_pub] = ACTIONS(1921),
    [anon_sym_return] = ACTIONS(1921),
    [anon_sym_static] = ACTIONS(1921),
    [anon_sym_struct] = ACTIONS(1921),
    [anon_sym_trait] = ACTIONS(1921),
    [anon_sym_type] = ACTIONS(1921),
    [anon_sym_union] = ACTIONS(1921),
    [anon_sym_unsafe] = ACTIONS(1921),
    [anon_sym_use] = ACTIONS(1921),
    [anon_sym_while] = ACTIONS(1921),
    [anon_sym_extern] = ACTIONS(1921),
    [anon_sym_yield] = ACTIONS(1921),
    [anon_sym_move] = ACTIONS(1921),
    [anon_sym_try] = ACTIONS(1921),
    [sym_integer_literal] = ACTIONS(1919),
    [aux_sym_string_literal_token1] = ACTIONS(1919),
    [sym_char_literal] = ACTIONS(1919),
    [anon_sym_true] = ACTIONS(1921),
    [anon_sym_false] = ACTIONS(1921),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1921),
    [sym_super] = ACTIONS(1921),
    [sym_crate] = ACTIONS(1921),
    [sym_metavariable] = ACTIONS(1919),
    [sym__raw_string_literal_start] = ACTIONS(1919),
    [sym_float_literal] = ACTIONS(1919),
  },
  [543] = {
    [sym_line_comment] = STATE(543),
    [sym_block_comment] = STATE(543),
    [ts_builtin_sym_end] = ACTIONS(1923),
    [sym_identifier] = ACTIONS(1925),
    [anon_sym_SEMI] = ACTIONS(1923),
    [anon_sym_macro_rules_BANG] = ACTIONS(1923),
    [anon_sym_LPAREN] = ACTIONS(1923),
    [anon_sym_LBRACK] = ACTIONS(1923),
    [anon_sym_LBRACE] = ACTIONS(1923),
    [anon_sym_RBRACE] = ACTIONS(1923),
    [anon_sym_STAR] = ACTIONS(1923),
    [anon_sym_u8] = ACTIONS(1925),
    [anon_sym_i8] = ACTIONS(1925),
    [anon_sym_u16] = ACTIONS(1925),
    [anon_sym_i16] = ACTIONS(1925),
    [anon_sym_u32] = ACTIONS(1925),
    [anon_sym_i32] = ACTIONS(1925),
    [anon_sym_u64] = ACTIONS(1925),
    [anon_sym_i64] = ACTIONS(1925),
    [anon_sym_u128] = ACTIONS(1925),
    [anon_sym_i128] = ACTIONS(1925),
    [anon_sym_isize] = ACTIONS(1925),
    [anon_sym_usize] = ACTIONS(1925),
    [anon_sym_f32] = ACTIONS(1925),
    [anon_sym_f64] = ACTIONS(1925),
    [anon_sym_bool] = ACTIONS(1925),
    [anon_sym_str] = ACTIONS(1925),
    [anon_sym_char] = ACTIONS(1925),
    [anon_sym_DASH] = ACTIONS(1923),
    [anon_sym_BANG] = ACTIONS(1923),
    [anon_sym_AMP] = ACTIONS(1923),
    [anon_sym_PIPE] = ACTIONS(1923),
    [anon_sym_LT] = ACTIONS(1923),
    [anon_sym_DOT_DOT] = ACTIONS(1923),
    [anon_sym_COLON_COLON] = ACTIONS(1923),
    [anon_sym_POUND] = ACTIONS(1923),
    [anon_sym_SQUOTE] = ACTIONS(1925),
    [anon_sym_async] = ACTIONS(1925),
    [anon_sym_break] = ACTIONS(1925),
    [anon_sym_const] = ACTIONS(1925),
    [anon_sym_continue] = ACTIONS(1925),
    [anon_sym_default] = ACTIONS(1925),
    [anon_sym_enum] = ACTIONS(1925),
    [anon_sym_fn] = ACTIONS(1925),
    [anon_sym_for] = ACTIONS(1925),
    [anon_sym_gen] = ACTIONS(1925),
    [anon_sym_if] = ACTIONS(1925),
    [anon_sym_impl] = ACTIONS(1925),
    [anon_sym_let] = ACTIONS(1925),
    [anon_sym_loop] = ACTIONS(1925),
    [anon_sym_match] = ACTIONS(1925),
    [anon_sym_mod] = ACTIONS(1925),
    [anon_sym_pub] = ACTIONS(1925),
    [anon_sym_return] = ACTIONS(1925),
    [anon_sym_static] = ACTIONS(1925),
    [anon_sym_struct] = ACTIONS(1925),
    [anon_sym_trait] = ACTIONS(1925),
    [anon_sym_type] = ACTIONS(1925),
    [anon_sym_union] = ACTIONS(1925),
    [anon_sym_unsafe] = ACTIONS(1925),
    [anon_sym_use] = ACTIONS(1925),
    [anon_sym_while] = ACTIONS(1925),
    [anon_sym_extern] = ACTIONS(1925),
    [anon_sym_yield] = ACTIONS(1925),
    [anon_sym_move] = ACTIONS(1925),
    [anon_sym_try] = ACTIONS(1925),
    [sym_integer_literal] = ACTIONS(1923),
    [aux_sym_string_literal_token1] = ACTIONS(1923),
    [sym_char_literal] = ACTIONS(1923),
    [anon_sym_true] = ACTIONS(1925),
    [anon_sym_false] = ACTIONS(1925),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1925),
    [sym_super] = ACTIONS(1925),
    [sym_crate] = ACTIONS(1925),
    [sym_metavariable] = ACTIONS(1923),
    [sym__raw_string_literal_start] = ACTIONS(1923),
    [sym_float_literal] = ACTIONS(1923),
  },
  [544] = {
    [sym_line_comment] = STATE(544),
    [sym_block_comment] = STATE(544),
    [ts_builtin_sym_end] = ACTIONS(1927),
    [sym_identifier] = ACTIONS(1929),
    [anon_sym_SEMI] = ACTIONS(1927),
    [anon_sym_macro_rules_BANG] = ACTIONS(1927),
    [anon_sym_LPAREN] = ACTIONS(1927),
    [anon_sym_LBRACK] = ACTIONS(1927),
    [anon_sym_LBRACE] = ACTIONS(1927),
    [anon_sym_RBRACE] = ACTIONS(1927),
    [anon_sym_STAR] = ACTIONS(1927),
    [anon_sym_u8] = ACTIONS(1929),
    [anon_sym_i8] = ACTIONS(1929),
    [anon_sym_u16] = ACTIONS(1929),
    [anon_sym_i16] = ACTIONS(1929),
    [anon_sym_u32] = ACTIONS(1929),
    [anon_sym_i32] = ACTIONS(1929),
    [anon_sym_u64] = ACTIONS(1929),
    [anon_sym_i64] = ACTIONS(1929),
    [anon_sym_u128] = ACTIONS(1929),
    [anon_sym_i128] = ACTIONS(1929),
    [anon_sym_isize] = ACTIONS(1929),
    [anon_sym_usize] = ACTIONS(1929),
    [anon_sym_f32] = ACTIONS(1929),
    [anon_sym_f64] = ACTIONS(1929),
    [anon_sym_bool] = ACTIONS(1929),
    [anon_sym_str] = ACTIONS(1929),
    [anon_sym_char] = ACTIONS(1929),
    [anon_sym_DASH] = ACTIONS(1927),
    [anon_sym_BANG] = ACTIONS(1927),
    [anon_sym_AMP] = ACTIONS(1927),
    [anon_sym_PIPE] = ACTIONS(1927),
    [anon_sym_LT] = ACTIONS(1927),
    [anon_sym_DOT_DOT] = ACTIONS(1927),
    [anon_sym_COLON_COLON] = ACTIONS(1927),
    [anon_sym_POUND] = ACTIONS(1927),
    [anon_sym_SQUOTE] = ACTIONS(1929),
    [anon_sym_async] = ACTIONS(1929),
    [anon_sym_break] = ACTIONS(1929),
    [anon_sym_const] = ACTIONS(1929),
    [anon_sym_continue] = ACTIONS(1929),
    [anon_sym_default] = ACTIONS(1929),
    [anon_sym_enum] = ACTIONS(1929),
    [anon_sym_fn] = ACTIONS(1929),
    [anon_sym_for] = ACTIONS(1929),
    [anon_sym_gen] = ACTIONS(1929),
    [anon_sym_if] = ACTIONS(1929),
    [anon_sym_impl] = ACTIONS(1929),
    [anon_sym_let] = ACTIONS(1929),
    [anon_sym_loop] = ACTIONS(1929),
    [anon_sym_match] = ACTIONS(1929),
    [anon_sym_mod] = ACTIONS(1929),
    [anon_sym_pub] = ACTIONS(1929),
    [anon_sym_return] = ACTIONS(1929),
    [anon_sym_static] = ACTIONS(1929),
    [anon_sym_struct] = ACTIONS(1929),
    [anon_sym_trait] = ACTIONS(1929),
    [anon_sym_type] = ACTIONS(1929),
    [anon_sym_union] = ACTIONS(1929),
    [anon_sym_unsafe] = ACTIONS(1929),
    [anon_sym_use] = ACTIONS(1929),
    [anon_sym_while] = ACTIONS(1929),
    [anon_sym_extern] = ACTIONS(1929),
    [anon_sym_yield] = ACTIONS(1929),
    [anon_sym_move] = ACTIONS(1929),
    [anon_sym_try] = ACTIONS(1929),
    [sym_integer_literal] = ACTIONS(1927),
    [aux_sym_string_literal_token1] = ACTIONS(1927),
    [sym_char_literal] = ACTIONS(1927),
    [anon_sym_true] = ACTIONS(1929),
    [anon_sym_false] = ACTIONS(1929),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1929),
    [sym_super] = ACTIONS(1929),
    [sym_crate] = ACTIONS(1929),
    [sym_metavariable] = ACTIONS(1927),
    [sym__raw_string_literal_start] = ACTIONS(1927),
    [sym_float_literal] = ACTIONS(1927),
  },
  [545] = {
    [sym_line_comment] = STATE(545),
    [sym_block_comment] = STATE(545),
    [ts_builtin_sym_end] = ACTIONS(1931),
    [sym_identifier] = ACTIONS(1933),
    [anon_sym_SEMI] = ACTIONS(1931),
    [anon_sym_macro_rules_BANG] = ACTIONS(1931),
    [anon_sym_LPAREN] = ACTIONS(1931),
    [anon_sym_LBRACK] = ACTIONS(1931),
    [anon_sym_LBRACE] = ACTIONS(1931),
    [anon_sym_RBRACE] = ACTIONS(1931),
    [anon_sym_STAR] = ACTIONS(1931),
    [anon_sym_u8] = ACTIONS(1933),
    [anon_sym_i8] = ACTIONS(1933),
    [anon_sym_u16] = ACTIONS(1933),
    [anon_sym_i16] = ACTIONS(1933),
    [anon_sym_u32] = ACTIONS(1933),
    [anon_sym_i32] = ACTIONS(1933),
    [anon_sym_u64] = ACTIONS(1933),
    [anon_sym_i64] = ACTIONS(1933),
    [anon_sym_u128] = ACTIONS(1933),
    [anon_sym_i128] = ACTIONS(1933),
    [anon_sym_isize] = ACTIONS(1933),
    [anon_sym_usize] = ACTIONS(1933),
    [anon_sym_f32] = ACTIONS(1933),
    [anon_sym_f64] = ACTIONS(1933),
    [anon_sym_bool] = ACTIONS(1933),
    [anon_sym_str] = ACTIONS(1933),
    [anon_sym_char] = ACTIONS(1933),
    [anon_sym_DASH] = ACTIONS(1931),
    [anon_sym_BANG] = ACTIONS(1931),
    [anon_sym_AMP] = ACTIONS(1931),
    [anon_sym_PIPE] = ACTIONS(1931),
    [anon_sym_LT] = ACTIONS(1931),
    [anon_sym_DOT_DOT] = ACTIONS(1931),
    [anon_sym_COLON_COLON] = ACTIONS(1931),
    [anon_sym_POUND] = ACTIONS(1931),
    [anon_sym_SQUOTE] = ACTIONS(1933),
    [anon_sym_async] = ACTIONS(1933),
    [anon_sym_break] = ACTIONS(1933),
    [anon_sym_const] = ACTIONS(1933),
    [anon_sym_continue] = ACTIONS(1933),
    [anon_sym_default] = ACTIONS(1933),
    [anon_sym_enum] = ACTIONS(1933),
    [anon_sym_fn] = ACTIONS(1933),
    [anon_sym_for] = ACTIONS(1933),
    [anon_sym_gen] = ACTIONS(1933),
    [anon_sym_if] = ACTIONS(1933),
    [anon_sym_impl] = ACTIONS(1933),
    [anon_sym_let] = ACTIONS(1933),
    [anon_sym_loop] = ACTIONS(1933),
    [anon_sym_match] = ACTIONS(1933),
    [anon_sym_mod] = ACTIONS(1933),
    [anon_sym_pub] = ACTIONS(1933),
    [anon_sym_return] = ACTIONS(1933),
    [anon_sym_static] = ACTIONS(1933),
    [anon_sym_struct] = ACTIONS(1933),
    [anon_sym_trait] = ACTIONS(1933),
    [anon_sym_type] = ACTIONS(1933),
    [anon_sym_union] = ACTIONS(1933),
    [anon_sym_unsafe] = ACTIONS(1933),
    [anon_sym_use] = ACTIONS(1933),
    [anon_sym_while] = ACTIONS(1933),
    [anon_sym_extern] = ACTIONS(1933),
    [anon_sym_yield] = ACTIONS(1933),
    [anon_sym_move] = ACTIONS(1933),
    [anon_sym_try] = ACTIONS(1933),
    [sym_integer_literal] = ACTIONS(1931),
    [aux_sym_string_literal_token1] = ACTIONS(1931),
    [sym_char_literal] = ACTIONS(1931),
    [anon_sym_true] = ACTIONS(1933),
    [anon_sym_false] = ACTIONS(1933),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1933),
    [sym_super] = ACTIONS(1933),
    [sym_crate] = ACTIONS(1933),
    [sym_metavariable] = ACTIONS(1931),
    [sym__raw_string_literal_start] = ACTIONS(1931),
    [sym_float_literal] = ACTIONS(1931),
  },
  [546] = {
    [sym_line_comment] = STATE(546),
    [sym_block_comment] = STATE(546),
    [ts_builtin_sym_end] = ACTIONS(1935),
    [sym_identifier] = ACTIONS(1937),
    [anon_sym_SEMI] = ACTIONS(1935),
    [anon_sym_macro_rules_BANG] = ACTIONS(1935),
    [anon_sym_LPAREN] = ACTIONS(1935),
    [anon_sym_LBRACK] = ACTIONS(1935),
    [anon_sym_LBRACE] = ACTIONS(1935),
    [anon_sym_RBRACE] = ACTIONS(1935),
    [anon_sym_STAR] = ACTIONS(1935),
    [anon_sym_u8] = ACTIONS(1937),
    [anon_sym_i8] = ACTIONS(1937),
    [anon_sym_u16] = ACTIONS(1937),
    [anon_sym_i16] = ACTIONS(1937),
    [anon_sym_u32] = ACTIONS(1937),
    [anon_sym_i32] = ACTIONS(1937),
    [anon_sym_u64] = ACTIONS(1937),
    [anon_sym_i64] = ACTIONS(1937),
    [anon_sym_u128] = ACTIONS(1937),
    [anon_sym_i128] = ACTIONS(1937),
    [anon_sym_isize] = ACTIONS(1937),
    [anon_sym_usize] = ACTIONS(1937),
    [anon_sym_f32] = ACTIONS(1937),
    [anon_sym_f64] = ACTIONS(1937),
    [anon_sym_bool] = ACTIONS(1937),
    [anon_sym_str] = ACTIONS(1937),
    [anon_sym_char] = ACTIONS(1937),
    [anon_sym_DASH] = ACTIONS(1935),
    [anon_sym_BANG] = ACTIONS(1935),
    [anon_sym_AMP] = ACTIONS(1935),
    [anon_sym_PIPE] = ACTIONS(1935),
    [anon_sym_LT] = ACTIONS(1935),
    [anon_sym_DOT_DOT] = ACTIONS(1935),
    [anon_sym_COLON_COLON] = ACTIONS(1935),
    [anon_sym_POUND] = ACTIONS(1935),
    [anon_sym_SQUOTE] = ACTIONS(1937),
    [anon_sym_async] = ACTIONS(1937),
    [anon_sym_break] = ACTIONS(1937),
    [anon_sym_const] = ACTIONS(1937),
    [anon_sym_continue] = ACTIONS(1937),
    [anon_sym_default] = ACTIONS(1937),
    [anon_sym_enum] = ACTIONS(1937),
    [anon_sym_fn] = ACTIONS(1937),
    [anon_sym_for] = ACTIONS(1937),
    [anon_sym_gen] = ACTIONS(1937),
    [anon_sym_if] = ACTIONS(1937),
    [anon_sym_impl] = ACTIONS(1937),
    [anon_sym_let] = ACTIONS(1937),
    [anon_sym_loop] = ACTIONS(1937),
    [anon_sym_match] = ACTIONS(1937),
    [anon_sym_mod] = ACTIONS(1937),
    [anon_sym_pub] = ACTIONS(1937),
    [anon_sym_return] = ACTIONS(1937),
    [anon_sym_static] = ACTIONS(1937),
    [anon_sym_struct] = ACTIONS(1937),
    [anon_sym_trait] = ACTIONS(1937),
    [anon_sym_type] = ACTIONS(1937),
    [anon_sym_union] = ACTIONS(1937),
    [anon_sym_unsafe] = ACTIONS(1937),
    [anon_sym_use] = ACTIONS(1937),
    [anon_sym_while] = ACTIONS(1937),
    [anon_sym_extern] = ACTIONS(1937),
    [anon_sym_yield] = ACTIONS(1937),
    [anon_sym_move] = ACTIONS(1937),
    [anon_sym_try] = ACTIONS(1937),
    [sym_integer_literal] = ACTIONS(1935),
    [aux_sym_string_literal_token1] = ACTIONS(1935),
    [sym_char_literal] = ACTIONS(1935),
    [anon_sym_true] = ACTIONS(1937),
    [anon_sym_false] = ACTIONS(1937),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1937),
    [sym_super] = ACTIONS(1937),
    [sym_crate] = ACTIONS(1937),
    [sym_metavariable] = ACTIONS(1935),
    [sym__raw_string_literal_start] = ACTIONS(1935),
    [sym_float_literal] = ACTIONS(1935),
  },
  [547] = {
    [sym_line_comment] = STATE(547),
    [sym_block_comment] = STATE(547),
    [ts_builtin_sym_end] = ACTIONS(1939),
    [sym_identifier] = ACTIONS(1941),
    [anon_sym_SEMI] = ACTIONS(1939),
    [anon_sym_macro_rules_BANG] = ACTIONS(1939),
    [anon_sym_LPAREN] = ACTIONS(1939),
    [anon_sym_LBRACK] = ACTIONS(1939),
    [anon_sym_LBRACE] = ACTIONS(1939),
    [anon_sym_RBRACE] = ACTIONS(1939),
    [anon_sym_STAR] = ACTIONS(1939),
    [anon_sym_u8] = ACTIONS(1941),
    [anon_sym_i8] = ACTIONS(1941),
    [anon_sym_u16] = ACTIONS(1941),
    [anon_sym_i16] = ACTIONS(1941),
    [anon_sym_u32] = ACTIONS(1941),
    [anon_sym_i32] = ACTIONS(1941),
    [anon_sym_u64] = ACTIONS(1941),
    [anon_sym_i64] = ACTIONS(1941),
    [anon_sym_u128] = ACTIONS(1941),
    [anon_sym_i128] = ACTIONS(1941),
    [anon_sym_isize] = ACTIONS(1941),
    [anon_sym_usize] = ACTIONS(1941),
    [anon_sym_f32] = ACTIONS(1941),
    [anon_sym_f64] = ACTIONS(1941),
    [anon_sym_bool] = ACTIONS(1941),
    [anon_sym_str] = ACTIONS(1941),
    [anon_sym_char] = ACTIONS(1941),
    [anon_sym_DASH] = ACTIONS(1939),
    [anon_sym_BANG] = ACTIONS(1939),
    [anon_sym_AMP] = ACTIONS(1939),
    [anon_sym_PIPE] = ACTIONS(1939),
    [anon_sym_LT] = ACTIONS(1939),
    [anon_sym_DOT_DOT] = ACTIONS(1939),
    [anon_sym_COLON_COLON] = ACTIONS(1939),
    [anon_sym_POUND] = ACTIONS(1939),
    [anon_sym_SQUOTE] = ACTIONS(1941),
    [anon_sym_async] = ACTIONS(1941),
    [anon_sym_break] = ACTIONS(1941),
    [anon_sym_const] = ACTIONS(1941),
    [anon_sym_continue] = ACTIONS(1941),
    [anon_sym_default] = ACTIONS(1941),
    [anon_sym_enum] = ACTIONS(1941),
    [anon_sym_fn] = ACTIONS(1941),
    [anon_sym_for] = ACTIONS(1941),
    [anon_sym_gen] = ACTIONS(1941),
    [anon_sym_if] = ACTIONS(1941),
    [anon_sym_impl] = ACTIONS(1941),
    [anon_sym_let] = ACTIONS(1941),
    [anon_sym_loop] = ACTIONS(1941),
    [anon_sym_match] = ACTIONS(1941),
    [anon_sym_mod] = ACTIONS(1941),
    [anon_sym_pub] = ACTIONS(1941),
    [anon_sym_return] = ACTIONS(1941),
    [anon_sym_static] = ACTIONS(1941),
    [anon_sym_struct] = ACTIONS(1941),
    [anon_sym_trait] = ACTIONS(1941),
    [anon_sym_type] = ACTIONS(1941),
    [anon_sym_union] = ACTIONS(1941),
    [anon_sym_unsafe] = ACTIONS(1941),
    [anon_sym_use] = ACTIONS(1941),
    [anon_sym_while] = ACTIONS(1941),
    [anon_sym_extern] = ACTIONS(1941),
    [anon_sym_yield] = ACTIONS(1941),
    [anon_sym_move] = ACTIONS(1941),
    [anon_sym_try] = ACTIONS(1941),
    [sym_integer_literal] = ACTIONS(1939),
    [aux_sym_string_literal_token1] = ACTIONS(1939),
    [sym_char_literal] = ACTIONS(1939),
    [anon_sym_true] = ACTIONS(1941),
    [anon_sym_false] = ACTIONS(1941),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1941),
    [sym_super] = ACTIONS(1941),
    [sym_crate] = ACTIONS(1941),
    [sym_metavariable] = ACTIONS(1939),
    [sym__raw_string_literal_start] = ACTIONS(1939),
    [sym_float_literal] = ACTIONS(1939),
  },
  [548] = {
    [sym_line_comment] = STATE(548),
    [sym_block_comment] = STATE(548),
    [ts_builtin_sym_end] = ACTIONS(1943),
    [sym_identifier] = ACTIONS(1945),
    [anon_sym_SEMI] = ACTIONS(1943),
    [anon_sym_macro_rules_BANG] = ACTIONS(1943),
    [anon_sym_LPAREN] = ACTIONS(1943),
    [anon_sym_LBRACK] = ACTIONS(1943),
    [anon_sym_LBRACE] = ACTIONS(1943),
    [anon_sym_RBRACE] = ACTIONS(1943),
    [anon_sym_STAR] = ACTIONS(1943),
    [anon_sym_u8] = ACTIONS(1945),
    [anon_sym_i8] = ACTIONS(1945),
    [anon_sym_u16] = ACTIONS(1945),
    [anon_sym_i16] = ACTIONS(1945),
    [anon_sym_u32] = ACTIONS(1945),
    [anon_sym_i32] = ACTIONS(1945),
    [anon_sym_u64] = ACTIONS(1945),
    [anon_sym_i64] = ACTIONS(1945),
    [anon_sym_u128] = ACTIONS(1945),
    [anon_sym_i128] = ACTIONS(1945),
    [anon_sym_isize] = ACTIONS(1945),
    [anon_sym_usize] = ACTIONS(1945),
    [anon_sym_f32] = ACTIONS(1945),
    [anon_sym_f64] = ACTIONS(1945),
    [anon_sym_bool] = ACTIONS(1945),
    [anon_sym_str] = ACTIONS(1945),
    [anon_sym_char] = ACTIONS(1945),
    [anon_sym_DASH] = ACTIONS(1943),
    [anon_sym_BANG] = ACTIONS(1943),
    [anon_sym_AMP] = ACTIONS(1943),
    [anon_sym_PIPE] = ACTIONS(1943),
    [anon_sym_LT] = ACTIONS(1943),
    [anon_sym_DOT_DOT] = ACTIONS(1943),
    [anon_sym_COLON_COLON] = ACTIONS(1943),
    [anon_sym_POUND] = ACTIONS(1943),
    [anon_sym_SQUOTE] = ACTIONS(1945),
    [anon_sym_async] = ACTIONS(1945),
    [anon_sym_break] = ACTIONS(1945),
    [anon_sym_const] = ACTIONS(1945),
    [anon_sym_continue] = ACTIONS(1945),
    [anon_sym_default] = ACTIONS(1945),
    [anon_sym_enum] = ACTIONS(1945),
    [anon_sym_fn] = ACTIONS(1945),
    [anon_sym_for] = ACTIONS(1945),
    [anon_sym_gen] = ACTIONS(1945),
    [anon_sym_if] = ACTIONS(1945),
    [anon_sym_impl] = ACTIONS(1945),
    [anon_sym_let] = ACTIONS(1945),
    [anon_sym_loop] = ACTIONS(1945),
    [anon_sym_match] = ACTIONS(1945),
    [anon_sym_mod] = ACTIONS(1945),
    [anon_sym_pub] = ACTIONS(1945),
    [anon_sym_return] = ACTIONS(1945),
    [anon_sym_static] = ACTIONS(1945),
    [anon_sym_struct] = ACTIONS(1945),
    [anon_sym_trait] = ACTIONS(1945),
    [anon_sym_type] = ACTIONS(1945),
    [anon_sym_union] = ACTIONS(1945),
    [anon_sym_unsafe] = ACTIONS(1945),
    [anon_sym_use] = ACTIONS(1945),
    [anon_sym_while] = ACTIONS(1945),
    [anon_sym_extern] = ACTIONS(1945),
    [anon_sym_yield] = ACTIONS(1945),
    [anon_sym_move] = ACTIONS(1945),
    [anon_sym_try] = ACTIONS(1945),
    [sym_integer_literal] = ACTIONS(1943),
    [aux_sym_string_literal_token1] = ACTIONS(1943),
    [sym_char_literal] = ACTIONS(1943),
    [anon_sym_true] = ACTIONS(1945),
    [anon_sym_false] = ACTIONS(1945),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1945),
    [sym_super] = ACTIONS(1945),
    [sym_crate] = ACTIONS(1945),
    [sym_metavariable] = ACTIONS(1943),
    [sym__raw_string_literal_start] = ACTIONS(1943),
    [sym_float_literal] = ACTIONS(1943),
  },
  [549] = {
    [sym_line_comment] = STATE(549),
    [sym_block_comment] = STATE(549),
    [ts_builtin_sym_end] = ACTIONS(1947),
    [sym_identifier] = ACTIONS(1949),
    [anon_sym_SEMI] = ACTIONS(1947),
    [anon_sym_macro_rules_BANG] = ACTIONS(1947),
    [anon_sym_LPAREN] = ACTIONS(1947),
    [anon_sym_LBRACK] = ACTIONS(1947),
    [anon_sym_LBRACE] = ACTIONS(1947),
    [anon_sym_RBRACE] = ACTIONS(1947),
    [anon_sym_STAR] = ACTIONS(1947),
    [anon_sym_u8] = ACTIONS(1949),
    [anon_sym_i8] = ACTIONS(1949),
    [anon_sym_u16] = ACTIONS(1949),
    [anon_sym_i16] = ACTIONS(1949),
    [anon_sym_u32] = ACTIONS(1949),
    [anon_sym_i32] = ACTIONS(1949),
    [anon_sym_u64] = ACTIONS(1949),
    [anon_sym_i64] = ACTIONS(1949),
    [anon_sym_u128] = ACTIONS(1949),
    [anon_sym_i128] = ACTIONS(1949),
    [anon_sym_isize] = ACTIONS(1949),
    [anon_sym_usize] = ACTIONS(1949),
    [anon_sym_f32] = ACTIONS(1949),
    [anon_sym_f64] = ACTIONS(1949),
    [anon_sym_bool] = ACTIONS(1949),
    [anon_sym_str] = ACTIONS(1949),
    [anon_sym_char] = ACTIONS(1949),
    [anon_sym_DASH] = ACTIONS(1947),
    [anon_sym_BANG] = ACTIONS(1947),
    [anon_sym_AMP] = ACTIONS(1947),
    [anon_sym_PIPE] = ACTIONS(1947),
    [anon_sym_LT] = ACTIONS(1947),
    [anon_sym_DOT_DOT] = ACTIONS(1947),
    [anon_sym_COLON_COLON] = ACTIONS(1947),
    [anon_sym_POUND] = ACTIONS(1947),
    [anon_sym_SQUOTE] = ACTIONS(1949),
    [anon_sym_async] = ACTIONS(1949),
    [anon_sym_break] = ACTIONS(1949),
    [anon_sym_const] = ACTIONS(1949),
    [anon_sym_continue] = ACTIONS(1949),
    [anon_sym_default] = ACTIONS(1949),
    [anon_sym_enum] = ACTIONS(1949),
    [anon_sym_fn] = ACTIONS(1949),
    [anon_sym_for] = ACTIONS(1949),
    [anon_sym_gen] = ACTIONS(1949),
    [anon_sym_if] = ACTIONS(1949),
    [anon_sym_impl] = ACTIONS(1949),
    [anon_sym_let] = ACTIONS(1949),
    [anon_sym_loop] = ACTIONS(1949),
    [anon_sym_match] = ACTIONS(1949),
    [anon_sym_mod] = ACTIONS(1949),
    [anon_sym_pub] = ACTIONS(1949),
    [anon_sym_return] = ACTIONS(1949),
    [anon_sym_static] = ACTIONS(1949),
    [anon_sym_struct] = ACTIONS(1949),
    [anon_sym_trait] = ACTIONS(1949),
    [anon_sym_type] = ACTIONS(1949),
    [anon_sym_union] = ACTIONS(1949),
    [anon_sym_unsafe] = ACTIONS(1949),
    [anon_sym_use] = ACTIONS(1949),
    [anon_sym_while] = ACTIONS(1949),
    [anon_sym_extern] = ACTIONS(1949),
    [anon_sym_yield] = ACTIONS(1949),
    [anon_sym_move] = ACTIONS(1949),
    [anon_sym_try] = ACTIONS(1949),
    [sym_integer_literal] = ACTIONS(1947),
    [aux_sym_string_literal_token1] = ACTIONS(1947),
    [sym_char_literal] = ACTIONS(1947),
    [anon_sym_true] = ACTIONS(1949),
    [anon_sym_false] = ACTIONS(1949),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1949),
    [sym_super] = ACTIONS(1949),
    [sym_crate] = ACTIONS(1949),
    [sym_metavariable] = ACTIONS(1947),
    [sym__raw_string_literal_start] = ACTIONS(1947),
    [sym_float_literal] = ACTIONS(1947),
  },
  [550] = {
    [sym_line_comment] = STATE(550),
    [sym_block_comment] = STATE(550),
    [ts_builtin_sym_end] = ACTIONS(1951),
    [sym_identifier] = ACTIONS(1953),
    [anon_sym_SEMI] = ACTIONS(1951),
    [anon_sym_macro_rules_BANG] = ACTIONS(1951),
    [anon_sym_LPAREN] = ACTIONS(1951),
    [anon_sym_LBRACK] = ACTIONS(1951),
    [anon_sym_LBRACE] = ACTIONS(1951),
    [anon_sym_RBRACE] = ACTIONS(1951),
    [anon_sym_STAR] = ACTIONS(1951),
    [anon_sym_u8] = ACTIONS(1953),
    [anon_sym_i8] = ACTIONS(1953),
    [anon_sym_u16] = ACTIONS(1953),
    [anon_sym_i16] = ACTIONS(1953),
    [anon_sym_u32] = ACTIONS(1953),
    [anon_sym_i32] = ACTIONS(1953),
    [anon_sym_u64] = ACTIONS(1953),
    [anon_sym_i64] = ACTIONS(1953),
    [anon_sym_u128] = ACTIONS(1953),
    [anon_sym_i128] = ACTIONS(1953),
    [anon_sym_isize] = ACTIONS(1953),
    [anon_sym_usize] = ACTIONS(1953),
    [anon_sym_f32] = ACTIONS(1953),
    [anon_sym_f64] = ACTIONS(1953),
    [anon_sym_bool] = ACTIONS(1953),
    [anon_sym_str] = ACTIONS(1953),
    [anon_sym_char] = ACTIONS(1953),
    [anon_sym_DASH] = ACTIONS(1951),
    [anon_sym_BANG] = ACTIONS(1951),
    [anon_sym_AMP] = ACTIONS(1951),
    [anon_sym_PIPE] = ACTIONS(1951),
    [anon_sym_LT] = ACTIONS(1951),
    [anon_sym_DOT_DOT] = ACTIONS(1951),
    [anon_sym_COLON_COLON] = ACTIONS(1951),
    [anon_sym_POUND] = ACTIONS(1951),
    [anon_sym_SQUOTE] = ACTIONS(1953),
    [anon_sym_async] = ACTIONS(1953),
    [anon_sym_break] = ACTIONS(1953),
    [anon_sym_const] = ACTIONS(1953),
    [anon_sym_continue] = ACTIONS(1953),
    [anon_sym_default] = ACTIONS(1953),
    [anon_sym_enum] = ACTIONS(1953),
    [anon_sym_fn] = ACTIONS(1953),
    [anon_sym_for] = ACTIONS(1953),
    [anon_sym_gen] = ACTIONS(1953),
    [anon_sym_if] = ACTIONS(1953),
    [anon_sym_impl] = ACTIONS(1953),
    [anon_sym_let] = ACTIONS(1953),
    [anon_sym_loop] = ACTIONS(1953),
    [anon_sym_match] = ACTIONS(1953),
    [anon_sym_mod] = ACTIONS(1953),
    [anon_sym_pub] = ACTIONS(1953),
    [anon_sym_return] = ACTIONS(1953),
    [anon_sym_static] = ACTIONS(1953),
    [anon_sym_struct] = ACTIONS(1953),
    [anon_sym_trait] = ACTIONS(1953),
    [anon_sym_type] = ACTIONS(1953),
    [anon_sym_union] = ACTIONS(1953),
    [anon_sym_unsafe] = ACTIONS(1953),
    [anon_sym_use] = ACTIONS(1953),
    [anon_sym_while] = ACTIONS(1953),
    [anon_sym_extern] = ACTIONS(1953),
    [anon_sym_yield] = ACTIONS(1953),
    [anon_sym_move] = ACTIONS(1953),
    [anon_sym_try] = ACTIONS(1953),
    [sym_integer_literal] = ACTIONS(1951),
    [aux_sym_string_literal_token1] = ACTIONS(1951),
    [sym_char_literal] = ACTIONS(1951),
    [anon_sym_true] = ACTIONS(1953),
    [anon_sym_false] = ACTIONS(1953),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1953),
    [sym_super] = ACTIONS(1953),
    [sym_crate] = ACTIONS(1953),
    [sym_metavariable] = ACTIONS(1951),
    [sym__raw_string_literal_start] = ACTIONS(1951),
    [sym_float_literal] = ACTIONS(1951),
  },
  [551] = {
    [sym_line_comment] = STATE(551),
    [sym_block_comment] = STATE(551),
    [ts_builtin_sym_end] = ACTIONS(1955),
    [sym_identifier] = ACTIONS(1957),
    [anon_sym_SEMI] = ACTIONS(1955),
    [anon_sym_macro_rules_BANG] = ACTIONS(1955),
    [anon_sym_LPAREN] = ACTIONS(1955),
    [anon_sym_LBRACK] = ACTIONS(1955),
    [anon_sym_LBRACE] = ACTIONS(1955),
    [anon_sym_RBRACE] = ACTIONS(1955),
    [anon_sym_STAR] = ACTIONS(1955),
    [anon_sym_u8] = ACTIONS(1957),
    [anon_sym_i8] = ACTIONS(1957),
    [anon_sym_u16] = ACTIONS(1957),
    [anon_sym_i16] = ACTIONS(1957),
    [anon_sym_u32] = ACTIONS(1957),
    [anon_sym_i32] = ACTIONS(1957),
    [anon_sym_u64] = ACTIONS(1957),
    [anon_sym_i64] = ACTIONS(1957),
    [anon_sym_u128] = ACTIONS(1957),
    [anon_sym_i128] = ACTIONS(1957),
    [anon_sym_isize] = ACTIONS(1957),
    [anon_sym_usize] = ACTIONS(1957),
    [anon_sym_f32] = ACTIONS(1957),
    [anon_sym_f64] = ACTIONS(1957),
    [anon_sym_bool] = ACTIONS(1957),
    [anon_sym_str] = ACTIONS(1957),
    [anon_sym_char] = ACTIONS(1957),
    [anon_sym_DASH] = ACTIONS(1955),
    [anon_sym_BANG] = ACTIONS(1955),
    [anon_sym_AMP] = ACTIONS(1955),
    [anon_sym_PIPE] = ACTIONS(1955),
    [anon_sym_LT] = ACTIONS(1955),
    [anon_sym_DOT_DOT] = ACTIONS(1955),
    [anon_sym_COLON_COLON] = ACTIONS(1955),
    [anon_sym_POUND] = ACTIONS(1955),
    [anon_sym_SQUOTE] = ACTIONS(1957),
    [anon_sym_async] = ACTIONS(1957),
    [anon_sym_break] = ACTIONS(1957),
    [anon_sym_const] = ACTIONS(1957),
    [anon_sym_continue] = ACTIONS(1957),
    [anon_sym_default] = ACTIONS(1957),
    [anon_sym_enum] = ACTIONS(1957),
    [anon_sym_fn] = ACTIONS(1957),
    [anon_sym_for] = ACTIONS(1957),
    [anon_sym_gen] = ACTIONS(1957),
    [anon_sym_if] = ACTIONS(1957),
    [anon_sym_impl] = ACTIONS(1957),
    [anon_sym_let] = ACTIONS(1957),
    [anon_sym_loop] = ACTIONS(1957),
    [anon_sym_match] = ACTIONS(1957),
    [anon_sym_mod] = ACTIONS(1957),
    [anon_sym_pub] = ACTIONS(1957),
    [anon_sym_return] = ACTIONS(1957),
    [anon_sym_static] = ACTIONS(1957),
    [anon_sym_struct] = ACTIONS(1957),
    [anon_sym_trait] = ACTIONS(1957),
    [anon_sym_type] = ACTIONS(1957),
    [anon_sym_union] = ACTIONS(1957),
    [anon_sym_unsafe] = ACTIONS(1957),
    [anon_sym_use] = ACTIONS(1957),
    [anon_sym_while] = ACTIONS(1957),
    [anon_sym_extern] = ACTIONS(1957),
    [anon_sym_yield] = ACTIONS(1957),
    [anon_sym_move] = ACTIONS(1957),
    [anon_sym_try] = ACTIONS(1957),
    [sym_integer_literal] = ACTIONS(1955),
    [aux_sym_string_literal_token1] = ACTIONS(1955),
    [sym_char_literal] = ACTIONS(1955),
    [anon_sym_true] = ACTIONS(1957),
    [anon_sym_false] = ACTIONS(1957),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1957),
    [sym_super] = ACTIONS(1957),
    [sym_crate] = ACTIONS(1957),
    [sym_metavariable] = ACTIONS(1955),
    [sym__raw_string_literal_start] = ACTIONS(1955),
    [sym_float_literal] = ACTIONS(1955),
  },
  [552] = {
    [sym_line_comment] = STATE(552),
    [sym_block_comment] = STATE(552),
    [ts_builtin_sym_end] = ACTIONS(1959),
    [sym_identifier] = ACTIONS(1961),
    [anon_sym_SEMI] = ACTIONS(1959),
    [anon_sym_macro_rules_BANG] = ACTIONS(1959),
    [anon_sym_LPAREN] = ACTIONS(1959),
    [anon_sym_LBRACK] = ACTIONS(1959),
    [anon_sym_LBRACE] = ACTIONS(1959),
    [anon_sym_RBRACE] = ACTIONS(1959),
    [anon_sym_STAR] = ACTIONS(1959),
    [anon_sym_u8] = ACTIONS(1961),
    [anon_sym_i8] = ACTIONS(1961),
    [anon_sym_u16] = ACTIONS(1961),
    [anon_sym_i16] = ACTIONS(1961),
    [anon_sym_u32] = ACTIONS(1961),
    [anon_sym_i32] = ACTIONS(1961),
    [anon_sym_u64] = ACTIONS(1961),
    [anon_sym_i64] = ACTIONS(1961),
    [anon_sym_u128] = ACTIONS(1961),
    [anon_sym_i128] = ACTIONS(1961),
    [anon_sym_isize] = ACTIONS(1961),
    [anon_sym_usize] = ACTIONS(1961),
    [anon_sym_f32] = ACTIONS(1961),
    [anon_sym_f64] = ACTIONS(1961),
    [anon_sym_bool] = ACTIONS(1961),
    [anon_sym_str] = ACTIONS(1961),
    [anon_sym_char] = ACTIONS(1961),
    [anon_sym_DASH] = ACTIONS(1959),
    [anon_sym_BANG] = ACTIONS(1959),
    [anon_sym_AMP] = ACTIONS(1959),
    [anon_sym_PIPE] = ACTIONS(1959),
    [anon_sym_LT] = ACTIONS(1959),
    [anon_sym_DOT_DOT] = ACTIONS(1959),
    [anon_sym_COLON_COLON] = ACTIONS(1959),
    [anon_sym_POUND] = ACTIONS(1959),
    [anon_sym_SQUOTE] = ACTIONS(1961),
    [anon_sym_async] = ACTIONS(1961),
    [anon_sym_break] = ACTIONS(1961),
    [anon_sym_const] = ACTIONS(1961),
    [anon_sym_continue] = ACTIONS(1961),
    [anon_sym_default] = ACTIONS(1961),
    [anon_sym_enum] = ACTIONS(1961),
    [anon_sym_fn] = ACTIONS(1961),
    [anon_sym_for] = ACTIONS(1961),
    [anon_sym_gen] = ACTIONS(1961),
    [anon_sym_if] = ACTIONS(1961),
    [anon_sym_impl] = ACTIONS(1961),
    [anon_sym_let] = ACTIONS(1961),
    [anon_sym_loop] = ACTIONS(1961),
    [anon_sym_match] = ACTIONS(1961),
    [anon_sym_mod] = ACTIONS(1961),
    [anon_sym_pub] = ACTIONS(1961),
    [anon_sym_return] = ACTIONS(1961),
    [anon_sym_static] = ACTIONS(1961),
    [anon_sym_struct] = ACTIONS(1961),
    [anon_sym_trait] = ACTIONS(1961),
    [anon_sym_type] = ACTIONS(1961),
    [anon_sym_union] = ACTIONS(1961),
    [anon_sym_unsafe] = ACTIONS(1961),
    [anon_sym_use] = ACTIONS(1961),
    [anon_sym_while] = ACTIONS(1961),
    [anon_sym_extern] = ACTIONS(1961),
    [anon_sym_yield] = ACTIONS(1961),
    [anon_sym_move] = ACTIONS(1961),
    [anon_sym_try] = ACTIONS(1961),
    [sym_integer_literal] = ACTIONS(1959),
    [aux_sym_string_literal_token1] = ACTIONS(1959),
    [sym_char_literal] = ACTIONS(1959),
    [anon_sym_true] = ACTIONS(1961),
    [anon_sym_false] = ACTIONS(1961),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1961),
    [sym_super] = ACTIONS(1961),
    [sym_crate] = ACTIONS(1961),
    [sym_metavariable] = ACTIONS(1959),
    [sym__raw_string_literal_start] = ACTIONS(1959),
    [sym_float_literal] = ACTIONS(1959),
  },
  [553] = {
    [sym_line_comment] = STATE(553),
    [sym_block_comment] = STATE(553),
    [ts_builtin_sym_end] = ACTIONS(1963),
    [sym_identifier] = ACTIONS(1965),
    [anon_sym_SEMI] = ACTIONS(1963),
    [anon_sym_macro_rules_BANG] = ACTIONS(1963),
    [anon_sym_LPAREN] = ACTIONS(1963),
    [anon_sym_LBRACK] = ACTIONS(1963),
    [anon_sym_LBRACE] = ACTIONS(1963),
    [anon_sym_RBRACE] = ACTIONS(1963),
    [anon_sym_STAR] = ACTIONS(1963),
    [anon_sym_u8] = ACTIONS(1965),
    [anon_sym_i8] = ACTIONS(1965),
    [anon_sym_u16] = ACTIONS(1965),
    [anon_sym_i16] = ACTIONS(1965),
    [anon_sym_u32] = ACTIONS(1965),
    [anon_sym_i32] = ACTIONS(1965),
    [anon_sym_u64] = ACTIONS(1965),
    [anon_sym_i64] = ACTIONS(1965),
    [anon_sym_u128] = ACTIONS(1965),
    [anon_sym_i128] = ACTIONS(1965),
    [anon_sym_isize] = ACTIONS(1965),
    [anon_sym_usize] = ACTIONS(1965),
    [anon_sym_f32] = ACTIONS(1965),
    [anon_sym_f64] = ACTIONS(1965),
    [anon_sym_bool] = ACTIONS(1965),
    [anon_sym_str] = ACTIONS(1965),
    [anon_sym_char] = ACTIONS(1965),
    [anon_sym_DASH] = ACTIONS(1963),
    [anon_sym_BANG] = ACTIONS(1963),
    [anon_sym_AMP] = ACTIONS(1963),
    [anon_sym_PIPE] = ACTIONS(1963),
    [anon_sym_LT] = ACTIONS(1963),
    [anon_sym_DOT_DOT] = ACTIONS(1963),
    [anon_sym_COLON_COLON] = ACTIONS(1963),
    [anon_sym_POUND] = ACTIONS(1963),
    [anon_sym_SQUOTE] = ACTIONS(1965),
    [anon_sym_async] = ACTIONS(1965),
    [anon_sym_break] = ACTIONS(1965),
    [anon_sym_const] = ACTIONS(1965),
    [anon_sym_continue] = ACTIONS(1965),
    [anon_sym_default] = ACTIONS(1965),
    [anon_sym_enum] = ACTIONS(1965),
    [anon_sym_fn] = ACTIONS(1965),
    [anon_sym_for] = ACTIONS(1965),
    [anon_sym_gen] = ACTIONS(1965),
    [anon_sym_if] = ACTIONS(1965),
    [anon_sym_impl] = ACTIONS(1965),
    [anon_sym_let] = ACTIONS(1965),
    [anon_sym_loop] = ACTIONS(1965),
    [anon_sym_match] = ACTIONS(1965),
    [anon_sym_mod] = ACTIONS(1965),
    [anon_sym_pub] = ACTIONS(1965),
    [anon_sym_return] = ACTIONS(1965),
    [anon_sym_static] = ACTIONS(1965),
    [anon_sym_struct] = ACTIONS(1965),
    [anon_sym_trait] = ACTIONS(1965),
    [anon_sym_type] = ACTIONS(1965),
    [anon_sym_union] = ACTIONS(1965),
    [anon_sym_unsafe] = ACTIONS(1965),
    [anon_sym_use] = ACTIONS(1965),
    [anon_sym_while] = ACTIONS(1965),
    [anon_sym_extern] = ACTIONS(1965),
    [anon_sym_yield] = ACTIONS(1965),
    [anon_sym_move] = ACTIONS(1965),
    [anon_sym_try] = ACTIONS(1965),
    [sym_integer_literal] = ACTIONS(1963),
    [aux_sym_string_literal_token1] = ACTIONS(1963),
    [sym_char_literal] = ACTIONS(1963),
    [anon_sym_true] = ACTIONS(1965),
    [anon_sym_false] = ACTIONS(1965),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1965),
    [sym_super] = ACTIONS(1965),
    [sym_crate] = ACTIONS(1965),
    [sym_metavariable] = ACTIONS(1963),
    [sym__raw_string_literal_start] = ACTIONS(1963),
    [sym_float_literal] = ACTIONS(1963),
  },
  [554] = {
    [sym_line_comment] = STATE(554),
    [sym_block_comment] = STATE(554),
    [ts_builtin_sym_end] = ACTIONS(1967),
    [sym_identifier] = ACTIONS(1969),
    [anon_sym_SEMI] = ACTIONS(1967),
    [anon_sym_macro_rules_BANG] = ACTIONS(1967),
    [anon_sym_LPAREN] = ACTIONS(1967),
    [anon_sym_LBRACK] = ACTIONS(1967),
    [anon_sym_LBRACE] = ACTIONS(1967),
    [anon_sym_RBRACE] = ACTIONS(1967),
    [anon_sym_STAR] = ACTIONS(1967),
    [anon_sym_u8] = ACTIONS(1969),
    [anon_sym_i8] = ACTIONS(1969),
    [anon_sym_u16] = ACTIONS(1969),
    [anon_sym_i16] = ACTIONS(1969),
    [anon_sym_u32] = ACTIONS(1969),
    [anon_sym_i32] = ACTIONS(1969),
    [anon_sym_u64] = ACTIONS(1969),
    [anon_sym_i64] = ACTIONS(1969),
    [anon_sym_u128] = ACTIONS(1969),
    [anon_sym_i128] = ACTIONS(1969),
    [anon_sym_isize] = ACTIONS(1969),
    [anon_sym_usize] = ACTIONS(1969),
    [anon_sym_f32] = ACTIONS(1969),
    [anon_sym_f64] = ACTIONS(1969),
    [anon_sym_bool] = ACTIONS(1969),
    [anon_sym_str] = ACTIONS(1969),
    [anon_sym_char] = ACTIONS(1969),
    [anon_sym_DASH] = ACTIONS(1967),
    [anon_sym_BANG] = ACTIONS(1967),
    [anon_sym_AMP] = ACTIONS(1967),
    [anon_sym_PIPE] = ACTIONS(1967),
    [anon_sym_LT] = ACTIONS(1967),
    [anon_sym_DOT_DOT] = ACTIONS(1967),
    [anon_sym_COLON_COLON] = ACTIONS(1967),
    [anon_sym_POUND] = ACTIONS(1967),
    [anon_sym_SQUOTE] = ACTIONS(1969),
    [anon_sym_async] = ACTIONS(1969),
    [anon_sym_break] = ACTIONS(1969),
    [anon_sym_const] = ACTIONS(1969),
    [anon_sym_continue] = ACTIONS(1969),
    [anon_sym_default] = ACTIONS(1969),
    [anon_sym_enum] = ACTIONS(1969),
    [anon_sym_fn] = ACTIONS(1969),
    [anon_sym_for] = ACTIONS(1969),
    [anon_sym_gen] = ACTIONS(1969),
    [anon_sym_if] = ACTIONS(1969),
    [anon_sym_impl] = ACTIONS(1969),
    [anon_sym_let] = ACTIONS(1969),
    [anon_sym_loop] = ACTIONS(1969),
    [anon_sym_match] = ACTIONS(1969),
    [anon_sym_mod] = ACTIONS(1969),
    [anon_sym_pub] = ACTIONS(1969),
    [anon_sym_return] = ACTIONS(1969),
    [anon_sym_static] = ACTIONS(1969),
    [anon_sym_struct] = ACTIONS(1969),
    [anon_sym_trait] = ACTIONS(1969),
    [anon_sym_type] = ACTIONS(1969),
    [anon_sym_union] = ACTIONS(1969),
    [anon_sym_unsafe] = ACTIONS(1969),
    [anon_sym_use] = ACTIONS(1969),
    [anon_sym_while] = ACTIONS(1969),
    [anon_sym_extern] = ACTIONS(1969),
    [anon_sym_yield] = ACTIONS(1969),
    [anon_sym_move] = ACTIONS(1969),
    [anon_sym_try] = ACTIONS(1969),
    [sym_integer_literal] = ACTIONS(1967),
    [aux_sym_string_literal_token1] = ACTIONS(1967),
    [sym_char_literal] = ACTIONS(1967),
    [anon_sym_true] = ACTIONS(1969),
    [anon_sym_false] = ACTIONS(1969),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1969),
    [sym_super] = ACTIONS(1969),
    [sym_crate] = ACTIONS(1969),
    [sym_metavariable] = ACTIONS(1967),
    [sym__raw_string_literal_start] = ACTIONS(1967),
    [sym_float_literal] = ACTIONS(1967),
  },
  [555] = {
    [sym_line_comment] = STATE(555),
    [sym_block_comment] = STATE(555),
    [ts_builtin_sym_end] = ACTIONS(1971),
    [sym_identifier] = ACTIONS(1973),
    [anon_sym_SEMI] = ACTIONS(1971),
    [anon_sym_macro_rules_BANG] = ACTIONS(1971),
    [anon_sym_LPAREN] = ACTIONS(1971),
    [anon_sym_LBRACK] = ACTIONS(1971),
    [anon_sym_LBRACE] = ACTIONS(1971),
    [anon_sym_RBRACE] = ACTIONS(1971),
    [anon_sym_STAR] = ACTIONS(1971),
    [anon_sym_u8] = ACTIONS(1973),
    [anon_sym_i8] = ACTIONS(1973),
    [anon_sym_u16] = ACTIONS(1973),
    [anon_sym_i16] = ACTIONS(1973),
    [anon_sym_u32] = ACTIONS(1973),
    [anon_sym_i32] = ACTIONS(1973),
    [anon_sym_u64] = ACTIONS(1973),
    [anon_sym_i64] = ACTIONS(1973),
    [anon_sym_u128] = ACTIONS(1973),
    [anon_sym_i128] = ACTIONS(1973),
    [anon_sym_isize] = ACTIONS(1973),
    [anon_sym_usize] = ACTIONS(1973),
    [anon_sym_f32] = ACTIONS(1973),
    [anon_sym_f64] = ACTIONS(1973),
    [anon_sym_bool] = ACTIONS(1973),
    [anon_sym_str] = ACTIONS(1973),
    [anon_sym_char] = ACTIONS(1973),
    [anon_sym_DASH] = ACTIONS(1971),
    [anon_sym_BANG] = ACTIONS(1971),
    [anon_sym_AMP] = ACTIONS(1971),
    [anon_sym_PIPE] = ACTIONS(1971),
    [anon_sym_LT] = ACTIONS(1971),
    [anon_sym_DOT_DOT] = ACTIONS(1971),
    [anon_sym_COLON_COLON] = ACTIONS(1971),
    [anon_sym_POUND] = ACTIONS(1971),
    [anon_sym_SQUOTE] = ACTIONS(1973),
    [anon_sym_async] = ACTIONS(1973),
    [anon_sym_break] = ACTIONS(1973),
    [anon_sym_const] = ACTIONS(1973),
    [anon_sym_continue] = ACTIONS(1973),
    [anon_sym_default] = ACTIONS(1973),
    [anon_sym_enum] = ACTIONS(1973),
    [anon_sym_fn] = ACTIONS(1973),
    [anon_sym_for] = ACTIONS(1973),
    [anon_sym_gen] = ACTIONS(1973),
    [anon_sym_if] = ACTIONS(1973),
    [anon_sym_impl] = ACTIONS(1973),
    [anon_sym_let] = ACTIONS(1973),
    [anon_sym_loop] = ACTIONS(1973),
    [anon_sym_match] = ACTIONS(1973),
    [anon_sym_mod] = ACTIONS(1973),
    [anon_sym_pub] = ACTIONS(1973),
    [anon_sym_return] = ACTIONS(1973),
    [anon_sym_static] = ACTIONS(1973),
    [anon_sym_struct] = ACTIONS(1973),
    [anon_sym_trait] = ACTIONS(1973),
    [anon_sym_type] = ACTIONS(1973),
    [anon_sym_union] = ACTIONS(1973),
    [anon_sym_unsafe] = ACTIONS(1973),
    [anon_sym_use] = ACTIONS(1973),
    [anon_sym_while] = ACTIONS(1973),
    [anon_sym_extern] = ACTIONS(1973),
    [anon_sym_yield] = ACTIONS(1973),
    [anon_sym_move] = ACTIONS(1973),
    [anon_sym_try] = ACTIONS(1973),
    [sym_integer_literal] = ACTIONS(1971),
    [aux_sym_string_literal_token1] = ACTIONS(1971),
    [sym_char_literal] = ACTIONS(1971),
    [anon_sym_true] = ACTIONS(1973),
    [anon_sym_false] = ACTIONS(1973),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1973),
    [sym_super] = ACTIONS(1973),
    [sym_crate] = ACTIONS(1973),
    [sym_metavariable] = ACTIONS(1971),
    [sym__raw_string_literal_start] = ACTIONS(1971),
    [sym_float_literal] = ACTIONS(1971),
  },
  [556] = {
    [sym_line_comment] = STATE(556),
    [sym_block_comment] = STATE(556),
    [ts_builtin_sym_end] = ACTIONS(1975),
    [sym_identifier] = ACTIONS(1977),
    [anon_sym_SEMI] = ACTIONS(1975),
    [anon_sym_macro_rules_BANG] = ACTIONS(1975),
    [anon_sym_LPAREN] = ACTIONS(1975),
    [anon_sym_LBRACK] = ACTIONS(1975),
    [anon_sym_LBRACE] = ACTIONS(1975),
    [anon_sym_RBRACE] = ACTIONS(1975),
    [anon_sym_STAR] = ACTIONS(1975),
    [anon_sym_u8] = ACTIONS(1977),
    [anon_sym_i8] = ACTIONS(1977),
    [anon_sym_u16] = ACTIONS(1977),
    [anon_sym_i16] = ACTIONS(1977),
    [anon_sym_u32] = ACTIONS(1977),
    [anon_sym_i32] = ACTIONS(1977),
    [anon_sym_u64] = ACTIONS(1977),
    [anon_sym_i64] = ACTIONS(1977),
    [anon_sym_u128] = ACTIONS(1977),
    [anon_sym_i128] = ACTIONS(1977),
    [anon_sym_isize] = ACTIONS(1977),
    [anon_sym_usize] = ACTIONS(1977),
    [anon_sym_f32] = ACTIONS(1977),
    [anon_sym_f64] = ACTIONS(1977),
    [anon_sym_bool] = ACTIONS(1977),
    [anon_sym_str] = ACTIONS(1977),
    [anon_sym_char] = ACTIONS(1977),
    [anon_sym_DASH] = ACTIONS(1975),
    [anon_sym_BANG] = ACTIONS(1975),
    [anon_sym_AMP] = ACTIONS(1975),
    [anon_sym_PIPE] = ACTIONS(1975),
    [anon_sym_LT] = ACTIONS(1975),
    [anon_sym_DOT_DOT] = ACTIONS(1975),
    [anon_sym_COLON_COLON] = ACTIONS(1975),
    [anon_sym_POUND] = ACTIONS(1975),
    [anon_sym_SQUOTE] = ACTIONS(1977),
    [anon_sym_async] = ACTIONS(1977),
    [anon_sym_break] = ACTIONS(1977),
    [anon_sym_const] = ACTIONS(1977),
    [anon_sym_continue] = ACTIONS(1977),
    [anon_sym_default] = ACTIONS(1977),
    [anon_sym_enum] = ACTIONS(1977),
    [anon_sym_fn] = ACTIONS(1977),
    [anon_sym_for] = ACTIONS(1977),
    [anon_sym_gen] = ACTIONS(1977),
    [anon_sym_if] = ACTIONS(1977),
    [anon_sym_impl] = ACTIONS(1977),
    [anon_sym_let] = ACTIONS(1977),
    [anon_sym_loop] = ACTIONS(1977),
    [anon_sym_match] = ACTIONS(1977),
    [anon_sym_mod] = ACTIONS(1977),
    [anon_sym_pub] = ACTIONS(1977),
    [anon_sym_return] = ACTIONS(1977),
    [anon_sym_static] = ACTIONS(1977),
    [anon_sym_struct] = ACTIONS(1977),
    [anon_sym_trait] = ACTIONS(1977),
    [anon_sym_type] = ACTIONS(1977),
    [anon_sym_union] = ACTIONS(1977),
    [anon_sym_unsafe] = ACTIONS(1977),
    [anon_sym_use] = ACTIONS(1977),
    [anon_sym_while] = ACTIONS(1977),
    [anon_sym_extern] = ACTIONS(1977),
    [anon_sym_yield] = ACTIONS(1977),
    [anon_sym_move] = ACTIONS(1977),
    [anon_sym_try] = ACTIONS(1977),
    [sym_integer_literal] = ACTIONS(1975),
    [aux_sym_string_literal_token1] = ACTIONS(1975),
    [sym_char_literal] = ACTIONS(1975),
    [anon_sym_true] = ACTIONS(1977),
    [anon_sym_false] = ACTIONS(1977),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1977),
    [sym_super] = ACTIONS(1977),
    [sym_crate] = ACTIONS(1977),
    [sym_metavariable] = ACTIONS(1975),
    [sym__raw_string_literal_start] = ACTIONS(1975),
    [sym_float_literal] = ACTIONS(1975),
  },
  [557] = {
    [sym_line_comment] = STATE(557),
    [sym_block_comment] = STATE(557),
    [ts_builtin_sym_end] = ACTIONS(1979),
    [sym_identifier] = ACTIONS(1981),
    [anon_sym_SEMI] = ACTIONS(1979),
    [anon_sym_macro_rules_BANG] = ACTIONS(1979),
    [anon_sym_LPAREN] = ACTIONS(1979),
    [anon_sym_LBRACK] = ACTIONS(1979),
    [anon_sym_LBRACE] = ACTIONS(1979),
    [anon_sym_RBRACE] = ACTIONS(1979),
    [anon_sym_STAR] = ACTIONS(1979),
    [anon_sym_u8] = ACTIONS(1981),
    [anon_sym_i8] = ACTIONS(1981),
    [anon_sym_u16] = ACTIONS(1981),
    [anon_sym_i16] = ACTIONS(1981),
    [anon_sym_u32] = ACTIONS(1981),
    [anon_sym_i32] = ACTIONS(1981),
    [anon_sym_u64] = ACTIONS(1981),
    [anon_sym_i64] = ACTIONS(1981),
    [anon_sym_u128] = ACTIONS(1981),
    [anon_sym_i128] = ACTIONS(1981),
    [anon_sym_isize] = ACTIONS(1981),
    [anon_sym_usize] = ACTIONS(1981),
    [anon_sym_f32] = ACTIONS(1981),
    [anon_sym_f64] = ACTIONS(1981),
    [anon_sym_bool] = ACTIONS(1981),
    [anon_sym_str] = ACTIONS(1981),
    [anon_sym_char] = ACTIONS(1981),
    [anon_sym_DASH] = ACTIONS(1979),
    [anon_sym_BANG] = ACTIONS(1979),
    [anon_sym_AMP] = ACTIONS(1979),
    [anon_sym_PIPE] = ACTIONS(1979),
    [anon_sym_LT] = ACTIONS(1979),
    [anon_sym_DOT_DOT] = ACTIONS(1979),
    [anon_sym_COLON_COLON] = ACTIONS(1979),
    [anon_sym_POUND] = ACTIONS(1979),
    [anon_sym_SQUOTE] = ACTIONS(1981),
    [anon_sym_async] = ACTIONS(1981),
    [anon_sym_break] = ACTIONS(1981),
    [anon_sym_const] = ACTIONS(1981),
    [anon_sym_continue] = ACTIONS(1981),
    [anon_sym_default] = ACTIONS(1981),
    [anon_sym_enum] = ACTIONS(1981),
    [anon_sym_fn] = ACTIONS(1981),
    [anon_sym_for] = ACTIONS(1981),
    [anon_sym_gen] = ACTIONS(1981),
    [anon_sym_if] = ACTIONS(1981),
    [anon_sym_impl] = ACTIONS(1981),
    [anon_sym_let] = ACTIONS(1981),
    [anon_sym_loop] = ACTIONS(1981),
    [anon_sym_match] = ACTIONS(1981),
    [anon_sym_mod] = ACTIONS(1981),
    [anon_sym_pub] = ACTIONS(1981),
    [anon_sym_return] = ACTIONS(1981),
    [anon_sym_static] = ACTIONS(1981),
    [anon_sym_struct] = ACTIONS(1981),
    [anon_sym_trait] = ACTIONS(1981),
    [anon_sym_type] = ACTIONS(1981),
    [anon_sym_union] = ACTIONS(1981),
    [anon_sym_unsafe] = ACTIONS(1981),
    [anon_sym_use] = ACTIONS(1981),
    [anon_sym_while] = ACTIONS(1981),
    [anon_sym_extern] = ACTIONS(1981),
    [anon_sym_yield] = ACTIONS(1981),
    [anon_sym_move] = ACTIONS(1981),
    [anon_sym_try] = ACTIONS(1981),
    [sym_integer_literal] = ACTIONS(1979),
    [aux_sym_string_literal_token1] = ACTIONS(1979),
    [sym_char_literal] = ACTIONS(1979),
    [anon_sym_true] = ACTIONS(1981),
    [anon_sym_false] = ACTIONS(1981),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1981),
    [sym_super] = ACTIONS(1981),
    [sym_crate] = ACTIONS(1981),
    [sym_metavariable] = ACTIONS(1979),
    [sym__raw_string_literal_start] = ACTIONS(1979),
    [sym_float_literal] = ACTIONS(1979),
  },
  [558] = {
    [sym_line_comment] = STATE(558),
    [sym_block_comment] = STATE(558),
    [ts_builtin_sym_end] = ACTIONS(1983),
    [sym_identifier] = ACTIONS(1985),
    [anon_sym_SEMI] = ACTIONS(1983),
    [anon_sym_macro_rules_BANG] = ACTIONS(1983),
    [anon_sym_LPAREN] = ACTIONS(1983),
    [anon_sym_LBRACK] = ACTIONS(1983),
    [anon_sym_LBRACE] = ACTIONS(1983),
    [anon_sym_RBRACE] = ACTIONS(1983),
    [anon_sym_STAR] = ACTIONS(1983),
    [anon_sym_u8] = ACTIONS(1985),
    [anon_sym_i8] = ACTIONS(1985),
    [anon_sym_u16] = ACTIONS(1985),
    [anon_sym_i16] = ACTIONS(1985),
    [anon_sym_u32] = ACTIONS(1985),
    [anon_sym_i32] = ACTIONS(1985),
    [anon_sym_u64] = ACTIONS(1985),
    [anon_sym_i64] = ACTIONS(1985),
    [anon_sym_u128] = ACTIONS(1985),
    [anon_sym_i128] = ACTIONS(1985),
    [anon_sym_isize] = ACTIONS(1985),
    [anon_sym_usize] = ACTIONS(1985),
    [anon_sym_f32] = ACTIONS(1985),
    [anon_sym_f64] = ACTIONS(1985),
    [anon_sym_bool] = ACTIONS(1985),
    [anon_sym_str] = ACTIONS(1985),
    [anon_sym_char] = ACTIONS(1985),
    [anon_sym_DASH] = ACTIONS(1983),
    [anon_sym_BANG] = ACTIONS(1983),
    [anon_sym_AMP] = ACTIONS(1983),
    [anon_sym_PIPE] = ACTIONS(1983),
    [anon_sym_LT] = ACTIONS(1983),
    [anon_sym_DOT_DOT] = ACTIONS(1983),
    [anon_sym_COLON_COLON] = ACTIONS(1983),
    [anon_sym_POUND] = ACTIONS(1983),
    [anon_sym_SQUOTE] = ACTIONS(1985),
    [anon_sym_async] = ACTIONS(1985),
    [anon_sym_break] = ACTIONS(1985),
    [anon_sym_const] = ACTIONS(1985),
    [anon_sym_continue] = ACTIONS(1985),
    [anon_sym_default] = ACTIONS(1985),
    [anon_sym_enum] = ACTIONS(1985),
    [anon_sym_fn] = ACTIONS(1985),
    [anon_sym_for] = ACTIONS(1985),
    [anon_sym_gen] = ACTIONS(1985),
    [anon_sym_if] = ACTIONS(1985),
    [anon_sym_impl] = ACTIONS(1985),
    [anon_sym_let] = ACTIONS(1985),
    [anon_sym_loop] = ACTIONS(1985),
    [anon_sym_match] = ACTIONS(1985),
    [anon_sym_mod] = ACTIONS(1985),
    [anon_sym_pub] = ACTIONS(1985),
    [anon_sym_return] = ACTIONS(1985),
    [anon_sym_static] = ACTIONS(1985),
    [anon_sym_struct] = ACTIONS(1985),
    [anon_sym_trait] = ACTIONS(1985),
    [anon_sym_type] = ACTIONS(1985),
    [anon_sym_union] = ACTIONS(1985),
    [anon_sym_unsafe] = ACTIONS(1985),
    [anon_sym_use] = ACTIONS(1985),
    [anon_sym_while] = ACTIONS(1985),
    [anon_sym_extern] = ACTIONS(1985),
    [anon_sym_yield] = ACTIONS(1985),
    [anon_sym_move] = ACTIONS(1985),
    [anon_sym_try] = ACTIONS(1985),
    [sym_integer_literal] = ACTIONS(1983),
    [aux_sym_string_literal_token1] = ACTIONS(1983),
    [sym_char_literal] = ACTIONS(1983),
    [anon_sym_true] = ACTIONS(1985),
    [anon_sym_false] = ACTIONS(1985),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1985),
    [sym_super] = ACTIONS(1985),
    [sym_crate] = ACTIONS(1985),
    [sym_metavariable] = ACTIONS(1983),
    [sym__raw_string_literal_start] = ACTIONS(1983),
    [sym_float_literal] = ACTIONS(1983),
  },
  [559] = {
    [sym_line_comment] = STATE(559),
    [sym_block_comment] = STATE(559),
    [ts_builtin_sym_end] = ACTIONS(1987),
    [sym_identifier] = ACTIONS(1989),
    [anon_sym_SEMI] = ACTIONS(1987),
    [anon_sym_macro_rules_BANG] = ACTIONS(1987),
    [anon_sym_LPAREN] = ACTIONS(1987),
    [anon_sym_LBRACK] = ACTIONS(1987),
    [anon_sym_LBRACE] = ACTIONS(1987),
    [anon_sym_RBRACE] = ACTIONS(1987),
    [anon_sym_STAR] = ACTIONS(1987),
    [anon_sym_u8] = ACTIONS(1989),
    [anon_sym_i8] = ACTIONS(1989),
    [anon_sym_u16] = ACTIONS(1989),
    [anon_sym_i16] = ACTIONS(1989),
    [anon_sym_u32] = ACTIONS(1989),
    [anon_sym_i32] = ACTIONS(1989),
    [anon_sym_u64] = ACTIONS(1989),
    [anon_sym_i64] = ACTIONS(1989),
    [anon_sym_u128] = ACTIONS(1989),
    [anon_sym_i128] = ACTIONS(1989),
    [anon_sym_isize] = ACTIONS(1989),
    [anon_sym_usize] = ACTIONS(1989),
    [anon_sym_f32] = ACTIONS(1989),
    [anon_sym_f64] = ACTIONS(1989),
    [anon_sym_bool] = ACTIONS(1989),
    [anon_sym_str] = ACTIONS(1989),
    [anon_sym_char] = ACTIONS(1989),
    [anon_sym_DASH] = ACTIONS(1987),
    [anon_sym_BANG] = ACTIONS(1987),
    [anon_sym_AMP] = ACTIONS(1987),
    [anon_sym_PIPE] = ACTIONS(1987),
    [anon_sym_LT] = ACTIONS(1987),
    [anon_sym_DOT_DOT] = ACTIONS(1987),
    [anon_sym_COLON_COLON] = ACTIONS(1987),
    [anon_sym_POUND] = ACTIONS(1987),
    [anon_sym_SQUOTE] = ACTIONS(1989),
    [anon_sym_async] = ACTIONS(1989),
    [anon_sym_break] = ACTIONS(1989),
    [anon_sym_const] = ACTIONS(1989),
    [anon_sym_continue] = ACTIONS(1989),
    [anon_sym_default] = ACTIONS(1989),
    [anon_sym_enum] = ACTIONS(1989),
    [anon_sym_fn] = ACTIONS(1989),
    [anon_sym_for] = ACTIONS(1989),
    [anon_sym_gen] = ACTIONS(1989),
    [anon_sym_if] = ACTIONS(1989),
    [anon_sym_impl] = ACTIONS(1989),
    [anon_sym_let] = ACTIONS(1989),
    [anon_sym_loop] = ACTIONS(1989),
    [anon_sym_match] = ACTIONS(1989),
    [anon_sym_mod] = ACTIONS(1989),
    [anon_sym_pub] = ACTIONS(1989),
    [anon_sym_return] = ACTIONS(1989),
    [anon_sym_static] = ACTIONS(1989),
    [anon_sym_struct] = ACTIONS(1989),
    [anon_sym_trait] = ACTIONS(1989),
    [anon_sym_type] = ACTIONS(1989),
    [anon_sym_union] = ACTIONS(1989),
    [anon_sym_unsafe] = ACTIONS(1989),
    [anon_sym_use] = ACTIONS(1989),
    [anon_sym_while] = ACTIONS(1989),
    [anon_sym_extern] = ACTIONS(1989),
    [anon_sym_yield] = ACTIONS(1989),
    [anon_sym_move] = ACTIONS(1989),
    [anon_sym_try] = ACTIONS(1989),
    [sym_integer_literal] = ACTIONS(1987),
    [aux_sym_string_literal_token1] = ACTIONS(1987),
    [sym_char_literal] = ACTIONS(1987),
    [anon_sym_true] = ACTIONS(1989),
    [anon_sym_false] = ACTIONS(1989),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1989),
    [sym_super] = ACTIONS(1989),
    [sym_crate] = ACTIONS(1989),
    [sym_metavariable] = ACTIONS(1987),
    [sym__raw_string_literal_start] = ACTIONS(1987),
    [sym_float_literal] = ACTIONS(1987),
  },
  [560] = {
    [sym_line_comment] = STATE(560),
    [sym_block_comment] = STATE(560),
    [ts_builtin_sym_end] = ACTIONS(1991),
    [sym_identifier] = ACTIONS(1993),
    [anon_sym_SEMI] = ACTIONS(1991),
    [anon_sym_macro_rules_BANG] = ACTIONS(1991),
    [anon_sym_LPAREN] = ACTIONS(1991),
    [anon_sym_LBRACK] = ACTIONS(1991),
    [anon_sym_LBRACE] = ACTIONS(1991),
    [anon_sym_RBRACE] = ACTIONS(1991),
    [anon_sym_STAR] = ACTIONS(1991),
    [anon_sym_u8] = ACTIONS(1993),
    [anon_sym_i8] = ACTIONS(1993),
    [anon_sym_u16] = ACTIONS(1993),
    [anon_sym_i16] = ACTIONS(1993),
    [anon_sym_u32] = ACTIONS(1993),
    [anon_sym_i32] = ACTIONS(1993),
    [anon_sym_u64] = ACTIONS(1993),
    [anon_sym_i64] = ACTIONS(1993),
    [anon_sym_u128] = ACTIONS(1993),
    [anon_sym_i128] = ACTIONS(1993),
    [anon_sym_isize] = ACTIONS(1993),
    [anon_sym_usize] = ACTIONS(1993),
    [anon_sym_f32] = ACTIONS(1993),
    [anon_sym_f64] = ACTIONS(1993),
    [anon_sym_bool] = ACTIONS(1993),
    [anon_sym_str] = ACTIONS(1993),
    [anon_sym_char] = ACTIONS(1993),
    [anon_sym_DASH] = ACTIONS(1991),
    [anon_sym_BANG] = ACTIONS(1991),
    [anon_sym_AMP] = ACTIONS(1991),
    [anon_sym_PIPE] = ACTIONS(1991),
    [anon_sym_LT] = ACTIONS(1991),
    [anon_sym_DOT_DOT] = ACTIONS(1991),
    [anon_sym_COLON_COLON] = ACTIONS(1991),
    [anon_sym_POUND] = ACTIONS(1991),
    [anon_sym_SQUOTE] = ACTIONS(1993),
    [anon_sym_async] = ACTIONS(1993),
    [anon_sym_break] = ACTIONS(1993),
    [anon_sym_const] = ACTIONS(1993),
    [anon_sym_continue] = ACTIONS(1993),
    [anon_sym_default] = ACTIONS(1993),
    [anon_sym_enum] = ACTIONS(1993),
    [anon_sym_fn] = ACTIONS(1993),
    [anon_sym_for] = ACTIONS(1993),
    [anon_sym_gen] = ACTIONS(1993),
    [anon_sym_if] = ACTIONS(1993),
    [anon_sym_impl] = ACTIONS(1993),
    [anon_sym_let] = ACTIONS(1993),
    [anon_sym_loop] = ACTIONS(1993),
    [anon_sym_match] = ACTIONS(1993),
    [anon_sym_mod] = ACTIONS(1993),
    [anon_sym_pub] = ACTIONS(1993),
    [anon_sym_return] = ACTIONS(1993),
    [anon_sym_static] = ACTIONS(1993),
    [anon_sym_struct] = ACTIONS(1993),
    [anon_sym_trait] = ACTIONS(1993),
    [anon_sym_type] = ACTIONS(1993),
    [anon_sym_union] = ACTIONS(1993),
    [anon_sym_unsafe] = ACTIONS(1993),
    [anon_sym_use] = ACTIONS(1993),
    [anon_sym_while] = ACTIONS(1993),
    [anon_sym_extern] = ACTIONS(1993),
    [anon_sym_yield] = ACTIONS(1993),
    [anon_sym_move] = ACTIONS(1993),
    [anon_sym_try] = ACTIONS(1993),
    [sym_integer_literal] = ACTIONS(1991),
    [aux_sym_string_literal_token1] = ACTIONS(1991),
    [sym_char_literal] = ACTIONS(1991),
    [anon_sym_true] = ACTIONS(1993),
    [anon_sym_false] = ACTIONS(1993),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1993),
    [sym_super] = ACTIONS(1993),
    [sym_crate] = ACTIONS(1993),
    [sym_metavariable] = ACTIONS(1991),
    [sym__raw_string_literal_start] = ACTIONS(1991),
    [sym_float_literal] = ACTIONS(1991),
  },
  [561] = {
    [sym_line_comment] = STATE(561),
    [sym_block_comment] = STATE(561),
    [ts_builtin_sym_end] = ACTIONS(1995),
    [sym_identifier] = ACTIONS(1997),
    [anon_sym_SEMI] = ACTIONS(1995),
    [anon_sym_macro_rules_BANG] = ACTIONS(1995),
    [anon_sym_LPAREN] = ACTIONS(1995),
    [anon_sym_LBRACK] = ACTIONS(1995),
    [anon_sym_LBRACE] = ACTIONS(1995),
    [anon_sym_RBRACE] = ACTIONS(1995),
    [anon_sym_STAR] = ACTIONS(1995),
    [anon_sym_u8] = ACTIONS(1997),
    [anon_sym_i8] = ACTIONS(1997),
    [anon_sym_u16] = ACTIONS(1997),
    [anon_sym_i16] = ACTIONS(1997),
    [anon_sym_u32] = ACTIONS(1997),
    [anon_sym_i32] = ACTIONS(1997),
    [anon_sym_u64] = ACTIONS(1997),
    [anon_sym_i64] = ACTIONS(1997),
    [anon_sym_u128] = ACTIONS(1997),
    [anon_sym_i128] = ACTIONS(1997),
    [anon_sym_isize] = ACTIONS(1997),
    [anon_sym_usize] = ACTIONS(1997),
    [anon_sym_f32] = ACTIONS(1997),
    [anon_sym_f64] = ACTIONS(1997),
    [anon_sym_bool] = ACTIONS(1997),
    [anon_sym_str] = ACTIONS(1997),
    [anon_sym_char] = ACTIONS(1997),
    [anon_sym_DASH] = ACTIONS(1995),
    [anon_sym_BANG] = ACTIONS(1995),
    [anon_sym_AMP] = ACTIONS(1995),
    [anon_sym_PIPE] = ACTIONS(1995),
    [anon_sym_LT] = ACTIONS(1995),
    [anon_sym_DOT_DOT] = ACTIONS(1995),
    [anon_sym_COLON_COLON] = ACTIONS(1995),
    [anon_sym_POUND] = ACTIONS(1995),
    [anon_sym_SQUOTE] = ACTIONS(1997),
    [anon_sym_async] = ACTIONS(1997),
    [anon_sym_break] = ACTIONS(1997),
    [anon_sym_const] = ACTIONS(1997),
    [anon_sym_continue] = ACTIONS(1997),
    [anon_sym_default] = ACTIONS(1997),
    [anon_sym_enum] = ACTIONS(1997),
    [anon_sym_fn] = ACTIONS(1997),
    [anon_sym_for] = ACTIONS(1997),
    [anon_sym_gen] = ACTIONS(1997),
    [anon_sym_if] = ACTIONS(1997),
    [anon_sym_impl] = ACTIONS(1997),
    [anon_sym_let] = ACTIONS(1997),
    [anon_sym_loop] = ACTIONS(1997),
    [anon_sym_match] = ACTIONS(1997),
    [anon_sym_mod] = ACTIONS(1997),
    [anon_sym_pub] = ACTIONS(1997),
    [anon_sym_return] = ACTIONS(1997),
    [anon_sym_static] = ACTIONS(1997),
    [anon_sym_struct] = ACTIONS(1997),
    [anon_sym_trait] = ACTIONS(1997),
    [anon_sym_type] = ACTIONS(1997),
    [anon_sym_union] = ACTIONS(1997),
    [anon_sym_unsafe] = ACTIONS(1997),
    [anon_sym_use] = ACTIONS(1997),
    [anon_sym_while] = ACTIONS(1997),
    [anon_sym_extern] = ACTIONS(1997),
    [anon_sym_yield] = ACTIONS(1997),
    [anon_sym_move] = ACTIONS(1997),
    [anon_sym_try] = ACTIONS(1997),
    [sym_integer_literal] = ACTIONS(1995),
    [aux_sym_string_literal_token1] = ACTIONS(1995),
    [sym_char_literal] = ACTIONS(1995),
    [anon_sym_true] = ACTIONS(1997),
    [anon_sym_false] = ACTIONS(1997),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1997),
    [sym_super] = ACTIONS(1997),
    [sym_crate] = ACTIONS(1997),
    [sym_metavariable] = ACTIONS(1995),
    [sym__raw_string_literal_start] = ACTIONS(1995),
    [sym_float_literal] = ACTIONS(1995),
  },
  [562] = {
    [sym_line_comment] = STATE(562),
    [sym_block_comment] = STATE(562),
    [ts_builtin_sym_end] = ACTIONS(1999),
    [sym_identifier] = ACTIONS(2001),
    [anon_sym_SEMI] = ACTIONS(1999),
    [anon_sym_macro_rules_BANG] = ACTIONS(1999),
    [anon_sym_LPAREN] = ACTIONS(1999),
    [anon_sym_LBRACK] = ACTIONS(1999),
    [anon_sym_LBRACE] = ACTIONS(1999),
    [anon_sym_RBRACE] = ACTIONS(1999),
    [anon_sym_STAR] = ACTIONS(1999),
    [anon_sym_u8] = ACTIONS(2001),
    [anon_sym_i8] = ACTIONS(2001),
    [anon_sym_u16] = ACTIONS(2001),
    [anon_sym_i16] = ACTIONS(2001),
    [anon_sym_u32] = ACTIONS(2001),
    [anon_sym_i32] = ACTIONS(2001),
    [anon_sym_u64] = ACTIONS(2001),
    [anon_sym_i64] = ACTIONS(2001),
    [anon_sym_u128] = ACTIONS(2001),
    [anon_sym_i128] = ACTIONS(2001),
    [anon_sym_isize] = ACTIONS(2001),
    [anon_sym_usize] = ACTIONS(2001),
    [anon_sym_f32] = ACTIONS(2001),
    [anon_sym_f64] = ACTIONS(2001),
    [anon_sym_bool] = ACTIONS(2001),
    [anon_sym_str] = ACTIONS(2001),
    [anon_sym_char] = ACTIONS(2001),
    [anon_sym_DASH] = ACTIONS(1999),
    [anon_sym_BANG] = ACTIONS(1999),
    [anon_sym_AMP] = ACTIONS(1999),
    [anon_sym_PIPE] = ACTIONS(1999),
    [anon_sym_LT] = ACTIONS(1999),
    [anon_sym_DOT_DOT] = ACTIONS(1999),
    [anon_sym_COLON_COLON] = ACTIONS(1999),
    [anon_sym_POUND] = ACTIONS(1999),
    [anon_sym_SQUOTE] = ACTIONS(2001),
    [anon_sym_async] = ACTIONS(2001),
    [anon_sym_break] = ACTIONS(2001),
    [anon_sym_const] = ACTIONS(2001),
    [anon_sym_continue] = ACTIONS(2001),
    [anon_sym_default] = ACTIONS(2001),
    [anon_sym_enum] = ACTIONS(2001),
    [anon_sym_fn] = ACTIONS(2001),
    [anon_sym_for] = ACTIONS(2001),
    [anon_sym_gen] = ACTIONS(2001),
    [anon_sym_if] = ACTIONS(2001),
    [anon_sym_impl] = ACTIONS(2001),
    [anon_sym_let] = ACTIONS(2001),
    [anon_sym_loop] = ACTIONS(2001),
    [anon_sym_match] = ACTIONS(2001),
    [anon_sym_mod] = ACTIONS(2001),
    [anon_sym_pub] = ACTIONS(2001),
    [anon_sym_return] = ACTIONS(2001),
    [anon_sym_static] = ACTIONS(2001),
    [anon_sym_struct] = ACTIONS(2001),
    [anon_sym_trait] = ACTIONS(2001),
    [anon_sym_type] = ACTIONS(2001),
    [anon_sym_union] = ACTIONS(2001),
    [anon_sym_unsafe] = ACTIONS(2001),
    [anon_sym_use] = ACTIONS(2001),
    [anon_sym_while] = ACTIONS(2001),
    [anon_sym_extern] = ACTIONS(2001),
    [anon_sym_yield] = ACTIONS(2001),
    [anon_sym_move] = ACTIONS(2001),
    [anon_sym_try] = ACTIONS(2001),
    [sym_integer_literal] = ACTIONS(1999),
    [aux_sym_string_literal_token1] = ACTIONS(1999),
    [sym_char_literal] = ACTIONS(1999),
    [anon_sym_true] = ACTIONS(2001),
    [anon_sym_false] = ACTIONS(2001),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2001),
    [sym_super] = ACTIONS(2001),
    [sym_crate] = ACTIONS(2001),
    [sym_metavariable] = ACTIONS(1999),
    [sym__raw_string_literal_start] = ACTIONS(1999),
    [sym_float_literal] = ACTIONS(1999),
  },
  [563] = {
    [sym_line_comment] = STATE(563),
    [sym_block_comment] = STATE(563),
    [ts_builtin_sym_end] = ACTIONS(2003),
    [sym_identifier] = ACTIONS(2005),
    [anon_sym_SEMI] = ACTIONS(2003),
    [anon_sym_macro_rules_BANG] = ACTIONS(2003),
    [anon_sym_LPAREN] = ACTIONS(2003),
    [anon_sym_LBRACK] = ACTIONS(2003),
    [anon_sym_LBRACE] = ACTIONS(2003),
    [anon_sym_RBRACE] = ACTIONS(2003),
    [anon_sym_STAR] = ACTIONS(2003),
    [anon_sym_u8] = ACTIONS(2005),
    [anon_sym_i8] = ACTIONS(2005),
    [anon_sym_u16] = ACTIONS(2005),
    [anon_sym_i16] = ACTIONS(2005),
    [anon_sym_u32] = ACTIONS(2005),
    [anon_sym_i32] = ACTIONS(2005),
    [anon_sym_u64] = ACTIONS(2005),
    [anon_sym_i64] = ACTIONS(2005),
    [anon_sym_u128] = ACTIONS(2005),
    [anon_sym_i128] = ACTIONS(2005),
    [anon_sym_isize] = ACTIONS(2005),
    [anon_sym_usize] = ACTIONS(2005),
    [anon_sym_f32] = ACTIONS(2005),
    [anon_sym_f64] = ACTIONS(2005),
    [anon_sym_bool] = ACTIONS(2005),
    [anon_sym_str] = ACTIONS(2005),
    [anon_sym_char] = ACTIONS(2005),
    [anon_sym_DASH] = ACTIONS(2003),
    [anon_sym_BANG] = ACTIONS(2003),
    [anon_sym_AMP] = ACTIONS(2003),
    [anon_sym_PIPE] = ACTIONS(2003),
    [anon_sym_LT] = ACTIONS(2003),
    [anon_sym_DOT_DOT] = ACTIONS(2003),
    [anon_sym_COLON_COLON] = ACTIONS(2003),
    [anon_sym_POUND] = ACTIONS(2003),
    [anon_sym_SQUOTE] = ACTIONS(2005),
    [anon_sym_async] = ACTIONS(2005),
    [anon_sym_break] = ACTIONS(2005),
    [anon_sym_const] = ACTIONS(2005),
    [anon_sym_continue] = ACTIONS(2005),
    [anon_sym_default] = ACTIONS(2005),
    [anon_sym_enum] = ACTIONS(2005),
    [anon_sym_fn] = ACTIONS(2005),
    [anon_sym_for] = ACTIONS(2005),
    [anon_sym_gen] = ACTIONS(2005),
    [anon_sym_if] = ACTIONS(2005),
    [anon_sym_impl] = ACTIONS(2005),
    [anon_sym_let] = ACTIONS(2005),
    [anon_sym_loop] = ACTIONS(2005),
    [anon_sym_match] = ACTIONS(2005),
    [anon_sym_mod] = ACTIONS(2005),
    [anon_sym_pub] = ACTIONS(2005),
    [anon_sym_return] = ACTIONS(2005),
    [anon_sym_static] = ACTIONS(2005),
    [anon_sym_struct] = ACTIONS(2005),
    [anon_sym_trait] = ACTIONS(2005),
    [anon_sym_type] = ACTIONS(2005),
    [anon_sym_union] = ACTIONS(2005),
    [anon_sym_unsafe] = ACTIONS(2005),
    [anon_sym_use] = ACTIONS(2005),
    [anon_sym_while] = ACTIONS(2005),
    [anon_sym_extern] = ACTIONS(2005),
    [anon_sym_yield] = ACTIONS(2005),
    [anon_sym_move] = ACTIONS(2005),
    [anon_sym_try] = ACTIONS(2005),
    [sym_integer_literal] = ACTIONS(2003),
    [aux_sym_string_literal_token1] = ACTIONS(2003),
    [sym_char_literal] = ACTIONS(2003),
    [anon_sym_true] = ACTIONS(2005),
    [anon_sym_false] = ACTIONS(2005),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2005),
    [sym_super] = ACTIONS(2005),
    [sym_crate] = ACTIONS(2005),
    [sym_metavariable] = ACTIONS(2003),
    [sym__raw_string_literal_start] = ACTIONS(2003),
    [sym_float_literal] = ACTIONS(2003),
  },
  [564] = {
    [sym_line_comment] = STATE(564),
    [sym_block_comment] = STATE(564),
    [ts_builtin_sym_end] = ACTIONS(2007),
    [sym_identifier] = ACTIONS(2009),
    [anon_sym_SEMI] = ACTIONS(2007),
    [anon_sym_macro_rules_BANG] = ACTIONS(2007),
    [anon_sym_LPAREN] = ACTIONS(2007),
    [anon_sym_LBRACK] = ACTIONS(2007),
    [anon_sym_LBRACE] = ACTIONS(2007),
    [anon_sym_RBRACE] = ACTIONS(2007),
    [anon_sym_STAR] = ACTIONS(2007),
    [anon_sym_u8] = ACTIONS(2009),
    [anon_sym_i8] = ACTIONS(2009),
    [anon_sym_u16] = ACTIONS(2009),
    [anon_sym_i16] = ACTIONS(2009),
    [anon_sym_u32] = ACTIONS(2009),
    [anon_sym_i32] = ACTIONS(2009),
    [anon_sym_u64] = ACTIONS(2009),
    [anon_sym_i64] = ACTIONS(2009),
    [anon_sym_u128] = ACTIONS(2009),
    [anon_sym_i128] = ACTIONS(2009),
    [anon_sym_isize] = ACTIONS(2009),
    [anon_sym_usize] = ACTIONS(2009),
    [anon_sym_f32] = ACTIONS(2009),
    [anon_sym_f64] = ACTIONS(2009),
    [anon_sym_bool] = ACTIONS(2009),
    [anon_sym_str] = ACTIONS(2009),
    [anon_sym_char] = ACTIONS(2009),
    [anon_sym_DASH] = ACTIONS(2007),
    [anon_sym_BANG] = ACTIONS(2007),
    [anon_sym_AMP] = ACTIONS(2007),
    [anon_sym_PIPE] = ACTIONS(2007),
    [anon_sym_LT] = ACTIONS(2007),
    [anon_sym_DOT_DOT] = ACTIONS(2007),
    [anon_sym_COLON_COLON] = ACTIONS(2007),
    [anon_sym_POUND] = ACTIONS(2007),
    [anon_sym_SQUOTE] = ACTIONS(2009),
    [anon_sym_async] = ACTIONS(2009),
    [anon_sym_break] = ACTIONS(2009),
    [anon_sym_const] = ACTIONS(2009),
    [anon_sym_continue] = ACTIONS(2009),
    [anon_sym_default] = ACTIONS(2009),
    [anon_sym_enum] = ACTIONS(2009),
    [anon_sym_fn] = ACTIONS(2009),
    [anon_sym_for] = ACTIONS(2009),
    [anon_sym_gen] = ACTIONS(2009),
    [anon_sym_if] = ACTIONS(2009),
    [anon_sym_impl] = ACTIONS(2009),
    [anon_sym_let] = ACTIONS(2009),
    [anon_sym_loop] = ACTIONS(2009),
    [anon_sym_match] = ACTIONS(2009),
    [anon_sym_mod] = ACTIONS(2009),
    [anon_sym_pub] = ACTIONS(2009),
    [anon_sym_return] = ACTIONS(2009),
    [anon_sym_static] = ACTIONS(2009),
    [anon_sym_struct] = ACTIONS(2009),
    [anon_sym_trait] = ACTIONS(2009),
    [anon_sym_type] = ACTIONS(2009),
    [anon_sym_union] = ACTIONS(2009),
    [anon_sym_unsafe] = ACTIONS(2009),
    [anon_sym_use] = ACTIONS(2009),
    [anon_sym_while] = ACTIONS(2009),
    [anon_sym_extern] = ACTIONS(2009),
    [anon_sym_yield] = ACTIONS(2009),
    [anon_sym_move] = ACTIONS(2009),
    [anon_sym_try] = ACTIONS(2009),
    [sym_integer_literal] = ACTIONS(2007),
    [aux_sym_string_literal_token1] = ACTIONS(2007),
    [sym_char_literal] = ACTIONS(2007),
    [anon_sym_true] = ACTIONS(2009),
    [anon_sym_false] = ACTIONS(2009),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2009),
    [sym_super] = ACTIONS(2009),
    [sym_crate] = ACTIONS(2009),
    [sym_metavariable] = ACTIONS(2007),
    [sym__raw_string_literal_start] = ACTIONS(2007),
    [sym_float_literal] = ACTIONS(2007),
  },
  [565] = {
    [sym_line_comment] = STATE(565),
    [sym_block_comment] = STATE(565),
    [ts_builtin_sym_end] = ACTIONS(2011),
    [sym_identifier] = ACTIONS(2013),
    [anon_sym_SEMI] = ACTIONS(2011),
    [anon_sym_macro_rules_BANG] = ACTIONS(2011),
    [anon_sym_LPAREN] = ACTIONS(2011),
    [anon_sym_LBRACK] = ACTIONS(2011),
    [anon_sym_LBRACE] = ACTIONS(2011),
    [anon_sym_RBRACE] = ACTIONS(2011),
    [anon_sym_STAR] = ACTIONS(2011),
    [anon_sym_u8] = ACTIONS(2013),
    [anon_sym_i8] = ACTIONS(2013),
    [anon_sym_u16] = ACTIONS(2013),
    [anon_sym_i16] = ACTIONS(2013),
    [anon_sym_u32] = ACTIONS(2013),
    [anon_sym_i32] = ACTIONS(2013),
    [anon_sym_u64] = ACTIONS(2013),
    [anon_sym_i64] = ACTIONS(2013),
    [anon_sym_u128] = ACTIONS(2013),
    [anon_sym_i128] = ACTIONS(2013),
    [anon_sym_isize] = ACTIONS(2013),
    [anon_sym_usize] = ACTIONS(2013),
    [anon_sym_f32] = ACTIONS(2013),
    [anon_sym_f64] = ACTIONS(2013),
    [anon_sym_bool] = ACTIONS(2013),
    [anon_sym_str] = ACTIONS(2013),
    [anon_sym_char] = ACTIONS(2013),
    [anon_sym_DASH] = ACTIONS(2011),
    [anon_sym_BANG] = ACTIONS(2011),
    [anon_sym_AMP] = ACTIONS(2011),
    [anon_sym_PIPE] = ACTIONS(2011),
    [anon_sym_LT] = ACTIONS(2011),
    [anon_sym_DOT_DOT] = ACTIONS(2011),
    [anon_sym_COLON_COLON] = ACTIONS(2011),
    [anon_sym_POUND] = ACTIONS(2011),
    [anon_sym_SQUOTE] = ACTIONS(2013),
    [anon_sym_async] = ACTIONS(2013),
    [anon_sym_break] = ACTIONS(2013),
    [anon_sym_const] = ACTIONS(2013),
    [anon_sym_continue] = ACTIONS(2013),
    [anon_sym_default] = ACTIONS(2013),
    [anon_sym_enum] = ACTIONS(2013),
    [anon_sym_fn] = ACTIONS(2013),
    [anon_sym_for] = ACTIONS(2013),
    [anon_sym_gen] = ACTIONS(2013),
    [anon_sym_if] = ACTIONS(2013),
    [anon_sym_impl] = ACTIONS(2013),
    [anon_sym_let] = ACTIONS(2013),
    [anon_sym_loop] = ACTIONS(2013),
    [anon_sym_match] = ACTIONS(2013),
    [anon_sym_mod] = ACTIONS(2013),
    [anon_sym_pub] = ACTIONS(2013),
    [anon_sym_return] = ACTIONS(2013),
    [anon_sym_static] = ACTIONS(2013),
    [anon_sym_struct] = ACTIONS(2013),
    [anon_sym_trait] = ACTIONS(2013),
    [anon_sym_type] = ACTIONS(2013),
    [anon_sym_union] = ACTIONS(2013),
    [anon_sym_unsafe] = ACTIONS(2013),
    [anon_sym_use] = ACTIONS(2013),
    [anon_sym_while] = ACTIONS(2013),
    [anon_sym_extern] = ACTIONS(2013),
    [anon_sym_yield] = ACTIONS(2013),
    [anon_sym_move] = ACTIONS(2013),
    [anon_sym_try] = ACTIONS(2013),
    [sym_integer_literal] = ACTIONS(2011),
    [aux_sym_string_literal_token1] = ACTIONS(2011),
    [sym_char_literal] = ACTIONS(2011),
    [anon_sym_true] = ACTIONS(2013),
    [anon_sym_false] = ACTIONS(2013),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2013),
    [sym_super] = ACTIONS(2013),
    [sym_crate] = ACTIONS(2013),
    [sym_metavariable] = ACTIONS(2011),
    [sym__raw_string_literal_start] = ACTIONS(2011),
    [sym_float_literal] = ACTIONS(2011),
  },
  [566] = {
    [sym_line_comment] = STATE(566),
    [sym_block_comment] = STATE(566),
    [ts_builtin_sym_end] = ACTIONS(2015),
    [sym_identifier] = ACTIONS(2017),
    [anon_sym_SEMI] = ACTIONS(2015),
    [anon_sym_macro_rules_BANG] = ACTIONS(2015),
    [anon_sym_LPAREN] = ACTIONS(2015),
    [anon_sym_LBRACK] = ACTIONS(2015),
    [anon_sym_LBRACE] = ACTIONS(2015),
    [anon_sym_RBRACE] = ACTIONS(2015),
    [anon_sym_STAR] = ACTIONS(2015),
    [anon_sym_u8] = ACTIONS(2017),
    [anon_sym_i8] = ACTIONS(2017),
    [anon_sym_u16] = ACTIONS(2017),
    [anon_sym_i16] = ACTIONS(2017),
    [anon_sym_u32] = ACTIONS(2017),
    [anon_sym_i32] = ACTIONS(2017),
    [anon_sym_u64] = ACTIONS(2017),
    [anon_sym_i64] = ACTIONS(2017),
    [anon_sym_u128] = ACTIONS(2017),
    [anon_sym_i128] = ACTIONS(2017),
    [anon_sym_isize] = ACTIONS(2017),
    [anon_sym_usize] = ACTIONS(2017),
    [anon_sym_f32] = ACTIONS(2017),
    [anon_sym_f64] = ACTIONS(2017),
    [anon_sym_bool] = ACTIONS(2017),
    [anon_sym_str] = ACTIONS(2017),
    [anon_sym_char] = ACTIONS(2017),
    [anon_sym_DASH] = ACTIONS(2015),
    [anon_sym_BANG] = ACTIONS(2015),
    [anon_sym_AMP] = ACTIONS(2015),
    [anon_sym_PIPE] = ACTIONS(2015),
    [anon_sym_LT] = ACTIONS(2015),
    [anon_sym_DOT_DOT] = ACTIONS(2015),
    [anon_sym_COLON_COLON] = ACTIONS(2015),
    [anon_sym_POUND] = ACTIONS(2015),
    [anon_sym_SQUOTE] = ACTIONS(2017),
    [anon_sym_async] = ACTIONS(2017),
    [anon_sym_break] = ACTIONS(2017),
    [anon_sym_const] = ACTIONS(2017),
    [anon_sym_continue] = ACTIONS(2017),
    [anon_sym_default] = ACTIONS(2017),
    [anon_sym_enum] = ACTIONS(2017),
    [anon_sym_fn] = ACTIONS(2017),
    [anon_sym_for] = ACTIONS(2017),
    [anon_sym_gen] = ACTIONS(2017),
    [anon_sym_if] = ACTIONS(2017),
    [anon_sym_impl] = ACTIONS(2017),
    [anon_sym_let] = ACTIONS(2017),
    [anon_sym_loop] = ACTIONS(2017),
    [anon_sym_match] = ACTIONS(2017),
    [anon_sym_mod] = ACTIONS(2017),
    [anon_sym_pub] = ACTIONS(2017),
    [anon_sym_return] = ACTIONS(2017),
    [anon_sym_static] = ACTIONS(2017),
    [anon_sym_struct] = ACTIONS(2017),
    [anon_sym_trait] = ACTIONS(2017),
    [anon_sym_type] = ACTIONS(2017),
    [anon_sym_union] = ACTIONS(2017),
    [anon_sym_unsafe] = ACTIONS(2017),
    [anon_sym_use] = ACTIONS(2017),
    [anon_sym_while] = ACTIONS(2017),
    [anon_sym_extern] = ACTIONS(2017),
    [anon_sym_yield] = ACTIONS(2017),
    [anon_sym_move] = ACTIONS(2017),
    [anon_sym_try] = ACTIONS(2017),
    [sym_integer_literal] = ACTIONS(2015),
    [aux_sym_string_literal_token1] = ACTIONS(2015),
    [sym_char_literal] = ACTIONS(2015),
    [anon_sym_true] = ACTIONS(2017),
    [anon_sym_false] = ACTIONS(2017),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2017),
    [sym_super] = ACTIONS(2017),
    [sym_crate] = ACTIONS(2017),
    [sym_metavariable] = ACTIONS(2015),
    [sym__raw_string_literal_start] = ACTIONS(2015),
    [sym_float_literal] = ACTIONS(2015),
  },
  [567] = {
    [sym_line_comment] = STATE(567),
    [sym_block_comment] = STATE(567),
    [ts_builtin_sym_end] = ACTIONS(2019),
    [sym_identifier] = ACTIONS(2021),
    [anon_sym_SEMI] = ACTIONS(2019),
    [anon_sym_macro_rules_BANG] = ACTIONS(2019),
    [anon_sym_LPAREN] = ACTIONS(2019),
    [anon_sym_LBRACK] = ACTIONS(2019),
    [anon_sym_LBRACE] = ACTIONS(2019),
    [anon_sym_RBRACE] = ACTIONS(2019),
    [anon_sym_STAR] = ACTIONS(2019),
    [anon_sym_u8] = ACTIONS(2021),
    [anon_sym_i8] = ACTIONS(2021),
    [anon_sym_u16] = ACTIONS(2021),
    [anon_sym_i16] = ACTIONS(2021),
    [anon_sym_u32] = ACTIONS(2021),
    [anon_sym_i32] = ACTIONS(2021),
    [anon_sym_u64] = ACTIONS(2021),
    [anon_sym_i64] = ACTIONS(2021),
    [anon_sym_u128] = ACTIONS(2021),
    [anon_sym_i128] = ACTIONS(2021),
    [anon_sym_isize] = ACTIONS(2021),
    [anon_sym_usize] = ACTIONS(2021),
    [anon_sym_f32] = ACTIONS(2021),
    [anon_sym_f64] = ACTIONS(2021),
    [anon_sym_bool] = ACTIONS(2021),
    [anon_sym_str] = ACTIONS(2021),
    [anon_sym_char] = ACTIONS(2021),
    [anon_sym_DASH] = ACTIONS(2019),
    [anon_sym_BANG] = ACTIONS(2019),
    [anon_sym_AMP] = ACTIONS(2019),
    [anon_sym_PIPE] = ACTIONS(2019),
    [anon_sym_LT] = ACTIONS(2019),
    [anon_sym_DOT_DOT] = ACTIONS(2019),
    [anon_sym_COLON_COLON] = ACTIONS(2019),
    [anon_sym_POUND] = ACTIONS(2019),
    [anon_sym_SQUOTE] = ACTIONS(2021),
    [anon_sym_async] = ACTIONS(2021),
    [anon_sym_break] = ACTIONS(2021),
    [anon_sym_const] = ACTIONS(2021),
    [anon_sym_continue] = ACTIONS(2021),
    [anon_sym_default] = ACTIONS(2021),
    [anon_sym_enum] = ACTIONS(2021),
    [anon_sym_fn] = ACTIONS(2021),
    [anon_sym_for] = ACTIONS(2021),
    [anon_sym_gen] = ACTIONS(2021),
    [anon_sym_if] = ACTIONS(2021),
    [anon_sym_impl] = ACTIONS(2021),
    [anon_sym_let] = ACTIONS(2021),
    [anon_sym_loop] = ACTIONS(2021),
    [anon_sym_match] = ACTIONS(2021),
    [anon_sym_mod] = ACTIONS(2021),
    [anon_sym_pub] = ACTIONS(2021),
    [anon_sym_return] = ACTIONS(2021),
    [anon_sym_static] = ACTIONS(2021),
    [anon_sym_struct] = ACTIONS(2021),
    [anon_sym_trait] = ACTIONS(2021),
    [anon_sym_type] = ACTIONS(2021),
    [anon_sym_union] = ACTIONS(2021),
    [anon_sym_unsafe] = ACTIONS(2021),
    [anon_sym_use] = ACTIONS(2021),
    [anon_sym_while] = ACTIONS(2021),
    [anon_sym_extern] = ACTIONS(2021),
    [anon_sym_yield] = ACTIONS(2021),
    [anon_sym_move] = ACTIONS(2021),
    [anon_sym_try] = ACTIONS(2021),
    [sym_integer_literal] = ACTIONS(2019),
    [aux_sym_string_literal_token1] = ACTIONS(2019),
    [sym_char_literal] = ACTIONS(2019),
    [anon_sym_true] = ACTIONS(2021),
    [anon_sym_false] = ACTIONS(2021),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2021),
    [sym_super] = ACTIONS(2021),
    [sym_crate] = ACTIONS(2021),
    [sym_metavariable] = ACTIONS(2019),
    [sym__raw_string_literal_start] = ACTIONS(2019),
    [sym_float_literal] = ACTIONS(2019),
  },
  [568] = {
    [sym_line_comment] = STATE(568),
    [sym_block_comment] = STATE(568),
    [ts_builtin_sym_end] = ACTIONS(2023),
    [sym_identifier] = ACTIONS(2025),
    [anon_sym_SEMI] = ACTIONS(2023),
    [anon_sym_macro_rules_BANG] = ACTIONS(2023),
    [anon_sym_LPAREN] = ACTIONS(2023),
    [anon_sym_LBRACK] = ACTIONS(2023),
    [anon_sym_LBRACE] = ACTIONS(2023),
    [anon_sym_RBRACE] = ACTIONS(2023),
    [anon_sym_STAR] = ACTIONS(2023),
    [anon_sym_u8] = ACTIONS(2025),
    [anon_sym_i8] = ACTIONS(2025),
    [anon_sym_u16] = ACTIONS(2025),
    [anon_sym_i16] = ACTIONS(2025),
    [anon_sym_u32] = ACTIONS(2025),
    [anon_sym_i32] = ACTIONS(2025),
    [anon_sym_u64] = ACTIONS(2025),
    [anon_sym_i64] = ACTIONS(2025),
    [anon_sym_u128] = ACTIONS(2025),
    [anon_sym_i128] = ACTIONS(2025),
    [anon_sym_isize] = ACTIONS(2025),
    [anon_sym_usize] = ACTIONS(2025),
    [anon_sym_f32] = ACTIONS(2025),
    [anon_sym_f64] = ACTIONS(2025),
    [anon_sym_bool] = ACTIONS(2025),
    [anon_sym_str] = ACTIONS(2025),
    [anon_sym_char] = ACTIONS(2025),
    [anon_sym_DASH] = ACTIONS(2023),
    [anon_sym_BANG] = ACTIONS(2023),
    [anon_sym_AMP] = ACTIONS(2023),
    [anon_sym_PIPE] = ACTIONS(2023),
    [anon_sym_LT] = ACTIONS(2023),
    [anon_sym_DOT_DOT] = ACTIONS(2023),
    [anon_sym_COLON_COLON] = ACTIONS(2023),
    [anon_sym_POUND] = ACTIONS(2023),
    [anon_sym_SQUOTE] = ACTIONS(2025),
    [anon_sym_async] = ACTIONS(2025),
    [anon_sym_break] = ACTIONS(2025),
    [anon_sym_const] = ACTIONS(2025),
    [anon_sym_continue] = ACTIONS(2025),
    [anon_sym_default] = ACTIONS(2025),
    [anon_sym_enum] = ACTIONS(2025),
    [anon_sym_fn] = ACTIONS(2025),
    [anon_sym_for] = ACTIONS(2025),
    [anon_sym_gen] = ACTIONS(2025),
    [anon_sym_if] = ACTIONS(2025),
    [anon_sym_impl] = ACTIONS(2025),
    [anon_sym_let] = ACTIONS(2025),
    [anon_sym_loop] = ACTIONS(2025),
    [anon_sym_match] = ACTIONS(2025),
    [anon_sym_mod] = ACTIONS(2025),
    [anon_sym_pub] = ACTIONS(2025),
    [anon_sym_return] = ACTIONS(2025),
    [anon_sym_static] = ACTIONS(2025),
    [anon_sym_struct] = ACTIONS(2025),
    [anon_sym_trait] = ACTIONS(2025),
    [anon_sym_type] = ACTIONS(2025),
    [anon_sym_union] = ACTIONS(2025),
    [anon_sym_unsafe] = ACTIONS(2025),
    [anon_sym_use] = ACTIONS(2025),
    [anon_sym_while] = ACTIONS(2025),
    [anon_sym_extern] = ACTIONS(2025),
    [anon_sym_yield] = ACTIONS(2025),
    [anon_sym_move] = ACTIONS(2025),
    [anon_sym_try] = ACTIONS(2025),
    [sym_integer_literal] = ACTIONS(2023),
    [aux_sym_string_literal_token1] = ACTIONS(2023),
    [sym_char_literal] = ACTIONS(2023),
    [anon_sym_true] = ACTIONS(2025),
    [anon_sym_false] = ACTIONS(2025),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2025),
    [sym_super] = ACTIONS(2025),
    [sym_crate] = ACTIONS(2025),
    [sym_metavariable] = ACTIONS(2023),
    [sym__raw_string_literal_start] = ACTIONS(2023),
    [sym_float_literal] = ACTIONS(2023),
  },
  [569] = {
    [sym_line_comment] = STATE(569),
    [sym_block_comment] = STATE(569),
    [ts_builtin_sym_end] = ACTIONS(2027),
    [sym_identifier] = ACTIONS(2029),
    [anon_sym_SEMI] = ACTIONS(2027),
    [anon_sym_macro_rules_BANG] = ACTIONS(2027),
    [anon_sym_LPAREN] = ACTIONS(2027),
    [anon_sym_LBRACK] = ACTIONS(2027),
    [anon_sym_LBRACE] = ACTIONS(2027),
    [anon_sym_RBRACE] = ACTIONS(2027),
    [anon_sym_STAR] = ACTIONS(2027),
    [anon_sym_u8] = ACTIONS(2029),
    [anon_sym_i8] = ACTIONS(2029),
    [anon_sym_u16] = ACTIONS(2029),
    [anon_sym_i16] = ACTIONS(2029),
    [anon_sym_u32] = ACTIONS(2029),
    [anon_sym_i32] = ACTIONS(2029),
    [anon_sym_u64] = ACTIONS(2029),
    [anon_sym_i64] = ACTIONS(2029),
    [anon_sym_u128] = ACTIONS(2029),
    [anon_sym_i128] = ACTIONS(2029),
    [anon_sym_isize] = ACTIONS(2029),
    [anon_sym_usize] = ACTIONS(2029),
    [anon_sym_f32] = ACTIONS(2029),
    [anon_sym_f64] = ACTIONS(2029),
    [anon_sym_bool] = ACTIONS(2029),
    [anon_sym_str] = ACTIONS(2029),
    [anon_sym_char] = ACTIONS(2029),
    [anon_sym_DASH] = ACTIONS(2027),
    [anon_sym_BANG] = ACTIONS(2027),
    [anon_sym_AMP] = ACTIONS(2027),
    [anon_sym_PIPE] = ACTIONS(2027),
    [anon_sym_LT] = ACTIONS(2027),
    [anon_sym_DOT_DOT] = ACTIONS(2027),
    [anon_sym_COLON_COLON] = ACTIONS(2027),
    [anon_sym_POUND] = ACTIONS(2027),
    [anon_sym_SQUOTE] = ACTIONS(2029),
    [anon_sym_async] = ACTIONS(2029),
    [anon_sym_break] = ACTIONS(2029),
    [anon_sym_const] = ACTIONS(2029),
    [anon_sym_continue] = ACTIONS(2029),
    [anon_sym_default] = ACTIONS(2029),
    [anon_sym_enum] = ACTIONS(2029),
    [anon_sym_fn] = ACTIONS(2029),
    [anon_sym_for] = ACTIONS(2029),
    [anon_sym_gen] = ACTIONS(2029),
    [anon_sym_if] = ACTIONS(2029),
    [anon_sym_impl] = ACTIONS(2029),
    [anon_sym_let] = ACTIONS(2029),
    [anon_sym_loop] = ACTIONS(2029),
    [anon_sym_match] = ACTIONS(2029),
    [anon_sym_mod] = ACTIONS(2029),
    [anon_sym_pub] = ACTIONS(2029),
    [anon_sym_return] = ACTIONS(2029),
    [anon_sym_static] = ACTIONS(2029),
    [anon_sym_struct] = ACTIONS(2029),
    [anon_sym_trait] = ACTIONS(2029),
    [anon_sym_type] = ACTIONS(2029),
    [anon_sym_union] = ACTIONS(2029),
    [anon_sym_unsafe] = ACTIONS(2029),
    [anon_sym_use] = ACTIONS(2029),
    [anon_sym_while] = ACTIONS(2029),
    [anon_sym_extern] = ACTIONS(2029),
    [anon_sym_yield] = ACTIONS(2029),
    [anon_sym_move] = ACTIONS(2029),
    [anon_sym_try] = ACTIONS(2029),
    [sym_integer_literal] = ACTIONS(2027),
    [aux_sym_string_literal_token1] = ACTIONS(2027),
    [sym_char_literal] = ACTIONS(2027),
    [anon_sym_true] = ACTIONS(2029),
    [anon_sym_false] = ACTIONS(2029),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2029),
    [sym_super] = ACTIONS(2029),
    [sym_crate] = ACTIONS(2029),
    [sym_metavariable] = ACTIONS(2027),
    [sym__raw_string_literal_start] = ACTIONS(2027),
    [sym_float_literal] = ACTIONS(2027),
  },
  [570] = {
    [sym_line_comment] = STATE(570),
    [sym_block_comment] = STATE(570),
    [ts_builtin_sym_end] = ACTIONS(2031),
    [sym_identifier] = ACTIONS(2033),
    [anon_sym_SEMI] = ACTIONS(2031),
    [anon_sym_macro_rules_BANG] = ACTIONS(2031),
    [anon_sym_LPAREN] = ACTIONS(2031),
    [anon_sym_LBRACK] = ACTIONS(2031),
    [anon_sym_LBRACE] = ACTIONS(2031),
    [anon_sym_RBRACE] = ACTIONS(2031),
    [anon_sym_STAR] = ACTIONS(2031),
    [anon_sym_u8] = ACTIONS(2033),
    [anon_sym_i8] = ACTIONS(2033),
    [anon_sym_u16] = ACTIONS(2033),
    [anon_sym_i16] = ACTIONS(2033),
    [anon_sym_u32] = ACTIONS(2033),
    [anon_sym_i32] = ACTIONS(2033),
    [anon_sym_u64] = ACTIONS(2033),
    [anon_sym_i64] = ACTIONS(2033),
    [anon_sym_u128] = ACTIONS(2033),
    [anon_sym_i128] = ACTIONS(2033),
    [anon_sym_isize] = ACTIONS(2033),
    [anon_sym_usize] = ACTIONS(2033),
    [anon_sym_f32] = ACTIONS(2033),
    [anon_sym_f64] = ACTIONS(2033),
    [anon_sym_bool] = ACTIONS(2033),
    [anon_sym_str] = ACTIONS(2033),
    [anon_sym_char] = ACTIONS(2033),
    [anon_sym_DASH] = ACTIONS(2031),
    [anon_sym_BANG] = ACTIONS(2031),
    [anon_sym_AMP] = ACTIONS(2031),
    [anon_sym_PIPE] = ACTIONS(2031),
    [anon_sym_LT] = ACTIONS(2031),
    [anon_sym_DOT_DOT] = ACTIONS(2031),
    [anon_sym_COLON_COLON] = ACTIONS(2031),
    [anon_sym_POUND] = ACTIONS(2031),
    [anon_sym_SQUOTE] = ACTIONS(2033),
    [anon_sym_async] = ACTIONS(2033),
    [anon_sym_break] = ACTIONS(2033),
    [anon_sym_const] = ACTIONS(2033),
    [anon_sym_continue] = ACTIONS(2033),
    [anon_sym_default] = ACTIONS(2033),
    [anon_sym_enum] = ACTIONS(2033),
    [anon_sym_fn] = ACTIONS(2033),
    [anon_sym_for] = ACTIONS(2033),
    [anon_sym_gen] = ACTIONS(2033),
    [anon_sym_if] = ACTIONS(2033),
    [anon_sym_impl] = ACTIONS(2033),
    [anon_sym_let] = ACTIONS(2033),
    [anon_sym_loop] = ACTIONS(2033),
    [anon_sym_match] = ACTIONS(2033),
    [anon_sym_mod] = ACTIONS(2033),
    [anon_sym_pub] = ACTIONS(2033),
    [anon_sym_return] = ACTIONS(2033),
    [anon_sym_static] = ACTIONS(2033),
    [anon_sym_struct] = ACTIONS(2033),
    [anon_sym_trait] = ACTIONS(2033),
    [anon_sym_type] = ACTIONS(2033),
    [anon_sym_union] = ACTIONS(2033),
    [anon_sym_unsafe] = ACTIONS(2033),
    [anon_sym_use] = ACTIONS(2033),
    [anon_sym_while] = ACTIONS(2033),
    [anon_sym_extern] = ACTIONS(2033),
    [anon_sym_yield] = ACTIONS(2033),
    [anon_sym_move] = ACTIONS(2033),
    [anon_sym_try] = ACTIONS(2033),
    [sym_integer_literal] = ACTIONS(2031),
    [aux_sym_string_literal_token1] = ACTIONS(2031),
    [sym_char_literal] = ACTIONS(2031),
    [anon_sym_true] = ACTIONS(2033),
    [anon_sym_false] = ACTIONS(2033),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2033),
    [sym_super] = ACTIONS(2033),
    [sym_crate] = ACTIONS(2033),
    [sym_metavariable] = ACTIONS(2031),
    [sym__raw_string_literal_start] = ACTIONS(2031),
    [sym_float_literal] = ACTIONS(2031),
  },
  [571] = {
    [sym_line_comment] = STATE(571),
    [sym_block_comment] = STATE(571),
    [ts_builtin_sym_end] = ACTIONS(2035),
    [sym_identifier] = ACTIONS(2037),
    [anon_sym_SEMI] = ACTIONS(2035),
    [anon_sym_macro_rules_BANG] = ACTIONS(2035),
    [anon_sym_LPAREN] = ACTIONS(2035),
    [anon_sym_LBRACK] = ACTIONS(2035),
    [anon_sym_LBRACE] = ACTIONS(2035),
    [anon_sym_RBRACE] = ACTIONS(2035),
    [anon_sym_STAR] = ACTIONS(2035),
    [anon_sym_u8] = ACTIONS(2037),
    [anon_sym_i8] = ACTIONS(2037),
    [anon_sym_u16] = ACTIONS(2037),
    [anon_sym_i16] = ACTIONS(2037),
    [anon_sym_u32] = ACTIONS(2037),
    [anon_sym_i32] = ACTIONS(2037),
    [anon_sym_u64] = ACTIONS(2037),
    [anon_sym_i64] = ACTIONS(2037),
    [anon_sym_u128] = ACTIONS(2037),
    [anon_sym_i128] = ACTIONS(2037),
    [anon_sym_isize] = ACTIONS(2037),
    [anon_sym_usize] = ACTIONS(2037),
    [anon_sym_f32] = ACTIONS(2037),
    [anon_sym_f64] = ACTIONS(2037),
    [anon_sym_bool] = ACTIONS(2037),
    [anon_sym_str] = ACTIONS(2037),
    [anon_sym_char] = ACTIONS(2037),
    [anon_sym_DASH] = ACTIONS(2035),
    [anon_sym_BANG] = ACTIONS(2035),
    [anon_sym_AMP] = ACTIONS(2035),
    [anon_sym_PIPE] = ACTIONS(2035),
    [anon_sym_LT] = ACTIONS(2035),
    [anon_sym_DOT_DOT] = ACTIONS(2035),
    [anon_sym_COLON_COLON] = ACTIONS(2035),
    [anon_sym_POUND] = ACTIONS(2035),
    [anon_sym_SQUOTE] = ACTIONS(2037),
    [anon_sym_async] = ACTIONS(2037),
    [anon_sym_break] = ACTIONS(2037),
    [anon_sym_const] = ACTIONS(2037),
    [anon_sym_continue] = ACTIONS(2037),
    [anon_sym_default] = ACTIONS(2037),
    [anon_sym_enum] = ACTIONS(2037),
    [anon_sym_fn] = ACTIONS(2037),
    [anon_sym_for] = ACTIONS(2037),
    [anon_sym_gen] = ACTIONS(2037),
    [anon_sym_if] = ACTIONS(2037),
    [anon_sym_impl] = ACTIONS(2037),
    [anon_sym_let] = ACTIONS(2037),
    [anon_sym_loop] = ACTIONS(2037),
    [anon_sym_match] = ACTIONS(2037),
    [anon_sym_mod] = ACTIONS(2037),
    [anon_sym_pub] = ACTIONS(2037),
    [anon_sym_return] = ACTIONS(2037),
    [anon_sym_static] = ACTIONS(2037),
    [anon_sym_struct] = ACTIONS(2037),
    [anon_sym_trait] = ACTIONS(2037),
    [anon_sym_type] = ACTIONS(2037),
    [anon_sym_union] = ACTIONS(2037),
    [anon_sym_unsafe] = ACTIONS(2037),
    [anon_sym_use] = ACTIONS(2037),
    [anon_sym_while] = ACTIONS(2037),
    [anon_sym_extern] = ACTIONS(2037),
    [anon_sym_yield] = ACTIONS(2037),
    [anon_sym_move] = ACTIONS(2037),
    [anon_sym_try] = ACTIONS(2037),
    [sym_integer_literal] = ACTIONS(2035),
    [aux_sym_string_literal_token1] = ACTIONS(2035),
    [sym_char_literal] = ACTIONS(2035),
    [anon_sym_true] = ACTIONS(2037),
    [anon_sym_false] = ACTIONS(2037),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2037),
    [sym_super] = ACTIONS(2037),
    [sym_crate] = ACTIONS(2037),
    [sym_metavariable] = ACTIONS(2035),
    [sym__raw_string_literal_start] = ACTIONS(2035),
    [sym_float_literal] = ACTIONS(2035),
  },
  [572] = {
    [sym_line_comment] = STATE(572),
    [sym_block_comment] = STATE(572),
    [ts_builtin_sym_end] = ACTIONS(2039),
    [sym_identifier] = ACTIONS(2041),
    [anon_sym_SEMI] = ACTIONS(2039),
    [anon_sym_macro_rules_BANG] = ACTIONS(2039),
    [anon_sym_LPAREN] = ACTIONS(2039),
    [anon_sym_LBRACK] = ACTIONS(2039),
    [anon_sym_LBRACE] = ACTIONS(2039),
    [anon_sym_RBRACE] = ACTIONS(2039),
    [anon_sym_STAR] = ACTIONS(2039),
    [anon_sym_u8] = ACTIONS(2041),
    [anon_sym_i8] = ACTIONS(2041),
    [anon_sym_u16] = ACTIONS(2041),
    [anon_sym_i16] = ACTIONS(2041),
    [anon_sym_u32] = ACTIONS(2041),
    [anon_sym_i32] = ACTIONS(2041),
    [anon_sym_u64] = ACTIONS(2041),
    [anon_sym_i64] = ACTIONS(2041),
    [anon_sym_u128] = ACTIONS(2041),
    [anon_sym_i128] = ACTIONS(2041),
    [anon_sym_isize] = ACTIONS(2041),
    [anon_sym_usize] = ACTIONS(2041),
    [anon_sym_f32] = ACTIONS(2041),
    [anon_sym_f64] = ACTIONS(2041),
    [anon_sym_bool] = ACTIONS(2041),
    [anon_sym_str] = ACTIONS(2041),
    [anon_sym_char] = ACTIONS(2041),
    [anon_sym_DASH] = ACTIONS(2039),
    [anon_sym_BANG] = ACTIONS(2039),
    [anon_sym_AMP] = ACTIONS(2039),
    [anon_sym_PIPE] = ACTIONS(2039),
    [anon_sym_LT] = ACTIONS(2039),
    [anon_sym_DOT_DOT] = ACTIONS(2039),
    [anon_sym_COLON_COLON] = ACTIONS(2039),
    [anon_sym_POUND] = ACTIONS(2039),
    [anon_sym_SQUOTE] = ACTIONS(2041),
    [anon_sym_async] = ACTIONS(2041),
    [anon_sym_break] = ACTIONS(2041),
    [anon_sym_const] = ACTIONS(2041),
    [anon_sym_continue] = ACTIONS(2041),
    [anon_sym_default] = ACTIONS(2041),
    [anon_sym_enum] = ACTIONS(2041),
    [anon_sym_fn] = ACTIONS(2041),
    [anon_sym_for] = ACTIONS(2041),
    [anon_sym_gen] = ACTIONS(2041),
    [anon_sym_if] = ACTIONS(2041),
    [anon_sym_impl] = ACTIONS(2041),
    [anon_sym_let] = ACTIONS(2041),
    [anon_sym_loop] = ACTIONS(2041),
    [anon_sym_match] = ACTIONS(2041),
    [anon_sym_mod] = ACTIONS(2041),
    [anon_sym_pub] = ACTIONS(2041),
    [anon_sym_return] = ACTIONS(2041),
    [anon_sym_static] = ACTIONS(2041),
    [anon_sym_struct] = ACTIONS(2041),
    [anon_sym_trait] = ACTIONS(2041),
    [anon_sym_type] = ACTIONS(2041),
    [anon_sym_union] = ACTIONS(2041),
    [anon_sym_unsafe] = ACTIONS(2041),
    [anon_sym_use] = ACTIONS(2041),
    [anon_sym_while] = ACTIONS(2041),
    [anon_sym_extern] = ACTIONS(2041),
    [anon_sym_yield] = ACTIONS(2041),
    [anon_sym_move] = ACTIONS(2041),
    [anon_sym_try] = ACTIONS(2041),
    [sym_integer_literal] = ACTIONS(2039),
    [aux_sym_string_literal_token1] = ACTIONS(2039),
    [sym_char_literal] = ACTIONS(2039),
    [anon_sym_true] = ACTIONS(2041),
    [anon_sym_false] = ACTIONS(2041),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2041),
    [sym_super] = ACTIONS(2041),
    [sym_crate] = ACTIONS(2041),
    [sym_metavariable] = ACTIONS(2039),
    [sym__raw_string_literal_start] = ACTIONS(2039),
    [sym_float_literal] = ACTIONS(2039),
  },
  [573] = {
    [sym_line_comment] = STATE(573),
    [sym_block_comment] = STATE(573),
    [ts_builtin_sym_end] = ACTIONS(2043),
    [sym_identifier] = ACTIONS(2045),
    [anon_sym_SEMI] = ACTIONS(2043),
    [anon_sym_macro_rules_BANG] = ACTIONS(2043),
    [anon_sym_LPAREN] = ACTIONS(2043),
    [anon_sym_LBRACK] = ACTIONS(2043),
    [anon_sym_LBRACE] = ACTIONS(2043),
    [anon_sym_RBRACE] = ACTIONS(2043),
    [anon_sym_STAR] = ACTIONS(2043),
    [anon_sym_u8] = ACTIONS(2045),
    [anon_sym_i8] = ACTIONS(2045),
    [anon_sym_u16] = ACTIONS(2045),
    [anon_sym_i16] = ACTIONS(2045),
    [anon_sym_u32] = ACTIONS(2045),
    [anon_sym_i32] = ACTIONS(2045),
    [anon_sym_u64] = ACTIONS(2045),
    [anon_sym_i64] = ACTIONS(2045),
    [anon_sym_u128] = ACTIONS(2045),
    [anon_sym_i128] = ACTIONS(2045),
    [anon_sym_isize] = ACTIONS(2045),
    [anon_sym_usize] = ACTIONS(2045),
    [anon_sym_f32] = ACTIONS(2045),
    [anon_sym_f64] = ACTIONS(2045),
    [anon_sym_bool] = ACTIONS(2045),
    [anon_sym_str] = ACTIONS(2045),
    [anon_sym_char] = ACTIONS(2045),
    [anon_sym_DASH] = ACTIONS(2043),
    [anon_sym_BANG] = ACTIONS(2043),
    [anon_sym_AMP] = ACTIONS(2043),
    [anon_sym_PIPE] = ACTIONS(2043),
    [anon_sym_LT] = ACTIONS(2043),
    [anon_sym_DOT_DOT] = ACTIONS(2043),
    [anon_sym_COLON_COLON] = ACTIONS(2043),
    [anon_sym_POUND] = ACTIONS(2043),
    [anon_sym_SQUOTE] = ACTIONS(2045),
    [anon_sym_async] = ACTIONS(2045),
    [anon_sym_break] = ACTIONS(2045),
    [anon_sym_const] = ACTIONS(2045),
    [anon_sym_continue] = ACTIONS(2045),
    [anon_sym_default] = ACTIONS(2045),
    [anon_sym_enum] = ACTIONS(2045),
    [anon_sym_fn] = ACTIONS(2045),
    [anon_sym_for] = ACTIONS(2045),
    [anon_sym_gen] = ACTIONS(2045),
    [anon_sym_if] = ACTIONS(2045),
    [anon_sym_impl] = ACTIONS(2045),
    [anon_sym_let] = ACTIONS(2045),
    [anon_sym_loop] = ACTIONS(2045),
    [anon_sym_match] = ACTIONS(2045),
    [anon_sym_mod] = ACTIONS(2045),
    [anon_sym_pub] = ACTIONS(2045),
    [anon_sym_return] = ACTIONS(2045),
    [anon_sym_static] = ACTIONS(2045),
    [anon_sym_struct] = ACTIONS(2045),
    [anon_sym_trait] = ACTIONS(2045),
    [anon_sym_type] = ACTIONS(2045),
    [anon_sym_union] = ACTIONS(2045),
    [anon_sym_unsafe] = ACTIONS(2045),
    [anon_sym_use] = ACTIONS(2045),
    [anon_sym_while] = ACTIONS(2045),
    [anon_sym_extern] = ACTIONS(2045),
    [anon_sym_yield] = ACTIONS(2045),
    [anon_sym_move] = ACTIONS(2045),
    [anon_sym_try] = ACTIONS(2045),
    [sym_integer_literal] = ACTIONS(2043),
    [aux_sym_string_literal_token1] = ACTIONS(2043),
    [sym_char_literal] = ACTIONS(2043),
    [anon_sym_true] = ACTIONS(2045),
    [anon_sym_false] = ACTIONS(2045),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2045),
    [sym_super] = ACTIONS(2045),
    [sym_crate] = ACTIONS(2045),
    [sym_metavariable] = ACTIONS(2043),
    [sym__raw_string_literal_start] = ACTIONS(2043),
    [sym_float_literal] = ACTIONS(2043),
  },
  [574] = {
    [sym_line_comment] = STATE(574),
    [sym_block_comment] = STATE(574),
    [ts_builtin_sym_end] = ACTIONS(2047),
    [sym_identifier] = ACTIONS(2049),
    [anon_sym_SEMI] = ACTIONS(2047),
    [anon_sym_macro_rules_BANG] = ACTIONS(2047),
    [anon_sym_LPAREN] = ACTIONS(2047),
    [anon_sym_LBRACK] = ACTIONS(2047),
    [anon_sym_LBRACE] = ACTIONS(2047),
    [anon_sym_RBRACE] = ACTIONS(2047),
    [anon_sym_STAR] = ACTIONS(2047),
    [anon_sym_u8] = ACTIONS(2049),
    [anon_sym_i8] = ACTIONS(2049),
    [anon_sym_u16] = ACTIONS(2049),
    [anon_sym_i16] = ACTIONS(2049),
    [anon_sym_u32] = ACTIONS(2049),
    [anon_sym_i32] = ACTIONS(2049),
    [anon_sym_u64] = ACTIONS(2049),
    [anon_sym_i64] = ACTIONS(2049),
    [anon_sym_u128] = ACTIONS(2049),
    [anon_sym_i128] = ACTIONS(2049),
    [anon_sym_isize] = ACTIONS(2049),
    [anon_sym_usize] = ACTIONS(2049),
    [anon_sym_f32] = ACTIONS(2049),
    [anon_sym_f64] = ACTIONS(2049),
    [anon_sym_bool] = ACTIONS(2049),
    [anon_sym_str] = ACTIONS(2049),
    [anon_sym_char] = ACTIONS(2049),
    [anon_sym_DASH] = ACTIONS(2047),
    [anon_sym_BANG] = ACTIONS(2047),
    [anon_sym_AMP] = ACTIONS(2047),
    [anon_sym_PIPE] = ACTIONS(2047),
    [anon_sym_LT] = ACTIONS(2047),
    [anon_sym_DOT_DOT] = ACTIONS(2047),
    [anon_sym_COLON_COLON] = ACTIONS(2047),
    [anon_sym_POUND] = ACTIONS(2047),
    [anon_sym_SQUOTE] = ACTIONS(2049),
    [anon_sym_async] = ACTIONS(2049),
    [anon_sym_break] = ACTIONS(2049),
    [anon_sym_const] = ACTIONS(2049),
    [anon_sym_continue] = ACTIONS(2049),
    [anon_sym_default] = ACTIONS(2049),
    [anon_sym_enum] = ACTIONS(2049),
    [anon_sym_fn] = ACTIONS(2049),
    [anon_sym_for] = ACTIONS(2049),
    [anon_sym_gen] = ACTIONS(2049),
    [anon_sym_if] = ACTIONS(2049),
    [anon_sym_impl] = ACTIONS(2049),
    [anon_sym_let] = ACTIONS(2049),
    [anon_sym_loop] = ACTIONS(2049),
    [anon_sym_match] = ACTIONS(2049),
    [anon_sym_mod] = ACTIONS(2049),
    [anon_sym_pub] = ACTIONS(2049),
    [anon_sym_return] = ACTIONS(2049),
    [anon_sym_static] = ACTIONS(2049),
    [anon_sym_struct] = ACTIONS(2049),
    [anon_sym_trait] = ACTIONS(2049),
    [anon_sym_type] = ACTIONS(2049),
    [anon_sym_union] = ACTIONS(2049),
    [anon_sym_unsafe] = ACTIONS(2049),
    [anon_sym_use] = ACTIONS(2049),
    [anon_sym_while] = ACTIONS(2049),
    [anon_sym_extern] = ACTIONS(2049),
    [anon_sym_yield] = ACTIONS(2049),
    [anon_sym_move] = ACTIONS(2049),
    [anon_sym_try] = ACTIONS(2049),
    [sym_integer_literal] = ACTIONS(2047),
    [aux_sym_string_literal_token1] = ACTIONS(2047),
    [sym_char_literal] = ACTIONS(2047),
    [anon_sym_true] = ACTIONS(2049),
    [anon_sym_false] = ACTIONS(2049),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2049),
    [sym_super] = ACTIONS(2049),
    [sym_crate] = ACTIONS(2049),
    [sym_metavariable] = ACTIONS(2047),
    [sym__raw_string_literal_start] = ACTIONS(2047),
    [sym_float_literal] = ACTIONS(2047),
  },
  [575] = {
    [sym_line_comment] = STATE(575),
    [sym_block_comment] = STATE(575),
    [ts_builtin_sym_end] = ACTIONS(2051),
    [sym_identifier] = ACTIONS(2053),
    [anon_sym_SEMI] = ACTIONS(2051),
    [anon_sym_macro_rules_BANG] = ACTIONS(2051),
    [anon_sym_LPAREN] = ACTIONS(2051),
    [anon_sym_LBRACK] = ACTIONS(2051),
    [anon_sym_LBRACE] = ACTIONS(2051),
    [anon_sym_RBRACE] = ACTIONS(2051),
    [anon_sym_STAR] = ACTIONS(2051),
    [anon_sym_u8] = ACTIONS(2053),
    [anon_sym_i8] = ACTIONS(2053),
    [anon_sym_u16] = ACTIONS(2053),
    [anon_sym_i16] = ACTIONS(2053),
    [anon_sym_u32] = ACTIONS(2053),
    [anon_sym_i32] = ACTIONS(2053),
    [anon_sym_u64] = ACTIONS(2053),
    [anon_sym_i64] = ACTIONS(2053),
    [anon_sym_u128] = ACTIONS(2053),
    [anon_sym_i128] = ACTIONS(2053),
    [anon_sym_isize] = ACTIONS(2053),
    [anon_sym_usize] = ACTIONS(2053),
    [anon_sym_f32] = ACTIONS(2053),
    [anon_sym_f64] = ACTIONS(2053),
    [anon_sym_bool] = ACTIONS(2053),
    [anon_sym_str] = ACTIONS(2053),
    [anon_sym_char] = ACTIONS(2053),
    [anon_sym_DASH] = ACTIONS(2051),
    [anon_sym_BANG] = ACTIONS(2051),
    [anon_sym_AMP] = ACTIONS(2051),
    [anon_sym_PIPE] = ACTIONS(2051),
    [anon_sym_LT] = ACTIONS(2051),
    [anon_sym_DOT_DOT] = ACTIONS(2051),
    [anon_sym_COLON_COLON] = ACTIONS(2051),
    [anon_sym_POUND] = ACTIONS(2051),
    [anon_sym_SQUOTE] = ACTIONS(2053),
    [anon_sym_async] = ACTIONS(2053),
    [anon_sym_break] = ACTIONS(2053),
    [anon_sym_const] = ACTIONS(2053),
    [anon_sym_continue] = ACTIONS(2053),
    [anon_sym_default] = ACTIONS(2053),
    [anon_sym_enum] = ACTIONS(2053),
    [anon_sym_fn] = ACTIONS(2053),
    [anon_sym_for] = ACTIONS(2053),
    [anon_sym_gen] = ACTIONS(2053),
    [anon_sym_if] = ACTIONS(2053),
    [anon_sym_impl] = ACTIONS(2053),
    [anon_sym_let] = ACTIONS(2053),
    [anon_sym_loop] = ACTIONS(2053),
    [anon_sym_match] = ACTIONS(2053),
    [anon_sym_mod] = ACTIONS(2053),
    [anon_sym_pub] = ACTIONS(2053),
    [anon_sym_return] = ACTIONS(2053),
    [anon_sym_static] = ACTIONS(2053),
    [anon_sym_struct] = ACTIONS(2053),
    [anon_sym_trait] = ACTIONS(2053),
    [anon_sym_type] = ACTIONS(2053),
    [anon_sym_union] = ACTIONS(2053),
    [anon_sym_unsafe] = ACTIONS(2053),
    [anon_sym_use] = ACTIONS(2053),
    [anon_sym_while] = ACTIONS(2053),
    [anon_sym_extern] = ACTIONS(2053),
    [anon_sym_yield] = ACTIONS(2053),
    [anon_sym_move] = ACTIONS(2053),
    [anon_sym_try] = ACTIONS(2053),
    [sym_integer_literal] = ACTIONS(2051),
    [aux_sym_string_literal_token1] = ACTIONS(2051),
    [sym_char_literal] = ACTIONS(2051),
    [anon_sym_true] = ACTIONS(2053),
    [anon_sym_false] = ACTIONS(2053),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2053),
    [sym_super] = ACTIONS(2053),
    [sym_crate] = ACTIONS(2053),
    [sym_metavariable] = ACTIONS(2051),
    [sym__raw_string_literal_start] = ACTIONS(2051),
    [sym_float_literal] = ACTIONS(2051),
  },
  [576] = {
    [sym_line_comment] = STATE(576),
    [sym_block_comment] = STATE(576),
    [ts_builtin_sym_end] = ACTIONS(2055),
    [sym_identifier] = ACTIONS(2057),
    [anon_sym_SEMI] = ACTIONS(2055),
    [anon_sym_macro_rules_BANG] = ACTIONS(2055),
    [anon_sym_LPAREN] = ACTIONS(2055),
    [anon_sym_LBRACK] = ACTIONS(2055),
    [anon_sym_LBRACE] = ACTIONS(2055),
    [anon_sym_RBRACE] = ACTIONS(2055),
    [anon_sym_STAR] = ACTIONS(2055),
    [anon_sym_u8] = ACTIONS(2057),
    [anon_sym_i8] = ACTIONS(2057),
    [anon_sym_u16] = ACTIONS(2057),
    [anon_sym_i16] = ACTIONS(2057),
    [anon_sym_u32] = ACTIONS(2057),
    [anon_sym_i32] = ACTIONS(2057),
    [anon_sym_u64] = ACTIONS(2057),
    [anon_sym_i64] = ACTIONS(2057),
    [anon_sym_u128] = ACTIONS(2057),
    [anon_sym_i128] = ACTIONS(2057),
    [anon_sym_isize] = ACTIONS(2057),
    [anon_sym_usize] = ACTIONS(2057),
    [anon_sym_f32] = ACTIONS(2057),
    [anon_sym_f64] = ACTIONS(2057),
    [anon_sym_bool] = ACTIONS(2057),
    [anon_sym_str] = ACTIONS(2057),
    [anon_sym_char] = ACTIONS(2057),
    [anon_sym_DASH] = ACTIONS(2055),
    [anon_sym_BANG] = ACTIONS(2055),
    [anon_sym_AMP] = ACTIONS(2055),
    [anon_sym_PIPE] = ACTIONS(2055),
    [anon_sym_LT] = ACTIONS(2055),
    [anon_sym_DOT_DOT] = ACTIONS(2055),
    [anon_sym_COLON_COLON] = ACTIONS(2055),
    [anon_sym_POUND] = ACTIONS(2055),
    [anon_sym_SQUOTE] = ACTIONS(2057),
    [anon_sym_async] = ACTIONS(2057),
    [anon_sym_break] = ACTIONS(2057),
    [anon_sym_const] = ACTIONS(2057),
    [anon_sym_continue] = ACTIONS(2057),
    [anon_sym_default] = ACTIONS(2057),
    [anon_sym_enum] = ACTIONS(2057),
    [anon_sym_fn] = ACTIONS(2057),
    [anon_sym_for] = ACTIONS(2057),
    [anon_sym_gen] = ACTIONS(2057),
    [anon_sym_if] = ACTIONS(2057),
    [anon_sym_impl] = ACTIONS(2057),
    [anon_sym_let] = ACTIONS(2057),
    [anon_sym_loop] = ACTIONS(2057),
    [anon_sym_match] = ACTIONS(2057),
    [anon_sym_mod] = ACTIONS(2057),
    [anon_sym_pub] = ACTIONS(2057),
    [anon_sym_return] = ACTIONS(2057),
    [anon_sym_static] = ACTIONS(2057),
    [anon_sym_struct] = ACTIONS(2057),
    [anon_sym_trait] = ACTIONS(2057),
    [anon_sym_type] = ACTIONS(2057),
    [anon_sym_union] = ACTIONS(2057),
    [anon_sym_unsafe] = ACTIONS(2057),
    [anon_sym_use] = ACTIONS(2057),
    [anon_sym_while] = ACTIONS(2057),
    [anon_sym_extern] = ACTIONS(2057),
    [anon_sym_yield] = ACTIONS(2057),
    [anon_sym_move] = ACTIONS(2057),
    [anon_sym_try] = ACTIONS(2057),
    [sym_integer_literal] = ACTIONS(2055),
    [aux_sym_string_literal_token1] = ACTIONS(2055),
    [sym_char_literal] = ACTIONS(2055),
    [anon_sym_true] = ACTIONS(2057),
    [anon_sym_false] = ACTIONS(2057),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2057),
    [sym_super] = ACTIONS(2057),
    [sym_crate] = ACTIONS(2057),
    [sym_metavariable] = ACTIONS(2055),
    [sym__raw_string_literal_start] = ACTIONS(2055),
    [sym_float_literal] = ACTIONS(2055),
  },
  [577] = {
    [sym_line_comment] = STATE(577),
    [sym_block_comment] = STATE(577),
    [ts_builtin_sym_end] = ACTIONS(2059),
    [sym_identifier] = ACTIONS(2061),
    [anon_sym_SEMI] = ACTIONS(2059),
    [anon_sym_macro_rules_BANG] = ACTIONS(2059),
    [anon_sym_LPAREN] = ACTIONS(2059),
    [anon_sym_LBRACK] = ACTIONS(2059),
    [anon_sym_LBRACE] = ACTIONS(2059),
    [anon_sym_RBRACE] = ACTIONS(2059),
    [anon_sym_STAR] = ACTIONS(2059),
    [anon_sym_u8] = ACTIONS(2061),
    [anon_sym_i8] = ACTIONS(2061),
    [anon_sym_u16] = ACTIONS(2061),
    [anon_sym_i16] = ACTIONS(2061),
    [anon_sym_u32] = ACTIONS(2061),
    [anon_sym_i32] = ACTIONS(2061),
    [anon_sym_u64] = ACTIONS(2061),
    [anon_sym_i64] = ACTIONS(2061),
    [anon_sym_u128] = ACTIONS(2061),
    [anon_sym_i128] = ACTIONS(2061),
    [anon_sym_isize] = ACTIONS(2061),
    [anon_sym_usize] = ACTIONS(2061),
    [anon_sym_f32] = ACTIONS(2061),
    [anon_sym_f64] = ACTIONS(2061),
    [anon_sym_bool] = ACTIONS(2061),
    [anon_sym_str] = ACTIONS(2061),
    [anon_sym_char] = ACTIONS(2061),
    [anon_sym_DASH] = ACTIONS(2059),
    [anon_sym_BANG] = ACTIONS(2059),
    [anon_sym_AMP] = ACTIONS(2059),
    [anon_sym_PIPE] = ACTIONS(2059),
    [anon_sym_LT] = ACTIONS(2059),
    [anon_sym_DOT_DOT] = ACTIONS(2059),
    [anon_sym_COLON_COLON] = ACTIONS(2059),
    [anon_sym_POUND] = ACTIONS(2059),
    [anon_sym_SQUOTE] = ACTIONS(2061),
    [anon_sym_async] = ACTIONS(2061),
    [anon_sym_break] = ACTIONS(2061),
    [anon_sym_const] = ACTIONS(2061),
    [anon_sym_continue] = ACTIONS(2061),
    [anon_sym_default] = ACTIONS(2061),
    [anon_sym_enum] = ACTIONS(2061),
    [anon_sym_fn] = ACTIONS(2061),
    [anon_sym_for] = ACTIONS(2061),
    [anon_sym_gen] = ACTIONS(2061),
    [anon_sym_if] = ACTIONS(2061),
    [anon_sym_impl] = ACTIONS(2061),
    [anon_sym_let] = ACTIONS(2061),
    [anon_sym_loop] = ACTIONS(2061),
    [anon_sym_match] = ACTIONS(2061),
    [anon_sym_mod] = ACTIONS(2061),
    [anon_sym_pub] = ACTIONS(2061),
    [anon_sym_return] = ACTIONS(2061),
    [anon_sym_static] = ACTIONS(2061),
    [anon_sym_struct] = ACTIONS(2061),
    [anon_sym_trait] = ACTIONS(2061),
    [anon_sym_type] = ACTIONS(2061),
    [anon_sym_union] = ACTIONS(2061),
    [anon_sym_unsafe] = ACTIONS(2061),
    [anon_sym_use] = ACTIONS(2061),
    [anon_sym_while] = ACTIONS(2061),
    [anon_sym_extern] = ACTIONS(2061),
    [anon_sym_yield] = ACTIONS(2061),
    [anon_sym_move] = ACTIONS(2061),
    [anon_sym_try] = ACTIONS(2061),
    [sym_integer_literal] = ACTIONS(2059),
    [aux_sym_string_literal_token1] = ACTIONS(2059),
    [sym_char_literal] = ACTIONS(2059),
    [anon_sym_true] = ACTIONS(2061),
    [anon_sym_false] = ACTIONS(2061),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2061),
    [sym_super] = ACTIONS(2061),
    [sym_crate] = ACTIONS(2061),
    [sym_metavariable] = ACTIONS(2059),
    [sym__raw_string_literal_start] = ACTIONS(2059),
    [sym_float_literal] = ACTIONS(2059),
  },
  [578] = {
    [sym_line_comment] = STATE(578),
    [sym_block_comment] = STATE(578),
    [ts_builtin_sym_end] = ACTIONS(2063),
    [sym_identifier] = ACTIONS(2065),
    [anon_sym_SEMI] = ACTIONS(2063),
    [anon_sym_macro_rules_BANG] = ACTIONS(2063),
    [anon_sym_LPAREN] = ACTIONS(2063),
    [anon_sym_LBRACK] = ACTIONS(2063),
    [anon_sym_LBRACE] = ACTIONS(2063),
    [anon_sym_RBRACE] = ACTIONS(2063),
    [anon_sym_STAR] = ACTIONS(2063),
    [anon_sym_u8] = ACTIONS(2065),
    [anon_sym_i8] = ACTIONS(2065),
    [anon_sym_u16] = ACTIONS(2065),
    [anon_sym_i16] = ACTIONS(2065),
    [anon_sym_u32] = ACTIONS(2065),
    [anon_sym_i32] = ACTIONS(2065),
    [anon_sym_u64] = ACTIONS(2065),
    [anon_sym_i64] = ACTIONS(2065),
    [anon_sym_u128] = ACTIONS(2065),
    [anon_sym_i128] = ACTIONS(2065),
    [anon_sym_isize] = ACTIONS(2065),
    [anon_sym_usize] = ACTIONS(2065),
    [anon_sym_f32] = ACTIONS(2065),
    [anon_sym_f64] = ACTIONS(2065),
    [anon_sym_bool] = ACTIONS(2065),
    [anon_sym_str] = ACTIONS(2065),
    [anon_sym_char] = ACTIONS(2065),
    [anon_sym_DASH] = ACTIONS(2063),
    [anon_sym_BANG] = ACTIONS(2063),
    [anon_sym_AMP] = ACTIONS(2063),
    [anon_sym_PIPE] = ACTIONS(2063),
    [anon_sym_LT] = ACTIONS(2063),
    [anon_sym_DOT_DOT] = ACTIONS(2063),
    [anon_sym_COLON_COLON] = ACTIONS(2063),
    [anon_sym_POUND] = ACTIONS(2063),
    [anon_sym_SQUOTE] = ACTIONS(2065),
    [anon_sym_async] = ACTIONS(2065),
    [anon_sym_break] = ACTIONS(2065),
    [anon_sym_const] = ACTIONS(2065),
    [anon_sym_continue] = ACTIONS(2065),
    [anon_sym_default] = ACTIONS(2065),
    [anon_sym_enum] = ACTIONS(2065),
    [anon_sym_fn] = ACTIONS(2065),
    [anon_sym_for] = ACTIONS(2065),
    [anon_sym_gen] = ACTIONS(2065),
    [anon_sym_if] = ACTIONS(2065),
    [anon_sym_impl] = ACTIONS(2065),
    [anon_sym_let] = ACTIONS(2065),
    [anon_sym_loop] = ACTIONS(2065),
    [anon_sym_match] = ACTIONS(2065),
    [anon_sym_mod] = ACTIONS(2065),
    [anon_sym_pub] = ACTIONS(2065),
    [anon_sym_return] = ACTIONS(2065),
    [anon_sym_static] = ACTIONS(2065),
    [anon_sym_struct] = ACTIONS(2065),
    [anon_sym_trait] = ACTIONS(2065),
    [anon_sym_type] = ACTIONS(2065),
    [anon_sym_union] = ACTIONS(2065),
    [anon_sym_unsafe] = ACTIONS(2065),
    [anon_sym_use] = ACTIONS(2065),
    [anon_sym_while] = ACTIONS(2065),
    [anon_sym_extern] = ACTIONS(2065),
    [anon_sym_yield] = ACTIONS(2065),
    [anon_sym_move] = ACTIONS(2065),
    [anon_sym_try] = ACTIONS(2065),
    [sym_integer_literal] = ACTIONS(2063),
    [aux_sym_string_literal_token1] = ACTIONS(2063),
    [sym_char_literal] = ACTIONS(2063),
    [anon_sym_true] = ACTIONS(2065),
    [anon_sym_false] = ACTIONS(2065),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2065),
    [sym_super] = ACTIONS(2065),
    [sym_crate] = ACTIONS(2065),
    [sym_metavariable] = ACTIONS(2063),
    [sym__raw_string_literal_start] = ACTIONS(2063),
    [sym_float_literal] = ACTIONS(2063),
  },
  [579] = {
    [sym_line_comment] = STATE(579),
    [sym_block_comment] = STATE(579),
    [ts_builtin_sym_end] = ACTIONS(2067),
    [sym_identifier] = ACTIONS(2069),
    [anon_sym_SEMI] = ACTIONS(2067),
    [anon_sym_macro_rules_BANG] = ACTIONS(2067),
    [anon_sym_LPAREN] = ACTIONS(2067),
    [anon_sym_LBRACK] = ACTIONS(2067),
    [anon_sym_LBRACE] = ACTIONS(2067),
    [anon_sym_RBRACE] = ACTIONS(2067),
    [anon_sym_STAR] = ACTIONS(2067),
    [anon_sym_u8] = ACTIONS(2069),
    [anon_sym_i8] = ACTIONS(2069),
    [anon_sym_u16] = ACTIONS(2069),
    [anon_sym_i16] = ACTIONS(2069),
    [anon_sym_u32] = ACTIONS(2069),
    [anon_sym_i32] = ACTIONS(2069),
    [anon_sym_u64] = ACTIONS(2069),
    [anon_sym_i64] = ACTIONS(2069),
    [anon_sym_u128] = ACTIONS(2069),
    [anon_sym_i128] = ACTIONS(2069),
    [anon_sym_isize] = ACTIONS(2069),
    [anon_sym_usize] = ACTIONS(2069),
    [anon_sym_f32] = ACTIONS(2069),
    [anon_sym_f64] = ACTIONS(2069),
    [anon_sym_bool] = ACTIONS(2069),
    [anon_sym_str] = ACTIONS(2069),
    [anon_sym_char] = ACTIONS(2069),
    [anon_sym_DASH] = ACTIONS(2067),
    [anon_sym_BANG] = ACTIONS(2067),
    [anon_sym_AMP] = ACTIONS(2067),
    [anon_sym_PIPE] = ACTIONS(2067),
    [anon_sym_LT] = ACTIONS(2067),
    [anon_sym_DOT_DOT] = ACTIONS(2067),
    [anon_sym_COLON_COLON] = ACTIONS(2067),
    [anon_sym_POUND] = ACTIONS(2067),
    [anon_sym_SQUOTE] = ACTIONS(2069),
    [anon_sym_async] = ACTIONS(2069),
    [anon_sym_break] = ACTIONS(2069),
    [anon_sym_const] = ACTIONS(2069),
    [anon_sym_continue] = ACTIONS(2069),
    [anon_sym_default] = ACTIONS(2069),
    [anon_sym_enum] = ACTIONS(2069),
    [anon_sym_fn] = ACTIONS(2069),
    [anon_sym_for] = ACTIONS(2069),
    [anon_sym_gen] = ACTIONS(2069),
    [anon_sym_if] = ACTIONS(2069),
    [anon_sym_impl] = ACTIONS(2069),
    [anon_sym_let] = ACTIONS(2069),
    [anon_sym_loop] = ACTIONS(2069),
    [anon_sym_match] = ACTIONS(2069),
    [anon_sym_mod] = ACTIONS(2069),
    [anon_sym_pub] = ACTIONS(2069),
    [anon_sym_return] = ACTIONS(2069),
    [anon_sym_static] = ACTIONS(2069),
    [anon_sym_struct] = ACTIONS(2069),
    [anon_sym_trait] = ACTIONS(2069),
    [anon_sym_type] = ACTIONS(2069),
    [anon_sym_union] = ACTIONS(2069),
    [anon_sym_unsafe] = ACTIONS(2069),
    [anon_sym_use] = ACTIONS(2069),
    [anon_sym_while] = ACTIONS(2069),
    [anon_sym_extern] = ACTIONS(2069),
    [anon_sym_yield] = ACTIONS(2069),
    [anon_sym_move] = ACTIONS(2069),
    [anon_sym_try] = ACTIONS(2069),
    [sym_integer_literal] = ACTIONS(2067),
    [aux_sym_string_literal_token1] = ACTIONS(2067),
    [sym_char_literal] = ACTIONS(2067),
    [anon_sym_true] = ACTIONS(2069),
    [anon_sym_false] = ACTIONS(2069),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2069),
    [sym_super] = ACTIONS(2069),
    [sym_crate] = ACTIONS(2069),
    [sym_metavariable] = ACTIONS(2067),
    [sym__raw_string_literal_start] = ACTIONS(2067),
    [sym_float_literal] = ACTIONS(2067),
  },
  [580] = {
    [sym_line_comment] = STATE(580),
    [sym_block_comment] = STATE(580),
    [ts_builtin_sym_end] = ACTIONS(2071),
    [sym_identifier] = ACTIONS(2073),
    [anon_sym_SEMI] = ACTIONS(2071),
    [anon_sym_macro_rules_BANG] = ACTIONS(2071),
    [anon_sym_LPAREN] = ACTIONS(2071),
    [anon_sym_LBRACK] = ACTIONS(2071),
    [anon_sym_LBRACE] = ACTIONS(2071),
    [anon_sym_RBRACE] = ACTIONS(2071),
    [anon_sym_STAR] = ACTIONS(2071),
    [anon_sym_u8] = ACTIONS(2073),
    [anon_sym_i8] = ACTIONS(2073),
    [anon_sym_u16] = ACTIONS(2073),
    [anon_sym_i16] = ACTIONS(2073),
    [anon_sym_u32] = ACTIONS(2073),
    [anon_sym_i32] = ACTIONS(2073),
    [anon_sym_u64] = ACTIONS(2073),
    [anon_sym_i64] = ACTIONS(2073),
    [anon_sym_u128] = ACTIONS(2073),
    [anon_sym_i128] = ACTIONS(2073),
    [anon_sym_isize] = ACTIONS(2073),
    [anon_sym_usize] = ACTIONS(2073),
    [anon_sym_f32] = ACTIONS(2073),
    [anon_sym_f64] = ACTIONS(2073),
    [anon_sym_bool] = ACTIONS(2073),
    [anon_sym_str] = ACTIONS(2073),
    [anon_sym_char] = ACTIONS(2073),
    [anon_sym_DASH] = ACTIONS(2071),
    [anon_sym_BANG] = ACTIONS(2071),
    [anon_sym_AMP] = ACTIONS(2071),
    [anon_sym_PIPE] = ACTIONS(2071),
    [anon_sym_LT] = ACTIONS(2071),
    [anon_sym_DOT_DOT] = ACTIONS(2071),
    [anon_sym_COLON_COLON] = ACTIONS(2071),
    [anon_sym_POUND] = ACTIONS(2071),
    [anon_sym_SQUOTE] = ACTIONS(2073),
    [anon_sym_async] = ACTIONS(2073),
    [anon_sym_break] = ACTIONS(2073),
    [anon_sym_const] = ACTIONS(2073),
    [anon_sym_continue] = ACTIONS(2073),
    [anon_sym_default] = ACTIONS(2073),
    [anon_sym_enum] = ACTIONS(2073),
    [anon_sym_fn] = ACTIONS(2073),
    [anon_sym_for] = ACTIONS(2073),
    [anon_sym_gen] = ACTIONS(2073),
    [anon_sym_if] = ACTIONS(2073),
    [anon_sym_impl] = ACTIONS(2073),
    [anon_sym_let] = ACTIONS(2073),
    [anon_sym_loop] = ACTIONS(2073),
    [anon_sym_match] = ACTIONS(2073),
    [anon_sym_mod] = ACTIONS(2073),
    [anon_sym_pub] = ACTIONS(2073),
    [anon_sym_return] = ACTIONS(2073),
    [anon_sym_static] = ACTIONS(2073),
    [anon_sym_struct] = ACTIONS(2073),
    [anon_sym_trait] = ACTIONS(2073),
    [anon_sym_type] = ACTIONS(2073),
    [anon_sym_union] = ACTIONS(2073),
    [anon_sym_unsafe] = ACTIONS(2073),
    [anon_sym_use] = ACTIONS(2073),
    [anon_sym_while] = ACTIONS(2073),
    [anon_sym_extern] = ACTIONS(2073),
    [anon_sym_yield] = ACTIONS(2073),
    [anon_sym_move] = ACTIONS(2073),
    [anon_sym_try] = ACTIONS(2073),
    [sym_integer_literal] = ACTIONS(2071),
    [aux_sym_string_literal_token1] = ACTIONS(2071),
    [sym_char_literal] = ACTIONS(2071),
    [anon_sym_true] = ACTIONS(2073),
    [anon_sym_false] = ACTIONS(2073),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2073),
    [sym_super] = ACTIONS(2073),
    [sym_crate] = ACTIONS(2073),
    [sym_metavariable] = ACTIONS(2071),
    [sym__raw_string_literal_start] = ACTIONS(2071),
    [sym_float_literal] = ACTIONS(2071),
  },
  [581] = {
    [sym_line_comment] = STATE(581),
    [sym_block_comment] = STATE(581),
    [ts_builtin_sym_end] = ACTIONS(2075),
    [sym_identifier] = ACTIONS(2077),
    [anon_sym_SEMI] = ACTIONS(2075),
    [anon_sym_macro_rules_BANG] = ACTIONS(2075),
    [anon_sym_LPAREN] = ACTIONS(2075),
    [anon_sym_LBRACK] = ACTIONS(2075),
    [anon_sym_LBRACE] = ACTIONS(2075),
    [anon_sym_RBRACE] = ACTIONS(2075),
    [anon_sym_STAR] = ACTIONS(2075),
    [anon_sym_u8] = ACTIONS(2077),
    [anon_sym_i8] = ACTIONS(2077),
    [anon_sym_u16] = ACTIONS(2077),
    [anon_sym_i16] = ACTIONS(2077),
    [anon_sym_u32] = ACTIONS(2077),
    [anon_sym_i32] = ACTIONS(2077),
    [anon_sym_u64] = ACTIONS(2077),
    [anon_sym_i64] = ACTIONS(2077),
    [anon_sym_u128] = ACTIONS(2077),
    [anon_sym_i128] = ACTIONS(2077),
    [anon_sym_isize] = ACTIONS(2077),
    [anon_sym_usize] = ACTIONS(2077),
    [anon_sym_f32] = ACTIONS(2077),
    [anon_sym_f64] = ACTIONS(2077),
    [anon_sym_bool] = ACTIONS(2077),
    [anon_sym_str] = ACTIONS(2077),
    [anon_sym_char] = ACTIONS(2077),
    [anon_sym_DASH] = ACTIONS(2075),
    [anon_sym_BANG] = ACTIONS(2075),
    [anon_sym_AMP] = ACTIONS(2075),
    [anon_sym_PIPE] = ACTIONS(2075),
    [anon_sym_LT] = ACTIONS(2075),
    [anon_sym_DOT_DOT] = ACTIONS(2075),
    [anon_sym_COLON_COLON] = ACTIONS(2075),
    [anon_sym_POUND] = ACTIONS(2075),
    [anon_sym_SQUOTE] = ACTIONS(2077),
    [anon_sym_async] = ACTIONS(2077),
    [anon_sym_break] = ACTIONS(2077),
    [anon_sym_const] = ACTIONS(2077),
    [anon_sym_continue] = ACTIONS(2077),
    [anon_sym_default] = ACTIONS(2077),
    [anon_sym_enum] = ACTIONS(2077),
    [anon_sym_fn] = ACTIONS(2077),
    [anon_sym_for] = ACTIONS(2077),
    [anon_sym_gen] = ACTIONS(2077),
    [anon_sym_if] = ACTIONS(2077),
    [anon_sym_impl] = ACTIONS(2077),
    [anon_sym_let] = ACTIONS(2077),
    [anon_sym_loop] = ACTIONS(2077),
    [anon_sym_match] = ACTIONS(2077),
    [anon_sym_mod] = ACTIONS(2077),
    [anon_sym_pub] = ACTIONS(2077),
    [anon_sym_return] = ACTIONS(2077),
    [anon_sym_static] = ACTIONS(2077),
    [anon_sym_struct] = ACTIONS(2077),
    [anon_sym_trait] = ACTIONS(2077),
    [anon_sym_type] = ACTIONS(2077),
    [anon_sym_union] = ACTIONS(2077),
    [anon_sym_unsafe] = ACTIONS(2077),
    [anon_sym_use] = ACTIONS(2077),
    [anon_sym_while] = ACTIONS(2077),
    [anon_sym_extern] = ACTIONS(2077),
    [anon_sym_yield] = ACTIONS(2077),
    [anon_sym_move] = ACTIONS(2077),
    [anon_sym_try] = ACTIONS(2077),
    [sym_integer_literal] = ACTIONS(2075),
    [aux_sym_string_literal_token1] = ACTIONS(2075),
    [sym_char_literal] = ACTIONS(2075),
    [anon_sym_true] = ACTIONS(2077),
    [anon_sym_false] = ACTIONS(2077),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2077),
    [sym_super] = ACTIONS(2077),
    [sym_crate] = ACTIONS(2077),
    [sym_metavariable] = ACTIONS(2075),
    [sym__raw_string_literal_start] = ACTIONS(2075),
    [sym_float_literal] = ACTIONS(2075),
  },
  [582] = {
    [sym_line_comment] = STATE(582),
    [sym_block_comment] = STATE(582),
    [ts_builtin_sym_end] = ACTIONS(2079),
    [sym_identifier] = ACTIONS(2081),
    [anon_sym_SEMI] = ACTIONS(2079),
    [anon_sym_macro_rules_BANG] = ACTIONS(2079),
    [anon_sym_LPAREN] = ACTIONS(2079),
    [anon_sym_LBRACK] = ACTIONS(2079),
    [anon_sym_LBRACE] = ACTIONS(2079),
    [anon_sym_RBRACE] = ACTIONS(2079),
    [anon_sym_STAR] = ACTIONS(2079),
    [anon_sym_u8] = ACTIONS(2081),
    [anon_sym_i8] = ACTIONS(2081),
    [anon_sym_u16] = ACTIONS(2081),
    [anon_sym_i16] = ACTIONS(2081),
    [anon_sym_u32] = ACTIONS(2081),
    [anon_sym_i32] = ACTIONS(2081),
    [anon_sym_u64] = ACTIONS(2081),
    [anon_sym_i64] = ACTIONS(2081),
    [anon_sym_u128] = ACTIONS(2081),
    [anon_sym_i128] = ACTIONS(2081),
    [anon_sym_isize] = ACTIONS(2081),
    [anon_sym_usize] = ACTIONS(2081),
    [anon_sym_f32] = ACTIONS(2081),
    [anon_sym_f64] = ACTIONS(2081),
    [anon_sym_bool] = ACTIONS(2081),
    [anon_sym_str] = ACTIONS(2081),
    [anon_sym_char] = ACTIONS(2081),
    [anon_sym_DASH] = ACTIONS(2079),
    [anon_sym_BANG] = ACTIONS(2079),
    [anon_sym_AMP] = ACTIONS(2079),
    [anon_sym_PIPE] = ACTIONS(2079),
    [anon_sym_LT] = ACTIONS(2079),
    [anon_sym_DOT_DOT] = ACTIONS(2079),
    [anon_sym_COLON_COLON] = ACTIONS(2079),
    [anon_sym_POUND] = ACTIONS(2079),
    [anon_sym_SQUOTE] = ACTIONS(2081),
    [anon_sym_async] = ACTIONS(2081),
    [anon_sym_break] = ACTIONS(2081),
    [anon_sym_const] = ACTIONS(2081),
    [anon_sym_continue] = ACTIONS(2081),
    [anon_sym_default] = ACTIONS(2081),
    [anon_sym_enum] = ACTIONS(2081),
    [anon_sym_fn] = ACTIONS(2081),
    [anon_sym_for] = ACTIONS(2081),
    [anon_sym_gen] = ACTIONS(2081),
    [anon_sym_if] = ACTIONS(2081),
    [anon_sym_impl] = ACTIONS(2081),
    [anon_sym_let] = ACTIONS(2081),
    [anon_sym_loop] = ACTIONS(2081),
    [anon_sym_match] = ACTIONS(2081),
    [anon_sym_mod] = ACTIONS(2081),
    [anon_sym_pub] = ACTIONS(2081),
    [anon_sym_return] = ACTIONS(2081),
    [anon_sym_static] = ACTIONS(2081),
    [anon_sym_struct] = ACTIONS(2081),
    [anon_sym_trait] = ACTIONS(2081),
    [anon_sym_type] = ACTIONS(2081),
    [anon_sym_union] = ACTIONS(2081),
    [anon_sym_unsafe] = ACTIONS(2081),
    [anon_sym_use] = ACTIONS(2081),
    [anon_sym_while] = ACTIONS(2081),
    [anon_sym_extern] = ACTIONS(2081),
    [anon_sym_yield] = ACTIONS(2081),
    [anon_sym_move] = ACTIONS(2081),
    [anon_sym_try] = ACTIONS(2081),
    [sym_integer_literal] = ACTIONS(2079),
    [aux_sym_string_literal_token1] = ACTIONS(2079),
    [sym_char_literal] = ACTIONS(2079),
    [anon_sym_true] = ACTIONS(2081),
    [anon_sym_false] = ACTIONS(2081),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2081),
    [sym_super] = ACTIONS(2081),
    [sym_crate] = ACTIONS(2081),
    [sym_metavariable] = ACTIONS(2079),
    [sym__raw_string_literal_start] = ACTIONS(2079),
    [sym_float_literal] = ACTIONS(2079),
  },
  [583] = {
    [sym_line_comment] = STATE(583),
    [sym_block_comment] = STATE(583),
    [ts_builtin_sym_end] = ACTIONS(2083),
    [sym_identifier] = ACTIONS(2085),
    [anon_sym_SEMI] = ACTIONS(2083),
    [anon_sym_macro_rules_BANG] = ACTIONS(2083),
    [anon_sym_LPAREN] = ACTIONS(2083),
    [anon_sym_LBRACK] = ACTIONS(2083),
    [anon_sym_LBRACE] = ACTIONS(2083),
    [anon_sym_RBRACE] = ACTIONS(2083),
    [anon_sym_STAR] = ACTIONS(2083),
    [anon_sym_u8] = ACTIONS(2085),
    [anon_sym_i8] = ACTIONS(2085),
    [anon_sym_u16] = ACTIONS(2085),
    [anon_sym_i16] = ACTIONS(2085),
    [anon_sym_u32] = ACTIONS(2085),
    [anon_sym_i32] = ACTIONS(2085),
    [anon_sym_u64] = ACTIONS(2085),
    [anon_sym_i64] = ACTIONS(2085),
    [anon_sym_u128] = ACTIONS(2085),
    [anon_sym_i128] = ACTIONS(2085),
    [anon_sym_isize] = ACTIONS(2085),
    [anon_sym_usize] = ACTIONS(2085),
    [anon_sym_f32] = ACTIONS(2085),
    [anon_sym_f64] = ACTIONS(2085),
    [anon_sym_bool] = ACTIONS(2085),
    [anon_sym_str] = ACTIONS(2085),
    [anon_sym_char] = ACTIONS(2085),
    [anon_sym_DASH] = ACTIONS(2083),
    [anon_sym_BANG] = ACTIONS(2083),
    [anon_sym_AMP] = ACTIONS(2083),
    [anon_sym_PIPE] = ACTIONS(2083),
    [anon_sym_LT] = ACTIONS(2083),
    [anon_sym_DOT_DOT] = ACTIONS(2083),
    [anon_sym_COLON_COLON] = ACTIONS(2083),
    [anon_sym_POUND] = ACTIONS(2083),
    [anon_sym_SQUOTE] = ACTIONS(2085),
    [anon_sym_async] = ACTIONS(2085),
    [anon_sym_break] = ACTIONS(2085),
    [anon_sym_const] = ACTIONS(2085),
    [anon_sym_continue] = ACTIONS(2085),
    [anon_sym_default] = ACTIONS(2085),
    [anon_sym_enum] = ACTIONS(2085),
    [anon_sym_fn] = ACTIONS(2085),
    [anon_sym_for] = ACTIONS(2085),
    [anon_sym_gen] = ACTIONS(2085),
    [anon_sym_if] = ACTIONS(2085),
    [anon_sym_impl] = ACTIONS(2085),
    [anon_sym_let] = ACTIONS(2085),
    [anon_sym_loop] = ACTIONS(2085),
    [anon_sym_match] = ACTIONS(2085),
    [anon_sym_mod] = ACTIONS(2085),
    [anon_sym_pub] = ACTIONS(2085),
    [anon_sym_return] = ACTIONS(2085),
    [anon_sym_static] = ACTIONS(2085),
    [anon_sym_struct] = ACTIONS(2085),
    [anon_sym_trait] = ACTIONS(2085),
    [anon_sym_type] = ACTIONS(2085),
    [anon_sym_union] = ACTIONS(2085),
    [anon_sym_unsafe] = ACTIONS(2085),
    [anon_sym_use] = ACTIONS(2085),
    [anon_sym_while] = ACTIONS(2085),
    [anon_sym_extern] = ACTIONS(2085),
    [anon_sym_yield] = ACTIONS(2085),
    [anon_sym_move] = ACTIONS(2085),
    [anon_sym_try] = ACTIONS(2085),
    [sym_integer_literal] = ACTIONS(2083),
    [aux_sym_string_literal_token1] = ACTIONS(2083),
    [sym_char_literal] = ACTIONS(2083),
    [anon_sym_true] = ACTIONS(2085),
    [anon_sym_false] = ACTIONS(2085),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2085),
    [sym_super] = ACTIONS(2085),
    [sym_crate] = ACTIONS(2085),
    [sym_metavariable] = ACTIONS(2083),
    [sym__raw_string_literal_start] = ACTIONS(2083),
    [sym_float_literal] = ACTIONS(2083),
  },
  [584] = {
    [sym_line_comment] = STATE(584),
    [sym_block_comment] = STATE(584),
    [ts_builtin_sym_end] = ACTIONS(2087),
    [sym_identifier] = ACTIONS(2089),
    [anon_sym_SEMI] = ACTIONS(2087),
    [anon_sym_macro_rules_BANG] = ACTIONS(2087),
    [anon_sym_LPAREN] = ACTIONS(2087),
    [anon_sym_LBRACK] = ACTIONS(2087),
    [anon_sym_LBRACE] = ACTIONS(2087),
    [anon_sym_RBRACE] = ACTIONS(2087),
    [anon_sym_STAR] = ACTIONS(2087),
    [anon_sym_u8] = ACTIONS(2089),
    [anon_sym_i8] = ACTIONS(2089),
    [anon_sym_u16] = ACTIONS(2089),
    [anon_sym_i16] = ACTIONS(2089),
    [anon_sym_u32] = ACTIONS(2089),
    [anon_sym_i32] = ACTIONS(2089),
    [anon_sym_u64] = ACTIONS(2089),
    [anon_sym_i64] = ACTIONS(2089),
    [anon_sym_u128] = ACTIONS(2089),
    [anon_sym_i128] = ACTIONS(2089),
    [anon_sym_isize] = ACTIONS(2089),
    [anon_sym_usize] = ACTIONS(2089),
    [anon_sym_f32] = ACTIONS(2089),
    [anon_sym_f64] = ACTIONS(2089),
    [anon_sym_bool] = ACTIONS(2089),
    [anon_sym_str] = ACTIONS(2089),
    [anon_sym_char] = ACTIONS(2089),
    [anon_sym_DASH] = ACTIONS(2087),
    [anon_sym_BANG] = ACTIONS(2087),
    [anon_sym_AMP] = ACTIONS(2087),
    [anon_sym_PIPE] = ACTIONS(2087),
    [anon_sym_LT] = ACTIONS(2087),
    [anon_sym_DOT_DOT] = ACTIONS(2087),
    [anon_sym_COLON_COLON] = ACTIONS(2087),
    [anon_sym_POUND] = ACTIONS(2087),
    [anon_sym_SQUOTE] = ACTIONS(2089),
    [anon_sym_async] = ACTIONS(2089),
    [anon_sym_break] = ACTIONS(2089),
    [anon_sym_const] = ACTIONS(2089),
    [anon_sym_continue] = ACTIONS(2089),
    [anon_sym_default] = ACTIONS(2089),
    [anon_sym_enum] = ACTIONS(2089),
    [anon_sym_fn] = ACTIONS(2089),
    [anon_sym_for] = ACTIONS(2089),
    [anon_sym_gen] = ACTIONS(2089),
    [anon_sym_if] = ACTIONS(2089),
    [anon_sym_impl] = ACTIONS(2089),
    [anon_sym_let] = ACTIONS(2089),
    [anon_sym_loop] = ACTIONS(2089),
    [anon_sym_match] = ACTIONS(2089),
    [anon_sym_mod] = ACTIONS(2089),
    [anon_sym_pub] = ACTIONS(2089),
    [anon_sym_return] = ACTIONS(2089),
    [anon_sym_static] = ACTIONS(2089),
    [anon_sym_struct] = ACTIONS(2089),
    [anon_sym_trait] = ACTIONS(2089),
    [anon_sym_type] = ACTIONS(2089),
    [anon_sym_union] = ACTIONS(2089),
    [anon_sym_unsafe] = ACTIONS(2089),
    [anon_sym_use] = ACTIONS(2089),
    [anon_sym_while] = ACTIONS(2089),
    [anon_sym_extern] = ACTIONS(2089),
    [anon_sym_yield] = ACTIONS(2089),
    [anon_sym_move] = ACTIONS(2089),
    [anon_sym_try] = ACTIONS(2089),
    [sym_integer_literal] = ACTIONS(2087),
    [aux_sym_string_literal_token1] = ACTIONS(2087),
    [sym_char_literal] = ACTIONS(2087),
    [anon_sym_true] = ACTIONS(2089),
    [anon_sym_false] = ACTIONS(2089),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2089),
    [sym_super] = ACTIONS(2089),
    [sym_crate] = ACTIONS(2089),
    [sym_metavariable] = ACTIONS(2087),
    [sym__raw_string_literal_start] = ACTIONS(2087),
    [sym_float_literal] = ACTIONS(2087),
  },
  [585] = {
    [sym_line_comment] = STATE(585),
    [sym_block_comment] = STATE(585),
    [ts_builtin_sym_end] = ACTIONS(2091),
    [sym_identifier] = ACTIONS(2093),
    [anon_sym_SEMI] = ACTIONS(2091),
    [anon_sym_macro_rules_BANG] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(2091),
    [anon_sym_LBRACK] = ACTIONS(2091),
    [anon_sym_LBRACE] = ACTIONS(2091),
    [anon_sym_RBRACE] = ACTIONS(2091),
    [anon_sym_STAR] = ACTIONS(2091),
    [anon_sym_u8] = ACTIONS(2093),
    [anon_sym_i8] = ACTIONS(2093),
    [anon_sym_u16] = ACTIONS(2093),
    [anon_sym_i16] = ACTIONS(2093),
    [anon_sym_u32] = ACTIONS(2093),
    [anon_sym_i32] = ACTIONS(2093),
    [anon_sym_u64] = ACTIONS(2093),
    [anon_sym_i64] = ACTIONS(2093),
    [anon_sym_u128] = ACTIONS(2093),
    [anon_sym_i128] = ACTIONS(2093),
    [anon_sym_isize] = ACTIONS(2093),
    [anon_sym_usize] = ACTIONS(2093),
    [anon_sym_f32] = ACTIONS(2093),
    [anon_sym_f64] = ACTIONS(2093),
    [anon_sym_bool] = ACTIONS(2093),
    [anon_sym_str] = ACTIONS(2093),
    [anon_sym_char] = ACTIONS(2093),
    [anon_sym_DASH] = ACTIONS(2091),
    [anon_sym_BANG] = ACTIONS(2091),
    [anon_sym_AMP] = ACTIONS(2091),
    [anon_sym_PIPE] = ACTIONS(2091),
    [anon_sym_LT] = ACTIONS(2091),
    [anon_sym_DOT_DOT] = ACTIONS(2091),
    [anon_sym_COLON_COLON] = ACTIONS(2091),
    [anon_sym_POUND] = ACTIONS(2091),
    [anon_sym_SQUOTE] = ACTIONS(2093),
    [anon_sym_async] = ACTIONS(2093),
    [anon_sym_break] = ACTIONS(2093),
    [anon_sym_const] = ACTIONS(2093),
    [anon_sym_continue] = ACTIONS(2093),
    [anon_sym_default] = ACTIONS(2093),
    [anon_sym_enum] = ACTIONS(2093),
    [anon_sym_fn] = ACTIONS(2093),
    [anon_sym_for] = ACTIONS(2093),
    [anon_sym_gen] = ACTIONS(2093),
    [anon_sym_if] = ACTIONS(2093),
    [anon_sym_impl] = ACTIONS(2093),
    [anon_sym_let] = ACTIONS(2093),
    [anon_sym_loop] = ACTIONS(2093),
    [anon_sym_match] = ACTIONS(2093),
    [anon_sym_mod] = ACTIONS(2093),
    [anon_sym_pub] = ACTIONS(2093),
    [anon_sym_return] = ACTIONS(2093),
    [anon_sym_static] = ACTIONS(2093),
    [anon_sym_struct] = ACTIONS(2093),
    [anon_sym_trait] = ACTIONS(2093),
    [anon_sym_type] = ACTIONS(2093),
    [anon_sym_union] = ACTIONS(2093),
    [anon_sym_unsafe] = ACTIONS(2093),
    [anon_sym_use] = ACTIONS(2093),
    [anon_sym_while] = ACTIONS(2093),
    [anon_sym_extern] = ACTIONS(2093),
    [anon_sym_yield] = ACTIONS(2093),
    [anon_sym_move] = ACTIONS(2093),
    [anon_sym_try] = ACTIONS(2093),
    [sym_integer_literal] = ACTIONS(2091),
    [aux_sym_string_literal_token1] = ACTIONS(2091),
    [sym_char_literal] = ACTIONS(2091),
    [anon_sym_true] = ACTIONS(2093),
    [anon_sym_false] = ACTIONS(2093),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2093),
    [sym_super] = ACTIONS(2093),
    [sym_crate] = ACTIONS(2093),
    [sym_metavariable] = ACTIONS(2091),
    [sym__raw_string_literal_start] = ACTIONS(2091),
    [sym_float_literal] = ACTIONS(2091),
  },
  [586] = {
    [sym_line_comment] = STATE(586),
    [sym_block_comment] = STATE(586),
    [ts_builtin_sym_end] = ACTIONS(2095),
    [sym_identifier] = ACTIONS(2097),
    [anon_sym_SEMI] = ACTIONS(2095),
    [anon_sym_macro_rules_BANG] = ACTIONS(2095),
    [anon_sym_LPAREN] = ACTIONS(2095),
    [anon_sym_LBRACK] = ACTIONS(2095),
    [anon_sym_LBRACE] = ACTIONS(2095),
    [anon_sym_RBRACE] = ACTIONS(2095),
    [anon_sym_STAR] = ACTIONS(2095),
    [anon_sym_u8] = ACTIONS(2097),
    [anon_sym_i8] = ACTIONS(2097),
    [anon_sym_u16] = ACTIONS(2097),
    [anon_sym_i16] = ACTIONS(2097),
    [anon_sym_u32] = ACTIONS(2097),
    [anon_sym_i32] = ACTIONS(2097),
    [anon_sym_u64] = ACTIONS(2097),
    [anon_sym_i64] = ACTIONS(2097),
    [anon_sym_u128] = ACTIONS(2097),
    [anon_sym_i128] = ACTIONS(2097),
    [anon_sym_isize] = ACTIONS(2097),
    [anon_sym_usize] = ACTIONS(2097),
    [anon_sym_f32] = ACTIONS(2097),
    [anon_sym_f64] = ACTIONS(2097),
    [anon_sym_bool] = ACTIONS(2097),
    [anon_sym_str] = ACTIONS(2097),
    [anon_sym_char] = ACTIONS(2097),
    [anon_sym_DASH] = ACTIONS(2095),
    [anon_sym_BANG] = ACTIONS(2095),
    [anon_sym_AMP] = ACTIONS(2095),
    [anon_sym_PIPE] = ACTIONS(2095),
    [anon_sym_LT] = ACTIONS(2095),
    [anon_sym_DOT_DOT] = ACTIONS(2095),
    [anon_sym_COLON_COLON] = ACTIONS(2095),
    [anon_sym_POUND] = ACTIONS(2095),
    [anon_sym_SQUOTE] = ACTIONS(2097),
    [anon_sym_async] = ACTIONS(2097),
    [anon_sym_break] = ACTIONS(2097),
    [anon_sym_const] = ACTIONS(2097),
    [anon_sym_continue] = ACTIONS(2097),
    [anon_sym_default] = ACTIONS(2097),
    [anon_sym_enum] = ACTIONS(2097),
    [anon_sym_fn] = ACTIONS(2097),
    [anon_sym_for] = ACTIONS(2097),
    [anon_sym_gen] = ACTIONS(2097),
    [anon_sym_if] = ACTIONS(2097),
    [anon_sym_impl] = ACTIONS(2097),
    [anon_sym_let] = ACTIONS(2097),
    [anon_sym_loop] = ACTIONS(2097),
    [anon_sym_match] = ACTIONS(2097),
    [anon_sym_mod] = ACTIONS(2097),
    [anon_sym_pub] = ACTIONS(2097),
    [anon_sym_return] = ACTIONS(2097),
    [anon_sym_static] = ACTIONS(2097),
    [anon_sym_struct] = ACTIONS(2097),
    [anon_sym_trait] = ACTIONS(2097),
    [anon_sym_type] = ACTIONS(2097),
    [anon_sym_union] = ACTIONS(2097),
    [anon_sym_unsafe] = ACTIONS(2097),
    [anon_sym_use] = ACTIONS(2097),
    [anon_sym_while] = ACTIONS(2097),
    [anon_sym_extern] = ACTIONS(2097),
    [anon_sym_yield] = ACTIONS(2097),
    [anon_sym_move] = ACTIONS(2097),
    [anon_sym_try] = ACTIONS(2097),
    [sym_integer_literal] = ACTIONS(2095),
    [aux_sym_string_literal_token1] = ACTIONS(2095),
    [sym_char_literal] = ACTIONS(2095),
    [anon_sym_true] = ACTIONS(2097),
    [anon_sym_false] = ACTIONS(2097),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2097),
    [sym_super] = ACTIONS(2097),
    [sym_crate] = ACTIONS(2097),
    [sym_metavariable] = ACTIONS(2095),
    [sym__raw_string_literal_start] = ACTIONS(2095),
    [sym_float_literal] = ACTIONS(2095),
  },
  [587] = {
    [sym_line_comment] = STATE(587),
    [sym_block_comment] = STATE(587),
    [ts_builtin_sym_end] = ACTIONS(2099),
    [sym_identifier] = ACTIONS(2101),
    [anon_sym_SEMI] = ACTIONS(2099),
    [anon_sym_macro_rules_BANG] = ACTIONS(2099),
    [anon_sym_LPAREN] = ACTIONS(2099),
    [anon_sym_LBRACK] = ACTIONS(2099),
    [anon_sym_LBRACE] = ACTIONS(2099),
    [anon_sym_RBRACE] = ACTIONS(2099),
    [anon_sym_STAR] = ACTIONS(2099),
    [anon_sym_u8] = ACTIONS(2101),
    [anon_sym_i8] = ACTIONS(2101),
    [anon_sym_u16] = ACTIONS(2101),
    [anon_sym_i16] = ACTIONS(2101),
    [anon_sym_u32] = ACTIONS(2101),
    [anon_sym_i32] = ACTIONS(2101),
    [anon_sym_u64] = ACTIONS(2101),
    [anon_sym_i64] = ACTIONS(2101),
    [anon_sym_u128] = ACTIONS(2101),
    [anon_sym_i128] = ACTIONS(2101),
    [anon_sym_isize] = ACTIONS(2101),
    [anon_sym_usize] = ACTIONS(2101),
    [anon_sym_f32] = ACTIONS(2101),
    [anon_sym_f64] = ACTIONS(2101),
    [anon_sym_bool] = ACTIONS(2101),
    [anon_sym_str] = ACTIONS(2101),
    [anon_sym_char] = ACTIONS(2101),
    [anon_sym_DASH] = ACTIONS(2099),
    [anon_sym_BANG] = ACTIONS(2099),
    [anon_sym_AMP] = ACTIONS(2099),
    [anon_sym_PIPE] = ACTIONS(2099),
    [anon_sym_LT] = ACTIONS(2099),
    [anon_sym_DOT_DOT] = ACTIONS(2099),
    [anon_sym_COLON_COLON] = ACTIONS(2099),
    [anon_sym_POUND] = ACTIONS(2099),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(2101),
    [anon_sym_break] = ACTIONS(2101),
    [anon_sym_const] = ACTIONS(2101),
    [anon_sym_continue] = ACTIONS(2101),
    [anon_sym_default] = ACTIONS(2101),
    [anon_sym_enum] = ACTIONS(2101),
    [anon_sym_fn] = ACTIONS(2101),
    [anon_sym_for] = ACTIONS(2101),
    [anon_sym_gen] = ACTIONS(2101),
    [anon_sym_if] = ACTIONS(2101),
    [anon_sym_impl] = ACTIONS(2101),
    [anon_sym_let] = ACTIONS(2101),
    [anon_sym_loop] = ACTIONS(2101),
    [anon_sym_match] = ACTIONS(2101),
    [anon_sym_mod] = ACTIONS(2101),
    [anon_sym_pub] = ACTIONS(2101),
    [anon_sym_return] = ACTIONS(2101),
    [anon_sym_static] = ACTIONS(2101),
    [anon_sym_struct] = ACTIONS(2101),
    [anon_sym_trait] = ACTIONS(2101),
    [anon_sym_type] = ACTIONS(2101),
    [anon_sym_union] = ACTIONS(2101),
    [anon_sym_unsafe] = ACTIONS(2101),
    [anon_sym_use] = ACTIONS(2101),
    [anon_sym_while] = ACTIONS(2101),
    [anon_sym_extern] = ACTIONS(2101),
    [anon_sym_yield] = ACTIONS(2101),
    [anon_sym_move] = ACTIONS(2101),
    [anon_sym_try] = ACTIONS(2101),
    [sym_integer_literal] = ACTIONS(2099),
    [aux_sym_string_literal_token1] = ACTIONS(2099),
    [sym_char_literal] = ACTIONS(2099),
    [anon_sym_true] = ACTIONS(2101),
    [anon_sym_false] = ACTIONS(2101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2101),
    [sym_super] = ACTIONS(2101),
    [sym_crate] = ACTIONS(2101),
    [sym_metavariable] = ACTIONS(2099),
    [sym__raw_string_literal_start] = ACTIONS(2099),
    [sym_float_literal] = ACTIONS(2099),
  },
  [588] = {
    [sym_line_comment] = STATE(588),
    [sym_block_comment] = STATE(588),
    [ts_builtin_sym_end] = ACTIONS(2103),
    [sym_identifier] = ACTIONS(2105),
    [anon_sym_SEMI] = ACTIONS(2103),
    [anon_sym_macro_rules_BANG] = ACTIONS(2103),
    [anon_sym_LPAREN] = ACTIONS(2103),
    [anon_sym_LBRACK] = ACTIONS(2103),
    [anon_sym_LBRACE] = ACTIONS(2103),
    [anon_sym_RBRACE] = ACTIONS(2103),
    [anon_sym_STAR] = ACTIONS(2103),
    [anon_sym_u8] = ACTIONS(2105),
    [anon_sym_i8] = ACTIONS(2105),
    [anon_sym_u16] = ACTIONS(2105),
    [anon_sym_i16] = ACTIONS(2105),
    [anon_sym_u32] = ACTIONS(2105),
    [anon_sym_i32] = ACTIONS(2105),
    [anon_sym_u64] = ACTIONS(2105),
    [anon_sym_i64] = ACTIONS(2105),
    [anon_sym_u128] = ACTIONS(2105),
    [anon_sym_i128] = ACTIONS(2105),
    [anon_sym_isize] = ACTIONS(2105),
    [anon_sym_usize] = ACTIONS(2105),
    [anon_sym_f32] = ACTIONS(2105),
    [anon_sym_f64] = ACTIONS(2105),
    [anon_sym_bool] = ACTIONS(2105),
    [anon_sym_str] = ACTIONS(2105),
    [anon_sym_char] = ACTIONS(2105),
    [anon_sym_DASH] = ACTIONS(2103),
    [anon_sym_BANG] = ACTIONS(2103),
    [anon_sym_AMP] = ACTIONS(2103),
    [anon_sym_PIPE] = ACTIONS(2103),
    [anon_sym_LT] = ACTIONS(2103),
    [anon_sym_DOT_DOT] = ACTIONS(2103),
    [anon_sym_COLON_COLON] = ACTIONS(2103),
    [anon_sym_POUND] = ACTIONS(2103),
    [anon_sym_SQUOTE] = ACTIONS(2105),
    [anon_sym_async] = ACTIONS(2105),
    [anon_sym_break] = ACTIONS(2105),
    [anon_sym_const] = ACTIONS(2105),
    [anon_sym_continue] = ACTIONS(2105),
    [anon_sym_default] = ACTIONS(2105),
    [anon_sym_enum] = ACTIONS(2105),
    [anon_sym_fn] = ACTIONS(2105),
    [anon_sym_for] = ACTIONS(2105),
    [anon_sym_gen] = ACTIONS(2105),
    [anon_sym_if] = ACTIONS(2105),
    [anon_sym_impl] = ACTIONS(2105),
    [anon_sym_let] = ACTIONS(2105),
    [anon_sym_loop] = ACTIONS(2105),
    [anon_sym_match] = ACTIONS(2105),
    [anon_sym_mod] = ACTIONS(2105),
    [anon_sym_pub] = ACTIONS(2105),
    [anon_sym_return] = ACTIONS(2105),
    [anon_sym_static] = ACTIONS(2105),
    [anon_sym_struct] = ACTIONS(2105),
    [anon_sym_trait] = ACTIONS(2105),
    [anon_sym_type] = ACTIONS(2105),
    [anon_sym_union] = ACTIONS(2105),
    [anon_sym_unsafe] = ACTIONS(2105),
    [anon_sym_use] = ACTIONS(2105),
    [anon_sym_while] = ACTIONS(2105),
    [anon_sym_extern] = ACTIONS(2105),
    [anon_sym_yield] = ACTIONS(2105),
    [anon_sym_move] = ACTIONS(2105),
    [anon_sym_try] = ACTIONS(2105),
    [sym_integer_literal] = ACTIONS(2103),
    [aux_sym_string_literal_token1] = ACTIONS(2103),
    [sym_char_literal] = ACTIONS(2103),
    [anon_sym_true] = ACTIONS(2105),
    [anon_sym_false] = ACTIONS(2105),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2105),
    [sym_super] = ACTIONS(2105),
    [sym_crate] = ACTIONS(2105),
    [sym_metavariable] = ACTIONS(2103),
    [sym__raw_string_literal_start] = ACTIONS(2103),
    [sym_float_literal] = ACTIONS(2103),
  },
  [589] = {
    [sym_line_comment] = STATE(589),
    [sym_block_comment] = STATE(589),
    [ts_builtin_sym_end] = ACTIONS(2107),
    [sym_identifier] = ACTIONS(2109),
    [anon_sym_SEMI] = ACTIONS(2107),
    [anon_sym_macro_rules_BANG] = ACTIONS(2107),
    [anon_sym_LPAREN] = ACTIONS(2107),
    [anon_sym_LBRACK] = ACTIONS(2107),
    [anon_sym_LBRACE] = ACTIONS(2107),
    [anon_sym_RBRACE] = ACTIONS(2107),
    [anon_sym_STAR] = ACTIONS(2107),
    [anon_sym_u8] = ACTIONS(2109),
    [anon_sym_i8] = ACTIONS(2109),
    [anon_sym_u16] = ACTIONS(2109),
    [anon_sym_i16] = ACTIONS(2109),
    [anon_sym_u32] = ACTIONS(2109),
    [anon_sym_i32] = ACTIONS(2109),
    [anon_sym_u64] = ACTIONS(2109),
    [anon_sym_i64] = ACTIONS(2109),
    [anon_sym_u128] = ACTIONS(2109),
    [anon_sym_i128] = ACTIONS(2109),
    [anon_sym_isize] = ACTIONS(2109),
    [anon_sym_usize] = ACTIONS(2109),
    [anon_sym_f32] = ACTIONS(2109),
    [anon_sym_f64] = ACTIONS(2109),
    [anon_sym_bool] = ACTIONS(2109),
    [anon_sym_str] = ACTIONS(2109),
    [anon_sym_char] = ACTIONS(2109),
    [anon_sym_DASH] = ACTIONS(2107),
    [anon_sym_BANG] = ACTIONS(2107),
    [anon_sym_AMP] = ACTIONS(2107),
    [anon_sym_PIPE] = ACTIONS(2107),
    [anon_sym_LT] = ACTIONS(2107),
    [anon_sym_DOT_DOT] = ACTIONS(2107),
    [anon_sym_COLON_COLON] = ACTIONS(2107),
    [anon_sym_POUND] = ACTIONS(2107),
    [anon_sym_SQUOTE] = ACTIONS(2109),
    [anon_sym_async] = ACTIONS(2109),
    [anon_sym_break] = ACTIONS(2109),
    [anon_sym_const] = ACTIONS(2109),
    [anon_sym_continue] = ACTIONS(2109),
    [anon_sym_default] = ACTIONS(2109),
    [anon_sym_enum] = ACTIONS(2109),
    [anon_sym_fn] = ACTIONS(2109),
    [anon_sym_for] = ACTIONS(2109),
    [anon_sym_gen] = ACTIONS(2109),
    [anon_sym_if] = ACTIONS(2109),
    [anon_sym_impl] = ACTIONS(2109),
    [anon_sym_let] = ACTIONS(2109),
    [anon_sym_loop] = ACTIONS(2109),
    [anon_sym_match] = ACTIONS(2109),
    [anon_sym_mod] = ACTIONS(2109),
    [anon_sym_pub] = ACTIONS(2109),
    [anon_sym_return] = ACTIONS(2109),
    [anon_sym_static] = ACTIONS(2109),
    [anon_sym_struct] = ACTIONS(2109),
    [anon_sym_trait] = ACTIONS(2109),
    [anon_sym_type] = ACTIONS(2109),
    [anon_sym_union] = ACTIONS(2109),
    [anon_sym_unsafe] = ACTIONS(2109),
    [anon_sym_use] = ACTIONS(2109),
    [anon_sym_while] = ACTIONS(2109),
    [anon_sym_extern] = ACTIONS(2109),
    [anon_sym_yield] = ACTIONS(2109),
    [anon_sym_move] = ACTIONS(2109),
    [anon_sym_try] = ACTIONS(2109),
    [sym_integer_literal] = ACTIONS(2107),
    [aux_sym_string_literal_token1] = ACTIONS(2107),
    [sym_char_literal] = ACTIONS(2107),
    [anon_sym_true] = ACTIONS(2109),
    [anon_sym_false] = ACTIONS(2109),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2109),
    [sym_super] = ACTIONS(2109),
    [sym_crate] = ACTIONS(2109),
    [sym_metavariable] = ACTIONS(2107),
    [sym__raw_string_literal_start] = ACTIONS(2107),
    [sym_float_literal] = ACTIONS(2107),
  },
  [590] = {
    [sym_line_comment] = STATE(590),
    [sym_block_comment] = STATE(590),
    [ts_builtin_sym_end] = ACTIONS(2111),
    [sym_identifier] = ACTIONS(2113),
    [anon_sym_SEMI] = ACTIONS(2111),
    [anon_sym_macro_rules_BANG] = ACTIONS(2111),
    [anon_sym_LPAREN] = ACTIONS(2111),
    [anon_sym_LBRACK] = ACTIONS(2111),
    [anon_sym_LBRACE] = ACTIONS(2111),
    [anon_sym_RBRACE] = ACTIONS(2111),
    [anon_sym_STAR] = ACTIONS(2111),
    [anon_sym_u8] = ACTIONS(2113),
    [anon_sym_i8] = ACTIONS(2113),
    [anon_sym_u16] = ACTIONS(2113),
    [anon_sym_i16] = ACTIONS(2113),
    [anon_sym_u32] = ACTIONS(2113),
    [anon_sym_i32] = ACTIONS(2113),
    [anon_sym_u64] = ACTIONS(2113),
    [anon_sym_i64] = ACTIONS(2113),
    [anon_sym_u128] = ACTIONS(2113),
    [anon_sym_i128] = ACTIONS(2113),
    [anon_sym_isize] = ACTIONS(2113),
    [anon_sym_usize] = ACTIONS(2113),
    [anon_sym_f32] = ACTIONS(2113),
    [anon_sym_f64] = ACTIONS(2113),
    [anon_sym_bool] = ACTIONS(2113),
    [anon_sym_str] = ACTIONS(2113),
    [anon_sym_char] = ACTIONS(2113),
    [anon_sym_DASH] = ACTIONS(2111),
    [anon_sym_BANG] = ACTIONS(2111),
    [anon_sym_AMP] = ACTIONS(2111),
    [anon_sym_PIPE] = ACTIONS(2111),
    [anon_sym_LT] = ACTIONS(2111),
    [anon_sym_DOT_DOT] = ACTIONS(2111),
    [anon_sym_COLON_COLON] = ACTIONS(2111),
    [anon_sym_POUND] = ACTIONS(2111),
    [anon_sym_SQUOTE] = ACTIONS(2113),
    [anon_sym_async] = ACTIONS(2113),
    [anon_sym_break] = ACTIONS(2113),
    [anon_sym_const] = ACTIONS(2113),
    [anon_sym_continue] = ACTIONS(2113),
    [anon_sym_default] = ACTIONS(2113),
    [anon_sym_enum] = ACTIONS(2113),
    [anon_sym_fn] = ACTIONS(2113),
    [anon_sym_for] = ACTIONS(2113),
    [anon_sym_gen] = ACTIONS(2113),
    [anon_sym_if] = ACTIONS(2113),
    [anon_sym_impl] = ACTIONS(2113),
    [anon_sym_let] = ACTIONS(2113),
    [anon_sym_loop] = ACTIONS(2113),
    [anon_sym_match] = ACTIONS(2113),
    [anon_sym_mod] = ACTIONS(2113),
    [anon_sym_pub] = ACTIONS(2113),
    [anon_sym_return] = ACTIONS(2113),
    [anon_sym_static] = ACTIONS(2113),
    [anon_sym_struct] = ACTIONS(2113),
    [anon_sym_trait] = ACTIONS(2113),
    [anon_sym_type] = ACTIONS(2113),
    [anon_sym_union] = ACTIONS(2113),
    [anon_sym_unsafe] = ACTIONS(2113),
    [anon_sym_use] = ACTIONS(2113),
    [anon_sym_while] = ACTIONS(2113),
    [anon_sym_extern] = ACTIONS(2113),
    [anon_sym_yield] = ACTIONS(2113),
    [anon_sym_move] = ACTIONS(2113),
    [anon_sym_try] = ACTIONS(2113),
    [sym_integer_literal] = ACTIONS(2111),
    [aux_sym_string_literal_token1] = ACTIONS(2111),
    [sym_char_literal] = ACTIONS(2111),
    [anon_sym_true] = ACTIONS(2113),
    [anon_sym_false] = ACTIONS(2113),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2113),
    [sym_super] = ACTIONS(2113),
    [sym_crate] = ACTIONS(2113),
    [sym_metavariable] = ACTIONS(2111),
    [sym__raw_string_literal_start] = ACTIONS(2111),
    [sym_float_literal] = ACTIONS(2111),
  },
  [591] = {
    [sym_line_comment] = STATE(591),
    [sym_block_comment] = STATE(591),
    [ts_builtin_sym_end] = ACTIONS(2115),
    [sym_identifier] = ACTIONS(2117),
    [anon_sym_SEMI] = ACTIONS(2115),
    [anon_sym_macro_rules_BANG] = ACTIONS(2115),
    [anon_sym_LPAREN] = ACTIONS(2115),
    [anon_sym_LBRACK] = ACTIONS(2115),
    [anon_sym_LBRACE] = ACTIONS(2115),
    [anon_sym_RBRACE] = ACTIONS(2115),
    [anon_sym_STAR] = ACTIONS(2115),
    [anon_sym_u8] = ACTIONS(2117),
    [anon_sym_i8] = ACTIONS(2117),
    [anon_sym_u16] = ACTIONS(2117),
    [anon_sym_i16] = ACTIONS(2117),
    [anon_sym_u32] = ACTIONS(2117),
    [anon_sym_i32] = ACTIONS(2117),
    [anon_sym_u64] = ACTIONS(2117),
    [anon_sym_i64] = ACTIONS(2117),
    [anon_sym_u128] = ACTIONS(2117),
    [anon_sym_i128] = ACTIONS(2117),
    [anon_sym_isize] = ACTIONS(2117),
    [anon_sym_usize] = ACTIONS(2117),
    [anon_sym_f32] = ACTIONS(2117),
    [anon_sym_f64] = ACTIONS(2117),
    [anon_sym_bool] = ACTIONS(2117),
    [anon_sym_str] = ACTIONS(2117),
    [anon_sym_char] = ACTIONS(2117),
    [anon_sym_DASH] = ACTIONS(2115),
    [anon_sym_BANG] = ACTIONS(2115),
    [anon_sym_AMP] = ACTIONS(2115),
    [anon_sym_PIPE] = ACTIONS(2115),
    [anon_sym_LT] = ACTIONS(2115),
    [anon_sym_DOT_DOT] = ACTIONS(2115),
    [anon_sym_COLON_COLON] = ACTIONS(2115),
    [anon_sym_POUND] = ACTIONS(2115),
    [anon_sym_SQUOTE] = ACTIONS(2117),
    [anon_sym_async] = ACTIONS(2117),
    [anon_sym_break] = ACTIONS(2117),
    [anon_sym_const] = ACTIONS(2117),
    [anon_sym_continue] = ACTIONS(2117),
    [anon_sym_default] = ACTIONS(2117),
    [anon_sym_enum] = ACTIONS(2117),
    [anon_sym_fn] = ACTIONS(2117),
    [anon_sym_for] = ACTIONS(2117),
    [anon_sym_gen] = ACTIONS(2117),
    [anon_sym_if] = ACTIONS(2117),
    [anon_sym_impl] = ACTIONS(2117),
    [anon_sym_let] = ACTIONS(2117),
    [anon_sym_loop] = ACTIONS(2117),
    [anon_sym_match] = ACTIONS(2117),
    [anon_sym_mod] = ACTIONS(2117),
    [anon_sym_pub] = ACTIONS(2117),
    [anon_sym_return] = ACTIONS(2117),
    [anon_sym_static] = ACTIONS(2117),
    [anon_sym_struct] = ACTIONS(2117),
    [anon_sym_trait] = ACTIONS(2117),
    [anon_sym_type] = ACTIONS(2117),
    [anon_sym_union] = ACTIONS(2117),
    [anon_sym_unsafe] = ACTIONS(2117),
    [anon_sym_use] = ACTIONS(2117),
    [anon_sym_while] = ACTIONS(2117),
    [anon_sym_extern] = ACTIONS(2117),
    [anon_sym_yield] = ACTIONS(2117),
    [anon_sym_move] = ACTIONS(2117),
    [anon_sym_try] = ACTIONS(2117),
    [sym_integer_literal] = ACTIONS(2115),
    [aux_sym_string_literal_token1] = ACTIONS(2115),
    [sym_char_literal] = ACTIONS(2115),
    [anon_sym_true] = ACTIONS(2117),
    [anon_sym_false] = ACTIONS(2117),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2117),
    [sym_super] = ACTIONS(2117),
    [sym_crate] = ACTIONS(2117),
    [sym_metavariable] = ACTIONS(2115),
    [sym__raw_string_literal_start] = ACTIONS(2115),
    [sym_float_literal] = ACTIONS(2115),
  },
  [592] = {
    [sym_line_comment] = STATE(592),
    [sym_block_comment] = STATE(592),
    [ts_builtin_sym_end] = ACTIONS(2119),
    [sym_identifier] = ACTIONS(2121),
    [anon_sym_SEMI] = ACTIONS(2119),
    [anon_sym_macro_rules_BANG] = ACTIONS(2119),
    [anon_sym_LPAREN] = ACTIONS(2119),
    [anon_sym_LBRACK] = ACTIONS(2119),
    [anon_sym_LBRACE] = ACTIONS(2119),
    [anon_sym_RBRACE] = ACTIONS(2119),
    [anon_sym_STAR] = ACTIONS(2119),
    [anon_sym_u8] = ACTIONS(2121),
    [anon_sym_i8] = ACTIONS(2121),
    [anon_sym_u16] = ACTIONS(2121),
    [anon_sym_i16] = ACTIONS(2121),
    [anon_sym_u32] = ACTIONS(2121),
    [anon_sym_i32] = ACTIONS(2121),
    [anon_sym_u64] = ACTIONS(2121),
    [anon_sym_i64] = ACTIONS(2121),
    [anon_sym_u128] = ACTIONS(2121),
    [anon_sym_i128] = ACTIONS(2121),
    [anon_sym_isize] = ACTIONS(2121),
    [anon_sym_usize] = ACTIONS(2121),
    [anon_sym_f32] = ACTIONS(2121),
    [anon_sym_f64] = ACTIONS(2121),
    [anon_sym_bool] = ACTIONS(2121),
    [anon_sym_str] = ACTIONS(2121),
    [anon_sym_char] = ACTIONS(2121),
    [anon_sym_DASH] = ACTIONS(2119),
    [anon_sym_BANG] = ACTIONS(2119),
    [anon_sym_AMP] = ACTIONS(2119),
    [anon_sym_PIPE] = ACTIONS(2119),
    [anon_sym_LT] = ACTIONS(2119),
    [anon_sym_DOT_DOT] = ACTIONS(2119),
    [anon_sym_COLON_COLON] = ACTIONS(2119),
    [anon_sym_POUND] = ACTIONS(2119),
    [anon_sym_SQUOTE] = ACTIONS(2121),
    [anon_sym_async] = ACTIONS(2121),
    [anon_sym_break] = ACTIONS(2121),
    [anon_sym_const] = ACTIONS(2121),
    [anon_sym_continue] = ACTIONS(2121),
    [anon_sym_default] = ACTIONS(2121),
    [anon_sym_enum] = ACTIONS(2121),
    [anon_sym_fn] = ACTIONS(2121),
    [anon_sym_for] = ACTIONS(2121),
    [anon_sym_gen] = ACTIONS(2121),
    [anon_sym_if] = ACTIONS(2121),
    [anon_sym_impl] = ACTIONS(2121),
    [anon_sym_let] = ACTIONS(2121),
    [anon_sym_loop] = ACTIONS(2121),
    [anon_sym_match] = ACTIONS(2121),
    [anon_sym_mod] = ACTIONS(2121),
    [anon_sym_pub] = ACTIONS(2121),
    [anon_sym_return] = ACTIONS(2121),
    [anon_sym_static] = ACTIONS(2121),
    [anon_sym_struct] = ACTIONS(2121),
    [anon_sym_trait] = ACTIONS(2121),
    [anon_sym_type] = ACTIONS(2121),
    [anon_sym_union] = ACTIONS(2121),
    [anon_sym_unsafe] = ACTIONS(2121),
    [anon_sym_use] = ACTIONS(2121),
    [anon_sym_while] = ACTIONS(2121),
    [anon_sym_extern] = ACTIONS(2121),
    [anon_sym_yield] = ACTIONS(2121),
    [anon_sym_move] = ACTIONS(2121),
    [anon_sym_try] = ACTIONS(2121),
    [sym_integer_literal] = ACTIONS(2119),
    [aux_sym_string_literal_token1] = ACTIONS(2119),
    [sym_char_literal] = ACTIONS(2119),
    [anon_sym_true] = ACTIONS(2121),
    [anon_sym_false] = ACTIONS(2121),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2121),
    [sym_super] = ACTIONS(2121),
    [sym_crate] = ACTIONS(2121),
    [sym_metavariable] = ACTIONS(2119),
    [sym__raw_string_literal_start] = ACTIONS(2119),
    [sym_float_literal] = ACTIONS(2119),
  },
  [593] = {
    [sym_line_comment] = STATE(593),
    [sym_block_comment] = STATE(593),
    [ts_builtin_sym_end] = ACTIONS(2123),
    [sym_identifier] = ACTIONS(2125),
    [anon_sym_SEMI] = ACTIONS(2123),
    [anon_sym_macro_rules_BANG] = ACTIONS(2123),
    [anon_sym_LPAREN] = ACTIONS(2123),
    [anon_sym_LBRACK] = ACTIONS(2123),
    [anon_sym_LBRACE] = ACTIONS(2123),
    [anon_sym_RBRACE] = ACTIONS(2123),
    [anon_sym_STAR] = ACTIONS(2123),
    [anon_sym_u8] = ACTIONS(2125),
    [anon_sym_i8] = ACTIONS(2125),
    [anon_sym_u16] = ACTIONS(2125),
    [anon_sym_i16] = ACTIONS(2125),
    [anon_sym_u32] = ACTIONS(2125),
    [anon_sym_i32] = ACTIONS(2125),
    [anon_sym_u64] = ACTIONS(2125),
    [anon_sym_i64] = ACTIONS(2125),
    [anon_sym_u128] = ACTIONS(2125),
    [anon_sym_i128] = ACTIONS(2125),
    [anon_sym_isize] = ACTIONS(2125),
    [anon_sym_usize] = ACTIONS(2125),
    [anon_sym_f32] = ACTIONS(2125),
    [anon_sym_f64] = ACTIONS(2125),
    [anon_sym_bool] = ACTIONS(2125),
    [anon_sym_str] = ACTIONS(2125),
    [anon_sym_char] = ACTIONS(2125),
    [anon_sym_DASH] = ACTIONS(2123),
    [anon_sym_BANG] = ACTIONS(2123),
    [anon_sym_AMP] = ACTIONS(2123),
    [anon_sym_PIPE] = ACTIONS(2123),
    [anon_sym_LT] = ACTIONS(2123),
    [anon_sym_DOT_DOT] = ACTIONS(2123),
    [anon_sym_COLON_COLON] = ACTIONS(2123),
    [anon_sym_POUND] = ACTIONS(2123),
    [anon_sym_SQUOTE] = ACTIONS(2125),
    [anon_sym_async] = ACTIONS(2125),
    [anon_sym_break] = ACTIONS(2125),
    [anon_sym_const] = ACTIONS(2125),
    [anon_sym_continue] = ACTIONS(2125),
    [anon_sym_default] = ACTIONS(2125),
    [anon_sym_enum] = ACTIONS(2125),
    [anon_sym_fn] = ACTIONS(2125),
    [anon_sym_for] = ACTIONS(2125),
    [anon_sym_gen] = ACTIONS(2125),
    [anon_sym_if] = ACTIONS(2125),
    [anon_sym_impl] = ACTIONS(2125),
    [anon_sym_let] = ACTIONS(2125),
    [anon_sym_loop] = ACTIONS(2125),
    [anon_sym_match] = ACTIONS(2125),
    [anon_sym_mod] = ACTIONS(2125),
    [anon_sym_pub] = ACTIONS(2125),
    [anon_sym_return] = ACTIONS(2125),
    [anon_sym_static] = ACTIONS(2125),
    [anon_sym_struct] = ACTIONS(2125),
    [anon_sym_trait] = ACTIONS(2125),
    [anon_sym_type] = ACTIONS(2125),
    [anon_sym_union] = ACTIONS(2125),
    [anon_sym_unsafe] = ACTIONS(2125),
    [anon_sym_use] = ACTIONS(2125),
    [anon_sym_while] = ACTIONS(2125),
    [anon_sym_extern] = ACTIONS(2125),
    [anon_sym_yield] = ACTIONS(2125),
    [anon_sym_move] = ACTIONS(2125),
    [anon_sym_try] = ACTIONS(2125),
    [sym_integer_literal] = ACTIONS(2123),
    [aux_sym_string_literal_token1] = ACTIONS(2123),
    [sym_char_literal] = ACTIONS(2123),
    [anon_sym_true] = ACTIONS(2125),
    [anon_sym_false] = ACTIONS(2125),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2125),
    [sym_super] = ACTIONS(2125),
    [sym_crate] = ACTIONS(2125),
    [sym_metavariable] = ACTIONS(2123),
    [sym__raw_string_literal_start] = ACTIONS(2123),
    [sym_float_literal] = ACTIONS(2123),
  },
  [594] = {
    [sym_line_comment] = STATE(594),
    [sym_block_comment] = STATE(594),
    [ts_builtin_sym_end] = ACTIONS(2127),
    [sym_identifier] = ACTIONS(2129),
    [anon_sym_SEMI] = ACTIONS(2127),
    [anon_sym_macro_rules_BANG] = ACTIONS(2127),
    [anon_sym_LPAREN] = ACTIONS(2127),
    [anon_sym_LBRACK] = ACTIONS(2127),
    [anon_sym_LBRACE] = ACTIONS(2127),
    [anon_sym_RBRACE] = ACTIONS(2127),
    [anon_sym_STAR] = ACTIONS(2127),
    [anon_sym_u8] = ACTIONS(2129),
    [anon_sym_i8] = ACTIONS(2129),
    [anon_sym_u16] = ACTIONS(2129),
    [anon_sym_i16] = ACTIONS(2129),
    [anon_sym_u32] = ACTIONS(2129),
    [anon_sym_i32] = ACTIONS(2129),
    [anon_sym_u64] = ACTIONS(2129),
    [anon_sym_i64] = ACTIONS(2129),
    [anon_sym_u128] = ACTIONS(2129),
    [anon_sym_i128] = ACTIONS(2129),
    [anon_sym_isize] = ACTIONS(2129),
    [anon_sym_usize] = ACTIONS(2129),
    [anon_sym_f32] = ACTIONS(2129),
    [anon_sym_f64] = ACTIONS(2129),
    [anon_sym_bool] = ACTIONS(2129),
    [anon_sym_str] = ACTIONS(2129),
    [anon_sym_char] = ACTIONS(2129),
    [anon_sym_DASH] = ACTIONS(2127),
    [anon_sym_BANG] = ACTIONS(2127),
    [anon_sym_AMP] = ACTIONS(2127),
    [anon_sym_PIPE] = ACTIONS(2127),
    [anon_sym_LT] = ACTIONS(2127),
    [anon_sym_DOT_DOT] = ACTIONS(2127),
    [anon_sym_COLON_COLON] = ACTIONS(2127),
    [anon_sym_POUND] = ACTIONS(2127),
    [anon_sym_SQUOTE] = ACTIONS(2129),
    [anon_sym_async] = ACTIONS(2129),
    [anon_sym_break] = ACTIONS(2129),
    [anon_sym_const] = ACTIONS(2129),
    [anon_sym_continue] = ACTIONS(2129),
    [anon_sym_default] = ACTIONS(2129),
    [anon_sym_enum] = ACTIONS(2129),
    [anon_sym_fn] = ACTIONS(2129),
    [anon_sym_for] = ACTIONS(2129),
    [anon_sym_gen] = ACTIONS(2129),
    [anon_sym_if] = ACTIONS(2129),
    [anon_sym_impl] = ACTIONS(2129),
    [anon_sym_let] = ACTIONS(2129),
    [anon_sym_loop] = ACTIONS(2129),
    [anon_sym_match] = ACTIONS(2129),
    [anon_sym_mod] = ACTIONS(2129),
    [anon_sym_pub] = ACTIONS(2129),
    [anon_sym_return] = ACTIONS(2129),
    [anon_sym_static] = ACTIONS(2129),
    [anon_sym_struct] = ACTIONS(2129),
    [anon_sym_trait] = ACTIONS(2129),
    [anon_sym_type] = ACTIONS(2129),
    [anon_sym_union] = ACTIONS(2129),
    [anon_sym_unsafe] = ACTIONS(2129),
    [anon_sym_use] = ACTIONS(2129),
    [anon_sym_while] = ACTIONS(2129),
    [anon_sym_extern] = ACTIONS(2129),
    [anon_sym_yield] = ACTIONS(2129),
    [anon_sym_move] = ACTIONS(2129),
    [anon_sym_try] = ACTIONS(2129),
    [sym_integer_literal] = ACTIONS(2127),
    [aux_sym_string_literal_token1] = ACTIONS(2127),
    [sym_char_literal] = ACTIONS(2127),
    [anon_sym_true] = ACTIONS(2129),
    [anon_sym_false] = ACTIONS(2129),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2129),
    [sym_super] = ACTIONS(2129),
    [sym_crate] = ACTIONS(2129),
    [sym_metavariable] = ACTIONS(2127),
    [sym__raw_string_literal_start] = ACTIONS(2127),
    [sym_float_literal] = ACTIONS(2127),
  },
  [595] = {
    [sym_line_comment] = STATE(595),
    [sym_block_comment] = STATE(595),
    [ts_builtin_sym_end] = ACTIONS(2131),
    [sym_identifier] = ACTIONS(2133),
    [anon_sym_SEMI] = ACTIONS(2131),
    [anon_sym_macro_rules_BANG] = ACTIONS(2131),
    [anon_sym_LPAREN] = ACTIONS(2131),
    [anon_sym_LBRACK] = ACTIONS(2131),
    [anon_sym_LBRACE] = ACTIONS(2131),
    [anon_sym_RBRACE] = ACTIONS(2131),
    [anon_sym_STAR] = ACTIONS(2131),
    [anon_sym_u8] = ACTIONS(2133),
    [anon_sym_i8] = ACTIONS(2133),
    [anon_sym_u16] = ACTIONS(2133),
    [anon_sym_i16] = ACTIONS(2133),
    [anon_sym_u32] = ACTIONS(2133),
    [anon_sym_i32] = ACTIONS(2133),
    [anon_sym_u64] = ACTIONS(2133),
    [anon_sym_i64] = ACTIONS(2133),
    [anon_sym_u128] = ACTIONS(2133),
    [anon_sym_i128] = ACTIONS(2133),
    [anon_sym_isize] = ACTIONS(2133),
    [anon_sym_usize] = ACTIONS(2133),
    [anon_sym_f32] = ACTIONS(2133),
    [anon_sym_f64] = ACTIONS(2133),
    [anon_sym_bool] = ACTIONS(2133),
    [anon_sym_str] = ACTIONS(2133),
    [anon_sym_char] = ACTIONS(2133),
    [anon_sym_DASH] = ACTIONS(2131),
    [anon_sym_BANG] = ACTIONS(2131),
    [anon_sym_AMP] = ACTIONS(2131),
    [anon_sym_PIPE] = ACTIONS(2131),
    [anon_sym_LT] = ACTIONS(2131),
    [anon_sym_DOT_DOT] = ACTIONS(2131),
    [anon_sym_COLON_COLON] = ACTIONS(2131),
    [anon_sym_POUND] = ACTIONS(2131),
    [anon_sym_SQUOTE] = ACTIONS(2133),
    [anon_sym_async] = ACTIONS(2133),
    [anon_sym_break] = ACTIONS(2133),
    [anon_sym_const] = ACTIONS(2133),
    [anon_sym_continue] = ACTIONS(2133),
    [anon_sym_default] = ACTIONS(2133),
    [anon_sym_enum] = ACTIONS(2133),
    [anon_sym_fn] = ACTIONS(2133),
    [anon_sym_for] = ACTIONS(2133),
    [anon_sym_gen] = ACTIONS(2133),
    [anon_sym_if] = ACTIONS(2133),
    [anon_sym_impl] = ACTIONS(2133),
    [anon_sym_let] = ACTIONS(2133),
    [anon_sym_loop] = ACTIONS(2133),
    [anon_sym_match] = ACTIONS(2133),
    [anon_sym_mod] = ACTIONS(2133),
    [anon_sym_pub] = ACTIONS(2133),
    [anon_sym_return] = ACTIONS(2133),
    [anon_sym_static] = ACTIONS(2133),
    [anon_sym_struct] = ACTIONS(2133),
    [anon_sym_trait] = ACTIONS(2133),
    [anon_sym_type] = ACTIONS(2133),
    [anon_sym_union] = ACTIONS(2133),
    [anon_sym_unsafe] = ACTIONS(2133),
    [anon_sym_use] = ACTIONS(2133),
    [anon_sym_while] = ACTIONS(2133),
    [anon_sym_extern] = ACTIONS(2133),
    [anon_sym_yield] = ACTIONS(2133),
    [anon_sym_move] = ACTIONS(2133),
    [anon_sym_try] = ACTIONS(2133),
    [sym_integer_literal] = ACTIONS(2131),
    [aux_sym_string_literal_token1] = ACTIONS(2131),
    [sym_char_literal] = ACTIONS(2131),
    [anon_sym_true] = ACTIONS(2133),
    [anon_sym_false] = ACTIONS(2133),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2133),
    [sym_super] = ACTIONS(2133),
    [sym_crate] = ACTIONS(2133),
    [sym_metavariable] = ACTIONS(2131),
    [sym__raw_string_literal_start] = ACTIONS(2131),
    [sym_float_literal] = ACTIONS(2131),
  },
  [596] = {
    [sym_line_comment] = STATE(596),
    [sym_block_comment] = STATE(596),
    [ts_builtin_sym_end] = ACTIONS(2135),
    [sym_identifier] = ACTIONS(2137),
    [anon_sym_SEMI] = ACTIONS(2135),
    [anon_sym_macro_rules_BANG] = ACTIONS(2135),
    [anon_sym_LPAREN] = ACTIONS(2135),
    [anon_sym_LBRACK] = ACTIONS(2135),
    [anon_sym_LBRACE] = ACTIONS(2135),
    [anon_sym_RBRACE] = ACTIONS(2135),
    [anon_sym_STAR] = ACTIONS(2135),
    [anon_sym_u8] = ACTIONS(2137),
    [anon_sym_i8] = ACTIONS(2137),
    [anon_sym_u16] = ACTIONS(2137),
    [anon_sym_i16] = ACTIONS(2137),
    [anon_sym_u32] = ACTIONS(2137),
    [anon_sym_i32] = ACTIONS(2137),
    [anon_sym_u64] = ACTIONS(2137),
    [anon_sym_i64] = ACTIONS(2137),
    [anon_sym_u128] = ACTIONS(2137),
    [anon_sym_i128] = ACTIONS(2137),
    [anon_sym_isize] = ACTIONS(2137),
    [anon_sym_usize] = ACTIONS(2137),
    [anon_sym_f32] = ACTIONS(2137),
    [anon_sym_f64] = ACTIONS(2137),
    [anon_sym_bool] = ACTIONS(2137),
    [anon_sym_str] = ACTIONS(2137),
    [anon_sym_char] = ACTIONS(2137),
    [anon_sym_DASH] = ACTIONS(2135),
    [anon_sym_BANG] = ACTIONS(2135),
    [anon_sym_AMP] = ACTIONS(2135),
    [anon_sym_PIPE] = ACTIONS(2135),
    [anon_sym_LT] = ACTIONS(2135),
    [anon_sym_DOT_DOT] = ACTIONS(2135),
    [anon_sym_COLON_COLON] = ACTIONS(2135),
    [anon_sym_POUND] = ACTIONS(2135),
    [anon_sym_SQUOTE] = ACTIONS(2137),
    [anon_sym_async] = ACTIONS(2137),
    [anon_sym_break] = ACTIONS(2137),
    [anon_sym_const] = ACTIONS(2137),
    [anon_sym_continue] = ACTIONS(2137),
    [anon_sym_default] = ACTIONS(2137),
    [anon_sym_enum] = ACTIONS(2137),
    [anon_sym_fn] = ACTIONS(2137),
    [anon_sym_for] = ACTIONS(2137),
    [anon_sym_gen] = ACTIONS(2137),
    [anon_sym_if] = ACTIONS(2137),
    [anon_sym_impl] = ACTIONS(2137),
    [anon_sym_let] = ACTIONS(2137),
    [anon_sym_loop] = ACTIONS(2137),
    [anon_sym_match] = ACTIONS(2137),
    [anon_sym_mod] = ACTIONS(2137),
    [anon_sym_pub] = ACTIONS(2137),
    [anon_sym_return] = ACTIONS(2137),
    [anon_sym_static] = ACTIONS(2137),
    [anon_sym_struct] = ACTIONS(2137),
    [anon_sym_trait] = ACTIONS(2137),
    [anon_sym_type] = ACTIONS(2137),
    [anon_sym_union] = ACTIONS(2137),
    [anon_sym_unsafe] = ACTIONS(2137),
    [anon_sym_use] = ACTIONS(2137),
    [anon_sym_while] = ACTIONS(2137),
    [anon_sym_extern] = ACTIONS(2137),
    [anon_sym_yield] = ACTIONS(2137),
    [anon_sym_move] = ACTIONS(2137),
    [anon_sym_try] = ACTIONS(2137),
    [sym_integer_literal] = ACTIONS(2135),
    [aux_sym_string_literal_token1] = ACTIONS(2135),
    [sym_char_literal] = ACTIONS(2135),
    [anon_sym_true] = ACTIONS(2137),
    [anon_sym_false] = ACTIONS(2137),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2137),
    [sym_super] = ACTIONS(2137),
    [sym_crate] = ACTIONS(2137),
    [sym_metavariable] = ACTIONS(2135),
    [sym__raw_string_literal_start] = ACTIONS(2135),
    [sym_float_literal] = ACTIONS(2135),
  },
  [597] = {
    [sym_line_comment] = STATE(597),
    [sym_block_comment] = STATE(597),
    [ts_builtin_sym_end] = ACTIONS(2139),
    [sym_identifier] = ACTIONS(2141),
    [anon_sym_SEMI] = ACTIONS(2139),
    [anon_sym_macro_rules_BANG] = ACTIONS(2139),
    [anon_sym_LPAREN] = ACTIONS(2139),
    [anon_sym_LBRACK] = ACTIONS(2139),
    [anon_sym_LBRACE] = ACTIONS(2139),
    [anon_sym_RBRACE] = ACTIONS(2139),
    [anon_sym_STAR] = ACTIONS(2139),
    [anon_sym_u8] = ACTIONS(2141),
    [anon_sym_i8] = ACTIONS(2141),
    [anon_sym_u16] = ACTIONS(2141),
    [anon_sym_i16] = ACTIONS(2141),
    [anon_sym_u32] = ACTIONS(2141),
    [anon_sym_i32] = ACTIONS(2141),
    [anon_sym_u64] = ACTIONS(2141),
    [anon_sym_i64] = ACTIONS(2141),
    [anon_sym_u128] = ACTIONS(2141),
    [anon_sym_i128] = ACTIONS(2141),
    [anon_sym_isize] = ACTIONS(2141),
    [anon_sym_usize] = ACTIONS(2141),
    [anon_sym_f32] = ACTIONS(2141),
    [anon_sym_f64] = ACTIONS(2141),
    [anon_sym_bool] = ACTIONS(2141),
    [anon_sym_str] = ACTIONS(2141),
    [anon_sym_char] = ACTIONS(2141),
    [anon_sym_DASH] = ACTIONS(2139),
    [anon_sym_BANG] = ACTIONS(2139),
    [anon_sym_AMP] = ACTIONS(2139),
    [anon_sym_PIPE] = ACTIONS(2139),
    [anon_sym_LT] = ACTIONS(2139),
    [anon_sym_DOT_DOT] = ACTIONS(2139),
    [anon_sym_COLON_COLON] = ACTIONS(2139),
    [anon_sym_POUND] = ACTIONS(2139),
    [anon_sym_SQUOTE] = ACTIONS(2141),
    [anon_sym_async] = ACTIONS(2141),
    [anon_sym_break] = ACTIONS(2141),
    [anon_sym_const] = ACTIONS(2141),
    [anon_sym_continue] = ACTIONS(2141),
    [anon_sym_default] = ACTIONS(2141),
    [anon_sym_enum] = ACTIONS(2141),
    [anon_sym_fn] = ACTIONS(2141),
    [anon_sym_for] = ACTIONS(2141),
    [anon_sym_gen] = ACTIONS(2141),
    [anon_sym_if] = ACTIONS(2141),
    [anon_sym_impl] = ACTIONS(2141),
    [anon_sym_let] = ACTIONS(2141),
    [anon_sym_loop] = ACTIONS(2141),
    [anon_sym_match] = ACTIONS(2141),
    [anon_sym_mod] = ACTIONS(2141),
    [anon_sym_pub] = ACTIONS(2141),
    [anon_sym_return] = ACTIONS(2141),
    [anon_sym_static] = ACTIONS(2141),
    [anon_sym_struct] = ACTIONS(2141),
    [anon_sym_trait] = ACTIONS(2141),
    [anon_sym_type] = ACTIONS(2141),
    [anon_sym_union] = ACTIONS(2141),
    [anon_sym_unsafe] = ACTIONS(2141),
    [anon_sym_use] = ACTIONS(2141),
    [anon_sym_while] = ACTIONS(2141),
    [anon_sym_extern] = ACTIONS(2141),
    [anon_sym_yield] = ACTIONS(2141),
    [anon_sym_move] = ACTIONS(2141),
    [anon_sym_try] = ACTIONS(2141),
    [sym_integer_literal] = ACTIONS(2139),
    [aux_sym_string_literal_token1] = ACTIONS(2139),
    [sym_char_literal] = ACTIONS(2139),
    [anon_sym_true] = ACTIONS(2141),
    [anon_sym_false] = ACTIONS(2141),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2141),
    [sym_super] = ACTIONS(2141),
    [sym_crate] = ACTIONS(2141),
    [sym_metavariable] = ACTIONS(2139),
    [sym__raw_string_literal_start] = ACTIONS(2139),
    [sym_float_literal] = ACTIONS(2139),
  },
  [598] = {
    [sym_line_comment] = STATE(598),
    [sym_block_comment] = STATE(598),
    [ts_builtin_sym_end] = ACTIONS(2143),
    [sym_identifier] = ACTIONS(2145),
    [anon_sym_SEMI] = ACTIONS(2143),
    [anon_sym_macro_rules_BANG] = ACTIONS(2143),
    [anon_sym_LPAREN] = ACTIONS(2143),
    [anon_sym_LBRACK] = ACTIONS(2143),
    [anon_sym_LBRACE] = ACTIONS(2143),
    [anon_sym_RBRACE] = ACTIONS(2143),
    [anon_sym_STAR] = ACTIONS(2143),
    [anon_sym_u8] = ACTIONS(2145),
    [anon_sym_i8] = ACTIONS(2145),
    [anon_sym_u16] = ACTIONS(2145),
    [anon_sym_i16] = ACTIONS(2145),
    [anon_sym_u32] = ACTIONS(2145),
    [anon_sym_i32] = ACTIONS(2145),
    [anon_sym_u64] = ACTIONS(2145),
    [anon_sym_i64] = ACTIONS(2145),
    [anon_sym_u128] = ACTIONS(2145),
    [anon_sym_i128] = ACTIONS(2145),
    [anon_sym_isize] = ACTIONS(2145),
    [anon_sym_usize] = ACTIONS(2145),
    [anon_sym_f32] = ACTIONS(2145),
    [anon_sym_f64] = ACTIONS(2145),
    [anon_sym_bool] = ACTIONS(2145),
    [anon_sym_str] = ACTIONS(2145),
    [anon_sym_char] = ACTIONS(2145),
    [anon_sym_DASH] = ACTIONS(2143),
    [anon_sym_BANG] = ACTIONS(2143),
    [anon_sym_AMP] = ACTIONS(2143),
    [anon_sym_PIPE] = ACTIONS(2143),
    [anon_sym_LT] = ACTIONS(2143),
    [anon_sym_DOT_DOT] = ACTIONS(2143),
    [anon_sym_COLON_COLON] = ACTIONS(2143),
    [anon_sym_POUND] = ACTIONS(2143),
    [anon_sym_SQUOTE] = ACTIONS(2145),
    [anon_sym_async] = ACTIONS(2145),
    [anon_sym_break] = ACTIONS(2145),
    [anon_sym_const] = ACTIONS(2145),
    [anon_sym_continue] = ACTIONS(2145),
    [anon_sym_default] = ACTIONS(2145),
    [anon_sym_enum] = ACTIONS(2145),
    [anon_sym_fn] = ACTIONS(2145),
    [anon_sym_for] = ACTIONS(2145),
    [anon_sym_gen] = ACTIONS(2145),
    [anon_sym_if] = ACTIONS(2145),
    [anon_sym_impl] = ACTIONS(2145),
    [anon_sym_let] = ACTIONS(2145),
    [anon_sym_loop] = ACTIONS(2145),
    [anon_sym_match] = ACTIONS(2145),
    [anon_sym_mod] = ACTIONS(2145),
    [anon_sym_pub] = ACTIONS(2145),
    [anon_sym_return] = ACTIONS(2145),
    [anon_sym_static] = ACTIONS(2145),
    [anon_sym_struct] = ACTIONS(2145),
    [anon_sym_trait] = ACTIONS(2145),
    [anon_sym_type] = ACTIONS(2145),
    [anon_sym_union] = ACTIONS(2145),
    [anon_sym_unsafe] = ACTIONS(2145),
    [anon_sym_use] = ACTIONS(2145),
    [anon_sym_while] = ACTIONS(2145),
    [anon_sym_extern] = ACTIONS(2145),
    [anon_sym_yield] = ACTIONS(2145),
    [anon_sym_move] = ACTIONS(2145),
    [anon_sym_try] = ACTIONS(2145),
    [sym_integer_literal] = ACTIONS(2143),
    [aux_sym_string_literal_token1] = ACTIONS(2143),
    [sym_char_literal] = ACTIONS(2143),
    [anon_sym_true] = ACTIONS(2145),
    [anon_sym_false] = ACTIONS(2145),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2145),
    [sym_super] = ACTIONS(2145),
    [sym_crate] = ACTIONS(2145),
    [sym_metavariable] = ACTIONS(2143),
    [sym__raw_string_literal_start] = ACTIONS(2143),
    [sym_float_literal] = ACTIONS(2143),
  },
  [599] = {
    [sym_line_comment] = STATE(599),
    [sym_block_comment] = STATE(599),
    [ts_builtin_sym_end] = ACTIONS(2147),
    [sym_identifier] = ACTIONS(2149),
    [anon_sym_SEMI] = ACTIONS(2147),
    [anon_sym_macro_rules_BANG] = ACTIONS(2147),
    [anon_sym_LPAREN] = ACTIONS(2147),
    [anon_sym_LBRACK] = ACTIONS(2147),
    [anon_sym_LBRACE] = ACTIONS(2147),
    [anon_sym_RBRACE] = ACTIONS(2147),
    [anon_sym_STAR] = ACTIONS(2147),
    [anon_sym_u8] = ACTIONS(2149),
    [anon_sym_i8] = ACTIONS(2149),
    [anon_sym_u16] = ACTIONS(2149),
    [anon_sym_i16] = ACTIONS(2149),
    [anon_sym_u32] = ACTIONS(2149),
    [anon_sym_i32] = ACTIONS(2149),
    [anon_sym_u64] = ACTIONS(2149),
    [anon_sym_i64] = ACTIONS(2149),
    [anon_sym_u128] = ACTIONS(2149),
    [anon_sym_i128] = ACTIONS(2149),
    [anon_sym_isize] = ACTIONS(2149),
    [anon_sym_usize] = ACTIONS(2149),
    [anon_sym_f32] = ACTIONS(2149),
    [anon_sym_f64] = ACTIONS(2149),
    [anon_sym_bool] = ACTIONS(2149),
    [anon_sym_str] = ACTIONS(2149),
    [anon_sym_char] = ACTIONS(2149),
    [anon_sym_DASH] = ACTIONS(2147),
    [anon_sym_BANG] = ACTIONS(2147),
    [anon_sym_AMP] = ACTIONS(2147),
    [anon_sym_PIPE] = ACTIONS(2147),
    [anon_sym_LT] = ACTIONS(2147),
    [anon_sym_DOT_DOT] = ACTIONS(2147),
    [anon_sym_COLON_COLON] = ACTIONS(2147),
    [anon_sym_POUND] = ACTIONS(2147),
    [anon_sym_SQUOTE] = ACTIONS(2149),
    [anon_sym_async] = ACTIONS(2149),
    [anon_sym_break] = ACTIONS(2149),
    [anon_sym_const] = ACTIONS(2149),
    [anon_sym_continue] = ACTIONS(2149),
    [anon_sym_default] = ACTIONS(2149),
    [anon_sym_enum] = ACTIONS(2149),
    [anon_sym_fn] = ACTIONS(2149),
    [anon_sym_for] = ACTIONS(2149),
    [anon_sym_gen] = ACTIONS(2149),
    [anon_sym_if] = ACTIONS(2149),
    [anon_sym_impl] = ACTIONS(2149),
    [anon_sym_let] = ACTIONS(2149),
    [anon_sym_loop] = ACTIONS(2149),
    [anon_sym_match] = ACTIONS(2149),
    [anon_sym_mod] = ACTIONS(2149),
    [anon_sym_pub] = ACTIONS(2149),
    [anon_sym_return] = ACTIONS(2149),
    [anon_sym_static] = ACTIONS(2149),
    [anon_sym_struct] = ACTIONS(2149),
    [anon_sym_trait] = ACTIONS(2149),
    [anon_sym_type] = ACTIONS(2149),
    [anon_sym_union] = ACTIONS(2149),
    [anon_sym_unsafe] = ACTIONS(2149),
    [anon_sym_use] = ACTIONS(2149),
    [anon_sym_while] = ACTIONS(2149),
    [anon_sym_extern] = ACTIONS(2149),
    [anon_sym_yield] = ACTIONS(2149),
    [anon_sym_move] = ACTIONS(2149),
    [anon_sym_try] = ACTIONS(2149),
    [sym_integer_literal] = ACTIONS(2147),
    [aux_sym_string_literal_token1] = ACTIONS(2147),
    [sym_char_literal] = ACTIONS(2147),
    [anon_sym_true] = ACTIONS(2149),
    [anon_sym_false] = ACTIONS(2149),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2149),
    [sym_super] = ACTIONS(2149),
    [sym_crate] = ACTIONS(2149),
    [sym_metavariable] = ACTIONS(2147),
    [sym__raw_string_literal_start] = ACTIONS(2147),
    [sym_float_literal] = ACTIONS(2147),
  },
  [600] = {
    [sym_line_comment] = STATE(600),
    [sym_block_comment] = STATE(600),
    [ts_builtin_sym_end] = ACTIONS(2151),
    [sym_identifier] = ACTIONS(2153),
    [anon_sym_SEMI] = ACTIONS(2151),
    [anon_sym_macro_rules_BANG] = ACTIONS(2151),
    [anon_sym_LPAREN] = ACTIONS(2151),
    [anon_sym_LBRACK] = ACTIONS(2151),
    [anon_sym_LBRACE] = ACTIONS(2151),
    [anon_sym_RBRACE] = ACTIONS(2151),
    [anon_sym_STAR] = ACTIONS(2151),
    [anon_sym_u8] = ACTIONS(2153),
    [anon_sym_i8] = ACTIONS(2153),
    [anon_sym_u16] = ACTIONS(2153),
    [anon_sym_i16] = ACTIONS(2153),
    [anon_sym_u32] = ACTIONS(2153),
    [anon_sym_i32] = ACTIONS(2153),
    [anon_sym_u64] = ACTIONS(2153),
    [anon_sym_i64] = ACTIONS(2153),
    [anon_sym_u128] = ACTIONS(2153),
    [anon_sym_i128] = ACTIONS(2153),
    [anon_sym_isize] = ACTIONS(2153),
    [anon_sym_usize] = ACTIONS(2153),
    [anon_sym_f32] = ACTIONS(2153),
    [anon_sym_f64] = ACTIONS(2153),
    [anon_sym_bool] = ACTIONS(2153),
    [anon_sym_str] = ACTIONS(2153),
    [anon_sym_char] = ACTIONS(2153),
    [anon_sym_DASH] = ACTIONS(2151),
    [anon_sym_BANG] = ACTIONS(2151),
    [anon_sym_AMP] = ACTIONS(2151),
    [anon_sym_PIPE] = ACTIONS(2151),
    [anon_sym_LT] = ACTIONS(2151),
    [anon_sym_DOT_DOT] = ACTIONS(2151),
    [anon_sym_COLON_COLON] = ACTIONS(2151),
    [anon_sym_POUND] = ACTIONS(2151),
    [anon_sym_SQUOTE] = ACTIONS(2153),
    [anon_sym_async] = ACTIONS(2153),
    [anon_sym_break] = ACTIONS(2153),
    [anon_sym_const] = ACTIONS(2153),
    [anon_sym_continue] = ACTIONS(2153),
    [anon_sym_default] = ACTIONS(2153),
    [anon_sym_enum] = ACTIONS(2153),
    [anon_sym_fn] = ACTIONS(2153),
    [anon_sym_for] = ACTIONS(2153),
    [anon_sym_gen] = ACTIONS(2153),
    [anon_sym_if] = ACTIONS(2153),
    [anon_sym_impl] = ACTIONS(2153),
    [anon_sym_let] = ACTIONS(2153),
    [anon_sym_loop] = ACTIONS(2153),
    [anon_sym_match] = ACTIONS(2153),
    [anon_sym_mod] = ACTIONS(2153),
    [anon_sym_pub] = ACTIONS(2153),
    [anon_sym_return] = ACTIONS(2153),
    [anon_sym_static] = ACTIONS(2153),
    [anon_sym_struct] = ACTIONS(2153),
    [anon_sym_trait] = ACTIONS(2153),
    [anon_sym_type] = ACTIONS(2153),
    [anon_sym_union] = ACTIONS(2153),
    [anon_sym_unsafe] = ACTIONS(2153),
    [anon_sym_use] = ACTIONS(2153),
    [anon_sym_while] = ACTIONS(2153),
    [anon_sym_extern] = ACTIONS(2153),
    [anon_sym_yield] = ACTIONS(2153),
    [anon_sym_move] = ACTIONS(2153),
    [anon_sym_try] = ACTIONS(2153),
    [sym_integer_literal] = ACTIONS(2151),
    [aux_sym_string_literal_token1] = ACTIONS(2151),
    [sym_char_literal] = ACTIONS(2151),
    [anon_sym_true] = ACTIONS(2153),
    [anon_sym_false] = ACTIONS(2153),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2153),
    [sym_super] = ACTIONS(2153),
    [sym_crate] = ACTIONS(2153),
    [sym_metavariable] = ACTIONS(2151),
    [sym__raw_string_literal_start] = ACTIONS(2151),
    [sym_float_literal] = ACTIONS(2151),
  },
  [601] = {
    [sym_line_comment] = STATE(601),
    [sym_block_comment] = STATE(601),
    [ts_builtin_sym_end] = ACTIONS(2155),
    [sym_identifier] = ACTIONS(2157),
    [anon_sym_SEMI] = ACTIONS(2155),
    [anon_sym_macro_rules_BANG] = ACTIONS(2155),
    [anon_sym_LPAREN] = ACTIONS(2155),
    [anon_sym_LBRACK] = ACTIONS(2155),
    [anon_sym_LBRACE] = ACTIONS(2155),
    [anon_sym_RBRACE] = ACTIONS(2155),
    [anon_sym_STAR] = ACTIONS(2155),
    [anon_sym_u8] = ACTIONS(2157),
    [anon_sym_i8] = ACTIONS(2157),
    [anon_sym_u16] = ACTIONS(2157),
    [anon_sym_i16] = ACTIONS(2157),
    [anon_sym_u32] = ACTIONS(2157),
    [anon_sym_i32] = ACTIONS(2157),
    [anon_sym_u64] = ACTIONS(2157),
    [anon_sym_i64] = ACTIONS(2157),
    [anon_sym_u128] = ACTIONS(2157),
    [anon_sym_i128] = ACTIONS(2157),
    [anon_sym_isize] = ACTIONS(2157),
    [anon_sym_usize] = ACTIONS(2157),
    [anon_sym_f32] = ACTIONS(2157),
    [anon_sym_f64] = ACTIONS(2157),
    [anon_sym_bool] = ACTIONS(2157),
    [anon_sym_str] = ACTIONS(2157),
    [anon_sym_char] = ACTIONS(2157),
    [anon_sym_DASH] = ACTIONS(2155),
    [anon_sym_BANG] = ACTIONS(2155),
    [anon_sym_AMP] = ACTIONS(2155),
    [anon_sym_PIPE] = ACTIONS(2155),
    [anon_sym_LT] = ACTIONS(2155),
    [anon_sym_DOT_DOT] = ACTIONS(2155),
    [anon_sym_COLON_COLON] = ACTIONS(2155),
    [anon_sym_POUND] = ACTIONS(2155),
    [anon_sym_SQUOTE] = ACTIONS(2157),
    [anon_sym_async] = ACTIONS(2157),
    [anon_sym_break] = ACTIONS(2157),
    [anon_sym_const] = ACTIONS(2157),
    [anon_sym_continue] = ACTIONS(2157),
    [anon_sym_default] = ACTIONS(2157),
    [anon_sym_enum] = ACTIONS(2157),
    [anon_sym_fn] = ACTIONS(2157),
    [anon_sym_for] = ACTIONS(2157),
    [anon_sym_gen] = ACTIONS(2157),
    [anon_sym_if] = ACTIONS(2157),
    [anon_sym_impl] = ACTIONS(2157),
    [anon_sym_let] = ACTIONS(2157),
    [anon_sym_loop] = ACTIONS(2157),
    [anon_sym_match] = ACTIONS(2157),
    [anon_sym_mod] = ACTIONS(2157),
    [anon_sym_pub] = ACTIONS(2157),
    [anon_sym_return] = ACTIONS(2157),
    [anon_sym_static] = ACTIONS(2157),
    [anon_sym_struct] = ACTIONS(2157),
    [anon_sym_trait] = ACTIONS(2157),
    [anon_sym_type] = ACTIONS(2157),
    [anon_sym_union] = ACTIONS(2157),
    [anon_sym_unsafe] = ACTIONS(2157),
    [anon_sym_use] = ACTIONS(2157),
    [anon_sym_while] = ACTIONS(2157),
    [anon_sym_extern] = ACTIONS(2157),
    [anon_sym_yield] = ACTIONS(2157),
    [anon_sym_move] = ACTIONS(2157),
    [anon_sym_try] = ACTIONS(2157),
    [sym_integer_literal] = ACTIONS(2155),
    [aux_sym_string_literal_token1] = ACTIONS(2155),
    [sym_char_literal] = ACTIONS(2155),
    [anon_sym_true] = ACTIONS(2157),
    [anon_sym_false] = ACTIONS(2157),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2157),
    [sym_super] = ACTIONS(2157),
    [sym_crate] = ACTIONS(2157),
    [sym_metavariable] = ACTIONS(2155),
    [sym__raw_string_literal_start] = ACTIONS(2155),
    [sym_float_literal] = ACTIONS(2155),
  },
  [602] = {
    [sym_line_comment] = STATE(602),
    [sym_block_comment] = STATE(602),
    [ts_builtin_sym_end] = ACTIONS(2159),
    [sym_identifier] = ACTIONS(2161),
    [anon_sym_SEMI] = ACTIONS(2159),
    [anon_sym_macro_rules_BANG] = ACTIONS(2159),
    [anon_sym_LPAREN] = ACTIONS(2159),
    [anon_sym_LBRACK] = ACTIONS(2159),
    [anon_sym_LBRACE] = ACTIONS(2159),
    [anon_sym_RBRACE] = ACTIONS(2159),
    [anon_sym_STAR] = ACTIONS(2159),
    [anon_sym_u8] = ACTIONS(2161),
    [anon_sym_i8] = ACTIONS(2161),
    [anon_sym_u16] = ACTIONS(2161),
    [anon_sym_i16] = ACTIONS(2161),
    [anon_sym_u32] = ACTIONS(2161),
    [anon_sym_i32] = ACTIONS(2161),
    [anon_sym_u64] = ACTIONS(2161),
    [anon_sym_i64] = ACTIONS(2161),
    [anon_sym_u128] = ACTIONS(2161),
    [anon_sym_i128] = ACTIONS(2161),
    [anon_sym_isize] = ACTIONS(2161),
    [anon_sym_usize] = ACTIONS(2161),
    [anon_sym_f32] = ACTIONS(2161),
    [anon_sym_f64] = ACTIONS(2161),
    [anon_sym_bool] = ACTIONS(2161),
    [anon_sym_str] = ACTIONS(2161),
    [anon_sym_char] = ACTIONS(2161),
    [anon_sym_DASH] = ACTIONS(2159),
    [anon_sym_BANG] = ACTIONS(2159),
    [anon_sym_AMP] = ACTIONS(2159),
    [anon_sym_PIPE] = ACTIONS(2159),
    [anon_sym_LT] = ACTIONS(2159),
    [anon_sym_DOT_DOT] = ACTIONS(2159),
    [anon_sym_COLON_COLON] = ACTIONS(2159),
    [anon_sym_POUND] = ACTIONS(2159),
    [anon_sym_SQUOTE] = ACTIONS(2161),
    [anon_sym_async] = ACTIONS(2161),
    [anon_sym_break] = ACTIONS(2161),
    [anon_sym_const] = ACTIONS(2161),
    [anon_sym_continue] = ACTIONS(2161),
    [anon_sym_default] = ACTIONS(2161),
    [anon_sym_enum] = ACTIONS(2161),
    [anon_sym_fn] = ACTIONS(2161),
    [anon_sym_for] = ACTIONS(2161),
    [anon_sym_gen] = ACTIONS(2161),
    [anon_sym_if] = ACTIONS(2161),
    [anon_sym_impl] = ACTIONS(2161),
    [anon_sym_let] = ACTIONS(2161),
    [anon_sym_loop] = ACTIONS(2161),
    [anon_sym_match] = ACTIONS(2161),
    [anon_sym_mod] = ACTIONS(2161),
    [anon_sym_pub] = ACTIONS(2161),
    [anon_sym_return] = ACTIONS(2161),
    [anon_sym_static] = ACTIONS(2161),
    [anon_sym_struct] = ACTIONS(2161),
    [anon_sym_trait] = ACTIONS(2161),
    [anon_sym_type] = ACTIONS(2161),
    [anon_sym_union] = ACTIONS(2161),
    [anon_sym_unsafe] = ACTIONS(2161),
    [anon_sym_use] = ACTIONS(2161),
    [anon_sym_while] = ACTIONS(2161),
    [anon_sym_extern] = ACTIONS(2161),
    [anon_sym_yield] = ACTIONS(2161),
    [anon_sym_move] = ACTIONS(2161),
    [anon_sym_try] = ACTIONS(2161),
    [sym_integer_literal] = ACTIONS(2159),
    [aux_sym_string_literal_token1] = ACTIONS(2159),
    [sym_char_literal] = ACTIONS(2159),
    [anon_sym_true] = ACTIONS(2161),
    [anon_sym_false] = ACTIONS(2161),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2161),
    [sym_super] = ACTIONS(2161),
    [sym_crate] = ACTIONS(2161),
    [sym_metavariable] = ACTIONS(2159),
    [sym__raw_string_literal_start] = ACTIONS(2159),
    [sym_float_literal] = ACTIONS(2159),
  },
  [603] = {
    [sym_line_comment] = STATE(603),
    [sym_block_comment] = STATE(603),
    [ts_builtin_sym_end] = ACTIONS(2163),
    [sym_identifier] = ACTIONS(2165),
    [anon_sym_SEMI] = ACTIONS(2163),
    [anon_sym_macro_rules_BANG] = ACTIONS(2163),
    [anon_sym_LPAREN] = ACTIONS(2163),
    [anon_sym_LBRACK] = ACTIONS(2163),
    [anon_sym_LBRACE] = ACTIONS(2163),
    [anon_sym_RBRACE] = ACTIONS(2163),
    [anon_sym_STAR] = ACTIONS(2163),
    [anon_sym_u8] = ACTIONS(2165),
    [anon_sym_i8] = ACTIONS(2165),
    [anon_sym_u16] = ACTIONS(2165),
    [anon_sym_i16] = ACTIONS(2165),
    [anon_sym_u32] = ACTIONS(2165),
    [anon_sym_i32] = ACTIONS(2165),
    [anon_sym_u64] = ACTIONS(2165),
    [anon_sym_i64] = ACTIONS(2165),
    [anon_sym_u128] = ACTIONS(2165),
    [anon_sym_i128] = ACTIONS(2165),
    [anon_sym_isize] = ACTIONS(2165),
    [anon_sym_usize] = ACTIONS(2165),
    [anon_sym_f32] = ACTIONS(2165),
    [anon_sym_f64] = ACTIONS(2165),
    [anon_sym_bool] = ACTIONS(2165),
    [anon_sym_str] = ACTIONS(2165),
    [anon_sym_char] = ACTIONS(2165),
    [anon_sym_DASH] = ACTIONS(2163),
    [anon_sym_BANG] = ACTIONS(2163),
    [anon_sym_AMP] = ACTIONS(2163),
    [anon_sym_PIPE] = ACTIONS(2163),
    [anon_sym_LT] = ACTIONS(2163),
    [anon_sym_DOT_DOT] = ACTIONS(2163),
    [anon_sym_COLON_COLON] = ACTIONS(2163),
    [anon_sym_POUND] = ACTIONS(2163),
    [anon_sym_SQUOTE] = ACTIONS(2165),
    [anon_sym_async] = ACTIONS(2165),
    [anon_sym_break] = ACTIONS(2165),
    [anon_sym_const] = ACTIONS(2165),
    [anon_sym_continue] = ACTIONS(2165),
    [anon_sym_default] = ACTIONS(2165),
    [anon_sym_enum] = ACTIONS(2165),
    [anon_sym_fn] = ACTIONS(2165),
    [anon_sym_for] = ACTIONS(2165),
    [anon_sym_gen] = ACTIONS(2165),
    [anon_sym_if] = ACTIONS(2165),
    [anon_sym_impl] = ACTIONS(2165),
    [anon_sym_let] = ACTIONS(2165),
    [anon_sym_loop] = ACTIONS(2165),
    [anon_sym_match] = ACTIONS(2165),
    [anon_sym_mod] = ACTIONS(2165),
    [anon_sym_pub] = ACTIONS(2165),
    [anon_sym_return] = ACTIONS(2165),
    [anon_sym_static] = ACTIONS(2165),
    [anon_sym_struct] = ACTIONS(2165),
    [anon_sym_trait] = ACTIONS(2165),
    [anon_sym_type] = ACTIONS(2165),
    [anon_sym_union] = ACTIONS(2165),
    [anon_sym_unsafe] = ACTIONS(2165),
    [anon_sym_use] = ACTIONS(2165),
    [anon_sym_while] = ACTIONS(2165),
    [anon_sym_extern] = ACTIONS(2165),
    [anon_sym_yield] = ACTIONS(2165),
    [anon_sym_move] = ACTIONS(2165),
    [anon_sym_try] = ACTIONS(2165),
    [sym_integer_literal] = ACTIONS(2163),
    [aux_sym_string_literal_token1] = ACTIONS(2163),
    [sym_char_literal] = ACTIONS(2163),
    [anon_sym_true] = ACTIONS(2165),
    [anon_sym_false] = ACTIONS(2165),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2165),
    [sym_super] = ACTIONS(2165),
    [sym_crate] = ACTIONS(2165),
    [sym_metavariable] = ACTIONS(2163),
    [sym__raw_string_literal_start] = ACTIONS(2163),
    [sym_float_literal] = ACTIONS(2163),
  },
  [604] = {
    [sym_line_comment] = STATE(604),
    [sym_block_comment] = STATE(604),
    [ts_builtin_sym_end] = ACTIONS(2167),
    [sym_identifier] = ACTIONS(2169),
    [anon_sym_SEMI] = ACTIONS(2167),
    [anon_sym_macro_rules_BANG] = ACTIONS(2167),
    [anon_sym_LPAREN] = ACTIONS(2167),
    [anon_sym_LBRACK] = ACTIONS(2167),
    [anon_sym_LBRACE] = ACTIONS(2167),
    [anon_sym_RBRACE] = ACTIONS(2167),
    [anon_sym_STAR] = ACTIONS(2167),
    [anon_sym_u8] = ACTIONS(2169),
    [anon_sym_i8] = ACTIONS(2169),
    [anon_sym_u16] = ACTIONS(2169),
    [anon_sym_i16] = ACTIONS(2169),
    [anon_sym_u32] = ACTIONS(2169),
    [anon_sym_i32] = ACTIONS(2169),
    [anon_sym_u64] = ACTIONS(2169),
    [anon_sym_i64] = ACTIONS(2169),
    [anon_sym_u128] = ACTIONS(2169),
    [anon_sym_i128] = ACTIONS(2169),
    [anon_sym_isize] = ACTIONS(2169),
    [anon_sym_usize] = ACTIONS(2169),
    [anon_sym_f32] = ACTIONS(2169),
    [anon_sym_f64] = ACTIONS(2169),
    [anon_sym_bool] = ACTIONS(2169),
    [anon_sym_str] = ACTIONS(2169),
    [anon_sym_char] = ACTIONS(2169),
    [anon_sym_DASH] = ACTIONS(2167),
    [anon_sym_BANG] = ACTIONS(2167),
    [anon_sym_AMP] = ACTIONS(2167),
    [anon_sym_PIPE] = ACTIONS(2167),
    [anon_sym_LT] = ACTIONS(2167),
    [anon_sym_DOT_DOT] = ACTIONS(2167),
    [anon_sym_COLON_COLON] = ACTIONS(2167),
    [anon_sym_POUND] = ACTIONS(2167),
    [anon_sym_SQUOTE] = ACTIONS(2169),
    [anon_sym_async] = ACTIONS(2169),
    [anon_sym_break] = ACTIONS(2169),
    [anon_sym_const] = ACTIONS(2169),
    [anon_sym_continue] = ACTIONS(2169),
    [anon_sym_default] = ACTIONS(2169),
    [anon_sym_enum] = ACTIONS(2169),
    [anon_sym_fn] = ACTIONS(2169),
    [anon_sym_for] = ACTIONS(2169),
    [anon_sym_gen] = ACTIONS(2169),
    [anon_sym_if] = ACTIONS(2169),
    [anon_sym_impl] = ACTIONS(2169),
    [anon_sym_let] = ACTIONS(2169),
    [anon_sym_loop] = ACTIONS(2169),
    [anon_sym_match] = ACTIONS(2169),
    [anon_sym_mod] = ACTIONS(2169),
    [anon_sym_pub] = ACTIONS(2169),
    [anon_sym_return] = ACTIONS(2169),
    [anon_sym_static] = ACTIONS(2169),
    [anon_sym_struct] = ACTIONS(2169),
    [anon_sym_trait] = ACTIONS(2169),
    [anon_sym_type] = ACTIONS(2169),
    [anon_sym_union] = ACTIONS(2169),
    [anon_sym_unsafe] = ACTIONS(2169),
    [anon_sym_use] = ACTIONS(2169),
    [anon_sym_while] = ACTIONS(2169),
    [anon_sym_extern] = ACTIONS(2169),
    [anon_sym_yield] = ACTIONS(2169),
    [anon_sym_move] = ACTIONS(2169),
    [anon_sym_try] = ACTIONS(2169),
    [sym_integer_literal] = ACTIONS(2167),
    [aux_sym_string_literal_token1] = ACTIONS(2167),
    [sym_char_literal] = ACTIONS(2167),
    [anon_sym_true] = ACTIONS(2169),
    [anon_sym_false] = ACTIONS(2169),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2169),
    [sym_super] = ACTIONS(2169),
    [sym_crate] = ACTIONS(2169),
    [sym_metavariable] = ACTIONS(2167),
    [sym__raw_string_literal_start] = ACTIONS(2167),
    [sym_float_literal] = ACTIONS(2167),
  },
  [605] = {
    [sym_line_comment] = STATE(605),
    [sym_block_comment] = STATE(605),
    [ts_builtin_sym_end] = ACTIONS(2171),
    [sym_identifier] = ACTIONS(2173),
    [anon_sym_SEMI] = ACTIONS(2171),
    [anon_sym_macro_rules_BANG] = ACTIONS(2171),
    [anon_sym_LPAREN] = ACTIONS(2171),
    [anon_sym_LBRACK] = ACTIONS(2171),
    [anon_sym_LBRACE] = ACTIONS(2171),
    [anon_sym_RBRACE] = ACTIONS(2171),
    [anon_sym_STAR] = ACTIONS(2171),
    [anon_sym_u8] = ACTIONS(2173),
    [anon_sym_i8] = ACTIONS(2173),
    [anon_sym_u16] = ACTIONS(2173),
    [anon_sym_i16] = ACTIONS(2173),
    [anon_sym_u32] = ACTIONS(2173),
    [anon_sym_i32] = ACTIONS(2173),
    [anon_sym_u64] = ACTIONS(2173),
    [anon_sym_i64] = ACTIONS(2173),
    [anon_sym_u128] = ACTIONS(2173),
    [anon_sym_i128] = ACTIONS(2173),
    [anon_sym_isize] = ACTIONS(2173),
    [anon_sym_usize] = ACTIONS(2173),
    [anon_sym_f32] = ACTIONS(2173),
    [anon_sym_f64] = ACTIONS(2173),
    [anon_sym_bool] = ACTIONS(2173),
    [anon_sym_str] = ACTIONS(2173),
    [anon_sym_char] = ACTIONS(2173),
    [anon_sym_DASH] = ACTIONS(2171),
    [anon_sym_BANG] = ACTIONS(2171),
    [anon_sym_AMP] = ACTIONS(2171),
    [anon_sym_PIPE] = ACTIONS(2171),
    [anon_sym_LT] = ACTIONS(2171),
    [anon_sym_DOT_DOT] = ACTIONS(2171),
    [anon_sym_COLON_COLON] = ACTIONS(2171),
    [anon_sym_POUND] = ACTIONS(2171),
    [anon_sym_SQUOTE] = ACTIONS(2173),
    [anon_sym_async] = ACTIONS(2173),
    [anon_sym_break] = ACTIONS(2173),
    [anon_sym_const] = ACTIONS(2173),
    [anon_sym_continue] = ACTIONS(2173),
    [anon_sym_default] = ACTIONS(2173),
    [anon_sym_enum] = ACTIONS(2173),
    [anon_sym_fn] = ACTIONS(2173),
    [anon_sym_for] = ACTIONS(2173),
    [anon_sym_gen] = ACTIONS(2173),
    [anon_sym_if] = ACTIONS(2173),
    [anon_sym_impl] = ACTIONS(2173),
    [anon_sym_let] = ACTIONS(2173),
    [anon_sym_loop] = ACTIONS(2173),
    [anon_sym_match] = ACTIONS(2173),
    [anon_sym_mod] = ACTIONS(2173),
    [anon_sym_pub] = ACTIONS(2173),
    [anon_sym_return] = ACTIONS(2173),
    [anon_sym_static] = ACTIONS(2173),
    [anon_sym_struct] = ACTIONS(2173),
    [anon_sym_trait] = ACTIONS(2173),
    [anon_sym_type] = ACTIONS(2173),
    [anon_sym_union] = ACTIONS(2173),
    [anon_sym_unsafe] = ACTIONS(2173),
    [anon_sym_use] = ACTIONS(2173),
    [anon_sym_while] = ACTIONS(2173),
    [anon_sym_extern] = ACTIONS(2173),
    [anon_sym_yield] = ACTIONS(2173),
    [anon_sym_move] = ACTIONS(2173),
    [anon_sym_try] = ACTIONS(2173),
    [sym_integer_literal] = ACTIONS(2171),
    [aux_sym_string_literal_token1] = ACTIONS(2171),
    [sym_char_literal] = ACTIONS(2171),
    [anon_sym_true] = ACTIONS(2173),
    [anon_sym_false] = ACTIONS(2173),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2173),
    [sym_super] = ACTIONS(2173),
    [sym_crate] = ACTIONS(2173),
    [sym_metavariable] = ACTIONS(2171),
    [sym__raw_string_literal_start] = ACTIONS(2171),
    [sym_float_literal] = ACTIONS(2171),
  },
  [606] = {
    [sym_line_comment] = STATE(606),
    [sym_block_comment] = STATE(606),
    [ts_builtin_sym_end] = ACTIONS(2175),
    [sym_identifier] = ACTIONS(2177),
    [anon_sym_SEMI] = ACTIONS(2175),
    [anon_sym_macro_rules_BANG] = ACTIONS(2175),
    [anon_sym_LPAREN] = ACTIONS(2175),
    [anon_sym_LBRACK] = ACTIONS(2175),
    [anon_sym_LBRACE] = ACTIONS(2175),
    [anon_sym_RBRACE] = ACTIONS(2175),
    [anon_sym_STAR] = ACTIONS(2175),
    [anon_sym_u8] = ACTIONS(2177),
    [anon_sym_i8] = ACTIONS(2177),
    [anon_sym_u16] = ACTIONS(2177),
    [anon_sym_i16] = ACTIONS(2177),
    [anon_sym_u32] = ACTIONS(2177),
    [anon_sym_i32] = ACTIONS(2177),
    [anon_sym_u64] = ACTIONS(2177),
    [anon_sym_i64] = ACTIONS(2177),
    [anon_sym_u128] = ACTIONS(2177),
    [anon_sym_i128] = ACTIONS(2177),
    [anon_sym_isize] = ACTIONS(2177),
    [anon_sym_usize] = ACTIONS(2177),
    [anon_sym_f32] = ACTIONS(2177),
    [anon_sym_f64] = ACTIONS(2177),
    [anon_sym_bool] = ACTIONS(2177),
    [anon_sym_str] = ACTIONS(2177),
    [anon_sym_char] = ACTIONS(2177),
    [anon_sym_DASH] = ACTIONS(2175),
    [anon_sym_BANG] = ACTIONS(2175),
    [anon_sym_AMP] = ACTIONS(2175),
    [anon_sym_PIPE] = ACTIONS(2175),
    [anon_sym_LT] = ACTIONS(2175),
    [anon_sym_DOT_DOT] = ACTIONS(2175),
    [anon_sym_COLON_COLON] = ACTIONS(2175),
    [anon_sym_POUND] = ACTIONS(2175),
    [anon_sym_SQUOTE] = ACTIONS(2177),
    [anon_sym_async] = ACTIONS(2177),
    [anon_sym_break] = ACTIONS(2177),
    [anon_sym_const] = ACTIONS(2177),
    [anon_sym_continue] = ACTIONS(2177),
    [anon_sym_default] = ACTIONS(2177),
    [anon_sym_enum] = ACTIONS(2177),
    [anon_sym_fn] = ACTIONS(2177),
    [anon_sym_for] = ACTIONS(2177),
    [anon_sym_gen] = ACTIONS(2177),
    [anon_sym_if] = ACTIONS(2177),
    [anon_sym_impl] = ACTIONS(2177),
    [anon_sym_let] = ACTIONS(2177),
    [anon_sym_loop] = ACTIONS(2177),
    [anon_sym_match] = ACTIONS(2177),
    [anon_sym_mod] = ACTIONS(2177),
    [anon_sym_pub] = ACTIONS(2177),
    [anon_sym_return] = ACTIONS(2177),
    [anon_sym_static] = ACTIONS(2177),
    [anon_sym_struct] = ACTIONS(2177),
    [anon_sym_trait] = ACTIONS(2177),
    [anon_sym_type] = ACTIONS(2177),
    [anon_sym_union] = ACTIONS(2177),
    [anon_sym_unsafe] = ACTIONS(2177),
    [anon_sym_use] = ACTIONS(2177),
    [anon_sym_while] = ACTIONS(2177),
    [anon_sym_extern] = ACTIONS(2177),
    [anon_sym_yield] = ACTIONS(2177),
    [anon_sym_move] = ACTIONS(2177),
    [anon_sym_try] = ACTIONS(2177),
    [sym_integer_literal] = ACTIONS(2175),
    [aux_sym_string_literal_token1] = ACTIONS(2175),
    [sym_char_literal] = ACTIONS(2175),
    [anon_sym_true] = ACTIONS(2177),
    [anon_sym_false] = ACTIONS(2177),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2177),
    [sym_super] = ACTIONS(2177),
    [sym_crate] = ACTIONS(2177),
    [sym_metavariable] = ACTIONS(2175),
    [sym__raw_string_literal_start] = ACTIONS(2175),
    [sym_float_literal] = ACTIONS(2175),
  },
  [607] = {
    [sym_line_comment] = STATE(607),
    [sym_block_comment] = STATE(607),
    [ts_builtin_sym_end] = ACTIONS(2179),
    [sym_identifier] = ACTIONS(2181),
    [anon_sym_SEMI] = ACTIONS(2179),
    [anon_sym_macro_rules_BANG] = ACTIONS(2179),
    [anon_sym_LPAREN] = ACTIONS(2179),
    [anon_sym_LBRACK] = ACTIONS(2179),
    [anon_sym_LBRACE] = ACTIONS(2179),
    [anon_sym_RBRACE] = ACTIONS(2179),
    [anon_sym_STAR] = ACTIONS(2179),
    [anon_sym_u8] = ACTIONS(2181),
    [anon_sym_i8] = ACTIONS(2181),
    [anon_sym_u16] = ACTIONS(2181),
    [anon_sym_i16] = ACTIONS(2181),
    [anon_sym_u32] = ACTIONS(2181),
    [anon_sym_i32] = ACTIONS(2181),
    [anon_sym_u64] = ACTIONS(2181),
    [anon_sym_i64] = ACTIONS(2181),
    [anon_sym_u128] = ACTIONS(2181),
    [anon_sym_i128] = ACTIONS(2181),
    [anon_sym_isize] = ACTIONS(2181),
    [anon_sym_usize] = ACTIONS(2181),
    [anon_sym_f32] = ACTIONS(2181),
    [anon_sym_f64] = ACTIONS(2181),
    [anon_sym_bool] = ACTIONS(2181),
    [anon_sym_str] = ACTIONS(2181),
    [anon_sym_char] = ACTIONS(2181),
    [anon_sym_DASH] = ACTIONS(2179),
    [anon_sym_BANG] = ACTIONS(2179),
    [anon_sym_AMP] = ACTIONS(2179),
    [anon_sym_PIPE] = ACTIONS(2179),
    [anon_sym_LT] = ACTIONS(2179),
    [anon_sym_DOT_DOT] = ACTIONS(2179),
    [anon_sym_COLON_COLON] = ACTIONS(2179),
    [anon_sym_POUND] = ACTIONS(2179),
    [anon_sym_SQUOTE] = ACTIONS(2181),
    [anon_sym_async] = ACTIONS(2181),
    [anon_sym_break] = ACTIONS(2181),
    [anon_sym_const] = ACTIONS(2181),
    [anon_sym_continue] = ACTIONS(2181),
    [anon_sym_default] = ACTIONS(2181),
    [anon_sym_enum] = ACTIONS(2181),
    [anon_sym_fn] = ACTIONS(2181),
    [anon_sym_for] = ACTIONS(2181),
    [anon_sym_gen] = ACTIONS(2181),
    [anon_sym_if] = ACTIONS(2181),
    [anon_sym_impl] = ACTIONS(2181),
    [anon_sym_let] = ACTIONS(2181),
    [anon_sym_loop] = ACTIONS(2181),
    [anon_sym_match] = ACTIONS(2181),
    [anon_sym_mod] = ACTIONS(2181),
    [anon_sym_pub] = ACTIONS(2181),
    [anon_sym_return] = ACTIONS(2181),
    [anon_sym_static] = ACTIONS(2181),
    [anon_sym_struct] = ACTIONS(2181),
    [anon_sym_trait] = ACTIONS(2181),
    [anon_sym_type] = ACTIONS(2181),
    [anon_sym_union] = ACTIONS(2181),
    [anon_sym_unsafe] = ACTIONS(2181),
    [anon_sym_use] = ACTIONS(2181),
    [anon_sym_while] = ACTIONS(2181),
    [anon_sym_extern] = ACTIONS(2181),
    [anon_sym_yield] = ACTIONS(2181),
    [anon_sym_move] = ACTIONS(2181),
    [anon_sym_try] = ACTIONS(2181),
    [sym_integer_literal] = ACTIONS(2179),
    [aux_sym_string_literal_token1] = ACTIONS(2179),
    [sym_char_literal] = ACTIONS(2179),
    [anon_sym_true] = ACTIONS(2181),
    [anon_sym_false] = ACTIONS(2181),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2181),
    [sym_super] = ACTIONS(2181),
    [sym_crate] = ACTIONS(2181),
    [sym_metavariable] = ACTIONS(2179),
    [sym__raw_string_literal_start] = ACTIONS(2179),
    [sym_float_literal] = ACTIONS(2179),
  },
  [608] = {
    [sym_line_comment] = STATE(608),
    [sym_block_comment] = STATE(608),
    [ts_builtin_sym_end] = ACTIONS(2183),
    [sym_identifier] = ACTIONS(2185),
    [anon_sym_SEMI] = ACTIONS(2183),
    [anon_sym_macro_rules_BANG] = ACTIONS(2183),
    [anon_sym_LPAREN] = ACTIONS(2183),
    [anon_sym_LBRACK] = ACTIONS(2183),
    [anon_sym_LBRACE] = ACTIONS(2183),
    [anon_sym_RBRACE] = ACTIONS(2183),
    [anon_sym_STAR] = ACTIONS(2183),
    [anon_sym_u8] = ACTIONS(2185),
    [anon_sym_i8] = ACTIONS(2185),
    [anon_sym_u16] = ACTIONS(2185),
    [anon_sym_i16] = ACTIONS(2185),
    [anon_sym_u32] = ACTIONS(2185),
    [anon_sym_i32] = ACTIONS(2185),
    [anon_sym_u64] = ACTIONS(2185),
    [anon_sym_i64] = ACTIONS(2185),
    [anon_sym_u128] = ACTIONS(2185),
    [anon_sym_i128] = ACTIONS(2185),
    [anon_sym_isize] = ACTIONS(2185),
    [anon_sym_usize] = ACTIONS(2185),
    [anon_sym_f32] = ACTIONS(2185),
    [anon_sym_f64] = ACTIONS(2185),
    [anon_sym_bool] = ACTIONS(2185),
    [anon_sym_str] = ACTIONS(2185),
    [anon_sym_char] = ACTIONS(2185),
    [anon_sym_DASH] = ACTIONS(2183),
    [anon_sym_BANG] = ACTIONS(2183),
    [anon_sym_AMP] = ACTIONS(2183),
    [anon_sym_PIPE] = ACTIONS(2183),
    [anon_sym_LT] = ACTIONS(2183),
    [anon_sym_DOT_DOT] = ACTIONS(2183),
    [anon_sym_COLON_COLON] = ACTIONS(2183),
    [anon_sym_POUND] = ACTIONS(2183),
    [anon_sym_SQUOTE] = ACTIONS(2185),
    [anon_sym_async] = ACTIONS(2185),
    [anon_sym_break] = ACTIONS(2185),
    [anon_sym_const] = ACTIONS(2185),
    [anon_sym_continue] = ACTIONS(2185),
    [anon_sym_default] = ACTIONS(2185),
    [anon_sym_enum] = ACTIONS(2185),
    [anon_sym_fn] = ACTIONS(2185),
    [anon_sym_for] = ACTIONS(2185),
    [anon_sym_gen] = ACTIONS(2185),
    [anon_sym_if] = ACTIONS(2185),
    [anon_sym_impl] = ACTIONS(2185),
    [anon_sym_let] = ACTIONS(2185),
    [anon_sym_loop] = ACTIONS(2185),
    [anon_sym_match] = ACTIONS(2185),
    [anon_sym_mod] = ACTIONS(2185),
    [anon_sym_pub] = ACTIONS(2185),
    [anon_sym_return] = ACTIONS(2185),
    [anon_sym_static] = ACTIONS(2185),
    [anon_sym_struct] = ACTIONS(2185),
    [anon_sym_trait] = ACTIONS(2185),
    [anon_sym_type] = ACTIONS(2185),
    [anon_sym_union] = ACTIONS(2185),
    [anon_sym_unsafe] = ACTIONS(2185),
    [anon_sym_use] = ACTIONS(2185),
    [anon_sym_while] = ACTIONS(2185),
    [anon_sym_extern] = ACTIONS(2185),
    [anon_sym_yield] = ACTIONS(2185),
    [anon_sym_move] = ACTIONS(2185),
    [anon_sym_try] = ACTIONS(2185),
    [sym_integer_literal] = ACTIONS(2183),
    [aux_sym_string_literal_token1] = ACTIONS(2183),
    [sym_char_literal] = ACTIONS(2183),
    [anon_sym_true] = ACTIONS(2185),
    [anon_sym_false] = ACTIONS(2185),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2185),
    [sym_super] = ACTIONS(2185),
    [sym_crate] = ACTIONS(2185),
    [sym_metavariable] = ACTIONS(2183),
    [sym__raw_string_literal_start] = ACTIONS(2183),
    [sym_float_literal] = ACTIONS(2183),
  },
  [609] = {
    [sym_line_comment] = STATE(609),
    [sym_block_comment] = STATE(609),
    [ts_builtin_sym_end] = ACTIONS(2187),
    [sym_identifier] = ACTIONS(2189),
    [anon_sym_SEMI] = ACTIONS(2187),
    [anon_sym_macro_rules_BANG] = ACTIONS(2187),
    [anon_sym_LPAREN] = ACTIONS(2187),
    [anon_sym_LBRACK] = ACTIONS(2187),
    [anon_sym_LBRACE] = ACTIONS(2187),
    [anon_sym_RBRACE] = ACTIONS(2187),
    [anon_sym_STAR] = ACTIONS(2187),
    [anon_sym_u8] = ACTIONS(2189),
    [anon_sym_i8] = ACTIONS(2189),
    [anon_sym_u16] = ACTIONS(2189),
    [anon_sym_i16] = ACTIONS(2189),
    [anon_sym_u32] = ACTIONS(2189),
    [anon_sym_i32] = ACTIONS(2189),
    [anon_sym_u64] = ACTIONS(2189),
    [anon_sym_i64] = ACTIONS(2189),
    [anon_sym_u128] = ACTIONS(2189),
    [anon_sym_i128] = ACTIONS(2189),
    [anon_sym_isize] = ACTIONS(2189),
    [anon_sym_usize] = ACTIONS(2189),
    [anon_sym_f32] = ACTIONS(2189),
    [anon_sym_f64] = ACTIONS(2189),
    [anon_sym_bool] = ACTIONS(2189),
    [anon_sym_str] = ACTIONS(2189),
    [anon_sym_char] = ACTIONS(2189),
    [anon_sym_DASH] = ACTIONS(2187),
    [anon_sym_BANG] = ACTIONS(2187),
    [anon_sym_AMP] = ACTIONS(2187),
    [anon_sym_PIPE] = ACTIONS(2187),
    [anon_sym_LT] = ACTIONS(2187),
    [anon_sym_DOT_DOT] = ACTIONS(2187),
    [anon_sym_COLON_COLON] = ACTIONS(2187),
    [anon_sym_POUND] = ACTIONS(2187),
    [anon_sym_SQUOTE] = ACTIONS(2189),
    [anon_sym_async] = ACTIONS(2189),
    [anon_sym_break] = ACTIONS(2189),
    [anon_sym_const] = ACTIONS(2189),
    [anon_sym_continue] = ACTIONS(2189),
    [anon_sym_default] = ACTIONS(2189),
    [anon_sym_enum] = ACTIONS(2189),
    [anon_sym_fn] = ACTIONS(2189),
    [anon_sym_for] = ACTIONS(2189),
    [anon_sym_gen] = ACTIONS(2189),
    [anon_sym_if] = ACTIONS(2189),
    [anon_sym_impl] = ACTIONS(2189),
    [anon_sym_let] = ACTIONS(2189),
    [anon_sym_loop] = ACTIONS(2189),
    [anon_sym_match] = ACTIONS(2189),
    [anon_sym_mod] = ACTIONS(2189),
    [anon_sym_pub] = ACTIONS(2189),
    [anon_sym_return] = ACTIONS(2189),
    [anon_sym_static] = ACTIONS(2189),
    [anon_sym_struct] = ACTIONS(2189),
    [anon_sym_trait] = ACTIONS(2189),
    [anon_sym_type] = ACTIONS(2189),
    [anon_sym_union] = ACTIONS(2189),
    [anon_sym_unsafe] = ACTIONS(2189),
    [anon_sym_use] = ACTIONS(2189),
    [anon_sym_while] = ACTIONS(2189),
    [anon_sym_extern] = ACTIONS(2189),
    [anon_sym_yield] = ACTIONS(2189),
    [anon_sym_move] = ACTIONS(2189),
    [anon_sym_try] = ACTIONS(2189),
    [sym_integer_literal] = ACTIONS(2187),
    [aux_sym_string_literal_token1] = ACTIONS(2187),
    [sym_char_literal] = ACTIONS(2187),
    [anon_sym_true] = ACTIONS(2189),
    [anon_sym_false] = ACTIONS(2189),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2189),
    [sym_super] = ACTIONS(2189),
    [sym_crate] = ACTIONS(2189),
    [sym_metavariable] = ACTIONS(2187),
    [sym__raw_string_literal_start] = ACTIONS(2187),
    [sym_float_literal] = ACTIONS(2187),
  },
  [610] = {
    [sym_line_comment] = STATE(610),
    [sym_block_comment] = STATE(610),
    [ts_builtin_sym_end] = ACTIONS(2191),
    [sym_identifier] = ACTIONS(2193),
    [anon_sym_SEMI] = ACTIONS(2191),
    [anon_sym_macro_rules_BANG] = ACTIONS(2191),
    [anon_sym_LPAREN] = ACTIONS(2191),
    [anon_sym_LBRACK] = ACTIONS(2191),
    [anon_sym_LBRACE] = ACTIONS(2191),
    [anon_sym_RBRACE] = ACTIONS(2191),
    [anon_sym_STAR] = ACTIONS(2191),
    [anon_sym_u8] = ACTIONS(2193),
    [anon_sym_i8] = ACTIONS(2193),
    [anon_sym_u16] = ACTIONS(2193),
    [anon_sym_i16] = ACTIONS(2193),
    [anon_sym_u32] = ACTIONS(2193),
    [anon_sym_i32] = ACTIONS(2193),
    [anon_sym_u64] = ACTIONS(2193),
    [anon_sym_i64] = ACTIONS(2193),
    [anon_sym_u128] = ACTIONS(2193),
    [anon_sym_i128] = ACTIONS(2193),
    [anon_sym_isize] = ACTIONS(2193),
    [anon_sym_usize] = ACTIONS(2193),
    [anon_sym_f32] = ACTIONS(2193),
    [anon_sym_f64] = ACTIONS(2193),
    [anon_sym_bool] = ACTIONS(2193),
    [anon_sym_str] = ACTIONS(2193),
    [anon_sym_char] = ACTIONS(2193),
    [anon_sym_DASH] = ACTIONS(2191),
    [anon_sym_BANG] = ACTIONS(2191),
    [anon_sym_AMP] = ACTIONS(2191),
    [anon_sym_PIPE] = ACTIONS(2191),
    [anon_sym_LT] = ACTIONS(2191),
    [anon_sym_DOT_DOT] = ACTIONS(2191),
    [anon_sym_COLON_COLON] = ACTIONS(2191),
    [anon_sym_POUND] = ACTIONS(2191),
    [anon_sym_SQUOTE] = ACTIONS(2193),
    [anon_sym_async] = ACTIONS(2193),
    [anon_sym_break] = ACTIONS(2193),
    [anon_sym_const] = ACTIONS(2193),
    [anon_sym_continue] = ACTIONS(2193),
    [anon_sym_default] = ACTIONS(2193),
    [anon_sym_enum] = ACTIONS(2193),
    [anon_sym_fn] = ACTIONS(2193),
    [anon_sym_for] = ACTIONS(2193),
    [anon_sym_gen] = ACTIONS(2193),
    [anon_sym_if] = ACTIONS(2193),
    [anon_sym_impl] = ACTIONS(2193),
    [anon_sym_let] = ACTIONS(2193),
    [anon_sym_loop] = ACTIONS(2193),
    [anon_sym_match] = ACTIONS(2193),
    [anon_sym_mod] = ACTIONS(2193),
    [anon_sym_pub] = ACTIONS(2193),
    [anon_sym_return] = ACTIONS(2193),
    [anon_sym_static] = ACTIONS(2193),
    [anon_sym_struct] = ACTIONS(2193),
    [anon_sym_trait] = ACTIONS(2193),
    [anon_sym_type] = ACTIONS(2193),
    [anon_sym_union] = ACTIONS(2193),
    [anon_sym_unsafe] = ACTIONS(2193),
    [anon_sym_use] = ACTIONS(2193),
    [anon_sym_while] = ACTIONS(2193),
    [anon_sym_extern] = ACTIONS(2193),
    [anon_sym_yield] = ACTIONS(2193),
    [anon_sym_move] = ACTIONS(2193),
    [anon_sym_try] = ACTIONS(2193),
    [sym_integer_literal] = ACTIONS(2191),
    [aux_sym_string_literal_token1] = ACTIONS(2191),
    [sym_char_literal] = ACTIONS(2191),
    [anon_sym_true] = ACTIONS(2193),
    [anon_sym_false] = ACTIONS(2193),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2193),
    [sym_super] = ACTIONS(2193),
    [sym_crate] = ACTIONS(2193),
    [sym_metavariable] = ACTIONS(2191),
    [sym__raw_string_literal_start] = ACTIONS(2191),
    [sym_float_literal] = ACTIONS(2191),
  },
  [611] = {
    [sym_line_comment] = STATE(611),
    [sym_block_comment] = STATE(611),
    [ts_builtin_sym_end] = ACTIONS(2195),
    [sym_identifier] = ACTIONS(2197),
    [anon_sym_SEMI] = ACTIONS(2195),
    [anon_sym_macro_rules_BANG] = ACTIONS(2195),
    [anon_sym_LPAREN] = ACTIONS(2195),
    [anon_sym_LBRACK] = ACTIONS(2195),
    [anon_sym_LBRACE] = ACTIONS(2195),
    [anon_sym_RBRACE] = ACTIONS(2195),
    [anon_sym_STAR] = ACTIONS(2195),
    [anon_sym_u8] = ACTIONS(2197),
    [anon_sym_i8] = ACTIONS(2197),
    [anon_sym_u16] = ACTIONS(2197),
    [anon_sym_i16] = ACTIONS(2197),
    [anon_sym_u32] = ACTIONS(2197),
    [anon_sym_i32] = ACTIONS(2197),
    [anon_sym_u64] = ACTIONS(2197),
    [anon_sym_i64] = ACTIONS(2197),
    [anon_sym_u128] = ACTIONS(2197),
    [anon_sym_i128] = ACTIONS(2197),
    [anon_sym_isize] = ACTIONS(2197),
    [anon_sym_usize] = ACTIONS(2197),
    [anon_sym_f32] = ACTIONS(2197),
    [anon_sym_f64] = ACTIONS(2197),
    [anon_sym_bool] = ACTIONS(2197),
    [anon_sym_str] = ACTIONS(2197),
    [anon_sym_char] = ACTIONS(2197),
    [anon_sym_DASH] = ACTIONS(2195),
    [anon_sym_BANG] = ACTIONS(2195),
    [anon_sym_AMP] = ACTIONS(2195),
    [anon_sym_PIPE] = ACTIONS(2195),
    [anon_sym_LT] = ACTIONS(2195),
    [anon_sym_DOT_DOT] = ACTIONS(2195),
    [anon_sym_COLON_COLON] = ACTIONS(2195),
    [anon_sym_POUND] = ACTIONS(2195),
    [anon_sym_SQUOTE] = ACTIONS(2197),
    [anon_sym_async] = ACTIONS(2197),
    [anon_sym_break] = ACTIONS(2197),
    [anon_sym_const] = ACTIONS(2197),
    [anon_sym_continue] = ACTIONS(2197),
    [anon_sym_default] = ACTIONS(2197),
    [anon_sym_enum] = ACTIONS(2197),
    [anon_sym_fn] = ACTIONS(2197),
    [anon_sym_for] = ACTIONS(2197),
    [anon_sym_gen] = ACTIONS(2197),
    [anon_sym_if] = ACTIONS(2197),
    [anon_sym_impl] = ACTIONS(2197),
    [anon_sym_let] = ACTIONS(2197),
    [anon_sym_loop] = ACTIONS(2197),
    [anon_sym_match] = ACTIONS(2197),
    [anon_sym_mod] = ACTIONS(2197),
    [anon_sym_pub] = ACTIONS(2197),
    [anon_sym_return] = ACTIONS(2197),
    [anon_sym_static] = ACTIONS(2197),
    [anon_sym_struct] = ACTIONS(2197),
    [anon_sym_trait] = ACTIONS(2197),
    [anon_sym_type] = ACTIONS(2197),
    [anon_sym_union] = ACTIONS(2197),
    [anon_sym_unsafe] = ACTIONS(2197),
    [anon_sym_use] = ACTIONS(2197),
    [anon_sym_while] = ACTIONS(2197),
    [anon_sym_extern] = ACTIONS(2197),
    [anon_sym_yield] = ACTIONS(2197),
    [anon_sym_move] = ACTIONS(2197),
    [anon_sym_try] = ACTIONS(2197),
    [sym_integer_literal] = ACTIONS(2195),
    [aux_sym_string_literal_token1] = ACTIONS(2195),
    [sym_char_literal] = ACTIONS(2195),
    [anon_sym_true] = ACTIONS(2197),
    [anon_sym_false] = ACTIONS(2197),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2197),
    [sym_super] = ACTIONS(2197),
    [sym_crate] = ACTIONS(2197),
    [sym_metavariable] = ACTIONS(2195),
    [sym__raw_string_literal_start] = ACTIONS(2195),
    [sym_float_literal] = ACTIONS(2195),
  },
  [612] = {
    [sym_line_comment] = STATE(612),
    [sym_block_comment] = STATE(612),
    [ts_builtin_sym_end] = ACTIONS(2199),
    [sym_identifier] = ACTIONS(2201),
    [anon_sym_SEMI] = ACTIONS(2199),
    [anon_sym_macro_rules_BANG] = ACTIONS(2199),
    [anon_sym_LPAREN] = ACTIONS(2199),
    [anon_sym_LBRACK] = ACTIONS(2199),
    [anon_sym_LBRACE] = ACTIONS(2199),
    [anon_sym_RBRACE] = ACTIONS(2199),
    [anon_sym_STAR] = ACTIONS(2199),
    [anon_sym_u8] = ACTIONS(2201),
    [anon_sym_i8] = ACTIONS(2201),
    [anon_sym_u16] = ACTIONS(2201),
    [anon_sym_i16] = ACTIONS(2201),
    [anon_sym_u32] = ACTIONS(2201),
    [anon_sym_i32] = ACTIONS(2201),
    [anon_sym_u64] = ACTIONS(2201),
    [anon_sym_i64] = ACTIONS(2201),
    [anon_sym_u128] = ACTIONS(2201),
    [anon_sym_i128] = ACTIONS(2201),
    [anon_sym_isize] = ACTIONS(2201),
    [anon_sym_usize] = ACTIONS(2201),
    [anon_sym_f32] = ACTIONS(2201),
    [anon_sym_f64] = ACTIONS(2201),
    [anon_sym_bool] = ACTIONS(2201),
    [anon_sym_str] = ACTIONS(2201),
    [anon_sym_char] = ACTIONS(2201),
    [anon_sym_DASH] = ACTIONS(2199),
    [anon_sym_BANG] = ACTIONS(2199),
    [anon_sym_AMP] = ACTIONS(2199),
    [anon_sym_PIPE] = ACTIONS(2199),
    [anon_sym_LT] = ACTIONS(2199),
    [anon_sym_DOT_DOT] = ACTIONS(2199),
    [anon_sym_COLON_COLON] = ACTIONS(2199),
    [anon_sym_POUND] = ACTIONS(2199),
    [anon_sym_SQUOTE] = ACTIONS(2201),
    [anon_sym_async] = ACTIONS(2201),
    [anon_sym_break] = ACTIONS(2201),
    [anon_sym_const] = ACTIONS(2201),
    [anon_sym_continue] = ACTIONS(2201),
    [anon_sym_default] = ACTIONS(2201),
    [anon_sym_enum] = ACTIONS(2201),
    [anon_sym_fn] = ACTIONS(2201),
    [anon_sym_for] = ACTIONS(2201),
    [anon_sym_gen] = ACTIONS(2201),
    [anon_sym_if] = ACTIONS(2201),
    [anon_sym_impl] = ACTIONS(2201),
    [anon_sym_let] = ACTIONS(2201),
    [anon_sym_loop] = ACTIONS(2201),
    [anon_sym_match] = ACTIONS(2201),
    [anon_sym_mod] = ACTIONS(2201),
    [anon_sym_pub] = ACTIONS(2201),
    [anon_sym_return] = ACTIONS(2201),
    [anon_sym_static] = ACTIONS(2201),
    [anon_sym_struct] = ACTIONS(2201),
    [anon_sym_trait] = ACTIONS(2201),
    [anon_sym_type] = ACTIONS(2201),
    [anon_sym_union] = ACTIONS(2201),
    [anon_sym_unsafe] = ACTIONS(2201),
    [anon_sym_use] = ACTIONS(2201),
    [anon_sym_while] = ACTIONS(2201),
    [anon_sym_extern] = ACTIONS(2201),
    [anon_sym_yield] = ACTIONS(2201),
    [anon_sym_move] = ACTIONS(2201),
    [anon_sym_try] = ACTIONS(2201),
    [sym_integer_literal] = ACTIONS(2199),
    [aux_sym_string_literal_token1] = ACTIONS(2199),
    [sym_char_literal] = ACTIONS(2199),
    [anon_sym_true] = ACTIONS(2201),
    [anon_sym_false] = ACTIONS(2201),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2201),
    [sym_super] = ACTIONS(2201),
    [sym_crate] = ACTIONS(2201),
    [sym_metavariable] = ACTIONS(2199),
    [sym__raw_string_literal_start] = ACTIONS(2199),
    [sym_float_literal] = ACTIONS(2199),
  },
  [613] = {
    [sym_line_comment] = STATE(613),
    [sym_block_comment] = STATE(613),
    [ts_builtin_sym_end] = ACTIONS(2203),
    [sym_identifier] = ACTIONS(2205),
    [anon_sym_SEMI] = ACTIONS(2203),
    [anon_sym_macro_rules_BANG] = ACTIONS(2203),
    [anon_sym_LPAREN] = ACTIONS(2203),
    [anon_sym_LBRACK] = ACTIONS(2203),
    [anon_sym_LBRACE] = ACTIONS(2203),
    [anon_sym_RBRACE] = ACTIONS(2203),
    [anon_sym_STAR] = ACTIONS(2203),
    [anon_sym_u8] = ACTIONS(2205),
    [anon_sym_i8] = ACTIONS(2205),
    [anon_sym_u16] = ACTIONS(2205),
    [anon_sym_i16] = ACTIONS(2205),
    [anon_sym_u32] = ACTIONS(2205),
    [anon_sym_i32] = ACTIONS(2205),
    [anon_sym_u64] = ACTIONS(2205),
    [anon_sym_i64] = ACTIONS(2205),
    [anon_sym_u128] = ACTIONS(2205),
    [anon_sym_i128] = ACTIONS(2205),
    [anon_sym_isize] = ACTIONS(2205),
    [anon_sym_usize] = ACTIONS(2205),
    [anon_sym_f32] = ACTIONS(2205),
    [anon_sym_f64] = ACTIONS(2205),
    [anon_sym_bool] = ACTIONS(2205),
    [anon_sym_str] = ACTIONS(2205),
    [anon_sym_char] = ACTIONS(2205),
    [anon_sym_DASH] = ACTIONS(2203),
    [anon_sym_BANG] = ACTIONS(2203),
    [anon_sym_AMP] = ACTIONS(2203),
    [anon_sym_PIPE] = ACTIONS(2203),
    [anon_sym_LT] = ACTIONS(2203),
    [anon_sym_DOT_DOT] = ACTIONS(2203),
    [anon_sym_COLON_COLON] = ACTIONS(2203),
    [anon_sym_POUND] = ACTIONS(2203),
    [anon_sym_SQUOTE] = ACTIONS(2205),
    [anon_sym_async] = ACTIONS(2205),
    [anon_sym_break] = ACTIONS(2205),
    [anon_sym_const] = ACTIONS(2205),
    [anon_sym_continue] = ACTIONS(2205),
    [anon_sym_default] = ACTIONS(2205),
    [anon_sym_enum] = ACTIONS(2205),
    [anon_sym_fn] = ACTIONS(2205),
    [anon_sym_for] = ACTIONS(2205),
    [anon_sym_gen] = ACTIONS(2205),
    [anon_sym_if] = ACTIONS(2205),
    [anon_sym_impl] = ACTIONS(2205),
    [anon_sym_let] = ACTIONS(2205),
    [anon_sym_loop] = ACTIONS(2205),
    [anon_sym_match] = ACTIONS(2205),
    [anon_sym_mod] = ACTIONS(2205),
    [anon_sym_pub] = ACTIONS(2205),
    [anon_sym_return] = ACTIONS(2205),
    [anon_sym_static] = ACTIONS(2205),
    [anon_sym_struct] = ACTIONS(2205),
    [anon_sym_trait] = ACTIONS(2205),
    [anon_sym_type] = ACTIONS(2205),
    [anon_sym_union] = ACTIONS(2205),
    [anon_sym_unsafe] = ACTIONS(2205),
    [anon_sym_use] = ACTIONS(2205),
    [anon_sym_while] = ACTIONS(2205),
    [anon_sym_extern] = ACTIONS(2205),
    [anon_sym_yield] = ACTIONS(2205),
    [anon_sym_move] = ACTIONS(2205),
    [anon_sym_try] = ACTIONS(2205),
    [sym_integer_literal] = ACTIONS(2203),
    [aux_sym_string_literal_token1] = ACTIONS(2203),
    [sym_char_literal] = ACTIONS(2203),
    [anon_sym_true] = ACTIONS(2205),
    [anon_sym_false] = ACTIONS(2205),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2205),
    [sym_super] = ACTIONS(2205),
    [sym_crate] = ACTIONS(2205),
    [sym_metavariable] = ACTIONS(2203),
    [sym__raw_string_literal_start] = ACTIONS(2203),
    [sym_float_literal] = ACTIONS(2203),
  },
  [614] = {
    [sym_line_comment] = STATE(614),
    [sym_block_comment] = STATE(614),
    [ts_builtin_sym_end] = ACTIONS(2207),
    [sym_identifier] = ACTIONS(2209),
    [anon_sym_SEMI] = ACTIONS(2207),
    [anon_sym_macro_rules_BANG] = ACTIONS(2207),
    [anon_sym_LPAREN] = ACTIONS(2207),
    [anon_sym_LBRACK] = ACTIONS(2207),
    [anon_sym_LBRACE] = ACTIONS(2207),
    [anon_sym_RBRACE] = ACTIONS(2207),
    [anon_sym_STAR] = ACTIONS(2207),
    [anon_sym_u8] = ACTIONS(2209),
    [anon_sym_i8] = ACTIONS(2209),
    [anon_sym_u16] = ACTIONS(2209),
    [anon_sym_i16] = ACTIONS(2209),
    [anon_sym_u32] = ACTIONS(2209),
    [anon_sym_i32] = ACTIONS(2209),
    [anon_sym_u64] = ACTIONS(2209),
    [anon_sym_i64] = ACTIONS(2209),
    [anon_sym_u128] = ACTIONS(2209),
    [anon_sym_i128] = ACTIONS(2209),
    [anon_sym_isize] = ACTIONS(2209),
    [anon_sym_usize] = ACTIONS(2209),
    [anon_sym_f32] = ACTIONS(2209),
    [anon_sym_f64] = ACTIONS(2209),
    [anon_sym_bool] = ACTIONS(2209),
    [anon_sym_str] = ACTIONS(2209),
    [anon_sym_char] = ACTIONS(2209),
    [anon_sym_DASH] = ACTIONS(2207),
    [anon_sym_BANG] = ACTIONS(2207),
    [anon_sym_AMP] = ACTIONS(2207),
    [anon_sym_PIPE] = ACTIONS(2207),
    [anon_sym_LT] = ACTIONS(2207),
    [anon_sym_DOT_DOT] = ACTIONS(2207),
    [anon_sym_COLON_COLON] = ACTIONS(2207),
    [anon_sym_POUND] = ACTIONS(2207),
    [anon_sym_SQUOTE] = ACTIONS(2209),
    [anon_sym_async] = ACTIONS(2209),
    [anon_sym_break] = ACTIONS(2209),
    [anon_sym_const] = ACTIONS(2209),
    [anon_sym_continue] = ACTIONS(2209),
    [anon_sym_default] = ACTIONS(2209),
    [anon_sym_enum] = ACTIONS(2209),
    [anon_sym_fn] = ACTIONS(2209),
    [anon_sym_for] = ACTIONS(2209),
    [anon_sym_gen] = ACTIONS(2209),
    [anon_sym_if] = ACTIONS(2209),
    [anon_sym_impl] = ACTIONS(2209),
    [anon_sym_let] = ACTIONS(2209),
    [anon_sym_loop] = ACTIONS(2209),
    [anon_sym_match] = ACTIONS(2209),
    [anon_sym_mod] = ACTIONS(2209),
    [anon_sym_pub] = ACTIONS(2209),
    [anon_sym_return] = ACTIONS(2209),
    [anon_sym_static] = ACTIONS(2209),
    [anon_sym_struct] = ACTIONS(2209),
    [anon_sym_trait] = ACTIONS(2209),
    [anon_sym_type] = ACTIONS(2209),
    [anon_sym_union] = ACTIONS(2209),
    [anon_sym_unsafe] = ACTIONS(2209),
    [anon_sym_use] = ACTIONS(2209),
    [anon_sym_while] = ACTIONS(2209),
    [anon_sym_extern] = ACTIONS(2209),
    [anon_sym_yield] = ACTIONS(2209),
    [anon_sym_move] = ACTIONS(2209),
    [anon_sym_try] = ACTIONS(2209),
    [sym_integer_literal] = ACTIONS(2207),
    [aux_sym_string_literal_token1] = ACTIONS(2207),
    [sym_char_literal] = ACTIONS(2207),
    [anon_sym_true] = ACTIONS(2209),
    [anon_sym_false] = ACTIONS(2209),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2209),
    [sym_super] = ACTIONS(2209),
    [sym_crate] = ACTIONS(2209),
    [sym_metavariable] = ACTIONS(2207),
    [sym__raw_string_literal_start] = ACTIONS(2207),
    [sym_float_literal] = ACTIONS(2207),
  },
  [615] = {
    [sym_line_comment] = STATE(615),
    [sym_block_comment] = STATE(615),
    [ts_builtin_sym_end] = ACTIONS(2211),
    [sym_identifier] = ACTIONS(2213),
    [anon_sym_SEMI] = ACTIONS(2211),
    [anon_sym_macro_rules_BANG] = ACTIONS(2211),
    [anon_sym_LPAREN] = ACTIONS(2211),
    [anon_sym_LBRACK] = ACTIONS(2211),
    [anon_sym_LBRACE] = ACTIONS(2211),
    [anon_sym_RBRACE] = ACTIONS(2211),
    [anon_sym_STAR] = ACTIONS(2211),
    [anon_sym_u8] = ACTIONS(2213),
    [anon_sym_i8] = ACTIONS(2213),
    [anon_sym_u16] = ACTIONS(2213),
    [anon_sym_i16] = ACTIONS(2213),
    [anon_sym_u32] = ACTIONS(2213),
    [anon_sym_i32] = ACTIONS(2213),
    [anon_sym_u64] = ACTIONS(2213),
    [anon_sym_i64] = ACTIONS(2213),
    [anon_sym_u128] = ACTIONS(2213),
    [anon_sym_i128] = ACTIONS(2213),
    [anon_sym_isize] = ACTIONS(2213),
    [anon_sym_usize] = ACTIONS(2213),
    [anon_sym_f32] = ACTIONS(2213),
    [anon_sym_f64] = ACTIONS(2213),
    [anon_sym_bool] = ACTIONS(2213),
    [anon_sym_str] = ACTIONS(2213),
    [anon_sym_char] = ACTIONS(2213),
    [anon_sym_DASH] = ACTIONS(2211),
    [anon_sym_BANG] = ACTIONS(2211),
    [anon_sym_AMP] = ACTIONS(2211),
    [anon_sym_PIPE] = ACTIONS(2211),
    [anon_sym_LT] = ACTIONS(2211),
    [anon_sym_DOT_DOT] = ACTIONS(2211),
    [anon_sym_COLON_COLON] = ACTIONS(2211),
    [anon_sym_POUND] = ACTIONS(2211),
    [anon_sym_SQUOTE] = ACTIONS(2213),
    [anon_sym_async] = ACTIONS(2213),
    [anon_sym_break] = ACTIONS(2213),
    [anon_sym_const] = ACTIONS(2213),
    [anon_sym_continue] = ACTIONS(2213),
    [anon_sym_default] = ACTIONS(2213),
    [anon_sym_enum] = ACTIONS(2213),
    [anon_sym_fn] = ACTIONS(2213),
    [anon_sym_for] = ACTIONS(2213),
    [anon_sym_gen] = ACTIONS(2213),
    [anon_sym_if] = ACTIONS(2213),
    [anon_sym_impl] = ACTIONS(2213),
    [anon_sym_let] = ACTIONS(2213),
    [anon_sym_loop] = ACTIONS(2213),
    [anon_sym_match] = ACTIONS(2213),
    [anon_sym_mod] = ACTIONS(2213),
    [anon_sym_pub] = ACTIONS(2213),
    [anon_sym_return] = ACTIONS(2213),
    [anon_sym_static] = ACTIONS(2213),
    [anon_sym_struct] = ACTIONS(2213),
    [anon_sym_trait] = ACTIONS(2213),
    [anon_sym_type] = ACTIONS(2213),
    [anon_sym_union] = ACTIONS(2213),
    [anon_sym_unsafe] = ACTIONS(2213),
    [anon_sym_use] = ACTIONS(2213),
    [anon_sym_while] = ACTIONS(2213),
    [anon_sym_extern] = ACTIONS(2213),
    [anon_sym_yield] = ACTIONS(2213),
    [anon_sym_move] = ACTIONS(2213),
    [anon_sym_try] = ACTIONS(2213),
    [sym_integer_literal] = ACTIONS(2211),
    [aux_sym_string_literal_token1] = ACTIONS(2211),
    [sym_char_literal] = ACTIONS(2211),
    [anon_sym_true] = ACTIONS(2213),
    [anon_sym_false] = ACTIONS(2213),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2213),
    [sym_super] = ACTIONS(2213),
    [sym_crate] = ACTIONS(2213),
    [sym_metavariable] = ACTIONS(2211),
    [sym__raw_string_literal_start] = ACTIONS(2211),
    [sym_float_literal] = ACTIONS(2211),
  },
  [616] = {
    [sym_line_comment] = STATE(616),
    [sym_block_comment] = STATE(616),
    [ts_builtin_sym_end] = ACTIONS(2215),
    [sym_identifier] = ACTIONS(2217),
    [anon_sym_SEMI] = ACTIONS(2215),
    [anon_sym_macro_rules_BANG] = ACTIONS(2215),
    [anon_sym_LPAREN] = ACTIONS(2215),
    [anon_sym_LBRACK] = ACTIONS(2215),
    [anon_sym_LBRACE] = ACTIONS(2215),
    [anon_sym_RBRACE] = ACTIONS(2215),
    [anon_sym_STAR] = ACTIONS(2215),
    [anon_sym_u8] = ACTIONS(2217),
    [anon_sym_i8] = ACTIONS(2217),
    [anon_sym_u16] = ACTIONS(2217),
    [anon_sym_i16] = ACTIONS(2217),
    [anon_sym_u32] = ACTIONS(2217),
    [anon_sym_i32] = ACTIONS(2217),
    [anon_sym_u64] = ACTIONS(2217),
    [anon_sym_i64] = ACTIONS(2217),
    [anon_sym_u128] = ACTIONS(2217),
    [anon_sym_i128] = ACTIONS(2217),
    [anon_sym_isize] = ACTIONS(2217),
    [anon_sym_usize] = ACTIONS(2217),
    [anon_sym_f32] = ACTIONS(2217),
    [anon_sym_f64] = ACTIONS(2217),
    [anon_sym_bool] = ACTIONS(2217),
    [anon_sym_str] = ACTIONS(2217),
    [anon_sym_char] = ACTIONS(2217),
    [anon_sym_DASH] = ACTIONS(2215),
    [anon_sym_BANG] = ACTIONS(2215),
    [anon_sym_AMP] = ACTIONS(2215),
    [anon_sym_PIPE] = ACTIONS(2215),
    [anon_sym_LT] = ACTIONS(2215),
    [anon_sym_DOT_DOT] = ACTIONS(2215),
    [anon_sym_COLON_COLON] = ACTIONS(2215),
    [anon_sym_POUND] = ACTIONS(2215),
    [anon_sym_SQUOTE] = ACTIONS(2217),
    [anon_sym_async] = ACTIONS(2217),
    [anon_sym_break] = ACTIONS(2217),
    [anon_sym_const] = ACTIONS(2217),
    [anon_sym_continue] = ACTIONS(2217),
    [anon_sym_default] = ACTIONS(2217),
    [anon_sym_enum] = ACTIONS(2217),
    [anon_sym_fn] = ACTIONS(2217),
    [anon_sym_for] = ACTIONS(2217),
    [anon_sym_gen] = ACTIONS(2217),
    [anon_sym_if] = ACTIONS(2217),
    [anon_sym_impl] = ACTIONS(2217),
    [anon_sym_let] = ACTIONS(2217),
    [anon_sym_loop] = ACTIONS(2217),
    [anon_sym_match] = ACTIONS(2217),
    [anon_sym_mod] = ACTIONS(2217),
    [anon_sym_pub] = ACTIONS(2217),
    [anon_sym_return] = ACTIONS(2217),
    [anon_sym_static] = ACTIONS(2217),
    [anon_sym_struct] = ACTIONS(2217),
    [anon_sym_trait] = ACTIONS(2217),
    [anon_sym_type] = ACTIONS(2217),
    [anon_sym_union] = ACTIONS(2217),
    [anon_sym_unsafe] = ACTIONS(2217),
    [anon_sym_use] = ACTIONS(2217),
    [anon_sym_while] = ACTIONS(2217),
    [anon_sym_extern] = ACTIONS(2217),
    [anon_sym_yield] = ACTIONS(2217),
    [anon_sym_move] = ACTIONS(2217),
    [anon_sym_try] = ACTIONS(2217),
    [sym_integer_literal] = ACTIONS(2215),
    [aux_sym_string_literal_token1] = ACTIONS(2215),
    [sym_char_literal] = ACTIONS(2215),
    [anon_sym_true] = ACTIONS(2217),
    [anon_sym_false] = ACTIONS(2217),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2217),
    [sym_super] = ACTIONS(2217),
    [sym_crate] = ACTIONS(2217),
    [sym_metavariable] = ACTIONS(2215),
    [sym__raw_string_literal_start] = ACTIONS(2215),
    [sym_float_literal] = ACTIONS(2215),
  },
  [617] = {
    [sym_line_comment] = STATE(617),
    [sym_block_comment] = STATE(617),
    [ts_builtin_sym_end] = ACTIONS(2219),
    [sym_identifier] = ACTIONS(2221),
    [anon_sym_SEMI] = ACTIONS(2219),
    [anon_sym_macro_rules_BANG] = ACTIONS(2219),
    [anon_sym_LPAREN] = ACTIONS(2219),
    [anon_sym_LBRACK] = ACTIONS(2219),
    [anon_sym_LBRACE] = ACTIONS(2219),
    [anon_sym_RBRACE] = ACTIONS(2219),
    [anon_sym_STAR] = ACTIONS(2219),
    [anon_sym_u8] = ACTIONS(2221),
    [anon_sym_i8] = ACTIONS(2221),
    [anon_sym_u16] = ACTIONS(2221),
    [anon_sym_i16] = ACTIONS(2221),
    [anon_sym_u32] = ACTIONS(2221),
    [anon_sym_i32] = ACTIONS(2221),
    [anon_sym_u64] = ACTIONS(2221),
    [anon_sym_i64] = ACTIONS(2221),
    [anon_sym_u128] = ACTIONS(2221),
    [anon_sym_i128] = ACTIONS(2221),
    [anon_sym_isize] = ACTIONS(2221),
    [anon_sym_usize] = ACTIONS(2221),
    [anon_sym_f32] = ACTIONS(2221),
    [anon_sym_f64] = ACTIONS(2221),
    [anon_sym_bool] = ACTIONS(2221),
    [anon_sym_str] = ACTIONS(2221),
    [anon_sym_char] = ACTIONS(2221),
    [anon_sym_DASH] = ACTIONS(2219),
    [anon_sym_BANG] = ACTIONS(2219),
    [anon_sym_AMP] = ACTIONS(2219),
    [anon_sym_PIPE] = ACTIONS(2219),
    [anon_sym_LT] = ACTIONS(2219),
    [anon_sym_DOT_DOT] = ACTIONS(2219),
    [anon_sym_COLON_COLON] = ACTIONS(2219),
    [anon_sym_POUND] = ACTIONS(2219),
    [anon_sym_SQUOTE] = ACTIONS(2221),
    [anon_sym_async] = ACTIONS(2221),
    [anon_sym_break] = ACTIONS(2221),
    [anon_sym_const] = ACTIONS(2221),
    [anon_sym_continue] = ACTIONS(2221),
    [anon_sym_default] = ACTIONS(2221),
    [anon_sym_enum] = ACTIONS(2221),
    [anon_sym_fn] = ACTIONS(2221),
    [anon_sym_for] = ACTIONS(2221),
    [anon_sym_gen] = ACTIONS(2221),
    [anon_sym_if] = ACTIONS(2221),
    [anon_sym_impl] = ACTIONS(2221),
    [anon_sym_let] = ACTIONS(2221),
    [anon_sym_loop] = ACTIONS(2221),
    [anon_sym_match] = ACTIONS(2221),
    [anon_sym_mod] = ACTIONS(2221),
    [anon_sym_pub] = ACTIONS(2221),
    [anon_sym_return] = ACTIONS(2221),
    [anon_sym_static] = ACTIONS(2221),
    [anon_sym_struct] = ACTIONS(2221),
    [anon_sym_trait] = ACTIONS(2221),
    [anon_sym_type] = ACTIONS(2221),
    [anon_sym_union] = ACTIONS(2221),
    [anon_sym_unsafe] = ACTIONS(2221),
    [anon_sym_use] = ACTIONS(2221),
    [anon_sym_while] = ACTIONS(2221),
    [anon_sym_extern] = ACTIONS(2221),
    [anon_sym_yield] = ACTIONS(2221),
    [anon_sym_move] = ACTIONS(2221),
    [anon_sym_try] = ACTIONS(2221),
    [sym_integer_literal] = ACTIONS(2219),
    [aux_sym_string_literal_token1] = ACTIONS(2219),
    [sym_char_literal] = ACTIONS(2219),
    [anon_sym_true] = ACTIONS(2221),
    [anon_sym_false] = ACTIONS(2221),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2221),
    [sym_super] = ACTIONS(2221),
    [sym_crate] = ACTIONS(2221),
    [sym_metavariable] = ACTIONS(2219),
    [sym__raw_string_literal_start] = ACTIONS(2219),
    [sym_float_literal] = ACTIONS(2219),
  },
  [618] = {
    [sym_line_comment] = STATE(618),
    [sym_block_comment] = STATE(618),
    [ts_builtin_sym_end] = ACTIONS(2223),
    [sym_identifier] = ACTIONS(2225),
    [anon_sym_SEMI] = ACTIONS(2223),
    [anon_sym_macro_rules_BANG] = ACTIONS(2223),
    [anon_sym_LPAREN] = ACTIONS(2223),
    [anon_sym_LBRACK] = ACTIONS(2223),
    [anon_sym_LBRACE] = ACTIONS(2223),
    [anon_sym_RBRACE] = ACTIONS(2223),
    [anon_sym_STAR] = ACTIONS(2223),
    [anon_sym_u8] = ACTIONS(2225),
    [anon_sym_i8] = ACTIONS(2225),
    [anon_sym_u16] = ACTIONS(2225),
    [anon_sym_i16] = ACTIONS(2225),
    [anon_sym_u32] = ACTIONS(2225),
    [anon_sym_i32] = ACTIONS(2225),
    [anon_sym_u64] = ACTIONS(2225),
    [anon_sym_i64] = ACTIONS(2225),
    [anon_sym_u128] = ACTIONS(2225),
    [anon_sym_i128] = ACTIONS(2225),
    [anon_sym_isize] = ACTIONS(2225),
    [anon_sym_usize] = ACTIONS(2225),
    [anon_sym_f32] = ACTIONS(2225),
    [anon_sym_f64] = ACTIONS(2225),
    [anon_sym_bool] = ACTIONS(2225),
    [anon_sym_str] = ACTIONS(2225),
    [anon_sym_char] = ACTIONS(2225),
    [anon_sym_DASH] = ACTIONS(2223),
    [anon_sym_BANG] = ACTIONS(2223),
    [anon_sym_AMP] = ACTIONS(2223),
    [anon_sym_PIPE] = ACTIONS(2223),
    [anon_sym_LT] = ACTIONS(2223),
    [anon_sym_DOT_DOT] = ACTIONS(2223),
    [anon_sym_COLON_COLON] = ACTIONS(2223),
    [anon_sym_POUND] = ACTIONS(2223),
    [anon_sym_SQUOTE] = ACTIONS(2225),
    [anon_sym_async] = ACTIONS(2225),
    [anon_sym_break] = ACTIONS(2225),
    [anon_sym_const] = ACTIONS(2225),
    [anon_sym_continue] = ACTIONS(2225),
    [anon_sym_default] = ACTIONS(2225),
    [anon_sym_enum] = ACTIONS(2225),
    [anon_sym_fn] = ACTIONS(2225),
    [anon_sym_for] = ACTIONS(2225),
    [anon_sym_gen] = ACTIONS(2225),
    [anon_sym_if] = ACTIONS(2225),
    [anon_sym_impl] = ACTIONS(2225),
    [anon_sym_let] = ACTIONS(2225),
    [anon_sym_loop] = ACTIONS(2225),
    [anon_sym_match] = ACTIONS(2225),
    [anon_sym_mod] = ACTIONS(2225),
    [anon_sym_pub] = ACTIONS(2225),
    [anon_sym_return] = ACTIONS(2225),
    [anon_sym_static] = ACTIONS(2225),
    [anon_sym_struct] = ACTIONS(2225),
    [anon_sym_trait] = ACTIONS(2225),
    [anon_sym_type] = ACTIONS(2225),
    [anon_sym_union] = ACTIONS(2225),
    [anon_sym_unsafe] = ACTIONS(2225),
    [anon_sym_use] = ACTIONS(2225),
    [anon_sym_while] = ACTIONS(2225),
    [anon_sym_extern] = ACTIONS(2225),
    [anon_sym_yield] = ACTIONS(2225),
    [anon_sym_move] = ACTIONS(2225),
    [anon_sym_try] = ACTIONS(2225),
    [sym_integer_literal] = ACTIONS(2223),
    [aux_sym_string_literal_token1] = ACTIONS(2223),
    [sym_char_literal] = ACTIONS(2223),
    [anon_sym_true] = ACTIONS(2225),
    [anon_sym_false] = ACTIONS(2225),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2225),
    [sym_super] = ACTIONS(2225),
    [sym_crate] = ACTIONS(2225),
    [sym_metavariable] = ACTIONS(2223),
    [sym__raw_string_literal_start] = ACTIONS(2223),
    [sym_float_literal] = ACTIONS(2223),
  },
  [619] = {
    [sym_line_comment] = STATE(619),
    [sym_block_comment] = STATE(619),
    [ts_builtin_sym_end] = ACTIONS(2227),
    [sym_identifier] = ACTIONS(2229),
    [anon_sym_SEMI] = ACTIONS(2227),
    [anon_sym_macro_rules_BANG] = ACTIONS(2227),
    [anon_sym_LPAREN] = ACTIONS(2227),
    [anon_sym_LBRACK] = ACTIONS(2227),
    [anon_sym_LBRACE] = ACTIONS(2227),
    [anon_sym_RBRACE] = ACTIONS(2227),
    [anon_sym_STAR] = ACTIONS(2227),
    [anon_sym_u8] = ACTIONS(2229),
    [anon_sym_i8] = ACTIONS(2229),
    [anon_sym_u16] = ACTIONS(2229),
    [anon_sym_i16] = ACTIONS(2229),
    [anon_sym_u32] = ACTIONS(2229),
    [anon_sym_i32] = ACTIONS(2229),
    [anon_sym_u64] = ACTIONS(2229),
    [anon_sym_i64] = ACTIONS(2229),
    [anon_sym_u128] = ACTIONS(2229),
    [anon_sym_i128] = ACTIONS(2229),
    [anon_sym_isize] = ACTIONS(2229),
    [anon_sym_usize] = ACTIONS(2229),
    [anon_sym_f32] = ACTIONS(2229),
    [anon_sym_f64] = ACTIONS(2229),
    [anon_sym_bool] = ACTIONS(2229),
    [anon_sym_str] = ACTIONS(2229),
    [anon_sym_char] = ACTIONS(2229),
    [anon_sym_DASH] = ACTIONS(2227),
    [anon_sym_BANG] = ACTIONS(2227),
    [anon_sym_AMP] = ACTIONS(2227),
    [anon_sym_PIPE] = ACTIONS(2227),
    [anon_sym_LT] = ACTIONS(2227),
    [anon_sym_DOT_DOT] = ACTIONS(2227),
    [anon_sym_COLON_COLON] = ACTIONS(2227),
    [anon_sym_POUND] = ACTIONS(2227),
    [anon_sym_SQUOTE] = ACTIONS(2229),
    [anon_sym_async] = ACTIONS(2229),
    [anon_sym_break] = ACTIONS(2229),
    [anon_sym_const] = ACTIONS(2229),
    [anon_sym_continue] = ACTIONS(2229),
    [anon_sym_default] = ACTIONS(2229),
    [anon_sym_enum] = ACTIONS(2229),
    [anon_sym_fn] = ACTIONS(2229),
    [anon_sym_for] = ACTIONS(2229),
    [anon_sym_gen] = ACTIONS(2229),
    [anon_sym_if] = ACTIONS(2229),
    [anon_sym_impl] = ACTIONS(2229),
    [anon_sym_let] = ACTIONS(2229),
    [anon_sym_loop] = ACTIONS(2229),
    [anon_sym_match] = ACTIONS(2229),
    [anon_sym_mod] = ACTIONS(2229),
    [anon_sym_pub] = ACTIONS(2229),
    [anon_sym_return] = ACTIONS(2229),
    [anon_sym_static] = ACTIONS(2229),
    [anon_sym_struct] = ACTIONS(2229),
    [anon_sym_trait] = ACTIONS(2229),
    [anon_sym_type] = ACTIONS(2229),
    [anon_sym_union] = ACTIONS(2229),
    [anon_sym_unsafe] = ACTIONS(2229),
    [anon_sym_use] = ACTIONS(2229),
    [anon_sym_while] = ACTIONS(2229),
    [anon_sym_extern] = ACTIONS(2229),
    [anon_sym_yield] = ACTIONS(2229),
    [anon_sym_move] = ACTIONS(2229),
    [anon_sym_try] = ACTIONS(2229),
    [sym_integer_literal] = ACTIONS(2227),
    [aux_sym_string_literal_token1] = ACTIONS(2227),
    [sym_char_literal] = ACTIONS(2227),
    [anon_sym_true] = ACTIONS(2229),
    [anon_sym_false] = ACTIONS(2229),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2229),
    [sym_super] = ACTIONS(2229),
    [sym_crate] = ACTIONS(2229),
    [sym_metavariable] = ACTIONS(2227),
    [sym__raw_string_literal_start] = ACTIONS(2227),
    [sym_float_literal] = ACTIONS(2227),
  },
  [620] = {
    [sym_line_comment] = STATE(620),
    [sym_block_comment] = STATE(620),
    [ts_builtin_sym_end] = ACTIONS(2231),
    [sym_identifier] = ACTIONS(2233),
    [anon_sym_SEMI] = ACTIONS(2231),
    [anon_sym_macro_rules_BANG] = ACTIONS(2231),
    [anon_sym_LPAREN] = ACTIONS(2231),
    [anon_sym_LBRACK] = ACTIONS(2231),
    [anon_sym_LBRACE] = ACTIONS(2231),
    [anon_sym_RBRACE] = ACTIONS(2231),
    [anon_sym_STAR] = ACTIONS(2231),
    [anon_sym_u8] = ACTIONS(2233),
    [anon_sym_i8] = ACTIONS(2233),
    [anon_sym_u16] = ACTIONS(2233),
    [anon_sym_i16] = ACTIONS(2233),
    [anon_sym_u32] = ACTIONS(2233),
    [anon_sym_i32] = ACTIONS(2233),
    [anon_sym_u64] = ACTIONS(2233),
    [anon_sym_i64] = ACTIONS(2233),
    [anon_sym_u128] = ACTIONS(2233),
    [anon_sym_i128] = ACTIONS(2233),
    [anon_sym_isize] = ACTIONS(2233),
    [anon_sym_usize] = ACTIONS(2233),
    [anon_sym_f32] = ACTIONS(2233),
    [anon_sym_f64] = ACTIONS(2233),
    [anon_sym_bool] = ACTIONS(2233),
    [anon_sym_str] = ACTIONS(2233),
    [anon_sym_char] = ACTIONS(2233),
    [anon_sym_DASH] = ACTIONS(2231),
    [anon_sym_BANG] = ACTIONS(2231),
    [anon_sym_AMP] = ACTIONS(2231),
    [anon_sym_PIPE] = ACTIONS(2231),
    [anon_sym_LT] = ACTIONS(2231),
    [anon_sym_DOT_DOT] = ACTIONS(2231),
    [anon_sym_COLON_COLON] = ACTIONS(2231),
    [anon_sym_POUND] = ACTIONS(2231),
    [anon_sym_SQUOTE] = ACTIONS(2233),
    [anon_sym_async] = ACTIONS(2233),
    [anon_sym_break] = ACTIONS(2233),
    [anon_sym_const] = ACTIONS(2233),
    [anon_sym_continue] = ACTIONS(2233),
    [anon_sym_default] = ACTIONS(2233),
    [anon_sym_enum] = ACTIONS(2233),
    [anon_sym_fn] = ACTIONS(2233),
    [anon_sym_for] = ACTIONS(2233),
    [anon_sym_gen] = ACTIONS(2233),
    [anon_sym_if] = ACTIONS(2233),
    [anon_sym_impl] = ACTIONS(2233),
    [anon_sym_let] = ACTIONS(2233),
    [anon_sym_loop] = ACTIONS(2233),
    [anon_sym_match] = ACTIONS(2233),
    [anon_sym_mod] = ACTIONS(2233),
    [anon_sym_pub] = ACTIONS(2233),
    [anon_sym_return] = ACTIONS(2233),
    [anon_sym_static] = ACTIONS(2233),
    [anon_sym_struct] = ACTIONS(2233),
    [anon_sym_trait] = ACTIONS(2233),
    [anon_sym_type] = ACTIONS(2233),
    [anon_sym_union] = ACTIONS(2233),
    [anon_sym_unsafe] = ACTIONS(2233),
    [anon_sym_use] = ACTIONS(2233),
    [anon_sym_while] = ACTIONS(2233),
    [anon_sym_extern] = ACTIONS(2233),
    [anon_sym_yield] = ACTIONS(2233),
    [anon_sym_move] = ACTIONS(2233),
    [anon_sym_try] = ACTIONS(2233),
    [sym_integer_literal] = ACTIONS(2231),
    [aux_sym_string_literal_token1] = ACTIONS(2231),
    [sym_char_literal] = ACTIONS(2231),
    [anon_sym_true] = ACTIONS(2233),
    [anon_sym_false] = ACTIONS(2233),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2233),
    [sym_super] = ACTIONS(2233),
    [sym_crate] = ACTIONS(2233),
    [sym_metavariable] = ACTIONS(2231),
    [sym__raw_string_literal_start] = ACTIONS(2231),
    [sym_float_literal] = ACTIONS(2231),
  },
  [621] = {
    [sym_line_comment] = STATE(621),
    [sym_block_comment] = STATE(621),
    [ts_builtin_sym_end] = ACTIONS(2235),
    [sym_identifier] = ACTIONS(2237),
    [anon_sym_SEMI] = ACTIONS(2235),
    [anon_sym_macro_rules_BANG] = ACTIONS(2235),
    [anon_sym_LPAREN] = ACTIONS(2235),
    [anon_sym_LBRACK] = ACTIONS(2235),
    [anon_sym_LBRACE] = ACTIONS(2235),
    [anon_sym_RBRACE] = ACTIONS(2235),
    [anon_sym_STAR] = ACTIONS(2235),
    [anon_sym_u8] = ACTIONS(2237),
    [anon_sym_i8] = ACTIONS(2237),
    [anon_sym_u16] = ACTIONS(2237),
    [anon_sym_i16] = ACTIONS(2237),
    [anon_sym_u32] = ACTIONS(2237),
    [anon_sym_i32] = ACTIONS(2237),
    [anon_sym_u64] = ACTIONS(2237),
    [anon_sym_i64] = ACTIONS(2237),
    [anon_sym_u128] = ACTIONS(2237),
    [anon_sym_i128] = ACTIONS(2237),
    [anon_sym_isize] = ACTIONS(2237),
    [anon_sym_usize] = ACTIONS(2237),
    [anon_sym_f32] = ACTIONS(2237),
    [anon_sym_f64] = ACTIONS(2237),
    [anon_sym_bool] = ACTIONS(2237),
    [anon_sym_str] = ACTIONS(2237),
    [anon_sym_char] = ACTIONS(2237),
    [anon_sym_DASH] = ACTIONS(2235),
    [anon_sym_BANG] = ACTIONS(2235),
    [anon_sym_AMP] = ACTIONS(2235),
    [anon_sym_PIPE] = ACTIONS(2235),
    [anon_sym_LT] = ACTIONS(2235),
    [anon_sym_DOT_DOT] = ACTIONS(2235),
    [anon_sym_COLON_COLON] = ACTIONS(2235),
    [anon_sym_POUND] = ACTIONS(2235),
    [anon_sym_SQUOTE] = ACTIONS(2237),
    [anon_sym_async] = ACTIONS(2237),
    [anon_sym_break] = ACTIONS(2237),
    [anon_sym_const] = ACTIONS(2237),
    [anon_sym_continue] = ACTIONS(2237),
    [anon_sym_default] = ACTIONS(2237),
    [anon_sym_enum] = ACTIONS(2237),
    [anon_sym_fn] = ACTIONS(2237),
    [anon_sym_for] = ACTIONS(2237),
    [anon_sym_gen] = ACTIONS(2237),
    [anon_sym_if] = ACTIONS(2237),
    [anon_sym_impl] = ACTIONS(2237),
    [anon_sym_let] = ACTIONS(2237),
    [anon_sym_loop] = ACTIONS(2237),
    [anon_sym_match] = ACTIONS(2237),
    [anon_sym_mod] = ACTIONS(2237),
    [anon_sym_pub] = ACTIONS(2237),
    [anon_sym_return] = ACTIONS(2237),
    [anon_sym_static] = ACTIONS(2237),
    [anon_sym_struct] = ACTIONS(2237),
    [anon_sym_trait] = ACTIONS(2237),
    [anon_sym_type] = ACTIONS(2237),
    [anon_sym_union] = ACTIONS(2237),
    [anon_sym_unsafe] = ACTIONS(2237),
    [anon_sym_use] = ACTIONS(2237),
    [anon_sym_while] = ACTIONS(2237),
    [anon_sym_extern] = ACTIONS(2237),
    [anon_sym_yield] = ACTIONS(2237),
    [anon_sym_move] = ACTIONS(2237),
    [anon_sym_try] = ACTIONS(2237),
    [sym_integer_literal] = ACTIONS(2235),
    [aux_sym_string_literal_token1] = ACTIONS(2235),
    [sym_char_literal] = ACTIONS(2235),
    [anon_sym_true] = ACTIONS(2237),
    [anon_sym_false] = ACTIONS(2237),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2237),
    [sym_super] = ACTIONS(2237),
    [sym_crate] = ACTIONS(2237),
    [sym_metavariable] = ACTIONS(2235),
    [sym__raw_string_literal_start] = ACTIONS(2235),
    [sym_float_literal] = ACTIONS(2235),
  },
  [622] = {
    [sym_line_comment] = STATE(622),
    [sym_block_comment] = STATE(622),
    [ts_builtin_sym_end] = ACTIONS(2239),
    [sym_identifier] = ACTIONS(2241),
    [anon_sym_SEMI] = ACTIONS(2239),
    [anon_sym_macro_rules_BANG] = ACTIONS(2239),
    [anon_sym_LPAREN] = ACTIONS(2239),
    [anon_sym_LBRACK] = ACTIONS(2239),
    [anon_sym_LBRACE] = ACTIONS(2239),
    [anon_sym_RBRACE] = ACTIONS(2239),
    [anon_sym_STAR] = ACTIONS(2239),
    [anon_sym_u8] = ACTIONS(2241),
    [anon_sym_i8] = ACTIONS(2241),
    [anon_sym_u16] = ACTIONS(2241),
    [anon_sym_i16] = ACTIONS(2241),
    [anon_sym_u32] = ACTIONS(2241),
    [anon_sym_i32] = ACTIONS(2241),
    [anon_sym_u64] = ACTIONS(2241),
    [anon_sym_i64] = ACTIONS(2241),
    [anon_sym_u128] = ACTIONS(2241),
    [anon_sym_i128] = ACTIONS(2241),
    [anon_sym_isize] = ACTIONS(2241),
    [anon_sym_usize] = ACTIONS(2241),
    [anon_sym_f32] = ACTIONS(2241),
    [anon_sym_f64] = ACTIONS(2241),
    [anon_sym_bool] = ACTIONS(2241),
    [anon_sym_str] = ACTIONS(2241),
    [anon_sym_char] = ACTIONS(2241),
    [anon_sym_DASH] = ACTIONS(2239),
    [anon_sym_BANG] = ACTIONS(2239),
    [anon_sym_AMP] = ACTIONS(2239),
    [anon_sym_PIPE] = ACTIONS(2239),
    [anon_sym_LT] = ACTIONS(2239),
    [anon_sym_DOT_DOT] = ACTIONS(2239),
    [anon_sym_COLON_COLON] = ACTIONS(2239),
    [anon_sym_POUND] = ACTIONS(2239),
    [anon_sym_SQUOTE] = ACTIONS(2241),
    [anon_sym_async] = ACTIONS(2241),
    [anon_sym_break] = ACTIONS(2241),
    [anon_sym_const] = ACTIONS(2241),
    [anon_sym_continue] = ACTIONS(2241),
    [anon_sym_default] = ACTIONS(2241),
    [anon_sym_enum] = ACTIONS(2241),
    [anon_sym_fn] = ACTIONS(2241),
    [anon_sym_for] = ACTIONS(2241),
    [anon_sym_gen] = ACTIONS(2241),
    [anon_sym_if] = ACTIONS(2241),
    [anon_sym_impl] = ACTIONS(2241),
    [anon_sym_let] = ACTIONS(2241),
    [anon_sym_loop] = ACTIONS(2241),
    [anon_sym_match] = ACTIONS(2241),
    [anon_sym_mod] = ACTIONS(2241),
    [anon_sym_pub] = ACTIONS(2241),
    [anon_sym_return] = ACTIONS(2241),
    [anon_sym_static] = ACTIONS(2241),
    [anon_sym_struct] = ACTIONS(2241),
    [anon_sym_trait] = ACTIONS(2241),
    [anon_sym_type] = ACTIONS(2241),
    [anon_sym_union] = ACTIONS(2241),
    [anon_sym_unsafe] = ACTIONS(2241),
    [anon_sym_use] = ACTIONS(2241),
    [anon_sym_while] = ACTIONS(2241),
    [anon_sym_extern] = ACTIONS(2241),
    [anon_sym_yield] = ACTIONS(2241),
    [anon_sym_move] = ACTIONS(2241),
    [anon_sym_try] = ACTIONS(2241),
    [sym_integer_literal] = ACTIONS(2239),
    [aux_sym_string_literal_token1] = ACTIONS(2239),
    [sym_char_literal] = ACTIONS(2239),
    [anon_sym_true] = ACTIONS(2241),
    [anon_sym_false] = ACTIONS(2241),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2241),
    [sym_super] = ACTIONS(2241),
    [sym_crate] = ACTIONS(2241),
    [sym_metavariable] = ACTIONS(2239),
    [sym__raw_string_literal_start] = ACTIONS(2239),
    [sym_float_literal] = ACTIONS(2239),
  },
  [623] = {
    [sym_line_comment] = STATE(623),
    [sym_block_comment] = STATE(623),
    [ts_builtin_sym_end] = ACTIONS(2243),
    [sym_identifier] = ACTIONS(2245),
    [anon_sym_SEMI] = ACTIONS(2243),
    [anon_sym_macro_rules_BANG] = ACTIONS(2243),
    [anon_sym_LPAREN] = ACTIONS(2243),
    [anon_sym_LBRACK] = ACTIONS(2243),
    [anon_sym_LBRACE] = ACTIONS(2243),
    [anon_sym_RBRACE] = ACTIONS(2243),
    [anon_sym_STAR] = ACTIONS(2243),
    [anon_sym_u8] = ACTIONS(2245),
    [anon_sym_i8] = ACTIONS(2245),
    [anon_sym_u16] = ACTIONS(2245),
    [anon_sym_i16] = ACTIONS(2245),
    [anon_sym_u32] = ACTIONS(2245),
    [anon_sym_i32] = ACTIONS(2245),
    [anon_sym_u64] = ACTIONS(2245),
    [anon_sym_i64] = ACTIONS(2245),
    [anon_sym_u128] = ACTIONS(2245),
    [anon_sym_i128] = ACTIONS(2245),
    [anon_sym_isize] = ACTIONS(2245),
    [anon_sym_usize] = ACTIONS(2245),
    [anon_sym_f32] = ACTIONS(2245),
    [anon_sym_f64] = ACTIONS(2245),
    [anon_sym_bool] = ACTIONS(2245),
    [anon_sym_str] = ACTIONS(2245),
    [anon_sym_char] = ACTIONS(2245),
    [anon_sym_DASH] = ACTIONS(2243),
    [anon_sym_BANG] = ACTIONS(2243),
    [anon_sym_AMP] = ACTIONS(2243),
    [anon_sym_PIPE] = ACTIONS(2243),
    [anon_sym_LT] = ACTIONS(2243),
    [anon_sym_DOT_DOT] = ACTIONS(2243),
    [anon_sym_COLON_COLON] = ACTIONS(2243),
    [anon_sym_POUND] = ACTIONS(2243),
    [anon_sym_SQUOTE] = ACTIONS(2245),
    [anon_sym_async] = ACTIONS(2245),
    [anon_sym_break] = ACTIONS(2245),
    [anon_sym_const] = ACTIONS(2245),
    [anon_sym_continue] = ACTIONS(2245),
    [anon_sym_default] = ACTIONS(2245),
    [anon_sym_enum] = ACTIONS(2245),
    [anon_sym_fn] = ACTIONS(2245),
    [anon_sym_for] = ACTIONS(2245),
    [anon_sym_gen] = ACTIONS(2245),
    [anon_sym_if] = ACTIONS(2245),
    [anon_sym_impl] = ACTIONS(2245),
    [anon_sym_let] = ACTIONS(2245),
    [anon_sym_loop] = ACTIONS(2245),
    [anon_sym_match] = ACTIONS(2245),
    [anon_sym_mod] = ACTIONS(2245),
    [anon_sym_pub] = ACTIONS(2245),
    [anon_sym_return] = ACTIONS(2245),
    [anon_sym_static] = ACTIONS(2245),
    [anon_sym_struct] = ACTIONS(2245),
    [anon_sym_trait] = ACTIONS(2245),
    [anon_sym_type] = ACTIONS(2245),
    [anon_sym_union] = ACTIONS(2245),
    [anon_sym_unsafe] = ACTIONS(2245),
    [anon_sym_use] = ACTIONS(2245),
    [anon_sym_while] = ACTIONS(2245),
    [anon_sym_extern] = ACTIONS(2245),
    [anon_sym_yield] = ACTIONS(2245),
    [anon_sym_move] = ACTIONS(2245),
    [anon_sym_try] = ACTIONS(2245),
    [sym_integer_literal] = ACTIONS(2243),
    [aux_sym_string_literal_token1] = ACTIONS(2243),
    [sym_char_literal] = ACTIONS(2243),
    [anon_sym_true] = ACTIONS(2245),
    [anon_sym_false] = ACTIONS(2245),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2245),
    [sym_super] = ACTIONS(2245),
    [sym_crate] = ACTIONS(2245),
    [sym_metavariable] = ACTIONS(2243),
    [sym__raw_string_literal_start] = ACTIONS(2243),
    [sym_float_literal] = ACTIONS(2243),
  },
  [624] = {
    [sym_line_comment] = STATE(624),
    [sym_block_comment] = STATE(624),
    [ts_builtin_sym_end] = ACTIONS(2247),
    [sym_identifier] = ACTIONS(2249),
    [anon_sym_SEMI] = ACTIONS(2247),
    [anon_sym_macro_rules_BANG] = ACTIONS(2247),
    [anon_sym_LPAREN] = ACTIONS(2247),
    [anon_sym_LBRACK] = ACTIONS(2247),
    [anon_sym_LBRACE] = ACTIONS(2247),
    [anon_sym_RBRACE] = ACTIONS(2247),
    [anon_sym_STAR] = ACTIONS(2247),
    [anon_sym_u8] = ACTIONS(2249),
    [anon_sym_i8] = ACTIONS(2249),
    [anon_sym_u16] = ACTIONS(2249),
    [anon_sym_i16] = ACTIONS(2249),
    [anon_sym_u32] = ACTIONS(2249),
    [anon_sym_i32] = ACTIONS(2249),
    [anon_sym_u64] = ACTIONS(2249),
    [anon_sym_i64] = ACTIONS(2249),
    [anon_sym_u128] = ACTIONS(2249),
    [anon_sym_i128] = ACTIONS(2249),
    [anon_sym_isize] = ACTIONS(2249),
    [anon_sym_usize] = ACTIONS(2249),
    [anon_sym_f32] = ACTIONS(2249),
    [anon_sym_f64] = ACTIONS(2249),
    [anon_sym_bool] = ACTIONS(2249),
    [anon_sym_str] = ACTIONS(2249),
    [anon_sym_char] = ACTIONS(2249),
    [anon_sym_DASH] = ACTIONS(2247),
    [anon_sym_BANG] = ACTIONS(2247),
    [anon_sym_AMP] = ACTIONS(2247),
    [anon_sym_PIPE] = ACTIONS(2247),
    [anon_sym_LT] = ACTIONS(2247),
    [anon_sym_DOT_DOT] = ACTIONS(2247),
    [anon_sym_COLON_COLON] = ACTIONS(2247),
    [anon_sym_POUND] = ACTIONS(2247),
    [anon_sym_SQUOTE] = ACTIONS(2249),
    [anon_sym_async] = ACTIONS(2249),
    [anon_sym_break] = ACTIONS(2249),
    [anon_sym_const] = ACTIONS(2249),
    [anon_sym_continue] = ACTIONS(2249),
    [anon_sym_default] = ACTIONS(2249),
    [anon_sym_enum] = ACTIONS(2249),
    [anon_sym_fn] = ACTIONS(2249),
    [anon_sym_for] = ACTIONS(2249),
    [anon_sym_gen] = ACTIONS(2249),
    [anon_sym_if] = ACTIONS(2249),
    [anon_sym_impl] = ACTIONS(2249),
    [anon_sym_let] = ACTIONS(2249),
    [anon_sym_loop] = ACTIONS(2249),
    [anon_sym_match] = ACTIONS(2249),
    [anon_sym_mod] = ACTIONS(2249),
    [anon_sym_pub] = ACTIONS(2249),
    [anon_sym_return] = ACTIONS(2249),
    [anon_sym_static] = ACTIONS(2249),
    [anon_sym_struct] = ACTIONS(2249),
    [anon_sym_trait] = ACTIONS(2249),
    [anon_sym_type] = ACTIONS(2249),
    [anon_sym_union] = ACTIONS(2249),
    [anon_sym_unsafe] = ACTIONS(2249),
    [anon_sym_use] = ACTIONS(2249),
    [anon_sym_while] = ACTIONS(2249),
    [anon_sym_extern] = ACTIONS(2249),
    [anon_sym_yield] = ACTIONS(2249),
    [anon_sym_move] = ACTIONS(2249),
    [anon_sym_try] = ACTIONS(2249),
    [sym_integer_literal] = ACTIONS(2247),
    [aux_sym_string_literal_token1] = ACTIONS(2247),
    [sym_char_literal] = ACTIONS(2247),
    [anon_sym_true] = ACTIONS(2249),
    [anon_sym_false] = ACTIONS(2249),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2249),
    [sym_super] = ACTIONS(2249),
    [sym_crate] = ACTIONS(2249),
    [sym_metavariable] = ACTIONS(2247),
    [sym__raw_string_literal_start] = ACTIONS(2247),
    [sym_float_literal] = ACTIONS(2247),
  },
  [625] = {
    [sym_line_comment] = STATE(625),
    [sym_block_comment] = STATE(625),
    [ts_builtin_sym_end] = ACTIONS(2251),
    [sym_identifier] = ACTIONS(2253),
    [anon_sym_SEMI] = ACTIONS(2251),
    [anon_sym_macro_rules_BANG] = ACTIONS(2251),
    [anon_sym_LPAREN] = ACTIONS(2251),
    [anon_sym_LBRACK] = ACTIONS(2251),
    [anon_sym_LBRACE] = ACTIONS(2251),
    [anon_sym_RBRACE] = ACTIONS(2251),
    [anon_sym_STAR] = ACTIONS(2251),
    [anon_sym_u8] = ACTIONS(2253),
    [anon_sym_i8] = ACTIONS(2253),
    [anon_sym_u16] = ACTIONS(2253),
    [anon_sym_i16] = ACTIONS(2253),
    [anon_sym_u32] = ACTIONS(2253),
    [anon_sym_i32] = ACTIONS(2253),
    [anon_sym_u64] = ACTIONS(2253),
    [anon_sym_i64] = ACTIONS(2253),
    [anon_sym_u128] = ACTIONS(2253),
    [anon_sym_i128] = ACTIONS(2253),
    [anon_sym_isize] = ACTIONS(2253),
    [anon_sym_usize] = ACTIONS(2253),
    [anon_sym_f32] = ACTIONS(2253),
    [anon_sym_f64] = ACTIONS(2253),
    [anon_sym_bool] = ACTIONS(2253),
    [anon_sym_str] = ACTIONS(2253),
    [anon_sym_char] = ACTIONS(2253),
    [anon_sym_DASH] = ACTIONS(2251),
    [anon_sym_BANG] = ACTIONS(2251),
    [anon_sym_AMP] = ACTIONS(2251),
    [anon_sym_PIPE] = ACTIONS(2251),
    [anon_sym_LT] = ACTIONS(2251),
    [anon_sym_DOT_DOT] = ACTIONS(2251),
    [anon_sym_COLON_COLON] = ACTIONS(2251),
    [anon_sym_POUND] = ACTIONS(2251),
    [anon_sym_SQUOTE] = ACTIONS(2253),
    [anon_sym_async] = ACTIONS(2253),
    [anon_sym_break] = ACTIONS(2253),
    [anon_sym_const] = ACTIONS(2253),
    [anon_sym_continue] = ACTIONS(2253),
    [anon_sym_default] = ACTIONS(2253),
    [anon_sym_enum] = ACTIONS(2253),
    [anon_sym_fn] = ACTIONS(2253),
    [anon_sym_for] = ACTIONS(2253),
    [anon_sym_gen] = ACTIONS(2253),
    [anon_sym_if] = ACTIONS(2253),
    [anon_sym_impl] = ACTIONS(2253),
    [anon_sym_let] = ACTIONS(2253),
    [anon_sym_loop] = ACTIONS(2253),
    [anon_sym_match] = ACTIONS(2253),
    [anon_sym_mod] = ACTIONS(2253),
    [anon_sym_pub] = ACTIONS(2253),
    [anon_sym_return] = ACTIONS(2253),
    [anon_sym_static] = ACTIONS(2253),
    [anon_sym_struct] = ACTIONS(2253),
    [anon_sym_trait] = ACTIONS(2253),
    [anon_sym_type] = ACTIONS(2253),
    [anon_sym_union] = ACTIONS(2253),
    [anon_sym_unsafe] = ACTIONS(2253),
    [anon_sym_use] = ACTIONS(2253),
    [anon_sym_while] = ACTIONS(2253),
    [anon_sym_extern] = ACTIONS(2253),
    [anon_sym_yield] = ACTIONS(2253),
    [anon_sym_move] = ACTIONS(2253),
    [anon_sym_try] = ACTIONS(2253),
    [sym_integer_literal] = ACTIONS(2251),
    [aux_sym_string_literal_token1] = ACTIONS(2251),
    [sym_char_literal] = ACTIONS(2251),
    [anon_sym_true] = ACTIONS(2253),
    [anon_sym_false] = ACTIONS(2253),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2253),
    [sym_super] = ACTIONS(2253),
    [sym_crate] = ACTIONS(2253),
    [sym_metavariable] = ACTIONS(2251),
    [sym__raw_string_literal_start] = ACTIONS(2251),
    [sym_float_literal] = ACTIONS(2251),
  },
  [626] = {
    [sym_line_comment] = STATE(626),
    [sym_block_comment] = STATE(626),
    [ts_builtin_sym_end] = ACTIONS(2255),
    [sym_identifier] = ACTIONS(2257),
    [anon_sym_SEMI] = ACTIONS(2255),
    [anon_sym_macro_rules_BANG] = ACTIONS(2255),
    [anon_sym_LPAREN] = ACTIONS(2255),
    [anon_sym_LBRACK] = ACTIONS(2255),
    [anon_sym_LBRACE] = ACTIONS(2255),
    [anon_sym_RBRACE] = ACTIONS(2255),
    [anon_sym_STAR] = ACTIONS(2255),
    [anon_sym_u8] = ACTIONS(2257),
    [anon_sym_i8] = ACTIONS(2257),
    [anon_sym_u16] = ACTIONS(2257),
    [anon_sym_i16] = ACTIONS(2257),
    [anon_sym_u32] = ACTIONS(2257),
    [anon_sym_i32] = ACTIONS(2257),
    [anon_sym_u64] = ACTIONS(2257),
    [anon_sym_i64] = ACTIONS(2257),
    [anon_sym_u128] = ACTIONS(2257),
    [anon_sym_i128] = ACTIONS(2257),
    [anon_sym_isize] = ACTIONS(2257),
    [anon_sym_usize] = ACTIONS(2257),
    [anon_sym_f32] = ACTIONS(2257),
    [anon_sym_f64] = ACTIONS(2257),
    [anon_sym_bool] = ACTIONS(2257),
    [anon_sym_str] = ACTIONS(2257),
    [anon_sym_char] = ACTIONS(2257),
    [anon_sym_DASH] = ACTIONS(2255),
    [anon_sym_BANG] = ACTIONS(2255),
    [anon_sym_AMP] = ACTIONS(2255),
    [anon_sym_PIPE] = ACTIONS(2255),
    [anon_sym_LT] = ACTIONS(2255),
    [anon_sym_DOT_DOT] = ACTIONS(2255),
    [anon_sym_COLON_COLON] = ACTIONS(2255),
    [anon_sym_POUND] = ACTIONS(2255),
    [anon_sym_SQUOTE] = ACTIONS(2257),
    [anon_sym_async] = ACTIONS(2257),
    [anon_sym_break] = ACTIONS(2257),
    [anon_sym_const] = ACTIONS(2257),
    [anon_sym_continue] = ACTIONS(2257),
    [anon_sym_default] = ACTIONS(2257),
    [anon_sym_enum] = ACTIONS(2257),
    [anon_sym_fn] = ACTIONS(2257),
    [anon_sym_for] = ACTIONS(2257),
    [anon_sym_gen] = ACTIONS(2257),
    [anon_sym_if] = ACTIONS(2257),
    [anon_sym_impl] = ACTIONS(2257),
    [anon_sym_let] = ACTIONS(2257),
    [anon_sym_loop] = ACTIONS(2257),
    [anon_sym_match] = ACTIONS(2257),
    [anon_sym_mod] = ACTIONS(2257),
    [anon_sym_pub] = ACTIONS(2257),
    [anon_sym_return] = ACTIONS(2257),
    [anon_sym_static] = ACTIONS(2257),
    [anon_sym_struct] = ACTIONS(2257),
    [anon_sym_trait] = ACTIONS(2257),
    [anon_sym_type] = ACTIONS(2257),
    [anon_sym_union] = ACTIONS(2257),
    [anon_sym_unsafe] = ACTIONS(2257),
    [anon_sym_use] = ACTIONS(2257),
    [anon_sym_while] = ACTIONS(2257),
    [anon_sym_extern] = ACTIONS(2257),
    [anon_sym_yield] = ACTIONS(2257),
    [anon_sym_move] = ACTIONS(2257),
    [anon_sym_try] = ACTIONS(2257),
    [sym_integer_literal] = ACTIONS(2255),
    [aux_sym_string_literal_token1] = ACTIONS(2255),
    [sym_char_literal] = ACTIONS(2255),
    [anon_sym_true] = ACTIONS(2257),
    [anon_sym_false] = ACTIONS(2257),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2257),
    [sym_super] = ACTIONS(2257),
    [sym_crate] = ACTIONS(2257),
    [sym_metavariable] = ACTIONS(2255),
    [sym__raw_string_literal_start] = ACTIONS(2255),
    [sym_float_literal] = ACTIONS(2255),
  },
  [627] = {
    [sym_line_comment] = STATE(627),
    [sym_block_comment] = STATE(627),
    [ts_builtin_sym_end] = ACTIONS(2259),
    [sym_identifier] = ACTIONS(2261),
    [anon_sym_SEMI] = ACTIONS(2259),
    [anon_sym_macro_rules_BANG] = ACTIONS(2259),
    [anon_sym_LPAREN] = ACTIONS(2259),
    [anon_sym_LBRACK] = ACTIONS(2259),
    [anon_sym_LBRACE] = ACTIONS(2259),
    [anon_sym_RBRACE] = ACTIONS(2259),
    [anon_sym_STAR] = ACTIONS(2259),
    [anon_sym_u8] = ACTIONS(2261),
    [anon_sym_i8] = ACTIONS(2261),
    [anon_sym_u16] = ACTIONS(2261),
    [anon_sym_i16] = ACTIONS(2261),
    [anon_sym_u32] = ACTIONS(2261),
    [anon_sym_i32] = ACTIONS(2261),
    [anon_sym_u64] = ACTIONS(2261),
    [anon_sym_i64] = ACTIONS(2261),
    [anon_sym_u128] = ACTIONS(2261),
    [anon_sym_i128] = ACTIONS(2261),
    [anon_sym_isize] = ACTIONS(2261),
    [anon_sym_usize] = ACTIONS(2261),
    [anon_sym_f32] = ACTIONS(2261),
    [anon_sym_f64] = ACTIONS(2261),
    [anon_sym_bool] = ACTIONS(2261),
    [anon_sym_str] = ACTIONS(2261),
    [anon_sym_char] = ACTIONS(2261),
    [anon_sym_DASH] = ACTIONS(2259),
    [anon_sym_BANG] = ACTIONS(2259),
    [anon_sym_AMP] = ACTIONS(2259),
    [anon_sym_PIPE] = ACTIONS(2259),
    [anon_sym_LT] = ACTIONS(2259),
    [anon_sym_DOT_DOT] = ACTIONS(2259),
    [anon_sym_COLON_COLON] = ACTIONS(2259),
    [anon_sym_POUND] = ACTIONS(2259),
    [anon_sym_SQUOTE] = ACTIONS(2261),
    [anon_sym_async] = ACTIONS(2261),
    [anon_sym_break] = ACTIONS(2261),
    [anon_sym_const] = ACTIONS(2261),
    [anon_sym_continue] = ACTIONS(2261),
    [anon_sym_default] = ACTIONS(2261),
    [anon_sym_enum] = ACTIONS(2261),
    [anon_sym_fn] = ACTIONS(2261),
    [anon_sym_for] = ACTIONS(2261),
    [anon_sym_gen] = ACTIONS(2261),
    [anon_sym_if] = ACTIONS(2261),
    [anon_sym_impl] = ACTIONS(2261),
    [anon_sym_let] = ACTIONS(2261),
    [anon_sym_loop] = ACTIONS(2261),
    [anon_sym_match] = ACTIONS(2261),
    [anon_sym_mod] = ACTIONS(2261),
    [anon_sym_pub] = ACTIONS(2261),
    [anon_sym_return] = ACTIONS(2261),
    [anon_sym_static] = ACTIONS(2261),
    [anon_sym_struct] = ACTIONS(2261),
    [anon_sym_trait] = ACTIONS(2261),
    [anon_sym_type] = ACTIONS(2261),
    [anon_sym_union] = ACTIONS(2261),
    [anon_sym_unsafe] = ACTIONS(2261),
    [anon_sym_use] = ACTIONS(2261),
    [anon_sym_while] = ACTIONS(2261),
    [anon_sym_extern] = ACTIONS(2261),
    [anon_sym_yield] = ACTIONS(2261),
    [anon_sym_move] = ACTIONS(2261),
    [anon_sym_try] = ACTIONS(2261),
    [sym_integer_literal] = ACTIONS(2259),
    [aux_sym_string_literal_token1] = ACTIONS(2259),
    [sym_char_literal] = ACTIONS(2259),
    [anon_sym_true] = ACTIONS(2261),
    [anon_sym_false] = ACTIONS(2261),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2261),
    [sym_super] = ACTIONS(2261),
    [sym_crate] = ACTIONS(2261),
    [sym_metavariable] = ACTIONS(2259),
    [sym__raw_string_literal_start] = ACTIONS(2259),
    [sym_float_literal] = ACTIONS(2259),
  },
  [628] = {
    [sym_line_comment] = STATE(628),
    [sym_block_comment] = STATE(628),
    [ts_builtin_sym_end] = ACTIONS(2263),
    [sym_identifier] = ACTIONS(2265),
    [anon_sym_SEMI] = ACTIONS(2263),
    [anon_sym_macro_rules_BANG] = ACTIONS(2263),
    [anon_sym_LPAREN] = ACTIONS(2263),
    [anon_sym_LBRACK] = ACTIONS(2263),
    [anon_sym_LBRACE] = ACTIONS(2263),
    [anon_sym_RBRACE] = ACTIONS(2263),
    [anon_sym_STAR] = ACTIONS(2263),
    [anon_sym_u8] = ACTIONS(2265),
    [anon_sym_i8] = ACTIONS(2265),
    [anon_sym_u16] = ACTIONS(2265),
    [anon_sym_i16] = ACTIONS(2265),
    [anon_sym_u32] = ACTIONS(2265),
    [anon_sym_i32] = ACTIONS(2265),
    [anon_sym_u64] = ACTIONS(2265),
    [anon_sym_i64] = ACTIONS(2265),
    [anon_sym_u128] = ACTIONS(2265),
    [anon_sym_i128] = ACTIONS(2265),
    [anon_sym_isize] = ACTIONS(2265),
    [anon_sym_usize] = ACTIONS(2265),
    [anon_sym_f32] = ACTIONS(2265),
    [anon_sym_f64] = ACTIONS(2265),
    [anon_sym_bool] = ACTIONS(2265),
    [anon_sym_str] = ACTIONS(2265),
    [anon_sym_char] = ACTIONS(2265),
    [anon_sym_DASH] = ACTIONS(2263),
    [anon_sym_BANG] = ACTIONS(2263),
    [anon_sym_AMP] = ACTIONS(2263),
    [anon_sym_PIPE] = ACTIONS(2263),
    [anon_sym_LT] = ACTIONS(2263),
    [anon_sym_DOT_DOT] = ACTIONS(2263),
    [anon_sym_COLON_COLON] = ACTIONS(2263),
    [anon_sym_POUND] = ACTIONS(2263),
    [anon_sym_SQUOTE] = ACTIONS(2265),
    [anon_sym_async] = ACTIONS(2265),
    [anon_sym_break] = ACTIONS(2265),
    [anon_sym_const] = ACTIONS(2265),
    [anon_sym_continue] = ACTIONS(2265),
    [anon_sym_default] = ACTIONS(2265),
    [anon_sym_enum] = ACTIONS(2265),
    [anon_sym_fn] = ACTIONS(2265),
    [anon_sym_for] = ACTIONS(2265),
    [anon_sym_gen] = ACTIONS(2265),
    [anon_sym_if] = ACTIONS(2265),
    [anon_sym_impl] = ACTIONS(2265),
    [anon_sym_let] = ACTIONS(2265),
    [anon_sym_loop] = ACTIONS(2265),
    [anon_sym_match] = ACTIONS(2265),
    [anon_sym_mod] = ACTIONS(2265),
    [anon_sym_pub] = ACTIONS(2265),
    [anon_sym_return] = ACTIONS(2265),
    [anon_sym_static] = ACTIONS(2265),
    [anon_sym_struct] = ACTIONS(2265),
    [anon_sym_trait] = ACTIONS(2265),
    [anon_sym_type] = ACTIONS(2265),
    [anon_sym_union] = ACTIONS(2265),
    [anon_sym_unsafe] = ACTIONS(2265),
    [anon_sym_use] = ACTIONS(2265),
    [anon_sym_while] = ACTIONS(2265),
    [anon_sym_extern] = ACTIONS(2265),
    [anon_sym_yield] = ACTIONS(2265),
    [anon_sym_move] = ACTIONS(2265),
    [anon_sym_try] = ACTIONS(2265),
    [sym_integer_literal] = ACTIONS(2263),
    [aux_sym_string_literal_token1] = ACTIONS(2263),
    [sym_char_literal] = ACTIONS(2263),
    [anon_sym_true] = ACTIONS(2265),
    [anon_sym_false] = ACTIONS(2265),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2265),
    [sym_super] = ACTIONS(2265),
    [sym_crate] = ACTIONS(2265),
    [sym_metavariable] = ACTIONS(2263),
    [sym__raw_string_literal_start] = ACTIONS(2263),
    [sym_float_literal] = ACTIONS(2263),
  },
  [629] = {
    [sym_line_comment] = STATE(629),
    [sym_block_comment] = STATE(629),
    [ts_builtin_sym_end] = ACTIONS(2267),
    [sym_identifier] = ACTIONS(2269),
    [anon_sym_SEMI] = ACTIONS(2267),
    [anon_sym_macro_rules_BANG] = ACTIONS(2267),
    [anon_sym_LPAREN] = ACTIONS(2267),
    [anon_sym_LBRACK] = ACTIONS(2267),
    [anon_sym_LBRACE] = ACTIONS(2267),
    [anon_sym_RBRACE] = ACTIONS(2267),
    [anon_sym_STAR] = ACTIONS(2267),
    [anon_sym_u8] = ACTIONS(2269),
    [anon_sym_i8] = ACTIONS(2269),
    [anon_sym_u16] = ACTIONS(2269),
    [anon_sym_i16] = ACTIONS(2269),
    [anon_sym_u32] = ACTIONS(2269),
    [anon_sym_i32] = ACTIONS(2269),
    [anon_sym_u64] = ACTIONS(2269),
    [anon_sym_i64] = ACTIONS(2269),
    [anon_sym_u128] = ACTIONS(2269),
    [anon_sym_i128] = ACTIONS(2269),
    [anon_sym_isize] = ACTIONS(2269),
    [anon_sym_usize] = ACTIONS(2269),
    [anon_sym_f32] = ACTIONS(2269),
    [anon_sym_f64] = ACTIONS(2269),
    [anon_sym_bool] = ACTIONS(2269),
    [anon_sym_str] = ACTIONS(2269),
    [anon_sym_char] = ACTIONS(2269),
    [anon_sym_DASH] = ACTIONS(2267),
    [anon_sym_BANG] = ACTIONS(2267),
    [anon_sym_AMP] = ACTIONS(2267),
    [anon_sym_PIPE] = ACTIONS(2267),
    [anon_sym_LT] = ACTIONS(2267),
    [anon_sym_DOT_DOT] = ACTIONS(2267),
    [anon_sym_COLON_COLON] = ACTIONS(2267),
    [anon_sym_POUND] = ACTIONS(2267),
    [anon_sym_SQUOTE] = ACTIONS(2269),
    [anon_sym_async] = ACTIONS(2269),
    [anon_sym_break] = ACTIONS(2269),
    [anon_sym_const] = ACTIONS(2269),
    [anon_sym_continue] = ACTIONS(2269),
    [anon_sym_default] = ACTIONS(2269),
    [anon_sym_enum] = ACTIONS(2269),
    [anon_sym_fn] = ACTIONS(2269),
    [anon_sym_for] = ACTIONS(2269),
    [anon_sym_gen] = ACTIONS(2269),
    [anon_sym_if] = ACTIONS(2269),
    [anon_sym_impl] = ACTIONS(2269),
    [anon_sym_let] = ACTIONS(2269),
    [anon_sym_loop] = ACTIONS(2269),
    [anon_sym_match] = ACTIONS(2269),
    [anon_sym_mod] = ACTIONS(2269),
    [anon_sym_pub] = ACTIONS(2269),
    [anon_sym_return] = ACTIONS(2269),
    [anon_sym_static] = ACTIONS(2269),
    [anon_sym_struct] = ACTIONS(2269),
    [anon_sym_trait] = ACTIONS(2269),
    [anon_sym_type] = ACTIONS(2269),
    [anon_sym_union] = ACTIONS(2269),
    [anon_sym_unsafe] = ACTIONS(2269),
    [anon_sym_use] = ACTIONS(2269),
    [anon_sym_while] = ACTIONS(2269),
    [anon_sym_extern] = ACTIONS(2269),
    [anon_sym_yield] = ACTIONS(2269),
    [anon_sym_move] = ACTIONS(2269),
    [anon_sym_try] = ACTIONS(2269),
    [sym_integer_literal] = ACTIONS(2267),
    [aux_sym_string_literal_token1] = ACTIONS(2267),
    [sym_char_literal] = ACTIONS(2267),
    [anon_sym_true] = ACTIONS(2269),
    [anon_sym_false] = ACTIONS(2269),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2269),
    [sym_super] = ACTIONS(2269),
    [sym_crate] = ACTIONS(2269),
    [sym_metavariable] = ACTIONS(2267),
    [sym__raw_string_literal_start] = ACTIONS(2267),
    [sym_float_literal] = ACTIONS(2267),
  },
  [630] = {
    [sym_line_comment] = STATE(630),
    [sym_block_comment] = STATE(630),
    [ts_builtin_sym_end] = ACTIONS(1273),
    [sym_identifier] = ACTIONS(1275),
    [anon_sym_SEMI] = ACTIONS(1273),
    [anon_sym_macro_rules_BANG] = ACTIONS(1273),
    [anon_sym_LPAREN] = ACTIONS(1273),
    [anon_sym_LBRACK] = ACTIONS(1273),
    [anon_sym_LBRACE] = ACTIONS(1273),
    [anon_sym_RBRACE] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(1273),
    [anon_sym_u8] = ACTIONS(1275),
    [anon_sym_i8] = ACTIONS(1275),
    [anon_sym_u16] = ACTIONS(1275),
    [anon_sym_i16] = ACTIONS(1275),
    [anon_sym_u32] = ACTIONS(1275),
    [anon_sym_i32] = ACTIONS(1275),
    [anon_sym_u64] = ACTIONS(1275),
    [anon_sym_i64] = ACTIONS(1275),
    [anon_sym_u128] = ACTIONS(1275),
    [anon_sym_i128] = ACTIONS(1275),
    [anon_sym_isize] = ACTIONS(1275),
    [anon_sym_usize] = ACTIONS(1275),
    [anon_sym_f32] = ACTIONS(1275),
    [anon_sym_f64] = ACTIONS(1275),
    [anon_sym_bool] = ACTIONS(1275),
    [anon_sym_str] = ACTIONS(1275),
    [anon_sym_char] = ACTIONS(1275),
    [anon_sym_DASH] = ACTIONS(1273),
    [anon_sym_BANG] = ACTIONS(1273),
    [anon_sym_AMP] = ACTIONS(1273),
    [anon_sym_PIPE] = ACTIONS(1273),
    [anon_sym_LT] = ACTIONS(1273),
    [anon_sym_DOT_DOT] = ACTIONS(1273),
    [anon_sym_COLON_COLON] = ACTIONS(1273),
    [anon_sym_POUND] = ACTIONS(1273),
    [anon_sym_SQUOTE] = ACTIONS(1275),
    [anon_sym_async] = ACTIONS(1275),
    [anon_sym_break] = ACTIONS(1275),
    [anon_sym_const] = ACTIONS(1275),
    [anon_sym_continue] = ACTIONS(1275),
    [anon_sym_default] = ACTIONS(1275),
    [anon_sym_enum] = ACTIONS(1275),
    [anon_sym_fn] = ACTIONS(1275),
    [anon_sym_for] = ACTIONS(1275),
    [anon_sym_gen] = ACTIONS(1275),
    [anon_sym_if] = ACTIONS(1275),
    [anon_sym_impl] = ACTIONS(1275),
    [anon_sym_let] = ACTIONS(1275),
    [anon_sym_loop] = ACTIONS(1275),
    [anon_sym_match] = ACTIONS(1275),
    [anon_sym_mod] = ACTIONS(1275),
    [anon_sym_pub] = ACTIONS(1275),
    [anon_sym_return] = ACTIONS(1275),
    [anon_sym_static] = ACTIONS(1275),
    [anon_sym_struct] = ACTIONS(1275),
    [anon_sym_trait] = ACTIONS(1275),
    [anon_sym_type] = ACTIONS(1275),
    [anon_sym_union] = ACTIONS(1275),
    [anon_sym_unsafe] = ACTIONS(1275),
    [anon_sym_use] = ACTIONS(1275),
    [anon_sym_while] = ACTIONS(1275),
    [anon_sym_extern] = ACTIONS(1275),
    [anon_sym_yield] = ACTIONS(1275),
    [anon_sym_move] = ACTIONS(1275),
    [anon_sym_try] = ACTIONS(1275),
    [sym_integer_literal] = ACTIONS(1273),
    [aux_sym_string_literal_token1] = ACTIONS(1273),
    [sym_char_literal] = ACTIONS(1273),
    [anon_sym_true] = ACTIONS(1275),
    [anon_sym_false] = ACTIONS(1275),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1275),
    [sym_super] = ACTIONS(1275),
    [sym_crate] = ACTIONS(1275),
    [sym_metavariable] = ACTIONS(1273),
    [sym__raw_string_literal_start] = ACTIONS(1273),
    [sym_float_literal] = ACTIONS(1273),
  },
  [631] = {
    [sym_line_comment] = STATE(631),
    [sym_block_comment] = STATE(631),
    [ts_builtin_sym_end] = ACTIONS(2271),
    [sym_identifier] = ACTIONS(2273),
    [anon_sym_SEMI] = ACTIONS(2271),
    [anon_sym_macro_rules_BANG] = ACTIONS(2271),
    [anon_sym_LPAREN] = ACTIONS(2271),
    [anon_sym_LBRACK] = ACTIONS(2271),
    [anon_sym_LBRACE] = ACTIONS(2271),
    [anon_sym_RBRACE] = ACTIONS(2271),
    [anon_sym_STAR] = ACTIONS(2271),
    [anon_sym_u8] = ACTIONS(2273),
    [anon_sym_i8] = ACTIONS(2273),
    [anon_sym_u16] = ACTIONS(2273),
    [anon_sym_i16] = ACTIONS(2273),
    [anon_sym_u32] = ACTIONS(2273),
    [anon_sym_i32] = ACTIONS(2273),
    [anon_sym_u64] = ACTIONS(2273),
    [anon_sym_i64] = ACTIONS(2273),
    [anon_sym_u128] = ACTIONS(2273),
    [anon_sym_i128] = ACTIONS(2273),
    [anon_sym_isize] = ACTIONS(2273),
    [anon_sym_usize] = ACTIONS(2273),
    [anon_sym_f32] = ACTIONS(2273),
    [anon_sym_f64] = ACTIONS(2273),
    [anon_sym_bool] = ACTIONS(2273),
    [anon_sym_str] = ACTIONS(2273),
    [anon_sym_char] = ACTIONS(2273),
    [anon_sym_DASH] = ACTIONS(2271),
    [anon_sym_BANG] = ACTIONS(2271),
    [anon_sym_AMP] = ACTIONS(2271),
    [anon_sym_PIPE] = ACTIONS(2271),
    [anon_sym_LT] = ACTIONS(2271),
    [anon_sym_DOT_DOT] = ACTIONS(2271),
    [anon_sym_COLON_COLON] = ACTIONS(2271),
    [anon_sym_POUND] = ACTIONS(2271),
    [anon_sym_SQUOTE] = ACTIONS(2273),
    [anon_sym_async] = ACTIONS(2273),
    [anon_sym_break] = ACTIONS(2273),
    [anon_sym_const] = ACTIONS(2273),
    [anon_sym_continue] = ACTIONS(2273),
    [anon_sym_default] = ACTIONS(2273),
    [anon_sym_enum] = ACTIONS(2273),
    [anon_sym_fn] = ACTIONS(2273),
    [anon_sym_for] = ACTIONS(2273),
    [anon_sym_gen] = ACTIONS(2273),
    [anon_sym_if] = ACTIONS(2273),
    [anon_sym_impl] = ACTIONS(2273),
    [anon_sym_let] = ACTIONS(2273),
    [anon_sym_loop] = ACTIONS(2273),
    [anon_sym_match] = ACTIONS(2273),
    [anon_sym_mod] = ACTIONS(2273),
    [anon_sym_pub] = ACTIONS(2273),
    [anon_sym_return] = ACTIONS(2273),
    [anon_sym_static] = ACTIONS(2273),
    [anon_sym_struct] = ACTIONS(2273),
    [anon_sym_trait] = ACTIONS(2273),
    [anon_sym_type] = ACTIONS(2273),
    [anon_sym_union] = ACTIONS(2273),
    [anon_sym_unsafe] = ACTIONS(2273),
    [anon_sym_use] = ACTIONS(2273),
    [anon_sym_while] = ACTIONS(2273),
    [anon_sym_extern] = ACTIONS(2273),
    [anon_sym_yield] = ACTIONS(2273),
    [anon_sym_move] = ACTIONS(2273),
    [anon_sym_try] = ACTIONS(2273),
    [sym_integer_literal] = ACTIONS(2271),
    [aux_sym_string_literal_token1] = ACTIONS(2271),
    [sym_char_literal] = ACTIONS(2271),
    [anon_sym_true] = ACTIONS(2273),
    [anon_sym_false] = ACTIONS(2273),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2273),
    [sym_super] = ACTIONS(2273),
    [sym_crate] = ACTIONS(2273),
    [sym_metavariable] = ACTIONS(2271),
    [sym__raw_string_literal_start] = ACTIONS(2271),
    [sym_float_literal] = ACTIONS(2271),
  },
  [632] = {
    [sym_line_comment] = STATE(632),
    [sym_block_comment] = STATE(632),
    [ts_builtin_sym_end] = ACTIONS(2275),
    [sym_identifier] = ACTIONS(2277),
    [anon_sym_SEMI] = ACTIONS(2275),
    [anon_sym_macro_rules_BANG] = ACTIONS(2275),
    [anon_sym_LPAREN] = ACTIONS(2275),
    [anon_sym_LBRACK] = ACTIONS(2275),
    [anon_sym_LBRACE] = ACTIONS(2275),
    [anon_sym_RBRACE] = ACTIONS(2275),
    [anon_sym_STAR] = ACTIONS(2275),
    [anon_sym_u8] = ACTIONS(2277),
    [anon_sym_i8] = ACTIONS(2277),
    [anon_sym_u16] = ACTIONS(2277),
    [anon_sym_i16] = ACTIONS(2277),
    [anon_sym_u32] = ACTIONS(2277),
    [anon_sym_i32] = ACTIONS(2277),
    [anon_sym_u64] = ACTIONS(2277),
    [anon_sym_i64] = ACTIONS(2277),
    [anon_sym_u128] = ACTIONS(2277),
    [anon_sym_i128] = ACTIONS(2277),
    [anon_sym_isize] = ACTIONS(2277),
    [anon_sym_usize] = ACTIONS(2277),
    [anon_sym_f32] = ACTIONS(2277),
    [anon_sym_f64] = ACTIONS(2277),
    [anon_sym_bool] = ACTIONS(2277),
    [anon_sym_str] = ACTIONS(2277),
    [anon_sym_char] = ACTIONS(2277),
    [anon_sym_DASH] = ACTIONS(2275),
    [anon_sym_BANG] = ACTIONS(2275),
    [anon_sym_AMP] = ACTIONS(2275),
    [anon_sym_PIPE] = ACTIONS(2275),
    [anon_sym_LT] = ACTIONS(2275),
    [anon_sym_DOT_DOT] = ACTIONS(2275),
    [anon_sym_COLON_COLON] = ACTIONS(2275),
    [anon_sym_POUND] = ACTIONS(2275),
    [anon_sym_SQUOTE] = ACTIONS(2277),
    [anon_sym_async] = ACTIONS(2277),
    [anon_sym_break] = ACTIONS(2277),
    [anon_sym_const] = ACTIONS(2277),
    [anon_sym_continue] = ACTIONS(2277),
    [anon_sym_default] = ACTIONS(2277),
    [anon_sym_enum] = ACTIONS(2277),
    [anon_sym_fn] = ACTIONS(2277),
    [anon_sym_for] = ACTIONS(2277),
    [anon_sym_gen] = ACTIONS(2277),
    [anon_sym_if] = ACTIONS(2277),
    [anon_sym_impl] = ACTIONS(2277),
    [anon_sym_let] = ACTIONS(2277),
    [anon_sym_loop] = ACTIONS(2277),
    [anon_sym_match] = ACTIONS(2277),
    [anon_sym_mod] = ACTIONS(2277),
    [anon_sym_pub] = ACTIONS(2277),
    [anon_sym_return] = ACTIONS(2277),
    [anon_sym_static] = ACTIONS(2277),
    [anon_sym_struct] = ACTIONS(2277),
    [anon_sym_trait] = ACTIONS(2277),
    [anon_sym_type] = ACTIONS(2277),
    [anon_sym_union] = ACTIONS(2277),
    [anon_sym_unsafe] = ACTIONS(2277),
    [anon_sym_use] = ACTIONS(2277),
    [anon_sym_while] = ACTIONS(2277),
    [anon_sym_extern] = ACTIONS(2277),
    [anon_sym_yield] = ACTIONS(2277),
    [anon_sym_move] = ACTIONS(2277),
    [anon_sym_try] = ACTIONS(2277),
    [sym_integer_literal] = ACTIONS(2275),
    [aux_sym_string_literal_token1] = ACTIONS(2275),
    [sym_char_literal] = ACTIONS(2275),
    [anon_sym_true] = ACTIONS(2277),
    [anon_sym_false] = ACTIONS(2277),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2277),
    [sym_super] = ACTIONS(2277),
    [sym_crate] = ACTIONS(2277),
    [sym_metavariable] = ACTIONS(2275),
    [sym__raw_string_literal_start] = ACTIONS(2275),
    [sym_float_literal] = ACTIONS(2275),
  },
  [633] = {
    [sym_line_comment] = STATE(633),
    [sym_block_comment] = STATE(633),
    [ts_builtin_sym_end] = ACTIONS(2279),
    [sym_identifier] = ACTIONS(2281),
    [anon_sym_SEMI] = ACTIONS(2279),
    [anon_sym_macro_rules_BANG] = ACTIONS(2279),
    [anon_sym_LPAREN] = ACTIONS(2279),
    [anon_sym_LBRACK] = ACTIONS(2279),
    [anon_sym_LBRACE] = ACTIONS(2279),
    [anon_sym_RBRACE] = ACTIONS(2279),
    [anon_sym_STAR] = ACTIONS(2279),
    [anon_sym_u8] = ACTIONS(2281),
    [anon_sym_i8] = ACTIONS(2281),
    [anon_sym_u16] = ACTIONS(2281),
    [anon_sym_i16] = ACTIONS(2281),
    [anon_sym_u32] = ACTIONS(2281),
    [anon_sym_i32] = ACTIONS(2281),
    [anon_sym_u64] = ACTIONS(2281),
    [anon_sym_i64] = ACTIONS(2281),
    [anon_sym_u128] = ACTIONS(2281),
    [anon_sym_i128] = ACTIONS(2281),
    [anon_sym_isize] = ACTIONS(2281),
    [anon_sym_usize] = ACTIONS(2281),
    [anon_sym_f32] = ACTIONS(2281),
    [anon_sym_f64] = ACTIONS(2281),
    [anon_sym_bool] = ACTIONS(2281),
    [anon_sym_str] = ACTIONS(2281),
    [anon_sym_char] = ACTIONS(2281),
    [anon_sym_DASH] = ACTIONS(2279),
    [anon_sym_BANG] = ACTIONS(2279),
    [anon_sym_AMP] = ACTIONS(2279),
    [anon_sym_PIPE] = ACTIONS(2279),
    [anon_sym_LT] = ACTIONS(2279),
    [anon_sym_DOT_DOT] = ACTIONS(2279),
    [anon_sym_COLON_COLON] = ACTIONS(2279),
    [anon_sym_POUND] = ACTIONS(2279),
    [anon_sym_SQUOTE] = ACTIONS(2281),
    [anon_sym_async] = ACTIONS(2281),
    [anon_sym_break] = ACTIONS(2281),
    [anon_sym_const] = ACTIONS(2281),
    [anon_sym_continue] = ACTIONS(2281),
    [anon_sym_default] = ACTIONS(2281),
    [anon_sym_enum] = ACTIONS(2281),
    [anon_sym_fn] = ACTIONS(2281),
    [anon_sym_for] = ACTIONS(2281),
    [anon_sym_gen] = ACTIONS(2281),
    [anon_sym_if] = ACTIONS(2281),
    [anon_sym_impl] = ACTIONS(2281),
    [anon_sym_let] = ACTIONS(2281),
    [anon_sym_loop] = ACTIONS(2281),
    [anon_sym_match] = ACTIONS(2281),
    [anon_sym_mod] = ACTIONS(2281),
    [anon_sym_pub] = ACTIONS(2281),
    [anon_sym_return] = ACTIONS(2281),
    [anon_sym_static] = ACTIONS(2281),
    [anon_sym_struct] = ACTIONS(2281),
    [anon_sym_trait] = ACTIONS(2281),
    [anon_sym_type] = ACTIONS(2281),
    [anon_sym_union] = ACTIONS(2281),
    [anon_sym_unsafe] = ACTIONS(2281),
    [anon_sym_use] = ACTIONS(2281),
    [anon_sym_while] = ACTIONS(2281),
    [anon_sym_extern] = ACTIONS(2281),
    [anon_sym_yield] = ACTIONS(2281),
    [anon_sym_move] = ACTIONS(2281),
    [anon_sym_try] = ACTIONS(2281),
    [sym_integer_literal] = ACTIONS(2279),
    [aux_sym_string_literal_token1] = ACTIONS(2279),
    [sym_char_literal] = ACTIONS(2279),
    [anon_sym_true] = ACTIONS(2281),
    [anon_sym_false] = ACTIONS(2281),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2281),
    [sym_super] = ACTIONS(2281),
    [sym_crate] = ACTIONS(2281),
    [sym_metavariable] = ACTIONS(2279),
    [sym__raw_string_literal_start] = ACTIONS(2279),
    [sym_float_literal] = ACTIONS(2279),
  },
  [634] = {
    [sym_line_comment] = STATE(634),
    [sym_block_comment] = STATE(634),
    [ts_builtin_sym_end] = ACTIONS(2283),
    [sym_identifier] = ACTIONS(2285),
    [anon_sym_SEMI] = ACTIONS(2283),
    [anon_sym_macro_rules_BANG] = ACTIONS(2283),
    [anon_sym_LPAREN] = ACTIONS(2283),
    [anon_sym_LBRACK] = ACTIONS(2283),
    [anon_sym_LBRACE] = ACTIONS(2283),
    [anon_sym_RBRACE] = ACTIONS(2283),
    [anon_sym_STAR] = ACTIONS(2283),
    [anon_sym_u8] = ACTIONS(2285),
    [anon_sym_i8] = ACTIONS(2285),
    [anon_sym_u16] = ACTIONS(2285),
    [anon_sym_i16] = ACTIONS(2285),
    [anon_sym_u32] = ACTIONS(2285),
    [anon_sym_i32] = ACTIONS(2285),
    [anon_sym_u64] = ACTIONS(2285),
    [anon_sym_i64] = ACTIONS(2285),
    [anon_sym_u128] = ACTIONS(2285),
    [anon_sym_i128] = ACTIONS(2285),
    [anon_sym_isize] = ACTIONS(2285),
    [anon_sym_usize] = ACTIONS(2285),
    [anon_sym_f32] = ACTIONS(2285),
    [anon_sym_f64] = ACTIONS(2285),
    [anon_sym_bool] = ACTIONS(2285),
    [anon_sym_str] = ACTIONS(2285),
    [anon_sym_char] = ACTIONS(2285),
    [anon_sym_DASH] = ACTIONS(2283),
    [anon_sym_BANG] = ACTIONS(2283),
    [anon_sym_AMP] = ACTIONS(2283),
    [anon_sym_PIPE] = ACTIONS(2283),
    [anon_sym_LT] = ACTIONS(2283),
    [anon_sym_DOT_DOT] = ACTIONS(2283),
    [anon_sym_COLON_COLON] = ACTIONS(2283),
    [anon_sym_POUND] = ACTIONS(2283),
    [anon_sym_SQUOTE] = ACTIONS(2285),
    [anon_sym_async] = ACTIONS(2285),
    [anon_sym_break] = ACTIONS(2285),
    [anon_sym_const] = ACTIONS(2285),
    [anon_sym_continue] = ACTIONS(2285),
    [anon_sym_default] = ACTIONS(2285),
    [anon_sym_enum] = ACTIONS(2285),
    [anon_sym_fn] = ACTIONS(2285),
    [anon_sym_for] = ACTIONS(2285),
    [anon_sym_gen] = ACTIONS(2285),
    [anon_sym_if] = ACTIONS(2285),
    [anon_sym_impl] = ACTIONS(2285),
    [anon_sym_let] = ACTIONS(2285),
    [anon_sym_loop] = ACTIONS(2285),
    [anon_sym_match] = ACTIONS(2285),
    [anon_sym_mod] = ACTIONS(2285),
    [anon_sym_pub] = ACTIONS(2285),
    [anon_sym_return] = ACTIONS(2285),
    [anon_sym_static] = ACTIONS(2285),
    [anon_sym_struct] = ACTIONS(2285),
    [anon_sym_trait] = ACTIONS(2285),
    [anon_sym_type] = ACTIONS(2285),
    [anon_sym_union] = ACTIONS(2285),
    [anon_sym_unsafe] = ACTIONS(2285),
    [anon_sym_use] = ACTIONS(2285),
    [anon_sym_while] = ACTIONS(2285),
    [anon_sym_extern] = ACTIONS(2285),
    [anon_sym_yield] = ACTIONS(2285),
    [anon_sym_move] = ACTIONS(2285),
    [anon_sym_try] = ACTIONS(2285),
    [sym_integer_literal] = ACTIONS(2283),
    [aux_sym_string_literal_token1] = ACTIONS(2283),
    [sym_char_literal] = ACTIONS(2283),
    [anon_sym_true] = ACTIONS(2285),
    [anon_sym_false] = ACTIONS(2285),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2285),
    [sym_super] = ACTIONS(2285),
    [sym_crate] = ACTIONS(2285),
    [sym_metavariable] = ACTIONS(2283),
    [sym__raw_string_literal_start] = ACTIONS(2283),
    [sym_float_literal] = ACTIONS(2283),
  },
  [635] = {
    [sym_line_comment] = STATE(635),
    [sym_block_comment] = STATE(635),
    [ts_builtin_sym_end] = ACTIONS(2287),
    [sym_identifier] = ACTIONS(2289),
    [anon_sym_SEMI] = ACTIONS(2287),
    [anon_sym_macro_rules_BANG] = ACTIONS(2287),
    [anon_sym_LPAREN] = ACTIONS(2287),
    [anon_sym_LBRACK] = ACTIONS(2287),
    [anon_sym_LBRACE] = ACTIONS(2287),
    [anon_sym_RBRACE] = ACTIONS(2287),
    [anon_sym_STAR] = ACTIONS(2287),
    [anon_sym_u8] = ACTIONS(2289),
    [anon_sym_i8] = ACTIONS(2289),
    [anon_sym_u16] = ACTIONS(2289),
    [anon_sym_i16] = ACTIONS(2289),
    [anon_sym_u32] = ACTIONS(2289),
    [anon_sym_i32] = ACTIONS(2289),
    [anon_sym_u64] = ACTIONS(2289),
    [anon_sym_i64] = ACTIONS(2289),
    [anon_sym_u128] = ACTIONS(2289),
    [anon_sym_i128] = ACTIONS(2289),
    [anon_sym_isize] = ACTIONS(2289),
    [anon_sym_usize] = ACTIONS(2289),
    [anon_sym_f32] = ACTIONS(2289),
    [anon_sym_f64] = ACTIONS(2289),
    [anon_sym_bool] = ACTIONS(2289),
    [anon_sym_str] = ACTIONS(2289),
    [anon_sym_char] = ACTIONS(2289),
    [anon_sym_DASH] = ACTIONS(2287),
    [anon_sym_BANG] = ACTIONS(2287),
    [anon_sym_AMP] = ACTIONS(2287),
    [anon_sym_PIPE] = ACTIONS(2287),
    [anon_sym_LT] = ACTIONS(2287),
    [anon_sym_DOT_DOT] = ACTIONS(2287),
    [anon_sym_COLON_COLON] = ACTIONS(2287),
    [anon_sym_POUND] = ACTIONS(2287),
    [anon_sym_SQUOTE] = ACTIONS(2289),
    [anon_sym_async] = ACTIONS(2289),
    [anon_sym_break] = ACTIONS(2289),
    [anon_sym_const] = ACTIONS(2289),
    [anon_sym_continue] = ACTIONS(2289),
    [anon_sym_default] = ACTIONS(2289),
    [anon_sym_enum] = ACTIONS(2289),
    [anon_sym_fn] = ACTIONS(2289),
    [anon_sym_for] = ACTIONS(2289),
    [anon_sym_gen] = ACTIONS(2289),
    [anon_sym_if] = ACTIONS(2289),
    [anon_sym_impl] = ACTIONS(2289),
    [anon_sym_let] = ACTIONS(2289),
    [anon_sym_loop] = ACTIONS(2289),
    [anon_sym_match] = ACTIONS(2289),
    [anon_sym_mod] = ACTIONS(2289),
    [anon_sym_pub] = ACTIONS(2289),
    [anon_sym_return] = ACTIONS(2289),
    [anon_sym_static] = ACTIONS(2289),
    [anon_sym_struct] = ACTIONS(2289),
    [anon_sym_trait] = ACTIONS(2289),
    [anon_sym_type] = ACTIONS(2289),
    [anon_sym_union] = ACTIONS(2289),
    [anon_sym_unsafe] = ACTIONS(2289),
    [anon_sym_use] = ACTIONS(2289),
    [anon_sym_while] = ACTIONS(2289),
    [anon_sym_extern] = ACTIONS(2289),
    [anon_sym_yield] = ACTIONS(2289),
    [anon_sym_move] = ACTIONS(2289),
    [anon_sym_try] = ACTIONS(2289),
    [sym_integer_literal] = ACTIONS(2287),
    [aux_sym_string_literal_token1] = ACTIONS(2287),
    [sym_char_literal] = ACTIONS(2287),
    [anon_sym_true] = ACTIONS(2289),
    [anon_sym_false] = ACTIONS(2289),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2289),
    [sym_super] = ACTIONS(2289),
    [sym_crate] = ACTIONS(2289),
    [sym_metavariable] = ACTIONS(2287),
    [sym__raw_string_literal_start] = ACTIONS(2287),
    [sym_float_literal] = ACTIONS(2287),
  },
  [636] = {
    [sym_line_comment] = STATE(636),
    [sym_block_comment] = STATE(636),
    [ts_builtin_sym_end] = ACTIONS(2291),
    [sym_identifier] = ACTIONS(2293),
    [anon_sym_SEMI] = ACTIONS(2291),
    [anon_sym_macro_rules_BANG] = ACTIONS(2291),
    [anon_sym_LPAREN] = ACTIONS(2291),
    [anon_sym_LBRACK] = ACTIONS(2291),
    [anon_sym_LBRACE] = ACTIONS(2291),
    [anon_sym_RBRACE] = ACTIONS(2291),
    [anon_sym_STAR] = ACTIONS(2291),
    [anon_sym_u8] = ACTIONS(2293),
    [anon_sym_i8] = ACTIONS(2293),
    [anon_sym_u16] = ACTIONS(2293),
    [anon_sym_i16] = ACTIONS(2293),
    [anon_sym_u32] = ACTIONS(2293),
    [anon_sym_i32] = ACTIONS(2293),
    [anon_sym_u64] = ACTIONS(2293),
    [anon_sym_i64] = ACTIONS(2293),
    [anon_sym_u128] = ACTIONS(2293),
    [anon_sym_i128] = ACTIONS(2293),
    [anon_sym_isize] = ACTIONS(2293),
    [anon_sym_usize] = ACTIONS(2293),
    [anon_sym_f32] = ACTIONS(2293),
    [anon_sym_f64] = ACTIONS(2293),
    [anon_sym_bool] = ACTIONS(2293),
    [anon_sym_str] = ACTIONS(2293),
    [anon_sym_char] = ACTIONS(2293),
    [anon_sym_DASH] = ACTIONS(2291),
    [anon_sym_BANG] = ACTIONS(2291),
    [anon_sym_AMP] = ACTIONS(2291),
    [anon_sym_PIPE] = ACTIONS(2291),
    [anon_sym_LT] = ACTIONS(2291),
    [anon_sym_DOT_DOT] = ACTIONS(2291),
    [anon_sym_COLON_COLON] = ACTIONS(2291),
    [anon_sym_POUND] = ACTIONS(2291),
    [anon_sym_SQUOTE] = ACTIONS(2293),
    [anon_sym_async] = ACTIONS(2293),
    [anon_sym_break] = ACTIONS(2293),
    [anon_sym_const] = ACTIONS(2293),
    [anon_sym_continue] = ACTIONS(2293),
    [anon_sym_default] = ACTIONS(2293),
    [anon_sym_enum] = ACTIONS(2293),
    [anon_sym_fn] = ACTIONS(2293),
    [anon_sym_for] = ACTIONS(2293),
    [anon_sym_gen] = ACTIONS(2293),
    [anon_sym_if] = ACTIONS(2293),
    [anon_sym_impl] = ACTIONS(2293),
    [anon_sym_let] = ACTIONS(2293),
    [anon_sym_loop] = ACTIONS(2293),
    [anon_sym_match] = ACTIONS(2293),
    [anon_sym_mod] = ACTIONS(2293),
    [anon_sym_pub] = ACTIONS(2293),
    [anon_sym_return] = ACTIONS(2293),
    [anon_sym_static] = ACTIONS(2293),
    [anon_sym_struct] = ACTIONS(2293),
    [anon_sym_trait] = ACTIONS(2293),
    [anon_sym_type] = ACTIONS(2293),
    [anon_sym_union] = ACTIONS(2293),
    [anon_sym_unsafe] = ACTIONS(2293),
    [anon_sym_use] = ACTIONS(2293),
    [anon_sym_while] = ACTIONS(2293),
    [anon_sym_extern] = ACTIONS(2293),
    [anon_sym_yield] = ACTIONS(2293),
    [anon_sym_move] = ACTIONS(2293),
    [anon_sym_try] = ACTIONS(2293),
    [sym_integer_literal] = ACTIONS(2291),
    [aux_sym_string_literal_token1] = ACTIONS(2291),
    [sym_char_literal] = ACTIONS(2291),
    [anon_sym_true] = ACTIONS(2293),
    [anon_sym_false] = ACTIONS(2293),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2293),
    [sym_super] = ACTIONS(2293),
    [sym_crate] = ACTIONS(2293),
    [sym_metavariable] = ACTIONS(2291),
    [sym__raw_string_literal_start] = ACTIONS(2291),
    [sym_float_literal] = ACTIONS(2291),
  },
  [637] = {
    [sym_line_comment] = STATE(637),
    [sym_block_comment] = STATE(637),
    [ts_builtin_sym_end] = ACTIONS(2295),
    [sym_identifier] = ACTIONS(2297),
    [anon_sym_SEMI] = ACTIONS(2295),
    [anon_sym_macro_rules_BANG] = ACTIONS(2295),
    [anon_sym_LPAREN] = ACTIONS(2295),
    [anon_sym_LBRACK] = ACTIONS(2295),
    [anon_sym_LBRACE] = ACTIONS(2295),
    [anon_sym_RBRACE] = ACTIONS(2295),
    [anon_sym_STAR] = ACTIONS(2295),
    [anon_sym_u8] = ACTIONS(2297),
    [anon_sym_i8] = ACTIONS(2297),
    [anon_sym_u16] = ACTIONS(2297),
    [anon_sym_i16] = ACTIONS(2297),
    [anon_sym_u32] = ACTIONS(2297),
    [anon_sym_i32] = ACTIONS(2297),
    [anon_sym_u64] = ACTIONS(2297),
    [anon_sym_i64] = ACTIONS(2297),
    [anon_sym_u128] = ACTIONS(2297),
    [anon_sym_i128] = ACTIONS(2297),
    [anon_sym_isize] = ACTIONS(2297),
    [anon_sym_usize] = ACTIONS(2297),
    [anon_sym_f32] = ACTIONS(2297),
    [anon_sym_f64] = ACTIONS(2297),
    [anon_sym_bool] = ACTIONS(2297),
    [anon_sym_str] = ACTIONS(2297),
    [anon_sym_char] = ACTIONS(2297),
    [anon_sym_DASH] = ACTIONS(2295),
    [anon_sym_BANG] = ACTIONS(2295),
    [anon_sym_AMP] = ACTIONS(2295),
    [anon_sym_PIPE] = ACTIONS(2295),
    [anon_sym_LT] = ACTIONS(2295),
    [anon_sym_DOT_DOT] = ACTIONS(2295),
    [anon_sym_COLON_COLON] = ACTIONS(2295),
    [anon_sym_POUND] = ACTIONS(2295),
    [anon_sym_SQUOTE] = ACTIONS(2297),
    [anon_sym_async] = ACTIONS(2297),
    [anon_sym_break] = ACTIONS(2297),
    [anon_sym_const] = ACTIONS(2297),
    [anon_sym_continue] = ACTIONS(2297),
    [anon_sym_default] = ACTIONS(2297),
    [anon_sym_enum] = ACTIONS(2297),
    [anon_sym_fn] = ACTIONS(2297),
    [anon_sym_for] = ACTIONS(2297),
    [anon_sym_gen] = ACTIONS(2297),
    [anon_sym_if] = ACTIONS(2297),
    [anon_sym_impl] = ACTIONS(2297),
    [anon_sym_let] = ACTIONS(2297),
    [anon_sym_loop] = ACTIONS(2297),
    [anon_sym_match] = ACTIONS(2297),
    [anon_sym_mod] = ACTIONS(2297),
    [anon_sym_pub] = ACTIONS(2297),
    [anon_sym_return] = ACTIONS(2297),
    [anon_sym_static] = ACTIONS(2297),
    [anon_sym_struct] = ACTIONS(2297),
    [anon_sym_trait] = ACTIONS(2297),
    [anon_sym_type] = ACTIONS(2297),
    [anon_sym_union] = ACTIONS(2297),
    [anon_sym_unsafe] = ACTIONS(2297),
    [anon_sym_use] = ACTIONS(2297),
    [anon_sym_while] = ACTIONS(2297),
    [anon_sym_extern] = ACTIONS(2297),
    [anon_sym_yield] = ACTIONS(2297),
    [anon_sym_move] = ACTIONS(2297),
    [anon_sym_try] = ACTIONS(2297),
    [sym_integer_literal] = ACTIONS(2295),
    [aux_sym_string_literal_token1] = ACTIONS(2295),
    [sym_char_literal] = ACTIONS(2295),
    [anon_sym_true] = ACTIONS(2297),
    [anon_sym_false] = ACTIONS(2297),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2297),
    [sym_super] = ACTIONS(2297),
    [sym_crate] = ACTIONS(2297),
    [sym_metavariable] = ACTIONS(2295),
    [sym__raw_string_literal_start] = ACTIONS(2295),
    [sym_float_literal] = ACTIONS(2295),
  },
  [638] = {
    [sym_line_comment] = STATE(638),
    [sym_block_comment] = STATE(638),
    [ts_builtin_sym_end] = ACTIONS(2299),
    [sym_identifier] = ACTIONS(2301),
    [anon_sym_SEMI] = ACTIONS(2299),
    [anon_sym_macro_rules_BANG] = ACTIONS(2299),
    [anon_sym_LPAREN] = ACTIONS(2299),
    [anon_sym_LBRACK] = ACTIONS(2299),
    [anon_sym_LBRACE] = ACTIONS(2299),
    [anon_sym_RBRACE] = ACTIONS(2299),
    [anon_sym_STAR] = ACTIONS(2299),
    [anon_sym_u8] = ACTIONS(2301),
    [anon_sym_i8] = ACTIONS(2301),
    [anon_sym_u16] = ACTIONS(2301),
    [anon_sym_i16] = ACTIONS(2301),
    [anon_sym_u32] = ACTIONS(2301),
    [anon_sym_i32] = ACTIONS(2301),
    [anon_sym_u64] = ACTIONS(2301),
    [anon_sym_i64] = ACTIONS(2301),
    [anon_sym_u128] = ACTIONS(2301),
    [anon_sym_i128] = ACTIONS(2301),
    [anon_sym_isize] = ACTIONS(2301),
    [anon_sym_usize] = ACTIONS(2301),
    [anon_sym_f32] = ACTIONS(2301),
    [anon_sym_f64] = ACTIONS(2301),
    [anon_sym_bool] = ACTIONS(2301),
    [anon_sym_str] = ACTIONS(2301),
    [anon_sym_char] = ACTIONS(2301),
    [anon_sym_DASH] = ACTIONS(2299),
    [anon_sym_BANG] = ACTIONS(2299),
    [anon_sym_AMP] = ACTIONS(2299),
    [anon_sym_PIPE] = ACTIONS(2299),
    [anon_sym_LT] = ACTIONS(2299),
    [anon_sym_DOT_DOT] = ACTIONS(2299),
    [anon_sym_COLON_COLON] = ACTIONS(2299),
    [anon_sym_POUND] = ACTIONS(2299),
    [anon_sym_SQUOTE] = ACTIONS(2301),
    [anon_sym_async] = ACTIONS(2301),
    [anon_sym_break] = ACTIONS(2301),
    [anon_sym_const] = ACTIONS(2301),
    [anon_sym_continue] = ACTIONS(2301),
    [anon_sym_default] = ACTIONS(2301),
    [anon_sym_enum] = ACTIONS(2301),
    [anon_sym_fn] = ACTIONS(2301),
    [anon_sym_for] = ACTIONS(2301),
    [anon_sym_gen] = ACTIONS(2301),
    [anon_sym_if] = ACTIONS(2301),
    [anon_sym_impl] = ACTIONS(2301),
    [anon_sym_let] = ACTIONS(2301),
    [anon_sym_loop] = ACTIONS(2301),
    [anon_sym_match] = ACTIONS(2301),
    [anon_sym_mod] = ACTIONS(2301),
    [anon_sym_pub] = ACTIONS(2301),
    [anon_sym_return] = ACTIONS(2301),
    [anon_sym_static] = ACTIONS(2301),
    [anon_sym_struct] = ACTIONS(2301),
    [anon_sym_trait] = ACTIONS(2301),
    [anon_sym_type] = ACTIONS(2301),
    [anon_sym_union] = ACTIONS(2301),
    [anon_sym_unsafe] = ACTIONS(2301),
    [anon_sym_use] = ACTIONS(2301),
    [anon_sym_while] = ACTIONS(2301),
    [anon_sym_extern] = ACTIONS(2301),
    [anon_sym_yield] = ACTIONS(2301),
    [anon_sym_move] = ACTIONS(2301),
    [anon_sym_try] = ACTIONS(2301),
    [sym_integer_literal] = ACTIONS(2299),
    [aux_sym_string_literal_token1] = ACTIONS(2299),
    [sym_char_literal] = ACTIONS(2299),
    [anon_sym_true] = ACTIONS(2301),
    [anon_sym_false] = ACTIONS(2301),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2301),
    [sym_super] = ACTIONS(2301),
    [sym_crate] = ACTIONS(2301),
    [sym_metavariable] = ACTIONS(2299),
    [sym__raw_string_literal_start] = ACTIONS(2299),
    [sym_float_literal] = ACTIONS(2299),
  },
  [639] = {
    [sym_empty_statement] = STATE(1072),
    [sym_macro_definition] = STATE(1072),
    [sym_attribute_item] = STATE(1072),
    [sym_inner_attribute_item] = STATE(1072),
    [sym_mod_item] = STATE(1072),
    [sym_foreign_mod_item] = STATE(1072),
    [sym_struct_item] = STATE(1072),
    [sym_union_item] = STATE(1072),
    [sym_enum_item] = STATE(1072),
    [sym_extern_crate_declaration] = STATE(1072),
    [sym_const_item] = STATE(1072),
    [sym_static_item] = STATE(1072),
    [sym_type_item] = STATE(1072),
    [sym_function_item] = STATE(1072),
    [sym_function_signature_item] = STATE(1072),
    [sym_function_modifiers] = STATE(3622),
    [sym_impl_item] = STATE(1072),
    [sym_trait_item] = STATE(1072),
    [sym_associated_type] = STATE(1072),
    [sym_let_declaration] = STATE(1072),
    [sym_use_declaration] = STATE(1072),
    [sym_extern_modifier] = STATE(2165),
    [sym_visibility_modifier] = STATE(1954),
    [sym_bracketed_type] = STATE(3353),
    [sym_generic_type_with_turbofish] = STATE(3379),
    [sym_macro_invocation] = STATE(1072),
    [sym_scoped_identifier] = STATE(3167),
    [sym_line_comment] = STATE(639),
    [sym_block_comment] = STATE(639),
    [aux_sym_declaration_list_repeat1] = STATE(639),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(2303),
    [anon_sym_SEMI] = ACTIONS(2306),
    [anon_sym_macro_rules_BANG] = ACTIONS(2309),
    [anon_sym_RBRACE] = ACTIONS(2312),
    [anon_sym_u8] = ACTIONS(2314),
    [anon_sym_i8] = ACTIONS(2314),
    [anon_sym_u16] = ACTIONS(2314),
    [anon_sym_i16] = ACTIONS(2314),
    [anon_sym_u32] = ACTIONS(2314),
    [anon_sym_i32] = ACTIONS(2314),
    [anon_sym_u64] = ACTIONS(2314),
    [anon_sym_i64] = ACTIONS(2314),
    [anon_sym_u128] = ACTIONS(2314),
    [anon_sym_i128] = ACTIONS(2314),
    [anon_sym_isize] = ACTIONS(2314),
    [anon_sym_usize] = ACTIONS(2314),
    [anon_sym_f32] = ACTIONS(2314),
    [anon_sym_f64] = ACTIONS(2314),
    [anon_sym_bool] = ACTIONS(2314),
    [anon_sym_str] = ACTIONS(2314),
    [anon_sym_char] = ACTIONS(2314),
    [anon_sym_LT] = ACTIONS(2317),
    [anon_sym_COLON_COLON] = ACTIONS(2320),
    [anon_sym_POUND] = ACTIONS(2323),
    [anon_sym_async] = ACTIONS(2326),
    [anon_sym_const] = ACTIONS(2329),
    [anon_sym_default] = ACTIONS(2332),
    [anon_sym_enum] = ACTIONS(2335),
    [anon_sym_fn] = ACTIONS(2338),
    [anon_sym_gen] = ACTIONS(2341),
    [anon_sym_impl] = ACTIONS(2344),
    [anon_sym_let] = ACTIONS(2347),
    [anon_sym_mod] = ACTIONS(2350),
    [anon_sym_pub] = ACTIONS(2353),
    [anon_sym_static] = ACTIONS(2356),
    [anon_sym_struct] = ACTIONS(2359),
    [anon_sym_trait] = ACTIONS(2362),
    [anon_sym_type] = ACTIONS(2365),
    [anon_sym_union] = ACTIONS(2368),
    [anon_sym_unsafe] = ACTIONS(2371),
    [anon_sym_use] = ACTIONS(2374),
    [anon_sym_extern] = ACTIONS(2377),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2380),
    [sym_super] = ACTIONS(2380),
    [sym_crate] = ACTIONS(2383),
    [sym_metavariable] = ACTIONS(2386),
  },
  [640] = {
    [sym_line_comment] = STATE(640),
    [sym_block_comment] = STATE(640),
    [ts_builtin_sym_end] = ACTIONS(2389),
    [sym_identifier] = ACTIONS(2391),
    [anon_sym_SEMI] = ACTIONS(2389),
    [anon_sym_macro_rules_BANG] = ACTIONS(2389),
    [anon_sym_LPAREN] = ACTIONS(2389),
    [anon_sym_LBRACK] = ACTIONS(2389),
    [anon_sym_LBRACE] = ACTIONS(2389),
    [anon_sym_RBRACE] = ACTIONS(2389),
    [anon_sym_STAR] = ACTIONS(2389),
    [anon_sym_u8] = ACTIONS(2391),
    [anon_sym_i8] = ACTIONS(2391),
    [anon_sym_u16] = ACTIONS(2391),
    [anon_sym_i16] = ACTIONS(2391),
    [anon_sym_u32] = ACTIONS(2391),
    [anon_sym_i32] = ACTIONS(2391),
    [anon_sym_u64] = ACTIONS(2391),
    [anon_sym_i64] = ACTIONS(2391),
    [anon_sym_u128] = ACTIONS(2391),
    [anon_sym_i128] = ACTIONS(2391),
    [anon_sym_isize] = ACTIONS(2391),
    [anon_sym_usize] = ACTIONS(2391),
    [anon_sym_f32] = ACTIONS(2391),
    [anon_sym_f64] = ACTIONS(2391),
    [anon_sym_bool] = ACTIONS(2391),
    [anon_sym_str] = ACTIONS(2391),
    [anon_sym_char] = ACTIONS(2391),
    [anon_sym_DASH] = ACTIONS(2389),
    [anon_sym_BANG] = ACTIONS(2389),
    [anon_sym_AMP] = ACTIONS(2389),
    [anon_sym_PIPE] = ACTIONS(2389),
    [anon_sym_LT] = ACTIONS(2389),
    [anon_sym_DOT_DOT] = ACTIONS(2389),
    [anon_sym_COLON_COLON] = ACTIONS(2389),
    [anon_sym_POUND] = ACTIONS(2389),
    [anon_sym_SQUOTE] = ACTIONS(2391),
    [anon_sym_async] = ACTIONS(2391),
    [anon_sym_break] = ACTIONS(2391),
    [anon_sym_const] = ACTIONS(2391),
    [anon_sym_continue] = ACTIONS(2391),
    [anon_sym_default] = ACTIONS(2391),
    [anon_sym_enum] = ACTIONS(2391),
    [anon_sym_fn] = ACTIONS(2391),
    [anon_sym_for] = ACTIONS(2391),
    [anon_sym_gen] = ACTIONS(2391),
    [anon_sym_if] = ACTIONS(2391),
    [anon_sym_impl] = ACTIONS(2391),
    [anon_sym_let] = ACTIONS(2391),
    [anon_sym_loop] = ACTIONS(2391),
    [anon_sym_match] = ACTIONS(2391),
    [anon_sym_mod] = ACTIONS(2391),
    [anon_sym_pub] = ACTIONS(2391),
    [anon_sym_return] = ACTIONS(2391),
    [anon_sym_static] = ACTIONS(2391),
    [anon_sym_struct] = ACTIONS(2391),
    [anon_sym_trait] = ACTIONS(2391),
    [anon_sym_type] = ACTIONS(2391),
    [anon_sym_union] = ACTIONS(2391),
    [anon_sym_unsafe] = ACTIONS(2391),
    [anon_sym_use] = ACTIONS(2391),
    [anon_sym_while] = ACTIONS(2391),
    [anon_sym_extern] = ACTIONS(2391),
    [anon_sym_yield] = ACTIONS(2391),
    [anon_sym_move] = ACTIONS(2391),
    [anon_sym_try] = ACTIONS(2391),
    [sym_integer_literal] = ACTIONS(2389),
    [aux_sym_string_literal_token1] = ACTIONS(2389),
    [sym_char_literal] = ACTIONS(2389),
    [anon_sym_true] = ACTIONS(2391),
    [anon_sym_false] = ACTIONS(2391),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2391),
    [sym_super] = ACTIONS(2391),
    [sym_crate] = ACTIONS(2391),
    [sym_metavariable] = ACTIONS(2389),
    [sym__raw_string_literal_start] = ACTIONS(2389),
    [sym_float_literal] = ACTIONS(2389),
  },
  [641] = {
    [sym_line_comment] = STATE(641),
    [sym_block_comment] = STATE(641),
    [ts_builtin_sym_end] = ACTIONS(2393),
    [sym_identifier] = ACTIONS(2395),
    [anon_sym_SEMI] = ACTIONS(2393),
    [anon_sym_macro_rules_BANG] = ACTIONS(2393),
    [anon_sym_LPAREN] = ACTIONS(2393),
    [anon_sym_LBRACK] = ACTIONS(2393),
    [anon_sym_LBRACE] = ACTIONS(2393),
    [anon_sym_RBRACE] = ACTIONS(2393),
    [anon_sym_STAR] = ACTIONS(2393),
    [anon_sym_u8] = ACTIONS(2395),
    [anon_sym_i8] = ACTIONS(2395),
    [anon_sym_u16] = ACTIONS(2395),
    [anon_sym_i16] = ACTIONS(2395),
    [anon_sym_u32] = ACTIONS(2395),
    [anon_sym_i32] = ACTIONS(2395),
    [anon_sym_u64] = ACTIONS(2395),
    [anon_sym_i64] = ACTIONS(2395),
    [anon_sym_u128] = ACTIONS(2395),
    [anon_sym_i128] = ACTIONS(2395),
    [anon_sym_isize] = ACTIONS(2395),
    [anon_sym_usize] = ACTIONS(2395),
    [anon_sym_f32] = ACTIONS(2395),
    [anon_sym_f64] = ACTIONS(2395),
    [anon_sym_bool] = ACTIONS(2395),
    [anon_sym_str] = ACTIONS(2395),
    [anon_sym_char] = ACTIONS(2395),
    [anon_sym_DASH] = ACTIONS(2393),
    [anon_sym_BANG] = ACTIONS(2393),
    [anon_sym_AMP] = ACTIONS(2393),
    [anon_sym_PIPE] = ACTIONS(2393),
    [anon_sym_LT] = ACTIONS(2393),
    [anon_sym_DOT_DOT] = ACTIONS(2393),
    [anon_sym_COLON_COLON] = ACTIONS(2393),
    [anon_sym_POUND] = ACTIONS(2393),
    [anon_sym_SQUOTE] = ACTIONS(2395),
    [anon_sym_async] = ACTIONS(2395),
    [anon_sym_break] = ACTIONS(2395),
    [anon_sym_const] = ACTIONS(2395),
    [anon_sym_continue] = ACTIONS(2395),
    [anon_sym_default] = ACTIONS(2395),
    [anon_sym_enum] = ACTIONS(2395),
    [anon_sym_fn] = ACTIONS(2395),
    [anon_sym_for] = ACTIONS(2395),
    [anon_sym_gen] = ACTIONS(2395),
    [anon_sym_if] = ACTIONS(2395),
    [anon_sym_impl] = ACTIONS(2395),
    [anon_sym_let] = ACTIONS(2395),
    [anon_sym_loop] = ACTIONS(2395),
    [anon_sym_match] = ACTIONS(2395),
    [anon_sym_mod] = ACTIONS(2395),
    [anon_sym_pub] = ACTIONS(2395),
    [anon_sym_return] = ACTIONS(2395),
    [anon_sym_static] = ACTIONS(2395),
    [anon_sym_struct] = ACTIONS(2395),
    [anon_sym_trait] = ACTIONS(2395),
    [anon_sym_type] = ACTIONS(2395),
    [anon_sym_union] = ACTIONS(2395),
    [anon_sym_unsafe] = ACTIONS(2395),
    [anon_sym_use] = ACTIONS(2395),
    [anon_sym_while] = ACTIONS(2395),
    [anon_sym_extern] = ACTIONS(2395),
    [anon_sym_yield] = ACTIONS(2395),
    [anon_sym_move] = ACTIONS(2395),
    [anon_sym_try] = ACTIONS(2395),
    [sym_integer_literal] = ACTIONS(2393),
    [aux_sym_string_literal_token1] = ACTIONS(2393),
    [sym_char_literal] = ACTIONS(2393),
    [anon_sym_true] = ACTIONS(2395),
    [anon_sym_false] = ACTIONS(2395),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2395),
    [sym_super] = ACTIONS(2395),
    [sym_crate] = ACTIONS(2395),
    [sym_metavariable] = ACTIONS(2393),
    [sym__raw_string_literal_start] = ACTIONS(2393),
    [sym_float_literal] = ACTIONS(2393),
  },
  [642] = {
    [sym_line_comment] = STATE(642),
    [sym_block_comment] = STATE(642),
    [ts_builtin_sym_end] = ACTIONS(2397),
    [sym_identifier] = ACTIONS(2399),
    [anon_sym_SEMI] = ACTIONS(2397),
    [anon_sym_macro_rules_BANG] = ACTIONS(2397),
    [anon_sym_LPAREN] = ACTIONS(2397),
    [anon_sym_LBRACK] = ACTIONS(2397),
    [anon_sym_LBRACE] = ACTIONS(2397),
    [anon_sym_RBRACE] = ACTIONS(2397),
    [anon_sym_STAR] = ACTIONS(2397),
    [anon_sym_u8] = ACTIONS(2399),
    [anon_sym_i8] = ACTIONS(2399),
    [anon_sym_u16] = ACTIONS(2399),
    [anon_sym_i16] = ACTIONS(2399),
    [anon_sym_u32] = ACTIONS(2399),
    [anon_sym_i32] = ACTIONS(2399),
    [anon_sym_u64] = ACTIONS(2399),
    [anon_sym_i64] = ACTIONS(2399),
    [anon_sym_u128] = ACTIONS(2399),
    [anon_sym_i128] = ACTIONS(2399),
    [anon_sym_isize] = ACTIONS(2399),
    [anon_sym_usize] = ACTIONS(2399),
    [anon_sym_f32] = ACTIONS(2399),
    [anon_sym_f64] = ACTIONS(2399),
    [anon_sym_bool] = ACTIONS(2399),
    [anon_sym_str] = ACTIONS(2399),
    [anon_sym_char] = ACTIONS(2399),
    [anon_sym_DASH] = ACTIONS(2397),
    [anon_sym_BANG] = ACTIONS(2397),
    [anon_sym_AMP] = ACTIONS(2397),
    [anon_sym_PIPE] = ACTIONS(2397),
    [anon_sym_LT] = ACTIONS(2397),
    [anon_sym_DOT_DOT] = ACTIONS(2397),
    [anon_sym_COLON_COLON] = ACTIONS(2397),
    [anon_sym_POUND] = ACTIONS(2397),
    [anon_sym_SQUOTE] = ACTIONS(2399),
    [anon_sym_async] = ACTIONS(2399),
    [anon_sym_break] = ACTIONS(2399),
    [anon_sym_const] = ACTIONS(2399),
    [anon_sym_continue] = ACTIONS(2399),
    [anon_sym_default] = ACTIONS(2399),
    [anon_sym_enum] = ACTIONS(2399),
    [anon_sym_fn] = ACTIONS(2399),
    [anon_sym_for] = ACTIONS(2399),
    [anon_sym_gen] = ACTIONS(2399),
    [anon_sym_if] = ACTIONS(2399),
    [anon_sym_impl] = ACTIONS(2399),
    [anon_sym_let] = ACTIONS(2399),
    [anon_sym_loop] = ACTIONS(2399),
    [anon_sym_match] = ACTIONS(2399),
    [anon_sym_mod] = ACTIONS(2399),
    [anon_sym_pub] = ACTIONS(2399),
    [anon_sym_return] = ACTIONS(2399),
    [anon_sym_static] = ACTIONS(2399),
    [anon_sym_struct] = ACTIONS(2399),
    [anon_sym_trait] = ACTIONS(2399),
    [anon_sym_type] = ACTIONS(2399),
    [anon_sym_union] = ACTIONS(2399),
    [anon_sym_unsafe] = ACTIONS(2399),
    [anon_sym_use] = ACTIONS(2399),
    [anon_sym_while] = ACTIONS(2399),
    [anon_sym_extern] = ACTIONS(2399),
    [anon_sym_yield] = ACTIONS(2399),
    [anon_sym_move] = ACTIONS(2399),
    [anon_sym_try] = ACTIONS(2399),
    [sym_integer_literal] = ACTIONS(2397),
    [aux_sym_string_literal_token1] = ACTIONS(2397),
    [sym_char_literal] = ACTIONS(2397),
    [anon_sym_true] = ACTIONS(2399),
    [anon_sym_false] = ACTIONS(2399),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2399),
    [sym_super] = ACTIONS(2399),
    [sym_crate] = ACTIONS(2399),
    [sym_metavariable] = ACTIONS(2397),
    [sym__raw_string_literal_start] = ACTIONS(2397),
    [sym_float_literal] = ACTIONS(2397),
  },
  [643] = {
    [sym_line_comment] = STATE(643),
    [sym_block_comment] = STATE(643),
    [ts_builtin_sym_end] = ACTIONS(2401),
    [sym_identifier] = ACTIONS(2403),
    [anon_sym_SEMI] = ACTIONS(2401),
    [anon_sym_macro_rules_BANG] = ACTIONS(2401),
    [anon_sym_LPAREN] = ACTIONS(2401),
    [anon_sym_LBRACK] = ACTIONS(2401),
    [anon_sym_LBRACE] = ACTIONS(2401),
    [anon_sym_RBRACE] = ACTIONS(2401),
    [anon_sym_STAR] = ACTIONS(2401),
    [anon_sym_u8] = ACTIONS(2403),
    [anon_sym_i8] = ACTIONS(2403),
    [anon_sym_u16] = ACTIONS(2403),
    [anon_sym_i16] = ACTIONS(2403),
    [anon_sym_u32] = ACTIONS(2403),
    [anon_sym_i32] = ACTIONS(2403),
    [anon_sym_u64] = ACTIONS(2403),
    [anon_sym_i64] = ACTIONS(2403),
    [anon_sym_u128] = ACTIONS(2403),
    [anon_sym_i128] = ACTIONS(2403),
    [anon_sym_isize] = ACTIONS(2403),
    [anon_sym_usize] = ACTIONS(2403),
    [anon_sym_f32] = ACTIONS(2403),
    [anon_sym_f64] = ACTIONS(2403),
    [anon_sym_bool] = ACTIONS(2403),
    [anon_sym_str] = ACTIONS(2403),
    [anon_sym_char] = ACTIONS(2403),
    [anon_sym_DASH] = ACTIONS(2401),
    [anon_sym_BANG] = ACTIONS(2401),
    [anon_sym_AMP] = ACTIONS(2401),
    [anon_sym_PIPE] = ACTIONS(2401),
    [anon_sym_LT] = ACTIONS(2401),
    [anon_sym_DOT_DOT] = ACTIONS(2401),
    [anon_sym_COLON_COLON] = ACTIONS(2401),
    [anon_sym_POUND] = ACTIONS(2401),
    [anon_sym_SQUOTE] = ACTIONS(2403),
    [anon_sym_async] = ACTIONS(2403),
    [anon_sym_break] = ACTIONS(2403),
    [anon_sym_const] = ACTIONS(2403),
    [anon_sym_continue] = ACTIONS(2403),
    [anon_sym_default] = ACTIONS(2403),
    [anon_sym_enum] = ACTIONS(2403),
    [anon_sym_fn] = ACTIONS(2403),
    [anon_sym_for] = ACTIONS(2403),
    [anon_sym_gen] = ACTIONS(2403),
    [anon_sym_if] = ACTIONS(2403),
    [anon_sym_impl] = ACTIONS(2403),
    [anon_sym_let] = ACTIONS(2403),
    [anon_sym_loop] = ACTIONS(2403),
    [anon_sym_match] = ACTIONS(2403),
    [anon_sym_mod] = ACTIONS(2403),
    [anon_sym_pub] = ACTIONS(2403),
    [anon_sym_return] = ACTIONS(2403),
    [anon_sym_static] = ACTIONS(2403),
    [anon_sym_struct] = ACTIONS(2403),
    [anon_sym_trait] = ACTIONS(2403),
    [anon_sym_type] = ACTIONS(2403),
    [anon_sym_union] = ACTIONS(2403),
    [anon_sym_unsafe] = ACTIONS(2403),
    [anon_sym_use] = ACTIONS(2403),
    [anon_sym_while] = ACTIONS(2403),
    [anon_sym_extern] = ACTIONS(2403),
    [anon_sym_yield] = ACTIONS(2403),
    [anon_sym_move] = ACTIONS(2403),
    [anon_sym_try] = ACTIONS(2403),
    [sym_integer_literal] = ACTIONS(2401),
    [aux_sym_string_literal_token1] = ACTIONS(2401),
    [sym_char_literal] = ACTIONS(2401),
    [anon_sym_true] = ACTIONS(2403),
    [anon_sym_false] = ACTIONS(2403),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2403),
    [sym_super] = ACTIONS(2403),
    [sym_crate] = ACTIONS(2403),
    [sym_metavariable] = ACTIONS(2401),
    [sym__raw_string_literal_start] = ACTIONS(2401),
    [sym_float_literal] = ACTIONS(2401),
  },
  [644] = {
    [sym_line_comment] = STATE(644),
    [sym_block_comment] = STATE(644),
    [ts_builtin_sym_end] = ACTIONS(2405),
    [sym_identifier] = ACTIONS(2407),
    [anon_sym_SEMI] = ACTIONS(2405),
    [anon_sym_macro_rules_BANG] = ACTIONS(2405),
    [anon_sym_LPAREN] = ACTIONS(2405),
    [anon_sym_LBRACK] = ACTIONS(2405),
    [anon_sym_LBRACE] = ACTIONS(2405),
    [anon_sym_RBRACE] = ACTIONS(2405),
    [anon_sym_STAR] = ACTIONS(2405),
    [anon_sym_u8] = ACTIONS(2407),
    [anon_sym_i8] = ACTIONS(2407),
    [anon_sym_u16] = ACTIONS(2407),
    [anon_sym_i16] = ACTIONS(2407),
    [anon_sym_u32] = ACTIONS(2407),
    [anon_sym_i32] = ACTIONS(2407),
    [anon_sym_u64] = ACTIONS(2407),
    [anon_sym_i64] = ACTIONS(2407),
    [anon_sym_u128] = ACTIONS(2407),
    [anon_sym_i128] = ACTIONS(2407),
    [anon_sym_isize] = ACTIONS(2407),
    [anon_sym_usize] = ACTIONS(2407),
    [anon_sym_f32] = ACTIONS(2407),
    [anon_sym_f64] = ACTIONS(2407),
    [anon_sym_bool] = ACTIONS(2407),
    [anon_sym_str] = ACTIONS(2407),
    [anon_sym_char] = ACTIONS(2407),
    [anon_sym_DASH] = ACTIONS(2405),
    [anon_sym_BANG] = ACTIONS(2405),
    [anon_sym_AMP] = ACTIONS(2405),
    [anon_sym_PIPE] = ACTIONS(2405),
    [anon_sym_LT] = ACTIONS(2405),
    [anon_sym_DOT_DOT] = ACTIONS(2405),
    [anon_sym_COLON_COLON] = ACTIONS(2405),
    [anon_sym_POUND] = ACTIONS(2405),
    [anon_sym_SQUOTE] = ACTIONS(2407),
    [anon_sym_async] = ACTIONS(2407),
    [anon_sym_break] = ACTIONS(2407),
    [anon_sym_const] = ACTIONS(2407),
    [anon_sym_continue] = ACTIONS(2407),
    [anon_sym_default] = ACTIONS(2407),
    [anon_sym_enum] = ACTIONS(2407),
    [anon_sym_fn] = ACTIONS(2407),
    [anon_sym_for] = ACTIONS(2407),
    [anon_sym_gen] = ACTIONS(2407),
    [anon_sym_if] = ACTIONS(2407),
    [anon_sym_impl] = ACTIONS(2407),
    [anon_sym_let] = ACTIONS(2407),
    [anon_sym_loop] = ACTIONS(2407),
    [anon_sym_match] = ACTIONS(2407),
    [anon_sym_mod] = ACTIONS(2407),
    [anon_sym_pub] = ACTIONS(2407),
    [anon_sym_return] = ACTIONS(2407),
    [anon_sym_static] = ACTIONS(2407),
    [anon_sym_struct] = ACTIONS(2407),
    [anon_sym_trait] = ACTIONS(2407),
    [anon_sym_type] = ACTIONS(2407),
    [anon_sym_union] = ACTIONS(2407),
    [anon_sym_unsafe] = ACTIONS(2407),
    [anon_sym_use] = ACTIONS(2407),
    [anon_sym_while] = ACTIONS(2407),
    [anon_sym_extern] = ACTIONS(2407),
    [anon_sym_yield] = ACTIONS(2407),
    [anon_sym_move] = ACTIONS(2407),
    [anon_sym_try] = ACTIONS(2407),
    [sym_integer_literal] = ACTIONS(2405),
    [aux_sym_string_literal_token1] = ACTIONS(2405),
    [sym_char_literal] = ACTIONS(2405),
    [anon_sym_true] = ACTIONS(2407),
    [anon_sym_false] = ACTIONS(2407),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2407),
    [sym_super] = ACTIONS(2407),
    [sym_crate] = ACTIONS(2407),
    [sym_metavariable] = ACTIONS(2405),
    [sym__raw_string_literal_start] = ACTIONS(2405),
    [sym_float_literal] = ACTIONS(2405),
  },
  [645] = {
    [sym_line_comment] = STATE(645),
    [sym_block_comment] = STATE(645),
    [ts_builtin_sym_end] = ACTIONS(2409),
    [sym_identifier] = ACTIONS(2411),
    [anon_sym_SEMI] = ACTIONS(2409),
    [anon_sym_macro_rules_BANG] = ACTIONS(2409),
    [anon_sym_LPAREN] = ACTIONS(2409),
    [anon_sym_LBRACK] = ACTIONS(2409),
    [anon_sym_LBRACE] = ACTIONS(2409),
    [anon_sym_RBRACE] = ACTIONS(2409),
    [anon_sym_STAR] = ACTIONS(2409),
    [anon_sym_u8] = ACTIONS(2411),
    [anon_sym_i8] = ACTIONS(2411),
    [anon_sym_u16] = ACTIONS(2411),
    [anon_sym_i16] = ACTIONS(2411),
    [anon_sym_u32] = ACTIONS(2411),
    [anon_sym_i32] = ACTIONS(2411),
    [anon_sym_u64] = ACTIONS(2411),
    [anon_sym_i64] = ACTIONS(2411),
    [anon_sym_u128] = ACTIONS(2411),
    [anon_sym_i128] = ACTIONS(2411),
    [anon_sym_isize] = ACTIONS(2411),
    [anon_sym_usize] = ACTIONS(2411),
    [anon_sym_f32] = ACTIONS(2411),
    [anon_sym_f64] = ACTIONS(2411),
    [anon_sym_bool] = ACTIONS(2411),
    [anon_sym_str] = ACTIONS(2411),
    [anon_sym_char] = ACTIONS(2411),
    [anon_sym_DASH] = ACTIONS(2409),
    [anon_sym_BANG] = ACTIONS(2409),
    [anon_sym_AMP] = ACTIONS(2409),
    [anon_sym_PIPE] = ACTIONS(2409),
    [anon_sym_LT] = ACTIONS(2409),
    [anon_sym_DOT_DOT] = ACTIONS(2409),
    [anon_sym_COLON_COLON] = ACTIONS(2409),
    [anon_sym_POUND] = ACTIONS(2409),
    [anon_sym_SQUOTE] = ACTIONS(2411),
    [anon_sym_async] = ACTIONS(2411),
    [anon_sym_break] = ACTIONS(2411),
    [anon_sym_const] = ACTIONS(2411),
    [anon_sym_continue] = ACTIONS(2411),
    [anon_sym_default] = ACTIONS(2411),
    [anon_sym_enum] = ACTIONS(2411),
    [anon_sym_fn] = ACTIONS(2411),
    [anon_sym_for] = ACTIONS(2411),
    [anon_sym_gen] = ACTIONS(2411),
    [anon_sym_if] = ACTIONS(2411),
    [anon_sym_impl] = ACTIONS(2411),
    [anon_sym_let] = ACTIONS(2411),
    [anon_sym_loop] = ACTIONS(2411),
    [anon_sym_match] = ACTIONS(2411),
    [anon_sym_mod] = ACTIONS(2411),
    [anon_sym_pub] = ACTIONS(2411),
    [anon_sym_return] = ACTIONS(2411),
    [anon_sym_static] = ACTIONS(2411),
    [anon_sym_struct] = ACTIONS(2411),
    [anon_sym_trait] = ACTIONS(2411),
    [anon_sym_type] = ACTIONS(2411),
    [anon_sym_union] = ACTIONS(2411),
    [anon_sym_unsafe] = ACTIONS(2411),
    [anon_sym_use] = ACTIONS(2411),
    [anon_sym_while] = ACTIONS(2411),
    [anon_sym_extern] = ACTIONS(2411),
    [anon_sym_yield] = ACTIONS(2411),
    [anon_sym_move] = ACTIONS(2411),
    [anon_sym_try] = ACTIONS(2411),
    [sym_integer_literal] = ACTIONS(2409),
    [aux_sym_string_literal_token1] = ACTIONS(2409),
    [sym_char_literal] = ACTIONS(2409),
    [anon_sym_true] = ACTIONS(2411),
    [anon_sym_false] = ACTIONS(2411),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2411),
    [sym_super] = ACTIONS(2411),
    [sym_crate] = ACTIONS(2411),
    [sym_metavariable] = ACTIONS(2409),
    [sym__raw_string_literal_start] = ACTIONS(2409),
    [sym_float_literal] = ACTIONS(2409),
  },
  [646] = {
    [sym_line_comment] = STATE(646),
    [sym_block_comment] = STATE(646),
    [ts_builtin_sym_end] = ACTIONS(1257),
    [sym_identifier] = ACTIONS(1259),
    [anon_sym_SEMI] = ACTIONS(1257),
    [anon_sym_macro_rules_BANG] = ACTIONS(1257),
    [anon_sym_LPAREN] = ACTIONS(1257),
    [anon_sym_LBRACK] = ACTIONS(1257),
    [anon_sym_LBRACE] = ACTIONS(1257),
    [anon_sym_RBRACE] = ACTIONS(1257),
    [anon_sym_STAR] = ACTIONS(1257),
    [anon_sym_u8] = ACTIONS(1259),
    [anon_sym_i8] = ACTIONS(1259),
    [anon_sym_u16] = ACTIONS(1259),
    [anon_sym_i16] = ACTIONS(1259),
    [anon_sym_u32] = ACTIONS(1259),
    [anon_sym_i32] = ACTIONS(1259),
    [anon_sym_u64] = ACTIONS(1259),
    [anon_sym_i64] = ACTIONS(1259),
    [anon_sym_u128] = ACTIONS(1259),
    [anon_sym_i128] = ACTIONS(1259),
    [anon_sym_isize] = ACTIONS(1259),
    [anon_sym_usize] = ACTIONS(1259),
    [anon_sym_f32] = ACTIONS(1259),
    [anon_sym_f64] = ACTIONS(1259),
    [anon_sym_bool] = ACTIONS(1259),
    [anon_sym_str] = ACTIONS(1259),
    [anon_sym_char] = ACTIONS(1259),
    [anon_sym_DASH] = ACTIONS(1257),
    [anon_sym_BANG] = ACTIONS(1257),
    [anon_sym_AMP] = ACTIONS(1257),
    [anon_sym_PIPE] = ACTIONS(1257),
    [anon_sym_LT] = ACTIONS(1257),
    [anon_sym_DOT_DOT] = ACTIONS(1257),
    [anon_sym_COLON_COLON] = ACTIONS(1257),
    [anon_sym_POUND] = ACTIONS(1257),
    [anon_sym_SQUOTE] = ACTIONS(1259),
    [anon_sym_async] = ACTIONS(1259),
    [anon_sym_break] = ACTIONS(1259),
    [anon_sym_const] = ACTIONS(1259),
    [anon_sym_continue] = ACTIONS(1259),
    [anon_sym_default] = ACTIONS(1259),
    [anon_sym_enum] = ACTIONS(1259),
    [anon_sym_fn] = ACTIONS(1259),
    [anon_sym_for] = ACTIONS(1259),
    [anon_sym_gen] = ACTIONS(1259),
    [anon_sym_if] = ACTIONS(1259),
    [anon_sym_impl] = ACTIONS(1259),
    [anon_sym_let] = ACTIONS(1259),
    [anon_sym_loop] = ACTIONS(1259),
    [anon_sym_match] = ACTIONS(1259),
    [anon_sym_mod] = ACTIONS(1259),
    [anon_sym_pub] = ACTIONS(1259),
    [anon_sym_return] = ACTIONS(1259),
    [anon_sym_static] = ACTIONS(1259),
    [anon_sym_struct] = ACTIONS(1259),
    [anon_sym_trait] = ACTIONS(1259),
    [anon_sym_type] = ACTIONS(1259),
    [anon_sym_union] = ACTIONS(1259),
    [anon_sym_unsafe] = ACTIONS(1259),
    [anon_sym_use] = ACTIONS(1259),
    [anon_sym_while] = ACTIONS(1259),
    [anon_sym_extern] = ACTIONS(1259),
    [anon_sym_yield] = ACTIONS(1259),
    [anon_sym_move] = ACTIONS(1259),
    [anon_sym_try] = ACTIONS(1259),
    [sym_integer_literal] = ACTIONS(1257),
    [aux_sym_string_literal_token1] = ACTIONS(1257),
    [sym_char_literal] = ACTIONS(1257),
    [anon_sym_true] = ACTIONS(1259),
    [anon_sym_false] = ACTIONS(1259),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1259),
    [sym_super] = ACTIONS(1259),
    [sym_crate] = ACTIONS(1259),
    [sym_metavariable] = ACTIONS(1257),
    [sym__raw_string_literal_start] = ACTIONS(1257),
    [sym_float_literal] = ACTIONS(1257),
  },
  [647] = {
    [sym_line_comment] = STATE(647),
    [sym_block_comment] = STATE(647),
    [ts_builtin_sym_end] = ACTIONS(2413),
    [sym_identifier] = ACTIONS(2415),
    [anon_sym_SEMI] = ACTIONS(2413),
    [anon_sym_macro_rules_BANG] = ACTIONS(2413),
    [anon_sym_LPAREN] = ACTIONS(2413),
    [anon_sym_LBRACK] = ACTIONS(2413),
    [anon_sym_LBRACE] = ACTIONS(2413),
    [anon_sym_RBRACE] = ACTIONS(2413),
    [anon_sym_STAR] = ACTIONS(2413),
    [anon_sym_u8] = ACTIONS(2415),
    [anon_sym_i8] = ACTIONS(2415),
    [anon_sym_u16] = ACTIONS(2415),
    [anon_sym_i16] = ACTIONS(2415),
    [anon_sym_u32] = ACTIONS(2415),
    [anon_sym_i32] = ACTIONS(2415),
    [anon_sym_u64] = ACTIONS(2415),
    [anon_sym_i64] = ACTIONS(2415),
    [anon_sym_u128] = ACTIONS(2415),
    [anon_sym_i128] = ACTIONS(2415),
    [anon_sym_isize] = ACTIONS(2415),
    [anon_sym_usize] = ACTIONS(2415),
    [anon_sym_f32] = ACTIONS(2415),
    [anon_sym_f64] = ACTIONS(2415),
    [anon_sym_bool] = ACTIONS(2415),
    [anon_sym_str] = ACTIONS(2415),
    [anon_sym_char] = ACTIONS(2415),
    [anon_sym_DASH] = ACTIONS(2413),
    [anon_sym_BANG] = ACTIONS(2413),
    [anon_sym_AMP] = ACTIONS(2413),
    [anon_sym_PIPE] = ACTIONS(2413),
    [anon_sym_LT] = ACTIONS(2413),
    [anon_sym_DOT_DOT] = ACTIONS(2413),
    [anon_sym_COLON_COLON] = ACTIONS(2413),
    [anon_sym_POUND] = ACTIONS(2413),
    [anon_sym_SQUOTE] = ACTIONS(2415),
    [anon_sym_async] = ACTIONS(2415),
    [anon_sym_break] = ACTIONS(2415),
    [anon_sym_const] = ACTIONS(2415),
    [anon_sym_continue] = ACTIONS(2415),
    [anon_sym_default] = ACTIONS(2415),
    [anon_sym_enum] = ACTIONS(2415),
    [anon_sym_fn] = ACTIONS(2415),
    [anon_sym_for] = ACTIONS(2415),
    [anon_sym_gen] = ACTIONS(2415),
    [anon_sym_if] = ACTIONS(2415),
    [anon_sym_impl] = ACTIONS(2415),
    [anon_sym_let] = ACTIONS(2415),
    [anon_sym_loop] = ACTIONS(2415),
    [anon_sym_match] = ACTIONS(2415),
    [anon_sym_mod] = ACTIONS(2415),
    [anon_sym_pub] = ACTIONS(2415),
    [anon_sym_return] = ACTIONS(2415),
    [anon_sym_static] = ACTIONS(2415),
    [anon_sym_struct] = ACTIONS(2415),
    [anon_sym_trait] = ACTIONS(2415),
    [anon_sym_type] = ACTIONS(2415),
    [anon_sym_union] = ACTIONS(2415),
    [anon_sym_unsafe] = ACTIONS(2415),
    [anon_sym_use] = ACTIONS(2415),
    [anon_sym_while] = ACTIONS(2415),
    [anon_sym_extern] = ACTIONS(2415),
    [anon_sym_yield] = ACTIONS(2415),
    [anon_sym_move] = ACTIONS(2415),
    [anon_sym_try] = ACTIONS(2415),
    [sym_integer_literal] = ACTIONS(2413),
    [aux_sym_string_literal_token1] = ACTIONS(2413),
    [sym_char_literal] = ACTIONS(2413),
    [anon_sym_true] = ACTIONS(2415),
    [anon_sym_false] = ACTIONS(2415),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2415),
    [sym_super] = ACTIONS(2415),
    [sym_crate] = ACTIONS(2415),
    [sym_metavariable] = ACTIONS(2413),
    [sym__raw_string_literal_start] = ACTIONS(2413),
    [sym_float_literal] = ACTIONS(2413),
  },
  [648] = {
    [sym_line_comment] = STATE(648),
    [sym_block_comment] = STATE(648),
    [ts_builtin_sym_end] = ACTIONS(2417),
    [sym_identifier] = ACTIONS(2419),
    [anon_sym_SEMI] = ACTIONS(2417),
    [anon_sym_macro_rules_BANG] = ACTIONS(2417),
    [anon_sym_LPAREN] = ACTIONS(2417),
    [anon_sym_LBRACK] = ACTIONS(2417),
    [anon_sym_LBRACE] = ACTIONS(2417),
    [anon_sym_RBRACE] = ACTIONS(2417),
    [anon_sym_STAR] = ACTIONS(2417),
    [anon_sym_u8] = ACTIONS(2419),
    [anon_sym_i8] = ACTIONS(2419),
    [anon_sym_u16] = ACTIONS(2419),
    [anon_sym_i16] = ACTIONS(2419),
    [anon_sym_u32] = ACTIONS(2419),
    [anon_sym_i32] = ACTIONS(2419),
    [anon_sym_u64] = ACTIONS(2419),
    [anon_sym_i64] = ACTIONS(2419),
    [anon_sym_u128] = ACTIONS(2419),
    [anon_sym_i128] = ACTIONS(2419),
    [anon_sym_isize] = ACTIONS(2419),
    [anon_sym_usize] = ACTIONS(2419),
    [anon_sym_f32] = ACTIONS(2419),
    [anon_sym_f64] = ACTIONS(2419),
    [anon_sym_bool] = ACTIONS(2419),
    [anon_sym_str] = ACTIONS(2419),
    [anon_sym_char] = ACTIONS(2419),
    [anon_sym_DASH] = ACTIONS(2417),
    [anon_sym_BANG] = ACTIONS(2417),
    [anon_sym_AMP] = ACTIONS(2417),
    [anon_sym_PIPE] = ACTIONS(2417),
    [anon_sym_LT] = ACTIONS(2417),
    [anon_sym_DOT_DOT] = ACTIONS(2417),
    [anon_sym_COLON_COLON] = ACTIONS(2417),
    [anon_sym_POUND] = ACTIONS(2417),
    [anon_sym_SQUOTE] = ACTIONS(2419),
    [anon_sym_async] = ACTIONS(2419),
    [anon_sym_break] = ACTIONS(2419),
    [anon_sym_const] = ACTIONS(2419),
    [anon_sym_continue] = ACTIONS(2419),
    [anon_sym_default] = ACTIONS(2419),
    [anon_sym_enum] = ACTIONS(2419),
    [anon_sym_fn] = ACTIONS(2419),
    [anon_sym_for] = ACTIONS(2419),
    [anon_sym_gen] = ACTIONS(2419),
    [anon_sym_if] = ACTIONS(2419),
    [anon_sym_impl] = ACTIONS(2419),
    [anon_sym_let] = ACTIONS(2419),
    [anon_sym_loop] = ACTIONS(2419),
    [anon_sym_match] = ACTIONS(2419),
    [anon_sym_mod] = ACTIONS(2419),
    [anon_sym_pub] = ACTIONS(2419),
    [anon_sym_return] = ACTIONS(2419),
    [anon_sym_static] = ACTIONS(2419),
    [anon_sym_struct] = ACTIONS(2419),
    [anon_sym_trait] = ACTIONS(2419),
    [anon_sym_type] = ACTIONS(2419),
    [anon_sym_union] = ACTIONS(2419),
    [anon_sym_unsafe] = ACTIONS(2419),
    [anon_sym_use] = ACTIONS(2419),
    [anon_sym_while] = ACTIONS(2419),
    [anon_sym_extern] = ACTIONS(2419),
    [anon_sym_yield] = ACTIONS(2419),
    [anon_sym_move] = ACTIONS(2419),
    [anon_sym_try] = ACTIONS(2419),
    [sym_integer_literal] = ACTIONS(2417),
    [aux_sym_string_literal_token1] = ACTIONS(2417),
    [sym_char_literal] = ACTIONS(2417),
    [anon_sym_true] = ACTIONS(2419),
    [anon_sym_false] = ACTIONS(2419),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2419),
    [sym_super] = ACTIONS(2419),
    [sym_crate] = ACTIONS(2419),
    [sym_metavariable] = ACTIONS(2417),
    [sym__raw_string_literal_start] = ACTIONS(2417),
    [sym_float_literal] = ACTIONS(2417),
  },
  [649] = {
    [sym_line_comment] = STATE(649),
    [sym_block_comment] = STATE(649),
    [ts_builtin_sym_end] = ACTIONS(1269),
    [sym_identifier] = ACTIONS(1271),
    [anon_sym_SEMI] = ACTIONS(1269),
    [anon_sym_macro_rules_BANG] = ACTIONS(1269),
    [anon_sym_LPAREN] = ACTIONS(1269),
    [anon_sym_LBRACK] = ACTIONS(1269),
    [anon_sym_LBRACE] = ACTIONS(1269),
    [anon_sym_RBRACE] = ACTIONS(1269),
    [anon_sym_STAR] = ACTIONS(1269),
    [anon_sym_u8] = ACTIONS(1271),
    [anon_sym_i8] = ACTIONS(1271),
    [anon_sym_u16] = ACTIONS(1271),
    [anon_sym_i16] = ACTIONS(1271),
    [anon_sym_u32] = ACTIONS(1271),
    [anon_sym_i32] = ACTIONS(1271),
    [anon_sym_u64] = ACTIONS(1271),
    [anon_sym_i64] = ACTIONS(1271),
    [anon_sym_u128] = ACTIONS(1271),
    [anon_sym_i128] = ACTIONS(1271),
    [anon_sym_isize] = ACTIONS(1271),
    [anon_sym_usize] = ACTIONS(1271),
    [anon_sym_f32] = ACTIONS(1271),
    [anon_sym_f64] = ACTIONS(1271),
    [anon_sym_bool] = ACTIONS(1271),
    [anon_sym_str] = ACTIONS(1271),
    [anon_sym_char] = ACTIONS(1271),
    [anon_sym_DASH] = ACTIONS(1269),
    [anon_sym_BANG] = ACTIONS(1269),
    [anon_sym_AMP] = ACTIONS(1269),
    [anon_sym_PIPE] = ACTIONS(1269),
    [anon_sym_LT] = ACTIONS(1269),
    [anon_sym_DOT_DOT] = ACTIONS(1269),
    [anon_sym_COLON_COLON] = ACTIONS(1269),
    [anon_sym_POUND] = ACTIONS(1269),
    [anon_sym_SQUOTE] = ACTIONS(1271),
    [anon_sym_async] = ACTIONS(1271),
    [anon_sym_break] = ACTIONS(1271),
    [anon_sym_const] = ACTIONS(1271),
    [anon_sym_continue] = ACTIONS(1271),
    [anon_sym_default] = ACTIONS(1271),
    [anon_sym_enum] = ACTIONS(1271),
    [anon_sym_fn] = ACTIONS(1271),
    [anon_sym_for] = ACTIONS(1271),
    [anon_sym_gen] = ACTIONS(1271),
    [anon_sym_if] = ACTIONS(1271),
    [anon_sym_impl] = ACTIONS(1271),
    [anon_sym_let] = ACTIONS(1271),
    [anon_sym_loop] = ACTIONS(1271),
    [anon_sym_match] = ACTIONS(1271),
    [anon_sym_mod] = ACTIONS(1271),
    [anon_sym_pub] = ACTIONS(1271),
    [anon_sym_return] = ACTIONS(1271),
    [anon_sym_static] = ACTIONS(1271),
    [anon_sym_struct] = ACTIONS(1271),
    [anon_sym_trait] = ACTIONS(1271),
    [anon_sym_type] = ACTIONS(1271),
    [anon_sym_union] = ACTIONS(1271),
    [anon_sym_unsafe] = ACTIONS(1271),
    [anon_sym_use] = ACTIONS(1271),
    [anon_sym_while] = ACTIONS(1271),
    [anon_sym_extern] = ACTIONS(1271),
    [anon_sym_yield] = ACTIONS(1271),
    [anon_sym_move] = ACTIONS(1271),
    [anon_sym_try] = ACTIONS(1271),
    [sym_integer_literal] = ACTIONS(1269),
    [aux_sym_string_literal_token1] = ACTIONS(1269),
    [sym_char_literal] = ACTIONS(1269),
    [anon_sym_true] = ACTIONS(1271),
    [anon_sym_false] = ACTIONS(1271),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1271),
    [sym_super] = ACTIONS(1271),
    [sym_crate] = ACTIONS(1271),
    [sym_metavariable] = ACTIONS(1269),
    [sym__raw_string_literal_start] = ACTIONS(1269),
    [sym_float_literal] = ACTIONS(1269),
  },
  [650] = {
    [sym_line_comment] = STATE(650),
    [sym_block_comment] = STATE(650),
    [ts_builtin_sym_end] = ACTIONS(1265),
    [sym_identifier] = ACTIONS(1267),
    [anon_sym_SEMI] = ACTIONS(1265),
    [anon_sym_macro_rules_BANG] = ACTIONS(1265),
    [anon_sym_LPAREN] = ACTIONS(1265),
    [anon_sym_LBRACK] = ACTIONS(1265),
    [anon_sym_LBRACE] = ACTIONS(1265),
    [anon_sym_RBRACE] = ACTIONS(1265),
    [anon_sym_STAR] = ACTIONS(1265),
    [anon_sym_u8] = ACTIONS(1267),
    [anon_sym_i8] = ACTIONS(1267),
    [anon_sym_u16] = ACTIONS(1267),
    [anon_sym_i16] = ACTIONS(1267),
    [anon_sym_u32] = ACTIONS(1267),
    [anon_sym_i32] = ACTIONS(1267),
    [anon_sym_u64] = ACTIONS(1267),
    [anon_sym_i64] = ACTIONS(1267),
    [anon_sym_u128] = ACTIONS(1267),
    [anon_sym_i128] = ACTIONS(1267),
    [anon_sym_isize] = ACTIONS(1267),
    [anon_sym_usize] = ACTIONS(1267),
    [anon_sym_f32] = ACTIONS(1267),
    [anon_sym_f64] = ACTIONS(1267),
    [anon_sym_bool] = ACTIONS(1267),
    [anon_sym_str] = ACTIONS(1267),
    [anon_sym_char] = ACTIONS(1267),
    [anon_sym_DASH] = ACTIONS(1265),
    [anon_sym_BANG] = ACTIONS(1265),
    [anon_sym_AMP] = ACTIONS(1265),
    [anon_sym_PIPE] = ACTIONS(1265),
    [anon_sym_LT] = ACTIONS(1265),
    [anon_sym_DOT_DOT] = ACTIONS(1265),
    [anon_sym_COLON_COLON] = ACTIONS(1265),
    [anon_sym_POUND] = ACTIONS(1265),
    [anon_sym_SQUOTE] = ACTIONS(1267),
    [anon_sym_async] = ACTIONS(1267),
    [anon_sym_break] = ACTIONS(1267),
    [anon_sym_const] = ACTIONS(1267),
    [anon_sym_continue] = ACTIONS(1267),
    [anon_sym_default] = ACTIONS(1267),
    [anon_sym_enum] = ACTIONS(1267),
    [anon_sym_fn] = ACTIONS(1267),
    [anon_sym_for] = ACTIONS(1267),
    [anon_sym_gen] = ACTIONS(1267),
    [anon_sym_if] = ACTIONS(1267),
    [anon_sym_impl] = ACTIONS(1267),
    [anon_sym_let] = ACTIONS(1267),
    [anon_sym_loop] = ACTIONS(1267),
    [anon_sym_match] = ACTIONS(1267),
    [anon_sym_mod] = ACTIONS(1267),
    [anon_sym_pub] = ACTIONS(1267),
    [anon_sym_return] = ACTIONS(1267),
    [anon_sym_static] = ACTIONS(1267),
    [anon_sym_struct] = ACTIONS(1267),
    [anon_sym_trait] = ACTIONS(1267),
    [anon_sym_type] = ACTIONS(1267),
    [anon_sym_union] = ACTIONS(1267),
    [anon_sym_unsafe] = ACTIONS(1267),
    [anon_sym_use] = ACTIONS(1267),
    [anon_sym_while] = ACTIONS(1267),
    [anon_sym_extern] = ACTIONS(1267),
    [anon_sym_yield] = ACTIONS(1267),
    [anon_sym_move] = ACTIONS(1267),
    [anon_sym_try] = ACTIONS(1267),
    [sym_integer_literal] = ACTIONS(1265),
    [aux_sym_string_literal_token1] = ACTIONS(1265),
    [sym_char_literal] = ACTIONS(1265),
    [anon_sym_true] = ACTIONS(1267),
    [anon_sym_false] = ACTIONS(1267),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1267),
    [sym_super] = ACTIONS(1267),
    [sym_crate] = ACTIONS(1267),
    [sym_metavariable] = ACTIONS(1265),
    [sym__raw_string_literal_start] = ACTIONS(1265),
    [sym_float_literal] = ACTIONS(1265),
  },
  [651] = {
    [sym_line_comment] = STATE(651),
    [sym_block_comment] = STATE(651),
    [ts_builtin_sym_end] = ACTIONS(1261),
    [sym_identifier] = ACTIONS(1263),
    [anon_sym_SEMI] = ACTIONS(1261),
    [anon_sym_macro_rules_BANG] = ACTIONS(1261),
    [anon_sym_LPAREN] = ACTIONS(1261),
    [anon_sym_LBRACK] = ACTIONS(1261),
    [anon_sym_LBRACE] = ACTIONS(1261),
    [anon_sym_RBRACE] = ACTIONS(1261),
    [anon_sym_STAR] = ACTIONS(1261),
    [anon_sym_u8] = ACTIONS(1263),
    [anon_sym_i8] = ACTIONS(1263),
    [anon_sym_u16] = ACTIONS(1263),
    [anon_sym_i16] = ACTIONS(1263),
    [anon_sym_u32] = ACTIONS(1263),
    [anon_sym_i32] = ACTIONS(1263),
    [anon_sym_u64] = ACTIONS(1263),
    [anon_sym_i64] = ACTIONS(1263),
    [anon_sym_u128] = ACTIONS(1263),
    [anon_sym_i128] = ACTIONS(1263),
    [anon_sym_isize] = ACTIONS(1263),
    [anon_sym_usize] = ACTIONS(1263),
    [anon_sym_f32] = ACTIONS(1263),
    [anon_sym_f64] = ACTIONS(1263),
    [anon_sym_bool] = ACTIONS(1263),
    [anon_sym_str] = ACTIONS(1263),
    [anon_sym_char] = ACTIONS(1263),
    [anon_sym_DASH] = ACTIONS(1261),
    [anon_sym_BANG] = ACTIONS(1261),
    [anon_sym_AMP] = ACTIONS(1261),
    [anon_sym_PIPE] = ACTIONS(1261),
    [anon_sym_LT] = ACTIONS(1261),
    [anon_sym_DOT_DOT] = ACTIONS(1261),
    [anon_sym_COLON_COLON] = ACTIONS(1261),
    [anon_sym_POUND] = ACTIONS(1261),
    [anon_sym_SQUOTE] = ACTIONS(1263),
    [anon_sym_async] = ACTIONS(1263),
    [anon_sym_break] = ACTIONS(1263),
    [anon_sym_const] = ACTIONS(1263),
    [anon_sym_continue] = ACTIONS(1263),
    [anon_sym_default] = ACTIONS(1263),
    [anon_sym_enum] = ACTIONS(1263),
    [anon_sym_fn] = ACTIONS(1263),
    [anon_sym_for] = ACTIONS(1263),
    [anon_sym_gen] = ACTIONS(1263),
    [anon_sym_if] = ACTIONS(1263),
    [anon_sym_impl] = ACTIONS(1263),
    [anon_sym_let] = ACTIONS(1263),
    [anon_sym_loop] = ACTIONS(1263),
    [anon_sym_match] = ACTIONS(1263),
    [anon_sym_mod] = ACTIONS(1263),
    [anon_sym_pub] = ACTIONS(1263),
    [anon_sym_return] = ACTIONS(1263),
    [anon_sym_static] = ACTIONS(1263),
    [anon_sym_struct] = ACTIONS(1263),
    [anon_sym_trait] = ACTIONS(1263),
    [anon_sym_type] = ACTIONS(1263),
    [anon_sym_union] = ACTIONS(1263),
    [anon_sym_unsafe] = ACTIONS(1263),
    [anon_sym_use] = ACTIONS(1263),
    [anon_sym_while] = ACTIONS(1263),
    [anon_sym_extern] = ACTIONS(1263),
    [anon_sym_yield] = ACTIONS(1263),
    [anon_sym_move] = ACTIONS(1263),
    [anon_sym_try] = ACTIONS(1263),
    [sym_integer_literal] = ACTIONS(1261),
    [aux_sym_string_literal_token1] = ACTIONS(1261),
    [sym_char_literal] = ACTIONS(1261),
    [anon_sym_true] = ACTIONS(1263),
    [anon_sym_false] = ACTIONS(1263),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1263),
    [sym_super] = ACTIONS(1263),
    [sym_crate] = ACTIONS(1263),
    [sym_metavariable] = ACTIONS(1261),
    [sym__raw_string_literal_start] = ACTIONS(1261),
    [sym_float_literal] = ACTIONS(1261),
  },
  [652] = {
    [sym_line_comment] = STATE(652),
    [sym_block_comment] = STATE(652),
    [ts_builtin_sym_end] = ACTIONS(2421),
    [sym_identifier] = ACTIONS(2423),
    [anon_sym_SEMI] = ACTIONS(2421),
    [anon_sym_macro_rules_BANG] = ACTIONS(2421),
    [anon_sym_LPAREN] = ACTIONS(2421),
    [anon_sym_LBRACK] = ACTIONS(2421),
    [anon_sym_LBRACE] = ACTIONS(2421),
    [anon_sym_RBRACE] = ACTIONS(2421),
    [anon_sym_STAR] = ACTIONS(2421),
    [anon_sym_u8] = ACTIONS(2423),
    [anon_sym_i8] = ACTIONS(2423),
    [anon_sym_u16] = ACTIONS(2423),
    [anon_sym_i16] = ACTIONS(2423),
    [anon_sym_u32] = ACTIONS(2423),
    [anon_sym_i32] = ACTIONS(2423),
    [anon_sym_u64] = ACTIONS(2423),
    [anon_sym_i64] = ACTIONS(2423),
    [anon_sym_u128] = ACTIONS(2423),
    [anon_sym_i128] = ACTIONS(2423),
    [anon_sym_isize] = ACTIONS(2423),
    [anon_sym_usize] = ACTIONS(2423),
    [anon_sym_f32] = ACTIONS(2423),
    [anon_sym_f64] = ACTIONS(2423),
    [anon_sym_bool] = ACTIONS(2423),
    [anon_sym_str] = ACTIONS(2423),
    [anon_sym_char] = ACTIONS(2423),
    [anon_sym_DASH] = ACTIONS(2421),
    [anon_sym_BANG] = ACTIONS(2421),
    [anon_sym_AMP] = ACTIONS(2421),
    [anon_sym_PIPE] = ACTIONS(2421),
    [anon_sym_LT] = ACTIONS(2421),
    [anon_sym_DOT_DOT] = ACTIONS(2421),
    [anon_sym_COLON_COLON] = ACTIONS(2421),
    [anon_sym_POUND] = ACTIONS(2421),
    [anon_sym_SQUOTE] = ACTIONS(2423),
    [anon_sym_async] = ACTIONS(2423),
    [anon_sym_break] = ACTIONS(2423),
    [anon_sym_const] = ACTIONS(2423),
    [anon_sym_continue] = ACTIONS(2423),
    [anon_sym_default] = ACTIONS(2423),
    [anon_sym_enum] = ACTIONS(2423),
    [anon_sym_fn] = ACTIONS(2423),
    [anon_sym_for] = ACTIONS(2423),
    [anon_sym_gen] = ACTIONS(2423),
    [anon_sym_if] = ACTIONS(2423),
    [anon_sym_impl] = ACTIONS(2423),
    [anon_sym_let] = ACTIONS(2423),
    [anon_sym_loop] = ACTIONS(2423),
    [anon_sym_match] = ACTIONS(2423),
    [anon_sym_mod] = ACTIONS(2423),
    [anon_sym_pub] = ACTIONS(2423),
    [anon_sym_return] = ACTIONS(2423),
    [anon_sym_static] = ACTIONS(2423),
    [anon_sym_struct] = ACTIONS(2423),
    [anon_sym_trait] = ACTIONS(2423),
    [anon_sym_type] = ACTIONS(2423),
    [anon_sym_union] = ACTIONS(2423),
    [anon_sym_unsafe] = ACTIONS(2423),
    [anon_sym_use] = ACTIONS(2423),
    [anon_sym_while] = ACTIONS(2423),
    [anon_sym_extern] = ACTIONS(2423),
    [anon_sym_yield] = ACTIONS(2423),
    [anon_sym_move] = ACTIONS(2423),
    [anon_sym_try] = ACTIONS(2423),
    [sym_integer_literal] = ACTIONS(2421),
    [aux_sym_string_literal_token1] = ACTIONS(2421),
    [sym_char_literal] = ACTIONS(2421),
    [anon_sym_true] = ACTIONS(2423),
    [anon_sym_false] = ACTIONS(2423),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2423),
    [sym_super] = ACTIONS(2423),
    [sym_crate] = ACTIONS(2423),
    [sym_metavariable] = ACTIONS(2421),
    [sym__raw_string_literal_start] = ACTIONS(2421),
    [sym_float_literal] = ACTIONS(2421),
  },
  [653] = {
    [sym_line_comment] = STATE(653),
    [sym_block_comment] = STATE(653),
    [ts_builtin_sym_end] = ACTIONS(2425),
    [sym_identifier] = ACTIONS(2427),
    [anon_sym_SEMI] = ACTIONS(2425),
    [anon_sym_macro_rules_BANG] = ACTIONS(2425),
    [anon_sym_LPAREN] = ACTIONS(2425),
    [anon_sym_LBRACK] = ACTIONS(2425),
    [anon_sym_LBRACE] = ACTIONS(2425),
    [anon_sym_RBRACE] = ACTIONS(2425),
    [anon_sym_STAR] = ACTIONS(2425),
    [anon_sym_u8] = ACTIONS(2427),
    [anon_sym_i8] = ACTIONS(2427),
    [anon_sym_u16] = ACTIONS(2427),
    [anon_sym_i16] = ACTIONS(2427),
    [anon_sym_u32] = ACTIONS(2427),
    [anon_sym_i32] = ACTIONS(2427),
    [anon_sym_u64] = ACTIONS(2427),
    [anon_sym_i64] = ACTIONS(2427),
    [anon_sym_u128] = ACTIONS(2427),
    [anon_sym_i128] = ACTIONS(2427),
    [anon_sym_isize] = ACTIONS(2427),
    [anon_sym_usize] = ACTIONS(2427),
    [anon_sym_f32] = ACTIONS(2427),
    [anon_sym_f64] = ACTIONS(2427),
    [anon_sym_bool] = ACTIONS(2427),
    [anon_sym_str] = ACTIONS(2427),
    [anon_sym_char] = ACTIONS(2427),
    [anon_sym_DASH] = ACTIONS(2425),
    [anon_sym_BANG] = ACTIONS(2425),
    [anon_sym_AMP] = ACTIONS(2425),
    [anon_sym_PIPE] = ACTIONS(2425),
    [anon_sym_LT] = ACTIONS(2425),
    [anon_sym_DOT_DOT] = ACTIONS(2425),
    [anon_sym_COLON_COLON] = ACTIONS(2425),
    [anon_sym_POUND] = ACTIONS(2425),
    [anon_sym_SQUOTE] = ACTIONS(2427),
    [anon_sym_async] = ACTIONS(2427),
    [anon_sym_break] = ACTIONS(2427),
    [anon_sym_const] = ACTIONS(2427),
    [anon_sym_continue] = ACTIONS(2427),
    [anon_sym_default] = ACTIONS(2427),
    [anon_sym_enum] = ACTIONS(2427),
    [anon_sym_fn] = ACTIONS(2427),
    [anon_sym_for] = ACTIONS(2427),
    [anon_sym_gen] = ACTIONS(2427),
    [anon_sym_if] = ACTIONS(2427),
    [anon_sym_impl] = ACTIONS(2427),
    [anon_sym_let] = ACTIONS(2427),
    [anon_sym_loop] = ACTIONS(2427),
    [anon_sym_match] = ACTIONS(2427),
    [anon_sym_mod] = ACTIONS(2427),
    [anon_sym_pub] = ACTIONS(2427),
    [anon_sym_return] = ACTIONS(2427),
    [anon_sym_static] = ACTIONS(2427),
    [anon_sym_struct] = ACTIONS(2427),
    [anon_sym_trait] = ACTIONS(2427),
    [anon_sym_type] = ACTIONS(2427),
    [anon_sym_union] = ACTIONS(2427),
    [anon_sym_unsafe] = ACTIONS(2427),
    [anon_sym_use] = ACTIONS(2427),
    [anon_sym_while] = ACTIONS(2427),
    [anon_sym_extern] = ACTIONS(2427),
    [anon_sym_yield] = ACTIONS(2427),
    [anon_sym_move] = ACTIONS(2427),
    [anon_sym_try] = ACTIONS(2427),
    [sym_integer_literal] = ACTIONS(2425),
    [aux_sym_string_literal_token1] = ACTIONS(2425),
    [sym_char_literal] = ACTIONS(2425),
    [anon_sym_true] = ACTIONS(2427),
    [anon_sym_false] = ACTIONS(2427),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2427),
    [sym_super] = ACTIONS(2427),
    [sym_crate] = ACTIONS(2427),
    [sym_metavariable] = ACTIONS(2425),
    [sym__raw_string_literal_start] = ACTIONS(2425),
    [sym_float_literal] = ACTIONS(2425),
  },
  [654] = {
    [sym_line_comment] = STATE(654),
    [sym_block_comment] = STATE(654),
    [ts_builtin_sym_end] = ACTIONS(2429),
    [sym_identifier] = ACTIONS(2431),
    [anon_sym_SEMI] = ACTIONS(2429),
    [anon_sym_macro_rules_BANG] = ACTIONS(2429),
    [anon_sym_LPAREN] = ACTIONS(2429),
    [anon_sym_LBRACK] = ACTIONS(2429),
    [anon_sym_LBRACE] = ACTIONS(2429),
    [anon_sym_RBRACE] = ACTIONS(2429),
    [anon_sym_STAR] = ACTIONS(2429),
    [anon_sym_u8] = ACTIONS(2431),
    [anon_sym_i8] = ACTIONS(2431),
    [anon_sym_u16] = ACTIONS(2431),
    [anon_sym_i16] = ACTIONS(2431),
    [anon_sym_u32] = ACTIONS(2431),
    [anon_sym_i32] = ACTIONS(2431),
    [anon_sym_u64] = ACTIONS(2431),
    [anon_sym_i64] = ACTIONS(2431),
    [anon_sym_u128] = ACTIONS(2431),
    [anon_sym_i128] = ACTIONS(2431),
    [anon_sym_isize] = ACTIONS(2431),
    [anon_sym_usize] = ACTIONS(2431),
    [anon_sym_f32] = ACTIONS(2431),
    [anon_sym_f64] = ACTIONS(2431),
    [anon_sym_bool] = ACTIONS(2431),
    [anon_sym_str] = ACTIONS(2431),
    [anon_sym_char] = ACTIONS(2431),
    [anon_sym_DASH] = ACTIONS(2429),
    [anon_sym_BANG] = ACTIONS(2429),
    [anon_sym_AMP] = ACTIONS(2429),
    [anon_sym_PIPE] = ACTIONS(2429),
    [anon_sym_LT] = ACTIONS(2429),
    [anon_sym_DOT_DOT] = ACTIONS(2429),
    [anon_sym_COLON_COLON] = ACTIONS(2429),
    [anon_sym_POUND] = ACTIONS(2429),
    [anon_sym_SQUOTE] = ACTIONS(2431),
    [anon_sym_async] = ACTIONS(2431),
    [anon_sym_break] = ACTIONS(2431),
    [anon_sym_const] = ACTIONS(2431),
    [anon_sym_continue] = ACTIONS(2431),
    [anon_sym_default] = ACTIONS(2431),
    [anon_sym_enum] = ACTIONS(2431),
    [anon_sym_fn] = ACTIONS(2431),
    [anon_sym_for] = ACTIONS(2431),
    [anon_sym_gen] = ACTIONS(2431),
    [anon_sym_if] = ACTIONS(2431),
    [anon_sym_impl] = ACTIONS(2431),
    [anon_sym_let] = ACTIONS(2431),
    [anon_sym_loop] = ACTIONS(2431),
    [anon_sym_match] = ACTIONS(2431),
    [anon_sym_mod] = ACTIONS(2431),
    [anon_sym_pub] = ACTIONS(2431),
    [anon_sym_return] = ACTIONS(2431),
    [anon_sym_static] = ACTIONS(2431),
    [anon_sym_struct] = ACTIONS(2431),
    [anon_sym_trait] = ACTIONS(2431),
    [anon_sym_type] = ACTIONS(2431),
    [anon_sym_union] = ACTIONS(2431),
    [anon_sym_unsafe] = ACTIONS(2431),
    [anon_sym_use] = ACTIONS(2431),
    [anon_sym_while] = ACTIONS(2431),
    [anon_sym_extern] = ACTIONS(2431),
    [anon_sym_yield] = ACTIONS(2431),
    [anon_sym_move] = ACTIONS(2431),
    [anon_sym_try] = ACTIONS(2431),
    [sym_integer_literal] = ACTIONS(2429),
    [aux_sym_string_literal_token1] = ACTIONS(2429),
    [sym_char_literal] = ACTIONS(2429),
    [anon_sym_true] = ACTIONS(2431),
    [anon_sym_false] = ACTIONS(2431),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2431),
    [sym_super] = ACTIONS(2431),
    [sym_crate] = ACTIONS(2431),
    [sym_metavariable] = ACTIONS(2429),
    [sym__raw_string_literal_start] = ACTIONS(2429),
    [sym_float_literal] = ACTIONS(2429),
  },
  [655] = {
    [sym_line_comment] = STATE(655),
    [sym_block_comment] = STATE(655),
    [ts_builtin_sym_end] = ACTIONS(2433),
    [sym_identifier] = ACTIONS(2435),
    [anon_sym_SEMI] = ACTIONS(2433),
    [anon_sym_macro_rules_BANG] = ACTIONS(2433),
    [anon_sym_LPAREN] = ACTIONS(2433),
    [anon_sym_LBRACK] = ACTIONS(2433),
    [anon_sym_LBRACE] = ACTIONS(2433),
    [anon_sym_RBRACE] = ACTIONS(2433),
    [anon_sym_STAR] = ACTIONS(2433),
    [anon_sym_u8] = ACTIONS(2435),
    [anon_sym_i8] = ACTIONS(2435),
    [anon_sym_u16] = ACTIONS(2435),
    [anon_sym_i16] = ACTIONS(2435),
    [anon_sym_u32] = ACTIONS(2435),
    [anon_sym_i32] = ACTIONS(2435),
    [anon_sym_u64] = ACTIONS(2435),
    [anon_sym_i64] = ACTIONS(2435),
    [anon_sym_u128] = ACTIONS(2435),
    [anon_sym_i128] = ACTIONS(2435),
    [anon_sym_isize] = ACTIONS(2435),
    [anon_sym_usize] = ACTIONS(2435),
    [anon_sym_f32] = ACTIONS(2435),
    [anon_sym_f64] = ACTIONS(2435),
    [anon_sym_bool] = ACTIONS(2435),
    [anon_sym_str] = ACTIONS(2435),
    [anon_sym_char] = ACTIONS(2435),
    [anon_sym_DASH] = ACTIONS(2433),
    [anon_sym_BANG] = ACTIONS(2433),
    [anon_sym_AMP] = ACTIONS(2433),
    [anon_sym_PIPE] = ACTIONS(2433),
    [anon_sym_LT] = ACTIONS(2433),
    [anon_sym_DOT_DOT] = ACTIONS(2433),
    [anon_sym_COLON_COLON] = ACTIONS(2433),
    [anon_sym_POUND] = ACTIONS(2433),
    [anon_sym_SQUOTE] = ACTIONS(2435),
    [anon_sym_async] = ACTIONS(2435),
    [anon_sym_break] = ACTIONS(2435),
    [anon_sym_const] = ACTIONS(2435),
    [anon_sym_continue] = ACTIONS(2435),
    [anon_sym_default] = ACTIONS(2435),
    [anon_sym_enum] = ACTIONS(2435),
    [anon_sym_fn] = ACTIONS(2435),
    [anon_sym_for] = ACTIONS(2435),
    [anon_sym_gen] = ACTIONS(2435),
    [anon_sym_if] = ACTIONS(2435),
    [anon_sym_impl] = ACTIONS(2435),
    [anon_sym_let] = ACTIONS(2435),
    [anon_sym_loop] = ACTIONS(2435),
    [anon_sym_match] = ACTIONS(2435),
    [anon_sym_mod] = ACTIONS(2435),
    [anon_sym_pub] = ACTIONS(2435),
    [anon_sym_return] = ACTIONS(2435),
    [anon_sym_static] = ACTIONS(2435),
    [anon_sym_struct] = ACTIONS(2435),
    [anon_sym_trait] = ACTIONS(2435),
    [anon_sym_type] = ACTIONS(2435),
    [anon_sym_union] = ACTIONS(2435),
    [anon_sym_unsafe] = ACTIONS(2435),
    [anon_sym_use] = ACTIONS(2435),
    [anon_sym_while] = ACTIONS(2435),
    [anon_sym_extern] = ACTIONS(2435),
    [anon_sym_yield] = ACTIONS(2435),
    [anon_sym_move] = ACTIONS(2435),
    [anon_sym_try] = ACTIONS(2435),
    [sym_integer_literal] = ACTIONS(2433),
    [aux_sym_string_literal_token1] = ACTIONS(2433),
    [sym_char_literal] = ACTIONS(2433),
    [anon_sym_true] = ACTIONS(2435),
    [anon_sym_false] = ACTIONS(2435),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2435),
    [sym_super] = ACTIONS(2435),
    [sym_crate] = ACTIONS(2435),
    [sym_metavariable] = ACTIONS(2433),
    [sym__raw_string_literal_start] = ACTIONS(2433),
    [sym_float_literal] = ACTIONS(2433),
  },
  [656] = {
    [sym_line_comment] = STATE(656),
    [sym_block_comment] = STATE(656),
    [ts_builtin_sym_end] = ACTIONS(2437),
    [sym_identifier] = ACTIONS(2439),
    [anon_sym_SEMI] = ACTIONS(2437),
    [anon_sym_macro_rules_BANG] = ACTIONS(2437),
    [anon_sym_LPAREN] = ACTIONS(2437),
    [anon_sym_LBRACK] = ACTIONS(2437),
    [anon_sym_LBRACE] = ACTIONS(2437),
    [anon_sym_RBRACE] = ACTIONS(2437),
    [anon_sym_STAR] = ACTIONS(2437),
    [anon_sym_u8] = ACTIONS(2439),
    [anon_sym_i8] = ACTIONS(2439),
    [anon_sym_u16] = ACTIONS(2439),
    [anon_sym_i16] = ACTIONS(2439),
    [anon_sym_u32] = ACTIONS(2439),
    [anon_sym_i32] = ACTIONS(2439),
    [anon_sym_u64] = ACTIONS(2439),
    [anon_sym_i64] = ACTIONS(2439),
    [anon_sym_u128] = ACTIONS(2439),
    [anon_sym_i128] = ACTIONS(2439),
    [anon_sym_isize] = ACTIONS(2439),
    [anon_sym_usize] = ACTIONS(2439),
    [anon_sym_f32] = ACTIONS(2439),
    [anon_sym_f64] = ACTIONS(2439),
    [anon_sym_bool] = ACTIONS(2439),
    [anon_sym_str] = ACTIONS(2439),
    [anon_sym_char] = ACTIONS(2439),
    [anon_sym_DASH] = ACTIONS(2437),
    [anon_sym_BANG] = ACTIONS(2437),
    [anon_sym_AMP] = ACTIONS(2437),
    [anon_sym_PIPE] = ACTIONS(2437),
    [anon_sym_LT] = ACTIONS(2437),
    [anon_sym_DOT_DOT] = ACTIONS(2437),
    [anon_sym_COLON_COLON] = ACTIONS(2437),
    [anon_sym_POUND] = ACTIONS(2437),
    [anon_sym_SQUOTE] = ACTIONS(2439),
    [anon_sym_async] = ACTIONS(2439),
    [anon_sym_break] = ACTIONS(2439),
    [anon_sym_const] = ACTIONS(2439),
    [anon_sym_continue] = ACTIONS(2439),
    [anon_sym_default] = ACTIONS(2439),
    [anon_sym_enum] = ACTIONS(2439),
    [anon_sym_fn] = ACTIONS(2439),
    [anon_sym_for] = ACTIONS(2439),
    [anon_sym_gen] = ACTIONS(2439),
    [anon_sym_if] = ACTIONS(2439),
    [anon_sym_impl] = ACTIONS(2439),
    [anon_sym_let] = ACTIONS(2439),
    [anon_sym_loop] = ACTIONS(2439),
    [anon_sym_match] = ACTIONS(2439),
    [anon_sym_mod] = ACTIONS(2439),
    [anon_sym_pub] = ACTIONS(2439),
    [anon_sym_return] = ACTIONS(2439),
    [anon_sym_static] = ACTIONS(2439),
    [anon_sym_struct] = ACTIONS(2439),
    [anon_sym_trait] = ACTIONS(2439),
    [anon_sym_type] = ACTIONS(2439),
    [anon_sym_union] = ACTIONS(2439),
    [anon_sym_unsafe] = ACTIONS(2439),
    [anon_sym_use] = ACTIONS(2439),
    [anon_sym_while] = ACTIONS(2439),
    [anon_sym_extern] = ACTIONS(2439),
    [anon_sym_yield] = ACTIONS(2439),
    [anon_sym_move] = ACTIONS(2439),
    [anon_sym_try] = ACTIONS(2439),
    [sym_integer_literal] = ACTIONS(2437),
    [aux_sym_string_literal_token1] = ACTIONS(2437),
    [sym_char_literal] = ACTIONS(2437),
    [anon_sym_true] = ACTIONS(2439),
    [anon_sym_false] = ACTIONS(2439),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2439),
    [sym_super] = ACTIONS(2439),
    [sym_crate] = ACTIONS(2439),
    [sym_metavariable] = ACTIONS(2437),
    [sym__raw_string_literal_start] = ACTIONS(2437),
    [sym_float_literal] = ACTIONS(2437),
  },
  [657] = {
    [sym_line_comment] = STATE(657),
    [sym_block_comment] = STATE(657),
    [ts_builtin_sym_end] = ACTIONS(2441),
    [sym_identifier] = ACTIONS(2443),
    [anon_sym_SEMI] = ACTIONS(2441),
    [anon_sym_macro_rules_BANG] = ACTIONS(2441),
    [anon_sym_LPAREN] = ACTIONS(2441),
    [anon_sym_LBRACK] = ACTIONS(2441),
    [anon_sym_LBRACE] = ACTIONS(2441),
    [anon_sym_RBRACE] = ACTIONS(2441),
    [anon_sym_STAR] = ACTIONS(2441),
    [anon_sym_u8] = ACTIONS(2443),
    [anon_sym_i8] = ACTIONS(2443),
    [anon_sym_u16] = ACTIONS(2443),
    [anon_sym_i16] = ACTIONS(2443),
    [anon_sym_u32] = ACTIONS(2443),
    [anon_sym_i32] = ACTIONS(2443),
    [anon_sym_u64] = ACTIONS(2443),
    [anon_sym_i64] = ACTIONS(2443),
    [anon_sym_u128] = ACTIONS(2443),
    [anon_sym_i128] = ACTIONS(2443),
    [anon_sym_isize] = ACTIONS(2443),
    [anon_sym_usize] = ACTIONS(2443),
    [anon_sym_f32] = ACTIONS(2443),
    [anon_sym_f64] = ACTIONS(2443),
    [anon_sym_bool] = ACTIONS(2443),
    [anon_sym_str] = ACTIONS(2443),
    [anon_sym_char] = ACTIONS(2443),
    [anon_sym_DASH] = ACTIONS(2441),
    [anon_sym_BANG] = ACTIONS(2441),
    [anon_sym_AMP] = ACTIONS(2441),
    [anon_sym_PIPE] = ACTIONS(2441),
    [anon_sym_LT] = ACTIONS(2441),
    [anon_sym_DOT_DOT] = ACTIONS(2441),
    [anon_sym_COLON_COLON] = ACTIONS(2441),
    [anon_sym_POUND] = ACTIONS(2441),
    [anon_sym_SQUOTE] = ACTIONS(2443),
    [anon_sym_async] = ACTIONS(2443),
    [anon_sym_break] = ACTIONS(2443),
    [anon_sym_const] = ACTIONS(2443),
    [anon_sym_continue] = ACTIONS(2443),
    [anon_sym_default] = ACTIONS(2443),
    [anon_sym_enum] = ACTIONS(2443),
    [anon_sym_fn] = ACTIONS(2443),
    [anon_sym_for] = ACTIONS(2443),
    [anon_sym_gen] = ACTIONS(2443),
    [anon_sym_if] = ACTIONS(2443),
    [anon_sym_impl] = ACTIONS(2443),
    [anon_sym_let] = ACTIONS(2443),
    [anon_sym_loop] = ACTIONS(2443),
    [anon_sym_match] = ACTIONS(2443),
    [anon_sym_mod] = ACTIONS(2443),
    [anon_sym_pub] = ACTIONS(2443),
    [anon_sym_return] = ACTIONS(2443),
    [anon_sym_static] = ACTIONS(2443),
    [anon_sym_struct] = ACTIONS(2443),
    [anon_sym_trait] = ACTIONS(2443),
    [anon_sym_type] = ACTIONS(2443),
    [anon_sym_union] = ACTIONS(2443),
    [anon_sym_unsafe] = ACTIONS(2443),
    [anon_sym_use] = ACTIONS(2443),
    [anon_sym_while] = ACTIONS(2443),
    [anon_sym_extern] = ACTIONS(2443),
    [anon_sym_yield] = ACTIONS(2443),
    [anon_sym_move] = ACTIONS(2443),
    [anon_sym_try] = ACTIONS(2443),
    [sym_integer_literal] = ACTIONS(2441),
    [aux_sym_string_literal_token1] = ACTIONS(2441),
    [sym_char_literal] = ACTIONS(2441),
    [anon_sym_true] = ACTIONS(2443),
    [anon_sym_false] = ACTIONS(2443),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2443),
    [sym_super] = ACTIONS(2443),
    [sym_crate] = ACTIONS(2443),
    [sym_metavariable] = ACTIONS(2441),
    [sym__raw_string_literal_start] = ACTIONS(2441),
    [sym_float_literal] = ACTIONS(2441),
  },
  [658] = {
    [sym_line_comment] = STATE(658),
    [sym_block_comment] = STATE(658),
    [ts_builtin_sym_end] = ACTIONS(2445),
    [sym_identifier] = ACTIONS(2447),
    [anon_sym_SEMI] = ACTIONS(2445),
    [anon_sym_macro_rules_BANG] = ACTIONS(2445),
    [anon_sym_LPAREN] = ACTIONS(2445),
    [anon_sym_LBRACK] = ACTIONS(2445),
    [anon_sym_LBRACE] = ACTIONS(2445),
    [anon_sym_RBRACE] = ACTIONS(2445),
    [anon_sym_STAR] = ACTIONS(2445),
    [anon_sym_u8] = ACTIONS(2447),
    [anon_sym_i8] = ACTIONS(2447),
    [anon_sym_u16] = ACTIONS(2447),
    [anon_sym_i16] = ACTIONS(2447),
    [anon_sym_u32] = ACTIONS(2447),
    [anon_sym_i32] = ACTIONS(2447),
    [anon_sym_u64] = ACTIONS(2447),
    [anon_sym_i64] = ACTIONS(2447),
    [anon_sym_u128] = ACTIONS(2447),
    [anon_sym_i128] = ACTIONS(2447),
    [anon_sym_isize] = ACTIONS(2447),
    [anon_sym_usize] = ACTIONS(2447),
    [anon_sym_f32] = ACTIONS(2447),
    [anon_sym_f64] = ACTIONS(2447),
    [anon_sym_bool] = ACTIONS(2447),
    [anon_sym_str] = ACTIONS(2447),
    [anon_sym_char] = ACTIONS(2447),
    [anon_sym_DASH] = ACTIONS(2445),
    [anon_sym_BANG] = ACTIONS(2445),
    [anon_sym_AMP] = ACTIONS(2445),
    [anon_sym_PIPE] = ACTIONS(2445),
    [anon_sym_LT] = ACTIONS(2445),
    [anon_sym_DOT_DOT] = ACTIONS(2445),
    [anon_sym_COLON_COLON] = ACTIONS(2445),
    [anon_sym_POUND] = ACTIONS(2445),
    [anon_sym_SQUOTE] = ACTIONS(2447),
    [anon_sym_async] = ACTIONS(2447),
    [anon_sym_break] = ACTIONS(2447),
    [anon_sym_const] = ACTIONS(2447),
    [anon_sym_continue] = ACTIONS(2447),
    [anon_sym_default] = ACTIONS(2447),
    [anon_sym_enum] = ACTIONS(2447),
    [anon_sym_fn] = ACTIONS(2447),
    [anon_sym_for] = ACTIONS(2447),
    [anon_sym_gen] = ACTIONS(2447),
    [anon_sym_if] = ACTIONS(2447),
    [anon_sym_impl] = ACTIONS(2447),
    [anon_sym_let] = ACTIONS(2447),
    [anon_sym_loop] = ACTIONS(2447),
    [anon_sym_match] = ACTIONS(2447),
    [anon_sym_mod] = ACTIONS(2447),
    [anon_sym_pub] = ACTIONS(2447),
    [anon_sym_return] = ACTIONS(2447),
    [anon_sym_static] = ACTIONS(2447),
    [anon_sym_struct] = ACTIONS(2447),
    [anon_sym_trait] = ACTIONS(2447),
    [anon_sym_type] = ACTIONS(2447),
    [anon_sym_union] = ACTIONS(2447),
    [anon_sym_unsafe] = ACTIONS(2447),
    [anon_sym_use] = ACTIONS(2447),
    [anon_sym_while] = ACTIONS(2447),
    [anon_sym_extern] = ACTIONS(2447),
    [anon_sym_yield] = ACTIONS(2447),
    [anon_sym_move] = ACTIONS(2447),
    [anon_sym_try] = ACTIONS(2447),
    [sym_integer_literal] = ACTIONS(2445),
    [aux_sym_string_literal_token1] = ACTIONS(2445),
    [sym_char_literal] = ACTIONS(2445),
    [anon_sym_true] = ACTIONS(2447),
    [anon_sym_false] = ACTIONS(2447),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2447),
    [sym_super] = ACTIONS(2447),
    [sym_crate] = ACTIONS(2447),
    [sym_metavariable] = ACTIONS(2445),
    [sym__raw_string_literal_start] = ACTIONS(2445),
    [sym_float_literal] = ACTIONS(2445),
  },
  [659] = {
    [sym_line_comment] = STATE(659),
    [sym_block_comment] = STATE(659),
    [ts_builtin_sym_end] = ACTIONS(2449),
    [sym_identifier] = ACTIONS(2451),
    [anon_sym_SEMI] = ACTIONS(2449),
    [anon_sym_macro_rules_BANG] = ACTIONS(2449),
    [anon_sym_LPAREN] = ACTIONS(2449),
    [anon_sym_LBRACK] = ACTIONS(2449),
    [anon_sym_LBRACE] = ACTIONS(2449),
    [anon_sym_RBRACE] = ACTIONS(2449),
    [anon_sym_STAR] = ACTIONS(2449),
    [anon_sym_u8] = ACTIONS(2451),
    [anon_sym_i8] = ACTIONS(2451),
    [anon_sym_u16] = ACTIONS(2451),
    [anon_sym_i16] = ACTIONS(2451),
    [anon_sym_u32] = ACTIONS(2451),
    [anon_sym_i32] = ACTIONS(2451),
    [anon_sym_u64] = ACTIONS(2451),
    [anon_sym_i64] = ACTIONS(2451),
    [anon_sym_u128] = ACTIONS(2451),
    [anon_sym_i128] = ACTIONS(2451),
    [anon_sym_isize] = ACTIONS(2451),
    [anon_sym_usize] = ACTIONS(2451),
    [anon_sym_f32] = ACTIONS(2451),
    [anon_sym_f64] = ACTIONS(2451),
    [anon_sym_bool] = ACTIONS(2451),
    [anon_sym_str] = ACTIONS(2451),
    [anon_sym_char] = ACTIONS(2451),
    [anon_sym_DASH] = ACTIONS(2449),
    [anon_sym_BANG] = ACTIONS(2449),
    [anon_sym_AMP] = ACTIONS(2449),
    [anon_sym_PIPE] = ACTIONS(2449),
    [anon_sym_LT] = ACTIONS(2449),
    [anon_sym_DOT_DOT] = ACTIONS(2449),
    [anon_sym_COLON_COLON] = ACTIONS(2449),
    [anon_sym_POUND] = ACTIONS(2449),
    [anon_sym_SQUOTE] = ACTIONS(2451),
    [anon_sym_async] = ACTIONS(2451),
    [anon_sym_break] = ACTIONS(2451),
    [anon_sym_const] = ACTIONS(2451),
    [anon_sym_continue] = ACTIONS(2451),
    [anon_sym_default] = ACTIONS(2451),
    [anon_sym_enum] = ACTIONS(2451),
    [anon_sym_fn] = ACTIONS(2451),
    [anon_sym_for] = ACTIONS(2451),
    [anon_sym_gen] = ACTIONS(2451),
    [anon_sym_if] = ACTIONS(2451),
    [anon_sym_impl] = ACTIONS(2451),
    [anon_sym_let] = ACTIONS(2451),
    [anon_sym_loop] = ACTIONS(2451),
    [anon_sym_match] = ACTIONS(2451),
    [anon_sym_mod] = ACTIONS(2451),
    [anon_sym_pub] = ACTIONS(2451),
    [anon_sym_return] = ACTIONS(2451),
    [anon_sym_static] = ACTIONS(2451),
    [anon_sym_struct] = ACTIONS(2451),
    [anon_sym_trait] = ACTIONS(2451),
    [anon_sym_type] = ACTIONS(2451),
    [anon_sym_union] = ACTIONS(2451),
    [anon_sym_unsafe] = ACTIONS(2451),
    [anon_sym_use] = ACTIONS(2451),
    [anon_sym_while] = ACTIONS(2451),
    [anon_sym_extern] = ACTIONS(2451),
    [anon_sym_yield] = ACTIONS(2451),
    [anon_sym_move] = ACTIONS(2451),
    [anon_sym_try] = ACTIONS(2451),
    [sym_integer_literal] = ACTIONS(2449),
    [aux_sym_string_literal_token1] = ACTIONS(2449),
    [sym_char_literal] = ACTIONS(2449),
    [anon_sym_true] = ACTIONS(2451),
    [anon_sym_false] = ACTIONS(2451),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2451),
    [sym_super] = ACTIONS(2451),
    [sym_crate] = ACTIONS(2451),
    [sym_metavariable] = ACTIONS(2449),
    [sym__raw_string_literal_start] = ACTIONS(2449),
    [sym_float_literal] = ACTIONS(2449),
  },
  [660] = {
    [sym_line_comment] = STATE(660),
    [sym_block_comment] = STATE(660),
    [ts_builtin_sym_end] = ACTIONS(2453),
    [sym_identifier] = ACTIONS(2455),
    [anon_sym_SEMI] = ACTIONS(2453),
    [anon_sym_macro_rules_BANG] = ACTIONS(2453),
    [anon_sym_LPAREN] = ACTIONS(2453),
    [anon_sym_LBRACK] = ACTIONS(2453),
    [anon_sym_LBRACE] = ACTIONS(2453),
    [anon_sym_RBRACE] = ACTIONS(2453),
    [anon_sym_STAR] = ACTIONS(2453),
    [anon_sym_u8] = ACTIONS(2455),
    [anon_sym_i8] = ACTIONS(2455),
    [anon_sym_u16] = ACTIONS(2455),
    [anon_sym_i16] = ACTIONS(2455),
    [anon_sym_u32] = ACTIONS(2455),
    [anon_sym_i32] = ACTIONS(2455),
    [anon_sym_u64] = ACTIONS(2455),
    [anon_sym_i64] = ACTIONS(2455),
    [anon_sym_u128] = ACTIONS(2455),
    [anon_sym_i128] = ACTIONS(2455),
    [anon_sym_isize] = ACTIONS(2455),
    [anon_sym_usize] = ACTIONS(2455),
    [anon_sym_f32] = ACTIONS(2455),
    [anon_sym_f64] = ACTIONS(2455),
    [anon_sym_bool] = ACTIONS(2455),
    [anon_sym_str] = ACTIONS(2455),
    [anon_sym_char] = ACTIONS(2455),
    [anon_sym_DASH] = ACTIONS(2453),
    [anon_sym_BANG] = ACTIONS(2453),
    [anon_sym_AMP] = ACTIONS(2453),
    [anon_sym_PIPE] = ACTIONS(2453),
    [anon_sym_LT] = ACTIONS(2453),
    [anon_sym_DOT_DOT] = ACTIONS(2453),
    [anon_sym_COLON_COLON] = ACTIONS(2453),
    [anon_sym_POUND] = ACTIONS(2453),
    [anon_sym_SQUOTE] = ACTIONS(2455),
    [anon_sym_async] = ACTIONS(2455),
    [anon_sym_break] = ACTIONS(2455),
    [anon_sym_const] = ACTIONS(2455),
    [anon_sym_continue] = ACTIONS(2455),
    [anon_sym_default] = ACTIONS(2455),
    [anon_sym_enum] = ACTIONS(2455),
    [anon_sym_fn] = ACTIONS(2455),
    [anon_sym_for] = ACTIONS(2455),
    [anon_sym_gen] = ACTIONS(2455),
    [anon_sym_if] = ACTIONS(2455),
    [anon_sym_impl] = ACTIONS(2455),
    [anon_sym_let] = ACTIONS(2455),
    [anon_sym_loop] = ACTIONS(2455),
    [anon_sym_match] = ACTIONS(2455),
    [anon_sym_mod] = ACTIONS(2455),
    [anon_sym_pub] = ACTIONS(2455),
    [anon_sym_return] = ACTIONS(2455),
    [anon_sym_static] = ACTIONS(2455),
    [anon_sym_struct] = ACTIONS(2455),
    [anon_sym_trait] = ACTIONS(2455),
    [anon_sym_type] = ACTIONS(2455),
    [anon_sym_union] = ACTIONS(2455),
    [anon_sym_unsafe] = ACTIONS(2455),
    [anon_sym_use] = ACTIONS(2455),
    [anon_sym_while] = ACTIONS(2455),
    [anon_sym_extern] = ACTIONS(2455),
    [anon_sym_yield] = ACTIONS(2455),
    [anon_sym_move] = ACTIONS(2455),
    [anon_sym_try] = ACTIONS(2455),
    [sym_integer_literal] = ACTIONS(2453),
    [aux_sym_string_literal_token1] = ACTIONS(2453),
    [sym_char_literal] = ACTIONS(2453),
    [anon_sym_true] = ACTIONS(2455),
    [anon_sym_false] = ACTIONS(2455),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2455),
    [sym_super] = ACTIONS(2455),
    [sym_crate] = ACTIONS(2455),
    [sym_metavariable] = ACTIONS(2453),
    [sym__raw_string_literal_start] = ACTIONS(2453),
    [sym_float_literal] = ACTIONS(2453),
  },
  [661] = {
    [sym_line_comment] = STATE(661),
    [sym_block_comment] = STATE(661),
    [ts_builtin_sym_end] = ACTIONS(2457),
    [sym_identifier] = ACTIONS(2459),
    [anon_sym_SEMI] = ACTIONS(2457),
    [anon_sym_macro_rules_BANG] = ACTIONS(2457),
    [anon_sym_LPAREN] = ACTIONS(2457),
    [anon_sym_LBRACK] = ACTIONS(2457),
    [anon_sym_LBRACE] = ACTIONS(2457),
    [anon_sym_RBRACE] = ACTIONS(2457),
    [anon_sym_STAR] = ACTIONS(2457),
    [anon_sym_u8] = ACTIONS(2459),
    [anon_sym_i8] = ACTIONS(2459),
    [anon_sym_u16] = ACTIONS(2459),
    [anon_sym_i16] = ACTIONS(2459),
    [anon_sym_u32] = ACTIONS(2459),
    [anon_sym_i32] = ACTIONS(2459),
    [anon_sym_u64] = ACTIONS(2459),
    [anon_sym_i64] = ACTIONS(2459),
    [anon_sym_u128] = ACTIONS(2459),
    [anon_sym_i128] = ACTIONS(2459),
    [anon_sym_isize] = ACTIONS(2459),
    [anon_sym_usize] = ACTIONS(2459),
    [anon_sym_f32] = ACTIONS(2459),
    [anon_sym_f64] = ACTIONS(2459),
    [anon_sym_bool] = ACTIONS(2459),
    [anon_sym_str] = ACTIONS(2459),
    [anon_sym_char] = ACTIONS(2459),
    [anon_sym_DASH] = ACTIONS(2457),
    [anon_sym_BANG] = ACTIONS(2457),
    [anon_sym_AMP] = ACTIONS(2457),
    [anon_sym_PIPE] = ACTIONS(2457),
    [anon_sym_LT] = ACTIONS(2457),
    [anon_sym_DOT_DOT] = ACTIONS(2457),
    [anon_sym_COLON_COLON] = ACTIONS(2457),
    [anon_sym_POUND] = ACTIONS(2457),
    [anon_sym_SQUOTE] = ACTIONS(2459),
    [anon_sym_async] = ACTIONS(2459),
    [anon_sym_break] = ACTIONS(2459),
    [anon_sym_const] = ACTIONS(2459),
    [anon_sym_continue] = ACTIONS(2459),
    [anon_sym_default] = ACTIONS(2459),
    [anon_sym_enum] = ACTIONS(2459),
    [anon_sym_fn] = ACTIONS(2459),
    [anon_sym_for] = ACTIONS(2459),
    [anon_sym_gen] = ACTIONS(2459),
    [anon_sym_if] = ACTIONS(2459),
    [anon_sym_impl] = ACTIONS(2459),
    [anon_sym_let] = ACTIONS(2459),
    [anon_sym_loop] = ACTIONS(2459),
    [anon_sym_match] = ACTIONS(2459),
    [anon_sym_mod] = ACTIONS(2459),
    [anon_sym_pub] = ACTIONS(2459),
    [anon_sym_return] = ACTIONS(2459),
    [anon_sym_static] = ACTIONS(2459),
    [anon_sym_struct] = ACTIONS(2459),
    [anon_sym_trait] = ACTIONS(2459),
    [anon_sym_type] = ACTIONS(2459),
    [anon_sym_union] = ACTIONS(2459),
    [anon_sym_unsafe] = ACTIONS(2459),
    [anon_sym_use] = ACTIONS(2459),
    [anon_sym_while] = ACTIONS(2459),
    [anon_sym_extern] = ACTIONS(2459),
    [anon_sym_yield] = ACTIONS(2459),
    [anon_sym_move] = ACTIONS(2459),
    [anon_sym_try] = ACTIONS(2459),
    [sym_integer_literal] = ACTIONS(2457),
    [aux_sym_string_literal_token1] = ACTIONS(2457),
    [sym_char_literal] = ACTIONS(2457),
    [anon_sym_true] = ACTIONS(2459),
    [anon_sym_false] = ACTIONS(2459),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2459),
    [sym_super] = ACTIONS(2459),
    [sym_crate] = ACTIONS(2459),
    [sym_metavariable] = ACTIONS(2457),
    [sym__raw_string_literal_start] = ACTIONS(2457),
    [sym_float_literal] = ACTIONS(2457),
  },
  [662] = {
    [sym_empty_statement] = STATE(1072),
    [sym_macro_definition] = STATE(1072),
    [sym_attribute_item] = STATE(1072),
    [sym_inner_attribute_item] = STATE(1072),
    [sym_mod_item] = STATE(1072),
    [sym_foreign_mod_item] = STATE(1072),
    [sym_struct_item] = STATE(1072),
    [sym_union_item] = STATE(1072),
    [sym_enum_item] = STATE(1072),
    [sym_extern_crate_declaration] = STATE(1072),
    [sym_const_item] = STATE(1072),
    [sym_static_item] = STATE(1072),
    [sym_type_item] = STATE(1072),
    [sym_function_item] = STATE(1072),
    [sym_function_signature_item] = STATE(1072),
    [sym_function_modifiers] = STATE(3622),
    [sym_impl_item] = STATE(1072),
    [sym_trait_item] = STATE(1072),
    [sym_associated_type] = STATE(1072),
    [sym_let_declaration] = STATE(1072),
    [sym_use_declaration] = STATE(1072),
    [sym_extern_modifier] = STATE(2165),
    [sym_visibility_modifier] = STATE(1954),
    [sym_bracketed_type] = STATE(3353),
    [sym_generic_type_with_turbofish] = STATE(3379),
    [sym_macro_invocation] = STATE(1072),
    [sym_scoped_identifier] = STATE(3167),
    [sym_line_comment] = STATE(662),
    [sym_block_comment] = STATE(662),
    [aux_sym_declaration_list_repeat1] = STATE(667),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(2461),
    [anon_sym_SEMI] = ACTIONS(2463),
    [anon_sym_macro_rules_BANG] = ACTIONS(2465),
    [anon_sym_RBRACE] = ACTIONS(2467),
    [anon_sym_u8] = ACTIONS(2469),
    [anon_sym_i8] = ACTIONS(2469),
    [anon_sym_u16] = ACTIONS(2469),
    [anon_sym_i16] = ACTIONS(2469),
    [anon_sym_u32] = ACTIONS(2469),
    [anon_sym_i32] = ACTIONS(2469),
    [anon_sym_u64] = ACTIONS(2469),
    [anon_sym_i64] = ACTIONS(2469),
    [anon_sym_u128] = ACTIONS(2469),
    [anon_sym_i128] = ACTIONS(2469),
    [anon_sym_isize] = ACTIONS(2469),
    [anon_sym_usize] = ACTIONS(2469),
    [anon_sym_f32] = ACTIONS(2469),
    [anon_sym_f64] = ACTIONS(2469),
    [anon_sym_bool] = ACTIONS(2469),
    [anon_sym_str] = ACTIONS(2469),
    [anon_sym_char] = ACTIONS(2469),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(2471),
    [anon_sym_POUND] = ACTIONS(2473),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(2475),
    [anon_sym_default] = ACTIONS(2477),
    [anon_sym_enum] = ACTIONS(2479),
    [anon_sym_fn] = ACTIONS(2481),
    [anon_sym_gen] = ACTIONS(2483),
    [anon_sym_impl] = ACTIONS(2485),
    [anon_sym_let] = ACTIONS(2487),
    [anon_sym_mod] = ACTIONS(2489),
    [anon_sym_pub] = ACTIONS(69),
    [anon_sym_static] = ACTIONS(2491),
    [anon_sym_struct] = ACTIONS(2493),
    [anon_sym_trait] = ACTIONS(2495),
    [anon_sym_type] = ACTIONS(2497),
    [anon_sym_union] = ACTIONS(2499),
    [anon_sym_unsafe] = ACTIONS(2501),
    [anon_sym_use] = ACTIONS(2503),
    [anon_sym_extern] = ACTIONS(2505),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2507),
    [sym_super] = ACTIONS(2507),
    [sym_crate] = ACTIONS(2509),
    [sym_metavariable] = ACTIONS(2511),
  },
  [663] = {
    [sym_line_comment] = STATE(663),
    [sym_block_comment] = STATE(663),
    [ts_builtin_sym_end] = ACTIONS(2513),
    [sym_identifier] = ACTIONS(2515),
    [anon_sym_SEMI] = ACTIONS(2513),
    [anon_sym_macro_rules_BANG] = ACTIONS(2513),
    [anon_sym_LPAREN] = ACTIONS(2513),
    [anon_sym_LBRACK] = ACTIONS(2513),
    [anon_sym_LBRACE] = ACTIONS(2513),
    [anon_sym_RBRACE] = ACTIONS(2513),
    [anon_sym_STAR] = ACTIONS(2513),
    [anon_sym_u8] = ACTIONS(2515),
    [anon_sym_i8] = ACTIONS(2515),
    [anon_sym_u16] = ACTIONS(2515),
    [anon_sym_i16] = ACTIONS(2515),
    [anon_sym_u32] = ACTIONS(2515),
    [anon_sym_i32] = ACTIONS(2515),
    [anon_sym_u64] = ACTIONS(2515),
    [anon_sym_i64] = ACTIONS(2515),
    [anon_sym_u128] = ACTIONS(2515),
    [anon_sym_i128] = ACTIONS(2515),
    [anon_sym_isize] = ACTIONS(2515),
    [anon_sym_usize] = ACTIONS(2515),
    [anon_sym_f32] = ACTIONS(2515),
    [anon_sym_f64] = ACTIONS(2515),
    [anon_sym_bool] = ACTIONS(2515),
    [anon_sym_str] = ACTIONS(2515),
    [anon_sym_char] = ACTIONS(2515),
    [anon_sym_DASH] = ACTIONS(2513),
    [anon_sym_BANG] = ACTIONS(2513),
    [anon_sym_AMP] = ACTIONS(2513),
    [anon_sym_PIPE] = ACTIONS(2513),
    [anon_sym_LT] = ACTIONS(2513),
    [anon_sym_DOT_DOT] = ACTIONS(2513),
    [anon_sym_COLON_COLON] = ACTIONS(2513),
    [anon_sym_POUND] = ACTIONS(2513),
    [anon_sym_SQUOTE] = ACTIONS(2515),
    [anon_sym_async] = ACTIONS(2515),
    [anon_sym_break] = ACTIONS(2515),
    [anon_sym_const] = ACTIONS(2515),
    [anon_sym_continue] = ACTIONS(2515),
    [anon_sym_default] = ACTIONS(2515),
    [anon_sym_enum] = ACTIONS(2515),
    [anon_sym_fn] = ACTIONS(2515),
    [anon_sym_for] = ACTIONS(2515),
    [anon_sym_gen] = ACTIONS(2515),
    [anon_sym_if] = ACTIONS(2515),
    [anon_sym_impl] = ACTIONS(2515),
    [anon_sym_let] = ACTIONS(2515),
    [anon_sym_loop] = ACTIONS(2515),
    [anon_sym_match] = ACTIONS(2515),
    [anon_sym_mod] = ACTIONS(2515),
    [anon_sym_pub] = ACTIONS(2515),
    [anon_sym_return] = ACTIONS(2515),
    [anon_sym_static] = ACTIONS(2515),
    [anon_sym_struct] = ACTIONS(2515),
    [anon_sym_trait] = ACTIONS(2515),
    [anon_sym_type] = ACTIONS(2515),
    [anon_sym_union] = ACTIONS(2515),
    [anon_sym_unsafe] = ACTIONS(2515),
    [anon_sym_use] = ACTIONS(2515),
    [anon_sym_while] = ACTIONS(2515),
    [anon_sym_extern] = ACTIONS(2515),
    [anon_sym_yield] = ACTIONS(2515),
    [anon_sym_move] = ACTIONS(2515),
    [anon_sym_try] = ACTIONS(2515),
    [sym_integer_literal] = ACTIONS(2513),
    [aux_sym_string_literal_token1] = ACTIONS(2513),
    [sym_char_literal] = ACTIONS(2513),
    [anon_sym_true] = ACTIONS(2515),
    [anon_sym_false] = ACTIONS(2515),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2515),
    [sym_super] = ACTIONS(2515),
    [sym_crate] = ACTIONS(2515),
    [sym_metavariable] = ACTIONS(2513),
    [sym__raw_string_literal_start] = ACTIONS(2513),
    [sym_float_literal] = ACTIONS(2513),
  },
  [664] = {
    [sym_line_comment] = STATE(664),
    [sym_block_comment] = STATE(664),
    [ts_builtin_sym_end] = ACTIONS(2517),
    [sym_identifier] = ACTIONS(2519),
    [anon_sym_SEMI] = ACTIONS(2517),
    [anon_sym_macro_rules_BANG] = ACTIONS(2517),
    [anon_sym_LPAREN] = ACTIONS(2517),
    [anon_sym_LBRACK] = ACTIONS(2517),
    [anon_sym_LBRACE] = ACTIONS(2517),
    [anon_sym_RBRACE] = ACTIONS(2517),
    [anon_sym_STAR] = ACTIONS(2517),
    [anon_sym_u8] = ACTIONS(2519),
    [anon_sym_i8] = ACTIONS(2519),
    [anon_sym_u16] = ACTIONS(2519),
    [anon_sym_i16] = ACTIONS(2519),
    [anon_sym_u32] = ACTIONS(2519),
    [anon_sym_i32] = ACTIONS(2519),
    [anon_sym_u64] = ACTIONS(2519),
    [anon_sym_i64] = ACTIONS(2519),
    [anon_sym_u128] = ACTIONS(2519),
    [anon_sym_i128] = ACTIONS(2519),
    [anon_sym_isize] = ACTIONS(2519),
    [anon_sym_usize] = ACTIONS(2519),
    [anon_sym_f32] = ACTIONS(2519),
    [anon_sym_f64] = ACTIONS(2519),
    [anon_sym_bool] = ACTIONS(2519),
    [anon_sym_str] = ACTIONS(2519),
    [anon_sym_char] = ACTIONS(2519),
    [anon_sym_DASH] = ACTIONS(2517),
    [anon_sym_BANG] = ACTIONS(2517),
    [anon_sym_AMP] = ACTIONS(2517),
    [anon_sym_PIPE] = ACTIONS(2517),
    [anon_sym_LT] = ACTIONS(2517),
    [anon_sym_DOT_DOT] = ACTIONS(2517),
    [anon_sym_COLON_COLON] = ACTIONS(2517),
    [anon_sym_POUND] = ACTIONS(2517),
    [anon_sym_SQUOTE] = ACTIONS(2519),
    [anon_sym_async] = ACTIONS(2519),
    [anon_sym_break] = ACTIONS(2519),
    [anon_sym_const] = ACTIONS(2519),
    [anon_sym_continue] = ACTIONS(2519),
    [anon_sym_default] = ACTIONS(2519),
    [anon_sym_enum] = ACTIONS(2519),
    [anon_sym_fn] = ACTIONS(2519),
    [anon_sym_for] = ACTIONS(2519),
    [anon_sym_gen] = ACTIONS(2519),
    [anon_sym_if] = ACTIONS(2519),
    [anon_sym_impl] = ACTIONS(2519),
    [anon_sym_let] = ACTIONS(2519),
    [anon_sym_loop] = ACTIONS(2519),
    [anon_sym_match] = ACTIONS(2519),
    [anon_sym_mod] = ACTIONS(2519),
    [anon_sym_pub] = ACTIONS(2519),
    [anon_sym_return] = ACTIONS(2519),
    [anon_sym_static] = ACTIONS(2519),
    [anon_sym_struct] = ACTIONS(2519),
    [anon_sym_trait] = ACTIONS(2519),
    [anon_sym_type] = ACTIONS(2519),
    [anon_sym_union] = ACTIONS(2519),
    [anon_sym_unsafe] = ACTIONS(2519),
    [anon_sym_use] = ACTIONS(2519),
    [anon_sym_while] = ACTIONS(2519),
    [anon_sym_extern] = ACTIONS(2519),
    [anon_sym_yield] = ACTIONS(2519),
    [anon_sym_move] = ACTIONS(2519),
    [anon_sym_try] = ACTIONS(2519),
    [sym_integer_literal] = ACTIONS(2517),
    [aux_sym_string_literal_token1] = ACTIONS(2517),
    [sym_char_literal] = ACTIONS(2517),
    [anon_sym_true] = ACTIONS(2519),
    [anon_sym_false] = ACTIONS(2519),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2519),
    [sym_super] = ACTIONS(2519),
    [sym_crate] = ACTIONS(2519),
    [sym_metavariable] = ACTIONS(2517),
    [sym__raw_string_literal_start] = ACTIONS(2517),
    [sym_float_literal] = ACTIONS(2517),
  },
  [665] = {
    [sym_line_comment] = STATE(665),
    [sym_block_comment] = STATE(665),
    [ts_builtin_sym_end] = ACTIONS(1463),
    [sym_identifier] = ACTIONS(1465),
    [anon_sym_SEMI] = ACTIONS(1463),
    [anon_sym_macro_rules_BANG] = ACTIONS(1463),
    [anon_sym_LPAREN] = ACTIONS(1463),
    [anon_sym_LBRACK] = ACTIONS(1463),
    [anon_sym_LBRACE] = ACTIONS(1463),
    [anon_sym_RBRACE] = ACTIONS(1463),
    [anon_sym_STAR] = ACTIONS(1463),
    [anon_sym_u8] = ACTIONS(1465),
    [anon_sym_i8] = ACTIONS(1465),
    [anon_sym_u16] = ACTIONS(1465),
    [anon_sym_i16] = ACTIONS(1465),
    [anon_sym_u32] = ACTIONS(1465),
    [anon_sym_i32] = ACTIONS(1465),
    [anon_sym_u64] = ACTIONS(1465),
    [anon_sym_i64] = ACTIONS(1465),
    [anon_sym_u128] = ACTIONS(1465),
    [anon_sym_i128] = ACTIONS(1465),
    [anon_sym_isize] = ACTIONS(1465),
    [anon_sym_usize] = ACTIONS(1465),
    [anon_sym_f32] = ACTIONS(1465),
    [anon_sym_f64] = ACTIONS(1465),
    [anon_sym_bool] = ACTIONS(1465),
    [anon_sym_str] = ACTIONS(1465),
    [anon_sym_char] = ACTIONS(1465),
    [anon_sym_DASH] = ACTIONS(1463),
    [anon_sym_BANG] = ACTIONS(1463),
    [anon_sym_AMP] = ACTIONS(1463),
    [anon_sym_PIPE] = ACTIONS(1463),
    [anon_sym_LT] = ACTIONS(1463),
    [anon_sym_DOT_DOT] = ACTIONS(1463),
    [anon_sym_COLON_COLON] = ACTIONS(1463),
    [anon_sym_POUND] = ACTIONS(1463),
    [anon_sym_SQUOTE] = ACTIONS(1465),
    [anon_sym_async] = ACTIONS(1465),
    [anon_sym_break] = ACTIONS(1465),
    [anon_sym_const] = ACTIONS(1465),
    [anon_sym_continue] = ACTIONS(1465),
    [anon_sym_default] = ACTIONS(1465),
    [anon_sym_enum] = ACTIONS(1465),
    [anon_sym_fn] = ACTIONS(1465),
    [anon_sym_for] = ACTIONS(1465),
    [anon_sym_gen] = ACTIONS(1465),
    [anon_sym_if] = ACTIONS(1465),
    [anon_sym_impl] = ACTIONS(1465),
    [anon_sym_let] = ACTIONS(1465),
    [anon_sym_loop] = ACTIONS(1465),
    [anon_sym_match] = ACTIONS(1465),
    [anon_sym_mod] = ACTIONS(1465),
    [anon_sym_pub] = ACTIONS(1465),
    [anon_sym_return] = ACTIONS(1465),
    [anon_sym_static] = ACTIONS(1465),
    [anon_sym_struct] = ACTIONS(1465),
    [anon_sym_trait] = ACTIONS(1465),
    [anon_sym_type] = ACTIONS(1465),
    [anon_sym_union] = ACTIONS(1465),
    [anon_sym_unsafe] = ACTIONS(1465),
    [anon_sym_use] = ACTIONS(1465),
    [anon_sym_while] = ACTIONS(1465),
    [anon_sym_extern] = ACTIONS(1465),
    [anon_sym_yield] = ACTIONS(1465),
    [anon_sym_move] = ACTIONS(1465),
    [anon_sym_try] = ACTIONS(1465),
    [sym_integer_literal] = ACTIONS(1463),
    [aux_sym_string_literal_token1] = ACTIONS(1463),
    [sym_char_literal] = ACTIONS(1463),
    [anon_sym_true] = ACTIONS(1465),
    [anon_sym_false] = ACTIONS(1465),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1465),
    [sym_super] = ACTIONS(1465),
    [sym_crate] = ACTIONS(1465),
    [sym_metavariable] = ACTIONS(1463),
    [sym__raw_string_literal_start] = ACTIONS(1463),
    [sym_float_literal] = ACTIONS(1463),
  },
  [666] = {
    [sym_line_comment] = STATE(666),
    [sym_block_comment] = STATE(666),
    [ts_builtin_sym_end] = ACTIONS(2521),
    [sym_identifier] = ACTIONS(2523),
    [anon_sym_SEMI] = ACTIONS(2521),
    [anon_sym_macro_rules_BANG] = ACTIONS(2521),
    [anon_sym_LPAREN] = ACTIONS(2521),
    [anon_sym_LBRACK] = ACTIONS(2521),
    [anon_sym_LBRACE] = ACTIONS(2521),
    [anon_sym_RBRACE] = ACTIONS(2521),
    [anon_sym_STAR] = ACTIONS(2521),
    [anon_sym_u8] = ACTIONS(2523),
    [anon_sym_i8] = ACTIONS(2523),
    [anon_sym_u16] = ACTIONS(2523),
    [anon_sym_i16] = ACTIONS(2523),
    [anon_sym_u32] = ACTIONS(2523),
    [anon_sym_i32] = ACTIONS(2523),
    [anon_sym_u64] = ACTIONS(2523),
    [anon_sym_i64] = ACTIONS(2523),
    [anon_sym_u128] = ACTIONS(2523),
    [anon_sym_i128] = ACTIONS(2523),
    [anon_sym_isize] = ACTIONS(2523),
    [anon_sym_usize] = ACTIONS(2523),
    [anon_sym_f32] = ACTIONS(2523),
    [anon_sym_f64] = ACTIONS(2523),
    [anon_sym_bool] = ACTIONS(2523),
    [anon_sym_str] = ACTIONS(2523),
    [anon_sym_char] = ACTIONS(2523),
    [anon_sym_DASH] = ACTIONS(2521),
    [anon_sym_BANG] = ACTIONS(2521),
    [anon_sym_AMP] = ACTIONS(2521),
    [anon_sym_PIPE] = ACTIONS(2521),
    [anon_sym_LT] = ACTIONS(2521),
    [anon_sym_DOT_DOT] = ACTIONS(2521),
    [anon_sym_COLON_COLON] = ACTIONS(2521),
    [anon_sym_POUND] = ACTIONS(2521),
    [anon_sym_SQUOTE] = ACTIONS(2523),
    [anon_sym_async] = ACTIONS(2523),
    [anon_sym_break] = ACTIONS(2523),
    [anon_sym_const] = ACTIONS(2523),
    [anon_sym_continue] = ACTIONS(2523),
    [anon_sym_default] = ACTIONS(2523),
    [anon_sym_enum] = ACTIONS(2523),
    [anon_sym_fn] = ACTIONS(2523),
    [anon_sym_for] = ACTIONS(2523),
    [anon_sym_gen] = ACTIONS(2523),
    [anon_sym_if] = ACTIONS(2523),
    [anon_sym_impl] = ACTIONS(2523),
    [anon_sym_let] = ACTIONS(2523),
    [anon_sym_loop] = ACTIONS(2523),
    [anon_sym_match] = ACTIONS(2523),
    [anon_sym_mod] = ACTIONS(2523),
    [anon_sym_pub] = ACTIONS(2523),
    [anon_sym_return] = ACTIONS(2523),
    [anon_sym_static] = ACTIONS(2523),
    [anon_sym_struct] = ACTIONS(2523),
    [anon_sym_trait] = ACTIONS(2523),
    [anon_sym_type] = ACTIONS(2523),
    [anon_sym_union] = ACTIONS(2523),
    [anon_sym_unsafe] = ACTIONS(2523),
    [anon_sym_use] = ACTIONS(2523),
    [anon_sym_while] = ACTIONS(2523),
    [anon_sym_extern] = ACTIONS(2523),
    [anon_sym_yield] = ACTIONS(2523),
    [anon_sym_move] = ACTIONS(2523),
    [anon_sym_try] = ACTIONS(2523),
    [sym_integer_literal] = ACTIONS(2521),
    [aux_sym_string_literal_token1] = ACTIONS(2521),
    [sym_char_literal] = ACTIONS(2521),
    [anon_sym_true] = ACTIONS(2523),
    [anon_sym_false] = ACTIONS(2523),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2523),
    [sym_super] = ACTIONS(2523),
    [sym_crate] = ACTIONS(2523),
    [sym_metavariable] = ACTIONS(2521),
    [sym__raw_string_literal_start] = ACTIONS(2521),
    [sym_float_literal] = ACTIONS(2521),
  },
  [667] = {
    [sym_empty_statement] = STATE(1072),
    [sym_macro_definition] = STATE(1072),
    [sym_attribute_item] = STATE(1072),
    [sym_inner_attribute_item] = STATE(1072),
    [sym_mod_item] = STATE(1072),
    [sym_foreign_mod_item] = STATE(1072),
    [sym_struct_item] = STATE(1072),
    [sym_union_item] = STATE(1072),
    [sym_enum_item] = STATE(1072),
    [sym_extern_crate_declaration] = STATE(1072),
    [sym_const_item] = STATE(1072),
    [sym_static_item] = STATE(1072),
    [sym_type_item] = STATE(1072),
    [sym_function_item] = STATE(1072),
    [sym_function_signature_item] = STATE(1072),
    [sym_function_modifiers] = STATE(3622),
    [sym_impl_item] = STATE(1072),
    [sym_trait_item] = STATE(1072),
    [sym_associated_type] = STATE(1072),
    [sym_let_declaration] = STATE(1072),
    [sym_use_declaration] = STATE(1072),
    [sym_extern_modifier] = STATE(2165),
    [sym_visibility_modifier] = STATE(1954),
    [sym_bracketed_type] = STATE(3353),
    [sym_generic_type_with_turbofish] = STATE(3379),
    [sym_macro_invocation] = STATE(1072),
    [sym_scoped_identifier] = STATE(3167),
    [sym_line_comment] = STATE(667),
    [sym_block_comment] = STATE(667),
    [aux_sym_declaration_list_repeat1] = STATE(639),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(2461),
    [anon_sym_SEMI] = ACTIONS(2463),
    [anon_sym_macro_rules_BANG] = ACTIONS(2465),
    [anon_sym_RBRACE] = ACTIONS(2525),
    [anon_sym_u8] = ACTIONS(2469),
    [anon_sym_i8] = ACTIONS(2469),
    [anon_sym_u16] = ACTIONS(2469),
    [anon_sym_i16] = ACTIONS(2469),
    [anon_sym_u32] = ACTIONS(2469),
    [anon_sym_i32] = ACTIONS(2469),
    [anon_sym_u64] = ACTIONS(2469),
    [anon_sym_i64] = ACTIONS(2469),
    [anon_sym_u128] = ACTIONS(2469),
    [anon_sym_i128] = ACTIONS(2469),
    [anon_sym_isize] = ACTIONS(2469),
    [anon_sym_usize] = ACTIONS(2469),
    [anon_sym_f32] = ACTIONS(2469),
    [anon_sym_f64] = ACTIONS(2469),
    [anon_sym_bool] = ACTIONS(2469),
    [anon_sym_str] = ACTIONS(2469),
    [anon_sym_char] = ACTIONS(2469),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(2471),
    [anon_sym_POUND] = ACTIONS(2473),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(2475),
    [anon_sym_default] = ACTIONS(2477),
    [anon_sym_enum] = ACTIONS(2479),
    [anon_sym_fn] = ACTIONS(2481),
    [anon_sym_gen] = ACTIONS(2483),
    [anon_sym_impl] = ACTIONS(2485),
    [anon_sym_let] = ACTIONS(2487),
    [anon_sym_mod] = ACTIONS(2489),
    [anon_sym_pub] = ACTIONS(69),
    [anon_sym_static] = ACTIONS(2491),
    [anon_sym_struct] = ACTIONS(2493),
    [anon_sym_trait] = ACTIONS(2495),
    [anon_sym_type] = ACTIONS(2497),
    [anon_sym_union] = ACTIONS(2499),
    [anon_sym_unsafe] = ACTIONS(2501),
    [anon_sym_use] = ACTIONS(2503),
    [anon_sym_extern] = ACTIONS(2505),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2507),
    [sym_super] = ACTIONS(2507),
    [sym_crate] = ACTIONS(2509),
    [sym_metavariable] = ACTIONS(2511),
  },
  [668] = {
    [sym_line_comment] = STATE(668),
    [sym_block_comment] = STATE(668),
    [ts_builtin_sym_end] = ACTIONS(2527),
    [sym_identifier] = ACTIONS(2529),
    [anon_sym_SEMI] = ACTIONS(2527),
    [anon_sym_macro_rules_BANG] = ACTIONS(2527),
    [anon_sym_LPAREN] = ACTIONS(2527),
    [anon_sym_LBRACK] = ACTIONS(2527),
    [anon_sym_LBRACE] = ACTIONS(2527),
    [anon_sym_RBRACE] = ACTIONS(2527),
    [anon_sym_STAR] = ACTIONS(2527),
    [anon_sym_u8] = ACTIONS(2529),
    [anon_sym_i8] = ACTIONS(2529),
    [anon_sym_u16] = ACTIONS(2529),
    [anon_sym_i16] = ACTIONS(2529),
    [anon_sym_u32] = ACTIONS(2529),
    [anon_sym_i32] = ACTIONS(2529),
    [anon_sym_u64] = ACTIONS(2529),
    [anon_sym_i64] = ACTIONS(2529),
    [anon_sym_u128] = ACTIONS(2529),
    [anon_sym_i128] = ACTIONS(2529),
    [anon_sym_isize] = ACTIONS(2529),
    [anon_sym_usize] = ACTIONS(2529),
    [anon_sym_f32] = ACTIONS(2529),
    [anon_sym_f64] = ACTIONS(2529),
    [anon_sym_bool] = ACTIONS(2529),
    [anon_sym_str] = ACTIONS(2529),
    [anon_sym_char] = ACTIONS(2529),
    [anon_sym_DASH] = ACTIONS(2527),
    [anon_sym_BANG] = ACTIONS(2527),
    [anon_sym_AMP] = ACTIONS(2527),
    [anon_sym_PIPE] = ACTIONS(2527),
    [anon_sym_LT] = ACTIONS(2527),
    [anon_sym_DOT_DOT] = ACTIONS(2527),
    [anon_sym_COLON_COLON] = ACTIONS(2527),
    [anon_sym_POUND] = ACTIONS(2527),
    [anon_sym_SQUOTE] = ACTIONS(2529),
    [anon_sym_async] = ACTIONS(2529),
    [anon_sym_break] = ACTIONS(2529),
    [anon_sym_const] = ACTIONS(2529),
    [anon_sym_continue] = ACTIONS(2529),
    [anon_sym_default] = ACTIONS(2529),
    [anon_sym_enum] = ACTIONS(2529),
    [anon_sym_fn] = ACTIONS(2529),
    [anon_sym_for] = ACTIONS(2529),
    [anon_sym_gen] = ACTIONS(2529),
    [anon_sym_if] = ACTIONS(2529),
    [anon_sym_impl] = ACTIONS(2529),
    [anon_sym_let] = ACTIONS(2529),
    [anon_sym_loop] = ACTIONS(2529),
    [anon_sym_match] = ACTIONS(2529),
    [anon_sym_mod] = ACTIONS(2529),
    [anon_sym_pub] = ACTIONS(2529),
    [anon_sym_return] = ACTIONS(2529),
    [anon_sym_static] = ACTIONS(2529),
    [anon_sym_struct] = ACTIONS(2529),
    [anon_sym_trait] = ACTIONS(2529),
    [anon_sym_type] = ACTIONS(2529),
    [anon_sym_union] = ACTIONS(2529),
    [anon_sym_unsafe] = ACTIONS(2529),
    [anon_sym_use] = ACTIONS(2529),
    [anon_sym_while] = ACTIONS(2529),
    [anon_sym_extern] = ACTIONS(2529),
    [anon_sym_yield] = ACTIONS(2529),
    [anon_sym_move] = ACTIONS(2529),
    [anon_sym_try] = ACTIONS(2529),
    [sym_integer_literal] = ACTIONS(2527),
    [aux_sym_string_literal_token1] = ACTIONS(2527),
    [sym_char_literal] = ACTIONS(2527),
    [anon_sym_true] = ACTIONS(2529),
    [anon_sym_false] = ACTIONS(2529),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2529),
    [sym_super] = ACTIONS(2529),
    [sym_crate] = ACTIONS(2529),
    [sym_metavariable] = ACTIONS(2527),
    [sym__raw_string_literal_start] = ACTIONS(2527),
    [sym_float_literal] = ACTIONS(2527),
  },
  [669] = {
    [sym_line_comment] = STATE(669),
    [sym_block_comment] = STATE(669),
    [ts_builtin_sym_end] = ACTIONS(2531),
    [sym_identifier] = ACTIONS(2533),
    [anon_sym_SEMI] = ACTIONS(2531),
    [anon_sym_macro_rules_BANG] = ACTIONS(2531),
    [anon_sym_LPAREN] = ACTIONS(2531),
    [anon_sym_LBRACK] = ACTIONS(2531),
    [anon_sym_LBRACE] = ACTIONS(2531),
    [anon_sym_RBRACE] = ACTIONS(2531),
    [anon_sym_STAR] = ACTIONS(2531),
    [anon_sym_u8] = ACTIONS(2533),
    [anon_sym_i8] = ACTIONS(2533),
    [anon_sym_u16] = ACTIONS(2533),
    [anon_sym_i16] = ACTIONS(2533),
    [anon_sym_u32] = ACTIONS(2533),
    [anon_sym_i32] = ACTIONS(2533),
    [anon_sym_u64] = ACTIONS(2533),
    [anon_sym_i64] = ACTIONS(2533),
    [anon_sym_u128] = ACTIONS(2533),
    [anon_sym_i128] = ACTIONS(2533),
    [anon_sym_isize] = ACTIONS(2533),
    [anon_sym_usize] = ACTIONS(2533),
    [anon_sym_f32] = ACTIONS(2533),
    [anon_sym_f64] = ACTIONS(2533),
    [anon_sym_bool] = ACTIONS(2533),
    [anon_sym_str] = ACTIONS(2533),
    [anon_sym_char] = ACTIONS(2533),
    [anon_sym_DASH] = ACTIONS(2531),
    [anon_sym_BANG] = ACTIONS(2531),
    [anon_sym_AMP] = ACTIONS(2531),
    [anon_sym_PIPE] = ACTIONS(2531),
    [anon_sym_LT] = ACTIONS(2531),
    [anon_sym_DOT_DOT] = ACTIONS(2531),
    [anon_sym_COLON_COLON] = ACTIONS(2531),
    [anon_sym_POUND] = ACTIONS(2531),
    [anon_sym_SQUOTE] = ACTIONS(2533),
    [anon_sym_async] = ACTIONS(2533),
    [anon_sym_break] = ACTIONS(2533),
    [anon_sym_const] = ACTIONS(2533),
    [anon_sym_continue] = ACTIONS(2533),
    [anon_sym_default] = ACTIONS(2533),
    [anon_sym_enum] = ACTIONS(2533),
    [anon_sym_fn] = ACTIONS(2533),
    [anon_sym_for] = ACTIONS(2533),
    [anon_sym_gen] = ACTIONS(2533),
    [anon_sym_if] = ACTIONS(2533),
    [anon_sym_impl] = ACTIONS(2533),
    [anon_sym_let] = ACTIONS(2533),
    [anon_sym_loop] = ACTIONS(2533),
    [anon_sym_match] = ACTIONS(2533),
    [anon_sym_mod] = ACTIONS(2533),
    [anon_sym_pub] = ACTIONS(2533),
    [anon_sym_return] = ACTIONS(2533),
    [anon_sym_static] = ACTIONS(2533),
    [anon_sym_struct] = ACTIONS(2533),
    [anon_sym_trait] = ACTIONS(2533),
    [anon_sym_type] = ACTIONS(2533),
    [anon_sym_union] = ACTIONS(2533),
    [anon_sym_unsafe] = ACTIONS(2533),
    [anon_sym_use] = ACTIONS(2533),
    [anon_sym_while] = ACTIONS(2533),
    [anon_sym_extern] = ACTIONS(2533),
    [anon_sym_yield] = ACTIONS(2533),
    [anon_sym_move] = ACTIONS(2533),
    [anon_sym_try] = ACTIONS(2533),
    [sym_integer_literal] = ACTIONS(2531),
    [aux_sym_string_literal_token1] = ACTIONS(2531),
    [sym_char_literal] = ACTIONS(2531),
    [anon_sym_true] = ACTIONS(2533),
    [anon_sym_false] = ACTIONS(2533),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2533),
    [sym_super] = ACTIONS(2533),
    [sym_crate] = ACTIONS(2533),
    [sym_metavariable] = ACTIONS(2531),
    [sym__raw_string_literal_start] = ACTIONS(2531),
    [sym_float_literal] = ACTIONS(2531),
  },
  [670] = {
    [sym_line_comment] = STATE(670),
    [sym_block_comment] = STATE(670),
    [ts_builtin_sym_end] = ACTIONS(2535),
    [sym_identifier] = ACTIONS(2537),
    [anon_sym_SEMI] = ACTIONS(2535),
    [anon_sym_macro_rules_BANG] = ACTIONS(2535),
    [anon_sym_LPAREN] = ACTIONS(2535),
    [anon_sym_LBRACK] = ACTIONS(2535),
    [anon_sym_LBRACE] = ACTIONS(2535),
    [anon_sym_RBRACE] = ACTIONS(2535),
    [anon_sym_STAR] = ACTIONS(2535),
    [anon_sym_u8] = ACTIONS(2537),
    [anon_sym_i8] = ACTIONS(2537),
    [anon_sym_u16] = ACTIONS(2537),
    [anon_sym_i16] = ACTIONS(2537),
    [anon_sym_u32] = ACTIONS(2537),
    [anon_sym_i32] = ACTIONS(2537),
    [anon_sym_u64] = ACTIONS(2537),
    [anon_sym_i64] = ACTIONS(2537),
    [anon_sym_u128] = ACTIONS(2537),
    [anon_sym_i128] = ACTIONS(2537),
    [anon_sym_isize] = ACTIONS(2537),
    [anon_sym_usize] = ACTIONS(2537),
    [anon_sym_f32] = ACTIONS(2537),
    [anon_sym_f64] = ACTIONS(2537),
    [anon_sym_bool] = ACTIONS(2537),
    [anon_sym_str] = ACTIONS(2537),
    [anon_sym_char] = ACTIONS(2537),
    [anon_sym_DASH] = ACTIONS(2535),
    [anon_sym_BANG] = ACTIONS(2535),
    [anon_sym_AMP] = ACTIONS(2535),
    [anon_sym_PIPE] = ACTIONS(2535),
    [anon_sym_LT] = ACTIONS(2535),
    [anon_sym_DOT_DOT] = ACTIONS(2535),
    [anon_sym_COLON_COLON] = ACTIONS(2535),
    [anon_sym_POUND] = ACTIONS(2535),
    [anon_sym_SQUOTE] = ACTIONS(2537),
    [anon_sym_async] = ACTIONS(2537),
    [anon_sym_break] = ACTIONS(2537),
    [anon_sym_const] = ACTIONS(2537),
    [anon_sym_continue] = ACTIONS(2537),
    [anon_sym_default] = ACTIONS(2537),
    [anon_sym_enum] = ACTIONS(2537),
    [anon_sym_fn] = ACTIONS(2537),
    [anon_sym_for] = ACTIONS(2537),
    [anon_sym_gen] = ACTIONS(2537),
    [anon_sym_if] = ACTIONS(2537),
    [anon_sym_impl] = ACTIONS(2537),
    [anon_sym_let] = ACTIONS(2537),
    [anon_sym_loop] = ACTIONS(2537),
    [anon_sym_match] = ACTIONS(2537),
    [anon_sym_mod] = ACTIONS(2537),
    [anon_sym_pub] = ACTIONS(2537),
    [anon_sym_return] = ACTIONS(2537),
    [anon_sym_static] = ACTIONS(2537),
    [anon_sym_struct] = ACTIONS(2537),
    [anon_sym_trait] = ACTIONS(2537),
    [anon_sym_type] = ACTIONS(2537),
    [anon_sym_union] = ACTIONS(2537),
    [anon_sym_unsafe] = ACTIONS(2537),
    [anon_sym_use] = ACTIONS(2537),
    [anon_sym_while] = ACTIONS(2537),
    [anon_sym_extern] = ACTIONS(2537),
    [anon_sym_yield] = ACTIONS(2537),
    [anon_sym_move] = ACTIONS(2537),
    [anon_sym_try] = ACTIONS(2537),
    [sym_integer_literal] = ACTIONS(2535),
    [aux_sym_string_literal_token1] = ACTIONS(2535),
    [sym_char_literal] = ACTIONS(2535),
    [anon_sym_true] = ACTIONS(2537),
    [anon_sym_false] = ACTIONS(2537),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2537),
    [sym_super] = ACTIONS(2537),
    [sym_crate] = ACTIONS(2537),
    [sym_metavariable] = ACTIONS(2535),
    [sym__raw_string_literal_start] = ACTIONS(2535),
    [sym_float_literal] = ACTIONS(2535),
  },
  [671] = {
    [sym_line_comment] = STATE(671),
    [sym_block_comment] = STATE(671),
    [ts_builtin_sym_end] = ACTIONS(2539),
    [sym_identifier] = ACTIONS(2541),
    [anon_sym_SEMI] = ACTIONS(2539),
    [anon_sym_macro_rules_BANG] = ACTIONS(2539),
    [anon_sym_LPAREN] = ACTIONS(2539),
    [anon_sym_LBRACK] = ACTIONS(2539),
    [anon_sym_LBRACE] = ACTIONS(2539),
    [anon_sym_RBRACE] = ACTIONS(2539),
    [anon_sym_STAR] = ACTIONS(2539),
    [anon_sym_u8] = ACTIONS(2541),
    [anon_sym_i8] = ACTIONS(2541),
    [anon_sym_u16] = ACTIONS(2541),
    [anon_sym_i16] = ACTIONS(2541),
    [anon_sym_u32] = ACTIONS(2541),
    [anon_sym_i32] = ACTIONS(2541),
    [anon_sym_u64] = ACTIONS(2541),
    [anon_sym_i64] = ACTIONS(2541),
    [anon_sym_u128] = ACTIONS(2541),
    [anon_sym_i128] = ACTIONS(2541),
    [anon_sym_isize] = ACTIONS(2541),
    [anon_sym_usize] = ACTIONS(2541),
    [anon_sym_f32] = ACTIONS(2541),
    [anon_sym_f64] = ACTIONS(2541),
    [anon_sym_bool] = ACTIONS(2541),
    [anon_sym_str] = ACTIONS(2541),
    [anon_sym_char] = ACTIONS(2541),
    [anon_sym_DASH] = ACTIONS(2539),
    [anon_sym_BANG] = ACTIONS(2539),
    [anon_sym_AMP] = ACTIONS(2539),
    [anon_sym_PIPE] = ACTIONS(2539),
    [anon_sym_LT] = ACTIONS(2539),
    [anon_sym_DOT_DOT] = ACTIONS(2539),
    [anon_sym_COLON_COLON] = ACTIONS(2539),
    [anon_sym_POUND] = ACTIONS(2539),
    [anon_sym_SQUOTE] = ACTIONS(2541),
    [anon_sym_async] = ACTIONS(2541),
    [anon_sym_break] = ACTIONS(2541),
    [anon_sym_const] = ACTIONS(2541),
    [anon_sym_continue] = ACTIONS(2541),
    [anon_sym_default] = ACTIONS(2541),
    [anon_sym_enum] = ACTIONS(2541),
    [anon_sym_fn] = ACTIONS(2541),
    [anon_sym_for] = ACTIONS(2541),
    [anon_sym_gen] = ACTIONS(2541),
    [anon_sym_if] = ACTIONS(2541),
    [anon_sym_impl] = ACTIONS(2541),
    [anon_sym_let] = ACTIONS(2541),
    [anon_sym_loop] = ACTIONS(2541),
    [anon_sym_match] = ACTIONS(2541),
    [anon_sym_mod] = ACTIONS(2541),
    [anon_sym_pub] = ACTIONS(2541),
    [anon_sym_return] = ACTIONS(2541),
    [anon_sym_static] = ACTIONS(2541),
    [anon_sym_struct] = ACTIONS(2541),
    [anon_sym_trait] = ACTIONS(2541),
    [anon_sym_type] = ACTIONS(2541),
    [anon_sym_union] = ACTIONS(2541),
    [anon_sym_unsafe] = ACTIONS(2541),
    [anon_sym_use] = ACTIONS(2541),
    [anon_sym_while] = ACTIONS(2541),
    [anon_sym_extern] = ACTIONS(2541),
    [anon_sym_yield] = ACTIONS(2541),
    [anon_sym_move] = ACTIONS(2541),
    [anon_sym_try] = ACTIONS(2541),
    [sym_integer_literal] = ACTIONS(2539),
    [aux_sym_string_literal_token1] = ACTIONS(2539),
    [sym_char_literal] = ACTIONS(2539),
    [anon_sym_true] = ACTIONS(2541),
    [anon_sym_false] = ACTIONS(2541),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2541),
    [sym_super] = ACTIONS(2541),
    [sym_crate] = ACTIONS(2541),
    [sym_metavariable] = ACTIONS(2539),
    [sym__raw_string_literal_start] = ACTIONS(2539),
    [sym_float_literal] = ACTIONS(2539),
  },
  [672] = {
    [sym_line_comment] = STATE(672),
    [sym_block_comment] = STATE(672),
    [ts_builtin_sym_end] = ACTIONS(2543),
    [sym_identifier] = ACTIONS(2545),
    [anon_sym_SEMI] = ACTIONS(2543),
    [anon_sym_macro_rules_BANG] = ACTIONS(2543),
    [anon_sym_LPAREN] = ACTIONS(2543),
    [anon_sym_LBRACK] = ACTIONS(2543),
    [anon_sym_LBRACE] = ACTIONS(2543),
    [anon_sym_RBRACE] = ACTIONS(2543),
    [anon_sym_STAR] = ACTIONS(2543),
    [anon_sym_u8] = ACTIONS(2545),
    [anon_sym_i8] = ACTIONS(2545),
    [anon_sym_u16] = ACTIONS(2545),
    [anon_sym_i16] = ACTIONS(2545),
    [anon_sym_u32] = ACTIONS(2545),
    [anon_sym_i32] = ACTIONS(2545),
    [anon_sym_u64] = ACTIONS(2545),
    [anon_sym_i64] = ACTIONS(2545),
    [anon_sym_u128] = ACTIONS(2545),
    [anon_sym_i128] = ACTIONS(2545),
    [anon_sym_isize] = ACTIONS(2545),
    [anon_sym_usize] = ACTIONS(2545),
    [anon_sym_f32] = ACTIONS(2545),
    [anon_sym_f64] = ACTIONS(2545),
    [anon_sym_bool] = ACTIONS(2545),
    [anon_sym_str] = ACTIONS(2545),
    [anon_sym_char] = ACTIONS(2545),
    [anon_sym_DASH] = ACTIONS(2543),
    [anon_sym_BANG] = ACTIONS(2543),
    [anon_sym_AMP] = ACTIONS(2543),
    [anon_sym_PIPE] = ACTIONS(2543),
    [anon_sym_LT] = ACTIONS(2543),
    [anon_sym_DOT_DOT] = ACTIONS(2543),
    [anon_sym_COLON_COLON] = ACTIONS(2543),
    [anon_sym_POUND] = ACTIONS(2543),
    [anon_sym_SQUOTE] = ACTIONS(2545),
    [anon_sym_async] = ACTIONS(2545),
    [anon_sym_break] = ACTIONS(2545),
    [anon_sym_const] = ACTIONS(2545),
    [anon_sym_continue] = ACTIONS(2545),
    [anon_sym_default] = ACTIONS(2545),
    [anon_sym_enum] = ACTIONS(2545),
    [anon_sym_fn] = ACTIONS(2545),
    [anon_sym_for] = ACTIONS(2545),
    [anon_sym_gen] = ACTIONS(2545),
    [anon_sym_if] = ACTIONS(2545),
    [anon_sym_impl] = ACTIONS(2545),
    [anon_sym_let] = ACTIONS(2545),
    [anon_sym_loop] = ACTIONS(2545),
    [anon_sym_match] = ACTIONS(2545),
    [anon_sym_mod] = ACTIONS(2545),
    [anon_sym_pub] = ACTIONS(2545),
    [anon_sym_return] = ACTIONS(2545),
    [anon_sym_static] = ACTIONS(2545),
    [anon_sym_struct] = ACTIONS(2545),
    [anon_sym_trait] = ACTIONS(2545),
    [anon_sym_type] = ACTIONS(2545),
    [anon_sym_union] = ACTIONS(2545),
    [anon_sym_unsafe] = ACTIONS(2545),
    [anon_sym_use] = ACTIONS(2545),
    [anon_sym_while] = ACTIONS(2545),
    [anon_sym_extern] = ACTIONS(2545),
    [anon_sym_yield] = ACTIONS(2545),
    [anon_sym_move] = ACTIONS(2545),
    [anon_sym_try] = ACTIONS(2545),
    [sym_integer_literal] = ACTIONS(2543),
    [aux_sym_string_literal_token1] = ACTIONS(2543),
    [sym_char_literal] = ACTIONS(2543),
    [anon_sym_true] = ACTIONS(2545),
    [anon_sym_false] = ACTIONS(2545),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2545),
    [sym_super] = ACTIONS(2545),
    [sym_crate] = ACTIONS(2545),
    [sym_metavariable] = ACTIONS(2543),
    [sym__raw_string_literal_start] = ACTIONS(2543),
    [sym_float_literal] = ACTIONS(2543),
  },
  [673] = {
    [sym_line_comment] = STATE(673),
    [sym_block_comment] = STATE(673),
    [ts_builtin_sym_end] = ACTIONS(2547),
    [sym_identifier] = ACTIONS(2549),
    [anon_sym_SEMI] = ACTIONS(2547),
    [anon_sym_macro_rules_BANG] = ACTIONS(2547),
    [anon_sym_LPAREN] = ACTIONS(2547),
    [anon_sym_LBRACK] = ACTIONS(2547),
    [anon_sym_LBRACE] = ACTIONS(2547),
    [anon_sym_RBRACE] = ACTIONS(2547),
    [anon_sym_STAR] = ACTIONS(2547),
    [anon_sym_u8] = ACTIONS(2549),
    [anon_sym_i8] = ACTIONS(2549),
    [anon_sym_u16] = ACTIONS(2549),
    [anon_sym_i16] = ACTIONS(2549),
    [anon_sym_u32] = ACTIONS(2549),
    [anon_sym_i32] = ACTIONS(2549),
    [anon_sym_u64] = ACTIONS(2549),
    [anon_sym_i64] = ACTIONS(2549),
    [anon_sym_u128] = ACTIONS(2549),
    [anon_sym_i128] = ACTIONS(2549),
    [anon_sym_isize] = ACTIONS(2549),
    [anon_sym_usize] = ACTIONS(2549),
    [anon_sym_f32] = ACTIONS(2549),
    [anon_sym_f64] = ACTIONS(2549),
    [anon_sym_bool] = ACTIONS(2549),
    [anon_sym_str] = ACTIONS(2549),
    [anon_sym_char] = ACTIONS(2549),
    [anon_sym_DASH] = ACTIONS(2547),
    [anon_sym_BANG] = ACTIONS(2547),
    [anon_sym_AMP] = ACTIONS(2547),
    [anon_sym_PIPE] = ACTIONS(2547),
    [anon_sym_LT] = ACTIONS(2547),
    [anon_sym_DOT_DOT] = ACTIONS(2547),
    [anon_sym_COLON_COLON] = ACTIONS(2547),
    [anon_sym_POUND] = ACTIONS(2547),
    [anon_sym_SQUOTE] = ACTIONS(2549),
    [anon_sym_async] = ACTIONS(2549),
    [anon_sym_break] = ACTIONS(2549),
    [anon_sym_const] = ACTIONS(2549),
    [anon_sym_continue] = ACTIONS(2549),
    [anon_sym_default] = ACTIONS(2549),
    [anon_sym_enum] = ACTIONS(2549),
    [anon_sym_fn] = ACTIONS(2549),
    [anon_sym_for] = ACTIONS(2549),
    [anon_sym_gen] = ACTIONS(2549),
    [anon_sym_if] = ACTIONS(2549),
    [anon_sym_impl] = ACTIONS(2549),
    [anon_sym_let] = ACTIONS(2549),
    [anon_sym_loop] = ACTIONS(2549),
    [anon_sym_match] = ACTIONS(2549),
    [anon_sym_mod] = ACTIONS(2549),
    [anon_sym_pub] = ACTIONS(2549),
    [anon_sym_return] = ACTIONS(2549),
    [anon_sym_static] = ACTIONS(2549),
    [anon_sym_struct] = ACTIONS(2549),
    [anon_sym_trait] = ACTIONS(2549),
    [anon_sym_type] = ACTIONS(2549),
    [anon_sym_union] = ACTIONS(2549),
    [anon_sym_unsafe] = ACTIONS(2549),
    [anon_sym_use] = ACTIONS(2549),
    [anon_sym_while] = ACTIONS(2549),
    [anon_sym_extern] = ACTIONS(2549),
    [anon_sym_yield] = ACTIONS(2549),
    [anon_sym_move] = ACTIONS(2549),
    [anon_sym_try] = ACTIONS(2549),
    [sym_integer_literal] = ACTIONS(2547),
    [aux_sym_string_literal_token1] = ACTIONS(2547),
    [sym_char_literal] = ACTIONS(2547),
    [anon_sym_true] = ACTIONS(2549),
    [anon_sym_false] = ACTIONS(2549),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2549),
    [sym_super] = ACTIONS(2549),
    [sym_crate] = ACTIONS(2549),
    [sym_metavariable] = ACTIONS(2547),
    [sym__raw_string_literal_start] = ACTIONS(2547),
    [sym_float_literal] = ACTIONS(2547),
  },
  [674] = {
    [sym_empty_statement] = STATE(1072),
    [sym_macro_definition] = STATE(1072),
    [sym_attribute_item] = STATE(1072),
    [sym_inner_attribute_item] = STATE(1072),
    [sym_mod_item] = STATE(1072),
    [sym_foreign_mod_item] = STATE(1072),
    [sym_struct_item] = STATE(1072),
    [sym_union_item] = STATE(1072),
    [sym_enum_item] = STATE(1072),
    [sym_extern_crate_declaration] = STATE(1072),
    [sym_const_item] = STATE(1072),
    [sym_static_item] = STATE(1072),
    [sym_type_item] = STATE(1072),
    [sym_function_item] = STATE(1072),
    [sym_function_signature_item] = STATE(1072),
    [sym_function_modifiers] = STATE(3622),
    [sym_impl_item] = STATE(1072),
    [sym_trait_item] = STATE(1072),
    [sym_associated_type] = STATE(1072),
    [sym_let_declaration] = STATE(1072),
    [sym_use_declaration] = STATE(1072),
    [sym_extern_modifier] = STATE(2165),
    [sym_visibility_modifier] = STATE(1954),
    [sym_bracketed_type] = STATE(3353),
    [sym_generic_type_with_turbofish] = STATE(3379),
    [sym_macro_invocation] = STATE(1072),
    [sym_scoped_identifier] = STATE(3167),
    [sym_line_comment] = STATE(674),
    [sym_block_comment] = STATE(674),
    [aux_sym_declaration_list_repeat1] = STATE(700),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(2461),
    [anon_sym_SEMI] = ACTIONS(2463),
    [anon_sym_macro_rules_BANG] = ACTIONS(2465),
    [anon_sym_RBRACE] = ACTIONS(2551),
    [anon_sym_u8] = ACTIONS(2469),
    [anon_sym_i8] = ACTIONS(2469),
    [anon_sym_u16] = ACTIONS(2469),
    [anon_sym_i16] = ACTIONS(2469),
    [anon_sym_u32] = ACTIONS(2469),
    [anon_sym_i32] = ACTIONS(2469),
    [anon_sym_u64] = ACTIONS(2469),
    [anon_sym_i64] = ACTIONS(2469),
    [anon_sym_u128] = ACTIONS(2469),
    [anon_sym_i128] = ACTIONS(2469),
    [anon_sym_isize] = ACTIONS(2469),
    [anon_sym_usize] = ACTIONS(2469),
    [anon_sym_f32] = ACTIONS(2469),
    [anon_sym_f64] = ACTIONS(2469),
    [anon_sym_bool] = ACTIONS(2469),
    [anon_sym_str] = ACTIONS(2469),
    [anon_sym_char] = ACTIONS(2469),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(2471),
    [anon_sym_POUND] = ACTIONS(2473),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(2475),
    [anon_sym_default] = ACTIONS(2477),
    [anon_sym_enum] = ACTIONS(2479),
    [anon_sym_fn] = ACTIONS(2481),
    [anon_sym_gen] = ACTIONS(2483),
    [anon_sym_impl] = ACTIONS(2485),
    [anon_sym_let] = ACTIONS(2487),
    [anon_sym_mod] = ACTIONS(2489),
    [anon_sym_pub] = ACTIONS(69),
    [anon_sym_static] = ACTIONS(2491),
    [anon_sym_struct] = ACTIONS(2493),
    [anon_sym_trait] = ACTIONS(2495),
    [anon_sym_type] = ACTIONS(2497),
    [anon_sym_union] = ACTIONS(2499),
    [anon_sym_unsafe] = ACTIONS(2501),
    [anon_sym_use] = ACTIONS(2503),
    [anon_sym_extern] = ACTIONS(2505),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2507),
    [sym_super] = ACTIONS(2507),
    [sym_crate] = ACTIONS(2509),
    [sym_metavariable] = ACTIONS(2511),
  },
  [675] = {
    [sym_line_comment] = STATE(675),
    [sym_block_comment] = STATE(675),
    [ts_builtin_sym_end] = ACTIONS(2553),
    [sym_identifier] = ACTIONS(2555),
    [anon_sym_SEMI] = ACTIONS(2553),
    [anon_sym_macro_rules_BANG] = ACTIONS(2553),
    [anon_sym_LPAREN] = ACTIONS(2553),
    [anon_sym_LBRACK] = ACTIONS(2553),
    [anon_sym_LBRACE] = ACTIONS(2553),
    [anon_sym_RBRACE] = ACTIONS(2553),
    [anon_sym_STAR] = ACTIONS(2553),
    [anon_sym_u8] = ACTIONS(2555),
    [anon_sym_i8] = ACTIONS(2555),
    [anon_sym_u16] = ACTIONS(2555),
    [anon_sym_i16] = ACTIONS(2555),
    [anon_sym_u32] = ACTIONS(2555),
    [anon_sym_i32] = ACTIONS(2555),
    [anon_sym_u64] = ACTIONS(2555),
    [anon_sym_i64] = ACTIONS(2555),
    [anon_sym_u128] = ACTIONS(2555),
    [anon_sym_i128] = ACTIONS(2555),
    [anon_sym_isize] = ACTIONS(2555),
    [anon_sym_usize] = ACTIONS(2555),
    [anon_sym_f32] = ACTIONS(2555),
    [anon_sym_f64] = ACTIONS(2555),
    [anon_sym_bool] = ACTIONS(2555),
    [anon_sym_str] = ACTIONS(2555),
    [anon_sym_char] = ACTIONS(2555),
    [anon_sym_DASH] = ACTIONS(2553),
    [anon_sym_BANG] = ACTIONS(2553),
    [anon_sym_AMP] = ACTIONS(2553),
    [anon_sym_PIPE] = ACTIONS(2553),
    [anon_sym_LT] = ACTIONS(2553),
    [anon_sym_DOT_DOT] = ACTIONS(2553),
    [anon_sym_COLON_COLON] = ACTIONS(2553),
    [anon_sym_POUND] = ACTIONS(2553),
    [anon_sym_SQUOTE] = ACTIONS(2555),
    [anon_sym_async] = ACTIONS(2555),
    [anon_sym_break] = ACTIONS(2555),
    [anon_sym_const] = ACTIONS(2555),
    [anon_sym_continue] = ACTIONS(2555),
    [anon_sym_default] = ACTIONS(2555),
    [anon_sym_enum] = ACTIONS(2555),
    [anon_sym_fn] = ACTIONS(2555),
    [anon_sym_for] = ACTIONS(2555),
    [anon_sym_gen] = ACTIONS(2555),
    [anon_sym_if] = ACTIONS(2555),
    [anon_sym_impl] = ACTIONS(2555),
    [anon_sym_let] = ACTIONS(2555),
    [anon_sym_loop] = ACTIONS(2555),
    [anon_sym_match] = ACTIONS(2555),
    [anon_sym_mod] = ACTIONS(2555),
    [anon_sym_pub] = ACTIONS(2555),
    [anon_sym_return] = ACTIONS(2555),
    [anon_sym_static] = ACTIONS(2555),
    [anon_sym_struct] = ACTIONS(2555),
    [anon_sym_trait] = ACTIONS(2555),
    [anon_sym_type] = ACTIONS(2555),
    [anon_sym_union] = ACTIONS(2555),
    [anon_sym_unsafe] = ACTIONS(2555),
    [anon_sym_use] = ACTIONS(2555),
    [anon_sym_while] = ACTIONS(2555),
    [anon_sym_extern] = ACTIONS(2555),
    [anon_sym_yield] = ACTIONS(2555),
    [anon_sym_move] = ACTIONS(2555),
    [anon_sym_try] = ACTIONS(2555),
    [sym_integer_literal] = ACTIONS(2553),
    [aux_sym_string_literal_token1] = ACTIONS(2553),
    [sym_char_literal] = ACTIONS(2553),
    [anon_sym_true] = ACTIONS(2555),
    [anon_sym_false] = ACTIONS(2555),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2555),
    [sym_super] = ACTIONS(2555),
    [sym_crate] = ACTIONS(2555),
    [sym_metavariable] = ACTIONS(2553),
    [sym__raw_string_literal_start] = ACTIONS(2553),
    [sym_float_literal] = ACTIONS(2553),
  },
  [676] = {
    [sym_line_comment] = STATE(676),
    [sym_block_comment] = STATE(676),
    [ts_builtin_sym_end] = ACTIONS(2557),
    [sym_identifier] = ACTIONS(2559),
    [anon_sym_SEMI] = ACTIONS(2557),
    [anon_sym_macro_rules_BANG] = ACTIONS(2557),
    [anon_sym_LPAREN] = ACTIONS(2557),
    [anon_sym_LBRACK] = ACTIONS(2557),
    [anon_sym_LBRACE] = ACTIONS(2557),
    [anon_sym_RBRACE] = ACTIONS(2557),
    [anon_sym_STAR] = ACTIONS(2557),
    [anon_sym_u8] = ACTIONS(2559),
    [anon_sym_i8] = ACTIONS(2559),
    [anon_sym_u16] = ACTIONS(2559),
    [anon_sym_i16] = ACTIONS(2559),
    [anon_sym_u32] = ACTIONS(2559),
    [anon_sym_i32] = ACTIONS(2559),
    [anon_sym_u64] = ACTIONS(2559),
    [anon_sym_i64] = ACTIONS(2559),
    [anon_sym_u128] = ACTIONS(2559),
    [anon_sym_i128] = ACTIONS(2559),
    [anon_sym_isize] = ACTIONS(2559),
    [anon_sym_usize] = ACTIONS(2559),
    [anon_sym_f32] = ACTIONS(2559),
    [anon_sym_f64] = ACTIONS(2559),
    [anon_sym_bool] = ACTIONS(2559),
    [anon_sym_str] = ACTIONS(2559),
    [anon_sym_char] = ACTIONS(2559),
    [anon_sym_DASH] = ACTIONS(2557),
    [anon_sym_BANG] = ACTIONS(2557),
    [anon_sym_AMP] = ACTIONS(2557),
    [anon_sym_PIPE] = ACTIONS(2557),
    [anon_sym_LT] = ACTIONS(2557),
    [anon_sym_DOT_DOT] = ACTIONS(2557),
    [anon_sym_COLON_COLON] = ACTIONS(2557),
    [anon_sym_POUND] = ACTIONS(2557),
    [anon_sym_SQUOTE] = ACTIONS(2559),
    [anon_sym_async] = ACTIONS(2559),
    [anon_sym_break] = ACTIONS(2559),
    [anon_sym_const] = ACTIONS(2559),
    [anon_sym_continue] = ACTIONS(2559),
    [anon_sym_default] = ACTIONS(2559),
    [anon_sym_enum] = ACTIONS(2559),
    [anon_sym_fn] = ACTIONS(2559),
    [anon_sym_for] = ACTIONS(2559),
    [anon_sym_gen] = ACTIONS(2559),
    [anon_sym_if] = ACTIONS(2559),
    [anon_sym_impl] = ACTIONS(2559),
    [anon_sym_let] = ACTIONS(2559),
    [anon_sym_loop] = ACTIONS(2559),
    [anon_sym_match] = ACTIONS(2559),
    [anon_sym_mod] = ACTIONS(2559),
    [anon_sym_pub] = ACTIONS(2559),
    [anon_sym_return] = ACTIONS(2559),
    [anon_sym_static] = ACTIONS(2559),
    [anon_sym_struct] = ACTIONS(2559),
    [anon_sym_trait] = ACTIONS(2559),
    [anon_sym_type] = ACTIONS(2559),
    [anon_sym_union] = ACTIONS(2559),
    [anon_sym_unsafe] = ACTIONS(2559),
    [anon_sym_use] = ACTIONS(2559),
    [anon_sym_while] = ACTIONS(2559),
    [anon_sym_extern] = ACTIONS(2559),
    [anon_sym_yield] = ACTIONS(2559),
    [anon_sym_move] = ACTIONS(2559),
    [anon_sym_try] = ACTIONS(2559),
    [sym_integer_literal] = ACTIONS(2557),
    [aux_sym_string_literal_token1] = ACTIONS(2557),
    [sym_char_literal] = ACTIONS(2557),
    [anon_sym_true] = ACTIONS(2559),
    [anon_sym_false] = ACTIONS(2559),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2559),
    [sym_super] = ACTIONS(2559),
    [sym_crate] = ACTIONS(2559),
    [sym_metavariable] = ACTIONS(2557),
    [sym__raw_string_literal_start] = ACTIONS(2557),
    [sym_float_literal] = ACTIONS(2557),
  },
  [677] = {
    [sym_line_comment] = STATE(677),
    [sym_block_comment] = STATE(677),
    [ts_builtin_sym_end] = ACTIONS(2561),
    [sym_identifier] = ACTIONS(2563),
    [anon_sym_SEMI] = ACTIONS(2561),
    [anon_sym_macro_rules_BANG] = ACTIONS(2561),
    [anon_sym_LPAREN] = ACTIONS(2561),
    [anon_sym_LBRACK] = ACTIONS(2561),
    [anon_sym_LBRACE] = ACTIONS(2561),
    [anon_sym_RBRACE] = ACTIONS(2561),
    [anon_sym_STAR] = ACTIONS(2561),
    [anon_sym_u8] = ACTIONS(2563),
    [anon_sym_i8] = ACTIONS(2563),
    [anon_sym_u16] = ACTIONS(2563),
    [anon_sym_i16] = ACTIONS(2563),
    [anon_sym_u32] = ACTIONS(2563),
    [anon_sym_i32] = ACTIONS(2563),
    [anon_sym_u64] = ACTIONS(2563),
    [anon_sym_i64] = ACTIONS(2563),
    [anon_sym_u128] = ACTIONS(2563),
    [anon_sym_i128] = ACTIONS(2563),
    [anon_sym_isize] = ACTIONS(2563),
    [anon_sym_usize] = ACTIONS(2563),
    [anon_sym_f32] = ACTIONS(2563),
    [anon_sym_f64] = ACTIONS(2563),
    [anon_sym_bool] = ACTIONS(2563),
    [anon_sym_str] = ACTIONS(2563),
    [anon_sym_char] = ACTIONS(2563),
    [anon_sym_DASH] = ACTIONS(2561),
    [anon_sym_BANG] = ACTIONS(2561),
    [anon_sym_AMP] = ACTIONS(2561),
    [anon_sym_PIPE] = ACTIONS(2561),
    [anon_sym_LT] = ACTIONS(2561),
    [anon_sym_DOT_DOT] = ACTIONS(2561),
    [anon_sym_COLON_COLON] = ACTIONS(2561),
    [anon_sym_POUND] = ACTIONS(2561),
    [anon_sym_SQUOTE] = ACTIONS(2563),
    [anon_sym_async] = ACTIONS(2563),
    [anon_sym_break] = ACTIONS(2563),
    [anon_sym_const] = ACTIONS(2563),
    [anon_sym_continue] = ACTIONS(2563),
    [anon_sym_default] = ACTIONS(2563),
    [anon_sym_enum] = ACTIONS(2563),
    [anon_sym_fn] = ACTIONS(2563),
    [anon_sym_for] = ACTIONS(2563),
    [anon_sym_gen] = ACTIONS(2563),
    [anon_sym_if] = ACTIONS(2563),
    [anon_sym_impl] = ACTIONS(2563),
    [anon_sym_let] = ACTIONS(2563),
    [anon_sym_loop] = ACTIONS(2563),
    [anon_sym_match] = ACTIONS(2563),
    [anon_sym_mod] = ACTIONS(2563),
    [anon_sym_pub] = ACTIONS(2563),
    [anon_sym_return] = ACTIONS(2563),
    [anon_sym_static] = ACTIONS(2563),
    [anon_sym_struct] = ACTIONS(2563),
    [anon_sym_trait] = ACTIONS(2563),
    [anon_sym_type] = ACTIONS(2563),
    [anon_sym_union] = ACTIONS(2563),
    [anon_sym_unsafe] = ACTIONS(2563),
    [anon_sym_use] = ACTIONS(2563),
    [anon_sym_while] = ACTIONS(2563),
    [anon_sym_extern] = ACTIONS(2563),
    [anon_sym_yield] = ACTIONS(2563),
    [anon_sym_move] = ACTIONS(2563),
    [anon_sym_try] = ACTIONS(2563),
    [sym_integer_literal] = ACTIONS(2561),
    [aux_sym_string_literal_token1] = ACTIONS(2561),
    [sym_char_literal] = ACTIONS(2561),
    [anon_sym_true] = ACTIONS(2563),
    [anon_sym_false] = ACTIONS(2563),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2563),
    [sym_super] = ACTIONS(2563),
    [sym_crate] = ACTIONS(2563),
    [sym_metavariable] = ACTIONS(2561),
    [sym__raw_string_literal_start] = ACTIONS(2561),
    [sym_float_literal] = ACTIONS(2561),
  },
  [678] = {
    [sym_line_comment] = STATE(678),
    [sym_block_comment] = STATE(678),
    [ts_builtin_sym_end] = ACTIONS(2565),
    [sym_identifier] = ACTIONS(2567),
    [anon_sym_SEMI] = ACTIONS(2565),
    [anon_sym_macro_rules_BANG] = ACTIONS(2565),
    [anon_sym_LPAREN] = ACTIONS(2565),
    [anon_sym_LBRACK] = ACTIONS(2565),
    [anon_sym_LBRACE] = ACTIONS(2565),
    [anon_sym_RBRACE] = ACTIONS(2565),
    [anon_sym_STAR] = ACTIONS(2565),
    [anon_sym_u8] = ACTIONS(2567),
    [anon_sym_i8] = ACTIONS(2567),
    [anon_sym_u16] = ACTIONS(2567),
    [anon_sym_i16] = ACTIONS(2567),
    [anon_sym_u32] = ACTIONS(2567),
    [anon_sym_i32] = ACTIONS(2567),
    [anon_sym_u64] = ACTIONS(2567),
    [anon_sym_i64] = ACTIONS(2567),
    [anon_sym_u128] = ACTIONS(2567),
    [anon_sym_i128] = ACTIONS(2567),
    [anon_sym_isize] = ACTIONS(2567),
    [anon_sym_usize] = ACTIONS(2567),
    [anon_sym_f32] = ACTIONS(2567),
    [anon_sym_f64] = ACTIONS(2567),
    [anon_sym_bool] = ACTIONS(2567),
    [anon_sym_str] = ACTIONS(2567),
    [anon_sym_char] = ACTIONS(2567),
    [anon_sym_DASH] = ACTIONS(2565),
    [anon_sym_BANG] = ACTIONS(2565),
    [anon_sym_AMP] = ACTIONS(2565),
    [anon_sym_PIPE] = ACTIONS(2565),
    [anon_sym_LT] = ACTIONS(2565),
    [anon_sym_DOT_DOT] = ACTIONS(2565),
    [anon_sym_COLON_COLON] = ACTIONS(2565),
    [anon_sym_POUND] = ACTIONS(2565),
    [anon_sym_SQUOTE] = ACTIONS(2567),
    [anon_sym_async] = ACTIONS(2567),
    [anon_sym_break] = ACTIONS(2567),
    [anon_sym_const] = ACTIONS(2567),
    [anon_sym_continue] = ACTIONS(2567),
    [anon_sym_default] = ACTIONS(2567),
    [anon_sym_enum] = ACTIONS(2567),
    [anon_sym_fn] = ACTIONS(2567),
    [anon_sym_for] = ACTIONS(2567),
    [anon_sym_gen] = ACTIONS(2567),
    [anon_sym_if] = ACTIONS(2567),
    [anon_sym_impl] = ACTIONS(2567),
    [anon_sym_let] = ACTIONS(2567),
    [anon_sym_loop] = ACTIONS(2567),
    [anon_sym_match] = ACTIONS(2567),
    [anon_sym_mod] = ACTIONS(2567),
    [anon_sym_pub] = ACTIONS(2567),
    [anon_sym_return] = ACTIONS(2567),
    [anon_sym_static] = ACTIONS(2567),
    [anon_sym_struct] = ACTIONS(2567),
    [anon_sym_trait] = ACTIONS(2567),
    [anon_sym_type] = ACTIONS(2567),
    [anon_sym_union] = ACTIONS(2567),
    [anon_sym_unsafe] = ACTIONS(2567),
    [anon_sym_use] = ACTIONS(2567),
    [anon_sym_while] = ACTIONS(2567),
    [anon_sym_extern] = ACTIONS(2567),
    [anon_sym_yield] = ACTIONS(2567),
    [anon_sym_move] = ACTIONS(2567),
    [anon_sym_try] = ACTIONS(2567),
    [sym_integer_literal] = ACTIONS(2565),
    [aux_sym_string_literal_token1] = ACTIONS(2565),
    [sym_char_literal] = ACTIONS(2565),
    [anon_sym_true] = ACTIONS(2567),
    [anon_sym_false] = ACTIONS(2567),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2567),
    [sym_super] = ACTIONS(2567),
    [sym_crate] = ACTIONS(2567),
    [sym_metavariable] = ACTIONS(2565),
    [sym__raw_string_literal_start] = ACTIONS(2565),
    [sym_float_literal] = ACTIONS(2565),
  },
  [679] = {
    [sym_line_comment] = STATE(679),
    [sym_block_comment] = STATE(679),
    [ts_builtin_sym_end] = ACTIONS(2569),
    [sym_identifier] = ACTIONS(2571),
    [anon_sym_SEMI] = ACTIONS(2569),
    [anon_sym_macro_rules_BANG] = ACTIONS(2569),
    [anon_sym_LPAREN] = ACTIONS(2569),
    [anon_sym_LBRACK] = ACTIONS(2569),
    [anon_sym_LBRACE] = ACTIONS(2569),
    [anon_sym_RBRACE] = ACTIONS(2569),
    [anon_sym_STAR] = ACTIONS(2569),
    [anon_sym_u8] = ACTIONS(2571),
    [anon_sym_i8] = ACTIONS(2571),
    [anon_sym_u16] = ACTIONS(2571),
    [anon_sym_i16] = ACTIONS(2571),
    [anon_sym_u32] = ACTIONS(2571),
    [anon_sym_i32] = ACTIONS(2571),
    [anon_sym_u64] = ACTIONS(2571),
    [anon_sym_i64] = ACTIONS(2571),
    [anon_sym_u128] = ACTIONS(2571),
    [anon_sym_i128] = ACTIONS(2571),
    [anon_sym_isize] = ACTIONS(2571),
    [anon_sym_usize] = ACTIONS(2571),
    [anon_sym_f32] = ACTIONS(2571),
    [anon_sym_f64] = ACTIONS(2571),
    [anon_sym_bool] = ACTIONS(2571),
    [anon_sym_str] = ACTIONS(2571),
    [anon_sym_char] = ACTIONS(2571),
    [anon_sym_DASH] = ACTIONS(2569),
    [anon_sym_BANG] = ACTIONS(2569),
    [anon_sym_AMP] = ACTIONS(2569),
    [anon_sym_PIPE] = ACTIONS(2569),
    [anon_sym_LT] = ACTIONS(2569),
    [anon_sym_DOT_DOT] = ACTIONS(2569),
    [anon_sym_COLON_COLON] = ACTIONS(2569),
    [anon_sym_POUND] = ACTIONS(2569),
    [anon_sym_SQUOTE] = ACTIONS(2571),
    [anon_sym_async] = ACTIONS(2571),
    [anon_sym_break] = ACTIONS(2571),
    [anon_sym_const] = ACTIONS(2571),
    [anon_sym_continue] = ACTIONS(2571),
    [anon_sym_default] = ACTIONS(2571),
    [anon_sym_enum] = ACTIONS(2571),
    [anon_sym_fn] = ACTIONS(2571),
    [anon_sym_for] = ACTIONS(2571),
    [anon_sym_gen] = ACTIONS(2571),
    [anon_sym_if] = ACTIONS(2571),
    [anon_sym_impl] = ACTIONS(2571),
    [anon_sym_let] = ACTIONS(2571),
    [anon_sym_loop] = ACTIONS(2571),
    [anon_sym_match] = ACTIONS(2571),
    [anon_sym_mod] = ACTIONS(2571),
    [anon_sym_pub] = ACTIONS(2571),
    [anon_sym_return] = ACTIONS(2571),
    [anon_sym_static] = ACTIONS(2571),
    [anon_sym_struct] = ACTIONS(2571),
    [anon_sym_trait] = ACTIONS(2571),
    [anon_sym_type] = ACTIONS(2571),
    [anon_sym_union] = ACTIONS(2571),
    [anon_sym_unsafe] = ACTIONS(2571),
    [anon_sym_use] = ACTIONS(2571),
    [anon_sym_while] = ACTIONS(2571),
    [anon_sym_extern] = ACTIONS(2571),
    [anon_sym_yield] = ACTIONS(2571),
    [anon_sym_move] = ACTIONS(2571),
    [anon_sym_try] = ACTIONS(2571),
    [sym_integer_literal] = ACTIONS(2569),
    [aux_sym_string_literal_token1] = ACTIONS(2569),
    [sym_char_literal] = ACTIONS(2569),
    [anon_sym_true] = ACTIONS(2571),
    [anon_sym_false] = ACTIONS(2571),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2571),
    [sym_super] = ACTIONS(2571),
    [sym_crate] = ACTIONS(2571),
    [sym_metavariable] = ACTIONS(2569),
    [sym__raw_string_literal_start] = ACTIONS(2569),
    [sym_float_literal] = ACTIONS(2569),
  },
  [680] = {
    [sym_line_comment] = STATE(680),
    [sym_block_comment] = STATE(680),
    [ts_builtin_sym_end] = ACTIONS(2573),
    [sym_identifier] = ACTIONS(2575),
    [anon_sym_SEMI] = ACTIONS(2573),
    [anon_sym_macro_rules_BANG] = ACTIONS(2573),
    [anon_sym_LPAREN] = ACTIONS(2573),
    [anon_sym_LBRACK] = ACTIONS(2573),
    [anon_sym_LBRACE] = ACTIONS(2573),
    [anon_sym_RBRACE] = ACTIONS(2573),
    [anon_sym_STAR] = ACTIONS(2573),
    [anon_sym_u8] = ACTIONS(2575),
    [anon_sym_i8] = ACTIONS(2575),
    [anon_sym_u16] = ACTIONS(2575),
    [anon_sym_i16] = ACTIONS(2575),
    [anon_sym_u32] = ACTIONS(2575),
    [anon_sym_i32] = ACTIONS(2575),
    [anon_sym_u64] = ACTIONS(2575),
    [anon_sym_i64] = ACTIONS(2575),
    [anon_sym_u128] = ACTIONS(2575),
    [anon_sym_i128] = ACTIONS(2575),
    [anon_sym_isize] = ACTIONS(2575),
    [anon_sym_usize] = ACTIONS(2575),
    [anon_sym_f32] = ACTIONS(2575),
    [anon_sym_f64] = ACTIONS(2575),
    [anon_sym_bool] = ACTIONS(2575),
    [anon_sym_str] = ACTIONS(2575),
    [anon_sym_char] = ACTIONS(2575),
    [anon_sym_DASH] = ACTIONS(2573),
    [anon_sym_BANG] = ACTIONS(2573),
    [anon_sym_AMP] = ACTIONS(2573),
    [anon_sym_PIPE] = ACTIONS(2573),
    [anon_sym_LT] = ACTIONS(2573),
    [anon_sym_DOT_DOT] = ACTIONS(2573),
    [anon_sym_COLON_COLON] = ACTIONS(2573),
    [anon_sym_POUND] = ACTIONS(2573),
    [anon_sym_SQUOTE] = ACTIONS(2575),
    [anon_sym_async] = ACTIONS(2575),
    [anon_sym_break] = ACTIONS(2575),
    [anon_sym_const] = ACTIONS(2575),
    [anon_sym_continue] = ACTIONS(2575),
    [anon_sym_default] = ACTIONS(2575),
    [anon_sym_enum] = ACTIONS(2575),
    [anon_sym_fn] = ACTIONS(2575),
    [anon_sym_for] = ACTIONS(2575),
    [anon_sym_gen] = ACTIONS(2575),
    [anon_sym_if] = ACTIONS(2575),
    [anon_sym_impl] = ACTIONS(2575),
    [anon_sym_let] = ACTIONS(2575),
    [anon_sym_loop] = ACTIONS(2575),
    [anon_sym_match] = ACTIONS(2575),
    [anon_sym_mod] = ACTIONS(2575),
    [anon_sym_pub] = ACTIONS(2575),
    [anon_sym_return] = ACTIONS(2575),
    [anon_sym_static] = ACTIONS(2575),
    [anon_sym_struct] = ACTIONS(2575),
    [anon_sym_trait] = ACTIONS(2575),
    [anon_sym_type] = ACTIONS(2575),
    [anon_sym_union] = ACTIONS(2575),
    [anon_sym_unsafe] = ACTIONS(2575),
    [anon_sym_use] = ACTIONS(2575),
    [anon_sym_while] = ACTIONS(2575),
    [anon_sym_extern] = ACTIONS(2575),
    [anon_sym_yield] = ACTIONS(2575),
    [anon_sym_move] = ACTIONS(2575),
    [anon_sym_try] = ACTIONS(2575),
    [sym_integer_literal] = ACTIONS(2573),
    [aux_sym_string_literal_token1] = ACTIONS(2573),
    [sym_char_literal] = ACTIONS(2573),
    [anon_sym_true] = ACTIONS(2575),
    [anon_sym_false] = ACTIONS(2575),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2575),
    [sym_super] = ACTIONS(2575),
    [sym_crate] = ACTIONS(2575),
    [sym_metavariable] = ACTIONS(2573),
    [sym__raw_string_literal_start] = ACTIONS(2573),
    [sym_float_literal] = ACTIONS(2573),
  },
  [681] = {
    [sym_line_comment] = STATE(681),
    [sym_block_comment] = STATE(681),
    [ts_builtin_sym_end] = ACTIONS(2577),
    [sym_identifier] = ACTIONS(2579),
    [anon_sym_SEMI] = ACTIONS(2577),
    [anon_sym_macro_rules_BANG] = ACTIONS(2577),
    [anon_sym_LPAREN] = ACTIONS(2577),
    [anon_sym_LBRACK] = ACTIONS(2577),
    [anon_sym_LBRACE] = ACTIONS(2577),
    [anon_sym_RBRACE] = ACTIONS(2577),
    [anon_sym_STAR] = ACTIONS(2577),
    [anon_sym_u8] = ACTIONS(2579),
    [anon_sym_i8] = ACTIONS(2579),
    [anon_sym_u16] = ACTIONS(2579),
    [anon_sym_i16] = ACTIONS(2579),
    [anon_sym_u32] = ACTIONS(2579),
    [anon_sym_i32] = ACTIONS(2579),
    [anon_sym_u64] = ACTIONS(2579),
    [anon_sym_i64] = ACTIONS(2579),
    [anon_sym_u128] = ACTIONS(2579),
    [anon_sym_i128] = ACTIONS(2579),
    [anon_sym_isize] = ACTIONS(2579),
    [anon_sym_usize] = ACTIONS(2579),
    [anon_sym_f32] = ACTIONS(2579),
    [anon_sym_f64] = ACTIONS(2579),
    [anon_sym_bool] = ACTIONS(2579),
    [anon_sym_str] = ACTIONS(2579),
    [anon_sym_char] = ACTIONS(2579),
    [anon_sym_DASH] = ACTIONS(2577),
    [anon_sym_BANG] = ACTIONS(2577),
    [anon_sym_AMP] = ACTIONS(2577),
    [anon_sym_PIPE] = ACTIONS(2577),
    [anon_sym_LT] = ACTIONS(2577),
    [anon_sym_DOT_DOT] = ACTIONS(2577),
    [anon_sym_COLON_COLON] = ACTIONS(2577),
    [anon_sym_POUND] = ACTIONS(2577),
    [anon_sym_SQUOTE] = ACTIONS(2579),
    [anon_sym_async] = ACTIONS(2579),
    [anon_sym_break] = ACTIONS(2579),
    [anon_sym_const] = ACTIONS(2579),
    [anon_sym_continue] = ACTIONS(2579),
    [anon_sym_default] = ACTIONS(2579),
    [anon_sym_enum] = ACTIONS(2579),
    [anon_sym_fn] = ACTIONS(2579),
    [anon_sym_for] = ACTIONS(2579),
    [anon_sym_gen] = ACTIONS(2579),
    [anon_sym_if] = ACTIONS(2579),
    [anon_sym_impl] = ACTIONS(2579),
    [anon_sym_let] = ACTIONS(2579),
    [anon_sym_loop] = ACTIONS(2579),
    [anon_sym_match] = ACTIONS(2579),
    [anon_sym_mod] = ACTIONS(2579),
    [anon_sym_pub] = ACTIONS(2579),
    [anon_sym_return] = ACTIONS(2579),
    [anon_sym_static] = ACTIONS(2579),
    [anon_sym_struct] = ACTIONS(2579),
    [anon_sym_trait] = ACTIONS(2579),
    [anon_sym_type] = ACTIONS(2579),
    [anon_sym_union] = ACTIONS(2579),
    [anon_sym_unsafe] = ACTIONS(2579),
    [anon_sym_use] = ACTIONS(2579),
    [anon_sym_while] = ACTIONS(2579),
    [anon_sym_extern] = ACTIONS(2579),
    [anon_sym_yield] = ACTIONS(2579),
    [anon_sym_move] = ACTIONS(2579),
    [anon_sym_try] = ACTIONS(2579),
    [sym_integer_literal] = ACTIONS(2577),
    [aux_sym_string_literal_token1] = ACTIONS(2577),
    [sym_char_literal] = ACTIONS(2577),
    [anon_sym_true] = ACTIONS(2579),
    [anon_sym_false] = ACTIONS(2579),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2579),
    [sym_super] = ACTIONS(2579),
    [sym_crate] = ACTIONS(2579),
    [sym_metavariable] = ACTIONS(2577),
    [sym__raw_string_literal_start] = ACTIONS(2577),
    [sym_float_literal] = ACTIONS(2577),
  },
  [682] = {
    [sym_line_comment] = STATE(682),
    [sym_block_comment] = STATE(682),
    [ts_builtin_sym_end] = ACTIONS(2581),
    [sym_identifier] = ACTIONS(2583),
    [anon_sym_SEMI] = ACTIONS(2581),
    [anon_sym_macro_rules_BANG] = ACTIONS(2581),
    [anon_sym_LPAREN] = ACTIONS(2581),
    [anon_sym_LBRACK] = ACTIONS(2581),
    [anon_sym_LBRACE] = ACTIONS(2581),
    [anon_sym_RBRACE] = ACTIONS(2581),
    [anon_sym_STAR] = ACTIONS(2581),
    [anon_sym_u8] = ACTIONS(2583),
    [anon_sym_i8] = ACTIONS(2583),
    [anon_sym_u16] = ACTIONS(2583),
    [anon_sym_i16] = ACTIONS(2583),
    [anon_sym_u32] = ACTIONS(2583),
    [anon_sym_i32] = ACTIONS(2583),
    [anon_sym_u64] = ACTIONS(2583),
    [anon_sym_i64] = ACTIONS(2583),
    [anon_sym_u128] = ACTIONS(2583),
    [anon_sym_i128] = ACTIONS(2583),
    [anon_sym_isize] = ACTIONS(2583),
    [anon_sym_usize] = ACTIONS(2583),
    [anon_sym_f32] = ACTIONS(2583),
    [anon_sym_f64] = ACTIONS(2583),
    [anon_sym_bool] = ACTIONS(2583),
    [anon_sym_str] = ACTIONS(2583),
    [anon_sym_char] = ACTIONS(2583),
    [anon_sym_DASH] = ACTIONS(2581),
    [anon_sym_BANG] = ACTIONS(2581),
    [anon_sym_AMP] = ACTIONS(2581),
    [anon_sym_PIPE] = ACTIONS(2581),
    [anon_sym_LT] = ACTIONS(2581),
    [anon_sym_DOT_DOT] = ACTIONS(2581),
    [anon_sym_COLON_COLON] = ACTIONS(2581),
    [anon_sym_POUND] = ACTIONS(2581),
    [anon_sym_SQUOTE] = ACTIONS(2583),
    [anon_sym_async] = ACTIONS(2583),
    [anon_sym_break] = ACTIONS(2583),
    [anon_sym_const] = ACTIONS(2583),
    [anon_sym_continue] = ACTIONS(2583),
    [anon_sym_default] = ACTIONS(2583),
    [anon_sym_enum] = ACTIONS(2583),
    [anon_sym_fn] = ACTIONS(2583),
    [anon_sym_for] = ACTIONS(2583),
    [anon_sym_gen] = ACTIONS(2583),
    [anon_sym_if] = ACTIONS(2583),
    [anon_sym_impl] = ACTIONS(2583),
    [anon_sym_let] = ACTIONS(2583),
    [anon_sym_loop] = ACTIONS(2583),
    [anon_sym_match] = ACTIONS(2583),
    [anon_sym_mod] = ACTIONS(2583),
    [anon_sym_pub] = ACTIONS(2583),
    [anon_sym_return] = ACTIONS(2583),
    [anon_sym_static] = ACTIONS(2583),
    [anon_sym_struct] = ACTIONS(2583),
    [anon_sym_trait] = ACTIONS(2583),
    [anon_sym_type] = ACTIONS(2583),
    [anon_sym_union] = ACTIONS(2583),
    [anon_sym_unsafe] = ACTIONS(2583),
    [anon_sym_use] = ACTIONS(2583),
    [anon_sym_while] = ACTIONS(2583),
    [anon_sym_extern] = ACTIONS(2583),
    [anon_sym_yield] = ACTIONS(2583),
    [anon_sym_move] = ACTIONS(2583),
    [anon_sym_try] = ACTIONS(2583),
    [sym_integer_literal] = ACTIONS(2581),
    [aux_sym_string_literal_token1] = ACTIONS(2581),
    [sym_char_literal] = ACTIONS(2581),
    [anon_sym_true] = ACTIONS(2583),
    [anon_sym_false] = ACTIONS(2583),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2583),
    [sym_super] = ACTIONS(2583),
    [sym_crate] = ACTIONS(2583),
    [sym_metavariable] = ACTIONS(2581),
    [sym__raw_string_literal_start] = ACTIONS(2581),
    [sym_float_literal] = ACTIONS(2581),
  },
  [683] = {
    [sym_line_comment] = STATE(683),
    [sym_block_comment] = STATE(683),
    [ts_builtin_sym_end] = ACTIONS(2585),
    [sym_identifier] = ACTIONS(2587),
    [anon_sym_SEMI] = ACTIONS(2585),
    [anon_sym_macro_rules_BANG] = ACTIONS(2585),
    [anon_sym_LPAREN] = ACTIONS(2585),
    [anon_sym_LBRACK] = ACTIONS(2585),
    [anon_sym_LBRACE] = ACTIONS(2585),
    [anon_sym_RBRACE] = ACTIONS(2585),
    [anon_sym_STAR] = ACTIONS(2585),
    [anon_sym_u8] = ACTIONS(2587),
    [anon_sym_i8] = ACTIONS(2587),
    [anon_sym_u16] = ACTIONS(2587),
    [anon_sym_i16] = ACTIONS(2587),
    [anon_sym_u32] = ACTIONS(2587),
    [anon_sym_i32] = ACTIONS(2587),
    [anon_sym_u64] = ACTIONS(2587),
    [anon_sym_i64] = ACTIONS(2587),
    [anon_sym_u128] = ACTIONS(2587),
    [anon_sym_i128] = ACTIONS(2587),
    [anon_sym_isize] = ACTIONS(2587),
    [anon_sym_usize] = ACTIONS(2587),
    [anon_sym_f32] = ACTIONS(2587),
    [anon_sym_f64] = ACTIONS(2587),
    [anon_sym_bool] = ACTIONS(2587),
    [anon_sym_str] = ACTIONS(2587),
    [anon_sym_char] = ACTIONS(2587),
    [anon_sym_DASH] = ACTIONS(2585),
    [anon_sym_BANG] = ACTIONS(2585),
    [anon_sym_AMP] = ACTIONS(2585),
    [anon_sym_PIPE] = ACTIONS(2585),
    [anon_sym_LT] = ACTIONS(2585),
    [anon_sym_DOT_DOT] = ACTIONS(2585),
    [anon_sym_COLON_COLON] = ACTIONS(2585),
    [anon_sym_POUND] = ACTIONS(2585),
    [anon_sym_SQUOTE] = ACTIONS(2587),
    [anon_sym_async] = ACTIONS(2587),
    [anon_sym_break] = ACTIONS(2587),
    [anon_sym_const] = ACTIONS(2587),
    [anon_sym_continue] = ACTIONS(2587),
    [anon_sym_default] = ACTIONS(2587),
    [anon_sym_enum] = ACTIONS(2587),
    [anon_sym_fn] = ACTIONS(2587),
    [anon_sym_for] = ACTIONS(2587),
    [anon_sym_gen] = ACTIONS(2587),
    [anon_sym_if] = ACTIONS(2587),
    [anon_sym_impl] = ACTIONS(2587),
    [anon_sym_let] = ACTIONS(2587),
    [anon_sym_loop] = ACTIONS(2587),
    [anon_sym_match] = ACTIONS(2587),
    [anon_sym_mod] = ACTIONS(2587),
    [anon_sym_pub] = ACTIONS(2587),
    [anon_sym_return] = ACTIONS(2587),
    [anon_sym_static] = ACTIONS(2587),
    [anon_sym_struct] = ACTIONS(2587),
    [anon_sym_trait] = ACTIONS(2587),
    [anon_sym_type] = ACTIONS(2587),
    [anon_sym_union] = ACTIONS(2587),
    [anon_sym_unsafe] = ACTIONS(2587),
    [anon_sym_use] = ACTIONS(2587),
    [anon_sym_while] = ACTIONS(2587),
    [anon_sym_extern] = ACTIONS(2587),
    [anon_sym_yield] = ACTIONS(2587),
    [anon_sym_move] = ACTIONS(2587),
    [anon_sym_try] = ACTIONS(2587),
    [sym_integer_literal] = ACTIONS(2585),
    [aux_sym_string_literal_token1] = ACTIONS(2585),
    [sym_char_literal] = ACTIONS(2585),
    [anon_sym_true] = ACTIONS(2587),
    [anon_sym_false] = ACTIONS(2587),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2587),
    [sym_super] = ACTIONS(2587),
    [sym_crate] = ACTIONS(2587),
    [sym_metavariable] = ACTIONS(2585),
    [sym__raw_string_literal_start] = ACTIONS(2585),
    [sym_float_literal] = ACTIONS(2585),
  },
  [684] = {
    [sym_line_comment] = STATE(684),
    [sym_block_comment] = STATE(684),
    [ts_builtin_sym_end] = ACTIONS(2589),
    [sym_identifier] = ACTIONS(2591),
    [anon_sym_SEMI] = ACTIONS(2589),
    [anon_sym_macro_rules_BANG] = ACTIONS(2589),
    [anon_sym_LPAREN] = ACTIONS(2589),
    [anon_sym_LBRACK] = ACTIONS(2589),
    [anon_sym_LBRACE] = ACTIONS(2589),
    [anon_sym_RBRACE] = ACTIONS(2589),
    [anon_sym_STAR] = ACTIONS(2589),
    [anon_sym_u8] = ACTIONS(2591),
    [anon_sym_i8] = ACTIONS(2591),
    [anon_sym_u16] = ACTIONS(2591),
    [anon_sym_i16] = ACTIONS(2591),
    [anon_sym_u32] = ACTIONS(2591),
    [anon_sym_i32] = ACTIONS(2591),
    [anon_sym_u64] = ACTIONS(2591),
    [anon_sym_i64] = ACTIONS(2591),
    [anon_sym_u128] = ACTIONS(2591),
    [anon_sym_i128] = ACTIONS(2591),
    [anon_sym_isize] = ACTIONS(2591),
    [anon_sym_usize] = ACTIONS(2591),
    [anon_sym_f32] = ACTIONS(2591),
    [anon_sym_f64] = ACTIONS(2591),
    [anon_sym_bool] = ACTIONS(2591),
    [anon_sym_str] = ACTIONS(2591),
    [anon_sym_char] = ACTIONS(2591),
    [anon_sym_DASH] = ACTIONS(2589),
    [anon_sym_BANG] = ACTIONS(2589),
    [anon_sym_AMP] = ACTIONS(2589),
    [anon_sym_PIPE] = ACTIONS(2589),
    [anon_sym_LT] = ACTIONS(2589),
    [anon_sym_DOT_DOT] = ACTIONS(2589),
    [anon_sym_COLON_COLON] = ACTIONS(2589),
    [anon_sym_POUND] = ACTIONS(2589),
    [anon_sym_SQUOTE] = ACTIONS(2591),
    [anon_sym_async] = ACTIONS(2591),
    [anon_sym_break] = ACTIONS(2591),
    [anon_sym_const] = ACTIONS(2591),
    [anon_sym_continue] = ACTIONS(2591),
    [anon_sym_default] = ACTIONS(2591),
    [anon_sym_enum] = ACTIONS(2591),
    [anon_sym_fn] = ACTIONS(2591),
    [anon_sym_for] = ACTIONS(2591),
    [anon_sym_gen] = ACTIONS(2591),
    [anon_sym_if] = ACTIONS(2591),
    [anon_sym_impl] = ACTIONS(2591),
    [anon_sym_let] = ACTIONS(2591),
    [anon_sym_loop] = ACTIONS(2591),
    [anon_sym_match] = ACTIONS(2591),
    [anon_sym_mod] = ACTIONS(2591),
    [anon_sym_pub] = ACTIONS(2591),
    [anon_sym_return] = ACTIONS(2591),
    [anon_sym_static] = ACTIONS(2591),
    [anon_sym_struct] = ACTIONS(2591),
    [anon_sym_trait] = ACTIONS(2591),
    [anon_sym_type] = ACTIONS(2591),
    [anon_sym_union] = ACTIONS(2591),
    [anon_sym_unsafe] = ACTIONS(2591),
    [anon_sym_use] = ACTIONS(2591),
    [anon_sym_while] = ACTIONS(2591),
    [anon_sym_extern] = ACTIONS(2591),
    [anon_sym_yield] = ACTIONS(2591),
    [anon_sym_move] = ACTIONS(2591),
    [anon_sym_try] = ACTIONS(2591),
    [sym_integer_literal] = ACTIONS(2589),
    [aux_sym_string_literal_token1] = ACTIONS(2589),
    [sym_char_literal] = ACTIONS(2589),
    [anon_sym_true] = ACTIONS(2591),
    [anon_sym_false] = ACTIONS(2591),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2591),
    [sym_super] = ACTIONS(2591),
    [sym_crate] = ACTIONS(2591),
    [sym_metavariable] = ACTIONS(2589),
    [sym__raw_string_literal_start] = ACTIONS(2589),
    [sym_float_literal] = ACTIONS(2589),
  },
  [685] = {
    [sym_line_comment] = STATE(685),
    [sym_block_comment] = STATE(685),
    [ts_builtin_sym_end] = ACTIONS(2593),
    [sym_identifier] = ACTIONS(2595),
    [anon_sym_SEMI] = ACTIONS(2593),
    [anon_sym_macro_rules_BANG] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2593),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LBRACE] = ACTIONS(2593),
    [anon_sym_RBRACE] = ACTIONS(2593),
    [anon_sym_STAR] = ACTIONS(2593),
    [anon_sym_u8] = ACTIONS(2595),
    [anon_sym_i8] = ACTIONS(2595),
    [anon_sym_u16] = ACTIONS(2595),
    [anon_sym_i16] = ACTIONS(2595),
    [anon_sym_u32] = ACTIONS(2595),
    [anon_sym_i32] = ACTIONS(2595),
    [anon_sym_u64] = ACTIONS(2595),
    [anon_sym_i64] = ACTIONS(2595),
    [anon_sym_u128] = ACTIONS(2595),
    [anon_sym_i128] = ACTIONS(2595),
    [anon_sym_isize] = ACTIONS(2595),
    [anon_sym_usize] = ACTIONS(2595),
    [anon_sym_f32] = ACTIONS(2595),
    [anon_sym_f64] = ACTIONS(2595),
    [anon_sym_bool] = ACTIONS(2595),
    [anon_sym_str] = ACTIONS(2595),
    [anon_sym_char] = ACTIONS(2595),
    [anon_sym_DASH] = ACTIONS(2593),
    [anon_sym_BANG] = ACTIONS(2593),
    [anon_sym_AMP] = ACTIONS(2593),
    [anon_sym_PIPE] = ACTIONS(2593),
    [anon_sym_LT] = ACTIONS(2593),
    [anon_sym_DOT_DOT] = ACTIONS(2593),
    [anon_sym_COLON_COLON] = ACTIONS(2593),
    [anon_sym_POUND] = ACTIONS(2593),
    [anon_sym_SQUOTE] = ACTIONS(2595),
    [anon_sym_async] = ACTIONS(2595),
    [anon_sym_break] = ACTIONS(2595),
    [anon_sym_const] = ACTIONS(2595),
    [anon_sym_continue] = ACTIONS(2595),
    [anon_sym_default] = ACTIONS(2595),
    [anon_sym_enum] = ACTIONS(2595),
    [anon_sym_fn] = ACTIONS(2595),
    [anon_sym_for] = ACTIONS(2595),
    [anon_sym_gen] = ACTIONS(2595),
    [anon_sym_if] = ACTIONS(2595),
    [anon_sym_impl] = ACTIONS(2595),
    [anon_sym_let] = ACTIONS(2595),
    [anon_sym_loop] = ACTIONS(2595),
    [anon_sym_match] = ACTIONS(2595),
    [anon_sym_mod] = ACTIONS(2595),
    [anon_sym_pub] = ACTIONS(2595),
    [anon_sym_return] = ACTIONS(2595),
    [anon_sym_static] = ACTIONS(2595),
    [anon_sym_struct] = ACTIONS(2595),
    [anon_sym_trait] = ACTIONS(2595),
    [anon_sym_type] = ACTIONS(2595),
    [anon_sym_union] = ACTIONS(2595),
    [anon_sym_unsafe] = ACTIONS(2595),
    [anon_sym_use] = ACTIONS(2595),
    [anon_sym_while] = ACTIONS(2595),
    [anon_sym_extern] = ACTIONS(2595),
    [anon_sym_yield] = ACTIONS(2595),
    [anon_sym_move] = ACTIONS(2595),
    [anon_sym_try] = ACTIONS(2595),
    [sym_integer_literal] = ACTIONS(2593),
    [aux_sym_string_literal_token1] = ACTIONS(2593),
    [sym_char_literal] = ACTIONS(2593),
    [anon_sym_true] = ACTIONS(2595),
    [anon_sym_false] = ACTIONS(2595),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2595),
    [sym_super] = ACTIONS(2595),
    [sym_crate] = ACTIONS(2595),
    [sym_metavariable] = ACTIONS(2593),
    [sym__raw_string_literal_start] = ACTIONS(2593),
    [sym_float_literal] = ACTIONS(2593),
  },
  [686] = {
    [sym_line_comment] = STATE(686),
    [sym_block_comment] = STATE(686),
    [ts_builtin_sym_end] = ACTIONS(2597),
    [sym_identifier] = ACTIONS(2599),
    [anon_sym_SEMI] = ACTIONS(2597),
    [anon_sym_macro_rules_BANG] = ACTIONS(2597),
    [anon_sym_LPAREN] = ACTIONS(2597),
    [anon_sym_LBRACK] = ACTIONS(2597),
    [anon_sym_LBRACE] = ACTIONS(2597),
    [anon_sym_RBRACE] = ACTIONS(2597),
    [anon_sym_STAR] = ACTIONS(2597),
    [anon_sym_u8] = ACTIONS(2599),
    [anon_sym_i8] = ACTIONS(2599),
    [anon_sym_u16] = ACTIONS(2599),
    [anon_sym_i16] = ACTIONS(2599),
    [anon_sym_u32] = ACTIONS(2599),
    [anon_sym_i32] = ACTIONS(2599),
    [anon_sym_u64] = ACTIONS(2599),
    [anon_sym_i64] = ACTIONS(2599),
    [anon_sym_u128] = ACTIONS(2599),
    [anon_sym_i128] = ACTIONS(2599),
    [anon_sym_isize] = ACTIONS(2599),
    [anon_sym_usize] = ACTIONS(2599),
    [anon_sym_f32] = ACTIONS(2599),
    [anon_sym_f64] = ACTIONS(2599),
    [anon_sym_bool] = ACTIONS(2599),
    [anon_sym_str] = ACTIONS(2599),
    [anon_sym_char] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2597),
    [anon_sym_BANG] = ACTIONS(2597),
    [anon_sym_AMP] = ACTIONS(2597),
    [anon_sym_PIPE] = ACTIONS(2597),
    [anon_sym_LT] = ACTIONS(2597),
    [anon_sym_DOT_DOT] = ACTIONS(2597),
    [anon_sym_COLON_COLON] = ACTIONS(2597),
    [anon_sym_POUND] = ACTIONS(2597),
    [anon_sym_SQUOTE] = ACTIONS(2599),
    [anon_sym_async] = ACTIONS(2599),
    [anon_sym_break] = ACTIONS(2599),
    [anon_sym_const] = ACTIONS(2599),
    [anon_sym_continue] = ACTIONS(2599),
    [anon_sym_default] = ACTIONS(2599),
    [anon_sym_enum] = ACTIONS(2599),
    [anon_sym_fn] = ACTIONS(2599),
    [anon_sym_for] = ACTIONS(2599),
    [anon_sym_gen] = ACTIONS(2599),
    [anon_sym_if] = ACTIONS(2599),
    [anon_sym_impl] = ACTIONS(2599),
    [anon_sym_let] = ACTIONS(2599),
    [anon_sym_loop] = ACTIONS(2599),
    [anon_sym_match] = ACTIONS(2599),
    [anon_sym_mod] = ACTIONS(2599),
    [anon_sym_pub] = ACTIONS(2599),
    [anon_sym_return] = ACTIONS(2599),
    [anon_sym_static] = ACTIONS(2599),
    [anon_sym_struct] = ACTIONS(2599),
    [anon_sym_trait] = ACTIONS(2599),
    [anon_sym_type] = ACTIONS(2599),
    [anon_sym_union] = ACTIONS(2599),
    [anon_sym_unsafe] = ACTIONS(2599),
    [anon_sym_use] = ACTIONS(2599),
    [anon_sym_while] = ACTIONS(2599),
    [anon_sym_extern] = ACTIONS(2599),
    [anon_sym_yield] = ACTIONS(2599),
    [anon_sym_move] = ACTIONS(2599),
    [anon_sym_try] = ACTIONS(2599),
    [sym_integer_literal] = ACTIONS(2597),
    [aux_sym_string_literal_token1] = ACTIONS(2597),
    [sym_char_literal] = ACTIONS(2597),
    [anon_sym_true] = ACTIONS(2599),
    [anon_sym_false] = ACTIONS(2599),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2599),
    [sym_super] = ACTIONS(2599),
    [sym_crate] = ACTIONS(2599),
    [sym_metavariable] = ACTIONS(2597),
    [sym__raw_string_literal_start] = ACTIONS(2597),
    [sym_float_literal] = ACTIONS(2597),
  },
  [687] = {
    [sym_line_comment] = STATE(687),
    [sym_block_comment] = STATE(687),
    [ts_builtin_sym_end] = ACTIONS(2601),
    [sym_identifier] = ACTIONS(2603),
    [anon_sym_SEMI] = ACTIONS(2601),
    [anon_sym_macro_rules_BANG] = ACTIONS(2601),
    [anon_sym_LPAREN] = ACTIONS(2601),
    [anon_sym_LBRACK] = ACTIONS(2601),
    [anon_sym_LBRACE] = ACTIONS(2601),
    [anon_sym_RBRACE] = ACTIONS(2601),
    [anon_sym_STAR] = ACTIONS(2601),
    [anon_sym_u8] = ACTIONS(2603),
    [anon_sym_i8] = ACTIONS(2603),
    [anon_sym_u16] = ACTIONS(2603),
    [anon_sym_i16] = ACTIONS(2603),
    [anon_sym_u32] = ACTIONS(2603),
    [anon_sym_i32] = ACTIONS(2603),
    [anon_sym_u64] = ACTIONS(2603),
    [anon_sym_i64] = ACTIONS(2603),
    [anon_sym_u128] = ACTIONS(2603),
    [anon_sym_i128] = ACTIONS(2603),
    [anon_sym_isize] = ACTIONS(2603),
    [anon_sym_usize] = ACTIONS(2603),
    [anon_sym_f32] = ACTIONS(2603),
    [anon_sym_f64] = ACTIONS(2603),
    [anon_sym_bool] = ACTIONS(2603),
    [anon_sym_str] = ACTIONS(2603),
    [anon_sym_char] = ACTIONS(2603),
    [anon_sym_DASH] = ACTIONS(2601),
    [anon_sym_BANG] = ACTIONS(2601),
    [anon_sym_AMP] = ACTIONS(2601),
    [anon_sym_PIPE] = ACTIONS(2601),
    [anon_sym_LT] = ACTIONS(2601),
    [anon_sym_DOT_DOT] = ACTIONS(2601),
    [anon_sym_COLON_COLON] = ACTIONS(2601),
    [anon_sym_POUND] = ACTIONS(2601),
    [anon_sym_SQUOTE] = ACTIONS(2603),
    [anon_sym_async] = ACTIONS(2603),
    [anon_sym_break] = ACTIONS(2603),
    [anon_sym_const] = ACTIONS(2603),
    [anon_sym_continue] = ACTIONS(2603),
    [anon_sym_default] = ACTIONS(2603),
    [anon_sym_enum] = ACTIONS(2603),
    [anon_sym_fn] = ACTIONS(2603),
    [anon_sym_for] = ACTIONS(2603),
    [anon_sym_gen] = ACTIONS(2603),
    [anon_sym_if] = ACTIONS(2603),
    [anon_sym_impl] = ACTIONS(2603),
    [anon_sym_let] = ACTIONS(2603),
    [anon_sym_loop] = ACTIONS(2603),
    [anon_sym_match] = ACTIONS(2603),
    [anon_sym_mod] = ACTIONS(2603),
    [anon_sym_pub] = ACTIONS(2603),
    [anon_sym_return] = ACTIONS(2603),
    [anon_sym_static] = ACTIONS(2603),
    [anon_sym_struct] = ACTIONS(2603),
    [anon_sym_trait] = ACTIONS(2603),
    [anon_sym_type] = ACTIONS(2603),
    [anon_sym_union] = ACTIONS(2603),
    [anon_sym_unsafe] = ACTIONS(2603),
    [anon_sym_use] = ACTIONS(2603),
    [anon_sym_while] = ACTIONS(2603),
    [anon_sym_extern] = ACTIONS(2603),
    [anon_sym_yield] = ACTIONS(2603),
    [anon_sym_move] = ACTIONS(2603),
    [anon_sym_try] = ACTIONS(2603),
    [sym_integer_literal] = ACTIONS(2601),
    [aux_sym_string_literal_token1] = ACTIONS(2601),
    [sym_char_literal] = ACTIONS(2601),
    [anon_sym_true] = ACTIONS(2603),
    [anon_sym_false] = ACTIONS(2603),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2603),
    [sym_super] = ACTIONS(2603),
    [sym_crate] = ACTIONS(2603),
    [sym_metavariable] = ACTIONS(2601),
    [sym__raw_string_literal_start] = ACTIONS(2601),
    [sym_float_literal] = ACTIONS(2601),
  },
  [688] = {
    [sym_line_comment] = STATE(688),
    [sym_block_comment] = STATE(688),
    [ts_builtin_sym_end] = ACTIONS(2605),
    [sym_identifier] = ACTIONS(2607),
    [anon_sym_SEMI] = ACTIONS(2605),
    [anon_sym_macro_rules_BANG] = ACTIONS(2605),
    [anon_sym_LPAREN] = ACTIONS(2605),
    [anon_sym_LBRACK] = ACTIONS(2605),
    [anon_sym_LBRACE] = ACTIONS(2605),
    [anon_sym_RBRACE] = ACTIONS(2605),
    [anon_sym_STAR] = ACTIONS(2605),
    [anon_sym_u8] = ACTIONS(2607),
    [anon_sym_i8] = ACTIONS(2607),
    [anon_sym_u16] = ACTIONS(2607),
    [anon_sym_i16] = ACTIONS(2607),
    [anon_sym_u32] = ACTIONS(2607),
    [anon_sym_i32] = ACTIONS(2607),
    [anon_sym_u64] = ACTIONS(2607),
    [anon_sym_i64] = ACTIONS(2607),
    [anon_sym_u128] = ACTIONS(2607),
    [anon_sym_i128] = ACTIONS(2607),
    [anon_sym_isize] = ACTIONS(2607),
    [anon_sym_usize] = ACTIONS(2607),
    [anon_sym_f32] = ACTIONS(2607),
    [anon_sym_f64] = ACTIONS(2607),
    [anon_sym_bool] = ACTIONS(2607),
    [anon_sym_str] = ACTIONS(2607),
    [anon_sym_char] = ACTIONS(2607),
    [anon_sym_DASH] = ACTIONS(2605),
    [anon_sym_BANG] = ACTIONS(2605),
    [anon_sym_AMP] = ACTIONS(2605),
    [anon_sym_PIPE] = ACTIONS(2605),
    [anon_sym_LT] = ACTIONS(2605),
    [anon_sym_DOT_DOT] = ACTIONS(2605),
    [anon_sym_COLON_COLON] = ACTIONS(2605),
    [anon_sym_POUND] = ACTIONS(2605),
    [anon_sym_SQUOTE] = ACTIONS(2607),
    [anon_sym_async] = ACTIONS(2607),
    [anon_sym_break] = ACTIONS(2607),
    [anon_sym_const] = ACTIONS(2607),
    [anon_sym_continue] = ACTIONS(2607),
    [anon_sym_default] = ACTIONS(2607),
    [anon_sym_enum] = ACTIONS(2607),
    [anon_sym_fn] = ACTIONS(2607),
    [anon_sym_for] = ACTIONS(2607),
    [anon_sym_gen] = ACTIONS(2607),
    [anon_sym_if] = ACTIONS(2607),
    [anon_sym_impl] = ACTIONS(2607),
    [anon_sym_let] = ACTIONS(2607),
    [anon_sym_loop] = ACTIONS(2607),
    [anon_sym_match] = ACTIONS(2607),
    [anon_sym_mod] = ACTIONS(2607),
    [anon_sym_pub] = ACTIONS(2607),
    [anon_sym_return] = ACTIONS(2607),
    [anon_sym_static] = ACTIONS(2607),
    [anon_sym_struct] = ACTIONS(2607),
    [anon_sym_trait] = ACTIONS(2607),
    [anon_sym_type] = ACTIONS(2607),
    [anon_sym_union] = ACTIONS(2607),
    [anon_sym_unsafe] = ACTIONS(2607),
    [anon_sym_use] = ACTIONS(2607),
    [anon_sym_while] = ACTIONS(2607),
    [anon_sym_extern] = ACTIONS(2607),
    [anon_sym_yield] = ACTIONS(2607),
    [anon_sym_move] = ACTIONS(2607),
    [anon_sym_try] = ACTIONS(2607),
    [sym_integer_literal] = ACTIONS(2605),
    [aux_sym_string_literal_token1] = ACTIONS(2605),
    [sym_char_literal] = ACTIONS(2605),
    [anon_sym_true] = ACTIONS(2607),
    [anon_sym_false] = ACTIONS(2607),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2607),
    [sym_super] = ACTIONS(2607),
    [sym_crate] = ACTIONS(2607),
    [sym_metavariable] = ACTIONS(2605),
    [sym__raw_string_literal_start] = ACTIONS(2605),
    [sym_float_literal] = ACTIONS(2605),
  },
  [689] = {
    [sym_line_comment] = STATE(689),
    [sym_block_comment] = STATE(689),
    [ts_builtin_sym_end] = ACTIONS(2609),
    [sym_identifier] = ACTIONS(2611),
    [anon_sym_SEMI] = ACTIONS(2609),
    [anon_sym_macro_rules_BANG] = ACTIONS(2609),
    [anon_sym_LPAREN] = ACTIONS(2609),
    [anon_sym_LBRACK] = ACTIONS(2609),
    [anon_sym_LBRACE] = ACTIONS(2609),
    [anon_sym_RBRACE] = ACTIONS(2609),
    [anon_sym_STAR] = ACTIONS(2609),
    [anon_sym_u8] = ACTIONS(2611),
    [anon_sym_i8] = ACTIONS(2611),
    [anon_sym_u16] = ACTIONS(2611),
    [anon_sym_i16] = ACTIONS(2611),
    [anon_sym_u32] = ACTIONS(2611),
    [anon_sym_i32] = ACTIONS(2611),
    [anon_sym_u64] = ACTIONS(2611),
    [anon_sym_i64] = ACTIONS(2611),
    [anon_sym_u128] = ACTIONS(2611),
    [anon_sym_i128] = ACTIONS(2611),
    [anon_sym_isize] = ACTIONS(2611),
    [anon_sym_usize] = ACTIONS(2611),
    [anon_sym_f32] = ACTIONS(2611),
    [anon_sym_f64] = ACTIONS(2611),
    [anon_sym_bool] = ACTIONS(2611),
    [anon_sym_str] = ACTIONS(2611),
    [anon_sym_char] = ACTIONS(2611),
    [anon_sym_DASH] = ACTIONS(2609),
    [anon_sym_BANG] = ACTIONS(2609),
    [anon_sym_AMP] = ACTIONS(2609),
    [anon_sym_PIPE] = ACTIONS(2609),
    [anon_sym_LT] = ACTIONS(2609),
    [anon_sym_DOT_DOT] = ACTIONS(2609),
    [anon_sym_COLON_COLON] = ACTIONS(2609),
    [anon_sym_POUND] = ACTIONS(2609),
    [anon_sym_SQUOTE] = ACTIONS(2611),
    [anon_sym_async] = ACTIONS(2611),
    [anon_sym_break] = ACTIONS(2611),
    [anon_sym_const] = ACTIONS(2611),
    [anon_sym_continue] = ACTIONS(2611),
    [anon_sym_default] = ACTIONS(2611),
    [anon_sym_enum] = ACTIONS(2611),
    [anon_sym_fn] = ACTIONS(2611),
    [anon_sym_for] = ACTIONS(2611),
    [anon_sym_gen] = ACTIONS(2611),
    [anon_sym_if] = ACTIONS(2611),
    [anon_sym_impl] = ACTIONS(2611),
    [anon_sym_let] = ACTIONS(2611),
    [anon_sym_loop] = ACTIONS(2611),
    [anon_sym_match] = ACTIONS(2611),
    [anon_sym_mod] = ACTIONS(2611),
    [anon_sym_pub] = ACTIONS(2611),
    [anon_sym_return] = ACTIONS(2611),
    [anon_sym_static] = ACTIONS(2611),
    [anon_sym_struct] = ACTIONS(2611),
    [anon_sym_trait] = ACTIONS(2611),
    [anon_sym_type] = ACTIONS(2611),
    [anon_sym_union] = ACTIONS(2611),
    [anon_sym_unsafe] = ACTIONS(2611),
    [anon_sym_use] = ACTIONS(2611),
    [anon_sym_while] = ACTIONS(2611),
    [anon_sym_extern] = ACTIONS(2611),
    [anon_sym_yield] = ACTIONS(2611),
    [anon_sym_move] = ACTIONS(2611),
    [anon_sym_try] = ACTIONS(2611),
    [sym_integer_literal] = ACTIONS(2609),
    [aux_sym_string_literal_token1] = ACTIONS(2609),
    [sym_char_literal] = ACTIONS(2609),
    [anon_sym_true] = ACTIONS(2611),
    [anon_sym_false] = ACTIONS(2611),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2611),
    [sym_super] = ACTIONS(2611),
    [sym_crate] = ACTIONS(2611),
    [sym_metavariable] = ACTIONS(2609),
    [sym__raw_string_literal_start] = ACTIONS(2609),
    [sym_float_literal] = ACTIONS(2609),
  },
  [690] = {
    [sym_line_comment] = STATE(690),
    [sym_block_comment] = STATE(690),
    [ts_builtin_sym_end] = ACTIONS(2613),
    [sym_identifier] = ACTIONS(2615),
    [anon_sym_SEMI] = ACTIONS(2613),
    [anon_sym_macro_rules_BANG] = ACTIONS(2613),
    [anon_sym_LPAREN] = ACTIONS(2613),
    [anon_sym_LBRACK] = ACTIONS(2613),
    [anon_sym_LBRACE] = ACTIONS(2613),
    [anon_sym_RBRACE] = ACTIONS(2613),
    [anon_sym_STAR] = ACTIONS(2613),
    [anon_sym_u8] = ACTIONS(2615),
    [anon_sym_i8] = ACTIONS(2615),
    [anon_sym_u16] = ACTIONS(2615),
    [anon_sym_i16] = ACTIONS(2615),
    [anon_sym_u32] = ACTIONS(2615),
    [anon_sym_i32] = ACTIONS(2615),
    [anon_sym_u64] = ACTIONS(2615),
    [anon_sym_i64] = ACTIONS(2615),
    [anon_sym_u128] = ACTIONS(2615),
    [anon_sym_i128] = ACTIONS(2615),
    [anon_sym_isize] = ACTIONS(2615),
    [anon_sym_usize] = ACTIONS(2615),
    [anon_sym_f32] = ACTIONS(2615),
    [anon_sym_f64] = ACTIONS(2615),
    [anon_sym_bool] = ACTIONS(2615),
    [anon_sym_str] = ACTIONS(2615),
    [anon_sym_char] = ACTIONS(2615),
    [anon_sym_DASH] = ACTIONS(2613),
    [anon_sym_BANG] = ACTIONS(2613),
    [anon_sym_AMP] = ACTIONS(2613),
    [anon_sym_PIPE] = ACTIONS(2613),
    [anon_sym_LT] = ACTIONS(2613),
    [anon_sym_DOT_DOT] = ACTIONS(2613),
    [anon_sym_COLON_COLON] = ACTIONS(2613),
    [anon_sym_POUND] = ACTIONS(2613),
    [anon_sym_SQUOTE] = ACTIONS(2615),
    [anon_sym_async] = ACTIONS(2615),
    [anon_sym_break] = ACTIONS(2615),
    [anon_sym_const] = ACTIONS(2615),
    [anon_sym_continue] = ACTIONS(2615),
    [anon_sym_default] = ACTIONS(2615),
    [anon_sym_enum] = ACTIONS(2615),
    [anon_sym_fn] = ACTIONS(2615),
    [anon_sym_for] = ACTIONS(2615),
    [anon_sym_gen] = ACTIONS(2615),
    [anon_sym_if] = ACTIONS(2615),
    [anon_sym_impl] = ACTIONS(2615),
    [anon_sym_let] = ACTIONS(2615),
    [anon_sym_loop] = ACTIONS(2615),
    [anon_sym_match] = ACTIONS(2615),
    [anon_sym_mod] = ACTIONS(2615),
    [anon_sym_pub] = ACTIONS(2615),
    [anon_sym_return] = ACTIONS(2615),
    [anon_sym_static] = ACTIONS(2615),
    [anon_sym_struct] = ACTIONS(2615),
    [anon_sym_trait] = ACTIONS(2615),
    [anon_sym_type] = ACTIONS(2615),
    [anon_sym_union] = ACTIONS(2615),
    [anon_sym_unsafe] = ACTIONS(2615),
    [anon_sym_use] = ACTIONS(2615),
    [anon_sym_while] = ACTIONS(2615),
    [anon_sym_extern] = ACTIONS(2615),
    [anon_sym_yield] = ACTIONS(2615),
    [anon_sym_move] = ACTIONS(2615),
    [anon_sym_try] = ACTIONS(2615),
    [sym_integer_literal] = ACTIONS(2613),
    [aux_sym_string_literal_token1] = ACTIONS(2613),
    [sym_char_literal] = ACTIONS(2613),
    [anon_sym_true] = ACTIONS(2615),
    [anon_sym_false] = ACTIONS(2615),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2615),
    [sym_super] = ACTIONS(2615),
    [sym_crate] = ACTIONS(2615),
    [sym_metavariable] = ACTIONS(2613),
    [sym__raw_string_literal_start] = ACTIONS(2613),
    [sym_float_literal] = ACTIONS(2613),
  },
  [691] = {
    [sym_line_comment] = STATE(691),
    [sym_block_comment] = STATE(691),
    [ts_builtin_sym_end] = ACTIONS(2617),
    [sym_identifier] = ACTIONS(2619),
    [anon_sym_SEMI] = ACTIONS(2617),
    [anon_sym_macro_rules_BANG] = ACTIONS(2617),
    [anon_sym_LPAREN] = ACTIONS(2617),
    [anon_sym_LBRACK] = ACTIONS(2617),
    [anon_sym_LBRACE] = ACTIONS(2617),
    [anon_sym_RBRACE] = ACTIONS(2617),
    [anon_sym_STAR] = ACTIONS(2617),
    [anon_sym_u8] = ACTIONS(2619),
    [anon_sym_i8] = ACTIONS(2619),
    [anon_sym_u16] = ACTIONS(2619),
    [anon_sym_i16] = ACTIONS(2619),
    [anon_sym_u32] = ACTIONS(2619),
    [anon_sym_i32] = ACTIONS(2619),
    [anon_sym_u64] = ACTIONS(2619),
    [anon_sym_i64] = ACTIONS(2619),
    [anon_sym_u128] = ACTIONS(2619),
    [anon_sym_i128] = ACTIONS(2619),
    [anon_sym_isize] = ACTIONS(2619),
    [anon_sym_usize] = ACTIONS(2619),
    [anon_sym_f32] = ACTIONS(2619),
    [anon_sym_f64] = ACTIONS(2619),
    [anon_sym_bool] = ACTIONS(2619),
    [anon_sym_str] = ACTIONS(2619),
    [anon_sym_char] = ACTIONS(2619),
    [anon_sym_DASH] = ACTIONS(2617),
    [anon_sym_BANG] = ACTIONS(2617),
    [anon_sym_AMP] = ACTIONS(2617),
    [anon_sym_PIPE] = ACTIONS(2617),
    [anon_sym_LT] = ACTIONS(2617),
    [anon_sym_DOT_DOT] = ACTIONS(2617),
    [anon_sym_COLON_COLON] = ACTIONS(2617),
    [anon_sym_POUND] = ACTIONS(2617),
    [anon_sym_SQUOTE] = ACTIONS(2619),
    [anon_sym_async] = ACTIONS(2619),
    [anon_sym_break] = ACTIONS(2619),
    [anon_sym_const] = ACTIONS(2619),
    [anon_sym_continue] = ACTIONS(2619),
    [anon_sym_default] = ACTIONS(2619),
    [anon_sym_enum] = ACTIONS(2619),
    [anon_sym_fn] = ACTIONS(2619),
    [anon_sym_for] = ACTIONS(2619),
    [anon_sym_gen] = ACTIONS(2619),
    [anon_sym_if] = ACTIONS(2619),
    [anon_sym_impl] = ACTIONS(2619),
    [anon_sym_let] = ACTIONS(2619),
    [anon_sym_loop] = ACTIONS(2619),
    [anon_sym_match] = ACTIONS(2619),
    [anon_sym_mod] = ACTIONS(2619),
    [anon_sym_pub] = ACTIONS(2619),
    [anon_sym_return] = ACTIONS(2619),
    [anon_sym_static] = ACTIONS(2619),
    [anon_sym_struct] = ACTIONS(2619),
    [anon_sym_trait] = ACTIONS(2619),
    [anon_sym_type] = ACTIONS(2619),
    [anon_sym_union] = ACTIONS(2619),
    [anon_sym_unsafe] = ACTIONS(2619),
    [anon_sym_use] = ACTIONS(2619),
    [anon_sym_while] = ACTIONS(2619),
    [anon_sym_extern] = ACTIONS(2619),
    [anon_sym_yield] = ACTIONS(2619),
    [anon_sym_move] = ACTIONS(2619),
    [anon_sym_try] = ACTIONS(2619),
    [sym_integer_literal] = ACTIONS(2617),
    [aux_sym_string_literal_token1] = ACTIONS(2617),
    [sym_char_literal] = ACTIONS(2617),
    [anon_sym_true] = ACTIONS(2619),
    [anon_sym_false] = ACTIONS(2619),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2619),
    [sym_super] = ACTIONS(2619),
    [sym_crate] = ACTIONS(2619),
    [sym_metavariable] = ACTIONS(2617),
    [sym__raw_string_literal_start] = ACTIONS(2617),
    [sym_float_literal] = ACTIONS(2617),
  },
  [692] = {
    [sym_line_comment] = STATE(692),
    [sym_block_comment] = STATE(692),
    [ts_builtin_sym_end] = ACTIONS(2621),
    [sym_identifier] = ACTIONS(2623),
    [anon_sym_SEMI] = ACTIONS(2621),
    [anon_sym_macro_rules_BANG] = ACTIONS(2621),
    [anon_sym_LPAREN] = ACTIONS(2621),
    [anon_sym_LBRACK] = ACTIONS(2621),
    [anon_sym_LBRACE] = ACTIONS(2621),
    [anon_sym_RBRACE] = ACTIONS(2621),
    [anon_sym_STAR] = ACTIONS(2621),
    [anon_sym_u8] = ACTIONS(2623),
    [anon_sym_i8] = ACTIONS(2623),
    [anon_sym_u16] = ACTIONS(2623),
    [anon_sym_i16] = ACTIONS(2623),
    [anon_sym_u32] = ACTIONS(2623),
    [anon_sym_i32] = ACTIONS(2623),
    [anon_sym_u64] = ACTIONS(2623),
    [anon_sym_i64] = ACTIONS(2623),
    [anon_sym_u128] = ACTIONS(2623),
    [anon_sym_i128] = ACTIONS(2623),
    [anon_sym_isize] = ACTIONS(2623),
    [anon_sym_usize] = ACTIONS(2623),
    [anon_sym_f32] = ACTIONS(2623),
    [anon_sym_f64] = ACTIONS(2623),
    [anon_sym_bool] = ACTIONS(2623),
    [anon_sym_str] = ACTIONS(2623),
    [anon_sym_char] = ACTIONS(2623),
    [anon_sym_DASH] = ACTIONS(2621),
    [anon_sym_BANG] = ACTIONS(2621),
    [anon_sym_AMP] = ACTIONS(2621),
    [anon_sym_PIPE] = ACTIONS(2621),
    [anon_sym_LT] = ACTIONS(2621),
    [anon_sym_DOT_DOT] = ACTIONS(2621),
    [anon_sym_COLON_COLON] = ACTIONS(2621),
    [anon_sym_POUND] = ACTIONS(2621),
    [anon_sym_SQUOTE] = ACTIONS(2623),
    [anon_sym_async] = ACTIONS(2623),
    [anon_sym_break] = ACTIONS(2623),
    [anon_sym_const] = ACTIONS(2623),
    [anon_sym_continue] = ACTIONS(2623),
    [anon_sym_default] = ACTIONS(2623),
    [anon_sym_enum] = ACTIONS(2623),
    [anon_sym_fn] = ACTIONS(2623),
    [anon_sym_for] = ACTIONS(2623),
    [anon_sym_gen] = ACTIONS(2623),
    [anon_sym_if] = ACTIONS(2623),
    [anon_sym_impl] = ACTIONS(2623),
    [anon_sym_let] = ACTIONS(2623),
    [anon_sym_loop] = ACTIONS(2623),
    [anon_sym_match] = ACTIONS(2623),
    [anon_sym_mod] = ACTIONS(2623),
    [anon_sym_pub] = ACTIONS(2623),
    [anon_sym_return] = ACTIONS(2623),
    [anon_sym_static] = ACTIONS(2623),
    [anon_sym_struct] = ACTIONS(2623),
    [anon_sym_trait] = ACTIONS(2623),
    [anon_sym_type] = ACTIONS(2623),
    [anon_sym_union] = ACTIONS(2623),
    [anon_sym_unsafe] = ACTIONS(2623),
    [anon_sym_use] = ACTIONS(2623),
    [anon_sym_while] = ACTIONS(2623),
    [anon_sym_extern] = ACTIONS(2623),
    [anon_sym_yield] = ACTIONS(2623),
    [anon_sym_move] = ACTIONS(2623),
    [anon_sym_try] = ACTIONS(2623),
    [sym_integer_literal] = ACTIONS(2621),
    [aux_sym_string_literal_token1] = ACTIONS(2621),
    [sym_char_literal] = ACTIONS(2621),
    [anon_sym_true] = ACTIONS(2623),
    [anon_sym_false] = ACTIONS(2623),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2623),
    [sym_super] = ACTIONS(2623),
    [sym_crate] = ACTIONS(2623),
    [sym_metavariable] = ACTIONS(2621),
    [sym__raw_string_literal_start] = ACTIONS(2621),
    [sym_float_literal] = ACTIONS(2621),
  },
  [693] = {
    [sym_line_comment] = STATE(693),
    [sym_block_comment] = STATE(693),
    [ts_builtin_sym_end] = ACTIONS(2625),
    [sym_identifier] = ACTIONS(2627),
    [anon_sym_SEMI] = ACTIONS(2625),
    [anon_sym_macro_rules_BANG] = ACTIONS(2625),
    [anon_sym_LPAREN] = ACTIONS(2625),
    [anon_sym_LBRACK] = ACTIONS(2625),
    [anon_sym_LBRACE] = ACTIONS(2625),
    [anon_sym_RBRACE] = ACTIONS(2625),
    [anon_sym_STAR] = ACTIONS(2625),
    [anon_sym_u8] = ACTIONS(2627),
    [anon_sym_i8] = ACTIONS(2627),
    [anon_sym_u16] = ACTIONS(2627),
    [anon_sym_i16] = ACTIONS(2627),
    [anon_sym_u32] = ACTIONS(2627),
    [anon_sym_i32] = ACTIONS(2627),
    [anon_sym_u64] = ACTIONS(2627),
    [anon_sym_i64] = ACTIONS(2627),
    [anon_sym_u128] = ACTIONS(2627),
    [anon_sym_i128] = ACTIONS(2627),
    [anon_sym_isize] = ACTIONS(2627),
    [anon_sym_usize] = ACTIONS(2627),
    [anon_sym_f32] = ACTIONS(2627),
    [anon_sym_f64] = ACTIONS(2627),
    [anon_sym_bool] = ACTIONS(2627),
    [anon_sym_str] = ACTIONS(2627),
    [anon_sym_char] = ACTIONS(2627),
    [anon_sym_DASH] = ACTIONS(2625),
    [anon_sym_BANG] = ACTIONS(2625),
    [anon_sym_AMP] = ACTIONS(2625),
    [anon_sym_PIPE] = ACTIONS(2625),
    [anon_sym_LT] = ACTIONS(2625),
    [anon_sym_DOT_DOT] = ACTIONS(2625),
    [anon_sym_COLON_COLON] = ACTIONS(2625),
    [anon_sym_POUND] = ACTIONS(2625),
    [anon_sym_SQUOTE] = ACTIONS(2627),
    [anon_sym_async] = ACTIONS(2627),
    [anon_sym_break] = ACTIONS(2627),
    [anon_sym_const] = ACTIONS(2627),
    [anon_sym_continue] = ACTIONS(2627),
    [anon_sym_default] = ACTIONS(2627),
    [anon_sym_enum] = ACTIONS(2627),
    [anon_sym_fn] = ACTIONS(2627),
    [anon_sym_for] = ACTIONS(2627),
    [anon_sym_gen] = ACTIONS(2627),
    [anon_sym_if] = ACTIONS(2627),
    [anon_sym_impl] = ACTIONS(2627),
    [anon_sym_let] = ACTIONS(2627),
    [anon_sym_loop] = ACTIONS(2627),
    [anon_sym_match] = ACTIONS(2627),
    [anon_sym_mod] = ACTIONS(2627),
    [anon_sym_pub] = ACTIONS(2627),
    [anon_sym_return] = ACTIONS(2627),
    [anon_sym_static] = ACTIONS(2627),
    [anon_sym_struct] = ACTIONS(2627),
    [anon_sym_trait] = ACTIONS(2627),
    [anon_sym_type] = ACTIONS(2627),
    [anon_sym_union] = ACTIONS(2627),
    [anon_sym_unsafe] = ACTIONS(2627),
    [anon_sym_use] = ACTIONS(2627),
    [anon_sym_while] = ACTIONS(2627),
    [anon_sym_extern] = ACTIONS(2627),
    [anon_sym_yield] = ACTIONS(2627),
    [anon_sym_move] = ACTIONS(2627),
    [anon_sym_try] = ACTIONS(2627),
    [sym_integer_literal] = ACTIONS(2625),
    [aux_sym_string_literal_token1] = ACTIONS(2625),
    [sym_char_literal] = ACTIONS(2625),
    [anon_sym_true] = ACTIONS(2627),
    [anon_sym_false] = ACTIONS(2627),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2627),
    [sym_super] = ACTIONS(2627),
    [sym_crate] = ACTIONS(2627),
    [sym_metavariable] = ACTIONS(2625),
    [sym__raw_string_literal_start] = ACTIONS(2625),
    [sym_float_literal] = ACTIONS(2625),
  },
  [694] = {
    [sym_line_comment] = STATE(694),
    [sym_block_comment] = STATE(694),
    [ts_builtin_sym_end] = ACTIONS(2629),
    [sym_identifier] = ACTIONS(2631),
    [anon_sym_SEMI] = ACTIONS(2629),
    [anon_sym_macro_rules_BANG] = ACTIONS(2629),
    [anon_sym_LPAREN] = ACTIONS(2629),
    [anon_sym_LBRACK] = ACTIONS(2629),
    [anon_sym_LBRACE] = ACTIONS(2629),
    [anon_sym_RBRACE] = ACTIONS(2629),
    [anon_sym_STAR] = ACTIONS(2629),
    [anon_sym_u8] = ACTIONS(2631),
    [anon_sym_i8] = ACTIONS(2631),
    [anon_sym_u16] = ACTIONS(2631),
    [anon_sym_i16] = ACTIONS(2631),
    [anon_sym_u32] = ACTIONS(2631),
    [anon_sym_i32] = ACTIONS(2631),
    [anon_sym_u64] = ACTIONS(2631),
    [anon_sym_i64] = ACTIONS(2631),
    [anon_sym_u128] = ACTIONS(2631),
    [anon_sym_i128] = ACTIONS(2631),
    [anon_sym_isize] = ACTIONS(2631),
    [anon_sym_usize] = ACTIONS(2631),
    [anon_sym_f32] = ACTIONS(2631),
    [anon_sym_f64] = ACTIONS(2631),
    [anon_sym_bool] = ACTIONS(2631),
    [anon_sym_str] = ACTIONS(2631),
    [anon_sym_char] = ACTIONS(2631),
    [anon_sym_DASH] = ACTIONS(2629),
    [anon_sym_BANG] = ACTIONS(2629),
    [anon_sym_AMP] = ACTIONS(2629),
    [anon_sym_PIPE] = ACTIONS(2629),
    [anon_sym_LT] = ACTIONS(2629),
    [anon_sym_DOT_DOT] = ACTIONS(2629),
    [anon_sym_COLON_COLON] = ACTIONS(2629),
    [anon_sym_POUND] = ACTIONS(2629),
    [anon_sym_SQUOTE] = ACTIONS(2631),
    [anon_sym_async] = ACTIONS(2631),
    [anon_sym_break] = ACTIONS(2631),
    [anon_sym_const] = ACTIONS(2631),
    [anon_sym_continue] = ACTIONS(2631),
    [anon_sym_default] = ACTIONS(2631),
    [anon_sym_enum] = ACTIONS(2631),
    [anon_sym_fn] = ACTIONS(2631),
    [anon_sym_for] = ACTIONS(2631),
    [anon_sym_gen] = ACTIONS(2631),
    [anon_sym_if] = ACTIONS(2631),
    [anon_sym_impl] = ACTIONS(2631),
    [anon_sym_let] = ACTIONS(2631),
    [anon_sym_loop] = ACTIONS(2631),
    [anon_sym_match] = ACTIONS(2631),
    [anon_sym_mod] = ACTIONS(2631),
    [anon_sym_pub] = ACTIONS(2631),
    [anon_sym_return] = ACTIONS(2631),
    [anon_sym_static] = ACTIONS(2631),
    [anon_sym_struct] = ACTIONS(2631),
    [anon_sym_trait] = ACTIONS(2631),
    [anon_sym_type] = ACTIONS(2631),
    [anon_sym_union] = ACTIONS(2631),
    [anon_sym_unsafe] = ACTIONS(2631),
    [anon_sym_use] = ACTIONS(2631),
    [anon_sym_while] = ACTIONS(2631),
    [anon_sym_extern] = ACTIONS(2631),
    [anon_sym_yield] = ACTIONS(2631),
    [anon_sym_move] = ACTIONS(2631),
    [anon_sym_try] = ACTIONS(2631),
    [sym_integer_literal] = ACTIONS(2629),
    [aux_sym_string_literal_token1] = ACTIONS(2629),
    [sym_char_literal] = ACTIONS(2629),
    [anon_sym_true] = ACTIONS(2631),
    [anon_sym_false] = ACTIONS(2631),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2631),
    [sym_super] = ACTIONS(2631),
    [sym_crate] = ACTIONS(2631),
    [sym_metavariable] = ACTIONS(2629),
    [sym__raw_string_literal_start] = ACTIONS(2629),
    [sym_float_literal] = ACTIONS(2629),
  },
  [695] = {
    [sym_line_comment] = STATE(695),
    [sym_block_comment] = STATE(695),
    [ts_builtin_sym_end] = ACTIONS(2633),
    [sym_identifier] = ACTIONS(2635),
    [anon_sym_SEMI] = ACTIONS(2633),
    [anon_sym_macro_rules_BANG] = ACTIONS(2633),
    [anon_sym_LPAREN] = ACTIONS(2633),
    [anon_sym_LBRACK] = ACTIONS(2633),
    [anon_sym_LBRACE] = ACTIONS(2633),
    [anon_sym_RBRACE] = ACTIONS(2633),
    [anon_sym_STAR] = ACTIONS(2633),
    [anon_sym_u8] = ACTIONS(2635),
    [anon_sym_i8] = ACTIONS(2635),
    [anon_sym_u16] = ACTIONS(2635),
    [anon_sym_i16] = ACTIONS(2635),
    [anon_sym_u32] = ACTIONS(2635),
    [anon_sym_i32] = ACTIONS(2635),
    [anon_sym_u64] = ACTIONS(2635),
    [anon_sym_i64] = ACTIONS(2635),
    [anon_sym_u128] = ACTIONS(2635),
    [anon_sym_i128] = ACTIONS(2635),
    [anon_sym_isize] = ACTIONS(2635),
    [anon_sym_usize] = ACTIONS(2635),
    [anon_sym_f32] = ACTIONS(2635),
    [anon_sym_f64] = ACTIONS(2635),
    [anon_sym_bool] = ACTIONS(2635),
    [anon_sym_str] = ACTIONS(2635),
    [anon_sym_char] = ACTIONS(2635),
    [anon_sym_DASH] = ACTIONS(2633),
    [anon_sym_BANG] = ACTIONS(2633),
    [anon_sym_AMP] = ACTIONS(2633),
    [anon_sym_PIPE] = ACTIONS(2633),
    [anon_sym_LT] = ACTIONS(2633),
    [anon_sym_DOT_DOT] = ACTIONS(2633),
    [anon_sym_COLON_COLON] = ACTIONS(2633),
    [anon_sym_POUND] = ACTIONS(2633),
    [anon_sym_SQUOTE] = ACTIONS(2635),
    [anon_sym_async] = ACTIONS(2635),
    [anon_sym_break] = ACTIONS(2635),
    [anon_sym_const] = ACTIONS(2635),
    [anon_sym_continue] = ACTIONS(2635),
    [anon_sym_default] = ACTIONS(2635),
    [anon_sym_enum] = ACTIONS(2635),
    [anon_sym_fn] = ACTIONS(2635),
    [anon_sym_for] = ACTIONS(2635),
    [anon_sym_gen] = ACTIONS(2635),
    [anon_sym_if] = ACTIONS(2635),
    [anon_sym_impl] = ACTIONS(2635),
    [anon_sym_let] = ACTIONS(2635),
    [anon_sym_loop] = ACTIONS(2635),
    [anon_sym_match] = ACTIONS(2635),
    [anon_sym_mod] = ACTIONS(2635),
    [anon_sym_pub] = ACTIONS(2635),
    [anon_sym_return] = ACTIONS(2635),
    [anon_sym_static] = ACTIONS(2635),
    [anon_sym_struct] = ACTIONS(2635),
    [anon_sym_trait] = ACTIONS(2635),
    [anon_sym_type] = ACTIONS(2635),
    [anon_sym_union] = ACTIONS(2635),
    [anon_sym_unsafe] = ACTIONS(2635),
    [anon_sym_use] = ACTIONS(2635),
    [anon_sym_while] = ACTIONS(2635),
    [anon_sym_extern] = ACTIONS(2635),
    [anon_sym_yield] = ACTIONS(2635),
    [anon_sym_move] = ACTIONS(2635),
    [anon_sym_try] = ACTIONS(2635),
    [sym_integer_literal] = ACTIONS(2633),
    [aux_sym_string_literal_token1] = ACTIONS(2633),
    [sym_char_literal] = ACTIONS(2633),
    [anon_sym_true] = ACTIONS(2635),
    [anon_sym_false] = ACTIONS(2635),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2635),
    [sym_super] = ACTIONS(2635),
    [sym_crate] = ACTIONS(2635),
    [sym_metavariable] = ACTIONS(2633),
    [sym__raw_string_literal_start] = ACTIONS(2633),
    [sym_float_literal] = ACTIONS(2633),
  },
  [696] = {
    [sym_line_comment] = STATE(696),
    [sym_block_comment] = STATE(696),
    [ts_builtin_sym_end] = ACTIONS(2637),
    [sym_identifier] = ACTIONS(2639),
    [anon_sym_SEMI] = ACTIONS(2637),
    [anon_sym_macro_rules_BANG] = ACTIONS(2637),
    [anon_sym_LPAREN] = ACTIONS(2637),
    [anon_sym_LBRACK] = ACTIONS(2637),
    [anon_sym_LBRACE] = ACTIONS(2637),
    [anon_sym_RBRACE] = ACTIONS(2637),
    [anon_sym_STAR] = ACTIONS(2637),
    [anon_sym_u8] = ACTIONS(2639),
    [anon_sym_i8] = ACTIONS(2639),
    [anon_sym_u16] = ACTIONS(2639),
    [anon_sym_i16] = ACTIONS(2639),
    [anon_sym_u32] = ACTIONS(2639),
    [anon_sym_i32] = ACTIONS(2639),
    [anon_sym_u64] = ACTIONS(2639),
    [anon_sym_i64] = ACTIONS(2639),
    [anon_sym_u128] = ACTIONS(2639),
    [anon_sym_i128] = ACTIONS(2639),
    [anon_sym_isize] = ACTIONS(2639),
    [anon_sym_usize] = ACTIONS(2639),
    [anon_sym_f32] = ACTIONS(2639),
    [anon_sym_f64] = ACTIONS(2639),
    [anon_sym_bool] = ACTIONS(2639),
    [anon_sym_str] = ACTIONS(2639),
    [anon_sym_char] = ACTIONS(2639),
    [anon_sym_DASH] = ACTIONS(2637),
    [anon_sym_BANG] = ACTIONS(2637),
    [anon_sym_AMP] = ACTIONS(2637),
    [anon_sym_PIPE] = ACTIONS(2637),
    [anon_sym_LT] = ACTIONS(2637),
    [anon_sym_DOT_DOT] = ACTIONS(2637),
    [anon_sym_COLON_COLON] = ACTIONS(2637),
    [anon_sym_POUND] = ACTIONS(2637),
    [anon_sym_SQUOTE] = ACTIONS(2639),
    [anon_sym_async] = ACTIONS(2639),
    [anon_sym_break] = ACTIONS(2639),
    [anon_sym_const] = ACTIONS(2639),
    [anon_sym_continue] = ACTIONS(2639),
    [anon_sym_default] = ACTIONS(2639),
    [anon_sym_enum] = ACTIONS(2639),
    [anon_sym_fn] = ACTIONS(2639),
    [anon_sym_for] = ACTIONS(2639),
    [anon_sym_gen] = ACTIONS(2639),
    [anon_sym_if] = ACTIONS(2639),
    [anon_sym_impl] = ACTIONS(2639),
    [anon_sym_let] = ACTIONS(2639),
    [anon_sym_loop] = ACTIONS(2639),
    [anon_sym_match] = ACTIONS(2639),
    [anon_sym_mod] = ACTIONS(2639),
    [anon_sym_pub] = ACTIONS(2639),
    [anon_sym_return] = ACTIONS(2639),
    [anon_sym_static] = ACTIONS(2639),
    [anon_sym_struct] = ACTIONS(2639),
    [anon_sym_trait] = ACTIONS(2639),
    [anon_sym_type] = ACTIONS(2639),
    [anon_sym_union] = ACTIONS(2639),
    [anon_sym_unsafe] = ACTIONS(2639),
    [anon_sym_use] = ACTIONS(2639),
    [anon_sym_while] = ACTIONS(2639),
    [anon_sym_extern] = ACTIONS(2639),
    [anon_sym_yield] = ACTIONS(2639),
    [anon_sym_move] = ACTIONS(2639),
    [anon_sym_try] = ACTIONS(2639),
    [sym_integer_literal] = ACTIONS(2637),
    [aux_sym_string_literal_token1] = ACTIONS(2637),
    [sym_char_literal] = ACTIONS(2637),
    [anon_sym_true] = ACTIONS(2639),
    [anon_sym_false] = ACTIONS(2639),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2639),
    [sym_super] = ACTIONS(2639),
    [sym_crate] = ACTIONS(2639),
    [sym_metavariable] = ACTIONS(2637),
    [sym__raw_string_literal_start] = ACTIONS(2637),
    [sym_float_literal] = ACTIONS(2637),
  },
  [697] = {
    [sym_line_comment] = STATE(697),
    [sym_block_comment] = STATE(697),
    [ts_builtin_sym_end] = ACTIONS(2641),
    [sym_identifier] = ACTIONS(2643),
    [anon_sym_SEMI] = ACTIONS(2641),
    [anon_sym_macro_rules_BANG] = ACTIONS(2641),
    [anon_sym_LPAREN] = ACTIONS(2641),
    [anon_sym_LBRACK] = ACTIONS(2641),
    [anon_sym_LBRACE] = ACTIONS(2641),
    [anon_sym_RBRACE] = ACTIONS(2641),
    [anon_sym_STAR] = ACTIONS(2641),
    [anon_sym_u8] = ACTIONS(2643),
    [anon_sym_i8] = ACTIONS(2643),
    [anon_sym_u16] = ACTIONS(2643),
    [anon_sym_i16] = ACTIONS(2643),
    [anon_sym_u32] = ACTIONS(2643),
    [anon_sym_i32] = ACTIONS(2643),
    [anon_sym_u64] = ACTIONS(2643),
    [anon_sym_i64] = ACTIONS(2643),
    [anon_sym_u128] = ACTIONS(2643),
    [anon_sym_i128] = ACTIONS(2643),
    [anon_sym_isize] = ACTIONS(2643),
    [anon_sym_usize] = ACTIONS(2643),
    [anon_sym_f32] = ACTIONS(2643),
    [anon_sym_f64] = ACTIONS(2643),
    [anon_sym_bool] = ACTIONS(2643),
    [anon_sym_str] = ACTIONS(2643),
    [anon_sym_char] = ACTIONS(2643),
    [anon_sym_DASH] = ACTIONS(2641),
    [anon_sym_BANG] = ACTIONS(2641),
    [anon_sym_AMP] = ACTIONS(2641),
    [anon_sym_PIPE] = ACTIONS(2641),
    [anon_sym_LT] = ACTIONS(2641),
    [anon_sym_DOT_DOT] = ACTIONS(2641),
    [anon_sym_COLON_COLON] = ACTIONS(2641),
    [anon_sym_POUND] = ACTIONS(2641),
    [anon_sym_SQUOTE] = ACTIONS(2643),
    [anon_sym_async] = ACTIONS(2643),
    [anon_sym_break] = ACTIONS(2643),
    [anon_sym_const] = ACTIONS(2643),
    [anon_sym_continue] = ACTIONS(2643),
    [anon_sym_default] = ACTIONS(2643),
    [anon_sym_enum] = ACTIONS(2643),
    [anon_sym_fn] = ACTIONS(2643),
    [anon_sym_for] = ACTIONS(2643),
    [anon_sym_gen] = ACTIONS(2643),
    [anon_sym_if] = ACTIONS(2643),
    [anon_sym_impl] = ACTIONS(2643),
    [anon_sym_let] = ACTIONS(2643),
    [anon_sym_loop] = ACTIONS(2643),
    [anon_sym_match] = ACTIONS(2643),
    [anon_sym_mod] = ACTIONS(2643),
    [anon_sym_pub] = ACTIONS(2643),
    [anon_sym_return] = ACTIONS(2643),
    [anon_sym_static] = ACTIONS(2643),
    [anon_sym_struct] = ACTIONS(2643),
    [anon_sym_trait] = ACTIONS(2643),
    [anon_sym_type] = ACTIONS(2643),
    [anon_sym_union] = ACTIONS(2643),
    [anon_sym_unsafe] = ACTIONS(2643),
    [anon_sym_use] = ACTIONS(2643),
    [anon_sym_while] = ACTIONS(2643),
    [anon_sym_extern] = ACTIONS(2643),
    [anon_sym_yield] = ACTIONS(2643),
    [anon_sym_move] = ACTIONS(2643),
    [anon_sym_try] = ACTIONS(2643),
    [sym_integer_literal] = ACTIONS(2641),
    [aux_sym_string_literal_token1] = ACTIONS(2641),
    [sym_char_literal] = ACTIONS(2641),
    [anon_sym_true] = ACTIONS(2643),
    [anon_sym_false] = ACTIONS(2643),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2643),
    [sym_super] = ACTIONS(2643),
    [sym_crate] = ACTIONS(2643),
    [sym_metavariable] = ACTIONS(2641),
    [sym__raw_string_literal_start] = ACTIONS(2641),
    [sym_float_literal] = ACTIONS(2641),
  },
  [698] = {
    [sym_line_comment] = STATE(698),
    [sym_block_comment] = STATE(698),
    [ts_builtin_sym_end] = ACTIONS(2645),
    [sym_identifier] = ACTIONS(2647),
    [anon_sym_SEMI] = ACTIONS(2645),
    [anon_sym_macro_rules_BANG] = ACTIONS(2645),
    [anon_sym_LPAREN] = ACTIONS(2645),
    [anon_sym_LBRACK] = ACTIONS(2645),
    [anon_sym_LBRACE] = ACTIONS(2645),
    [anon_sym_RBRACE] = ACTIONS(2645),
    [anon_sym_STAR] = ACTIONS(2645),
    [anon_sym_u8] = ACTIONS(2647),
    [anon_sym_i8] = ACTIONS(2647),
    [anon_sym_u16] = ACTIONS(2647),
    [anon_sym_i16] = ACTIONS(2647),
    [anon_sym_u32] = ACTIONS(2647),
    [anon_sym_i32] = ACTIONS(2647),
    [anon_sym_u64] = ACTIONS(2647),
    [anon_sym_i64] = ACTIONS(2647),
    [anon_sym_u128] = ACTIONS(2647),
    [anon_sym_i128] = ACTIONS(2647),
    [anon_sym_isize] = ACTIONS(2647),
    [anon_sym_usize] = ACTIONS(2647),
    [anon_sym_f32] = ACTIONS(2647),
    [anon_sym_f64] = ACTIONS(2647),
    [anon_sym_bool] = ACTIONS(2647),
    [anon_sym_str] = ACTIONS(2647),
    [anon_sym_char] = ACTIONS(2647),
    [anon_sym_DASH] = ACTIONS(2645),
    [anon_sym_BANG] = ACTIONS(2645),
    [anon_sym_AMP] = ACTIONS(2645),
    [anon_sym_PIPE] = ACTIONS(2645),
    [anon_sym_LT] = ACTIONS(2645),
    [anon_sym_DOT_DOT] = ACTIONS(2645),
    [anon_sym_COLON_COLON] = ACTIONS(2645),
    [anon_sym_POUND] = ACTIONS(2645),
    [anon_sym_SQUOTE] = ACTIONS(2647),
    [anon_sym_async] = ACTIONS(2647),
    [anon_sym_break] = ACTIONS(2647),
    [anon_sym_const] = ACTIONS(2647),
    [anon_sym_continue] = ACTIONS(2647),
    [anon_sym_default] = ACTIONS(2647),
    [anon_sym_enum] = ACTIONS(2647),
    [anon_sym_fn] = ACTIONS(2647),
    [anon_sym_for] = ACTIONS(2647),
    [anon_sym_gen] = ACTIONS(2647),
    [anon_sym_if] = ACTIONS(2647),
    [anon_sym_impl] = ACTIONS(2647),
    [anon_sym_let] = ACTIONS(2647),
    [anon_sym_loop] = ACTIONS(2647),
    [anon_sym_match] = ACTIONS(2647),
    [anon_sym_mod] = ACTIONS(2647),
    [anon_sym_pub] = ACTIONS(2647),
    [anon_sym_return] = ACTIONS(2647),
    [anon_sym_static] = ACTIONS(2647),
    [anon_sym_struct] = ACTIONS(2647),
    [anon_sym_trait] = ACTIONS(2647),
    [anon_sym_type] = ACTIONS(2647),
    [anon_sym_union] = ACTIONS(2647),
    [anon_sym_unsafe] = ACTIONS(2647),
    [anon_sym_use] = ACTIONS(2647),
    [anon_sym_while] = ACTIONS(2647),
    [anon_sym_extern] = ACTIONS(2647),
    [anon_sym_yield] = ACTIONS(2647),
    [anon_sym_move] = ACTIONS(2647),
    [anon_sym_try] = ACTIONS(2647),
    [sym_integer_literal] = ACTIONS(2645),
    [aux_sym_string_literal_token1] = ACTIONS(2645),
    [sym_char_literal] = ACTIONS(2645),
    [anon_sym_true] = ACTIONS(2647),
    [anon_sym_false] = ACTIONS(2647),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2647),
    [sym_super] = ACTIONS(2647),
    [sym_crate] = ACTIONS(2647),
    [sym_metavariable] = ACTIONS(2645),
    [sym__raw_string_literal_start] = ACTIONS(2645),
    [sym_float_literal] = ACTIONS(2645),
  },
  [699] = {
    [sym_attribute_item] = STATE(1468),
    [sym_inner_attribute_item] = STATE(1468),
    [sym_bracketed_type] = STATE(3553),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3287),
    [sym_macro_invocation] = STATE(2757),
    [sym_scoped_identifier] = STATE(2151),
    [sym_scoped_type_identifier] = STATE(2982),
    [sym_match_arm] = STATE(1469),
    [sym_match_pattern] = STATE(3616),
    [sym_const_block] = STATE(2757),
    [sym__pattern] = STATE(2907),
    [sym_tuple_pattern] = STATE(2757),
    [sym_slice_pattern] = STATE(2757),
    [sym_tuple_struct_pattern] = STATE(2757),
    [sym_struct_pattern] = STATE(2757),
    [sym_remaining_field_pattern] = STATE(2757),
    [sym_mut_pattern] = STATE(2757),
    [sym_range_pattern] = STATE(2757),
    [sym_ref_pattern] = STATE(2757),
    [sym_captured_pattern] = STATE(2757),
    [sym_reference_pattern] = STATE(2757),
    [sym_or_pattern] = STATE(2757),
    [sym__literal_pattern] = STATE(2330),
    [sym_negative_literal] = STATE(2364),
    [sym_string_literal] = STATE(2364),
    [sym_raw_string_literal] = STATE(2364),
    [sym_boolean_literal] = STATE(2364),
    [sym_line_comment] = STATE(699),
    [sym_block_comment] = STATE(699),
    [aux_sym_match_block_repeat1] = STATE(699),
    [aux_sym_match_arm_repeat1] = STATE(769),
    [sym_identifier] = ACTIONS(2649),
    [anon_sym_LPAREN] = ACTIONS(2652),
    [anon_sym_LBRACK] = ACTIONS(2655),
    [anon_sym_u8] = ACTIONS(2658),
    [anon_sym_i8] = ACTIONS(2658),
    [anon_sym_u16] = ACTIONS(2658),
    [anon_sym_i16] = ACTIONS(2658),
    [anon_sym_u32] = ACTIONS(2658),
    [anon_sym_i32] = ACTIONS(2658),
    [anon_sym_u64] = ACTIONS(2658),
    [anon_sym_i64] = ACTIONS(2658),
    [anon_sym_u128] = ACTIONS(2658),
    [anon_sym_i128] = ACTIONS(2658),
    [anon_sym_isize] = ACTIONS(2658),
    [anon_sym_usize] = ACTIONS(2658),
    [anon_sym_f32] = ACTIONS(2658),
    [anon_sym_f64] = ACTIONS(2658),
    [anon_sym_bool] = ACTIONS(2658),
    [anon_sym_str] = ACTIONS(2658),
    [anon_sym_char] = ACTIONS(2658),
    [anon_sym_DASH] = ACTIONS(2661),
    [anon_sym_AMP] = ACTIONS(2664),
    [anon_sym_PIPE] = ACTIONS(2667),
    [anon_sym_LT] = ACTIONS(2670),
    [anon_sym__] = ACTIONS(2673),
    [anon_sym_DOT_DOT] = ACTIONS(2676),
    [anon_sym_COLON_COLON] = ACTIONS(2679),
    [anon_sym_POUND] = ACTIONS(2682),
    [anon_sym_const] = ACTIONS(2685),
    [anon_sym_default] = ACTIONS(2688),
    [anon_sym_gen] = ACTIONS(2688),
    [anon_sym_union] = ACTIONS(2688),
    [anon_sym_ref] = ACTIONS(2691),
    [sym_mutable_specifier] = ACTIONS(2694),
    [sym_integer_literal] = ACTIONS(2697),
    [aux_sym_string_literal_token1] = ACTIONS(2700),
    [sym_char_literal] = ACTIONS(2697),
    [anon_sym_true] = ACTIONS(2703),
    [anon_sym_false] = ACTIONS(2703),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2706),
    [sym_super] = ACTIONS(2706),
    [sym_crate] = ACTIONS(2706),
    [sym_metavariable] = ACTIONS(2709),
    [sym__raw_string_literal_start] = ACTIONS(2712),
    [sym_float_literal] = ACTIONS(2697),
  },
  [700] = {
    [sym_empty_statement] = STATE(1072),
    [sym_macro_definition] = STATE(1072),
    [sym_attribute_item] = STATE(1072),
    [sym_inner_attribute_item] = STATE(1072),
    [sym_mod_item] = STATE(1072),
    [sym_foreign_mod_item] = STATE(1072),
    [sym_struct_item] = STATE(1072),
    [sym_union_item] = STATE(1072),
    [sym_enum_item] = STATE(1072),
    [sym_extern_crate_declaration] = STATE(1072),
    [sym_const_item] = STATE(1072),
    [sym_static_item] = STATE(1072),
    [sym_type_item] = STATE(1072),
    [sym_function_item] = STATE(1072),
    [sym_function_signature_item] = STATE(1072),
    [sym_function_modifiers] = STATE(3622),
    [sym_impl_item] = STATE(1072),
    [sym_trait_item] = STATE(1072),
    [sym_associated_type] = STATE(1072),
    [sym_let_declaration] = STATE(1072),
    [sym_use_declaration] = STATE(1072),
    [sym_extern_modifier] = STATE(2165),
    [sym_visibility_modifier] = STATE(1954),
    [sym_bracketed_type] = STATE(3353),
    [sym_generic_type_with_turbofish] = STATE(3379),
    [sym_macro_invocation] = STATE(1072),
    [sym_scoped_identifier] = STATE(3167),
    [sym_line_comment] = STATE(700),
    [sym_block_comment] = STATE(700),
    [aux_sym_declaration_list_repeat1] = STATE(639),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(2461),
    [anon_sym_SEMI] = ACTIONS(2463),
    [anon_sym_macro_rules_BANG] = ACTIONS(2465),
    [anon_sym_RBRACE] = ACTIONS(2715),
    [anon_sym_u8] = ACTIONS(2469),
    [anon_sym_i8] = ACTIONS(2469),
    [anon_sym_u16] = ACTIONS(2469),
    [anon_sym_i16] = ACTIONS(2469),
    [anon_sym_u32] = ACTIONS(2469),
    [anon_sym_i32] = ACTIONS(2469),
    [anon_sym_u64] = ACTIONS(2469),
    [anon_sym_i64] = ACTIONS(2469),
    [anon_sym_u128] = ACTIONS(2469),
    [anon_sym_i128] = ACTIONS(2469),
    [anon_sym_isize] = ACTIONS(2469),
    [anon_sym_usize] = ACTIONS(2469),
    [anon_sym_f32] = ACTIONS(2469),
    [anon_sym_f64] = ACTIONS(2469),
    [anon_sym_bool] = ACTIONS(2469),
    [anon_sym_str] = ACTIONS(2469),
    [anon_sym_char] = ACTIONS(2469),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(2471),
    [anon_sym_POUND] = ACTIONS(2473),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(2475),
    [anon_sym_default] = ACTIONS(2477),
    [anon_sym_enum] = ACTIONS(2479),
    [anon_sym_fn] = ACTIONS(2481),
    [anon_sym_gen] = ACTIONS(2483),
    [anon_sym_impl] = ACTIONS(2485),
    [anon_sym_let] = ACTIONS(2487),
    [anon_sym_mod] = ACTIONS(2489),
    [anon_sym_pub] = ACTIONS(69),
    [anon_sym_static] = ACTIONS(2491),
    [anon_sym_struct] = ACTIONS(2493),
    [anon_sym_trait] = ACTIONS(2495),
    [anon_sym_type] = ACTIONS(2497),
    [anon_sym_union] = ACTIONS(2499),
    [anon_sym_unsafe] = ACTIONS(2501),
    [anon_sym_use] = ACTIONS(2503),
    [anon_sym_extern] = ACTIONS(2505),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2507),
    [sym_super] = ACTIONS(2507),
    [sym_crate] = ACTIONS(2509),
    [sym_metavariable] = ACTIONS(2511),
  },
  [701] = {
    [sym_line_comment] = STATE(701),
    [sym_block_comment] = STATE(701),
    [ts_builtin_sym_end] = ACTIONS(2717),
    [sym_identifier] = ACTIONS(2719),
    [anon_sym_SEMI] = ACTIONS(2717),
    [anon_sym_macro_rules_BANG] = ACTIONS(2717),
    [anon_sym_LPAREN] = ACTIONS(2717),
    [anon_sym_LBRACK] = ACTIONS(2717),
    [anon_sym_LBRACE] = ACTIONS(2717),
    [anon_sym_RBRACE] = ACTIONS(2717),
    [anon_sym_STAR] = ACTIONS(2717),
    [anon_sym_u8] = ACTIONS(2719),
    [anon_sym_i8] = ACTIONS(2719),
    [anon_sym_u16] = ACTIONS(2719),
    [anon_sym_i16] = ACTIONS(2719),
    [anon_sym_u32] = ACTIONS(2719),
    [anon_sym_i32] = ACTIONS(2719),
    [anon_sym_u64] = ACTIONS(2719),
    [anon_sym_i64] = ACTIONS(2719),
    [anon_sym_u128] = ACTIONS(2719),
    [anon_sym_i128] = ACTIONS(2719),
    [anon_sym_isize] = ACTIONS(2719),
    [anon_sym_usize] = ACTIONS(2719),
    [anon_sym_f32] = ACTIONS(2719),
    [anon_sym_f64] = ACTIONS(2719),
    [anon_sym_bool] = ACTIONS(2719),
    [anon_sym_str] = ACTIONS(2719),
    [anon_sym_char] = ACTIONS(2719),
    [anon_sym_DASH] = ACTIONS(2717),
    [anon_sym_BANG] = ACTIONS(2717),
    [anon_sym_AMP] = ACTIONS(2717),
    [anon_sym_PIPE] = ACTIONS(2717),
    [anon_sym_LT] = ACTIONS(2717),
    [anon_sym_DOT_DOT] = ACTIONS(2717),
    [anon_sym_COLON_COLON] = ACTIONS(2717),
    [anon_sym_POUND] = ACTIONS(2717),
    [anon_sym_SQUOTE] = ACTIONS(2719),
    [anon_sym_async] = ACTIONS(2719),
    [anon_sym_break] = ACTIONS(2719),
    [anon_sym_const] = ACTIONS(2719),
    [anon_sym_continue] = ACTIONS(2719),
    [anon_sym_default] = ACTIONS(2719),
    [anon_sym_enum] = ACTIONS(2719),
    [anon_sym_fn] = ACTIONS(2719),
    [anon_sym_for] = ACTIONS(2719),
    [anon_sym_gen] = ACTIONS(2719),
    [anon_sym_if] = ACTIONS(2719),
    [anon_sym_impl] = ACTIONS(2719),
    [anon_sym_let] = ACTIONS(2719),
    [anon_sym_loop] = ACTIONS(2719),
    [anon_sym_match] = ACTIONS(2719),
    [anon_sym_mod] = ACTIONS(2719),
    [anon_sym_pub] = ACTIONS(2719),
    [anon_sym_return] = ACTIONS(2719),
    [anon_sym_static] = ACTIONS(2719),
    [anon_sym_struct] = ACTIONS(2719),
    [anon_sym_trait] = ACTIONS(2719),
    [anon_sym_type] = ACTIONS(2719),
    [anon_sym_union] = ACTIONS(2719),
    [anon_sym_unsafe] = ACTIONS(2719),
    [anon_sym_use] = ACTIONS(2719),
    [anon_sym_while] = ACTIONS(2719),
    [anon_sym_extern] = ACTIONS(2719),
    [anon_sym_yield] = ACTIONS(2719),
    [anon_sym_move] = ACTIONS(2719),
    [anon_sym_try] = ACTIONS(2719),
    [sym_integer_literal] = ACTIONS(2717),
    [aux_sym_string_literal_token1] = ACTIONS(2717),
    [sym_char_literal] = ACTIONS(2717),
    [anon_sym_true] = ACTIONS(2719),
    [anon_sym_false] = ACTIONS(2719),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2719),
    [sym_super] = ACTIONS(2719),
    [sym_crate] = ACTIONS(2719),
    [sym_metavariable] = ACTIONS(2717),
    [sym__raw_string_literal_start] = ACTIONS(2717),
    [sym_float_literal] = ACTIONS(2717),
  },
  [702] = {
    [sym_line_comment] = STATE(702),
    [sym_block_comment] = STATE(702),
    [ts_builtin_sym_end] = ACTIONS(2721),
    [sym_identifier] = ACTIONS(2723),
    [anon_sym_SEMI] = ACTIONS(2721),
    [anon_sym_macro_rules_BANG] = ACTIONS(2721),
    [anon_sym_LPAREN] = ACTIONS(2721),
    [anon_sym_LBRACK] = ACTIONS(2721),
    [anon_sym_LBRACE] = ACTIONS(2721),
    [anon_sym_RBRACE] = ACTIONS(2721),
    [anon_sym_STAR] = ACTIONS(2721),
    [anon_sym_u8] = ACTIONS(2723),
    [anon_sym_i8] = ACTIONS(2723),
    [anon_sym_u16] = ACTIONS(2723),
    [anon_sym_i16] = ACTIONS(2723),
    [anon_sym_u32] = ACTIONS(2723),
    [anon_sym_i32] = ACTIONS(2723),
    [anon_sym_u64] = ACTIONS(2723),
    [anon_sym_i64] = ACTIONS(2723),
    [anon_sym_u128] = ACTIONS(2723),
    [anon_sym_i128] = ACTIONS(2723),
    [anon_sym_isize] = ACTIONS(2723),
    [anon_sym_usize] = ACTIONS(2723),
    [anon_sym_f32] = ACTIONS(2723),
    [anon_sym_f64] = ACTIONS(2723),
    [anon_sym_bool] = ACTIONS(2723),
    [anon_sym_str] = ACTIONS(2723),
    [anon_sym_char] = ACTIONS(2723),
    [anon_sym_DASH] = ACTIONS(2721),
    [anon_sym_BANG] = ACTIONS(2721),
    [anon_sym_AMP] = ACTIONS(2721),
    [anon_sym_PIPE] = ACTIONS(2721),
    [anon_sym_LT] = ACTIONS(2721),
    [anon_sym_DOT_DOT] = ACTIONS(2721),
    [anon_sym_COLON_COLON] = ACTIONS(2721),
    [anon_sym_POUND] = ACTIONS(2721),
    [anon_sym_SQUOTE] = ACTIONS(2723),
    [anon_sym_async] = ACTIONS(2723),
    [anon_sym_break] = ACTIONS(2723),
    [anon_sym_const] = ACTIONS(2723),
    [anon_sym_continue] = ACTIONS(2723),
    [anon_sym_default] = ACTIONS(2723),
    [anon_sym_enum] = ACTIONS(2723),
    [anon_sym_fn] = ACTIONS(2723),
    [anon_sym_for] = ACTIONS(2723),
    [anon_sym_gen] = ACTIONS(2723),
    [anon_sym_if] = ACTIONS(2723),
    [anon_sym_impl] = ACTIONS(2723),
    [anon_sym_let] = ACTIONS(2723),
    [anon_sym_loop] = ACTIONS(2723),
    [anon_sym_match] = ACTIONS(2723),
    [anon_sym_mod] = ACTIONS(2723),
    [anon_sym_pub] = ACTIONS(2723),
    [anon_sym_return] = ACTIONS(2723),
    [anon_sym_static] = ACTIONS(2723),
    [anon_sym_struct] = ACTIONS(2723),
    [anon_sym_trait] = ACTIONS(2723),
    [anon_sym_type] = ACTIONS(2723),
    [anon_sym_union] = ACTIONS(2723),
    [anon_sym_unsafe] = ACTIONS(2723),
    [anon_sym_use] = ACTIONS(2723),
    [anon_sym_while] = ACTIONS(2723),
    [anon_sym_extern] = ACTIONS(2723),
    [anon_sym_yield] = ACTIONS(2723),
    [anon_sym_move] = ACTIONS(2723),
    [anon_sym_try] = ACTIONS(2723),
    [sym_integer_literal] = ACTIONS(2721),
    [aux_sym_string_literal_token1] = ACTIONS(2721),
    [sym_char_literal] = ACTIONS(2721),
    [anon_sym_true] = ACTIONS(2723),
    [anon_sym_false] = ACTIONS(2723),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2723),
    [sym_super] = ACTIONS(2723),
    [sym_crate] = ACTIONS(2723),
    [sym_metavariable] = ACTIONS(2721),
    [sym__raw_string_literal_start] = ACTIONS(2721),
    [sym_float_literal] = ACTIONS(2721),
  },
  [703] = {
    [sym_line_comment] = STATE(703),
    [sym_block_comment] = STATE(703),
    [ts_builtin_sym_end] = ACTIONS(2725),
    [sym_identifier] = ACTIONS(2727),
    [anon_sym_SEMI] = ACTIONS(2725),
    [anon_sym_macro_rules_BANG] = ACTIONS(2725),
    [anon_sym_LPAREN] = ACTIONS(2725),
    [anon_sym_LBRACK] = ACTIONS(2725),
    [anon_sym_LBRACE] = ACTIONS(2725),
    [anon_sym_RBRACE] = ACTIONS(2725),
    [anon_sym_STAR] = ACTIONS(2725),
    [anon_sym_u8] = ACTIONS(2727),
    [anon_sym_i8] = ACTIONS(2727),
    [anon_sym_u16] = ACTIONS(2727),
    [anon_sym_i16] = ACTIONS(2727),
    [anon_sym_u32] = ACTIONS(2727),
    [anon_sym_i32] = ACTIONS(2727),
    [anon_sym_u64] = ACTIONS(2727),
    [anon_sym_i64] = ACTIONS(2727),
    [anon_sym_u128] = ACTIONS(2727),
    [anon_sym_i128] = ACTIONS(2727),
    [anon_sym_isize] = ACTIONS(2727),
    [anon_sym_usize] = ACTIONS(2727),
    [anon_sym_f32] = ACTIONS(2727),
    [anon_sym_f64] = ACTIONS(2727),
    [anon_sym_bool] = ACTIONS(2727),
    [anon_sym_str] = ACTIONS(2727),
    [anon_sym_char] = ACTIONS(2727),
    [anon_sym_DASH] = ACTIONS(2725),
    [anon_sym_BANG] = ACTIONS(2725),
    [anon_sym_AMP] = ACTIONS(2725),
    [anon_sym_PIPE] = ACTIONS(2725),
    [anon_sym_LT] = ACTIONS(2725),
    [anon_sym_DOT_DOT] = ACTIONS(2725),
    [anon_sym_COLON_COLON] = ACTIONS(2725),
    [anon_sym_POUND] = ACTIONS(2725),
    [anon_sym_SQUOTE] = ACTIONS(2727),
    [anon_sym_async] = ACTIONS(2727),
    [anon_sym_break] = ACTIONS(2727),
    [anon_sym_const] = ACTIONS(2727),
    [anon_sym_continue] = ACTIONS(2727),
    [anon_sym_default] = ACTIONS(2727),
    [anon_sym_enum] = ACTIONS(2727),
    [anon_sym_fn] = ACTIONS(2727),
    [anon_sym_for] = ACTIONS(2727),
    [anon_sym_gen] = ACTIONS(2727),
    [anon_sym_if] = ACTIONS(2727),
    [anon_sym_impl] = ACTIONS(2727),
    [anon_sym_let] = ACTIONS(2727),
    [anon_sym_loop] = ACTIONS(2727),
    [anon_sym_match] = ACTIONS(2727),
    [anon_sym_mod] = ACTIONS(2727),
    [anon_sym_pub] = ACTIONS(2727),
    [anon_sym_return] = ACTIONS(2727),
    [anon_sym_static] = ACTIONS(2727),
    [anon_sym_struct] = ACTIONS(2727),
    [anon_sym_trait] = ACTIONS(2727),
    [anon_sym_type] = ACTIONS(2727),
    [anon_sym_union] = ACTIONS(2727),
    [anon_sym_unsafe] = ACTIONS(2727),
    [anon_sym_use] = ACTIONS(2727),
    [anon_sym_while] = ACTIONS(2727),
    [anon_sym_extern] = ACTIONS(2727),
    [anon_sym_yield] = ACTIONS(2727),
    [anon_sym_move] = ACTIONS(2727),
    [anon_sym_try] = ACTIONS(2727),
    [sym_integer_literal] = ACTIONS(2725),
    [aux_sym_string_literal_token1] = ACTIONS(2725),
    [sym_char_literal] = ACTIONS(2725),
    [anon_sym_true] = ACTIONS(2727),
    [anon_sym_false] = ACTIONS(2727),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2727),
    [sym_super] = ACTIONS(2727),
    [sym_crate] = ACTIONS(2727),
    [sym_metavariable] = ACTIONS(2725),
    [sym__raw_string_literal_start] = ACTIONS(2725),
    [sym_float_literal] = ACTIONS(2725),
  },
  [704] = {
    [sym_line_comment] = STATE(704),
    [sym_block_comment] = STATE(704),
    [ts_builtin_sym_end] = ACTIONS(2729),
    [sym_identifier] = ACTIONS(2731),
    [anon_sym_SEMI] = ACTIONS(2729),
    [anon_sym_macro_rules_BANG] = ACTIONS(2729),
    [anon_sym_LPAREN] = ACTIONS(2729),
    [anon_sym_LBRACK] = ACTIONS(2729),
    [anon_sym_LBRACE] = ACTIONS(2729),
    [anon_sym_RBRACE] = ACTIONS(2729),
    [anon_sym_STAR] = ACTIONS(2729),
    [anon_sym_u8] = ACTIONS(2731),
    [anon_sym_i8] = ACTIONS(2731),
    [anon_sym_u16] = ACTIONS(2731),
    [anon_sym_i16] = ACTIONS(2731),
    [anon_sym_u32] = ACTIONS(2731),
    [anon_sym_i32] = ACTIONS(2731),
    [anon_sym_u64] = ACTIONS(2731),
    [anon_sym_i64] = ACTIONS(2731),
    [anon_sym_u128] = ACTIONS(2731),
    [anon_sym_i128] = ACTIONS(2731),
    [anon_sym_isize] = ACTIONS(2731),
    [anon_sym_usize] = ACTIONS(2731),
    [anon_sym_f32] = ACTIONS(2731),
    [anon_sym_f64] = ACTIONS(2731),
    [anon_sym_bool] = ACTIONS(2731),
    [anon_sym_str] = ACTIONS(2731),
    [anon_sym_char] = ACTIONS(2731),
    [anon_sym_DASH] = ACTIONS(2729),
    [anon_sym_BANG] = ACTIONS(2729),
    [anon_sym_AMP] = ACTIONS(2729),
    [anon_sym_PIPE] = ACTIONS(2729),
    [anon_sym_LT] = ACTIONS(2729),
    [anon_sym_DOT_DOT] = ACTIONS(2729),
    [anon_sym_COLON_COLON] = ACTIONS(2729),
    [anon_sym_POUND] = ACTIONS(2729),
    [anon_sym_SQUOTE] = ACTIONS(2731),
    [anon_sym_async] = ACTIONS(2731),
    [anon_sym_break] = ACTIONS(2731),
    [anon_sym_const] = ACTIONS(2731),
    [anon_sym_continue] = ACTIONS(2731),
    [anon_sym_default] = ACTIONS(2731),
    [anon_sym_enum] = ACTIONS(2731),
    [anon_sym_fn] = ACTIONS(2731),
    [anon_sym_for] = ACTIONS(2731),
    [anon_sym_gen] = ACTIONS(2731),
    [anon_sym_if] = ACTIONS(2731),
    [anon_sym_impl] = ACTIONS(2731),
    [anon_sym_let] = ACTIONS(2731),
    [anon_sym_loop] = ACTIONS(2731),
    [anon_sym_match] = ACTIONS(2731),
    [anon_sym_mod] = ACTIONS(2731),
    [anon_sym_pub] = ACTIONS(2731),
    [anon_sym_return] = ACTIONS(2731),
    [anon_sym_static] = ACTIONS(2731),
    [anon_sym_struct] = ACTIONS(2731),
    [anon_sym_trait] = ACTIONS(2731),
    [anon_sym_type] = ACTIONS(2731),
    [anon_sym_union] = ACTIONS(2731),
    [anon_sym_unsafe] = ACTIONS(2731),
    [anon_sym_use] = ACTIONS(2731),
    [anon_sym_while] = ACTIONS(2731),
    [anon_sym_extern] = ACTIONS(2731),
    [anon_sym_yield] = ACTIONS(2731),
    [anon_sym_move] = ACTIONS(2731),
    [anon_sym_try] = ACTIONS(2731),
    [sym_integer_literal] = ACTIONS(2729),
    [aux_sym_string_literal_token1] = ACTIONS(2729),
    [sym_char_literal] = ACTIONS(2729),
    [anon_sym_true] = ACTIONS(2731),
    [anon_sym_false] = ACTIONS(2731),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2731),
    [sym_super] = ACTIONS(2731),
    [sym_crate] = ACTIONS(2731),
    [sym_metavariable] = ACTIONS(2729),
    [sym__raw_string_literal_start] = ACTIONS(2729),
    [sym_float_literal] = ACTIONS(2729),
  },
  [705] = {
    [sym_line_comment] = STATE(705),
    [sym_block_comment] = STATE(705),
    [ts_builtin_sym_end] = ACTIONS(2733),
    [sym_identifier] = ACTIONS(2735),
    [anon_sym_SEMI] = ACTIONS(2733),
    [anon_sym_macro_rules_BANG] = ACTIONS(2733),
    [anon_sym_LPAREN] = ACTIONS(2733),
    [anon_sym_LBRACK] = ACTIONS(2733),
    [anon_sym_LBRACE] = ACTIONS(2733),
    [anon_sym_RBRACE] = ACTIONS(2733),
    [anon_sym_STAR] = ACTIONS(2733),
    [anon_sym_u8] = ACTIONS(2735),
    [anon_sym_i8] = ACTIONS(2735),
    [anon_sym_u16] = ACTIONS(2735),
    [anon_sym_i16] = ACTIONS(2735),
    [anon_sym_u32] = ACTIONS(2735),
    [anon_sym_i32] = ACTIONS(2735),
    [anon_sym_u64] = ACTIONS(2735),
    [anon_sym_i64] = ACTIONS(2735),
    [anon_sym_u128] = ACTIONS(2735),
    [anon_sym_i128] = ACTIONS(2735),
    [anon_sym_isize] = ACTIONS(2735),
    [anon_sym_usize] = ACTIONS(2735),
    [anon_sym_f32] = ACTIONS(2735),
    [anon_sym_f64] = ACTIONS(2735),
    [anon_sym_bool] = ACTIONS(2735),
    [anon_sym_str] = ACTIONS(2735),
    [anon_sym_char] = ACTIONS(2735),
    [anon_sym_DASH] = ACTIONS(2733),
    [anon_sym_BANG] = ACTIONS(2733),
    [anon_sym_AMP] = ACTIONS(2733),
    [anon_sym_PIPE] = ACTIONS(2733),
    [anon_sym_LT] = ACTIONS(2733),
    [anon_sym_DOT_DOT] = ACTIONS(2733),
    [anon_sym_COLON_COLON] = ACTIONS(2733),
    [anon_sym_POUND] = ACTIONS(2733),
    [anon_sym_SQUOTE] = ACTIONS(2735),
    [anon_sym_async] = ACTIONS(2735),
    [anon_sym_break] = ACTIONS(2735),
    [anon_sym_const] = ACTIONS(2735),
    [anon_sym_continue] = ACTIONS(2735),
    [anon_sym_default] = ACTIONS(2735),
    [anon_sym_enum] = ACTIONS(2735),
    [anon_sym_fn] = ACTIONS(2735),
    [anon_sym_for] = ACTIONS(2735),
    [anon_sym_gen] = ACTIONS(2735),
    [anon_sym_if] = ACTIONS(2735),
    [anon_sym_impl] = ACTIONS(2735),
    [anon_sym_let] = ACTIONS(2735),
    [anon_sym_loop] = ACTIONS(2735),
    [anon_sym_match] = ACTIONS(2735),
    [anon_sym_mod] = ACTIONS(2735),
    [anon_sym_pub] = ACTIONS(2735),
    [anon_sym_return] = ACTIONS(2735),
    [anon_sym_static] = ACTIONS(2735),
    [anon_sym_struct] = ACTIONS(2735),
    [anon_sym_trait] = ACTIONS(2735),
    [anon_sym_type] = ACTIONS(2735),
    [anon_sym_union] = ACTIONS(2735),
    [anon_sym_unsafe] = ACTIONS(2735),
    [anon_sym_use] = ACTIONS(2735),
    [anon_sym_while] = ACTIONS(2735),
    [anon_sym_extern] = ACTIONS(2735),
    [anon_sym_yield] = ACTIONS(2735),
    [anon_sym_move] = ACTIONS(2735),
    [anon_sym_try] = ACTIONS(2735),
    [sym_integer_literal] = ACTIONS(2733),
    [aux_sym_string_literal_token1] = ACTIONS(2733),
    [sym_char_literal] = ACTIONS(2733),
    [anon_sym_true] = ACTIONS(2735),
    [anon_sym_false] = ACTIONS(2735),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2735),
    [sym_super] = ACTIONS(2735),
    [sym_crate] = ACTIONS(2735),
    [sym_metavariable] = ACTIONS(2733),
    [sym__raw_string_literal_start] = ACTIONS(2733),
    [sym_float_literal] = ACTIONS(2733),
  },
  [706] = {
    [sym_line_comment] = STATE(706),
    [sym_block_comment] = STATE(706),
    [ts_builtin_sym_end] = ACTIONS(2737),
    [sym_identifier] = ACTIONS(2739),
    [anon_sym_SEMI] = ACTIONS(2737),
    [anon_sym_macro_rules_BANG] = ACTIONS(2737),
    [anon_sym_LPAREN] = ACTIONS(2737),
    [anon_sym_LBRACK] = ACTIONS(2737),
    [anon_sym_LBRACE] = ACTIONS(2737),
    [anon_sym_RBRACE] = ACTIONS(2737),
    [anon_sym_STAR] = ACTIONS(2737),
    [anon_sym_u8] = ACTIONS(2739),
    [anon_sym_i8] = ACTIONS(2739),
    [anon_sym_u16] = ACTIONS(2739),
    [anon_sym_i16] = ACTIONS(2739),
    [anon_sym_u32] = ACTIONS(2739),
    [anon_sym_i32] = ACTIONS(2739),
    [anon_sym_u64] = ACTIONS(2739),
    [anon_sym_i64] = ACTIONS(2739),
    [anon_sym_u128] = ACTIONS(2739),
    [anon_sym_i128] = ACTIONS(2739),
    [anon_sym_isize] = ACTIONS(2739),
    [anon_sym_usize] = ACTIONS(2739),
    [anon_sym_f32] = ACTIONS(2739),
    [anon_sym_f64] = ACTIONS(2739),
    [anon_sym_bool] = ACTIONS(2739),
    [anon_sym_str] = ACTIONS(2739),
    [anon_sym_char] = ACTIONS(2739),
    [anon_sym_DASH] = ACTIONS(2737),
    [anon_sym_BANG] = ACTIONS(2737),
    [anon_sym_AMP] = ACTIONS(2737),
    [anon_sym_PIPE] = ACTIONS(2737),
    [anon_sym_LT] = ACTIONS(2737),
    [anon_sym_DOT_DOT] = ACTIONS(2737),
    [anon_sym_COLON_COLON] = ACTIONS(2737),
    [anon_sym_POUND] = ACTIONS(2737),
    [anon_sym_SQUOTE] = ACTIONS(2739),
    [anon_sym_async] = ACTIONS(2739),
    [anon_sym_break] = ACTIONS(2739),
    [anon_sym_const] = ACTIONS(2739),
    [anon_sym_continue] = ACTIONS(2739),
    [anon_sym_default] = ACTIONS(2739),
    [anon_sym_enum] = ACTIONS(2739),
    [anon_sym_fn] = ACTIONS(2739),
    [anon_sym_for] = ACTIONS(2739),
    [anon_sym_gen] = ACTIONS(2739),
    [anon_sym_if] = ACTIONS(2739),
    [anon_sym_impl] = ACTIONS(2739),
    [anon_sym_let] = ACTIONS(2739),
    [anon_sym_loop] = ACTIONS(2739),
    [anon_sym_match] = ACTIONS(2739),
    [anon_sym_mod] = ACTIONS(2739),
    [anon_sym_pub] = ACTIONS(2739),
    [anon_sym_return] = ACTIONS(2739),
    [anon_sym_static] = ACTIONS(2739),
    [anon_sym_struct] = ACTIONS(2739),
    [anon_sym_trait] = ACTIONS(2739),
    [anon_sym_type] = ACTIONS(2739),
    [anon_sym_union] = ACTIONS(2739),
    [anon_sym_unsafe] = ACTIONS(2739),
    [anon_sym_use] = ACTIONS(2739),
    [anon_sym_while] = ACTIONS(2739),
    [anon_sym_extern] = ACTIONS(2739),
    [anon_sym_yield] = ACTIONS(2739),
    [anon_sym_move] = ACTIONS(2739),
    [anon_sym_try] = ACTIONS(2739),
    [sym_integer_literal] = ACTIONS(2737),
    [aux_sym_string_literal_token1] = ACTIONS(2737),
    [sym_char_literal] = ACTIONS(2737),
    [anon_sym_true] = ACTIONS(2739),
    [anon_sym_false] = ACTIONS(2739),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2739),
    [sym_super] = ACTIONS(2739),
    [sym_crate] = ACTIONS(2739),
    [sym_metavariable] = ACTIONS(2737),
    [sym__raw_string_literal_start] = ACTIONS(2737),
    [sym_float_literal] = ACTIONS(2737),
  },
  [707] = {
    [sym_line_comment] = STATE(707),
    [sym_block_comment] = STATE(707),
    [ts_builtin_sym_end] = ACTIONS(2741),
    [sym_identifier] = ACTIONS(2743),
    [anon_sym_SEMI] = ACTIONS(2741),
    [anon_sym_macro_rules_BANG] = ACTIONS(2741),
    [anon_sym_LPAREN] = ACTIONS(2741),
    [anon_sym_LBRACK] = ACTIONS(2741),
    [anon_sym_LBRACE] = ACTIONS(2741),
    [anon_sym_RBRACE] = ACTIONS(2741),
    [anon_sym_STAR] = ACTIONS(2741),
    [anon_sym_u8] = ACTIONS(2743),
    [anon_sym_i8] = ACTIONS(2743),
    [anon_sym_u16] = ACTIONS(2743),
    [anon_sym_i16] = ACTIONS(2743),
    [anon_sym_u32] = ACTIONS(2743),
    [anon_sym_i32] = ACTIONS(2743),
    [anon_sym_u64] = ACTIONS(2743),
    [anon_sym_i64] = ACTIONS(2743),
    [anon_sym_u128] = ACTIONS(2743),
    [anon_sym_i128] = ACTIONS(2743),
    [anon_sym_isize] = ACTIONS(2743),
    [anon_sym_usize] = ACTIONS(2743),
    [anon_sym_f32] = ACTIONS(2743),
    [anon_sym_f64] = ACTIONS(2743),
    [anon_sym_bool] = ACTIONS(2743),
    [anon_sym_str] = ACTIONS(2743),
    [anon_sym_char] = ACTIONS(2743),
    [anon_sym_DASH] = ACTIONS(2741),
    [anon_sym_BANG] = ACTIONS(2741),
    [anon_sym_AMP] = ACTIONS(2741),
    [anon_sym_PIPE] = ACTIONS(2741),
    [anon_sym_LT] = ACTIONS(2741),
    [anon_sym_DOT_DOT] = ACTIONS(2741),
    [anon_sym_COLON_COLON] = ACTIONS(2741),
    [anon_sym_POUND] = ACTIONS(2741),
    [anon_sym_SQUOTE] = ACTIONS(2743),
    [anon_sym_async] = ACTIONS(2743),
    [anon_sym_break] = ACTIONS(2743),
    [anon_sym_const] = ACTIONS(2743),
    [anon_sym_continue] = ACTIONS(2743),
    [anon_sym_default] = ACTIONS(2743),
    [anon_sym_enum] = ACTIONS(2743),
    [anon_sym_fn] = ACTIONS(2743),
    [anon_sym_for] = ACTIONS(2743),
    [anon_sym_gen] = ACTIONS(2743),
    [anon_sym_if] = ACTIONS(2743),
    [anon_sym_impl] = ACTIONS(2743),
    [anon_sym_let] = ACTIONS(2743),
    [anon_sym_loop] = ACTIONS(2743),
    [anon_sym_match] = ACTIONS(2743),
    [anon_sym_mod] = ACTIONS(2743),
    [anon_sym_pub] = ACTIONS(2743),
    [anon_sym_return] = ACTIONS(2743),
    [anon_sym_static] = ACTIONS(2743),
    [anon_sym_struct] = ACTIONS(2743),
    [anon_sym_trait] = ACTIONS(2743),
    [anon_sym_type] = ACTIONS(2743),
    [anon_sym_union] = ACTIONS(2743),
    [anon_sym_unsafe] = ACTIONS(2743),
    [anon_sym_use] = ACTIONS(2743),
    [anon_sym_while] = ACTIONS(2743),
    [anon_sym_extern] = ACTIONS(2743),
    [anon_sym_yield] = ACTIONS(2743),
    [anon_sym_move] = ACTIONS(2743),
    [anon_sym_try] = ACTIONS(2743),
    [sym_integer_literal] = ACTIONS(2741),
    [aux_sym_string_literal_token1] = ACTIONS(2741),
    [sym_char_literal] = ACTIONS(2741),
    [anon_sym_true] = ACTIONS(2743),
    [anon_sym_false] = ACTIONS(2743),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2743),
    [sym_super] = ACTIONS(2743),
    [sym_crate] = ACTIONS(2743),
    [sym_metavariable] = ACTIONS(2741),
    [sym__raw_string_literal_start] = ACTIONS(2741),
    [sym_float_literal] = ACTIONS(2741),
  },
  [708] = {
    [sym_line_comment] = STATE(708),
    [sym_block_comment] = STATE(708),
    [ts_builtin_sym_end] = ACTIONS(2745),
    [sym_identifier] = ACTIONS(2747),
    [anon_sym_SEMI] = ACTIONS(2745),
    [anon_sym_macro_rules_BANG] = ACTIONS(2745),
    [anon_sym_LPAREN] = ACTIONS(2745),
    [anon_sym_LBRACK] = ACTIONS(2745),
    [anon_sym_LBRACE] = ACTIONS(2745),
    [anon_sym_RBRACE] = ACTIONS(2745),
    [anon_sym_STAR] = ACTIONS(2745),
    [anon_sym_u8] = ACTIONS(2747),
    [anon_sym_i8] = ACTIONS(2747),
    [anon_sym_u16] = ACTIONS(2747),
    [anon_sym_i16] = ACTIONS(2747),
    [anon_sym_u32] = ACTIONS(2747),
    [anon_sym_i32] = ACTIONS(2747),
    [anon_sym_u64] = ACTIONS(2747),
    [anon_sym_i64] = ACTIONS(2747),
    [anon_sym_u128] = ACTIONS(2747),
    [anon_sym_i128] = ACTIONS(2747),
    [anon_sym_isize] = ACTIONS(2747),
    [anon_sym_usize] = ACTIONS(2747),
    [anon_sym_f32] = ACTIONS(2747),
    [anon_sym_f64] = ACTIONS(2747),
    [anon_sym_bool] = ACTIONS(2747),
    [anon_sym_str] = ACTIONS(2747),
    [anon_sym_char] = ACTIONS(2747),
    [anon_sym_DASH] = ACTIONS(2745),
    [anon_sym_BANG] = ACTIONS(2745),
    [anon_sym_AMP] = ACTIONS(2745),
    [anon_sym_PIPE] = ACTIONS(2745),
    [anon_sym_LT] = ACTIONS(2745),
    [anon_sym_DOT_DOT] = ACTIONS(2745),
    [anon_sym_COLON_COLON] = ACTIONS(2745),
    [anon_sym_POUND] = ACTIONS(2745),
    [anon_sym_SQUOTE] = ACTIONS(2747),
    [anon_sym_async] = ACTIONS(2747),
    [anon_sym_break] = ACTIONS(2747),
    [anon_sym_const] = ACTIONS(2747),
    [anon_sym_continue] = ACTIONS(2747),
    [anon_sym_default] = ACTIONS(2747),
    [anon_sym_enum] = ACTIONS(2747),
    [anon_sym_fn] = ACTIONS(2747),
    [anon_sym_for] = ACTIONS(2747),
    [anon_sym_gen] = ACTIONS(2747),
    [anon_sym_if] = ACTIONS(2747),
    [anon_sym_impl] = ACTIONS(2747),
    [anon_sym_let] = ACTIONS(2747),
    [anon_sym_loop] = ACTIONS(2747),
    [anon_sym_match] = ACTIONS(2747),
    [anon_sym_mod] = ACTIONS(2747),
    [anon_sym_pub] = ACTIONS(2747),
    [anon_sym_return] = ACTIONS(2747),
    [anon_sym_static] = ACTIONS(2747),
    [anon_sym_struct] = ACTIONS(2747),
    [anon_sym_trait] = ACTIONS(2747),
    [anon_sym_type] = ACTIONS(2747),
    [anon_sym_union] = ACTIONS(2747),
    [anon_sym_unsafe] = ACTIONS(2747),
    [anon_sym_use] = ACTIONS(2747),
    [anon_sym_while] = ACTIONS(2747),
    [anon_sym_extern] = ACTIONS(2747),
    [anon_sym_yield] = ACTIONS(2747),
    [anon_sym_move] = ACTIONS(2747),
    [anon_sym_try] = ACTIONS(2747),
    [sym_integer_literal] = ACTIONS(2745),
    [aux_sym_string_literal_token1] = ACTIONS(2745),
    [sym_char_literal] = ACTIONS(2745),
    [anon_sym_true] = ACTIONS(2747),
    [anon_sym_false] = ACTIONS(2747),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2747),
    [sym_super] = ACTIONS(2747),
    [sym_crate] = ACTIONS(2747),
    [sym_metavariable] = ACTIONS(2745),
    [sym__raw_string_literal_start] = ACTIONS(2745),
    [sym_float_literal] = ACTIONS(2745),
  },
  [709] = {
    [sym_line_comment] = STATE(709),
    [sym_block_comment] = STATE(709),
    [ts_builtin_sym_end] = ACTIONS(2749),
    [sym_identifier] = ACTIONS(2751),
    [anon_sym_SEMI] = ACTIONS(2749),
    [anon_sym_macro_rules_BANG] = ACTIONS(2749),
    [anon_sym_LPAREN] = ACTIONS(2749),
    [anon_sym_LBRACK] = ACTIONS(2749),
    [anon_sym_LBRACE] = ACTIONS(2749),
    [anon_sym_RBRACE] = ACTIONS(2749),
    [anon_sym_STAR] = ACTIONS(2749),
    [anon_sym_u8] = ACTIONS(2751),
    [anon_sym_i8] = ACTIONS(2751),
    [anon_sym_u16] = ACTIONS(2751),
    [anon_sym_i16] = ACTIONS(2751),
    [anon_sym_u32] = ACTIONS(2751),
    [anon_sym_i32] = ACTIONS(2751),
    [anon_sym_u64] = ACTIONS(2751),
    [anon_sym_i64] = ACTIONS(2751),
    [anon_sym_u128] = ACTIONS(2751),
    [anon_sym_i128] = ACTIONS(2751),
    [anon_sym_isize] = ACTIONS(2751),
    [anon_sym_usize] = ACTIONS(2751),
    [anon_sym_f32] = ACTIONS(2751),
    [anon_sym_f64] = ACTIONS(2751),
    [anon_sym_bool] = ACTIONS(2751),
    [anon_sym_str] = ACTIONS(2751),
    [anon_sym_char] = ACTIONS(2751),
    [anon_sym_DASH] = ACTIONS(2749),
    [anon_sym_BANG] = ACTIONS(2749),
    [anon_sym_AMP] = ACTIONS(2749),
    [anon_sym_PIPE] = ACTIONS(2749),
    [anon_sym_LT] = ACTIONS(2749),
    [anon_sym_DOT_DOT] = ACTIONS(2749),
    [anon_sym_COLON_COLON] = ACTIONS(2749),
    [anon_sym_POUND] = ACTIONS(2749),
    [anon_sym_SQUOTE] = ACTIONS(2751),
    [anon_sym_async] = ACTIONS(2751),
    [anon_sym_break] = ACTIONS(2751),
    [anon_sym_const] = ACTIONS(2751),
    [anon_sym_continue] = ACTIONS(2751),
    [anon_sym_default] = ACTIONS(2751),
    [anon_sym_enum] = ACTIONS(2751),
    [anon_sym_fn] = ACTIONS(2751),
    [anon_sym_for] = ACTIONS(2751),
    [anon_sym_gen] = ACTIONS(2751),
    [anon_sym_if] = ACTIONS(2751),
    [anon_sym_impl] = ACTIONS(2751),
    [anon_sym_let] = ACTIONS(2751),
    [anon_sym_loop] = ACTIONS(2751),
    [anon_sym_match] = ACTIONS(2751),
    [anon_sym_mod] = ACTIONS(2751),
    [anon_sym_pub] = ACTIONS(2751),
    [anon_sym_return] = ACTIONS(2751),
    [anon_sym_static] = ACTIONS(2751),
    [anon_sym_struct] = ACTIONS(2751),
    [anon_sym_trait] = ACTIONS(2751),
    [anon_sym_type] = ACTIONS(2751),
    [anon_sym_union] = ACTIONS(2751),
    [anon_sym_unsafe] = ACTIONS(2751),
    [anon_sym_use] = ACTIONS(2751),
    [anon_sym_while] = ACTIONS(2751),
    [anon_sym_extern] = ACTIONS(2751),
    [anon_sym_yield] = ACTIONS(2751),
    [anon_sym_move] = ACTIONS(2751),
    [anon_sym_try] = ACTIONS(2751),
    [sym_integer_literal] = ACTIONS(2749),
    [aux_sym_string_literal_token1] = ACTIONS(2749),
    [sym_char_literal] = ACTIONS(2749),
    [anon_sym_true] = ACTIONS(2751),
    [anon_sym_false] = ACTIONS(2751),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2751),
    [sym_super] = ACTIONS(2751),
    [sym_crate] = ACTIONS(2751),
    [sym_metavariable] = ACTIONS(2749),
    [sym__raw_string_literal_start] = ACTIONS(2749),
    [sym_float_literal] = ACTIONS(2749),
  },
  [710] = {
    [sym_line_comment] = STATE(710),
    [sym_block_comment] = STATE(710),
    [ts_builtin_sym_end] = ACTIONS(2753),
    [sym_identifier] = ACTIONS(2755),
    [anon_sym_SEMI] = ACTIONS(2753),
    [anon_sym_macro_rules_BANG] = ACTIONS(2753),
    [anon_sym_LPAREN] = ACTIONS(2753),
    [anon_sym_LBRACK] = ACTIONS(2753),
    [anon_sym_LBRACE] = ACTIONS(2753),
    [anon_sym_RBRACE] = ACTIONS(2753),
    [anon_sym_STAR] = ACTIONS(2753),
    [anon_sym_u8] = ACTIONS(2755),
    [anon_sym_i8] = ACTIONS(2755),
    [anon_sym_u16] = ACTIONS(2755),
    [anon_sym_i16] = ACTIONS(2755),
    [anon_sym_u32] = ACTIONS(2755),
    [anon_sym_i32] = ACTIONS(2755),
    [anon_sym_u64] = ACTIONS(2755),
    [anon_sym_i64] = ACTIONS(2755),
    [anon_sym_u128] = ACTIONS(2755),
    [anon_sym_i128] = ACTIONS(2755),
    [anon_sym_isize] = ACTIONS(2755),
    [anon_sym_usize] = ACTIONS(2755),
    [anon_sym_f32] = ACTIONS(2755),
    [anon_sym_f64] = ACTIONS(2755),
    [anon_sym_bool] = ACTIONS(2755),
    [anon_sym_str] = ACTIONS(2755),
    [anon_sym_char] = ACTIONS(2755),
    [anon_sym_DASH] = ACTIONS(2753),
    [anon_sym_BANG] = ACTIONS(2753),
    [anon_sym_AMP] = ACTIONS(2753),
    [anon_sym_PIPE] = ACTIONS(2753),
    [anon_sym_LT] = ACTIONS(2753),
    [anon_sym_DOT_DOT] = ACTIONS(2753),
    [anon_sym_COLON_COLON] = ACTIONS(2753),
    [anon_sym_POUND] = ACTIONS(2753),
    [anon_sym_SQUOTE] = ACTIONS(2755),
    [anon_sym_async] = ACTIONS(2755),
    [anon_sym_break] = ACTIONS(2755),
    [anon_sym_const] = ACTIONS(2755),
    [anon_sym_continue] = ACTIONS(2755),
    [anon_sym_default] = ACTIONS(2755),
    [anon_sym_enum] = ACTIONS(2755),
    [anon_sym_fn] = ACTIONS(2755),
    [anon_sym_for] = ACTIONS(2755),
    [anon_sym_gen] = ACTIONS(2755),
    [anon_sym_if] = ACTIONS(2755),
    [anon_sym_impl] = ACTIONS(2755),
    [anon_sym_let] = ACTIONS(2755),
    [anon_sym_loop] = ACTIONS(2755),
    [anon_sym_match] = ACTIONS(2755),
    [anon_sym_mod] = ACTIONS(2755),
    [anon_sym_pub] = ACTIONS(2755),
    [anon_sym_return] = ACTIONS(2755),
    [anon_sym_static] = ACTIONS(2755),
    [anon_sym_struct] = ACTIONS(2755),
    [anon_sym_trait] = ACTIONS(2755),
    [anon_sym_type] = ACTIONS(2755),
    [anon_sym_union] = ACTIONS(2755),
    [anon_sym_unsafe] = ACTIONS(2755),
    [anon_sym_use] = ACTIONS(2755),
    [anon_sym_while] = ACTIONS(2755),
    [anon_sym_extern] = ACTIONS(2755),
    [anon_sym_yield] = ACTIONS(2755),
    [anon_sym_move] = ACTIONS(2755),
    [anon_sym_try] = ACTIONS(2755),
    [sym_integer_literal] = ACTIONS(2753),
    [aux_sym_string_literal_token1] = ACTIONS(2753),
    [sym_char_literal] = ACTIONS(2753),
    [anon_sym_true] = ACTIONS(2755),
    [anon_sym_false] = ACTIONS(2755),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2755),
    [sym_super] = ACTIONS(2755),
    [sym_crate] = ACTIONS(2755),
    [sym_metavariable] = ACTIONS(2753),
    [sym__raw_string_literal_start] = ACTIONS(2753),
    [sym_float_literal] = ACTIONS(2753),
  },
  [711] = {
    [sym_line_comment] = STATE(711),
    [sym_block_comment] = STATE(711),
    [ts_builtin_sym_end] = ACTIONS(2757),
    [sym_identifier] = ACTIONS(2759),
    [anon_sym_SEMI] = ACTIONS(2757),
    [anon_sym_macro_rules_BANG] = ACTIONS(2757),
    [anon_sym_LPAREN] = ACTIONS(2757),
    [anon_sym_LBRACK] = ACTIONS(2757),
    [anon_sym_LBRACE] = ACTIONS(2757),
    [anon_sym_RBRACE] = ACTIONS(2757),
    [anon_sym_STAR] = ACTIONS(2757),
    [anon_sym_u8] = ACTIONS(2759),
    [anon_sym_i8] = ACTIONS(2759),
    [anon_sym_u16] = ACTIONS(2759),
    [anon_sym_i16] = ACTIONS(2759),
    [anon_sym_u32] = ACTIONS(2759),
    [anon_sym_i32] = ACTIONS(2759),
    [anon_sym_u64] = ACTIONS(2759),
    [anon_sym_i64] = ACTIONS(2759),
    [anon_sym_u128] = ACTIONS(2759),
    [anon_sym_i128] = ACTIONS(2759),
    [anon_sym_isize] = ACTIONS(2759),
    [anon_sym_usize] = ACTIONS(2759),
    [anon_sym_f32] = ACTIONS(2759),
    [anon_sym_f64] = ACTIONS(2759),
    [anon_sym_bool] = ACTIONS(2759),
    [anon_sym_str] = ACTIONS(2759),
    [anon_sym_char] = ACTIONS(2759),
    [anon_sym_DASH] = ACTIONS(2757),
    [anon_sym_BANG] = ACTIONS(2757),
    [anon_sym_AMP] = ACTIONS(2757),
    [anon_sym_PIPE] = ACTIONS(2757),
    [anon_sym_LT] = ACTIONS(2757),
    [anon_sym_DOT_DOT] = ACTIONS(2757),
    [anon_sym_COLON_COLON] = ACTIONS(2757),
    [anon_sym_POUND] = ACTIONS(2757),
    [anon_sym_SQUOTE] = ACTIONS(2759),
    [anon_sym_async] = ACTIONS(2759),
    [anon_sym_break] = ACTIONS(2759),
    [anon_sym_const] = ACTIONS(2759),
    [anon_sym_continue] = ACTIONS(2759),
    [anon_sym_default] = ACTIONS(2759),
    [anon_sym_enum] = ACTIONS(2759),
    [anon_sym_fn] = ACTIONS(2759),
    [anon_sym_for] = ACTIONS(2759),
    [anon_sym_gen] = ACTIONS(2759),
    [anon_sym_if] = ACTIONS(2759),
    [anon_sym_impl] = ACTIONS(2759),
    [anon_sym_let] = ACTIONS(2759),
    [anon_sym_loop] = ACTIONS(2759),
    [anon_sym_match] = ACTIONS(2759),
    [anon_sym_mod] = ACTIONS(2759),
    [anon_sym_pub] = ACTIONS(2759),
    [anon_sym_return] = ACTIONS(2759),
    [anon_sym_static] = ACTIONS(2759),
    [anon_sym_struct] = ACTIONS(2759),
    [anon_sym_trait] = ACTIONS(2759),
    [anon_sym_type] = ACTIONS(2759),
    [anon_sym_union] = ACTIONS(2759),
    [anon_sym_unsafe] = ACTIONS(2759),
    [anon_sym_use] = ACTIONS(2759),
    [anon_sym_while] = ACTIONS(2759),
    [anon_sym_extern] = ACTIONS(2759),
    [anon_sym_yield] = ACTIONS(2759),
    [anon_sym_move] = ACTIONS(2759),
    [anon_sym_try] = ACTIONS(2759),
    [sym_integer_literal] = ACTIONS(2757),
    [aux_sym_string_literal_token1] = ACTIONS(2757),
    [sym_char_literal] = ACTIONS(2757),
    [anon_sym_true] = ACTIONS(2759),
    [anon_sym_false] = ACTIONS(2759),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2759),
    [sym_super] = ACTIONS(2759),
    [sym_crate] = ACTIONS(2759),
    [sym_metavariable] = ACTIONS(2757),
    [sym__raw_string_literal_start] = ACTIONS(2757),
    [sym_float_literal] = ACTIONS(2757),
  },
  [712] = {
    [sym_line_comment] = STATE(712),
    [sym_block_comment] = STATE(712),
    [ts_builtin_sym_end] = ACTIONS(2761),
    [sym_identifier] = ACTIONS(2763),
    [anon_sym_SEMI] = ACTIONS(2761),
    [anon_sym_macro_rules_BANG] = ACTIONS(2761),
    [anon_sym_LPAREN] = ACTIONS(2761),
    [anon_sym_LBRACK] = ACTIONS(2761),
    [anon_sym_LBRACE] = ACTIONS(2761),
    [anon_sym_RBRACE] = ACTIONS(2761),
    [anon_sym_STAR] = ACTIONS(2761),
    [anon_sym_u8] = ACTIONS(2763),
    [anon_sym_i8] = ACTIONS(2763),
    [anon_sym_u16] = ACTIONS(2763),
    [anon_sym_i16] = ACTIONS(2763),
    [anon_sym_u32] = ACTIONS(2763),
    [anon_sym_i32] = ACTIONS(2763),
    [anon_sym_u64] = ACTIONS(2763),
    [anon_sym_i64] = ACTIONS(2763),
    [anon_sym_u128] = ACTIONS(2763),
    [anon_sym_i128] = ACTIONS(2763),
    [anon_sym_isize] = ACTIONS(2763),
    [anon_sym_usize] = ACTIONS(2763),
    [anon_sym_f32] = ACTIONS(2763),
    [anon_sym_f64] = ACTIONS(2763),
    [anon_sym_bool] = ACTIONS(2763),
    [anon_sym_str] = ACTIONS(2763),
    [anon_sym_char] = ACTIONS(2763),
    [anon_sym_DASH] = ACTIONS(2761),
    [anon_sym_BANG] = ACTIONS(2761),
    [anon_sym_AMP] = ACTIONS(2761),
    [anon_sym_PIPE] = ACTIONS(2761),
    [anon_sym_LT] = ACTIONS(2761),
    [anon_sym_DOT_DOT] = ACTIONS(2761),
    [anon_sym_COLON_COLON] = ACTIONS(2761),
    [anon_sym_POUND] = ACTIONS(2761),
    [anon_sym_SQUOTE] = ACTIONS(2763),
    [anon_sym_async] = ACTIONS(2763),
    [anon_sym_break] = ACTIONS(2763),
    [anon_sym_const] = ACTIONS(2763),
    [anon_sym_continue] = ACTIONS(2763),
    [anon_sym_default] = ACTIONS(2763),
    [anon_sym_enum] = ACTIONS(2763),
    [anon_sym_fn] = ACTIONS(2763),
    [anon_sym_for] = ACTIONS(2763),
    [anon_sym_gen] = ACTIONS(2763),
    [anon_sym_if] = ACTIONS(2763),
    [anon_sym_impl] = ACTIONS(2763),
    [anon_sym_let] = ACTIONS(2763),
    [anon_sym_loop] = ACTIONS(2763),
    [anon_sym_match] = ACTIONS(2763),
    [anon_sym_mod] = ACTIONS(2763),
    [anon_sym_pub] = ACTIONS(2763),
    [anon_sym_return] = ACTIONS(2763),
    [anon_sym_static] = ACTIONS(2763),
    [anon_sym_struct] = ACTIONS(2763),
    [anon_sym_trait] = ACTIONS(2763),
    [anon_sym_type] = ACTIONS(2763),
    [anon_sym_union] = ACTIONS(2763),
    [anon_sym_unsafe] = ACTIONS(2763),
    [anon_sym_use] = ACTIONS(2763),
    [anon_sym_while] = ACTIONS(2763),
    [anon_sym_extern] = ACTIONS(2763),
    [anon_sym_yield] = ACTIONS(2763),
    [anon_sym_move] = ACTIONS(2763),
    [anon_sym_try] = ACTIONS(2763),
    [sym_integer_literal] = ACTIONS(2761),
    [aux_sym_string_literal_token1] = ACTIONS(2761),
    [sym_char_literal] = ACTIONS(2761),
    [anon_sym_true] = ACTIONS(2763),
    [anon_sym_false] = ACTIONS(2763),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2763),
    [sym_super] = ACTIONS(2763),
    [sym_crate] = ACTIONS(2763),
    [sym_metavariable] = ACTIONS(2761),
    [sym__raw_string_literal_start] = ACTIONS(2761),
    [sym_float_literal] = ACTIONS(2761),
  },
  [713] = {
    [sym_line_comment] = STATE(713),
    [sym_block_comment] = STATE(713),
    [ts_builtin_sym_end] = ACTIONS(2765),
    [sym_identifier] = ACTIONS(2767),
    [anon_sym_SEMI] = ACTIONS(2765),
    [anon_sym_macro_rules_BANG] = ACTIONS(2765),
    [anon_sym_LPAREN] = ACTIONS(2765),
    [anon_sym_LBRACK] = ACTIONS(2765),
    [anon_sym_LBRACE] = ACTIONS(2765),
    [anon_sym_RBRACE] = ACTIONS(2765),
    [anon_sym_STAR] = ACTIONS(2765),
    [anon_sym_u8] = ACTIONS(2767),
    [anon_sym_i8] = ACTIONS(2767),
    [anon_sym_u16] = ACTIONS(2767),
    [anon_sym_i16] = ACTIONS(2767),
    [anon_sym_u32] = ACTIONS(2767),
    [anon_sym_i32] = ACTIONS(2767),
    [anon_sym_u64] = ACTIONS(2767),
    [anon_sym_i64] = ACTIONS(2767),
    [anon_sym_u128] = ACTIONS(2767),
    [anon_sym_i128] = ACTIONS(2767),
    [anon_sym_isize] = ACTIONS(2767),
    [anon_sym_usize] = ACTIONS(2767),
    [anon_sym_f32] = ACTIONS(2767),
    [anon_sym_f64] = ACTIONS(2767),
    [anon_sym_bool] = ACTIONS(2767),
    [anon_sym_str] = ACTIONS(2767),
    [anon_sym_char] = ACTIONS(2767),
    [anon_sym_DASH] = ACTIONS(2765),
    [anon_sym_BANG] = ACTIONS(2765),
    [anon_sym_AMP] = ACTIONS(2765),
    [anon_sym_PIPE] = ACTIONS(2765),
    [anon_sym_LT] = ACTIONS(2765),
    [anon_sym_DOT_DOT] = ACTIONS(2765),
    [anon_sym_COLON_COLON] = ACTIONS(2765),
    [anon_sym_POUND] = ACTIONS(2765),
    [anon_sym_SQUOTE] = ACTIONS(2767),
    [anon_sym_async] = ACTIONS(2767),
    [anon_sym_break] = ACTIONS(2767),
    [anon_sym_const] = ACTIONS(2767),
    [anon_sym_continue] = ACTIONS(2767),
    [anon_sym_default] = ACTIONS(2767),
    [anon_sym_enum] = ACTIONS(2767),
    [anon_sym_fn] = ACTIONS(2767),
    [anon_sym_for] = ACTIONS(2767),
    [anon_sym_gen] = ACTIONS(2767),
    [anon_sym_if] = ACTIONS(2767),
    [anon_sym_impl] = ACTIONS(2767),
    [anon_sym_let] = ACTIONS(2767),
    [anon_sym_loop] = ACTIONS(2767),
    [anon_sym_match] = ACTIONS(2767),
    [anon_sym_mod] = ACTIONS(2767),
    [anon_sym_pub] = ACTIONS(2767),
    [anon_sym_return] = ACTIONS(2767),
    [anon_sym_static] = ACTIONS(2767),
    [anon_sym_struct] = ACTIONS(2767),
    [anon_sym_trait] = ACTIONS(2767),
    [anon_sym_type] = ACTIONS(2767),
    [anon_sym_union] = ACTIONS(2767),
    [anon_sym_unsafe] = ACTIONS(2767),
    [anon_sym_use] = ACTIONS(2767),
    [anon_sym_while] = ACTIONS(2767),
    [anon_sym_extern] = ACTIONS(2767),
    [anon_sym_yield] = ACTIONS(2767),
    [anon_sym_move] = ACTIONS(2767),
    [anon_sym_try] = ACTIONS(2767),
    [sym_integer_literal] = ACTIONS(2765),
    [aux_sym_string_literal_token1] = ACTIONS(2765),
    [sym_char_literal] = ACTIONS(2765),
    [anon_sym_true] = ACTIONS(2767),
    [anon_sym_false] = ACTIONS(2767),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2767),
    [sym_super] = ACTIONS(2767),
    [sym_crate] = ACTIONS(2767),
    [sym_metavariable] = ACTIONS(2765),
    [sym__raw_string_literal_start] = ACTIONS(2765),
    [sym_float_literal] = ACTIONS(2765),
  },
  [714] = {
    [sym_line_comment] = STATE(714),
    [sym_block_comment] = STATE(714),
    [ts_builtin_sym_end] = ACTIONS(2769),
    [sym_identifier] = ACTIONS(2771),
    [anon_sym_SEMI] = ACTIONS(2769),
    [anon_sym_macro_rules_BANG] = ACTIONS(2769),
    [anon_sym_LPAREN] = ACTIONS(2769),
    [anon_sym_LBRACK] = ACTIONS(2769),
    [anon_sym_LBRACE] = ACTIONS(2769),
    [anon_sym_RBRACE] = ACTIONS(2769),
    [anon_sym_STAR] = ACTIONS(2769),
    [anon_sym_u8] = ACTIONS(2771),
    [anon_sym_i8] = ACTIONS(2771),
    [anon_sym_u16] = ACTIONS(2771),
    [anon_sym_i16] = ACTIONS(2771),
    [anon_sym_u32] = ACTIONS(2771),
    [anon_sym_i32] = ACTIONS(2771),
    [anon_sym_u64] = ACTIONS(2771),
    [anon_sym_i64] = ACTIONS(2771),
    [anon_sym_u128] = ACTIONS(2771),
    [anon_sym_i128] = ACTIONS(2771),
    [anon_sym_isize] = ACTIONS(2771),
    [anon_sym_usize] = ACTIONS(2771),
    [anon_sym_f32] = ACTIONS(2771),
    [anon_sym_f64] = ACTIONS(2771),
    [anon_sym_bool] = ACTIONS(2771),
    [anon_sym_str] = ACTIONS(2771),
    [anon_sym_char] = ACTIONS(2771),
    [anon_sym_DASH] = ACTIONS(2769),
    [anon_sym_BANG] = ACTIONS(2769),
    [anon_sym_AMP] = ACTIONS(2769),
    [anon_sym_PIPE] = ACTIONS(2769),
    [anon_sym_LT] = ACTIONS(2769),
    [anon_sym_DOT_DOT] = ACTIONS(2769),
    [anon_sym_COLON_COLON] = ACTIONS(2769),
    [anon_sym_POUND] = ACTIONS(2769),
    [anon_sym_SQUOTE] = ACTIONS(2771),
    [anon_sym_async] = ACTIONS(2771),
    [anon_sym_break] = ACTIONS(2771),
    [anon_sym_const] = ACTIONS(2771),
    [anon_sym_continue] = ACTIONS(2771),
    [anon_sym_default] = ACTIONS(2771),
    [anon_sym_enum] = ACTIONS(2771),
    [anon_sym_fn] = ACTIONS(2771),
    [anon_sym_for] = ACTIONS(2771),
    [anon_sym_gen] = ACTIONS(2771),
    [anon_sym_if] = ACTIONS(2771),
    [anon_sym_impl] = ACTIONS(2771),
    [anon_sym_let] = ACTIONS(2771),
    [anon_sym_loop] = ACTIONS(2771),
    [anon_sym_match] = ACTIONS(2771),
    [anon_sym_mod] = ACTIONS(2771),
    [anon_sym_pub] = ACTIONS(2771),
    [anon_sym_return] = ACTIONS(2771),
    [anon_sym_static] = ACTIONS(2771),
    [anon_sym_struct] = ACTIONS(2771),
    [anon_sym_trait] = ACTIONS(2771),
    [anon_sym_type] = ACTIONS(2771),
    [anon_sym_union] = ACTIONS(2771),
    [anon_sym_unsafe] = ACTIONS(2771),
    [anon_sym_use] = ACTIONS(2771),
    [anon_sym_while] = ACTIONS(2771),
    [anon_sym_extern] = ACTIONS(2771),
    [anon_sym_yield] = ACTIONS(2771),
    [anon_sym_move] = ACTIONS(2771),
    [anon_sym_try] = ACTIONS(2771),
    [sym_integer_literal] = ACTIONS(2769),
    [aux_sym_string_literal_token1] = ACTIONS(2769),
    [sym_char_literal] = ACTIONS(2769),
    [anon_sym_true] = ACTIONS(2771),
    [anon_sym_false] = ACTIONS(2771),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2771),
    [sym_super] = ACTIONS(2771),
    [sym_crate] = ACTIONS(2771),
    [sym_metavariable] = ACTIONS(2769),
    [sym__raw_string_literal_start] = ACTIONS(2769),
    [sym_float_literal] = ACTIONS(2769),
  },
  [715] = {
    [sym_line_comment] = STATE(715),
    [sym_block_comment] = STATE(715),
    [ts_builtin_sym_end] = ACTIONS(2773),
    [sym_identifier] = ACTIONS(2775),
    [anon_sym_SEMI] = ACTIONS(2773),
    [anon_sym_macro_rules_BANG] = ACTIONS(2773),
    [anon_sym_LPAREN] = ACTIONS(2773),
    [anon_sym_LBRACK] = ACTIONS(2773),
    [anon_sym_LBRACE] = ACTIONS(2773),
    [anon_sym_RBRACE] = ACTIONS(2773),
    [anon_sym_STAR] = ACTIONS(2773),
    [anon_sym_u8] = ACTIONS(2775),
    [anon_sym_i8] = ACTIONS(2775),
    [anon_sym_u16] = ACTIONS(2775),
    [anon_sym_i16] = ACTIONS(2775),
    [anon_sym_u32] = ACTIONS(2775),
    [anon_sym_i32] = ACTIONS(2775),
    [anon_sym_u64] = ACTIONS(2775),
    [anon_sym_i64] = ACTIONS(2775),
    [anon_sym_u128] = ACTIONS(2775),
    [anon_sym_i128] = ACTIONS(2775),
    [anon_sym_isize] = ACTIONS(2775),
    [anon_sym_usize] = ACTIONS(2775),
    [anon_sym_f32] = ACTIONS(2775),
    [anon_sym_f64] = ACTIONS(2775),
    [anon_sym_bool] = ACTIONS(2775),
    [anon_sym_str] = ACTIONS(2775),
    [anon_sym_char] = ACTIONS(2775),
    [anon_sym_DASH] = ACTIONS(2773),
    [anon_sym_BANG] = ACTIONS(2773),
    [anon_sym_AMP] = ACTIONS(2773),
    [anon_sym_PIPE] = ACTIONS(2773),
    [anon_sym_LT] = ACTIONS(2773),
    [anon_sym_DOT_DOT] = ACTIONS(2773),
    [anon_sym_COLON_COLON] = ACTIONS(2773),
    [anon_sym_POUND] = ACTIONS(2773),
    [anon_sym_SQUOTE] = ACTIONS(2775),
    [anon_sym_async] = ACTIONS(2775),
    [anon_sym_break] = ACTIONS(2775),
    [anon_sym_const] = ACTIONS(2775),
    [anon_sym_continue] = ACTIONS(2775),
    [anon_sym_default] = ACTIONS(2775),
    [anon_sym_enum] = ACTIONS(2775),
    [anon_sym_fn] = ACTIONS(2775),
    [anon_sym_for] = ACTIONS(2775),
    [anon_sym_gen] = ACTIONS(2775),
    [anon_sym_if] = ACTIONS(2775),
    [anon_sym_impl] = ACTIONS(2775),
    [anon_sym_let] = ACTIONS(2775),
    [anon_sym_loop] = ACTIONS(2775),
    [anon_sym_match] = ACTIONS(2775),
    [anon_sym_mod] = ACTIONS(2775),
    [anon_sym_pub] = ACTIONS(2775),
    [anon_sym_return] = ACTIONS(2775),
    [anon_sym_static] = ACTIONS(2775),
    [anon_sym_struct] = ACTIONS(2775),
    [anon_sym_trait] = ACTIONS(2775),
    [anon_sym_type] = ACTIONS(2775),
    [anon_sym_union] = ACTIONS(2775),
    [anon_sym_unsafe] = ACTIONS(2775),
    [anon_sym_use] = ACTIONS(2775),
    [anon_sym_while] = ACTIONS(2775),
    [anon_sym_extern] = ACTIONS(2775),
    [anon_sym_yield] = ACTIONS(2775),
    [anon_sym_move] = ACTIONS(2775),
    [anon_sym_try] = ACTIONS(2775),
    [sym_integer_literal] = ACTIONS(2773),
    [aux_sym_string_literal_token1] = ACTIONS(2773),
    [sym_char_literal] = ACTIONS(2773),
    [anon_sym_true] = ACTIONS(2775),
    [anon_sym_false] = ACTIONS(2775),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2775),
    [sym_super] = ACTIONS(2775),
    [sym_crate] = ACTIONS(2775),
    [sym_metavariable] = ACTIONS(2773),
    [sym__raw_string_literal_start] = ACTIONS(2773),
    [sym_float_literal] = ACTIONS(2773),
  },
  [716] = {
    [sym_line_comment] = STATE(716),
    [sym_block_comment] = STATE(716),
    [ts_builtin_sym_end] = ACTIONS(2777),
    [sym_identifier] = ACTIONS(2779),
    [anon_sym_SEMI] = ACTIONS(2777),
    [anon_sym_macro_rules_BANG] = ACTIONS(2777),
    [anon_sym_LPAREN] = ACTIONS(2777),
    [anon_sym_LBRACK] = ACTIONS(2777),
    [anon_sym_LBRACE] = ACTIONS(2777),
    [anon_sym_RBRACE] = ACTIONS(2777),
    [anon_sym_STAR] = ACTIONS(2777),
    [anon_sym_u8] = ACTIONS(2779),
    [anon_sym_i8] = ACTIONS(2779),
    [anon_sym_u16] = ACTIONS(2779),
    [anon_sym_i16] = ACTIONS(2779),
    [anon_sym_u32] = ACTIONS(2779),
    [anon_sym_i32] = ACTIONS(2779),
    [anon_sym_u64] = ACTIONS(2779),
    [anon_sym_i64] = ACTIONS(2779),
    [anon_sym_u128] = ACTIONS(2779),
    [anon_sym_i128] = ACTIONS(2779),
    [anon_sym_isize] = ACTIONS(2779),
    [anon_sym_usize] = ACTIONS(2779),
    [anon_sym_f32] = ACTIONS(2779),
    [anon_sym_f64] = ACTIONS(2779),
    [anon_sym_bool] = ACTIONS(2779),
    [anon_sym_str] = ACTIONS(2779),
    [anon_sym_char] = ACTIONS(2779),
    [anon_sym_DASH] = ACTIONS(2777),
    [anon_sym_BANG] = ACTIONS(2777),
    [anon_sym_AMP] = ACTIONS(2777),
    [anon_sym_PIPE] = ACTIONS(2777),
    [anon_sym_LT] = ACTIONS(2777),
    [anon_sym_DOT_DOT] = ACTIONS(2777),
    [anon_sym_COLON_COLON] = ACTIONS(2777),
    [anon_sym_POUND] = ACTIONS(2777),
    [anon_sym_SQUOTE] = ACTIONS(2779),
    [anon_sym_async] = ACTIONS(2779),
    [anon_sym_break] = ACTIONS(2779),
    [anon_sym_const] = ACTIONS(2779),
    [anon_sym_continue] = ACTIONS(2779),
    [anon_sym_default] = ACTIONS(2779),
    [anon_sym_enum] = ACTIONS(2779),
    [anon_sym_fn] = ACTIONS(2779),
    [anon_sym_for] = ACTIONS(2779),
    [anon_sym_gen] = ACTIONS(2779),
    [anon_sym_if] = ACTIONS(2779),
    [anon_sym_impl] = ACTIONS(2779),
    [anon_sym_let] = ACTIONS(2779),
    [anon_sym_loop] = ACTIONS(2779),
    [anon_sym_match] = ACTIONS(2779),
    [anon_sym_mod] = ACTIONS(2779),
    [anon_sym_pub] = ACTIONS(2779),
    [anon_sym_return] = ACTIONS(2779),
    [anon_sym_static] = ACTIONS(2779),
    [anon_sym_struct] = ACTIONS(2779),
    [anon_sym_trait] = ACTIONS(2779),
    [anon_sym_type] = ACTIONS(2779),
    [anon_sym_union] = ACTIONS(2779),
    [anon_sym_unsafe] = ACTIONS(2779),
    [anon_sym_use] = ACTIONS(2779),
    [anon_sym_while] = ACTIONS(2779),
    [anon_sym_extern] = ACTIONS(2779),
    [anon_sym_yield] = ACTIONS(2779),
    [anon_sym_move] = ACTIONS(2779),
    [anon_sym_try] = ACTIONS(2779),
    [sym_integer_literal] = ACTIONS(2777),
    [aux_sym_string_literal_token1] = ACTIONS(2777),
    [sym_char_literal] = ACTIONS(2777),
    [anon_sym_true] = ACTIONS(2779),
    [anon_sym_false] = ACTIONS(2779),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2779),
    [sym_super] = ACTIONS(2779),
    [sym_crate] = ACTIONS(2779),
    [sym_metavariable] = ACTIONS(2777),
    [sym__raw_string_literal_start] = ACTIONS(2777),
    [sym_float_literal] = ACTIONS(2777),
  },
  [717] = {
    [sym_line_comment] = STATE(717),
    [sym_block_comment] = STATE(717),
    [ts_builtin_sym_end] = ACTIONS(2781),
    [sym_identifier] = ACTIONS(2783),
    [anon_sym_SEMI] = ACTIONS(2781),
    [anon_sym_macro_rules_BANG] = ACTIONS(2781),
    [anon_sym_LPAREN] = ACTIONS(2781),
    [anon_sym_LBRACK] = ACTIONS(2781),
    [anon_sym_LBRACE] = ACTIONS(2781),
    [anon_sym_RBRACE] = ACTIONS(2781),
    [anon_sym_STAR] = ACTIONS(2781),
    [anon_sym_u8] = ACTIONS(2783),
    [anon_sym_i8] = ACTIONS(2783),
    [anon_sym_u16] = ACTIONS(2783),
    [anon_sym_i16] = ACTIONS(2783),
    [anon_sym_u32] = ACTIONS(2783),
    [anon_sym_i32] = ACTIONS(2783),
    [anon_sym_u64] = ACTIONS(2783),
    [anon_sym_i64] = ACTIONS(2783),
    [anon_sym_u128] = ACTIONS(2783),
    [anon_sym_i128] = ACTIONS(2783),
    [anon_sym_isize] = ACTIONS(2783),
    [anon_sym_usize] = ACTIONS(2783),
    [anon_sym_f32] = ACTIONS(2783),
    [anon_sym_f64] = ACTIONS(2783),
    [anon_sym_bool] = ACTIONS(2783),
    [anon_sym_str] = ACTIONS(2783),
    [anon_sym_char] = ACTIONS(2783),
    [anon_sym_DASH] = ACTIONS(2781),
    [anon_sym_BANG] = ACTIONS(2781),
    [anon_sym_AMP] = ACTIONS(2781),
    [anon_sym_PIPE] = ACTIONS(2781),
    [anon_sym_LT] = ACTIONS(2781),
    [anon_sym_DOT_DOT] = ACTIONS(2781),
    [anon_sym_COLON_COLON] = ACTIONS(2781),
    [anon_sym_POUND] = ACTIONS(2781),
    [anon_sym_SQUOTE] = ACTIONS(2783),
    [anon_sym_async] = ACTIONS(2783),
    [anon_sym_break] = ACTIONS(2783),
    [anon_sym_const] = ACTIONS(2783),
    [anon_sym_continue] = ACTIONS(2783),
    [anon_sym_default] = ACTIONS(2783),
    [anon_sym_enum] = ACTIONS(2783),
    [anon_sym_fn] = ACTIONS(2783),
    [anon_sym_for] = ACTIONS(2783),
    [anon_sym_gen] = ACTIONS(2783),
    [anon_sym_if] = ACTIONS(2783),
    [anon_sym_impl] = ACTIONS(2783),
    [anon_sym_let] = ACTIONS(2783),
    [anon_sym_loop] = ACTIONS(2783),
    [anon_sym_match] = ACTIONS(2783),
    [anon_sym_mod] = ACTIONS(2783),
    [anon_sym_pub] = ACTIONS(2783),
    [anon_sym_return] = ACTIONS(2783),
    [anon_sym_static] = ACTIONS(2783),
    [anon_sym_struct] = ACTIONS(2783),
    [anon_sym_trait] = ACTIONS(2783),
    [anon_sym_type] = ACTIONS(2783),
    [anon_sym_union] = ACTIONS(2783),
    [anon_sym_unsafe] = ACTIONS(2783),
    [anon_sym_use] = ACTIONS(2783),
    [anon_sym_while] = ACTIONS(2783),
    [anon_sym_extern] = ACTIONS(2783),
    [anon_sym_yield] = ACTIONS(2783),
    [anon_sym_move] = ACTIONS(2783),
    [anon_sym_try] = ACTIONS(2783),
    [sym_integer_literal] = ACTIONS(2781),
    [aux_sym_string_literal_token1] = ACTIONS(2781),
    [sym_char_literal] = ACTIONS(2781),
    [anon_sym_true] = ACTIONS(2783),
    [anon_sym_false] = ACTIONS(2783),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2783),
    [sym_super] = ACTIONS(2783),
    [sym_crate] = ACTIONS(2783),
    [sym_metavariable] = ACTIONS(2781),
    [sym__raw_string_literal_start] = ACTIONS(2781),
    [sym_float_literal] = ACTIONS(2781),
  },
  [718] = {
    [sym_line_comment] = STATE(718),
    [sym_block_comment] = STATE(718),
    [ts_builtin_sym_end] = ACTIONS(2785),
    [sym_identifier] = ACTIONS(2787),
    [anon_sym_SEMI] = ACTIONS(2785),
    [anon_sym_macro_rules_BANG] = ACTIONS(2785),
    [anon_sym_LPAREN] = ACTIONS(2785),
    [anon_sym_LBRACK] = ACTIONS(2785),
    [anon_sym_LBRACE] = ACTIONS(2785),
    [anon_sym_RBRACE] = ACTIONS(2785),
    [anon_sym_STAR] = ACTIONS(2785),
    [anon_sym_u8] = ACTIONS(2787),
    [anon_sym_i8] = ACTIONS(2787),
    [anon_sym_u16] = ACTIONS(2787),
    [anon_sym_i16] = ACTIONS(2787),
    [anon_sym_u32] = ACTIONS(2787),
    [anon_sym_i32] = ACTIONS(2787),
    [anon_sym_u64] = ACTIONS(2787),
    [anon_sym_i64] = ACTIONS(2787),
    [anon_sym_u128] = ACTIONS(2787),
    [anon_sym_i128] = ACTIONS(2787),
    [anon_sym_isize] = ACTIONS(2787),
    [anon_sym_usize] = ACTIONS(2787),
    [anon_sym_f32] = ACTIONS(2787),
    [anon_sym_f64] = ACTIONS(2787),
    [anon_sym_bool] = ACTIONS(2787),
    [anon_sym_str] = ACTIONS(2787),
    [anon_sym_char] = ACTIONS(2787),
    [anon_sym_DASH] = ACTIONS(2785),
    [anon_sym_BANG] = ACTIONS(2785),
    [anon_sym_AMP] = ACTIONS(2785),
    [anon_sym_PIPE] = ACTIONS(2785),
    [anon_sym_LT] = ACTIONS(2785),
    [anon_sym_DOT_DOT] = ACTIONS(2785),
    [anon_sym_COLON_COLON] = ACTIONS(2785),
    [anon_sym_POUND] = ACTIONS(2785),
    [anon_sym_SQUOTE] = ACTIONS(2787),
    [anon_sym_async] = ACTIONS(2787),
    [anon_sym_break] = ACTIONS(2787),
    [anon_sym_const] = ACTIONS(2787),
    [anon_sym_continue] = ACTIONS(2787),
    [anon_sym_default] = ACTIONS(2787),
    [anon_sym_enum] = ACTIONS(2787),
    [anon_sym_fn] = ACTIONS(2787),
    [anon_sym_for] = ACTIONS(2787),
    [anon_sym_gen] = ACTIONS(2787),
    [anon_sym_if] = ACTIONS(2787),
    [anon_sym_impl] = ACTIONS(2787),
    [anon_sym_let] = ACTIONS(2787),
    [anon_sym_loop] = ACTIONS(2787),
    [anon_sym_match] = ACTIONS(2787),
    [anon_sym_mod] = ACTIONS(2787),
    [anon_sym_pub] = ACTIONS(2787),
    [anon_sym_return] = ACTIONS(2787),
    [anon_sym_static] = ACTIONS(2787),
    [anon_sym_struct] = ACTIONS(2787),
    [anon_sym_trait] = ACTIONS(2787),
    [anon_sym_type] = ACTIONS(2787),
    [anon_sym_union] = ACTIONS(2787),
    [anon_sym_unsafe] = ACTIONS(2787),
    [anon_sym_use] = ACTIONS(2787),
    [anon_sym_while] = ACTIONS(2787),
    [anon_sym_extern] = ACTIONS(2787),
    [anon_sym_yield] = ACTIONS(2787),
    [anon_sym_move] = ACTIONS(2787),
    [anon_sym_try] = ACTIONS(2787),
    [sym_integer_literal] = ACTIONS(2785),
    [aux_sym_string_literal_token1] = ACTIONS(2785),
    [sym_char_literal] = ACTIONS(2785),
    [anon_sym_true] = ACTIONS(2787),
    [anon_sym_false] = ACTIONS(2787),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2787),
    [sym_super] = ACTIONS(2787),
    [sym_crate] = ACTIONS(2787),
    [sym_metavariable] = ACTIONS(2785),
    [sym__raw_string_literal_start] = ACTIONS(2785),
    [sym_float_literal] = ACTIONS(2785),
  },
  [719] = {
    [sym_line_comment] = STATE(719),
    [sym_block_comment] = STATE(719),
    [ts_builtin_sym_end] = ACTIONS(2789),
    [sym_identifier] = ACTIONS(2791),
    [anon_sym_SEMI] = ACTIONS(2789),
    [anon_sym_macro_rules_BANG] = ACTIONS(2789),
    [anon_sym_LPAREN] = ACTIONS(2789),
    [anon_sym_LBRACK] = ACTIONS(2789),
    [anon_sym_LBRACE] = ACTIONS(2789),
    [anon_sym_RBRACE] = ACTIONS(2789),
    [anon_sym_STAR] = ACTIONS(2789),
    [anon_sym_u8] = ACTIONS(2791),
    [anon_sym_i8] = ACTIONS(2791),
    [anon_sym_u16] = ACTIONS(2791),
    [anon_sym_i16] = ACTIONS(2791),
    [anon_sym_u32] = ACTIONS(2791),
    [anon_sym_i32] = ACTIONS(2791),
    [anon_sym_u64] = ACTIONS(2791),
    [anon_sym_i64] = ACTIONS(2791),
    [anon_sym_u128] = ACTIONS(2791),
    [anon_sym_i128] = ACTIONS(2791),
    [anon_sym_isize] = ACTIONS(2791),
    [anon_sym_usize] = ACTIONS(2791),
    [anon_sym_f32] = ACTIONS(2791),
    [anon_sym_f64] = ACTIONS(2791),
    [anon_sym_bool] = ACTIONS(2791),
    [anon_sym_str] = ACTIONS(2791),
    [anon_sym_char] = ACTIONS(2791),
    [anon_sym_DASH] = ACTIONS(2789),
    [anon_sym_BANG] = ACTIONS(2789),
    [anon_sym_AMP] = ACTIONS(2789),
    [anon_sym_PIPE] = ACTIONS(2789),
    [anon_sym_LT] = ACTIONS(2789),
    [anon_sym_DOT_DOT] = ACTIONS(2789),
    [anon_sym_COLON_COLON] = ACTIONS(2789),
    [anon_sym_POUND] = ACTIONS(2789),
    [anon_sym_SQUOTE] = ACTIONS(2791),
    [anon_sym_async] = ACTIONS(2791),
    [anon_sym_break] = ACTIONS(2791),
    [anon_sym_const] = ACTIONS(2791),
    [anon_sym_continue] = ACTIONS(2791),
    [anon_sym_default] = ACTIONS(2791),
    [anon_sym_enum] = ACTIONS(2791),
    [anon_sym_fn] = ACTIONS(2791),
    [anon_sym_for] = ACTIONS(2791),
    [anon_sym_gen] = ACTIONS(2791),
    [anon_sym_if] = ACTIONS(2791),
    [anon_sym_impl] = ACTIONS(2791),
    [anon_sym_let] = ACTIONS(2791),
    [anon_sym_loop] = ACTIONS(2791),
    [anon_sym_match] = ACTIONS(2791),
    [anon_sym_mod] = ACTIONS(2791),
    [anon_sym_pub] = ACTIONS(2791),
    [anon_sym_return] = ACTIONS(2791),
    [anon_sym_static] = ACTIONS(2791),
    [anon_sym_struct] = ACTIONS(2791),
    [anon_sym_trait] = ACTIONS(2791),
    [anon_sym_type] = ACTIONS(2791),
    [anon_sym_union] = ACTIONS(2791),
    [anon_sym_unsafe] = ACTIONS(2791),
    [anon_sym_use] = ACTIONS(2791),
    [anon_sym_while] = ACTIONS(2791),
    [anon_sym_extern] = ACTIONS(2791),
    [anon_sym_yield] = ACTIONS(2791),
    [anon_sym_move] = ACTIONS(2791),
    [anon_sym_try] = ACTIONS(2791),
    [sym_integer_literal] = ACTIONS(2789),
    [aux_sym_string_literal_token1] = ACTIONS(2789),
    [sym_char_literal] = ACTIONS(2789),
    [anon_sym_true] = ACTIONS(2791),
    [anon_sym_false] = ACTIONS(2791),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2791),
    [sym_super] = ACTIONS(2791),
    [sym_crate] = ACTIONS(2791),
    [sym_metavariable] = ACTIONS(2789),
    [sym__raw_string_literal_start] = ACTIONS(2789),
    [sym_float_literal] = ACTIONS(2789),
  },
  [720] = {
    [sym_line_comment] = STATE(720),
    [sym_block_comment] = STATE(720),
    [ts_builtin_sym_end] = ACTIONS(2793),
    [sym_identifier] = ACTIONS(2795),
    [anon_sym_SEMI] = ACTIONS(2793),
    [anon_sym_macro_rules_BANG] = ACTIONS(2793),
    [anon_sym_LPAREN] = ACTIONS(2793),
    [anon_sym_LBRACK] = ACTIONS(2793),
    [anon_sym_LBRACE] = ACTIONS(2793),
    [anon_sym_RBRACE] = ACTIONS(2793),
    [anon_sym_STAR] = ACTIONS(2793),
    [anon_sym_u8] = ACTIONS(2795),
    [anon_sym_i8] = ACTIONS(2795),
    [anon_sym_u16] = ACTIONS(2795),
    [anon_sym_i16] = ACTIONS(2795),
    [anon_sym_u32] = ACTIONS(2795),
    [anon_sym_i32] = ACTIONS(2795),
    [anon_sym_u64] = ACTIONS(2795),
    [anon_sym_i64] = ACTIONS(2795),
    [anon_sym_u128] = ACTIONS(2795),
    [anon_sym_i128] = ACTIONS(2795),
    [anon_sym_isize] = ACTIONS(2795),
    [anon_sym_usize] = ACTIONS(2795),
    [anon_sym_f32] = ACTIONS(2795),
    [anon_sym_f64] = ACTIONS(2795),
    [anon_sym_bool] = ACTIONS(2795),
    [anon_sym_str] = ACTIONS(2795),
    [anon_sym_char] = ACTIONS(2795),
    [anon_sym_DASH] = ACTIONS(2793),
    [anon_sym_BANG] = ACTIONS(2793),
    [anon_sym_AMP] = ACTIONS(2793),
    [anon_sym_PIPE] = ACTIONS(2793),
    [anon_sym_LT] = ACTIONS(2793),
    [anon_sym_DOT_DOT] = ACTIONS(2793),
    [anon_sym_COLON_COLON] = ACTIONS(2793),
    [anon_sym_POUND] = ACTIONS(2793),
    [anon_sym_SQUOTE] = ACTIONS(2795),
    [anon_sym_async] = ACTIONS(2795),
    [anon_sym_break] = ACTIONS(2795),
    [anon_sym_const] = ACTIONS(2795),
    [anon_sym_continue] = ACTIONS(2795),
    [anon_sym_default] = ACTIONS(2795),
    [anon_sym_enum] = ACTIONS(2795),
    [anon_sym_fn] = ACTIONS(2795),
    [anon_sym_for] = ACTIONS(2795),
    [anon_sym_gen] = ACTIONS(2795),
    [anon_sym_if] = ACTIONS(2795),
    [anon_sym_impl] = ACTIONS(2795),
    [anon_sym_let] = ACTIONS(2795),
    [anon_sym_loop] = ACTIONS(2795),
    [anon_sym_match] = ACTIONS(2795),
    [anon_sym_mod] = ACTIONS(2795),
    [anon_sym_pub] = ACTIONS(2795),
    [anon_sym_return] = ACTIONS(2795),
    [anon_sym_static] = ACTIONS(2795),
    [anon_sym_struct] = ACTIONS(2795),
    [anon_sym_trait] = ACTIONS(2795),
    [anon_sym_type] = ACTIONS(2795),
    [anon_sym_union] = ACTIONS(2795),
    [anon_sym_unsafe] = ACTIONS(2795),
    [anon_sym_use] = ACTIONS(2795),
    [anon_sym_while] = ACTIONS(2795),
    [anon_sym_extern] = ACTIONS(2795),
    [anon_sym_yield] = ACTIONS(2795),
    [anon_sym_move] = ACTIONS(2795),
    [anon_sym_try] = ACTIONS(2795),
    [sym_integer_literal] = ACTIONS(2793),
    [aux_sym_string_literal_token1] = ACTIONS(2793),
    [sym_char_literal] = ACTIONS(2793),
    [anon_sym_true] = ACTIONS(2795),
    [anon_sym_false] = ACTIONS(2795),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2795),
    [sym_super] = ACTIONS(2795),
    [sym_crate] = ACTIONS(2795),
    [sym_metavariable] = ACTIONS(2793),
    [sym__raw_string_literal_start] = ACTIONS(2793),
    [sym_float_literal] = ACTIONS(2793),
  },
  [721] = {
    [sym_line_comment] = STATE(721),
    [sym_block_comment] = STATE(721),
    [ts_builtin_sym_end] = ACTIONS(2797),
    [sym_identifier] = ACTIONS(2799),
    [anon_sym_SEMI] = ACTIONS(2797),
    [anon_sym_macro_rules_BANG] = ACTIONS(2797),
    [anon_sym_LPAREN] = ACTIONS(2797),
    [anon_sym_LBRACK] = ACTIONS(2797),
    [anon_sym_LBRACE] = ACTIONS(2797),
    [anon_sym_RBRACE] = ACTIONS(2797),
    [anon_sym_STAR] = ACTIONS(2797),
    [anon_sym_u8] = ACTIONS(2799),
    [anon_sym_i8] = ACTIONS(2799),
    [anon_sym_u16] = ACTIONS(2799),
    [anon_sym_i16] = ACTIONS(2799),
    [anon_sym_u32] = ACTIONS(2799),
    [anon_sym_i32] = ACTIONS(2799),
    [anon_sym_u64] = ACTIONS(2799),
    [anon_sym_i64] = ACTIONS(2799),
    [anon_sym_u128] = ACTIONS(2799),
    [anon_sym_i128] = ACTIONS(2799),
    [anon_sym_isize] = ACTIONS(2799),
    [anon_sym_usize] = ACTIONS(2799),
    [anon_sym_f32] = ACTIONS(2799),
    [anon_sym_f64] = ACTIONS(2799),
    [anon_sym_bool] = ACTIONS(2799),
    [anon_sym_str] = ACTIONS(2799),
    [anon_sym_char] = ACTIONS(2799),
    [anon_sym_DASH] = ACTIONS(2797),
    [anon_sym_BANG] = ACTIONS(2797),
    [anon_sym_AMP] = ACTIONS(2797),
    [anon_sym_PIPE] = ACTIONS(2797),
    [anon_sym_LT] = ACTIONS(2797),
    [anon_sym_DOT_DOT] = ACTIONS(2797),
    [anon_sym_COLON_COLON] = ACTIONS(2797),
    [anon_sym_POUND] = ACTIONS(2797),
    [anon_sym_SQUOTE] = ACTIONS(2799),
    [anon_sym_async] = ACTIONS(2799),
    [anon_sym_break] = ACTIONS(2799),
    [anon_sym_const] = ACTIONS(2799),
    [anon_sym_continue] = ACTIONS(2799),
    [anon_sym_default] = ACTIONS(2799),
    [anon_sym_enum] = ACTIONS(2799),
    [anon_sym_fn] = ACTIONS(2799),
    [anon_sym_for] = ACTIONS(2799),
    [anon_sym_gen] = ACTIONS(2799),
    [anon_sym_if] = ACTIONS(2799),
    [anon_sym_impl] = ACTIONS(2799),
    [anon_sym_let] = ACTIONS(2799),
    [anon_sym_loop] = ACTIONS(2799),
    [anon_sym_match] = ACTIONS(2799),
    [anon_sym_mod] = ACTIONS(2799),
    [anon_sym_pub] = ACTIONS(2799),
    [anon_sym_return] = ACTIONS(2799),
    [anon_sym_static] = ACTIONS(2799),
    [anon_sym_struct] = ACTIONS(2799),
    [anon_sym_trait] = ACTIONS(2799),
    [anon_sym_type] = ACTIONS(2799),
    [anon_sym_union] = ACTIONS(2799),
    [anon_sym_unsafe] = ACTIONS(2799),
    [anon_sym_use] = ACTIONS(2799),
    [anon_sym_while] = ACTIONS(2799),
    [anon_sym_extern] = ACTIONS(2799),
    [anon_sym_yield] = ACTIONS(2799),
    [anon_sym_move] = ACTIONS(2799),
    [anon_sym_try] = ACTIONS(2799),
    [sym_integer_literal] = ACTIONS(2797),
    [aux_sym_string_literal_token1] = ACTIONS(2797),
    [sym_char_literal] = ACTIONS(2797),
    [anon_sym_true] = ACTIONS(2799),
    [anon_sym_false] = ACTIONS(2799),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2799),
    [sym_super] = ACTIONS(2799),
    [sym_crate] = ACTIONS(2799),
    [sym_metavariable] = ACTIONS(2797),
    [sym__raw_string_literal_start] = ACTIONS(2797),
    [sym_float_literal] = ACTIONS(2797),
  },
  [722] = {
    [sym_line_comment] = STATE(722),
    [sym_block_comment] = STATE(722),
    [ts_builtin_sym_end] = ACTIONS(2801),
    [sym_identifier] = ACTIONS(2803),
    [anon_sym_SEMI] = ACTIONS(2801),
    [anon_sym_macro_rules_BANG] = ACTIONS(2801),
    [anon_sym_LPAREN] = ACTIONS(2801),
    [anon_sym_LBRACK] = ACTIONS(2801),
    [anon_sym_LBRACE] = ACTIONS(2801),
    [anon_sym_RBRACE] = ACTIONS(2801),
    [anon_sym_STAR] = ACTIONS(2801),
    [anon_sym_u8] = ACTIONS(2803),
    [anon_sym_i8] = ACTIONS(2803),
    [anon_sym_u16] = ACTIONS(2803),
    [anon_sym_i16] = ACTIONS(2803),
    [anon_sym_u32] = ACTIONS(2803),
    [anon_sym_i32] = ACTIONS(2803),
    [anon_sym_u64] = ACTIONS(2803),
    [anon_sym_i64] = ACTIONS(2803),
    [anon_sym_u128] = ACTIONS(2803),
    [anon_sym_i128] = ACTIONS(2803),
    [anon_sym_isize] = ACTIONS(2803),
    [anon_sym_usize] = ACTIONS(2803),
    [anon_sym_f32] = ACTIONS(2803),
    [anon_sym_f64] = ACTIONS(2803),
    [anon_sym_bool] = ACTIONS(2803),
    [anon_sym_str] = ACTIONS(2803),
    [anon_sym_char] = ACTIONS(2803),
    [anon_sym_DASH] = ACTIONS(2801),
    [anon_sym_BANG] = ACTIONS(2801),
    [anon_sym_AMP] = ACTIONS(2801),
    [anon_sym_PIPE] = ACTIONS(2801),
    [anon_sym_LT] = ACTIONS(2801),
    [anon_sym_DOT_DOT] = ACTIONS(2801),
    [anon_sym_COLON_COLON] = ACTIONS(2801),
    [anon_sym_POUND] = ACTIONS(2801),
    [anon_sym_SQUOTE] = ACTIONS(2803),
    [anon_sym_async] = ACTIONS(2803),
    [anon_sym_break] = ACTIONS(2803),
    [anon_sym_const] = ACTIONS(2803),
    [anon_sym_continue] = ACTIONS(2803),
    [anon_sym_default] = ACTIONS(2803),
    [anon_sym_enum] = ACTIONS(2803),
    [anon_sym_fn] = ACTIONS(2803),
    [anon_sym_for] = ACTIONS(2803),
    [anon_sym_gen] = ACTIONS(2803),
    [anon_sym_if] = ACTIONS(2803),
    [anon_sym_impl] = ACTIONS(2803),
    [anon_sym_let] = ACTIONS(2803),
    [anon_sym_loop] = ACTIONS(2803),
    [anon_sym_match] = ACTIONS(2803),
    [anon_sym_mod] = ACTIONS(2803),
    [anon_sym_pub] = ACTIONS(2803),
    [anon_sym_return] = ACTIONS(2803),
    [anon_sym_static] = ACTIONS(2803),
    [anon_sym_struct] = ACTIONS(2803),
    [anon_sym_trait] = ACTIONS(2803),
    [anon_sym_type] = ACTIONS(2803),
    [anon_sym_union] = ACTIONS(2803),
    [anon_sym_unsafe] = ACTIONS(2803),
    [anon_sym_use] = ACTIONS(2803),
    [anon_sym_while] = ACTIONS(2803),
    [anon_sym_extern] = ACTIONS(2803),
    [anon_sym_yield] = ACTIONS(2803),
    [anon_sym_move] = ACTIONS(2803),
    [anon_sym_try] = ACTIONS(2803),
    [sym_integer_literal] = ACTIONS(2801),
    [aux_sym_string_literal_token1] = ACTIONS(2801),
    [sym_char_literal] = ACTIONS(2801),
    [anon_sym_true] = ACTIONS(2803),
    [anon_sym_false] = ACTIONS(2803),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2803),
    [sym_super] = ACTIONS(2803),
    [sym_crate] = ACTIONS(2803),
    [sym_metavariable] = ACTIONS(2801),
    [sym__raw_string_literal_start] = ACTIONS(2801),
    [sym_float_literal] = ACTIONS(2801),
  },
  [723] = {
    [sym_line_comment] = STATE(723),
    [sym_block_comment] = STATE(723),
    [ts_builtin_sym_end] = ACTIONS(2805),
    [sym_identifier] = ACTIONS(2807),
    [anon_sym_SEMI] = ACTIONS(2805),
    [anon_sym_macro_rules_BANG] = ACTIONS(2805),
    [anon_sym_LPAREN] = ACTIONS(2805),
    [anon_sym_LBRACK] = ACTIONS(2805),
    [anon_sym_LBRACE] = ACTIONS(2805),
    [anon_sym_RBRACE] = ACTIONS(2805),
    [anon_sym_STAR] = ACTIONS(2805),
    [anon_sym_u8] = ACTIONS(2807),
    [anon_sym_i8] = ACTIONS(2807),
    [anon_sym_u16] = ACTIONS(2807),
    [anon_sym_i16] = ACTIONS(2807),
    [anon_sym_u32] = ACTIONS(2807),
    [anon_sym_i32] = ACTIONS(2807),
    [anon_sym_u64] = ACTIONS(2807),
    [anon_sym_i64] = ACTIONS(2807),
    [anon_sym_u128] = ACTIONS(2807),
    [anon_sym_i128] = ACTIONS(2807),
    [anon_sym_isize] = ACTIONS(2807),
    [anon_sym_usize] = ACTIONS(2807),
    [anon_sym_f32] = ACTIONS(2807),
    [anon_sym_f64] = ACTIONS(2807),
    [anon_sym_bool] = ACTIONS(2807),
    [anon_sym_str] = ACTIONS(2807),
    [anon_sym_char] = ACTIONS(2807),
    [anon_sym_DASH] = ACTIONS(2805),
    [anon_sym_BANG] = ACTIONS(2805),
    [anon_sym_AMP] = ACTIONS(2805),
    [anon_sym_PIPE] = ACTIONS(2805),
    [anon_sym_LT] = ACTIONS(2805),
    [anon_sym_DOT_DOT] = ACTIONS(2805),
    [anon_sym_COLON_COLON] = ACTIONS(2805),
    [anon_sym_POUND] = ACTIONS(2805),
    [anon_sym_SQUOTE] = ACTIONS(2807),
    [anon_sym_async] = ACTIONS(2807),
    [anon_sym_break] = ACTIONS(2807),
    [anon_sym_const] = ACTIONS(2807),
    [anon_sym_continue] = ACTIONS(2807),
    [anon_sym_default] = ACTIONS(2807),
    [anon_sym_enum] = ACTIONS(2807),
    [anon_sym_fn] = ACTIONS(2807),
    [anon_sym_for] = ACTIONS(2807),
    [anon_sym_gen] = ACTIONS(2807),
    [anon_sym_if] = ACTIONS(2807),
    [anon_sym_impl] = ACTIONS(2807),
    [anon_sym_let] = ACTIONS(2807),
    [anon_sym_loop] = ACTIONS(2807),
    [anon_sym_match] = ACTIONS(2807),
    [anon_sym_mod] = ACTIONS(2807),
    [anon_sym_pub] = ACTIONS(2807),
    [anon_sym_return] = ACTIONS(2807),
    [anon_sym_static] = ACTIONS(2807),
    [anon_sym_struct] = ACTIONS(2807),
    [anon_sym_trait] = ACTIONS(2807),
    [anon_sym_type] = ACTIONS(2807),
    [anon_sym_union] = ACTIONS(2807),
    [anon_sym_unsafe] = ACTIONS(2807),
    [anon_sym_use] = ACTIONS(2807),
    [anon_sym_while] = ACTIONS(2807),
    [anon_sym_extern] = ACTIONS(2807),
    [anon_sym_yield] = ACTIONS(2807),
    [anon_sym_move] = ACTIONS(2807),
    [anon_sym_try] = ACTIONS(2807),
    [sym_integer_literal] = ACTIONS(2805),
    [aux_sym_string_literal_token1] = ACTIONS(2805),
    [sym_char_literal] = ACTIONS(2805),
    [anon_sym_true] = ACTIONS(2807),
    [anon_sym_false] = ACTIONS(2807),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2807),
    [sym_super] = ACTIONS(2807),
    [sym_crate] = ACTIONS(2807),
    [sym_metavariable] = ACTIONS(2805),
    [sym__raw_string_literal_start] = ACTIONS(2805),
    [sym_float_literal] = ACTIONS(2805),
  },
  [724] = {
    [sym_line_comment] = STATE(724),
    [sym_block_comment] = STATE(724),
    [ts_builtin_sym_end] = ACTIONS(2809),
    [sym_identifier] = ACTIONS(2811),
    [anon_sym_SEMI] = ACTIONS(2809),
    [anon_sym_macro_rules_BANG] = ACTIONS(2809),
    [anon_sym_LPAREN] = ACTIONS(2809),
    [anon_sym_LBRACK] = ACTIONS(2809),
    [anon_sym_LBRACE] = ACTIONS(2809),
    [anon_sym_RBRACE] = ACTIONS(2809),
    [anon_sym_STAR] = ACTIONS(2809),
    [anon_sym_u8] = ACTIONS(2811),
    [anon_sym_i8] = ACTIONS(2811),
    [anon_sym_u16] = ACTIONS(2811),
    [anon_sym_i16] = ACTIONS(2811),
    [anon_sym_u32] = ACTIONS(2811),
    [anon_sym_i32] = ACTIONS(2811),
    [anon_sym_u64] = ACTIONS(2811),
    [anon_sym_i64] = ACTIONS(2811),
    [anon_sym_u128] = ACTIONS(2811),
    [anon_sym_i128] = ACTIONS(2811),
    [anon_sym_isize] = ACTIONS(2811),
    [anon_sym_usize] = ACTIONS(2811),
    [anon_sym_f32] = ACTIONS(2811),
    [anon_sym_f64] = ACTIONS(2811),
    [anon_sym_bool] = ACTIONS(2811),
    [anon_sym_str] = ACTIONS(2811),
    [anon_sym_char] = ACTIONS(2811),
    [anon_sym_DASH] = ACTIONS(2809),
    [anon_sym_BANG] = ACTIONS(2809),
    [anon_sym_AMP] = ACTIONS(2809),
    [anon_sym_PIPE] = ACTIONS(2809),
    [anon_sym_LT] = ACTIONS(2809),
    [anon_sym_DOT_DOT] = ACTIONS(2809),
    [anon_sym_COLON_COLON] = ACTIONS(2809),
    [anon_sym_POUND] = ACTIONS(2809),
    [anon_sym_SQUOTE] = ACTIONS(2811),
    [anon_sym_async] = ACTIONS(2811),
    [anon_sym_break] = ACTIONS(2811),
    [anon_sym_const] = ACTIONS(2811),
    [anon_sym_continue] = ACTIONS(2811),
    [anon_sym_default] = ACTIONS(2811),
    [anon_sym_enum] = ACTIONS(2811),
    [anon_sym_fn] = ACTIONS(2811),
    [anon_sym_for] = ACTIONS(2811),
    [anon_sym_gen] = ACTIONS(2811),
    [anon_sym_if] = ACTIONS(2811),
    [anon_sym_impl] = ACTIONS(2811),
    [anon_sym_let] = ACTIONS(2811),
    [anon_sym_loop] = ACTIONS(2811),
    [anon_sym_match] = ACTIONS(2811),
    [anon_sym_mod] = ACTIONS(2811),
    [anon_sym_pub] = ACTIONS(2811),
    [anon_sym_return] = ACTIONS(2811),
    [anon_sym_static] = ACTIONS(2811),
    [anon_sym_struct] = ACTIONS(2811),
    [anon_sym_trait] = ACTIONS(2811),
    [anon_sym_type] = ACTIONS(2811),
    [anon_sym_union] = ACTIONS(2811),
    [anon_sym_unsafe] = ACTIONS(2811),
    [anon_sym_use] = ACTIONS(2811),
    [anon_sym_while] = ACTIONS(2811),
    [anon_sym_extern] = ACTIONS(2811),
    [anon_sym_yield] = ACTIONS(2811),
    [anon_sym_move] = ACTIONS(2811),
    [anon_sym_try] = ACTIONS(2811),
    [sym_integer_literal] = ACTIONS(2809),
    [aux_sym_string_literal_token1] = ACTIONS(2809),
    [sym_char_literal] = ACTIONS(2809),
    [anon_sym_true] = ACTIONS(2811),
    [anon_sym_false] = ACTIONS(2811),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2811),
    [sym_super] = ACTIONS(2811),
    [sym_crate] = ACTIONS(2811),
    [sym_metavariable] = ACTIONS(2809),
    [sym__raw_string_literal_start] = ACTIONS(2809),
    [sym_float_literal] = ACTIONS(2809),
  },
  [725] = {
    [sym_line_comment] = STATE(725),
    [sym_block_comment] = STATE(725),
    [ts_builtin_sym_end] = ACTIONS(2813),
    [sym_identifier] = ACTIONS(2815),
    [anon_sym_SEMI] = ACTIONS(2813),
    [anon_sym_macro_rules_BANG] = ACTIONS(2813),
    [anon_sym_LPAREN] = ACTIONS(2813),
    [anon_sym_LBRACK] = ACTIONS(2813),
    [anon_sym_LBRACE] = ACTIONS(2813),
    [anon_sym_RBRACE] = ACTIONS(2813),
    [anon_sym_STAR] = ACTIONS(2813),
    [anon_sym_u8] = ACTIONS(2815),
    [anon_sym_i8] = ACTIONS(2815),
    [anon_sym_u16] = ACTIONS(2815),
    [anon_sym_i16] = ACTIONS(2815),
    [anon_sym_u32] = ACTIONS(2815),
    [anon_sym_i32] = ACTIONS(2815),
    [anon_sym_u64] = ACTIONS(2815),
    [anon_sym_i64] = ACTIONS(2815),
    [anon_sym_u128] = ACTIONS(2815),
    [anon_sym_i128] = ACTIONS(2815),
    [anon_sym_isize] = ACTIONS(2815),
    [anon_sym_usize] = ACTIONS(2815),
    [anon_sym_f32] = ACTIONS(2815),
    [anon_sym_f64] = ACTIONS(2815),
    [anon_sym_bool] = ACTIONS(2815),
    [anon_sym_str] = ACTIONS(2815),
    [anon_sym_char] = ACTIONS(2815),
    [anon_sym_DASH] = ACTIONS(2813),
    [anon_sym_BANG] = ACTIONS(2813),
    [anon_sym_AMP] = ACTIONS(2813),
    [anon_sym_PIPE] = ACTIONS(2813),
    [anon_sym_LT] = ACTIONS(2813),
    [anon_sym_DOT_DOT] = ACTIONS(2813),
    [anon_sym_COLON_COLON] = ACTIONS(2813),
    [anon_sym_POUND] = ACTIONS(2813),
    [anon_sym_SQUOTE] = ACTIONS(2815),
    [anon_sym_async] = ACTIONS(2815),
    [anon_sym_break] = ACTIONS(2815),
    [anon_sym_const] = ACTIONS(2815),
    [anon_sym_continue] = ACTIONS(2815),
    [anon_sym_default] = ACTIONS(2815),
    [anon_sym_enum] = ACTIONS(2815),
    [anon_sym_fn] = ACTIONS(2815),
    [anon_sym_for] = ACTIONS(2815),
    [anon_sym_gen] = ACTIONS(2815),
    [anon_sym_if] = ACTIONS(2815),
    [anon_sym_impl] = ACTIONS(2815),
    [anon_sym_let] = ACTIONS(2815),
    [anon_sym_loop] = ACTIONS(2815),
    [anon_sym_match] = ACTIONS(2815),
    [anon_sym_mod] = ACTIONS(2815),
    [anon_sym_pub] = ACTIONS(2815),
    [anon_sym_return] = ACTIONS(2815),
    [anon_sym_static] = ACTIONS(2815),
    [anon_sym_struct] = ACTIONS(2815),
    [anon_sym_trait] = ACTIONS(2815),
    [anon_sym_type] = ACTIONS(2815),
    [anon_sym_union] = ACTIONS(2815),
    [anon_sym_unsafe] = ACTIONS(2815),
    [anon_sym_use] = ACTIONS(2815),
    [anon_sym_while] = ACTIONS(2815),
    [anon_sym_extern] = ACTIONS(2815),
    [anon_sym_yield] = ACTIONS(2815),
    [anon_sym_move] = ACTIONS(2815),
    [anon_sym_try] = ACTIONS(2815),
    [sym_integer_literal] = ACTIONS(2813),
    [aux_sym_string_literal_token1] = ACTIONS(2813),
    [sym_char_literal] = ACTIONS(2813),
    [anon_sym_true] = ACTIONS(2815),
    [anon_sym_false] = ACTIONS(2815),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2815),
    [sym_super] = ACTIONS(2815),
    [sym_crate] = ACTIONS(2815),
    [sym_metavariable] = ACTIONS(2813),
    [sym__raw_string_literal_start] = ACTIONS(2813),
    [sym_float_literal] = ACTIONS(2813),
  },
  [726] = {
    [sym_line_comment] = STATE(726),
    [sym_block_comment] = STATE(726),
    [ts_builtin_sym_end] = ACTIONS(2817),
    [sym_identifier] = ACTIONS(2819),
    [anon_sym_SEMI] = ACTIONS(2817),
    [anon_sym_macro_rules_BANG] = ACTIONS(2817),
    [anon_sym_LPAREN] = ACTIONS(2817),
    [anon_sym_LBRACK] = ACTIONS(2817),
    [anon_sym_LBRACE] = ACTIONS(2817),
    [anon_sym_RBRACE] = ACTIONS(2817),
    [anon_sym_STAR] = ACTIONS(2817),
    [anon_sym_u8] = ACTIONS(2819),
    [anon_sym_i8] = ACTIONS(2819),
    [anon_sym_u16] = ACTIONS(2819),
    [anon_sym_i16] = ACTIONS(2819),
    [anon_sym_u32] = ACTIONS(2819),
    [anon_sym_i32] = ACTIONS(2819),
    [anon_sym_u64] = ACTIONS(2819),
    [anon_sym_i64] = ACTIONS(2819),
    [anon_sym_u128] = ACTIONS(2819),
    [anon_sym_i128] = ACTIONS(2819),
    [anon_sym_isize] = ACTIONS(2819),
    [anon_sym_usize] = ACTIONS(2819),
    [anon_sym_f32] = ACTIONS(2819),
    [anon_sym_f64] = ACTIONS(2819),
    [anon_sym_bool] = ACTIONS(2819),
    [anon_sym_str] = ACTIONS(2819),
    [anon_sym_char] = ACTIONS(2819),
    [anon_sym_DASH] = ACTIONS(2817),
    [anon_sym_BANG] = ACTIONS(2817),
    [anon_sym_AMP] = ACTIONS(2817),
    [anon_sym_PIPE] = ACTIONS(2817),
    [anon_sym_LT] = ACTIONS(2817),
    [anon_sym_DOT_DOT] = ACTIONS(2817),
    [anon_sym_COLON_COLON] = ACTIONS(2817),
    [anon_sym_POUND] = ACTIONS(2817),
    [anon_sym_SQUOTE] = ACTIONS(2819),
    [anon_sym_async] = ACTIONS(2819),
    [anon_sym_break] = ACTIONS(2819),
    [anon_sym_const] = ACTIONS(2819),
    [anon_sym_continue] = ACTIONS(2819),
    [anon_sym_default] = ACTIONS(2819),
    [anon_sym_enum] = ACTIONS(2819),
    [anon_sym_fn] = ACTIONS(2819),
    [anon_sym_for] = ACTIONS(2819),
    [anon_sym_gen] = ACTIONS(2819),
    [anon_sym_if] = ACTIONS(2819),
    [anon_sym_impl] = ACTIONS(2819),
    [anon_sym_let] = ACTIONS(2819),
    [anon_sym_loop] = ACTIONS(2819),
    [anon_sym_match] = ACTIONS(2819),
    [anon_sym_mod] = ACTIONS(2819),
    [anon_sym_pub] = ACTIONS(2819),
    [anon_sym_return] = ACTIONS(2819),
    [anon_sym_static] = ACTIONS(2819),
    [anon_sym_struct] = ACTIONS(2819),
    [anon_sym_trait] = ACTIONS(2819),
    [anon_sym_type] = ACTIONS(2819),
    [anon_sym_union] = ACTIONS(2819),
    [anon_sym_unsafe] = ACTIONS(2819),
    [anon_sym_use] = ACTIONS(2819),
    [anon_sym_while] = ACTIONS(2819),
    [anon_sym_extern] = ACTIONS(2819),
    [anon_sym_yield] = ACTIONS(2819),
    [anon_sym_move] = ACTIONS(2819),
    [anon_sym_try] = ACTIONS(2819),
    [sym_integer_literal] = ACTIONS(2817),
    [aux_sym_string_literal_token1] = ACTIONS(2817),
    [sym_char_literal] = ACTIONS(2817),
    [anon_sym_true] = ACTIONS(2819),
    [anon_sym_false] = ACTIONS(2819),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2819),
    [sym_super] = ACTIONS(2819),
    [sym_crate] = ACTIONS(2819),
    [sym_metavariable] = ACTIONS(2817),
    [sym__raw_string_literal_start] = ACTIONS(2817),
    [sym_float_literal] = ACTIONS(2817),
  },
  [727] = {
    [sym_line_comment] = STATE(727),
    [sym_block_comment] = STATE(727),
    [ts_builtin_sym_end] = ACTIONS(2821),
    [sym_identifier] = ACTIONS(2823),
    [anon_sym_SEMI] = ACTIONS(2821),
    [anon_sym_macro_rules_BANG] = ACTIONS(2821),
    [anon_sym_LPAREN] = ACTIONS(2821),
    [anon_sym_LBRACK] = ACTIONS(2821),
    [anon_sym_LBRACE] = ACTIONS(2821),
    [anon_sym_RBRACE] = ACTIONS(2821),
    [anon_sym_STAR] = ACTIONS(2821),
    [anon_sym_u8] = ACTIONS(2823),
    [anon_sym_i8] = ACTIONS(2823),
    [anon_sym_u16] = ACTIONS(2823),
    [anon_sym_i16] = ACTIONS(2823),
    [anon_sym_u32] = ACTIONS(2823),
    [anon_sym_i32] = ACTIONS(2823),
    [anon_sym_u64] = ACTIONS(2823),
    [anon_sym_i64] = ACTIONS(2823),
    [anon_sym_u128] = ACTIONS(2823),
    [anon_sym_i128] = ACTIONS(2823),
    [anon_sym_isize] = ACTIONS(2823),
    [anon_sym_usize] = ACTIONS(2823),
    [anon_sym_f32] = ACTIONS(2823),
    [anon_sym_f64] = ACTIONS(2823),
    [anon_sym_bool] = ACTIONS(2823),
    [anon_sym_str] = ACTIONS(2823),
    [anon_sym_char] = ACTIONS(2823),
    [anon_sym_DASH] = ACTIONS(2821),
    [anon_sym_BANG] = ACTIONS(2821),
    [anon_sym_AMP] = ACTIONS(2821),
    [anon_sym_PIPE] = ACTIONS(2821),
    [anon_sym_LT] = ACTIONS(2821),
    [anon_sym_DOT_DOT] = ACTIONS(2821),
    [anon_sym_COLON_COLON] = ACTIONS(2821),
    [anon_sym_POUND] = ACTIONS(2821),
    [anon_sym_SQUOTE] = ACTIONS(2823),
    [anon_sym_async] = ACTIONS(2823),
    [anon_sym_break] = ACTIONS(2823),
    [anon_sym_const] = ACTIONS(2823),
    [anon_sym_continue] = ACTIONS(2823),
    [anon_sym_default] = ACTIONS(2823),
    [anon_sym_enum] = ACTIONS(2823),
    [anon_sym_fn] = ACTIONS(2823),
    [anon_sym_for] = ACTIONS(2823),
    [anon_sym_gen] = ACTIONS(2823),
    [anon_sym_if] = ACTIONS(2823),
    [anon_sym_impl] = ACTIONS(2823),
    [anon_sym_let] = ACTIONS(2823),
    [anon_sym_loop] = ACTIONS(2823),
    [anon_sym_match] = ACTIONS(2823),
    [anon_sym_mod] = ACTIONS(2823),
    [anon_sym_pub] = ACTIONS(2823),
    [anon_sym_return] = ACTIONS(2823),
    [anon_sym_static] = ACTIONS(2823),
    [anon_sym_struct] = ACTIONS(2823),
    [anon_sym_trait] = ACTIONS(2823),
    [anon_sym_type] = ACTIONS(2823),
    [anon_sym_union] = ACTIONS(2823),
    [anon_sym_unsafe] = ACTIONS(2823),
    [anon_sym_use] = ACTIONS(2823),
    [anon_sym_while] = ACTIONS(2823),
    [anon_sym_extern] = ACTIONS(2823),
    [anon_sym_yield] = ACTIONS(2823),
    [anon_sym_move] = ACTIONS(2823),
    [anon_sym_try] = ACTIONS(2823),
    [sym_integer_literal] = ACTIONS(2821),
    [aux_sym_string_literal_token1] = ACTIONS(2821),
    [sym_char_literal] = ACTIONS(2821),
    [anon_sym_true] = ACTIONS(2823),
    [anon_sym_false] = ACTIONS(2823),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2823),
    [sym_super] = ACTIONS(2823),
    [sym_crate] = ACTIONS(2823),
    [sym_metavariable] = ACTIONS(2821),
    [sym__raw_string_literal_start] = ACTIONS(2821),
    [sym_float_literal] = ACTIONS(2821),
  },
  [728] = {
    [sym_line_comment] = STATE(728),
    [sym_block_comment] = STATE(728),
    [ts_builtin_sym_end] = ACTIONS(2825),
    [sym_identifier] = ACTIONS(2827),
    [anon_sym_SEMI] = ACTIONS(2825),
    [anon_sym_macro_rules_BANG] = ACTIONS(2825),
    [anon_sym_LPAREN] = ACTIONS(2825),
    [anon_sym_LBRACK] = ACTIONS(2825),
    [anon_sym_LBRACE] = ACTIONS(2825),
    [anon_sym_RBRACE] = ACTIONS(2825),
    [anon_sym_STAR] = ACTIONS(2825),
    [anon_sym_u8] = ACTIONS(2827),
    [anon_sym_i8] = ACTIONS(2827),
    [anon_sym_u16] = ACTIONS(2827),
    [anon_sym_i16] = ACTIONS(2827),
    [anon_sym_u32] = ACTIONS(2827),
    [anon_sym_i32] = ACTIONS(2827),
    [anon_sym_u64] = ACTIONS(2827),
    [anon_sym_i64] = ACTIONS(2827),
    [anon_sym_u128] = ACTIONS(2827),
    [anon_sym_i128] = ACTIONS(2827),
    [anon_sym_isize] = ACTIONS(2827),
    [anon_sym_usize] = ACTIONS(2827),
    [anon_sym_f32] = ACTIONS(2827),
    [anon_sym_f64] = ACTIONS(2827),
    [anon_sym_bool] = ACTIONS(2827),
    [anon_sym_str] = ACTIONS(2827),
    [anon_sym_char] = ACTIONS(2827),
    [anon_sym_DASH] = ACTIONS(2825),
    [anon_sym_BANG] = ACTIONS(2825),
    [anon_sym_AMP] = ACTIONS(2825),
    [anon_sym_PIPE] = ACTIONS(2825),
    [anon_sym_LT] = ACTIONS(2825),
    [anon_sym_DOT_DOT] = ACTIONS(2825),
    [anon_sym_COLON_COLON] = ACTIONS(2825),
    [anon_sym_POUND] = ACTIONS(2825),
    [anon_sym_SQUOTE] = ACTIONS(2827),
    [anon_sym_async] = ACTIONS(2827),
    [anon_sym_break] = ACTIONS(2827),
    [anon_sym_const] = ACTIONS(2827),
    [anon_sym_continue] = ACTIONS(2827),
    [anon_sym_default] = ACTIONS(2827),
    [anon_sym_enum] = ACTIONS(2827),
    [anon_sym_fn] = ACTIONS(2827),
    [anon_sym_for] = ACTIONS(2827),
    [anon_sym_gen] = ACTIONS(2827),
    [anon_sym_if] = ACTIONS(2827),
    [anon_sym_impl] = ACTIONS(2827),
    [anon_sym_let] = ACTIONS(2827),
    [anon_sym_loop] = ACTIONS(2827),
    [anon_sym_match] = ACTIONS(2827),
    [anon_sym_mod] = ACTIONS(2827),
    [anon_sym_pub] = ACTIONS(2827),
    [anon_sym_return] = ACTIONS(2827),
    [anon_sym_static] = ACTIONS(2827),
    [anon_sym_struct] = ACTIONS(2827),
    [anon_sym_trait] = ACTIONS(2827),
    [anon_sym_type] = ACTIONS(2827),
    [anon_sym_union] = ACTIONS(2827),
    [anon_sym_unsafe] = ACTIONS(2827),
    [anon_sym_use] = ACTIONS(2827),
    [anon_sym_while] = ACTIONS(2827),
    [anon_sym_extern] = ACTIONS(2827),
    [anon_sym_yield] = ACTIONS(2827),
    [anon_sym_move] = ACTIONS(2827),
    [anon_sym_try] = ACTIONS(2827),
    [sym_integer_literal] = ACTIONS(2825),
    [aux_sym_string_literal_token1] = ACTIONS(2825),
    [sym_char_literal] = ACTIONS(2825),
    [anon_sym_true] = ACTIONS(2827),
    [anon_sym_false] = ACTIONS(2827),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2827),
    [sym_super] = ACTIONS(2827),
    [sym_crate] = ACTIONS(2827),
    [sym_metavariable] = ACTIONS(2825),
    [sym__raw_string_literal_start] = ACTIONS(2825),
    [sym_float_literal] = ACTIONS(2825),
  },
  [729] = {
    [sym_line_comment] = STATE(729),
    [sym_block_comment] = STATE(729),
    [ts_builtin_sym_end] = ACTIONS(2829),
    [sym_identifier] = ACTIONS(2831),
    [anon_sym_SEMI] = ACTIONS(2829),
    [anon_sym_macro_rules_BANG] = ACTIONS(2829),
    [anon_sym_LPAREN] = ACTIONS(2829),
    [anon_sym_LBRACK] = ACTIONS(2829),
    [anon_sym_LBRACE] = ACTIONS(2829),
    [anon_sym_RBRACE] = ACTIONS(2829),
    [anon_sym_STAR] = ACTIONS(2829),
    [anon_sym_u8] = ACTIONS(2831),
    [anon_sym_i8] = ACTIONS(2831),
    [anon_sym_u16] = ACTIONS(2831),
    [anon_sym_i16] = ACTIONS(2831),
    [anon_sym_u32] = ACTIONS(2831),
    [anon_sym_i32] = ACTIONS(2831),
    [anon_sym_u64] = ACTIONS(2831),
    [anon_sym_i64] = ACTIONS(2831),
    [anon_sym_u128] = ACTIONS(2831),
    [anon_sym_i128] = ACTIONS(2831),
    [anon_sym_isize] = ACTIONS(2831),
    [anon_sym_usize] = ACTIONS(2831),
    [anon_sym_f32] = ACTIONS(2831),
    [anon_sym_f64] = ACTIONS(2831),
    [anon_sym_bool] = ACTIONS(2831),
    [anon_sym_str] = ACTIONS(2831),
    [anon_sym_char] = ACTIONS(2831),
    [anon_sym_DASH] = ACTIONS(2829),
    [anon_sym_BANG] = ACTIONS(2829),
    [anon_sym_AMP] = ACTIONS(2829),
    [anon_sym_PIPE] = ACTIONS(2829),
    [anon_sym_LT] = ACTIONS(2829),
    [anon_sym_DOT_DOT] = ACTIONS(2829),
    [anon_sym_COLON_COLON] = ACTIONS(2829),
    [anon_sym_POUND] = ACTIONS(2829),
    [anon_sym_SQUOTE] = ACTIONS(2831),
    [anon_sym_async] = ACTIONS(2831),
    [anon_sym_break] = ACTIONS(2831),
    [anon_sym_const] = ACTIONS(2831),
    [anon_sym_continue] = ACTIONS(2831),
    [anon_sym_default] = ACTIONS(2831),
    [anon_sym_enum] = ACTIONS(2831),
    [anon_sym_fn] = ACTIONS(2831),
    [anon_sym_for] = ACTIONS(2831),
    [anon_sym_gen] = ACTIONS(2831),
    [anon_sym_if] = ACTIONS(2831),
    [anon_sym_impl] = ACTIONS(2831),
    [anon_sym_let] = ACTIONS(2831),
    [anon_sym_loop] = ACTIONS(2831),
    [anon_sym_match] = ACTIONS(2831),
    [anon_sym_mod] = ACTIONS(2831),
    [anon_sym_pub] = ACTIONS(2831),
    [anon_sym_return] = ACTIONS(2831),
    [anon_sym_static] = ACTIONS(2831),
    [anon_sym_struct] = ACTIONS(2831),
    [anon_sym_trait] = ACTIONS(2831),
    [anon_sym_type] = ACTIONS(2831),
    [anon_sym_union] = ACTIONS(2831),
    [anon_sym_unsafe] = ACTIONS(2831),
    [anon_sym_use] = ACTIONS(2831),
    [anon_sym_while] = ACTIONS(2831),
    [anon_sym_extern] = ACTIONS(2831),
    [anon_sym_yield] = ACTIONS(2831),
    [anon_sym_move] = ACTIONS(2831),
    [anon_sym_try] = ACTIONS(2831),
    [sym_integer_literal] = ACTIONS(2829),
    [aux_sym_string_literal_token1] = ACTIONS(2829),
    [sym_char_literal] = ACTIONS(2829),
    [anon_sym_true] = ACTIONS(2831),
    [anon_sym_false] = ACTIONS(2831),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2831),
    [sym_super] = ACTIONS(2831),
    [sym_crate] = ACTIONS(2831),
    [sym_metavariable] = ACTIONS(2829),
    [sym__raw_string_literal_start] = ACTIONS(2829),
    [sym_float_literal] = ACTIONS(2829),
  },
  [730] = {
    [sym_line_comment] = STATE(730),
    [sym_block_comment] = STATE(730),
    [ts_builtin_sym_end] = ACTIONS(2833),
    [sym_identifier] = ACTIONS(2835),
    [anon_sym_SEMI] = ACTIONS(2833),
    [anon_sym_macro_rules_BANG] = ACTIONS(2833),
    [anon_sym_LPAREN] = ACTIONS(2833),
    [anon_sym_LBRACK] = ACTIONS(2833),
    [anon_sym_LBRACE] = ACTIONS(2833),
    [anon_sym_RBRACE] = ACTIONS(2833),
    [anon_sym_STAR] = ACTIONS(2833),
    [anon_sym_u8] = ACTIONS(2835),
    [anon_sym_i8] = ACTIONS(2835),
    [anon_sym_u16] = ACTIONS(2835),
    [anon_sym_i16] = ACTIONS(2835),
    [anon_sym_u32] = ACTIONS(2835),
    [anon_sym_i32] = ACTIONS(2835),
    [anon_sym_u64] = ACTIONS(2835),
    [anon_sym_i64] = ACTIONS(2835),
    [anon_sym_u128] = ACTIONS(2835),
    [anon_sym_i128] = ACTIONS(2835),
    [anon_sym_isize] = ACTIONS(2835),
    [anon_sym_usize] = ACTIONS(2835),
    [anon_sym_f32] = ACTIONS(2835),
    [anon_sym_f64] = ACTIONS(2835),
    [anon_sym_bool] = ACTIONS(2835),
    [anon_sym_str] = ACTIONS(2835),
    [anon_sym_char] = ACTIONS(2835),
    [anon_sym_DASH] = ACTIONS(2833),
    [anon_sym_BANG] = ACTIONS(2833),
    [anon_sym_AMP] = ACTIONS(2833),
    [anon_sym_PIPE] = ACTIONS(2833),
    [anon_sym_LT] = ACTIONS(2833),
    [anon_sym_DOT_DOT] = ACTIONS(2833),
    [anon_sym_COLON_COLON] = ACTIONS(2833),
    [anon_sym_POUND] = ACTIONS(2833),
    [anon_sym_SQUOTE] = ACTIONS(2835),
    [anon_sym_async] = ACTIONS(2835),
    [anon_sym_break] = ACTIONS(2835),
    [anon_sym_const] = ACTIONS(2835),
    [anon_sym_continue] = ACTIONS(2835),
    [anon_sym_default] = ACTIONS(2835),
    [anon_sym_enum] = ACTIONS(2835),
    [anon_sym_fn] = ACTIONS(2835),
    [anon_sym_for] = ACTIONS(2835),
    [anon_sym_gen] = ACTIONS(2835),
    [anon_sym_if] = ACTIONS(2835),
    [anon_sym_impl] = ACTIONS(2835),
    [anon_sym_let] = ACTIONS(2835),
    [anon_sym_loop] = ACTIONS(2835),
    [anon_sym_match] = ACTIONS(2835),
    [anon_sym_mod] = ACTIONS(2835),
    [anon_sym_pub] = ACTIONS(2835),
    [anon_sym_return] = ACTIONS(2835),
    [anon_sym_static] = ACTIONS(2835),
    [anon_sym_struct] = ACTIONS(2835),
    [anon_sym_trait] = ACTIONS(2835),
    [anon_sym_type] = ACTIONS(2835),
    [anon_sym_union] = ACTIONS(2835),
    [anon_sym_unsafe] = ACTIONS(2835),
    [anon_sym_use] = ACTIONS(2835),
    [anon_sym_while] = ACTIONS(2835),
    [anon_sym_extern] = ACTIONS(2835),
    [anon_sym_yield] = ACTIONS(2835),
    [anon_sym_move] = ACTIONS(2835),
    [anon_sym_try] = ACTIONS(2835),
    [sym_integer_literal] = ACTIONS(2833),
    [aux_sym_string_literal_token1] = ACTIONS(2833),
    [sym_char_literal] = ACTIONS(2833),
    [anon_sym_true] = ACTIONS(2835),
    [anon_sym_false] = ACTIONS(2835),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2835),
    [sym_super] = ACTIONS(2835),
    [sym_crate] = ACTIONS(2835),
    [sym_metavariable] = ACTIONS(2833),
    [sym__raw_string_literal_start] = ACTIONS(2833),
    [sym_float_literal] = ACTIONS(2833),
  },
  [731] = {
    [sym_line_comment] = STATE(731),
    [sym_block_comment] = STATE(731),
    [ts_builtin_sym_end] = ACTIONS(2837),
    [sym_identifier] = ACTIONS(2839),
    [anon_sym_SEMI] = ACTIONS(2837),
    [anon_sym_macro_rules_BANG] = ACTIONS(2837),
    [anon_sym_LPAREN] = ACTIONS(2837),
    [anon_sym_LBRACK] = ACTIONS(2837),
    [anon_sym_LBRACE] = ACTIONS(2837),
    [anon_sym_RBRACE] = ACTIONS(2837),
    [anon_sym_STAR] = ACTIONS(2837),
    [anon_sym_u8] = ACTIONS(2839),
    [anon_sym_i8] = ACTIONS(2839),
    [anon_sym_u16] = ACTIONS(2839),
    [anon_sym_i16] = ACTIONS(2839),
    [anon_sym_u32] = ACTIONS(2839),
    [anon_sym_i32] = ACTIONS(2839),
    [anon_sym_u64] = ACTIONS(2839),
    [anon_sym_i64] = ACTIONS(2839),
    [anon_sym_u128] = ACTIONS(2839),
    [anon_sym_i128] = ACTIONS(2839),
    [anon_sym_isize] = ACTIONS(2839),
    [anon_sym_usize] = ACTIONS(2839),
    [anon_sym_f32] = ACTIONS(2839),
    [anon_sym_f64] = ACTIONS(2839),
    [anon_sym_bool] = ACTIONS(2839),
    [anon_sym_str] = ACTIONS(2839),
    [anon_sym_char] = ACTIONS(2839),
    [anon_sym_DASH] = ACTIONS(2837),
    [anon_sym_BANG] = ACTIONS(2837),
    [anon_sym_AMP] = ACTIONS(2837),
    [anon_sym_PIPE] = ACTIONS(2837),
    [anon_sym_LT] = ACTIONS(2837),
    [anon_sym_DOT_DOT] = ACTIONS(2837),
    [anon_sym_COLON_COLON] = ACTIONS(2837),
    [anon_sym_POUND] = ACTIONS(2837),
    [anon_sym_SQUOTE] = ACTIONS(2839),
    [anon_sym_async] = ACTIONS(2839),
    [anon_sym_break] = ACTIONS(2839),
    [anon_sym_const] = ACTIONS(2839),
    [anon_sym_continue] = ACTIONS(2839),
    [anon_sym_default] = ACTIONS(2839),
    [anon_sym_enum] = ACTIONS(2839),
    [anon_sym_fn] = ACTIONS(2839),
    [anon_sym_for] = ACTIONS(2839),
    [anon_sym_gen] = ACTIONS(2839),
    [anon_sym_if] = ACTIONS(2839),
    [anon_sym_impl] = ACTIONS(2839),
    [anon_sym_let] = ACTIONS(2839),
    [anon_sym_loop] = ACTIONS(2839),
    [anon_sym_match] = ACTIONS(2839),
    [anon_sym_mod] = ACTIONS(2839),
    [anon_sym_pub] = ACTIONS(2839),
    [anon_sym_return] = ACTIONS(2839),
    [anon_sym_static] = ACTIONS(2839),
    [anon_sym_struct] = ACTIONS(2839),
    [anon_sym_trait] = ACTIONS(2839),
    [anon_sym_type] = ACTIONS(2839),
    [anon_sym_union] = ACTIONS(2839),
    [anon_sym_unsafe] = ACTIONS(2839),
    [anon_sym_use] = ACTIONS(2839),
    [anon_sym_while] = ACTIONS(2839),
    [anon_sym_extern] = ACTIONS(2839),
    [anon_sym_yield] = ACTIONS(2839),
    [anon_sym_move] = ACTIONS(2839),
    [anon_sym_try] = ACTIONS(2839),
    [sym_integer_literal] = ACTIONS(2837),
    [aux_sym_string_literal_token1] = ACTIONS(2837),
    [sym_char_literal] = ACTIONS(2837),
    [anon_sym_true] = ACTIONS(2839),
    [anon_sym_false] = ACTIONS(2839),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2839),
    [sym_super] = ACTIONS(2839),
    [sym_crate] = ACTIONS(2839),
    [sym_metavariable] = ACTIONS(2837),
    [sym__raw_string_literal_start] = ACTIONS(2837),
    [sym_float_literal] = ACTIONS(2837),
  },
  [732] = {
    [sym_line_comment] = STATE(732),
    [sym_block_comment] = STATE(732),
    [ts_builtin_sym_end] = ACTIONS(2841),
    [sym_identifier] = ACTIONS(2843),
    [anon_sym_SEMI] = ACTIONS(2841),
    [anon_sym_macro_rules_BANG] = ACTIONS(2841),
    [anon_sym_LPAREN] = ACTIONS(2841),
    [anon_sym_LBRACK] = ACTIONS(2841),
    [anon_sym_LBRACE] = ACTIONS(2841),
    [anon_sym_RBRACE] = ACTIONS(2841),
    [anon_sym_STAR] = ACTIONS(2841),
    [anon_sym_u8] = ACTIONS(2843),
    [anon_sym_i8] = ACTIONS(2843),
    [anon_sym_u16] = ACTIONS(2843),
    [anon_sym_i16] = ACTIONS(2843),
    [anon_sym_u32] = ACTIONS(2843),
    [anon_sym_i32] = ACTIONS(2843),
    [anon_sym_u64] = ACTIONS(2843),
    [anon_sym_i64] = ACTIONS(2843),
    [anon_sym_u128] = ACTIONS(2843),
    [anon_sym_i128] = ACTIONS(2843),
    [anon_sym_isize] = ACTIONS(2843),
    [anon_sym_usize] = ACTIONS(2843),
    [anon_sym_f32] = ACTIONS(2843),
    [anon_sym_f64] = ACTIONS(2843),
    [anon_sym_bool] = ACTIONS(2843),
    [anon_sym_str] = ACTIONS(2843),
    [anon_sym_char] = ACTIONS(2843),
    [anon_sym_DASH] = ACTIONS(2841),
    [anon_sym_BANG] = ACTIONS(2841),
    [anon_sym_AMP] = ACTIONS(2841),
    [anon_sym_PIPE] = ACTIONS(2841),
    [anon_sym_LT] = ACTIONS(2841),
    [anon_sym_DOT_DOT] = ACTIONS(2841),
    [anon_sym_COLON_COLON] = ACTIONS(2841),
    [anon_sym_POUND] = ACTIONS(2841),
    [anon_sym_SQUOTE] = ACTIONS(2843),
    [anon_sym_async] = ACTIONS(2843),
    [anon_sym_break] = ACTIONS(2843),
    [anon_sym_const] = ACTIONS(2843),
    [anon_sym_continue] = ACTIONS(2843),
    [anon_sym_default] = ACTIONS(2843),
    [anon_sym_enum] = ACTIONS(2843),
    [anon_sym_fn] = ACTIONS(2843),
    [anon_sym_for] = ACTIONS(2843),
    [anon_sym_gen] = ACTIONS(2843),
    [anon_sym_if] = ACTIONS(2843),
    [anon_sym_impl] = ACTIONS(2843),
    [anon_sym_let] = ACTIONS(2843),
    [anon_sym_loop] = ACTIONS(2843),
    [anon_sym_match] = ACTIONS(2843),
    [anon_sym_mod] = ACTIONS(2843),
    [anon_sym_pub] = ACTIONS(2843),
    [anon_sym_return] = ACTIONS(2843),
    [anon_sym_static] = ACTIONS(2843),
    [anon_sym_struct] = ACTIONS(2843),
    [anon_sym_trait] = ACTIONS(2843),
    [anon_sym_type] = ACTIONS(2843),
    [anon_sym_union] = ACTIONS(2843),
    [anon_sym_unsafe] = ACTIONS(2843),
    [anon_sym_use] = ACTIONS(2843),
    [anon_sym_while] = ACTIONS(2843),
    [anon_sym_extern] = ACTIONS(2843),
    [anon_sym_yield] = ACTIONS(2843),
    [anon_sym_move] = ACTIONS(2843),
    [anon_sym_try] = ACTIONS(2843),
    [sym_integer_literal] = ACTIONS(2841),
    [aux_sym_string_literal_token1] = ACTIONS(2841),
    [sym_char_literal] = ACTIONS(2841),
    [anon_sym_true] = ACTIONS(2843),
    [anon_sym_false] = ACTIONS(2843),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2843),
    [sym_super] = ACTIONS(2843),
    [sym_crate] = ACTIONS(2843),
    [sym_metavariable] = ACTIONS(2841),
    [sym__raw_string_literal_start] = ACTIONS(2841),
    [sym_float_literal] = ACTIONS(2841),
  },
  [733] = {
    [sym_line_comment] = STATE(733),
    [sym_block_comment] = STATE(733),
    [ts_builtin_sym_end] = ACTIONS(2845),
    [sym_identifier] = ACTIONS(2847),
    [anon_sym_SEMI] = ACTIONS(2845),
    [anon_sym_macro_rules_BANG] = ACTIONS(2845),
    [anon_sym_LPAREN] = ACTIONS(2845),
    [anon_sym_LBRACK] = ACTIONS(2845),
    [anon_sym_LBRACE] = ACTIONS(2845),
    [anon_sym_RBRACE] = ACTIONS(2845),
    [anon_sym_STAR] = ACTIONS(2845),
    [anon_sym_u8] = ACTIONS(2847),
    [anon_sym_i8] = ACTIONS(2847),
    [anon_sym_u16] = ACTIONS(2847),
    [anon_sym_i16] = ACTIONS(2847),
    [anon_sym_u32] = ACTIONS(2847),
    [anon_sym_i32] = ACTIONS(2847),
    [anon_sym_u64] = ACTIONS(2847),
    [anon_sym_i64] = ACTIONS(2847),
    [anon_sym_u128] = ACTIONS(2847),
    [anon_sym_i128] = ACTIONS(2847),
    [anon_sym_isize] = ACTIONS(2847),
    [anon_sym_usize] = ACTIONS(2847),
    [anon_sym_f32] = ACTIONS(2847),
    [anon_sym_f64] = ACTIONS(2847),
    [anon_sym_bool] = ACTIONS(2847),
    [anon_sym_str] = ACTIONS(2847),
    [anon_sym_char] = ACTIONS(2847),
    [anon_sym_DASH] = ACTIONS(2845),
    [anon_sym_BANG] = ACTIONS(2845),
    [anon_sym_AMP] = ACTIONS(2845),
    [anon_sym_PIPE] = ACTIONS(2845),
    [anon_sym_LT] = ACTIONS(2845),
    [anon_sym_DOT_DOT] = ACTIONS(2845),
    [anon_sym_COLON_COLON] = ACTIONS(2845),
    [anon_sym_POUND] = ACTIONS(2845),
    [anon_sym_SQUOTE] = ACTIONS(2847),
    [anon_sym_async] = ACTIONS(2847),
    [anon_sym_break] = ACTIONS(2847),
    [anon_sym_const] = ACTIONS(2847),
    [anon_sym_continue] = ACTIONS(2847),
    [anon_sym_default] = ACTIONS(2847),
    [anon_sym_enum] = ACTIONS(2847),
    [anon_sym_fn] = ACTIONS(2847),
    [anon_sym_for] = ACTIONS(2847),
    [anon_sym_gen] = ACTIONS(2847),
    [anon_sym_if] = ACTIONS(2847),
    [anon_sym_impl] = ACTIONS(2847),
    [anon_sym_let] = ACTIONS(2847),
    [anon_sym_loop] = ACTIONS(2847),
    [anon_sym_match] = ACTIONS(2847),
    [anon_sym_mod] = ACTIONS(2847),
    [anon_sym_pub] = ACTIONS(2847),
    [anon_sym_return] = ACTIONS(2847),
    [anon_sym_static] = ACTIONS(2847),
    [anon_sym_struct] = ACTIONS(2847),
    [anon_sym_trait] = ACTIONS(2847),
    [anon_sym_type] = ACTIONS(2847),
    [anon_sym_union] = ACTIONS(2847),
    [anon_sym_unsafe] = ACTIONS(2847),
    [anon_sym_use] = ACTIONS(2847),
    [anon_sym_while] = ACTIONS(2847),
    [anon_sym_extern] = ACTIONS(2847),
    [anon_sym_yield] = ACTIONS(2847),
    [anon_sym_move] = ACTIONS(2847),
    [anon_sym_try] = ACTIONS(2847),
    [sym_integer_literal] = ACTIONS(2845),
    [aux_sym_string_literal_token1] = ACTIONS(2845),
    [sym_char_literal] = ACTIONS(2845),
    [anon_sym_true] = ACTIONS(2847),
    [anon_sym_false] = ACTIONS(2847),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2847),
    [sym_super] = ACTIONS(2847),
    [sym_crate] = ACTIONS(2847),
    [sym_metavariable] = ACTIONS(2845),
    [sym__raw_string_literal_start] = ACTIONS(2845),
    [sym_float_literal] = ACTIONS(2845),
  },
  [734] = {
    [sym_line_comment] = STATE(734),
    [sym_block_comment] = STATE(734),
    [ts_builtin_sym_end] = ACTIONS(2849),
    [sym_identifier] = ACTIONS(2851),
    [anon_sym_SEMI] = ACTIONS(2849),
    [anon_sym_macro_rules_BANG] = ACTIONS(2849),
    [anon_sym_LPAREN] = ACTIONS(2849),
    [anon_sym_LBRACK] = ACTIONS(2849),
    [anon_sym_LBRACE] = ACTIONS(2849),
    [anon_sym_RBRACE] = ACTIONS(2849),
    [anon_sym_STAR] = ACTIONS(2849),
    [anon_sym_u8] = ACTIONS(2851),
    [anon_sym_i8] = ACTIONS(2851),
    [anon_sym_u16] = ACTIONS(2851),
    [anon_sym_i16] = ACTIONS(2851),
    [anon_sym_u32] = ACTIONS(2851),
    [anon_sym_i32] = ACTIONS(2851),
    [anon_sym_u64] = ACTIONS(2851),
    [anon_sym_i64] = ACTIONS(2851),
    [anon_sym_u128] = ACTIONS(2851),
    [anon_sym_i128] = ACTIONS(2851),
    [anon_sym_isize] = ACTIONS(2851),
    [anon_sym_usize] = ACTIONS(2851),
    [anon_sym_f32] = ACTIONS(2851),
    [anon_sym_f64] = ACTIONS(2851),
    [anon_sym_bool] = ACTIONS(2851),
    [anon_sym_str] = ACTIONS(2851),
    [anon_sym_char] = ACTIONS(2851),
    [anon_sym_DASH] = ACTIONS(2849),
    [anon_sym_BANG] = ACTIONS(2849),
    [anon_sym_AMP] = ACTIONS(2849),
    [anon_sym_PIPE] = ACTIONS(2849),
    [anon_sym_LT] = ACTIONS(2849),
    [anon_sym_DOT_DOT] = ACTIONS(2849),
    [anon_sym_COLON_COLON] = ACTIONS(2849),
    [anon_sym_POUND] = ACTIONS(2849),
    [anon_sym_SQUOTE] = ACTIONS(2851),
    [anon_sym_async] = ACTIONS(2851),
    [anon_sym_break] = ACTIONS(2851),
    [anon_sym_const] = ACTIONS(2851),
    [anon_sym_continue] = ACTIONS(2851),
    [anon_sym_default] = ACTIONS(2851),
    [anon_sym_enum] = ACTIONS(2851),
    [anon_sym_fn] = ACTIONS(2851),
    [anon_sym_for] = ACTIONS(2851),
    [anon_sym_gen] = ACTIONS(2851),
    [anon_sym_if] = ACTIONS(2851),
    [anon_sym_impl] = ACTIONS(2851),
    [anon_sym_let] = ACTIONS(2851),
    [anon_sym_loop] = ACTIONS(2851),
    [anon_sym_match] = ACTIONS(2851),
    [anon_sym_mod] = ACTIONS(2851),
    [anon_sym_pub] = ACTIONS(2851),
    [anon_sym_return] = ACTIONS(2851),
    [anon_sym_static] = ACTIONS(2851),
    [anon_sym_struct] = ACTIONS(2851),
    [anon_sym_trait] = ACTIONS(2851),
    [anon_sym_type] = ACTIONS(2851),
    [anon_sym_union] = ACTIONS(2851),
    [anon_sym_unsafe] = ACTIONS(2851),
    [anon_sym_use] = ACTIONS(2851),
    [anon_sym_while] = ACTIONS(2851),
    [anon_sym_extern] = ACTIONS(2851),
    [anon_sym_yield] = ACTIONS(2851),
    [anon_sym_move] = ACTIONS(2851),
    [anon_sym_try] = ACTIONS(2851),
    [sym_integer_literal] = ACTIONS(2849),
    [aux_sym_string_literal_token1] = ACTIONS(2849),
    [sym_char_literal] = ACTIONS(2849),
    [anon_sym_true] = ACTIONS(2851),
    [anon_sym_false] = ACTIONS(2851),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2851),
    [sym_super] = ACTIONS(2851),
    [sym_crate] = ACTIONS(2851),
    [sym_metavariable] = ACTIONS(2849),
    [sym__raw_string_literal_start] = ACTIONS(2849),
    [sym_float_literal] = ACTIONS(2849),
  },
  [735] = {
    [sym_line_comment] = STATE(735),
    [sym_block_comment] = STATE(735),
    [ts_builtin_sym_end] = ACTIONS(2853),
    [sym_identifier] = ACTIONS(2855),
    [anon_sym_SEMI] = ACTIONS(2853),
    [anon_sym_macro_rules_BANG] = ACTIONS(2853),
    [anon_sym_LPAREN] = ACTIONS(2853),
    [anon_sym_LBRACK] = ACTIONS(2853),
    [anon_sym_LBRACE] = ACTIONS(2853),
    [anon_sym_RBRACE] = ACTIONS(2853),
    [anon_sym_STAR] = ACTIONS(2853),
    [anon_sym_u8] = ACTIONS(2855),
    [anon_sym_i8] = ACTIONS(2855),
    [anon_sym_u16] = ACTIONS(2855),
    [anon_sym_i16] = ACTIONS(2855),
    [anon_sym_u32] = ACTIONS(2855),
    [anon_sym_i32] = ACTIONS(2855),
    [anon_sym_u64] = ACTIONS(2855),
    [anon_sym_i64] = ACTIONS(2855),
    [anon_sym_u128] = ACTIONS(2855),
    [anon_sym_i128] = ACTIONS(2855),
    [anon_sym_isize] = ACTIONS(2855),
    [anon_sym_usize] = ACTIONS(2855),
    [anon_sym_f32] = ACTIONS(2855),
    [anon_sym_f64] = ACTIONS(2855),
    [anon_sym_bool] = ACTIONS(2855),
    [anon_sym_str] = ACTIONS(2855),
    [anon_sym_char] = ACTIONS(2855),
    [anon_sym_DASH] = ACTIONS(2853),
    [anon_sym_BANG] = ACTIONS(2853),
    [anon_sym_AMP] = ACTIONS(2853),
    [anon_sym_PIPE] = ACTIONS(2853),
    [anon_sym_LT] = ACTIONS(2853),
    [anon_sym_DOT_DOT] = ACTIONS(2853),
    [anon_sym_COLON_COLON] = ACTIONS(2853),
    [anon_sym_POUND] = ACTIONS(2853),
    [anon_sym_SQUOTE] = ACTIONS(2855),
    [anon_sym_async] = ACTIONS(2855),
    [anon_sym_break] = ACTIONS(2855),
    [anon_sym_const] = ACTIONS(2855),
    [anon_sym_continue] = ACTIONS(2855),
    [anon_sym_default] = ACTIONS(2855),
    [anon_sym_enum] = ACTIONS(2855),
    [anon_sym_fn] = ACTIONS(2855),
    [anon_sym_for] = ACTIONS(2855),
    [anon_sym_gen] = ACTIONS(2855),
    [anon_sym_if] = ACTIONS(2855),
    [anon_sym_impl] = ACTIONS(2855),
    [anon_sym_let] = ACTIONS(2855),
    [anon_sym_loop] = ACTIONS(2855),
    [anon_sym_match] = ACTIONS(2855),
    [anon_sym_mod] = ACTIONS(2855),
    [anon_sym_pub] = ACTIONS(2855),
    [anon_sym_return] = ACTIONS(2855),
    [anon_sym_static] = ACTIONS(2855),
    [anon_sym_struct] = ACTIONS(2855),
    [anon_sym_trait] = ACTIONS(2855),
    [anon_sym_type] = ACTIONS(2855),
    [anon_sym_union] = ACTIONS(2855),
    [anon_sym_unsafe] = ACTIONS(2855),
    [anon_sym_use] = ACTIONS(2855),
    [anon_sym_while] = ACTIONS(2855),
    [anon_sym_extern] = ACTIONS(2855),
    [anon_sym_yield] = ACTIONS(2855),
    [anon_sym_move] = ACTIONS(2855),
    [anon_sym_try] = ACTIONS(2855),
    [sym_integer_literal] = ACTIONS(2853),
    [aux_sym_string_literal_token1] = ACTIONS(2853),
    [sym_char_literal] = ACTIONS(2853),
    [anon_sym_true] = ACTIONS(2855),
    [anon_sym_false] = ACTIONS(2855),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2855),
    [sym_super] = ACTIONS(2855),
    [sym_crate] = ACTIONS(2855),
    [sym_metavariable] = ACTIONS(2853),
    [sym__raw_string_literal_start] = ACTIONS(2853),
    [sym_float_literal] = ACTIONS(2853),
  },
  [736] = {
    [sym_line_comment] = STATE(736),
    [sym_block_comment] = STATE(736),
    [ts_builtin_sym_end] = ACTIONS(2857),
    [sym_identifier] = ACTIONS(2859),
    [anon_sym_SEMI] = ACTIONS(2857),
    [anon_sym_macro_rules_BANG] = ACTIONS(2857),
    [anon_sym_LPAREN] = ACTIONS(2857),
    [anon_sym_LBRACK] = ACTIONS(2857),
    [anon_sym_LBRACE] = ACTIONS(2857),
    [anon_sym_RBRACE] = ACTIONS(2857),
    [anon_sym_STAR] = ACTIONS(2857),
    [anon_sym_u8] = ACTIONS(2859),
    [anon_sym_i8] = ACTIONS(2859),
    [anon_sym_u16] = ACTIONS(2859),
    [anon_sym_i16] = ACTIONS(2859),
    [anon_sym_u32] = ACTIONS(2859),
    [anon_sym_i32] = ACTIONS(2859),
    [anon_sym_u64] = ACTIONS(2859),
    [anon_sym_i64] = ACTIONS(2859),
    [anon_sym_u128] = ACTIONS(2859),
    [anon_sym_i128] = ACTIONS(2859),
    [anon_sym_isize] = ACTIONS(2859),
    [anon_sym_usize] = ACTIONS(2859),
    [anon_sym_f32] = ACTIONS(2859),
    [anon_sym_f64] = ACTIONS(2859),
    [anon_sym_bool] = ACTIONS(2859),
    [anon_sym_str] = ACTIONS(2859),
    [anon_sym_char] = ACTIONS(2859),
    [anon_sym_DASH] = ACTIONS(2857),
    [anon_sym_BANG] = ACTIONS(2857),
    [anon_sym_AMP] = ACTIONS(2857),
    [anon_sym_PIPE] = ACTIONS(2857),
    [anon_sym_LT] = ACTIONS(2857),
    [anon_sym_DOT_DOT] = ACTIONS(2857),
    [anon_sym_COLON_COLON] = ACTIONS(2857),
    [anon_sym_POUND] = ACTIONS(2857),
    [anon_sym_SQUOTE] = ACTIONS(2859),
    [anon_sym_async] = ACTIONS(2859),
    [anon_sym_break] = ACTIONS(2859),
    [anon_sym_const] = ACTIONS(2859),
    [anon_sym_continue] = ACTIONS(2859),
    [anon_sym_default] = ACTIONS(2859),
    [anon_sym_enum] = ACTIONS(2859),
    [anon_sym_fn] = ACTIONS(2859),
    [anon_sym_for] = ACTIONS(2859),
    [anon_sym_gen] = ACTIONS(2859),
    [anon_sym_if] = ACTIONS(2859),
    [anon_sym_impl] = ACTIONS(2859),
    [anon_sym_let] = ACTIONS(2859),
    [anon_sym_loop] = ACTIONS(2859),
    [anon_sym_match] = ACTIONS(2859),
    [anon_sym_mod] = ACTIONS(2859),
    [anon_sym_pub] = ACTIONS(2859),
    [anon_sym_return] = ACTIONS(2859),
    [anon_sym_static] = ACTIONS(2859),
    [anon_sym_struct] = ACTIONS(2859),
    [anon_sym_trait] = ACTIONS(2859),
    [anon_sym_type] = ACTIONS(2859),
    [anon_sym_union] = ACTIONS(2859),
    [anon_sym_unsafe] = ACTIONS(2859),
    [anon_sym_use] = ACTIONS(2859),
    [anon_sym_while] = ACTIONS(2859),
    [anon_sym_extern] = ACTIONS(2859),
    [anon_sym_yield] = ACTIONS(2859),
    [anon_sym_move] = ACTIONS(2859),
    [anon_sym_try] = ACTIONS(2859),
    [sym_integer_literal] = ACTIONS(2857),
    [aux_sym_string_literal_token1] = ACTIONS(2857),
    [sym_char_literal] = ACTIONS(2857),
    [anon_sym_true] = ACTIONS(2859),
    [anon_sym_false] = ACTIONS(2859),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2859),
    [sym_super] = ACTIONS(2859),
    [sym_crate] = ACTIONS(2859),
    [sym_metavariable] = ACTIONS(2857),
    [sym__raw_string_literal_start] = ACTIONS(2857),
    [sym_float_literal] = ACTIONS(2857),
  },
  [737] = {
    [sym_line_comment] = STATE(737),
    [sym_block_comment] = STATE(737),
    [ts_builtin_sym_end] = ACTIONS(2861),
    [sym_identifier] = ACTIONS(2863),
    [anon_sym_SEMI] = ACTIONS(2861),
    [anon_sym_macro_rules_BANG] = ACTIONS(2861),
    [anon_sym_LPAREN] = ACTIONS(2861),
    [anon_sym_LBRACK] = ACTIONS(2861),
    [anon_sym_LBRACE] = ACTIONS(2861),
    [anon_sym_RBRACE] = ACTIONS(2861),
    [anon_sym_STAR] = ACTIONS(2861),
    [anon_sym_u8] = ACTIONS(2863),
    [anon_sym_i8] = ACTIONS(2863),
    [anon_sym_u16] = ACTIONS(2863),
    [anon_sym_i16] = ACTIONS(2863),
    [anon_sym_u32] = ACTIONS(2863),
    [anon_sym_i32] = ACTIONS(2863),
    [anon_sym_u64] = ACTIONS(2863),
    [anon_sym_i64] = ACTIONS(2863),
    [anon_sym_u128] = ACTIONS(2863),
    [anon_sym_i128] = ACTIONS(2863),
    [anon_sym_isize] = ACTIONS(2863),
    [anon_sym_usize] = ACTIONS(2863),
    [anon_sym_f32] = ACTIONS(2863),
    [anon_sym_f64] = ACTIONS(2863),
    [anon_sym_bool] = ACTIONS(2863),
    [anon_sym_str] = ACTIONS(2863),
    [anon_sym_char] = ACTIONS(2863),
    [anon_sym_DASH] = ACTIONS(2861),
    [anon_sym_BANG] = ACTIONS(2861),
    [anon_sym_AMP] = ACTIONS(2861),
    [anon_sym_PIPE] = ACTIONS(2861),
    [anon_sym_LT] = ACTIONS(2861),
    [anon_sym_DOT_DOT] = ACTIONS(2861),
    [anon_sym_COLON_COLON] = ACTIONS(2861),
    [anon_sym_POUND] = ACTIONS(2861),
    [anon_sym_SQUOTE] = ACTIONS(2863),
    [anon_sym_async] = ACTIONS(2863),
    [anon_sym_break] = ACTIONS(2863),
    [anon_sym_const] = ACTIONS(2863),
    [anon_sym_continue] = ACTIONS(2863),
    [anon_sym_default] = ACTIONS(2863),
    [anon_sym_enum] = ACTIONS(2863),
    [anon_sym_fn] = ACTIONS(2863),
    [anon_sym_for] = ACTIONS(2863),
    [anon_sym_gen] = ACTIONS(2863),
    [anon_sym_if] = ACTIONS(2863),
    [anon_sym_impl] = ACTIONS(2863),
    [anon_sym_let] = ACTIONS(2863),
    [anon_sym_loop] = ACTIONS(2863),
    [anon_sym_match] = ACTIONS(2863),
    [anon_sym_mod] = ACTIONS(2863),
    [anon_sym_pub] = ACTIONS(2863),
    [anon_sym_return] = ACTIONS(2863),
    [anon_sym_static] = ACTIONS(2863),
    [anon_sym_struct] = ACTIONS(2863),
    [anon_sym_trait] = ACTIONS(2863),
    [anon_sym_type] = ACTIONS(2863),
    [anon_sym_union] = ACTIONS(2863),
    [anon_sym_unsafe] = ACTIONS(2863),
    [anon_sym_use] = ACTIONS(2863),
    [anon_sym_while] = ACTIONS(2863),
    [anon_sym_extern] = ACTIONS(2863),
    [anon_sym_yield] = ACTIONS(2863),
    [anon_sym_move] = ACTIONS(2863),
    [anon_sym_try] = ACTIONS(2863),
    [sym_integer_literal] = ACTIONS(2861),
    [aux_sym_string_literal_token1] = ACTIONS(2861),
    [sym_char_literal] = ACTIONS(2861),
    [anon_sym_true] = ACTIONS(2863),
    [anon_sym_false] = ACTIONS(2863),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2863),
    [sym_super] = ACTIONS(2863),
    [sym_crate] = ACTIONS(2863),
    [sym_metavariable] = ACTIONS(2861),
    [sym__raw_string_literal_start] = ACTIONS(2861),
    [sym_float_literal] = ACTIONS(2861),
  },
  [738] = {
    [sym_line_comment] = STATE(738),
    [sym_block_comment] = STATE(738),
    [ts_builtin_sym_end] = ACTIONS(2865),
    [sym_identifier] = ACTIONS(2867),
    [anon_sym_SEMI] = ACTIONS(2865),
    [anon_sym_macro_rules_BANG] = ACTIONS(2865),
    [anon_sym_LPAREN] = ACTIONS(2865),
    [anon_sym_LBRACK] = ACTIONS(2865),
    [anon_sym_LBRACE] = ACTIONS(2865),
    [anon_sym_RBRACE] = ACTIONS(2865),
    [anon_sym_STAR] = ACTIONS(2865),
    [anon_sym_u8] = ACTIONS(2867),
    [anon_sym_i8] = ACTIONS(2867),
    [anon_sym_u16] = ACTIONS(2867),
    [anon_sym_i16] = ACTIONS(2867),
    [anon_sym_u32] = ACTIONS(2867),
    [anon_sym_i32] = ACTIONS(2867),
    [anon_sym_u64] = ACTIONS(2867),
    [anon_sym_i64] = ACTIONS(2867),
    [anon_sym_u128] = ACTIONS(2867),
    [anon_sym_i128] = ACTIONS(2867),
    [anon_sym_isize] = ACTIONS(2867),
    [anon_sym_usize] = ACTIONS(2867),
    [anon_sym_f32] = ACTIONS(2867),
    [anon_sym_f64] = ACTIONS(2867),
    [anon_sym_bool] = ACTIONS(2867),
    [anon_sym_str] = ACTIONS(2867),
    [anon_sym_char] = ACTIONS(2867),
    [anon_sym_DASH] = ACTIONS(2865),
    [anon_sym_BANG] = ACTIONS(2865),
    [anon_sym_AMP] = ACTIONS(2865),
    [anon_sym_PIPE] = ACTIONS(2865),
    [anon_sym_LT] = ACTIONS(2865),
    [anon_sym_DOT_DOT] = ACTIONS(2865),
    [anon_sym_COLON_COLON] = ACTIONS(2865),
    [anon_sym_POUND] = ACTIONS(2865),
    [anon_sym_SQUOTE] = ACTIONS(2867),
    [anon_sym_async] = ACTIONS(2867),
    [anon_sym_break] = ACTIONS(2867),
    [anon_sym_const] = ACTIONS(2867),
    [anon_sym_continue] = ACTIONS(2867),
    [anon_sym_default] = ACTIONS(2867),
    [anon_sym_enum] = ACTIONS(2867),
    [anon_sym_fn] = ACTIONS(2867),
    [anon_sym_for] = ACTIONS(2867),
    [anon_sym_gen] = ACTIONS(2867),
    [anon_sym_if] = ACTIONS(2867),
    [anon_sym_impl] = ACTIONS(2867),
    [anon_sym_let] = ACTIONS(2867),
    [anon_sym_loop] = ACTIONS(2867),
    [anon_sym_match] = ACTIONS(2867),
    [anon_sym_mod] = ACTIONS(2867),
    [anon_sym_pub] = ACTIONS(2867),
    [anon_sym_return] = ACTIONS(2867),
    [anon_sym_static] = ACTIONS(2867),
    [anon_sym_struct] = ACTIONS(2867),
    [anon_sym_trait] = ACTIONS(2867),
    [anon_sym_type] = ACTIONS(2867),
    [anon_sym_union] = ACTIONS(2867),
    [anon_sym_unsafe] = ACTIONS(2867),
    [anon_sym_use] = ACTIONS(2867),
    [anon_sym_while] = ACTIONS(2867),
    [anon_sym_extern] = ACTIONS(2867),
    [anon_sym_yield] = ACTIONS(2867),
    [anon_sym_move] = ACTIONS(2867),
    [anon_sym_try] = ACTIONS(2867),
    [sym_integer_literal] = ACTIONS(2865),
    [aux_sym_string_literal_token1] = ACTIONS(2865),
    [sym_char_literal] = ACTIONS(2865),
    [anon_sym_true] = ACTIONS(2867),
    [anon_sym_false] = ACTIONS(2867),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2867),
    [sym_super] = ACTIONS(2867),
    [sym_crate] = ACTIONS(2867),
    [sym_metavariable] = ACTIONS(2865),
    [sym__raw_string_literal_start] = ACTIONS(2865),
    [sym_float_literal] = ACTIONS(2865),
  },
  [739] = {
    [sym_line_comment] = STATE(739),
    [sym_block_comment] = STATE(739),
    [ts_builtin_sym_end] = ACTIONS(2869),
    [sym_identifier] = ACTIONS(2871),
    [anon_sym_SEMI] = ACTIONS(2869),
    [anon_sym_macro_rules_BANG] = ACTIONS(2869),
    [anon_sym_LPAREN] = ACTIONS(2869),
    [anon_sym_LBRACK] = ACTIONS(2869),
    [anon_sym_LBRACE] = ACTIONS(2869),
    [anon_sym_RBRACE] = ACTIONS(2869),
    [anon_sym_STAR] = ACTIONS(2869),
    [anon_sym_u8] = ACTIONS(2871),
    [anon_sym_i8] = ACTIONS(2871),
    [anon_sym_u16] = ACTIONS(2871),
    [anon_sym_i16] = ACTIONS(2871),
    [anon_sym_u32] = ACTIONS(2871),
    [anon_sym_i32] = ACTIONS(2871),
    [anon_sym_u64] = ACTIONS(2871),
    [anon_sym_i64] = ACTIONS(2871),
    [anon_sym_u128] = ACTIONS(2871),
    [anon_sym_i128] = ACTIONS(2871),
    [anon_sym_isize] = ACTIONS(2871),
    [anon_sym_usize] = ACTIONS(2871),
    [anon_sym_f32] = ACTIONS(2871),
    [anon_sym_f64] = ACTIONS(2871),
    [anon_sym_bool] = ACTIONS(2871),
    [anon_sym_str] = ACTIONS(2871),
    [anon_sym_char] = ACTIONS(2871),
    [anon_sym_DASH] = ACTIONS(2869),
    [anon_sym_BANG] = ACTIONS(2869),
    [anon_sym_AMP] = ACTIONS(2869),
    [anon_sym_PIPE] = ACTIONS(2869),
    [anon_sym_LT] = ACTIONS(2869),
    [anon_sym_DOT_DOT] = ACTIONS(2869),
    [anon_sym_COLON_COLON] = ACTIONS(2869),
    [anon_sym_POUND] = ACTIONS(2869),
    [anon_sym_SQUOTE] = ACTIONS(2871),
    [anon_sym_async] = ACTIONS(2871),
    [anon_sym_break] = ACTIONS(2871),
    [anon_sym_const] = ACTIONS(2871),
    [anon_sym_continue] = ACTIONS(2871),
    [anon_sym_default] = ACTIONS(2871),
    [anon_sym_enum] = ACTIONS(2871),
    [anon_sym_fn] = ACTIONS(2871),
    [anon_sym_for] = ACTIONS(2871),
    [anon_sym_gen] = ACTIONS(2871),
    [anon_sym_if] = ACTIONS(2871),
    [anon_sym_impl] = ACTIONS(2871),
    [anon_sym_let] = ACTIONS(2871),
    [anon_sym_loop] = ACTIONS(2871),
    [anon_sym_match] = ACTIONS(2871),
    [anon_sym_mod] = ACTIONS(2871),
    [anon_sym_pub] = ACTIONS(2871),
    [anon_sym_return] = ACTIONS(2871),
    [anon_sym_static] = ACTIONS(2871),
    [anon_sym_struct] = ACTIONS(2871),
    [anon_sym_trait] = ACTIONS(2871),
    [anon_sym_type] = ACTIONS(2871),
    [anon_sym_union] = ACTIONS(2871),
    [anon_sym_unsafe] = ACTIONS(2871),
    [anon_sym_use] = ACTIONS(2871),
    [anon_sym_while] = ACTIONS(2871),
    [anon_sym_extern] = ACTIONS(2871),
    [anon_sym_yield] = ACTIONS(2871),
    [anon_sym_move] = ACTIONS(2871),
    [anon_sym_try] = ACTIONS(2871),
    [sym_integer_literal] = ACTIONS(2869),
    [aux_sym_string_literal_token1] = ACTIONS(2869),
    [sym_char_literal] = ACTIONS(2869),
    [anon_sym_true] = ACTIONS(2871),
    [anon_sym_false] = ACTIONS(2871),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2871),
    [sym_super] = ACTIONS(2871),
    [sym_crate] = ACTIONS(2871),
    [sym_metavariable] = ACTIONS(2869),
    [sym__raw_string_literal_start] = ACTIONS(2869),
    [sym_float_literal] = ACTIONS(2869),
  },
  [740] = {
    [sym_line_comment] = STATE(740),
    [sym_block_comment] = STATE(740),
    [ts_builtin_sym_end] = ACTIONS(2873),
    [sym_identifier] = ACTIONS(2875),
    [anon_sym_SEMI] = ACTIONS(2873),
    [anon_sym_macro_rules_BANG] = ACTIONS(2873),
    [anon_sym_LPAREN] = ACTIONS(2873),
    [anon_sym_LBRACK] = ACTIONS(2873),
    [anon_sym_LBRACE] = ACTIONS(2873),
    [anon_sym_RBRACE] = ACTIONS(2873),
    [anon_sym_STAR] = ACTIONS(2873),
    [anon_sym_u8] = ACTIONS(2875),
    [anon_sym_i8] = ACTIONS(2875),
    [anon_sym_u16] = ACTIONS(2875),
    [anon_sym_i16] = ACTIONS(2875),
    [anon_sym_u32] = ACTIONS(2875),
    [anon_sym_i32] = ACTIONS(2875),
    [anon_sym_u64] = ACTIONS(2875),
    [anon_sym_i64] = ACTIONS(2875),
    [anon_sym_u128] = ACTIONS(2875),
    [anon_sym_i128] = ACTIONS(2875),
    [anon_sym_isize] = ACTIONS(2875),
    [anon_sym_usize] = ACTIONS(2875),
    [anon_sym_f32] = ACTIONS(2875),
    [anon_sym_f64] = ACTIONS(2875),
    [anon_sym_bool] = ACTIONS(2875),
    [anon_sym_str] = ACTIONS(2875),
    [anon_sym_char] = ACTIONS(2875),
    [anon_sym_DASH] = ACTIONS(2873),
    [anon_sym_BANG] = ACTIONS(2873),
    [anon_sym_AMP] = ACTIONS(2873),
    [anon_sym_PIPE] = ACTIONS(2873),
    [anon_sym_LT] = ACTIONS(2873),
    [anon_sym_DOT_DOT] = ACTIONS(2873),
    [anon_sym_COLON_COLON] = ACTIONS(2873),
    [anon_sym_POUND] = ACTIONS(2873),
    [anon_sym_SQUOTE] = ACTIONS(2875),
    [anon_sym_async] = ACTIONS(2875),
    [anon_sym_break] = ACTIONS(2875),
    [anon_sym_const] = ACTIONS(2875),
    [anon_sym_continue] = ACTIONS(2875),
    [anon_sym_default] = ACTIONS(2875),
    [anon_sym_enum] = ACTIONS(2875),
    [anon_sym_fn] = ACTIONS(2875),
    [anon_sym_for] = ACTIONS(2875),
    [anon_sym_gen] = ACTIONS(2875),
    [anon_sym_if] = ACTIONS(2875),
    [anon_sym_impl] = ACTIONS(2875),
    [anon_sym_let] = ACTIONS(2875),
    [anon_sym_loop] = ACTIONS(2875),
    [anon_sym_match] = ACTIONS(2875),
    [anon_sym_mod] = ACTIONS(2875),
    [anon_sym_pub] = ACTIONS(2875),
    [anon_sym_return] = ACTIONS(2875),
    [anon_sym_static] = ACTIONS(2875),
    [anon_sym_struct] = ACTIONS(2875),
    [anon_sym_trait] = ACTIONS(2875),
    [anon_sym_type] = ACTIONS(2875),
    [anon_sym_union] = ACTIONS(2875),
    [anon_sym_unsafe] = ACTIONS(2875),
    [anon_sym_use] = ACTIONS(2875),
    [anon_sym_while] = ACTIONS(2875),
    [anon_sym_extern] = ACTIONS(2875),
    [anon_sym_yield] = ACTIONS(2875),
    [anon_sym_move] = ACTIONS(2875),
    [anon_sym_try] = ACTIONS(2875),
    [sym_integer_literal] = ACTIONS(2873),
    [aux_sym_string_literal_token1] = ACTIONS(2873),
    [sym_char_literal] = ACTIONS(2873),
    [anon_sym_true] = ACTIONS(2875),
    [anon_sym_false] = ACTIONS(2875),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2875),
    [sym_super] = ACTIONS(2875),
    [sym_crate] = ACTIONS(2875),
    [sym_metavariable] = ACTIONS(2873),
    [sym__raw_string_literal_start] = ACTIONS(2873),
    [sym_float_literal] = ACTIONS(2873),
  },
  [741] = {
    [sym_line_comment] = STATE(741),
    [sym_block_comment] = STATE(741),
    [ts_builtin_sym_end] = ACTIONS(2877),
    [sym_identifier] = ACTIONS(2879),
    [anon_sym_SEMI] = ACTIONS(2877),
    [anon_sym_macro_rules_BANG] = ACTIONS(2877),
    [anon_sym_LPAREN] = ACTIONS(2877),
    [anon_sym_LBRACK] = ACTIONS(2877),
    [anon_sym_LBRACE] = ACTIONS(2877),
    [anon_sym_RBRACE] = ACTIONS(2877),
    [anon_sym_STAR] = ACTIONS(2877),
    [anon_sym_u8] = ACTIONS(2879),
    [anon_sym_i8] = ACTIONS(2879),
    [anon_sym_u16] = ACTIONS(2879),
    [anon_sym_i16] = ACTIONS(2879),
    [anon_sym_u32] = ACTIONS(2879),
    [anon_sym_i32] = ACTIONS(2879),
    [anon_sym_u64] = ACTIONS(2879),
    [anon_sym_i64] = ACTIONS(2879),
    [anon_sym_u128] = ACTIONS(2879),
    [anon_sym_i128] = ACTIONS(2879),
    [anon_sym_isize] = ACTIONS(2879),
    [anon_sym_usize] = ACTIONS(2879),
    [anon_sym_f32] = ACTIONS(2879),
    [anon_sym_f64] = ACTIONS(2879),
    [anon_sym_bool] = ACTIONS(2879),
    [anon_sym_str] = ACTIONS(2879),
    [anon_sym_char] = ACTIONS(2879),
    [anon_sym_DASH] = ACTIONS(2877),
    [anon_sym_BANG] = ACTIONS(2877),
    [anon_sym_AMP] = ACTIONS(2877),
    [anon_sym_PIPE] = ACTIONS(2877),
    [anon_sym_LT] = ACTIONS(2877),
    [anon_sym_DOT_DOT] = ACTIONS(2877),
    [anon_sym_COLON_COLON] = ACTIONS(2877),
    [anon_sym_POUND] = ACTIONS(2877),
    [anon_sym_SQUOTE] = ACTIONS(2879),
    [anon_sym_async] = ACTIONS(2879),
    [anon_sym_break] = ACTIONS(2879),
    [anon_sym_const] = ACTIONS(2879),
    [anon_sym_continue] = ACTIONS(2879),
    [anon_sym_default] = ACTIONS(2879),
    [anon_sym_enum] = ACTIONS(2879),
    [anon_sym_fn] = ACTIONS(2879),
    [anon_sym_for] = ACTIONS(2879),
    [anon_sym_gen] = ACTIONS(2879),
    [anon_sym_if] = ACTIONS(2879),
    [anon_sym_impl] = ACTIONS(2879),
    [anon_sym_let] = ACTIONS(2879),
    [anon_sym_loop] = ACTIONS(2879),
    [anon_sym_match] = ACTIONS(2879),
    [anon_sym_mod] = ACTIONS(2879),
    [anon_sym_pub] = ACTIONS(2879),
    [anon_sym_return] = ACTIONS(2879),
    [anon_sym_static] = ACTIONS(2879),
    [anon_sym_struct] = ACTIONS(2879),
    [anon_sym_trait] = ACTIONS(2879),
    [anon_sym_type] = ACTIONS(2879),
    [anon_sym_union] = ACTIONS(2879),
    [anon_sym_unsafe] = ACTIONS(2879),
    [anon_sym_use] = ACTIONS(2879),
    [anon_sym_while] = ACTIONS(2879),
    [anon_sym_extern] = ACTIONS(2879),
    [anon_sym_yield] = ACTIONS(2879),
    [anon_sym_move] = ACTIONS(2879),
    [anon_sym_try] = ACTIONS(2879),
    [sym_integer_literal] = ACTIONS(2877),
    [aux_sym_string_literal_token1] = ACTIONS(2877),
    [sym_char_literal] = ACTIONS(2877),
    [anon_sym_true] = ACTIONS(2879),
    [anon_sym_false] = ACTIONS(2879),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2879),
    [sym_super] = ACTIONS(2879),
    [sym_crate] = ACTIONS(2879),
    [sym_metavariable] = ACTIONS(2877),
    [sym__raw_string_literal_start] = ACTIONS(2877),
    [sym_float_literal] = ACTIONS(2877),
  },
  [742] = {
    [sym_line_comment] = STATE(742),
    [sym_block_comment] = STATE(742),
    [ts_builtin_sym_end] = ACTIONS(2881),
    [sym_identifier] = ACTIONS(2883),
    [anon_sym_SEMI] = ACTIONS(2881),
    [anon_sym_macro_rules_BANG] = ACTIONS(2881),
    [anon_sym_LPAREN] = ACTIONS(2881),
    [anon_sym_LBRACK] = ACTIONS(2881),
    [anon_sym_LBRACE] = ACTIONS(2881),
    [anon_sym_RBRACE] = ACTIONS(2881),
    [anon_sym_STAR] = ACTIONS(2881),
    [anon_sym_u8] = ACTIONS(2883),
    [anon_sym_i8] = ACTIONS(2883),
    [anon_sym_u16] = ACTIONS(2883),
    [anon_sym_i16] = ACTIONS(2883),
    [anon_sym_u32] = ACTIONS(2883),
    [anon_sym_i32] = ACTIONS(2883),
    [anon_sym_u64] = ACTIONS(2883),
    [anon_sym_i64] = ACTIONS(2883),
    [anon_sym_u128] = ACTIONS(2883),
    [anon_sym_i128] = ACTIONS(2883),
    [anon_sym_isize] = ACTIONS(2883),
    [anon_sym_usize] = ACTIONS(2883),
    [anon_sym_f32] = ACTIONS(2883),
    [anon_sym_f64] = ACTIONS(2883),
    [anon_sym_bool] = ACTIONS(2883),
    [anon_sym_str] = ACTIONS(2883),
    [anon_sym_char] = ACTIONS(2883),
    [anon_sym_DASH] = ACTIONS(2881),
    [anon_sym_BANG] = ACTIONS(2881),
    [anon_sym_AMP] = ACTIONS(2881),
    [anon_sym_PIPE] = ACTIONS(2881),
    [anon_sym_LT] = ACTIONS(2881),
    [anon_sym_DOT_DOT] = ACTIONS(2881),
    [anon_sym_COLON_COLON] = ACTIONS(2881),
    [anon_sym_POUND] = ACTIONS(2881),
    [anon_sym_SQUOTE] = ACTIONS(2883),
    [anon_sym_async] = ACTIONS(2883),
    [anon_sym_break] = ACTIONS(2883),
    [anon_sym_const] = ACTIONS(2883),
    [anon_sym_continue] = ACTIONS(2883),
    [anon_sym_default] = ACTIONS(2883),
    [anon_sym_enum] = ACTIONS(2883),
    [anon_sym_fn] = ACTIONS(2883),
    [anon_sym_for] = ACTIONS(2883),
    [anon_sym_gen] = ACTIONS(2883),
    [anon_sym_if] = ACTIONS(2883),
    [anon_sym_impl] = ACTIONS(2883),
    [anon_sym_let] = ACTIONS(2883),
    [anon_sym_loop] = ACTIONS(2883),
    [anon_sym_match] = ACTIONS(2883),
    [anon_sym_mod] = ACTIONS(2883),
    [anon_sym_pub] = ACTIONS(2883),
    [anon_sym_return] = ACTIONS(2883),
    [anon_sym_static] = ACTIONS(2883),
    [anon_sym_struct] = ACTIONS(2883),
    [anon_sym_trait] = ACTIONS(2883),
    [anon_sym_type] = ACTIONS(2883),
    [anon_sym_union] = ACTIONS(2883),
    [anon_sym_unsafe] = ACTIONS(2883),
    [anon_sym_use] = ACTIONS(2883),
    [anon_sym_while] = ACTIONS(2883),
    [anon_sym_extern] = ACTIONS(2883),
    [anon_sym_yield] = ACTIONS(2883),
    [anon_sym_move] = ACTIONS(2883),
    [anon_sym_try] = ACTIONS(2883),
    [sym_integer_literal] = ACTIONS(2881),
    [aux_sym_string_literal_token1] = ACTIONS(2881),
    [sym_char_literal] = ACTIONS(2881),
    [anon_sym_true] = ACTIONS(2883),
    [anon_sym_false] = ACTIONS(2883),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2883),
    [sym_super] = ACTIONS(2883),
    [sym_crate] = ACTIONS(2883),
    [sym_metavariable] = ACTIONS(2881),
    [sym__raw_string_literal_start] = ACTIONS(2881),
    [sym_float_literal] = ACTIONS(2881),
  },
  [743] = {
    [sym_line_comment] = STATE(743),
    [sym_block_comment] = STATE(743),
    [ts_builtin_sym_end] = ACTIONS(2885),
    [sym_identifier] = ACTIONS(2887),
    [anon_sym_SEMI] = ACTIONS(2885),
    [anon_sym_macro_rules_BANG] = ACTIONS(2885),
    [anon_sym_LPAREN] = ACTIONS(2885),
    [anon_sym_LBRACK] = ACTIONS(2885),
    [anon_sym_LBRACE] = ACTIONS(2885),
    [anon_sym_RBRACE] = ACTIONS(2885),
    [anon_sym_STAR] = ACTIONS(2885),
    [anon_sym_u8] = ACTIONS(2887),
    [anon_sym_i8] = ACTIONS(2887),
    [anon_sym_u16] = ACTIONS(2887),
    [anon_sym_i16] = ACTIONS(2887),
    [anon_sym_u32] = ACTIONS(2887),
    [anon_sym_i32] = ACTIONS(2887),
    [anon_sym_u64] = ACTIONS(2887),
    [anon_sym_i64] = ACTIONS(2887),
    [anon_sym_u128] = ACTIONS(2887),
    [anon_sym_i128] = ACTIONS(2887),
    [anon_sym_isize] = ACTIONS(2887),
    [anon_sym_usize] = ACTIONS(2887),
    [anon_sym_f32] = ACTIONS(2887),
    [anon_sym_f64] = ACTIONS(2887),
    [anon_sym_bool] = ACTIONS(2887),
    [anon_sym_str] = ACTIONS(2887),
    [anon_sym_char] = ACTIONS(2887),
    [anon_sym_DASH] = ACTIONS(2885),
    [anon_sym_BANG] = ACTIONS(2885),
    [anon_sym_AMP] = ACTIONS(2885),
    [anon_sym_PIPE] = ACTIONS(2885),
    [anon_sym_LT] = ACTIONS(2885),
    [anon_sym_DOT_DOT] = ACTIONS(2885),
    [anon_sym_COLON_COLON] = ACTIONS(2885),
    [anon_sym_POUND] = ACTIONS(2885),
    [anon_sym_SQUOTE] = ACTIONS(2887),
    [anon_sym_async] = ACTIONS(2887),
    [anon_sym_break] = ACTIONS(2887),
    [anon_sym_const] = ACTIONS(2887),
    [anon_sym_continue] = ACTIONS(2887),
    [anon_sym_default] = ACTIONS(2887),
    [anon_sym_enum] = ACTIONS(2887),
    [anon_sym_fn] = ACTIONS(2887),
    [anon_sym_for] = ACTIONS(2887),
    [anon_sym_gen] = ACTIONS(2887),
    [anon_sym_if] = ACTIONS(2887),
    [anon_sym_impl] = ACTIONS(2887),
    [anon_sym_let] = ACTIONS(2887),
    [anon_sym_loop] = ACTIONS(2887),
    [anon_sym_match] = ACTIONS(2887),
    [anon_sym_mod] = ACTIONS(2887),
    [anon_sym_pub] = ACTIONS(2887),
    [anon_sym_return] = ACTIONS(2887),
    [anon_sym_static] = ACTIONS(2887),
    [anon_sym_struct] = ACTIONS(2887),
    [anon_sym_trait] = ACTIONS(2887),
    [anon_sym_type] = ACTIONS(2887),
    [anon_sym_union] = ACTIONS(2887),
    [anon_sym_unsafe] = ACTIONS(2887),
    [anon_sym_use] = ACTIONS(2887),
    [anon_sym_while] = ACTIONS(2887),
    [anon_sym_extern] = ACTIONS(2887),
    [anon_sym_yield] = ACTIONS(2887),
    [anon_sym_move] = ACTIONS(2887),
    [anon_sym_try] = ACTIONS(2887),
    [sym_integer_literal] = ACTIONS(2885),
    [aux_sym_string_literal_token1] = ACTIONS(2885),
    [sym_char_literal] = ACTIONS(2885),
    [anon_sym_true] = ACTIONS(2887),
    [anon_sym_false] = ACTIONS(2887),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2887),
    [sym_super] = ACTIONS(2887),
    [sym_crate] = ACTIONS(2887),
    [sym_metavariable] = ACTIONS(2885),
    [sym__raw_string_literal_start] = ACTIONS(2885),
    [sym_float_literal] = ACTIONS(2885),
  },
  [744] = {
    [sym_line_comment] = STATE(744),
    [sym_block_comment] = STATE(744),
    [ts_builtin_sym_end] = ACTIONS(2889),
    [sym_identifier] = ACTIONS(2891),
    [anon_sym_SEMI] = ACTIONS(2889),
    [anon_sym_macro_rules_BANG] = ACTIONS(2889),
    [anon_sym_LPAREN] = ACTIONS(2889),
    [anon_sym_LBRACK] = ACTIONS(2889),
    [anon_sym_LBRACE] = ACTIONS(2889),
    [anon_sym_RBRACE] = ACTIONS(2889),
    [anon_sym_STAR] = ACTIONS(2889),
    [anon_sym_u8] = ACTIONS(2891),
    [anon_sym_i8] = ACTIONS(2891),
    [anon_sym_u16] = ACTIONS(2891),
    [anon_sym_i16] = ACTIONS(2891),
    [anon_sym_u32] = ACTIONS(2891),
    [anon_sym_i32] = ACTIONS(2891),
    [anon_sym_u64] = ACTIONS(2891),
    [anon_sym_i64] = ACTIONS(2891),
    [anon_sym_u128] = ACTIONS(2891),
    [anon_sym_i128] = ACTIONS(2891),
    [anon_sym_isize] = ACTIONS(2891),
    [anon_sym_usize] = ACTIONS(2891),
    [anon_sym_f32] = ACTIONS(2891),
    [anon_sym_f64] = ACTIONS(2891),
    [anon_sym_bool] = ACTIONS(2891),
    [anon_sym_str] = ACTIONS(2891),
    [anon_sym_char] = ACTIONS(2891),
    [anon_sym_DASH] = ACTIONS(2889),
    [anon_sym_BANG] = ACTIONS(2889),
    [anon_sym_AMP] = ACTIONS(2889),
    [anon_sym_PIPE] = ACTIONS(2889),
    [anon_sym_LT] = ACTIONS(2889),
    [anon_sym_DOT_DOT] = ACTIONS(2889),
    [anon_sym_COLON_COLON] = ACTIONS(2889),
    [anon_sym_POUND] = ACTIONS(2889),
    [anon_sym_SQUOTE] = ACTIONS(2891),
    [anon_sym_async] = ACTIONS(2891),
    [anon_sym_break] = ACTIONS(2891),
    [anon_sym_const] = ACTIONS(2891),
    [anon_sym_continue] = ACTIONS(2891),
    [anon_sym_default] = ACTIONS(2891),
    [anon_sym_enum] = ACTIONS(2891),
    [anon_sym_fn] = ACTIONS(2891),
    [anon_sym_for] = ACTIONS(2891),
    [anon_sym_gen] = ACTIONS(2891),
    [anon_sym_if] = ACTIONS(2891),
    [anon_sym_impl] = ACTIONS(2891),
    [anon_sym_let] = ACTIONS(2891),
    [anon_sym_loop] = ACTIONS(2891),
    [anon_sym_match] = ACTIONS(2891),
    [anon_sym_mod] = ACTIONS(2891),
    [anon_sym_pub] = ACTIONS(2891),
    [anon_sym_return] = ACTIONS(2891),
    [anon_sym_static] = ACTIONS(2891),
    [anon_sym_struct] = ACTIONS(2891),
    [anon_sym_trait] = ACTIONS(2891),
    [anon_sym_type] = ACTIONS(2891),
    [anon_sym_union] = ACTIONS(2891),
    [anon_sym_unsafe] = ACTIONS(2891),
    [anon_sym_use] = ACTIONS(2891),
    [anon_sym_while] = ACTIONS(2891),
    [anon_sym_extern] = ACTIONS(2891),
    [anon_sym_yield] = ACTIONS(2891),
    [anon_sym_move] = ACTIONS(2891),
    [anon_sym_try] = ACTIONS(2891),
    [sym_integer_literal] = ACTIONS(2889),
    [aux_sym_string_literal_token1] = ACTIONS(2889),
    [sym_char_literal] = ACTIONS(2889),
    [anon_sym_true] = ACTIONS(2891),
    [anon_sym_false] = ACTIONS(2891),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2891),
    [sym_super] = ACTIONS(2891),
    [sym_crate] = ACTIONS(2891),
    [sym_metavariable] = ACTIONS(2889),
    [sym__raw_string_literal_start] = ACTIONS(2889),
    [sym_float_literal] = ACTIONS(2889),
  },
  [745] = {
    [sym_line_comment] = STATE(745),
    [sym_block_comment] = STATE(745),
    [ts_builtin_sym_end] = ACTIONS(2893),
    [sym_identifier] = ACTIONS(2895),
    [anon_sym_SEMI] = ACTIONS(2893),
    [anon_sym_macro_rules_BANG] = ACTIONS(2893),
    [anon_sym_LPAREN] = ACTIONS(2893),
    [anon_sym_LBRACK] = ACTIONS(2893),
    [anon_sym_LBRACE] = ACTIONS(2893),
    [anon_sym_RBRACE] = ACTIONS(2893),
    [anon_sym_STAR] = ACTIONS(2893),
    [anon_sym_u8] = ACTIONS(2895),
    [anon_sym_i8] = ACTIONS(2895),
    [anon_sym_u16] = ACTIONS(2895),
    [anon_sym_i16] = ACTIONS(2895),
    [anon_sym_u32] = ACTIONS(2895),
    [anon_sym_i32] = ACTIONS(2895),
    [anon_sym_u64] = ACTIONS(2895),
    [anon_sym_i64] = ACTIONS(2895),
    [anon_sym_u128] = ACTIONS(2895),
    [anon_sym_i128] = ACTIONS(2895),
    [anon_sym_isize] = ACTIONS(2895),
    [anon_sym_usize] = ACTIONS(2895),
    [anon_sym_f32] = ACTIONS(2895),
    [anon_sym_f64] = ACTIONS(2895),
    [anon_sym_bool] = ACTIONS(2895),
    [anon_sym_str] = ACTIONS(2895),
    [anon_sym_char] = ACTIONS(2895),
    [anon_sym_DASH] = ACTIONS(2893),
    [anon_sym_BANG] = ACTIONS(2893),
    [anon_sym_AMP] = ACTIONS(2893),
    [anon_sym_PIPE] = ACTIONS(2893),
    [anon_sym_LT] = ACTIONS(2893),
    [anon_sym_DOT_DOT] = ACTIONS(2893),
    [anon_sym_COLON_COLON] = ACTIONS(2893),
    [anon_sym_POUND] = ACTIONS(2893),
    [anon_sym_SQUOTE] = ACTIONS(2895),
    [anon_sym_async] = ACTIONS(2895),
    [anon_sym_break] = ACTIONS(2895),
    [anon_sym_const] = ACTIONS(2895),
    [anon_sym_continue] = ACTIONS(2895),
    [anon_sym_default] = ACTIONS(2895),
    [anon_sym_enum] = ACTIONS(2895),
    [anon_sym_fn] = ACTIONS(2895),
    [anon_sym_for] = ACTIONS(2895),
    [anon_sym_gen] = ACTIONS(2895),
    [anon_sym_if] = ACTIONS(2895),
    [anon_sym_impl] = ACTIONS(2895),
    [anon_sym_let] = ACTIONS(2895),
    [anon_sym_loop] = ACTIONS(2895),
    [anon_sym_match] = ACTIONS(2895),
    [anon_sym_mod] = ACTIONS(2895),
    [anon_sym_pub] = ACTIONS(2895),
    [anon_sym_return] = ACTIONS(2895),
    [anon_sym_static] = ACTIONS(2895),
    [anon_sym_struct] = ACTIONS(2895),
    [anon_sym_trait] = ACTIONS(2895),
    [anon_sym_type] = ACTIONS(2895),
    [anon_sym_union] = ACTIONS(2895),
    [anon_sym_unsafe] = ACTIONS(2895),
    [anon_sym_use] = ACTIONS(2895),
    [anon_sym_while] = ACTIONS(2895),
    [anon_sym_extern] = ACTIONS(2895),
    [anon_sym_yield] = ACTIONS(2895),
    [anon_sym_move] = ACTIONS(2895),
    [anon_sym_try] = ACTIONS(2895),
    [sym_integer_literal] = ACTIONS(2893),
    [aux_sym_string_literal_token1] = ACTIONS(2893),
    [sym_char_literal] = ACTIONS(2893),
    [anon_sym_true] = ACTIONS(2895),
    [anon_sym_false] = ACTIONS(2895),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2895),
    [sym_super] = ACTIONS(2895),
    [sym_crate] = ACTIONS(2895),
    [sym_metavariable] = ACTIONS(2893),
    [sym__raw_string_literal_start] = ACTIONS(2893),
    [sym_float_literal] = ACTIONS(2893),
  },
  [746] = {
    [sym_line_comment] = STATE(746),
    [sym_block_comment] = STATE(746),
    [ts_builtin_sym_end] = ACTIONS(2897),
    [sym_identifier] = ACTIONS(2899),
    [anon_sym_SEMI] = ACTIONS(2897),
    [anon_sym_macro_rules_BANG] = ACTIONS(2897),
    [anon_sym_LPAREN] = ACTIONS(2897),
    [anon_sym_LBRACK] = ACTIONS(2897),
    [anon_sym_LBRACE] = ACTIONS(2897),
    [anon_sym_RBRACE] = ACTIONS(2897),
    [anon_sym_STAR] = ACTIONS(2897),
    [anon_sym_u8] = ACTIONS(2899),
    [anon_sym_i8] = ACTIONS(2899),
    [anon_sym_u16] = ACTIONS(2899),
    [anon_sym_i16] = ACTIONS(2899),
    [anon_sym_u32] = ACTIONS(2899),
    [anon_sym_i32] = ACTIONS(2899),
    [anon_sym_u64] = ACTIONS(2899),
    [anon_sym_i64] = ACTIONS(2899),
    [anon_sym_u128] = ACTIONS(2899),
    [anon_sym_i128] = ACTIONS(2899),
    [anon_sym_isize] = ACTIONS(2899),
    [anon_sym_usize] = ACTIONS(2899),
    [anon_sym_f32] = ACTIONS(2899),
    [anon_sym_f64] = ACTIONS(2899),
    [anon_sym_bool] = ACTIONS(2899),
    [anon_sym_str] = ACTIONS(2899),
    [anon_sym_char] = ACTIONS(2899),
    [anon_sym_DASH] = ACTIONS(2897),
    [anon_sym_BANG] = ACTIONS(2897),
    [anon_sym_AMP] = ACTIONS(2897),
    [anon_sym_PIPE] = ACTIONS(2897),
    [anon_sym_LT] = ACTIONS(2897),
    [anon_sym_DOT_DOT] = ACTIONS(2897),
    [anon_sym_COLON_COLON] = ACTIONS(2897),
    [anon_sym_POUND] = ACTIONS(2897),
    [anon_sym_SQUOTE] = ACTIONS(2899),
    [anon_sym_async] = ACTIONS(2899),
    [anon_sym_break] = ACTIONS(2899),
    [anon_sym_const] = ACTIONS(2899),
    [anon_sym_continue] = ACTIONS(2899),
    [anon_sym_default] = ACTIONS(2899),
    [anon_sym_enum] = ACTIONS(2899),
    [anon_sym_fn] = ACTIONS(2899),
    [anon_sym_for] = ACTIONS(2899),
    [anon_sym_gen] = ACTIONS(2899),
    [anon_sym_if] = ACTIONS(2899),
    [anon_sym_impl] = ACTIONS(2899),
    [anon_sym_let] = ACTIONS(2899),
    [anon_sym_loop] = ACTIONS(2899),
    [anon_sym_match] = ACTIONS(2899),
    [anon_sym_mod] = ACTIONS(2899),
    [anon_sym_pub] = ACTIONS(2899),
    [anon_sym_return] = ACTIONS(2899),
    [anon_sym_static] = ACTIONS(2899),
    [anon_sym_struct] = ACTIONS(2899),
    [anon_sym_trait] = ACTIONS(2899),
    [anon_sym_type] = ACTIONS(2899),
    [anon_sym_union] = ACTIONS(2899),
    [anon_sym_unsafe] = ACTIONS(2899),
    [anon_sym_use] = ACTIONS(2899),
    [anon_sym_while] = ACTIONS(2899),
    [anon_sym_extern] = ACTIONS(2899),
    [anon_sym_yield] = ACTIONS(2899),
    [anon_sym_move] = ACTIONS(2899),
    [anon_sym_try] = ACTIONS(2899),
    [sym_integer_literal] = ACTIONS(2897),
    [aux_sym_string_literal_token1] = ACTIONS(2897),
    [sym_char_literal] = ACTIONS(2897),
    [anon_sym_true] = ACTIONS(2899),
    [anon_sym_false] = ACTIONS(2899),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2899),
    [sym_super] = ACTIONS(2899),
    [sym_crate] = ACTIONS(2899),
    [sym_metavariable] = ACTIONS(2897),
    [sym__raw_string_literal_start] = ACTIONS(2897),
    [sym_float_literal] = ACTIONS(2897),
  },
  [747] = {
    [sym_line_comment] = STATE(747),
    [sym_block_comment] = STATE(747),
    [ts_builtin_sym_end] = ACTIONS(2901),
    [sym_identifier] = ACTIONS(2903),
    [anon_sym_SEMI] = ACTIONS(2901),
    [anon_sym_macro_rules_BANG] = ACTIONS(2901),
    [anon_sym_LPAREN] = ACTIONS(2901),
    [anon_sym_LBRACK] = ACTIONS(2901),
    [anon_sym_LBRACE] = ACTIONS(2901),
    [anon_sym_RBRACE] = ACTIONS(2901),
    [anon_sym_STAR] = ACTIONS(2901),
    [anon_sym_u8] = ACTIONS(2903),
    [anon_sym_i8] = ACTIONS(2903),
    [anon_sym_u16] = ACTIONS(2903),
    [anon_sym_i16] = ACTIONS(2903),
    [anon_sym_u32] = ACTIONS(2903),
    [anon_sym_i32] = ACTIONS(2903),
    [anon_sym_u64] = ACTIONS(2903),
    [anon_sym_i64] = ACTIONS(2903),
    [anon_sym_u128] = ACTIONS(2903),
    [anon_sym_i128] = ACTIONS(2903),
    [anon_sym_isize] = ACTIONS(2903),
    [anon_sym_usize] = ACTIONS(2903),
    [anon_sym_f32] = ACTIONS(2903),
    [anon_sym_f64] = ACTIONS(2903),
    [anon_sym_bool] = ACTIONS(2903),
    [anon_sym_str] = ACTIONS(2903),
    [anon_sym_char] = ACTIONS(2903),
    [anon_sym_DASH] = ACTIONS(2901),
    [anon_sym_BANG] = ACTIONS(2901),
    [anon_sym_AMP] = ACTIONS(2901),
    [anon_sym_PIPE] = ACTIONS(2901),
    [anon_sym_LT] = ACTIONS(2901),
    [anon_sym_DOT_DOT] = ACTIONS(2901),
    [anon_sym_COLON_COLON] = ACTIONS(2901),
    [anon_sym_POUND] = ACTIONS(2901),
    [anon_sym_SQUOTE] = ACTIONS(2903),
    [anon_sym_async] = ACTIONS(2903),
    [anon_sym_break] = ACTIONS(2903),
    [anon_sym_const] = ACTIONS(2903),
    [anon_sym_continue] = ACTIONS(2903),
    [anon_sym_default] = ACTIONS(2903),
    [anon_sym_enum] = ACTIONS(2903),
    [anon_sym_fn] = ACTIONS(2903),
    [anon_sym_for] = ACTIONS(2903),
    [anon_sym_gen] = ACTIONS(2903),
    [anon_sym_if] = ACTIONS(2903),
    [anon_sym_impl] = ACTIONS(2903),
    [anon_sym_let] = ACTIONS(2903),
    [anon_sym_loop] = ACTIONS(2903),
    [anon_sym_match] = ACTIONS(2903),
    [anon_sym_mod] = ACTIONS(2903),
    [anon_sym_pub] = ACTIONS(2903),
    [anon_sym_return] = ACTIONS(2903),
    [anon_sym_static] = ACTIONS(2903),
    [anon_sym_struct] = ACTIONS(2903),
    [anon_sym_trait] = ACTIONS(2903),
    [anon_sym_type] = ACTIONS(2903),
    [anon_sym_union] = ACTIONS(2903),
    [anon_sym_unsafe] = ACTIONS(2903),
    [anon_sym_use] = ACTIONS(2903),
    [anon_sym_while] = ACTIONS(2903),
    [anon_sym_extern] = ACTIONS(2903),
    [anon_sym_yield] = ACTIONS(2903),
    [anon_sym_move] = ACTIONS(2903),
    [anon_sym_try] = ACTIONS(2903),
    [sym_integer_literal] = ACTIONS(2901),
    [aux_sym_string_literal_token1] = ACTIONS(2901),
    [sym_char_literal] = ACTIONS(2901),
    [anon_sym_true] = ACTIONS(2903),
    [anon_sym_false] = ACTIONS(2903),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2903),
    [sym_super] = ACTIONS(2903),
    [sym_crate] = ACTIONS(2903),
    [sym_metavariable] = ACTIONS(2901),
    [sym__raw_string_literal_start] = ACTIONS(2901),
    [sym_float_literal] = ACTIONS(2901),
  },
  [748] = {
    [sym_line_comment] = STATE(748),
    [sym_block_comment] = STATE(748),
    [ts_builtin_sym_end] = ACTIONS(2905),
    [sym_identifier] = ACTIONS(2907),
    [anon_sym_SEMI] = ACTIONS(2905),
    [anon_sym_macro_rules_BANG] = ACTIONS(2905),
    [anon_sym_LPAREN] = ACTIONS(2905),
    [anon_sym_LBRACK] = ACTIONS(2905),
    [anon_sym_LBRACE] = ACTIONS(2905),
    [anon_sym_RBRACE] = ACTIONS(2905),
    [anon_sym_STAR] = ACTIONS(2905),
    [anon_sym_u8] = ACTIONS(2907),
    [anon_sym_i8] = ACTIONS(2907),
    [anon_sym_u16] = ACTIONS(2907),
    [anon_sym_i16] = ACTIONS(2907),
    [anon_sym_u32] = ACTIONS(2907),
    [anon_sym_i32] = ACTIONS(2907),
    [anon_sym_u64] = ACTIONS(2907),
    [anon_sym_i64] = ACTIONS(2907),
    [anon_sym_u128] = ACTIONS(2907),
    [anon_sym_i128] = ACTIONS(2907),
    [anon_sym_isize] = ACTIONS(2907),
    [anon_sym_usize] = ACTIONS(2907),
    [anon_sym_f32] = ACTIONS(2907),
    [anon_sym_f64] = ACTIONS(2907),
    [anon_sym_bool] = ACTIONS(2907),
    [anon_sym_str] = ACTIONS(2907),
    [anon_sym_char] = ACTIONS(2907),
    [anon_sym_DASH] = ACTIONS(2905),
    [anon_sym_BANG] = ACTIONS(2905),
    [anon_sym_AMP] = ACTIONS(2905),
    [anon_sym_PIPE] = ACTIONS(2905),
    [anon_sym_LT] = ACTIONS(2905),
    [anon_sym_DOT_DOT] = ACTIONS(2905),
    [anon_sym_COLON_COLON] = ACTIONS(2905),
    [anon_sym_POUND] = ACTIONS(2905),
    [anon_sym_SQUOTE] = ACTIONS(2907),
    [anon_sym_async] = ACTIONS(2907),
    [anon_sym_break] = ACTIONS(2907),
    [anon_sym_const] = ACTIONS(2907),
    [anon_sym_continue] = ACTIONS(2907),
    [anon_sym_default] = ACTIONS(2907),
    [anon_sym_enum] = ACTIONS(2907),
    [anon_sym_fn] = ACTIONS(2907),
    [anon_sym_for] = ACTIONS(2907),
    [anon_sym_gen] = ACTIONS(2907),
    [anon_sym_if] = ACTIONS(2907),
    [anon_sym_impl] = ACTIONS(2907),
    [anon_sym_let] = ACTIONS(2907),
    [anon_sym_loop] = ACTIONS(2907),
    [anon_sym_match] = ACTIONS(2907),
    [anon_sym_mod] = ACTIONS(2907),
    [anon_sym_pub] = ACTIONS(2907),
    [anon_sym_return] = ACTIONS(2907),
    [anon_sym_static] = ACTIONS(2907),
    [anon_sym_struct] = ACTIONS(2907),
    [anon_sym_trait] = ACTIONS(2907),
    [anon_sym_type] = ACTIONS(2907),
    [anon_sym_union] = ACTIONS(2907),
    [anon_sym_unsafe] = ACTIONS(2907),
    [anon_sym_use] = ACTIONS(2907),
    [anon_sym_while] = ACTIONS(2907),
    [anon_sym_extern] = ACTIONS(2907),
    [anon_sym_yield] = ACTIONS(2907),
    [anon_sym_move] = ACTIONS(2907),
    [anon_sym_try] = ACTIONS(2907),
    [sym_integer_literal] = ACTIONS(2905),
    [aux_sym_string_literal_token1] = ACTIONS(2905),
    [sym_char_literal] = ACTIONS(2905),
    [anon_sym_true] = ACTIONS(2907),
    [anon_sym_false] = ACTIONS(2907),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2907),
    [sym_super] = ACTIONS(2907),
    [sym_crate] = ACTIONS(2907),
    [sym_metavariable] = ACTIONS(2905),
    [sym__raw_string_literal_start] = ACTIONS(2905),
    [sym_float_literal] = ACTIONS(2905),
  },
  [749] = {
    [sym_line_comment] = STATE(749),
    [sym_block_comment] = STATE(749),
    [ts_builtin_sym_end] = ACTIONS(2909),
    [sym_identifier] = ACTIONS(2911),
    [anon_sym_SEMI] = ACTIONS(2909),
    [anon_sym_macro_rules_BANG] = ACTIONS(2909),
    [anon_sym_LPAREN] = ACTIONS(2909),
    [anon_sym_LBRACK] = ACTIONS(2909),
    [anon_sym_LBRACE] = ACTIONS(2909),
    [anon_sym_RBRACE] = ACTIONS(2909),
    [anon_sym_STAR] = ACTIONS(2909),
    [anon_sym_u8] = ACTIONS(2911),
    [anon_sym_i8] = ACTIONS(2911),
    [anon_sym_u16] = ACTIONS(2911),
    [anon_sym_i16] = ACTIONS(2911),
    [anon_sym_u32] = ACTIONS(2911),
    [anon_sym_i32] = ACTIONS(2911),
    [anon_sym_u64] = ACTIONS(2911),
    [anon_sym_i64] = ACTIONS(2911),
    [anon_sym_u128] = ACTIONS(2911),
    [anon_sym_i128] = ACTIONS(2911),
    [anon_sym_isize] = ACTIONS(2911),
    [anon_sym_usize] = ACTIONS(2911),
    [anon_sym_f32] = ACTIONS(2911),
    [anon_sym_f64] = ACTIONS(2911),
    [anon_sym_bool] = ACTIONS(2911),
    [anon_sym_str] = ACTIONS(2911),
    [anon_sym_char] = ACTIONS(2911),
    [anon_sym_DASH] = ACTIONS(2909),
    [anon_sym_BANG] = ACTIONS(2909),
    [anon_sym_AMP] = ACTIONS(2909),
    [anon_sym_PIPE] = ACTIONS(2909),
    [anon_sym_LT] = ACTIONS(2909),
    [anon_sym_DOT_DOT] = ACTIONS(2909),
    [anon_sym_COLON_COLON] = ACTIONS(2909),
    [anon_sym_POUND] = ACTIONS(2909),
    [anon_sym_SQUOTE] = ACTIONS(2911),
    [anon_sym_async] = ACTIONS(2911),
    [anon_sym_break] = ACTIONS(2911),
    [anon_sym_const] = ACTIONS(2911),
    [anon_sym_continue] = ACTIONS(2911),
    [anon_sym_default] = ACTIONS(2911),
    [anon_sym_enum] = ACTIONS(2911),
    [anon_sym_fn] = ACTIONS(2911),
    [anon_sym_for] = ACTIONS(2911),
    [anon_sym_gen] = ACTIONS(2911),
    [anon_sym_if] = ACTIONS(2911),
    [anon_sym_impl] = ACTIONS(2911),
    [anon_sym_let] = ACTIONS(2911),
    [anon_sym_loop] = ACTIONS(2911),
    [anon_sym_match] = ACTIONS(2911),
    [anon_sym_mod] = ACTIONS(2911),
    [anon_sym_pub] = ACTIONS(2911),
    [anon_sym_return] = ACTIONS(2911),
    [anon_sym_static] = ACTIONS(2911),
    [anon_sym_struct] = ACTIONS(2911),
    [anon_sym_trait] = ACTIONS(2911),
    [anon_sym_type] = ACTIONS(2911),
    [anon_sym_union] = ACTIONS(2911),
    [anon_sym_unsafe] = ACTIONS(2911),
    [anon_sym_use] = ACTIONS(2911),
    [anon_sym_while] = ACTIONS(2911),
    [anon_sym_extern] = ACTIONS(2911),
    [anon_sym_yield] = ACTIONS(2911),
    [anon_sym_move] = ACTIONS(2911),
    [anon_sym_try] = ACTIONS(2911),
    [sym_integer_literal] = ACTIONS(2909),
    [aux_sym_string_literal_token1] = ACTIONS(2909),
    [sym_char_literal] = ACTIONS(2909),
    [anon_sym_true] = ACTIONS(2911),
    [anon_sym_false] = ACTIONS(2911),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2911),
    [sym_super] = ACTIONS(2911),
    [sym_crate] = ACTIONS(2911),
    [sym_metavariable] = ACTIONS(2909),
    [sym__raw_string_literal_start] = ACTIONS(2909),
    [sym_float_literal] = ACTIONS(2909),
  },
  [750] = {
    [sym_line_comment] = STATE(750),
    [sym_block_comment] = STATE(750),
    [ts_builtin_sym_end] = ACTIONS(2913),
    [sym_identifier] = ACTIONS(2915),
    [anon_sym_SEMI] = ACTIONS(2913),
    [anon_sym_macro_rules_BANG] = ACTIONS(2913),
    [anon_sym_LPAREN] = ACTIONS(2913),
    [anon_sym_LBRACK] = ACTIONS(2913),
    [anon_sym_LBRACE] = ACTIONS(2913),
    [anon_sym_RBRACE] = ACTIONS(2913),
    [anon_sym_STAR] = ACTIONS(2913),
    [anon_sym_u8] = ACTIONS(2915),
    [anon_sym_i8] = ACTIONS(2915),
    [anon_sym_u16] = ACTIONS(2915),
    [anon_sym_i16] = ACTIONS(2915),
    [anon_sym_u32] = ACTIONS(2915),
    [anon_sym_i32] = ACTIONS(2915),
    [anon_sym_u64] = ACTIONS(2915),
    [anon_sym_i64] = ACTIONS(2915),
    [anon_sym_u128] = ACTIONS(2915),
    [anon_sym_i128] = ACTIONS(2915),
    [anon_sym_isize] = ACTIONS(2915),
    [anon_sym_usize] = ACTIONS(2915),
    [anon_sym_f32] = ACTIONS(2915),
    [anon_sym_f64] = ACTIONS(2915),
    [anon_sym_bool] = ACTIONS(2915),
    [anon_sym_str] = ACTIONS(2915),
    [anon_sym_char] = ACTIONS(2915),
    [anon_sym_DASH] = ACTIONS(2913),
    [anon_sym_BANG] = ACTIONS(2913),
    [anon_sym_AMP] = ACTIONS(2913),
    [anon_sym_PIPE] = ACTIONS(2913),
    [anon_sym_LT] = ACTIONS(2913),
    [anon_sym_DOT_DOT] = ACTIONS(2913),
    [anon_sym_COLON_COLON] = ACTIONS(2913),
    [anon_sym_POUND] = ACTIONS(2913),
    [anon_sym_SQUOTE] = ACTIONS(2915),
    [anon_sym_async] = ACTIONS(2915),
    [anon_sym_break] = ACTIONS(2915),
    [anon_sym_const] = ACTIONS(2915),
    [anon_sym_continue] = ACTIONS(2915),
    [anon_sym_default] = ACTIONS(2915),
    [anon_sym_enum] = ACTIONS(2915),
    [anon_sym_fn] = ACTIONS(2915),
    [anon_sym_for] = ACTIONS(2915),
    [anon_sym_gen] = ACTIONS(2915),
    [anon_sym_if] = ACTIONS(2915),
    [anon_sym_impl] = ACTIONS(2915),
    [anon_sym_let] = ACTIONS(2915),
    [anon_sym_loop] = ACTIONS(2915),
    [anon_sym_match] = ACTIONS(2915),
    [anon_sym_mod] = ACTIONS(2915),
    [anon_sym_pub] = ACTIONS(2915),
    [anon_sym_return] = ACTIONS(2915),
    [anon_sym_static] = ACTIONS(2915),
    [anon_sym_struct] = ACTIONS(2915),
    [anon_sym_trait] = ACTIONS(2915),
    [anon_sym_type] = ACTIONS(2915),
    [anon_sym_union] = ACTIONS(2915),
    [anon_sym_unsafe] = ACTIONS(2915),
    [anon_sym_use] = ACTIONS(2915),
    [anon_sym_while] = ACTIONS(2915),
    [anon_sym_extern] = ACTIONS(2915),
    [anon_sym_yield] = ACTIONS(2915),
    [anon_sym_move] = ACTIONS(2915),
    [anon_sym_try] = ACTIONS(2915),
    [sym_integer_literal] = ACTIONS(2913),
    [aux_sym_string_literal_token1] = ACTIONS(2913),
    [sym_char_literal] = ACTIONS(2913),
    [anon_sym_true] = ACTIONS(2915),
    [anon_sym_false] = ACTIONS(2915),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2915),
    [sym_super] = ACTIONS(2915),
    [sym_crate] = ACTIONS(2915),
    [sym_metavariable] = ACTIONS(2913),
    [sym__raw_string_literal_start] = ACTIONS(2913),
    [sym_float_literal] = ACTIONS(2913),
  },
  [751] = {
    [sym_line_comment] = STATE(751),
    [sym_block_comment] = STATE(751),
    [ts_builtin_sym_end] = ACTIONS(2917),
    [sym_identifier] = ACTIONS(2919),
    [anon_sym_SEMI] = ACTIONS(2917),
    [anon_sym_macro_rules_BANG] = ACTIONS(2917),
    [anon_sym_LPAREN] = ACTIONS(2917),
    [anon_sym_LBRACK] = ACTIONS(2917),
    [anon_sym_LBRACE] = ACTIONS(2917),
    [anon_sym_RBRACE] = ACTIONS(2917),
    [anon_sym_STAR] = ACTIONS(2917),
    [anon_sym_u8] = ACTIONS(2919),
    [anon_sym_i8] = ACTIONS(2919),
    [anon_sym_u16] = ACTIONS(2919),
    [anon_sym_i16] = ACTIONS(2919),
    [anon_sym_u32] = ACTIONS(2919),
    [anon_sym_i32] = ACTIONS(2919),
    [anon_sym_u64] = ACTIONS(2919),
    [anon_sym_i64] = ACTIONS(2919),
    [anon_sym_u128] = ACTIONS(2919),
    [anon_sym_i128] = ACTIONS(2919),
    [anon_sym_isize] = ACTIONS(2919),
    [anon_sym_usize] = ACTIONS(2919),
    [anon_sym_f32] = ACTIONS(2919),
    [anon_sym_f64] = ACTIONS(2919),
    [anon_sym_bool] = ACTIONS(2919),
    [anon_sym_str] = ACTIONS(2919),
    [anon_sym_char] = ACTIONS(2919),
    [anon_sym_DASH] = ACTIONS(2917),
    [anon_sym_BANG] = ACTIONS(2917),
    [anon_sym_AMP] = ACTIONS(2917),
    [anon_sym_PIPE] = ACTIONS(2917),
    [anon_sym_LT] = ACTIONS(2917),
    [anon_sym_DOT_DOT] = ACTIONS(2917),
    [anon_sym_COLON_COLON] = ACTIONS(2917),
    [anon_sym_POUND] = ACTIONS(2917),
    [anon_sym_SQUOTE] = ACTIONS(2919),
    [anon_sym_async] = ACTIONS(2919),
    [anon_sym_break] = ACTIONS(2919),
    [anon_sym_const] = ACTIONS(2919),
    [anon_sym_continue] = ACTIONS(2919),
    [anon_sym_default] = ACTIONS(2919),
    [anon_sym_enum] = ACTIONS(2919),
    [anon_sym_fn] = ACTIONS(2919),
    [anon_sym_for] = ACTIONS(2919),
    [anon_sym_gen] = ACTIONS(2919),
    [anon_sym_if] = ACTIONS(2919),
    [anon_sym_impl] = ACTIONS(2919),
    [anon_sym_let] = ACTIONS(2919),
    [anon_sym_loop] = ACTIONS(2919),
    [anon_sym_match] = ACTIONS(2919),
    [anon_sym_mod] = ACTIONS(2919),
    [anon_sym_pub] = ACTIONS(2919),
    [anon_sym_return] = ACTIONS(2919),
    [anon_sym_static] = ACTIONS(2919),
    [anon_sym_struct] = ACTIONS(2919),
    [anon_sym_trait] = ACTIONS(2919),
    [anon_sym_type] = ACTIONS(2919),
    [anon_sym_union] = ACTIONS(2919),
    [anon_sym_unsafe] = ACTIONS(2919),
    [anon_sym_use] = ACTIONS(2919),
    [anon_sym_while] = ACTIONS(2919),
    [anon_sym_extern] = ACTIONS(2919),
    [anon_sym_yield] = ACTIONS(2919),
    [anon_sym_move] = ACTIONS(2919),
    [anon_sym_try] = ACTIONS(2919),
    [sym_integer_literal] = ACTIONS(2917),
    [aux_sym_string_literal_token1] = ACTIONS(2917),
    [sym_char_literal] = ACTIONS(2917),
    [anon_sym_true] = ACTIONS(2919),
    [anon_sym_false] = ACTIONS(2919),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2919),
    [sym_super] = ACTIONS(2919),
    [sym_crate] = ACTIONS(2919),
    [sym_metavariable] = ACTIONS(2917),
    [sym__raw_string_literal_start] = ACTIONS(2917),
    [sym_float_literal] = ACTIONS(2917),
  },
  [752] = {
    [sym_line_comment] = STATE(752),
    [sym_block_comment] = STATE(752),
    [ts_builtin_sym_end] = ACTIONS(2921),
    [sym_identifier] = ACTIONS(2923),
    [anon_sym_SEMI] = ACTIONS(2921),
    [anon_sym_macro_rules_BANG] = ACTIONS(2921),
    [anon_sym_LPAREN] = ACTIONS(2921),
    [anon_sym_LBRACK] = ACTIONS(2921),
    [anon_sym_LBRACE] = ACTIONS(2921),
    [anon_sym_RBRACE] = ACTIONS(2921),
    [anon_sym_STAR] = ACTIONS(2921),
    [anon_sym_u8] = ACTIONS(2923),
    [anon_sym_i8] = ACTIONS(2923),
    [anon_sym_u16] = ACTIONS(2923),
    [anon_sym_i16] = ACTIONS(2923),
    [anon_sym_u32] = ACTIONS(2923),
    [anon_sym_i32] = ACTIONS(2923),
    [anon_sym_u64] = ACTIONS(2923),
    [anon_sym_i64] = ACTIONS(2923),
    [anon_sym_u128] = ACTIONS(2923),
    [anon_sym_i128] = ACTIONS(2923),
    [anon_sym_isize] = ACTIONS(2923),
    [anon_sym_usize] = ACTIONS(2923),
    [anon_sym_f32] = ACTIONS(2923),
    [anon_sym_f64] = ACTIONS(2923),
    [anon_sym_bool] = ACTIONS(2923),
    [anon_sym_str] = ACTIONS(2923),
    [anon_sym_char] = ACTIONS(2923),
    [anon_sym_DASH] = ACTIONS(2921),
    [anon_sym_BANG] = ACTIONS(2921),
    [anon_sym_AMP] = ACTIONS(2921),
    [anon_sym_PIPE] = ACTIONS(2921),
    [anon_sym_LT] = ACTIONS(2921),
    [anon_sym_DOT_DOT] = ACTIONS(2921),
    [anon_sym_COLON_COLON] = ACTIONS(2921),
    [anon_sym_POUND] = ACTIONS(2921),
    [anon_sym_SQUOTE] = ACTIONS(2923),
    [anon_sym_async] = ACTIONS(2923),
    [anon_sym_break] = ACTIONS(2923),
    [anon_sym_const] = ACTIONS(2923),
    [anon_sym_continue] = ACTIONS(2923),
    [anon_sym_default] = ACTIONS(2923),
    [anon_sym_enum] = ACTIONS(2923),
    [anon_sym_fn] = ACTIONS(2923),
    [anon_sym_for] = ACTIONS(2923),
    [anon_sym_gen] = ACTIONS(2923),
    [anon_sym_if] = ACTIONS(2923),
    [anon_sym_impl] = ACTIONS(2923),
    [anon_sym_let] = ACTIONS(2923),
    [anon_sym_loop] = ACTIONS(2923),
    [anon_sym_match] = ACTIONS(2923),
    [anon_sym_mod] = ACTIONS(2923),
    [anon_sym_pub] = ACTIONS(2923),
    [anon_sym_return] = ACTIONS(2923),
    [anon_sym_static] = ACTIONS(2923),
    [anon_sym_struct] = ACTIONS(2923),
    [anon_sym_trait] = ACTIONS(2923),
    [anon_sym_type] = ACTIONS(2923),
    [anon_sym_union] = ACTIONS(2923),
    [anon_sym_unsafe] = ACTIONS(2923),
    [anon_sym_use] = ACTIONS(2923),
    [anon_sym_while] = ACTIONS(2923),
    [anon_sym_extern] = ACTIONS(2923),
    [anon_sym_yield] = ACTIONS(2923),
    [anon_sym_move] = ACTIONS(2923),
    [anon_sym_try] = ACTIONS(2923),
    [sym_integer_literal] = ACTIONS(2921),
    [aux_sym_string_literal_token1] = ACTIONS(2921),
    [sym_char_literal] = ACTIONS(2921),
    [anon_sym_true] = ACTIONS(2923),
    [anon_sym_false] = ACTIONS(2923),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2923),
    [sym_super] = ACTIONS(2923),
    [sym_crate] = ACTIONS(2923),
    [sym_metavariable] = ACTIONS(2921),
    [sym__raw_string_literal_start] = ACTIONS(2921),
    [sym_float_literal] = ACTIONS(2921),
  },
  [753] = {
    [sym_line_comment] = STATE(753),
    [sym_block_comment] = STATE(753),
    [ts_builtin_sym_end] = ACTIONS(2925),
    [sym_identifier] = ACTIONS(2927),
    [anon_sym_SEMI] = ACTIONS(2925),
    [anon_sym_macro_rules_BANG] = ACTIONS(2925),
    [anon_sym_LPAREN] = ACTIONS(2925),
    [anon_sym_LBRACK] = ACTIONS(2925),
    [anon_sym_LBRACE] = ACTIONS(2925),
    [anon_sym_RBRACE] = ACTIONS(2925),
    [anon_sym_STAR] = ACTIONS(2925),
    [anon_sym_u8] = ACTIONS(2927),
    [anon_sym_i8] = ACTIONS(2927),
    [anon_sym_u16] = ACTIONS(2927),
    [anon_sym_i16] = ACTIONS(2927),
    [anon_sym_u32] = ACTIONS(2927),
    [anon_sym_i32] = ACTIONS(2927),
    [anon_sym_u64] = ACTIONS(2927),
    [anon_sym_i64] = ACTIONS(2927),
    [anon_sym_u128] = ACTIONS(2927),
    [anon_sym_i128] = ACTIONS(2927),
    [anon_sym_isize] = ACTIONS(2927),
    [anon_sym_usize] = ACTIONS(2927),
    [anon_sym_f32] = ACTIONS(2927),
    [anon_sym_f64] = ACTIONS(2927),
    [anon_sym_bool] = ACTIONS(2927),
    [anon_sym_str] = ACTIONS(2927),
    [anon_sym_char] = ACTIONS(2927),
    [anon_sym_DASH] = ACTIONS(2925),
    [anon_sym_BANG] = ACTIONS(2925),
    [anon_sym_AMP] = ACTIONS(2925),
    [anon_sym_PIPE] = ACTIONS(2925),
    [anon_sym_LT] = ACTIONS(2925),
    [anon_sym_DOT_DOT] = ACTIONS(2925),
    [anon_sym_COLON_COLON] = ACTIONS(2925),
    [anon_sym_POUND] = ACTIONS(2925),
    [anon_sym_SQUOTE] = ACTIONS(2927),
    [anon_sym_async] = ACTIONS(2927),
    [anon_sym_break] = ACTIONS(2927),
    [anon_sym_const] = ACTIONS(2927),
    [anon_sym_continue] = ACTIONS(2927),
    [anon_sym_default] = ACTIONS(2927),
    [anon_sym_enum] = ACTIONS(2927),
    [anon_sym_fn] = ACTIONS(2927),
    [anon_sym_for] = ACTIONS(2927),
    [anon_sym_gen] = ACTIONS(2927),
    [anon_sym_if] = ACTIONS(2927),
    [anon_sym_impl] = ACTIONS(2927),
    [anon_sym_let] = ACTIONS(2927),
    [anon_sym_loop] = ACTIONS(2927),
    [anon_sym_match] = ACTIONS(2927),
    [anon_sym_mod] = ACTIONS(2927),
    [anon_sym_pub] = ACTIONS(2927),
    [anon_sym_return] = ACTIONS(2927),
    [anon_sym_static] = ACTIONS(2927),
    [anon_sym_struct] = ACTIONS(2927),
    [anon_sym_trait] = ACTIONS(2927),
    [anon_sym_type] = ACTIONS(2927),
    [anon_sym_union] = ACTIONS(2927),
    [anon_sym_unsafe] = ACTIONS(2927),
    [anon_sym_use] = ACTIONS(2927),
    [anon_sym_while] = ACTIONS(2927),
    [anon_sym_extern] = ACTIONS(2927),
    [anon_sym_yield] = ACTIONS(2927),
    [anon_sym_move] = ACTIONS(2927),
    [anon_sym_try] = ACTIONS(2927),
    [sym_integer_literal] = ACTIONS(2925),
    [aux_sym_string_literal_token1] = ACTIONS(2925),
    [sym_char_literal] = ACTIONS(2925),
    [anon_sym_true] = ACTIONS(2927),
    [anon_sym_false] = ACTIONS(2927),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2927),
    [sym_super] = ACTIONS(2927),
    [sym_crate] = ACTIONS(2927),
    [sym_metavariable] = ACTIONS(2925),
    [sym__raw_string_literal_start] = ACTIONS(2925),
    [sym_float_literal] = ACTIONS(2925),
  },
  [754] = {
    [sym_line_comment] = STATE(754),
    [sym_block_comment] = STATE(754),
    [ts_builtin_sym_end] = ACTIONS(2929),
    [sym_identifier] = ACTIONS(2931),
    [anon_sym_SEMI] = ACTIONS(2929),
    [anon_sym_macro_rules_BANG] = ACTIONS(2929),
    [anon_sym_LPAREN] = ACTIONS(2929),
    [anon_sym_LBRACK] = ACTIONS(2929),
    [anon_sym_LBRACE] = ACTIONS(2929),
    [anon_sym_RBRACE] = ACTIONS(2929),
    [anon_sym_STAR] = ACTIONS(2929),
    [anon_sym_u8] = ACTIONS(2931),
    [anon_sym_i8] = ACTIONS(2931),
    [anon_sym_u16] = ACTIONS(2931),
    [anon_sym_i16] = ACTIONS(2931),
    [anon_sym_u32] = ACTIONS(2931),
    [anon_sym_i32] = ACTIONS(2931),
    [anon_sym_u64] = ACTIONS(2931),
    [anon_sym_i64] = ACTIONS(2931),
    [anon_sym_u128] = ACTIONS(2931),
    [anon_sym_i128] = ACTIONS(2931),
    [anon_sym_isize] = ACTIONS(2931),
    [anon_sym_usize] = ACTIONS(2931),
    [anon_sym_f32] = ACTIONS(2931),
    [anon_sym_f64] = ACTIONS(2931),
    [anon_sym_bool] = ACTIONS(2931),
    [anon_sym_str] = ACTIONS(2931),
    [anon_sym_char] = ACTIONS(2931),
    [anon_sym_DASH] = ACTIONS(2929),
    [anon_sym_BANG] = ACTIONS(2929),
    [anon_sym_AMP] = ACTIONS(2929),
    [anon_sym_PIPE] = ACTIONS(2929),
    [anon_sym_LT] = ACTIONS(2929),
    [anon_sym_DOT_DOT] = ACTIONS(2929),
    [anon_sym_COLON_COLON] = ACTIONS(2929),
    [anon_sym_POUND] = ACTIONS(2929),
    [anon_sym_SQUOTE] = ACTIONS(2931),
    [anon_sym_async] = ACTIONS(2931),
    [anon_sym_break] = ACTIONS(2931),
    [anon_sym_const] = ACTIONS(2931),
    [anon_sym_continue] = ACTIONS(2931),
    [anon_sym_default] = ACTIONS(2931),
    [anon_sym_enum] = ACTIONS(2931),
    [anon_sym_fn] = ACTIONS(2931),
    [anon_sym_for] = ACTIONS(2931),
    [anon_sym_gen] = ACTIONS(2931),
    [anon_sym_if] = ACTIONS(2931),
    [anon_sym_impl] = ACTIONS(2931),
    [anon_sym_let] = ACTIONS(2931),
    [anon_sym_loop] = ACTIONS(2931),
    [anon_sym_match] = ACTIONS(2931),
    [anon_sym_mod] = ACTIONS(2931),
    [anon_sym_pub] = ACTIONS(2931),
    [anon_sym_return] = ACTIONS(2931),
    [anon_sym_static] = ACTIONS(2931),
    [anon_sym_struct] = ACTIONS(2931),
    [anon_sym_trait] = ACTIONS(2931),
    [anon_sym_type] = ACTIONS(2931),
    [anon_sym_union] = ACTIONS(2931),
    [anon_sym_unsafe] = ACTIONS(2931),
    [anon_sym_use] = ACTIONS(2931),
    [anon_sym_while] = ACTIONS(2931),
    [anon_sym_extern] = ACTIONS(2931),
    [anon_sym_yield] = ACTIONS(2931),
    [anon_sym_move] = ACTIONS(2931),
    [anon_sym_try] = ACTIONS(2931),
    [sym_integer_literal] = ACTIONS(2929),
    [aux_sym_string_literal_token1] = ACTIONS(2929),
    [sym_char_literal] = ACTIONS(2929),
    [anon_sym_true] = ACTIONS(2931),
    [anon_sym_false] = ACTIONS(2931),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2931),
    [sym_super] = ACTIONS(2931),
    [sym_crate] = ACTIONS(2931),
    [sym_metavariable] = ACTIONS(2929),
    [sym__raw_string_literal_start] = ACTIONS(2929),
    [sym_float_literal] = ACTIONS(2929),
  },
  [755] = {
    [sym_line_comment] = STATE(755),
    [sym_block_comment] = STATE(755),
    [ts_builtin_sym_end] = ACTIONS(2933),
    [sym_identifier] = ACTIONS(2935),
    [anon_sym_SEMI] = ACTIONS(2933),
    [anon_sym_macro_rules_BANG] = ACTIONS(2933),
    [anon_sym_LPAREN] = ACTIONS(2933),
    [anon_sym_LBRACK] = ACTIONS(2933),
    [anon_sym_LBRACE] = ACTIONS(2933),
    [anon_sym_RBRACE] = ACTIONS(2933),
    [anon_sym_STAR] = ACTIONS(2933),
    [anon_sym_u8] = ACTIONS(2935),
    [anon_sym_i8] = ACTIONS(2935),
    [anon_sym_u16] = ACTIONS(2935),
    [anon_sym_i16] = ACTIONS(2935),
    [anon_sym_u32] = ACTIONS(2935),
    [anon_sym_i32] = ACTIONS(2935),
    [anon_sym_u64] = ACTIONS(2935),
    [anon_sym_i64] = ACTIONS(2935),
    [anon_sym_u128] = ACTIONS(2935),
    [anon_sym_i128] = ACTIONS(2935),
    [anon_sym_isize] = ACTIONS(2935),
    [anon_sym_usize] = ACTIONS(2935),
    [anon_sym_f32] = ACTIONS(2935),
    [anon_sym_f64] = ACTIONS(2935),
    [anon_sym_bool] = ACTIONS(2935),
    [anon_sym_str] = ACTIONS(2935),
    [anon_sym_char] = ACTIONS(2935),
    [anon_sym_DASH] = ACTIONS(2933),
    [anon_sym_BANG] = ACTIONS(2933),
    [anon_sym_AMP] = ACTIONS(2933),
    [anon_sym_PIPE] = ACTIONS(2933),
    [anon_sym_LT] = ACTIONS(2933),
    [anon_sym_DOT_DOT] = ACTIONS(2933),
    [anon_sym_COLON_COLON] = ACTIONS(2933),
    [anon_sym_POUND] = ACTIONS(2933),
    [anon_sym_SQUOTE] = ACTIONS(2935),
    [anon_sym_async] = ACTIONS(2935),
    [anon_sym_break] = ACTIONS(2935),
    [anon_sym_const] = ACTIONS(2935),
    [anon_sym_continue] = ACTIONS(2935),
    [anon_sym_default] = ACTIONS(2935),
    [anon_sym_enum] = ACTIONS(2935),
    [anon_sym_fn] = ACTIONS(2935),
    [anon_sym_for] = ACTIONS(2935),
    [anon_sym_gen] = ACTIONS(2935),
    [anon_sym_if] = ACTIONS(2935),
    [anon_sym_impl] = ACTIONS(2935),
    [anon_sym_let] = ACTIONS(2935),
    [anon_sym_loop] = ACTIONS(2935),
    [anon_sym_match] = ACTIONS(2935),
    [anon_sym_mod] = ACTIONS(2935),
    [anon_sym_pub] = ACTIONS(2935),
    [anon_sym_return] = ACTIONS(2935),
    [anon_sym_static] = ACTIONS(2935),
    [anon_sym_struct] = ACTIONS(2935),
    [anon_sym_trait] = ACTIONS(2935),
    [anon_sym_type] = ACTIONS(2935),
    [anon_sym_union] = ACTIONS(2935),
    [anon_sym_unsafe] = ACTIONS(2935),
    [anon_sym_use] = ACTIONS(2935),
    [anon_sym_while] = ACTIONS(2935),
    [anon_sym_extern] = ACTIONS(2935),
    [anon_sym_yield] = ACTIONS(2935),
    [anon_sym_move] = ACTIONS(2935),
    [anon_sym_try] = ACTIONS(2935),
    [sym_integer_literal] = ACTIONS(2933),
    [aux_sym_string_literal_token1] = ACTIONS(2933),
    [sym_char_literal] = ACTIONS(2933),
    [anon_sym_true] = ACTIONS(2935),
    [anon_sym_false] = ACTIONS(2935),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2935),
    [sym_super] = ACTIONS(2935),
    [sym_crate] = ACTIONS(2935),
    [sym_metavariable] = ACTIONS(2933),
    [sym__raw_string_literal_start] = ACTIONS(2933),
    [sym_float_literal] = ACTIONS(2933),
  },
  [756] = {
    [sym_line_comment] = STATE(756),
    [sym_block_comment] = STATE(756),
    [ts_builtin_sym_end] = ACTIONS(2937),
    [sym_identifier] = ACTIONS(2939),
    [anon_sym_SEMI] = ACTIONS(2937),
    [anon_sym_macro_rules_BANG] = ACTIONS(2937),
    [anon_sym_LPAREN] = ACTIONS(2937),
    [anon_sym_LBRACK] = ACTIONS(2937),
    [anon_sym_LBRACE] = ACTIONS(2937),
    [anon_sym_RBRACE] = ACTIONS(2937),
    [anon_sym_STAR] = ACTIONS(2937),
    [anon_sym_u8] = ACTIONS(2939),
    [anon_sym_i8] = ACTIONS(2939),
    [anon_sym_u16] = ACTIONS(2939),
    [anon_sym_i16] = ACTIONS(2939),
    [anon_sym_u32] = ACTIONS(2939),
    [anon_sym_i32] = ACTIONS(2939),
    [anon_sym_u64] = ACTIONS(2939),
    [anon_sym_i64] = ACTIONS(2939),
    [anon_sym_u128] = ACTIONS(2939),
    [anon_sym_i128] = ACTIONS(2939),
    [anon_sym_isize] = ACTIONS(2939),
    [anon_sym_usize] = ACTIONS(2939),
    [anon_sym_f32] = ACTIONS(2939),
    [anon_sym_f64] = ACTIONS(2939),
    [anon_sym_bool] = ACTIONS(2939),
    [anon_sym_str] = ACTIONS(2939),
    [anon_sym_char] = ACTIONS(2939),
    [anon_sym_DASH] = ACTIONS(2937),
    [anon_sym_BANG] = ACTIONS(2937),
    [anon_sym_AMP] = ACTIONS(2937),
    [anon_sym_PIPE] = ACTIONS(2937),
    [anon_sym_LT] = ACTIONS(2937),
    [anon_sym_DOT_DOT] = ACTIONS(2937),
    [anon_sym_COLON_COLON] = ACTIONS(2937),
    [anon_sym_POUND] = ACTIONS(2937),
    [anon_sym_SQUOTE] = ACTIONS(2939),
    [anon_sym_async] = ACTIONS(2939),
    [anon_sym_break] = ACTIONS(2939),
    [anon_sym_const] = ACTIONS(2939),
    [anon_sym_continue] = ACTIONS(2939),
    [anon_sym_default] = ACTIONS(2939),
    [anon_sym_enum] = ACTIONS(2939),
    [anon_sym_fn] = ACTIONS(2939),
    [anon_sym_for] = ACTIONS(2939),
    [anon_sym_gen] = ACTIONS(2939),
    [anon_sym_if] = ACTIONS(2939),
    [anon_sym_impl] = ACTIONS(2939),
    [anon_sym_let] = ACTIONS(2939),
    [anon_sym_loop] = ACTIONS(2939),
    [anon_sym_match] = ACTIONS(2939),
    [anon_sym_mod] = ACTIONS(2939),
    [anon_sym_pub] = ACTIONS(2939),
    [anon_sym_return] = ACTIONS(2939),
    [anon_sym_static] = ACTIONS(2939),
    [anon_sym_struct] = ACTIONS(2939),
    [anon_sym_trait] = ACTIONS(2939),
    [anon_sym_type] = ACTIONS(2939),
    [anon_sym_union] = ACTIONS(2939),
    [anon_sym_unsafe] = ACTIONS(2939),
    [anon_sym_use] = ACTIONS(2939),
    [anon_sym_while] = ACTIONS(2939),
    [anon_sym_extern] = ACTIONS(2939),
    [anon_sym_yield] = ACTIONS(2939),
    [anon_sym_move] = ACTIONS(2939),
    [anon_sym_try] = ACTIONS(2939),
    [sym_integer_literal] = ACTIONS(2937),
    [aux_sym_string_literal_token1] = ACTIONS(2937),
    [sym_char_literal] = ACTIONS(2937),
    [anon_sym_true] = ACTIONS(2939),
    [anon_sym_false] = ACTIONS(2939),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2939),
    [sym_super] = ACTIONS(2939),
    [sym_crate] = ACTIONS(2939),
    [sym_metavariable] = ACTIONS(2937),
    [sym__raw_string_literal_start] = ACTIONS(2937),
    [sym_float_literal] = ACTIONS(2937),
  },
  [757] = {
    [sym_line_comment] = STATE(757),
    [sym_block_comment] = STATE(757),
    [ts_builtin_sym_end] = ACTIONS(2941),
    [sym_identifier] = ACTIONS(2943),
    [anon_sym_SEMI] = ACTIONS(2941),
    [anon_sym_macro_rules_BANG] = ACTIONS(2941),
    [anon_sym_LPAREN] = ACTIONS(2941),
    [anon_sym_LBRACK] = ACTIONS(2941),
    [anon_sym_LBRACE] = ACTIONS(2941),
    [anon_sym_RBRACE] = ACTIONS(2941),
    [anon_sym_STAR] = ACTIONS(2941),
    [anon_sym_u8] = ACTIONS(2943),
    [anon_sym_i8] = ACTIONS(2943),
    [anon_sym_u16] = ACTIONS(2943),
    [anon_sym_i16] = ACTIONS(2943),
    [anon_sym_u32] = ACTIONS(2943),
    [anon_sym_i32] = ACTIONS(2943),
    [anon_sym_u64] = ACTIONS(2943),
    [anon_sym_i64] = ACTIONS(2943),
    [anon_sym_u128] = ACTIONS(2943),
    [anon_sym_i128] = ACTIONS(2943),
    [anon_sym_isize] = ACTIONS(2943),
    [anon_sym_usize] = ACTIONS(2943),
    [anon_sym_f32] = ACTIONS(2943),
    [anon_sym_f64] = ACTIONS(2943),
    [anon_sym_bool] = ACTIONS(2943),
    [anon_sym_str] = ACTIONS(2943),
    [anon_sym_char] = ACTIONS(2943),
    [anon_sym_DASH] = ACTIONS(2941),
    [anon_sym_BANG] = ACTIONS(2941),
    [anon_sym_AMP] = ACTIONS(2941),
    [anon_sym_PIPE] = ACTIONS(2941),
    [anon_sym_LT] = ACTIONS(2941),
    [anon_sym_DOT_DOT] = ACTIONS(2941),
    [anon_sym_COLON_COLON] = ACTIONS(2941),
    [anon_sym_POUND] = ACTIONS(2941),
    [anon_sym_SQUOTE] = ACTIONS(2943),
    [anon_sym_async] = ACTIONS(2943),
    [anon_sym_break] = ACTIONS(2943),
    [anon_sym_const] = ACTIONS(2943),
    [anon_sym_continue] = ACTIONS(2943),
    [anon_sym_default] = ACTIONS(2943),
    [anon_sym_enum] = ACTIONS(2943),
    [anon_sym_fn] = ACTIONS(2943),
    [anon_sym_for] = ACTIONS(2943),
    [anon_sym_gen] = ACTIONS(2943),
    [anon_sym_if] = ACTIONS(2943),
    [anon_sym_impl] = ACTIONS(2943),
    [anon_sym_let] = ACTIONS(2943),
    [anon_sym_loop] = ACTIONS(2943),
    [anon_sym_match] = ACTIONS(2943),
    [anon_sym_mod] = ACTIONS(2943),
    [anon_sym_pub] = ACTIONS(2943),
    [anon_sym_return] = ACTIONS(2943),
    [anon_sym_static] = ACTIONS(2943),
    [anon_sym_struct] = ACTIONS(2943),
    [anon_sym_trait] = ACTIONS(2943),
    [anon_sym_type] = ACTIONS(2943),
    [anon_sym_union] = ACTIONS(2943),
    [anon_sym_unsafe] = ACTIONS(2943),
    [anon_sym_use] = ACTIONS(2943),
    [anon_sym_while] = ACTIONS(2943),
    [anon_sym_extern] = ACTIONS(2943),
    [anon_sym_yield] = ACTIONS(2943),
    [anon_sym_move] = ACTIONS(2943),
    [anon_sym_try] = ACTIONS(2943),
    [sym_integer_literal] = ACTIONS(2941),
    [aux_sym_string_literal_token1] = ACTIONS(2941),
    [sym_char_literal] = ACTIONS(2941),
    [anon_sym_true] = ACTIONS(2943),
    [anon_sym_false] = ACTIONS(2943),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2943),
    [sym_super] = ACTIONS(2943),
    [sym_crate] = ACTIONS(2943),
    [sym_metavariable] = ACTIONS(2941),
    [sym__raw_string_literal_start] = ACTIONS(2941),
    [sym_float_literal] = ACTIONS(2941),
  },
  [758] = {
    [sym_line_comment] = STATE(758),
    [sym_block_comment] = STATE(758),
    [ts_builtin_sym_end] = ACTIONS(2945),
    [sym_identifier] = ACTIONS(2947),
    [anon_sym_SEMI] = ACTIONS(2945),
    [anon_sym_macro_rules_BANG] = ACTIONS(2945),
    [anon_sym_LPAREN] = ACTIONS(2945),
    [anon_sym_LBRACK] = ACTIONS(2945),
    [anon_sym_LBRACE] = ACTIONS(2945),
    [anon_sym_RBRACE] = ACTIONS(2945),
    [anon_sym_STAR] = ACTIONS(2945),
    [anon_sym_u8] = ACTIONS(2947),
    [anon_sym_i8] = ACTIONS(2947),
    [anon_sym_u16] = ACTIONS(2947),
    [anon_sym_i16] = ACTIONS(2947),
    [anon_sym_u32] = ACTIONS(2947),
    [anon_sym_i32] = ACTIONS(2947),
    [anon_sym_u64] = ACTIONS(2947),
    [anon_sym_i64] = ACTIONS(2947),
    [anon_sym_u128] = ACTIONS(2947),
    [anon_sym_i128] = ACTIONS(2947),
    [anon_sym_isize] = ACTIONS(2947),
    [anon_sym_usize] = ACTIONS(2947),
    [anon_sym_f32] = ACTIONS(2947),
    [anon_sym_f64] = ACTIONS(2947),
    [anon_sym_bool] = ACTIONS(2947),
    [anon_sym_str] = ACTIONS(2947),
    [anon_sym_char] = ACTIONS(2947),
    [anon_sym_DASH] = ACTIONS(2945),
    [anon_sym_BANG] = ACTIONS(2945),
    [anon_sym_AMP] = ACTIONS(2945),
    [anon_sym_PIPE] = ACTIONS(2945),
    [anon_sym_LT] = ACTIONS(2945),
    [anon_sym_DOT_DOT] = ACTIONS(2945),
    [anon_sym_COLON_COLON] = ACTIONS(2945),
    [anon_sym_POUND] = ACTIONS(2945),
    [anon_sym_SQUOTE] = ACTIONS(2947),
    [anon_sym_async] = ACTIONS(2947),
    [anon_sym_break] = ACTIONS(2947),
    [anon_sym_const] = ACTIONS(2947),
    [anon_sym_continue] = ACTIONS(2947),
    [anon_sym_default] = ACTIONS(2947),
    [anon_sym_enum] = ACTIONS(2947),
    [anon_sym_fn] = ACTIONS(2947),
    [anon_sym_for] = ACTIONS(2947),
    [anon_sym_gen] = ACTIONS(2947),
    [anon_sym_if] = ACTIONS(2947),
    [anon_sym_impl] = ACTIONS(2947),
    [anon_sym_let] = ACTIONS(2947),
    [anon_sym_loop] = ACTIONS(2947),
    [anon_sym_match] = ACTIONS(2947),
    [anon_sym_mod] = ACTIONS(2947),
    [anon_sym_pub] = ACTIONS(2947),
    [anon_sym_return] = ACTIONS(2947),
    [anon_sym_static] = ACTIONS(2947),
    [anon_sym_struct] = ACTIONS(2947),
    [anon_sym_trait] = ACTIONS(2947),
    [anon_sym_type] = ACTIONS(2947),
    [anon_sym_union] = ACTIONS(2947),
    [anon_sym_unsafe] = ACTIONS(2947),
    [anon_sym_use] = ACTIONS(2947),
    [anon_sym_while] = ACTIONS(2947),
    [anon_sym_extern] = ACTIONS(2947),
    [anon_sym_yield] = ACTIONS(2947),
    [anon_sym_move] = ACTIONS(2947),
    [anon_sym_try] = ACTIONS(2947),
    [sym_integer_literal] = ACTIONS(2945),
    [aux_sym_string_literal_token1] = ACTIONS(2945),
    [sym_char_literal] = ACTIONS(2945),
    [anon_sym_true] = ACTIONS(2947),
    [anon_sym_false] = ACTIONS(2947),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2947),
    [sym_super] = ACTIONS(2947),
    [sym_crate] = ACTIONS(2947),
    [sym_metavariable] = ACTIONS(2945),
    [sym__raw_string_literal_start] = ACTIONS(2945),
    [sym_float_literal] = ACTIONS(2945),
  },
  [759] = {
    [sym_line_comment] = STATE(759),
    [sym_block_comment] = STATE(759),
    [ts_builtin_sym_end] = ACTIONS(2949),
    [sym_identifier] = ACTIONS(2951),
    [anon_sym_SEMI] = ACTIONS(2949),
    [anon_sym_macro_rules_BANG] = ACTIONS(2949),
    [anon_sym_LPAREN] = ACTIONS(2949),
    [anon_sym_LBRACK] = ACTIONS(2949),
    [anon_sym_LBRACE] = ACTIONS(2949),
    [anon_sym_RBRACE] = ACTIONS(2949),
    [anon_sym_STAR] = ACTIONS(2949),
    [anon_sym_u8] = ACTIONS(2951),
    [anon_sym_i8] = ACTIONS(2951),
    [anon_sym_u16] = ACTIONS(2951),
    [anon_sym_i16] = ACTIONS(2951),
    [anon_sym_u32] = ACTIONS(2951),
    [anon_sym_i32] = ACTIONS(2951),
    [anon_sym_u64] = ACTIONS(2951),
    [anon_sym_i64] = ACTIONS(2951),
    [anon_sym_u128] = ACTIONS(2951),
    [anon_sym_i128] = ACTIONS(2951),
    [anon_sym_isize] = ACTIONS(2951),
    [anon_sym_usize] = ACTIONS(2951),
    [anon_sym_f32] = ACTIONS(2951),
    [anon_sym_f64] = ACTIONS(2951),
    [anon_sym_bool] = ACTIONS(2951),
    [anon_sym_str] = ACTIONS(2951),
    [anon_sym_char] = ACTIONS(2951),
    [anon_sym_DASH] = ACTIONS(2949),
    [anon_sym_BANG] = ACTIONS(2949),
    [anon_sym_AMP] = ACTIONS(2949),
    [anon_sym_PIPE] = ACTIONS(2949),
    [anon_sym_LT] = ACTIONS(2949),
    [anon_sym_DOT_DOT] = ACTIONS(2949),
    [anon_sym_COLON_COLON] = ACTIONS(2949),
    [anon_sym_POUND] = ACTIONS(2949),
    [anon_sym_SQUOTE] = ACTIONS(2951),
    [anon_sym_async] = ACTIONS(2951),
    [anon_sym_break] = ACTIONS(2951),
    [anon_sym_const] = ACTIONS(2951),
    [anon_sym_continue] = ACTIONS(2951),
    [anon_sym_default] = ACTIONS(2951),
    [anon_sym_enum] = ACTIONS(2951),
    [anon_sym_fn] = ACTIONS(2951),
    [anon_sym_for] = ACTIONS(2951),
    [anon_sym_gen] = ACTIONS(2951),
    [anon_sym_if] = ACTIONS(2951),
    [anon_sym_impl] = ACTIONS(2951),
    [anon_sym_let] = ACTIONS(2951),
    [anon_sym_loop] = ACTIONS(2951),
    [anon_sym_match] = ACTIONS(2951),
    [anon_sym_mod] = ACTIONS(2951),
    [anon_sym_pub] = ACTIONS(2951),
    [anon_sym_return] = ACTIONS(2951),
    [anon_sym_static] = ACTIONS(2951),
    [anon_sym_struct] = ACTIONS(2951),
    [anon_sym_trait] = ACTIONS(2951),
    [anon_sym_type] = ACTIONS(2951),
    [anon_sym_union] = ACTIONS(2951),
    [anon_sym_unsafe] = ACTIONS(2951),
    [anon_sym_use] = ACTIONS(2951),
    [anon_sym_while] = ACTIONS(2951),
    [anon_sym_extern] = ACTIONS(2951),
    [anon_sym_yield] = ACTIONS(2951),
    [anon_sym_move] = ACTIONS(2951),
    [anon_sym_try] = ACTIONS(2951),
    [sym_integer_literal] = ACTIONS(2949),
    [aux_sym_string_literal_token1] = ACTIONS(2949),
    [sym_char_literal] = ACTIONS(2949),
    [anon_sym_true] = ACTIONS(2951),
    [anon_sym_false] = ACTIONS(2951),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2951),
    [sym_super] = ACTIONS(2951),
    [sym_crate] = ACTIONS(2951),
    [sym_metavariable] = ACTIONS(2949),
    [sym__raw_string_literal_start] = ACTIONS(2949),
    [sym_float_literal] = ACTIONS(2949),
  },
  [760] = {
    [sym_line_comment] = STATE(760),
    [sym_block_comment] = STATE(760),
    [ts_builtin_sym_end] = ACTIONS(2953),
    [sym_identifier] = ACTIONS(2955),
    [anon_sym_SEMI] = ACTIONS(2953),
    [anon_sym_macro_rules_BANG] = ACTIONS(2953),
    [anon_sym_LPAREN] = ACTIONS(2953),
    [anon_sym_LBRACK] = ACTIONS(2953),
    [anon_sym_LBRACE] = ACTIONS(2953),
    [anon_sym_RBRACE] = ACTIONS(2953),
    [anon_sym_STAR] = ACTIONS(2953),
    [anon_sym_u8] = ACTIONS(2955),
    [anon_sym_i8] = ACTIONS(2955),
    [anon_sym_u16] = ACTIONS(2955),
    [anon_sym_i16] = ACTIONS(2955),
    [anon_sym_u32] = ACTIONS(2955),
    [anon_sym_i32] = ACTIONS(2955),
    [anon_sym_u64] = ACTIONS(2955),
    [anon_sym_i64] = ACTIONS(2955),
    [anon_sym_u128] = ACTIONS(2955),
    [anon_sym_i128] = ACTIONS(2955),
    [anon_sym_isize] = ACTIONS(2955),
    [anon_sym_usize] = ACTIONS(2955),
    [anon_sym_f32] = ACTIONS(2955),
    [anon_sym_f64] = ACTIONS(2955),
    [anon_sym_bool] = ACTIONS(2955),
    [anon_sym_str] = ACTIONS(2955),
    [anon_sym_char] = ACTIONS(2955),
    [anon_sym_DASH] = ACTIONS(2953),
    [anon_sym_BANG] = ACTIONS(2953),
    [anon_sym_AMP] = ACTIONS(2953),
    [anon_sym_PIPE] = ACTIONS(2953),
    [anon_sym_LT] = ACTIONS(2953),
    [anon_sym_DOT_DOT] = ACTIONS(2953),
    [anon_sym_COLON_COLON] = ACTIONS(2953),
    [anon_sym_POUND] = ACTIONS(2953),
    [anon_sym_SQUOTE] = ACTIONS(2955),
    [anon_sym_async] = ACTIONS(2955),
    [anon_sym_break] = ACTIONS(2955),
    [anon_sym_const] = ACTIONS(2955),
    [anon_sym_continue] = ACTIONS(2955),
    [anon_sym_default] = ACTIONS(2955),
    [anon_sym_enum] = ACTIONS(2955),
    [anon_sym_fn] = ACTIONS(2955),
    [anon_sym_for] = ACTIONS(2955),
    [anon_sym_gen] = ACTIONS(2955),
    [anon_sym_if] = ACTIONS(2955),
    [anon_sym_impl] = ACTIONS(2955),
    [anon_sym_let] = ACTIONS(2955),
    [anon_sym_loop] = ACTIONS(2955),
    [anon_sym_match] = ACTIONS(2955),
    [anon_sym_mod] = ACTIONS(2955),
    [anon_sym_pub] = ACTIONS(2955),
    [anon_sym_return] = ACTIONS(2955),
    [anon_sym_static] = ACTIONS(2955),
    [anon_sym_struct] = ACTIONS(2955),
    [anon_sym_trait] = ACTIONS(2955),
    [anon_sym_type] = ACTIONS(2955),
    [anon_sym_union] = ACTIONS(2955),
    [anon_sym_unsafe] = ACTIONS(2955),
    [anon_sym_use] = ACTIONS(2955),
    [anon_sym_while] = ACTIONS(2955),
    [anon_sym_extern] = ACTIONS(2955),
    [anon_sym_yield] = ACTIONS(2955),
    [anon_sym_move] = ACTIONS(2955),
    [anon_sym_try] = ACTIONS(2955),
    [sym_integer_literal] = ACTIONS(2953),
    [aux_sym_string_literal_token1] = ACTIONS(2953),
    [sym_char_literal] = ACTIONS(2953),
    [anon_sym_true] = ACTIONS(2955),
    [anon_sym_false] = ACTIONS(2955),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2955),
    [sym_super] = ACTIONS(2955),
    [sym_crate] = ACTIONS(2955),
    [sym_metavariable] = ACTIONS(2953),
    [sym__raw_string_literal_start] = ACTIONS(2953),
    [sym_float_literal] = ACTIONS(2953),
  },
  [761] = {
    [sym_line_comment] = STATE(761),
    [sym_block_comment] = STATE(761),
    [ts_builtin_sym_end] = ACTIONS(2957),
    [sym_identifier] = ACTIONS(2959),
    [anon_sym_SEMI] = ACTIONS(2957),
    [anon_sym_macro_rules_BANG] = ACTIONS(2957),
    [anon_sym_LPAREN] = ACTIONS(2957),
    [anon_sym_LBRACK] = ACTIONS(2957),
    [anon_sym_LBRACE] = ACTIONS(2957),
    [anon_sym_RBRACE] = ACTIONS(2957),
    [anon_sym_STAR] = ACTIONS(2957),
    [anon_sym_u8] = ACTIONS(2959),
    [anon_sym_i8] = ACTIONS(2959),
    [anon_sym_u16] = ACTIONS(2959),
    [anon_sym_i16] = ACTIONS(2959),
    [anon_sym_u32] = ACTIONS(2959),
    [anon_sym_i32] = ACTIONS(2959),
    [anon_sym_u64] = ACTIONS(2959),
    [anon_sym_i64] = ACTIONS(2959),
    [anon_sym_u128] = ACTIONS(2959),
    [anon_sym_i128] = ACTIONS(2959),
    [anon_sym_isize] = ACTIONS(2959),
    [anon_sym_usize] = ACTIONS(2959),
    [anon_sym_f32] = ACTIONS(2959),
    [anon_sym_f64] = ACTIONS(2959),
    [anon_sym_bool] = ACTIONS(2959),
    [anon_sym_str] = ACTIONS(2959),
    [anon_sym_char] = ACTIONS(2959),
    [anon_sym_DASH] = ACTIONS(2957),
    [anon_sym_BANG] = ACTIONS(2957),
    [anon_sym_AMP] = ACTIONS(2957),
    [anon_sym_PIPE] = ACTIONS(2957),
    [anon_sym_LT] = ACTIONS(2957),
    [anon_sym_DOT_DOT] = ACTIONS(2957),
    [anon_sym_COLON_COLON] = ACTIONS(2957),
    [anon_sym_POUND] = ACTIONS(2957),
    [anon_sym_SQUOTE] = ACTIONS(2959),
    [anon_sym_async] = ACTIONS(2959),
    [anon_sym_break] = ACTIONS(2959),
    [anon_sym_const] = ACTIONS(2959),
    [anon_sym_continue] = ACTIONS(2959),
    [anon_sym_default] = ACTIONS(2959),
    [anon_sym_enum] = ACTIONS(2959),
    [anon_sym_fn] = ACTIONS(2959),
    [anon_sym_for] = ACTIONS(2959),
    [anon_sym_gen] = ACTIONS(2959),
    [anon_sym_if] = ACTIONS(2959),
    [anon_sym_impl] = ACTIONS(2959),
    [anon_sym_let] = ACTIONS(2959),
    [anon_sym_loop] = ACTIONS(2959),
    [anon_sym_match] = ACTIONS(2959),
    [anon_sym_mod] = ACTIONS(2959),
    [anon_sym_pub] = ACTIONS(2959),
    [anon_sym_return] = ACTIONS(2959),
    [anon_sym_static] = ACTIONS(2959),
    [anon_sym_struct] = ACTIONS(2959),
    [anon_sym_trait] = ACTIONS(2959),
    [anon_sym_type] = ACTIONS(2959),
    [anon_sym_union] = ACTIONS(2959),
    [anon_sym_unsafe] = ACTIONS(2959),
    [anon_sym_use] = ACTIONS(2959),
    [anon_sym_while] = ACTIONS(2959),
    [anon_sym_extern] = ACTIONS(2959),
    [anon_sym_yield] = ACTIONS(2959),
    [anon_sym_move] = ACTIONS(2959),
    [anon_sym_try] = ACTIONS(2959),
    [sym_integer_literal] = ACTIONS(2957),
    [aux_sym_string_literal_token1] = ACTIONS(2957),
    [sym_char_literal] = ACTIONS(2957),
    [anon_sym_true] = ACTIONS(2959),
    [anon_sym_false] = ACTIONS(2959),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2959),
    [sym_super] = ACTIONS(2959),
    [sym_crate] = ACTIONS(2959),
    [sym_metavariable] = ACTIONS(2957),
    [sym__raw_string_literal_start] = ACTIONS(2957),
    [sym_float_literal] = ACTIONS(2957),
  },
  [762] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym_closure_expression] = STATE(3019),
    [sym_closure_parameters] = STATE(220),
    [sym__pattern] = STATE(2737),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(762),
    [sym_block_comment] = STATE(762),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_RPAREN] = ACTIONS(2965),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1515),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COMMA] = ACTIONS(2973),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_static] = ACTIONS(1523),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [anon_sym_move] = ACTIONS(1527),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [763] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym_closure_expression] = STATE(2866),
    [sym_closure_parameters] = STATE(220),
    [sym__pattern] = STATE(2724),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(763),
    [sym_block_comment] = STATE(763),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_RPAREN] = ACTIONS(2985),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1515),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COMMA] = ACTIONS(1521),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_static] = ACTIONS(1523),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [anon_sym_move] = ACTIONS(1527),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [764] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym_closure_expression] = STATE(3177),
    [sym_closure_parameters] = STATE(220),
    [sym__pattern] = STATE(2962),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(764),
    [sym_block_comment] = STATE(764),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_RPAREN] = ACTIONS(2987),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1515),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_static] = ACTIONS(1523),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [anon_sym_move] = ACTIONS(1527),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [765] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym_closure_expression] = STATE(3177),
    [sym_closure_parameters] = STATE(220),
    [sym__pattern] = STATE(2962),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(765),
    [sym_block_comment] = STATE(765),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_RPAREN] = ACTIONS(2989),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1515),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_static] = ACTIONS(1523),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [anon_sym_move] = ACTIONS(1527),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [766] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym_closure_expression] = STATE(3177),
    [sym_closure_parameters] = STATE(220),
    [sym__pattern] = STATE(2962),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(766),
    [sym_block_comment] = STATE(766),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_RPAREN] = ACTIONS(2991),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1515),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_static] = ACTIONS(1523),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [anon_sym_move] = ACTIONS(1527),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [767] = {
    [sym_attribute_item] = STATE(1468),
    [sym_inner_attribute_item] = STATE(1468),
    [sym_bracketed_type] = STATE(3553),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3287),
    [sym_macro_invocation] = STATE(2757),
    [sym_scoped_identifier] = STATE(2151),
    [sym_scoped_type_identifier] = STATE(2982),
    [sym_match_pattern] = STATE(3444),
    [sym_const_block] = STATE(2757),
    [sym__pattern] = STATE(2907),
    [sym_tuple_pattern] = STATE(2757),
    [sym_slice_pattern] = STATE(2757),
    [sym_tuple_struct_pattern] = STATE(2757),
    [sym_struct_pattern] = STATE(2757),
    [sym_remaining_field_pattern] = STATE(2757),
    [sym_mut_pattern] = STATE(2757),
    [sym_range_pattern] = STATE(2757),
    [sym_ref_pattern] = STATE(2757),
    [sym_captured_pattern] = STATE(2757),
    [sym_reference_pattern] = STATE(2757),
    [sym_or_pattern] = STATE(2757),
    [sym__literal_pattern] = STATE(2330),
    [sym_negative_literal] = STATE(2364),
    [sym_string_literal] = STATE(2364),
    [sym_raw_string_literal] = STATE(2364),
    [sym_boolean_literal] = STATE(2364),
    [sym_line_comment] = STATE(767),
    [sym_block_comment] = STATE(767),
    [aux_sym_match_arm_repeat1] = STATE(1045),
    [sym_identifier] = ACTIONS(1645),
    [anon_sym_LPAREN] = ACTIONS(1647),
    [anon_sym_LBRACK] = ACTIONS(1649),
    [anon_sym_u8] = ACTIONS(1653),
    [anon_sym_i8] = ACTIONS(1653),
    [anon_sym_u16] = ACTIONS(1653),
    [anon_sym_i16] = ACTIONS(1653),
    [anon_sym_u32] = ACTIONS(1653),
    [anon_sym_i32] = ACTIONS(1653),
    [anon_sym_u64] = ACTIONS(1653),
    [anon_sym_i64] = ACTIONS(1653),
    [anon_sym_u128] = ACTIONS(1653),
    [anon_sym_i128] = ACTIONS(1653),
    [anon_sym_isize] = ACTIONS(1653),
    [anon_sym_usize] = ACTIONS(1653),
    [anon_sym_f32] = ACTIONS(1653),
    [anon_sym_f64] = ACTIONS(1653),
    [anon_sym_bool] = ACTIONS(1653),
    [anon_sym_str] = ACTIONS(1653),
    [anon_sym_char] = ACTIONS(1653),
    [anon_sym_DASH] = ACTIONS(1655),
    [anon_sym_AMP] = ACTIONS(1657),
    [anon_sym_PIPE] = ACTIONS(1659),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1661),
    [anon_sym_DOT_DOT] = ACTIONS(1663),
    [anon_sym_COLON_COLON] = ACTIONS(1665),
    [anon_sym_POUND] = ACTIONS(1667),
    [anon_sym_const] = ACTIONS(1669),
    [anon_sym_default] = ACTIONS(1671),
    [anon_sym_gen] = ACTIONS(1671),
    [anon_sym_union] = ACTIONS(1671),
    [anon_sym_ref] = ACTIONS(1673),
    [sym_mutable_specifier] = ACTIONS(1675),
    [sym_integer_literal] = ACTIONS(1677),
    [aux_sym_string_literal_token1] = ACTIONS(1679),
    [sym_char_literal] = ACTIONS(1677),
    [anon_sym_true] = ACTIONS(1681),
    [anon_sym_false] = ACTIONS(1681),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1683),
    [sym_super] = ACTIONS(1683),
    [sym_crate] = ACTIONS(1683),
    [sym_metavariable] = ACTIONS(1685),
    [sym__raw_string_literal_start] = ACTIONS(1687),
    [sym_float_literal] = ACTIONS(1677),
  },
  [768] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym_closure_expression] = STATE(3177),
    [sym_closure_parameters] = STATE(220),
    [sym__pattern] = STATE(2962),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(768),
    [sym_block_comment] = STATE(768),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_RPAREN] = ACTIONS(2993),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1515),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_static] = ACTIONS(1523),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [anon_sym_move] = ACTIONS(1527),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [769] = {
    [sym_attribute_item] = STATE(1468),
    [sym_inner_attribute_item] = STATE(1468),
    [sym_bracketed_type] = STATE(3553),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3287),
    [sym_macro_invocation] = STATE(2757),
    [sym_scoped_identifier] = STATE(2151),
    [sym_scoped_type_identifier] = STATE(2982),
    [sym_match_pattern] = STATE(3596),
    [sym_const_block] = STATE(2757),
    [sym__pattern] = STATE(2907),
    [sym_tuple_pattern] = STATE(2757),
    [sym_slice_pattern] = STATE(2757),
    [sym_tuple_struct_pattern] = STATE(2757),
    [sym_struct_pattern] = STATE(2757),
    [sym_remaining_field_pattern] = STATE(2757),
    [sym_mut_pattern] = STATE(2757),
    [sym_range_pattern] = STATE(2757),
    [sym_ref_pattern] = STATE(2757),
    [sym_captured_pattern] = STATE(2757),
    [sym_reference_pattern] = STATE(2757),
    [sym_or_pattern] = STATE(2757),
    [sym__literal_pattern] = STATE(2330),
    [sym_negative_literal] = STATE(2364),
    [sym_string_literal] = STATE(2364),
    [sym_raw_string_literal] = STATE(2364),
    [sym_boolean_literal] = STATE(2364),
    [sym_line_comment] = STATE(769),
    [sym_block_comment] = STATE(769),
    [aux_sym_match_arm_repeat1] = STATE(1045),
    [sym_identifier] = ACTIONS(1645),
    [anon_sym_LPAREN] = ACTIONS(1647),
    [anon_sym_LBRACK] = ACTIONS(1649),
    [anon_sym_u8] = ACTIONS(1653),
    [anon_sym_i8] = ACTIONS(1653),
    [anon_sym_u16] = ACTIONS(1653),
    [anon_sym_i16] = ACTIONS(1653),
    [anon_sym_u32] = ACTIONS(1653),
    [anon_sym_i32] = ACTIONS(1653),
    [anon_sym_u64] = ACTIONS(1653),
    [anon_sym_i64] = ACTIONS(1653),
    [anon_sym_u128] = ACTIONS(1653),
    [anon_sym_i128] = ACTIONS(1653),
    [anon_sym_isize] = ACTIONS(1653),
    [anon_sym_usize] = ACTIONS(1653),
    [anon_sym_f32] = ACTIONS(1653),
    [anon_sym_f64] = ACTIONS(1653),
    [anon_sym_bool] = ACTIONS(1653),
    [anon_sym_str] = ACTIONS(1653),
    [anon_sym_char] = ACTIONS(1653),
    [anon_sym_DASH] = ACTIONS(1655),
    [anon_sym_AMP] = ACTIONS(1657),
    [anon_sym_PIPE] = ACTIONS(1659),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1661),
    [anon_sym_DOT_DOT] = ACTIONS(1663),
    [anon_sym_COLON_COLON] = ACTIONS(1665),
    [anon_sym_POUND] = ACTIONS(1667),
    [anon_sym_const] = ACTIONS(1669),
    [anon_sym_default] = ACTIONS(1671),
    [anon_sym_gen] = ACTIONS(1671),
    [anon_sym_union] = ACTIONS(1671),
    [anon_sym_ref] = ACTIONS(1673),
    [sym_mutable_specifier] = ACTIONS(1675),
    [sym_integer_literal] = ACTIONS(1677),
    [aux_sym_string_literal_token1] = ACTIONS(1679),
    [sym_char_literal] = ACTIONS(1677),
    [anon_sym_true] = ACTIONS(1681),
    [anon_sym_false] = ACTIONS(1681),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1683),
    [sym_super] = ACTIONS(1683),
    [sym_crate] = ACTIONS(1683),
    [sym_metavariable] = ACTIONS(1685),
    [sym__raw_string_literal_start] = ACTIONS(1687),
    [sym_float_literal] = ACTIONS(1677),
  },
  [770] = {
    [sym_attribute_item] = STATE(1405),
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_const_parameter] = STATE(2953),
    [sym_constrained_type_parameter] = STATE(2540),
    [sym_optional_type_parameter] = STATE(2953),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2994),
    [sym_bracketed_type] = STATE(3361),
    [sym_qualified_type] = STATE(3468),
    [sym_lifetime] = STATE(2338),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(770),
    [sym_block_comment] = STATE(770),
    [aux_sym_enum_variant_list_repeat1] = STATE(2077),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(2995),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_POUND] = ACTIONS(2997),
    [anon_sym_SQUOTE] = ACTIONS(2999),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(3001),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(3003),
  },
  [771] = {
    [sym_attribute_item] = STATE(1405),
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym_visibility_modifier] = STATE(913),
    [sym__type] = STATE(2514),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(771),
    [sym_block_comment] = STATE(771),
    [aux_sym_enum_variant_list_repeat1] = STATE(782),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_RPAREN] = ACTIONS(3007),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COMMA] = ACTIONS(3009),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_POUND] = ACTIONS(2997),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_pub] = ACTIONS(3013),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(3015),
    [sym_metavariable] = ACTIONS(1621),
  },
  [772] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym_closure_expression] = STATE(3177),
    [sym_closure_parameters] = STATE(220),
    [sym__pattern] = STATE(2962),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(772),
    [sym_block_comment] = STATE(772),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1515),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_static] = ACTIONS(1523),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [anon_sym_move] = ACTIONS(1527),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [773] = {
    [sym_attribute_item] = STATE(1405),
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym_visibility_modifier] = STATE(916),
    [sym__type] = STATE(2990),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(773),
    [sym_block_comment] = STATE(773),
    [aux_sym_enum_variant_list_repeat1] = STATE(783),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_RPAREN] = ACTIONS(3017),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_POUND] = ACTIONS(2997),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_pub] = ACTIONS(3013),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(3015),
    [sym_metavariable] = ACTIONS(1621),
  },
  [774] = {
    [sym_attribute_item] = STATE(1405),
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym_visibility_modifier] = STATE(916),
    [sym__type] = STATE(2990),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(774),
    [sym_block_comment] = STATE(774),
    [aux_sym_enum_variant_list_repeat1] = STATE(783),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_RPAREN] = ACTIONS(3019),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_POUND] = ACTIONS(2997),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_pub] = ACTIONS(3013),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(3015),
    [sym_metavariable] = ACTIONS(1621),
  },
  [775] = {
    [sym_attribute_item] = STATE(1405),
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym_visibility_modifier] = STATE(916),
    [sym__type] = STATE(2990),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(775),
    [sym_block_comment] = STATE(775),
    [aux_sym_enum_variant_list_repeat1] = STATE(783),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_RPAREN] = ACTIONS(3021),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_POUND] = ACTIONS(2997),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_pub] = ACTIONS(3013),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(3015),
    [sym_metavariable] = ACTIONS(1621),
  },
  [776] = {
    [sym_attribute_item] = STATE(1405),
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym_visibility_modifier] = STATE(916),
    [sym__type] = STATE(2990),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(776),
    [sym_block_comment] = STATE(776),
    [aux_sym_enum_variant_list_repeat1] = STATE(783),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_RPAREN] = ACTIONS(3023),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_POUND] = ACTIONS(2997),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_pub] = ACTIONS(3013),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(3015),
    [sym_metavariable] = ACTIONS(1621),
  },
  [777] = {
    [sym_attribute_item] = STATE(1405),
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym_visibility_modifier] = STATE(916),
    [sym__type] = STATE(2990),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(777),
    [sym_block_comment] = STATE(777),
    [aux_sym_enum_variant_list_repeat1] = STATE(783),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_RPAREN] = ACTIONS(3025),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_POUND] = ACTIONS(2997),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_pub] = ACTIONS(3013),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(3015),
    [sym_metavariable] = ACTIONS(1621),
  },
  [778] = {
    [sym_attribute_item] = STATE(1405),
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym_visibility_modifier] = STATE(916),
    [sym__type] = STATE(2990),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(778),
    [sym_block_comment] = STATE(778),
    [aux_sym_enum_variant_list_repeat1] = STATE(783),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_RPAREN] = ACTIONS(3027),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_POUND] = ACTIONS(2997),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_pub] = ACTIONS(3013),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(3015),
    [sym_metavariable] = ACTIONS(1621),
  },
  [779] = {
    [sym_attribute_item] = STATE(1405),
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym_visibility_modifier] = STATE(916),
    [sym__type] = STATE(2990),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(779),
    [sym_block_comment] = STATE(779),
    [aux_sym_enum_variant_list_repeat1] = STATE(783),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_POUND] = ACTIONS(2997),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_pub] = ACTIONS(3013),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(3015),
    [sym_metavariable] = ACTIONS(1621),
  },
  [780] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2738),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(780),
    [sym_block_comment] = STATE(780),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_RBRACK] = ACTIONS(3029),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COMMA] = ACTIONS(3031),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [781] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2669),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(781),
    [sym_block_comment] = STATE(781),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_RPAREN] = ACTIONS(3033),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COMMA] = ACTIONS(3035),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [782] = {
    [sym_attribute_item] = STATE(1405),
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym_visibility_modifier] = STATE(904),
    [sym__type] = STATE(2589),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(782),
    [sym_block_comment] = STATE(782),
    [aux_sym_enum_variant_list_repeat1] = STATE(1061),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_POUND] = ACTIONS(2997),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_pub] = ACTIONS(3013),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(3015),
    [sym_metavariable] = ACTIONS(1621),
  },
  [783] = {
    [sym_attribute_item] = STATE(1405),
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym_visibility_modifier] = STATE(907),
    [sym__type] = STATE(2916),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(783),
    [sym_block_comment] = STATE(783),
    [aux_sym_enum_variant_list_repeat1] = STATE(1061),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_POUND] = ACTIONS(2997),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_pub] = ACTIONS(3013),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(3015),
    [sym_metavariable] = ACTIONS(1621),
  },
  [784] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2671),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(784),
    [sym_block_comment] = STATE(784),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_RPAREN] = ACTIONS(3037),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COMMA] = ACTIONS(3039),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [785] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2700),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(785),
    [sym_block_comment] = STATE(785),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_RPAREN] = ACTIONS(3041),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COMMA] = ACTIONS(3043),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [786] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2670),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(786),
    [sym_block_comment] = STATE(786),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_RBRACK] = ACTIONS(1545),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COMMA] = ACTIONS(1551),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [787] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2723),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(787),
    [sym_block_comment] = STATE(787),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_RPAREN] = ACTIONS(3045),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COMMA] = ACTIONS(3047),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [788] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2638),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(788),
    [sym_block_comment] = STATE(788),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_RPAREN] = ACTIONS(3049),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [789] = {
    [sym_parameter] = STATE(2768),
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2593),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(789),
    [sym_block_comment] = STATE(789),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(3051),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(3053),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3055),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [790] = {
    [sym_parameter] = STATE(3214),
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2980),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(790),
    [sym_block_comment] = STATE(790),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(3053),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3055),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [791] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2638),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(791),
    [sym_block_comment] = STATE(791),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_RBRACK] = ACTIONS(3057),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [792] = {
    [sym_parameter] = STATE(2768),
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2417),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(792),
    [sym_block_comment] = STATE(792),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(3051),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(3053),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3055),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [793] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2690),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(793),
    [sym_block_comment] = STATE(793),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1305),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3059),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3061),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [794] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2638),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(794),
    [sym_block_comment] = STATE(794),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_RPAREN] = ACTIONS(3063),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [795] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2638),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(795),
    [sym_block_comment] = STATE(795),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_RBRACK] = ACTIONS(3065),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [796] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2638),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(796),
    [sym_block_comment] = STATE(796),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_RPAREN] = ACTIONS(3067),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [797] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2638),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(797),
    [sym_block_comment] = STATE(797),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_RPAREN] = ACTIONS(3069),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [798] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2638),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(798),
    [sym_block_comment] = STATE(798),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_RBRACK] = ACTIONS(3071),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [799] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2638),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(799),
    [sym_block_comment] = STATE(799),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_RPAREN] = ACTIONS(3073),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [800] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2638),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(800),
    [sym_block_comment] = STATE(800),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_RPAREN] = ACTIONS(3075),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [801] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2638),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(801),
    [sym_block_comment] = STATE(801),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_RPAREN] = ACTIONS(3077),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [802] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2638),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(802),
    [sym_block_comment] = STATE(802),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_RPAREN] = ACTIONS(3079),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [803] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2638),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(803),
    [sym_block_comment] = STATE(803),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_RBRACK] = ACTIONS(3081),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [804] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(3333),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(804),
    [sym_block_comment] = STATE(804),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [805] = {
    [sym_bracketed_type] = STATE(3553),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3287),
    [sym_macro_invocation] = STATE(2757),
    [sym_scoped_identifier] = STATE(2151),
    [sym_scoped_type_identifier] = STATE(2982),
    [sym_const_block] = STATE(2757),
    [sym__pattern] = STATE(2877),
    [sym_tuple_pattern] = STATE(2757),
    [sym_slice_pattern] = STATE(2757),
    [sym_tuple_struct_pattern] = STATE(2757),
    [sym_struct_pattern] = STATE(2757),
    [sym_remaining_field_pattern] = STATE(2757),
    [sym_mut_pattern] = STATE(2757),
    [sym_range_pattern] = STATE(2757),
    [sym_ref_pattern] = STATE(2757),
    [sym_captured_pattern] = STATE(2757),
    [sym_reference_pattern] = STATE(2757),
    [sym_or_pattern] = STATE(2757),
    [sym__literal_pattern] = STATE(2330),
    [sym_negative_literal] = STATE(2364),
    [sym_string_literal] = STATE(2364),
    [sym_raw_string_literal] = STATE(2364),
    [sym_boolean_literal] = STATE(2364),
    [sym_line_comment] = STATE(805),
    [sym_block_comment] = STATE(805),
    [sym_identifier] = ACTIONS(1645),
    [anon_sym_LPAREN] = ACTIONS(1647),
    [anon_sym_LBRACK] = ACTIONS(1649),
    [anon_sym_u8] = ACTIONS(1653),
    [anon_sym_i8] = ACTIONS(1653),
    [anon_sym_u16] = ACTIONS(1653),
    [anon_sym_i16] = ACTIONS(1653),
    [anon_sym_u32] = ACTIONS(1653),
    [anon_sym_i32] = ACTIONS(1653),
    [anon_sym_u64] = ACTIONS(1653),
    [anon_sym_i64] = ACTIONS(1653),
    [anon_sym_u128] = ACTIONS(1653),
    [anon_sym_i128] = ACTIONS(1653),
    [anon_sym_isize] = ACTIONS(1653),
    [anon_sym_usize] = ACTIONS(1653),
    [anon_sym_f32] = ACTIONS(1653),
    [anon_sym_f64] = ACTIONS(1653),
    [anon_sym_bool] = ACTIONS(1653),
    [anon_sym_str] = ACTIONS(1653),
    [anon_sym_char] = ACTIONS(1653),
    [anon_sym_DASH] = ACTIONS(1655),
    [anon_sym_AMP] = ACTIONS(1657),
    [anon_sym_PIPE] = ACTIONS(1659),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1661),
    [anon_sym_DOT_DOT] = ACTIONS(1663),
    [anon_sym_COLON_COLON] = ACTIONS(1665),
    [anon_sym_const] = ACTIONS(1669),
    [anon_sym_default] = ACTIONS(1671),
    [anon_sym_gen] = ACTIONS(1671),
    [anon_sym_union] = ACTIONS(1671),
    [anon_sym_ref] = ACTIONS(1673),
    [sym_mutable_specifier] = ACTIONS(1675),
    [sym_integer_literal] = ACTIONS(1677),
    [aux_sym_string_literal_token1] = ACTIONS(1679),
    [sym_char_literal] = ACTIONS(1677),
    [anon_sym_true] = ACTIONS(1681),
    [anon_sym_false] = ACTIONS(1681),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1683),
    [sym_super] = ACTIONS(1683),
    [sym_crate] = ACTIONS(1683),
    [sym_metavariable] = ACTIONS(1685),
    [sym__raw_string_literal_start] = ACTIONS(1687),
    [sym_float_literal] = ACTIONS(1677),
  },
  [806] = {
    [sym_bracketed_type] = STATE(3553),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3287),
    [sym_macro_invocation] = STATE(2757),
    [sym_scoped_identifier] = STATE(2151),
    [sym_scoped_type_identifier] = STATE(2982),
    [sym_const_block] = STATE(2757),
    [sym__pattern] = STATE(3006),
    [sym_tuple_pattern] = STATE(2757),
    [sym_slice_pattern] = STATE(2757),
    [sym_tuple_struct_pattern] = STATE(2757),
    [sym_struct_pattern] = STATE(2757),
    [sym_remaining_field_pattern] = STATE(2757),
    [sym_mut_pattern] = STATE(2757),
    [sym_range_pattern] = STATE(2757),
    [sym_ref_pattern] = STATE(2757),
    [sym_captured_pattern] = STATE(2757),
    [sym_reference_pattern] = STATE(2757),
    [sym_or_pattern] = STATE(2757),
    [sym__literal_pattern] = STATE(2330),
    [sym_negative_literal] = STATE(2364),
    [sym_string_literal] = STATE(2364),
    [sym_raw_string_literal] = STATE(2364),
    [sym_boolean_literal] = STATE(2364),
    [sym_line_comment] = STATE(806),
    [sym_block_comment] = STATE(806),
    [sym_identifier] = ACTIONS(1645),
    [anon_sym_LPAREN] = ACTIONS(1647),
    [anon_sym_LBRACK] = ACTIONS(1649),
    [anon_sym_u8] = ACTIONS(1653),
    [anon_sym_i8] = ACTIONS(1653),
    [anon_sym_u16] = ACTIONS(1653),
    [anon_sym_i16] = ACTIONS(1653),
    [anon_sym_u32] = ACTIONS(1653),
    [anon_sym_i32] = ACTIONS(1653),
    [anon_sym_u64] = ACTIONS(1653),
    [anon_sym_i64] = ACTIONS(1653),
    [anon_sym_u128] = ACTIONS(1653),
    [anon_sym_i128] = ACTIONS(1653),
    [anon_sym_isize] = ACTIONS(1653),
    [anon_sym_usize] = ACTIONS(1653),
    [anon_sym_f32] = ACTIONS(1653),
    [anon_sym_f64] = ACTIONS(1653),
    [anon_sym_bool] = ACTIONS(1653),
    [anon_sym_str] = ACTIONS(1653),
    [anon_sym_char] = ACTIONS(1653),
    [anon_sym_DASH] = ACTIONS(1655),
    [anon_sym_AMP] = ACTIONS(1657),
    [anon_sym_PIPE] = ACTIONS(1659),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1661),
    [anon_sym_DOT_DOT] = ACTIONS(1663),
    [anon_sym_COLON_COLON] = ACTIONS(1665),
    [anon_sym_const] = ACTIONS(1669),
    [anon_sym_default] = ACTIONS(1671),
    [anon_sym_gen] = ACTIONS(1671),
    [anon_sym_union] = ACTIONS(1671),
    [anon_sym_ref] = ACTIONS(1673),
    [sym_mutable_specifier] = ACTIONS(1675),
    [sym_integer_literal] = ACTIONS(1677),
    [aux_sym_string_literal_token1] = ACTIONS(1679),
    [sym_char_literal] = ACTIONS(1677),
    [anon_sym_true] = ACTIONS(1681),
    [anon_sym_false] = ACTIONS(1681),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1683),
    [sym_super] = ACTIONS(1683),
    [sym_crate] = ACTIONS(1683),
    [sym_metavariable] = ACTIONS(1685),
    [sym__raw_string_literal_start] = ACTIONS(1687),
    [sym_float_literal] = ACTIONS(1677),
  },
  [807] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2143),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(807),
    [sym_block_comment] = STATE(807),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [808] = {
    [sym_bracketed_type] = STATE(3553),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3287),
    [sym_macro_invocation] = STATE(2757),
    [sym_scoped_identifier] = STATE(2151),
    [sym_scoped_type_identifier] = STATE(2982),
    [sym_const_block] = STATE(2757),
    [sym__pattern] = STATE(2886),
    [sym_tuple_pattern] = STATE(2757),
    [sym_slice_pattern] = STATE(2757),
    [sym_tuple_struct_pattern] = STATE(2757),
    [sym_struct_pattern] = STATE(2757),
    [sym_remaining_field_pattern] = STATE(2757),
    [sym_mut_pattern] = STATE(2757),
    [sym_range_pattern] = STATE(2757),
    [sym_ref_pattern] = STATE(2757),
    [sym_captured_pattern] = STATE(2757),
    [sym_reference_pattern] = STATE(2757),
    [sym_or_pattern] = STATE(2757),
    [sym__literal_pattern] = STATE(2330),
    [sym_negative_literal] = STATE(2364),
    [sym_string_literal] = STATE(2364),
    [sym_raw_string_literal] = STATE(2364),
    [sym_boolean_literal] = STATE(2364),
    [sym_line_comment] = STATE(808),
    [sym_block_comment] = STATE(808),
    [sym_identifier] = ACTIONS(1645),
    [anon_sym_LPAREN] = ACTIONS(1647),
    [anon_sym_LBRACK] = ACTIONS(1649),
    [anon_sym_u8] = ACTIONS(1653),
    [anon_sym_i8] = ACTIONS(1653),
    [anon_sym_u16] = ACTIONS(1653),
    [anon_sym_i16] = ACTIONS(1653),
    [anon_sym_u32] = ACTIONS(1653),
    [anon_sym_i32] = ACTIONS(1653),
    [anon_sym_u64] = ACTIONS(1653),
    [anon_sym_i64] = ACTIONS(1653),
    [anon_sym_u128] = ACTIONS(1653),
    [anon_sym_i128] = ACTIONS(1653),
    [anon_sym_isize] = ACTIONS(1653),
    [anon_sym_usize] = ACTIONS(1653),
    [anon_sym_f32] = ACTIONS(1653),
    [anon_sym_f64] = ACTIONS(1653),
    [anon_sym_bool] = ACTIONS(1653),
    [anon_sym_str] = ACTIONS(1653),
    [anon_sym_char] = ACTIONS(1653),
    [anon_sym_DASH] = ACTIONS(1655),
    [anon_sym_AMP] = ACTIONS(1657),
    [anon_sym_PIPE] = ACTIONS(1659),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1661),
    [anon_sym_DOT_DOT] = ACTIONS(1663),
    [anon_sym_COLON_COLON] = ACTIONS(1665),
    [anon_sym_const] = ACTIONS(1669),
    [anon_sym_default] = ACTIONS(1671),
    [anon_sym_gen] = ACTIONS(1671),
    [anon_sym_union] = ACTIONS(1671),
    [anon_sym_ref] = ACTIONS(1673),
    [sym_mutable_specifier] = ACTIONS(1675),
    [sym_integer_literal] = ACTIONS(1677),
    [aux_sym_string_literal_token1] = ACTIONS(1679),
    [sym_char_literal] = ACTIONS(1677),
    [anon_sym_true] = ACTIONS(1681),
    [anon_sym_false] = ACTIONS(1681),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1683),
    [sym_super] = ACTIONS(1683),
    [sym_crate] = ACTIONS(1683),
    [sym_metavariable] = ACTIONS(1685),
    [sym__raw_string_literal_start] = ACTIONS(1687),
    [sym_float_literal] = ACTIONS(1677),
  },
  [809] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2119),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(809),
    [sym_block_comment] = STATE(809),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [810] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2134),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(810),
    [sym_block_comment] = STATE(810),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(3083),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [811] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2124),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(811),
    [sym_block_comment] = STATE(811),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [812] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(3051),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(812),
    [sym_block_comment] = STATE(812),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [813] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2412),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(813),
    [sym_block_comment] = STATE(813),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [814] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(3239),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(814),
    [sym_block_comment] = STATE(814),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [815] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2122),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(815),
    [sym_block_comment] = STATE(815),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [816] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2436),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(816),
    [sym_block_comment] = STATE(816),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(3085),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [817] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(3114),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(817),
    [sym_block_comment] = STATE(817),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [818] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2493),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(818),
    [sym_block_comment] = STATE(818),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(3087),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [819] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2638),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(819),
    [sym_block_comment] = STATE(819),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [820] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(3313),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(820),
    [sym_block_comment] = STATE(820),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [821] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2933),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(821),
    [sym_block_comment] = STATE(821),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [822] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2411),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(822),
    [sym_block_comment] = STATE(822),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [823] = {
    [sym_bracketed_type] = STATE(3553),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3287),
    [sym_macro_invocation] = STATE(2757),
    [sym_scoped_identifier] = STATE(2151),
    [sym_scoped_type_identifier] = STATE(2982),
    [sym_const_block] = STATE(2757),
    [sym__pattern] = STATE(2935),
    [sym_tuple_pattern] = STATE(2757),
    [sym_slice_pattern] = STATE(2757),
    [sym_tuple_struct_pattern] = STATE(2757),
    [sym_struct_pattern] = STATE(2757),
    [sym_remaining_field_pattern] = STATE(2757),
    [sym_mut_pattern] = STATE(2757),
    [sym_range_pattern] = STATE(2757),
    [sym_ref_pattern] = STATE(2757),
    [sym_captured_pattern] = STATE(2757),
    [sym_reference_pattern] = STATE(2757),
    [sym_or_pattern] = STATE(2757),
    [sym__literal_pattern] = STATE(2330),
    [sym_negative_literal] = STATE(2364),
    [sym_string_literal] = STATE(2364),
    [sym_raw_string_literal] = STATE(2364),
    [sym_boolean_literal] = STATE(2364),
    [sym_line_comment] = STATE(823),
    [sym_block_comment] = STATE(823),
    [sym_identifier] = ACTIONS(1645),
    [anon_sym_LPAREN] = ACTIONS(1647),
    [anon_sym_LBRACK] = ACTIONS(1649),
    [anon_sym_u8] = ACTIONS(1653),
    [anon_sym_i8] = ACTIONS(1653),
    [anon_sym_u16] = ACTIONS(1653),
    [anon_sym_i16] = ACTIONS(1653),
    [anon_sym_u32] = ACTIONS(1653),
    [anon_sym_i32] = ACTIONS(1653),
    [anon_sym_u64] = ACTIONS(1653),
    [anon_sym_i64] = ACTIONS(1653),
    [anon_sym_u128] = ACTIONS(1653),
    [anon_sym_i128] = ACTIONS(1653),
    [anon_sym_isize] = ACTIONS(1653),
    [anon_sym_usize] = ACTIONS(1653),
    [anon_sym_f32] = ACTIONS(1653),
    [anon_sym_f64] = ACTIONS(1653),
    [anon_sym_bool] = ACTIONS(1653),
    [anon_sym_str] = ACTIONS(1653),
    [anon_sym_char] = ACTIONS(1653),
    [anon_sym_DASH] = ACTIONS(1655),
    [anon_sym_AMP] = ACTIONS(1657),
    [anon_sym_PIPE] = ACTIONS(1659),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1661),
    [anon_sym_DOT_DOT] = ACTIONS(1663),
    [anon_sym_COLON_COLON] = ACTIONS(1665),
    [anon_sym_const] = ACTIONS(1669),
    [anon_sym_default] = ACTIONS(1671),
    [anon_sym_gen] = ACTIONS(1671),
    [anon_sym_union] = ACTIONS(1671),
    [anon_sym_ref] = ACTIONS(1673),
    [sym_mutable_specifier] = ACTIONS(3089),
    [sym_integer_literal] = ACTIONS(1677),
    [aux_sym_string_literal_token1] = ACTIONS(1679),
    [sym_char_literal] = ACTIONS(1677),
    [anon_sym_true] = ACTIONS(1681),
    [anon_sym_false] = ACTIONS(1681),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1683),
    [sym_super] = ACTIONS(1683),
    [sym_crate] = ACTIONS(1683),
    [sym_metavariable] = ACTIONS(1685),
    [sym__raw_string_literal_start] = ACTIONS(1687),
    [sym_float_literal] = ACTIONS(1677),
  },
  [824] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2530),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(824),
    [sym_block_comment] = STATE(824),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3091),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [825] = {
    [sym_bracketed_type] = STATE(3553),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3287),
    [sym_macro_invocation] = STATE(2757),
    [sym_scoped_identifier] = STATE(2151),
    [sym_scoped_type_identifier] = STATE(2982),
    [sym_const_block] = STATE(2757),
    [sym__pattern] = STATE(2942),
    [sym_tuple_pattern] = STATE(2757),
    [sym_slice_pattern] = STATE(2757),
    [sym_tuple_struct_pattern] = STATE(2757),
    [sym_struct_pattern] = STATE(2757),
    [sym_remaining_field_pattern] = STATE(2757),
    [sym_mut_pattern] = STATE(2757),
    [sym_range_pattern] = STATE(2757),
    [sym_ref_pattern] = STATE(2757),
    [sym_captured_pattern] = STATE(2757),
    [sym_reference_pattern] = STATE(2757),
    [sym_or_pattern] = STATE(2757),
    [sym__literal_pattern] = STATE(2330),
    [sym_negative_literal] = STATE(2364),
    [sym_string_literal] = STATE(2364),
    [sym_raw_string_literal] = STATE(2364),
    [sym_boolean_literal] = STATE(2364),
    [sym_line_comment] = STATE(825),
    [sym_block_comment] = STATE(825),
    [sym_identifier] = ACTIONS(1645),
    [anon_sym_LPAREN] = ACTIONS(1647),
    [anon_sym_LBRACK] = ACTIONS(1649),
    [anon_sym_u8] = ACTIONS(1653),
    [anon_sym_i8] = ACTIONS(1653),
    [anon_sym_u16] = ACTIONS(1653),
    [anon_sym_i16] = ACTIONS(1653),
    [anon_sym_u32] = ACTIONS(1653),
    [anon_sym_i32] = ACTIONS(1653),
    [anon_sym_u64] = ACTIONS(1653),
    [anon_sym_i64] = ACTIONS(1653),
    [anon_sym_u128] = ACTIONS(1653),
    [anon_sym_i128] = ACTIONS(1653),
    [anon_sym_isize] = ACTIONS(1653),
    [anon_sym_usize] = ACTIONS(1653),
    [anon_sym_f32] = ACTIONS(1653),
    [anon_sym_f64] = ACTIONS(1653),
    [anon_sym_bool] = ACTIONS(1653),
    [anon_sym_str] = ACTIONS(1653),
    [anon_sym_char] = ACTIONS(1653),
    [anon_sym_DASH] = ACTIONS(1655),
    [anon_sym_AMP] = ACTIONS(1657),
    [anon_sym_PIPE] = ACTIONS(1659),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1661),
    [anon_sym_DOT_DOT] = ACTIONS(1663),
    [anon_sym_COLON_COLON] = ACTIONS(1665),
    [anon_sym_const] = ACTIONS(1669),
    [anon_sym_default] = ACTIONS(1671),
    [anon_sym_gen] = ACTIONS(1671),
    [anon_sym_union] = ACTIONS(1671),
    [anon_sym_ref] = ACTIONS(1673),
    [sym_mutable_specifier] = ACTIONS(1675),
    [sym_integer_literal] = ACTIONS(1677),
    [aux_sym_string_literal_token1] = ACTIONS(1679),
    [sym_char_literal] = ACTIONS(1677),
    [anon_sym_true] = ACTIONS(1681),
    [anon_sym_false] = ACTIONS(1681),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1683),
    [sym_super] = ACTIONS(1683),
    [sym_crate] = ACTIONS(1683),
    [sym_metavariable] = ACTIONS(1685),
    [sym__raw_string_literal_start] = ACTIONS(1687),
    [sym_float_literal] = ACTIONS(1677),
  },
  [826] = {
    [sym_bracketed_type] = STATE(3553),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3287),
    [sym_macro_invocation] = STATE(2757),
    [sym_scoped_identifier] = STATE(2151),
    [sym_scoped_type_identifier] = STATE(2982),
    [sym_const_block] = STATE(2757),
    [sym__pattern] = STATE(2872),
    [sym_tuple_pattern] = STATE(2757),
    [sym_slice_pattern] = STATE(2757),
    [sym_tuple_struct_pattern] = STATE(2757),
    [sym_struct_pattern] = STATE(2757),
    [sym_remaining_field_pattern] = STATE(2757),
    [sym_mut_pattern] = STATE(2757),
    [sym_range_pattern] = STATE(2757),
    [sym_ref_pattern] = STATE(2757),
    [sym_captured_pattern] = STATE(2757),
    [sym_reference_pattern] = STATE(2757),
    [sym_or_pattern] = STATE(2757),
    [sym__literal_pattern] = STATE(2330),
    [sym_negative_literal] = STATE(2364),
    [sym_string_literal] = STATE(2364),
    [sym_raw_string_literal] = STATE(2364),
    [sym_boolean_literal] = STATE(2364),
    [sym_line_comment] = STATE(826),
    [sym_block_comment] = STATE(826),
    [sym_identifier] = ACTIONS(1645),
    [anon_sym_LPAREN] = ACTIONS(1647),
    [anon_sym_LBRACK] = ACTIONS(1649),
    [anon_sym_u8] = ACTIONS(1653),
    [anon_sym_i8] = ACTIONS(1653),
    [anon_sym_u16] = ACTIONS(1653),
    [anon_sym_i16] = ACTIONS(1653),
    [anon_sym_u32] = ACTIONS(1653),
    [anon_sym_i32] = ACTIONS(1653),
    [anon_sym_u64] = ACTIONS(1653),
    [anon_sym_i64] = ACTIONS(1653),
    [anon_sym_u128] = ACTIONS(1653),
    [anon_sym_i128] = ACTIONS(1653),
    [anon_sym_isize] = ACTIONS(1653),
    [anon_sym_usize] = ACTIONS(1653),
    [anon_sym_f32] = ACTIONS(1653),
    [anon_sym_f64] = ACTIONS(1653),
    [anon_sym_bool] = ACTIONS(1653),
    [anon_sym_str] = ACTIONS(1653),
    [anon_sym_char] = ACTIONS(1653),
    [anon_sym_DASH] = ACTIONS(1655),
    [anon_sym_AMP] = ACTIONS(1657),
    [anon_sym_PIPE] = ACTIONS(1659),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1661),
    [anon_sym_DOT_DOT] = ACTIONS(1663),
    [anon_sym_COLON_COLON] = ACTIONS(1665),
    [anon_sym_const] = ACTIONS(1669),
    [anon_sym_default] = ACTIONS(1671),
    [anon_sym_gen] = ACTIONS(1671),
    [anon_sym_union] = ACTIONS(1671),
    [anon_sym_ref] = ACTIONS(1673),
    [sym_mutable_specifier] = ACTIONS(1675),
    [sym_integer_literal] = ACTIONS(1677),
    [aux_sym_string_literal_token1] = ACTIONS(1679),
    [sym_char_literal] = ACTIONS(1677),
    [anon_sym_true] = ACTIONS(1681),
    [anon_sym_false] = ACTIONS(1681),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1683),
    [sym_super] = ACTIONS(1683),
    [sym_crate] = ACTIONS(1683),
    [sym_metavariable] = ACTIONS(1685),
    [sym__raw_string_literal_start] = ACTIONS(1687),
    [sym_float_literal] = ACTIONS(1677),
  },
  [827] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(3120),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(827),
    [sym_block_comment] = STATE(827),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [828] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2106),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(828),
    [sym_block_comment] = STATE(828),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [829] = {
    [sym_bracketed_type] = STATE(3553),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3287),
    [sym_macro_invocation] = STATE(2757),
    [sym_scoped_identifier] = STATE(2151),
    [sym_scoped_type_identifier] = STATE(2982),
    [sym_const_block] = STATE(2757),
    [sym__pattern] = STATE(2936),
    [sym_tuple_pattern] = STATE(2757),
    [sym_slice_pattern] = STATE(2757),
    [sym_tuple_struct_pattern] = STATE(2757),
    [sym_struct_pattern] = STATE(2757),
    [sym_remaining_field_pattern] = STATE(2757),
    [sym_mut_pattern] = STATE(2757),
    [sym_range_pattern] = STATE(2757),
    [sym_ref_pattern] = STATE(2757),
    [sym_captured_pattern] = STATE(2757),
    [sym_reference_pattern] = STATE(2757),
    [sym_or_pattern] = STATE(2757),
    [sym__literal_pattern] = STATE(2330),
    [sym_negative_literal] = STATE(2364),
    [sym_string_literal] = STATE(2364),
    [sym_raw_string_literal] = STATE(2364),
    [sym_boolean_literal] = STATE(2364),
    [sym_line_comment] = STATE(829),
    [sym_block_comment] = STATE(829),
    [sym_identifier] = ACTIONS(1645),
    [anon_sym_LPAREN] = ACTIONS(1647),
    [anon_sym_LBRACK] = ACTIONS(1649),
    [anon_sym_u8] = ACTIONS(1653),
    [anon_sym_i8] = ACTIONS(1653),
    [anon_sym_u16] = ACTIONS(1653),
    [anon_sym_i16] = ACTIONS(1653),
    [anon_sym_u32] = ACTIONS(1653),
    [anon_sym_i32] = ACTIONS(1653),
    [anon_sym_u64] = ACTIONS(1653),
    [anon_sym_i64] = ACTIONS(1653),
    [anon_sym_u128] = ACTIONS(1653),
    [anon_sym_i128] = ACTIONS(1653),
    [anon_sym_isize] = ACTIONS(1653),
    [anon_sym_usize] = ACTIONS(1653),
    [anon_sym_f32] = ACTIONS(1653),
    [anon_sym_f64] = ACTIONS(1653),
    [anon_sym_bool] = ACTIONS(1653),
    [anon_sym_str] = ACTIONS(1653),
    [anon_sym_char] = ACTIONS(1653),
    [anon_sym_DASH] = ACTIONS(1655),
    [anon_sym_AMP] = ACTIONS(1657),
    [anon_sym_PIPE] = ACTIONS(1659),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1661),
    [anon_sym_DOT_DOT] = ACTIONS(1663),
    [anon_sym_COLON_COLON] = ACTIONS(1665),
    [anon_sym_const] = ACTIONS(1669),
    [anon_sym_default] = ACTIONS(1671),
    [anon_sym_gen] = ACTIONS(1671),
    [anon_sym_union] = ACTIONS(1671),
    [anon_sym_ref] = ACTIONS(1673),
    [sym_mutable_specifier] = ACTIONS(1675),
    [sym_integer_literal] = ACTIONS(1677),
    [aux_sym_string_literal_token1] = ACTIONS(1679),
    [sym_char_literal] = ACTIONS(1677),
    [anon_sym_true] = ACTIONS(1681),
    [anon_sym_false] = ACTIONS(1681),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1683),
    [sym_super] = ACTIONS(1683),
    [sym_crate] = ACTIONS(1683),
    [sym_metavariable] = ACTIONS(1685),
    [sym__raw_string_literal_start] = ACTIONS(1687),
    [sym_float_literal] = ACTIONS(1677),
  },
  [830] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(2142),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(830),
    [sym_block_comment] = STATE(830),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [831] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(3308),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(831),
    [sym_block_comment] = STATE(831),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [832] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(3326),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(832),
    [sym_block_comment] = STATE(832),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [833] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(3330),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(833),
    [sym_block_comment] = STATE(833),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [834] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(3331),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(834),
    [sym_block_comment] = STATE(834),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [835] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(3332),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(835),
    [sym_block_comment] = STATE(835),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [836] = {
    [sym_bracketed_type] = STATE(3599),
    [sym_generic_type] = STATE(3427),
    [sym_generic_type_with_turbofish] = STATE(3337),
    [sym_macro_invocation] = STATE(2136),
    [sym_scoped_identifier] = STATE(1963),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_const_block] = STATE(2136),
    [sym__pattern] = STATE(3007),
    [sym_tuple_pattern] = STATE(2136),
    [sym_slice_pattern] = STATE(2136),
    [sym_tuple_struct_pattern] = STATE(2136),
    [sym_struct_pattern] = STATE(2136),
    [sym_remaining_field_pattern] = STATE(2136),
    [sym_mut_pattern] = STATE(2136),
    [sym_range_pattern] = STATE(2136),
    [sym_ref_pattern] = STATE(2136),
    [sym_captured_pattern] = STATE(2136),
    [sym_reference_pattern] = STATE(2136),
    [sym_or_pattern] = STATE(2136),
    [sym__literal_pattern] = STATE(2025),
    [sym_negative_literal] = STATE(2037),
    [sym_string_literal] = STATE(2037),
    [sym_raw_string_literal] = STATE(2037),
    [sym_boolean_literal] = STATE(2037),
    [sym_line_comment] = STATE(836),
    [sym_block_comment] = STATE(836),
    [sym_identifier] = ACTIONS(2961),
    [anon_sym_LPAREN] = ACTIONS(2963),
    [anon_sym_LBRACK] = ACTIONS(2967),
    [anon_sym_u8] = ACTIONS(2969),
    [anon_sym_i8] = ACTIONS(2969),
    [anon_sym_u16] = ACTIONS(2969),
    [anon_sym_i16] = ACTIONS(2969),
    [anon_sym_u32] = ACTIONS(2969),
    [anon_sym_i32] = ACTIONS(2969),
    [anon_sym_u64] = ACTIONS(2969),
    [anon_sym_i64] = ACTIONS(2969),
    [anon_sym_u128] = ACTIONS(2969),
    [anon_sym_i128] = ACTIONS(2969),
    [anon_sym_isize] = ACTIONS(2969),
    [anon_sym_usize] = ACTIONS(2969),
    [anon_sym_f32] = ACTIONS(2969),
    [anon_sym_f64] = ACTIONS(2969),
    [anon_sym_bool] = ACTIONS(2969),
    [anon_sym_str] = ACTIONS(2969),
    [anon_sym_char] = ACTIONS(2969),
    [anon_sym_DASH] = ACTIONS(1295),
    [anon_sym_AMP] = ACTIONS(2971),
    [anon_sym_PIPE] = ACTIONS(1301),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym__] = ACTIONS(1517),
    [anon_sym_DOT_DOT] = ACTIONS(1519),
    [anon_sym_COLON_COLON] = ACTIONS(2975),
    [anon_sym_const] = ACTIONS(2977),
    [anon_sym_default] = ACTIONS(2979),
    [anon_sym_gen] = ACTIONS(2979),
    [anon_sym_union] = ACTIONS(2979),
    [anon_sym_ref] = ACTIONS(1333),
    [sym_mutable_specifier] = ACTIONS(1525),
    [sym_integer_literal] = ACTIONS(1339),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1339),
    [anon_sym_true] = ACTIONS(1343),
    [anon_sym_false] = ACTIONS(1343),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2981),
    [sym_super] = ACTIONS(2981),
    [sym_crate] = ACTIONS(2981),
    [sym_metavariable] = ACTIONS(2983),
    [sym__raw_string_literal_start] = ACTIONS(1351),
    [sym_float_literal] = ACTIONS(1339),
  },
  [837] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(3262),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(837),
    [sym_block_comment] = STATE(837),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_PLUS] = ACTIONS(3093),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_mutable_specifier] = ACTIONS(3095),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [838] = {
    [sym_function_modifiers] = STATE(3367),
    [sym_removed_trait_bound] = STATE(1416),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1381),
    [sym_bracketed_type] = STATE(3544),
    [sym_lifetime] = STATE(3348),
    [sym_array_type] = STATE(1416),
    [sym_for_lifetimes] = STATE(1584),
    [sym_function_type] = STATE(1416),
    [sym_tuple_type] = STATE(1416),
    [sym_unit_type] = STATE(1416),
    [sym_generic_type] = STATE(1083),
    [sym_generic_type_with_turbofish] = STATE(3535),
    [sym_bounded_type] = STATE(1416),
    [sym_reference_type] = STATE(1416),
    [sym_pointer_type] = STATE(1416),
    [sym_never_type] = STATE(1416),
    [sym_abstract_type] = STATE(1416),
    [sym_dynamic_type] = STATE(1416),
    [sym_macro_invocation] = STATE(1416),
    [sym_scoped_identifier] = STATE(3224),
    [sym_scoped_type_identifier] = STATE(1034),
    [sym_line_comment] = STATE(838),
    [sym_block_comment] = STATE(838),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3097),
    [anon_sym_LPAREN] = ACTIONS(3099),
    [anon_sym_LBRACK] = ACTIONS(3101),
    [anon_sym_PLUS] = ACTIONS(3103),
    [anon_sym_STAR] = ACTIONS(3105),
    [anon_sym_QMARK] = ACTIONS(3107),
    [anon_sym_u8] = ACTIONS(3109),
    [anon_sym_i8] = ACTIONS(3109),
    [anon_sym_u16] = ACTIONS(3109),
    [anon_sym_i16] = ACTIONS(3109),
    [anon_sym_u32] = ACTIONS(3109),
    [anon_sym_i32] = ACTIONS(3109),
    [anon_sym_u64] = ACTIONS(3109),
    [anon_sym_i64] = ACTIONS(3109),
    [anon_sym_u128] = ACTIONS(3109),
    [anon_sym_i128] = ACTIONS(3109),
    [anon_sym_isize] = ACTIONS(3109),
    [anon_sym_usize] = ACTIONS(3109),
    [anon_sym_f32] = ACTIONS(3109),
    [anon_sym_f64] = ACTIONS(3109),
    [anon_sym_bool] = ACTIONS(3109),
    [anon_sym_str] = ACTIONS(3109),
    [anon_sym_char] = ACTIONS(3109),
    [anon_sym_BANG] = ACTIONS(3111),
    [anon_sym_AMP] = ACTIONS(3113),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(3115),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(3117),
    [anon_sym_fn] = ACTIONS(3119),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(3121),
    [anon_sym_impl] = ACTIONS(3123),
    [anon_sym_union] = ACTIONS(3121),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(3125),
    [sym_mutable_specifier] = ACTIONS(3127),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3129),
    [sym_super] = ACTIONS(3129),
    [sym_crate] = ACTIONS(3129),
    [sym_metavariable] = ACTIONS(3131),
  },
  [839] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1992),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(839),
    [sym_block_comment] = STATE(839),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_PLUS] = ACTIONS(3093),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_mutable_specifier] = ACTIONS(3133),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3135),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [840] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1992),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(840),
    [sym_block_comment] = STATE(840),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_PLUS] = ACTIONS(3093),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_mutable_specifier] = ACTIONS(3137),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [841] = {
    [sym_function_modifiers] = STATE(3341),
    [sym_removed_trait_bound] = STATE(1684),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1772),
    [sym_bracketed_type] = STATE(3559),
    [sym_lifetime] = STATE(3394),
    [sym_array_type] = STATE(1684),
    [sym_for_lifetimes] = STATE(1591),
    [sym_function_type] = STATE(1684),
    [sym_tuple_type] = STATE(1684),
    [sym_unit_type] = STATE(1684),
    [sym_generic_type] = STATE(1649),
    [sym_generic_type_with_turbofish] = STATE(3551),
    [sym_bounded_type] = STATE(1684),
    [sym_reference_type] = STATE(1684),
    [sym_pointer_type] = STATE(1684),
    [sym_never_type] = STATE(1684),
    [sym_abstract_type] = STATE(1684),
    [sym_dynamic_type] = STATE(1684),
    [sym_macro_invocation] = STATE(1684),
    [sym_scoped_identifier] = STATE(3288),
    [sym_scoped_type_identifier] = STATE(1548),
    [sym_line_comment] = STATE(841),
    [sym_block_comment] = STATE(841),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3139),
    [anon_sym_LPAREN] = ACTIONS(3141),
    [anon_sym_LBRACK] = ACTIONS(3143),
    [anon_sym_PLUS] = ACTIONS(3145),
    [anon_sym_STAR] = ACTIONS(3147),
    [anon_sym_QMARK] = ACTIONS(3149),
    [anon_sym_u8] = ACTIONS(3151),
    [anon_sym_i8] = ACTIONS(3151),
    [anon_sym_u16] = ACTIONS(3151),
    [anon_sym_i16] = ACTIONS(3151),
    [anon_sym_u32] = ACTIONS(3151),
    [anon_sym_i32] = ACTIONS(3151),
    [anon_sym_u64] = ACTIONS(3151),
    [anon_sym_i64] = ACTIONS(3151),
    [anon_sym_u128] = ACTIONS(3151),
    [anon_sym_i128] = ACTIONS(3151),
    [anon_sym_isize] = ACTIONS(3151),
    [anon_sym_usize] = ACTIONS(3151),
    [anon_sym_f32] = ACTIONS(3151),
    [anon_sym_f64] = ACTIONS(3151),
    [anon_sym_bool] = ACTIONS(3151),
    [anon_sym_str] = ACTIONS(3151),
    [anon_sym_char] = ACTIONS(3151),
    [anon_sym_BANG] = ACTIONS(3153),
    [anon_sym_AMP] = ACTIONS(3155),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(3157),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(3159),
    [anon_sym_fn] = ACTIONS(3161),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(3163),
    [anon_sym_impl] = ACTIONS(3165),
    [anon_sym_union] = ACTIONS(3163),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(3167),
    [sym_mutable_specifier] = ACTIONS(3169),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3171),
    [sym_super] = ACTIONS(3171),
    [sym_crate] = ACTIONS(3171),
    [sym_metavariable] = ACTIONS(3173),
  },
  [842] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_type_parameters] = STATE(953),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2354),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(2402),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(2194),
    [sym_line_comment] = STATE(842),
    [sym_block_comment] = STATE(842),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3175),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(3177),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(3179),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [843] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_higher_ranked_trait_bound] = STATE(2166),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2169),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(2171),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(843),
    [sym_block_comment] = STATE(843),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(3181),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [844] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2532),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(844),
    [sym_block_comment] = STATE(844),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3183),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [845] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(3000),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(845),
    [sym_block_comment] = STATE(845),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_RPAREN] = ACTIONS(3185),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [846] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2728),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(846),
    [sym_block_comment] = STATE(846),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_RPAREN] = ACTIONS(3187),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [847] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(3000),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(847),
    [sym_block_comment] = STATE(847),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_RPAREN] = ACTIONS(3189),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [848] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2004),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(840),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(848),
    [sym_block_comment] = STATE(848),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_mutable_specifier] = ACTIONS(3191),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [849] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2571),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(849),
    [sym_block_comment] = STATE(849),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_RPAREN] = ACTIONS(3193),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [850] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(3000),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(850),
    [sym_block_comment] = STATE(850),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_RPAREN] = ACTIONS(3195),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [851] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(3000),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(851),
    [sym_block_comment] = STATE(851),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_RPAREN] = ACTIONS(3197),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [852] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(3000),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(852),
    [sym_block_comment] = STATE(852),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_RPAREN] = ACTIONS(3199),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [853] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_higher_ranked_trait_bound] = STATE(2219),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2220),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(2224),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(853),
    [sym_block_comment] = STATE(853),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(3181),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [854] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_higher_ranked_trait_bound] = STATE(2219),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2227),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(2228),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(854),
    [sym_block_comment] = STATE(854),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(3181),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [855] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_higher_ranked_trait_bound] = STATE(2219),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2227),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(2224),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(855),
    [sym_block_comment] = STATE(855),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(3181),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [856] = {
    [sym_function_modifiers] = STATE(3367),
    [sym_removed_trait_bound] = STATE(1416),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1437),
    [sym_bracketed_type] = STATE(3544),
    [sym_lifetime] = STATE(838),
    [sym_array_type] = STATE(1416),
    [sym_for_lifetimes] = STATE(1584),
    [sym_function_type] = STATE(1416),
    [sym_tuple_type] = STATE(1416),
    [sym_unit_type] = STATE(1416),
    [sym_generic_type] = STATE(1083),
    [sym_generic_type_with_turbofish] = STATE(3535),
    [sym_bounded_type] = STATE(1416),
    [sym_reference_type] = STATE(1416),
    [sym_pointer_type] = STATE(1416),
    [sym_never_type] = STATE(1416),
    [sym_abstract_type] = STATE(1416),
    [sym_dynamic_type] = STATE(1416),
    [sym_macro_invocation] = STATE(1416),
    [sym_scoped_identifier] = STATE(3224),
    [sym_scoped_type_identifier] = STATE(1034),
    [sym_line_comment] = STATE(856),
    [sym_block_comment] = STATE(856),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3097),
    [anon_sym_LPAREN] = ACTIONS(3099),
    [anon_sym_LBRACK] = ACTIONS(3101),
    [anon_sym_STAR] = ACTIONS(3105),
    [anon_sym_QMARK] = ACTIONS(3107),
    [anon_sym_u8] = ACTIONS(3109),
    [anon_sym_i8] = ACTIONS(3109),
    [anon_sym_u16] = ACTIONS(3109),
    [anon_sym_i16] = ACTIONS(3109),
    [anon_sym_u32] = ACTIONS(3109),
    [anon_sym_i32] = ACTIONS(3109),
    [anon_sym_u64] = ACTIONS(3109),
    [anon_sym_i64] = ACTIONS(3109),
    [anon_sym_u128] = ACTIONS(3109),
    [anon_sym_i128] = ACTIONS(3109),
    [anon_sym_isize] = ACTIONS(3109),
    [anon_sym_usize] = ACTIONS(3109),
    [anon_sym_f32] = ACTIONS(3109),
    [anon_sym_f64] = ACTIONS(3109),
    [anon_sym_bool] = ACTIONS(3109),
    [anon_sym_str] = ACTIONS(3109),
    [anon_sym_char] = ACTIONS(3109),
    [anon_sym_BANG] = ACTIONS(3111),
    [anon_sym_AMP] = ACTIONS(3113),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(3115),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(3117),
    [anon_sym_fn] = ACTIONS(3119),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(3121),
    [anon_sym_impl] = ACTIONS(3123),
    [anon_sym_union] = ACTIONS(3121),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(3125),
    [sym_mutable_specifier] = ACTIONS(3201),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3129),
    [sym_super] = ACTIONS(3129),
    [sym_crate] = ACTIONS(3129),
    [sym_metavariable] = ACTIONS(3131),
  },
  [857] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2994),
    [sym_bracketed_type] = STATE(3361),
    [sym_qualified_type] = STATE(3468),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(857),
    [sym_block_comment] = STATE(857),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [858] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_type_parameters] = STATE(939),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2317),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(2397),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(2192),
    [sym_line_comment] = STATE(858),
    [sym_block_comment] = STATE(858),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3203),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(3205),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(3179),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [859] = {
    [sym_function_modifiers] = STATE(3341),
    [sym_removed_trait_bound] = STATE(1684),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1752),
    [sym_bracketed_type] = STATE(3559),
    [sym_lifetime] = STATE(841),
    [sym_array_type] = STATE(1684),
    [sym_for_lifetimes] = STATE(1591),
    [sym_function_type] = STATE(1684),
    [sym_tuple_type] = STATE(1684),
    [sym_unit_type] = STATE(1684),
    [sym_generic_type] = STATE(1649),
    [sym_generic_type_with_turbofish] = STATE(3551),
    [sym_bounded_type] = STATE(1684),
    [sym_reference_type] = STATE(1684),
    [sym_pointer_type] = STATE(1684),
    [sym_never_type] = STATE(1684),
    [sym_abstract_type] = STATE(1684),
    [sym_dynamic_type] = STATE(1684),
    [sym_macro_invocation] = STATE(1684),
    [sym_scoped_identifier] = STATE(3288),
    [sym_scoped_type_identifier] = STATE(1548),
    [sym_line_comment] = STATE(859),
    [sym_block_comment] = STATE(859),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3139),
    [anon_sym_LPAREN] = ACTIONS(3141),
    [anon_sym_LBRACK] = ACTIONS(3143),
    [anon_sym_STAR] = ACTIONS(3147),
    [anon_sym_QMARK] = ACTIONS(3149),
    [anon_sym_u8] = ACTIONS(3151),
    [anon_sym_i8] = ACTIONS(3151),
    [anon_sym_u16] = ACTIONS(3151),
    [anon_sym_i16] = ACTIONS(3151),
    [anon_sym_u32] = ACTIONS(3151),
    [anon_sym_i32] = ACTIONS(3151),
    [anon_sym_u64] = ACTIONS(3151),
    [anon_sym_i64] = ACTIONS(3151),
    [anon_sym_u128] = ACTIONS(3151),
    [anon_sym_i128] = ACTIONS(3151),
    [anon_sym_isize] = ACTIONS(3151),
    [anon_sym_usize] = ACTIONS(3151),
    [anon_sym_f32] = ACTIONS(3151),
    [anon_sym_f64] = ACTIONS(3151),
    [anon_sym_bool] = ACTIONS(3151),
    [anon_sym_str] = ACTIONS(3151),
    [anon_sym_char] = ACTIONS(3151),
    [anon_sym_BANG] = ACTIONS(3153),
    [anon_sym_AMP] = ACTIONS(3155),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(3157),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(3159),
    [anon_sym_fn] = ACTIONS(3161),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(3163),
    [anon_sym_impl] = ACTIONS(3165),
    [anon_sym_union] = ACTIONS(3163),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(3167),
    [sym_mutable_specifier] = ACTIONS(3207),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3171),
    [sym_super] = ACTIONS(3171),
    [sym_crate] = ACTIONS(3171),
    [sym_metavariable] = ACTIONS(3173),
  },
  [860] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_type_parameters] = STATE(955),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2394),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(2398),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(2152),
    [sym_line_comment] = STATE(860),
    [sym_block_comment] = STATE(860),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3209),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(3211),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(3179),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [861] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_type_parameters] = STATE(889),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2312),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(2315),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(2167),
    [sym_line_comment] = STATE(861),
    [sym_block_comment] = STATE(861),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3213),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(3215),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(3179),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [862] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(3000),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(862),
    [sym_block_comment] = STATE(862),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_RPAREN] = ACTIONS(3217),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [863] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2747),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(863),
    [sym_block_comment] = STATE(863),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_RPAREN] = ACTIONS(3219),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [864] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(3117),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(837),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(864),
    [sym_block_comment] = STATE(864),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [sym_mutable_specifier] = ACTIONS(3221),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [865] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2661),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(865),
    [sym_block_comment] = STATE(865),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3223),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [866] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2576),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(866),
    [sym_block_comment] = STATE(866),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [867] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(3129),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(867),
    [sym_block_comment] = STATE(867),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [868] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2930),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(868),
    [sym_block_comment] = STATE(868),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [869] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(3262),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(869),
    [sym_block_comment] = STATE(869),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [870] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2202),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(870),
    [sym_block_comment] = STATE(870),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [871] = {
    [sym_function_modifiers] = STATE(3367),
    [sym_removed_trait_bound] = STATE(1416),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1380),
    [sym_bracketed_type] = STATE(3544),
    [sym_lifetime] = STATE(3348),
    [sym_array_type] = STATE(1416),
    [sym_for_lifetimes] = STATE(1584),
    [sym_function_type] = STATE(1416),
    [sym_tuple_type] = STATE(1416),
    [sym_unit_type] = STATE(1416),
    [sym_generic_type] = STATE(1083),
    [sym_generic_type_with_turbofish] = STATE(3535),
    [sym_bounded_type] = STATE(1416),
    [sym_reference_type] = STATE(1416),
    [sym_pointer_type] = STATE(1416),
    [sym_never_type] = STATE(1416),
    [sym_abstract_type] = STATE(1416),
    [sym_dynamic_type] = STATE(1416),
    [sym_macro_invocation] = STATE(1416),
    [sym_scoped_identifier] = STATE(3224),
    [sym_scoped_type_identifier] = STATE(1034),
    [sym_line_comment] = STATE(871),
    [sym_block_comment] = STATE(871),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3097),
    [anon_sym_LPAREN] = ACTIONS(3099),
    [anon_sym_LBRACK] = ACTIONS(3101),
    [anon_sym_STAR] = ACTIONS(3105),
    [anon_sym_QMARK] = ACTIONS(3107),
    [anon_sym_u8] = ACTIONS(3109),
    [anon_sym_i8] = ACTIONS(3109),
    [anon_sym_u16] = ACTIONS(3109),
    [anon_sym_i16] = ACTIONS(3109),
    [anon_sym_u32] = ACTIONS(3109),
    [anon_sym_i32] = ACTIONS(3109),
    [anon_sym_u64] = ACTIONS(3109),
    [anon_sym_i64] = ACTIONS(3109),
    [anon_sym_u128] = ACTIONS(3109),
    [anon_sym_i128] = ACTIONS(3109),
    [anon_sym_isize] = ACTIONS(3109),
    [anon_sym_usize] = ACTIONS(3109),
    [anon_sym_f32] = ACTIONS(3109),
    [anon_sym_f64] = ACTIONS(3109),
    [anon_sym_bool] = ACTIONS(3109),
    [anon_sym_str] = ACTIONS(3109),
    [anon_sym_char] = ACTIONS(3109),
    [anon_sym_BANG] = ACTIONS(3111),
    [anon_sym_AMP] = ACTIONS(3113),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(3115),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(3117),
    [anon_sym_fn] = ACTIONS(3119),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(3121),
    [anon_sym_impl] = ACTIONS(3123),
    [anon_sym_union] = ACTIONS(3121),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(3125),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3129),
    [sym_super] = ACTIONS(3129),
    [sym_crate] = ACTIONS(3129),
    [sym_metavariable] = ACTIONS(3131),
  },
  [872] = {
    [sym_function_modifiers] = STATE(3367),
    [sym_removed_trait_bound] = STATE(1416),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1381),
    [sym_bracketed_type] = STATE(3544),
    [sym_lifetime] = STATE(3348),
    [sym_array_type] = STATE(1416),
    [sym_for_lifetimes] = STATE(1584),
    [sym_function_type] = STATE(1416),
    [sym_tuple_type] = STATE(1416),
    [sym_unit_type] = STATE(1416),
    [sym_generic_type] = STATE(1083),
    [sym_generic_type_with_turbofish] = STATE(3535),
    [sym_bounded_type] = STATE(1416),
    [sym_reference_type] = STATE(1416),
    [sym_pointer_type] = STATE(1416),
    [sym_never_type] = STATE(1416),
    [sym_abstract_type] = STATE(1416),
    [sym_dynamic_type] = STATE(1416),
    [sym_macro_invocation] = STATE(1416),
    [sym_scoped_identifier] = STATE(3224),
    [sym_scoped_type_identifier] = STATE(1034),
    [sym_line_comment] = STATE(872),
    [sym_block_comment] = STATE(872),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3097),
    [anon_sym_LPAREN] = ACTIONS(3099),
    [anon_sym_LBRACK] = ACTIONS(3101),
    [anon_sym_STAR] = ACTIONS(3105),
    [anon_sym_QMARK] = ACTIONS(3107),
    [anon_sym_u8] = ACTIONS(3109),
    [anon_sym_i8] = ACTIONS(3109),
    [anon_sym_u16] = ACTIONS(3109),
    [anon_sym_i16] = ACTIONS(3109),
    [anon_sym_u32] = ACTIONS(3109),
    [anon_sym_i32] = ACTIONS(3109),
    [anon_sym_u64] = ACTIONS(3109),
    [anon_sym_i64] = ACTIONS(3109),
    [anon_sym_u128] = ACTIONS(3109),
    [anon_sym_i128] = ACTIONS(3109),
    [anon_sym_isize] = ACTIONS(3109),
    [anon_sym_usize] = ACTIONS(3109),
    [anon_sym_f32] = ACTIONS(3109),
    [anon_sym_f64] = ACTIONS(3109),
    [anon_sym_bool] = ACTIONS(3109),
    [anon_sym_str] = ACTIONS(3109),
    [anon_sym_char] = ACTIONS(3109),
    [anon_sym_BANG] = ACTIONS(3111),
    [anon_sym_AMP] = ACTIONS(3113),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(3115),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(3117),
    [anon_sym_fn] = ACTIONS(3119),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(3121),
    [anon_sym_impl] = ACTIONS(3123),
    [anon_sym_union] = ACTIONS(3121),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(3125),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3129),
    [sym_super] = ACTIONS(3129),
    [sym_crate] = ACTIONS(3129),
    [sym_metavariable] = ACTIONS(3131),
  },
  [873] = {
    [sym_function_modifiers] = STATE(3341),
    [sym_removed_trait_bound] = STATE(1684),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1777),
    [sym_bracketed_type] = STATE(3559),
    [sym_lifetime] = STATE(3394),
    [sym_array_type] = STATE(1684),
    [sym_for_lifetimes] = STATE(1591),
    [sym_function_type] = STATE(1684),
    [sym_tuple_type] = STATE(1684),
    [sym_unit_type] = STATE(1684),
    [sym_generic_type] = STATE(1649),
    [sym_generic_type_with_turbofish] = STATE(3551),
    [sym_bounded_type] = STATE(1684),
    [sym_reference_type] = STATE(1684),
    [sym_pointer_type] = STATE(1684),
    [sym_never_type] = STATE(1684),
    [sym_abstract_type] = STATE(1684),
    [sym_dynamic_type] = STATE(1684),
    [sym_macro_invocation] = STATE(1684),
    [sym_scoped_identifier] = STATE(3288),
    [sym_scoped_type_identifier] = STATE(1548),
    [sym_line_comment] = STATE(873),
    [sym_block_comment] = STATE(873),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3139),
    [anon_sym_LPAREN] = ACTIONS(3141),
    [anon_sym_LBRACK] = ACTIONS(3143),
    [anon_sym_STAR] = ACTIONS(3147),
    [anon_sym_QMARK] = ACTIONS(3149),
    [anon_sym_u8] = ACTIONS(3151),
    [anon_sym_i8] = ACTIONS(3151),
    [anon_sym_u16] = ACTIONS(3151),
    [anon_sym_i16] = ACTIONS(3151),
    [anon_sym_u32] = ACTIONS(3151),
    [anon_sym_i32] = ACTIONS(3151),
    [anon_sym_u64] = ACTIONS(3151),
    [anon_sym_i64] = ACTIONS(3151),
    [anon_sym_u128] = ACTIONS(3151),
    [anon_sym_i128] = ACTIONS(3151),
    [anon_sym_isize] = ACTIONS(3151),
    [anon_sym_usize] = ACTIONS(3151),
    [anon_sym_f32] = ACTIONS(3151),
    [anon_sym_f64] = ACTIONS(3151),
    [anon_sym_bool] = ACTIONS(3151),
    [anon_sym_str] = ACTIONS(3151),
    [anon_sym_char] = ACTIONS(3151),
    [anon_sym_BANG] = ACTIONS(3153),
    [anon_sym_AMP] = ACTIONS(3155),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(3157),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(3159),
    [anon_sym_fn] = ACTIONS(3161),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(3163),
    [anon_sym_impl] = ACTIONS(3165),
    [anon_sym_union] = ACTIONS(3163),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(3167),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3171),
    [sym_super] = ACTIONS(3171),
    [sym_crate] = ACTIONS(3171),
    [sym_metavariable] = ACTIONS(3173),
  },
  [874] = {
    [sym_function_modifiers] = STATE(3341),
    [sym_removed_trait_bound] = STATE(1684),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1728),
    [sym_bracketed_type] = STATE(3559),
    [sym_lifetime] = STATE(3394),
    [sym_array_type] = STATE(1684),
    [sym_for_lifetimes] = STATE(1591),
    [sym_function_type] = STATE(1684),
    [sym_tuple_type] = STATE(1684),
    [sym_unit_type] = STATE(1684),
    [sym_generic_type] = STATE(1649),
    [sym_generic_type_with_turbofish] = STATE(3551),
    [sym_bounded_type] = STATE(1684),
    [sym_reference_type] = STATE(1684),
    [sym_pointer_type] = STATE(1684),
    [sym_never_type] = STATE(1684),
    [sym_abstract_type] = STATE(1684),
    [sym_dynamic_type] = STATE(1684),
    [sym_macro_invocation] = STATE(1684),
    [sym_scoped_identifier] = STATE(3288),
    [sym_scoped_type_identifier] = STATE(1548),
    [sym_line_comment] = STATE(874),
    [sym_block_comment] = STATE(874),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3139),
    [anon_sym_LPAREN] = ACTIONS(3141),
    [anon_sym_LBRACK] = ACTIONS(3143),
    [anon_sym_STAR] = ACTIONS(3147),
    [anon_sym_QMARK] = ACTIONS(3149),
    [anon_sym_u8] = ACTIONS(3151),
    [anon_sym_i8] = ACTIONS(3151),
    [anon_sym_u16] = ACTIONS(3151),
    [anon_sym_i16] = ACTIONS(3151),
    [anon_sym_u32] = ACTIONS(3151),
    [anon_sym_i32] = ACTIONS(3151),
    [anon_sym_u64] = ACTIONS(3151),
    [anon_sym_i64] = ACTIONS(3151),
    [anon_sym_u128] = ACTIONS(3151),
    [anon_sym_i128] = ACTIONS(3151),
    [anon_sym_isize] = ACTIONS(3151),
    [anon_sym_usize] = ACTIONS(3151),
    [anon_sym_f32] = ACTIONS(3151),
    [anon_sym_f64] = ACTIONS(3151),
    [anon_sym_bool] = ACTIONS(3151),
    [anon_sym_str] = ACTIONS(3151),
    [anon_sym_char] = ACTIONS(3151),
    [anon_sym_BANG] = ACTIONS(3153),
    [anon_sym_AMP] = ACTIONS(3155),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(3157),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(3159),
    [anon_sym_fn] = ACTIONS(3161),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(3163),
    [anon_sym_impl] = ACTIONS(3165),
    [anon_sym_union] = ACTIONS(3163),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(3167),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3171),
    [sym_super] = ACTIONS(3171),
    [sym_crate] = ACTIONS(3171),
    [sym_metavariable] = ACTIONS(3173),
  },
  [875] = {
    [sym_function_modifiers] = STATE(3341),
    [sym_removed_trait_bound] = STATE(1684),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1778),
    [sym_bracketed_type] = STATE(3559),
    [sym_lifetime] = STATE(3394),
    [sym_array_type] = STATE(1684),
    [sym_for_lifetimes] = STATE(1591),
    [sym_function_type] = STATE(1684),
    [sym_tuple_type] = STATE(1684),
    [sym_unit_type] = STATE(1684),
    [sym_generic_type] = STATE(1649),
    [sym_generic_type_with_turbofish] = STATE(3551),
    [sym_bounded_type] = STATE(1684),
    [sym_reference_type] = STATE(1684),
    [sym_pointer_type] = STATE(1684),
    [sym_never_type] = STATE(1684),
    [sym_abstract_type] = STATE(1684),
    [sym_dynamic_type] = STATE(1684),
    [sym_macro_invocation] = STATE(1684),
    [sym_scoped_identifier] = STATE(3288),
    [sym_scoped_type_identifier] = STATE(1548),
    [sym_line_comment] = STATE(875),
    [sym_block_comment] = STATE(875),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3139),
    [anon_sym_LPAREN] = ACTIONS(3141),
    [anon_sym_LBRACK] = ACTIONS(3143),
    [anon_sym_STAR] = ACTIONS(3147),
    [anon_sym_QMARK] = ACTIONS(3149),
    [anon_sym_u8] = ACTIONS(3151),
    [anon_sym_i8] = ACTIONS(3151),
    [anon_sym_u16] = ACTIONS(3151),
    [anon_sym_i16] = ACTIONS(3151),
    [anon_sym_u32] = ACTIONS(3151),
    [anon_sym_i32] = ACTIONS(3151),
    [anon_sym_u64] = ACTIONS(3151),
    [anon_sym_i64] = ACTIONS(3151),
    [anon_sym_u128] = ACTIONS(3151),
    [anon_sym_i128] = ACTIONS(3151),
    [anon_sym_isize] = ACTIONS(3151),
    [anon_sym_usize] = ACTIONS(3151),
    [anon_sym_f32] = ACTIONS(3151),
    [anon_sym_f64] = ACTIONS(3151),
    [anon_sym_bool] = ACTIONS(3151),
    [anon_sym_str] = ACTIONS(3151),
    [anon_sym_char] = ACTIONS(3151),
    [anon_sym_BANG] = ACTIONS(3153),
    [anon_sym_AMP] = ACTIONS(3155),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(3157),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(3159),
    [anon_sym_fn] = ACTIONS(3161),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(3163),
    [anon_sym_impl] = ACTIONS(3165),
    [anon_sym_union] = ACTIONS(3163),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(3167),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3171),
    [sym_super] = ACTIONS(3171),
    [sym_crate] = ACTIONS(3171),
    [sym_metavariable] = ACTIONS(3173),
  },
  [876] = {
    [sym_function_modifiers] = STATE(3341),
    [sym_removed_trait_bound] = STATE(1684),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1730),
    [sym_bracketed_type] = STATE(3559),
    [sym_lifetime] = STATE(3394),
    [sym_array_type] = STATE(1684),
    [sym_for_lifetimes] = STATE(1591),
    [sym_function_type] = STATE(1684),
    [sym_tuple_type] = STATE(1684),
    [sym_unit_type] = STATE(1684),
    [sym_generic_type] = STATE(1649),
    [sym_generic_type_with_turbofish] = STATE(3551),
    [sym_bounded_type] = STATE(1684),
    [sym_reference_type] = STATE(1684),
    [sym_pointer_type] = STATE(1684),
    [sym_never_type] = STATE(1684),
    [sym_abstract_type] = STATE(1684),
    [sym_dynamic_type] = STATE(1684),
    [sym_macro_invocation] = STATE(1684),
    [sym_scoped_identifier] = STATE(3288),
    [sym_scoped_type_identifier] = STATE(1548),
    [sym_line_comment] = STATE(876),
    [sym_block_comment] = STATE(876),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3139),
    [anon_sym_LPAREN] = ACTIONS(3141),
    [anon_sym_LBRACK] = ACTIONS(3143),
    [anon_sym_STAR] = ACTIONS(3147),
    [anon_sym_QMARK] = ACTIONS(3149),
    [anon_sym_u8] = ACTIONS(3151),
    [anon_sym_i8] = ACTIONS(3151),
    [anon_sym_u16] = ACTIONS(3151),
    [anon_sym_i16] = ACTIONS(3151),
    [anon_sym_u32] = ACTIONS(3151),
    [anon_sym_i32] = ACTIONS(3151),
    [anon_sym_u64] = ACTIONS(3151),
    [anon_sym_i64] = ACTIONS(3151),
    [anon_sym_u128] = ACTIONS(3151),
    [anon_sym_i128] = ACTIONS(3151),
    [anon_sym_isize] = ACTIONS(3151),
    [anon_sym_usize] = ACTIONS(3151),
    [anon_sym_f32] = ACTIONS(3151),
    [anon_sym_f64] = ACTIONS(3151),
    [anon_sym_bool] = ACTIONS(3151),
    [anon_sym_str] = ACTIONS(3151),
    [anon_sym_char] = ACTIONS(3151),
    [anon_sym_BANG] = ACTIONS(3153),
    [anon_sym_AMP] = ACTIONS(3155),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(3157),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(3159),
    [anon_sym_fn] = ACTIONS(3161),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(3163),
    [anon_sym_impl] = ACTIONS(3165),
    [anon_sym_union] = ACTIONS(3163),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(3167),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3171),
    [sym_super] = ACTIONS(3171),
    [sym_crate] = ACTIONS(3171),
    [sym_metavariable] = ACTIONS(3173),
  },
  [877] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2747),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(877),
    [sym_block_comment] = STATE(877),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [878] = {
    [sym_function_modifiers] = STATE(3341),
    [sym_removed_trait_bound] = STATE(1684),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1731),
    [sym_bracketed_type] = STATE(3559),
    [sym_lifetime] = STATE(3394),
    [sym_array_type] = STATE(1684),
    [sym_for_lifetimes] = STATE(1591),
    [sym_function_type] = STATE(1684),
    [sym_tuple_type] = STATE(1684),
    [sym_unit_type] = STATE(1684),
    [sym_generic_type] = STATE(1649),
    [sym_generic_type_with_turbofish] = STATE(3551),
    [sym_bounded_type] = STATE(1684),
    [sym_reference_type] = STATE(1684),
    [sym_pointer_type] = STATE(1684),
    [sym_never_type] = STATE(1684),
    [sym_abstract_type] = STATE(1684),
    [sym_dynamic_type] = STATE(1684),
    [sym_macro_invocation] = STATE(1684),
    [sym_scoped_identifier] = STATE(3288),
    [sym_scoped_type_identifier] = STATE(1548),
    [sym_line_comment] = STATE(878),
    [sym_block_comment] = STATE(878),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3139),
    [anon_sym_LPAREN] = ACTIONS(3141),
    [anon_sym_LBRACK] = ACTIONS(3143),
    [anon_sym_STAR] = ACTIONS(3147),
    [anon_sym_QMARK] = ACTIONS(3149),
    [anon_sym_u8] = ACTIONS(3151),
    [anon_sym_i8] = ACTIONS(3151),
    [anon_sym_u16] = ACTIONS(3151),
    [anon_sym_i16] = ACTIONS(3151),
    [anon_sym_u32] = ACTIONS(3151),
    [anon_sym_i32] = ACTIONS(3151),
    [anon_sym_u64] = ACTIONS(3151),
    [anon_sym_i64] = ACTIONS(3151),
    [anon_sym_u128] = ACTIONS(3151),
    [anon_sym_i128] = ACTIONS(3151),
    [anon_sym_isize] = ACTIONS(3151),
    [anon_sym_usize] = ACTIONS(3151),
    [anon_sym_f32] = ACTIONS(3151),
    [anon_sym_f64] = ACTIONS(3151),
    [anon_sym_bool] = ACTIONS(3151),
    [anon_sym_str] = ACTIONS(3151),
    [anon_sym_char] = ACTIONS(3151),
    [anon_sym_BANG] = ACTIONS(3153),
    [anon_sym_AMP] = ACTIONS(3155),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(3157),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(3159),
    [anon_sym_fn] = ACTIONS(3161),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(3163),
    [anon_sym_impl] = ACTIONS(3165),
    [anon_sym_union] = ACTIONS(3163),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(3167),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3171),
    [sym_super] = ACTIONS(3171),
    [sym_crate] = ACTIONS(3171),
    [sym_metavariable] = ACTIONS(3173),
  },
  [879] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2014),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(1999),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(879),
    [sym_block_comment] = STATE(879),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [880] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2767),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(880),
    [sym_block_comment] = STATE(880),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [881] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2368),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(881),
    [sym_block_comment] = STATE(881),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [882] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2371),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(882),
    [sym_block_comment] = STATE(882),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [883] = {
    [sym_function_modifiers] = STATE(3341),
    [sym_removed_trait_bound] = STATE(1684),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1736),
    [sym_bracketed_type] = STATE(3559),
    [sym_lifetime] = STATE(3394),
    [sym_array_type] = STATE(1684),
    [sym_for_lifetimes] = STATE(1591),
    [sym_function_type] = STATE(1684),
    [sym_tuple_type] = STATE(1684),
    [sym_unit_type] = STATE(1684),
    [sym_generic_type] = STATE(1649),
    [sym_generic_type_with_turbofish] = STATE(3551),
    [sym_bounded_type] = STATE(1684),
    [sym_reference_type] = STATE(1684),
    [sym_pointer_type] = STATE(1684),
    [sym_never_type] = STATE(1684),
    [sym_abstract_type] = STATE(1684),
    [sym_dynamic_type] = STATE(1684),
    [sym_macro_invocation] = STATE(1684),
    [sym_scoped_identifier] = STATE(3288),
    [sym_scoped_type_identifier] = STATE(1548),
    [sym_line_comment] = STATE(883),
    [sym_block_comment] = STATE(883),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3139),
    [anon_sym_LPAREN] = ACTIONS(3141),
    [anon_sym_LBRACK] = ACTIONS(3143),
    [anon_sym_STAR] = ACTIONS(3147),
    [anon_sym_QMARK] = ACTIONS(3149),
    [anon_sym_u8] = ACTIONS(3151),
    [anon_sym_i8] = ACTIONS(3151),
    [anon_sym_u16] = ACTIONS(3151),
    [anon_sym_i16] = ACTIONS(3151),
    [anon_sym_u32] = ACTIONS(3151),
    [anon_sym_i32] = ACTIONS(3151),
    [anon_sym_u64] = ACTIONS(3151),
    [anon_sym_i64] = ACTIONS(3151),
    [anon_sym_u128] = ACTIONS(3151),
    [anon_sym_i128] = ACTIONS(3151),
    [anon_sym_isize] = ACTIONS(3151),
    [anon_sym_usize] = ACTIONS(3151),
    [anon_sym_f32] = ACTIONS(3151),
    [anon_sym_f64] = ACTIONS(3151),
    [anon_sym_bool] = ACTIONS(3151),
    [anon_sym_str] = ACTIONS(3151),
    [anon_sym_char] = ACTIONS(3151),
    [anon_sym_BANG] = ACTIONS(3153),
    [anon_sym_AMP] = ACTIONS(3155),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(3157),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(3159),
    [anon_sym_fn] = ACTIONS(3161),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(3163),
    [anon_sym_impl] = ACTIONS(3165),
    [anon_sym_union] = ACTIONS(3163),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(3167),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3171),
    [sym_super] = ACTIONS(3171),
    [sym_crate] = ACTIONS(3171),
    [sym_metavariable] = ACTIONS(3173),
  },
  [884] = {
    [sym_function_modifiers] = STATE(3341),
    [sym_removed_trait_bound] = STATE(1684),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1737),
    [sym_bracketed_type] = STATE(3559),
    [sym_lifetime] = STATE(3394),
    [sym_array_type] = STATE(1684),
    [sym_for_lifetimes] = STATE(1591),
    [sym_function_type] = STATE(1684),
    [sym_tuple_type] = STATE(1684),
    [sym_unit_type] = STATE(1684),
    [sym_generic_type] = STATE(1649),
    [sym_generic_type_with_turbofish] = STATE(3551),
    [sym_bounded_type] = STATE(1684),
    [sym_reference_type] = STATE(1684),
    [sym_pointer_type] = STATE(1684),
    [sym_never_type] = STATE(1684),
    [sym_abstract_type] = STATE(1684),
    [sym_dynamic_type] = STATE(1684),
    [sym_macro_invocation] = STATE(1684),
    [sym_scoped_identifier] = STATE(3288),
    [sym_scoped_type_identifier] = STATE(1548),
    [sym_line_comment] = STATE(884),
    [sym_block_comment] = STATE(884),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3139),
    [anon_sym_LPAREN] = ACTIONS(3141),
    [anon_sym_LBRACK] = ACTIONS(3143),
    [anon_sym_STAR] = ACTIONS(3147),
    [anon_sym_QMARK] = ACTIONS(3149),
    [anon_sym_u8] = ACTIONS(3151),
    [anon_sym_i8] = ACTIONS(3151),
    [anon_sym_u16] = ACTIONS(3151),
    [anon_sym_i16] = ACTIONS(3151),
    [anon_sym_u32] = ACTIONS(3151),
    [anon_sym_i32] = ACTIONS(3151),
    [anon_sym_u64] = ACTIONS(3151),
    [anon_sym_i64] = ACTIONS(3151),
    [anon_sym_u128] = ACTIONS(3151),
    [anon_sym_i128] = ACTIONS(3151),
    [anon_sym_isize] = ACTIONS(3151),
    [anon_sym_usize] = ACTIONS(3151),
    [anon_sym_f32] = ACTIONS(3151),
    [anon_sym_f64] = ACTIONS(3151),
    [anon_sym_bool] = ACTIONS(3151),
    [anon_sym_str] = ACTIONS(3151),
    [anon_sym_char] = ACTIONS(3151),
    [anon_sym_BANG] = ACTIONS(3153),
    [anon_sym_AMP] = ACTIONS(3155),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(3157),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(3159),
    [anon_sym_fn] = ACTIONS(3161),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(3163),
    [anon_sym_impl] = ACTIONS(3165),
    [anon_sym_union] = ACTIONS(3163),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(3167),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3171),
    [sym_super] = ACTIONS(3171),
    [sym_crate] = ACTIONS(3171),
    [sym_metavariable] = ACTIONS(3173),
  },
  [885] = {
    [sym_function_modifiers] = STATE(3341),
    [sym_removed_trait_bound] = STATE(1684),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1738),
    [sym_bracketed_type] = STATE(3559),
    [sym_lifetime] = STATE(3394),
    [sym_array_type] = STATE(1684),
    [sym_for_lifetimes] = STATE(1591),
    [sym_function_type] = STATE(1684),
    [sym_tuple_type] = STATE(1684),
    [sym_unit_type] = STATE(1684),
    [sym_generic_type] = STATE(1649),
    [sym_generic_type_with_turbofish] = STATE(3551),
    [sym_bounded_type] = STATE(1684),
    [sym_reference_type] = STATE(1684),
    [sym_pointer_type] = STATE(1684),
    [sym_never_type] = STATE(1684),
    [sym_abstract_type] = STATE(1684),
    [sym_dynamic_type] = STATE(1684),
    [sym_macro_invocation] = STATE(1684),
    [sym_scoped_identifier] = STATE(3288),
    [sym_scoped_type_identifier] = STATE(1548),
    [sym_line_comment] = STATE(885),
    [sym_block_comment] = STATE(885),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3139),
    [anon_sym_LPAREN] = ACTIONS(3141),
    [anon_sym_LBRACK] = ACTIONS(3143),
    [anon_sym_STAR] = ACTIONS(3147),
    [anon_sym_QMARK] = ACTIONS(3149),
    [anon_sym_u8] = ACTIONS(3151),
    [anon_sym_i8] = ACTIONS(3151),
    [anon_sym_u16] = ACTIONS(3151),
    [anon_sym_i16] = ACTIONS(3151),
    [anon_sym_u32] = ACTIONS(3151),
    [anon_sym_i32] = ACTIONS(3151),
    [anon_sym_u64] = ACTIONS(3151),
    [anon_sym_i64] = ACTIONS(3151),
    [anon_sym_u128] = ACTIONS(3151),
    [anon_sym_i128] = ACTIONS(3151),
    [anon_sym_isize] = ACTIONS(3151),
    [anon_sym_usize] = ACTIONS(3151),
    [anon_sym_f32] = ACTIONS(3151),
    [anon_sym_f64] = ACTIONS(3151),
    [anon_sym_bool] = ACTIONS(3151),
    [anon_sym_str] = ACTIONS(3151),
    [anon_sym_char] = ACTIONS(3151),
    [anon_sym_BANG] = ACTIONS(3153),
    [anon_sym_AMP] = ACTIONS(3155),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(3157),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(3159),
    [anon_sym_fn] = ACTIONS(3161),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(3163),
    [anon_sym_impl] = ACTIONS(3165),
    [anon_sym_union] = ACTIONS(3163),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(3167),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3171),
    [sym_super] = ACTIONS(3171),
    [sym_crate] = ACTIONS(3171),
    [sym_metavariable] = ACTIONS(3173),
  },
  [886] = {
    [sym_function_modifiers] = STATE(3367),
    [sym_removed_trait_bound] = STATE(1416),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1382),
    [sym_bracketed_type] = STATE(3544),
    [sym_lifetime] = STATE(3348),
    [sym_array_type] = STATE(1416),
    [sym_for_lifetimes] = STATE(1584),
    [sym_function_type] = STATE(1416),
    [sym_tuple_type] = STATE(1416),
    [sym_unit_type] = STATE(1416),
    [sym_generic_type] = STATE(1083),
    [sym_generic_type_with_turbofish] = STATE(3535),
    [sym_bounded_type] = STATE(1416),
    [sym_reference_type] = STATE(1416),
    [sym_pointer_type] = STATE(1416),
    [sym_never_type] = STATE(1416),
    [sym_abstract_type] = STATE(1416),
    [sym_dynamic_type] = STATE(1416),
    [sym_macro_invocation] = STATE(1416),
    [sym_scoped_identifier] = STATE(3224),
    [sym_scoped_type_identifier] = STATE(1034),
    [sym_line_comment] = STATE(886),
    [sym_block_comment] = STATE(886),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3097),
    [anon_sym_LPAREN] = ACTIONS(3099),
    [anon_sym_LBRACK] = ACTIONS(3101),
    [anon_sym_STAR] = ACTIONS(3105),
    [anon_sym_QMARK] = ACTIONS(3107),
    [anon_sym_u8] = ACTIONS(3109),
    [anon_sym_i8] = ACTIONS(3109),
    [anon_sym_u16] = ACTIONS(3109),
    [anon_sym_i16] = ACTIONS(3109),
    [anon_sym_u32] = ACTIONS(3109),
    [anon_sym_i32] = ACTIONS(3109),
    [anon_sym_u64] = ACTIONS(3109),
    [anon_sym_i64] = ACTIONS(3109),
    [anon_sym_u128] = ACTIONS(3109),
    [anon_sym_i128] = ACTIONS(3109),
    [anon_sym_isize] = ACTIONS(3109),
    [anon_sym_usize] = ACTIONS(3109),
    [anon_sym_f32] = ACTIONS(3109),
    [anon_sym_f64] = ACTIONS(3109),
    [anon_sym_bool] = ACTIONS(3109),
    [anon_sym_str] = ACTIONS(3109),
    [anon_sym_char] = ACTIONS(3109),
    [anon_sym_BANG] = ACTIONS(3111),
    [anon_sym_AMP] = ACTIONS(3113),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(3115),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(3117),
    [anon_sym_fn] = ACTIONS(3119),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(3121),
    [anon_sym_impl] = ACTIONS(3123),
    [anon_sym_union] = ACTIONS(3121),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(3125),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3129),
    [sym_super] = ACTIONS(3129),
    [sym_crate] = ACTIONS(3129),
    [sym_metavariable] = ACTIONS(3131),
  },
  [887] = {
    [sym_function_modifiers] = STATE(3341),
    [sym_removed_trait_bound] = STATE(1684),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1739),
    [sym_bracketed_type] = STATE(3559),
    [sym_lifetime] = STATE(3394),
    [sym_array_type] = STATE(1684),
    [sym_for_lifetimes] = STATE(1591),
    [sym_function_type] = STATE(1684),
    [sym_tuple_type] = STATE(1684),
    [sym_unit_type] = STATE(1684),
    [sym_generic_type] = STATE(1649),
    [sym_generic_type_with_turbofish] = STATE(3551),
    [sym_bounded_type] = STATE(1684),
    [sym_reference_type] = STATE(1684),
    [sym_pointer_type] = STATE(1684),
    [sym_never_type] = STATE(1684),
    [sym_abstract_type] = STATE(1684),
    [sym_dynamic_type] = STATE(1684),
    [sym_macro_invocation] = STATE(1684),
    [sym_scoped_identifier] = STATE(3288),
    [sym_scoped_type_identifier] = STATE(1548),
    [sym_line_comment] = STATE(887),
    [sym_block_comment] = STATE(887),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3139),
    [anon_sym_LPAREN] = ACTIONS(3141),
    [anon_sym_LBRACK] = ACTIONS(3143),
    [anon_sym_STAR] = ACTIONS(3147),
    [anon_sym_QMARK] = ACTIONS(3149),
    [anon_sym_u8] = ACTIONS(3151),
    [anon_sym_i8] = ACTIONS(3151),
    [anon_sym_u16] = ACTIONS(3151),
    [anon_sym_i16] = ACTIONS(3151),
    [anon_sym_u32] = ACTIONS(3151),
    [anon_sym_i32] = ACTIONS(3151),
    [anon_sym_u64] = ACTIONS(3151),
    [anon_sym_i64] = ACTIONS(3151),
    [anon_sym_u128] = ACTIONS(3151),
    [anon_sym_i128] = ACTIONS(3151),
    [anon_sym_isize] = ACTIONS(3151),
    [anon_sym_usize] = ACTIONS(3151),
    [anon_sym_f32] = ACTIONS(3151),
    [anon_sym_f64] = ACTIONS(3151),
    [anon_sym_bool] = ACTIONS(3151),
    [anon_sym_str] = ACTIONS(3151),
    [anon_sym_char] = ACTIONS(3151),
    [anon_sym_BANG] = ACTIONS(3153),
    [anon_sym_AMP] = ACTIONS(3155),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(3157),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(3159),
    [anon_sym_fn] = ACTIONS(3161),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(3163),
    [anon_sym_impl] = ACTIONS(3165),
    [anon_sym_union] = ACTIONS(3163),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(3167),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3171),
    [sym_super] = ACTIONS(3171),
    [sym_crate] = ACTIONS(3171),
    [sym_metavariable] = ACTIONS(3173),
  },
  [888] = {
    [sym_function_modifiers] = STATE(3367),
    [sym_removed_trait_bound] = STATE(1416),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1407),
    [sym_bracketed_type] = STATE(3544),
    [sym_lifetime] = STATE(3348),
    [sym_array_type] = STATE(1416),
    [sym_for_lifetimes] = STATE(1584),
    [sym_function_type] = STATE(1416),
    [sym_tuple_type] = STATE(1416),
    [sym_unit_type] = STATE(1416),
    [sym_generic_type] = STATE(1083),
    [sym_generic_type_with_turbofish] = STATE(3535),
    [sym_bounded_type] = STATE(1416),
    [sym_reference_type] = STATE(1416),
    [sym_pointer_type] = STATE(1416),
    [sym_never_type] = STATE(1416),
    [sym_abstract_type] = STATE(1416),
    [sym_dynamic_type] = STATE(1416),
    [sym_macro_invocation] = STATE(1416),
    [sym_scoped_identifier] = STATE(3224),
    [sym_scoped_type_identifier] = STATE(1034),
    [sym_line_comment] = STATE(888),
    [sym_block_comment] = STATE(888),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3097),
    [anon_sym_LPAREN] = ACTIONS(3099),
    [anon_sym_LBRACK] = ACTIONS(3101),
    [anon_sym_STAR] = ACTIONS(3105),
    [anon_sym_QMARK] = ACTIONS(3107),
    [anon_sym_u8] = ACTIONS(3109),
    [anon_sym_i8] = ACTIONS(3109),
    [anon_sym_u16] = ACTIONS(3109),
    [anon_sym_i16] = ACTIONS(3109),
    [anon_sym_u32] = ACTIONS(3109),
    [anon_sym_i32] = ACTIONS(3109),
    [anon_sym_u64] = ACTIONS(3109),
    [anon_sym_i64] = ACTIONS(3109),
    [anon_sym_u128] = ACTIONS(3109),
    [anon_sym_i128] = ACTIONS(3109),
    [anon_sym_isize] = ACTIONS(3109),
    [anon_sym_usize] = ACTIONS(3109),
    [anon_sym_f32] = ACTIONS(3109),
    [anon_sym_f64] = ACTIONS(3109),
    [anon_sym_bool] = ACTIONS(3109),
    [anon_sym_str] = ACTIONS(3109),
    [anon_sym_char] = ACTIONS(3109),
    [anon_sym_BANG] = ACTIONS(3111),
    [anon_sym_AMP] = ACTIONS(3113),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(3115),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(3117),
    [anon_sym_fn] = ACTIONS(3119),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(3121),
    [anon_sym_impl] = ACTIONS(3123),
    [anon_sym_union] = ACTIONS(3121),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(3125),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3129),
    [sym_super] = ACTIONS(3129),
    [sym_crate] = ACTIONS(3129),
    [sym_metavariable] = ACTIONS(3131),
  },
  [889] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2365),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(2366),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(2175),
    [sym_line_comment] = STATE(889),
    [sym_block_comment] = STATE(889),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3225),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(3227),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [890] = {
    [sym_function_modifiers] = STATE(3367),
    [sym_removed_trait_bound] = STATE(1416),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1409),
    [sym_bracketed_type] = STATE(3544),
    [sym_lifetime] = STATE(3348),
    [sym_array_type] = STATE(1416),
    [sym_for_lifetimes] = STATE(1584),
    [sym_function_type] = STATE(1416),
    [sym_tuple_type] = STATE(1416),
    [sym_unit_type] = STATE(1416),
    [sym_generic_type] = STATE(1083),
    [sym_generic_type_with_turbofish] = STATE(3535),
    [sym_bounded_type] = STATE(1416),
    [sym_reference_type] = STATE(1416),
    [sym_pointer_type] = STATE(1416),
    [sym_never_type] = STATE(1416),
    [sym_abstract_type] = STATE(1416),
    [sym_dynamic_type] = STATE(1416),
    [sym_macro_invocation] = STATE(1416),
    [sym_scoped_identifier] = STATE(3224),
    [sym_scoped_type_identifier] = STATE(1034),
    [sym_line_comment] = STATE(890),
    [sym_block_comment] = STATE(890),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3097),
    [anon_sym_LPAREN] = ACTIONS(3099),
    [anon_sym_LBRACK] = ACTIONS(3101),
    [anon_sym_STAR] = ACTIONS(3105),
    [anon_sym_QMARK] = ACTIONS(3107),
    [anon_sym_u8] = ACTIONS(3109),
    [anon_sym_i8] = ACTIONS(3109),
    [anon_sym_u16] = ACTIONS(3109),
    [anon_sym_i16] = ACTIONS(3109),
    [anon_sym_u32] = ACTIONS(3109),
    [anon_sym_i32] = ACTIONS(3109),
    [anon_sym_u64] = ACTIONS(3109),
    [anon_sym_i64] = ACTIONS(3109),
    [anon_sym_u128] = ACTIONS(3109),
    [anon_sym_i128] = ACTIONS(3109),
    [anon_sym_isize] = ACTIONS(3109),
    [anon_sym_usize] = ACTIONS(3109),
    [anon_sym_f32] = ACTIONS(3109),
    [anon_sym_f64] = ACTIONS(3109),
    [anon_sym_bool] = ACTIONS(3109),
    [anon_sym_str] = ACTIONS(3109),
    [anon_sym_char] = ACTIONS(3109),
    [anon_sym_BANG] = ACTIONS(3111),
    [anon_sym_AMP] = ACTIONS(3113),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(3115),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(3117),
    [anon_sym_fn] = ACTIONS(3119),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(3121),
    [anon_sym_impl] = ACTIONS(3123),
    [anon_sym_union] = ACTIONS(3121),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(3125),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3129),
    [sym_super] = ACTIONS(3129),
    [sym_crate] = ACTIONS(3129),
    [sym_metavariable] = ACTIONS(3131),
  },
  [891] = {
    [sym_function_modifiers] = STATE(3367),
    [sym_removed_trait_bound] = STATE(1416),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1463),
    [sym_bracketed_type] = STATE(3544),
    [sym_lifetime] = STATE(3348),
    [sym_array_type] = STATE(1416),
    [sym_for_lifetimes] = STATE(1584),
    [sym_function_type] = STATE(1416),
    [sym_tuple_type] = STATE(1416),
    [sym_unit_type] = STATE(1416),
    [sym_generic_type] = STATE(1083),
    [sym_generic_type_with_turbofish] = STATE(3535),
    [sym_bounded_type] = STATE(1416),
    [sym_reference_type] = STATE(1416),
    [sym_pointer_type] = STATE(1416),
    [sym_never_type] = STATE(1416),
    [sym_abstract_type] = STATE(1416),
    [sym_dynamic_type] = STATE(1416),
    [sym_macro_invocation] = STATE(1416),
    [sym_scoped_identifier] = STATE(3224),
    [sym_scoped_type_identifier] = STATE(1034),
    [sym_line_comment] = STATE(891),
    [sym_block_comment] = STATE(891),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3097),
    [anon_sym_LPAREN] = ACTIONS(3099),
    [anon_sym_LBRACK] = ACTIONS(3101),
    [anon_sym_STAR] = ACTIONS(3105),
    [anon_sym_QMARK] = ACTIONS(3107),
    [anon_sym_u8] = ACTIONS(3109),
    [anon_sym_i8] = ACTIONS(3109),
    [anon_sym_u16] = ACTIONS(3109),
    [anon_sym_i16] = ACTIONS(3109),
    [anon_sym_u32] = ACTIONS(3109),
    [anon_sym_i32] = ACTIONS(3109),
    [anon_sym_u64] = ACTIONS(3109),
    [anon_sym_i64] = ACTIONS(3109),
    [anon_sym_u128] = ACTIONS(3109),
    [anon_sym_i128] = ACTIONS(3109),
    [anon_sym_isize] = ACTIONS(3109),
    [anon_sym_usize] = ACTIONS(3109),
    [anon_sym_f32] = ACTIONS(3109),
    [anon_sym_f64] = ACTIONS(3109),
    [anon_sym_bool] = ACTIONS(3109),
    [anon_sym_str] = ACTIONS(3109),
    [anon_sym_char] = ACTIONS(3109),
    [anon_sym_BANG] = ACTIONS(3111),
    [anon_sym_AMP] = ACTIONS(3113),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(3115),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(3117),
    [anon_sym_fn] = ACTIONS(3119),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(3121),
    [anon_sym_impl] = ACTIONS(3123),
    [anon_sym_union] = ACTIONS(3121),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(3125),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3129),
    [sym_super] = ACTIONS(3129),
    [sym_crate] = ACTIONS(3129),
    [sym_metavariable] = ACTIONS(3131),
  },
  [892] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2345),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(892),
    [sym_block_comment] = STATE(892),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [893] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2006),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(893),
    [sym_block_comment] = STATE(893),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [894] = {
    [sym_function_modifiers] = STATE(3341),
    [sym_removed_trait_bound] = STATE(1684),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1693),
    [sym_bracketed_type] = STATE(3559),
    [sym_lifetime] = STATE(3394),
    [sym_array_type] = STATE(1684),
    [sym_for_lifetimes] = STATE(1591),
    [sym_function_type] = STATE(1684),
    [sym_tuple_type] = STATE(1684),
    [sym_unit_type] = STATE(1684),
    [sym_generic_type] = STATE(1649),
    [sym_generic_type_with_turbofish] = STATE(3551),
    [sym_bounded_type] = STATE(1684),
    [sym_reference_type] = STATE(1684),
    [sym_pointer_type] = STATE(1684),
    [sym_never_type] = STATE(1684),
    [sym_abstract_type] = STATE(1684),
    [sym_dynamic_type] = STATE(1684),
    [sym_macro_invocation] = STATE(1684),
    [sym_scoped_identifier] = STATE(3288),
    [sym_scoped_type_identifier] = STATE(1548),
    [sym_line_comment] = STATE(894),
    [sym_block_comment] = STATE(894),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3139),
    [anon_sym_LPAREN] = ACTIONS(3141),
    [anon_sym_LBRACK] = ACTIONS(3143),
    [anon_sym_STAR] = ACTIONS(3147),
    [anon_sym_QMARK] = ACTIONS(3149),
    [anon_sym_u8] = ACTIONS(3151),
    [anon_sym_i8] = ACTIONS(3151),
    [anon_sym_u16] = ACTIONS(3151),
    [anon_sym_i16] = ACTIONS(3151),
    [anon_sym_u32] = ACTIONS(3151),
    [anon_sym_i32] = ACTIONS(3151),
    [anon_sym_u64] = ACTIONS(3151),
    [anon_sym_i64] = ACTIONS(3151),
    [anon_sym_u128] = ACTIONS(3151),
    [anon_sym_i128] = ACTIONS(3151),
    [anon_sym_isize] = ACTIONS(3151),
    [anon_sym_usize] = ACTIONS(3151),
    [anon_sym_f32] = ACTIONS(3151),
    [anon_sym_f64] = ACTIONS(3151),
    [anon_sym_bool] = ACTIONS(3151),
    [anon_sym_str] = ACTIONS(3151),
    [anon_sym_char] = ACTIONS(3151),
    [anon_sym_BANG] = ACTIONS(3153),
    [anon_sym_AMP] = ACTIONS(3155),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(3157),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(3159),
    [anon_sym_fn] = ACTIONS(3161),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(3163),
    [anon_sym_impl] = ACTIONS(3165),
    [anon_sym_union] = ACTIONS(3163),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(3167),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3171),
    [sym_super] = ACTIONS(3171),
    [sym_crate] = ACTIONS(3171),
    [sym_metavariable] = ACTIONS(3173),
  },
  [895] = {
    [sym_function_modifiers] = STATE(3367),
    [sym_removed_trait_bound] = STATE(1416),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1412),
    [sym_bracketed_type] = STATE(3544),
    [sym_lifetime] = STATE(3348),
    [sym_array_type] = STATE(1416),
    [sym_for_lifetimes] = STATE(1584),
    [sym_function_type] = STATE(1416),
    [sym_tuple_type] = STATE(1416),
    [sym_unit_type] = STATE(1416),
    [sym_generic_type] = STATE(1083),
    [sym_generic_type_with_turbofish] = STATE(3535),
    [sym_bounded_type] = STATE(1416),
    [sym_reference_type] = STATE(1416),
    [sym_pointer_type] = STATE(1416),
    [sym_never_type] = STATE(1416),
    [sym_abstract_type] = STATE(1416),
    [sym_dynamic_type] = STATE(1416),
    [sym_macro_invocation] = STATE(1416),
    [sym_scoped_identifier] = STATE(3224),
    [sym_scoped_type_identifier] = STATE(1034),
    [sym_line_comment] = STATE(895),
    [sym_block_comment] = STATE(895),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3097),
    [anon_sym_LPAREN] = ACTIONS(3099),
    [anon_sym_LBRACK] = ACTIONS(3101),
    [anon_sym_STAR] = ACTIONS(3105),
    [anon_sym_QMARK] = ACTIONS(3107),
    [anon_sym_u8] = ACTIONS(3109),
    [anon_sym_i8] = ACTIONS(3109),
    [anon_sym_u16] = ACTIONS(3109),
    [anon_sym_i16] = ACTIONS(3109),
    [anon_sym_u32] = ACTIONS(3109),
    [anon_sym_i32] = ACTIONS(3109),
    [anon_sym_u64] = ACTIONS(3109),
    [anon_sym_i64] = ACTIONS(3109),
    [anon_sym_u128] = ACTIONS(3109),
    [anon_sym_i128] = ACTIONS(3109),
    [anon_sym_isize] = ACTIONS(3109),
    [anon_sym_usize] = ACTIONS(3109),
    [anon_sym_f32] = ACTIONS(3109),
    [anon_sym_f64] = ACTIONS(3109),
    [anon_sym_bool] = ACTIONS(3109),
    [anon_sym_str] = ACTIONS(3109),
    [anon_sym_char] = ACTIONS(3109),
    [anon_sym_BANG] = ACTIONS(3111),
    [anon_sym_AMP] = ACTIONS(3113),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(3115),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(3117),
    [anon_sym_fn] = ACTIONS(3119),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(3121),
    [anon_sym_impl] = ACTIONS(3123),
    [anon_sym_union] = ACTIONS(3121),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(3125),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3129),
    [sym_super] = ACTIONS(3129),
    [sym_crate] = ACTIONS(3129),
    [sym_metavariable] = ACTIONS(3131),
  },
  [896] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2571),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(896),
    [sym_block_comment] = STATE(896),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [897] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2642),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(897),
    [sym_block_comment] = STATE(897),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [898] = {
    [sym_function_modifiers] = STATE(3367),
    [sym_removed_trait_bound] = STATE(1416),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1415),
    [sym_bracketed_type] = STATE(3544),
    [sym_lifetime] = STATE(3348),
    [sym_array_type] = STATE(1416),
    [sym_for_lifetimes] = STATE(1584),
    [sym_function_type] = STATE(1416),
    [sym_tuple_type] = STATE(1416),
    [sym_unit_type] = STATE(1416),
    [sym_generic_type] = STATE(1083),
    [sym_generic_type_with_turbofish] = STATE(3535),
    [sym_bounded_type] = STATE(1416),
    [sym_reference_type] = STATE(1416),
    [sym_pointer_type] = STATE(1416),
    [sym_never_type] = STATE(1416),
    [sym_abstract_type] = STATE(1416),
    [sym_dynamic_type] = STATE(1416),
    [sym_macro_invocation] = STATE(1416),
    [sym_scoped_identifier] = STATE(3224),
    [sym_scoped_type_identifier] = STATE(1034),
    [sym_line_comment] = STATE(898),
    [sym_block_comment] = STATE(898),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3097),
    [anon_sym_LPAREN] = ACTIONS(3099),
    [anon_sym_LBRACK] = ACTIONS(3101),
    [anon_sym_STAR] = ACTIONS(3105),
    [anon_sym_QMARK] = ACTIONS(3107),
    [anon_sym_u8] = ACTIONS(3109),
    [anon_sym_i8] = ACTIONS(3109),
    [anon_sym_u16] = ACTIONS(3109),
    [anon_sym_i16] = ACTIONS(3109),
    [anon_sym_u32] = ACTIONS(3109),
    [anon_sym_i32] = ACTIONS(3109),
    [anon_sym_u64] = ACTIONS(3109),
    [anon_sym_i64] = ACTIONS(3109),
    [anon_sym_u128] = ACTIONS(3109),
    [anon_sym_i128] = ACTIONS(3109),
    [anon_sym_isize] = ACTIONS(3109),
    [anon_sym_usize] = ACTIONS(3109),
    [anon_sym_f32] = ACTIONS(3109),
    [anon_sym_f64] = ACTIONS(3109),
    [anon_sym_bool] = ACTIONS(3109),
    [anon_sym_str] = ACTIONS(3109),
    [anon_sym_char] = ACTIONS(3109),
    [anon_sym_BANG] = ACTIONS(3111),
    [anon_sym_AMP] = ACTIONS(3113),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(3115),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(3117),
    [anon_sym_fn] = ACTIONS(3119),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(3121),
    [anon_sym_impl] = ACTIONS(3123),
    [anon_sym_union] = ACTIONS(3121),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(3125),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3129),
    [sym_super] = ACTIONS(3129),
    [sym_crate] = ACTIONS(3129),
    [sym_metavariable] = ACTIONS(3131),
  },
  [899] = {
    [sym_line_comment] = STATE(899),
    [sym_block_comment] = STATE(899),
    [sym_identifier] = ACTIONS(2101),
    [anon_sym_LPAREN] = ACTIONS(2099),
    [anon_sym_LBRACK] = ACTIONS(2099),
    [anon_sym_RBRACK] = ACTIONS(2099),
    [anon_sym_LBRACE] = ACTIONS(2099),
    [anon_sym_STAR] = ACTIONS(2099),
    [anon_sym_u8] = ACTIONS(2101),
    [anon_sym_i8] = ACTIONS(2101),
    [anon_sym_u16] = ACTIONS(2101),
    [anon_sym_i16] = ACTIONS(2101),
    [anon_sym_u32] = ACTIONS(2101),
    [anon_sym_i32] = ACTIONS(2101),
    [anon_sym_u64] = ACTIONS(2101),
    [anon_sym_i64] = ACTIONS(2101),
    [anon_sym_u128] = ACTIONS(2101),
    [anon_sym_i128] = ACTIONS(2101),
    [anon_sym_isize] = ACTIONS(2101),
    [anon_sym_usize] = ACTIONS(2101),
    [anon_sym_f32] = ACTIONS(2101),
    [anon_sym_f64] = ACTIONS(2101),
    [anon_sym_bool] = ACTIONS(2101),
    [anon_sym_str] = ACTIONS(2101),
    [anon_sym_char] = ACTIONS(2101),
    [anon_sym_DASH] = ACTIONS(2099),
    [anon_sym_BANG] = ACTIONS(2099),
    [anon_sym_AMP] = ACTIONS(2099),
    [anon_sym_PIPE] = ACTIONS(2099),
    [anon_sym_LT] = ACTIONS(2099),
    [anon_sym__] = ACTIONS(2101),
    [anon_sym_DOT_DOT] = ACTIONS(2099),
    [anon_sym_COMMA] = ACTIONS(2099),
    [anon_sym_COLON_COLON] = ACTIONS(2099),
    [anon_sym_POUND] = ACTIONS(2099),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(2101),
    [anon_sym_break] = ACTIONS(2101),
    [anon_sym_const] = ACTIONS(2101),
    [anon_sym_continue] = ACTIONS(2101),
    [anon_sym_default] = ACTIONS(2101),
    [anon_sym_for] = ACTIONS(2101),
    [anon_sym_gen] = ACTIONS(2101),
    [anon_sym_if] = ACTIONS(2101),
    [anon_sym_loop] = ACTIONS(2101),
    [anon_sym_match] = ACTIONS(2101),
    [anon_sym_return] = ACTIONS(2101),
    [anon_sym_static] = ACTIONS(2101),
    [anon_sym_union] = ACTIONS(2101),
    [anon_sym_unsafe] = ACTIONS(2101),
    [anon_sym_while] = ACTIONS(2101),
    [anon_sym_ref] = ACTIONS(2101),
    [sym_mutable_specifier] = ACTIONS(2101),
    [anon_sym_yield] = ACTIONS(2101),
    [anon_sym_move] = ACTIONS(2101),
    [anon_sym_try] = ACTIONS(2101),
    [sym_integer_literal] = ACTIONS(2099),
    [aux_sym_string_literal_token1] = ACTIONS(2099),
    [sym_char_literal] = ACTIONS(2099),
    [anon_sym_true] = ACTIONS(2101),
    [anon_sym_false] = ACTIONS(2101),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(2101),
    [sym_super] = ACTIONS(2101),
    [sym_crate] = ACTIONS(2101),
    [sym_metavariable] = ACTIONS(2099),
    [sym__raw_string_literal_start] = ACTIONS(2099),
    [sym_float_literal] = ACTIONS(2099),
  },
  [900] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2951),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(900),
    [sym_block_comment] = STATE(900),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [901] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1998),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(901),
    [sym_block_comment] = STATE(901),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [902] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2661),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(902),
    [sym_block_comment] = STATE(902),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [903] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2408),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(903),
    [sym_block_comment] = STATE(903),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [904] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2618),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(904),
    [sym_block_comment] = STATE(904),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [905] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2012),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(905),
    [sym_block_comment] = STATE(905),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [906] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2450),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(906),
    [sym_block_comment] = STATE(906),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [907] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(3035),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(907),
    [sym_block_comment] = STATE(907),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [908] = {
    [sym_function_modifiers] = STATE(3367),
    [sym_removed_trait_bound] = STATE(1416),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1460),
    [sym_bracketed_type] = STATE(3544),
    [sym_lifetime] = STATE(3348),
    [sym_array_type] = STATE(1416),
    [sym_for_lifetimes] = STATE(1584),
    [sym_function_type] = STATE(1416),
    [sym_tuple_type] = STATE(1416),
    [sym_unit_type] = STATE(1416),
    [sym_generic_type] = STATE(1083),
    [sym_generic_type_with_turbofish] = STATE(3535),
    [sym_bounded_type] = STATE(1416),
    [sym_reference_type] = STATE(1416),
    [sym_pointer_type] = STATE(1416),
    [sym_never_type] = STATE(1416),
    [sym_abstract_type] = STATE(1416),
    [sym_dynamic_type] = STATE(1416),
    [sym_macro_invocation] = STATE(1416),
    [sym_scoped_identifier] = STATE(3224),
    [sym_scoped_type_identifier] = STATE(1034),
    [sym_line_comment] = STATE(908),
    [sym_block_comment] = STATE(908),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3097),
    [anon_sym_LPAREN] = ACTIONS(3099),
    [anon_sym_LBRACK] = ACTIONS(3101),
    [anon_sym_STAR] = ACTIONS(3105),
    [anon_sym_QMARK] = ACTIONS(3107),
    [anon_sym_u8] = ACTIONS(3109),
    [anon_sym_i8] = ACTIONS(3109),
    [anon_sym_u16] = ACTIONS(3109),
    [anon_sym_i16] = ACTIONS(3109),
    [anon_sym_u32] = ACTIONS(3109),
    [anon_sym_i32] = ACTIONS(3109),
    [anon_sym_u64] = ACTIONS(3109),
    [anon_sym_i64] = ACTIONS(3109),
    [anon_sym_u128] = ACTIONS(3109),
    [anon_sym_i128] = ACTIONS(3109),
    [anon_sym_isize] = ACTIONS(3109),
    [anon_sym_usize] = ACTIONS(3109),
    [anon_sym_f32] = ACTIONS(3109),
    [anon_sym_f64] = ACTIONS(3109),
    [anon_sym_bool] = ACTIONS(3109),
    [anon_sym_str] = ACTIONS(3109),
    [anon_sym_char] = ACTIONS(3109),
    [anon_sym_BANG] = ACTIONS(3111),
    [anon_sym_AMP] = ACTIONS(3113),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(3115),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(3117),
    [anon_sym_fn] = ACTIONS(3119),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(3121),
    [anon_sym_impl] = ACTIONS(3123),
    [anon_sym_union] = ACTIONS(3121),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(3125),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3129),
    [sym_super] = ACTIONS(3129),
    [sym_crate] = ACTIONS(3129),
    [sym_metavariable] = ACTIONS(3131),
  },
  [909] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(3002),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(909),
    [sym_block_comment] = STATE(909),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [910] = {
    [sym_function_modifiers] = STATE(3367),
    [sym_removed_trait_bound] = STATE(1416),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1461),
    [sym_bracketed_type] = STATE(3544),
    [sym_lifetime] = STATE(3348),
    [sym_array_type] = STATE(1416),
    [sym_for_lifetimes] = STATE(1584),
    [sym_function_type] = STATE(1416),
    [sym_tuple_type] = STATE(1416),
    [sym_unit_type] = STATE(1416),
    [sym_generic_type] = STATE(1083),
    [sym_generic_type_with_turbofish] = STATE(3535),
    [sym_bounded_type] = STATE(1416),
    [sym_reference_type] = STATE(1416),
    [sym_pointer_type] = STATE(1416),
    [sym_never_type] = STATE(1416),
    [sym_abstract_type] = STATE(1416),
    [sym_dynamic_type] = STATE(1416),
    [sym_macro_invocation] = STATE(1416),
    [sym_scoped_identifier] = STATE(3224),
    [sym_scoped_type_identifier] = STATE(1034),
    [sym_line_comment] = STATE(910),
    [sym_block_comment] = STATE(910),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3097),
    [anon_sym_LPAREN] = ACTIONS(3099),
    [anon_sym_LBRACK] = ACTIONS(3101),
    [anon_sym_STAR] = ACTIONS(3105),
    [anon_sym_QMARK] = ACTIONS(3107),
    [anon_sym_u8] = ACTIONS(3109),
    [anon_sym_i8] = ACTIONS(3109),
    [anon_sym_u16] = ACTIONS(3109),
    [anon_sym_i16] = ACTIONS(3109),
    [anon_sym_u32] = ACTIONS(3109),
    [anon_sym_i32] = ACTIONS(3109),
    [anon_sym_u64] = ACTIONS(3109),
    [anon_sym_i64] = ACTIONS(3109),
    [anon_sym_u128] = ACTIONS(3109),
    [anon_sym_i128] = ACTIONS(3109),
    [anon_sym_isize] = ACTIONS(3109),
    [anon_sym_usize] = ACTIONS(3109),
    [anon_sym_f32] = ACTIONS(3109),
    [anon_sym_f64] = ACTIONS(3109),
    [anon_sym_bool] = ACTIONS(3109),
    [anon_sym_str] = ACTIONS(3109),
    [anon_sym_char] = ACTIONS(3109),
    [anon_sym_BANG] = ACTIONS(3111),
    [anon_sym_AMP] = ACTIONS(3113),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(3115),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(3117),
    [anon_sym_fn] = ACTIONS(3119),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(3121),
    [anon_sym_impl] = ACTIONS(3123),
    [anon_sym_union] = ACTIONS(3121),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(3125),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3129),
    [sym_super] = ACTIONS(3129),
    [sym_crate] = ACTIONS(3129),
    [sym_metavariable] = ACTIONS(3131),
  },
  [911] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2851),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(911),
    [sym_block_comment] = STATE(911),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [912] = {
    [sym_function_modifiers] = STATE(3367),
    [sym_removed_trait_bound] = STATE(1416),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1466),
    [sym_bracketed_type] = STATE(3544),
    [sym_lifetime] = STATE(3348),
    [sym_array_type] = STATE(1416),
    [sym_for_lifetimes] = STATE(1584),
    [sym_function_type] = STATE(1416),
    [sym_tuple_type] = STATE(1416),
    [sym_unit_type] = STATE(1416),
    [sym_generic_type] = STATE(1083),
    [sym_generic_type_with_turbofish] = STATE(3535),
    [sym_bounded_type] = STATE(1416),
    [sym_reference_type] = STATE(1416),
    [sym_pointer_type] = STATE(1416),
    [sym_never_type] = STATE(1416),
    [sym_abstract_type] = STATE(1416),
    [sym_dynamic_type] = STATE(1416),
    [sym_macro_invocation] = STATE(1416),
    [sym_scoped_identifier] = STATE(3224),
    [sym_scoped_type_identifier] = STATE(1034),
    [sym_line_comment] = STATE(912),
    [sym_block_comment] = STATE(912),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3097),
    [anon_sym_LPAREN] = ACTIONS(3099),
    [anon_sym_LBRACK] = ACTIONS(3101),
    [anon_sym_STAR] = ACTIONS(3105),
    [anon_sym_QMARK] = ACTIONS(3107),
    [anon_sym_u8] = ACTIONS(3109),
    [anon_sym_i8] = ACTIONS(3109),
    [anon_sym_u16] = ACTIONS(3109),
    [anon_sym_i16] = ACTIONS(3109),
    [anon_sym_u32] = ACTIONS(3109),
    [anon_sym_i32] = ACTIONS(3109),
    [anon_sym_u64] = ACTIONS(3109),
    [anon_sym_i64] = ACTIONS(3109),
    [anon_sym_u128] = ACTIONS(3109),
    [anon_sym_i128] = ACTIONS(3109),
    [anon_sym_isize] = ACTIONS(3109),
    [anon_sym_usize] = ACTIONS(3109),
    [anon_sym_f32] = ACTIONS(3109),
    [anon_sym_f64] = ACTIONS(3109),
    [anon_sym_bool] = ACTIONS(3109),
    [anon_sym_str] = ACTIONS(3109),
    [anon_sym_char] = ACTIONS(3109),
    [anon_sym_BANG] = ACTIONS(3111),
    [anon_sym_AMP] = ACTIONS(3113),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(3115),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(3117),
    [anon_sym_fn] = ACTIONS(3119),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(3121),
    [anon_sym_impl] = ACTIONS(3123),
    [anon_sym_union] = ACTIONS(3121),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(3125),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3129),
    [sym_super] = ACTIONS(3129),
    [sym_crate] = ACTIONS(3129),
    [sym_metavariable] = ACTIONS(3131),
  },
  [913] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2589),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(913),
    [sym_block_comment] = STATE(913),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [914] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2002),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(914),
    [sym_block_comment] = STATE(914),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [915] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2201),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(915),
    [sym_block_comment] = STATE(915),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [916] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2916),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(916),
    [sym_block_comment] = STATE(916),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [917] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2904),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(917),
    [sym_block_comment] = STATE(917),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [918] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2586),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(918),
    [sym_block_comment] = STATE(918),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [919] = {
    [sym_function_modifiers] = STATE(3367),
    [sym_removed_trait_bound] = STATE(1416),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1392),
    [sym_bracketed_type] = STATE(3544),
    [sym_lifetime] = STATE(3348),
    [sym_array_type] = STATE(1416),
    [sym_for_lifetimes] = STATE(1584),
    [sym_function_type] = STATE(1416),
    [sym_tuple_type] = STATE(1416),
    [sym_unit_type] = STATE(1416),
    [sym_generic_type] = STATE(1083),
    [sym_generic_type_with_turbofish] = STATE(3535),
    [sym_bounded_type] = STATE(1416),
    [sym_reference_type] = STATE(1416),
    [sym_pointer_type] = STATE(1416),
    [sym_never_type] = STATE(1416),
    [sym_abstract_type] = STATE(1416),
    [sym_dynamic_type] = STATE(1416),
    [sym_macro_invocation] = STATE(1416),
    [sym_scoped_identifier] = STATE(3224),
    [sym_scoped_type_identifier] = STATE(1034),
    [sym_line_comment] = STATE(919),
    [sym_block_comment] = STATE(919),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3097),
    [anon_sym_LPAREN] = ACTIONS(3099),
    [anon_sym_LBRACK] = ACTIONS(3101),
    [anon_sym_STAR] = ACTIONS(3105),
    [anon_sym_QMARK] = ACTIONS(3107),
    [anon_sym_u8] = ACTIONS(3109),
    [anon_sym_i8] = ACTIONS(3109),
    [anon_sym_u16] = ACTIONS(3109),
    [anon_sym_i16] = ACTIONS(3109),
    [anon_sym_u32] = ACTIONS(3109),
    [anon_sym_i32] = ACTIONS(3109),
    [anon_sym_u64] = ACTIONS(3109),
    [anon_sym_i64] = ACTIONS(3109),
    [anon_sym_u128] = ACTIONS(3109),
    [anon_sym_i128] = ACTIONS(3109),
    [anon_sym_isize] = ACTIONS(3109),
    [anon_sym_usize] = ACTIONS(3109),
    [anon_sym_f32] = ACTIONS(3109),
    [anon_sym_f64] = ACTIONS(3109),
    [anon_sym_bool] = ACTIONS(3109),
    [anon_sym_str] = ACTIONS(3109),
    [anon_sym_char] = ACTIONS(3109),
    [anon_sym_BANG] = ACTIONS(3111),
    [anon_sym_AMP] = ACTIONS(3113),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(3115),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(3117),
    [anon_sym_fn] = ACTIONS(3119),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(3121),
    [anon_sym_impl] = ACTIONS(3123),
    [anon_sym_union] = ACTIONS(3121),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(3125),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3129),
    [sym_super] = ACTIONS(3129),
    [sym_crate] = ACTIONS(3129),
    [sym_metavariable] = ACTIONS(3131),
  },
  [920] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2963),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(920),
    [sym_block_comment] = STATE(920),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [921] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2313),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(921),
    [sym_block_comment] = STATE(921),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [922] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2318),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(922),
    [sym_block_comment] = STATE(922),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [923] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2326),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(923),
    [sym_block_comment] = STATE(923),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [924] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1997),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(924),
    [sym_block_comment] = STATE(924),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [925] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2327),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(925),
    [sym_block_comment] = STATE(925),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [926] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2250),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(926),
    [sym_block_comment] = STATE(926),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [927] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1992),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(927),
    [sym_block_comment] = STATE(927),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [928] = {
    [sym_function_modifiers] = STATE(3367),
    [sym_removed_trait_bound] = STATE(1416),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1393),
    [sym_bracketed_type] = STATE(3544),
    [sym_lifetime] = STATE(3348),
    [sym_array_type] = STATE(1416),
    [sym_for_lifetimes] = STATE(1584),
    [sym_function_type] = STATE(1416),
    [sym_tuple_type] = STATE(1416),
    [sym_unit_type] = STATE(1416),
    [sym_generic_type] = STATE(1083),
    [sym_generic_type_with_turbofish] = STATE(3535),
    [sym_bounded_type] = STATE(1416),
    [sym_reference_type] = STATE(1416),
    [sym_pointer_type] = STATE(1416),
    [sym_never_type] = STATE(1416),
    [sym_abstract_type] = STATE(1416),
    [sym_dynamic_type] = STATE(1416),
    [sym_macro_invocation] = STATE(1416),
    [sym_scoped_identifier] = STATE(3224),
    [sym_scoped_type_identifier] = STATE(1034),
    [sym_line_comment] = STATE(928),
    [sym_block_comment] = STATE(928),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3097),
    [anon_sym_LPAREN] = ACTIONS(3099),
    [anon_sym_LBRACK] = ACTIONS(3101),
    [anon_sym_STAR] = ACTIONS(3105),
    [anon_sym_QMARK] = ACTIONS(3107),
    [anon_sym_u8] = ACTIONS(3109),
    [anon_sym_i8] = ACTIONS(3109),
    [anon_sym_u16] = ACTIONS(3109),
    [anon_sym_i16] = ACTIONS(3109),
    [anon_sym_u32] = ACTIONS(3109),
    [anon_sym_i32] = ACTIONS(3109),
    [anon_sym_u64] = ACTIONS(3109),
    [anon_sym_i64] = ACTIONS(3109),
    [anon_sym_u128] = ACTIONS(3109),
    [anon_sym_i128] = ACTIONS(3109),
    [anon_sym_isize] = ACTIONS(3109),
    [anon_sym_usize] = ACTIONS(3109),
    [anon_sym_f32] = ACTIONS(3109),
    [anon_sym_f64] = ACTIONS(3109),
    [anon_sym_bool] = ACTIONS(3109),
    [anon_sym_str] = ACTIONS(3109),
    [anon_sym_char] = ACTIONS(3109),
    [anon_sym_BANG] = ACTIONS(3111),
    [anon_sym_AMP] = ACTIONS(3113),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(3115),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(3117),
    [anon_sym_fn] = ACTIONS(3119),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(3121),
    [anon_sym_impl] = ACTIONS(3123),
    [anon_sym_union] = ACTIONS(3121),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(3125),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3129),
    [sym_super] = ACTIONS(3129),
    [sym_crate] = ACTIONS(3129),
    [sym_metavariable] = ACTIONS(3131),
  },
  [929] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2517),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(929),
    [sym_block_comment] = STATE(929),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [930] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2237),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(930),
    [sym_block_comment] = STATE(930),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [931] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(3260),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(931),
    [sym_block_comment] = STATE(931),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [932] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2728),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(932),
    [sym_block_comment] = STATE(932),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [933] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2014),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(1999),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(933),
    [sym_block_comment] = STATE(933),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(2999),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [934] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2407),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(934),
    [sym_block_comment] = STATE(934),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [935] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2356),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(935),
    [sym_block_comment] = STATE(935),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [936] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(3271),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(936),
    [sym_block_comment] = STATE(936),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [937] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2783),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(937),
    [sym_block_comment] = STATE(937),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [938] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(3026),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(938),
    [sym_block_comment] = STATE(938),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [939] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2346),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(2400),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(2193),
    [sym_line_comment] = STATE(939),
    [sym_block_comment] = STATE(939),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3229),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(3231),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [940] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2909),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(940),
    [sym_block_comment] = STATE(940),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [941] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2415),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(941),
    [sym_block_comment] = STATE(941),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [942] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2373),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(942),
    [sym_block_comment] = STATE(942),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [943] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2014),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(943),
    [sym_block_comment] = STATE(943),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [944] = {
    [sym_function_modifiers] = STATE(3341),
    [sym_removed_trait_bound] = STATE(1684),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1771),
    [sym_bracketed_type] = STATE(3559),
    [sym_lifetime] = STATE(3394),
    [sym_array_type] = STATE(1684),
    [sym_for_lifetimes] = STATE(1591),
    [sym_function_type] = STATE(1684),
    [sym_tuple_type] = STATE(1684),
    [sym_unit_type] = STATE(1684),
    [sym_generic_type] = STATE(1649),
    [sym_generic_type_with_turbofish] = STATE(3551),
    [sym_bounded_type] = STATE(1684),
    [sym_reference_type] = STATE(1684),
    [sym_pointer_type] = STATE(1684),
    [sym_never_type] = STATE(1684),
    [sym_abstract_type] = STATE(1684),
    [sym_dynamic_type] = STATE(1684),
    [sym_macro_invocation] = STATE(1684),
    [sym_scoped_identifier] = STATE(3288),
    [sym_scoped_type_identifier] = STATE(1548),
    [sym_line_comment] = STATE(944),
    [sym_block_comment] = STATE(944),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3139),
    [anon_sym_LPAREN] = ACTIONS(3141),
    [anon_sym_LBRACK] = ACTIONS(3143),
    [anon_sym_STAR] = ACTIONS(3147),
    [anon_sym_QMARK] = ACTIONS(3149),
    [anon_sym_u8] = ACTIONS(3151),
    [anon_sym_i8] = ACTIONS(3151),
    [anon_sym_u16] = ACTIONS(3151),
    [anon_sym_i16] = ACTIONS(3151),
    [anon_sym_u32] = ACTIONS(3151),
    [anon_sym_i32] = ACTIONS(3151),
    [anon_sym_u64] = ACTIONS(3151),
    [anon_sym_i64] = ACTIONS(3151),
    [anon_sym_u128] = ACTIONS(3151),
    [anon_sym_i128] = ACTIONS(3151),
    [anon_sym_isize] = ACTIONS(3151),
    [anon_sym_usize] = ACTIONS(3151),
    [anon_sym_f32] = ACTIONS(3151),
    [anon_sym_f64] = ACTIONS(3151),
    [anon_sym_bool] = ACTIONS(3151),
    [anon_sym_str] = ACTIONS(3151),
    [anon_sym_char] = ACTIONS(3151),
    [anon_sym_BANG] = ACTIONS(3153),
    [anon_sym_AMP] = ACTIONS(3155),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(3157),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(3159),
    [anon_sym_fn] = ACTIONS(3161),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(3163),
    [anon_sym_impl] = ACTIONS(3165),
    [anon_sym_union] = ACTIONS(3163),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(3167),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3171),
    [sym_super] = ACTIONS(3171),
    [sym_crate] = ACTIONS(3171),
    [sym_metavariable] = ACTIONS(3173),
  },
  [945] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2784),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(945),
    [sym_block_comment] = STATE(945),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [946] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2380),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(946),
    [sym_block_comment] = STATE(946),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [947] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2383),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(947),
    [sym_block_comment] = STATE(947),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [948] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2541),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(948),
    [sym_block_comment] = STATE(948),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [949] = {
    [sym_function_modifiers] = STATE(3341),
    [sym_removed_trait_bound] = STATE(1684),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1772),
    [sym_bracketed_type] = STATE(3559),
    [sym_lifetime] = STATE(3394),
    [sym_array_type] = STATE(1684),
    [sym_for_lifetimes] = STATE(1591),
    [sym_function_type] = STATE(1684),
    [sym_tuple_type] = STATE(1684),
    [sym_unit_type] = STATE(1684),
    [sym_generic_type] = STATE(1649),
    [sym_generic_type_with_turbofish] = STATE(3551),
    [sym_bounded_type] = STATE(1684),
    [sym_reference_type] = STATE(1684),
    [sym_pointer_type] = STATE(1684),
    [sym_never_type] = STATE(1684),
    [sym_abstract_type] = STATE(1684),
    [sym_dynamic_type] = STATE(1684),
    [sym_macro_invocation] = STATE(1684),
    [sym_scoped_identifier] = STATE(3288),
    [sym_scoped_type_identifier] = STATE(1548),
    [sym_line_comment] = STATE(949),
    [sym_block_comment] = STATE(949),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3139),
    [anon_sym_LPAREN] = ACTIONS(3141),
    [anon_sym_LBRACK] = ACTIONS(3143),
    [anon_sym_STAR] = ACTIONS(3147),
    [anon_sym_QMARK] = ACTIONS(3149),
    [anon_sym_u8] = ACTIONS(3151),
    [anon_sym_i8] = ACTIONS(3151),
    [anon_sym_u16] = ACTIONS(3151),
    [anon_sym_i16] = ACTIONS(3151),
    [anon_sym_u32] = ACTIONS(3151),
    [anon_sym_i32] = ACTIONS(3151),
    [anon_sym_u64] = ACTIONS(3151),
    [anon_sym_i64] = ACTIONS(3151),
    [anon_sym_u128] = ACTIONS(3151),
    [anon_sym_i128] = ACTIONS(3151),
    [anon_sym_isize] = ACTIONS(3151),
    [anon_sym_usize] = ACTIONS(3151),
    [anon_sym_f32] = ACTIONS(3151),
    [anon_sym_f64] = ACTIONS(3151),
    [anon_sym_bool] = ACTIONS(3151),
    [anon_sym_str] = ACTIONS(3151),
    [anon_sym_char] = ACTIONS(3151),
    [anon_sym_BANG] = ACTIONS(3153),
    [anon_sym_AMP] = ACTIONS(3155),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(3157),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(3159),
    [anon_sym_fn] = ACTIONS(3161),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(3163),
    [anon_sym_impl] = ACTIONS(3165),
    [anon_sym_union] = ACTIONS(3163),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(3167),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3171),
    [sym_super] = ACTIONS(3171),
    [sym_crate] = ACTIONS(3171),
    [sym_metavariable] = ACTIONS(3173),
  },
  [950] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2801),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(950),
    [sym_block_comment] = STATE(950),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [951] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2427),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(951),
    [sym_block_comment] = STATE(951),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [952] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2550),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(952),
    [sym_block_comment] = STATE(952),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [953] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2385),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(2404),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(2168),
    [sym_line_comment] = STATE(953),
    [sym_block_comment] = STATE(953),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3233),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(3235),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [954] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2900),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(954),
    [sym_block_comment] = STATE(954),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [955] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2363),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(2405),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(2155),
    [sym_line_comment] = STATE(955),
    [sym_block_comment] = STATE(955),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3237),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(3239),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [956] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2218),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(956),
    [sym_block_comment] = STATE(956),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [957] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2310),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(957),
    [sym_block_comment] = STATE(957),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [958] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2399),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(958),
    [sym_block_comment] = STATE(958),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [959] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2401),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(959),
    [sym_block_comment] = STATE(959),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [960] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2403),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(960),
    [sym_block_comment] = STATE(960),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [961] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2678),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(961),
    [sym_block_comment] = STATE(961),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [962] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2847),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(962),
    [sym_block_comment] = STATE(962),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [963] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2437),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(963),
    [sym_block_comment] = STATE(963),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [964] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2686),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(964),
    [sym_block_comment] = STATE(964),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [965] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2853),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(965),
    [sym_block_comment] = STATE(965),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [966] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2701),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(966),
    [sym_block_comment] = STATE(966),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [967] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2225),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(967),
    [sym_block_comment] = STATE(967),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [968] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2319),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(968),
    [sym_block_comment] = STATE(968),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [969] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2320),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(969),
    [sym_block_comment] = STATE(969),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [970] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2934),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(970),
    [sym_block_comment] = STATE(970),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [971] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2880),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(971),
    [sym_block_comment] = STATE(971),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [972] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2478),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(972),
    [sym_block_comment] = STATE(972),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [973] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2322),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(973),
    [sym_block_comment] = STATE(973),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [974] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2323),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(974),
    [sym_block_comment] = STATE(974),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [975] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2324),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(975),
    [sym_block_comment] = STATE(975),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [976] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2325),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(976),
    [sym_block_comment] = STATE(976),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [977] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2226),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(977),
    [sym_block_comment] = STATE(977),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [978] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2522),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(978),
    [sym_block_comment] = STATE(978),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [979] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2240),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(979),
    [sym_block_comment] = STATE(979),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [980] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2706),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(980),
    [sym_block_comment] = STATE(980),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [981] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2331),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(981),
    [sym_block_comment] = STATE(981),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [982] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2332),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(982),
    [sym_block_comment] = STATE(982),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [983] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2229),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(983),
    [sym_block_comment] = STATE(983),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [984] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2243),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(984),
    [sym_block_comment] = STATE(984),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [985] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2896),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(985),
    [sym_block_comment] = STATE(985),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [986] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2230),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(986),
    [sym_block_comment] = STATE(986),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [987] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2750),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(987),
    [sym_block_comment] = STATE(987),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [988] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2231),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(988),
    [sym_block_comment] = STATE(988),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [989] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2578),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(989),
    [sym_block_comment] = STATE(989),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [990] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2350),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(990),
    [sym_block_comment] = STATE(990),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [991] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1985),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(991),
    [sym_block_comment] = STATE(991),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [992] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2912),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(992),
    [sym_block_comment] = STATE(992),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [993] = {
    [sym_function_modifiers] = STATE(3341),
    [sym_removed_trait_bound] = STATE(1684),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1723),
    [sym_bracketed_type] = STATE(3559),
    [sym_lifetime] = STATE(3394),
    [sym_array_type] = STATE(1684),
    [sym_for_lifetimes] = STATE(1591),
    [sym_function_type] = STATE(1684),
    [sym_tuple_type] = STATE(1684),
    [sym_unit_type] = STATE(1684),
    [sym_generic_type] = STATE(1649),
    [sym_generic_type_with_turbofish] = STATE(3551),
    [sym_bounded_type] = STATE(1684),
    [sym_reference_type] = STATE(1684),
    [sym_pointer_type] = STATE(1684),
    [sym_never_type] = STATE(1684),
    [sym_abstract_type] = STATE(1684),
    [sym_dynamic_type] = STATE(1684),
    [sym_macro_invocation] = STATE(1684),
    [sym_scoped_identifier] = STATE(3288),
    [sym_scoped_type_identifier] = STATE(1548),
    [sym_line_comment] = STATE(993),
    [sym_block_comment] = STATE(993),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3139),
    [anon_sym_LPAREN] = ACTIONS(3141),
    [anon_sym_LBRACK] = ACTIONS(3143),
    [anon_sym_STAR] = ACTIONS(3147),
    [anon_sym_QMARK] = ACTIONS(3149),
    [anon_sym_u8] = ACTIONS(3151),
    [anon_sym_i8] = ACTIONS(3151),
    [anon_sym_u16] = ACTIONS(3151),
    [anon_sym_i16] = ACTIONS(3151),
    [anon_sym_u32] = ACTIONS(3151),
    [anon_sym_i32] = ACTIONS(3151),
    [anon_sym_u64] = ACTIONS(3151),
    [anon_sym_i64] = ACTIONS(3151),
    [anon_sym_u128] = ACTIONS(3151),
    [anon_sym_i128] = ACTIONS(3151),
    [anon_sym_isize] = ACTIONS(3151),
    [anon_sym_usize] = ACTIONS(3151),
    [anon_sym_f32] = ACTIONS(3151),
    [anon_sym_f64] = ACTIONS(3151),
    [anon_sym_bool] = ACTIONS(3151),
    [anon_sym_str] = ACTIONS(3151),
    [anon_sym_char] = ACTIONS(3151),
    [anon_sym_BANG] = ACTIONS(3153),
    [anon_sym_AMP] = ACTIONS(3155),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(3157),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(3159),
    [anon_sym_fn] = ACTIONS(3161),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(3163),
    [anon_sym_impl] = ACTIONS(3165),
    [anon_sym_union] = ACTIONS(3163),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(3167),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3171),
    [sym_super] = ACTIONS(3171),
    [sym_crate] = ACTIONS(3171),
    [sym_metavariable] = ACTIONS(3173),
  },
  [994] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2351),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(994),
    [sym_block_comment] = STATE(994),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [995] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(3000),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(995),
    [sym_block_comment] = STATE(995),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [996] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2396),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(996),
    [sym_block_comment] = STATE(996),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [997] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2003),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(997),
    [sym_block_comment] = STATE(997),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [998] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2861),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(998),
    [sym_block_comment] = STATE(998),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [999] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2002),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(999),
    [sym_block_comment] = STATE(999),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3241),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [1000] = {
    [sym_function_modifiers] = STATE(3367),
    [sym_removed_trait_bound] = STATE(1416),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1436),
    [sym_bracketed_type] = STATE(3544),
    [sym_lifetime] = STATE(3348),
    [sym_array_type] = STATE(1416),
    [sym_for_lifetimes] = STATE(1584),
    [sym_function_type] = STATE(1416),
    [sym_tuple_type] = STATE(1416),
    [sym_unit_type] = STATE(1416),
    [sym_generic_type] = STATE(1083),
    [sym_generic_type_with_turbofish] = STATE(3535),
    [sym_bounded_type] = STATE(1416),
    [sym_reference_type] = STATE(1416),
    [sym_pointer_type] = STATE(1416),
    [sym_never_type] = STATE(1416),
    [sym_abstract_type] = STATE(1416),
    [sym_dynamic_type] = STATE(1416),
    [sym_macro_invocation] = STATE(1416),
    [sym_scoped_identifier] = STATE(3224),
    [sym_scoped_type_identifier] = STATE(1034),
    [sym_line_comment] = STATE(1000),
    [sym_block_comment] = STATE(1000),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3097),
    [anon_sym_LPAREN] = ACTIONS(3099),
    [anon_sym_LBRACK] = ACTIONS(3101),
    [anon_sym_STAR] = ACTIONS(3105),
    [anon_sym_QMARK] = ACTIONS(3107),
    [anon_sym_u8] = ACTIONS(3109),
    [anon_sym_i8] = ACTIONS(3109),
    [anon_sym_u16] = ACTIONS(3109),
    [anon_sym_i16] = ACTIONS(3109),
    [anon_sym_u32] = ACTIONS(3109),
    [anon_sym_i32] = ACTIONS(3109),
    [anon_sym_u64] = ACTIONS(3109),
    [anon_sym_i64] = ACTIONS(3109),
    [anon_sym_u128] = ACTIONS(3109),
    [anon_sym_i128] = ACTIONS(3109),
    [anon_sym_isize] = ACTIONS(3109),
    [anon_sym_usize] = ACTIONS(3109),
    [anon_sym_f32] = ACTIONS(3109),
    [anon_sym_f64] = ACTIONS(3109),
    [anon_sym_bool] = ACTIONS(3109),
    [anon_sym_str] = ACTIONS(3109),
    [anon_sym_char] = ACTIONS(3109),
    [anon_sym_BANG] = ACTIONS(3111),
    [anon_sym_AMP] = ACTIONS(3113),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(3115),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(3117),
    [anon_sym_fn] = ACTIONS(3119),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(3121),
    [anon_sym_impl] = ACTIONS(3123),
    [anon_sym_union] = ACTIONS(3121),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(3125),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3129),
    [sym_super] = ACTIONS(3129),
    [sym_crate] = ACTIONS(3129),
    [sym_metavariable] = ACTIONS(3131),
  },
  [1001] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(3325),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(1001),
    [sym_block_comment] = STATE(1001),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [1002] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2016),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(1002),
    [sym_block_comment] = STATE(1002),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [1003] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2008),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(1003),
    [sym_block_comment] = STATE(1003),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [1004] = {
    [sym_function_modifiers] = STATE(3341),
    [sym_removed_trait_bound] = STATE(1684),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(1860),
    [sym_bracketed_type] = STATE(3559),
    [sym_lifetime] = STATE(3394),
    [sym_array_type] = STATE(1684),
    [sym_for_lifetimes] = STATE(1591),
    [sym_function_type] = STATE(1684),
    [sym_tuple_type] = STATE(1684),
    [sym_unit_type] = STATE(1684),
    [sym_generic_type] = STATE(1649),
    [sym_generic_type_with_turbofish] = STATE(3551),
    [sym_bounded_type] = STATE(1684),
    [sym_reference_type] = STATE(1684),
    [sym_pointer_type] = STATE(1684),
    [sym_never_type] = STATE(1684),
    [sym_abstract_type] = STATE(1684),
    [sym_dynamic_type] = STATE(1684),
    [sym_macro_invocation] = STATE(1684),
    [sym_scoped_identifier] = STATE(3288),
    [sym_scoped_type_identifier] = STATE(1548),
    [sym_line_comment] = STATE(1004),
    [sym_block_comment] = STATE(1004),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3139),
    [anon_sym_LPAREN] = ACTIONS(3141),
    [anon_sym_LBRACK] = ACTIONS(3143),
    [anon_sym_STAR] = ACTIONS(3147),
    [anon_sym_QMARK] = ACTIONS(3149),
    [anon_sym_u8] = ACTIONS(3151),
    [anon_sym_i8] = ACTIONS(3151),
    [anon_sym_u16] = ACTIONS(3151),
    [anon_sym_i16] = ACTIONS(3151),
    [anon_sym_u32] = ACTIONS(3151),
    [anon_sym_i32] = ACTIONS(3151),
    [anon_sym_u64] = ACTIONS(3151),
    [anon_sym_i64] = ACTIONS(3151),
    [anon_sym_u128] = ACTIONS(3151),
    [anon_sym_i128] = ACTIONS(3151),
    [anon_sym_isize] = ACTIONS(3151),
    [anon_sym_usize] = ACTIONS(3151),
    [anon_sym_f32] = ACTIONS(3151),
    [anon_sym_f64] = ACTIONS(3151),
    [anon_sym_bool] = ACTIONS(3151),
    [anon_sym_str] = ACTIONS(3151),
    [anon_sym_char] = ACTIONS(3151),
    [anon_sym_BANG] = ACTIONS(3153),
    [anon_sym_AMP] = ACTIONS(3155),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(3157),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(3159),
    [anon_sym_fn] = ACTIONS(3161),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(3163),
    [anon_sym_impl] = ACTIONS(3165),
    [anon_sym_union] = ACTIONS(3163),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(3167),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3171),
    [sym_super] = ACTIONS(3171),
    [sym_crate] = ACTIONS(3171),
    [sym_metavariable] = ACTIONS(3173),
  },
  [1005] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2680),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(1005),
    [sym_block_comment] = STATE(1005),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [1006] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2532),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(1006),
    [sym_block_comment] = STATE(1006),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [1007] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2009),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(1007),
    [sym_block_comment] = STATE(1007),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [1008] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2010),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(1008),
    [sym_block_comment] = STATE(1008),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [1009] = {
    [sym_function_modifiers] = STATE(3386),
    [sym_removed_trait_bound] = STATE(1991),
    [sym_extern_modifier] = STATE(2387),
    [sym__type] = STATE(2511),
    [sym_bracketed_type] = STATE(3361),
    [sym_lifetime] = STATE(3635),
    [sym_array_type] = STATE(1991),
    [sym_for_lifetimes] = STATE(1565),
    [sym_function_type] = STATE(1991),
    [sym_tuple_type] = STATE(1991),
    [sym_unit_type] = STATE(1991),
    [sym_generic_type] = STATE(1962),
    [sym_generic_type_with_turbofish] = STATE(3641),
    [sym_bounded_type] = STATE(1991),
    [sym_reference_type] = STATE(1991),
    [sym_pointer_type] = STATE(1991),
    [sym_never_type] = STATE(1991),
    [sym_abstract_type] = STATE(1991),
    [sym_dynamic_type] = STATE(1991),
    [sym_macro_invocation] = STATE(1991),
    [sym_scoped_identifier] = STATE(3121),
    [sym_scoped_type_identifier] = STATE(1936),
    [sym_line_comment] = STATE(1009),
    [sym_block_comment] = STATE(1009),
    [aux_sym_function_modifiers_repeat1] = STATE(2239),
    [sym_identifier] = ACTIONS(3005),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_STAR] = ACTIONS(1289),
    [anon_sym_QMARK] = ACTIONS(1291),
    [anon_sym_u8] = ACTIONS(1603),
    [anon_sym_i8] = ACTIONS(1603),
    [anon_sym_u16] = ACTIONS(1603),
    [anon_sym_i16] = ACTIONS(1603),
    [anon_sym_u32] = ACTIONS(1603),
    [anon_sym_i32] = ACTIONS(1603),
    [anon_sym_u64] = ACTIONS(1603),
    [anon_sym_i64] = ACTIONS(1603),
    [anon_sym_u128] = ACTIONS(1603),
    [anon_sym_i128] = ACTIONS(1603),
    [anon_sym_isize] = ACTIONS(1603),
    [anon_sym_usize] = ACTIONS(1603),
    [anon_sym_f32] = ACTIONS(1603),
    [anon_sym_f64] = ACTIONS(1603),
    [anon_sym_bool] = ACTIONS(1603),
    [anon_sym_str] = ACTIONS(1603),
    [anon_sym_char] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1297),
    [anon_sym_AMP] = ACTIONS(1605),
    [anon_sym_LT] = ACTIONS(29),
    [anon_sym_COLON_COLON] = ACTIONS(1609),
    [anon_sym_SQUOTE] = ACTIONS(3011),
    [anon_sym_async] = ACTIONS(1317),
    [anon_sym_const] = ACTIONS(1317),
    [anon_sym_default] = ACTIONS(1613),
    [anon_sym_fn] = ACTIONS(1323),
    [anon_sym_for] = ACTIONS(1325),
    [anon_sym_gen] = ACTIONS(1615),
    [anon_sym_impl] = ACTIONS(1329),
    [anon_sym_union] = ACTIONS(1615),
    [anon_sym_unsafe] = ACTIONS(1317),
    [anon_sym_extern] = ACTIONS(1331),
    [anon_sym_dyn] = ACTIONS(1335),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1619),
    [sym_super] = ACTIONS(1619),
    [sym_crate] = ACTIONS(1619),
    [sym_metavariable] = ACTIONS(1621),
  },
  [1010] = {
    [sym_attribute_item] = STATE(1011),
    [sym_line_comment] = STATE(1010),
    [sym_block_comment] = STATE(1010),
    [aux_sym_enum_variant_list_repeat1] = STATE(1010),
    [sym_identifier] = ACTIONS(3243),
    [anon_sym_LPAREN] = ACTIONS(789),
    [anon_sym_LBRACK] = ACTIONS(789),
    [anon_sym_RBRACK] = ACTIONS(789),
    [anon_sym_LBRACE] = ACTIONS(789),
    [anon_sym_STAR] = ACTIONS(789),
    [anon_sym_u8] = ACTIONS(3243),
    [anon_sym_i8] = ACTIONS(3243),
    [anon_sym_u16] = ACTIONS(3243),
    [anon_sym_i16] = ACTIONS(3243),
    [anon_sym_u32] = ACTIONS(3243),
    [anon_sym_i32] = ACTIONS(3243),
    [anon_sym_u64] = ACTIONS(3243),
    [anon_sym_i64] = ACTIONS(3243),
    [anon_sym_u128] = ACTIONS(3243),
    [anon_sym_i128] = ACTIONS(3243),
    [anon_sym_isize] = ACTIONS(3243),
    [anon_sym_usize] = ACTIONS(3243),
    [anon_sym_f32] = ACTIONS(3243),
    [anon_sym_f64] = ACTIONS(3243),
    [anon_sym_bool] = ACTIONS(3243),
    [anon_sym_str] = ACTIONS(3243),
    [anon_sym_char] = ACTIONS(3243),
    [anon_sym_DASH] = ACTIONS(789),
    [anon_sym_BANG] = ACTIONS(789),
    [anon_sym_AMP] = ACTIONS(789),
    [anon_sym_PIPE] = ACTIONS(789),
    [anon_sym_LT] = ACTIONS(789),
    [anon_sym_DOT_DOT] = ACTIONS(789),
    [anon_sym_COMMA] = ACTIONS(789),
    [anon_sym_COLON_COLON] = ACTIONS(789),
    [anon_sym_POUND] = ACTIONS(815),
    [anon_sym_SQUOTE] = ACTIONS(3243),
    [anon_sym_async] = ACTIONS(3243),
    [anon_sym_break] = ACTIONS(3243),
    [anon_sym_const] = ACTIONS(3243),
    [anon_sym_continue] = ACTIONS(3243),
    [anon_sym_default] = ACTIONS(3243),
    [anon_sym_for] = ACTIONS(3243),
    [anon_sym_gen] = ACTIONS(3243),
    [anon_sym_if] = ACTIONS(3243),
    [anon_sym_loop] = ACTIONS(3243),
    [anon_sym_match] = ACTIONS(3243),
    [anon_sym_return] = ACTIONS(3243),
    [anon_sym_static] = ACTIONS(3243),
    [anon_sym_union] = ACTIONS(3243),
    [anon_sym_unsafe] = ACTIONS(3243),
    [anon_sym_while] = ACTIONS(3243),
    [anon_sym_yield] = ACTIONS(3243),
    [anon_sym_move] = ACTIONS(3243),
    [anon_sym_try] = ACTIONS(3243),
    [sym_integer_literal] = ACTIONS(789),
    [aux_sym_string_literal_token1] = ACTIONS(789),
    [sym_char_literal] = ACTIONS(789),
    [anon_sym_true] = ACTIONS(3243),
    [anon_sym_false] = ACTIONS(3243),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3243),
    [sym_super] = ACTIONS(3243),
    [sym_crate] = ACTIONS(3243),
    [sym_metavariable] = ACTIONS(789),
    [sym__raw_string_literal_start] = ACTIONS(789),
    [sym_float_literal] = ACTIONS(789),
  },
  [1011] = {
    [sym_line_comment] = STATE(1011),
    [sym_block_comment] = STATE(1011),
    [sym_identifier] = ACTIONS(3245),
    [anon_sym_LPAREN] = ACTIONS(3247),
    [anon_sym_LBRACK] = ACTIONS(3247),
    [anon_sym_RBRACK] = ACTIONS(3247),
    [anon_sym_LBRACE] = ACTIONS(3247),
    [anon_sym_STAR] = ACTIONS(3247),
    [anon_sym_u8] = ACTIONS(3245),
    [anon_sym_i8] = ACTIONS(3245),
    [anon_sym_u16] = ACTIONS(3245),
    [anon_sym_i16] = ACTIONS(3245),
    [anon_sym_u32] = ACTIONS(3245),
    [anon_sym_i32] = ACTIONS(3245),
    [anon_sym_u64] = ACTIONS(3245),
    [anon_sym_i64] = ACTIONS(3245),
    [anon_sym_u128] = ACTIONS(3245),
    [anon_sym_i128] = ACTIONS(3245),
    [anon_sym_isize] = ACTIONS(3245),
    [anon_sym_usize] = ACTIONS(3245),
    [anon_sym_f32] = ACTIONS(3245),
    [anon_sym_f64] = ACTIONS(3245),
    [anon_sym_bool] = ACTIONS(3245),
    [anon_sym_str] = ACTIONS(3245),
    [anon_sym_char] = ACTIONS(3245),
    [anon_sym_DASH] = ACTIONS(3247),
    [anon_sym_BANG] = ACTIONS(3247),
    [anon_sym_AMP] = ACTIONS(3247),
    [anon_sym_PIPE] = ACTIONS(3247),
    [anon_sym_LT] = ACTIONS(3247),
    [anon_sym_DOT_DOT] = ACTIONS(3247),
    [anon_sym_COMMA] = ACTIONS(3247),
    [anon_sym_COLON_COLON] = ACTIONS(3247),
    [anon_sym_POUND] = ACTIONS(3247),
    [anon_sym_SQUOTE] = ACTIONS(3245),
    [anon_sym_async] = ACTIONS(3245),
    [anon_sym_break] = ACTIONS(3245),
    [anon_sym_const] = ACTIONS(3245),
    [anon_sym_continue] = ACTIONS(3245),
    [anon_sym_default] = ACTIONS(3245),
    [anon_sym_for] = ACTIONS(3245),
    [anon_sym_gen] = ACTIONS(3245),
    [anon_sym_if] = ACTIONS(3245),
    [anon_sym_loop] = ACTIONS(3245),
    [anon_sym_match] = ACTIONS(3245),
    [anon_sym_return] = ACTIONS(3245),
    [anon_sym_static] = ACTIONS(3245),
    [anon_sym_union] = ACTIONS(3245),
    [anon_sym_unsafe] = ACTIONS(3245),
    [anon_sym_while] = ACTIONS(3245),
    [anon_sym_yield] = ACTIONS(3245),
    [anon_sym_move] = ACTIONS(3245),
    [anon_sym_try] = ACTIONS(3245),
    [sym_integer_literal] = ACTIONS(3247),
    [aux_sym_string_literal_token1] = ACTIONS(3247),
    [sym_char_literal] = ACTIONS(3247),
    [anon_sym_true] = ACTIONS(3245),
    [anon_sym_false] = ACTIONS(3245),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3245),
    [sym_super] = ACTIONS(3245),
    [sym_crate] = ACTIONS(3245),
    [sym_metavariable] = ACTIONS(3247),
    [sym__raw_string_literal_start] = ACTIONS(3247),
    [sym_float_literal] = ACTIONS(3247),
  },
  [1012] = {
    [sym_line_comment] = STATE(1012),
    [sym_block_comment] = STATE(1012),
    [sym_identifier] = ACTIONS(3249),
    [anon_sym_LPAREN] = ACTIONS(3251),
    [anon_sym_LBRACK] = ACTIONS(3251),
    [anon_sym_LBRACE] = ACTIONS(3251),
    [anon_sym_STAR] = ACTIONS(3251),
    [anon_sym_u8] = ACTIONS(3249),
    [anon_sym_i8] = ACTIONS(3249),
    [anon_sym_u16] = ACTIONS(3249),
    [anon_sym_i16] = ACTIONS(3249),
    [anon_sym_u32] = ACTIONS(3249),
    [anon_sym_i32] = ACTIONS(3249),
    [anon_sym_u64] = ACTIONS(3249),
    [anon_sym_i64] = ACTIONS(3249),
    [anon_sym_u128] = ACTIONS(3249),
    [anon_sym_i128] = ACTIONS(3249),
    [anon_sym_isize] = ACTIONS(3249),
    [anon_sym_usize] = ACTIONS(3249),
    [anon_sym_f32] = ACTIONS(3249),
    [anon_sym_f64] = ACTIONS(3249),
    [anon_sym_bool] = ACTIONS(3249),
    [anon_sym_str] = ACTIONS(3249),
    [anon_sym_char] = ACTIONS(3249),
    [anon_sym_DASH] = ACTIONS(3249),
    [anon_sym_BANG] = ACTIONS(3251),
    [anon_sym_AMP] = ACTIONS(3251),
    [anon_sym_PIPE] = ACTIONS(3251),
    [anon_sym_LT] = ACTIONS(3251),
    [anon_sym__] = ACTIONS(3249),
    [anon_sym_DOT_DOT] = ACTIONS(3251),
    [anon_sym_COLON_COLON] = ACTIONS(3251),
    [anon_sym_DASH_GT] = ACTIONS(3251),
    [anon_sym_SQUOTE] = ACTIONS(3249),
    [anon_sym_async] = ACTIONS(3249),
    [anon_sym_break] = ACTIONS(3249),
    [anon_sym_const] = ACTIONS(3249),
    [anon_sym_continue] = ACTIONS(3249),
    [anon_sym_default] = ACTIONS(3249),
    [anon_sym_for] = ACTIONS(3249),
    [anon_sym_gen] = ACTIONS(3249),
    [anon_sym_if] = ACTIONS(3249),
    [anon_sym_loop] = ACTIONS(3249),
    [anon_sym_match] = ACTIONS(3249),
    [anon_sym_return] = ACTIONS(3249),
    [anon_sym_static] = ACTIONS(3249),
    [anon_sym_union] = ACTIONS(3249),
    [anon_sym_unsafe] = ACTIONS(3249),
    [anon_sym_while] = ACTIONS(3249),
    [anon_sym_yield] = ACTIONS(3249),
    [anon_sym_move] = ACTIONS(3249),
    [anon_sym_try] = ACTIONS(3249),
    [sym_integer_literal] = ACTIONS(3251),
    [aux_sym_string_literal_token1] = ACTIONS(3251),
    [sym_char_literal] = ACTIONS(3251),
    [anon_sym_true] = ACTIONS(3249),
    [anon_sym_false] = ACTIONS(3249),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(3249),
    [sym_super] = ACTIONS(3249),
    [sym_crate] = ACTIONS(3249),
    [sym_metavariable] = ACTIONS(3251),
    [sym__raw_string_literal_start] = ACTIONS(3251),
    [sym_float_literal] = ACTIONS(3251),
  },
  [1013] = {
    [sym_line_comment] = STATE(1013),
    [sym_block_comment] = STATE(1013),
    [sym_identifier] = ACTIONS(1587),
    [anon_sym_LPAREN] = ACTIONS(1589),
    [anon_sym_LBRACK] = ACTIONS(1589),
    [anon_sym_LBRACE] = ACTIONS(1589),
    [anon_sym_STAR] = ACTIONS(1589),
    [anon_sym_u8] = ACTIONS(1587),
    [anon_sym_i8] = ACTIONS(1587),
    [anon_sym_u16] = ACTIONS(1587),
    [anon_sym_i16] = ACTIONS(1587),
    [anon_sym_u32] = ACTIONS(1587),
    [anon_sym_i32] = ACTIONS(1587),
    [anon_sym_u64] = ACTIONS(1587),
    [anon_sym_i64] = ACTIONS(1587),
    [anon_sym_u128] = ACTIONS(1587),
    [anon_sym_i128] = ACTIONS(1587),
    [anon_sym_isize] = ACTIONS(1587),
    [anon_sym_usize] = ACTIONS(1587),
    [anon_sym_f32] = ACTIONS(1587),
    [anon_sym_f64] = ACTIONS(1587),
    [anon_sym_bool] = ACTIONS(1587),
    [anon_sym_str] = ACTIONS(1587),
    [anon_sym_char] = ACTIONS(1587),
    [anon_sym_DASH] = ACTIONS(1587),
    [anon_sym_BANG] = ACTIONS(1589),
    [anon_sym_AMP] = ACTIONS(1589),
    [anon_sym_PIPE] = ACTIONS(1589),
    [anon_sym_LT] = ACTIONS(1589),
    [anon_sym__] = ACTIONS(1587),
    [anon_sym_DOT_DOT] = ACTIONS(1589),
    [anon_sym_COLON_COLON] = ACTIONS(1589),
    [anon_sym_DASH_GT] = ACTIONS(1589),
    [anon_sym_SQUOTE] = ACTIONS(1587),
    [anon_sym_async] = ACTIONS(1587),
    [anon_sym_break] = ACTIONS(1587),
    [anon_sym_const] = ACTIONS(1587),
    [anon_sym_continue] = ACTIONS(1587),
    [anon_sym_default] = ACTIONS(1587),
    [anon_sym_for] = ACTIONS(1587),
    [anon_sym_gen] = ACTIONS(1587),
    [anon_sym_if] = ACTIONS(1587),
    [anon_sym_loop] = ACTIONS(1587),
    [anon_sym_match] = ACTIONS(1587),
    [anon_sym_return] = ACTIONS(1587),
    [anon_sym_static] = ACTIONS(1587),
    [anon_sym_union] = ACTIONS(1587),
    [anon_sym_unsafe] = ACTIONS(1587),
    [anon_sym_while] = ACTIONS(1587),
    [anon_sym_yield] = ACTIONS(1587),
    [anon_sym_move] = ACTIONS(1587),
    [anon_sym_try] = ACTIONS(1587),
    [sym_integer_literal] = ACTIONS(1589),
    [aux_sym_string_literal_token1] = ACTIONS(1589),
    [sym_char_literal] = ACTIONS(1589),
    [anon_sym_true] = ACTIONS(1587),
    [anon_sym_false] = ACTIONS(1587),
    [anon_sym_SLASH_SLASH] = ACTIONS(103),
    [anon_sym_SLASH_STAR] = ACTIONS(105),
    [sym_self] = ACTIONS(1587),
    [sym_super] = ACTIONS(1587),
    [sym_crate] = ACTIONS(1587),
    [sym_metavariable] = ACTIONS(1589),
    [sym__raw_string_literal_start] = ACTIONS(1589),
    [sym_float_literal] = ACTIONS(1589),
  },
};

static const uint16_t ts_small_parse_table[] = {
  [0] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1014), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1097), 18,
      sym__raw_string_literal_start,
      sym_float_literal,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON_COLON,
      sym_integer_literal,
      aux_sym_string_literal_token1,
      sym_char_literal,
      sym_metavariable,
    ACTIONS(3253), 42,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_SQUOTE,
      anon_sym_async,
      anon_sym_break,
      anon_sym_const,
      anon_sym_continue,
      anon_sym_default,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_if,
      anon_sym_loop,
      anon_sym_match,
      anon_sym_return,
      anon_sym_static,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_while,
      anon_sym_yield,
      anon_sym_move,
      anon_sym_try,
      anon_sym_true,
      anon_sym_false,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [75] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1015), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2099), 17,
      sym__raw_string_literal_start,
      sym_float_literal,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COLON_COLON,
      sym_integer_literal,
      aux_sym_string_literal_token1,
      sym_char_literal,
      sym_metavariable,
    ACTIONS(2101), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym__,
      anon_sym_DOT_DOT,
      anon_sym_SQUOTE,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_ref,
      anon_sym_dyn,
      sym_mutable_specifier,
      anon_sym_true,
      anon_sym_false,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [146] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1016), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3257), 20,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      sym_metavariable,
    ACTIONS(3255), 35,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_where,
      anon_sym_extern,
      anon_sym_else,
      anon_sym_dyn,
      sym_mutable_specifier,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [216] = 21,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1295), 1,
      anon_sym_DASH,
    ACTIONS(1341), 1,
      aux_sym_string_literal_token1,
    ACTIONS(1351), 1,
      sym__raw_string_literal_start,
    ACTIONS(2471), 1,
      anon_sym_COLON_COLON,
    ACTIONS(3259), 1,
      sym_identifier,
    ACTIONS(3269), 1,
      sym_metavariable,
    STATE(2074), 1,
      sym_scoped_identifier,
    STATE(2129), 1,
      sym__literal_pattern,
    STATE(3353), 1,
      sym_bracketed_type,
    STATE(3379), 1,
      sym_generic_type_with_turbofish,
    ACTIONS(1343), 2,
      anon_sym_true,
      anon_sym_false,
    STATE(1017), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1339), 3,
      sym_float_literal,
      sym_integer_literal,
      sym_char_literal,
    ACTIONS(3263), 3,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_in,
    ACTIONS(3267), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(2037), 4,
      sym_negative_literal,
      sym_string_literal,
      sym_raw_string_literal,
      sym_boolean_literal,
    ACTIONS(3261), 7,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
    ACTIONS(3265), 20,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
  [316] = 21,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1295), 1,
      anon_sym_DASH,
    ACTIONS(1341), 1,
      aux_sym_string_literal_token1,
    ACTIONS(1351), 1,
      sym__raw_string_literal_start,
    ACTIONS(2471), 1,
      anon_sym_COLON_COLON,
    ACTIONS(3271), 1,
      sym_identifier,
    ACTIONS(3281), 1,
      sym_metavariable,
    STATE(2079), 1,
      sym_scoped_identifier,
    STATE(2121), 1,
      sym__literal_pattern,
    STATE(3353), 1,
      sym_bracketed_type,
    STATE(3379), 1,
      sym_generic_type_with_turbofish,
    ACTIONS(1343), 2,
      anon_sym_true,
      anon_sym_false,
    STATE(1018), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1339), 3,
      sym_float_literal,
      sym_integer_literal,
      sym_char_literal,
    ACTIONS(3275), 3,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_in,
    ACTIONS(3279), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(2037), 4,
      sym_negative_literal,
      sym_string_literal,
      sym_raw_string_literal,
      sym_boolean_literal,
    ACTIONS(3273), 7,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
    ACTIONS(3277), 20,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
  [416] = 12,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(343), 1,
      anon_sym_LBRACE,
    ACTIONS(3287), 1,
      anon_sym_BANG,
    ACTIONS(3289), 1,
      anon_sym_COLON_COLON,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(3293), 1,
      anon_sym_move,
    STATE(1421), 1,
      sym_block,
    STATE(3569), 1,
      sym_label,
    STATE(1019), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3285), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3283), 28,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_else,
  [495] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3297), 1,
      anon_sym_LPAREN,
    ACTIONS(3301), 1,
      anon_sym_BANG,
    ACTIONS(3303), 1,
      anon_sym_COLON_COLON,
    ACTIONS(3305), 1,
      anon_sym_LT2,
    STATE(1089), 1,
      sym_type_arguments,
    STATE(1095), 1,
      sym_parameters,
    STATE(1020), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3299), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3295), 27,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [572] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1021), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1051), 9,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1049), 40,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_if,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [636] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3297), 1,
      anon_sym_LPAREN,
    ACTIONS(3305), 1,
      anon_sym_LT2,
    ACTIONS(3311), 1,
      anon_sym_COLON_COLON,
    STATE(1089), 1,
      sym_type_arguments,
    STATE(1095), 1,
      sym_parameters,
    STATE(1022), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3309), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3307), 27,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [710] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1023), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1047), 9,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1045), 40,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_if,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [774] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1024), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1475), 9,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1477), 40,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_if,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [838] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1025), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1257), 9,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1259), 40,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_if,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [902] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1026), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1415), 9,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1417), 40,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_if,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [966] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1027), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1269), 9,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1271), 40,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_if,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [1030] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3297), 1,
      anon_sym_LPAREN,
    ACTIONS(3305), 1,
      anon_sym_LT2,
    ACTIONS(3311), 1,
      anon_sym_COLON_COLON,
    STATE(1089), 1,
      sym_type_arguments,
    STATE(1095), 1,
      sym_parameters,
    STATE(1028), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3315), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3313), 27,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [1104] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1029), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1273), 9,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1275), 40,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_if,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [1168] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3297), 1,
      anon_sym_LPAREN,
    ACTIONS(3305), 1,
      anon_sym_LT2,
    ACTIONS(3311), 1,
      anon_sym_COLON_COLON,
    STATE(1089), 1,
      sym_type_arguments,
    STATE(1095), 1,
      sym_parameters,
    STATE(1030), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3319), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3317), 27,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [1242] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1031), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1265), 9,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1267), 40,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_if,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [1306] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1032), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1261), 9,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1263), 40,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_if,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [1370] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1033), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2187), 9,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_EQ,
      anon_sym_LT,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2189), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [1433] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3297), 1,
      anon_sym_LPAREN,
    ACTIONS(3305), 1,
      anon_sym_LT2,
    STATE(1090), 1,
      sym_type_arguments,
    STATE(1096), 1,
      sym_parameters,
    STATE(1034), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3323), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3321), 27,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [1504] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1035), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2733), 9,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_EQ,
      anon_sym_LT,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2735), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [1567] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3297), 1,
      anon_sym_LPAREN,
    ACTIONS(3305), 1,
      anon_sym_LT2,
    STATE(1090), 1,
      sym_type_arguments,
    STATE(1096), 1,
      sym_parameters,
    STATE(1036), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3327), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3325), 27,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [1638] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1037), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1907), 9,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_EQ,
      anon_sym_LT,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1909), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [1701] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3333), 1,
      anon_sym_BANG,
    ACTIONS(3335), 1,
      anon_sym_COLON_COLON,
    STATE(1038), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3331), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3329), 29,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
      anon_sym_LT2,
  [1768] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1039), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1755), 9,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_EQ,
      anon_sym_LT,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1757), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [1831] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3297), 1,
      anon_sym_LPAREN,
    ACTIONS(3305), 1,
      anon_sym_LT2,
    STATE(1090), 1,
      sym_type_arguments,
    STATE(1096), 1,
      sym_parameters,
    STATE(1040), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3339), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3337), 27,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [1902] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3345), 1,
      anon_sym_BANG,
    ACTIONS(3347), 1,
      anon_sym_COLON_COLON,
    STATE(1041), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3343), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3341), 29,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
      anon_sym_LT2,
  [1969] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3353), 1,
      anon_sym_BANG,
    ACTIONS(3355), 1,
      anon_sym_COLON_COLON,
    STATE(1042), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3351), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3349), 29,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
      anon_sym_LT2,
  [2036] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3297), 1,
      anon_sym_LPAREN,
    ACTIONS(3305), 1,
      anon_sym_LT2,
    STATE(1090), 1,
      sym_type_arguments,
    STATE(1096), 1,
      sym_parameters,
    STATE(1043), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3359), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3357), 27,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [2107] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1044), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2199), 9,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_EQ,
      anon_sym_LT,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2201), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [2170] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3365), 1,
      anon_sym_POUND,
    STATE(1468), 2,
      sym_attribute_item,
      sym_inner_attribute_item,
    STATE(1045), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_match_arm_repeat1,
    ACTIONS(3363), 14,
      sym__raw_string_literal_start,
      sym_float_literal,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_DASH,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON_COLON,
      sym_integer_literal,
      aux_sym_string_literal_token1,
      sym_char_literal,
      sym_metavariable,
    ACTIONS(3361), 30,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym__,
      anon_sym_const,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
      anon_sym_ref,
      sym_mutable_specifier,
      anon_sym_true,
      anon_sym_false,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [2237] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3372), 1,
      anon_sym_BANG,
    ACTIONS(3374), 1,
      anon_sym_COLON_COLON,
    STATE(1046), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3370), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3368), 29,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
      anon_sym_LT2,
  [2304] = 27,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1331), 1,
      anon_sym_extern,
    ACTIONS(3107), 1,
      anon_sym_QMARK,
    ACTIONS(3119), 1,
      anon_sym_fn,
    ACTIONS(3376), 1,
      sym_identifier,
    ACTIONS(3378), 1,
      anon_sym_LPAREN,
    ACTIONS(3382), 1,
      anon_sym_COLON_COLON,
    ACTIONS(3384), 1,
      anon_sym_default,
    ACTIONS(3386), 1,
      anon_sym_for,
    ACTIONS(3392), 1,
      sym_metavariable,
    STATE(1043), 1,
      sym_scoped_type_identifier,
    STATE(1093), 1,
      sym_generic_type,
    STATE(1584), 1,
      sym_for_lifetimes,
    STATE(2239), 1,
      aux_sym_function_modifiers_repeat1,
    STATE(2387), 1,
      sym_extern_modifier,
    STATE(3367), 1,
      sym_function_modifiers,
    STATE(3513), 1,
      sym_scoped_identifier,
    STATE(3546), 1,
      sym_generic_type_with_turbofish,
    STATE(3556), 1,
      sym_bracketed_type,
    ACTIONS(3388), 2,
      anon_sym_gen,
      anon_sym_union,
    STATE(1047), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1317), 3,
      anon_sym_async,
      anon_sym_const,
      anon_sym_unsafe,
    ACTIONS(3390), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(1439), 3,
      sym_removed_trait_bound,
      sym_function_type,
      sym_tuple_type,
    ACTIONS(3380), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [2410] = 27,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1291), 1,
      anon_sym_QMARK,
    ACTIONS(1323), 1,
      anon_sym_fn,
    ACTIONS(1331), 1,
      anon_sym_extern,
    ACTIONS(3394), 1,
      sym_identifier,
    ACTIONS(3396), 1,
      anon_sym_LPAREN,
    ACTIONS(3400), 1,
      anon_sym_COLON_COLON,
    ACTIONS(3402), 1,
      anon_sym_default,
    ACTIONS(3404), 1,
      anon_sym_for,
    ACTIONS(3410), 1,
      sym_metavariable,
    STATE(1565), 1,
      sym_for_lifetimes,
    STATE(1935), 1,
      sym_scoped_type_identifier,
    STATE(1966), 1,
      sym_generic_type,
    STATE(2239), 1,
      aux_sym_function_modifiers_repeat1,
    STATE(2387), 1,
      sym_extern_modifier,
    STATE(3369), 1,
      sym_generic_type_with_turbofish,
    STATE(3386), 1,
      sym_function_modifiers,
    STATE(3462), 1,
      sym_scoped_identifier,
    STATE(3499), 1,
      sym_bracketed_type,
    ACTIONS(3406), 2,
      anon_sym_gen,
      anon_sym_union,
    STATE(1048), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1317), 3,
      anon_sym_async,
      anon_sym_const,
      anon_sym_unsafe,
    ACTIONS(3408), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(1994), 3,
      sym_removed_trait_bound,
      sym_function_type,
      sym_tuple_type,
    ACTIONS(3398), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [2516] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1049), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3372), 16,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3374), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [2578] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1050), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3333), 16,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3335), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [2640] = 27,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1325), 1,
      anon_sym_for,
    ACTIONS(1331), 1,
      anon_sym_extern,
    ACTIONS(3149), 1,
      anon_sym_QMARK,
    ACTIONS(3161), 1,
      anon_sym_fn,
    ACTIONS(3412), 1,
      sym_identifier,
    ACTIONS(3414), 1,
      anon_sym_LPAREN,
    ACTIONS(3418), 1,
      anon_sym_COLON_COLON,
    ACTIONS(3420), 1,
      anon_sym_default,
    ACTIONS(3426), 1,
      sym_metavariable,
    STATE(1540), 1,
      sym_scoped_type_identifier,
    STATE(1591), 1,
      sym_for_lifetimes,
    STATE(1619), 1,
      sym_generic_type,
    STATE(2239), 1,
      aux_sym_function_modifiers_repeat1,
    STATE(2387), 1,
      sym_extern_modifier,
    STATE(3341), 1,
      sym_function_modifiers,
    STATE(3538), 1,
      sym_scoped_identifier,
    STATE(3554), 1,
      sym_generic_type_with_turbofish,
    STATE(3560), 1,
      sym_bracketed_type,
    ACTIONS(3422), 2,
      anon_sym_gen,
      anon_sym_union,
    STATE(1051), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1317), 3,
      anon_sym_async,
      anon_sym_const,
      anon_sym_unsafe,
    ACTIONS(3424), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(1729), 3,
      sym_removed_trait_bound,
      sym_function_type,
      sym_tuple_type,
    ACTIONS(3416), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [2746] = 24,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1599), 1,
      anon_sym_LBRACK,
    ACTIONS(3396), 1,
      anon_sym_LPAREN,
    ACTIONS(3400), 1,
      anon_sym_COLON_COLON,
    ACTIONS(3410), 1,
      sym_metavariable,
    ACTIONS(3428), 1,
      sym_identifier,
    ACTIONS(3432), 1,
      anon_sym_STAR,
    ACTIONS(3436), 1,
      anon_sym_AMP,
    ACTIONS(3438), 1,
      anon_sym_SQUOTE,
    ACTIONS(3440), 1,
      anon_sym_for,
    STATE(2664), 1,
      sym_where_predicate,
    STATE(2698), 1,
      sym_scoped_type_identifier,
    STATE(3038), 1,
      sym_generic_type,
    STATE(3369), 1,
      sym_generic_type_with_turbofish,
    STATE(3462), 1,
      sym_scoped_identifier,
    STATE(3499), 1,
      sym_bracketed_type,
    ACTIONS(3430), 2,
      anon_sym_SEMI,
      anon_sym_LBRACE,
    STATE(1052), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3406), 3,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
    ACTIONS(3408), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(3293), 6,
      sym_higher_ranked_trait_bound,
      sym_lifetime,
      sym_array_type,
      sym_tuple_type,
      sym_reference_type,
      sym_pointer_type,
    ACTIONS(3434), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [2846] = 27,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1291), 1,
      anon_sym_QMARK,
    ACTIONS(1323), 1,
      anon_sym_fn,
    ACTIONS(1325), 1,
      anon_sym_for,
    ACTIONS(1331), 1,
      anon_sym_extern,
    ACTIONS(3396), 1,
      anon_sym_LPAREN,
    ACTIONS(3400), 1,
      anon_sym_COLON_COLON,
    ACTIONS(3402), 1,
      anon_sym_default,
    ACTIONS(3410), 1,
      sym_metavariable,
    ACTIONS(3442), 1,
      sym_identifier,
    STATE(1565), 1,
      sym_for_lifetimes,
    STATE(1933), 1,
      sym_scoped_type_identifier,
    STATE(1980), 1,
      sym_generic_type,
    STATE(2239), 1,
      aux_sym_function_modifiers_repeat1,
    STATE(2387), 1,
      sym_extern_modifier,
    STATE(3369), 1,
      sym_generic_type_with_turbofish,
    STATE(3386), 1,
      sym_function_modifiers,
    STATE(3462), 1,
      sym_scoped_identifier,
    STATE(3499), 1,
      sym_bracketed_type,
    ACTIONS(3406), 2,
      anon_sym_gen,
      anon_sym_union,
    STATE(1053), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1317), 3,
      anon_sym_async,
      anon_sym_const,
      anon_sym_unsafe,
    ACTIONS(3408), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(2005), 3,
      sym_removed_trait_bound,
      sym_function_type,
      sym_tuple_type,
    ACTIONS(3398), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [2952] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1054), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3345), 16,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3347), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [3014] = 27,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1325), 1,
      anon_sym_for,
    ACTIONS(1331), 1,
      anon_sym_extern,
    ACTIONS(3107), 1,
      anon_sym_QMARK,
    ACTIONS(3119), 1,
      anon_sym_fn,
    ACTIONS(3378), 1,
      anon_sym_LPAREN,
    ACTIONS(3382), 1,
      anon_sym_COLON_COLON,
    ACTIONS(3384), 1,
      anon_sym_default,
    ACTIONS(3392), 1,
      sym_metavariable,
    ACTIONS(3444), 1,
      sym_identifier,
    STATE(1040), 1,
      sym_scoped_type_identifier,
    STATE(1158), 1,
      sym_generic_type,
    STATE(1584), 1,
      sym_for_lifetimes,
    STATE(2239), 1,
      aux_sym_function_modifiers_repeat1,
    STATE(2387), 1,
      sym_extern_modifier,
    STATE(3367), 1,
      sym_function_modifiers,
    STATE(3513), 1,
      sym_scoped_identifier,
    STATE(3546), 1,
      sym_generic_type_with_turbofish,
    STATE(3556), 1,
      sym_bracketed_type,
    ACTIONS(3388), 2,
      anon_sym_gen,
      anon_sym_union,
    STATE(1055), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1317), 3,
      anon_sym_async,
      anon_sym_const,
      anon_sym_unsafe,
    ACTIONS(3390), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(1410), 3,
      sym_removed_trait_bound,
      sym_function_type,
      sym_tuple_type,
    ACTIONS(3380), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [3120] = 24,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1599), 1,
      anon_sym_LBRACK,
    ACTIONS(3396), 1,
      anon_sym_LPAREN,
    ACTIONS(3400), 1,
      anon_sym_COLON_COLON,
    ACTIONS(3410), 1,
      sym_metavariable,
    ACTIONS(3428), 1,
      sym_identifier,
    ACTIONS(3432), 1,
      anon_sym_STAR,
    ACTIONS(3436), 1,
      anon_sym_AMP,
    ACTIONS(3438), 1,
      anon_sym_SQUOTE,
    ACTIONS(3440), 1,
      anon_sym_for,
    STATE(2664), 1,
      sym_where_predicate,
    STATE(2698), 1,
      sym_scoped_type_identifier,
    STATE(3038), 1,
      sym_generic_type,
    STATE(3369), 1,
      sym_generic_type_with_turbofish,
    STATE(3462), 1,
      sym_scoped_identifier,
    STATE(3499), 1,
      sym_bracketed_type,
    ACTIONS(3446), 2,
      anon_sym_SEMI,
      anon_sym_LBRACE,
    STATE(1056), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3406), 3,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
    ACTIONS(3408), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(3293), 6,
      sym_higher_ranked_trait_bound,
      sym_lifetime,
      sym_array_type,
      sym_tuple_type,
      sym_reference_type,
      sym_pointer_type,
    ACTIONS(3434), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [3220] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3301), 1,
      anon_sym_BANG,
    ACTIONS(3448), 1,
      anon_sym_LBRACE,
    ACTIONS(3450), 1,
      anon_sym_COLON_COLON,
    STATE(1473), 1,
      sym_field_initializer_list,
    STATE(1057), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1357), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1359), 28,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_else,
  [3290] = 27,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1331), 1,
      anon_sym_extern,
    ACTIONS(3149), 1,
      anon_sym_QMARK,
    ACTIONS(3161), 1,
      anon_sym_fn,
    ACTIONS(3414), 1,
      anon_sym_LPAREN,
    ACTIONS(3418), 1,
      anon_sym_COLON_COLON,
    ACTIONS(3420), 1,
      anon_sym_default,
    ACTIONS(3426), 1,
      sym_metavariable,
    ACTIONS(3452), 1,
      sym_identifier,
    ACTIONS(3454), 1,
      anon_sym_for,
    STATE(1557), 1,
      sym_scoped_type_identifier,
    STATE(1591), 1,
      sym_for_lifetimes,
    STATE(1628), 1,
      sym_generic_type,
    STATE(2239), 1,
      aux_sym_function_modifiers_repeat1,
    STATE(2387), 1,
      sym_extern_modifier,
    STATE(3341), 1,
      sym_function_modifiers,
    STATE(3538), 1,
      sym_scoped_identifier,
    STATE(3554), 1,
      sym_generic_type_with_turbofish,
    STATE(3560), 1,
      sym_bracketed_type,
    ACTIONS(3422), 2,
      anon_sym_gen,
      anon_sym_union,
    STATE(1058), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1317), 3,
      anon_sym_async,
      anon_sym_const,
      anon_sym_unsafe,
    ACTIONS(3424), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(1697), 3,
      sym_removed_trait_bound,
      sym_function_type,
      sym_tuple_type,
    ACTIONS(3416), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [3396] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3289), 1,
      anon_sym_COLON_COLON,
    ACTIONS(3456), 1,
      anon_sym_BANG,
    STATE(1059), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3458), 6,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
    ACTIONS(3285), 16,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
      anon_sym_as,
    ACTIONS(3283), 23,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [3464] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1060), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3353), 16,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3355), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [3526] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3460), 1,
      anon_sym_POUND,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(1061), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_enum_variant_list_repeat1,
    ACTIONS(789), 11,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      sym_integer_literal,
      sym_metavariable,
    ACTIONS(3243), 33,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_pub,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_dyn,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [3592] = 12,
    ACTIONS(19), 1,
      anon_sym_LBRACE,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3289), 1,
      anon_sym_COLON_COLON,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(3456), 1,
      anon_sym_BANG,
    ACTIONS(3463), 1,
      anon_sym_move,
    STATE(395), 1,
      sym_block,
    STATE(3355), 1,
      sym_label,
    STATE(1062), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3285), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3283), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [3667] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1063), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3467), 13,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      sym_metavariable,
    ACTIONS(3465), 33,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_where,
      anon_sym_extern,
      anon_sym_dyn,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [3728] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3469), 1,
      anon_sym_LBRACE,
    STATE(1064), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3333), 16,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3335), 29,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_as,
      anon_sym_else,
  [3791] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1065), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3473), 13,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      sym_metavariable,
    ACTIONS(3471), 33,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_where,
      anon_sym_extern,
      anon_sym_dyn,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [3852] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1066), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3477), 13,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      sym_metavariable,
    ACTIONS(3475), 33,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_where,
      anon_sym_extern,
      anon_sym_dyn,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [3913] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1067), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3481), 13,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      sym_metavariable,
    ACTIONS(3479), 33,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_where,
      anon_sym_extern,
      anon_sym_dyn,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [3974] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1068), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3485), 13,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      sym_metavariable,
    ACTIONS(3483), 33,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_where,
      anon_sym_extern,
      anon_sym_dyn,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [4035] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1069), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3489), 13,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      sym_metavariable,
    ACTIONS(3487), 33,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_where,
      anon_sym_extern,
      anon_sym_dyn,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [4096] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3495), 1,
      anon_sym_RBRACE,
    STATE(1070), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3493), 15,
      sym__raw_string_literal_start,
      sym_float_literal,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_DASH,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_integer_literal,
      aux_sym_string_literal_token1,
      sym_char_literal,
      sym_metavariable,
    ACTIONS(3491), 30,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym__,
      anon_sym_const,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
      anon_sym_ref,
      sym_mutable_specifier,
      anon_sym_true,
      anon_sym_false,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [4159] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3497), 1,
      anon_sym_LBRACE,
    STATE(1071), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3345), 16,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3347), 29,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_as,
      anon_sym_else,
  [4222] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1072), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3501), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(3499), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [4283] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3503), 1,
      anon_sym_LBRACE,
    STATE(1073), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3353), 16,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3355), 29,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_as,
      anon_sym_else,
  [4346] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3509), 1,
      anon_sym_RBRACE,
    STATE(1074), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3507), 15,
      sym__raw_string_literal_start,
      sym_float_literal,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_DASH,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_integer_literal,
      aux_sym_string_literal_token1,
      sym_char_literal,
      sym_metavariable,
    ACTIONS(3505), 30,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym__,
      anon_sym_const,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
      anon_sym_ref,
      sym_mutable_specifier,
      anon_sym_true,
      anon_sym_false,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [4409] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1075), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3513), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3511), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [4470] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1076), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3517), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3515), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [4531] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1077), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3521), 13,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      sym_metavariable,
    ACTIONS(3519), 33,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_where,
      anon_sym_extern,
      anon_sym_dyn,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [4592] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3523), 1,
      anon_sym_LBRACE,
    STATE(1078), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3372), 16,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3374), 29,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_as,
      anon_sym_else,
  [4655] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1079), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1583), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1585), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [4716] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1080), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2857), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2859), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [4777] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3525), 1,
      anon_sym_COLON_COLON,
    STATE(1081), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3323), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3321), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [4840] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3527), 1,
      anon_sym_COLON_COLON,
    STATE(1082), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3323), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3321), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [4903] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3529), 1,
      anon_sym_COLON_COLON,
    STATE(1083), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3323), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3321), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [4966] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1084), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2539), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2541), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [5027] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1085), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2573), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2575), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [5088] = 21,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1655), 1,
      anon_sym_DASH,
    ACTIONS(1679), 1,
      aux_sym_string_literal_token1,
    ACTIONS(1687), 1,
      sym__raw_string_literal_start,
    ACTIONS(3275), 1,
      anon_sym_if,
    ACTIONS(3531), 1,
      sym_identifier,
    ACTIONS(3535), 1,
      anon_sym_COLON_COLON,
    ACTIONS(3539), 1,
      sym_metavariable,
    STATE(2632), 1,
      sym_scoped_identifier,
    STATE(2864), 1,
      sym__literal_pattern,
    STATE(3495), 1,
      sym_bracketed_type,
    STATE(3508), 1,
      sym_generic_type_with_turbofish,
    ACTIONS(1681), 2,
      anon_sym_true,
      anon_sym_false,
    ACTIONS(3273), 2,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
    STATE(1086), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1677), 3,
      sym_float_literal,
      sym_integer_literal,
      sym_char_literal,
    ACTIONS(3537), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(2364), 4,
      sym_negative_literal,
      sym_string_literal,
      sym_raw_string_literal,
      sym_boolean_literal,
    ACTIONS(3533), 20,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
  [5181] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1087), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3543), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3541), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [5242] = 21,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1655), 1,
      anon_sym_DASH,
    ACTIONS(1679), 1,
      aux_sym_string_literal_token1,
    ACTIONS(1687), 1,
      sym__raw_string_literal_start,
    ACTIONS(3263), 1,
      anon_sym_if,
    ACTIONS(3535), 1,
      anon_sym_COLON_COLON,
    ACTIONS(3545), 1,
      sym_identifier,
    ACTIONS(3551), 1,
      sym_metavariable,
    STATE(2718), 1,
      sym_scoped_identifier,
    STATE(2882), 1,
      sym__literal_pattern,
    STATE(3495), 1,
      sym_bracketed_type,
    STATE(3508), 1,
      sym_generic_type_with_turbofish,
    ACTIONS(1681), 2,
      anon_sym_true,
      anon_sym_false,
    ACTIONS(3261), 2,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
    STATE(1088), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1677), 3,
      sym_float_literal,
      sym_integer_literal,
      sym_char_literal,
    ACTIONS(3549), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(2364), 4,
      sym_negative_literal,
      sym_string_literal,
      sym_raw_string_literal,
      sym_boolean_literal,
    ACTIONS(3547), 20,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
  [5335] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1089), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3555), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3553), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [5396] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1090), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3559), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3557), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [5457] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1091), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3255), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3257), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [5518] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3565), 1,
      anon_sym_DASH_GT,
    STATE(1092), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3563), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3561), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [5581] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3529), 1,
      anon_sym_COLON_COLON,
    STATE(1093), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3359), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3357), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [5644] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3529), 1,
      anon_sym_COLON_COLON,
    STATE(1094), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3327), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3325), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [5707] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3571), 1,
      anon_sym_DASH_GT,
    STATE(1095), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3569), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3567), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [5770] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3577), 1,
      anon_sym_DASH_GT,
    STATE(1096), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3575), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3573), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [5833] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1097), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2535), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2537), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [5894] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3579), 1,
      anon_sym_else,
    STATE(1387), 1,
      sym_else_clause,
    STATE(1098), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1253), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1251), 29,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
  [5959] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1099), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2641), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2643), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [6020] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1100), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2645), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2647), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [6081] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1101), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2821), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2823), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [6142] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1102), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2885), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2887), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [6203] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1103), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2889), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2891), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [6264] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1104), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2909), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2911), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [6325] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1105), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2945), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2947), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [6386] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1106), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2131), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2133), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [6447] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1107), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2159), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2161), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [6508] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1108), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2183), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2185), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [6569] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1109), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2605), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2607), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [6630] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1110), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2625), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2627), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [6691] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1111), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2717), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2719), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [6752] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1112), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2725), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2727), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [6813] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1113), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1899), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1901), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [6874] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1114), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1983), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1985), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [6935] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1115), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3343), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3341), 29,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
      anon_sym_LT2,
  [6996] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1116), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3583), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3581), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_DASH_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [7057] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3589), 1,
      anon_sym_DASH_GT,
    STATE(1117), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3587), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3585), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [7120] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3595), 1,
      anon_sym_DASH_GT,
    STATE(1118), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3593), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3591), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [7183] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3601), 1,
      anon_sym_DASH_GT,
    STATE(1119), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3599), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3597), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [7246] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1120), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2111), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2113), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [7307] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1121), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2123), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2125), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [7368] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1122), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2127), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2129), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [7429] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1123), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2135), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2137), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [7490] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1124), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2139), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2141), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [7551] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1125), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2151), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2153), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [7612] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1126), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2163), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2165), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [7673] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1127), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2171), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2173), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [7734] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1128), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2175), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2177), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [7795] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1129), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2179), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2181), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [7856] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1130), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2191), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2193), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [7917] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1131), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2195), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2197), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [7978] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1132), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2203), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2205), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [8039] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1133), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2211), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2213), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [8100] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1134), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2251), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2253), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [8161] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1135), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2255), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2257), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [8222] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1136), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2259), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2261), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [8283] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1137), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2263), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2265), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [8344] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1138), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2957), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2959), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [8405] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1139), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1699), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1701), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [8466] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1140), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2271), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2273), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [8527] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1141), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2275), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2277), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [8588] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1142), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2283), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2285), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [8649] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1143), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2287), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2289), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [8710] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1144), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2291), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2293), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [8771] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1145), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2295), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2297), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [8832] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1146), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2299), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2301), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [8893] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1147), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2389), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2391), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [8954] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1148), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2393), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2395), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [9015] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1149), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2397), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2399), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [9076] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1150), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2401), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2403), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [9137] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1151), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2405), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2407), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [9198] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1152), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2409), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2411), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [9259] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1153), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2413), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2415), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [9320] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1154), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2425), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2427), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [9381] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1155), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2445), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2447), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [9442] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1156), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3605), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3603), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [9503] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1157), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3609), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3607), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_DASH_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [9564] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3529), 1,
      anon_sym_COLON_COLON,
    STATE(1158), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3339), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3337), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [9627] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3615), 1,
      anon_sym_DASH_GT,
    STATE(1159), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3613), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3611), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [9690] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1160), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2527), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2529), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [9751] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1161), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2531), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2533), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [9812] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1162), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2543), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2545), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [9873] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1163), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2553), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2555), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [9934] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1164), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2557), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2559), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [9995] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1165), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2561), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2563), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [10056] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1166), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2565), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2567), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [10117] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1167), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2569), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2571), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [10178] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1168), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2589), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2591), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [10239] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1169), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2593), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2595), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [10300] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1170), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2597), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2599), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [10361] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1171), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2609), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2611), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [10422] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1172), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2613), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2615), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [10483] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1173), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2617), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2619), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [10544] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1174), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2621), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2623), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [10605] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1175), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2633), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2635), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [10666] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1176), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2637), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2639), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [10727] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1177), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2721), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2723), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [10788] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1178), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2749), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2751), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [10849] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1179), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2753), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2755), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [10910] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1180), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2757), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2759), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [10971] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1181), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2761), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2763), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [11032] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1182), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2765), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2767), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [11093] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1183), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2769), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2771), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [11154] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1184), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2773), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2775), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [11215] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1185), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2777), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2779), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [11276] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1186), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2781), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2783), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [11337] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1187), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2785), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2787), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [11398] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1188), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2789), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2791), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [11459] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1189), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2793), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2795), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [11520] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1190), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2797), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2799), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [11581] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1191), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2801), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2803), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [11642] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1192), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2805), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2807), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [11703] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1193), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2809), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2811), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [11764] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1194), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2813), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2815), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [11825] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1195), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2817), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2819), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [11886] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1196), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2825), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2827), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [11947] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1197), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2829), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2831), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [12008] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1198), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2833), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2835), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [12069] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1199), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2837), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2839), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [12130] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1200), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2841), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2843), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [12191] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1201), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2845), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2847), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [12252] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1202), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2849), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2851), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [12313] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1203), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2853), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2855), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [12374] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1204), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2861), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2863), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [12435] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1205), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2865), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2867), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [12496] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1206), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2869), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2871), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [12557] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1207), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2877), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2879), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [12618] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1208), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2905), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2907), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [12679] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1209), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2913), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2915), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [12740] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1210), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3619), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3617), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [12801] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1211), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3623), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3621), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_DASH_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [12862] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1212), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2953), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2955), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [12923] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1213), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2267), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2269), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [12984] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1214), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2155), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2157), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [13045] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1215), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2741), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2743), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [13106] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1216), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2745), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2747), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [13167] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1217), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2873), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2875), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [13228] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1218), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2881), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2883), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [13289] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1219), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1743), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1745), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [13350] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1220), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2075), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2077), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [13411] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1221), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2079), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2081), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [13472] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1222), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2083), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2085), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [13533] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1223), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2087), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2089), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [13594] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1224), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2091), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2093), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [13655] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1225), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2095), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2097), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [13716] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1226), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2103), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2105), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [13777] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1227), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2107), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2109), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [13838] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1228), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2115), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2117), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [13899] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1229), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2119), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2121), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [13960] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1230), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3627), 13,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      sym_metavariable,
    ACTIONS(3625), 33,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_where,
      anon_sym_extern,
      anon_sym_dyn,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [14021] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1231), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2147), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2149), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [14082] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1232), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2167), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2169), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [14143] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1233), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2207), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2209), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [14204] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1234), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2215), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2217), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [14265] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1235), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2219), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2221), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [14326] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1236), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2223), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2225), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [14387] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1237), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2227), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2229), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [14448] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1238), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2231), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2233), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [14509] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1239), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2235), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2237), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [14570] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1240), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2239), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2241), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [14631] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1241), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2243), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2245), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [14692] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1242), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2247), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2249), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [14753] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1243), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2279), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2281), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [14814] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1244), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2417), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2419), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [14875] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1245), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2421), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2423), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [14936] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1246), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2429), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2431), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [14997] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1247), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2433), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2435), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [15058] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1248), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2437), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2439), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [15119] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1249), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2441), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2443), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [15180] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1250), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2453), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2455), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [15241] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1251), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2457), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2459), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [15302] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1252), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2513), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2515), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [15363] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1253), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2517), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2519), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [15424] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1254), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2521), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2523), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [15485] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1255), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2547), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2549), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [15546] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1256), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2577), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2579), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [15607] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1257), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2581), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2583), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [15668] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1258), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2585), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2587), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [15729] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1259), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2601), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2603), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [15790] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1260), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2729), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2731), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [15851] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1261), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2737), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2739), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [15912] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1262), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3631), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3629), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [15973] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1263), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3635), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3633), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_DASH_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [16034] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1264), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2893), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2895), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [16095] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1265), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2897), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2899), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [16156] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1266), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2901), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2903), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [16217] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1267), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2917), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2919), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [16278] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1268), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2921), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2923), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [16339] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1269), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2925), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2927), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [16400] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1270), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2929), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2931), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [16461] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1271), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2933), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2935), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [16522] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1272), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2937), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2939), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [16583] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1273), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2941), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2943), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [16644] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1274), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2949), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2951), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [16705] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1275), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1703), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1705), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [16766] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1276), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1707), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1709), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [16827] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1277), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1711), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1713), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [16888] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1278), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1715), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1717), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [16949] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1279), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1719), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1721), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [17010] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1280), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1723), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1725), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [17071] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1281), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1727), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1729), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [17132] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1282), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1731), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1733), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [17193] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1283), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1735), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1737), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [17254] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1284), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1739), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1741), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [17315] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1285), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1747), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1749), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [17376] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1286), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1751), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1753), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [17437] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1287), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1759), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1761), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [17498] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1288), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1763), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1765), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [17559] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1289), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1767), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1769), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [17620] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1290), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1771), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1773), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [17681] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1291), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1775), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1777), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [17742] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1292), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1779), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1781), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [17803] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1293), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1783), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1785), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [17864] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1294), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1787), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1789), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [17925] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1295), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1791), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1793), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [17986] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1296), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1795), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1797), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [18047] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1297), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1799), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1801), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [18108] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1298), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1803), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1805), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [18169] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1299), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1807), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1809), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [18230] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1300), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1811), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1813), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [18291] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1301), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1815), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1817), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [18352] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1302), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1819), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1821), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [18413] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1303), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1823), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1825), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [18474] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1304), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1827), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1829), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [18535] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1305), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1831), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1833), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [18596] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1306), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1835), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1837), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [18657] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1307), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1839), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1841), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [18718] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1308), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1843), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1845), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [18779] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1309), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1847), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1849), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [18840] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1310), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1851), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1853), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [18901] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1311), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1855), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1857), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [18962] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1312), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3639), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3637), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [19023] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1313), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3643), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3641), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_DASH_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [19084] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1314), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1859), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1861), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [19145] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1315), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1863), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1865), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [19206] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1316), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1867), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1869), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [19267] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1317), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1871), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1873), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [19328] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1318), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1875), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1877), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [19389] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1319), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1879), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1881), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [19450] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1320), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1883), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1885), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [19511] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1321), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1887), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1889), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [19572] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1322), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1891), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1893), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [19633] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1323), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1895), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1897), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [19694] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1324), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1903), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1905), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [19755] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1325), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1911), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1913), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [19816] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1326), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1915), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1917), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [19877] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1327), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1919), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1921), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [19938] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1328), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1923), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1925), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [19999] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1329), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1927), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1929), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [20060] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1330), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1931), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1933), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [20121] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1331), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1935), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1937), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [20182] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1332), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1939), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1941), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [20243] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1333), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1943), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1945), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [20304] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1334), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1947), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1949), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [20365] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1335), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1951), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1953), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [20426] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1336), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1955), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1957), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [20487] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1337), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1959), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1961), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [20548] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1338), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1963), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1965), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [20609] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1339), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1967), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1969), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [20670] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1340), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1971), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1973), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [20731] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1341), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1975), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1977), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [20792] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1342), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1979), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1981), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [20853] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1343), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1987), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1989), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [20914] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1344), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1991), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1993), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [20975] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1345), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1995), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(1997), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [21036] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1346), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1999), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2001), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [21097] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1347), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2003), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2005), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [21158] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1348), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2007), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2009), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [21219] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1349), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2011), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2013), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [21280] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1350), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2015), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2017), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [21341] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1351), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2019), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2021), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [21402] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1352), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2023), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2025), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [21463] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1353), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2027), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2029), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [21524] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1354), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2031), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2033), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [21585] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1355), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2035), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2037), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [21646] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1356), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2039), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2041), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [21707] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1357), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2043), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2045), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [21768] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1358), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2047), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2049), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [21829] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1359), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2051), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2053), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [21890] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1360), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2055), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2057), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [21951] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1361), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2059), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2061), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [22012] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1362), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2063), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2065), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [22073] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1363), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2067), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2069), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [22134] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1364), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2071), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2073), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [22195] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(1384), 1,
      sym_label,
    STATE(1365), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3647), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3645), 29,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_else,
  [22260] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1366), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3651), 13,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      sym_metavariable,
    ACTIONS(3649), 33,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_where,
      anon_sym_extern,
      anon_sym_dyn,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [22321] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1367), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3655), 13,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_EQ,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      sym_metavariable,
    ACTIONS(3653), 33,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_where,
      anon_sym_extern,
      anon_sym_dyn,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [22382] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1368), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2099), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2101), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [22443] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3659), 1,
      anon_sym_LPAREN,
    STATE(1481), 1,
      sym_arguments,
    STATE(1369), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3661), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3657), 29,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [22508] = 12,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1229), 1,
      anon_sym_LBRACE,
    ACTIONS(3287), 1,
      anon_sym_BANG,
    ACTIONS(3289), 1,
      anon_sym_COLON_COLON,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(3663), 1,
      anon_sym_move,
    STATE(461), 1,
      sym_block,
    STATE(3619), 1,
      sym_label,
    STATE(1370), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3285), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3283), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_as,
  [22583] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3665), 1,
      anon_sym_COLON_COLON,
    STATE(1371), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1357), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1359), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [22646] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1372), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2143), 7,
      anon_sym_SEMI,
      anon_sym_macro_rules_BANG,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_metavariable,
    ACTIONS(2145), 39,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [22707] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3669), 2,
      anon_sym_LBRACE,
      anon_sym_COLON_COLON,
    STATE(1373), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3671), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3667), 28,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_else,
  [22769] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1374), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3675), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3673), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [22829] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1375), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1489), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1487), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [22889] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1376), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1271), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1269), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [22949] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    STATE(1377), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3681), 14,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(3677), 27,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_else,
  [23017] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1378), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3691), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3689), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [23077] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1379), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3695), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3693), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [23137] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1380), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3699), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3697), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [23197] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1381), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3703), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3701), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [23257] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1382), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3707), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3705), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [23317] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1383), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3707), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3705), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [23377] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1384), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3711), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3709), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [23437] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1385), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1045), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1047), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [23497] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1386), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3715), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3713), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [23557] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1387), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1279), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1277), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [23617] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1388), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3719), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3717), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [23677] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1389), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3723), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3721), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [23737] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1390), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1263), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1261), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [23797] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1391), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1431), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1429), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [23857] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1392), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3727), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3725), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [23917] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1393), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3731), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3729), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [23977] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    STATE(1394), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3735), 14,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(3733), 28,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [24043] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1395), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3671), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3667), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [24103] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1396), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3493), 15,
      sym__raw_string_literal_start,
      sym_float_literal,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_DASH,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_integer_literal,
      aux_sym_string_literal_token1,
      sym_char_literal,
      sym_metavariable,
    ACTIONS(3491), 30,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym__,
      anon_sym_const,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
      anon_sym_ref,
      sym_mutable_specifier,
      anon_sym_true,
      anon_sym_false,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [24163] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1397), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3739), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3737), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [24223] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1398), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3743), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3741), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [24283] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1399), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3747), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3745), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [24343] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1400), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3751), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3749), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [24403] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1401), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3507), 15,
      sym__raw_string_literal_start,
      sym_float_literal,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_DASH,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_integer_literal,
      aux_sym_string_literal_token1,
      sym_char_literal,
      sym_metavariable,
    ACTIONS(3505), 30,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym__,
      anon_sym_const,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
      anon_sym_ref,
      sym_mutable_specifier,
      anon_sym_true,
      anon_sym_false,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [24463] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    STATE(1402), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3755), 14,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(3753), 28,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [24529] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1403), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3759), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3757), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [24589] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1404), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1461), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1459), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [24649] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1405), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3247), 12,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      anon_sym_SQUOTE,
      sym_integer_literal,
      sym_metavariable,
    ACTIONS(3245), 33,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_pub,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_dyn,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [24709] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1406), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3763), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3761), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [24769] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1407), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3767), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3765), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [24829] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1408), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1275), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1273), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [24889] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1409), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3771), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3769), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [24949] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1410), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3339), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3337), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [25009] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1411), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3775), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3773), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [25069] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1412), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3779), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3777), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [25129] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1413), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1007), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1009), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [25189] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1414), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3783), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3781), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [25249] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1415), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3787), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3785), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [25309] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1416), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3323), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3321), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [25369] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1417), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1049), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1051), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [25429] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    STATE(1418), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3791), 14,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(3789), 28,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [25495] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1419), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1397), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1395), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [25555] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1420), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1457), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1455), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [25615] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1421), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1449), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1447), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [25675] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1422), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1453), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1451), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [25735] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1423), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1481), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1479), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [25795] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1424), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1435), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1433), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [25855] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1425), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1485), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1483), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [25915] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1426), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1367), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1365), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [25975] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1427), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1405), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1403), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [26035] = 23,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1599), 1,
      anon_sym_LBRACK,
    ACTIONS(3396), 1,
      anon_sym_LPAREN,
    ACTIONS(3400), 1,
      anon_sym_COLON_COLON,
    ACTIONS(3410), 1,
      sym_metavariable,
    ACTIONS(3428), 1,
      sym_identifier,
    ACTIONS(3432), 1,
      anon_sym_STAR,
    ACTIONS(3436), 1,
      anon_sym_AMP,
    ACTIONS(3438), 1,
      anon_sym_SQUOTE,
    ACTIONS(3440), 1,
      anon_sym_for,
    STATE(2492), 1,
      sym_where_predicate,
    STATE(2698), 1,
      sym_scoped_type_identifier,
    STATE(3038), 1,
      sym_generic_type,
    STATE(3369), 1,
      sym_generic_type_with_turbofish,
    STATE(3462), 1,
      sym_scoped_identifier,
    STATE(3499), 1,
      sym_bracketed_type,
    STATE(1428), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3406), 3,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
    ACTIONS(3408), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(3293), 6,
      sym_higher_ranked_trait_bound,
      sym_lifetime,
      sym_array_type,
      sym_tuple_type,
      sym_reference_type,
      sym_pointer_type,
    ACTIONS(3434), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [26131] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1429), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1259), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1257), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [26191] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1430), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1413), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1411), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [26251] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    STATE(1431), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3795), 14,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(3793), 28,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [26317] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1432), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3799), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3797), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [26377] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1433), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3803), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3801), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [26437] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1434), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3807), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3805), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [26497] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1435), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3811), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3809), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [26557] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1436), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3815), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3813), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [26617] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1437), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3819), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3817), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [26677] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1438), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3823), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3821), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [26737] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1439), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3359), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3357), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [26797] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1440), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3327), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3325), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [26857] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1441), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1357), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1359), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [26917] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1442), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3827), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3825), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [26977] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1443), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1401), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1399), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [27037] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1444), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3831), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3829), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [27097] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1445), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3835), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3833), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [27157] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1446), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1363), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1361), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [27217] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1447), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3839), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3837), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [27277] = 23,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1599), 1,
      anon_sym_LBRACK,
    ACTIONS(3396), 1,
      anon_sym_LPAREN,
    ACTIONS(3400), 1,
      anon_sym_COLON_COLON,
    ACTIONS(3410), 1,
      sym_metavariable,
    ACTIONS(3428), 1,
      sym_identifier,
    ACTIONS(3432), 1,
      anon_sym_STAR,
    ACTIONS(3436), 1,
      anon_sym_AMP,
    ACTIONS(3438), 1,
      anon_sym_SQUOTE,
    ACTIONS(3440), 1,
      anon_sym_for,
    STATE(2664), 1,
      sym_where_predicate,
    STATE(2698), 1,
      sym_scoped_type_identifier,
    STATE(3038), 1,
      sym_generic_type,
    STATE(3369), 1,
      sym_generic_type_with_turbofish,
    STATE(3462), 1,
      sym_scoped_identifier,
    STATE(3499), 1,
      sym_bracketed_type,
    STATE(1448), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3406), 3,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
    ACTIONS(3408), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(3293), 6,
      sym_higher_ranked_trait_bound,
      sym_lifetime,
      sym_array_type,
      sym_tuple_type,
      sym_reference_type,
      sym_pointer_type,
    ACTIONS(3434), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [27373] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1449), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1439), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1437), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [27433] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1450), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3843), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3841), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [27493] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1451), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1027), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1029), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [27553] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1452), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3847), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3845), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [27613] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3287), 1,
      anon_sym_BANG,
    ACTIONS(3289), 1,
      anon_sym_COLON_COLON,
    STATE(1453), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3285), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3283), 28,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_else,
  [27677] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1454), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1267), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1265), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [27737] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1455), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1469), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1467), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [27797] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3849), 2,
      anon_sym_LBRACE,
      anon_sym_COLON_COLON,
    STATE(1456), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3671), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3667), 28,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_else,
  [27859] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1457), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3853), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3851), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [27919] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1458), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3857), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3855), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [27979] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1459), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1357), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1359), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [28039] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1460), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3861), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3859), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [28099] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1461), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3865), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3863), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [28159] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1462), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2099), 12,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      anon_sym_SQUOTE,
      sym_integer_literal,
      sym_metavariable,
    ACTIONS(2101), 33,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_pub,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_dyn,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [28219] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1463), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3869), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3867), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [28279] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1464), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2527), 15,
      sym__raw_string_literal_start,
      sym_float_literal,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_DASH,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_integer_literal,
      aux_sym_string_literal_token1,
      sym_char_literal,
      sym_metavariable,
    ACTIONS(2529), 30,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym__,
      anon_sym_const,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
      anon_sym_ref,
      sym_mutable_specifier,
      anon_sym_true,
      anon_sym_false,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [28339] = 12,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(407), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(3871), 1,
      anon_sym_BANG,
    ACTIONS(3873), 1,
      anon_sym_COLON_COLON,
    ACTIONS(3875), 1,
      anon_sym_move,
    STATE(1747), 1,
      sym_block,
    STATE(3620), 1,
      sym_label,
    STATE(1465), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3285), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3283), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [28413] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1466), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3879), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3877), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [28473] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1467), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1409), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1407), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [28533] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1468), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3883), 15,
      sym__raw_string_literal_start,
      sym_float_literal,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_DASH,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_integer_literal,
      aux_sym_string_literal_token1,
      sym_char_literal,
      sym_metavariable,
    ACTIONS(3881), 30,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym__,
      anon_sym_const,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
      anon_sym_ref,
      sym_mutable_specifier,
      anon_sym_true,
      anon_sym_false,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [28593] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1469), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3887), 15,
      sym__raw_string_literal_start,
      sym_float_literal,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_DASH,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_integer_literal,
      aux_sym_string_literal_token1,
      sym_char_literal,
      sym_metavariable,
    ACTIONS(3885), 30,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym__,
      anon_sym_const,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
      anon_sym_ref,
      sym_mutable_specifier,
      anon_sym_true,
      anon_sym_false,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [28653] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1470), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3891), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3889), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [28713] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1471), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1357), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1359), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [28773] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1472), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1011), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1013), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [28833] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1473), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3895), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3893), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [28893] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1474), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1357), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1359), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [28953] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1475), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1417), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1415), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [29013] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1476), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3899), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3897), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [29073] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1477), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3903), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3901), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [29133] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1478), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3907), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3905), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [29193] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1479), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3911), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3909), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [29253] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1480), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3915), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3913), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [29313] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1481), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3919), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3917), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [29373] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1482), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3661), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3657), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [29433] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1483), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3923), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3921), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [29493] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1484), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3927), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3925), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [29553] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1485), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3931), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3929), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [29613] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1486), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3935), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3933), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [29673] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3301), 1,
      anon_sym_BANG,
    ACTIONS(3937), 1,
      anon_sym_COLON_COLON,
    STATE(1487), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1357), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1359), 28,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_else,
  [29737] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1488), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3941), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3939), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [29797] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1489), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(995), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(997), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [29857] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1490), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1357), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1359), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [29917] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1491), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1023), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1025), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [29977] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1492), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3945), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3943), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [30037] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1493), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3949), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3947), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [30097] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1494), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1477), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1475), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [30157] = 25,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1323), 1,
      anon_sym_fn,
    ACTIONS(1331), 1,
      anon_sym_extern,
    ACTIONS(3181), 1,
      anon_sym_for,
    ACTIONS(3400), 1,
      anon_sym_COLON_COLON,
    ACTIONS(3402), 1,
      anon_sym_default,
    ACTIONS(3410), 1,
      sym_metavariable,
    ACTIONS(3951), 1,
      sym_identifier,
    STATE(1565), 1,
      sym_for_lifetimes,
    STATE(1938), 1,
      sym_scoped_type_identifier,
    STATE(1978), 1,
      sym_generic_type,
    STATE(2239), 1,
      aux_sym_function_modifiers_repeat1,
    STATE(2387), 1,
      sym_extern_modifier,
    STATE(3369), 1,
      sym_generic_type_with_turbofish,
    STATE(3386), 1,
      sym_function_modifiers,
    STATE(3462), 1,
      sym_scoped_identifier,
    STATE(3499), 1,
      sym_bracketed_type,
    ACTIONS(3406), 2,
      anon_sym_gen,
      anon_sym_union,
    STATE(1495), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1995), 2,
      sym_higher_ranked_trait_bound,
      sym_function_type,
    ACTIONS(1317), 3,
      anon_sym_async,
      anon_sym_const,
      anon_sym_unsafe,
    ACTIONS(3408), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(3398), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [30256] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3289), 1,
      anon_sym_COLON_COLON,
    STATE(1496), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3285), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3283), 28,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_else,
  [30317] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3287), 1,
      anon_sym_BANG,
    ACTIONS(3293), 1,
      anon_sym_move,
    ACTIONS(3953), 1,
      anon_sym_COLON_COLON,
    STATE(1421), 1,
      sym_block,
    STATE(3569), 1,
      sym_label,
    STATE(1497), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3285), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3283), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
  [30386] = 25,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1331), 1,
      anon_sym_extern,
    ACTIONS(3161), 1,
      anon_sym_fn,
    ACTIONS(3418), 1,
      anon_sym_COLON_COLON,
    ACTIONS(3420), 1,
      anon_sym_default,
    ACTIONS(3426), 1,
      sym_metavariable,
    ACTIONS(3955), 1,
      sym_identifier,
    ACTIONS(3957), 1,
      anon_sym_for,
    STATE(1556), 1,
      sym_scoped_type_identifier,
    STATE(1591), 1,
      sym_for_lifetimes,
    STATE(1635), 1,
      sym_generic_type,
    STATE(2239), 1,
      aux_sym_function_modifiers_repeat1,
    STATE(2387), 1,
      sym_extern_modifier,
    STATE(3341), 1,
      sym_function_modifiers,
    STATE(3538), 1,
      sym_scoped_identifier,
    STATE(3554), 1,
      sym_generic_type_with_turbofish,
    STATE(3560), 1,
      sym_bracketed_type,
    ACTIONS(3422), 2,
      anon_sym_gen,
      anon_sym_union,
    STATE(1498), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1700), 2,
      sym_higher_ranked_trait_bound,
      sym_function_type,
    ACTIONS(1317), 3,
      anon_sym_async,
      anon_sym_const,
      anon_sym_unsafe,
    ACTIONS(3424), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(3416), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [30485] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3287), 1,
      anon_sym_BANG,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(3293), 1,
      anon_sym_move,
    ACTIONS(3953), 1,
      anon_sym_COLON_COLON,
    STATE(1421), 1,
      sym_block,
    STATE(3569), 1,
      sym_label,
    STATE(1499), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3285), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3283), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [30556] = 25,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1331), 1,
      anon_sym_extern,
    ACTIONS(3119), 1,
      anon_sym_fn,
    ACTIONS(3382), 1,
      anon_sym_COLON_COLON,
    ACTIONS(3384), 1,
      anon_sym_default,
    ACTIONS(3392), 1,
      sym_metavariable,
    ACTIONS(3959), 1,
      sym_identifier,
    ACTIONS(3961), 1,
      anon_sym_for,
    STATE(1036), 1,
      sym_scoped_type_identifier,
    STATE(1094), 1,
      sym_generic_type,
    STATE(1584), 1,
      sym_for_lifetimes,
    STATE(2239), 1,
      aux_sym_function_modifiers_repeat1,
    STATE(2387), 1,
      sym_extern_modifier,
    STATE(3367), 1,
      sym_function_modifiers,
    STATE(3513), 1,
      sym_scoped_identifier,
    STATE(3546), 1,
      sym_generic_type_with_turbofish,
    STATE(3556), 1,
      sym_bracketed_type,
    ACTIONS(3388), 2,
      anon_sym_gen,
      anon_sym_union,
    STATE(1440), 2,
      sym_higher_ranked_trait_bound,
      sym_function_type,
    STATE(1500), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1317), 3,
      anon_sym_async,
      anon_sym_const,
      anon_sym_unsafe,
    ACTIONS(3390), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(3380), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [30655] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3963), 1,
      anon_sym_COLON_COLON,
    STATE(1501), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1357), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1359), 28,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_else,
  [30716] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(3989), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3991), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1502), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3939), 7,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_else,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [30808] = 21,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(347), 1,
      anon_sym_EQ,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3989), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3991), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1503), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(341), 17,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_COMMA,
      anon_sym_else,
  [30898] = 19,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1295), 1,
      anon_sym_DASH,
    ACTIONS(1341), 1,
      aux_sym_string_literal_token1,
    ACTIONS(1351), 1,
      sym__raw_string_literal_start,
    ACTIONS(2471), 1,
      anon_sym_COLON_COLON,
    ACTIONS(3271), 1,
      sym_identifier,
    ACTIONS(3281), 1,
      sym_metavariable,
    STATE(2079), 1,
      sym_scoped_identifier,
    STATE(2121), 1,
      sym__literal_pattern,
    STATE(3353), 1,
      sym_bracketed_type,
    STATE(3379), 1,
      sym_generic_type_with_turbofish,
    ACTIONS(1343), 2,
      anon_sym_true,
      anon_sym_false,
    STATE(1504), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1339), 3,
      sym_float_literal,
      sym_integer_literal,
      sym_char_literal,
    ACTIONS(3279), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(2037), 4,
      sym_negative_literal,
      sym_string_literal,
      sym_raw_string_literal,
      sym_boolean_literal,
    ACTIONS(3277), 20,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
  [30984] = 18,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3681), 2,
      anon_sym_EQ,
      anon_sym_DOT_DOT,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    STATE(1505), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3677), 20,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_else,
  [31068] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1506), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3681), 9,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(3677), 25,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_else,
  [31138] = 21,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3989), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3995), 1,
      anon_sym_EQ,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3991), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1507), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3993), 17,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_COMMA,
      anon_sym_else,
  [31228] = 21,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3989), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3999), 1,
      anon_sym_EQ,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3991), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1508), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3997), 17,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_COMMA,
      anon_sym_else,
  [31318] = 19,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1295), 1,
      anon_sym_DASH,
    ACTIONS(1341), 1,
      aux_sym_string_literal_token1,
    ACTIONS(1351), 1,
      sym__raw_string_literal_start,
    ACTIONS(2471), 1,
      anon_sym_COLON_COLON,
    ACTIONS(3259), 1,
      sym_identifier,
    ACTIONS(3269), 1,
      sym_metavariable,
    STATE(2074), 1,
      sym_scoped_identifier,
    STATE(2129), 1,
      sym__literal_pattern,
    STATE(3353), 1,
      sym_bracketed_type,
    STATE(3379), 1,
      sym_generic_type_with_turbofish,
    ACTIONS(1343), 2,
      anon_sym_true,
      anon_sym_false,
    STATE(1509), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1339), 3,
      sym_float_literal,
      sym_integer_literal,
      sym_char_literal,
    ACTIONS(3267), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(2037), 4,
      sym_negative_literal,
      sym_string_literal,
      sym_raw_string_literal,
      sym_boolean_literal,
    ACTIONS(3265), 20,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
  [31404] = 15,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    STATE(1510), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3681), 4,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(3677), 25,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_else,
  [31482] = 19,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4003), 2,
      anon_sym_EQ,
      anon_sym_DOT_DOT,
    STATE(1511), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4001), 19,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_else,
  [31568] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4005), 1,
      anon_sym_LPAREN,
    ACTIONS(4007), 1,
      anon_sym_BANG,
    ACTIONS(4009), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4011), 1,
      anon_sym_LT2,
    STATE(1618), 1,
      sym_type_arguments,
    STATE(1642), 1,
      sym_parameters,
    STATE(1512), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3299), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3295), 20,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [31638] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    STATE(1513), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3681), 11,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(3677), 25,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_else,
  [31706] = 21,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3989), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4015), 1,
      anon_sym_EQ,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3991), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1514), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4013), 17,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_COMMA,
      anon_sym_else,
  [31796] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3448), 1,
      anon_sym_LBRACE,
    ACTIONS(3450), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4017), 1,
      anon_sym_BANG,
    STATE(1473), 1,
      sym_field_initializer_list,
    STATE(1515), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1357), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1359), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [31862] = 19,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(389), 2,
      anon_sym_EQ,
      anon_sym_DOT_DOT,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    STATE(1516), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(387), 19,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_else,
  [31948] = 21,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3989), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4021), 1,
      anon_sym_EQ,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3991), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1517), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4019), 17,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_COMMA,
      anon_sym_else,
  [32038] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(3989), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3991), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1518), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3889), 7,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_else,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [32130] = 21,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3989), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4025), 1,
      anon_sym_EQ,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3991), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1519), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4023), 17,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_COMMA,
      anon_sym_else,
  [32220] = 19,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1655), 1,
      anon_sym_DASH,
    ACTIONS(1679), 1,
      aux_sym_string_literal_token1,
    ACTIONS(1687), 1,
      sym__raw_string_literal_start,
    ACTIONS(3531), 1,
      sym_identifier,
    ACTIONS(3535), 1,
      anon_sym_COLON_COLON,
    ACTIONS(3539), 1,
      sym_metavariable,
    STATE(2632), 1,
      sym_scoped_identifier,
    STATE(2864), 1,
      sym__literal_pattern,
    STATE(3495), 1,
      sym_bracketed_type,
    STATE(3508), 1,
      sym_generic_type_with_turbofish,
    ACTIONS(1681), 2,
      anon_sym_true,
      anon_sym_false,
    STATE(1520), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1677), 3,
      sym_float_literal,
      sym_integer_literal,
      sym_char_literal,
    ACTIONS(3537), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(2364), 4,
      sym_negative_literal,
      sym_string_literal,
      sym_raw_string_literal,
      sym_boolean_literal,
    ACTIONS(3533), 20,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
  [32306] = 13,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    STATE(1521), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3681), 6,
      anon_sym_CARET,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(3677), 25,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_else,
  [32380] = 12,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    STATE(1522), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3681), 7,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(3677), 25,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_else,
  [32452] = 19,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1655), 1,
      anon_sym_DASH,
    ACTIONS(1679), 1,
      aux_sym_string_literal_token1,
    ACTIONS(1687), 1,
      sym__raw_string_literal_start,
    ACTIONS(3535), 1,
      anon_sym_COLON_COLON,
    ACTIONS(3545), 1,
      sym_identifier,
    ACTIONS(3551), 1,
      sym_metavariable,
    STATE(2718), 1,
      sym_scoped_identifier,
    STATE(2882), 1,
      sym__literal_pattern,
    STATE(3495), 1,
      sym_bracketed_type,
    STATE(3508), 1,
      sym_generic_type_with_turbofish,
    ACTIONS(1681), 2,
      anon_sym_true,
      anon_sym_false,
    STATE(1523), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1677), 3,
      sym_float_literal,
      sym_integer_literal,
      sym_char_literal,
    ACTIONS(3549), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(2364), 4,
      sym_negative_literal,
      sym_string_literal,
      sym_raw_string_literal,
      sym_boolean_literal,
    ACTIONS(3547), 20,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
  [32538] = 14,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    STATE(1524), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3681), 5,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(3677), 25,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_else,
  [32614] = 17,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3681), 2,
      anon_sym_EQ,
      anon_sym_DOT_DOT,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    STATE(1525), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3677), 21,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_else,
  [32696] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(3989), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3991), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1526), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3821), 7,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_else,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [32788] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3289), 1,
      anon_sym_COLON_COLON,
    ACTIONS(3456), 1,
      anon_sym_BANG,
    ACTIONS(4027), 1,
      sym_identifier,
    STATE(1527), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3285), 16,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
      anon_sym_as,
    ACTIONS(3283), 23,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [32851] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4005), 1,
      anon_sym_LPAREN,
    ACTIONS(4011), 1,
      anon_sym_LT2,
    ACTIONS(4029), 1,
      anon_sym_COLON_COLON,
    STATE(1618), 1,
      sym_type_arguments,
    STATE(1642), 1,
      sym_parameters,
    STATE(1528), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3319), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3317), 20,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [32918] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1529), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4033), 10,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      sym_metavariable,
    ACTIONS(4031), 32,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_dyn,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [32975] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4005), 1,
      anon_sym_LPAREN,
    ACTIONS(4011), 1,
      anon_sym_LT2,
    ACTIONS(4029), 1,
      anon_sym_COLON_COLON,
    STATE(1618), 1,
      sym_type_arguments,
    STATE(1642), 1,
      sym_parameters,
    STATE(1530), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3315), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3313), 20,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [33042] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1531), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3655), 10,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      sym_metavariable,
    ACTIONS(3653), 32,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_dyn,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [33099] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4039), 1,
      anon_sym_COLON_COLON,
    STATE(1532), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4037), 9,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_SQUOTE,
      sym_metavariable,
    ACTIONS(4035), 32,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_dyn,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [33158] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1533), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3627), 10,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      sym_metavariable,
    ACTIONS(3625), 32,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_dyn,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [33215] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1534), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4044), 10,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      sym_metavariable,
    ACTIONS(4042), 32,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_dyn,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [33272] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1535), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4048), 10,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      sym_metavariable,
    ACTIONS(4046), 32,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_dyn,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [33329] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4005), 1,
      anon_sym_LPAREN,
    ACTIONS(4011), 1,
      anon_sym_LT2,
    ACTIONS(4029), 1,
      anon_sym_COLON_COLON,
    STATE(1618), 1,
      sym_type_arguments,
    STATE(1642), 1,
      sym_parameters,
    STATE(1536), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3309), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3307), 20,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [33396] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4050), 1,
      anon_sym_LPAREN,
    STATE(1537), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4037), 9,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      sym_metavariable,
    ACTIONS(4035), 32,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_dyn,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [33455] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3301), 1,
      anon_sym_BANG,
    ACTIONS(4053), 1,
      anon_sym_COLON_COLON,
    STATE(1473), 1,
      sym_field_initializer_list,
    STATE(1538), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1357), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1359), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
  [33518] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4007), 1,
      anon_sym_BANG,
    ACTIONS(4055), 1,
      anon_sym_LBRACE,
    ACTIONS(4057), 1,
      anon_sym_COLON_COLON,
    STATE(1689), 1,
      sym_field_initializer_list,
    STATE(1539), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1357), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1359), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [33583] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4005), 1,
      anon_sym_LPAREN,
    ACTIONS(4011), 1,
      anon_sym_LT2,
    STATE(1620), 1,
      sym_type_arguments,
    STATE(1646), 1,
      sym_parameters,
    STATE(1540), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3339), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3337), 20,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [33647] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1541), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3353), 16,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3355), 25,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
  [33703] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3497), 1,
      anon_sym_LBRACE,
    STATE(1542), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3345), 16,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3347), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
  [33761] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3372), 1,
      anon_sym_BANG,
    ACTIONS(3374), 1,
      anon_sym_COLON_COLON,
    STATE(1543), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3370), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3368), 22,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
      anon_sym_LT2,
  [33821] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3333), 1,
      anon_sym_BANG,
    ACTIONS(3335), 1,
      anon_sym_COLON_COLON,
    STATE(1544), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3331), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3329), 22,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
      anon_sym_LT2,
  [33881] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3345), 1,
      anon_sym_BANG,
    ACTIONS(3347), 1,
      anon_sym_COLON_COLON,
    STATE(1545), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3343), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3341), 22,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
      anon_sym_LT2,
  [33941] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3353), 1,
      anon_sym_BANG,
    ACTIONS(3355), 1,
      anon_sym_COLON_COLON,
    STATE(1546), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3351), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3349), 22,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
      anon_sym_LT2,
  [34001] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1547), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3333), 16,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3335), 25,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
  [34057] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4005), 1,
      anon_sym_LPAREN,
    ACTIONS(4011), 1,
      anon_sym_LT2,
    STATE(1620), 1,
      sym_type_arguments,
    STATE(1646), 1,
      sym_parameters,
    STATE(1548), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3323), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3321), 20,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [34121] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1549), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3345), 16,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3347), 25,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
  [34177] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3503), 1,
      anon_sym_LBRACE,
    STATE(1550), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3353), 16,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3355), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
  [34235] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3523), 1,
      anon_sym_LBRACE,
    STATE(1551), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3372), 16,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3374), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
  [34293] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3937), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4017), 1,
      anon_sym_BANG,
    STATE(1552), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1357), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1359), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [34353] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3469), 1,
      anon_sym_LBRACE,
    STATE(1553), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3333), 16,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3335), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
  [34411] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1554), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3372), 16,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3374), 25,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
  [34467] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3287), 1,
      anon_sym_BANG,
    ACTIONS(3953), 1,
      anon_sym_COLON_COLON,
    STATE(1555), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3285), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3283), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
  [34527] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4005), 1,
      anon_sym_LPAREN,
    ACTIONS(4011), 1,
      anon_sym_LT2,
    STATE(1620), 1,
      sym_type_arguments,
    STATE(1646), 1,
      sym_parameters,
    STATE(1556), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3327), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3325), 20,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [34591] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4005), 1,
      anon_sym_LPAREN,
    ACTIONS(4011), 1,
      anon_sym_LT2,
    STATE(1620), 1,
      sym_type_arguments,
    STATE(1646), 1,
      sym_parameters,
    STATE(1557), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3359), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3357), 20,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [34655] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3301), 1,
      anon_sym_BANG,
    ACTIONS(4059), 1,
      anon_sym_COLON_COLON,
    STATE(1558), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1357), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1359), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
  [34715] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4061), 1,
      anon_sym_COLON_COLON,
    STATE(1559), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1357), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1359), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
  [34772] = 25,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(921), 1,
      anon_sym_RBRACK,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4063), 1,
      anon_sym_SEMI,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4069), 1,
      anon_sym_COMMA,
    STATE(2771), 1,
      aux_sym_arguments_repeat1,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1560), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [34867] = 25,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(772), 1,
      anon_sym_RBRACK,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4071), 1,
      anon_sym_SEMI,
    ACTIONS(4073), 1,
      anon_sym_COMMA,
    STATE(3055), 1,
      aux_sym_arguments_repeat1,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1561), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [34962] = 25,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(903), 1,
      anon_sym_RBRACK,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4075), 1,
      anon_sym_SEMI,
    ACTIONS(4077), 1,
      anon_sym_COMMA,
    STATE(2797), 1,
      aux_sym_arguments_repeat1,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1562), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [35057] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3849), 2,
      anon_sym_LBRACE,
      anon_sym_COLON_COLON,
    STATE(1563), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3671), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3667), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [35114] = 25,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(343), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4083), 1,
      anon_sym_CARET,
    ACTIONS(4085), 1,
      anon_sym_AMP,
    ACTIONS(4087), 1,
      anon_sym_PIPE,
    ACTIONS(4089), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4091), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4097), 1,
      anon_sym_EQ,
    ACTIONS(4103), 1,
      anon_sym_DOT_DOT,
    STATE(1423), 1,
      sym_block,
    STATE(3569), 1,
      sym_label,
    ACTIONS(4079), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4093), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4101), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4105), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1564), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4081), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4099), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4095), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [35209] = 22,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1331), 1,
      anon_sym_extern,
    ACTIONS(3400), 1,
      anon_sym_COLON_COLON,
    ACTIONS(3402), 1,
      anon_sym_default,
    ACTIONS(3410), 1,
      sym_metavariable,
    ACTIONS(4107), 1,
      sym_identifier,
    ACTIONS(4109), 1,
      anon_sym_fn,
    STATE(2239), 1,
      aux_sym_function_modifiers_repeat1,
    STATE(2387), 1,
      sym_extern_modifier,
    STATE(2741), 1,
      sym_scoped_type_identifier,
    STATE(3369), 1,
      sym_generic_type_with_turbofish,
    STATE(3427), 1,
      sym_generic_type,
    STATE(3462), 1,
      sym_scoped_identifier,
    STATE(3499), 1,
      sym_bracketed_type,
    STATE(3520), 1,
      sym_function_modifiers,
    ACTIONS(3406), 2,
      anon_sym_gen,
      anon_sym_union,
    STATE(1565), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1317), 3,
      anon_sym_async,
      anon_sym_const,
      anon_sym_unsafe,
    ACTIONS(3408), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(3398), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [35298] = 25,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(343), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4083), 1,
      anon_sym_CARET,
    ACTIONS(4085), 1,
      anon_sym_AMP,
    ACTIONS(4087), 1,
      anon_sym_PIPE,
    ACTIONS(4089), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4091), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4097), 1,
      anon_sym_EQ,
    ACTIONS(4103), 1,
      anon_sym_DOT_DOT,
    STATE(1424), 1,
      sym_block,
    STATE(3569), 1,
      sym_label,
    ACTIONS(4079), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4093), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4101), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4105), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1566), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4081), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4099), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4095), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [35393] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4111), 1,
      anon_sym_else,
    STATE(1773), 1,
      sym_else_clause,
    STATE(1567), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1253), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1251), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [35452] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3669), 1,
      anon_sym_COLON_COLON,
    STATE(1568), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3671), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3667), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
  [35509] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1569), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3353), 16,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3355), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
  [35564] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4113), 1,
      anon_sym_SQUOTE,
    STATE(1683), 1,
      sym_label,
    STATE(1570), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3647), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3645), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [35623] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1571), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3372), 16,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3374), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
  [35678] = 25,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1229), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4083), 1,
      anon_sym_CARET,
    ACTIONS(4085), 1,
      anon_sym_AMP,
    ACTIONS(4087), 1,
      anon_sym_PIPE,
    ACTIONS(4089), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4091), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4097), 1,
      anon_sym_EQ,
    ACTIONS(4103), 1,
      anon_sym_DOT_DOT,
    STATE(476), 1,
      sym_block,
    STATE(3619), 1,
      sym_label,
    ACTIONS(4079), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4093), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4101), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4105), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1572), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4081), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4099), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4095), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [35773] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3871), 1,
      anon_sym_BANG,
    ACTIONS(3873), 1,
      anon_sym_COLON_COLON,
    STATE(1573), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3285), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3283), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [35832] = 25,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1229), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4083), 1,
      anon_sym_CARET,
    ACTIONS(4085), 1,
      anon_sym_AMP,
    ACTIONS(4087), 1,
      anon_sym_PIPE,
    ACTIONS(4089), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4091), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4097), 1,
      anon_sym_EQ,
    ACTIONS(4103), 1,
      anon_sym_DOT_DOT,
    STATE(477), 1,
      sym_block,
    STATE(3619), 1,
      sym_label,
    ACTIONS(4079), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4093), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4101), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4105), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1574), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4081), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4099), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4095), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [35927] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4007), 1,
      anon_sym_BANG,
    ACTIONS(4115), 1,
      anon_sym_COLON_COLON,
    STATE(1575), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1357), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1359), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [35986] = 25,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(407), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4083), 1,
      anon_sym_CARET,
    ACTIONS(4085), 1,
      anon_sym_AMP,
    ACTIONS(4087), 1,
      anon_sym_PIPE,
    ACTIONS(4089), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4091), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4097), 1,
      anon_sym_EQ,
    ACTIONS(4103), 1,
      anon_sym_DOT_DOT,
    STATE(1779), 1,
      sym_block,
    STATE(3620), 1,
      sym_label,
    ACTIONS(4079), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4093), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4101), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4105), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1576), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4081), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4099), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4095), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [36081] = 25,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(407), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4083), 1,
      anon_sym_CARET,
    ACTIONS(4085), 1,
      anon_sym_AMP,
    ACTIONS(4087), 1,
      anon_sym_PIPE,
    ACTIONS(4089), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4091), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4097), 1,
      anon_sym_EQ,
    ACTIONS(4103), 1,
      anon_sym_DOT_DOT,
    STATE(1785), 1,
      sym_block,
    STATE(3620), 1,
      sym_label,
    ACTIONS(4079), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4093), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4101), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4105), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1577), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4081), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4099), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4095), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [36176] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1578), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3619), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3617), 25,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
  [36231] = 25,
    ACTIONS(19), 1,
      anon_sym_LBRACE,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4083), 1,
      anon_sym_CARET,
    ACTIONS(4085), 1,
      anon_sym_AMP,
    ACTIONS(4087), 1,
      anon_sym_PIPE,
    ACTIONS(4089), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4091), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4097), 1,
      anon_sym_EQ,
    ACTIONS(4103), 1,
      anon_sym_DOT_DOT,
    STATE(392), 1,
      sym_block,
    STATE(3355), 1,
      sym_label,
    ACTIONS(4079), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4093), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4101), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4105), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1579), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4081), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4099), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4095), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [36326] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3953), 1,
      anon_sym_COLON_COLON,
    STATE(1580), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3285), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3283), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
  [36383] = 25,
    ACTIONS(19), 1,
      anon_sym_LBRACE,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4083), 1,
      anon_sym_CARET,
    ACTIONS(4085), 1,
      anon_sym_AMP,
    ACTIONS(4087), 1,
      anon_sym_PIPE,
    ACTIONS(4089), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4091), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4097), 1,
      anon_sym_EQ,
    ACTIONS(4103), 1,
      anon_sym_DOT_DOT,
    STATE(403), 1,
      sym_block,
    STATE(3355), 1,
      sym_label,
    ACTIONS(4079), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4093), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4101), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4105), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1581), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4081), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4099), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4095), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [36478] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1384), 1,
      sym_label,
    STATE(1582), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3647), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3645), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
  [36535] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1583), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3631), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3629), 25,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
  [36590] = 22,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1331), 1,
      anon_sym_extern,
    ACTIONS(3400), 1,
      anon_sym_COLON_COLON,
    ACTIONS(3402), 1,
      anon_sym_default,
    ACTIONS(3410), 1,
      sym_metavariable,
    ACTIONS(4117), 1,
      sym_identifier,
    ACTIONS(4119), 1,
      anon_sym_fn,
    STATE(2239), 1,
      aux_sym_function_modifiers_repeat1,
    STATE(2387), 1,
      sym_extern_modifier,
    STATE(2616), 1,
      sym_scoped_type_identifier,
    STATE(3369), 1,
      sym_generic_type_with_turbofish,
    STATE(3400), 1,
      sym_function_modifiers,
    STATE(3427), 1,
      sym_generic_type,
    STATE(3462), 1,
      sym_scoped_identifier,
    STATE(3499), 1,
      sym_bracketed_type,
    ACTIONS(3406), 2,
      anon_sym_gen,
      anon_sym_union,
    STATE(1584), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1317), 3,
      anon_sym_async,
      anon_sym_const,
      anon_sym_unsafe,
    ACTIONS(3408), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(3398), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [36679] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1585), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3333), 16,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3335), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
  [36734] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3669), 2,
      anon_sym_LBRACE,
      anon_sym_COLON_COLON,
    STATE(1586), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3671), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3667), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [36791] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1587), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3639), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3637), 25,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
  [36846] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1588), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3345), 16,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_BANG,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3347), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
  [36901] = 25,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(750), 1,
      anon_sym_RBRACK,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4121), 1,
      anon_sym_SEMI,
    ACTIONS(4123), 1,
      anon_sym_COMMA,
    STATE(3016), 1,
      aux_sym_arguments_repeat1,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1589), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [36996] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1590), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3605), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3603), 25,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
  [37051] = 22,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1331), 1,
      anon_sym_extern,
    ACTIONS(3400), 1,
      anon_sym_COLON_COLON,
    ACTIONS(3402), 1,
      anon_sym_default,
    ACTIONS(3410), 1,
      sym_metavariable,
    ACTIONS(4125), 1,
      sym_identifier,
    ACTIONS(4127), 1,
      anon_sym_fn,
    STATE(2239), 1,
      aux_sym_function_modifiers_repeat1,
    STATE(2387), 1,
      sym_extern_modifier,
    STATE(2574), 1,
      sym_scoped_type_identifier,
    STATE(3369), 1,
      sym_generic_type_with_turbofish,
    STATE(3427), 1,
      sym_generic_type,
    STATE(3462), 1,
      sym_scoped_identifier,
    STATE(3499), 1,
      sym_bracketed_type,
    STATE(3514), 1,
      sym_function_modifiers,
    ACTIONS(3406), 2,
      anon_sym_gen,
      anon_sym_union,
    STATE(1591), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1317), 3,
      anon_sym_async,
      anon_sym_const,
      anon_sym_unsafe,
    ACTIONS(3408), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(3398), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [37140] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3849), 1,
      anon_sym_COLON_COLON,
    STATE(1592), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3671), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3667), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
  [37197] = 14,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4083), 1,
      anon_sym_CARET,
    ACTIONS(4085), 1,
      anon_sym_AMP,
    ACTIONS(4079), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4093), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    STATE(1593), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4081), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3681), 5,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(3677), 21,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
  [37269] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1594), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1263), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1261), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
      anon_sym_else,
  [37323] = 19,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4083), 1,
      anon_sym_CARET,
    ACTIONS(4085), 1,
      anon_sym_AMP,
    ACTIONS(4087), 1,
      anon_sym_PIPE,
    ACTIONS(4089), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4091), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4003), 2,
      anon_sym_EQ,
      anon_sym_DOT_DOT,
    ACTIONS(4079), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4093), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4101), 2,
      anon_sym_GT,
      anon_sym_LT,
    STATE(1595), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4081), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4099), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4001), 15,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
  [37405] = 17,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4083), 1,
      anon_sym_CARET,
    ACTIONS(4085), 1,
      anon_sym_AMP,
    ACTIONS(4087), 1,
      anon_sym_PIPE,
    ACTIONS(3681), 2,
      anon_sym_EQ,
      anon_sym_DOT_DOT,
    ACTIONS(4079), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4093), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4101), 2,
      anon_sym_GT,
      anon_sym_LT,
    STATE(1596), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4081), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4099), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3677), 17,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
  [37483] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4129), 1,
      anon_sym_COLON_COLON,
    STATE(1597), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1357), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1359), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [37539] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1598), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3513), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3511), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
  [37593] = 24,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4131), 1,
      anon_sym_RPAREN,
    ACTIONS(4133), 1,
      anon_sym_COMMA,
    STATE(2917), 1,
      aux_sym_arguments_repeat1,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1599), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [37685] = 15,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4083), 1,
      anon_sym_CARET,
    ACTIONS(4085), 1,
      anon_sym_AMP,
    ACTIONS(4087), 1,
      anon_sym_PIPE,
    ACTIONS(4079), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4093), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    STATE(1600), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4081), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3681), 4,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(3677), 21,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
  [37759] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1601), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3343), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3341), 22,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
      anon_sym_LT2,
  [37813] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1602), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3583), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3581), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_DASH_GT,
      anon_sym_as,
  [37867] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4135), 1,
      anon_sym_DASH_GT,
    STATE(1603), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3587), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3585), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [37923] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4083), 1,
      anon_sym_CARET,
    ACTIONS(4085), 1,
      anon_sym_AMP,
    ACTIONS(4087), 1,
      anon_sym_PIPE,
    ACTIONS(4089), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4091), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4097), 1,
      anon_sym_EQ,
    ACTIONS(4137), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4079), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4093), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4101), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4139), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1604), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3939), 3,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_SQUOTE,
    ACTIONS(4081), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4099), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4095), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [38011] = 18,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4083), 1,
      anon_sym_CARET,
    ACTIONS(4085), 1,
      anon_sym_AMP,
    ACTIONS(4087), 1,
      anon_sym_PIPE,
    ACTIONS(4089), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3681), 2,
      anon_sym_EQ,
      anon_sym_DOT_DOT,
    ACTIONS(4079), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4093), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4101), 2,
      anon_sym_GT,
      anon_sym_LT,
    STATE(1605), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4081), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4099), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3677), 16,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
  [38091] = 19,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4083), 1,
      anon_sym_CARET,
    ACTIONS(4085), 1,
      anon_sym_AMP,
    ACTIONS(4087), 1,
      anon_sym_PIPE,
    ACTIONS(4089), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4091), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(389), 2,
      anon_sym_EQ,
      anon_sym_DOT_DOT,
    ACTIONS(4079), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4093), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4101), 2,
      anon_sym_GT,
      anon_sym_LT,
    STATE(1606), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4081), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4099), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(387), 15,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
  [38173] = 21,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(347), 1,
      anon_sym_EQ,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4083), 1,
      anon_sym_CARET,
    ACTIONS(4085), 1,
      anon_sym_AMP,
    ACTIONS(4087), 1,
      anon_sym_PIPE,
    ACTIONS(4089), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4091), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4137), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4079), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4093), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4101), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4139), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1607), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4081), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4099), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(341), 13,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_SQUOTE,
  [38259] = 21,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4015), 1,
      anon_sym_EQ,
    ACTIONS(4083), 1,
      anon_sym_CARET,
    ACTIONS(4085), 1,
      anon_sym_AMP,
    ACTIONS(4087), 1,
      anon_sym_PIPE,
    ACTIONS(4089), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4091), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4137), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4079), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4093), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4101), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4139), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1608), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4081), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4099), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4013), 13,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_SQUOTE,
  [38345] = 21,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4025), 1,
      anon_sym_EQ,
    ACTIONS(4083), 1,
      anon_sym_CARET,
    ACTIONS(4085), 1,
      anon_sym_AMP,
    ACTIONS(4087), 1,
      anon_sym_PIPE,
    ACTIONS(4089), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4091), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4137), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4079), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4093), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4101), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4139), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1609), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4081), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4099), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4023), 13,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_SQUOTE,
  [38431] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4141), 1,
      anon_sym_DASH_GT,
    STATE(1610), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3593), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3591), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [38487] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4143), 1,
      anon_sym_DASH_GT,
    STATE(1611), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3599), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3597), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [38543] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1612), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3517), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3515), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
  [38597] = 24,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4145), 1,
      anon_sym_RPAREN,
    ACTIONS(4147), 1,
      anon_sym_COMMA,
    STATE(2813), 1,
      aux_sym_arguments_repeat1,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1613), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [38689] = 24,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(935), 1,
      anon_sym_RBRACK,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4149), 1,
      anon_sym_COMMA,
    STATE(2821), 1,
      aux_sym_arguments_repeat1,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1614), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [38781] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1615), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3543), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3541), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
  [38835] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1616), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3609), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3607), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_DASH_GT,
      anon_sym_as,
  [38889] = 24,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(949), 1,
      anon_sym_RPAREN,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4151), 1,
      anon_sym_COMMA,
    STATE(2860), 1,
      aux_sym_arguments_repeat1,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1617), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [38981] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1618), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3555), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3553), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
  [39035] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4153), 1,
      anon_sym_COLON_COLON,
    STATE(1619), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3339), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3337), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [39091] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1620), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3559), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3557), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
  [39145] = 21,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4021), 1,
      anon_sym_EQ,
    ACTIONS(4083), 1,
      anon_sym_CARET,
    ACTIONS(4085), 1,
      anon_sym_AMP,
    ACTIONS(4087), 1,
      anon_sym_PIPE,
    ACTIONS(4089), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4091), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4137), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4079), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4093), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4101), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4139), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1621), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4081), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4099), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4019), 13,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_SQUOTE,
  [39231] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4155), 1,
      anon_sym_DASH_GT,
    STATE(1622), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3613), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3611), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [39287] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1623), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4157), 3,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_COMMA,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [39375] = 24,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(967), 1,
      anon_sym_RPAREN,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4159), 1,
      anon_sym_COMMA,
    STATE(2870), 1,
      aux_sym_arguments_repeat1,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1624), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [39467] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4161), 1,
      anon_sym_DASH_GT,
    STATE(1625), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3563), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3561), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [39523] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4163), 1,
      anon_sym_LPAREN,
    STATE(1698), 1,
      sym_arguments,
    STATE(1626), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3661), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3657), 22,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [39581] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4079), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1627), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4081), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3681), 9,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(3677), 21,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
  [39647] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4153), 1,
      anon_sym_COLON_COLON,
    STATE(1628), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3359), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3357), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [39703] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1629), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3623), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3621), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_DASH_GT,
      anon_sym_as,
  [39757] = 24,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(919), 1,
      anon_sym_RBRACK,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4165), 1,
      anon_sym_COMMA,
    STATE(2961), 1,
      aux_sym_arguments_repeat1,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1630), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [39849] = 21,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3995), 1,
      anon_sym_EQ,
    ACTIONS(4083), 1,
      anon_sym_CARET,
    ACTIONS(4085), 1,
      anon_sym_AMP,
    ACTIONS(4087), 1,
      anon_sym_PIPE,
    ACTIONS(4089), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4091), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4137), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4079), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4093), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4101), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4139), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1631), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4081), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4099), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3993), 13,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_SQUOTE,
  [39935] = 21,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3999), 1,
      anon_sym_EQ,
    ACTIONS(4083), 1,
      anon_sym_CARET,
    ACTIONS(4085), 1,
      anon_sym_AMP,
    ACTIONS(4087), 1,
      anon_sym_PIPE,
    ACTIONS(4089), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4091), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4137), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4079), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4093), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4101), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4139), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1632), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4081), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4099), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3997), 13,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_SQUOTE,
  [40021] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4167), 1,
      anon_sym_COLON_COLON,
    STATE(1633), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3323), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3321), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [40077] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1634), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3635), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3633), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_DASH_GT,
      anon_sym_as,
  [40131] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4153), 1,
      anon_sym_COLON_COLON,
    STATE(1635), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3327), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3325), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [40187] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1636), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3643), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3641), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_DASH_GT,
      anon_sym_as,
  [40241] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3873), 1,
      anon_sym_COLON_COLON,
    STATE(1637), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3285), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3283), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [40297] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1638), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4169), 3,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_COMMA,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [40385] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    STATE(1639), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4081), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3681), 11,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(3677), 21,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
  [40449] = 13,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4085), 1,
      anon_sym_AMP,
    ACTIONS(4079), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4093), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    STATE(1640), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4081), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3681), 6,
      anon_sym_CARET,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(3677), 21,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
  [40519] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4083), 1,
      anon_sym_CARET,
    ACTIONS(4085), 1,
      anon_sym_AMP,
    ACTIONS(4087), 1,
      anon_sym_PIPE,
    ACTIONS(4089), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4091), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4097), 1,
      anon_sym_EQ,
    ACTIONS(4137), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4079), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4093), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4101), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4139), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1641), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3821), 3,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_SQUOTE,
    ACTIONS(4081), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4099), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4095), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [40607] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4171), 1,
      anon_sym_DASH_GT,
    STATE(1642), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3569), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3567), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [40663] = 18,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4173), 1,
      sym_identifier,
    ACTIONS(4175), 1,
      anon_sym_LBRACE,
    ACTIONS(4177), 1,
      anon_sym_RBRACE,
    ACTIONS(4179), 1,
      anon_sym_STAR,
    ACTIONS(4183), 1,
      anon_sym_COMMA,
    ACTIONS(4185), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4189), 1,
      sym_metavariable,
    STATE(2477), 1,
      sym_scoped_identifier,
    STATE(3047), 1,
      sym__use_clause,
    STATE(3423), 1,
      sym_generic_type_with_turbofish,
    STATE(3517), 1,
      sym_bracketed_type,
    STATE(1643), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4187), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(2910), 4,
      sym_scoped_use_list,
      sym_use_list,
      sym_use_as_clause,
      sym_use_wildcard,
    ACTIONS(4181), 20,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
  [40743] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4191), 1,
      anon_sym_COLON_COLON,
    STATE(1644), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3323), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3321), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [40799] = 12,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4079), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4093), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    STATE(1645), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4081), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3681), 7,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(3677), 21,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
  [40867] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4193), 1,
      anon_sym_DASH_GT,
    STATE(1646), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3575), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3573), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [40923] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4195), 1,
      anon_sym_COLON_COLON,
    STATE(1647), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1357), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1359), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [40979] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1648), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1275), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1273), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
      anon_sym_else,
  [41033] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4153), 1,
      anon_sym_COLON_COLON,
    STATE(1649), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3323), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3321), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [41089] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1650), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1259), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1257), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
      anon_sym_else,
  [41143] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1651), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1271), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1269), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
      anon_sym_else,
  [41197] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1652), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1267), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1265), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
      anon_sym_else,
  [41251] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4083), 1,
      anon_sym_CARET,
    ACTIONS(4085), 1,
      anon_sym_AMP,
    ACTIONS(4087), 1,
      anon_sym_PIPE,
    ACTIONS(4089), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4091), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4097), 1,
      anon_sym_EQ,
    ACTIONS(4137), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4079), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4093), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4101), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4139), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1653), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3889), 3,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_SQUOTE,
    ACTIONS(4081), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4099), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4095), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [41339] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(289), 1,
      anon_sym_RBRACE,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4197), 1,
      anon_sym_SEMI,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1654), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [41428] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1655), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3747), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3745), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [41481] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4199), 1,
      anon_sym_RPAREN,
    ACTIONS(4201), 1,
      anon_sym_COMMA,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1656), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [41570] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4203), 1,
      anon_sym_LBRACK,
    ACTIONS(4205), 1,
      anon_sym_QMARK,
    ACTIONS(4207), 1,
      anon_sym_DOT,
    STATE(1657), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3755), 14,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(3753), 21,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [41629] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1215), 1,
      anon_sym_RPAREN,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4201), 1,
      anon_sym_COMMA,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1658), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [41718] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1659), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3891), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3889), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [41771] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1660), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1357), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1359), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [41824] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1661), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3831), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3829), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [41877] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(4209), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(1662), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [41964] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1663), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3899), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3897), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [42017] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1664), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1357), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1359), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [42070] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4211), 1,
      anon_sym_RPAREN,
    ACTIONS(4213), 1,
      anon_sym_COMMA,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1665), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [42159] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1666), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3661), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3657), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [42212] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1667), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3799), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3797), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [42265] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1668), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3907), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3905), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [42318] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1669), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3911), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3909), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [42371] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1670), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3915), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3913), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [42424] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1671), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1357), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1359), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [42477] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1672), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3927), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3925), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [42530] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1673), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3931), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3929), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [42583] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1674), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3935), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3933), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [42636] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1675), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3715), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3713), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [42689] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4203), 1,
      anon_sym_LBRACK,
    ACTIONS(4205), 1,
      anon_sym_QMARK,
    ACTIONS(4207), 1,
      anon_sym_DOT,
    STATE(1676), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3735), 14,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(3733), 21,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [42748] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4203), 1,
      anon_sym_LBRACK,
    ACTIONS(4205), 1,
      anon_sym_QMARK,
    ACTIONS(4207), 1,
      anon_sym_DOT,
    STATE(1677), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3791), 14,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(3789), 21,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [42807] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4215), 1,
      anon_sym_SEMI,
    ACTIONS(4217), 1,
      anon_sym_else,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1678), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [42896] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4219), 1,
      anon_sym_RBRACE,
    ACTIONS(4221), 1,
      anon_sym_COMMA,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1679), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [42985] = 19,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4203), 1,
      anon_sym_LBRACK,
    ACTIONS(4205), 1,
      anon_sym_QMARK,
    ACTIONS(4207), 1,
      anon_sym_DOT,
    ACTIONS(4227), 1,
      anon_sym_CARET,
    ACTIONS(4229), 1,
      anon_sym_AMP,
    ACTIONS(4231), 1,
      anon_sym_PIPE,
    ACTIONS(4233), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4235), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4243), 1,
      anon_sym_as,
    ACTIONS(389), 2,
      anon_sym_EQ,
      anon_sym_DOT_DOT,
    ACTIONS(4223), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4237), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4241), 2,
      anon_sym_GT,
      anon_sym_LT,
    STATE(1680), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4225), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4239), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(387), 14,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [43066] = 21,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(347), 1,
      anon_sym_EQ,
    ACTIONS(4203), 1,
      anon_sym_LBRACK,
    ACTIONS(4205), 1,
      anon_sym_QMARK,
    ACTIONS(4207), 1,
      anon_sym_DOT,
    ACTIONS(4227), 1,
      anon_sym_CARET,
    ACTIONS(4229), 1,
      anon_sym_AMP,
    ACTIONS(4231), 1,
      anon_sym_PIPE,
    ACTIONS(4233), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4235), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4243), 1,
      anon_sym_as,
    ACTIONS(4245), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4223), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4237), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4241), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4247), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1681), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4225), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4239), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(341), 12,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [43151] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1682), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3783), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3781), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [43204] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1683), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3711), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3709), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [43257] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1684), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3323), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3321), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [43310] = 21,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4015), 1,
      anon_sym_EQ,
    ACTIONS(4203), 1,
      anon_sym_LBRACK,
    ACTIONS(4205), 1,
      anon_sym_QMARK,
    ACTIONS(4207), 1,
      anon_sym_DOT,
    ACTIONS(4227), 1,
      anon_sym_CARET,
    ACTIONS(4229), 1,
      anon_sym_AMP,
    ACTIONS(4231), 1,
      anon_sym_PIPE,
    ACTIONS(4233), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4235), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4243), 1,
      anon_sym_as,
    ACTIONS(4245), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4223), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4237), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4241), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4247), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1685), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4225), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4239), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4013), 12,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [43395] = 21,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4025), 1,
      anon_sym_EQ,
    ACTIONS(4203), 1,
      anon_sym_LBRACK,
    ACTIONS(4205), 1,
      anon_sym_QMARK,
    ACTIONS(4207), 1,
      anon_sym_DOT,
    ACTIONS(4227), 1,
      anon_sym_CARET,
    ACTIONS(4229), 1,
      anon_sym_AMP,
    ACTIONS(4231), 1,
      anon_sym_PIPE,
    ACTIONS(4233), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4235), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4243), 1,
      anon_sym_as,
    ACTIONS(4245), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4223), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4237), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4241), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4247), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1686), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4225), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4239), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4023), 12,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [43480] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4083), 1,
      anon_sym_CARET,
    ACTIONS(4085), 1,
      anon_sym_AMP,
    ACTIONS(4087), 1,
      anon_sym_PIPE,
    ACTIONS(4091), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4097), 1,
      anon_sym_EQ,
    ACTIONS(4103), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4251), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4079), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4093), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4101), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4105), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(4249), 2,
      anon_sym_LBRACE,
      anon_sym_SQUOTE,
    STATE(1687), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4081), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4099), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4095), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [43567] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4257), 1,
      anon_sym_CARET,
    ACTIONS(4259), 1,
      anon_sym_AMP,
    ACTIONS(4261), 1,
      anon_sym_PIPE,
    ACTIONS(4263), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4265), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4271), 1,
      anon_sym_EQ,
    ACTIONS(4277), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3939), 2,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
    ACTIONS(4253), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4267), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4275), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4279), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1688), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4255), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4273), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4269), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [43654] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1689), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3895), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3893), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [43707] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4281), 1,
      anon_sym_SEMI,
    ACTIONS(4283), 1,
      anon_sym_else,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1690), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [43796] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1691), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3843), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3841), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [43849] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1692), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3811), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3809), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [43902] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1693), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3815), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3813), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [43955] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1694), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3255), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3257), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [44008] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(4285), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(1695), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [44095] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1696), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3903), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3901), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [44148] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1697), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3359), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3357), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [44201] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1698), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3919), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3917), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [44254] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1699), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3923), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3921), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [44307] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1700), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3327), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3325), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [44360] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1701), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3941), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3939), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [44413] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4257), 1,
      anon_sym_CARET,
    ACTIONS(4259), 1,
      anon_sym_AMP,
    ACTIONS(4261), 1,
      anon_sym_PIPE,
    ACTIONS(4263), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4265), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4271), 1,
      anon_sym_EQ,
    ACTIONS(4287), 1,
      anon_sym_LBRACE,
    ACTIONS(4289), 1,
      anon_sym_DOT_DOT,
    STATE(393), 1,
      sym_match_block,
    ACTIONS(4253), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4267), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4275), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4291), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1702), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4255), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4273), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4269), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [44502] = 21,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4021), 1,
      anon_sym_EQ,
    ACTIONS(4257), 1,
      anon_sym_CARET,
    ACTIONS(4259), 1,
      anon_sym_AMP,
    ACTIONS(4261), 1,
      anon_sym_PIPE,
    ACTIONS(4263), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4265), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4277), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4253), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4267), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4275), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4279), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1703), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4255), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4273), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4019), 12,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [44587] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4257), 1,
      anon_sym_CARET,
    ACTIONS(4259), 1,
      anon_sym_AMP,
    ACTIONS(4261), 1,
      anon_sym_PIPE,
    ACTIONS(4263), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4265), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4271), 1,
      anon_sym_EQ,
    ACTIONS(4277), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3821), 2,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
    ACTIONS(4253), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4267), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4275), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4279), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1704), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4255), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4273), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4269), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [44674] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(4293), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(1705), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [44761] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1706), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1357), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1359), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [44814] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1707), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3945), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3943), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [44867] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    STATE(1708), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4255), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3681), 11,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(3677), 20,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [44930] = 13,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4259), 1,
      anon_sym_AMP,
    ACTIONS(4253), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4267), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    STATE(1709), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4255), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3681), 6,
      anon_sym_CARET,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(3677), 20,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [44999] = 12,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4253), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4267), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    STATE(1710), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4255), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3681), 7,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(3677), 20,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [45066] = 14,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4257), 1,
      anon_sym_CARET,
    ACTIONS(4259), 1,
      anon_sym_AMP,
    ACTIONS(4253), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4267), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    STATE(1711), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4255), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3681), 5,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(3677), 20,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [45137] = 17,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4257), 1,
      anon_sym_CARET,
    ACTIONS(4259), 1,
      anon_sym_AMP,
    ACTIONS(4261), 1,
      anon_sym_PIPE,
    ACTIONS(3681), 2,
      anon_sym_EQ,
      anon_sym_DOT_DOT,
    ACTIONS(4253), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4267), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4275), 2,
      anon_sym_GT,
      anon_sym_LT,
    STATE(1712), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4255), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4273), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3677), 16,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [45214] = 18,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4257), 1,
      anon_sym_CARET,
    ACTIONS(4259), 1,
      anon_sym_AMP,
    ACTIONS(4261), 1,
      anon_sym_PIPE,
    ACTIONS(4263), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3681), 2,
      anon_sym_EQ,
      anon_sym_DOT_DOT,
    ACTIONS(4253), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4267), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4275), 2,
      anon_sym_GT,
      anon_sym_LT,
    STATE(1713), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4255), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4273), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3677), 15,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [45293] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4253), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1714), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4255), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3681), 9,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(3677), 20,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [45358] = 21,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3995), 1,
      anon_sym_EQ,
    ACTIONS(4257), 1,
      anon_sym_CARET,
    ACTIONS(4259), 1,
      anon_sym_AMP,
    ACTIONS(4261), 1,
      anon_sym_PIPE,
    ACTIONS(4263), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4265), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4277), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4253), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4267), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4275), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4279), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1715), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4255), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4273), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3993), 12,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [45443] = 21,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3999), 1,
      anon_sym_EQ,
    ACTIONS(4257), 1,
      anon_sym_CARET,
    ACTIONS(4259), 1,
      anon_sym_AMP,
    ACTIONS(4261), 1,
      anon_sym_PIPE,
    ACTIONS(4263), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4265), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4277), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4253), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4267), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4275), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4279), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1716), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4255), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4273), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3997), 12,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [45528] = 15,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4257), 1,
      anon_sym_CARET,
    ACTIONS(4259), 1,
      anon_sym_AMP,
    ACTIONS(4261), 1,
      anon_sym_PIPE,
    ACTIONS(4253), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4267), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    STATE(1717), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4255), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3681), 4,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(3677), 20,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [45601] = 19,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4257), 1,
      anon_sym_CARET,
    ACTIONS(4259), 1,
      anon_sym_AMP,
    ACTIONS(4261), 1,
      anon_sym_PIPE,
    ACTIONS(4263), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4265), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4003), 2,
      anon_sym_EQ,
      anon_sym_DOT_DOT,
    ACTIONS(4253), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4267), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4275), 2,
      anon_sym_GT,
      anon_sym_LT,
    STATE(1718), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4255), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4273), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4001), 14,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [45682] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1719), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3949), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3947), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [45735] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1720), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3691), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3689), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [45788] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1721), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3695), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3693), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [45841] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1722), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3723), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3721), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [45894] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1723), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3707), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3705), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [45947] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1724), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3707), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3705), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [46000] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4203), 1,
      anon_sym_LBRACK,
    ACTIONS(4205), 1,
      anon_sym_QMARK,
    ACTIONS(4207), 1,
      anon_sym_DOT,
    STATE(1725), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3795), 14,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(3793), 21,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [46059] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4257), 1,
      anon_sym_CARET,
    ACTIONS(4259), 1,
      anon_sym_AMP,
    ACTIONS(4261), 1,
      anon_sym_PIPE,
    ACTIONS(4263), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4265), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4271), 1,
      anon_sym_EQ,
    ACTIONS(4277), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3889), 2,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
    ACTIONS(4253), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4267), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4275), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4279), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1726), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4255), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4273), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4269), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [46146] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1727), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3763), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3761), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [46199] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1728), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3771), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3769), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [46252] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1729), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3339), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3337), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [46305] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1730), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3779), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3777), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [46358] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1731), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3787), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3785), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [46411] = 17,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4173), 1,
      sym_identifier,
    ACTIONS(4175), 1,
      anon_sym_LBRACE,
    ACTIONS(4179), 1,
      anon_sym_STAR,
    ACTIONS(4185), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4189), 1,
      sym_metavariable,
    ACTIONS(4295), 1,
      anon_sym_RBRACE,
    STATE(2477), 1,
      sym_scoped_identifier,
    STATE(3166), 1,
      sym__use_clause,
    STATE(3423), 1,
      sym_generic_type_with_turbofish,
    STATE(3517), 1,
      sym_bracketed_type,
    STATE(1732), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4187), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(2910), 4,
      sym_scoped_use_list,
      sym_use_list,
      sym_use_as_clause,
      sym_use_wildcard,
    ACTIONS(4181), 20,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
  [46488] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1733), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3739), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3737), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [46541] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1734), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3853), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3851), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [46594] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1735), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3857), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3855), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [46647] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1736), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3861), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3859), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [46700] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1737), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3865), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3863), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [46753] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1738), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3879), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3877), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [46806] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1739), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3727), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3725), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [46859] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(4297), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(1740), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [46946] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1741), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3751), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3749), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [46999] = 21,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4083), 1,
      anon_sym_CARET,
    ACTIONS(4085), 1,
      anon_sym_AMP,
    ACTIONS(4087), 1,
      anon_sym_PIPE,
    ACTIONS(4091), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4097), 1,
      anon_sym_EQ,
    ACTIONS(4103), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4079), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4093), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4101), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4105), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1742), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4081), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4299), 3,
      anon_sym_LBRACE,
      anon_sym_AMP_AMP,
      anon_sym_SQUOTE,
    ACTIONS(4099), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4095), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [47084] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1743), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3759), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3757), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [47137] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1744), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3775), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3773), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [47190] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1745), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1583), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1585), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [47243] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1746), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1397), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1395), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [47296] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1747), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1449), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1447), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [47349] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1748), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1453), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1451), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [47402] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1749), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1367), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1365), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [47455] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1750), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1405), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1403), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [47508] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3939), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1751), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [47595] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1752), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3819), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3817), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [47648] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1753), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1401), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1399), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [47701] = 21,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4021), 1,
      anon_sym_EQ,
    ACTIONS(4203), 1,
      anon_sym_LBRACK,
    ACTIONS(4205), 1,
      anon_sym_QMARK,
    ACTIONS(4207), 1,
      anon_sym_DOT,
    ACTIONS(4227), 1,
      anon_sym_CARET,
    ACTIONS(4229), 1,
      anon_sym_AMP,
    ACTIONS(4231), 1,
      anon_sym_PIPE,
    ACTIONS(4233), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4235), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4243), 1,
      anon_sym_as,
    ACTIONS(4245), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4223), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4237), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4241), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4247), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1754), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4225), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4239), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4019), 12,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [47786] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(4301), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(1755), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [47873] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1756), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1363), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1361), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [47926] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1757), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1439), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1437), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [47979] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3821), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1758), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [48066] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1759), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1409), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1407), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [48119] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4203), 1,
      anon_sym_LBRACK,
    ACTIONS(4205), 1,
      anon_sym_QMARK,
    ACTIONS(4207), 1,
      anon_sym_DOT,
    ACTIONS(4243), 1,
      anon_sym_as,
    STATE(1760), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4225), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3681), 11,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(3677), 20,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [48182] = 13,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4203), 1,
      anon_sym_LBRACK,
    ACTIONS(4205), 1,
      anon_sym_QMARK,
    ACTIONS(4207), 1,
      anon_sym_DOT,
    ACTIONS(4229), 1,
      anon_sym_AMP,
    ACTIONS(4243), 1,
      anon_sym_as,
    ACTIONS(4223), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4237), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    STATE(1761), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4225), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3681), 6,
      anon_sym_CARET,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(3677), 20,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [48251] = 12,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4203), 1,
      anon_sym_LBRACK,
    ACTIONS(4205), 1,
      anon_sym_QMARK,
    ACTIONS(4207), 1,
      anon_sym_DOT,
    ACTIONS(4243), 1,
      anon_sym_as,
    ACTIONS(4223), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4237), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    STATE(1762), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4225), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3681), 7,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(3677), 20,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [48318] = 14,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4203), 1,
      anon_sym_LBRACK,
    ACTIONS(4205), 1,
      anon_sym_QMARK,
    ACTIONS(4207), 1,
      anon_sym_DOT,
    ACTIONS(4227), 1,
      anon_sym_CARET,
    ACTIONS(4229), 1,
      anon_sym_AMP,
    ACTIONS(4243), 1,
      anon_sym_as,
    ACTIONS(4223), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4237), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    STATE(1763), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4225), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3681), 5,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(3677), 20,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [48389] = 17,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4203), 1,
      anon_sym_LBRACK,
    ACTIONS(4205), 1,
      anon_sym_QMARK,
    ACTIONS(4207), 1,
      anon_sym_DOT,
    ACTIONS(4227), 1,
      anon_sym_CARET,
    ACTIONS(4229), 1,
      anon_sym_AMP,
    ACTIONS(4231), 1,
      anon_sym_PIPE,
    ACTIONS(4243), 1,
      anon_sym_as,
    ACTIONS(3681), 2,
      anon_sym_EQ,
      anon_sym_DOT_DOT,
    ACTIONS(4223), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4237), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4241), 2,
      anon_sym_GT,
      anon_sym_LT,
    STATE(1764), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4225), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4239), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3677), 16,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [48466] = 18,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4203), 1,
      anon_sym_LBRACK,
    ACTIONS(4205), 1,
      anon_sym_QMARK,
    ACTIONS(4207), 1,
      anon_sym_DOT,
    ACTIONS(4227), 1,
      anon_sym_CARET,
    ACTIONS(4229), 1,
      anon_sym_AMP,
    ACTIONS(4231), 1,
      anon_sym_PIPE,
    ACTIONS(4233), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4243), 1,
      anon_sym_as,
    ACTIONS(3681), 2,
      anon_sym_EQ,
      anon_sym_DOT_DOT,
    ACTIONS(4223), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4237), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4241), 2,
      anon_sym_GT,
      anon_sym_LT,
    STATE(1765), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4225), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4239), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3677), 15,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [48545] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4203), 1,
      anon_sym_LBRACK,
    ACTIONS(4205), 1,
      anon_sym_QMARK,
    ACTIONS(4207), 1,
      anon_sym_DOT,
    ACTIONS(4243), 1,
      anon_sym_as,
    ACTIONS(4223), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(1766), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4225), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3681), 9,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(3677), 20,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [48610] = 21,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3995), 1,
      anon_sym_EQ,
    ACTIONS(4203), 1,
      anon_sym_LBRACK,
    ACTIONS(4205), 1,
      anon_sym_QMARK,
    ACTIONS(4207), 1,
      anon_sym_DOT,
    ACTIONS(4227), 1,
      anon_sym_CARET,
    ACTIONS(4229), 1,
      anon_sym_AMP,
    ACTIONS(4231), 1,
      anon_sym_PIPE,
    ACTIONS(4233), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4235), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4243), 1,
      anon_sym_as,
    ACTIONS(4245), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4223), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4237), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4241), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4247), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1767), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4225), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4239), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3993), 12,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [48695] = 21,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3999), 1,
      anon_sym_EQ,
    ACTIONS(4203), 1,
      anon_sym_LBRACK,
    ACTIONS(4205), 1,
      anon_sym_QMARK,
    ACTIONS(4207), 1,
      anon_sym_DOT,
    ACTIONS(4227), 1,
      anon_sym_CARET,
    ACTIONS(4229), 1,
      anon_sym_AMP,
    ACTIONS(4231), 1,
      anon_sym_PIPE,
    ACTIONS(4233), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4235), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4243), 1,
      anon_sym_as,
    ACTIONS(4245), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4223), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4237), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4241), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4247), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1768), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4225), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4239), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3997), 12,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [48780] = 15,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4203), 1,
      anon_sym_LBRACK,
    ACTIONS(4205), 1,
      anon_sym_QMARK,
    ACTIONS(4207), 1,
      anon_sym_DOT,
    ACTIONS(4227), 1,
      anon_sym_CARET,
    ACTIONS(4229), 1,
      anon_sym_AMP,
    ACTIONS(4231), 1,
      anon_sym_PIPE,
    ACTIONS(4243), 1,
      anon_sym_as,
    ACTIONS(4223), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4237), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    STATE(1769), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4225), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3681), 4,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(3677), 20,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [48853] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1770), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3743), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3741), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [48906] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1771), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3699), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3697), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [48959] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1772), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3703), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3701), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [49012] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1773), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1279), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1277), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [49065] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1774), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1431), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1429), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [49118] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3889), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1775), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [49205] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1776), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1461), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1459), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [49258] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1777), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3767), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3765), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [49311] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1778), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3869), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3867), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [49364] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1779), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1481), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1479), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [49417] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1780), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1485), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1483), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [49470] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1781), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1413), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1411), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [49523] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1782), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1469), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1467), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [49576] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1783), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1489), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1487), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [49629] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4303), 1,
      anon_sym_SEMI,
    ACTIONS(4305), 1,
      anon_sym_else,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1784), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [49718] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1785), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1435), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1433), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [49771] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1786), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(995), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(997), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [49824] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1787), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1007), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1009), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [49877] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1788), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1457), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1455), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [49930] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1789), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1011), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1013), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [49983] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4203), 1,
      anon_sym_LBRACK,
    ACTIONS(4205), 1,
      anon_sym_QMARK,
    ACTIONS(4207), 1,
      anon_sym_DOT,
    ACTIONS(4227), 1,
      anon_sym_CARET,
    ACTIONS(4229), 1,
      anon_sym_AMP,
    ACTIONS(4231), 1,
      anon_sym_PIPE,
    ACTIONS(4233), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4235), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4243), 1,
      anon_sym_as,
    ACTIONS(4245), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4309), 1,
      anon_sym_EQ,
    ACTIONS(3939), 2,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
    ACTIONS(4223), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4237), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4241), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4247), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1790), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4225), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4239), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4307), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [50070] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1791), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1023), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1025), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [50123] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1792), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1477), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1475), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [50176] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4203), 1,
      anon_sym_LBRACK,
    ACTIONS(4205), 1,
      anon_sym_QMARK,
    ACTIONS(4207), 1,
      anon_sym_DOT,
    ACTIONS(4227), 1,
      anon_sym_CARET,
    ACTIONS(4229), 1,
      anon_sym_AMP,
    ACTIONS(4231), 1,
      anon_sym_PIPE,
    ACTIONS(4233), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4235), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4243), 1,
      anon_sym_as,
    ACTIONS(4245), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4309), 1,
      anon_sym_EQ,
    ACTIONS(3821), 2,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
    ACTIONS(4223), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4237), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4241), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4247), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1793), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4225), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4239), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4307), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [50263] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1794), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1027), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1029), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [50316] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1795), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1417), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1415), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [50369] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4203), 1,
      anon_sym_LBRACK,
    ACTIONS(4205), 1,
      anon_sym_QMARK,
    ACTIONS(4207), 1,
      anon_sym_DOT,
    ACTIONS(4227), 1,
      anon_sym_CARET,
    ACTIONS(4229), 1,
      anon_sym_AMP,
    ACTIONS(4231), 1,
      anon_sym_PIPE,
    ACTIONS(4233), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4235), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4243), 1,
      anon_sym_as,
    ACTIONS(4245), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4309), 1,
      anon_sym_EQ,
    ACTIONS(3889), 2,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
    ACTIONS(4223), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4237), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4241), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4247), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1796), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4225), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4239), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4307), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [50456] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1797), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1045), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1047), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [50509] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1798), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1049), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1051), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [50562] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4311), 1,
      anon_sym_RPAREN,
    ACTIONS(4313), 1,
      anon_sym_COMMA,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1799), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [50651] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(4315), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(1800), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [50738] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(295), 1,
      anon_sym_RBRACE,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4197), 1,
      anon_sym_SEMI,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1801), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [50827] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(4317), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(1802), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [50914] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1803), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3823), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3821), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [50967] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4257), 1,
      anon_sym_CARET,
    ACTIONS(4259), 1,
      anon_sym_AMP,
    ACTIONS(4261), 1,
      anon_sym_PIPE,
    ACTIONS(4263), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4265), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4271), 1,
      anon_sym_EQ,
    ACTIONS(4289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4319), 1,
      anon_sym_LBRACE,
    STATE(1449), 1,
      sym_match_block,
    ACTIONS(4253), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4267), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4275), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4291), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1804), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4255), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4273), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4269), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [51056] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1805), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1357), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1359), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [51109] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(127), 1,
      anon_sym_RBRACE,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4197), 1,
      anon_sym_SEMI,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1806), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [51198] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(297), 1,
      anon_sym_RBRACE,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4197), 1,
      anon_sym_SEMI,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1807), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [51287] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4321), 1,
      anon_sym_SEMI,
    ACTIONS(4323), 1,
      anon_sym_else,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1808), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [51376] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1809), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3803), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3801), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [51429] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1083), 1,
      anon_sym_RPAREN,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4201), 1,
      anon_sym_COMMA,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1810), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [51518] = 17,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4173), 1,
      sym_identifier,
    ACTIONS(4175), 1,
      anon_sym_LBRACE,
    ACTIONS(4179), 1,
      anon_sym_STAR,
    ACTIONS(4185), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4189), 1,
      sym_metavariable,
    ACTIONS(4325), 1,
      anon_sym_RBRACE,
    STATE(2477), 1,
      sym_scoped_identifier,
    STATE(3166), 1,
      sym__use_clause,
    STATE(3423), 1,
      sym_generic_type_with_turbofish,
    STATE(3517), 1,
      sym_bracketed_type,
    STATE(1811), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4187), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(2910), 4,
      sym_scoped_use_list,
      sym_use_list,
      sym_use_as_clause,
      sym_use_wildcard,
    ACTIONS(4181), 20,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
  [51595] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4327), 1,
      anon_sym_SEMI,
    ACTIONS(4329), 1,
      anon_sym_else,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1812), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [51684] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4331), 1,
      anon_sym_RBRACE,
    ACTIONS(4333), 1,
      anon_sym_COMMA,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1813), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [51773] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(299), 1,
      anon_sym_RBRACE,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4197), 1,
      anon_sym_SEMI,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1814), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [51862] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1065), 1,
      anon_sym_RPAREN,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4201), 1,
      anon_sym_COMMA,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1815), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [51951] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(4335), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(1816), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [52038] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4337), 1,
      anon_sym_SEMI,
    ACTIONS(4339), 1,
      anon_sym_else,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1817), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [52127] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4197), 1,
      anon_sym_SEMI,
    ACTIONS(4341), 1,
      anon_sym_RBRACE,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1818), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [52216] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4201), 1,
      anon_sym_COMMA,
    ACTIONS(4343), 1,
      anon_sym_RPAREN,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1819), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [52305] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4345), 1,
      anon_sym_SEMI,
    ACTIONS(4347), 1,
      anon_sym_else,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1820), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [52394] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1821), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3847), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3845), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [52447] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4349), 1,
      anon_sym_SEMI,
    ACTIONS(4351), 1,
      anon_sym_else,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1822), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [52536] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(303), 1,
      anon_sym_RBRACE,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4197), 1,
      anon_sym_SEMI,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1823), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [52625] = 19,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4257), 1,
      anon_sym_CARET,
    ACTIONS(4259), 1,
      anon_sym_AMP,
    ACTIONS(4261), 1,
      anon_sym_PIPE,
    ACTIONS(4263), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4265), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(389), 2,
      anon_sym_EQ,
      anon_sym_DOT_DOT,
    ACTIONS(4253), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4267), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4275), 2,
      anon_sym_GT,
      anon_sym_LT,
    STATE(1824), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4255), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4273), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(387), 14,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [52706] = 21,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(347), 1,
      anon_sym_EQ,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4257), 1,
      anon_sym_CARET,
    ACTIONS(4259), 1,
      anon_sym_AMP,
    ACTIONS(4261), 1,
      anon_sym_PIPE,
    ACTIONS(4263), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4265), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4277), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4253), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4267), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4275), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4279), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1825), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4255), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4273), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(341), 12,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [52791] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4257), 1,
      anon_sym_CARET,
    ACTIONS(4259), 1,
      anon_sym_AMP,
    ACTIONS(4261), 1,
      anon_sym_PIPE,
    ACTIONS(4263), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4265), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4271), 1,
      anon_sym_EQ,
    ACTIONS(4289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4353), 1,
      anon_sym_LBRACE,
    STATE(480), 1,
      sym_match_block,
    ACTIONS(4253), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4267), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4275), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4291), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1826), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4255), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4273), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4269), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [52880] = 21,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4015), 1,
      anon_sym_EQ,
    ACTIONS(4257), 1,
      anon_sym_CARET,
    ACTIONS(4259), 1,
      anon_sym_AMP,
    ACTIONS(4261), 1,
      anon_sym_PIPE,
    ACTIONS(4263), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4265), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4277), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4253), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4267), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4275), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4279), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1827), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4255), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4273), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4013), 12,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [52965] = 21,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4025), 1,
      anon_sym_EQ,
    ACTIONS(4257), 1,
      anon_sym_CARET,
    ACTIONS(4259), 1,
      anon_sym_AMP,
    ACTIONS(4261), 1,
      anon_sym_PIPE,
    ACTIONS(4263), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4265), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4277), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4253), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4267), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4275), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4279), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1828), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4255), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4273), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4023), 12,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [53050] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(305), 1,
      anon_sym_RBRACE,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4197), 1,
      anon_sym_SEMI,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1829), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [53139] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1830), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3807), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3805), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [53192] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(291), 1,
      anon_sym_RBRACE,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4197), 1,
      anon_sym_SEMI,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1831), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [53281] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4197), 1,
      anon_sym_SEMI,
    ACTIONS(4355), 1,
      anon_sym_RBRACE,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1832), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [53370] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(307), 1,
      anon_sym_RBRACE,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4197), 1,
      anon_sym_SEMI,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1833), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [53459] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4197), 1,
      anon_sym_SEMI,
    ACTIONS(4357), 1,
      anon_sym_RBRACE,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1834), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [53548] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1835), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3675), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3673), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [53601] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4203), 1,
      anon_sym_LBRACK,
    ACTIONS(4205), 1,
      anon_sym_QMARK,
    ACTIONS(4207), 1,
      anon_sym_DOT,
    ACTIONS(4243), 1,
      anon_sym_as,
    STATE(1836), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3681), 14,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(3677), 20,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [53662] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(311), 1,
      anon_sym_RBRACE,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4197), 1,
      anon_sym_SEMI,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1837), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [53751] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4257), 1,
      anon_sym_CARET,
    ACTIONS(4259), 1,
      anon_sym_AMP,
    ACTIONS(4261), 1,
      anon_sym_PIPE,
    ACTIONS(4263), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4265), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4271), 1,
      anon_sym_EQ,
    ACTIONS(4289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4359), 1,
      anon_sym_LBRACE,
    STATE(1757), 1,
      sym_match_block,
    ACTIONS(4253), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4267), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4275), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4291), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1838), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4255), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4273), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4269), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [53840] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(313), 1,
      anon_sym_RBRACE,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4197), 1,
      anon_sym_SEMI,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1839), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [53929] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(315), 1,
      anon_sym_RBRACE,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4197), 1,
      anon_sym_SEMI,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1840), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [54018] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4197), 1,
      anon_sym_SEMI,
    ACTIONS(4361), 1,
      anon_sym_RBRACE,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1841), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [54107] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(319), 1,
      anon_sym_RBRACE,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4197), 1,
      anon_sym_SEMI,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1842), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [54196] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(321), 1,
      anon_sym_RBRACE,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4197), 1,
      anon_sym_SEMI,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1843), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [54285] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1225), 1,
      anon_sym_RPAREN,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4201), 1,
      anon_sym_COMMA,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1844), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [54374] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(323), 1,
      anon_sym_RBRACE,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4197), 1,
      anon_sym_SEMI,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1845), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [54463] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4197), 1,
      anon_sym_SEMI,
    ACTIONS(4363), 1,
      anon_sym_RBRACE,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1846), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [54552] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(327), 1,
      anon_sym_RBRACE,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4197), 1,
      anon_sym_SEMI,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1847), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [54641] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(329), 1,
      anon_sym_RBRACE,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4197), 1,
      anon_sym_SEMI,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1848), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [54730] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(331), 1,
      anon_sym_RBRACE,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4197), 1,
      anon_sym_SEMI,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1849), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [54819] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4197), 1,
      anon_sym_SEMI,
    ACTIONS(4365), 1,
      anon_sym_RBRACE,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1850), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [54908] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(335), 1,
      anon_sym_RBRACE,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4197), 1,
      anon_sym_SEMI,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1851), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [54997] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(337), 1,
      anon_sym_RBRACE,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4197), 1,
      anon_sym_SEMI,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1852), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [55086] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(123), 1,
      anon_sym_RBRACE,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4197), 1,
      anon_sym_SEMI,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1853), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [55175] = 23,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4197), 1,
      anon_sym_SEMI,
    ACTIONS(4367), 1,
      anon_sym_RBRACE,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1854), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [55264] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(4369), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(1855), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [55351] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1856), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3719), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3717), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [55404] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1857), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3827), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3825), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [55457] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1858), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3835), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3833), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [55510] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1859), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3839), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3837), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [55563] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1860), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3731), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3729), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [55616] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1861), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3671), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3667), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [55669] = 21,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(4083), 1,
      anon_sym_CARET,
    ACTIONS(4085), 1,
      anon_sym_AMP,
    ACTIONS(4087), 1,
      anon_sym_PIPE,
    ACTIONS(4091), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4097), 1,
      anon_sym_EQ,
    ACTIONS(4103), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4079), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4093), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4101), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4105), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1862), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4081), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4371), 3,
      anon_sym_LBRACE,
      anon_sym_AMP_AMP,
      anon_sym_SQUOTE,
    ACTIONS(4099), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4095), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [55754] = 19,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4203), 1,
      anon_sym_LBRACK,
    ACTIONS(4205), 1,
      anon_sym_QMARK,
    ACTIONS(4207), 1,
      anon_sym_DOT,
    ACTIONS(4227), 1,
      anon_sym_CARET,
    ACTIONS(4229), 1,
      anon_sym_AMP,
    ACTIONS(4231), 1,
      anon_sym_PIPE,
    ACTIONS(4233), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4235), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4243), 1,
      anon_sym_as,
    ACTIONS(4003), 2,
      anon_sym_EQ,
      anon_sym_DOT_DOT,
    ACTIONS(4223), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4237), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4241), 2,
      anon_sym_GT,
      anon_sym_LT,
    STATE(1863), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4225), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4239), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4001), 14,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [55835] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4201), 1,
      anon_sym_COMMA,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1864), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [55921] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4373), 1,
      anon_sym_SEMI,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1865), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [56007] = 21,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4203), 1,
      anon_sym_LBRACK,
    ACTIONS(4205), 1,
      anon_sym_QMARK,
    ACTIONS(4207), 1,
      anon_sym_DOT,
    ACTIONS(4227), 1,
      anon_sym_CARET,
    ACTIONS(4229), 1,
      anon_sym_AMP,
    ACTIONS(4231), 1,
      anon_sym_PIPE,
    ACTIONS(4235), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4243), 1,
      anon_sym_as,
    ACTIONS(4309), 1,
      anon_sym_EQ,
    ACTIONS(4375), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4223), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4237), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4241), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4371), 2,
      anon_sym_EQ_GT,
      anon_sym_AMP_AMP,
    ACTIONS(4377), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1866), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4225), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4239), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4307), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [56091] = 21,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4203), 1,
      anon_sym_LBRACK,
    ACTIONS(4205), 1,
      anon_sym_QMARK,
    ACTIONS(4207), 1,
      anon_sym_DOT,
    ACTIONS(4227), 1,
      anon_sym_CARET,
    ACTIONS(4229), 1,
      anon_sym_AMP,
    ACTIONS(4231), 1,
      anon_sym_PIPE,
    ACTIONS(4235), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4243), 1,
      anon_sym_as,
    ACTIONS(4309), 1,
      anon_sym_EQ,
    ACTIONS(4375), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4223), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4237), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4241), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4299), 2,
      anon_sym_EQ_GT,
      anon_sym_AMP_AMP,
    ACTIONS(4377), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1867), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4225), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4239), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4307), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [56175] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4203), 1,
      anon_sym_LBRACK,
    ACTIONS(4205), 1,
      anon_sym_QMARK,
    ACTIONS(4207), 1,
      anon_sym_DOT,
    ACTIONS(4227), 1,
      anon_sym_CARET,
    ACTIONS(4229), 1,
      anon_sym_AMP,
    ACTIONS(4231), 1,
      anon_sym_PIPE,
    ACTIONS(4235), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4243), 1,
      anon_sym_as,
    ACTIONS(4249), 1,
      anon_sym_EQ_GT,
    ACTIONS(4309), 1,
      anon_sym_EQ,
    ACTIONS(4375), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4379), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4223), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(4237), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4241), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4377), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1868), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4225), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4239), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(4307), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [56261] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4381), 1,
      anon_sym_COMMA,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1869), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [56347] = 16,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4173), 1,
      sym_identifier,
    ACTIONS(4175), 1,
      anon_sym_LBRACE,
    ACTIONS(4179), 1,
      anon_sym_STAR,
    ACTIONS(4185), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4189), 1,
      sym_metavariable,
    STATE(2477), 1,
      sym_scoped_identifier,
    STATE(3166), 1,
      sym__use_clause,
    STATE(3423), 1,
      sym_generic_type_with_turbofish,
    STATE(3517), 1,
      sym_bracketed_type,
    STATE(1870), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4187), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(2910), 4,
      sym_scoped_use_list,
      sym_use_list,
      sym_use_as_clause,
      sym_use_wildcard,
    ACTIONS(4181), 20,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
  [56421] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4383), 1,
      anon_sym_SEMI,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1871), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [56507] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4385), 1,
      anon_sym_RBRACK,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1872), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [56593] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4387), 1,
      anon_sym_SEMI,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1873), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [56679] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4389), 1,
      anon_sym_RBRACK,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1874), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [56765] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4391), 1,
      anon_sym_RBRACK,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1875), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [56851] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4393), 1,
      anon_sym_RBRACK,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1876), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [56937] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4395), 1,
      anon_sym_RBRACK,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1877), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [57023] = 16,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4173), 1,
      sym_identifier,
    ACTIONS(4175), 1,
      anon_sym_LBRACE,
    ACTIONS(4179), 1,
      anon_sym_STAR,
    ACTIONS(4185), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4189), 1,
      sym_metavariable,
    STATE(2477), 1,
      sym_scoped_identifier,
    STATE(3347), 1,
      sym__use_clause,
    STATE(3423), 1,
      sym_generic_type_with_turbofish,
    STATE(3517), 1,
      sym_bracketed_type,
    STATE(1878), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4187), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(2910), 4,
      sym_scoped_use_list,
      sym_use_list,
      sym_use_as_clause,
      sym_use_wildcard,
    ACTIONS(4181), 20,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
  [57097] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4397), 1,
      anon_sym_SEMI,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1879), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [57183] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4399), 1,
      anon_sym_COMMA,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1880), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [57269] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4401), 1,
      anon_sym_SEMI,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1881), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [57355] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4403), 1,
      anon_sym_SEMI,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1882), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [57441] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4405), 1,
      anon_sym_SEMI,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1883), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [57527] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4407), 1,
      anon_sym_SEMI,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1884), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [57613] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4409), 1,
      anon_sym_SEMI,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1885), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [57699] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4197), 1,
      anon_sym_SEMI,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1886), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [57785] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4411), 1,
      anon_sym_SEMI,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1887), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [57871] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4413), 1,
      anon_sym_RBRACK,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1888), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [57957] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4415), 1,
      anon_sym_SEMI,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1889), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [58043] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4417), 1,
      anon_sym_RBRACK,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1890), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [58129] = 16,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4173), 1,
      sym_identifier,
    ACTIONS(4175), 1,
      anon_sym_LBRACE,
    ACTIONS(4179), 1,
      anon_sym_STAR,
    ACTIONS(4185), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4189), 1,
      sym_metavariable,
    STATE(2477), 1,
      sym_scoped_identifier,
    STATE(3422), 1,
      sym__use_clause,
    STATE(3423), 1,
      sym_generic_type_with_turbofish,
    STATE(3517), 1,
      sym_bracketed_type,
    STATE(1891), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4187), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(2910), 4,
      sym_scoped_use_list,
      sym_use_list,
      sym_use_as_clause,
      sym_use_wildcard,
    ACTIONS(4181), 20,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
  [58203] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4419), 1,
      anon_sym_RBRACK,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1892), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [58289] = 16,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4173), 1,
      sym_identifier,
    ACTIONS(4175), 1,
      anon_sym_LBRACE,
    ACTIONS(4179), 1,
      anon_sym_STAR,
    ACTIONS(4185), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4189), 1,
      sym_metavariable,
    STATE(2477), 1,
      sym_scoped_identifier,
    STATE(3423), 1,
      sym_generic_type_with_turbofish,
    STATE(3510), 1,
      sym__use_clause,
    STATE(3517), 1,
      sym_bracketed_type,
    STATE(1893), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4187), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(2910), 4,
      sym_scoped_use_list,
      sym_use_list,
      sym_use_as_clause,
      sym_use_wildcard,
    ACTIONS(4181), 20,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
  [58363] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4421), 1,
      anon_sym_COMMA,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1894), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [58449] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4423), 1,
      anon_sym_RBRACK,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1895), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [58535] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4425), 1,
      anon_sym_RBRACK,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1896), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [58621] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4427), 1,
      anon_sym_RBRACK,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1897), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [58707] = 16,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4173), 1,
      sym_identifier,
    ACTIONS(4175), 1,
      anon_sym_LBRACE,
    ACTIONS(4179), 1,
      anon_sym_STAR,
    ACTIONS(4185), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4189), 1,
      sym_metavariable,
    STATE(2477), 1,
      sym_scoped_identifier,
    STATE(3423), 1,
      sym_generic_type_with_turbofish,
    STATE(3441), 1,
      sym__use_clause,
    STATE(3517), 1,
      sym_bracketed_type,
    STATE(1898), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4187), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(2910), 4,
      sym_scoped_use_list,
      sym_use_list,
      sym_use_as_clause,
      sym_use_wildcard,
    ACTIONS(4181), 20,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
  [58781] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4429), 1,
      anon_sym_COMMA,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1899), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [58867] = 22,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3679), 1,
      anon_sym_LBRACK,
    ACTIONS(3683), 1,
      anon_sym_QMARK,
    ACTIONS(3685), 1,
      anon_sym_DOT,
    ACTIONS(3687), 1,
      anon_sym_as,
    ACTIONS(3969), 1,
      anon_sym_CARET,
    ACTIONS(3971), 1,
      anon_sym_AMP,
    ACTIONS(3973), 1,
      anon_sym_PIPE,
    ACTIONS(3975), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3977), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3983), 1,
      anon_sym_EQ,
    ACTIONS(4065), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4431), 1,
      anon_sym_SEMI,
    ACTIONS(3965), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(3979), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3987), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(4067), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1900), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3967), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3985), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3981), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [58953] = 17,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3783), 1,
      anon_sym_where,
    ACTIONS(4433), 1,
      sym_identifier,
    ACTIONS(4437), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4443), 1,
      sym_metavariable,
    STATE(2759), 1,
      sym_scoped_type_identifier,
    STATE(3110), 1,
      sym_generic_type,
    STATE(3399), 1,
      sym_scoped_identifier,
    STATE(3505), 1,
      sym_generic_type_with_turbofish,
    STATE(3532), 1,
      sym_bracketed_type,
    STATE(1901), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3781), 3,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
    ACTIONS(4439), 3,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
    ACTIONS(4441), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(4435), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [59028] = 17,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3783), 1,
      anon_sym_where,
    ACTIONS(4437), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4443), 1,
      sym_metavariable,
    ACTIONS(4445), 1,
      sym_identifier,
    STATE(2841), 1,
      sym_scoped_type_identifier,
    STATE(3126), 1,
      sym_generic_type,
    STATE(3399), 1,
      sym_scoped_identifier,
    STATE(3505), 1,
      sym_generic_type_with_turbofish,
    STATE(3532), 1,
      sym_bracketed_type,
    STATE(1902), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3781), 3,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
    ACTIONS(4439), 3,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
    ACTIONS(4441), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(4435), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [59103] = 17,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3783), 1,
      anon_sym_where,
    ACTIONS(4437), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4443), 1,
      sym_metavariable,
    ACTIONS(4447), 1,
      sym_identifier,
    STATE(2937), 1,
      sym_scoped_type_identifier,
    STATE(3107), 1,
      sym_generic_type,
    STATE(3399), 1,
      sym_scoped_identifier,
    STATE(3505), 1,
      sym_generic_type_with_turbofish,
    STATE(3532), 1,
      sym_bracketed_type,
    STATE(1903), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3781), 3,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
    ACTIONS(4439), 3,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
    ACTIONS(4441), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(4435), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [59178] = 17,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3783), 1,
      anon_sym_where,
    ACTIONS(4437), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4443), 1,
      sym_metavariable,
    ACTIONS(4449), 1,
      sym_identifier,
    STATE(3052), 1,
      sym_scoped_type_identifier,
    STATE(3328), 1,
      sym_generic_type,
    STATE(3399), 1,
      sym_scoped_identifier,
    STATE(3505), 1,
      sym_generic_type_with_turbofish,
    STATE(3532), 1,
      sym_bracketed_type,
    STATE(1904), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3781), 3,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
    ACTIONS(4439), 3,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
    ACTIONS(4441), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(4435), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [59253] = 17,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3783), 1,
      anon_sym_where,
    ACTIONS(4437), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4443), 1,
      sym_metavariable,
    ACTIONS(4451), 1,
      sym_identifier,
    STATE(3050), 1,
      sym_scoped_type_identifier,
    STATE(3321), 1,
      sym_generic_type,
    STATE(3399), 1,
      sym_scoped_identifier,
    STATE(3505), 1,
      sym_generic_type_with_turbofish,
    STATE(3532), 1,
      sym_bracketed_type,
    STATE(1905), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3781), 3,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
    ACTIONS(4439), 3,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
    ACTIONS(4441), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(4435), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [59328] = 17,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3783), 1,
      anon_sym_where,
    ACTIONS(4437), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4443), 1,
      sym_metavariable,
    ACTIONS(4453), 1,
      sym_identifier,
    STATE(2764), 1,
      sym_scoped_type_identifier,
    STATE(3263), 1,
      sym_generic_type,
    STATE(3399), 1,
      sym_scoped_identifier,
    STATE(3505), 1,
      sym_generic_type_with_turbofish,
    STATE(3532), 1,
      sym_bracketed_type,
    STATE(1906), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3781), 3,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
    ACTIONS(4439), 3,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
    ACTIONS(4441), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(4435), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [59403] = 17,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3783), 1,
      anon_sym_where,
    ACTIONS(4437), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4443), 1,
      sym_metavariable,
    ACTIONS(4455), 1,
      sym_identifier,
    STATE(3042), 1,
      sym_scoped_type_identifier,
    STATE(3316), 1,
      sym_generic_type,
    STATE(3399), 1,
      sym_scoped_identifier,
    STATE(3505), 1,
      sym_generic_type_with_turbofish,
    STATE(3532), 1,
      sym_bracketed_type,
    STATE(1907), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3781), 3,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
    ACTIONS(4439), 3,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
    ACTIONS(4441), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(4435), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [59478] = 17,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3783), 1,
      anon_sym_where,
    ACTIONS(4437), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4443), 1,
      sym_metavariable,
    ACTIONS(4457), 1,
      sym_identifier,
    STATE(3048), 1,
      sym_scoped_type_identifier,
    STATE(3319), 1,
      sym_generic_type,
    STATE(3399), 1,
      sym_scoped_identifier,
    STATE(3505), 1,
      sym_generic_type_with_turbofish,
    STATE(3532), 1,
      sym_bracketed_type,
    STATE(1908), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3781), 3,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
    ACTIONS(4439), 3,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
    ACTIONS(4441), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(4435), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [59553] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1909), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4461), 3,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      sym_metavariable,
    ACTIONS(4459), 29,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [59600] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1910), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4465), 3,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      sym_metavariable,
    ACTIONS(4463), 29,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [59647] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1911), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4469), 3,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      sym_metavariable,
    ACTIONS(4467), 29,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_extern,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [59694] = 13,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2471), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4471), 1,
      sym_identifier,
    ACTIONS(4477), 1,
      sym_metavariable,
    STATE(2268), 1,
      sym_scoped_identifier,
    STATE(3353), 1,
      sym_bracketed_type,
    STATE(3379), 1,
      sym_generic_type_with_turbofish,
    STATE(3393), 1,
      sym_attribute,
    STATE(1912), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4475), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(4473), 20,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
  [59756] = 13,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2471), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4471), 1,
      sym_identifier,
    ACTIONS(4477), 1,
      sym_metavariable,
    STATE(2268), 1,
      sym_scoped_identifier,
    STATE(3353), 1,
      sym_bracketed_type,
    STATE(3379), 1,
      sym_generic_type_with_turbofish,
    STATE(3541), 1,
      sym_attribute,
    STATE(1913), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4475), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(4473), 20,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
  [59818] = 13,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2471), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4471), 1,
      sym_identifier,
    ACTIONS(4477), 1,
      sym_metavariable,
    STATE(2268), 1,
      sym_scoped_identifier,
    STATE(3353), 1,
      sym_bracketed_type,
    STATE(3379), 1,
      sym_generic_type_with_turbofish,
    STATE(3425), 1,
      sym_attribute,
    STATE(1914), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4475), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(4473), 20,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
  [59880] = 13,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2471), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4471), 1,
      sym_identifier,
    ACTIONS(4477), 1,
      sym_metavariable,
    STATE(2268), 1,
      sym_scoped_identifier,
    STATE(3353), 1,
      sym_bracketed_type,
    STATE(3360), 1,
      sym_attribute,
    STATE(3379), 1,
      sym_generic_type_with_turbofish,
    STATE(1915), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4475), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(4473), 20,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
  [59942] = 13,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2471), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4471), 1,
      sym_identifier,
    ACTIONS(4477), 1,
      sym_metavariable,
    STATE(2268), 1,
      sym_scoped_identifier,
    STATE(3353), 1,
      sym_bracketed_type,
    STATE(3379), 1,
      sym_generic_type_with_turbofish,
    STATE(3449), 1,
      sym_attribute,
    STATE(1916), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4475), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(4473), 20,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
  [60004] = 13,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2471), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4471), 1,
      sym_identifier,
    ACTIONS(4477), 1,
      sym_metavariable,
    STATE(2268), 1,
      sym_scoped_identifier,
    STATE(3353), 1,
      sym_bracketed_type,
    STATE(3370), 1,
      sym_attribute,
    STATE(3379), 1,
      sym_generic_type_with_turbofish,
    STATE(1917), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4475), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(4473), 20,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
  [60066] = 13,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2471), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4471), 1,
      sym_identifier,
    ACTIONS(4477), 1,
      sym_metavariable,
    STATE(2268), 1,
      sym_scoped_identifier,
    STATE(3353), 1,
      sym_bracketed_type,
    STATE(3379), 1,
      sym_generic_type_with_turbofish,
    STATE(3448), 1,
      sym_attribute,
    STATE(1918), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4475), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(4473), 20,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
  [60128] = 13,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2471), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4471), 1,
      sym_identifier,
    ACTIONS(4477), 1,
      sym_metavariable,
    STATE(2268), 1,
      sym_scoped_identifier,
    STATE(3353), 1,
      sym_bracketed_type,
    STATE(3379), 1,
      sym_generic_type_with_turbofish,
    STATE(3453), 1,
      sym_attribute,
    STATE(1919), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4475), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(4473), 20,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
  [60190] = 12,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4479), 1,
      sym_identifier,
    ACTIONS(4483), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4487), 1,
      sym_metavariable,
    STATE(3099), 1,
      sym_scoped_identifier,
    STATE(3423), 1,
      sym_generic_type_with_turbofish,
    STATE(3517), 1,
      sym_bracketed_type,
    STATE(1920), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4485), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(4481), 20,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
  [60249] = 12,
    ACTIONS(29), 1,
      anon_sym_LT,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4483), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4489), 1,
      sym_identifier,
    ACTIONS(4495), 1,
      sym_metavariable,
    STATE(3202), 1,
      sym_scoped_identifier,
    STATE(3423), 1,
      sym_generic_type_with_turbofish,
    STATE(3517), 1,
      sym_bracketed_type,
    STATE(1921), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4493), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(4491), 20,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
  [60308] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3299), 1,
      anon_sym_COLON,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    ACTIONS(4499), 1,
      anon_sym_BANG,
    ACTIONS(4501), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    STATE(1953), 1,
      sym_type_arguments,
    STATE(1982), 1,
      sym_parameters,
    STATE(1922), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3295), 14,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [60356] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1011), 1,
      anon_sym_DOT_DOT,
    STATE(1923), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1013), 20,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_else,
      anon_sym_in,
  [60392] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1027), 1,
      anon_sym_DOT_DOT,
    STATE(1924), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1029), 20,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_else,
      anon_sym_in,
  [60428] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3370), 1,
      anon_sym_COLON,
    ACTIONS(3374), 2,
      anon_sym_BANG,
      anon_sym_COLON_COLON,
    STATE(1925), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3368), 17,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_for,
      anon_sym_where,
      anon_sym_else,
      anon_sym_LT2,
  [60465] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3319), 1,
      anon_sym_COLON,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4505), 1,
      anon_sym_COLON_COLON,
    STATE(1953), 1,
      sym_type_arguments,
    STATE(1982), 1,
      sym_parameters,
    STATE(1926), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3317), 14,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [60510] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3309), 1,
      anon_sym_COLON,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4505), 1,
      anon_sym_COLON_COLON,
    STATE(1953), 1,
      sym_type_arguments,
    STATE(1982), 1,
      sym_parameters,
    STATE(1927), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3307), 14,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [60555] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3331), 1,
      anon_sym_COLON,
    ACTIONS(3335), 2,
      anon_sym_BANG,
      anon_sym_COLON_COLON,
    STATE(1928), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3329), 17,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_for,
      anon_sym_where,
      anon_sym_else,
      anon_sym_LT2,
  [60592] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3315), 1,
      anon_sym_COLON,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4505), 1,
      anon_sym_COLON_COLON,
    STATE(1953), 1,
      sym_type_arguments,
    STATE(1982), 1,
      sym_parameters,
    STATE(1929), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3313), 14,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [60637] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3343), 1,
      anon_sym_COLON,
    ACTIONS(3347), 2,
      anon_sym_BANG,
      anon_sym_COLON_COLON,
    STATE(1930), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3341), 17,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_for,
      anon_sym_where,
      anon_sym_else,
      anon_sym_LT2,
  [60674] = 14,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4499), 1,
      anon_sym_BANG,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4509), 1,
      anon_sym_LPAREN,
    ACTIONS(4511), 1,
      anon_sym_LBRACE,
    ACTIONS(4513), 1,
      anon_sym_COLON,
    ACTIONS(4515), 1,
      anon_sym_AT,
    ACTIONS(4517), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4521), 1,
      anon_sym_COLON_COLON,
    STATE(1953), 1,
      sym_type_arguments,
    ACTIONS(4519), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1931), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4507), 9,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [60727] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3351), 1,
      anon_sym_COLON,
    ACTIONS(3355), 2,
      anon_sym_BANG,
      anon_sym_COLON_COLON,
    STATE(1932), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3349), 17,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_for,
      anon_sym_where,
      anon_sym_else,
      anon_sym_LT2,
  [60764] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    STATE(1955), 1,
      sym_type_arguments,
    STATE(1975), 1,
      sym_parameters,
    STATE(1933), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3337), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [60804] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3372), 2,
      anon_sym_COLON,
      anon_sym_DOT_DOT,
    STATE(1934), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3368), 3,
      anon_sym_LBRACE,
      anon_sym_for,
      anon_sym_LT2,
    ACTIONS(3374), 14,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_else,
      anon_sym_in,
  [60840] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    STATE(1955), 1,
      sym_type_arguments,
    STATE(1975), 1,
      sym_parameters,
    STATE(1935), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3357), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [60880] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    STATE(1955), 1,
      sym_type_arguments,
    STATE(1975), 1,
      sym_parameters,
    STATE(1936), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3321), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [60920] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3345), 2,
      anon_sym_COLON,
      anon_sym_DOT_DOT,
    STATE(1937), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3341), 3,
      anon_sym_LBRACE,
      anon_sym_for,
      anon_sym_LT2,
    ACTIONS(3347), 14,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_else,
      anon_sym_in,
  [60956] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    STATE(1955), 1,
      sym_type_arguments,
    STATE(1975), 1,
      sym_parameters,
    STATE(1938), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3325), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [60996] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3353), 2,
      anon_sym_COLON,
      anon_sym_DOT_DOT,
    STATE(1939), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3349), 3,
      anon_sym_LBRACE,
      anon_sym_for,
      anon_sym_LT2,
    ACTIONS(3355), 14,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_else,
      anon_sym_in,
  [61032] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3333), 2,
      anon_sym_COLON,
      anon_sym_DOT_DOT,
    STATE(1940), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3329), 3,
      anon_sym_LBRACE,
      anon_sym_for,
      anon_sym_LT2,
    ACTIONS(3335), 14,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_else,
      anon_sym_in,
  [61068] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3353), 2,
      anon_sym_COLON,
      anon_sym_DOT_DOT,
    STATE(1941), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3355), 16,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_else,
      anon_sym_in,
  [61101] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3345), 2,
      anon_sym_COLON,
      anon_sym_DOT_DOT,
    STATE(1942), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3347), 16,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_else,
      anon_sym_in,
  [61134] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3333), 2,
      anon_sym_COLON,
      anon_sym_DOT_DOT,
    STATE(1943), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3335), 16,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_else,
      anon_sym_in,
  [61167] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3372), 2,
      anon_sym_COLON,
      anon_sym_DOT_DOT,
    STATE(1944), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3374), 16,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_else,
      anon_sym_in,
  [61200] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3605), 1,
      anon_sym_COLON,
    STATE(1945), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3603), 17,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_for,
      anon_sym_where,
      anon_sym_else,
  [61233] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3619), 1,
      anon_sym_COLON,
    STATE(1946), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3617), 17,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_for,
      anon_sym_where,
      anon_sym_else,
  [61266] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3639), 1,
      anon_sym_COLON,
    STATE(1947), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3637), 17,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_for,
      anon_sym_where,
      anon_sym_else,
  [61299] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1948), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3341), 18,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_for,
      anon_sym_where,
      anon_sym_else,
      anon_sym_LT2,
  [61330] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3631), 1,
      anon_sym_COLON,
    STATE(1949), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3629), 17,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_for,
      anon_sym_where,
      anon_sym_else,
  [61363] = 19,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4525), 1,
      anon_sym_const,
    ACTIONS(4527), 1,
      anon_sym_enum,
    ACTIONS(4529), 1,
      anon_sym_fn,
    ACTIONS(4531), 1,
      anon_sym_mod,
    ACTIONS(4533), 1,
      anon_sym_static,
    ACTIONS(4535), 1,
      anon_sym_struct,
    ACTIONS(4537), 1,
      anon_sym_trait,
    ACTIONS(4539), 1,
      anon_sym_type,
    ACTIONS(4541), 1,
      anon_sym_union,
    ACTIONS(4543), 1,
      anon_sym_unsafe,
    ACTIONS(4545), 1,
      anon_sym_use,
    ACTIONS(4547), 1,
      anon_sym_extern,
    STATE(2181), 1,
      sym_extern_modifier,
    STATE(2239), 1,
      aux_sym_function_modifiers_repeat1,
    STATE(3564), 1,
      sym_function_modifiers,
    ACTIONS(4523), 2,
      anon_sym_async,
      anon_sym_default,
    STATE(1950), 2,
      sym_line_comment,
      sym_block_comment,
  [61423] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4551), 1,
      anon_sym_COLON,
    ACTIONS(4553), 1,
      anon_sym_BANG,
    ACTIONS(4555), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4559), 1,
      anon_sym_COLON_COLON,
    STATE(1952), 1,
      sym_type_arguments,
    ACTIONS(4557), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1951), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4549), 9,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [61467] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3543), 1,
      anon_sym_COLON,
    STATE(1952), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3541), 16,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_for,
      anon_sym_where,
      anon_sym_else,
  [61499] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3555), 1,
      anon_sym_COLON,
    STATE(1953), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3553), 16,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_for,
      anon_sym_where,
      anon_sym_else,
  [61531] = 19,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4561), 1,
      anon_sym_const,
    ACTIONS(4563), 1,
      anon_sym_enum,
    ACTIONS(4565), 1,
      anon_sym_fn,
    ACTIONS(4567), 1,
      anon_sym_mod,
    ACTIONS(4569), 1,
      anon_sym_static,
    ACTIONS(4571), 1,
      anon_sym_struct,
    ACTIONS(4573), 1,
      anon_sym_trait,
    ACTIONS(4575), 1,
      anon_sym_type,
    ACTIONS(4577), 1,
      anon_sym_union,
    ACTIONS(4579), 1,
      anon_sym_unsafe,
    ACTIONS(4581), 1,
      anon_sym_use,
    ACTIONS(4583), 1,
      anon_sym_extern,
    STATE(2170), 1,
      sym_extern_modifier,
    STATE(2239), 1,
      aux_sym_function_modifiers_repeat1,
    STATE(3629), 1,
      sym_function_modifiers,
    ACTIONS(4523), 2,
      anon_sym_async,
      anon_sym_default,
    STATE(1954), 2,
      sym_line_comment,
      sym_block_comment,
  [61591] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3559), 1,
      anon_sym_COLON,
    STATE(1955), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3557), 16,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_for,
      anon_sym_where,
      anon_sym_else,
  [61623] = 12,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4551), 1,
      anon_sym_COLON,
    ACTIONS(4553), 1,
      anon_sym_BANG,
    ACTIONS(4555), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4585), 1,
      anon_sym_COLON_COLON,
    STATE(1952), 1,
      sym_type_arguments,
    ACTIONS(4557), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1956), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4549), 3,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_COMMA,
    ACTIONS(3458), 6,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
  [61669] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3323), 1,
      anon_sym_COLON,
    ACTIONS(4587), 1,
      anon_sym_COLON_COLON,
    STATE(1957), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3321), 14,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [61702] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1958), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3581), 16,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_DASH_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [61731] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4589), 1,
      anon_sym_DASH_GT,
    STATE(1959), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3585), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [61762] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4591), 1,
      anon_sym_DASH_GT,
    STATE(1960), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3591), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [61793] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4593), 1,
      anon_sym_DASH_GT,
    STATE(1961), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3597), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [61824] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3323), 1,
      anon_sym_COLON,
    ACTIONS(4595), 1,
      anon_sym_COLON_COLON,
    STATE(1962), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3321), 14,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [61857] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4499), 1,
      anon_sym_BANG,
    ACTIONS(4509), 1,
      anon_sym_LPAREN,
    ACTIONS(4513), 1,
      anon_sym_COLON,
    ACTIONS(4517), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4597), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4519), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1963), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4507), 9,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [61898] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1964), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1475), 16,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
      anon_sym_in,
  [61927] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4599), 1,
      anon_sym_DASH_GT,
    STATE(1965), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3561), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [61958] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3359), 1,
      anon_sym_COLON,
    ACTIONS(4595), 1,
      anon_sym_COLON_COLON,
    STATE(1966), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3357), 14,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [61991] = 16,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4499), 1,
      anon_sym_BANG,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4511), 1,
      anon_sym_LBRACE,
    ACTIONS(4515), 1,
      anon_sym_AT,
    ACTIONS(4517), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4601), 1,
      anon_sym_LPAREN,
    ACTIONS(4603), 1,
      anon_sym_RBRACK,
    ACTIONS(4606), 1,
      anon_sym_COLON_COLON,
    STATE(1953), 1,
      sym_type_arguments,
    STATE(1982), 1,
      sym_parameters,
    ACTIONS(3295), 2,
      anon_sym_SEMI,
      anon_sym_PLUS,
    ACTIONS(4507), 2,
      anon_sym_PIPE,
      anon_sym_COMMA,
    ACTIONS(4519), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1967), 2,
      sym_line_comment,
      sym_block_comment,
  [62044] = 17,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3295), 1,
      anon_sym_PLUS,
    ACTIONS(4499), 1,
      anon_sym_BANG,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4507), 1,
      anon_sym_PIPE,
    ACTIONS(4511), 1,
      anon_sym_LBRACE,
    ACTIONS(4513), 1,
      anon_sym_COLON,
    ACTIONS(4515), 1,
      anon_sym_AT,
    ACTIONS(4517), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4608), 1,
      anon_sym_LPAREN,
    ACTIONS(4610), 1,
      anon_sym_COLON_COLON,
    STATE(1953), 1,
      sym_type_arguments,
    STATE(1982), 1,
      sym_parameters,
    ACTIONS(4519), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(4603), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(1968), 2,
      sym_line_comment,
      sym_block_comment,
  [62099] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1969), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1415), 16,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
      anon_sym_in,
  [62128] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3323), 1,
      anon_sym_COLON,
    ACTIONS(4612), 1,
      anon_sym_COLON_COLON,
    STATE(1970), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3321), 14,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [62161] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1971), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1047), 16,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
      anon_sym_in,
  [62190] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4614), 1,
      anon_sym_LPAREN,
    STATE(1972), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4035), 15,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_mod,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
  [62221] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1973), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3621), 16,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_DASH_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [62250] = 16,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4499), 1,
      anon_sym_BANG,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4507), 1,
      anon_sym_PIPE,
    ACTIONS(4511), 1,
      anon_sym_LBRACE,
    ACTIONS(4513), 1,
      anon_sym_COLON,
    ACTIONS(4515), 1,
      anon_sym_AT,
    ACTIONS(4517), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4616), 1,
      anon_sym_LPAREN,
    ACTIONS(4618), 1,
      anon_sym_COLON_COLON,
    STATE(1953), 1,
      sym_type_arguments,
    STATE(1982), 1,
      sym_parameters,
    ACTIONS(4519), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1974), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3295), 3,
      anon_sym_RPAREN,
      anon_sym_PLUS,
      anon_sym_COMMA,
  [62303] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4620), 1,
      anon_sym_DASH_GT,
    STATE(1975), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3573), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [62334] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1976), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1051), 16,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
      anon_sym_in,
  [62363] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1977), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3641), 16,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_DASH_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [62392] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3327), 1,
      anon_sym_COLON,
    ACTIONS(4595), 1,
      anon_sym_COLON_COLON,
    STATE(1978), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3325), 14,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [62425] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1979), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3607), 16,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_DASH_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [62454] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3339), 1,
      anon_sym_COLON,
    ACTIONS(4595), 1,
      anon_sym_COLON_COLON,
    STATE(1980), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3337), 14,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [62487] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4622), 1,
      anon_sym_DASH_GT,
    STATE(1981), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3611), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [62518] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4624), 1,
      anon_sym_DASH_GT,
    STATE(1982), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3567), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [62549] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4553), 1,
      anon_sym_BANG,
    ACTIONS(4555), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4626), 1,
      anon_sym_COLON_COLON,
    STATE(1952), 1,
      sym_type_arguments,
    ACTIONS(4557), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1983), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4549), 3,
      anon_sym_RBRACK,
      anon_sym_PIPE,
      anon_sym_COMMA,
    ACTIONS(3458), 6,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
  [62592] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1984), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3633), 16,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_DASH_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [62621] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1985), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3813), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [62649] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1986), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3781), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [62677] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1987), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4046), 15,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_mod,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
  [62705] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3963), 1,
      anon_sym_COLON_COLON,
    STATE(1988), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4037), 14,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_mod,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
  [62735] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1989), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3689), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [62763] = 12,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4549), 1,
      anon_sym_PIPE,
    ACTIONS(4551), 1,
      anon_sym_COLON,
    ACTIONS(4553), 1,
      anon_sym_BANG,
    ACTIONS(4555), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4628), 1,
      anon_sym_COLON_COLON,
    STATE(1952), 1,
      sym_type_arguments,
    ACTIONS(4557), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(1990), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3458), 6,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
  [62807] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1991), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3321), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [62835] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1992), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3701), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [62863] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1993), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3855), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [62891] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1994), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3357), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [62919] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1995), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3325), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [62947] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1996), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3809), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [62975] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1997), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3697), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [63003] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1998), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3863), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [63031] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(1999), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3705), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [63059] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2000), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4031), 15,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_mod,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
  [63087] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2001), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3761), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [63115] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2002), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3765), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [63143] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2003), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3769), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [63171] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2004), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3817), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [63199] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2005), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3337), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [63227] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2006), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3859), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [63255] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2007), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3693), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [63283] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2008), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3867), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [63311] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2009), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3777), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [63339] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2010), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3785), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [63367] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2011), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3851), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [63395] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2012), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3877), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [63423] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4630), 1,
      anon_sym_COLON_COLON,
    STATE(2013), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4037), 14,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_mod,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
  [63453] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2014), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3705), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [63481] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2015), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4042), 15,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_mod,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      sym_identifier,
  [63509] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2016), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3725), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [63537] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4551), 1,
      anon_sym_COLON,
    ACTIONS(4555), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4559), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4557), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(2017), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4549), 9,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [63572] = 16,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    ACTIONS(4499), 1,
      anon_sym_BANG,
    ACTIONS(4501), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4632), 1,
      anon_sym_COLON,
    ACTIONS(4634), 1,
      anon_sym_EQ,
    ACTIONS(4636), 1,
      anon_sym_GT,
    ACTIONS(4638), 1,
      anon_sym_COMMA,
    STATE(1953), 1,
      sym_type_arguments,
    STATE(1982), 1,
      sym_parameters,
    STATE(2809), 1,
      sym_trait_bounds,
    STATE(2811), 1,
      aux_sym_type_parameters_repeat1,
    ACTIONS(3295), 2,
      anon_sym_PLUS,
      anon_sym_as,
    STATE(2018), 2,
      sym_line_comment,
      sym_block_comment,
  [63623] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1023), 1,
      anon_sym_DOT_DOT,
    STATE(2019), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1025), 13,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [63652] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1007), 1,
      anon_sym_DOT_DOT,
    STATE(2020), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1009), 13,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [63681] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4642), 1,
      anon_sym_pat,
    STATE(151), 1,
      sym_fragment_specifier,
    STATE(2021), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4640), 12,
      anon_sym_block,
      anon_sym_expr,
      anon_sym_ident,
      anon_sym_item,
      anon_sym_lifetime,
      anon_sym_literal,
      anon_sym_meta,
      anon_sym_path,
      anon_sym_stmt,
      anon_sym_tt,
      anon_sym_ty,
      anon_sym_vis,
  [63712] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3345), 2,
      anon_sym_COLON,
      anon_sym_DOT_DOT,
    STATE(2022), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3341), 3,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_LT2,
    ACTIONS(4644), 3,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(3347), 5,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
  [63744] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3353), 2,
      anon_sym_COLON,
      anon_sym_DOT_DOT,
    STATE(2023), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3349), 3,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_LT2,
    ACTIONS(4647), 3,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(3355), 5,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
  [63776] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3333), 2,
      anon_sym_COLON,
      anon_sym_DOT_DOT,
    STATE(2024), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3329), 3,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_LT2,
    ACTIONS(4650), 3,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(3335), 5,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
  [63808] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4517), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4519), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(2025), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4507), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [63838] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3372), 2,
      anon_sym_COLON,
      anon_sym_DOT_DOT,
    STATE(2026), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3368), 3,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_LT2,
    ACTIONS(4653), 3,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(3374), 5,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
  [63870] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4647), 1,
      anon_sym_LPAREN,
    ACTIONS(3353), 2,
      anon_sym_COLON,
      anon_sym_DOT_DOT,
    STATE(2027), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3349), 5,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_COMMA,
      anon_sym_LT2,
    ACTIONS(3355), 5,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
  [63902] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4650), 1,
      anon_sym_LPAREN,
    ACTIONS(3333), 2,
      anon_sym_COLON,
      anon_sym_DOT_DOT,
    STATE(2028), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3329), 5,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_COMMA,
      anon_sym_LT2,
    ACTIONS(3335), 5,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
  [63934] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3372), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4653), 2,
      anon_sym_LPAREN,
      anon_sym_RBRACK,
    STATE(2029), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3368), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_LT2,
    ACTIONS(3374), 6,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
  [63966] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4653), 1,
      anon_sym_LPAREN,
    ACTIONS(3372), 2,
      anon_sym_COLON,
      anon_sym_DOT_DOT,
    STATE(2030), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3368), 5,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_COMMA,
      anon_sym_LT2,
    ACTIONS(3374), 5,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
  [63998] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3345), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4644), 2,
      anon_sym_LPAREN,
      anon_sym_RBRACK,
    STATE(2031), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3341), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_LT2,
    ACTIONS(3347), 6,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
  [64030] = 13,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4656), 1,
      anon_sym_LPAREN,
    ACTIONS(4658), 1,
      anon_sym_LBRACE,
    ACTIONS(4660), 1,
      anon_sym_BANG,
    ACTIONS(4662), 1,
      anon_sym_AT,
    ACTIONS(4664), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4668), 1,
      anon_sym_COLON_COLON,
    STATE(1953), 1,
      sym_type_arguments,
    ACTIONS(4666), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(2032), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4507), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [64074] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3333), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4650), 2,
      anon_sym_LPAREN,
      anon_sym_RBRACK,
    STATE(2033), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3329), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_LT2,
    ACTIONS(3335), 6,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
  [64106] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4672), 1,
      anon_sym_DOT_DOT,
    STATE(2034), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4670), 12,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [64134] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4644), 1,
      anon_sym_LPAREN,
    ACTIONS(3345), 2,
      anon_sym_COLON,
      anon_sym_DOT_DOT,
    STATE(2035), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3341), 5,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_COMMA,
      anon_sym_LT2,
    ACTIONS(3347), 5,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
  [64166] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3353), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4647), 2,
      anon_sym_LPAREN,
      anon_sym_RBRACK,
    STATE(2036), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3349), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_LT2,
    ACTIONS(3355), 6,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
  [64198] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4676), 1,
      anon_sym_DOT_DOT,
    STATE(2037), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4674), 12,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [64226] = 14,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(2999), 1,
      anon_sym_SQUOTE,
    ACTIONS(4678), 1,
      sym_identifier,
    ACTIONS(4680), 1,
      anon_sym_GT,
    ACTIONS(4682), 1,
      anon_sym_const,
    ACTIONS(4684), 1,
      sym_metavariable,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2059), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(2572), 1,
      sym_lifetime,
    STATE(3014), 1,
      sym_constrained_type_parameter,
    STATE(2038), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(3270), 2,
      sym_const_parameter,
      sym_optional_type_parameter,
  [64271] = 14,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(2999), 1,
      anon_sym_SQUOTE,
    ACTIONS(4678), 1,
      sym_identifier,
    ACTIONS(4682), 1,
      anon_sym_const,
    ACTIONS(4684), 1,
      sym_metavariable,
    ACTIONS(4686), 1,
      anon_sym_GT,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2059), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(2572), 1,
      sym_lifetime,
    STATE(3014), 1,
      sym_constrained_type_parameter,
    STATE(2039), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(3270), 2,
      sym_const_parameter,
      sym_optional_type_parameter,
  [64316] = 14,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(2999), 1,
      anon_sym_SQUOTE,
    ACTIONS(4678), 1,
      sym_identifier,
    ACTIONS(4682), 1,
      anon_sym_const,
    ACTIONS(4684), 1,
      sym_metavariable,
    ACTIONS(4688), 1,
      anon_sym_GT,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2059), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(2572), 1,
      sym_lifetime,
    STATE(3014), 1,
      sym_constrained_type_parameter,
    STATE(2040), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(3270), 2,
      sym_const_parameter,
      sym_optional_type_parameter,
  [64361] = 14,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(2999), 1,
      anon_sym_SQUOTE,
    ACTIONS(4678), 1,
      sym_identifier,
    ACTIONS(4682), 1,
      anon_sym_const,
    ACTIONS(4684), 1,
      sym_metavariable,
    ACTIONS(4690), 1,
      anon_sym_GT,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2059), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(2572), 1,
      sym_lifetime,
    STATE(3014), 1,
      sym_constrained_type_parameter,
    STATE(2041), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(3270), 2,
      sym_const_parameter,
      sym_optional_type_parameter,
  [64406] = 10,
    ACTIONS(19), 1,
      anon_sym_LBRACE,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(4537), 1,
      anon_sym_trait,
    ACTIONS(4692), 1,
      anon_sym_impl,
    STATE(379), 1,
      sym_block,
    STATE(3355), 1,
      sym_label,
    STATE(2042), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3458), 6,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
  [64443] = 14,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(2999), 1,
      anon_sym_SQUOTE,
    ACTIONS(4678), 1,
      sym_identifier,
    ACTIONS(4682), 1,
      anon_sym_const,
    ACTIONS(4684), 1,
      sym_metavariable,
    ACTIONS(4694), 1,
      anon_sym_GT,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2059), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(2572), 1,
      sym_lifetime,
    STATE(3014), 1,
      sym_constrained_type_parameter,
    STATE(2043), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(3270), 2,
      sym_const_parameter,
      sym_optional_type_parameter,
  [64488] = 14,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(2999), 1,
      anon_sym_SQUOTE,
    ACTIONS(4678), 1,
      sym_identifier,
    ACTIONS(4682), 1,
      anon_sym_const,
    ACTIONS(4684), 1,
      sym_metavariable,
    ACTIONS(4696), 1,
      anon_sym_GT,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2059), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(2572), 1,
      sym_lifetime,
    STATE(3014), 1,
      sym_constrained_type_parameter,
    STATE(2044), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(3270), 2,
      sym_const_parameter,
      sym_optional_type_parameter,
  [64533] = 14,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(2999), 1,
      anon_sym_SQUOTE,
    ACTIONS(4678), 1,
      sym_identifier,
    ACTIONS(4682), 1,
      anon_sym_const,
    ACTIONS(4684), 1,
      sym_metavariable,
    ACTIONS(4698), 1,
      anon_sym_GT,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2059), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(2572), 1,
      sym_lifetime,
    STATE(3014), 1,
      sym_constrained_type_parameter,
    STATE(2045), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(3270), 2,
      sym_const_parameter,
      sym_optional_type_parameter,
  [64578] = 14,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(2999), 1,
      anon_sym_SQUOTE,
    ACTIONS(4678), 1,
      sym_identifier,
    ACTIONS(4682), 1,
      anon_sym_const,
    ACTIONS(4684), 1,
      sym_metavariable,
    ACTIONS(4700), 1,
      anon_sym_GT,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2059), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(2572), 1,
      sym_lifetime,
    STATE(3014), 1,
      sym_constrained_type_parameter,
    STATE(2046), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(3270), 2,
      sym_const_parameter,
      sym_optional_type_parameter,
  [64623] = 14,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(2999), 1,
      anon_sym_SQUOTE,
    ACTIONS(4678), 1,
      sym_identifier,
    ACTIONS(4682), 1,
      anon_sym_const,
    ACTIONS(4684), 1,
      sym_metavariable,
    ACTIONS(4702), 1,
      anon_sym_GT,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2059), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(2572), 1,
      sym_lifetime,
    STATE(3014), 1,
      sym_constrained_type_parameter,
    STATE(2047), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(3270), 2,
      sym_const_parameter,
      sym_optional_type_parameter,
  [64668] = 14,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(2999), 1,
      anon_sym_SQUOTE,
    ACTIONS(4678), 1,
      sym_identifier,
    ACTIONS(4682), 1,
      anon_sym_const,
    ACTIONS(4684), 1,
      sym_metavariable,
    ACTIONS(4704), 1,
      anon_sym_GT,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2059), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(2572), 1,
      sym_lifetime,
    STATE(3014), 1,
      sym_constrained_type_parameter,
    STATE(2048), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(3270), 2,
      sym_const_parameter,
      sym_optional_type_parameter,
  [64713] = 14,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(2999), 1,
      anon_sym_SQUOTE,
    ACTIONS(4678), 1,
      sym_identifier,
    ACTIONS(4682), 1,
      anon_sym_const,
    ACTIONS(4684), 1,
      sym_metavariable,
    ACTIONS(4706), 1,
      anon_sym_GT,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2059), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(2572), 1,
      sym_lifetime,
    STATE(3014), 1,
      sym_constrained_type_parameter,
    STATE(2049), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(3270), 2,
      sym_const_parameter,
      sym_optional_type_parameter,
  [64758] = 14,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(2999), 1,
      anon_sym_SQUOTE,
    ACTIONS(4678), 1,
      sym_identifier,
    ACTIONS(4682), 1,
      anon_sym_const,
    ACTIONS(4684), 1,
      sym_metavariable,
    ACTIONS(4708), 1,
      anon_sym_GT,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2059), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(2572), 1,
      sym_lifetime,
    STATE(3014), 1,
      sym_constrained_type_parameter,
    STATE(2050), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(3270), 2,
      sym_const_parameter,
      sym_optional_type_parameter,
  [64803] = 14,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(2999), 1,
      anon_sym_SQUOTE,
    ACTIONS(4678), 1,
      sym_identifier,
    ACTIONS(4682), 1,
      anon_sym_const,
    ACTIONS(4684), 1,
      sym_metavariable,
    ACTIONS(4710), 1,
      anon_sym_GT,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2059), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(2667), 1,
      sym_lifetime,
    STATE(3014), 1,
      sym_constrained_type_parameter,
    STATE(2051), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(3270), 2,
      sym_const_parameter,
      sym_optional_type_parameter,
  [64848] = 14,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(2999), 1,
      anon_sym_SQUOTE,
    ACTIONS(4678), 1,
      sym_identifier,
    ACTIONS(4682), 1,
      anon_sym_const,
    ACTIONS(4684), 1,
      sym_metavariable,
    ACTIONS(4712), 1,
      anon_sym_GT,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2059), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(2572), 1,
      sym_lifetime,
    STATE(3014), 1,
      sym_constrained_type_parameter,
    STATE(2052), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(3270), 2,
      sym_const_parameter,
      sym_optional_type_parameter,
  [64893] = 14,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(2999), 1,
      anon_sym_SQUOTE,
    ACTIONS(4678), 1,
      sym_identifier,
    ACTIONS(4682), 1,
      anon_sym_const,
    ACTIONS(4684), 1,
      sym_metavariable,
    ACTIONS(4714), 1,
      anon_sym_GT,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2059), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(2572), 1,
      sym_lifetime,
    STATE(3014), 1,
      sym_constrained_type_parameter,
    STATE(2053), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(3270), 2,
      sym_const_parameter,
      sym_optional_type_parameter,
  [64938] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    ACTIONS(4499), 1,
      anon_sym_BANG,
    ACTIONS(4501), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4716), 1,
      anon_sym_for,
    STATE(1953), 1,
      sym_type_arguments,
    STATE(1982), 1,
      sym_parameters,
    STATE(2054), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3295), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [64976] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    ACTIONS(4499), 1,
      anon_sym_BANG,
    ACTIONS(4501), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4718), 1,
      anon_sym_for,
    STATE(1953), 1,
      sym_type_arguments,
    STATE(1982), 1,
      sym_parameters,
    STATE(2055), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3295), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [65014] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4630), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4722), 1,
      anon_sym_COLON,
    STATE(2056), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4720), 9,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [65042] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1341), 1,
      aux_sym_string_literal_token1,
    ACTIONS(4726), 1,
      sym_crate,
    STATE(2199), 1,
      sym_string_literal,
    STATE(2057), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4724), 8,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
  [65072] = 9,
    ACTIONS(19), 1,
      anon_sym_LBRACE,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(4728), 1,
      anon_sym_move,
    STATE(381), 1,
      sym_block,
    STATE(3355), 1,
      sym_label,
    STATE(2058), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3458), 6,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
  [65106] = 13,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(2999), 1,
      anon_sym_SQUOTE,
    ACTIONS(4682), 1,
      anon_sym_const,
    ACTIONS(4730), 1,
      sym_identifier,
    ACTIONS(4732), 1,
      sym_metavariable,
    STATE(1061), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2722), 1,
      sym_lifetime,
    STATE(2781), 1,
      sym_constrained_type_parameter,
    STATE(2059), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(3310), 2,
      sym_const_parameter,
      sym_optional_type_parameter,
  [65148] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4630), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4736), 1,
      anon_sym_COLON,
    STATE(2060), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4734), 9,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [65176] = 13,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(2999), 1,
      anon_sym_SQUOTE,
    ACTIONS(4682), 1,
      anon_sym_const,
    ACTIONS(4738), 1,
      sym_identifier,
    ACTIONS(4740), 1,
      sym_metavariable,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2077), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(2486), 1,
      sym_lifetime,
    STATE(2540), 1,
      sym_constrained_type_parameter,
    STATE(2061), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(2953), 2,
      sym_const_parameter,
      sym_optional_type_parameter,
  [65218] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    ACTIONS(4499), 1,
      anon_sym_BANG,
    ACTIONS(4501), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4742), 1,
      anon_sym_for,
    STATE(1953), 1,
      sym_type_arguments,
    STATE(1982), 1,
      sym_parameters,
    STATE(2062), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3295), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [65256] = 12,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3299), 1,
      anon_sym_COLON,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    ACTIONS(4499), 1,
      anon_sym_BANG,
    ACTIONS(4501), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4744), 1,
      anon_sym_EQ,
    STATE(1982), 1,
      sym_parameters,
    STATE(2358), 1,
      sym_type_arguments,
    STATE(2063), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3295), 3,
      anon_sym_PLUS,
      anon_sym_GT,
      anon_sym_COMMA,
  [65296] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2064), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1265), 11,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [65320] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2065), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1273), 11,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [65344] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3372), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3368), 2,
      anon_sym_LBRACE,
      anon_sym_LT2,
    STATE(2066), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3374), 8,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_if,
  [65372] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3333), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3329), 2,
      anon_sym_LBRACE,
      anon_sym_LT2,
    STATE(2067), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3335), 8,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_if,
  [65400] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3345), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3341), 2,
      anon_sym_LBRACE,
      anon_sym_LT2,
    STATE(2068), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3347), 8,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_if,
  [65428] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3353), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3349), 2,
      anon_sym_LBRACE,
      anon_sym_LT2,
    STATE(2069), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3355), 8,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_if,
  [65456] = 13,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(2999), 1,
      anon_sym_SQUOTE,
    ACTIONS(4682), 1,
      anon_sym_const,
    ACTIONS(4746), 1,
      sym_identifier,
    ACTIONS(4748), 1,
      sym_metavariable,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2071), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(2422), 1,
      sym_lifetime,
    STATE(2600), 1,
      sym_constrained_type_parameter,
    STATE(2070), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(3040), 2,
      sym_const_parameter,
      sym_optional_type_parameter,
  [65498] = 13,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(2999), 1,
      anon_sym_SQUOTE,
    ACTIONS(4682), 1,
      anon_sym_const,
    ACTIONS(4750), 1,
      sym_identifier,
    ACTIONS(4752), 1,
      sym_metavariable,
    STATE(1061), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2425), 1,
      sym_lifetime,
    STATE(2536), 1,
      sym_constrained_type_parameter,
    STATE(2071), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(2796), 2,
      sym_const_parameter,
      sym_optional_type_parameter,
  [65540] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1341), 1,
      aux_sym_string_literal_token1,
    ACTIONS(4754), 1,
      sym_crate,
    STATE(2199), 1,
      sym_string_literal,
    STATE(2072), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4724), 8,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
  [65570] = 13,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(2999), 1,
      anon_sym_SQUOTE,
    ACTIONS(4678), 1,
      sym_identifier,
    ACTIONS(4682), 1,
      anon_sym_const,
    ACTIONS(4684), 1,
      sym_metavariable,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2059), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(2572), 1,
      sym_lifetime,
    STATE(3014), 1,
      sym_constrained_type_parameter,
    STATE(2073), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(3270), 2,
      sym_const_parameter,
      sym_optional_type_parameter,
  [65612] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4736), 1,
      anon_sym_COLON,
    ACTIONS(4756), 1,
      anon_sym_COLON_COLON,
    STATE(2074), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4734), 9,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [65640] = 13,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(2999), 1,
      anon_sym_SQUOTE,
    ACTIONS(4682), 1,
      anon_sym_const,
    ACTIONS(4738), 1,
      sym_identifier,
    ACTIONS(4740), 1,
      sym_metavariable,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2077), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(2382), 1,
      sym_lifetime,
    STATE(2540), 1,
      sym_constrained_type_parameter,
    STATE(2075), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(2953), 2,
      sym_const_parameter,
      sym_optional_type_parameter,
  [65682] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4736), 1,
      anon_sym_COLON,
    ACTIONS(4758), 1,
      anon_sym_COLON_COLON,
    STATE(2076), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4734), 9,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [65710] = 13,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(2999), 1,
      anon_sym_SQUOTE,
    ACTIONS(4682), 1,
      anon_sym_const,
    ACTIONS(4760), 1,
      sym_identifier,
    ACTIONS(4762), 1,
      sym_metavariable,
    STATE(1061), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2428), 1,
      sym_lifetime,
    STATE(2539), 1,
      sym_constrained_type_parameter,
    STATE(2077), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(2837), 2,
      sym_const_parameter,
      sym_optional_type_parameter,
  [65752] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2078), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1269), 11,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [65776] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4722), 1,
      anon_sym_COLON,
    ACTIONS(4756), 1,
      anon_sym_COLON_COLON,
    STATE(2079), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4720), 9,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [65804] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4766), 1,
      anon_sym_COLON,
    ACTIONS(4768), 1,
      anon_sym_COLON_COLON,
    STATE(2080), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4764), 9,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [65832] = 9,
    ACTIONS(19), 1,
      anon_sym_LBRACE,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(4770), 1,
      sym_identifier,
    STATE(397), 1,
      sym_block,
    STATE(3355), 1,
      sym_label,
    STATE(2081), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4772), 6,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
  [65866] = 12,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(4774), 1,
      sym_identifier,
    ACTIONS(4776), 1,
      anon_sym_RBRACE,
    ACTIONS(4778), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4780), 1,
      anon_sym_COMMA,
    ACTIONS(4782), 1,
      sym_integer_literal,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2466), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(2082), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(2758), 3,
      sym_shorthand_field_initializer,
      sym_field_initializer,
      sym_base_field_initializer,
  [65906] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4722), 1,
      anon_sym_COLON,
    ACTIONS(4758), 1,
      anon_sym_COLON_COLON,
    STATE(2083), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4720), 9,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [65934] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4551), 1,
      anon_sym_COLON,
    ACTIONS(4553), 1,
      anon_sym_BANG,
    ACTIONS(4555), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4585), 1,
      anon_sym_COLON_COLON,
    STATE(1952), 1,
      sym_type_arguments,
    ACTIONS(4557), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(2084), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4549), 3,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_COMMA,
  [65972] = 12,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(4774), 1,
      sym_identifier,
    ACTIONS(4778), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4782), 1,
      sym_integer_literal,
    ACTIONS(4784), 1,
      anon_sym_RBRACE,
    ACTIONS(4786), 1,
      anon_sym_COMMA,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2466), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(2085), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(2905), 3,
      sym_shorthand_field_initializer,
      sym_field_initializer,
      sym_base_field_initializer,
  [66012] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    ACTIONS(4499), 1,
      anon_sym_BANG,
    ACTIONS(4501), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4788), 1,
      anon_sym_for,
    STATE(1953), 1,
      sym_type_arguments,
    STATE(1982), 1,
      sym_parameters,
    STATE(2086), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3295), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [66050] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2087), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1261), 11,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [66074] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1341), 1,
      aux_sym_string_literal_token1,
    ACTIONS(4790), 1,
      sym_crate,
    STATE(2199), 1,
      sym_string_literal,
    STATE(2088), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4724), 8,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
  [66104] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1341), 1,
      aux_sym_string_literal_token1,
    ACTIONS(4792), 1,
      sym_crate,
    STATE(2199), 1,
      sym_string_literal,
    STATE(2089), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4724), 8,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
  [66134] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    ACTIONS(4499), 1,
      anon_sym_BANG,
    ACTIONS(4501), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4794), 1,
      anon_sym_for,
    STATE(1953), 1,
      sym_type_arguments,
    STATE(1982), 1,
      sym_parameters,
    STATE(2090), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3295), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [66172] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    ACTIONS(4499), 1,
      anon_sym_BANG,
    ACTIONS(4501), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4796), 1,
      anon_sym_for,
    STATE(1953), 1,
      sym_type_arguments,
    STATE(1982), 1,
      sym_parameters,
    STATE(2091), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3295), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [66210] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    ACTIONS(4499), 1,
      anon_sym_BANG,
    ACTIONS(4501), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4798), 1,
      anon_sym_for,
    STATE(1953), 1,
      sym_type_arguments,
    STATE(1982), 1,
      sym_parameters,
    STATE(2092), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3295), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [66248] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2093), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1257), 11,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [66272] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    ACTIONS(4499), 1,
      anon_sym_BANG,
    ACTIONS(4501), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4800), 1,
      anon_sym_for,
    STATE(1953), 1,
      sym_type_arguments,
    STATE(1982), 1,
      sym_parameters,
    STATE(2094), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3295), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [66310] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4768), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4804), 1,
      anon_sym_COLON,
    STATE(2095), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4802), 9,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [66338] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2096), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4806), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [66361] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4553), 1,
      anon_sym_BANG,
    ACTIONS(4587), 1,
      anon_sym_COLON_COLON,
    STATE(1952), 1,
      sym_type_arguments,
    STATE(2097), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3458), 6,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
  [66392] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2098), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4808), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [66415] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2099), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4810), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [66438] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2100), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4812), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [66461] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2101), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4814), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [66484] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2102), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4816), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [66507] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2103), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4818), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [66530] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2104), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4820), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [66553] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2105), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4822), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [66576] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2106), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4824), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [66599] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2107), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4826), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [66622] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2108), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4828), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [66645] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(4774), 1,
      sym_identifier,
    ACTIONS(4778), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4782), 1,
      sym_integer_literal,
    ACTIONS(4830), 1,
      anon_sym_RBRACE,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2466), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(2109), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(3096), 3,
      sym_shorthand_field_initializer,
      sym_field_initializer,
      sym_base_field_initializer,
  [66682] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3305), 1,
      anon_sym_LT2,
    ACTIONS(3525), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4832), 1,
      anon_sym_BANG,
    STATE(1087), 1,
      sym_type_arguments,
    STATE(2110), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3458), 6,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
  [66713] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2111), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1455), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [66736] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4011), 1,
      anon_sym_LT2,
    ACTIONS(4167), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4834), 1,
      anon_sym_BANG,
    STATE(1615), 1,
      sym_type_arguments,
    STATE(2112), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3458), 6,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
  [66767] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2113), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4836), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [66790] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2114), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4838), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [66813] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2115), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4840), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [66836] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2116), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4842), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [66859] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(4774), 1,
      sym_identifier,
    ACTIONS(4778), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4782), 1,
      sym_integer_literal,
    ACTIONS(4844), 1,
      anon_sym_RBRACE,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2466), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(2117), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(3096), 3,
      sym_shorthand_field_initializer,
      sym_field_initializer,
      sym_base_field_initializer,
  [66896] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4846), 1,
      anon_sym_LBRACE,
    STATE(1955), 1,
      sym_type_arguments,
    STATE(1975), 1,
      sym_parameters,
    STATE(2118), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3321), 5,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_PLUS,
      anon_sym_COMMA,
  [66929] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2119), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4848), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [66952] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2120), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4850), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [66975] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2121), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4720), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [66998] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2122), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4852), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [67021] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(4774), 1,
      sym_identifier,
    ACTIONS(4778), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4782), 1,
      sym_integer_literal,
    ACTIONS(4854), 1,
      anon_sym_RBRACE,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2466), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(2123), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(3096), 3,
      sym_shorthand_field_initializer,
      sym_field_initializer,
      sym_base_field_initializer,
  [67058] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2124), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4856), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [67081] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4499), 1,
      anon_sym_BANG,
    ACTIONS(4509), 1,
      anon_sym_LPAREN,
    ACTIONS(4513), 1,
      anon_sym_COLON,
    ACTIONS(4517), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4858), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4519), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(2125), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4507), 3,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_COMMA,
  [67116] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2126), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4860), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [67139] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1601), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(2111), 1,
      sym_block,
    STATE(3611), 1,
      sym_label,
    STATE(2127), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3458), 6,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
  [67170] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2128), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4862), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [67193] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2129), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4734), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [67216] = 13,
    ACTIONS(69), 1,
      anon_sym_pub,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(4864), 1,
      sym_identifier,
    ACTIONS(4866), 1,
      anon_sym_RBRACE,
    ACTIONS(4868), 1,
      anon_sym_COMMA,
    ACTIONS(4870), 1,
      sym_crate,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2244), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(2787), 1,
      sym_enum_variant,
    STATE(3612), 1,
      sym_visibility_modifier,
    STATE(2130), 2,
      sym_line_comment,
      sym_block_comment,
  [67257] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2131), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4872), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [67280] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4553), 1,
      anon_sym_BANG,
    ACTIONS(4555), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4626), 1,
      anon_sym_COLON_COLON,
    STATE(1952), 1,
      sym_type_arguments,
    ACTIONS(4557), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(2132), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4549), 3,
      anon_sym_RBRACK,
      anon_sym_PIPE,
      anon_sym_COMMA,
  [67315] = 13,
    ACTIONS(69), 1,
      anon_sym_pub,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(4870), 1,
      sym_crate,
    ACTIONS(4874), 1,
      sym_identifier,
    ACTIONS(4876), 1,
      anon_sym_RBRACE,
    ACTIONS(4878), 1,
      anon_sym_COMMA,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2246), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(2802), 1,
      sym_field_declaration,
    STATE(3407), 1,
      sym_visibility_modifier,
    STATE(2133), 2,
      sym_line_comment,
      sym_block_comment,
  [67356] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2134), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4880), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [67379] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2135), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4882), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [67402] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2136), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4507), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [67425] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2137), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4884), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [67448] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(4774), 1,
      sym_identifier,
    ACTIONS(4778), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4782), 1,
      sym_integer_literal,
    ACTIONS(4886), 1,
      anon_sym_RBRACE,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2466), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(2138), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(3096), 3,
      sym_shorthand_field_initializer,
      sym_field_initializer,
      sym_base_field_initializer,
  [67485] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4888), 1,
      anon_sym_BANG,
    ACTIONS(4890), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4894), 1,
      anon_sym_COLON_COLON,
    STATE(1952), 1,
      sym_type_arguments,
    ACTIONS(4892), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(2139), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4549), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [67520] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2140), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4896), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [67543] = 13,
    ACTIONS(69), 1,
      anon_sym_pub,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(4864), 1,
      sym_identifier,
    ACTIONS(4870), 1,
      sym_crate,
    ACTIONS(4898), 1,
      anon_sym_RBRACE,
    ACTIONS(4900), 1,
      anon_sym_COMMA,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2213), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(2969), 1,
      sym_enum_variant,
    STATE(3612), 1,
      sym_visibility_modifier,
    STATE(2141), 2,
      sym_line_comment,
      sym_block_comment,
  [67584] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2142), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4902), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [67607] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2143), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4904), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [67630] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2144), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4906), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [67653] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2145), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4908), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [67676] = 13,
    ACTIONS(69), 1,
      anon_sym_pub,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(4870), 1,
      sym_crate,
    ACTIONS(4874), 1,
      sym_identifier,
    ACTIONS(4910), 1,
      anon_sym_RBRACE,
    ACTIONS(4912), 1,
      anon_sym_COMMA,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2200), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(2949), 1,
      sym_field_declaration,
    STATE(3407), 1,
      sym_visibility_modifier,
    STATE(2146), 2,
      sym_line_comment,
      sym_block_comment,
  [67717] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2147), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4914), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_else,
      anon_sym_in,
  [67740] = 12,
    ACTIONS(69), 1,
      anon_sym_pub,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(4870), 1,
      sym_crate,
    ACTIONS(4874), 1,
      sym_identifier,
    ACTIONS(4916), 1,
      anon_sym_RBRACE,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2214), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(3246), 1,
      sym_field_declaration,
    STATE(3407), 1,
      sym_visibility_modifier,
    STATE(2148), 2,
      sym_line_comment,
      sym_block_comment,
  [67778] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4920), 1,
      anon_sym_PLUS,
    STATE(2154), 1,
      aux_sym_trait_bounds_repeat1,
    STATE(2149), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4918), 7,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_where,
  [67804] = 12,
    ACTIONS(69), 1,
      anon_sym_pub,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(4864), 1,
      sym_identifier,
    ACTIONS(4870), 1,
      sym_crate,
    ACTIONS(4922), 1,
      anon_sym_RBRACE,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2198), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(3233), 1,
      sym_enum_variant,
    STATE(3612), 1,
      sym_visibility_modifier,
    STATE(2150), 2,
      sym_line_comment,
      sym_block_comment,
  [67842] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4656), 1,
      anon_sym_LPAREN,
    ACTIONS(4660), 1,
      anon_sym_BANG,
    ACTIONS(4664), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4924), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4666), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(2151), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4507), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [67874] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4926), 1,
      anon_sym_for,
    STATE(1955), 1,
      sym_type_arguments,
    STATE(1975), 1,
      sym_parameters,
    STATE(2152), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3321), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [67906] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4555), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4626), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4928), 1,
      anon_sym_RBRACK,
    ACTIONS(3321), 2,
      anon_sym_SEMI,
      anon_sym_PLUS,
    ACTIONS(4549), 2,
      anon_sym_PIPE,
      anon_sym_COMMA,
    ACTIONS(4557), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(2153), 2,
      sym_line_comment,
      sym_block_comment,
  [67938] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4933), 1,
      anon_sym_PLUS,
    STATE(2154), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_trait_bounds_repeat1,
    ACTIONS(4931), 7,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_where,
  [67962] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4936), 1,
      anon_sym_for,
    STATE(1955), 1,
      sym_type_arguments,
    STATE(1975), 1,
      sym_parameters,
    STATE(2155), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3321), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [67994] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(4774), 1,
      sym_identifier,
    ACTIONS(4778), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4782), 1,
      sym_integer_literal,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2466), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(2156), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(3096), 3,
      sym_shorthand_field_initializer,
      sym_field_initializer,
      sym_base_field_initializer,
  [68028] = 12,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4938), 1,
      anon_sym_SEMI,
    ACTIONS(4940), 1,
      anon_sym_LPAREN,
    ACTIONS(4942), 1,
      anon_sym_LBRACE,
    ACTIONS(4944), 1,
      anon_sym_LT,
    ACTIONS(4946), 1,
      anon_sym_where,
    STATE(758), 1,
      sym_field_declaration_list,
    STATE(2272), 1,
      sym_type_parameters,
    STATE(2891), 1,
      sym_ordered_field_declaration_list,
    STATE(3320), 1,
      sym_where_clause,
    STATE(2157), 2,
      sym_line_comment,
      sym_block_comment,
  [68066] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3372), 1,
      anon_sym_DOT_DOT,
    STATE(2158), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3374), 8,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_if,
  [68090] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3333), 1,
      anon_sym_DOT_DOT,
    STATE(2159), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3335), 8,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_if,
  [68114] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4948), 1,
      anon_sym_SEMI,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    STATE(680), 1,
      sym_declaration_list,
    STATE(2160), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3458), 6,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
  [68142] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3345), 1,
      anon_sym_DOT_DOT,
    STATE(2161), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3347), 8,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_if,
  [68166] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3353), 1,
      anon_sym_DOT_DOT,
    STATE(2162), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3355), 8,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_if,
  [68190] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3321), 1,
      anon_sym_PLUS,
    ACTIONS(4549), 1,
      anon_sym_PIPE,
    ACTIONS(4551), 1,
      anon_sym_COLON,
    ACTIONS(4555), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4585), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4557), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(4928), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(2163), 2,
      sym_line_comment,
      sym_block_comment,
  [68224] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4952), 1,
      anon_sym_COLON_COLON,
    STATE(1952), 1,
      sym_type_arguments,
    STATE(2164), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3458), 6,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
  [68252] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4954), 1,
      anon_sym_SEMI,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    STATE(1085), 1,
      sym_declaration_list,
    STATE(2165), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3458), 6,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
  [68280] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4920), 1,
      anon_sym_PLUS,
    STATE(2149), 1,
      aux_sym_trait_bounds_repeat1,
    STATE(2166), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4958), 7,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_where,
  [68306] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4960), 1,
      anon_sym_for,
    STATE(1955), 1,
      sym_type_arguments,
    STATE(1975), 1,
      sym_parameters,
    STATE(2167), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3321), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [68338] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4962), 1,
      anon_sym_for,
    STATE(1955), 1,
      sym_type_arguments,
    STATE(1975), 1,
      sym_parameters,
    STATE(2168), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3321), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [68370] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4964), 1,
      anon_sym_PLUS,
    STATE(2149), 1,
      aux_sym_trait_bounds_repeat1,
    STATE(2169), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4958), 7,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_where,
  [68396] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(4966), 1,
      anon_sym_SEMI,
    STATE(1112), 1,
      sym_declaration_list,
    STATE(2170), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3458), 6,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
  [68424] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4968), 1,
      anon_sym_PLUS,
    STATE(2149), 1,
      aux_sym_trait_bounds_repeat1,
    STATE(2171), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4958), 7,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_where,
  [68450] = 12,
    ACTIONS(69), 1,
      anon_sym_pub,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(4864), 1,
      sym_identifier,
    ACTIONS(4870), 1,
      sym_crate,
    ACTIONS(4970), 1,
      anon_sym_RBRACE,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2198), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(3233), 1,
      sym_enum_variant,
    STATE(3612), 1,
      sym_visibility_modifier,
    STATE(2172), 2,
      sym_line_comment,
      sym_block_comment,
  [68488] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3305), 1,
      anon_sym_LT2,
    ACTIONS(4972), 1,
      anon_sym_COLON_COLON,
    STATE(1087), 1,
      sym_type_arguments,
    STATE(2173), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3458), 6,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
  [68516] = 12,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4940), 1,
      anon_sym_LPAREN,
    ACTIONS(4944), 1,
      anon_sym_LT,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4974), 1,
      anon_sym_SEMI,
    ACTIONS(4976), 1,
      anon_sym_LBRACE,
    STATE(1151), 1,
      sym_field_declaration_list,
    STATE(2260), 1,
      sym_type_parameters,
    STATE(2810), 1,
      sym_ordered_field_declaration_list,
    STATE(3277), 1,
      sym_where_clause,
    STATE(2174), 2,
      sym_line_comment,
      sym_block_comment,
  [68554] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4978), 1,
      anon_sym_for,
    STATE(1955), 1,
      sym_type_arguments,
    STATE(1975), 1,
      sym_parameters,
    STATE(2175), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3321), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [68586] = 12,
    ACTIONS(69), 1,
      anon_sym_pub,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(4864), 1,
      sym_identifier,
    ACTIONS(4870), 1,
      sym_crate,
    ACTIONS(4980), 1,
      anon_sym_RBRACE,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2198), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(3233), 1,
      sym_enum_variant,
    STATE(3612), 1,
      sym_visibility_modifier,
    STATE(2176), 2,
      sym_line_comment,
      sym_block_comment,
  [68624] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4549), 1,
      anon_sym_PIPE,
    ACTIONS(4551), 1,
      anon_sym_COLON,
    ACTIONS(4555), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4628), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4557), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(2177), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3321), 3,
      anon_sym_RPAREN,
      anon_sym_PLUS,
      anon_sym_COMMA,
  [68656] = 12,
    ACTIONS(69), 1,
      anon_sym_pub,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(4870), 1,
      sym_crate,
    ACTIONS(4874), 1,
      sym_identifier,
    ACTIONS(4982), 1,
      anon_sym_RBRACE,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2214), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(3246), 1,
      sym_field_declaration,
    STATE(3407), 1,
      sym_visibility_modifier,
    STATE(2178), 2,
      sym_line_comment,
      sym_block_comment,
  [68694] = 12,
    ACTIONS(69), 1,
      anon_sym_pub,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(4864), 1,
      sym_identifier,
    ACTIONS(4870), 1,
      sym_crate,
    ACTIONS(4984), 1,
      anon_sym_RBRACE,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2198), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(3233), 1,
      sym_enum_variant,
    STATE(3612), 1,
      sym_visibility_modifier,
    STATE(2179), 2,
      sym_line_comment,
      sym_block_comment,
  [68732] = 12,
    ACTIONS(69), 1,
      anon_sym_pub,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(4870), 1,
      sym_crate,
    ACTIONS(4874), 1,
      sym_identifier,
    ACTIONS(4986), 1,
      anon_sym_RBRACE,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2214), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(3246), 1,
      sym_field_declaration,
    STATE(3407), 1,
      sym_visibility_modifier,
    STATE(2180), 2,
      sym_line_comment,
      sym_block_comment,
  [68770] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(4988), 1,
      anon_sym_SEMI,
    STATE(703), 1,
      sym_declaration_list,
    STATE(2181), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3458), 6,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
  [68798] = 12,
    ACTIONS(69), 1,
      anon_sym_pub,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(4864), 1,
      sym_identifier,
    ACTIONS(4870), 1,
      sym_crate,
    ACTIONS(4990), 1,
      anon_sym_RBRACE,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2198), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(3233), 1,
      sym_enum_variant,
    STATE(3612), 1,
      sym_visibility_modifier,
    STATE(2182), 2,
      sym_line_comment,
      sym_block_comment,
  [68836] = 12,
    ACTIONS(69), 1,
      anon_sym_pub,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(4870), 1,
      sym_crate,
    ACTIONS(4874), 1,
      sym_identifier,
    ACTIONS(4992), 1,
      anon_sym_RBRACE,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2214), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(3246), 1,
      sym_field_declaration,
    STATE(3407), 1,
      sym_visibility_modifier,
    STATE(2183), 2,
      sym_line_comment,
      sym_block_comment,
  [68874] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4011), 1,
      anon_sym_LT2,
    ACTIONS(4994), 1,
      anon_sym_COLON_COLON,
    STATE(1615), 1,
      sym_type_arguments,
    STATE(2184), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3458), 6,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
  [68902] = 12,
    ACTIONS(69), 1,
      anon_sym_pub,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(4870), 1,
      sym_crate,
    ACTIONS(4874), 1,
      sym_identifier,
    ACTIONS(4996), 1,
      anon_sym_RBRACE,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2214), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(3246), 1,
      sym_field_declaration,
    STATE(3407), 1,
      sym_visibility_modifier,
    STATE(2185), 2,
      sym_line_comment,
      sym_block_comment,
  [68940] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4998), 1,
      anon_sym_LPAREN,
    ACTIONS(5003), 1,
      anon_sym_LBRACK,
    ACTIONS(5006), 1,
      anon_sym_LBRACE,
    STATE(3404), 1,
      sym_macro_rule,
    STATE(3487), 1,
      sym_token_tree_pattern,
    ACTIONS(5001), 3,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
    STATE(2186), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_macro_definition_repeat1,
  [68972] = 12,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4940), 1,
      anon_sym_LPAREN,
    ACTIONS(4942), 1,
      anon_sym_LBRACE,
    ACTIONS(4944), 1,
      anon_sym_LT,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5009), 1,
      anon_sym_SEMI,
    STATE(644), 1,
      sym_field_declaration_list,
    STATE(2259), 1,
      sym_type_parameters,
    STATE(2888), 1,
      sym_ordered_field_declaration_list,
    STATE(3314), 1,
      sym_where_clause,
    STATE(2187), 2,
      sym_line_comment,
      sym_block_comment,
  [69010] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4549), 1,
      anon_sym_PIPE,
    ACTIONS(4551), 1,
      anon_sym_COLON,
    ACTIONS(4553), 1,
      anon_sym_BANG,
    ACTIONS(4555), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4628), 1,
      anon_sym_COLON_COLON,
    STATE(1952), 1,
      sym_type_arguments,
    ACTIONS(4557), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(2188), 2,
      sym_line_comment,
      sym_block_comment,
  [69046] = 12,
    ACTIONS(69), 1,
      anon_sym_pub,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(4870), 1,
      sym_crate,
    ACTIONS(4874), 1,
      sym_identifier,
    ACTIONS(5011), 1,
      anon_sym_RBRACE,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2214), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(3246), 1,
      sym_field_declaration,
    STATE(3407), 1,
      sym_visibility_modifier,
    STATE(2189), 2,
      sym_line_comment,
      sym_block_comment,
  [69084] = 12,
    ACTIONS(69), 1,
      anon_sym_pub,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(4864), 1,
      sym_identifier,
    ACTIONS(4870), 1,
      sym_crate,
    ACTIONS(5013), 1,
      anon_sym_RBRACE,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2198), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(3233), 1,
      sym_enum_variant,
    STATE(3612), 1,
      sym_visibility_modifier,
    STATE(2190), 2,
      sym_line_comment,
      sym_block_comment,
  [69122] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4499), 1,
      anon_sym_BANG,
    ACTIONS(4509), 1,
      anon_sym_LPAREN,
    ACTIONS(4517), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5015), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4519), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(2191), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4507), 3,
      anon_sym_RBRACK,
      anon_sym_PIPE,
      anon_sym_COMMA,
  [69154] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(5017), 1,
      anon_sym_for,
    STATE(1955), 1,
      sym_type_arguments,
    STATE(1975), 1,
      sym_parameters,
    STATE(2192), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3321), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [69186] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(5019), 1,
      anon_sym_for,
    STATE(1955), 1,
      sym_type_arguments,
    STATE(1975), 1,
      sym_parameters,
    STATE(2193), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3321), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [69218] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(5021), 1,
      anon_sym_for,
    STATE(1955), 1,
      sym_type_arguments,
    STATE(1975), 1,
      sym_parameters,
    STATE(2194), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3321), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [69250] = 12,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4940), 1,
      anon_sym_LPAREN,
    ACTIONS(4944), 1,
      anon_sym_LT,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4976), 1,
      anon_sym_LBRACE,
    ACTIONS(5023), 1,
      anon_sym_SEMI,
    STATE(1105), 1,
      sym_field_declaration_list,
    STATE(2273), 1,
      sym_type_parameters,
    STATE(3049), 1,
      sym_ordered_field_declaration_list,
    STATE(3324), 1,
      sym_where_clause,
    STATE(2195), 2,
      sym_line_comment,
      sym_block_comment,
  [69288] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5025), 1,
      anon_sym_SEMI,
    ACTIONS(5027), 1,
      anon_sym_LBRACE,
    ACTIONS(5029), 1,
      anon_sym_DASH_GT,
    STATE(687), 1,
      sym_block,
    STATE(2442), 1,
      sym_where_clause,
    STATE(3615), 1,
      sym_label,
    STATE(2196), 2,
      sym_line_comment,
      sym_block_comment,
  [69323] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5027), 1,
      anon_sym_LBRACE,
    ACTIONS(5031), 1,
      anon_sym_SEMI,
    ACTIONS(5033), 1,
      anon_sym_DASH_GT,
    STATE(524), 1,
      sym_block,
    STATE(2487), 1,
      sym_where_clause,
    STATE(3615), 1,
      sym_label,
    STATE(2197), 2,
      sym_line_comment,
      sym_block_comment,
  [69358] = 11,
    ACTIONS(69), 1,
      anon_sym_pub,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(4864), 1,
      sym_identifier,
    ACTIONS(4870), 1,
      sym_crate,
    STATE(1061), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(3230), 1,
      sym_enum_variant,
    STATE(3612), 1,
      sym_visibility_modifier,
    STATE(2198), 2,
      sym_line_comment,
      sym_block_comment,
  [69393] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2199), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5035), 8,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
  [69414] = 11,
    ACTIONS(69), 1,
      anon_sym_pub,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(4870), 1,
      sym_crate,
    ACTIONS(4874), 1,
      sym_identifier,
    STATE(1061), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(3027), 1,
      sym_field_declaration,
    STATE(3407), 1,
      sym_visibility_modifier,
    STATE(2200), 2,
      sym_line_comment,
      sym_block_comment,
  [69449] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5027), 1,
      anon_sym_LBRACE,
    ACTIONS(5037), 1,
      anon_sym_SEMI,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    STATE(574), 1,
      sym_block,
    STATE(2456), 1,
      sym_where_clause,
    STATE(3615), 1,
      sym_label,
    STATE(2201), 2,
      sym_line_comment,
      sym_block_comment,
  [69484] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5027), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5041), 1,
      anon_sym_SEMI,
    STATE(751), 1,
      sym_block,
    STATE(2465), 1,
      sym_where_clause,
    STATE(3615), 1,
      sym_label,
    STATE(2202), 2,
      sym_line_comment,
      sym_block_comment,
  [69519] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    ACTIONS(5045), 1,
      anon_sym_LT,
    STATE(1106), 1,
      sym_declaration_list,
    STATE(2352), 1,
      sym_type_parameters,
    STATE(2583), 1,
      sym_trait_bounds,
    STATE(3327), 1,
      sym_where_clause,
    STATE(2203), 2,
      sym_line_comment,
      sym_block_comment,
  [69554] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    ACTIONS(5045), 1,
      anon_sym_LT,
    ACTIONS(5047), 1,
      anon_sym_SEMI,
    ACTIONS(5049), 1,
      anon_sym_EQ,
    STATE(2353), 1,
      sym_type_parameters,
    STATE(2966), 1,
      sym_trait_bounds,
    STATE(3640), 1,
      sym_where_clause,
    STATE(2204), 2,
      sym_line_comment,
      sym_block_comment,
  [69589] = 11,
    ACTIONS(69), 1,
      anon_sym_pub,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(4864), 1,
      sym_identifier,
    ACTIONS(4870), 1,
      sym_crate,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2198), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(3233), 1,
      sym_enum_variant,
    STATE(3612), 1,
      sym_visibility_modifier,
    STATE(2205), 2,
      sym_line_comment,
      sym_block_comment,
  [69624] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1519), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5051), 1,
      sym_identifier,
    ACTIONS(5053), 1,
      anon_sym_RBRACE,
    ACTIONS(5055), 1,
      anon_sym_COMMA,
    ACTIONS(5057), 1,
      anon_sym_ref,
    ACTIONS(5059), 1,
      sym_mutable_specifier,
    STATE(2206), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(2901), 2,
      sym_field_pattern,
      sym_remaining_field_pattern,
  [69657] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5027), 1,
      anon_sym_LBRACE,
    ACTIONS(5061), 1,
      anon_sym_SEMI,
    ACTIONS(5063), 1,
      anon_sym_DASH_GT,
    STATE(597), 1,
      sym_block,
    STATE(2413), 1,
      sym_where_clause,
    STATE(3615), 1,
      sym_label,
    STATE(2207), 2,
      sym_line_comment,
      sym_block_comment,
  [69692] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    ACTIONS(5045), 1,
      anon_sym_LT,
    STATE(636), 1,
      sym_declaration_list,
    STATE(2374), 1,
      sym_type_parameters,
    STATE(2646), 1,
      sym_trait_bounds,
    STATE(3339), 1,
      sym_where_clause,
    STATE(2208), 2,
      sym_line_comment,
      sym_block_comment,
  [69727] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1519), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5051), 1,
      sym_identifier,
    ACTIONS(5057), 1,
      anon_sym_ref,
    ACTIONS(5059), 1,
      sym_mutable_specifier,
    ACTIONS(5065), 1,
      anon_sym_RBRACE,
    ACTIONS(5067), 1,
      anon_sym_COMMA,
    STATE(2209), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(2776), 2,
      sym_field_pattern,
      sym_remaining_field_pattern,
  [69760] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1519), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5051), 1,
      sym_identifier,
    ACTIONS(5057), 1,
      anon_sym_ref,
    ACTIONS(5059), 1,
      sym_mutable_specifier,
    ACTIONS(5069), 1,
      anon_sym_RBRACE,
    ACTIONS(5071), 1,
      anon_sym_COMMA,
    STATE(2210), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(2777), 2,
      sym_field_pattern,
      sym_remaining_field_pattern,
  [69793] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5073), 1,
      anon_sym_SEMI,
    ACTIONS(5075), 1,
      anon_sym_LBRACE,
    ACTIONS(5077), 1,
      anon_sym_DASH_GT,
    STATE(1124), 1,
      sym_block,
    STATE(2424), 1,
      sym_where_clause,
    STATE(3618), 1,
      sym_label,
    STATE(2211), 2,
      sym_line_comment,
      sym_block_comment,
  [69828] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    ACTIONS(5045), 1,
      anon_sym_LT,
    STATE(1144), 1,
      sym_declaration_list,
    STATE(2386), 1,
      sym_type_parameters,
    STATE(2554), 1,
      sym_trait_bounds,
    STATE(3242), 1,
      sym_where_clause,
    STATE(2212), 2,
      sym_line_comment,
      sym_block_comment,
  [69863] = 11,
    ACTIONS(69), 1,
      anon_sym_pub,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(4864), 1,
      sym_identifier,
    ACTIONS(4870), 1,
      sym_crate,
    STATE(1061), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(3031), 1,
      sym_enum_variant,
    STATE(3612), 1,
      sym_visibility_modifier,
    STATE(2213), 2,
      sym_line_comment,
      sym_block_comment,
  [69898] = 11,
    ACTIONS(69), 1,
      anon_sym_pub,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(4870), 1,
      sym_crate,
    ACTIONS(4874), 1,
      sym_identifier,
    STATE(1061), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(3115), 1,
      sym_field_declaration,
    STATE(3407), 1,
      sym_visibility_modifier,
    STATE(2214), 2,
      sym_line_comment,
      sym_block_comment,
  [69933] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5075), 1,
      anon_sym_LBRACE,
    ACTIONS(5079), 1,
      anon_sym_SEMI,
    ACTIONS(5081), 1,
      anon_sym_DASH_GT,
    STATE(1165), 1,
      sym_block,
    STATE(2434), 1,
      sym_where_clause,
    STATE(3618), 1,
      sym_label,
    STATE(2215), 2,
      sym_line_comment,
      sym_block_comment,
  [69968] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5075), 1,
      anon_sym_LBRACE,
    ACTIONS(5083), 1,
      anon_sym_SEMI,
    ACTIONS(5085), 1,
      anon_sym_DASH_GT,
    STATE(1197), 1,
      sym_block,
    STATE(2438), 1,
      sym_where_clause,
    STATE(3618), 1,
      sym_label,
    STATE(2216), 2,
      sym_line_comment,
      sym_block_comment,
  [70003] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    ACTIONS(5045), 1,
      anon_sym_LT,
    STATE(1206), 1,
      sym_declaration_list,
    STATE(2311), 1,
      sym_type_parameters,
    STATE(2711), 1,
      sym_trait_bounds,
    STATE(3217), 1,
      sym_where_clause,
    STATE(2217), 2,
      sym_line_comment,
      sym_block_comment,
  [70038] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5075), 1,
      anon_sym_LBRACE,
    ACTIONS(5087), 1,
      anon_sym_SEMI,
    STATE(1216), 1,
      sym_block,
    STATE(2443), 1,
      sym_where_clause,
    STATE(3618), 1,
      sym_label,
    STATE(2218), 2,
      sym_line_comment,
      sym_block_comment,
  [70073] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2219), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4931), 8,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_where,
  [70094] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2220), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4931), 8,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_where,
  [70115] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5075), 1,
      anon_sym_LBRACE,
    ACTIONS(5089), 1,
      anon_sym_SEMI,
    ACTIONS(5091), 1,
      anon_sym_DASH_GT,
    STATE(1245), 1,
      sym_block,
    STATE(2448), 1,
      sym_where_clause,
    STATE(3618), 1,
      sym_label,
    STATE(2221), 2,
      sym_line_comment,
      sym_block_comment,
  [70150] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5075), 1,
      anon_sym_LBRACE,
    ACTIONS(5093), 1,
      anon_sym_SEMI,
    ACTIONS(5095), 1,
      anon_sym_DASH_GT,
    STATE(1259), 1,
      sym_block,
    STATE(2449), 1,
      sym_where_clause,
    STATE(3618), 1,
      sym_label,
    STATE(2222), 2,
      sym_line_comment,
      sym_block_comment,
  [70185] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5027), 1,
      anon_sym_LBRACE,
    ACTIONS(5097), 1,
      anon_sym_SEMI,
    ACTIONS(5099), 1,
      anon_sym_DASH_GT,
    STATE(729), 1,
      sym_block,
    STATE(2423), 1,
      sym_where_clause,
    STATE(3615), 1,
      sym_label,
    STATE(2223), 2,
      sym_line_comment,
      sym_block_comment,
  [70220] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2224), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4931), 8,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_where,
  [70241] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5075), 1,
      anon_sym_LBRACE,
    ACTIONS(5101), 1,
      anon_sym_SEMI,
    STATE(1267), 1,
      sym_block,
    STATE(2451), 1,
      sym_where_clause,
    STATE(3618), 1,
      sym_label,
    STATE(2225), 2,
      sym_line_comment,
      sym_block_comment,
  [70276] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5075), 1,
      anon_sym_LBRACE,
    ACTIONS(5103), 1,
      anon_sym_SEMI,
    STATE(1300), 1,
      sym_block,
    STATE(2454), 1,
      sym_where_clause,
    STATE(3618), 1,
      sym_label,
    STATE(2226), 2,
      sym_line_comment,
      sym_block_comment,
  [70311] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2227), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4931), 8,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_where,
  [70332] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2228), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4931), 8,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
      anon_sym_where,
  [70353] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5075), 1,
      anon_sym_LBRACE,
    ACTIONS(5105), 1,
      anon_sym_SEMI,
    STATE(1338), 1,
      sym_block,
    STATE(2457), 1,
      sym_where_clause,
    STATE(3618), 1,
      sym_label,
    STATE(2229), 2,
      sym_line_comment,
      sym_block_comment,
  [70388] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5075), 1,
      anon_sym_LBRACE,
    ACTIONS(5107), 1,
      anon_sym_SEMI,
    STATE(1348), 1,
      sym_block,
    STATE(2458), 1,
      sym_where_clause,
    STATE(3618), 1,
      sym_label,
    STATE(2230), 2,
      sym_line_comment,
      sym_block_comment,
  [70423] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5075), 1,
      anon_sym_LBRACE,
    ACTIONS(5109), 1,
      anon_sym_SEMI,
    STATE(1358), 1,
      sym_block,
    STATE(2459), 1,
      sym_where_clause,
    STATE(3618), 1,
      sym_label,
    STATE(2231), 2,
      sym_line_comment,
      sym_block_comment,
  [70458] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    ACTIONS(5045), 1,
      anon_sym_LT,
    STATE(739), 1,
      sym_declaration_list,
    STATE(2314), 1,
      sym_type_parameters,
    STATE(2666), 1,
      sym_trait_bounds,
    STATE(3176), 1,
      sym_where_clause,
    STATE(2232), 2,
      sym_line_comment,
      sym_block_comment,
  [70493] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1341), 1,
      aux_sym_string_literal_token1,
    STATE(2199), 1,
      sym_string_literal,
    STATE(2233), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4724), 6,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
  [70518] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5027), 1,
      anon_sym_LBRACE,
    ACTIONS(5111), 1,
      anon_sym_SEMI,
    ACTIONS(5113), 1,
      anon_sym_DASH_GT,
    STATE(677), 1,
      sym_block,
    STATE(2507), 1,
      sym_where_clause,
    STATE(3615), 1,
      sym_label,
    STATE(2234), 2,
      sym_line_comment,
      sym_block_comment,
  [70553] = 11,
    ACTIONS(69), 1,
      anon_sym_pub,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(4870), 1,
      sym_crate,
    ACTIONS(4874), 1,
      sym_identifier,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2214), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(3246), 1,
      sym_field_declaration,
    STATE(3407), 1,
      sym_visibility_modifier,
    STATE(2235), 2,
      sym_line_comment,
      sym_block_comment,
  [70588] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4768), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4888), 1,
      anon_sym_BANG,
    STATE(2236), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3458), 6,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
  [70613] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5027), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5115), 1,
      anon_sym_SEMI,
    STATE(515), 1,
      sym_block,
    STATE(2484), 1,
      sym_where_clause,
    STATE(3615), 1,
      sym_label,
    STATE(2237), 2,
      sym_line_comment,
      sym_block_comment,
  [70648] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4573), 1,
      anon_sym_trait,
    ACTIONS(5117), 1,
      anon_sym_impl,
    STATE(2238), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3458), 6,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
  [70673] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5119), 1,
      anon_sym_fn,
    ACTIONS(5121), 1,
      anon_sym_extern,
    STATE(2245), 1,
      aux_sym_function_modifiers_repeat1,
    STATE(2387), 1,
      sym_extern_modifier,
    STATE(2239), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4523), 4,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_unsafe,
  [70702] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5027), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5123), 1,
      anon_sym_SEMI,
    STATE(553), 1,
      sym_block,
    STATE(2441), 1,
      sym_where_clause,
    STATE(3615), 1,
      sym_label,
    STATE(2240), 2,
      sym_line_comment,
      sym_block_comment,
  [70737] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    ACTIONS(5045), 1,
      anon_sym_LT,
    STATE(595), 1,
      sym_declaration_list,
    STATE(2376), 1,
      sym_type_parameters,
    STATE(2521), 1,
      sym_trait_bounds,
    STATE(3071), 1,
      sym_where_clause,
    STATE(2241), 2,
      sym_line_comment,
      sym_block_comment,
  [70772] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4499), 1,
      anon_sym_BANG,
    ACTIONS(4507), 1,
      anon_sym_PIPE,
    ACTIONS(4509), 1,
      anon_sym_LPAREN,
    ACTIONS(4513), 1,
      anon_sym_COLON,
    ACTIONS(4517), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5125), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4519), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(2242), 2,
      sym_line_comment,
      sym_block_comment,
  [70805] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5027), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5127), 1,
      anon_sym_SEMI,
    STATE(564), 1,
      sym_block,
    STATE(2418), 1,
      sym_where_clause,
    STATE(3615), 1,
      sym_label,
    STATE(2243), 2,
      sym_line_comment,
      sym_block_comment,
  [70840] = 11,
    ACTIONS(69), 1,
      anon_sym_pub,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(4864), 1,
      sym_identifier,
    ACTIONS(4870), 1,
      sym_crate,
    STATE(1061), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2839), 1,
      sym_enum_variant,
    STATE(3612), 1,
      sym_visibility_modifier,
    STATE(2244), 2,
      sym_line_comment,
      sym_block_comment,
  [70875] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5132), 1,
      anon_sym_fn,
    ACTIONS(5134), 1,
      anon_sym_extern,
    STATE(2387), 1,
      sym_extern_modifier,
    STATE(2245), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_function_modifiers_repeat1,
    ACTIONS(5129), 4,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_unsafe,
  [70902] = 11,
    ACTIONS(69), 1,
      anon_sym_pub,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(4870), 1,
      sym_crate,
    ACTIONS(4874), 1,
      sym_identifier,
    STATE(1061), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2849), 1,
      sym_field_declaration,
    STATE(3407), 1,
      sym_visibility_modifier,
    STATE(2246), 2,
      sym_line_comment,
      sym_block_comment,
  [70937] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1519), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5051), 1,
      sym_identifier,
    ACTIONS(5057), 1,
      anon_sym_ref,
    ACTIONS(5059), 1,
      sym_mutable_specifier,
    ACTIONS(5137), 1,
      anon_sym_RBRACE,
    ACTIONS(5139), 1,
      anon_sym_COMMA,
    STATE(2247), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(2857), 2,
      sym_field_pattern,
      sym_remaining_field_pattern,
  [70970] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5027), 1,
      anon_sym_LBRACE,
    ACTIONS(5141), 1,
      anon_sym_SEMI,
    ACTIONS(5143), 1,
      anon_sym_DASH_GT,
    STATE(652), 1,
      sym_block,
    STATE(2416), 1,
      sym_where_clause,
    STATE(3615), 1,
      sym_label,
    STATE(2248), 2,
      sym_line_comment,
      sym_block_comment,
  [71005] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    ACTIONS(5045), 1,
      anon_sym_LT,
    ACTIONS(5145), 1,
      anon_sym_SEMI,
    ACTIONS(5147), 1,
      anon_sym_EQ,
    STATE(2388), 1,
      sym_type_parameters,
    STATE(2911), 1,
      sym_trait_bounds,
    STATE(3521), 1,
      sym_where_clause,
    STATE(2249), 2,
      sym_line_comment,
      sym_block_comment,
  [71040] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5027), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5149), 1,
      anon_sym_SEMI,
    STATE(708), 1,
      sym_block,
    STATE(2494), 1,
      sym_where_clause,
    STATE(3615), 1,
      sym_label,
    STATE(2250), 2,
      sym_line_comment,
      sym_block_comment,
  [71075] = 11,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5075), 1,
      anon_sym_LBRACE,
    ACTIONS(5151), 1,
      anon_sym_SEMI,
    ACTIONS(5153), 1,
      anon_sym_DASH_GT,
    STATE(1309), 1,
      sym_block,
    STATE(2455), 1,
      sym_where_clause,
    STATE(3618), 1,
      sym_label,
    STATE(2251), 2,
      sym_line_comment,
      sym_block_comment,
  [71110] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1519), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5051), 1,
      sym_identifier,
    ACTIONS(5057), 1,
      anon_sym_ref,
    ACTIONS(5059), 1,
      sym_mutable_specifier,
    ACTIONS(5155), 1,
      anon_sym_RBRACE,
    STATE(2252), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(3058), 2,
      sym_field_pattern,
      sym_remaining_field_pattern,
  [71140] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5157), 1,
      anon_sym_LPAREN,
    ACTIONS(5159), 1,
      anon_sym_LBRACK,
    ACTIONS(5161), 1,
      anon_sym_LBRACE,
    ACTIONS(5163), 1,
      anon_sym_RBRACE,
    STATE(2298), 1,
      aux_sym_macro_definition_repeat1,
    STATE(3322), 1,
      sym_macro_rule,
    STATE(3487), 1,
      sym_token_tree_pattern,
    STATE(2253), 2,
      sym_line_comment,
      sym_block_comment,
  [71172] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5157), 1,
      anon_sym_LPAREN,
    ACTIONS(5159), 1,
      anon_sym_LBRACK,
    ACTIONS(5161), 1,
      anon_sym_LBRACE,
    ACTIONS(5165), 1,
      anon_sym_RPAREN,
    STATE(2186), 1,
      aux_sym_macro_definition_repeat1,
    STATE(3144), 1,
      sym_macro_rule,
    STATE(3487), 1,
      sym_token_tree_pattern,
    STATE(2254), 2,
      sym_line_comment,
      sym_block_comment,
  [71204] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5157), 1,
      anon_sym_LPAREN,
    ACTIONS(5159), 1,
      anon_sym_LBRACK,
    ACTIONS(5161), 1,
      anon_sym_LBRACE,
    ACTIONS(5167), 1,
      anon_sym_RBRACK,
    STATE(2186), 1,
      aux_sym_macro_definition_repeat1,
    STATE(3154), 1,
      sym_macro_rule,
    STATE(3487), 1,
      sym_token_tree_pattern,
    STATE(2255), 2,
      sym_line_comment,
      sym_block_comment,
  [71236] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5169), 1,
      sym_identifier,
    STATE(2256), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4772), 6,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
  [71258] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5157), 1,
      anon_sym_LPAREN,
    ACTIONS(5159), 1,
      anon_sym_LBRACK,
    ACTIONS(5161), 1,
      anon_sym_LBRACE,
    ACTIONS(5171), 1,
      anon_sym_RBRACE,
    STATE(2301), 1,
      aux_sym_macro_definition_repeat1,
    STATE(3097), 1,
      sym_macro_rule,
    STATE(3487), 1,
      sym_token_tree_pattern,
    STATE(2257), 2,
      sym_line_comment,
      sym_block_comment,
  [71290] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5173), 1,
      sym_identifier,
    STATE(2258), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4772), 6,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
  [71312] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4940), 1,
      anon_sym_LPAREN,
    ACTIONS(4942), 1,
      anon_sym_LBRACE,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5175), 1,
      anon_sym_SEMI,
    STATE(735), 1,
      sym_field_declaration_list,
    STATE(2924), 1,
      sym_ordered_field_declaration_list,
    STATE(3118), 1,
      sym_where_clause,
    STATE(2259), 2,
      sym_line_comment,
      sym_block_comment,
  [71344] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4940), 1,
      anon_sym_LPAREN,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4976), 1,
      anon_sym_LBRACE,
    ACTIONS(5177), 1,
      anon_sym_SEMI,
    STATE(1203), 1,
      sym_field_declaration_list,
    STATE(2854), 1,
      sym_ordered_field_declaration_list,
    STATE(3197), 1,
      sym_where_clause,
    STATE(2260), 2,
      sym_line_comment,
      sym_block_comment,
  [71376] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4768), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5179), 1,
      anon_sym_LPAREN,
    ACTIONS(5181), 1,
      anon_sym_LBRACK,
    ACTIONS(5183), 1,
      anon_sym_RBRACK,
    ACTIONS(5185), 1,
      anon_sym_LBRACE,
    ACTIONS(5187), 1,
      anon_sym_EQ,
    STATE(3581), 1,
      sym_delim_token_tree,
    STATE(2261), 2,
      sym_line_comment,
      sym_block_comment,
  [71408] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1519), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5051), 1,
      sym_identifier,
    ACTIONS(5057), 1,
      anon_sym_ref,
    ACTIONS(5059), 1,
      sym_mutable_specifier,
    ACTIONS(5189), 1,
      anon_sym_RBRACE,
    STATE(2262), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(3058), 2,
      sym_field_pattern,
      sym_remaining_field_pattern,
  [71438] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1519), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5051), 1,
      sym_identifier,
    ACTIONS(5057), 1,
      anon_sym_ref,
    ACTIONS(5059), 1,
      sym_mutable_specifier,
    ACTIONS(5191), 1,
      anon_sym_RBRACE,
    STATE(2263), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(3058), 2,
      sym_field_pattern,
      sym_remaining_field_pattern,
  [71468] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5157), 1,
      anon_sym_LPAREN,
    ACTIONS(5159), 1,
      anon_sym_LBRACK,
    ACTIONS(5161), 1,
      anon_sym_LBRACE,
    ACTIONS(5165), 1,
      anon_sym_RBRACK,
    STATE(2186), 1,
      aux_sym_macro_definition_repeat1,
    STATE(3146), 1,
      sym_macro_rule,
    STATE(3487), 1,
      sym_token_tree_pattern,
    STATE(2264), 2,
      sym_line_comment,
      sym_block_comment,
  [71500] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5193), 1,
      sym_identifier,
    STATE(2265), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4772), 6,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
  [71522] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4175), 1,
      anon_sym_LBRACE,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(5197), 1,
      anon_sym_STAR,
    STATE(2819), 1,
      sym_use_list,
    STATE(3311), 1,
      sym_type_arguments,
    ACTIONS(5195), 2,
      sym_identifier,
      sym_super,
    STATE(2266), 2,
      sym_line_comment,
      sym_block_comment,
  [71552] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5157), 1,
      anon_sym_LPAREN,
    ACTIONS(5159), 1,
      anon_sym_LBRACK,
    ACTIONS(5161), 1,
      anon_sym_LBRACE,
    ACTIONS(5167), 1,
      anon_sym_RPAREN,
    STATE(2186), 1,
      aux_sym_macro_definition_repeat1,
    STATE(3150), 1,
      sym_macro_rule,
    STATE(3487), 1,
      sym_token_tree_pattern,
    STATE(2267), 2,
      sym_line_comment,
      sym_block_comment,
  [71584] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4756), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5179), 1,
      anon_sym_LPAREN,
    ACTIONS(5181), 1,
      anon_sym_LBRACK,
    ACTIONS(5185), 1,
      anon_sym_LBRACE,
    ACTIONS(5199), 1,
      anon_sym_RBRACK,
    ACTIONS(5201), 1,
      anon_sym_EQ,
    STATE(3503), 1,
      sym_delim_token_tree,
    STATE(2268), 2,
      sym_line_comment,
      sym_block_comment,
  [71616] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1519), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5051), 1,
      sym_identifier,
    ACTIONS(5057), 1,
      anon_sym_ref,
    ACTIONS(5059), 1,
      sym_mutable_specifier,
    ACTIONS(5203), 1,
      anon_sym_RBRACE,
    STATE(2269), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(3058), 2,
      sym_field_pattern,
      sym_remaining_field_pattern,
  [71646] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4940), 1,
      anon_sym_LPAREN,
    ACTIONS(4976), 1,
      anon_sym_LBRACE,
    ACTIONS(5207), 1,
      anon_sym_EQ,
    ACTIONS(5205), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(2270), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(3029), 2,
      sym_field_declaration_list,
      sym_ordered_field_declaration_list,
  [71674] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5157), 1,
      anon_sym_LPAREN,
    ACTIONS(5159), 1,
      anon_sym_LBRACK,
    ACTIONS(5161), 1,
      anon_sym_LBRACE,
    ACTIONS(5209), 1,
      anon_sym_RBRACE,
    STATE(2186), 1,
      aux_sym_macro_definition_repeat1,
    STATE(3164), 1,
      sym_macro_rule,
    STATE(3487), 1,
      sym_token_tree_pattern,
    STATE(2271), 2,
      sym_line_comment,
      sym_block_comment,
  [71706] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4940), 1,
      anon_sym_LPAREN,
    ACTIONS(4942), 1,
      anon_sym_LBRACE,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5211), 1,
      anon_sym_SEMI,
    STATE(615), 1,
      sym_field_declaration_list,
    STATE(2971), 1,
      sym_ordered_field_declaration_list,
    STATE(3205), 1,
      sym_where_clause,
    STATE(2272), 2,
      sym_line_comment,
      sym_block_comment,
  [71738] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4940), 1,
      anon_sym_LPAREN,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4976), 1,
      anon_sym_LBRACE,
    ACTIONS(5213), 1,
      anon_sym_SEMI,
    STATE(1133), 1,
      sym_field_declaration_list,
    STATE(2803), 1,
      sym_ordered_field_declaration_list,
    STATE(3225), 1,
      sym_where_clause,
    STATE(2273), 2,
      sym_line_comment,
      sym_block_comment,
  [71770] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5157), 1,
      anon_sym_LPAREN,
    ACTIONS(5159), 1,
      anon_sym_LBRACK,
    ACTIONS(5161), 1,
      anon_sym_LBRACE,
    ACTIONS(5215), 1,
      anon_sym_RPAREN,
    STATE(2281), 1,
      aux_sym_macro_definition_repeat1,
    STATE(3243), 1,
      sym_macro_rule,
    STATE(3487), 1,
      sym_token_tree_pattern,
    STATE(2274), 2,
      sym_line_comment,
      sym_block_comment,
  [71802] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5157), 1,
      anon_sym_LPAREN,
    ACTIONS(5159), 1,
      anon_sym_LBRACK,
    ACTIONS(5161), 1,
      anon_sym_LBRACE,
    ACTIONS(5215), 1,
      anon_sym_RBRACK,
    STATE(2282), 1,
      aux_sym_macro_definition_repeat1,
    STATE(3244), 1,
      sym_macro_rule,
    STATE(3487), 1,
      sym_token_tree_pattern,
    STATE(2275), 2,
      sym_line_comment,
      sym_block_comment,
  [71834] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5157), 1,
      anon_sym_LPAREN,
    ACTIONS(5159), 1,
      anon_sym_LBRACK,
    ACTIONS(5161), 1,
      anon_sym_LBRACE,
    ACTIONS(5217), 1,
      anon_sym_RPAREN,
    STATE(2283), 1,
      aux_sym_macro_definition_repeat1,
    STATE(3245), 1,
      sym_macro_rule,
    STATE(3487), 1,
      sym_token_tree_pattern,
    STATE(2276), 2,
      sym_line_comment,
      sym_block_comment,
  [71866] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5157), 1,
      anon_sym_LPAREN,
    ACTIONS(5159), 1,
      anon_sym_LBRACK,
    ACTIONS(5161), 1,
      anon_sym_LBRACE,
    ACTIONS(5217), 1,
      anon_sym_RBRACK,
    STATE(2284), 1,
      aux_sym_macro_definition_repeat1,
    STATE(3247), 1,
      sym_macro_rule,
    STATE(3487), 1,
      sym_token_tree_pattern,
    STATE(2277), 2,
      sym_line_comment,
      sym_block_comment,
  [71898] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5157), 1,
      anon_sym_LPAREN,
    ACTIONS(5159), 1,
      anon_sym_LBRACK,
    ACTIONS(5161), 1,
      anon_sym_LBRACE,
    ACTIONS(5219), 1,
      anon_sym_RBRACE,
    STATE(2186), 1,
      aux_sym_macro_definition_repeat1,
    STATE(3147), 1,
      sym_macro_rule,
    STATE(3487), 1,
      sym_token_tree_pattern,
    STATE(2278), 2,
      sym_line_comment,
      sym_block_comment,
  [71930] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4175), 1,
      anon_sym_LBRACE,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(5197), 1,
      anon_sym_STAR,
    STATE(2819), 1,
      sym_use_list,
    STATE(3158), 1,
      sym_type_arguments,
    ACTIONS(5195), 2,
      sym_identifier,
      sym_super,
    STATE(2279), 2,
      sym_line_comment,
      sym_block_comment,
  [71960] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4517), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5223), 1,
      anon_sym_COLON,
    ACTIONS(5225), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4519), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(5221), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(2280), 2,
      sym_line_comment,
      sym_block_comment,
  [71988] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5157), 1,
      anon_sym_LPAREN,
    ACTIONS(5159), 1,
      anon_sym_LBRACK,
    ACTIONS(5161), 1,
      anon_sym_LBRACE,
    ACTIONS(5227), 1,
      anon_sym_RPAREN,
    STATE(2186), 1,
      aux_sym_macro_definition_repeat1,
    STATE(3264), 1,
      sym_macro_rule,
    STATE(3487), 1,
      sym_token_tree_pattern,
    STATE(2281), 2,
      sym_line_comment,
      sym_block_comment,
  [72020] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5157), 1,
      anon_sym_LPAREN,
    ACTIONS(5159), 1,
      anon_sym_LBRACK,
    ACTIONS(5161), 1,
      anon_sym_LBRACE,
    ACTIONS(5227), 1,
      anon_sym_RBRACK,
    STATE(2186), 1,
      aux_sym_macro_definition_repeat1,
    STATE(3265), 1,
      sym_macro_rule,
    STATE(3487), 1,
      sym_token_tree_pattern,
    STATE(2282), 2,
      sym_line_comment,
      sym_block_comment,
  [72052] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5157), 1,
      anon_sym_LPAREN,
    ACTIONS(5159), 1,
      anon_sym_LBRACK,
    ACTIONS(5161), 1,
      anon_sym_LBRACE,
    ACTIONS(5229), 1,
      anon_sym_RPAREN,
    STATE(2186), 1,
      aux_sym_macro_definition_repeat1,
    STATE(3266), 1,
      sym_macro_rule,
    STATE(3487), 1,
      sym_token_tree_pattern,
    STATE(2283), 2,
      sym_line_comment,
      sym_block_comment,
  [72084] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5157), 1,
      anon_sym_LPAREN,
    ACTIONS(5159), 1,
      anon_sym_LBRACK,
    ACTIONS(5161), 1,
      anon_sym_LBRACE,
    ACTIONS(5229), 1,
      anon_sym_RBRACK,
    STATE(2186), 1,
      aux_sym_macro_definition_repeat1,
    STATE(3267), 1,
      sym_macro_rule,
    STATE(3487), 1,
      sym_token_tree_pattern,
    STATE(2284), 2,
      sym_line_comment,
      sym_block_comment,
  [72116] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4517), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5223), 1,
      anon_sym_COLON,
    ACTIONS(5231), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4519), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(5221), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(2285), 2,
      sym_line_comment,
      sym_block_comment,
  [72144] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4517), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5231), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4519), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(2286), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3321), 3,
      anon_sym_RPAREN,
      anon_sym_PLUS,
      anon_sym_COMMA,
  [72170] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4890), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4894), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4892), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(2287), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4549), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [72196] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5233), 1,
      anon_sym_trait,
    STATE(2288), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3458), 6,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
  [72218] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4517), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5237), 1,
      anon_sym_COLON,
    ACTIONS(5239), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4519), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(5235), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(2289), 2,
      sym_line_comment,
      sym_block_comment,
  [72246] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1519), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5051), 1,
      sym_identifier,
    ACTIONS(5057), 1,
      anon_sym_ref,
    ACTIONS(5059), 1,
      sym_mutable_specifier,
    ACTIONS(5241), 1,
      anon_sym_RBRACE,
    STATE(2290), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(3058), 2,
      sym_field_pattern,
      sym_remaining_field_pattern,
  [72276] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1519), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5051), 1,
      sym_identifier,
    ACTIONS(5057), 1,
      anon_sym_ref,
    ACTIONS(5059), 1,
      sym_mutable_specifier,
    ACTIONS(5243), 1,
      anon_sym_RBRACE,
    STATE(2291), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(3058), 2,
      sym_field_pattern,
      sym_remaining_field_pattern,
  [72306] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5157), 1,
      anon_sym_LPAREN,
    ACTIONS(5159), 1,
      anon_sym_LBRACK,
    ACTIONS(5161), 1,
      anon_sym_LBRACE,
    ACTIONS(5245), 1,
      anon_sym_RPAREN,
    STATE(2254), 1,
      aux_sym_macro_definition_repeat1,
    STATE(3145), 1,
      sym_macro_rule,
    STATE(3487), 1,
      sym_token_tree_pattern,
    STATE(2292), 2,
      sym_line_comment,
      sym_block_comment,
  [72338] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5157), 1,
      anon_sym_LPAREN,
    ACTIONS(5159), 1,
      anon_sym_LBRACK,
    ACTIONS(5161), 1,
      anon_sym_LBRACE,
    ACTIONS(5245), 1,
      anon_sym_RBRACK,
    STATE(2264), 1,
      aux_sym_macro_definition_repeat1,
    STATE(3100), 1,
      sym_macro_rule,
    STATE(3487), 1,
      sym_token_tree_pattern,
    STATE(2293), 2,
      sym_line_comment,
      sym_block_comment,
  [72370] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5157), 1,
      anon_sym_LPAREN,
    ACTIONS(5159), 1,
      anon_sym_LBRACK,
    ACTIONS(5161), 1,
      anon_sym_LBRACE,
    ACTIONS(5247), 1,
      anon_sym_RBRACE,
    STATE(2278), 1,
      aux_sym_macro_definition_repeat1,
    STATE(3061), 1,
      sym_macro_rule,
    STATE(3487), 1,
      sym_token_tree_pattern,
    STATE(2294), 2,
      sym_line_comment,
      sym_block_comment,
  [72402] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5157), 1,
      anon_sym_LPAREN,
    ACTIONS(5159), 1,
      anon_sym_LBRACK,
    ACTIONS(5161), 1,
      anon_sym_LBRACE,
    ACTIONS(5249), 1,
      anon_sym_RPAREN,
    STATE(2267), 1,
      aux_sym_macro_definition_repeat1,
    STATE(3300), 1,
      sym_macro_rule,
    STATE(3487), 1,
      sym_token_tree_pattern,
    STATE(2295), 2,
      sym_line_comment,
      sym_block_comment,
  [72434] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5157), 1,
      anon_sym_LPAREN,
    ACTIONS(5159), 1,
      anon_sym_LBRACK,
    ACTIONS(5161), 1,
      anon_sym_LBRACE,
    ACTIONS(5249), 1,
      anon_sym_RBRACK,
    STATE(2255), 1,
      aux_sym_macro_definition_repeat1,
    STATE(3219), 1,
      sym_macro_rule,
    STATE(3487), 1,
      sym_token_tree_pattern,
    STATE(2296), 2,
      sym_line_comment,
      sym_block_comment,
  [72466] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5157), 1,
      anon_sym_LPAREN,
    ACTIONS(5159), 1,
      anon_sym_LBRACK,
    ACTIONS(5161), 1,
      anon_sym_LBRACE,
    ACTIONS(5251), 1,
      anon_sym_RBRACE,
    STATE(2271), 1,
      aux_sym_macro_definition_repeat1,
    STATE(3223), 1,
      sym_macro_rule,
    STATE(3487), 1,
      sym_token_tree_pattern,
    STATE(2297), 2,
      sym_line_comment,
      sym_block_comment,
  [72498] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5157), 1,
      anon_sym_LPAREN,
    ACTIONS(5159), 1,
      anon_sym_LBRACK,
    ACTIONS(5161), 1,
      anon_sym_LBRACE,
    ACTIONS(5253), 1,
      anon_sym_RBRACE,
    STATE(2186), 1,
      aux_sym_macro_definition_repeat1,
    STATE(3062), 1,
      sym_macro_rule,
    STATE(3487), 1,
      sym_token_tree_pattern,
    STATE(2298), 2,
      sym_line_comment,
      sym_block_comment,
  [72530] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4940), 1,
      anon_sym_LPAREN,
    ACTIONS(4976), 1,
      anon_sym_LBRACE,
    ACTIONS(5257), 1,
      anon_sym_EQ,
    ACTIONS(5255), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(2299), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(3023), 2,
      sym_field_declaration_list,
      sym_ordered_field_declaration_list,
  [72558] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4517), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5259), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4519), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(2300), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3321), 3,
      anon_sym_SEMI,
      anon_sym_RBRACK,
      anon_sym_PLUS,
  [72584] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5157), 1,
      anon_sym_LPAREN,
    ACTIONS(5159), 1,
      anon_sym_LBRACK,
    ACTIONS(5161), 1,
      anon_sym_LBRACE,
    ACTIONS(5261), 1,
      anon_sym_RBRACE,
    STATE(2186), 1,
      aux_sym_macro_definition_repeat1,
    STATE(3063), 1,
      sym_macro_rule,
    STATE(3487), 1,
      sym_token_tree_pattern,
    STATE(2301), 2,
      sym_line_comment,
      sym_block_comment,
  [72616] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1519), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5051), 1,
      sym_identifier,
    ACTIONS(5057), 1,
      anon_sym_ref,
    ACTIONS(5059), 1,
      sym_mutable_specifier,
    ACTIONS(5263), 1,
      anon_sym_RBRACE,
    STATE(2302), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(3058), 2,
      sym_field_pattern,
      sym_remaining_field_pattern,
  [72646] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5265), 1,
      aux_sym_line_comment_token1,
    ACTIONS(5267), 1,
      aux_sym_line_comment_token3,
    ACTIONS(5269), 1,
      anon_sym_BANG2,
    ACTIONS(5271), 1,
      anon_sym_SLASH2,
    STATE(3395), 1,
      sym__outer_line_doc_comment_marker,
    STATE(3562), 1,
      sym__line_doc_comment_marker,
    STATE(3638), 1,
      sym__inner_line_doc_comment_marker,
    STATE(2303), 2,
      sym_line_comment,
      sym_block_comment,
  [72678] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5273), 1,
      anon_sym_trait,
    STATE(2304), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3458), 6,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
  [72700] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4758), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5179), 1,
      anon_sym_LPAREN,
    ACTIONS(5181), 1,
      anon_sym_LBRACK,
    ACTIONS(5185), 1,
      anon_sym_LBRACE,
    ACTIONS(5199), 1,
      anon_sym_RBRACK,
    ACTIONS(5201), 1,
      anon_sym_EQ,
    STATE(3503), 1,
      sym_delim_token_tree,
    STATE(2305), 2,
      sym_line_comment,
      sym_block_comment,
  [72732] = 10,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4630), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5179), 1,
      anon_sym_LPAREN,
    ACTIONS(5181), 1,
      anon_sym_LBRACK,
    ACTIONS(5185), 1,
      anon_sym_LBRACE,
    ACTIONS(5199), 1,
      anon_sym_RBRACK,
    ACTIONS(5201), 1,
      anon_sym_EQ,
    STATE(3503), 1,
      sym_delim_token_tree,
    STATE(2306), 2,
      sym_line_comment,
      sym_block_comment,
  [72764] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4517), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5225), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4519), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(2307), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3321), 3,
      anon_sym_RPAREN,
      anon_sym_PLUS,
      anon_sym_COMMA,
  [72790] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5275), 1,
      sym_identifier,
    STATE(2308), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4772), 6,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
  [72812] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1519), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5051), 1,
      sym_identifier,
    ACTIONS(5057), 1,
      anon_sym_ref,
    ACTIONS(5059), 1,
      sym_mutable_specifier,
    ACTIONS(5277), 1,
      anon_sym_RBRACE,
    STATE(2309), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(3058), 2,
      sym_field_pattern,
      sym_remaining_field_pattern,
  [72842] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5279), 1,
      anon_sym_SEMI,
    STATE(1218), 1,
      sym_declaration_list,
    STATE(2874), 1,
      sym_where_clause,
    STATE(2310), 2,
      sym_line_comment,
      sym_block_comment,
  [72871] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    STATE(1257), 1,
      sym_declaration_list,
    STATE(2523), 1,
      sym_trait_bounds,
    STATE(3290), 1,
      sym_where_clause,
    STATE(2311), 2,
      sym_line_comment,
      sym_block_comment,
  [72900] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5281), 1,
      anon_sym_SEMI,
    STATE(635), 1,
      sym_declaration_list,
    STATE(2780), 1,
      sym_where_clause,
    STATE(2312), 2,
      sym_line_comment,
      sym_block_comment,
  [72929] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5283), 1,
      anon_sym_SEMI,
    STATE(504), 1,
      sym_declaration_list,
    STATE(2940), 1,
      sym_where_clause,
    STATE(2313), 2,
      sym_line_comment,
      sym_block_comment,
  [72958] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    STATE(682), 1,
      sym_declaration_list,
    STATE(2727), 1,
      sym_trait_bounds,
    STATE(3294), 1,
      sym_where_clause,
    STATE(2314), 2,
      sym_line_comment,
      sym_block_comment,
  [72987] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4595), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4960), 1,
      anon_sym_for,
    STATE(2315), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3321), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [73010] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4944), 1,
      anon_sym_LT,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5285), 1,
      anon_sym_LBRACE,
    STATE(1097), 1,
      sym_enum_variant_list,
    STATE(2736), 1,
      sym_type_parameters,
    STATE(3309), 1,
      sym_where_clause,
    STATE(2316), 2,
      sym_line_comment,
      sym_block_comment,
  [73039] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5287), 1,
      anon_sym_SEMI,
    STATE(1100), 1,
      sym_declaration_list,
    STATE(3045), 1,
      sym_where_clause,
    STATE(2317), 2,
      sym_line_comment,
      sym_block_comment,
  [73068] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5289), 1,
      anon_sym_SEMI,
    STATE(506), 1,
      sym_declaration_list,
    STATE(2948), 1,
      sym_where_clause,
    STATE(2318), 2,
      sym_line_comment,
      sym_block_comment,
  [73097] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5291), 1,
      anon_sym_SEMI,
    STATE(1275), 1,
      sym_declaration_list,
    STATE(2889), 1,
      sym_where_clause,
    STATE(2319), 2,
      sym_line_comment,
      sym_block_comment,
  [73126] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5293), 1,
      anon_sym_SEMI,
    STATE(1277), 1,
      sym_declaration_list,
    STATE(2890), 1,
      sym_where_clause,
    STATE(2320), 2,
      sym_line_comment,
      sym_block_comment,
  [73155] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4672), 1,
      anon_sym_DOT_DOT,
    STATE(2321), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4670), 5,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_if,
  [73176] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5295), 1,
      anon_sym_SEMI,
    STATE(1289), 1,
      sym_declaration_list,
    STATE(2892), 1,
      sym_where_clause,
    STATE(2322), 2,
      sym_line_comment,
      sym_block_comment,
  [73205] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5297), 1,
      anon_sym_SEMI,
    STATE(1291), 1,
      sym_declaration_list,
    STATE(2893), 1,
      sym_where_clause,
    STATE(2323), 2,
      sym_line_comment,
      sym_block_comment,
  [73234] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5299), 1,
      anon_sym_SEMI,
    STATE(1293), 1,
      sym_declaration_list,
    STATE(2894), 1,
      sym_where_clause,
    STATE(2324), 2,
      sym_line_comment,
      sym_block_comment,
  [73263] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5301), 1,
      anon_sym_SEMI,
    STATE(1295), 1,
      sym_declaration_list,
    STATE(2895), 1,
      sym_where_clause,
    STATE(2325), 2,
      sym_line_comment,
      sym_block_comment,
  [73292] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5303), 1,
      anon_sym_SEMI,
    STATE(508), 1,
      sym_declaration_list,
    STATE(2955), 1,
      sym_where_clause,
    STATE(2326), 2,
      sym_line_comment,
      sym_block_comment,
  [73321] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5305), 1,
      anon_sym_SEMI,
    STATE(510), 1,
      sym_declaration_list,
    STATE(2957), 1,
      sym_where_clause,
    STATE(2327), 2,
      sym_line_comment,
      sym_block_comment,
  [73350] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4944), 1,
      anon_sym_LT,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4976), 1,
      anon_sym_LBRACE,
    STATE(1108), 1,
      sym_field_declaration_list,
    STATE(2682), 1,
      sym_type_parameters,
    STATE(3317), 1,
      sym_where_clause,
    STATE(2328), 2,
      sym_line_comment,
      sym_block_comment,
  [73379] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5307), 1,
      anon_sym_RBRACK,
    ACTIONS(4862), 2,
      anon_sym_PIPE,
      anon_sym_COMMA,
    STATE(2329), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3581), 3,
      anon_sym_SEMI,
      anon_sym_PLUS,
      anon_sym_DASH_GT,
  [73402] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4664), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4666), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(2330), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4507), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [73425] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5310), 1,
      anon_sym_SEMI,
    STATE(1330), 1,
      sym_declaration_list,
    STATE(2898), 1,
      sym_where_clause,
    STATE(2331), 2,
      sym_line_comment,
      sym_block_comment,
  [73454] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5312), 1,
      anon_sym_SEMI,
    STATE(1332), 1,
      sym_declaration_list,
    STATE(2899), 1,
      sym_where_clause,
    STATE(2332), 2,
      sym_line_comment,
      sym_block_comment,
  [73483] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5314), 1,
      anon_sym_RBRACK,
    ACTIONS(4896), 2,
      anon_sym_PIPE,
      anon_sym_COMMA,
    STATE(2333), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3607), 3,
      anon_sym_SEMI,
      anon_sym_PLUS,
      anon_sym_DASH_GT,
  [73506] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4612), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5317), 1,
      anon_sym_GT,
    ACTIONS(5319), 1,
      anon_sym_COMMA,
    STATE(2826), 1,
      aux_sym_type_parameters_repeat1,
    ACTIONS(3321), 2,
      anon_sym_PLUS,
      anon_sym_as,
    STATE(2334), 2,
      sym_line_comment,
      sym_block_comment,
  [73533] = 8,
    ACTIONS(19), 1,
      anon_sym_LBRACE,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(5321), 1,
      anon_sym_if,
    STATE(3355), 1,
      sym_label,
    STATE(404), 2,
      sym_if_expression,
      sym_block,
    STATE(2335), 2,
      sym_line_comment,
      sym_block_comment,
  [73560] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3607), 2,
      anon_sym_PLUS,
      anon_sym_DASH_GT,
    ACTIONS(4896), 2,
      anon_sym_COLON,
      anon_sym_PIPE,
    ACTIONS(5314), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(2336), 2,
      sym_line_comment,
      sym_block_comment,
  [73583] = 9,
    ACTIONS(27), 1,
      anon_sym_PIPE,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5323), 1,
      sym_identifier,
    ACTIONS(5325), 1,
      anon_sym_ref,
    ACTIONS(5327), 1,
      sym_mutable_specifier,
    ACTIONS(5329), 1,
      anon_sym_move,
    STATE(243), 1,
      sym_closure_parameters,
    STATE(2337), 2,
      sym_line_comment,
      sym_block_comment,
  [73612] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3093), 1,
      anon_sym_PLUS,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    ACTIONS(5317), 1,
      anon_sym_GT,
    ACTIONS(5319), 1,
      anon_sym_COMMA,
    STATE(2826), 1,
      aux_sym_type_parameters_repeat1,
    STATE(2832), 1,
      sym_trait_bounds,
    STATE(2338), 2,
      sym_line_comment,
      sym_block_comment,
  [73641] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    ACTIONS(5331), 1,
      anon_sym_PLUS,
    ACTIONS(5333), 1,
      anon_sym_GT,
    ACTIONS(5335), 1,
      anon_sym_COMMA,
    STATE(2973), 1,
      sym_trait_bounds,
    STATE(2983), 1,
      aux_sym_type_arguments_repeat1,
    STATE(2339), 2,
      sym_line_comment,
      sym_block_comment,
  [73670] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    ACTIONS(5331), 1,
      anon_sym_PLUS,
    ACTIONS(5337), 1,
      anon_sym_GT,
    ACTIONS(5339), 1,
      anon_sym_COMMA,
    STATE(2925), 1,
      sym_trait_bounds,
    STATE(2926), 1,
      aux_sym_type_arguments_repeat1,
    STATE(2340), 2,
      sym_line_comment,
      sym_block_comment,
  [73699] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3093), 1,
      anon_sym_PLUS,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    ACTIONS(5337), 1,
      anon_sym_GT,
    ACTIONS(5339), 1,
      anon_sym_COMMA,
    STATE(2925), 1,
      sym_trait_bounds,
    STATE(2926), 1,
      aux_sym_type_arguments_repeat1,
    STATE(2341), 2,
      sym_line_comment,
      sym_block_comment,
  [73728] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1229), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(5341), 1,
      anon_sym_if,
    STATE(3619), 1,
      sym_label,
    STATE(459), 2,
      sym_if_expression,
      sym_block,
    STATE(2342), 2,
      sym_line_comment,
      sym_block_comment,
  [73755] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4634), 1,
      anon_sym_EQ,
    ACTIONS(4636), 1,
      anon_sym_GT,
    ACTIONS(4638), 1,
      anon_sym_COMMA,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    STATE(2809), 1,
      sym_trait_bounds,
    STATE(2811), 1,
      aux_sym_type_parameters_repeat1,
    STATE(2343), 2,
      sym_line_comment,
      sym_block_comment,
  [73784] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4634), 1,
      anon_sym_EQ,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    ACTIONS(5343), 1,
      anon_sym_GT,
    ACTIONS(5345), 1,
      anon_sym_COMMA,
    STATE(2809), 1,
      sym_trait_bounds,
    STATE(2865), 1,
      aux_sym_type_parameters_repeat1,
    STATE(2344), 2,
      sym_line_comment,
      sym_block_comment,
  [73813] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5347), 1,
      anon_sym_SEMI,
    STATE(685), 1,
      sym_declaration_list,
    STATE(2908), 1,
      sym_where_clause,
    STATE(2345), 2,
      sym_line_comment,
      sym_block_comment,
  [73842] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5349), 1,
      anon_sym_SEMI,
    STATE(1126), 1,
      sym_declaration_list,
    STATE(2799), 1,
      sym_where_clause,
    STATE(2346), 2,
      sym_line_comment,
      sym_block_comment,
  [73871] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2347), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5001), 6,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
  [73890] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4942), 1,
      anon_sym_LBRACE,
    ACTIONS(4944), 1,
      anon_sym_LT,
    ACTIONS(4946), 1,
      anon_sym_where,
    STATE(608), 1,
      sym_field_declaration_list,
    STATE(2579), 1,
      sym_type_parameters,
    STATE(3105), 1,
      sym_where_clause,
    STATE(2348), 2,
      sym_line_comment,
      sym_block_comment,
  [73919] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(407), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(5351), 1,
      anon_sym_if,
    STATE(3620), 1,
      sym_label,
    STATE(1780), 2,
      sym_if_expression,
      sym_block,
    STATE(2349), 2,
      sym_line_comment,
      sym_block_comment,
  [73946] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5353), 1,
      anon_sym_SEMI,
    STATE(545), 1,
      sym_declaration_list,
    STATE(3054), 1,
      sym_where_clause,
    STATE(2350), 2,
      sym_line_comment,
      sym_block_comment,
  [73975] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5355), 1,
      anon_sym_SEMI,
    STATE(547), 1,
      sym_declaration_list,
    STATE(2921), 1,
      sym_where_clause,
    STATE(2351), 2,
      sym_line_comment,
      sym_block_comment,
  [74004] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    STATE(1136), 1,
      sym_declaration_list,
    STATE(2547), 1,
      sym_trait_bounds,
    STATE(3229), 1,
      sym_where_clause,
    STATE(2352), 2,
      sym_line_comment,
      sym_block_comment,
  [74033] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    ACTIONS(5357), 1,
      anon_sym_SEMI,
    ACTIONS(5359), 1,
      anon_sym_EQ,
    STATE(2805), 1,
      sym_trait_bounds,
    STATE(3408), 1,
      sym_where_clause,
    STATE(2353), 2,
      sym_line_comment,
      sym_block_comment,
  [74062] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5361), 1,
      anon_sym_SEMI,
    STATE(1143), 1,
      sym_declaration_list,
    STATE(2806), 1,
      sym_where_clause,
    STATE(2354), 2,
      sym_line_comment,
      sym_block_comment,
  [74091] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4944), 1,
      anon_sym_LT,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5363), 1,
      anon_sym_LBRACE,
    STATE(640), 1,
      sym_enum_variant_list,
    STATE(2733), 1,
      sym_type_parameters,
    STATE(3231), 1,
      sym_where_clause,
    STATE(2355), 2,
      sym_line_comment,
      sym_block_comment,
  [74120] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5365), 1,
      anon_sym_SEMI,
    STATE(581), 1,
      sym_declaration_list,
    STATE(2766), 1,
      sym_where_clause,
    STATE(2356), 2,
      sym_line_comment,
      sym_block_comment,
  [74149] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4942), 1,
      anon_sym_LBRACE,
    ACTIONS(4944), 1,
      anon_sym_LT,
    ACTIONS(4946), 1,
      anon_sym_where,
    STATE(645), 1,
      sym_field_declaration_list,
    STATE(2551), 1,
      sym_type_parameters,
    STATE(3077), 1,
      sym_where_clause,
    STATE(2357), 2,
      sym_line_comment,
      sym_block_comment,
  [74178] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3555), 1,
      anon_sym_COLON,
    ACTIONS(5367), 1,
      anon_sym_EQ,
    STATE(2358), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3553), 4,
      anon_sym_PLUS,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
  [74201] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4944), 1,
      anon_sym_LT,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5285), 1,
      anon_sym_LBRACE,
    STATE(1147), 1,
      sym_enum_variant_list,
    STATE(2563), 1,
      sym_type_parameters,
    STATE(3257), 1,
      sym_where_clause,
    STATE(2359), 2,
      sym_line_comment,
      sym_block_comment,
  [74230] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3093), 1,
      anon_sym_PLUS,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    ACTIONS(5333), 1,
      anon_sym_GT,
    ACTIONS(5335), 1,
      anon_sym_COMMA,
    STATE(2973), 1,
      sym_trait_bounds,
    STATE(2983), 1,
      aux_sym_type_arguments_repeat1,
    STATE(2360), 2,
      sym_line_comment,
      sym_block_comment,
  [74259] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4944), 1,
      anon_sym_LT,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4976), 1,
      anon_sym_LBRACE,
    STATE(1152), 1,
      sym_field_declaration_list,
    STATE(2573), 1,
      sym_type_parameters,
    STATE(3292), 1,
      sym_where_clause,
    STATE(2361), 2,
      sym_line_comment,
      sym_block_comment,
  [74288] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4896), 2,
      anon_sym_COLON,
      anon_sym_PIPE,
    STATE(2362), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3607), 4,
      anon_sym_RPAREN,
      anon_sym_PLUS,
      anon_sym_COMMA,
      anon_sym_DASH_GT,
  [74309] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5369), 1,
      anon_sym_SEMI,
    STATE(603), 1,
      sym_declaration_list,
    STATE(2846), 1,
      sym_where_clause,
    STATE(2363), 2,
      sym_line_comment,
      sym_block_comment,
  [74338] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4676), 1,
      anon_sym_DOT_DOT,
    STATE(2364), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4674), 5,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_if,
  [74359] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5371), 1,
      anon_sym_SEMI,
    STATE(721), 1,
      sym_declaration_list,
    STATE(2798), 1,
      sym_where_clause,
    STATE(2365), 2,
      sym_line_comment,
      sym_block_comment,
  [74388] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4595), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4978), 1,
      anon_sym_for,
    STATE(2366), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3321), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [74411] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3581), 2,
      anon_sym_PLUS,
      anon_sym_DASH_GT,
    ACTIONS(4862), 2,
      anon_sym_COLON,
      anon_sym_PIPE,
    ACTIONS(5307), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(2367), 2,
      sym_line_comment,
      sym_block_comment,
  [74434] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5373), 1,
      anon_sym_SEMI,
    STATE(488), 1,
      sym_declaration_list,
    STATE(2822), 1,
      sym_where_clause,
    STATE(2368), 2,
      sym_line_comment,
      sym_block_comment,
  [74463] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    ACTIONS(5331), 1,
      anon_sym_PLUS,
    ACTIONS(5375), 1,
      anon_sym_GT,
    ACTIONS(5377), 1,
      anon_sym_COMMA,
    STATE(2823), 1,
      sym_trait_bounds,
    STATE(2824), 1,
      aux_sym_type_arguments_repeat1,
    STATE(2369), 2,
      sym_line_comment,
      sym_block_comment,
  [74492] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3093), 1,
      anon_sym_PLUS,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    ACTIONS(5375), 1,
      anon_sym_GT,
    ACTIONS(5377), 1,
      anon_sym_COMMA,
    STATE(2823), 1,
      sym_trait_bounds,
    STATE(2824), 1,
      aux_sym_type_arguments_repeat1,
    STATE(2370), 2,
      sym_line_comment,
      sym_block_comment,
  [74521] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5379), 1,
      anon_sym_SEMI,
    STATE(490), 1,
      sym_declaration_list,
    STATE(2833), 1,
      sym_where_clause,
    STATE(2371), 2,
      sym_line_comment,
      sym_block_comment,
  [74550] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4862), 2,
      anon_sym_COLON,
      anon_sym_PIPE,
    STATE(2372), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3581), 4,
      anon_sym_RPAREN,
      anon_sym_PLUS,
      anon_sym_COMMA,
      anon_sym_DASH_GT,
  [74571] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5381), 1,
      anon_sym_SEMI,
    STATE(585), 1,
      sym_declaration_list,
    STATE(2858), 1,
      sym_where_clause,
    STATE(2373), 2,
      sym_line_comment,
      sym_block_comment,
  [74600] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    STATE(726), 1,
      sym_declaration_list,
    STATE(2745), 1,
      sym_trait_bounds,
    STATE(3106), 1,
      sym_where_clause,
    STATE(2374), 2,
      sym_line_comment,
      sym_block_comment,
  [74629] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4634), 1,
      anon_sym_EQ,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    ACTIONS(5383), 1,
      anon_sym_GT,
    ACTIONS(5385), 1,
      anon_sym_COMMA,
    STATE(2793), 1,
      aux_sym_type_parameters_repeat1,
    STATE(2809), 1,
      sym_trait_bounds,
    STATE(2375), 2,
      sym_line_comment,
      sym_block_comment,
  [74658] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    STATE(627), 1,
      sym_declaration_list,
    STATE(2743), 1,
      sym_trait_bounds,
    STATE(3281), 1,
      sym_where_clause,
    STATE(2376), 2,
      sym_line_comment,
      sym_block_comment,
  [74687] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4517), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5225), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4519), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(5235), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(2377), 2,
      sym_line_comment,
      sym_block_comment,
  [74712] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(343), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(5387), 1,
      anon_sym_if,
    STATE(3569), 1,
      sym_label,
    STATE(1425), 2,
      sym_if_expression,
      sym_block,
    STATE(2378), 2,
      sym_line_comment,
      sym_block_comment,
  [74739] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4634), 1,
      anon_sym_EQ,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    ACTIONS(5389), 1,
      anon_sym_GT,
    ACTIONS(5391), 1,
      anon_sym_COMMA,
    STATE(2809), 1,
      sym_trait_bounds,
    STATE(2842), 1,
      aux_sym_type_parameters_repeat1,
    STATE(2379), 2,
      sym_line_comment,
      sym_block_comment,
  [74768] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5393), 1,
      anon_sym_SEMI,
    STATE(1169), 1,
      sym_declaration_list,
    STATE(2844), 1,
      sym_where_clause,
    STATE(2380), 2,
      sym_line_comment,
      sym_block_comment,
  [74797] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4517), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5231), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4519), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(5235), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(2381), 2,
      sym_line_comment,
      sym_block_comment,
  [74822] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    ACTIONS(5395), 1,
      anon_sym_GT,
    ACTIONS(5397), 1,
      anon_sym_COMMA,
    STATE(2826), 1,
      aux_sym_type_parameters_repeat1,
    STATE(2832), 1,
      sym_trait_bounds,
    STATE(3018), 1,
      aux_sym_for_lifetimes_repeat1,
    STATE(2382), 2,
      sym_line_comment,
      sym_block_comment,
  [74851] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5399), 1,
      anon_sym_SEMI,
    STATE(1173), 1,
      sym_declaration_list,
    STATE(2845), 1,
      sym_where_clause,
    STATE(2383), 2,
      sym_line_comment,
      sym_block_comment,
  [74880] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4517), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5231), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4519), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(5401), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(2384), 2,
      sym_line_comment,
      sym_block_comment,
  [74905] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5403), 1,
      anon_sym_SEMI,
    STATE(1190), 1,
      sym_declaration_list,
    STATE(2850), 1,
      sym_where_clause,
    STATE(2385), 2,
      sym_line_comment,
      sym_block_comment,
  [74934] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    STATE(1195), 1,
      sym_declaration_list,
    STATE(2692), 1,
      sym_trait_bounds,
    STATE(3175), 1,
      sym_where_clause,
    STATE(2386), 2,
      sym_line_comment,
      sym_block_comment,
  [74963] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2387), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3458), 6,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
  [74982] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    ACTIONS(5405), 1,
      anon_sym_SEMI,
    ACTIONS(5407), 1,
      anon_sym_EQ,
    STATE(3036), 1,
      sym_trait_bounds,
    STATE(3542), 1,
      sym_where_clause,
    STATE(2388), 2,
      sym_line_comment,
      sym_block_comment,
  [75011] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1007), 1,
      anon_sym_DOT_DOT,
    STATE(2389), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1009), 5,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_if,
  [75032] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4944), 1,
      anon_sym_LT,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5363), 1,
      anon_sym_LBRACE,
    STATE(670), 1,
      sym_enum_variant_list,
    STATE(2621), 1,
      sym_type_parameters,
    STATE(3255), 1,
      sym_where_clause,
    STATE(2390), 2,
      sym_line_comment,
      sym_block_comment,
  [75061] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1011), 1,
      anon_sym_DOT_DOT,
    STATE(2391), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1013), 5,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_if,
  [75082] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1023), 1,
      anon_sym_DOT_DOT,
    STATE(2392), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1025), 5,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_if,
  [75103] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1027), 1,
      anon_sym_DOT_DOT,
    STATE(2393), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1029), 5,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_if,
  [75124] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5409), 1,
      anon_sym_SEMI,
    STATE(698), 1,
      sym_declaration_list,
    STATE(2761), 1,
      sym_where_clause,
    STATE(2394), 2,
      sym_line_comment,
      sym_block_comment,
  [75153] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1519), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5051), 1,
      sym_identifier,
    ACTIONS(5057), 1,
      anon_sym_ref,
    ACTIONS(5059), 1,
      sym_mutable_specifier,
    STATE(2395), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(3058), 2,
      sym_field_pattern,
      sym_remaining_field_pattern,
  [75180] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5411), 1,
      anon_sym_SEMI,
    STATE(588), 1,
      sym_declaration_list,
    STATE(2867), 1,
      sym_where_clause,
    STATE(2396), 2,
      sym_line_comment,
      sym_block_comment,
  [75209] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4595), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5017), 1,
      anon_sym_for,
    STATE(2397), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3321), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [75232] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4595), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4926), 1,
      anon_sym_for,
    STATE(2398), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3321), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [75255] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5413), 1,
      anon_sym_SEMI,
    STATE(1220), 1,
      sym_declaration_list,
    STATE(2875), 1,
      sym_where_clause,
    STATE(2399), 2,
      sym_line_comment,
      sym_block_comment,
  [75284] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4595), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5019), 1,
      anon_sym_for,
    STATE(2400), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3321), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [75307] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5415), 1,
      anon_sym_SEMI,
    STATE(1224), 1,
      sym_declaration_list,
    STATE(2878), 1,
      sym_where_clause,
    STATE(2401), 2,
      sym_line_comment,
      sym_block_comment,
  [75336] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4595), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5021), 1,
      anon_sym_for,
    STATE(2402), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3321), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [75359] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5417), 1,
      anon_sym_SEMI,
    STATE(1226), 1,
      sym_declaration_list,
    STATE(2879), 1,
      sym_where_clause,
    STATE(2403), 2,
      sym_line_comment,
      sym_block_comment,
  [75388] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4595), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4962), 1,
      anon_sym_for,
    STATE(2404), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3321), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [75411] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4595), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4936), 1,
      anon_sym_for,
    STATE(2405), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3321), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [75434] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4517), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5225), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4519), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(5401), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(2406), 2,
      sym_line_comment,
      sym_block_comment,
  [75459] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5419), 1,
      anon_sym_SEMI,
    STATE(742), 1,
      sym_declaration_list,
    STATE(2762), 1,
      sym_where_clause,
    STATE(2407), 2,
      sym_line_comment,
      sym_block_comment,
  [75488] = 9,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5421), 1,
      anon_sym_SEMI,
    STATE(691), 1,
      sym_declaration_list,
    STATE(2970), 1,
      sym_where_clause,
    STATE(2408), 2,
      sym_line_comment,
      sym_block_comment,
  [75517] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4505), 1,
      anon_sym_COLON_COLON,
    STATE(1953), 1,
      sym_type_arguments,
    STATE(1960), 1,
      sym_parameters,
    STATE(2409), 2,
      sym_line_comment,
      sym_block_comment,
  [75543] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4507), 2,
      anon_sym_COLON,
      anon_sym_PIPE,
    STATE(2410), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3321), 3,
      anon_sym_RPAREN,
      anon_sym_PLUS,
      anon_sym_COMMA,
  [75563] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4856), 1,
      anon_sym_PIPE,
    ACTIONS(5423), 1,
      anon_sym_SEMI,
    ACTIONS(5425), 1,
      anon_sym_COLON,
    ACTIONS(5427), 1,
      anon_sym_EQ,
    ACTIONS(5429), 1,
      anon_sym_else,
    STATE(2411), 2,
      sym_line_comment,
      sym_block_comment,
  [75589] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4856), 1,
      anon_sym_PIPE,
    ACTIONS(5431), 1,
      anon_sym_SEMI,
    ACTIONS(5433), 1,
      anon_sym_COLON,
    ACTIONS(5435), 1,
      anon_sym_EQ,
    ACTIONS(5437), 1,
      anon_sym_else,
    STATE(2412), 2,
      sym_line_comment,
      sym_block_comment,
  [75615] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(5027), 1,
      anon_sym_LBRACE,
    ACTIONS(5439), 1,
      anon_sym_SEMI,
    STATE(679), 1,
      sym_block,
    STATE(3615), 1,
      sym_label,
    STATE(2413), 2,
      sym_line_comment,
      sym_block_comment,
  [75641] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4175), 1,
      anon_sym_LBRACE,
    ACTIONS(5443), 1,
      anon_sym_STAR,
    STATE(2816), 1,
      sym_use_list,
    ACTIONS(5441), 2,
      sym_identifier,
      sym_super,
    STATE(2414), 2,
      sym_line_comment,
      sym_block_comment,
  [75665] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(407), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    STATE(1670), 1,
      sym_block,
    STATE(3620), 1,
      sym_label,
    STATE(2415), 2,
      sym_line_comment,
      sym_block_comment,
  [75691] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(5027), 1,
      anon_sym_LBRACE,
    ACTIONS(5445), 1,
      anon_sym_SEMI,
    STATE(513), 1,
      sym_block,
    STATE(3615), 1,
      sym_label,
    STATE(2416), 2,
      sym_line_comment,
      sym_block_comment,
  [75717] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4848), 1,
      anon_sym_RPAREN,
    ACTIONS(5447), 1,
      anon_sym_COLON,
    ACTIONS(5449), 1,
      anon_sym_PIPE,
    ACTIONS(5451), 1,
      anon_sym_COMMA,
    STATE(3034), 1,
      aux_sym_closure_parameters_repeat1,
    STATE(2417), 2,
      sym_line_comment,
      sym_block_comment,
  [75743] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(5027), 1,
      anon_sym_LBRACE,
    ACTIONS(5453), 1,
      anon_sym_SEMI,
    STATE(576), 1,
      sym_block,
    STATE(3615), 1,
      sym_label,
    STATE(2418), 2,
      sym_line_comment,
      sym_block_comment,
  [75769] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    ACTIONS(5375), 1,
      anon_sym_GT,
    ACTIONS(5377), 1,
      anon_sym_COMMA,
    STATE(2823), 1,
      sym_trait_bounds,
    STATE(2824), 1,
      aux_sym_type_arguments_repeat1,
    STATE(2419), 2,
      sym_line_comment,
      sym_block_comment,
  [75795] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5455), 1,
      anon_sym_RPAREN,
    ACTIONS(5457), 1,
      anon_sym_COMMA,
    STATE(2835), 1,
      aux_sym_parameters_repeat1,
    ACTIONS(4507), 2,
      anon_sym_COLON,
      anon_sym_PIPE,
    STATE(2420), 2,
      sym_line_comment,
      sym_block_comment,
  [75819] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(343), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(5459), 1,
      anon_sym_move,
    STATE(1419), 1,
      sym_block,
    STATE(3569), 1,
      sym_label,
    STATE(2421), 2,
      sym_line_comment,
      sym_block_comment,
  [75845] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    ACTIONS(5461), 1,
      anon_sym_GT,
    ACTIONS(5463), 1,
      anon_sym_COMMA,
    STATE(2795), 1,
      aux_sym_type_parameters_repeat1,
    STATE(2832), 1,
      sym_trait_bounds,
    STATE(2422), 2,
      sym_line_comment,
      sym_block_comment,
  [75871] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(5027), 1,
      anon_sym_LBRACE,
    ACTIONS(5465), 1,
      anon_sym_SEMI,
    STATE(655), 1,
      sym_block,
    STATE(3615), 1,
      sym_label,
    STATE(2423), 2,
      sym_line_comment,
      sym_block_comment,
  [75897] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(5075), 1,
      anon_sym_LBRACE,
    ACTIONS(5467), 1,
      anon_sym_SEMI,
    STATE(1167), 1,
      sym_block,
    STATE(3618), 1,
      sym_label,
    STATE(2424), 2,
      sym_line_comment,
      sym_block_comment,
  [75923] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4696), 1,
      anon_sym_GT,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    ACTIONS(5469), 1,
      anon_sym_COMMA,
    STATE(2832), 1,
      sym_trait_bounds,
    STATE(2843), 1,
      aux_sym_type_parameters_repeat1,
    STATE(2425), 2,
      sym_line_comment,
      sym_block_comment,
  [75949] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4634), 1,
      anon_sym_EQ,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    STATE(2809), 1,
      sym_trait_bounds,
    ACTIONS(5471), 2,
      anon_sym_GT,
      anon_sym_COMMA,
    STATE(2426), 2,
      sym_line_comment,
      sym_block_comment,
  [75973] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(407), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    STATE(1691), 1,
      sym_block,
    STATE(3620), 1,
      sym_label,
    STATE(2427), 2,
      sym_line_comment,
      sym_block_comment,
  [75999] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4702), 1,
      anon_sym_GT,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    ACTIONS(5473), 1,
      anon_sym_COMMA,
    STATE(2832), 1,
      sym_trait_bounds,
    STATE(2947), 1,
      aux_sym_type_parameters_repeat1,
    STATE(2428), 2,
      sym_line_comment,
      sym_block_comment,
  [76025] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4175), 1,
      anon_sym_LBRACE,
    ACTIONS(5197), 1,
      anon_sym_STAR,
    STATE(2819), 1,
      sym_use_list,
    ACTIONS(5195), 2,
      sym_identifier,
      sym_super,
    STATE(2429), 2,
      sym_line_comment,
      sym_block_comment,
  [76049] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3809), 1,
      anon_sym_PLUS,
    ACTIONS(4872), 2,
      anon_sym_COLON,
      anon_sym_PIPE,
    ACTIONS(5475), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(2430), 2,
      sym_line_comment,
      sym_block_comment,
  [76071] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5478), 1,
      anon_sym_RPAREN,
    ACTIONS(5480), 1,
      anon_sym_COLON,
    ACTIONS(5482), 1,
      anon_sym_PIPE,
    ACTIONS(5484), 1,
      anon_sym_COMMA,
    STATE(2987), 1,
      aux_sym_slice_pattern_repeat1,
    STATE(2431), 2,
      sym_line_comment,
      sym_block_comment,
  [76097] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2432), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3257), 5,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_GT,
      anon_sym_COMMA,
      anon_sym_as,
  [76115] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1499), 1,
      anon_sym_RPAREN,
    ACTIONS(5486), 1,
      anon_sym_COMMA,
    STATE(2869), 1,
      aux_sym_parameters_repeat1,
    ACTIONS(4507), 2,
      anon_sym_COLON,
      anon_sym_PIPE,
    STATE(2433), 2,
      sym_line_comment,
      sym_block_comment,
  [76139] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(5075), 1,
      anon_sym_LBRACE,
    ACTIONS(5488), 1,
      anon_sym_SEMI,
    STATE(1214), 1,
      sym_block,
    STATE(3618), 1,
      sym_label,
    STATE(2434), 2,
      sym_line_comment,
      sym_block_comment,
  [76165] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5492), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5494), 1,
      anon_sym_as,
    STATE(2435), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5490), 3,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
  [76187] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5482), 1,
      anon_sym_PIPE,
    ACTIONS(5496), 1,
      anon_sym_SEMI,
    ACTIONS(5498), 1,
      anon_sym_COLON,
    ACTIONS(5500), 1,
      anon_sym_EQ,
    ACTIONS(5502), 1,
      anon_sym_else,
    STATE(2436), 2,
      sym_line_comment,
      sym_block_comment,
  [76213] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(407), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    STATE(1809), 1,
      sym_block,
    STATE(3620), 1,
      sym_label,
    STATE(2437), 2,
      sym_line_comment,
      sym_block_comment,
  [76239] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(5075), 1,
      anon_sym_LBRACE,
    ACTIONS(5504), 1,
      anon_sym_SEMI,
    STATE(1247), 1,
      sym_block,
    STATE(3618), 1,
      sym_label,
    STATE(2438), 2,
      sym_line_comment,
      sym_block_comment,
  [76265] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5506), 1,
      anon_sym_RPAREN,
    ACTIONS(5508), 1,
      anon_sym_COMMA,
    STATE(3009), 1,
      aux_sym_parameters_repeat1,
    ACTIONS(4507), 2,
      anon_sym_COLON,
      anon_sym_PIPE,
    STATE(2439), 2,
      sym_line_comment,
      sym_block_comment,
  [76289] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5512), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5514), 1,
      anon_sym_as,
    STATE(2440), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5510), 3,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
  [76311] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(5027), 1,
      anon_sym_LBRACE,
    ACTIONS(5516), 1,
      anon_sym_SEMI,
    STATE(572), 1,
      sym_block,
    STATE(3615), 1,
      sym_label,
    STATE(2441), 2,
      sym_line_comment,
      sym_block_comment,
  [76337] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(5027), 1,
      anon_sym_LBRACE,
    ACTIONS(5518), 1,
      anon_sym_SEMI,
    STATE(526), 1,
      sym_block,
    STATE(3615), 1,
      sym_label,
    STATE(2442), 2,
      sym_line_comment,
      sym_block_comment,
  [76363] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(5075), 1,
      anon_sym_LBRACE,
    ACTIONS(5520), 1,
      anon_sym_SEMI,
    STATE(1269), 1,
      sym_block,
    STATE(3618), 1,
      sym_label,
    STATE(2443), 2,
      sym_line_comment,
      sym_block_comment,
  [76389] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5522), 1,
      anon_sym_RPAREN,
    ACTIONS(5525), 1,
      anon_sym_COMMA,
    STATE(3009), 1,
      aux_sym_parameters_repeat1,
    ACTIONS(4507), 2,
      anon_sym_COLON,
      anon_sym_PIPE,
    STATE(2444), 2,
      sym_line_comment,
      sym_block_comment,
  [76413] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2445), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5528), 5,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_where,
  [76431] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3321), 1,
      anon_sym_PLUS,
    ACTIONS(4507), 2,
      anon_sym_COLON,
      anon_sym_PIPE,
    ACTIONS(5530), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(2446), 2,
      sym_line_comment,
      sym_block_comment,
  [76453] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5514), 1,
      anon_sym_as,
    ACTIONS(5533), 1,
      anon_sym_COLON_COLON,
    STATE(2447), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5510), 3,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
  [76475] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(5075), 1,
      anon_sym_LBRACE,
    ACTIONS(5535), 1,
      anon_sym_SEMI,
    STATE(1298), 1,
      sym_block,
    STATE(3618), 1,
      sym_label,
    STATE(2448), 2,
      sym_line_comment,
      sym_block_comment,
  [76501] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(5075), 1,
      anon_sym_LBRACE,
    ACTIONS(5537), 1,
      anon_sym_SEMI,
    STATE(1311), 1,
      sym_block,
    STATE(3618), 1,
      sym_label,
    STATE(2449), 2,
      sym_line_comment,
      sym_block_comment,
  [76527] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(343), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    STATE(1433), 1,
      sym_block,
    STATE(3569), 1,
      sym_label,
    STATE(2450), 2,
      sym_line_comment,
      sym_block_comment,
  [76553] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(5075), 1,
      anon_sym_LBRACE,
    ACTIONS(5539), 1,
      anon_sym_SEMI,
    STATE(1316), 1,
      sym_block,
    STATE(3618), 1,
      sym_label,
    STATE(2451), 2,
      sym_line_comment,
      sym_block_comment,
  [76579] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2452), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3485), 5,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_where,
  [76597] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2453), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5541), 5,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_where,
  [76615] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(5075), 1,
      anon_sym_LBRACE,
    ACTIONS(5543), 1,
      anon_sym_SEMI,
    STATE(1340), 1,
      sym_block,
    STATE(3618), 1,
      sym_label,
    STATE(2454), 2,
      sym_line_comment,
      sym_block_comment,
  [76641] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(5075), 1,
      anon_sym_LBRACE,
    ACTIONS(5545), 1,
      anon_sym_SEMI,
    STATE(1346), 1,
      sym_block,
    STATE(3618), 1,
      sym_label,
    STATE(2455), 2,
      sym_line_comment,
      sym_block_comment,
  [76667] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(5027), 1,
      anon_sym_LBRACE,
    ACTIONS(5547), 1,
      anon_sym_SEMI,
    STATE(580), 1,
      sym_block,
    STATE(3615), 1,
      sym_label,
    STATE(2456), 2,
      sym_line_comment,
      sym_block_comment,
  [76693] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(5075), 1,
      anon_sym_LBRACE,
    ACTIONS(5549), 1,
      anon_sym_SEMI,
    STATE(1356), 1,
      sym_block,
    STATE(3618), 1,
      sym_label,
    STATE(2457), 2,
      sym_line_comment,
      sym_block_comment,
  [76719] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(5075), 1,
      anon_sym_LBRACE,
    ACTIONS(5551), 1,
      anon_sym_SEMI,
    STATE(1360), 1,
      sym_block,
    STATE(3618), 1,
      sym_label,
    STATE(2458), 2,
      sym_line_comment,
      sym_block_comment,
  [76745] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(5075), 1,
      anon_sym_LBRACE,
    ACTIONS(5553), 1,
      anon_sym_SEMI,
    STATE(1364), 1,
      sym_block,
    STATE(3618), 1,
      sym_label,
    STATE(2459), 2,
      sym_line_comment,
      sym_block_comment,
  [76771] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2460), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3651), 5,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_where,
  [76789] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(407), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(5555), 1,
      anon_sym_move,
    STATE(1746), 1,
      sym_block,
    STATE(3620), 1,
      sym_label,
    STATE(2461), 2,
      sym_line_comment,
      sym_block_comment,
  [76815] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2462), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3521), 5,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_where,
  [76833] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2463), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3627), 5,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_where,
  [76851] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2464), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5557), 5,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_where,
  [76869] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(5027), 1,
      anon_sym_LBRACE,
    ACTIONS(5559), 1,
      anon_sym_SEMI,
    STATE(529), 1,
      sym_block,
    STATE(3615), 1,
      sym_label,
    STATE(2465), 2,
      sym_line_comment,
      sym_block_comment,
  [76895] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2997), 1,
      anon_sym_POUND,
    ACTIONS(5561), 1,
      sym_identifier,
    ACTIONS(5563), 1,
      sym_integer_literal,
    STATE(1061), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(1405), 1,
      sym_attribute_item,
    STATE(2466), 2,
      sym_line_comment,
      sym_block_comment,
  [76921] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2467), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3655), 5,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_where,
  [76939] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2468), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5565), 5,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_where,
  [76957] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2469), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5567), 5,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_where,
  [76975] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4005), 1,
      anon_sym_LPAREN,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4505), 1,
      anon_sym_COLON_COLON,
    STATE(1610), 1,
      sym_parameters,
    STATE(1953), 1,
      sym_type_arguments,
    STATE(2470), 2,
      sym_line_comment,
      sym_block_comment,
  [77001] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2471), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5569), 5,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_where,
  [77019] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    ACTIONS(5337), 1,
      anon_sym_GT,
    ACTIONS(5339), 1,
      anon_sym_COMMA,
    STATE(2925), 1,
      sym_trait_bounds,
    STATE(2926), 1,
      aux_sym_type_arguments_repeat1,
    STATE(2472), 2,
      sym_line_comment,
      sym_block_comment,
  [77045] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2473), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5571), 5,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_where,
  [77063] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2474), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5573), 5,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_where,
  [77081] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5575), 1,
      anon_sym_RPAREN,
    ACTIONS(5577), 1,
      anon_sym_COMMA,
    STATE(2927), 1,
      aux_sym_parameters_repeat1,
    ACTIONS(4507), 2,
      anon_sym_COLON,
      anon_sym_PIPE,
    STATE(2475), 2,
      sym_line_comment,
      sym_block_comment,
  [77105] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1503), 1,
      anon_sym_RPAREN,
    ACTIONS(5579), 1,
      anon_sym_COMMA,
    STATE(2932), 1,
      aux_sym_parameters_repeat1,
    ACTIONS(4507), 2,
      anon_sym_COLON,
      anon_sym_PIPE,
    STATE(2476), 2,
      sym_line_comment,
      sym_block_comment,
  [77129] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5514), 1,
      anon_sym_as,
    ACTIONS(5581), 1,
      anon_sym_COLON_COLON,
    STATE(2477), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5510), 3,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
  [77151] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(343), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    STATE(1450), 1,
      sym_block,
    STATE(3569), 1,
      sym_label,
    STATE(2478), 2,
      sym_line_comment,
      sym_block_comment,
  [77177] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2479), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5583), 5,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_where,
  [77195] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2480), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5585), 5,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_where,
  [77213] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5587), 1,
      anon_sym_STAR_SLASH,
    ACTIONS(5589), 1,
      sym__outer_block_doc_comment_marker,
    ACTIONS(5591), 1,
      sym__inner_block_doc_comment_marker,
    ACTIONS(5593), 1,
      sym__block_comment_content,
    STATE(3334), 1,
      sym__block_doc_comment_marker,
    STATE(2481), 2,
      sym_line_comment,
      sym_block_comment,
  [77239] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2482), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3489), 5,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_where,
  [77257] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2483), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5595), 5,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_where,
  [77275] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(5027), 1,
      anon_sym_LBRACE,
    ACTIONS(5597), 1,
      anon_sym_SEMI,
    STATE(555), 1,
      sym_block,
    STATE(3615), 1,
      sym_label,
    STATE(2484), 2,
      sym_line_comment,
      sym_block_comment,
  [77301] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1229), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(5599), 1,
      anon_sym_move,
    STATE(458), 1,
      sym_block,
    STATE(3619), 1,
      sym_label,
    STATE(2485), 2,
      sym_line_comment,
      sym_block_comment,
  [77327] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    ACTIONS(5317), 1,
      anon_sym_GT,
    ACTIONS(5319), 1,
      anon_sym_COMMA,
    STATE(2826), 1,
      aux_sym_type_parameters_repeat1,
    STATE(2832), 1,
      sym_trait_bounds,
    STATE(2486), 2,
      sym_line_comment,
      sym_block_comment,
  [77353] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(5027), 1,
      anon_sym_LBRACE,
    ACTIONS(5601), 1,
      anon_sym_SEMI,
    STATE(562), 1,
      sym_block,
    STATE(3615), 1,
      sym_label,
    STATE(2487), 2,
      sym_line_comment,
      sym_block_comment,
  [77379] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5605), 1,
      anon_sym_COMMA,
    ACTIONS(5603), 3,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_SQUOTE,
    STATE(2488), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_where_clause_repeat1,
  [77399] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5530), 1,
      anon_sym_RBRACK,
    ACTIONS(3321), 2,
      anon_sym_SEMI,
      anon_sym_PLUS,
    ACTIONS(4507), 2,
      anon_sym_PIPE,
      anon_sym_COMMA,
    STATE(2489), 2,
      sym_line_comment,
      sym_block_comment,
  [77421] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4505), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4632), 1,
      anon_sym_COLON,
    STATE(1953), 1,
      sym_type_arguments,
    STATE(2726), 1,
      sym_trait_bounds,
    STATE(2490), 2,
      sym_line_comment,
      sym_block_comment,
  [77447] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4872), 2,
      anon_sym_COLON,
      anon_sym_PIPE,
    STATE(2491), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3809), 3,
      anon_sym_RPAREN,
      anon_sym_PLUS,
      anon_sym_COMMA,
  [77467] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5610), 1,
      anon_sym_COMMA,
    STATE(2504), 1,
      aux_sym_where_clause_repeat1,
    STATE(2492), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5608), 3,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_SQUOTE,
  [77489] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5482), 1,
      anon_sym_PIPE,
    ACTIONS(5612), 1,
      anon_sym_SEMI,
    ACTIONS(5614), 1,
      anon_sym_COLON,
    ACTIONS(5616), 1,
      anon_sym_EQ,
    ACTIONS(5618), 1,
      anon_sym_else,
    STATE(2493), 2,
      sym_line_comment,
      sym_block_comment,
  [77515] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(5027), 1,
      anon_sym_LBRACE,
    ACTIONS(5620), 1,
      anon_sym_SEMI,
    STATE(753), 1,
      sym_block,
    STATE(3615), 1,
      sym_label,
    STATE(2494), 2,
      sym_line_comment,
      sym_block_comment,
  [77541] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1501), 1,
      anon_sym_RPAREN,
    ACTIONS(5622), 1,
      anon_sym_COMMA,
    STATE(2897), 1,
      aux_sym_parameters_repeat1,
    ACTIONS(4507), 2,
      anon_sym_COLON,
      anon_sym_PIPE,
    STATE(2495), 2,
      sym_line_comment,
      sym_block_comment,
  [77565] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2496), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3473), 5,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_where,
  [77583] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2497), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3477), 5,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_where,
  [77601] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2498), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3481), 5,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_where,
  [77619] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4634), 1,
      anon_sym_EQ,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    STATE(2809), 1,
      sym_trait_bounds,
    ACTIONS(5624), 2,
      anon_sym_GT,
      anon_sym_COMMA,
    STATE(2499), 2,
      sym_line_comment,
      sym_block_comment,
  [77643] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5475), 1,
      anon_sym_RBRACK,
    ACTIONS(3809), 2,
      anon_sym_SEMI,
      anon_sym_PLUS,
    ACTIONS(4872), 2,
      anon_sym_PIPE,
      anon_sym_COMMA,
    STATE(2500), 2,
      sym_line_comment,
      sym_block_comment,
  [77665] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    ACTIONS(5331), 1,
      anon_sym_PLUS,
    STATE(3116), 1,
      sym_trait_bounds,
    ACTIONS(5626), 2,
      anon_sym_GT,
      anon_sym_COMMA,
    STATE(2501), 2,
      sym_line_comment,
      sym_block_comment,
  [77689] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3093), 1,
      anon_sym_PLUS,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    STATE(3116), 1,
      sym_trait_bounds,
    ACTIONS(5626), 2,
      anon_sym_GT,
      anon_sym_COMMA,
    STATE(2502), 2,
      sym_line_comment,
      sym_block_comment,
  [77713] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    ACTIONS(5333), 1,
      anon_sym_GT,
    ACTIONS(5335), 1,
      anon_sym_COMMA,
    STATE(2973), 1,
      sym_trait_bounds,
    STATE(2983), 1,
      aux_sym_type_arguments_repeat1,
    STATE(2503), 2,
      sym_line_comment,
      sym_block_comment,
  [77739] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5628), 1,
      anon_sym_COMMA,
    STATE(2488), 1,
      aux_sym_where_clause_repeat1,
    STATE(2504), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3430), 3,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_SQUOTE,
  [77761] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2505), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5630), 5,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_where,
  [77779] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2506), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5632), 5,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_where,
  [77797] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(5027), 1,
      anon_sym_LBRACE,
    ACTIONS(5634), 1,
      anon_sym_SEMI,
    STATE(601), 1,
      sym_block,
    STATE(3615), 1,
      sym_label,
    STATE(2507), 2,
      sym_line_comment,
      sym_block_comment,
  [77823] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2508), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5636), 5,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_where,
  [77841] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4517), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5223), 1,
      anon_sym_COLON,
    ACTIONS(5239), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4519), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(2509), 2,
      sym_line_comment,
      sym_block_comment,
  [77865] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4517), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5237), 1,
      anon_sym_COLON,
    ACTIONS(5239), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4519), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(2510), 2,
      sym_line_comment,
      sym_block_comment,
  [77889] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(343), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    STATE(1480), 1,
      sym_block,
    STATE(3569), 1,
      sym_label,
    STATE(2511), 2,
      sym_line_comment,
      sym_block_comment,
  [77915] = 8,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3297), 1,
      anon_sym_LPAREN,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4505), 1,
      anon_sym_COLON_COLON,
    STATE(1118), 1,
      sym_parameters,
    STATE(1953), 1,
      sym_type_arguments,
    STATE(2512), 2,
      sym_line_comment,
      sym_block_comment,
  [77941] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2513), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3467), 5,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_where,
  [77959] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5638), 1,
      anon_sym_RPAREN,
    ACTIONS(5640), 1,
      anon_sym_COMMA,
    STATE(3004), 1,
      aux_sym_ordered_field_declaration_list_repeat1,
    STATE(2514), 2,
      sym_line_comment,
      sym_block_comment,
  [77982] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    STATE(3158), 1,
      sym_type_arguments,
    ACTIONS(5642), 2,
      sym_identifier,
      sym_super,
    STATE(2515), 2,
      sym_line_comment,
      sym_block_comment,
  [78003] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    STATE(3311), 1,
      sym_type_arguments,
    ACTIONS(5642), 2,
      sym_identifier,
      sym_super,
    STATE(2516), 2,
      sym_line_comment,
      sym_block_comment,
  [78024] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5644), 1,
      anon_sym_SEMI,
    ACTIONS(5646), 1,
      anon_sym_EQ,
    ACTIONS(5648), 1,
      anon_sym_else,
    STATE(2517), 2,
      sym_line_comment,
      sym_block_comment,
  [78047] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5650), 1,
      aux_sym_token_repetition_pattern_token1,
    STATE(2518), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5652), 3,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_QMARK,
  [78066] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(343), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(3524), 1,
      sym_block,
    STATE(3569), 1,
      sym_label,
    STATE(2519), 2,
      sym_line_comment,
      sym_block_comment,
  [78089] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    STATE(3158), 1,
      sym_type_arguments,
    ACTIONS(5654), 2,
      sym_identifier,
      sym_super,
    STATE(2520), 2,
      sym_line_comment,
      sym_block_comment,
  [78110] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    STATE(626), 1,
      sym_declaration_list,
    STATE(3268), 1,
      sym_where_clause,
    STATE(2521), 2,
      sym_line_comment,
      sym_block_comment,
  [78133] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5656), 1,
      anon_sym_SEMI,
    STATE(3451), 1,
      sym_where_clause,
    STATE(2522), 2,
      sym_line_comment,
      sym_block_comment,
  [78156] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    STATE(1306), 1,
      sym_declaration_list,
    STATE(3060), 1,
      sym_where_clause,
    STATE(2523), 2,
      sym_line_comment,
      sym_block_comment,
  [78179] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5658), 1,
      anon_sym_DQUOTE,
    STATE(2713), 1,
      aux_sym_string_literal_repeat1,
    ACTIONS(5660), 2,
      sym_string_content,
      sym_escape_sequence,
    STATE(2524), 2,
      sym_line_comment,
      sym_block_comment,
  [78200] = 7,
    ACTIONS(19), 1,
      anon_sym_LBRACE,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(378), 1,
      sym_block,
    STATE(3355), 1,
      sym_label,
    STATE(2525), 2,
      sym_line_comment,
      sym_block_comment,
  [78223] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(343), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(3502), 1,
      sym_block,
    STATE(3569), 1,
      sym_label,
    STATE(2526), 2,
      sym_line_comment,
      sym_block_comment,
  [78246] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(343), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(3440), 1,
      sym_block,
    STATE(3569), 1,
      sym_label,
    STATE(2527), 2,
      sym_line_comment,
      sym_block_comment,
  [78269] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    STATE(3158), 1,
      sym_type_arguments,
    ACTIONS(5195), 2,
      sym_identifier,
      sym_super,
    STATE(2528), 2,
      sym_line_comment,
      sym_block_comment,
  [78290] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    STATE(3311), 1,
      sym_type_arguments,
    ACTIONS(5195), 2,
      sym_identifier,
      sym_super,
    STATE(2529), 2,
      sym_line_comment,
      sym_block_comment,
  [78311] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5662), 1,
      anon_sym_COLON,
    STATE(2530), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4856), 3,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_COMMA,
  [78330] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(343), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(1420), 1,
      sym_block,
    STATE(3569), 1,
      sym_label,
    STATE(2531), 2,
      sym_line_comment,
      sym_block_comment,
  [78353] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    STATE(2532), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5664), 3,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_COMMA,
  [78372] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2533), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1035), 4,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
  [78389] = 7,
    ACTIONS(19), 1,
      anon_sym_LBRACE,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(396), 1,
      sym_block,
    STATE(3355), 1,
      sym_label,
    STATE(2534), 2,
      sym_line_comment,
      sym_block_comment,
  [78412] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(343), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(3431), 1,
      sym_block,
    STATE(3569), 1,
      sym_label,
    STATE(2535), 2,
      sym_line_comment,
      sym_block_comment,
  [78435] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4696), 1,
      anon_sym_GT,
    ACTIONS(5469), 1,
      anon_sym_COMMA,
    ACTIONS(5666), 1,
      anon_sym_EQ,
    STATE(2843), 1,
      aux_sym_type_parameters_repeat1,
    STATE(2536), 2,
      sym_line_comment,
      sym_block_comment,
  [78458] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4517), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5239), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4519), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(2537), 2,
      sym_line_comment,
      sym_block_comment,
  [78479] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    STATE(3311), 1,
      sym_type_arguments,
    ACTIONS(5654), 2,
      sym_identifier,
      sym_super,
    STATE(2538), 2,
      sym_line_comment,
      sym_block_comment,
  [78500] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4702), 1,
      anon_sym_GT,
    ACTIONS(5473), 1,
      anon_sym_COMMA,
    ACTIONS(5666), 1,
      anon_sym_EQ,
    STATE(2947), 1,
      aux_sym_type_parameters_repeat1,
    STATE(2539), 2,
      sym_line_comment,
      sym_block_comment,
  [78523] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5317), 1,
      anon_sym_GT,
    ACTIONS(5319), 1,
      anon_sym_COMMA,
    ACTIONS(5666), 1,
      anon_sym_EQ,
    STATE(2826), 1,
      aux_sym_type_parameters_repeat1,
    STATE(2540), 2,
      sym_line_comment,
      sym_block_comment,
  [78546] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5668), 1,
      anon_sym_SEMI,
    ACTIONS(5670), 1,
      anon_sym_EQ,
    ACTIONS(5672), 1,
      anon_sym_else,
    STATE(2541), 2,
      sym_line_comment,
      sym_block_comment,
  [78569] = 7,
    ACTIONS(19), 1,
      anon_sym_LBRACE,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(382), 1,
      sym_block,
    STATE(3355), 1,
      sym_label,
    STATE(2542), 2,
      sym_line_comment,
      sym_block_comment,
  [78592] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(5674), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5676), 1,
      anon_sym_for,
    STATE(1953), 1,
      sym_type_arguments,
    STATE(2543), 2,
      sym_line_comment,
      sym_block_comment,
  [78615] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1229), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(463), 1,
      sym_block,
    STATE(3619), 1,
      sym_label,
    STATE(2544), 2,
      sym_line_comment,
      sym_block_comment,
  [78638] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1229), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(466), 1,
      sym_block,
    STATE(3619), 1,
      sym_label,
    STATE(2545), 2,
      sym_line_comment,
      sym_block_comment,
  [78661] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5678), 1,
      anon_sym_DQUOTE,
    STATE(2564), 1,
      aux_sym_string_literal_repeat1,
    ACTIONS(5660), 2,
      sym_string_content,
      sym_escape_sequence,
    STATE(2546), 2,
      sym_line_comment,
      sym_block_comment,
  [78682] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    STATE(1183), 1,
      sym_declaration_list,
    STATE(3157), 1,
      sym_where_clause,
    STATE(2547), 2,
      sym_line_comment,
      sym_block_comment,
  [78705] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    ACTIONS(5075), 1,
      anon_sym_LBRACE,
    STATE(2918), 1,
      sym_block,
    STATE(3618), 1,
      sym_label,
    STATE(2548), 2,
      sym_line_comment,
      sym_block_comment,
  [78728] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(407), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(1749), 1,
      sym_block,
    STATE(3620), 1,
      sym_label,
    STATE(2549), 2,
      sym_line_comment,
      sym_block_comment,
  [78751] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5680), 1,
      anon_sym_SEMI,
    STATE(3442), 1,
      sym_where_clause,
    STATE(2550), 2,
      sym_line_comment,
      sym_block_comment,
  [78774] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4942), 1,
      anon_sym_LBRACE,
    ACTIONS(4946), 1,
      anon_sym_where,
    STATE(738), 1,
      sym_field_declaration_list,
    STATE(3134), 1,
      sym_where_clause,
    STATE(2551), 2,
      sym_line_comment,
      sym_block_comment,
  [78797] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(343), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(1404), 1,
      sym_block,
    STATE(3569), 1,
      sym_label,
    STATE(2552), 2,
      sym_line_comment,
      sym_block_comment,
  [78820] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4011), 1,
      anon_sym_LT2,
    ACTIONS(4167), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4834), 1,
      anon_sym_BANG,
    STATE(1615), 1,
      sym_type_arguments,
    STATE(2553), 2,
      sym_line_comment,
      sym_block_comment,
  [78843] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    STATE(1194), 1,
      sym_declaration_list,
    STATE(3169), 1,
      sym_where_clause,
    STATE(2554), 2,
      sym_line_comment,
      sym_block_comment,
  [78866] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(343), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(3445), 1,
      sym_block,
    STATE(3569), 1,
      sym_label,
    STATE(2555), 2,
      sym_line_comment,
      sym_block_comment,
  [78889] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1229), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(469), 1,
      sym_block,
    STATE(3619), 1,
      sym_label,
    STATE(2556), 2,
      sym_line_comment,
      sym_block_comment,
  [78912] = 7,
    ACTIONS(19), 1,
      anon_sym_LBRACE,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(383), 1,
      sym_block,
    STATE(3355), 1,
      sym_label,
    STATE(2557), 2,
      sym_line_comment,
      sym_block_comment,
  [78935] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5682), 1,
      anon_sym_LPAREN,
    ACTIONS(5684), 1,
      anon_sym_LBRACK,
    ACTIONS(5686), 1,
      anon_sym_LBRACE,
    STATE(1964), 1,
      sym_delim_token_tree,
    STATE(2558), 2,
      sym_line_comment,
      sym_block_comment,
  [78958] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1229), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(470), 1,
      sym_block,
    STATE(3619), 1,
      sym_label,
    STATE(2559), 2,
      sym_line_comment,
      sym_block_comment,
  [78981] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1229), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(452), 1,
      sym_block,
    STATE(3619), 1,
      sym_label,
    STATE(2560), 2,
      sym_line_comment,
      sym_block_comment,
  [79004] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(5674), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5688), 1,
      anon_sym_for,
    STATE(1953), 1,
      sym_type_arguments,
    STATE(2561), 2,
      sym_line_comment,
      sym_block_comment,
  [79027] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1229), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(479), 1,
      sym_block,
    STATE(3619), 1,
      sym_label,
    STATE(2562), 2,
      sym_line_comment,
      sym_block_comment,
  [79050] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5285), 1,
      anon_sym_LBRACE,
    STATE(1199), 1,
      sym_enum_variant_list,
    STATE(3188), 1,
      sym_where_clause,
    STATE(2563), 2,
      sym_line_comment,
      sym_block_comment,
  [79073] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5690), 1,
      anon_sym_DQUOTE,
    STATE(2734), 1,
      aux_sym_string_literal_repeat1,
    ACTIONS(5660), 2,
      sym_string_content,
      sym_escape_sequence,
    STATE(2564), 2,
      sym_line_comment,
      sym_block_comment,
  [79094] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5682), 1,
      anon_sym_LPAREN,
    ACTIONS(5684), 1,
      anon_sym_LBRACK,
    ACTIONS(5686), 1,
      anon_sym_LBRACE,
    STATE(1969), 1,
      sym_delim_token_tree,
    STATE(2565), 2,
      sym_line_comment,
      sym_block_comment,
  [79117] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4011), 1,
      anon_sym_LT2,
    ACTIONS(5692), 1,
      sym_identifier,
    ACTIONS(5694), 1,
      sym_super,
    STATE(1563), 1,
      sym_type_arguments,
    STATE(2566), 2,
      sym_line_comment,
      sym_block_comment,
  [79140] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4011), 1,
      anon_sym_LT2,
    ACTIONS(5692), 1,
      sym_identifier,
    ACTIONS(5694), 1,
      sym_super,
    STATE(1586), 1,
      sym_type_arguments,
    STATE(2567), 2,
      sym_line_comment,
      sym_block_comment,
  [79163] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4742), 1,
      anon_sym_for,
    ACTIONS(5674), 1,
      anon_sym_COLON_COLON,
    STATE(1953), 1,
      sym_type_arguments,
    STATE(2568), 2,
      sym_line_comment,
      sym_block_comment,
  [79186] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(5195), 1,
      sym_super,
    ACTIONS(5696), 1,
      sym_identifier,
    STATE(3158), 1,
      sym_type_arguments,
    STATE(2569), 2,
      sym_line_comment,
      sym_block_comment,
  [79209] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(5195), 1,
      sym_super,
    ACTIONS(5696), 1,
      sym_identifier,
    STATE(3311), 1,
      sym_type_arguments,
    STATE(2570), 2,
      sym_line_comment,
      sym_block_comment,
  [79232] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5698), 1,
      anon_sym_RPAREN,
    ACTIONS(5700), 1,
      anon_sym_COMMA,
    STATE(2920), 1,
      aux_sym_tuple_type_repeat1,
    STATE(2571), 2,
      sym_line_comment,
      sym_block_comment,
  [79255] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    STATE(2832), 1,
      sym_trait_bounds,
    ACTIONS(5702), 2,
      anon_sym_GT,
      anon_sym_COMMA,
    STATE(2572), 2,
      sym_line_comment,
      sym_block_comment,
  [79276] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4976), 1,
      anon_sym_LBRACE,
    STATE(1205), 1,
      sym_field_declaration_list,
    STATE(3213), 1,
      sym_where_clause,
    STATE(2573), 2,
      sym_line_comment,
      sym_block_comment,
  [79299] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4005), 1,
      anon_sym_LPAREN,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    STATE(1611), 1,
      sym_parameters,
    STATE(1955), 1,
      sym_type_arguments,
    STATE(2574), 2,
      sym_line_comment,
      sym_block_comment,
  [79322] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    ACTIONS(4944), 1,
      anon_sym_LT,
    STATE(2196), 1,
      sym_parameters,
    STATE(3273), 1,
      sym_type_parameters,
    STATE(2575), 2,
      sym_line_comment,
      sym_block_comment,
  [79345] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5704), 1,
      anon_sym_SEMI,
    STATE(3531), 1,
      sym_where_clause,
    STATE(2576), 2,
      sym_line_comment,
      sym_block_comment,
  [79368] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1229), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(460), 1,
      sym_block,
    STATE(3619), 1,
      sym_label,
    STATE(2577), 2,
      sym_line_comment,
      sym_block_comment,
  [79391] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5331), 1,
      anon_sym_PLUS,
    STATE(2578), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5706), 3,
      anon_sym_COLON,
      anon_sym_GT,
      anon_sym_COMMA,
  [79410] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4942), 1,
      anon_sym_LBRACE,
    ACTIONS(4946), 1,
      anon_sym_where,
    STATE(632), 1,
      sym_field_declaration_list,
    STATE(3059), 1,
      sym_where_clause,
    STATE(2579), 2,
      sym_line_comment,
      sym_block_comment,
  [79433] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2580), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4299), 4,
      anon_sym_LBRACE,
      anon_sym_EQ_GT,
      anon_sym_AMP_AMP,
      anon_sym_SQUOTE,
  [79450] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5708), 1,
      anon_sym_LPAREN,
    ACTIONS(5710), 1,
      anon_sym_LBRACK,
    ACTIONS(5712), 1,
      anon_sym_LBRACE,
    STATE(402), 1,
      sym_delim_token_tree,
    STATE(2581), 2,
      sym_line_comment,
      sym_block_comment,
  [79473] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(343), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(1455), 1,
      sym_block,
    STATE(3569), 1,
      sym_label,
    STATE(2582), 2,
      sym_line_comment,
      sym_block_comment,
  [79496] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    STATE(1135), 1,
      sym_declaration_list,
    STATE(3227), 1,
      sym_where_clause,
    STATE(2583), 2,
      sym_line_comment,
      sym_block_comment,
  [79519] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5575), 1,
      anon_sym_RPAREN,
    ACTIONS(5577), 1,
      anon_sym_COMMA,
    STATE(2927), 1,
      aux_sym_parameters_repeat1,
    STATE(2584), 2,
      sym_line_comment,
      sym_block_comment,
  [79542] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    ACTIONS(4944), 1,
      anon_sym_LT,
    STATE(2223), 1,
      sym_parameters,
    STATE(3220), 1,
      sym_type_parameters,
    STATE(2585), 2,
      sym_line_comment,
      sym_block_comment,
  [79565] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5714), 1,
      anon_sym_SEMI,
    STATE(3576), 1,
      sym_where_clause,
    STATE(2586), 2,
      sym_line_comment,
      sym_block_comment,
  [79588] = 7,
    ACTIONS(19), 1,
      anon_sym_LBRACE,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(400), 1,
      sym_block,
    STATE(3355), 1,
      sym_label,
    STATE(2587), 2,
      sym_line_comment,
      sym_block_comment,
  [79611] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1229), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(473), 1,
      sym_block,
    STATE(3619), 1,
      sym_label,
    STATE(2588), 2,
      sym_line_comment,
      sym_block_comment,
  [79634] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5716), 1,
      anon_sym_RPAREN,
    ACTIONS(5718), 1,
      anon_sym_COMMA,
    STATE(2979), 1,
      aux_sym_ordered_field_declaration_list_repeat1,
    STATE(2589), 2,
      sym_line_comment,
      sym_block_comment,
  [79657] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(343), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(3437), 1,
      sym_block,
    STATE(3569), 1,
      sym_label,
    STATE(2590), 2,
      sym_line_comment,
      sym_block_comment,
  [79680] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(5654), 1,
      sym_super,
    ACTIONS(5720), 1,
      sym_identifier,
    STATE(3158), 1,
      sym_type_arguments,
    STATE(2591), 2,
      sym_line_comment,
      sym_block_comment,
  [79703] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1503), 1,
      anon_sym_RPAREN,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5579), 1,
      anon_sym_COMMA,
    STATE(2932), 1,
      aux_sym_parameters_repeat1,
    STATE(2592), 2,
      sym_line_comment,
      sym_block_comment,
  [79726] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5447), 1,
      anon_sym_COLON,
    ACTIONS(5449), 1,
      anon_sym_PIPE,
    ACTIONS(5451), 1,
      anon_sym_COMMA,
    STATE(3034), 1,
      aux_sym_closure_parameters_repeat1,
    STATE(2593), 2,
      sym_line_comment,
      sym_block_comment,
  [79749] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5722), 1,
      anon_sym_COLON_COLON,
    STATE(2594), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4764), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [79768] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5724), 1,
      sym_identifier,
    STATE(2595), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5726), 3,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
  [79787] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(407), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(1748), 1,
      sym_block,
    STATE(3620), 1,
      sym_label,
    STATE(2596), 2,
      sym_line_comment,
      sym_block_comment,
  [79810] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(407), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(1750), 1,
      sym_block,
    STATE(3620), 1,
      sym_label,
    STATE(2597), 2,
      sym_line_comment,
      sym_block_comment,
  [79833] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5728), 1,
      anon_sym_DQUOTE,
    STATE(2606), 1,
      aux_sym_string_literal_repeat1,
    ACTIONS(5660), 2,
      sym_string_content,
      sym_escape_sequence,
    STATE(2598), 2,
      sym_line_comment,
      sym_block_comment,
  [79854] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1229), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(465), 1,
      sym_block,
    STATE(3619), 1,
      sym_label,
    STATE(2599), 2,
      sym_line_comment,
      sym_block_comment,
  [79877] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5461), 1,
      anon_sym_GT,
    ACTIONS(5463), 1,
      anon_sym_COMMA,
    ACTIONS(5666), 1,
      anon_sym_EQ,
    STATE(2795), 1,
      aux_sym_type_parameters_repeat1,
    STATE(2600), 2,
      sym_line_comment,
      sym_block_comment,
  [79900] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(407), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(1753), 1,
      sym_block,
    STATE(3620), 1,
      sym_label,
    STATE(2601), 2,
      sym_line_comment,
      sym_block_comment,
  [79923] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5730), 1,
      anon_sym_LPAREN,
    ACTIONS(5732), 1,
      anon_sym_LBRACK,
    ACTIONS(5734), 1,
      anon_sym_LBRACE,
    STATE(1024), 1,
      sym_delim_token_tree,
    STATE(2602), 2,
      sym_line_comment,
      sym_block_comment,
  [79946] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(407), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(1756), 1,
      sym_block,
    STATE(3620), 1,
      sym_label,
    STATE(2603), 2,
      sym_line_comment,
      sym_block_comment,
  [79969] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(407), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(1567), 1,
      sym_block,
    STATE(3620), 1,
      sym_label,
    STATE(2604), 2,
      sym_line_comment,
      sym_block_comment,
  [79992] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(407), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(1759), 1,
      sym_block,
    STATE(3620), 1,
      sym_label,
    STATE(2605), 2,
      sym_line_comment,
      sym_block_comment,
  [80015] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5736), 1,
      anon_sym_DQUOTE,
    STATE(2734), 1,
      aux_sym_string_literal_repeat1,
    ACTIONS(5660), 2,
      sym_string_content,
      sym_escape_sequence,
    STATE(2606), 2,
      sym_line_comment,
      sym_block_comment,
  [80036] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5730), 1,
      anon_sym_LPAREN,
    ACTIONS(5732), 1,
      anon_sym_LBRACK,
    ACTIONS(5734), 1,
      anon_sym_LBRACE,
    STATE(1026), 1,
      sym_delim_token_tree,
    STATE(2607), 2,
      sym_line_comment,
      sym_block_comment,
  [80059] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(5195), 1,
      sym_super,
    ACTIONS(5720), 1,
      sym_identifier,
    STATE(3158), 1,
      sym_type_arguments,
    STATE(2608), 2,
      sym_line_comment,
      sym_block_comment,
  [80082] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(5195), 1,
      sym_super,
    ACTIONS(5720), 1,
      sym_identifier,
    STATE(3311), 1,
      sym_type_arguments,
    STATE(2609), 2,
      sym_line_comment,
      sym_block_comment,
  [80105] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1499), 1,
      anon_sym_RPAREN,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5486), 1,
      anon_sym_COMMA,
    STATE(2869), 1,
      aux_sym_parameters_repeat1,
    STATE(2610), 2,
      sym_line_comment,
      sym_block_comment,
  [80128] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5738), 1,
      anon_sym_LPAREN,
    ACTIONS(5740), 1,
      anon_sym_LBRACK,
    ACTIONS(5742), 1,
      anon_sym_LBRACE,
    STATE(2636), 1,
      sym_token_tree,
    STATE(2611), 2,
      sym_line_comment,
      sym_block_comment,
  [80151] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4507), 2,
      anon_sym_COLON,
      anon_sym_PIPE,
    ACTIONS(5744), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(2612), 2,
      sym_line_comment,
      sym_block_comment,
  [80170] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(407), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(1776), 1,
      sym_block,
    STATE(3620), 1,
      sym_label,
    STATE(2613), 2,
      sym_line_comment,
      sym_block_comment,
  [80193] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5746), 1,
      anon_sym_COLON_COLON,
    STATE(2614), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4720), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [80212] = 7,
    ACTIONS(19), 1,
      anon_sym_LBRACE,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(385), 1,
      sym_block,
    STATE(3355), 1,
      sym_label,
    STATE(2615), 2,
      sym_line_comment,
      sym_block_comment,
  [80235] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3297), 1,
      anon_sym_LPAREN,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    STATE(1119), 1,
      sym_parameters,
    STATE(1955), 1,
      sym_type_arguments,
    STATE(2616), 2,
      sym_line_comment,
      sym_block_comment,
  [80258] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(407), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(1782), 1,
      sym_block,
    STATE(3620), 1,
      sym_label,
    STATE(2617), 2,
      sym_line_comment,
      sym_block_comment,
  [80281] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5748), 1,
      anon_sym_RPAREN,
    ACTIONS(5750), 1,
      anon_sym_COMMA,
    STATE(2923), 1,
      aux_sym_ordered_field_declaration_list_repeat1,
    STATE(2618), 2,
      sym_line_comment,
      sym_block_comment,
  [80304] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5752), 1,
      anon_sym_DQUOTE,
    STATE(2623), 1,
      aux_sym_string_literal_repeat1,
    ACTIONS(5660), 2,
      sym_string_content,
      sym_escape_sequence,
    STATE(2619), 2,
      sym_line_comment,
      sym_block_comment,
  [80325] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(407), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(1788), 1,
      sym_block,
    STATE(3620), 1,
      sym_label,
    STATE(2620), 2,
      sym_line_comment,
      sym_block_comment,
  [80348] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5363), 1,
      anon_sym_LBRACE,
    STATE(594), 1,
      sym_enum_variant_list,
    STATE(3131), 1,
      sym_where_clause,
    STATE(2621), 2,
      sym_line_comment,
      sym_block_comment,
  [80371] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5754), 1,
      anon_sym_LPAREN,
    ACTIONS(5756), 1,
      anon_sym_LBRACK,
    ACTIONS(5758), 1,
      anon_sym_LBRACE,
    STATE(1792), 1,
      sym_delim_token_tree,
    STATE(2622), 2,
      sym_line_comment,
      sym_block_comment,
  [80394] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5760), 1,
      anon_sym_DQUOTE,
    STATE(2734), 1,
      aux_sym_string_literal_repeat1,
    ACTIONS(5660), 2,
      sym_string_content,
      sym_escape_sequence,
    STATE(2623), 2,
      sym_line_comment,
      sym_block_comment,
  [80415] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5754), 1,
      anon_sym_LPAREN,
    ACTIONS(5756), 1,
      anon_sym_LBRACK,
    ACTIONS(5758), 1,
      anon_sym_LBRACE,
    STATE(1795), 1,
      sym_delim_token_tree,
    STATE(2624), 2,
      sym_line_comment,
      sym_block_comment,
  [80438] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(5654), 1,
      sym_super,
    ACTIONS(5762), 1,
      sym_identifier,
    STATE(3158), 1,
      sym_type_arguments,
    STATE(2625), 2,
      sym_line_comment,
      sym_block_comment,
  [80461] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(5654), 1,
      sym_super,
    ACTIONS(5762), 1,
      sym_identifier,
    STATE(3311), 1,
      sym_type_arguments,
    STATE(2626), 2,
      sym_line_comment,
      sym_block_comment,
  [80484] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(343), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(1422), 1,
      sym_block,
    STATE(3569), 1,
      sym_label,
    STATE(2627), 2,
      sym_line_comment,
      sym_block_comment,
  [80507] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5764), 1,
      anon_sym_COLON_COLON,
    STATE(2628), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4720), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [80526] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(343), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(1427), 1,
      sym_block,
    STATE(3569), 1,
      sym_label,
    STATE(2629), 2,
      sym_line_comment,
      sym_block_comment,
  [80549] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5766), 1,
      anon_sym_DQUOTE,
    STATE(2631), 1,
      aux_sym_string_literal_repeat1,
    ACTIONS(5660), 2,
      sym_string_content,
      sym_escape_sequence,
    STATE(2630), 2,
      sym_line_comment,
      sym_block_comment,
  [80570] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5768), 1,
      anon_sym_DQUOTE,
    STATE(2734), 1,
      aux_sym_string_literal_repeat1,
    ACTIONS(5660), 2,
      sym_string_content,
      sym_escape_sequence,
    STATE(2631), 2,
      sym_line_comment,
      sym_block_comment,
  [80591] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5770), 1,
      anon_sym_COLON_COLON,
    STATE(2632), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4720), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [80610] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(5654), 1,
      sym_super,
    ACTIONS(5772), 1,
      sym_identifier,
    STATE(3158), 1,
      sym_type_arguments,
    STATE(2633), 2,
      sym_line_comment,
      sym_block_comment,
  [80633] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(5654), 1,
      sym_super,
    ACTIONS(5772), 1,
      sym_identifier,
    STATE(3311), 1,
      sym_type_arguments,
    STATE(2634), 2,
      sym_line_comment,
      sym_block_comment,
  [80656] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5774), 1,
      anon_sym_DQUOTE,
    STATE(2702), 1,
      aux_sym_string_literal_repeat1,
    ACTIONS(5660), 2,
      sym_string_content,
      sym_escape_sequence,
    STATE(2635), 2,
      sym_line_comment,
      sym_block_comment,
  [80677] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2636), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5776), 4,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
  [80694] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(343), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(1426), 1,
      sym_block,
    STATE(3569), 1,
      sym_label,
    STATE(2637), 2,
      sym_line_comment,
      sym_block_comment,
  [80717] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5482), 1,
      anon_sym_PIPE,
    STATE(2638), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5778), 3,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_COMMA,
  [80736] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(5642), 1,
      sym_super,
    ACTIONS(5780), 1,
      sym_identifier,
    STATE(3158), 1,
      sym_type_arguments,
    STATE(2639), 2,
      sym_line_comment,
      sym_block_comment,
  [80759] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(5642), 1,
      sym_super,
    ACTIONS(5780), 1,
      sym_identifier,
    STATE(3311), 1,
      sym_type_arguments,
    STATE(2640), 2,
      sym_line_comment,
      sym_block_comment,
  [80782] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5782), 1,
      anon_sym_COMMA,
    ACTIONS(5778), 2,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
    STATE(2641), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_slice_pattern_repeat1,
  [80801] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5785), 1,
      anon_sym_SEMI,
    STATE(3389), 1,
      sym_where_clause,
    STATE(2642), 2,
      sym_line_comment,
      sym_block_comment,
  [80824] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(5195), 1,
      sym_super,
    ACTIONS(5762), 1,
      sym_identifier,
    STATE(3158), 1,
      sym_type_arguments,
    STATE(2643), 2,
      sym_line_comment,
      sym_block_comment,
  [80847] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(5195), 1,
      sym_super,
    ACTIONS(5762), 1,
      sym_identifier,
    STATE(3311), 1,
      sym_type_arguments,
    STATE(2644), 2,
      sym_line_comment,
      sym_block_comment,
  [80870] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(5654), 1,
      sym_super,
    ACTIONS(5787), 1,
      sym_identifier,
    STATE(3158), 1,
      sym_type_arguments,
    STATE(2645), 2,
      sym_line_comment,
      sym_block_comment,
  [80893] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    STATE(725), 1,
      sym_declaration_list,
    STATE(3276), 1,
      sym_where_clause,
    STATE(2646), 2,
      sym_line_comment,
      sym_block_comment,
  [80916] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(5654), 1,
      sym_super,
    ACTIONS(5789), 1,
      sym_identifier,
    STATE(3158), 1,
      sym_type_arguments,
    STATE(2647), 2,
      sym_line_comment,
      sym_block_comment,
  [80939] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(5654), 1,
      sym_super,
    ACTIONS(5789), 1,
      sym_identifier,
    STATE(3311), 1,
      sym_type_arguments,
    STATE(2648), 2,
      sym_line_comment,
      sym_block_comment,
  [80962] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(5654), 1,
      sym_super,
    ACTIONS(5791), 1,
      sym_identifier,
    STATE(3158), 1,
      sym_type_arguments,
    STATE(2649), 2,
      sym_line_comment,
      sym_block_comment,
  [80985] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(5654), 1,
      sym_super,
    ACTIONS(5791), 1,
      sym_identifier,
    STATE(3311), 1,
      sym_type_arguments,
    STATE(2650), 2,
      sym_line_comment,
      sym_block_comment,
  [81008] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(5195), 1,
      sym_super,
    ACTIONS(5791), 1,
      sym_identifier,
    STATE(3158), 1,
      sym_type_arguments,
    STATE(2651), 2,
      sym_line_comment,
      sym_block_comment,
  [81031] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(5195), 1,
      sym_super,
    ACTIONS(5791), 1,
      sym_identifier,
    STATE(3311), 1,
      sym_type_arguments,
    STATE(2652), 2,
      sym_line_comment,
      sym_block_comment,
  [81054] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2653), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3257), 4,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_GT,
      anon_sym_COMMA,
  [81071] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1601), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(2111), 1,
      sym_block,
    STATE(3611), 1,
      sym_label,
    STATE(2654), 2,
      sym_line_comment,
      sym_block_comment,
  [81094] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3305), 1,
      anon_sym_LT2,
    ACTIONS(5195), 1,
      sym_super,
    ACTIONS(5793), 1,
      sym_identifier,
    STATE(1373), 1,
      sym_type_arguments,
    STATE(2655), 2,
      sym_line_comment,
      sym_block_comment,
  [81117] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1229), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(464), 1,
      sym_block,
    STATE(3619), 1,
      sym_label,
    STATE(2656), 2,
      sym_line_comment,
      sym_block_comment,
  [81140] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4517), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5225), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4519), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(2657), 2,
      sym_line_comment,
      sym_block_comment,
  [81161] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    ACTIONS(4944), 1,
      anon_sym_LT,
    STATE(2207), 1,
      sym_parameters,
    STATE(3323), 1,
      sym_type_parameters,
    STATE(2658), 2,
      sym_line_comment,
      sym_block_comment,
  [81184] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4553), 1,
      anon_sym_BANG,
    ACTIONS(4587), 1,
      anon_sym_COLON_COLON,
    STATE(1952), 1,
      sym_type_arguments,
    STATE(2659), 2,
      sym_line_comment,
      sym_block_comment,
  [81207] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    ACTIONS(4944), 1,
      anon_sym_LT,
    STATE(2211), 1,
      sym_parameters,
    STATE(3235), 1,
      sym_type_parameters,
    STATE(2660), 2,
      sym_line_comment,
      sym_block_comment,
  [81230] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    STATE(2661), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5795), 3,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_COMMA,
  [81249] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5797), 1,
      anon_sym_in,
    STATE(2662), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5799), 3,
      sym_self,
      sym_super,
      sym_crate,
  [81268] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5801), 1,
      anon_sym_LBRACE,
    ACTIONS(5803), 1,
      anon_sym_for,
    ACTIONS(5805), 1,
      anon_sym_loop,
    ACTIONS(5807), 1,
      anon_sym_while,
    STATE(2663), 2,
      sym_line_comment,
      sym_block_comment,
  [81291] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2664), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5603), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
  [81308] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4664), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5809), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4666), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(2665), 2,
      sym_line_comment,
      sym_block_comment,
  [81329] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    STATE(681), 1,
      sym_declaration_list,
    STATE(3284), 1,
      sym_where_clause,
    STATE(2666), 2,
      sym_line_comment,
      sym_block_comment,
  [81352] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    STATE(2832), 1,
      sym_trait_bounds,
    ACTIONS(5811), 2,
      anon_sym_GT,
      anon_sym_COMMA,
    STATE(2667), 2,
      sym_line_comment,
      sym_block_comment,
  [81373] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3305), 1,
      anon_sym_LT2,
    ACTIONS(3525), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4832), 1,
      anon_sym_BANG,
    STATE(1087), 1,
      sym_type_arguments,
    STATE(2668), 2,
      sym_line_comment,
      sym_block_comment,
  [81396] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5482), 1,
      anon_sym_PIPE,
    ACTIONS(5814), 1,
      anon_sym_RPAREN,
    ACTIONS(5816), 1,
      anon_sym_COMMA,
    STATE(2995), 1,
      aux_sym_slice_pattern_repeat1,
    STATE(2669), 2,
      sym_line_comment,
      sym_block_comment,
  [81419] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5482), 1,
      anon_sym_PIPE,
    ACTIONS(5818), 1,
      anon_sym_RBRACK,
    ACTIONS(5820), 1,
      anon_sym_COMMA,
    STATE(2943), 1,
      aux_sym_slice_pattern_repeat1,
    STATE(2670), 2,
      sym_line_comment,
      sym_block_comment,
  [81442] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5478), 1,
      anon_sym_RPAREN,
    ACTIONS(5482), 1,
      anon_sym_PIPE,
    ACTIONS(5484), 1,
      anon_sym_COMMA,
    STATE(2987), 1,
      aux_sym_slice_pattern_repeat1,
    STATE(2671), 2,
      sym_line_comment,
      sym_block_comment,
  [81465] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(343), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(3461), 1,
      sym_block,
    STATE(3569), 1,
      sym_label,
    STATE(2672), 2,
      sym_line_comment,
      sym_block_comment,
  [81488] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5822), 1,
      aux_sym_token_repetition_pattern_token1,
    STATE(2673), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5824), 3,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_QMARK,
  [81507] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    ACTIONS(4944), 1,
      anon_sym_LT,
    STATE(2216), 1,
      sym_parameters,
    STATE(3261), 1,
      sym_type_parameters,
    STATE(2674), 2,
      sym_line_comment,
      sym_block_comment,
  [81530] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2675), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(989), 4,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
  [81547] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(343), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(1443), 1,
      sym_block,
    STATE(3569), 1,
      sym_label,
    STATE(2676), 2,
      sym_line_comment,
      sym_block_comment,
  [81570] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5179), 1,
      anon_sym_LPAREN,
    ACTIONS(5181), 1,
      anon_sym_LBRACK,
    ACTIONS(5185), 1,
      anon_sym_LBRACE,
    STATE(1494), 1,
      sym_delim_token_tree,
    STATE(2677), 2,
      sym_line_comment,
      sym_block_comment,
  [81593] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5826), 1,
      anon_sym_SEMI,
    ACTIONS(5828), 1,
      anon_sym_EQ,
    ACTIONS(5830), 1,
      anon_sym_else,
    STATE(2678), 2,
      sym_line_comment,
      sym_block_comment,
  [81616] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(343), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(1446), 1,
      sym_block,
    STATE(3569), 1,
      sym_label,
    STATE(2679), 2,
      sym_line_comment,
      sym_block_comment,
  [81639] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5832), 1,
      anon_sym_SEMI,
    ACTIONS(5834), 1,
      anon_sym_EQ,
    ACTIONS(5836), 1,
      anon_sym_else,
    STATE(2680), 2,
      sym_line_comment,
      sym_block_comment,
  [81662] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(343), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(1098), 1,
      sym_block,
    STATE(3569), 1,
      sym_label,
    STATE(2681), 2,
      sym_line_comment,
      sym_block_comment,
  [81685] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4976), 1,
      anon_sym_LBRACE,
    STATE(1141), 1,
      sym_field_declaration_list,
    STATE(3238), 1,
      sym_where_clause,
    STATE(2682), 2,
      sym_line_comment,
      sym_block_comment,
  [81708] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5722), 1,
      anon_sym_COLON_COLON,
    STATE(2683), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4802), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [81727] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5838), 1,
      anon_sym_LBRACE,
    ACTIONS(5840), 1,
      anon_sym_for,
    ACTIONS(5842), 1,
      anon_sym_loop,
    ACTIONS(5844), 1,
      anon_sym_while,
    STATE(2684), 2,
      sym_line_comment,
      sym_block_comment,
  [81750] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5846), 1,
      aux_sym_token_repetition_pattern_token1,
    STATE(2685), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5848), 3,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_QMARK,
  [81769] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5850), 1,
      anon_sym_SEMI,
    STATE(3465), 1,
      sym_where_clause,
    STATE(2686), 2,
      sym_line_comment,
      sym_block_comment,
  [81792] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(5674), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5852), 1,
      anon_sym_for,
    STATE(1953), 1,
      sym_type_arguments,
    STATE(2687), 2,
      sym_line_comment,
      sym_block_comment,
  [81815] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(343), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(3368), 1,
      sym_block,
    STATE(3569), 1,
      sym_label,
    STATE(2688), 2,
      sym_line_comment,
      sym_block_comment,
  [81838] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4507), 2,
      anon_sym_COLON,
      anon_sym_PIPE,
    ACTIONS(5854), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(2689), 2,
      sym_line_comment,
      sym_block_comment,
  [81857] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5856), 1,
      anon_sym_COLON,
    STATE(2690), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4856), 3,
      anon_sym_RPAREN,
      anon_sym_PIPE,
      anon_sym_COMMA,
  [81876] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5858), 1,
      aux_sym_token_repetition_pattern_token1,
    STATE(2691), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5860), 3,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_QMARK,
  [81895] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    STATE(1242), 1,
      sym_declaration_list,
    STATE(3278), 1,
      sym_where_clause,
    STATE(2692), 2,
      sym_line_comment,
      sym_block_comment,
  [81918] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1501), 1,
      anon_sym_RPAREN,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5622), 1,
      anon_sym_COMMA,
    STATE(2897), 1,
      aux_sym_parameters_repeat1,
    STATE(2693), 2,
      sym_line_comment,
      sym_block_comment,
  [81941] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    ACTIONS(4944), 1,
      anon_sym_LT,
    STATE(2222), 1,
      sym_parameters,
    STATE(3275), 1,
      sym_type_parameters,
    STATE(2694), 2,
      sym_line_comment,
      sym_block_comment,
  [81964] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(343), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(1467), 1,
      sym_block,
    STATE(3569), 1,
      sym_label,
    STATE(2695), 2,
      sym_line_comment,
      sym_block_comment,
  [81987] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(343), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(3447), 1,
      sym_block,
    STATE(3569), 1,
      sym_label,
    STATE(2696), 2,
      sym_line_comment,
      sym_block_comment,
  [82010] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(343), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(3452), 1,
      sym_block,
    STATE(3569), 1,
      sym_label,
    STATE(2697), 2,
      sym_line_comment,
      sym_block_comment,
  [82033] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    STATE(1955), 1,
      sym_type_arguments,
    STATE(2725), 1,
      sym_trait_bounds,
    STATE(2698), 2,
      sym_line_comment,
      sym_block_comment,
  [82056] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5746), 1,
      anon_sym_COLON_COLON,
    STATE(2699), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4734), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [82075] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5482), 1,
      anon_sym_PIPE,
    ACTIONS(5862), 1,
      anon_sym_RPAREN,
    ACTIONS(5864), 1,
      anon_sym_COMMA,
    STATE(2825), 1,
      aux_sym_slice_pattern_repeat1,
    STATE(2700), 2,
      sym_line_comment,
      sym_block_comment,
  [82098] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5866), 1,
      anon_sym_SEMI,
    STATE(3498), 1,
      sym_where_clause,
    STATE(2701), 2,
      sym_line_comment,
      sym_block_comment,
  [82121] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5868), 1,
      anon_sym_DQUOTE,
    STATE(2734), 1,
      aux_sym_string_literal_repeat1,
    ACTIONS(5660), 2,
      sym_string_content,
      sym_escape_sequence,
    STATE(2702), 2,
      sym_line_comment,
      sym_block_comment,
  [82142] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5179), 1,
      anon_sym_LPAREN,
    ACTIONS(5181), 1,
      anon_sym_LBRACK,
    ACTIONS(5185), 1,
      anon_sym_LBRACE,
    STATE(1475), 1,
      sym_delim_token_tree,
    STATE(2703), 2,
      sym_line_comment,
      sym_block_comment,
  [82165] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(343), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(3569), 1,
      sym_label,
    STATE(3590), 1,
      sym_block,
    STATE(2704), 2,
      sym_line_comment,
      sym_block_comment,
  [82188] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(343), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(3569), 1,
      sym_label,
    STATE(3592), 1,
      sym_block,
    STATE(2705), 2,
      sym_line_comment,
      sym_block_comment,
  [82211] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5331), 1,
      anon_sym_PLUS,
    STATE(2706), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5870), 3,
      anon_sym_COLON,
      anon_sym_GT,
      anon_sym_COMMA,
  [82230] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(343), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(3569), 1,
      sym_label,
    STATE(3591), 1,
      sym_block,
    STATE(2707), 2,
      sym_line_comment,
      sym_block_comment,
  [82253] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3305), 1,
      anon_sym_LT2,
    ACTIONS(5195), 1,
      sym_super,
    ACTIONS(5872), 1,
      sym_identifier,
    STATE(1592), 1,
      sym_type_arguments,
    STATE(2708), 2,
      sym_line_comment,
      sym_block_comment,
  [82276] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5874), 1,
      anon_sym_COMMA,
    ACTIONS(4157), 2,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
    STATE(2709), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_arguments_repeat1,
  [82295] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(343), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(3569), 1,
      sym_label,
    STATE(3579), 1,
      sym_block,
    STATE(2710), 2,
      sym_line_comment,
      sym_block_comment,
  [82318] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    STATE(1256), 1,
      sym_declaration_list,
    STATE(3289), 1,
      sym_where_clause,
    STATE(2711), 2,
      sym_line_comment,
      sym_block_comment,
  [82341] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5764), 1,
      anon_sym_COLON_COLON,
    STATE(2712), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4734), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [82360] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5877), 1,
      anon_sym_DQUOTE,
    STATE(2734), 1,
      aux_sym_string_literal_repeat1,
    ACTIONS(5660), 2,
      sym_string_content,
      sym_escape_sequence,
    STATE(2713), 2,
      sym_line_comment,
      sym_block_comment,
  [82381] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3305), 1,
      anon_sym_LT2,
    ACTIONS(5195), 1,
      sym_super,
    ACTIONS(5872), 1,
      sym_identifier,
    STATE(1568), 1,
      sym_type_arguments,
    STATE(2714), 2,
      sym_line_comment,
      sym_block_comment,
  [82404] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(343), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(3533), 1,
      sym_block,
    STATE(3569), 1,
      sym_label,
    STATE(2715), 2,
      sym_line_comment,
      sym_block_comment,
  [82427] = 7,
    ACTIONS(19), 1,
      anon_sym_LBRACE,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(398), 1,
      sym_block,
    STATE(3355), 1,
      sym_label,
    STATE(2716), 2,
      sym_line_comment,
      sym_block_comment,
  [82450] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4517), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5231), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4519), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(2717), 2,
      sym_line_comment,
      sym_block_comment,
  [82471] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5770), 1,
      anon_sym_COLON_COLON,
    STATE(2718), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4734), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [82490] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4175), 1,
      anon_sym_LBRACE,
    STATE(2765), 1,
      sym_use_list,
    ACTIONS(5879), 2,
      sym_identifier,
      sym_super,
    STATE(2719), 2,
      sym_line_comment,
      sym_block_comment,
  [82511] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5881), 1,
      anon_sym_LBRACE,
    ACTIONS(5883), 1,
      anon_sym_for,
    ACTIONS(5885), 1,
      anon_sym_loop,
    ACTIONS(5887), 1,
      anon_sym_while,
    STATE(2720), 2,
      sym_line_comment,
      sym_block_comment,
  [82534] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    STATE(3116), 1,
      sym_trait_bounds,
    ACTIONS(5626), 2,
      anon_sym_GT,
      anon_sym_COMMA,
    STATE(2721), 2,
      sym_line_comment,
      sym_block_comment,
  [82555] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    STATE(2832), 1,
      sym_trait_bounds,
    ACTIONS(5889), 2,
      anon_sym_GT,
      anon_sym_COMMA,
    STATE(2722), 2,
      sym_line_comment,
      sym_block_comment,
  [82576] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5482), 1,
      anon_sym_PIPE,
    ACTIONS(5891), 1,
      anon_sym_RPAREN,
    ACTIONS(5893), 1,
      anon_sym_COMMA,
    STATE(2828), 1,
      aux_sym_slice_pattern_repeat1,
    STATE(2723), 2,
      sym_line_comment,
      sym_block_comment,
  [82599] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5482), 1,
      anon_sym_PIPE,
    ACTIONS(5895), 1,
      anon_sym_RPAREN,
    ACTIONS(5897), 1,
      anon_sym_COMMA,
    STATE(2868), 1,
      aux_sym_tuple_pattern_repeat1,
    STATE(2724), 2,
      sym_line_comment,
      sym_block_comment,
  [82622] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2725), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5899), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
  [82639] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2726), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5901), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SQUOTE,
  [82656] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    STATE(521), 1,
      sym_declaration_list,
    STATE(3216), 1,
      sym_where_clause,
    STATE(2727), 2,
      sym_line_comment,
      sym_block_comment,
  [82679] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5903), 1,
      anon_sym_RPAREN,
    ACTIONS(5905), 1,
      anon_sym_COMMA,
    STATE(2906), 1,
      aux_sym_tuple_type_repeat1,
    STATE(2728), 2,
      sym_line_comment,
      sym_block_comment,
  [82702] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4517), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5259), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4519), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(2729), 2,
      sym_line_comment,
      sym_block_comment,
  [82723] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5708), 1,
      anon_sym_LPAREN,
    ACTIONS(5710), 1,
      anon_sym_LBRACK,
    ACTIONS(5712), 1,
      anon_sym_LBRACE,
    STATE(387), 1,
      sym_delim_token_tree,
    STATE(2730), 2,
      sym_line_comment,
      sym_block_comment,
  [82746] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5907), 1,
      anon_sym_LBRACE,
    ACTIONS(5909), 1,
      anon_sym_for,
    ACTIONS(5911), 1,
      anon_sym_loop,
    ACTIONS(5913), 1,
      anon_sym_while,
    STATE(2731), 2,
      sym_line_comment,
      sym_block_comment,
  [82769] = 7,
    ACTIONS(19), 1,
      anon_sym_LBRACE,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(332), 1,
      sym_block,
    STATE(3355), 1,
      sym_label,
    STATE(2732), 2,
      sym_line_comment,
      sym_block_comment,
  [82792] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5363), 1,
      anon_sym_LBRACE,
    STATE(731), 1,
      sym_enum_variant_list,
    STATE(3065), 1,
      sym_where_clause,
    STATE(2733), 2,
      sym_line_comment,
      sym_block_comment,
  [82815] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5915), 1,
      anon_sym_DQUOTE,
    ACTIONS(5917), 2,
      sym_string_content,
      sym_escape_sequence,
    STATE(2734), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_string_literal_repeat1,
  [82834] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5920), 1,
      sym_identifier,
    STATE(2735), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5922), 3,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_union,
  [82853] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5285), 1,
      anon_sym_LBRACE,
    STATE(1122), 1,
      sym_enum_variant_list,
    STATE(3108), 1,
      sym_where_clause,
    STATE(2736), 2,
      sym_line_comment,
      sym_block_comment,
  [82876] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5482), 1,
      anon_sym_PIPE,
    ACTIONS(5924), 1,
      anon_sym_RPAREN,
    ACTIONS(5926), 1,
      anon_sym_COMMA,
    STATE(2773), 1,
      aux_sym_tuple_pattern_repeat1,
    STATE(2737), 2,
      sym_line_comment,
      sym_block_comment,
  [82899] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5482), 1,
      anon_sym_PIPE,
    ACTIONS(5928), 1,
      anon_sym_RBRACK,
    ACTIONS(5930), 1,
      anon_sym_COMMA,
    STATE(2775), 1,
      aux_sym_slice_pattern_repeat1,
    STATE(2738), 2,
      sym_line_comment,
      sym_block_comment,
  [82922] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(5654), 1,
      sym_super,
    ACTIONS(5720), 1,
      sym_identifier,
    STATE(3311), 1,
      sym_type_arguments,
    STATE(2739), 2,
      sym_line_comment,
      sym_block_comment,
  [82945] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(5654), 1,
      sym_super,
    ACTIONS(5696), 1,
      sym_identifier,
    STATE(3158), 1,
      sym_type_arguments,
    STATE(2740), 2,
      sym_line_comment,
      sym_block_comment,
  [82968] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    STATE(1955), 1,
      sym_type_arguments,
    STATE(1961), 1,
      sym_parameters,
    STATE(2741), 2,
      sym_line_comment,
      sym_block_comment,
  [82991] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4796), 1,
      anon_sym_for,
    ACTIONS(5674), 1,
      anon_sym_COLON_COLON,
    STATE(1953), 1,
      sym_type_arguments,
    STATE(2742), 2,
      sym_line_comment,
      sym_block_comment,
  [83014] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    STATE(714), 1,
      sym_declaration_list,
    STATE(3152), 1,
      sym_where_clause,
    STATE(2743), 2,
      sym_line_comment,
      sym_block_comment,
  [83037] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(5654), 1,
      sym_super,
    ACTIONS(5696), 1,
      sym_identifier,
    STATE(3311), 1,
      sym_type_arguments,
    STATE(2744), 2,
      sym_line_comment,
      sym_block_comment,
  [83060] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    STATE(624), 1,
      sym_declaration_list,
    STATE(3240), 1,
      sym_where_clause,
    STATE(2745), 2,
      sym_line_comment,
      sym_block_comment,
  [83083] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5506), 1,
      anon_sym_RPAREN,
    ACTIONS(5508), 1,
      anon_sym_COMMA,
    STATE(3009), 1,
      aux_sym_parameters_repeat1,
    STATE(2746), 2,
      sym_line_comment,
      sym_block_comment,
  [83106] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5932), 1,
      anon_sym_RPAREN,
    ACTIONS(5934), 1,
      anon_sym_COMMA,
    STATE(2779), 1,
      aux_sym_tuple_type_repeat1,
    STATE(2747), 2,
      sym_line_comment,
      sym_block_comment,
  [83129] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(5674), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5936), 1,
      anon_sym_for,
    STATE(1953), 1,
      sym_type_arguments,
    STATE(2748), 2,
      sym_line_comment,
      sym_block_comment,
  [83152] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(5674), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5938), 1,
      anon_sym_for,
    STATE(1953), 1,
      sym_type_arguments,
    STATE(2749), 2,
      sym_line_comment,
      sym_block_comment,
  [83175] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5940), 1,
      anon_sym_SEMI,
    STATE(3492), 1,
      sym_where_clause,
    STATE(2750), 2,
      sym_line_comment,
      sym_block_comment,
  [83198] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3305), 1,
      anon_sym_LT2,
    ACTIONS(5195), 1,
      sym_super,
    ACTIONS(5793), 1,
      sym_identifier,
    STATE(1456), 1,
      sym_type_arguments,
    STATE(2751), 2,
      sym_line_comment,
      sym_block_comment,
  [83221] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(5674), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5942), 1,
      anon_sym_for,
    STATE(1953), 1,
      sym_type_arguments,
    STATE(2752), 2,
      sym_line_comment,
      sym_block_comment,
  [83244] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5944), 1,
      anon_sym_in,
    STATE(2753), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5946), 3,
      sym_self,
      sym_super,
      sym_crate,
  [83263] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5455), 1,
      anon_sym_RPAREN,
    ACTIONS(5457), 1,
      anon_sym_COMMA,
    STATE(2835), 1,
      aux_sym_parameters_repeat1,
    STATE(2754), 2,
      sym_line_comment,
      sym_block_comment,
  [83286] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(343), 1,
      anon_sym_LBRACE,
    ACTIONS(3291), 1,
      anon_sym_SQUOTE,
    STATE(3488), 1,
      sym_block,
    STATE(3569), 1,
      sym_label,
    STATE(2755), 2,
      sym_line_comment,
      sym_block_comment,
  [83309] = 7,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(5654), 1,
      sym_super,
    ACTIONS(5787), 1,
      sym_identifier,
    STATE(3311), 1,
      sym_type_arguments,
    STATE(2756), 2,
      sym_line_comment,
      sym_block_comment,
  [83332] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2757), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4507), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [83348] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5948), 1,
      anon_sym_RBRACE,
    ACTIONS(5950), 1,
      anon_sym_COMMA,
    STATE(2808), 1,
      aux_sym_field_initializer_list_repeat1,
    STATE(2758), 2,
      sym_line_comment,
      sym_block_comment,
  [83368] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(5952), 1,
      anon_sym_for,
    STATE(1955), 1,
      sym_type_arguments,
    STATE(2759), 2,
      sym_line_comment,
      sym_block_comment,
  [83388] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(5954), 1,
      anon_sym_SEMI,
    STATE(1149), 1,
      sym_declaration_list,
    STATE(2760), 2,
      sym_line_comment,
      sym_block_comment,
  [83408] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(5956), 1,
      anon_sym_SEMI,
    STATE(606), 1,
      sym_declaration_list,
    STATE(2761), 2,
      sym_line_comment,
      sym_block_comment,
  [83428] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(5958), 1,
      anon_sym_SEMI,
    STATE(755), 1,
      sym_declaration_list,
    STATE(2762), 2,
      sym_line_comment,
      sym_block_comment,
  [83448] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2763), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5960), 3,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_QMARK,
  [83464] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(5962), 1,
      anon_sym_for,
    STATE(1955), 1,
      sym_type_arguments,
    STATE(2764), 2,
      sym_line_comment,
      sym_block_comment,
  [83484] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2765), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5964), 3,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
  [83500] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(5966), 1,
      anon_sym_SEMI,
    STATE(757), 1,
      sym_declaration_list,
    STATE(2766), 2,
      sym_line_comment,
      sym_block_comment,
  [83520] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5968), 2,
      anon_sym_GT,
      anon_sym_COMMA,
    STATE(2767), 2,
      sym_line_comment,
      sym_block_comment,
  [83538] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5451), 1,
      anon_sym_COMMA,
    ACTIONS(5970), 1,
      anon_sym_PIPE,
    STATE(3034), 1,
      aux_sym_closure_parameters_repeat1,
    STATE(2768), 2,
      sym_line_comment,
      sym_block_comment,
  [83558] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5972), 1,
      anon_sym_RBRACE,
    ACTIONS(5974), 1,
      anon_sym_COMMA,
    STATE(2769), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_struct_pattern_repeat1,
  [83576] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2770), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4826), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [83592] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(935), 1,
      anon_sym_RBRACK,
    ACTIONS(4149), 1,
      anon_sym_COMMA,
    STATE(2709), 1,
      aux_sym_arguments_repeat1,
    STATE(2771), 2,
      sym_line_comment,
      sym_block_comment,
  [83612] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2772), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4836), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [83628] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2989), 1,
      anon_sym_RPAREN,
    ACTIONS(5977), 1,
      anon_sym_COMMA,
    STATE(2964), 1,
      aux_sym_tuple_pattern_repeat1,
    STATE(2773), 2,
      sym_line_comment,
      sym_block_comment,
  [83648] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3448), 1,
      anon_sym_LBRACE,
    ACTIONS(5979), 1,
      anon_sym_COLON_COLON,
    STATE(1477), 1,
      sym_field_initializer_list,
    STATE(2774), 2,
      sym_line_comment,
      sym_block_comment,
  [83668] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3057), 1,
      anon_sym_RBRACK,
    ACTIONS(5981), 1,
      anon_sym_COMMA,
    STATE(2641), 1,
      aux_sym_slice_pattern_repeat1,
    STATE(2775), 2,
      sym_line_comment,
      sym_block_comment,
  [83688] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5983), 1,
      anon_sym_RBRACE,
    ACTIONS(5985), 1,
      anon_sym_COMMA,
    STATE(2827), 1,
      aux_sym_struct_pattern_repeat1,
    STATE(2776), 2,
      sym_line_comment,
      sym_block_comment,
  [83708] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5987), 1,
      anon_sym_RBRACE,
    ACTIONS(5989), 1,
      anon_sym_COMMA,
    STATE(2830), 1,
      aux_sym_struct_pattern_repeat1,
    STATE(2777), 2,
      sym_line_comment,
      sym_block_comment,
  [83728] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2778), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5991), 3,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
  [83744] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3185), 1,
      anon_sym_RPAREN,
    ACTIONS(5993), 1,
      anon_sym_COMMA,
    STATE(3001), 1,
      aux_sym_tuple_type_repeat1,
    STATE(2779), 2,
      sym_line_comment,
      sym_block_comment,
  [83764] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(5995), 1,
      anon_sym_SEMI,
    STATE(723), 1,
      sym_declaration_list,
    STATE(2780), 2,
      sym_line_comment,
      sym_block_comment,
  [83784] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5666), 1,
      anon_sym_EQ,
    ACTIONS(5889), 2,
      anon_sym_GT,
      anon_sym_COMMA,
    STATE(2781), 2,
      sym_line_comment,
      sym_block_comment,
  [83802] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5455), 1,
      anon_sym_RPAREN,
    ACTIONS(5457), 1,
      anon_sym_COMMA,
    STATE(2835), 1,
      aux_sym_parameters_repeat1,
    STATE(2782), 2,
      sym_line_comment,
      sym_block_comment,
  [83822] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5997), 2,
      anon_sym_GT,
      anon_sym_COMMA,
    STATE(2783), 2,
      sym_line_comment,
      sym_block_comment,
  [83840] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5999), 1,
      anon_sym_SEMI,
    ACTIONS(6001), 1,
      anon_sym_EQ,
    STATE(2784), 2,
      sym_line_comment,
      sym_block_comment,
  [83860] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2785), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6003), 3,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
  [83876] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2786), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4896), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [83892] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6005), 1,
      anon_sym_RBRACE,
    ACTIONS(6007), 1,
      anon_sym_COMMA,
    STATE(2838), 1,
      aux_sym_enum_variant_list_repeat2,
    STATE(2787), 2,
      sym_line_comment,
      sym_block_comment,
  [83912] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6009), 1,
      anon_sym_RBRACE,
    ACTIONS(6011), 1,
      anon_sym_COMMA,
    STATE(2788), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_field_initializer_list_repeat1,
  [83930] = 6,
    ACTIONS(27), 1,
      anon_sym_PIPE,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6014), 1,
      anon_sym_move,
    STATE(243), 1,
      sym_closure_parameters,
    STATE(2789), 2,
      sym_line_comment,
      sym_block_comment,
  [83950] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4846), 1,
      anon_sym_LBRACE,
    STATE(1955), 1,
      sym_type_arguments,
    STATE(2790), 2,
      sym_line_comment,
      sym_block_comment,
  [83970] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2791), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4908), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [83986] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2792), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4810), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [84002] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4708), 1,
      anon_sym_GT,
    ACTIONS(6016), 1,
      anon_sym_COMMA,
    STATE(2913), 1,
      aux_sym_type_parameters_repeat1,
    STATE(2793), 2,
      sym_line_comment,
      sym_block_comment,
  [84022] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2794), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4816), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [84038] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4696), 1,
      anon_sym_GT,
    ACTIONS(5469), 1,
      anon_sym_COMMA,
    STATE(2913), 1,
      aux_sym_type_parameters_repeat1,
    STATE(2795), 2,
      sym_line_comment,
      sym_block_comment,
  [84058] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4696), 1,
      anon_sym_GT,
    ACTIONS(5469), 1,
      anon_sym_COMMA,
    STATE(2843), 1,
      aux_sym_type_parameters_repeat1,
    STATE(2796), 2,
      sym_line_comment,
      sym_block_comment,
  [84078] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(919), 1,
      anon_sym_RBRACK,
    ACTIONS(4165), 1,
      anon_sym_COMMA,
    STATE(2709), 1,
      aux_sym_arguments_repeat1,
    STATE(2797), 2,
      sym_line_comment,
      sym_block_comment,
  [84098] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(6018), 1,
      anon_sym_SEMI,
    STATE(621), 1,
      sym_declaration_list,
    STATE(2798), 2,
      sym_line_comment,
      sym_block_comment,
  [84118] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(6020), 1,
      anon_sym_SEMI,
    STATE(1171), 1,
      sym_declaration_list,
    STATE(2799), 2,
      sym_line_comment,
      sym_block_comment,
  [84138] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3093), 1,
      anon_sym_PLUS,
    ACTIONS(6022), 1,
      sym_mutable_specifier,
    ACTIONS(6024), 1,
      sym_self,
    STATE(2800), 2,
      sym_line_comment,
      sym_block_comment,
  [84158] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(6026), 1,
      anon_sym_SEMI,
    ACTIONS(6028), 1,
      anon_sym_EQ,
    STATE(2801), 2,
      sym_line_comment,
      sym_block_comment,
  [84178] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6030), 1,
      anon_sym_RBRACE,
    ACTIONS(6032), 1,
      anon_sym_COMMA,
    STATE(2848), 1,
      aux_sym_field_declaration_list_repeat1,
    STATE(2802), 2,
      sym_line_comment,
      sym_block_comment,
  [84198] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(6034), 1,
      anon_sym_SEMI,
    STATE(3438), 1,
      sym_where_clause,
    STATE(2803), 2,
      sym_line_comment,
      sym_block_comment,
  [84218] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2804), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6036), 3,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_QMARK,
  [84234] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(6038), 1,
      anon_sym_SEMI,
    STATE(3460), 1,
      sym_where_clause,
    STATE(2805), 2,
      sym_line_comment,
      sym_block_comment,
  [84254] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(6040), 1,
      anon_sym_SEMI,
    STATE(1192), 1,
      sym_declaration_list,
    STATE(2806), 2,
      sym_line_comment,
      sym_block_comment,
  [84274] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5854), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(2807), 2,
      sym_line_comment,
      sym_block_comment,
  [84292] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4886), 1,
      anon_sym_RBRACE,
    ACTIONS(6042), 1,
      anon_sym_COMMA,
    STATE(2788), 1,
      aux_sym_field_initializer_list_repeat1,
    STATE(2808), 2,
      sym_line_comment,
      sym_block_comment,
  [84312] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2809), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6044), 3,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
  [84328] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(6046), 1,
      anon_sym_SEMI,
    STATE(3571), 1,
      sym_where_clause,
    STATE(2810), 2,
      sym_line_comment,
      sym_block_comment,
  [84348] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4698), 1,
      anon_sym_GT,
    ACTIONS(6048), 1,
      anon_sym_COMMA,
    STATE(2913), 1,
      aux_sym_type_parameters_repeat1,
    STATE(2811), 2,
      sym_line_comment,
      sym_block_comment,
  [84368] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4325), 1,
      anon_sym_RBRACE,
    ACTIONS(6050), 1,
      anon_sym_COMMA,
    STATE(3003), 1,
      aux_sym_use_list_repeat1,
    STATE(2812), 2,
      sym_line_comment,
      sym_block_comment,
  [84388] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(949), 1,
      anon_sym_RPAREN,
    ACTIONS(4151), 1,
      anon_sym_COMMA,
    STATE(2709), 1,
      aux_sym_arguments_repeat1,
    STATE(2813), 2,
      sym_line_comment,
      sym_block_comment,
  [84408] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2814), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(997), 3,
      anon_sym_COLON,
      anon_sym_GT,
      anon_sym_COMMA,
  [84424] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2815), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6052), 3,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
  [84440] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2816), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6054), 3,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
  [84456] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2817), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6056), 3,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
  [84472] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2818), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6058), 3,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
  [84488] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2819), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6060), 3,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
  [84504] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2820), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6062), 3,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
  [84520] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(955), 1,
      anon_sym_RBRACK,
    ACTIONS(6064), 1,
      anon_sym_COMMA,
    STATE(2709), 1,
      aux_sym_arguments_repeat1,
    STATE(2821), 2,
      sym_line_comment,
      sym_block_comment,
  [84540] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(6066), 1,
      anon_sym_SEMI,
    STATE(531), 1,
      sym_declaration_list,
    STATE(2822), 2,
      sym_line_comment,
      sym_block_comment,
  [84560] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1629), 1,
      anon_sym_GT,
    ACTIONS(6068), 1,
      anon_sym_COMMA,
    STATE(2863), 1,
      aux_sym_type_arguments_repeat1,
    STATE(2823), 2,
      sym_line_comment,
      sym_block_comment,
  [84580] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1629), 1,
      anon_sym_GT,
    ACTIONS(6068), 1,
      anon_sym_COMMA,
    STATE(3037), 1,
      aux_sym_type_arguments_repeat1,
    STATE(2824), 2,
      sym_line_comment,
      sym_block_comment,
  [84600] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3067), 1,
      anon_sym_RPAREN,
    ACTIONS(6070), 1,
      anon_sym_COMMA,
    STATE(2641), 1,
      aux_sym_slice_pattern_repeat1,
    STATE(2825), 2,
      sym_line_comment,
      sym_block_comment,
  [84620] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4702), 1,
      anon_sym_GT,
    ACTIONS(5473), 1,
      anon_sym_COMMA,
    STATE(2913), 1,
      aux_sym_type_parameters_repeat1,
    STATE(2826), 2,
      sym_line_comment,
      sym_block_comment,
  [84640] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5189), 1,
      anon_sym_RBRACE,
    ACTIONS(6072), 1,
      anon_sym_COMMA,
    STATE(2769), 1,
      aux_sym_struct_pattern_repeat1,
    STATE(2827), 2,
      sym_line_comment,
      sym_block_comment,
  [84660] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3073), 1,
      anon_sym_RPAREN,
    ACTIONS(6074), 1,
      anon_sym_COMMA,
    STATE(2641), 1,
      aux_sym_slice_pattern_repeat1,
    STATE(2828), 2,
      sym_line_comment,
      sym_block_comment,
  [84680] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6076), 1,
      anon_sym_GT,
    ACTIONS(6078), 1,
      anon_sym_COMMA,
    STATE(3018), 1,
      aux_sym_for_lifetimes_repeat1,
    STATE(2829), 2,
      sym_line_comment,
      sym_block_comment,
  [84700] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5191), 1,
      anon_sym_RBRACE,
    ACTIONS(6080), 1,
      anon_sym_COMMA,
    STATE(2769), 1,
      aux_sym_struct_pattern_repeat1,
    STATE(2830), 2,
      sym_line_comment,
      sym_block_comment,
  [84720] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6082), 1,
      sym_identifier,
    ACTIONS(6084), 1,
      anon_sym_ref,
    ACTIONS(6086), 1,
      sym_mutable_specifier,
    STATE(2831), 2,
      sym_line_comment,
      sym_block_comment,
  [84740] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2832), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6088), 3,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COMMA,
  [84756] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(6090), 1,
      anon_sym_SEMI,
    STATE(533), 1,
      sym_declaration_list,
    STATE(2833), 2,
      sym_line_comment,
      sym_block_comment,
  [84776] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4830), 1,
      anon_sym_RBRACE,
    ACTIONS(6092), 1,
      anon_sym_COMMA,
    STATE(2788), 1,
      aux_sym_field_initializer_list_repeat1,
    STATE(2834), 2,
      sym_line_comment,
      sym_block_comment,
  [84796] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1499), 1,
      anon_sym_RPAREN,
    ACTIONS(5486), 1,
      anon_sym_COMMA,
    STATE(2840), 1,
      aux_sym_parameters_repeat1,
    STATE(2835), 2,
      sym_line_comment,
      sym_block_comment,
  [84816] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1499), 1,
      anon_sym_RPAREN,
    ACTIONS(5486), 1,
      anon_sym_COMMA,
    STATE(2869), 1,
      aux_sym_parameters_repeat1,
    STATE(2836), 2,
      sym_line_comment,
      sym_block_comment,
  [84836] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4702), 1,
      anon_sym_GT,
    ACTIONS(5473), 1,
      anon_sym_COMMA,
    STATE(2947), 1,
      aux_sym_type_parameters_repeat1,
    STATE(2837), 2,
      sym_line_comment,
      sym_block_comment,
  [84856] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4980), 1,
      anon_sym_RBRACE,
    ACTIONS(6094), 1,
      anon_sym_COMMA,
    STATE(3015), 1,
      aux_sym_enum_variant_list_repeat2,
    STATE(2838), 2,
      sym_line_comment,
      sym_block_comment,
  [84876] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4980), 1,
      anon_sym_RBRACE,
    ACTIONS(6094), 1,
      anon_sym_COMMA,
    STATE(2871), 1,
      aux_sym_enum_variant_list_repeat2,
    STATE(2839), 2,
      sym_line_comment,
      sym_block_comment,
  [84896] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5854), 1,
      anon_sym_RPAREN,
    ACTIONS(6096), 1,
      anon_sym_COMMA,
    STATE(2840), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_parameters_repeat1,
  [84914] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(6099), 1,
      anon_sym_for,
    STATE(1955), 1,
      sym_type_arguments,
    STATE(2841), 2,
      sym_line_comment,
      sym_block_comment,
  [84934] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4686), 1,
      anon_sym_GT,
    ACTIONS(6101), 1,
      anon_sym_COMMA,
    STATE(2913), 1,
      aux_sym_type_parameters_repeat1,
    STATE(2842), 2,
      sym_line_comment,
      sym_block_comment,
  [84954] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4714), 1,
      anon_sym_GT,
    ACTIONS(6103), 1,
      anon_sym_COMMA,
    STATE(2913), 1,
      aux_sym_type_parameters_repeat1,
    STATE(2843), 2,
      sym_line_comment,
      sym_block_comment,
  [84974] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(6105), 1,
      anon_sym_SEMI,
    STATE(1222), 1,
      sym_declaration_list,
    STATE(2844), 2,
      sym_line_comment,
      sym_block_comment,
  [84994] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(6107), 1,
      anon_sym_SEMI,
    STATE(1228), 1,
      sym_declaration_list,
    STATE(2845), 2,
      sym_line_comment,
      sym_block_comment,
  [85014] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(6109), 1,
      anon_sym_SEMI,
    STATE(689), 1,
      sym_declaration_list,
    STATE(2846), 2,
      sym_line_comment,
      sym_block_comment,
  [85034] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(6111), 1,
      anon_sym_SEMI,
    ACTIONS(6113), 1,
      anon_sym_EQ,
    STATE(2847), 2,
      sym_line_comment,
      sym_block_comment,
  [85054] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4982), 1,
      anon_sym_RBRACE,
    ACTIONS(6115), 1,
      anon_sym_COMMA,
    STATE(3020), 1,
      aux_sym_field_declaration_list_repeat1,
    STATE(2848), 2,
      sym_line_comment,
      sym_block_comment,
  [85074] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4982), 1,
      anon_sym_RBRACE,
    ACTIONS(6115), 1,
      anon_sym_COMMA,
    STATE(2881), 1,
      aux_sym_field_declaration_list_repeat1,
    STATE(2849), 2,
      sym_line_comment,
      sym_block_comment,
  [85094] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(6117), 1,
      anon_sym_SEMI,
    STATE(1239), 1,
      sym_declaration_list,
    STATE(2850), 2,
      sym_line_comment,
      sym_block_comment,
  [85114] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(6119), 1,
      anon_sym_SEMI,
    ACTIONS(6121), 1,
      anon_sym_EQ,
    STATE(2851), 2,
      sym_line_comment,
      sym_block_comment,
  [85134] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2852), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4906), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [85150] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(6123), 1,
      anon_sym_SEMI,
    ACTIONS(6125), 1,
      anon_sym_EQ,
    STATE(2853), 2,
      sym_line_comment,
      sym_block_comment,
  [85170] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(6127), 1,
      anon_sym_SEMI,
    STATE(3497), 1,
      sym_where_clause,
    STATE(2854), 2,
      sym_line_comment,
      sym_block_comment,
  [85190] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2855), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4812), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [85206] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6131), 1,
      anon_sym_COLON,
    ACTIONS(6129), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(2856), 2,
      sym_line_comment,
      sym_block_comment,
  [85224] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6133), 1,
      anon_sym_RBRACE,
    ACTIONS(6135), 1,
      anon_sym_COMMA,
    STATE(2993), 1,
      aux_sym_struct_pattern_repeat1,
    STATE(2857), 2,
      sym_line_comment,
      sym_block_comment,
  [85244] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(6137), 1,
      anon_sym_SEMI,
    STATE(492), 1,
      sym_declaration_list,
    STATE(2858), 2,
      sym_line_comment,
      sym_block_comment,
  [85264] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6141), 1,
      anon_sym_COLON,
    ACTIONS(6139), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(2859), 2,
      sym_line_comment,
      sym_block_comment,
  [85282] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(957), 1,
      anon_sym_RPAREN,
    ACTIONS(6143), 1,
      anon_sym_COMMA,
    STATE(2709), 1,
      aux_sym_arguments_repeat1,
    STATE(2860), 2,
      sym_line_comment,
      sym_block_comment,
  [85302] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(6145), 1,
      anon_sym_SEMI,
    ACTIONS(6147), 1,
      anon_sym_EQ,
    STATE(2861), 2,
      sym_line_comment,
      sym_block_comment,
  [85322] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2862), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6149), 3,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
  [85338] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1607), 1,
      anon_sym_GT,
    ACTIONS(6151), 1,
      anon_sym_COMMA,
    STATE(3037), 1,
      aux_sym_type_arguments_repeat1,
    STATE(2863), 2,
      sym_line_comment,
      sym_block_comment,
  [85358] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2864), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4720), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [85374] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4688), 1,
      anon_sym_GT,
    ACTIONS(6153), 1,
      anon_sym_COMMA,
    STATE(2913), 1,
      aux_sym_type_parameters_repeat1,
    STATE(2865), 2,
      sym_line_comment,
      sym_block_comment,
  [85394] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5895), 1,
      anon_sym_RPAREN,
    ACTIONS(5897), 1,
      anon_sym_COMMA,
    STATE(2868), 1,
      aux_sym_tuple_pattern_repeat1,
    STATE(2866), 2,
      sym_line_comment,
      sym_block_comment,
  [85414] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(6155), 1,
      anon_sym_SEMI,
    STATE(494), 1,
      sym_declaration_list,
    STATE(2867), 2,
      sym_line_comment,
      sym_block_comment,
  [85434] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2993), 1,
      anon_sym_RPAREN,
    ACTIONS(6157), 1,
      anon_sym_COMMA,
    STATE(2964), 1,
      aux_sym_tuple_pattern_repeat1,
    STATE(2868), 2,
      sym_line_comment,
      sym_block_comment,
  [85454] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1507), 1,
      anon_sym_RPAREN,
    ACTIONS(6159), 1,
      anon_sym_COMMA,
    STATE(2840), 1,
      aux_sym_parameters_repeat1,
    STATE(2869), 2,
      sym_line_comment,
      sym_block_comment,
  [85474] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1031), 1,
      anon_sym_RPAREN,
    ACTIONS(6161), 1,
      anon_sym_COMMA,
    STATE(2709), 1,
      aux_sym_arguments_repeat1,
    STATE(2870), 2,
      sym_line_comment,
      sym_block_comment,
  [85494] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4984), 1,
      anon_sym_RBRACE,
    ACTIONS(6163), 1,
      anon_sym_COMMA,
    STATE(3015), 1,
      aux_sym_enum_variant_list_repeat2,
    STATE(2871), 2,
      sym_line_comment,
      sym_block_comment,
  [85514] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2872), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4904), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [85530] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2873), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4862), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [85546] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(6165), 1,
      anon_sym_SEMI,
    STATE(1271), 1,
      sym_declaration_list,
    STATE(2874), 2,
      sym_line_comment,
      sym_block_comment,
  [85566] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(6167), 1,
      anon_sym_SEMI,
    STATE(1273), 1,
      sym_declaration_list,
    STATE(2875), 2,
      sym_line_comment,
      sym_block_comment,
  [85586] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2876), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4850), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [85602] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2877), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4824), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [85618] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(6169), 1,
      anon_sym_SEMI,
    STATE(1279), 1,
      sym_declaration_list,
    STATE(2878), 2,
      sym_line_comment,
      sym_block_comment,
  [85638] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(6171), 1,
      anon_sym_SEMI,
    STATE(1281), 1,
      sym_declaration_list,
    STATE(2879), 2,
      sym_line_comment,
      sym_block_comment,
  [85658] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(6173), 1,
      anon_sym_SEMI,
    ACTIONS(6175), 1,
      anon_sym_EQ,
    STATE(2880), 2,
      sym_line_comment,
      sym_block_comment,
  [85678] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4986), 1,
      anon_sym_RBRACE,
    ACTIONS(6177), 1,
      anon_sym_COMMA,
    STATE(3020), 1,
      aux_sym_field_declaration_list_repeat1,
    STATE(2881), 2,
      sym_line_comment,
      sym_block_comment,
  [85698] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2882), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4734), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [85714] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6179), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4249), 2,
      anon_sym_LBRACE,
      anon_sym_SQUOTE,
    STATE(2883), 2,
      sym_line_comment,
      sym_block_comment,
  [85732] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2884), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4884), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [85748] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2885), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4840), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [85764] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2886), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4902), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [85780] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6179), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6181), 2,
      anon_sym_LBRACE,
      anon_sym_SQUOTE,
    STATE(2887), 2,
      sym_line_comment,
      sym_block_comment,
  [85798] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(6183), 1,
      anon_sym_SEMI,
    STATE(3509), 1,
      sym_where_clause,
    STATE(2888), 2,
      sym_line_comment,
      sym_block_comment,
  [85818] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(6185), 1,
      anon_sym_SEMI,
    STATE(1318), 1,
      sym_declaration_list,
    STATE(2889), 2,
      sym_line_comment,
      sym_block_comment,
  [85838] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(6187), 1,
      anon_sym_SEMI,
    STATE(1320), 1,
      sym_declaration_list,
    STATE(2890), 2,
      sym_line_comment,
      sym_block_comment,
  [85858] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(6189), 1,
      anon_sym_SEMI,
    STATE(3527), 1,
      sym_where_clause,
    STATE(2891), 2,
      sym_line_comment,
      sym_block_comment,
  [85878] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(6191), 1,
      anon_sym_SEMI,
    STATE(1326), 1,
      sym_declaration_list,
    STATE(2892), 2,
      sym_line_comment,
      sym_block_comment,
  [85898] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(6193), 1,
      anon_sym_SEMI,
    STATE(1328), 1,
      sym_declaration_list,
    STATE(2893), 2,
      sym_line_comment,
      sym_block_comment,
  [85918] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(6195), 1,
      anon_sym_SEMI,
    STATE(1334), 1,
      sym_declaration_list,
    STATE(2894), 2,
      sym_line_comment,
      sym_block_comment,
  [85938] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(6197), 1,
      anon_sym_SEMI,
    STATE(1336), 1,
      sym_declaration_list,
    STATE(2895), 2,
      sym_line_comment,
      sym_block_comment,
  [85958] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(6199), 1,
      anon_sym_SEMI,
    ACTIONS(6201), 1,
      anon_sym_EQ,
    STATE(2896), 2,
      sym_line_comment,
      sym_block_comment,
  [85978] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1495), 1,
      anon_sym_RPAREN,
    ACTIONS(6203), 1,
      anon_sym_COMMA,
    STATE(2840), 1,
      aux_sym_parameters_repeat1,
    STATE(2897), 2,
      sym_line_comment,
      sym_block_comment,
  [85998] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(6205), 1,
      anon_sym_SEMI,
    STATE(1352), 1,
      sym_declaration_list,
    STATE(2898), 2,
      sym_line_comment,
      sym_block_comment,
  [86018] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(6207), 1,
      anon_sym_SEMI,
    STATE(1354), 1,
      sym_declaration_list,
    STATE(2899), 2,
      sym_line_comment,
      sym_block_comment,
  [86038] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(6209), 1,
      anon_sym_SEMI,
    ACTIONS(6211), 1,
      anon_sym_EQ,
    STATE(2900), 2,
      sym_line_comment,
      sym_block_comment,
  [86058] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6213), 1,
      anon_sym_RBRACE,
    ACTIONS(6215), 1,
      anon_sym_COMMA,
    STATE(2998), 1,
      aux_sym_struct_pattern_repeat1,
    STATE(2901), 2,
      sym_line_comment,
      sym_block_comment,
  [86078] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6219), 1,
      anon_sym_COLON,
    ACTIONS(6217), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(2902), 2,
      sym_line_comment,
      sym_block_comment,
  [86096] = 6,
    ACTIONS(27), 1,
      anon_sym_PIPE,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6221), 1,
      anon_sym_move,
    STATE(219), 1,
      sym_closure_parameters,
    STATE(2903), 2,
      sym_line_comment,
      sym_block_comment,
  [86116] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(6223), 1,
      anon_sym_SEMI,
    ACTIONS(6225), 1,
      anon_sym_RBRACK,
    STATE(2904), 2,
      sym_line_comment,
      sym_block_comment,
  [86136] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6227), 1,
      anon_sym_RBRACE,
    ACTIONS(6229), 1,
      anon_sym_COMMA,
    STATE(2834), 1,
      aux_sym_field_initializer_list_repeat1,
    STATE(2905), 2,
      sym_line_comment,
      sym_block_comment,
  [86156] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3197), 1,
      anon_sym_RPAREN,
    ACTIONS(6231), 1,
      anon_sym_COMMA,
    STATE(3001), 1,
      aux_sym_tuple_type_repeat1,
    STATE(2906), 2,
      sym_line_comment,
      sym_block_comment,
  [86176] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6233), 1,
      anon_sym_EQ_GT,
    ACTIONS(6235), 1,
      anon_sym_PIPE,
    ACTIONS(6237), 1,
      anon_sym_if,
    STATE(2907), 2,
      sym_line_comment,
      sym_block_comment,
  [86196] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(6239), 1,
      anon_sym_SEMI,
    STATE(583), 1,
      sym_declaration_list,
    STATE(2908), 2,
      sym_line_comment,
      sym_block_comment,
  [86216] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(6241), 2,
      anon_sym_GT,
      anon_sym_COMMA,
    STATE(2909), 2,
      sym_line_comment,
      sym_block_comment,
  [86234] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2910), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5510), 3,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
  [86250] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(6243), 1,
      anon_sym_SEMI,
    STATE(3515), 1,
      sym_where_clause,
    STATE(2911), 2,
      sym_line_comment,
      sym_block_comment,
  [86270] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(6245), 1,
      anon_sym_SEMI,
    ACTIONS(6247), 1,
      anon_sym_RBRACK,
    STATE(2912), 2,
      sym_line_comment,
      sym_block_comment,
  [86290] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5702), 1,
      anon_sym_GT,
    ACTIONS(6249), 1,
      anon_sym_COMMA,
    STATE(2913), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_type_parameters_repeat1,
  [86308] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3305), 1,
      anon_sym_LT2,
    ACTIONS(4972), 1,
      anon_sym_COLON_COLON,
    STATE(1087), 1,
      sym_type_arguments,
    STATE(2914), 2,
      sym_line_comment,
      sym_block_comment,
  [86328] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2915), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4872), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [86344] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(6252), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(2916), 2,
      sym_line_comment,
      sym_block_comment,
  [86362] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(967), 1,
      anon_sym_RPAREN,
    ACTIONS(4159), 1,
      anon_sym_COMMA,
    STATE(2709), 1,
      aux_sym_arguments_repeat1,
    STATE(2917), 2,
      sym_line_comment,
      sym_block_comment,
  [86382] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2918), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1455), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [86398] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2919), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4808), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [86414] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3195), 1,
      anon_sym_RPAREN,
    ACTIONS(6254), 1,
      anon_sym_COMMA,
    STATE(3001), 1,
      aux_sym_tuple_type_repeat1,
    STATE(2920), 2,
      sym_line_comment,
      sym_block_comment,
  [86434] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(6256), 1,
      anon_sym_SEMI,
    STATE(570), 1,
      sym_declaration_list,
    STATE(2921), 2,
      sym_line_comment,
      sym_block_comment,
  [86454] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5575), 1,
      anon_sym_RPAREN,
    ACTIONS(5577), 1,
      anon_sym_COMMA,
    STATE(2927), 1,
      aux_sym_parameters_repeat1,
    STATE(2922), 2,
      sym_line_comment,
      sym_block_comment,
  [86474] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6258), 1,
      anon_sym_RPAREN,
    ACTIONS(6260), 1,
      anon_sym_COMMA,
    STATE(2997), 1,
      aux_sym_ordered_field_declaration_list_repeat1,
    STATE(2923), 2,
      sym_line_comment,
      sym_block_comment,
  [86494] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(6262), 1,
      anon_sym_SEMI,
    STATE(3583), 1,
      sym_where_clause,
    STATE(2924), 2,
      sym_line_comment,
      sym_block_comment,
  [86514] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1627), 1,
      anon_sym_GT,
    ACTIONS(6264), 1,
      anon_sym_COMMA,
    STATE(2931), 1,
      aux_sym_type_arguments_repeat1,
    STATE(2925), 2,
      sym_line_comment,
      sym_block_comment,
  [86534] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1627), 1,
      anon_sym_GT,
    ACTIONS(6264), 1,
      anon_sym_COMMA,
    STATE(3037), 1,
      aux_sym_type_arguments_repeat1,
    STATE(2926), 2,
      sym_line_comment,
      sym_block_comment,
  [86554] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1503), 1,
      anon_sym_RPAREN,
    ACTIONS(5579), 1,
      anon_sym_COMMA,
    STATE(2840), 1,
      aux_sym_parameters_repeat1,
    STATE(2927), 2,
      sym_line_comment,
      sym_block_comment,
  [86574] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1503), 1,
      anon_sym_RPAREN,
    ACTIONS(5579), 1,
      anon_sym_COMMA,
    STATE(2932), 1,
      aux_sym_parameters_repeat1,
    STATE(2928), 2,
      sym_line_comment,
      sym_block_comment,
  [86594] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6266), 1,
      anon_sym_GT,
    ACTIONS(6268), 1,
      anon_sym_COMMA,
    STATE(2929), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_for_lifetimes_repeat1,
  [86612] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(6271), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(2930), 2,
      sym_line_comment,
      sym_block_comment,
  [86630] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1635), 1,
      anon_sym_GT,
    ACTIONS(6273), 1,
      anon_sym_COMMA,
    STATE(3037), 1,
      aux_sym_type_arguments_repeat1,
    STATE(2931), 2,
      sym_line_comment,
      sym_block_comment,
  [86650] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1509), 1,
      anon_sym_RPAREN,
    ACTIONS(6275), 1,
      anon_sym_COMMA,
    STATE(2840), 1,
      aux_sym_parameters_repeat1,
    STATE(2932), 2,
      sym_line_comment,
      sym_block_comment,
  [86670] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5482), 1,
      anon_sym_PIPE,
    ACTIONS(6277), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(2933), 2,
      sym_line_comment,
      sym_block_comment,
  [86688] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(6279), 1,
      anon_sym_SEMI,
    ACTIONS(6281), 1,
      anon_sym_EQ,
    STATE(2934), 2,
      sym_line_comment,
      sym_block_comment,
  [86708] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2935), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4880), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [86724] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2936), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4848), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [86740] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4960), 1,
      anon_sym_for,
    STATE(1955), 1,
      sym_type_arguments,
    STATE(2937), 2,
      sym_line_comment,
      sym_block_comment,
  [86760] = 6,
    ACTIONS(27), 1,
      anon_sym_PIPE,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6283), 1,
      anon_sym_move,
    STATE(223), 1,
      sym_closure_parameters,
    STATE(2938), 2,
      sym_line_comment,
      sym_block_comment,
  [86780] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(4952), 1,
      anon_sym_COLON_COLON,
    STATE(1952), 1,
      sym_type_arguments,
    STATE(2939), 2,
      sym_line_comment,
      sym_block_comment,
  [86800] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(6285), 1,
      anon_sym_SEMI,
    STATE(541), 1,
      sym_declaration_list,
    STATE(2940), 2,
      sym_line_comment,
      sym_block_comment,
  [86820] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4612), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6287), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(2941), 2,
      sym_line_comment,
      sym_block_comment,
  [86838] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2942), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4852), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [86854] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3071), 1,
      anon_sym_RBRACK,
    ACTIONS(6289), 1,
      anon_sym_COMMA,
    STATE(2641), 1,
      aux_sym_slice_pattern_repeat1,
    STATE(2943), 2,
      sym_line_comment,
      sym_block_comment,
  [86874] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6291), 1,
      sym_identifier,
    ACTIONS(6293), 1,
      anon_sym_ref,
    ACTIONS(6295), 1,
      sym_mutable_specifier,
    STATE(2944), 2,
      sym_line_comment,
      sym_block_comment,
  [86894] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(5744), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(2945), 2,
      sym_line_comment,
      sym_block_comment,
  [86912] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4011), 1,
      anon_sym_LT2,
    ACTIONS(4994), 1,
      anon_sym_COLON_COLON,
    STATE(1615), 1,
      sym_type_arguments,
    STATE(2946), 2,
      sym_line_comment,
      sym_block_comment,
  [86932] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4690), 1,
      anon_sym_GT,
    ACTIONS(6297), 1,
      anon_sym_COMMA,
    STATE(2913), 1,
      aux_sym_type_parameters_repeat1,
    STATE(2947), 2,
      sym_line_comment,
      sym_block_comment,
  [86952] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(6299), 1,
      anon_sym_SEMI,
    STATE(543), 1,
      sym_declaration_list,
    STATE(2948), 2,
      sym_line_comment,
      sym_block_comment,
  [86972] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6301), 1,
      anon_sym_RBRACE,
    ACTIONS(6303), 1,
      anon_sym_COMMA,
    STATE(3022), 1,
      aux_sym_field_declaration_list_repeat1,
    STATE(2949), 2,
      sym_line_comment,
      sym_block_comment,
  [86992] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4768), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4888), 1,
      anon_sym_BANG,
    ACTIONS(6305), 1,
      sym_identifier,
    STATE(2950), 2,
      sym_line_comment,
      sym_block_comment,
  [87012] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(6307), 1,
      anon_sym_SEMI,
    ACTIONS(6309), 1,
      anon_sym_EQ,
    STATE(2951), 2,
      sym_line_comment,
      sym_block_comment,
  [87032] = 6,
    ACTIONS(27), 1,
      anon_sym_PIPE,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6311), 1,
      anon_sym_move,
    STATE(233), 1,
      sym_closure_parameters,
    STATE(2952), 2,
      sym_line_comment,
      sym_block_comment,
  [87052] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5317), 1,
      anon_sym_GT,
    ACTIONS(5319), 1,
      anon_sym_COMMA,
    STATE(2826), 1,
      aux_sym_type_parameters_repeat1,
    STATE(2953), 2,
      sym_line_comment,
      sym_block_comment,
  [87072] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2954), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6313), 3,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
  [87088] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(6315), 1,
      anon_sym_SEMI,
    STATE(549), 1,
      sym_declaration_list,
    STATE(2955), 2,
      sym_line_comment,
      sym_block_comment,
  [87108] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4055), 1,
      anon_sym_LBRACE,
    ACTIONS(6317), 1,
      anon_sym_COLON_COLON,
    STATE(1696), 1,
      sym_field_initializer_list,
    STATE(2956), 2,
      sym_line_comment,
      sym_block_comment,
  [87128] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(6319), 1,
      anon_sym_SEMI,
    STATE(551), 1,
      sym_declaration_list,
    STATE(2957), 2,
      sym_line_comment,
      sym_block_comment,
  [87148] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6321), 1,
      anon_sym_LPAREN,
    ACTIONS(6323), 1,
      anon_sym_LBRACK,
    ACTIONS(6325), 1,
      anon_sym_LBRACE,
    STATE(2958), 2,
      sym_line_comment,
      sym_block_comment,
  [87168] = 6,
    ACTIONS(27), 1,
      anon_sym_PIPE,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6327), 1,
      anon_sym_move,
    STATE(234), 1,
      sym_closure_parameters,
    STATE(2959), 2,
      sym_line_comment,
      sym_block_comment,
  [87188] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6329), 1,
      sym_identifier,
    ACTIONS(6331), 1,
      anon_sym_await,
    ACTIONS(6333), 1,
      sym_integer_literal,
    STATE(2960), 2,
      sym_line_comment,
      sym_block_comment,
  [87208] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(999), 1,
      anon_sym_RBRACK,
    ACTIONS(6335), 1,
      anon_sym_COMMA,
    STATE(2709), 1,
      aux_sym_arguments_repeat1,
    STATE(2961), 2,
      sym_line_comment,
      sym_block_comment,
  [87228] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5482), 1,
      anon_sym_PIPE,
    ACTIONS(6337), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(2962), 2,
      sym_line_comment,
      sym_block_comment,
  [87246] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(6339), 1,
      anon_sym_SEMI,
    ACTIONS(6341), 1,
      anon_sym_EQ,
    STATE(2963), 2,
      sym_line_comment,
      sym_block_comment,
  [87266] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6337), 1,
      anon_sym_RPAREN,
    ACTIONS(6343), 1,
      anon_sym_COMMA,
    STATE(2964), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_tuple_pattern_repeat1,
  [87284] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(6346), 1,
      anon_sym_SEMI,
    STATE(642), 1,
      sym_declaration_list,
    STATE(2965), 2,
      sym_line_comment,
      sym_block_comment,
  [87304] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(6348), 1,
      anon_sym_SEMI,
    STATE(3403), 1,
      sym_where_clause,
    STATE(2966), 2,
      sym_line_comment,
      sym_block_comment,
  [87324] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4944), 1,
      anon_sym_LT,
    ACTIONS(6350), 1,
      anon_sym_EQ,
    STATE(3605), 1,
      sym_type_parameters,
    STATE(2967), 2,
      sym_line_comment,
      sym_block_comment,
  [87344] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2968), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6352), 3,
      sym_string_content,
      anon_sym_DQUOTE,
      sym_escape_sequence,
  [87360] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6354), 1,
      anon_sym_RBRACE,
    ACTIONS(6356), 1,
      anon_sym_COMMA,
    STATE(3030), 1,
      aux_sym_enum_variant_list_repeat2,
    STATE(2969), 2,
      sym_line_comment,
      sym_block_comment,
  [87380] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(6358), 1,
      anon_sym_SEMI,
    STATE(591), 1,
      sym_declaration_list,
    STATE(2970), 2,
      sym_line_comment,
      sym_block_comment,
  [87400] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(6360), 1,
      anon_sym_SEMI,
    STATE(3623), 1,
      sym_where_clause,
    STATE(2971), 2,
      sym_line_comment,
      sym_block_comment,
  [87420] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2972), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4860), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [87436] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1625), 1,
      anon_sym_GT,
    ACTIONS(6362), 1,
      anon_sym_COMMA,
    STATE(3033), 1,
      aux_sym_type_arguments_repeat1,
    STATE(2973), 2,
      sym_line_comment,
      sym_block_comment,
  [87456] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3448), 1,
      anon_sym_LBRACE,
    ACTIONS(6364), 1,
      anon_sym_COLON_COLON,
    STATE(1477), 1,
      sym_field_initializer_list,
    STATE(2974), 2,
      sym_line_comment,
      sym_block_comment,
  [87476] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2975), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4820), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [87492] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6366), 1,
      anon_sym_LPAREN,
    ACTIONS(6368), 1,
      anon_sym_LBRACK,
    ACTIONS(6370), 1,
      anon_sym_LBRACE,
    STATE(2976), 2,
      sym_line_comment,
      sym_block_comment,
  [87512] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6372), 1,
      anon_sym_LPAREN,
    ACTIONS(6374), 1,
      anon_sym_LBRACK,
    ACTIONS(6376), 1,
      anon_sym_LBRACE,
    STATE(2977), 2,
      sym_line_comment,
      sym_block_comment,
  [87532] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2978), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4838), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [87548] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6378), 1,
      anon_sym_RPAREN,
    ACTIONS(6380), 1,
      anon_sym_COMMA,
    STATE(2997), 1,
      aux_sym_ordered_field_declaration_list_repeat1,
    STATE(2979), 2,
      sym_line_comment,
      sym_block_comment,
  [87568] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5447), 1,
      anon_sym_COLON,
    ACTIONS(6382), 2,
      anon_sym_PIPE,
      anon_sym_COMMA,
    STATE(2980), 2,
      sym_line_comment,
      sym_block_comment,
  [87586] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6384), 1,
      anon_sym_LPAREN,
    ACTIONS(6386), 1,
      anon_sym_LBRACK,
    ACTIONS(6388), 1,
      anon_sym_LBRACE,
    STATE(2981), 2,
      sym_line_comment,
      sym_block_comment,
  [87606] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(6390), 1,
      anon_sym_LBRACE,
    STATE(1955), 1,
      sym_type_arguments,
    STATE(2982), 2,
      sym_line_comment,
      sym_block_comment,
  [87626] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1625), 1,
      anon_sym_GT,
    ACTIONS(6362), 1,
      anon_sym_COMMA,
    STATE(3037), 1,
      aux_sym_type_arguments_repeat1,
    STATE(2983), 2,
      sym_line_comment,
      sym_block_comment,
  [87646] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2984), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4828), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [87662] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2985), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4914), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [87678] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2986), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4882), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [87694] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3063), 1,
      anon_sym_RPAREN,
    ACTIONS(6392), 1,
      anon_sym_COMMA,
    STATE(2641), 1,
      aux_sym_slice_pattern_repeat1,
    STATE(2987), 2,
      sym_line_comment,
      sym_block_comment,
  [87714] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2988), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4822), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [87730] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6382), 1,
      anon_sym_PIPE,
    ACTIONS(6394), 1,
      anon_sym_COMMA,
    STATE(2989), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_closure_parameters_repeat1,
  [87748] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(6397), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(2990), 2,
      sym_line_comment,
      sym_block_comment,
  [87766] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(6399), 1,
      anon_sym_SEMI,
    STATE(1103), 1,
      sym_declaration_list,
    STATE(2991), 2,
      sym_line_comment,
      sym_block_comment,
  [87786] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6403), 1,
      anon_sym_COLON,
    ACTIONS(6401), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(2992), 2,
      sym_line_comment,
      sym_block_comment,
  [87804] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5155), 1,
      anon_sym_RBRACE,
    ACTIONS(6405), 1,
      anon_sym_COMMA,
    STATE(2769), 1,
      aux_sym_struct_pattern_repeat1,
    STATE(2993), 2,
      sym_line_comment,
      sym_block_comment,
  [87824] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5331), 1,
      anon_sym_PLUS,
    ACTIONS(6407), 1,
      anon_sym_GT,
    ACTIONS(6409), 1,
      anon_sym_as,
    STATE(2994), 2,
      sym_line_comment,
      sym_block_comment,
  [87844] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3049), 1,
      anon_sym_RPAREN,
    ACTIONS(6411), 1,
      anon_sym_COMMA,
    STATE(2641), 1,
      aux_sym_slice_pattern_repeat1,
    STATE(2995), 2,
      sym_line_comment,
      sym_block_comment,
  [87864] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(2996), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6413), 3,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_QMARK,
  [87880] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6415), 1,
      anon_sym_RPAREN,
    ACTIONS(6417), 1,
      anon_sym_COMMA,
    STATE(2997), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_ordered_field_declaration_list_repeat1,
  [87898] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5277), 1,
      anon_sym_RBRACE,
    ACTIONS(6420), 1,
      anon_sym_COMMA,
    STATE(2769), 1,
      aux_sym_struct_pattern_repeat1,
    STATE(2998), 2,
      sym_line_comment,
      sym_block_comment,
  [87918] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5506), 1,
      anon_sym_RPAREN,
    ACTIONS(5508), 1,
      anon_sym_COMMA,
    STATE(3009), 1,
      aux_sym_parameters_repeat1,
    STATE(2999), 2,
      sym_line_comment,
      sym_block_comment,
  [87938] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(6422), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(3000), 2,
      sym_line_comment,
      sym_block_comment,
  [87956] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6422), 1,
      anon_sym_RPAREN,
    ACTIONS(6424), 1,
      anon_sym_COMMA,
    STATE(3001), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_tuple_type_repeat1,
  [87974] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(6427), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(3002), 2,
      sym_line_comment,
      sym_block_comment,
  [87992] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6429), 1,
      anon_sym_RBRACE,
    ACTIONS(6431), 1,
      anon_sym_COMMA,
    STATE(3003), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_use_list_repeat1,
  [88010] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6434), 1,
      anon_sym_RPAREN,
    ACTIONS(6436), 1,
      anon_sym_COMMA,
    STATE(2997), 1,
      aux_sym_ordered_field_declaration_list_repeat1,
    STATE(3004), 2,
      sym_line_comment,
      sym_block_comment,
  [88030] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(3005), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4814), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [88046] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(3006), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4856), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [88062] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5482), 1,
      anon_sym_PIPE,
    ACTIONS(6438), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(3007), 2,
      sym_line_comment,
      sym_block_comment,
  [88080] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(3008), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4818), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [88096] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1501), 1,
      anon_sym_RPAREN,
    ACTIONS(5622), 1,
      anon_sym_COMMA,
    STATE(2840), 1,
      aux_sym_parameters_repeat1,
    STATE(3009), 2,
      sym_line_comment,
      sym_block_comment,
  [88116] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6440), 1,
      sym_identifier,
    ACTIONS(6442), 1,
      anon_sym_await,
    ACTIONS(6444), 1,
      sym_integer_literal,
    STATE(3010), 2,
      sym_line_comment,
      sym_block_comment,
  [88136] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(3011), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4842), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [88152] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1501), 1,
      anon_sym_RPAREN,
    ACTIONS(5622), 1,
      anon_sym_COMMA,
    STATE(2897), 1,
      aux_sym_parameters_repeat1,
    STATE(3012), 2,
      sym_line_comment,
      sym_block_comment,
  [88172] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4632), 1,
      anon_sym_COLON,
    ACTIONS(4952), 1,
      anon_sym_COLON_COLON,
    STATE(2725), 1,
      sym_trait_bounds,
    STATE(3013), 2,
      sym_line_comment,
      sym_block_comment,
  [88192] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5666), 1,
      anon_sym_EQ,
    ACTIONS(5702), 2,
      anon_sym_GT,
      anon_sym_COMMA,
    STATE(3014), 2,
      sym_line_comment,
      sym_block_comment,
  [88210] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6446), 1,
      anon_sym_RBRACE,
    ACTIONS(6448), 1,
      anon_sym_COMMA,
    STATE(3015), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_enum_variant_list_repeat2,
  [88228] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(921), 1,
      anon_sym_RBRACK,
    ACTIONS(4069), 1,
      anon_sym_COMMA,
    STATE(2709), 1,
      aux_sym_arguments_repeat1,
    STATE(3016), 2,
      sym_line_comment,
      sym_block_comment,
  [88248] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3011), 1,
      anon_sym_SQUOTE,
    ACTIONS(6451), 1,
      anon_sym_GT,
    STATE(3094), 1,
      sym_lifetime,
    STATE(3017), 2,
      sym_line_comment,
      sym_block_comment,
  [88268] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6451), 1,
      anon_sym_GT,
    ACTIONS(6453), 1,
      anon_sym_COMMA,
    STATE(2929), 1,
      aux_sym_for_lifetimes_repeat1,
    STATE(3018), 2,
      sym_line_comment,
      sym_block_comment,
  [88288] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5924), 1,
      anon_sym_RPAREN,
    ACTIONS(5926), 1,
      anon_sym_COMMA,
    STATE(2773), 1,
      aux_sym_tuple_pattern_repeat1,
    STATE(3019), 2,
      sym_line_comment,
      sym_block_comment,
  [88308] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6455), 1,
      anon_sym_RBRACE,
    ACTIONS(6457), 1,
      anon_sym_COMMA,
    STATE(3020), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_field_declaration_list_repeat1,
  [88326] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(3021), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4806), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [88342] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5011), 1,
      anon_sym_RBRACE,
    ACTIONS(6460), 1,
      anon_sym_COMMA,
    STATE(3020), 1,
      aux_sym_field_declaration_list_repeat1,
    STATE(3022), 2,
      sym_line_comment,
      sym_block_comment,
  [88362] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6464), 1,
      anon_sym_EQ,
    ACTIONS(6462), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(3023), 2,
      sym_line_comment,
      sym_block_comment,
  [88380] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4916), 1,
      anon_sym_RBRACE,
    ACTIONS(6466), 1,
      anon_sym_COMMA,
    STATE(3020), 1,
      aux_sym_field_declaration_list_repeat1,
    STATE(3024), 2,
      sym_line_comment,
      sym_block_comment,
  [88400] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4922), 1,
      anon_sym_RBRACE,
    ACTIONS(6468), 1,
      anon_sym_COMMA,
    STATE(3015), 1,
      aux_sym_enum_variant_list_repeat2,
    STATE(3025), 2,
      sym_line_comment,
      sym_block_comment,
  [88420] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(6470), 1,
      anon_sym_SEMI,
    ACTIONS(6472), 1,
      anon_sym_RBRACK,
    STATE(3026), 2,
      sym_line_comment,
      sym_block_comment,
  [88440] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5011), 1,
      anon_sym_RBRACE,
    ACTIONS(6460), 1,
      anon_sym_COMMA,
    STATE(3024), 1,
      aux_sym_field_declaration_list_repeat1,
    STATE(3027), 2,
      sym_line_comment,
      sym_block_comment,
  [88460] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(3028), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6474), 3,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
  [88476] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6478), 1,
      anon_sym_EQ,
    ACTIONS(6476), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(3029), 2,
      sym_line_comment,
      sym_block_comment,
  [88494] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5013), 1,
      anon_sym_RBRACE,
    ACTIONS(6480), 1,
      anon_sym_COMMA,
    STATE(3015), 1,
      aux_sym_enum_variant_list_repeat2,
    STATE(3030), 2,
      sym_line_comment,
      sym_block_comment,
  [88514] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5013), 1,
      anon_sym_RBRACE,
    ACTIONS(6480), 1,
      anon_sym_COMMA,
    STATE(3025), 1,
      aux_sym_enum_variant_list_repeat2,
    STATE(3031), 2,
      sym_line_comment,
      sym_block_comment,
  [88534] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(6482), 1,
      anon_sym_COLON_COLON,
    STATE(1952), 1,
      sym_type_arguments,
    STATE(3032), 2,
      sym_line_comment,
      sym_block_comment,
  [88554] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1633), 1,
      anon_sym_GT,
    ACTIONS(6484), 1,
      anon_sym_COMMA,
    STATE(3037), 1,
      aux_sym_type_arguments_repeat1,
    STATE(3033), 2,
      sym_line_comment,
      sym_block_comment,
  [88574] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5451), 1,
      anon_sym_COMMA,
    ACTIONS(6486), 1,
      anon_sym_PIPE,
    STATE(2989), 1,
      aux_sym_closure_parameters_repeat1,
    STATE(3034), 2,
      sym_line_comment,
      sym_block_comment,
  [88594] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(6488), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(3035), 2,
      sym_line_comment,
      sym_block_comment,
  [88612] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(6490), 1,
      anon_sym_SEMI,
    STATE(3391), 1,
      sym_where_clause,
    STATE(3036), 2,
      sym_line_comment,
      sym_block_comment,
  [88632] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5626), 1,
      anon_sym_GT,
    ACTIONS(6492), 1,
      anon_sym_COMMA,
    STATE(3037), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_type_arguments_repeat1,
  [88650] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4595), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4632), 1,
      anon_sym_COLON,
    STATE(2725), 1,
      sym_trait_bounds,
    STATE(3038), 2,
      sym_line_comment,
      sym_block_comment,
  [88670] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4612), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5401), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(3039), 2,
      sym_line_comment,
      sym_block_comment,
  [88688] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5461), 1,
      anon_sym_GT,
    ACTIONS(5463), 1,
      anon_sym_COMMA,
    STATE(2795), 1,
      aux_sym_type_parameters_repeat1,
    STATE(3040), 2,
      sym_line_comment,
      sym_block_comment,
  [88708] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(6495), 1,
      anon_sym_SEMI,
    STATE(744), 1,
      sym_declaration_list,
    STATE(3041), 2,
      sym_line_comment,
      sym_block_comment,
  [88728] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(5021), 1,
      anon_sym_for,
    STATE(1955), 1,
      sym_type_arguments,
    STATE(3042), 2,
      sym_line_comment,
      sym_block_comment,
  [88748] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6499), 1,
      anon_sym_COLON,
    ACTIONS(6497), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(3043), 2,
      sym_line_comment,
      sym_block_comment,
  [88766] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(3044), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6501), 3,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_QMARK,
  [88782] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    ACTIONS(6503), 1,
      anon_sym_SEMI,
    STATE(1128), 1,
      sym_declaration_list,
    STATE(3045), 2,
      sym_line_comment,
      sym_block_comment,
  [88802] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4944), 1,
      anon_sym_LT,
    ACTIONS(6505), 1,
      anon_sym_EQ,
    STATE(3602), 1,
      sym_type_parameters,
    STATE(3046), 2,
      sym_line_comment,
      sym_block_comment,
  [88822] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6507), 1,
      anon_sym_RBRACE,
    ACTIONS(6509), 1,
      anon_sym_COMMA,
    STATE(2812), 1,
      aux_sym_use_list_repeat1,
    STATE(3047), 2,
      sym_line_comment,
      sym_block_comment,
  [88842] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(6511), 1,
      anon_sym_for,
    STATE(1955), 1,
      sym_type_arguments,
    STATE(3048), 2,
      sym_line_comment,
      sym_block_comment,
  [88862] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4946), 1,
      anon_sym_where,
    ACTIONS(6513), 1,
      anon_sym_SEMI,
    STATE(3358), 1,
      sym_where_clause,
    STATE(3049), 2,
      sym_line_comment,
      sym_block_comment,
  [88882] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(6515), 1,
      anon_sym_for,
    STATE(1955), 1,
      sym_type_arguments,
    STATE(3050), 2,
      sym_line_comment,
      sym_block_comment,
  [88902] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5482), 1,
      anon_sym_PIPE,
    ACTIONS(6517), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(3051), 2,
      sym_line_comment,
      sym_block_comment,
  [88920] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4503), 1,
      anon_sym_LT2,
    ACTIONS(6519), 1,
      anon_sym_for,
    STATE(1955), 1,
      sym_type_arguments,
    STATE(3052), 2,
      sym_line_comment,
      sym_block_comment,
  [88940] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6521), 1,
      sym_identifier,
    ACTIONS(6523), 1,
      anon_sym_ref,
    ACTIONS(6525), 1,
      sym_mutable_specifier,
    STATE(3053), 2,
      sym_line_comment,
      sym_block_comment,
  [88960] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    ACTIONS(6527), 1,
      anon_sym_SEMI,
    STATE(568), 1,
      sym_declaration_list,
    STATE(3054), 2,
      sym_line_comment,
      sym_block_comment,
  [88980] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(903), 1,
      anon_sym_RBRACK,
    ACTIONS(4077), 1,
      anon_sym_COMMA,
    STATE(2709), 1,
      aux_sym_arguments_repeat1,
    STATE(3055), 2,
      sym_line_comment,
      sym_block_comment,
  [89000] = 6,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3011), 1,
      anon_sym_SQUOTE,
    ACTIONS(6529), 1,
      anon_sym_GT,
    STATE(3094), 1,
      sym_lifetime,
    STATE(3056), 2,
      sym_line_comment,
      sym_block_comment,
  [89020] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6531), 1,
      sym_identifier,
    ACTIONS(6533), 1,
      sym_mutable_specifier,
    STATE(3057), 2,
      sym_line_comment,
      sym_block_comment,
  [89037] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5972), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(3058), 2,
      sym_line_comment,
      sym_block_comment,
  [89052] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4942), 1,
      anon_sym_LBRACE,
    STATE(719), 1,
      sym_field_declaration_list,
    STATE(3059), 2,
      sym_line_comment,
      sym_block_comment,
  [89069] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    STATE(1344), 1,
      sym_declaration_list,
    STATE(3060), 2,
      sym_line_comment,
      sym_block_comment,
  [89086] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5219), 1,
      anon_sym_RBRACE,
    ACTIONS(6535), 1,
      anon_sym_SEMI,
    STATE(3061), 2,
      sym_line_comment,
      sym_block_comment,
  [89103] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6535), 1,
      anon_sym_SEMI,
    ACTIONS(6537), 1,
      anon_sym_RBRACE,
    STATE(3062), 2,
      sym_line_comment,
      sym_block_comment,
  [89120] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6535), 1,
      anon_sym_SEMI,
    ACTIONS(6539), 1,
      anon_sym_RBRACE,
    STATE(3063), 2,
      sym_line_comment,
      sym_block_comment,
  [89137] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6541), 1,
      anon_sym_LT,
    STATE(1003), 1,
      sym_type_parameters,
    STATE(3064), 2,
      sym_line_comment,
      sym_block_comment,
  [89154] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5363), 1,
      anon_sym_LBRACE,
    STATE(657), 1,
      sym_enum_variant_list,
    STATE(3065), 2,
      sym_line_comment,
      sym_block_comment,
  [89171] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5441), 1,
      sym_super,
    ACTIONS(6543), 1,
      sym_identifier,
    STATE(3066), 2,
      sym_line_comment,
      sym_block_comment,
  [89188] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5195), 2,
      sym_identifier,
      sym_super,
    STATE(3067), 2,
      sym_line_comment,
      sym_block_comment,
  [89203] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6545), 1,
      sym_identifier,
    ACTIONS(6547), 1,
      sym_super,
    STATE(3068), 2,
      sym_line_comment,
      sym_block_comment,
  [89220] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3297), 1,
      anon_sym_LPAREN,
    STATE(1159), 1,
      sym_parameters,
    STATE(3069), 2,
      sym_line_comment,
      sym_block_comment,
  [89237] = 5,
    ACTIONS(27), 1,
      anon_sym_PIPE,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(219), 1,
      sym_closure_parameters,
    STATE(3070), 2,
      sym_line_comment,
      sym_block_comment,
  [89254] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    STATE(625), 1,
      sym_declaration_list,
    STATE(3071), 2,
      sym_line_comment,
      sym_block_comment,
  [89271] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5401), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(3072), 2,
      sym_line_comment,
      sym_block_comment,
  [89286] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6549), 1,
      sym_identifier,
    ACTIONS(6551), 1,
      sym_super,
    STATE(3073), 2,
      sym_line_comment,
      sym_block_comment,
  [89303] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6553), 1,
      sym_identifier,
    ACTIONS(6555), 1,
      sym_super,
    STATE(3074), 2,
      sym_line_comment,
      sym_block_comment,
  [89320] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6557), 2,
      anon_sym_const,
      sym_mutable_specifier,
    STATE(3075), 2,
      sym_line_comment,
      sym_block_comment,
  [89335] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5879), 1,
      sym_super,
    ACTIONS(6559), 1,
      sym_identifier,
    STATE(3076), 2,
      sym_line_comment,
      sym_block_comment,
  [89352] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4942), 1,
      anon_sym_LBRACE,
    STATE(737), 1,
      sym_field_declaration_list,
    STATE(3077), 2,
      sym_line_comment,
      sym_block_comment,
  [89369] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6561), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(3078), 2,
      sym_line_comment,
      sym_block_comment,
  [89384] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5441), 2,
      sym_identifier,
      sym_super,
    STATE(3079), 2,
      sym_line_comment,
      sym_block_comment,
  [89399] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4005), 1,
      anon_sym_LPAREN,
    STATE(1625), 1,
      sym_parameters,
    STATE(3080), 2,
      sym_line_comment,
      sym_block_comment,
  [89416] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6563), 2,
      sym_identifier,
      sym_super,
    STATE(3081), 2,
      sym_line_comment,
      sym_block_comment,
  [89431] = 5,
    ACTIONS(27), 1,
      anon_sym_PIPE,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(230), 1,
      sym_closure_parameters,
    STATE(3082), 2,
      sym_line_comment,
      sym_block_comment,
  [89448] = 5,
    ACTIONS(27), 1,
      anon_sym_PIPE,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(231), 1,
      sym_closure_parameters,
    STATE(3083), 2,
      sym_line_comment,
      sym_block_comment,
  [89465] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5441), 1,
      sym_super,
    ACTIONS(6565), 1,
      sym_identifier,
    STATE(3084), 2,
      sym_line_comment,
      sym_block_comment,
  [89482] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5692), 1,
      sym_identifier,
    ACTIONS(5694), 1,
      sym_super,
    STATE(3085), 2,
      sym_line_comment,
      sym_block_comment,
  [89499] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5642), 2,
      sym_identifier,
      sym_super,
    STATE(3086), 2,
      sym_line_comment,
      sym_block_comment,
  [89514] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6567), 1,
      sym_identifier,
    ACTIONS(6569), 1,
      sym_super,
    STATE(3087), 2,
      sym_line_comment,
      sym_block_comment,
  [89531] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6551), 2,
      sym_identifier,
      sym_super,
    STATE(3088), 2,
      sym_line_comment,
      sym_block_comment,
  [89546] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6571), 1,
      anon_sym_RPAREN,
    ACTIONS(6573), 1,
      anon_sym_COLON_COLON,
    STATE(3089), 2,
      sym_line_comment,
      sym_block_comment,
  [89563] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6543), 1,
      sym_identifier,
    ACTIONS(6575), 1,
      sym_super,
    STATE(3090), 2,
      sym_line_comment,
      sym_block_comment,
  [89580] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6577), 1,
      anon_sym_RPAREN,
    ACTIONS(6579), 1,
      anon_sym_COLON_COLON,
    STATE(3091), 2,
      sym_line_comment,
      sym_block_comment,
  [89597] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5195), 1,
      sym_super,
    ACTIONS(5696), 1,
      sym_identifier,
    STATE(3092), 2,
      sym_line_comment,
      sym_block_comment,
  [89614] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6551), 1,
      sym_super,
    ACTIONS(6581), 1,
      sym_identifier,
    STATE(3093), 2,
      sym_line_comment,
      sym_block_comment,
  [89631] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6266), 2,
      anon_sym_GT,
      anon_sym_COMMA,
    STATE(3094), 2,
      sym_line_comment,
      sym_block_comment,
  [89646] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6577), 1,
      anon_sym_RPAREN,
    ACTIONS(6583), 1,
      anon_sym_COLON_COLON,
    STATE(3095), 2,
      sym_line_comment,
      sym_block_comment,
  [89663] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6009), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(3096), 2,
      sym_line_comment,
      sym_block_comment,
  [89678] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5261), 1,
      anon_sym_RBRACE,
    ACTIONS(6535), 1,
      anon_sym_SEMI,
    STATE(3097), 2,
      sym_line_comment,
      sym_block_comment,
  [89695] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6585), 1,
      anon_sym_SEMI,
    ACTIONS(6587), 1,
      anon_sym_as,
    STATE(3098), 2,
      sym_line_comment,
      sym_block_comment,
  [89712] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6577), 1,
      anon_sym_RPAREN,
    ACTIONS(6589), 1,
      anon_sym_COLON_COLON,
    STATE(3099), 2,
      sym_line_comment,
      sym_block_comment,
  [89729] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5165), 1,
      anon_sym_RBRACK,
    ACTIONS(6535), 1,
      anon_sym_SEMI,
    STATE(3100), 2,
      sym_line_comment,
      sym_block_comment,
  [89746] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6541), 1,
      anon_sym_LT,
    STATE(875), 1,
      sym_type_parameters,
    STATE(3101), 2,
      sym_line_comment,
      sym_block_comment,
  [89763] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4005), 1,
      anon_sym_LPAREN,
    STATE(1603), 1,
      sym_parameters,
    STATE(3102), 2,
      sym_line_comment,
      sym_block_comment,
  [89780] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3011), 1,
      anon_sym_SQUOTE,
    STATE(2829), 1,
      sym_lifetime,
    STATE(3103), 2,
      sym_line_comment,
      sym_block_comment,
  [89797] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6591), 2,
      sym_identifier,
      sym_super,
    STATE(3104), 2,
      sym_line_comment,
      sym_block_comment,
  [89812] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4942), 1,
      anon_sym_LBRACE,
    STATE(631), 1,
      sym_field_declaration_list,
    STATE(3105), 2,
      sym_line_comment,
      sym_block_comment,
  [89829] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    STATE(623), 1,
      sym_declaration_list,
    STATE(3106), 2,
      sym_line_comment,
      sym_block_comment,
  [89846] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4595), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4960), 1,
      anon_sym_for,
    STATE(3107), 2,
      sym_line_comment,
      sym_block_comment,
  [89863] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5285), 1,
      anon_sym_LBRACE,
    STATE(1163), 1,
      sym_enum_variant_list,
    STATE(3108), 2,
      sym_line_comment,
      sym_block_comment,
  [89880] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6565), 1,
      sym_identifier,
    ACTIONS(6575), 1,
      sym_super,
    STATE(3109), 2,
      sym_line_comment,
      sym_block_comment,
  [89897] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4595), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5952), 1,
      anon_sym_for,
    STATE(3110), 2,
      sym_line_comment,
      sym_block_comment,
  [89914] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5441), 1,
      sym_super,
    ACTIONS(6593), 1,
      sym_identifier,
    STATE(3111), 2,
      sym_line_comment,
      sym_block_comment,
  [89931] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4005), 1,
      anon_sym_LPAREN,
    STATE(1622), 1,
      sym_parameters,
    STATE(3112), 2,
      sym_line_comment,
      sym_block_comment,
  [89948] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6082), 1,
      sym_identifier,
    ACTIONS(6086), 1,
      sym_mutable_specifier,
    STATE(3113), 2,
      sym_line_comment,
      sym_block_comment,
  [89965] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5482), 1,
      anon_sym_PIPE,
    ACTIONS(6595), 1,
      anon_sym_in,
    STATE(3114), 2,
      sym_line_comment,
      sym_block_comment,
  [89982] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6597), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(3115), 2,
      sym_line_comment,
      sym_block_comment,
  [89997] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6599), 2,
      anon_sym_GT,
      anon_sym_COMMA,
    STATE(3116), 2,
      sym_line_comment,
      sym_block_comment,
  [90012] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3817), 1,
      anon_sym_COLON,
    ACTIONS(5331), 1,
      anon_sym_PLUS,
    STATE(3117), 2,
      sym_line_comment,
      sym_block_comment,
  [90029] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4942), 1,
      anon_sym_LBRACE,
    STATE(663), 1,
      sym_field_declaration_list,
    STATE(3118), 2,
      sym_line_comment,
      sym_block_comment,
  [90046] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3448), 1,
      anon_sym_LBRACE,
    STATE(1477), 1,
      sym_field_initializer_list,
    STATE(3119), 2,
      sym_line_comment,
      sym_block_comment,
  [90063] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5482), 1,
      anon_sym_PIPE,
    ACTIONS(6601), 1,
      anon_sym_in,
    STATE(3120), 2,
      sym_line_comment,
      sym_block_comment,
  [90080] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4499), 1,
      anon_sym_BANG,
    ACTIONS(6603), 1,
      anon_sym_COLON_COLON,
    STATE(3121), 2,
      sym_line_comment,
      sym_block_comment,
  [90097] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6605), 2,
      sym_identifier,
      sym_super,
    STATE(3122), 2,
      sym_line_comment,
      sym_block_comment,
  [90112] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6607), 2,
      sym_identifier,
      sym_metavariable,
    STATE(3123), 2,
      sym_line_comment,
      sym_block_comment,
  [90127] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6541), 1,
      anon_sym_LT,
    STATE(1053), 1,
      sym_type_parameters,
    STATE(3124), 2,
      sym_line_comment,
      sym_block_comment,
  [90144] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5441), 1,
      sym_super,
    ACTIONS(6609), 1,
      sym_identifier,
    STATE(3125), 2,
      sym_line_comment,
      sym_block_comment,
  [90161] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4595), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6099), 1,
      anon_sym_for,
    STATE(3126), 2,
      sym_line_comment,
      sym_block_comment,
  [90178] = 5,
    ACTIONS(27), 1,
      anon_sym_PIPE,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(223), 1,
      sym_closure_parameters,
    STATE(3127), 2,
      sym_line_comment,
      sym_block_comment,
  [90195] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6605), 1,
      sym_super,
    ACTIONS(6611), 1,
      sym_identifier,
    STATE(3128), 2,
      sym_line_comment,
      sym_block_comment,
  [90212] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3867), 1,
      anon_sym_COLON,
    ACTIONS(5331), 1,
      anon_sym_PLUS,
    STATE(3129), 2,
      sym_line_comment,
      sym_block_comment,
  [90229] = 5,
    ACTIONS(27), 1,
      anon_sym_PIPE,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(243), 1,
      sym_closure_parameters,
    STATE(3130), 2,
      sym_line_comment,
      sym_block_comment,
  [90246] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5363), 1,
      anon_sym_LBRACE,
    STATE(675), 1,
      sym_enum_variant_list,
    STATE(3131), 2,
      sym_line_comment,
      sym_block_comment,
  [90263] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6613), 2,
      anon_sym_const,
      sym_mutable_specifier,
    STATE(3132), 2,
      sym_line_comment,
      sym_block_comment,
  [90278] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5879), 1,
      sym_super,
    ACTIONS(6615), 1,
      sym_identifier,
    STATE(3133), 2,
      sym_line_comment,
      sym_block_comment,
  [90295] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4942), 1,
      anon_sym_LBRACE,
    STATE(666), 1,
      sym_field_declaration_list,
    STATE(3134), 2,
      sym_line_comment,
      sym_block_comment,
  [90312] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4768), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4888), 1,
      anon_sym_BANG,
    STATE(3135), 2,
      sym_line_comment,
      sym_block_comment,
  [90329] = 5,
    ACTIONS(27), 1,
      anon_sym_PIPE,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(227), 1,
      sym_closure_parameters,
    STATE(3136), 2,
      sym_line_comment,
      sym_block_comment,
  [90346] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5854), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(3137), 2,
      sym_line_comment,
      sym_block_comment,
  [90361] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5694), 2,
      sym_identifier,
      sym_super,
    STATE(3138), 2,
      sym_line_comment,
      sym_block_comment,
  [90376] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6575), 1,
      sym_super,
    ACTIONS(6617), 1,
      sym_identifier,
    STATE(3139), 2,
      sym_line_comment,
      sym_block_comment,
  [90393] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5744), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(3140), 2,
      sym_line_comment,
      sym_block_comment,
  [90408] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5195), 1,
      sym_super,
    ACTIONS(5720), 1,
      sym_identifier,
    STATE(3141), 2,
      sym_line_comment,
      sym_block_comment,
  [90425] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6551), 1,
      sym_super,
    ACTIONS(6611), 1,
      sym_identifier,
    STATE(3142), 2,
      sym_line_comment,
      sym_block_comment,
  [90442] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    STATE(1965), 1,
      sym_parameters,
    STATE(3143), 2,
      sym_line_comment,
      sym_block_comment,
  [90459] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6535), 1,
      anon_sym_SEMI,
    ACTIONS(6619), 1,
      anon_sym_RPAREN,
    STATE(3144), 2,
      sym_line_comment,
      sym_block_comment,
  [90476] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5165), 1,
      anon_sym_RPAREN,
    ACTIONS(6535), 1,
      anon_sym_SEMI,
    STATE(3145), 2,
      sym_line_comment,
      sym_block_comment,
  [90493] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6535), 1,
      anon_sym_SEMI,
    ACTIONS(6619), 1,
      anon_sym_RBRACK,
    STATE(3146), 2,
      sym_line_comment,
      sym_block_comment,
  [90510] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6535), 1,
      anon_sym_SEMI,
    ACTIONS(6621), 1,
      anon_sym_RBRACE,
    STATE(3147), 2,
      sym_line_comment,
      sym_block_comment,
  [90527] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5879), 1,
      sym_super,
    ACTIONS(6623), 1,
      sym_identifier,
    STATE(3148), 2,
      sym_line_comment,
      sym_block_comment,
  [90544] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6625), 1,
      sym_identifier,
    ACTIONS(6627), 1,
      sym_mutable_specifier,
    STATE(3149), 2,
      sym_line_comment,
      sym_block_comment,
  [90561] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6535), 1,
      anon_sym_SEMI,
    ACTIONS(6629), 1,
      anon_sym_RPAREN,
    STATE(3150), 2,
      sym_line_comment,
      sym_block_comment,
  [90578] = 5,
    ACTIONS(27), 1,
      anon_sym_PIPE,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(233), 1,
      sym_closure_parameters,
    STATE(3151), 2,
      sym_line_comment,
      sym_block_comment,
  [90595] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    STATE(616), 1,
      sym_declaration_list,
    STATE(3152), 2,
      sym_line_comment,
      sym_block_comment,
  [90612] = 5,
    ACTIONS(27), 1,
      anon_sym_PIPE,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(234), 1,
      sym_closure_parameters,
    STATE(3153), 2,
      sym_line_comment,
      sym_block_comment,
  [90629] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6535), 1,
      anon_sym_SEMI,
    ACTIONS(6629), 1,
      anon_sym_RBRACK,
    STATE(3154), 2,
      sym_line_comment,
      sym_block_comment,
  [90646] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6631), 1,
      sym_identifier,
    ACTIONS(6633), 1,
      sym_super,
    STATE(3155), 2,
      sym_line_comment,
      sym_block_comment,
  [90663] = 5,
    ACTIONS(27), 1,
      anon_sym_PIPE,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(235), 1,
      sym_closure_parameters,
    STATE(3156), 2,
      sym_line_comment,
      sym_block_comment,
  [90680] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    STATE(1234), 1,
      sym_declaration_list,
    STATE(3157), 2,
      sym_line_comment,
      sym_block_comment,
  [90697] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3849), 2,
      anon_sym_LPAREN,
      anon_sym_COLON_COLON,
    STATE(3158), 2,
      sym_line_comment,
      sym_block_comment,
  [90712] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6563), 1,
      sym_super,
    ACTIONS(6635), 1,
      sym_identifier,
    STATE(3159), 2,
      sym_line_comment,
      sym_block_comment,
  [90729] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6637), 2,
      anon_sym_const,
      sym_mutable_specifier,
    STATE(3160), 2,
      sym_line_comment,
      sym_block_comment,
  [90744] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5654), 1,
      sym_super,
    ACTIONS(5762), 1,
      sym_identifier,
    STATE(3161), 2,
      sym_line_comment,
      sym_block_comment,
  [90761] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6605), 1,
      sym_super,
    ACTIONS(6639), 1,
      sym_identifier,
    STATE(3162), 2,
      sym_line_comment,
      sym_block_comment,
  [90778] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4055), 1,
      anon_sym_LBRACE,
    STATE(1696), 1,
      sym_field_initializer_list,
    STATE(3163), 2,
      sym_line_comment,
      sym_block_comment,
  [90795] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6535), 1,
      anon_sym_SEMI,
    ACTIONS(6641), 1,
      anon_sym_RBRACE,
    STATE(3164), 2,
      sym_line_comment,
      sym_block_comment,
  [90812] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4660), 1,
      anon_sym_BANG,
    ACTIONS(4758), 1,
      anon_sym_COLON_COLON,
    STATE(3165), 2,
      sym_line_comment,
      sym_block_comment,
  [90829] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6429), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(3166), 2,
      sym_line_comment,
      sym_block_comment,
  [90844] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4660), 1,
      anon_sym_BANG,
    ACTIONS(4756), 1,
      anon_sym_COLON_COLON,
    STATE(3167), 2,
      sym_line_comment,
      sym_block_comment,
  [90861] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    STATE(1959), 1,
      sym_parameters,
    STATE(3168), 2,
      sym_line_comment,
      sym_block_comment,
  [90878] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    STATE(1240), 1,
      sym_declaration_list,
    STATE(3169), 2,
      sym_line_comment,
      sym_block_comment,
  [90895] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6633), 1,
      sym_super,
    ACTIONS(6643), 1,
      sym_identifier,
    STATE(3170), 2,
      sym_line_comment,
      sym_block_comment,
  [90912] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6645), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(3171), 2,
      sym_line_comment,
      sym_block_comment,
  [90927] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6575), 1,
      sym_super,
    ACTIONS(6647), 1,
      sym_identifier,
    STATE(3172), 2,
      sym_line_comment,
      sym_block_comment,
  [90944] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5654), 1,
      sym_super,
    ACTIONS(5772), 1,
      sym_identifier,
    STATE(3173), 2,
      sym_line_comment,
      sym_block_comment,
  [90961] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6605), 1,
      sym_super,
    ACTIONS(6649), 1,
      sym_identifier,
    STATE(3174), 2,
      sym_line_comment,
      sym_block_comment,
  [90978] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    STATE(1241), 1,
      sym_declaration_list,
    STATE(3175), 2,
      sym_line_comment,
      sym_block_comment,
  [90995] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    STATE(673), 1,
      sym_declaration_list,
    STATE(3176), 2,
      sym_line_comment,
      sym_block_comment,
  [91012] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6337), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(3177), 2,
      sym_line_comment,
      sym_block_comment,
  [91027] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6651), 1,
      anon_sym_SEMI,
    ACTIONS(6653), 1,
      anon_sym_as,
    STATE(3178), 2,
      sym_line_comment,
      sym_block_comment,
  [91044] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6573), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6655), 1,
      anon_sym_RPAREN,
    STATE(3179), 2,
      sym_line_comment,
      sym_block_comment,
  [91061] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6579), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6657), 1,
      anon_sym_RPAREN,
    STATE(3180), 2,
      sym_line_comment,
      sym_block_comment,
  [91078] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5441), 1,
      sym_super,
    ACTIONS(6659), 1,
      sym_identifier,
    STATE(3181), 2,
      sym_line_comment,
      sym_block_comment,
  [91095] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6661), 1,
      sym_identifier,
    ACTIONS(6663), 1,
      sym_super,
    STATE(3182), 2,
      sym_line_comment,
      sym_block_comment,
  [91112] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6575), 1,
      sym_super,
    ACTIONS(6665), 1,
      sym_identifier,
    STATE(3183), 2,
      sym_line_comment,
      sym_block_comment,
  [91129] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6667), 2,
      anon_sym_const,
      sym_mutable_specifier,
    STATE(3184), 2,
      sym_line_comment,
      sym_block_comment,
  [91144] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5642), 1,
      sym_super,
    ACTIONS(5780), 1,
      sym_identifier,
    STATE(3185), 2,
      sym_line_comment,
      sym_block_comment,
  [91161] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6591), 1,
      sym_super,
    ACTIONS(6669), 1,
      sym_identifier,
    STATE(3186), 2,
      sym_line_comment,
      sym_block_comment,
  [91178] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3011), 1,
      anon_sym_SQUOTE,
    STATE(3094), 1,
      sym_lifetime,
    STATE(3187), 2,
      sym_line_comment,
      sym_block_comment,
  [91195] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5285), 1,
      anon_sym_LBRACE,
    STATE(1249), 1,
      sym_enum_variant_list,
    STATE(3188), 2,
      sym_line_comment,
      sym_block_comment,
  [91212] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6671), 2,
      sym_float_literal,
      sym_integer_literal,
    STATE(3189), 2,
      sym_line_comment,
      sym_block_comment,
  [91227] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5879), 1,
      sym_super,
    ACTIONS(6631), 1,
      sym_identifier,
    STATE(3190), 2,
      sym_line_comment,
      sym_block_comment,
  [91244] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6575), 1,
      sym_super,
    ACTIONS(6609), 1,
      sym_identifier,
    STATE(3191), 2,
      sym_line_comment,
      sym_block_comment,
  [91261] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6559), 1,
      sym_identifier,
    ACTIONS(6633), 1,
      sym_super,
    STATE(3192), 2,
      sym_line_comment,
      sym_block_comment,
  [91278] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5195), 1,
      sym_super,
    ACTIONS(5762), 1,
      sym_identifier,
    STATE(3193), 2,
      sym_line_comment,
      sym_block_comment,
  [91295] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6551), 1,
      sym_super,
    ACTIONS(6639), 1,
      sym_identifier,
    STATE(3194), 2,
      sym_line_comment,
      sym_block_comment,
  [91312] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6583), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6657), 1,
      anon_sym_RPAREN,
    STATE(3195), 2,
      sym_line_comment,
      sym_block_comment,
  [91329] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6633), 1,
      sym_super,
    ACTIONS(6673), 1,
      sym_identifier,
    STATE(3196), 2,
      sym_line_comment,
      sym_block_comment,
  [91346] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4976), 1,
      anon_sym_LBRACE,
    STATE(1252), 1,
      sym_field_declaration_list,
    STATE(3197), 2,
      sym_line_comment,
      sym_block_comment,
  [91363] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5654), 1,
      sym_super,
    ACTIONS(5787), 1,
      sym_identifier,
    STATE(3198), 2,
      sym_line_comment,
      sym_block_comment,
  [91380] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6605), 1,
      sym_super,
    ACTIONS(6675), 1,
      sym_identifier,
    STATE(3199), 2,
      sym_line_comment,
      sym_block_comment,
  [91397] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6677), 2,
      anon_sym_const,
      sym_mutable_specifier,
    STATE(3200), 2,
      sym_line_comment,
      sym_block_comment,
  [91412] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6633), 1,
      sym_super,
    ACTIONS(6679), 1,
      sym_identifier,
    STATE(3201), 2,
      sym_line_comment,
      sym_block_comment,
  [91429] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6589), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6657), 1,
      anon_sym_RPAREN,
    STATE(3202), 2,
      sym_line_comment,
      sym_block_comment,
  [91446] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5654), 1,
      sym_super,
    ACTIONS(5789), 1,
      sym_identifier,
    STATE(3203), 2,
      sym_line_comment,
      sym_block_comment,
  [91463] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6605), 1,
      sym_super,
    ACTIONS(6681), 1,
      sym_identifier,
    STATE(3204), 2,
      sym_line_comment,
      sym_block_comment,
  [91480] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4942), 1,
      anon_sym_LBRACE,
    STATE(711), 1,
      sym_field_declaration_list,
    STATE(3205), 2,
      sym_line_comment,
      sym_block_comment,
  [91497] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6633), 1,
      sym_super,
    ACTIONS(6683), 1,
      sym_identifier,
    STATE(3206), 2,
      sym_line_comment,
      sym_block_comment,
  [91514] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5654), 1,
      sym_super,
    ACTIONS(5791), 1,
      sym_identifier,
    STATE(3207), 2,
      sym_line_comment,
      sym_block_comment,
  [91531] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6605), 1,
      sym_super,
    ACTIONS(6685), 1,
      sym_identifier,
    STATE(3208), 2,
      sym_line_comment,
      sym_block_comment,
  [91548] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5879), 1,
      sym_super,
    ACTIONS(6683), 1,
      sym_identifier,
    STATE(3209), 2,
      sym_line_comment,
      sym_block_comment,
  [91565] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3297), 1,
      anon_sym_LPAREN,
    STATE(1092), 1,
      sym_parameters,
    STATE(3210), 2,
      sym_line_comment,
      sym_block_comment,
  [91582] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5195), 1,
      sym_super,
    ACTIONS(5791), 1,
      sym_identifier,
    STATE(3211), 2,
      sym_line_comment,
      sym_block_comment,
  [91599] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6551), 1,
      sym_super,
    ACTIONS(6685), 1,
      sym_identifier,
    STATE(3212), 2,
      sym_line_comment,
      sym_block_comment,
  [91616] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4976), 1,
      anon_sym_LBRACE,
    STATE(1254), 1,
      sym_field_declaration_list,
    STATE(3213), 2,
      sym_line_comment,
      sym_block_comment,
  [91633] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6382), 2,
      anon_sym_PIPE,
      anon_sym_COMMA,
    STATE(3214), 2,
      sym_line_comment,
      sym_block_comment,
  [91648] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5654), 1,
      sym_super,
    ACTIONS(5720), 1,
      sym_identifier,
    STATE(3215), 2,
      sym_line_comment,
      sym_block_comment,
  [91665] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    STATE(560), 1,
      sym_declaration_list,
    STATE(3216), 2,
      sym_line_comment,
      sym_block_comment,
  [91682] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    STATE(1255), 1,
      sym_declaration_list,
    STATE(3217), 2,
      sym_line_comment,
      sym_block_comment,
  [91699] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6687), 2,
      sym_identifier,
      sym_metavariable,
    STATE(3218), 2,
      sym_line_comment,
      sym_block_comment,
  [91714] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5167), 1,
      anon_sym_RBRACK,
    ACTIONS(6535), 1,
      anon_sym_SEMI,
    STATE(3219), 2,
      sym_line_comment,
      sym_block_comment,
  [91731] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    STATE(2248), 1,
      sym_parameters,
    STATE(3220), 2,
      sym_line_comment,
      sym_block_comment,
  [91748] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6689), 1,
      anon_sym_LPAREN,
    ACTIONS(6691), 1,
      anon_sym_COLON_COLON,
    STATE(3221), 2,
      sym_line_comment,
      sym_block_comment,
  [91765] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6693), 2,
      anon_sym_const,
      sym_mutable_specifier,
    STATE(3222), 2,
      sym_line_comment,
      sym_block_comment,
  [91780] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5209), 1,
      anon_sym_RBRACE,
    ACTIONS(6535), 1,
      anon_sym_SEMI,
    STATE(3223), 2,
      sym_line_comment,
      sym_block_comment,
  [91797] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6695), 1,
      anon_sym_BANG,
    ACTIONS(6697), 1,
      anon_sym_COLON_COLON,
    STATE(3224), 2,
      sym_line_comment,
      sym_block_comment,
  [91814] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4976), 1,
      anon_sym_LBRACE,
    STATE(1180), 1,
      sym_field_declaration_list,
    STATE(3225), 2,
      sym_line_comment,
      sym_block_comment,
  [91831] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4249), 1,
      anon_sym_EQ_GT,
    ACTIONS(6699), 1,
      anon_sym_AMP_AMP,
    STATE(3226), 2,
      sym_line_comment,
      sym_block_comment,
  [91848] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    STATE(1181), 1,
      sym_declaration_list,
    STATE(3227), 2,
      sym_line_comment,
      sym_block_comment,
  [91865] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6181), 1,
      anon_sym_EQ_GT,
    ACTIONS(6699), 1,
      anon_sym_AMP_AMP,
    STATE(3228), 2,
      sym_line_comment,
      sym_block_comment,
  [91882] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    STATE(1182), 1,
      sym_declaration_list,
    STATE(3229), 2,
      sym_line_comment,
      sym_block_comment,
  [91899] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6701), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(3230), 2,
      sym_line_comment,
      sym_block_comment,
  [91914] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5363), 1,
      anon_sym_LBRACE,
    STATE(730), 1,
      sym_enum_variant_list,
    STATE(3231), 2,
      sym_line_comment,
      sym_block_comment,
  [91931] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6703), 1,
      anon_sym_SEMI,
    ACTIONS(6705), 1,
      anon_sym_as,
    STATE(3232), 2,
      sym_line_comment,
      sym_block_comment,
  [91948] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6446), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(3233), 2,
      sym_line_comment,
      sym_block_comment,
  [91963] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6541), 1,
      anon_sym_LT,
    STATE(1055), 1,
      sym_type_parameters,
    STATE(3234), 2,
      sym_line_comment,
      sym_block_comment,
  [91980] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    STATE(2215), 1,
      sym_parameters,
    STATE(3235), 2,
      sym_line_comment,
      sym_block_comment,
  [91997] = 5,
    ACTIONS(27), 1,
      anon_sym_PIPE,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    STATE(242), 1,
      sym_closure_parameters,
    STATE(3236), 2,
      sym_line_comment,
      sym_block_comment,
  [92014] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5441), 1,
      sym_super,
    ACTIONS(6707), 1,
      sym_identifier,
    STATE(3237), 2,
      sym_line_comment,
      sym_block_comment,
  [92031] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4976), 1,
      anon_sym_LBRACE,
    STATE(1188), 1,
      sym_field_declaration_list,
    STATE(3238), 2,
      sym_line_comment,
      sym_block_comment,
  [92048] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5482), 1,
      anon_sym_PIPE,
    ACTIONS(6709), 1,
      anon_sym_EQ,
    STATE(3239), 2,
      sym_line_comment,
      sym_block_comment,
  [92065] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    STATE(511), 1,
      sym_declaration_list,
    STATE(3240), 2,
      sym_line_comment,
      sym_block_comment,
  [92082] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5879), 2,
      sym_identifier,
      sym_super,
    STATE(3241), 2,
      sym_line_comment,
      sym_block_comment,
  [92097] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    STATE(1193), 1,
      sym_declaration_list,
    STATE(3242), 2,
      sym_line_comment,
      sym_block_comment,
  [92114] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5227), 1,
      anon_sym_RPAREN,
    ACTIONS(6535), 1,
      anon_sym_SEMI,
    STATE(3243), 2,
      sym_line_comment,
      sym_block_comment,
  [92131] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5227), 1,
      anon_sym_RBRACK,
    ACTIONS(6535), 1,
      anon_sym_SEMI,
    STATE(3244), 2,
      sym_line_comment,
      sym_block_comment,
  [92148] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5229), 1,
      anon_sym_RPAREN,
    ACTIONS(6535), 1,
      anon_sym_SEMI,
    STATE(3245), 2,
      sym_line_comment,
      sym_block_comment,
  [92165] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6455), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(3246), 2,
      sym_line_comment,
      sym_block_comment,
  [92180] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5229), 1,
      anon_sym_RBRACK,
    ACTIONS(6535), 1,
      anon_sym_SEMI,
    STATE(3247), 2,
      sym_line_comment,
      sym_block_comment,
  [92197] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6711), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(3248), 2,
      sym_line_comment,
      sym_block_comment,
  [92212] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5195), 1,
      sym_super,
    ACTIONS(5872), 1,
      sym_identifier,
    STATE(3249), 2,
      sym_line_comment,
      sym_block_comment,
  [92229] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6633), 2,
      sym_identifier,
      sym_super,
    STATE(3250), 2,
      sym_line_comment,
      sym_block_comment,
  [92244] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5654), 2,
      sym_identifier,
      sym_super,
    STATE(3251), 2,
      sym_line_comment,
      sym_block_comment,
  [92259] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3305), 1,
      anon_sym_LT2,
    STATE(1395), 1,
      sym_type_arguments,
    STATE(3252), 2,
      sym_line_comment,
      sym_block_comment,
  [92276] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6551), 1,
      sym_super,
    ACTIONS(6713), 1,
      sym_identifier,
    STATE(3253), 2,
      sym_line_comment,
      sym_block_comment,
  [92293] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6715), 2,
      sym__block_comment_content,
      anon_sym_STAR_SLASH,
    STATE(3254), 2,
      sym_line_comment,
      sym_block_comment,
  [92308] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5363), 1,
      anon_sym_LBRACE,
    STATE(593), 1,
      sym_enum_variant_list,
    STATE(3255), 2,
      sym_line_comment,
      sym_block_comment,
  [92325] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5480), 1,
      anon_sym_COLON,
    ACTIONS(5482), 1,
      anon_sym_PIPE,
    STATE(3256), 2,
      sym_line_comment,
      sym_block_comment,
  [92342] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5285), 1,
      anon_sym_LBRACE,
    STATE(1198), 1,
      sym_enum_variant_list,
    STATE(3257), 2,
      sym_line_comment,
      sym_block_comment,
  [92359] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6717), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(3258), 2,
      sym_line_comment,
      sym_block_comment,
  [92374] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4011), 1,
      anon_sym_LT2,
    STATE(1861), 1,
      sym_type_arguments,
    STATE(3259), 2,
      sym_line_comment,
      sym_block_comment,
  [92391] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3697), 1,
      anon_sym_COLON,
    ACTIONS(5331), 1,
      anon_sym_PLUS,
    STATE(3260), 2,
      sym_line_comment,
      sym_block_comment,
  [92408] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    STATE(2221), 1,
      sym_parameters,
    STATE(3261), 2,
      sym_line_comment,
      sym_block_comment,
  [92425] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3701), 1,
      anon_sym_COLON,
    ACTIONS(5331), 1,
      anon_sym_PLUS,
    STATE(3262), 2,
      sym_line_comment,
      sym_block_comment,
  [92442] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4595), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5962), 1,
      anon_sym_for,
    STATE(3263), 2,
      sym_line_comment,
      sym_block_comment,
  [92459] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6535), 1,
      anon_sym_SEMI,
    ACTIONS(6719), 1,
      anon_sym_RPAREN,
    STATE(3264), 2,
      sym_line_comment,
      sym_block_comment,
  [92476] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6535), 1,
      anon_sym_SEMI,
    ACTIONS(6719), 1,
      anon_sym_RBRACK,
    STATE(3265), 2,
      sym_line_comment,
      sym_block_comment,
  [92493] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6535), 1,
      anon_sym_SEMI,
    ACTIONS(6721), 1,
      anon_sym_RPAREN,
    STATE(3266), 2,
      sym_line_comment,
      sym_block_comment,
  [92510] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6535), 1,
      anon_sym_SEMI,
    ACTIONS(6721), 1,
      anon_sym_RBRACK,
    STATE(3267), 2,
      sym_line_comment,
      sym_block_comment,
  [92527] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    STATE(712), 1,
      sym_declaration_list,
    STATE(3268), 2,
      sym_line_comment,
      sym_block_comment,
  [92544] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5235), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(3269), 2,
      sym_line_comment,
      sym_block_comment,
  [92559] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5702), 2,
      anon_sym_GT,
      anon_sym_COMMA,
    STATE(3270), 2,
      sym_line_comment,
      sym_block_comment,
  [92574] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5039), 1,
      anon_sym_PLUS,
    ACTIONS(6723), 1,
      anon_sym_GT,
    STATE(3271), 2,
      sym_line_comment,
      sym_block_comment,
  [92591] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6615), 1,
      sym_identifier,
    ACTIONS(6633), 1,
      sym_super,
    STATE(3272), 2,
      sym_line_comment,
      sym_block_comment,
  [92608] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    STATE(2197), 1,
      sym_parameters,
    STATE(3273), 2,
      sym_line_comment,
      sym_block_comment,
  [92625] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    STATE(1981), 1,
      sym_parameters,
    STATE(3274), 2,
      sym_line_comment,
      sym_block_comment,
  [92642] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    STATE(2251), 1,
      sym_parameters,
    STATE(3275), 2,
      sym_line_comment,
      sym_block_comment,
  [92659] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    STATE(622), 1,
      sym_declaration_list,
    STATE(3276), 2,
      sym_line_comment,
      sym_block_comment,
  [92676] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4976), 1,
      anon_sym_LBRACE,
    STATE(1201), 1,
      sym_field_declaration_list,
    STATE(3277), 2,
      sym_line_comment,
      sym_block_comment,
  [92693] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    STATE(1296), 1,
      sym_declaration_list,
    STATE(3278), 2,
      sym_line_comment,
      sym_block_comment,
  [92710] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6725), 2,
      anon_sym_const,
      sym_mutable_specifier,
    STATE(3279), 2,
      sym_line_comment,
      sym_block_comment,
  [92725] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6575), 1,
      sym_super,
    ACTIONS(6593), 1,
      sym_identifier,
    STATE(3280), 2,
      sym_line_comment,
      sym_block_comment,
  [92742] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    STATE(713), 1,
      sym_declaration_list,
    STATE(3281), 2,
      sym_line_comment,
      sym_block_comment,
  [92759] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4944), 1,
      anon_sym_LT,
    STATE(867), 1,
      sym_type_parameters,
    STATE(3282), 2,
      sym_line_comment,
      sym_block_comment,
  [92776] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5654), 1,
      sym_super,
    ACTIONS(5696), 1,
      sym_identifier,
    STATE(3283), 2,
      sym_line_comment,
      sym_block_comment,
  [92793] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    STATE(519), 1,
      sym_declaration_list,
    STATE(3284), 2,
      sym_line_comment,
      sym_block_comment,
  [92810] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6581), 1,
      sym_identifier,
    ACTIONS(6605), 1,
      sym_super,
    STATE(3285), 2,
      sym_line_comment,
      sym_block_comment,
  [92827] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6727), 1,
      anon_sym_LBRACK,
    ACTIONS(6729), 1,
      anon_sym_BANG,
    STATE(3286), 2,
      sym_line_comment,
      sym_block_comment,
  [92844] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6731), 1,
      anon_sym_LPAREN,
    ACTIONS(6733), 1,
      anon_sym_COLON_COLON,
    STATE(3287), 2,
      sym_line_comment,
      sym_block_comment,
  [92861] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6735), 1,
      anon_sym_BANG,
    ACTIONS(6737), 1,
      anon_sym_COLON_COLON,
    STATE(3288), 2,
      sym_line_comment,
      sym_block_comment,
  [92878] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    STATE(1304), 1,
      sym_declaration_list,
    STATE(3289), 2,
      sym_line_comment,
      sym_block_comment,
  [92895] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    STATE(1305), 1,
      sym_declaration_list,
    STATE(3290), 2,
      sym_line_comment,
      sym_block_comment,
  [92912] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6541), 1,
      anon_sym_LT,
    STATE(1051), 1,
      sym_type_parameters,
    STATE(3291), 2,
      sym_line_comment,
      sym_block_comment,
  [92929] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4976), 1,
      anon_sym_LBRACE,
    STATE(1204), 1,
      sym_field_declaration_list,
    STATE(3292), 2,
      sym_line_comment,
      sym_block_comment,
  [92946] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5043), 1,
      anon_sym_COLON,
    STATE(2725), 1,
      sym_trait_bounds,
    STATE(3293), 2,
      sym_line_comment,
      sym_block_comment,
  [92963] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    STATE(520), 1,
      sym_declaration_list,
    STATE(3294), 2,
      sym_line_comment,
      sym_block_comment,
  [92980] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6541), 1,
      anon_sym_LT,
    STATE(891), 1,
      sym_type_parameters,
    STATE(3295), 2,
      sym_line_comment,
      sym_block_comment,
  [92997] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6739), 2,
      sym__block_comment_content,
      anon_sym_STAR_SLASH,
    STATE(3296), 2,
      sym_line_comment,
      sym_block_comment,
  [93012] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3297), 1,
      anon_sym_LPAREN,
    STATE(1117), 1,
      sym_parameters,
    STATE(3297), 2,
      sym_line_comment,
      sym_block_comment,
  [93029] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5879), 1,
      sym_super,
    ACTIONS(6741), 1,
      sym_identifier,
    STATE(3298), 2,
      sym_line_comment,
      sym_block_comment,
  [93046] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5195), 1,
      sym_super,
    ACTIONS(5793), 1,
      sym_identifier,
    STATE(3299), 2,
      sym_line_comment,
      sym_block_comment,
  [93063] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5167), 1,
      anon_sym_RPAREN,
    ACTIONS(6535), 1,
      anon_sym_SEMI,
    STATE(3300), 2,
      sym_line_comment,
      sym_block_comment,
  [93080] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6689), 1,
      anon_sym_LPAREN,
    ACTIONS(6743), 1,
      anon_sym_COLON_COLON,
    STATE(3301), 2,
      sym_line_comment,
      sym_block_comment,
  [93097] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6745), 1,
      anon_sym_SEMI,
    ACTIONS(6747), 1,
      anon_sym_as,
    STATE(3302), 2,
      sym_line_comment,
      sym_block_comment,
  [93114] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6663), 2,
      sym_identifier,
      sym_super,
    STATE(3303), 2,
      sym_line_comment,
      sym_block_comment,
  [93129] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6689), 1,
      anon_sym_LPAREN,
    ACTIONS(6749), 1,
      anon_sym_COLON_COLON,
    STATE(3304), 2,
      sym_line_comment,
      sym_block_comment,
  [93146] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6751), 2,
      sym_identifier,
      sym_metavariable,
    STATE(3305), 2,
      sym_line_comment,
      sym_block_comment,
  [93161] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6753), 1,
      anon_sym_LBRACK,
    ACTIONS(6755), 1,
      anon_sym_BANG,
    STATE(3306), 2,
      sym_line_comment,
      sym_block_comment,
  [93178] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6757), 2,
      sym_identifier,
      sym_metavariable,
    STATE(3307), 2,
      sym_line_comment,
      sym_block_comment,
  [93193] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5482), 1,
      anon_sym_PIPE,
    ACTIONS(6759), 1,
      anon_sym_in,
    STATE(3308), 2,
      sym_line_comment,
      sym_block_comment,
  [93210] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5285), 1,
      anon_sym_LBRACE,
    STATE(1121), 1,
      sym_enum_variant_list,
    STATE(3309), 2,
      sym_line_comment,
      sym_block_comment,
  [93227] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5889), 2,
      anon_sym_GT,
      anon_sym_COMMA,
    STATE(3310), 2,
      sym_line_comment,
      sym_block_comment,
  [93242] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3669), 2,
      anon_sym_LPAREN,
      anon_sym_COLON_COLON,
    STATE(3311), 2,
      sym_line_comment,
      sym_block_comment,
  [93257] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6761), 2,
      sym_identifier,
      sym_metavariable,
    STATE(3312), 2,
      sym_line_comment,
      sym_block_comment,
  [93272] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5482), 1,
      anon_sym_PIPE,
    ACTIONS(6763), 1,
      anon_sym_EQ,
    STATE(3313), 2,
      sym_line_comment,
      sym_block_comment,
  [93289] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4942), 1,
      anon_sym_LBRACE,
    STATE(733), 1,
      sym_field_declaration_list,
    STATE(3314), 2,
      sym_line_comment,
      sym_block_comment,
  [93306] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6575), 2,
      sym_identifier,
      sym_super,
    STATE(3315), 2,
      sym_line_comment,
      sym_block_comment,
  [93321] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4595), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5021), 1,
      anon_sym_for,
    STATE(3316), 2,
      sym_line_comment,
      sym_block_comment,
  [93338] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4976), 1,
      anon_sym_LBRACE,
    STATE(1140), 1,
      sym_field_declaration_list,
    STATE(3317), 2,
      sym_line_comment,
      sym_block_comment,
  [93355] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6765), 2,
      sym_identifier,
      sym_metavariable,
    STATE(3318), 2,
      sym_line_comment,
      sym_block_comment,
  [93370] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4595), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6511), 1,
      anon_sym_for,
    STATE(3319), 2,
      sym_line_comment,
      sym_block_comment,
  [93387] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4942), 1,
      anon_sym_LBRACE,
    STATE(611), 1,
      sym_field_declaration_list,
    STATE(3320), 2,
      sym_line_comment,
      sym_block_comment,
  [93404] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4595), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6515), 1,
      anon_sym_for,
    STATE(3321), 2,
      sym_line_comment,
      sym_block_comment,
  [93421] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5253), 1,
      anon_sym_RBRACE,
    ACTIONS(6535), 1,
      anon_sym_SEMI,
    STATE(3322), 2,
      sym_line_comment,
      sym_block_comment,
  [93438] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4497), 1,
      anon_sym_LPAREN,
    STATE(2234), 1,
      sym_parameters,
    STATE(3323), 2,
      sym_line_comment,
      sym_block_comment,
  [93455] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4976), 1,
      anon_sym_LBRACE,
    STATE(1131), 1,
      sym_field_declaration_list,
    STATE(3324), 2,
      sym_line_comment,
      sym_block_comment,
  [93472] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3765), 1,
      anon_sym_COLON,
    ACTIONS(5331), 1,
      anon_sym_PLUS,
    STATE(3325), 2,
      sym_line_comment,
      sym_block_comment,
  [93489] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5482), 1,
      anon_sym_PIPE,
    ACTIONS(6767), 1,
      anon_sym_in,
    STATE(3326), 2,
      sym_line_comment,
      sym_block_comment,
  [93506] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4956), 1,
      anon_sym_LBRACE,
    STATE(1134), 1,
      sym_declaration_list,
    STATE(3327), 2,
      sym_line_comment,
      sym_block_comment,
  [93523] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4595), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6519), 1,
      anon_sym_for,
    STATE(3328), 2,
      sym_line_comment,
      sym_block_comment,
  [93540] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6769), 1,
      anon_sym_LBRACK,
    ACTIONS(6771), 1,
      anon_sym_BANG,
    STATE(3329), 2,
      sym_line_comment,
      sym_block_comment,
  [93557] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5482), 1,
      anon_sym_PIPE,
    ACTIONS(6773), 1,
      anon_sym_in,
    STATE(3330), 2,
      sym_line_comment,
      sym_block_comment,
  [93574] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5482), 1,
      anon_sym_PIPE,
    ACTIONS(6775), 1,
      anon_sym_in,
    STATE(3331), 2,
      sym_line_comment,
      sym_block_comment,
  [93591] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5482), 1,
      anon_sym_PIPE,
    ACTIONS(6777), 1,
      anon_sym_in,
    STATE(3332), 2,
      sym_line_comment,
      sym_block_comment,
  [93608] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5482), 1,
      anon_sym_PIPE,
    ACTIONS(6779), 1,
      anon_sym_in,
    STATE(3333), 2,
      sym_line_comment,
      sym_block_comment,
  [93625] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6781), 1,
      anon_sym_STAR_SLASH,
    ACTIONS(6783), 1,
      sym__block_comment_content,
    STATE(3334), 2,
      sym_line_comment,
      sym_block_comment,
  [93642] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6521), 1,
      sym_identifier,
    ACTIONS(6525), 1,
      sym_mutable_specifier,
    STATE(3335), 2,
      sym_line_comment,
      sym_block_comment,
  [93659] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6785), 2,
      sym_float_literal,
      sym_integer_literal,
    STATE(3336), 2,
      sym_line_comment,
      sym_block_comment,
  [93674] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6689), 1,
      anon_sym_LPAREN,
    ACTIONS(6787), 1,
      anon_sym_COLON_COLON,
    STATE(3337), 2,
      sym_line_comment,
      sym_block_comment,
  [93691] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6789), 1,
      sym_identifier,
    ACTIONS(6791), 1,
      sym_mutable_specifier,
    STATE(3338), 2,
      sym_line_comment,
      sym_block_comment,
  [93708] = 5,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4950), 1,
      anon_sym_LBRACE,
    STATE(724), 1,
      sym_declaration_list,
    STATE(3339), 2,
      sym_line_comment,
      sym_block_comment,
  [93725] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6793), 2,
      anon_sym_const,
      sym_mutable_specifier,
    STATE(3340), 2,
      sym_line_comment,
      sym_block_comment,
  [93740] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6795), 1,
      anon_sym_fn,
    STATE(3341), 2,
      sym_line_comment,
      sym_block_comment,
  [93754] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6797), 1,
      anon_sym_RBRACE,
    STATE(3342), 2,
      sym_line_comment,
      sym_block_comment,
  [93768] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6799), 1,
      sym_identifier,
    STATE(3343), 2,
      sym_line_comment,
      sym_block_comment,
  [93782] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6801), 1,
      sym_identifier,
    STATE(3344), 2,
      sym_line_comment,
      sym_block_comment,
  [93796] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6803), 1,
      sym_identifier,
    STATE(3345), 2,
      sym_line_comment,
      sym_block_comment,
  [93810] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6805), 1,
      sym_identifier,
    STATE(3346), 2,
      sym_line_comment,
      sym_block_comment,
  [93824] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6807), 1,
      anon_sym_SEMI,
    STATE(3347), 2,
      sym_line_comment,
      sym_block_comment,
  [93838] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3103), 1,
      anon_sym_PLUS,
    STATE(3348), 2,
      sym_line_comment,
      sym_block_comment,
  [93852] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6809), 1,
      sym_identifier,
    STATE(3349), 2,
      sym_line_comment,
      sym_block_comment,
  [93866] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6811), 1,
      sym_identifier,
    STATE(3350), 2,
      sym_line_comment,
      sym_block_comment,
  [93880] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4061), 1,
      anon_sym_COLON_COLON,
    STATE(3351), 2,
      sym_line_comment,
      sym_block_comment,
  [93894] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6813), 1,
      anon_sym_EQ_GT,
    STATE(3352), 2,
      sym_line_comment,
      sym_block_comment,
  [93908] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4630), 1,
      anon_sym_COLON_COLON,
    STATE(3353), 2,
      sym_line_comment,
      sym_block_comment,
  [93922] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5862), 1,
      anon_sym_RPAREN,
    STATE(3354), 2,
      sym_line_comment,
      sym_block_comment,
  [93936] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6815), 1,
      anon_sym_COLON,
    STATE(3355), 2,
      sym_line_comment,
      sym_block_comment,
  [93950] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6030), 1,
      anon_sym_RBRACE,
    STATE(3356), 2,
      sym_line_comment,
      sym_block_comment,
  [93964] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6817), 1,
      anon_sym_COLON,
    STATE(3357), 2,
      sym_line_comment,
      sym_block_comment,
  [93978] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6819), 1,
      anon_sym_SEMI,
    STATE(3358), 2,
      sym_line_comment,
      sym_block_comment,
  [93992] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5983), 1,
      anon_sym_RBRACE,
    STATE(3359), 2,
      sym_line_comment,
      sym_block_comment,
  [94006] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6821), 1,
      anon_sym_RBRACK,
    STATE(3360), 2,
      sym_line_comment,
      sym_block_comment,
  [94020] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4612), 1,
      anon_sym_COLON_COLON,
    STATE(3361), 2,
      sym_line_comment,
      sym_block_comment,
  [94034] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6823), 1,
      sym_identifier,
    STATE(3362), 2,
      sym_line_comment,
      sym_block_comment,
  [94048] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6825), 1,
      sym_identifier,
    STATE(3363), 2,
      sym_line_comment,
      sym_block_comment,
  [94062] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6827), 1,
      sym_identifier,
    STATE(3364), 2,
      sym_line_comment,
      sym_block_comment,
  [94076] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6829), 1,
      anon_sym_SEMI,
    STATE(3365), 2,
      sym_line_comment,
      sym_block_comment,
  [94090] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6831), 1,
      sym_identifier,
    STATE(3366), 2,
      sym_line_comment,
      sym_block_comment,
  [94104] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6833), 1,
      anon_sym_fn,
    STATE(3367), 2,
      sym_line_comment,
      sym_block_comment,
  [94118] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6835), 1,
      anon_sym_SEMI,
    STATE(3368), 2,
      sym_line_comment,
      sym_block_comment,
  [94132] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6837), 1,
      anon_sym_COLON_COLON,
    STATE(3369), 2,
      sym_line_comment,
      sym_block_comment,
  [94146] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6839), 1,
      anon_sym_RBRACK,
    STATE(3370), 2,
      sym_line_comment,
      sym_block_comment,
  [94160] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6841), 1,
      anon_sym_SEMI,
    STATE(3371), 2,
      sym_line_comment,
      sym_block_comment,
  [94174] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5838), 1,
      anon_sym_LBRACE,
    STATE(3372), 2,
      sym_line_comment,
      sym_block_comment,
  [94188] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6843), 1,
      anon_sym_COLON,
    STATE(3373), 2,
      sym_line_comment,
      sym_block_comment,
  [94202] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6845), 1,
      anon_sym_RBRACE,
    STATE(3374), 2,
      sym_line_comment,
      sym_block_comment,
  [94216] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6847), 1,
      sym_identifier,
    STATE(3375), 2,
      sym_line_comment,
      sym_block_comment,
  [94230] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6621), 1,
      anon_sym_SEMI,
    STATE(3376), 2,
      sym_line_comment,
      sym_block_comment,
  [94244] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6849), 1,
      sym_identifier,
    STATE(3377), 2,
      sym_line_comment,
      sym_block_comment,
  [94258] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6851), 1,
      sym_identifier,
    STATE(3378), 2,
      sym_line_comment,
      sym_block_comment,
  [94272] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6853), 1,
      anon_sym_COLON_COLON,
    STATE(3379), 2,
      sym_line_comment,
      sym_block_comment,
  [94286] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6855), 1,
      sym_identifier,
    STATE(3380), 2,
      sym_line_comment,
      sym_block_comment,
  [94300] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6857), 1,
      sym_identifier,
    STATE(3381), 2,
      sym_line_comment,
      sym_block_comment,
  [94314] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6859), 1,
      sym_identifier,
    STATE(3382), 2,
      sym_line_comment,
      sym_block_comment,
  [94328] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6861), 1,
      sym_identifier,
    STATE(3383), 2,
      sym_line_comment,
      sym_block_comment,
  [94342] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6863), 1,
      sym_identifier,
    STATE(3384), 2,
      sym_line_comment,
      sym_block_comment,
  [94356] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6005), 1,
      anon_sym_RBRACE,
    STATE(3385), 2,
      sym_line_comment,
      sym_block_comment,
  [94370] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6865), 1,
      anon_sym_fn,
    STATE(3386), 2,
      sym_line_comment,
      sym_block_comment,
  [94384] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6867), 1,
      sym_identifier,
    STATE(3387), 2,
      sym_line_comment,
      sym_block_comment,
  [94398] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(985), 1,
      anon_sym_EQ_GT,
    STATE(3388), 2,
      sym_line_comment,
      sym_block_comment,
  [94412] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6869), 1,
      anon_sym_SEMI,
    STATE(3389), 2,
      sym_line_comment,
      sym_block_comment,
  [94426] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6871), 1,
      anon_sym_COLON,
    STATE(3390), 2,
      sym_line_comment,
      sym_block_comment,
  [94440] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6873), 1,
      anon_sym_SEMI,
    STATE(3391), 2,
      sym_line_comment,
      sym_block_comment,
  [94454] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6301), 1,
      anon_sym_RBRACE,
    STATE(3392), 2,
      sym_line_comment,
      sym_block_comment,
  [94468] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6875), 1,
      anon_sym_RBRACK,
    STATE(3393), 2,
      sym_line_comment,
      sym_block_comment,
  [94482] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3145), 1,
      anon_sym_PLUS,
    STATE(3394), 2,
      sym_line_comment,
      sym_block_comment,
  [94496] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6877), 1,
      sym__line_doc_content,
    STATE(3395), 2,
      sym_line_comment,
      sym_block_comment,
  [94510] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6879), 1,
      anon_sym_COLON,
    STATE(3396), 2,
      sym_line_comment,
      sym_block_comment,
  [94524] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4972), 1,
      anon_sym_COLON_COLON,
    STATE(3397), 2,
      sym_line_comment,
      sym_block_comment,
  [94538] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6881), 1,
      anon_sym_COLON,
    STATE(3398), 2,
      sym_line_comment,
      sym_block_comment,
  [94552] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6883), 1,
      anon_sym_COLON_COLON,
    STATE(3399), 2,
      sym_line_comment,
      sym_block_comment,
  [94566] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6885), 1,
      anon_sym_fn,
    STATE(3400), 2,
      sym_line_comment,
      sym_block_comment,
  [94580] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6887), 1,
      anon_sym_SEMI,
    STATE(3401), 2,
      sym_line_comment,
      sym_block_comment,
  [94594] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5801), 1,
      anon_sym_LBRACE,
    STATE(3402), 2,
      sym_line_comment,
      sym_block_comment,
  [94608] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6889), 1,
      anon_sym_SEMI,
    STATE(3403), 2,
      sym_line_comment,
      sym_block_comment,
  [94622] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6535), 1,
      anon_sym_SEMI,
    STATE(3404), 2,
      sym_line_comment,
      sym_block_comment,
  [94636] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6133), 1,
      anon_sym_RBRACE,
    STATE(3405), 2,
      sym_line_comment,
      sym_block_comment,
  [94650] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6891), 1,
      anon_sym_RBRACE,
    STATE(3406), 2,
      sym_line_comment,
      sym_block_comment,
  [94664] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6893), 1,
      sym_identifier,
    STATE(3407), 2,
      sym_line_comment,
      sym_block_comment,
  [94678] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6895), 1,
      anon_sym_SEMI,
    STATE(3408), 2,
      sym_line_comment,
      sym_block_comment,
  [94692] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6897), 1,
      anon_sym_RPAREN,
    STATE(3409), 2,
      sym_line_comment,
      sym_block_comment,
  [94706] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5891), 1,
      anon_sym_RPAREN,
    STATE(3410), 2,
      sym_line_comment,
      sym_block_comment,
  [94720] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6899), 1,
      sym_identifier,
    STATE(3411), 2,
      sym_line_comment,
      sym_block_comment,
  [94734] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6641), 1,
      anon_sym_SEMI,
    STATE(3412), 2,
      sym_line_comment,
      sym_block_comment,
  [94748] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6901), 1,
      anon_sym_SEMI,
    STATE(3413), 2,
      sym_line_comment,
      sym_block_comment,
  [94762] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5818), 1,
      anon_sym_RBRACK,
    STATE(3414), 2,
      sym_line_comment,
      sym_block_comment,
  [94776] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6903), 1,
      sym_identifier,
    STATE(3415), 2,
      sym_line_comment,
      sym_block_comment,
  [94790] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6905), 1,
      anon_sym_RBRACE,
    STATE(3416), 2,
      sym_line_comment,
      sym_block_comment,
  [94804] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4131), 1,
      anon_sym_RPAREN,
    STATE(3417), 2,
      sym_line_comment,
      sym_block_comment,
  [94818] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6907), 1,
      sym_identifier,
    STATE(3418), 2,
      sym_line_comment,
      sym_block_comment,
  [94832] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6909), 1,
      sym_identifier,
    STATE(3419), 2,
      sym_line_comment,
      sym_block_comment,
  [94846] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6911), 1,
      sym_identifier,
    STATE(3420), 2,
      sym_line_comment,
      sym_block_comment,
  [94860] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6913), 1,
      sym_identifier,
    STATE(3421), 2,
      sym_line_comment,
      sym_block_comment,
  [94874] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6915), 1,
      anon_sym_SEMI,
    STATE(3422), 2,
      sym_line_comment,
      sym_block_comment,
  [94888] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6917), 1,
      anon_sym_COLON_COLON,
    STATE(3423), 2,
      sym_line_comment,
      sym_block_comment,
  [94902] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6919), 1,
      anon_sym_COLON,
    STATE(3424), 2,
      sym_line_comment,
      sym_block_comment,
  [94916] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6921), 1,
      anon_sym_RBRACK,
    STATE(3425), 2,
      sym_line_comment,
      sym_block_comment,
  [94930] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5895), 1,
      anon_sym_RPAREN,
    STATE(3426), 2,
      sym_line_comment,
      sym_block_comment,
  [94944] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4595), 1,
      anon_sym_COLON_COLON,
    STATE(3427), 2,
      sym_line_comment,
      sym_block_comment,
  [94958] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6923), 1,
      sym__raw_string_literal_end,
    STATE(3428), 2,
      sym_line_comment,
      sym_block_comment,
  [94972] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6925), 1,
      sym_identifier,
    STATE(3429), 2,
      sym_line_comment,
      sym_block_comment,
  [94986] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6927), 1,
      sym_identifier,
    STATE(3430), 2,
      sym_line_comment,
      sym_block_comment,
  [95000] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6929), 1,
      anon_sym_SEMI,
    STATE(3431), 2,
      sym_line_comment,
      sym_block_comment,
  [95014] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6931), 1,
      anon_sym_RPAREN,
    STATE(3432), 2,
      sym_line_comment,
      sym_block_comment,
  [95028] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6933), 1,
      sym_identifier,
    STATE(3433), 2,
      sym_line_comment,
      sym_block_comment,
  [95042] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6935), 1,
      sym_identifier,
    STATE(3434), 2,
      sym_line_comment,
      sym_block_comment,
  [95056] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6937), 1,
      sym_identifier,
    STATE(3435), 2,
      sym_line_comment,
      sym_block_comment,
  [95070] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5987), 1,
      anon_sym_RBRACE,
    STATE(3436), 2,
      sym_line_comment,
      sym_block_comment,
  [95084] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6939), 1,
      anon_sym_SEMI,
    STATE(3437), 2,
      sym_line_comment,
      sym_block_comment,
  [95098] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6941), 1,
      anon_sym_SEMI,
    STATE(3438), 2,
      sym_line_comment,
      sym_block_comment,
  [95112] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5575), 1,
      anon_sym_RPAREN,
    STATE(3439), 2,
      sym_line_comment,
      sym_block_comment,
  [95126] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6943), 1,
      anon_sym_SEMI,
    STATE(3440), 2,
      sym_line_comment,
      sym_block_comment,
  [95140] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6945), 1,
      anon_sym_SEMI,
    STATE(3441), 2,
      sym_line_comment,
      sym_block_comment,
  [95154] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6947), 1,
      anon_sym_SEMI,
    STATE(3442), 2,
      sym_line_comment,
      sym_block_comment,
  [95168] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4952), 1,
      anon_sym_COLON_COLON,
    STATE(3443), 2,
      sym_line_comment,
      sym_block_comment,
  [95182] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6949), 1,
      anon_sym_EQ_GT,
    STATE(3444), 2,
      sym_line_comment,
      sym_block_comment,
  [95196] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6951), 1,
      anon_sym_SEMI,
    STATE(3445), 2,
      sym_line_comment,
      sym_block_comment,
  [95210] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5814), 1,
      anon_sym_RPAREN,
    STATE(3446), 2,
      sym_line_comment,
      sym_block_comment,
  [95224] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6953), 1,
      anon_sym_SEMI,
    STATE(3447), 2,
      sym_line_comment,
      sym_block_comment,
  [95238] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6955), 1,
      anon_sym_RBRACK,
    STATE(3448), 2,
      sym_line_comment,
      sym_block_comment,
  [95252] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6957), 1,
      anon_sym_RBRACK,
    STATE(3449), 2,
      sym_line_comment,
      sym_block_comment,
  [95266] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6959), 1,
      sym_identifier,
    STATE(3450), 2,
      sym_line_comment,
      sym_block_comment,
  [95280] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6961), 1,
      anon_sym_SEMI,
    STATE(3451), 2,
      sym_line_comment,
      sym_block_comment,
  [95294] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6963), 1,
      anon_sym_SEMI,
    STATE(3452), 2,
      sym_line_comment,
      sym_block_comment,
  [95308] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6965), 1,
      anon_sym_RBRACK,
    STATE(3453), 2,
      sym_line_comment,
      sym_block_comment,
  [95322] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6967), 1,
      sym_identifier,
    STATE(3454), 2,
      sym_line_comment,
      sym_block_comment,
  [95336] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5506), 1,
      anon_sym_RPAREN,
    STATE(3455), 2,
      sym_line_comment,
      sym_block_comment,
  [95350] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6969), 1,
      sym__raw_string_literal_end,
    STATE(3456), 2,
      sym_line_comment,
      sym_block_comment,
  [95364] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6971), 1,
      ts_builtin_sym_end,
    STATE(3457), 2,
      sym_line_comment,
      sym_block_comment,
  [95378] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6973), 1,
      aux_sym_line_comment_token2,
    STATE(3458), 2,
      sym_line_comment,
      sym_block_comment,
  [95392] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6975), 1,
      sym_self,
    STATE(3459), 2,
      sym_line_comment,
      sym_block_comment,
  [95406] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6977), 1,
      anon_sym_SEMI,
    STATE(3460), 2,
      sym_line_comment,
      sym_block_comment,
  [95420] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6979), 1,
      anon_sym_SEMI,
    STATE(3461), 2,
      sym_line_comment,
      sym_block_comment,
  [95434] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6981), 1,
      anon_sym_COLON_COLON,
    STATE(3462), 2,
      sym_line_comment,
      sym_block_comment,
  [95448] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6769), 1,
      anon_sym_LBRACK,
    STATE(3463), 2,
      sym_line_comment,
      sym_block_comment,
  [95462] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5253), 1,
      anon_sym_SEMI,
    STATE(3464), 2,
      sym_line_comment,
      sym_block_comment,
  [95476] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6983), 1,
      anon_sym_SEMI,
    STATE(3465), 2,
      sym_line_comment,
      sym_block_comment,
  [95490] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6985), 1,
      anon_sym_RBRACE,
    STATE(3466), 2,
      sym_line_comment,
      sym_block_comment,
  [95504] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(921), 1,
      anon_sym_RBRACK,
    STATE(3467), 2,
      sym_line_comment,
      sym_block_comment,
  [95518] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6407), 1,
      anon_sym_GT,
    STATE(3468), 2,
      sym_line_comment,
      sym_block_comment,
  [95532] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6987), 1,
      sym__raw_string_literal_end,
    STATE(3469), 2,
      sym_line_comment,
      sym_block_comment,
  [95546] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6989), 1,
      anon_sym_LPAREN,
    STATE(3470), 2,
      sym_line_comment,
      sym_block_comment,
  [95560] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5924), 1,
      anon_sym_RPAREN,
    STATE(3471), 2,
      sym_line_comment,
      sym_block_comment,
  [95574] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6991), 1,
      sym_identifier,
    STATE(3472), 2,
      sym_line_comment,
      sym_block_comment,
  [95588] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6993), 1,
      sym_identifier,
    STATE(3473), 2,
      sym_line_comment,
      sym_block_comment,
  [95602] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6995), 1,
      sym__raw_string_literal_end,
    STATE(3474), 2,
      sym_line_comment,
      sym_block_comment,
  [95616] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6997), 1,
      anon_sym_COLON_COLON,
    STATE(3475), 2,
      sym_line_comment,
      sym_block_comment,
  [95630] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6999), 1,
      anon_sym_COLON,
    STATE(3476), 2,
      sym_line_comment,
      sym_block_comment,
  [95644] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5928), 1,
      anon_sym_RBRACK,
    STATE(3477), 2,
      sym_line_comment,
      sym_block_comment,
  [95658] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7001), 1,
      sym__line_doc_content,
    STATE(3478), 2,
      sym_line_comment,
      sym_block_comment,
  [95672] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7003), 1,
      anon_sym_STAR_SLASH,
    STATE(3479), 2,
      sym_line_comment,
      sym_block_comment,
  [95686] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7005), 1,
      anon_sym_COLON,
    STATE(3480), 2,
      sym_line_comment,
      sym_block_comment,
  [95700] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7007), 1,
      sym_identifier,
    STATE(3481), 2,
      sym_line_comment,
      sym_block_comment,
  [95714] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7009), 1,
      anon_sym_COLON,
    STATE(3482), 2,
      sym_line_comment,
      sym_block_comment,
  [95728] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6354), 1,
      anon_sym_RBRACE,
    STATE(3483), 2,
      sym_line_comment,
      sym_block_comment,
  [95742] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(979), 1,
      anon_sym_EQ_GT,
    STATE(3484), 2,
      sym_line_comment,
      sym_block_comment,
  [95756] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5219), 1,
      anon_sym_SEMI,
    STATE(3485), 2,
      sym_line_comment,
      sym_block_comment,
  [95770] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(750), 1,
      anon_sym_RBRACK,
    STATE(3486), 2,
      sym_line_comment,
      sym_block_comment,
  [95784] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7011), 1,
      anon_sym_EQ_GT,
    STATE(3487), 2,
      sym_line_comment,
      sym_block_comment,
  [95798] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7013), 1,
      anon_sym_SEMI,
    STATE(3488), 2,
      sym_line_comment,
      sym_block_comment,
  [95812] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7015), 1,
      sym_identifier,
    STATE(3489), 2,
      sym_line_comment,
      sym_block_comment,
  [95826] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7017), 1,
      sym_raw_string_literal_content,
    STATE(3490), 2,
      sym_line_comment,
      sym_block_comment,
  [95840] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7019), 1,
      anon_sym_LPAREN,
    STATE(3491), 2,
      sym_line_comment,
      sym_block_comment,
  [95854] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7021), 1,
      anon_sym_SEMI,
    STATE(3492), 2,
      sym_line_comment,
      sym_block_comment,
  [95868] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7023), 1,
      anon_sym_SEMI,
    STATE(3493), 2,
      sym_line_comment,
      sym_block_comment,
  [95882] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4195), 1,
      anon_sym_COLON_COLON,
    STATE(3494), 2,
      sym_line_comment,
      sym_block_comment,
  [95896] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5764), 1,
      anon_sym_COLON_COLON,
    STATE(3495), 2,
      sym_line_comment,
      sym_block_comment,
  [95910] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4037), 1,
      sym_identifier,
    STATE(3496), 2,
      sym_line_comment,
      sym_block_comment,
  [95924] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7025), 1,
      anon_sym_SEMI,
    STATE(3497), 2,
      sym_line_comment,
      sym_block_comment,
  [95938] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7027), 1,
      anon_sym_SEMI,
    STATE(3498), 2,
      sym_line_comment,
      sym_block_comment,
  [95952] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7029), 1,
      anon_sym_COLON_COLON,
    STATE(3499), 2,
      sym_line_comment,
      sym_block_comment,
  [95966] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5261), 1,
      anon_sym_SEMI,
    STATE(3500), 2,
      sym_line_comment,
      sym_block_comment,
  [95980] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5948), 1,
      anon_sym_RBRACE,
    STATE(3501), 2,
      sym_line_comment,
      sym_block_comment,
  [95994] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7031), 1,
      anon_sym_SEMI,
    STATE(3502), 2,
      sym_line_comment,
      sym_block_comment,
  [96008] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7033), 1,
      anon_sym_RBRACK,
    STATE(3503), 2,
      sym_line_comment,
      sym_block_comment,
  [96022] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7035), 1,
      sym_identifier,
    STATE(3504), 2,
      sym_line_comment,
      sym_block_comment,
  [96036] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7037), 1,
      anon_sym_COLON_COLON,
    STATE(3505), 2,
      sym_line_comment,
      sym_block_comment,
  [96050] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7039), 1,
      anon_sym_RBRACE,
    STATE(3506), 2,
      sym_line_comment,
      sym_block_comment,
  [96064] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5209), 1,
      anon_sym_SEMI,
    STATE(3507), 2,
      sym_line_comment,
      sym_block_comment,
  [96078] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7041), 1,
      anon_sym_COLON_COLON,
    STATE(3508), 2,
      sym_line_comment,
      sym_block_comment,
  [96092] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7043), 1,
      anon_sym_SEMI,
    STATE(3509), 2,
      sym_line_comment,
      sym_block_comment,
  [96106] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7045), 1,
      anon_sym_SEMI,
    STATE(3510), 2,
      sym_line_comment,
      sym_block_comment,
  [96120] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4994), 1,
      anon_sym_COLON_COLON,
    STATE(3511), 2,
      sym_line_comment,
      sym_block_comment,
  [96134] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7047), 1,
      anon_sym_SEMI,
    STATE(3512), 2,
      sym_line_comment,
      sym_block_comment,
  [96148] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7049), 1,
      anon_sym_COLON_COLON,
    STATE(3513), 2,
      sym_line_comment,
      sym_block_comment,
  [96162] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7051), 1,
      anon_sym_fn,
    STATE(3514), 2,
      sym_line_comment,
      sym_block_comment,
  [96176] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7053), 1,
      anon_sym_SEMI,
    STATE(3515), 2,
      sym_line_comment,
      sym_block_comment,
  [96190] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7055), 1,
      anon_sym_LBRACE,
    STATE(3516), 2,
      sym_line_comment,
      sym_block_comment,
  [96204] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6583), 1,
      anon_sym_COLON_COLON,
    STATE(3517), 2,
      sym_line_comment,
      sym_block_comment,
  [96218] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7057), 1,
      sym__line_doc_content,
    STATE(3518), 2,
      sym_line_comment,
      sym_block_comment,
  [96232] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6482), 1,
      anon_sym_COLON_COLON,
    STATE(3519), 2,
      sym_line_comment,
      sym_block_comment,
  [96246] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7059), 1,
      anon_sym_fn,
    STATE(3520), 2,
      sym_line_comment,
      sym_block_comment,
  [96260] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7061), 1,
      anon_sym_SEMI,
    STATE(3521), 2,
      sym_line_comment,
      sym_block_comment,
  [96274] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7063), 1,
      anon_sym_SEMI,
    STATE(3522), 2,
      sym_line_comment,
      sym_block_comment,
  [96288] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7065), 1,
      sym_identifier,
    STATE(3523), 2,
      sym_line_comment,
      sym_block_comment,
  [96302] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7067), 1,
      anon_sym_SEMI,
    STATE(3524), 2,
      sym_line_comment,
      sym_block_comment,
  [96316] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7069), 1,
      sym_raw_string_literal_content,
    STATE(3525), 2,
      sym_line_comment,
      sym_block_comment,
  [96330] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7071), 1,
      sym__raw_string_literal_end,
    STATE(3526), 2,
      sym_line_comment,
      sym_block_comment,
  [96344] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7073), 1,
      anon_sym_SEMI,
    STATE(3527), 2,
      sym_line_comment,
      sym_block_comment,
  [96358] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4145), 1,
      anon_sym_RPAREN,
    STATE(3528), 2,
      sym_line_comment,
      sym_block_comment,
  [96372] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7075), 1,
      anon_sym_COLON_COLON,
    STATE(3529), 2,
      sym_line_comment,
      sym_block_comment,
  [96386] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7077), 1,
      anon_sym_RBRACE,
    STATE(3530), 2,
      sym_line_comment,
      sym_block_comment,
  [96400] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7079), 1,
      anon_sym_SEMI,
    STATE(3531), 2,
      sym_line_comment,
      sym_block_comment,
  [96414] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7081), 1,
      anon_sym_COLON_COLON,
    STATE(3532), 2,
      sym_line_comment,
      sym_block_comment,
  [96428] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7083), 1,
      anon_sym_SEMI,
    STATE(3533), 2,
      sym_line_comment,
      sym_block_comment,
  [96442] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7085), 1,
      anon_sym_RBRACE,
    STATE(3534), 2,
      sym_line_comment,
      sym_block_comment,
  [96456] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7087), 1,
      anon_sym_COLON_COLON,
    STATE(3535), 2,
      sym_line_comment,
      sym_block_comment,
  [96470] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7089), 1,
      anon_sym_EQ_GT,
    STATE(3536), 2,
      sym_line_comment,
      sym_block_comment,
  [96484] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3963), 1,
      anon_sym_COLON_COLON,
    STATE(3537), 2,
      sym_line_comment,
      sym_block_comment,
  [96498] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7091), 1,
      anon_sym_COLON_COLON,
    STATE(3538), 2,
      sym_line_comment,
      sym_block_comment,
  [96512] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7093), 1,
      anon_sym_LBRACE,
    STATE(3539), 2,
      sym_line_comment,
      sym_block_comment,
  [96526] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7095), 1,
      anon_sym_STAR_SLASH,
    STATE(3540), 2,
      sym_line_comment,
      sym_block_comment,
  [96540] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7097), 1,
      anon_sym_RBRACK,
    STATE(3541), 2,
      sym_line_comment,
      sym_block_comment,
  [96554] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7099), 1,
      anon_sym_SEMI,
    STATE(3542), 2,
      sym_line_comment,
      sym_block_comment,
  [96568] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7101), 1,
      sym_raw_string_literal_content,
    STATE(3543), 2,
      sym_line_comment,
      sym_block_comment,
  [96582] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3527), 1,
      anon_sym_COLON_COLON,
    STATE(3544), 2,
      sym_line_comment,
      sym_block_comment,
  [96596] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4768), 1,
      anon_sym_COLON_COLON,
    STATE(3545), 2,
      sym_line_comment,
      sym_block_comment,
  [96610] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7103), 1,
      anon_sym_COLON_COLON,
    STATE(3546), 2,
      sym_line_comment,
      sym_block_comment,
  [96624] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7105), 1,
      sym_identifier,
    STATE(3547), 2,
      sym_line_comment,
      sym_block_comment,
  [96638] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7107), 1,
      anon_sym_LBRACE,
    STATE(3548), 2,
      sym_line_comment,
      sym_block_comment,
  [96652] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7109), 1,
      sym_raw_string_literal_content,
    STATE(3549), 2,
      sym_line_comment,
      sym_block_comment,
  [96666] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5225), 1,
      anon_sym_COLON_COLON,
    STATE(3550), 2,
      sym_line_comment,
      sym_block_comment,
  [96680] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7111), 1,
      anon_sym_COLON_COLON,
    STATE(3551), 2,
      sym_line_comment,
      sym_block_comment,
  [96694] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5881), 1,
      anon_sym_LBRACE,
    STATE(3552), 2,
      sym_line_comment,
      sym_block_comment,
  [96708] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5809), 1,
      anon_sym_COLON_COLON,
    STATE(3553), 2,
      sym_line_comment,
      sym_block_comment,
  [96722] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7113), 1,
      anon_sym_COLON_COLON,
    STATE(3554), 2,
      sym_line_comment,
      sym_block_comment,
  [96736] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5907), 1,
      anon_sym_LBRACE,
    STATE(3555), 2,
      sym_line_comment,
      sym_block_comment,
  [96750] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7115), 1,
      anon_sym_COLON_COLON,
    STATE(3556), 2,
      sym_line_comment,
      sym_block_comment,
  [96764] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5231), 1,
      anon_sym_COLON_COLON,
    STATE(3557), 2,
      sym_line_comment,
      sym_block_comment,
  [96778] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5259), 1,
      anon_sym_COLON_COLON,
    STATE(3558), 2,
      sym_line_comment,
      sym_block_comment,
  [96792] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4191), 1,
      anon_sym_COLON_COLON,
    STATE(3559), 2,
      sym_line_comment,
      sym_block_comment,
  [96806] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7117), 1,
      anon_sym_COLON_COLON,
    STATE(3560), 2,
      sym_line_comment,
      sym_block_comment,
  [96820] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7119), 1,
      anon_sym_RPAREN,
    STATE(3561), 2,
      sym_line_comment,
      sym_block_comment,
  [96834] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7121), 1,
      sym__line_doc_content,
    STATE(3562), 2,
      sym_line_comment,
      sym_block_comment,
  [96848] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7123), 1,
      sym_raw_string_literal_content,
    STATE(3563), 2,
      sym_line_comment,
      sym_block_comment,
  [96862] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7125), 1,
      anon_sym_fn,
    STATE(3564), 2,
      sym_line_comment,
      sym_block_comment,
  [96876] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7127), 1,
      anon_sym_COLON,
    STATE(3565), 2,
      sym_line_comment,
      sym_block_comment,
  [96890] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7129), 1,
      anon_sym_LBRACK,
    STATE(3566), 2,
      sym_line_comment,
      sym_block_comment,
  [96904] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7131), 1,
      anon_sym_SEMI,
    STATE(3567), 2,
      sym_line_comment,
      sym_block_comment,
  [96918] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7133), 1,
      anon_sym_LBRACK,
    STATE(3568), 2,
      sym_line_comment,
      sym_block_comment,
  [96932] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7135), 1,
      anon_sym_COLON,
    STATE(3569), 2,
      sym_line_comment,
      sym_block_comment,
  [96946] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7137), 1,
      anon_sym_COLON,
    STATE(3570), 2,
      sym_line_comment,
      sym_block_comment,
  [96960] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7139), 1,
      anon_sym_SEMI,
    STATE(3571), 2,
      sym_line_comment,
      sym_block_comment,
  [96974] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5478), 1,
      anon_sym_RPAREN,
    STATE(3572), 2,
      sym_line_comment,
      sym_block_comment,
  [96988] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7141), 1,
      sym_identifier,
    STATE(3573), 2,
      sym_line_comment,
      sym_block_comment,
  [97002] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7143), 1,
      anon_sym_LT,
    STATE(3574), 2,
      sym_line_comment,
      sym_block_comment,
  [97016] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7145), 1,
      anon_sym_COLON,
    STATE(3575), 2,
      sym_line_comment,
      sym_block_comment,
  [97030] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7147), 1,
      anon_sym_SEMI,
    STATE(3576), 2,
      sym_line_comment,
      sym_block_comment,
  [97044] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7149), 1,
      anon_sym_COLON,
    STATE(3577), 2,
      sym_line_comment,
      sym_block_comment,
  [97058] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7151), 1,
      sym_identifier,
    STATE(3578), 2,
      sym_line_comment,
      sym_block_comment,
  [97072] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7153), 1,
      anon_sym_SEMI,
    STATE(3579), 2,
      sym_line_comment,
      sym_block_comment,
  [97086] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6537), 1,
      anon_sym_SEMI,
    STATE(3580), 2,
      sym_line_comment,
      sym_block_comment,
  [97100] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7155), 1,
      anon_sym_RBRACK,
    STATE(3581), 2,
      sym_line_comment,
      sym_block_comment,
  [97114] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7157), 1,
      sym_identifier,
    STATE(3582), 2,
      sym_line_comment,
      sym_block_comment,
  [97128] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7159), 1,
      anon_sym_SEMI,
    STATE(3583), 2,
      sym_line_comment,
      sym_block_comment,
  [97142] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6227), 1,
      anon_sym_RBRACE,
    STATE(3584), 2,
      sym_line_comment,
      sym_block_comment,
  [97156] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7161), 1,
      anon_sym_COLON,
    STATE(3585), 2,
      sym_line_comment,
      sym_block_comment,
  [97170] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7163), 1,
      sym_identifier,
    STATE(3586), 2,
      sym_line_comment,
      sym_block_comment,
  [97184] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6507), 1,
      anon_sym_RBRACE,
    STATE(3587), 2,
      sym_line_comment,
      sym_block_comment,
  [97198] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7165), 1,
      anon_sym_RPAREN,
    STATE(3588), 2,
      sym_line_comment,
      sym_block_comment,
  [97212] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7167), 1,
      anon_sym_COLON,
    STATE(3589), 2,
      sym_line_comment,
      sym_block_comment,
  [97226] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7169), 1,
      anon_sym_SEMI,
    STATE(3590), 2,
      sym_line_comment,
      sym_block_comment,
  [97240] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7171), 1,
      anon_sym_SEMI,
    STATE(3591), 2,
      sym_line_comment,
      sym_block_comment,
  [97254] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7173), 1,
      anon_sym_SEMI,
    STATE(3592), 2,
      sym_line_comment,
      sym_block_comment,
  [97268] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7175), 1,
      sym__raw_string_literal_end,
    STATE(3593), 2,
      sym_line_comment,
      sym_block_comment,
  [97282] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5455), 1,
      anon_sym_RPAREN,
    STATE(3594), 2,
      sym_line_comment,
      sym_block_comment,
  [97296] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7177), 1,
      anon_sym_COLON,
    STATE(3595), 2,
      sym_line_comment,
      sym_block_comment,
  [97310] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7179), 1,
      anon_sym_EQ_GT,
    STATE(3596), 2,
      sym_line_comment,
      sym_block_comment,
  [97324] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7181), 1,
      sym_identifier,
    STATE(3597), 2,
      sym_line_comment,
      sym_block_comment,
  [97338] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7183), 1,
      anon_sym_COLON,
    STATE(3598), 2,
      sym_line_comment,
      sym_block_comment,
  [97352] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5239), 1,
      anon_sym_COLON_COLON,
    STATE(3599), 2,
      sym_line_comment,
      sym_block_comment,
  [97366] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7185), 1,
      anon_sym_COLON,
    STATE(3600), 2,
      sym_line_comment,
      sym_block_comment,
  [97380] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7187), 1,
      anon_sym_LBRACK,
    STATE(3601), 2,
      sym_line_comment,
      sym_block_comment,
  [97394] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7189), 1,
      anon_sym_EQ,
    STATE(3602), 2,
      sym_line_comment,
      sym_block_comment,
  [97408] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7191), 1,
      sym_identifier,
    STATE(3603), 2,
      sym_line_comment,
      sym_block_comment,
  [97422] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7193), 1,
      sym_identifier,
    STATE(3604), 2,
      sym_line_comment,
      sym_block_comment,
  [97436] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7195), 1,
      anon_sym_EQ,
    STATE(3605), 2,
      sym_line_comment,
      sym_block_comment,
  [97450] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7197), 1,
      anon_sym_COLON,
    STATE(3606), 2,
      sym_line_comment,
      sym_block_comment,
  [97464] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4529), 1,
      anon_sym_fn,
    STATE(3607), 2,
      sym_line_comment,
      sym_block_comment,
  [97478] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7199), 1,
      anon_sym_COLON,
    STATE(3608), 2,
      sym_line_comment,
      sym_block_comment,
  [97492] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7201), 1,
      anon_sym_LBRACK,
    STATE(3609), 2,
      sym_line_comment,
      sym_block_comment,
  [97506] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7203), 1,
      anon_sym_LBRACK,
    STATE(3610), 2,
      sym_line_comment,
      sym_block_comment,
  [97520] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7205), 1,
      anon_sym_COLON,
    STATE(3611), 2,
      sym_line_comment,
      sym_block_comment,
  [97534] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7207), 1,
      sym_identifier,
    STATE(3612), 2,
      sym_line_comment,
      sym_block_comment,
  [97548] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7209), 1,
      anon_sym_RPAREN,
    STATE(3613), 2,
      sym_line_comment,
      sym_block_comment,
  [97562] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7211), 1,
      anon_sym_COLON,
    STATE(3614), 2,
      sym_line_comment,
      sym_block_comment,
  [97576] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7213), 1,
      anon_sym_COLON,
    STATE(3615), 2,
      sym_line_comment,
      sym_block_comment,
  [97590] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7215), 1,
      anon_sym_EQ_GT,
    STATE(3616), 2,
      sym_line_comment,
      sym_block_comment,
  [97604] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7217), 1,
      sym_identifier,
    STATE(3617), 2,
      sym_line_comment,
      sym_block_comment,
  [97618] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7219), 1,
      anon_sym_COLON,
    STATE(3618), 2,
      sym_line_comment,
      sym_block_comment,
  [97632] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_COLON,
    STATE(3619), 2,
      sym_line_comment,
      sym_block_comment,
  [97646] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7223), 1,
      anon_sym_COLON,
    STATE(3620), 2,
      sym_line_comment,
      sym_block_comment,
  [97660] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6539), 1,
      anon_sym_SEMI,
    STATE(3621), 2,
      sym_line_comment,
      sym_block_comment,
  [97674] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(4565), 1,
      anon_sym_fn,
    STATE(3622), 2,
      sym_line_comment,
      sym_block_comment,
  [97688] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7225), 1,
      anon_sym_SEMI,
    STATE(3623), 2,
      sym_line_comment,
      sym_block_comment,
  [97702] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(772), 1,
      anon_sym_RBRACK,
    STATE(3624), 2,
      sym_line_comment,
      sym_block_comment,
  [97716] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7227), 1,
      sym_identifier,
    STATE(3625), 2,
      sym_line_comment,
      sym_block_comment,
  [97730] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7229), 1,
      sym_identifier,
    STATE(3626), 2,
      sym_line_comment,
      sym_block_comment,
  [97744] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7231), 1,
      anon_sym_RPAREN,
    STATE(3627), 2,
      sym_line_comment,
      sym_block_comment,
  [97758] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7233), 1,
      sym_identifier,
    STATE(3628), 2,
      sym_line_comment,
      sym_block_comment,
  [97772] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7235), 1,
      anon_sym_fn,
    STATE(3629), 2,
      sym_line_comment,
      sym_block_comment,
  [97786] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7237), 1,
      sym_raw_string_literal_content,
    STATE(3630), 2,
      sym_line_comment,
      sym_block_comment,
  [97800] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7239), 1,
      sym_identifier,
    STATE(3631), 2,
      sym_line_comment,
      sym_block_comment,
  [97814] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7241), 1,
      sym_identifier,
    STATE(3632), 2,
      sym_line_comment,
      sym_block_comment,
  [97828] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7243), 1,
      sym_identifier,
    STATE(3633), 2,
      sym_line_comment,
      sym_block_comment,
  [97842] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7245), 1,
      sym_identifier,
    STATE(3634), 2,
      sym_line_comment,
      sym_block_comment,
  [97856] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(3093), 1,
      anon_sym_PLUS,
    STATE(3635), 2,
      sym_line_comment,
      sym_block_comment,
  [97870] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7247), 1,
      sym_identifier,
    STATE(3636), 2,
      sym_line_comment,
      sym_block_comment,
  [97884] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(903), 1,
      anon_sym_RBRACK,
    STATE(3637), 2,
      sym_line_comment,
      sym_block_comment,
  [97898] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7249), 1,
      sym__line_doc_content,
    STATE(3638), 2,
      sym_line_comment,
      sym_block_comment,
  [97912] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6213), 1,
      anon_sym_RBRACE,
    STATE(3639), 2,
      sym_line_comment,
      sym_block_comment,
  [97926] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7251), 1,
      anon_sym_SEMI,
    STATE(3640), 2,
      sym_line_comment,
      sym_block_comment,
  [97940] = 4,
    ACTIONS(103), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(105), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7253), 1,
      anon_sym_COLON_COLON,
    STATE(3641), 2,
      sym_line_comment,
      sym_block_comment,
  [97954] = 1,
    ACTIONS(7255), 1,
      ts_builtin_sym_end,
  [97958] = 1,
    ACTIONS(7257), 1,
      ts_builtin_sym_end,
  [97962] = 1,
    ACTIONS(7259), 1,
      ts_builtin_sym_end,
  [97966] = 1,
    ACTIONS(7261), 1,
      ts_builtin_sym_end,
  [97970] = 1,
    ACTIONS(7263), 1,
      ts_builtin_sym_end,
  [97974] = 1,
    ACTIONS(7265), 1,
      ts_builtin_sym_end,
  [97978] = 1,
    ACTIONS(7267), 1,
      ts_builtin_sym_end,
};

static const uint32_t ts_small_parse_table_map[] = {
  [SMALL_STATE(1014)] = 0,
  [SMALL_STATE(1015)] = 75,
  [SMALL_STATE(1016)] = 146,
  [SMALL_STATE(1017)] = 216,
  [SMALL_STATE(1018)] = 316,
  [SMALL_STATE(1019)] = 416,
  [SMALL_STATE(1020)] = 495,
  [SMALL_STATE(1021)] = 572,
  [SMALL_STATE(1022)] = 636,
  [SMALL_STATE(1023)] = 710,
  [SMALL_STATE(1024)] = 774,
  [SMALL_STATE(1025)] = 838,
  [SMALL_STATE(1026)] = 902,
  [SMALL_STATE(1027)] = 966,
  [SMALL_STATE(1028)] = 1030,
  [SMALL_STATE(1029)] = 1104,
  [SMALL_STATE(1030)] = 1168,
  [SMALL_STATE(1031)] = 1242,
  [SMALL_STATE(1032)] = 1306,
  [SMALL_STATE(1033)] = 1370,
  [SMALL_STATE(1034)] = 1433,
  [SMALL_STATE(1035)] = 1504,
  [SMALL_STATE(1036)] = 1567,
  [SMALL_STATE(1037)] = 1638,
  [SMALL_STATE(1038)] = 1701,
  [SMALL_STATE(1039)] = 1768,
  [SMALL_STATE(1040)] = 1831,
  [SMALL_STATE(1041)] = 1902,
  [SMALL_STATE(1042)] = 1969,
  [SMALL_STATE(1043)] = 2036,
  [SMALL_STATE(1044)] = 2107,
  [SMALL_STATE(1045)] = 2170,
  [SMALL_STATE(1046)] = 2237,
  [SMALL_STATE(1047)] = 2304,
  [SMALL_STATE(1048)] = 2410,
  [SMALL_STATE(1049)] = 2516,
  [SMALL_STATE(1050)] = 2578,
  [SMALL_STATE(1051)] = 2640,
  [SMALL_STATE(1052)] = 2746,
  [SMALL_STATE(1053)] = 2846,
  [SMALL_STATE(1054)] = 2952,
  [SMALL_STATE(1055)] = 3014,
  [SMALL_STATE(1056)] = 3120,
  [SMALL_STATE(1057)] = 3220,
  [SMALL_STATE(1058)] = 3290,
  [SMALL_STATE(1059)] = 3396,
  [SMALL_STATE(1060)] = 3464,
  [SMALL_STATE(1061)] = 3526,
  [SMALL_STATE(1062)] = 3592,
  [SMALL_STATE(1063)] = 3667,
  [SMALL_STATE(1064)] = 3728,
  [SMALL_STATE(1065)] = 3791,
  [SMALL_STATE(1066)] = 3852,
  [SMALL_STATE(1067)] = 3913,
  [SMALL_STATE(1068)] = 3974,
  [SMALL_STATE(1069)] = 4035,
  [SMALL_STATE(1070)] = 4096,
  [SMALL_STATE(1071)] = 4159,
  [SMALL_STATE(1072)] = 4222,
  [SMALL_STATE(1073)] = 4283,
  [SMALL_STATE(1074)] = 4346,
  [SMALL_STATE(1075)] = 4409,
  [SMALL_STATE(1076)] = 4470,
  [SMALL_STATE(1077)] = 4531,
  [SMALL_STATE(1078)] = 4592,
  [SMALL_STATE(1079)] = 4655,
  [SMALL_STATE(1080)] = 4716,
  [SMALL_STATE(1081)] = 4777,
  [SMALL_STATE(1082)] = 4840,
  [SMALL_STATE(1083)] = 4903,
  [SMALL_STATE(1084)] = 4966,
  [SMALL_STATE(1085)] = 5027,
  [SMALL_STATE(1086)] = 5088,
  [SMALL_STATE(1087)] = 5181,
  [SMALL_STATE(1088)] = 5242,
  [SMALL_STATE(1089)] = 5335,
  [SMALL_STATE(1090)] = 5396,
  [SMALL_STATE(1091)] = 5457,
  [SMALL_STATE(1092)] = 5518,
  [SMALL_STATE(1093)] = 5581,
  [SMALL_STATE(1094)] = 5644,
  [SMALL_STATE(1095)] = 5707,
  [SMALL_STATE(1096)] = 5770,
  [SMALL_STATE(1097)] = 5833,
  [SMALL_STATE(1098)] = 5894,
  [SMALL_STATE(1099)] = 5959,
  [SMALL_STATE(1100)] = 6020,
  [SMALL_STATE(1101)] = 6081,
  [SMALL_STATE(1102)] = 6142,
  [SMALL_STATE(1103)] = 6203,
  [SMALL_STATE(1104)] = 6264,
  [SMALL_STATE(1105)] = 6325,
  [SMALL_STATE(1106)] = 6386,
  [SMALL_STATE(1107)] = 6447,
  [SMALL_STATE(1108)] = 6508,
  [SMALL_STATE(1109)] = 6569,
  [SMALL_STATE(1110)] = 6630,
  [SMALL_STATE(1111)] = 6691,
  [SMALL_STATE(1112)] = 6752,
  [SMALL_STATE(1113)] = 6813,
  [SMALL_STATE(1114)] = 6874,
  [SMALL_STATE(1115)] = 6935,
  [SMALL_STATE(1116)] = 6996,
  [SMALL_STATE(1117)] = 7057,
  [SMALL_STATE(1118)] = 7120,
  [SMALL_STATE(1119)] = 7183,
  [SMALL_STATE(1120)] = 7246,
  [SMALL_STATE(1121)] = 7307,
  [SMALL_STATE(1122)] = 7368,
  [SMALL_STATE(1123)] = 7429,
  [SMALL_STATE(1124)] = 7490,
  [SMALL_STATE(1125)] = 7551,
  [SMALL_STATE(1126)] = 7612,
  [SMALL_STATE(1127)] = 7673,
  [SMALL_STATE(1128)] = 7734,
  [SMALL_STATE(1129)] = 7795,
  [SMALL_STATE(1130)] = 7856,
  [SMALL_STATE(1131)] = 7917,
  [SMALL_STATE(1132)] = 7978,
  [SMALL_STATE(1133)] = 8039,
  [SMALL_STATE(1134)] = 8100,
  [SMALL_STATE(1135)] = 8161,
  [SMALL_STATE(1136)] = 8222,
  [SMALL_STATE(1137)] = 8283,
  [SMALL_STATE(1138)] = 8344,
  [SMALL_STATE(1139)] = 8405,
  [SMALL_STATE(1140)] = 8466,
  [SMALL_STATE(1141)] = 8527,
  [SMALL_STATE(1142)] = 8588,
  [SMALL_STATE(1143)] = 8649,
  [SMALL_STATE(1144)] = 8710,
  [SMALL_STATE(1145)] = 8771,
  [SMALL_STATE(1146)] = 8832,
  [SMALL_STATE(1147)] = 8893,
  [SMALL_STATE(1148)] = 8954,
  [SMALL_STATE(1149)] = 9015,
  [SMALL_STATE(1150)] = 9076,
  [SMALL_STATE(1151)] = 9137,
  [SMALL_STATE(1152)] = 9198,
  [SMALL_STATE(1153)] = 9259,
  [SMALL_STATE(1154)] = 9320,
  [SMALL_STATE(1155)] = 9381,
  [SMALL_STATE(1156)] = 9442,
  [SMALL_STATE(1157)] = 9503,
  [SMALL_STATE(1158)] = 9564,
  [SMALL_STATE(1159)] = 9627,
  [SMALL_STATE(1160)] = 9690,
  [SMALL_STATE(1161)] = 9751,
  [SMALL_STATE(1162)] = 9812,
  [SMALL_STATE(1163)] = 9873,
  [SMALL_STATE(1164)] = 9934,
  [SMALL_STATE(1165)] = 9995,
  [SMALL_STATE(1166)] = 10056,
  [SMALL_STATE(1167)] = 10117,
  [SMALL_STATE(1168)] = 10178,
  [SMALL_STATE(1169)] = 10239,
  [SMALL_STATE(1170)] = 10300,
  [SMALL_STATE(1171)] = 10361,
  [SMALL_STATE(1172)] = 10422,
  [SMALL_STATE(1173)] = 10483,
  [SMALL_STATE(1174)] = 10544,
  [SMALL_STATE(1175)] = 10605,
  [SMALL_STATE(1176)] = 10666,
  [SMALL_STATE(1177)] = 10727,
  [SMALL_STATE(1178)] = 10788,
  [SMALL_STATE(1179)] = 10849,
  [SMALL_STATE(1180)] = 10910,
  [SMALL_STATE(1181)] = 10971,
  [SMALL_STATE(1182)] = 11032,
  [SMALL_STATE(1183)] = 11093,
  [SMALL_STATE(1184)] = 11154,
  [SMALL_STATE(1185)] = 11215,
  [SMALL_STATE(1186)] = 11276,
  [SMALL_STATE(1187)] = 11337,
  [SMALL_STATE(1188)] = 11398,
  [SMALL_STATE(1189)] = 11459,
  [SMALL_STATE(1190)] = 11520,
  [SMALL_STATE(1191)] = 11581,
  [SMALL_STATE(1192)] = 11642,
  [SMALL_STATE(1193)] = 11703,
  [SMALL_STATE(1194)] = 11764,
  [SMALL_STATE(1195)] = 11825,
  [SMALL_STATE(1196)] = 11886,
  [SMALL_STATE(1197)] = 11947,
  [SMALL_STATE(1198)] = 12008,
  [SMALL_STATE(1199)] = 12069,
  [SMALL_STATE(1200)] = 12130,
  [SMALL_STATE(1201)] = 12191,
  [SMALL_STATE(1202)] = 12252,
  [SMALL_STATE(1203)] = 12313,
  [SMALL_STATE(1204)] = 12374,
  [SMALL_STATE(1205)] = 12435,
  [SMALL_STATE(1206)] = 12496,
  [SMALL_STATE(1207)] = 12557,
  [SMALL_STATE(1208)] = 12618,
  [SMALL_STATE(1209)] = 12679,
  [SMALL_STATE(1210)] = 12740,
  [SMALL_STATE(1211)] = 12801,
  [SMALL_STATE(1212)] = 12862,
  [SMALL_STATE(1213)] = 12923,
  [SMALL_STATE(1214)] = 12984,
  [SMALL_STATE(1215)] = 13045,
  [SMALL_STATE(1216)] = 13106,
  [SMALL_STATE(1217)] = 13167,
  [SMALL_STATE(1218)] = 13228,
  [SMALL_STATE(1219)] = 13289,
  [SMALL_STATE(1220)] = 13350,
  [SMALL_STATE(1221)] = 13411,
  [SMALL_STATE(1222)] = 13472,
  [SMALL_STATE(1223)] = 13533,
  [SMALL_STATE(1224)] = 13594,
  [SMALL_STATE(1225)] = 13655,
  [SMALL_STATE(1226)] = 13716,
  [SMALL_STATE(1227)] = 13777,
  [SMALL_STATE(1228)] = 13838,
  [SMALL_STATE(1229)] = 13899,
  [SMALL_STATE(1230)] = 13960,
  [SMALL_STATE(1231)] = 14021,
  [SMALL_STATE(1232)] = 14082,
  [SMALL_STATE(1233)] = 14143,
  [SMALL_STATE(1234)] = 14204,
  [SMALL_STATE(1235)] = 14265,
  [SMALL_STATE(1236)] = 14326,
  [SMALL_STATE(1237)] = 14387,
  [SMALL_STATE(1238)] = 14448,
  [SMALL_STATE(1239)] = 14509,
  [SMALL_STATE(1240)] = 14570,
  [SMALL_STATE(1241)] = 14631,
  [SMALL_STATE(1242)] = 14692,
  [SMALL_STATE(1243)] = 14753,
  [SMALL_STATE(1244)] = 14814,
  [SMALL_STATE(1245)] = 14875,
  [SMALL_STATE(1246)] = 14936,
  [SMALL_STATE(1247)] = 14997,
  [SMALL_STATE(1248)] = 15058,
  [SMALL_STATE(1249)] = 15119,
  [SMALL_STATE(1250)] = 15180,
  [SMALL_STATE(1251)] = 15241,
  [SMALL_STATE(1252)] = 15302,
  [SMALL_STATE(1253)] = 15363,
  [SMALL_STATE(1254)] = 15424,
  [SMALL_STATE(1255)] = 15485,
  [SMALL_STATE(1256)] = 15546,
  [SMALL_STATE(1257)] = 15607,
  [SMALL_STATE(1258)] = 15668,
  [SMALL_STATE(1259)] = 15729,
  [SMALL_STATE(1260)] = 15790,
  [SMALL_STATE(1261)] = 15851,
  [SMALL_STATE(1262)] = 15912,
  [SMALL_STATE(1263)] = 15973,
  [SMALL_STATE(1264)] = 16034,
  [SMALL_STATE(1265)] = 16095,
  [SMALL_STATE(1266)] = 16156,
  [SMALL_STATE(1267)] = 16217,
  [SMALL_STATE(1268)] = 16278,
  [SMALL_STATE(1269)] = 16339,
  [SMALL_STATE(1270)] = 16400,
  [SMALL_STATE(1271)] = 16461,
  [SMALL_STATE(1272)] = 16522,
  [SMALL_STATE(1273)] = 16583,
  [SMALL_STATE(1274)] = 16644,
  [SMALL_STATE(1275)] = 16705,
  [SMALL_STATE(1276)] = 16766,
  [SMALL_STATE(1277)] = 16827,
  [SMALL_STATE(1278)] = 16888,
  [SMALL_STATE(1279)] = 16949,
  [SMALL_STATE(1280)] = 17010,
  [SMALL_STATE(1281)] = 17071,
  [SMALL_STATE(1282)] = 17132,
  [SMALL_STATE(1283)] = 17193,
  [SMALL_STATE(1284)] = 17254,
  [SMALL_STATE(1285)] = 17315,
  [SMALL_STATE(1286)] = 17376,
  [SMALL_STATE(1287)] = 17437,
  [SMALL_STATE(1288)] = 17498,
  [SMALL_STATE(1289)] = 17559,
  [SMALL_STATE(1290)] = 17620,
  [SMALL_STATE(1291)] = 17681,
  [SMALL_STATE(1292)] = 17742,
  [SMALL_STATE(1293)] = 17803,
  [SMALL_STATE(1294)] = 17864,
  [SMALL_STATE(1295)] = 17925,
  [SMALL_STATE(1296)] = 17986,
  [SMALL_STATE(1297)] = 18047,
  [SMALL_STATE(1298)] = 18108,
  [SMALL_STATE(1299)] = 18169,
  [SMALL_STATE(1300)] = 18230,
  [SMALL_STATE(1301)] = 18291,
  [SMALL_STATE(1302)] = 18352,
  [SMALL_STATE(1303)] = 18413,
  [SMALL_STATE(1304)] = 18474,
  [SMALL_STATE(1305)] = 18535,
  [SMALL_STATE(1306)] = 18596,
  [SMALL_STATE(1307)] = 18657,
  [SMALL_STATE(1308)] = 18718,
  [SMALL_STATE(1309)] = 18779,
  [SMALL_STATE(1310)] = 18840,
  [SMALL_STATE(1311)] = 18901,
  [SMALL_STATE(1312)] = 18962,
  [SMALL_STATE(1313)] = 19023,
  [SMALL_STATE(1314)] = 19084,
  [SMALL_STATE(1315)] = 19145,
  [SMALL_STATE(1316)] = 19206,
  [SMALL_STATE(1317)] = 19267,
  [SMALL_STATE(1318)] = 19328,
  [SMALL_STATE(1319)] = 19389,
  [SMALL_STATE(1320)] = 19450,
  [SMALL_STATE(1321)] = 19511,
  [SMALL_STATE(1322)] = 19572,
  [SMALL_STATE(1323)] = 19633,
  [SMALL_STATE(1324)] = 19694,
  [SMALL_STATE(1325)] = 19755,
  [SMALL_STATE(1326)] = 19816,
  [SMALL_STATE(1327)] = 19877,
  [SMALL_STATE(1328)] = 19938,
  [SMALL_STATE(1329)] = 19999,
  [SMALL_STATE(1330)] = 20060,
  [SMALL_STATE(1331)] = 20121,
  [SMALL_STATE(1332)] = 20182,
  [SMALL_STATE(1333)] = 20243,
  [SMALL_STATE(1334)] = 20304,
  [SMALL_STATE(1335)] = 20365,
  [SMALL_STATE(1336)] = 20426,
  [SMALL_STATE(1337)] = 20487,
  [SMALL_STATE(1338)] = 20548,
  [SMALL_STATE(1339)] = 20609,
  [SMALL_STATE(1340)] = 20670,
  [SMALL_STATE(1341)] = 20731,
  [SMALL_STATE(1342)] = 20792,
  [SMALL_STATE(1343)] = 20853,
  [SMALL_STATE(1344)] = 20914,
  [SMALL_STATE(1345)] = 20975,
  [SMALL_STATE(1346)] = 21036,
  [SMALL_STATE(1347)] = 21097,
  [SMALL_STATE(1348)] = 21158,
  [SMALL_STATE(1349)] = 21219,
  [SMALL_STATE(1350)] = 21280,
  [SMALL_STATE(1351)] = 21341,
  [SMALL_STATE(1352)] = 21402,
  [SMALL_STATE(1353)] = 21463,
  [SMALL_STATE(1354)] = 21524,
  [SMALL_STATE(1355)] = 21585,
  [SMALL_STATE(1356)] = 21646,
  [SMALL_STATE(1357)] = 21707,
  [SMALL_STATE(1358)] = 21768,
  [SMALL_STATE(1359)] = 21829,
  [SMALL_STATE(1360)] = 21890,
  [SMALL_STATE(1361)] = 21951,
  [SMALL_STATE(1362)] = 22012,
  [SMALL_STATE(1363)] = 22073,
  [SMALL_STATE(1364)] = 22134,
  [SMALL_STATE(1365)] = 22195,
  [SMALL_STATE(1366)] = 22260,
  [SMALL_STATE(1367)] = 22321,
  [SMALL_STATE(1368)] = 22382,
  [SMALL_STATE(1369)] = 22443,
  [SMALL_STATE(1370)] = 22508,
  [SMALL_STATE(1371)] = 22583,
  [SMALL_STATE(1372)] = 22646,
  [SMALL_STATE(1373)] = 22707,
  [SMALL_STATE(1374)] = 22769,
  [SMALL_STATE(1375)] = 22829,
  [SMALL_STATE(1376)] = 22889,
  [SMALL_STATE(1377)] = 22949,
  [SMALL_STATE(1378)] = 23017,
  [SMALL_STATE(1379)] = 23077,
  [SMALL_STATE(1380)] = 23137,
  [SMALL_STATE(1381)] = 23197,
  [SMALL_STATE(1382)] = 23257,
  [SMALL_STATE(1383)] = 23317,
  [SMALL_STATE(1384)] = 23377,
  [SMALL_STATE(1385)] = 23437,
  [SMALL_STATE(1386)] = 23497,
  [SMALL_STATE(1387)] = 23557,
  [SMALL_STATE(1388)] = 23617,
  [SMALL_STATE(1389)] = 23677,
  [SMALL_STATE(1390)] = 23737,
  [SMALL_STATE(1391)] = 23797,
  [SMALL_STATE(1392)] = 23857,
  [SMALL_STATE(1393)] = 23917,
  [SMALL_STATE(1394)] = 23977,
  [SMALL_STATE(1395)] = 24043,
  [SMALL_STATE(1396)] = 24103,
  [SMALL_STATE(1397)] = 24163,
  [SMALL_STATE(1398)] = 24223,
  [SMALL_STATE(1399)] = 24283,
  [SMALL_STATE(1400)] = 24343,
  [SMALL_STATE(1401)] = 24403,
  [SMALL_STATE(1402)] = 24463,
  [SMALL_STATE(1403)] = 24529,
  [SMALL_STATE(1404)] = 24589,
  [SMALL_STATE(1405)] = 24649,
  [SMALL_STATE(1406)] = 24709,
  [SMALL_STATE(1407)] = 24769,
  [SMALL_STATE(1408)] = 24829,
  [SMALL_STATE(1409)] = 24889,
  [SMALL_STATE(1410)] = 24949,
  [SMALL_STATE(1411)] = 25009,
  [SMALL_STATE(1412)] = 25069,
  [SMALL_STATE(1413)] = 25129,
  [SMALL_STATE(1414)] = 25189,
  [SMALL_STATE(1415)] = 25249,
  [SMALL_STATE(1416)] = 25309,
  [SMALL_STATE(1417)] = 25369,
  [SMALL_STATE(1418)] = 25429,
  [SMALL_STATE(1419)] = 25495,
  [SMALL_STATE(1420)] = 25555,
  [SMALL_STATE(1421)] = 25615,
  [SMALL_STATE(1422)] = 25675,
  [SMALL_STATE(1423)] = 25735,
  [SMALL_STATE(1424)] = 25795,
  [SMALL_STATE(1425)] = 25855,
  [SMALL_STATE(1426)] = 25915,
  [SMALL_STATE(1427)] = 25975,
  [SMALL_STATE(1428)] = 26035,
  [SMALL_STATE(1429)] = 26131,
  [SMALL_STATE(1430)] = 26191,
  [SMALL_STATE(1431)] = 26251,
  [SMALL_STATE(1432)] = 26317,
  [SMALL_STATE(1433)] = 26377,
  [SMALL_STATE(1434)] = 26437,
  [SMALL_STATE(1435)] = 26497,
  [SMALL_STATE(1436)] = 26557,
  [SMALL_STATE(1437)] = 26617,
  [SMALL_STATE(1438)] = 26677,
  [SMALL_STATE(1439)] = 26737,
  [SMALL_STATE(1440)] = 26797,
  [SMALL_STATE(1441)] = 26857,
  [SMALL_STATE(1442)] = 26917,
  [SMALL_STATE(1443)] = 26977,
  [SMALL_STATE(1444)] = 27037,
  [SMALL_STATE(1445)] = 27097,
  [SMALL_STATE(1446)] = 27157,
  [SMALL_STATE(1447)] = 27217,
  [SMALL_STATE(1448)] = 27277,
  [SMALL_STATE(1449)] = 27373,
  [SMALL_STATE(1450)] = 27433,
  [SMALL_STATE(1451)] = 27493,
  [SMALL_STATE(1452)] = 27553,
  [SMALL_STATE(1453)] = 27613,
  [SMALL_STATE(1454)] = 27677,
  [SMALL_STATE(1455)] = 27737,
  [SMALL_STATE(1456)] = 27797,
  [SMALL_STATE(1457)] = 27859,
  [SMALL_STATE(1458)] = 27919,
  [SMALL_STATE(1459)] = 27979,
  [SMALL_STATE(1460)] = 28039,
  [SMALL_STATE(1461)] = 28099,
  [SMALL_STATE(1462)] = 28159,
  [SMALL_STATE(1463)] = 28219,
  [SMALL_STATE(1464)] = 28279,
  [SMALL_STATE(1465)] = 28339,
  [SMALL_STATE(1466)] = 28413,
  [SMALL_STATE(1467)] = 28473,
  [SMALL_STATE(1468)] = 28533,
  [SMALL_STATE(1469)] = 28593,
  [SMALL_STATE(1470)] = 28653,
  [SMALL_STATE(1471)] = 28713,
  [SMALL_STATE(1472)] = 28773,
  [SMALL_STATE(1473)] = 28833,
  [SMALL_STATE(1474)] = 28893,
  [SMALL_STATE(1475)] = 28953,
  [SMALL_STATE(1476)] = 29013,
  [SMALL_STATE(1477)] = 29073,
  [SMALL_STATE(1478)] = 29133,
  [SMALL_STATE(1479)] = 29193,
  [SMALL_STATE(1480)] = 29253,
  [SMALL_STATE(1481)] = 29313,
  [SMALL_STATE(1482)] = 29373,
  [SMALL_STATE(1483)] = 29433,
  [SMALL_STATE(1484)] = 29493,
  [SMALL_STATE(1485)] = 29553,
  [SMALL_STATE(1486)] = 29613,
  [SMALL_STATE(1487)] = 29673,
  [SMALL_STATE(1488)] = 29737,
  [SMALL_STATE(1489)] = 29797,
  [SMALL_STATE(1490)] = 29857,
  [SMALL_STATE(1491)] = 29917,
  [SMALL_STATE(1492)] = 29977,
  [SMALL_STATE(1493)] = 30037,
  [SMALL_STATE(1494)] = 30097,
  [SMALL_STATE(1495)] = 30157,
  [SMALL_STATE(1496)] = 30256,
  [SMALL_STATE(1497)] = 30317,
  [SMALL_STATE(1498)] = 30386,
  [SMALL_STATE(1499)] = 30485,
  [SMALL_STATE(1500)] = 30556,
  [SMALL_STATE(1501)] = 30655,
  [SMALL_STATE(1502)] = 30716,
  [SMALL_STATE(1503)] = 30808,
  [SMALL_STATE(1504)] = 30898,
  [SMALL_STATE(1505)] = 30984,
  [SMALL_STATE(1506)] = 31068,
  [SMALL_STATE(1507)] = 31138,
  [SMALL_STATE(1508)] = 31228,
  [SMALL_STATE(1509)] = 31318,
  [SMALL_STATE(1510)] = 31404,
  [SMALL_STATE(1511)] = 31482,
  [SMALL_STATE(1512)] = 31568,
  [SMALL_STATE(1513)] = 31638,
  [SMALL_STATE(1514)] = 31706,
  [SMALL_STATE(1515)] = 31796,
  [SMALL_STATE(1516)] = 31862,
  [SMALL_STATE(1517)] = 31948,
  [SMALL_STATE(1518)] = 32038,
  [SMALL_STATE(1519)] = 32130,
  [SMALL_STATE(1520)] = 32220,
  [SMALL_STATE(1521)] = 32306,
  [SMALL_STATE(1522)] = 32380,
  [SMALL_STATE(1523)] = 32452,
  [SMALL_STATE(1524)] = 32538,
  [SMALL_STATE(1525)] = 32614,
  [SMALL_STATE(1526)] = 32696,
  [SMALL_STATE(1527)] = 32788,
  [SMALL_STATE(1528)] = 32851,
  [SMALL_STATE(1529)] = 32918,
  [SMALL_STATE(1530)] = 32975,
  [SMALL_STATE(1531)] = 33042,
  [SMALL_STATE(1532)] = 33099,
  [SMALL_STATE(1533)] = 33158,
  [SMALL_STATE(1534)] = 33215,
  [SMALL_STATE(1535)] = 33272,
  [SMALL_STATE(1536)] = 33329,
  [SMALL_STATE(1537)] = 33396,
  [SMALL_STATE(1538)] = 33455,
  [SMALL_STATE(1539)] = 33518,
  [SMALL_STATE(1540)] = 33583,
  [SMALL_STATE(1541)] = 33647,
  [SMALL_STATE(1542)] = 33703,
  [SMALL_STATE(1543)] = 33761,
  [SMALL_STATE(1544)] = 33821,
  [SMALL_STATE(1545)] = 33881,
  [SMALL_STATE(1546)] = 33941,
  [SMALL_STATE(1547)] = 34001,
  [SMALL_STATE(1548)] = 34057,
  [SMALL_STATE(1549)] = 34121,
  [SMALL_STATE(1550)] = 34177,
  [SMALL_STATE(1551)] = 34235,
  [SMALL_STATE(1552)] = 34293,
  [SMALL_STATE(1553)] = 34353,
  [SMALL_STATE(1554)] = 34411,
  [SMALL_STATE(1555)] = 34467,
  [SMALL_STATE(1556)] = 34527,
  [SMALL_STATE(1557)] = 34591,
  [SMALL_STATE(1558)] = 34655,
  [SMALL_STATE(1559)] = 34715,
  [SMALL_STATE(1560)] = 34772,
  [SMALL_STATE(1561)] = 34867,
  [SMALL_STATE(1562)] = 34962,
  [SMALL_STATE(1563)] = 35057,
  [SMALL_STATE(1564)] = 35114,
  [SMALL_STATE(1565)] = 35209,
  [SMALL_STATE(1566)] = 35298,
  [SMALL_STATE(1567)] = 35393,
  [SMALL_STATE(1568)] = 35452,
  [SMALL_STATE(1569)] = 35509,
  [SMALL_STATE(1570)] = 35564,
  [SMALL_STATE(1571)] = 35623,
  [SMALL_STATE(1572)] = 35678,
  [SMALL_STATE(1573)] = 35773,
  [SMALL_STATE(1574)] = 35832,
  [SMALL_STATE(1575)] = 35927,
  [SMALL_STATE(1576)] = 35986,
  [SMALL_STATE(1577)] = 36081,
  [SMALL_STATE(1578)] = 36176,
  [SMALL_STATE(1579)] = 36231,
  [SMALL_STATE(1580)] = 36326,
  [SMALL_STATE(1581)] = 36383,
  [SMALL_STATE(1582)] = 36478,
  [SMALL_STATE(1583)] = 36535,
  [SMALL_STATE(1584)] = 36590,
  [SMALL_STATE(1585)] = 36679,
  [SMALL_STATE(1586)] = 36734,
  [SMALL_STATE(1587)] = 36791,
  [SMALL_STATE(1588)] = 36846,
  [SMALL_STATE(1589)] = 36901,
  [SMALL_STATE(1590)] = 36996,
  [SMALL_STATE(1591)] = 37051,
  [SMALL_STATE(1592)] = 37140,
  [SMALL_STATE(1593)] = 37197,
  [SMALL_STATE(1594)] = 37269,
  [SMALL_STATE(1595)] = 37323,
  [SMALL_STATE(1596)] = 37405,
  [SMALL_STATE(1597)] = 37483,
  [SMALL_STATE(1598)] = 37539,
  [SMALL_STATE(1599)] = 37593,
  [SMALL_STATE(1600)] = 37685,
  [SMALL_STATE(1601)] = 37759,
  [SMALL_STATE(1602)] = 37813,
  [SMALL_STATE(1603)] = 37867,
  [SMALL_STATE(1604)] = 37923,
  [SMALL_STATE(1605)] = 38011,
  [SMALL_STATE(1606)] = 38091,
  [SMALL_STATE(1607)] = 38173,
  [SMALL_STATE(1608)] = 38259,
  [SMALL_STATE(1609)] = 38345,
  [SMALL_STATE(1610)] = 38431,
  [SMALL_STATE(1611)] = 38487,
  [SMALL_STATE(1612)] = 38543,
  [SMALL_STATE(1613)] = 38597,
  [SMALL_STATE(1614)] = 38689,
  [SMALL_STATE(1615)] = 38781,
  [SMALL_STATE(1616)] = 38835,
  [SMALL_STATE(1617)] = 38889,
  [SMALL_STATE(1618)] = 38981,
  [SMALL_STATE(1619)] = 39035,
  [SMALL_STATE(1620)] = 39091,
  [SMALL_STATE(1621)] = 39145,
  [SMALL_STATE(1622)] = 39231,
  [SMALL_STATE(1623)] = 39287,
  [SMALL_STATE(1624)] = 39375,
  [SMALL_STATE(1625)] = 39467,
  [SMALL_STATE(1626)] = 39523,
  [SMALL_STATE(1627)] = 39581,
  [SMALL_STATE(1628)] = 39647,
  [SMALL_STATE(1629)] = 39703,
  [SMALL_STATE(1630)] = 39757,
  [SMALL_STATE(1631)] = 39849,
  [SMALL_STATE(1632)] = 39935,
  [SMALL_STATE(1633)] = 40021,
  [SMALL_STATE(1634)] = 40077,
  [SMALL_STATE(1635)] = 40131,
  [SMALL_STATE(1636)] = 40187,
  [SMALL_STATE(1637)] = 40241,
  [SMALL_STATE(1638)] = 40297,
  [SMALL_STATE(1639)] = 40385,
  [SMALL_STATE(1640)] = 40449,
  [SMALL_STATE(1641)] = 40519,
  [SMALL_STATE(1642)] = 40607,
  [SMALL_STATE(1643)] = 40663,
  [SMALL_STATE(1644)] = 40743,
  [SMALL_STATE(1645)] = 40799,
  [SMALL_STATE(1646)] = 40867,
  [SMALL_STATE(1647)] = 40923,
  [SMALL_STATE(1648)] = 40979,
  [SMALL_STATE(1649)] = 41033,
  [SMALL_STATE(1650)] = 41089,
  [SMALL_STATE(1651)] = 41143,
  [SMALL_STATE(1652)] = 41197,
  [SMALL_STATE(1653)] = 41251,
  [SMALL_STATE(1654)] = 41339,
  [SMALL_STATE(1655)] = 41428,
  [SMALL_STATE(1656)] = 41481,
  [SMALL_STATE(1657)] = 41570,
  [SMALL_STATE(1658)] = 41629,
  [SMALL_STATE(1659)] = 41718,
  [SMALL_STATE(1660)] = 41771,
  [SMALL_STATE(1661)] = 41824,
  [SMALL_STATE(1662)] = 41877,
  [SMALL_STATE(1663)] = 41964,
  [SMALL_STATE(1664)] = 42017,
  [SMALL_STATE(1665)] = 42070,
  [SMALL_STATE(1666)] = 42159,
  [SMALL_STATE(1667)] = 42212,
  [SMALL_STATE(1668)] = 42265,
  [SMALL_STATE(1669)] = 42318,
  [SMALL_STATE(1670)] = 42371,
  [SMALL_STATE(1671)] = 42424,
  [SMALL_STATE(1672)] = 42477,
  [SMALL_STATE(1673)] = 42530,
  [SMALL_STATE(1674)] = 42583,
  [SMALL_STATE(1675)] = 42636,
  [SMALL_STATE(1676)] = 42689,
  [SMALL_STATE(1677)] = 42748,
  [SMALL_STATE(1678)] = 42807,
  [SMALL_STATE(1679)] = 42896,
  [SMALL_STATE(1680)] = 42985,
  [SMALL_STATE(1681)] = 43066,
  [SMALL_STATE(1682)] = 43151,
  [SMALL_STATE(1683)] = 43204,
  [SMALL_STATE(1684)] = 43257,
  [SMALL_STATE(1685)] = 43310,
  [SMALL_STATE(1686)] = 43395,
  [SMALL_STATE(1687)] = 43480,
  [SMALL_STATE(1688)] = 43567,
  [SMALL_STATE(1689)] = 43654,
  [SMALL_STATE(1690)] = 43707,
  [SMALL_STATE(1691)] = 43796,
  [SMALL_STATE(1692)] = 43849,
  [SMALL_STATE(1693)] = 43902,
  [SMALL_STATE(1694)] = 43955,
  [SMALL_STATE(1695)] = 44008,
  [SMALL_STATE(1696)] = 44095,
  [SMALL_STATE(1697)] = 44148,
  [SMALL_STATE(1698)] = 44201,
  [SMALL_STATE(1699)] = 44254,
  [SMALL_STATE(1700)] = 44307,
  [SMALL_STATE(1701)] = 44360,
  [SMALL_STATE(1702)] = 44413,
  [SMALL_STATE(1703)] = 44502,
  [SMALL_STATE(1704)] = 44587,
  [SMALL_STATE(1705)] = 44674,
  [SMALL_STATE(1706)] = 44761,
  [SMALL_STATE(1707)] = 44814,
  [SMALL_STATE(1708)] = 44867,
  [SMALL_STATE(1709)] = 44930,
  [SMALL_STATE(1710)] = 44999,
  [SMALL_STATE(1711)] = 45066,
  [SMALL_STATE(1712)] = 45137,
  [SMALL_STATE(1713)] = 45214,
  [SMALL_STATE(1714)] = 45293,
  [SMALL_STATE(1715)] = 45358,
  [SMALL_STATE(1716)] = 45443,
  [SMALL_STATE(1717)] = 45528,
  [SMALL_STATE(1718)] = 45601,
  [SMALL_STATE(1719)] = 45682,
  [SMALL_STATE(1720)] = 45735,
  [SMALL_STATE(1721)] = 45788,
  [SMALL_STATE(1722)] = 45841,
  [SMALL_STATE(1723)] = 45894,
  [SMALL_STATE(1724)] = 45947,
  [SMALL_STATE(1725)] = 46000,
  [SMALL_STATE(1726)] = 46059,
  [SMALL_STATE(1727)] = 46146,
  [SMALL_STATE(1728)] = 46199,
  [SMALL_STATE(1729)] = 46252,
  [SMALL_STATE(1730)] = 46305,
  [SMALL_STATE(1731)] = 46358,
  [SMALL_STATE(1732)] = 46411,
  [SMALL_STATE(1733)] = 46488,
  [SMALL_STATE(1734)] = 46541,
  [SMALL_STATE(1735)] = 46594,
  [SMALL_STATE(1736)] = 46647,
  [SMALL_STATE(1737)] = 46700,
  [SMALL_STATE(1738)] = 46753,
  [SMALL_STATE(1739)] = 46806,
  [SMALL_STATE(1740)] = 46859,
  [SMALL_STATE(1741)] = 46946,
  [SMALL_STATE(1742)] = 46999,
  [SMALL_STATE(1743)] = 47084,
  [SMALL_STATE(1744)] = 47137,
  [SMALL_STATE(1745)] = 47190,
  [SMALL_STATE(1746)] = 47243,
  [SMALL_STATE(1747)] = 47296,
  [SMALL_STATE(1748)] = 47349,
  [SMALL_STATE(1749)] = 47402,
  [SMALL_STATE(1750)] = 47455,
  [SMALL_STATE(1751)] = 47508,
  [SMALL_STATE(1752)] = 47595,
  [SMALL_STATE(1753)] = 47648,
  [SMALL_STATE(1754)] = 47701,
  [SMALL_STATE(1755)] = 47786,
  [SMALL_STATE(1756)] = 47873,
  [SMALL_STATE(1757)] = 47926,
  [SMALL_STATE(1758)] = 47979,
  [SMALL_STATE(1759)] = 48066,
  [SMALL_STATE(1760)] = 48119,
  [SMALL_STATE(1761)] = 48182,
  [SMALL_STATE(1762)] = 48251,
  [SMALL_STATE(1763)] = 48318,
  [SMALL_STATE(1764)] = 48389,
  [SMALL_STATE(1765)] = 48466,
  [SMALL_STATE(1766)] = 48545,
  [SMALL_STATE(1767)] = 48610,
  [SMALL_STATE(1768)] = 48695,
  [SMALL_STATE(1769)] = 48780,
  [SMALL_STATE(1770)] = 48853,
  [SMALL_STATE(1771)] = 48906,
  [SMALL_STATE(1772)] = 48959,
  [SMALL_STATE(1773)] = 49012,
  [SMALL_STATE(1774)] = 49065,
  [SMALL_STATE(1775)] = 49118,
  [SMALL_STATE(1776)] = 49205,
  [SMALL_STATE(1777)] = 49258,
  [SMALL_STATE(1778)] = 49311,
  [SMALL_STATE(1779)] = 49364,
  [SMALL_STATE(1780)] = 49417,
  [SMALL_STATE(1781)] = 49470,
  [SMALL_STATE(1782)] = 49523,
  [SMALL_STATE(1783)] = 49576,
  [SMALL_STATE(1784)] = 49629,
  [SMALL_STATE(1785)] = 49718,
  [SMALL_STATE(1786)] = 49771,
  [SMALL_STATE(1787)] = 49824,
  [SMALL_STATE(1788)] = 49877,
  [SMALL_STATE(1789)] = 49930,
  [SMALL_STATE(1790)] = 49983,
  [SMALL_STATE(1791)] = 50070,
  [SMALL_STATE(1792)] = 50123,
  [SMALL_STATE(1793)] = 50176,
  [SMALL_STATE(1794)] = 50263,
  [SMALL_STATE(1795)] = 50316,
  [SMALL_STATE(1796)] = 50369,
  [SMALL_STATE(1797)] = 50456,
  [SMALL_STATE(1798)] = 50509,
  [SMALL_STATE(1799)] = 50562,
  [SMALL_STATE(1800)] = 50651,
  [SMALL_STATE(1801)] = 50738,
  [SMALL_STATE(1802)] = 50827,
  [SMALL_STATE(1803)] = 50914,
  [SMALL_STATE(1804)] = 50967,
  [SMALL_STATE(1805)] = 51056,
  [SMALL_STATE(1806)] = 51109,
  [SMALL_STATE(1807)] = 51198,
  [SMALL_STATE(1808)] = 51287,
  [SMALL_STATE(1809)] = 51376,
  [SMALL_STATE(1810)] = 51429,
  [SMALL_STATE(1811)] = 51518,
  [SMALL_STATE(1812)] = 51595,
  [SMALL_STATE(1813)] = 51684,
  [SMALL_STATE(1814)] = 51773,
  [SMALL_STATE(1815)] = 51862,
  [SMALL_STATE(1816)] = 51951,
  [SMALL_STATE(1817)] = 52038,
  [SMALL_STATE(1818)] = 52127,
  [SMALL_STATE(1819)] = 52216,
  [SMALL_STATE(1820)] = 52305,
  [SMALL_STATE(1821)] = 52394,
  [SMALL_STATE(1822)] = 52447,
  [SMALL_STATE(1823)] = 52536,
  [SMALL_STATE(1824)] = 52625,
  [SMALL_STATE(1825)] = 52706,
  [SMALL_STATE(1826)] = 52791,
  [SMALL_STATE(1827)] = 52880,
  [SMALL_STATE(1828)] = 52965,
  [SMALL_STATE(1829)] = 53050,
  [SMALL_STATE(1830)] = 53139,
  [SMALL_STATE(1831)] = 53192,
  [SMALL_STATE(1832)] = 53281,
  [SMALL_STATE(1833)] = 53370,
  [SMALL_STATE(1834)] = 53459,
  [SMALL_STATE(1835)] = 53548,
  [SMALL_STATE(1836)] = 53601,
  [SMALL_STATE(1837)] = 53662,
  [SMALL_STATE(1838)] = 53751,
  [SMALL_STATE(1839)] = 53840,
  [SMALL_STATE(1840)] = 53929,
  [SMALL_STATE(1841)] = 54018,
  [SMALL_STATE(1842)] = 54107,
  [SMALL_STATE(1843)] = 54196,
  [SMALL_STATE(1844)] = 54285,
  [SMALL_STATE(1845)] = 54374,
  [SMALL_STATE(1846)] = 54463,
  [SMALL_STATE(1847)] = 54552,
  [SMALL_STATE(1848)] = 54641,
  [SMALL_STATE(1849)] = 54730,
  [SMALL_STATE(1850)] = 54819,
  [SMALL_STATE(1851)] = 54908,
  [SMALL_STATE(1852)] = 54997,
  [SMALL_STATE(1853)] = 55086,
  [SMALL_STATE(1854)] = 55175,
  [SMALL_STATE(1855)] = 55264,
  [SMALL_STATE(1856)] = 55351,
  [SMALL_STATE(1857)] = 55404,
  [SMALL_STATE(1858)] = 55457,
  [SMALL_STATE(1859)] = 55510,
  [SMALL_STATE(1860)] = 55563,
  [SMALL_STATE(1861)] = 55616,
  [SMALL_STATE(1862)] = 55669,
  [SMALL_STATE(1863)] = 55754,
  [SMALL_STATE(1864)] = 55835,
  [SMALL_STATE(1865)] = 55921,
  [SMALL_STATE(1866)] = 56007,
  [SMALL_STATE(1867)] = 56091,
  [SMALL_STATE(1868)] = 56175,
  [SMALL_STATE(1869)] = 56261,
  [SMALL_STATE(1870)] = 56347,
  [SMALL_STATE(1871)] = 56421,
  [SMALL_STATE(1872)] = 56507,
  [SMALL_STATE(1873)] = 56593,
  [SMALL_STATE(1874)] = 56679,
  [SMALL_STATE(1875)] = 56765,
  [SMALL_STATE(1876)] = 56851,
  [SMALL_STATE(1877)] = 56937,
  [SMALL_STATE(1878)] = 57023,
  [SMALL_STATE(1879)] = 57097,
  [SMALL_STATE(1880)] = 57183,
  [SMALL_STATE(1881)] = 57269,
  [SMALL_STATE(1882)] = 57355,
  [SMALL_STATE(1883)] = 57441,
  [SMALL_STATE(1884)] = 57527,
  [SMALL_STATE(1885)] = 57613,
  [SMALL_STATE(1886)] = 57699,
  [SMALL_STATE(1887)] = 57785,
  [SMALL_STATE(1888)] = 57871,
  [SMALL_STATE(1889)] = 57957,
  [SMALL_STATE(1890)] = 58043,
  [SMALL_STATE(1891)] = 58129,
  [SMALL_STATE(1892)] = 58203,
  [SMALL_STATE(1893)] = 58289,
  [SMALL_STATE(1894)] = 58363,
  [SMALL_STATE(1895)] = 58449,
  [SMALL_STATE(1896)] = 58535,
  [SMALL_STATE(1897)] = 58621,
  [SMALL_STATE(1898)] = 58707,
  [SMALL_STATE(1899)] = 58781,
  [SMALL_STATE(1900)] = 58867,
  [SMALL_STATE(1901)] = 58953,
  [SMALL_STATE(1902)] = 59028,
  [SMALL_STATE(1903)] = 59103,
  [SMALL_STATE(1904)] = 59178,
  [SMALL_STATE(1905)] = 59253,
  [SMALL_STATE(1906)] = 59328,
  [SMALL_STATE(1907)] = 59403,
  [SMALL_STATE(1908)] = 59478,
  [SMALL_STATE(1909)] = 59553,
  [SMALL_STATE(1910)] = 59600,
  [SMALL_STATE(1911)] = 59647,
  [SMALL_STATE(1912)] = 59694,
  [SMALL_STATE(1913)] = 59756,
  [SMALL_STATE(1914)] = 59818,
  [SMALL_STATE(1915)] = 59880,
  [SMALL_STATE(1916)] = 59942,
  [SMALL_STATE(1917)] = 60004,
  [SMALL_STATE(1918)] = 60066,
  [SMALL_STATE(1919)] = 60128,
  [SMALL_STATE(1920)] = 60190,
  [SMALL_STATE(1921)] = 60249,
  [SMALL_STATE(1922)] = 60308,
  [SMALL_STATE(1923)] = 60356,
  [SMALL_STATE(1924)] = 60392,
  [SMALL_STATE(1925)] = 60428,
  [SMALL_STATE(1926)] = 60465,
  [SMALL_STATE(1927)] = 60510,
  [SMALL_STATE(1928)] = 60555,
  [SMALL_STATE(1929)] = 60592,
  [SMALL_STATE(1930)] = 60637,
  [SMALL_STATE(1931)] = 60674,
  [SMALL_STATE(1932)] = 60727,
  [SMALL_STATE(1933)] = 60764,
  [SMALL_STATE(1934)] = 60804,
  [SMALL_STATE(1935)] = 60840,
  [SMALL_STATE(1936)] = 60880,
  [SMALL_STATE(1937)] = 60920,
  [SMALL_STATE(1938)] = 60956,
  [SMALL_STATE(1939)] = 60996,
  [SMALL_STATE(1940)] = 61032,
  [SMALL_STATE(1941)] = 61068,
  [SMALL_STATE(1942)] = 61101,
  [SMALL_STATE(1943)] = 61134,
  [SMALL_STATE(1944)] = 61167,
  [SMALL_STATE(1945)] = 61200,
  [SMALL_STATE(1946)] = 61233,
  [SMALL_STATE(1947)] = 61266,
  [SMALL_STATE(1948)] = 61299,
  [SMALL_STATE(1949)] = 61330,
  [SMALL_STATE(1950)] = 61363,
  [SMALL_STATE(1951)] = 61423,
  [SMALL_STATE(1952)] = 61467,
  [SMALL_STATE(1953)] = 61499,
  [SMALL_STATE(1954)] = 61531,
  [SMALL_STATE(1955)] = 61591,
  [SMALL_STATE(1956)] = 61623,
  [SMALL_STATE(1957)] = 61669,
  [SMALL_STATE(1958)] = 61702,
  [SMALL_STATE(1959)] = 61731,
  [SMALL_STATE(1960)] = 61762,
  [SMALL_STATE(1961)] = 61793,
  [SMALL_STATE(1962)] = 61824,
  [SMALL_STATE(1963)] = 61857,
  [SMALL_STATE(1964)] = 61898,
  [SMALL_STATE(1965)] = 61927,
  [SMALL_STATE(1966)] = 61958,
  [SMALL_STATE(1967)] = 61991,
  [SMALL_STATE(1968)] = 62044,
  [SMALL_STATE(1969)] = 62099,
  [SMALL_STATE(1970)] = 62128,
  [SMALL_STATE(1971)] = 62161,
  [SMALL_STATE(1972)] = 62190,
  [SMALL_STATE(1973)] = 62221,
  [SMALL_STATE(1974)] = 62250,
  [SMALL_STATE(1975)] = 62303,
  [SMALL_STATE(1976)] = 62334,
  [SMALL_STATE(1977)] = 62363,
  [SMALL_STATE(1978)] = 62392,
  [SMALL_STATE(1979)] = 62425,
  [SMALL_STATE(1980)] = 62454,
  [SMALL_STATE(1981)] = 62487,
  [SMALL_STATE(1982)] = 62518,
  [SMALL_STATE(1983)] = 62549,
  [SMALL_STATE(1984)] = 62592,
  [SMALL_STATE(1985)] = 62621,
  [SMALL_STATE(1986)] = 62649,
  [SMALL_STATE(1987)] = 62677,
  [SMALL_STATE(1988)] = 62705,
  [SMALL_STATE(1989)] = 62735,
  [SMALL_STATE(1990)] = 62763,
  [SMALL_STATE(1991)] = 62807,
  [SMALL_STATE(1992)] = 62835,
  [SMALL_STATE(1993)] = 62863,
  [SMALL_STATE(1994)] = 62891,
  [SMALL_STATE(1995)] = 62919,
  [SMALL_STATE(1996)] = 62947,
  [SMALL_STATE(1997)] = 62975,
  [SMALL_STATE(1998)] = 63003,
  [SMALL_STATE(1999)] = 63031,
  [SMALL_STATE(2000)] = 63059,
  [SMALL_STATE(2001)] = 63087,
  [SMALL_STATE(2002)] = 63115,
  [SMALL_STATE(2003)] = 63143,
  [SMALL_STATE(2004)] = 63171,
  [SMALL_STATE(2005)] = 63199,
  [SMALL_STATE(2006)] = 63227,
  [SMALL_STATE(2007)] = 63255,
  [SMALL_STATE(2008)] = 63283,
  [SMALL_STATE(2009)] = 63311,
  [SMALL_STATE(2010)] = 63339,
  [SMALL_STATE(2011)] = 63367,
  [SMALL_STATE(2012)] = 63395,
  [SMALL_STATE(2013)] = 63423,
  [SMALL_STATE(2014)] = 63453,
  [SMALL_STATE(2015)] = 63481,
  [SMALL_STATE(2016)] = 63509,
  [SMALL_STATE(2017)] = 63537,
  [SMALL_STATE(2018)] = 63572,
  [SMALL_STATE(2019)] = 63623,
  [SMALL_STATE(2020)] = 63652,
  [SMALL_STATE(2021)] = 63681,
  [SMALL_STATE(2022)] = 63712,
  [SMALL_STATE(2023)] = 63744,
  [SMALL_STATE(2024)] = 63776,
  [SMALL_STATE(2025)] = 63808,
  [SMALL_STATE(2026)] = 63838,
  [SMALL_STATE(2027)] = 63870,
  [SMALL_STATE(2028)] = 63902,
  [SMALL_STATE(2029)] = 63934,
  [SMALL_STATE(2030)] = 63966,
  [SMALL_STATE(2031)] = 63998,
  [SMALL_STATE(2032)] = 64030,
  [SMALL_STATE(2033)] = 64074,
  [SMALL_STATE(2034)] = 64106,
  [SMALL_STATE(2035)] = 64134,
  [SMALL_STATE(2036)] = 64166,
  [SMALL_STATE(2037)] = 64198,
  [SMALL_STATE(2038)] = 64226,
  [SMALL_STATE(2039)] = 64271,
  [SMALL_STATE(2040)] = 64316,
  [SMALL_STATE(2041)] = 64361,
  [SMALL_STATE(2042)] = 64406,
  [SMALL_STATE(2043)] = 64443,
  [SMALL_STATE(2044)] = 64488,
  [SMALL_STATE(2045)] = 64533,
  [SMALL_STATE(2046)] = 64578,
  [SMALL_STATE(2047)] = 64623,
  [SMALL_STATE(2048)] = 64668,
  [SMALL_STATE(2049)] = 64713,
  [SMALL_STATE(2050)] = 64758,
  [SMALL_STATE(2051)] = 64803,
  [SMALL_STATE(2052)] = 64848,
  [SMALL_STATE(2053)] = 64893,
  [SMALL_STATE(2054)] = 64938,
  [SMALL_STATE(2055)] = 64976,
  [SMALL_STATE(2056)] = 65014,
  [SMALL_STATE(2057)] = 65042,
  [SMALL_STATE(2058)] = 65072,
  [SMALL_STATE(2059)] = 65106,
  [SMALL_STATE(2060)] = 65148,
  [SMALL_STATE(2061)] = 65176,
  [SMALL_STATE(2062)] = 65218,
  [SMALL_STATE(2063)] = 65256,
  [SMALL_STATE(2064)] = 65296,
  [SMALL_STATE(2065)] = 65320,
  [SMALL_STATE(2066)] = 65344,
  [SMALL_STATE(2067)] = 65372,
  [SMALL_STATE(2068)] = 65400,
  [SMALL_STATE(2069)] = 65428,
  [SMALL_STATE(2070)] = 65456,
  [SMALL_STATE(2071)] = 65498,
  [SMALL_STATE(2072)] = 65540,
  [SMALL_STATE(2073)] = 65570,
  [SMALL_STATE(2074)] = 65612,
  [SMALL_STATE(2075)] = 65640,
  [SMALL_STATE(2076)] = 65682,
  [SMALL_STATE(2077)] = 65710,
  [SMALL_STATE(2078)] = 65752,
  [SMALL_STATE(2079)] = 65776,
  [SMALL_STATE(2080)] = 65804,
  [SMALL_STATE(2081)] = 65832,
  [SMALL_STATE(2082)] = 65866,
  [SMALL_STATE(2083)] = 65906,
  [SMALL_STATE(2084)] = 65934,
  [SMALL_STATE(2085)] = 65972,
  [SMALL_STATE(2086)] = 66012,
  [SMALL_STATE(2087)] = 66050,
  [SMALL_STATE(2088)] = 66074,
  [SMALL_STATE(2089)] = 66104,
  [SMALL_STATE(2090)] = 66134,
  [SMALL_STATE(2091)] = 66172,
  [SMALL_STATE(2092)] = 66210,
  [SMALL_STATE(2093)] = 66248,
  [SMALL_STATE(2094)] = 66272,
  [SMALL_STATE(2095)] = 66310,
  [SMALL_STATE(2096)] = 66338,
  [SMALL_STATE(2097)] = 66361,
  [SMALL_STATE(2098)] = 66392,
  [SMALL_STATE(2099)] = 66415,
  [SMALL_STATE(2100)] = 66438,
  [SMALL_STATE(2101)] = 66461,
  [SMALL_STATE(2102)] = 66484,
  [SMALL_STATE(2103)] = 66507,
  [SMALL_STATE(2104)] = 66530,
  [SMALL_STATE(2105)] = 66553,
  [SMALL_STATE(2106)] = 66576,
  [SMALL_STATE(2107)] = 66599,
  [SMALL_STATE(2108)] = 66622,
  [SMALL_STATE(2109)] = 66645,
  [SMALL_STATE(2110)] = 66682,
  [SMALL_STATE(2111)] = 66713,
  [SMALL_STATE(2112)] = 66736,
  [SMALL_STATE(2113)] = 66767,
  [SMALL_STATE(2114)] = 66790,
  [SMALL_STATE(2115)] = 66813,
  [SMALL_STATE(2116)] = 66836,
  [SMALL_STATE(2117)] = 66859,
  [SMALL_STATE(2118)] = 66896,
  [SMALL_STATE(2119)] = 66929,
  [SMALL_STATE(2120)] = 66952,
  [SMALL_STATE(2121)] = 66975,
  [SMALL_STATE(2122)] = 66998,
  [SMALL_STATE(2123)] = 67021,
  [SMALL_STATE(2124)] = 67058,
  [SMALL_STATE(2125)] = 67081,
  [SMALL_STATE(2126)] = 67116,
  [SMALL_STATE(2127)] = 67139,
  [SMALL_STATE(2128)] = 67170,
  [SMALL_STATE(2129)] = 67193,
  [SMALL_STATE(2130)] = 67216,
  [SMALL_STATE(2131)] = 67257,
  [SMALL_STATE(2132)] = 67280,
  [SMALL_STATE(2133)] = 67315,
  [SMALL_STATE(2134)] = 67356,
  [SMALL_STATE(2135)] = 67379,
  [SMALL_STATE(2136)] = 67402,
  [SMALL_STATE(2137)] = 67425,
  [SMALL_STATE(2138)] = 67448,
  [SMALL_STATE(2139)] = 67485,
  [SMALL_STATE(2140)] = 67520,
  [SMALL_STATE(2141)] = 67543,
  [SMALL_STATE(2142)] = 67584,
  [SMALL_STATE(2143)] = 67607,
  [SMALL_STATE(2144)] = 67630,
  [SMALL_STATE(2145)] = 67653,
  [SMALL_STATE(2146)] = 67676,
  [SMALL_STATE(2147)] = 67717,
  [SMALL_STATE(2148)] = 67740,
  [SMALL_STATE(2149)] = 67778,
  [SMALL_STATE(2150)] = 67804,
  [SMALL_STATE(2151)] = 67842,
  [SMALL_STATE(2152)] = 67874,
  [SMALL_STATE(2153)] = 67906,
  [SMALL_STATE(2154)] = 67938,
  [SMALL_STATE(2155)] = 67962,
  [SMALL_STATE(2156)] = 67994,
  [SMALL_STATE(2157)] = 68028,
  [SMALL_STATE(2158)] = 68066,
  [SMALL_STATE(2159)] = 68090,
  [SMALL_STATE(2160)] = 68114,
  [SMALL_STATE(2161)] = 68142,
  [SMALL_STATE(2162)] = 68166,
  [SMALL_STATE(2163)] = 68190,
  [SMALL_STATE(2164)] = 68224,
  [SMALL_STATE(2165)] = 68252,
  [SMALL_STATE(2166)] = 68280,
  [SMALL_STATE(2167)] = 68306,
  [SMALL_STATE(2168)] = 68338,
  [SMALL_STATE(2169)] = 68370,
  [SMALL_STATE(2170)] = 68396,
  [SMALL_STATE(2171)] = 68424,
  [SMALL_STATE(2172)] = 68450,
  [SMALL_STATE(2173)] = 68488,
  [SMALL_STATE(2174)] = 68516,
  [SMALL_STATE(2175)] = 68554,
  [SMALL_STATE(2176)] = 68586,
  [SMALL_STATE(2177)] = 68624,
  [SMALL_STATE(2178)] = 68656,
  [SMALL_STATE(2179)] = 68694,
  [SMALL_STATE(2180)] = 68732,
  [SMALL_STATE(2181)] = 68770,
  [SMALL_STATE(2182)] = 68798,
  [SMALL_STATE(2183)] = 68836,
  [SMALL_STATE(2184)] = 68874,
  [SMALL_STATE(2185)] = 68902,
  [SMALL_STATE(2186)] = 68940,
  [SMALL_STATE(2187)] = 68972,
  [SMALL_STATE(2188)] = 69010,
  [SMALL_STATE(2189)] = 69046,
  [SMALL_STATE(2190)] = 69084,
  [SMALL_STATE(2191)] = 69122,
  [SMALL_STATE(2192)] = 69154,
  [SMALL_STATE(2193)] = 69186,
  [SMALL_STATE(2194)] = 69218,
  [SMALL_STATE(2195)] = 69250,
  [SMALL_STATE(2196)] = 69288,
  [SMALL_STATE(2197)] = 69323,
  [SMALL_STATE(2198)] = 69358,
  [SMALL_STATE(2199)] = 69393,
  [SMALL_STATE(2200)] = 69414,
  [SMALL_STATE(2201)] = 69449,
  [SMALL_STATE(2202)] = 69484,
  [SMALL_STATE(2203)] = 69519,
  [SMALL_STATE(2204)] = 69554,
  [SMALL_STATE(2205)] = 69589,
  [SMALL_STATE(2206)] = 69624,
  [SMALL_STATE(2207)] = 69657,
  [SMALL_STATE(2208)] = 69692,
  [SMALL_STATE(2209)] = 69727,
  [SMALL_STATE(2210)] = 69760,
  [SMALL_STATE(2211)] = 69793,
  [SMALL_STATE(2212)] = 69828,
  [SMALL_STATE(2213)] = 69863,
  [SMALL_STATE(2214)] = 69898,
  [SMALL_STATE(2215)] = 69933,
  [SMALL_STATE(2216)] = 69968,
  [SMALL_STATE(2217)] = 70003,
  [SMALL_STATE(2218)] = 70038,
  [SMALL_STATE(2219)] = 70073,
  [SMALL_STATE(2220)] = 70094,
  [SMALL_STATE(2221)] = 70115,
  [SMALL_STATE(2222)] = 70150,
  [SMALL_STATE(2223)] = 70185,
  [SMALL_STATE(2224)] = 70220,
  [SMALL_STATE(2225)] = 70241,
  [SMALL_STATE(2226)] = 70276,
  [SMALL_STATE(2227)] = 70311,
  [SMALL_STATE(2228)] = 70332,
  [SMALL_STATE(2229)] = 70353,
  [SMALL_STATE(2230)] = 70388,
  [SMALL_STATE(2231)] = 70423,
  [SMALL_STATE(2232)] = 70458,
  [SMALL_STATE(2233)] = 70493,
  [SMALL_STATE(2234)] = 70518,
  [SMALL_STATE(2235)] = 70553,
  [SMALL_STATE(2236)] = 70588,
  [SMALL_STATE(2237)] = 70613,
  [SMALL_STATE(2238)] = 70648,
  [SMALL_STATE(2239)] = 70673,
  [SMALL_STATE(2240)] = 70702,
  [SMALL_STATE(2241)] = 70737,
  [SMALL_STATE(2242)] = 70772,
  [SMALL_STATE(2243)] = 70805,
  [SMALL_STATE(2244)] = 70840,
  [SMALL_STATE(2245)] = 70875,
  [SMALL_STATE(2246)] = 70902,
  [SMALL_STATE(2247)] = 70937,
  [SMALL_STATE(2248)] = 70970,
  [SMALL_STATE(2249)] = 71005,
  [SMALL_STATE(2250)] = 71040,
  [SMALL_STATE(2251)] = 71075,
  [SMALL_STATE(2252)] = 71110,
  [SMALL_STATE(2253)] = 71140,
  [SMALL_STATE(2254)] = 71172,
  [SMALL_STATE(2255)] = 71204,
  [SMALL_STATE(2256)] = 71236,
  [SMALL_STATE(2257)] = 71258,
  [SMALL_STATE(2258)] = 71290,
  [SMALL_STATE(2259)] = 71312,
  [SMALL_STATE(2260)] = 71344,
  [SMALL_STATE(2261)] = 71376,
  [SMALL_STATE(2262)] = 71408,
  [SMALL_STATE(2263)] = 71438,
  [SMALL_STATE(2264)] = 71468,
  [SMALL_STATE(2265)] = 71500,
  [SMALL_STATE(2266)] = 71522,
  [SMALL_STATE(2267)] = 71552,
  [SMALL_STATE(2268)] = 71584,
  [SMALL_STATE(2269)] = 71616,
  [SMALL_STATE(2270)] = 71646,
  [SMALL_STATE(2271)] = 71674,
  [SMALL_STATE(2272)] = 71706,
  [SMALL_STATE(2273)] = 71738,
  [SMALL_STATE(2274)] = 71770,
  [SMALL_STATE(2275)] = 71802,
  [SMALL_STATE(2276)] = 71834,
  [SMALL_STATE(2277)] = 71866,
  [SMALL_STATE(2278)] = 71898,
  [SMALL_STATE(2279)] = 71930,
  [SMALL_STATE(2280)] = 71960,
  [SMALL_STATE(2281)] = 71988,
  [SMALL_STATE(2282)] = 72020,
  [SMALL_STATE(2283)] = 72052,
  [SMALL_STATE(2284)] = 72084,
  [SMALL_STATE(2285)] = 72116,
  [SMALL_STATE(2286)] = 72144,
  [SMALL_STATE(2287)] = 72170,
  [SMALL_STATE(2288)] = 72196,
  [SMALL_STATE(2289)] = 72218,
  [SMALL_STATE(2290)] = 72246,
  [SMALL_STATE(2291)] = 72276,
  [SMALL_STATE(2292)] = 72306,
  [SMALL_STATE(2293)] = 72338,
  [SMALL_STATE(2294)] = 72370,
  [SMALL_STATE(2295)] = 72402,
  [SMALL_STATE(2296)] = 72434,
  [SMALL_STATE(2297)] = 72466,
  [SMALL_STATE(2298)] = 72498,
  [SMALL_STATE(2299)] = 72530,
  [SMALL_STATE(2300)] = 72558,
  [SMALL_STATE(2301)] = 72584,
  [SMALL_STATE(2302)] = 72616,
  [SMALL_STATE(2303)] = 72646,
  [SMALL_STATE(2304)] = 72678,
  [SMALL_STATE(2305)] = 72700,
  [SMALL_STATE(2306)] = 72732,
  [SMALL_STATE(2307)] = 72764,
  [SMALL_STATE(2308)] = 72790,
  [SMALL_STATE(2309)] = 72812,
  [SMALL_STATE(2310)] = 72842,
  [SMALL_STATE(2311)] = 72871,
  [SMALL_STATE(2312)] = 72900,
  [SMALL_STATE(2313)] = 72929,
  [SMALL_STATE(2314)] = 72958,
  [SMALL_STATE(2315)] = 72987,
  [SMALL_STATE(2316)] = 73010,
  [SMALL_STATE(2317)] = 73039,
  [SMALL_STATE(2318)] = 73068,
  [SMALL_STATE(2319)] = 73097,
  [SMALL_STATE(2320)] = 73126,
  [SMALL_STATE(2321)] = 73155,
  [SMALL_STATE(2322)] = 73176,
  [SMALL_STATE(2323)] = 73205,
  [SMALL_STATE(2324)] = 73234,
  [SMALL_STATE(2325)] = 73263,
  [SMALL_STATE(2326)] = 73292,
  [SMALL_STATE(2327)] = 73321,
  [SMALL_STATE(2328)] = 73350,
  [SMALL_STATE(2329)] = 73379,
  [SMALL_STATE(2330)] = 73402,
  [SMALL_STATE(2331)] = 73425,
  [SMALL_STATE(2332)] = 73454,
  [SMALL_STATE(2333)] = 73483,
  [SMALL_STATE(2334)] = 73506,
  [SMALL_STATE(2335)] = 73533,
  [SMALL_STATE(2336)] = 73560,
  [SMALL_STATE(2337)] = 73583,
  [SMALL_STATE(2338)] = 73612,
  [SMALL_STATE(2339)] = 73641,
  [SMALL_STATE(2340)] = 73670,
  [SMALL_STATE(2341)] = 73699,
  [SMALL_STATE(2342)] = 73728,
  [SMALL_STATE(2343)] = 73755,
  [SMALL_STATE(2344)] = 73784,
  [SMALL_STATE(2345)] = 73813,
  [SMALL_STATE(2346)] = 73842,
  [SMALL_STATE(2347)] = 73871,
  [SMALL_STATE(2348)] = 73890,
  [SMALL_STATE(2349)] = 73919,
  [SMALL_STATE(2350)] = 73946,
  [SMALL_STATE(2351)] = 73975,
  [SMALL_STATE(2352)] = 74004,
  [SMALL_STATE(2353)] = 74033,
  [SMALL_STATE(2354)] = 74062,
  [SMALL_STATE(2355)] = 74091,
  [SMALL_STATE(2356)] = 74120,
  [SMALL_STATE(2357)] = 74149,
  [SMALL_STATE(2358)] = 74178,
  [SMALL_STATE(2359)] = 74201,
  [SMALL_STATE(2360)] = 74230,
  [SMALL_STATE(2361)] = 74259,
  [SMALL_STATE(2362)] = 74288,
  [SMALL_STATE(2363)] = 74309,
  [SMALL_STATE(2364)] = 74338,
  [SMALL_STATE(2365)] = 74359,
  [SMALL_STATE(2366)] = 74388,
  [SMALL_STATE(2367)] = 74411,
  [SMALL_STATE(2368)] = 74434,
  [SMALL_STATE(2369)] = 74463,
  [SMALL_STATE(2370)] = 74492,
  [SMALL_STATE(2371)] = 74521,
  [SMALL_STATE(2372)] = 74550,
  [SMALL_STATE(2373)] = 74571,
  [SMALL_STATE(2374)] = 74600,
  [SMALL_STATE(2375)] = 74629,
  [SMALL_STATE(2376)] = 74658,
  [SMALL_STATE(2377)] = 74687,
  [SMALL_STATE(2378)] = 74712,
  [SMALL_STATE(2379)] = 74739,
  [SMALL_STATE(2380)] = 74768,
  [SMALL_STATE(2381)] = 74797,
  [SMALL_STATE(2382)] = 74822,
  [SMALL_STATE(2383)] = 74851,
  [SMALL_STATE(2384)] = 74880,
  [SMALL_STATE(2385)] = 74905,
  [SMALL_STATE(2386)] = 74934,
  [SMALL_STATE(2387)] = 74963,
  [SMALL_STATE(2388)] = 74982,
  [SMALL_STATE(2389)] = 75011,
  [SMALL_STATE(2390)] = 75032,
  [SMALL_STATE(2391)] = 75061,
  [SMALL_STATE(2392)] = 75082,
  [SMALL_STATE(2393)] = 75103,
  [SMALL_STATE(2394)] = 75124,
  [SMALL_STATE(2395)] = 75153,
  [SMALL_STATE(2396)] = 75180,
  [SMALL_STATE(2397)] = 75209,
  [SMALL_STATE(2398)] = 75232,
  [SMALL_STATE(2399)] = 75255,
  [SMALL_STATE(2400)] = 75284,
  [SMALL_STATE(2401)] = 75307,
  [SMALL_STATE(2402)] = 75336,
  [SMALL_STATE(2403)] = 75359,
  [SMALL_STATE(2404)] = 75388,
  [SMALL_STATE(2405)] = 75411,
  [SMALL_STATE(2406)] = 75434,
  [SMALL_STATE(2407)] = 75459,
  [SMALL_STATE(2408)] = 75488,
  [SMALL_STATE(2409)] = 75517,
  [SMALL_STATE(2410)] = 75543,
  [SMALL_STATE(2411)] = 75563,
  [SMALL_STATE(2412)] = 75589,
  [SMALL_STATE(2413)] = 75615,
  [SMALL_STATE(2414)] = 75641,
  [SMALL_STATE(2415)] = 75665,
  [SMALL_STATE(2416)] = 75691,
  [SMALL_STATE(2417)] = 75717,
  [SMALL_STATE(2418)] = 75743,
  [SMALL_STATE(2419)] = 75769,
  [SMALL_STATE(2420)] = 75795,
  [SMALL_STATE(2421)] = 75819,
  [SMALL_STATE(2422)] = 75845,
  [SMALL_STATE(2423)] = 75871,
  [SMALL_STATE(2424)] = 75897,
  [SMALL_STATE(2425)] = 75923,
  [SMALL_STATE(2426)] = 75949,
  [SMALL_STATE(2427)] = 75973,
  [SMALL_STATE(2428)] = 75999,
  [SMALL_STATE(2429)] = 76025,
  [SMALL_STATE(2430)] = 76049,
  [SMALL_STATE(2431)] = 76071,
  [SMALL_STATE(2432)] = 76097,
  [SMALL_STATE(2433)] = 76115,
  [SMALL_STATE(2434)] = 76139,
  [SMALL_STATE(2435)] = 76165,
  [SMALL_STATE(2436)] = 76187,
  [SMALL_STATE(2437)] = 76213,
  [SMALL_STATE(2438)] = 76239,
  [SMALL_STATE(2439)] = 76265,
  [SMALL_STATE(2440)] = 76289,
  [SMALL_STATE(2441)] = 76311,
  [SMALL_STATE(2442)] = 76337,
  [SMALL_STATE(2443)] = 76363,
  [SMALL_STATE(2444)] = 76389,
  [SMALL_STATE(2445)] = 76413,
  [SMALL_STATE(2446)] = 76431,
  [SMALL_STATE(2447)] = 76453,
  [SMALL_STATE(2448)] = 76475,
  [SMALL_STATE(2449)] = 76501,
  [SMALL_STATE(2450)] = 76527,
  [SMALL_STATE(2451)] = 76553,
  [SMALL_STATE(2452)] = 76579,
  [SMALL_STATE(2453)] = 76597,
  [SMALL_STATE(2454)] = 76615,
  [SMALL_STATE(2455)] = 76641,
  [SMALL_STATE(2456)] = 76667,
  [SMALL_STATE(2457)] = 76693,
  [SMALL_STATE(2458)] = 76719,
  [SMALL_STATE(2459)] = 76745,
  [SMALL_STATE(2460)] = 76771,
  [SMALL_STATE(2461)] = 76789,
  [SMALL_STATE(2462)] = 76815,
  [SMALL_STATE(2463)] = 76833,
  [SMALL_STATE(2464)] = 76851,
  [SMALL_STATE(2465)] = 76869,
  [SMALL_STATE(2466)] = 76895,
  [SMALL_STATE(2467)] = 76921,
  [SMALL_STATE(2468)] = 76939,
  [SMALL_STATE(2469)] = 76957,
  [SMALL_STATE(2470)] = 76975,
  [SMALL_STATE(2471)] = 77001,
  [SMALL_STATE(2472)] = 77019,
  [SMALL_STATE(2473)] = 77045,
  [SMALL_STATE(2474)] = 77063,
  [SMALL_STATE(2475)] = 77081,
  [SMALL_STATE(2476)] = 77105,
  [SMALL_STATE(2477)] = 77129,
  [SMALL_STATE(2478)] = 77151,
  [SMALL_STATE(2479)] = 77177,
  [SMALL_STATE(2480)] = 77195,
  [SMALL_STATE(2481)] = 77213,
  [SMALL_STATE(2482)] = 77239,
  [SMALL_STATE(2483)] = 77257,
  [SMALL_STATE(2484)] = 77275,
  [SMALL_STATE(2485)] = 77301,
  [SMALL_STATE(2486)] = 77327,
  [SMALL_STATE(2487)] = 77353,
  [SMALL_STATE(2488)] = 77379,
  [SMALL_STATE(2489)] = 77399,
  [SMALL_STATE(2490)] = 77421,
  [SMALL_STATE(2491)] = 77447,
  [SMALL_STATE(2492)] = 77467,
  [SMALL_STATE(2493)] = 77489,
  [SMALL_STATE(2494)] = 77515,
  [SMALL_STATE(2495)] = 77541,
  [SMALL_STATE(2496)] = 77565,
  [SMALL_STATE(2497)] = 77583,
  [SMALL_STATE(2498)] = 77601,
  [SMALL_STATE(2499)] = 77619,
  [SMALL_STATE(2500)] = 77643,
  [SMALL_STATE(2501)] = 77665,
  [SMALL_STATE(2502)] = 77689,
  [SMALL_STATE(2503)] = 77713,
  [SMALL_STATE(2504)] = 77739,
  [SMALL_STATE(2505)] = 77761,
  [SMALL_STATE(2506)] = 77779,
  [SMALL_STATE(2507)] = 77797,
  [SMALL_STATE(2508)] = 77823,
  [SMALL_STATE(2509)] = 77841,
  [SMALL_STATE(2510)] = 77865,
  [SMALL_STATE(2511)] = 77889,
  [SMALL_STATE(2512)] = 77915,
  [SMALL_STATE(2513)] = 77941,
  [SMALL_STATE(2514)] = 77959,
  [SMALL_STATE(2515)] = 77982,
  [SMALL_STATE(2516)] = 78003,
  [SMALL_STATE(2517)] = 78024,
  [SMALL_STATE(2518)] = 78047,
  [SMALL_STATE(2519)] = 78066,
  [SMALL_STATE(2520)] = 78089,
  [SMALL_STATE(2521)] = 78110,
  [SMALL_STATE(2522)] = 78133,
  [SMALL_STATE(2523)] = 78156,
  [SMALL_STATE(2524)] = 78179,
  [SMALL_STATE(2525)] = 78200,
  [SMALL_STATE(2526)] = 78223,
  [SMALL_STATE(2527)] = 78246,
  [SMALL_STATE(2528)] = 78269,
  [SMALL_STATE(2529)] = 78290,
  [SMALL_STATE(2530)] = 78311,
  [SMALL_STATE(2531)] = 78330,
  [SMALL_STATE(2532)] = 78353,
  [SMALL_STATE(2533)] = 78372,
  [SMALL_STATE(2534)] = 78389,
  [SMALL_STATE(2535)] = 78412,
  [SMALL_STATE(2536)] = 78435,
  [SMALL_STATE(2537)] = 78458,
  [SMALL_STATE(2538)] = 78479,
  [SMALL_STATE(2539)] = 78500,
  [SMALL_STATE(2540)] = 78523,
  [SMALL_STATE(2541)] = 78546,
  [SMALL_STATE(2542)] = 78569,
  [SMALL_STATE(2543)] = 78592,
  [SMALL_STATE(2544)] = 78615,
  [SMALL_STATE(2545)] = 78638,
  [SMALL_STATE(2546)] = 78661,
  [SMALL_STATE(2547)] = 78682,
  [SMALL_STATE(2548)] = 78705,
  [SMALL_STATE(2549)] = 78728,
  [SMALL_STATE(2550)] = 78751,
  [SMALL_STATE(2551)] = 78774,
  [SMALL_STATE(2552)] = 78797,
  [SMALL_STATE(2553)] = 78820,
  [SMALL_STATE(2554)] = 78843,
  [SMALL_STATE(2555)] = 78866,
  [SMALL_STATE(2556)] = 78889,
  [SMALL_STATE(2557)] = 78912,
  [SMALL_STATE(2558)] = 78935,
  [SMALL_STATE(2559)] = 78958,
  [SMALL_STATE(2560)] = 78981,
  [SMALL_STATE(2561)] = 79004,
  [SMALL_STATE(2562)] = 79027,
  [SMALL_STATE(2563)] = 79050,
  [SMALL_STATE(2564)] = 79073,
  [SMALL_STATE(2565)] = 79094,
  [SMALL_STATE(2566)] = 79117,
  [SMALL_STATE(2567)] = 79140,
  [SMALL_STATE(2568)] = 79163,
  [SMALL_STATE(2569)] = 79186,
  [SMALL_STATE(2570)] = 79209,
  [SMALL_STATE(2571)] = 79232,
  [SMALL_STATE(2572)] = 79255,
  [SMALL_STATE(2573)] = 79276,
  [SMALL_STATE(2574)] = 79299,
  [SMALL_STATE(2575)] = 79322,
  [SMALL_STATE(2576)] = 79345,
  [SMALL_STATE(2577)] = 79368,
  [SMALL_STATE(2578)] = 79391,
  [SMALL_STATE(2579)] = 79410,
  [SMALL_STATE(2580)] = 79433,
  [SMALL_STATE(2581)] = 79450,
  [SMALL_STATE(2582)] = 79473,
  [SMALL_STATE(2583)] = 79496,
  [SMALL_STATE(2584)] = 79519,
  [SMALL_STATE(2585)] = 79542,
  [SMALL_STATE(2586)] = 79565,
  [SMALL_STATE(2587)] = 79588,
  [SMALL_STATE(2588)] = 79611,
  [SMALL_STATE(2589)] = 79634,
  [SMALL_STATE(2590)] = 79657,
  [SMALL_STATE(2591)] = 79680,
  [SMALL_STATE(2592)] = 79703,
  [SMALL_STATE(2593)] = 79726,
  [SMALL_STATE(2594)] = 79749,
  [SMALL_STATE(2595)] = 79768,
  [SMALL_STATE(2596)] = 79787,
  [SMALL_STATE(2597)] = 79810,
  [SMALL_STATE(2598)] = 79833,
  [SMALL_STATE(2599)] = 79854,
  [SMALL_STATE(2600)] = 79877,
  [SMALL_STATE(2601)] = 79900,
  [SMALL_STATE(2602)] = 79923,
  [SMALL_STATE(2603)] = 79946,
  [SMALL_STATE(2604)] = 79969,
  [SMALL_STATE(2605)] = 79992,
  [SMALL_STATE(2606)] = 80015,
  [SMALL_STATE(2607)] = 80036,
  [SMALL_STATE(2608)] = 80059,
  [SMALL_STATE(2609)] = 80082,
  [SMALL_STATE(2610)] = 80105,
  [SMALL_STATE(2611)] = 80128,
  [SMALL_STATE(2612)] = 80151,
  [SMALL_STATE(2613)] = 80170,
  [SMALL_STATE(2614)] = 80193,
  [SMALL_STATE(2615)] = 80212,
  [SMALL_STATE(2616)] = 80235,
  [SMALL_STATE(2617)] = 80258,
  [SMALL_STATE(2618)] = 80281,
  [SMALL_STATE(2619)] = 80304,
  [SMALL_STATE(2620)] = 80325,
  [SMALL_STATE(2621)] = 80348,
  [SMALL_STATE(2622)] = 80371,
  [SMALL_STATE(2623)] = 80394,
  [SMALL_STATE(2624)] = 80415,
  [SMALL_STATE(2625)] = 80438,
  [SMALL_STATE(2626)] = 80461,
  [SMALL_STATE(2627)] = 80484,
  [SMALL_STATE(2628)] = 80507,
  [SMALL_STATE(2629)] = 80526,
  [SMALL_STATE(2630)] = 80549,
  [SMALL_STATE(2631)] = 80570,
  [SMALL_STATE(2632)] = 80591,
  [SMALL_STATE(2633)] = 80610,
  [SMALL_STATE(2634)] = 80633,
  [SMALL_STATE(2635)] = 80656,
  [SMALL_STATE(2636)] = 80677,
  [SMALL_STATE(2637)] = 80694,
  [SMALL_STATE(2638)] = 80717,
  [SMALL_STATE(2639)] = 80736,
  [SMALL_STATE(2640)] = 80759,
  [SMALL_STATE(2641)] = 80782,
  [SMALL_STATE(2642)] = 80801,
  [SMALL_STATE(2643)] = 80824,
  [SMALL_STATE(2644)] = 80847,
  [SMALL_STATE(2645)] = 80870,
  [SMALL_STATE(2646)] = 80893,
  [SMALL_STATE(2647)] = 80916,
  [SMALL_STATE(2648)] = 80939,
  [SMALL_STATE(2649)] = 80962,
  [SMALL_STATE(2650)] = 80985,
  [SMALL_STATE(2651)] = 81008,
  [SMALL_STATE(2652)] = 81031,
  [SMALL_STATE(2653)] = 81054,
  [SMALL_STATE(2654)] = 81071,
  [SMALL_STATE(2655)] = 81094,
  [SMALL_STATE(2656)] = 81117,
  [SMALL_STATE(2657)] = 81140,
  [SMALL_STATE(2658)] = 81161,
  [SMALL_STATE(2659)] = 81184,
  [SMALL_STATE(2660)] = 81207,
  [SMALL_STATE(2661)] = 81230,
  [SMALL_STATE(2662)] = 81249,
  [SMALL_STATE(2663)] = 81268,
  [SMALL_STATE(2664)] = 81291,
  [SMALL_STATE(2665)] = 81308,
  [SMALL_STATE(2666)] = 81329,
  [SMALL_STATE(2667)] = 81352,
  [SMALL_STATE(2668)] = 81373,
  [SMALL_STATE(2669)] = 81396,
  [SMALL_STATE(2670)] = 81419,
  [SMALL_STATE(2671)] = 81442,
  [SMALL_STATE(2672)] = 81465,
  [SMALL_STATE(2673)] = 81488,
  [SMALL_STATE(2674)] = 81507,
  [SMALL_STATE(2675)] = 81530,
  [SMALL_STATE(2676)] = 81547,
  [SMALL_STATE(2677)] = 81570,
  [SMALL_STATE(2678)] = 81593,
  [SMALL_STATE(2679)] = 81616,
  [SMALL_STATE(2680)] = 81639,
  [SMALL_STATE(2681)] = 81662,
  [SMALL_STATE(2682)] = 81685,
  [SMALL_STATE(2683)] = 81708,
  [SMALL_STATE(2684)] = 81727,
  [SMALL_STATE(2685)] = 81750,
  [SMALL_STATE(2686)] = 81769,
  [SMALL_STATE(2687)] = 81792,
  [SMALL_STATE(2688)] = 81815,
  [SMALL_STATE(2689)] = 81838,
  [SMALL_STATE(2690)] = 81857,
  [SMALL_STATE(2691)] = 81876,
  [SMALL_STATE(2692)] = 81895,
  [SMALL_STATE(2693)] = 81918,
  [SMALL_STATE(2694)] = 81941,
  [SMALL_STATE(2695)] = 81964,
  [SMALL_STATE(2696)] = 81987,
  [SMALL_STATE(2697)] = 82010,
  [SMALL_STATE(2698)] = 82033,
  [SMALL_STATE(2699)] = 82056,
  [SMALL_STATE(2700)] = 82075,
  [SMALL_STATE(2701)] = 82098,
  [SMALL_STATE(2702)] = 82121,
  [SMALL_STATE(2703)] = 82142,
  [SMALL_STATE(2704)] = 82165,
  [SMALL_STATE(2705)] = 82188,
  [SMALL_STATE(2706)] = 82211,
  [SMALL_STATE(2707)] = 82230,
  [SMALL_STATE(2708)] = 82253,
  [SMALL_STATE(2709)] = 82276,
  [SMALL_STATE(2710)] = 82295,
  [SMALL_STATE(2711)] = 82318,
  [SMALL_STATE(2712)] = 82341,
  [SMALL_STATE(2713)] = 82360,
  [SMALL_STATE(2714)] = 82381,
  [SMALL_STATE(2715)] = 82404,
  [SMALL_STATE(2716)] = 82427,
  [SMALL_STATE(2717)] = 82450,
  [SMALL_STATE(2718)] = 82471,
  [SMALL_STATE(2719)] = 82490,
  [SMALL_STATE(2720)] = 82511,
  [SMALL_STATE(2721)] = 82534,
  [SMALL_STATE(2722)] = 82555,
  [SMALL_STATE(2723)] = 82576,
  [SMALL_STATE(2724)] = 82599,
  [SMALL_STATE(2725)] = 82622,
  [SMALL_STATE(2726)] = 82639,
  [SMALL_STATE(2727)] = 82656,
  [SMALL_STATE(2728)] = 82679,
  [SMALL_STATE(2729)] = 82702,
  [SMALL_STATE(2730)] = 82723,
  [SMALL_STATE(2731)] = 82746,
  [SMALL_STATE(2732)] = 82769,
  [SMALL_STATE(2733)] = 82792,
  [SMALL_STATE(2734)] = 82815,
  [SMALL_STATE(2735)] = 82834,
  [SMALL_STATE(2736)] = 82853,
  [SMALL_STATE(2737)] = 82876,
  [SMALL_STATE(2738)] = 82899,
  [SMALL_STATE(2739)] = 82922,
  [SMALL_STATE(2740)] = 82945,
  [SMALL_STATE(2741)] = 82968,
  [SMALL_STATE(2742)] = 82991,
  [SMALL_STATE(2743)] = 83014,
  [SMALL_STATE(2744)] = 83037,
  [SMALL_STATE(2745)] = 83060,
  [SMALL_STATE(2746)] = 83083,
  [SMALL_STATE(2747)] = 83106,
  [SMALL_STATE(2748)] = 83129,
  [SMALL_STATE(2749)] = 83152,
  [SMALL_STATE(2750)] = 83175,
  [SMALL_STATE(2751)] = 83198,
  [SMALL_STATE(2752)] = 83221,
  [SMALL_STATE(2753)] = 83244,
  [SMALL_STATE(2754)] = 83263,
  [SMALL_STATE(2755)] = 83286,
  [SMALL_STATE(2756)] = 83309,
  [SMALL_STATE(2757)] = 83332,
  [SMALL_STATE(2758)] = 83348,
  [SMALL_STATE(2759)] = 83368,
  [SMALL_STATE(2760)] = 83388,
  [SMALL_STATE(2761)] = 83408,
  [SMALL_STATE(2762)] = 83428,
  [SMALL_STATE(2763)] = 83448,
  [SMALL_STATE(2764)] = 83464,
  [SMALL_STATE(2765)] = 83484,
  [SMALL_STATE(2766)] = 83500,
  [SMALL_STATE(2767)] = 83520,
  [SMALL_STATE(2768)] = 83538,
  [SMALL_STATE(2769)] = 83558,
  [SMALL_STATE(2770)] = 83576,
  [SMALL_STATE(2771)] = 83592,
  [SMALL_STATE(2772)] = 83612,
  [SMALL_STATE(2773)] = 83628,
  [SMALL_STATE(2774)] = 83648,
  [SMALL_STATE(2775)] = 83668,
  [SMALL_STATE(2776)] = 83688,
  [SMALL_STATE(2777)] = 83708,
  [SMALL_STATE(2778)] = 83728,
  [SMALL_STATE(2779)] = 83744,
  [SMALL_STATE(2780)] = 83764,
  [SMALL_STATE(2781)] = 83784,
  [SMALL_STATE(2782)] = 83802,
  [SMALL_STATE(2783)] = 83822,
  [SMALL_STATE(2784)] = 83840,
  [SMALL_STATE(2785)] = 83860,
  [SMALL_STATE(2786)] = 83876,
  [SMALL_STATE(2787)] = 83892,
  [SMALL_STATE(2788)] = 83912,
  [SMALL_STATE(2789)] = 83930,
  [SMALL_STATE(2790)] = 83950,
  [SMALL_STATE(2791)] = 83970,
  [SMALL_STATE(2792)] = 83986,
  [SMALL_STATE(2793)] = 84002,
  [SMALL_STATE(2794)] = 84022,
  [SMALL_STATE(2795)] = 84038,
  [SMALL_STATE(2796)] = 84058,
  [SMALL_STATE(2797)] = 84078,
  [SMALL_STATE(2798)] = 84098,
  [SMALL_STATE(2799)] = 84118,
  [SMALL_STATE(2800)] = 84138,
  [SMALL_STATE(2801)] = 84158,
  [SMALL_STATE(2802)] = 84178,
  [SMALL_STATE(2803)] = 84198,
  [SMALL_STATE(2804)] = 84218,
  [SMALL_STATE(2805)] = 84234,
  [SMALL_STATE(2806)] = 84254,
  [SMALL_STATE(2807)] = 84274,
  [SMALL_STATE(2808)] = 84292,
  [SMALL_STATE(2809)] = 84312,
  [SMALL_STATE(2810)] = 84328,
  [SMALL_STATE(2811)] = 84348,
  [SMALL_STATE(2812)] = 84368,
  [SMALL_STATE(2813)] = 84388,
  [SMALL_STATE(2814)] = 84408,
  [SMALL_STATE(2815)] = 84424,
  [SMALL_STATE(2816)] = 84440,
  [SMALL_STATE(2817)] = 84456,
  [SMALL_STATE(2818)] = 84472,
  [SMALL_STATE(2819)] = 84488,
  [SMALL_STATE(2820)] = 84504,
  [SMALL_STATE(2821)] = 84520,
  [SMALL_STATE(2822)] = 84540,
  [SMALL_STATE(2823)] = 84560,
  [SMALL_STATE(2824)] = 84580,
  [SMALL_STATE(2825)] = 84600,
  [SMALL_STATE(2826)] = 84620,
  [SMALL_STATE(2827)] = 84640,
  [SMALL_STATE(2828)] = 84660,
  [SMALL_STATE(2829)] = 84680,
  [SMALL_STATE(2830)] = 84700,
  [SMALL_STATE(2831)] = 84720,
  [SMALL_STATE(2832)] = 84740,
  [SMALL_STATE(2833)] = 84756,
  [SMALL_STATE(2834)] = 84776,
  [SMALL_STATE(2835)] = 84796,
  [SMALL_STATE(2836)] = 84816,
  [SMALL_STATE(2837)] = 84836,
  [SMALL_STATE(2838)] = 84856,
  [SMALL_STATE(2839)] = 84876,
  [SMALL_STATE(2840)] = 84896,
  [SMALL_STATE(2841)] = 84914,
  [SMALL_STATE(2842)] = 84934,
  [SMALL_STATE(2843)] = 84954,
  [SMALL_STATE(2844)] = 84974,
  [SMALL_STATE(2845)] = 84994,
  [SMALL_STATE(2846)] = 85014,
  [SMALL_STATE(2847)] = 85034,
  [SMALL_STATE(2848)] = 85054,
  [SMALL_STATE(2849)] = 85074,
  [SMALL_STATE(2850)] = 85094,
  [SMALL_STATE(2851)] = 85114,
  [SMALL_STATE(2852)] = 85134,
  [SMALL_STATE(2853)] = 85150,
  [SMALL_STATE(2854)] = 85170,
  [SMALL_STATE(2855)] = 85190,
  [SMALL_STATE(2856)] = 85206,
  [SMALL_STATE(2857)] = 85224,
  [SMALL_STATE(2858)] = 85244,
  [SMALL_STATE(2859)] = 85264,
  [SMALL_STATE(2860)] = 85282,
  [SMALL_STATE(2861)] = 85302,
  [SMALL_STATE(2862)] = 85322,
  [SMALL_STATE(2863)] = 85338,
  [SMALL_STATE(2864)] = 85358,
  [SMALL_STATE(2865)] = 85374,
  [SMALL_STATE(2866)] = 85394,
  [SMALL_STATE(2867)] = 85414,
  [SMALL_STATE(2868)] = 85434,
  [SMALL_STATE(2869)] = 85454,
  [SMALL_STATE(2870)] = 85474,
  [SMALL_STATE(2871)] = 85494,
  [SMALL_STATE(2872)] = 85514,
  [SMALL_STATE(2873)] = 85530,
  [SMALL_STATE(2874)] = 85546,
  [SMALL_STATE(2875)] = 85566,
  [SMALL_STATE(2876)] = 85586,
  [SMALL_STATE(2877)] = 85602,
  [SMALL_STATE(2878)] = 85618,
  [SMALL_STATE(2879)] = 85638,
  [SMALL_STATE(2880)] = 85658,
  [SMALL_STATE(2881)] = 85678,
  [SMALL_STATE(2882)] = 85698,
  [SMALL_STATE(2883)] = 85714,
  [SMALL_STATE(2884)] = 85732,
  [SMALL_STATE(2885)] = 85748,
  [SMALL_STATE(2886)] = 85764,
  [SMALL_STATE(2887)] = 85780,
  [SMALL_STATE(2888)] = 85798,
  [SMALL_STATE(2889)] = 85818,
  [SMALL_STATE(2890)] = 85838,
  [SMALL_STATE(2891)] = 85858,
  [SMALL_STATE(2892)] = 85878,
  [SMALL_STATE(2893)] = 85898,
  [SMALL_STATE(2894)] = 85918,
  [SMALL_STATE(2895)] = 85938,
  [SMALL_STATE(2896)] = 85958,
  [SMALL_STATE(2897)] = 85978,
  [SMALL_STATE(2898)] = 85998,
  [SMALL_STATE(2899)] = 86018,
  [SMALL_STATE(2900)] = 86038,
  [SMALL_STATE(2901)] = 86058,
  [SMALL_STATE(2902)] = 86078,
  [SMALL_STATE(2903)] = 86096,
  [SMALL_STATE(2904)] = 86116,
  [SMALL_STATE(2905)] = 86136,
  [SMALL_STATE(2906)] = 86156,
  [SMALL_STATE(2907)] = 86176,
  [SMALL_STATE(2908)] = 86196,
  [SMALL_STATE(2909)] = 86216,
  [SMALL_STATE(2910)] = 86234,
  [SMALL_STATE(2911)] = 86250,
  [SMALL_STATE(2912)] = 86270,
  [SMALL_STATE(2913)] = 86290,
  [SMALL_STATE(2914)] = 86308,
  [SMALL_STATE(2915)] = 86328,
  [SMALL_STATE(2916)] = 86344,
  [SMALL_STATE(2917)] = 86362,
  [SMALL_STATE(2918)] = 86382,
  [SMALL_STATE(2919)] = 86398,
  [SMALL_STATE(2920)] = 86414,
  [SMALL_STATE(2921)] = 86434,
  [SMALL_STATE(2922)] = 86454,
  [SMALL_STATE(2923)] = 86474,
  [SMALL_STATE(2924)] = 86494,
  [SMALL_STATE(2925)] = 86514,
  [SMALL_STATE(2926)] = 86534,
  [SMALL_STATE(2927)] = 86554,
  [SMALL_STATE(2928)] = 86574,
  [SMALL_STATE(2929)] = 86594,
  [SMALL_STATE(2930)] = 86612,
  [SMALL_STATE(2931)] = 86630,
  [SMALL_STATE(2932)] = 86650,
  [SMALL_STATE(2933)] = 86670,
  [SMALL_STATE(2934)] = 86688,
  [SMALL_STATE(2935)] = 86708,
  [SMALL_STATE(2936)] = 86724,
  [SMALL_STATE(2937)] = 86740,
  [SMALL_STATE(2938)] = 86760,
  [SMALL_STATE(2939)] = 86780,
  [SMALL_STATE(2940)] = 86800,
  [SMALL_STATE(2941)] = 86820,
  [SMALL_STATE(2942)] = 86838,
  [SMALL_STATE(2943)] = 86854,
  [SMALL_STATE(2944)] = 86874,
  [SMALL_STATE(2945)] = 86894,
  [SMALL_STATE(2946)] = 86912,
  [SMALL_STATE(2947)] = 86932,
  [SMALL_STATE(2948)] = 86952,
  [SMALL_STATE(2949)] = 86972,
  [SMALL_STATE(2950)] = 86992,
  [SMALL_STATE(2951)] = 87012,
  [SMALL_STATE(2952)] = 87032,
  [SMALL_STATE(2953)] = 87052,
  [SMALL_STATE(2954)] = 87072,
  [SMALL_STATE(2955)] = 87088,
  [SMALL_STATE(2956)] = 87108,
  [SMALL_STATE(2957)] = 87128,
  [SMALL_STATE(2958)] = 87148,
  [SMALL_STATE(2959)] = 87168,
  [SMALL_STATE(2960)] = 87188,
  [SMALL_STATE(2961)] = 87208,
  [SMALL_STATE(2962)] = 87228,
  [SMALL_STATE(2963)] = 87246,
  [SMALL_STATE(2964)] = 87266,
  [SMALL_STATE(2965)] = 87284,
  [SMALL_STATE(2966)] = 87304,
  [SMALL_STATE(2967)] = 87324,
  [SMALL_STATE(2968)] = 87344,
  [SMALL_STATE(2969)] = 87360,
  [SMALL_STATE(2970)] = 87380,
  [SMALL_STATE(2971)] = 87400,
  [SMALL_STATE(2972)] = 87420,
  [SMALL_STATE(2973)] = 87436,
  [SMALL_STATE(2974)] = 87456,
  [SMALL_STATE(2975)] = 87476,
  [SMALL_STATE(2976)] = 87492,
  [SMALL_STATE(2977)] = 87512,
  [SMALL_STATE(2978)] = 87532,
  [SMALL_STATE(2979)] = 87548,
  [SMALL_STATE(2980)] = 87568,
  [SMALL_STATE(2981)] = 87586,
  [SMALL_STATE(2982)] = 87606,
  [SMALL_STATE(2983)] = 87626,
  [SMALL_STATE(2984)] = 87646,
  [SMALL_STATE(2985)] = 87662,
  [SMALL_STATE(2986)] = 87678,
  [SMALL_STATE(2987)] = 87694,
  [SMALL_STATE(2988)] = 87714,
  [SMALL_STATE(2989)] = 87730,
  [SMALL_STATE(2990)] = 87748,
  [SMALL_STATE(2991)] = 87766,
  [SMALL_STATE(2992)] = 87786,
  [SMALL_STATE(2993)] = 87804,
  [SMALL_STATE(2994)] = 87824,
  [SMALL_STATE(2995)] = 87844,
  [SMALL_STATE(2996)] = 87864,
  [SMALL_STATE(2997)] = 87880,
  [SMALL_STATE(2998)] = 87898,
  [SMALL_STATE(2999)] = 87918,
  [SMALL_STATE(3000)] = 87938,
  [SMALL_STATE(3001)] = 87956,
  [SMALL_STATE(3002)] = 87974,
  [SMALL_STATE(3003)] = 87992,
  [SMALL_STATE(3004)] = 88010,
  [SMALL_STATE(3005)] = 88030,
  [SMALL_STATE(3006)] = 88046,
  [SMALL_STATE(3007)] = 88062,
  [SMALL_STATE(3008)] = 88080,
  [SMALL_STATE(3009)] = 88096,
  [SMALL_STATE(3010)] = 88116,
  [SMALL_STATE(3011)] = 88136,
  [SMALL_STATE(3012)] = 88152,
  [SMALL_STATE(3013)] = 88172,
  [SMALL_STATE(3014)] = 88192,
  [SMALL_STATE(3015)] = 88210,
  [SMALL_STATE(3016)] = 88228,
  [SMALL_STATE(3017)] = 88248,
  [SMALL_STATE(3018)] = 88268,
  [SMALL_STATE(3019)] = 88288,
  [SMALL_STATE(3020)] = 88308,
  [SMALL_STATE(3021)] = 88326,
  [SMALL_STATE(3022)] = 88342,
  [SMALL_STATE(3023)] = 88362,
  [SMALL_STATE(3024)] = 88380,
  [SMALL_STATE(3025)] = 88400,
  [SMALL_STATE(3026)] = 88420,
  [SMALL_STATE(3027)] = 88440,
  [SMALL_STATE(3028)] = 88460,
  [SMALL_STATE(3029)] = 88476,
  [SMALL_STATE(3030)] = 88494,
  [SMALL_STATE(3031)] = 88514,
  [SMALL_STATE(3032)] = 88534,
  [SMALL_STATE(3033)] = 88554,
  [SMALL_STATE(3034)] = 88574,
  [SMALL_STATE(3035)] = 88594,
  [SMALL_STATE(3036)] = 88612,
  [SMALL_STATE(3037)] = 88632,
  [SMALL_STATE(3038)] = 88650,
  [SMALL_STATE(3039)] = 88670,
  [SMALL_STATE(3040)] = 88688,
  [SMALL_STATE(3041)] = 88708,
  [SMALL_STATE(3042)] = 88728,
  [SMALL_STATE(3043)] = 88748,
  [SMALL_STATE(3044)] = 88766,
  [SMALL_STATE(3045)] = 88782,
  [SMALL_STATE(3046)] = 88802,
  [SMALL_STATE(3047)] = 88822,
  [SMALL_STATE(3048)] = 88842,
  [SMALL_STATE(3049)] = 88862,
  [SMALL_STATE(3050)] = 88882,
  [SMALL_STATE(3051)] = 88902,
  [SMALL_STATE(3052)] = 88920,
  [SMALL_STATE(3053)] = 88940,
  [SMALL_STATE(3054)] = 88960,
  [SMALL_STATE(3055)] = 88980,
  [SMALL_STATE(3056)] = 89000,
  [SMALL_STATE(3057)] = 89020,
  [SMALL_STATE(3058)] = 89037,
  [SMALL_STATE(3059)] = 89052,
  [SMALL_STATE(3060)] = 89069,
  [SMALL_STATE(3061)] = 89086,
  [SMALL_STATE(3062)] = 89103,
  [SMALL_STATE(3063)] = 89120,
  [SMALL_STATE(3064)] = 89137,
  [SMALL_STATE(3065)] = 89154,
  [SMALL_STATE(3066)] = 89171,
  [SMALL_STATE(3067)] = 89188,
  [SMALL_STATE(3068)] = 89203,
  [SMALL_STATE(3069)] = 89220,
  [SMALL_STATE(3070)] = 89237,
  [SMALL_STATE(3071)] = 89254,
  [SMALL_STATE(3072)] = 89271,
  [SMALL_STATE(3073)] = 89286,
  [SMALL_STATE(3074)] = 89303,
  [SMALL_STATE(3075)] = 89320,
  [SMALL_STATE(3076)] = 89335,
  [SMALL_STATE(3077)] = 89352,
  [SMALL_STATE(3078)] = 89369,
  [SMALL_STATE(3079)] = 89384,
  [SMALL_STATE(3080)] = 89399,
  [SMALL_STATE(3081)] = 89416,
  [SMALL_STATE(3082)] = 89431,
  [SMALL_STATE(3083)] = 89448,
  [SMALL_STATE(3084)] = 89465,
  [SMALL_STATE(3085)] = 89482,
  [SMALL_STATE(3086)] = 89499,
  [SMALL_STATE(3087)] = 89514,
  [SMALL_STATE(3088)] = 89531,
  [SMALL_STATE(3089)] = 89546,
  [SMALL_STATE(3090)] = 89563,
  [SMALL_STATE(3091)] = 89580,
  [SMALL_STATE(3092)] = 89597,
  [SMALL_STATE(3093)] = 89614,
  [SMALL_STATE(3094)] = 89631,
  [SMALL_STATE(3095)] = 89646,
  [SMALL_STATE(3096)] = 89663,
  [SMALL_STATE(3097)] = 89678,
  [SMALL_STATE(3098)] = 89695,
  [SMALL_STATE(3099)] = 89712,
  [SMALL_STATE(3100)] = 89729,
  [SMALL_STATE(3101)] = 89746,
  [SMALL_STATE(3102)] = 89763,
  [SMALL_STATE(3103)] = 89780,
  [SMALL_STATE(3104)] = 89797,
  [SMALL_STATE(3105)] = 89812,
  [SMALL_STATE(3106)] = 89829,
  [SMALL_STATE(3107)] = 89846,
  [SMALL_STATE(3108)] = 89863,
  [SMALL_STATE(3109)] = 89880,
  [SMALL_STATE(3110)] = 89897,
  [SMALL_STATE(3111)] = 89914,
  [SMALL_STATE(3112)] = 89931,
  [SMALL_STATE(3113)] = 89948,
  [SMALL_STATE(3114)] = 89965,
  [SMALL_STATE(3115)] = 89982,
  [SMALL_STATE(3116)] = 89997,
  [SMALL_STATE(3117)] = 90012,
  [SMALL_STATE(3118)] = 90029,
  [SMALL_STATE(3119)] = 90046,
  [SMALL_STATE(3120)] = 90063,
  [SMALL_STATE(3121)] = 90080,
  [SMALL_STATE(3122)] = 90097,
  [SMALL_STATE(3123)] = 90112,
  [SMALL_STATE(3124)] = 90127,
  [SMALL_STATE(3125)] = 90144,
  [SMALL_STATE(3126)] = 90161,
  [SMALL_STATE(3127)] = 90178,
  [SMALL_STATE(3128)] = 90195,
  [SMALL_STATE(3129)] = 90212,
  [SMALL_STATE(3130)] = 90229,
  [SMALL_STATE(3131)] = 90246,
  [SMALL_STATE(3132)] = 90263,
  [SMALL_STATE(3133)] = 90278,
  [SMALL_STATE(3134)] = 90295,
  [SMALL_STATE(3135)] = 90312,
  [SMALL_STATE(3136)] = 90329,
  [SMALL_STATE(3137)] = 90346,
  [SMALL_STATE(3138)] = 90361,
  [SMALL_STATE(3139)] = 90376,
  [SMALL_STATE(3140)] = 90393,
  [SMALL_STATE(3141)] = 90408,
  [SMALL_STATE(3142)] = 90425,
  [SMALL_STATE(3143)] = 90442,
  [SMALL_STATE(3144)] = 90459,
  [SMALL_STATE(3145)] = 90476,
  [SMALL_STATE(3146)] = 90493,
  [SMALL_STATE(3147)] = 90510,
  [SMALL_STATE(3148)] = 90527,
  [SMALL_STATE(3149)] = 90544,
  [SMALL_STATE(3150)] = 90561,
  [SMALL_STATE(3151)] = 90578,
  [SMALL_STATE(3152)] = 90595,
  [SMALL_STATE(3153)] = 90612,
  [SMALL_STATE(3154)] = 90629,
  [SMALL_STATE(3155)] = 90646,
  [SMALL_STATE(3156)] = 90663,
  [SMALL_STATE(3157)] = 90680,
  [SMALL_STATE(3158)] = 90697,
  [SMALL_STATE(3159)] = 90712,
  [SMALL_STATE(3160)] = 90729,
  [SMALL_STATE(3161)] = 90744,
  [SMALL_STATE(3162)] = 90761,
  [SMALL_STATE(3163)] = 90778,
  [SMALL_STATE(3164)] = 90795,
  [SMALL_STATE(3165)] = 90812,
  [SMALL_STATE(3166)] = 90829,
  [SMALL_STATE(3167)] = 90844,
  [SMALL_STATE(3168)] = 90861,
  [SMALL_STATE(3169)] = 90878,
  [SMALL_STATE(3170)] = 90895,
  [SMALL_STATE(3171)] = 90912,
  [SMALL_STATE(3172)] = 90927,
  [SMALL_STATE(3173)] = 90944,
  [SMALL_STATE(3174)] = 90961,
  [SMALL_STATE(3175)] = 90978,
  [SMALL_STATE(3176)] = 90995,
  [SMALL_STATE(3177)] = 91012,
  [SMALL_STATE(3178)] = 91027,
  [SMALL_STATE(3179)] = 91044,
  [SMALL_STATE(3180)] = 91061,
  [SMALL_STATE(3181)] = 91078,
  [SMALL_STATE(3182)] = 91095,
  [SMALL_STATE(3183)] = 91112,
  [SMALL_STATE(3184)] = 91129,
  [SMALL_STATE(3185)] = 91144,
  [SMALL_STATE(3186)] = 91161,
  [SMALL_STATE(3187)] = 91178,
  [SMALL_STATE(3188)] = 91195,
  [SMALL_STATE(3189)] = 91212,
  [SMALL_STATE(3190)] = 91227,
  [SMALL_STATE(3191)] = 91244,
  [SMALL_STATE(3192)] = 91261,
  [SMALL_STATE(3193)] = 91278,
  [SMALL_STATE(3194)] = 91295,
  [SMALL_STATE(3195)] = 91312,
  [SMALL_STATE(3196)] = 91329,
  [SMALL_STATE(3197)] = 91346,
  [SMALL_STATE(3198)] = 91363,
  [SMALL_STATE(3199)] = 91380,
  [SMALL_STATE(3200)] = 91397,
  [SMALL_STATE(3201)] = 91412,
  [SMALL_STATE(3202)] = 91429,
  [SMALL_STATE(3203)] = 91446,
  [SMALL_STATE(3204)] = 91463,
  [SMALL_STATE(3205)] = 91480,
  [SMALL_STATE(3206)] = 91497,
  [SMALL_STATE(3207)] = 91514,
  [SMALL_STATE(3208)] = 91531,
  [SMALL_STATE(3209)] = 91548,
  [SMALL_STATE(3210)] = 91565,
  [SMALL_STATE(3211)] = 91582,
  [SMALL_STATE(3212)] = 91599,
  [SMALL_STATE(3213)] = 91616,
  [SMALL_STATE(3214)] = 91633,
  [SMALL_STATE(3215)] = 91648,
  [SMALL_STATE(3216)] = 91665,
  [SMALL_STATE(3217)] = 91682,
  [SMALL_STATE(3218)] = 91699,
  [SMALL_STATE(3219)] = 91714,
  [SMALL_STATE(3220)] = 91731,
  [SMALL_STATE(3221)] = 91748,
  [SMALL_STATE(3222)] = 91765,
  [SMALL_STATE(3223)] = 91780,
  [SMALL_STATE(3224)] = 91797,
  [SMALL_STATE(3225)] = 91814,
  [SMALL_STATE(3226)] = 91831,
  [SMALL_STATE(3227)] = 91848,
  [SMALL_STATE(3228)] = 91865,
  [SMALL_STATE(3229)] = 91882,
  [SMALL_STATE(3230)] = 91899,
  [SMALL_STATE(3231)] = 91914,
  [SMALL_STATE(3232)] = 91931,
  [SMALL_STATE(3233)] = 91948,
  [SMALL_STATE(3234)] = 91963,
  [SMALL_STATE(3235)] = 91980,
  [SMALL_STATE(3236)] = 91997,
  [SMALL_STATE(3237)] = 92014,
  [SMALL_STATE(3238)] = 92031,
  [SMALL_STATE(3239)] = 92048,
  [SMALL_STATE(3240)] = 92065,
  [SMALL_STATE(3241)] = 92082,
  [SMALL_STATE(3242)] = 92097,
  [SMALL_STATE(3243)] = 92114,
  [SMALL_STATE(3244)] = 92131,
  [SMALL_STATE(3245)] = 92148,
  [SMALL_STATE(3246)] = 92165,
  [SMALL_STATE(3247)] = 92180,
  [SMALL_STATE(3248)] = 92197,
  [SMALL_STATE(3249)] = 92212,
  [SMALL_STATE(3250)] = 92229,
  [SMALL_STATE(3251)] = 92244,
  [SMALL_STATE(3252)] = 92259,
  [SMALL_STATE(3253)] = 92276,
  [SMALL_STATE(3254)] = 92293,
  [SMALL_STATE(3255)] = 92308,
  [SMALL_STATE(3256)] = 92325,
  [SMALL_STATE(3257)] = 92342,
  [SMALL_STATE(3258)] = 92359,
  [SMALL_STATE(3259)] = 92374,
  [SMALL_STATE(3260)] = 92391,
  [SMALL_STATE(3261)] = 92408,
  [SMALL_STATE(3262)] = 92425,
  [SMALL_STATE(3263)] = 92442,
  [SMALL_STATE(3264)] = 92459,
  [SMALL_STATE(3265)] = 92476,
  [SMALL_STATE(3266)] = 92493,
  [SMALL_STATE(3267)] = 92510,
  [SMALL_STATE(3268)] = 92527,
  [SMALL_STATE(3269)] = 92544,
  [SMALL_STATE(3270)] = 92559,
  [SMALL_STATE(3271)] = 92574,
  [SMALL_STATE(3272)] = 92591,
  [SMALL_STATE(3273)] = 92608,
  [SMALL_STATE(3274)] = 92625,
  [SMALL_STATE(3275)] = 92642,
  [SMALL_STATE(3276)] = 92659,
  [SMALL_STATE(3277)] = 92676,
  [SMALL_STATE(3278)] = 92693,
  [SMALL_STATE(3279)] = 92710,
  [SMALL_STATE(3280)] = 92725,
  [SMALL_STATE(3281)] = 92742,
  [SMALL_STATE(3282)] = 92759,
  [SMALL_STATE(3283)] = 92776,
  [SMALL_STATE(3284)] = 92793,
  [SMALL_STATE(3285)] = 92810,
  [SMALL_STATE(3286)] = 92827,
  [SMALL_STATE(3287)] = 92844,
  [SMALL_STATE(3288)] = 92861,
  [SMALL_STATE(3289)] = 92878,
  [SMALL_STATE(3290)] = 92895,
  [SMALL_STATE(3291)] = 92912,
  [SMALL_STATE(3292)] = 92929,
  [SMALL_STATE(3293)] = 92946,
  [SMALL_STATE(3294)] = 92963,
  [SMALL_STATE(3295)] = 92980,
  [SMALL_STATE(3296)] = 92997,
  [SMALL_STATE(3297)] = 93012,
  [SMALL_STATE(3298)] = 93029,
  [SMALL_STATE(3299)] = 93046,
  [SMALL_STATE(3300)] = 93063,
  [SMALL_STATE(3301)] = 93080,
  [SMALL_STATE(3302)] = 93097,
  [SMALL_STATE(3303)] = 93114,
  [SMALL_STATE(3304)] = 93129,
  [SMALL_STATE(3305)] = 93146,
  [SMALL_STATE(3306)] = 93161,
  [SMALL_STATE(3307)] = 93178,
  [SMALL_STATE(3308)] = 93193,
  [SMALL_STATE(3309)] = 93210,
  [SMALL_STATE(3310)] = 93227,
  [SMALL_STATE(3311)] = 93242,
  [SMALL_STATE(3312)] = 93257,
  [SMALL_STATE(3313)] = 93272,
  [SMALL_STATE(3314)] = 93289,
  [SMALL_STATE(3315)] = 93306,
  [SMALL_STATE(3316)] = 93321,
  [SMALL_STATE(3317)] = 93338,
  [SMALL_STATE(3318)] = 93355,
  [SMALL_STATE(3319)] = 93370,
  [SMALL_STATE(3320)] = 93387,
  [SMALL_STATE(3321)] = 93404,
  [SMALL_STATE(3322)] = 93421,
  [SMALL_STATE(3323)] = 93438,
  [SMALL_STATE(3324)] = 93455,
  [SMALL_STATE(3325)] = 93472,
  [SMALL_STATE(3326)] = 93489,
  [SMALL_STATE(3327)] = 93506,
  [SMALL_STATE(3328)] = 93523,
  [SMALL_STATE(3329)] = 93540,
  [SMALL_STATE(3330)] = 93557,
  [SMALL_STATE(3331)] = 93574,
  [SMALL_STATE(3332)] = 93591,
  [SMALL_STATE(3333)] = 93608,
  [SMALL_STATE(3334)] = 93625,
  [SMALL_STATE(3335)] = 93642,
  [SMALL_STATE(3336)] = 93659,
  [SMALL_STATE(3337)] = 93674,
  [SMALL_STATE(3338)] = 93691,
  [SMALL_STATE(3339)] = 93708,
  [SMALL_STATE(3340)] = 93725,
  [SMALL_STATE(3341)] = 93740,
  [SMALL_STATE(3342)] = 93754,
  [SMALL_STATE(3343)] = 93768,
  [SMALL_STATE(3344)] = 93782,
  [SMALL_STATE(3345)] = 93796,
  [SMALL_STATE(3346)] = 93810,
  [SMALL_STATE(3347)] = 93824,
  [SMALL_STATE(3348)] = 93838,
  [SMALL_STATE(3349)] = 93852,
  [SMALL_STATE(3350)] = 93866,
  [SMALL_STATE(3351)] = 93880,
  [SMALL_STATE(3352)] = 93894,
  [SMALL_STATE(3353)] = 93908,
  [SMALL_STATE(3354)] = 93922,
  [SMALL_STATE(3355)] = 93936,
  [SMALL_STATE(3356)] = 93950,
  [SMALL_STATE(3357)] = 93964,
  [SMALL_STATE(3358)] = 93978,
  [SMALL_STATE(3359)] = 93992,
  [SMALL_STATE(3360)] = 94006,
  [SMALL_STATE(3361)] = 94020,
  [SMALL_STATE(3362)] = 94034,
  [SMALL_STATE(3363)] = 94048,
  [SMALL_STATE(3364)] = 94062,
  [SMALL_STATE(3365)] = 94076,
  [SMALL_STATE(3366)] = 94090,
  [SMALL_STATE(3367)] = 94104,
  [SMALL_STATE(3368)] = 94118,
  [SMALL_STATE(3369)] = 94132,
  [SMALL_STATE(3370)] = 94146,
  [SMALL_STATE(3371)] = 94160,
  [SMALL_STATE(3372)] = 94174,
  [SMALL_STATE(3373)] = 94188,
  [SMALL_STATE(3374)] = 94202,
  [SMALL_STATE(3375)] = 94216,
  [SMALL_STATE(3376)] = 94230,
  [SMALL_STATE(3377)] = 94244,
  [SMALL_STATE(3378)] = 94258,
  [SMALL_STATE(3379)] = 94272,
  [SMALL_STATE(3380)] = 94286,
  [SMALL_STATE(3381)] = 94300,
  [SMALL_STATE(3382)] = 94314,
  [SMALL_STATE(3383)] = 94328,
  [SMALL_STATE(3384)] = 94342,
  [SMALL_STATE(3385)] = 94356,
  [SMALL_STATE(3386)] = 94370,
  [SMALL_STATE(3387)] = 94384,
  [SMALL_STATE(3388)] = 94398,
  [SMALL_STATE(3389)] = 94412,
  [SMALL_STATE(3390)] = 94426,
  [SMALL_STATE(3391)] = 94440,
  [SMALL_STATE(3392)] = 94454,
  [SMALL_STATE(3393)] = 94468,
  [SMALL_STATE(3394)] = 94482,
  [SMALL_STATE(3395)] = 94496,
  [SMALL_STATE(3396)] = 94510,
  [SMALL_STATE(3397)] = 94524,
  [SMALL_STATE(3398)] = 94538,
  [SMALL_STATE(3399)] = 94552,
  [SMALL_STATE(3400)] = 94566,
  [SMALL_STATE(3401)] = 94580,
  [SMALL_STATE(3402)] = 94594,
  [SMALL_STATE(3403)] = 94608,
  [SMALL_STATE(3404)] = 94622,
  [SMALL_STATE(3405)] = 94636,
  [SMALL_STATE(3406)] = 94650,
  [SMALL_STATE(3407)] = 94664,
  [SMALL_STATE(3408)] = 94678,
  [SMALL_STATE(3409)] = 94692,
  [SMALL_STATE(3410)] = 94706,
  [SMALL_STATE(3411)] = 94720,
  [SMALL_STATE(3412)] = 94734,
  [SMALL_STATE(3413)] = 94748,
  [SMALL_STATE(3414)] = 94762,
  [SMALL_STATE(3415)] = 94776,
  [SMALL_STATE(3416)] = 94790,
  [SMALL_STATE(3417)] = 94804,
  [SMALL_STATE(3418)] = 94818,
  [SMALL_STATE(3419)] = 94832,
  [SMALL_STATE(3420)] = 94846,
  [SMALL_STATE(3421)] = 94860,
  [SMALL_STATE(3422)] = 94874,
  [SMALL_STATE(3423)] = 94888,
  [SMALL_STATE(3424)] = 94902,
  [SMALL_STATE(3425)] = 94916,
  [SMALL_STATE(3426)] = 94930,
  [SMALL_STATE(3427)] = 94944,
  [SMALL_STATE(3428)] = 94958,
  [SMALL_STATE(3429)] = 94972,
  [SMALL_STATE(3430)] = 94986,
  [SMALL_STATE(3431)] = 95000,
  [SMALL_STATE(3432)] = 95014,
  [SMALL_STATE(3433)] = 95028,
  [SMALL_STATE(3434)] = 95042,
  [SMALL_STATE(3435)] = 95056,
  [SMALL_STATE(3436)] = 95070,
  [SMALL_STATE(3437)] = 95084,
  [SMALL_STATE(3438)] = 95098,
  [SMALL_STATE(3439)] = 95112,
  [SMALL_STATE(3440)] = 95126,
  [SMALL_STATE(3441)] = 95140,
  [SMALL_STATE(3442)] = 95154,
  [SMALL_STATE(3443)] = 95168,
  [SMALL_STATE(3444)] = 95182,
  [SMALL_STATE(3445)] = 95196,
  [SMALL_STATE(3446)] = 95210,
  [SMALL_STATE(3447)] = 95224,
  [SMALL_STATE(3448)] = 95238,
  [SMALL_STATE(3449)] = 95252,
  [SMALL_STATE(3450)] = 95266,
  [SMALL_STATE(3451)] = 95280,
  [SMALL_STATE(3452)] = 95294,
  [SMALL_STATE(3453)] = 95308,
  [SMALL_STATE(3454)] = 95322,
  [SMALL_STATE(3455)] = 95336,
  [SMALL_STATE(3456)] = 95350,
  [SMALL_STATE(3457)] = 95364,
  [SMALL_STATE(3458)] = 95378,
  [SMALL_STATE(3459)] = 95392,
  [SMALL_STATE(3460)] = 95406,
  [SMALL_STATE(3461)] = 95420,
  [SMALL_STATE(3462)] = 95434,
  [SMALL_STATE(3463)] = 95448,
  [SMALL_STATE(3464)] = 95462,
  [SMALL_STATE(3465)] = 95476,
  [SMALL_STATE(3466)] = 95490,
  [SMALL_STATE(3467)] = 95504,
  [SMALL_STATE(3468)] = 95518,
  [SMALL_STATE(3469)] = 95532,
  [SMALL_STATE(3470)] = 95546,
  [SMALL_STATE(3471)] = 95560,
  [SMALL_STATE(3472)] = 95574,
  [SMALL_STATE(3473)] = 95588,
  [SMALL_STATE(3474)] = 95602,
  [SMALL_STATE(3475)] = 95616,
  [SMALL_STATE(3476)] = 95630,
  [SMALL_STATE(3477)] = 95644,
  [SMALL_STATE(3478)] = 95658,
  [SMALL_STATE(3479)] = 95672,
  [SMALL_STATE(3480)] = 95686,
  [SMALL_STATE(3481)] = 95700,
  [SMALL_STATE(3482)] = 95714,
  [SMALL_STATE(3483)] = 95728,
  [SMALL_STATE(3484)] = 95742,
  [SMALL_STATE(3485)] = 95756,
  [SMALL_STATE(3486)] = 95770,
  [SMALL_STATE(3487)] = 95784,
  [SMALL_STATE(3488)] = 95798,
  [SMALL_STATE(3489)] = 95812,
  [SMALL_STATE(3490)] = 95826,
  [SMALL_STATE(3491)] = 95840,
  [SMALL_STATE(3492)] = 95854,
  [SMALL_STATE(3493)] = 95868,
  [SMALL_STATE(3494)] = 95882,
  [SMALL_STATE(3495)] = 95896,
  [SMALL_STATE(3496)] = 95910,
  [SMALL_STATE(3497)] = 95924,
  [SMALL_STATE(3498)] = 95938,
  [SMALL_STATE(3499)] = 95952,
  [SMALL_STATE(3500)] = 95966,
  [SMALL_STATE(3501)] = 95980,
  [SMALL_STATE(3502)] = 95994,
  [SMALL_STATE(3503)] = 96008,
  [SMALL_STATE(3504)] = 96022,
  [SMALL_STATE(3505)] = 96036,
  [SMALL_STATE(3506)] = 96050,
  [SMALL_STATE(3507)] = 96064,
  [SMALL_STATE(3508)] = 96078,
  [SMALL_STATE(3509)] = 96092,
  [SMALL_STATE(3510)] = 96106,
  [SMALL_STATE(3511)] = 96120,
  [SMALL_STATE(3512)] = 96134,
  [SMALL_STATE(3513)] = 96148,
  [SMALL_STATE(3514)] = 96162,
  [SMALL_STATE(3515)] = 96176,
  [SMALL_STATE(3516)] = 96190,
  [SMALL_STATE(3517)] = 96204,
  [SMALL_STATE(3518)] = 96218,
  [SMALL_STATE(3519)] = 96232,
  [SMALL_STATE(3520)] = 96246,
  [SMALL_STATE(3521)] = 96260,
  [SMALL_STATE(3522)] = 96274,
  [SMALL_STATE(3523)] = 96288,
  [SMALL_STATE(3524)] = 96302,
  [SMALL_STATE(3525)] = 96316,
  [SMALL_STATE(3526)] = 96330,
  [SMALL_STATE(3527)] = 96344,
  [SMALL_STATE(3528)] = 96358,
  [SMALL_STATE(3529)] = 96372,
  [SMALL_STATE(3530)] = 96386,
  [SMALL_STATE(3531)] = 96400,
  [SMALL_STATE(3532)] = 96414,
  [SMALL_STATE(3533)] = 96428,
  [SMALL_STATE(3534)] = 96442,
  [SMALL_STATE(3535)] = 96456,
  [SMALL_STATE(3536)] = 96470,
  [SMALL_STATE(3537)] = 96484,
  [SMALL_STATE(3538)] = 96498,
  [SMALL_STATE(3539)] = 96512,
  [SMALL_STATE(3540)] = 96526,
  [SMALL_STATE(3541)] = 96540,
  [SMALL_STATE(3542)] = 96554,
  [SMALL_STATE(3543)] = 96568,
  [SMALL_STATE(3544)] = 96582,
  [SMALL_STATE(3545)] = 96596,
  [SMALL_STATE(3546)] = 96610,
  [SMALL_STATE(3547)] = 96624,
  [SMALL_STATE(3548)] = 96638,
  [SMALL_STATE(3549)] = 96652,
  [SMALL_STATE(3550)] = 96666,
  [SMALL_STATE(3551)] = 96680,
  [SMALL_STATE(3552)] = 96694,
  [SMALL_STATE(3553)] = 96708,
  [SMALL_STATE(3554)] = 96722,
  [SMALL_STATE(3555)] = 96736,
  [SMALL_STATE(3556)] = 96750,
  [SMALL_STATE(3557)] = 96764,
  [SMALL_STATE(3558)] = 96778,
  [SMALL_STATE(3559)] = 96792,
  [SMALL_STATE(3560)] = 96806,
  [SMALL_STATE(3561)] = 96820,
  [SMALL_STATE(3562)] = 96834,
  [SMALL_STATE(3563)] = 96848,
  [SMALL_STATE(3564)] = 96862,
  [SMALL_STATE(3565)] = 96876,
  [SMALL_STATE(3566)] = 96890,
  [SMALL_STATE(3567)] = 96904,
  [SMALL_STATE(3568)] = 96918,
  [SMALL_STATE(3569)] = 96932,
  [SMALL_STATE(3570)] = 96946,
  [SMALL_STATE(3571)] = 96960,
  [SMALL_STATE(3572)] = 96974,
  [SMALL_STATE(3573)] = 96988,
  [SMALL_STATE(3574)] = 97002,
  [SMALL_STATE(3575)] = 97016,
  [SMALL_STATE(3576)] = 97030,
  [SMALL_STATE(3577)] = 97044,
  [SMALL_STATE(3578)] = 97058,
  [SMALL_STATE(3579)] = 97072,
  [SMALL_STATE(3580)] = 97086,
  [SMALL_STATE(3581)] = 97100,
  [SMALL_STATE(3582)] = 97114,
  [SMALL_STATE(3583)] = 97128,
  [SMALL_STATE(3584)] = 97142,
  [SMALL_STATE(3585)] = 97156,
  [SMALL_STATE(3586)] = 97170,
  [SMALL_STATE(3587)] = 97184,
  [SMALL_STATE(3588)] = 97198,
  [SMALL_STATE(3589)] = 97212,
  [SMALL_STATE(3590)] = 97226,
  [SMALL_STATE(3591)] = 97240,
  [SMALL_STATE(3592)] = 97254,
  [SMALL_STATE(3593)] = 97268,
  [SMALL_STATE(3594)] = 97282,
  [SMALL_STATE(3595)] = 97296,
  [SMALL_STATE(3596)] = 97310,
  [SMALL_STATE(3597)] = 97324,
  [SMALL_STATE(3598)] = 97338,
  [SMALL_STATE(3599)] = 97352,
  [SMALL_STATE(3600)] = 97366,
  [SMALL_STATE(3601)] = 97380,
  [SMALL_STATE(3602)] = 97394,
  [SMALL_STATE(3603)] = 97408,
  [SMALL_STATE(3604)] = 97422,
  [SMALL_STATE(3605)] = 97436,
  [SMALL_STATE(3606)] = 97450,
  [SMALL_STATE(3607)] = 97464,
  [SMALL_STATE(3608)] = 97478,
  [SMALL_STATE(3609)] = 97492,
  [SMALL_STATE(3610)] = 97506,
  [SMALL_STATE(3611)] = 97520,
  [SMALL_STATE(3612)] = 97534,
  [SMALL_STATE(3613)] = 97548,
  [SMALL_STATE(3614)] = 97562,
  [SMALL_STATE(3615)] = 97576,
  [SMALL_STATE(3616)] = 97590,
  [SMALL_STATE(3617)] = 97604,
  [SMALL_STATE(3618)] = 97618,
  [SMALL_STATE(3619)] = 97632,
  [SMALL_STATE(3620)] = 97646,
  [SMALL_STATE(3621)] = 97660,
  [SMALL_STATE(3622)] = 97674,
  [SMALL_STATE(3623)] = 97688,
  [SMALL_STATE(3624)] = 97702,
  [SMALL_STATE(3625)] = 97716,
  [SMALL_STATE(3626)] = 97730,
  [SMALL_STATE(3627)] = 97744,
  [SMALL_STATE(3628)] = 97758,
  [SMALL_STATE(3629)] = 97772,
  [SMALL_STATE(3630)] = 97786,
  [SMALL_STATE(3631)] = 97800,
  [SMALL_STATE(3632)] = 97814,
  [SMALL_STATE(3633)] = 97828,
  [SMALL_STATE(3634)] = 97842,
  [SMALL_STATE(3635)] = 97856,
  [SMALL_STATE(3636)] = 97870,
  [SMALL_STATE(3637)] = 97884,
  [SMALL_STATE(3638)] = 97898,
  [SMALL_STATE(3639)] = 97912,
  [SMALL_STATE(3640)] = 97926,
  [SMALL_STATE(3641)] = 97940,
  [SMALL_STATE(3642)] = 97954,
  [SMALL_STATE(3643)] = 97958,
  [SMALL_STATE(3644)] = 97962,
  [SMALL_STATE(3645)] = 97966,
  [SMALL_STATE(3646)] = 97970,
  [SMALL_STATE(3647)] = 97974,
  [SMALL_STATE(3648)] = 97978,
};

static const TSParseActionEntry ts_parse_actions[] = {
  [0] = {.entry = {.count = 0, .reusable = false}},
  [1] = {.entry = {.count = 1, .reusable = false}}, RECOVER(),
  [3] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2303),
  [5] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2481),
  [7] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_source_file, 0, 0, 0),
  [9] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1515),
  [11] = {.entry = {.count = 1, .reusable = true}}, SHIFT(736),
  [13] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2595),
  [15] = {.entry = {.count = 1, .reusable = true}}, SHIFT(145),
  [17] = {.entry = {.count = 1, .reusable = true}}, SHIFT(134),
  [19] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2),
  [21] = {.entry = {.count = 1, .reusable = true}}, SHIFT(291),
  [23] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1496),
  [25] = {.entry = {.count = 1, .reusable = true}}, SHIFT(238),
  [27] = {.entry = {.count = 1, .reusable = true}}, SHIFT(789),
  [29] = {.entry = {.count = 1, .reusable = true}}, SHIFT(857),
  [31] = {.entry = {.count = 1, .reusable = true}}, SHIFT(38),
  [33] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3298),
  [35] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3286),
  [37] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3586),
  [39] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2058),
  [41] = {.entry = {.count = 1, .reusable = false}}, SHIFT(39),
  [43] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2081),
  [45] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1365),
  [47] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1059),
  [49] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3454),
  [51] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3123),
  [53] = {.entry = {.count = 1, .reusable = false}}, SHIFT(827),
  [55] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1062),
  [57] = {.entry = {.count = 1, .reusable = false}}, SHIFT(181),
  [59] = {.entry = {.count = 1, .reusable = false}}, SHIFT(860),
  [61] = {.entry = {.count = 1, .reusable = false}}, SHIFT(816),
  [63] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2534),
  [65] = {.entry = {.count = 1, .reusable = false}}, SHIFT(315),
  [67] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3481),
  [69] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1972),
  [71] = {.entry = {.count = 1, .reusable = false}}, SHIFT(41),
  [73] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2337),
  [75] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3363),
  [77] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3573),
  [79] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3411),
  [81] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1527),
  [83] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2042),
  [85] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1898),
  [87] = {.entry = {.count = 1, .reusable = false}}, SHIFT(166),
  [89] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2057),
  [91] = {.entry = {.count = 1, .reusable = false}}, SHIFT(36),
  [93] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3130),
  [95] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2557),
  [97] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1489),
  [99] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2524),
  [101] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1413),
  [103] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2303),
  [105] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2481),
  [107] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4),
  [109] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1501),
  [111] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3537),
  [113] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1988),
  [115] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1501),
  [117] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3563),
  [119] = {.entry = {.count = 1, .reusable = true}}, SHIFT(374),
  [121] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3286),
  [123] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1652),
  [125] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_source_file, 1, 0, 0),
  [127] = {.entry = {.count = 1, .reusable = true}}, SHIFT(370),
  [129] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_source_file, 2, 0, 0),
  [131] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0),
  [133] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1515),
  [136] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(736),
  [139] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2595),
  [142] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(145),
  [145] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(134),
  [148] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2),
  [151] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(291),
  [154] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1496),
  [157] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(238),
  [160] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(789),
  [163] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(857),
  [166] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(38),
  [169] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(3298),
  [172] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(3286),
  [175] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(3586),
  [178] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2058),
  [181] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(39),
  [184] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2081),
  [187] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1365),
  [190] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1059),
  [193] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(3454),
  [196] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(3123),
  [199] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(827),
  [202] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1062),
  [205] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(181),
  [208] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(860),
  [211] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(816),
  [214] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2534),
  [217] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(315),
  [220] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(3481),
  [223] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1972),
  [226] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(41),
  [229] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2337),
  [232] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(3363),
  [235] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(3573),
  [238] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(3411),
  [241] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1527),
  [244] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2042),
  [247] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1898),
  [250] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(166),
  [253] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2057),
  [256] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(36),
  [259] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(3130),
  [262] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2557),
  [265] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1489),
  [268] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2524),
  [271] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1413),
  [274] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1501),
  [277] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(3537),
  [280] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1988),
  [283] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1501),
  [286] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(3563),
  [289] = {.entry = {.count = 1, .reusable = true}}, SHIFT(373),
  [291] = {.entry = {.count = 1, .reusable = true}}, SHIFT(372),
  [293] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1408),
  [295] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1429),
  [297] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1376),
  [299] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1454),
  [301] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2065),
  [303] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2093),
  [305] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2078),
  [307] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2064),
  [309] = {.entry = {.count = 1, .reusable = true}}, SHIFT(630),
  [311] = {.entry = {.count = 1, .reusable = true}}, SHIFT(646),
  [313] = {.entry = {.count = 1, .reusable = true}}, SHIFT(649),
  [315] = {.entry = {.count = 1, .reusable = true}}, SHIFT(650),
  [317] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1029),
  [319] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1025),
  [321] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1027),
  [323] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1031),
  [325] = {.entry = {.count = 1, .reusable = true}}, SHIFT(456),
  [327] = {.entry = {.count = 1, .reusable = true}}, SHIFT(454),
  [329] = {.entry = {.count = 1, .reusable = true}}, SHIFT(453),
  [331] = {.entry = {.count = 1, .reusable = true}}, SHIFT(457),
  [333] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1648),
  [335] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1650),
  [337] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1651),
  [339] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1057),
  [341] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_break_expression, 2, 0, 0),
  [343] = {.entry = {.count = 1, .reusable = true}}, SHIFT(10),
  [345] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2663),
  [347] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_break_expression, 2, 0, 0),
  [349] = {.entry = {.count = 1, .reusable = false}}, SHIFT(291),
  [351] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2421),
  [353] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2531),
  [355] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1453),
  [357] = {.entry = {.count = 1, .reusable = false}}, SHIFT(831),
  [359] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1019),
  [361] = {.entry = {.count = 1, .reusable = false}}, SHIFT(188),
  [363] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2627),
  [365] = {.entry = {.count = 1, .reusable = false}}, SHIFT(338),
  [367] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2789),
  [369] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2637),
  [371] = {.entry = {.count = 1, .reusable = false}}, SHIFT(189),
  [373] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2629),
  [375] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_yield_expression, 1, 0, 0),
  [377] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_yield_expression, 1, 0, 0),
  [379] = {.entry = {.count = 1, .reusable = false}}, SHIFT(238),
  [381] = {.entry = {.count = 1, .reusable = false}}, SHIFT(789),
  [383] = {.entry = {.count = 1, .reusable = false}}, SHIFT(857),
  [385] = {.entry = {.count = 1, .reusable = false}}, SHIFT(40),
  [387] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_range_expression, 2, 0, 0),
  [389] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_range_expression, 2, 0, 0),
  [391] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_range_expression, 1, 0, 0),
  [393] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_range_expression, 1, 0, 0),
  [395] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_break_expression, 1, 0, 0),
  [397] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_break_expression, 1, 0, 0),
  [399] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3418),
  [401] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_return_expression, 1, 0, 0),
  [403] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_return_expression, 1, 0, 0),
  [405] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1539),
  [407] = {.entry = {.count = 1, .reusable = true}}, SHIFT(31),
  [409] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2731),
  [411] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1637),
  [413] = {.entry = {.count = 1, .reusable = false}}, SHIFT(292),
  [415] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3068),
  [417] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2461),
  [419] = {.entry = {.count = 1, .reusable = false}}, SHIFT(44),
  [421] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2620),
  [423] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1570),
  [425] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1573),
  [427] = {.entry = {.count = 1, .reusable = false}}, SHIFT(835),
  [429] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1465),
  [431] = {.entry = {.count = 1, .reusable = false}}, SHIFT(194),
  [433] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2596),
  [435] = {.entry = {.count = 1, .reusable = false}}, SHIFT(364),
  [437] = {.entry = {.count = 1, .reusable = false}}, SHIFT(47),
  [439] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2952),
  [441] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2549),
  [443] = {.entry = {.count = 1, .reusable = false}}, SHIFT(195),
  [445] = {.entry = {.count = 1, .reusable = false}}, SHIFT(46),
  [447] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3151),
  [449] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2597),
  [451] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1786),
  [453] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2630),
  [455] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1787),
  [457] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1647),
  [459] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3494),
  [461] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1647),
  [463] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3549),
  [465] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3634),
  [467] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1538),
  [469] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1580),
  [471] = {.entry = {.count = 1, .reusable = false}}, SHIFT(308),
  [473] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3148),
  [475] = {.entry = {.count = 1, .reusable = false}}, SHIFT(57),
  [477] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1555),
  [479] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1499),
  [481] = {.entry = {.count = 1, .reusable = false}}, SHIFT(65),
  [483] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2903),
  [485] = {.entry = {.count = 1, .reusable = false}}, SHIFT(66),
  [487] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3070),
  [489] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1559),
  [491] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3351),
  [493] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1559),
  [495] = {.entry = {.count = 1, .reusable = true}}, SHIFT(149),
  [497] = {.entry = {.count = 1, .reusable = true}}, SHIFT(138),
  [499] = {.entry = {.count = 1, .reusable = false}}, SHIFT(216),
  [501] = {.entry = {.count = 1, .reusable = false}}, SHIFT(50),
  [503] = {.entry = {.count = 1, .reusable = false}}, SHIFT(276),
  [505] = {.entry = {.count = 1, .reusable = false}}, SHIFT(63),
  [507] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1582),
  [509] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1497),
  [511] = {.entry = {.count = 1, .reusable = false}}, SHIFT(60),
  [513] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2959),
  [515] = {.entry = {.count = 1, .reusable = false}}, SHIFT(53),
  [517] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3153),
  [519] = {.entry = {.count = 1, .reusable = false}}, SHIFT(215),
  [521] = {.entry = {.count = 1, .reusable = false}}, SHIFT(64),
  [523] = {.entry = {.count = 1, .reusable = false}}, SHIFT(240),
  [525] = {.entry = {.count = 1, .reusable = false}}, SHIFT(58),
  [527] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_token_tree_pattern_repeat1, 2, 0, 0), SHIFT_REPEAT(170),
  [530] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_token_tree_pattern_repeat1, 2, 0, 0), SHIFT_REPEAT(171),
  [533] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_token_tree_pattern_repeat1, 2, 0, 0), SHIFT_REPEAT(68),
  [536] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_token_tree_pattern_repeat1, 2, 0, 0),
  [538] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_token_tree_pattern_repeat1, 2, 0, 0), SHIFT_REPEAT(69),
  [541] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_token_tree_pattern_repeat1, 2, 0, 0), SHIFT_REPEAT(70),
  [544] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_token_tree_pattern_repeat1, 2, 0, 0), SHIFT_REPEAT(171),
  [547] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_token_tree_pattern_repeat1, 2, 0, 0), SHIFT_REPEAT(3491),
  [550] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_token_tree_pattern_repeat1, 2, 0, 0), SHIFT_REPEAT(177),
  [553] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_token_tree_pattern_repeat1, 2, 0, 0), SHIFT_REPEAT(2619),
  [556] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_token_tree_pattern_repeat1, 2, 0, 0), SHIFT_REPEAT(180),
  [559] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_token_tree_pattern_repeat1, 2, 0, 0), SHIFT_REPEAT(173),
  [562] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_token_tree_pattern_repeat1, 2, 0, 0), SHIFT_REPEAT(3543),
  [565] = {.entry = {.count = 1, .reusable = false}}, SHIFT(170),
  [567] = {.entry = {.count = 1, .reusable = true}}, SHIFT(171),
  [569] = {.entry = {.count = 1, .reusable = true}}, SHIFT(68),
  [571] = {.entry = {.count = 1, .reusable = true}}, SHIFT(172),
  [573] = {.entry = {.count = 1, .reusable = true}}, SHIFT(69),
  [575] = {.entry = {.count = 1, .reusable = true}}, SHIFT(70),
  [577] = {.entry = {.count = 1, .reusable = false}}, SHIFT(171),
  [579] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3491),
  [581] = {.entry = {.count = 1, .reusable = true}}, SHIFT(177),
  [583] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2619),
  [585] = {.entry = {.count = 1, .reusable = false}}, SHIFT(180),
  [587] = {.entry = {.count = 1, .reusable = true}}, SHIFT(173),
  [589] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3543),
  [591] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3484),
  [593] = {.entry = {.count = 1, .reusable = true}}, SHIFT(174),
  [595] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3388),
  [597] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2673),
  [599] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2518),
  [601] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_token_tree_repeat1, 2, 0, 0), SHIFT_REPEAT(152),
  [604] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_token_tree_repeat1, 2, 0, 0), SHIFT_REPEAT(171),
  [607] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_token_tree_repeat1, 2, 0, 0), SHIFT_REPEAT(91),
  [610] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_token_tree_repeat1, 2, 0, 0),
  [612] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_token_tree_repeat1, 2, 0, 0), SHIFT_REPEAT(92),
  [615] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_token_tree_repeat1, 2, 0, 0), SHIFT_REPEAT(93),
  [618] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_token_tree_repeat1, 2, 0, 0), SHIFT_REPEAT(171),
  [621] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_token_tree_repeat1, 2, 0, 0), SHIFT_REPEAT(3470),
  [624] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_token_tree_repeat1, 2, 0, 0), SHIFT_REPEAT(177),
  [627] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_token_tree_repeat1, 2, 0, 0), SHIFT_REPEAT(2619),
  [630] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_token_tree_repeat1, 2, 0, 0), SHIFT_REPEAT(180),
  [633] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_token_tree_repeat1, 2, 0, 0), SHIFT_REPEAT(152),
  [636] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_token_tree_repeat1, 2, 0, 0), SHIFT_REPEAT(3543),
  [639] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_delim_token_tree_repeat1, 2, 0, 0), SHIFT_REPEAT(214),
  [642] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_delim_token_tree_repeat1, 2, 0, 0), SHIFT_REPEAT(213),
  [645] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_delim_token_tree_repeat1, 2, 0, 0), SHIFT_REPEAT(107),
  [648] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_delim_token_tree_repeat1, 2, 0, 0),
  [650] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_delim_token_tree_repeat1, 2, 0, 0), SHIFT_REPEAT(108),
  [653] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_delim_token_tree_repeat1, 2, 0, 0), SHIFT_REPEAT(109),
  [656] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_delim_token_tree_repeat1, 2, 0, 0), SHIFT_REPEAT(213),
  [659] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_delim_token_tree_repeat1, 2, 0, 0), SHIFT_REPEAT(214),
  [662] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_delim_token_tree_repeat1, 2, 0, 0), SHIFT_REPEAT(199),
  [665] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_delim_token_tree_repeat1, 2, 0, 0), SHIFT_REPEAT(2546),
  [668] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_delim_token_tree_repeat1, 2, 0, 0), SHIFT_REPEAT(200),
  [671] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_delim_token_tree_repeat1, 2, 0, 0), SHIFT_REPEAT(3490),
  [674] = {.entry = {.count = 1, .reusable = false}}, SHIFT(214),
  [676] = {.entry = {.count = 1, .reusable = true}}, SHIFT(213),
  [678] = {.entry = {.count = 1, .reusable = true}}, SHIFT(107),
  [680] = {.entry = {.count = 1, .reusable = true}}, SHIFT(108),
  [682] = {.entry = {.count = 1, .reusable = true}}, SHIFT(109),
  [684] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1797),
  [686] = {.entry = {.count = 1, .reusable = false}}, SHIFT(213),
  [688] = {.entry = {.count = 1, .reusable = true}}, SHIFT(214),
  [690] = {.entry = {.count = 1, .reusable = true}}, SHIFT(199),
  [692] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2546),
  [694] = {.entry = {.count = 1, .reusable = false}}, SHIFT(200),
  [696] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3490),
  [698] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1417),
  [700] = {.entry = {.count = 1, .reusable = false}}, SHIFT(152),
  [702] = {.entry = {.count = 1, .reusable = true}}, SHIFT(91),
  [704] = {.entry = {.count = 1, .reusable = true}}, SHIFT(92),
  [706] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2675),
  [708] = {.entry = {.count = 1, .reusable = true}}, SHIFT(93),
  [710] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3470),
  [712] = {.entry = {.count = 1, .reusable = true}}, SHIFT(152),
  [714] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2685),
  [716] = {.entry = {.count = 1, .reusable = true}}, SHIFT(197),
  [718] = {.entry = {.count = 1, .reusable = true}}, SHIFT(175),
  [720] = {.entry = {.count = 1, .reusable = true}}, SHIFT(384),
  [722] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2691),
  [724] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1971),
  [726] = {.entry = {.count = 1, .reusable = true}}, SHIFT(388),
  [728] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1976),
  [730] = {.entry = {.count = 1, .reusable = true}}, SHIFT(209),
  [732] = {.entry = {.count = 1, .reusable = true}}, SHIFT(210),
  [734] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1023),
  [736] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1021),
  [738] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2533),
  [740] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1385),
  [742] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1798),
  [744] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1386),
  [746] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3624),
  [748] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3463),
  [750] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1719),
  [752] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3467),
  [754] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__token_pattern, 1, 0, 0),
  [756] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__token_pattern, 1, 0, 0),
  [758] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym__non_special_token_repeat1, 2, 0, 0),
  [760] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__non_special_token_repeat1, 2, 0, 0), SHIFT_REPEAT(171),
  [763] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__non_special_token_repeat1, 2, 0, 0),
  [765] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__non_special_token_repeat1, 2, 0, 0), SHIFT_REPEAT(171),
  [768] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1675),
  [770] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3486),
  [772] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1493),
  [774] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3637),
  [776] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1835),
  [778] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3528),
  [780] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0), SHIFT(1057),
  [783] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0), SHIFT(145),
  [786] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0), SHIFT(134),
  [789] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0),
  [791] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0), SHIFT(10),
  [794] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0), SHIFT(291),
  [797] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0), SHIFT(1496),
  [800] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0), SHIFT(238),
  [803] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0), SHIFT(789),
  [806] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0), SHIFT(857),
  [809] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0), SHIFT(38),
  [812] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0), SHIFT(3298),
  [815] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0), SHIFT_REPEAT(3463),
  [818] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0), SHIFT(3586),
  [821] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0), SHIFT(2421),
  [824] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0), SHIFT(39),
  [827] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0), SHIFT(2531),
  [830] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0), SHIFT(1365),
  [833] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0), SHIFT(1453),
  [836] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0), SHIFT(831),
  [839] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0), SHIFT(1019),
  [842] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0), SHIFT(188),
  [845] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0), SHIFT(2627),
  [848] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0), SHIFT(338),
  [851] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0), SHIFT(41),
  [854] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0), SHIFT(2789),
  [857] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0), SHIFT(2637),
  [860] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0), SHIFT(189),
  [863] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0), SHIFT(36),
  [866] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0), SHIFT(3130),
  [869] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0), SHIFT(2629),
  [872] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0), SHIFT(1489),
  [875] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0), SHIFT(2524),
  [878] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0), SHIFT(1413),
  [881] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0), SHIFT(1501),
  [884] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0), SHIFT(3537),
  [887] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0), SHIFT(1501),
  [890] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0), SHIFT(3563),
  [893] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1374),
  [895] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3417),
  [897] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_token_tree_repeat1, 1, 0, 0),
  [899] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_token_tree_repeat1, 1, 0, 0),
  [901] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1432),
  [903] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1399),
  [905] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1447),
  [907] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1444),
  [909] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1667),
  [911] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_fragment_specifier, 1, 0, 0),
  [913] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_fragment_specifier, 1, 0, 0),
  [915] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_token_binding_pattern, 3, 0, 198),
  [917] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_token_binding_pattern, 3, 0, 198),
  [919] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1486),
  [921] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1655),
  [923] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_token_repetition_pattern, 4, 0, 0),
  [925] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_token_repetition_pattern, 4, 0, 0),
  [927] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_token_repetition_pattern, 5, 0, 0),
  [929] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_token_repetition_pattern, 5, 0, 0),
  [931] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__non_delim_token, 1, 0, 0),
  [933] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__non_delim_token, 1, 0, 0),
  [935] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1674),
  [937] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_token_repetition_pattern, 6, 0, 0),
  [939] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_token_repetition_pattern, 6, 0, 0),
  [941] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_token_repetition, 4, 0, 0),
  [943] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_token_repetition, 4, 0, 0),
  [945] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_token_repetition, 5, 0, 0),
  [947] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_token_repetition, 5, 0, 0),
  [949] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1733),
  [951] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_token_repetition, 6, 0, 0),
  [953] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_token_repetition, 6, 0, 0),
  [955] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1744),
  [957] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1857),
  [959] = {.entry = {.count = 1, .reusable = true}}, SHIFT(276),
  [961] = {.entry = {.count = 1, .reusable = true}}, SHIFT(215),
  [963] = {.entry = {.count = 1, .reusable = true}}, SHIFT(61),
  [965] = {.entry = {.count = 1, .reusable = false}}, SHIFT(814),
  [967] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1397),
  [969] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1859),
  [971] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1661),
  [973] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym__non_special_token_repeat1, 1, 0, 0),
  [975] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__non_special_token_repeat1, 1, 0, 0),
  [977] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_token_tree_pattern, 2, 0, 0),
  [979] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_token_tree_pattern, 2, 0, 0),
  [981] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2021),
  [983] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_token_tree_pattern, 3, 0, 0),
  [985] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_token_tree_pattern, 3, 0, 0),
  [987] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_token_tree, 3, 0, 0),
  [989] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_token_tree, 3, 0, 0),
  [991] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_token_tree_pattern_repeat1, 1, 0, 0),
  [993] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_token_tree_pattern_repeat1, 1, 0, 0),
  [995] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__literal, 1, 0, 0),
  [997] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__literal, 1, 0, 0),
  [999] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1411),
  [1001] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__non_special_token_repeat1, 2, 0, 0), SHIFT_REPEAT(213),
  [1004] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__non_special_token_repeat1, 2, 0, 0), SHIFT_REPEAT(213),
  [1007] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_boolean_literal, 1, 0, 0),
  [1009] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_boolean_literal, 1, 0, 0),
  [1011] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_string_literal, 2, 0, 0),
  [1013] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_string_literal, 2, 0, 0),
  [1015] = {.entry = {.count = 1, .reusable = true}}, SHIFT(292),
  [1017] = {.entry = {.count = 1, .reusable = true}}, SHIFT(216),
  [1019] = {.entry = {.count = 1, .reusable = true}}, SHIFT(48),
  [1021] = {.entry = {.count = 1, .reusable = false}}, SHIFT(820),
  [1023] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_raw_string_literal, 3, 0, 0),
  [1025] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_raw_string_literal, 3, 0, 0),
  [1027] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_string_literal, 3, 0, 0),
  [1029] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_string_literal, 3, 0, 0),
  [1031] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1442),
  [1033] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_token_tree, 2, 0, 0),
  [1035] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_token_tree, 2, 0, 0),
  [1037] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__delim_tokens, 1, 0, 0),
  [1039] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__delim_tokens, 1, 0, 0),
  [1041] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_delim_token_tree_repeat1, 1, 0, 0),
  [1043] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_delim_token_tree_repeat1, 1, 0, 0),
  [1045] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_delim_token_tree, 2, 0, 0),
  [1047] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_delim_token_tree, 2, 0, 0),
  [1049] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_delim_token_tree, 3, 0, 0),
  [1051] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_delim_token_tree, 3, 0, 0),
  [1053] = {.entry = {.count = 1, .reusable = true}}, SHIFT(64),
  [1055] = {.entry = {.count = 1, .reusable = false}}, SHIFT(297),
  [1057] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3184),
  [1059] = {.entry = {.count = 1, .reusable = true}}, SHIFT(50),
  [1061] = {.entry = {.count = 1, .reusable = false}}, SHIFT(247),
  [1063] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3132),
  [1065] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1741),
  [1067] = {.entry = {.count = 1, .reusable = true}}, SHIFT(308),
  [1069] = {.entry = {.count = 1, .reusable = true}}, SHIFT(240),
  [1071] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1438),
  [1073] = {.entry = {.count = 1, .reusable = true}}, SHIFT(58),
  [1075] = {.entry = {.count = 1, .reusable = true}}, SHIFT(972),
  [1077] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1488),
  [1079] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1009),
  [1081] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1398),
  [1083] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1672),
  [1085] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1470),
  [1087] = {.entry = {.count = 1, .reusable = true}}, SHIFT(906),
  [1089] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1770),
  [1091] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(1057),
  [1094] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(145),
  [1097] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0),
  [1099] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(134),
  [1102] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(10),
  [1105] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(291),
  [1108] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(1496),
  [1111] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(238),
  [1114] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(789),
  [1117] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(857),
  [1120] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(38),
  [1123] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(3298),
  [1126] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(3586),
  [1129] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(2421),
  [1132] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(39),
  [1135] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(2531),
  [1138] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(1365),
  [1141] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(1453),
  [1144] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(831),
  [1147] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(1019),
  [1150] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(188),
  [1153] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(2627),
  [1156] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(338),
  [1159] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(41),
  [1162] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(2789),
  [1165] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(2637),
  [1168] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(189),
  [1171] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(36),
  [1174] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(3130),
  [1177] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(2629),
  [1180] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(1489),
  [1183] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(2524),
  [1186] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(1413),
  [1189] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(1501),
  [1192] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(3537),
  [1195] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(1501),
  [1198] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(3563),
  [1201] = {.entry = {.count = 1, .reusable = true}}, SHIFT(40),
  [1203] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1701),
  [1205] = {.entry = {.count = 1, .reusable = true}}, SHIFT(941),
  [1207] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1803),
  [1209] = {.entry = {.count = 1, .reusable = true}}, SHIFT(951),
  [1211] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1659),
  [1213] = {.entry = {.count = 1, .reusable = true}}, SHIFT(963),
  [1215] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1484),
  [1217] = {.entry = {.count = 1, .reusable = false}}, SHIFT(294),
  [1219] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3279),
  [1221] = {.entry = {.count = 1, .reusable = false}}, SHIFT(319),
  [1223] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3075),
  [1225] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1400),
  [1227] = {.entry = {.count = 1, .reusable = true}}, SHIFT(56),
  [1229] = {.entry = {.count = 1, .reusable = true}}, SHIFT(27),
  [1231] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2485),
  [1233] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2599),
  [1235] = {.entry = {.count = 1, .reusable = false}}, SHIFT(833),
  [1237] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1370),
  [1239] = {.entry = {.count = 1, .reusable = false}}, SHIFT(191),
  [1241] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2544),
  [1243] = {.entry = {.count = 1, .reusable = false}}, SHIFT(359),
  [1245] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2656),
  [1247] = {.entry = {.count = 1, .reusable = false}}, SHIFT(192),
  [1249] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2545),
  [1251] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_if_expression, 3, 0, 30),
  [1253] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_if_expression, 3, 0, 30),
  [1255] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2335),
  [1257] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block, 3, 0, 0),
  [1259] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block, 3, 0, 0),
  [1261] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block, 6, 0, 0),
  [1263] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block, 6, 0, 0),
  [1265] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block, 5, 0, 0),
  [1267] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block, 5, 0, 0),
  [1269] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block, 4, 0, 0),
  [1271] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block, 4, 0, 0),
  [1273] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block, 2, 0, 0),
  [1275] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block, 2, 0, 0),
  [1277] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_if_expression, 4, 0, 76),
  [1279] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_if_expression, 4, 0, 76),
  [1281] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1974),
  [1283] = {.entry = {.count = 1, .reusable = true}}, SHIFT(417),
  [1285] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1958),
  [1287] = {.entry = {.count = 1, .reusable = true}}, SHIFT(424),
  [1289] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3160),
  [1291] = {.entry = {.count = 1, .reusable = true}}, SHIFT(991),
  [1293] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2177),
  [1295] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3336),
  [1297] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1986),
  [1299] = {.entry = {.count = 1, .reusable = true}}, SHIFT(426),
  [1301] = {.entry = {.count = 1, .reusable = true}}, SHIFT(809),
  [1303] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2439),
  [1305] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2096),
  [1307] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3248),
  [1309] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3455),
  [1311] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3170),
  [1313] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3609),
  [1315] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3472),
  [1317] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2387),
  [1319] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2127),
  [1321] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1990),
  [1323] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3143),
  [1325] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3574),
  [1327] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2188),
  [1329] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1048),
  [1331] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2233),
  [1333] = {.entry = {.count = 1, .reusable = false}}, SHIFT(815),
  [1335] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1495),
  [1337] = {.entry = {.count = 1, .reusable = false}}, SHIFT(793),
  [1339] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2037),
  [1341] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2635),
  [1343] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2020),
  [1345] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2280),
  [1347] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2657),
  [1349] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2307),
  [1351] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3630),
  [1353] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_expression_statement, 1, 0, 0),
  [1355] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_expression_statement, 1, 0, 0),
  [1357] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__expression_except_range, 1, 0, 0),
  [1359] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__expression_except_range, 1, 0, 0),
  [1361] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_gen_block, 3, 0, 0),
  [1363] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_gen_block, 3, 0, 0),
  [1365] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_unsafe_block, 2, 0, 0),
  [1367] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_unsafe_block, 2, 0, 0),
  [1369] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1968),
  [1371] = {.entry = {.count = 1, .reusable = true}}, SHIFT(418),
  [1373] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2372),
  [1375] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2163),
  [1377] = {.entry = {.count = 1, .reusable = true}}, SHIFT(427),
  [1379] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2444),
  [1381] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3627),
  [1383] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3196),
  [1385] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1956),
  [1387] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2084),
  [1389] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2285),
  [1391] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2717),
  [1393] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2286),
  [1395] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_async_block, 2, 0, 0),
  [1397] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_async_block, 2, 0, 0),
  [1399] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_async_block, 3, 0, 0),
  [1401] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_async_block, 3, 0, 0),
  [1403] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_try_block, 2, 0, 0),
  [1405] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_try_block, 2, 0, 0),
  [1407] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_while_expression, 3, 0, 38),
  [1409] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_while_expression, 3, 0, 38),
  [1411] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_match_block, 3, 0, 0),
  [1413] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_match_block, 3, 0, 0),
  [1415] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_invocation, 3, 0, 39),
  [1417] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_invocation, 3, 0, 39),
  [1419] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1602),
  [1421] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2475),
  [1423] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3439),
  [1425] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2367),
  [1427] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3409),
  [1429] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_match_block, 2, 0, 0),
  [1431] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_match_block, 2, 0, 0),
  [1433] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_for_expression, 7, 0, 242),
  [1435] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_for_expression, 7, 0, 242),
  [1437] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_match_expression, 3, 0, 33),
  [1439] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_match_expression, 3, 0, 33),
  [1441] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1116),
  [1443] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2420),
  [1445] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3594),
  [1447] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_gen_block, 2, 0, 0),
  [1449] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_gen_block, 2, 0, 0),
  [1451] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_loop_expression, 2, 0, 8),
  [1453] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_loop_expression, 2, 0, 8),
  [1455] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_const_block, 2, 0, 8),
  [1457] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_const_block, 2, 0, 8),
  [1459] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_loop_expression, 4, 0, 99),
  [1461] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_loop_expression, 4, 0, 99),
  [1463] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__statement, 1, 0, 0),
  [1465] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__statement, 1, 0, 0),
  [1467] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_while_expression, 5, 0, 146),
  [1469] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_while_expression, 5, 0, 146),
  [1471] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2329),
  [1473] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3561),
  [1475] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_invocation, 3, 0, 28),
  [1477] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_invocation, 3, 0, 28),
  [1479] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_for_expression, 5, 0, 116),
  [1481] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_for_expression, 5, 0, 116),
  [1483] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_else_clause, 2, 0, 0),
  [1485] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_else_clause, 2, 0, 0),
  [1487] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_match_block, 4, 0, 0),
  [1489] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_match_block, 4, 0, 0),
  [1491] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1636),
  [1493] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2689),
  [1495] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1984),
  [1497] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1977),
  [1499] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1211),
  [1501] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1973),
  [1503] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1629),
  [1505] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1313),
  [1507] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1263),
  [1509] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1634),
  [1511] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2491),
  [1513] = {.entry = {.count = 1, .reusable = true}}, SHIFT(430),
  [1515] = {.entry = {.count = 1, .reusable = true}}, SHIFT(792),
  [1517] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2136),
  [1519] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2096),
  [1521] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3426),
  [1523] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2938),
  [1525] = {.entry = {.count = 1, .reusable = false}}, SHIFT(811),
  [1527] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3127),
  [1529] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2430),
  [1531] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2500),
  [1533] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2495),
  [1535] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2433),
  [1537] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2476),
  [1539] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2612),
  [1541] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1967),
  [1543] = {.entry = {.count = 1, .reusable = true}}, SHIFT(419),
  [1545] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2098),
  [1547] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2153),
  [1549] = {.entry = {.count = 1, .reusable = true}}, SHIFT(432),
  [1551] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3414),
  [1553] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3201),
  [1555] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1983),
  [1557] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2132),
  [1559] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2729),
  [1561] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2300),
  [1563] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2384),
  [1565] = {.entry = {.count = 1, .reusable = true}}, SHIFT(428),
  [1567] = {.entry = {.count = 1, .reusable = false}}, SHIFT(431),
  [1569] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2377),
  [1571] = {.entry = {.count = 1, .reusable = false}}, SHIFT(425),
  [1573] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2381),
  [1575] = {.entry = {.count = 1, .reusable = false}}, SHIFT(429),
  [1577] = {.entry = {.count = 1, .reusable = false}}, SHIFT(434),
  [1579] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2406),
  [1581] = {.entry = {.count = 1, .reusable = false}}, SHIFT(433),
  [1583] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_label, 2, 0, 0),
  [1585] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_label, 2, 0, 0),
  [1587] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_closure_parameters, 3, 0, 0),
  [1589] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_closure_parameters, 3, 0, 0),
  [1591] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_closure_parameters, 2, 0, 0),
  [1593] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_closure_parameters, 2, 0, 0),
  [1595] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2063),
  [1597] = {.entry = {.count = 1, .reusable = true}}, SHIFT(846),
  [1599] = {.entry = {.count = 1, .reusable = true}}, SHIFT(917),
  [1601] = {.entry = {.count = 1, .reusable = true}}, SHIFT(15),
  [1603] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1957),
  [1605] = {.entry = {.count = 1, .reusable = true}}, SHIFT(848),
  [1607] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1262),
  [1609] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3192),
  [1611] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3435),
  [1613] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2097),
  [1615] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2659),
  [1617] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2814),
  [1619] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3361),
  [1621] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1970),
  [1623] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1947),
  [1625] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1946),
  [1627] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1578),
  [1629] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1210),
  [1631] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1312),
  [1633] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1949),
  [1635] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1583),
  [1637] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1587),
  [1639] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2342),
  [1641] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_match_arm, 3, 0, 173),
  [1643] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_match_arm, 3, 0, 173),
  [1645] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2032),
  [1647] = {.entry = {.count = 1, .reusable = true}}, SHIFT(762),
  [1649] = {.entry = {.count = 1, .reusable = true}}, SHIFT(780),
  [1651] = {.entry = {.count = 1, .reusable = true}}, SHIFT(482),
  [1653] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2287),
  [1655] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3189),
  [1657] = {.entry = {.count = 1, .reusable = true}}, SHIFT(823),
  [1659] = {.entry = {.count = 1, .reusable = true}}, SHIFT(829),
  [1661] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2757),
  [1663] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3021),
  [1665] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3182),
  [1667] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3329),
  [1669] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2548),
  [1671] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2139),
  [1673] = {.entry = {.count = 1, .reusable = false}}, SHIFT(825),
  [1675] = {.entry = {.count = 1, .reusable = false}}, SHIFT(806),
  [1677] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2364),
  [1679] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2598),
  [1681] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2389),
  [1683] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2665),
  [1685] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2665),
  [1687] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3525),
  [1689] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_match_arm, 4, 0, 117),
  [1691] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_match_arm, 4, 0, 117),
  [1693] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1774),
  [1695] = {.entry = {.count = 1, .reusable = true}}, SHIFT(391),
  [1697] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1391),
  [1699] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_associated_type, 4, 0, 85),
  [1701] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_associated_type, 4, 0, 85),
  [1703] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 211),
  [1705] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 211),
  [1707] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 212),
  [1709] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 212),
  [1711] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 213),
  [1713] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 213),
  [1715] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 165),
  [1717] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 165),
  [1719] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 214),
  [1721] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 214),
  [1723] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 167),
  [1725] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 167),
  [1727] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 215),
  [1729] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 215),
  [1731] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 7, 0, 216),
  [1733] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 7, 0, 216),
  [1735] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 7, 0, 217),
  [1737] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 7, 0, 217),
  [1739] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 7, 0, 218),
  [1741] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 7, 0, 218),
  [1743] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 6, 0, 162),
  [1745] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 6, 0, 162),
  [1747] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_static_item, 7, 0, 219),
  [1749] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_static_item, 7, 0, 219),
  [1751] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_static_item, 7, 0, 202),
  [1753] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_static_item, 7, 0, 202),
  [1755] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_declaration_list, 5, 0, 0),
  [1757] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_declaration_list, 5, 0, 0),
  [1759] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 7, 0, 181),
  [1761] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 7, 0, 181),
  [1763] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 222),
  [1765] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 222),
  [1767] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 223),
  [1769] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 223),
  [1771] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 224),
  [1773] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 224),
  [1775] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 225),
  [1777] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 225),
  [1779] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 226),
  [1781] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 226),
  [1783] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 227),
  [1785] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 227),
  [1787] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 228),
  [1789] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 228),
  [1791] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 229),
  [1793] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 229),
  [1795] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 7, 0, 230),
  [1797] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 7, 0, 230),
  [1799] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 7, 0, 189),
  [1801] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 7, 0, 189),
  [1803] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 7, 0, 231),
  [1805] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 7, 0, 231),
  [1807] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 7, 0, 232),
  [1809] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 7, 0, 232),
  [1811] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 7, 0, 233),
  [1813] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 7, 0, 233),
  [1815] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 7, 0, 138),
  [1817] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 7, 0, 138),
  [1819] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 7, 0, 192),
  [1821] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 7, 0, 192),
  [1823] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 7, 0, 234),
  [1825] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 7, 0, 234),
  [1827] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 7, 0, 235),
  [1829] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 7, 0, 235),
  [1831] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 7, 0, 236),
  [1833] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 7, 0, 236),
  [1835] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 7, 0, 237),
  [1837] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 7, 0, 237),
  [1839] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_extern_crate_declaration, 7, 0, 238),
  [1841] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_extern_crate_declaration, 7, 0, 238),
  [1843] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 7, 0, 239),
  [1845] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 7, 0, 239),
  [1847] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 7, 0, 240),
  [1849] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 7, 0, 240),
  [1851] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 7, 0, 196),
  [1853] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 7, 0, 196),
  [1855] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 7, 0, 241),
  [1857] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 7, 0, 241),
  [1859] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_variant_list, 6, 0, 0),
  [1861] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_variant_list, 6, 0, 0),
  [1863] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 8, 0, 205),
  [1865] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 8, 0, 205),
  [1867] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 8, 0, 245),
  [1869] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 8, 0, 245),
  [1871] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 210),
  [1873] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 210),
  [1875] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 246),
  [1877] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 246),
  [1879] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 212),
  [1881] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 212),
  [1883] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 247),
  [1885] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 247),
  [1887] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 8, 0, 248),
  [1889] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 8, 0, 248),
  [1891] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 8, 0, 249),
  [1893] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 8, 0, 249),
  [1895] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 8, 0, 250),
  [1897] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 8, 0, 250),
  [1899] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_definition, 4, 0, 54),
  [1901] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_definition, 4, 0, 54),
  [1903] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_static_item, 8, 0, 251),
  [1905] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_static_item, 8, 0, 251),
  [1907] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_declaration_list, 6, 0, 0),
  [1909] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_declaration_list, 6, 0, 0),
  [1911] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 222),
  [1913] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 222),
  [1915] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 253),
  [1917] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 253),
  [1919] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 224),
  [1921] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 224),
  [1923] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 254),
  [1925] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 254),
  [1927] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 255),
  [1929] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 255),
  [1931] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 256),
  [1933] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 256),
  [1935] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 257),
  [1937] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 257),
  [1939] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 258),
  [1941] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 258),
  [1943] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 226),
  [1945] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 226),
  [1947] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 259),
  [1949] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 259),
  [1951] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 228),
  [1953] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 228),
  [1955] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 260),
  [1957] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 260),
  [1959] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 8, 0, 261),
  [1961] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 8, 0, 261),
  [1963] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 8, 0, 262),
  [1965] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 8, 0, 262),
  [1967] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 8, 0, 232),
  [1969] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 8, 0, 232),
  [1971] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 8, 0, 263),
  [1973] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 8, 0, 263),
  [1975] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_const_item, 8, 0, 251),
  [1977] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_const_item, 8, 0, 251),
  [1979] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_static_item, 8, 0, 264),
  [1981] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_static_item, 8, 0, 264),
  [1983] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_definition, 4, 0, 6),
  [1985] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_definition, 4, 0, 6),
  [1987] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 8, 0, 234),
  [1989] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 8, 0, 234),
  [1991] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 8, 0, 265),
  [1993] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 8, 0, 265),
  [1995] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 8, 0, 239),
  [1997] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 8, 0, 239),
  [1999] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 8, 0, 266),
  [2001] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 8, 0, 266),
  [2003] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 8, 0, 267),
  [2005] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 8, 0, 267),
  [2007] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 8, 0, 268),
  [2009] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 8, 0, 268),
  [2011] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 9, 0, 269),
  [2013] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 9, 0, 269),
  [2015] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_static_item, 9, 0, 270),
  [2017] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_static_item, 9, 0, 270),
  [2019] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 255),
  [2021] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 255),
  [2023] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 271),
  [2025] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 271),
  [2027] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 257),
  [2029] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 257),
  [2031] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 272),
  [2033] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 272),
  [2035] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 9, 0, 261),
  [2037] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 9, 0, 261),
  [2039] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 9, 0, 273),
  [2041] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 9, 0, 273),
  [2043] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 9, 0, 274),
  [2045] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 9, 0, 274),
  [2047] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 9, 0, 275),
  [2049] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 9, 0, 275),
  [2051] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 9, 0, 267),
  [2053] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 9, 0, 267),
  [2055] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 9, 0, 276),
  [2057] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 9, 0, 276),
  [2059] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 10, 0, 277),
  [2061] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 10, 0, 277),
  [2063] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_static_item, 10, 0, 278),
  [2065] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_static_item, 10, 0, 278),
  [2067] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 10, 0, 274),
  [2069] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 10, 0, 274),
  [2071] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 10, 0, 279),
  [2073] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 10, 0, 279),
  [2075] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 6, 0, 163),
  [2077] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 6, 0, 163),
  [2079] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 6, 0, 121),
  [2081] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 6, 0, 121),
  [2083] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 6, 0, 164),
  [2085] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 6, 0, 164),
  [2087] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 6, 0, 165),
  [2089] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 6, 0, 165),
  [2091] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 6, 0, 166),
  [2093] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 6, 0, 166),
  [2095] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 6, 0, 167),
  [2097] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 6, 0, 167),
  [2099] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_attribute_item, 4, 0, 0),
  [2101] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_attribute_item, 4, 0, 0),
  [2103] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 6, 0, 168),
  [2105] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 6, 0, 168),
  [2107] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 6, 0, 124),
  [2109] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 6, 0, 124),
  [2111] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_variant_list, 2, 0, 0),
  [2113] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_variant_list, 2, 0, 0),
  [2115] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 6, 0, 169),
  [2117] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 6, 0, 169),
  [2119] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 6, 0, 170),
  [2121] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 6, 0, 170),
  [2123] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_item, 4, 0, 72),
  [2125] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_item, 4, 0, 72),
  [2127] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_item, 4, 0, 73),
  [2129] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_item, 4, 0, 73),
  [2131] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 3, 0, 29),
  [2133] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 3, 0, 29),
  [2135] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 4, 0, 74),
  [2137] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 4, 0, 74),
  [2139] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 4, 0, 75),
  [2141] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 4, 0, 75),
  [2143] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 6, 0, 171),
  [2145] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 6, 0, 171),
  [2147] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 6, 0, 172),
  [2149] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 6, 0, 172),
  [2151] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 4, 0, 80),
  [2153] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 4, 0, 80),
  [2155] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 6, 0, 157),
  [2157] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 6, 0, 157),
  [2159] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_associated_type, 3, 0, 7),
  [2161] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_associated_type, 3, 0, 7),
  [2163] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 4, 0, 81),
  [2165] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 4, 0, 81),
  [2167] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_static_item, 6, 0, 175),
  [2169] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_static_item, 6, 0, 175),
  [2171] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 4, 0, 22),
  [2173] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 4, 0, 22),
  [2175] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 4, 0, 82),
  [2177] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 4, 0, 82),
  [2179] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 4, 0, 83),
  [2181] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 4, 0, 83),
  [2183] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_union_item, 3, 0, 29),
  [2185] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_union_item, 3, 0, 29),
  [2187] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_declaration_list, 2, 0, 0),
  [2189] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_declaration_list, 2, 0, 0),
  [2191] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 4, 0, 29),
  [2193] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 4, 0, 29),
  [2195] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 4, 0, 72),
  [2197] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 4, 0, 72),
  [2199] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_declaration_list, 4, 0, 0),
  [2201] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_declaration_list, 4, 0, 0),
  [2203] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 4, 0, 85),
  [2205] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 4, 0, 85),
  [2207] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 6, 0, 73),
  [2209] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 6, 0, 73),
  [2211] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 4, 0, 73),
  [2213] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 4, 0, 73),
  [2215] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 6, 0, 180),
  [2217] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 6, 0, 180),
  [2219] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 6, 0, 131),
  [2221] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 6, 0, 131),
  [2223] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 6, 0, 181),
  [2225] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 6, 0, 181),
  [2227] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_associated_type, 6, 0, 132),
  [2229] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_associated_type, 6, 0, 132),
  [2231] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 6, 0, 133),
  [2233] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 6, 0, 133),
  [2235] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 6, 0, 182),
  [2237] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 6, 0, 182),
  [2239] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 6, 0, 183),
  [2241] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 6, 0, 183),
  [2243] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 6, 0, 184),
  [2245] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 6, 0, 184),
  [2247] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 6, 0, 185),
  [2249] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 6, 0, 185),
  [2251] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 4, 0, 72),
  [2253] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 4, 0, 72),
  [2255] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 4, 0, 86),
  [2257] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 4, 0, 86),
  [2259] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 4, 0, 73),
  [2261] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 4, 0, 73),
  [2263] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_associated_type, 4, 0, 7),
  [2265] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_associated_type, 4, 0, 7),
  [2267] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 6, 0, 113),
  [2269] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 6, 0, 113),
  [2271] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_union_item, 4, 0, 72),
  [2273] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_union_item, 4, 0, 72),
  [2275] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_union_item, 4, 0, 73),
  [2277] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_union_item, 4, 0, 73),
  [2279] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_extern_crate_declaration, 6, 0, 186),
  [2281] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_extern_crate_declaration, 6, 0, 186),
  [2283] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 4, 0, 64),
  [2285] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 4, 0, 64),
  [2287] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 4, 0, 88),
  [2289] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 4, 0, 88),
  [2291] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 4, 0, 89),
  [2293] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 4, 0, 89),
  [2295] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_extern_crate_declaration, 4, 0, 94),
  [2297] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_extern_crate_declaration, 4, 0, 94),
  [2299] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_declaration_list, 3, 0, 0),
  [2301] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_declaration_list, 3, 0, 0),
  [2303] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(3165),
  [2306] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1080),
  [2309] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2735),
  [2312] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0),
  [2314] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(3545),
  [2317] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(857),
  [2320] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(3250),
  [2323] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(3306),
  [2326] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2387),
  [2329] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2265),
  [2332] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2236),
  [2335] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(3633),
  [2338] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(3307),
  [2341] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(3135),
  [2344] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(858),
  [2347] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(818),
  [2350] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(3343),
  [2353] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1972),
  [2356] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2944),
  [2359] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(3344),
  [2362] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(3345),
  [2365] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(3346),
  [2368] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2950),
  [2371] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2238),
  [2374] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1891),
  [2377] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2088),
  [2380] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(3353),
  [2383] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2013),
  [2386] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(3353),
  [2389] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_item, 4, 0, 89),
  [2391] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_item, 4, 0, 89),
  [2393] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_mod_item, 4, 0, 94),
  [2395] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_mod_item, 4, 0, 94),
  [2397] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_mod_item, 4, 0, 95),
  [2399] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_mod_item, 4, 0, 95),
  [2401] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 4, 0, 96),
  [2403] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 4, 0, 96),
  [2405] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 4, 0, 89),
  [2407] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 4, 0, 89),
  [2409] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_union_item, 4, 0, 89),
  [2411] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_union_item, 4, 0, 89),
  [2413] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_declaration, 4, 0, 97),
  [2415] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_use_declaration, 4, 0, 97),
  [2417] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 6, 0, 189),
  [2419] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 6, 0, 189),
  [2421] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 6, 0, 190),
  [2423] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 6, 0, 190),
  [2425] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_definition, 5, 0, 54),
  [2427] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_definition, 5, 0, 54),
  [2429] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 6, 0, 141),
  [2431] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 6, 0, 141),
  [2433] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 6, 0, 191),
  [2435] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 6, 0, 191),
  [2437] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_const_item, 6, 0, 175),
  [2439] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_const_item, 6, 0, 175),
  [2441] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_item, 6, 0, 184),
  [2443] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_item, 6, 0, 184),
  [2445] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_definition, 5, 0, 6),
  [2447] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_definition, 5, 0, 6),
  [2449] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 1, 0, 0),
  [2451] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 1, 0, 0),
  [2453] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 6, 0, 89),
  [2455] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 6, 0, 89),
  [2457] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 6, 0, 138),
  [2459] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 6, 0, 138),
  [2461] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3165),
  [2463] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1080),
  [2465] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2735),
  [2467] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1110),
  [2469] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3545),
  [2471] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3250),
  [2473] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3306),
  [2475] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2265),
  [2477] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2236),
  [2479] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3633),
  [2481] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3307),
  [2483] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3135),
  [2485] = {.entry = {.count = 1, .reusable = false}}, SHIFT(858),
  [2487] = {.entry = {.count = 1, .reusable = false}}, SHIFT(818),
  [2489] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3343),
  [2491] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2944),
  [2493] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3344),
  [2495] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3345),
  [2497] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3346),
  [2499] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2950),
  [2501] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2238),
  [2503] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1891),
  [2505] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2088),
  [2507] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3353),
  [2509] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2013),
  [2511] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3353),
  [2513] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 6, 0, 184),
  [2515] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 6, 0, 184),
  [2517] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 6, 0, 192),
  [2519] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 6, 0, 192),
  [2521] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_union_item, 6, 0, 184),
  [2523] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_union_item, 6, 0, 184),
  [2525] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1146),
  [2527] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_inner_attribute_item, 5, 0, 0),
  [2529] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_inner_attribute_item, 5, 0, 0),
  [2531] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_const_item, 5, 0, 111),
  [2533] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_const_item, 5, 0, 111),
  [2535] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_item, 3, 0, 29),
  [2537] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_item, 3, 0, 29),
  [2539] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_foreign_mod_item, 2, 0, 0),
  [2541] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_foreign_mod_item, 2, 0, 0),
  [2543] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_variant_list, 3, 0, 0),
  [2545] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_variant_list, 3, 0, 0),
  [2547] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 6, 0, 193),
  [2549] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 6, 0, 193),
  [2551] = {.entry = {.count = 1, .reusable = true}}, SHIFT(693),
  [2553] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_item, 5, 0, 112),
  [2555] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_item, 5, 0, 112),
  [2557] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 5, 0, 113),
  [2559] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 5, 0, 113),
  [2561] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 5, 0, 114),
  [2563] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 5, 0, 114),
  [2565] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 5, 0, 74),
  [2567] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 5, 0, 74),
  [2569] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 5, 0, 115),
  [2571] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 5, 0, 115),
  [2573] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_foreign_mod_item, 2, 0, 8),
  [2575] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_foreign_mod_item, 2, 0, 8),
  [2577] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 6, 0, 194),
  [2579] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 6, 0, 194),
  [2581] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 6, 0, 195),
  [2583] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 6, 0, 195),
  [2585] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 6, 0, 196),
  [2587] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 6, 0, 196),
  [2589] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 5, 0, 121),
  [2591] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 5, 0, 121),
  [2593] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 5, 0, 122),
  [2595] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 5, 0, 122),
  [2597] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 5, 0, 80),
  [2599] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 5, 0, 80),
  [2601] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 6, 0, 197),
  [2603] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 6, 0, 197),
  [2605] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_declaration, 3, 0, 37),
  [2607] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_use_declaration, 3, 0, 37),
  [2609] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 5, 0, 123),
  [2611] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 5, 0, 123),
  [2613] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 5, 0, 124),
  [2615] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 5, 0, 124),
  [2617] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 5, 0, 125),
  [2619] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 5, 0, 125),
  [2621] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 5, 0, 101),
  [2623] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 5, 0, 101),
  [2625] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_declaration_list, 2, 0, 0),
  [2627] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_declaration_list, 2, 0, 0),
  [2629] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_expression_statement, 2, 0, 0),
  [2631] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_expression_statement, 2, 0, 0),
  [2633] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 5, 0, 117),
  [2635] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 5, 0, 117),
  [2637] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 5, 0, 126),
  [2639] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 5, 0, 126),
  [2641] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 3, 0, 22),
  [2643] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 3, 0, 22),
  [2645] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 3, 0, 31),
  [2647] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 3, 0, 31),
  [2649] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_match_block_repeat1, 2, 0, 0), SHIFT_REPEAT(2032),
  [2652] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_match_block_repeat1, 2, 0, 0), SHIFT_REPEAT(762),
  [2655] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_match_block_repeat1, 2, 0, 0), SHIFT_REPEAT(780),
  [2658] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_match_block_repeat1, 2, 0, 0), SHIFT_REPEAT(2287),
  [2661] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_match_block_repeat1, 2, 0, 0), SHIFT_REPEAT(3189),
  [2664] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_match_block_repeat1, 2, 0, 0), SHIFT_REPEAT(823),
  [2667] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_match_block_repeat1, 2, 0, 0), SHIFT_REPEAT(829),
  [2670] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_match_block_repeat1, 2, 0, 0), SHIFT_REPEAT(857),
  [2673] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_match_block_repeat1, 2, 0, 0), SHIFT_REPEAT(2757),
  [2676] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_match_block_repeat1, 2, 0, 0), SHIFT_REPEAT(3021),
  [2679] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_match_block_repeat1, 2, 0, 0), SHIFT_REPEAT(3182),
  [2682] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_match_block_repeat1, 2, 0, 0), SHIFT_REPEAT(3329),
  [2685] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_match_block_repeat1, 2, 0, 0), SHIFT_REPEAT(2548),
  [2688] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_match_block_repeat1, 2, 0, 0), SHIFT_REPEAT(2139),
  [2691] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_match_block_repeat1, 2, 0, 0), SHIFT_REPEAT(825),
  [2694] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_match_block_repeat1, 2, 0, 0), SHIFT_REPEAT(806),
  [2697] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_match_block_repeat1, 2, 0, 0), SHIFT_REPEAT(2364),
  [2700] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_match_block_repeat1, 2, 0, 0), SHIFT_REPEAT(2598),
  [2703] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_match_block_repeat1, 2, 0, 0), SHIFT_REPEAT(2389),
  [2706] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_match_block_repeat1, 2, 0, 0), SHIFT_REPEAT(2665),
  [2709] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_match_block_repeat1, 2, 0, 0), SHIFT_REPEAT(2665),
  [2712] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_match_block_repeat1, 2, 0, 0), SHIFT_REPEAT(3525),
  [2715] = {.entry = {.count = 1, .reusable = true}}, SHIFT(638),
  [2717] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_foreign_mod_item, 3, 0, 0),
  [2719] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_foreign_mod_item, 3, 0, 0),
  [2721] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_static_item, 5, 0, 111),
  [2723] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_static_item, 5, 0, 111),
  [2725] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_foreign_mod_item, 3, 0, 44),
  [2727] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_foreign_mod_item, 3, 0, 44),
  [2729] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_definition, 7, 0, 54),
  [2731] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_definition, 7, 0, 54),
  [2733] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_declaration_list, 3, 0, 0),
  [2735] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_declaration_list, 3, 0, 0),
  [2737] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_definition, 7, 0, 6),
  [2739] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_definition, 7, 0, 6),
  [2741] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 6, 0, 158),
  [2743] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 6, 0, 158),
  [2745] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 6, 0, 159),
  [2747] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 6, 0, 159),
  [2749] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 5, 0, 29),
  [2751] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 5, 0, 29),
  [2753] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 5, 0, 73),
  [2755] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 5, 0, 73),
  [2757] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 5, 0, 112),
  [2759] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 5, 0, 112),
  [2761] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 5, 0, 129),
  [2763] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 5, 0, 129),
  [2765] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 5, 0, 112),
  [2767] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 5, 0, 112),
  [2769] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 5, 0, 130),
  [2771] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 5, 0, 130),
  [2773] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 5, 0, 131),
  [2775] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 5, 0, 131),
  [2777] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_associated_type, 5, 0, 87),
  [2779] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_associated_type, 5, 0, 87),
  [2781] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_associated_type, 5, 0, 85),
  [2783] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_associated_type, 5, 0, 85),
  [2785] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_associated_type, 5, 0, 132),
  [2787] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_associated_type, 5, 0, 132),
  [2789] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_union_item, 5, 0, 112),
  [2791] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_union_item, 5, 0, 112),
  [2793] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 5, 0, 133),
  [2795] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 5, 0, 133),
  [2797] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 5, 0, 134),
  [2799] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 5, 0, 134),
  [2801] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 5, 0, 64),
  [2803] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 5, 0, 64),
  [2805] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 5, 0, 135),
  [2807] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 5, 0, 135),
  [2809] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 5, 0, 136),
  [2811] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 5, 0, 136),
  [2813] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 5, 0, 137),
  [2815] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 5, 0, 137),
  [2817] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 5, 0, 138),
  [2819] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 5, 0, 138),
  [2821] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 3, 0, 32),
  [2823] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 3, 0, 32),
  [2825] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 5, 0, 141),
  [2827] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 5, 0, 141),
  [2829] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 5, 0, 142),
  [2831] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 5, 0, 142),
  [2833] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_item, 5, 0, 136),
  [2835] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_item, 5, 0, 136),
  [2837] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_item, 5, 0, 138),
  [2839] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_item, 5, 0, 138),
  [2841] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 5, 0, 89),
  [2843] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 5, 0, 89),
  [2845] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 5, 0, 136),
  [2847] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 5, 0, 136),
  [2849] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 5, 0, 143),
  [2851] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 5, 0, 143),
  [2853] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 5, 0, 138),
  [2855] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 5, 0, 138),
  [2857] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_empty_statement, 1, 0, 0),
  [2859] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_empty_statement, 1, 0, 0),
  [2861] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_union_item, 5, 0, 136),
  [2863] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_union_item, 5, 0, 136),
  [2865] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_union_item, 5, 0, 138),
  [2867] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_union_item, 5, 0, 138),
  [2869] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 5, 0, 144),
  [2871] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 5, 0, 144),
  [2873] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 6, 0, 160),
  [2875] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 6, 0, 160),
  [2877] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_extern_crate_declaration, 5, 0, 145),
  [2879] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_extern_crate_declaration, 5, 0, 145),
  [2881] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 6, 0, 161),
  [2883] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 6, 0, 161),
  [2885] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_mod_item, 3, 0, 6),
  [2887] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_mod_item, 3, 0, 6),
  [2889] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_mod_item, 3, 0, 34),
  [2891] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_mod_item, 3, 0, 34),
  [2893] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_const_item, 7, 0, 202),
  [2895] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_const_item, 7, 0, 202),
  [2897] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_variant_list, 5, 0, 0),
  [2899] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_variant_list, 5, 0, 0),
  [2901] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 7, 0, 205),
  [2903] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 7, 0, 205),
  [2905] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_definition, 6, 0, 54),
  [2907] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_definition, 6, 0, 54),
  [2909] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 3, 0, 7),
  [2911] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 3, 0, 7),
  [2913] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_definition, 6, 0, 6),
  [2915] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_definition, 6, 0, 6),
  [2917] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 7, 0, 206),
  [2919] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 7, 0, 206),
  [2921] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 7, 0, 158),
  [2923] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 7, 0, 158),
  [2925] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 7, 0, 207),
  [2927] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 7, 0, 207),
  [2929] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 160),
  [2931] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 160),
  [2933] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 208),
  [2935] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 208),
  [2937] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 162),
  [2939] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 162),
  [2941] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 209),
  [2943] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 209),
  [2945] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 3, 0, 29),
  [2947] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 3, 0, 29),
  [2949] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 210),
  [2951] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 210),
  [2953] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_variant_list, 4, 0, 0),
  [2955] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_variant_list, 4, 0, 0),
  [2957] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_associated_type, 4, 0, 87),
  [2959] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_associated_type, 4, 0, 87),
  [2961] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1931),
  [2963] = {.entry = {.count = 1, .reusable = true}}, SHIFT(763),
  [2965] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2915),
  [2967] = {.entry = {.count = 1, .reusable = true}}, SHIFT(786),
  [2969] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2017),
  [2971] = {.entry = {.count = 1, .reusable = true}}, SHIFT(810),
  [2973] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3471),
  [2975] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3272),
  [2977] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2654),
  [2979] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1951),
  [2981] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2537),
  [2983] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2537),
  [2985] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2131),
  [2987] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2104),
  [2989] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2770),
  [2991] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2975),
  [2993] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2107),
  [2995] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2018),
  [2997] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3566),
  [2999] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3604),
  [3001] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2258),
  [3003] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2334),
  [3005] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1922),
  [3007] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2480),
  [3009] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3613),
  [3011] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3472),
  [3013] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1537),
  [3015] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1532),
  [3017] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2506),
  [3019] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2464),
  [3021] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2468),
  [3023] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2471),
  [3025] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2505),
  [3027] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2445),
  [3029] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2919),
  [3031] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3477),
  [3033] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2137),
  [3035] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3446),
  [3037] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2128),
  [3039] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3572),
  [3041] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2873),
  [3043] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3354),
  [3045] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2884),
  [3047] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3410),
  [3049] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2135),
  [3051] = {.entry = {.count = 1, .reusable = true}}, SHIFT(438),
  [3053] = {.entry = {.count = 1, .reusable = false}}, SHIFT(824),
  [3055] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2509),
  [3057] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2772),
  [3059] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3258),
  [3061] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2289),
  [3063] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2108),
  [3065] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2978),
  [3067] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2984),
  [3069] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3008),
  [3071] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2113),
  [3073] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2986),
  [3075] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2972),
  [3077] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2126),
  [3079] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2103),
  [3081] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2114),
  [3083] = {.entry = {.count = 1, .reusable = false}}, SHIFT(807),
  [3085] = {.entry = {.count = 1, .reusable = false}}, SHIFT(813),
  [3087] = {.entry = {.count = 1, .reusable = false}}, SHIFT(822),
  [3089] = {.entry = {.count = 1, .reusable = false}}, SHIFT(826),
  [3091] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2510),
  [3093] = {.entry = {.count = 1, .reusable = true}}, SHIFT(943),
  [3095] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1001),
  [3097] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1020),
  [3099] = {.entry = {.count = 1, .reusable = true}}, SHIFT(863),
  [3101] = {.entry = {.count = 1, .reusable = true}}, SHIFT(938),
  [3103] = {.entry = {.count = 1, .reusable = true}}, SHIFT(886),
  [3105] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3200),
  [3107] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1000),
  [3109] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1081),
  [3111] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1414),
  [3113] = {.entry = {.count = 1, .reusable = true}}, SHIFT(856),
  [3115] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3155),
  [3117] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2110),
  [3119] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3210),
  [3121] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2668),
  [3123] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1047),
  [3125] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1500),
  [3127] = {.entry = {.count = 1, .reusable = false}}, SHIFT(888),
  [3129] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3544),
  [3131] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1082),
  [3133] = {.entry = {.count = 1, .reusable = false}}, SHIFT(999),
  [3135] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3039),
  [3137] = {.entry = {.count = 1, .reusable = false}}, SHIFT(914),
  [3139] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1512),
  [3141] = {.entry = {.count = 1, .reusable = true}}, SHIFT(849),
  [3143] = {.entry = {.count = 1, .reusable = true}}, SHIFT(992),
  [3145] = {.entry = {.count = 1, .reusable = true}}, SHIFT(993),
  [3147] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3340),
  [3149] = {.entry = {.count = 1, .reusable = true}}, SHIFT(894),
  [3151] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1633),
  [3153] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1682),
  [3155] = {.entry = {.count = 1, .reusable = true}}, SHIFT(859),
  [3157] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3206),
  [3159] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2112),
  [3161] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3080),
  [3163] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2553),
  [3165] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1058),
  [3167] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1498),
  [3169] = {.entry = {.count = 1, .reusable = false}}, SHIFT(873),
  [3171] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3559),
  [3173] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1644),
  [3175] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2091),
  [3177] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1905),
  [3179] = {.entry = {.count = 1, .reusable = true}}, SHIFT(770),
  [3181] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3064),
  [3183] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3078),
  [3185] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1406),
  [3187] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1996),
  [3189] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1457),
  [3191] = {.entry = {.count = 1, .reusable = false}}, SHIFT(927),
  [3193] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1692),
  [3195] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1727),
  [3197] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2001),
  [3199] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1734),
  [3201] = {.entry = {.count = 1, .reusable = false}}, SHIFT(872),
  [3203] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2054),
  [3205] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1907),
  [3207] = {.entry = {.count = 1, .reusable = false}}, SHIFT(949),
  [3209] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2086),
  [3211] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1903),
  [3213] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2062),
  [3215] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1906),
  [3217] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2011),
  [3219] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1435),
  [3221] = {.entry = {.count = 1, .reusable = false}}, SHIFT(869),
  [3223] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3171),
  [3225] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2094),
  [3227] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1901),
  [3229] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2090),
  [3231] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1908),
  [3233] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2092),
  [3235] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1904),
  [3237] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2055),
  [3239] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1902),
  [3241] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2941),
  [3243] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0),
  [3245] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_enum_variant_list_repeat1, 1, 0, 0),
  [3247] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_enum_variant_list_repeat1, 1, 0, 0),
  [3249] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_closure_parameters, 4, 0, 0),
  [3251] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_closure_parameters, 4, 0, 0),
  [3253] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0),
  [3255] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_lifetime, 2, 0, 0),
  [3257] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_lifetime, 2, 0, 0),
  [3259] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2076),
  [3261] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_range_pattern, 2, 0, 0),
  [3263] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_range_pattern, 2, 0, 0),
  [3265] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2095),
  [3267] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2060),
  [3269] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2060),
  [3271] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2083),
  [3273] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_range_pattern, 2, 0, 1),
  [3275] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_range_pattern, 2, 0, 1),
  [3277] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2080),
  [3279] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2056),
  [3281] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2056),
  [3283] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__expression_except_range, 1, 0, 1),
  [3285] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__expression_except_range, 1, 0, 1),
  [3287] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2677),
  [3289] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3237),
  [3291] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3586),
  [3293] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2679),
  [3295] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__type, 1, 0, 5),
  [3297] = {.entry = {.count = 1, .reusable = true}}, SHIFT(394),
  [3299] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__type, 1, 0, 5),
  [3301] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2703),
  [3303] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2625),
  [3305] = {.entry = {.count = 1, .reusable = true}}, SHIFT(450),
  [3307] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_abstract_type, 2, 0, 24),
  [3309] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_abstract_type, 2, 0, 24),
  [3311] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2643),
  [3313] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_abstract_type, 4, 0, 105),
  [3315] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_abstract_type, 4, 0, 105),
  [3317] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_dynamic_type, 2, 0, 24),
  [3319] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_dynamic_type, 2, 0, 24),
  [3321] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__type, 1, 0, 0),
  [3323] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__type, 1, 0, 0),
  [3325] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_dynamic_type, 2, 0, 25),
  [3327] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_dynamic_type, 2, 0, 25),
  [3329] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_scoped_type_identifier, 3, 0, 17),
  [3331] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_scoped_type_identifier, 3, 0, 17),
  [3333] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_scoped_identifier, 3, 0, 16),
  [3335] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_scoped_identifier, 3, 0, 16),
  [3337] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_abstract_type, 4, 0, 106),
  [3339] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_abstract_type, 4, 0, 106),
  [3341] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_scoped_type_identifier, 3, 0, 41),
  [3343] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_scoped_type_identifier, 3, 0, 41),
  [3345] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_scoped_identifier, 3, 0, 40),
  [3347] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_scoped_identifier, 3, 0, 40),
  [3349] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_scoped_type_identifier, 3, 0, 46),
  [3351] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_scoped_type_identifier, 3, 0, 46),
  [3353] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_scoped_identifier, 3, 0, 45),
  [3355] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_scoped_identifier, 3, 0, 45),
  [3357] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_abstract_type, 2, 0, 25),
  [3359] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_abstract_type, 2, 0, 25),
  [3361] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_match_arm_repeat1, 2, 0, 0),
  [3363] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_match_arm_repeat1, 2, 0, 0),
  [3365] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_match_arm_repeat1, 2, 0, 0), SHIFT_REPEAT(3329),
  [3368] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_scoped_type_identifier, 2, 0, 7),
  [3370] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_scoped_type_identifier, 2, 0, 7),
  [3372] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_scoped_identifier, 2, 0, 6),
  [3374] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_scoped_identifier, 2, 0, 6),
  [3376] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1022),
  [3378] = {.entry = {.count = 1, .reusable = true}}, SHIFT(877),
  [3380] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3397),
  [3382] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3190),
  [3384] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2173),
  [3386] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3234),
  [3388] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2914),
  [3390] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3556),
  [3392] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3556),
  [3394] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1927),
  [3396] = {.entry = {.count = 1, .reusable = true}}, SHIFT(932),
  [3398] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3443),
  [3400] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3076),
  [3402] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2164),
  [3404] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3124),
  [3406] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2939),
  [3408] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3499),
  [3410] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3499),
  [3412] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1530),
  [3414] = {.entry = {.count = 1, .reusable = true}}, SHIFT(896),
  [3416] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3511),
  [3418] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3209),
  [3420] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2184),
  [3422] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2946),
  [3424] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3560),
  [3426] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3560),
  [3428] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2490),
  [3430] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_where_clause, 3, 0, 0),
  [3432] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3222),
  [3434] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3013),
  [3436] = {.entry = {.count = 1, .reusable = true}}, SHIFT(864),
  [3438] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3617),
  [3440] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3282),
  [3442] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1929),
  [3444] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1028),
  [3446] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_where_clause, 4, 0, 0),
  [3448] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2085),
  [3450] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2751),
  [3452] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1536),
  [3454] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3291),
  [3456] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2581),
  [3458] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_function_modifiers_repeat1, 1, 0, 0),
  [3460] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0), SHIFT_REPEAT(3566),
  [3463] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2525),
  [3465] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_parameters, 4, 0, 120),
  [3467] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_parameters, 4, 0, 120),
  [3469] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_scoped_type_identifier_in_expression_position, 3, 0, 17),
  [3471] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_parameters, 5, 0, 77),
  [3473] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_parameters, 5, 0, 77),
  [3475] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_parameters, 5, 0, 0),
  [3477] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_parameters, 5, 0, 0),
  [3479] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_parameters, 5, 0, 120),
  [3481] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_parameters, 5, 0, 120),
  [3483] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_parameters, 6, 0, 120),
  [3485] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_parameters, 6, 0, 120),
  [3487] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_parameters, 6, 0, 0),
  [3489] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_parameters, 6, 0, 0),
  [3491] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_match_arm, 4, 0, 173),
  [3493] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_match_arm, 4, 0, 173),
  [3495] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_last_match_arm, 4, 0, 173),
  [3497] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_scoped_type_identifier_in_expression_position, 3, 0, 41),
  [3499] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 1, 0, 0),
  [3501] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_declaration_list_repeat1, 1, 0, 0),
  [3503] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_scoped_type_identifier_in_expression_position, 3, 0, 46),
  [3505] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_match_arm, 5, 0, 117),
  [3507] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_match_arm, 5, 0, 117),
  [3509] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_last_match_arm, 5, 0, 117),
  [3511] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_expression, 3, 0, 49),
  [3513] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_expression, 3, 0, 49),
  [3515] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_expression, 3, 0, 50),
  [3517] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_expression, 3, 0, 50),
  [3519] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_parameters, 4, 0, 77),
  [3521] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_parameters, 4, 0, 77),
  [3523] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_scoped_type_identifier_in_expression_position, 2, 0, 7),
  [3525] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3090),
  [3527] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3161),
  [3529] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3489),
  [3531] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2614),
  [3533] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2594),
  [3535] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3303),
  [3537] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2628),
  [3539] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2628),
  [3541] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_generic_type, 2, 0, 19),
  [3543] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_generic_type, 2, 0, 19),
  [3545] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2699),
  [3547] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2683),
  [3549] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2712),
  [3551] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2712),
  [3553] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_generic_type, 2, 0, 20),
  [3555] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_generic_type, 2, 0, 20),
  [3557] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_generic_type, 2, 0, 21),
  [3559] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_generic_type, 2, 0, 21),
  [3561] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_type, 2, 0, 23),
  [3563] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_type, 2, 0, 23),
  [3565] = {.entry = {.count = 1, .reusable = true}}, SHIFT(890),
  [3567] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_type, 2, 0, 26),
  [3569] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_type, 2, 0, 26),
  [3571] = {.entry = {.count = 1, .reusable = true}}, SHIFT(895),
  [3573] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_type, 2, 0, 27),
  [3575] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_type, 2, 0, 27),
  [3577] = {.entry = {.count = 1, .reusable = true}}, SHIFT(898),
  [3579] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2378),
  [3581] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameters, 2, 0, 0),
  [3583] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parameters, 2, 0, 0),
  [3585] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_type, 3, 0, 65),
  [3587] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_type, 3, 0, 65),
  [3589] = {.entry = {.count = 1, .reusable = true}}, SHIFT(908),
  [3591] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_type, 3, 0, 67),
  [3593] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_type, 3, 0, 67),
  [3595] = {.entry = {.count = 1, .reusable = true}}, SHIFT(910),
  [3597] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_type, 3, 0, 68),
  [3599] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_type, 3, 0, 68),
  [3601] = {.entry = {.count = 1, .reusable = true}}, SHIFT(912),
  [3603] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_arguments, 3, 0, 0),
  [3605] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_arguments, 3, 0, 0),
  [3607] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameters, 3, 0, 0),
  [3609] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parameters, 3, 0, 0),
  [3611] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_type, 4, 0, 108),
  [3613] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_type, 4, 0, 108),
  [3615] = {.entry = {.count = 1, .reusable = true}}, SHIFT(919),
  [3617] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_arguments, 4, 0, 0),
  [3619] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_arguments, 4, 0, 0),
  [3621] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameters, 4, 0, 0),
  [3623] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parameters, 4, 0, 0),
  [3625] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_parameters, 4, 0, 0),
  [3627] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_parameters, 4, 0, 0),
  [3629] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_arguments, 5, 0, 0),
  [3631] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_arguments, 5, 0, 0),
  [3633] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameters, 5, 0, 0),
  [3635] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parameters, 5, 0, 0),
  [3637] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_arguments, 6, 0, 0),
  [3639] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_arguments, 6, 0, 0),
  [3641] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameters, 6, 0, 0),
  [3643] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parameters, 6, 0, 0),
  [3645] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_continue_expression, 1, 0, 0),
  [3647] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_continue_expression, 1, 0, 0),
  [3649] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_parameters, 3, 0, 77),
  [3651] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_parameters, 3, 0, 77),
  [3653] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_parameters, 3, 0, 0),
  [3655] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_parameters, 3, 0, 0),
  [3657] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__expression, 1, 0, 0),
  [3659] = {.entry = {.count = 1, .reusable = true}}, SHIFT(142),
  [3661] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__expression, 1, 0, 0),
  [3663] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2559),
  [3665] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3252),
  [3667] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_generic_function, 3, 0, 42),
  [3669] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_generic_type_with_turbofish, 3, 0, 52),
  [3671] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_generic_function, 3, 0, 42),
  [3673] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_arguments, 2, 0, 0),
  [3675] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_arguments, 2, 0, 0),
  [3677] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_binary_expression, 3, 0, 47),
  [3679] = {.entry = {.count = 1, .reusable = true}}, SHIFT(259),
  [3681] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_binary_expression, 3, 0, 47),
  [3683] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1483),
  [3685] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2960),
  [3687] = {.entry = {.count = 1, .reusable = true}}, SHIFT(928),
  [3689] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_type, 3, 0, 0),
  [3691] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_tuple_type, 3, 0, 0),
  [3693] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_type, 3, 0, 63),
  [3695] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_array_type, 3, 0, 63),
  [3697] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_pointer_type, 3, 0, 64),
  [3699] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_pointer_type, 3, 0, 64),
  [3701] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_reference_type, 3, 0, 64),
  [3703] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_reference_type, 3, 0, 64),
  [3705] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_bounded_type, 3, 0, 0),
  [3707] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_bounded_type, 3, 0, 0),
  [3709] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_continue_expression, 2, 0, 0),
  [3711] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_continue_expression, 2, 0, 0),
  [3713] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_expression, 2, 0, 0),
  [3715] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_array_expression, 2, 0, 0),
  [3717] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_await_expression, 3, 0, 0),
  [3719] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_await_expression, 3, 0, 0),
  [3721] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_initializer_list, 4, 0, 0),
  [3723] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_initializer_list, 4, 0, 0),
  [3725] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_type, 6, 0, 201),
  [3727] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_type, 6, 0, 201),
  [3729] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_cast_expression, 3, 0, 51),
  [3731] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_cast_expression, 3, 0, 51),
  [3733] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_unary_expression, 2, 0, 0),
  [3735] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_unary_expression, 2, 0, 0),
  [3737] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_arguments, 4, 0, 0),
  [3739] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_arguments, 4, 0, 0),
  [3741] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_expression, 4, 0, 0),
  [3743] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_tuple_expression, 4, 0, 0),
  [3745] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_expression, 4, 0, 0),
  [3747] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_array_expression, 4, 0, 0),
  [3749] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_expression, 6, 0, 0),
  [3751] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_tuple_expression, 6, 0, 0),
  [3753] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_reference_expression, 4, 0, 55),
  [3755] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_reference_expression, 4, 0, 55),
  [3757] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_expression, 6, 0, 147),
  [3759] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_array_expression, 6, 0, 147),
  [3761] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_type, 4, 0, 0),
  [3763] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_tuple_type, 4, 0, 0),
  [3765] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_reference_type, 4, 0, 103),
  [3767] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_reference_type, 4, 0, 103),
  [3769] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_type, 4, 0, 104),
  [3771] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_type, 4, 0, 104),
  [3773] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_expression, 6, 0, 0),
  [3775] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_array_expression, 6, 0, 0),
  [3777] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_type, 4, 0, 107),
  [3779] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_type, 4, 0, 107),
  [3781] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_never_type, 1, 0, 0),
  [3783] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_never_type, 1, 0, 0),
  [3785] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_type, 4, 0, 109),
  [3787] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_type, 4, 0, 109),
  [3789] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_reference_expression, 2, 0, 4),
  [3791] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_reference_expression, 2, 0, 4),
  [3793] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_reference_expression, 3, 0, 18),
  [3795] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_reference_expression, 3, 0, 18),
  [3797] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_unit_expression, 2, 0, 0),
  [3799] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_unit_expression, 2, 0, 0),
  [3801] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_closure_expression, 6, 0, 176),
  [3803] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_closure_expression, 6, 0, 176),
  [3805] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_initializer_list, 5, 0, 0),
  [3807] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_initializer_list, 5, 0, 0),
  [3809] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_unit_type, 2, 0, 0),
  [3811] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_unit_type, 2, 0, 0),
  [3813] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_removed_trait_bound, 2, 0, 0),
  [3815] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_removed_trait_bound, 2, 0, 0),
  [3817] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_reference_type, 2, 0, 22),
  [3819] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_reference_type, 2, 0, 22),
  [3821] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_closure_expression, 3, 0, 35),
  [3823] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_closure_expression, 3, 0, 35),
  [3825] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_arguments, 5, 0, 0),
  [3827] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_arguments, 5, 0, 0),
  [3829] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_arguments, 6, 0, 0),
  [3831] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_arguments, 6, 0, 0),
  [3833] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_expression, 7, 0, 0),
  [3835] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_tuple_expression, 7, 0, 0),
  [3837] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_expression, 7, 0, 0),
  [3839] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_array_expression, 7, 0, 0),
  [3841] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_closure_expression, 5, 0, 128),
  [3843] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_closure_expression, 5, 0, 128),
  [3845] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_initializer_list, 2, 0, 0),
  [3847] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_initializer_list, 2, 0, 0),
  [3849] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_generic_type_with_turbofish, 3, 0, 43),
  [3851] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_type, 5, 0, 0),
  [3853] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_tuple_type, 5, 0, 0),
  [3855] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_type, 5, 0, 151),
  [3857] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_array_type, 5, 0, 151),
  [3859] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_type, 5, 0, 153),
  [3861] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_type, 5, 0, 153),
  [3863] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_type, 5, 0, 154),
  [3865] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_type, 5, 0, 154),
  [3867] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_higher_ranked_trait_bound, 3, 0, 80),
  [3869] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_higher_ranked_trait_bound, 3, 0, 80),
  [3871] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2622),
  [3873] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3074),
  [3875] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2603),
  [3877] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_type, 5, 0, 155),
  [3879] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_type, 5, 0, 155),
  [3881] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_match_arm_repeat1, 1, 0, 0),
  [3883] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_match_arm_repeat1, 1, 0, 0),
  [3885] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_match_block_repeat1, 1, 0, 0),
  [3887] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_match_block_repeat1, 1, 0, 0),
  [3889] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_closure_expression, 4, 0, 84),
  [3891] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_closure_expression, 4, 0, 84),
  [3893] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_expression, 2, 0, 10),
  [3895] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_expression, 2, 0, 10),
  [3897] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_initializer_list, 3, 0, 0),
  [3899] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_initializer_list, 3, 0, 0),
  [3901] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_expression, 2, 0, 11),
  [3903] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_expression, 2, 0, 11),
  [3905] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_arguments, 3, 0, 0),
  [3907] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_arguments, 3, 0, 0),
  [3909] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_index_expression, 4, 0, 0),
  [3911] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_index_expression, 4, 0, 0),
  [3913] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_closure_expression, 4, 0, 98),
  [3915] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_closure_expression, 4, 0, 98),
  [3917] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_call_expression, 2, 0, 12),
  [3919] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_call_expression, 2, 0, 12),
  [3921] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_try_expression, 2, 0, 0),
  [3923] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_try_expression, 2, 0, 0),
  [3925] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_expression, 5, 0, 0),
  [3927] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_tuple_expression, 5, 0, 0),
  [3929] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_expression, 5, 0, 100),
  [3931] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_array_expression, 5, 0, 100),
  [3933] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_expression, 5, 0, 0),
  [3935] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_array_expression, 5, 0, 0),
  [3937] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2655),
  [3939] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_closure_expression, 2, 0, 13),
  [3941] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_closure_expression, 2, 0, 13),
  [3943] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parenthesized_expression, 3, 0, 0),
  [3945] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parenthesized_expression, 3, 0, 0),
  [3947] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_expression, 3, 0, 0),
  [3949] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_array_expression, 3, 0, 0),
  [3951] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1926),
  [3953] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3181),
  [3955] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1528),
  [3957] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3101),
  [3959] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1030),
  [3961] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3295),
  [3963] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3299),
  [3965] = {.entry = {.count = 1, .reusable = false}}, SHIFT(268),
  [3967] = {.entry = {.count = 1, .reusable = false}}, SHIFT(270),
  [3969] = {.entry = {.count = 1, .reusable = false}}, SHIFT(271),
  [3971] = {.entry = {.count = 1, .reusable = false}}, SHIFT(273),
  [3973] = {.entry = {.count = 1, .reusable = false}}, SHIFT(274),
  [3975] = {.entry = {.count = 1, .reusable = true}}, SHIFT(275),
  [3977] = {.entry = {.count = 1, .reusable = true}}, SHIFT(277),
  [3979] = {.entry = {.count = 1, .reusable = false}}, SHIFT(278),
  [3981] = {.entry = {.count = 1, .reusable = true}}, SHIFT(288),
  [3983] = {.entry = {.count = 1, .reusable = false}}, SHIFT(289),
  [3985] = {.entry = {.count = 1, .reusable = true}}, SHIFT(290),
  [3987] = {.entry = {.count = 1, .reusable = false}}, SHIFT(290),
  [3989] = {.entry = {.count = 1, .reusable = false}}, SHIFT(42),
  [3991] = {.entry = {.count = 1, .reusable = true}}, SHIFT(312),
  [3993] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_compound_assignment_expr, 3, 0, 47),
  [3995] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_compound_assignment_expr, 3, 0, 47),
  [3997] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_assignment_expression, 3, 0, 48),
  [3999] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_assignment_expression, 3, 0, 48),
  [4001] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_range_expression, 3, 0, 0),
  [4003] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_range_expression, 3, 0, 0),
  [4005] = {.entry = {.count = 1, .reusable = true}}, SHIFT(389),
  [4007] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2624),
  [4009] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2649),
  [4011] = {.entry = {.count = 1, .reusable = true}}, SHIFT(448),
  [4013] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_return_expression, 2, 0, 0),
  [4015] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_return_expression, 2, 0, 0),
  [4017] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2730),
  [4019] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_break_expression, 3, 0, 0),
  [4021] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_break_expression, 3, 0, 0),
  [4023] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_yield_expression, 2, 0, 0),
  [4025] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_yield_expression, 2, 0, 0),
  [4027] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2348),
  [4029] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2651),
  [4031] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_visibility_modifier, 4, 0, 0),
  [4033] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_visibility_modifier, 4, 0, 0),
  [4035] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_visibility_modifier, 1, 0, 0),
  [4037] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_visibility_modifier, 1, 0, 0),
  [4039] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_visibility_modifier, 1, 0, 0), SHIFT(3283),
  [4042] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_visibility_modifier, 5, 0, 127),
  [4044] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_visibility_modifier, 5, 0, 127),
  [4046] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_visibility_modifier, 5, 0, 0),
  [4048] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_visibility_modifier, 5, 0, 0),
  [4050] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_visibility_modifier, 1, 0, 0), SHIFT(2662),
  [4053] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2708),
  [4055] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2082),
  [4057] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2566),
  [4059] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2714),
  [4061] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3249),
  [4063] = {.entry = {.count = 1, .reusable = true}}, SHIFT(347),
  [4065] = {.entry = {.count = 1, .reusable = false}}, SHIFT(37),
  [4067] = {.entry = {.count = 1, .reusable = true}}, SHIFT(293),
  [4069] = {.entry = {.count = 1, .reusable = true}}, SHIFT(158),
  [4071] = {.entry = {.count = 1, .reusable = true}}, SHIFT(295),
  [4073] = {.entry = {.count = 1, .reusable = true}}, SHIFT(146),
  [4075] = {.entry = {.count = 1, .reusable = true}}, SHIFT(279),
  [4077] = {.entry = {.count = 1, .reusable = true}}, SHIFT(153),
  [4079] = {.entry = {.count = 1, .reusable = false}}, SHIFT(299),
  [4081] = {.entry = {.count = 1, .reusable = false}}, SHIFT(300),
  [4083] = {.entry = {.count = 1, .reusable = false}}, SHIFT(301),
  [4085] = {.entry = {.count = 1, .reusable = false}}, SHIFT(302),
  [4087] = {.entry = {.count = 1, .reusable = false}}, SHIFT(303),
  [4089] = {.entry = {.count = 1, .reusable = true}}, SHIFT(326),
  [4091] = {.entry = {.count = 1, .reusable = true}}, SHIFT(305),
  [4093] = {.entry = {.count = 1, .reusable = false}}, SHIFT(306),
  [4095] = {.entry = {.count = 1, .reusable = true}}, SHIFT(307),
  [4097] = {.entry = {.count = 1, .reusable = false}}, SHIFT(309),
  [4099] = {.entry = {.count = 1, .reusable = true}}, SHIFT(310),
  [4101] = {.entry = {.count = 1, .reusable = false}}, SHIFT(310),
  [4103] = {.entry = {.count = 1, .reusable = false}}, SHIFT(55),
  [4105] = {.entry = {.count = 1, .reusable = true}}, SHIFT(333),
  [4107] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2409),
  [4109] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3168),
  [4111] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2349),
  [4113] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3578),
  [4115] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2567),
  [4117] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2512),
  [4119] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3297),
  [4121] = {.entry = {.count = 1, .reusable = true}}, SHIFT(341),
  [4123] = {.entry = {.count = 1, .reusable = true}}, SHIFT(154),
  [4125] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2470),
  [4127] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3102),
  [4129] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3259),
  [4131] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1478),
  [4133] = {.entry = {.count = 1, .reusable = true}}, SHIFT(167),
  [4135] = {.entry = {.count = 1, .reusable = true}}, SHIFT(883),
  [4137] = {.entry = {.count = 1, .reusable = false}}, SHIFT(59),
  [4139] = {.entry = {.count = 1, .reusable = true}}, SHIFT(263),
  [4141] = {.entry = {.count = 1, .reusable = true}}, SHIFT(884),
  [4143] = {.entry = {.count = 1, .reusable = true}}, SHIFT(885),
  [4145] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1668),
  [4147] = {.entry = {.count = 1, .reusable = true}}, SHIFT(162),
  [4149] = {.entry = {.count = 1, .reusable = true}}, SHIFT(164),
  [4151] = {.entry = {.count = 1, .reusable = true}}, SHIFT(165),
  [4153] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3582),
  [4155] = {.entry = {.count = 1, .reusable = true}}, SHIFT(887),
  [4157] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_arguments_repeat1, 2, 0, 0),
  [4159] = {.entry = {.count = 1, .reusable = true}}, SHIFT(186),
  [4161] = {.entry = {.count = 1, .reusable = true}}, SHIFT(874),
  [4163] = {.entry = {.count = 1, .reusable = true}}, SHIFT(140),
  [4165] = {.entry = {.count = 1, .reusable = true}}, SHIFT(178),
  [4167] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3191),
  [4169] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_arguments_repeat1, 3, 0, 0),
  [4171] = {.entry = {.count = 1, .reusable = true}}, SHIFT(876),
  [4173] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2440),
  [4175] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1643),
  [4177] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3028),
  [4179] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2862),
  [4181] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2435),
  [4183] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3587),
  [4185] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2719),
  [4187] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2447),
  [4189] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2447),
  [4191] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3207),
  [4193] = {.entry = {.count = 1, .reusable = true}}, SHIFT(878),
  [4195] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3085),
  [4197] = {.entry = {.count = 1, .reusable = true}}, SHIFT(694),
  [4199] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1445),
  [4201] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1014),
  [4203] = {.entry = {.count = 1, .reusable = true}}, SHIFT(340),
  [4205] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1699),
  [4207] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3010),
  [4209] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_variant, 3, 0, 156),
  [4211] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1492),
  [4213] = {.entry = {.count = 1, .reusable = true}}, SHIFT(222),
  [4215] = {.entry = {.count = 1, .reusable = true}}, SHIFT(495),
  [4217] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2526),
  [4219] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_last_match_arm, 3, 0, 173),
  [4221] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1070),
  [4223] = {.entry = {.count = 1, .reusable = false}}, SHIFT(365),
  [4225] = {.entry = {.count = 1, .reusable = false}}, SHIFT(367),
  [4227] = {.entry = {.count = 1, .reusable = false}}, SHIFT(369),
  [4229] = {.entry = {.count = 1, .reusable = false}}, SHIFT(248),
  [4231] = {.entry = {.count = 1, .reusable = false}}, SHIFT(249),
  [4233] = {.entry = {.count = 1, .reusable = true}}, SHIFT(250),
  [4235] = {.entry = {.count = 1, .reusable = true}}, SHIFT(251),
  [4237] = {.entry = {.count = 1, .reusable = false}}, SHIFT(252),
  [4239] = {.entry = {.count = 1, .reusable = true}}, SHIFT(255),
  [4241] = {.entry = {.count = 1, .reusable = false}}, SHIFT(255),
  [4243] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1004),
  [4245] = {.entry = {.count = 1, .reusable = false}}, SHIFT(51),
  [4247] = {.entry = {.count = 1, .reusable = true}}, SHIFT(272),
  [4249] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__condition, 1, 0, 0),
  [4251] = {.entry = {.count = 1, .reusable = true}}, SHIFT(218),
  [4253] = {.entry = {.count = 1, .reusable = false}}, SHIFT(321),
  [4255] = {.entry = {.count = 1, .reusable = false}}, SHIFT(322),
  [4257] = {.entry = {.count = 1, .reusable = false}}, SHIFT(323),
  [4259] = {.entry = {.count = 1, .reusable = false}}, SHIFT(324),
  [4261] = {.entry = {.count = 1, .reusable = false}}, SHIFT(325),
  [4263] = {.entry = {.count = 1, .reusable = true}}, SHIFT(304),
  [4265] = {.entry = {.count = 1, .reusable = true}}, SHIFT(327),
  [4267] = {.entry = {.count = 1, .reusable = false}}, SHIFT(328),
  [4269] = {.entry = {.count = 1, .reusable = true}}, SHIFT(329),
  [4271] = {.entry = {.count = 1, .reusable = false}}, SHIFT(330),
  [4273] = {.entry = {.count = 1, .reusable = true}}, SHIFT(331),
  [4275] = {.entry = {.count = 1, .reusable = false}}, SHIFT(331),
  [4277] = {.entry = {.count = 1, .reusable = false}}, SHIFT(54),
  [4279] = {.entry = {.count = 1, .reusable = true}}, SHIFT(267),
  [4281] = {.entry = {.count = 1, .reusable = true}}, SHIFT(598),
  [4283] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2555),
  [4285] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_initializer, 4, 0, 187),
  [4287] = {.entry = {.count = 1, .reusable = true}}, SHIFT(474),
  [4289] = {.entry = {.count = 1, .reusable = false}}, SHIFT(62),
  [4291] = {.entry = {.count = 1, .reusable = true}}, SHIFT(256),
  [4293] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_initializer, 4, 0, 188),
  [4295] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2778),
  [4297] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_initializer, 3, 0, 139),
  [4299] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__let_chain, 3, 0, 0),
  [4301] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_initializer, 3, 0, 140),
  [4303] = {.entry = {.count = 1, .reusable = true}}, SHIFT(695),
  [4305] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2527),
  [4307] = {.entry = {.count = 1, .reusable = true}}, SHIFT(253),
  [4309] = {.entry = {.count = 1, .reusable = false}}, SHIFT(254),
  [4311] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1707),
  [4313] = {.entry = {.count = 1, .reusable = true}}, SHIFT(228),
  [4315] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_variant, 4, 0, 203),
  [4317] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_variant, 4, 0, 204),
  [4319] = {.entry = {.count = 1, .reusable = true}}, SHIFT(475),
  [4321] = {.entry = {.count = 1, .reusable = true}}, SHIFT(534),
  [4323] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2715),
  [4325] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2954),
  [4327] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1175),
  [4329] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2697),
  [4331] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_last_match_arm, 4, 0, 117),
  [4333] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1074),
  [4335] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_base_field_initializer, 2, 0, 0),
  [4337] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1372),
  [4339] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2705),
  [4341] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1390),
  [4343] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1858),
  [4345] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1282),
  [4347] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2707),
  [4349] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1321),
  [4351] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2710),
  [4353] = {.entry = {.count = 1, .reusable = true}}, SHIFT(467),
  [4355] = {.entry = {.count = 1, .reusable = true}}, SHIFT(371),
  [4357] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2087),
  [4359] = {.entry = {.count = 1, .reusable = true}}, SHIFT(472),
  [4361] = {.entry = {.count = 1, .reusable = true}}, SHIFT(651),
  [4363] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1032),
  [4365] = {.entry = {.count = 1, .reusable = true}}, SHIFT(455),
  [4367] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1594),
  [4369] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_variant, 5, 0, 244),
  [4371] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_condition, 4, 0, 117),
  [4373] = {.entry = {.count = 1, .reusable = true}}, SHIFT(500),
  [4375] = {.entry = {.count = 1, .reusable = false}}, SHIFT(49),
  [4377] = {.entry = {.count = 1, .reusable = true}}, SHIFT(269),
  [4379] = {.entry = {.count = 1, .reusable = true}}, SHIFT(237),
  [4381] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1401),
  [4383] = {.entry = {.count = 1, .reusable = true}}, SHIFT(578),
  [4385] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1485),
  [4387] = {.entry = {.count = 1, .reusable = true}}, SHIFT(745),
  [4389] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1669),
  [4391] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1673),
  [4393] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1743),
  [4395] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1458),
  [4397] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1264),
  [4399] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1396),
  [4401] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1324),
  [4403] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1341),
  [4405] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1350),
  [4407] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1362),
  [4409] = {.entry = {.count = 1, .reusable = true}}, SHIFT(538),
  [4411] = {.entry = {.count = 1, .reusable = true}}, SHIFT(566),
  [4413] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1735),
  [4415] = {.entry = {.count = 1, .reusable = true}}, SHIFT(556),
  [4417] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1403),
  [4419] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1479),
  [4421] = {.entry = {.count = 1, .reusable = true}}, SHIFT(226),
  [4423] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1993),
  [4425] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_attribute, 3, 0, 110),
  [4427] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_attribute, 3, 0, 18),
  [4429] = {.entry = {.count = 1, .reusable = true}}, SHIFT(236),
  [4431] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1286),
  [4433] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2687),
  [4435] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3519),
  [4437] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3133),
  [4439] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3032),
  [4441] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3532),
  [4443] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3532),
  [4445] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2561),
  [4447] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2568),
  [4449] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2752),
  [4451] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2749),
  [4453] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2543),
  [4455] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2742),
  [4457] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2748),
  [4459] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_for_lifetimes, 6, 0, 0),
  [4461] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_for_lifetimes, 6, 0, 0),
  [4463] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_for_lifetimes, 5, 0, 0),
  [4465] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_for_lifetimes, 5, 0, 0),
  [4467] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_for_lifetimes, 4, 0, 0),
  [4469] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_for_lifetimes, 4, 0, 0),
  [4471] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2305),
  [4473] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2261),
  [4475] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2306),
  [4477] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2306),
  [4479] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3091),
  [4481] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3089),
  [4483] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3241),
  [4485] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3095),
  [4487] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3095),
  [4489] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3180),
  [4491] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3179),
  [4493] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3195),
  [4495] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3195),
  [4497] = {.entry = {.count = 1, .reusable = true}}, SHIFT(376),
  [4499] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2565),
  [4501] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2740),
  [4503] = {.entry = {.count = 1, .reusable = true}}, SHIFT(451),
  [4505] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2569),
  [4507] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__pattern, 1, 0, 0),
  [4509] = {.entry = {.count = 1, .reusable = true}}, SHIFT(784),
  [4511] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2247),
  [4513] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__pattern, 1, 0, 0),
  [4515] = {.entry = {.count = 1, .reusable = true}}, SHIFT(828),
  [4517] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1017),
  [4519] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1509),
  [4521] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2591),
  [4523] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2387),
  [4525] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2256),
  [4527] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3429),
  [4529] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3305),
  [4531] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3362),
  [4533] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2831),
  [4535] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3364),
  [4537] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3349),
  [4539] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3375),
  [4541] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3377),
  [4543] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2288),
  [4545] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1878),
  [4547] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2072),
  [4549] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__pattern, 1, 0, 1),
  [4551] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__pattern, 1, 0, 1),
  [4553] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2558),
  [4555] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1018),
  [4557] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1504),
  [4559] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3109),
  [4561] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2308),
  [4563] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3381),
  [4565] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3312),
  [4567] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3382),
  [4569] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3053),
  [4571] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3383),
  [4573] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3378),
  [4575] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3628),
  [4577] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3384),
  [4579] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2304),
  [4581] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1893),
  [4583] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2089),
  [4585] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3172),
  [4587] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3280),
  [4589] = {.entry = {.count = 1, .reusable = true}}, SHIFT(893),
  [4591] = {.entry = {.count = 1, .reusable = true}}, SHIFT(901),
  [4593] = {.entry = {.count = 1, .reusable = true}}, SHIFT(905),
  [4595] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3626),
  [4597] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2739),
  [4599] = {.entry = {.count = 1, .reusable = true}}, SHIFT(997),
  [4601] = {.entry = {.count = 1, .reusable = true}}, SHIFT(401),
  [4603] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__type, 1, 0, 5), REDUCE(sym__pattern, 1, 0, 0),
  [4606] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2647),
  [4608] = {.entry = {.count = 1, .reusable = true}}, SHIFT(390),
  [4610] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2645),
  [4612] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3283),
  [4614] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2753),
  [4616] = {.entry = {.count = 1, .reusable = true}}, SHIFT(380),
  [4618] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2633),
  [4620] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1008),
  [4622] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1002),
  [4624] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1007),
  [4626] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3183),
  [4628] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3139),
  [4630] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3251),
  [4632] = {.entry = {.count = 1, .reusable = false}}, SHIFT(843),
  [4634] = {.entry = {.count = 1, .reusable = true}}, SHIFT(937),
  [4636] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1366),
  [4638] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2045),
  [4640] = {.entry = {.count = 1, .reusable = true}}, SHIFT(150),
  [4642] = {.entry = {.count = 1, .reusable = false}}, SHIFT(150),
  [4644] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_scoped_identifier, 3, 0, 40), REDUCE(sym_scoped_type_identifier, 3, 0, 41),
  [4647] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_scoped_identifier, 3, 0, 45), REDUCE(sym_scoped_type_identifier, 3, 0, 46),
  [4650] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_scoped_identifier, 3, 0, 16), REDUCE(sym_scoped_type_identifier, 3, 0, 17),
  [4653] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_scoped_identifier, 2, 0, 6), REDUCE(sym_scoped_type_identifier, 2, 0, 7),
  [4656] = {.entry = {.count = 1, .reusable = true}}, SHIFT(785),
  [4658] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2209),
  [4660] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2607),
  [4662] = {.entry = {.count = 1, .reusable = true}}, SHIFT(805),
  [4664] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1088),
  [4666] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1523),
  [4668] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2639),
  [4670] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_negative_literal, 2, 0, 0),
  [4672] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_negative_literal, 2, 0, 0),
  [4674] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__literal_pattern, 1, 0, 0),
  [4676] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__literal_pattern, 1, 0, 0),
  [4678] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2426),
  [4680] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1069),
  [4682] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3433),
  [4684] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3270),
  [4686] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2498),
  [4688] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1067),
  [4690] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1066),
  [4692] = {.entry = {.count = 1, .reusable = true}}, SHIFT(861),
  [4694] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1068),
  [4696] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2463),
  [4698] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1077),
  [4700] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2452),
  [4702] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1230),
  [4704] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2482),
  [4706] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2496),
  [4708] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2462),
  [4710] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1533),
  [4712] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1065),
  [4714] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2497),
  [4716] = {.entry = {.count = 1, .reusable = true}}, SHIFT(946),
  [4718] = {.entry = {.count = 1, .reusable = true}}, SHIFT(942),
  [4720] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_range_pattern, 3, 0, 1),
  [4722] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_range_pattern, 3, 0, 1),
  [4724] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_extern_modifier, 1, 0, 0),
  [4726] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3504),
  [4728] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2542),
  [4730] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2499),
  [4732] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3310),
  [4734] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_range_pattern, 3, 0, 0),
  [4736] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_range_pattern, 3, 0, 0),
  [4738] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2343),
  [4740] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2953),
  [4742] = {.entry = {.count = 1, .reusable = true}}, SHIFT(934),
  [4744] = {.entry = {.count = 1, .reusable = true}}, SHIFT(980),
  [4746] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2375),
  [4748] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3040),
  [4750] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2379),
  [4752] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2796),
  [4754] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3350),
  [4756] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2538),
  [4758] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2520),
  [4760] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2344),
  [4762] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2837),
  [4764] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_range_pattern, 3, 0, 56),
  [4766] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_range_pattern, 3, 0, 56),
  [4768] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3315),
  [4770] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3373),
  [4772] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_function_modifiers_repeat1, 1, 0, 0),
  [4774] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2902),
  [4776] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1821),
  [4778] = {.entry = {.count = 1, .reusable = true}}, SHIFT(258),
  [4780] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3501),
  [4782] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3600),
  [4784] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1452),
  [4786] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3584),
  [4788] = {.entry = {.count = 1, .reusable = true}}, SHIFT(892),
  [4790] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3380),
  [4792] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3420),
  [4794] = {.entry = {.count = 1, .reusable = true}}, SHIFT(959),
  [4796] = {.entry = {.count = 1, .reusable = true}}, SHIFT(957),
  [4798] = {.entry = {.count = 1, .reusable = true}}, SHIFT(975),
  [4800] = {.entry = {.count = 1, .reusable = true}}, SHIFT(923),
  [4802] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_range_pattern, 3, 0, 60),
  [4804] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_range_pattern, 3, 0, 60),
  [4806] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_remaining_field_pattern, 1, 0, 0),
  [4808] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_slice_pattern, 2, 0, 0),
  [4810] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_struct_pattern, 4, 0, 62),
  [4812] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_slice_pattern, 3, 0, 0),
  [4814] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_pattern, 6, 0, 58),
  [4816] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_pattern, 4, 0, 57),
  [4818] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_struct_pattern, 6, 0, 62),
  [4820] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_pattern, 5, 0, 0),
  [4822] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_pattern, 5, 0, 57),
  [4824] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_captured_pattern, 3, 0, 0),
  [4826] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_pattern, 4, 0, 0),
  [4828] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_struct_pattern, 5, 0, 57),
  [4830] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1389),
  [4832] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2677),
  [4834] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2622),
  [4836] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_slice_pattern, 4, 0, 0),
  [4838] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_slice_pattern, 5, 0, 0),
  [4840] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_pattern, 3, 0, 57),
  [4842] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_pattern, 6, 0, 57),
  [4844] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1434),
  [4846] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2206),
  [4848] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_or_pattern, 2, 0, 0),
  [4850] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_pattern, 3, 0, 58),
  [4852] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ref_pattern, 2, 0, 0),
  [4854] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1830),
  [4856] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_mut_pattern, 2, 0, 0),
  [4858] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2756),
  [4860] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_struct_pattern, 6, 0, 57),
  [4862] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_struct_pattern, 3, 0, 57),
  [4864] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2270),
  [4866] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1120),
  [4868] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3385),
  [4870] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3496),
  [4872] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_pattern, 2, 0, 0),
  [4874] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3396),
  [4876] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1033),
  [4878] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3356),
  [4880] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_reference_pattern, 2, 0, 0),
  [4882] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_struct_pattern, 5, 0, 62),
  [4884] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_struct_pattern, 3, 0, 62),
  [4886] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1722),
  [4888] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2602),
  [4890] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1086),
  [4892] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1520),
  [4894] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3159),
  [4896] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_struct_pattern, 4, 0, 57),
  [4898] = {.entry = {.count = 1, .reusable = true}}, SHIFT(590),
  [4900] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3483),
  [4902] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_or_pattern, 3, 0, 0),
  [4904] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_reference_pattern, 3, 0, 0),
  [4906] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_pattern, 3, 0, 0),
  [4908] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_pattern, 4, 0, 58),
  [4910] = {.entry = {.count = 1, .reusable = true}}, SHIFT(609),
  [4912] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3392),
  [4914] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_pattern, 5, 0, 58),
  [4916] = {.entry = {.count = 1, .reusable = true}}, SHIFT(501),
  [4918] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_bounds, 3, 0, 0),
  [4920] = {.entry = {.count = 1, .reusable = true}}, SHIFT(853),
  [4922] = {.entry = {.count = 1, .reusable = true}}, SHIFT(746),
  [4924] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2640),
  [4926] = {.entry = {.count = 1, .reusable = true}}, SHIFT(903),
  [4928] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__type, 1, 0, 0), REDUCE(sym__pattern, 1, 0, 1),
  [4931] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_trait_bounds_repeat1, 2, 0, 0),
  [4933] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_trait_bounds_repeat1, 2, 0, 0), SHIFT_REPEAT(853),
  [4936] = {.entry = {.count = 1, .reusable = true}}, SHIFT(996),
  [4938] = {.entry = {.count = 1, .reusable = true}}, SHIFT(749),
  [4940] = {.entry = {.count = 1, .reusable = true}}, SHIFT(771),
  [4942] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2146),
  [4944] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2061),
  [4946] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1428),
  [4948] = {.entry = {.count = 1, .reusable = true}}, SHIFT(671),
  [4950] = {.entry = {.count = 1, .reusable = true}}, SHIFT(674),
  [4952] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3111),
  [4954] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1084),
  [4956] = {.entry = {.count = 1, .reusable = true}}, SHIFT(662),
  [4958] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_bounds, 2, 0, 0),
  [4960] = {.entry = {.count = 1, .reusable = true}}, SHIFT(935),
  [4962] = {.entry = {.count = 1, .reusable = true}}, SHIFT(976),
  [4964] = {.entry = {.count = 1, .reusable = true}}, SHIFT(854),
  [4966] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1111),
  [4968] = {.entry = {.count = 1, .reusable = true}}, SHIFT(855),
  [4970] = {.entry = {.count = 1, .reusable = true}}, SHIFT(527),
  [4972] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3066),
  [4974] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1150),
  [4976] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2133),
  [4978] = {.entry = {.count = 1, .reusable = true}}, SHIFT(925),
  [4980] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1212),
  [4982] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1044),
  [4984] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1265),
  [4986] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1039),
  [4988] = {.entry = {.count = 1, .reusable = true}}, SHIFT(701),
  [4990] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1314),
  [4992] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1037),
  [4994] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3125),
  [4996] = {.entry = {.count = 1, .reusable = true}}, SHIFT(539),
  [4998] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_macro_definition_repeat1, 2, 0, 0), SHIFT_REPEAT(76),
  [5001] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_macro_definition_repeat1, 2, 0, 0),
  [5003] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_macro_definition_repeat1, 2, 0, 0), SHIFT_REPEAT(71),
  [5006] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_macro_definition_repeat1, 2, 0, 0), SHIFT_REPEAT(75),
  [5009] = {.entry = {.count = 1, .reusable = true}}, SHIFT(643),
  [5011] = {.entry = {.count = 1, .reusable = true}}, SHIFT(612),
  [5013] = {.entry = {.count = 1, .reusable = true}}, SHIFT(760),
  [5015] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2648),
  [5017] = {.entry = {.count = 1, .reusable = true}}, SHIFT(947),
  [5019] = {.entry = {.count = 1, .reusable = true}}, SHIFT(960),
  [5021] = {.entry = {.count = 1, .reusable = true}}, SHIFT(958),
  [5023] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1104),
  [5025] = {.entry = {.count = 1, .reusable = true}}, SHIFT(683),
  [5027] = {.entry = {.count = 1, .reusable = true}}, SHIFT(19),
  [5029] = {.entry = {.count = 1, .reusable = true}}, SHIFT(984),
  [5031] = {.entry = {.count = 1, .reusable = true}}, SHIFT(523),
  [5033] = {.entry = {.count = 1, .reusable = true}}, SHIFT(915),
  [5035] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_extern_modifier, 2, 0, 0),
  [5037] = {.entry = {.count = 1, .reusable = true}}, SHIFT(573),
  [5039] = {.entry = {.count = 1, .reusable = true}}, SHIFT(879),
  [5041] = {.entry = {.count = 1, .reusable = true}}, SHIFT(747),
  [5043] = {.entry = {.count = 1, .reusable = true}}, SHIFT(843),
  [5045] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2070),
  [5047] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1107),
  [5049] = {.entry = {.count = 1, .reusable = true}}, SHIFT(952),
  [5051] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2856),
  [5053] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2115),
  [5055] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3639),
  [5057] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3149),
  [5059] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3523),
  [5061] = {.entry = {.count = 1, .reusable = true}}, SHIFT(596),
  [5063] = {.entry = {.count = 1, .reusable = true}}, SHIFT(926),
  [5065] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2876),
  [5067] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3359),
  [5069] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2885),
  [5071] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3436),
  [5073] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1123),
  [5075] = {.entry = {.count = 1, .reusable = true}}, SHIFT(23),
  [5077] = {.entry = {.count = 1, .reusable = true}}, SHIFT(956),
  [5079] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1164),
  [5081] = {.entry = {.count = 1, .reusable = true}}, SHIFT(967),
  [5083] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1196),
  [5085] = {.entry = {.count = 1, .reusable = true}}, SHIFT(977),
  [5087] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1215),
  [5089] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1244),
  [5091] = {.entry = {.count = 1, .reusable = true}}, SHIFT(983),
  [5093] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1258),
  [5095] = {.entry = {.count = 1, .reusable = true}}, SHIFT(986),
  [5097] = {.entry = {.count = 1, .reusable = true}}, SHIFT(728),
  [5099] = {.entry = {.count = 1, .reusable = true}}, SHIFT(930),
  [5101] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1266),
  [5103] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1299),
  [5105] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1337),
  [5107] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1347),
  [5109] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1357),
  [5111] = {.entry = {.count = 1, .reusable = true}}, SHIFT(676),
  [5113] = {.entry = {.count = 1, .reusable = true}}, SHIFT(870),
  [5115] = {.entry = {.count = 1, .reusable = true}}, SHIFT(514),
  [5117] = {.entry = {.count = 1, .reusable = true}}, SHIFT(842),
  [5119] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_modifiers, 1, 0, 0),
  [5121] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2233),
  [5123] = {.entry = {.count = 1, .reusable = true}}, SHIFT(552),
  [5125] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2634),
  [5127] = {.entry = {.count = 1, .reusable = true}}, SHIFT(563),
  [5129] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_function_modifiers_repeat1, 2, 0, 0), SHIFT_REPEAT(2387),
  [5132] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_function_modifiers_repeat1, 2, 0, 0),
  [5134] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_function_modifiers_repeat1, 2, 0, 0), SHIFT_REPEAT(2233),
  [5137] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2120),
  [5139] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3405),
  [5141] = {.entry = {.count = 1, .reusable = true}}, SHIFT(648),
  [5143] = {.entry = {.count = 1, .reusable = true}}, SHIFT(979),
  [5145] = {.entry = {.count = 1, .reusable = true}}, SHIFT(602),
  [5147] = {.entry = {.count = 1, .reusable = true}}, SHIFT(987),
  [5149] = {.entry = {.count = 1, .reusable = true}}, SHIFT(707),
  [5151] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1308),
  [5153] = {.entry = {.count = 1, .reusable = true}}, SHIFT(988),
  [5155] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2147),
  [5157] = {.entry = {.count = 1, .reusable = true}}, SHIFT(76),
  [5159] = {.entry = {.count = 1, .reusable = true}}, SHIFT(71),
  [5161] = {.entry = {.count = 1, .reusable = true}}, SHIFT(75),
  [5163] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1113),
  [5165] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3376),
  [5167] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3412),
  [5169] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3476),
  [5171] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1114),
  [5173] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3390),
  [5175] = {.entry = {.count = 1, .reusable = true}}, SHIFT(734),
  [5177] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1202),
  [5179] = {.entry = {.count = 1, .reusable = true}}, SHIFT(124),
  [5181] = {.entry = {.count = 1, .reusable = true}}, SHIFT(128),
  [5183] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_attribute, 1, 0, 1),
  [5185] = {.entry = {.count = 1, .reusable = true}}, SHIFT(129),
  [5187] = {.entry = {.count = 1, .reusable = true}}, SHIFT(284),
  [5189] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2985),
  [5191] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2988),
  [5193] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3570),
  [5195] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1054),
  [5197] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2818),
  [5199] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_attribute, 1, 0, 0),
  [5201] = {.entry = {.count = 1, .reusable = true}}, SHIFT(287),
  [5203] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2101),
  [5205] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_variant, 1, 0, 71),
  [5207] = {.entry = {.count = 1, .reusable = true}}, SHIFT(334),
  [5209] = {.entry = {.count = 1, .reusable = true}}, SHIFT(658),
  [5211] = {.entry = {.count = 1, .reusable = true}}, SHIFT(613),
  [5213] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1132),
  [5215] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3464),
  [5217] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3500),
  [5219] = {.entry = {.count = 1, .reusable = true}}, SHIFT(653),
  [5221] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_self_parameter, 1, 0, 0),
  [5223] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1006),
  [5225] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3173),
  [5227] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3580),
  [5229] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3621),
  [5231] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3198),
  [5233] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3632),
  [5235] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_self_parameter, 2, 0, 0),
  [5237] = {.entry = {.count = 1, .reusable = false}}, SHIFT(902),
  [5239] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3215),
  [5241] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3005),
  [5243] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3011),
  [5245] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3485),
  [5247] = {.entry = {.count = 1, .reusable = true}}, SHIFT(537),
  [5249] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3507),
  [5251] = {.entry = {.count = 1, .reusable = true}}, SHIFT(558),
  [5253] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1154),
  [5255] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_variant, 2, 0, 6),
  [5257] = {.entry = {.count = 1, .reusable = true}}, SHIFT(261),
  [5259] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3203),
  [5261] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1155),
  [5263] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2116),
  [5265] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3458),
  [5267] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3642),
  [5269] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3478),
  [5271] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3518),
  [5273] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3419),
  [5275] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3589),
  [5277] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2105),
  [5279] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1217),
  [5281] = {.entry = {.count = 1, .reusable = true}}, SHIFT(634),
  [5283] = {.entry = {.count = 1, .reusable = true}}, SHIFT(503),
  [5285] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2130),
  [5287] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1099),
  [5289] = {.entry = {.count = 1, .reusable = true}}, SHIFT(505),
  [5291] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1274),
  [5293] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1276),
  [5295] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1288),
  [5297] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1290),
  [5299] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1292),
  [5301] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1294),
  [5303] = {.entry = {.count = 1, .reusable = true}}, SHIFT(507),
  [5305] = {.entry = {.count = 1, .reusable = true}}, SHIFT(509),
  [5307] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_parameters, 2, 0, 0), REDUCE(sym_tuple_struct_pattern, 3, 0, 57),
  [5310] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1329),
  [5312] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1331),
  [5314] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_parameters, 3, 0, 0), REDUCE(sym_tuple_struct_pattern, 4, 0, 57),
  [5317] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1367),
  [5319] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2047),
  [5321] = {.entry = {.count = 1, .reusable = true}}, SHIFT(181),
  [5323] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3398),
  [5325] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3113),
  [5327] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3366),
  [5329] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3082),
  [5331] = {.entry = {.count = 1, .reusable = true}}, SHIFT(933),
  [5333] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1945),
  [5335] = {.entry = {.count = 1, .reusable = true}}, SHIFT(441),
  [5337] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1590),
  [5339] = {.entry = {.count = 1, .reusable = true}}, SHIFT(442),
  [5341] = {.entry = {.count = 1, .reusable = true}}, SHIFT(191),
  [5343] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1063),
  [5345] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2040),
  [5347] = {.entry = {.count = 1, .reusable = true}}, SHIFT(684),
  [5349] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1125),
  [5351] = {.entry = {.count = 1, .reusable = true}}, SHIFT(194),
  [5353] = {.entry = {.count = 1, .reusable = true}}, SHIFT(544),
  [5355] = {.entry = {.count = 1, .reusable = true}}, SHIFT(546),
  [5357] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1139),
  [5359] = {.entry = {.count = 1, .reusable = true}}, SHIFT(964),
  [5361] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1142),
  [5363] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2141),
  [5365] = {.entry = {.count = 1, .reusable = true}}, SHIFT(498),
  [5367] = {.entry = {.count = 1, .reusable = true}}, SHIFT(989),
  [5369] = {.entry = {.count = 1, .reusable = true}}, SHIFT(600),
  [5371] = {.entry = {.count = 1, .reusable = true}}, SHIFT(720),
  [5373] = {.entry = {.count = 1, .reusable = true}}, SHIFT(759),
  [5375] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1156),
  [5377] = {.entry = {.count = 1, .reusable = true}}, SHIFT(443),
  [5379] = {.entry = {.count = 1, .reusable = true}}, SHIFT(489),
  [5381] = {.entry = {.count = 1, .reusable = true}}, SHIFT(584),
  [5383] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2460),
  [5385] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2050),
  [5387] = {.entry = {.count = 1, .reusable = true}}, SHIFT(188),
  [5389] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2513),
  [5391] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2039),
  [5393] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1168),
  [5395] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1531),
  [5397] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2051),
  [5399] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1172),
  [5401] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_self_parameter, 3, 0, 0),
  [5403] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1189),
  [5405] = {.entry = {.count = 1, .reusable = true}}, SHIFT(487),
  [5407] = {.entry = {.count = 1, .reusable = true}}, SHIFT(897),
  [5409] = {.entry = {.count = 1, .reusable = true}}, SHIFT(697),
  [5411] = {.entry = {.count = 1, .reusable = true}}, SHIFT(586),
  [5413] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1219),
  [5415] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1223),
  [5417] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1225),
  [5419] = {.entry = {.count = 1, .reusable = true}}, SHIFT(740),
  [5421] = {.entry = {.count = 1, .reusable = true}}, SHIFT(690),
  [5423] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1129),
  [5425] = {.entry = {.count = 1, .reusable = true}}, SHIFT(961),
  [5427] = {.entry = {.count = 1, .reusable = true}}, SHIFT(349),
  [5429] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2688),
  [5431] = {.entry = {.count = 1, .reusable = true}}, SHIFT(607),
  [5433] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1005),
  [5435] = {.entry = {.count = 1, .reusable = true}}, SHIFT(298),
  [5437] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2755),
  [5439] = {.entry = {.count = 1, .reusable = true}}, SHIFT(678),
  [5441] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1050),
  [5443] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2815),
  [5445] = {.entry = {.count = 1, .reusable = true}}, SHIFT(512),
  [5447] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1006),
  [5449] = {.entry = {.count = 1, .reusable = true}}, SHIFT(437),
  [5451] = {.entry = {.count = 1, .reusable = true}}, SHIFT(790),
  [5453] = {.entry = {.count = 1, .reusable = true}}, SHIFT(575),
  [5455] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1157),
  [5457] = {.entry = {.count = 1, .reusable = true}}, SHIFT(410),
  [5459] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2676),
  [5461] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2467),
  [5463] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2044),
  [5465] = {.entry = {.count = 1, .reusable = true}}, SHIFT(654),
  [5467] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1166),
  [5469] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2053),
  [5471] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_type_parameters_repeat1, 2, 0, 77),
  [5473] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2041),
  [5475] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_unit_type, 2, 0, 0), REDUCE(sym_tuple_pattern, 2, 0, 0),
  [5478] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2140),
  [5480] = {.entry = {.count = 1, .reusable = true}}, SHIFT(844),
  [5482] = {.entry = {.count = 1, .reusable = true}}, SHIFT(830),
  [5484] = {.entry = {.count = 1, .reusable = true}}, SHIFT(794),
  [5486] = {.entry = {.count = 1, .reusable = true}}, SHIFT(414),
  [5488] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1213),
  [5490] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__use_clause, 1, 0, 1),
  [5492] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2414),
  [5494] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3597),
  [5496] = {.entry = {.count = 1, .reusable = true}}, SHIFT(727),
  [5498] = {.entry = {.count = 1, .reusable = true}}, SHIFT(929),
  [5500] = {.entry = {.count = 1, .reusable = true}}, SHIFT(339),
  [5502] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2519),
  [5504] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1246),
  [5506] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1979),
  [5508] = {.entry = {.count = 1, .reusable = true}}, SHIFT(411),
  [5510] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__use_clause, 1, 0, 0),
  [5512] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2279),
  [5514] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3421),
  [5516] = {.entry = {.count = 1, .reusable = true}}, SHIFT(571),
  [5518] = {.entry = {.count = 1, .reusable = true}}, SHIFT(525),
  [5520] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1268),
  [5522] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__pattern, 1, 0, 0), SHIFT(1979),
  [5525] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__pattern, 1, 0, 0), SHIFT(411),
  [5528] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ordered_field_declaration_list, 7, 0, 252),
  [5530] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__type, 1, 0, 0), REDUCE(sym__pattern, 1, 0, 0),
  [5533] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2429),
  [5535] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1297),
  [5537] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1310),
  [5539] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1315),
  [5541] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ordered_field_declaration_list, 4, 0, 64),
  [5543] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1339),
  [5545] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1345),
  [5547] = {.entry = {.count = 1, .reusable = true}}, SHIFT(579),
  [5549] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1355),
  [5551] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1359),
  [5553] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1363),
  [5555] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2601),
  [5557] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ordered_field_declaration_list, 4, 0, 22),
  [5559] = {.entry = {.count = 1, .reusable = true}}, SHIFT(528),
  [5561] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2859),
  [5563] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3480),
  [5565] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ordered_field_declaration_list, 5, 0, 64),
  [5567] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ordered_field_declaration_list, 5, 0, 220),
  [5569] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ordered_field_declaration_list, 5, 0, 177),
  [5571] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ordered_field_declaration_list, 3, 0, 0),
  [5573] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ordered_field_declaration_list, 5, 0, 103),
  [5575] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1616),
  [5577] = {.entry = {.count = 1, .reusable = true}}, SHIFT(412),
  [5579] = {.entry = {.count = 1, .reusable = true}}, SHIFT(415),
  [5581] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2266),
  [5583] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ordered_field_declaration_list, 4, 0, 177),
  [5585] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ordered_field_declaration_list, 2, 0, 0),
  [5587] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3644),
  [5589] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3254),
  [5591] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3296),
  [5593] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3540),
  [5595] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ordered_field_declaration_list, 3, 0, 22),
  [5597] = {.entry = {.count = 1, .reusable = true}}, SHIFT(554),
  [5599] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2556),
  [5601] = {.entry = {.count = 1, .reusable = true}}, SHIFT(561),
  [5603] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_where_clause_repeat1, 2, 0, 0),
  [5605] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_where_clause_repeat1, 2, 0, 0), SHIFT_REPEAT(1448),
  [5608] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_where_clause, 2, 0, 0),
  [5610] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1052),
  [5612] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1101),
  [5614] = {.entry = {.count = 1, .reusable = true}}, SHIFT(948),
  [5616] = {.entry = {.count = 1, .reusable = true}}, SHIFT(344),
  [5618] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2672),
  [5620] = {.entry = {.count = 1, .reusable = true}}, SHIFT(752),
  [5622] = {.entry = {.count = 1, .reusable = true}}, SHIFT(407),
  [5624] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_type_parameters_repeat1, 3, 0, 120),
  [5626] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_type_arguments_repeat1, 2, 0, 0),
  [5628] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1056),
  [5630] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ordered_field_declaration_list, 6, 0, 220),
  [5632] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ordered_field_declaration_list, 6, 0, 103),
  [5634] = {.entry = {.count = 1, .reusable = true}}, SHIFT(629),
  [5636] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ordered_field_declaration_list, 6, 0, 252),
  [5638] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2483),
  [5640] = {.entry = {.count = 1, .reusable = true}}, SHIFT(774),
  [5642] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2161),
  [5644] = {.entry = {.count = 1, .reusable = true}}, SHIFT(692),
  [5646] = {.entry = {.count = 1, .reusable = true}}, SHIFT(336),
  [5648] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2590),
  [5650] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2996),
  [5652] = {.entry = {.count = 1, .reusable = false}}, SHIFT(155),
  [5654] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1942),
  [5656] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1303),
  [5658] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1472),
  [5660] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2968),
  [5662] = {.entry = {.count = 1, .reusable = true}}, SHIFT(902),
  [5664] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameter, 3, 0, 61),
  [5666] = {.entry = {.count = 1, .reusable = true}}, SHIFT(940),
  [5668] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1174),
  [5670] = {.entry = {.count = 1, .reusable = true}}, SHIFT(351),
  [5672] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2696),
  [5674] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2608),
  [5676] = {.entry = {.count = 1, .reusable = true}}, SHIFT(921),
  [5678] = {.entry = {.count = 1, .reusable = true}}, SHIFT(202),
  [5680] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1184),
  [5682] = {.entry = {.count = 1, .reusable = true}}, SHIFT(99),
  [5684] = {.entry = {.count = 1, .reusable = true}}, SHIFT(100),
  [5686] = {.entry = {.count = 1, .reusable = true}}, SHIFT(101),
  [5688] = {.entry = {.count = 1, .reusable = true}}, SHIFT(881),
  [5690] = {.entry = {.count = 1, .reusable = true}}, SHIFT(204),
  [5692] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1542),
  [5694] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1588),
  [5696] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1930),
  [5698] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1720),
  [5700] = {.entry = {.count = 1, .reusable = true}}, SHIFT(850),
  [5702] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_type_parameters_repeat1, 2, 0, 0),
  [5704] = {.entry = {.count = 1, .reusable = true}}, SHIFT(518),
  [5706] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_binding, 4, 0, 199),
  [5708] = {.entry = {.count = 1, .reusable = true}}, SHIFT(113),
  [5710] = {.entry = {.count = 1, .reusable = true}}, SHIFT(102),
  [5712] = {.entry = {.count = 1, .reusable = true}}, SHIFT(106),
  [5714] = {.entry = {.count = 1, .reusable = true}}, SHIFT(664),
  [5716] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2453),
  [5718] = {.entry = {.count = 1, .reusable = true}}, SHIFT(775),
  [5720] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1937),
  [5722] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3081),
  [5724] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2958),
  [5726] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2981),
  [5728] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2391),
  [5730] = {.entry = {.count = 1, .reusable = true}}, SHIFT(114),
  [5732] = {.entry = {.count = 1, .reusable = true}}, SHIFT(115),
  [5734] = {.entry = {.count = 1, .reusable = true}}, SHIFT(116),
  [5736] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2393),
  [5738] = {.entry = {.count = 1, .reusable = true}}, SHIFT(121),
  [5740] = {.entry = {.count = 1, .reusable = true}}, SHIFT(130),
  [5742] = {.entry = {.count = 1, .reusable = true}}, SHIFT(131),
  [5744] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_parameters_repeat1, 3, 0, 0),
  [5746] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2515),
  [5748] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2474),
  [5750] = {.entry = {.count = 1, .reusable = true}}, SHIFT(773),
  [5752] = {.entry = {.count = 1, .reusable = true}}, SHIFT(182),
  [5754] = {.entry = {.count = 1, .reusable = true}}, SHIFT(122),
  [5756] = {.entry = {.count = 1, .reusable = true}}, SHIFT(123),
  [5758] = {.entry = {.count = 1, .reusable = true}}, SHIFT(84),
  [5760] = {.entry = {.count = 1, .reusable = true}}, SHIFT(185),
  [5762] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1041),
  [5764] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3086),
  [5766] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1789),
  [5768] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1794),
  [5770] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2516),
  [5772] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2035),
  [5774] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1923),
  [5776] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_rule, 3, 0, 48),
  [5778] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_slice_pattern_repeat1, 2, 0, 0),
  [5780] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2068),
  [5782] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_slice_pattern_repeat1, 2, 0, 0), SHIFT_REPEAT(819),
  [5785] = {.entry = {.count = 1, .reusable = true}}, SHIFT(618),
  [5787] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2022),
  [5789] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2031),
  [5791] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1545),
  [5793] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1071),
  [5795] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameter, 4, 0, 101),
  [5797] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1921),
  [5799] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3588),
  [5801] = {.entry = {.count = 1, .reusable = true}}, SHIFT(13),
  [5803] = {.entry = {.count = 1, .reusable = true}}, SHIFT(832),
  [5805] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2552),
  [5807] = {.entry = {.count = 1, .reusable = true}}, SHIFT(190),
  [5809] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3185),
  [5811] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_type_parameters_repeat1, 2, 0, 0), REDUCE(aux_sym_for_lifetimes_repeat1, 2, 0, 0),
  [5814] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2099),
  [5816] = {.entry = {.count = 1, .reusable = true}}, SHIFT(788),
  [5818] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2100),
  [5820] = {.entry = {.count = 1, .reusable = true}}, SHIFT(798),
  [5822] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3044),
  [5824] = {.entry = {.count = 1, .reusable = false}}, SHIFT(156),
  [5826] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1229),
  [5828] = {.entry = {.count = 1, .reusable = true}}, SHIFT(354),
  [5830] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2704),
  [5832] = {.entry = {.count = 1, .reusable = true}}, SHIFT(592),
  [5834] = {.entry = {.count = 1, .reusable = true}}, SHIFT(262),
  [5836] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2535),
  [5838] = {.entry = {.count = 1, .reusable = true}}, SHIFT(8),
  [5840] = {.entry = {.count = 1, .reusable = true}}, SHIFT(817),
  [5842] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2716),
  [5844] = {.entry = {.count = 1, .reusable = true}}, SHIFT(187),
  [5846] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2763),
  [5848] = {.entry = {.count = 1, .reusable = false}}, SHIFT(160),
  [5850] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1236),
  [5852] = {.entry = {.count = 1, .reusable = true}}, SHIFT(990),
  [5854] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_parameters_repeat1, 2, 0, 0),
  [5856] = {.entry = {.count = 1, .reusable = true}}, SHIFT(865),
  [5858] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2804),
  [5860] = {.entry = {.count = 1, .reusable = false}}, SHIFT(161),
  [5862] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2786),
  [5864] = {.entry = {.count = 1, .reusable = true}}, SHIFT(796),
  [5866] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1253),
  [5868] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1924),
  [5870] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_binding, 3, 0, 148),
  [5872] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1549),
  [5874] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_arguments_repeat1, 2, 0, 0), SHIFT_REPEAT(205),
  [5877] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1451),
  [5879] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1049),
  [5881] = {.entry = {.count = 1, .reusable = true}}, SHIFT(29),
  [5883] = {.entry = {.count = 1, .reusable = true}}, SHIFT(834),
  [5885] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2577),
  [5887] = {.entry = {.count = 1, .reusable = true}}, SHIFT(193),
  [5889] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_type_parameters_repeat1, 3, 0, 0),
  [5891] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2792),
  [5893] = {.entry = {.count = 1, .reusable = true}}, SHIFT(799),
  [5895] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2144),
  [5897] = {.entry = {.count = 1, .reusable = true}}, SHIFT(768),
  [5899] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_where_predicate, 2, 0, 79),
  [5901] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_where_predicate, 2, 0, 78),
  [5903] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1989),
  [5905] = {.entry = {.count = 1, .reusable = true}}, SHIFT(851),
  [5907] = {.entry = {.count = 1, .reusable = true}}, SHIFT(33),
  [5909] = {.entry = {.count = 1, .reusable = true}}, SHIFT(804),
  [5911] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2613),
  [5913] = {.entry = {.count = 1, .reusable = true}}, SHIFT(196),
  [5915] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_string_literal_repeat1, 2, 0, 0),
  [5917] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_string_literal_repeat1, 2, 0, 0), SHIFT_REPEAT(2968),
  [5920] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2977),
  [5922] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2976),
  [5924] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2852),
  [5926] = {.entry = {.count = 1, .reusable = true}}, SHIFT(765),
  [5928] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2855),
  [5930] = {.entry = {.count = 1, .reusable = true}}, SHIFT(791),
  [5932] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1378),
  [5934] = {.entry = {.count = 1, .reusable = true}}, SHIFT(845),
  [5936] = {.entry = {.count = 1, .reusable = true}}, SHIFT(968),
  [5938] = {.entry = {.count = 1, .reusable = true}}, SHIFT(973),
  [5940] = {.entry = {.count = 1, .reusable = true}}, SHIFT(715),
  [5942] = {.entry = {.count = 1, .reusable = true}}, SHIFT(981),
  [5944] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1920),
  [5946] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3432),
  [5948] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1663),
  [5950] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2138),
  [5952] = {.entry = {.count = 1, .reusable = true}}, SHIFT(994),
  [5954] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1148),
  [5956] = {.entry = {.count = 1, .reusable = true}}, SHIFT(605),
  [5958] = {.entry = {.count = 1, .reusable = true}}, SHIFT(754),
  [5960] = {.entry = {.count = 1, .reusable = true}}, SHIFT(161),
  [5962] = {.entry = {.count = 1, .reusable = true}}, SHIFT(922),
  [5964] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_scoped_use_list, 2, 0, 36),
  [5966] = {.entry = {.count = 1, .reusable = true}}, SHIFT(756),
  [5968] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_const_parameter, 4, 0, 111),
  [5970] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1013),
  [5972] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_struct_pattern_repeat1, 2, 0, 0),
  [5974] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_struct_pattern_repeat1, 2, 0, 0), SHIFT_REPEAT(2395),
  [5977] = {.entry = {.count = 1, .reusable = true}}, SHIFT(766),
  [5979] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3073),
  [5981] = {.entry = {.count = 1, .reusable = true}}, SHIFT(795),
  [5983] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2791),
  [5985] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2262),
  [5987] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2794),
  [5989] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2263),
  [5991] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_list, 5, 0, 0),
  [5993] = {.entry = {.count = 1, .reusable = true}}, SHIFT(847),
  [5995] = {.entry = {.count = 1, .reusable = true}}, SHIFT(722),
  [5997] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_optional_type_parameter, 3, 0, 118),
  [5999] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1161),
  [6001] = {.entry = {.count = 1, .reusable = true}}, SHIFT(350),
  [6003] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_list, 3, 0, 0),
  [6005] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1162),
  [6007] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2176),
  [6009] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_field_initializer_list_repeat1, 2, 0, 0),
  [6011] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_field_initializer_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2156),
  [6014] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3082),
  [6016] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2049),
  [6018] = {.entry = {.count = 1, .reusable = true}}, SHIFT(620),
  [6020] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1170),
  [6022] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3459),
  [6024] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3269),
  [6026] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1177),
  [6028] = {.entry = {.count = 1, .reusable = true}}, SHIFT(352),
  [6030] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1035),
  [6032] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2178),
  [6034] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1179),
  [6036] = {.entry = {.count = 1, .reusable = true}}, SHIFT(163),
  [6038] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1187),
  [6040] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1191),
  [6042] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2123),
  [6044] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_constrained_type_parameter, 2, 0, 78),
  [6046] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1200),
  [6048] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2052),
  [6050] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1732),
  [6052] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_wildcard, 3, 0, 1),
  [6054] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_scoped_use_list, 3, 0, 90),
  [6056] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_as_clause, 3, 0, 91),
  [6058] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_wildcard, 3, 0, 0),
  [6060] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_scoped_use_list, 3, 0, 92),
  [6062] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_as_clause, 3, 0, 93),
  [6064] = {.entry = {.count = 1, .reusable = true}}, SHIFT(168),
  [6066] = {.entry = {.count = 1, .reusable = true}}, SHIFT(530),
  [6068] = {.entry = {.count = 1, .reusable = true}}, SHIFT(439),
  [6070] = {.entry = {.count = 1, .reusable = true}}, SHIFT(800),
  [6072] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2290),
  [6074] = {.entry = {.count = 1, .reusable = true}}, SHIFT(797),
  [6076] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1911),
  [6078] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3017),
  [6080] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2291),
  [6082] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3424),
  [6084] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3057),
  [6086] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3415),
  [6088] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_constrained_type_parameter, 2, 0, 79),
  [6090] = {.entry = {.count = 1, .reusable = true}}, SHIFT(532),
  [6092] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2117),
  [6094] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2179),
  [6096] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_parameters_repeat1, 2, 0, 0), SHIFT_REPEAT(416),
  [6099] = {.entry = {.count = 1, .reusable = true}}, SHIFT(882),
  [6101] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2046),
  [6103] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2048),
  [6105] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1221),
  [6107] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1227),
  [6109] = {.entry = {.count = 1, .reusable = true}}, SHIFT(686),
  [6111] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1232),
  [6113] = {.entry = {.count = 1, .reusable = true}}, SHIFT(355),
  [6115] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2180),
  [6117] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1238),
  [6119] = {.entry = {.count = 1, .reusable = true}}, SHIFT(656),
  [6121] = {.entry = {.count = 1, .reusable = true}}, SHIFT(266),
  [6123] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1248),
  [6125] = {.entry = {.count = 1, .reusable = true}}, SHIFT(356),
  [6127] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1251),
  [6129] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_pattern, 1, 0, 59),
  [6131] = {.entry = {.count = 1, .reusable = true}}, SHIFT(812),
  [6133] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2145),
  [6135] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2252),
  [6137] = {.entry = {.count = 1, .reusable = true}}, SHIFT(491),
  [6139] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_shorthand_field_initializer, 2, 0, 0),
  [6141] = {.entry = {.count = 1, .reusable = true}}, SHIFT(346),
  [6143] = {.entry = {.count = 1, .reusable = true}}, SHIFT(169),
  [6145] = {.entry = {.count = 1, .reusable = true}}, SHIFT(604),
  [6147] = {.entry = {.count = 1, .reusable = true}}, SHIFT(265),
  [6149] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_wildcard, 1, 0, 0),
  [6151] = {.entry = {.count = 1, .reusable = true}}, SHIFT(444),
  [6153] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2043),
  [6155] = {.entry = {.count = 1, .reusable = true}}, SHIFT(493),
  [6157] = {.entry = {.count = 1, .reusable = true}}, SHIFT(764),
  [6159] = {.entry = {.count = 1, .reusable = true}}, SHIFT(413),
  [6161] = {.entry = {.count = 1, .reusable = true}}, SHIFT(148),
  [6163] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2182),
  [6165] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1270),
  [6167] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1272),
  [6169] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1278),
  [6171] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1280),
  [6173] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1285),
  [6175] = {.entry = {.count = 1, .reusable = true}}, SHIFT(357),
  [6177] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2183),
  [6179] = {.entry = {.count = 1, .reusable = true}}, SHIFT(221),
  [6181] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__condition, 1, 0, 9),
  [6183] = {.entry = {.count = 1, .reusable = true}}, SHIFT(732),
  [6185] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1317),
  [6187] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1319),
  [6189] = {.entry = {.count = 1, .reusable = true}}, SHIFT(610),
  [6191] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1325),
  [6193] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1327),
  [6195] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1333),
  [6197] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1335),
  [6199] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1342),
  [6201] = {.entry = {.count = 1, .reusable = true}}, SHIFT(358),
  [6203] = {.entry = {.count = 1, .reusable = true}}, SHIFT(408),
  [6205] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1351),
  [6207] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1353),
  [6209] = {.entry = {.count = 1, .reusable = true}}, SHIFT(669),
  [6211] = {.entry = {.count = 1, .reusable = true}}, SHIFT(320),
  [6213] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2102),
  [6215] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2309),
  [6217] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_shorthand_field_initializer, 1, 0, 0),
  [6219] = {.entry = {.count = 1, .reusable = true}}, SHIFT(314),
  [6221] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3083),
  [6223] = {.entry = {.count = 1, .reusable = true}}, SHIFT(283),
  [6225] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2007),
  [6227] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1476),
  [6229] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2109),
  [6231] = {.entry = {.count = 1, .reusable = true}}, SHIFT(862),
  [6233] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_match_pattern, 1, 0, 0),
  [6235] = {.entry = {.count = 1, .reusable = true}}, SHIFT(808),
  [6237] = {.entry = {.count = 1, .reusable = true}}, SHIFT(183),
  [6239] = {.entry = {.count = 1, .reusable = true}}, SHIFT(582),
  [6241] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_optional_type_parameter, 3, 0, 119),
  [6243] = {.entry = {.count = 1, .reusable = true}}, SHIFT(761),
  [6245] = {.entry = {.count = 1, .reusable = true}}, SHIFT(362),
  [6247] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1721),
  [6249] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_type_parameters_repeat1, 2, 0, 0), SHIFT_REPEAT(2073),
  [6252] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_ordered_field_declaration_list_repeat1, 3, 0, 64),
  [6254] = {.entry = {.count = 1, .reusable = true}}, SHIFT(852),
  [6256] = {.entry = {.count = 1, .reusable = true}}, SHIFT(569),
  [6258] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2508),
  [6260] = {.entry = {.count = 1, .reusable = true}}, SHIFT(778),
  [6262] = {.entry = {.count = 1, .reusable = true}}, SHIFT(661),
  [6264] = {.entry = {.count = 1, .reusable = true}}, SHIFT(446),
  [6266] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_for_lifetimes_repeat1, 2, 0, 0),
  [6268] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_for_lifetimes_repeat1, 2, 0, 0), SHIFT_REPEAT(3187),
  [6271] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_declaration, 4, 0, 221),
  [6273] = {.entry = {.count = 1, .reusable = true}}, SHIFT(447),
  [6275] = {.entry = {.count = 1, .reusable = true}}, SHIFT(406),
  [6277] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_pattern, 4, 0, 200),
  [6279] = {.entry = {.count = 1, .reusable = true}}, SHIFT(702),
  [6281] = {.entry = {.count = 1, .reusable = true}}, SHIFT(342),
  [6283] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3136),
  [6285] = {.entry = {.count = 1, .reusable = true}}, SHIFT(540),
  [6287] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_self_parameter, 4, 0, 0),
  [6289] = {.entry = {.count = 1, .reusable = true}}, SHIFT(803),
  [6291] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3575),
  [6293] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3335),
  [6295] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3625),
  [6297] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2038),
  [6299] = {.entry = {.count = 1, .reusable = true}}, SHIFT(542),
  [6301] = {.entry = {.count = 1, .reusable = true}}, SHIFT(705),
  [6303] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2189),
  [6305] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2328),
  [6307] = {.entry = {.count = 1, .reusable = true}}, SHIFT(499),
  [6309] = {.entry = {.count = 1, .reusable = true}}, SHIFT(282),
  [6311] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3156),
  [6313] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_list, 4, 0, 0),
  [6315] = {.entry = {.count = 1, .reusable = true}}, SHIFT(548),
  [6317] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3087),
  [6319] = {.entry = {.count = 1, .reusable = true}}, SHIFT(550),
  [6321] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2295),
  [6323] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2296),
  [6325] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2297),
  [6327] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3236),
  [6329] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1076),
  [6331] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1388),
  [6333] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1075),
  [6335] = {.entry = {.count = 1, .reusable = true}}, SHIFT(147),
  [6337] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_tuple_pattern_repeat1, 2, 0, 0),
  [6339] = {.entry = {.count = 1, .reusable = true}}, SHIFT(557),
  [6341] = {.entry = {.count = 1, .reusable = true}}, SHIFT(286),
  [6343] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_tuple_pattern_repeat1, 2, 0, 0), SHIFT_REPEAT(772),
  [6346] = {.entry = {.count = 1, .reusable = true}}, SHIFT(641),
  [6348] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1138),
  [6350] = {.entry = {.count = 1, .reusable = true}}, SHIFT(918),
  [6352] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_string_literal_repeat1, 1, 0, 0),
  [6354] = {.entry = {.count = 1, .reusable = true}}, SHIFT(672),
  [6356] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2190),
  [6358] = {.entry = {.count = 1, .reusable = true}}, SHIFT(589),
  [6360] = {.entry = {.count = 1, .reusable = true}}, SHIFT(710),
  [6362] = {.entry = {.count = 1, .reusable = true}}, SHIFT(445),
  [6364] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3253),
  [6366] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2274),
  [6368] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2275),
  [6370] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2253),
  [6372] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2276),
  [6374] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2277),
  [6376] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2257),
  [6378] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2469),
  [6380] = {.entry = {.count = 1, .reusable = true}}, SHIFT(777),
  [6382] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_closure_parameters_repeat1, 2, 0, 0),
  [6384] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2292),
  [6386] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2293),
  [6388] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2294),
  [6390] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2210),
  [6392] = {.entry = {.count = 1, .reusable = true}}, SHIFT(801),
  [6394] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_closure_parameters_repeat1, 2, 0, 0), SHIFT_REPEAT(790),
  [6397] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_ordered_field_declaration_list_repeat1, 2, 0, 22),
  [6399] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1102),
  [6401] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_pattern, 2, 0, 102),
  [6403] = {.entry = {.count = 1, .reusable = true}}, SHIFT(821),
  [6405] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2269),
  [6407] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3475),
  [6409] = {.entry = {.count = 1, .reusable = true}}, SHIFT(936),
  [6411] = {.entry = {.count = 1, .reusable = true}}, SHIFT(802),
  [6413] = {.entry = {.count = 1, .reusable = true}}, SHIFT(156),
  [6415] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_ordered_field_declaration_list_repeat1, 2, 0, 178),
  [6417] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_ordered_field_declaration_list_repeat1, 2, 0, 178), SHIFT_REPEAT(779),
  [6420] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2302),
  [6422] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_tuple_type_repeat1, 2, 0, 0),
  [6424] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_tuple_type_repeat1, 2, 0, 0), SHIFT_REPEAT(995),
  [6427] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_declaration, 3, 0, 179),
  [6429] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_use_list_repeat1, 2, 0, 0),
  [6431] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_use_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1870),
  [6434] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2479),
  [6436] = {.entry = {.count = 1, .reusable = true}}, SHIFT(776),
  [6438] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_pattern, 5, 0, 243),
  [6440] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1612),
  [6442] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1856),
  [6444] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1598),
  [6446] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_enum_variant_list_repeat2, 2, 0, 0),
  [6448] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_enum_variant_list_repeat2, 2, 0, 0), SHIFT_REPEAT(2205),
  [6451] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1910),
  [6453] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3056),
  [6455] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_field_declaration_list_repeat1, 2, 0, 0),
  [6457] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_field_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2235),
  [6460] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2148),
  [6462] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_variant, 3, 0, 34),
  [6464] = {.entry = {.count = 1, .reusable = true}}, SHIFT(280),
  [6466] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2185),
  [6468] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2172),
  [6470] = {.entry = {.count = 1, .reusable = true}}, SHIFT(348),
  [6472] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1379),
  [6474] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_list, 2, 0, 0),
  [6476] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_variant, 2, 0, 11),
  [6478] = {.entry = {.count = 1, .reusable = true}}, SHIFT(260),
  [6480] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2150),
  [6482] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3084),
  [6484] = {.entry = {.count = 1, .reusable = true}}, SHIFT(440),
  [6486] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1012),
  [6488] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_ordered_field_declaration_list_repeat1, 4, 0, 103),
  [6490] = {.entry = {.count = 1, .reusable = true}}, SHIFT(718),
  [6492] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_type_arguments_repeat1, 2, 0, 0), SHIFT_REPEAT(449),
  [6495] = {.entry = {.count = 1, .reusable = true}}, SHIFT(743),
  [6497] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_pattern, 3, 0, 149),
  [6499] = {.entry = {.count = 1, .reusable = true}}, SHIFT(836),
  [6501] = {.entry = {.count = 1, .reusable = true}}, SHIFT(159),
  [6503] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1127),
  [6505] = {.entry = {.count = 1, .reusable = true}}, SHIFT(966),
  [6507] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2785),
  [6509] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1811),
  [6511] = {.entry = {.count = 1, .reusable = true}}, SHIFT(969),
  [6513] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1130),
  [6515] = {.entry = {.count = 1, .reusable = true}}, SHIFT(974),
  [6517] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_pattern, 3, 0, 150),
  [6519] = {.entry = {.count = 1, .reusable = true}}, SHIFT(982),
  [6521] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3585),
  [6523] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3338),
  [6525] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3631),
  [6527] = {.entry = {.count = 1, .reusable = true}}, SHIFT(567),
  [6529] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1909),
  [6531] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3357),
  [6533] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3547),
  [6535] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2347),
  [6537] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1208),
  [6539] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1209),
  [6541] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2075),
  [6543] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1038),
  [6545] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1551),
  [6547] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1571),
  [6549] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1073),
  [6551] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1060),
  [6553] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1553),
  [6555] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1585),
  [6557] = {.entry = {.count = 1, .reusable = true}}, SHIFT(335),
  [6559] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1925),
  [6561] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_variadic_parameter, 3, 0, 152),
  [6563] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2159),
  [6565] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1940),
  [6567] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1550),
  [6569] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1569),
  [6571] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2015),
  [6573] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3079),
  [6575] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1943),
  [6577] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1987),
  [6579] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2528),
  [6581] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1932),
  [6583] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3067),
  [6585] = {.entry = {.count = 1, .reusable = true}}, SHIFT(741),
  [6587] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3430),
  [6589] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2529),
  [6591] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2162),
  [6593] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1928),
  [6595] = {.entry = {.count = 1, .reusable = true}}, SHIFT(361),
  [6597] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_field_declaration_list_repeat1, 3, 0, 0),
  [6599] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_type_arguments_repeat1, 3, 0, 0),
  [6601] = {.entry = {.count = 1, .reusable = true}}, SHIFT(318),
  [6603] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2744),
  [6605] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1941),
  [6607] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2658),
  [6609] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1544),
  [6611] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1939),
  [6613] = {.entry = {.count = 1, .reusable = true}}, SHIFT(257),
  [6615] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1934),
  [6617] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2028),
  [6619] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3401),
  [6621] = {.entry = {.count = 1, .reusable = true}}, SHIFT(748),
  [6623] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1554),
  [6625] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2992),
  [6627] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3387),
  [6629] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3413),
  [6631] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1046),
  [6633] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1944),
  [6635] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2067),
  [6637] = {.entry = {.count = 1, .reusable = true}}, SHIFT(924),
  [6639] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1042),
  [6641] = {.entry = {.count = 1, .reusable = true}}, SHIFT(750),
  [6643] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2030),
  [6645] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_variadic_parameter, 4, 0, 32),
  [6647] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2024),
  [6649] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2027),
  [6651] = {.entry = {.count = 1, .reusable = true}}, SHIFT(637),
  [6653] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3434),
  [6655] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1534),
  [6657] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1535),
  [6659] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1547),
  [6661] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2066),
  [6663] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2158),
  [6665] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2033),
  [6667] = {.entry = {.count = 1, .reusable = true}}, SHIFT(313),
  [6669] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2069),
  [6671] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2321),
  [6673] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2026),
  [6675] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2023),
  [6677] = {.entry = {.count = 1, .reusable = true}}, SHIFT(871),
  [6679] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2029),
  [6681] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2036),
  [6683] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1543),
  [6685] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1546),
  [6687] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2575),
  [6689] = {.entry = {.count = 1, .reusable = true}}, SHIFT(781),
  [6691] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3174),
  [6693] = {.entry = {.count = 1, .reusable = true}}, SHIFT(931),
  [6695] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2703),
  [6697] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2626),
  [6699] = {.entry = {.count = 1, .reusable = true}}, SHIFT(241),
  [6701] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_enum_variant_list_repeat2, 3, 0, 0),
  [6703] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1145),
  [6705] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3450),
  [6707] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1064),
  [6709] = {.entry = {.count = 1, .reusable = true}}, SHIFT(296),
  [6711] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_variadic_parameter, 1, 0, 0),
  [6713] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1541),
  [6715] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__block_doc_comment_marker, 1, 0, 3),
  [6717] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_variadic_parameter, 2, 0, 0),
  [6719] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3365),
  [6721] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3371),
  [6723] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_qualified_type, 3, 0, 66),
  [6725] = {.entry = {.count = 1, .reusable = true}}, SHIFT(317),
  [6727] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1914),
  [6729] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3601),
  [6731] = {.entry = {.count = 1, .reusable = true}}, SHIFT(787),
  [6733] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3186),
  [6735] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2624),
  [6737] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2650),
  [6739] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__block_doc_comment_marker, 1, 0, 2),
  [6741] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1078),
  [6743] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3199),
  [6745] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1207),
  [6747] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3473),
  [6749] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3204),
  [6751] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2585),
  [6753] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1917),
  [6755] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3568),
  [6757] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2660),
  [6759] = {.entry = {.count = 1, .reusable = true}}, SHIFT(343),
  [6761] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2674),
  [6763] = {.entry = {.count = 1, .reusable = true}}, SHIFT(316),
  [6765] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2694),
  [6767] = {.entry = {.count = 1, .reusable = true}}, SHIFT(353),
  [6769] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1913),
  [6771] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3610),
  [6773] = {.entry = {.count = 1, .reusable = true}}, SHIFT(360),
  [6775] = {.entry = {.count = 1, .reusable = true}}, SHIFT(363),
  [6777] = {.entry = {.count = 1, .reusable = true}}, SHIFT(366),
  [6779] = {.entry = {.count = 1, .reusable = true}}, SHIFT(368),
  [6781] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3647),
  [6783] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3479),
  [6785] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2034),
  [6787] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3128),
  [6789] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3595),
  [6791] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3636),
  [6793] = {.entry = {.count = 1, .reusable = true}}, SHIFT(944),
  [6795] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3102),
  [6797] = {.entry = {.count = 1, .reusable = true}}, SHIFT(386),
  [6799] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2991),
  [6801] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2195),
  [6803] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2203),
  [6805] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2204),
  [6807] = {.entry = {.count = 1, .reusable = true}}, SHIFT(647),
  [6809] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2208),
  [6811] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3098),
  [6813] = {.entry = {.count = 1, .reusable = true}}, SHIFT(337),
  [6815] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3372),
  [6817] = {.entry = {.count = 1, .reusable = true}}, SHIFT(900),
  [6819] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1178),
  [6821] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1462),
  [6823] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2965),
  [6825] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2157),
  [6827] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2187),
  [6829] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1260),
  [6831] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3424),
  [6833] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3297),
  [6835] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1231),
  [6837] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3093),
  [6839] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1368),
  [6841] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1261),
  [6843] = {.entry = {.count = 1, .reusable = true}}, SHIFT(954),
  [6845] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1375),
  [6847] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2967),
  [6849] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2357),
  [6851] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2212),
  [6853] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3122),
  [6855] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3232),
  [6857] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2359),
  [6859] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2760),
  [6861] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2174),
  [6863] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2361),
  [6865] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3168),
  [6867] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3043),
  [6869] = {.entry = {.count = 1, .reusable = true}}, SHIFT(502),
  [6871] = {.entry = {.count = 1, .reusable = true}}, SHIFT(880),
  [6873] = {.entry = {.count = 1, .reusable = true}}, SHIFT(619),
  [6875] = {.entry = {.count = 1, .reusable = true}}, SHIFT(668),
  [6877] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__line_doc_comment_marker, 1, 0, 3),
  [6879] = {.entry = {.count = 1, .reusable = true}}, SHIFT(909),
  [6881] = {.entry = {.count = 1, .reusable = true}}, SHIFT(970),
  [6883] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2609),
  [6885] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3069),
  [6887] = {.entry = {.count = 1, .reusable = true}}, SHIFT(704),
  [6889] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1185),
  [6891] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1781),
  [6893] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3482),
  [6895] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1186),
  [6897] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2336),
  [6899] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2249),
  [6901] = {.entry = {.count = 1, .reusable = true}}, SHIFT(706),
  [6903] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3357),
  [6905] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1783),
  [6907] = {.entry = {.count = 1, .reusable = true}}, SHIFT(435),
  [6909] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2217),
  [6911] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3302),
  [6913] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2820),
  [6915] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1109),
  [6917] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3088),
  [6919] = {.entry = {.count = 1, .reusable = true}}, SHIFT(998),
  [6921] = {.entry = {.count = 1, .reusable = true}}, SHIFT(587),
  [6923] = {.entry = {.count = 1, .reusable = true}}, SHIFT(184),
  [6925] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2355),
  [6927] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3567),
  [6929] = {.entry = {.count = 1, .reusable = true}}, SHIFT(535),
  [6931] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2000),
  [6933] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3390),
  [6935] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3493),
  [6937] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2653),
  [6939] = {.entry = {.count = 1, .reusable = true}}, SHIFT(496),
  [6941] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1233),
  [6943] = {.entry = {.count = 1, .reusable = true}}, SHIFT(497),
  [6945] = {.entry = {.count = 1, .reusable = true}}, SHIFT(688),
  [6947] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1235),
  [6949] = {.entry = {.count = 1, .reusable = true}}, SHIFT(264),
  [6951] = {.entry = {.count = 1, .reusable = true}}, SHIFT(536),
  [6953] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1283),
  [6955] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1464),
  [6957] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1160),
  [6959] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3512),
  [6961] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1343),
  [6963] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1284),
  [6965] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1015),
  [6967] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2390),
  [6969] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2019),
  [6971] = {.entry = {.count = 1, .reusable = true}},  ACCEPT_INPUT(),
  [6973] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3645),
  [6975] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3072),
  [6977] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1237),
  [6979] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1176),
  [6981] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2570),
  [6983] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1287),
  [6985] = {.entry = {.count = 1, .reusable = true}}, SHIFT(468),
  [6987] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1491),
  [6989] = {.entry = {.count = 1, .reusable = true}}, SHIFT(90),
  [6991] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1016),
  [6993] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3522),
  [6995] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1791),
  [6997] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_bracketed_type, 3, 0, 0),
  [6999] = {.entry = {.count = 1, .reusable = true}}, SHIFT(911),
  [7001] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__inner_line_doc_comment_marker, 1, 0, 0),
  [7003] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3648),
  [7005] = {.entry = {.count = 1, .reusable = true}}, SHIFT(345),
  [7007] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3041),
  [7009] = {.entry = {.count = 1, .reusable = true}}, SHIFT(868),
  [7011] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2611),
  [7013] = {.entry = {.count = 1, .reusable = true}}, SHIFT(599),
  [7015] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1115),
  [7017] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3526),
  [7019] = {.entry = {.count = 1, .reusable = true}}, SHIFT(81),
  [7021] = {.entry = {.count = 1, .reusable = true}}, SHIFT(617),
  [7023] = {.entry = {.count = 1, .reusable = true}}, SHIFT(633),
  [7025] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1301),
  [7027] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1302),
  [7029] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3092),
  [7031] = {.entry = {.count = 1, .reusable = true}}, SHIFT(565),
  [7033] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_attribute, 2, 0, 70),
  [7035] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3178),
  [7037] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3142),
  [7039] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1430),
  [7041] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3104),
  [7043] = {.entry = {.count = 1, .reusable = true}}, SHIFT(660),
  [7045] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1153),
  [7047] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1243),
  [7049] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2644),
  [7051] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3112),
  [7053] = {.entry = {.count = 1, .reusable = true}}, SHIFT(716),
  [7055] = {.entry = {.count = 1, .reusable = true}}, SHIFT(17),
  [7057] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__outer_line_doc_comment_marker, 1, 0, 0),
  [7059] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3274),
  [7061] = {.entry = {.count = 1, .reusable = true}}, SHIFT(628),
  [7063] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1307),
  [7065] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2992),
  [7067] = {.entry = {.count = 1, .reusable = true}}, SHIFT(696),
  [7069] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3593),
  [7071] = {.entry = {.count = 1, .reusable = true}}, SHIFT(203),
  [7073] = {.entry = {.count = 1, .reusable = true}}, SHIFT(709),
  [7075] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3138),
  [7077] = {.entry = {.count = 1, .reusable = true}}, SHIFT(478),
  [7079] = {.entry = {.count = 1, .reusable = true}}, SHIFT(559),
  [7081] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3141),
  [7083] = {.entry = {.count = 1, .reusable = true}}, SHIFT(577),
  [7085] = {.entry = {.count = 1, .reusable = true}}, SHIFT(405),
  [7087] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3162),
  [7089] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_match_pattern, 3, 0, 174),
  [7091] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2652),
  [7093] = {.entry = {.count = 1, .reusable = true}}, SHIFT(21),
  [7095] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3646),
  [7097] = {.entry = {.count = 1, .reusable = true}}, SHIFT(899),
  [7099] = {.entry = {.count = 1, .reusable = true}}, SHIFT(717),
  [7101] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3428),
  [7103] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3194),
  [7105] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3598),
  [7107] = {.entry = {.count = 1, .reusable = true}}, SHIFT(25),
  [7109] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3474),
  [7111] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3208),
  [7113] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3212),
  [7115] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3193),
  [7117] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3211),
  [7119] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2333),
  [7121] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3643),
  [7123] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3469),
  [7125] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3218),
  [7127] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2663),
  [7129] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1915),
  [7131] = {.entry = {.count = 1, .reusable = true}}, SHIFT(522),
  [7133] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1916),
  [7135] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3402),
  [7137] = {.entry = {.count = 1, .reusable = true}}, SHIFT(945),
  [7139] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1250),
  [7141] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2241),
  [7143] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3103),
  [7145] = {.entry = {.count = 1, .reusable = true}}, SHIFT(950),
  [7147] = {.entry = {.count = 1, .reusable = true}}, SHIFT(517),
  [7149] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2684),
  [7151] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1745),
  [7153] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1361),
  [7155] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_attribute, 2, 0, 69),
  [7157] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1601),
  [7159] = {.entry = {.count = 1, .reusable = true}}, SHIFT(516),
  [7161] = {.entry = {.count = 1, .reusable = true}}, SHIFT(962),
  [7163] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1079),
  [7165] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1529),
  [7167] = {.entry = {.count = 1, .reusable = true}}, SHIFT(965),
  [7169] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1322),
  [7171] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1349),
  [7173] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1323),
  [7175] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2392),
  [7177] = {.entry = {.count = 1, .reusable = true}}, SHIFT(971),
  [7179] = {.entry = {.count = 1, .reusable = true}}, SHIFT(285),
  [7181] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2817),
  [7183] = {.entry = {.count = 1, .reusable = true}}, SHIFT(920),
  [7185] = {.entry = {.count = 1, .reusable = true}}, SHIFT(311),
  [7187] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1912),
  [7189] = {.entry = {.count = 1, .reusable = true}}, SHIFT(978),
  [7191] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1694),
  [7193] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2432),
  [7195] = {.entry = {.count = 1, .reusable = true}}, SHIFT(866),
  [7197] = {.entry = {.count = 1, .reusable = true}}, SHIFT(985),
  [7199] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2720),
  [7201] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1919),
  [7203] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1918),
  [7205] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3516),
  [7207] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2299),
  [7209] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2473),
  [7211] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2731),
  [7213] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3539),
  [7215] = {.entry = {.count = 1, .reusable = true}}, SHIFT(281),
  [7217] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1091),
  [7219] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3548),
  [7221] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3552),
  [7223] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3555),
  [7225] = {.entry = {.count = 1, .reusable = true}}, SHIFT(614),
  [7227] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3585),
  [7229] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1948),
  [7231] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2362),
  [7233] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3046),
  [7235] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3318),
  [7237] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3456),
  [7239] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3595),
  [7241] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2232),
  [7243] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2316),
  [7245] = {.entry = {.count = 1, .reusable = true}}, SHIFT(436),
  [7247] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3606),
  [7249] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__line_doc_comment_marker, 1, 0, 2),
  [7251] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1137),
  [7253] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3285),
  [7255] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_line_comment, 2, 0, 0),
  [7257] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_line_comment, 3, 0, 14),
  [7259] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block_comment, 2, 0, 0),
  [7261] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_line_comment, 3, 0, 0),
  [7263] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block_comment, 3, 0, 0),
  [7265] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block_comment, 3, 0, 15),
  [7267] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block_comment, 4, 0, 53),
};

enum ts_external_scanner_symbol_identifiers {
  ts_external_token_string_content = 0,
  ts_external_token__raw_string_literal_start = 1,
  ts_external_token_raw_string_literal_content = 2,
  ts_external_token__raw_string_literal_end = 3,
  ts_external_token_float_literal = 4,
  ts_external_token__outer_block_doc_comment_marker = 5,
  ts_external_token__inner_block_doc_comment_marker = 6,
  ts_external_token__block_comment_content = 7,
  ts_external_token__line_doc_content = 8,
  ts_external_token__error_sentinel = 9,
};

static const TSSymbol ts_external_scanner_symbol_map[EXTERNAL_TOKEN_COUNT] = {
  [ts_external_token_string_content] = sym_string_content,
  [ts_external_token__raw_string_literal_start] = sym__raw_string_literal_start,
  [ts_external_token_raw_string_literal_content] = sym_raw_string_literal_content,
  [ts_external_token__raw_string_literal_end] = sym__raw_string_literal_end,
  [ts_external_token_float_literal] = sym_float_literal,
  [ts_external_token__outer_block_doc_comment_marker] = sym__outer_block_doc_comment_marker,
  [ts_external_token__inner_block_doc_comment_marker] = sym__inner_block_doc_comment_marker,
  [ts_external_token__block_comment_content] = sym__block_comment_content,
  [ts_external_token__line_doc_content] = sym__line_doc_content,
  [ts_external_token__error_sentinel] = sym__error_sentinel,
};

static const bool ts_external_scanner_states[10][EXTERNAL_TOKEN_COUNT] = {
  [1] = {
    [ts_external_token_string_content] = true,
    [ts_external_token__raw_string_literal_start] = true,
    [ts_external_token_raw_string_literal_content] = true,
    [ts_external_token__raw_string_literal_end] = true,
    [ts_external_token_float_literal] = true,
    [ts_external_token__outer_block_doc_comment_marker] = true,
    [ts_external_token__inner_block_doc_comment_marker] = true,
    [ts_external_token__block_comment_content] = true,
    [ts_external_token__line_doc_content] = true,
    [ts_external_token__error_sentinel] = true,
  },
  [2] = {
    [ts_external_token__raw_string_literal_start] = true,
    [ts_external_token_float_literal] = true,
  },
  [3] = {
    [ts_external_token__outer_block_doc_comment_marker] = true,
    [ts_external_token__inner_block_doc_comment_marker] = true,
    [ts_external_token__block_comment_content] = true,
  },
  [4] = {
    [ts_external_token_string_content] = true,
  },
  [5] = {
    [ts_external_token_float_literal] = true,
  },
  [6] = {
    [ts_external_token__block_comment_content] = true,
  },
  [7] = {
    [ts_external_token__line_doc_content] = true,
  },
  [8] = {
    [ts_external_token__raw_string_literal_end] = true,
  },
  [9] = {
    [ts_external_token_raw_string_literal_content] = true,
  },
};

#ifdef __cplusplus
extern "C" {
#endif
void *tree_sitter_rust_external_scanner_create(void);
void tree_sitter_rust_external_scanner_destroy(void *);
bool tree_sitter_rust_external_scanner_scan(void *, TSLexer *, const bool *);
unsigned tree_sitter_rust_external_scanner_serialize(void *, char *);
void tree_sitter_rust_external_scanner_deserialize(void *, const char *, unsigned);

#ifdef TREE_SITTER_HIDE_SYMBOLS
#define TS_PUBLIC
#elif defined(_WIN32)
#define TS_PUBLIC __declspec(dllexport)
#elif defined __GNUC__ || defined __clang__
#define TS_PUBLIC __attribute__((visibility("default")))
#else
#define TS_PUBLIC
#endif

TS_PUBLIC const TSLanguage *tree_sitter_rust(void) {
  static const TSLanguage language = {
    .version = LANGUAGE_VERSION,
    .symbol_count = SYMBOL_COUNT,
    .alias_count = ALIAS_COUNT,
    .token_count = TOKEN_COUNT,
    .external_token_count = EXTERNAL_TOKEN_COUNT,
    .state_count = STATE_COUNT,
    .large_state_count = LARGE_STATE_COUNT,
    .production_id_count = PRODUCTION_ID_COUNT,
    .field_count = FIELD_COUNT,
    .max_alias_sequence_length = MAX_ALIAS_SEQUENCE_LENGTH,
    .parse_table = &ts_parse_table[0][0],
    .small_parse_table = ts_small_parse_table,
    .small_parse_table_map = ts_small_parse_table_map,
    .parse_actions = ts_parse_actions,
    .symbol_names = ts_symbol_names,
    .field_names = ts_field_names,
    .field_map_slices = ts_field_map_slices,
    .field_map_entries = ts_field_map_entries,
    .symbol_metadata = ts_symbol_metadata,
    .public_symbol_map = ts_symbol_map,
    .alias_map = ts_non_terminal_alias_map,
    .alias_sequences = &ts_alias_sequences[0][0],
    .lex_modes = ts_lex_modes,
    .lex_fn = ts_lex,
    .keyword_lex_fn = ts_lex_keywords,
    .keyword_capture_token = sym_identifier,
    .external_scanner = {
      &ts_external_scanner_states[0][0],
      ts_external_scanner_symbol_map,
      tree_sitter_rust_external_scanner_create,
      tree_sitter_rust_external_scanner_destroy,
      tree_sitter_rust_external_scanner_scan,
      tree_sitter_rust_external_scanner_serialize,
      tree_sitter_rust_external_scanner_deserialize,
    },
    .primary_state_ids = ts_primary_state_ids,
  };
  return &language;
}
#ifdef __cplusplus
}
#endif
