#include "tree_sitter/parser.h"

#if defined(__GNUC__) || defined(__clang__)
#pragma GCC diagnostic ignored "-Wmissing-field-initializers"
#endif

#define LANGUAGE_VERSION 14
#define STATE_COUNT 1425
#define LARGE_STATE_COUNT 29
#define SYMBOL_COUNT 214
#define ALIAS_COUNT 5
#define TOKEN_COUNT 95
#define EXTERNAL_TOKEN_COUNT 0
#define FIELD_COUNT 35
#define MAX_ALIAS_SEQUENCE_LENGTH 9
#define PRODUCTION_ID_COUNT 110

enum ts_symbol_identifiers {
  sym_identifier = 1,
  aux_sym_source_file_token1 = 2,
  anon_sym_SEMI = 3,
  anon_sym_NULL = 4,
  anon_sym_package = 5,
  anon_sym_import = 6,
  anon_sym_DOT = 7,
  sym_blank_identifier = 8,
  anon_sym_LPAREN = 9,
  anon_sym_RPAREN = 10,
  anon_sym_const = 11,
  anon_sym_COMMA = 12,
  anon_sym_EQ = 13,
  anon_sym_var = 14,
  anon_sym_func = 15,
  anon_sym_LBRACK = 16,
  anon_sym_RBRACK = 17,
  anon_sym_DOT_DOT_DOT = 18,
  anon_sym_type = 19,
  anon_sym_STAR = 20,
  anon_sym_struct = 21,
  anon_sym_TILDE = 22,
  anon_sym_LBRACE = 23,
  anon_sym_RBRACE = 24,
  anon_sym_interface = 25,
  anon_sym_PIPE = 26,
  anon_sym_map = 27,
  anon_sym_chan = 28,
  anon_sym_LT_DASH = 29,
  anon_sym_COLON_EQ = 30,
  anon_sym_PLUS_PLUS = 31,
  anon_sym_DASH_DASH = 32,
  anon_sym_STAR_EQ = 33,
  anon_sym_SLASH_EQ = 34,
  anon_sym_PERCENT_EQ = 35,
  anon_sym_LT_LT_EQ = 36,
  anon_sym_GT_GT_EQ = 37,
  anon_sym_AMP_EQ = 38,
  anon_sym_AMP_CARET_EQ = 39,
  anon_sym_PLUS_EQ = 40,
  anon_sym_DASH_EQ = 41,
  anon_sym_PIPE_EQ = 42,
  anon_sym_CARET_EQ = 43,
  anon_sym_COLON = 44,
  anon_sym_fallthrough = 45,
  anon_sym_break = 46,
  anon_sym_continue = 47,
  anon_sym_goto = 48,
  anon_sym_return = 49,
  anon_sym_go = 50,
  anon_sym_defer = 51,
  anon_sym_if = 52,
  anon_sym_else = 53,
  anon_sym_for = 54,
  anon_sym_range = 55,
  anon_sym_switch = 56,
  anon_sym_case = 57,
  anon_sym_default = 58,
  anon_sym_select = 59,
  anon_sym_new = 60,
  anon_sym_make = 61,
  anon_sym_PLUS = 62,
  anon_sym_DASH = 63,
  anon_sym_BANG = 64,
  anon_sym_CARET = 65,
  anon_sym_AMP = 66,
  anon_sym_SLASH = 67,
  anon_sym_PERCENT = 68,
  anon_sym_LT_LT = 69,
  anon_sym_GT_GT = 70,
  anon_sym_AMP_CARET = 71,
  anon_sym_EQ_EQ = 72,
  anon_sym_BANG_EQ = 73,
  anon_sym_LT = 74,
  anon_sym_LT_EQ = 75,
  anon_sym_GT = 76,
  anon_sym_GT_EQ = 77,
  anon_sym_AMP_AMP = 78,
  anon_sym_PIPE_PIPE = 79,
  anon_sym_BQUOTE = 80,
  aux_sym_raw_string_literal_token1 = 81,
  anon_sym_DQUOTE = 82,
  aux_sym_interpreted_string_literal_token1 = 83,
  anon_sym_DQUOTE2 = 84,
  sym_escape_sequence = 85,
  sym_int_literal = 86,
  sym_float_literal = 87,
  sym_imaginary_literal = 88,
  sym_rune_literal = 89,
  sym_nil = 90,
  sym_true = 91,
  sym_false = 92,
  sym_iota = 93,
  sym_comment = 94,
  sym_source_file = 95,
  sym_package_clause = 96,
  sym_import_declaration = 97,
  sym_import_spec = 98,
  sym_dot = 99,
  sym_import_spec_list = 100,
  sym__declaration = 101,
  sym_const_declaration = 102,
  sym_const_spec = 103,
  sym_var_declaration = 104,
  sym_var_spec = 105,
  sym_var_spec_list = 106,
  sym_function_declaration = 107,
  sym_method_declaration = 108,
  sym_type_parameter_list = 109,
  sym_type_parameter_declaration = 110,
  sym_parameter_list = 111,
  sym_parameter_declaration = 112,
  sym_variadic_parameter_declaration = 113,
  sym_type_alias = 114,
  sym_type_declaration = 115,
  sym_type_spec = 116,
  sym_expression_list = 117,
  sym_parenthesized_type = 118,
  sym__simple_type = 119,
  sym_generic_type = 120,
  sym_type_arguments = 121,
  sym_pointer_type = 122,
  sym_array_type = 123,
  sym_implicit_length_array_type = 124,
  sym_slice_type = 125,
  sym_struct_type = 126,
  sym_negated_type = 127,
  sym_field_declaration_list = 128,
  sym_field_declaration = 129,
  sym_interface_type = 130,
  sym_method_elem = 131,
  sym_type_elem = 132,
  sym_map_type = 133,
  sym_channel_type = 134,
  sym_function_type = 135,
  sym_block = 136,
  sym__statement_list = 137,
  sym__statement = 138,
  sym_empty_statement = 139,
  sym__simple_statement = 140,
  sym_expression_statement = 141,
  sym_send_statement = 142,
  sym_receive_statement = 143,
  sym_inc_statement = 144,
  sym_dec_statement = 145,
  sym_assignment_statement = 146,
  sym_short_var_declaration = 147,
  sym_labeled_statement = 148,
  sym_empty_labeled_statement = 149,
  sym_fallthrough_statement = 150,
  sym_break_statement = 151,
  sym_continue_statement = 152,
  sym_goto_statement = 153,
  sym_return_statement = 154,
  sym_go_statement = 155,
  sym_defer_statement = 156,
  sym_if_statement = 157,
  sym_for_statement = 158,
  sym_for_clause = 159,
  sym_range_clause = 160,
  sym_expression_switch_statement = 161,
  sym_expression_case = 162,
  sym_default_case = 163,
  sym_type_switch_statement = 164,
  sym__type_switch_header = 165,
  sym_type_case = 166,
  sym_select_statement = 167,
  sym_communication_case = 168,
  sym__expression = 169,
  sym_parenthesized_expression = 170,
  sym_call_expression = 171,
  sym_variadic_argument = 172,
  sym_special_argument_list = 173,
  sym_argument_list = 174,
  sym_selector_expression = 175,
  sym_index_expression = 176,
  sym_slice_expression = 177,
  sym_type_assertion_expression = 178,
  sym_type_conversion_expression = 179,
  sym_type_instantiation_expression = 180,
  sym_composite_literal = 181,
  sym_literal_value = 182,
  sym_literal_element = 183,
  sym_keyed_element = 184,
  sym_func_literal = 185,
  sym_unary_expression = 186,
  sym_binary_expression = 187,
  sym_qualified_type = 188,
  sym_raw_string_literal = 189,
  sym_interpreted_string_literal = 190,
  aux_sym_source_file_repeat1 = 191,
  aux_sym_import_spec_list_repeat1 = 192,
  aux_sym_const_declaration_repeat1 = 193,
  aux_sym_const_spec_repeat1 = 194,
  aux_sym_var_spec_repeat1 = 195,
  aux_sym_var_spec_list_repeat1 = 196,
  aux_sym_type_parameter_list_repeat1 = 197,
  aux_sym_parameter_list_repeat1 = 198,
  aux_sym_type_declaration_repeat1 = 199,
  aux_sym_expression_list_repeat1 = 200,
  aux_sym_type_arguments_repeat1 = 201,
  aux_sym_field_declaration_list_repeat1 = 202,
  aux_sym_field_declaration_repeat1 = 203,
  aux_sym_interface_type_repeat1 = 204,
  aux_sym_type_elem_repeat1 = 205,
  aux_sym__statement_list_repeat1 = 206,
  aux_sym_expression_switch_statement_repeat1 = 207,
  aux_sym_type_switch_statement_repeat1 = 208,
  aux_sym_type_case_repeat1 = 209,
  aux_sym_select_statement_repeat1 = 210,
  aux_sym_argument_list_repeat1 = 211,
  aux_sym_literal_value_repeat1 = 212,
  aux_sym_interpreted_string_literal_repeat1 = 213,
  alias_sym_field_identifier = 214,
  alias_sym_label_name = 215,
  alias_sym_package_identifier = 216,
  alias_sym_type_constraint = 217,
  alias_sym_type_identifier = 218,
};

static const char * const ts_symbol_names[] = {
  [ts_builtin_sym_end] = "end",
  [sym_identifier] = "identifier",
  [aux_sym_source_file_token1] = "source_file_token1",
  [anon_sym_SEMI] = ";",
  [anon_sym_NULL] = "\0",
  [anon_sym_package] = "package",
  [anon_sym_import] = "import",
  [anon_sym_DOT] = ".",
  [sym_blank_identifier] = "blank_identifier",
  [anon_sym_LPAREN] = "(",
  [anon_sym_RPAREN] = ")",
  [anon_sym_const] = "const",
  [anon_sym_COMMA] = ",",
  [anon_sym_EQ] = "=",
  [anon_sym_var] = "var",
  [anon_sym_func] = "func",
  [anon_sym_LBRACK] = "[",
  [anon_sym_RBRACK] = "]",
  [anon_sym_DOT_DOT_DOT] = "...",
  [anon_sym_type] = "type",
  [anon_sym_STAR] = "*",
  [anon_sym_struct] = "struct",
  [anon_sym_TILDE] = "~",
  [anon_sym_LBRACE] = "{",
  [anon_sym_RBRACE] = "}",
  [anon_sym_interface] = "interface",
  [anon_sym_PIPE] = "|",
  [anon_sym_map] = "map",
  [anon_sym_chan] = "chan",
  [anon_sym_LT_DASH] = "<-",
  [anon_sym_COLON_EQ] = ":=",
  [anon_sym_PLUS_PLUS] = "++",
  [anon_sym_DASH_DASH] = "--",
  [anon_sym_STAR_EQ] = "*=",
  [anon_sym_SLASH_EQ] = "/=",
  [anon_sym_PERCENT_EQ] = "%=",
  [anon_sym_LT_LT_EQ] = "<<=",
  [anon_sym_GT_GT_EQ] = ">>=",
  [anon_sym_AMP_EQ] = "&=",
  [anon_sym_AMP_CARET_EQ] = "&^=",
  [anon_sym_PLUS_EQ] = "+=",
  [anon_sym_DASH_EQ] = "-=",
  [anon_sym_PIPE_EQ] = "|=",
  [anon_sym_CARET_EQ] = "^=",
  [anon_sym_COLON] = ":",
  [anon_sym_fallthrough] = "fallthrough",
  [anon_sym_break] = "break",
  [anon_sym_continue] = "continue",
  [anon_sym_goto] = "goto",
  [anon_sym_return] = "return",
  [anon_sym_go] = "go",
  [anon_sym_defer] = "defer",
  [anon_sym_if] = "if",
  [anon_sym_else] = "else",
  [anon_sym_for] = "for",
  [anon_sym_range] = "range",
  [anon_sym_switch] = "switch",
  [anon_sym_case] = "case",
  [anon_sym_default] = "default",
  [anon_sym_select] = "select",
  [anon_sym_new] = "identifier",
  [anon_sym_make] = "identifier",
  [anon_sym_PLUS] = "+",
  [anon_sym_DASH] = "-",
  [anon_sym_BANG] = "!",
  [anon_sym_CARET] = "^",
  [anon_sym_AMP] = "&",
  [anon_sym_SLASH] = "/",
  [anon_sym_PERCENT] = "%",
  [anon_sym_LT_LT] = "<<",
  [anon_sym_GT_GT] = ">>",
  [anon_sym_AMP_CARET] = "&^",
  [anon_sym_EQ_EQ] = "==",
  [anon_sym_BANG_EQ] = "!=",
  [anon_sym_LT] = "<",
  [anon_sym_LT_EQ] = "<=",
  [anon_sym_GT] = ">",
  [anon_sym_GT_EQ] = ">=",
  [anon_sym_AMP_AMP] = "&&",
  [anon_sym_PIPE_PIPE] = "||",
  [anon_sym_BQUOTE] = "`",
  [aux_sym_raw_string_literal_token1] = "raw_string_literal_content",
  [anon_sym_DQUOTE] = "\"",
  [aux_sym_interpreted_string_literal_token1] = "interpreted_string_literal_content",
  [anon_sym_DQUOTE2] = "\"",
  [sym_escape_sequence] = "escape_sequence",
  [sym_int_literal] = "int_literal",
  [sym_float_literal] = "float_literal",
  [sym_imaginary_literal] = "imaginary_literal",
  [sym_rune_literal] = "rune_literal",
  [sym_nil] = "nil",
  [sym_true] = "true",
  [sym_false] = "false",
  [sym_iota] = "iota",
  [sym_comment] = "comment",
  [sym_source_file] = "source_file",
  [sym_package_clause] = "package_clause",
  [sym_import_declaration] = "import_declaration",
  [sym_import_spec] = "import_spec",
  [sym_dot] = "dot",
  [sym_import_spec_list] = "import_spec_list",
  [sym__declaration] = "_declaration",
  [sym_const_declaration] = "const_declaration",
  [sym_const_spec] = "const_spec",
  [sym_var_declaration] = "var_declaration",
  [sym_var_spec] = "var_spec",
  [sym_var_spec_list] = "var_spec_list",
  [sym_function_declaration] = "function_declaration",
  [sym_method_declaration] = "method_declaration",
  [sym_type_parameter_list] = "type_parameter_list",
  [sym_type_parameter_declaration] = "type_parameter_declaration",
  [sym_parameter_list] = "parameter_list",
  [sym_parameter_declaration] = "parameter_declaration",
  [sym_variadic_parameter_declaration] = "variadic_parameter_declaration",
  [sym_type_alias] = "type_alias",
  [sym_type_declaration] = "type_declaration",
  [sym_type_spec] = "type_spec",
  [sym_expression_list] = "expression_list",
  [sym_parenthesized_type] = "parenthesized_type",
  [sym__simple_type] = "_simple_type",
  [sym_generic_type] = "generic_type",
  [sym_type_arguments] = "type_arguments",
  [sym_pointer_type] = "pointer_type",
  [sym_array_type] = "array_type",
  [sym_implicit_length_array_type] = "implicit_length_array_type",
  [sym_slice_type] = "slice_type",
  [sym_struct_type] = "struct_type",
  [sym_negated_type] = "negated_type",
  [sym_field_declaration_list] = "field_declaration_list",
  [sym_field_declaration] = "field_declaration",
  [sym_interface_type] = "interface_type",
  [sym_method_elem] = "method_elem",
  [sym_type_elem] = "type_elem",
  [sym_map_type] = "map_type",
  [sym_channel_type] = "channel_type",
  [sym_function_type] = "function_type",
  [sym_block] = "block",
  [sym__statement_list] = "_statement_list",
  [sym__statement] = "_statement",
  [sym_empty_statement] = "empty_statement",
  [sym__simple_statement] = "_simple_statement",
  [sym_expression_statement] = "expression_statement",
  [sym_send_statement] = "send_statement",
  [sym_receive_statement] = "receive_statement",
  [sym_inc_statement] = "inc_statement",
  [sym_dec_statement] = "dec_statement",
  [sym_assignment_statement] = "assignment_statement",
  [sym_short_var_declaration] = "short_var_declaration",
  [sym_labeled_statement] = "labeled_statement",
  [sym_empty_labeled_statement] = "labeled_statement",
  [sym_fallthrough_statement] = "fallthrough_statement",
  [sym_break_statement] = "break_statement",
  [sym_continue_statement] = "continue_statement",
  [sym_goto_statement] = "goto_statement",
  [sym_return_statement] = "return_statement",
  [sym_go_statement] = "go_statement",
  [sym_defer_statement] = "defer_statement",
  [sym_if_statement] = "if_statement",
  [sym_for_statement] = "for_statement",
  [sym_for_clause] = "for_clause",
  [sym_range_clause] = "range_clause",
  [sym_expression_switch_statement] = "expression_switch_statement",
  [sym_expression_case] = "expression_case",
  [sym_default_case] = "default_case",
  [sym_type_switch_statement] = "type_switch_statement",
  [sym__type_switch_header] = "_type_switch_header",
  [sym_type_case] = "type_case",
  [sym_select_statement] = "select_statement",
  [sym_communication_case] = "communication_case",
  [sym__expression] = "_expression",
  [sym_parenthesized_expression] = "parenthesized_expression",
  [sym_call_expression] = "call_expression",
  [sym_variadic_argument] = "variadic_argument",
  [sym_special_argument_list] = "argument_list",
  [sym_argument_list] = "argument_list",
  [sym_selector_expression] = "selector_expression",
  [sym_index_expression] = "index_expression",
  [sym_slice_expression] = "slice_expression",
  [sym_type_assertion_expression] = "type_assertion_expression",
  [sym_type_conversion_expression] = "type_conversion_expression",
  [sym_type_instantiation_expression] = "type_instantiation_expression",
  [sym_composite_literal] = "composite_literal",
  [sym_literal_value] = "literal_value",
  [sym_literal_element] = "literal_element",
  [sym_keyed_element] = "keyed_element",
  [sym_func_literal] = "func_literal",
  [sym_unary_expression] = "unary_expression",
  [sym_binary_expression] = "binary_expression",
  [sym_qualified_type] = "qualified_type",
  [sym_raw_string_literal] = "raw_string_literal",
  [sym_interpreted_string_literal] = "interpreted_string_literal",
  [aux_sym_source_file_repeat1] = "source_file_repeat1",
  [aux_sym_import_spec_list_repeat1] = "import_spec_list_repeat1",
  [aux_sym_const_declaration_repeat1] = "const_declaration_repeat1",
  [aux_sym_const_spec_repeat1] = "const_spec_repeat1",
  [aux_sym_var_spec_repeat1] = "var_spec_repeat1",
  [aux_sym_var_spec_list_repeat1] = "var_spec_list_repeat1",
  [aux_sym_type_parameter_list_repeat1] = "type_parameter_list_repeat1",
  [aux_sym_parameter_list_repeat1] = "parameter_list_repeat1",
  [aux_sym_type_declaration_repeat1] = "type_declaration_repeat1",
  [aux_sym_expression_list_repeat1] = "expression_list_repeat1",
  [aux_sym_type_arguments_repeat1] = "type_arguments_repeat1",
  [aux_sym_field_declaration_list_repeat1] = "field_declaration_list_repeat1",
  [aux_sym_field_declaration_repeat1] = "field_declaration_repeat1",
  [aux_sym_interface_type_repeat1] = "interface_type_repeat1",
  [aux_sym_type_elem_repeat1] = "type_elem_repeat1",
  [aux_sym__statement_list_repeat1] = "_statement_list_repeat1",
  [aux_sym_expression_switch_statement_repeat1] = "expression_switch_statement_repeat1",
  [aux_sym_type_switch_statement_repeat1] = "type_switch_statement_repeat1",
  [aux_sym_type_case_repeat1] = "type_case_repeat1",
  [aux_sym_select_statement_repeat1] = "select_statement_repeat1",
  [aux_sym_argument_list_repeat1] = "argument_list_repeat1",
  [aux_sym_literal_value_repeat1] = "literal_value_repeat1",
  [aux_sym_interpreted_string_literal_repeat1] = "interpreted_string_literal_repeat1",
  [alias_sym_field_identifier] = "field_identifier",
  [alias_sym_label_name] = "label_name",
  [alias_sym_package_identifier] = "package_identifier",
  [alias_sym_type_constraint] = "type_constraint",
  [alias_sym_type_identifier] = "type_identifier",
};

static const TSSymbol ts_symbol_map[] = {
  [ts_builtin_sym_end] = ts_builtin_sym_end,
  [sym_identifier] = sym_identifier,
  [aux_sym_source_file_token1] = aux_sym_source_file_token1,
  [anon_sym_SEMI] = anon_sym_SEMI,
  [anon_sym_NULL] = anon_sym_NULL,
  [anon_sym_package] = anon_sym_package,
  [anon_sym_import] = anon_sym_import,
  [anon_sym_DOT] = anon_sym_DOT,
  [sym_blank_identifier] = sym_blank_identifier,
  [anon_sym_LPAREN] = anon_sym_LPAREN,
  [anon_sym_RPAREN] = anon_sym_RPAREN,
  [anon_sym_const] = anon_sym_const,
  [anon_sym_COMMA] = anon_sym_COMMA,
  [anon_sym_EQ] = anon_sym_EQ,
  [anon_sym_var] = anon_sym_var,
  [anon_sym_func] = anon_sym_func,
  [anon_sym_LBRACK] = anon_sym_LBRACK,
  [anon_sym_RBRACK] = anon_sym_RBRACK,
  [anon_sym_DOT_DOT_DOT] = anon_sym_DOT_DOT_DOT,
  [anon_sym_type] = anon_sym_type,
  [anon_sym_STAR] = anon_sym_STAR,
  [anon_sym_struct] = anon_sym_struct,
  [anon_sym_TILDE] = anon_sym_TILDE,
  [anon_sym_LBRACE] = anon_sym_LBRACE,
  [anon_sym_RBRACE] = anon_sym_RBRACE,
  [anon_sym_interface] = anon_sym_interface,
  [anon_sym_PIPE] = anon_sym_PIPE,
  [anon_sym_map] = anon_sym_map,
  [anon_sym_chan] = anon_sym_chan,
  [anon_sym_LT_DASH] = anon_sym_LT_DASH,
  [anon_sym_COLON_EQ] = anon_sym_COLON_EQ,
  [anon_sym_PLUS_PLUS] = anon_sym_PLUS_PLUS,
  [anon_sym_DASH_DASH] = anon_sym_DASH_DASH,
  [anon_sym_STAR_EQ] = anon_sym_STAR_EQ,
  [anon_sym_SLASH_EQ] = anon_sym_SLASH_EQ,
  [anon_sym_PERCENT_EQ] = anon_sym_PERCENT_EQ,
  [anon_sym_LT_LT_EQ] = anon_sym_LT_LT_EQ,
  [anon_sym_GT_GT_EQ] = anon_sym_GT_GT_EQ,
  [anon_sym_AMP_EQ] = anon_sym_AMP_EQ,
  [anon_sym_AMP_CARET_EQ] = anon_sym_AMP_CARET_EQ,
  [anon_sym_PLUS_EQ] = anon_sym_PLUS_EQ,
  [anon_sym_DASH_EQ] = anon_sym_DASH_EQ,
  [anon_sym_PIPE_EQ] = anon_sym_PIPE_EQ,
  [anon_sym_CARET_EQ] = anon_sym_CARET_EQ,
  [anon_sym_COLON] = anon_sym_COLON,
  [anon_sym_fallthrough] = anon_sym_fallthrough,
  [anon_sym_break] = anon_sym_break,
  [anon_sym_continue] = anon_sym_continue,
  [anon_sym_goto] = anon_sym_goto,
  [anon_sym_return] = anon_sym_return,
  [anon_sym_go] = anon_sym_go,
  [anon_sym_defer] = anon_sym_defer,
  [anon_sym_if] = anon_sym_if,
  [anon_sym_else] = anon_sym_else,
  [anon_sym_for] = anon_sym_for,
  [anon_sym_range] = anon_sym_range,
  [anon_sym_switch] = anon_sym_switch,
  [anon_sym_case] = anon_sym_case,
  [anon_sym_default] = anon_sym_default,
  [anon_sym_select] = anon_sym_select,
  [anon_sym_new] = sym_identifier,
  [anon_sym_make] = sym_identifier,
  [anon_sym_PLUS] = anon_sym_PLUS,
  [anon_sym_DASH] = anon_sym_DASH,
  [anon_sym_BANG] = anon_sym_BANG,
  [anon_sym_CARET] = anon_sym_CARET,
  [anon_sym_AMP] = anon_sym_AMP,
  [anon_sym_SLASH] = anon_sym_SLASH,
  [anon_sym_PERCENT] = anon_sym_PERCENT,
  [anon_sym_LT_LT] = anon_sym_LT_LT,
  [anon_sym_GT_GT] = anon_sym_GT_GT,
  [anon_sym_AMP_CARET] = anon_sym_AMP_CARET,
  [anon_sym_EQ_EQ] = anon_sym_EQ_EQ,
  [anon_sym_BANG_EQ] = anon_sym_BANG_EQ,
  [anon_sym_LT] = anon_sym_LT,
  [anon_sym_LT_EQ] = anon_sym_LT_EQ,
  [anon_sym_GT] = anon_sym_GT,
  [anon_sym_GT_EQ] = anon_sym_GT_EQ,
  [anon_sym_AMP_AMP] = anon_sym_AMP_AMP,
  [anon_sym_PIPE_PIPE] = anon_sym_PIPE_PIPE,
  [anon_sym_BQUOTE] = anon_sym_BQUOTE,
  [aux_sym_raw_string_literal_token1] = aux_sym_raw_string_literal_token1,
  [anon_sym_DQUOTE] = anon_sym_DQUOTE,
  [aux_sym_interpreted_string_literal_token1] = aux_sym_interpreted_string_literal_token1,
  [anon_sym_DQUOTE2] = anon_sym_DQUOTE,
  [sym_escape_sequence] = sym_escape_sequence,
  [sym_int_literal] = sym_int_literal,
  [sym_float_literal] = sym_float_literal,
  [sym_imaginary_literal] = sym_imaginary_literal,
  [sym_rune_literal] = sym_rune_literal,
  [sym_nil] = sym_nil,
  [sym_true] = sym_true,
  [sym_false] = sym_false,
  [sym_iota] = sym_iota,
  [sym_comment] = sym_comment,
  [sym_source_file] = sym_source_file,
  [sym_package_clause] = sym_package_clause,
  [sym_import_declaration] = sym_import_declaration,
  [sym_import_spec] = sym_import_spec,
  [sym_dot] = sym_dot,
  [sym_import_spec_list] = sym_import_spec_list,
  [sym__declaration] = sym__declaration,
  [sym_const_declaration] = sym_const_declaration,
  [sym_const_spec] = sym_const_spec,
  [sym_var_declaration] = sym_var_declaration,
  [sym_var_spec] = sym_var_spec,
  [sym_var_spec_list] = sym_var_spec_list,
  [sym_function_declaration] = sym_function_declaration,
  [sym_method_declaration] = sym_method_declaration,
  [sym_type_parameter_list] = sym_type_parameter_list,
  [sym_type_parameter_declaration] = sym_type_parameter_declaration,
  [sym_parameter_list] = sym_parameter_list,
  [sym_parameter_declaration] = sym_parameter_declaration,
  [sym_variadic_parameter_declaration] = sym_variadic_parameter_declaration,
  [sym_type_alias] = sym_type_alias,
  [sym_type_declaration] = sym_type_declaration,
  [sym_type_spec] = sym_type_spec,
  [sym_expression_list] = sym_expression_list,
  [sym_parenthesized_type] = sym_parenthesized_type,
  [sym__simple_type] = sym__simple_type,
  [sym_generic_type] = sym_generic_type,
  [sym_type_arguments] = sym_type_arguments,
  [sym_pointer_type] = sym_pointer_type,
  [sym_array_type] = sym_array_type,
  [sym_implicit_length_array_type] = sym_implicit_length_array_type,
  [sym_slice_type] = sym_slice_type,
  [sym_struct_type] = sym_struct_type,
  [sym_negated_type] = sym_negated_type,
  [sym_field_declaration_list] = sym_field_declaration_list,
  [sym_field_declaration] = sym_field_declaration,
  [sym_interface_type] = sym_interface_type,
  [sym_method_elem] = sym_method_elem,
  [sym_type_elem] = sym_type_elem,
  [sym_map_type] = sym_map_type,
  [sym_channel_type] = sym_channel_type,
  [sym_function_type] = sym_function_type,
  [sym_block] = sym_block,
  [sym__statement_list] = sym__statement_list,
  [sym__statement] = sym__statement,
  [sym_empty_statement] = sym_empty_statement,
  [sym__simple_statement] = sym__simple_statement,
  [sym_expression_statement] = sym_expression_statement,
  [sym_send_statement] = sym_send_statement,
  [sym_receive_statement] = sym_receive_statement,
  [sym_inc_statement] = sym_inc_statement,
  [sym_dec_statement] = sym_dec_statement,
  [sym_assignment_statement] = sym_assignment_statement,
  [sym_short_var_declaration] = sym_short_var_declaration,
  [sym_labeled_statement] = sym_labeled_statement,
  [sym_empty_labeled_statement] = sym_labeled_statement,
  [sym_fallthrough_statement] = sym_fallthrough_statement,
  [sym_break_statement] = sym_break_statement,
  [sym_continue_statement] = sym_continue_statement,
  [sym_goto_statement] = sym_goto_statement,
  [sym_return_statement] = sym_return_statement,
  [sym_go_statement] = sym_go_statement,
  [sym_defer_statement] = sym_defer_statement,
  [sym_if_statement] = sym_if_statement,
  [sym_for_statement] = sym_for_statement,
  [sym_for_clause] = sym_for_clause,
  [sym_range_clause] = sym_range_clause,
  [sym_expression_switch_statement] = sym_expression_switch_statement,
  [sym_expression_case] = sym_expression_case,
  [sym_default_case] = sym_default_case,
  [sym_type_switch_statement] = sym_type_switch_statement,
  [sym__type_switch_header] = sym__type_switch_header,
  [sym_type_case] = sym_type_case,
  [sym_select_statement] = sym_select_statement,
  [sym_communication_case] = sym_communication_case,
  [sym__expression] = sym__expression,
  [sym_parenthesized_expression] = sym_parenthesized_expression,
  [sym_call_expression] = sym_call_expression,
  [sym_variadic_argument] = sym_variadic_argument,
  [sym_special_argument_list] = sym_argument_list,
  [sym_argument_list] = sym_argument_list,
  [sym_selector_expression] = sym_selector_expression,
  [sym_index_expression] = sym_index_expression,
  [sym_slice_expression] = sym_slice_expression,
  [sym_type_assertion_expression] = sym_type_assertion_expression,
  [sym_type_conversion_expression] = sym_type_conversion_expression,
  [sym_type_instantiation_expression] = sym_type_instantiation_expression,
  [sym_composite_literal] = sym_composite_literal,
  [sym_literal_value] = sym_literal_value,
  [sym_literal_element] = sym_literal_element,
  [sym_keyed_element] = sym_keyed_element,
  [sym_func_literal] = sym_func_literal,
  [sym_unary_expression] = sym_unary_expression,
  [sym_binary_expression] = sym_binary_expression,
  [sym_qualified_type] = sym_qualified_type,
  [sym_raw_string_literal] = sym_raw_string_literal,
  [sym_interpreted_string_literal] = sym_interpreted_string_literal,
  [aux_sym_source_file_repeat1] = aux_sym_source_file_repeat1,
  [aux_sym_import_spec_list_repeat1] = aux_sym_import_spec_list_repeat1,
  [aux_sym_const_declaration_repeat1] = aux_sym_const_declaration_repeat1,
  [aux_sym_const_spec_repeat1] = aux_sym_const_spec_repeat1,
  [aux_sym_var_spec_repeat1] = aux_sym_var_spec_repeat1,
  [aux_sym_var_spec_list_repeat1] = aux_sym_var_spec_list_repeat1,
  [aux_sym_type_parameter_list_repeat1] = aux_sym_type_parameter_list_repeat1,
  [aux_sym_parameter_list_repeat1] = aux_sym_parameter_list_repeat1,
  [aux_sym_type_declaration_repeat1] = aux_sym_type_declaration_repeat1,
  [aux_sym_expression_list_repeat1] = aux_sym_expression_list_repeat1,
  [aux_sym_type_arguments_repeat1] = aux_sym_type_arguments_repeat1,
  [aux_sym_field_declaration_list_repeat1] = aux_sym_field_declaration_list_repeat1,
  [aux_sym_field_declaration_repeat1] = aux_sym_field_declaration_repeat1,
  [aux_sym_interface_type_repeat1] = aux_sym_interface_type_repeat1,
  [aux_sym_type_elem_repeat1] = aux_sym_type_elem_repeat1,
  [aux_sym__statement_list_repeat1] = aux_sym__statement_list_repeat1,
  [aux_sym_expression_switch_statement_repeat1] = aux_sym_expression_switch_statement_repeat1,
  [aux_sym_type_switch_statement_repeat1] = aux_sym_type_switch_statement_repeat1,
  [aux_sym_type_case_repeat1] = aux_sym_type_case_repeat1,
  [aux_sym_select_statement_repeat1] = aux_sym_select_statement_repeat1,
  [aux_sym_argument_list_repeat1] = aux_sym_argument_list_repeat1,
  [aux_sym_literal_value_repeat1] = aux_sym_literal_value_repeat1,
  [aux_sym_interpreted_string_literal_repeat1] = aux_sym_interpreted_string_literal_repeat1,
  [alias_sym_field_identifier] = alias_sym_field_identifier,
  [alias_sym_label_name] = alias_sym_label_name,
  [alias_sym_package_identifier] = alias_sym_package_identifier,
  [alias_sym_type_constraint] = alias_sym_type_constraint,
  [alias_sym_type_identifier] = alias_sym_type_identifier,
};

static const TSSymbolMetadata ts_symbol_metadata[] = {
  [ts_builtin_sym_end] = {
    .visible = false,
    .named = true,
  },
  [sym_identifier] = {
    .visible = true,
    .named = true,
  },
  [aux_sym_source_file_token1] = {
    .visible = false,
    .named = false,
  },
  [anon_sym_SEMI] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_NULL] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_package] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_import] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DOT] = {
    .visible = true,
    .named = false,
  },
  [sym_blank_identifier] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_LPAREN] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_RPAREN] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_const] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_COMMA] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_var] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_func] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LBRACK] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_RBRACK] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DOT_DOT_DOT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_type] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STAR] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_struct] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_TILDE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LBRACE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_RBRACE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_interface] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PIPE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_map] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_chan] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT_DASH] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_COLON_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PLUS_PLUS] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DASH_DASH] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STAR_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SLASH_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PERCENT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT_LT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GT_GT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AMP_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AMP_CARET_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PLUS_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DASH_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PIPE_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_CARET_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_COLON] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_fallthrough] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_break] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_continue] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_goto] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_return] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_go] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_defer] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_if] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_else] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_for] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_range] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_switch] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_case] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_default] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_select] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_new] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_make] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_PLUS] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DASH] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_BANG] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_CARET] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AMP] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SLASH] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PERCENT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT_LT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GT_GT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AMP_CARET] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQ_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_BANG_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AMP_AMP] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PIPE_PIPE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_BQUOTE] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_raw_string_literal_token1] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_DQUOTE] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_interpreted_string_literal_token1] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_DQUOTE2] = {
    .visible = true,
    .named = false,
  },
  [sym_escape_sequence] = {
    .visible = true,
    .named = true,
  },
  [sym_int_literal] = {
    .visible = true,
    .named = true,
  },
  [sym_float_literal] = {
    .visible = true,
    .named = true,
  },
  [sym_imaginary_literal] = {
    .visible = true,
    .named = true,
  },
  [sym_rune_literal] = {
    .visible = true,
    .named = true,
  },
  [sym_nil] = {
    .visible = true,
    .named = true,
  },
  [sym_true] = {
    .visible = true,
    .named = true,
  },
  [sym_false] = {
    .visible = true,
    .named = true,
  },
  [sym_iota] = {
    .visible = true,
    .named = true,
  },
  [sym_comment] = {
    .visible = true,
    .named = true,
  },
  [sym_source_file] = {
    .visible = true,
    .named = true,
  },
  [sym_package_clause] = {
    .visible = true,
    .named = true,
  },
  [sym_import_declaration] = {
    .visible = true,
    .named = true,
  },
  [sym_import_spec] = {
    .visible = true,
    .named = true,
  },
  [sym_dot] = {
    .visible = true,
    .named = true,
  },
  [sym_import_spec_list] = {
    .visible = true,
    .named = true,
  },
  [sym__declaration] = {
    .visible = false,
    .named = true,
  },
  [sym_const_declaration] = {
    .visible = true,
    .named = true,
  },
  [sym_const_spec] = {
    .visible = true,
    .named = true,
  },
  [sym_var_declaration] = {
    .visible = true,
    .named = true,
  },
  [sym_var_spec] = {
    .visible = true,
    .named = true,
  },
  [sym_var_spec_list] = {
    .visible = true,
    .named = true,
  },
  [sym_function_declaration] = {
    .visible = true,
    .named = true,
  },
  [sym_method_declaration] = {
    .visible = true,
    .named = true,
  },
  [sym_type_parameter_list] = {
    .visible = true,
    .named = true,
  },
  [sym_type_parameter_declaration] = {
    .visible = true,
    .named = true,
  },
  [sym_parameter_list] = {
    .visible = true,
    .named = true,
  },
  [sym_parameter_declaration] = {
    .visible = true,
    .named = true,
  },
  [sym_variadic_parameter_declaration] = {
    .visible = true,
    .named = true,
  },
  [sym_type_alias] = {
    .visible = true,
    .named = true,
  },
  [sym_type_declaration] = {
    .visible = true,
    .named = true,
  },
  [sym_type_spec] = {
    .visible = true,
    .named = true,
  },
  [sym_expression_list] = {
    .visible = true,
    .named = true,
  },
  [sym_parenthesized_type] = {
    .visible = true,
    .named = true,
  },
  [sym__simple_type] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym_generic_type] = {
    .visible = true,
    .named = true,
  },
  [sym_type_arguments] = {
    .visible = true,
    .named = true,
  },
  [sym_pointer_type] = {
    .visible = true,
    .named = true,
  },
  [sym_array_type] = {
    .visible = true,
    .named = true,
  },
  [sym_implicit_length_array_type] = {
    .visible = true,
    .named = true,
  },
  [sym_slice_type] = {
    .visible = true,
    .named = true,
  },
  [sym_struct_type] = {
    .visible = true,
    .named = true,
  },
  [sym_negated_type] = {
    .visible = true,
    .named = true,
  },
  [sym_field_declaration_list] = {
    .visible = true,
    .named = true,
  },
  [sym_field_declaration] = {
    .visible = true,
    .named = true,
  },
  [sym_interface_type] = {
    .visible = true,
    .named = true,
  },
  [sym_method_elem] = {
    .visible = true,
    .named = true,
  },
  [sym_type_elem] = {
    .visible = true,
    .named = true,
  },
  [sym_map_type] = {
    .visible = true,
    .named = true,
  },
  [sym_channel_type] = {
    .visible = true,
    .named = true,
  },
  [sym_function_type] = {
    .visible = true,
    .named = true,
  },
  [sym_block] = {
    .visible = true,
    .named = true,
  },
  [sym__statement_list] = {
    .visible = false,
    .named = true,
  },
  [sym__statement] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym_empty_statement] = {
    .visible = true,
    .named = true,
  },
  [sym__simple_statement] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym_expression_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_send_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_receive_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_inc_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_dec_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_assignment_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_short_var_declaration] = {
    .visible = true,
    .named = true,
  },
  [sym_labeled_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_empty_labeled_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_fallthrough_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_break_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_continue_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_goto_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_return_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_go_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_defer_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_if_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_for_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_for_clause] = {
    .visible = true,
    .named = true,
  },
  [sym_range_clause] = {
    .visible = true,
    .named = true,
  },
  [sym_expression_switch_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_expression_case] = {
    .visible = true,
    .named = true,
  },
  [sym_default_case] = {
    .visible = true,
    .named = true,
  },
  [sym_type_switch_statement] = {
    .visible = true,
    .named = true,
  },
  [sym__type_switch_header] = {
    .visible = false,
    .named = true,
  },
  [sym_type_case] = {
    .visible = true,
    .named = true,
  },
  [sym_select_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_communication_case] = {
    .visible = true,
    .named = true,
  },
  [sym__expression] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym_parenthesized_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_call_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_variadic_argument] = {
    .visible = true,
    .named = true,
  },
  [sym_special_argument_list] = {
    .visible = true,
    .named = true,
  },
  [sym_argument_list] = {
    .visible = true,
    .named = true,
  },
  [sym_selector_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_index_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_slice_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_type_assertion_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_type_conversion_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_type_instantiation_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_composite_literal] = {
    .visible = true,
    .named = true,
  },
  [sym_literal_value] = {
    .visible = true,
    .named = true,
  },
  [sym_literal_element] = {
    .visible = true,
    .named = true,
  },
  [sym_keyed_element] = {
    .visible = true,
    .named = true,
  },
  [sym_func_literal] = {
    .visible = true,
    .named = true,
  },
  [sym_unary_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_binary_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_qualified_type] = {
    .visible = true,
    .named = true,
  },
  [sym_raw_string_literal] = {
    .visible = true,
    .named = true,
  },
  [sym_interpreted_string_literal] = {
    .visible = true,
    .named = true,
  },
  [aux_sym_source_file_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_import_spec_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_const_declaration_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_const_spec_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_var_spec_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_var_spec_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_type_parameter_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_parameter_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_type_declaration_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_expression_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_type_arguments_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_field_declaration_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_field_declaration_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_interface_type_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_type_elem_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__statement_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_expression_switch_statement_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_type_switch_statement_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_type_case_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_select_statement_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_argument_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_literal_value_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_interpreted_string_literal_repeat1] = {
    .visible = false,
    .named = false,
  },
  [alias_sym_field_identifier] = {
    .visible = true,
    .named = true,
  },
  [alias_sym_label_name] = {
    .visible = true,
    .named = true,
  },
  [alias_sym_package_identifier] = {
    .visible = true,
    .named = true,
  },
  [alias_sym_type_constraint] = {
    .visible = true,
    .named = true,
  },
  [alias_sym_type_identifier] = {
    .visible = true,
    .named = true,
  },
};

enum ts_field_identifiers {
  field_alias = 1,
  field_alternative = 2,
  field_arguments = 3,
  field_body = 4,
  field_capacity = 5,
  field_channel = 6,
  field_communication = 7,
  field_condition = 8,
  field_consequence = 9,
  field_element = 10,
  field_end = 11,
  field_field = 12,
  field_function = 13,
  field_index = 14,
  field_initializer = 15,
  field_key = 16,
  field_label = 17,
  field_left = 18,
  field_length = 19,
  field_name = 20,
  field_operand = 21,
  field_operator = 22,
  field_package = 23,
  field_parameters = 24,
  field_path = 25,
  field_receiver = 26,
  field_result = 27,
  field_right = 28,
  field_start = 29,
  field_tag = 30,
  field_type = 31,
  field_type_arguments = 32,
  field_type_parameters = 33,
  field_update = 34,
  field_value = 35,
};

static const char * const ts_field_names[] = {
  [0] = NULL,
  [field_alias] = "alias",
  [field_alternative] = "alternative",
  [field_arguments] = "arguments",
  [field_body] = "body",
  [field_capacity] = "capacity",
  [field_channel] = "channel",
  [field_communication] = "communication",
  [field_condition] = "condition",
  [field_consequence] = "consequence",
  [field_element] = "element",
  [field_end] = "end",
  [field_field] = "field",
  [field_function] = "function",
  [field_index] = "index",
  [field_initializer] = "initializer",
  [field_key] = "key",
  [field_label] = "label",
  [field_left] = "left",
  [field_length] = "length",
  [field_name] = "name",
  [field_operand] = "operand",
  [field_operator] = "operator",
  [field_package] = "package",
  [field_parameters] = "parameters",
  [field_path] = "path",
  [field_receiver] = "receiver",
  [field_result] = "result",
  [field_right] = "right",
  [field_start] = "start",
  [field_tag] = "tag",
  [field_type] = "type",
  [field_type_arguments] = "type_arguments",
  [field_type_parameters] = "type_parameters",
  [field_update] = "update",
  [field_value] = "value",
};

static const TSFieldMapSlice ts_field_map_slices[PRODUCTION_ID_COUNT] = {
  [3] = {.index = 0, .length = 1},
  [4] = {.index = 1, .length = 1},
  [5] = {.index = 2, .length = 1},
  [6] = {.index = 3, .length = 2},
  [7] = {.index = 5, .length = 1},
  [9] = {.index = 6, .length = 1},
  [10] = {.index = 7, .length = 2},
  [11] = {.index = 9, .length = 2},
  [12] = {.index = 11, .length = 2},
  [13] = {.index = 11, .length = 2},
  [14] = {.index = 9, .length = 2},
  [15] = {.index = 13, .length = 2},
  [16] = {.index = 13, .length = 2},
  [17] = {.index = 15, .length = 2},
  [18] = {.index = 17, .length = 2},
  [19] = {.index = 19, .length = 1},
  [20] = {.index = 20, .length = 2},
  [21] = {.index = 22, .length = 2},
  [22] = {.index = 24, .length = 2},
  [23] = {.index = 26, .length = 1},
  [24] = {.index = 17, .length = 2},
  [25] = {.index = 19, .length = 1},
  [26] = {.index = 27, .length = 1},
  [27] = {.index = 28, .length = 1},
  [28] = {.index = 29, .length = 2},
  [29] = {.index = 31, .length = 1},
  [30] = {.index = 32, .length = 1},
  [31] = {.index = 33, .length = 2},
  [32] = {.index = 35, .length = 3},
  [33] = {.index = 38, .length = 2},
  [34] = {.index = 40, .length = 2},
  [35] = {.index = 42, .length = 2},
  [36] = {.index = 44, .length = 3},
  [37] = {.index = 47, .length = 2},
  [38] = {.index = 49, .length = 1},
  [39] = {.index = 50, .length = 3},
  [40] = {.index = 53, .length = 2},
  [41] = {.index = 55, .length = 1},
  [42] = {.index = 56, .length = 3},
  [43] = {.index = 59, .length = 3},
  [44] = {.index = 62, .length = 3},
  [45] = {.index = 65, .length = 3},
  [46] = {.index = 68, .length = 3},
  [47] = {.index = 71, .length = 1},
  [48] = {.index = 72, .length = 2},
  [49] = {.index = 74, .length = 2},
  [50] = {.index = 76, .length = 3},
  [51] = {.index = 55, .length = 1},
  [52] = {.index = 17, .length = 2},
  [53] = {.index = 79, .length = 2},
  [54] = {.index = 79, .length = 2},
  [55] = {.index = 81, .length = 2},
  [56] = {.index = 83, .length = 1},
  [57] = {.index = 84, .length = 1},
  [58] = {.index = 85, .length = 1},
  [59] = {.index = 86, .length = 3},
  [60] = {.index = 89, .length = 1},
  [61] = {.index = 90, .length = 2},
  [62] = {.index = 92, .length = 1},
  [63] = {.index = 93, .length = 2},
  [64] = {.index = 95, .length = 3},
  [65] = {.index = 98, .length = 3},
  [66] = {.index = 101, .length = 3},
  [67] = {.index = 74, .length = 2},
  [68] = {.index = 17, .length = 2},
  [69] = {.index = 104, .length = 4},
  [70] = {.index = 108, .length = 4},
  [71] = {.index = 112, .length = 4},
  [72] = {.index = 116, .length = 4},
  [73] = {.index = 120, .length = 4},
  [74] = {.index = 124, .length = 2},
  [75] = {.index = 124, .length = 2},
  [76] = {.index = 49, .length = 1},
  [77] = {.index = 126, .length = 3},
  [78] = {.index = 50, .length = 3},
  [79] = {.index = 129, .length = 3},
  [80] = {.index = 132, .length = 2},
  [81] = {.index = 134, .length = 3},
  [82] = {.index = 137, .length = 3},
  [83] = {.index = 140, .length = 2},
  [84] = {.index = 142, .length = 2},
  [85] = {.index = 144, .length = 2},
  [86] = {.index = 146, .length = 2},
  [87] = {.index = 148, .length = 1},
  [88] = {.index = 149, .length = 1},
  [89] = {.index = 150, .length = 2},
  [90] = {.index = 152, .length = 2},
  [91] = {.index = 154, .length = 2},
  [92] = {.index = 156, .length = 2},
  [93] = {.index = 158, .length = 4},
  [94] = {.index = 162, .length = 4},
  [95] = {.index = 50, .length = 3},
  [96] = {.index = 166, .length = 5},
  [97] = {.index = 171, .length = 5},
  [98] = {.index = 176, .length = 4},
  [99] = {.index = 180, .length = 3},
  [100] = {.index = 183, .length = 2},
  [101] = {.index = 185, .length = 1},
  [102] = {.index = 186, .length = 3},
  [103] = {.index = 189, .length = 4},
  [104] = {.index = 193, .length = 2},
  [105] = {.index = 195, .length = 3},
  [106] = {.index = 198, .length = 2},
  [107] = {.index = 200, .length = 2},
  [108] = {.index = 202, .length = 4},
  [109] = {.index = 206, .length = 3},
};

static const TSFieldMapEntry ts_field_map_entries[] = {
  [0] =
    {field_path, 0},
  [1] =
    {field_name, 0},
  [2] =
    {field_parameters, 1},
  [3] =
    {field_operand, 1},
    {field_operator, 0},
  [5] =
    {field_value, 1},
  [6] =
    {field_body, 1},
  [7] =
    {field_arguments, 1},
    {field_function, 0},
  [9] =
    {field_type, 0},
    {field_type_arguments, 1},
  [11] =
    {field_body, 1},
    {field_type, 0},
  [13] =
    {field_name, 0},
    {field_path, 1},
  [15] =
    {field_name, 0},
    {field_name, 1},
  [17] =
    {field_name, 0},
    {field_type, 1},
  [19] =
    {field_type, 0},
  [20] =
    {field_name, 1},
    {field_parameters, 2},
  [22] =
    {field_parameters, 1},
    {field_result, 2},
  [24] =
    {field_body, 2},
    {field_parameters, 1},
  [26] =
    {field_element, 2},
  [27] =
    {field_label, 0},
  [28] =
    {field_value, 2},
  [29] =
    {field_condition, 1},
    {field_consequence, 2},
  [31] =
    {field_right, 1},
  [32] =
    {field_body, 2},
  [33] =
    {field_name, 2},
    {field_package, 0},
  [35] =
    {field_left, 0},
    {field_operator, 1},
    {field_right, 2},
  [38] =
    {field_left, 0},
    {field_right, 2},
  [40] =
    {field_field, 2},
    {field_operand, 0},
  [42] =
    {field_channel, 0},
    {field_value, 2},
  [44] =
    {field_arguments, 2},
    {field_function, 0},
    {field_type_arguments, 1},
  [47] =
    {field_name, 0},
    {field_value, 2},
  [49] =
    {field_name, 1},
  [50] =
    {field_name, 0},
    {field_name, 1, .inherited = true},
    {field_type, 2},
  [53] =
    {field_name, 0, .inherited = true},
    {field_name, 1, .inherited = true},
  [55] =
    {field_type, 1},
  [56] =
    {field_name, 1},
    {field_parameters, 3},
    {field_type_parameters, 2},
  [59] =
    {field_name, 1},
    {field_parameters, 2},
    {field_result, 3},
  [62] =
    {field_body, 3},
    {field_name, 1},
    {field_parameters, 2},
  [65] =
    {field_name, 2},
    {field_parameters, 3},
    {field_receiver, 1},
  [68] =
    {field_body, 3},
    {field_parameters, 1},
    {field_result, 2},
  [71] =
    {field_element, 3},
  [72] =
    {field_element, 3},
    {field_length, 1},
  [74] =
    {field_name, 0},
    {field_type, 2},
  [76] =
    {field_name, 0},
    {field_type, 2},
    {field_type_parameters, 1},
  [79] =
    {field_tag, 1},
    {field_type, 0},
  [81] =
    {field_name, 0},
    {field_parameters, 1},
  [83] =
    {field_update, 2},
  [84] =
    {field_condition, 1},
  [85] =
    {field_initializer, 0},
  [86] =
    {field_alias, 1, .inherited = true},
    {field_initializer, 1, .inherited = true},
    {field_value, 1, .inherited = true},
  [89] =
    {field_right, 0},
  [90] =
    {field_operand, 2},
    {field_type, 0},
  [92] =
    {field_operand, 0},
  [93] =
    {field_index, 2},
    {field_operand, 0},
  [95] =
    {field_name, 0},
    {field_type, 1},
    {field_value, 3},
  [98] =
    {field_name, 0},
    {field_name, 1},
    {field_value, 3},
  [101] =
    {field_name, 0},
    {field_name, 1, .inherited = true},
    {field_value, 3},
  [104] =
    {field_name, 1},
    {field_parameters, 3},
    {field_result, 4},
    {field_type_parameters, 2},
  [108] =
    {field_body, 4},
    {field_name, 1},
    {field_parameters, 3},
    {field_type_parameters, 2},
  [112] =
    {field_body, 4},
    {field_name, 1},
    {field_parameters, 2},
    {field_result, 3},
  [116] =
    {field_name, 2},
    {field_parameters, 3},
    {field_receiver, 1},
    {field_result, 4},
  [120] =
    {field_body, 4},
    {field_name, 2},
    {field_parameters, 3},
    {field_receiver, 1},
  [124] =
    {field_tag, 2},
    {field_type, 1},
  [126] =
    {field_name, 0},
    {field_tag, 2},
    {field_type, 1},
  [129] =
    {field_name, 0},
    {field_parameters, 1},
    {field_result, 2},
  [132] =
    {field_key, 2},
    {field_value, 4},
  [134] =
    {field_condition, 3},
    {field_consequence, 4},
    {field_initializer, 1},
  [137] =
    {field_alternative, 4},
    {field_condition, 1},
    {field_consequence, 2},
  [140] =
    {field_condition, 1},
    {field_update, 3},
  [142] =
    {field_left, 0},
    {field_right, 3},
  [144] =
    {field_initializer, 0},
    {field_update, 3},
  [146] =
    {field_condition, 2},
    {field_initializer, 0},
  [148] =
    {field_initializer, 1},
  [149] =
    {field_communication, 1},
  [150] =
    {field_key, 0},
    {field_value, 2},
  [152] =
    {field_operand, 0},
    {field_type, 3},
  [154] =
    {field_end, 3},
    {field_operand, 0},
  [156] =
    {field_operand, 0},
    {field_start, 2},
  [158] =
    {field_name, 0},
    {field_name, 1},
    {field_type, 2},
    {field_value, 4},
  [162] =
    {field_name, 0},
    {field_name, 1, .inherited = true},
    {field_type, 2},
    {field_value, 4},
  [166] =
    {field_body, 5},
    {field_name, 1},
    {field_parameters, 3},
    {field_result, 4},
    {field_type_parameters, 2},
  [171] =
    {field_body, 5},
    {field_name, 2},
    {field_parameters, 3},
    {field_receiver, 1},
    {field_result, 4},
  [176] =
    {field_name, 0},
    {field_name, 1, .inherited = true},
    {field_tag, 3},
    {field_type, 2},
  [180] =
    {field_condition, 2},
    {field_initializer, 0},
    {field_update, 4},
  [183] =
    {field_initializer, 1},
    {field_value, 3},
  [185] =
    {field_value, 0},
  [186] =
    {field_end, 4},
    {field_operand, 0},
    {field_start, 2},
  [189] =
    {field_alternative, 6},
    {field_condition, 3},
    {field_consequence, 4},
    {field_initializer, 1},
  [193] =
    {field_type, 1},
    {field_type, 2},
  [195] =
    {field_capacity, 5},
    {field_end, 3},
    {field_operand, 0},
  [198] =
    {field_alias, 0},
    {field_value, 2},
  [200] =
    {field_initializer, 0},
    {field_value, 2},
  [202] =
    {field_capacity, 6},
    {field_end, 4},
    {field_operand, 0},
    {field_start, 2},
  [206] =
    {field_alias, 2},
    {field_initializer, 0},
    {field_value, 4},
};

static const TSSymbol ts_alias_sequences[PRODUCTION_ID_COUNT][MAX_ALIAS_SEQUENCE_LENGTH] = {
  [0] = {0},
  [1] = {
    [0] = alias_sym_type_identifier,
  },
  [2] = {
    [1] = alias_sym_package_identifier,
  },
  [8] = {
    [1] = alias_sym_label_name,
  },
  [11] = {
    [0] = alias_sym_type_identifier,
  },
  [12] = {
    [0] = alias_sym_type_identifier,
  },
  [16] = {
    [0] = alias_sym_package_identifier,
  },
  [24] = {
    [0] = alias_sym_type_identifier,
  },
  [25] = {
    [0] = alias_sym_type_identifier,
  },
  [26] = {
    [0] = alias_sym_label_name,
  },
  [31] = {
    [0] = alias_sym_package_identifier,
    [2] = alias_sym_type_identifier,
  },
  [34] = {
    [2] = alias_sym_field_identifier,
  },
  [45] = {
    [2] = alias_sym_field_identifier,
  },
  [49] = {
    [0] = alias_sym_type_identifier,
  },
  [50] = {
    [0] = alias_sym_type_identifier,
  },
  [51] = {
    [1] = alias_sym_type_identifier,
  },
  [52] = {
    [0] = alias_sym_field_identifier,
  },
  [53] = {
    [0] = alias_sym_type_identifier,
  },
  [55] = {
    [0] = alias_sym_field_identifier,
  },
  [68] = {
    [1] = alias_sym_type_constraint,
  },
  [72] = {
    [2] = alias_sym_field_identifier,
  },
  [73] = {
    [2] = alias_sym_field_identifier,
  },
  [74] = {
    [1] = alias_sym_type_identifier,
  },
  [76] = {
    [1] = alias_sym_field_identifier,
  },
  [77] = {
    [0] = alias_sym_field_identifier,
  },
  [78] = {
    [0] = alias_sym_field_identifier,
  },
  [79] = {
    [0] = alias_sym_field_identifier,
  },
  [95] = {
    [2] = alias_sym_type_constraint,
  },
  [97] = {
    [2] = alias_sym_field_identifier,
  },
  [98] = {
    [0] = alias_sym_field_identifier,
  },
};

static const uint16_t ts_non_terminal_alias_map[] = {
  sym_type_elem, 2,
    sym_type_elem,
    alias_sym_type_constraint,
  0,
};

static const TSStateId ts_primary_state_ids[STATE_COUNT] = {
  [0] = 0,
  [1] = 1,
  [2] = 2,
  [3] = 3,
  [4] = 4,
  [5] = 5,
  [6] = 6,
  [7] = 7,
  [8] = 8,
  [9] = 9,
  [10] = 10,
  [11] = 11,
  [12] = 12,
  [13] = 12,
  [14] = 12,
  [15] = 12,
  [16] = 12,
  [17] = 12,
  [18] = 12,
  [19] = 19,
  [20] = 20,
  [21] = 21,
  [22] = 22,
  [23] = 23,
  [24] = 24,
  [25] = 25,
  [26] = 26,
  [27] = 27,
  [28] = 28,
  [29] = 29,
  [30] = 29,
  [31] = 29,
  [32] = 29,
  [33] = 29,
  [34] = 29,
  [35] = 35,
  [36] = 36,
  [37] = 35,
  [38] = 35,
  [39] = 36,
  [40] = 35,
  [41] = 36,
  [42] = 35,
  [43] = 36,
  [44] = 36,
  [45] = 36,
  [46] = 35,
  [47] = 47,
  [48] = 48,
  [49] = 49,
  [50] = 50,
  [51] = 51,
  [52] = 52,
  [53] = 53,
  [54] = 54,
  [55] = 55,
  [56] = 56,
  [57] = 51,
  [58] = 58,
  [59] = 51,
  [60] = 52,
  [61] = 61,
  [62] = 55,
  [63] = 58,
  [64] = 58,
  [65] = 52,
  [66] = 55,
  [67] = 58,
  [68] = 52,
  [69] = 55,
  [70] = 58,
  [71] = 52,
  [72] = 55,
  [73] = 73,
  [74] = 58,
  [75] = 52,
  [76] = 56,
  [77] = 51,
  [78] = 51,
  [79] = 51,
  [80] = 55,
  [81] = 81,
  [82] = 81,
  [83] = 83,
  [84] = 83,
  [85] = 85,
  [86] = 86,
  [87] = 87,
  [88] = 88,
  [89] = 85,
  [90] = 83,
  [91] = 91,
  [92] = 92,
  [93] = 91,
  [94] = 81,
  [95] = 95,
  [96] = 92,
  [97] = 85,
  [98] = 86,
  [99] = 99,
  [100] = 91,
  [101] = 101,
  [102] = 83,
  [103] = 103,
  [104] = 88,
  [105] = 91,
  [106] = 81,
  [107] = 107,
  [108] = 81,
  [109] = 85,
  [110] = 86,
  [111] = 111,
  [112] = 112,
  [113] = 113,
  [114] = 83,
  [115] = 115,
  [116] = 116,
  [117] = 91,
  [118] = 81,
  [119] = 85,
  [120] = 86,
  [121] = 85,
  [122] = 86,
  [123] = 123,
  [124] = 88,
  [125] = 92,
  [126] = 126,
  [127] = 83,
  [128] = 86,
  [129] = 129,
  [130] = 91,
  [131] = 131,
  [132] = 132,
  [133] = 132,
  [134] = 134,
  [135] = 135,
  [136] = 136,
  [137] = 137,
  [138] = 138,
  [139] = 139,
  [140] = 140,
  [141] = 131,
  [142] = 142,
  [143] = 134,
  [144] = 137,
  [145] = 138,
  [146] = 140,
  [147] = 131,
  [148] = 142,
  [149] = 134,
  [150] = 137,
  [151] = 138,
  [152] = 140,
  [153] = 142,
  [154] = 134,
  [155] = 137,
  [156] = 138,
  [157] = 140,
  [158] = 131,
  [159] = 142,
  [160] = 134,
  [161] = 137,
  [162] = 138,
  [163] = 140,
  [164] = 131,
  [165] = 142,
  [166] = 134,
  [167] = 137,
  [168] = 138,
  [169] = 140,
  [170] = 131,
  [171] = 142,
  [172] = 172,
  [173] = 172,
  [174] = 174,
  [175] = 175,
  [176] = 176,
  [177] = 136,
  [178] = 139,
  [179] = 134,
  [180] = 137,
  [181] = 138,
  [182] = 140,
  [183] = 131,
  [184] = 142,
  [185] = 172,
  [186] = 174,
  [187] = 136,
  [188] = 134,
  [189] = 172,
  [190] = 174,
  [191] = 191,
  [192] = 192,
  [193] = 136,
  [194] = 136,
  [195] = 195,
  [196] = 137,
  [197] = 172,
  [198] = 174,
  [199] = 138,
  [200] = 139,
  [201] = 140,
  [202] = 131,
  [203] = 142,
  [204] = 172,
  [205] = 205,
  [206] = 174,
  [207] = 207,
  [208] = 174,
  [209] = 172,
  [210] = 174,
  [211] = 172,
  [212] = 174,
  [213] = 172,
  [214] = 172,
  [215] = 215,
  [216] = 191,
  [217] = 217,
  [218] = 217,
  [219] = 132,
  [220] = 215,
  [221] = 191,
  [222] = 217,
  [223] = 132,
  [224] = 215,
  [225] = 191,
  [226] = 217,
  [227] = 132,
  [228] = 215,
  [229] = 191,
  [230] = 217,
  [231] = 132,
  [232] = 215,
  [233] = 191,
  [234] = 217,
  [235] = 215,
  [236] = 236,
  [237] = 237,
  [238] = 238,
  [239] = 239,
  [240] = 240,
  [241] = 241,
  [242] = 242,
  [243] = 243,
  [244] = 244,
  [245] = 245,
  [246] = 246,
  [247] = 247,
  [248] = 248,
  [249] = 249,
  [250] = 250,
  [251] = 251,
  [252] = 252,
  [253] = 253,
  [254] = 254,
  [255] = 255,
  [256] = 256,
  [257] = 257,
  [258] = 258,
  [259] = 259,
  [260] = 260,
  [261] = 261,
  [262] = 262,
  [263] = 263,
  [264] = 264,
  [265] = 265,
  [266] = 266,
  [267] = 267,
  [268] = 268,
  [269] = 269,
  [270] = 270,
  [271] = 271,
  [272] = 272,
  [273] = 273,
  [274] = 274,
  [275] = 275,
  [276] = 276,
  [277] = 277,
  [278] = 278,
  [279] = 279,
  [280] = 280,
  [281] = 281,
  [282] = 282,
  [283] = 283,
  [284] = 284,
  [285] = 285,
  [286] = 286,
  [287] = 287,
  [288] = 288,
  [289] = 289,
  [290] = 290,
  [291] = 243,
  [292] = 240,
  [293] = 245,
  [294] = 239,
  [295] = 244,
  [296] = 242,
  [297] = 247,
  [298] = 241,
  [299] = 252,
  [300] = 262,
  [301] = 278,
  [302] = 279,
  [303] = 280,
  [304] = 281,
  [305] = 282,
  [306] = 285,
  [307] = 288,
  [308] = 287,
  [309] = 289,
  [310] = 255,
  [311] = 286,
  [312] = 270,
  [313] = 275,
  [314] = 261,
  [315] = 253,
  [316] = 257,
  [317] = 267,
  [318] = 268,
  [319] = 263,
  [320] = 254,
  [321] = 250,
  [322] = 248,
  [323] = 260,
  [324] = 258,
  [325] = 264,
  [326] = 249,
  [327] = 269,
  [328] = 271,
  [329] = 272,
  [330] = 273,
  [331] = 277,
  [332] = 251,
  [333] = 274,
  [334] = 265,
  [335] = 283,
  [336] = 284,
  [337] = 276,
  [338] = 266,
  [339] = 239,
  [340] = 340,
  [341] = 245,
  [342] = 241,
  [343] = 244,
  [344] = 242,
  [345] = 243,
  [346] = 247,
  [347] = 252,
  [348] = 271,
  [349] = 251,
  [350] = 262,
  [351] = 264,
  [352] = 263,
  [353] = 257,
  [354] = 261,
  [355] = 266,
  [356] = 267,
  [357] = 268,
  [358] = 254,
  [359] = 269,
  [360] = 273,
  [361] = 272,
  [362] = 274,
  [363] = 276,
  [364] = 277,
  [365] = 279,
  [366] = 280,
  [367] = 281,
  [368] = 282,
  [369] = 285,
  [370] = 288,
  [371] = 287,
  [372] = 289,
  [373] = 255,
  [374] = 286,
  [375] = 270,
  [376] = 275,
  [377] = 253,
  [378] = 250,
  [379] = 249,
  [380] = 265,
  [381] = 260,
  [382] = 258,
  [383] = 283,
  [384] = 284,
  [385] = 385,
  [386] = 248,
  [387] = 278,
  [388] = 388,
  [389] = 388,
  [390] = 245,
  [391] = 241,
  [392] = 252,
  [393] = 239,
  [394] = 394,
  [395] = 395,
  [396] = 274,
  [397] = 248,
  [398] = 260,
  [399] = 287,
  [400] = 282,
  [401] = 278,
  [402] = 250,
  [403] = 249,
  [404] = 279,
  [405] = 289,
  [406] = 271,
  [407] = 255,
  [408] = 285,
  [409] = 273,
  [410] = 286,
  [411] = 277,
  [412] = 251,
  [413] = 253,
  [414] = 258,
  [415] = 265,
  [416] = 270,
  [417] = 275,
  [418] = 272,
  [419] = 263,
  [420] = 283,
  [421] = 288,
  [422] = 284,
  [423] = 264,
  [424] = 424,
  [425] = 280,
  [426] = 239,
  [427] = 257,
  [428] = 281,
  [429] = 261,
  [430] = 266,
  [431] = 267,
  [432] = 268,
  [433] = 276,
  [434] = 254,
  [435] = 269,
  [436] = 262,
  [437] = 244,
  [438] = 247,
  [439] = 247,
  [440] = 243,
  [441] = 241,
  [442] = 388,
  [443] = 242,
  [444] = 243,
  [445] = 340,
  [446] = 388,
  [447] = 245,
  [448] = 244,
  [449] = 242,
  [450] = 450,
  [451] = 451,
  [452] = 452,
  [453] = 453,
  [454] = 454,
  [455] = 252,
  [456] = 456,
  [457] = 457,
  [458] = 458,
  [459] = 239,
  [460] = 460,
  [461] = 457,
  [462] = 264,
  [463] = 273,
  [464] = 277,
  [465] = 262,
  [466] = 265,
  [467] = 283,
  [468] = 284,
  [469] = 257,
  [470] = 261,
  [471] = 266,
  [472] = 267,
  [473] = 268,
  [474] = 254,
  [475] = 269,
  [476] = 271,
  [477] = 272,
  [478] = 274,
  [479] = 276,
  [480] = 278,
  [481] = 279,
  [482] = 280,
  [483] = 281,
  [484] = 282,
  [485] = 285,
  [486] = 288,
  [487] = 287,
  [488] = 289,
  [489] = 255,
  [490] = 286,
  [491] = 270,
  [492] = 275,
  [493] = 253,
  [494] = 494,
  [495] = 250,
  [496] = 249,
  [497] = 454,
  [498] = 388,
  [499] = 458,
  [500] = 245,
  [501] = 388,
  [502] = 241,
  [503] = 248,
  [504] = 251,
  [505] = 244,
  [506] = 242,
  [507] = 243,
  [508] = 247,
  [509] = 263,
  [510] = 494,
  [511] = 494,
  [512] = 260,
  [513] = 258,
  [514] = 340,
  [515] = 515,
  [516] = 516,
  [517] = 239,
  [518] = 515,
  [519] = 519,
  [520] = 515,
  [521] = 516,
  [522] = 522,
  [523] = 523,
  [524] = 239,
  [525] = 252,
  [526] = 515,
  [527] = 516,
  [528] = 522,
  [529] = 424,
  [530] = 515,
  [531] = 515,
  [532] = 522,
  [533] = 243,
  [534] = 273,
  [535] = 277,
  [536] = 424,
  [537] = 251,
  [538] = 262,
  [539] = 265,
  [540] = 283,
  [541] = 284,
  [542] = 257,
  [543] = 261,
  [544] = 266,
  [545] = 267,
  [546] = 268,
  [547] = 254,
  [548] = 269,
  [549] = 271,
  [550] = 272,
  [551] = 274,
  [552] = 276,
  [553] = 278,
  [554] = 279,
  [555] = 280,
  [556] = 281,
  [557] = 282,
  [558] = 285,
  [559] = 288,
  [560] = 287,
  [561] = 289,
  [562] = 255,
  [563] = 286,
  [564] = 270,
  [565] = 275,
  [566] = 242,
  [567] = 253,
  [568] = 568,
  [569] = 264,
  [570] = 250,
  [571] = 249,
  [572] = 572,
  [573] = 573,
  [574] = 574,
  [575] = 575,
  [576] = 576,
  [577] = 574,
  [578] = 578,
  [579] = 568,
  [580] = 580,
  [581] = 581,
  [582] = 244,
  [583] = 242,
  [584] = 243,
  [585] = 247,
  [586] = 244,
  [587] = 576,
  [588] = 576,
  [589] = 574,
  [590] = 263,
  [591] = 568,
  [592] = 239,
  [593] = 593,
  [594] = 248,
  [595] = 260,
  [596] = 258,
  [597] = 597,
  [598] = 247,
  [599] = 452,
  [600] = 600,
  [601] = 452,
  [602] = 602,
  [603] = 340,
  [604] = 604,
  [605] = 605,
  [606] = 340,
  [607] = 452,
  [608] = 602,
  [609] = 609,
  [610] = 605,
  [611] = 456,
  [612] = 612,
  [613] = 613,
  [614] = 613,
  [615] = 615,
  [616] = 613,
  [617] = 615,
  [618] = 618,
  [619] = 602,
  [620] = 604,
  [621] = 452,
  [622] = 602,
  [623] = 623,
  [624] = 605,
  [625] = 615,
  [626] = 615,
  [627] = 604,
  [628] = 618,
  [629] = 602,
  [630] = 618,
  [631] = 605,
  [632] = 615,
  [633] = 604,
  [634] = 634,
  [635] = 452,
  [636] = 602,
  [637] = 604,
  [638] = 638,
  [639] = 605,
  [640] = 615,
  [641] = 604,
  [642] = 612,
  [643] = 605,
  [644] = 644,
  [645] = 645,
  [646] = 646,
  [647] = 647,
  [648] = 456,
  [649] = 649,
  [650] = 650,
  [651] = 651,
  [652] = 652,
  [653] = 647,
  [654] = 654,
  [655] = 644,
  [656] = 656,
  [657] = 657,
  [658] = 658,
  [659] = 659,
  [660] = 660,
  [661] = 661,
  [662] = 650,
  [663] = 663,
  [664] = 645,
  [665] = 665,
  [666] = 654,
  [667] = 647,
  [668] = 668,
  [669] = 654,
  [670] = 644,
  [671] = 660,
  [672] = 650,
  [673] = 663,
  [674] = 645,
  [675] = 647,
  [676] = 654,
  [677] = 644,
  [678] = 660,
  [679] = 660,
  [680] = 650,
  [681] = 663,
  [682] = 645,
  [683] = 647,
  [684] = 654,
  [685] = 663,
  [686] = 660,
  [687] = 650,
  [688] = 663,
  [689] = 645,
  [690] = 647,
  [691] = 654,
  [692] = 644,
  [693] = 660,
  [694] = 650,
  [695] = 663,
  [696] = 645,
  [697] = 649,
  [698] = 649,
  [699] = 644,
  [700] = 700,
  [701] = 701,
  [702] = 702,
  [703] = 703,
  [704] = 704,
  [705] = 705,
  [706] = 700,
  [707] = 707,
  [708] = 708,
  [709] = 708,
  [710] = 710,
  [711] = 711,
  [712] = 700,
  [713] = 711,
  [714] = 714,
  [715] = 714,
  [716] = 702,
  [717] = 714,
  [718] = 718,
  [719] = 700,
  [720] = 720,
  [721] = 703,
  [722] = 707,
  [723] = 702,
  [724] = 708,
  [725] = 704,
  [726] = 705,
  [727] = 707,
  [728] = 728,
  [729] = 729,
  [730] = 730,
  [731] = 704,
  [732] = 707,
  [733] = 708,
  [734] = 705,
  [735] = 711,
  [736] = 703,
  [737] = 703,
  [738] = 702,
  [739] = 705,
  [740] = 711,
  [741] = 714,
  [742] = 700,
  [743] = 743,
  [744] = 703,
  [745] = 702,
  [746] = 711,
  [747] = 705,
  [748] = 728,
  [749] = 749,
  [750] = 707,
  [751] = 711,
  [752] = 720,
  [753] = 714,
  [754] = 703,
  [755] = 702,
  [756] = 705,
  [757] = 757,
  [758] = 758,
  [759] = 759,
  [760] = 707,
  [761] = 708,
  [762] = 708,
  [763] = 714,
  [764] = 764,
  [765] = 757,
  [766] = 730,
  [767] = 728,
  [768] = 757,
  [769] = 710,
  [770] = 730,
  [771] = 728,
  [772] = 757,
  [773] = 730,
  [774] = 728,
  [775] = 757,
  [776] = 730,
  [777] = 728,
  [778] = 720,
  [779] = 757,
  [780] = 730,
  [781] = 700,
  [782] = 782,
  [783] = 783,
  [784] = 784,
  [785] = 785,
  [786] = 786,
  [787] = 787,
  [788] = 783,
  [789] = 782,
  [790] = 784,
  [791] = 785,
  [792] = 785,
  [793] = 782,
  [794] = 784,
  [795] = 783,
  [796] = 796,
  [797] = 797,
  [798] = 798,
  [799] = 798,
  [800] = 800,
  [801] = 801,
  [802] = 802,
  [803] = 803,
  [804] = 804,
  [805] = 805,
  [806] = 803,
  [807] = 804,
  [808] = 804,
  [809] = 809,
  [810] = 810,
  [811] = 811,
  [812] = 812,
  [813] = 813,
  [814] = 814,
  [815] = 815,
  [816] = 816,
  [817] = 817,
  [818] = 818,
  [819] = 819,
  [820] = 820,
  [821] = 821,
  [822] = 822,
  [823] = 823,
  [824] = 824,
  [825] = 825,
  [826] = 826,
  [827] = 827,
  [828] = 828,
  [829] = 829,
  [830] = 830,
  [831] = 831,
  [832] = 832,
  [833] = 833,
  [834] = 834,
  [835] = 835,
  [836] = 836,
  [837] = 837,
  [838] = 838,
  [839] = 839,
  [840] = 840,
  [841] = 841,
  [842] = 839,
  [843] = 843,
  [844] = 798,
  [845] = 845,
  [846] = 846,
  [847] = 846,
  [848] = 848,
  [849] = 849,
  [850] = 803,
  [851] = 848,
  [852] = 852,
  [853] = 853,
  [854] = 852,
  [855] = 798,
  [856] = 835,
  [857] = 833,
  [858] = 828,
  [859] = 813,
  [860] = 817,
  [861] = 820,
  [862] = 811,
  [863] = 829,
  [864] = 834,
  [865] = 810,
  [866] = 819,
  [867] = 816,
  [868] = 836,
  [869] = 837,
  [870] = 818,
  [871] = 871,
  [872] = 814,
  [873] = 823,
  [874] = 832,
  [875] = 827,
  [876] = 826,
  [877] = 803,
  [878] = 822,
  [879] = 821,
  [880] = 830,
  [881] = 812,
  [882] = 882,
  [883] = 815,
  [884] = 831,
  [885] = 825,
  [886] = 797,
  [887] = 887,
  [888] = 796,
  [889] = 889,
  [890] = 890,
  [891] = 891,
  [892] = 887,
  [893] = 798,
  [894] = 890,
  [895] = 895,
  [896] = 891,
  [897] = 798,
  [898] = 898,
  [899] = 899,
  [900] = 900,
  [901] = 901,
  [902] = 902,
  [903] = 890,
  [904] = 891,
  [905] = 887,
  [906] = 906,
  [907] = 907,
  [908] = 908,
  [909] = 909,
  [910] = 803,
  [911] = 911,
  [912] = 912,
  [913] = 913,
  [914] = 914,
  [915] = 915,
  [916] = 916,
  [917] = 917,
  [918] = 803,
  [919] = 829,
  [920] = 920,
  [921] = 921,
  [922] = 816,
  [923] = 923,
  [924] = 924,
  [925] = 831,
  [926] = 828,
  [927] = 927,
  [928] = 832,
  [929] = 929,
  [930] = 827,
  [931] = 931,
  [932] = 819,
  [933] = 933,
  [934] = 934,
  [935] = 797,
  [936] = 821,
  [937] = 937,
  [938] = 938,
  [939] = 834,
  [940] = 940,
  [941] = 818,
  [942] = 942,
  [943] = 943,
  [944] = 823,
  [945] = 826,
  [946] = 830,
  [947] = 947,
  [948] = 812,
  [949] = 949,
  [950] = 815,
  [951] = 951,
  [952] = 952,
  [953] = 953,
  [954] = 954,
  [955] = 955,
  [956] = 956,
  [957] = 957,
  [958] = 958,
  [959] = 959,
  [960] = 960,
  [961] = 961,
  [962] = 796,
  [963] = 963,
  [964] = 964,
  [965] = 820,
  [966] = 836,
  [967] = 837,
  [968] = 814,
  [969] = 969,
  [970] = 822,
  [971] = 971,
  [972] = 972,
  [973] = 973,
  [974] = 974,
  [975] = 835,
  [976] = 976,
  [977] = 977,
  [978] = 833,
  [979] = 811,
  [980] = 980,
  [981] = 981,
  [982] = 813,
  [983] = 817,
  [984] = 810,
  [985] = 985,
  [986] = 986,
  [987] = 987,
  [988] = 988,
  [989] = 989,
  [990] = 990,
  [991] = 991,
  [992] = 992,
  [993] = 993,
  [994] = 994,
  [995] = 995,
  [996] = 996,
  [997] = 997,
  [998] = 998,
  [999] = 999,
  [1000] = 1000,
  [1001] = 1001,
  [1002] = 1002,
  [1003] = 1003,
  [1004] = 1004,
  [1005] = 1005,
  [1006] = 1006,
  [1007] = 1007,
  [1008] = 1008,
  [1009] = 1009,
  [1010] = 1010,
  [1011] = 1011,
  [1012] = 938,
  [1013] = 1013,
  [1014] = 1014,
  [1015] = 1015,
  [1016] = 1016,
  [1017] = 1017,
  [1018] = 1018,
  [1019] = 1019,
  [1020] = 1020,
  [1021] = 1021,
  [1022] = 1022,
  [1023] = 825,
  [1024] = 1024,
  [1025] = 1024,
  [1026] = 1008,
  [1027] = 871,
  [1028] = 1028,
  [1029] = 871,
  [1030] = 1030,
  [1031] = 871,
  [1032] = 1028,
  [1033] = 1033,
  [1034] = 1034,
  [1035] = 1035,
  [1036] = 1036,
  [1037] = 1009,
  [1038] = 1035,
  [1039] = 1034,
  [1040] = 1035,
  [1041] = 1041,
  [1042] = 871,
  [1043] = 1043,
  [1044] = 1044,
  [1045] = 1024,
  [1046] = 1046,
  [1047] = 1030,
  [1048] = 1048,
  [1049] = 1030,
  [1050] = 1043,
  [1051] = 1051,
  [1052] = 1052,
  [1053] = 1034,
  [1054] = 924,
  [1055] = 871,
  [1056] = 1056,
  [1057] = 1057,
  [1058] = 1058,
  [1059] = 1059,
  [1060] = 1060,
  [1061] = 1061,
  [1062] = 900,
  [1063] = 1063,
  [1064] = 1064,
  [1065] = 1065,
  [1066] = 1066,
  [1067] = 1067,
  [1068] = 1068,
  [1069] = 1069,
  [1070] = 1070,
  [1071] = 1071,
  [1072] = 1072,
  [1073] = 889,
  [1074] = 1067,
  [1075] = 1067,
  [1076] = 1076,
  [1077] = 1061,
  [1078] = 1070,
  [1079] = 929,
  [1080] = 1076,
  [1081] = 1061,
  [1082] = 900,
  [1083] = 1083,
  [1084] = 249,
  [1085] = 1070,
  [1086] = 1086,
  [1087] = 1087,
  [1088] = 1061,
  [1089] = 900,
  [1090] = 253,
  [1091] = 1065,
  [1092] = 1092,
  [1093] = 1093,
  [1094] = 248,
  [1095] = 1070,
  [1096] = 889,
  [1097] = 1067,
  [1098] = 1018,
  [1099] = 1099,
  [1100] = 1061,
  [1101] = 1101,
  [1102] = 1102,
  [1103] = 1067,
  [1104] = 1104,
  [1105] = 1105,
  [1106] = 1106,
  [1107] = 1107,
  [1108] = 1108,
  [1109] = 1061,
  [1110] = 1070,
  [1111] = 1111,
  [1112] = 1112,
  [1113] = 1113,
  [1114] = 889,
  [1115] = 1115,
  [1116] = 1116,
  [1117] = 1117,
  [1118] = 1071,
  [1119] = 1066,
  [1120] = 1120,
  [1121] = 1070,
  [1122] = 1113,
  [1123] = 1123,
  [1124] = 1124,
  [1125] = 961,
  [1126] = 1126,
  [1127] = 1127,
  [1128] = 1069,
  [1129] = 889,
  [1130] = 1067,
  [1131] = 900,
  [1132] = 1070,
  [1133] = 1133,
  [1134] = 1061,
  [1135] = 1135,
  [1136] = 1136,
  [1137] = 1137,
  [1138] = 1138,
  [1139] = 1139,
  [1140] = 1140,
  [1141] = 889,
  [1142] = 250,
  [1143] = 251,
  [1144] = 900,
  [1145] = 1145,
  [1146] = 1146,
  [1147] = 1137,
  [1148] = 1148,
  [1149] = 1149,
  [1150] = 1150,
  [1151] = 1151,
  [1152] = 1149,
  [1153] = 1145,
  [1154] = 1154,
  [1155] = 1155,
  [1156] = 1146,
  [1157] = 1157,
  [1158] = 797,
  [1159] = 1068,
  [1160] = 1151,
  [1161] = 1161,
  [1162] = 1140,
  [1163] = 1104,
  [1164] = 1164,
  [1165] = 1165,
  [1166] = 1166,
  [1167] = 1167,
  [1168] = 1168,
  [1169] = 1169,
  [1170] = 1154,
  [1171] = 1171,
  [1172] = 1164,
  [1173] = 796,
  [1174] = 1167,
  [1175] = 1168,
  [1176] = 1169,
  [1177] = 1149,
  [1178] = 1145,
  [1179] = 1154,
  [1180] = 1155,
  [1181] = 1155,
  [1182] = 1146,
  [1183] = 1183,
  [1184] = 1184,
  [1185] = 1111,
  [1186] = 1149,
  [1187] = 1161,
  [1188] = 1151,
  [1189] = 1145,
  [1190] = 1092,
  [1191] = 1191,
  [1192] = 1154,
  [1193] = 1155,
  [1194] = 1164,
  [1195] = 1195,
  [1196] = 1167,
  [1197] = 1168,
  [1198] = 1198,
  [1199] = 1169,
  [1200] = 1149,
  [1201] = 1169,
  [1202] = 1154,
  [1203] = 1155,
  [1204] = 1058,
  [1205] = 1205,
  [1206] = 1101,
  [1207] = 1102,
  [1208] = 1208,
  [1209] = 1164,
  [1210] = 1210,
  [1211] = 1164,
  [1212] = 1106,
  [1213] = 1167,
  [1214] = 1168,
  [1215] = 1165,
  [1216] = 1169,
  [1217] = 1217,
  [1218] = 1149,
  [1219] = 1117,
  [1220] = 1145,
  [1221] = 1154,
  [1222] = 1222,
  [1223] = 1155,
  [1224] = 1224,
  [1225] = 1225,
  [1226] = 1048,
  [1227] = 1167,
  [1228] = 1228,
  [1229] = 1135,
  [1230] = 1148,
  [1231] = 1231,
  [1232] = 1168,
  [1233] = 1233,
  [1234] = 796,
  [1235] = 1235,
  [1236] = 1169,
  [1237] = 1237,
  [1238] = 1238,
  [1239] = 1161,
  [1240] = 1240,
  [1241] = 1241,
  [1242] = 1164,
  [1243] = 1165,
  [1244] = 1244,
  [1245] = 1167,
  [1246] = 1168,
  [1247] = 1247,
  [1248] = 1248,
  [1249] = 1145,
  [1250] = 937,
  [1251] = 1251,
  [1252] = 1252,
  [1253] = 1253,
  [1254] = 1254,
  [1255] = 1255,
  [1256] = 1256,
  [1257] = 1255,
  [1258] = 1254,
  [1259] = 1259,
  [1260] = 1255,
  [1261] = 1261,
  [1262] = 1252,
  [1263] = 1263,
  [1264] = 1264,
  [1265] = 954,
  [1266] = 1266,
  [1267] = 1267,
  [1268] = 1252,
  [1269] = 1269,
  [1270] = 1270,
  [1271] = 1271,
  [1272] = 1272,
  [1273] = 1273,
  [1274] = 1270,
  [1275] = 1013,
  [1276] = 1276,
  [1277] = 1270,
  [1278] = 1278,
  [1279] = 1279,
  [1280] = 1280,
  [1281] = 1255,
  [1282] = 1014,
  [1283] = 1283,
  [1284] = 1256,
  [1285] = 1285,
  [1286] = 1259,
  [1287] = 1285,
  [1288] = 1252,
  [1289] = 1289,
  [1290] = 1290,
  [1291] = 1252,
  [1292] = 1292,
  [1293] = 1270,
  [1294] = 1259,
  [1295] = 1255,
  [1296] = 1256,
  [1297] = 1297,
  [1298] = 952,
  [1299] = 1252,
  [1300] = 1300,
  [1301] = 1301,
  [1302] = 1259,
  [1303] = 1303,
  [1304] = 1304,
  [1305] = 1256,
  [1306] = 1256,
  [1307] = 1270,
  [1308] = 1254,
  [1309] = 1259,
  [1310] = 1310,
  [1311] = 1256,
  [1312] = 1312,
  [1313] = 1259,
  [1314] = 1270,
  [1315] = 1315,
  [1316] = 1316,
  [1317] = 1255,
  [1318] = 1303,
  [1319] = 1303,
  [1320] = 1303,
  [1321] = 1303,
  [1322] = 1303,
  [1323] = 1323,
  [1324] = 1324,
  [1325] = 1325,
  [1326] = 1326,
  [1327] = 1327,
  [1328] = 1328,
  [1329] = 1328,
  [1330] = 1330,
  [1331] = 1331,
  [1332] = 1325,
  [1333] = 1333,
  [1334] = 1334,
  [1335] = 1335,
  [1336] = 1336,
  [1337] = 1337,
  [1338] = 1328,
  [1339] = 1339,
  [1340] = 1340,
  [1341] = 1325,
  [1342] = 1342,
  [1343] = 1340,
  [1344] = 1334,
  [1345] = 1345,
  [1346] = 1346,
  [1347] = 1347,
  [1348] = 1327,
  [1349] = 1326,
  [1350] = 1350,
  [1351] = 1351,
  [1352] = 1350,
  [1353] = 1326,
  [1354] = 1354,
  [1355] = 1355,
  [1356] = 1356,
  [1357] = 1357,
  [1358] = 1358,
  [1359] = 1359,
  [1360] = 1325,
  [1361] = 1327,
  [1362] = 1362,
  [1363] = 1363,
  [1364] = 1336,
  [1365] = 1365,
  [1366] = 1366,
  [1367] = 1367,
  [1368] = 1326,
  [1369] = 1365,
  [1370] = 1328,
  [1371] = 1371,
  [1372] = 1357,
  [1373] = 1358,
  [1374] = 1325,
  [1375] = 1334,
  [1376] = 1350,
  [1377] = 1359,
  [1378] = 1378,
  [1379] = 1379,
  [1380] = 1380,
  [1381] = 1340,
  [1382] = 1359,
  [1383] = 1359,
  [1384] = 1358,
  [1385] = 1359,
  [1386] = 1327,
  [1387] = 1327,
  [1388] = 1365,
  [1389] = 1327,
  [1390] = 1326,
  [1391] = 1334,
  [1392] = 1359,
  [1393] = 1325,
  [1394] = 1358,
  [1395] = 1326,
  [1396] = 1350,
  [1397] = 1397,
  [1398] = 1398,
  [1399] = 1366,
  [1400] = 1400,
  [1401] = 1327,
  [1402] = 1402,
  [1403] = 1403,
  [1404] = 1358,
  [1405] = 1334,
  [1406] = 1406,
  [1407] = 1407,
  [1408] = 1328,
  [1409] = 1350,
  [1410] = 1326,
  [1411] = 1357,
  [1412] = 1358,
  [1413] = 1358,
  [1414] = 1400,
  [1415] = 1334,
  [1416] = 1400,
  [1417] = 1366,
  [1418] = 1400,
  [1419] = 1350,
  [1420] = 1400,
  [1421] = 1421,
  [1422] = 1400,
  [1423] = 1423,
  [1424] = 1328,
};

static TSCharacterRange sym_identifier_character_set_1[] = {
  {'A', 'Z'}, {'_', '_'}, {'a', 'z'}, {0xaa, 0xaa}, {0xb5, 0xb5}, {0xba, 0xba}, {0xc0, 0xd6}, {0xd8, 0xf6},
  {0xf8, 0x2c1}, {0x2c6, 0x2d1}, {0x2e0, 0x2e4}, {0x2ec, 0x2ec}, {0x2ee, 0x2ee}, {0x370, 0x374}, {0x376, 0x377}, {0x37b, 0x37d},
  {0x37f, 0x37f}, {0x386, 0x386}, {0x388, 0x38a}, {0x38c, 0x38c}, {0x38e, 0x3a1}, {0x3a3, 0x3f5}, {0x3f7, 0x481}, {0x48a, 0x52f},
  {0x531, 0x556}, {0x559, 0x559}, {0x560, 0x588}, {0x5d0, 0x5ea}, {0x5ef, 0x5f2}, {0x620, 0x64a}, {0x66e, 0x66f}, {0x671, 0x6d3},
  {0x6d5, 0x6d5}, {0x6e5, 0x6e6}, {0x6ee, 0x6ef}, {0x6fa, 0x6fc}, {0x6ff, 0x6ff}, {0x710, 0x710}, {0x712, 0x72f}, {0x74d, 0x7a5},
  {0x7b1, 0x7b1}, {0x7ca, 0x7ea}, {0x7f4, 0x7f5}, {0x7fa, 0x7fa}, {0x800, 0x815}, {0x81a, 0x81a}, {0x824, 0x824}, {0x828, 0x828},
  {0x840, 0x858}, {0x860, 0x86a}, {0x870, 0x887}, {0x889, 0x88e}, {0x8a0, 0x8c9}, {0x904, 0x939}, {0x93d, 0x93d}, {0x950, 0x950},
  {0x958, 0x961}, {0x971, 0x980}, {0x985, 0x98c}, {0x98f, 0x990}, {0x993, 0x9a8}, {0x9aa, 0x9b0}, {0x9b2, 0x9b2}, {0x9b6, 0x9b9},
  {0x9bd, 0x9bd}, {0x9ce, 0x9ce}, {0x9dc, 0x9dd}, {0x9df, 0x9e1}, {0x9f0, 0x9f1}, {0x9fc, 0x9fc}, {0xa05, 0xa0a}, {0xa0f, 0xa10},
  {0xa13, 0xa28}, {0xa2a, 0xa30}, {0xa32, 0xa33}, {0xa35, 0xa36}, {0xa38, 0xa39}, {0xa59, 0xa5c}, {0xa5e, 0xa5e}, {0xa72, 0xa74},
  {0xa85, 0xa8d}, {0xa8f, 0xa91}, {0xa93, 0xaa8}, {0xaaa, 0xab0}, {0xab2, 0xab3}, {0xab5, 0xab9}, {0xabd, 0xabd}, {0xad0, 0xad0},
  {0xae0, 0xae1}, {0xaf9, 0xaf9}, {0xb05, 0xb0c}, {0xb0f, 0xb10}, {0xb13, 0xb28}, {0xb2a, 0xb30}, {0xb32, 0xb33}, {0xb35, 0xb39},
  {0xb3d, 0xb3d}, {0xb5c, 0xb5d}, {0xb5f, 0xb61}, {0xb71, 0xb71}, {0xb83, 0xb83}, {0xb85, 0xb8a}, {0xb8e, 0xb90}, {0xb92, 0xb95},
  {0xb99, 0xb9a}, {0xb9c, 0xb9c}, {0xb9e, 0xb9f}, {0xba3, 0xba4}, {0xba8, 0xbaa}, {0xbae, 0xbb9}, {0xbd0, 0xbd0}, {0xc05, 0xc0c},
  {0xc0e, 0xc10}, {0xc12, 0xc28}, {0xc2a, 0xc39}, {0xc3d, 0xc3d}, {0xc58, 0xc5a}, {0xc5d, 0xc5d}, {0xc60, 0xc61}, {0xc80, 0xc80},
  {0xc85, 0xc8c}, {0xc8e, 0xc90}, {0xc92, 0xca8}, {0xcaa, 0xcb3}, {0xcb5, 0xcb9}, {0xcbd, 0xcbd}, {0xcdd, 0xcde}, {0xce0, 0xce1},
  {0xcf1, 0xcf2}, {0xd04, 0xd0c}, {0xd0e, 0xd10}, {0xd12, 0xd3a}, {0xd3d, 0xd3d}, {0xd4e, 0xd4e}, {0xd54, 0xd56}, {0xd5f, 0xd61},
  {0xd7a, 0xd7f}, {0xd85, 0xd96}, {0xd9a, 0xdb1}, {0xdb3, 0xdbb}, {0xdbd, 0xdbd}, {0xdc0, 0xdc6}, {0xe01, 0xe30}, {0xe32, 0xe32},
  {0xe40, 0xe46}, {0xe81, 0xe82}, {0xe84, 0xe84}, {0xe86, 0xe8a}, {0xe8c, 0xea3}, {0xea5, 0xea5}, {0xea7, 0xeb0}, {0xeb2, 0xeb2},
  {0xebd, 0xebd}, {0xec0, 0xec4}, {0xec6, 0xec6}, {0xedc, 0xedf}, {0xf00, 0xf00}, {0xf40, 0xf47}, {0xf49, 0xf6c}, {0xf88, 0xf8c},
  {0x1000, 0x102a}, {0x103f, 0x103f}, {0x1050, 0x1055}, {0x105a, 0x105d}, {0x1061, 0x1061}, {0x1065, 0x1066}, {0x106e, 0x1070}, {0x1075, 0x1081},
  {0x108e, 0x108e}, {0x10a0, 0x10c5}, {0x10c7, 0x10c7}, {0x10cd, 0x10cd}, {0x10d0, 0x10fa}, {0x10fc, 0x1248}, {0x124a, 0x124d}, {0x1250, 0x1256},
  {0x1258, 0x1258}, {0x125a, 0x125d}, {0x1260, 0x1288}, {0x128a, 0x128d}, {0x1290, 0x12b0}, {0x12b2, 0x12b5}, {0x12b8, 0x12be}, {0x12c0, 0x12c0},
  {0x12c2, 0x12c5}, {0x12c8, 0x12d6}, {0x12d8, 0x1310}, {0x1312, 0x1315}, {0x1318, 0x135a}, {0x1380, 0x138f}, {0x13a0, 0x13f5}, {0x13f8, 0x13fd},
  {0x1401, 0x166c}, {0x166f, 0x167f}, {0x1681, 0x169a}, {0x16a0, 0x16ea}, {0x16ee, 0x16f8}, {0x1700, 0x1711}, {0x171f, 0x1731}, {0x1740, 0x1751},
  {0x1760, 0x176c}, {0x176e, 0x1770}, {0x1780, 0x17b3}, {0x17d7, 0x17d7}, {0x17dc, 0x17dc}, {0x1820, 0x1878}, {0x1880, 0x18a8}, {0x18aa, 0x18aa},
  {0x18b0, 0x18f5}, {0x1900, 0x191e}, {0x1950, 0x196d}, {0x1970, 0x1974}, {0x1980, 0x19ab}, {0x19b0, 0x19c9}, {0x1a00, 0x1a16}, {0x1a20, 0x1a54},
  {0x1aa7, 0x1aa7}, {0x1b05, 0x1b33}, {0x1b45, 0x1b4c}, {0x1b83, 0x1ba0}, {0x1bae, 0x1baf}, {0x1bba, 0x1be5}, {0x1c00, 0x1c23}, {0x1c4d, 0x1c4f},
  {0x1c5a, 0x1c7d}, {0x1c80, 0x1c88}, {0x1c90, 0x1cba}, {0x1cbd, 0x1cbf}, {0x1ce9, 0x1cec}, {0x1cee, 0x1cf3}, {0x1cf5, 0x1cf6}, {0x1cfa, 0x1cfa},
  {0x1d00, 0x1dbf}, {0x1e00, 0x1f15}, {0x1f18, 0x1f1d}, {0x1f20, 0x1f45}, {0x1f48, 0x1f4d}, {0x1f50, 0x1f57}, {0x1f59, 0x1f59}, {0x1f5b, 0x1f5b},
  {0x1f5d, 0x1f5d}, {0x1f5f, 0x1f7d}, {0x1f80, 0x1fb4}, {0x1fb6, 0x1fbc}, {0x1fbe, 0x1fbe}, {0x1fc2, 0x1fc4}, {0x1fc6, 0x1fcc}, {0x1fd0, 0x1fd3},
  {0x1fd6, 0x1fdb}, {0x1fe0, 0x1fec}, {0x1ff2, 0x1ff4}, {0x1ff6, 0x1ffc}, {0x2071, 0x2071}, {0x207f, 0x207f}, {0x2090, 0x209c}, {0x2102, 0x2102},
  {0x2107, 0x2107}, {0x210a, 0x2113}, {0x2115, 0x2115}, {0x2118, 0x211d}, {0x2124, 0x2124}, {0x2126, 0x2126}, {0x2128, 0x2128}, {0x212a, 0x2139},
  {0x213c, 0x213f}, {0x2145, 0x2149}, {0x214e, 0x214e}, {0x2160, 0x2188}, {0x2c00, 0x2ce4}, {0x2ceb, 0x2cee}, {0x2cf2, 0x2cf3}, {0x2d00, 0x2d25},
  {0x2d27, 0x2d27}, {0x2d2d, 0x2d2d}, {0x2d30, 0x2d67}, {0x2d6f, 0x2d6f}, {0x2d80, 0x2d96}, {0x2da0, 0x2da6}, {0x2da8, 0x2dae}, {0x2db0, 0x2db6},
  {0x2db8, 0x2dbe}, {0x2dc0, 0x2dc6}, {0x2dc8, 0x2dce}, {0x2dd0, 0x2dd6}, {0x2dd8, 0x2dde}, {0x3005, 0x3007}, {0x3021, 0x3029}, {0x3031, 0x3035},
  {0x3038, 0x303c}, {0x3041, 0x3096}, {0x309d, 0x309f}, {0x30a1, 0x30fa}, {0x30fc, 0x30ff}, {0x3105, 0x312f}, {0x3131, 0x318e}, {0x31a0, 0x31bf},
  {0x31f0, 0x31ff}, {0x3400, 0x4dbf}, {0x4e00, 0xa48c}, {0xa4d0, 0xa4fd}, {0xa500, 0xa60c}, {0xa610, 0xa61f}, {0xa62a, 0xa62b}, {0xa640, 0xa66e},
  {0xa67f, 0xa69d}, {0xa6a0, 0xa6ef}, {0xa717, 0xa71f}, {0xa722, 0xa788}, {0xa78b, 0xa7ca}, {0xa7d0, 0xa7d1}, {0xa7d3, 0xa7d3}, {0xa7d5, 0xa7d9},
  {0xa7f2, 0xa801}, {0xa803, 0xa805}, {0xa807, 0xa80a}, {0xa80c, 0xa822}, {0xa840, 0xa873}, {0xa882, 0xa8b3}, {0xa8f2, 0xa8f7}, {0xa8fb, 0xa8fb},
  {0xa8fd, 0xa8fe}, {0xa90a, 0xa925}, {0xa930, 0xa946}, {0xa960, 0xa97c}, {0xa984, 0xa9b2}, {0xa9cf, 0xa9cf}, {0xa9e0, 0xa9e4}, {0xa9e6, 0xa9ef},
  {0xa9fa, 0xa9fe}, {0xaa00, 0xaa28}, {0xaa40, 0xaa42}, {0xaa44, 0xaa4b}, {0xaa60, 0xaa76}, {0xaa7a, 0xaa7a}, {0xaa7e, 0xaaaf}, {0xaab1, 0xaab1},
  {0xaab5, 0xaab6}, {0xaab9, 0xaabd}, {0xaac0, 0xaac0}, {0xaac2, 0xaac2}, {0xaadb, 0xaadd}, {0xaae0, 0xaaea}, {0xaaf2, 0xaaf4}, {0xab01, 0xab06},
  {0xab09, 0xab0e}, {0xab11, 0xab16}, {0xab20, 0xab26}, {0xab28, 0xab2e}, {0xab30, 0xab5a}, {0xab5c, 0xab69}, {0xab70, 0xabe2}, {0xac00, 0xd7a3},
  {0xd7b0, 0xd7c6}, {0xd7cb, 0xd7fb}, {0xf900, 0xfa6d}, {0xfa70, 0xfad9}, {0xfb00, 0xfb06}, {0xfb13, 0xfb17}, {0xfb1d, 0xfb1d}, {0xfb1f, 0xfb28},
  {0xfb2a, 0xfb36}, {0xfb38, 0xfb3c}, {0xfb3e, 0xfb3e}, {0xfb40, 0xfb41}, {0xfb43, 0xfb44}, {0xfb46, 0xfbb1}, {0xfbd3, 0xfc5d}, {0xfc64, 0xfd3d},
  {0xfd50, 0xfd8f}, {0xfd92, 0xfdc7}, {0xfdf0, 0xfdf9}, {0xfe71, 0xfe71}, {0xfe73, 0xfe73}, {0xfe77, 0xfe77}, {0xfe79, 0xfe79}, {0xfe7b, 0xfe7b},
  {0xfe7d, 0xfe7d}, {0xfe7f, 0xfefc}, {0xff21, 0xff3a}, {0xff41, 0xff5a}, {0xff66, 0xff9d}, {0xffa0, 0xffbe}, {0xffc2, 0xffc7}, {0xffca, 0xffcf},
  {0xffd2, 0xffd7}, {0xffda, 0xffdc}, {0x10000, 0x1000b}, {0x1000d, 0x10026}, {0x10028, 0x1003a}, {0x1003c, 0x1003d}, {0x1003f, 0x1004d}, {0x10050, 0x1005d},
  {0x10080, 0x100fa}, {0x10140, 0x10174}, {0x10280, 0x1029c}, {0x102a0, 0x102d0}, {0x10300, 0x1031f}, {0x1032d, 0x1034a}, {0x10350, 0x10375}, {0x10380, 0x1039d},
  {0x103a0, 0x103c3}, {0x103c8, 0x103cf}, {0x103d1, 0x103d5}, {0x10400, 0x1049d}, {0x104b0, 0x104d3}, {0x104d8, 0x104fb}, {0x10500, 0x10527}, {0x10530, 0x10563},
  {0x10570, 0x1057a}, {0x1057c, 0x1058a}, {0x1058c, 0x10592}, {0x10594, 0x10595}, {0x10597, 0x105a1}, {0x105a3, 0x105b1}, {0x105b3, 0x105b9}, {0x105bb, 0x105bc},
  {0x10600, 0x10736}, {0x10740, 0x10755}, {0x10760, 0x10767}, {0x10780, 0x10785}, {0x10787, 0x107b0}, {0x107b2, 0x107ba}, {0x10800, 0x10805}, {0x10808, 0x10808},
  {0x1080a, 0x10835}, {0x10837, 0x10838}, {0x1083c, 0x1083c}, {0x1083f, 0x10855}, {0x10860, 0x10876}, {0x10880, 0x1089e}, {0x108e0, 0x108f2}, {0x108f4, 0x108f5},
  {0x10900, 0x10915}, {0x10920, 0x10939}, {0x10980, 0x109b7}, {0x109be, 0x109bf}, {0x10a00, 0x10a00}, {0x10a10, 0x10a13}, {0x10a15, 0x10a17}, {0x10a19, 0x10a35},
  {0x10a60, 0x10a7c}, {0x10a80, 0x10a9c}, {0x10ac0, 0x10ac7}, {0x10ac9, 0x10ae4}, {0x10b00, 0x10b35}, {0x10b40, 0x10b55}, {0x10b60, 0x10b72}, {0x10b80, 0x10b91},
  {0x10c00, 0x10c48}, {0x10c80, 0x10cb2}, {0x10cc0, 0x10cf2}, {0x10d00, 0x10d23}, {0x10e80, 0x10ea9}, {0x10eb0, 0x10eb1}, {0x10f00, 0x10f1c}, {0x10f27, 0x10f27},
  {0x10f30, 0x10f45}, {0x10f70, 0x10f81}, {0x10fb0, 0x10fc4}, {0x10fe0, 0x10ff6}, {0x11003, 0x11037}, {0x11071, 0x11072}, {0x11075, 0x11075}, {0x11083, 0x110af},
  {0x110d0, 0x110e8}, {0x11103, 0x11126}, {0x11144, 0x11144}, {0x11147, 0x11147}, {0x11150, 0x11172}, {0x11176, 0x11176}, {0x11183, 0x111b2}, {0x111c1, 0x111c4},
  {0x111da, 0x111da}, {0x111dc, 0x111dc}, {0x11200, 0x11211}, {0x11213, 0x1122b}, {0x1123f, 0x11240}, {0x11280, 0x11286}, {0x11288, 0x11288}, {0x1128a, 0x1128d},
  {0x1128f, 0x1129d}, {0x1129f, 0x112a8}, {0x112b0, 0x112de}, {0x11305, 0x1130c}, {0x1130f, 0x11310}, {0x11313, 0x11328}, {0x1132a, 0x11330}, {0x11332, 0x11333},
  {0x11335, 0x11339}, {0x1133d, 0x1133d}, {0x11350, 0x11350}, {0x1135d, 0x11361}, {0x11400, 0x11434}, {0x11447, 0x1144a}, {0x1145f, 0x11461}, {0x11480, 0x114af},
  {0x114c4, 0x114c5}, {0x114c7, 0x114c7}, {0x11580, 0x115ae}, {0x115d8, 0x115db}, {0x11600, 0x1162f}, {0x11644, 0x11644}, {0x11680, 0x116aa}, {0x116b8, 0x116b8},
  {0x11700, 0x1171a}, {0x11740, 0x11746}, {0x11800, 0x1182b}, {0x118a0, 0x118df}, {0x118ff, 0x11906}, {0x11909, 0x11909}, {0x1190c, 0x11913}, {0x11915, 0x11916},
  {0x11918, 0x1192f}, {0x1193f, 0x1193f}, {0x11941, 0x11941}, {0x119a0, 0x119a7}, {0x119aa, 0x119d0}, {0x119e1, 0x119e1}, {0x119e3, 0x119e3}, {0x11a00, 0x11a00},
  {0x11a0b, 0x11a32}, {0x11a3a, 0x11a3a}, {0x11a50, 0x11a50}, {0x11a5c, 0x11a89}, {0x11a9d, 0x11a9d}, {0x11ab0, 0x11af8}, {0x11c00, 0x11c08}, {0x11c0a, 0x11c2e},
  {0x11c40, 0x11c40}, {0x11c72, 0x11c8f}, {0x11d00, 0x11d06}, {0x11d08, 0x11d09}, {0x11d0b, 0x11d30}, {0x11d46, 0x11d46}, {0x11d60, 0x11d65}, {0x11d67, 0x11d68},
  {0x11d6a, 0x11d89}, {0x11d98, 0x11d98}, {0x11ee0, 0x11ef2}, {0x11f02, 0x11f02}, {0x11f04, 0x11f10}, {0x11f12, 0x11f33}, {0x11fb0, 0x11fb0}, {0x12000, 0x12399},
  {0x12400, 0x1246e}, {0x12480, 0x12543}, {0x12f90, 0x12ff0}, {0x13000, 0x1342f}, {0x13441, 0x13446}, {0x14400, 0x14646}, {0x16800, 0x16a38}, {0x16a40, 0x16a5e},
  {0x16a70, 0x16abe}, {0x16ad0, 0x16aed}, {0x16b00, 0x16b2f}, {0x16b40, 0x16b43}, {0x16b63, 0x16b77}, {0x16b7d, 0x16b8f}, {0x16e40, 0x16e7f}, {0x16f00, 0x16f4a},
  {0x16f50, 0x16f50}, {0x16f93, 0x16f9f}, {0x16fe0, 0x16fe1}, {0x16fe3, 0x16fe3}, {0x17000, 0x187f7}, {0x18800, 0x18cd5}, {0x18d00, 0x18d08}, {0x1aff0, 0x1aff3},
  {0x1aff5, 0x1affb}, {0x1affd, 0x1affe}, {0x1b000, 0x1b122}, {0x1b132, 0x1b132}, {0x1b150, 0x1b152}, {0x1b155, 0x1b155}, {0x1b164, 0x1b167}, {0x1b170, 0x1b2fb},
  {0x1bc00, 0x1bc6a}, {0x1bc70, 0x1bc7c}, {0x1bc80, 0x1bc88}, {0x1bc90, 0x1bc99}, {0x1d400, 0x1d454}, {0x1d456, 0x1d49c}, {0x1d49e, 0x1d49f}, {0x1d4a2, 0x1d4a2},
  {0x1d4a5, 0x1d4a6}, {0x1d4a9, 0x1d4ac}, {0x1d4ae, 0x1d4b9}, {0x1d4bb, 0x1d4bb}, {0x1d4bd, 0x1d4c3}, {0x1d4c5, 0x1d505}, {0x1d507, 0x1d50a}, {0x1d50d, 0x1d514},
  {0x1d516, 0x1d51c}, {0x1d51e, 0x1d539}, {0x1d53b, 0x1d53e}, {0x1d540, 0x1d544}, {0x1d546, 0x1d546}, {0x1d54a, 0x1d550}, {0x1d552, 0x1d6a5}, {0x1d6a8, 0x1d6c0},
  {0x1d6c2, 0x1d6da}, {0x1d6dc, 0x1d6fa}, {0x1d6fc, 0x1d714}, {0x1d716, 0x1d734}, {0x1d736, 0x1d74e}, {0x1d750, 0x1d76e}, {0x1d770, 0x1d788}, {0x1d78a, 0x1d7a8},
  {0x1d7aa, 0x1d7c2}, {0x1d7c4, 0x1d7cb}, {0x1df00, 0x1df1e}, {0x1df25, 0x1df2a}, {0x1e030, 0x1e06d}, {0x1e100, 0x1e12c}, {0x1e137, 0x1e13d}, {0x1e14e, 0x1e14e},
  {0x1e290, 0x1e2ad}, {0x1e2c0, 0x1e2eb}, {0x1e4d0, 0x1e4eb}, {0x1e7e0, 0x1e7e6}, {0x1e7e8, 0x1e7eb}, {0x1e7ed, 0x1e7ee}, {0x1e7f0, 0x1e7fe}, {0x1e800, 0x1e8c4},
  {0x1e900, 0x1e943}, {0x1e94b, 0x1e94b}, {0x1ee00, 0x1ee03}, {0x1ee05, 0x1ee1f}, {0x1ee21, 0x1ee22}, {0x1ee24, 0x1ee24}, {0x1ee27, 0x1ee27}, {0x1ee29, 0x1ee32},
  {0x1ee34, 0x1ee37}, {0x1ee39, 0x1ee39}, {0x1ee3b, 0x1ee3b}, {0x1ee42, 0x1ee42}, {0x1ee47, 0x1ee47}, {0x1ee49, 0x1ee49}, {0x1ee4b, 0x1ee4b}, {0x1ee4d, 0x1ee4f},
  {0x1ee51, 0x1ee52}, {0x1ee54, 0x1ee54}, {0x1ee57, 0x1ee57}, {0x1ee59, 0x1ee59}, {0x1ee5b, 0x1ee5b}, {0x1ee5d, 0x1ee5d}, {0x1ee5f, 0x1ee5f}, {0x1ee61, 0x1ee62},
  {0x1ee64, 0x1ee64}, {0x1ee67, 0x1ee6a}, {0x1ee6c, 0x1ee72}, {0x1ee74, 0x1ee77}, {0x1ee79, 0x1ee7c}, {0x1ee7e, 0x1ee7e}, {0x1ee80, 0x1ee89}, {0x1ee8b, 0x1ee9b},
  {0x1eea1, 0x1eea3}, {0x1eea5, 0x1eea9}, {0x1eeab, 0x1eebb}, {0x20000, 0x2a6df}, {0x2a700, 0x2b739}, {0x2b740, 0x2b81d}, {0x2b820, 0x2cea1}, {0x2ceb0, 0x2ebe0},
  {0x2ebf0, 0x2ee5d}, {0x2f800, 0x2fa1d}, {0x30000, 0x3134a}, {0x31350, 0x323af},
};

static TSCharacterRange sym_identifier_character_set_2[] = {
  {'0', '9'}, {'A', 'Z'}, {'_', '_'}, {'a', 'z'}, {0xaa, 0xaa}, {0xb5, 0xb5}, {0xb7, 0xb7}, {0xba, 0xba},
  {0xc0, 0xd6}, {0xd8, 0xf6}, {0xf8, 0x2c1}, {0x2c6, 0x2d1}, {0x2e0, 0x2e4}, {0x2ec, 0x2ec}, {0x2ee, 0x2ee}, {0x300, 0x374},
  {0x376, 0x377}, {0x37b, 0x37d}, {0x37f, 0x37f}, {0x386, 0x38a}, {0x38c, 0x38c}, {0x38e, 0x3a1}, {0x3a3, 0x3f5}, {0x3f7, 0x481},
  {0x483, 0x487}, {0x48a, 0x52f}, {0x531, 0x556}, {0x559, 0x559}, {0x560, 0x588}, {0x591, 0x5bd}, {0x5bf, 0x5bf}, {0x5c1, 0x5c2},
  {0x5c4, 0x5c5}, {0x5c7, 0x5c7}, {0x5d0, 0x5ea}, {0x5ef, 0x5f2}, {0x610, 0x61a}, {0x620, 0x669}, {0x66e, 0x6d3}, {0x6d5, 0x6dc},
  {0x6df, 0x6e8}, {0x6ea, 0x6fc}, {0x6ff, 0x6ff}, {0x710, 0x74a}, {0x74d, 0x7b1}, {0x7c0, 0x7f5}, {0x7fa, 0x7fa}, {0x7fd, 0x7fd},
  {0x800, 0x82d}, {0x840, 0x85b}, {0x860, 0x86a}, {0x870, 0x887}, {0x889, 0x88e}, {0x898, 0x8e1}, {0x8e3, 0x963}, {0x966, 0x96f},
  {0x971, 0x983}, {0x985, 0x98c}, {0x98f, 0x990}, {0x993, 0x9a8}, {0x9aa, 0x9b0}, {0x9b2, 0x9b2}, {0x9b6, 0x9b9}, {0x9bc, 0x9c4},
  {0x9c7, 0x9c8}, {0x9cb, 0x9ce}, {0x9d7, 0x9d7}, {0x9dc, 0x9dd}, {0x9df, 0x9e3}, {0x9e6, 0x9f1}, {0x9fc, 0x9fc}, {0x9fe, 0x9fe},
  {0xa01, 0xa03}, {0xa05, 0xa0a}, {0xa0f, 0xa10}, {0xa13, 0xa28}, {0xa2a, 0xa30}, {0xa32, 0xa33}, {0xa35, 0xa36}, {0xa38, 0xa39},
  {0xa3c, 0xa3c}, {0xa3e, 0xa42}, {0xa47, 0xa48}, {0xa4b, 0xa4d}, {0xa51, 0xa51}, {0xa59, 0xa5c}, {0xa5e, 0xa5e}, {0xa66, 0xa75},
  {0xa81, 0xa83}, {0xa85, 0xa8d}, {0xa8f, 0xa91}, {0xa93, 0xaa8}, {0xaaa, 0xab0}, {0xab2, 0xab3}, {0xab5, 0xab9}, {0xabc, 0xac5},
  {0xac7, 0xac9}, {0xacb, 0xacd}, {0xad0, 0xad0}, {0xae0, 0xae3}, {0xae6, 0xaef}, {0xaf9, 0xaff}, {0xb01, 0xb03}, {0xb05, 0xb0c},
  {0xb0f, 0xb10}, {0xb13, 0xb28}, {0xb2a, 0xb30}, {0xb32, 0xb33}, {0xb35, 0xb39}, {0xb3c, 0xb44}, {0xb47, 0xb48}, {0xb4b, 0xb4d},
  {0xb55, 0xb57}, {0xb5c, 0xb5d}, {0xb5f, 0xb63}, {0xb66, 0xb6f}, {0xb71, 0xb71}, {0xb82, 0xb83}, {0xb85, 0xb8a}, {0xb8e, 0xb90},
  {0xb92, 0xb95}, {0xb99, 0xb9a}, {0xb9c, 0xb9c}, {0xb9e, 0xb9f}, {0xba3, 0xba4}, {0xba8, 0xbaa}, {0xbae, 0xbb9}, {0xbbe, 0xbc2},
  {0xbc6, 0xbc8}, {0xbca, 0xbcd}, {0xbd0, 0xbd0}, {0xbd7, 0xbd7}, {0xbe6, 0xbef}, {0xc00, 0xc0c}, {0xc0e, 0xc10}, {0xc12, 0xc28},
  {0xc2a, 0xc39}, {0xc3c, 0xc44}, {0xc46, 0xc48}, {0xc4a, 0xc4d}, {0xc55, 0xc56}, {0xc58, 0xc5a}, {0xc5d, 0xc5d}, {0xc60, 0xc63},
  {0xc66, 0xc6f}, {0xc80, 0xc83}, {0xc85, 0xc8c}, {0xc8e, 0xc90}, {0xc92, 0xca8}, {0xcaa, 0xcb3}, {0xcb5, 0xcb9}, {0xcbc, 0xcc4},
  {0xcc6, 0xcc8}, {0xcca, 0xccd}, {0xcd5, 0xcd6}, {0xcdd, 0xcde}, {0xce0, 0xce3}, {0xce6, 0xcef}, {0xcf1, 0xcf3}, {0xd00, 0xd0c},
  {0xd0e, 0xd10}, {0xd12, 0xd44}, {0xd46, 0xd48}, {0xd4a, 0xd4e}, {0xd54, 0xd57}, {0xd5f, 0xd63}, {0xd66, 0xd6f}, {0xd7a, 0xd7f},
  {0xd81, 0xd83}, {0xd85, 0xd96}, {0xd9a, 0xdb1}, {0xdb3, 0xdbb}, {0xdbd, 0xdbd}, {0xdc0, 0xdc6}, {0xdca, 0xdca}, {0xdcf, 0xdd4},
  {0xdd6, 0xdd6}, {0xdd8, 0xddf}, {0xde6, 0xdef}, {0xdf2, 0xdf3}, {0xe01, 0xe3a}, {0xe40, 0xe4e}, {0xe50, 0xe59}, {0xe81, 0xe82},
  {0xe84, 0xe84}, {0xe86, 0xe8a}, {0xe8c, 0xea3}, {0xea5, 0xea5}, {0xea7, 0xebd}, {0xec0, 0xec4}, {0xec6, 0xec6}, {0xec8, 0xece},
  {0xed0, 0xed9}, {0xedc, 0xedf}, {0xf00, 0xf00}, {0xf18, 0xf19}, {0xf20, 0xf29}, {0xf35, 0xf35}, {0xf37, 0xf37}, {0xf39, 0xf39},
  {0xf3e, 0xf47}, {0xf49, 0xf6c}, {0xf71, 0xf84}, {0xf86, 0xf97}, {0xf99, 0xfbc}, {0xfc6, 0xfc6}, {0x1000, 0x1049}, {0x1050, 0x109d},
  {0x10a0, 0x10c5}, {0x10c7, 0x10c7}, {0x10cd, 0x10cd}, {0x10d0, 0x10fa}, {0x10fc, 0x1248}, {0x124a, 0x124d}, {0x1250, 0x1256}, {0x1258, 0x1258},
  {0x125a, 0x125d}, {0x1260, 0x1288}, {0x128a, 0x128d}, {0x1290, 0x12b0}, {0x12b2, 0x12b5}, {0x12b8, 0x12be}, {0x12c0, 0x12c0}, {0x12c2, 0x12c5},
  {0x12c8, 0x12d6}, {0x12d8, 0x1310}, {0x1312, 0x1315}, {0x1318, 0x135a}, {0x135d, 0x135f}, {0x1369, 0x1371}, {0x1380, 0x138f}, {0x13a0, 0x13f5},
  {0x13f8, 0x13fd}, {0x1401, 0x166c}, {0x166f, 0x167f}, {0x1681, 0x169a}, {0x16a0, 0x16ea}, {0x16ee, 0x16f8}, {0x1700, 0x1715}, {0x171f, 0x1734},
  {0x1740, 0x1753}, {0x1760, 0x176c}, {0x176e, 0x1770}, {0x1772, 0x1773}, {0x1780, 0x17d3}, {0x17d7, 0x17d7}, {0x17dc, 0x17dd}, {0x17e0, 0x17e9},
  {0x180b, 0x180d}, {0x180f, 0x1819}, {0x1820, 0x1878}, {0x1880, 0x18aa}, {0x18b0, 0x18f5}, {0x1900, 0x191e}, {0x1920, 0x192b}, {0x1930, 0x193b},
  {0x1946, 0x196d}, {0x1970, 0x1974}, {0x1980, 0x19ab}, {0x19b0, 0x19c9}, {0x19d0, 0x19da}, {0x1a00, 0x1a1b}, {0x1a20, 0x1a5e}, {0x1a60, 0x1a7c},
  {0x1a7f, 0x1a89}, {0x1a90, 0x1a99}, {0x1aa7, 0x1aa7}, {0x1ab0, 0x1abd}, {0x1abf, 0x1ace}, {0x1b00, 0x1b4c}, {0x1b50, 0x1b59}, {0x1b6b, 0x1b73},
  {0x1b80, 0x1bf3}, {0x1c00, 0x1c37}, {0x1c40, 0x1c49}, {0x1c4d, 0x1c7d}, {0x1c80, 0x1c88}, {0x1c90, 0x1cba}, {0x1cbd, 0x1cbf}, {0x1cd0, 0x1cd2},
  {0x1cd4, 0x1cfa}, {0x1d00, 0x1f15}, {0x1f18, 0x1f1d}, {0x1f20, 0x1f45}, {0x1f48, 0x1f4d}, {0x1f50, 0x1f57}, {0x1f59, 0x1f59}, {0x1f5b, 0x1f5b},
  {0x1f5d, 0x1f5d}, {0x1f5f, 0x1f7d}, {0x1f80, 0x1fb4}, {0x1fb6, 0x1fbc}, {0x1fbe, 0x1fbe}, {0x1fc2, 0x1fc4}, {0x1fc6, 0x1fcc}, {0x1fd0, 0x1fd3},
  {0x1fd6, 0x1fdb}, {0x1fe0, 0x1fec}, {0x1ff2, 0x1ff4}, {0x1ff6, 0x1ffc}, {0x200c, 0x200d}, {0x203f, 0x2040}, {0x2054, 0x2054}, {0x2071, 0x2071},
  {0x207f, 0x207f}, {0x2090, 0x209c}, {0x20d0, 0x20dc}, {0x20e1, 0x20e1}, {0x20e5, 0x20f0}, {0x2102, 0x2102}, {0x2107, 0x2107}, {0x210a, 0x2113},
  {0x2115, 0x2115}, {0x2118, 0x211d}, {0x2124, 0x2124}, {0x2126, 0x2126}, {0x2128, 0x2128}, {0x212a, 0x2139}, {0x213c, 0x213f}, {0x2145, 0x2149},
  {0x214e, 0x214e}, {0x2160, 0x2188}, {0x2c00, 0x2ce4}, {0x2ceb, 0x2cf3}, {0x2d00, 0x2d25}, {0x2d27, 0x2d27}, {0x2d2d, 0x2d2d}, {0x2d30, 0x2d67},
  {0x2d6f, 0x2d6f}, {0x2d7f, 0x2d96}, {0x2da0, 0x2da6}, {0x2da8, 0x2dae}, {0x2db0, 0x2db6}, {0x2db8, 0x2dbe}, {0x2dc0, 0x2dc6}, {0x2dc8, 0x2dce},
  {0x2dd0, 0x2dd6}, {0x2dd8, 0x2dde}, {0x2de0, 0x2dff}, {0x3005, 0x3007}, {0x3021, 0x302f}, {0x3031, 0x3035}, {0x3038, 0x303c}, {0x3041, 0x3096},
  {0x3099, 0x309a}, {0x309d, 0x309f}, {0x30a1, 0x30ff}, {0x3105, 0x312f}, {0x3131, 0x318e}, {0x31a0, 0x31bf}, {0x31f0, 0x31ff}, {0x3400, 0x4dbf},
  {0x4e00, 0xa48c}, {0xa4d0, 0xa4fd}, {0xa500, 0xa60c}, {0xa610, 0xa62b}, {0xa640, 0xa66f}, {0xa674, 0xa67d}, {0xa67f, 0xa6f1}, {0xa717, 0xa71f},
  {0xa722, 0xa788}, {0xa78b, 0xa7ca}, {0xa7d0, 0xa7d1}, {0xa7d3, 0xa7d3}, {0xa7d5, 0xa7d9}, {0xa7f2, 0xa827}, {0xa82c, 0xa82c}, {0xa840, 0xa873},
  {0xa880, 0xa8c5}, {0xa8d0, 0xa8d9}, {0xa8e0, 0xa8f7}, {0xa8fb, 0xa8fb}, {0xa8fd, 0xa92d}, {0xa930, 0xa953}, {0xa960, 0xa97c}, {0xa980, 0xa9c0},
  {0xa9cf, 0xa9d9}, {0xa9e0, 0xa9fe}, {0xaa00, 0xaa36}, {0xaa40, 0xaa4d}, {0xaa50, 0xaa59}, {0xaa60, 0xaa76}, {0xaa7a, 0xaac2}, {0xaadb, 0xaadd},
  {0xaae0, 0xaaef}, {0xaaf2, 0xaaf6}, {0xab01, 0xab06}, {0xab09, 0xab0e}, {0xab11, 0xab16}, {0xab20, 0xab26}, {0xab28, 0xab2e}, {0xab30, 0xab5a},
  {0xab5c, 0xab69}, {0xab70, 0xabea}, {0xabec, 0xabed}, {0xabf0, 0xabf9}, {0xac00, 0xd7a3}, {0xd7b0, 0xd7c6}, {0xd7cb, 0xd7fb}, {0xf900, 0xfa6d},
  {0xfa70, 0xfad9}, {0xfb00, 0xfb06}, {0xfb13, 0xfb17}, {0xfb1d, 0xfb28}, {0xfb2a, 0xfb36}, {0xfb38, 0xfb3c}, {0xfb3e, 0xfb3e}, {0xfb40, 0xfb41},
  {0xfb43, 0xfb44}, {0xfb46, 0xfbb1}, {0xfbd3, 0xfc5d}, {0xfc64, 0xfd3d}, {0xfd50, 0xfd8f}, {0xfd92, 0xfdc7}, {0xfdf0, 0xfdf9}, {0xfe00, 0xfe0f},
  {0xfe20, 0xfe2f}, {0xfe33, 0xfe34}, {0xfe4d, 0xfe4f}, {0xfe71, 0xfe71}, {0xfe73, 0xfe73}, {0xfe77, 0xfe77}, {0xfe79, 0xfe79}, {0xfe7b, 0xfe7b},
  {0xfe7d, 0xfe7d}, {0xfe7f, 0xfefc}, {0xff10, 0xff19}, {0xff21, 0xff3a}, {0xff3f, 0xff3f}, {0xff41, 0xff5a}, {0xff65, 0xffbe}, {0xffc2, 0xffc7},
  {0xffca, 0xffcf}, {0xffd2, 0xffd7}, {0xffda, 0xffdc}, {0x10000, 0x1000b}, {0x1000d, 0x10026}, {0x10028, 0x1003a}, {0x1003c, 0x1003d}, {0x1003f, 0x1004d},
  {0x10050, 0x1005d}, {0x10080, 0x100fa}, {0x10140, 0x10174}, {0x101fd, 0x101fd}, {0x10280, 0x1029c}, {0x102a0, 0x102d0}, {0x102e0, 0x102e0}, {0x10300, 0x1031f},
  {0x1032d, 0x1034a}, {0x10350, 0x1037a}, {0x10380, 0x1039d}, {0x103a0, 0x103c3}, {0x103c8, 0x103cf}, {0x103d1, 0x103d5}, {0x10400, 0x1049d}, {0x104a0, 0x104a9},
  {0x104b0, 0x104d3}, {0x104d8, 0x104fb}, {0x10500, 0x10527}, {0x10530, 0x10563}, {0x10570, 0x1057a}, {0x1057c, 0x1058a}, {0x1058c, 0x10592}, {0x10594, 0x10595},
  {0x10597, 0x105a1}, {0x105a3, 0x105b1}, {0x105b3, 0x105b9}, {0x105bb, 0x105bc}, {0x10600, 0x10736}, {0x10740, 0x10755}, {0x10760, 0x10767}, {0x10780, 0x10785},
  {0x10787, 0x107b0}, {0x107b2, 0x107ba}, {0x10800, 0x10805}, {0x10808, 0x10808}, {0x1080a, 0x10835}, {0x10837, 0x10838}, {0x1083c, 0x1083c}, {0x1083f, 0x10855},
  {0x10860, 0x10876}, {0x10880, 0x1089e}, {0x108e0, 0x108f2}, {0x108f4, 0x108f5}, {0x10900, 0x10915}, {0x10920, 0x10939}, {0x10980, 0x109b7}, {0x109be, 0x109bf},
  {0x10a00, 0x10a03}, {0x10a05, 0x10a06}, {0x10a0c, 0x10a13}, {0x10a15, 0x10a17}, {0x10a19, 0x10a35}, {0x10a38, 0x10a3a}, {0x10a3f, 0x10a3f}, {0x10a60, 0x10a7c},
  {0x10a80, 0x10a9c}, {0x10ac0, 0x10ac7}, {0x10ac9, 0x10ae6}, {0x10b00, 0x10b35}, {0x10b40, 0x10b55}, {0x10b60, 0x10b72}, {0x10b80, 0x10b91}, {0x10c00, 0x10c48},
  {0x10c80, 0x10cb2}, {0x10cc0, 0x10cf2}, {0x10d00, 0x10d27}, {0x10d30, 0x10d39}, {0x10e80, 0x10ea9}, {0x10eab, 0x10eac}, {0x10eb0, 0x10eb1}, {0x10efd, 0x10f1c},
  {0x10f27, 0x10f27}, {0x10f30, 0x10f50}, {0x10f70, 0x10f85}, {0x10fb0, 0x10fc4}, {0x10fe0, 0x10ff6}, {0x11000, 0x11046}, {0x11066, 0x11075}, {0x1107f, 0x110ba},
  {0x110c2, 0x110c2}, {0x110d0, 0x110e8}, {0x110f0, 0x110f9}, {0x11100, 0x11134}, {0x11136, 0x1113f}, {0x11144, 0x11147}, {0x11150, 0x11173}, {0x11176, 0x11176},
  {0x11180, 0x111c4}, {0x111c9, 0x111cc}, {0x111ce, 0x111da}, {0x111dc, 0x111dc}, {0x11200, 0x11211}, {0x11213, 0x11237}, {0x1123e, 0x11241}, {0x11280, 0x11286},
  {0x11288, 0x11288}, {0x1128a, 0x1128d}, {0x1128f, 0x1129d}, {0x1129f, 0x112a8}, {0x112b0, 0x112ea}, {0x112f0, 0x112f9}, {0x11300, 0x11303}, {0x11305, 0x1130c},
  {0x1130f, 0x11310}, {0x11313, 0x11328}, {0x1132a, 0x11330}, {0x11332, 0x11333}, {0x11335, 0x11339}, {0x1133b, 0x11344}, {0x11347, 0x11348}, {0x1134b, 0x1134d},
  {0x11350, 0x11350}, {0x11357, 0x11357}, {0x1135d, 0x11363}, {0x11366, 0x1136c}, {0x11370, 0x11374}, {0x11400, 0x1144a}, {0x11450, 0x11459}, {0x1145e, 0x11461},
  {0x11480, 0x114c5}, {0x114c7, 0x114c7}, {0x114d0, 0x114d9}, {0x11580, 0x115b5}, {0x115b8, 0x115c0}, {0x115d8, 0x115dd}, {0x11600, 0x11640}, {0x11644, 0x11644},
  {0x11650, 0x11659}, {0x11680, 0x116b8}, {0x116c0, 0x116c9}, {0x11700, 0x1171a}, {0x1171d, 0x1172b}, {0x11730, 0x11739}, {0x11740, 0x11746}, {0x11800, 0x1183a},
  {0x118a0, 0x118e9}, {0x118ff, 0x11906}, {0x11909, 0x11909}, {0x1190c, 0x11913}, {0x11915, 0x11916}, {0x11918, 0x11935}, {0x11937, 0x11938}, {0x1193b, 0x11943},
  {0x11950, 0x11959}, {0x119a0, 0x119a7}, {0x119aa, 0x119d7}, {0x119da, 0x119e1}, {0x119e3, 0x119e4}, {0x11a00, 0x11a3e}, {0x11a47, 0x11a47}, {0x11a50, 0x11a99},
  {0x11a9d, 0x11a9d}, {0x11ab0, 0x11af8}, {0x11c00, 0x11c08}, {0x11c0a, 0x11c36}, {0x11c38, 0x11c40}, {0x11c50, 0x11c59}, {0x11c72, 0x11c8f}, {0x11c92, 0x11ca7},
  {0x11ca9, 0x11cb6}, {0x11d00, 0x11d06}, {0x11d08, 0x11d09}, {0x11d0b, 0x11d36}, {0x11d3a, 0x11d3a}, {0x11d3c, 0x11d3d}, {0x11d3f, 0x11d47}, {0x11d50, 0x11d59},
  {0x11d60, 0x11d65}, {0x11d67, 0x11d68}, {0x11d6a, 0x11d8e}, {0x11d90, 0x11d91}, {0x11d93, 0x11d98}, {0x11da0, 0x11da9}, {0x11ee0, 0x11ef6}, {0x11f00, 0x11f10},
  {0x11f12, 0x11f3a}, {0x11f3e, 0x11f42}, {0x11f50, 0x11f59}, {0x11fb0, 0x11fb0}, {0x12000, 0x12399}, {0x12400, 0x1246e}, {0x12480, 0x12543}, {0x12f90, 0x12ff0},
  {0x13000, 0x1342f}, {0x13440, 0x13455}, {0x14400, 0x14646}, {0x16800, 0x16a38}, {0x16a40, 0x16a5e}, {0x16a60, 0x16a69}, {0x16a70, 0x16abe}, {0x16ac0, 0x16ac9},
  {0x16ad0, 0x16aed}, {0x16af0, 0x16af4}, {0x16b00, 0x16b36}, {0x16b40, 0x16b43}, {0x16b50, 0x16b59}, {0x16b63, 0x16b77}, {0x16b7d, 0x16b8f}, {0x16e40, 0x16e7f},
  {0x16f00, 0x16f4a}, {0x16f4f, 0x16f87}, {0x16f8f, 0x16f9f}, {0x16fe0, 0x16fe1}, {0x16fe3, 0x16fe4}, {0x16ff0, 0x16ff1}, {0x17000, 0x187f7}, {0x18800, 0x18cd5},
  {0x18d00, 0x18d08}, {0x1aff0, 0x1aff3}, {0x1aff5, 0x1affb}, {0x1affd, 0x1affe}, {0x1b000, 0x1b122}, {0x1b132, 0x1b132}, {0x1b150, 0x1b152}, {0x1b155, 0x1b155},
  {0x1b164, 0x1b167}, {0x1b170, 0x1b2fb}, {0x1bc00, 0x1bc6a}, {0x1bc70, 0x1bc7c}, {0x1bc80, 0x1bc88}, {0x1bc90, 0x1bc99}, {0x1bc9d, 0x1bc9e}, {0x1cf00, 0x1cf2d},
  {0x1cf30, 0x1cf46}, {0x1d165, 0x1d169}, {0x1d16d, 0x1d172}, {0x1d17b, 0x1d182}, {0x1d185, 0x1d18b}, {0x1d1aa, 0x1d1ad}, {0x1d242, 0x1d244}, {0x1d400, 0x1d454},
  {0x1d456, 0x1d49c}, {0x1d49e, 0x1d49f}, {0x1d4a2, 0x1d4a2}, {0x1d4a5, 0x1d4a6}, {0x1d4a9, 0x1d4ac}, {0x1d4ae, 0x1d4b9}, {0x1d4bb, 0x1d4bb}, {0x1d4bd, 0x1d4c3},
  {0x1d4c5, 0x1d505}, {0x1d507, 0x1d50a}, {0x1d50d, 0x1d514}, {0x1d516, 0x1d51c}, {0x1d51e, 0x1d539}, {0x1d53b, 0x1d53e}, {0x1d540, 0x1d544}, {0x1d546, 0x1d546},
  {0x1d54a, 0x1d550}, {0x1d552, 0x1d6a5}, {0x1d6a8, 0x1d6c0}, {0x1d6c2, 0x1d6da}, {0x1d6dc, 0x1d6fa}, {0x1d6fc, 0x1d714}, {0x1d716, 0x1d734}, {0x1d736, 0x1d74e},
  {0x1d750, 0x1d76e}, {0x1d770, 0x1d788}, {0x1d78a, 0x1d7a8}, {0x1d7aa, 0x1d7c2}, {0x1d7c4, 0x1d7cb}, {0x1d7ce, 0x1d7ff}, {0x1da00, 0x1da36}, {0x1da3b, 0x1da6c},
  {0x1da75, 0x1da75}, {0x1da84, 0x1da84}, {0x1da9b, 0x1da9f}, {0x1daa1, 0x1daaf}, {0x1df00, 0x1df1e}, {0x1df25, 0x1df2a}, {0x1e000, 0x1e006}, {0x1e008, 0x1e018},
  {0x1e01b, 0x1e021}, {0x1e023, 0x1e024}, {0x1e026, 0x1e02a}, {0x1e030, 0x1e06d}, {0x1e08f, 0x1e08f}, {0x1e100, 0x1e12c}, {0x1e130, 0x1e13d}, {0x1e140, 0x1e149},
  {0x1e14e, 0x1e14e}, {0x1e290, 0x1e2ae}, {0x1e2c0, 0x1e2f9}, {0x1e4d0, 0x1e4f9}, {0x1e7e0, 0x1e7e6}, {0x1e7e8, 0x1e7eb}, {0x1e7ed, 0x1e7ee}, {0x1e7f0, 0x1e7fe},
  {0x1e800, 0x1e8c4}, {0x1e8d0, 0x1e8d6}, {0x1e900, 0x1e94b}, {0x1e950, 0x1e959}, {0x1ee00, 0x1ee03}, {0x1ee05, 0x1ee1f}, {0x1ee21, 0x1ee22}, {0x1ee24, 0x1ee24},
  {0x1ee27, 0x1ee27}, {0x1ee29, 0x1ee32}, {0x1ee34, 0x1ee37}, {0x1ee39, 0x1ee39}, {0x1ee3b, 0x1ee3b}, {0x1ee42, 0x1ee42}, {0x1ee47, 0x1ee47}, {0x1ee49, 0x1ee49},
  {0x1ee4b, 0x1ee4b}, {0x1ee4d, 0x1ee4f}, {0x1ee51, 0x1ee52}, {0x1ee54, 0x1ee54}, {0x1ee57, 0x1ee57}, {0x1ee59, 0x1ee59}, {0x1ee5b, 0x1ee5b}, {0x1ee5d, 0x1ee5d},
  {0x1ee5f, 0x1ee5f}, {0x1ee61, 0x1ee62}, {0x1ee64, 0x1ee64}, {0x1ee67, 0x1ee6a}, {0x1ee6c, 0x1ee72}, {0x1ee74, 0x1ee77}, {0x1ee79, 0x1ee7c}, {0x1ee7e, 0x1ee7e},
  {0x1ee80, 0x1ee89}, {0x1ee8b, 0x1ee9b}, {0x1eea1, 0x1eea3}, {0x1eea5, 0x1eea9}, {0x1eeab, 0x1eebb}, {0x1fbf0, 0x1fbf9}, {0x20000, 0x2a6df}, {0x2a700, 0x2b739},
  {0x2b740, 0x2b81d}, {0x2b820, 0x2cea1}, {0x2ceb0, 0x2ebe0}, {0x2ebf0, 0x2ee5d}, {0x2f800, 0x2fa1d}, {0x30000, 0x3134a}, {0x31350, 0x323af}, {0xe0100, 0xe01ef},
};

static TSCharacterRange sym_rune_literal_character_set_1[] = {
  {'"', '"'}, {'\'', '\''}, {'0', '7'}, {'U', 'U'}, {'\\', '\\'}, {'a', 'b'}, {'f', 'f'}, {'n', 'n'},
  {'r', 'r'}, {'t', 'v'}, {'x', 'x'},
};

static bool ts_lex(TSLexer *lexer, TSStateId state) {
  START_LEXER();
  eof = lexer->eof(lexer);
  switch (state) {
    case 0:
      if (eof) ADVANCE(59);
      ADVANCE_MAP(
        0, 62,
        '!', 106,
        '"', 148,
        '%', 115,
        '&', 110,
        '\'', 22,
        '(', 66,
        ')', 67,
        '*', 75,
        '+', 100,
        ',', 68,
        '-', 103,
        '.', 65,
        '/', 113,
        '0', 152,
        ':', 98,
        ';', 61,
        '<', 124,
        '=', 70,
        '>', 129,
        '[', 71,
        '\\', 20,
        ']', 72,
        '^', 108,
        '`', 135,
        '{', 77,
        '|', 80,
        '}', 78,
        '~', 76,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(57);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(154);
      if (set_contains(sym_identifier_character_set_1, 668, lookahead)) ADVANCE(134);
      END_STATE();
    case 1:
      ADVANCE_MAP(
        0, 62,
        '\n', 60,
        '!', 17,
        '%', 114,
        '&', 111,
        '(', 66,
        '*', 74,
        '+', 99,
        ',', 68,
        '-', 102,
        '.', 63,
        '/', 112,
        ';', 61,
        '<', 127,
        '=', 19,
        '>', 130,
        '[', 71,
        '^', 107,
        '{', 77,
        '|', 81,
        '}', 78,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(1);
      if (set_contains(sym_identifier_character_set_1, 668, lookahead)) ADVANCE(134);
      END_STATE();
    case 2:
      if (lookahead == '\n') SKIP(16);
      if (lookahead == '"') ADVANCE(148);
      if (lookahead == '/') ADVANCE(143);
      if (lookahead == '\\') ADVANCE(20);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(146);
      if (lookahead != 0) ADVANCE(147);
      END_STATE();
    case 3:
      ADVANCE_MAP(
        '!', 17,
        '"', 142,
        '%', 114,
        '&', 111,
        '(', 66,
        ')', 67,
        '*', 74,
        '+', 99,
        ',', 68,
        '-', 102,
        '.', 64,
        '/', 112,
        ':', 98,
        '<', 125,
        '=', 70,
        '>', 130,
        '[', 71,
        ']', 72,
        '^', 107,
        '`', 135,
        '{', 77,
        '|', 81,
        '~', 76,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(3);
      if (set_contains(sym_identifier_character_set_1, 668, lookahead)) ADVANCE(134);
      END_STATE();
    case 4:
      ADVANCE_MAP(
        '!', 17,
        '%', 115,
        '&', 110,
        '(', 66,
        '*', 75,
        '+', 100,
        ',', 68,
        '-', 103,
        '.', 63,
        '/', 113,
        ':', 18,
        ';', 61,
        '<', 124,
        '=', 70,
        '>', 129,
        '[', 71,
        '^', 108,
        '{', 77,
        '|', 80,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(4);
      END_STATE();
    case 5:
      ADVANCE_MAP(
        '!', 17,
        '%', 115,
        '&', 110,
        '(', 66,
        '*', 75,
        '+', 101,
        ',', 68,
        '-', 104,
        '.', 63,
        '/', 113,
        ':', 18,
        '<', 126,
        '=', 70,
        '>', 129,
        '[', 71,
        '^', 108,
        '{', 77,
        '|', 80,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(5);
      END_STATE();
    case 6:
      ADVANCE_MAP(
        '!', 17,
        '%', 114,
        '&', 111,
        '(', 66,
        ')', 67,
        '*', 74,
        '+', 99,
        ',', 68,
        '-', 102,
        '.', 64,
        '/', 112,
        ':', 98,
        ';', 61,
        '<', 127,
        '=', 70,
        '>', 130,
        '[', 71,
        ']', 72,
        '^', 107,
        '{', 77,
        '|', 81,
        '}', 78,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(6);
      END_STATE();
    case 7:
      if (lookahead == '\'') ADVANCE(162);
      END_STATE();
    case 8:
      if (lookahead == '*') ADVANCE(10);
      if (lookahead == '/') ADVANCE(164);
      END_STATE();
    case 9:
      if (lookahead == '*') ADVANCE(9);
      if (lookahead == '/') ADVANCE(163);
      if (lookahead != 0) ADVANCE(10);
      END_STATE();
    case 10:
      if (lookahead == '*') ADVANCE(9);
      if (lookahead != 0) ADVANCE(10);
      END_STATE();
    case 11:
      if (lookahead == '-') ADVANCE(82);
      END_STATE();
    case 12:
      if (lookahead == '.') ADVANCE(14);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(158);
      END_STATE();
    case 13:
      if (lookahead == '.') ADVANCE(160);
      if (lookahead == '_') ADVANCE(35);
      if (lookahead == 'i') ADVANCE(161);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(26);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(13);
      END_STATE();
    case 14:
      if (lookahead == '.') ADVANCE(73);
      END_STATE();
    case 15:
      if (lookahead == '.') ADVANCE(40);
      if (lookahead == '_') ADVANCE(39);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(155);
      END_STATE();
    case 16:
      if (lookahead == '/') ADVANCE(8);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(16);
      END_STATE();
    case 17:
      if (lookahead == '=') ADVANCE(123);
      END_STATE();
    case 18:
      if (lookahead == '=') ADVANCE(83);
      END_STATE();
    case 19:
      if (lookahead == '=') ADVANCE(122);
      END_STATE();
    case 20:
      if (lookahead == 'U') ADVANCE(53);
      if (lookahead == 'u') ADVANCE(45);
      if (lookahead == 'x') ADVANCE(41);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(151);
      if (lookahead != 0) ADVANCE(149);
      END_STATE();
    case 21:
      if (lookahead == 'U') ADVANCE(54);
      if (lookahead == 'u') ADVANCE(46);
      if (lookahead == 'x') ADVANCE(42);
      if (('0' <= lookahead && lookahead <= '7')) ADVANCE(32);
      if (set_contains(sym_rune_literal_character_set_1, 11, lookahead)) ADVANCE(7);
      END_STATE();
    case 22:
      if (lookahead == '\\') ADVANCE(21);
      if (lookahead != 0 &&
          lookahead != '\'') ADVANCE(7);
      END_STATE();
    case 23:
      if (lookahead == '_') ADVANCE(28);
      if (lookahead == '0' ||
          lookahead == '1') ADVANCE(156);
      END_STATE();
    case 24:
      if (lookahead == '_') ADVANCE(40);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(26);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(24);
      END_STATE();
    case 25:
      if (lookahead == '_') ADVANCE(31);
      if (('0' <= lookahead && lookahead <= '7')) ADVANCE(157);
      END_STATE();
    case 26:
      if (lookahead == '+' ||
          lookahead == '-') ADVANCE(36);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(159);
      END_STATE();
    case 27:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(26);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(24);
      END_STATE();
    case 28:
      if (lookahead == '0' ||
          lookahead == '1') ADVANCE(156);
      END_STATE();
    case 29:
      if (lookahead == '8' ||
          lookahead == '9') ADVANCE(13);
      if (('0' <= lookahead && lookahead <= '7')) ADVANCE(153);
      END_STATE();
    case 30:
      if (('0' <= lookahead && lookahead <= '7')) ADVANCE(7);
      END_STATE();
    case 31:
      if (('0' <= lookahead && lookahead <= '7')) ADVANCE(157);
      END_STATE();
    case 32:
      if (('0' <= lookahead && lookahead <= '7')) ADVANCE(30);
      END_STATE();
    case 33:
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(154);
      END_STATE();
    case 34:
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(158);
      END_STATE();
    case 35:
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(13);
      END_STATE();
    case 36:
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(159);
      END_STATE();
    case 37:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(7);
      END_STATE();
    case 38:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(149);
      END_STATE();
    case 39:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(155);
      END_STATE();
    case 40:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(24);
      END_STATE();
    case 41:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(38);
      END_STATE();
    case 42:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(37);
      END_STATE();
    case 43:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(41);
      END_STATE();
    case 44:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(42);
      END_STATE();
    case 45:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(43);
      END_STATE();
    case 46:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(44);
      END_STATE();
    case 47:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(45);
      END_STATE();
    case 48:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(46);
      END_STATE();
    case 49:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(47);
      END_STATE();
    case 50:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(48);
      END_STATE();
    case 51:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(49);
      END_STATE();
    case 52:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(50);
      END_STATE();
    case 53:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(51);
      END_STATE();
    case 54:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(52);
      END_STATE();
    case 55:
      if (eof) ADVANCE(59);
      ADVANCE_MAP(
        0, 62,
        '\n', 60,
        '!', 105,
        '"', 142,
        '&', 109,
        '\'', 22,
        '(', 66,
        ')', 67,
        '*', 74,
        '+', 99,
        ',', 68,
        '-', 102,
        '.', 34,
        '/', 8,
        '0', 152,
        ';', 61,
        '<', 11,
        '=', 69,
        '[', 71,
        '^', 107,
        '`', 135,
        '{', 77,
        '|', 79,
        '}', 78,
        '~', 76,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(55);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(154);
      if (set_contains(sym_identifier_character_set_1, 668, lookahead)) ADVANCE(134);
      END_STATE();
    case 56:
      if (eof) ADVANCE(59);
      ADVANCE_MAP(
        0, 62,
        '\n', 60,
        '!', 17,
        '"', 142,
        '%', 115,
        '&', 110,
        '(', 66,
        ')', 67,
        '*', 75,
        '+', 100,
        ',', 68,
        '-', 103,
        '.', 63,
        '/', 113,
        ':', 98,
        ';', 61,
        '<', 124,
        '=', 70,
        '>', 129,
        '[', 71,
        '^', 108,
        '`', 135,
        '{', 77,
        '|', 80,
        '}', 78,
        '~', 76,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(56);
      if (set_contains(sym_identifier_character_set_1, 668, lookahead)) ADVANCE(134);
      END_STATE();
    case 57:
      if (eof) ADVANCE(59);
      ADVANCE_MAP(
        0, 62,
        '!', 106,
        '"', 142,
        '%', 115,
        '&', 110,
        '\'', 22,
        '(', 66,
        ')', 67,
        '*', 75,
        '+', 100,
        ',', 68,
        '-', 103,
        '.', 65,
        '/', 113,
        '0', 152,
        ':', 98,
        ';', 61,
        '<', 124,
        '=', 70,
        '>', 129,
        '[', 71,
        ']', 72,
        '^', 108,
        '`', 135,
        '{', 77,
        '|', 80,
        '}', 78,
        '~', 76,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(57);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(154);
      if (set_contains(sym_identifier_character_set_1, 668, lookahead)) ADVANCE(134);
      END_STATE();
    case 58:
      if (eof) ADVANCE(59);
      ADVANCE_MAP(
        '!', 105,
        '"', 142,
        '&', 109,
        '\'', 22,
        '(', 66,
        ')', 67,
        '*', 74,
        '+', 99,
        ',', 68,
        '-', 102,
        '.', 12,
        '/', 8,
        '0', 152,
        ':', 97,
        ';', 61,
        '<', 11,
        '[', 71,
        ']', 72,
        '^', 107,
        '`', 135,
        '{', 77,
        '}', 78,
        '~', 76,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(58);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(154);
      if (set_contains(sym_identifier_character_set_1, 668, lookahead)) ADVANCE(134);
      END_STATE();
    case 59:
      ACCEPT_TOKEN(ts_builtin_sym_end);
      END_STATE();
    case 60:
      ACCEPT_TOKEN(aux_sym_source_file_token1);
      if (lookahead == '\n') ADVANCE(60);
      END_STATE();
    case 61:
      ACCEPT_TOKEN(anon_sym_SEMI);
      END_STATE();
    case 62:
      ACCEPT_TOKEN(anon_sym_NULL);
      END_STATE();
    case 63:
      ACCEPT_TOKEN(anon_sym_DOT);
      END_STATE();
    case 64:
      ACCEPT_TOKEN(anon_sym_DOT);
      if (lookahead == '.') ADVANCE(14);
      END_STATE();
    case 65:
      ACCEPT_TOKEN(anon_sym_DOT);
      if (lookahead == '.') ADVANCE(14);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(158);
      END_STATE();
    case 66:
      ACCEPT_TOKEN(anon_sym_LPAREN);
      END_STATE();
    case 67:
      ACCEPT_TOKEN(anon_sym_RPAREN);
      END_STATE();
    case 68:
      ACCEPT_TOKEN(anon_sym_COMMA);
      END_STATE();
    case 69:
      ACCEPT_TOKEN(anon_sym_EQ);
      END_STATE();
    case 70:
      ACCEPT_TOKEN(anon_sym_EQ);
      if (lookahead == '=') ADVANCE(122);
      END_STATE();
    case 71:
      ACCEPT_TOKEN(anon_sym_LBRACK);
      END_STATE();
    case 72:
      ACCEPT_TOKEN(anon_sym_RBRACK);
      END_STATE();
    case 73:
      ACCEPT_TOKEN(anon_sym_DOT_DOT_DOT);
      END_STATE();
    case 74:
      ACCEPT_TOKEN(anon_sym_STAR);
      END_STATE();
    case 75:
      ACCEPT_TOKEN(anon_sym_STAR);
      if (lookahead == '=') ADVANCE(86);
      END_STATE();
    case 76:
      ACCEPT_TOKEN(anon_sym_TILDE);
      END_STATE();
    case 77:
      ACCEPT_TOKEN(anon_sym_LBRACE);
      END_STATE();
    case 78:
      ACCEPT_TOKEN(anon_sym_RBRACE);
      END_STATE();
    case 79:
      ACCEPT_TOKEN(anon_sym_PIPE);
      END_STATE();
    case 80:
      ACCEPT_TOKEN(anon_sym_PIPE);
      if (lookahead == '=') ADVANCE(95);
      if (lookahead == '|') ADVANCE(133);
      END_STATE();
    case 81:
      ACCEPT_TOKEN(anon_sym_PIPE);
      if (lookahead == '|') ADVANCE(133);
      END_STATE();
    case 82:
      ACCEPT_TOKEN(anon_sym_LT_DASH);
      END_STATE();
    case 83:
      ACCEPT_TOKEN(anon_sym_COLON_EQ);
      END_STATE();
    case 84:
      ACCEPT_TOKEN(anon_sym_PLUS_PLUS);
      END_STATE();
    case 85:
      ACCEPT_TOKEN(anon_sym_DASH_DASH);
      END_STATE();
    case 86:
      ACCEPT_TOKEN(anon_sym_STAR_EQ);
      END_STATE();
    case 87:
      ACCEPT_TOKEN(anon_sym_SLASH_EQ);
      END_STATE();
    case 88:
      ACCEPT_TOKEN(anon_sym_PERCENT_EQ);
      END_STATE();
    case 89:
      ACCEPT_TOKEN(anon_sym_LT_LT_EQ);
      END_STATE();
    case 90:
      ACCEPT_TOKEN(anon_sym_GT_GT_EQ);
      END_STATE();
    case 91:
      ACCEPT_TOKEN(anon_sym_AMP_EQ);
      END_STATE();
    case 92:
      ACCEPT_TOKEN(anon_sym_AMP_CARET_EQ);
      END_STATE();
    case 93:
      ACCEPT_TOKEN(anon_sym_PLUS_EQ);
      END_STATE();
    case 94:
      ACCEPT_TOKEN(anon_sym_DASH_EQ);
      END_STATE();
    case 95:
      ACCEPT_TOKEN(anon_sym_PIPE_EQ);
      END_STATE();
    case 96:
      ACCEPT_TOKEN(anon_sym_CARET_EQ);
      END_STATE();
    case 97:
      ACCEPT_TOKEN(anon_sym_COLON);
      END_STATE();
    case 98:
      ACCEPT_TOKEN(anon_sym_COLON);
      if (lookahead == '=') ADVANCE(83);
      END_STATE();
    case 99:
      ACCEPT_TOKEN(anon_sym_PLUS);
      END_STATE();
    case 100:
      ACCEPT_TOKEN(anon_sym_PLUS);
      if (lookahead == '+') ADVANCE(84);
      if (lookahead == '=') ADVANCE(93);
      END_STATE();
    case 101:
      ACCEPT_TOKEN(anon_sym_PLUS);
      if (lookahead == '=') ADVANCE(93);
      END_STATE();
    case 102:
      ACCEPT_TOKEN(anon_sym_DASH);
      END_STATE();
    case 103:
      ACCEPT_TOKEN(anon_sym_DASH);
      if (lookahead == '-') ADVANCE(85);
      if (lookahead == '=') ADVANCE(94);
      END_STATE();
    case 104:
      ACCEPT_TOKEN(anon_sym_DASH);
      if (lookahead == '=') ADVANCE(94);
      END_STATE();
    case 105:
      ACCEPT_TOKEN(anon_sym_BANG);
      END_STATE();
    case 106:
      ACCEPT_TOKEN(anon_sym_BANG);
      if (lookahead == '=') ADVANCE(123);
      END_STATE();
    case 107:
      ACCEPT_TOKEN(anon_sym_CARET);
      END_STATE();
    case 108:
      ACCEPT_TOKEN(anon_sym_CARET);
      if (lookahead == '=') ADVANCE(96);
      END_STATE();
    case 109:
      ACCEPT_TOKEN(anon_sym_AMP);
      END_STATE();
    case 110:
      ACCEPT_TOKEN(anon_sym_AMP);
      if (lookahead == '&') ADVANCE(132);
      if (lookahead == '=') ADVANCE(91);
      if (lookahead == '^') ADVANCE(121);
      END_STATE();
    case 111:
      ACCEPT_TOKEN(anon_sym_AMP);
      if (lookahead == '&') ADVANCE(132);
      if (lookahead == '^') ADVANCE(120);
      END_STATE();
    case 112:
      ACCEPT_TOKEN(anon_sym_SLASH);
      if (lookahead == '*') ADVANCE(10);
      if (lookahead == '/') ADVANCE(164);
      END_STATE();
    case 113:
      ACCEPT_TOKEN(anon_sym_SLASH);
      if (lookahead == '*') ADVANCE(10);
      if (lookahead == '/') ADVANCE(164);
      if (lookahead == '=') ADVANCE(87);
      END_STATE();
    case 114:
      ACCEPT_TOKEN(anon_sym_PERCENT);
      END_STATE();
    case 115:
      ACCEPT_TOKEN(anon_sym_PERCENT);
      if (lookahead == '=') ADVANCE(88);
      END_STATE();
    case 116:
      ACCEPT_TOKEN(anon_sym_LT_LT);
      END_STATE();
    case 117:
      ACCEPT_TOKEN(anon_sym_LT_LT);
      if (lookahead == '=') ADVANCE(89);
      END_STATE();
    case 118:
      ACCEPT_TOKEN(anon_sym_GT_GT);
      END_STATE();
    case 119:
      ACCEPT_TOKEN(anon_sym_GT_GT);
      if (lookahead == '=') ADVANCE(90);
      END_STATE();
    case 120:
      ACCEPT_TOKEN(anon_sym_AMP_CARET);
      END_STATE();
    case 121:
      ACCEPT_TOKEN(anon_sym_AMP_CARET);
      if (lookahead == '=') ADVANCE(92);
      END_STATE();
    case 122:
      ACCEPT_TOKEN(anon_sym_EQ_EQ);
      END_STATE();
    case 123:
      ACCEPT_TOKEN(anon_sym_BANG_EQ);
      END_STATE();
    case 124:
      ACCEPT_TOKEN(anon_sym_LT);
      if (lookahead == '-') ADVANCE(82);
      if (lookahead == '<') ADVANCE(117);
      if (lookahead == '=') ADVANCE(128);
      END_STATE();
    case 125:
      ACCEPT_TOKEN(anon_sym_LT);
      if (lookahead == '-') ADVANCE(82);
      if (lookahead == '<') ADVANCE(116);
      if (lookahead == '=') ADVANCE(128);
      END_STATE();
    case 126:
      ACCEPT_TOKEN(anon_sym_LT);
      if (lookahead == '<') ADVANCE(117);
      if (lookahead == '=') ADVANCE(128);
      END_STATE();
    case 127:
      ACCEPT_TOKEN(anon_sym_LT);
      if (lookahead == '<') ADVANCE(116);
      if (lookahead == '=') ADVANCE(128);
      END_STATE();
    case 128:
      ACCEPT_TOKEN(anon_sym_LT_EQ);
      END_STATE();
    case 129:
      ACCEPT_TOKEN(anon_sym_GT);
      if (lookahead == '=') ADVANCE(131);
      if (lookahead == '>') ADVANCE(119);
      END_STATE();
    case 130:
      ACCEPT_TOKEN(anon_sym_GT);
      if (lookahead == '=') ADVANCE(131);
      if (lookahead == '>') ADVANCE(118);
      END_STATE();
    case 131:
      ACCEPT_TOKEN(anon_sym_GT_EQ);
      END_STATE();
    case 132:
      ACCEPT_TOKEN(anon_sym_AMP_AMP);
      END_STATE();
    case 133:
      ACCEPT_TOKEN(anon_sym_PIPE_PIPE);
      END_STATE();
    case 134:
      ACCEPT_TOKEN(sym_identifier);
      if (set_contains(sym_identifier_character_set_2, 776, lookahead)) ADVANCE(134);
      END_STATE();
    case 135:
      ACCEPT_TOKEN(anon_sym_BQUOTE);
      END_STATE();
    case 136:
      ACCEPT_TOKEN(aux_sym_raw_string_literal_token1);
      if (lookahead == '\n') ADVANCE(141);
      if (lookahead != 0 &&
          lookahead != '`') ADVANCE(136);
      END_STATE();
    case 137:
      ACCEPT_TOKEN(aux_sym_raw_string_literal_token1);
      if (lookahead == '*') ADVANCE(139);
      if (lookahead == '/') ADVANCE(136);
      if (lookahead != 0 &&
          lookahead != '`') ADVANCE(141);
      END_STATE();
    case 138:
      ACCEPT_TOKEN(aux_sym_raw_string_literal_token1);
      if (lookahead == '*') ADVANCE(138);
      if (lookahead == '/') ADVANCE(141);
      if (lookahead != 0 &&
          lookahead != '`') ADVANCE(139);
      END_STATE();
    case 139:
      ACCEPT_TOKEN(aux_sym_raw_string_literal_token1);
      if (lookahead == '*') ADVANCE(138);
      if (lookahead != 0 &&
          lookahead != '`') ADVANCE(139);
      END_STATE();
    case 140:
      ACCEPT_TOKEN(aux_sym_raw_string_literal_token1);
      if (lookahead == '/') ADVANCE(137);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(140);
      if (lookahead != 0 &&
          lookahead != '`') ADVANCE(141);
      END_STATE();
    case 141:
      ACCEPT_TOKEN(aux_sym_raw_string_literal_token1);
      if (lookahead != 0 &&
          lookahead != '`') ADVANCE(141);
      END_STATE();
    case 142:
      ACCEPT_TOKEN(anon_sym_DQUOTE);
      END_STATE();
    case 143:
      ACCEPT_TOKEN(aux_sym_interpreted_string_literal_token1);
      if (lookahead == '*') ADVANCE(145);
      if (lookahead == '/') ADVANCE(147);
      if (lookahead != 0 &&
          lookahead != '\n' &&
          lookahead != '"' &&
          lookahead != '\\') ADVANCE(147);
      END_STATE();
    case 144:
      ACCEPT_TOKEN(aux_sym_interpreted_string_literal_token1);
      if (lookahead == '*') ADVANCE(144);
      if (lookahead == '/') ADVANCE(147);
      if (lookahead != 0 &&
          lookahead != '\n' &&
          lookahead != '"' &&
          lookahead != '\\') ADVANCE(145);
      END_STATE();
    case 145:
      ACCEPT_TOKEN(aux_sym_interpreted_string_literal_token1);
      if (lookahead == '*') ADVANCE(144);
      if (lookahead != 0 &&
          lookahead != '\n' &&
          lookahead != '"' &&
          lookahead != '\\') ADVANCE(145);
      END_STATE();
    case 146:
      ACCEPT_TOKEN(aux_sym_interpreted_string_literal_token1);
      if (lookahead == '/') ADVANCE(143);
      if (lookahead == '\t' ||
          (0x0b <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(146);
      if (lookahead != 0 &&
          (lookahead < '\t' || '\r' < lookahead) &&
          lookahead != '"' &&
          lookahead != '\\') ADVANCE(147);
      END_STATE();
    case 147:
      ACCEPT_TOKEN(aux_sym_interpreted_string_literal_token1);
      if (lookahead != 0 &&
          lookahead != '\n' &&
          lookahead != '"' &&
          lookahead != '\\') ADVANCE(147);
      END_STATE();
    case 148:
      ACCEPT_TOKEN(anon_sym_DQUOTE2);
      END_STATE();
    case 149:
      ACCEPT_TOKEN(sym_escape_sequence);
      END_STATE();
    case 150:
      ACCEPT_TOKEN(sym_escape_sequence);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(149);
      END_STATE();
    case 151:
      ACCEPT_TOKEN(sym_escape_sequence);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(150);
      END_STATE();
    case 152:
      ACCEPT_TOKEN(sym_int_literal);
      ADVANCE_MAP(
        '.', 160,
        '_', 29,
        'i', 161,
        'B', 23,
        'b', 23,
        'E', 26,
        'e', 26,
        'O', 25,
        'o', 25,
        'X', 15,
        'x', 15,
        '8', 13,
        '9', 13,
      );
      if (('0' <= lookahead && lookahead <= '7')) ADVANCE(153);
      END_STATE();
    case 153:
      ACCEPT_TOKEN(sym_int_literal);
      if (lookahead == '.') ADVANCE(160);
      if (lookahead == '_') ADVANCE(29);
      if (lookahead == 'i') ADVANCE(161);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(26);
      if (lookahead == '8' ||
          lookahead == '9') ADVANCE(13);
      if (('0' <= lookahead && lookahead <= '7')) ADVANCE(153);
      END_STATE();
    case 154:
      ACCEPT_TOKEN(sym_int_literal);
      if (lookahead == '.') ADVANCE(160);
      if (lookahead == '_') ADVANCE(33);
      if (lookahead == 'i') ADVANCE(161);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(26);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(154);
      END_STATE();
    case 155:
      ACCEPT_TOKEN(sym_int_literal);
      if (lookahead == '.') ADVANCE(27);
      if (lookahead == '_') ADVANCE(39);
      if (lookahead == 'i') ADVANCE(161);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(26);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(155);
      END_STATE();
    case 156:
      ACCEPT_TOKEN(sym_int_literal);
      if (lookahead == '_') ADVANCE(28);
      if (lookahead == 'i') ADVANCE(161);
      if (lookahead == '0' ||
          lookahead == '1') ADVANCE(156);
      END_STATE();
    case 157:
      ACCEPT_TOKEN(sym_int_literal);
      if (lookahead == '_') ADVANCE(31);
      if (lookahead == 'i') ADVANCE(161);
      if (('0' <= lookahead && lookahead <= '7')) ADVANCE(157);
      END_STATE();
    case 158:
      ACCEPT_TOKEN(sym_float_literal);
      if (lookahead == '_') ADVANCE(34);
      if (lookahead == 'i') ADVANCE(161);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(26);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(158);
      END_STATE();
    case 159:
      ACCEPT_TOKEN(sym_float_literal);
      if (lookahead == '_') ADVANCE(36);
      if (lookahead == 'i') ADVANCE(161);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(159);
      END_STATE();
    case 160:
      ACCEPT_TOKEN(sym_float_literal);
      if (lookahead == 'i') ADVANCE(161);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(26);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(158);
      END_STATE();
    case 161:
      ACCEPT_TOKEN(sym_imaginary_literal);
      END_STATE();
    case 162:
      ACCEPT_TOKEN(sym_rune_literal);
      END_STATE();
    case 163:
      ACCEPT_TOKEN(sym_comment);
      END_STATE();
    case 164:
      ACCEPT_TOKEN(sym_comment);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(164);
      END_STATE();
    default:
      return false;
  }
}

static bool ts_lex_keywords(TSLexer *lexer, TSStateId state) {
  START_LEXER();
  eof = lexer->eof(lexer);
  switch (state) {
    case 0:
      ADVANCE_MAP(
        '_', 1,
        'b', 2,
        'c', 3,
        'd', 4,
        'e', 5,
        'f', 6,
        'g', 7,
        'i', 8,
        'm', 9,
        'n', 10,
        'p', 11,
        'r', 12,
        's', 13,
        't', 14,
        'v', 15,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(0);
      END_STATE();
    case 1:
      ACCEPT_TOKEN(sym_blank_identifier);
      END_STATE();
    case 2:
      if (lookahead == 'r') ADVANCE(16);
      END_STATE();
    case 3:
      if (lookahead == 'a') ADVANCE(17);
      if (lookahead == 'h') ADVANCE(18);
      if (lookahead == 'o') ADVANCE(19);
      END_STATE();
    case 4:
      if (lookahead == 'e') ADVANCE(20);
      END_STATE();
    case 5:
      if (lookahead == 'l') ADVANCE(21);
      END_STATE();
    case 6:
      if (lookahead == 'a') ADVANCE(22);
      if (lookahead == 'o') ADVANCE(23);
      if (lookahead == 'u') ADVANCE(24);
      END_STATE();
    case 7:
      if (lookahead == 'o') ADVANCE(25);
      END_STATE();
    case 8:
      if (lookahead == 'f') ADVANCE(26);
      if (lookahead == 'm') ADVANCE(27);
      if (lookahead == 'n') ADVANCE(28);
      if (lookahead == 'o') ADVANCE(29);
      END_STATE();
    case 9:
      if (lookahead == 'a') ADVANCE(30);
      END_STATE();
    case 10:
      if (lookahead == 'e') ADVANCE(31);
      if (lookahead == 'i') ADVANCE(32);
      END_STATE();
    case 11:
      if (lookahead == 'a') ADVANCE(33);
      END_STATE();
    case 12:
      if (lookahead == 'a') ADVANCE(34);
      if (lookahead == 'e') ADVANCE(35);
      END_STATE();
    case 13:
      if (lookahead == 'e') ADVANCE(36);
      if (lookahead == 't') ADVANCE(37);
      if (lookahead == 'w') ADVANCE(38);
      END_STATE();
    case 14:
      if (lookahead == 'r') ADVANCE(39);
      if (lookahead == 'y') ADVANCE(40);
      END_STATE();
    case 15:
      if (lookahead == 'a') ADVANCE(41);
      END_STATE();
    case 16:
      if (lookahead == 'e') ADVANCE(42);
      END_STATE();
    case 17:
      if (lookahead == 's') ADVANCE(43);
      END_STATE();
    case 18:
      if (lookahead == 'a') ADVANCE(44);
      END_STATE();
    case 19:
      if (lookahead == 'n') ADVANCE(45);
      END_STATE();
    case 20:
      if (lookahead == 'f') ADVANCE(46);
      END_STATE();
    case 21:
      if (lookahead == 's') ADVANCE(47);
      END_STATE();
    case 22:
      if (lookahead == 'l') ADVANCE(48);
      END_STATE();
    case 23:
      if (lookahead == 'r') ADVANCE(49);
      END_STATE();
    case 24:
      if (lookahead == 'n') ADVANCE(50);
      END_STATE();
    case 25:
      ACCEPT_TOKEN(anon_sym_go);
      if (lookahead == 't') ADVANCE(51);
      END_STATE();
    case 26:
      ACCEPT_TOKEN(anon_sym_if);
      END_STATE();
    case 27:
      if (lookahead == 'p') ADVANCE(52);
      END_STATE();
    case 28:
      if (lookahead == 't') ADVANCE(53);
      END_STATE();
    case 29:
      if (lookahead == 't') ADVANCE(54);
      END_STATE();
    case 30:
      if (lookahead == 'k') ADVANCE(55);
      if (lookahead == 'p') ADVANCE(56);
      END_STATE();
    case 31:
      if (lookahead == 'w') ADVANCE(57);
      END_STATE();
    case 32:
      if (lookahead == 'l') ADVANCE(58);
      END_STATE();
    case 33:
      if (lookahead == 'c') ADVANCE(59);
      END_STATE();
    case 34:
      if (lookahead == 'n') ADVANCE(60);
      END_STATE();
    case 35:
      if (lookahead == 't') ADVANCE(61);
      END_STATE();
    case 36:
      if (lookahead == 'l') ADVANCE(62);
      END_STATE();
    case 37:
      if (lookahead == 'r') ADVANCE(63);
      END_STATE();
    case 38:
      if (lookahead == 'i') ADVANCE(64);
      END_STATE();
    case 39:
      if (lookahead == 'u') ADVANCE(65);
      END_STATE();
    case 40:
      if (lookahead == 'p') ADVANCE(66);
      END_STATE();
    case 41:
      if (lookahead == 'r') ADVANCE(67);
      END_STATE();
    case 42:
      if (lookahead == 'a') ADVANCE(68);
      END_STATE();
    case 43:
      if (lookahead == 'e') ADVANCE(69);
      END_STATE();
    case 44:
      if (lookahead == 'n') ADVANCE(70);
      END_STATE();
    case 45:
      if (lookahead == 's') ADVANCE(71);
      if (lookahead == 't') ADVANCE(72);
      END_STATE();
    case 46:
      if (lookahead == 'a') ADVANCE(73);
      if (lookahead == 'e') ADVANCE(74);
      END_STATE();
    case 47:
      if (lookahead == 'e') ADVANCE(75);
      END_STATE();
    case 48:
      if (lookahead == 'l') ADVANCE(76);
      if (lookahead == 's') ADVANCE(77);
      END_STATE();
    case 49:
      ACCEPT_TOKEN(anon_sym_for);
      END_STATE();
    case 50:
      if (lookahead == 'c') ADVANCE(78);
      END_STATE();
    case 51:
      if (lookahead == 'o') ADVANCE(79);
      END_STATE();
    case 52:
      if (lookahead == 'o') ADVANCE(80);
      END_STATE();
    case 53:
      if (lookahead == 'e') ADVANCE(81);
      END_STATE();
    case 54:
      if (lookahead == 'a') ADVANCE(82);
      END_STATE();
    case 55:
      if (lookahead == 'e') ADVANCE(83);
      END_STATE();
    case 56:
      ACCEPT_TOKEN(anon_sym_map);
      END_STATE();
    case 57:
      ACCEPT_TOKEN(anon_sym_new);
      END_STATE();
    case 58:
      ACCEPT_TOKEN(sym_nil);
      END_STATE();
    case 59:
      if (lookahead == 'k') ADVANCE(84);
      END_STATE();
    case 60:
      if (lookahead == 'g') ADVANCE(85);
      END_STATE();
    case 61:
      if (lookahead == 'u') ADVANCE(86);
      END_STATE();
    case 62:
      if (lookahead == 'e') ADVANCE(87);
      END_STATE();
    case 63:
      if (lookahead == 'u') ADVANCE(88);
      END_STATE();
    case 64:
      if (lookahead == 't') ADVANCE(89);
      END_STATE();
    case 65:
      if (lookahead == 'e') ADVANCE(90);
      END_STATE();
    case 66:
      if (lookahead == 'e') ADVANCE(91);
      END_STATE();
    case 67:
      ACCEPT_TOKEN(anon_sym_var);
      END_STATE();
    case 68:
      if (lookahead == 'k') ADVANCE(92);
      END_STATE();
    case 69:
      ACCEPT_TOKEN(anon_sym_case);
      END_STATE();
    case 70:
      ACCEPT_TOKEN(anon_sym_chan);
      END_STATE();
    case 71:
      if (lookahead == 't') ADVANCE(93);
      END_STATE();
    case 72:
      if (lookahead == 'i') ADVANCE(94);
      END_STATE();
    case 73:
      if (lookahead == 'u') ADVANCE(95);
      END_STATE();
    case 74:
      if (lookahead == 'r') ADVANCE(96);
      END_STATE();
    case 75:
      ACCEPT_TOKEN(anon_sym_else);
      END_STATE();
    case 76:
      if (lookahead == 't') ADVANCE(97);
      END_STATE();
    case 77:
      if (lookahead == 'e') ADVANCE(98);
      END_STATE();
    case 78:
      ACCEPT_TOKEN(anon_sym_func);
      END_STATE();
    case 79:
      ACCEPT_TOKEN(anon_sym_goto);
      END_STATE();
    case 80:
      if (lookahead == 'r') ADVANCE(99);
      END_STATE();
    case 81:
      if (lookahead == 'r') ADVANCE(100);
      END_STATE();
    case 82:
      ACCEPT_TOKEN(sym_iota);
      END_STATE();
    case 83:
      ACCEPT_TOKEN(anon_sym_make);
      END_STATE();
    case 84:
      if (lookahead == 'a') ADVANCE(101);
      END_STATE();
    case 85:
      if (lookahead == 'e') ADVANCE(102);
      END_STATE();
    case 86:
      if (lookahead == 'r') ADVANCE(103);
      END_STATE();
    case 87:
      if (lookahead == 'c') ADVANCE(104);
      END_STATE();
    case 88:
      if (lookahead == 'c') ADVANCE(105);
      END_STATE();
    case 89:
      if (lookahead == 'c') ADVANCE(106);
      END_STATE();
    case 90:
      ACCEPT_TOKEN(sym_true);
      END_STATE();
    case 91:
      ACCEPT_TOKEN(anon_sym_type);
      END_STATE();
    case 92:
      ACCEPT_TOKEN(anon_sym_break);
      END_STATE();
    case 93:
      ACCEPT_TOKEN(anon_sym_const);
      END_STATE();
    case 94:
      if (lookahead == 'n') ADVANCE(107);
      END_STATE();
    case 95:
      if (lookahead == 'l') ADVANCE(108);
      END_STATE();
    case 96:
      ACCEPT_TOKEN(anon_sym_defer);
      END_STATE();
    case 97:
      if (lookahead == 'h') ADVANCE(109);
      END_STATE();
    case 98:
      ACCEPT_TOKEN(sym_false);
      END_STATE();
    case 99:
      if (lookahead == 't') ADVANCE(110);
      END_STATE();
    case 100:
      if (lookahead == 'f') ADVANCE(111);
      END_STATE();
    case 101:
      if (lookahead == 'g') ADVANCE(112);
      END_STATE();
    case 102:
      ACCEPT_TOKEN(anon_sym_range);
      END_STATE();
    case 103:
      if (lookahead == 'n') ADVANCE(113);
      END_STATE();
    case 104:
      if (lookahead == 't') ADVANCE(114);
      END_STATE();
    case 105:
      if (lookahead == 't') ADVANCE(115);
      END_STATE();
    case 106:
      if (lookahead == 'h') ADVANCE(116);
      END_STATE();
    case 107:
      if (lookahead == 'u') ADVANCE(117);
      END_STATE();
    case 108:
      if (lookahead == 't') ADVANCE(118);
      END_STATE();
    case 109:
      if (lookahead == 'r') ADVANCE(119);
      END_STATE();
    case 110:
      ACCEPT_TOKEN(anon_sym_import);
      END_STATE();
    case 111:
      if (lookahead == 'a') ADVANCE(120);
      END_STATE();
    case 112:
      if (lookahead == 'e') ADVANCE(121);
      END_STATE();
    case 113:
      ACCEPT_TOKEN(anon_sym_return);
      END_STATE();
    case 114:
      ACCEPT_TOKEN(anon_sym_select);
      END_STATE();
    case 115:
      ACCEPT_TOKEN(anon_sym_struct);
      END_STATE();
    case 116:
      ACCEPT_TOKEN(anon_sym_switch);
      END_STATE();
    case 117:
      if (lookahead == 'e') ADVANCE(122);
      END_STATE();
    case 118:
      ACCEPT_TOKEN(anon_sym_default);
      END_STATE();
    case 119:
      if (lookahead == 'o') ADVANCE(123);
      END_STATE();
    case 120:
      if (lookahead == 'c') ADVANCE(124);
      END_STATE();
    case 121:
      ACCEPT_TOKEN(anon_sym_package);
      END_STATE();
    case 122:
      ACCEPT_TOKEN(anon_sym_continue);
      END_STATE();
    case 123:
      if (lookahead == 'u') ADVANCE(125);
      END_STATE();
    case 124:
      if (lookahead == 'e') ADVANCE(126);
      END_STATE();
    case 125:
      if (lookahead == 'g') ADVANCE(127);
      END_STATE();
    case 126:
      ACCEPT_TOKEN(anon_sym_interface);
      END_STATE();
    case 127:
      if (lookahead == 'h') ADVANCE(128);
      END_STATE();
    case 128:
      ACCEPT_TOKEN(anon_sym_fallthrough);
      END_STATE();
    default:
      return false;
  }
}

static const TSLexMode ts_lex_modes[STATE_COUNT] = {
  [0] = {.lex_state = 0},
  [1] = {.lex_state = 58},
  [2] = {.lex_state = 58},
  [3] = {.lex_state = 58},
  [4] = {.lex_state = 58},
  [5] = {.lex_state = 58},
  [6] = {.lex_state = 58},
  [7] = {.lex_state = 58},
  [8] = {.lex_state = 58},
  [9] = {.lex_state = 58},
  [10] = {.lex_state = 58},
  [11] = {.lex_state = 58},
  [12] = {.lex_state = 58},
  [13] = {.lex_state = 58},
  [14] = {.lex_state = 58},
  [15] = {.lex_state = 58},
  [16] = {.lex_state = 58},
  [17] = {.lex_state = 58},
  [18] = {.lex_state = 58},
  [19] = {.lex_state = 58},
  [20] = {.lex_state = 58},
  [21] = {.lex_state = 58},
  [22] = {.lex_state = 58},
  [23] = {.lex_state = 58},
  [24] = {.lex_state = 58},
  [25] = {.lex_state = 58},
  [26] = {.lex_state = 58},
  [27] = {.lex_state = 58},
  [28] = {.lex_state = 55},
  [29] = {.lex_state = 58},
  [30] = {.lex_state = 58},
  [31] = {.lex_state = 58},
  [32] = {.lex_state = 58},
  [33] = {.lex_state = 58},
  [34] = {.lex_state = 58},
  [35] = {.lex_state = 58},
  [36] = {.lex_state = 58},
  [37] = {.lex_state = 58},
  [38] = {.lex_state = 58},
  [39] = {.lex_state = 58},
  [40] = {.lex_state = 58},
  [41] = {.lex_state = 58},
  [42] = {.lex_state = 58},
  [43] = {.lex_state = 58},
  [44] = {.lex_state = 58},
  [45] = {.lex_state = 58},
  [46] = {.lex_state = 58},
  [47] = {.lex_state = 58},
  [48] = {.lex_state = 58},
  [49] = {.lex_state = 58},
  [50] = {.lex_state = 58},
  [51] = {.lex_state = 58},
  [52] = {.lex_state = 58},
  [53] = {.lex_state = 58},
  [54] = {.lex_state = 58},
  [55] = {.lex_state = 58},
  [56] = {.lex_state = 58},
  [57] = {.lex_state = 58},
  [58] = {.lex_state = 58},
  [59] = {.lex_state = 58},
  [60] = {.lex_state = 58},
  [61] = {.lex_state = 58},
  [62] = {.lex_state = 58},
  [63] = {.lex_state = 58},
  [64] = {.lex_state = 58},
  [65] = {.lex_state = 58},
  [66] = {.lex_state = 58},
  [67] = {.lex_state = 58},
  [68] = {.lex_state = 58},
  [69] = {.lex_state = 58},
  [70] = {.lex_state = 58},
  [71] = {.lex_state = 58},
  [72] = {.lex_state = 58},
  [73] = {.lex_state = 58},
  [74] = {.lex_state = 58},
  [75] = {.lex_state = 58},
  [76] = {.lex_state = 58},
  [77] = {.lex_state = 58},
  [78] = {.lex_state = 58},
  [79] = {.lex_state = 58},
  [80] = {.lex_state = 58},
  [81] = {.lex_state = 58},
  [82] = {.lex_state = 58},
  [83] = {.lex_state = 58},
  [84] = {.lex_state = 58},
  [85] = {.lex_state = 58},
  [86] = {.lex_state = 58},
  [87] = {.lex_state = 58},
  [88] = {.lex_state = 58},
  [89] = {.lex_state = 58},
  [90] = {.lex_state = 58},
  [91] = {.lex_state = 58},
  [92] = {.lex_state = 58},
  [93] = {.lex_state = 58},
  [94] = {.lex_state = 58},
  [95] = {.lex_state = 58},
  [96] = {.lex_state = 58},
  [97] = {.lex_state = 58},
  [98] = {.lex_state = 58},
  [99] = {.lex_state = 58},
  [100] = {.lex_state = 58},
  [101] = {.lex_state = 58},
  [102] = {.lex_state = 58},
  [103] = {.lex_state = 58},
  [104] = {.lex_state = 58},
  [105] = {.lex_state = 58},
  [106] = {.lex_state = 58},
  [107] = {.lex_state = 58},
  [108] = {.lex_state = 58},
  [109] = {.lex_state = 58},
  [110] = {.lex_state = 58},
  [111] = {.lex_state = 58},
  [112] = {.lex_state = 58},
  [113] = {.lex_state = 58},
  [114] = {.lex_state = 58},
  [115] = {.lex_state = 58},
  [116] = {.lex_state = 58},
  [117] = {.lex_state = 58},
  [118] = {.lex_state = 58},
  [119] = {.lex_state = 58},
  [120] = {.lex_state = 58},
  [121] = {.lex_state = 58},
  [122] = {.lex_state = 58},
  [123] = {.lex_state = 58},
  [124] = {.lex_state = 58},
  [125] = {.lex_state = 58},
  [126] = {.lex_state = 58},
  [127] = {.lex_state = 58},
  [128] = {.lex_state = 58},
  [129] = {.lex_state = 58},
  [130] = {.lex_state = 58},
  [131] = {.lex_state = 58},
  [132] = {.lex_state = 58},
  [133] = {.lex_state = 58},
  [134] = {.lex_state = 58},
  [135] = {.lex_state = 58},
  [136] = {.lex_state = 58},
  [137] = {.lex_state = 58},
  [138] = {.lex_state = 58},
  [139] = {.lex_state = 58},
  [140] = {.lex_state = 58},
  [141] = {.lex_state = 58},
  [142] = {.lex_state = 58},
  [143] = {.lex_state = 58},
  [144] = {.lex_state = 58},
  [145] = {.lex_state = 58},
  [146] = {.lex_state = 58},
  [147] = {.lex_state = 58},
  [148] = {.lex_state = 58},
  [149] = {.lex_state = 58},
  [150] = {.lex_state = 58},
  [151] = {.lex_state = 58},
  [152] = {.lex_state = 58},
  [153] = {.lex_state = 58},
  [154] = {.lex_state = 58},
  [155] = {.lex_state = 58},
  [156] = {.lex_state = 58},
  [157] = {.lex_state = 58},
  [158] = {.lex_state = 58},
  [159] = {.lex_state = 58},
  [160] = {.lex_state = 58},
  [161] = {.lex_state = 58},
  [162] = {.lex_state = 58},
  [163] = {.lex_state = 58},
  [164] = {.lex_state = 58},
  [165] = {.lex_state = 58},
  [166] = {.lex_state = 58},
  [167] = {.lex_state = 58},
  [168] = {.lex_state = 58},
  [169] = {.lex_state = 58},
  [170] = {.lex_state = 58},
  [171] = {.lex_state = 58},
  [172] = {.lex_state = 58},
  [173] = {.lex_state = 58},
  [174] = {.lex_state = 58},
  [175] = {.lex_state = 58},
  [176] = {.lex_state = 58},
  [177] = {.lex_state = 58},
  [178] = {.lex_state = 58},
  [179] = {.lex_state = 58},
  [180] = {.lex_state = 58},
  [181] = {.lex_state = 58},
  [182] = {.lex_state = 58},
  [183] = {.lex_state = 58},
  [184] = {.lex_state = 58},
  [185] = {.lex_state = 58},
  [186] = {.lex_state = 58},
  [187] = {.lex_state = 58},
  [188] = {.lex_state = 58},
  [189] = {.lex_state = 58},
  [190] = {.lex_state = 58},
  [191] = {.lex_state = 58},
  [192] = {.lex_state = 58},
  [193] = {.lex_state = 58},
  [194] = {.lex_state = 58},
  [195] = {.lex_state = 58},
  [196] = {.lex_state = 58},
  [197] = {.lex_state = 58},
  [198] = {.lex_state = 58},
  [199] = {.lex_state = 58},
  [200] = {.lex_state = 58},
  [201] = {.lex_state = 58},
  [202] = {.lex_state = 58},
  [203] = {.lex_state = 58},
  [204] = {.lex_state = 58},
  [205] = {.lex_state = 58},
  [206] = {.lex_state = 58},
  [207] = {.lex_state = 58},
  [208] = {.lex_state = 58},
  [209] = {.lex_state = 58},
  [210] = {.lex_state = 58},
  [211] = {.lex_state = 58},
  [212] = {.lex_state = 58},
  [213] = {.lex_state = 58},
  [214] = {.lex_state = 58},
  [215] = {.lex_state = 58},
  [216] = {.lex_state = 58},
  [217] = {.lex_state = 58},
  [218] = {.lex_state = 58},
  [219] = {.lex_state = 58},
  [220] = {.lex_state = 58},
  [221] = {.lex_state = 58},
  [222] = {.lex_state = 58},
  [223] = {.lex_state = 58},
  [224] = {.lex_state = 58},
  [225] = {.lex_state = 58},
  [226] = {.lex_state = 58},
  [227] = {.lex_state = 58},
  [228] = {.lex_state = 58},
  [229] = {.lex_state = 58},
  [230] = {.lex_state = 58},
  [231] = {.lex_state = 58},
  [232] = {.lex_state = 58},
  [233] = {.lex_state = 58},
  [234] = {.lex_state = 58},
  [235] = {.lex_state = 58},
  [236] = {.lex_state = 3},
  [237] = {.lex_state = 56},
  [238] = {.lex_state = 56},
  [239] = {.lex_state = 56},
  [240] = {.lex_state = 56},
  [241] = {.lex_state = 56},
  [242] = {.lex_state = 56},
  [243] = {.lex_state = 56},
  [244] = {.lex_state = 56},
  [245] = {.lex_state = 56},
  [246] = {.lex_state = 58},
  [247] = {.lex_state = 56},
  [248] = {.lex_state = 56},
  [249] = {.lex_state = 56},
  [250] = {.lex_state = 56},
  [251] = {.lex_state = 56},
  [252] = {.lex_state = 56},
  [253] = {.lex_state = 56},
  [254] = {.lex_state = 56},
  [255] = {.lex_state = 56},
  [256] = {.lex_state = 4},
  [257] = {.lex_state = 56},
  [258] = {.lex_state = 56},
  [259] = {.lex_state = 4},
  [260] = {.lex_state = 56},
  [261] = {.lex_state = 56},
  [262] = {.lex_state = 56},
  [263] = {.lex_state = 56},
  [264] = {.lex_state = 56},
  [265] = {.lex_state = 56},
  [266] = {.lex_state = 56},
  [267] = {.lex_state = 56},
  [268] = {.lex_state = 56},
  [269] = {.lex_state = 56},
  [270] = {.lex_state = 56},
  [271] = {.lex_state = 56},
  [272] = {.lex_state = 56},
  [273] = {.lex_state = 56},
  [274] = {.lex_state = 56},
  [275] = {.lex_state = 56},
  [276] = {.lex_state = 56},
  [277] = {.lex_state = 56},
  [278] = {.lex_state = 56},
  [279] = {.lex_state = 56},
  [280] = {.lex_state = 56},
  [281] = {.lex_state = 56},
  [282] = {.lex_state = 56},
  [283] = {.lex_state = 56},
  [284] = {.lex_state = 56},
  [285] = {.lex_state = 56},
  [286] = {.lex_state = 56},
  [287] = {.lex_state = 56},
  [288] = {.lex_state = 56},
  [289] = {.lex_state = 56},
  [290] = {.lex_state = 4},
  [291] = {.lex_state = 4},
  [292] = {.lex_state = 4},
  [293] = {.lex_state = 4},
  [294] = {.lex_state = 4},
  [295] = {.lex_state = 4},
  [296] = {.lex_state = 4},
  [297] = {.lex_state = 4},
  [298] = {.lex_state = 4},
  [299] = {.lex_state = 4},
  [300] = {.lex_state = 4},
  [301] = {.lex_state = 4},
  [302] = {.lex_state = 4},
  [303] = {.lex_state = 4},
  [304] = {.lex_state = 4},
  [305] = {.lex_state = 4},
  [306] = {.lex_state = 4},
  [307] = {.lex_state = 4},
  [308] = {.lex_state = 4},
  [309] = {.lex_state = 4},
  [310] = {.lex_state = 4},
  [311] = {.lex_state = 4},
  [312] = {.lex_state = 4},
  [313] = {.lex_state = 4},
  [314] = {.lex_state = 4},
  [315] = {.lex_state = 4},
  [316] = {.lex_state = 4},
  [317] = {.lex_state = 4},
  [318] = {.lex_state = 4},
  [319] = {.lex_state = 4},
  [320] = {.lex_state = 4},
  [321] = {.lex_state = 4},
  [322] = {.lex_state = 4},
  [323] = {.lex_state = 4},
  [324] = {.lex_state = 4},
  [325] = {.lex_state = 4},
  [326] = {.lex_state = 4},
  [327] = {.lex_state = 4},
  [328] = {.lex_state = 4},
  [329] = {.lex_state = 4},
  [330] = {.lex_state = 4},
  [331] = {.lex_state = 4},
  [332] = {.lex_state = 4},
  [333] = {.lex_state = 4},
  [334] = {.lex_state = 4},
  [335] = {.lex_state = 4},
  [336] = {.lex_state = 4},
  [337] = {.lex_state = 4},
  [338] = {.lex_state = 4},
  [339] = {.lex_state = 5},
  [340] = {.lex_state = 5},
  [341] = {.lex_state = 5},
  [342] = {.lex_state = 5},
  [343] = {.lex_state = 5},
  [344] = {.lex_state = 5},
  [345] = {.lex_state = 5},
  [346] = {.lex_state = 5},
  [347] = {.lex_state = 5},
  [348] = {.lex_state = 5},
  [349] = {.lex_state = 5},
  [350] = {.lex_state = 5},
  [351] = {.lex_state = 5},
  [352] = {.lex_state = 5},
  [353] = {.lex_state = 5},
  [354] = {.lex_state = 5},
  [355] = {.lex_state = 5},
  [356] = {.lex_state = 5},
  [357] = {.lex_state = 5},
  [358] = {.lex_state = 5},
  [359] = {.lex_state = 5},
  [360] = {.lex_state = 5},
  [361] = {.lex_state = 5},
  [362] = {.lex_state = 5},
  [363] = {.lex_state = 5},
  [364] = {.lex_state = 5},
  [365] = {.lex_state = 5},
  [366] = {.lex_state = 5},
  [367] = {.lex_state = 5},
  [368] = {.lex_state = 5},
  [369] = {.lex_state = 5},
  [370] = {.lex_state = 5},
  [371] = {.lex_state = 5},
  [372] = {.lex_state = 5},
  [373] = {.lex_state = 5},
  [374] = {.lex_state = 5},
  [375] = {.lex_state = 5},
  [376] = {.lex_state = 5},
  [377] = {.lex_state = 5},
  [378] = {.lex_state = 5},
  [379] = {.lex_state = 5},
  [380] = {.lex_state = 5},
  [381] = {.lex_state = 5},
  [382] = {.lex_state = 5},
  [383] = {.lex_state = 5},
  [384] = {.lex_state = 5},
  [385] = {.lex_state = 56},
  [386] = {.lex_state = 5},
  [387] = {.lex_state = 5},
  [388] = {.lex_state = 55},
  [389] = {.lex_state = 55},
  [390] = {.lex_state = 6},
  [391] = {.lex_state = 6},
  [392] = {.lex_state = 6},
  [393] = {.lex_state = 6},
  [394] = {.lex_state = 55},
  [395] = {.lex_state = 55},
  [396] = {.lex_state = 6},
  [397] = {.lex_state = 6},
  [398] = {.lex_state = 6},
  [399] = {.lex_state = 6},
  [400] = {.lex_state = 6},
  [401] = {.lex_state = 6},
  [402] = {.lex_state = 6},
  [403] = {.lex_state = 6},
  [404] = {.lex_state = 6},
  [405] = {.lex_state = 6},
  [406] = {.lex_state = 6},
  [407] = {.lex_state = 6},
  [408] = {.lex_state = 6},
  [409] = {.lex_state = 6},
  [410] = {.lex_state = 6},
  [411] = {.lex_state = 6},
  [412] = {.lex_state = 6},
  [413] = {.lex_state = 6},
  [414] = {.lex_state = 6},
  [415] = {.lex_state = 6},
  [416] = {.lex_state = 6},
  [417] = {.lex_state = 6},
  [418] = {.lex_state = 6},
  [419] = {.lex_state = 6},
  [420] = {.lex_state = 6},
  [421] = {.lex_state = 6},
  [422] = {.lex_state = 6},
  [423] = {.lex_state = 6},
  [424] = {.lex_state = 1},
  [425] = {.lex_state = 6},
  [426] = {.lex_state = 1},
  [427] = {.lex_state = 6},
  [428] = {.lex_state = 6},
  [429] = {.lex_state = 6},
  [430] = {.lex_state = 6},
  [431] = {.lex_state = 6},
  [432] = {.lex_state = 6},
  [433] = {.lex_state = 6},
  [434] = {.lex_state = 6},
  [435] = {.lex_state = 6},
  [436] = {.lex_state = 6},
  [437] = {.lex_state = 6},
  [438] = {.lex_state = 1},
  [439] = {.lex_state = 6},
  [440] = {.lex_state = 1},
  [441] = {.lex_state = 1},
  [442] = {.lex_state = 0},
  [443] = {.lex_state = 6},
  [444] = {.lex_state = 6},
  [445] = {.lex_state = 1},
  [446] = {.lex_state = 0},
  [447] = {.lex_state = 1},
  [448] = {.lex_state = 1},
  [449] = {.lex_state = 1},
  [450] = {.lex_state = 1},
  [451] = {.lex_state = 1},
  [452] = {.lex_state = 0},
  [453] = {.lex_state = 3},
  [454] = {.lex_state = 55},
  [455] = {.lex_state = 1},
  [456] = {.lex_state = 1},
  [457] = {.lex_state = 55},
  [458] = {.lex_state = 55},
  [459] = {.lex_state = 3},
  [460] = {.lex_state = 3},
  [461] = {.lex_state = 55},
  [462] = {.lex_state = 1},
  [463] = {.lex_state = 1},
  [464] = {.lex_state = 1},
  [465] = {.lex_state = 1},
  [466] = {.lex_state = 1},
  [467] = {.lex_state = 1},
  [468] = {.lex_state = 1},
  [469] = {.lex_state = 1},
  [470] = {.lex_state = 1},
  [471] = {.lex_state = 1},
  [472] = {.lex_state = 1},
  [473] = {.lex_state = 1},
  [474] = {.lex_state = 1},
  [475] = {.lex_state = 1},
  [476] = {.lex_state = 1},
  [477] = {.lex_state = 1},
  [478] = {.lex_state = 1},
  [479] = {.lex_state = 1},
  [480] = {.lex_state = 1},
  [481] = {.lex_state = 1},
  [482] = {.lex_state = 1},
  [483] = {.lex_state = 1},
  [484] = {.lex_state = 1},
  [485] = {.lex_state = 1},
  [486] = {.lex_state = 1},
  [487] = {.lex_state = 1},
  [488] = {.lex_state = 1},
  [489] = {.lex_state = 1},
  [490] = {.lex_state = 1},
  [491] = {.lex_state = 1},
  [492] = {.lex_state = 1},
  [493] = {.lex_state = 1},
  [494] = {.lex_state = 0},
  [495] = {.lex_state = 1},
  [496] = {.lex_state = 1},
  [497] = {.lex_state = 55},
  [498] = {.lex_state = 55},
  [499] = {.lex_state = 55},
  [500] = {.lex_state = 3},
  [501] = {.lex_state = 55},
  [502] = {.lex_state = 3},
  [503] = {.lex_state = 1},
  [504] = {.lex_state = 1},
  [505] = {.lex_state = 3},
  [506] = {.lex_state = 3},
  [507] = {.lex_state = 3},
  [508] = {.lex_state = 3},
  [509] = {.lex_state = 1},
  [510] = {.lex_state = 0},
  [511] = {.lex_state = 0},
  [512] = {.lex_state = 1},
  [513] = {.lex_state = 1},
  [514] = {.lex_state = 6},
  [515] = {.lex_state = 6},
  [516] = {.lex_state = 0},
  [517] = {.lex_state = 6},
  [518] = {.lex_state = 6},
  [519] = {.lex_state = 55},
  [520] = {.lex_state = 6},
  [521] = {.lex_state = 0},
  [522] = {.lex_state = 0},
  [523] = {.lex_state = 6},
  [524] = {.lex_state = 6},
  [525] = {.lex_state = 3},
  [526] = {.lex_state = 6},
  [527] = {.lex_state = 0},
  [528] = {.lex_state = 0},
  [529] = {.lex_state = 6},
  [530] = {.lex_state = 6},
  [531] = {.lex_state = 6},
  [532] = {.lex_state = 0},
  [533] = {.lex_state = 6},
  [534] = {.lex_state = 3},
  [535] = {.lex_state = 3},
  [536] = {.lex_state = 6},
  [537] = {.lex_state = 3},
  [538] = {.lex_state = 3},
  [539] = {.lex_state = 3},
  [540] = {.lex_state = 3},
  [541] = {.lex_state = 3},
  [542] = {.lex_state = 3},
  [543] = {.lex_state = 3},
  [544] = {.lex_state = 3},
  [545] = {.lex_state = 3},
  [546] = {.lex_state = 3},
  [547] = {.lex_state = 3},
  [548] = {.lex_state = 3},
  [549] = {.lex_state = 3},
  [550] = {.lex_state = 3},
  [551] = {.lex_state = 3},
  [552] = {.lex_state = 3},
  [553] = {.lex_state = 3},
  [554] = {.lex_state = 3},
  [555] = {.lex_state = 3},
  [556] = {.lex_state = 3},
  [557] = {.lex_state = 3},
  [558] = {.lex_state = 3},
  [559] = {.lex_state = 3},
  [560] = {.lex_state = 3},
  [561] = {.lex_state = 3},
  [562] = {.lex_state = 3},
  [563] = {.lex_state = 3},
  [564] = {.lex_state = 3},
  [565] = {.lex_state = 3},
  [566] = {.lex_state = 6},
  [567] = {.lex_state = 3},
  [568] = {.lex_state = 0},
  [569] = {.lex_state = 3},
  [570] = {.lex_state = 3},
  [571] = {.lex_state = 3},
  [572] = {.lex_state = 6},
  [573] = {.lex_state = 0},
  [574] = {.lex_state = 0},
  [575] = {.lex_state = 6},
  [576] = {.lex_state = 0},
  [577] = {.lex_state = 0},
  [578] = {.lex_state = 0},
  [579] = {.lex_state = 0},
  [580] = {.lex_state = 6},
  [581] = {.lex_state = 0},
  [582] = {.lex_state = 6},
  [583] = {.lex_state = 6},
  [584] = {.lex_state = 6},
  [585] = {.lex_state = 6},
  [586] = {.lex_state = 6},
  [587] = {.lex_state = 0},
  [588] = {.lex_state = 0},
  [589] = {.lex_state = 0},
  [590] = {.lex_state = 3},
  [591] = {.lex_state = 0},
  [592] = {.lex_state = 6},
  [593] = {.lex_state = 0},
  [594] = {.lex_state = 3},
  [595] = {.lex_state = 3},
  [596] = {.lex_state = 3},
  [597] = {.lex_state = 0},
  [598] = {.lex_state = 6},
  [599] = {.lex_state = 0},
  [600] = {.lex_state = 6},
  [601] = {.lex_state = 0},
  [602] = {.lex_state = 6},
  [603] = {.lex_state = 6},
  [604] = {.lex_state = 6},
  [605] = {.lex_state = 6},
  [606] = {.lex_state = 6},
  [607] = {.lex_state = 0},
  [608] = {.lex_state = 6},
  [609] = {.lex_state = 0},
  [610] = {.lex_state = 6},
  [611] = {.lex_state = 6},
  [612] = {.lex_state = 0},
  [613] = {.lex_state = 0},
  [614] = {.lex_state = 0},
  [615] = {.lex_state = 6},
  [616] = {.lex_state = 0},
  [617] = {.lex_state = 6},
  [618] = {.lex_state = 0},
  [619] = {.lex_state = 6},
  [620] = {.lex_state = 6},
  [621] = {.lex_state = 0},
  [622] = {.lex_state = 6},
  [623] = {.lex_state = 0},
  [624] = {.lex_state = 6},
  [625] = {.lex_state = 6},
  [626] = {.lex_state = 6},
  [627] = {.lex_state = 6},
  [628] = {.lex_state = 0},
  [629] = {.lex_state = 6},
  [630] = {.lex_state = 0},
  [631] = {.lex_state = 6},
  [632] = {.lex_state = 6},
  [633] = {.lex_state = 6},
  [634] = {.lex_state = 0},
  [635] = {.lex_state = 0},
  [636] = {.lex_state = 6},
  [637] = {.lex_state = 6},
  [638] = {.lex_state = 0},
  [639] = {.lex_state = 6},
  [640] = {.lex_state = 6},
  [641] = {.lex_state = 6},
  [642] = {.lex_state = 0},
  [643] = {.lex_state = 6},
  [644] = {.lex_state = 0},
  [645] = {.lex_state = 6},
  [646] = {.lex_state = 0},
  [647] = {.lex_state = 6},
  [648] = {.lex_state = 6},
  [649] = {.lex_state = 0},
  [650] = {.lex_state = 0},
  [651] = {.lex_state = 0},
  [652] = {.lex_state = 0},
  [653] = {.lex_state = 6},
  [654] = {.lex_state = 6},
  [655] = {.lex_state = 0},
  [656] = {.lex_state = 6},
  [657] = {.lex_state = 6},
  [658] = {.lex_state = 6},
  [659] = {.lex_state = 6},
  [660] = {.lex_state = 0},
  [661] = {.lex_state = 0},
  [662] = {.lex_state = 0},
  [663] = {.lex_state = 6},
  [664] = {.lex_state = 6},
  [665] = {.lex_state = 0},
  [666] = {.lex_state = 6},
  [667] = {.lex_state = 6},
  [668] = {.lex_state = 6},
  [669] = {.lex_state = 6},
  [670] = {.lex_state = 0},
  [671] = {.lex_state = 0},
  [672] = {.lex_state = 0},
  [673] = {.lex_state = 6},
  [674] = {.lex_state = 6},
  [675] = {.lex_state = 6},
  [676] = {.lex_state = 6},
  [677] = {.lex_state = 0},
  [678] = {.lex_state = 0},
  [679] = {.lex_state = 0},
  [680] = {.lex_state = 0},
  [681] = {.lex_state = 6},
  [682] = {.lex_state = 6},
  [683] = {.lex_state = 6},
  [684] = {.lex_state = 6},
  [685] = {.lex_state = 6},
  [686] = {.lex_state = 0},
  [687] = {.lex_state = 0},
  [688] = {.lex_state = 6},
  [689] = {.lex_state = 6},
  [690] = {.lex_state = 6},
  [691] = {.lex_state = 6},
  [692] = {.lex_state = 0},
  [693] = {.lex_state = 0},
  [694] = {.lex_state = 0},
  [695] = {.lex_state = 6},
  [696] = {.lex_state = 6},
  [697] = {.lex_state = 0},
  [698] = {.lex_state = 0},
  [699] = {.lex_state = 0},
  [700] = {.lex_state = 0},
  [701] = {.lex_state = 0},
  [702] = {.lex_state = 0},
  [703] = {.lex_state = 0},
  [704] = {.lex_state = 0},
  [705] = {.lex_state = 0},
  [706] = {.lex_state = 0},
  [707] = {.lex_state = 0},
  [708] = {.lex_state = 0},
  [709] = {.lex_state = 0},
  [710] = {.lex_state = 0},
  [711] = {.lex_state = 0},
  [712] = {.lex_state = 0},
  [713] = {.lex_state = 0},
  [714] = {.lex_state = 0},
  [715] = {.lex_state = 0},
  [716] = {.lex_state = 0},
  [717] = {.lex_state = 0},
  [718] = {.lex_state = 0},
  [719] = {.lex_state = 0},
  [720] = {.lex_state = 0},
  [721] = {.lex_state = 0},
  [722] = {.lex_state = 0},
  [723] = {.lex_state = 0},
  [724] = {.lex_state = 0},
  [725] = {.lex_state = 0},
  [726] = {.lex_state = 0},
  [727] = {.lex_state = 0},
  [728] = {.lex_state = 0},
  [729] = {.lex_state = 0},
  [730] = {.lex_state = 0},
  [731] = {.lex_state = 0},
  [732] = {.lex_state = 0},
  [733] = {.lex_state = 0},
  [734] = {.lex_state = 0},
  [735] = {.lex_state = 0},
  [736] = {.lex_state = 0},
  [737] = {.lex_state = 0},
  [738] = {.lex_state = 0},
  [739] = {.lex_state = 0},
  [740] = {.lex_state = 0},
  [741] = {.lex_state = 0},
  [742] = {.lex_state = 0},
  [743] = {.lex_state = 0},
  [744] = {.lex_state = 0},
  [745] = {.lex_state = 0},
  [746] = {.lex_state = 0},
  [747] = {.lex_state = 0},
  [748] = {.lex_state = 0},
  [749] = {.lex_state = 0},
  [750] = {.lex_state = 0},
  [751] = {.lex_state = 0},
  [752] = {.lex_state = 0},
  [753] = {.lex_state = 0},
  [754] = {.lex_state = 0},
  [755] = {.lex_state = 0},
  [756] = {.lex_state = 0},
  [757] = {.lex_state = 0},
  [758] = {.lex_state = 0},
  [759] = {.lex_state = 0},
  [760] = {.lex_state = 0},
  [761] = {.lex_state = 0},
  [762] = {.lex_state = 0},
  [763] = {.lex_state = 0},
  [764] = {.lex_state = 6},
  [765] = {.lex_state = 0},
  [766] = {.lex_state = 0},
  [767] = {.lex_state = 0},
  [768] = {.lex_state = 0},
  [769] = {.lex_state = 0},
  [770] = {.lex_state = 0},
  [771] = {.lex_state = 0},
  [772] = {.lex_state = 0},
  [773] = {.lex_state = 0},
  [774] = {.lex_state = 0},
  [775] = {.lex_state = 0},
  [776] = {.lex_state = 0},
  [777] = {.lex_state = 0},
  [778] = {.lex_state = 0},
  [779] = {.lex_state = 0},
  [780] = {.lex_state = 0},
  [781] = {.lex_state = 0},
  [782] = {.lex_state = 55},
  [783] = {.lex_state = 55},
  [784] = {.lex_state = 55},
  [785] = {.lex_state = 55},
  [786] = {.lex_state = 55},
  [787] = {.lex_state = 55},
  [788] = {.lex_state = 0},
  [789] = {.lex_state = 0},
  [790] = {.lex_state = 0},
  [791] = {.lex_state = 0},
  [792] = {.lex_state = 55},
  [793] = {.lex_state = 55},
  [794] = {.lex_state = 55},
  [795] = {.lex_state = 55},
  [796] = {.lex_state = 0},
  [797] = {.lex_state = 0},
  [798] = {.lex_state = 56},
  [799] = {.lex_state = 56},
  [800] = {.lex_state = 0},
  [801] = {.lex_state = 0},
  [802] = {.lex_state = 0},
  [803] = {.lex_state = 55},
  [804] = {.lex_state = 0},
  [805] = {.lex_state = 0},
  [806] = {.lex_state = 55},
  [807] = {.lex_state = 0},
  [808] = {.lex_state = 0},
  [809] = {.lex_state = 0},
  [810] = {.lex_state = 55},
  [811] = {.lex_state = 55},
  [812] = {.lex_state = 55},
  [813] = {.lex_state = 55},
  [814] = {.lex_state = 55},
  [815] = {.lex_state = 55},
  [816] = {.lex_state = 55},
  [817] = {.lex_state = 55},
  [818] = {.lex_state = 55},
  [819] = {.lex_state = 55},
  [820] = {.lex_state = 55},
  [821] = {.lex_state = 55},
  [822] = {.lex_state = 55},
  [823] = {.lex_state = 55},
  [824] = {.lex_state = 0},
  [825] = {.lex_state = 55},
  [826] = {.lex_state = 55},
  [827] = {.lex_state = 55},
  [828] = {.lex_state = 55},
  [829] = {.lex_state = 55},
  [830] = {.lex_state = 55},
  [831] = {.lex_state = 55},
  [832] = {.lex_state = 55},
  [833] = {.lex_state = 55},
  [834] = {.lex_state = 55},
  [835] = {.lex_state = 55},
  [836] = {.lex_state = 55},
  [837] = {.lex_state = 55},
  [838] = {.lex_state = 0},
  [839] = {.lex_state = 3},
  [840] = {.lex_state = 0},
  [841] = {.lex_state = 0},
  [842] = {.lex_state = 3},
  [843] = {.lex_state = 56},
  [844] = {.lex_state = 3},
  [845] = {.lex_state = 55},
  [846] = {.lex_state = 3},
  [847] = {.lex_state = 3},
  [848] = {.lex_state = 3},
  [849] = {.lex_state = 55},
  [850] = {.lex_state = 0},
  [851] = {.lex_state = 3},
  [852] = {.lex_state = 3},
  [853] = {.lex_state = 56},
  [854] = {.lex_state = 3},
  [855] = {.lex_state = 3},
  [856] = {.lex_state = 0},
  [857] = {.lex_state = 0},
  [858] = {.lex_state = 0},
  [859] = {.lex_state = 0},
  [860] = {.lex_state = 0},
  [861] = {.lex_state = 0},
  [862] = {.lex_state = 0},
  [863] = {.lex_state = 0},
  [864] = {.lex_state = 0},
  [865] = {.lex_state = 0},
  [866] = {.lex_state = 0},
  [867] = {.lex_state = 0},
  [868] = {.lex_state = 0},
  [869] = {.lex_state = 0},
  [870] = {.lex_state = 0},
  [871] = {.lex_state = 0},
  [872] = {.lex_state = 0},
  [873] = {.lex_state = 0},
  [874] = {.lex_state = 0},
  [875] = {.lex_state = 0},
  [876] = {.lex_state = 0},
  [877] = {.lex_state = 0},
  [878] = {.lex_state = 0},
  [879] = {.lex_state = 0},
  [880] = {.lex_state = 0},
  [881] = {.lex_state = 0},
  [882] = {.lex_state = 3},
  [883] = {.lex_state = 0},
  [884] = {.lex_state = 0},
  [885] = {.lex_state = 0},
  [886] = {.lex_state = 55},
  [887] = {.lex_state = 0},
  [888] = {.lex_state = 55},
  [889] = {.lex_state = 0},
  [890] = {.lex_state = 0},
  [891] = {.lex_state = 0},
  [892] = {.lex_state = 0},
  [893] = {.lex_state = 56},
  [894] = {.lex_state = 0},
  [895] = {.lex_state = 55},
  [896] = {.lex_state = 0},
  [897] = {.lex_state = 56},
  [898] = {.lex_state = 55},
  [899] = {.lex_state = 55},
  [900] = {.lex_state = 0},
  [901] = {.lex_state = 55},
  [902] = {.lex_state = 55},
  [903] = {.lex_state = 0},
  [904] = {.lex_state = 0},
  [905] = {.lex_state = 0},
  [906] = {.lex_state = 0},
  [907] = {.lex_state = 55},
  [908] = {.lex_state = 55},
  [909] = {.lex_state = 55},
  [910] = {.lex_state = 55},
  [911] = {.lex_state = 55},
  [912] = {.lex_state = 55},
  [913] = {.lex_state = 55},
  [914] = {.lex_state = 55},
  [915] = {.lex_state = 55},
  [916] = {.lex_state = 55},
  [917] = {.lex_state = 55},
  [918] = {.lex_state = 55},
  [919] = {.lex_state = 55},
  [920] = {.lex_state = 55},
  [921] = {.lex_state = 55},
  [922] = {.lex_state = 55},
  [923] = {.lex_state = 55},
  [924] = {.lex_state = 55},
  [925] = {.lex_state = 55},
  [926] = {.lex_state = 55},
  [927] = {.lex_state = 55},
  [928] = {.lex_state = 55},
  [929] = {.lex_state = 55},
  [930] = {.lex_state = 55},
  [931] = {.lex_state = 55},
  [932] = {.lex_state = 55},
  [933] = {.lex_state = 55},
  [934] = {.lex_state = 55},
  [935] = {.lex_state = 0},
  [936] = {.lex_state = 55},
  [937] = {.lex_state = 55},
  [938] = {.lex_state = 3},
  [939] = {.lex_state = 55},
  [940] = {.lex_state = 55},
  [941] = {.lex_state = 55},
  [942] = {.lex_state = 55},
  [943] = {.lex_state = 55},
  [944] = {.lex_state = 55},
  [945] = {.lex_state = 55},
  [946] = {.lex_state = 55},
  [947] = {.lex_state = 55},
  [948] = {.lex_state = 55},
  [949] = {.lex_state = 55},
  [950] = {.lex_state = 55},
  [951] = {.lex_state = 0},
  [952] = {.lex_state = 55},
  [953] = {.lex_state = 0},
  [954] = {.lex_state = 55},
  [955] = {.lex_state = 55},
  [956] = {.lex_state = 55},
  [957] = {.lex_state = 55},
  [958] = {.lex_state = 0},
  [959] = {.lex_state = 55},
  [960] = {.lex_state = 0},
  [961] = {.lex_state = 55},
  [962] = {.lex_state = 0},
  [963] = {.lex_state = 55},
  [964] = {.lex_state = 0},
  [965] = {.lex_state = 55},
  [966] = {.lex_state = 55},
  [967] = {.lex_state = 55},
  [968] = {.lex_state = 55},
  [969] = {.lex_state = 0},
  [970] = {.lex_state = 55},
  [971] = {.lex_state = 55},
  [972] = {.lex_state = 0},
  [973] = {.lex_state = 55},
  [974] = {.lex_state = 0},
  [975] = {.lex_state = 55},
  [976] = {.lex_state = 55},
  [977] = {.lex_state = 55},
  [978] = {.lex_state = 55},
  [979] = {.lex_state = 55},
  [980] = {.lex_state = 55},
  [981] = {.lex_state = 55},
  [982] = {.lex_state = 55},
  [983] = {.lex_state = 55},
  [984] = {.lex_state = 55},
  [985] = {.lex_state = 55},
  [986] = {.lex_state = 55},
  [987] = {.lex_state = 55},
  [988] = {.lex_state = 55},
  [989] = {.lex_state = 55},
  [990] = {.lex_state = 55},
  [991] = {.lex_state = 55},
  [992] = {.lex_state = 55},
  [993] = {.lex_state = 55},
  [994] = {.lex_state = 55},
  [995] = {.lex_state = 55},
  [996] = {.lex_state = 55},
  [997] = {.lex_state = 0},
  [998] = {.lex_state = 0},
  [999] = {.lex_state = 55},
  [1000] = {.lex_state = 55},
  [1001] = {.lex_state = 0},
  [1002] = {.lex_state = 55},
  [1003] = {.lex_state = 55},
  [1004] = {.lex_state = 0},
  [1005] = {.lex_state = 0},
  [1006] = {.lex_state = 55},
  [1007] = {.lex_state = 0},
  [1008] = {.lex_state = 55},
  [1009] = {.lex_state = 55},
  [1010] = {.lex_state = 55},
  [1011] = {.lex_state = 55},
  [1012] = {.lex_state = 3},
  [1013] = {.lex_state = 55},
  [1014] = {.lex_state = 55},
  [1015] = {.lex_state = 0},
  [1016] = {.lex_state = 0},
  [1017] = {.lex_state = 55},
  [1018] = {.lex_state = 55},
  [1019] = {.lex_state = 0},
  [1020] = {.lex_state = 55},
  [1021] = {.lex_state = 55},
  [1022] = {.lex_state = 55},
  [1023] = {.lex_state = 55},
  [1024] = {.lex_state = 55},
  [1025] = {.lex_state = 55},
  [1026] = {.lex_state = 55},
  [1027] = {.lex_state = 0},
  [1028] = {.lex_state = 55},
  [1029] = {.lex_state = 0},
  [1030] = {.lex_state = 55},
  [1031] = {.lex_state = 0},
  [1032] = {.lex_state = 55},
  [1033] = {.lex_state = 0},
  [1034] = {.lex_state = 55},
  [1035] = {.lex_state = 55},
  [1036] = {.lex_state = 0},
  [1037] = {.lex_state = 55},
  [1038] = {.lex_state = 55},
  [1039] = {.lex_state = 55},
  [1040] = {.lex_state = 55},
  [1041] = {.lex_state = 0},
  [1042] = {.lex_state = 0},
  [1043] = {.lex_state = 55},
  [1044] = {.lex_state = 55},
  [1045] = {.lex_state = 55},
  [1046] = {.lex_state = 55},
  [1047] = {.lex_state = 55},
  [1048] = {.lex_state = 55},
  [1049] = {.lex_state = 55},
  [1050] = {.lex_state = 55},
  [1051] = {.lex_state = 55},
  [1052] = {.lex_state = 0},
  [1053] = {.lex_state = 55},
  [1054] = {.lex_state = 55},
  [1055] = {.lex_state = 0},
  [1056] = {.lex_state = 55},
  [1057] = {.lex_state = 0},
  [1058] = {.lex_state = 55},
  [1059] = {.lex_state = 0},
  [1060] = {.lex_state = 55},
  [1061] = {.lex_state = 2},
  [1062] = {.lex_state = 0},
  [1063] = {.lex_state = 55},
  [1064] = {.lex_state = 0},
  [1065] = {.lex_state = 55},
  [1066] = {.lex_state = 55},
  [1067] = {.lex_state = 0},
  [1068] = {.lex_state = 55},
  [1069] = {.lex_state = 58},
  [1070] = {.lex_state = 2},
  [1071] = {.lex_state = 58},
  [1072] = {.lex_state = 55},
  [1073] = {.lex_state = 0},
  [1074] = {.lex_state = 0},
  [1075] = {.lex_state = 0},
  [1076] = {.lex_state = 0},
  [1077] = {.lex_state = 2},
  [1078] = {.lex_state = 2},
  [1079] = {.lex_state = 0},
  [1080] = {.lex_state = 0},
  [1081] = {.lex_state = 2},
  [1082] = {.lex_state = 0},
  [1083] = {.lex_state = 0},
  [1084] = {.lex_state = 55},
  [1085] = {.lex_state = 2},
  [1086] = {.lex_state = 55},
  [1087] = {.lex_state = 55},
  [1088] = {.lex_state = 2},
  [1089] = {.lex_state = 0},
  [1090] = {.lex_state = 55},
  [1091] = {.lex_state = 55},
  [1092] = {.lex_state = 55},
  [1093] = {.lex_state = 0},
  [1094] = {.lex_state = 55},
  [1095] = {.lex_state = 2},
  [1096] = {.lex_state = 0},
  [1097] = {.lex_state = 0},
  [1098] = {.lex_state = 0},
  [1099] = {.lex_state = 0},
  [1100] = {.lex_state = 2},
  [1101] = {.lex_state = 55},
  [1102] = {.lex_state = 55},
  [1103] = {.lex_state = 0},
  [1104] = {.lex_state = 55},
  [1105] = {.lex_state = 0},
  [1106] = {.lex_state = 55},
  [1107] = {.lex_state = 55},
  [1108] = {.lex_state = 55},
  [1109] = {.lex_state = 2},
  [1110] = {.lex_state = 2},
  [1111] = {.lex_state = 55},
  [1112] = {.lex_state = 55},
  [1113] = {.lex_state = 55},
  [1114] = {.lex_state = 0},
  [1115] = {.lex_state = 2},
  [1116] = {.lex_state = 0},
  [1117] = {.lex_state = 55},
  [1118] = {.lex_state = 58},
  [1119] = {.lex_state = 55},
  [1120] = {.lex_state = 55},
  [1121] = {.lex_state = 2},
  [1122] = {.lex_state = 55},
  [1123] = {.lex_state = 55},
  [1124] = {.lex_state = 55},
  [1125] = {.lex_state = 0},
  [1126] = {.lex_state = 55},
  [1127] = {.lex_state = 55},
  [1128] = {.lex_state = 58},
  [1129] = {.lex_state = 0},
  [1130] = {.lex_state = 0},
  [1131] = {.lex_state = 0},
  [1132] = {.lex_state = 2},
  [1133] = {.lex_state = 55},
  [1134] = {.lex_state = 2},
  [1135] = {.lex_state = 55},
  [1136] = {.lex_state = 0},
  [1137] = {.lex_state = 55},
  [1138] = {.lex_state = 0},
  [1139] = {.lex_state = 0},
  [1140] = {.lex_state = 55},
  [1141] = {.lex_state = 0},
  [1142] = {.lex_state = 55},
  [1143] = {.lex_state = 55},
  [1144] = {.lex_state = 0},
  [1145] = {.lex_state = 0},
  [1146] = {.lex_state = 0},
  [1147] = {.lex_state = 55},
  [1148] = {.lex_state = 0},
  [1149] = {.lex_state = 0},
  [1150] = {.lex_state = 0},
  [1151] = {.lex_state = 0},
  [1152] = {.lex_state = 0},
  [1153] = {.lex_state = 0},
  [1154] = {.lex_state = 0},
  [1155] = {.lex_state = 0},
  [1156] = {.lex_state = 0},
  [1157] = {.lex_state = 0},
  [1158] = {.lex_state = 0},
  [1159] = {.lex_state = 55},
  [1160] = {.lex_state = 0},
  [1161] = {.lex_state = 0},
  [1162] = {.lex_state = 55},
  [1163] = {.lex_state = 55},
  [1164] = {.lex_state = 0},
  [1165] = {.lex_state = 0},
  [1166] = {.lex_state = 55},
  [1167] = {.lex_state = 0},
  [1168] = {.lex_state = 0},
  [1169] = {.lex_state = 0},
  [1170] = {.lex_state = 0},
  [1171] = {.lex_state = 0},
  [1172] = {.lex_state = 0},
  [1173] = {.lex_state = 0},
  [1174] = {.lex_state = 0},
  [1175] = {.lex_state = 0},
  [1176] = {.lex_state = 0},
  [1177] = {.lex_state = 0},
  [1178] = {.lex_state = 0},
  [1179] = {.lex_state = 0},
  [1180] = {.lex_state = 0},
  [1181] = {.lex_state = 0},
  [1182] = {.lex_state = 0},
  [1183] = {.lex_state = 55},
  [1184] = {.lex_state = 0},
  [1185] = {.lex_state = 55},
  [1186] = {.lex_state = 0},
  [1187] = {.lex_state = 0},
  [1188] = {.lex_state = 0},
  [1189] = {.lex_state = 0},
  [1190] = {.lex_state = 55},
  [1191] = {.lex_state = 0},
  [1192] = {.lex_state = 0},
  [1193] = {.lex_state = 0},
  [1194] = {.lex_state = 0},
  [1195] = {.lex_state = 0},
  [1196] = {.lex_state = 0},
  [1197] = {.lex_state = 0},
  [1198] = {.lex_state = 0},
  [1199] = {.lex_state = 0},
  [1200] = {.lex_state = 0},
  [1201] = {.lex_state = 0},
  [1202] = {.lex_state = 0},
  [1203] = {.lex_state = 0},
  [1204] = {.lex_state = 55},
  [1205] = {.lex_state = 0},
  [1206] = {.lex_state = 55},
  [1207] = {.lex_state = 55},
  [1208] = {.lex_state = 0},
  [1209] = {.lex_state = 0},
  [1210] = {.lex_state = 0},
  [1211] = {.lex_state = 0},
  [1212] = {.lex_state = 55},
  [1213] = {.lex_state = 0},
  [1214] = {.lex_state = 0},
  [1215] = {.lex_state = 0},
  [1216] = {.lex_state = 0},
  [1217] = {.lex_state = 0},
  [1218] = {.lex_state = 0},
  [1219] = {.lex_state = 55},
  [1220] = {.lex_state = 0},
  [1221] = {.lex_state = 0},
  [1222] = {.lex_state = 0},
  [1223] = {.lex_state = 0},
  [1224] = {.lex_state = 0},
  [1225] = {.lex_state = 0},
  [1226] = {.lex_state = 0},
  [1227] = {.lex_state = 0},
  [1228] = {.lex_state = 0},
  [1229] = {.lex_state = 55},
  [1230] = {.lex_state = 0},
  [1231] = {.lex_state = 55},
  [1232] = {.lex_state = 0},
  [1233] = {.lex_state = 0},
  [1234] = {.lex_state = 0},
  [1235] = {.lex_state = 0},
  [1236] = {.lex_state = 0},
  [1237] = {.lex_state = 0},
  [1238] = {.lex_state = 0},
  [1239] = {.lex_state = 0},
  [1240] = {.lex_state = 0},
  [1241] = {.lex_state = 0},
  [1242] = {.lex_state = 0},
  [1243] = {.lex_state = 0},
  [1244] = {.lex_state = 0},
  [1245] = {.lex_state = 0},
  [1246] = {.lex_state = 0},
  [1247] = {.lex_state = 55},
  [1248] = {.lex_state = 0},
  [1249] = {.lex_state = 0},
  [1250] = {.lex_state = 0},
  [1251] = {.lex_state = 0},
  [1252] = {.lex_state = 0},
  [1253] = {.lex_state = 0},
  [1254] = {.lex_state = 0},
  [1255] = {.lex_state = 0},
  [1256] = {.lex_state = 0},
  [1257] = {.lex_state = 0},
  [1258] = {.lex_state = 0},
  [1259] = {.lex_state = 0},
  [1260] = {.lex_state = 0},
  [1261] = {.lex_state = 0},
  [1262] = {.lex_state = 0},
  [1263] = {.lex_state = 0},
  [1264] = {.lex_state = 58},
  [1265] = {.lex_state = 0},
  [1266] = {.lex_state = 0},
  [1267] = {.lex_state = 0},
  [1268] = {.lex_state = 0},
  [1269] = {.lex_state = 0},
  [1270] = {.lex_state = 0},
  [1271] = {.lex_state = 0},
  [1272] = {.lex_state = 0},
  [1273] = {.lex_state = 0},
  [1274] = {.lex_state = 0},
  [1275] = {.lex_state = 0},
  [1276] = {.lex_state = 0},
  [1277] = {.lex_state = 0},
  [1278] = {.lex_state = 0},
  [1279] = {.lex_state = 0},
  [1280] = {.lex_state = 0},
  [1281] = {.lex_state = 0},
  [1282] = {.lex_state = 0},
  [1283] = {.lex_state = 0},
  [1284] = {.lex_state = 0},
  [1285] = {.lex_state = 0},
  [1286] = {.lex_state = 0},
  [1287] = {.lex_state = 0},
  [1288] = {.lex_state = 0},
  [1289] = {.lex_state = 0},
  [1290] = {.lex_state = 0},
  [1291] = {.lex_state = 0},
  [1292] = {.lex_state = 0},
  [1293] = {.lex_state = 0},
  [1294] = {.lex_state = 0},
  [1295] = {.lex_state = 0},
  [1296] = {.lex_state = 0},
  [1297] = {.lex_state = 0},
  [1298] = {.lex_state = 0},
  [1299] = {.lex_state = 0},
  [1300] = {.lex_state = 0},
  [1301] = {.lex_state = 0},
  [1302] = {.lex_state = 0},
  [1303] = {.lex_state = 0},
  [1304] = {.lex_state = 0},
  [1305] = {.lex_state = 0},
  [1306] = {.lex_state = 0},
  [1307] = {.lex_state = 0},
  [1308] = {.lex_state = 0},
  [1309] = {.lex_state = 0},
  [1310] = {.lex_state = 0},
  [1311] = {.lex_state = 0},
  [1312] = {.lex_state = 0},
  [1313] = {.lex_state = 0},
  [1314] = {.lex_state = 0},
  [1315] = {.lex_state = 0},
  [1316] = {.lex_state = 0},
  [1317] = {.lex_state = 0},
  [1318] = {.lex_state = 0},
  [1319] = {.lex_state = 0},
  [1320] = {.lex_state = 0},
  [1321] = {.lex_state = 0},
  [1322] = {.lex_state = 0},
  [1323] = {.lex_state = 0},
  [1324] = {.lex_state = 0},
  [1325] = {.lex_state = 0},
  [1326] = {.lex_state = 0},
  [1327] = {.lex_state = 0},
  [1328] = {.lex_state = 0},
  [1329] = {.lex_state = 0},
  [1330] = {.lex_state = 0},
  [1331] = {.lex_state = 0},
  [1332] = {.lex_state = 0},
  [1333] = {.lex_state = 0},
  [1334] = {.lex_state = 0},
  [1335] = {.lex_state = 0},
  [1336] = {.lex_state = 0},
  [1337] = {.lex_state = 0},
  [1338] = {.lex_state = 0},
  [1339] = {.lex_state = 0},
  [1340] = {.lex_state = 0},
  [1341] = {.lex_state = 0},
  [1342] = {.lex_state = 0},
  [1343] = {.lex_state = 0},
  [1344] = {.lex_state = 0},
  [1345] = {.lex_state = 0},
  [1346] = {.lex_state = 0},
  [1347] = {.lex_state = 0},
  [1348] = {.lex_state = 0},
  [1349] = {.lex_state = 0},
  [1350] = {.lex_state = 0},
  [1351] = {.lex_state = 0},
  [1352] = {.lex_state = 0},
  [1353] = {.lex_state = 0},
  [1354] = {.lex_state = 0},
  [1355] = {.lex_state = 0},
  [1356] = {.lex_state = 0},
  [1357] = {.lex_state = 0},
  [1358] = {.lex_state = 140},
  [1359] = {.lex_state = 0},
  [1360] = {.lex_state = 0},
  [1361] = {.lex_state = 0},
  [1362] = {.lex_state = 0},
  [1363] = {.lex_state = 0},
  [1364] = {.lex_state = 0},
  [1365] = {.lex_state = 0},
  [1366] = {.lex_state = 0},
  [1367] = {.lex_state = 0},
  [1368] = {.lex_state = 0},
  [1369] = {.lex_state = 0},
  [1370] = {.lex_state = 0},
  [1371] = {.lex_state = 0},
  [1372] = {.lex_state = 0},
  [1373] = {.lex_state = 140},
  [1374] = {.lex_state = 0},
  [1375] = {.lex_state = 0},
  [1376] = {.lex_state = 0},
  [1377] = {.lex_state = 0},
  [1378] = {.lex_state = 0},
  [1379] = {.lex_state = 0},
  [1380] = {.lex_state = 0},
  [1381] = {.lex_state = 0},
  [1382] = {.lex_state = 0},
  [1383] = {.lex_state = 0},
  [1384] = {.lex_state = 140},
  [1385] = {.lex_state = 0},
  [1386] = {.lex_state = 0},
  [1387] = {.lex_state = 0},
  [1388] = {.lex_state = 0},
  [1389] = {.lex_state = 0},
  [1390] = {.lex_state = 0},
  [1391] = {.lex_state = 0},
  [1392] = {.lex_state = 0},
  [1393] = {.lex_state = 0},
  [1394] = {.lex_state = 140},
  [1395] = {.lex_state = 0},
  [1396] = {.lex_state = 0},
  [1397] = {.lex_state = 0},
  [1398] = {.lex_state = 0},
  [1399] = {.lex_state = 0},
  [1400] = {.lex_state = 0},
  [1401] = {.lex_state = 0},
  [1402] = {.lex_state = 0},
  [1403] = {.lex_state = 0},
  [1404] = {.lex_state = 140},
  [1405] = {.lex_state = 0},
  [1406] = {.lex_state = 0},
  [1407] = {.lex_state = 0},
  [1408] = {.lex_state = 0},
  [1409] = {.lex_state = 0},
  [1410] = {.lex_state = 0},
  [1411] = {.lex_state = 0},
  [1412] = {.lex_state = 140},
  [1413] = {.lex_state = 140},
  [1414] = {.lex_state = 0},
  [1415] = {.lex_state = 0},
  [1416] = {.lex_state = 0},
  [1417] = {.lex_state = 0},
  [1418] = {.lex_state = 0},
  [1419] = {.lex_state = 0},
  [1420] = {.lex_state = 0},
  [1421] = {.lex_state = 0},
  [1422] = {.lex_state = 0},
  [1423] = {.lex_state = 0},
  [1424] = {.lex_state = 0},
};

static const uint16_t ts_parse_table[LARGE_STATE_COUNT][SYMBOL_COUNT] = {
  [0] = {
    [ts_builtin_sym_end] = ACTIONS(1),
    [sym_identifier] = ACTIONS(1),
    [anon_sym_SEMI] = ACTIONS(1),
    [anon_sym_NULL] = ACTIONS(1),
    [anon_sym_package] = ACTIONS(1),
    [anon_sym_import] = ACTIONS(1),
    [anon_sym_DOT] = ACTIONS(1),
    [sym_blank_identifier] = ACTIONS(1),
    [anon_sym_LPAREN] = ACTIONS(1),
    [anon_sym_RPAREN] = ACTIONS(1),
    [anon_sym_const] = ACTIONS(1),
    [anon_sym_COMMA] = ACTIONS(1),
    [anon_sym_EQ] = ACTIONS(1),
    [anon_sym_var] = ACTIONS(1),
    [anon_sym_func] = ACTIONS(1),
    [anon_sym_LBRACK] = ACTIONS(1),
    [anon_sym_RBRACK] = ACTIONS(1),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1),
    [anon_sym_type] = ACTIONS(1),
    [anon_sym_STAR] = ACTIONS(1),
    [anon_sym_struct] = ACTIONS(1),
    [anon_sym_TILDE] = ACTIONS(1),
    [anon_sym_LBRACE] = ACTIONS(1),
    [anon_sym_RBRACE] = ACTIONS(1),
    [anon_sym_interface] = ACTIONS(1),
    [anon_sym_PIPE] = ACTIONS(1),
    [anon_sym_map] = ACTIONS(1),
    [anon_sym_chan] = ACTIONS(1),
    [anon_sym_LT_DASH] = ACTIONS(1),
    [anon_sym_COLON_EQ] = ACTIONS(1),
    [anon_sym_PLUS_PLUS] = ACTIONS(1),
    [anon_sym_DASH_DASH] = ACTIONS(1),
    [anon_sym_STAR_EQ] = ACTIONS(1),
    [anon_sym_SLASH_EQ] = ACTIONS(1),
    [anon_sym_PERCENT_EQ] = ACTIONS(1),
    [anon_sym_LT_LT_EQ] = ACTIONS(1),
    [anon_sym_GT_GT_EQ] = ACTIONS(1),
    [anon_sym_AMP_EQ] = ACTIONS(1),
    [anon_sym_AMP_CARET_EQ] = ACTIONS(1),
    [anon_sym_PLUS_EQ] = ACTIONS(1),
    [anon_sym_DASH_EQ] = ACTIONS(1),
    [anon_sym_PIPE_EQ] = ACTIONS(1),
    [anon_sym_CARET_EQ] = ACTIONS(1),
    [anon_sym_COLON] = ACTIONS(1),
    [anon_sym_fallthrough] = ACTIONS(1),
    [anon_sym_break] = ACTIONS(1),
    [anon_sym_continue] = ACTIONS(1),
    [anon_sym_goto] = ACTIONS(1),
    [anon_sym_return] = ACTIONS(1),
    [anon_sym_go] = ACTIONS(1),
    [anon_sym_defer] = ACTIONS(1),
    [anon_sym_if] = ACTIONS(1),
    [anon_sym_else] = ACTIONS(1),
    [anon_sym_for] = ACTIONS(1),
    [anon_sym_range] = ACTIONS(1),
    [anon_sym_switch] = ACTIONS(1),
    [anon_sym_case] = ACTIONS(1),
    [anon_sym_default] = ACTIONS(1),
    [anon_sym_select] = ACTIONS(1),
    [anon_sym_new] = ACTIONS(1),
    [anon_sym_make] = ACTIONS(1),
    [anon_sym_PLUS] = ACTIONS(1),
    [anon_sym_DASH] = ACTIONS(1),
    [anon_sym_BANG] = ACTIONS(1),
    [anon_sym_CARET] = ACTIONS(1),
    [anon_sym_AMP] = ACTIONS(1),
    [anon_sym_SLASH] = ACTIONS(1),
    [anon_sym_PERCENT] = ACTIONS(1),
    [anon_sym_LT_LT] = ACTIONS(1),
    [anon_sym_GT_GT] = ACTIONS(1),
    [anon_sym_AMP_CARET] = ACTIONS(1),
    [anon_sym_EQ_EQ] = ACTIONS(1),
    [anon_sym_BANG_EQ] = ACTIONS(1),
    [anon_sym_LT] = ACTIONS(1),
    [anon_sym_LT_EQ] = ACTIONS(1),
    [anon_sym_GT] = ACTIONS(1),
    [anon_sym_GT_EQ] = ACTIONS(1),
    [anon_sym_AMP_AMP] = ACTIONS(1),
    [anon_sym_PIPE_PIPE] = ACTIONS(1),
    [anon_sym_BQUOTE] = ACTIONS(1),
    [anon_sym_DQUOTE] = ACTIONS(1),
    [anon_sym_DQUOTE2] = ACTIONS(1),
    [sym_escape_sequence] = ACTIONS(1),
    [sym_int_literal] = ACTIONS(1),
    [sym_float_literal] = ACTIONS(1),
    [sym_imaginary_literal] = ACTIONS(1),
    [sym_rune_literal] = ACTIONS(1),
    [sym_nil] = ACTIONS(1),
    [sym_true] = ACTIONS(1),
    [sym_false] = ACTIONS(1),
    [sym_iota] = ACTIONS(1),
    [sym_comment] = ACTIONS(3),
  },
  [1] = {
    [sym_source_file] = STATE(1363),
    [sym_package_clause] = STATE(1087),
    [sym_import_declaration] = STATE(1087),
    [sym__declaration] = STATE(985),
    [sym_const_declaration] = STATE(985),
    [sym_var_declaration] = STATE(985),
    [sym_function_declaration] = STATE(1087),
    [sym_method_declaration] = STATE(1087),
    [sym_type_declaration] = STATE(985),
    [sym_expression_list] = STATE(804),
    [sym_parenthesized_type] = STATE(1303),
    [sym__simple_type] = STATE(1303),
    [sym_generic_type] = STATE(1131),
    [sym_pointer_type] = STATE(867),
    [sym_array_type] = STATE(1131),
    [sym_implicit_length_array_type] = STATE(1293),
    [sym_slice_type] = STATE(1131),
    [sym_struct_type] = STATE(1131),
    [sym_negated_type] = STATE(850),
    [sym_interface_type] = STATE(867),
    [sym_map_type] = STATE(1131),
    [sym_channel_type] = STATE(867),
    [sym_function_type] = STATE(867),
    [sym_block] = STATE(985),
    [sym__statement] = STATE(1231),
    [sym_empty_statement] = STATE(985),
    [sym__simple_statement] = STATE(985),
    [sym_expression_statement] = STATE(937),
    [sym_send_statement] = STATE(937),
    [sym_inc_statement] = STATE(937),
    [sym_dec_statement] = STATE(937),
    [sym_assignment_statement] = STATE(937),
    [sym_short_var_declaration] = STATE(937),
    [sym_labeled_statement] = STATE(985),
    [sym_fallthrough_statement] = STATE(985),
    [sym_break_statement] = STATE(985),
    [sym_continue_statement] = STATE(985),
    [sym_goto_statement] = STATE(985),
    [sym_return_statement] = STATE(985),
    [sym_go_statement] = STATE(985),
    [sym_defer_statement] = STATE(985),
    [sym_if_statement] = STATE(985),
    [sym_for_statement] = STATE(985),
    [sym_expression_switch_statement] = STATE(985),
    [sym_type_switch_statement] = STATE(985),
    [sym_select_statement] = STATE(985),
    [sym__expression] = STATE(240),
    [sym_parenthesized_expression] = STATE(263),
    [sym_call_expression] = STATE(263),
    [sym_selector_expression] = STATE(263),
    [sym_index_expression] = STATE(263),
    [sym_slice_expression] = STATE(263),
    [sym_type_assertion_expression] = STATE(263),
    [sym_type_conversion_expression] = STATE(263),
    [sym_type_instantiation_expression] = STATE(263),
    [sym_composite_literal] = STATE(263),
    [sym_func_literal] = STATE(263),
    [sym_unary_expression] = STATE(263),
    [sym_binary_expression] = STATE(263),
    [sym_qualified_type] = STATE(1027),
    [sym_raw_string_literal] = STATE(263),
    [sym_interpreted_string_literal] = STATE(263),
    [aux_sym_source_file_repeat1] = STATE(3),
    [ts_builtin_sym_end] = ACTIONS(5),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_SEMI] = ACTIONS(9),
    [anon_sym_package] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_var] = ACTIONS(19),
    [anon_sym_func] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_type] = ACTIONS(25),
    [anon_sym_STAR] = ACTIONS(27),
    [anon_sym_struct] = ACTIONS(29),
    [anon_sym_TILDE] = ACTIONS(31),
    [anon_sym_LBRACE] = ACTIONS(33),
    [anon_sym_interface] = ACTIONS(35),
    [anon_sym_map] = ACTIONS(37),
    [anon_sym_chan] = ACTIONS(39),
    [anon_sym_LT_DASH] = ACTIONS(41),
    [anon_sym_fallthrough] = ACTIONS(43),
    [anon_sym_break] = ACTIONS(45),
    [anon_sym_continue] = ACTIONS(47),
    [anon_sym_goto] = ACTIONS(49),
    [anon_sym_return] = ACTIONS(51),
    [anon_sym_go] = ACTIONS(53),
    [anon_sym_defer] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_for] = ACTIONS(59),
    [anon_sym_switch] = ACTIONS(61),
    [anon_sym_select] = ACTIONS(63),
    [anon_sym_new] = ACTIONS(65),
    [anon_sym_make] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(67),
    [anon_sym_DASH] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_CARET] = ACTIONS(67),
    [anon_sym_AMP] = ACTIONS(67),
    [anon_sym_BQUOTE] = ACTIONS(69),
    [anon_sym_DQUOTE] = ACTIONS(71),
    [sym_int_literal] = ACTIONS(73),
    [sym_float_literal] = ACTIONS(73),
    [sym_imaginary_literal] = ACTIONS(75),
    [sym_rune_literal] = ACTIONS(75),
    [sym_nil] = ACTIONS(73),
    [sym_true] = ACTIONS(73),
    [sym_false] = ACTIONS(73),
    [sym_iota] = ACTIONS(73),
    [sym_comment] = ACTIONS(3),
  },
  [2] = {
    [sym_package_clause] = STATE(1231),
    [sym_import_declaration] = STATE(1231),
    [sym__declaration] = STATE(985),
    [sym_const_declaration] = STATE(985),
    [sym_var_declaration] = STATE(985),
    [sym_function_declaration] = STATE(1231),
    [sym_method_declaration] = STATE(1231),
    [sym_type_declaration] = STATE(985),
    [sym_expression_list] = STATE(804),
    [sym_parenthesized_type] = STATE(1303),
    [sym__simple_type] = STATE(1303),
    [sym_generic_type] = STATE(1131),
    [sym_pointer_type] = STATE(867),
    [sym_array_type] = STATE(1131),
    [sym_implicit_length_array_type] = STATE(1293),
    [sym_slice_type] = STATE(1131),
    [sym_struct_type] = STATE(1131),
    [sym_negated_type] = STATE(850),
    [sym_interface_type] = STATE(867),
    [sym_map_type] = STATE(1131),
    [sym_channel_type] = STATE(867),
    [sym_function_type] = STATE(867),
    [sym_block] = STATE(985),
    [sym__statement] = STATE(1231),
    [sym_empty_statement] = STATE(985),
    [sym__simple_statement] = STATE(985),
    [sym_expression_statement] = STATE(937),
    [sym_send_statement] = STATE(937),
    [sym_inc_statement] = STATE(937),
    [sym_dec_statement] = STATE(937),
    [sym_assignment_statement] = STATE(937),
    [sym_short_var_declaration] = STATE(937),
    [sym_labeled_statement] = STATE(985),
    [sym_fallthrough_statement] = STATE(985),
    [sym_break_statement] = STATE(985),
    [sym_continue_statement] = STATE(985),
    [sym_goto_statement] = STATE(985),
    [sym_return_statement] = STATE(985),
    [sym_go_statement] = STATE(985),
    [sym_defer_statement] = STATE(985),
    [sym_if_statement] = STATE(985),
    [sym_for_statement] = STATE(985),
    [sym_expression_switch_statement] = STATE(985),
    [sym_type_switch_statement] = STATE(985),
    [sym_select_statement] = STATE(985),
    [sym__expression] = STATE(240),
    [sym_parenthesized_expression] = STATE(263),
    [sym_call_expression] = STATE(263),
    [sym_selector_expression] = STATE(263),
    [sym_index_expression] = STATE(263),
    [sym_slice_expression] = STATE(263),
    [sym_type_assertion_expression] = STATE(263),
    [sym_type_conversion_expression] = STATE(263),
    [sym_type_instantiation_expression] = STATE(263),
    [sym_composite_literal] = STATE(263),
    [sym_func_literal] = STATE(263),
    [sym_unary_expression] = STATE(263),
    [sym_binary_expression] = STATE(263),
    [sym_qualified_type] = STATE(1027),
    [sym_raw_string_literal] = STATE(263),
    [sym_interpreted_string_literal] = STATE(263),
    [aux_sym_source_file_repeat1] = STATE(2),
    [ts_builtin_sym_end] = ACTIONS(77),
    [sym_identifier] = ACTIONS(79),
    [anon_sym_SEMI] = ACTIONS(82),
    [anon_sym_package] = ACTIONS(85),
    [anon_sym_import] = ACTIONS(88),
    [anon_sym_LPAREN] = ACTIONS(91),
    [anon_sym_const] = ACTIONS(94),
    [anon_sym_var] = ACTIONS(97),
    [anon_sym_func] = ACTIONS(100),
    [anon_sym_LBRACK] = ACTIONS(103),
    [anon_sym_type] = ACTIONS(106),
    [anon_sym_STAR] = ACTIONS(109),
    [anon_sym_struct] = ACTIONS(112),
    [anon_sym_TILDE] = ACTIONS(115),
    [anon_sym_LBRACE] = ACTIONS(118),
    [anon_sym_interface] = ACTIONS(121),
    [anon_sym_map] = ACTIONS(124),
    [anon_sym_chan] = ACTIONS(127),
    [anon_sym_LT_DASH] = ACTIONS(130),
    [anon_sym_fallthrough] = ACTIONS(133),
    [anon_sym_break] = ACTIONS(136),
    [anon_sym_continue] = ACTIONS(139),
    [anon_sym_goto] = ACTIONS(142),
    [anon_sym_return] = ACTIONS(145),
    [anon_sym_go] = ACTIONS(148),
    [anon_sym_defer] = ACTIONS(151),
    [anon_sym_if] = ACTIONS(154),
    [anon_sym_for] = ACTIONS(157),
    [anon_sym_switch] = ACTIONS(160),
    [anon_sym_select] = ACTIONS(163),
    [anon_sym_new] = ACTIONS(166),
    [anon_sym_make] = ACTIONS(166),
    [anon_sym_PLUS] = ACTIONS(169),
    [anon_sym_DASH] = ACTIONS(169),
    [anon_sym_BANG] = ACTIONS(169),
    [anon_sym_CARET] = ACTIONS(169),
    [anon_sym_AMP] = ACTIONS(169),
    [anon_sym_BQUOTE] = ACTIONS(172),
    [anon_sym_DQUOTE] = ACTIONS(175),
    [sym_int_literal] = ACTIONS(178),
    [sym_float_literal] = ACTIONS(178),
    [sym_imaginary_literal] = ACTIONS(181),
    [sym_rune_literal] = ACTIONS(181),
    [sym_nil] = ACTIONS(178),
    [sym_true] = ACTIONS(178),
    [sym_false] = ACTIONS(178),
    [sym_iota] = ACTIONS(178),
    [sym_comment] = ACTIONS(3),
  },
  [3] = {
    [sym_package_clause] = STATE(1086),
    [sym_import_declaration] = STATE(1086),
    [sym__declaration] = STATE(985),
    [sym_const_declaration] = STATE(985),
    [sym_var_declaration] = STATE(985),
    [sym_function_declaration] = STATE(1086),
    [sym_method_declaration] = STATE(1086),
    [sym_type_declaration] = STATE(985),
    [sym_expression_list] = STATE(804),
    [sym_parenthesized_type] = STATE(1303),
    [sym__simple_type] = STATE(1303),
    [sym_generic_type] = STATE(1131),
    [sym_pointer_type] = STATE(867),
    [sym_array_type] = STATE(1131),
    [sym_implicit_length_array_type] = STATE(1293),
    [sym_slice_type] = STATE(1131),
    [sym_struct_type] = STATE(1131),
    [sym_negated_type] = STATE(850),
    [sym_interface_type] = STATE(867),
    [sym_map_type] = STATE(1131),
    [sym_channel_type] = STATE(867),
    [sym_function_type] = STATE(867),
    [sym_block] = STATE(985),
    [sym__statement] = STATE(1231),
    [sym_empty_statement] = STATE(985),
    [sym__simple_statement] = STATE(985),
    [sym_expression_statement] = STATE(937),
    [sym_send_statement] = STATE(937),
    [sym_inc_statement] = STATE(937),
    [sym_dec_statement] = STATE(937),
    [sym_assignment_statement] = STATE(937),
    [sym_short_var_declaration] = STATE(937),
    [sym_labeled_statement] = STATE(985),
    [sym_fallthrough_statement] = STATE(985),
    [sym_break_statement] = STATE(985),
    [sym_continue_statement] = STATE(985),
    [sym_goto_statement] = STATE(985),
    [sym_return_statement] = STATE(985),
    [sym_go_statement] = STATE(985),
    [sym_defer_statement] = STATE(985),
    [sym_if_statement] = STATE(985),
    [sym_for_statement] = STATE(985),
    [sym_expression_switch_statement] = STATE(985),
    [sym_type_switch_statement] = STATE(985),
    [sym_select_statement] = STATE(985),
    [sym__expression] = STATE(240),
    [sym_parenthesized_expression] = STATE(263),
    [sym_call_expression] = STATE(263),
    [sym_selector_expression] = STATE(263),
    [sym_index_expression] = STATE(263),
    [sym_slice_expression] = STATE(263),
    [sym_type_assertion_expression] = STATE(263),
    [sym_type_conversion_expression] = STATE(263),
    [sym_type_instantiation_expression] = STATE(263),
    [sym_composite_literal] = STATE(263),
    [sym_func_literal] = STATE(263),
    [sym_unary_expression] = STATE(263),
    [sym_binary_expression] = STATE(263),
    [sym_qualified_type] = STATE(1027),
    [sym_raw_string_literal] = STATE(263),
    [sym_interpreted_string_literal] = STATE(263),
    [aux_sym_source_file_repeat1] = STATE(2),
    [ts_builtin_sym_end] = ACTIONS(184),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_SEMI] = ACTIONS(9),
    [anon_sym_package] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_var] = ACTIONS(19),
    [anon_sym_func] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_type] = ACTIONS(25),
    [anon_sym_STAR] = ACTIONS(27),
    [anon_sym_struct] = ACTIONS(29),
    [anon_sym_TILDE] = ACTIONS(31),
    [anon_sym_LBRACE] = ACTIONS(33),
    [anon_sym_interface] = ACTIONS(35),
    [anon_sym_map] = ACTIONS(37),
    [anon_sym_chan] = ACTIONS(39),
    [anon_sym_LT_DASH] = ACTIONS(41),
    [anon_sym_fallthrough] = ACTIONS(43),
    [anon_sym_break] = ACTIONS(45),
    [anon_sym_continue] = ACTIONS(47),
    [anon_sym_goto] = ACTIONS(49),
    [anon_sym_return] = ACTIONS(51),
    [anon_sym_go] = ACTIONS(53),
    [anon_sym_defer] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_for] = ACTIONS(59),
    [anon_sym_switch] = ACTIONS(61),
    [anon_sym_select] = ACTIONS(63),
    [anon_sym_new] = ACTIONS(65),
    [anon_sym_make] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(67),
    [anon_sym_DASH] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_CARET] = ACTIONS(67),
    [anon_sym_AMP] = ACTIONS(67),
    [anon_sym_BQUOTE] = ACTIONS(69),
    [anon_sym_DQUOTE] = ACTIONS(71),
    [sym_int_literal] = ACTIONS(73),
    [sym_float_literal] = ACTIONS(73),
    [sym_imaginary_literal] = ACTIONS(75),
    [sym_rune_literal] = ACTIONS(75),
    [sym_nil] = ACTIONS(73),
    [sym_true] = ACTIONS(73),
    [sym_false] = ACTIONS(73),
    [sym_iota] = ACTIONS(73),
    [sym_comment] = ACTIONS(3),
  },
  [4] = {
    [sym__declaration] = STATE(985),
    [sym_const_declaration] = STATE(985),
    [sym_var_declaration] = STATE(985),
    [sym_type_declaration] = STATE(985),
    [sym_expression_list] = STATE(804),
    [sym_parenthesized_type] = STATE(1303),
    [sym__simple_type] = STATE(1303),
    [sym_generic_type] = STATE(1131),
    [sym_pointer_type] = STATE(867),
    [sym_array_type] = STATE(1131),
    [sym_implicit_length_array_type] = STATE(1293),
    [sym_slice_type] = STATE(1131),
    [sym_struct_type] = STATE(1131),
    [sym_negated_type] = STATE(850),
    [sym_interface_type] = STATE(867),
    [sym_map_type] = STATE(1131),
    [sym_channel_type] = STATE(867),
    [sym_function_type] = STATE(867),
    [sym_block] = STATE(985),
    [sym__statement_list] = STATE(1248),
    [sym__statement] = STATE(913),
    [sym_empty_statement] = STATE(985),
    [sym__simple_statement] = STATE(985),
    [sym_expression_statement] = STATE(937),
    [sym_send_statement] = STATE(937),
    [sym_inc_statement] = STATE(937),
    [sym_dec_statement] = STATE(937),
    [sym_assignment_statement] = STATE(937),
    [sym_short_var_declaration] = STATE(937),
    [sym_labeled_statement] = STATE(985),
    [sym_empty_labeled_statement] = STATE(1248),
    [sym_fallthrough_statement] = STATE(985),
    [sym_break_statement] = STATE(985),
    [sym_continue_statement] = STATE(985),
    [sym_goto_statement] = STATE(985),
    [sym_return_statement] = STATE(985),
    [sym_go_statement] = STATE(985),
    [sym_defer_statement] = STATE(985),
    [sym_if_statement] = STATE(985),
    [sym_for_statement] = STATE(985),
    [sym_expression_switch_statement] = STATE(985),
    [sym_type_switch_statement] = STATE(985),
    [sym_select_statement] = STATE(985),
    [sym__expression] = STATE(240),
    [sym_parenthesized_expression] = STATE(263),
    [sym_call_expression] = STATE(263),
    [sym_selector_expression] = STATE(263),
    [sym_index_expression] = STATE(263),
    [sym_slice_expression] = STATE(263),
    [sym_type_assertion_expression] = STATE(263),
    [sym_type_conversion_expression] = STATE(263),
    [sym_type_instantiation_expression] = STATE(263),
    [sym_composite_literal] = STATE(263),
    [sym_func_literal] = STATE(263),
    [sym_unary_expression] = STATE(263),
    [sym_binary_expression] = STATE(263),
    [sym_qualified_type] = STATE(1027),
    [sym_raw_string_literal] = STATE(263),
    [sym_interpreted_string_literal] = STATE(263),
    [sym_identifier] = ACTIONS(186),
    [anon_sym_SEMI] = ACTIONS(9),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_var] = ACTIONS(19),
    [anon_sym_func] = ACTIONS(188),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_type] = ACTIONS(25),
    [anon_sym_STAR] = ACTIONS(27),
    [anon_sym_struct] = ACTIONS(29),
    [anon_sym_TILDE] = ACTIONS(31),
    [anon_sym_LBRACE] = ACTIONS(33),
    [anon_sym_RBRACE] = ACTIONS(190),
    [anon_sym_interface] = ACTIONS(35),
    [anon_sym_map] = ACTIONS(37),
    [anon_sym_chan] = ACTIONS(39),
    [anon_sym_LT_DASH] = ACTIONS(41),
    [anon_sym_fallthrough] = ACTIONS(43),
    [anon_sym_break] = ACTIONS(45),
    [anon_sym_continue] = ACTIONS(47),
    [anon_sym_goto] = ACTIONS(49),
    [anon_sym_return] = ACTIONS(51),
    [anon_sym_go] = ACTIONS(53),
    [anon_sym_defer] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_for] = ACTIONS(59),
    [anon_sym_switch] = ACTIONS(61),
    [anon_sym_case] = ACTIONS(192),
    [anon_sym_default] = ACTIONS(192),
    [anon_sym_select] = ACTIONS(63),
    [anon_sym_new] = ACTIONS(65),
    [anon_sym_make] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(67),
    [anon_sym_DASH] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_CARET] = ACTIONS(67),
    [anon_sym_AMP] = ACTIONS(67),
    [anon_sym_BQUOTE] = ACTIONS(69),
    [anon_sym_DQUOTE] = ACTIONS(71),
    [sym_int_literal] = ACTIONS(73),
    [sym_float_literal] = ACTIONS(73),
    [sym_imaginary_literal] = ACTIONS(75),
    [sym_rune_literal] = ACTIONS(75),
    [sym_nil] = ACTIONS(73),
    [sym_true] = ACTIONS(73),
    [sym_false] = ACTIONS(73),
    [sym_iota] = ACTIONS(73),
    [sym_comment] = ACTIONS(3),
  },
  [5] = {
    [sym__declaration] = STATE(985),
    [sym_const_declaration] = STATE(985),
    [sym_var_declaration] = STATE(985),
    [sym_type_declaration] = STATE(985),
    [sym_expression_list] = STATE(804),
    [sym_parenthesized_type] = STATE(1303),
    [sym__simple_type] = STATE(1303),
    [sym_generic_type] = STATE(1131),
    [sym_pointer_type] = STATE(867),
    [sym_array_type] = STATE(1131),
    [sym_implicit_length_array_type] = STATE(1293),
    [sym_slice_type] = STATE(1131),
    [sym_struct_type] = STATE(1131),
    [sym_negated_type] = STATE(850),
    [sym_interface_type] = STATE(867),
    [sym_map_type] = STATE(1131),
    [sym_channel_type] = STATE(867),
    [sym_function_type] = STATE(867),
    [sym_block] = STATE(985),
    [sym__statement_list] = STATE(1244),
    [sym__statement] = STATE(913),
    [sym_empty_statement] = STATE(985),
    [sym__simple_statement] = STATE(985),
    [sym_expression_statement] = STATE(937),
    [sym_send_statement] = STATE(937),
    [sym_inc_statement] = STATE(937),
    [sym_dec_statement] = STATE(937),
    [sym_assignment_statement] = STATE(937),
    [sym_short_var_declaration] = STATE(937),
    [sym_labeled_statement] = STATE(985),
    [sym_empty_labeled_statement] = STATE(1244),
    [sym_fallthrough_statement] = STATE(985),
    [sym_break_statement] = STATE(985),
    [sym_continue_statement] = STATE(985),
    [sym_goto_statement] = STATE(985),
    [sym_return_statement] = STATE(985),
    [sym_go_statement] = STATE(985),
    [sym_defer_statement] = STATE(985),
    [sym_if_statement] = STATE(985),
    [sym_for_statement] = STATE(985),
    [sym_expression_switch_statement] = STATE(985),
    [sym_type_switch_statement] = STATE(985),
    [sym_select_statement] = STATE(985),
    [sym__expression] = STATE(240),
    [sym_parenthesized_expression] = STATE(263),
    [sym_call_expression] = STATE(263),
    [sym_selector_expression] = STATE(263),
    [sym_index_expression] = STATE(263),
    [sym_slice_expression] = STATE(263),
    [sym_type_assertion_expression] = STATE(263),
    [sym_type_conversion_expression] = STATE(263),
    [sym_type_instantiation_expression] = STATE(263),
    [sym_composite_literal] = STATE(263),
    [sym_func_literal] = STATE(263),
    [sym_unary_expression] = STATE(263),
    [sym_binary_expression] = STATE(263),
    [sym_qualified_type] = STATE(1027),
    [sym_raw_string_literal] = STATE(263),
    [sym_interpreted_string_literal] = STATE(263),
    [sym_identifier] = ACTIONS(186),
    [anon_sym_SEMI] = ACTIONS(9),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_var] = ACTIONS(19),
    [anon_sym_func] = ACTIONS(188),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_type] = ACTIONS(25),
    [anon_sym_STAR] = ACTIONS(27),
    [anon_sym_struct] = ACTIONS(29),
    [anon_sym_TILDE] = ACTIONS(31),
    [anon_sym_LBRACE] = ACTIONS(33),
    [anon_sym_RBRACE] = ACTIONS(194),
    [anon_sym_interface] = ACTIONS(35),
    [anon_sym_map] = ACTIONS(37),
    [anon_sym_chan] = ACTIONS(39),
    [anon_sym_LT_DASH] = ACTIONS(41),
    [anon_sym_fallthrough] = ACTIONS(43),
    [anon_sym_break] = ACTIONS(45),
    [anon_sym_continue] = ACTIONS(47),
    [anon_sym_goto] = ACTIONS(49),
    [anon_sym_return] = ACTIONS(51),
    [anon_sym_go] = ACTIONS(53),
    [anon_sym_defer] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_for] = ACTIONS(59),
    [anon_sym_switch] = ACTIONS(61),
    [anon_sym_case] = ACTIONS(196),
    [anon_sym_default] = ACTIONS(196),
    [anon_sym_select] = ACTIONS(63),
    [anon_sym_new] = ACTIONS(65),
    [anon_sym_make] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(67),
    [anon_sym_DASH] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_CARET] = ACTIONS(67),
    [anon_sym_AMP] = ACTIONS(67),
    [anon_sym_BQUOTE] = ACTIONS(69),
    [anon_sym_DQUOTE] = ACTIONS(71),
    [sym_int_literal] = ACTIONS(73),
    [sym_float_literal] = ACTIONS(73),
    [sym_imaginary_literal] = ACTIONS(75),
    [sym_rune_literal] = ACTIONS(75),
    [sym_nil] = ACTIONS(73),
    [sym_true] = ACTIONS(73),
    [sym_false] = ACTIONS(73),
    [sym_iota] = ACTIONS(73),
    [sym_comment] = ACTIONS(3),
  },
  [6] = {
    [sym__declaration] = STATE(985),
    [sym_const_declaration] = STATE(985),
    [sym_var_declaration] = STATE(985),
    [sym_type_declaration] = STATE(985),
    [sym_expression_list] = STATE(804),
    [sym_parenthesized_type] = STATE(1303),
    [sym__simple_type] = STATE(1303),
    [sym_generic_type] = STATE(1131),
    [sym_pointer_type] = STATE(867),
    [sym_array_type] = STATE(1131),
    [sym_implicit_length_array_type] = STATE(1293),
    [sym_slice_type] = STATE(1131),
    [sym_struct_type] = STATE(1131),
    [sym_negated_type] = STATE(850),
    [sym_interface_type] = STATE(867),
    [sym_map_type] = STATE(1131),
    [sym_channel_type] = STATE(867),
    [sym_function_type] = STATE(867),
    [sym_block] = STATE(985),
    [sym__statement_list] = STATE(1184),
    [sym__statement] = STATE(913),
    [sym_empty_statement] = STATE(985),
    [sym__simple_statement] = STATE(985),
    [sym_expression_statement] = STATE(937),
    [sym_send_statement] = STATE(937),
    [sym_inc_statement] = STATE(937),
    [sym_dec_statement] = STATE(937),
    [sym_assignment_statement] = STATE(937),
    [sym_short_var_declaration] = STATE(937),
    [sym_labeled_statement] = STATE(985),
    [sym_empty_labeled_statement] = STATE(1184),
    [sym_fallthrough_statement] = STATE(985),
    [sym_break_statement] = STATE(985),
    [sym_continue_statement] = STATE(985),
    [sym_goto_statement] = STATE(985),
    [sym_return_statement] = STATE(985),
    [sym_go_statement] = STATE(985),
    [sym_defer_statement] = STATE(985),
    [sym_if_statement] = STATE(985),
    [sym_for_statement] = STATE(985),
    [sym_expression_switch_statement] = STATE(985),
    [sym_type_switch_statement] = STATE(985),
    [sym_select_statement] = STATE(985),
    [sym__expression] = STATE(240),
    [sym_parenthesized_expression] = STATE(263),
    [sym_call_expression] = STATE(263),
    [sym_selector_expression] = STATE(263),
    [sym_index_expression] = STATE(263),
    [sym_slice_expression] = STATE(263),
    [sym_type_assertion_expression] = STATE(263),
    [sym_type_conversion_expression] = STATE(263),
    [sym_type_instantiation_expression] = STATE(263),
    [sym_composite_literal] = STATE(263),
    [sym_func_literal] = STATE(263),
    [sym_unary_expression] = STATE(263),
    [sym_binary_expression] = STATE(263),
    [sym_qualified_type] = STATE(1027),
    [sym_raw_string_literal] = STATE(263),
    [sym_interpreted_string_literal] = STATE(263),
    [sym_identifier] = ACTIONS(186),
    [anon_sym_SEMI] = ACTIONS(9),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_var] = ACTIONS(19),
    [anon_sym_func] = ACTIONS(188),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_type] = ACTIONS(25),
    [anon_sym_STAR] = ACTIONS(27),
    [anon_sym_struct] = ACTIONS(29),
    [anon_sym_TILDE] = ACTIONS(31),
    [anon_sym_LBRACE] = ACTIONS(33),
    [anon_sym_RBRACE] = ACTIONS(198),
    [anon_sym_interface] = ACTIONS(35),
    [anon_sym_map] = ACTIONS(37),
    [anon_sym_chan] = ACTIONS(39),
    [anon_sym_LT_DASH] = ACTIONS(41),
    [anon_sym_fallthrough] = ACTIONS(43),
    [anon_sym_break] = ACTIONS(45),
    [anon_sym_continue] = ACTIONS(47),
    [anon_sym_goto] = ACTIONS(49),
    [anon_sym_return] = ACTIONS(51),
    [anon_sym_go] = ACTIONS(53),
    [anon_sym_defer] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_for] = ACTIONS(59),
    [anon_sym_switch] = ACTIONS(61),
    [anon_sym_case] = ACTIONS(200),
    [anon_sym_default] = ACTIONS(200),
    [anon_sym_select] = ACTIONS(63),
    [anon_sym_new] = ACTIONS(65),
    [anon_sym_make] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(67),
    [anon_sym_DASH] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_CARET] = ACTIONS(67),
    [anon_sym_AMP] = ACTIONS(67),
    [anon_sym_BQUOTE] = ACTIONS(69),
    [anon_sym_DQUOTE] = ACTIONS(71),
    [sym_int_literal] = ACTIONS(73),
    [sym_float_literal] = ACTIONS(73),
    [sym_imaginary_literal] = ACTIONS(75),
    [sym_rune_literal] = ACTIONS(75),
    [sym_nil] = ACTIONS(73),
    [sym_true] = ACTIONS(73),
    [sym_false] = ACTIONS(73),
    [sym_iota] = ACTIONS(73),
    [sym_comment] = ACTIONS(3),
  },
  [7] = {
    [sym__declaration] = STATE(985),
    [sym_const_declaration] = STATE(985),
    [sym_var_declaration] = STATE(985),
    [sym_type_declaration] = STATE(985),
    [sym_expression_list] = STATE(804),
    [sym_parenthesized_type] = STATE(1303),
    [sym__simple_type] = STATE(1303),
    [sym_generic_type] = STATE(1131),
    [sym_pointer_type] = STATE(867),
    [sym_array_type] = STATE(1131),
    [sym_implicit_length_array_type] = STATE(1293),
    [sym_slice_type] = STATE(1131),
    [sym_struct_type] = STATE(1131),
    [sym_negated_type] = STATE(850),
    [sym_interface_type] = STATE(867),
    [sym_map_type] = STATE(1131),
    [sym_channel_type] = STATE(867),
    [sym_function_type] = STATE(867),
    [sym_block] = STATE(985),
    [sym__statement_list] = STATE(1237),
    [sym__statement] = STATE(913),
    [sym_empty_statement] = STATE(985),
    [sym__simple_statement] = STATE(985),
    [sym_expression_statement] = STATE(937),
    [sym_send_statement] = STATE(937),
    [sym_inc_statement] = STATE(937),
    [sym_dec_statement] = STATE(937),
    [sym_assignment_statement] = STATE(937),
    [sym_short_var_declaration] = STATE(937),
    [sym_labeled_statement] = STATE(985),
    [sym_empty_labeled_statement] = STATE(1237),
    [sym_fallthrough_statement] = STATE(985),
    [sym_break_statement] = STATE(985),
    [sym_continue_statement] = STATE(985),
    [sym_goto_statement] = STATE(985),
    [sym_return_statement] = STATE(985),
    [sym_go_statement] = STATE(985),
    [sym_defer_statement] = STATE(985),
    [sym_if_statement] = STATE(985),
    [sym_for_statement] = STATE(985),
    [sym_expression_switch_statement] = STATE(985),
    [sym_type_switch_statement] = STATE(985),
    [sym_select_statement] = STATE(985),
    [sym__expression] = STATE(240),
    [sym_parenthesized_expression] = STATE(263),
    [sym_call_expression] = STATE(263),
    [sym_selector_expression] = STATE(263),
    [sym_index_expression] = STATE(263),
    [sym_slice_expression] = STATE(263),
    [sym_type_assertion_expression] = STATE(263),
    [sym_type_conversion_expression] = STATE(263),
    [sym_type_instantiation_expression] = STATE(263),
    [sym_composite_literal] = STATE(263),
    [sym_func_literal] = STATE(263),
    [sym_unary_expression] = STATE(263),
    [sym_binary_expression] = STATE(263),
    [sym_qualified_type] = STATE(1027),
    [sym_raw_string_literal] = STATE(263),
    [sym_interpreted_string_literal] = STATE(263),
    [sym_identifier] = ACTIONS(186),
    [anon_sym_SEMI] = ACTIONS(9),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_var] = ACTIONS(19),
    [anon_sym_func] = ACTIONS(188),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_type] = ACTIONS(25),
    [anon_sym_STAR] = ACTIONS(27),
    [anon_sym_struct] = ACTIONS(29),
    [anon_sym_TILDE] = ACTIONS(31),
    [anon_sym_LBRACE] = ACTIONS(33),
    [anon_sym_RBRACE] = ACTIONS(202),
    [anon_sym_interface] = ACTIONS(35),
    [anon_sym_map] = ACTIONS(37),
    [anon_sym_chan] = ACTIONS(39),
    [anon_sym_LT_DASH] = ACTIONS(41),
    [anon_sym_fallthrough] = ACTIONS(43),
    [anon_sym_break] = ACTIONS(45),
    [anon_sym_continue] = ACTIONS(47),
    [anon_sym_goto] = ACTIONS(49),
    [anon_sym_return] = ACTIONS(51),
    [anon_sym_go] = ACTIONS(53),
    [anon_sym_defer] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_for] = ACTIONS(59),
    [anon_sym_switch] = ACTIONS(61),
    [anon_sym_case] = ACTIONS(204),
    [anon_sym_default] = ACTIONS(204),
    [anon_sym_select] = ACTIONS(63),
    [anon_sym_new] = ACTIONS(65),
    [anon_sym_make] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(67),
    [anon_sym_DASH] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_CARET] = ACTIONS(67),
    [anon_sym_AMP] = ACTIONS(67),
    [anon_sym_BQUOTE] = ACTIONS(69),
    [anon_sym_DQUOTE] = ACTIONS(71),
    [sym_int_literal] = ACTIONS(73),
    [sym_float_literal] = ACTIONS(73),
    [sym_imaginary_literal] = ACTIONS(75),
    [sym_rune_literal] = ACTIONS(75),
    [sym_nil] = ACTIONS(73),
    [sym_true] = ACTIONS(73),
    [sym_false] = ACTIONS(73),
    [sym_iota] = ACTIONS(73),
    [sym_comment] = ACTIONS(3),
  },
  [8] = {
    [sym__declaration] = STATE(985),
    [sym_const_declaration] = STATE(985),
    [sym_var_declaration] = STATE(985),
    [sym_type_declaration] = STATE(985),
    [sym_expression_list] = STATE(804),
    [sym_parenthesized_type] = STATE(1303),
    [sym__simple_type] = STATE(1303),
    [sym_generic_type] = STATE(1131),
    [sym_pointer_type] = STATE(867),
    [sym_array_type] = STATE(1131),
    [sym_implicit_length_array_type] = STATE(1293),
    [sym_slice_type] = STATE(1131),
    [sym_struct_type] = STATE(1131),
    [sym_negated_type] = STATE(850),
    [sym_interface_type] = STATE(867),
    [sym_map_type] = STATE(1131),
    [sym_channel_type] = STATE(867),
    [sym_function_type] = STATE(867),
    [sym_block] = STATE(985),
    [sym__statement_list] = STATE(1228),
    [sym__statement] = STATE(913),
    [sym_empty_statement] = STATE(985),
    [sym__simple_statement] = STATE(985),
    [sym_expression_statement] = STATE(937),
    [sym_send_statement] = STATE(937),
    [sym_inc_statement] = STATE(937),
    [sym_dec_statement] = STATE(937),
    [sym_assignment_statement] = STATE(937),
    [sym_short_var_declaration] = STATE(937),
    [sym_labeled_statement] = STATE(985),
    [sym_empty_labeled_statement] = STATE(1228),
    [sym_fallthrough_statement] = STATE(985),
    [sym_break_statement] = STATE(985),
    [sym_continue_statement] = STATE(985),
    [sym_goto_statement] = STATE(985),
    [sym_return_statement] = STATE(985),
    [sym_go_statement] = STATE(985),
    [sym_defer_statement] = STATE(985),
    [sym_if_statement] = STATE(985),
    [sym_for_statement] = STATE(985),
    [sym_expression_switch_statement] = STATE(985),
    [sym_type_switch_statement] = STATE(985),
    [sym_select_statement] = STATE(985),
    [sym__expression] = STATE(240),
    [sym_parenthesized_expression] = STATE(263),
    [sym_call_expression] = STATE(263),
    [sym_selector_expression] = STATE(263),
    [sym_index_expression] = STATE(263),
    [sym_slice_expression] = STATE(263),
    [sym_type_assertion_expression] = STATE(263),
    [sym_type_conversion_expression] = STATE(263),
    [sym_type_instantiation_expression] = STATE(263),
    [sym_composite_literal] = STATE(263),
    [sym_func_literal] = STATE(263),
    [sym_unary_expression] = STATE(263),
    [sym_binary_expression] = STATE(263),
    [sym_qualified_type] = STATE(1027),
    [sym_raw_string_literal] = STATE(263),
    [sym_interpreted_string_literal] = STATE(263),
    [sym_identifier] = ACTIONS(186),
    [anon_sym_SEMI] = ACTIONS(9),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_var] = ACTIONS(19),
    [anon_sym_func] = ACTIONS(188),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_type] = ACTIONS(25),
    [anon_sym_STAR] = ACTIONS(27),
    [anon_sym_struct] = ACTIONS(29),
    [anon_sym_TILDE] = ACTIONS(31),
    [anon_sym_LBRACE] = ACTIONS(33),
    [anon_sym_RBRACE] = ACTIONS(206),
    [anon_sym_interface] = ACTIONS(35),
    [anon_sym_map] = ACTIONS(37),
    [anon_sym_chan] = ACTIONS(39),
    [anon_sym_LT_DASH] = ACTIONS(41),
    [anon_sym_fallthrough] = ACTIONS(43),
    [anon_sym_break] = ACTIONS(45),
    [anon_sym_continue] = ACTIONS(47),
    [anon_sym_goto] = ACTIONS(49),
    [anon_sym_return] = ACTIONS(51),
    [anon_sym_go] = ACTIONS(53),
    [anon_sym_defer] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_for] = ACTIONS(59),
    [anon_sym_switch] = ACTIONS(61),
    [anon_sym_case] = ACTIONS(208),
    [anon_sym_default] = ACTIONS(208),
    [anon_sym_select] = ACTIONS(63),
    [anon_sym_new] = ACTIONS(65),
    [anon_sym_make] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(67),
    [anon_sym_DASH] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_CARET] = ACTIONS(67),
    [anon_sym_AMP] = ACTIONS(67),
    [anon_sym_BQUOTE] = ACTIONS(69),
    [anon_sym_DQUOTE] = ACTIONS(71),
    [sym_int_literal] = ACTIONS(73),
    [sym_float_literal] = ACTIONS(73),
    [sym_imaginary_literal] = ACTIONS(75),
    [sym_rune_literal] = ACTIONS(75),
    [sym_nil] = ACTIONS(73),
    [sym_true] = ACTIONS(73),
    [sym_false] = ACTIONS(73),
    [sym_iota] = ACTIONS(73),
    [sym_comment] = ACTIONS(3),
  },
  [9] = {
    [sym__declaration] = STATE(985),
    [sym_const_declaration] = STATE(985),
    [sym_var_declaration] = STATE(985),
    [sym_type_declaration] = STATE(985),
    [sym_expression_list] = STATE(804),
    [sym_parenthesized_type] = STATE(1303),
    [sym__simple_type] = STATE(1303),
    [sym_generic_type] = STATE(1131),
    [sym_pointer_type] = STATE(867),
    [sym_array_type] = STATE(1131),
    [sym_implicit_length_array_type] = STATE(1293),
    [sym_slice_type] = STATE(1131),
    [sym_struct_type] = STATE(1131),
    [sym_negated_type] = STATE(850),
    [sym_interface_type] = STATE(867),
    [sym_map_type] = STATE(1131),
    [sym_channel_type] = STATE(867),
    [sym_function_type] = STATE(867),
    [sym_block] = STATE(985),
    [sym__statement] = STATE(921),
    [sym_empty_statement] = STATE(985),
    [sym__simple_statement] = STATE(985),
    [sym_expression_statement] = STATE(937),
    [sym_send_statement] = STATE(937),
    [sym_inc_statement] = STATE(937),
    [sym_dec_statement] = STATE(937),
    [sym_assignment_statement] = STATE(937),
    [sym_short_var_declaration] = STATE(937),
    [sym_labeled_statement] = STATE(985),
    [sym_empty_labeled_statement] = STATE(1191),
    [sym_fallthrough_statement] = STATE(985),
    [sym_break_statement] = STATE(985),
    [sym_continue_statement] = STATE(985),
    [sym_goto_statement] = STATE(985),
    [sym_return_statement] = STATE(985),
    [sym_go_statement] = STATE(985),
    [sym_defer_statement] = STATE(985),
    [sym_if_statement] = STATE(985),
    [sym_for_statement] = STATE(985),
    [sym_expression_switch_statement] = STATE(985),
    [sym_type_switch_statement] = STATE(985),
    [sym_select_statement] = STATE(985),
    [sym__expression] = STATE(240),
    [sym_parenthesized_expression] = STATE(263),
    [sym_call_expression] = STATE(263),
    [sym_selector_expression] = STATE(263),
    [sym_index_expression] = STATE(263),
    [sym_slice_expression] = STATE(263),
    [sym_type_assertion_expression] = STATE(263),
    [sym_type_conversion_expression] = STATE(263),
    [sym_type_instantiation_expression] = STATE(263),
    [sym_composite_literal] = STATE(263),
    [sym_func_literal] = STATE(263),
    [sym_unary_expression] = STATE(263),
    [sym_binary_expression] = STATE(263),
    [sym_qualified_type] = STATE(1027),
    [sym_raw_string_literal] = STATE(263),
    [sym_interpreted_string_literal] = STATE(263),
    [sym_identifier] = ACTIONS(186),
    [anon_sym_SEMI] = ACTIONS(9),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_var] = ACTIONS(19),
    [anon_sym_func] = ACTIONS(188),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_type] = ACTIONS(25),
    [anon_sym_STAR] = ACTIONS(27),
    [anon_sym_struct] = ACTIONS(29),
    [anon_sym_TILDE] = ACTIONS(31),
    [anon_sym_LBRACE] = ACTIONS(33),
    [anon_sym_RBRACE] = ACTIONS(210),
    [anon_sym_interface] = ACTIONS(35),
    [anon_sym_map] = ACTIONS(37),
    [anon_sym_chan] = ACTIONS(39),
    [anon_sym_LT_DASH] = ACTIONS(41),
    [anon_sym_fallthrough] = ACTIONS(43),
    [anon_sym_break] = ACTIONS(45),
    [anon_sym_continue] = ACTIONS(47),
    [anon_sym_goto] = ACTIONS(49),
    [anon_sym_return] = ACTIONS(51),
    [anon_sym_go] = ACTIONS(53),
    [anon_sym_defer] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_for] = ACTIONS(59),
    [anon_sym_switch] = ACTIONS(61),
    [anon_sym_case] = ACTIONS(212),
    [anon_sym_default] = ACTIONS(212),
    [anon_sym_select] = ACTIONS(63),
    [anon_sym_new] = ACTIONS(65),
    [anon_sym_make] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(67),
    [anon_sym_DASH] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_CARET] = ACTIONS(67),
    [anon_sym_AMP] = ACTIONS(67),
    [anon_sym_BQUOTE] = ACTIONS(69),
    [anon_sym_DQUOTE] = ACTIONS(71),
    [sym_int_literal] = ACTIONS(73),
    [sym_float_literal] = ACTIONS(73),
    [sym_imaginary_literal] = ACTIONS(75),
    [sym_rune_literal] = ACTIONS(75),
    [sym_nil] = ACTIONS(73),
    [sym_true] = ACTIONS(73),
    [sym_false] = ACTIONS(73),
    [sym_iota] = ACTIONS(73),
    [sym_comment] = ACTIONS(3),
  },
  [10] = {
    [sym__declaration] = STATE(985),
    [sym_const_declaration] = STATE(985),
    [sym_var_declaration] = STATE(985),
    [sym_type_declaration] = STATE(985),
    [sym_expression_list] = STATE(804),
    [sym_parenthesized_type] = STATE(1303),
    [sym__simple_type] = STATE(1303),
    [sym_generic_type] = STATE(1131),
    [sym_pointer_type] = STATE(867),
    [sym_array_type] = STATE(1131),
    [sym_implicit_length_array_type] = STATE(1293),
    [sym_slice_type] = STATE(1131),
    [sym_struct_type] = STATE(1131),
    [sym_negated_type] = STATE(850),
    [sym_interface_type] = STATE(867),
    [sym_map_type] = STATE(1131),
    [sym_channel_type] = STATE(867),
    [sym_function_type] = STATE(867),
    [sym_block] = STATE(985),
    [sym__statement] = STATE(921),
    [sym_empty_statement] = STATE(985),
    [sym__simple_statement] = STATE(985),
    [sym_expression_statement] = STATE(937),
    [sym_send_statement] = STATE(937),
    [sym_inc_statement] = STATE(937),
    [sym_dec_statement] = STATE(937),
    [sym_assignment_statement] = STATE(937),
    [sym_short_var_declaration] = STATE(937),
    [sym_labeled_statement] = STATE(985),
    [sym_empty_labeled_statement] = STATE(1235),
    [sym_fallthrough_statement] = STATE(985),
    [sym_break_statement] = STATE(985),
    [sym_continue_statement] = STATE(985),
    [sym_goto_statement] = STATE(985),
    [sym_return_statement] = STATE(985),
    [sym_go_statement] = STATE(985),
    [sym_defer_statement] = STATE(985),
    [sym_if_statement] = STATE(985),
    [sym_for_statement] = STATE(985),
    [sym_expression_switch_statement] = STATE(985),
    [sym_type_switch_statement] = STATE(985),
    [sym_select_statement] = STATE(985),
    [sym__expression] = STATE(240),
    [sym_parenthesized_expression] = STATE(263),
    [sym_call_expression] = STATE(263),
    [sym_selector_expression] = STATE(263),
    [sym_index_expression] = STATE(263),
    [sym_slice_expression] = STATE(263),
    [sym_type_assertion_expression] = STATE(263),
    [sym_type_conversion_expression] = STATE(263),
    [sym_type_instantiation_expression] = STATE(263),
    [sym_composite_literal] = STATE(263),
    [sym_func_literal] = STATE(263),
    [sym_unary_expression] = STATE(263),
    [sym_binary_expression] = STATE(263),
    [sym_qualified_type] = STATE(1027),
    [sym_raw_string_literal] = STATE(263),
    [sym_interpreted_string_literal] = STATE(263),
    [sym_identifier] = ACTIONS(186),
    [anon_sym_SEMI] = ACTIONS(9),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_var] = ACTIONS(19),
    [anon_sym_func] = ACTIONS(188),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_type] = ACTIONS(25),
    [anon_sym_STAR] = ACTIONS(27),
    [anon_sym_struct] = ACTIONS(29),
    [anon_sym_TILDE] = ACTIONS(31),
    [anon_sym_LBRACE] = ACTIONS(33),
    [anon_sym_RBRACE] = ACTIONS(214),
    [anon_sym_interface] = ACTIONS(35),
    [anon_sym_map] = ACTIONS(37),
    [anon_sym_chan] = ACTIONS(39),
    [anon_sym_LT_DASH] = ACTIONS(41),
    [anon_sym_fallthrough] = ACTIONS(43),
    [anon_sym_break] = ACTIONS(45),
    [anon_sym_continue] = ACTIONS(47),
    [anon_sym_goto] = ACTIONS(49),
    [anon_sym_return] = ACTIONS(51),
    [anon_sym_go] = ACTIONS(53),
    [anon_sym_defer] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_for] = ACTIONS(59),
    [anon_sym_switch] = ACTIONS(61),
    [anon_sym_case] = ACTIONS(216),
    [anon_sym_default] = ACTIONS(216),
    [anon_sym_select] = ACTIONS(63),
    [anon_sym_new] = ACTIONS(65),
    [anon_sym_make] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(67),
    [anon_sym_DASH] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_CARET] = ACTIONS(67),
    [anon_sym_AMP] = ACTIONS(67),
    [anon_sym_BQUOTE] = ACTIONS(69),
    [anon_sym_DQUOTE] = ACTIONS(71),
    [sym_int_literal] = ACTIONS(73),
    [sym_float_literal] = ACTIONS(73),
    [sym_imaginary_literal] = ACTIONS(75),
    [sym_rune_literal] = ACTIONS(75),
    [sym_nil] = ACTIONS(73),
    [sym_true] = ACTIONS(73),
    [sym_false] = ACTIONS(73),
    [sym_iota] = ACTIONS(73),
    [sym_comment] = ACTIONS(3),
  },
  [11] = {
    [sym__declaration] = STATE(985),
    [sym_const_declaration] = STATE(985),
    [sym_var_declaration] = STATE(985),
    [sym_type_declaration] = STATE(985),
    [sym_expression_list] = STATE(804),
    [sym_parenthesized_type] = STATE(1303),
    [sym__simple_type] = STATE(1303),
    [sym_generic_type] = STATE(1131),
    [sym_pointer_type] = STATE(867),
    [sym_array_type] = STATE(1131),
    [sym_implicit_length_array_type] = STATE(1293),
    [sym_slice_type] = STATE(1131),
    [sym_struct_type] = STATE(1131),
    [sym_negated_type] = STATE(850),
    [sym_interface_type] = STATE(867),
    [sym_map_type] = STATE(1131),
    [sym_channel_type] = STATE(867),
    [sym_function_type] = STATE(867),
    [sym_block] = STATE(985),
    [sym__statement] = STATE(1010),
    [sym_empty_statement] = STATE(985),
    [sym__simple_statement] = STATE(985),
    [sym_expression_statement] = STATE(937),
    [sym_send_statement] = STATE(937),
    [sym_inc_statement] = STATE(937),
    [sym_dec_statement] = STATE(937),
    [sym_assignment_statement] = STATE(937),
    [sym_short_var_declaration] = STATE(937),
    [sym_labeled_statement] = STATE(985),
    [sym_fallthrough_statement] = STATE(985),
    [sym_break_statement] = STATE(985),
    [sym_continue_statement] = STATE(985),
    [sym_goto_statement] = STATE(985),
    [sym_return_statement] = STATE(985),
    [sym_go_statement] = STATE(985),
    [sym_defer_statement] = STATE(985),
    [sym_if_statement] = STATE(985),
    [sym_for_statement] = STATE(985),
    [sym_expression_switch_statement] = STATE(985),
    [sym_type_switch_statement] = STATE(985),
    [sym_select_statement] = STATE(985),
    [sym__expression] = STATE(240),
    [sym_parenthesized_expression] = STATE(263),
    [sym_call_expression] = STATE(263),
    [sym_selector_expression] = STATE(263),
    [sym_index_expression] = STATE(263),
    [sym_slice_expression] = STATE(263),
    [sym_type_assertion_expression] = STATE(263),
    [sym_type_conversion_expression] = STATE(263),
    [sym_type_instantiation_expression] = STATE(263),
    [sym_composite_literal] = STATE(263),
    [sym_func_literal] = STATE(263),
    [sym_unary_expression] = STATE(263),
    [sym_binary_expression] = STATE(263),
    [sym_qualified_type] = STATE(1027),
    [sym_raw_string_literal] = STATE(263),
    [sym_interpreted_string_literal] = STATE(263),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_SEMI] = ACTIONS(9),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_var] = ACTIONS(19),
    [anon_sym_func] = ACTIONS(188),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_type] = ACTIONS(25),
    [anon_sym_STAR] = ACTIONS(27),
    [anon_sym_struct] = ACTIONS(29),
    [anon_sym_TILDE] = ACTIONS(31),
    [anon_sym_LBRACE] = ACTIONS(33),
    [anon_sym_RBRACE] = ACTIONS(218),
    [anon_sym_interface] = ACTIONS(35),
    [anon_sym_map] = ACTIONS(37),
    [anon_sym_chan] = ACTIONS(39),
    [anon_sym_LT_DASH] = ACTIONS(41),
    [anon_sym_fallthrough] = ACTIONS(43),
    [anon_sym_break] = ACTIONS(45),
    [anon_sym_continue] = ACTIONS(47),
    [anon_sym_goto] = ACTIONS(49),
    [anon_sym_return] = ACTIONS(51),
    [anon_sym_go] = ACTIONS(53),
    [anon_sym_defer] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_for] = ACTIONS(59),
    [anon_sym_switch] = ACTIONS(61),
    [anon_sym_case] = ACTIONS(220),
    [anon_sym_default] = ACTIONS(220),
    [anon_sym_select] = ACTIONS(63),
    [anon_sym_new] = ACTIONS(65),
    [anon_sym_make] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(67),
    [anon_sym_DASH] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_CARET] = ACTIONS(67),
    [anon_sym_AMP] = ACTIONS(67),
    [anon_sym_BQUOTE] = ACTIONS(69),
    [anon_sym_DQUOTE] = ACTIONS(71),
    [sym_int_literal] = ACTIONS(73),
    [sym_float_literal] = ACTIONS(73),
    [sym_imaginary_literal] = ACTIONS(75),
    [sym_rune_literal] = ACTIONS(75),
    [sym_nil] = ACTIONS(73),
    [sym_true] = ACTIONS(73),
    [sym_false] = ACTIONS(73),
    [sym_iota] = ACTIONS(73),
    [sym_comment] = ACTIONS(3),
  },
  [12] = {
    [sym__declaration] = STATE(985),
    [sym_const_declaration] = STATE(985),
    [sym_var_declaration] = STATE(985),
    [sym_type_declaration] = STATE(985),
    [sym_expression_list] = STATE(804),
    [sym_parenthesized_type] = STATE(1303),
    [sym__simple_type] = STATE(1303),
    [sym_generic_type] = STATE(1131),
    [sym_pointer_type] = STATE(867),
    [sym_array_type] = STATE(1131),
    [sym_implicit_length_array_type] = STATE(1293),
    [sym_slice_type] = STATE(1131),
    [sym_struct_type] = STATE(1131),
    [sym_negated_type] = STATE(850),
    [sym_interface_type] = STATE(867),
    [sym_map_type] = STATE(1131),
    [sym_channel_type] = STATE(867),
    [sym_function_type] = STATE(867),
    [sym_block] = STATE(985),
    [sym__statement_list] = STATE(1327),
    [sym__statement] = STATE(913),
    [sym_empty_statement] = STATE(985),
    [sym__simple_statement] = STATE(985),
    [sym_expression_statement] = STATE(937),
    [sym_send_statement] = STATE(937),
    [sym_inc_statement] = STATE(937),
    [sym_dec_statement] = STATE(937),
    [sym_assignment_statement] = STATE(937),
    [sym_short_var_declaration] = STATE(937),
    [sym_labeled_statement] = STATE(985),
    [sym_empty_labeled_statement] = STATE(1327),
    [sym_fallthrough_statement] = STATE(985),
    [sym_break_statement] = STATE(985),
    [sym_continue_statement] = STATE(985),
    [sym_goto_statement] = STATE(985),
    [sym_return_statement] = STATE(985),
    [sym_go_statement] = STATE(985),
    [sym_defer_statement] = STATE(985),
    [sym_if_statement] = STATE(985),
    [sym_for_statement] = STATE(985),
    [sym_expression_switch_statement] = STATE(985),
    [sym_type_switch_statement] = STATE(985),
    [sym_select_statement] = STATE(985),
    [sym__expression] = STATE(240),
    [sym_parenthesized_expression] = STATE(263),
    [sym_call_expression] = STATE(263),
    [sym_selector_expression] = STATE(263),
    [sym_index_expression] = STATE(263),
    [sym_slice_expression] = STATE(263),
    [sym_type_assertion_expression] = STATE(263),
    [sym_type_conversion_expression] = STATE(263),
    [sym_type_instantiation_expression] = STATE(263),
    [sym_composite_literal] = STATE(263),
    [sym_func_literal] = STATE(263),
    [sym_unary_expression] = STATE(263),
    [sym_binary_expression] = STATE(263),
    [sym_qualified_type] = STATE(1027),
    [sym_raw_string_literal] = STATE(263),
    [sym_interpreted_string_literal] = STATE(263),
    [sym_identifier] = ACTIONS(186),
    [anon_sym_SEMI] = ACTIONS(9),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_var] = ACTIONS(19),
    [anon_sym_func] = ACTIONS(188),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_type] = ACTIONS(25),
    [anon_sym_STAR] = ACTIONS(27),
    [anon_sym_struct] = ACTIONS(29),
    [anon_sym_TILDE] = ACTIONS(31),
    [anon_sym_LBRACE] = ACTIONS(33),
    [anon_sym_RBRACE] = ACTIONS(222),
    [anon_sym_interface] = ACTIONS(35),
    [anon_sym_map] = ACTIONS(37),
    [anon_sym_chan] = ACTIONS(39),
    [anon_sym_LT_DASH] = ACTIONS(41),
    [anon_sym_fallthrough] = ACTIONS(43),
    [anon_sym_break] = ACTIONS(45),
    [anon_sym_continue] = ACTIONS(47),
    [anon_sym_goto] = ACTIONS(49),
    [anon_sym_return] = ACTIONS(51),
    [anon_sym_go] = ACTIONS(53),
    [anon_sym_defer] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_for] = ACTIONS(59),
    [anon_sym_switch] = ACTIONS(61),
    [anon_sym_select] = ACTIONS(63),
    [anon_sym_new] = ACTIONS(65),
    [anon_sym_make] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(67),
    [anon_sym_DASH] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_CARET] = ACTIONS(67),
    [anon_sym_AMP] = ACTIONS(67),
    [anon_sym_BQUOTE] = ACTIONS(69),
    [anon_sym_DQUOTE] = ACTIONS(71),
    [sym_int_literal] = ACTIONS(73),
    [sym_float_literal] = ACTIONS(73),
    [sym_imaginary_literal] = ACTIONS(75),
    [sym_rune_literal] = ACTIONS(75),
    [sym_nil] = ACTIONS(73),
    [sym_true] = ACTIONS(73),
    [sym_false] = ACTIONS(73),
    [sym_iota] = ACTIONS(73),
    [sym_comment] = ACTIONS(3),
  },
  [13] = {
    [sym__declaration] = STATE(985),
    [sym_const_declaration] = STATE(985),
    [sym_var_declaration] = STATE(985),
    [sym_type_declaration] = STATE(985),
    [sym_expression_list] = STATE(804),
    [sym_parenthesized_type] = STATE(1303),
    [sym__simple_type] = STATE(1303),
    [sym_generic_type] = STATE(1131),
    [sym_pointer_type] = STATE(867),
    [sym_array_type] = STATE(1131),
    [sym_implicit_length_array_type] = STATE(1293),
    [sym_slice_type] = STATE(1131),
    [sym_struct_type] = STATE(1131),
    [sym_negated_type] = STATE(850),
    [sym_interface_type] = STATE(867),
    [sym_map_type] = STATE(1131),
    [sym_channel_type] = STATE(867),
    [sym_function_type] = STATE(867),
    [sym_block] = STATE(985),
    [sym__statement_list] = STATE(1348),
    [sym__statement] = STATE(913),
    [sym_empty_statement] = STATE(985),
    [sym__simple_statement] = STATE(985),
    [sym_expression_statement] = STATE(937),
    [sym_send_statement] = STATE(937),
    [sym_inc_statement] = STATE(937),
    [sym_dec_statement] = STATE(937),
    [sym_assignment_statement] = STATE(937),
    [sym_short_var_declaration] = STATE(937),
    [sym_labeled_statement] = STATE(985),
    [sym_empty_labeled_statement] = STATE(1348),
    [sym_fallthrough_statement] = STATE(985),
    [sym_break_statement] = STATE(985),
    [sym_continue_statement] = STATE(985),
    [sym_goto_statement] = STATE(985),
    [sym_return_statement] = STATE(985),
    [sym_go_statement] = STATE(985),
    [sym_defer_statement] = STATE(985),
    [sym_if_statement] = STATE(985),
    [sym_for_statement] = STATE(985),
    [sym_expression_switch_statement] = STATE(985),
    [sym_type_switch_statement] = STATE(985),
    [sym_select_statement] = STATE(985),
    [sym__expression] = STATE(240),
    [sym_parenthesized_expression] = STATE(263),
    [sym_call_expression] = STATE(263),
    [sym_selector_expression] = STATE(263),
    [sym_index_expression] = STATE(263),
    [sym_slice_expression] = STATE(263),
    [sym_type_assertion_expression] = STATE(263),
    [sym_type_conversion_expression] = STATE(263),
    [sym_type_instantiation_expression] = STATE(263),
    [sym_composite_literal] = STATE(263),
    [sym_func_literal] = STATE(263),
    [sym_unary_expression] = STATE(263),
    [sym_binary_expression] = STATE(263),
    [sym_qualified_type] = STATE(1027),
    [sym_raw_string_literal] = STATE(263),
    [sym_interpreted_string_literal] = STATE(263),
    [sym_identifier] = ACTIONS(186),
    [anon_sym_SEMI] = ACTIONS(9),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_var] = ACTIONS(19),
    [anon_sym_func] = ACTIONS(188),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_type] = ACTIONS(25),
    [anon_sym_STAR] = ACTIONS(27),
    [anon_sym_struct] = ACTIONS(29),
    [anon_sym_TILDE] = ACTIONS(31),
    [anon_sym_LBRACE] = ACTIONS(33),
    [anon_sym_RBRACE] = ACTIONS(224),
    [anon_sym_interface] = ACTIONS(35),
    [anon_sym_map] = ACTIONS(37),
    [anon_sym_chan] = ACTIONS(39),
    [anon_sym_LT_DASH] = ACTIONS(41),
    [anon_sym_fallthrough] = ACTIONS(43),
    [anon_sym_break] = ACTIONS(45),
    [anon_sym_continue] = ACTIONS(47),
    [anon_sym_goto] = ACTIONS(49),
    [anon_sym_return] = ACTIONS(51),
    [anon_sym_go] = ACTIONS(53),
    [anon_sym_defer] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_for] = ACTIONS(59),
    [anon_sym_switch] = ACTIONS(61),
    [anon_sym_select] = ACTIONS(63),
    [anon_sym_new] = ACTIONS(65),
    [anon_sym_make] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(67),
    [anon_sym_DASH] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_CARET] = ACTIONS(67),
    [anon_sym_AMP] = ACTIONS(67),
    [anon_sym_BQUOTE] = ACTIONS(69),
    [anon_sym_DQUOTE] = ACTIONS(71),
    [sym_int_literal] = ACTIONS(73),
    [sym_float_literal] = ACTIONS(73),
    [sym_imaginary_literal] = ACTIONS(75),
    [sym_rune_literal] = ACTIONS(75),
    [sym_nil] = ACTIONS(73),
    [sym_true] = ACTIONS(73),
    [sym_false] = ACTIONS(73),
    [sym_iota] = ACTIONS(73),
    [sym_comment] = ACTIONS(3),
  },
  [14] = {
    [sym__declaration] = STATE(985),
    [sym_const_declaration] = STATE(985),
    [sym_var_declaration] = STATE(985),
    [sym_type_declaration] = STATE(985),
    [sym_expression_list] = STATE(804),
    [sym_parenthesized_type] = STATE(1303),
    [sym__simple_type] = STATE(1303),
    [sym_generic_type] = STATE(1131),
    [sym_pointer_type] = STATE(867),
    [sym_array_type] = STATE(1131),
    [sym_implicit_length_array_type] = STATE(1293),
    [sym_slice_type] = STATE(1131),
    [sym_struct_type] = STATE(1131),
    [sym_negated_type] = STATE(850),
    [sym_interface_type] = STATE(867),
    [sym_map_type] = STATE(1131),
    [sym_channel_type] = STATE(867),
    [sym_function_type] = STATE(867),
    [sym_block] = STATE(985),
    [sym__statement_list] = STATE(1386),
    [sym__statement] = STATE(913),
    [sym_empty_statement] = STATE(985),
    [sym__simple_statement] = STATE(985),
    [sym_expression_statement] = STATE(937),
    [sym_send_statement] = STATE(937),
    [sym_inc_statement] = STATE(937),
    [sym_dec_statement] = STATE(937),
    [sym_assignment_statement] = STATE(937),
    [sym_short_var_declaration] = STATE(937),
    [sym_labeled_statement] = STATE(985),
    [sym_empty_labeled_statement] = STATE(1386),
    [sym_fallthrough_statement] = STATE(985),
    [sym_break_statement] = STATE(985),
    [sym_continue_statement] = STATE(985),
    [sym_goto_statement] = STATE(985),
    [sym_return_statement] = STATE(985),
    [sym_go_statement] = STATE(985),
    [sym_defer_statement] = STATE(985),
    [sym_if_statement] = STATE(985),
    [sym_for_statement] = STATE(985),
    [sym_expression_switch_statement] = STATE(985),
    [sym_type_switch_statement] = STATE(985),
    [sym_select_statement] = STATE(985),
    [sym__expression] = STATE(240),
    [sym_parenthesized_expression] = STATE(263),
    [sym_call_expression] = STATE(263),
    [sym_selector_expression] = STATE(263),
    [sym_index_expression] = STATE(263),
    [sym_slice_expression] = STATE(263),
    [sym_type_assertion_expression] = STATE(263),
    [sym_type_conversion_expression] = STATE(263),
    [sym_type_instantiation_expression] = STATE(263),
    [sym_composite_literal] = STATE(263),
    [sym_func_literal] = STATE(263),
    [sym_unary_expression] = STATE(263),
    [sym_binary_expression] = STATE(263),
    [sym_qualified_type] = STATE(1027),
    [sym_raw_string_literal] = STATE(263),
    [sym_interpreted_string_literal] = STATE(263),
    [sym_identifier] = ACTIONS(186),
    [anon_sym_SEMI] = ACTIONS(9),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_var] = ACTIONS(19),
    [anon_sym_func] = ACTIONS(188),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_type] = ACTIONS(25),
    [anon_sym_STAR] = ACTIONS(27),
    [anon_sym_struct] = ACTIONS(29),
    [anon_sym_TILDE] = ACTIONS(31),
    [anon_sym_LBRACE] = ACTIONS(33),
    [anon_sym_RBRACE] = ACTIONS(226),
    [anon_sym_interface] = ACTIONS(35),
    [anon_sym_map] = ACTIONS(37),
    [anon_sym_chan] = ACTIONS(39),
    [anon_sym_LT_DASH] = ACTIONS(41),
    [anon_sym_fallthrough] = ACTIONS(43),
    [anon_sym_break] = ACTIONS(45),
    [anon_sym_continue] = ACTIONS(47),
    [anon_sym_goto] = ACTIONS(49),
    [anon_sym_return] = ACTIONS(51),
    [anon_sym_go] = ACTIONS(53),
    [anon_sym_defer] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_for] = ACTIONS(59),
    [anon_sym_switch] = ACTIONS(61),
    [anon_sym_select] = ACTIONS(63),
    [anon_sym_new] = ACTIONS(65),
    [anon_sym_make] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(67),
    [anon_sym_DASH] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_CARET] = ACTIONS(67),
    [anon_sym_AMP] = ACTIONS(67),
    [anon_sym_BQUOTE] = ACTIONS(69),
    [anon_sym_DQUOTE] = ACTIONS(71),
    [sym_int_literal] = ACTIONS(73),
    [sym_float_literal] = ACTIONS(73),
    [sym_imaginary_literal] = ACTIONS(75),
    [sym_rune_literal] = ACTIONS(75),
    [sym_nil] = ACTIONS(73),
    [sym_true] = ACTIONS(73),
    [sym_false] = ACTIONS(73),
    [sym_iota] = ACTIONS(73),
    [sym_comment] = ACTIONS(3),
  },
  [15] = {
    [sym__declaration] = STATE(985),
    [sym_const_declaration] = STATE(985),
    [sym_var_declaration] = STATE(985),
    [sym_type_declaration] = STATE(985),
    [sym_expression_list] = STATE(804),
    [sym_parenthesized_type] = STATE(1303),
    [sym__simple_type] = STATE(1303),
    [sym_generic_type] = STATE(1131),
    [sym_pointer_type] = STATE(867),
    [sym_array_type] = STATE(1131),
    [sym_implicit_length_array_type] = STATE(1293),
    [sym_slice_type] = STATE(1131),
    [sym_struct_type] = STATE(1131),
    [sym_negated_type] = STATE(850),
    [sym_interface_type] = STATE(867),
    [sym_map_type] = STATE(1131),
    [sym_channel_type] = STATE(867),
    [sym_function_type] = STATE(867),
    [sym_block] = STATE(985),
    [sym__statement_list] = STATE(1361),
    [sym__statement] = STATE(913),
    [sym_empty_statement] = STATE(985),
    [sym__simple_statement] = STATE(985),
    [sym_expression_statement] = STATE(937),
    [sym_send_statement] = STATE(937),
    [sym_inc_statement] = STATE(937),
    [sym_dec_statement] = STATE(937),
    [sym_assignment_statement] = STATE(937),
    [sym_short_var_declaration] = STATE(937),
    [sym_labeled_statement] = STATE(985),
    [sym_empty_labeled_statement] = STATE(1361),
    [sym_fallthrough_statement] = STATE(985),
    [sym_break_statement] = STATE(985),
    [sym_continue_statement] = STATE(985),
    [sym_goto_statement] = STATE(985),
    [sym_return_statement] = STATE(985),
    [sym_go_statement] = STATE(985),
    [sym_defer_statement] = STATE(985),
    [sym_if_statement] = STATE(985),
    [sym_for_statement] = STATE(985),
    [sym_expression_switch_statement] = STATE(985),
    [sym_type_switch_statement] = STATE(985),
    [sym_select_statement] = STATE(985),
    [sym__expression] = STATE(240),
    [sym_parenthesized_expression] = STATE(263),
    [sym_call_expression] = STATE(263),
    [sym_selector_expression] = STATE(263),
    [sym_index_expression] = STATE(263),
    [sym_slice_expression] = STATE(263),
    [sym_type_assertion_expression] = STATE(263),
    [sym_type_conversion_expression] = STATE(263),
    [sym_type_instantiation_expression] = STATE(263),
    [sym_composite_literal] = STATE(263),
    [sym_func_literal] = STATE(263),
    [sym_unary_expression] = STATE(263),
    [sym_binary_expression] = STATE(263),
    [sym_qualified_type] = STATE(1027),
    [sym_raw_string_literal] = STATE(263),
    [sym_interpreted_string_literal] = STATE(263),
    [sym_identifier] = ACTIONS(186),
    [anon_sym_SEMI] = ACTIONS(9),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_var] = ACTIONS(19),
    [anon_sym_func] = ACTIONS(188),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_type] = ACTIONS(25),
    [anon_sym_STAR] = ACTIONS(27),
    [anon_sym_struct] = ACTIONS(29),
    [anon_sym_TILDE] = ACTIONS(31),
    [anon_sym_LBRACE] = ACTIONS(33),
    [anon_sym_RBRACE] = ACTIONS(228),
    [anon_sym_interface] = ACTIONS(35),
    [anon_sym_map] = ACTIONS(37),
    [anon_sym_chan] = ACTIONS(39),
    [anon_sym_LT_DASH] = ACTIONS(41),
    [anon_sym_fallthrough] = ACTIONS(43),
    [anon_sym_break] = ACTIONS(45),
    [anon_sym_continue] = ACTIONS(47),
    [anon_sym_goto] = ACTIONS(49),
    [anon_sym_return] = ACTIONS(51),
    [anon_sym_go] = ACTIONS(53),
    [anon_sym_defer] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_for] = ACTIONS(59),
    [anon_sym_switch] = ACTIONS(61),
    [anon_sym_select] = ACTIONS(63),
    [anon_sym_new] = ACTIONS(65),
    [anon_sym_make] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(67),
    [anon_sym_DASH] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_CARET] = ACTIONS(67),
    [anon_sym_AMP] = ACTIONS(67),
    [anon_sym_BQUOTE] = ACTIONS(69),
    [anon_sym_DQUOTE] = ACTIONS(71),
    [sym_int_literal] = ACTIONS(73),
    [sym_float_literal] = ACTIONS(73),
    [sym_imaginary_literal] = ACTIONS(75),
    [sym_rune_literal] = ACTIONS(75),
    [sym_nil] = ACTIONS(73),
    [sym_true] = ACTIONS(73),
    [sym_false] = ACTIONS(73),
    [sym_iota] = ACTIONS(73),
    [sym_comment] = ACTIONS(3),
  },
  [16] = {
    [sym__declaration] = STATE(985),
    [sym_const_declaration] = STATE(985),
    [sym_var_declaration] = STATE(985),
    [sym_type_declaration] = STATE(985),
    [sym_expression_list] = STATE(804),
    [sym_parenthesized_type] = STATE(1303),
    [sym__simple_type] = STATE(1303),
    [sym_generic_type] = STATE(1131),
    [sym_pointer_type] = STATE(867),
    [sym_array_type] = STATE(1131),
    [sym_implicit_length_array_type] = STATE(1293),
    [sym_slice_type] = STATE(1131),
    [sym_struct_type] = STATE(1131),
    [sym_negated_type] = STATE(850),
    [sym_interface_type] = STATE(867),
    [sym_map_type] = STATE(1131),
    [sym_channel_type] = STATE(867),
    [sym_function_type] = STATE(867),
    [sym_block] = STATE(985),
    [sym__statement_list] = STATE(1401),
    [sym__statement] = STATE(913),
    [sym_empty_statement] = STATE(985),
    [sym__simple_statement] = STATE(985),
    [sym_expression_statement] = STATE(937),
    [sym_send_statement] = STATE(937),
    [sym_inc_statement] = STATE(937),
    [sym_dec_statement] = STATE(937),
    [sym_assignment_statement] = STATE(937),
    [sym_short_var_declaration] = STATE(937),
    [sym_labeled_statement] = STATE(985),
    [sym_empty_labeled_statement] = STATE(1401),
    [sym_fallthrough_statement] = STATE(985),
    [sym_break_statement] = STATE(985),
    [sym_continue_statement] = STATE(985),
    [sym_goto_statement] = STATE(985),
    [sym_return_statement] = STATE(985),
    [sym_go_statement] = STATE(985),
    [sym_defer_statement] = STATE(985),
    [sym_if_statement] = STATE(985),
    [sym_for_statement] = STATE(985),
    [sym_expression_switch_statement] = STATE(985),
    [sym_type_switch_statement] = STATE(985),
    [sym_select_statement] = STATE(985),
    [sym__expression] = STATE(240),
    [sym_parenthesized_expression] = STATE(263),
    [sym_call_expression] = STATE(263),
    [sym_selector_expression] = STATE(263),
    [sym_index_expression] = STATE(263),
    [sym_slice_expression] = STATE(263),
    [sym_type_assertion_expression] = STATE(263),
    [sym_type_conversion_expression] = STATE(263),
    [sym_type_instantiation_expression] = STATE(263),
    [sym_composite_literal] = STATE(263),
    [sym_func_literal] = STATE(263),
    [sym_unary_expression] = STATE(263),
    [sym_binary_expression] = STATE(263),
    [sym_qualified_type] = STATE(1027),
    [sym_raw_string_literal] = STATE(263),
    [sym_interpreted_string_literal] = STATE(263),
    [sym_identifier] = ACTIONS(186),
    [anon_sym_SEMI] = ACTIONS(9),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_var] = ACTIONS(19),
    [anon_sym_func] = ACTIONS(188),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_type] = ACTIONS(25),
    [anon_sym_STAR] = ACTIONS(27),
    [anon_sym_struct] = ACTIONS(29),
    [anon_sym_TILDE] = ACTIONS(31),
    [anon_sym_LBRACE] = ACTIONS(33),
    [anon_sym_RBRACE] = ACTIONS(230),
    [anon_sym_interface] = ACTIONS(35),
    [anon_sym_map] = ACTIONS(37),
    [anon_sym_chan] = ACTIONS(39),
    [anon_sym_LT_DASH] = ACTIONS(41),
    [anon_sym_fallthrough] = ACTIONS(43),
    [anon_sym_break] = ACTIONS(45),
    [anon_sym_continue] = ACTIONS(47),
    [anon_sym_goto] = ACTIONS(49),
    [anon_sym_return] = ACTIONS(51),
    [anon_sym_go] = ACTIONS(53),
    [anon_sym_defer] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_for] = ACTIONS(59),
    [anon_sym_switch] = ACTIONS(61),
    [anon_sym_select] = ACTIONS(63),
    [anon_sym_new] = ACTIONS(65),
    [anon_sym_make] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(67),
    [anon_sym_DASH] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_CARET] = ACTIONS(67),
    [anon_sym_AMP] = ACTIONS(67),
    [anon_sym_BQUOTE] = ACTIONS(69),
    [anon_sym_DQUOTE] = ACTIONS(71),
    [sym_int_literal] = ACTIONS(73),
    [sym_float_literal] = ACTIONS(73),
    [sym_imaginary_literal] = ACTIONS(75),
    [sym_rune_literal] = ACTIONS(75),
    [sym_nil] = ACTIONS(73),
    [sym_true] = ACTIONS(73),
    [sym_false] = ACTIONS(73),
    [sym_iota] = ACTIONS(73),
    [sym_comment] = ACTIONS(3),
  },
  [17] = {
    [sym__declaration] = STATE(985),
    [sym_const_declaration] = STATE(985),
    [sym_var_declaration] = STATE(985),
    [sym_type_declaration] = STATE(985),
    [sym_expression_list] = STATE(804),
    [sym_parenthesized_type] = STATE(1303),
    [sym__simple_type] = STATE(1303),
    [sym_generic_type] = STATE(1131),
    [sym_pointer_type] = STATE(867),
    [sym_array_type] = STATE(1131),
    [sym_implicit_length_array_type] = STATE(1293),
    [sym_slice_type] = STATE(1131),
    [sym_struct_type] = STATE(1131),
    [sym_negated_type] = STATE(850),
    [sym_interface_type] = STATE(867),
    [sym_map_type] = STATE(1131),
    [sym_channel_type] = STATE(867),
    [sym_function_type] = STATE(867),
    [sym_block] = STATE(985),
    [sym__statement_list] = STATE(1389),
    [sym__statement] = STATE(913),
    [sym_empty_statement] = STATE(985),
    [sym__simple_statement] = STATE(985),
    [sym_expression_statement] = STATE(937),
    [sym_send_statement] = STATE(937),
    [sym_inc_statement] = STATE(937),
    [sym_dec_statement] = STATE(937),
    [sym_assignment_statement] = STATE(937),
    [sym_short_var_declaration] = STATE(937),
    [sym_labeled_statement] = STATE(985),
    [sym_empty_labeled_statement] = STATE(1389),
    [sym_fallthrough_statement] = STATE(985),
    [sym_break_statement] = STATE(985),
    [sym_continue_statement] = STATE(985),
    [sym_goto_statement] = STATE(985),
    [sym_return_statement] = STATE(985),
    [sym_go_statement] = STATE(985),
    [sym_defer_statement] = STATE(985),
    [sym_if_statement] = STATE(985),
    [sym_for_statement] = STATE(985),
    [sym_expression_switch_statement] = STATE(985),
    [sym_type_switch_statement] = STATE(985),
    [sym_select_statement] = STATE(985),
    [sym__expression] = STATE(240),
    [sym_parenthesized_expression] = STATE(263),
    [sym_call_expression] = STATE(263),
    [sym_selector_expression] = STATE(263),
    [sym_index_expression] = STATE(263),
    [sym_slice_expression] = STATE(263),
    [sym_type_assertion_expression] = STATE(263),
    [sym_type_conversion_expression] = STATE(263),
    [sym_type_instantiation_expression] = STATE(263),
    [sym_composite_literal] = STATE(263),
    [sym_func_literal] = STATE(263),
    [sym_unary_expression] = STATE(263),
    [sym_binary_expression] = STATE(263),
    [sym_qualified_type] = STATE(1027),
    [sym_raw_string_literal] = STATE(263),
    [sym_interpreted_string_literal] = STATE(263),
    [sym_identifier] = ACTIONS(186),
    [anon_sym_SEMI] = ACTIONS(9),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_var] = ACTIONS(19),
    [anon_sym_func] = ACTIONS(188),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_type] = ACTIONS(25),
    [anon_sym_STAR] = ACTIONS(27),
    [anon_sym_struct] = ACTIONS(29),
    [anon_sym_TILDE] = ACTIONS(31),
    [anon_sym_LBRACE] = ACTIONS(33),
    [anon_sym_RBRACE] = ACTIONS(232),
    [anon_sym_interface] = ACTIONS(35),
    [anon_sym_map] = ACTIONS(37),
    [anon_sym_chan] = ACTIONS(39),
    [anon_sym_LT_DASH] = ACTIONS(41),
    [anon_sym_fallthrough] = ACTIONS(43),
    [anon_sym_break] = ACTIONS(45),
    [anon_sym_continue] = ACTIONS(47),
    [anon_sym_goto] = ACTIONS(49),
    [anon_sym_return] = ACTIONS(51),
    [anon_sym_go] = ACTIONS(53),
    [anon_sym_defer] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_for] = ACTIONS(59),
    [anon_sym_switch] = ACTIONS(61),
    [anon_sym_select] = ACTIONS(63),
    [anon_sym_new] = ACTIONS(65),
    [anon_sym_make] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(67),
    [anon_sym_DASH] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_CARET] = ACTIONS(67),
    [anon_sym_AMP] = ACTIONS(67),
    [anon_sym_BQUOTE] = ACTIONS(69),
    [anon_sym_DQUOTE] = ACTIONS(71),
    [sym_int_literal] = ACTIONS(73),
    [sym_float_literal] = ACTIONS(73),
    [sym_imaginary_literal] = ACTIONS(75),
    [sym_rune_literal] = ACTIONS(75),
    [sym_nil] = ACTIONS(73),
    [sym_true] = ACTIONS(73),
    [sym_false] = ACTIONS(73),
    [sym_iota] = ACTIONS(73),
    [sym_comment] = ACTIONS(3),
  },
  [18] = {
    [sym__declaration] = STATE(985),
    [sym_const_declaration] = STATE(985),
    [sym_var_declaration] = STATE(985),
    [sym_type_declaration] = STATE(985),
    [sym_expression_list] = STATE(804),
    [sym_parenthesized_type] = STATE(1303),
    [sym__simple_type] = STATE(1303),
    [sym_generic_type] = STATE(1131),
    [sym_pointer_type] = STATE(867),
    [sym_array_type] = STATE(1131),
    [sym_implicit_length_array_type] = STATE(1293),
    [sym_slice_type] = STATE(1131),
    [sym_struct_type] = STATE(1131),
    [sym_negated_type] = STATE(850),
    [sym_interface_type] = STATE(867),
    [sym_map_type] = STATE(1131),
    [sym_channel_type] = STATE(867),
    [sym_function_type] = STATE(867),
    [sym_block] = STATE(985),
    [sym__statement_list] = STATE(1387),
    [sym__statement] = STATE(913),
    [sym_empty_statement] = STATE(985),
    [sym__simple_statement] = STATE(985),
    [sym_expression_statement] = STATE(937),
    [sym_send_statement] = STATE(937),
    [sym_inc_statement] = STATE(937),
    [sym_dec_statement] = STATE(937),
    [sym_assignment_statement] = STATE(937),
    [sym_short_var_declaration] = STATE(937),
    [sym_labeled_statement] = STATE(985),
    [sym_empty_labeled_statement] = STATE(1387),
    [sym_fallthrough_statement] = STATE(985),
    [sym_break_statement] = STATE(985),
    [sym_continue_statement] = STATE(985),
    [sym_goto_statement] = STATE(985),
    [sym_return_statement] = STATE(985),
    [sym_go_statement] = STATE(985),
    [sym_defer_statement] = STATE(985),
    [sym_if_statement] = STATE(985),
    [sym_for_statement] = STATE(985),
    [sym_expression_switch_statement] = STATE(985),
    [sym_type_switch_statement] = STATE(985),
    [sym_select_statement] = STATE(985),
    [sym__expression] = STATE(240),
    [sym_parenthesized_expression] = STATE(263),
    [sym_call_expression] = STATE(263),
    [sym_selector_expression] = STATE(263),
    [sym_index_expression] = STATE(263),
    [sym_slice_expression] = STATE(263),
    [sym_type_assertion_expression] = STATE(263),
    [sym_type_conversion_expression] = STATE(263),
    [sym_type_instantiation_expression] = STATE(263),
    [sym_composite_literal] = STATE(263),
    [sym_func_literal] = STATE(263),
    [sym_unary_expression] = STATE(263),
    [sym_binary_expression] = STATE(263),
    [sym_qualified_type] = STATE(1027),
    [sym_raw_string_literal] = STATE(263),
    [sym_interpreted_string_literal] = STATE(263),
    [sym_identifier] = ACTIONS(186),
    [anon_sym_SEMI] = ACTIONS(9),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_var] = ACTIONS(19),
    [anon_sym_func] = ACTIONS(188),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_type] = ACTIONS(25),
    [anon_sym_STAR] = ACTIONS(27),
    [anon_sym_struct] = ACTIONS(29),
    [anon_sym_TILDE] = ACTIONS(31),
    [anon_sym_LBRACE] = ACTIONS(33),
    [anon_sym_RBRACE] = ACTIONS(234),
    [anon_sym_interface] = ACTIONS(35),
    [anon_sym_map] = ACTIONS(37),
    [anon_sym_chan] = ACTIONS(39),
    [anon_sym_LT_DASH] = ACTIONS(41),
    [anon_sym_fallthrough] = ACTIONS(43),
    [anon_sym_break] = ACTIONS(45),
    [anon_sym_continue] = ACTIONS(47),
    [anon_sym_goto] = ACTIONS(49),
    [anon_sym_return] = ACTIONS(51),
    [anon_sym_go] = ACTIONS(53),
    [anon_sym_defer] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_for] = ACTIONS(59),
    [anon_sym_switch] = ACTIONS(61),
    [anon_sym_select] = ACTIONS(63),
    [anon_sym_new] = ACTIONS(65),
    [anon_sym_make] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(67),
    [anon_sym_DASH] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_CARET] = ACTIONS(67),
    [anon_sym_AMP] = ACTIONS(67),
    [anon_sym_BQUOTE] = ACTIONS(69),
    [anon_sym_DQUOTE] = ACTIONS(71),
    [sym_int_literal] = ACTIONS(73),
    [sym_float_literal] = ACTIONS(73),
    [sym_imaginary_literal] = ACTIONS(75),
    [sym_rune_literal] = ACTIONS(75),
    [sym_nil] = ACTIONS(73),
    [sym_true] = ACTIONS(73),
    [sym_false] = ACTIONS(73),
    [sym_iota] = ACTIONS(73),
    [sym_comment] = ACTIONS(3),
  },
  [19] = {
    [sym__declaration] = STATE(985),
    [sym_const_declaration] = STATE(985),
    [sym_var_declaration] = STATE(985),
    [sym_type_declaration] = STATE(985),
    [sym_expression_list] = STATE(804),
    [sym_parenthesized_type] = STATE(1303),
    [sym__simple_type] = STATE(1303),
    [sym_generic_type] = STATE(1131),
    [sym_pointer_type] = STATE(867),
    [sym_array_type] = STATE(1131),
    [sym_implicit_length_array_type] = STATE(1293),
    [sym_slice_type] = STATE(1131),
    [sym_struct_type] = STATE(1131),
    [sym_negated_type] = STATE(850),
    [sym_interface_type] = STATE(867),
    [sym_map_type] = STATE(1131),
    [sym_channel_type] = STATE(867),
    [sym_function_type] = STATE(867),
    [sym_block] = STATE(985),
    [sym__statement] = STATE(1010),
    [sym_empty_statement] = STATE(985),
    [sym__simple_statement] = STATE(985),
    [sym_expression_statement] = STATE(937),
    [sym_send_statement] = STATE(937),
    [sym_inc_statement] = STATE(937),
    [sym_dec_statement] = STATE(937),
    [sym_assignment_statement] = STATE(937),
    [sym_short_var_declaration] = STATE(937),
    [sym_labeled_statement] = STATE(985),
    [sym_fallthrough_statement] = STATE(985),
    [sym_break_statement] = STATE(985),
    [sym_continue_statement] = STATE(985),
    [sym_goto_statement] = STATE(985),
    [sym_return_statement] = STATE(985),
    [sym_go_statement] = STATE(985),
    [sym_defer_statement] = STATE(985),
    [sym_if_statement] = STATE(985),
    [sym_for_statement] = STATE(985),
    [sym_expression_switch_statement] = STATE(985),
    [sym_type_switch_statement] = STATE(985),
    [sym_select_statement] = STATE(985),
    [sym__expression] = STATE(240),
    [sym_parenthesized_expression] = STATE(263),
    [sym_call_expression] = STATE(263),
    [sym_selector_expression] = STATE(263),
    [sym_index_expression] = STATE(263),
    [sym_slice_expression] = STATE(263),
    [sym_type_assertion_expression] = STATE(263),
    [sym_type_conversion_expression] = STATE(263),
    [sym_type_instantiation_expression] = STATE(263),
    [sym_composite_literal] = STATE(263),
    [sym_func_literal] = STATE(263),
    [sym_unary_expression] = STATE(263),
    [sym_binary_expression] = STATE(263),
    [sym_qualified_type] = STATE(1027),
    [sym_raw_string_literal] = STATE(263),
    [sym_interpreted_string_literal] = STATE(263),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_SEMI] = ACTIONS(9),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_var] = ACTIONS(19),
    [anon_sym_func] = ACTIONS(188),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_type] = ACTIONS(25),
    [anon_sym_STAR] = ACTIONS(27),
    [anon_sym_struct] = ACTIONS(29),
    [anon_sym_TILDE] = ACTIONS(31),
    [anon_sym_LBRACE] = ACTIONS(33),
    [anon_sym_interface] = ACTIONS(35),
    [anon_sym_map] = ACTIONS(37),
    [anon_sym_chan] = ACTIONS(39),
    [anon_sym_LT_DASH] = ACTIONS(41),
    [anon_sym_fallthrough] = ACTIONS(43),
    [anon_sym_break] = ACTIONS(45),
    [anon_sym_continue] = ACTIONS(47),
    [anon_sym_goto] = ACTIONS(49),
    [anon_sym_return] = ACTIONS(51),
    [anon_sym_go] = ACTIONS(53),
    [anon_sym_defer] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_for] = ACTIONS(59),
    [anon_sym_switch] = ACTIONS(61),
    [anon_sym_select] = ACTIONS(63),
    [anon_sym_new] = ACTIONS(65),
    [anon_sym_make] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(67),
    [anon_sym_DASH] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_CARET] = ACTIONS(67),
    [anon_sym_AMP] = ACTIONS(67),
    [anon_sym_BQUOTE] = ACTIONS(69),
    [anon_sym_DQUOTE] = ACTIONS(71),
    [sym_int_literal] = ACTIONS(73),
    [sym_float_literal] = ACTIONS(73),
    [sym_imaginary_literal] = ACTIONS(75),
    [sym_rune_literal] = ACTIONS(75),
    [sym_nil] = ACTIONS(73),
    [sym_true] = ACTIONS(73),
    [sym_false] = ACTIONS(73),
    [sym_iota] = ACTIONS(73),
    [sym_comment] = ACTIONS(3),
  },
  [20] = {
    [sym__declaration] = STATE(985),
    [sym_const_declaration] = STATE(985),
    [sym_var_declaration] = STATE(985),
    [sym_type_declaration] = STATE(985),
    [sym_expression_list] = STATE(804),
    [sym_parenthesized_type] = STATE(1303),
    [sym__simple_type] = STATE(1303),
    [sym_generic_type] = STATE(1131),
    [sym_pointer_type] = STATE(867),
    [sym_array_type] = STATE(1131),
    [sym_implicit_length_array_type] = STATE(1293),
    [sym_slice_type] = STATE(1131),
    [sym_struct_type] = STATE(1131),
    [sym_negated_type] = STATE(850),
    [sym_interface_type] = STATE(867),
    [sym_map_type] = STATE(1131),
    [sym_channel_type] = STATE(867),
    [sym_function_type] = STATE(867),
    [sym_block] = STATE(985),
    [sym__statement] = STATE(921),
    [sym_empty_statement] = STATE(985),
    [sym__simple_statement] = STATE(985),
    [sym_expression_statement] = STATE(937),
    [sym_send_statement] = STATE(937),
    [sym_inc_statement] = STATE(937),
    [sym_dec_statement] = STATE(937),
    [sym_assignment_statement] = STATE(937),
    [sym_short_var_declaration] = STATE(937),
    [sym_labeled_statement] = STATE(985),
    [sym_fallthrough_statement] = STATE(985),
    [sym_break_statement] = STATE(985),
    [sym_continue_statement] = STATE(985),
    [sym_goto_statement] = STATE(985),
    [sym_return_statement] = STATE(985),
    [sym_go_statement] = STATE(985),
    [sym_defer_statement] = STATE(985),
    [sym_if_statement] = STATE(985),
    [sym_for_statement] = STATE(985),
    [sym_expression_switch_statement] = STATE(985),
    [sym_type_switch_statement] = STATE(985),
    [sym_select_statement] = STATE(985),
    [sym__expression] = STATE(240),
    [sym_parenthesized_expression] = STATE(263),
    [sym_call_expression] = STATE(263),
    [sym_selector_expression] = STATE(263),
    [sym_index_expression] = STATE(263),
    [sym_slice_expression] = STATE(263),
    [sym_type_assertion_expression] = STATE(263),
    [sym_type_conversion_expression] = STATE(263),
    [sym_type_instantiation_expression] = STATE(263),
    [sym_composite_literal] = STATE(263),
    [sym_func_literal] = STATE(263),
    [sym_unary_expression] = STATE(263),
    [sym_binary_expression] = STATE(263),
    [sym_qualified_type] = STATE(1027),
    [sym_raw_string_literal] = STATE(263),
    [sym_interpreted_string_literal] = STATE(263),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_SEMI] = ACTIONS(9),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_var] = ACTIONS(19),
    [anon_sym_func] = ACTIONS(188),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_type] = ACTIONS(25),
    [anon_sym_STAR] = ACTIONS(27),
    [anon_sym_struct] = ACTIONS(29),
    [anon_sym_TILDE] = ACTIONS(31),
    [anon_sym_LBRACE] = ACTIONS(33),
    [anon_sym_interface] = ACTIONS(35),
    [anon_sym_map] = ACTIONS(37),
    [anon_sym_chan] = ACTIONS(39),
    [anon_sym_LT_DASH] = ACTIONS(41),
    [anon_sym_fallthrough] = ACTIONS(43),
    [anon_sym_break] = ACTIONS(45),
    [anon_sym_continue] = ACTIONS(47),
    [anon_sym_goto] = ACTIONS(49),
    [anon_sym_return] = ACTIONS(51),
    [anon_sym_go] = ACTIONS(53),
    [anon_sym_defer] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_for] = ACTIONS(59),
    [anon_sym_switch] = ACTIONS(61),
    [anon_sym_select] = ACTIONS(63),
    [anon_sym_new] = ACTIONS(65),
    [anon_sym_make] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(67),
    [anon_sym_DASH] = ACTIONS(67),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_CARET] = ACTIONS(67),
    [anon_sym_AMP] = ACTIONS(67),
    [anon_sym_BQUOTE] = ACTIONS(69),
    [anon_sym_DQUOTE] = ACTIONS(71),
    [sym_int_literal] = ACTIONS(73),
    [sym_float_literal] = ACTIONS(73),
    [sym_imaginary_literal] = ACTIONS(75),
    [sym_rune_literal] = ACTIONS(75),
    [sym_nil] = ACTIONS(73),
    [sym_true] = ACTIONS(73),
    [sym_false] = ACTIONS(73),
    [sym_iota] = ACTIONS(73),
    [sym_comment] = ACTIONS(3),
  },
  [21] = {
    [sym_expression_list] = STATE(809),
    [sym_parenthesized_type] = STATE(1320),
    [sym__simple_type] = STATE(1320),
    [sym_generic_type] = STATE(1144),
    [sym_pointer_type] = STATE(867),
    [sym_array_type] = STATE(1144),
    [sym_implicit_length_array_type] = STATE(1270),
    [sym_slice_type] = STATE(1144),
    [sym_struct_type] = STATE(1144),
    [sym_negated_type] = STATE(850),
    [sym_interface_type] = STATE(867),
    [sym_map_type] = STATE(1144),
    [sym_channel_type] = STATE(867),
    [sym_function_type] = STATE(867),
    [sym_block] = STATE(988),
    [sym__simple_statement] = STATE(1345),
    [sym_expression_statement] = STATE(1250),
    [sym_send_statement] = STATE(1250),
    [sym_inc_statement] = STATE(1250),
    [sym_dec_statement] = STATE(1250),
    [sym_assignment_statement] = STATE(1250),
    [sym_short_var_declaration] = STATE(1250),
    [sym_for_clause] = STATE(1271),
    [sym_range_clause] = STATE(1271),
    [sym__expression] = STATE(259),
    [sym_parenthesized_expression] = STATE(319),
    [sym_call_expression] = STATE(319),
    [sym_selector_expression] = STATE(319),
    [sym_index_expression] = STATE(319),
    [sym_slice_expression] = STATE(319),
    [sym_type_assertion_expression] = STATE(319),
    [sym_type_conversion_expression] = STATE(319),
    [sym_type_instantiation_expression] = STATE(319),
    [sym_composite_literal] = STATE(319),
    [sym_func_literal] = STATE(319),
    [sym_unary_expression] = STATE(319),
    [sym_binary_expression] = STATE(319),
    [sym_qualified_type] = STATE(1029),
    [sym_raw_string_literal] = STATE(319),
    [sym_interpreted_string_literal] = STATE(319),
    [sym_identifier] = ACTIONS(236),
    [anon_sym_SEMI] = ACTIONS(238),
    [anon_sym_LPAREN] = ACTIONS(240),
    [anon_sym_func] = ACTIONS(242),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(244),
    [anon_sym_struct] = ACTIONS(29),
    [anon_sym_TILDE] = ACTIONS(31),
    [anon_sym_LBRACE] = ACTIONS(33),
    [anon_sym_interface] = ACTIONS(35),
    [anon_sym_map] = ACTIONS(37),
    [anon_sym_chan] = ACTIONS(39),
    [anon_sym_LT_DASH] = ACTIONS(246),
    [anon_sym_range] = ACTIONS(248),
    [anon_sym_new] = ACTIONS(250),
    [anon_sym_make] = ACTIONS(250),
    [anon_sym_PLUS] = ACTIONS(252),
    [anon_sym_DASH] = ACTIONS(252),
    [anon_sym_BANG] = ACTIONS(252),
    [anon_sym_CARET] = ACTIONS(252),
    [anon_sym_AMP] = ACTIONS(252),
    [anon_sym_BQUOTE] = ACTIONS(254),
    [anon_sym_DQUOTE] = ACTIONS(256),
    [sym_int_literal] = ACTIONS(258),
    [sym_float_literal] = ACTIONS(258),
    [sym_imaginary_literal] = ACTIONS(260),
    [sym_rune_literal] = ACTIONS(260),
    [sym_nil] = ACTIONS(258),
    [sym_true] = ACTIONS(258),
    [sym_false] = ACTIONS(258),
    [sym_iota] = ACTIONS(258),
    [sym_comment] = ACTIONS(3),
  },
  [22] = {
    [sym_expression_list] = STATE(802),
    [sym_parenthesized_type] = STATE(1320),
    [sym__simple_type] = STATE(1320),
    [sym_generic_type] = STATE(1144),
    [sym_pointer_type] = STATE(867),
    [sym_array_type] = STATE(1144),
    [sym_implicit_length_array_type] = STATE(1270),
    [sym_slice_type] = STATE(1144),
    [sym_struct_type] = STATE(1144),
    [sym_negated_type] = STATE(850),
    [sym_interface_type] = STATE(867),
    [sym_map_type] = STATE(1144),
    [sym_channel_type] = STATE(867),
    [sym_function_type] = STATE(867),
    [sym__simple_statement] = STATE(1403),
    [sym_expression_statement] = STATE(1250),
    [sym_send_statement] = STATE(1250),
    [sym_inc_statement] = STATE(1250),
    [sym_dec_statement] = STATE(1250),
    [sym_assignment_statement] = STATE(1250),
    [sym_short_var_declaration] = STATE(1250),
    [sym__type_switch_header] = STATE(1407),
    [sym__expression] = STATE(290),
    [sym_parenthesized_expression] = STATE(319),
    [sym_call_expression] = STATE(319),
    [sym_selector_expression] = STATE(319),
    [sym_index_expression] = STATE(319),
    [sym_slice_expression] = STATE(319),
    [sym_type_assertion_expression] = STATE(319),
    [sym_type_conversion_expression] = STATE(319),
    [sym_type_instantiation_expression] = STATE(319),
    [sym_composite_literal] = STATE(319),
    [sym_func_literal] = STATE(319),
    [sym_unary_expression] = STATE(319),
    [sym_binary_expression] = STATE(319),
    [sym_qualified_type] = STATE(1029),
    [sym_raw_string_literal] = STATE(319),
    [sym_interpreted_string_literal] = STATE(319),
    [sym_identifier] = ACTIONS(236),
    [anon_sym_LPAREN] = ACTIONS(240),
    [anon_sym_func] = ACTIONS(242),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(244),
    [anon_sym_struct] = ACTIONS(29),
    [anon_sym_TILDE] = ACTIONS(31),
    [anon_sym_LBRACE] = ACTIONS(262),
    [anon_sym_interface] = ACTIONS(35),
    [anon_sym_map] = ACTIONS(37),
    [anon_sym_chan] = ACTIONS(39),
    [anon_sym_LT_DASH] = ACTIONS(246),
    [anon_sym_new] = ACTIONS(250),
    [anon_sym_make] = ACTIONS(250),
    [anon_sym_PLUS] = ACTIONS(252),
    [anon_sym_DASH] = ACTIONS(252),
    [anon_sym_BANG] = ACTIONS(252),
    [anon_sym_CARET] = ACTIONS(252),
    [anon_sym_AMP] = ACTIONS(252),
    [anon_sym_BQUOTE] = ACTIONS(254),
    [anon_sym_DQUOTE] = ACTIONS(256),
    [sym_int_literal] = ACTIONS(258),
    [sym_float_literal] = ACTIONS(258),
    [sym_imaginary_literal] = ACTIONS(260),
    [sym_rune_literal] = ACTIONS(260),
    [sym_nil] = ACTIONS(258),
    [sym_true] = ACTIONS(258),
    [sym_false] = ACTIONS(258),
    [sym_iota] = ACTIONS(258),
    [sym_comment] = ACTIONS(3),
  },
  [23] = {
    [sym_expression_list] = STATE(807),
    [sym_parenthesized_type] = STATE(1320),
    [sym__simple_type] = STATE(1320),
    [sym_generic_type] = STATE(1144),
    [sym_pointer_type] = STATE(867),
    [sym_array_type] = STATE(1144),
    [sym_implicit_length_array_type] = STATE(1270),
    [sym_slice_type] = STATE(1144),
    [sym_struct_type] = STATE(1144),
    [sym_negated_type] = STATE(850),
    [sym_interface_type] = STATE(867),
    [sym_map_type] = STATE(1144),
    [sym_channel_type] = STATE(867),
    [sym_function_type] = STATE(867),
    [sym__simple_statement] = STATE(1402),
    [sym_expression_statement] = STATE(1250),
    [sym_send_statement] = STATE(1250),
    [sym_inc_statement] = STATE(1250),
    [sym_dec_statement] = STATE(1250),
    [sym_assignment_statement] = STATE(1250),
    [sym_short_var_declaration] = STATE(1250),
    [sym__expression] = STATE(292),
    [sym_parenthesized_expression] = STATE(319),
    [sym_call_expression] = STATE(319),
    [sym_selector_expression] = STATE(319),
    [sym_index_expression] = STATE(319),
    [sym_slice_expression] = STATE(319),
    [sym_type_assertion_expression] = STATE(319),
    [sym_type_conversion_expression] = STATE(319),
    [sym_type_instantiation_expression] = STATE(319),
    [sym_composite_literal] = STATE(319),
    [sym_func_literal] = STATE(319),
    [sym_unary_expression] = STATE(319),
    [sym_binary_expression] = STATE(319),
    [sym_qualified_type] = STATE(1029),
    [sym_raw_string_literal] = STATE(319),
    [sym_interpreted_string_literal] = STATE(319),
    [sym_identifier] = ACTIONS(236),
    [anon_sym_LPAREN] = ACTIONS(240),
    [anon_sym_func] = ACTIONS(242),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(244),
    [anon_sym_struct] = ACTIONS(29),
    [anon_sym_TILDE] = ACTIONS(31),
    [anon_sym_LBRACE] = ACTIONS(264),
    [anon_sym_interface] = ACTIONS(35),
    [anon_sym_map] = ACTIONS(37),
    [anon_sym_chan] = ACTIONS(39),
    [anon_sym_LT_DASH] = ACTIONS(246),
    [anon_sym_new] = ACTIONS(250),
    [anon_sym_make] = ACTIONS(250),
    [anon_sym_PLUS] = ACTIONS(252),
    [anon_sym_DASH] = ACTIONS(252),
    [anon_sym_BANG] = ACTIONS(252),
    [anon_sym_CARET] = ACTIONS(252),
    [anon_sym_AMP] = ACTIONS(252),
    [anon_sym_BQUOTE] = ACTIONS(254),
    [anon_sym_DQUOTE] = ACTIONS(256),
    [sym_int_literal] = ACTIONS(258),
    [sym_float_literal] = ACTIONS(258),
    [sym_imaginary_literal] = ACTIONS(260),
    [sym_rune_literal] = ACTIONS(260),
    [sym_nil] = ACTIONS(258),
    [sym_true] = ACTIONS(258),
    [sym_false] = ACTIONS(258),
    [sym_iota] = ACTIONS(258),
    [sym_comment] = ACTIONS(3),
  },
  [24] = {
    [sym_expression_list] = STATE(807),
    [sym_parenthesized_type] = STATE(1320),
    [sym__simple_type] = STATE(1320),
    [sym_generic_type] = STATE(1144),
    [sym_pointer_type] = STATE(867),
    [sym_array_type] = STATE(1144),
    [sym_implicit_length_array_type] = STATE(1270),
    [sym_slice_type] = STATE(1144),
    [sym_struct_type] = STATE(1144),
    [sym_negated_type] = STATE(850),
    [sym_interface_type] = STATE(867),
    [sym_map_type] = STATE(1144),
    [sym_channel_type] = STATE(867),
    [sym_function_type] = STATE(867),
    [sym__simple_statement] = STATE(1347),
    [sym_expression_statement] = STATE(1250),
    [sym_send_statement] = STATE(1250),
    [sym_inc_statement] = STATE(1250),
    [sym_dec_statement] = STATE(1250),
    [sym_assignment_statement] = STATE(1250),
    [sym_short_var_declaration] = STATE(1250),
    [sym__expression] = STATE(292),
    [sym_parenthesized_expression] = STATE(319),
    [sym_call_expression] = STATE(319),
    [sym_selector_expression] = STATE(319),
    [sym_index_expression] = STATE(319),
    [sym_slice_expression] = STATE(319),
    [sym_type_assertion_expression] = STATE(319),
    [sym_type_conversion_expression] = STATE(319),
    [sym_type_instantiation_expression] = STATE(319),
    [sym_composite_literal] = STATE(319),
    [sym_func_literal] = STATE(319),
    [sym_unary_expression] = STATE(319),
    [sym_binary_expression] = STATE(319),
    [sym_qualified_type] = STATE(1029),
    [sym_raw_string_literal] = STATE(319),
    [sym_interpreted_string_literal] = STATE(319),
    [sym_identifier] = ACTIONS(236),
    [anon_sym_LPAREN] = ACTIONS(240),
    [anon_sym_func] = ACTIONS(242),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(244),
    [anon_sym_struct] = ACTIONS(29),
    [anon_sym_TILDE] = ACTIONS(31),
    [anon_sym_LBRACE] = ACTIONS(266),
    [anon_sym_interface] = ACTIONS(35),
    [anon_sym_map] = ACTIONS(37),
    [anon_sym_chan] = ACTIONS(39),
    [anon_sym_LT_DASH] = ACTIONS(246),
    [anon_sym_new] = ACTIONS(250),
    [anon_sym_make] = ACTIONS(250),
    [anon_sym_PLUS] = ACTIONS(252),
    [anon_sym_DASH] = ACTIONS(252),
    [anon_sym_BANG] = ACTIONS(252),
    [anon_sym_CARET] = ACTIONS(252),
    [anon_sym_AMP] = ACTIONS(252),
    [anon_sym_BQUOTE] = ACTIONS(254),
    [anon_sym_DQUOTE] = ACTIONS(256),
    [sym_int_literal] = ACTIONS(258),
    [sym_float_literal] = ACTIONS(258),
    [sym_imaginary_literal] = ACTIONS(260),
    [sym_rune_literal] = ACTIONS(260),
    [sym_nil] = ACTIONS(258),
    [sym_true] = ACTIONS(258),
    [sym_false] = ACTIONS(258),
    [sym_iota] = ACTIONS(258),
    [sym_comment] = ACTIONS(3),
  },
  [25] = {
    [sym_expression_list] = STATE(807),
    [sym_parenthesized_type] = STATE(1320),
    [sym__simple_type] = STATE(1320),
    [sym_generic_type] = STATE(1144),
    [sym_pointer_type] = STATE(867),
    [sym_array_type] = STATE(1144),
    [sym_implicit_length_array_type] = STATE(1270),
    [sym_slice_type] = STATE(1144),
    [sym_struct_type] = STATE(1144),
    [sym_negated_type] = STATE(850),
    [sym_interface_type] = STATE(867),
    [sym_map_type] = STATE(1144),
    [sym_channel_type] = STATE(867),
    [sym_function_type] = STATE(867),
    [sym__simple_statement] = STATE(1351),
    [sym_expression_statement] = STATE(1250),
    [sym_send_statement] = STATE(1250),
    [sym_inc_statement] = STATE(1250),
    [sym_dec_statement] = STATE(1250),
    [sym_assignment_statement] = STATE(1250),
    [sym_short_var_declaration] = STATE(1250),
    [sym__expression] = STATE(292),
    [sym_parenthesized_expression] = STATE(319),
    [sym_call_expression] = STATE(319),
    [sym_selector_expression] = STATE(319),
    [sym_index_expression] = STATE(319),
    [sym_slice_expression] = STATE(319),
    [sym_type_assertion_expression] = STATE(319),
    [sym_type_conversion_expression] = STATE(319),
    [sym_type_instantiation_expression] = STATE(319),
    [sym_composite_literal] = STATE(319),
    [sym_func_literal] = STATE(319),
    [sym_unary_expression] = STATE(319),
    [sym_binary_expression] = STATE(319),
    [sym_qualified_type] = STATE(1029),
    [sym_raw_string_literal] = STATE(319),
    [sym_interpreted_string_literal] = STATE(319),
    [sym_identifier] = ACTIONS(236),
    [anon_sym_LPAREN] = ACTIONS(240),
    [anon_sym_func] = ACTIONS(242),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(244),
    [anon_sym_struct] = ACTIONS(29),
    [anon_sym_TILDE] = ACTIONS(31),
    [anon_sym_LBRACE] = ACTIONS(268),
    [anon_sym_interface] = ACTIONS(35),
    [anon_sym_map] = ACTIONS(37),
    [anon_sym_chan] = ACTIONS(39),
    [anon_sym_LT_DASH] = ACTIONS(246),
    [anon_sym_new] = ACTIONS(250),
    [anon_sym_make] = ACTIONS(250),
    [anon_sym_PLUS] = ACTIONS(252),
    [anon_sym_DASH] = ACTIONS(252),
    [anon_sym_BANG] = ACTIONS(252),
    [anon_sym_CARET] = ACTIONS(252),
    [anon_sym_AMP] = ACTIONS(252),
    [anon_sym_BQUOTE] = ACTIONS(254),
    [anon_sym_DQUOTE] = ACTIONS(256),
    [sym_int_literal] = ACTIONS(258),
    [sym_float_literal] = ACTIONS(258),
    [sym_imaginary_literal] = ACTIONS(260),
    [sym_rune_literal] = ACTIONS(260),
    [sym_nil] = ACTIONS(258),
    [sym_true] = ACTIONS(258),
    [sym_false] = ACTIONS(258),
    [sym_iota] = ACTIONS(258),
    [sym_comment] = ACTIONS(3),
  },
  [26] = {
    [sym_expression_list] = STATE(807),
    [sym_parenthesized_type] = STATE(1320),
    [sym__simple_type] = STATE(1320),
    [sym_generic_type] = STATE(1144),
    [sym_pointer_type] = STATE(867),
    [sym_array_type] = STATE(1144),
    [sym_implicit_length_array_type] = STATE(1270),
    [sym_slice_type] = STATE(1144),
    [sym_struct_type] = STATE(1144),
    [sym_negated_type] = STATE(850),
    [sym_interface_type] = STATE(867),
    [sym_map_type] = STATE(1144),
    [sym_channel_type] = STATE(867),
    [sym_function_type] = STATE(867),
    [sym__simple_statement] = STATE(1324),
    [sym_expression_statement] = STATE(1250),
    [sym_send_statement] = STATE(1250),
    [sym_inc_statement] = STATE(1250),
    [sym_dec_statement] = STATE(1250),
    [sym_assignment_statement] = STATE(1250),
    [sym_short_var_declaration] = STATE(1250),
    [sym__expression] = STATE(292),
    [sym_parenthesized_expression] = STATE(319),
    [sym_call_expression] = STATE(319),
    [sym_selector_expression] = STATE(319),
    [sym_index_expression] = STATE(319),
    [sym_slice_expression] = STATE(319),
    [sym_type_assertion_expression] = STATE(319),
    [sym_type_conversion_expression] = STATE(319),
    [sym_type_instantiation_expression] = STATE(319),
    [sym_composite_literal] = STATE(319),
    [sym_func_literal] = STATE(319),
    [sym_unary_expression] = STATE(319),
    [sym_binary_expression] = STATE(319),
    [sym_qualified_type] = STATE(1029),
    [sym_raw_string_literal] = STATE(319),
    [sym_interpreted_string_literal] = STATE(319),
    [sym_identifier] = ACTIONS(236),
    [anon_sym_LPAREN] = ACTIONS(240),
    [anon_sym_func] = ACTIONS(242),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(244),
    [anon_sym_struct] = ACTIONS(29),
    [anon_sym_TILDE] = ACTIONS(31),
    [anon_sym_LBRACE] = ACTIONS(270),
    [anon_sym_interface] = ACTIONS(35),
    [anon_sym_map] = ACTIONS(37),
    [anon_sym_chan] = ACTIONS(39),
    [anon_sym_LT_DASH] = ACTIONS(246),
    [anon_sym_new] = ACTIONS(250),
    [anon_sym_make] = ACTIONS(250),
    [anon_sym_PLUS] = ACTIONS(252),
    [anon_sym_DASH] = ACTIONS(252),
    [anon_sym_BANG] = ACTIONS(252),
    [anon_sym_CARET] = ACTIONS(252),
    [anon_sym_AMP] = ACTIONS(252),
    [anon_sym_BQUOTE] = ACTIONS(254),
    [anon_sym_DQUOTE] = ACTIONS(256),
    [sym_int_literal] = ACTIONS(258),
    [sym_float_literal] = ACTIONS(258),
    [sym_imaginary_literal] = ACTIONS(260),
    [sym_rune_literal] = ACTIONS(260),
    [sym_nil] = ACTIONS(258),
    [sym_true] = ACTIONS(258),
    [sym_false] = ACTIONS(258),
    [sym_iota] = ACTIONS(258),
    [sym_comment] = ACTIONS(3),
  },
  [27] = {
    [sym_expression_list] = STATE(808),
    [sym_parenthesized_type] = STATE(1320),
    [sym__simple_type] = STATE(1320),
    [sym_generic_type] = STATE(1144),
    [sym_pointer_type] = STATE(867),
    [sym_array_type] = STATE(1144),
    [sym_implicit_length_array_type] = STATE(1270),
    [sym_slice_type] = STATE(1144),
    [sym_struct_type] = STATE(1144),
    [sym_negated_type] = STATE(850),
    [sym_interface_type] = STATE(867),
    [sym_map_type] = STATE(1144),
    [sym_channel_type] = STATE(867),
    [sym_function_type] = STATE(867),
    [sym__simple_statement] = STATE(1330),
    [sym_expression_statement] = STATE(1250),
    [sym_send_statement] = STATE(1250),
    [sym_inc_statement] = STATE(1250),
    [sym_dec_statement] = STATE(1250),
    [sym_assignment_statement] = STATE(1250),
    [sym_short_var_declaration] = STATE(1250),
    [sym__expression] = STATE(256),
    [sym_parenthesized_expression] = STATE(319),
    [sym_call_expression] = STATE(319),
    [sym_selector_expression] = STATE(319),
    [sym_index_expression] = STATE(319),
    [sym_slice_expression] = STATE(319),
    [sym_type_assertion_expression] = STATE(319),
    [sym_type_conversion_expression] = STATE(319),
    [sym_type_instantiation_expression] = STATE(319),
    [sym_composite_literal] = STATE(319),
    [sym_func_literal] = STATE(319),
    [sym_unary_expression] = STATE(319),
    [sym_binary_expression] = STATE(319),
    [sym_qualified_type] = STATE(1029),
    [sym_raw_string_literal] = STATE(319),
    [sym_interpreted_string_literal] = STATE(319),
    [sym_identifier] = ACTIONS(236),
    [anon_sym_LPAREN] = ACTIONS(240),
    [anon_sym_func] = ACTIONS(242),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(244),
    [anon_sym_struct] = ACTIONS(29),
    [anon_sym_TILDE] = ACTIONS(31),
    [anon_sym_interface] = ACTIONS(35),
    [anon_sym_map] = ACTIONS(37),
    [anon_sym_chan] = ACTIONS(39),
    [anon_sym_LT_DASH] = ACTIONS(246),
    [anon_sym_new] = ACTIONS(250),
    [anon_sym_make] = ACTIONS(250),
    [anon_sym_PLUS] = ACTIONS(252),
    [anon_sym_DASH] = ACTIONS(252),
    [anon_sym_BANG] = ACTIONS(252),
    [anon_sym_CARET] = ACTIONS(252),
    [anon_sym_AMP] = ACTIONS(252),
    [anon_sym_BQUOTE] = ACTIONS(254),
    [anon_sym_DQUOTE] = ACTIONS(256),
    [sym_int_literal] = ACTIONS(258),
    [sym_float_literal] = ACTIONS(258),
    [sym_imaginary_literal] = ACTIONS(260),
    [sym_rune_literal] = ACTIONS(260),
    [sym_nil] = ACTIONS(258),
    [sym_true] = ACTIONS(258),
    [sym_false] = ACTIONS(258),
    [sym_iota] = ACTIONS(258),
    [sym_comment] = ACTIONS(3),
  },
  [28] = {
    [sym_expression_list] = STATE(976),
    [sym_parenthesized_type] = STATE(1319),
    [sym__simple_type] = STATE(1319),
    [sym_generic_type] = STATE(1082),
    [sym_pointer_type] = STATE(867),
    [sym_array_type] = STATE(1082),
    [sym_implicit_length_array_type] = STATE(1277),
    [sym_slice_type] = STATE(1082),
    [sym_struct_type] = STATE(1082),
    [sym_negated_type] = STATE(850),
    [sym_interface_type] = STATE(867),
    [sym_map_type] = STATE(1082),
    [sym_channel_type] = STATE(867),
    [sym_function_type] = STATE(867),
    [sym__expression] = STATE(424),
    [sym_parenthesized_expression] = STATE(509),
    [sym_call_expression] = STATE(509),
    [sym_selector_expression] = STATE(509),
    [sym_index_expression] = STATE(509),
    [sym_slice_expression] = STATE(509),
    [sym_type_assertion_expression] = STATE(509),
    [sym_type_conversion_expression] = STATE(509),
    [sym_type_instantiation_expression] = STATE(509),
    [sym_composite_literal] = STATE(509),
    [sym_func_literal] = STATE(509),
    [sym_unary_expression] = STATE(509),
    [sym_binary_expression] = STATE(509),
    [sym_qualified_type] = STATE(1055),
    [sym_raw_string_literal] = STATE(509),
    [sym_interpreted_string_literal] = STATE(509),
    [sym_identifier] = ACTIONS(272),
    [aux_sym_source_file_token1] = ACTIONS(274),
    [anon_sym_SEMI] = ACTIONS(276),
    [anon_sym_NULL] = ACTIONS(276),
    [anon_sym_LPAREN] = ACTIONS(278),
    [anon_sym_func] = ACTIONS(280),
    [anon_sym_LBRACK] = ACTIONS(282),
    [anon_sym_STAR] = ACTIONS(284),
    [anon_sym_struct] = ACTIONS(29),
    [anon_sym_TILDE] = ACTIONS(286),
    [anon_sym_RBRACE] = ACTIONS(276),
    [anon_sym_interface] = ACTIONS(35),
    [anon_sym_map] = ACTIONS(37),
    [anon_sym_chan] = ACTIONS(39),
    [anon_sym_LT_DASH] = ACTIONS(288),
    [anon_sym_case] = ACTIONS(276),
    [anon_sym_default] = ACTIONS(276),
    [anon_sym_new] = ACTIONS(290),
    [anon_sym_make] = ACTIONS(290),
    [anon_sym_PLUS] = ACTIONS(292),
    [anon_sym_DASH] = ACTIONS(292),
    [anon_sym_BANG] = ACTIONS(292),
    [anon_sym_CARET] = ACTIONS(292),
    [anon_sym_AMP] = ACTIONS(292),
    [anon_sym_BQUOTE] = ACTIONS(294),
    [anon_sym_DQUOTE] = ACTIONS(296),
    [sym_int_literal] = ACTIONS(298),
    [sym_float_literal] = ACTIONS(298),
    [sym_imaginary_literal] = ACTIONS(298),
    [sym_rune_literal] = ACTIONS(298),
    [sym_nil] = ACTIONS(298),
    [sym_true] = ACTIONS(298),
    [sym_false] = ACTIONS(298),
    [sym_iota] = ACTIONS(298),
    [sym_comment] = ACTIONS(300),
  },
};

static const uint16_t ts_small_parse_table[] = {
  [0] = 32,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(306), 1,
      anon_sym_COMMA,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(312), 1,
      anon_sym_LBRACE,
    ACTIONS(314), 1,
      anon_sym_RBRACE,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    STATE(572), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1067), 1,
      sym_literal_element,
    STATE(1224), 1,
      sym_literal_value,
    STATE(1245), 1,
      sym_keyed_element,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [129] = 32,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(312), 1,
      anon_sym_LBRACE,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(330), 1,
      anon_sym_COMMA,
    ACTIONS(332), 1,
      anon_sym_RBRACE,
    STATE(572), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1075), 1,
      sym_literal_element,
    STATE(1167), 1,
      sym_keyed_element,
    STATE(1224), 1,
      sym_literal_value,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [258] = 32,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(312), 1,
      anon_sym_LBRACE,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(334), 1,
      anon_sym_COMMA,
    ACTIONS(336), 1,
      anon_sym_RBRACE,
    STATE(572), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1130), 1,
      sym_literal_element,
    STATE(1174), 1,
      sym_keyed_element,
    STATE(1224), 1,
      sym_literal_value,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [387] = 32,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(312), 1,
      anon_sym_LBRACE,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(338), 1,
      anon_sym_COMMA,
    ACTIONS(340), 1,
      anon_sym_RBRACE,
    STATE(572), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1103), 1,
      sym_literal_element,
    STATE(1224), 1,
      sym_literal_value,
    STATE(1227), 1,
      sym_keyed_element,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [516] = 32,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(312), 1,
      anon_sym_LBRACE,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(342), 1,
      anon_sym_COMMA,
    ACTIONS(344), 1,
      anon_sym_RBRACE,
    STATE(572), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1074), 1,
      sym_literal_element,
    STATE(1196), 1,
      sym_keyed_element,
    STATE(1224), 1,
      sym_literal_value,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [645] = 32,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(312), 1,
      anon_sym_LBRACE,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(346), 1,
      anon_sym_COMMA,
    ACTIONS(348), 1,
      anon_sym_RBRACE,
    STATE(572), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1097), 1,
      sym_literal_element,
    STATE(1213), 1,
      sym_keyed_element,
    STATE(1224), 1,
      sym_literal_value,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [774] = 31,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(312), 1,
      anon_sym_LBRACE,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(350), 1,
      anon_sym_RBRACE,
    STATE(572), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1150), 1,
      sym_literal_element,
    STATE(1224), 1,
      sym_literal_value,
    STATE(1253), 1,
      sym_keyed_element,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [900] = 31,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(312), 1,
      anon_sym_LBRACE,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(352), 1,
      anon_sym_RBRACE,
    STATE(572), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1150), 1,
      sym_literal_element,
    STATE(1224), 1,
      sym_literal_value,
    STATE(1253), 1,
      sym_keyed_element,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [1026] = 31,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(312), 1,
      anon_sym_LBRACE,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(354), 1,
      anon_sym_RBRACE,
    STATE(572), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1150), 1,
      sym_literal_element,
    STATE(1224), 1,
      sym_literal_value,
    STATE(1253), 1,
      sym_keyed_element,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [1152] = 31,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(312), 1,
      anon_sym_LBRACE,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(356), 1,
      anon_sym_RBRACE,
    STATE(572), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1150), 1,
      sym_literal_element,
    STATE(1224), 1,
      sym_literal_value,
    STATE(1253), 1,
      sym_keyed_element,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [1278] = 31,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(312), 1,
      anon_sym_LBRACE,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(358), 1,
      anon_sym_RBRACE,
    STATE(572), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1150), 1,
      sym_literal_element,
    STATE(1224), 1,
      sym_literal_value,
    STATE(1253), 1,
      sym_keyed_element,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [1404] = 31,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(312), 1,
      anon_sym_LBRACE,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(360), 1,
      anon_sym_RBRACE,
    STATE(572), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1150), 1,
      sym_literal_element,
    STATE(1224), 1,
      sym_literal_value,
    STATE(1253), 1,
      sym_keyed_element,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [1530] = 31,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(312), 1,
      anon_sym_LBRACE,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(362), 1,
      anon_sym_RBRACE,
    STATE(572), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1150), 1,
      sym_literal_element,
    STATE(1224), 1,
      sym_literal_value,
    STATE(1253), 1,
      sym_keyed_element,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [1656] = 31,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(312), 1,
      anon_sym_LBRACE,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(364), 1,
      anon_sym_RBRACE,
    STATE(572), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1150), 1,
      sym_literal_element,
    STATE(1224), 1,
      sym_literal_value,
    STATE(1253), 1,
      sym_keyed_element,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [1782] = 31,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(312), 1,
      anon_sym_LBRACE,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(366), 1,
      anon_sym_RBRACE,
    STATE(572), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1150), 1,
      sym_literal_element,
    STATE(1224), 1,
      sym_literal_value,
    STATE(1253), 1,
      sym_keyed_element,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [1908] = 31,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(312), 1,
      anon_sym_LBRACE,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(368), 1,
      anon_sym_RBRACE,
    STATE(572), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1150), 1,
      sym_literal_element,
    STATE(1224), 1,
      sym_literal_value,
    STATE(1253), 1,
      sym_keyed_element,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [2034] = 31,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(312), 1,
      anon_sym_LBRACE,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(370), 1,
      anon_sym_RBRACE,
    STATE(572), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1150), 1,
      sym_literal_element,
    STATE(1224), 1,
      sym_literal_value,
    STATE(1253), 1,
      sym_keyed_element,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [2160] = 31,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(312), 1,
      anon_sym_LBRACE,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(372), 1,
      anon_sym_RBRACE,
    STATE(572), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1150), 1,
      sym_literal_element,
    STATE(1224), 1,
      sym_literal_value,
    STATE(1253), 1,
      sym_keyed_element,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [2286] = 30,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(312), 1,
      anon_sym_LBRACE,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    STATE(572), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1150), 1,
      sym_literal_element,
    STATE(1224), 1,
      sym_literal_value,
    STATE(1253), 1,
      sym_keyed_element,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [2409] = 29,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(312), 1,
      anon_sym_LBRACE,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    STATE(572), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1224), 1,
      sym_literal_value,
    STATE(1269), 1,
      sym_literal_element,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [2529] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(374), 1,
      sym_identifier,
    ACTIONS(376), 1,
      anon_sym_LPAREN,
    ACTIONS(378), 1,
      anon_sym_func,
    ACTIONS(380), 1,
      anon_sym_STAR,
    ACTIONS(382), 1,
      anon_sym_LT_DASH,
    ACTIONS(388), 1,
      anon_sym_BQUOTE,
    ACTIONS(390), 1,
      anon_sym_DQUOTE,
    STATE(460), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1031), 1,
      sym_qualified_type,
    STATE(1273), 1,
      sym_expression_list,
    STATE(1307), 1,
      sym_implicit_length_array_type,
    ACTIONS(384), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(394), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1322), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(1354), 2,
      sym_send_statement,
      sym_receive_statement,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(386), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1089), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(392), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(590), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [2647] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(396), 1,
      sym_identifier,
    ACTIONS(398), 1,
      anon_sym_STAR,
    ACTIONS(400), 1,
      anon_sym_LBRACE,
    ACTIONS(402), 1,
      anon_sym_LT_DASH,
    STATE(523), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    STATE(1362), 1,
      sym_expression_list,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(404), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [2764] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(406), 1,
      sym_identifier,
    ACTIONS(408), 1,
      anon_sym_STAR,
    ACTIONS(410), 1,
      anon_sym_COLON,
    STATE(631), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1215), 1,
      sym_type_elem,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1019), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [2881] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(414), 1,
      anon_sym_RPAREN,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    STATE(575), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    STATE(1290), 1,
      sym_variadic_argument,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [2998] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(422), 1,
      anon_sym_range,
    STATE(529), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    STATE(1275), 1,
      sym_expression_list,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [3115] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(422), 1,
      anon_sym_range,
    STATE(529), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    STATE(1282), 1,
      sym_expression_list,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [3232] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    ACTIONS(424), 1,
      anon_sym_RPAREN,
    STATE(520), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1169), 1,
      sym_variadic_argument,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [3349] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(406), 1,
      sym_identifier,
    ACTIONS(408), 1,
      anon_sym_STAR,
    ACTIONS(426), 1,
      anon_sym_RBRACK,
    STATE(669), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1165), 1,
      sym_type_elem,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1019), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [3466] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(406), 1,
      sym_identifier,
    ACTIONS(408), 1,
      anon_sym_STAR,
    ACTIONS(428), 1,
      anon_sym_COLON,
    STATE(639), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1215), 1,
      sym_type_elem,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1019), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [3583] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    ACTIONS(430), 1,
      anon_sym_RPAREN,
    STATE(575), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    STATE(1290), 1,
      sym_variadic_argument,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [3700] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(406), 1,
      sym_identifier,
    ACTIONS(408), 1,
      anon_sym_STAR,
    ACTIONS(432), 1,
      anon_sym_COLON,
    STATE(643), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1215), 1,
      sym_type_elem,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1019), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [3817] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    ACTIONS(434), 1,
      anon_sym_RPAREN,
    STATE(575), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    STATE(1290), 1,
      sym_variadic_argument,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [3934] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(436), 1,
      anon_sym_RBRACK,
    ACTIONS(438), 1,
      anon_sym_DOT_DOT_DOT,
    STATE(666), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [4051] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    ACTIONS(440), 1,
      anon_sym_RPAREN,
    STATE(526), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1201), 1,
      sym_variadic_argument,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [4168] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    ACTIONS(442), 1,
      anon_sym_RPAREN,
    STATE(575), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    STATE(1290), 1,
      sym_variadic_argument,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [4285] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    ACTIONS(444), 1,
      anon_sym_RPAREN,
    STATE(575), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    STATE(1290), 1,
      sym_variadic_argument,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [4402] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    ACTIONS(446), 1,
      anon_sym_RPAREN,
    STATE(575), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    STATE(1290), 1,
      sym_variadic_argument,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [4519] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    ACTIONS(448), 1,
      anon_sym_RPAREN,
    STATE(530), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1176), 1,
      sym_variadic_argument,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [4636] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    ACTIONS(450), 1,
      anon_sym_RPAREN,
    STATE(575), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    STATE(1290), 1,
      sym_variadic_argument,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [4753] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    ACTIONS(452), 1,
      anon_sym_RPAREN,
    STATE(575), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    STATE(1290), 1,
      sym_variadic_argument,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [4870] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    ACTIONS(454), 1,
      anon_sym_RPAREN,
    STATE(531), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1199), 1,
      sym_variadic_argument,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [4987] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    ACTIONS(456), 1,
      anon_sym_RPAREN,
    STATE(575), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    STATE(1290), 1,
      sym_variadic_argument,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [5104] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    ACTIONS(458), 1,
      anon_sym_RPAREN,
    STATE(575), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    STATE(1290), 1,
      sym_variadic_argument,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [5221] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    ACTIONS(460), 1,
      anon_sym_RPAREN,
    STATE(518), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1216), 1,
      sym_variadic_argument,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [5338] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(426), 1,
      anon_sym_RBRACK,
    ACTIONS(462), 1,
      sym_identifier,
    STATE(669), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1171), 1,
      sym_type_parameter_declaration,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [5455] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    ACTIONS(464), 1,
      anon_sym_RPAREN,
    STATE(575), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    STATE(1290), 1,
      sym_variadic_argument,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [5572] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    ACTIONS(466), 1,
      anon_sym_RPAREN,
    STATE(575), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    STATE(1290), 1,
      sym_variadic_argument,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [5689] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(406), 1,
      sym_identifier,
    ACTIONS(408), 1,
      anon_sym_STAR,
    ACTIONS(468), 1,
      anon_sym_RBRACK,
    STATE(654), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1215), 1,
      sym_type_elem,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1019), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [5806] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(406), 1,
      sym_identifier,
    ACTIONS(408), 1,
      anon_sym_STAR,
    ACTIONS(470), 1,
      anon_sym_COLON,
    STATE(605), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1215), 1,
      sym_type_elem,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1019), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [5923] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(406), 1,
      sym_identifier,
    ACTIONS(408), 1,
      anon_sym_STAR,
    ACTIONS(472), 1,
      anon_sym_COLON,
    STATE(610), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1215), 1,
      sym_type_elem,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1019), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [6040] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(406), 1,
      sym_identifier,
    ACTIONS(408), 1,
      anon_sym_STAR,
    ACTIONS(474), 1,
      anon_sym_COLON,
    STATE(624), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1215), 1,
      sym_type_elem,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1019), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [6157] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    ACTIONS(476), 1,
      anon_sym_RPAREN,
    STATE(515), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1236), 1,
      sym_variadic_argument,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [6274] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    ACTIONS(478), 1,
      anon_sym_RPAREN,
    STATE(606), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [6388] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    ACTIONS(480), 1,
      anon_sym_RPAREN,
    STATE(606), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [6502] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(436), 1,
      anon_sym_RBRACK,
    STATE(666), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [6616] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(468), 1,
      anon_sym_RBRACK,
    STATE(654), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [6730] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(482), 1,
      anon_sym_RBRACK,
    STATE(620), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [6844] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    ACTIONS(484), 1,
      anon_sym_RPAREN,
    STATE(606), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [6958] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    STATE(529), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    STATE(1367), 1,
      sym_expression_list,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [7072] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(396), 1,
      sym_identifier,
    ACTIONS(398), 1,
      anon_sym_STAR,
    ACTIONS(402), 1,
      anon_sym_LT_DASH,
    STATE(536), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    STATE(1275), 1,
      sym_expression_list,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(404), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [7186] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(486), 1,
      anon_sym_RBRACK,
    STATE(604), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [7300] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(488), 1,
      anon_sym_RBRACK,
    STATE(676), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [7414] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(490), 1,
      anon_sym_RBRACK,
    STATE(626), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [7528] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(396), 1,
      sym_identifier,
    ACTIONS(398), 1,
      anon_sym_STAR,
    ACTIONS(402), 1,
      anon_sym_LT_DASH,
    STATE(536), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    STATE(1282), 1,
      sym_expression_list,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(404), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [7642] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(492), 1,
      anon_sym_RBRACK,
    STATE(625), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [7756] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    ACTIONS(494), 1,
      anon_sym_RPAREN,
    STATE(606), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [7870] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(272), 1,
      sym_identifier,
    ACTIONS(280), 1,
      anon_sym_func,
    ACTIONS(496), 1,
      anon_sym_LPAREN,
    ACTIONS(498), 1,
      anon_sym_STAR,
    ACTIONS(500), 1,
      anon_sym_LT_DASH,
    ACTIONS(504), 1,
      anon_sym_BQUOTE,
    ACTIONS(506), 1,
      anon_sym_DQUOTE,
    STATE(424), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(999), 1,
      sym_expression_list,
    STATE(1055), 1,
      sym_qualified_type,
    STATE(1277), 1,
      sym_implicit_length_array_type,
    ACTIONS(290), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(508), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1319), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(502), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1082), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(298), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(509), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [7984] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(272), 1,
      sym_identifier,
    ACTIONS(280), 1,
      anon_sym_func,
    ACTIONS(496), 1,
      anon_sym_LPAREN,
    ACTIONS(498), 1,
      anon_sym_STAR,
    ACTIONS(500), 1,
      anon_sym_LT_DASH,
    ACTIONS(504), 1,
      anon_sym_BQUOTE,
    ACTIONS(506), 1,
      anon_sym_DQUOTE,
    STATE(424), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1014), 1,
      sym_expression_list,
    STATE(1055), 1,
      sym_qualified_type,
    STATE(1277), 1,
      sym_implicit_length_array_type,
    ACTIONS(290), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(508), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1319), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(502), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1082), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(298), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(509), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [8098] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(510), 1,
      anon_sym_RBRACK,
    STATE(627), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [8212] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    ACTIONS(512), 1,
      anon_sym_RPAREN,
    STATE(606), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [8326] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(272), 1,
      sym_identifier,
    ACTIONS(280), 1,
      anon_sym_func,
    ACTIONS(496), 1,
      anon_sym_LPAREN,
    ACTIONS(498), 1,
      anon_sym_STAR,
    ACTIONS(500), 1,
      anon_sym_LT_DASH,
    ACTIONS(504), 1,
      anon_sym_BQUOTE,
    ACTIONS(506), 1,
      anon_sym_DQUOTE,
    STATE(424), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1000), 1,
      sym_expression_list,
    STATE(1055), 1,
      sym_qualified_type,
    STATE(1277), 1,
      sym_implicit_length_array_type,
    ACTIONS(290), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(508), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1319), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(502), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1082), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(298), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(509), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [8440] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(514), 1,
      anon_sym_RBRACK,
    STATE(617), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [8554] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(272), 1,
      sym_identifier,
    ACTIONS(280), 1,
      anon_sym_func,
    ACTIONS(496), 1,
      anon_sym_LPAREN,
    ACTIONS(498), 1,
      anon_sym_STAR,
    ACTIONS(500), 1,
      anon_sym_LT_DASH,
    ACTIONS(504), 1,
      anon_sym_BQUOTE,
    ACTIONS(506), 1,
      anon_sym_DQUOTE,
    STATE(424), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(991), 1,
      sym_expression_list,
    STATE(1055), 1,
      sym_qualified_type,
    STATE(1277), 1,
      sym_implicit_length_array_type,
    ACTIONS(290), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(508), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1319), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(502), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1082), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(298), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(509), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [8668] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(516), 1,
      anon_sym_RBRACK,
    STATE(684), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [8782] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(272), 1,
      sym_identifier,
    ACTIONS(280), 1,
      anon_sym_func,
    ACTIONS(496), 1,
      anon_sym_LPAREN,
    ACTIONS(498), 1,
      anon_sym_STAR,
    ACTIONS(500), 1,
      anon_sym_LT_DASH,
    ACTIONS(504), 1,
      anon_sym_BQUOTE,
    ACTIONS(506), 1,
      anon_sym_DQUOTE,
    STATE(424), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(994), 1,
      sym_expression_list,
    STATE(1055), 1,
      sym_qualified_type,
    STATE(1277), 1,
      sym_implicit_length_array_type,
    ACTIONS(290), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(508), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1319), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(502), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1082), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(298), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(509), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [8896] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(272), 1,
      sym_identifier,
    ACTIONS(280), 1,
      anon_sym_func,
    ACTIONS(496), 1,
      anon_sym_LPAREN,
    ACTIONS(498), 1,
      anon_sym_STAR,
    ACTIONS(500), 1,
      anon_sym_LT_DASH,
    ACTIONS(504), 1,
      anon_sym_BQUOTE,
    ACTIONS(506), 1,
      anon_sym_DQUOTE,
    STATE(424), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1013), 1,
      sym_expression_list,
    STATE(1055), 1,
      sym_qualified_type,
    STATE(1277), 1,
      sym_implicit_length_array_type,
    ACTIONS(290), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(508), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1319), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(502), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1082), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(298), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(509), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [9010] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(518), 1,
      anon_sym_RBRACK,
    STATE(615), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [9124] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    ACTIONS(520), 1,
      anon_sym_RPAREN,
    STATE(606), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [9238] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(272), 1,
      sym_identifier,
    ACTIONS(280), 1,
      anon_sym_func,
    ACTIONS(496), 1,
      anon_sym_LPAREN,
    ACTIONS(498), 1,
      anon_sym_STAR,
    ACTIONS(500), 1,
      anon_sym_LT_DASH,
    ACTIONS(504), 1,
      anon_sym_BQUOTE,
    ACTIONS(506), 1,
      anon_sym_DQUOTE,
    STATE(424), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1002), 1,
      sym_expression_list,
    STATE(1055), 1,
      sym_qualified_type,
    STATE(1277), 1,
      sym_implicit_length_array_type,
    ACTIONS(290), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(508), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1319), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(502), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1082), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(298), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(509), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [9352] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    ACTIONS(522), 1,
      anon_sym_RPAREN,
    STATE(606), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [9466] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(524), 1,
      anon_sym_RBRACK,
    STATE(633), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [9580] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    ACTIONS(526), 1,
      anon_sym_RPAREN,
    STATE(606), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [9694] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(272), 1,
      sym_identifier,
    ACTIONS(280), 1,
      anon_sym_func,
    ACTIONS(496), 1,
      anon_sym_LPAREN,
    ACTIONS(498), 1,
      anon_sym_STAR,
    ACTIONS(500), 1,
      anon_sym_LT_DASH,
    ACTIONS(504), 1,
      anon_sym_BQUOTE,
    ACTIONS(506), 1,
      anon_sym_DQUOTE,
    STATE(424), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(980), 1,
      sym_expression_list,
    STATE(1055), 1,
      sym_qualified_type,
    STATE(1277), 1,
      sym_implicit_length_array_type,
    ACTIONS(290), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(508), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1319), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(502), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1082), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(298), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(509), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [9808] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    STATE(575), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    STATE(1290), 1,
      sym_variadic_argument,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [9922] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(272), 1,
      sym_identifier,
    ACTIONS(280), 1,
      anon_sym_func,
    ACTIONS(496), 1,
      anon_sym_LPAREN,
    ACTIONS(498), 1,
      anon_sym_STAR,
    ACTIONS(500), 1,
      anon_sym_LT_DASH,
    ACTIONS(504), 1,
      anon_sym_BQUOTE,
    ACTIONS(506), 1,
      anon_sym_DQUOTE,
    STATE(424), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(995), 1,
      sym_expression_list,
    STATE(1055), 1,
      sym_qualified_type,
    STATE(1277), 1,
      sym_implicit_length_array_type,
    ACTIONS(290), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(508), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1319), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(502), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1082), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(298), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(509), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [10036] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(528), 1,
      anon_sym_RBRACK,
    STATE(691), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [10150] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(272), 1,
      sym_identifier,
    ACTIONS(280), 1,
      anon_sym_func,
    ACTIONS(496), 1,
      anon_sym_LPAREN,
    ACTIONS(498), 1,
      anon_sym_STAR,
    ACTIONS(500), 1,
      anon_sym_LT_DASH,
    ACTIONS(504), 1,
      anon_sym_BQUOTE,
    ACTIONS(506), 1,
      anon_sym_DQUOTE,
    STATE(424), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(996), 1,
      sym_expression_list,
    STATE(1055), 1,
      sym_qualified_type,
    STATE(1277), 1,
      sym_implicit_length_array_type,
    ACTIONS(290), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(508), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1319), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(502), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1082), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(298), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(509), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [10264] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(272), 1,
      sym_identifier,
    ACTIONS(280), 1,
      anon_sym_func,
    ACTIONS(496), 1,
      anon_sym_LPAREN,
    ACTIONS(498), 1,
      anon_sym_STAR,
    ACTIONS(500), 1,
      anon_sym_LT_DASH,
    ACTIONS(504), 1,
      anon_sym_BQUOTE,
    ACTIONS(506), 1,
      anon_sym_DQUOTE,
    STATE(424), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(920), 1,
      sym_expression_list,
    STATE(1055), 1,
      sym_qualified_type,
    STATE(1277), 1,
      sym_implicit_length_array_type,
    ACTIONS(290), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(508), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1319), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(502), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1082), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(298), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(509), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [10378] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(530), 1,
      anon_sym_RBRACK,
    STATE(640), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [10492] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    ACTIONS(532), 1,
      anon_sym_RPAREN,
    STATE(606), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [10606] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(534), 1,
      anon_sym_RBRACK,
    STATE(637), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [10720] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    ACTIONS(536), 1,
      anon_sym_RPAREN,
    STATE(606), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [10834] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(538), 1,
      anon_sym_RBRACK,
    STATE(641), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [10948] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    ACTIONS(540), 1,
      anon_sym_RPAREN,
    STATE(606), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [11062] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(542), 1,
      anon_sym_SEMI,
    STATE(656), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [11176] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    STATE(529), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    STATE(1275), 1,
      sym_expression_list,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [11290] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    STATE(529), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    STATE(1282), 1,
      sym_expression_list,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [11404] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    STATE(580), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    STATE(1282), 1,
      sym_expression_list,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [11518] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(426), 1,
      anon_sym_RBRACK,
    STATE(669), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [11632] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    ACTIONS(544), 1,
      anon_sym_RPAREN,
    STATE(606), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [11746] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(546), 1,
      anon_sym_SEMI,
    STATE(658), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [11860] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(548), 1,
      anon_sym_RBRACK,
    STATE(632), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [11974] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(272), 1,
      sym_identifier,
    ACTIONS(280), 1,
      anon_sym_func,
    ACTIONS(496), 1,
      anon_sym_LPAREN,
    ACTIONS(498), 1,
      anon_sym_STAR,
    ACTIONS(500), 1,
      anon_sym_LT_DASH,
    ACTIONS(504), 1,
      anon_sym_BQUOTE,
    ACTIONS(506), 1,
      anon_sym_DQUOTE,
    STATE(440), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1055), 1,
      sym_qualified_type,
    STATE(1277), 1,
      sym_implicit_length_array_type,
    ACTIONS(290), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(508), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1319), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(502), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1082), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(298), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(509), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [12085] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    STATE(696), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [12196] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    STATE(645), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [12307] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    STATE(390), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [12418] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(396), 1,
      sym_identifier,
    ACTIONS(398), 1,
      anon_sym_STAR,
    ACTIONS(402), 1,
      anon_sym_LT_DASH,
    STATE(600), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(404), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [12529] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    STATE(514), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [12640] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    STATE(391), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [12751] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    STATE(586), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [12862] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    STATE(611), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [12973] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    STATE(566), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [13084] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    STATE(533), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [13195] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    STATE(598), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [13306] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    STATE(390), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [13417] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    STATE(391), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [13528] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    STATE(437), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [13639] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    STATE(443), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [13750] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    STATE(444), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [13861] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    STATE(439), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [13972] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(272), 1,
      sym_identifier,
    ACTIONS(280), 1,
      anon_sym_func,
    ACTIONS(496), 1,
      anon_sym_LPAREN,
    ACTIONS(498), 1,
      anon_sym_STAR,
    ACTIONS(500), 1,
      anon_sym_LT_DASH,
    ACTIONS(504), 1,
      anon_sym_BQUOTE,
    ACTIONS(506), 1,
      anon_sym_DQUOTE,
    STATE(447), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1055), 1,
      sym_qualified_type,
    STATE(1277), 1,
      sym_implicit_length_array_type,
    ACTIONS(290), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(508), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1319), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(502), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1082), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(298), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(509), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [14083] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(272), 1,
      sym_identifier,
    ACTIONS(280), 1,
      anon_sym_func,
    ACTIONS(496), 1,
      anon_sym_LPAREN,
    ACTIONS(498), 1,
      anon_sym_STAR,
    ACTIONS(500), 1,
      anon_sym_LT_DASH,
    ACTIONS(504), 1,
      anon_sym_BQUOTE,
    ACTIONS(506), 1,
      anon_sym_DQUOTE,
    STATE(441), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1055), 1,
      sym_qualified_type,
    STATE(1277), 1,
      sym_implicit_length_array_type,
    ACTIONS(290), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(508), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1319), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(502), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1082), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(298), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(509), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [14194] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(272), 1,
      sym_identifier,
    ACTIONS(280), 1,
      anon_sym_func,
    ACTIONS(496), 1,
      anon_sym_LPAREN,
    ACTIONS(498), 1,
      anon_sym_STAR,
    ACTIONS(500), 1,
      anon_sym_LT_DASH,
    ACTIONS(504), 1,
      anon_sym_BQUOTE,
    ACTIONS(506), 1,
      anon_sym_DQUOTE,
    STATE(448), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1055), 1,
      sym_qualified_type,
    STATE(1277), 1,
      sym_implicit_length_array_type,
    ACTIONS(290), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(508), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1319), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(502), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1082), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(298), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(509), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [14305] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(272), 1,
      sym_identifier,
    ACTIONS(280), 1,
      anon_sym_func,
    ACTIONS(496), 1,
      anon_sym_LPAREN,
    ACTIONS(498), 1,
      anon_sym_STAR,
    ACTIONS(500), 1,
      anon_sym_LT_DASH,
    ACTIONS(504), 1,
      anon_sym_BQUOTE,
    ACTIONS(506), 1,
      anon_sym_DQUOTE,
    STATE(449), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1055), 1,
      sym_qualified_type,
    STATE(1277), 1,
      sym_implicit_length_array_type,
    ACTIONS(290), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(508), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1319), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(502), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1082), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(298), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(509), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [14416] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(272), 1,
      sym_identifier,
    ACTIONS(280), 1,
      anon_sym_func,
    ACTIONS(496), 1,
      anon_sym_LPAREN,
    ACTIONS(498), 1,
      anon_sym_STAR,
    ACTIONS(500), 1,
      anon_sym_LT_DASH,
    ACTIONS(504), 1,
      anon_sym_BQUOTE,
    ACTIONS(506), 1,
      anon_sym_DQUOTE,
    STATE(438), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1055), 1,
      sym_qualified_type,
    STATE(1277), 1,
      sym_implicit_length_array_type,
    ACTIONS(290), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(508), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1319), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(502), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1082), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(298), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(509), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [14527] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(236), 1,
      sym_identifier,
    ACTIONS(240), 1,
      anon_sym_LPAREN,
    ACTIONS(242), 1,
      anon_sym_func,
    ACTIONS(244), 1,
      anon_sym_STAR,
    ACTIONS(246), 1,
      anon_sym_LT_DASH,
    ACTIONS(254), 1,
      anon_sym_BQUOTE,
    ACTIONS(256), 1,
      anon_sym_DQUOTE,
    STATE(293), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1029), 1,
      sym_qualified_type,
    STATE(1270), 1,
      sym_implicit_length_array_type,
    ACTIONS(250), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(260), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1320), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(252), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1144), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(258), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(319), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [14638] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(236), 1,
      sym_identifier,
    ACTIONS(240), 1,
      anon_sym_LPAREN,
    ACTIONS(242), 1,
      anon_sym_func,
    ACTIONS(244), 1,
      anon_sym_STAR,
    ACTIONS(246), 1,
      anon_sym_LT_DASH,
    ACTIONS(254), 1,
      anon_sym_BQUOTE,
    ACTIONS(256), 1,
      anon_sym_DQUOTE,
    STATE(298), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1029), 1,
      sym_qualified_type,
    STATE(1270), 1,
      sym_implicit_length_array_type,
    ACTIONS(250), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(260), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1320), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(252), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1144), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(258), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(319), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [14749] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(236), 1,
      sym_identifier,
    ACTIONS(240), 1,
      anon_sym_LPAREN,
    ACTIONS(242), 1,
      anon_sym_func,
    ACTIONS(244), 1,
      anon_sym_STAR,
    ACTIONS(246), 1,
      anon_sym_LT_DASH,
    ACTIONS(254), 1,
      anon_sym_BQUOTE,
    ACTIONS(256), 1,
      anon_sym_DQUOTE,
    STATE(295), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1029), 1,
      sym_qualified_type,
    STATE(1270), 1,
      sym_implicit_length_array_type,
    ACTIONS(250), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(260), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1320), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(252), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1144), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(258), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(319), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [14860] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(236), 1,
      sym_identifier,
    ACTIONS(240), 1,
      anon_sym_LPAREN,
    ACTIONS(242), 1,
      anon_sym_func,
    ACTIONS(244), 1,
      anon_sym_STAR,
    ACTIONS(246), 1,
      anon_sym_LT_DASH,
    ACTIONS(254), 1,
      anon_sym_BQUOTE,
    ACTIONS(256), 1,
      anon_sym_DQUOTE,
    STATE(296), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1029), 1,
      sym_qualified_type,
    STATE(1270), 1,
      sym_implicit_length_array_type,
    ACTIONS(250), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(260), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1320), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(252), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1144), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(258), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(319), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [14971] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(236), 1,
      sym_identifier,
    ACTIONS(240), 1,
      anon_sym_LPAREN,
    ACTIONS(242), 1,
      anon_sym_func,
    ACTIONS(244), 1,
      anon_sym_STAR,
    ACTIONS(246), 1,
      anon_sym_LT_DASH,
    ACTIONS(254), 1,
      anon_sym_BQUOTE,
    ACTIONS(256), 1,
      anon_sym_DQUOTE,
    STATE(291), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1029), 1,
      sym_qualified_type,
    STATE(1270), 1,
      sym_implicit_length_array_type,
    ACTIONS(250), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(260), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1320), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(252), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1144), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(258), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(319), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [15082] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(236), 1,
      sym_identifier,
    ACTIONS(240), 1,
      anon_sym_LPAREN,
    ACTIONS(242), 1,
      anon_sym_func,
    ACTIONS(244), 1,
      anon_sym_STAR,
    ACTIONS(246), 1,
      anon_sym_LT_DASH,
    ACTIONS(254), 1,
      anon_sym_BQUOTE,
    ACTIONS(256), 1,
      anon_sym_DQUOTE,
    STATE(297), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1029), 1,
      sym_qualified_type,
    STATE(1270), 1,
      sym_implicit_length_array_type,
    ACTIONS(250), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(260), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1320), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(252), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1144), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(258), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(319), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [15193] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(550), 1,
      sym_identifier,
    ACTIONS(552), 1,
      anon_sym_LPAREN,
    ACTIONS(554), 1,
      anon_sym_func,
    ACTIONS(556), 1,
      anon_sym_STAR,
    ACTIONS(558), 1,
      anon_sym_LT_DASH,
    ACTIONS(564), 1,
      anon_sym_BQUOTE,
    ACTIONS(566), 1,
      anon_sym_DQUOTE,
    STATE(341), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1042), 1,
      sym_qualified_type,
    STATE(1314), 1,
      sym_implicit_length_array_type,
    ACTIONS(560), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(570), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1321), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(562), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1062), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(568), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(352), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [15304] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(550), 1,
      sym_identifier,
    ACTIONS(552), 1,
      anon_sym_LPAREN,
    ACTIONS(554), 1,
      anon_sym_func,
    ACTIONS(556), 1,
      anon_sym_STAR,
    ACTIONS(558), 1,
      anon_sym_LT_DASH,
    ACTIONS(564), 1,
      anon_sym_BQUOTE,
    ACTIONS(566), 1,
      anon_sym_DQUOTE,
    STATE(342), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1042), 1,
      sym_qualified_type,
    STATE(1314), 1,
      sym_implicit_length_array_type,
    ACTIONS(560), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(570), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1321), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(562), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1062), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(568), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(352), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [15415] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(550), 1,
      sym_identifier,
    ACTIONS(552), 1,
      anon_sym_LPAREN,
    ACTIONS(554), 1,
      anon_sym_func,
    ACTIONS(556), 1,
      anon_sym_STAR,
    ACTIONS(558), 1,
      anon_sym_LT_DASH,
    ACTIONS(564), 1,
      anon_sym_BQUOTE,
    ACTIONS(566), 1,
      anon_sym_DQUOTE,
    STATE(343), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1042), 1,
      sym_qualified_type,
    STATE(1314), 1,
      sym_implicit_length_array_type,
    ACTIONS(560), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(570), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1321), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(562), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1062), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(568), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(352), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [15526] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(550), 1,
      sym_identifier,
    ACTIONS(552), 1,
      anon_sym_LPAREN,
    ACTIONS(554), 1,
      anon_sym_func,
    ACTIONS(556), 1,
      anon_sym_STAR,
    ACTIONS(558), 1,
      anon_sym_LT_DASH,
    ACTIONS(564), 1,
      anon_sym_BQUOTE,
    ACTIONS(566), 1,
      anon_sym_DQUOTE,
    STATE(344), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1042), 1,
      sym_qualified_type,
    STATE(1314), 1,
      sym_implicit_length_array_type,
    ACTIONS(560), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(570), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1321), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(562), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1062), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(568), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(352), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [15637] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(550), 1,
      sym_identifier,
    ACTIONS(552), 1,
      anon_sym_LPAREN,
    ACTIONS(554), 1,
      anon_sym_func,
    ACTIONS(556), 1,
      anon_sym_STAR,
    ACTIONS(558), 1,
      anon_sym_LT_DASH,
    ACTIONS(564), 1,
      anon_sym_BQUOTE,
    ACTIONS(566), 1,
      anon_sym_DQUOTE,
    STATE(345), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1042), 1,
      sym_qualified_type,
    STATE(1314), 1,
      sym_implicit_length_array_type,
    ACTIONS(560), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(570), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1321), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(562), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1062), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(568), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(352), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [15748] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(550), 1,
      sym_identifier,
    ACTIONS(552), 1,
      anon_sym_LPAREN,
    ACTIONS(554), 1,
      anon_sym_func,
    ACTIONS(556), 1,
      anon_sym_STAR,
    ACTIONS(558), 1,
      anon_sym_LT_DASH,
    ACTIONS(564), 1,
      anon_sym_BQUOTE,
    ACTIONS(566), 1,
      anon_sym_DQUOTE,
    STATE(346), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1042), 1,
      sym_qualified_type,
    STATE(1314), 1,
      sym_implicit_length_array_type,
    ACTIONS(560), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(570), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1321), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(562), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1062), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(568), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(352), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [15859] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(374), 1,
      sym_identifier,
    ACTIONS(376), 1,
      anon_sym_LPAREN,
    ACTIONS(378), 1,
      anon_sym_func,
    ACTIONS(380), 1,
      anon_sym_STAR,
    ACTIONS(382), 1,
      anon_sym_LT_DASH,
    ACTIONS(388), 1,
      anon_sym_BQUOTE,
    ACTIONS(390), 1,
      anon_sym_DQUOTE,
    STATE(500), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1031), 1,
      sym_qualified_type,
    STATE(1307), 1,
      sym_implicit_length_array_type,
    ACTIONS(384), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(394), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1322), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(386), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1089), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(392), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(590), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [15970] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(374), 1,
      sym_identifier,
    ACTIONS(376), 1,
      anon_sym_LPAREN,
    ACTIONS(378), 1,
      anon_sym_func,
    ACTIONS(380), 1,
      anon_sym_STAR,
    ACTIONS(382), 1,
      anon_sym_LT_DASH,
    ACTIONS(388), 1,
      anon_sym_BQUOTE,
    ACTIONS(390), 1,
      anon_sym_DQUOTE,
    STATE(502), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1031), 1,
      sym_qualified_type,
    STATE(1307), 1,
      sym_implicit_length_array_type,
    ACTIONS(384), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(394), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1322), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(386), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1089), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(392), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(590), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [16081] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(374), 1,
      sym_identifier,
    ACTIONS(376), 1,
      anon_sym_LPAREN,
    ACTIONS(378), 1,
      anon_sym_func,
    ACTIONS(380), 1,
      anon_sym_STAR,
    ACTIONS(382), 1,
      anon_sym_LT_DASH,
    ACTIONS(388), 1,
      anon_sym_BQUOTE,
    ACTIONS(390), 1,
      anon_sym_DQUOTE,
    STATE(505), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1031), 1,
      sym_qualified_type,
    STATE(1307), 1,
      sym_implicit_length_array_type,
    ACTIONS(384), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(394), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1322), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(386), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1089), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(392), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(590), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [16192] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(374), 1,
      sym_identifier,
    ACTIONS(376), 1,
      anon_sym_LPAREN,
    ACTIONS(378), 1,
      anon_sym_func,
    ACTIONS(380), 1,
      anon_sym_STAR,
    ACTIONS(382), 1,
      anon_sym_LT_DASH,
    ACTIONS(388), 1,
      anon_sym_BQUOTE,
    ACTIONS(390), 1,
      anon_sym_DQUOTE,
    STATE(506), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1031), 1,
      sym_qualified_type,
    STATE(1307), 1,
      sym_implicit_length_array_type,
    ACTIONS(384), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(394), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1322), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(386), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1089), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(392), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(590), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [16303] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(374), 1,
      sym_identifier,
    ACTIONS(376), 1,
      anon_sym_LPAREN,
    ACTIONS(378), 1,
      anon_sym_func,
    ACTIONS(380), 1,
      anon_sym_STAR,
    ACTIONS(382), 1,
      anon_sym_LT_DASH,
    ACTIONS(388), 1,
      anon_sym_BQUOTE,
    ACTIONS(390), 1,
      anon_sym_DQUOTE,
    STATE(507), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1031), 1,
      sym_qualified_type,
    STATE(1307), 1,
      sym_implicit_length_array_type,
    ACTIONS(384), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(394), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1322), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(386), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1089), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(392), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(590), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [16414] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(374), 1,
      sym_identifier,
    ACTIONS(376), 1,
      anon_sym_LPAREN,
    ACTIONS(378), 1,
      anon_sym_func,
    ACTIONS(380), 1,
      anon_sym_STAR,
    ACTIONS(382), 1,
      anon_sym_LT_DASH,
    ACTIONS(388), 1,
      anon_sym_BQUOTE,
    ACTIONS(390), 1,
      anon_sym_DQUOTE,
    STATE(508), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1031), 1,
      sym_qualified_type,
    STATE(1307), 1,
      sym_implicit_length_array_type,
    ACTIONS(384), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(394), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1322), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(386), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1089), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(392), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(590), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [16525] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    ACTIONS(572), 1,
      anon_sym_STAR,
    STATE(390), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1016), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [16636] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(15), 1,
      anon_sym_LPAREN,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(27), 1,
      anon_sym_STAR,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(41), 1,
      anon_sym_LT_DASH,
    ACTIONS(69), 1,
      anon_sym_BQUOTE,
    ACTIONS(71), 1,
      anon_sym_DQUOTE,
    ACTIONS(188), 1,
      anon_sym_func,
    ACTIONS(574), 1,
      sym_identifier,
    STATE(245), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1027), 1,
      sym_qualified_type,
    STATE(1293), 1,
      sym_implicit_length_array_type,
    ACTIONS(65), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(75), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1016), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(67), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1131), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(73), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(263), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [16747] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    ACTIONS(576), 1,
      anon_sym_chan,
    STATE(390), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [16858] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(272), 1,
      sym_identifier,
    ACTIONS(280), 1,
      anon_sym_func,
    ACTIONS(496), 1,
      anon_sym_LPAREN,
    ACTIONS(498), 1,
      anon_sym_STAR,
    ACTIONS(500), 1,
      anon_sym_LT_DASH,
    ACTIONS(504), 1,
      anon_sym_BQUOTE,
    ACTIONS(506), 1,
      anon_sym_DQUOTE,
    STATE(451), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1055), 1,
      sym_qualified_type,
    STATE(1277), 1,
      sym_implicit_length_array_type,
    ACTIONS(290), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(508), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1319), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(502), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1082), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(298), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(509), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [16969] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(396), 1,
      sym_identifier,
    ACTIONS(398), 1,
      anon_sym_STAR,
    ACTIONS(402), 1,
      anon_sym_LT_DASH,
    STATE(659), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(404), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [17080] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(272), 1,
      sym_identifier,
    ACTIONS(280), 1,
      anon_sym_func,
    ACTIONS(496), 1,
      anon_sym_LPAREN,
    ACTIONS(498), 1,
      anon_sym_STAR,
    ACTIONS(500), 1,
      anon_sym_LT_DASH,
    ACTIONS(504), 1,
      anon_sym_BQUOTE,
    ACTIONS(506), 1,
      anon_sym_DQUOTE,
    STATE(445), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1055), 1,
      sym_qualified_type,
    STATE(1277), 1,
      sym_implicit_length_array_type,
    ACTIONS(290), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(508), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1319), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(502), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1082), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(298), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(509), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [17191] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(396), 1,
      sym_identifier,
    ACTIONS(398), 1,
      anon_sym_STAR,
    ACTIONS(402), 1,
      anon_sym_LT_DASH,
    STATE(648), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(404), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [17302] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(396), 1,
      sym_identifier,
    ACTIONS(398), 1,
      anon_sym_STAR,
    ACTIONS(402), 1,
      anon_sym_LT_DASH,
    STATE(390), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(404), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [17413] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(396), 1,
      sym_identifier,
    ACTIONS(398), 1,
      anon_sym_STAR,
    ACTIONS(402), 1,
      anon_sym_LT_DASH,
    STATE(391), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(404), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [17524] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(396), 1,
      sym_identifier,
    ACTIONS(398), 1,
      anon_sym_STAR,
    ACTIONS(402), 1,
      anon_sym_LT_DASH,
    STATE(582), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(404), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [17635] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(396), 1,
      sym_identifier,
    ACTIONS(398), 1,
      anon_sym_STAR,
    ACTIONS(402), 1,
      anon_sym_LT_DASH,
    STATE(583), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(404), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [17746] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(396), 1,
      sym_identifier,
    ACTIONS(398), 1,
      anon_sym_STAR,
    ACTIONS(402), 1,
      anon_sym_LT_DASH,
    STATE(584), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(404), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [17857] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(396), 1,
      sym_identifier,
    ACTIONS(398), 1,
      anon_sym_STAR,
    ACTIONS(402), 1,
      anon_sym_LT_DASH,
    STATE(585), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(404), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [17968] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    STATE(390), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1016), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [18079] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(576), 1,
      anon_sym_chan,
    STATE(390), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [18190] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    STATE(606), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [18301] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(15), 1,
      anon_sym_LPAREN,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(27), 1,
      anon_sym_STAR,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(41), 1,
      anon_sym_LT_DASH,
    ACTIONS(69), 1,
      anon_sym_BQUOTE,
    ACTIONS(71), 1,
      anon_sym_DQUOTE,
    ACTIONS(188), 1,
      anon_sym_func,
    ACTIONS(574), 1,
      sym_identifier,
    STATE(245), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1027), 1,
      sym_qualified_type,
    STATE(1293), 1,
      sym_implicit_length_array_type,
    ACTIONS(65), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(75), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1303), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(67), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1131), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(73), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(263), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [18412] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(272), 1,
      sym_identifier,
    ACTIONS(280), 1,
      anon_sym_func,
    ACTIONS(496), 1,
      anon_sym_LPAREN,
    ACTIONS(498), 1,
      anon_sym_STAR,
    ACTIONS(500), 1,
      anon_sym_LT_DASH,
    ACTIONS(504), 1,
      anon_sym_BQUOTE,
    ACTIONS(506), 1,
      anon_sym_DQUOTE,
    STATE(447), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1055), 1,
      sym_qualified_type,
    STATE(1277), 1,
      sym_implicit_length_array_type,
    ACTIONS(290), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(508), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1016), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(502), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1082), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(298), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(509), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [18523] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(272), 1,
      sym_identifier,
    ACTIONS(280), 1,
      anon_sym_func,
    ACTIONS(496), 1,
      anon_sym_LPAREN,
    ACTIONS(498), 1,
      anon_sym_STAR,
    ACTIONS(500), 1,
      anon_sym_LT_DASH,
    ACTIONS(504), 1,
      anon_sym_BQUOTE,
    ACTIONS(506), 1,
      anon_sym_DQUOTE,
    ACTIONS(576), 1,
      anon_sym_chan,
    STATE(447), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1055), 1,
      sym_qualified_type,
    STATE(1277), 1,
      sym_implicit_length_array_type,
    ACTIONS(290), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(508), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1319), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(502), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1082), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(298), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(509), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [18634] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    STATE(619), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [18745] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(396), 1,
      sym_identifier,
    ACTIONS(398), 1,
      anon_sym_STAR,
    ACTIONS(402), 1,
      anon_sym_LT_DASH,
    STATE(668), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(404), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [18856] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(396), 1,
      sym_identifier,
    ACTIONS(398), 1,
      anon_sym_STAR,
    ACTIONS(402), 1,
      anon_sym_LT_DASH,
    STATE(603), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(404), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [18967] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(550), 1,
      sym_identifier,
    ACTIONS(552), 1,
      anon_sym_LPAREN,
    ACTIONS(554), 1,
      anon_sym_func,
    ACTIONS(556), 1,
      anon_sym_STAR,
    ACTIONS(558), 1,
      anon_sym_LT_DASH,
    ACTIONS(564), 1,
      anon_sym_BQUOTE,
    ACTIONS(566), 1,
      anon_sym_DQUOTE,
    STATE(340), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1042), 1,
      sym_qualified_type,
    STATE(1314), 1,
      sym_implicit_length_array_type,
    ACTIONS(560), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(570), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1321), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(562), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1062), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(568), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(352), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [19078] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(272), 1,
      sym_identifier,
    ACTIONS(280), 1,
      anon_sym_func,
    ACTIONS(496), 1,
      anon_sym_LPAREN,
    ACTIONS(498), 1,
      anon_sym_STAR,
    ACTIONS(500), 1,
      anon_sym_LT_DASH,
    ACTIONS(504), 1,
      anon_sym_BQUOTE,
    ACTIONS(506), 1,
      anon_sym_DQUOTE,
    STATE(450), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1055), 1,
      sym_qualified_type,
    STATE(1277), 1,
      sym_implicit_length_array_type,
    ACTIONS(290), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(508), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1319), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(502), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1082), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(298), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(509), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [19189] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(15), 1,
      anon_sym_LPAREN,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(27), 1,
      anon_sym_STAR,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(41), 1,
      anon_sym_LT_DASH,
    ACTIONS(69), 1,
      anon_sym_BQUOTE,
    ACTIONS(71), 1,
      anon_sym_DQUOTE,
    ACTIONS(188), 1,
      anon_sym_func,
    ACTIONS(574), 1,
      sym_identifier,
    STATE(241), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1027), 1,
      sym_qualified_type,
    STATE(1293), 1,
      sym_implicit_length_array_type,
    ACTIONS(65), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(75), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1303), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(67), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1131), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(73), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(263), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [19300] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(236), 1,
      sym_identifier,
    ACTIONS(240), 1,
      anon_sym_LPAREN,
    ACTIONS(242), 1,
      anon_sym_func,
    ACTIONS(244), 1,
      anon_sym_STAR,
    ACTIONS(246), 1,
      anon_sym_LT_DASH,
    ACTIONS(254), 1,
      anon_sym_BQUOTE,
    ACTIONS(256), 1,
      anon_sym_DQUOTE,
    STATE(293), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1029), 1,
      sym_qualified_type,
    STATE(1270), 1,
      sym_implicit_length_array_type,
    ACTIONS(250), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(260), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1016), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(252), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1144), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(258), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(319), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [19411] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(236), 1,
      sym_identifier,
    ACTIONS(240), 1,
      anon_sym_LPAREN,
    ACTIONS(242), 1,
      anon_sym_func,
    ACTIONS(244), 1,
      anon_sym_STAR,
    ACTIONS(246), 1,
      anon_sym_LT_DASH,
    ACTIONS(254), 1,
      anon_sym_BQUOTE,
    ACTIONS(256), 1,
      anon_sym_DQUOTE,
    ACTIONS(576), 1,
      anon_sym_chan,
    STATE(293), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1029), 1,
      sym_qualified_type,
    STATE(1270), 1,
      sym_implicit_length_array_type,
    ACTIONS(250), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(260), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1320), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(252), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1144), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(258), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(319), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [19522] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(15), 1,
      anon_sym_LPAREN,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(27), 1,
      anon_sym_STAR,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(41), 1,
      anon_sym_LT_DASH,
    ACTIONS(69), 1,
      anon_sym_BQUOTE,
    ACTIONS(71), 1,
      anon_sym_DQUOTE,
    ACTIONS(188), 1,
      anon_sym_func,
    ACTIONS(574), 1,
      sym_identifier,
    STATE(244), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1027), 1,
      sym_qualified_type,
    STATE(1293), 1,
      sym_implicit_length_array_type,
    ACTIONS(65), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(75), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1303), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(67), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1131), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(73), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(263), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [19633] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(272), 1,
      sym_identifier,
    ACTIONS(280), 1,
      anon_sym_func,
    ACTIONS(496), 1,
      anon_sym_LPAREN,
    ACTIONS(498), 1,
      anon_sym_STAR,
    ACTIONS(500), 1,
      anon_sym_LT_DASH,
    ACTIONS(504), 1,
      anon_sym_BQUOTE,
    ACTIONS(506), 1,
      anon_sym_DQUOTE,
    STATE(456), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1055), 1,
      sym_qualified_type,
    STATE(1277), 1,
      sym_implicit_length_array_type,
    ACTIONS(290), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(508), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1319), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(502), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1082), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(298), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(509), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [19744] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(15), 1,
      anon_sym_LPAREN,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(27), 1,
      anon_sym_STAR,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(41), 1,
      anon_sym_LT_DASH,
    ACTIONS(69), 1,
      anon_sym_BQUOTE,
    ACTIONS(71), 1,
      anon_sym_DQUOTE,
    ACTIONS(188), 1,
      anon_sym_func,
    ACTIONS(574), 1,
      sym_identifier,
    STATE(242), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1027), 1,
      sym_qualified_type,
    STATE(1293), 1,
      sym_implicit_length_array_type,
    ACTIONS(65), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(75), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1303), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(67), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1131), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(73), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(263), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [19855] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(15), 1,
      anon_sym_LPAREN,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(27), 1,
      anon_sym_STAR,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(41), 1,
      anon_sym_LT_DASH,
    ACTIONS(69), 1,
      anon_sym_BQUOTE,
    ACTIONS(71), 1,
      anon_sym_DQUOTE,
    ACTIONS(188), 1,
      anon_sym_func,
    ACTIONS(574), 1,
      sym_identifier,
    STATE(243), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1027), 1,
      sym_qualified_type,
    STATE(1293), 1,
      sym_implicit_length_array_type,
    ACTIONS(65), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(75), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1303), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(67), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1131), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(73), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(263), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [19966] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(15), 1,
      anon_sym_LPAREN,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(27), 1,
      anon_sym_STAR,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(41), 1,
      anon_sym_LT_DASH,
    ACTIONS(69), 1,
      anon_sym_BQUOTE,
    ACTIONS(71), 1,
      anon_sym_DQUOTE,
    ACTIONS(188), 1,
      anon_sym_func,
    ACTIONS(574), 1,
      sym_identifier,
    STATE(247), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1027), 1,
      sym_qualified_type,
    STATE(1293), 1,
      sym_implicit_length_array_type,
    ACTIONS(65), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(75), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1303), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(67), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1131), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(73), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(263), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [20077] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    STATE(390), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1016), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [20188] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    STATE(764), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [20299] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(396), 1,
      sym_identifier,
    ACTIONS(398), 1,
      anon_sym_STAR,
    ACTIONS(402), 1,
      anon_sym_LT_DASH,
    ACTIONS(576), 1,
      anon_sym_chan,
    STATE(390), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(404), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [20410] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    STATE(657), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [20521] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(15), 1,
      anon_sym_LPAREN,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(27), 1,
      anon_sym_STAR,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(41), 1,
      anon_sym_LT_DASH,
    ACTIONS(69), 1,
      anon_sym_BQUOTE,
    ACTIONS(71), 1,
      anon_sym_DQUOTE,
    ACTIONS(188), 1,
      anon_sym_func,
    ACTIONS(574), 1,
      sym_identifier,
    ACTIONS(576), 1,
      anon_sym_chan,
    STATE(245), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1027), 1,
      sym_qualified_type,
    STATE(1293), 1,
      sym_implicit_length_array_type,
    ACTIONS(65), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(75), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1303), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(67), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1131), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(73), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(263), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [20632] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(396), 1,
      sym_identifier,
    ACTIONS(398), 1,
      anon_sym_STAR,
    ACTIONS(402), 1,
      anon_sym_LT_DASH,
    STATE(390), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1016), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(404), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [20743] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(550), 1,
      sym_identifier,
    ACTIONS(552), 1,
      anon_sym_LPAREN,
    ACTIONS(554), 1,
      anon_sym_func,
    ACTIONS(556), 1,
      anon_sym_STAR,
    ACTIONS(558), 1,
      anon_sym_LT_DASH,
    ACTIONS(564), 1,
      anon_sym_BQUOTE,
    ACTIONS(566), 1,
      anon_sym_DQUOTE,
    ACTIONS(576), 1,
      anon_sym_chan,
    STATE(341), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1042), 1,
      sym_qualified_type,
    STATE(1314), 1,
      sym_implicit_length_array_type,
    ACTIONS(560), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(570), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1321), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(562), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1062), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(568), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(352), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [20854] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(550), 1,
      sym_identifier,
    ACTIONS(552), 1,
      anon_sym_LPAREN,
    ACTIONS(554), 1,
      anon_sym_func,
    ACTIONS(556), 1,
      anon_sym_STAR,
    ACTIONS(558), 1,
      anon_sym_LT_DASH,
    ACTIONS(564), 1,
      anon_sym_BQUOTE,
    ACTIONS(566), 1,
      anon_sym_DQUOTE,
    STATE(341), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1042), 1,
      sym_qualified_type,
    STATE(1314), 1,
      sym_implicit_length_array_type,
    ACTIONS(560), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(570), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1016), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(562), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1062), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(568), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(352), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [20965] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(374), 1,
      sym_identifier,
    ACTIONS(376), 1,
      anon_sym_LPAREN,
    ACTIONS(378), 1,
      anon_sym_func,
    ACTIONS(380), 1,
      anon_sym_STAR,
    ACTIONS(382), 1,
      anon_sym_LT_DASH,
    ACTIONS(388), 1,
      anon_sym_BQUOTE,
    ACTIONS(390), 1,
      anon_sym_DQUOTE,
    ACTIONS(576), 1,
      anon_sym_chan,
    STATE(500), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1031), 1,
      sym_qualified_type,
    STATE(1307), 1,
      sym_implicit_length_array_type,
    ACTIONS(384), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(394), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1322), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(386), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1089), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(392), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(590), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [21076] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(406), 1,
      sym_identifier,
    ACTIONS(408), 1,
      anon_sym_STAR,
    STATE(390), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1016), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [21187] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(374), 1,
      sym_identifier,
    ACTIONS(376), 1,
      anon_sym_LPAREN,
    ACTIONS(378), 1,
      anon_sym_func,
    ACTIONS(380), 1,
      anon_sym_STAR,
    ACTIONS(382), 1,
      anon_sym_LT_DASH,
    ACTIONS(388), 1,
      anon_sym_BQUOTE,
    ACTIONS(390), 1,
      anon_sym_DQUOTE,
    STATE(500), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(1031), 1,
      sym_qualified_type,
    STATE(1307), 1,
      sym_implicit_length_array_type,
    ACTIONS(384), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(394), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1016), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(386), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(1089), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(392), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(590), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [21298] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    ACTIONS(572), 1,
      anon_sym_STAR,
    STATE(653), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1238), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [21409] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    STATE(602), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [21520] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    STATE(685), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [21631] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    STATE(663), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [21742] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    STATE(664), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [21853] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    ACTIONS(572), 1,
      anon_sym_STAR,
    STATE(667), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1238), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [21964] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    STATE(608), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [22075] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    STATE(673), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [22186] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    STATE(674), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [22297] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    ACTIONS(572), 1,
      anon_sym_STAR,
    STATE(675), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1238), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [22408] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    STATE(622), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [22519] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    STATE(681), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [22630] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    STATE(682), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [22741] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    ACTIONS(572), 1,
      anon_sym_STAR,
    STATE(683), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1238), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [22852] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    STATE(629), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [22963] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    STATE(688), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [23074] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    STATE(689), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [23185] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    ACTIONS(572), 1,
      anon_sym_STAR,
    STATE(690), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1238), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [23296] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(412), 1,
      sym_identifier,
    ACTIONS(416), 1,
      anon_sym_STAR,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    STATE(636), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [23407] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(310), 1,
      anon_sym_STAR,
    ACTIONS(316), 1,
      anon_sym_LT_DASH,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    STATE(695), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1318), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(320), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [23518] = 26,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(23), 1,
      anon_sym_LBRACK,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(302), 1,
      sym_identifier,
    ACTIONS(304), 1,
      anon_sym_LPAREN,
    ACTIONS(308), 1,
      anon_sym_func,
    ACTIONS(322), 1,
      anon_sym_BQUOTE,
    ACTIONS(324), 1,
      anon_sym_DQUOTE,
    ACTIONS(418), 1,
      anon_sym_LT_DASH,
    ACTIONS(572), 1,
      anon_sym_STAR,
    STATE(647), 1,
      sym__expression,
    STATE(850), 1,
      sym_negated_type,
    STATE(871), 1,
      sym_qualified_type,
    STATE(1274), 1,
      sym_implicit_length_array_type,
    ACTIONS(318), 2,
      anon_sym_new,
      anon_sym_make,
    ACTIONS(328), 2,
      sym_imaginary_literal,
      sym_rune_literal,
    STATE(1238), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 4,
      sym_pointer_type,
      sym_interface_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(420), 5,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
    STATE(900), 5,
      sym_generic_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_map_type,
    ACTIONS(326), 6,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
    STATE(419), 14,
      sym_parenthesized_expression,
      sym_call_expression,
      sym_selector_expression,
      sym_index_expression,
      sym_slice_expression,
      sym_type_assertion_expression,
      sym_type_conversion_expression,
      sym_type_instantiation_expression,
      sym_composite_literal,
      sym_func_literal,
      sym_unary_expression,
      sym_binary_expression,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [23629] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(312), 1,
      anon_sym_LBRACE,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(580), 1,
      anon_sym_DOT,
    ACTIONS(583), 1,
      anon_sym_LPAREN,
    ACTIONS(587), 1,
      anon_sym_COMMA,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(591), 1,
      anon_sym_LBRACK,
    ACTIONS(597), 1,
      anon_sym_STAR,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    STATE(414), 1,
      sym_literal_value,
    STATE(597), 1,
      aux_sym_var_spec_repeat1,
    STATE(875), 1,
      sym_type_arguments,
    STATE(1251), 1,
      sym_type_elem,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1125), 2,
      sym_parenthesized_type,
      sym__simple_type,
    ACTIONS(600), 5,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(595), 14,
      anon_sym_RBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [23729] = 10,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(595), 1,
      aux_sym_source_file_token1,
    ACTIONS(608), 1,
      anon_sym_DOT,
    ACTIONS(611), 1,
      anon_sym_LPAREN,
    ACTIONS(614), 1,
      anon_sym_LBRACK,
    ACTIONS(618), 1,
      anon_sym_LBRACE,
    ACTIONS(620), 1,
      anon_sym_COLON,
    STATE(258), 1,
      sym_literal_value,
    STATE(875), 1,
      sym_type_arguments,
    ACTIONS(600), 41,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [23800] = 10,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(595), 1,
      aux_sym_source_file_token1,
    ACTIONS(608), 1,
      anon_sym_DOT,
    ACTIONS(611), 1,
      anon_sym_LPAREN,
    ACTIONS(614), 1,
      anon_sym_LBRACK,
    ACTIONS(618), 1,
      anon_sym_LBRACE,
    ACTIONS(622), 1,
      anon_sym_COLON,
    STATE(258), 1,
      sym_literal_value,
    STATE(875), 1,
      sym_type_arguments,
    ACTIONS(600), 41,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [23871] = 9,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(595), 1,
      aux_sym_source_file_token1,
    ACTIONS(608), 1,
      anon_sym_DOT,
    ACTIONS(611), 1,
      anon_sym_LPAREN,
    ACTIONS(614), 1,
      anon_sym_LBRACK,
    ACTIONS(618), 1,
      anon_sym_LBRACE,
    STATE(258), 1,
      sym_literal_value,
    STATE(875), 1,
      sym_type_arguments,
    ACTIONS(600), 41,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [23939] = 19,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(624), 1,
      aux_sym_source_file_token1,
    ACTIONS(628), 1,
      anon_sym_DOT,
    ACTIONS(630), 1,
      anon_sym_LPAREN,
    ACTIONS(632), 1,
      anon_sym_COMMA,
    ACTIONS(636), 1,
      anon_sym_LBRACK,
    ACTIONS(642), 1,
      anon_sym_LT_DASH,
    ACTIONS(644), 1,
      anon_sym_PLUS_PLUS,
    ACTIONS(646), 1,
      anon_sym_DASH_DASH,
    ACTIONS(650), 1,
      anon_sym_AMP_AMP,
    ACTIONS(652), 1,
      anon_sym_PIPE_PIPE,
    STATE(260), 1,
      sym_argument_list,
    STATE(797), 1,
      aux_sym_expression_list_repeat1,
    STATE(1291), 1,
      sym_type_arguments,
    ACTIONS(640), 4,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(626), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
    ACTIONS(648), 6,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
    ACTIONS(638), 7,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
    ACTIONS(634), 13,
      anon_sym_EQ,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
  [24027] = 8,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(628), 1,
      anon_sym_DOT,
    ACTIONS(630), 1,
      anon_sym_LPAREN,
    ACTIONS(636), 1,
      anon_sym_LBRACK,
    ACTIONS(654), 1,
      aux_sym_source_file_token1,
    STATE(260), 1,
      sym_argument_list,
    STATE(1291), 1,
      sym_type_arguments,
    ACTIONS(656), 41,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [24092] = 10,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(628), 1,
      anon_sym_DOT,
    ACTIONS(630), 1,
      anon_sym_LPAREN,
    ACTIONS(636), 1,
      anon_sym_LBRACK,
    ACTIONS(654), 1,
      aux_sym_source_file_token1,
    STATE(260), 1,
      sym_argument_list,
    STATE(1291), 1,
      sym_type_arguments,
    ACTIONS(640), 4,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(638), 7,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
    ACTIONS(656), 30,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_RBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [24161] = 11,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(628), 1,
      anon_sym_DOT,
    ACTIONS(630), 1,
      anon_sym_LPAREN,
    ACTIONS(636), 1,
      anon_sym_LBRACK,
    ACTIONS(654), 1,
      aux_sym_source_file_token1,
    STATE(260), 1,
      sym_argument_list,
    STATE(1291), 1,
      sym_type_arguments,
    ACTIONS(640), 4,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(648), 6,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
    ACTIONS(638), 7,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
    ACTIONS(656), 24,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_RBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [24232] = 9,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(628), 1,
      anon_sym_DOT,
    ACTIONS(630), 1,
      anon_sym_LPAREN,
    ACTIONS(636), 1,
      anon_sym_LBRACK,
    ACTIONS(654), 1,
      aux_sym_source_file_token1,
    STATE(260), 1,
      sym_argument_list,
    STATE(1291), 1,
      sym_type_arguments,
    ACTIONS(638), 7,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
    ACTIONS(656), 34,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [24299] = 8,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(628), 1,
      anon_sym_DOT,
    ACTIONS(630), 1,
      anon_sym_LPAREN,
    ACTIONS(636), 1,
      anon_sym_LBRACK,
    ACTIONS(658), 1,
      aux_sym_source_file_token1,
    STATE(260), 1,
      sym_argument_list,
    STATE(1291), 1,
      sym_type_arguments,
    ACTIONS(660), 41,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [24364] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(77), 17,
      ts_builtin_sym_end,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_TILDE,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BANG,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_BQUOTE,
      anon_sym_DQUOTE,
      sym_imaginary_literal,
      sym_rune_literal,
    ACTIONS(662), 30,
      anon_sym_package,
      anon_sym_import,
      anon_sym_const,
      anon_sym_var,
      anon_sym_func,
      anon_sym_type,
      anon_sym_struct,
      anon_sym_interface,
      anon_sym_map,
      anon_sym_chan,
      anon_sym_fallthrough,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_goto,
      anon_sym_return,
      anon_sym_go,
      anon_sym_defer,
      anon_sym_if,
      anon_sym_for,
      anon_sym_switch,
      anon_sym_select,
      anon_sym_new,
      anon_sym_make,
      sym_identifier,
      sym_int_literal,
      sym_float_literal,
      sym_nil,
      sym_true,
      sym_false,
      sym_iota,
  [24419] = 12,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(628), 1,
      anon_sym_DOT,
    ACTIONS(630), 1,
      anon_sym_LPAREN,
    ACTIONS(636), 1,
      anon_sym_LBRACK,
    ACTIONS(650), 1,
      anon_sym_AMP_AMP,
    ACTIONS(654), 1,
      aux_sym_source_file_token1,
    STATE(260), 1,
      sym_argument_list,
    STATE(1291), 1,
      sym_type_arguments,
    ACTIONS(640), 4,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(648), 6,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
    ACTIONS(638), 7,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
    ACTIONS(656), 23,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_RBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PIPE_PIPE,
  [24492] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(664), 1,
      aux_sym_source_file_token1,
    ACTIONS(666), 45,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_else,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [24546] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(668), 1,
      aux_sym_source_file_token1,
    ACTIONS(670), 45,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [24600] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(672), 1,
      aux_sym_source_file_token1,
    ACTIONS(674), 45,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [24654] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(676), 1,
      aux_sym_source_file_token1,
    ACTIONS(678), 45,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_else,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [24708] = 5,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(595), 1,
      aux_sym_source_file_token1,
    ACTIONS(680), 1,
      anon_sym_LPAREN,
    STATE(260), 1,
      sym_special_argument_list,
    ACTIONS(600), 43,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [24766] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(682), 1,
      aux_sym_source_file_token1,
    ACTIONS(684), 45,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [24820] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(686), 1,
      aux_sym_source_file_token1,
    ACTIONS(688), 44,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [24873] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(690), 1,
      aux_sym_source_file_token1,
    ACTIONS(692), 44,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [24926] = 22,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym_LBRACE,
    ACTIONS(624), 1,
      anon_sym_SEMI,
    ACTIONS(634), 1,
      anon_sym_EQ,
    ACTIONS(694), 1,
      anon_sym_DOT,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_COMMA,
    ACTIONS(700), 1,
      anon_sym_LBRACK,
    ACTIONS(706), 1,
      anon_sym_LT_DASH,
    ACTIONS(710), 1,
      anon_sym_PLUS_PLUS,
    ACTIONS(712), 1,
      anon_sym_DASH_DASH,
    ACTIONS(718), 1,
      anon_sym_AMP_AMP,
    ACTIONS(720), 1,
      anon_sym_PIPE_PIPE,
    STATE(323), 1,
      sym_argument_list,
    STATE(797), 1,
      aux_sym_expression_list_repeat1,
    STATE(915), 1,
      sym_block,
    STATE(1268), 1,
      sym_type_arguments,
    ACTIONS(716), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(704), 4,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(714), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(702), 7,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
    ACTIONS(708), 12,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
  [25017] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(722), 1,
      aux_sym_source_file_token1,
    ACTIONS(724), 44,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [25070] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(726), 1,
      aux_sym_source_file_token1,
    ACTIONS(728), 44,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [25123] = 22,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym_LBRACE,
    ACTIONS(624), 1,
      anon_sym_SEMI,
    ACTIONS(634), 1,
      anon_sym_EQ,
    ACTIONS(694), 1,
      anon_sym_DOT,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_COMMA,
    ACTIONS(700), 1,
      anon_sym_LBRACK,
    ACTIONS(706), 1,
      anon_sym_LT_DASH,
    ACTIONS(710), 1,
      anon_sym_PLUS_PLUS,
    ACTIONS(712), 1,
      anon_sym_DASH_DASH,
    ACTIONS(718), 1,
      anon_sym_AMP_AMP,
    ACTIONS(720), 1,
      anon_sym_PIPE_PIPE,
    STATE(323), 1,
      sym_argument_list,
    STATE(797), 1,
      aux_sym_expression_list_repeat1,
    STATE(992), 1,
      sym_block,
    STATE(1268), 1,
      sym_type_arguments,
    ACTIONS(716), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(704), 4,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(714), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(702), 7,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
    ACTIONS(708), 12,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
  [25214] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(730), 1,
      aux_sym_source_file_token1,
    ACTIONS(732), 44,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [25267] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(734), 1,
      aux_sym_source_file_token1,
    ACTIONS(736), 44,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [25320] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(738), 1,
      aux_sym_source_file_token1,
    ACTIONS(740), 44,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [25373] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(595), 1,
      aux_sym_source_file_token1,
    ACTIONS(600), 44,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [25426] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(742), 1,
      aux_sym_source_file_token1,
    ACTIONS(744), 44,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [25479] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(746), 1,
      aux_sym_source_file_token1,
    ACTIONS(748), 44,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [25532] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(750), 1,
      aux_sym_source_file_token1,
    ACTIONS(752), 44,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [25585] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(754), 1,
      aux_sym_source_file_token1,
    ACTIONS(756), 44,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [25638] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(758), 1,
      aux_sym_source_file_token1,
    ACTIONS(760), 44,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [25691] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(762), 1,
      aux_sym_source_file_token1,
    ACTIONS(764), 44,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [25744] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(766), 1,
      aux_sym_source_file_token1,
    ACTIONS(768), 44,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [25797] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(770), 1,
      aux_sym_source_file_token1,
    ACTIONS(772), 44,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [25850] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(774), 1,
      aux_sym_source_file_token1,
    ACTIONS(776), 44,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [25903] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(778), 1,
      aux_sym_source_file_token1,
    ACTIONS(780), 44,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [25956] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(782), 1,
      aux_sym_source_file_token1,
    ACTIONS(784), 44,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [26009] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(786), 1,
      aux_sym_source_file_token1,
    ACTIONS(788), 44,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [26062] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(790), 1,
      aux_sym_source_file_token1,
    ACTIONS(792), 44,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [26115] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(794), 1,
      aux_sym_source_file_token1,
    ACTIONS(796), 44,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [26168] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(798), 1,
      aux_sym_source_file_token1,
    ACTIONS(800), 44,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [26221] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(802), 1,
      aux_sym_source_file_token1,
    ACTIONS(804), 44,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [26274] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(806), 1,
      aux_sym_source_file_token1,
    ACTIONS(808), 44,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [26327] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(810), 1,
      aux_sym_source_file_token1,
    ACTIONS(812), 44,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [26380] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(814), 1,
      aux_sym_source_file_token1,
    ACTIONS(816), 44,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [26433] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(818), 1,
      aux_sym_source_file_token1,
    ACTIONS(820), 44,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [26486] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(822), 1,
      aux_sym_source_file_token1,
    ACTIONS(824), 44,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [26539] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(826), 1,
      aux_sym_source_file_token1,
    ACTIONS(828), 44,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [26592] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(830), 1,
      aux_sym_source_file_token1,
    ACTIONS(832), 44,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [26645] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(834), 1,
      aux_sym_source_file_token1,
    ACTIONS(836), 44,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [26698] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(838), 1,
      aux_sym_source_file_token1,
    ACTIONS(840), 44,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [26751] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(842), 1,
      aux_sym_source_file_token1,
    ACTIONS(844), 44,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [26804] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(624), 1,
      anon_sym_SEMI,
    ACTIONS(634), 1,
      anon_sym_EQ,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_COMMA,
    ACTIONS(700), 1,
      anon_sym_LBRACK,
    ACTIONS(706), 1,
      anon_sym_LT_DASH,
    ACTIONS(710), 1,
      anon_sym_PLUS_PLUS,
    ACTIONS(712), 1,
      anon_sym_DASH_DASH,
    ACTIONS(718), 1,
      anon_sym_AMP_AMP,
    ACTIONS(720), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(846), 1,
      anon_sym_DOT,
    ACTIONS(848), 1,
      anon_sym_LBRACE,
    STATE(323), 1,
      sym_argument_list,
    STATE(797), 1,
      aux_sym_expression_list_repeat1,
    STATE(1268), 1,
      sym_type_arguments,
    ACTIONS(716), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(704), 4,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(714), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(702), 7,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
    ACTIONS(708), 12,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
  [26892] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(656), 1,
      anon_sym_EQ,
    ACTIONS(694), 1,
      anon_sym_DOT,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(700), 1,
      anon_sym_LBRACK,
    STATE(323), 1,
      sym_argument_list,
    STATE(1268), 1,
      sym_type_arguments,
    ACTIONS(716), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(704), 4,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(714), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(702), 7,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
    ACTIONS(654), 20,
      anon_sym_SEMI,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [26961] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(624), 1,
      anon_sym_LBRACE,
    ACTIONS(634), 1,
      anon_sym_EQ,
    ACTIONS(694), 1,
      anon_sym_DOT,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(698), 1,
      anon_sym_COMMA,
    ACTIONS(700), 1,
      anon_sym_LBRACK,
    ACTIONS(710), 1,
      anon_sym_PLUS_PLUS,
    ACTIONS(712), 1,
      anon_sym_DASH_DASH,
    ACTIONS(718), 1,
      anon_sym_AMP_AMP,
    ACTIONS(720), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(850), 1,
      anon_sym_LT_DASH,
    STATE(323), 1,
      sym_argument_list,
    STATE(797), 1,
      aux_sym_expression_list_repeat1,
    STATE(1268), 1,
      sym_type_arguments,
    ACTIONS(716), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(704), 4,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(714), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(702), 7,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
    ACTIONS(708), 12,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
  [27046] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(694), 1,
      anon_sym_DOT,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(700), 1,
      anon_sym_LBRACK,
    STATE(323), 1,
      sym_argument_list,
    STATE(1268), 1,
      sym_type_arguments,
    ACTIONS(660), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(658), 24,
      anon_sym_SEMI,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [27107] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(580), 1,
      anon_sym_DOT,
    ACTIONS(852), 1,
      anon_sym_LPAREN,
    ACTIONS(855), 1,
      anon_sym_LBRACK,
    STATE(324), 1,
      sym_literal_value,
    STATE(875), 1,
      sym_type_arguments,
    ACTIONS(600), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(595), 24,
      anon_sym_SEMI,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [27168] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(694), 1,
      anon_sym_DOT,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(700), 1,
      anon_sym_LBRACK,
    STATE(323), 1,
      sym_argument_list,
    STATE(1268), 1,
      sym_type_arguments,
    ACTIONS(656), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(702), 7,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
    ACTIONS(654), 24,
      anon_sym_SEMI,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [27231] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(694), 1,
      anon_sym_DOT,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(700), 1,
      anon_sym_LBRACK,
    STATE(323), 1,
      sym_argument_list,
    STATE(1268), 1,
      sym_type_arguments,
    ACTIONS(656), 3,
      anon_sym_EQ,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(704), 4,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(702), 7,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
    ACTIONS(654), 24,
      anon_sym_SEMI,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [27296] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(656), 1,
      anon_sym_EQ,
    ACTIONS(694), 1,
      anon_sym_DOT,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(700), 1,
      anon_sym_LBRACK,
    ACTIONS(718), 1,
      anon_sym_AMP_AMP,
    STATE(323), 1,
      sym_argument_list,
    STATE(1268), 1,
      sym_type_arguments,
    ACTIONS(716), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(704), 4,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(714), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(702), 7,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
    ACTIONS(654), 19,
      anon_sym_SEMI,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_PIPE,
  [27367] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(694), 1,
      anon_sym_DOT,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    ACTIONS(700), 1,
      anon_sym_LBRACK,
    STATE(323), 1,
      sym_argument_list,
    STATE(1268), 1,
      sym_type_arguments,
    ACTIONS(656), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(654), 24,
      anon_sym_SEMI,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [27428] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(859), 1,
      anon_sym_LPAREN,
    STATE(323), 1,
      sym_special_argument_list,
    ACTIONS(600), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(595), 26,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [27482] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(740), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(738), 27,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [27531] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(800), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(798), 27,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [27580] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(804), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(802), 27,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [27629] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(808), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(806), 27,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [27678] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(812), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(810), 27,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [27727] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(816), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(814), 27,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [27776] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(828), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(826), 27,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [27825] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(840), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(838), 27,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [27874] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(836), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(834), 27,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [27923] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(844), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(842), 27,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [27972] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(692), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(690), 27,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [28021] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(832), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(830), 27,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [28070] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(768), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(766), 27,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [28119] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(788), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(786), 27,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [28168] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(736), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(734), 27,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [28217] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(684), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(682), 27,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [28266] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(724), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(722), 27,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [28315] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(756), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(754), 27,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [28364] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(760), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(758), 27,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [28413] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(600), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(595), 27,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [28462] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(688), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(686), 27,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [28511] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(674), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(672), 27,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [28560] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(666), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(664), 27,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [28609] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(732), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(730), 27,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [28658] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(728), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(726), 27,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [28707] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(744), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(742), 27,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [28756] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(670), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(668), 27,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [28805] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(764), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(762), 27,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [28854] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(772), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(770), 27,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [28903] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(776), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(774), 27,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [28952] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(780), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(778), 27,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [29001] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(796), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(794), 27,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [29050] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(678), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(676), 27,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [29099] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(784), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(782), 27,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [29148] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(748), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(746), 27,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [29197] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(820), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(818), 27,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [29246] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(824), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(822), 27,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [29295] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(792), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(790), 27,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [29344] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(752), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(750), 27,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_DASH,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [29393] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(580), 1,
      anon_sym_DOT,
    ACTIONS(852), 1,
      anon_sym_LPAREN,
    ACTIONS(855), 1,
      anon_sym_LBRACK,
    ACTIONS(861), 1,
      anon_sym_LBRACE,
    STATE(382), 1,
      sym_literal_value,
    STATE(875), 1,
      sym_type_arguments,
    ACTIONS(600), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(595), 19,
      anon_sym_COMMA,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [29452] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(863), 1,
      anon_sym_DOT,
    ACTIONS(865), 1,
      anon_sym_LPAREN,
    ACTIONS(869), 1,
      anon_sym_EQ,
    ACTIONS(871), 1,
      anon_sym_LBRACK,
    ACTIONS(881), 1,
      anon_sym_AMP_AMP,
    ACTIONS(883), 1,
      anon_sym_PIPE_PIPE,
    STATE(381), 1,
      sym_argument_list,
    STATE(1262), 1,
      sym_type_arguments,
    ACTIONS(879), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(875), 4,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(877), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(873), 7,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
    ACTIONS(867), 13,
      anon_sym_COMMA,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
  [29520] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(863), 1,
      anon_sym_DOT,
    ACTIONS(865), 1,
      anon_sym_LPAREN,
    ACTIONS(871), 1,
      anon_sym_LBRACK,
    STATE(381), 1,
      sym_argument_list,
    STATE(1262), 1,
      sym_type_arguments,
    ACTIONS(660), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(658), 19,
      anon_sym_COMMA,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [29576] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(863), 1,
      anon_sym_DOT,
    ACTIONS(865), 1,
      anon_sym_LPAREN,
    ACTIONS(871), 1,
      anon_sym_LBRACK,
    STATE(381), 1,
      sym_argument_list,
    STATE(1262), 1,
      sym_type_arguments,
    ACTIONS(656), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(654), 19,
      anon_sym_COMMA,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [29632] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(863), 1,
      anon_sym_DOT,
    ACTIONS(865), 1,
      anon_sym_LPAREN,
    ACTIONS(871), 1,
      anon_sym_LBRACK,
    STATE(381), 1,
      sym_argument_list,
    STATE(1262), 1,
      sym_type_arguments,
    ACTIONS(656), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(873), 7,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
    ACTIONS(654), 19,
      anon_sym_COMMA,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [29690] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(863), 1,
      anon_sym_DOT,
    ACTIONS(865), 1,
      anon_sym_LPAREN,
    ACTIONS(871), 1,
      anon_sym_LBRACK,
    STATE(381), 1,
      sym_argument_list,
    STATE(1262), 1,
      sym_type_arguments,
    ACTIONS(656), 3,
      anon_sym_EQ,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(875), 4,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(873), 7,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
    ACTIONS(654), 19,
      anon_sym_COMMA,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [29750] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(656), 1,
      anon_sym_EQ,
    ACTIONS(863), 1,
      anon_sym_DOT,
    ACTIONS(865), 1,
      anon_sym_LPAREN,
    ACTIONS(871), 1,
      anon_sym_LBRACK,
    STATE(381), 1,
      sym_argument_list,
    STATE(1262), 1,
      sym_type_arguments,
    ACTIONS(879), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(875), 4,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(877), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(873), 7,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
    ACTIONS(654), 15,
      anon_sym_COMMA,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [29814] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(656), 1,
      anon_sym_EQ,
    ACTIONS(863), 1,
      anon_sym_DOT,
    ACTIONS(865), 1,
      anon_sym_LPAREN,
    ACTIONS(871), 1,
      anon_sym_LBRACK,
    ACTIONS(881), 1,
      anon_sym_AMP_AMP,
    STATE(381), 1,
      sym_argument_list,
    STATE(1262), 1,
      sym_type_arguments,
    ACTIONS(879), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(875), 4,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(877), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(873), 7,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
    ACTIONS(654), 14,
      anon_sym_COMMA,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_PIPE,
  [29880] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(885), 1,
      anon_sym_LPAREN,
    STATE(381), 1,
      sym_special_argument_list,
    ACTIONS(600), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(595), 21,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [29929] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(772), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(770), 22,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [29973] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(678), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(676), 22,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [30017] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(740), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(738), 22,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [30061] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(744), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(742), 22,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [30105] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(600), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(595), 22,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [30149] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(724), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(722), 22,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [30193] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(736), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(734), 22,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [30237] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(752), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(750), 22,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [30281] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(756), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(754), 22,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [30325] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(760), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(758), 22,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [30369] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(688), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(686), 22,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [30413] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(764), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(762), 22,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [30457] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(780), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(778), 22,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [30501] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(776), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(774), 22,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [30545] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(784), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(782), 22,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [30589] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(792), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(790), 22,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [30633] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(796), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(794), 22,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [30677] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(804), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(802), 22,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [30721] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(808), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(806), 22,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [30765] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(812), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(810), 22,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [30809] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(816), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(814), 22,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [30853] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(828), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(826), 22,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [30897] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(840), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(838), 22,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [30941] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(836), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(834), 22,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [30985] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(844), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(842), 22,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [31029] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(692), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(690), 22,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [31073] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(832), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(830), 22,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [31117] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(768), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(766), 22,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [31161] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(788), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(786), 22,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [31205] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(684), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(682), 22,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [31249] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(674), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(672), 22,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [31293] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(670), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(668), 22,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [31337] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(748), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(746), 22,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [31381] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(732), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(730), 22,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [31425] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(728), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(726), 22,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [31469] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(820), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(818), 22,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [31513] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(824), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(822), 22,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [31557] = 24,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(887), 1,
      sym_identifier,
    ACTIONS(889), 1,
      aux_sym_source_file_token1,
    ACTIONS(893), 1,
      anon_sym_DOT,
    ACTIONS(895), 1,
      anon_sym_LPAREN,
    ACTIONS(897), 1,
      anon_sym_COMMA,
    ACTIONS(899), 1,
      anon_sym_func,
    ACTIONS(901), 1,
      anon_sym_LBRACK,
    ACTIONS(903), 1,
      anon_sym_STAR,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(907), 1,
      anon_sym_TILDE,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(911), 1,
      anon_sym_map,
    ACTIONS(913), 1,
      anon_sym_chan,
    ACTIONS(915), 1,
      anon_sym_LT_DASH,
    ACTIONS(917), 1,
      anon_sym_BQUOTE,
    ACTIONS(919), 1,
      anon_sym_DQUOTE,
    STATE(634), 1,
      aux_sym_field_declaration_repeat1,
    STATE(827), 1,
      sym_type_arguments,
    STATE(806), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(901), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(1123), 2,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
    ACTIONS(891), 3,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [31643] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(666), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(664), 22,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [31687] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(800), 14,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(798), 22,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [31731] = 17,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(887), 1,
      sym_identifier,
    ACTIONS(899), 1,
      anon_sym_func,
    ACTIONS(903), 1,
      anon_sym_STAR,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(907), 1,
      anon_sym_TILDE,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(911), 1,
      anon_sym_map,
    ACTIONS(913), 1,
      anon_sym_chan,
    ACTIONS(915), 1,
      anon_sym_LT_DASH,
    ACTIONS(921), 1,
      aux_sym_source_file_token1,
    ACTIONS(925), 1,
      anon_sym_LPAREN,
    ACTIONS(927), 1,
      anon_sym_LBRACK,
    STATE(806), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(836), 2,
      sym_parameter_list,
      sym__simple_type,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(923), 10,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_EQ,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_BQUOTE,
      anon_sym_DQUOTE,
  [31802] = 17,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(907), 1,
      anon_sym_TILDE,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(921), 1,
      aux_sym_source_file_token1,
    ACTIONS(925), 1,
      anon_sym_LPAREN,
    ACTIONS(929), 1,
      sym_identifier,
    ACTIONS(931), 1,
      anon_sym_func,
    ACTIONS(933), 1,
      anon_sym_LBRACK,
    ACTIONS(935), 1,
      anon_sym_STAR,
    ACTIONS(937), 1,
      anon_sym_map,
    ACTIONS(939), 1,
      anon_sym_chan,
    ACTIONS(941), 1,
      anon_sym_LT_DASH,
    STATE(803), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(836), 2,
      sym_parameter_list,
      sym__simple_type,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
    ACTIONS(923), 10,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_EQ,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_BQUOTE,
      anon_sym_DQUOTE,
  [31873] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(943), 1,
      anon_sym_DOT,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(660), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(658), 22,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [31925] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(943), 1,
      anon_sym_DOT,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(656), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(654), 22,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [31977] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(949), 1,
      anon_sym_LPAREN,
    STATE(398), 1,
      sym_special_argument_list,
    ACTIONS(600), 8,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(595), 23,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [32022] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(312), 1,
      anon_sym_LBRACE,
    ACTIONS(580), 1,
      anon_sym_DOT,
    ACTIONS(855), 1,
      anon_sym_LBRACK,
    STATE(414), 1,
      sym_literal_value,
    STATE(875), 1,
      sym_type_arguments,
    ACTIONS(852), 2,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
    ACTIONS(600), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(595), 19,
      anon_sym_SEMI,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [32075] = 20,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(286), 1,
      anon_sym_TILDE,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(951), 1,
      aux_sym_source_file_token1,
    ACTIONS(955), 1,
      anon_sym_LPAREN,
    ACTIONS(957), 1,
      anon_sym_COMMA,
    ACTIONS(959), 1,
      anon_sym_EQ,
    ACTIONS(961), 1,
      anon_sym_LBRACK,
    ACTIONS(963), 1,
      anon_sym_STAR,
    ACTIONS(965), 1,
      anon_sym_LT_DASH,
    STATE(395), 1,
      aux_sym_const_spec_repeat1,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1423), 2,
      sym_parenthesized_type,
      sym__simple_type,
    ACTIONS(953), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [32150] = 20,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(286), 1,
      anon_sym_TILDE,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(955), 1,
      anon_sym_LPAREN,
    ACTIONS(957), 1,
      anon_sym_COMMA,
    ACTIONS(961), 1,
      anon_sym_LBRACK,
    ACTIONS(963), 1,
      anon_sym_STAR,
    ACTIONS(965), 1,
      anon_sym_LT_DASH,
    ACTIONS(967), 1,
      aux_sym_source_file_token1,
    ACTIONS(971), 1,
      anon_sym_EQ,
    STATE(786), 1,
      aux_sym_const_spec_repeat1,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1337), 2,
      sym_parenthesized_type,
      sym__simple_type,
    ACTIONS(969), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [32225] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(784), 8,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(782), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [32265] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(666), 8,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(664), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [32305] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(732), 8,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(730), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [32345] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(836), 8,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(834), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [32385] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(816), 8,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(814), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [32425] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(800), 8,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(798), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [32465] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(674), 8,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(672), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [32505] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(670), 8,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(668), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [32545] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(804), 8,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(802), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [32585] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(844), 8,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(842), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [32625] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(772), 8,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(770), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [32665] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(692), 8,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(690), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [32705] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(828), 8,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(826), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [32745] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(780), 8,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(778), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [32785] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(832), 8,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(830), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [32825] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(796), 8,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(794), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [32865] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(678), 8,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(676), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [32905] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(684), 8,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(682), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [32945] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(728), 8,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(726), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [32985] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(748), 8,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(746), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [33025] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(768), 8,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(766), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [33065] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(788), 8,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(786), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [33105] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(776), 8,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(774), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [33145] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(600), 8,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(595), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [33185] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(820), 8,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(818), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [33225] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(840), 8,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(838), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [33265] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(824), 8,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(822), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [33305] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(744), 8,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(742), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [33345] = 15,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(708), 1,
      aux_sym_source_file_token1,
    ACTIONS(973), 1,
      anon_sym_DOT,
    ACTIONS(975), 1,
      anon_sym_LPAREN,
    ACTIONS(977), 1,
      anon_sym_COMMA,
    ACTIONS(979), 1,
      anon_sym_LBRACK,
    ACTIONS(987), 1,
      anon_sym_AMP_AMP,
    ACTIONS(989), 1,
      anon_sym_PIPE_PIPE,
    STATE(512), 1,
      sym_argument_list,
    STATE(886), 1,
      aux_sym_expression_list_repeat1,
    STATE(1299), 1,
      sym_type_arguments,
    ACTIONS(983), 4,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(634), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
    ACTIONS(985), 6,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
    ACTIONS(981), 7,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [33409] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(808), 8,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(806), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [33449] = 9,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(595), 1,
      aux_sym_source_file_token1,
    ACTIONS(608), 1,
      anon_sym_DOT,
    ACTIONS(611), 1,
      anon_sym_LPAREN,
    ACTIONS(614), 1,
      anon_sym_LBRACK,
    ACTIONS(991), 1,
      anon_sym_LBRACE,
    STATE(513), 1,
      sym_literal_value,
    STATE(875), 1,
      sym_type_arguments,
    ACTIONS(600), 25,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_COMMA,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [33501] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(724), 8,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(722), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [33541] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(812), 8,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(810), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [33581] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(736), 8,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(734), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [33621] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(752), 8,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(750), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [33661] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(756), 8,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(754), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [33701] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(760), 8,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(758), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [33741] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(792), 8,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(790), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [33781] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(688), 8,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(686), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [33821] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(764), 8,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(762), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [33861] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(740), 8,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(738), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [33901] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(656), 5,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
    ACTIONS(654), 14,
      anon_sym_SEMI,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [33954] = 12,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(654), 1,
      aux_sym_source_file_token1,
    ACTIONS(973), 1,
      anon_sym_DOT,
    ACTIONS(975), 1,
      anon_sym_LPAREN,
    ACTIONS(979), 1,
      anon_sym_LBRACK,
    ACTIONS(987), 1,
      anon_sym_AMP_AMP,
    STATE(512), 1,
      sym_argument_list,
    STATE(1299), 1,
      sym_type_arguments,
    ACTIONS(983), 4,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(985), 6,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
    ACTIONS(656), 7,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PIPE_PIPE,
    ACTIONS(981), 7,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [34011] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(656), 2,
      anon_sym_EQ,
      anon_sym_COLON,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
    ACTIONS(654), 6,
      anon_sym_SEMI,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PIPE_PIPE,
  [34074] = 11,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(654), 1,
      aux_sym_source_file_token1,
    ACTIONS(973), 1,
      anon_sym_DOT,
    ACTIONS(975), 1,
      anon_sym_LPAREN,
    ACTIONS(979), 1,
      anon_sym_LBRACK,
    STATE(512), 1,
      sym_argument_list,
    STATE(1299), 1,
      sym_type_arguments,
    ACTIONS(983), 4,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(985), 6,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
    ACTIONS(981), 7,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
    ACTIONS(656), 8,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [34129] = 8,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(654), 1,
      aux_sym_source_file_token1,
    ACTIONS(973), 1,
      anon_sym_DOT,
    ACTIONS(975), 1,
      anon_sym_LPAREN,
    ACTIONS(979), 1,
      anon_sym_LBRACK,
    STATE(512), 1,
      sym_argument_list,
    STATE(1299), 1,
      sym_type_arguments,
    ACTIONS(656), 25,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_COMMA,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [34178] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1009), 1,
      anon_sym_LPAREN,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    STATE(868), 2,
      sym_parameter_list,
      sym__simple_type,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    ACTIONS(921), 7,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_PIPE,
      anon_sym_COLON,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [34243] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(656), 4,
      anon_sym_EQ,
      anon_sym_COLON,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
    ACTIONS(654), 11,
      anon_sym_SEMI,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [34300] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(656), 2,
      anon_sym_EQ,
      anon_sym_COLON,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
    ACTIONS(654), 7,
      anon_sym_SEMI,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
      anon_sym_COLON_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [34361] = 13,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(867), 1,
      aux_sym_source_file_token1,
    ACTIONS(973), 1,
      anon_sym_DOT,
    ACTIONS(975), 1,
      anon_sym_LPAREN,
    ACTIONS(979), 1,
      anon_sym_LBRACK,
    ACTIONS(987), 1,
      anon_sym_AMP_AMP,
    ACTIONS(989), 1,
      anon_sym_PIPE_PIPE,
    STATE(512), 1,
      sym_argument_list,
    STATE(1299), 1,
      sym_type_arguments,
    ACTIONS(983), 4,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(869), 6,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
    ACTIONS(985), 6,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
    ACTIONS(981), 7,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [34420] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(1009), 1,
      anon_sym_LPAREN,
    ACTIONS(1015), 1,
      sym_identifier,
    ACTIONS(1017), 1,
      anon_sym_func,
    ACTIONS(1019), 1,
      anon_sym_LBRACK,
    ACTIONS(1021), 1,
      anon_sym_STAR,
    ACTIONS(1023), 1,
      anon_sym_LT_DASH,
    STATE(850), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(868), 2,
      sym_parameter_list,
      sym__simple_type,
    ACTIONS(921), 7,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_PIPE,
      anon_sym_COLON,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [34485] = 8,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(658), 1,
      aux_sym_source_file_token1,
    ACTIONS(973), 1,
      anon_sym_DOT,
    ACTIONS(975), 1,
      anon_sym_LPAREN,
    ACTIONS(979), 1,
      anon_sym_LBRACK,
    STATE(512), 1,
      sym_argument_list,
    STATE(1299), 1,
      sym_type_arguments,
    ACTIONS(660), 25,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_COMMA,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [34534] = 9,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(654), 1,
      aux_sym_source_file_token1,
    ACTIONS(973), 1,
      anon_sym_DOT,
    ACTIONS(975), 1,
      anon_sym_LPAREN,
    ACTIONS(979), 1,
      anon_sym_LBRACK,
    STATE(512), 1,
      sym_argument_list,
    STATE(1299), 1,
      sym_type_arguments,
    ACTIONS(981), 7,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
    ACTIONS(656), 18,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [34585] = 10,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(654), 1,
      aux_sym_source_file_token1,
    ACTIONS(973), 1,
      anon_sym_DOT,
    ACTIONS(975), 1,
      anon_sym_LPAREN,
    ACTIONS(979), 1,
      anon_sym_LBRACK,
    STATE(512), 1,
      sym_argument_list,
    STATE(1299), 1,
      sym_type_arguments,
    ACTIONS(983), 4,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(981), 7,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
    ACTIONS(656), 14,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [34638] = 13,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(973), 1,
      anon_sym_DOT,
    ACTIONS(975), 1,
      anon_sym_LPAREN,
    ACTIONS(979), 1,
      anon_sym_LBRACK,
    ACTIONS(987), 1,
      anon_sym_AMP_AMP,
    ACTIONS(989), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1025), 1,
      aux_sym_source_file_token1,
    STATE(512), 1,
      sym_argument_list,
    STATE(1299), 1,
      sym_type_arguments,
    ACTIONS(983), 4,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1027), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
    ACTIONS(985), 6,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
    ACTIONS(981), 7,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [34696] = 13,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(973), 1,
      anon_sym_DOT,
    ACTIONS(975), 1,
      anon_sym_LPAREN,
    ACTIONS(979), 1,
      anon_sym_LBRACK,
    ACTIONS(987), 1,
      anon_sym_AMP_AMP,
    ACTIONS(989), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1029), 1,
      aux_sym_source_file_token1,
    STATE(512), 1,
      sym_argument_list,
    STATE(1299), 1,
      sym_type_arguments,
    ACTIONS(983), 4,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1031), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
    ACTIONS(985), 6,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
    ACTIONS(981), 7,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [34754] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(1009), 1,
      anon_sym_LPAREN,
    ACTIONS(1015), 1,
      sym_identifier,
    ACTIONS(1017), 1,
      anon_sym_func,
    ACTIONS(1019), 1,
      anon_sym_LBRACK,
    ACTIONS(1021), 1,
      anon_sym_STAR,
    ACTIONS(1023), 1,
      anon_sym_LT_DASH,
    ACTIONS(1033), 1,
      anon_sym_LBRACE,
    STATE(411), 1,
      sym_block,
    STATE(850), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(889), 2,
      sym_parameter_list,
      sym__simple_type,
    ACTIONS(921), 4,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PIPE,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [34822] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1035), 1,
      anon_sym_DOT,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1039), 1,
      anon_sym_RPAREN,
    ACTIONS(1041), 1,
      anon_sym_COMMA,
    ACTIONS(1044), 1,
      anon_sym_LBRACK,
    ACTIONS(1046), 1,
      anon_sym_DOT_DOT_DOT,
    STATE(609), 1,
      aux_sym_var_spec_repeat1,
    STATE(875), 1,
      sym_type_arguments,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1312), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [34896] = 19,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1050), 1,
      sym_identifier,
    ACTIONS(1054), 1,
      anon_sym_LPAREN,
    ACTIONS(1056), 1,
      anon_sym_func,
    ACTIONS(1058), 1,
      anon_sym_LBRACK,
    ACTIONS(1060), 1,
      anon_sym_STAR,
    ACTIONS(1062), 1,
      anon_sym_struct,
    ACTIONS(1064), 1,
      anon_sym_TILDE,
    ACTIONS(1066), 1,
      anon_sym_LBRACE,
    ACTIONS(1068), 1,
      anon_sym_interface,
    ACTIONS(1070), 1,
      anon_sym_map,
    ACTIONS(1072), 1,
      anon_sym_chan,
    ACTIONS(1074), 1,
      anon_sym_LT_DASH,
    STATE(1111), 1,
      sym_block,
    ACTIONS(1048), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(1052), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
    STATE(918), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(924), 2,
      sym_parameter_list,
      sym__simple_type,
    STATE(922), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [34966] = 5,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(595), 1,
      aux_sym_source_file_token1,
    ACTIONS(1076), 1,
      anon_sym_LPAREN,
    STATE(512), 1,
      sym_special_argument_list,
    ACTIONS(600), 27,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [35008] = 13,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(973), 1,
      anon_sym_DOT,
    ACTIONS(975), 1,
      anon_sym_LPAREN,
    ACTIONS(979), 1,
      anon_sym_LBRACK,
    ACTIONS(987), 1,
      anon_sym_AMP_AMP,
    ACTIONS(989), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1078), 1,
      aux_sym_source_file_token1,
    STATE(512), 1,
      sym_argument_list,
    STATE(1299), 1,
      sym_type_arguments,
    ACTIONS(983), 4,
      anon_sym_PIPE,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1080), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
    ACTIONS(985), 6,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
    ACTIONS(981), 7,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [35066] = 19,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1050), 1,
      sym_identifier,
    ACTIONS(1054), 1,
      anon_sym_LPAREN,
    ACTIONS(1056), 1,
      anon_sym_func,
    ACTIONS(1058), 1,
      anon_sym_LBRACK,
    ACTIONS(1060), 1,
      anon_sym_STAR,
    ACTIONS(1062), 1,
      anon_sym_struct,
    ACTIONS(1064), 1,
      anon_sym_TILDE,
    ACTIONS(1066), 1,
      anon_sym_LBRACE,
    ACTIONS(1068), 1,
      anon_sym_interface,
    ACTIONS(1070), 1,
      anon_sym_map,
    ACTIONS(1072), 1,
      anon_sym_chan,
    ACTIONS(1074), 1,
      anon_sym_LT_DASH,
    STATE(1101), 1,
      sym_block,
    ACTIONS(1082), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(1084), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
    STATE(918), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1008), 2,
      sym_parameter_list,
      sym__simple_type,
    STATE(922), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [35136] = 19,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1050), 1,
      sym_identifier,
    ACTIONS(1054), 1,
      anon_sym_LPAREN,
    ACTIONS(1056), 1,
      anon_sym_func,
    ACTIONS(1058), 1,
      anon_sym_LBRACK,
    ACTIONS(1060), 1,
      anon_sym_STAR,
    ACTIONS(1062), 1,
      anon_sym_struct,
    ACTIONS(1064), 1,
      anon_sym_TILDE,
    ACTIONS(1066), 1,
      anon_sym_LBRACE,
    ACTIONS(1068), 1,
      anon_sym_interface,
    ACTIONS(1070), 1,
      anon_sym_map,
    ACTIONS(1072), 1,
      anon_sym_chan,
    ACTIONS(1074), 1,
      anon_sym_LT_DASH,
    STATE(1106), 1,
      sym_block,
    ACTIONS(1086), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(1088), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
    STATE(918), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1009), 2,
      sym_parameter_list,
      sym__simple_type,
    STATE(922), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [35206] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(580), 1,
      anon_sym_DOT,
    ACTIONS(852), 1,
      anon_sym_LPAREN,
    ACTIONS(855), 1,
      anon_sym_LBRACK,
    ACTIONS(1090), 1,
      anon_sym_LBRACE,
    STATE(596), 1,
      sym_literal_value,
    STATE(875), 1,
      sym_type_arguments,
    ACTIONS(600), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(595), 17,
      anon_sym_COMMA,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [35256] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(634), 1,
      anon_sym_EQ,
    ACTIONS(706), 1,
      anon_sym_LT_DASH,
    ACTIONS(708), 1,
      anon_sym_COLON_EQ,
    ACTIONS(1092), 1,
      anon_sym_DOT,
    ACTIONS(1094), 1,
      anon_sym_LPAREN,
    ACTIONS(1096), 1,
      anon_sym_COMMA,
    ACTIONS(1098), 1,
      anon_sym_LBRACK,
    ACTIONS(1102), 1,
      anon_sym_PIPE,
    ACTIONS(1104), 1,
      anon_sym_COLON,
    ACTIONS(1114), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1116), 1,
      anon_sym_PIPE_PIPE,
    STATE(595), 1,
      sym_argument_list,
    STATE(935), 1,
      aux_sym_expression_list_repeat1,
    STATE(1252), 1,
      sym_type_arguments,
    ACTIONS(1108), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1112), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1106), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1110), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1100), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [35328] = 19,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(887), 1,
      sym_identifier,
    ACTIONS(899), 1,
      anon_sym_func,
    ACTIONS(903), 1,
      anon_sym_STAR,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(907), 1,
      anon_sym_TILDE,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(911), 1,
      anon_sym_map,
    ACTIONS(913), 1,
      anon_sym_chan,
    ACTIONS(915), 1,
      anon_sym_LT_DASH,
    ACTIONS(925), 1,
      anon_sym_LPAREN,
    ACTIONS(927), 1,
      anon_sym_LBRACK,
    ACTIONS(1082), 1,
      aux_sym_source_file_token1,
    ACTIONS(1118), 1,
      anon_sym_LBRACE,
    STATE(1206), 1,
      sym_block,
    ACTIONS(1084), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
    STATE(806), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1026), 2,
      sym_parameter_list,
      sym__simple_type,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [35397] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(742), 1,
      aux_sym_source_file_token1,
    ACTIONS(744), 28,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [35434] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(778), 1,
      aux_sym_source_file_token1,
    ACTIONS(780), 28,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [35471] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(794), 1,
      aux_sym_source_file_token1,
    ACTIONS(796), 28,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [35508] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(738), 1,
      aux_sym_source_file_token1,
    ACTIONS(740), 28,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [35545] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(746), 1,
      aux_sym_source_file_token1,
    ACTIONS(748), 28,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [35582] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(818), 1,
      aux_sym_source_file_token1,
    ACTIONS(820), 28,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [35619] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(822), 1,
      aux_sym_source_file_token1,
    ACTIONS(824), 28,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [35656] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(722), 1,
      aux_sym_source_file_token1,
    ACTIONS(724), 28,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [35693] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(734), 1,
      aux_sym_source_file_token1,
    ACTIONS(736), 28,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [35730] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(750), 1,
      aux_sym_source_file_token1,
    ACTIONS(752), 28,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [35767] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(754), 1,
      aux_sym_source_file_token1,
    ACTIONS(756), 28,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [35804] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(758), 1,
      aux_sym_source_file_token1,
    ACTIONS(760), 28,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [35841] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(686), 1,
      aux_sym_source_file_token1,
    ACTIONS(688), 28,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [35878] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(762), 1,
      aux_sym_source_file_token1,
    ACTIONS(764), 28,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [35915] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(770), 1,
      aux_sym_source_file_token1,
    ACTIONS(772), 28,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [35952] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(774), 1,
      aux_sym_source_file_token1,
    ACTIONS(776), 28,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [35989] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(782), 1,
      aux_sym_source_file_token1,
    ACTIONS(784), 28,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [36026] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(790), 1,
      aux_sym_source_file_token1,
    ACTIONS(792), 28,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [36063] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(798), 1,
      aux_sym_source_file_token1,
    ACTIONS(800), 28,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [36100] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(802), 1,
      aux_sym_source_file_token1,
    ACTIONS(804), 28,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [36137] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(806), 1,
      aux_sym_source_file_token1,
    ACTIONS(808), 28,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [36174] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(810), 1,
      aux_sym_source_file_token1,
    ACTIONS(812), 28,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [36211] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(814), 1,
      aux_sym_source_file_token1,
    ACTIONS(816), 28,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [36248] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(826), 1,
      aux_sym_source_file_token1,
    ACTIONS(828), 28,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [36285] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(838), 1,
      aux_sym_source_file_token1,
    ACTIONS(840), 28,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [36322] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(834), 1,
      aux_sym_source_file_token1,
    ACTIONS(836), 28,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [36359] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(842), 1,
      aux_sym_source_file_token1,
    ACTIONS(844), 28,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [36396] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(690), 1,
      aux_sym_source_file_token1,
    ACTIONS(692), 28,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [36433] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(830), 1,
      aux_sym_source_file_token1,
    ACTIONS(832), 28,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [36470] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(766), 1,
      aux_sym_source_file_token1,
    ACTIONS(768), 28,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [36507] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(786), 1,
      aux_sym_source_file_token1,
    ACTIONS(788), 28,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [36544] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(682), 1,
      aux_sym_source_file_token1,
    ACTIONS(684), 28,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [36581] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1120), 1,
      sym_identifier,
    ACTIONS(1122), 1,
      anon_sym_RPAREN,
    ACTIONS(1124), 1,
      anon_sym_COMMA,
    ACTIONS(1126), 1,
      anon_sym_DOT_DOT_DOT,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1187), 2,
      sym_parameter_declaration,
      sym_variadic_parameter_declaration,
    STATE(1267), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [36650] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(672), 1,
      aux_sym_source_file_token1,
    ACTIONS(674), 28,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [36687] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(668), 1,
      aux_sym_source_file_token1,
    ACTIONS(670), 28,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [36724] = 19,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(887), 1,
      sym_identifier,
    ACTIONS(899), 1,
      anon_sym_func,
    ACTIONS(903), 1,
      anon_sym_STAR,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(907), 1,
      anon_sym_TILDE,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(911), 1,
      anon_sym_map,
    ACTIONS(913), 1,
      anon_sym_chan,
    ACTIONS(915), 1,
      anon_sym_LT_DASH,
    ACTIONS(925), 1,
      anon_sym_LPAREN,
    ACTIONS(927), 1,
      anon_sym_LBRACK,
    ACTIONS(1048), 1,
      aux_sym_source_file_token1,
    ACTIONS(1118), 1,
      anon_sym_LBRACE,
    STATE(1185), 1,
      sym_block,
    ACTIONS(1052), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
    STATE(806), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1054), 2,
      sym_parameter_list,
      sym__simple_type,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [36793] = 17,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1050), 1,
      sym_identifier,
    ACTIONS(1054), 1,
      anon_sym_LPAREN,
    ACTIONS(1056), 1,
      anon_sym_func,
    ACTIONS(1058), 1,
      anon_sym_LBRACK,
    ACTIONS(1060), 1,
      anon_sym_STAR,
    ACTIONS(1062), 1,
      anon_sym_struct,
    ACTIONS(1064), 1,
      anon_sym_TILDE,
    ACTIONS(1068), 1,
      anon_sym_interface,
    ACTIONS(1070), 1,
      anon_sym_map,
    ACTIONS(1072), 1,
      anon_sym_chan,
    ACTIONS(1074), 1,
      anon_sym_LT_DASH,
    ACTIONS(921), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    STATE(918), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(966), 2,
      sym_parameter_list,
      sym__simple_type,
    ACTIONS(923), 3,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LBRACE,
    STATE(922), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [36858] = 19,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(887), 1,
      sym_identifier,
    ACTIONS(899), 1,
      anon_sym_func,
    ACTIONS(903), 1,
      anon_sym_STAR,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(907), 1,
      anon_sym_TILDE,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(911), 1,
      anon_sym_map,
    ACTIONS(913), 1,
      anon_sym_chan,
    ACTIONS(915), 1,
      anon_sym_LT_DASH,
    ACTIONS(925), 1,
      anon_sym_LPAREN,
    ACTIONS(927), 1,
      anon_sym_LBRACK,
    ACTIONS(1086), 1,
      aux_sym_source_file_token1,
    ACTIONS(1118), 1,
      anon_sym_LBRACE,
    STATE(1212), 1,
      sym_block,
    ACTIONS(1088), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
    STATE(806), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1037), 2,
      sym_parameter_list,
      sym__simple_type,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [36927] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1092), 1,
      anon_sym_DOT,
    ACTIONS(1094), 1,
      anon_sym_LPAREN,
    ACTIONS(1098), 1,
      anon_sym_LBRACK,
    STATE(595), 1,
      sym_argument_list,
    STATE(1252), 1,
      sym_type_arguments,
    ACTIONS(660), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(658), 17,
      anon_sym_COMMA,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [36974] = 17,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1054), 1,
      anon_sym_LPAREN,
    ACTIONS(1062), 1,
      anon_sym_struct,
    ACTIONS(1064), 1,
      anon_sym_TILDE,
    ACTIONS(1068), 1,
      anon_sym_interface,
    ACTIONS(1128), 1,
      sym_identifier,
    ACTIONS(1130), 1,
      anon_sym_func,
    ACTIONS(1132), 1,
      anon_sym_LBRACK,
    ACTIONS(1134), 1,
      anon_sym_STAR,
    ACTIONS(1136), 1,
      anon_sym_map,
    ACTIONS(1138), 1,
      anon_sym_chan,
    ACTIONS(1140), 1,
      anon_sym_LT_DASH,
    ACTIONS(921), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    STATE(910), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(966), 2,
      sym_parameter_list,
      sym__simple_type,
    ACTIONS(923), 3,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LBRACE,
    STATE(922), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [37039] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1092), 1,
      anon_sym_DOT,
    ACTIONS(1094), 1,
      anon_sym_LPAREN,
    ACTIONS(1098), 1,
      anon_sym_LBRACK,
    STATE(595), 1,
      sym_argument_list,
    STATE(1252), 1,
      sym_type_arguments,
    ACTIONS(656), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(654), 17,
      anon_sym_COMMA,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [37086] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(664), 1,
      aux_sym_source_file_token1,
    ACTIONS(666), 28,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [37123] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(676), 1,
      aux_sym_source_file_token1,
    ACTIONS(678), 28,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [37160] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1092), 1,
      anon_sym_DOT,
    ACTIONS(1094), 1,
      anon_sym_LPAREN,
    ACTIONS(1098), 1,
      anon_sym_LBRACK,
    STATE(595), 1,
      sym_argument_list,
    STATE(1252), 1,
      sym_type_arguments,
    ACTIONS(1108), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(656), 5,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1100), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
    ACTIONS(654), 12,
      anon_sym_COMMA,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [37211] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1092), 1,
      anon_sym_DOT,
    ACTIONS(1094), 1,
      anon_sym_LPAREN,
    ACTIONS(1098), 1,
      anon_sym_LBRACK,
    ACTIONS(1102), 1,
      anon_sym_PIPE,
    STATE(595), 1,
      sym_argument_list,
    STATE(1252), 1,
      sym_type_arguments,
    ACTIONS(1108), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1106), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(656), 4,
      anon_sym_EQ,
      anon_sym_COLON,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1100), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
    ACTIONS(654), 9,
      anon_sym_COMMA,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [37266] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1092), 1,
      anon_sym_DOT,
    ACTIONS(1094), 1,
      anon_sym_LPAREN,
    ACTIONS(1098), 1,
      anon_sym_LBRACK,
    ACTIONS(1102), 1,
      anon_sym_PIPE,
    STATE(595), 1,
      sym_argument_list,
    STATE(1252), 1,
      sym_type_arguments,
    ACTIONS(656), 2,
      anon_sym_EQ,
      anon_sym_COLON,
    ACTIONS(1108), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1112), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1106), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1110), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(654), 5,
      anon_sym_COMMA,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
    ACTIONS(1100), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [37325] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1092), 1,
      anon_sym_DOT,
    ACTIONS(1094), 1,
      anon_sym_LPAREN,
    ACTIONS(1098), 1,
      anon_sym_LBRACK,
    ACTIONS(1102), 1,
      anon_sym_PIPE,
    ACTIONS(1114), 1,
      anon_sym_AMP_AMP,
    STATE(595), 1,
      sym_argument_list,
    STATE(1252), 1,
      sym_type_arguments,
    ACTIONS(656), 2,
      anon_sym_EQ,
      anon_sym_COLON,
    ACTIONS(1108), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1112), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1106), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(654), 4,
      anon_sym_COMMA,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PIPE_PIPE,
    ACTIONS(1110), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1100), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [37386] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(595), 1,
      aux_sym_source_file_token1,
    ACTIONS(600), 28,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [37423] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1120), 1,
      sym_identifier,
    ACTIONS(1126), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(1142), 1,
      anon_sym_RPAREN,
    ACTIONS(1144), 1,
      anon_sym_COMMA,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1161), 2,
      sym_parameter_declaration,
      sym_variadic_parameter_declaration,
    STATE(1267), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [37492] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1120), 1,
      sym_identifier,
    ACTIONS(1126), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(1146), 1,
      anon_sym_RPAREN,
    ACTIONS(1148), 1,
      anon_sym_COMMA,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1239), 2,
      sym_parameter_declaration,
      sym_variadic_parameter_declaration,
    STATE(1267), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [37561] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(730), 1,
      aux_sym_source_file_token1,
    ACTIONS(732), 28,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [37598] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(726), 1,
      aux_sym_source_file_token1,
    ACTIONS(728), 28,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [37635] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(869), 2,
      anon_sym_EQ,
      anon_sym_COLON,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(867), 3,
      anon_sym_SEMI,
      anon_sym_COMMA,
      anon_sym_COLON_EQ,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [37698] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(943), 1,
      anon_sym_DOT,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(1152), 1,
      anon_sym_RPAREN,
    ACTIONS(1154), 1,
      anon_sym_COMMA,
    ACTIONS(1156), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(1160), 1,
      anon_sym_PIPE,
    ACTIONS(1170), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1172), 1,
      anon_sym_PIPE_PIPE,
    STATE(398), 1,
      sym_argument_list,
    STATE(1181), 1,
      aux_sym_argument_list_repeat1,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(1164), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1168), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1162), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1166), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1158), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [37764] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1120), 1,
      sym_identifier,
    ACTIONS(1126), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(1174), 1,
      anon_sym_RPAREN,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1267), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(1304), 2,
      sym_parameter_declaration,
      sym_variadic_parameter_declaration,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [37830] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(312), 1,
      anon_sym_LBRACE,
    ACTIONS(580), 1,
      anon_sym_DOT,
    ACTIONS(611), 1,
      anon_sym_PIPE,
    ACTIONS(855), 1,
      anon_sym_LBRACK,
    ACTIONS(1039), 1,
      anon_sym_COMMA,
    STATE(414), 1,
      sym_literal_value,
    STATE(875), 1,
      sym_type_arguments,
    ACTIONS(852), 2,
      anon_sym_LPAREN,
      anon_sym_RBRACK,
    ACTIONS(600), 4,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(595), 15,
      anon_sym_STAR,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [37882] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(943), 1,
      anon_sym_DOT,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(1156), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(1160), 1,
      anon_sym_PIPE,
    ACTIONS(1170), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1172), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1176), 1,
      anon_sym_RPAREN,
    ACTIONS(1178), 1,
      anon_sym_COMMA,
    STATE(398), 1,
      sym_argument_list,
    STATE(1223), 1,
      aux_sym_argument_list_repeat1,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(1164), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1168), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1162), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1166), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1158), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [37948] = 17,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(887), 1,
      sym_identifier,
    ACTIONS(899), 1,
      anon_sym_func,
    ACTIONS(903), 1,
      anon_sym_STAR,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(907), 1,
      anon_sym_TILDE,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(911), 1,
      anon_sym_map,
    ACTIONS(913), 1,
      anon_sym_chan,
    ACTIONS(915), 1,
      anon_sym_LT_DASH,
    ACTIONS(925), 1,
      anon_sym_LPAREN,
    ACTIONS(927), 1,
      anon_sym_LBRACK,
    ACTIONS(1180), 1,
      aux_sym_source_file_token1,
    STATE(806), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1127), 2,
      sym_parameter_list,
      sym__simple_type,
    ACTIONS(1182), 3,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [38012] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(943), 1,
      anon_sym_DOT,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(1156), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(1160), 1,
      anon_sym_PIPE,
    ACTIONS(1170), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1172), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1184), 1,
      anon_sym_RPAREN,
    ACTIONS(1186), 1,
      anon_sym_COMMA,
    STATE(398), 1,
      sym_argument_list,
    STATE(1193), 1,
      aux_sym_argument_list_repeat1,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(1164), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1168), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1162), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1166), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1158), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [38078] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1120), 1,
      sym_identifier,
    ACTIONS(1126), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(1188), 1,
      anon_sym_RPAREN,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1267), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(1304), 2,
      sym_parameter_declaration,
      sym_variadic_parameter_declaration,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [38144] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1120), 1,
      sym_identifier,
    ACTIONS(1126), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(1190), 1,
      anon_sym_RPAREN,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1267), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(1304), 2,
      sym_parameter_declaration,
      sym_variadic_parameter_declaration,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [38210] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(708), 1,
      anon_sym_COLON_EQ,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(1096), 1,
      anon_sym_COMMA,
    ACTIONS(1192), 1,
      anon_sym_DOT,
    ACTIONS(1196), 1,
      anon_sym_LBRACE,
    ACTIONS(1198), 1,
      anon_sym_PIPE,
    ACTIONS(1208), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1210), 1,
      anon_sym_PIPE_PIPE,
    STATE(398), 1,
      sym_argument_list,
    STATE(935), 1,
      aux_sym_expression_list_repeat1,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(1202), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1206), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1200), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1204), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1194), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [38276] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(312), 1,
      anon_sym_LBRACE,
    ACTIONS(608), 1,
      anon_sym_DOT,
    ACTIONS(852), 1,
      anon_sym_LPAREN,
    ACTIONS(855), 1,
      anon_sym_LBRACK,
    STATE(414), 1,
      sym_literal_value,
    STATE(875), 1,
      sym_type_arguments,
    ACTIONS(600), 5,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(595), 17,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [38324] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1212), 1,
      anon_sym_LPAREN,
    STATE(595), 1,
      sym_special_argument_list,
    ACTIONS(600), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(595), 19,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [38364] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(943), 1,
      anon_sym_DOT,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(1156), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(1160), 1,
      anon_sym_PIPE,
    ACTIONS(1170), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1172), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1214), 1,
      anon_sym_RPAREN,
    ACTIONS(1216), 1,
      anon_sym_COMMA,
    STATE(398), 1,
      sym_argument_list,
    STATE(1155), 1,
      aux_sym_argument_list_repeat1,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(1164), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1168), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1162), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1166), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1158), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [38430] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1120), 1,
      sym_identifier,
    ACTIONS(1126), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(1218), 1,
      anon_sym_RPAREN,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1267), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(1304), 2,
      sym_parameter_declaration,
      sym_variadic_parameter_declaration,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [38496] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1120), 1,
      sym_identifier,
    ACTIONS(1126), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(1220), 1,
      anon_sym_RPAREN,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1267), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(1304), 2,
      sym_parameter_declaration,
      sym_variadic_parameter_declaration,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [38562] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1096), 1,
      anon_sym_COMMA,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    STATE(398), 1,
      sym_argument_list,
    STATE(935), 1,
      aux_sym_expression_list_repeat1,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(708), 2,
      anon_sym_SEMI,
      anon_sym_COLON,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [38626] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(943), 1,
      anon_sym_DOT,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(1156), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(1160), 1,
      anon_sym_PIPE,
    ACTIONS(1170), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1172), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1222), 1,
      anon_sym_RPAREN,
    ACTIONS(1224), 1,
      anon_sym_COMMA,
    STATE(398), 1,
      sym_argument_list,
    STATE(1180), 1,
      aux_sym_argument_list_repeat1,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(1164), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1168), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1162), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1166), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1158), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [38692] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(943), 1,
      anon_sym_DOT,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(1156), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(1160), 1,
      anon_sym_PIPE,
    ACTIONS(1170), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1172), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1226), 1,
      anon_sym_RPAREN,
    ACTIONS(1228), 1,
      anon_sym_COMMA,
    STATE(398), 1,
      sym_argument_list,
    STATE(1203), 1,
      aux_sym_argument_list_repeat1,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(1164), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1168), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1162), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1166), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1158), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [38758] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1120), 1,
      sym_identifier,
    ACTIONS(1126), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(1230), 1,
      anon_sym_RPAREN,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1267), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(1304), 2,
      sym_parameter_declaration,
      sym_variadic_parameter_declaration,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [38824] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(943), 1,
      anon_sym_DOT,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(1160), 1,
      anon_sym_PIPE,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(1164), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1168), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1162), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1166), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(654), 5,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
    ACTIONS(1158), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [38879] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(780), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(778), 20,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [38914] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(796), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(794), 20,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [38949] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(708), 1,
      anon_sym_LBRACE,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(1198), 1,
      anon_sym_PIPE,
    ACTIONS(1208), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1210), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1232), 1,
      anon_sym_COMMA,
    STATE(398), 1,
      sym_argument_list,
    STATE(1158), 1,
      aux_sym_expression_list_repeat1,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(1202), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1206), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1200), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1204), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1194), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [39012] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(678), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(676), 20,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [39047] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(740), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(738), 20,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [39082] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(748), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(746), 20,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [39117] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(820), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(818), 20,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [39152] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(824), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(822), 20,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [39187] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(724), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(722), 20,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [39222] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(736), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(734), 20,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [39257] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(752), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(750), 20,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [39292] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(756), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(754), 20,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [39327] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(760), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(758), 20,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [39362] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(688), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(686), 20,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [39397] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(764), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(762), 20,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [39432] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(772), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(770), 20,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [39467] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(776), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(774), 20,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [39502] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(784), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(782), 20,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [39537] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(792), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(790), 20,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [39572] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(800), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(798), 20,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [39607] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(804), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(802), 20,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [39642] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(808), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(806), 20,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [39677] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(812), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(810), 20,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [39712] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(816), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(814), 20,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [39747] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(828), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(826), 20,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [39782] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(840), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(838), 20,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [39817] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(836), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(834), 20,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [39852] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(844), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(842), 20,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [39887] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(692), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(690), 20,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [39922] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(832), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(830), 20,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [39957] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(768), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(766), 20,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [39992] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(788), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(786), 20,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [40027] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(943), 1,
      anon_sym_DOT,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(1160), 1,
      anon_sym_PIPE,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(656), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1164), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1162), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1158), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
    ACTIONS(654), 9,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [40080] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(684), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(682), 20,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [40115] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(899), 1,
      anon_sym_func,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(911), 1,
      anon_sym_map,
    ACTIONS(913), 1,
      anon_sym_chan,
    ACTIONS(1234), 1,
      sym_identifier,
    ACTIONS(1236), 1,
      anon_sym_LPAREN,
    ACTIONS(1238), 1,
      anon_sym_LBRACK,
    ACTIONS(1240), 1,
      anon_sym_STAR,
    ACTIONS(1242), 1,
      anon_sym_TILDE,
    ACTIONS(1244), 1,
      anon_sym_RBRACE,
    ACTIONS(1246), 1,
      anon_sym_LT_DASH,
    STATE(806), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(961), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(1133), 2,
      sym_method_elem,
      sym_type_elem,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [40178] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(744), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(742), 20,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [40213] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(674), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(672), 20,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [40248] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(670), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(668), 20,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [40283] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1248), 3,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_COLON,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [40342] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(587), 1,
      anon_sym_COMMA,
    ACTIONS(887), 1,
      sym_identifier,
    ACTIONS(899), 1,
      anon_sym_func,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(911), 1,
      anon_sym_map,
    ACTIONS(913), 1,
      anon_sym_chan,
    ACTIONS(1236), 1,
      anon_sym_LPAREN,
    ACTIONS(1238), 1,
      anon_sym_LBRACK,
    ACTIONS(1240), 1,
      anon_sym_STAR,
    ACTIONS(1242), 1,
      anon_sym_TILDE,
    ACTIONS(1246), 1,
      anon_sym_LT_DASH,
    ACTIONS(1250), 1,
      anon_sym_EQ,
    STATE(800), 1,
      aux_sym_var_spec_repeat1,
    STATE(909), 1,
      sym_parenthesized_type,
    STATE(911), 1,
      sym__simple_type,
    STATE(806), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [40409] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(899), 1,
      anon_sym_func,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(911), 1,
      anon_sym_map,
    ACTIONS(913), 1,
      anon_sym_chan,
    ACTIONS(1234), 1,
      sym_identifier,
    ACTIONS(1236), 1,
      anon_sym_LPAREN,
    ACTIONS(1238), 1,
      anon_sym_LBRACK,
    ACTIONS(1240), 1,
      anon_sym_STAR,
    ACTIONS(1242), 1,
      anon_sym_TILDE,
    ACTIONS(1246), 1,
      anon_sym_LT_DASH,
    ACTIONS(1252), 1,
      anon_sym_RBRACE,
    STATE(806), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(961), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(1133), 2,
      sym_method_elem,
      sym_type_elem,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [40472] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(943), 1,
      anon_sym_DOT,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(1156), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(1160), 1,
      anon_sym_PIPE,
    ACTIONS(1170), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1172), 1,
      anon_sym_PIPE_PIPE,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(1164), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1168), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1254), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(1162), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1166), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1158), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [40533] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(899), 1,
      anon_sym_func,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(911), 1,
      anon_sym_map,
    ACTIONS(913), 1,
      anon_sym_chan,
    ACTIONS(1234), 1,
      sym_identifier,
    ACTIONS(1236), 1,
      anon_sym_LPAREN,
    ACTIONS(1238), 1,
      anon_sym_LBRACK,
    ACTIONS(1240), 1,
      anon_sym_STAR,
    ACTIONS(1242), 1,
      anon_sym_TILDE,
    ACTIONS(1246), 1,
      anon_sym_LT_DASH,
    ACTIONS(1256), 1,
      anon_sym_RBRACE,
    STATE(806), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(961), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(1038), 2,
      sym_method_elem,
      sym_type_elem,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [40596] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(899), 1,
      anon_sym_func,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(911), 1,
      anon_sym_map,
    ACTIONS(913), 1,
      anon_sym_chan,
    ACTIONS(1234), 1,
      sym_identifier,
    ACTIONS(1236), 1,
      anon_sym_LPAREN,
    ACTIONS(1238), 1,
      anon_sym_LBRACK,
    ACTIONS(1240), 1,
      anon_sym_STAR,
    ACTIONS(1242), 1,
      anon_sym_TILDE,
    ACTIONS(1246), 1,
      anon_sym_LT_DASH,
    ACTIONS(1258), 1,
      anon_sym_RBRACE,
    STATE(806), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(961), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(1133), 2,
      sym_method_elem,
      sym_type_elem,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [40659] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1120), 1,
      sym_identifier,
    ACTIONS(1126), 1,
      anon_sym_DOT_DOT_DOT,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1267), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(1304), 2,
      sym_parameter_declaration,
      sym_variadic_parameter_declaration,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [40722] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(899), 1,
      anon_sym_func,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(911), 1,
      anon_sym_map,
    ACTIONS(913), 1,
      anon_sym_chan,
    ACTIONS(1234), 1,
      sym_identifier,
    ACTIONS(1236), 1,
      anon_sym_LPAREN,
    ACTIONS(1238), 1,
      anon_sym_LBRACK,
    ACTIONS(1240), 1,
      anon_sym_STAR,
    ACTIONS(1242), 1,
      anon_sym_TILDE,
    ACTIONS(1246), 1,
      anon_sym_LT_DASH,
    ACTIONS(1260), 1,
      anon_sym_RBRACE,
    STATE(806), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(961), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(1133), 2,
      sym_method_elem,
      sym_type_elem,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [40785] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(708), 1,
      anon_sym_SEMI,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1096), 1,
      anon_sym_COMMA,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1262), 1,
      anon_sym_DOT,
    STATE(398), 1,
      sym_argument_list,
    STATE(935), 1,
      aux_sym_expression_list_repeat1,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [40848] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(587), 1,
      anon_sym_COMMA,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(597), 1,
      aux_sym_var_spec_repeat1,
    STATE(1251), 1,
      sym_type_elem,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1125), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [40913] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(1202), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(656), 3,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1194), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
    ACTIONS(654), 12,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [40962] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(1198), 1,
      anon_sym_PIPE,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(656), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1202), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1200), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1194), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
    ACTIONS(654), 9,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_COLON_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [41015] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(1198), 1,
      anon_sym_PIPE,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(1202), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1206), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1200), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1204), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(654), 5,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_COLON_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
    ACTIONS(1194), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [41070] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(1198), 1,
      anon_sym_PIPE,
    ACTIONS(1208), 1,
      anon_sym_AMP_AMP,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(1202), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1206), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1200), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(654), 4,
      anon_sym_COMMA,
      anon_sym_LBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PIPE_PIPE,
    ACTIONS(1204), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1194), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [41127] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(943), 1,
      anon_sym_DOT,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(1164), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(656), 3,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1158), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
    ACTIONS(654), 12,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [41176] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(899), 1,
      anon_sym_func,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(911), 1,
      anon_sym_map,
    ACTIONS(913), 1,
      anon_sym_chan,
    ACTIONS(1234), 1,
      sym_identifier,
    ACTIONS(1236), 1,
      anon_sym_LPAREN,
    ACTIONS(1238), 1,
      anon_sym_LBRACK,
    ACTIONS(1240), 1,
      anon_sym_STAR,
    ACTIONS(1242), 1,
      anon_sym_TILDE,
    ACTIONS(1246), 1,
      anon_sym_LT_DASH,
    ACTIONS(1264), 1,
      anon_sym_RBRACE,
    STATE(806), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(961), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(1040), 2,
      sym_method_elem,
      sym_type_elem,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [41239] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(899), 1,
      anon_sym_func,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(911), 1,
      anon_sym_map,
    ACTIONS(913), 1,
      anon_sym_chan,
    ACTIONS(1234), 1,
      sym_identifier,
    ACTIONS(1236), 1,
      anon_sym_LPAREN,
    ACTIONS(1238), 1,
      anon_sym_LBRACK,
    ACTIONS(1240), 1,
      anon_sym_STAR,
    ACTIONS(1242), 1,
      anon_sym_TILDE,
    ACTIONS(1246), 1,
      anon_sym_LT_DASH,
    ACTIONS(1266), 1,
      anon_sym_RBRACE,
    STATE(806), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(961), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(1035), 2,
      sym_method_elem,
      sym_type_elem,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [41302] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(899), 1,
      anon_sym_func,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(911), 1,
      anon_sym_map,
    ACTIONS(913), 1,
      anon_sym_chan,
    ACTIONS(1234), 1,
      sym_identifier,
    ACTIONS(1236), 1,
      anon_sym_LPAREN,
    ACTIONS(1238), 1,
      anon_sym_LBRACK,
    ACTIONS(1240), 1,
      anon_sym_STAR,
    ACTIONS(1242), 1,
      anon_sym_TILDE,
    ACTIONS(1246), 1,
      anon_sym_LT_DASH,
    ACTIONS(1268), 1,
      anon_sym_RBRACE,
    STATE(806), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(961), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(1133), 2,
      sym_method_elem,
      sym_type_elem,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [41365] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(600), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(595), 20,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [41400] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(899), 1,
      anon_sym_func,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(911), 1,
      anon_sym_map,
    ACTIONS(913), 1,
      anon_sym_chan,
    ACTIONS(1234), 1,
      sym_identifier,
    ACTIONS(1236), 1,
      anon_sym_LPAREN,
    ACTIONS(1238), 1,
      anon_sym_LBRACK,
    ACTIONS(1240), 1,
      anon_sym_STAR,
    ACTIONS(1242), 1,
      anon_sym_TILDE,
    ACTIONS(1246), 1,
      anon_sym_LT_DASH,
    ACTIONS(1270), 1,
      anon_sym_RBRACE,
    STATE(806), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(961), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(1133), 2,
      sym_method_elem,
      sym_type_elem,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [41463] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(580), 1,
      anon_sym_DOT,
    ACTIONS(852), 1,
      anon_sym_LPAREN,
    ACTIONS(855), 1,
      anon_sym_LBRACK,
    STATE(414), 1,
      sym_literal_value,
    STATE(875), 1,
      sym_type_arguments,
    ACTIONS(600), 5,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(595), 17,
      anon_sym_COMMA,
      anon_sym_STAR,
      anon_sym_LBRACE,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [41508] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(587), 1,
      anon_sym_COMMA,
    ACTIONS(887), 1,
      sym_identifier,
    ACTIONS(899), 1,
      anon_sym_func,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(911), 1,
      anon_sym_map,
    ACTIONS(913), 1,
      anon_sym_chan,
    ACTIONS(1236), 1,
      anon_sym_LPAREN,
    ACTIONS(1238), 1,
      anon_sym_LBRACK,
    ACTIONS(1240), 1,
      anon_sym_STAR,
    ACTIONS(1242), 1,
      anon_sym_TILDE,
    ACTIONS(1246), 1,
      anon_sym_LT_DASH,
    ACTIONS(1272), 1,
      anon_sym_EQ,
    STATE(573), 1,
      aux_sym_var_spec_repeat1,
    STATE(806), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(912), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [41573] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(666), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(664), 20,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [41608] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(732), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(730), 20,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [41643] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(728), 7,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_COLON,
      anon_sym_AMP,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(726), 20,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_LT_DASH,
      anon_sym_COLON_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
  [41678] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(587), 1,
      anon_sym_COMMA,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(800), 1,
      aux_sym_var_spec_repeat1,
    STATE(1266), 1,
      sym_type_elem,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1125), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [41743] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(943), 1,
      anon_sym_DOT,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(1160), 1,
      anon_sym_PIPE,
    ACTIONS(1170), 1,
      anon_sym_AMP_AMP,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(1164), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1168), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1162), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(654), 4,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_PIPE_PIPE,
    ACTIONS(1166), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1158), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [41800] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(1009), 1,
      anon_sym_LPAREN,
    ACTIONS(1015), 1,
      sym_identifier,
    ACTIONS(1017), 1,
      anon_sym_func,
    ACTIONS(1019), 1,
      anon_sym_LBRACK,
    ACTIONS(1021), 1,
      anon_sym_STAR,
    ACTIONS(1023), 1,
      anon_sym_LT_DASH,
    ACTIONS(1274), 1,
      anon_sym_LBRACE,
    STATE(364), 1,
      sym_block,
    STATE(850), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1073), 2,
      sym_parameter_list,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [41862] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym_LBRACE,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(1198), 1,
      anon_sym_PIPE,
    ACTIONS(1208), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1210), 1,
      anon_sym_PIPE_PIPE,
    STATE(398), 1,
      sym_argument_list,
    STATE(917), 1,
      sym_block,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(1202), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1206), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1200), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1204), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1194), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [41922] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(33), 1,
      anon_sym_LBRACE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(1009), 1,
      anon_sym_LPAREN,
    ACTIONS(1015), 1,
      sym_identifier,
    ACTIONS(1017), 1,
      anon_sym_func,
    ACTIONS(1019), 1,
      anon_sym_LBRACK,
    ACTIONS(1021), 1,
      anon_sym_STAR,
    ACTIONS(1023), 1,
      anon_sym_LT_DASH,
    STATE(277), 1,
      sym_block,
    STATE(850), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1114), 2,
      sym_parameter_list,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [41984] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(1160), 1,
      anon_sym_PIPE,
    ACTIONS(1170), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1172), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1276), 1,
      anon_sym_RPAREN,
    ACTIONS(1278), 1,
      anon_sym_COMMA,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(1164), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1168), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1162), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1166), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1158), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [42044] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(1198), 1,
      anon_sym_PIPE,
    ACTIONS(1208), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1210), 1,
      anon_sym_PIPE_PIPE,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(867), 2,
      anon_sym_COMMA,
      anon_sym_LBRACE,
    ACTIONS(1202), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1206), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1200), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1204), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1194), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [42102] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1280), 1,
      anon_sym_RBRACK,
    ACTIONS(1282), 1,
      anon_sym_COLON,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [42162] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1284), 1,
      anon_sym_RBRACK,
    ACTIONS(1286), 1,
      anon_sym_COLON,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [42222] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(1160), 1,
      anon_sym_PIPE,
    ACTIONS(1170), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1172), 1,
      anon_sym_PIPE_PIPE,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(867), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(1164), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1168), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1162), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1166), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1158), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [42280] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(1009), 1,
      anon_sym_LPAREN,
    ACTIONS(1015), 1,
      sym_identifier,
    ACTIONS(1017), 1,
      anon_sym_func,
    ACTIONS(1019), 1,
      anon_sym_LBRACK,
    ACTIONS(1021), 1,
      anon_sym_STAR,
    ACTIONS(1023), 1,
      anon_sym_LT_DASH,
    ACTIONS(1288), 1,
      anon_sym_LBRACE,
    STATE(464), 1,
      sym_block,
    STATE(850), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1141), 2,
      sym_parameter_list,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [42342] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(1160), 1,
      anon_sym_PIPE,
    ACTIONS(1170), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1172), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1290), 1,
      anon_sym_RPAREN,
    ACTIONS(1292), 1,
      anon_sym_COMMA,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(1164), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1168), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1162), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1166), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1158), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [42402] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(587), 1,
      anon_sym_COMMA,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(800), 1,
      aux_sym_var_spec_repeat1,
    STATE(1297), 1,
      sym_parenthesized_type,
    STATE(1300), 1,
      sym__simple_type,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [42466] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1294), 1,
      anon_sym_RBRACK,
    ACTIONS(1296), 1,
      anon_sym_COLON,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [42526] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1078), 2,
      anon_sym_SEMI,
      anon_sym_COLON,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [42584] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(33), 1,
      anon_sym_LBRACE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(1009), 1,
      anon_sym_LPAREN,
    ACTIONS(1017), 1,
      anon_sym_func,
    ACTIONS(1019), 1,
      anon_sym_LBRACK,
    ACTIONS(1021), 1,
      anon_sym_STAR,
    ACTIONS(1023), 1,
      anon_sym_LT_DASH,
    ACTIONS(1298), 1,
      sym_identifier,
    STATE(277), 1,
      sym_block,
    STATE(850), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1114), 2,
      sym_parameter_list,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [42646] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1300), 1,
      anon_sym_RBRACK,
    STATE(1261), 1,
      sym_type_elem,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1125), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [42708] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1302), 1,
      anon_sym_RBRACK,
    STATE(1261), 1,
      sym_type_elem,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1125), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [42770] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1304), 1,
      anon_sym_RBRACK,
    ACTIONS(1306), 1,
      anon_sym_COLON,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [42830] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1308), 1,
      anon_sym_RBRACK,
    STATE(1261), 1,
      sym_type_elem,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1125), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [42892] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1310), 1,
      anon_sym_RBRACK,
    ACTIONS(1312), 1,
      anon_sym_COLON,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [42952] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1314), 1,
      anon_sym_RBRACK,
    STATE(1261), 1,
      sym_type_elem,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1125), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [43014] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(1160), 1,
      anon_sym_PIPE,
    ACTIONS(1170), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1172), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1316), 1,
      anon_sym_RPAREN,
    ACTIONS(1318), 1,
      anon_sym_COMMA,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(1164), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1168), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1162), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1166), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1158), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [43074] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1320), 1,
      anon_sym_RBRACK,
    ACTIONS(1322), 1,
      anon_sym_COLON,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [43134] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(1009), 1,
      anon_sym_LPAREN,
    ACTIONS(1015), 1,
      sym_identifier,
    ACTIONS(1017), 1,
      anon_sym_func,
    ACTIONS(1019), 1,
      anon_sym_LBRACK,
    ACTIONS(1021), 1,
      anon_sym_STAR,
    ACTIONS(1023), 1,
      anon_sym_LT_DASH,
    ACTIONS(1324), 1,
      anon_sym_LBRACE,
    STATE(331), 1,
      sym_block,
    STATE(850), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1129), 2,
      sym_parameter_list,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [43196] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(1160), 1,
      anon_sym_PIPE,
    ACTIONS(1170), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1172), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1326), 1,
      anon_sym_RPAREN,
    ACTIONS(1328), 1,
      anon_sym_COMMA,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(1164), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1168), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1162), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1166), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1158), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [43256] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(899), 1,
      anon_sym_func,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(911), 1,
      anon_sym_map,
    ACTIONS(913), 1,
      anon_sym_chan,
    ACTIONS(1234), 1,
      sym_identifier,
    ACTIONS(1236), 1,
      anon_sym_LPAREN,
    ACTIONS(1238), 1,
      anon_sym_LBRACK,
    ACTIONS(1240), 1,
      anon_sym_STAR,
    ACTIONS(1242), 1,
      anon_sym_TILDE,
    ACTIONS(1246), 1,
      anon_sym_LT_DASH,
    STATE(806), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(961), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(1133), 2,
      sym_method_elem,
      sym_type_elem,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [43316] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1330), 1,
      anon_sym_RBRACK,
    ACTIONS(1332), 1,
      anon_sym_COLON,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [43376] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1334), 1,
      anon_sym_RBRACK,
    ACTIONS(1336), 1,
      anon_sym_COLON,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [43436] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1338), 1,
      anon_sym_RBRACK,
    ACTIONS(1340), 1,
      anon_sym_COLON,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [43496] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1342), 1,
      anon_sym_RBRACK,
    ACTIONS(1344), 1,
      anon_sym_COLON,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [43556] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1346), 1,
      anon_sym_RBRACK,
    STATE(1261), 1,
      sym_type_elem,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1125), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [43618] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(1160), 1,
      anon_sym_PIPE,
    ACTIONS(1170), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1172), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1348), 1,
      anon_sym_RPAREN,
    ACTIONS(1350), 1,
      anon_sym_COMMA,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(1164), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1168), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1162), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1166), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1158), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [43678] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1352), 1,
      anon_sym_RBRACK,
    STATE(1261), 1,
      sym_type_elem,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1125), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [43740] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1354), 1,
      anon_sym_RBRACK,
    ACTIONS(1356), 1,
      anon_sym_COLON,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [43800] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1358), 1,
      anon_sym_RBRACK,
    ACTIONS(1360), 1,
      anon_sym_COLON,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [43860] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1362), 1,
      anon_sym_RBRACK,
    ACTIONS(1364), 1,
      anon_sym_COLON,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [43920] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(887), 1,
      sym_identifier,
    ACTIONS(899), 1,
      anon_sym_func,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(911), 1,
      anon_sym_map,
    ACTIONS(913), 1,
      anon_sym_chan,
    ACTIONS(1236), 1,
      anon_sym_LPAREN,
    ACTIONS(1238), 1,
      anon_sym_LBRACK,
    ACTIONS(1240), 1,
      anon_sym_STAR,
    ACTIONS(1242), 1,
      anon_sym_TILDE,
    ACTIONS(1246), 1,
      anon_sym_LT_DASH,
    ACTIONS(1366), 1,
      anon_sym_COMMA,
    STATE(801), 1,
      aux_sym_field_declaration_repeat1,
    STATE(898), 1,
      sym_parenthesized_type,
    STATE(902), 1,
      sym__simple_type,
    STATE(806), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [43984] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(1009), 1,
      anon_sym_LPAREN,
    ACTIONS(1015), 1,
      sym_identifier,
    ACTIONS(1017), 1,
      anon_sym_func,
    ACTIONS(1019), 1,
      anon_sym_LBRACK,
    ACTIONS(1021), 1,
      anon_sym_STAR,
    ACTIONS(1023), 1,
      anon_sym_LT_DASH,
    ACTIONS(1368), 1,
      anon_sym_LBRACE,
    STATE(535), 1,
      sym_block,
    STATE(850), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1096), 2,
      sym_parameter_list,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [44046] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(1160), 1,
      anon_sym_PIPE,
    ACTIONS(1170), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1172), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1370), 1,
      anon_sym_RPAREN,
    ACTIONS(1372), 1,
      anon_sym_COMMA,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(1164), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1168), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1162), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1166), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1158), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [44106] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1374), 1,
      anon_sym_RBRACK,
    ACTIONS(1376), 1,
      anon_sym_COLON,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [44166] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(887), 1,
      sym_identifier,
    ACTIONS(899), 1,
      anon_sym_func,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(911), 1,
      anon_sym_map,
    ACTIONS(913), 1,
      anon_sym_chan,
    ACTIONS(1236), 1,
      anon_sym_LPAREN,
    ACTIONS(1240), 1,
      anon_sym_STAR,
    ACTIONS(1242), 1,
      anon_sym_TILDE,
    ACTIONS(1246), 1,
      anon_sym_LT_DASH,
    ACTIONS(1378), 1,
      anon_sym_EQ,
    ACTIONS(1380), 1,
      anon_sym_LBRACK,
    STATE(759), 1,
      sym_type_parameter_list,
    STATE(806), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1022), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [44228] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1382), 1,
      anon_sym_RBRACK,
    ACTIONS(1384), 1,
      anon_sym_COLON,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [44288] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1386), 1,
      anon_sym_RBRACK,
    ACTIONS(1388), 1,
      anon_sym_COLON,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [44348] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1390), 1,
      anon_sym_RBRACK,
    ACTIONS(1392), 1,
      anon_sym_COLON,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [44408] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(33), 1,
      anon_sym_LBRACE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(1009), 1,
      anon_sym_LPAREN,
    ACTIONS(1017), 1,
      anon_sym_func,
    ACTIONS(1019), 1,
      anon_sym_LBRACK,
    ACTIONS(1021), 1,
      anon_sym_STAR,
    ACTIONS(1023), 1,
      anon_sym_LT_DASH,
    ACTIONS(1394), 1,
      sym_identifier,
    STATE(277), 1,
      sym_block,
    STATE(850), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1114), 2,
      sym_parameter_list,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [44470] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1396), 1,
      anon_sym_RBRACK,
    ACTIONS(1398), 1,
      anon_sym_COLON,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [44530] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1400), 1,
      anon_sym_RPAREN,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1194), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [44589] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1402), 1,
      anon_sym_RBRACK,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [44646] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1404), 1,
      anon_sym_type,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1393), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [44705] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(1160), 1,
      anon_sym_PIPE,
    ACTIONS(1170), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1172), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1406), 1,
      anon_sym_RPAREN,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(1164), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1168), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1162), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1166), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1158), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [44762] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(1078), 1,
      anon_sym_LBRACE,
    ACTIONS(1198), 1,
      anon_sym_PIPE,
    ACTIONS(1208), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1210), 1,
      anon_sym_PIPE_PIPE,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(1202), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1206), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1200), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1204), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1194), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [44819] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(1215), 1,
      sym_type_elem,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1125), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [44878] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1408), 1,
      anon_sym_RBRACK,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1195), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [44937] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1410), 1,
      anon_sym_type,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1393), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [44996] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1412), 1,
      anon_sym_type,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1393), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [45055] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(1160), 1,
      anon_sym_PIPE,
    ACTIONS(1170), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1172), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1414), 1,
      anon_sym_RPAREN,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(1164), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1168), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1162), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1166), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1158), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [45112] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1416), 1,
      anon_sym_RBRACK,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [45169] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1418), 1,
      anon_sym_RPAREN,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1164), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [45228] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1420), 1,
      anon_sym_SEMI,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [45285] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1422), 1,
      anon_sym_COLON,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [45342] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1424), 1,
      anon_sym_SEMI,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [45399] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(1198), 1,
      anon_sym_PIPE,
    ACTIONS(1208), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1210), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1426), 1,
      anon_sym_LBRACE,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(1202), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1206), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1200), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1204), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1194), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [45456] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1428), 1,
      anon_sym_RBRACK,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1195), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [45515] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(1261), 1,
      sym_type_elem,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1125), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [45574] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1430), 1,
      anon_sym_RBRACK,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1195), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [45633] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1432), 1,
      anon_sym_RBRACK,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [45690] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1434), 1,
      anon_sym_RBRACK,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [45747] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1436), 1,
      anon_sym_type,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1325), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [45806] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1438), 1,
      anon_sym_RBRACK,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [45863] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(1160), 1,
      anon_sym_PIPE,
    ACTIONS(1170), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1172), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1440), 1,
      anon_sym_RPAREN,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(1164), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1168), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1162), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1166), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1158), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [45920] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(1198), 1,
      anon_sym_PIPE,
    ACTIONS(1208), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1210), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1442), 1,
      anon_sym_LBRACE,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(1202), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1206), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1200), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1204), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1194), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [45977] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1444), 1,
      anon_sym_RBRACK,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [46034] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1446), 1,
      anon_sym_RPAREN,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1242), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [46093] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1448), 1,
      anon_sym_RBRACK,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1195), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [46152] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1450), 1,
      anon_sym_RBRACK,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1195), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [46211] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1452), 1,
      anon_sym_RBRACK,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [46268] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1454), 1,
      anon_sym_RBRACK,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [46325] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(1160), 1,
      anon_sym_PIPE,
    ACTIONS(1170), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1172), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1456), 1,
      anon_sym_RPAREN,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(1164), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1168), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1162), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1166), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1158), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [46382] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1458), 1,
      anon_sym_RBRACK,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [46439] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1460), 1,
      anon_sym_RPAREN,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1172), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [46498] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1462), 1,
      anon_sym_RBRACK,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1195), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [46557] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1464), 1,
      anon_sym_RBRACK,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1195), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [46616] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1466), 1,
      anon_sym_RBRACK,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1195), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [46675] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1468), 1,
      anon_sym_RBRACK,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [46732] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1470), 1,
      anon_sym_RBRACK,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [46789] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(1160), 1,
      anon_sym_PIPE,
    ACTIONS(1170), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1172), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1472), 1,
      anon_sym_RPAREN,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(1164), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1168), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1162), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1166), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1158), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [46846] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1474), 1,
      anon_sym_RBRACK,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [46903] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1476), 1,
      anon_sym_RBRACK,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [46960] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1478), 1,
      anon_sym_RBRACK,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1195), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [47019] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1480), 1,
      anon_sym_RBRACK,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1195), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [47078] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1482), 1,
      anon_sym_RBRACK,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [47135] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1484), 1,
      anon_sym_RBRACK,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [47192] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(1160), 1,
      anon_sym_PIPE,
    ACTIONS(1170), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1172), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1486), 1,
      anon_sym_RPAREN,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(1164), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1168), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1162), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1166), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(1158), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [47249] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1488), 1,
      anon_sym_RBRACK,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [47306] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1490), 1,
      anon_sym_RPAREN,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1211), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [47365] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1492), 1,
      anon_sym_RBRACK,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1195), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [47424] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1494), 1,
      anon_sym_RBRACK,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1195), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [47483] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1496), 1,
      anon_sym_RBRACK,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [47540] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(993), 1,
      anon_sym_DOT,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1498), 1,
      anon_sym_RBRACK,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [47597] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(1165), 1,
      sym_type_elem,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1125), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [47656] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(1243), 1,
      sym_type_elem,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1125), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [47715] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1500), 1,
      anon_sym_RPAREN,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1209), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [47774] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(857), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [47830] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1195), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [47886] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(864), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [47942] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(1015), 1,
      sym_identifier,
    ACTIONS(1017), 1,
      anon_sym_func,
    ACTIONS(1019), 1,
      anon_sym_LBRACK,
    ACTIONS(1021), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1502), 1,
      anon_sym_LT_DASH,
    STATE(850), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(874), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [47998] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1388), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [48054] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(873), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [48110] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(1015), 1,
      sym_identifier,
    ACTIONS(1017), 1,
      anon_sym_func,
    ACTIONS(1019), 1,
      anon_sym_LBRACK,
    ACTIONS(1021), 1,
      anon_sym_STAR,
    ACTIONS(1023), 1,
      anon_sym_LT_DASH,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(850), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(857), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [48166] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(929), 1,
      sym_identifier,
    ACTIONS(931), 1,
      anon_sym_func,
    ACTIONS(937), 1,
      anon_sym_map,
    ACTIONS(1236), 1,
      anon_sym_LPAREN,
    ACTIONS(1242), 1,
      anon_sym_TILDE,
    ACTIONS(1504), 1,
      anon_sym_LBRACK,
    ACTIONS(1506), 1,
      anon_sym_STAR,
    ACTIONS(1508), 1,
      anon_sym_chan,
    ACTIONS(1510), 1,
      anon_sym_LT_DASH,
    STATE(803), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(830), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [48222] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(1015), 1,
      sym_identifier,
    ACTIONS(1017), 1,
      anon_sym_func,
    ACTIONS(1019), 1,
      anon_sym_LBRACK,
    ACTIONS(1021), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1502), 1,
      anon_sym_LT_DASH,
    STATE(850), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(881), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [48278] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(929), 1,
      sym_identifier,
    ACTIONS(931), 1,
      anon_sym_func,
    ACTIONS(937), 1,
      anon_sym_map,
    ACTIONS(939), 1,
      anon_sym_chan,
    ACTIONS(1236), 1,
      anon_sym_LPAREN,
    ACTIONS(1242), 1,
      anon_sym_TILDE,
    ACTIONS(1504), 1,
      anon_sym_LBRACK,
    ACTIONS(1506), 1,
      anon_sym_STAR,
    ACTIONS(1512), 1,
      anon_sym_LT_DASH,
    STATE(803), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(812), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [48334] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1226), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [48390] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(887), 1,
      sym_identifier,
    ACTIONS(899), 1,
      anon_sym_func,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(911), 1,
      anon_sym_map,
    ACTIONS(913), 1,
      anon_sym_chan,
    ACTIONS(1236), 1,
      anon_sym_LPAREN,
    ACTIONS(1238), 1,
      anon_sym_LBRACK,
    ACTIONS(1240), 1,
      anon_sym_STAR,
    ACTIONS(1242), 1,
      anon_sym_TILDE,
    ACTIONS(1246), 1,
      anon_sym_LT_DASH,
    STATE(806), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(820), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [48446] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1062), 1,
      anon_sym_struct,
    ACTIONS(1068), 1,
      anon_sym_interface,
    ACTIONS(1128), 1,
      sym_identifier,
    ACTIONS(1130), 1,
      anon_sym_func,
    ACTIONS(1136), 1,
      anon_sym_map,
    ACTIONS(1138), 1,
      anon_sym_chan,
    ACTIONS(1514), 1,
      anon_sym_LPAREN,
    ACTIONS(1516), 1,
      anon_sym_LBRACK,
    ACTIONS(1518), 1,
      anon_sym_STAR,
    ACTIONS(1520), 1,
      anon_sym_TILDE,
    ACTIONS(1522), 1,
      anon_sym_LT_DASH,
    STATE(910), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(978), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(922), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [48502] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(929), 1,
      sym_identifier,
    ACTIONS(931), 1,
      anon_sym_func,
    ACTIONS(937), 1,
      anon_sym_map,
    ACTIONS(939), 1,
      anon_sym_chan,
    ACTIONS(1236), 1,
      anon_sym_LPAREN,
    ACTIONS(1242), 1,
      anon_sym_TILDE,
    ACTIONS(1504), 1,
      anon_sym_LBRACK,
    ACTIONS(1506), 1,
      anon_sym_STAR,
    ACTIONS(1510), 1,
      anon_sym_LT_DASH,
    STATE(803), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(820), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [48558] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(929), 1,
      sym_identifier,
    ACTIONS(931), 1,
      anon_sym_func,
    ACTIONS(937), 1,
      anon_sym_map,
    ACTIONS(939), 1,
      anon_sym_chan,
    ACTIONS(1236), 1,
      anon_sym_LPAREN,
    ACTIONS(1242), 1,
      anon_sym_TILDE,
    ACTIONS(1504), 1,
      anon_sym_LBRACK,
    ACTIONS(1506), 1,
      anon_sym_STAR,
    ACTIONS(1510), 1,
      anon_sym_LT_DASH,
    STATE(803), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(837), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [48614] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(1015), 1,
      sym_identifier,
    ACTIONS(1017), 1,
      anon_sym_func,
    ACTIONS(1019), 1,
      anon_sym_LBRACK,
    ACTIONS(1021), 1,
      anon_sym_STAR,
    ACTIONS(1023), 1,
      anon_sym_LT_DASH,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(850), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(869), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [48670] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(1015), 1,
      sym_identifier,
    ACTIONS(1017), 1,
      anon_sym_func,
    ACTIONS(1019), 1,
      anon_sym_LBRACK,
    ACTIONS(1021), 1,
      anon_sym_STAR,
    ACTIONS(1023), 1,
      anon_sym_LT_DASH,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(850), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(864), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [48726] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(887), 1,
      sym_identifier,
    ACTIONS(899), 1,
      anon_sym_func,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(911), 1,
      anon_sym_map,
    ACTIONS(913), 1,
      anon_sym_chan,
    ACTIONS(1236), 1,
      anon_sym_LPAREN,
    ACTIONS(1238), 1,
      anon_sym_LBRACK,
    ACTIONS(1240), 1,
      anon_sym_STAR,
    ACTIONS(1242), 1,
      anon_sym_TILDE,
    ACTIONS(1246), 1,
      anon_sym_LT_DASH,
    STATE(806), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(837), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [48782] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1233), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [48838] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(929), 1,
      sym_identifier,
    ACTIONS(931), 1,
      anon_sym_func,
    ACTIONS(937), 1,
      anon_sym_map,
    ACTIONS(939), 1,
      anon_sym_chan,
    ACTIONS(1236), 1,
      anon_sym_LPAREN,
    ACTIONS(1242), 1,
      anon_sym_TILDE,
    ACTIONS(1504), 1,
      anon_sym_LBRACK,
    ACTIONS(1506), 1,
      anon_sym_STAR,
    ACTIONS(1510), 1,
      anon_sym_LT_DASH,
    STATE(803), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(833), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [48894] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1062), 1,
      anon_sym_struct,
    ACTIONS(1068), 1,
      anon_sym_interface,
    ACTIONS(1128), 1,
      sym_identifier,
    ACTIONS(1130), 1,
      anon_sym_func,
    ACTIONS(1136), 1,
      anon_sym_map,
    ACTIONS(1138), 1,
      anon_sym_chan,
    ACTIONS(1514), 1,
      anon_sym_LPAREN,
    ACTIONS(1516), 1,
      anon_sym_LBRACK,
    ACTIONS(1518), 1,
      anon_sym_STAR,
    ACTIONS(1520), 1,
      anon_sym_TILDE,
    ACTIONS(1522), 1,
      anon_sym_LT_DASH,
    STATE(910), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(926), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(922), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [48950] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(887), 1,
      sym_identifier,
    ACTIONS(899), 1,
      anon_sym_func,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(911), 1,
      anon_sym_map,
    ACTIONS(913), 1,
      anon_sym_chan,
    ACTIONS(1236), 1,
      anon_sym_LPAREN,
    ACTIONS(1238), 1,
      anon_sym_LBRACK,
    ACTIONS(1240), 1,
      anon_sym_STAR,
    ACTIONS(1242), 1,
      anon_sym_TILDE,
    ACTIONS(1524), 1,
      anon_sym_LT_DASH,
    STATE(806), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(832), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [49006] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1526), 1,
      anon_sym_chan,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(880), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [49062] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(887), 1,
      sym_identifier,
    ACTIONS(899), 1,
      anon_sym_func,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(911), 1,
      anon_sym_map,
    ACTIONS(913), 1,
      anon_sym_chan,
    ACTIONS(1236), 1,
      anon_sym_LPAREN,
    ACTIONS(1238), 1,
      anon_sym_LBRACK,
    ACTIONS(1240), 1,
      anon_sym_STAR,
    ACTIONS(1242), 1,
      anon_sym_TILDE,
    ACTIONS(1246), 1,
      anon_sym_LT_DASH,
    STATE(806), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(834), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [49118] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1528), 1,
      anon_sym_LT_DASH,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(881), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [49174] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1365), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [49230] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(887), 1,
      sym_identifier,
    ACTIONS(899), 1,
      anon_sym_func,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(911), 1,
      anon_sym_map,
    ACTIONS(913), 1,
      anon_sym_chan,
    ACTIONS(1236), 1,
      anon_sym_LPAREN,
    ACTIONS(1238), 1,
      anon_sym_LBRACK,
    ACTIONS(1240), 1,
      anon_sym_STAR,
    ACTIONS(1242), 1,
      anon_sym_TILDE,
    ACTIONS(1246), 1,
      anon_sym_LT_DASH,
    STATE(806), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(823), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [49286] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(887), 1,
      sym_identifier,
    ACTIONS(899), 1,
      anon_sym_func,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(911), 1,
      anon_sym_map,
    ACTIONS(1236), 1,
      anon_sym_LPAREN,
    ACTIONS(1238), 1,
      anon_sym_LBRACK,
    ACTIONS(1240), 1,
      anon_sym_STAR,
    ACTIONS(1242), 1,
      anon_sym_TILDE,
    ACTIONS(1246), 1,
      anon_sym_LT_DASH,
    ACTIONS(1530), 1,
      anon_sym_chan,
    STATE(806), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(830), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [49342] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1374), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [49398] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1279), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [49454] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1232), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [49510] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1369), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [49566] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1050), 1,
      sym_identifier,
    ACTIONS(1056), 1,
      anon_sym_func,
    ACTIONS(1062), 1,
      anon_sym_struct,
    ACTIONS(1068), 1,
      anon_sym_interface,
    ACTIONS(1070), 1,
      anon_sym_map,
    ACTIONS(1514), 1,
      anon_sym_LPAREN,
    ACTIONS(1520), 1,
      anon_sym_TILDE,
    ACTIONS(1532), 1,
      anon_sym_LBRACK,
    ACTIONS(1534), 1,
      anon_sym_STAR,
    ACTIONS(1536), 1,
      anon_sym_chan,
    ACTIONS(1538), 1,
      anon_sym_LT_DASH,
    STATE(918), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(946), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(922), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [49622] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1050), 1,
      sym_identifier,
    ACTIONS(1056), 1,
      anon_sym_func,
    ACTIONS(1062), 1,
      anon_sym_struct,
    ACTIONS(1068), 1,
      anon_sym_interface,
    ACTIONS(1070), 1,
      anon_sym_map,
    ACTIONS(1072), 1,
      anon_sym_chan,
    ACTIONS(1514), 1,
      anon_sym_LPAREN,
    ACTIONS(1520), 1,
      anon_sym_TILDE,
    ACTIONS(1532), 1,
      anon_sym_LBRACK,
    ACTIONS(1534), 1,
      anon_sym_STAR,
    ACTIONS(1540), 1,
      anon_sym_LT_DASH,
    STATE(918), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(948), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(922), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [49678] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(1015), 1,
      sym_identifier,
    ACTIONS(1017), 1,
      anon_sym_func,
    ACTIONS(1019), 1,
      anon_sym_LBRACK,
    ACTIONS(1021), 1,
      anon_sym_STAR,
    ACTIONS(1023), 1,
      anon_sym_LT_DASH,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(850), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(873), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [49734] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1050), 1,
      sym_identifier,
    ACTIONS(1056), 1,
      anon_sym_func,
    ACTIONS(1062), 1,
      anon_sym_struct,
    ACTIONS(1068), 1,
      anon_sym_interface,
    ACTIONS(1070), 1,
      anon_sym_map,
    ACTIONS(1072), 1,
      anon_sym_chan,
    ACTIONS(1514), 1,
      anon_sym_LPAREN,
    ACTIONS(1520), 1,
      anon_sym_TILDE,
    ACTIONS(1532), 1,
      anon_sym_LBRACK,
    ACTIONS(1534), 1,
      anon_sym_STAR,
    ACTIONS(1538), 1,
      anon_sym_LT_DASH,
    STATE(918), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(965), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(922), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [49790] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(929), 1,
      sym_identifier,
    ACTIONS(931), 1,
      anon_sym_func,
    ACTIONS(937), 1,
      anon_sym_map,
    ACTIONS(939), 1,
      anon_sym_chan,
    ACTIONS(1236), 1,
      anon_sym_LPAREN,
    ACTIONS(1242), 1,
      anon_sym_TILDE,
    ACTIONS(1504), 1,
      anon_sym_LBRACK,
    ACTIONS(1506), 1,
      anon_sym_STAR,
    ACTIONS(1512), 1,
      anon_sym_LT_DASH,
    STATE(803), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(832), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [49846] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    ACTIONS(1528), 1,
      anon_sym_LT_DASH,
    STATE(874), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [49902] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(929), 1,
      sym_identifier,
    ACTIONS(931), 1,
      anon_sym_func,
    ACTIONS(937), 1,
      anon_sym_map,
    ACTIONS(939), 1,
      anon_sym_chan,
    ACTIONS(1236), 1,
      anon_sym_LPAREN,
    ACTIONS(1242), 1,
      anon_sym_TILDE,
    ACTIONS(1504), 1,
      anon_sym_LBRACK,
    ACTIONS(1506), 1,
      anon_sym_STAR,
    ACTIONS(1510), 1,
      anon_sym_LT_DASH,
    STATE(803), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(834), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [49958] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(929), 1,
      sym_identifier,
    ACTIONS(931), 1,
      anon_sym_func,
    ACTIONS(937), 1,
      anon_sym_map,
    ACTIONS(939), 1,
      anon_sym_chan,
    ACTIONS(1236), 1,
      anon_sym_LPAREN,
    ACTIONS(1242), 1,
      anon_sym_TILDE,
    ACTIONS(1504), 1,
      anon_sym_LBRACK,
    ACTIONS(1506), 1,
      anon_sym_STAR,
    ACTIONS(1510), 1,
      anon_sym_LT_DASH,
    STATE(803), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(823), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [50014] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1062), 1,
      anon_sym_struct,
    ACTIONS(1068), 1,
      anon_sym_interface,
    ACTIONS(1128), 1,
      sym_identifier,
    ACTIONS(1130), 1,
      anon_sym_func,
    ACTIONS(1136), 1,
      anon_sym_map,
    ACTIONS(1138), 1,
      anon_sym_chan,
    ACTIONS(1514), 1,
      anon_sym_LPAREN,
    ACTIONS(1516), 1,
      anon_sym_LBRACK,
    ACTIONS(1518), 1,
      anon_sym_STAR,
    ACTIONS(1520), 1,
      anon_sym_TILDE,
    ACTIONS(1522), 1,
      anon_sym_LT_DASH,
    STATE(910), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(965), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(922), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [50070] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1050), 1,
      sym_identifier,
    ACTIONS(1056), 1,
      anon_sym_func,
    ACTIONS(1062), 1,
      anon_sym_struct,
    ACTIONS(1068), 1,
      anon_sym_interface,
    ACTIONS(1070), 1,
      anon_sym_map,
    ACTIONS(1072), 1,
      anon_sym_chan,
    ACTIONS(1514), 1,
      anon_sym_LPAREN,
    ACTIONS(1520), 1,
      anon_sym_TILDE,
    ACTIONS(1532), 1,
      anon_sym_LBRACK,
    ACTIONS(1534), 1,
      anon_sym_STAR,
    ACTIONS(1538), 1,
      anon_sym_LT_DASH,
    STATE(918), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(967), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(922), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [50126] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1050), 1,
      sym_identifier,
    ACTIONS(1056), 1,
      anon_sym_func,
    ACTIONS(1062), 1,
      anon_sym_struct,
    ACTIONS(1068), 1,
      anon_sym_interface,
    ACTIONS(1070), 1,
      anon_sym_map,
    ACTIONS(1072), 1,
      anon_sym_chan,
    ACTIONS(1514), 1,
      anon_sym_LPAREN,
    ACTIONS(1520), 1,
      anon_sym_TILDE,
    ACTIONS(1532), 1,
      anon_sym_LBRACK,
    ACTIONS(1534), 1,
      anon_sym_STAR,
    ACTIONS(1538), 1,
      anon_sym_LT_DASH,
    STATE(918), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(978), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(922), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [50182] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1292), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [50238] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1050), 1,
      sym_identifier,
    ACTIONS(1056), 1,
      anon_sym_func,
    ACTIONS(1062), 1,
      anon_sym_struct,
    ACTIONS(1068), 1,
      anon_sym_interface,
    ACTIONS(1070), 1,
      anon_sym_map,
    ACTIONS(1072), 1,
      anon_sym_chan,
    ACTIONS(1514), 1,
      anon_sym_LPAREN,
    ACTIONS(1520), 1,
      anon_sym_TILDE,
    ACTIONS(1532), 1,
      anon_sym_LBRACK,
    ACTIONS(1534), 1,
      anon_sym_STAR,
    ACTIONS(1540), 1,
      anon_sym_LT_DASH,
    STATE(918), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(928), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(922), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [50294] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1050), 1,
      sym_identifier,
    ACTIONS(1056), 1,
      anon_sym_func,
    ACTIONS(1062), 1,
      anon_sym_struct,
    ACTIONS(1068), 1,
      anon_sym_interface,
    ACTIONS(1070), 1,
      anon_sym_map,
    ACTIONS(1072), 1,
      anon_sym_chan,
    ACTIONS(1514), 1,
      anon_sym_LPAREN,
    ACTIONS(1520), 1,
      anon_sym_TILDE,
    ACTIONS(1532), 1,
      anon_sym_LBRACK,
    ACTIONS(1534), 1,
      anon_sym_STAR,
    ACTIONS(1538), 1,
      anon_sym_LT_DASH,
    STATE(918), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(939), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(922), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [50350] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(861), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [50406] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1050), 1,
      sym_identifier,
    ACTIONS(1056), 1,
      anon_sym_func,
    ACTIONS(1062), 1,
      anon_sym_struct,
    ACTIONS(1068), 1,
      anon_sym_interface,
    ACTIONS(1070), 1,
      anon_sym_map,
    ACTIONS(1072), 1,
      anon_sym_chan,
    ACTIONS(1514), 1,
      anon_sym_LPAREN,
    ACTIONS(1520), 1,
      anon_sym_TILDE,
    ACTIONS(1532), 1,
      anon_sym_LBRACK,
    ACTIONS(1534), 1,
      anon_sym_STAR,
    ACTIONS(1538), 1,
      anon_sym_LT_DASH,
    STATE(918), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(944), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(922), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [50462] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1332), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [50518] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1356), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [50574] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(576), 1,
      anon_sym_chan,
    ACTIONS(1015), 1,
      sym_identifier,
    ACTIONS(1017), 1,
      anon_sym_func,
    ACTIONS(1019), 1,
      anon_sym_LBRACK,
    ACTIONS(1021), 1,
      anon_sym_STAR,
    ACTIONS(1023), 1,
      anon_sym_LT_DASH,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(850), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(880), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [50630] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(1015), 1,
      sym_identifier,
    ACTIONS(1017), 1,
      anon_sym_func,
    ACTIONS(1019), 1,
      anon_sym_LBRACK,
    ACTIONS(1021), 1,
      anon_sym_STAR,
    ACTIONS(1023), 1,
      anon_sym_LT_DASH,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(850), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(861), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [50686] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(37), 1,
      anon_sym_map,
    ACTIONS(39), 1,
      anon_sym_chan,
    ACTIONS(1015), 1,
      sym_identifier,
    ACTIONS(1017), 1,
      anon_sym_func,
    ACTIONS(1019), 1,
      anon_sym_LBRACK,
    ACTIONS(1021), 1,
      anon_sym_STAR,
    ACTIONS(1023), 1,
      anon_sym_LT_DASH,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(850), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(858), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [50742] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(869), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [50798] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1062), 1,
      anon_sym_struct,
    ACTIONS(1068), 1,
      anon_sym_interface,
    ACTIONS(1128), 1,
      sym_identifier,
    ACTIONS(1130), 1,
      anon_sym_func,
    ACTIONS(1136), 1,
      anon_sym_map,
    ACTIONS(1138), 1,
      anon_sym_chan,
    ACTIONS(1514), 1,
      anon_sym_LPAREN,
    ACTIONS(1516), 1,
      anon_sym_LBRACK,
    ACTIONS(1518), 1,
      anon_sym_STAR,
    ACTIONS(1520), 1,
      anon_sym_TILDE,
    ACTIONS(1542), 1,
      anon_sym_LT_DASH,
    STATE(910), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(928), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(922), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [50854] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1062), 1,
      anon_sym_struct,
    ACTIONS(1068), 1,
      anon_sym_interface,
    ACTIONS(1128), 1,
      sym_identifier,
    ACTIONS(1130), 1,
      anon_sym_func,
    ACTIONS(1136), 1,
      anon_sym_map,
    ACTIONS(1138), 1,
      anon_sym_chan,
    ACTIONS(1514), 1,
      anon_sym_LPAREN,
    ACTIONS(1516), 1,
      anon_sym_LBRACK,
    ACTIONS(1518), 1,
      anon_sym_STAR,
    ACTIONS(1520), 1,
      anon_sym_TILDE,
    ACTIONS(1522), 1,
      anon_sym_LT_DASH,
    STATE(910), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(939), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(922), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [50910] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1062), 1,
      anon_sym_struct,
    ACTIONS(1068), 1,
      anon_sym_interface,
    ACTIONS(1128), 1,
      sym_identifier,
    ACTIONS(1130), 1,
      anon_sym_func,
    ACTIONS(1136), 1,
      anon_sym_map,
    ACTIONS(1138), 1,
      anon_sym_chan,
    ACTIONS(1514), 1,
      anon_sym_LPAREN,
    ACTIONS(1516), 1,
      anon_sym_LBRACK,
    ACTIONS(1518), 1,
      anon_sym_STAR,
    ACTIONS(1520), 1,
      anon_sym_TILDE,
    ACTIONS(1522), 1,
      anon_sym_LT_DASH,
    STATE(910), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(944), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(922), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [50966] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1375), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [51022] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(887), 1,
      sym_identifier,
    ACTIONS(899), 1,
      anon_sym_func,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(911), 1,
      anon_sym_map,
    ACTIONS(913), 1,
      anon_sym_chan,
    ACTIONS(1236), 1,
      anon_sym_LPAREN,
    ACTIONS(1238), 1,
      anon_sym_LBRACK,
    ACTIONS(1240), 1,
      anon_sym_STAR,
    ACTIONS(1242), 1,
      anon_sym_TILDE,
    ACTIONS(1246), 1,
      anon_sym_LT_DASH,
    STATE(806), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(934), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [51078] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(887), 1,
      sym_identifier,
    ACTIONS(899), 1,
      anon_sym_func,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(911), 1,
      anon_sym_map,
    ACTIONS(913), 1,
      anon_sym_chan,
    ACTIONS(1236), 1,
      anon_sym_LPAREN,
    ACTIONS(1238), 1,
      anon_sym_LBRACK,
    ACTIONS(1240), 1,
      anon_sym_STAR,
    ACTIONS(1242), 1,
      anon_sym_TILDE,
    ACTIONS(1246), 1,
      anon_sym_LT_DASH,
    STATE(806), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(959), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [51134] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1062), 1,
      anon_sym_struct,
    ACTIONS(1068), 1,
      anon_sym_interface,
    ACTIONS(1128), 1,
      sym_identifier,
    ACTIONS(1130), 1,
      anon_sym_func,
    ACTIONS(1136), 1,
      anon_sym_map,
    ACTIONS(1514), 1,
      anon_sym_LPAREN,
    ACTIONS(1516), 1,
      anon_sym_LBRACK,
    ACTIONS(1518), 1,
      anon_sym_STAR,
    ACTIONS(1520), 1,
      anon_sym_TILDE,
    ACTIONS(1522), 1,
      anon_sym_LT_DASH,
    ACTIONS(1544), 1,
      anon_sym_chan,
    STATE(910), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(946), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(922), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [51190] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(887), 1,
      sym_identifier,
    ACTIONS(899), 1,
      anon_sym_func,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(911), 1,
      anon_sym_map,
    ACTIONS(913), 1,
      anon_sym_chan,
    ACTIONS(1236), 1,
      anon_sym_LPAREN,
    ACTIONS(1238), 1,
      anon_sym_LBRACK,
    ACTIONS(1240), 1,
      anon_sym_STAR,
    ACTIONS(1242), 1,
      anon_sym_TILDE,
    ACTIONS(1524), 1,
      anon_sym_LT_DASH,
    STATE(806), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(812), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [51246] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1062), 1,
      anon_sym_struct,
    ACTIONS(1068), 1,
      anon_sym_interface,
    ACTIONS(1128), 1,
      sym_identifier,
    ACTIONS(1130), 1,
      anon_sym_func,
    ACTIONS(1136), 1,
      anon_sym_map,
    ACTIONS(1138), 1,
      anon_sym_chan,
    ACTIONS(1514), 1,
      anon_sym_LPAREN,
    ACTIONS(1516), 1,
      anon_sym_LBRACK,
    ACTIONS(1518), 1,
      anon_sym_STAR,
    ACTIONS(1520), 1,
      anon_sym_TILDE,
    ACTIONS(1542), 1,
      anon_sym_LT_DASH,
    STATE(910), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(948), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(922), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [51302] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1062), 1,
      anon_sym_struct,
    ACTIONS(1068), 1,
      anon_sym_interface,
    ACTIONS(1128), 1,
      sym_identifier,
    ACTIONS(1130), 1,
      anon_sym_func,
    ACTIONS(1136), 1,
      anon_sym_map,
    ACTIONS(1138), 1,
      anon_sym_chan,
    ACTIONS(1514), 1,
      anon_sym_LPAREN,
    ACTIONS(1516), 1,
      anon_sym_LBRACK,
    ACTIONS(1518), 1,
      anon_sym_STAR,
    ACTIONS(1520), 1,
      anon_sym_TILDE,
    ACTIONS(1522), 1,
      anon_sym_LT_DASH,
    STATE(910), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(967), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(922), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [51358] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    ACTIONS(947), 1,
      anon_sym_LBRACK,
    ACTIONS(999), 1,
      anon_sym_PIPE,
    ACTIONS(1007), 1,
      anon_sym_AMP_AMP,
    ACTIONS(1150), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(1546), 1,
      anon_sym_DOT,
    STATE(398), 1,
      sym_argument_list,
    STATE(1288), 1,
      sym_type_arguments,
    ACTIONS(997), 2,
      anon_sym_AMP,
      anon_sym_SLASH,
    ACTIONS(1005), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1001), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
    ACTIONS(1003), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_EQ,
      anon_sym_GT_EQ,
    ACTIONS(995), 5,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_AMP_CARET,
  [51412] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1334), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [51468] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1168), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [51524] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1393), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [51580] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1391), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [51636] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(887), 1,
      sym_identifier,
    ACTIONS(899), 1,
      anon_sym_func,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(911), 1,
      anon_sym_map,
    ACTIONS(913), 1,
      anon_sym_chan,
    ACTIONS(1236), 1,
      anon_sym_LPAREN,
    ACTIONS(1238), 1,
      anon_sym_LBRACK,
    ACTIONS(1240), 1,
      anon_sym_STAR,
    ACTIONS(1242), 1,
      anon_sym_TILDE,
    ACTIONS(1246), 1,
      anon_sym_LT_DASH,
    STATE(806), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1048), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [51692] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1246), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [51748] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1360), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [51804] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1344), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [51860] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1175), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [51916] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1325), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [51972] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1415), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [52028] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1197), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [52084] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1341), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [52140] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(929), 1,
      sym_identifier,
    ACTIONS(931), 1,
      anon_sym_func,
    ACTIONS(937), 1,
      anon_sym_map,
    ACTIONS(939), 1,
      anon_sym_chan,
    ACTIONS(1236), 1,
      anon_sym_LPAREN,
    ACTIONS(1242), 1,
      anon_sym_TILDE,
    ACTIONS(1504), 1,
      anon_sym_LBRACK,
    ACTIONS(1506), 1,
      anon_sym_STAR,
    ACTIONS(1510), 1,
      anon_sym_LT_DASH,
    STATE(803), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(828), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [52196] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1405), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [52252] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(29), 1,
      anon_sym_struct,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(35), 1,
      anon_sym_interface,
    ACTIONS(578), 1,
      sym_identifier,
    ACTIONS(589), 1,
      anon_sym_func,
    ACTIONS(602), 1,
      anon_sym_map,
    ACTIONS(604), 1,
      anon_sym_chan,
    ACTIONS(606), 1,
      anon_sym_LT_DASH,
    ACTIONS(1011), 1,
      anon_sym_LBRACK,
    ACTIONS(1013), 1,
      anon_sym_STAR,
    ACTIONS(1037), 1,
      anon_sym_LPAREN,
    STATE(877), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(1214), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(867), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [52308] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(887), 1,
      sym_identifier,
    ACTIONS(899), 1,
      anon_sym_func,
    ACTIONS(905), 1,
      anon_sym_struct,
    ACTIONS(909), 1,
      anon_sym_interface,
    ACTIONS(911), 1,
      anon_sym_map,
    ACTIONS(913), 1,
      anon_sym_chan,
    ACTIONS(1236), 1,
      anon_sym_LPAREN,
    ACTIONS(1238), 1,
      anon_sym_LBRACK,
    ACTIONS(1240), 1,
      anon_sym_STAR,
    ACTIONS(1242), 1,
      anon_sym_TILDE,
    ACTIONS(1246), 1,
      anon_sym_LT_DASH,
    STATE(806), 2,
      sym_negated_type,
      sym_qualified_type,
    STATE(833), 2,
      sym_parenthesized_type,
      sym__simple_type,
    STATE(816), 9,
      sym_generic_type,
      sym_pointer_type,
      sym_array_type,
      sym_slice_type,
      sym_struct_type,
      sym_interface_type,
      sym_map_type,
      sym_channel_type,
      sym_function_type,
  [52364] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1550), 1,
      aux_sym_source_file_token1,
    ACTIONS(1548), 21,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LPAREN,
      anon_sym_EQ,
      anon_sym_func,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_struct,
      anon_sym_TILDE,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_interface,
      anon_sym_PIPE,
      anon_sym_map,
      anon_sym_chan,
      anon_sym_LT_DASH,
      anon_sym_case,
      anon_sym_default,
      sym_identifier,
      anon_sym_BQUOTE,
      anon_sym_DQUOTE,
  [52394] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1554), 1,
      aux_sym_source_file_token1,
    ACTIONS(1552), 21,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LPAREN,
      anon_sym_EQ,
      anon_sym_func,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_struct,
      anon_sym_TILDE,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_interface,
      anon_sym_PIPE,
      anon_sym_map,
      anon_sym_chan,
      anon_sym_LT_DASH,
      anon_sym_case,
      anon_sym_default,
      sym_identifier,
      anon_sym_BQUOTE,
      anon_sym_DQUOTE,
  [52424] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1558), 1,
      aux_sym_source_file_token1,
    ACTIONS(1556), 21,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LPAREN,
      anon_sym_EQ,
      anon_sym_func,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_struct,
      anon_sym_TILDE,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_interface,
      anon_sym_PIPE,
      anon_sym_map,
      anon_sym_chan,
      anon_sym_LT_DASH,
      anon_sym_case,
      anon_sym_default,
      sym_identifier,
      anon_sym_BQUOTE,
      anon_sym_DQUOTE,
  [52454] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1562), 1,
      aux_sym_source_file_token1,
    ACTIONS(1560), 21,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LPAREN,
      anon_sym_EQ,
      anon_sym_func,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_struct,
      anon_sym_TILDE,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_interface,
      anon_sym_PIPE,
      anon_sym_map,
      anon_sym_chan,
      anon_sym_LT_DASH,
      anon_sym_case,
      anon_sym_default,
      sym_identifier,
      anon_sym_BQUOTE,
      anon_sym_DQUOTE,
  [52484] = 5,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1566), 1,
      aux_sym_source_file_token1,
    ACTIONS(1568), 1,
      anon_sym_COMMA,
    STATE(786), 1,
      aux_sym_const_spec_repeat1,
    ACTIONS(1564), 17,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LPAREN,
      anon_sym_EQ,
      anon_sym_func,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_struct,
      anon_sym_TILDE,
      anon_sym_RBRACE,
      anon_sym_interface,
      anon_sym_map,
      anon_sym_chan,
      anon_sym_LT_DASH,
      anon_sym_case,
      anon_sym_default,
      sym_identifier,
  [52516] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1566), 1,
      aux_sym_source_file_token1,
    ACTIONS(1564), 18,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_func,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_struct,
      anon_sym_TILDE,
      anon_sym_RBRACE,
      anon_sym_interface,
      anon_sym_map,
      anon_sym_chan,
      anon_sym_LT_DASH,
      anon_sym_case,
      anon_sym_default,
      sym_identifier,
  [52543] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1552), 6,
      anon_sym_func,
      anon_sym_struct,
      anon_sym_interface,
      anon_sym_map,
      anon_sym_chan,
      sym_identifier,
    ACTIONS(1554), 12,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_STAR,
      anon_sym_TILDE,
      anon_sym_LBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON,
  [52569] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1548), 6,
      anon_sym_func,
      anon_sym_struct,
      anon_sym_interface,
      anon_sym_map,
      anon_sym_chan,
      sym_identifier,
    ACTIONS(1550), 12,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_STAR,
      anon_sym_TILDE,
      anon_sym_LBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON,
  [52595] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1556), 6,
      anon_sym_func,
      anon_sym_struct,
      anon_sym_interface,
      anon_sym_map,
      anon_sym_chan,
      sym_identifier,
    ACTIONS(1558), 12,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_STAR,
      anon_sym_TILDE,
      anon_sym_LBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON,
  [52621] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1560), 6,
      anon_sym_func,
      anon_sym_struct,
      anon_sym_interface,
      anon_sym_map,
      anon_sym_chan,
      sym_identifier,
    ACTIONS(1562), 12,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_STAR,
      anon_sym_TILDE,
      anon_sym_LBRACE,
      anon_sym_PIPE,
      anon_sym_LT_DASH,
      anon_sym_COLON,
  [52647] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1562), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(1560), 14,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LPAREN,
      anon_sym_func,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_struct,
      anon_sym_TILDE,
      anon_sym_LBRACE,
      anon_sym_interface,
      anon_sym_map,
      anon_sym_chan,
      anon_sym_LT_DASH,
      sym_identifier,
  [52671] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1550), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(1548), 14,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LPAREN,
      anon_sym_func,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_struct,
      anon_sym_TILDE,
      anon_sym_LBRACE,
      anon_sym_interface,
      anon_sym_map,
      anon_sym_chan,
      anon_sym_LT_DASH,
      sym_identifier,
  [52695] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1558), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(1556), 14,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LPAREN,
      anon_sym_func,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_struct,
      anon_sym_TILDE,
      anon_sym_LBRACE,
      anon_sym_interface,
      anon_sym_map,
      anon_sym_chan,
      anon_sym_LT_DASH,
      sym_identifier,
  [52719] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1554), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(1552), 14,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LPAREN,
      anon_sym_func,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_struct,
      anon_sym_TILDE,
      anon_sym_LBRACE,
      anon_sym_interface,
      anon_sym_map,
      anon_sym_chan,
      anon_sym_LT_DASH,
      sym_identifier,
  [52743] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1571), 1,
      anon_sym_COMMA,
    STATE(796), 1,
      aux_sym_expression_list_repeat1,
    ACTIONS(867), 13,
      anon_sym_EQ,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
  [52768] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(698), 1,
      anon_sym_COMMA,
    STATE(796), 1,
      aux_sym_expression_list_repeat1,
    ACTIONS(1574), 13,
      anon_sym_EQ,
      anon_sym_COLON_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
  [52793] = 6,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(893), 1,
      anon_sym_DOT,
    ACTIONS(1039), 1,
      aux_sym_source_file_token1,
    ACTIONS(1578), 1,
      anon_sym_LBRACK,
    STATE(827), 1,
      sym_type_arguments,
    ACTIONS(1576), 10,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_EQ,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_BQUOTE,
      anon_sym_DQUOTE,
  [52821] = 6,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(893), 1,
      anon_sym_DOT,
    ACTIONS(1039), 1,
      aux_sym_source_file_token1,
    ACTIONS(1581), 1,
      anon_sym_LBRACK,
    STATE(827), 1,
      sym_type_arguments,
    ACTIONS(1576), 10,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_EQ,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_BQUOTE,
      anon_sym_DQUOTE,
  [52849] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1587), 1,
      anon_sym_COMMA,
    STATE(800), 1,
      aux_sym_var_spec_repeat1,
    ACTIONS(1583), 6,
      anon_sym_func,
      anon_sym_struct,
      anon_sym_interface,
      anon_sym_map,
      anon_sym_chan,
      sym_identifier,
    ACTIONS(1585), 6,
      anon_sym_LPAREN,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_TILDE,
      anon_sym_LT_DASH,
  [52875] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1594), 1,
      anon_sym_COMMA,
    STATE(801), 1,
      aux_sym_field_declaration_repeat1,
    ACTIONS(1592), 5,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_TILDE,
      anon_sym_LT_DASH,
    ACTIONS(1590), 6,
      anon_sym_func,
      anon_sym_struct,
      anon_sym_interface,
      anon_sym_map,
      anon_sym_chan,
      sym_identifier,
  [52900] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1599), 1,
      anon_sym_COLON_EQ,
    ACTIONS(1597), 12,
      anon_sym_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
  [52921] = 5,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1601), 1,
      aux_sym_source_file_token1,
    ACTIONS(1605), 1,
      anon_sym_LBRACK,
    STATE(819), 1,
      sym_type_arguments,
    ACTIONS(1603), 10,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_EQ,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_BQUOTE,
      anon_sym_DQUOTE,
  [52946] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1610), 1,
      anon_sym_COLON_EQ,
    ACTIONS(1608), 12,
      anon_sym_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
  [52967] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1612), 6,
      anon_sym_func,
      anon_sym_struct,
      anon_sym_interface,
      anon_sym_map,
      anon_sym_chan,
      sym_identifier,
    ACTIONS(1614), 7,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_TILDE,
      anon_sym_LT_DASH,
  [52988] = 5,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1581), 1,
      anon_sym_LBRACK,
    ACTIONS(1601), 1,
      aux_sym_source_file_token1,
    STATE(819), 1,
      sym_type_arguments,
    ACTIONS(1603), 10,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_EQ,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_BQUOTE,
      anon_sym_DQUOTE,
  [53013] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1618), 1,
      anon_sym_COLON_EQ,
    ACTIONS(1616), 12,
      anon_sym_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
  [53034] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1620), 1,
      anon_sym_COLON_EQ,
    ACTIONS(1597), 12,
      anon_sym_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
  [53055] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1622), 1,
      anon_sym_EQ,
    ACTIONS(1624), 1,
      anon_sym_COLON_EQ,
    ACTIONS(1597), 11,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_AMP_CARET_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_CARET_EQ,
  [53078] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1626), 1,
      aux_sym_source_file_token1,
    ACTIONS(1628), 11,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_BQUOTE,
      anon_sym_DQUOTE,
  [53098] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1630), 1,
      aux_sym_source_file_token1,
    ACTIONS(1632), 11,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_BQUOTE,
      anon_sym_DQUOTE,
  [53118] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1634), 1,
      aux_sym_source_file_token1,
    ACTIONS(1636), 11,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_BQUOTE,
      anon_sym_DQUOTE,
  [53138] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1638), 1,
      aux_sym_source_file_token1,
    ACTIONS(1640), 11,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_BQUOTE,
      anon_sym_DQUOTE,
  [53158] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1642), 1,
      aux_sym_source_file_token1,
    ACTIONS(1644), 11,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_BQUOTE,
      anon_sym_DQUOTE,
  [53178] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1646), 1,
      aux_sym_source_file_token1,
    ACTIONS(1648), 11,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_BQUOTE,
      anon_sym_DQUOTE,
  [53198] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1601), 1,
      aux_sym_source_file_token1,
    ACTIONS(1603), 11,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_BQUOTE,
      anon_sym_DQUOTE,
  [53218] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1650), 1,
      aux_sym_source_file_token1,
    ACTIONS(1652), 11,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_BQUOTE,
      anon_sym_DQUOTE,
  [53238] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1654), 1,
      aux_sym_source_file_token1,
    ACTIONS(1656), 11,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_BQUOTE,
      anon_sym_DQUOTE,
  [53258] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1658), 1,
      aux_sym_source_file_token1,
    ACTIONS(1660), 11,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_BQUOTE,
      anon_sym_DQUOTE,
  [53278] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1662), 1,
      aux_sym_source_file_token1,
    ACTIONS(1664), 11,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_BQUOTE,
      anon_sym_DQUOTE,
  [53298] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1666), 1,
      aux_sym_source_file_token1,
    ACTIONS(1668), 11,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_BQUOTE,
      anon_sym_DQUOTE,
  [53318] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1670), 1,
      aux_sym_source_file_token1,
    ACTIONS(1672), 11,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_BQUOTE,
      anon_sym_DQUOTE,
  [53338] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1674), 1,
      aux_sym_source_file_token1,
    ACTIONS(1676), 11,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_BQUOTE,
      anon_sym_DQUOTE,
  [53358] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1678), 6,
      anon_sym_func,
      anon_sym_struct,
      anon_sym_interface,
      anon_sym_map,
      anon_sym_chan,
      sym_identifier,
    ACTIONS(1680), 6,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_TILDE,
      anon_sym_LT_DASH,
  [53378] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1682), 1,
      aux_sym_source_file_token1,
    ACTIONS(1684), 11,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_BQUOTE,
      anon_sym_DQUOTE,
  [53398] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1686), 1,
      aux_sym_source_file_token1,
    ACTIONS(1688), 11,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_BQUOTE,
      anon_sym_DQUOTE,
  [53418] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1690), 1,
      aux_sym_source_file_token1,
    ACTIONS(1692), 11,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_BQUOTE,
      anon_sym_DQUOTE,
  [53438] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1694), 1,
      aux_sym_source_file_token1,
    ACTIONS(1696), 11,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_BQUOTE,
      anon_sym_DQUOTE,
  [53458] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1698), 1,
      aux_sym_source_file_token1,
    ACTIONS(1700), 11,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_BQUOTE,
      anon_sym_DQUOTE,
  [53478] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1634), 1,
      aux_sym_source_file_token1,
    ACTIONS(1636), 11,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_BQUOTE,
      anon_sym_DQUOTE,
  [53498] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1702), 1,
      aux_sym_source_file_token1,
    ACTIONS(1704), 11,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_BQUOTE,
      anon_sym_DQUOTE,
  [53518] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1706), 1,
      aux_sym_source_file_token1,
    ACTIONS(1708), 11,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_BQUOTE,
      anon_sym_DQUOTE,
  [53538] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1710), 1,
      aux_sym_source_file_token1,
    ACTIONS(1712), 11,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_BQUOTE,
      anon_sym_DQUOTE,
  [53558] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1714), 1,
      aux_sym_source_file_token1,
    ACTIONS(1716), 11,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_BQUOTE,
      anon_sym_DQUOTE,
  [53578] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1718), 1,
      aux_sym_source_file_token1,
    ACTIONS(1720), 11,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_BQUOTE,
      anon_sym_DQUOTE,
  [53598] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1722), 1,
      aux_sym_source_file_token1,
    ACTIONS(1724), 11,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_BQUOTE,
      anon_sym_DQUOTE,
  [53618] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1634), 1,
      aux_sym_source_file_token1,
    ACTIONS(1636), 11,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_PIPE,
      anon_sym_case,
      anon_sym_default,
      anon_sym_BQUOTE,
      anon_sym_DQUOTE,
  [53638] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1728), 5,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_TILDE,
      anon_sym_LT_DASH,
    ACTIONS(1726), 6,
      anon_sym_func,
      anon_sym_struct,
      anon_sym_interface,
      anon_sym_map,
      anon_sym_chan,
      sym_identifier,
  [53657] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1730), 1,
      sym_identifier,
    ACTIONS(1732), 1,
      anon_sym_DOT,
    ACTIONS(1734), 1,
      sym_blank_identifier,
    ACTIONS(1736), 1,
      anon_sym_LPAREN,
    ACTIONS(1738), 1,
      anon_sym_BQUOTE,
    ACTIONS(1740), 1,
      anon_sym_DQUOTE,
    STATE(1128), 1,
      sym_dot,
    STATE(1122), 2,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
    STATE(1135), 2,
      sym_import_spec,
      sym_import_spec_list,
  [53690] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1744), 5,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_TILDE,
      anon_sym_LT_DASH,
    ACTIONS(1742), 6,
      anon_sym_func,
      anon_sym_struct,
      anon_sym_interface,
      anon_sym_map,
      anon_sym_chan,
      sym_identifier,
  [53709] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1748), 5,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_TILDE,
      anon_sym_LT_DASH,
    ACTIONS(1746), 6,
      anon_sym_func,
      anon_sym_struct,
      anon_sym_interface,
      anon_sym_map,
      anon_sym_chan,
      sym_identifier,
  [53728] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(69), 1,
      anon_sym_BQUOTE,
    ACTIONS(71), 1,
      anon_sym_DQUOTE,
    ACTIONS(1732), 1,
      anon_sym_DOT,
    ACTIONS(1750), 1,
      sym_identifier,
    ACTIONS(1752), 1,
      sym_blank_identifier,
    ACTIONS(1754), 1,
      anon_sym_LPAREN,
    STATE(1069), 1,
      sym_dot,
    STATE(1113), 2,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
    STATE(1229), 2,
      sym_import_spec,
      sym_import_spec_list,
  [53761] = 9,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(893), 1,
      anon_sym_DOT,
    ACTIONS(917), 1,
      anon_sym_BQUOTE,
    ACTIONS(919), 1,
      anon_sym_DQUOTE,
    ACTIONS(1581), 1,
      anon_sym_LBRACK,
    ACTIONS(1756), 1,
      aux_sym_source_file_token1,
    STATE(827), 1,
      sym_type_arguments,
    STATE(1107), 2,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
    ACTIONS(1758), 3,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
  [53792] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1760), 1,
      anon_sym_DOT,
    ACTIONS(1762), 1,
      anon_sym_LBRACK,
    STATE(875), 1,
      sym_type_arguments,
    ACTIONS(1039), 8,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_PIPE,
      anon_sym_COLON,
  [53815] = 8,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(917), 1,
      anon_sym_BQUOTE,
    ACTIONS(919), 1,
      anon_sym_DQUOTE,
    ACTIONS(1581), 1,
      anon_sym_LBRACK,
    ACTIONS(1765), 1,
      aux_sym_source_file_token1,
    STATE(819), 1,
      sym_type_arguments,
    STATE(1108), 2,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
    ACTIONS(1767), 3,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
  [53843] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(69), 1,
      anon_sym_BQUOTE,
    ACTIONS(71), 1,
      anon_sym_DQUOTE,
    ACTIONS(1732), 1,
      anon_sym_DOT,
    ACTIONS(1750), 1,
      sym_identifier,
    ACTIONS(1752), 1,
      sym_blank_identifier,
    ACTIONS(1769), 1,
      anon_sym_RPAREN,
    STATE(1028), 1,
      sym_import_spec,
    STATE(1069), 1,
      sym_dot,
    STATE(1113), 2,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [53875] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(69), 1,
      anon_sym_BQUOTE,
    ACTIONS(71), 1,
      anon_sym_DQUOTE,
    ACTIONS(1732), 1,
      anon_sym_DOT,
    ACTIONS(1750), 1,
      sym_identifier,
    ACTIONS(1752), 1,
      sym_blank_identifier,
    ACTIONS(1771), 1,
      anon_sym_RPAREN,
    STATE(1032), 1,
      sym_import_spec,
    STATE(1069), 1,
      sym_dot,
    STATE(1113), 2,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [53907] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(69), 1,
      anon_sym_BQUOTE,
    ACTIONS(71), 1,
      anon_sym_DQUOTE,
    ACTIONS(1732), 1,
      anon_sym_DOT,
    ACTIONS(1750), 1,
      sym_identifier,
    ACTIONS(1752), 1,
      sym_blank_identifier,
    ACTIONS(1773), 1,
      anon_sym_RPAREN,
    STATE(1060), 1,
      sym_import_spec,
    STATE(1069), 1,
      sym_dot,
    STATE(1113), 2,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [53939] = 8,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(917), 1,
      anon_sym_BQUOTE,
    ACTIONS(919), 1,
      anon_sym_DQUOTE,
    ACTIONS(1581), 1,
      anon_sym_LBRACK,
    ACTIONS(1775), 1,
      aux_sym_source_file_token1,
    STATE(819), 1,
      sym_type_arguments,
    STATE(1056), 2,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
    ACTIONS(1777), 3,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
  [53967] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1779), 1,
      anon_sym_LBRACK,
    STATE(866), 1,
      sym_type_arguments,
    ACTIONS(1601), 8,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_PIPE,
      anon_sym_COLON,
  [53987] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(69), 1,
      anon_sym_BQUOTE,
    ACTIONS(71), 1,
      anon_sym_DQUOTE,
    ACTIONS(1732), 1,
      anon_sym_DOT,
    ACTIONS(1750), 1,
      sym_identifier,
    ACTIONS(1752), 1,
      sym_blank_identifier,
    ACTIONS(1782), 1,
      anon_sym_RPAREN,
    STATE(1060), 1,
      sym_import_spec,
    STATE(1069), 1,
      sym_dot,
    STATE(1113), 2,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [54019] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(69), 1,
      anon_sym_BQUOTE,
    ACTIONS(71), 1,
      anon_sym_DQUOTE,
    ACTIONS(1732), 1,
      anon_sym_DOT,
    ACTIONS(1750), 1,
      sym_identifier,
    ACTIONS(1752), 1,
      sym_blank_identifier,
    ACTIONS(1784), 1,
      anon_sym_RPAREN,
    STATE(1060), 1,
      sym_import_spec,
    STATE(1069), 1,
      sym_dot,
    STATE(1113), 2,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [54051] = 8,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(893), 1,
      anon_sym_DOT,
    ACTIONS(925), 1,
      anon_sym_LPAREN,
    ACTIONS(1039), 1,
      aux_sym_source_file_token1,
    ACTIONS(1581), 1,
      anon_sym_LBRACK,
    STATE(519), 1,
      sym_parameter_list,
    STATE(827), 1,
      sym_type_arguments,
    ACTIONS(1576), 4,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_PIPE,
  [54079] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(69), 1,
      anon_sym_BQUOTE,
    ACTIONS(71), 1,
      anon_sym_DQUOTE,
    ACTIONS(1732), 1,
      anon_sym_DOT,
    ACTIONS(1750), 1,
      sym_identifier,
    ACTIONS(1752), 1,
      sym_blank_identifier,
    ACTIONS(1786), 1,
      anon_sym_RPAREN,
    STATE(1060), 1,
      sym_import_spec,
    STATE(1069), 1,
      sym_dot,
    STATE(1113), 2,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [54111] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1760), 1,
      anon_sym_DOT,
    ACTIONS(1788), 1,
      anon_sym_LBRACK,
    STATE(875), 1,
      sym_type_arguments,
    ACTIONS(1039), 7,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_PIPE,
      anon_sym_COLON,
  [54133] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1718), 9,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_PIPE,
      anon_sym_COLON,
  [54148] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1710), 9,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_PIPE,
      anon_sym_COLON,
  [54163] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1694), 9,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_PIPE,
      anon_sym_COLON,
  [54178] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1638), 9,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_PIPE,
      anon_sym_COLON,
  [54193] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1650), 9,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_PIPE,
      anon_sym_COLON,
  [54208] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1662), 9,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_PIPE,
      anon_sym_COLON,
  [54223] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1630), 9,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_PIPE,
      anon_sym_COLON,
  [54238] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1698), 9,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_PIPE,
      anon_sym_COLON,
  [54253] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1714), 9,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_PIPE,
      anon_sym_COLON,
  [54268] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1626), 9,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_PIPE,
      anon_sym_COLON,
  [54283] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1658), 9,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_PIPE,
      anon_sym_COLON,
  [54298] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1601), 9,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_PIPE,
      anon_sym_COLON,
  [54313] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1722), 9,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_PIPE,
      anon_sym_COLON,
  [54328] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1634), 9,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_PIPE,
      anon_sym_COLON,
  [54343] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1654), 9,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_PIPE,
      anon_sym_COLON,
  [54358] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(312), 1,
      anon_sym_LBRACE,
    ACTIONS(1779), 1,
      anon_sym_LBRACK,
    STATE(423), 1,
      sym_literal_value,
    STATE(866), 1,
      sym_type_arguments,
    ACTIONS(1601), 5,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PIPE,
  [54381] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1642), 9,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_PIPE,
      anon_sym_COLON,
  [54396] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1674), 9,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_PIPE,
      anon_sym_COLON,
  [54411] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1706), 9,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_PIPE,
      anon_sym_COLON,
  [54426] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1690), 9,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_PIPE,
      anon_sym_COLON,
  [54441] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1686), 9,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_PIPE,
      anon_sym_COLON,
  [54456] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1788), 1,
      anon_sym_LBRACK,
    STATE(866), 1,
      sym_type_arguments,
    ACTIONS(1601), 7,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_PIPE,
      anon_sym_COLON,
  [54475] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1670), 9,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_PIPE,
      anon_sym_COLON,
  [54490] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1666), 9,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_PIPE,
      anon_sym_COLON,
  [54505] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1634), 9,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_PIPE,
      anon_sym_COLON,
  [54520] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1634), 9,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_PIPE,
      anon_sym_COLON,
  [54535] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(69), 1,
      anon_sym_BQUOTE,
    ACTIONS(71), 1,
      anon_sym_DQUOTE,
    ACTIONS(1732), 1,
      anon_sym_DOT,
    ACTIONS(1750), 1,
      sym_identifier,
    ACTIONS(1752), 1,
      sym_blank_identifier,
    STATE(1060), 1,
      sym_import_spec,
    STATE(1069), 1,
      sym_dot,
    STATE(1113), 2,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [54564] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1646), 9,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_PIPE,
      anon_sym_COLON,
  [54579] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1702), 9,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_PIPE,
      anon_sym_COLON,
  [54594] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1682), 9,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_PIPE,
      anon_sym_COLON,
  [54609] = 5,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(977), 1,
      anon_sym_COMMA,
    ACTIONS(1574), 1,
      aux_sym_source_file_token1,
    STATE(888), 1,
      aux_sym_expression_list_repeat1,
    ACTIONS(1790), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [54629] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(1792), 1,
      sym_identifier,
    ACTIONS(1794), 1,
      anon_sym_STAR,
    ACTIONS(1796), 1,
      anon_sym_RBRACE,
    STATE(849), 1,
      sym_qualified_type,
    STATE(899), 1,
      sym_generic_type,
    STATE(1120), 1,
      sym_field_declaration,
    STATE(1280), 1,
      sym_negated_type,
  [54657] = 5,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(867), 1,
      aux_sym_source_file_token1,
    ACTIONS(1798), 1,
      anon_sym_COMMA,
    STATE(888), 1,
      aux_sym_expression_list_repeat1,
    ACTIONS(869), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [54677] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1033), 1,
      anon_sym_LBRACE,
    STATE(429), 1,
      sym_block,
    ACTIONS(1722), 6,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_PIPE,
  [54695] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(1792), 1,
      sym_identifier,
    ACTIONS(1794), 1,
      anon_sym_STAR,
    ACTIONS(1801), 1,
      anon_sym_RBRACE,
    STATE(849), 1,
      sym_qualified_type,
    STATE(899), 1,
      sym_generic_type,
    STATE(1120), 1,
      sym_field_declaration,
    STATE(1280), 1,
      sym_negated_type,
  [54723] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(1792), 1,
      sym_identifier,
    ACTIONS(1794), 1,
      anon_sym_STAR,
    ACTIONS(1803), 1,
      anon_sym_RBRACE,
    STATE(849), 1,
      sym_qualified_type,
    STATE(899), 1,
      sym_generic_type,
    STATE(1039), 1,
      sym_field_declaration,
    STATE(1280), 1,
      sym_negated_type,
  [54751] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(1792), 1,
      sym_identifier,
    ACTIONS(1794), 1,
      anon_sym_STAR,
    ACTIONS(1805), 1,
      anon_sym_RBRACE,
    STATE(849), 1,
      sym_qualified_type,
    STATE(899), 1,
      sym_generic_type,
    STATE(1120), 1,
      sym_field_declaration,
    STATE(1280), 1,
      sym_negated_type,
  [54779] = 6,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1807), 1,
      anon_sym_DOT,
    ACTIONS(1809), 1,
      anon_sym_LBRACK,
    STATE(930), 1,
      sym_type_arguments,
    ACTIONS(1039), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(1576), 3,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LBRACE,
  [54801] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(1792), 1,
      sym_identifier,
    ACTIONS(1794), 1,
      anon_sym_STAR,
    ACTIONS(1812), 1,
      anon_sym_RBRACE,
    STATE(849), 1,
      sym_qualified_type,
    STATE(899), 1,
      sym_generic_type,
    STATE(1120), 1,
      sym_field_declaration,
    STATE(1280), 1,
      sym_negated_type,
  [54829] = 6,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(917), 1,
      anon_sym_BQUOTE,
    ACTIONS(919), 1,
      anon_sym_DQUOTE,
    ACTIONS(1765), 1,
      aux_sym_source_file_token1,
    STATE(1108), 2,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
    ACTIONS(1767), 3,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
  [54851] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(1792), 1,
      sym_identifier,
    ACTIONS(1794), 1,
      anon_sym_STAR,
    ACTIONS(1814), 1,
      anon_sym_RBRACE,
    STATE(849), 1,
      sym_qualified_type,
    STATE(899), 1,
      sym_generic_type,
    STATE(1053), 1,
      sym_field_declaration,
    STATE(1280), 1,
      sym_negated_type,
  [54879] = 6,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1807), 1,
      anon_sym_DOT,
    ACTIONS(1816), 1,
      anon_sym_LBRACK,
    STATE(930), 1,
      sym_type_arguments,
    ACTIONS(1039), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(1576), 3,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LBRACE,
  [54901] = 7,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(917), 1,
      anon_sym_BQUOTE,
    ACTIONS(919), 1,
      anon_sym_DQUOTE,
    ACTIONS(1818), 1,
      aux_sym_source_file_token1,
    STATE(1063), 1,
      sym_raw_string_literal,
    STATE(1072), 1,
      sym_interpreted_string_literal,
    ACTIONS(1820), 3,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
  [54925] = 6,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(917), 1,
      anon_sym_BQUOTE,
    ACTIONS(919), 1,
      anon_sym_DQUOTE,
    ACTIONS(1775), 1,
      aux_sym_source_file_token1,
    STATE(1056), 2,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
    ACTIONS(1777), 3,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
  [54947] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(312), 1,
      anon_sym_LBRACE,
    STATE(423), 1,
      sym_literal_value,
    ACTIONS(1601), 6,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_PIPE,
  [54965] = 6,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(917), 1,
      anon_sym_BQUOTE,
    ACTIONS(919), 1,
      anon_sym_DQUOTE,
    ACTIONS(1822), 1,
      aux_sym_source_file_token1,
    STATE(1112), 2,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
    ACTIONS(1824), 3,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
  [54987] = 7,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(917), 1,
      anon_sym_BQUOTE,
    ACTIONS(919), 1,
      anon_sym_DQUOTE,
    ACTIONS(1818), 1,
      aux_sym_source_file_token1,
    STATE(1124), 1,
      sym_raw_string_literal,
    STATE(1126), 1,
      sym_interpreted_string_literal,
    ACTIONS(1820), 3,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
  [55011] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(1792), 1,
      sym_identifier,
    ACTIONS(1794), 1,
      anon_sym_STAR,
    ACTIONS(1826), 1,
      anon_sym_RBRACE,
    STATE(849), 1,
      sym_qualified_type,
    STATE(899), 1,
      sym_generic_type,
    STATE(1120), 1,
      sym_field_declaration,
    STATE(1280), 1,
      sym_negated_type,
  [55039] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(1792), 1,
      sym_identifier,
    ACTIONS(1794), 1,
      anon_sym_STAR,
    ACTIONS(1828), 1,
      anon_sym_RBRACE,
    STATE(849), 1,
      sym_qualified_type,
    STATE(899), 1,
      sym_generic_type,
    STATE(1034), 1,
      sym_field_declaration,
    STATE(1280), 1,
      sym_negated_type,
  [55067] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(1792), 1,
      sym_identifier,
    ACTIONS(1794), 1,
      anon_sym_STAR,
    ACTIONS(1830), 1,
      anon_sym_RBRACE,
    STATE(849), 1,
      sym_qualified_type,
    STATE(899), 1,
      sym_generic_type,
    STATE(1120), 1,
      sym_field_declaration,
    STATE(1280), 1,
      sym_negated_type,
  [55095] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(1792), 1,
      sym_identifier,
    ACTIONS(1794), 1,
      anon_sym_STAR,
    STATE(849), 1,
      sym_qualified_type,
    STATE(899), 1,
      sym_generic_type,
    STATE(1120), 1,
      sym_field_declaration,
    STATE(1280), 1,
      sym_negated_type,
  [55120] = 4,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1832), 1,
      sym_identifier,
    ACTIONS(1834), 1,
      aux_sym_source_file_token1,
    ACTIONS(1836), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [55137] = 5,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1838), 1,
      aux_sym_source_file_token1,
    STATE(916), 1,
      aux_sym__statement_list_repeat1,
    ACTIONS(1840), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
    ACTIONS(212), 3,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [55156] = 4,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1842), 1,
      aux_sym_source_file_token1,
    ACTIONS(1846), 1,
      anon_sym_EQ,
    ACTIONS(1844), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [55173] = 5,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1848), 1,
      anon_sym_LBRACK,
    STATE(932), 1,
      sym_type_arguments,
    ACTIONS(1601), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(1603), 3,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LBRACE,
  [55192] = 4,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1842), 1,
      aux_sym_source_file_token1,
    ACTIONS(1851), 1,
      anon_sym_EQ,
    ACTIONS(1844), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [55209] = 4,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1853), 1,
      aux_sym_source_file_token1,
    ACTIONS(1857), 1,
      anon_sym_EQ,
    ACTIONS(1855), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [55226] = 5,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1859), 1,
      aux_sym_source_file_token1,
    STATE(908), 1,
      aux_sym__statement_list_repeat1,
    ACTIONS(1861), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
    ACTIONS(1863), 3,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [55245] = 4,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1865), 1,
      sym_identifier,
    ACTIONS(1867), 1,
      aux_sym_source_file_token1,
    ACTIONS(1869), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [55262] = 4,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1871), 1,
      aux_sym_source_file_token1,
    ACTIONS(1875), 1,
      anon_sym_else,
    ACTIONS(1873), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [55279] = 5,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1877), 1,
      aux_sym_source_file_token1,
    STATE(916), 1,
      aux_sym__statement_list_repeat1,
    ACTIONS(1880), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
    ACTIONS(1883), 3,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [55298] = 4,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1885), 1,
      aux_sym_source_file_token1,
    ACTIONS(1889), 1,
      anon_sym_else,
    ACTIONS(1887), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [55315] = 5,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1816), 1,
      anon_sym_LBRACK,
    STATE(932), 1,
      sym_type_arguments,
    ACTIONS(1601), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(1603), 3,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LBRACE,
  [55334] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1698), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(1700), 4,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
  [55348] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1891), 1,
      aux_sym_source_file_token1,
    ACTIONS(1893), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [55362] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1895), 1,
      aux_sym_source_file_token1,
    ACTIONS(1883), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [55376] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1601), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(1603), 4,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
  [55390] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1897), 1,
      aux_sym_source_file_token1,
    ACTIONS(1899), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [55404] = 5,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1066), 1,
      anon_sym_LBRACE,
    STATE(1102), 1,
      sym_block,
    ACTIONS(1901), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(1903), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [55422] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1702), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(1704), 4,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
  [55436] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1694), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(1696), 4,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
  [55450] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1905), 1,
      aux_sym_source_file_token1,
    ACTIONS(1907), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [55464] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1706), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(1708), 4,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
  [55478] = 5,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1909), 1,
      aux_sym_source_file_token1,
    ACTIONS(1913), 1,
      anon_sym_PIPE,
    STATE(1018), 1,
      aux_sym_type_elem_repeat1,
    ACTIONS(1911), 3,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
  [55496] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1690), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(1692), 4,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
  [55510] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1915), 1,
      aux_sym_source_file_token1,
    ACTIONS(1917), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [55524] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1658), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(1660), 4,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
  [55538] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1919), 1,
      aux_sym_source_file_token1,
    ACTIONS(1921), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [55552] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1923), 1,
      aux_sym_source_file_token1,
    ACTIONS(1925), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [55566] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1096), 1,
      anon_sym_COMMA,
    ACTIONS(1790), 1,
      anon_sym_COLON,
    STATE(962), 1,
      aux_sym_expression_list_repeat1,
    ACTIONS(1574), 3,
      anon_sym_SEMI,
      anon_sym_EQ,
      anon_sym_COLON_EQ,
  [55584] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1666), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(1668), 4,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
  [55598] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1927), 1,
      aux_sym_source_file_token1,
    ACTIONS(1929), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [55612] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1039), 1,
      anon_sym_LBRACE,
    ACTIONS(1760), 1,
      anon_sym_DOT,
    ACTIONS(1762), 1,
      anon_sym_LBRACK,
    ACTIONS(1931), 1,
      anon_sym_LPAREN,
    STATE(499), 1,
      sym_parameter_list,
    STATE(875), 1,
      sym_type_arguments,
  [55634] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1714), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(1716), 4,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
  [55648] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1933), 1,
      aux_sym_source_file_token1,
    ACTIONS(1935), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [55662] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1654), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(1656), 4,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
  [55676] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1937), 1,
      aux_sym_source_file_token1,
    ACTIONS(1939), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [55690] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1941), 1,
      aux_sym_source_file_token1,
    ACTIONS(1943), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [55704] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1674), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(1676), 4,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
  [55718] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1686), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(1688), 4,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
  [55732] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1634), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(1636), 4,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
  [55746] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1945), 1,
      aux_sym_source_file_token1,
    ACTIONS(1947), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [55760] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1634), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(1636), 4,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
  [55774] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1949), 1,
      aux_sym_source_file_token1,
    ACTIONS(1951), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [55788] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1646), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(1648), 4,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
  [55802] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1953), 1,
      anon_sym_RBRACE,
    ACTIONS(1955), 1,
      anon_sym_case,
    ACTIONS(1958), 1,
      anon_sym_default,
    STATE(951), 3,
      sym_expression_case,
      sym_default_case,
      aux_sym_expression_switch_statement_repeat1,
  [55820] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1961), 1,
      aux_sym_source_file_token1,
    ACTIONS(1963), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [55834] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1965), 1,
      anon_sym_RBRACE,
    ACTIONS(1967), 1,
      anon_sym_case,
    ACTIONS(1969), 1,
      anon_sym_default,
    STATE(1015), 3,
      sym_expression_case,
      sym_default_case,
      aux_sym_expression_switch_statement_repeat1,
  [55852] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1971), 1,
      aux_sym_source_file_token1,
    ACTIONS(1973), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [55866] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1975), 1,
      aux_sym_source_file_token1,
    ACTIONS(1977), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [55880] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1979), 1,
      aux_sym_source_file_token1,
    ACTIONS(1981), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [55894] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1983), 1,
      aux_sym_source_file_token1,
    ACTIONS(1985), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [55908] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1969), 1,
      anon_sym_default,
    ACTIONS(1987), 1,
      anon_sym_RBRACE,
    ACTIONS(1989), 1,
      anon_sym_case,
    STATE(974), 3,
      sym_default_case,
      sym_type_case,
      aux_sym_type_switch_statement_repeat1,
  [55926] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1991), 1,
      aux_sym_source_file_token1,
    ACTIONS(1993), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [55940] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1967), 1,
      anon_sym_case,
    ACTIONS(1969), 1,
      anon_sym_default,
    ACTIONS(1995), 1,
      anon_sym_RBRACE,
    STATE(951), 3,
      sym_expression_case,
      sym_default_case,
      aux_sym_expression_switch_statement_repeat1,
  [55958] = 5,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1913), 1,
      anon_sym_PIPE,
    ACTIONS(1997), 1,
      aux_sym_source_file_token1,
    STATE(929), 1,
      aux_sym_type_elem_repeat1,
    ACTIONS(1999), 3,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
  [55976] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(869), 1,
      anon_sym_COLON,
    ACTIONS(2001), 1,
      anon_sym_COMMA,
    STATE(962), 1,
      aux_sym_expression_list_repeat1,
    ACTIONS(867), 3,
      anon_sym_SEMI,
      anon_sym_EQ,
      anon_sym_COLON_EQ,
  [55994] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2004), 1,
      aux_sym_source_file_token1,
    ACTIONS(2006), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [56008] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1967), 1,
      anon_sym_case,
    ACTIONS(1969), 1,
      anon_sym_default,
    ACTIONS(2008), 1,
      anon_sym_RBRACE,
    STATE(951), 3,
      sym_expression_case,
      sym_default_case,
      aux_sym_expression_switch_statement_repeat1,
  [56026] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1662), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(1664), 4,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
  [56040] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1722), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(1724), 4,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
  [56054] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1634), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(1636), 4,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
  [56068] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1642), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(1644), 4,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
  [56082] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1967), 1,
      anon_sym_case,
    ACTIONS(1969), 1,
      anon_sym_default,
    ACTIONS(2010), 1,
      anon_sym_RBRACE,
    STATE(960), 3,
      sym_expression_case,
      sym_default_case,
      aux_sym_expression_switch_statement_repeat1,
  [56100] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1670), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(1672), 4,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
  [56114] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2012), 1,
      aux_sym_source_file_token1,
    ACTIONS(2014), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [56128] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2016), 1,
      anon_sym_RBRACE,
    ACTIONS(2018), 1,
      anon_sym_case,
    ACTIONS(2021), 1,
      anon_sym_default,
    STATE(972), 3,
      sym_default_case,
      sym_communication_case,
      aux_sym_select_statement_repeat1,
  [56146] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2024), 1,
      aux_sym_source_file_token1,
    ACTIONS(2026), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [56160] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2028), 1,
      anon_sym_RBRACE,
    ACTIONS(2030), 1,
      anon_sym_case,
    ACTIONS(2033), 1,
      anon_sym_default,
    STATE(974), 3,
      sym_default_case,
      sym_type_case,
      aux_sym_type_switch_statement_repeat1,
  [56178] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1718), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(1720), 4,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
  [56192] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2036), 1,
      aux_sym_source_file_token1,
    ACTIONS(2038), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [56206] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2040), 1,
      aux_sym_source_file_token1,
    ACTIONS(2042), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [56220] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1710), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(1712), 4,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
  [56234] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1630), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(1632), 4,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
  [56248] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2044), 1,
      aux_sym_source_file_token1,
    ACTIONS(2046), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [56262] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2048), 1,
      aux_sym_source_file_token1,
    ACTIONS(2050), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [56276] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1638), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(1640), 4,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
  [56290] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1650), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(1652), 4,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
  [56304] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1626), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(1628), 4,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
  [56318] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2052), 1,
      aux_sym_source_file_token1,
    ACTIONS(2054), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [56332] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2056), 1,
      aux_sym_source_file_token1,
    ACTIONS(2058), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [56346] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2060), 1,
      aux_sym_source_file_token1,
    ACTIONS(2062), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [56360] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2064), 1,
      aux_sym_source_file_token1,
    ACTIONS(2066), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [56374] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2068), 1,
      aux_sym_source_file_token1,
    ACTIONS(2070), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [56388] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2072), 1,
      aux_sym_source_file_token1,
    ACTIONS(2074), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [56402] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2076), 1,
      aux_sym_source_file_token1,
    ACTIONS(2078), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [56416] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2080), 1,
      aux_sym_source_file_token1,
    ACTIONS(2082), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [56430] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2084), 1,
      aux_sym_source_file_token1,
    ACTIONS(2086), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [56444] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2088), 1,
      aux_sym_source_file_token1,
    ACTIONS(2090), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [56458] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2092), 1,
      aux_sym_source_file_token1,
    ACTIONS(2094), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [56472] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2096), 1,
      aux_sym_source_file_token1,
    ACTIONS(2098), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [56486] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1967), 1,
      anon_sym_case,
    ACTIONS(1969), 1,
      anon_sym_default,
    ACTIONS(2100), 1,
      anon_sym_RBRACE,
    STATE(951), 3,
      sym_expression_case,
      sym_default_case,
      aux_sym_expression_switch_statement_repeat1,
  [56504] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1967), 1,
      anon_sym_case,
    ACTIONS(1969), 1,
      anon_sym_default,
    ACTIONS(2102), 1,
      anon_sym_RBRACE,
    STATE(997), 3,
      sym_expression_case,
      sym_default_case,
      aux_sym_expression_switch_statement_repeat1,
  [56522] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2104), 1,
      aux_sym_source_file_token1,
    ACTIONS(2106), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [56536] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2108), 1,
      aux_sym_source_file_token1,
    ACTIONS(2110), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [56550] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1969), 1,
      anon_sym_default,
    ACTIONS(1989), 1,
      anon_sym_case,
    ACTIONS(2112), 1,
      anon_sym_RBRACE,
    STATE(958), 3,
      sym_default_case,
      sym_type_case,
      aux_sym_type_switch_statement_repeat1,
  [56568] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2108), 1,
      aux_sym_source_file_token1,
    ACTIONS(2110), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [56582] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2114), 1,
      aux_sym_source_file_token1,
    ACTIONS(2116), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [56596] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1967), 1,
      anon_sym_case,
    ACTIONS(1969), 1,
      anon_sym_default,
    ACTIONS(2118), 1,
      anon_sym_RBRACE,
    STATE(964), 3,
      sym_expression_case,
      sym_default_case,
      aux_sym_expression_switch_statement_repeat1,
  [56614] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1969), 1,
      anon_sym_default,
    ACTIONS(2120), 1,
      anon_sym_RBRACE,
    ACTIONS(2122), 1,
      anon_sym_case,
    STATE(1007), 3,
      sym_default_case,
      sym_communication_case,
      aux_sym_select_statement_repeat1,
  [56632] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2124), 1,
      aux_sym_source_file_token1,
    ACTIONS(2126), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [56646] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1969), 1,
      anon_sym_default,
    ACTIONS(2122), 1,
      anon_sym_case,
    ACTIONS(2128), 1,
      anon_sym_RBRACE,
    STATE(972), 3,
      sym_default_case,
      sym_communication_case,
      aux_sym_select_statement_repeat1,
  [56664] = 5,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1066), 1,
      anon_sym_LBRACE,
    STATE(1140), 1,
      sym_block,
    ACTIONS(2130), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(2132), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [56682] = 5,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1066), 1,
      anon_sym_LBRACE,
    STATE(1104), 1,
      sym_block,
    ACTIONS(2134), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(2136), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [56700] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2138), 1,
      aux_sym_source_file_token1,
    ACTIONS(2140), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [56714] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2142), 1,
      aux_sym_source_file_token1,
    ACTIONS(2144), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [56728] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1039), 1,
      anon_sym_LBRACE,
    ACTIONS(1760), 1,
      anon_sym_DOT,
    ACTIONS(1762), 1,
      anon_sym_LBRACK,
    ACTIONS(2146), 1,
      anon_sym_LPAREN,
    STATE(458), 1,
      sym_parameter_list,
    STATE(875), 1,
      sym_type_arguments,
  [56750] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2148), 1,
      aux_sym_source_file_token1,
    ACTIONS(2150), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [56764] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2152), 1,
      aux_sym_source_file_token1,
    ACTIONS(2154), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [56778] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1967), 1,
      anon_sym_case,
    ACTIONS(1969), 1,
      anon_sym_default,
    ACTIONS(2156), 1,
      anon_sym_RBRACE,
    STATE(951), 3,
      sym_expression_case,
      sym_default_case,
      aux_sym_expression_switch_statement_repeat1,
  [56796] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1674), 6,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_PIPE,
  [56808] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2158), 1,
      aux_sym_source_file_token1,
    ACTIONS(2160), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [56822] = 5,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2162), 1,
      aux_sym_source_file_token1,
    ACTIONS(2166), 1,
      anon_sym_PIPE,
    STATE(1018), 1,
      aux_sym_type_elem_repeat1,
    ACTIONS(2164), 3,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
  [56840] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2169), 1,
      anon_sym_LPAREN,
    ACTIONS(2171), 1,
      anon_sym_LBRACK,
    ACTIONS(2173), 1,
      anon_sym_PIPE,
    STATE(1079), 1,
      aux_sym_type_elem_repeat1,
    ACTIONS(1997), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
  [56860] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2175), 1,
      aux_sym_source_file_token1,
    ACTIONS(2177), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [56874] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2179), 1,
      aux_sym_source_file_token1,
    ACTIONS(2181), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [56888] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2183), 1,
      aux_sym_source_file_token1,
    ACTIONS(2185), 5,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [56902] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1682), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(1684), 4,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
  [56916] = 5,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2187), 1,
      aux_sym_source_file_token1,
    ACTIONS(2191), 1,
      anon_sym_RBRACE,
    STATE(1046), 1,
      aux_sym_field_declaration_list_repeat1,
    ACTIONS(2189), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [56933] = 5,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2193), 1,
      aux_sym_source_file_token1,
    ACTIONS(2197), 1,
      anon_sym_RBRACE,
    STATE(1046), 1,
      aux_sym_field_declaration_list_repeat1,
    ACTIONS(2195), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [56950] = 5,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1118), 1,
      anon_sym_LBRACE,
    ACTIONS(2130), 1,
      aux_sym_source_file_token1,
    STATE(1162), 1,
      sym_block,
    ACTIONS(2132), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [56967] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1601), 1,
      anon_sym_LPAREN,
    ACTIONS(1779), 1,
      anon_sym_LBRACK,
    ACTIONS(2199), 1,
      anon_sym_LBRACE,
    STATE(264), 1,
      sym_literal_value,
    STATE(866), 1,
      sym_type_arguments,
  [56986] = 5,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2201), 1,
      aux_sym_source_file_token1,
    ACTIONS(2205), 1,
      anon_sym_RPAREN,
    STATE(1050), 1,
      aux_sym_import_spec_list_repeat1,
    ACTIONS(2203), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [57003] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1601), 1,
      anon_sym_LPAREN,
    ACTIONS(1779), 1,
      anon_sym_LBRACK,
    ACTIONS(2207), 1,
      anon_sym_LBRACE,
    STATE(325), 1,
      sym_literal_value,
    STATE(866), 1,
      sym_type_arguments,
  [57022] = 5,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2209), 1,
      aux_sym_source_file_token1,
    ACTIONS(2213), 1,
      anon_sym_RBRACE,
    STATE(1051), 1,
      aux_sym_interface_type_repeat1,
    ACTIONS(2211), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [57039] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1090), 1,
      anon_sym_LBRACE,
    ACTIONS(1601), 1,
      anon_sym_LPAREN,
    ACTIONS(1779), 1,
      anon_sym_LBRACK,
    STATE(569), 1,
      sym_literal_value,
    STATE(866), 1,
      sym_type_arguments,
  [57058] = 5,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2215), 1,
      aux_sym_source_file_token1,
    ACTIONS(2219), 1,
      anon_sym_RPAREN,
    STATE(1043), 1,
      aux_sym_import_spec_list_repeat1,
    ACTIONS(2217), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [57075] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2221), 1,
      sym_identifier,
    ACTIONS(2224), 1,
      anon_sym_RPAREN,
    STATE(1033), 1,
      aux_sym_type_declaration_repeat1,
    STATE(1166), 2,
      sym_type_alias,
      sym_type_spec,
  [57092] = 5,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2226), 1,
      aux_sym_source_file_token1,
    ACTIONS(2230), 1,
      anon_sym_RBRACE,
    STATE(1025), 1,
      aux_sym_field_declaration_list_repeat1,
    ACTIONS(2228), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [57109] = 5,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2232), 1,
      aux_sym_source_file_token1,
    ACTIONS(2236), 1,
      anon_sym_RBRACE,
    STATE(1047), 1,
      aux_sym_interface_type_repeat1,
    ACTIONS(2234), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [57126] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2238), 1,
      sym_identifier,
    ACTIONS(2240), 1,
      anon_sym_RPAREN,
    STATE(1052), 1,
      aux_sym_type_declaration_repeat1,
    STATE(1166), 2,
      sym_type_alias,
      sym_type_spec,
  [57143] = 5,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1118), 1,
      anon_sym_LBRACE,
    ACTIONS(2134), 1,
      aux_sym_source_file_token1,
    STATE(1163), 1,
      sym_block,
    ACTIONS(2136), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [57160] = 5,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2242), 1,
      aux_sym_source_file_token1,
    ACTIONS(2246), 1,
      anon_sym_RBRACE,
    STATE(1030), 1,
      aux_sym_interface_type_repeat1,
    ACTIONS(2244), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [57177] = 5,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2248), 1,
      aux_sym_source_file_token1,
    ACTIONS(2252), 1,
      anon_sym_RBRACE,
    STATE(1045), 1,
      aux_sym_field_declaration_list_repeat1,
    ACTIONS(2250), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [57194] = 5,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2254), 1,
      aux_sym_source_file_token1,
    ACTIONS(2258), 1,
      anon_sym_RBRACE,
    STATE(1049), 1,
      aux_sym_interface_type_repeat1,
    ACTIONS(2256), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [57211] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(31), 1,
      anon_sym_TILDE,
    ACTIONS(2260), 1,
      sym_identifier,
    STATE(845), 1,
      sym_qualified_type,
    STATE(895), 1,
      sym_generic_type,
    STATE(1280), 1,
      sym_negated_type,
  [57230] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(861), 1,
      anon_sym_LBRACE,
    ACTIONS(1601), 1,
      anon_sym_LPAREN,
    ACTIONS(1779), 1,
      anon_sym_LBRACK,
    STATE(351), 1,
      sym_literal_value,
    STATE(866), 1,
      sym_type_arguments,
  [57249] = 5,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2262), 1,
      aux_sym_source_file_token1,
    ACTIONS(2266), 1,
      anon_sym_RPAREN,
    STATE(1044), 1,
      aux_sym_import_spec_list_repeat1,
    ACTIONS(2264), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [57266] = 5,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2268), 1,
      aux_sym_source_file_token1,
    ACTIONS(2274), 1,
      anon_sym_RPAREN,
    STATE(1044), 1,
      aux_sym_import_spec_list_repeat1,
    ACTIONS(2271), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [57283] = 5,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2276), 1,
      aux_sym_source_file_token1,
    ACTIONS(2280), 1,
      anon_sym_RBRACE,
    STATE(1046), 1,
      aux_sym_field_declaration_list_repeat1,
    ACTIONS(2278), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [57300] = 5,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2282), 1,
      aux_sym_source_file_token1,
    ACTIONS(2288), 1,
      anon_sym_RBRACE,
    STATE(1046), 1,
      aux_sym_field_declaration_list_repeat1,
    ACTIONS(2285), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [57317] = 5,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2290), 1,
      aux_sym_source_file_token1,
    ACTIONS(2294), 1,
      anon_sym_RBRACE,
    STATE(1051), 1,
      aux_sym_interface_type_repeat1,
    ACTIONS(2292), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [57334] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2162), 1,
      aux_sym_source_file_token1,
    ACTIONS(2164), 4,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
      anon_sym_PIPE,
  [57347] = 5,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2296), 1,
      aux_sym_source_file_token1,
    ACTIONS(2300), 1,
      anon_sym_RBRACE,
    STATE(1051), 1,
      aux_sym_interface_type_repeat1,
    ACTIONS(2298), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [57364] = 5,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2302), 1,
      aux_sym_source_file_token1,
    ACTIONS(2306), 1,
      anon_sym_RPAREN,
    STATE(1044), 1,
      aux_sym_import_spec_list_repeat1,
    ACTIONS(2304), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [57381] = 5,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2308), 1,
      aux_sym_source_file_token1,
    ACTIONS(2314), 1,
      anon_sym_RBRACE,
    STATE(1051), 1,
      aux_sym_interface_type_repeat1,
    ACTIONS(2311), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [57398] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2238), 1,
      sym_identifier,
    ACTIONS(2316), 1,
      anon_sym_RPAREN,
    STATE(1033), 1,
      aux_sym_type_declaration_repeat1,
    STATE(1166), 2,
      sym_type_alias,
      sym_type_spec,
  [57415] = 5,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2318), 1,
      aux_sym_source_file_token1,
    ACTIONS(2322), 1,
      anon_sym_RBRACE,
    STATE(1024), 1,
      aux_sym_field_declaration_list_repeat1,
    ACTIONS(2320), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [57432] = 5,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(1118), 1,
      anon_sym_LBRACE,
    ACTIONS(1901), 1,
      aux_sym_source_file_token1,
    STATE(1207), 1,
      sym_block,
    ACTIONS(1903), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [57449] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1601), 1,
      anon_sym_LPAREN,
    ACTIONS(1779), 1,
      anon_sym_LBRACK,
    ACTIONS(2324), 1,
      anon_sym_LBRACE,
    STATE(462), 1,
      sym_literal_value,
    STATE(866), 1,
      sym_type_arguments,
  [57468] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2326), 1,
      aux_sym_source_file_token1,
    ACTIONS(2328), 3,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
  [57480] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2330), 1,
      sym_identifier,
    ACTIONS(2332), 1,
      anon_sym_RPAREN,
    STATE(1064), 1,
      aux_sym_const_declaration_repeat1,
    STATE(1183), 1,
      sym_const_spec,
  [57496] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2334), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(2336), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [57508] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2338), 1,
      sym_identifier,
    ACTIONS(2340), 1,
      anon_sym_RPAREN,
    STATE(1136), 1,
      aux_sym_var_spec_list_repeat1,
    STATE(1247), 1,
      sym_var_spec,
  [57524] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2342), 1,
      aux_sym_source_file_token1,
    ACTIONS(2274), 3,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RPAREN,
  [57536] = 4,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2346), 1,
      anon_sym_DQUOTE2,
    STATE(1070), 1,
      aux_sym_interpreted_string_literal_repeat1,
    ACTIONS(2344), 2,
      aux_sym_interpreted_string_literal_token1,
      sym_escape_sequence,
  [57550] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(861), 1,
      anon_sym_LBRACE,
    STATE(351), 1,
      sym_literal_value,
    ACTIONS(1601), 2,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
  [57564] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2348), 1,
      aux_sym_source_file_token1,
    ACTIONS(2350), 3,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
  [57576] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2352), 1,
      sym_identifier,
    ACTIONS(2355), 1,
      anon_sym_RPAREN,
    STATE(1064), 1,
      aux_sym_const_declaration_repeat1,
    STATE(1183), 1,
      sym_const_spec,
  [57592] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2357), 1,
      aux_sym_source_file_token1,
    ACTIONS(2359), 3,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RPAREN,
  [57604] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2361), 1,
      aux_sym_source_file_token1,
    ACTIONS(2363), 3,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RPAREN,
  [57616] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2365), 1,
      anon_sym_COMMA,
    ACTIONS(2367), 1,
      anon_sym_RBRACE,
    ACTIONS(2369), 1,
      anon_sym_COLON,
    STATE(1153), 1,
      aux_sym_literal_value_repeat1,
  [57632] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2371), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(2373), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [57644] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(69), 1,
      anon_sym_BQUOTE,
    ACTIONS(71), 1,
      anon_sym_DQUOTE,
    STATE(1065), 2,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [57658] = 4,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2377), 1,
      anon_sym_DQUOTE2,
    STATE(1115), 1,
      aux_sym_interpreted_string_literal_repeat1,
    ACTIONS(2375), 2,
      aux_sym_interpreted_string_literal_token1,
      sym_escape_sequence,
  [57672] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(69), 1,
      anon_sym_BQUOTE,
    ACTIONS(71), 1,
      anon_sym_DQUOTE,
    STATE(1066), 2,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [57686] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2348), 1,
      aux_sym_source_file_token1,
    ACTIONS(2350), 3,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
  [57698] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1274), 1,
      anon_sym_LBRACE,
    STATE(354), 1,
      sym_block,
    ACTIONS(1722), 2,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
  [57712] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2369), 1,
      anon_sym_COLON,
    ACTIONS(2379), 1,
      anon_sym_COMMA,
    ACTIONS(2381), 1,
      anon_sym_RBRACE,
    STATE(1249), 1,
      aux_sym_literal_value_repeat1,
  [57728] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2369), 1,
      anon_sym_COLON,
    ACTIONS(2383), 1,
      anon_sym_COMMA,
    ACTIONS(2385), 1,
      anon_sym_RBRACE,
    STATE(1189), 1,
      aux_sym_literal_value_repeat1,
  [57744] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2146), 1,
      anon_sym_LPAREN,
    ACTIONS(2387), 1,
      anon_sym_LBRACK,
    STATE(454), 1,
      sym_parameter_list,
    STATE(1287), 1,
      sym_type_parameter_list,
  [57760] = 4,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2391), 1,
      anon_sym_DQUOTE2,
    STATE(1078), 1,
      aux_sym_interpreted_string_literal_repeat1,
    ACTIONS(2389), 2,
      aux_sym_interpreted_string_literal_token1,
      sym_escape_sequence,
  [57774] = 4,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2393), 1,
      anon_sym_DQUOTE2,
    STATE(1115), 1,
      aux_sym_interpreted_string_literal_repeat1,
    ACTIONS(2375), 2,
      aux_sym_interpreted_string_literal_token1,
      sym_escape_sequence,
  [57788] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2173), 1,
      anon_sym_PIPE,
    STATE(1098), 1,
      aux_sym_type_elem_repeat1,
    ACTIONS(1909), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
  [57802] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1931), 1,
      anon_sym_LPAREN,
    ACTIONS(2387), 1,
      anon_sym_LBRACK,
    STATE(497), 1,
      sym_parameter_list,
    STATE(1285), 1,
      sym_type_parameter_list,
  [57818] = 4,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2397), 1,
      anon_sym_DQUOTE2,
    STATE(1132), 1,
      aux_sym_interpreted_string_literal_repeat1,
    ACTIONS(2395), 2,
      aux_sym_interpreted_string_literal_token1,
      sym_escape_sequence,
  [57832] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2324), 1,
      anon_sym_LBRACE,
    STATE(462), 1,
      sym_literal_value,
    ACTIONS(1601), 2,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
  [57846] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2330), 1,
      sym_identifier,
    ACTIONS(2399), 1,
      anon_sym_RPAREN,
    STATE(1057), 1,
      aux_sym_const_declaration_repeat1,
    STATE(1183), 1,
      sym_const_spec,
  [57862] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(668), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(670), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [57874] = 4,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2401), 1,
      anon_sym_DQUOTE2,
    STATE(1115), 1,
      aux_sym_interpreted_string_literal_repeat1,
    ACTIONS(2375), 2,
      aux_sym_interpreted_string_literal_token1,
      sym_escape_sequence,
  [57888] = 4,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2403), 1,
      ts_builtin_sym_end,
    ACTIONS(2405), 1,
      aux_sym_source_file_token1,
    ACTIONS(2407), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [57902] = 4,
    ACTIONS(184), 1,
      ts_builtin_sym_end,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2405), 1,
      aux_sym_source_file_token1,
    ACTIONS(2407), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [57916] = 4,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2411), 1,
      anon_sym_DQUOTE2,
    STATE(1095), 1,
      aux_sym_interpreted_string_literal_repeat1,
    ACTIONS(2409), 2,
      aux_sym_interpreted_string_literal_token1,
      sym_escape_sequence,
  [57930] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1090), 1,
      anon_sym_LBRACE,
    STATE(569), 1,
      sym_literal_value,
    ACTIONS(1601), 2,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
  [57944] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(682), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(684), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [57956] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2357), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(2359), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [57968] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2413), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(2415), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [57980] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2238), 1,
      sym_identifier,
    ACTIONS(2417), 1,
      anon_sym_LPAREN,
    STATE(1011), 2,
      sym_type_alias,
      sym_type_spec,
  [57994] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(664), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(666), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [58006] = 4,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2419), 1,
      anon_sym_DQUOTE2,
    STATE(1115), 1,
      aux_sym_interpreted_string_literal_repeat1,
    ACTIONS(2375), 2,
      aux_sym_interpreted_string_literal_token1,
      sym_escape_sequence,
  [58020] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1368), 1,
      anon_sym_LBRACE,
    STATE(543), 1,
      sym_block,
    ACTIONS(1722), 2,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
  [58034] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2369), 1,
      anon_sym_COLON,
    ACTIONS(2421), 1,
      anon_sym_COMMA,
    ACTIONS(2423), 1,
      anon_sym_RBRACE,
    STATE(1220), 1,
      aux_sym_literal_value_repeat1,
  [58050] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2425), 1,
      anon_sym_PIPE,
    STATE(1098), 1,
      aux_sym_type_elem_repeat1,
    ACTIONS(2162), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
  [58064] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym_LBRACE,
    ACTIONS(2428), 1,
      anon_sym_if,
    STATE(1020), 2,
      sym_block,
      sym_if_statement,
  [58078] = 4,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2432), 1,
      anon_sym_DQUOTE2,
    STATE(1121), 1,
      aux_sym_interpreted_string_literal_repeat1,
    ACTIONS(2430), 2,
      aux_sym_interpreted_string_literal_token1,
      sym_escape_sequence,
  [58092] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2434), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(2436), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [58104] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2438), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(2440), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [58116] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2369), 1,
      anon_sym_COLON,
    ACTIONS(2442), 1,
      anon_sym_COMMA,
    ACTIONS(2444), 1,
      anon_sym_RBRACE,
    STATE(1145), 1,
      aux_sym_literal_value_repeat1,
  [58132] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2446), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(2448), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [58144] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym_LBRACE,
    ACTIONS(2428), 1,
      anon_sym_if,
    STATE(927), 2,
      sym_block,
      sym_if_statement,
  [58158] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2450), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(2452), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [58170] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2454), 1,
      aux_sym_source_file_token1,
    ACTIONS(2456), 3,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
  [58182] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2458), 1,
      aux_sym_source_file_token1,
    ACTIONS(2460), 3,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
  [58194] = 4,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2464), 1,
      anon_sym_DQUOTE2,
    STATE(1110), 1,
      aux_sym_interpreted_string_literal_repeat1,
    ACTIONS(2462), 2,
      aux_sym_interpreted_string_literal_token1,
      sym_escape_sequence,
  [58208] = 4,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2466), 1,
      anon_sym_DQUOTE2,
    STATE(1115), 1,
      aux_sym_interpreted_string_literal_repeat1,
    ACTIONS(2375), 2,
      aux_sym_interpreted_string_literal_token1,
      sym_escape_sequence,
  [58222] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2468), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(2470), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [58234] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2472), 1,
      aux_sym_source_file_token1,
    ACTIONS(2474), 3,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
  [58246] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2476), 1,
      aux_sym_source_file_token1,
    ACTIONS(2478), 3,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RPAREN,
  [58258] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym_LBRACE,
    STATE(261), 1,
      sym_block,
    ACTIONS(1722), 2,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
  [58272] = 4,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2483), 1,
      anon_sym_DQUOTE2,
    STATE(1115), 1,
      aux_sym_interpreted_string_literal_repeat1,
    ACTIONS(2480), 2,
      aux_sym_interpreted_string_literal_token1,
      sym_escape_sequence,
  [58286] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2485), 1,
      sym_identifier,
    ACTIONS(2488), 1,
      anon_sym_RPAREN,
    STATE(1116), 1,
      aux_sym_var_spec_list_repeat1,
    STATE(1247), 1,
      sym_var_spec,
  [58302] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2490), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(2492), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [58314] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1738), 1,
      anon_sym_BQUOTE,
    ACTIONS(1740), 1,
      anon_sym_DQUOTE,
    STATE(1119), 2,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [58328] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2361), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(2363), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [58340] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2494), 1,
      aux_sym_source_file_token1,
    ACTIONS(2288), 3,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
  [58352] = 4,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2496), 1,
      anon_sym_DQUOTE2,
    STATE(1115), 1,
      aux_sym_interpreted_string_literal_repeat1,
    ACTIONS(2375), 2,
      aux_sym_interpreted_string_literal_token1,
      sym_escape_sequence,
  [58366] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2476), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(2478), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [58378] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2498), 1,
      aux_sym_source_file_token1,
    ACTIONS(2500), 3,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
  [58390] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2348), 1,
      aux_sym_source_file_token1,
    ACTIONS(2350), 3,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
  [58402] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2173), 1,
      anon_sym_PIPE,
    STATE(1079), 1,
      aux_sym_type_elem_repeat1,
    ACTIONS(1997), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
  [58416] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2348), 1,
      aux_sym_source_file_token1,
    ACTIONS(2350), 3,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
  [58428] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2502), 1,
      aux_sym_source_file_token1,
    ACTIONS(2504), 3,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
  [58440] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1738), 1,
      anon_sym_BQUOTE,
    ACTIONS(1740), 1,
      anon_sym_DQUOTE,
    STATE(1091), 2,
      sym_raw_string_literal,
      sym_interpreted_string_literal,
  [58454] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1324), 1,
      anon_sym_LBRACE,
    STATE(314), 1,
      sym_block,
    ACTIONS(1722), 2,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
  [58468] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2369), 1,
      anon_sym_COLON,
    ACTIONS(2506), 1,
      anon_sym_COMMA,
    ACTIONS(2508), 1,
      anon_sym_RBRACE,
    STATE(1178), 1,
      aux_sym_literal_value_repeat1,
  [58484] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2199), 1,
      anon_sym_LBRACE,
    STATE(264), 1,
      sym_literal_value,
    ACTIONS(1601), 2,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
  [58498] = 4,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2510), 1,
      anon_sym_DQUOTE2,
    STATE(1115), 1,
      aux_sym_interpreted_string_literal_repeat1,
    ACTIONS(2375), 2,
      aux_sym_interpreted_string_literal_token1,
      sym_escape_sequence,
  [58512] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2512), 1,
      aux_sym_source_file_token1,
    ACTIONS(2314), 3,
      anon_sym_SEMI,
      anon_sym_NULL,
      anon_sym_RBRACE,
  [58524] = 4,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2516), 1,
      anon_sym_DQUOTE2,
    STATE(1085), 1,
      aux_sym_interpreted_string_literal_repeat1,
    ACTIONS(2514), 2,
      aux_sym_interpreted_string_literal_token1,
      sym_escape_sequence,
  [58538] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2518), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(2520), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [58550] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2338), 1,
      sym_identifier,
    ACTIONS(2522), 1,
      anon_sym_RPAREN,
    STATE(1116), 1,
      aux_sym_var_spec_list_repeat1,
    STATE(1247), 1,
      sym_var_spec,
  [58566] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2524), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(2526), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [58578] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2338), 1,
      sym_identifier,
    ACTIONS(2528), 1,
      anon_sym_LPAREN,
    STATE(990), 2,
      sym_var_spec,
      sym_var_spec_list,
  [58592] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2530), 1,
      anon_sym_COMMA,
    STATE(1139), 1,
      aux_sym_type_case_repeat1,
    ACTIONS(2533), 2,
      anon_sym_RBRACK,
      anon_sym_COLON,
  [58606] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2535), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(2537), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [58618] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1288), 1,
      anon_sym_LBRACE,
    STATE(470), 1,
      sym_block,
    ACTIONS(1722), 2,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
  [58632] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(672), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(674), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [58644] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(676), 2,
      ts_builtin_sym_end,
      aux_sym_source_file_token1,
    ACTIONS(678), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [58656] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2207), 1,
      anon_sym_LBRACE,
    STATE(325), 1,
      sym_literal_value,
    ACTIONS(1601), 2,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
  [58670] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(354), 1,
      anon_sym_RBRACE,
    ACTIONS(2539), 1,
      anon_sym_COMMA,
    STATE(1198), 1,
      aux_sym_literal_value_repeat1,
  [58683] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1218), 1,
      anon_sym_RPAREN,
    ACTIONS(2541), 1,
      anon_sym_COMMA,
    STATE(1208), 1,
      aux_sym_parameter_list_repeat1,
  [58696] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2524), 1,
      aux_sym_source_file_token1,
    ACTIONS(2526), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [58707] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1009), 1,
      anon_sym_LPAREN,
    ACTIONS(2543), 1,
      sym_identifier,
    STATE(612), 1,
      sym_parameter_list,
  [58720] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(480), 1,
      anon_sym_RPAREN,
    ACTIONS(2545), 1,
      anon_sym_COMMA,
    STATE(1173), 1,
      aux_sym_expression_list_repeat1,
  [58733] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2369), 1,
      anon_sym_COLON,
    ACTIONS(2547), 2,
      anon_sym_COMMA,
      anon_sym_RBRACE,
  [58744] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1300), 1,
      anon_sym_RBRACK,
    ACTIONS(2549), 1,
      anon_sym_COMMA,
    STATE(1205), 1,
      aux_sym_type_arguments_repeat1,
  [58757] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(478), 1,
      anon_sym_RPAREN,
    ACTIONS(2551), 1,
      anon_sym_COMMA,
    STATE(1173), 1,
      aux_sym_expression_list_repeat1,
  [58770] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(364), 1,
      anon_sym_RBRACE,
    ACTIONS(2553), 1,
      anon_sym_COMMA,
    STATE(1198), 1,
      aux_sym_literal_value_repeat1,
  [58783] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1448), 1,
      anon_sym_RBRACK,
    ACTIONS(2555), 1,
      anon_sym_COMMA,
    STATE(1139), 1,
      aux_sym_type_case_repeat1,
  [58796] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(442), 1,
      anon_sym_RPAREN,
    ACTIONS(2557), 1,
      anon_sym_COMMA,
    STATE(1157), 1,
      aux_sym_argument_list_repeat1,
  [58809] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1174), 1,
      anon_sym_RPAREN,
    ACTIONS(2559), 1,
      anon_sym_COMMA,
    STATE(1208), 1,
      aux_sym_parameter_list_repeat1,
  [58822] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1254), 1,
      anon_sym_RPAREN,
    ACTIONS(2561), 1,
      anon_sym_COMMA,
    STATE(1157), 1,
      aux_sym_argument_list_repeat1,
  [58835] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1232), 1,
      anon_sym_COMMA,
    ACTIONS(1574), 1,
      anon_sym_LBRACE,
    STATE(1234), 1,
      aux_sym_expression_list_repeat1,
  [58848] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2371), 1,
      aux_sym_source_file_token1,
    ACTIONS(2373), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [58859] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1302), 1,
      anon_sym_RBRACK,
    ACTIONS(2564), 1,
      anon_sym_COMMA,
    STATE(1205), 1,
      aux_sym_type_arguments_repeat1,
  [58872] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2566), 1,
      anon_sym_RPAREN,
    ACTIONS(2568), 1,
      anon_sym_COMMA,
    STATE(1182), 1,
      aux_sym_parameter_list_repeat1,
  [58885] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2535), 1,
      aux_sym_source_file_token1,
    ACTIONS(2537), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [58896] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2446), 1,
      aux_sym_source_file_token1,
    ACTIONS(2448), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [58907] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2570), 1,
      anon_sym_RPAREN,
    ACTIONS(2572), 1,
      anon_sym_COMMA,
    STATE(1186), 1,
      aux_sym_expression_list_repeat1,
  [58920] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2574), 1,
      anon_sym_COMMA,
    ACTIONS(2576), 1,
      anon_sym_RBRACK,
    STATE(1188), 1,
      aux_sym_type_arguments_repeat1,
  [58933] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2578), 1,
      aux_sym_source_file_token1,
    ACTIONS(2580), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [58944] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2383), 1,
      anon_sym_COMMA,
    ACTIONS(2385), 1,
      anon_sym_RBRACE,
    STATE(1189), 1,
      aux_sym_literal_value_repeat1,
  [58957] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2582), 1,
      anon_sym_COMMA,
    ACTIONS(2584), 1,
      anon_sym_RBRACK,
    STATE(1192), 1,
      aux_sym_type_case_repeat1,
  [58970] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1184), 1,
      anon_sym_RPAREN,
    ACTIONS(1186), 1,
      anon_sym_COMMA,
    STATE(1193), 1,
      aux_sym_argument_list_repeat1,
  [58983] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1462), 1,
      anon_sym_RBRACK,
    ACTIONS(2586), 1,
      anon_sym_COMMA,
    STATE(1139), 1,
      aux_sym_type_case_repeat1,
  [58996] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2588), 1,
      anon_sym_COMMA,
    ACTIONS(2590), 1,
      anon_sym_RBRACK,
    STATE(1222), 1,
      aux_sym_type_parameter_list_repeat1,
  [59009] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2592), 1,
      anon_sym_RPAREN,
    ACTIONS(2594), 1,
      anon_sym_COMMA,
    STATE(1177), 1,
      aux_sym_expression_list_repeat1,
  [59022] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(867), 1,
      anon_sym_RPAREN,
    ACTIONS(2596), 1,
      anon_sym_COMMA,
    STATE(1173), 1,
      aux_sym_expression_list_repeat1,
  [59035] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2506), 1,
      anon_sym_COMMA,
    ACTIONS(2508), 1,
      anon_sym_RBRACE,
    STATE(1178), 1,
      aux_sym_literal_value_repeat1,
  [59048] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2599), 1,
      anon_sym_COMMA,
    ACTIONS(2601), 1,
      anon_sym_RBRACK,
    STATE(1179), 1,
      aux_sym_type_case_repeat1,
  [59061] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1222), 1,
      anon_sym_RPAREN,
    ACTIONS(1224), 1,
      anon_sym_COMMA,
    STATE(1180), 1,
      aux_sym_argument_list_repeat1,
  [59074] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(494), 1,
      anon_sym_RPAREN,
    ACTIONS(2603), 1,
      anon_sym_COMMA,
    STATE(1173), 1,
      aux_sym_expression_list_repeat1,
  [59087] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(356), 1,
      anon_sym_RBRACE,
    ACTIONS(2605), 1,
      anon_sym_COMMA,
    STATE(1198), 1,
      aux_sym_literal_value_repeat1,
  [59100] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1464), 1,
      anon_sym_RBRACK,
    ACTIONS(2607), 1,
      anon_sym_COMMA,
    STATE(1139), 1,
      aux_sym_type_case_repeat1,
  [59113] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(450), 1,
      anon_sym_RPAREN,
    ACTIONS(2609), 1,
      anon_sym_COMMA,
    STATE(1157), 1,
      aux_sym_argument_list_repeat1,
  [59126] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(444), 1,
      anon_sym_RPAREN,
    ACTIONS(2611), 1,
      anon_sym_COMMA,
    STATE(1157), 1,
      aux_sym_argument_list_repeat1,
  [59139] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1188), 1,
      anon_sym_RPAREN,
    ACTIONS(2613), 1,
      anon_sym_COMMA,
    STATE(1208), 1,
      aux_sym_parameter_list_repeat1,
  [59152] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2615), 1,
      aux_sym_source_file_token1,
    ACTIONS(2617), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [59163] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2619), 3,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [59172] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2468), 1,
      aux_sym_source_file_token1,
    ACTIONS(2470), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [59183] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(522), 1,
      anon_sym_RPAREN,
    ACTIONS(2621), 1,
      anon_sym_COMMA,
    STATE(1173), 1,
      aux_sym_expression_list_repeat1,
  [59196] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2623), 1,
      anon_sym_RPAREN,
    ACTIONS(2625), 1,
      anon_sym_COMMA,
    STATE(1156), 1,
      aux_sym_parameter_list_repeat1,
  [59209] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1308), 1,
      anon_sym_RBRACK,
    ACTIONS(2627), 1,
      anon_sym_COMMA,
    STATE(1205), 1,
      aux_sym_type_arguments_repeat1,
  [59222] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(350), 1,
      anon_sym_RBRACE,
    ACTIONS(2629), 1,
      anon_sym_COMMA,
    STATE(1198), 1,
      aux_sym_literal_value_repeat1,
  [59235] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2413), 1,
      aux_sym_source_file_token1,
    ACTIONS(2415), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [59246] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(214), 3,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [59255] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1428), 1,
      anon_sym_RBRACK,
    ACTIONS(2631), 1,
      anon_sym_COMMA,
    STATE(1139), 1,
      aux_sym_type_case_repeat1,
  [59268] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(430), 1,
      anon_sym_RPAREN,
    ACTIONS(2633), 1,
      anon_sym_COMMA,
    STATE(1157), 1,
      aux_sym_argument_list_repeat1,
  [59281] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2635), 1,
      anon_sym_RPAREN,
    ACTIONS(2637), 1,
      anon_sym_COMMA,
    STATE(1200), 1,
      aux_sym_expression_list_repeat1,
  [59294] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2533), 3,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
  [59303] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2379), 1,
      anon_sym_COMMA,
    ACTIONS(2381), 1,
      anon_sym_RBRACE,
    STATE(1249), 1,
      aux_sym_literal_value_repeat1,
  [59316] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2639), 1,
      anon_sym_COMMA,
    ACTIONS(2641), 1,
      anon_sym_RBRACK,
    STATE(1202), 1,
      aux_sym_type_case_repeat1,
  [59329] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2547), 1,
      anon_sym_RBRACE,
    ACTIONS(2643), 1,
      anon_sym_COMMA,
    STATE(1198), 1,
      aux_sym_literal_value_repeat1,
  [59342] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1226), 1,
      anon_sym_RPAREN,
    ACTIONS(1228), 1,
      anon_sym_COMMA,
    STATE(1203), 1,
      aux_sym_argument_list_repeat1,
  [59355] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(520), 1,
      anon_sym_RPAREN,
    ACTIONS(2646), 1,
      anon_sym_COMMA,
    STATE(1173), 1,
      aux_sym_expression_list_repeat1,
  [59368] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1214), 1,
      anon_sym_RPAREN,
    ACTIONS(1216), 1,
      anon_sym_COMMA,
    STATE(1155), 1,
      aux_sym_argument_list_repeat1,
  [59381] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1478), 1,
      anon_sym_RBRACK,
    ACTIONS(2648), 1,
      anon_sym_COMMA,
    STATE(1139), 1,
      aux_sym_type_case_repeat1,
  [59394] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(456), 1,
      anon_sym_RPAREN,
    ACTIONS(2650), 1,
      anon_sym_COMMA,
    STATE(1157), 1,
      aux_sym_argument_list_repeat1,
  [59407] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2334), 1,
      aux_sym_source_file_token1,
    ACTIONS(2336), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [59418] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2652), 1,
      anon_sym_COMMA,
    ACTIONS(2655), 1,
      anon_sym_RBRACK,
    STATE(1205), 1,
      aux_sym_type_arguments_repeat1,
  [59431] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2434), 1,
      aux_sym_source_file_token1,
    ACTIONS(2436), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [59442] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2438), 1,
      aux_sym_source_file_token1,
    ACTIONS(2440), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [59453] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2657), 1,
      anon_sym_RPAREN,
    ACTIONS(2659), 1,
      anon_sym_COMMA,
    STATE(1208), 1,
      aux_sym_parameter_list_repeat1,
  [59466] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2662), 1,
      anon_sym_RPAREN,
    ACTIONS(2664), 1,
      anon_sym_COMMA,
    STATE(1152), 1,
      aux_sym_expression_list_repeat1,
  [59479] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2330), 1,
      sym_identifier,
    ACTIONS(2666), 1,
      anon_sym_LPAREN,
    STATE(986), 1,
      sym_const_spec,
  [59492] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2668), 1,
      anon_sym_RPAREN,
    ACTIONS(2670), 1,
      anon_sym_COMMA,
    STATE(1218), 1,
      aux_sym_expression_list_repeat1,
  [59505] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2450), 1,
      aux_sym_source_file_token1,
    ACTIONS(2452), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [59516] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2421), 1,
      anon_sym_COMMA,
    ACTIONS(2423), 1,
      anon_sym_RBRACE,
    STATE(1220), 1,
      aux_sym_literal_value_repeat1,
  [59529] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2672), 1,
      anon_sym_COMMA,
    ACTIONS(2674), 1,
      anon_sym_RBRACK,
    STATE(1221), 1,
      aux_sym_type_case_repeat1,
  [59542] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2676), 1,
      anon_sym_COMMA,
    ACTIONS(2678), 1,
      anon_sym_RBRACK,
    STATE(1160), 1,
      aux_sym_type_arguments_repeat1,
  [59555] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1176), 1,
      anon_sym_RPAREN,
    ACTIONS(1178), 1,
      anon_sym_COMMA,
    STATE(1223), 1,
      aux_sym_argument_list_repeat1,
  [59568] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2680), 1,
      sym_identifier,
    ACTIONS(2682), 1,
      anon_sym_RBRACK,
    STATE(1323), 1,
      sym_type_parameter_declaration,
  [59581] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(532), 1,
      anon_sym_RPAREN,
    ACTIONS(2684), 1,
      anon_sym_COMMA,
    STATE(1173), 1,
      aux_sym_expression_list_repeat1,
  [59594] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2490), 1,
      aux_sym_source_file_token1,
    ACTIONS(2492), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [59605] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(372), 1,
      anon_sym_RBRACE,
    ACTIONS(2686), 1,
      anon_sym_COMMA,
    STATE(1198), 1,
      aux_sym_literal_value_repeat1,
  [59618] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1492), 1,
      anon_sym_RBRACK,
    ACTIONS(2688), 1,
      anon_sym_COMMA,
    STATE(1139), 1,
      aux_sym_type_case_repeat1,
  [59631] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2682), 1,
      anon_sym_RBRACK,
    ACTIONS(2690), 1,
      anon_sym_COMMA,
    STATE(1241), 1,
      aux_sym_type_parameter_list_repeat1,
  [59644] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(464), 1,
      anon_sym_RPAREN,
    ACTIONS(2692), 1,
      anon_sym_COMMA,
    STATE(1157), 1,
      aux_sym_argument_list_repeat1,
  [59657] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1248), 3,
      anon_sym_COMMA,
      anon_sym_RBRACE,
      anon_sym_COLON,
  [59666] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2694), 1,
      anon_sym_COMMA,
    ACTIONS(2696), 1,
      anon_sym_COLON,
    STATE(1139), 1,
      aux_sym_type_case_repeat1,
  [59679] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2162), 3,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PIPE,
  [59688] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2442), 1,
      anon_sym_COMMA,
    ACTIONS(2444), 1,
      anon_sym_RBRACE,
    STATE(1145), 1,
      aux_sym_literal_value_repeat1,
  [59701] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2698), 3,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [59710] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2518), 1,
      aux_sym_source_file_token1,
    ACTIONS(2520), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [59721] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1009), 1,
      anon_sym_LPAREN,
    ACTIONS(2700), 1,
      sym_identifier,
    STATE(642), 1,
      sym_parameter_list,
  [59734] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2405), 1,
      aux_sym_source_file_token1,
    ACTIONS(2407), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [59745] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2702), 1,
      anon_sym_COMMA,
    ACTIONS(2704), 1,
      anon_sym_RBRACK,
    STATE(1170), 1,
      aux_sym_type_case_repeat1,
  [59758] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2694), 1,
      anon_sym_COMMA,
    ACTIONS(2706), 1,
      anon_sym_COLON,
    STATE(1225), 1,
      aux_sym_type_case_repeat1,
  [59771] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(867), 1,
      anon_sym_LBRACE,
    ACTIONS(2708), 1,
      anon_sym_COMMA,
    STATE(1234), 1,
      aux_sym_expression_list_repeat1,
  [59784] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2711), 3,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [59793] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1152), 1,
      anon_sym_RPAREN,
    ACTIONS(1154), 1,
      anon_sym_COMMA,
    STATE(1181), 1,
      aux_sym_argument_list_repeat1,
  [59806] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2713), 3,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [59815] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2169), 1,
      anon_sym_LPAREN,
    ACTIONS(2171), 1,
      anon_sym_LBRACK,
    ACTIONS(2715), 1,
      anon_sym_RPAREN,
  [59828] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2717), 1,
      anon_sym_RPAREN,
    ACTIONS(2719), 1,
      anon_sym_COMMA,
    STATE(1146), 1,
      aux_sym_parameter_list_repeat1,
  [59841] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2680), 1,
      sym_identifier,
    ACTIONS(2721), 1,
      anon_sym_RBRACK,
    STATE(1323), 1,
      sym_type_parameter_declaration,
  [59854] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2723), 1,
      anon_sym_COMMA,
    ACTIONS(2726), 1,
      anon_sym_RBRACK,
    STATE(1241), 1,
      aux_sym_type_parameter_list_repeat1,
  [59867] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2728), 1,
      anon_sym_RPAREN,
    ACTIONS(2730), 1,
      anon_sym_COMMA,
    STATE(1149), 1,
      aux_sym_expression_list_repeat1,
  [59880] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2732), 1,
      anon_sym_COMMA,
    ACTIONS(2734), 1,
      anon_sym_RBRACK,
    STATE(1151), 1,
      aux_sym_type_arguments_repeat1,
  [59893] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2736), 3,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [59902] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2365), 1,
      anon_sym_COMMA,
    ACTIONS(2367), 1,
      anon_sym_RBRACE,
    STATE(1153), 1,
      aux_sym_literal_value_repeat1,
  [59915] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2738), 1,
      anon_sym_COMMA,
    ACTIONS(2740), 1,
      anon_sym_RBRACK,
    STATE(1154), 1,
      aux_sym_type_case_repeat1,
  [59928] = 3,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2742), 1,
      aux_sym_source_file_token1,
    ACTIONS(2744), 2,
      anon_sym_SEMI,
      anon_sym_NULL,
  [59939] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2746), 3,
      anon_sym_RBRACE,
      anon_sym_case,
      anon_sym_default,
  [59948] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(360), 1,
      anon_sym_RBRACE,
    ACTIONS(2748), 1,
      anon_sym_COMMA,
    STATE(1198), 1,
      aux_sym_literal_value_repeat1,
  [59961] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1927), 2,
      anon_sym_SEMI,
      anon_sym_LBRACE,
  [59969] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2750), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
  [59977] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1094), 1,
      anon_sym_LPAREN,
    STATE(542), 1,
      sym_argument_list,
  [59987] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2547), 2,
      anon_sym_COMMA,
      anon_sym_RBRACE,
  [59995] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2752), 1,
      anon_sym_LBRACE,
    STATE(884), 1,
      sym_field_declaration_list,
  [60005] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2754), 1,
      sym_identifier,
    ACTIONS(2756), 1,
      anon_sym_LPAREN,
  [60015] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2146), 1,
      anon_sym_LPAREN,
    STATE(498), 1,
      sym_parameter_list,
  [60025] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2758), 1,
      sym_identifier,
    ACTIONS(2760), 1,
      anon_sym_LPAREN,
  [60035] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2762), 1,
      anon_sym_LBRACE,
    STATE(925), 1,
      sym_field_declaration_list,
  [60045] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1009), 1,
      anon_sym_LPAREN,
    STATE(452), 1,
      sym_parameter_list,
  [60055] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2764), 1,
      sym_identifier,
    ACTIONS(2766), 1,
      anon_sym_LPAREN,
  [60065] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2655), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
  [60073] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(865), 1,
      anon_sym_LPAREN,
    STATE(353), 1,
      sym_argument_list,
  [60083] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2768), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [60091] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2770), 2,
      anon_sym_BQUOTE,
      anon_sym_DQUOTE,
  [60099] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1971), 2,
      anon_sym_SEMI,
      anon_sym_LBRACE,
  [60107] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2772), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
  [60115] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2774), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [60123] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(696), 1,
      anon_sym_LPAREN,
    STATE(316), 1,
      sym_argument_list,
  [60133] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2776), 2,
      anon_sym_COMMA,
      anon_sym_RBRACE,
  [60141] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2207), 1,
      anon_sym_LBRACE,
    STATE(325), 1,
      sym_literal_value,
  [60151] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym_LBRACE,
    STATE(992), 1,
      sym_block,
  [60161] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2355), 2,
      anon_sym_RPAREN,
      sym_identifier,
  [60169] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2778), 2,
      anon_sym_EQ,
      anon_sym_COLON_EQ,
  [60177] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(312), 1,
      anon_sym_LBRACE,
    STATE(423), 1,
      sym_literal_value,
  [60187] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2148), 2,
      anon_sym_SEMI,
      anon_sym_LBRACE,
  [60195] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2680), 1,
      sym_identifier,
    STATE(1171), 1,
      sym_type_parameter_declaration,
  [60205] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2324), 1,
      anon_sym_LBRACE,
    STATE(462), 1,
      sym_literal_value,
  [60215] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2780), 1,
      sym_identifier,
    ACTIONS(2782), 1,
      anon_sym_LPAREN,
  [60225] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2784), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [60233] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2786), 1,
      anon_sym_LBRACK,
    STATE(819), 1,
      sym_type_arguments,
  [60243] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2780), 1,
      sym_identifier,
    ACTIONS(2788), 1,
      anon_sym_LPAREN,
  [60253] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2152), 2,
      anon_sym_SEMI,
      anon_sym_LBRACE,
  [60261] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2488), 2,
      anon_sym_RPAREN,
      sym_identifier,
  [60269] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1931), 1,
      anon_sym_LPAREN,
    STATE(388), 1,
      sym_parameter_list,
  [60279] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1931), 1,
      anon_sym_LPAREN,
    STATE(461), 1,
      sym_parameter_list,
  [60289] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1009), 1,
      anon_sym_LPAREN,
    STATE(601), 1,
      sym_parameter_list,
  [60299] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2146), 1,
      anon_sym_LPAREN,
    STATE(457), 1,
      sym_parameter_list,
  [60309] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(945), 1,
      anon_sym_LPAREN,
    STATE(427), 1,
      sym_argument_list,
  [60319] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2680), 1,
      sym_identifier,
    STATE(1323), 1,
      sym_type_parameter_declaration,
  [60329] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1254), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [60337] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2790), 1,
      anon_sym_LPAREN,
    STATE(257), 1,
      sym_argument_list,
  [60347] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2792), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [60355] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2199), 1,
      anon_sym_LBRACE,
    STATE(264), 1,
      sym_literal_value,
  [60365] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1009), 1,
      anon_sym_LPAREN,
    STATE(607), 1,
      sym_parameter_list,
  [60375] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2794), 1,
      sym_identifier,
    ACTIONS(2796), 1,
      anon_sym_LPAREN,
  [60385] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1009), 1,
      anon_sym_LPAREN,
    STATE(442), 1,
      sym_parameter_list,
  [60395] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2798), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [60403] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1961), 2,
      anon_sym_SEMI,
      anon_sym_LBRACE,
  [60411] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2800), 1,
      anon_sym_LPAREN,
    STATE(469), 1,
      sym_argument_list,
  [60421] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2798), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [60429] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2758), 1,
      sym_identifier,
    ACTIONS(2802), 1,
      anon_sym_LPAREN,
  [60439] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1009), 1,
      anon_sym_LPAREN,
    STATE(621), 1,
      sym_parameter_list,
  [60449] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2804), 1,
      anon_sym_LPAREN,
    ACTIONS(2806), 1,
      anon_sym_LBRACK,
  [60459] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2657), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [60467] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1931), 1,
      anon_sym_LPAREN,
    STATE(389), 1,
      sym_parameter_list,
  [60477] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1009), 1,
      anon_sym_LPAREN,
    STATE(446), 1,
      sym_parameter_list,
  [60487] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1090), 1,
      anon_sym_LBRACE,
    STATE(569), 1,
      sym_literal_value,
  [60497] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2808), 1,
      anon_sym_LBRACE,
    STATE(831), 1,
      sym_field_declaration_list,
  [60507] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1009), 1,
      anon_sym_LPAREN,
    STATE(599), 1,
      sym_parameter_list,
  [60517] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2780), 1,
      sym_identifier,
    ACTIONS(2810), 1,
      anon_sym_LPAREN,
  [60527] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2146), 1,
      anon_sym_LPAREN,
    STATE(501), 1,
      sym_parameter_list,
  [60537] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2812), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [60545] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1009), 1,
      anon_sym_LPAREN,
    STATE(635), 1,
      sym_parameter_list,
  [60555] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(861), 1,
      anon_sym_LBRACE,
    STATE(351), 1,
      sym_literal_value,
  [60565] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2224), 2,
      anon_sym_RPAREN,
      sym_identifier,
  [60573] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2780), 1,
      sym_identifier,
    ACTIONS(2814), 1,
      anon_sym_LPAREN,
  [60583] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2816), 1,
      sym_identifier,
    ACTIONS(2818), 1,
      anon_sym_LPAREN,
  [60593] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2169), 1,
      anon_sym_LPAREN,
    ACTIONS(2171), 1,
      anon_sym_LBRACK,
  [60603] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2820), 1,
      anon_sym_LPAREN,
    ACTIONS(2822), 1,
      anon_sym_LBRACK,
  [60613] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2824), 1,
      anon_sym_LPAREN,
    ACTIONS(2826), 1,
      anon_sym_LBRACK,
  [60623] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2828), 1,
      anon_sym_LPAREN,
    ACTIONS(2830), 1,
      anon_sym_LBRACK,
  [60633] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2832), 1,
      anon_sym_LPAREN,
    ACTIONS(2834), 1,
      anon_sym_LBRACK,
  [60643] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2726), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
  [60651] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2836), 1,
      anon_sym_LBRACE,
  [60658] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2838), 1,
      anon_sym_RPAREN,
  [60665] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2840), 1,
      anon_sym_BQUOTE,
  [60672] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2842), 1,
      anon_sym_RBRACE,
  [60679] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2844), 1,
      anon_sym_RPAREN,
  [60686] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2846), 1,
      anon_sym_RPAREN,
  [60693] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2848), 1,
      anon_sym_SEMI,
  [60700] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2850), 1,
      anon_sym_RPAREN,
  [60707] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2852), 1,
      anon_sym_RPAREN,
  [60714] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2854), 1,
      anon_sym_COLON,
  [60721] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2856), 1,
      anon_sym_RBRACK,
  [60728] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2858), 1,
      anon_sym_LBRACE,
  [60735] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2860), 1,
      sym_identifier,
  [60742] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2862), 1,
      anon_sym_EQ,
  [60749] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2864), 1,
      anon_sym_RPAREN,
  [60756] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2866), 1,
      sym_identifier,
  [60763] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2623), 1,
      anon_sym_RPAREN,
  [60770] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2868), 1,
      anon_sym_RPAREN,
  [60777] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2870), 1,
      anon_sym_LBRACE,
  [60784] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2566), 1,
      anon_sym_RPAREN,
  [60791] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2872), 1,
      anon_sym_RBRACK,
  [60798] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2874), 1,
      anon_sym_SEMI,
  [60805] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2876), 1,
      sym_identifier,
  [60812] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2878), 1,
      anon_sym_LBRACE,
  [60819] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2880), 1,
      anon_sym_RBRACE,
  [60826] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2882), 1,
      anon_sym_BQUOTE,
  [60833] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2508), 1,
      anon_sym_RBRACE,
  [60840] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2884), 1,
      anon_sym_LBRACE,
  [60847] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2385), 1,
      anon_sym_RBRACE,
  [60854] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2886), 1,
      anon_sym_BQUOTE,
  [60861] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2888), 1,
      anon_sym_COLON,
  [60868] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2890), 1,
      anon_sym_LBRACE,
  [60875] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2892), 1,
      anon_sym_LBRACE,
  [60882] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2894), 1,
      anon_sym_LBRACE,
  [60889] = 2,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2896), 1,
      aux_sym_raw_string_literal_token1,
  [60896] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2898), 1,
      anon_sym_chan,
  [60903] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2900), 1,
      anon_sym_RPAREN,
  [60910] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2902), 1,
      anon_sym_RBRACE,
  [60917] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2904), 1,
      anon_sym_COLON_EQ,
  [60924] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2906), 1,
      ts_builtin_sym_end,
  [60931] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2908), 1,
      sym_identifier,
  [60938] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2910), 1,
      anon_sym_RPAREN,
  [60945] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2912), 1,
      sym_identifier,
  [60952] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2914), 1,
      anon_sym_COLON,
  [60959] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2916), 1,
      anon_sym_BQUOTE,
  [60966] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2715), 1,
      anon_sym_RPAREN,
  [60973] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2918), 1,
      anon_sym_RPAREN,
  [60980] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2920), 1,
      anon_sym_RPAREN,
  [60987] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2922), 1,
      anon_sym_LBRACE,
  [60994] = 2,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2924), 1,
      aux_sym_raw_string_literal_token1,
  [61001] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2926), 1,
      anon_sym_RPAREN,
  [61008] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2928), 1,
      anon_sym_RBRACK,
  [61015] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2444), 1,
      anon_sym_RBRACE,
  [61022] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2930), 1,
      anon_sym_chan,
  [61029] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2932), 1,
      anon_sym_RPAREN,
  [61036] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2934), 1,
      anon_sym_LBRACE,
  [61043] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2936), 1,
      anon_sym_RBRACK,
  [61050] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2717), 1,
      anon_sym_RPAREN,
  [61057] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2938), 1,
      anon_sym_chan,
  [61064] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2940), 1,
      anon_sym_chan,
  [61071] = 2,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2942), 1,
      aux_sym_raw_string_literal_token1,
  [61078] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2944), 1,
      anon_sym_chan,
  [61085] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2946), 1,
      anon_sym_RBRACE,
  [61092] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2948), 1,
      anon_sym_RBRACE,
  [61099] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2950), 1,
      anon_sym_RPAREN,
  [61106] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2952), 1,
      anon_sym_RBRACE,
  [61113] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2954), 1,
      anon_sym_BQUOTE,
  [61120] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2956), 1,
      anon_sym_RBRACK,
  [61127] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2958), 1,
      anon_sym_chan,
  [61134] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2960), 1,
      anon_sym_RPAREN,
  [61141] = 2,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2962), 1,
      aux_sym_raw_string_literal_token1,
  [61148] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2964), 1,
      anon_sym_BQUOTE,
  [61155] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2367), 1,
      anon_sym_RBRACE,
  [61162] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2966), 1,
      sym_identifier,
  [61169] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2968), 1,
      anon_sym_LBRACE,
  [61176] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2970), 1,
      sym_identifier,
  [61183] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2972), 1,
      anon_sym_LBRACK,
  [61190] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2974), 1,
      anon_sym_RBRACE,
  [61197] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2976), 1,
      anon_sym_LBRACE,
  [61204] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2978), 1,
      anon_sym_SEMI,
  [61211] = 2,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2980), 1,
      aux_sym_raw_string_literal_token1,
  [61218] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2982), 1,
      anon_sym_RBRACK,
  [61225] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2984), 1,
      sym_identifier,
  [61232] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2986), 1,
      anon_sym_LBRACE,
  [61239] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2988), 1,
      anon_sym_RPAREN,
  [61246] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2423), 1,
      anon_sym_RBRACE,
  [61253] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2990), 1,
      anon_sym_BQUOTE,
  [61260] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2992), 1,
      anon_sym_LBRACE,
  [61267] = 2,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2994), 1,
      aux_sym_raw_string_literal_token1,
  [61274] = 2,
    ACTIONS(300), 1,
      sym_comment,
    ACTIONS(2996), 1,
      aux_sym_raw_string_literal_token1,
  [61281] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2998), 1,
      anon_sym_LBRACK,
  [61288] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3000), 1,
      anon_sym_RBRACK,
  [61295] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3002), 1,
      anon_sym_LBRACK,
  [61302] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3004), 1,
      sym_identifier,
  [61309] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3006), 1,
      anon_sym_LBRACK,
  [61316] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2381), 1,
      anon_sym_RBRACE,
  [61323] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3008), 1,
      anon_sym_LBRACK,
  [61330] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3010), 1,
      anon_sym_RPAREN,
  [61337] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3012), 1,
      anon_sym_LBRACK,
  [61344] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3014), 1,
      anon_sym_EQ,
  [61351] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3016), 1,
      anon_sym_RPAREN,
};

static const uint32_t ts_small_parse_table_map[] = {
  [SMALL_STATE(29)] = 0,
  [SMALL_STATE(30)] = 129,
  [SMALL_STATE(31)] = 258,
  [SMALL_STATE(32)] = 387,
  [SMALL_STATE(33)] = 516,
  [SMALL_STATE(34)] = 645,
  [SMALL_STATE(35)] = 774,
  [SMALL_STATE(36)] = 900,
  [SMALL_STATE(37)] = 1026,
  [SMALL_STATE(38)] = 1152,
  [SMALL_STATE(39)] = 1278,
  [SMALL_STATE(40)] = 1404,
  [SMALL_STATE(41)] = 1530,
  [SMALL_STATE(42)] = 1656,
  [SMALL_STATE(43)] = 1782,
  [SMALL_STATE(44)] = 1908,
  [SMALL_STATE(45)] = 2034,
  [SMALL_STATE(46)] = 2160,
  [SMALL_STATE(47)] = 2286,
  [SMALL_STATE(48)] = 2409,
  [SMALL_STATE(49)] = 2529,
  [SMALL_STATE(50)] = 2647,
  [SMALL_STATE(51)] = 2764,
  [SMALL_STATE(52)] = 2881,
  [SMALL_STATE(53)] = 2998,
  [SMALL_STATE(54)] = 3115,
  [SMALL_STATE(55)] = 3232,
  [SMALL_STATE(56)] = 3349,
  [SMALL_STATE(57)] = 3466,
  [SMALL_STATE(58)] = 3583,
  [SMALL_STATE(59)] = 3700,
  [SMALL_STATE(60)] = 3817,
  [SMALL_STATE(61)] = 3934,
  [SMALL_STATE(62)] = 4051,
  [SMALL_STATE(63)] = 4168,
  [SMALL_STATE(64)] = 4285,
  [SMALL_STATE(65)] = 4402,
  [SMALL_STATE(66)] = 4519,
  [SMALL_STATE(67)] = 4636,
  [SMALL_STATE(68)] = 4753,
  [SMALL_STATE(69)] = 4870,
  [SMALL_STATE(70)] = 4987,
  [SMALL_STATE(71)] = 5104,
  [SMALL_STATE(72)] = 5221,
  [SMALL_STATE(73)] = 5338,
  [SMALL_STATE(74)] = 5455,
  [SMALL_STATE(75)] = 5572,
  [SMALL_STATE(76)] = 5689,
  [SMALL_STATE(77)] = 5806,
  [SMALL_STATE(78)] = 5923,
  [SMALL_STATE(79)] = 6040,
  [SMALL_STATE(80)] = 6157,
  [SMALL_STATE(81)] = 6274,
  [SMALL_STATE(82)] = 6388,
  [SMALL_STATE(83)] = 6502,
  [SMALL_STATE(84)] = 6616,
  [SMALL_STATE(85)] = 6730,
  [SMALL_STATE(86)] = 6844,
  [SMALL_STATE(87)] = 6958,
  [SMALL_STATE(88)] = 7072,
  [SMALL_STATE(89)] = 7186,
  [SMALL_STATE(90)] = 7300,
  [SMALL_STATE(91)] = 7414,
  [SMALL_STATE(92)] = 7528,
  [SMALL_STATE(93)] = 7642,
  [SMALL_STATE(94)] = 7756,
  [SMALL_STATE(95)] = 7870,
  [SMALL_STATE(96)] = 7984,
  [SMALL_STATE(97)] = 8098,
  [SMALL_STATE(98)] = 8212,
  [SMALL_STATE(99)] = 8326,
  [SMALL_STATE(100)] = 8440,
  [SMALL_STATE(101)] = 8554,
  [SMALL_STATE(102)] = 8668,
  [SMALL_STATE(103)] = 8782,
  [SMALL_STATE(104)] = 8896,
  [SMALL_STATE(105)] = 9010,
  [SMALL_STATE(106)] = 9124,
  [SMALL_STATE(107)] = 9238,
  [SMALL_STATE(108)] = 9352,
  [SMALL_STATE(109)] = 9466,
  [SMALL_STATE(110)] = 9580,
  [SMALL_STATE(111)] = 9694,
  [SMALL_STATE(112)] = 9808,
  [SMALL_STATE(113)] = 9922,
  [SMALL_STATE(114)] = 10036,
  [SMALL_STATE(115)] = 10150,
  [SMALL_STATE(116)] = 10264,
  [SMALL_STATE(117)] = 10378,
  [SMALL_STATE(118)] = 10492,
  [SMALL_STATE(119)] = 10606,
  [SMALL_STATE(120)] = 10720,
  [SMALL_STATE(121)] = 10834,
  [SMALL_STATE(122)] = 10948,
  [SMALL_STATE(123)] = 11062,
  [SMALL_STATE(124)] = 11176,
  [SMALL_STATE(125)] = 11290,
  [SMALL_STATE(126)] = 11404,
  [SMALL_STATE(127)] = 11518,
  [SMALL_STATE(128)] = 11632,
  [SMALL_STATE(129)] = 11746,
  [SMALL_STATE(130)] = 11860,
  [SMALL_STATE(131)] = 11974,
  [SMALL_STATE(132)] = 12085,
  [SMALL_STATE(133)] = 12196,
  [SMALL_STATE(134)] = 12307,
  [SMALL_STATE(135)] = 12418,
  [SMALL_STATE(136)] = 12529,
  [SMALL_STATE(137)] = 12640,
  [SMALL_STATE(138)] = 12751,
  [SMALL_STATE(139)] = 12862,
  [SMALL_STATE(140)] = 12973,
  [SMALL_STATE(141)] = 13084,
  [SMALL_STATE(142)] = 13195,
  [SMALL_STATE(143)] = 13306,
  [SMALL_STATE(144)] = 13417,
  [SMALL_STATE(145)] = 13528,
  [SMALL_STATE(146)] = 13639,
  [SMALL_STATE(147)] = 13750,
  [SMALL_STATE(148)] = 13861,
  [SMALL_STATE(149)] = 13972,
  [SMALL_STATE(150)] = 14083,
  [SMALL_STATE(151)] = 14194,
  [SMALL_STATE(152)] = 14305,
  [SMALL_STATE(153)] = 14416,
  [SMALL_STATE(154)] = 14527,
  [SMALL_STATE(155)] = 14638,
  [SMALL_STATE(156)] = 14749,
  [SMALL_STATE(157)] = 14860,
  [SMALL_STATE(158)] = 14971,
  [SMALL_STATE(159)] = 15082,
  [SMALL_STATE(160)] = 15193,
  [SMALL_STATE(161)] = 15304,
  [SMALL_STATE(162)] = 15415,
  [SMALL_STATE(163)] = 15526,
  [SMALL_STATE(164)] = 15637,
  [SMALL_STATE(165)] = 15748,
  [SMALL_STATE(166)] = 15859,
  [SMALL_STATE(167)] = 15970,
  [SMALL_STATE(168)] = 16081,
  [SMALL_STATE(169)] = 16192,
  [SMALL_STATE(170)] = 16303,
  [SMALL_STATE(171)] = 16414,
  [SMALL_STATE(172)] = 16525,
  [SMALL_STATE(173)] = 16636,
  [SMALL_STATE(174)] = 16747,
  [SMALL_STATE(175)] = 16858,
  [SMALL_STATE(176)] = 16969,
  [SMALL_STATE(177)] = 17080,
  [SMALL_STATE(178)] = 17191,
  [SMALL_STATE(179)] = 17302,
  [SMALL_STATE(180)] = 17413,
  [SMALL_STATE(181)] = 17524,
  [SMALL_STATE(182)] = 17635,
  [SMALL_STATE(183)] = 17746,
  [SMALL_STATE(184)] = 17857,
  [SMALL_STATE(185)] = 17968,
  [SMALL_STATE(186)] = 18079,
  [SMALL_STATE(187)] = 18190,
  [SMALL_STATE(188)] = 18301,
  [SMALL_STATE(189)] = 18412,
  [SMALL_STATE(190)] = 18523,
  [SMALL_STATE(191)] = 18634,
  [SMALL_STATE(192)] = 18745,
  [SMALL_STATE(193)] = 18856,
  [SMALL_STATE(194)] = 18967,
  [SMALL_STATE(195)] = 19078,
  [SMALL_STATE(196)] = 19189,
  [SMALL_STATE(197)] = 19300,
  [SMALL_STATE(198)] = 19411,
  [SMALL_STATE(199)] = 19522,
  [SMALL_STATE(200)] = 19633,
  [SMALL_STATE(201)] = 19744,
  [SMALL_STATE(202)] = 19855,
  [SMALL_STATE(203)] = 19966,
  [SMALL_STATE(204)] = 20077,
  [SMALL_STATE(205)] = 20188,
  [SMALL_STATE(206)] = 20299,
  [SMALL_STATE(207)] = 20410,
  [SMALL_STATE(208)] = 20521,
  [SMALL_STATE(209)] = 20632,
  [SMALL_STATE(210)] = 20743,
  [SMALL_STATE(211)] = 20854,
  [SMALL_STATE(212)] = 20965,
  [SMALL_STATE(213)] = 21076,
  [SMALL_STATE(214)] = 21187,
  [SMALL_STATE(215)] = 21298,
  [SMALL_STATE(216)] = 21409,
  [SMALL_STATE(217)] = 21520,
  [SMALL_STATE(218)] = 21631,
  [SMALL_STATE(219)] = 21742,
  [SMALL_STATE(220)] = 21853,
  [SMALL_STATE(221)] = 21964,
  [SMALL_STATE(222)] = 22075,
  [SMALL_STATE(223)] = 22186,
  [SMALL_STATE(224)] = 22297,
  [SMALL_STATE(225)] = 22408,
  [SMALL_STATE(226)] = 22519,
  [SMALL_STATE(227)] = 22630,
  [SMALL_STATE(228)] = 22741,
  [SMALL_STATE(229)] = 22852,
  [SMALL_STATE(230)] = 22963,
  [SMALL_STATE(231)] = 23074,
  [SMALL_STATE(232)] = 23185,
  [SMALL_STATE(233)] = 23296,
  [SMALL_STATE(234)] = 23407,
  [SMALL_STATE(235)] = 23518,
  [SMALL_STATE(236)] = 23629,
  [SMALL_STATE(237)] = 23729,
  [SMALL_STATE(238)] = 23800,
  [SMALL_STATE(239)] = 23871,
  [SMALL_STATE(240)] = 23939,
  [SMALL_STATE(241)] = 24027,
  [SMALL_STATE(242)] = 24092,
  [SMALL_STATE(243)] = 24161,
  [SMALL_STATE(244)] = 24232,
  [SMALL_STATE(245)] = 24299,
  [SMALL_STATE(246)] = 24364,
  [SMALL_STATE(247)] = 24419,
  [SMALL_STATE(248)] = 24492,
  [SMALL_STATE(249)] = 24546,
  [SMALL_STATE(250)] = 24600,
  [SMALL_STATE(251)] = 24654,
  [SMALL_STATE(252)] = 24708,
  [SMALL_STATE(253)] = 24766,
  [SMALL_STATE(254)] = 24820,
  [SMALL_STATE(255)] = 24873,
  [SMALL_STATE(256)] = 24926,
  [SMALL_STATE(257)] = 25017,
  [SMALL_STATE(258)] = 25070,
  [SMALL_STATE(259)] = 25123,
  [SMALL_STATE(260)] = 25214,
  [SMALL_STATE(261)] = 25267,
  [SMALL_STATE(262)] = 25320,
  [SMALL_STATE(263)] = 25373,
  [SMALL_STATE(264)] = 25426,
  [SMALL_STATE(265)] = 25479,
  [SMALL_STATE(266)] = 25532,
  [SMALL_STATE(267)] = 25585,
  [SMALL_STATE(268)] = 25638,
  [SMALL_STATE(269)] = 25691,
  [SMALL_STATE(270)] = 25744,
  [SMALL_STATE(271)] = 25797,
  [SMALL_STATE(272)] = 25850,
  [SMALL_STATE(273)] = 25903,
  [SMALL_STATE(274)] = 25956,
  [SMALL_STATE(275)] = 26009,
  [SMALL_STATE(276)] = 26062,
  [SMALL_STATE(277)] = 26115,
  [SMALL_STATE(278)] = 26168,
  [SMALL_STATE(279)] = 26221,
  [SMALL_STATE(280)] = 26274,
  [SMALL_STATE(281)] = 26327,
  [SMALL_STATE(282)] = 26380,
  [SMALL_STATE(283)] = 26433,
  [SMALL_STATE(284)] = 26486,
  [SMALL_STATE(285)] = 26539,
  [SMALL_STATE(286)] = 26592,
  [SMALL_STATE(287)] = 26645,
  [SMALL_STATE(288)] = 26698,
  [SMALL_STATE(289)] = 26751,
  [SMALL_STATE(290)] = 26804,
  [SMALL_STATE(291)] = 26892,
  [SMALL_STATE(292)] = 26961,
  [SMALL_STATE(293)] = 27046,
  [SMALL_STATE(294)] = 27107,
  [SMALL_STATE(295)] = 27168,
  [SMALL_STATE(296)] = 27231,
  [SMALL_STATE(297)] = 27296,
  [SMALL_STATE(298)] = 27367,
  [SMALL_STATE(299)] = 27428,
  [SMALL_STATE(300)] = 27482,
  [SMALL_STATE(301)] = 27531,
  [SMALL_STATE(302)] = 27580,
  [SMALL_STATE(303)] = 27629,
  [SMALL_STATE(304)] = 27678,
  [SMALL_STATE(305)] = 27727,
  [SMALL_STATE(306)] = 27776,
  [SMALL_STATE(307)] = 27825,
  [SMALL_STATE(308)] = 27874,
  [SMALL_STATE(309)] = 27923,
  [SMALL_STATE(310)] = 27972,
  [SMALL_STATE(311)] = 28021,
  [SMALL_STATE(312)] = 28070,
  [SMALL_STATE(313)] = 28119,
  [SMALL_STATE(314)] = 28168,
  [SMALL_STATE(315)] = 28217,
  [SMALL_STATE(316)] = 28266,
  [SMALL_STATE(317)] = 28315,
  [SMALL_STATE(318)] = 28364,
  [SMALL_STATE(319)] = 28413,
  [SMALL_STATE(320)] = 28462,
  [SMALL_STATE(321)] = 28511,
  [SMALL_STATE(322)] = 28560,
  [SMALL_STATE(323)] = 28609,
  [SMALL_STATE(324)] = 28658,
  [SMALL_STATE(325)] = 28707,
  [SMALL_STATE(326)] = 28756,
  [SMALL_STATE(327)] = 28805,
  [SMALL_STATE(328)] = 28854,
  [SMALL_STATE(329)] = 28903,
  [SMALL_STATE(330)] = 28952,
  [SMALL_STATE(331)] = 29001,
  [SMALL_STATE(332)] = 29050,
  [SMALL_STATE(333)] = 29099,
  [SMALL_STATE(334)] = 29148,
  [SMALL_STATE(335)] = 29197,
  [SMALL_STATE(336)] = 29246,
  [SMALL_STATE(337)] = 29295,
  [SMALL_STATE(338)] = 29344,
  [SMALL_STATE(339)] = 29393,
  [SMALL_STATE(340)] = 29452,
  [SMALL_STATE(341)] = 29520,
  [SMALL_STATE(342)] = 29576,
  [SMALL_STATE(343)] = 29632,
  [SMALL_STATE(344)] = 29690,
  [SMALL_STATE(345)] = 29750,
  [SMALL_STATE(346)] = 29814,
  [SMALL_STATE(347)] = 29880,
  [SMALL_STATE(348)] = 29929,
  [SMALL_STATE(349)] = 29973,
  [SMALL_STATE(350)] = 30017,
  [SMALL_STATE(351)] = 30061,
  [SMALL_STATE(352)] = 30105,
  [SMALL_STATE(353)] = 30149,
  [SMALL_STATE(354)] = 30193,
  [SMALL_STATE(355)] = 30237,
  [SMALL_STATE(356)] = 30281,
  [SMALL_STATE(357)] = 30325,
  [SMALL_STATE(358)] = 30369,
  [SMALL_STATE(359)] = 30413,
  [SMALL_STATE(360)] = 30457,
  [SMALL_STATE(361)] = 30501,
  [SMALL_STATE(362)] = 30545,
  [SMALL_STATE(363)] = 30589,
  [SMALL_STATE(364)] = 30633,
  [SMALL_STATE(365)] = 30677,
  [SMALL_STATE(366)] = 30721,
  [SMALL_STATE(367)] = 30765,
  [SMALL_STATE(368)] = 30809,
  [SMALL_STATE(369)] = 30853,
  [SMALL_STATE(370)] = 30897,
  [SMALL_STATE(371)] = 30941,
  [SMALL_STATE(372)] = 30985,
  [SMALL_STATE(373)] = 31029,
  [SMALL_STATE(374)] = 31073,
  [SMALL_STATE(375)] = 31117,
  [SMALL_STATE(376)] = 31161,
  [SMALL_STATE(377)] = 31205,
  [SMALL_STATE(378)] = 31249,
  [SMALL_STATE(379)] = 31293,
  [SMALL_STATE(380)] = 31337,
  [SMALL_STATE(381)] = 31381,
  [SMALL_STATE(382)] = 31425,
  [SMALL_STATE(383)] = 31469,
  [SMALL_STATE(384)] = 31513,
  [SMALL_STATE(385)] = 31557,
  [SMALL_STATE(386)] = 31643,
  [SMALL_STATE(387)] = 31687,
  [SMALL_STATE(388)] = 31731,
  [SMALL_STATE(389)] = 31802,
  [SMALL_STATE(390)] = 31873,
  [SMALL_STATE(391)] = 31925,
  [SMALL_STATE(392)] = 31977,
  [SMALL_STATE(393)] = 32022,
  [SMALL_STATE(394)] = 32075,
  [SMALL_STATE(395)] = 32150,
  [SMALL_STATE(396)] = 32225,
  [SMALL_STATE(397)] = 32265,
  [SMALL_STATE(398)] = 32305,
  [SMALL_STATE(399)] = 32345,
  [SMALL_STATE(400)] = 32385,
  [SMALL_STATE(401)] = 32425,
  [SMALL_STATE(402)] = 32465,
  [SMALL_STATE(403)] = 32505,
  [SMALL_STATE(404)] = 32545,
  [SMALL_STATE(405)] = 32585,
  [SMALL_STATE(406)] = 32625,
  [SMALL_STATE(407)] = 32665,
  [SMALL_STATE(408)] = 32705,
  [SMALL_STATE(409)] = 32745,
  [SMALL_STATE(410)] = 32785,
  [SMALL_STATE(411)] = 32825,
  [SMALL_STATE(412)] = 32865,
  [SMALL_STATE(413)] = 32905,
  [SMALL_STATE(414)] = 32945,
  [SMALL_STATE(415)] = 32985,
  [SMALL_STATE(416)] = 33025,
  [SMALL_STATE(417)] = 33065,
  [SMALL_STATE(418)] = 33105,
  [SMALL_STATE(419)] = 33145,
  [SMALL_STATE(420)] = 33185,
  [SMALL_STATE(421)] = 33225,
  [SMALL_STATE(422)] = 33265,
  [SMALL_STATE(423)] = 33305,
  [SMALL_STATE(424)] = 33345,
  [SMALL_STATE(425)] = 33409,
  [SMALL_STATE(426)] = 33449,
  [SMALL_STATE(427)] = 33501,
  [SMALL_STATE(428)] = 33541,
  [SMALL_STATE(429)] = 33581,
  [SMALL_STATE(430)] = 33621,
  [SMALL_STATE(431)] = 33661,
  [SMALL_STATE(432)] = 33701,
  [SMALL_STATE(433)] = 33741,
  [SMALL_STATE(434)] = 33781,
  [SMALL_STATE(435)] = 33821,
  [SMALL_STATE(436)] = 33861,
  [SMALL_STATE(437)] = 33901,
  [SMALL_STATE(438)] = 33954,
  [SMALL_STATE(439)] = 34011,
  [SMALL_STATE(440)] = 34074,
  [SMALL_STATE(441)] = 34129,
  [SMALL_STATE(442)] = 34178,
  [SMALL_STATE(443)] = 34243,
  [SMALL_STATE(444)] = 34300,
  [SMALL_STATE(445)] = 34361,
  [SMALL_STATE(446)] = 34420,
  [SMALL_STATE(447)] = 34485,
  [SMALL_STATE(448)] = 34534,
  [SMALL_STATE(449)] = 34585,
  [SMALL_STATE(450)] = 34638,
  [SMALL_STATE(451)] = 34696,
  [SMALL_STATE(452)] = 34754,
  [SMALL_STATE(453)] = 34822,
  [SMALL_STATE(454)] = 34896,
  [SMALL_STATE(455)] = 34966,
  [SMALL_STATE(456)] = 35008,
  [SMALL_STATE(457)] = 35066,
  [SMALL_STATE(458)] = 35136,
  [SMALL_STATE(459)] = 35206,
  [SMALL_STATE(460)] = 35256,
  [SMALL_STATE(461)] = 35328,
  [SMALL_STATE(462)] = 35397,
  [SMALL_STATE(463)] = 35434,
  [SMALL_STATE(464)] = 35471,
  [SMALL_STATE(465)] = 35508,
  [SMALL_STATE(466)] = 35545,
  [SMALL_STATE(467)] = 35582,
  [SMALL_STATE(468)] = 35619,
  [SMALL_STATE(469)] = 35656,
  [SMALL_STATE(470)] = 35693,
  [SMALL_STATE(471)] = 35730,
  [SMALL_STATE(472)] = 35767,
  [SMALL_STATE(473)] = 35804,
  [SMALL_STATE(474)] = 35841,
  [SMALL_STATE(475)] = 35878,
  [SMALL_STATE(476)] = 35915,
  [SMALL_STATE(477)] = 35952,
  [SMALL_STATE(478)] = 35989,
  [SMALL_STATE(479)] = 36026,
  [SMALL_STATE(480)] = 36063,
  [SMALL_STATE(481)] = 36100,
  [SMALL_STATE(482)] = 36137,
  [SMALL_STATE(483)] = 36174,
  [SMALL_STATE(484)] = 36211,
  [SMALL_STATE(485)] = 36248,
  [SMALL_STATE(486)] = 36285,
  [SMALL_STATE(487)] = 36322,
  [SMALL_STATE(488)] = 36359,
  [SMALL_STATE(489)] = 36396,
  [SMALL_STATE(490)] = 36433,
  [SMALL_STATE(491)] = 36470,
  [SMALL_STATE(492)] = 36507,
  [SMALL_STATE(493)] = 36544,
  [SMALL_STATE(494)] = 36581,
  [SMALL_STATE(495)] = 36650,
  [SMALL_STATE(496)] = 36687,
  [SMALL_STATE(497)] = 36724,
  [SMALL_STATE(498)] = 36793,
  [SMALL_STATE(499)] = 36858,
  [SMALL_STATE(500)] = 36927,
  [SMALL_STATE(501)] = 36974,
  [SMALL_STATE(502)] = 37039,
  [SMALL_STATE(503)] = 37086,
  [SMALL_STATE(504)] = 37123,
  [SMALL_STATE(505)] = 37160,
  [SMALL_STATE(506)] = 37211,
  [SMALL_STATE(507)] = 37266,
  [SMALL_STATE(508)] = 37325,
  [SMALL_STATE(509)] = 37386,
  [SMALL_STATE(510)] = 37423,
  [SMALL_STATE(511)] = 37492,
  [SMALL_STATE(512)] = 37561,
  [SMALL_STATE(513)] = 37598,
  [SMALL_STATE(514)] = 37635,
  [SMALL_STATE(515)] = 37698,
  [SMALL_STATE(516)] = 37764,
  [SMALL_STATE(517)] = 37830,
  [SMALL_STATE(518)] = 37882,
  [SMALL_STATE(519)] = 37948,
  [SMALL_STATE(520)] = 38012,
  [SMALL_STATE(521)] = 38078,
  [SMALL_STATE(522)] = 38144,
  [SMALL_STATE(523)] = 38210,
  [SMALL_STATE(524)] = 38276,
  [SMALL_STATE(525)] = 38324,
  [SMALL_STATE(526)] = 38364,
  [SMALL_STATE(527)] = 38430,
  [SMALL_STATE(528)] = 38496,
  [SMALL_STATE(529)] = 38562,
  [SMALL_STATE(530)] = 38626,
  [SMALL_STATE(531)] = 38692,
  [SMALL_STATE(532)] = 38758,
  [SMALL_STATE(533)] = 38824,
  [SMALL_STATE(534)] = 38879,
  [SMALL_STATE(535)] = 38914,
  [SMALL_STATE(536)] = 38949,
  [SMALL_STATE(537)] = 39012,
  [SMALL_STATE(538)] = 39047,
  [SMALL_STATE(539)] = 39082,
  [SMALL_STATE(540)] = 39117,
  [SMALL_STATE(541)] = 39152,
  [SMALL_STATE(542)] = 39187,
  [SMALL_STATE(543)] = 39222,
  [SMALL_STATE(544)] = 39257,
  [SMALL_STATE(545)] = 39292,
  [SMALL_STATE(546)] = 39327,
  [SMALL_STATE(547)] = 39362,
  [SMALL_STATE(548)] = 39397,
  [SMALL_STATE(549)] = 39432,
  [SMALL_STATE(550)] = 39467,
  [SMALL_STATE(551)] = 39502,
  [SMALL_STATE(552)] = 39537,
  [SMALL_STATE(553)] = 39572,
  [SMALL_STATE(554)] = 39607,
  [SMALL_STATE(555)] = 39642,
  [SMALL_STATE(556)] = 39677,
  [SMALL_STATE(557)] = 39712,
  [SMALL_STATE(558)] = 39747,
  [SMALL_STATE(559)] = 39782,
  [SMALL_STATE(560)] = 39817,
  [SMALL_STATE(561)] = 39852,
  [SMALL_STATE(562)] = 39887,
  [SMALL_STATE(563)] = 39922,
  [SMALL_STATE(564)] = 39957,
  [SMALL_STATE(565)] = 39992,
  [SMALL_STATE(566)] = 40027,
  [SMALL_STATE(567)] = 40080,
  [SMALL_STATE(568)] = 40115,
  [SMALL_STATE(569)] = 40178,
  [SMALL_STATE(570)] = 40213,
  [SMALL_STATE(571)] = 40248,
  [SMALL_STATE(572)] = 40283,
  [SMALL_STATE(573)] = 40342,
  [SMALL_STATE(574)] = 40409,
  [SMALL_STATE(575)] = 40472,
  [SMALL_STATE(576)] = 40533,
  [SMALL_STATE(577)] = 40596,
  [SMALL_STATE(578)] = 40659,
  [SMALL_STATE(579)] = 40722,
  [SMALL_STATE(580)] = 40785,
  [SMALL_STATE(581)] = 40848,
  [SMALL_STATE(582)] = 40913,
  [SMALL_STATE(583)] = 40962,
  [SMALL_STATE(584)] = 41015,
  [SMALL_STATE(585)] = 41070,
  [SMALL_STATE(586)] = 41127,
  [SMALL_STATE(587)] = 41176,
  [SMALL_STATE(588)] = 41239,
  [SMALL_STATE(589)] = 41302,
  [SMALL_STATE(590)] = 41365,
  [SMALL_STATE(591)] = 41400,
  [SMALL_STATE(592)] = 41463,
  [SMALL_STATE(593)] = 41508,
  [SMALL_STATE(594)] = 41573,
  [SMALL_STATE(595)] = 41608,
  [SMALL_STATE(596)] = 41643,
  [SMALL_STATE(597)] = 41678,
  [SMALL_STATE(598)] = 41743,
  [SMALL_STATE(599)] = 41800,
  [SMALL_STATE(600)] = 41862,
  [SMALL_STATE(601)] = 41922,
  [SMALL_STATE(602)] = 41984,
  [SMALL_STATE(603)] = 42044,
  [SMALL_STATE(604)] = 42102,
  [SMALL_STATE(605)] = 42162,
  [SMALL_STATE(606)] = 42222,
  [SMALL_STATE(607)] = 42280,
  [SMALL_STATE(608)] = 42342,
  [SMALL_STATE(609)] = 42402,
  [SMALL_STATE(610)] = 42466,
  [SMALL_STATE(611)] = 42526,
  [SMALL_STATE(612)] = 42584,
  [SMALL_STATE(613)] = 42646,
  [SMALL_STATE(614)] = 42708,
  [SMALL_STATE(615)] = 42770,
  [SMALL_STATE(616)] = 42830,
  [SMALL_STATE(617)] = 42892,
  [SMALL_STATE(618)] = 42952,
  [SMALL_STATE(619)] = 43014,
  [SMALL_STATE(620)] = 43074,
  [SMALL_STATE(621)] = 43134,
  [SMALL_STATE(622)] = 43196,
  [SMALL_STATE(623)] = 43256,
  [SMALL_STATE(624)] = 43316,
  [SMALL_STATE(625)] = 43376,
  [SMALL_STATE(626)] = 43436,
  [SMALL_STATE(627)] = 43496,
  [SMALL_STATE(628)] = 43556,
  [SMALL_STATE(629)] = 43618,
  [SMALL_STATE(630)] = 43678,
  [SMALL_STATE(631)] = 43740,
  [SMALL_STATE(632)] = 43800,
  [SMALL_STATE(633)] = 43860,
  [SMALL_STATE(634)] = 43920,
  [SMALL_STATE(635)] = 43984,
  [SMALL_STATE(636)] = 44046,
  [SMALL_STATE(637)] = 44106,
  [SMALL_STATE(638)] = 44166,
  [SMALL_STATE(639)] = 44228,
  [SMALL_STATE(640)] = 44288,
  [SMALL_STATE(641)] = 44348,
  [SMALL_STATE(642)] = 44408,
  [SMALL_STATE(643)] = 44470,
  [SMALL_STATE(644)] = 44530,
  [SMALL_STATE(645)] = 44589,
  [SMALL_STATE(646)] = 44646,
  [SMALL_STATE(647)] = 44705,
  [SMALL_STATE(648)] = 44762,
  [SMALL_STATE(649)] = 44819,
  [SMALL_STATE(650)] = 44878,
  [SMALL_STATE(651)] = 44937,
  [SMALL_STATE(652)] = 44996,
  [SMALL_STATE(653)] = 45055,
  [SMALL_STATE(654)] = 45112,
  [SMALL_STATE(655)] = 45169,
  [SMALL_STATE(656)] = 45228,
  [SMALL_STATE(657)] = 45285,
  [SMALL_STATE(658)] = 45342,
  [SMALL_STATE(659)] = 45399,
  [SMALL_STATE(660)] = 45456,
  [SMALL_STATE(661)] = 45515,
  [SMALL_STATE(662)] = 45574,
  [SMALL_STATE(663)] = 45633,
  [SMALL_STATE(664)] = 45690,
  [SMALL_STATE(665)] = 45747,
  [SMALL_STATE(666)] = 45806,
  [SMALL_STATE(667)] = 45863,
  [SMALL_STATE(668)] = 45920,
  [SMALL_STATE(669)] = 45977,
  [SMALL_STATE(670)] = 46034,
  [SMALL_STATE(671)] = 46093,
  [SMALL_STATE(672)] = 46152,
  [SMALL_STATE(673)] = 46211,
  [SMALL_STATE(674)] = 46268,
  [SMALL_STATE(675)] = 46325,
  [SMALL_STATE(676)] = 46382,
  [SMALL_STATE(677)] = 46439,
  [SMALL_STATE(678)] = 46498,
  [SMALL_STATE(679)] = 46557,
  [SMALL_STATE(680)] = 46616,
  [SMALL_STATE(681)] = 46675,
  [SMALL_STATE(682)] = 46732,
  [SMALL_STATE(683)] = 46789,
  [SMALL_STATE(684)] = 46846,
  [SMALL_STATE(685)] = 46903,
  [SMALL_STATE(686)] = 46960,
  [SMALL_STATE(687)] = 47019,
  [SMALL_STATE(688)] = 47078,
  [SMALL_STATE(689)] = 47135,
  [SMALL_STATE(690)] = 47192,
  [SMALL_STATE(691)] = 47249,
  [SMALL_STATE(692)] = 47306,
  [SMALL_STATE(693)] = 47365,
  [SMALL_STATE(694)] = 47424,
  [SMALL_STATE(695)] = 47483,
  [SMALL_STATE(696)] = 47540,
  [SMALL_STATE(697)] = 47597,
  [SMALL_STATE(698)] = 47656,
  [SMALL_STATE(699)] = 47715,
  [SMALL_STATE(700)] = 47774,
  [SMALL_STATE(701)] = 47830,
  [SMALL_STATE(702)] = 47886,
  [SMALL_STATE(703)] = 47942,
  [SMALL_STATE(704)] = 47998,
  [SMALL_STATE(705)] = 48054,
  [SMALL_STATE(706)] = 48110,
  [SMALL_STATE(707)] = 48166,
  [SMALL_STATE(708)] = 48222,
  [SMALL_STATE(709)] = 48278,
  [SMALL_STATE(710)] = 48334,
  [SMALL_STATE(711)] = 48390,
  [SMALL_STATE(712)] = 48446,
  [SMALL_STATE(713)] = 48502,
  [SMALL_STATE(714)] = 48558,
  [SMALL_STATE(715)] = 48614,
  [SMALL_STATE(716)] = 48670,
  [SMALL_STATE(717)] = 48726,
  [SMALL_STATE(718)] = 48782,
  [SMALL_STATE(719)] = 48838,
  [SMALL_STATE(720)] = 48894,
  [SMALL_STATE(721)] = 48950,
  [SMALL_STATE(722)] = 49006,
  [SMALL_STATE(723)] = 49062,
  [SMALL_STATE(724)] = 49118,
  [SMALL_STATE(725)] = 49174,
  [SMALL_STATE(726)] = 49230,
  [SMALL_STATE(727)] = 49286,
  [SMALL_STATE(728)] = 49342,
  [SMALL_STATE(729)] = 49398,
  [SMALL_STATE(730)] = 49454,
  [SMALL_STATE(731)] = 49510,
  [SMALL_STATE(732)] = 49566,
  [SMALL_STATE(733)] = 49622,
  [SMALL_STATE(734)] = 49678,
  [SMALL_STATE(735)] = 49734,
  [SMALL_STATE(736)] = 49790,
  [SMALL_STATE(737)] = 49846,
  [SMALL_STATE(738)] = 49902,
  [SMALL_STATE(739)] = 49958,
  [SMALL_STATE(740)] = 50014,
  [SMALL_STATE(741)] = 50070,
  [SMALL_STATE(742)] = 50126,
  [SMALL_STATE(743)] = 50182,
  [SMALL_STATE(744)] = 50238,
  [SMALL_STATE(745)] = 50294,
  [SMALL_STATE(746)] = 50350,
  [SMALL_STATE(747)] = 50406,
  [SMALL_STATE(748)] = 50462,
  [SMALL_STATE(749)] = 50518,
  [SMALL_STATE(750)] = 50574,
  [SMALL_STATE(751)] = 50630,
  [SMALL_STATE(752)] = 50686,
  [SMALL_STATE(753)] = 50742,
  [SMALL_STATE(754)] = 50798,
  [SMALL_STATE(755)] = 50854,
  [SMALL_STATE(756)] = 50910,
  [SMALL_STATE(757)] = 50966,
  [SMALL_STATE(758)] = 51022,
  [SMALL_STATE(759)] = 51078,
  [SMALL_STATE(760)] = 51134,
  [SMALL_STATE(761)] = 51190,
  [SMALL_STATE(762)] = 51246,
  [SMALL_STATE(763)] = 51302,
  [SMALL_STATE(764)] = 51358,
  [SMALL_STATE(765)] = 51412,
  [SMALL_STATE(766)] = 51468,
  [SMALL_STATE(767)] = 51524,
  [SMALL_STATE(768)] = 51580,
  [SMALL_STATE(769)] = 51636,
  [SMALL_STATE(770)] = 51692,
  [SMALL_STATE(771)] = 51748,
  [SMALL_STATE(772)] = 51804,
  [SMALL_STATE(773)] = 51860,
  [SMALL_STATE(774)] = 51916,
  [SMALL_STATE(775)] = 51972,
  [SMALL_STATE(776)] = 52028,
  [SMALL_STATE(777)] = 52084,
  [SMALL_STATE(778)] = 52140,
  [SMALL_STATE(779)] = 52196,
  [SMALL_STATE(780)] = 52252,
  [SMALL_STATE(781)] = 52308,
  [SMALL_STATE(782)] = 52364,
  [SMALL_STATE(783)] = 52394,
  [SMALL_STATE(784)] = 52424,
  [SMALL_STATE(785)] = 52454,
  [SMALL_STATE(786)] = 52484,
  [SMALL_STATE(787)] = 52516,
  [SMALL_STATE(788)] = 52543,
  [SMALL_STATE(789)] = 52569,
  [SMALL_STATE(790)] = 52595,
  [SMALL_STATE(791)] = 52621,
  [SMALL_STATE(792)] = 52647,
  [SMALL_STATE(793)] = 52671,
  [SMALL_STATE(794)] = 52695,
  [SMALL_STATE(795)] = 52719,
  [SMALL_STATE(796)] = 52743,
  [SMALL_STATE(797)] = 52768,
  [SMALL_STATE(798)] = 52793,
  [SMALL_STATE(799)] = 52821,
  [SMALL_STATE(800)] = 52849,
  [SMALL_STATE(801)] = 52875,
  [SMALL_STATE(802)] = 52900,
  [SMALL_STATE(803)] = 52921,
  [SMALL_STATE(804)] = 52946,
  [SMALL_STATE(805)] = 52967,
  [SMALL_STATE(806)] = 52988,
  [SMALL_STATE(807)] = 53013,
  [SMALL_STATE(808)] = 53034,
  [SMALL_STATE(809)] = 53055,
  [SMALL_STATE(810)] = 53078,
  [SMALL_STATE(811)] = 53098,
  [SMALL_STATE(812)] = 53118,
  [SMALL_STATE(813)] = 53138,
  [SMALL_STATE(814)] = 53158,
  [SMALL_STATE(815)] = 53178,
  [SMALL_STATE(816)] = 53198,
  [SMALL_STATE(817)] = 53218,
  [SMALL_STATE(818)] = 53238,
  [SMALL_STATE(819)] = 53258,
  [SMALL_STATE(820)] = 53278,
  [SMALL_STATE(821)] = 53298,
  [SMALL_STATE(822)] = 53318,
  [SMALL_STATE(823)] = 53338,
  [SMALL_STATE(824)] = 53358,
  [SMALL_STATE(825)] = 53378,
  [SMALL_STATE(826)] = 53398,
  [SMALL_STATE(827)] = 53418,
  [SMALL_STATE(828)] = 53438,
  [SMALL_STATE(829)] = 53458,
  [SMALL_STATE(830)] = 53478,
  [SMALL_STATE(831)] = 53498,
  [SMALL_STATE(832)] = 53518,
  [SMALL_STATE(833)] = 53538,
  [SMALL_STATE(834)] = 53558,
  [SMALL_STATE(835)] = 53578,
  [SMALL_STATE(836)] = 53598,
  [SMALL_STATE(837)] = 53618,
  [SMALL_STATE(838)] = 53638,
  [SMALL_STATE(839)] = 53657,
  [SMALL_STATE(840)] = 53690,
  [SMALL_STATE(841)] = 53709,
  [SMALL_STATE(842)] = 53728,
  [SMALL_STATE(843)] = 53761,
  [SMALL_STATE(844)] = 53792,
  [SMALL_STATE(845)] = 53815,
  [SMALL_STATE(846)] = 53843,
  [SMALL_STATE(847)] = 53875,
  [SMALL_STATE(848)] = 53907,
  [SMALL_STATE(849)] = 53939,
  [SMALL_STATE(850)] = 53967,
  [SMALL_STATE(851)] = 53987,
  [SMALL_STATE(852)] = 54019,
  [SMALL_STATE(853)] = 54051,
  [SMALL_STATE(854)] = 54079,
  [SMALL_STATE(855)] = 54111,
  [SMALL_STATE(856)] = 54133,
  [SMALL_STATE(857)] = 54148,
  [SMALL_STATE(858)] = 54163,
  [SMALL_STATE(859)] = 54178,
  [SMALL_STATE(860)] = 54193,
  [SMALL_STATE(861)] = 54208,
  [SMALL_STATE(862)] = 54223,
  [SMALL_STATE(863)] = 54238,
  [SMALL_STATE(864)] = 54253,
  [SMALL_STATE(865)] = 54268,
  [SMALL_STATE(866)] = 54283,
  [SMALL_STATE(867)] = 54298,
  [SMALL_STATE(868)] = 54313,
  [SMALL_STATE(869)] = 54328,
  [SMALL_STATE(870)] = 54343,
  [SMALL_STATE(871)] = 54358,
  [SMALL_STATE(872)] = 54381,
  [SMALL_STATE(873)] = 54396,
  [SMALL_STATE(874)] = 54411,
  [SMALL_STATE(875)] = 54426,
  [SMALL_STATE(876)] = 54441,
  [SMALL_STATE(877)] = 54456,
  [SMALL_STATE(878)] = 54475,
  [SMALL_STATE(879)] = 54490,
  [SMALL_STATE(880)] = 54505,
  [SMALL_STATE(881)] = 54520,
  [SMALL_STATE(882)] = 54535,
  [SMALL_STATE(883)] = 54564,
  [SMALL_STATE(884)] = 54579,
  [SMALL_STATE(885)] = 54594,
  [SMALL_STATE(886)] = 54609,
  [SMALL_STATE(887)] = 54629,
  [SMALL_STATE(888)] = 54657,
  [SMALL_STATE(889)] = 54677,
  [SMALL_STATE(890)] = 54695,
  [SMALL_STATE(891)] = 54723,
  [SMALL_STATE(892)] = 54751,
  [SMALL_STATE(893)] = 54779,
  [SMALL_STATE(894)] = 54801,
  [SMALL_STATE(895)] = 54829,
  [SMALL_STATE(896)] = 54851,
  [SMALL_STATE(897)] = 54879,
  [SMALL_STATE(898)] = 54901,
  [SMALL_STATE(899)] = 54925,
  [SMALL_STATE(900)] = 54947,
  [SMALL_STATE(901)] = 54965,
  [SMALL_STATE(902)] = 54987,
  [SMALL_STATE(903)] = 55011,
  [SMALL_STATE(904)] = 55039,
  [SMALL_STATE(905)] = 55067,
  [SMALL_STATE(906)] = 55095,
  [SMALL_STATE(907)] = 55120,
  [SMALL_STATE(908)] = 55137,
  [SMALL_STATE(909)] = 55156,
  [SMALL_STATE(910)] = 55173,
  [SMALL_STATE(911)] = 55192,
  [SMALL_STATE(912)] = 55209,
  [SMALL_STATE(913)] = 55226,
  [SMALL_STATE(914)] = 55245,
  [SMALL_STATE(915)] = 55262,
  [SMALL_STATE(916)] = 55279,
  [SMALL_STATE(917)] = 55298,
  [SMALL_STATE(918)] = 55315,
  [SMALL_STATE(919)] = 55334,
  [SMALL_STATE(920)] = 55348,
  [SMALL_STATE(921)] = 55362,
  [SMALL_STATE(922)] = 55376,
  [SMALL_STATE(923)] = 55390,
  [SMALL_STATE(924)] = 55404,
  [SMALL_STATE(925)] = 55422,
  [SMALL_STATE(926)] = 55436,
  [SMALL_STATE(927)] = 55450,
  [SMALL_STATE(928)] = 55464,
  [SMALL_STATE(929)] = 55478,
  [SMALL_STATE(930)] = 55496,
  [SMALL_STATE(931)] = 55510,
  [SMALL_STATE(932)] = 55524,
  [SMALL_STATE(933)] = 55538,
  [SMALL_STATE(934)] = 55552,
  [SMALL_STATE(935)] = 55566,
  [SMALL_STATE(936)] = 55584,
  [SMALL_STATE(937)] = 55598,
  [SMALL_STATE(938)] = 55612,
  [SMALL_STATE(939)] = 55634,
  [SMALL_STATE(940)] = 55648,
  [SMALL_STATE(941)] = 55662,
  [SMALL_STATE(942)] = 55676,
  [SMALL_STATE(943)] = 55690,
  [SMALL_STATE(944)] = 55704,
  [SMALL_STATE(945)] = 55718,
  [SMALL_STATE(946)] = 55732,
  [SMALL_STATE(947)] = 55746,
  [SMALL_STATE(948)] = 55760,
  [SMALL_STATE(949)] = 55774,
  [SMALL_STATE(950)] = 55788,
  [SMALL_STATE(951)] = 55802,
  [SMALL_STATE(952)] = 55820,
  [SMALL_STATE(953)] = 55834,
  [SMALL_STATE(954)] = 55852,
  [SMALL_STATE(955)] = 55866,
  [SMALL_STATE(956)] = 55880,
  [SMALL_STATE(957)] = 55894,
  [SMALL_STATE(958)] = 55908,
  [SMALL_STATE(959)] = 55926,
  [SMALL_STATE(960)] = 55940,
  [SMALL_STATE(961)] = 55958,
  [SMALL_STATE(962)] = 55976,
  [SMALL_STATE(963)] = 55994,
  [SMALL_STATE(964)] = 56008,
  [SMALL_STATE(965)] = 56026,
  [SMALL_STATE(966)] = 56040,
  [SMALL_STATE(967)] = 56054,
  [SMALL_STATE(968)] = 56068,
  [SMALL_STATE(969)] = 56082,
  [SMALL_STATE(970)] = 56100,
  [SMALL_STATE(971)] = 56114,
  [SMALL_STATE(972)] = 56128,
  [SMALL_STATE(973)] = 56146,
  [SMALL_STATE(974)] = 56160,
  [SMALL_STATE(975)] = 56178,
  [SMALL_STATE(976)] = 56192,
  [SMALL_STATE(977)] = 56206,
  [SMALL_STATE(978)] = 56220,
  [SMALL_STATE(979)] = 56234,
  [SMALL_STATE(980)] = 56248,
  [SMALL_STATE(981)] = 56262,
  [SMALL_STATE(982)] = 56276,
  [SMALL_STATE(983)] = 56290,
  [SMALL_STATE(984)] = 56304,
  [SMALL_STATE(985)] = 56318,
  [SMALL_STATE(986)] = 56332,
  [SMALL_STATE(987)] = 56346,
  [SMALL_STATE(988)] = 56360,
  [SMALL_STATE(989)] = 56374,
  [SMALL_STATE(990)] = 56388,
  [SMALL_STATE(991)] = 56402,
  [SMALL_STATE(992)] = 56416,
  [SMALL_STATE(993)] = 56430,
  [SMALL_STATE(994)] = 56444,
  [SMALL_STATE(995)] = 56458,
  [SMALL_STATE(996)] = 56472,
  [SMALL_STATE(997)] = 56486,
  [SMALL_STATE(998)] = 56504,
  [SMALL_STATE(999)] = 56522,
  [SMALL_STATE(1000)] = 56536,
  [SMALL_STATE(1001)] = 56550,
  [SMALL_STATE(1002)] = 56568,
  [SMALL_STATE(1003)] = 56582,
  [SMALL_STATE(1004)] = 56596,
  [SMALL_STATE(1005)] = 56614,
  [SMALL_STATE(1006)] = 56632,
  [SMALL_STATE(1007)] = 56646,
  [SMALL_STATE(1008)] = 56664,
  [SMALL_STATE(1009)] = 56682,
  [SMALL_STATE(1010)] = 56700,
  [SMALL_STATE(1011)] = 56714,
  [SMALL_STATE(1012)] = 56728,
  [SMALL_STATE(1013)] = 56750,
  [SMALL_STATE(1014)] = 56764,
  [SMALL_STATE(1015)] = 56778,
  [SMALL_STATE(1016)] = 56796,
  [SMALL_STATE(1017)] = 56808,
  [SMALL_STATE(1018)] = 56822,
  [SMALL_STATE(1019)] = 56840,
  [SMALL_STATE(1020)] = 56860,
  [SMALL_STATE(1021)] = 56874,
  [SMALL_STATE(1022)] = 56888,
  [SMALL_STATE(1023)] = 56902,
  [SMALL_STATE(1024)] = 56916,
  [SMALL_STATE(1025)] = 56933,
  [SMALL_STATE(1026)] = 56950,
  [SMALL_STATE(1027)] = 56967,
  [SMALL_STATE(1028)] = 56986,
  [SMALL_STATE(1029)] = 57003,
  [SMALL_STATE(1030)] = 57022,
  [SMALL_STATE(1031)] = 57039,
  [SMALL_STATE(1032)] = 57058,
  [SMALL_STATE(1033)] = 57075,
  [SMALL_STATE(1034)] = 57092,
  [SMALL_STATE(1035)] = 57109,
  [SMALL_STATE(1036)] = 57126,
  [SMALL_STATE(1037)] = 57143,
  [SMALL_STATE(1038)] = 57160,
  [SMALL_STATE(1039)] = 57177,
  [SMALL_STATE(1040)] = 57194,
  [SMALL_STATE(1041)] = 57211,
  [SMALL_STATE(1042)] = 57230,
  [SMALL_STATE(1043)] = 57249,
  [SMALL_STATE(1044)] = 57266,
  [SMALL_STATE(1045)] = 57283,
  [SMALL_STATE(1046)] = 57300,
  [SMALL_STATE(1047)] = 57317,
  [SMALL_STATE(1048)] = 57334,
  [SMALL_STATE(1049)] = 57347,
  [SMALL_STATE(1050)] = 57364,
  [SMALL_STATE(1051)] = 57381,
  [SMALL_STATE(1052)] = 57398,
  [SMALL_STATE(1053)] = 57415,
  [SMALL_STATE(1054)] = 57432,
  [SMALL_STATE(1055)] = 57449,
  [SMALL_STATE(1056)] = 57468,
  [SMALL_STATE(1057)] = 57480,
  [SMALL_STATE(1058)] = 57496,
  [SMALL_STATE(1059)] = 57508,
  [SMALL_STATE(1060)] = 57524,
  [SMALL_STATE(1061)] = 57536,
  [SMALL_STATE(1062)] = 57550,
  [SMALL_STATE(1063)] = 57564,
  [SMALL_STATE(1064)] = 57576,
  [SMALL_STATE(1065)] = 57592,
  [SMALL_STATE(1066)] = 57604,
  [SMALL_STATE(1067)] = 57616,
  [SMALL_STATE(1068)] = 57632,
  [SMALL_STATE(1069)] = 57644,
  [SMALL_STATE(1070)] = 57658,
  [SMALL_STATE(1071)] = 57672,
  [SMALL_STATE(1072)] = 57686,
  [SMALL_STATE(1073)] = 57698,
  [SMALL_STATE(1074)] = 57712,
  [SMALL_STATE(1075)] = 57728,
  [SMALL_STATE(1076)] = 57744,
  [SMALL_STATE(1077)] = 57760,
  [SMALL_STATE(1078)] = 57774,
  [SMALL_STATE(1079)] = 57788,
  [SMALL_STATE(1080)] = 57802,
  [SMALL_STATE(1081)] = 57818,
  [SMALL_STATE(1082)] = 57832,
  [SMALL_STATE(1083)] = 57846,
  [SMALL_STATE(1084)] = 57862,
  [SMALL_STATE(1085)] = 57874,
  [SMALL_STATE(1086)] = 57888,
  [SMALL_STATE(1087)] = 57902,
  [SMALL_STATE(1088)] = 57916,
  [SMALL_STATE(1089)] = 57930,
  [SMALL_STATE(1090)] = 57944,
  [SMALL_STATE(1091)] = 57956,
  [SMALL_STATE(1092)] = 57968,
  [SMALL_STATE(1093)] = 57980,
  [SMALL_STATE(1094)] = 57994,
  [SMALL_STATE(1095)] = 58006,
  [SMALL_STATE(1096)] = 58020,
  [SMALL_STATE(1097)] = 58034,
  [SMALL_STATE(1098)] = 58050,
  [SMALL_STATE(1099)] = 58064,
  [SMALL_STATE(1100)] = 58078,
  [SMALL_STATE(1101)] = 58092,
  [SMALL_STATE(1102)] = 58104,
  [SMALL_STATE(1103)] = 58116,
  [SMALL_STATE(1104)] = 58132,
  [SMALL_STATE(1105)] = 58144,
  [SMALL_STATE(1106)] = 58158,
  [SMALL_STATE(1107)] = 58170,
  [SMALL_STATE(1108)] = 58182,
  [SMALL_STATE(1109)] = 58194,
  [SMALL_STATE(1110)] = 58208,
  [SMALL_STATE(1111)] = 58222,
  [SMALL_STATE(1112)] = 58234,
  [SMALL_STATE(1113)] = 58246,
  [SMALL_STATE(1114)] = 58258,
  [SMALL_STATE(1115)] = 58272,
  [SMALL_STATE(1116)] = 58286,
  [SMALL_STATE(1117)] = 58302,
  [SMALL_STATE(1118)] = 58314,
  [SMALL_STATE(1119)] = 58328,
  [SMALL_STATE(1120)] = 58340,
  [SMALL_STATE(1121)] = 58352,
  [SMALL_STATE(1122)] = 58366,
  [SMALL_STATE(1123)] = 58378,
  [SMALL_STATE(1124)] = 58390,
  [SMALL_STATE(1125)] = 58402,
  [SMALL_STATE(1126)] = 58416,
  [SMALL_STATE(1127)] = 58428,
  [SMALL_STATE(1128)] = 58440,
  [SMALL_STATE(1129)] = 58454,
  [SMALL_STATE(1130)] = 58468,
  [SMALL_STATE(1131)] = 58484,
  [SMALL_STATE(1132)] = 58498,
  [SMALL_STATE(1133)] = 58512,
  [SMALL_STATE(1134)] = 58524,
  [SMALL_STATE(1135)] = 58538,
  [SMALL_STATE(1136)] = 58550,
  [SMALL_STATE(1137)] = 58566,
  [SMALL_STATE(1138)] = 58578,
  [SMALL_STATE(1139)] = 58592,
  [SMALL_STATE(1140)] = 58606,
  [SMALL_STATE(1141)] = 58618,
  [SMALL_STATE(1142)] = 58632,
  [SMALL_STATE(1143)] = 58644,
  [SMALL_STATE(1144)] = 58656,
  [SMALL_STATE(1145)] = 58670,
  [SMALL_STATE(1146)] = 58683,
  [SMALL_STATE(1147)] = 58696,
  [SMALL_STATE(1148)] = 58707,
  [SMALL_STATE(1149)] = 58720,
  [SMALL_STATE(1150)] = 58733,
  [SMALL_STATE(1151)] = 58744,
  [SMALL_STATE(1152)] = 58757,
  [SMALL_STATE(1153)] = 58770,
  [SMALL_STATE(1154)] = 58783,
  [SMALL_STATE(1155)] = 58796,
  [SMALL_STATE(1156)] = 58809,
  [SMALL_STATE(1157)] = 58822,
  [SMALL_STATE(1158)] = 58835,
  [SMALL_STATE(1159)] = 58848,
  [SMALL_STATE(1160)] = 58859,
  [SMALL_STATE(1161)] = 58872,
  [SMALL_STATE(1162)] = 58885,
  [SMALL_STATE(1163)] = 58896,
  [SMALL_STATE(1164)] = 58907,
  [SMALL_STATE(1165)] = 58920,
  [SMALL_STATE(1166)] = 58933,
  [SMALL_STATE(1167)] = 58944,
  [SMALL_STATE(1168)] = 58957,
  [SMALL_STATE(1169)] = 58970,
  [SMALL_STATE(1170)] = 58983,
  [SMALL_STATE(1171)] = 58996,
  [SMALL_STATE(1172)] = 59009,
  [SMALL_STATE(1173)] = 59022,
  [SMALL_STATE(1174)] = 59035,
  [SMALL_STATE(1175)] = 59048,
  [SMALL_STATE(1176)] = 59061,
  [SMALL_STATE(1177)] = 59074,
  [SMALL_STATE(1178)] = 59087,
  [SMALL_STATE(1179)] = 59100,
  [SMALL_STATE(1180)] = 59113,
  [SMALL_STATE(1181)] = 59126,
  [SMALL_STATE(1182)] = 59139,
  [SMALL_STATE(1183)] = 59152,
  [SMALL_STATE(1184)] = 59163,
  [SMALL_STATE(1185)] = 59172,
  [SMALL_STATE(1186)] = 59183,
  [SMALL_STATE(1187)] = 59196,
  [SMALL_STATE(1188)] = 59209,
  [SMALL_STATE(1189)] = 59222,
  [SMALL_STATE(1190)] = 59235,
  [SMALL_STATE(1191)] = 59246,
  [SMALL_STATE(1192)] = 59255,
  [SMALL_STATE(1193)] = 59268,
  [SMALL_STATE(1194)] = 59281,
  [SMALL_STATE(1195)] = 59294,
  [SMALL_STATE(1196)] = 59303,
  [SMALL_STATE(1197)] = 59316,
  [SMALL_STATE(1198)] = 59329,
  [SMALL_STATE(1199)] = 59342,
  [SMALL_STATE(1200)] = 59355,
  [SMALL_STATE(1201)] = 59368,
  [SMALL_STATE(1202)] = 59381,
  [SMALL_STATE(1203)] = 59394,
  [SMALL_STATE(1204)] = 59407,
  [SMALL_STATE(1205)] = 59418,
  [SMALL_STATE(1206)] = 59431,
  [SMALL_STATE(1207)] = 59442,
  [SMALL_STATE(1208)] = 59453,
  [SMALL_STATE(1209)] = 59466,
  [SMALL_STATE(1210)] = 59479,
  [SMALL_STATE(1211)] = 59492,
  [SMALL_STATE(1212)] = 59505,
  [SMALL_STATE(1213)] = 59516,
  [SMALL_STATE(1214)] = 59529,
  [SMALL_STATE(1215)] = 59542,
  [SMALL_STATE(1216)] = 59555,
  [SMALL_STATE(1217)] = 59568,
  [SMALL_STATE(1218)] = 59581,
  [SMALL_STATE(1219)] = 59594,
  [SMALL_STATE(1220)] = 59605,
  [SMALL_STATE(1221)] = 59618,
  [SMALL_STATE(1222)] = 59631,
  [SMALL_STATE(1223)] = 59644,
  [SMALL_STATE(1224)] = 59657,
  [SMALL_STATE(1225)] = 59666,
  [SMALL_STATE(1226)] = 59679,
  [SMALL_STATE(1227)] = 59688,
  [SMALL_STATE(1228)] = 59701,
  [SMALL_STATE(1229)] = 59710,
  [SMALL_STATE(1230)] = 59721,
  [SMALL_STATE(1231)] = 59734,
  [SMALL_STATE(1232)] = 59745,
  [SMALL_STATE(1233)] = 59758,
  [SMALL_STATE(1234)] = 59771,
  [SMALL_STATE(1235)] = 59784,
  [SMALL_STATE(1236)] = 59793,
  [SMALL_STATE(1237)] = 59806,
  [SMALL_STATE(1238)] = 59815,
  [SMALL_STATE(1239)] = 59828,
  [SMALL_STATE(1240)] = 59841,
  [SMALL_STATE(1241)] = 59854,
  [SMALL_STATE(1242)] = 59867,
  [SMALL_STATE(1243)] = 59880,
  [SMALL_STATE(1244)] = 59893,
  [SMALL_STATE(1245)] = 59902,
  [SMALL_STATE(1246)] = 59915,
  [SMALL_STATE(1247)] = 59928,
  [SMALL_STATE(1248)] = 59939,
  [SMALL_STATE(1249)] = 59948,
  [SMALL_STATE(1250)] = 59961,
  [SMALL_STATE(1251)] = 59969,
  [SMALL_STATE(1252)] = 59977,
  [SMALL_STATE(1253)] = 59987,
  [SMALL_STATE(1254)] = 59995,
  [SMALL_STATE(1255)] = 60005,
  [SMALL_STATE(1256)] = 60015,
  [SMALL_STATE(1257)] = 60025,
  [SMALL_STATE(1258)] = 60035,
  [SMALL_STATE(1259)] = 60045,
  [SMALL_STATE(1260)] = 60055,
  [SMALL_STATE(1261)] = 60065,
  [SMALL_STATE(1262)] = 60073,
  [SMALL_STATE(1263)] = 60083,
  [SMALL_STATE(1264)] = 60091,
  [SMALL_STATE(1265)] = 60099,
  [SMALL_STATE(1266)] = 60107,
  [SMALL_STATE(1267)] = 60115,
  [SMALL_STATE(1268)] = 60123,
  [SMALL_STATE(1269)] = 60133,
  [SMALL_STATE(1270)] = 60141,
  [SMALL_STATE(1271)] = 60151,
  [SMALL_STATE(1272)] = 60161,
  [SMALL_STATE(1273)] = 60169,
  [SMALL_STATE(1274)] = 60177,
  [SMALL_STATE(1275)] = 60187,
  [SMALL_STATE(1276)] = 60195,
  [SMALL_STATE(1277)] = 60205,
  [SMALL_STATE(1278)] = 60215,
  [SMALL_STATE(1279)] = 60225,
  [SMALL_STATE(1280)] = 60233,
  [SMALL_STATE(1281)] = 60243,
  [SMALL_STATE(1282)] = 60253,
  [SMALL_STATE(1283)] = 60261,
  [SMALL_STATE(1284)] = 60269,
  [SMALL_STATE(1285)] = 60279,
  [SMALL_STATE(1286)] = 60289,
  [SMALL_STATE(1287)] = 60299,
  [SMALL_STATE(1288)] = 60309,
  [SMALL_STATE(1289)] = 60319,
  [SMALL_STATE(1290)] = 60329,
  [SMALL_STATE(1291)] = 60337,
  [SMALL_STATE(1292)] = 60347,
  [SMALL_STATE(1293)] = 60355,
  [SMALL_STATE(1294)] = 60365,
  [SMALL_STATE(1295)] = 60375,
  [SMALL_STATE(1296)] = 60385,
  [SMALL_STATE(1297)] = 60395,
  [SMALL_STATE(1298)] = 60403,
  [SMALL_STATE(1299)] = 60411,
  [SMALL_STATE(1300)] = 60421,
  [SMALL_STATE(1301)] = 60429,
  [SMALL_STATE(1302)] = 60439,
  [SMALL_STATE(1303)] = 60449,
  [SMALL_STATE(1304)] = 60459,
  [SMALL_STATE(1305)] = 60467,
  [SMALL_STATE(1306)] = 60477,
  [SMALL_STATE(1307)] = 60487,
  [SMALL_STATE(1308)] = 60497,
  [SMALL_STATE(1309)] = 60507,
  [SMALL_STATE(1310)] = 60517,
  [SMALL_STATE(1311)] = 60527,
  [SMALL_STATE(1312)] = 60537,
  [SMALL_STATE(1313)] = 60545,
  [SMALL_STATE(1314)] = 60555,
  [SMALL_STATE(1315)] = 60565,
  [SMALL_STATE(1316)] = 60573,
  [SMALL_STATE(1317)] = 60583,
  [SMALL_STATE(1318)] = 60593,
  [SMALL_STATE(1319)] = 60603,
  [SMALL_STATE(1320)] = 60613,
  [SMALL_STATE(1321)] = 60623,
  [SMALL_STATE(1322)] = 60633,
  [SMALL_STATE(1323)] = 60643,
  [SMALL_STATE(1324)] = 60651,
  [SMALL_STATE(1325)] = 60658,
  [SMALL_STATE(1326)] = 60665,
  [SMALL_STATE(1327)] = 60672,
  [SMALL_STATE(1328)] = 60679,
  [SMALL_STATE(1329)] = 60686,
  [SMALL_STATE(1330)] = 60693,
  [SMALL_STATE(1331)] = 60700,
  [SMALL_STATE(1332)] = 60707,
  [SMALL_STATE(1333)] = 60714,
  [SMALL_STATE(1334)] = 60721,
  [SMALL_STATE(1335)] = 60728,
  [SMALL_STATE(1336)] = 60735,
  [SMALL_STATE(1337)] = 60742,
  [SMALL_STATE(1338)] = 60749,
  [SMALL_STATE(1339)] = 60756,
  [SMALL_STATE(1340)] = 60763,
  [SMALL_STATE(1341)] = 60770,
  [SMALL_STATE(1342)] = 60777,
  [SMALL_STATE(1343)] = 60784,
  [SMALL_STATE(1344)] = 60791,
  [SMALL_STATE(1345)] = 60798,
  [SMALL_STATE(1346)] = 60805,
  [SMALL_STATE(1347)] = 60812,
  [SMALL_STATE(1348)] = 60819,
  [SMALL_STATE(1349)] = 60826,
  [SMALL_STATE(1350)] = 60833,
  [SMALL_STATE(1351)] = 60840,
  [SMALL_STATE(1352)] = 60847,
  [SMALL_STATE(1353)] = 60854,
  [SMALL_STATE(1354)] = 60861,
  [SMALL_STATE(1355)] = 60868,
  [SMALL_STATE(1356)] = 60875,
  [SMALL_STATE(1357)] = 60882,
  [SMALL_STATE(1358)] = 60889,
  [SMALL_STATE(1359)] = 60896,
  [SMALL_STATE(1360)] = 60903,
  [SMALL_STATE(1361)] = 60910,
  [SMALL_STATE(1362)] = 60917,
  [SMALL_STATE(1363)] = 60924,
  [SMALL_STATE(1364)] = 60931,
  [SMALL_STATE(1365)] = 60938,
  [SMALL_STATE(1366)] = 60945,
  [SMALL_STATE(1367)] = 60952,
  [SMALL_STATE(1368)] = 60959,
  [SMALL_STATE(1369)] = 60966,
  [SMALL_STATE(1370)] = 60973,
  [SMALL_STATE(1371)] = 60980,
  [SMALL_STATE(1372)] = 60987,
  [SMALL_STATE(1373)] = 60994,
  [SMALL_STATE(1374)] = 61001,
  [SMALL_STATE(1375)] = 61008,
  [SMALL_STATE(1376)] = 61015,
  [SMALL_STATE(1377)] = 61022,
  [SMALL_STATE(1378)] = 61029,
  [SMALL_STATE(1379)] = 61036,
  [SMALL_STATE(1380)] = 61043,
  [SMALL_STATE(1381)] = 61050,
  [SMALL_STATE(1382)] = 61057,
  [SMALL_STATE(1383)] = 61064,
  [SMALL_STATE(1384)] = 61071,
  [SMALL_STATE(1385)] = 61078,
  [SMALL_STATE(1386)] = 61085,
  [SMALL_STATE(1387)] = 61092,
  [SMALL_STATE(1388)] = 61099,
  [SMALL_STATE(1389)] = 61106,
  [SMALL_STATE(1390)] = 61113,
  [SMALL_STATE(1391)] = 61120,
  [SMALL_STATE(1392)] = 61127,
  [SMALL_STATE(1393)] = 61134,
  [SMALL_STATE(1394)] = 61141,
  [SMALL_STATE(1395)] = 61148,
  [SMALL_STATE(1396)] = 61155,
  [SMALL_STATE(1397)] = 61162,
  [SMALL_STATE(1398)] = 61169,
  [SMALL_STATE(1399)] = 61176,
  [SMALL_STATE(1400)] = 61183,
  [SMALL_STATE(1401)] = 61190,
  [SMALL_STATE(1402)] = 61197,
  [SMALL_STATE(1403)] = 61204,
  [SMALL_STATE(1404)] = 61211,
  [SMALL_STATE(1405)] = 61218,
  [SMALL_STATE(1406)] = 61225,
  [SMALL_STATE(1407)] = 61232,
  [SMALL_STATE(1408)] = 61239,
  [SMALL_STATE(1409)] = 61246,
  [SMALL_STATE(1410)] = 61253,
  [SMALL_STATE(1411)] = 61260,
  [SMALL_STATE(1412)] = 61267,
  [SMALL_STATE(1413)] = 61274,
  [SMALL_STATE(1414)] = 61281,
  [SMALL_STATE(1415)] = 61288,
  [SMALL_STATE(1416)] = 61295,
  [SMALL_STATE(1417)] = 61302,
  [SMALL_STATE(1418)] = 61309,
  [SMALL_STATE(1419)] = 61316,
  [SMALL_STATE(1420)] = 61323,
  [SMALL_STATE(1421)] = 61330,
  [SMALL_STATE(1422)] = 61337,
  [SMALL_STATE(1423)] = 61344,
  [SMALL_STATE(1424)] = 61351,
};

static const TSParseActionEntry ts_parse_actions[] = {
  [0] = {.entry = {.count = 0, .reusable = false}},
  [1] = {.entry = {.count = 1, .reusable = false}}, RECOVER(),
  [3] = {.entry = {.count = 1, .reusable = true}}, SHIFT_EXTRA(),
  [5] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_source_file, 0, 0, 0),
  [7] = {.entry = {.count = 1, .reusable = false}}, SHIFT(238),
  [9] = {.entry = {.count = 1, .reusable = true}}, SHIFT(981),
  [11] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1336),
  [13] = {.entry = {.count = 1, .reusable = false}}, SHIFT(839),
  [15] = {.entry = {.count = 1, .reusable = true}}, SHIFT(235),
  [17] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1210),
  [19] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1138),
  [21] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1148),
  [23] = {.entry = {.count = 1, .reusable = true}}, SHIFT(61),
  [25] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1093),
  [27] = {.entry = {.count = 1, .reusable = true}}, SHIFT(173),
  [29] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1254),
  [31] = {.entry = {.count = 1, .reusable = true}}, SHIFT(752),
  [33] = {.entry = {.count = 1, .reusable = true}}, SHIFT(12),
  [35] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1411),
  [37] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1400),
  [39] = {.entry = {.count = 1, .reusable = false}}, SHIFT(703),
  [41] = {.entry = {.count = 1, .reusable = true}}, SHIFT(208),
  [43] = {.entry = {.count = 1, .reusable = false}}, SHIFT(923),
  [45] = {.entry = {.count = 1, .reusable = false}}, SHIFT(907),
  [47] = {.entry = {.count = 1, .reusable = false}}, SHIFT(914),
  [49] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1406),
  [51] = {.entry = {.count = 1, .reusable = false}}, SHIFT(28),
  [53] = {.entry = {.count = 1, .reusable = false}}, SHIFT(175),
  [55] = {.entry = {.count = 1, .reusable = false}}, SHIFT(195),
  [57] = {.entry = {.count = 1, .reusable = false}}, SHIFT(27),
  [59] = {.entry = {.count = 1, .reusable = false}}, SHIFT(21),
  [61] = {.entry = {.count = 1, .reusable = false}}, SHIFT(22),
  [63] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1398),
  [65] = {.entry = {.count = 1, .reusable = false}}, SHIFT(252),
  [67] = {.entry = {.count = 1, .reusable = true}}, SHIFT(188),
  [69] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1412),
  [71] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1134),
  [73] = {.entry = {.count = 1, .reusable = false}}, SHIFT(263),
  [75] = {.entry = {.count = 1, .reusable = true}}, SHIFT(263),
  [77] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0),
  [79] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(238),
  [82] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(981),
  [85] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1364),
  [88] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(842),
  [91] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(235),
  [94] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1210),
  [97] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1138),
  [100] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1230),
  [103] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(61),
  [106] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1093),
  [109] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(173),
  [112] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1254),
  [115] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(752),
  [118] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(12),
  [121] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1411),
  [124] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1400),
  [127] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(703),
  [130] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(208),
  [133] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(923),
  [136] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(907),
  [139] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(914),
  [142] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1406),
  [145] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(28),
  [148] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(175),
  [151] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(195),
  [154] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(27),
  [157] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(21),
  [160] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(22),
  [163] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1398),
  [166] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(252),
  [169] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(188),
  [172] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1412),
  [175] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1134),
  [178] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(263),
  [181] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(263),
  [184] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_source_file, 1, 0, 0),
  [186] = {.entry = {.count = 1, .reusable = false}}, SHIFT(237),
  [188] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1286),
  [190] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_default_case, 2, 0, 0),
  [192] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_default_case, 2, 0, 0),
  [194] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_expression_case, 3, 0, 7),
  [196] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_expression_case, 3, 0, 7),
  [198] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_case, 4, 0, 104),
  [200] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_case, 4, 0, 104),
  [202] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_communication_case, 3, 0, 88),
  [204] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_communication_case, 3, 0, 88),
  [206] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_case, 3, 0, 41),
  [208] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_case, 3, 0, 41),
  [210] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__statement_list, 2, 0, 0),
  [212] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__statement_list, 2, 0, 0),
  [214] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__statement_list, 3, 0, 0),
  [216] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__statement_list, 3, 0, 0),
  [218] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_empty_labeled_statement, 2, 0, 26),
  [220] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_empty_labeled_statement, 2, 0, 26),
  [222] = {.entry = {.count = 1, .reusable = true}}, SHIFT(248),
  [224] = {.entry = {.count = 1, .reusable = true}}, SHIFT(594),
  [226] = {.entry = {.count = 1, .reusable = true}}, SHIFT(397),
  [228] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1094),
  [230] = {.entry = {.count = 1, .reusable = true}}, SHIFT(503),
  [232] = {.entry = {.count = 1, .reusable = true}}, SHIFT(322),
  [234] = {.entry = {.count = 1, .reusable = true}}, SHIFT(386),
  [236] = {.entry = {.count = 1, .reusable = false}}, SHIFT(294),
  [238] = {.entry = {.count = 1, .reusable = true}}, SHIFT(123),
  [240] = {.entry = {.count = 1, .reusable = true}}, SHIFT(224),
  [242] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1302),
  [244] = {.entry = {.count = 1, .reusable = true}}, SHIFT(197),
  [246] = {.entry = {.count = 1, .reusable = true}}, SHIFT(198),
  [248] = {.entry = {.count = 1, .reusable = false}}, SHIFT(176),
  [250] = {.entry = {.count = 1, .reusable = false}}, SHIFT(299),
  [252] = {.entry = {.count = 1, .reusable = true}}, SHIFT(154),
  [254] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1394),
  [256] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1061),
  [258] = {.entry = {.count = 1, .reusable = false}}, SHIFT(319),
  [260] = {.entry = {.count = 1, .reusable = true}}, SHIFT(319),
  [262] = {.entry = {.count = 1, .reusable = true}}, SHIFT(998),
  [264] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_for_clause, 3, 0, 58),
  [266] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_for_clause, 2, 0, 0),
  [268] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_for_clause, 4, 0, 86),
  [270] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_for_clause, 3, 0, 57),
  [272] = {.entry = {.count = 1, .reusable = false}}, SHIFT(426),
  [274] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_return_statement, 1, 0, 0),
  [276] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_return_statement, 1, 0, 0),
  [278] = {.entry = {.count = 1, .reusable = false}}, SHIFT(220),
  [280] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1294),
  [282] = {.entry = {.count = 1, .reusable = false}}, SHIFT(61),
  [284] = {.entry = {.count = 1, .reusable = false}}, SHIFT(189),
  [286] = {.entry = {.count = 1, .reusable = false}}, SHIFT(752),
  [288] = {.entry = {.count = 1, .reusable = false}}, SHIFT(190),
  [290] = {.entry = {.count = 1, .reusable = false}}, SHIFT(455),
  [292] = {.entry = {.count = 1, .reusable = false}}, SHIFT(149),
  [294] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1384),
  [296] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1100),
  [298] = {.entry = {.count = 1, .reusable = false}}, SHIFT(509),
  [300] = {.entry = {.count = 1, .reusable = false}}, SHIFT_EXTRA(),
  [302] = {.entry = {.count = 1, .reusable = false}}, SHIFT(393),
  [304] = {.entry = {.count = 1, .reusable = true}}, SHIFT(215),
  [306] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1396),
  [308] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1259),
  [310] = {.entry = {.count = 1, .reusable = true}}, SHIFT(185),
  [312] = {.entry = {.count = 1, .reusable = true}}, SHIFT(30),
  [314] = {.entry = {.count = 1, .reusable = true}}, SHIFT(466),
  [316] = {.entry = {.count = 1, .reusable = true}}, SHIFT(186),
  [318] = {.entry = {.count = 1, .reusable = false}}, SHIFT(392),
  [320] = {.entry = {.count = 1, .reusable = true}}, SHIFT(143),
  [322] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1373),
  [324] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1081),
  [326] = {.entry = {.count = 1, .reusable = false}}, SHIFT(419),
  [328] = {.entry = {.count = 1, .reusable = true}}, SHIFT(419),
  [330] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1352),
  [332] = {.entry = {.count = 1, .reusable = true}}, SHIFT(415),
  [334] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1350),
  [336] = {.entry = {.count = 1, .reusable = true}}, SHIFT(334),
  [338] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1376),
  [340] = {.entry = {.count = 1, .reusable = true}}, SHIFT(265),
  [342] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1419),
  [344] = {.entry = {.count = 1, .reusable = true}}, SHIFT(380),
  [346] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1409),
  [348] = {.entry = {.count = 1, .reusable = true}}, SHIFT(539),
  [350] = {.entry = {.count = 1, .reusable = true}}, SHIFT(433),
  [352] = {.entry = {.count = 1, .reusable = true}}, SHIFT(560),
  [354] = {.entry = {.count = 1, .reusable = true}}, SHIFT(276),
  [356] = {.entry = {.count = 1, .reusable = true}}, SHIFT(337),
  [358] = {.entry = {.count = 1, .reusable = true}}, SHIFT(287),
  [360] = {.entry = {.count = 1, .reusable = true}}, SHIFT(363),
  [362] = {.entry = {.count = 1, .reusable = true}}, SHIFT(399),
  [364] = {.entry = {.count = 1, .reusable = true}}, SHIFT(479),
  [366] = {.entry = {.count = 1, .reusable = true}}, SHIFT(487),
  [368] = {.entry = {.count = 1, .reusable = true}}, SHIFT(308),
  [370] = {.entry = {.count = 1, .reusable = true}}, SHIFT(371),
  [372] = {.entry = {.count = 1, .reusable = true}}, SHIFT(552),
  [374] = {.entry = {.count = 1, .reusable = false}}, SHIFT(459),
  [376] = {.entry = {.count = 1, .reusable = true}}, SHIFT(232),
  [378] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1313),
  [380] = {.entry = {.count = 1, .reusable = true}}, SHIFT(214),
  [382] = {.entry = {.count = 1, .reusable = true}}, SHIFT(212),
  [384] = {.entry = {.count = 1, .reusable = false}}, SHIFT(525),
  [386] = {.entry = {.count = 1, .reusable = true}}, SHIFT(166),
  [388] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1413),
  [390] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1109),
  [392] = {.entry = {.count = 1, .reusable = false}}, SHIFT(590),
  [394] = {.entry = {.count = 1, .reusable = true}}, SHIFT(590),
  [396] = {.entry = {.count = 1, .reusable = false}}, SHIFT(592),
  [398] = {.entry = {.count = 1, .reusable = true}}, SHIFT(209),
  [400] = {.entry = {.count = 1, .reusable = true}}, SHIFT(953),
  [402] = {.entry = {.count = 1, .reusable = true}}, SHIFT(206),
  [404] = {.entry = {.count = 1, .reusable = true}}, SHIFT(179),
  [406] = {.entry = {.count = 1, .reusable = false}}, SHIFT(517),
  [408] = {.entry = {.count = 1, .reusable = true}}, SHIFT(213),
  [410] = {.entry = {.count = 1, .reusable = true}}, SHIFT(130),
  [412] = {.entry = {.count = 1, .reusable = false}}, SHIFT(524),
  [414] = {.entry = {.count = 1, .reusable = true}}, SHIFT(255),
  [416] = {.entry = {.count = 1, .reusable = true}}, SHIFT(204),
  [418] = {.entry = {.count = 1, .reusable = true}}, SHIFT(174),
  [420] = {.entry = {.count = 1, .reusable = true}}, SHIFT(134),
  [422] = {.entry = {.count = 1, .reusable = false}}, SHIFT(192),
  [424] = {.entry = {.count = 1, .reusable = true}}, SHIFT(422),
  [426] = {.entry = {.count = 1, .reusable = true}}, SHIFT(723),
  [428] = {.entry = {.count = 1, .reusable = true}}, SHIFT(117),
  [430] = {.entry = {.count = 1, .reusable = true}}, SHIFT(428),
  [432] = {.entry = {.count = 1, .reusable = true}}, SHIFT(91),
  [434] = {.entry = {.count = 1, .reusable = true}}, SHIFT(407),
  [436] = {.entry = {.count = 1, .reusable = true}}, SHIFT(716),
  [438] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1380),
  [440] = {.entry = {.count = 1, .reusable = true}}, SHIFT(468),
  [442] = {.entry = {.count = 1, .reusable = true}}, SHIFT(483),
  [444] = {.entry = {.count = 1, .reusable = true}}, SHIFT(281),
  [446] = {.entry = {.count = 1, .reusable = true}}, SHIFT(489),
  [448] = {.entry = {.count = 1, .reusable = true}}, SHIFT(336),
  [450] = {.entry = {.count = 1, .reusable = true}}, SHIFT(304),
  [452] = {.entry = {.count = 1, .reusable = true}}, SHIFT(310),
  [454] = {.entry = {.count = 1, .reusable = true}}, SHIFT(384),
  [456] = {.entry = {.count = 1, .reusable = true}}, SHIFT(367),
  [458] = {.entry = {.count = 1, .reusable = true}}, SHIFT(373),
  [460] = {.entry = {.count = 1, .reusable = true}}, SHIFT(541),
  [462] = {.entry = {.count = 1, .reusable = false}}, SHIFT(236),
  [464] = {.entry = {.count = 1, .reusable = true}}, SHIFT(556),
  [466] = {.entry = {.count = 1, .reusable = true}}, SHIFT(562),
  [468] = {.entry = {.count = 1, .reusable = true}}, SHIFT(702),
  [470] = {.entry = {.count = 1, .reusable = true}}, SHIFT(100),
  [472] = {.entry = {.count = 1, .reusable = true}}, SHIFT(105),
  [474] = {.entry = {.count = 1, .reusable = true}}, SHIFT(93),
  [476] = {.entry = {.count = 1, .reusable = true}}, SHIFT(284),
  [478] = {.entry = {.count = 1, .reusable = true}}, SHIFT(274),
  [480] = {.entry = {.count = 1, .reusable = true}}, SHIFT(478),
  [482] = {.entry = {.count = 1, .reusable = true}}, SHIFT(485),
  [484] = {.entry = {.count = 1, .reusable = true}}, SHIFT(486),
  [486] = {.entry = {.count = 1, .reusable = true}}, SHIFT(285),
  [488] = {.entry = {.count = 1, .reusable = true}}, SHIFT(738),
  [490] = {.entry = {.count = 1, .reusable = true}}, SHIFT(271),
  [492] = {.entry = {.count = 1, .reusable = true}}, SHIFT(328),
  [494] = {.entry = {.count = 1, .reusable = true}}, SHIFT(333),
  [496] = {.entry = {.count = 1, .reusable = true}}, SHIFT(220),
  [498] = {.entry = {.count = 1, .reusable = true}}, SHIFT(189),
  [500] = {.entry = {.count = 1, .reusable = true}}, SHIFT(190),
  [502] = {.entry = {.count = 1, .reusable = true}}, SHIFT(149),
  [504] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1384),
  [506] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1100),
  [508] = {.entry = {.count = 1, .reusable = true}}, SHIFT(509),
  [510] = {.entry = {.count = 1, .reusable = true}}, SHIFT(306),
  [512] = {.entry = {.count = 1, .reusable = true}}, SHIFT(307),
  [514] = {.entry = {.count = 1, .reusable = true}}, SHIFT(406),
  [516] = {.entry = {.count = 1, .reusable = true}}, SHIFT(745),
  [518] = {.entry = {.count = 1, .reusable = true}}, SHIFT(476),
  [520] = {.entry = {.count = 1, .reusable = true}}, SHIFT(362),
  [522] = {.entry = {.count = 1, .reusable = true}}, SHIFT(396),
  [524] = {.entry = {.count = 1, .reusable = true}}, SHIFT(369),
  [526] = {.entry = {.count = 1, .reusable = true}}, SHIFT(370),
  [528] = {.entry = {.count = 1, .reusable = true}}, SHIFT(755),
  [530] = {.entry = {.count = 1, .reusable = true}}, SHIFT(549),
  [532] = {.entry = {.count = 1, .reusable = true}}, SHIFT(551),
  [534] = {.entry = {.count = 1, .reusable = true}}, SHIFT(408),
  [536] = {.entry = {.count = 1, .reusable = true}}, SHIFT(421),
  [538] = {.entry = {.count = 1, .reusable = true}}, SHIFT(558),
  [540] = {.entry = {.count = 1, .reusable = true}}, SHIFT(559),
  [542] = {.entry = {.count = 1, .reusable = true}}, SHIFT(24),
  [544] = {.entry = {.count = 1, .reusable = true}}, SHIFT(288),
  [546] = {.entry = {.count = 1, .reusable = true}}, SHIFT(23),
  [548] = {.entry = {.count = 1, .reusable = true}}, SHIFT(348),
  [550] = {.entry = {.count = 1, .reusable = false}}, SHIFT(339),
  [552] = {.entry = {.count = 1, .reusable = true}}, SHIFT(228),
  [554] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1309),
  [556] = {.entry = {.count = 1, .reusable = true}}, SHIFT(211),
  [558] = {.entry = {.count = 1, .reusable = true}}, SHIFT(210),
  [560] = {.entry = {.count = 1, .reusable = false}}, SHIFT(347),
  [562] = {.entry = {.count = 1, .reusable = true}}, SHIFT(160),
  [564] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1404),
  [566] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1088),
  [568] = {.entry = {.count = 1, .reusable = false}}, SHIFT(352),
  [570] = {.entry = {.count = 1, .reusable = true}}, SHIFT(352),
  [572] = {.entry = {.count = 1, .reusable = true}}, SHIFT(172),
  [574] = {.entry = {.count = 1, .reusable = false}}, SHIFT(239),
  [576] = {.entry = {.count = 1, .reusable = false}}, SHIFT(708),
  [578] = {.entry = {.count = 1, .reusable = false}}, SHIFT(855),
  [580] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__expression, 1, 0, 0), SHIFT(1417),
  [583] = {.entry = {.count = 3, .reusable = true}}, REDUCE(sym__simple_type, 1, -1, 1), REDUCE(sym__expression, 1, 0, 0), SHIFT(731),
  [587] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1346),
  [589] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1296),
  [591] = {.entry = {.count = 3, .reusable = true}}, REDUCE(sym__simple_type, 1, -1, 1), REDUCE(sym__expression, 1, 0, 0), SHIFT(76),
  [595] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__expression, 1, 0, 0),
  [597] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__expression, 1, 0, 0), SHIFT(705),
  [600] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__expression, 1, 0, 0),
  [602] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1414),
  [604] = {.entry = {.count = 1, .reusable = false}}, SHIFT(737),
  [606] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1383),
  [608] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym__expression, 1, 0, 0), SHIFT(1417),
  [611] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym__simple_type, 1, -1, 1), REDUCE(sym__expression, 1, 0, 0),
  [614] = {.entry = {.count = 3, .reusable = false}}, REDUCE(sym__simple_type, 1, -1, 1), REDUCE(sym__expression, 1, 0, 0), SHIFT(649),
  [618] = {.entry = {.count = 1, .reusable = false}}, SHIFT(32),
  [620] = {.entry = {.count = 1, .reusable = false}}, SHIFT(11),
  [622] = {.entry = {.count = 1, .reusable = false}}, SHIFT(19),
  [624] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_expression_statement, 1, 0, 0),
  [626] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_expression_statement, 1, 0, 0),
  [628] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1255),
  [630] = {.entry = {.count = 1, .reusable = false}}, SHIFT(80),
  [632] = {.entry = {.count = 1, .reusable = false}}, SHIFT(194),
  [634] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_expression_list, 1, 0, 0),
  [636] = {.entry = {.count = 1, .reusable = false}}, SHIFT(59),
  [638] = {.entry = {.count = 1, .reusable = false}}, SHIFT(196),
  [640] = {.entry = {.count = 1, .reusable = false}}, SHIFT(199),
  [642] = {.entry = {.count = 1, .reusable = false}}, SHIFT(200),
  [644] = {.entry = {.count = 1, .reusable = false}}, SHIFT(952),
  [646] = {.entry = {.count = 1, .reusable = false}}, SHIFT(954),
  [648] = {.entry = {.count = 1, .reusable = false}}, SHIFT(201),
  [650] = {.entry = {.count = 1, .reusable = false}}, SHIFT(202),
  [652] = {.entry = {.count = 1, .reusable = false}}, SHIFT(203),
  [654] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_binary_expression, 3, 0, 32),
  [656] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_binary_expression, 3, 0, 32),
  [658] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_unary_expression, 2, 0, 6),
  [660] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_unary_expression, 2, 0, 6),
  [662] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0),
  [664] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block, 2, 0, 0),
  [666] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block, 2, 0, 0),
  [668] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_interpreted_string_literal, 3, 0, 0),
  [670] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_interpreted_string_literal, 3, 0, 0),
  [672] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_raw_string_literal, 3, 0, 0),
  [674] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_raw_string_literal, 3, 0, 0),
  [676] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block, 3, 0, 0),
  [678] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block, 3, 0, 0),
  [680] = {.entry = {.count = 1, .reusable = false}}, SHIFT(699),
  [682] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_interpreted_string_literal, 2, 0, 0),
  [684] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_interpreted_string_literal, 2, 0, 0),
  [686] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_instantiation_expression, 4, -1, 19),
  [688] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_instantiation_expression, 4, -1, 19),
  [690] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_argument_list, 5, 0, 0),
  [692] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_argument_list, 5, 0, 0),
  [694] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1257),
  [696] = {.entry = {.count = 1, .reusable = true}}, SHIFT(66),
  [698] = {.entry = {.count = 1, .reusable = true}}, SHIFT(194),
  [700] = {.entry = {.count = 1, .reusable = true}}, SHIFT(79),
  [702] = {.entry = {.count = 1, .reusable = false}}, SHIFT(155),
  [704] = {.entry = {.count = 1, .reusable = false}}, SHIFT(156),
  [706] = {.entry = {.count = 1, .reusable = true}}, SHIFT(139),
  [708] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_expression_list, 1, 0, 0),
  [710] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1298),
  [712] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1265),
  [714] = {.entry = {.count = 1, .reusable = true}}, SHIFT(157),
  [716] = {.entry = {.count = 1, .reusable = false}}, SHIFT(157),
  [718] = {.entry = {.count = 1, .reusable = true}}, SHIFT(158),
  [720] = {.entry = {.count = 1, .reusable = true}}, SHIFT(159),
  [722] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_call_expression, 3, 0, 36),
  [724] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_call_expression, 3, 0, 36),
  [726] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_composite_literal, 2, 0, 12),
  [728] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_composite_literal, 2, 0, 12),
  [730] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_call_expression, 2, 0, 10),
  [732] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_call_expression, 2, 0, 10),
  [734] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_func_literal, 4, 0, 46),
  [736] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_func_literal, 4, 0, 46),
  [738] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_special_argument_list, 2, 0, 0),
  [740] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_special_argument_list, 2, 0, 0),
  [742] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_composite_literal, 2, 0, 13),
  [744] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_composite_literal, 2, 0, 13),
  [746] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_literal_value, 2, 0, 0),
  [748] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_literal_value, 2, 0, 0),
  [750] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_special_argument_list, 3, 0, 0),
  [752] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_special_argument_list, 3, 0, 0),
  [754] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_literal_value, 3, 0, 0),
  [756] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_literal_value, 3, 0, 0),
  [758] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_conversion_expression, 4, -1, 61),
  [760] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_conversion_expression, 4, -1, 61),
  [762] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_argument_list, 3, 0, 0),
  [764] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_argument_list, 3, 0, 0),
  [766] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_slice_expression, 7, 0, 105),
  [768] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_slice_expression, 7, 0, 105),
  [770] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_slice_expression, 4, 0, 62),
  [772] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_slice_expression, 4, 0, 62),
  [774] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_index_expression, 4, 0, 63),
  [776] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_index_expression, 4, 0, 63),
  [778] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parenthesized_expression, 3, 0, 0),
  [780] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parenthesized_expression, 3, 0, 0),
  [782] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_special_argument_list, 4, 0, 0),
  [784] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_special_argument_list, 4, 0, 0),
  [786] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_slice_expression, 8, 0, 108),
  [788] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_slice_expression, 8, 0, 108),
  [790] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_literal_value, 4, 0, 0),
  [792] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_literal_value, 4, 0, 0),
  [794] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_func_literal, 3, 0, 22),
  [796] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_func_literal, 3, 0, 22),
  [798] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_conversion_expression, 5, -1, 61),
  [800] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_conversion_expression, 5, -1, 61),
  [802] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_instantiation_expression, 5, -1, 19),
  [804] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_instantiation_expression, 5, -1, 19),
  [806] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_assertion_expression, 5, 0, 90),
  [808] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_assertion_expression, 5, 0, 90),
  [810] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_argument_list, 4, 0, 0),
  [812] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_argument_list, 4, 0, 0),
  [814] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_slice_expression, 5, 0, 91),
  [816] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_slice_expression, 5, 0, 91),
  [818] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_selector_expression, 3, 0, 34),
  [820] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_selector_expression, 3, 0, 34),
  [822] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_argument_list, 2, 0, 0),
  [824] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_argument_list, 2, 0, 0),
  [826] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_slice_expression, 5, 0, 92),
  [828] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_slice_expression, 5, 0, 92),
  [830] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_slice_expression, 6, 0, 102),
  [832] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_slice_expression, 6, 0, 102),
  [834] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_literal_value, 5, 0, 0),
  [836] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_literal_value, 5, 0, 0),
  [838] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_special_argument_list, 5, 0, 0),
  [840] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_special_argument_list, 5, 0, 0),
  [842] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_instantiation_expression, 6, -1, 19),
  [844] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_instantiation_expression, 6, -1, 19),
  [846] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1301),
  [848] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1004),
  [850] = {.entry = {.count = 1, .reusable = true}}, SHIFT(178),
  [852] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__simple_type, 1, -1, 1), REDUCE(sym__expression, 1, 0, 0),
  [855] = {.entry = {.count = 3, .reusable = true}}, REDUCE(sym__simple_type, 1, -1, 1), REDUCE(sym__expression, 1, 0, 0), SHIFT(649),
  [859] = {.entry = {.count = 1, .reusable = true}}, SHIFT(677),
  [861] = {.entry = {.count = 1, .reusable = true}}, SHIFT(33),
  [863] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1260),
  [865] = {.entry = {.count = 1, .reusable = true}}, SHIFT(69),
  [867] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_expression_list_repeat1, 2, 0, 0),
  [869] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_expression_list_repeat1, 2, 0, 0),
  [871] = {.entry = {.count = 1, .reusable = true}}, SHIFT(51),
  [873] = {.entry = {.count = 1, .reusable = false}}, SHIFT(161),
  [875] = {.entry = {.count = 1, .reusable = false}}, SHIFT(162),
  [877] = {.entry = {.count = 1, .reusable = true}}, SHIFT(163),
  [879] = {.entry = {.count = 1, .reusable = false}}, SHIFT(163),
  [881] = {.entry = {.count = 1, .reusable = true}}, SHIFT(164),
  [883] = {.entry = {.count = 1, .reusable = true}}, SHIFT(165),
  [885] = {.entry = {.count = 1, .reusable = true}}, SHIFT(644),
  [887] = {.entry = {.count = 1, .reusable = false}}, SHIFT(799),
  [889] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_declaration, 1, 0, 25),
  [891] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_declaration, 1, 0, 25),
  [893] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1399),
  [895] = {.entry = {.count = 1, .reusable = false}}, SHIFT(704),
  [897] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1397),
  [899] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1284),
  [901] = {.entry = {.count = 1, .reusable = false}}, SHIFT(56),
  [903] = {.entry = {.count = 1, .reusable = false}}, SHIFT(726),
  [905] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1308),
  [907] = {.entry = {.count = 1, .reusable = false}}, SHIFT(778),
  [909] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1357),
  [911] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1416),
  [913] = {.entry = {.count = 1, .reusable = false}}, SHIFT(721),
  [915] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1359),
  [917] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1412),
  [919] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1134),
  [921] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_type, 2, 0, 5),
  [923] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_type, 2, 0, 5),
  [925] = {.entry = {.count = 1, .reusable = false}}, SHIFT(511),
  [927] = {.entry = {.count = 1, .reusable = false}}, SHIFT(127),
  [929] = {.entry = {.count = 1, .reusable = false}}, SHIFT(798),
  [931] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1305),
  [933] = {.entry = {.count = 1, .reusable = false}}, SHIFT(90),
  [935] = {.entry = {.count = 1, .reusable = false}}, SHIFT(739),
  [937] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1418),
  [939] = {.entry = {.count = 1, .reusable = false}}, SHIFT(736),
  [941] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1392),
  [943] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1281),
  [945] = {.entry = {.count = 1, .reusable = true}}, SHIFT(55),
  [947] = {.entry = {.count = 1, .reusable = true}}, SHIFT(77),
  [949] = {.entry = {.count = 1, .reusable = true}}, SHIFT(655),
  [951] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_const_spec, 1, 0, 4),
  [953] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_const_spec, 1, 0, 4),
  [955] = {.entry = {.count = 1, .reusable = false}}, SHIFT(731),
  [957] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1339),
  [959] = {.entry = {.count = 1, .reusable = false}}, SHIFT(116),
  [961] = {.entry = {.count = 1, .reusable = false}}, SHIFT(84),
  [963] = {.entry = {.count = 1, .reusable = false}}, SHIFT(705),
  [965] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1383),
  [967] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_const_spec, 2, 0, 17),
  [969] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_const_spec, 2, 0, 17),
  [971] = {.entry = {.count = 1, .reusable = false}}, SHIFT(103),
  [973] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1295),
  [975] = {.entry = {.count = 1, .reusable = false}}, SHIFT(62),
  [977] = {.entry = {.count = 1, .reusable = false}}, SHIFT(177),
  [979] = {.entry = {.count = 1, .reusable = false}}, SHIFT(78),
  [981] = {.entry = {.count = 1, .reusable = false}}, SHIFT(150),
  [983] = {.entry = {.count = 1, .reusable = false}}, SHIFT(151),
  [985] = {.entry = {.count = 1, .reusable = false}}, SHIFT(152),
  [987] = {.entry = {.count = 1, .reusable = false}}, SHIFT(131),
  [989] = {.entry = {.count = 1, .reusable = false}}, SHIFT(153),
  [991] = {.entry = {.count = 1, .reusable = false}}, SHIFT(29),
  [993] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1281),
  [995] = {.entry = {.count = 1, .reusable = true}}, SHIFT(144),
  [997] = {.entry = {.count = 1, .reusable = false}}, SHIFT(144),
  [999] = {.entry = {.count = 1, .reusable = false}}, SHIFT(145),
  [1001] = {.entry = {.count = 1, .reusable = true}}, SHIFT(145),
  [1003] = {.entry = {.count = 1, .reusable = true}}, SHIFT(146),
  [1005] = {.entry = {.count = 1, .reusable = false}}, SHIFT(146),
  [1007] = {.entry = {.count = 1, .reusable = true}}, SHIFT(147),
  [1009] = {.entry = {.count = 1, .reusable = true}}, SHIFT(494),
  [1011] = {.entry = {.count = 1, .reusable = true}}, SHIFT(84),
  [1013] = {.entry = {.count = 1, .reusable = true}}, SHIFT(705),
  [1015] = {.entry = {.count = 1, .reusable = false}}, SHIFT(844),
  [1017] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1306),
  [1019] = {.entry = {.count = 1, .reusable = true}}, SHIFT(83),
  [1021] = {.entry = {.count = 1, .reusable = true}}, SHIFT(734),
  [1023] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1377),
  [1025] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_defer_statement, 2, 0, 0),
  [1027] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_defer_statement, 2, 0, 0),
  [1029] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_go_statement, 2, 0, 0),
  [1031] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_go_statement, 2, 0, 0),
  [1033] = {.entry = {.count = 1, .reusable = true}}, SHIFT(14),
  [1035] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1417),
  [1037] = {.entry = {.count = 1, .reusable = true}}, SHIFT(731),
  [1039] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__simple_type, 1, -1, 1),
  [1041] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__simple_type, 1, -1, 1), SHIFT(1346),
  [1044] = {.entry = {.count = 1, .reusable = true}}, SHIFT(76),
  [1046] = {.entry = {.count = 1, .reusable = true}}, SHIFT(743),
  [1048] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_declaration, 3, 0, 20),
  [1050] = {.entry = {.count = 1, .reusable = false}}, SHIFT(897),
  [1052] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_declaration, 3, 0, 20),
  [1054] = {.entry = {.count = 1, .reusable = false}}, SHIFT(510),
  [1056] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1256),
  [1058] = {.entry = {.count = 1, .reusable = false}}, SHIFT(102),
  [1060] = {.entry = {.count = 1, .reusable = false}}, SHIFT(747),
  [1062] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1258),
  [1064] = {.entry = {.count = 1, .reusable = false}}, SHIFT(720),
  [1066] = {.entry = {.count = 1, .reusable = false}}, SHIFT(15),
  [1068] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1372),
  [1070] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1420),
  [1072] = {.entry = {.count = 1, .reusable = false}}, SHIFT(744),
  [1074] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1382),
  [1076] = {.entry = {.count = 1, .reusable = false}}, SHIFT(670),
  [1078] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_send_statement, 3, 0, 35),
  [1080] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_send_statement, 3, 0, 35),
  [1082] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_declaration, 4, 0, 42),
  [1084] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_declaration, 4, 0, 42),
  [1086] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_method_declaration, 4, 0, 45),
  [1088] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_method_declaration, 4, 0, 45),
  [1090] = {.entry = {.count = 1, .reusable = true}}, SHIFT(34),
  [1092] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1317),
  [1094] = {.entry = {.count = 1, .reusable = true}}, SHIFT(72),
  [1096] = {.entry = {.count = 1, .reusable = true}}, SHIFT(136),
  [1098] = {.entry = {.count = 1, .reusable = true}}, SHIFT(57),
  [1100] = {.entry = {.count = 1, .reusable = true}}, SHIFT(167),
  [1102] = {.entry = {.count = 1, .reusable = false}}, SHIFT(168),
  [1104] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_receive_statement, 1, 0, 60),
  [1106] = {.entry = {.count = 1, .reusable = true}}, SHIFT(168),
  [1108] = {.entry = {.count = 1, .reusable = false}}, SHIFT(167),
  [1110] = {.entry = {.count = 1, .reusable = true}}, SHIFT(169),
  [1112] = {.entry = {.count = 1, .reusable = false}}, SHIFT(169),
  [1114] = {.entry = {.count = 1, .reusable = true}}, SHIFT(170),
  [1116] = {.entry = {.count = 1, .reusable = true}}, SHIFT(171),
  [1118] = {.entry = {.count = 1, .reusable = false}}, SHIFT(12),
  [1120] = {.entry = {.count = 1, .reusable = false}}, SHIFT(453),
  [1122] = {.entry = {.count = 1, .reusable = true}}, SHIFT(788),
  [1124] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1340),
  [1126] = {.entry = {.count = 1, .reusable = true}}, SHIFT(729),
  [1128] = {.entry = {.count = 1, .reusable = false}}, SHIFT(893),
  [1130] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1311),
  [1132] = {.entry = {.count = 1, .reusable = false}}, SHIFT(114),
  [1134] = {.entry = {.count = 1, .reusable = false}}, SHIFT(756),
  [1136] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1422),
  [1138] = {.entry = {.count = 1, .reusable = false}}, SHIFT(754),
  [1140] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1385),
  [1142] = {.entry = {.count = 1, .reusable = true}}, SHIFT(795),
  [1144] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1343),
  [1146] = {.entry = {.count = 1, .reusable = true}}, SHIFT(783),
  [1148] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1381),
  [1150] = {.entry = {.count = 1, .reusable = true}}, SHIFT(148),
  [1152] = {.entry = {.count = 1, .reusable = true}}, SHIFT(269),
  [1154] = {.entry = {.count = 1, .reusable = true}}, SHIFT(64),
  [1156] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1263),
  [1158] = {.entry = {.count = 1, .reusable = true}}, SHIFT(137),
  [1160] = {.entry = {.count = 1, .reusable = false}}, SHIFT(138),
  [1162] = {.entry = {.count = 1, .reusable = true}}, SHIFT(138),
  [1164] = {.entry = {.count = 1, .reusable = false}}, SHIFT(137),
  [1166] = {.entry = {.count = 1, .reusable = true}}, SHIFT(140),
  [1168] = {.entry = {.count = 1, .reusable = false}}, SHIFT(140),
  [1170] = {.entry = {.count = 1, .reusable = true}}, SHIFT(141),
  [1172] = {.entry = {.count = 1, .reusable = true}}, SHIFT(142),
  [1174] = {.entry = {.count = 1, .reusable = true}}, SHIFT(790),
  [1176] = {.entry = {.count = 1, .reusable = true}}, SHIFT(548),
  [1178] = {.entry = {.count = 1, .reusable = true}}, SHIFT(74),
  [1180] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_method_elem, 2, 0, 55),
  [1182] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_method_elem, 2, 0, 55),
  [1184] = {.entry = {.count = 1, .reusable = true}}, SHIFT(435),
  [1186] = {.entry = {.count = 1, .reusable = true}}, SHIFT(58),
  [1188] = {.entry = {.count = 1, .reusable = true}}, SHIFT(794),
  [1190] = {.entry = {.count = 1, .reusable = true}}, SHIFT(792),
  [1192] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1278),
  [1194] = {.entry = {.count = 1, .reusable = true}}, SHIFT(180),
  [1196] = {.entry = {.count = 1, .reusable = true}}, SHIFT(969),
  [1198] = {.entry = {.count = 1, .reusable = false}}, SHIFT(181),
  [1200] = {.entry = {.count = 1, .reusable = true}}, SHIFT(181),
  [1202] = {.entry = {.count = 1, .reusable = false}}, SHIFT(180),
  [1204] = {.entry = {.count = 1, .reusable = true}}, SHIFT(182),
  [1206] = {.entry = {.count = 1, .reusable = false}}, SHIFT(182),
  [1208] = {.entry = {.count = 1, .reusable = true}}, SHIFT(183),
  [1210] = {.entry = {.count = 1, .reusable = true}}, SHIFT(184),
  [1212] = {.entry = {.count = 1, .reusable = true}}, SHIFT(692),
  [1214] = {.entry = {.count = 1, .reusable = true}}, SHIFT(475),
  [1216] = {.entry = {.count = 1, .reusable = true}}, SHIFT(63),
  [1218] = {.entry = {.count = 1, .reusable = true}}, SHIFT(784),
  [1220] = {.entry = {.count = 1, .reusable = true}}, SHIFT(785),
  [1222] = {.entry = {.count = 1, .reusable = true}}, SHIFT(327),
  [1224] = {.entry = {.count = 1, .reusable = true}}, SHIFT(67),
  [1226] = {.entry = {.count = 1, .reusable = true}}, SHIFT(359),
  [1228] = {.entry = {.count = 1, .reusable = true}}, SHIFT(70),
  [1230] = {.entry = {.count = 1, .reusable = true}}, SHIFT(791),
  [1232] = {.entry = {.count = 1, .reusable = true}}, SHIFT(193),
  [1234] = {.entry = {.count = 1, .reusable = false}}, SHIFT(853),
  [1236] = {.entry = {.count = 1, .reusable = true}}, SHIFT(704),
  [1238] = {.entry = {.count = 1, .reusable = true}}, SHIFT(127),
  [1240] = {.entry = {.count = 1, .reusable = true}}, SHIFT(726),
  [1242] = {.entry = {.count = 1, .reusable = true}}, SHIFT(778),
  [1244] = {.entry = {.count = 1, .reusable = true}}, SHIFT(860),
  [1246] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1359),
  [1248] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_literal_element, 1, 0, 0),
  [1250] = {.entry = {.count = 1, .reusable = true}}, SHIFT(115),
  [1252] = {.entry = {.count = 1, .reusable = true}}, SHIFT(885),
  [1254] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_argument_list_repeat1, 2, 0, 0),
  [1256] = {.entry = {.count = 1, .reusable = true}}, SHIFT(826),
  [1258] = {.entry = {.count = 1, .reusable = true}}, SHIFT(825),
  [1260] = {.entry = {.count = 1, .reusable = true}}, SHIFT(817),
  [1262] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1316),
  [1264] = {.entry = {.count = 1, .reusable = true}}, SHIFT(945),
  [1266] = {.entry = {.count = 1, .reusable = true}}, SHIFT(876),
  [1268] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1023),
  [1270] = {.entry = {.count = 1, .reusable = true}}, SHIFT(983),
  [1272] = {.entry = {.count = 1, .reusable = true}}, SHIFT(111),
  [1274] = {.entry = {.count = 1, .reusable = true}}, SHIFT(18),
  [1276] = {.entry = {.count = 1, .reusable = true}}, SHIFT(432),
  [1278] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1424),
  [1280] = {.entry = {.count = 1, .reusable = true}}, SHIFT(286),
  [1282] = {.entry = {.count = 1, .reusable = true}}, SHIFT(133),
  [1284] = {.entry = {.count = 1, .reusable = true}}, SHIFT(418),
  [1286] = {.entry = {.count = 1, .reusable = true}}, SHIFT(119),
  [1288] = {.entry = {.count = 1, .reusable = true}}, SHIFT(16),
  [1290] = {.entry = {.count = 1, .reusable = true}}, SHIFT(473),
  [1292] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1328),
  [1294] = {.entry = {.count = 1, .reusable = true}}, SHIFT(477),
  [1296] = {.entry = {.count = 1, .reusable = true}}, SHIFT(85),
  [1298] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1012),
  [1300] = {.entry = {.count = 1, .reusable = true}}, SHIFT(979),
  [1302] = {.entry = {.count = 1, .reusable = true}}, SHIFT(862),
  [1304] = {.entry = {.count = 1, .reusable = true}}, SHIFT(484),
  [1306] = {.entry = {.count = 1, .reusable = true}}, SHIFT(222),
  [1308] = {.entry = {.count = 1, .reusable = true}}, SHIFT(811),
  [1310] = {.entry = {.count = 1, .reusable = true}}, SHIFT(400),
  [1312] = {.entry = {.count = 1, .reusable = true}}, SHIFT(218),
  [1314] = {.entry = {.count = 1, .reusable = true}}, SHIFT(984),
  [1316] = {.entry = {.count = 1, .reusable = true}}, SHIFT(268),
  [1318] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1370),
  [1320] = {.entry = {.count = 1, .reusable = true}}, SHIFT(490),
  [1322] = {.entry = {.count = 1, .reusable = true}}, SHIFT(223),
  [1324] = {.entry = {.count = 1, .reusable = true}}, SHIFT(17),
  [1326] = {.entry = {.count = 1, .reusable = true}}, SHIFT(318),
  [1328] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1408),
  [1330] = {.entry = {.count = 1, .reusable = true}}, SHIFT(329),
  [1332] = {.entry = {.count = 1, .reusable = true}}, SHIFT(97),
  [1334] = {.entry = {.count = 1, .reusable = true}}, SHIFT(305),
  [1336] = {.entry = {.count = 1, .reusable = true}}, SHIFT(226),
  [1338] = {.entry = {.count = 1, .reusable = true}}, SHIFT(282),
  [1340] = {.entry = {.count = 1, .reusable = true}}, SHIFT(217),
  [1342] = {.entry = {.count = 1, .reusable = true}}, SHIFT(311),
  [1344] = {.entry = {.count = 1, .reusable = true}}, SHIFT(227),
  [1346] = {.entry = {.count = 1, .reusable = true}}, SHIFT(865),
  [1348] = {.entry = {.count = 1, .reusable = true}}, SHIFT(357),
  [1350] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1338),
  [1352] = {.entry = {.count = 1, .reusable = true}}, SHIFT(810),
  [1354] = {.entry = {.count = 1, .reusable = true}}, SHIFT(361),
  [1356] = {.entry = {.count = 1, .reusable = true}}, SHIFT(109),
  [1358] = {.entry = {.count = 1, .reusable = true}}, SHIFT(368),
  [1360] = {.entry = {.count = 1, .reusable = true}}, SHIFT(230),
  [1362] = {.entry = {.count = 1, .reusable = true}}, SHIFT(374),
  [1364] = {.entry = {.count = 1, .reusable = true}}, SHIFT(231),
  [1366] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1397),
  [1368] = {.entry = {.count = 1, .reusable = true}}, SHIFT(13),
  [1370] = {.entry = {.count = 1, .reusable = true}}, SHIFT(546),
  [1372] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1329),
  [1374] = {.entry = {.count = 1, .reusable = true}}, SHIFT(410),
  [1376] = {.entry = {.count = 1, .reusable = true}}, SHIFT(219),
  [1378] = {.entry = {.count = 1, .reusable = true}}, SHIFT(758),
  [1380] = {.entry = {.count = 1, .reusable = true}}, SHIFT(73),
  [1382] = {.entry = {.count = 1, .reusable = true}}, SHIFT(550),
  [1384] = {.entry = {.count = 1, .reusable = true}}, SHIFT(121),
  [1386] = {.entry = {.count = 1, .reusable = true}}, SHIFT(557),
  [1388] = {.entry = {.count = 1, .reusable = true}}, SHIFT(234),
  [1390] = {.entry = {.count = 1, .reusable = true}}, SHIFT(563),
  [1392] = {.entry = {.count = 1, .reusable = true}}, SHIFT(132),
  [1394] = {.entry = {.count = 1, .reusable = false}}, SHIFT(938),
  [1396] = {.entry = {.count = 1, .reusable = true}}, SHIFT(272),
  [1398] = {.entry = {.count = 1, .reusable = true}}, SHIFT(89),
  [1400] = {.entry = {.count = 1, .reusable = true}}, SHIFT(350),
  [1402] = {.entry = {.count = 1, .reusable = true}}, SHIFT(275),
  [1404] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1331),
  [1406] = {.entry = {.count = 1, .reusable = true}}, SHIFT(273),
  [1408] = {.entry = {.count = 1, .reusable = true}}, SHIFT(289),
  [1410] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1371),
  [1412] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1378),
  [1414] = {.entry = {.count = 1, .reusable = true}}, SHIFT(409),
  [1416] = {.entry = {.count = 1, .reusable = true}}, SHIFT(746),
  [1418] = {.entry = {.count = 1, .reusable = true}}, SHIFT(436),
  [1420] = {.entry = {.count = 1, .reusable = true}}, SHIFT(26),
  [1422] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_receive_statement, 3, 0, 33),
  [1424] = {.entry = {.count = 1, .reusable = true}}, SHIFT(25),
  [1426] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_range_clause, 2, 0, 29),
  [1428] = {.entry = {.count = 1, .reusable = true}}, SHIFT(404),
  [1430] = {.entry = {.count = 1, .reusable = true}}, SHIFT(405),
  [1432] = {.entry = {.count = 1, .reusable = true}}, SHIFT(416),
  [1434] = {.entry = {.count = 1, .reusable = true}}, SHIFT(417),
  [1436] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1421),
  [1438] = {.entry = {.count = 1, .reusable = true}}, SHIFT(751),
  [1440] = {.entry = {.count = 1, .reusable = true}}, SHIFT(463),
  [1442] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_range_clause, 4, 0, 84),
  [1444] = {.entry = {.count = 1, .reusable = true}}, SHIFT(711),
  [1446] = {.entry = {.count = 1, .reusable = true}}, SHIFT(465),
  [1448] = {.entry = {.count = 1, .reusable = true}}, SHIFT(481),
  [1450] = {.entry = {.count = 1, .reusable = true}}, SHIFT(488),
  [1452] = {.entry = {.count = 1, .reusable = true}}, SHIFT(491),
  [1454] = {.entry = {.count = 1, .reusable = true}}, SHIFT(492),
  [1456] = {.entry = {.count = 1, .reusable = true}}, SHIFT(330),
  [1458] = {.entry = {.count = 1, .reusable = true}}, SHIFT(713),
  [1460] = {.entry = {.count = 1, .reusable = true}}, SHIFT(300),
  [1462] = {.entry = {.count = 1, .reusable = true}}, SHIFT(279),
  [1464] = {.entry = {.count = 1, .reusable = true}}, SHIFT(302),
  [1466] = {.entry = {.count = 1, .reusable = true}}, SHIFT(309),
  [1468] = {.entry = {.count = 1, .reusable = true}}, SHIFT(312),
  [1470] = {.entry = {.count = 1, .reusable = true}}, SHIFT(313),
  [1472] = {.entry = {.count = 1, .reusable = true}}, SHIFT(360),
  [1474] = {.entry = {.count = 1, .reusable = true}}, SHIFT(735),
  [1476] = {.entry = {.count = 1, .reusable = true}}, SHIFT(270),
  [1478] = {.entry = {.count = 1, .reusable = true}}, SHIFT(365),
  [1480] = {.entry = {.count = 1, .reusable = true}}, SHIFT(372),
  [1482] = {.entry = {.count = 1, .reusable = true}}, SHIFT(375),
  [1484] = {.entry = {.count = 1, .reusable = true}}, SHIFT(376),
  [1486] = {.entry = {.count = 1, .reusable = true}}, SHIFT(534),
  [1488] = {.entry = {.count = 1, .reusable = true}}, SHIFT(740),
  [1490] = {.entry = {.count = 1, .reusable = true}}, SHIFT(538),
  [1492] = {.entry = {.count = 1, .reusable = true}}, SHIFT(554),
  [1494] = {.entry = {.count = 1, .reusable = true}}, SHIFT(561),
  [1496] = {.entry = {.count = 1, .reusable = true}}, SHIFT(564),
  [1498] = {.entry = {.count = 1, .reusable = true}}, SHIFT(565),
  [1500] = {.entry = {.count = 1, .reusable = true}}, SHIFT(262),
  [1502] = {.entry = {.count = 1, .reusable = true}}, SHIFT(750),
  [1504] = {.entry = {.count = 1, .reusable = true}}, SHIFT(90),
  [1506] = {.entry = {.count = 1, .reusable = true}}, SHIFT(739),
  [1508] = {.entry = {.count = 1, .reusable = false}}, SHIFT(709),
  [1510] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1392),
  [1512] = {.entry = {.count = 1, .reusable = true}}, SHIFT(707),
  [1514] = {.entry = {.count = 1, .reusable = true}}, SHIFT(725),
  [1516] = {.entry = {.count = 1, .reusable = true}}, SHIFT(114),
  [1518] = {.entry = {.count = 1, .reusable = true}}, SHIFT(756),
  [1520] = {.entry = {.count = 1, .reusable = true}}, SHIFT(720),
  [1522] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1385),
  [1524] = {.entry = {.count = 1, .reusable = true}}, SHIFT(727),
  [1526] = {.entry = {.count = 1, .reusable = false}}, SHIFT(724),
  [1528] = {.entry = {.count = 1, .reusable = true}}, SHIFT(722),
  [1530] = {.entry = {.count = 1, .reusable = false}}, SHIFT(761),
  [1532] = {.entry = {.count = 1, .reusable = true}}, SHIFT(102),
  [1534] = {.entry = {.count = 1, .reusable = true}}, SHIFT(747),
  [1536] = {.entry = {.count = 1, .reusable = false}}, SHIFT(733),
  [1538] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1382),
  [1540] = {.entry = {.count = 1, .reusable = true}}, SHIFT(732),
  [1542] = {.entry = {.count = 1, .reusable = true}}, SHIFT(760),
  [1544] = {.entry = {.count = 1, .reusable = false}}, SHIFT(762),
  [1546] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1310),
  [1548] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parameter_list, 3, 0, 0),
  [1550] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameter_list, 3, 0, 0),
  [1552] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parameter_list, 2, 0, 0),
  [1554] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameter_list, 2, 0, 0),
  [1556] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parameter_list, 4, 0, 0),
  [1558] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameter_list, 4, 0, 0),
  [1560] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parameter_list, 5, 0, 0),
  [1562] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameter_list, 5, 0, 0),
  [1564] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_const_spec_repeat1, 2, 0, 0),
  [1566] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_const_spec_repeat1, 2, 0, 0),
  [1568] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_const_spec_repeat1, 2, 0, 0), SHIFT_REPEAT(1339),
  [1571] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_expression_list_repeat1, 2, 0, 0), SHIFT_REPEAT(194),
  [1574] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_expression_list, 2, 0, 0),
  [1576] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__simple_type, 1, -1, 1),
  [1578] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym__simple_type, 1, -1, 1), SHIFT(697),
  [1581] = {.entry = {.count = 1, .reusable = false}}, SHIFT(697),
  [1583] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_var_spec_repeat1, 2, 0, 40),
  [1585] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_var_spec_repeat1, 2, 0, 40),
  [1587] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_var_spec_repeat1, 2, 0, 40), SHIFT_REPEAT(1346),
  [1590] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_field_declaration_repeat1, 2, 0, 40),
  [1592] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_field_declaration_repeat1, 2, 0, 40),
  [1594] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_field_declaration_repeat1, 2, 0, 40), SHIFT_REPEAT(1397),
  [1597] = {.entry = {.count = 1, .reusable = true}}, SHIFT(124),
  [1599] = {.entry = {.count = 1, .reusable = true}}, SHIFT(126),
  [1601] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__simple_type, 1, 0, 0),
  [1603] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__simple_type, 1, 0, 0),
  [1605] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym__simple_type, 1, 0, 0), SHIFT(697),
  [1608] = {.entry = {.count = 1, .reusable = true}}, SHIFT(104),
  [1610] = {.entry = {.count = 1, .reusable = true}}, SHIFT(96),
  [1612] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_var_spec_repeat1, 2, 0, 38),
  [1614] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_var_spec_repeat1, 2, 0, 38),
  [1616] = {.entry = {.count = 1, .reusable = true}}, SHIFT(88),
  [1618] = {.entry = {.count = 1, .reusable = true}}, SHIFT(92),
  [1620] = {.entry = {.count = 1, .reusable = true}}, SHIFT(125),
  [1622] = {.entry = {.count = 1, .reusable = true}}, SHIFT(53),
  [1624] = {.entry = {.count = 1, .reusable = true}}, SHIFT(54),
  [1626] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_arguments, 5, 2, 0),
  [1628] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_arguments, 5, 2, 0),
  [1630] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_arguments, 4, 2, 0),
  [1632] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_arguments, 4, 2, 0),
  [1634] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_channel_type, 3, 0, 27),
  [1636] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_channel_type, 3, 0, 27),
  [1638] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_declaration_list, 5, 0, 0),
  [1640] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_declaration_list, 5, 0, 0),
  [1642] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_interface_type, 4, 0, 0),
  [1644] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_interface_type, 4, 0, 0),
  [1646] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_qualified_type, 3, 0, 31),
  [1648] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_qualified_type, 3, 0, 31),
  [1650] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_interface_type, 6, 0, 0),
  [1652] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_interface_type, 6, 0, 0),
  [1654] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_declaration_list, 2, 0, 0),
  [1656] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_declaration_list, 2, 0, 0),
  [1658] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_generic_type, 2, 1, 14),
  [1660] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_generic_type, 2, 1, 14),
  [1662] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_type, 4, 0, 48),
  [1664] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_array_type, 4, 0, 48),
  [1666] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parenthesized_type, 3, 0, 0),
  [1668] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parenthesized_type, 3, 0, 0),
  [1670] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_arguments, 3, 2, 0),
  [1672] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_arguments, 3, 2, 0),
  [1674] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_pointer_type, 2, 0, 0),
  [1676] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_pointer_type, 2, 0, 0),
  [1678] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_field_declaration_repeat1, 2, 0, 76),
  [1680] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_field_declaration_repeat1, 2, 0, 76),
  [1682] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_interface_type, 5, 0, 0),
  [1684] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_interface_type, 5, 0, 0),
  [1686] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_interface_type, 3, 0, 0),
  [1688] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_interface_type, 3, 0, 0),
  [1690] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_generic_type, 2, 1, 11),
  [1692] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_generic_type, 2, 1, 11),
  [1694] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_negated_type, 2, 0, 0),
  [1696] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_negated_type, 2, 0, 0),
  [1698] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_declaration_list, 3, 0, 0),
  [1700] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_declaration_list, 3, 0, 0),
  [1702] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_type, 2, 0, 0),
  [1704] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_type, 2, 0, 0),
  [1706] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_channel_type, 2, 0, 7),
  [1708] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_channel_type, 2, 0, 7),
  [1710] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_map_type, 5, 0, 80),
  [1712] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_map_type, 5, 0, 80),
  [1714] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_slice_type, 3, 0, 23),
  [1716] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_slice_type, 3, 0, 23),
  [1718] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_declaration_list, 4, 0, 0),
  [1720] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_declaration_list, 4, 0, 0),
  [1722] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_type, 3, 0, 21),
  [1724] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_type, 3, 0, 21),
  [1726] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_parameter_list, 4, 0, 0),
  [1728] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_parameter_list, 4, 0, 0),
  [1730] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1118),
  [1732] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1264),
  [1734] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1128),
  [1736] = {.entry = {.count = 1, .reusable = true}}, SHIFT(847),
  [1738] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1358),
  [1740] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1077),
  [1742] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_parameter_list, 5, 0, 0),
  [1744] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_parameter_list, 5, 0, 0),
  [1746] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_parameter_list, 3, 0, 0),
  [1748] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_parameter_list, 3, 0, 0),
  [1750] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1071),
  [1752] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1069),
  [1754] = {.entry = {.count = 1, .reusable = true}}, SHIFT(846),
  [1756] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_declaration, 2, 0, 51),
  [1758] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_declaration, 2, 0, 51),
  [1760] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1417),
  [1762] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__simple_type, 1, -1, 1), SHIFT(649),
  [1765] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_declaration, 2, 0, 41),
  [1767] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_declaration, 2, 0, 41),
  [1769] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1190),
  [1771] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1092),
  [1773] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1058),
  [1775] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_declaration, 1, 0, 19),
  [1777] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_declaration, 1, 0, 19),
  [1779] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__simple_type, 1, 0, 0), SHIFT(649),
  [1782] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1204),
  [1784] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1159),
  [1786] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1068),
  [1788] = {.entry = {.count = 1, .reusable = true}}, SHIFT(649),
  [1790] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_expression_list, 2, 0, 0),
  [1792] = {.entry = {.count = 1, .reusable = true}}, SHIFT(385),
  [1794] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1041),
  [1796] = {.entry = {.count = 1, .reusable = true}}, SHIFT(975),
  [1798] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_expression_list_repeat1, 2, 0, 0), SHIFT_REPEAT(177),
  [1801] = {.entry = {.count = 1, .reusable = true}}, SHIFT(813),
  [1803] = {.entry = {.count = 1, .reusable = true}}, SHIFT(941),
  [1805] = {.entry = {.count = 1, .reusable = true}}, SHIFT(835),
  [1807] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1366),
  [1809] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym__simple_type, 1, -1, 1), SHIFT(698),
  [1812] = {.entry = {.count = 1, .reusable = true}}, SHIFT(982),
  [1814] = {.entry = {.count = 1, .reusable = true}}, SHIFT(870),
  [1816] = {.entry = {.count = 1, .reusable = false}}, SHIFT(698),
  [1818] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_declaration, 3, 0, 78),
  [1820] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_declaration, 3, 0, 78),
  [1822] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_declaration, 2, 0, 52),
  [1824] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_declaration, 2, 0, 52),
  [1826] = {.entry = {.count = 1, .reusable = true}}, SHIFT(859),
  [1828] = {.entry = {.count = 1, .reusable = true}}, SHIFT(818),
  [1830] = {.entry = {.count = 1, .reusable = true}}, SHIFT(856),
  [1832] = {.entry = {.count = 1, .reusable = false}}, SHIFT(942),
  [1834] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_break_statement, 1, 0, 0),
  [1836] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_break_statement, 1, 0, 0),
  [1838] = {.entry = {.count = 1, .reusable = true}}, SHIFT(10),
  [1840] = {.entry = {.count = 1, .reusable = false}}, SHIFT(10),
  [1842] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_var_spec, 3, 0, 39),
  [1844] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_var_spec, 3, 0, 39),
  [1846] = {.entry = {.count = 1, .reusable = false}}, SHIFT(99),
  [1848] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym__simple_type, 1, 0, 0), SHIFT(698),
  [1851] = {.entry = {.count = 1, .reusable = false}}, SHIFT(107),
  [1853] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_var_spec, 2, 0, 18),
  [1855] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_var_spec, 2, 0, 18),
  [1857] = {.entry = {.count = 1, .reusable = false}}, SHIFT(113),
  [1859] = {.entry = {.count = 1, .reusable = true}}, SHIFT(9),
  [1861] = {.entry = {.count = 1, .reusable = false}}, SHIFT(9),
  [1863] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__statement_list, 1, 0, 0),
  [1865] = {.entry = {.count = 1, .reusable = false}}, SHIFT(947),
  [1867] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_continue_statement, 1, 0, 0),
  [1869] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_continue_statement, 1, 0, 0),
  [1871] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_if_statement, 3, 0, 28),
  [1873] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_if_statement, 3, 0, 28),
  [1875] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1099),
  [1877] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__statement_list_repeat1, 2, 0, 0), SHIFT_REPEAT(20),
  [1880] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__statement_list_repeat1, 2, 0, 0), SHIFT_REPEAT(20),
  [1883] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym__statement_list_repeat1, 2, 0, 0),
  [1885] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_if_statement, 5, 0, 81),
  [1887] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_if_statement, 5, 0, 81),
  [1889] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1105),
  [1891] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_const_spec, 3, 0, 37),
  [1893] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_const_spec, 3, 0, 37),
  [1895] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__statement_list_repeat1, 2, 0, 0),
  [1897] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_fallthrough_statement, 1, 0, 0),
  [1899] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_fallthrough_statement, 1, 0, 0),
  [1901] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_declaration, 4, 0, 43),
  [1903] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_declaration, 4, 0, 43),
  [1905] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_if_statement, 7, 0, 103),
  [1907] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_if_statement, 7, 0, 103),
  [1909] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_elem, 2, 0, 0),
  [1911] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_elem, 2, 0, 0),
  [1913] = {.entry = {.count = 1, .reusable = false}}, SHIFT(769),
  [1915] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_expression_switch_statement, 7, 0, 100),
  [1917] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_expression_switch_statement, 7, 0, 100),
  [1919] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_declaration, 4, 0, 0),
  [1921] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_declaration, 4, 0, 0),
  [1923] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_alias, 3, 0, 49),
  [1925] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_alias, 3, 0, 49),
  [1927] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__simple_statement, 1, 0, 0),
  [1929] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__simple_statement, 1, 0, 0),
  [1931] = {.entry = {.count = 1, .reusable = true}}, SHIFT(511),
  [1933] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_expression_switch_statement, 6, 0, 87),
  [1935] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_expression_switch_statement, 6, 0, 87),
  [1937] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_break_statement, 2, 0, 8),
  [1939] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_break_statement, 2, 0, 8),
  [1941] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_var_spec_list, 3, 0, 0),
  [1943] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_var_spec_list, 3, 0, 0),
  [1945] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_continue_statement, 2, 0, 8),
  [1947] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_continue_statement, 2, 0, 8),
  [1949] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_expression_switch_statement, 4, 0, 0),
  [1951] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_expression_switch_statement, 4, 0, 0),
  [1953] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_expression_switch_statement_repeat1, 2, 0, 0),
  [1955] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_expression_switch_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(87),
  [1958] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_expression_switch_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1333),
  [1961] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_inc_statement, 2, 0, 0),
  [1963] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_inc_statement, 2, 0, 0),
  [1965] = {.entry = {.count = 1, .reusable = true}}, SHIFT(977),
  [1967] = {.entry = {.count = 1, .reusable = true}}, SHIFT(87),
  [1969] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1333),
  [1971] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_dec_statement, 2, 0, 0),
  [1973] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_dec_statement, 2, 0, 0),
  [1975] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_expression_switch_statement, 6, 0, 100),
  [1977] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_expression_switch_statement, 6, 0, 100),
  [1979] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_switch_statement, 4, 0, 59),
  [1981] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_switch_statement, 4, 0, 59),
  [1983] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_switch_statement, 5, 0, 59),
  [1985] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_switch_statement, 5, 0, 59),
  [1987] = {.entry = {.count = 1, .reusable = true}}, SHIFT(957),
  [1989] = {.entry = {.count = 1, .reusable = true}}, SHIFT(718),
  [1991] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_spec, 3, 0, 50),
  [1993] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_spec, 3, 0, 50),
  [1995] = {.entry = {.count = 1, .reusable = true}}, SHIFT(931),
  [1997] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_elem, 1, 0, 0),
  [1999] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_elem, 1, 0, 0),
  [2001] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_expression_list_repeat1, 2, 0, 0), SHIFT_REPEAT(136),
  [2004] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_expression_switch_statement, 4, 0, 7),
  [2006] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_expression_switch_statement, 4, 0, 7),
  [2008] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1003),
  [2010] = {.entry = {.count = 1, .reusable = true}}, SHIFT(955),
  [2012] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_select_statement, 4, 0, 0),
  [2014] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_select_statement, 4, 0, 0),
  [2016] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_select_statement_repeat1, 2, 0, 0),
  [2018] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_select_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(49),
  [2021] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_select_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1333),
  [2024] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_goto_statement, 2, 0, 8),
  [2026] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_goto_statement, 2, 0, 8),
  [2028] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_type_switch_statement_repeat1, 2, 0, 0),
  [2030] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_type_switch_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(718),
  [2033] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_type_switch_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1333),
  [2036] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_return_statement, 2, 0, 0),
  [2038] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_return_statement, 2, 0, 0),
  [2040] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_expression_switch_statement, 5, 0, 87),
  [2042] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_expression_switch_statement, 5, 0, 87),
  [2044] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_var_spec, 3, 0, 37),
  [2046] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_var_spec, 3, 0, 37),
  [2048] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_empty_statement, 1, 0, 0),
  [2050] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_empty_statement, 1, 0, 0),
  [2052] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__statement, 1, 0, 0),
  [2054] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__statement, 1, 0, 0),
  [2056] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_const_declaration, 2, 0, 0),
  [2058] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_const_declaration, 2, 0, 0),
  [2060] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_const_declaration, 3, 0, 0),
  [2062] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_const_declaration, 3, 0, 0),
  [2064] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_for_statement, 2, 0, 9),
  [2066] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_for_statement, 2, 0, 9),
  [2068] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_const_declaration, 4, 0, 0),
  [2070] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_const_declaration, 4, 0, 0),
  [2072] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_var_declaration, 2, 0, 0),
  [2074] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_var_declaration, 2, 0, 0),
  [2076] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_const_spec, 4, 0, 64),
  [2078] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_const_spec, 4, 0, 64),
  [2080] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_for_statement, 3, 0, 30),
  [2082] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_for_statement, 3, 0, 30),
  [2084] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_expression_switch_statement, 3, 0, 0),
  [2086] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_expression_switch_statement, 3, 0, 0),
  [2088] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_const_spec, 4, 0, 65),
  [2090] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_const_spec, 4, 0, 65),
  [2092] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_var_spec, 4, 0, 64),
  [2094] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_var_spec, 4, 0, 64),
  [2096] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_var_spec, 4, 0, 66),
  [2098] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_var_spec, 4, 0, 66),
  [2100] = {.entry = {.count = 1, .reusable = true}}, SHIFT(949),
  [2102] = {.entry = {.count = 1, .reusable = true}}, SHIFT(993),
  [2104] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_const_spec, 5, 0, 93),
  [2106] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_const_spec, 5, 0, 93),
  [2108] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_var_spec, 5, 0, 94),
  [2110] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_var_spec, 5, 0, 94),
  [2112] = {.entry = {.count = 1, .reusable = true}}, SHIFT(956),
  [2114] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_expression_switch_statement, 5, 0, 7),
  [2116] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_expression_switch_statement, 5, 0, 7),
  [2118] = {.entry = {.count = 1, .reusable = true}}, SHIFT(963),
  [2120] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1006),
  [2122] = {.entry = {.count = 1, .reusable = true}}, SHIFT(49),
  [2124] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_select_statement, 3, 0, 0),
  [2126] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_select_statement, 3, 0, 0),
  [2128] = {.entry = {.count = 1, .reusable = true}}, SHIFT(971),
  [2130] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_declaration, 5, 0, 69),
  [2132] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_declaration, 5, 0, 69),
  [2134] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_method_declaration, 5, 0, 72),
  [2136] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_method_declaration, 5, 0, 72),
  [2138] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_labeled_statement, 3, 0, 26),
  [2140] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_labeled_statement, 3, 0, 26),
  [2142] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_declaration, 2, 0, 0),
  [2144] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_declaration, 2, 0, 0),
  [2146] = {.entry = {.count = 1, .reusable = true}}, SHIFT(510),
  [2148] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_assignment_statement, 3, 0, 32),
  [2150] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_assignment_statement, 3, 0, 32),
  [2152] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_short_var_declaration, 3, 0, 33),
  [2154] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_short_var_declaration, 3, 0, 33),
  [2156] = {.entry = {.count = 1, .reusable = true}}, SHIFT(940),
  [2158] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_declaration, 3, 0, 0),
  [2160] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_declaration, 3, 0, 0),
  [2162] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_type_elem_repeat1, 2, 0, 0),
  [2164] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_type_elem_repeat1, 2, 0, 0),
  [2166] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_type_elem_repeat1, 2, 0, 0), SHIFT_REPEAT(769),
  [2169] = {.entry = {.count = 1, .reusable = true}}, SHIFT(216),
  [2171] = {.entry = {.count = 1, .reusable = true}}, SHIFT(766),
  [2173] = {.entry = {.count = 1, .reusable = true}}, SHIFT(710),
  [2175] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_if_statement, 5, 0, 82),
  [2177] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_if_statement, 5, 0, 82),
  [2179] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_var_spec_list, 2, 0, 0),
  [2181] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_var_spec_list, 2, 0, 0),
  [2183] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_spec, 2, 0, 24),
  [2185] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_spec, 2, 0, 24),
  [2187] = {.entry = {.count = 1, .reusable = true}}, SHIFT(903),
  [2189] = {.entry = {.count = 1, .reusable = false}}, SHIFT(903),
  [2191] = {.entry = {.count = 1, .reusable = false}}, SHIFT(856),
  [2193] = {.entry = {.count = 1, .reusable = true}}, SHIFT(890),
  [2195] = {.entry = {.count = 1, .reusable = false}}, SHIFT(890),
  [2197] = {.entry = {.count = 1, .reusable = false}}, SHIFT(835),
  [2199] = {.entry = {.count = 1, .reusable = true}}, SHIFT(32),
  [2201] = {.entry = {.count = 1, .reusable = true}}, SHIFT(851),
  [2203] = {.entry = {.count = 1, .reusable = false}}, SHIFT(851),
  [2205] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1147),
  [2207] = {.entry = {.count = 1, .reusable = true}}, SHIFT(31),
  [2209] = {.entry = {.count = 1, .reusable = true}}, SHIFT(579),
  [2211] = {.entry = {.count = 1, .reusable = false}}, SHIFT(579),
  [2213] = {.entry = {.count = 1, .reusable = false}}, SHIFT(825),
  [2215] = {.entry = {.count = 1, .reusable = true}}, SHIFT(848),
  [2217] = {.entry = {.count = 1, .reusable = false}}, SHIFT(848),
  [2219] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1137),
  [2221] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_type_declaration_repeat1, 2, 0, 0), SHIFT_REPEAT(638),
  [2224] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_type_declaration_repeat1, 2, 0, 0),
  [2226] = {.entry = {.count = 1, .reusable = true}}, SHIFT(892),
  [2228] = {.entry = {.count = 1, .reusable = false}}, SHIFT(892),
  [2230] = {.entry = {.count = 1, .reusable = false}}, SHIFT(829),
  [2232] = {.entry = {.count = 1, .reusable = true}}, SHIFT(574),
  [2234] = {.entry = {.count = 1, .reusable = false}}, SHIFT(574),
  [2236] = {.entry = {.count = 1, .reusable = false}}, SHIFT(872),
  [2238] = {.entry = {.count = 1, .reusable = true}}, SHIFT(638),
  [2240] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1017),
  [2242] = {.entry = {.count = 1, .reusable = true}}, SHIFT(577),
  [2244] = {.entry = {.count = 1, .reusable = false}}, SHIFT(577),
  [2246] = {.entry = {.count = 1, .reusable = false}}, SHIFT(814),
  [2248] = {.entry = {.count = 1, .reusable = true}}, SHIFT(887),
  [2250] = {.entry = {.count = 1, .reusable = false}}, SHIFT(887),
  [2252] = {.entry = {.count = 1, .reusable = false}}, SHIFT(919),
  [2254] = {.entry = {.count = 1, .reusable = true}}, SHIFT(589),
  [2256] = {.entry = {.count = 1, .reusable = false}}, SHIFT(589),
  [2258] = {.entry = {.count = 1, .reusable = false}}, SHIFT(968),
  [2260] = {.entry = {.count = 1, .reusable = true}}, SHIFT(843),
  [2262] = {.entry = {.count = 1, .reusable = true}}, SHIFT(854),
  [2264] = {.entry = {.count = 1, .reusable = false}}, SHIFT(854),
  [2266] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1058),
  [2268] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_import_spec_list_repeat1, 2, 0, 0), SHIFT_REPEAT(882),
  [2271] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_import_spec_list_repeat1, 2, 0, 0), SHIFT_REPEAT(882),
  [2274] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_import_spec_list_repeat1, 2, 0, 0),
  [2276] = {.entry = {.count = 1, .reusable = true}}, SHIFT(894),
  [2278] = {.entry = {.count = 1, .reusable = false}}, SHIFT(894),
  [2280] = {.entry = {.count = 1, .reusable = false}}, SHIFT(975),
  [2282] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_field_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(906),
  [2285] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_field_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(906),
  [2288] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_field_declaration_list_repeat1, 2, 0, 0),
  [2290] = {.entry = {.count = 1, .reusable = true}}, SHIFT(568),
  [2292] = {.entry = {.count = 1, .reusable = false}}, SHIFT(568),
  [2294] = {.entry = {.count = 1, .reusable = false}}, SHIFT(885),
  [2296] = {.entry = {.count = 1, .reusable = true}}, SHIFT(591),
  [2298] = {.entry = {.count = 1, .reusable = false}}, SHIFT(591),
  [2300] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1023),
  [2302] = {.entry = {.count = 1, .reusable = true}}, SHIFT(852),
  [2304] = {.entry = {.count = 1, .reusable = false}}, SHIFT(852),
  [2306] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1204),
  [2308] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_interface_type_repeat1, 2, 0, 0), SHIFT_REPEAT(623),
  [2311] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_interface_type_repeat1, 2, 0, 0), SHIFT_REPEAT(623),
  [2314] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_interface_type_repeat1, 2, 0, 0),
  [2316] = {.entry = {.count = 1, .reusable = true}}, SHIFT(933),
  [2318] = {.entry = {.count = 1, .reusable = true}}, SHIFT(905),
  [2320] = {.entry = {.count = 1, .reusable = false}}, SHIFT(905),
  [2322] = {.entry = {.count = 1, .reusable = false}}, SHIFT(863),
  [2324] = {.entry = {.count = 1, .reusable = true}}, SHIFT(29),
  [2326] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_declaration, 2, 0, 54),
  [2328] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_declaration, 2, 0, 54),
  [2330] = {.entry = {.count = 1, .reusable = true}}, SHIFT(394),
  [2332] = {.entry = {.count = 1, .reusable = true}}, SHIFT(989),
  [2334] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_import_spec_list, 4, 0, 0),
  [2336] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_import_spec_list, 4, 0, 0),
  [2338] = {.entry = {.count = 1, .reusable = true}}, SHIFT(593),
  [2340] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1021),
  [2342] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_import_spec_list_repeat1, 2, 0, 0),
  [2344] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1070),
  [2346] = {.entry = {.count = 1, .reusable = true}}, SHIFT(315),
  [2348] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_declaration, 4, 0, 98),
  [2350] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_declaration, 4, 0, 98),
  [2352] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_const_declaration_repeat1, 2, 0, 0), SHIFT_REPEAT(394),
  [2355] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_const_declaration_repeat1, 2, 0, 0),
  [2357] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_import_spec, 2, 0, 15),
  [2359] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_import_spec, 2, 0, 15),
  [2361] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_import_spec, 2, 0, 16),
  [2363] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_import_spec, 2, 0, 16),
  [2365] = {.entry = {.count = 1, .reusable = true}}, SHIFT(42),
  [2367] = {.entry = {.count = 1, .reusable = true}}, SHIFT(472),
  [2369] = {.entry = {.count = 1, .reusable = true}}, SHIFT(48),
  [2371] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_import_spec_list, 5, 0, 0),
  [2373] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_import_spec_list, 5, 0, 0),
  [2375] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1115),
  [2377] = {.entry = {.count = 1, .reusable = true}}, SHIFT(326),
  [2379] = {.entry = {.count = 1, .reusable = true}}, SHIFT(40),
  [2381] = {.entry = {.count = 1, .reusable = true}}, SHIFT(356),
  [2383] = {.entry = {.count = 1, .reusable = true}}, SHIFT(35),
  [2385] = {.entry = {.count = 1, .reusable = true}}, SHIFT(431),
  [2387] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1276),
  [2389] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1078),
  [2391] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1090),
  [2393] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1084),
  [2395] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1132),
  [2397] = {.entry = {.count = 1, .reusable = true}}, SHIFT(413),
  [2399] = {.entry = {.count = 1, .reusable = true}}, SHIFT(987),
  [2401] = {.entry = {.count = 1, .reusable = true}}, SHIFT(249),
  [2403] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_source_file, 2, 0, 0),
  [2405] = {.entry = {.count = 1, .reusable = true}}, SHIFT(246),
  [2407] = {.entry = {.count = 1, .reusable = false}}, SHIFT(246),
  [2409] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1095),
  [2411] = {.entry = {.count = 1, .reusable = true}}, SHIFT(377),
  [2413] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_import_spec_list, 2, 0, 0),
  [2415] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_import_spec_list, 2, 0, 0),
  [2417] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1036),
  [2419] = {.entry = {.count = 1, .reusable = true}}, SHIFT(379),
  [2421] = {.entry = {.count = 1, .reusable = true}}, SHIFT(46),
  [2423] = {.entry = {.count = 1, .reusable = true}}, SHIFT(545),
  [2425] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_type_elem_repeat1, 2, 0, 0), SHIFT_REPEAT(710),
  [2428] = {.entry = {.count = 1, .reusable = true}}, SHIFT(27),
  [2430] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1121),
  [2432] = {.entry = {.count = 1, .reusable = true}}, SHIFT(493),
  [2434] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_declaration, 5, 0, 70),
  [2436] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_declaration, 5, 0, 70),
  [2438] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_declaration, 5, 0, 71),
  [2440] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_declaration, 5, 0, 71),
  [2442] = {.entry = {.count = 1, .reusable = true}}, SHIFT(37),
  [2444] = {.entry = {.count = 1, .reusable = true}}, SHIFT(267),
  [2446] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_method_declaration, 6, 0, 97),
  [2448] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_method_declaration, 6, 0, 97),
  [2450] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_method_declaration, 5, 0, 73),
  [2452] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_method_declaration, 5, 0, 73),
  [2454] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_declaration, 3, 0, 74),
  [2456] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_declaration, 3, 0, 74),
  [2458] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_declaration, 3, 0, 75),
  [2460] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_declaration, 3, 0, 75),
  [2462] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1110),
  [2464] = {.entry = {.count = 1, .reusable = true}}, SHIFT(567),
  [2466] = {.entry = {.count = 1, .reusable = true}}, SHIFT(571),
  [2468] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_declaration, 4, 0, 44),
  [2470] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_declaration, 4, 0, 44),
  [2472] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_declaration, 3, 0, 77),
  [2474] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_declaration, 3, 0, 77),
  [2476] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_import_spec, 1, 0, 3),
  [2478] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_import_spec, 1, 0, 3),
  [2480] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_interpreted_string_literal_repeat1, 2, 0, 0), SHIFT_REPEAT(1115),
  [2483] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_interpreted_string_literal_repeat1, 2, 0, 0),
  [2485] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_var_spec_list_repeat1, 2, 0, 0), SHIFT_REPEAT(593),
  [2488] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_var_spec_list_repeat1, 2, 0, 0),
  [2490] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_package_clause, 2, 0, 2),
  [2492] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_package_clause, 2, 0, 2),
  [2494] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_field_declaration_list_repeat1, 2, 0, 0),
  [2496] = {.entry = {.count = 1, .reusable = true}}, SHIFT(496),
  [2498] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_declaration, 2, 0, 53),
  [2500] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_declaration, 2, 0, 53),
  [2502] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_method_elem, 3, 0, 79),
  [2504] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_method_elem, 3, 0, 79),
  [2506] = {.entry = {.count = 1, .reusable = true}}, SHIFT(38),
  [2508] = {.entry = {.count = 1, .reusable = true}}, SHIFT(317),
  [2510] = {.entry = {.count = 1, .reusable = true}}, SHIFT(403),
  [2512] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_interface_type_repeat1, 2, 0, 0),
  [2514] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1085),
  [2516] = {.entry = {.count = 1, .reusable = true}}, SHIFT(253),
  [2518] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_import_declaration, 2, 0, 0),
  [2520] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_import_declaration, 2, 0, 0),
  [2522] = {.entry = {.count = 1, .reusable = true}}, SHIFT(943),
  [2524] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_import_spec_list, 3, 0, 0),
  [2526] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_import_spec_list, 3, 0, 0),
  [2528] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1059),
  [2530] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_type_case_repeat1, 2, 0, 0), SHIFT_REPEAT(701),
  [2533] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_type_case_repeat1, 2, 0, 0),
  [2535] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_declaration, 6, 0, 96),
  [2537] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_declaration, 6, 0, 96),
  [2539] = {.entry = {.count = 1, .reusable = true}}, SHIFT(39),
  [2541] = {.entry = {.count = 1, .reusable = true}}, SHIFT(528),
  [2543] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1076),
  [2545] = {.entry = {.count = 1, .reusable = true}}, SHIFT(86),
  [2547] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_literal_value_repeat1, 2, 0, 0),
  [2549] = {.entry = {.count = 1, .reusable = true}}, SHIFT(618),
  [2551] = {.entry = {.count = 1, .reusable = true}}, SHIFT(128),
  [2553] = {.entry = {.count = 1, .reusable = true}}, SHIFT(43),
  [2555] = {.entry = {.count = 1, .reusable = true}}, SHIFT(672),
  [2557] = {.entry = {.count = 1, .reusable = true}}, SHIFT(65),
  [2559] = {.entry = {.count = 1, .reusable = true}}, SHIFT(532),
  [2561] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_argument_list_repeat1, 2, 0, 0), SHIFT_REPEAT(112),
  [2564] = {.entry = {.count = 1, .reusable = true}}, SHIFT(628),
  [2566] = {.entry = {.count = 1, .reusable = true}}, SHIFT(793),
  [2568] = {.entry = {.count = 1, .reusable = true}}, SHIFT(521),
  [2570] = {.entry = {.count = 1, .reusable = true}}, SHIFT(430),
  [2572] = {.entry = {.count = 1, .reusable = true}}, SHIFT(108),
  [2574] = {.entry = {.count = 1, .reusable = true}}, SHIFT(616),
  [2576] = {.entry = {.count = 1, .reusable = true}}, SHIFT(822),
  [2578] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1315),
  [2580] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1315),
  [2582] = {.entry = {.count = 1, .reusable = true}}, SHIFT(660),
  [2584] = {.entry = {.count = 1, .reusable = true}}, SHIFT(434),
  [2586] = {.entry = {.count = 1, .reusable = true}}, SHIFT(650),
  [2588] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1217),
  [2590] = {.entry = {.count = 1, .reusable = true}}, SHIFT(841),
  [2592] = {.entry = {.count = 1, .reusable = true}}, SHIFT(338),
  [2594] = {.entry = {.count = 1, .reusable = true}}, SHIFT(94),
  [2596] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_expression_list_repeat1, 2, 0, 0), SHIFT_REPEAT(187),
  [2599] = {.entry = {.count = 1, .reusable = true}}, SHIFT(679),
  [2601] = {.entry = {.count = 1, .reusable = true}}, SHIFT(320),
  [2603] = {.entry = {.count = 1, .reusable = true}}, SHIFT(98),
  [2605] = {.entry = {.count = 1, .reusable = true}}, SHIFT(44),
  [2607] = {.entry = {.count = 1, .reusable = true}}, SHIFT(680),
  [2609] = {.entry = {.count = 1, .reusable = true}}, SHIFT(68),
  [2611] = {.entry = {.count = 1, .reusable = true}}, SHIFT(52),
  [2613] = {.entry = {.count = 1, .reusable = true}}, SHIFT(522),
  [2615] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1272),
  [2617] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1272),
  [2619] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_case, 5, 0, 104),
  [2621] = {.entry = {.count = 1, .reusable = true}}, SHIFT(120),
  [2623] = {.entry = {.count = 1, .reusable = true}}, SHIFT(789),
  [2625] = {.entry = {.count = 1, .reusable = true}}, SHIFT(516),
  [2627] = {.entry = {.count = 1, .reusable = true}}, SHIFT(630),
  [2629] = {.entry = {.count = 1, .reusable = true}}, SHIFT(41),
  [2631] = {.entry = {.count = 1, .reusable = true}}, SHIFT(662),
  [2633] = {.entry = {.count = 1, .reusable = true}}, SHIFT(60),
  [2635] = {.entry = {.count = 1, .reusable = true}}, SHIFT(355),
  [2637] = {.entry = {.count = 1, .reusable = true}}, SHIFT(106),
  [2639] = {.entry = {.count = 1, .reusable = true}}, SHIFT(686),
  [2641] = {.entry = {.count = 1, .reusable = true}}, SHIFT(358),
  [2643] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_literal_value_repeat1, 2, 0, 0), SHIFT_REPEAT(47),
  [2646] = {.entry = {.count = 1, .reusable = true}}, SHIFT(110),
  [2648] = {.entry = {.count = 1, .reusable = true}}, SHIFT(687),
  [2650] = {.entry = {.count = 1, .reusable = true}}, SHIFT(71),
  [2652] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_type_arguments_repeat1, 2, 0, 0), SHIFT_REPEAT(661),
  [2655] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_type_arguments_repeat1, 2, 0, 0),
  [2657] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_parameter_list_repeat1, 2, 0, 0),
  [2659] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_parameter_list_repeat1, 2, 0, 0), SHIFT_REPEAT(578),
  [2662] = {.entry = {.count = 1, .reusable = true}}, SHIFT(266),
  [2664] = {.entry = {.count = 1, .reusable = true}}, SHIFT(81),
  [2666] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1083),
  [2668] = {.entry = {.count = 1, .reusable = true}}, SHIFT(544),
  [2670] = {.entry = {.count = 1, .reusable = true}}, SHIFT(118),
  [2672] = {.entry = {.count = 1, .reusable = true}}, SHIFT(693),
  [2674] = {.entry = {.count = 1, .reusable = true}}, SHIFT(547),
  [2676] = {.entry = {.count = 1, .reusable = true}}, SHIFT(614),
  [2678] = {.entry = {.count = 1, .reusable = true}}, SHIFT(878),
  [2680] = {.entry = {.count = 1, .reusable = true}}, SHIFT(581),
  [2682] = {.entry = {.count = 1, .reusable = true}}, SHIFT(838),
  [2684] = {.entry = {.count = 1, .reusable = true}}, SHIFT(122),
  [2686] = {.entry = {.count = 1, .reusable = true}}, SHIFT(36),
  [2688] = {.entry = {.count = 1, .reusable = true}}, SHIFT(694),
  [2690] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1240),
  [2692] = {.entry = {.count = 1, .reusable = true}}, SHIFT(75),
  [2694] = {.entry = {.count = 1, .reusable = true}}, SHIFT(701),
  [2696] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6),
  [2698] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_case, 4, 0, 41),
  [2700] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1080),
  [2702] = {.entry = {.count = 1, .reusable = true}}, SHIFT(678),
  [2704] = {.entry = {.count = 1, .reusable = true}}, SHIFT(254),
  [2706] = {.entry = {.count = 1, .reusable = true}}, SHIFT(8),
  [2708] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_expression_list_repeat1, 2, 0, 0), SHIFT_REPEAT(193),
  [2711] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__statement_list, 4, 0, 0),
  [2713] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_communication_case, 4, 0, 88),
  [2715] = {.entry = {.count = 1, .reusable = true}}, SHIFT(879),
  [2717] = {.entry = {.count = 1, .reusable = true}}, SHIFT(782),
  [2719] = {.entry = {.count = 1, .reusable = true}}, SHIFT(527),
  [2721] = {.entry = {.count = 1, .reusable = true}}, SHIFT(840),
  [2723] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_type_parameter_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1289),
  [2726] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_type_parameter_list_repeat1, 2, 0, 0),
  [2728] = {.entry = {.count = 1, .reusable = true}}, SHIFT(471),
  [2730] = {.entry = {.count = 1, .reusable = true}}, SHIFT(82),
  [2732] = {.entry = {.count = 1, .reusable = true}}, SHIFT(613),
  [2734] = {.entry = {.count = 1, .reusable = true}}, SHIFT(970),
  [2736] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_expression_case, 4, 0, 7),
  [2738] = {.entry = {.count = 1, .reusable = true}}, SHIFT(671),
  [2740] = {.entry = {.count = 1, .reusable = true}}, SHIFT(474),
  [2742] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1283),
  [2744] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1283),
  [2746] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_default_case, 3, 0, 0),
  [2748] = {.entry = {.count = 1, .reusable = true}}, SHIFT(45),
  [2750] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_parameter_declaration, 2, 0, 68),
  [2752] = {.entry = {.count = 1, .reusable = true}}, SHIFT(896),
  [2754] = {.entry = {.count = 1, .reusable = true}}, SHIFT(283),
  [2756] = {.entry = {.count = 1, .reusable = true}}, SHIFT(728),
  [2758] = {.entry = {.count = 1, .reusable = true}}, SHIFT(335),
  [2760] = {.entry = {.count = 1, .reusable = true}}, SHIFT(774),
  [2762] = {.entry = {.count = 1, .reusable = true}}, SHIFT(891),
  [2764] = {.entry = {.count = 1, .reusable = true}}, SHIFT(383),
  [2766] = {.entry = {.count = 1, .reusable = true}}, SHIFT(777),
  [2768] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_variadic_argument, 2, 0, 0),
  [2770] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_dot, 1, 0, 0),
  [2772] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_parameter_declaration, 3, 0, 95),
  [2774] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameter_declaration, 1, 0, 19),
  [2776] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_keyed_element, 3, 0, 89),
  [2778] = {.entry = {.count = 1, .reusable = true}}, SHIFT(207),
  [2780] = {.entry = {.count = 1, .reusable = true}}, SHIFT(420),
  [2782] = {.entry = {.count = 1, .reusable = true}}, SHIFT(652),
  [2784] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_variadic_parameter_declaration, 2, 0, 41),
  [2786] = {.entry = {.count = 1, .reusable = true}}, SHIFT(697),
  [2788] = {.entry = {.count = 1, .reusable = true}}, SHIFT(767),
  [2790] = {.entry = {.count = 1, .reusable = true}}, SHIFT(80),
  [2792] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_variadic_parameter_declaration, 3, 0, 67),
  [2794] = {.entry = {.count = 1, .reusable = true}}, SHIFT(467),
  [2796] = {.entry = {.count = 1, .reusable = true}}, SHIFT(771),
  [2798] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameter_declaration, 3, 0, 39),
  [2800] = {.entry = {.count = 1, .reusable = true}}, SHIFT(62),
  [2802] = {.entry = {.count = 1, .reusable = true}}, SHIFT(665),
  [2804] = {.entry = {.count = 1, .reusable = true}}, SHIFT(191),
  [2806] = {.entry = {.count = 1, .reusable = true}}, SHIFT(730),
  [2808] = {.entry = {.count = 1, .reusable = true}}, SHIFT(904),
  [2810] = {.entry = {.count = 1, .reusable = true}}, SHIFT(646),
  [2812] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameter_declaration, 2, 0, 18),
  [2814] = {.entry = {.count = 1, .reusable = true}}, SHIFT(651),
  [2816] = {.entry = {.count = 1, .reusable = true}}, SHIFT(540),
  [2818] = {.entry = {.count = 1, .reusable = true}}, SHIFT(748),
  [2820] = {.entry = {.count = 1, .reusable = true}}, SHIFT(221),
  [2822] = {.entry = {.count = 1, .reusable = true}}, SHIFT(770),
  [2824] = {.entry = {.count = 1, .reusable = true}}, SHIFT(225),
  [2826] = {.entry = {.count = 1, .reusable = true}}, SHIFT(773),
  [2828] = {.entry = {.count = 1, .reusable = true}}, SHIFT(229),
  [2830] = {.entry = {.count = 1, .reusable = true}}, SHIFT(776),
  [2832] = {.entry = {.count = 1, .reusable = true}}, SHIFT(233),
  [2834] = {.entry = {.count = 1, .reusable = true}}, SHIFT(780),
  [2836] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_for_clause, 4, 0, 83),
  [2838] = {.entry = {.count = 1, .reusable = true}}, SHIFT(303),
  [2840] = {.entry = {.count = 1, .reusable = true}}, SHIFT(495),
  [2842] = {.entry = {.count = 1, .reusable = true}}, SHIFT(251),
  [2844] = {.entry = {.count = 1, .reusable = true}}, SHIFT(480),
  [2846] = {.entry = {.count = 1, .reusable = true}}, SHIFT(553),
  [2848] = {.entry = {.count = 1, .reusable = true}}, SHIFT(135),
  [2850] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1342),
  [2852] = {.entry = {.count = 1, .reusable = true}}, SHIFT(555),
  [2854] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4),
  [2856] = {.entry = {.count = 1, .reusable = true}}, SHIFT(700),
  [2858] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__type_switch_header, 7, 0, 107),
  [2860] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1117),
  [2862] = {.entry = {.count = 1, .reusable = true}}, SHIFT(95),
  [2864] = {.entry = {.count = 1, .reusable = true}}, SHIFT(387),
  [2866] = {.entry = {.count = 1, .reusable = true}}, SHIFT(787),
  [2868] = {.entry = {.count = 1, .reusable = true}}, SHIFT(366),
  [2870] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__type_switch_header, 9, 0, 109),
  [2872] = {.entry = {.count = 1, .reusable = true}}, SHIFT(719),
  [2874] = {.entry = {.count = 1, .reusable = true}}, SHIFT(129),
  [2876] = {.entry = {.count = 1, .reusable = true}}, SHIFT(805),
  [2878] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_for_clause, 3, 0, 56),
  [2880] = {.entry = {.count = 1, .reusable = true}}, SHIFT(537),
  [2882] = {.entry = {.count = 1, .reusable = true}}, SHIFT(570),
  [2884] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_for_clause, 5, 0, 99),
  [2886] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1142),
  [2888] = {.entry = {.count = 1, .reusable = true}}, SHIFT(7),
  [2890] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__type_switch_header, 5, 0, 101),
  [2892] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_implicit_length_array_type, 4, 0, 47),
  [2894] = {.entry = {.count = 1, .reusable = true}}, SHIFT(576),
  [2896] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1353),
  [2898] = {.entry = {.count = 1, .reusable = true}}, SHIFT(717),
  [2900] = {.entry = {.count = 1, .reusable = true}}, SHIFT(482),
  [2902] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1143),
  [2904] = {.entry = {.count = 1, .reusable = true}}, SHIFT(205),
  [2906] = {.entry = {.count = 1, .reusable = true}},  ACCEPT_INPUT(),
  [2908] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1219),
  [2910] = {.entry = {.count = 1, .reusable = true}}, SHIFT(936),
  [2912] = {.entry = {.count = 1, .reusable = true}}, SHIFT(950),
  [2914] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5),
  [2916] = {.entry = {.count = 1, .reusable = true}}, SHIFT(402),
  [2918] = {.entry = {.count = 1, .reusable = true}}, SHIFT(278),
  [2920] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1379),
  [2922] = {.entry = {.count = 1, .reusable = true}}, SHIFT(587),
  [2924] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1368),
  [2926] = {.entry = {.count = 1, .reusable = true}}, SHIFT(280),
  [2928] = {.entry = {.count = 1, .reusable = true}}, SHIFT(706),
  [2930] = {.entry = {.count = 1, .reusable = true}}, SHIFT(715),
  [2932] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1335),
  [2934] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__type_switch_header, 7, 0, 106),
  [2936] = {.entry = {.count = 1, .reusable = true}}, SHIFT(749),
  [2938] = {.entry = {.count = 1, .reusable = true}}, SHIFT(741),
  [2940] = {.entry = {.count = 1, .reusable = true}}, SHIFT(753),
  [2942] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1326),
  [2944] = {.entry = {.count = 1, .reusable = true}}, SHIFT(763),
  [2946] = {.entry = {.count = 1, .reusable = true}}, SHIFT(412),
  [2948] = {.entry = {.count = 1, .reusable = true}}, SHIFT(349),
  [2950] = {.entry = {.count = 1, .reusable = true}}, SHIFT(821),
  [2952] = {.entry = {.count = 1, .reusable = true}}, SHIFT(332),
  [2954] = {.entry = {.count = 1, .reusable = true}}, SHIFT(378),
  [2956] = {.entry = {.count = 1, .reusable = true}}, SHIFT(781),
  [2958] = {.entry = {.count = 1, .reusable = true}}, SHIFT(714),
  [2960] = {.entry = {.count = 1, .reusable = true}}, SHIFT(425),
  [2962] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1395),
  [2964] = {.entry = {.count = 1, .reusable = true}}, SHIFT(321),
  [2966] = {.entry = {.count = 1, .reusable = true}}, SHIFT(824),
  [2968] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1005),
  [2970] = {.entry = {.count = 1, .reusable = true}}, SHIFT(815),
  [2972] = {.entry = {.count = 1, .reusable = true}}, SHIFT(757),
  [2974] = {.entry = {.count = 1, .reusable = true}}, SHIFT(504),
  [2976] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_for_clause, 4, 0, 85),
  [2978] = {.entry = {.count = 1, .reusable = true}}, SHIFT(50),
  [2980] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1390),
  [2982] = {.entry = {.count = 1, .reusable = true}}, SHIFT(712),
  [2984] = {.entry = {.count = 1, .reusable = true}}, SHIFT(973),
  [2986] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1001),
  [2988] = {.entry = {.count = 1, .reusable = true}}, SHIFT(301),
  [2990] = {.entry = {.count = 1, .reusable = true}}, SHIFT(250),
  [2992] = {.entry = {.count = 1, .reusable = true}}, SHIFT(588),
  [2994] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1410),
  [2996] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1349),
  [2998] = {.entry = {.count = 1, .reusable = true}}, SHIFT(765),
  [3000] = {.entry = {.count = 1, .reusable = true}}, SHIFT(742),
  [3002] = {.entry = {.count = 1, .reusable = true}}, SHIFT(768),
  [3004] = {.entry = {.count = 1, .reusable = true}}, SHIFT(883),
  [3006] = {.entry = {.count = 1, .reusable = true}}, SHIFT(772),
  [3008] = {.entry = {.count = 1, .reusable = true}}, SHIFT(775),
  [3010] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1355),
  [3012] = {.entry = {.count = 1, .reusable = true}}, SHIFT(779),
  [3014] = {.entry = {.count = 1, .reusable = true}}, SHIFT(101),
  [3016] = {.entry = {.count = 1, .reusable = true}}, SHIFT(401),
};

#ifdef __cplusplus
extern "C" {
#endif
#ifdef TREE_SITTER_HIDE_SYMBOLS
#define TS_PUBLIC
#elif defined(_WIN32)
#define TS_PUBLIC __declspec(dllexport)
#elif defined __GNUC__ || defined __clang__
#define TS_PUBLIC __attribute__((visibility("default")))
#else
#define TS_PUBLIC
#endif

TS_PUBLIC const TSLanguage *tree_sitter_go(void) {
  static const TSLanguage language = {
    .version = LANGUAGE_VERSION,
    .symbol_count = SYMBOL_COUNT,
    .alias_count = ALIAS_COUNT,
    .token_count = TOKEN_COUNT,
    .external_token_count = EXTERNAL_TOKEN_COUNT,
    .state_count = STATE_COUNT,
    .large_state_count = LARGE_STATE_COUNT,
    .production_id_count = PRODUCTION_ID_COUNT,
    .field_count = FIELD_COUNT,
    .max_alias_sequence_length = MAX_ALIAS_SEQUENCE_LENGTH,
    .parse_table = &ts_parse_table[0][0],
    .small_parse_table = ts_small_parse_table,
    .small_parse_table_map = ts_small_parse_table_map,
    .parse_actions = ts_parse_actions,
    .symbol_names = ts_symbol_names,
    .field_names = ts_field_names,
    .field_map_slices = ts_field_map_slices,
    .field_map_entries = ts_field_map_entries,
    .symbol_metadata = ts_symbol_metadata,
    .public_symbol_map = ts_symbol_map,
    .alias_map = ts_non_terminal_alias_map,
    .alias_sequences = &ts_alias_sequences[0][0],
    .lex_modes = ts_lex_modes,
    .lex_fn = ts_lex,
    .keyword_lex_fn = ts_lex_keywords,
    .keyword_capture_token = sym_identifier,
    .primary_state_ids = ts_primary_state_ids,
  };
  return &language;
}
#ifdef __cplusplus
}
#endif
